
      program analysis

use decimal, only:dop,long
use global
use aglobal
use plot
use versions

      implicit none

      integer(long)           :: i,task,version,release,ierr
      character(len=c5)       :: string

!-----------------------------------------------------------------------
! initialise variables
!-----------------------------------------------------------------------
      name='.'
      laenge=1
      call abspath(name,laenge)

!-----------------------------------------------------------------------
! set up version number as a string to call correct programs
! No longer needed in Quantics
!-----------------------------------------------------------------------
      call getrevno(progver,version,release)
      write(aversion,'(2i5)') version,release
      call rmblnk(aversion)

!-----------------------------------------------------------------------
! default print command
!-----------------------------------------------------------------------
      pprcmd='lpr'

!-----------------------------------------------------------------------
! write out header
!-----------------------------------------------------------------------
      do i=1,72
         string(i:i)='*'
      enddo
      write(6,*)
      write(6,*)
      write(6,'(a72)') string
      write(6,*)
      write(6,*) '    THE QUANTICS WAVEPACKET PROPAGATION ANALYSIS PACKAGE'
      write(6,*)
      write(6,'(10x,a,i5)') 'Program Version : ',version
      write(6,'(10x,a,i5)') 'Release         : ',release
      write(6,*)
      write(6,'(a72)') string

      write(6,*)
      write(6,'(2a)') 'Present directory is: ', name(1:laenge)

      call mygetarg(1,string)


!-----------------------------------------------------------------------
! main menu
!-----------------------------------------------------------------------
      task=-1
   10 continue
      write(6,*)
      write(6,'(a)') '  0 = stop'
      write(6,'(a)') '  1 = list / change directory'
      write(6,'(a)') '  2 = analyse convergence'
      write(6,'(a)') '  3 = analyse integrator'
      write(6,'(a)') '  4 = analyse results'
      write(6,'(a)') '  5 = analyse system evolution'
      write(6,'(a)') '  6 = analyse potential surface'
      write(6,'(a)') '  7 = compare calculations'
      if(string .eq. '-h' ) stop
      read(5,*,iostat=ierr) task

      if (ierr.ne.0) then
         write(6,*)
         write(6,'(a)') 'ERROR: input read error'
         goto 10
      endif

      if (task .lt. 0 .or. task .gt. 7) then
         write(6,*)
         write(6,*) 'ERROR: Please select a valid option'
         go to 10
      endif

!-----------------------------------------------------------------------
! now do task
!-----------------------------------------------------------------------
      if (task .eq. 0) then
         go to 999
      else if (task .eq. 1) then
         call changedir
      else if (task .eq. 2) then
         call analconv
      else if (task .eq. 3) then
         call analint
      else if (task .eq. 4) then
         call analres
      else if (task .eq. 5) then
         call analevol
      else if (task .eq. 6) then
         call analpes
      else if (task .eq. 7) then
         write(6,'(/,a)') 'Not yet implemented'
      endif

      go to 10

  999 continue
      stop
      contains

!#######################################################################

      subroutine changedir

      implicit none

      character(len=c5)       :: line,name1

      write(6,*)
      write(6,'(2a)') 'present directory is: ',name(1:laenge)

      write(6,*)
      line='ls -C '//name(1:laenge)
      call excmd(line)

   10 continue
      write(6,*)
      write(6,'(a)') 'change directory ? (no or new path)'
      read(5,'(a)') name1
      if (name1 .eq. 'no') go to 999
      laenge=index(name1,' ')-1

      call abspath1(name1,laenge,name)
      name=name1
      write(6,*)
      write(6,'(2a)') 'New directory: ',name(1:laenge)

      write(6,*)
      line='ls -C '//name(1:laenge)
      call excmd(line)

      go to 10

  999 continue
      return
      end subroutine changedir

!#######################################################################

      subroutine analres

      implicit none

      integer(long)           :: task,s
      integer(long), external :: slen
      character(len=c5)       :: line
      logical(kind=4)         :: lexist

   10 continue
      write(6,*)
      write(6,'(a)') ' *** Plot Results ***'
      write(6,'(a)') ' 0 = return to main menu '
      write(6,'(a)') ' 1 = Plot autocorrelation function'
      write(6,'(a)') &
           ' 2 = Plot Fourier Transform of autocorrelation function'
      write(6,'(a)') &
           ' 3 = Plot spectrum from autocorrelation function'
      write(6,'(a)') ' 4 = Plot eigenvalues from matrix diagonalisation'

      read(5,*) task

      if (task .eq. 0) then
         go to 999
      else if (task .eq. 1) then
         inquire(file=name(1:laenge)//'/auto',exist=lexist) 
         if (lexist) then
            line='rdauto'
            s=slen(line)
            line=line(1:s)//' -inter -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,a)') 'Needs an auto file from a propagation run'
         endif
      else if (task .eq. 2) then
         inquire(file=name(1:laenge)//'/auto',exist=lexist) 
         if (lexist) then
            line='autospec'
            s=slen(line)
            line=line(1:s)//' -inter -FT -i '//name(1:laenge)//'/'
            call excmd(line)
         else
            write(6,'(/,a)') 'Needs an auto file from a propagation run'
         endif
      else if (task .eq. 3) then
         inquire(file=name(1:laenge)//'/auto',exist=lexist) 
         if (lexist) then
            line='autospec'
            s=slen(line)
            line=line(1:s)//' -inter -i '//name(1:laenge)//'/'
            call excmd(line)
         else
            write(6,'(/,a)') 'Needs an auto file from a propagation run'
         endif
      else if (task .eq. 4) then
         inquire(file=name(1:laenge)//'/eigval',exist=lexist) 
         if (lexist) then
            line='rdeigval'
            s=slen(line)
            line=line(1:s)//' -inter -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,a)') &
              'Needs an eigval file from a diagonalisation run'
         endif
      endif
      go to 10

  999 continue
      return
      end subroutine analres

!#######################################################################

      subroutine analevol

      implicit none

      integer(long)           :: task,s
      integer(long), external :: slen
      character(len=c5)       :: line
      logical(kind=4)         :: lexist

   10 continue
      write(6,*)
      write(6,'(a)') ' *** Analyse system evolution***'
      write(6,'(a)') ' 0 = return to main menu '
      write(6,'(a)') ' 1 = Plot 1d density evolution'
      write(6,'(a)') ' 2 = Plot 2d density evolution'
      write(6,'(a)') ' 3 = Plot diabatic state population'

      read(5,*) task

      if (task .eq. 0) then
         go to 999
      else if (task .eq. 1) then
         inquire(file=name(1:laenge)//'/gridpop',exist=lexist) 
         if (lexist) then
            line='showd1d'
            s=slen(line)
            line=line(1:s)//' -inter -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,a)') &
              'Needs an gridpop file from a propagation run'
         endif
      else if (task .eq. 2) then
         inquire(file=name(1:laenge)//'/psi',exist=lexist) 
         if (lexist) then
            line='showsys'
            s=slen(line)
            line=line(1:s)//' -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,a)') &
              'Needs a psi file from a propagation run'
         endif
      else if (task .eq. 3) then
         inquire(file=name(1:laenge)//'/check',exist=lexist) 
         if (lexist) then
            line='plstate'
            s=slen(line)
            line=line(1:s)//' -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,a)') &
              'Needs a check file from a propagation run'
         endif

      endif
      go to 10

  999 continue
      return
      end subroutine analevol

!#######################################################################

      subroutine analpes

      implicit none

      integer(long)           :: task,s
      integer(long), external :: slen
      character(len=c5)       :: line
      logical(kind=4)         :: lexist

   10 continue
      write(6,*)
      write(6,'(a)') ' *** Analyse PES ***'
      write(6,'(a)') ' 0 = return to main menu '
      write(6,'(a)') ' 1 = Plot potential energy surface'

      read(5,*) task

      if (task .eq. 0) then
         go to 999
      else if (task .eq. 1) then
         inquire(file=name(1:laenge)//'/oper',exist=lexist) 
         if (lexist) then
            line='showsys'
            s=slen(line)
            line=line(1:s)//' -pes -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,a)') 'Needs an oper file'
         endif
      endif
      go to 10

  999 continue
      return
      end subroutine analpes

!#######################################################################

      subroutine analint

      implicit none

      integer(long)           :: task,s
      integer(long), external :: slen
      character(len=c5)       :: line
      logical(kind=4)         :: lexist

      line=' '

   10 continue
      write(6,*)
      write(6,'(a)') ' *** Plot Results ***'
      write(6,'(a)') ' 0 = return to main menu '
      write(6,'(a)') ' 1 = Analyse step size '
      write(6,'(a)') ' 2 = Look at timing file '
      write(6,'(a)') ' 3 = Type update file '
      write(6,'(a)') ' 4 = Plot update step size '
      call prnmenu

      read(5,*) task

      if (task .eq. 0) then
         go to 999
      else if (task .eq. 1) then
         inquire(file=name(1:laenge)//'/steps',exist=lexist) 
         if (lexist) then
            line='rdsteps'
            s=slen(line)
            line=line(1:s)//' -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,a)') &
              'Needs a steps file from a propagation run'
         endif
      else if (task .eq. 2) then
         inquire(file=name(1:laenge)//'/steps',exist=lexist) 
         if (lexist) then
            line='cat '//name(1:laenge)//'/timing'
            call excmd(line)
         else
            write(6,'(/,a)') 'Needs a timing file'
         endif
      else if (task .eq. 3) then
         inquire(file=name(1:laenge)//'/update',exist=lexist) 
         if (lexist) then
            line='rdupdate'
            s=slen(line)
            line=line(1:s)//' -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,2a)') &
                'Needs an update file from a propagation using',&
                ' the CMF integrator'
         endif
      else if (task .eq. 4) then
         inquire(file=name(1:laenge)//'/update',exist=lexist) 
         if (lexist) then
            line='rdupdate'
            s=slen(line)
            line=line(1:s)//' -inter -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,2a)') &
                'Needs an update file from a propagation using',&
                ' the CMF integrator'
         endif

      else if (task .ge. 99) then
         call prntask(task,line)
      endif
      go to 10

  999 continue
      return
      end subroutine analint

!#######################################################################

      subroutine analconv

      implicit none

      integer(long)           :: task,s
      integer(long), external :: slen
      character(len=c5)       :: line
      logical(kind=4)         :: lexist

      line=' '

   10 continue
      write(6,*)
      write(6,'(a)') ' *** Analyse convergence ***'
      write(6,'(a)') ' 0 = return to main menu '
      write(6,'(a)') ' 1 = check orthonormality of spfs in psi file'
      write(6,'(a)') ' 2 = check orthonormality of spfs in restart file'
      write(6,'(a)') ' 3 = plot populations of natural orbitals'
      write(6,'(a)') ' 4 = plot populations of grid edges'
      write(6,'(a)') ' 5 = plot time-evolution of norm of wavefunction'
      write(6,'(a)') ' 6 = norm of wavefunction on restart file'
      call prnmenu

      read(5,*) task

      if (task .eq. 0) then
         go to 999
      else if (task .eq. 1) then
         inquire(file=name(1:laenge)//'/psi',exist=lexist) 
         if (lexist) then
            line='ortho'
            s=slen(line)
            line=line(1:s)//' -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,a)') &
              'Needs a psi file from a propagation run'
         endif
      else if (task .eq. 2) then
         inquire(file=name(1:laenge)//'/restart',exist=lexist) 
         if (lexist) then
            line='ortho'
            s=slen(line)
            line=line(1:s)//' -r -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,a)') 'Needs a restart file'
         endif
      else if (task .eq. 3) then
         inquire(file=name(1:laenge)//'/restart',exist=lexist) 
         if (lexist) then
            line='rdcheck natpop 1 1'
            s=slen(line)
            line=line(1:s)//' -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,a)') &
              'Needs a check file from a propagation run'
         endif
      else if (task .eq. 4) then
         inquire(file=name(1:laenge)//'/restart',exist=lexist) 
         if (lexist) then
            line='rdgpop'
            s=slen(line)
            line=line(1:s)//' -i '//name(1:laenge)
            s=slen(line)
            line=line(1:s)//' 0'
            call excmd(line)
         else
            write(6,'(/,a)') &
              'Needs a gridpop file from a propagation run'
         endif
      else if (task .eq. 5) then
         inquire(file=name(1:laenge)//'/psi',exist=lexist) 
         if (lexist) then
            line='norm'
            s=slen(line)
            line=line(1:s)//' -inter -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,a)') &
              'Needs a psi file from a propagation run'
         endif
      else if (task .eq. 6) then
         inquire(file=name(1:laenge)//'/restart',exist=lexist) 
         if (lexist) then
            line='norm'
            s=slen(line)
            line=line(1:s)//' -r -i '//name(1:laenge)
            call excmd(line)
         else
            write(6,'(/,a)') &
              'Needs a restart file'
         endif
      
      else if (task .ge. 99) then
         call prntask(task,line)
      endif
      go to 10

  999 continue
      return
      end subroutine analconv

!#######################################################################

      subroutine prnmenu

      implicit none

      integer(long), external :: slen

      write(6,*)
      write(6,'(a)') ' 99 = print last screen output '
      write(6,'(3a)') &
          '100 = change printer (',pprcmd(1:slen(pprcmd)),')'

      return
      end subroutine prnmenu

!#######################################################################

      subroutine prntask(task,line)

      implicit none

      integer(long)                 :: s
      integer(long), intent(in)     :: task
      integer(long), external       :: slen
      character(len=c5)             :: line1
      character(len=c5), intent(in) :: line

      if (task .eq. 99) then
         write(6,*)
         s=slen(line)
         if (s .eq. 0) then
            write(6,'(a)') 'First select data to print'
            return
         else 
            s=s-1
         endif
         line1=line(1:s)//' | '//pprcmd
         call excmd(line1)
      else if (task .eq. 100) then
  122    continue
         write(6,*)
         write(6,'(a)')&
              'enter print command (e.g. "lpr -Pps2" or filename) :'
         write(6,*)
         read(5,*,err=122) pprcmd
      endif

      return
      end subroutine prntask

      end program analysis

