!***********************************************************************
!
!                      AUSWERT subroutine AOVERLAP
!
!     This subroutine calculates the overlap between the A-vectors
!     of two different calculations. The wave functions must not differ 
!     in the number of spfs but may differ in the DVR's in particular
!     in the number of grid points for the various modes.
!     Some additional checks are made in the subroutine
!     'aovercheck' to ensure compatibility.
!     The variables concerning the first wave function have the ending
!     'c1', variables concerning the second wave function have the 
!     ending 'c2' or no special ending at all. This is intended to
!     reduce the number of additional variables.
!     The time-overlap table is saved as 'aovl_2' in the directory 
!     of the first calculation. The 2 obove stands for the name of the
!     second calculation.
!
!     NB for G-MCTDH calculations the overlap of the GWPs is not
!     allowed for and the aoverlap will be meanlingless
!
!     05/99 HDM
!
!
!***********************************************************************
      
      module aoverlapmod

      use decimal, only : long,dop
      use maxv, only : maxdim, maxsta

!-----------------------------------------------------------------------
!     pointers for overlap routine
!-----------------------------------------------------------------------
      integer(long), save :: dgldim1,dgldim0,adim1,dmatdim1,agmatdim1,&
                             gsmatdim1,gwpinfodim1

      complex(dop), dimension(:), allocatable, save    :: agmat1

      end module aoverlapmod

!#######################################################################

      program aoverlap

use decimal, only:long,dop,sip
use global
use globmemmod
use aglobal
use dvrdatmod
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod
use aoverlapmod
use channels
use versions


use rdfilesmod, only: rdpsi
use compdat, only: getcpsi
use xvlib, only: vvaxzz
use logdat
use compdat, only: savecsys,savecpsi
use iofile, only: rdpsiinfo
use auswutil, only: savecdat
use zeigauswmod, only: zeigausw


use module_interface

      implicit none

      integer(long)                        :: ilbl,zeig,zeig1,m,s,&
                                              chkdvr,chkgrd,chkpsi,chkdat
      character(len=c5)                    :: filename
      logical(kind=4)                      :: readsys,lerr

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      call aovrinput

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='aovrlp.log',form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
! get memory sizes
!-----------------------------------------------------------------------
      call rdmemdim(ipsi)

!-----------------------------------------------------------------------
! Allocate mememory
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef
      call alloc_compdat

!-----------------------------------------------------------------------
! set no. of comparison data sets
!-----------------------------------------------------------------------
      ncomp=2

!-----------------------------------------------------------------------
! read data from psi file in order to set dgldim0
!-----------------------------------------------------------------------
      read(ipsi) filever(ipsi)
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)

!-----------------------------------------------------------------------
! save data as set 1
!-----------------------------------------------------------------------
      dgldim0=dgldim
      call savecsys(1)
      call savecpsi(1)
      call savecdat(1)

!-----------------------------------------------------------------------
! read data from comparison directory
!-----------------------------------------------------------------------
      read(ipsi1) filever(ipsi1)
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      call rdpsiinfo(ipsi1,chkdvr,chkgrd,chkpsi,chkdat)
      filename='psi1'
      call chksyserr(filename,lerr)
      if (lerr) go to 999

!-----------------------------------------------------------------------
! save comparison set data as set 2
!-----------------------------------------------------------------------
      dgldim1=dgldim
      call savecsys(2)
      call savecpsi(2)
      call savecdat(2)

!-----------------------------------------------------------------------
! read data from psi file
!-----------------------------------------------------------------------
      rewind ipsi
      read(ipsi) filever(ipsi)
      chkdvr=5
      chkgrd=2
      chkpsi=1
      chkdat=1
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      if (chkerr(1) .ne. 0) go to 999

!-----------------------------------------------------------------------
! compare psi and psi1 data.
!-----------------------------------------------------------------------
      if (chkerr(2) .ne. 0 .or. chkerr(3) .ne. 0) then
         write(6,'(a)') ' Different primitive bases! (That is OK)'
         readsys=.true.
      endif
      if (chkerr(4) .ne. 0) then
         write(6,'(a)')' Different nmode, nstate, or npacket! (Stop)'
         message='Different nmode, nstate, or npacket'
         call errormsg
      endif
      if (chkerr(5) .ne. 0) then
         write(6,'(a)')&
          ' Different electronic states or packets! (Stop)'
         message='Different lmult, leb, lmulpack'
         call errormsg
      endif

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
!-----------------------------------------------------------------------
      lpsi=.true.
      lpsi1=.true.
      call zeigausw

      zeig=1
      do s=1,nstate
         zeig1=block(s)
         do m=1,nmode
            if( cdim(m,s,1) .ne. cdim(m,s,2)) then
               write(6,'(a)') ' Number of spf''s differ'
               write(6,'(a,i3,a,i2,a,2i4)') &
              ' m=', m,'  s=',s,'  dim=', cdim(m,s,1), cdim(m,s,2)
               stop
            end if
         enddo
      enddo


!-----------------------------------------------------------------------
! assign memory arrays
!-----------------------------------------------------------------------
      call memausw

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
! call overlap routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

!-----------------------------------------------------------------------
! get lengths of comparison data arrays
!-----------------------------------------------------------------------
      call getcpsi(2)
      dgldim1=dgldim
      adim1=adim
      dmatdim1=dmatdim
      agmatdim1=agmatdim
      gsmatdim1=gsmatdim
      gwpinfodim1=gwpinfodim
      call getcpsi(1)

allocate(agmat(agmatdim))
allocate(agmat1(agmatdim1))

      call aovrlap(mc(mcpsi),mc(mcpsi1),ms(mspsi),mi(mijindx),&
           mi(mijindx1),mc(mcadgwp),mc(mcadgwp1),agmat,agmat1)

deallocate(mc,mr,mi,ml,ms,mf)
deallocate(agmat,agmat1)

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_psidef

      call dealloc_compdat

      stop
  999 continue
      message= 'Error reading :'//trim(filename)
      write(6,'(a)') message
      stop
      contains

!***********************************************************************
!     calculates the overlap
!***********************************************************************

      subroutine aovrlap (psi,psi1,spsi,jindx,jindx1,adgwp,adgwp1,agmat,&
                 agmat1)

      implicit none

      integer(long)                                   :: nstep,nstep1,i
      integer(long), dimension(1)                     :: idum
      integer(long), dimension(adim), intent(out)     :: jindx,jindx1
      integer(long), dimension(gwpinfodim)            :: gwpdep
      integer(long), dimension(gwpinfodim1)           :: gwpdep1
      complex(dop)                                    :: ovl,ovl1,ovl2
      complex(sip), dimension(spsidim), intent(out)   :: spsi
      complex(dop), dimension(dgldim), intent(out)    :: psi,psi1
      complex(dop), dimension(gsmatdim), intent(out)  :: adgwp
      complex(dop), dimension(gsmatdim1), intent(out) :: adgwp1
      complex(dop), dimension(agmatdim), intent(out)  :: agmat
      complex(dop), dimension(agmatdim1), intent(out) :: agmat1
      real(dop)                                       :: out21,time,dt,&
                                                         error,hilbert,pherr,&
                                                         phcorrect,normsq,&
                                                         norm1sq
      logical(kind=4)                                 :: lover

!-----------------------------------------------------------------------
! calculate time step for data
!-----------------------------------------------------------------------
      call getcpsi(1)
      out21=cout2(2)
      lover=.false.
      if (out2.ge.out21) then
         if (mod(out2,out21).lt.tiny) then
            nstep1 = int((out2+tiny)/out21)
            nstep = 1
         else
            lover=.true.
         endif
      else
         if (mod(out21,out2).lt.tiny) then
            nstep = int((out21+tiny)/out2)
            nstep1 = 1
         else
            lover=.true.
         endif
      endif

      if (lover) then
!        12600 = 2*2*2*3*3*5*5*7
         if( out2 .ge. out21 ) then
            nstep1 = nint(12600.d0*out2/out21)
            nstep  = 12600
            else
            nstep  = nint(12600.d0*out21/out2)
            nstep1 = 12600
         end if
         do  i = 10, 2, -1
            if (mod(nstep1,i).eq.0 .and. mod(nstep,i).eq.0) then
               nstep1 = nstep1/i
               nstep  = nstep/i
            end if
         end do
         do  i = 6, 2, -1
            if (mod(nstep1,i).eq.0 .and. mod(nstep,i).eq.0) then
               nstep1 = nstep1/i
               nstep  = nstep/i
            end if
         end do
         write(6,'(a,2f15.5)') ' output steps [fs]  : ',&
                                 out2/fs, out21/fs
         if(  abs(nstep*out2-nstep1*out21) .lt. &
            tiny*(nstep*out2+nstep1*out21) ) then
            write(6,'(a,f15.5)')&
                ' common output step : ',nstep*out2/fs
         else
            write(6,'(a)') ' output steps are incompatible '
            write(6,'(1x,2f16.6,e11.3)') nstep*out2/fs,&
                 nstep1*out21/fs, nstep*out2-nstep1*out21
            stop
         endif
      endif

      nstep  = nstep *step
      nstep1 = nstep1*step
      dt     = out2/fs*nstep
      time   = tinit/fs

      write(6,'(80a)') ('-',i=1,80)
      write(6,'(a12,6a11)')&
        '    Time[fs]','      Error','    Hilbert',&
         '      Phase','   Ph. corr','    ||1||^2','    ||2||^2'


   10 continue
      call getcpsi(2)
      call rdpsi(ipsi1,psi1,spsi,jindx1,agmat1,idum,adgwp1,gwpdep1)
      if (lend) goto 77
      
      call getcpsi(1) 
      call rdpsi(ipsi,psi,spsi,jindx,agmat,idum,adgwp,gwpdep)
      if (lend) goto 77
      
      if(lgnu) then
!.....lgnu is (mis)-used to switch on the -abs option.
         do i = 1, adim
            psi1(i) = abs(psi1(i))
            psi (i) = abs(psi (i))
         enddo
      endif

      call aoverlap0(ovl,ovl1,ovl2,psi,psi1)

      normsq=dble(ovl)
      norm1sq=dble(ovl1)

      error=normsq+norm1sq-2.0_dop*dble(ovl2)
      if (error .lt. 0.0_dop) error=0.0_dop
      error=dsqrt(error)

      hilbert=min(abs(ovl2)/dsqrt(normsq*norm1sq),1.0_dop)
!     hilbert=acos(hilbert)*(180.0_dop/pi)
      hilbert=acos(hilbert)

!     pherr=atan2(dimag(ovl2),dble(ovl2))*(180.0_dop/pi)
      pherr=atan2(dimag(ovl2),dble(ovl2))

      phcorrect=normsq+norm1sq-2.0_dop*abs(ovl2)
      if (phcorrect .lt. 0.0_dop) phcorrect=0.0_dop
      phcorrect=dsqrt(phcorrect)

      if(laeout.gt.0) write(iaus,'(f12.4,6f11.6)') time,&
          error,hilbert,pherr,phcorrect,normsq,norm1sq 
      write(6,'(f12.4,6f11.6)') time,&
          error,hilbert,pherr,phcorrect,normsq,norm1sq 

      time=time + dt

!-----------------------------------------------------------------------
! if more data, go to beginning of loop
!-----------------------------------------------------------------------
      call getcpsi(2) 
      do i=2,nstep1
         call rdpsi(ipsi1,psi1,spsi,jindx1,agmat1,idum,adgwp1,gwpdep1)
         if (lend) goto 77
      enddo
      call getcpsi(1) 
      do i=2,nstep
         call rdpsi(ipsi,psi,spsi,jindx,agmat,idum,adgwp,gwpdep)
         if (lend) goto 77
      enddo
      goto 10

 77   continue
      write(6,'(a12,6a11)')&
        '    Time[fs]','      Error','    Hilbert',&
         '      Phase','   Ph. corr','    ||1||^2','    ||2||^2'
      write(6,'(3a)') (' ++++++  A-overlap  ++++++',i=1,3)
!      write(6,'(80a)') ('-',i=1,80)
      if(laeout.gt.0) close(iaus)
      close(ipsi)
      close(ipsi1)

      return
      end subroutine aovrlap

!#######################################################################

      subroutine aoverlap0(ovl,ovl1,ovl2,psi,psi1)

      implicit none

      complex(dop), intent(out)                     :: ovl,ovl1,ovl2
      complex(dop), dimension(dgldim0), intent(in) :: psi
      complex(dop), dimension(dgldim1), intent(in) :: psi1

!-----------------------------------------------------------------------
! check that calculation allowed
!-----------------------------------------------------------------------
      if(cadim(1) .ne. cadim(2) ) then
         routine='aoverlap0'
         message='A-vectors differ in number of spfs'
         call errormsg
      end if

!-----------------------------------------------------------------------
! calculate Sum_J A_J^*  * A_J within each state (psi1*psi1)
!-----------------------------------------------------------------------
      call getcpsi(2)
      ovl1=(0.0_dop,0.0_dop)
      call vvaxzz(psi1,psi1,ovl1,adim)

!-----------------------------------------------------------------------
! calculate Sum_J A_J^*  * A_J within each state (psi*psi)
!-----------------------------------------------------------------------
      call getcpsi(1)
      ovl=(0.0_dop,0.0_dop)
      call vvaxzz(psi,psi,ovl,adim)
               
!-----------------------------------------------------------------------
! calculate Sum_J A_J^*  * A_J within each state (psi*psi1)
!-----------------------------------------------------------------------
      call getcpsi(1)
      ovl2=(0.0_dop,0.0_dop)
      call vvaxzz(psi,psi1,ovl2,adim)
      
      return
      end subroutine aoverlap0

!#######################################################################

      subroutine ardover(unit)

      implicit none

      integer(long)             :: i
      integer(long), intent(in) :: unit
      logical(kind=4)           :: lerr
      real(dop)                 :: time,error,hilbert,pherr,phcorrect,nsq,n1sq
      character(len=80)         :: string

      lerr=.false.
      read(unit,*) 
      read(unit,'(a)') string
      read(unit,*) 
      read(unit,'(a)') string

      read(unit,*,end=999) 
      read(unit,*,end=999) 

      write(6,'(3a)') (' ++++++  A-overlap  ++++++',i=1,3)
      write(6,'(a)') ' ***********  Data read from file ! ***********' 
      write(6,'(a12,6a11)')&
        '  Time in fs','      Error','    Hilbert',&
         '      Phase','   Ph. corr','    ||1||^2','    ||2||^2'


   10 continue
      read(unit,*,end=999) time,error,hilbert,pherr,phcorrect,nsq,n1sq
      write(6,'(f12.4,6f11.6)') time,&
          error,hilbert,pherr,phcorrect,nsq,n1sq

      go to 10

  999 continue

      write(6,'(a12,6a11)')&
        '    Time[fs]','      Error','    Hilbert',&
         '      Phase','   Ph. corr','    ||1||^2','    ||2||^2'
      write(6,'(a)') ' ***********  Data read from file ! ***********' 
      write(6,'(3a)') (' ++++++  A-overlap  ++++++',i=1,3)

      return
      end subroutine ardover

!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------
      subroutine aovrinput

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: iarg,ilbl,llaenge,i,ierr
      character(len=80)       :: buf
      character(len=c5)       :: filename,flabel
!-----------------------------------------------------------------------

      filein  = 'psi'
      filein1 = '  '
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:2) .eq. '-f' .or.  buf(1:2) .eq. '-i') then
            i = i+1
            call mygetarg(i,buf)
            filein = buf
         else if( buf(1:2) .eq. '-o' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf(1:2) .eq. '-n' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:2) .eq. '-w' ) then
            lovwr=.true.
        else if( buf(1:4) .eq. '-abs' ) then
           lgnu = .true.
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,a,/10x,a)')&
       'Purpose: Calculates the difference between two A-vectors.',&
       'See review Eqs.(159-163) for a definition of the output'
            write(6,'(/1x,a)')&
       'Usage  : aoverlap83 [-f -i -o -n -w -ver -h -?]  [psi2] .'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a)') &
                     ' -f FILE : The wavefunction is read '&
                    ,'from file FILE rather than from ./psi'&
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(a)') ' -i FILE : Same as -f FILE.'
            write(6,'(2a,/,11x,a/,11x,a)') &
                     ' -o FILE : The difference is written to file ',&
                     'FILE.pl rather than to ./aovl_XXX',&
             'The string FILE may be a relative or a full path-name.',&
             'If FILE=no, i.e. "-o no", no aovl-file will be opened.'
            write(6,'(2a)') ' -abs    : Take the absolute value', &
                 ' of A, when computing the overlap.'
            write(6,'(2a)') ' -n step : Compute the A-overlap', &
                 ' only every step-th output.'
            write(6,'(2a)') &
                     ' -w      : An existing aoverlap file is ',&
                     'overwritten.'
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(2a/,a)') ' The argument psi2 is the name of',&
                            ' the file containing the comparison', &
                            ' wavefunction.' 
            write(6,'(2a)') ' If no argument is given, ',&
                 'the user is prompted for the missing argument.'  
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      
      if (iarg .lt. i) goto 200
      call mygetarg(i,filein1)

 200  continue
      if (filein1 .eq. ' ') then
         write(6,*) 'Name of comparison file   :'
         read(5,'(a)') filein1
      endif

      laein=index(filein,' ')-1
      if (filein(laein:laein) .eq. '/') then 
         filein = filein(1:laein)//'psi'
         laein = laein + 3
      end if
      call abspath(filein,laein)
      laein1=index(filein1,' ')-1
      if (filein1(laein1:laein1) .eq. '/') then 
         filein1 = filein1(1:laein1)//'psi'
         laein1 = laein1 + 3
      end if
      call abspath(filein1,laein1)

!-----------------------------------------------------------------------
! get stem of comparison name
! If file name is psi, use directory name.
! Else use file name
!-----------------------------------------------------------------------
      llaenge=laein1
      ilbl=llaenge
   11 continue
      if (filein1(ilbl:ilbl) .eq. '/') then
         continue
      else
        ilbl=ilbl-1
        go to 11
      endif

      if (filein1(ilbl+1:laein1) .eq. 'psi') then
         llaenge = ilbl
         ilbl=ilbl-1
   12    continue
         if (filein1(ilbl:ilbl) .eq. '/') then
            continue
         else
           ilbl=ilbl-1
           go to 12
         endif
      endif
      flabel=filein1(ilbl+1:llaenge)
      llaenge=index(flabel,' ' )-1

      if (fileout(1:2) .eq. 'no') then
         laeout = 0
         goto 203
      else
         if (fileout .eq. ' ') then
            fileout = 'aovl_'//flabel(1:llaenge)
         endif
         laeout=index(fileout,'  ')-1
         call abspath(fileout,laeout)
      end if

!-----------------------------------------------------------------------
! check for existence of overlap file, and read if there
!-----------------------------------------------------------------------
      if (lovwr) go to 203
      filename=fileout(1:laeout)
      open (iaus,file=filename,form='formatted',&
            status='old',err=203)
      call ardover(iaus)
      close (iaus)
      stop

  203 continue

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      write(6,'(80a)') ('*',i=1,80)
      filename=filein(1:laein)
      open(ipsi,file=filename,form='unformatted',status='old',&
           err=1000)
      write(6,'(2a)') ' The inputfile is : ',filein(1:laein)

      filename=filein1(1:laein1)
      open(ipsi1,file=filename,form='unformatted',status='old',&
           err=1000)
      write(6,'(2a)') ' The comparison file is : ',&
                        filein1(1:laein1)

      if(laeout .eq. 0) then
         write(6,'(2a)') ' No outputfile '
      else
         filename=fileout(1:laeout)
         open (iaus,file=filename,form='formatted',status='unknown',&
              err=1000)
         write(6,'(2a)') ' The outputfile is : ',fileout(1:laeout)

         write(iaus,'(a)')     '# Overlap of'
         ilbl = max(1,laein-77)
         write(iaus,'(2a)')    '# ',filein(ilbl:laein)
         write(iaus,'(a)')     '# and'
         ilbl = max(1,laein1-77) 
         write(iaus,'(2a)')    '# ',filein1(ilbl:laein1)
         write(iaus,'(a)')     '# '

         write(iaus,'(a12,6a11)')&
              '# Time in fs','      Error','    Hilbert',&
              '      Phase','   Ph. corr','    ||1||^2','    ||2||^2'
      end if
         
         write(6,'(3a)') (' ++++++  A-overlap  ++++++',i=1,3)

      return 
      
 1000 write(6,*)' Cannot open file : '
      write(6,*) filename
      stop
      
      end subroutine aovrinput

      end program aoverlap

