!***********************************************************************
!
!                       AUTOSPEC
!
!  Computes the spectrum from the autocorrelation function (HDM 01/99)
!
!
!***********************************************************************

      module autospecmod

use decimal, only:dop,long
use keyunitsmod
use logdat
use versions

      integer(long), save         :: epoints,iexp,maxtp,ncos
      real(dop), save             :: emin,emax,eoff,tcut,tau,dipolem, &
                                     dipole,phcorr,dt0
      character(len=16), save     :: unit,unite,ceoff
      logical(kind=4), save       :: lft,lnorm,lresol,lctrace, llinear,&
                                     lexpect

      end module autospecmod

!#######################################################################

      program autospec

use global
use aglobal
use maxv, only: maxdim
use dvrdatmod
use griddatmod
use autospecmod
use plot
use channels
use plotutils, only: plotdefs,plotmenu,databnds,dotask,initplot

      implicit none

      integer(long)           :: i,j,iauto,ierr,s,ndata,plottask,task,&
                                 plotdim,dimx,dimy,nsets,nplots
      integer(long), external :: slen
      integer(long), dimension(:), allocatable  :: idum
      real(dop)               :: sum0,sum1,sum2,cc,fac,eau,emina,emaxa,&
                                 a,b,t,t0,dt,delt,pia,mx0,mx1,mx2,mx3,&
                                 delea,e,dele,a0,b0,mom0,mom1,mom2,&
                                 sum3, asum0, asum2, sum4, sum5,&
                                 told,norm

      real(dop), dimension(1)                   :: rdum,ydum,xdum,tdum
      real(dop), dimension(:), allocatable      :: ener
      real(dop), dimension(:,:), allocatable    :: sp
      complex(dop), dimension(:), allocatable   :: auto
      character(len=80)                         :: buffer
      character(len=c5)                         :: filename
      character(len=10)                         :: punit

!-----------------------------------------------------------------------
!     Set defaults. Set up the input.
!-----------------------------------------------------------------------
      epoints = 1000
      call setunits
      call specinput

      if (.not. linter) then
         write(6,'(a,2g10.4E1,1x,a,3x,"(",2g10.4E1," au)")') &
           ' Emin, Emax : ',emin,emax,trim(unit),emina,emaxa
         write(6,'(a,f9.6,a,f8.3,a)') &
           ' E-offset   : ', eoff,' au,   ', eoff*ev,' eV'
         if( tau .lt. 9999999.9 ) then
            write(6,'(a,f9.1,i3)') ' tau,  iexp : ', tau, iexp
         else
            write(6,'(a,e9.2,i3)') ' tau,  iexp : ', tau, iexp
         endif 
         write(6,'(a,i6)') ' No. of plot-points: ', epoints 
      end if

!-----------------------------------------------------------------------
!     Determine maxdim - space in labels array
!-----------------------------------------------------------------------
      maxdim = 3

!-----------------------------------------------------------------------
!     Determine the number ot t-points on auto-file..
!-----------------------------------------------------------------------
      if(lresol .and. filein.eq.'resolution') then
         maxtp = nint(tcut)+10
      else
         maxtp=1
 3       read(20,*,end=1,err=2)
         maxtp=maxtp+1
         goto 3
 1       continue
      endif

!-----------------------------------------------------------------------
! set nplottask and ncalcpar before allocating memory for plotting routines
! dvrdat and grddat needed for label arrays
!-----------------------------------------------------------------------
      call alloc_dvrdat
      call alloc_grddat
      nplottask=3
      ncalcpar=3
      call alloc_plot

      rewind 20

      allocate(ener(0:epoints))
      allocate(sp(0:epoints,0:3))
      allocate(auto(maxtp))
      allocate(idum(maxdim))

! interactive plot variables
      lctrace = .false.
      task = 0
      dt = fs
      i=0
      asum0=0.d0
      asum2=0.d0
      tdum(1)=0
      idum(1)=0
! tdum and idum lines not present
!-----------------------------------------------------------------------
!     Read the autocorrelation function.
!-----------------------------------------------------------------------
      if(lresol .and. filein.eq.'resolution') then
         continue
      else
  9      read(20,900,end=31,err=32) buffer
         if( buffer(1:9) .eq. '#    time' ) then
            lexpect=.false.
         elseif( buffer(1:18) .eq. '# time[fs]    norm' .or. &
                 buffer(1:18) .eq. '# time[au]    norm' ) then
            lexpect=.true.
            write(6,'(a)') 'Reading expectation file!'
         else
            goto 9
         endif
         if(lexpect) then
            read(20,*,end=31,err=32) t0, norm, a0, b0
         else
            read(20,*,end=31,err=32) t0, a0, b0
         endif
         if ((abs(a0)+abs(b0)).lt.tiny) then
            if(filein(laein-5:laein) .eq. 'ctrace') then
               write(6,*) 'Input seems to be a ctrace file!'
               write(6,*) 'Using sin instead of exp !'
               lctrace=.true.
            else
               write(6,*) 'Input could be a ctrace file!'
               write(6,*) 'The autocorrelation is zero at t=0.'
               write(6,*) 'set -ctr to use sin instead of exp !'
               if(filein(laein-3:laein) .eq. 'auto') then
                  write(6,'(a,/a/a/)') '******** WARNING ********', &
                  ' The input-filename is "auto" (not "ctrace") ?'
               endif
            endif
         else if( abs(a0-1.0_dop)+abs(b0) .gt. 1.0e-9_dop ) then
            write(6,*) '############ --- WARNING --- #####################'
            write(6,*) ' Incorrect first data line '
            write(6,*) 'time =', t0,' auto =', a0,' ', b0
         end if
         if(lexpect) then
            read(20,*,end=31,err=32) t, norm, a, b
         else
            read(20,*,end=31,err=32) t, a, b
         endif
         auto(1) = dcmplx(a,b)*exp(dcmplx(0.0_dop,phcorr*t*t))
         dt0  = t-t0
         dt   = dt0*fs
         tcut = tcut - 0.99*(t-t0)
         told = t
      endif

      if(.not. lresol) then
         iauto=1
   34    continue
         if(lexpect) then
            read(20,*,end=30,err=32) t, norm, a, b
         else
            read(20,*,end=30,err=32) t, a, b
         endif
         if(abs(t-told).le.1.d-9) goto 34 ! ignoring double lines due to dynspf
         iauto = iauto+1
         told=t
         if(iauto .gt. maxtp) goto 30
         auto(iauto) = dcmplx(a,b)*exp(dcmplx(0.0_dop,phcorr*t*t)) 
         asum0 = asum0 + a**2 + b**2
         asum2 = asum2 + (a**2 + b**2)*i**2
         if( abs(t-t0-iauto*dt0) .gt. 2.5d-7 ) then
            write(6,'(/a)') ' ### Time points not equally spaced! ###'
            write(6,'(4x,a,9x,a,13x,a,11x,a)') 'no','dt','t(is)','t(should)'
            write(6,'(i6,f15.7, 2f17.7)') iauto, dt0, t, t0+i*dt0
            write(6,'(a,/a,/a,/a)') &
                 ' (Remove duplicated lines in auto if this file was', &
                 '  generated by a continuation of a crashed run.', &
                 '  If the auto file was generated by a multi packet run,', &
                 '  one cannot use plspec/autospec but must use filter.)'
            write(6,'(58a1)') ('-',j=1,58)
            stop
         endif
         if( t .gt. tcut ) goto 30
         go to 34

 33      backspace 20            ! Check if the unreadable line starts
         read(20,'(a)') buffer   ! with a '#' or with a  blank.
         if( buffer(1:1) .eq. '#' ) goto 34
         if( buffer      .eq. ' ' ) goto 34
         goto 32                 ! Goto error message

      else if (lresol) then
         asum0=1.d0
         if(i.le.1) i=maxtp
         a0 = 1.0_dop
         b0 = 0.0_dop
         do j = 1, i
            auto(j) = 1.0_dop*exp(dcmplx(0.0_dop,phcorr*t*t))
            iauto = j
            asum2=j*j
            t = j*dt/fs
            if( t .gt. tcut-1.d-9 ) goto 30
         enddo
      endif

!-----------------------------------------------------------------------

 30   continue
      tfin=t
!.....fs is set to 1.0 if unit='no'. Otherweise fs = 41.3414
      if( fs .gt. 40.0_dop ) then
         write(6,'(a,i6,a,f10.3,a)') &
          ' No. of t-points =', iauto,',   t-final =', t,' fs'
      else
         write(6,'(a,i6,a,f10.3,a)') &
          ' No. of t-points =', iauto,',   t-final =', t,' (no units)'
      endif

      if (abs(phcorr) .gt. 1.d-12) then
         do i = 2, iauto
            t=dt*(i-1)/fs
            auto(i) = auto(i)*exp(dcmplx(0.d0,-phcorr*t*tfin*0.5d0))
         enddo
      endif

!-----------------------------------------------------------------------
! initialise interactive plotting
!-----------------------------------------------------------------------
      ndata=0
      if (linter) then

!-----------------------------------------------------------------------
! interactive plotting
! open file for data to be plotted
!-----------------------------------------------------------------------
         filename=name(1:laenge)//'/tmpplot.pl'
         open(iaus,file=filename,status='unknown')

!-----------------------------------------------------------------------
! zero dummy variables
!-----------------------------------------------------------------------
         xdum(1)=0.0
         ydum(1)=0.0
         rdum(1)=0.0

         call plotdefs

         plotdim=1
         lplener=.true.

         call initplot(filename,plotdim)

!-----------------------------------------------------------------------
! set program specific data, e.g. plot tasks
! do not exceed nplottask or ncalcpar
!-----------------------------------------------------------------------
         plottask=2

         cplottask(1)='Plot spectrum: no filter'
         cplottask(2)='             : cos filter'
         cplottask(3)='             : cos^2 filter'
         do i=1,nplottask
            cptlen(i)=slen(cplottask(i))
         enddo

         axunitlab(fener)=unit
         eunit=1.0
         call keyeunits(eunit,unit,ierr)
         if (ierr .eq. 0) then
            axunit(fener)=eunit
         else
            write(6,*) 'Unknown energy unit'
            stop
         endif
     
         acalcpar(1)= 'damping time, tau, (fs)'
         acalcpar(2)= 'damping exponent (1 or 2)'
         s=slen(axunitlab(fener))
         acalcpar(3)= 'E offset ('//axunitlab(fener)(1:s)//')'
         rcalcpar(1)= tau
         rcalcpar(2)= iexp
         rcalcpar(3)= eoff

         bound(1,fener)=emin*axunit(fener)
         bound(2,fener)=emax*axunit(fener)
         ibound(1,fener)=1
         ibound(2,fener)=epoints+1
      endif

!-----------------------------------------------------------------------
! call menu
! before calling menu, change rcalcpar(3) to relevant units 
!-----------------------------------------------------------------------
  200 continue
      if (linter) then
         s=slen(axunitlab(fener))
         acalcpar(3)= 'E offset ('//axunitlab(fener)(1:s)//')'
         eoff=rcalcpar(3)
         rcalcpar(3)=eoff/axunit(fener)
         call plotmenu(task,plottask,plotdim)
         rcalcpar(3)=eoff

         if (.not. lrecalc) go to 250
         ndata=0

         if (plottask .eq. 1) then
            continue
         endif

! reset bounds
         emin=bound(1,fener)/axunit(fener)
         emax=bound(2,fener)/axunit(fener)
! set energy unit
         unit=axunitlab(fener)
         tau=rcalcpar(1)
         iexp=nint(rcalcpar(2))
         eoff=rcalcpar(3)
      endif

      dele = (emax-emin)/dble(epoints)
!-----------------------------------------------------------------------
! check for sufficient density of time-points and energy points
!-----------------------------------------------------------------------
      delt = abs(dt)/(2.0_dop*pi)
      emaxa = emax
      emina = emin
      delea = dele
      call keylowc(unit)
      call keyeunits(emaxa,unit,ierr)
      call keyeunits(emina,unit,ierr)
      call keyeunits(delea,unit,ierr)
      if(ierr .gt. 0) then
         write(6,*)' unknown unit: ', unit
         stop
      end if
      fac  = abs(emaxa-emina)*delt
      if(fac .ge. 1.0_dop) write(6,800) 1.0_dop/delt, ev/delt, fac 

      asum0 = min(sqrt(10.0_dop*asum2/asum0)/dble(iauto),1.0_dop)
      i = 5*int(fac*iauto*asum0) + 120
      if(i.gt.1.10_dop*epoints+10) then
         write(6,'(a,/a,i5,a,/a,/a,i6,/a)') &
          ' ################ WARNING ################################', &
          ' The number of plot-points (=',epoints,') may be too low.', &
          ' For high-quality plots use the -q option with an arguement', &
          ' larger than:', i, &
          ' -----------------------------------------------------------'
      endif
 
      if (.not. linter) then
         write(22,'(a,a)') '#',filein(1:laein)
         if(tau.lt.9.d+39) then
            if(lft) then
               write(22,925) trim(unit),iauto,tau,iexp,eoff
            elseif(dipole.eq.0.d0) then
               write(22,926) trim(unit),iauto,tau,iexp,eoff
            else
               write(22,927) trim(unit),dipole,iauto,tau,iexp,eoff
            endif
         else
            if(lft) then
               write(22,928) trim(unit),iauto,trim(ceoff)
            elseif(dipole.eq.0.d0) then
               write(22,929) trim(unit),iauto,trim(ceoff)
            else
               write(22,930) trim(unit),dipole,iauto,trim(ceoff)
            endif
         endif
         write(22,'(a)') '#'
         punit = '['//trim(unit)//']   '
         if(llinear) then
            write(22,931) punit,"g3", "g4 (g'0)", "g5 (g'1)", "g2"
         else
            write(22,932) punit,'g0', 'g1', 'g2', "g5 (g'1)"
         endif
      endif

!-----------------------------------------------------------------------
! calculate fourier transform / spectrum
!-----------------------------------------------------------------------
      tau = tau*fs
      pia = pi/dble(2*iauto+2)
      
      mx0 = 0.0_dop
      mx1 = 0.0_dop
      mx2 = 0.0_dop
      mx3 = 0.d0
      mom0 = 0.0_dop
      mom1 = 0.0_dop
      mom2 = 0.0_dop

      do j = 0, epoints
         e = emin + j*dele
         eau = e
         call keyeunits(eau,unit,ierr)
         eau = eau - eoff
         if (.not.lctrace) then
            sum0 = 0.5_dop*a0
            sum1 = 0.5_dop*a0
            sum2 = 0.5_dop*a0
            sum3 = 0.5_dop*a0
            sum4 = 0.5_dop*a0
            sum5 = 0.5_dop*a0
         else
            sum0 = 0.0_dop
            sum1 = 0.0_dop
            sum2 = 0.0_dop
            sum3 = 0.0_dop
            sum4 = 0.0_dop
            sum5 = 0.0_dop
         endif            
         do i = 1, iauto
            t = i*dt
            if (.not.lctrace) then
               cc   =  dble( exp(dcmplx(0.0_dop,eau*t))*auto(i) )&
                    * exp( -(t/tau)**iexp )
            else
               cc = -sin(eau*t)*dble(auto(i))&
                    * exp( -(t/tau)**iexp )
            endif
            sum0 = sum0 + cc
            sum1 = sum1 + cc*cos(pia*i)
            sum2 = sum2 + cc*cos(pia*i)**2
            sum3 = sum3 + cc*(iauto+1-i)/dble(iauto)
            sum4 = sum4 + cc*(iauto+1-i)/dble(iauto+1)
            sum5 = sum5 + cc*( (iauto+1-i)*cos(2*pia*i)/dble(iauto+1) &
                        +  sin(2*pia*i)/pi )
         end do
         if(llinear) then
            sum0 = sum3
            sum3 = sum2
            sum1 = sum4
            sum2 = sum5
         else
            sum3 = sum5
         endif
         eau = eau + eoff
         if( unit .eq. 'nmwl' ) eau = eau**3
         eau = eau*dipolem        ! Multiply with dipole moment for -Mb
         if( lft ) eau = 1.0_dop
         sp(j,0) = eau*sum0*dt/pi
         sp(j,1) = eau*sum1*dt/pi
         sp(j,2) = eau*sum2*dt/pi
         sp(j,3) = eau*sum3*dt/pi
         mx0 = max(mx0,sp(j,0))
         mx1 = max(mx1,sp(j,1))
         mx2 = max(mx2,sp(j,2))
         mx3 = max(mx3,sp(j,3))
         mom0 = mom0 + sp(j,1)
         mom1 = mom1 + sp(j,1)*e
         mom2 = mom2 + sp(j,1)*e*e
      end do
      mom0 = mom0*delea
      mom1 = mom1*delea
      mom2 = mom2*delea
      mom1 = mom1/mom0
      mom2 = mom2/mom0
      write(6,'(4(a,es11.4))') 'Moments: M0 =',mom0, &
         ',  M1 =',mom1,',  M2 =',mom2,',  width =',sqrt(mom2-mom1*mom1)

      if( lnorm )  then
         do j = 0, epoints
            sp(j,0) = sp(j,0)/mx0
            sp(j,1) = sp(j,1)/mx1
            sp(j,2) = sp(j,2)/mx2
            sp(j,3) = sp(j,3)/mx3
         end do
      end if

      if (.not. linter) then
         do j = 0, epoints
            e = emin + j*dele
            write(22,920) e, sp(j,0), sp(j,1), sp(j,2), sp(j,3)
         end do
      else
         do j = 0, epoints
            ener(j) = emin + j*dele
            call keyeunits(ener(j),unit,ierr)
         end do
      endif

  250 continue

      if( .not.linter ) stop
      
!-----------------------------------------------------------------------
! interactive plotting
!-----------------------------------------------------------------------
      if (lrebound) then
         dimx=epoints+1
         dimy=1
         nsets=1
         nplots=1
         call databnds(sp(0,0),dimx,dimy,nsets,nplots,dimx,1,1,1)
      endif

!-----------------------------------------------------------------------
! do task
! TASK=0 STOP  else return to menu
!-----------------------------------------------------------------------
      axislab(1)=&
         'Energy ['//axunitlab(fener)(1:slen(axunitlab(fener)))//']'
      axislab(3)='Intensity'

      if (task .gt. 0) then
         if (plottask .eq. 1) then
            plttitle='Spectrum: no filter'
            call dotask(task,plottask,plotdim,ener,ydum(1),sp(0,0),&
                    dimx,dimy,nsets,nplots,dimx,1,1,1,xdum,1,&
                    tdum,1,ener,dimx,rdum,1,1,1,1,idum)
         else if (plottask .eq. 2) then
            plttitle='Spectrum; cos filter'
            call dotask(task,plottask,plotdim,ener,ydum(1),sp(0,1),&
                    dimx,dimy,nsets,nplots,dimx,1,1,1,xdum,1,&
                    tdum,1,ener,dimx,rdum,1,1,1,1,idum)
         else if (plottask .eq. 3) then
            plttitle='Spectrum; cos^2 filter'
            call dotask(task,plottask,plotdim,ener,ydum(1),sp(0,2),&
                    dimx,dimy,nsets,nplots,dimx,1,1,1,xdum,1,&
                    tdum,1,ener,dimx,rdum,1,1,1,1,idum)
         endif

! change energy units of read in energy shift to au
         if (task .eq. 802) then
            rcalcpar(3)=rcalcpar(3)*axunit(fener)
         endif
         go to 200
      endif

      close(iaus,status='delete')
         
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_plot
      call dealloc_plot
      if(allocated(ener)) deallocate(ener)
      if(allocated(sp)) deallocate(sp)
      if(allocated(auto)) deallocate(auto)

      stop

 2    write(6,'(a)') ' Error reading auto file'
      stop

 800  format(' Warning: Energy interval too large for time-',&
           'resolution.',/,' max-del-E =', 1pe11.3,' au, ',&
           1pe11.3,' eV',/,' You will see the spectrum',&
           0pf7.2,' times'/)
 900  format(a80)
 920  format(5(1x,1pe17.8))
 925  format('# ',a,', no energy prefactor, t-points =',i6, &
         ', damping tau =', 1pe11.3,'fs, order =',i3,',  ',a)
 926  format('# ',a,', with energy prefactor, t-points =',i6, &
         ', damping tau =', 1pe11.3,'fs, order =',i3,',  ',a)
 927  format('# ',a,', -Mb',1pe10.3,', t-points =',i6, &
             ', damping tau =',1pe11.3,'fs, order =',i3,',  ',a)
 928  format('# ',a,', no energy prefactor, t-points =',i6, &
             ', no damping,  ',a)
 929  format('# ',a,', with energy prefactor, t-points =',i6, &
             ', no damping,  ',a)
 930  format('#    ',a16,' damping tau =', g12.4,'fs,  order =',i3, &
             ' , t-points =',i7)
 931  format('#    Energy',15x,a,16x,a,10x,a)
 932  format('# Energy',a,16x,a,15x,a,15x,a,13x,a)
      
      stop

 31   write(6,'(a)') ' No data. Check the input (auto) file.'
      write(6,'(a,/a)') ' buffer:', buffer
      write(6,'(a,3e18.6)') ' t0, a0, b0 :', t0, a0, b0
      write(6,'(a,3e18.6)') ' t1, a1, b1 :', t, a, b
      stop
 32   write(6,'(a)') &
         ' Error occurred when reading the input (auto) file.'
      stop

      contains

!#######################################################################

      subroutine specinput

use global
use aglobal
use autospecmod

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: i,iarg,ierr
      character(len=80)       :: buf
      integer(long)           :: ncos2

!-----------------------------------------------------------------------

      filein = '  '
      name   = './'
      fileout = '  '
      ceoff   = ' ' 

      eoff = 0.0_dop
      tcut = 9.9e+49_dop
      dipole=0.0_dop
      dipolem=1.0_dop
      phcorr=0.0_dop
      unit = 'au'
      lft    = .true.
      lnorm  = .false.
      lgnu   = .false.
      lresol = .false.
      lctrace = .false.
      llinear = .false.

      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf .eq. '-inter' ) then
            linter=.true.
         else if( buf .eq. '-r' ) then
            lresol=.true.
         else if( buf .eq. '-ctr' ) then
            lctrace=.true.
         else if( buf .eq. '-lin' ) then
            llinear=.true.

         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf(1:4) .eq. '-FT ' ) then
            lft = .true.
         else if( buf(1:4) .eq. '-EP ' ) then
            lft = .false.
         else if( buf(1:4) .eq. '-Mb ' ) then
            lft = .false.
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) dipole
            if (ierr .ne. 0) then
               write(6,*)'Argument for -Mb must be real number. Not :'
               write(6,*) buf
               stop
            end if
!........On input dipolem = Norm of D*Psi (see MCTDH log file).
            dipolem=2.689093_dop*dipole*dipole
!...........2.689093_dop=pi*e^2*r0^2/(3*c*eps0*hbar*Mb).
!........On output dipolem = conversion factor to Mb. 1Mb = 10^-18 cm^2

         else if( buf(1:6) .eq. '-nofs ' ) then
            fs = 1.d0
         else if( buf(1:3) .eq. '-n ' ) then
            lnorm = .true.
         else if( buf(1:4) .eq. '-ph ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) phcorr
            if (ierr .ne. 0) then
               write(6,*)'Argument for -ph must be real number. Not :'
               write(6,*) buf
               stop
            end if
         else if( buf(1:3) .eq. '-t ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) tcut
            if (ierr .ne. 0) then
               write(6,*)'Argument for -t must be real number. Not :'
               write(6,*) buf
               stop
            end if
         else if( buf(1:3) .eq. '-g ' .or. buf(1:3) .eq. '-gg') then 
            lgnu = .true.                                ! -gg from plspec
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ncos
            if (ierr .ne. 0) then
               write(6,*)&
              'Argument for -g must be a non-negative integer.'
               write(6,*)'Not :', buf
               stop
            end if
         else if( buf(1:3) .eq. '-p ' .or.  buf(1:3) .eq. '-q ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) epoints
            if (ierr .ne. 0) then
               write(6,*)'Argument for -p must be real number. Not :'
               write(6,*) buf
               stop
            end if
         else if( buf(1:3) .eq. '-e ' ) then
            i = i+1
            call mygetarg(i,buf)
            ceoff='-e '//trim(buf)
            read(buf,*,iostat=ierr) eoff
            if (ierr .ne. 0) then
               write(6,*)'Argument for -e must be real number. Not :'
               write(6,*) buf
               stop
            end if
            i = i+1
            call mygetarg(i,unite)
            ceoff='-e '//trim(buf)//' '//trim(unite)
            call keylowc(unite)
            call keyeunits(eoff,unite,ierr)
            if(ierr .gt. 0) then
               write(6,*)' unknown eoff-unit: ', unite
               stop
            end if
            unite = 'au'
         else if( buf .eq. '-gnupl4 ') then 
            lgnu4 = .true.                 ! gnuplot ver < 5
         else if( buf(1:3) .eq. '-h ' .or. buf(1:3) .eq. '-? ' ) then
            write(6,'(/,a/,(9x,a))') &
       'Purpose: Computation of the spectrum by fourier-transforming',&
       'the autocorrelation function.',&
       'Three spectra are generated acording to the weight functions',&
       'cos^n(pi*t/(2*T)) (n=0,1,2). The option -lin allows to use',&
       'a second set of filters. In plspec the filters of this',&
       'second set are enabled through the option -g3, -g4, or -g5.',&
       'Additionally there is the weight function exp(-(t/tau)^iexp).',&
       'The conversion factor from damping time to FWHM energy width',&
       'is 1.32 eV fs and 2.2 eV fs for iexp=1 or 2, respectively.',&
       'tau and iexp can be skipped if no damping is required.'
            write(6,'(/,2a,/18x,a)')&
       'Usage : autospec [-f -i -o -e -n -r -ctr -ph -p -q -g -t ',&!no -ctr .507
       '-FT -EP -Mb -ver -h -? -inter -gnupl4] ',& ! no -inter .507
       ' [emin emax unit tau iexp]'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a)') &
               ' -f FILE : The autocorrelation is read '&
              ,'from file FILE rather than from ./auto'&
              ,'  The string FILE may be a relative or full path-name.'
            write(6,'(a)') ' -i DIR  : Take DIR as input directory.'
            write(6,'(2a,/,9x,a/,9x,a)') &
               ' -o FILE : The spectrum is written to',&
               ' file FILE.pl','  rather than to ./spectrum.pl',&
               '  The string FILE may be a relative or full path-name.'
            write(6,'(2a,/,13x,a)') &
               ' -e R Unit : The offset energy is set to R in ',&
               'units Unit; e.g. 0.3 ev',&
               'Note, a positive R shifts the spectrum to the right.'
            write(6,'(2a)') &
               ' -n      : The maxima of the three spectra are',&
               ' normalised to 1.'
            write(6,'(a)') &
               ' -r      : Plot the resolution rather than the spectrum'
            write(6,'(a)') &
               ' -ph  ph : Use phase correction exp(i*ph*1.0e-6_dop*t^2)'
            write(6,'(2a,i5/,9x,a)') &
               ' -p I    : The no. of plot-points is I rather',&
               ' than', epoints,&
               '  Use "-p 2000" for final plots'
            write(6,'(a)')  ' -q I    : Same as -p.'
            write(6,'(a,10(/11x,a))') &
           ' -lin    : A second set of filter functions is used, namely', &
           'g(t) = cos^3(pi*t/(2*T) (column 2), (g3)', &
           'g(t) = 1 - t/T (column 3), (g4)', &
           'g(t) = (1-t/T)*cos(pi*t/T) + sin(pi*t/T)/pi (column 4),(g5).', &
           'Additionally, column 5 displays g2.', &
           'The filters g4 and g5 yield a positive semi-definite', &
           'Fourier-transform. When using plspec, these three filters', &
           'are chosen by the option -g3, -g4, or -g5, respectively.', &
           'When the option -lin is not set, the columns 2,3, and 4', &
           'refer to the filters cos^0, cos^1, and cos^2, respectively.', &
           'I.e. g0, g1, and g2. Additionally, column 5 displays g5.'
            write(6,'(2a,/11x,2a)') &
               ' -g ncos : GNUplot command lines are written to ',&
               'the output file.','ncos is the exponent of the',&
               ' cosine damping function.'
           write(6,'(2a)') &
               ' -t tcut : The autocorrelation function is read only',&
               ' up to t=tcut'
            write(6,'(2a/,11x,a)') ' -FT     : ',&
               'No multiplication with the energy prefactor. Fourier',&
               'transform only. (default!)'
            write(6,'(2a)') ' -EP     : Multiplication with the ',&
               'energy prefactor.'
            write(6,'(2a,/a)') ' -Mb norm: Absorption spectrum in Mb.',&
             ' norm is ||D*Psi|| (in au)',&
             '           where D is the Dipole moment. -Mb sets -EP.'
            write(6,'(a,/a)') &
             ' -ctr    : Perform a sin-transform rather than an exp one.', &
             '           (This is for ctrace files.)'
            write(6,'(a,a)') &
             ' -nofs   : No transformation to fs. Use when', &
                       " 'time-not-fs' was set in mctdh."
            write(6,'(a)')&
               ' -ver    : Version information about the program '
            write(6,'(a)')&
               ' -inter  : Interactive plotting '
            write(6,'(a)')&
               ' -gnupl4  : Use gnuplot version < 5 for plotting '
            write(6,'(a,/a)') &
               ' -h      : Print this help text. ',&
               ' -?      : Print this help text. '
            write(6,'(2a/,11x,a/2a/2a/2a/11x,a/2a/2a/2a/2a/a)') &
               'Following the options one may provide the ',&
               'arguments (separated by blank):',&
               'Emin, Emax, Unit, tau, iexp ',&
               'If not enough arguments are given, the user is ',&
               'prompted for the missing ','arguments. ', &
               'If tau is not given, tau=0 is assumed.', &
               "Use the Unit 'no' if the propagation is run with the ", &
               "keywords: 'time-not-fs','energy-not-ev'.",&
               'When using the options -EP or -Mb one must make sure ',&
               'that the zero point','of the energy is set ',&
               'appropriately. Use the option -e if the total energy',&
               'differs from photon energy. ',&
               'Note: the entered energy is added (not substracted).',&
               '1 Mb = 10^-18 cm^2. The norm ||D*Psi|| can be ',&
               'found in the log file','of the operate run.'
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            if(ichar(buf(2:2)).ge.46 .and. ichar(buf(2:2)).le.57) &
                 goto 111     ! Found a negative number, not an option.
            write(6,*) 'unknown option : ', buf(1:50)
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      
 111  read(buf,*,iostat=ierr) emin
      if (ierr .ne. 0 .or. abs(emin) .gt. 1.0e55_dop) then
         write(6,*)&
              'The first argument (emin) must be real number. Not :'
         write(6,*) buf(1:60)
         stop
      end if

      i=i+1
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) emax
      if (ierr .ne. 0) then
       write(6,*)'The second argument (emax) must be real number. Not :'
       write(6,*) buf
       stop
      end if

      i=i+1
      if (iarg .lt. i) goto 250
      call mygetarg(i,buf)
      unit = trim(buf)
!.....unit = 'no' is to be used, when the keywords 'time-not-fs' and
!.....'energy-not-ev' are set during the propagation. 
      if(unit .eq. 'no' ) then
         unit = 'au'
         fs = 1.0_dop
      endif

      i=i+1
      if (iarg .lt. i) goto 300
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) tau
      if (ierr .ne. 0) then
       write(6,*)'The forth argument (tau)  must be real number. Not :'
       write(6,*) buf
       stop
      end if

      i=i+1
      if (iarg .lt. i) goto 400
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) iexp
      if (ierr .ne. 0) then
       write(6,*)'The fifth argument (iexp)  must be an integer. Not :'
       write(6,*) buf
       stop
      end if
      goto 500

!-----------------------------------------------------------------------
!  Prompting for missing input
!-----------------------------------------------------------------------
 200  write(6,'(a)') ' E-min, E-max, unit =? (e.g:  2., 5., ev)'
      read(5,*) emin, emax, unit
      goto 300

 250  write(6,'(a)') ' Energy unit? (e.g: ev, cm-1, nmwl, au, kcal/mol)'
      read(5,*)  unit

! 300  write(6,'(a)') ' tau =? (damping time in fs)'
!      read(5,*,err=500,end=500) tau
 300  write(6,'(a)') ' No damping is assumed (tau=0)'
      tau=0.d0

 400  if( abs(tau) .le. 1.0e-9_dop ) then
         tau  = 1.0e+40_dop
         iexp = 1
      else
         write(6,'(a)') ' iexp =? (damping order)'
         read(5,*)  iexp
      end if

!-----------------------------------------------------------------------

 500  if( abs(tau) .le. 1.0e-9_dop ) then
         tau  = 1.0d20
         iexp = 1
      end if
      if( iexp .le. 0 ) then
         write(6,'(/a,i3,a/,a/)') &
           '### WARNING : iexp =',iexp,' is not allowed! (must be > 0)', &
           '### Input a zero for tau, if no damping is wanted.'
         stop
      endif
      if( iexp .gt. 2 ) then
         write(6,'(/a,i3,a/)') &
            '### WARNING : unusual damping exponent, iexp =',iexp,' ###'
      endif

      if( .not.lft  .and. (emin.lt.0.0_dop .or. emax.lt.0.0_dop) ) then
         write(6,'(2a)') &
            ' Negative energies are allowed only when', &
            ' -EP is not set, but -FT is set.'
         stop
      end if

!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
! Otherwise take name from filein
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         filein=name(1:laenge)//'/auto'
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

      open(20,file=filein(1:laein),form='formatted',&
              status='old',err=98)
      if(laein .le. 58 ) then
      write(6,'(2a)') ' The  inputfile is : ',filein(1:laein)
      else
      write(6,'(a)') ' The inputfile is : '
      write(6,'(a)')   filein(1:laein)
      end if

  95  if (fileout .eq. ' ') then
         fileout=name(1:laenge)//'/spectrum.pl'
         laeout=index(fileout,' ')-1
      else
         laeout=index(fileout,' ')-1
      endif
      if (fileout(laeout:laeout) .eq. '/') then 
         fileout = fileout(1:laeout)//'spectrum.pl'
         laeout  = laeout + 11
      else if( fileout(laeout-2:laeout) .ne. '.pl' ) then
         fileout = fileout(1:laeout)//'.pl'
         laeout  = laeout + 3
      end if
      call abspath(fileout,laeout)
      if (.not. linter) then
         open(22,file=fileout(1:laeout),form='formatted',&
              status='unknown',err=99)
         if(laeout .lt. 55 ) then
            write(6,'(2a)') ' The outputfile is : ',fileout(1:laeout)
         else
            write(6,'(a)') ' The outputfile is : '
            write(6,'(a)')   fileout(1:laeout)
         end if
         if(lgnu) write(6,'(a,i2)') &
            ' GNUPLOT command lines written for n-cos =',ncos 
      endif
      if(lft) write(6,'(a)')      ' No prefactor!' 
      if(.not.lft) write(6,'(a)') ' With prefactor!' 
      if(lnorm) write(6,'(a)') ' Spectrum will be normalised' 

      if( lgnu .and. .not. linter) then
         if( ncos .lt. 0 .or.  ncos .gt. 2 ) then
            write(6,'(a)') '#########################################'
            write(6,'(a,i3)') ' illegal value of ncos =', ncos
            write(6,'(a,i3)') ' ncos must be 0, 1 or 2 '
            write(6,'(a)') '#########################################' 
            stop
         end if
         ncos2=ncos+2
         if(llinear) ncos=ncos+3
         i = index(unit,' ') - 1
         write(22,'(a)') 'set style data lines' 
         write(22,'(a)') 'set nologscale' 
         write(22,'(a)') 'set xzeroaxis' 
         write(22,'(a,e14.7,a,e14.7,a)') 'set xr [',emin,':',emax,']'
!........fs is set to 1.0 if unit='no'. Otherweise fs = 41.3414
         if(fs.gt.40.0_dop) then
            write(22,'(3a)') 'set xlabel "Energy[',unit(1:i),']"'
         else
            write(22,'(3a)') 'set xlabel "Energy"'
         endif
         if( tau .gt. 9.9e+6_dop ) then
            write(22,'(a,i2,a,3x,a,f9.2,a)') &
                 'set title "Spectrum,  n-cos =', ncos,&
                 ',', filein(1:laein),phcorr,'"' 
         else
            write(22,'(a,i2,a,f9.1,i3,a,3x,a,f9.2,a)') &
                 'set title "Spectrum,  n-cos =', ncos,&
                 ', tau :', tau, iexp,',', filein(1:laein-4),phcorr,'"'
         end if 
         write(22,'(a,i1,a)') 'plot "-" us 1:',ncos+2,' notit'
         write(22,'(a)') ' ' 
      end if
      phcorr = phcorr*1.0e-6_dop

      return 
      
 98   if(lresol .and. tcut.lt.1.0e+49_dop) then
         filein='resolution'
         laein=10
         goto 95
      endif
      write(6,'(a)')' Cannot open input (auto) file : '
      write(6,'(a)') filein
      stop
      
 99   write(6,'(a)')' Cannot open output (spectrum) file : '
      write(6,'(a)') fileout
      stop

      end subroutine specinput

      end program autospec

