!#######################################################################
!
! Checks a SQL database, clears out duplicated entries,outputs info
!
!#######################################################################

      program checkdb

      use maxv
      use decimal, only: sip, dop, long
      use db_sql
      use sqlite
      use global 
      use versions
      use dd_db, only: distdb1
      use dirdyn
      use keyunitsmod
      use channels, only: idump

      implicit none

      integer(long), external  :: myiargc
      character(len=10)        :: buf,string
      real(dop)                :: quantver
      real(dop)                :: minver
      integer(long)                               :: ierr,nddstate
      character(len=c5)                           :: dbname,dbname1,filename
      integer(long)                               :: i,iarg,timestamp
      character(len=c3)                           :: routine
      type(SQLITE_DATABASE)                       :: dbin
      logical(kind=4)                             :: busy
! options
      integer(long)                               :: nmaxrec = 0, &
                                                     nignore = 0
      real(dop)                                   :: ener = 99999.0_dop
      logical(kind=4)                             :: lbatch = .false.
      logical(kind=4)                             :: lexist = .false.
      logical(kind=4)                             :: ldupe = .false., &
                                                     lclean = .false., &
                                                     lstclean = .false., &
                                                     lcutpes = .false., &
                                                     lrddb = .false.,&
                                                     lcreate = .false.
! system clock
      real,save    :: clockrate 
      integer      :: cr,cm

!Version of the sql database 1 - Simon, 2 - new
      integer(long)                                :: sql_vers 

! First initialize the system_clock
      call system_clock(count_rate=cr)
      call system_clock(count_max=cm)
      clockrate = real(cr)

! Setting defaults
      dbname = './'
      routine = 'checkdb'
      busy = .true.

! read options
      iarg = myiargc()

      i=0
      do while (i .lt. iarg)
         i=i+1
         call mygetarg(i,buf)
         if ( buf .eq. '-ver' ) then
            call wrversion(6)
            stop
         else if ( buf .eq. '-i' ) then
            i=i+1
            call mygetarg(i,dbname)
         else if ( buf .eq. '-batch' ) then
            lbatch = .true.
         else if ( buf .eq. '-mindb' ) then
            i=i+1
            call mygetarg(i,buf)
            if (i .gt. iarg .or. buf(1:1) .eq. '-') then ! no argument
               i=i-1
               ddmindb = 0.20
            else
               read(buf,*) ddmindb
            endif
         else if ( buf .eq. '-d' ) then
            ldupe = .true.
         else if ( buf .eq. '-c' ) then
            lclean = .true.
         else if ( buf .eq. '-sc' ) then
            lstclean = .true.
         else if ( buf .eq. '-rd' ) then
            lrddb = .true.
         else if ( buf .eq. '-cre' ) then
            lcreate = .true.
         else if ( buf .eq. '-ener' ) then
            lcutpes = .true.
            i=i+1
            if (i .le. iarg) then
               call mygetarg(i,buf)
               read(buf,*,iostat=ierr) ener
               if (ierr .gt. 0) then
                  write(6,*) 'Error: maximum energy (ener) must be a number.'
                  stop
               endif
            else
               write(6,*) 'Error: must give maximum energy (ener).'
               stop
            endif

            call mygetarg(i+1,buf)
            if (buf .eq. ',') then
               i=i+2
               if (i .le. iarg) then
                  call mygetarg(i,buf)
                  call keylowc(buf)
                  call keyeunits(ener,buf,ierr)
                  if (ierr .eq. 1) then
                     write(6,*) 'Error: no energy unit after comma (ener).'
                     stop
                  endif
               else
                  write(6,*) 'Error: no energy unit after comma (ener).'
                  stop
               endif
            endif

         else if ( buf .eq. '-r' ) then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*) nignore
         else if ( buf .eq. '-mx' ) then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*) nmaxrec
         else if ( buf .eq. '-h' .or. buf(1:2) .eq. '-?') then
            write(6,'(/,78a1,/)')  ('*',i=1,78)
            write(6,'(a)') ' Analyse program "checkdb" '
            write(6,'(1x,a)')&
       'Purpose : Checks and cleans an SQL DD DB for validity.'
            write(6,'(1x,a,/21x,a)') &
       'Usage   : checkdb [-batch -i NAME -d -c -sc -mindb R -r I -mx I',&
                 '-ener R [, unit] -r I -mx I -rd -cre -ver -h -?] '
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(/,a)') ' Options : '
            write(6,'(a)') &
            ' -i NAME : The DB is read from directory NAME '
            write(6,'(a)') &
            ' -d      : The DB is cleaned by removing duplicate points.'
            write(6,'(a,f6.2,a)') &
            ' -mindb R  : tolerance used for duplicate points is R&
               & (default: ',ddmindb,')'
            write(6,'(a)') &
            ' -c      : The DB is cleaned by removing points from&
           & failed calculations.'
            write(6,'(a,/10x,a)') &
            ' -sc     : The DB is strongly cleaned by removing points from&
           & failed and',' interpolated calculations.'
            write(6,'(a)') &
            ' -r nignore : The DB entry nignore is removed '
            write(6,'(a)') &
            ' -mx nmaxrec : Write only the first nmaxrec records to the DB '
            write(6,'(a,/10x,a)') &
            ' -ener R [, unit] : Remove points with adiabatic energies&
           & above R.',' Note if units are given a space is needed before&
           & the comma.'
            write(6,'(a)') &
            ' -rd     : Read DB to test integrity of records'
            write(6,'(a)') &
            ' -cre    : create template.dat and QC frequency file (if present)'
            write(6,'(a)') &
            ' -batch  : Batch mode - no output to the screen. '
            write(6,'(a)')&
            ' -ver    : Version information about the program.'
            write(6,'(a,/a)') &
            ' -h,-?   : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else
            write(6,'(2a)') 'Unknown argument :',buf
            stop
         endif
      end do

      laenge=index(dbname,' ')-1
      call abspath(dbname,laenge)
      dbname1 = trim(dbname) // '/database.sql'

      !Read write only, version and arrays are dummies
      ! call makeflagarray(flagarray)
      ! call makeinfoarray(nddstate, infoarray)
      call dbopen(dbname1,dbin,sqlvers,quanticsver,infoarray,flagarray, 1)

      if (lbatch) then
         ilog=2
         open(ilog,file=trim(dbname)//'/checkdb.log')
      else
         ilog=6
      endif
      !Check which version this database is using
      call dbgetallversion(dbin,sql_vers,quantver)

      call dbrdrefdb(dbin, natm, nroot, nddstate, nbasis, dde0,&
                     dimref, nfam, dercpdim, dbpntgp, nsymop,&
                     nsocstate, nddmult)
      if (natm.le.0) then
         write(ilog,*)'FATAL: number of atoms is wrong: ', natm
         stop
      endif
      if (.not.lbatch) then
         write(ilog,*)
         write(ilog,*) 'Database name:  ', trim(dbname1)
         write(ilog,'(a,f6.2)') ' Quantics version :', quantver
         write(ilog,'(a,i6)') ' SQL version      :', sql_vers
         write(ilog,'(3x,a14,i0)')    'No. of atoms: ', natm
         write(ilog,'(3x,a14,i0)')    'Nroot:        ', nroot
         write(ilog,'(3x,a14,i0)')    'Nddstate:     ', nddstate
         write(ilog,'(3x,a14,i0)')    'Nbasis:       ', nbasis
         write(ilog,'(3x,a10,f20.10)')'dde0:         ', dde0
         write(ilog,'(3x,a14,i0)')    'dimref:       ', dimref
         write(ilog,'(3x,a14,i0)')    'nfam:         ', nfam
         write(ilog,'(3x,a14,i0)')    'dercpdim:     ', dercpdim
         write(ilog,'(3x,a14,i0)')    'dbpntgp :      ', dbpntgp
         if (dbpntgp .gt. 0) then
            write(ilog,'(3x,a14,i0)') '  nsymop:     ', nsymop
         endif
         write(ilog,'(3x,a14,i0)')    'Nsocstate :      ', nsocstate
         if (nsocstate .ne. nddstate) then
            write(ilog,'(3x,a14,7i4)')   &
                                '  Nddmult :      ', (nddmult(i),i=1,7)
         endif

      endif


      call dbgetentries( dbin, ilog, lpesdb, lgradb, lhesdb, lapesdb, lagradb, &
                         lahesdb, ldipdb, ladipdb, lmocodb, lnactdb, ltransdb, &
                         lnmdb, lcasocdb, lsymdb, lcpesdb, lctransdb, lcivecdb, &
                         ltemplate, lqcfreq)
      call dbgetnrec( dbin, dbnrec )
      write(ilog,'(1x,/,a19,i0)')   'Number of records: ', dbnrec
      write(ilog,*)

      if (dbnrec .eq. 0) stop

! Set variables
      ndofddpes = 3*natm
      ndofdd = ndofddpes - 6   ! Assuming non-linear (should store in DB?)

! Set sizes and allocate memory
      dbmemdim =dbnrec
      maxdim = natm*3
      maxsta = max(nddstate,nsocstate)
      maxmoco = nbasis
      call alloc_dddb
      call alloc_dirdyn_rediab(dbnrec,nddstate,ndofddpes)

! Check for duplicates (defined by < mindb)
      if (ldupe) call dbdeduplicate( dbin, ddmindb )

! Delete a single record
      if (nignore .gt. 0) call dbdelrec( dbin, nignore )

! Truncate DB
      if (nmaxrec .gt. 0) call dbtrunc( dbin, nmaxrec )

! Clean DB (remove failed points)
      if (lclean) call dbclean( dbin )

! Strong clean DB (remove failed and interpolated points)
      if (lstclean) call dbstclean( dbin )

! Ignore points with S0 adiabatic energies above a maximum value
      if (lcutpes) call dbpescut( dbin, ener )

! Read all records 
      if (lrddb) call readdb

      if (lcreate) then
! Create template.dat
         filename = trim(dbname) // '/template.dat'
         inquire(file=filename,exist=lexist)
         if (lexist) then
            write(6,'(a)') 'Template.dat file exists. Overwrite (Y/N)?'
            read(5,*) string
            if (trim(string) .eq. 'N' .or. trim(string) .eq. 'n') go to 100
         endif

         write(6,'(a)') 'Opening file : ',trim(filename)
         open(idump,file=trim(filename),form='formatted',status='unknown')
         call dbrdtemplate( dbin, idump )
         close(idump)

  100    continue

! Create qcfreq.out
         if (lqcfreq) then
            filename = trim(dbname) // '/qcfreq.out'
            inquire(file=filename,exist=lexist)
            if (lexist) then
               write(6,'(a)') 'Qcfreq.out file exists. Overwrite (Y/N)?'
               read(5,*) string
               if (trim(string) .eq. 'N' .or. trim(string) .eq. 'n') go to 110
            endif

            write(6,'(a)') 'Opening file : ',trim(filename)
            open(idump,file=trim(filename),form='formatted',status='unknown')
            call dbrdqcfreq( dbin, idump )
            close(idump)

  110       continue
         endif
      endif

! End
      call dealloc_dddb
      call dealloc_dirdyn_rediab
      if (lbatch) close(ilog)

      stop

contains

!-----------------------------------------------------------------------
! DBDEDUPLICATE
!
!     Go through database and remove all entries that have
!     duplicated geometries
!     If dist is given it is the threshold distance below which we
!     assume two geometries are the same.
!     If it's not given 'tiny' is used
!-----------------------------------------------------------------------

      subroutine dbdeduplicate( db, ddmindb )

      implicit none

      type(SQLITE_DATABASE), intent(inout)      :: db
      real(dop), intent(in), optional           :: ddmindb
      real(dop)                                 :: mindist,mindisp
      real(dop)                                 :: distr,dist,disp
      integer(long)                             :: irec,jrec,nrec,&
                                                   ndof,natm,iloc,ndupe
      logical(kind=4)                           :: dupes
      logical(kind=4),dimension(:),allocatable  :: dupe
      real(dop), dimension(:),allocatable       :: gx,gx2
      logical(kind=4)                           :: busy=.true.
      character(len=c5)                         :: message
      character(len=c3)                         :: routine
      character(len=250)                        :: cmd
      character(len=3)                          :: goahead

      real(dop), external                       :: dnrm2
!
! initialise variables
! r1 will be min distance. iloc is location of minimum distance
!
      if (present(ddmindb)) distr = ddmindb
      if (.not.present(ddmindb)) distr = tiny

      call dbgetnrec( db, nrec )

      dupes = .false.
      if (nrec.lt.2) return
      do while (busy)
         call sqlite3_begin(db, SQLITE_TRANS_IMMEDIATE)
         busy = db%error.eq.SQLITE_BUSY
      enddo
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while beginning transaction: ', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif
      call dbrdrefdb(db,natm,nroot, nddstate,nbasis,dde0,dimref,nfam,&
           dercpdim,dbpntgp,nsymop,nsocstate,nddmult)
      call dbrdgeo( db, natm, 1,dbnrec, dbgeo )
      ndof = 3*natm
      allocate(gx(ndof))
      allocate(gx2(ndof))
      allocate(dupe(nrec))
      dupe = .false.
      ndupe=0
      do irec = 2, dbnrec
         mindisp = 99999.0_dop
         mindist = 99999.0_dop
         gx(1:ndofddpes) = dbgeo(1:ndofddpes,irec)

         do jrec = 1, irec-1
            if (dupe(jrec)) cycle

            gx2(1:ndofddpes) = dbgeo(1:ndofddpes,jrec)
            call distdb1(gx,gx2,dist,disp)

            if (dist .lt. mindist) then
               iloc = jrec
               mindist = dist
            endif
            mindisp=min(mindisp,disp)
         enddo

         if (mindisp .lt. distr) then
            dupe(irec) = .true.
            ndupe=ndupe+1
            write(ilog,'(a,i0,1x,i0,a,2f8.4)') 'DB Records too close ',&
                  irec,iloc,' Distance / Displacement = ',mindist,mindisp
         endif

      enddo

      dupes = any(dupe)
      if (.not.dupes) then
         Write(6,'(/,a)') ' No redundant records '
         return
      else
         Write(6,'(/,a,i7)') ' No. redundant records :',ndupe
      endif

      if (.not. lbatch) then
         Write(6,'(/,a)') ' Delete redundant records ? <Y/N> '
         read(5,*) goahead
         if (goahead .eq.'n'.or. goahead .eq. 'N') return
      endif

      write(ilog,'(/,a)') ' ** Removing redundant records ** '

 1000 format('UPDATE dbentry SET id=',i0,' WHERE id=',i0)
 1010 format('DELETE FROM dbentry WHERE id=',i0)
      jrec = 1
      do irec = 1, dbnrec
         if (.not.dupe(irec)) then
            if (jrec.ne.irec) then
               ! do the move irec -> jrec
               write(cmd,1000) jrec, irec
               call sqlite3_do(db, cmd)
            endif
            jrec = jrec + 1
         else
            ! delete the row
            write(cmd,1010) irec
            call sqlite3_do(db, cmd)
         endif
      enddo

      call sqlite3_commit( db )
      if (db%error.ne.SQLITE_OK) then
         routine = 'Dbdeduplicate (checkdb.F90)'
         write(message,'(2a)') 'sql error while removing duplicates: ', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif

      if (irec.ne.jrec) then
         write(ilog,*)'Duplicate records removed.'
         call dbgetnrec( db, dbnrec )
         write(ilog,'(a19,i0)')     'Number of records: ', dbnrec
      endif

      deallocate(dupe)
      deallocate(gx)
      deallocate(gx2)

      return
      end subroutine dbdeduplicate

!-----------------------------------------------------------------------
! DBDELREC
!
!     Go through database and remove record nignore
!-----------------------------------------------------------------------

      subroutine dbdelrec( db, nignore )

      implicit none

      type(SQLITE_DATABASE), intent(inout)      :: db
      integer(long), intent(in)                 :: nignore
      integer(long)                             :: irec,nrec
      logical(kind=4)                           :: busy=.true.
      character(len=c5)                         :: message
      character(len=c3)                         :: routine
      character(len=250)                        :: cmd

      call dbgetnrec( db, nrec )

      do while (busy)
         call sqlite3_begin(db, SQLITE_TRANS_IMMEDIATE)
         busy = db%error.eq.SQLITE_BUSY
      enddo
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while beginning &
            &transaction: ', db%errmsg
         call dberrmsg( routine, message )
      endif

      ! delete the row
      write(cmd,1020) nignore
      call sqlite3_do(db, cmd)

      do irec = nignore+1, dbnrec
         ! do the move irec -> irec-1
         write(cmd,1010) irec-1, irec
         call sqlite3_do(db, cmd)
      enddo

      call sqlite3_commit( db )
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while removing &
            &duplicates: ', db%errmsg
         call dberrmsg( routine, message )
      endif

      write(ilog,*)'Record removed: ',nignore
      call dbgetnrec( db, dbnrec )
      write(ilog,'(a19,i7)')     ' Number of records: ', dbnrec

 1010 format('UPDATE dbentry SET id=',i0,' WHERE id=',i0)
 1020 format('DELETE FROM dbentry WHERE id=',i0)

      return
      end subroutine dbdelrec

!-----------------------------------------------------------------------
! DBTRUNC
!
!     Truncate database to record nmaxrec
!-----------------------------------------------------------------------

      subroutine dbtrunc( db, nmaxrec )

      implicit none

      type(SQLITE_DATABASE), intent(inout)      :: db
      integer(long), intent(in)                 :: nmaxrec
      integer(long)                             :: irec,jrec
      logical(kind=4)                           :: busy=.true.
      character(len=c5)                         :: message
      character(len=c3)                         :: routine
      character(len=250)                        :: cmd

      real(dop) :: tt1,tt2

      call cpu_time(tt1)

      do while (busy)
         call sqlite3_begin(db, SQLITE_TRANS_IMMEDIATE)
         busy = db%error.eq.SQLITE_BUSY
      enddo
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while beginning &
            &transaction: ', db%errmsg
         call dberrmsg( routine, message )
      endif
      jrec = 1

    !  do irec = dbnrec,nmaxrec+1,-1
    !     print*,'Deleting record: ',irec
    !     ! delete the row
    !     write(cmd,1020) irec
    !     call sqlite3_do(db, cmd)
    !  enddo

      !Get current number of entries
      call dbrdentry(db, 1,dbnrec, dbinterp, timestamp)
      !Delete based on the status in dbentry
      !Set up command
      write(cmd,'(a45,i0)') 'DELETE FROM dbentry WHERE dbentry.id >',nmaxrec
      !Run command
      call sqlite3_do(db, cmd)
      !Commit the transaction
      call sqlite3_commit(db)

      if (dbin%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while removing &
            &duplicates: ', db%errmsg
         call dberrmsg(routine, message)
      endif
      call cpu_time(tt2)

      write(ilog,*)'DB truncated '
      call dbgetnrec( db, dbnrec )
      write(ilog,'(a19,i0)')     ' Number of records: ', dbnrec
      call cpu_time(tt2)
      write(6,'(/,a,f10.3,a)') 'Total time taken  ',tt2-tt1,'s'


 1020 format('DELETE FROM dbentry WHERE id=',i0)

      return
      end subroutine dbtrunc

!-----------------------------------------------------------------------
! DBCLEAN
!
!     Remove failed QC calculations (entries where interp is -1)
!
!-----------------------------------------------------------------------

      subroutine dbclean( db )

      implicit none

      type(SQLITE_DATABASE), intent(inout)      :: db
      integer(long)                             :: irec,jrec
      logical(kind=4)                           :: busy=.true.
      character(len=c5)                         :: message
      character(len=c3)                         :: routine
      character(len=250)                        :: cmd

      do while (busy)
         call sqlite3_begin(db, SQLITE_TRANS_IMMEDIATE)
         busy = db%error.eq.SQLITE_BUSY
      enddo
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while beginning &
            &transaction: ', db%errmsg
         call dberrmsg( routine, message )
      endif

      call dbrdentry(db, 1,dbnrec, dbinterp, timestamp)

      !Delete based on the status in dbentry
      !Set up command
      write(cmd,'(a45)') 'DELETE FROM dbentry WHERE dbentry.status =-1'
      !Run command
      call sqlite3_do(db, cmd)
      !Commit the transaction
      call sqlite3_commit(db)
      if (dbin%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while removing &
            &duplicates: ', db%errmsg
         call dberrmsg(routine, message)
      endif

      write(ilog,*) &
      'Records from failed calculations were removed.'
      write(ilog,*)'Number of initial records:', dbnrec
      call dbgetnrec(db, dbnrec)
      write(ilog,'(a29,i0)')     'Number of remaining records: ', dbnrec
      !Should now run a vacuum command
      !Currently doesn't work here
      write(ilog,*) "Please run VACUUM command to release free space in DB"
      write(ilog,*) "e.g sqlite3 database.sql VACUUM"
      ! call sqlite3_do(db,'VACUUM')
      ! call sqlite3_commit(db)
      ! if (dbin%error.ne.SQLITE_OK) then
      !    write(message,'(2a)') 'sql error while reducing file size: ', db%errmsg
      !    call dberrmsg(routine, message)
      ! endif

      return
      end subroutine dbclean

!-----------------------------------------------------------------------
! DBSTCLEAN
!
!     Strong clean: Remove all records from failed QC and 
!     interpolations (entries where interp .ne. 0)
!
!-----------------------------------------------------------------------

      subroutine dbstclean( db )

      implicit none

      type(SQLITE_DATABASE), intent(inout)      :: db
      integer(long)                             :: irec,jrec,dbnrec0,id0
      logical(kind=4)                           :: busy=.true.
      logical(kind=4)                           :: finished
      character(len=c5)                         :: message
      character(len=c3)                         :: routine
      character(len=250)                        :: cmd
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt

! Get initial number of records
      call dbgetnrec(db, dbnrec0)

! Delete records
      do while (busy)
         call sqlite3_begin(db, SQLITE_TRANS_IMMEDIATE)
         busy = db%error.eq.SQLITE_BUSY
      enddo
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while beginning &
            &transaction: ', db%errmsg
         call dberrmsg( routine, message )
      endif
      !Get current number of entries
      call dbrdentry(db, 1,dbnrec, dbinterp, timestamp)
      !Delete based on the status in dbentry
      !Set up command
      write(cmd,'(a45)') 'DELETE FROM dbentry WHERE dbentry.status !=0'
      !Run command
      call sqlite3_do(db, cmd)
      !Commit the transaction
      call sqlite3_commit(db)
      if (dbin%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while removing &
            &duplicates: ', db%errmsg
         call dberrmsg(routine, message)
      endif

! Get new number of records
      call dbgetnrec(db, dbnrec)
! Get id and correct for missing entries
      call sqlite3_query_table( db, 'dbentry', cols_p )
      call sqlite3_prepare_select(db, 'dbentry', cols_p, stmt)
      do irec = 1, dbnrec
         call sqlite3_next_row(stmt, cols_p, finished)
         call sqlite3_get_column(cols_p(1), id0)

         write(cmd,1010) irec,id0
         call sqlite3_do(db, cmd)

      enddo
      call sqlite3_finalize(stmt)

      write(ilog,*) &
      'Records from failed and interpolated calculations were removed.'
      write(ilog,*)'Number of initial records:', dbnrec0
      write(ilog,'(a29,i0)')     'Number of remaining records: ', dbnrec

      !Should now run a vacuum command
      !Currently doesn't work here
      write(ilog,*) "Please run VACUUM command to release free space in DB"
      write(ilog,*) "e.g sqlite3 database.sql VACUUM"
      !call sqlite3_do(db,'VACUUM')
      !call sqlite3_commit(db)
      if (dbin%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while reducing file size: ', db%errmsg
         call dberrmsg(routine, message)
      endif

 1010 format('UPDATE dbentry SET id=',i0,' WHERE id=',i0)

      return
      end subroutine dbstclean

!-----------------------------------------------------------------------
! DBCUT
!
! Cut DB by removing points with high energies
!     
!
!-----------------------------------------------------------------------

      subroutine dbpescut( db, ener )

      implicit none

      type(SQLITE_DATABASE), intent(inout)      :: db
      integer(long)                             :: sql_vers  !SQL database version
      real(dop), intent(in)                     :: ener ! max energy
      integer(long)                             :: irec,jrec
      logical(kind=4)                           :: busy=.true.
      character(len=c5)                         :: message
      character(len=c3)                         :: routine
      character(len=250)                        :: cmd
      character(len=9)                          :: colname

      sql_vers = db%db_vers

      do while (busy)
         call sqlite3_begin(db, SQLITE_TRANS_IMMEDIATE)
         busy = db%error.eq.SQLITE_BUSY
      enddo
      if (dbin%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while beginning &
            &transaction: ', db%errmsg
         call dberrmsg( routine, message )
      endif

      !Get current number of entries
      call dbrdentry(db, 1,dbnrec, dbinterp, timestamp)
      !Delete based on the value in apes
      !Set up command, is an embedded query
      !must look up name of column based on sql version
      if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         colname = "E_1 > "
      else
         colname = "state1 > "
      end if

      write(cmd,'(a116,a9,f7.3,a1)') 'delete from dbentry WHERE dbentry.id IN(SELECT dbentry.id &
      From dbentry join apes ON dbentry.id = apes.id WHERE apes.', colname, ener,')'
      write(ilog,*)cmd
      call sqlite3_do(db, cmd)
      !Commit the transaction
      call sqlite3_commit(db)
      if (dbin%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while removing &
            &duplicates: ', db%errmsg
         call dberrmsg(routine, message)
      endif

      write(ilog,*) &
      'Records with high energies were removed.'
      write(ilog,*)'Number of initial records:', dbnrec
      call dbgetnrec(db, dbnrec)
      write(ilog,'(a29,i0)')     'Number of remaining records: ', dbnrec
      !Should now run a vacuum command
      !Currently doesn't work here
      write(ilog,*) "Please run VACUUM command to release free space in DB"
      write(ilog,*) "e.g sqlite3 database.sql VACUUM"
      !call sqlite3_do(db,'VACUUM')
      !call sqlite3_commit(db)
      if (dbin%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while reducing file size: ', db%errmsg
         call dberrmsg(routine, message)
      endif
      

      return
      end subroutine dbpescut

!#######################################################################

      subroutine readdb

      implicit none

      integer   :: c1,c2
      real(dop) :: t1,t2,tt1,tt2

!     Check there are 1..nrec entries.  The DB is malformed if
!     this is not so.  Will also read what is supposed to be there.

      !Version does matter
      call dbrdrefdbgeom(dbin, natm, mref, atnam, xref)

      call cpu_time(tt1)
      call system_clock(c1)

!READ IN AND LOOK FOR ERRORS
      call cpu_time(t1)
      call dbrdentry(dbin, 1,dbnrec, dbinterp, timestamp)
      call cpu_time(t2)
      write(6,'(a,f10.3,a)') 'Read dbentry list             ',t2-t1,'s'
      call cpu_time(t1)
      call dbrdgeo(dbin, natm, 1,dbnrec, dbgeo)
      call cpu_time(t2)
      write(6,'(a,f10.3,a)') 'Read Geometries               ',t2-t1,'s'
      if (lpesdb) then
         call cpu_time(t1)
         call dbrdpes(dbin, nddstate, 1,dbnrec, dbener)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read Diabatic Energies        ',t2-t1,'s'
      endif
      if (lgradb) then
         call cpu_time(t1)
         call dbrdgra(dbin, nddstate, natm, 1,dbnrec, dbgrad)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read Diabatic Gradients       ',t2-t1,'s'
      endif
      if (lhesdb) then
         call cpu_time(t1)
         call dbrdhes(dbin, nddstate, natm, 1,dbnrec, dbhess)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read Diabatic Hessians        ',t2-t1,'s'
      endif
      if (lapesdb) then
         call cpu_time(t1)
         call dbrdapes(dbin, nddstate, 1,dbnrec, dbadener)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read Adiabatic Energies       ',t2-t1,'s'
      endif
      if (lagradb) then
         call cpu_time(t1)
         call dbrdagra(dbin, nddstate, natm, 1,dbnrec, dbadgrad)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read Adiabatic Gradients      ',t2-t1,'s'
      endif
      if (lahesdb) then
         call cpu_time(t1)
         call dbrdahes(dbin, nddstate, natm, 1,dbnrec, dbadhess)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read Adiabatic Hessians       ',t2-t1,'s'
      endif
      if (ldipdb) then
         call cpu_time(t1)
         call dbrddip(dbin, 1,dbnrec, dbdipmom)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read Diabatic Dipole Moments  ',t2-t1,'s'
      endif
      if (ladipdb) then
         call cpu_time(t1)
         call dbrdadip(dbin, 1,dbnrec, dbaddipmom)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read Adiabatic Dipole Moments ',t2-t1,'s'
      endif
      if (lmocodb) then
         call cpu_time(t1)
         call dbrdmoco(dbin, nbasis, 1,dbnrec, dbmoco)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read MO coefficients          ',t2-t1,'s'
      endif
      if (lnactdb) then
         call cpu_time(t1)
         call dbrdnact(dbin, natm, dercpdim, 1,dbnrec, dbdercp)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read NACTs                    ',t2-t1,'s'
      endif
      if (ltransdb) then
         call cpu_time(t1)
         call dbrdtrans(dbin, nddstate, 1,dbnrec, dbtrans)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read ADT matrices             ',t2-t1,'s'
      endif
      if (lnmdb) then
         call cpu_time(t1)
         call dbrdnm(dbin, ndofdd, 1,dbnrec, dbnm)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read Normal Mode Coordinates  ',t2-t1,'s'
      endif
      if (lcasocdb) then
         call cpu_time(t1)
         call dbrdcasoc( dbin, nddstate, 1,dbnrec, dbadsoc )
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read SOCs                     ',t2-t1,'s'
      endif
      if (lcpesdb) then
         call cpu_time(t1)
         call dbrdcpes(dbin, nddstate, 1,dbnrec, dbcener)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read Complex Diabatic Energies  ',t2-t1,'s'
      endif
      if (lctransdb) then
         call cpu_time(t1)
         call dbrdtrans(dbin, nddstate, 1,dbnrec, dbtrans)
         call cpu_time(t2)
         write(6,'(a,f10.3,a)') 'Read Complex ADT matrices       ',t2-t1,'s'
      endif

      call cpu_time(tt2)
      call system_clock(c2)
      write(6,'(/,a,2(f10.3,a))') 'Total time to read DB  ', &
                                real(c2-c1)/clockrate,'s (wall)', &
                                tt2-tt1,'s (cpu)'

      return
      end subroutine readdb

      end program checkdb

