!#######################################################################
! 
! COMPARE
!
! Program needed by the TEST script to compare the files generated
! by a new version of the MCTDH program with the comparison files 
! generated by a standard version.
!
! 4/98 GW
!
!
!#######################################################################

      program compare

use decimal, only:dop,long,sip
use global
use dglobal
use versions
use aglobal
use maxv
use dvrdatmod
use griddatmod
use psidef

use iofile, only: rdchkinfo,rdrhoinfo
use rdfilesmod, only: rdchk,rdgridpop,lrdchk
use logdat
use channels, only: filever

      implicit none

      real(dop), parameter        :: herrlim = 1.0e-12_dop, serrlim = 1.0e-6_dop
      real(dop), parameter        :: herrliman = 1.0e-6_dop, serrliman = 1.0e-3_dop
      integer(long)               :: idat,itst,icmp,isum,iaver,nst,i,&
                                     ilbl,ilblc,chkdvr,chkgrd,chkpsi,chkdat,jlbl

      real(dop)                   :: averr,maxerr,error,time,cpu1,cpu2,&
                                     sys1,sys2,&
                                     mem1,mem2,re,im,timec,rec,imc,re2,re2c,&
                                     im2,im2c,norm,normc,trace

      integer(long)                        :: zeig1,zeig2,zeig3,zeig4,zeig5,&
                                              zeig6,vvdim,vsdim
      integer(long), dimension(:,:), allocatable :: dimtst
      real(dop), dimension(:), allocatable :: vv1,vv2
      real(sip), dimension(:), allocatable :: vs1
      real(dop), dimension(:), allocatable :: gpop1,gpop2,gpop1c,gpop2c

      character(len=100)           :: cname,host1,host2,string
      character(len=c5)           :: filename
      logical(kind=4)             :: hpassed,spassed,lrun,lex,lerr,lgpel
      logical(kind=4), external   :: int2log

      idat=11
      itst=12
      icmp=13
      isum=14

      call mygetarg(1,name)
      if (name .eq. '-ver') then
         call wrversion(6)
         stop
      elseif (name .eq. '-h') then
         write(6,'(a)') ' Analyse program "compare". '
         write(6,'(a)') ' Purpose : Compares data files for elk_test.'
         write(6,'(a)') ' Usage   : compare [-h -ver] name cname'
         write(6,'(a,/a,/a)') &
       ' "name"  is the path to the name directory of the test.',&
       ' "cname" is the path to the name directory of the compare set.',&
       '---------------------------------------------------------------'
         stop
      endif
      call mygetarg(2,cname)
      
      ilbl=index(name,' ')-1
      ilblc=index(cname,' ')-1
      
      open(idat,file='results',position='append')
      open(isum,file='summary',position='append')
      hpassed=.true.
      spassed=.true.

      dname=name
      dlaenge=index(dname,' ')-1
!-----------------------------------------------------------------------
! read title of test from input file
!-----------------------------------------------------------------------
      open(itst,file=name(1:ilbl)//'.inp',form='formatted',&
           status='old')
      do i=1,3
         read(itst,'(a)') string
         write(idat,'(a)') string
!        write(isum,'(a)') string
      enddo
      close(itst)

!-----------------------------------------------------------------------
! first check if run finished
! then see if files to be checked are there, i.e. has test run at all
! Check type of calculation as some require different tests
! Also get density type
!-----------------------------------------------------------------------
      lrun=.false.
      dentype=0

      filename=name(1:ilbl)//'/log'

      open(itst,file=filename,status='old',form='formatted',err=5)
 
   11 continue
      read(itst,'(a)',end=13) string
      if (string(1:27) .eq. 'Program terminated normally') then
         lrun=.true.
      endif
      if (string(1:29) .eq. 'Density of type I propagation') then
         dentype=1
      endif
      if (string(1:28) .eq. 'Density of type I relaxation') then
         dentype=1
      endif
      if (string(1:30) .eq. 'Density of type II propagation') then
         dentype=2
      endif
      if (string(1:29) .eq. 'Density of type II relaxation') then
         dentype=2
      endif
      if (string(1:17) .eq. 'Real Wavefunction') then
         psitype=11
      endif
      if (string(4:33) .eq. 'Running Zagreb Surface Hopping') then
         psitype=3
      endif
      go to 11
 
   13 continue
      close(itst)
      if(.not.lrun)&
         write(idat,'(a)') 'Program did not terminate normally'

! No overlap for trajectories
      if (psitype .ne. 3) then
         filename=name(1:ilbl)//'/ovl_'//name(1:ilbl)
         inquire(file=filename,exist=lex)
         if (.not. lex) then
            write(idat,'(a)') 'No overlap file.'
            lrun=.false.
         endif
      endif

! No auto file for RealWP or trajectories
      if (psitype .ne. 11 .and. psitype .ne. 3) then
         filename=name(1:ilbl)//'/auto'
         inquire(file=filename,exist=lex)
         if (.not. lex) then
            write(idat,'(a)') 'No auto file.'
            lrun=.false.
         endif
      endif

      if (dentype .eq. 0) then
         filename=name(1:ilbl)//'/check'
         inquire(file=filename,exist=lex)
         if (.not. lex) then
            write(idat,'(a)') 'No check file.'
            lrun=.false.
         endif

         filename=name(1:ilbl)//'/gridpop'
         inquire(file=filename,exist=lex)
         if (.not. lex) then
            write(idat,'(a)') 'No gridpop file.'
            lrun=.false.
         endif
      endif

    5 continue
      if (.not. lrun) go to 500

!-----------------------------------------------------------------------
! compare memory, timing etc.
!-----------------------------------------------------------------------
      open(itst,file=name(1:ilbl)//'/log',status='old',&
           form='formatted')
      open(icmp,file=cname(1:ilblc)//'/log',status='old',&
           form='formatted',err=600)

   10 continue
      read(itst,'(a)',end=15) string
      if (string(1:9) .eq. 'Total cpu') &
             read(string,'(30x,f8.2)') cpu1
      if (string(1:9) .eq. 'Total sys') &
             read(string,'(30x,f8.2)') sys1
      if (string(8:11) .eq. 'Host') host1=string
      if (string(1:11) .eq. 'Memory size') then 
         jlbl=index(string,':')+3
         read(string(jlbl:jlbl+8),'(f8.3)') mem1
         mem1 = mem1+tiny
      endif
      go to 10
   15 continue
      read(icmp,'(a)',end=20) string
      if (string(1:9) .eq. 'Total cpu') &
             read(string,'(30x,f8.2)') cpu2
      if (string(1:9) .eq. 'Total sys') &
             read(string,'(30x,f8.2)') sys2
      if (string(8:11) .eq. 'Host') host2=string
      if (string(1:11) .eq. 'Memory size') then 
         jlbl=index(string,':')+3
         read(string(jlbl:jlbl+8),'(f8.3)') mem2
         mem2 = mem2+tiny
      endif
      go to 15
   20 continue
      
      write(idat,*) 
      write(idat,'(a)') '*** TIMING / MEMORY INFORMATION ***'
      write(idat,*)
      write(idat,'(a)') 'For '//name(1:ilbl)
      write(idat,'(a)') host1
      write(idat,'(2(a,f12.1))') 'Cputime = ',cpu1,&
            '  System time = ',sys1
      write(idat,'(a,f8.2)') 'Memory required = ',mem1
      write(idat,*)
      write(idat,'(a)') 'For '//cname(1:ilblc)
      write(idat,'(a)') host2
      write(idat,'(2(a,f12.1))') 'Cputime = ',cpu2,&
            '  System time = ',sys2
      write(idat,'(a,f8.2)') 'Memory required = ',mem2
      write(idat,*)
      write(idat,'(a,f12.6)') &
             'CPU ratio standard/test    = ',cpu2/cpu1
      write(idat,'(a,f12.6)') &
             'System time ratio standard/test    = ',sys2/sys1
      write(idat,'(a,f12.6)') &
             'Memory ratio standard/test = ',mem2/mem1

!-----------------------------------------------------------------------
! compare overlap
!-----------------------------------------------------------------------
      if (psitype .eq. 3) go to 120

      filename=name(1:ilbl)//'/ovl_'//name(1:ilbl)
      open(itst,file=filename,status='old',form='formatted')
      call cmpover(itst,averr,maxerr,i)

      if (i .eq. 0) then
         lrun=.false.
         averr=999999.0_dop
         maxerr=99999.0_dop
      else
         averr=averr/dble(i)
      endif
      if (maxerr .gt. herrlim*1.0d4) hpassed=.false.
      if (maxerr .gt. serrlim*1.0d3) spassed=.false.
      write(idat,*)
      write(idat,'(a)') '**** OVERLAP CHECK ****'
      write(idat,'(a,i5)') 'Number of data  : ',i
      write(idat,'(2(a,e20.8))') &
         'Av. error = ',averr,'     Max. error = ',maxerr
      close(itst)

  120 continue

!-----------------------------------------------------------------------
! compare auto files
! no auto file created for realwp calcualtion - psitype 11
!-----------------------------------------------------------------------
      if (psitype .eq. 11 .or. psitype .eq. 3) go to 170

      open(itst,file=name(1:ilbl)//'/auto',status='old',&
           form='formatted',err=160)
      open(icmp,file=cname(1:ilblc)//'/auto',status='old',&
           form='formatted')
      read(itst,*)
      read(icmp,*)

      i=0
      averr=0.0_dop
      maxerr=0.0_dop
  150 continue
      read(itst,*,end=160,err=160) time,re,im
      read(icmp,*,end=160,err=160) timec,rec,imc

      i=i+1
      error=abs(time-timec) 
      maxerr=max(error,maxerr)
      averr=averr+error
      error=abs(re-rec) 
      maxerr=max(error,maxerr)
      averr=averr+error
      error=abs(im-imc)
      maxerr=max(error,maxerr)
      averr=averr+error

      go to 150

  160 continue
      if (i .eq. 0) then
         lrun=.false.
         averr=999999.0_dop
         maxerr=99999.0_dop
      else
         averr=averr/dble(3*i)
      endif
      if (maxerr .gt. 10.0_dop*herrlim) hpassed=.false.
      if (maxerr .gt. serrlim) spassed=.false.
      write(idat,*)
      write(idat,'(a)') '**** AUTO CHECK ****'
      write(idat,'(a,i5)') 'Number of data  : ',i
      write(idat,'(2(a,e20.8))')&
         'Av. error = ',averr,'     Max. error = ',maxerr
      close(itst)
      close(icmp)

  170 continue

!-----------------------------------------------------------------------
! compare expectation files, two operators are required
!-----------------------------------------------------------------------
      open(itst,file=name(1:ilbl)//'/expectation',status='old',&
           form='formatted',err=299)
      open(icmp,file=cname(1:ilblc)//'/expectation',status='old',&
           form='formatted')
 230  read(itst,'(a20)') string(1:20)
      if(string(1:1) .eq. '#' .or. string(1:20) &
                     .eq. '                    ') goto 230 
 240  read(icmp,'(a20)') string(1:20)
      if(string(1:1) .eq. '#' .or. string(1:20) &
                     .eq. '                    ') goto 240 

      backspace itst
      backspace icmp
      i=0
      averr=0.0_dop
      maxerr=0.0_dop
 250  continue
      read(itst,*,end=260,err=260) time,norm,re,im,re2,im2
      read(icmp,*,end=260,err=260) timec,normc,rec,imc,re2c,im2c
      i=i+1
      error=abs(time-timec) 
      maxerr=max(error,maxerr)
      averr=averr+error
      error=abs(norm-normc)
      maxerr=max(error,maxerr)
      averr=averr+error
      error=abs(re-rec) 
      maxerr=max(error,maxerr)
      averr=averr+error
      error=abs(re2-re2c)
      maxerr=max(error,maxerr)
      averr=averr+error
      error=abs(im-imc)
      maxerr=max(error,maxerr)
      averr=averr+error
      error=abs(im2-im2c)
      maxerr=max(error,maxerr)
      averr=averr+error
      
      go to 250

 260  continue
      if (i .eq. 0) then
         lrun=.false.
         averr=999999.0_dop
         maxerr=99999.0_dop
      else
         averr=averr/dble(3*i)
      endif
      if (maxerr .gt. 10.0_dop*herrlim) hpassed=.false.
      if (maxerr .gt. serrlim) spassed=.false.
      write(idat,*)
      write(idat,'(a)') '**** EXPECTATION CHECK ****'
      write(idat,'(a,i5)') 'Number of data  : ',i
      write(idat,'(2(a,e20.8))')&
         'Av. error = ',averr,'     Max. error = ',maxerr

      close(itst)
      close(icmp)

 299  continue

!-----------------------------------------------------------------------
! compare check files
!-----------------------------------------------------------------------
      open(itst,file=name(1:ilbl)//'/check',status='old',&
           form='unformatted')

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(itst)

      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef
      if (ldynspf) call alloc_dynwf
      allocate (dimtst(maxdim,maxsta))

      rewind(itst)
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      
!        --- read the headers of check files          
      read(itst) filever(itst)
      call rdchkinfo(itst,chkdvr,chkgrd,chkpsi,chkdat)
      dimtst=dim  ! store dim
      filename=cname(1:ilblc)//'/check'
      open(icmp,file=filename,status='old',form='unformatted')
      read(icmp) filever(icmp)
      call rdchkinfo(icmp,chkdvr,chkgrd,chkpsi,chkdat)
      call chksyserr('error_in_elk_test:compare',lerr)
      
      iaver=0
      averr=0.0_dop
      maxerr=0.0_dop
      
      if (filever(itst).le.11.0000_dop) then
         routine='Compare'
         message='MCTDH version used in elk_test too old '//&
                 'for Compare program. Must be at least 11.0'
         call warnmsg
         go to 500
      endif
      if (filever(icmp).le.10.1004_dop) then
         routine='Compare'
         message='MCTDH version used to generate test set too old '//&
                 'for Compare program. Must be at least 10.1'
         call warnmsg
         go to 500
      endif
      if (filever(icmp).le.11.0000_dop .and. lmulpack) then
         routine='Compare'
         message='Cannot compare CHECK for multi-packet calculations with a '//&
                 'version less than 11.0'
         call warnmsg
         go to 200
      endif

!-----------------------------------------------------------------------
! TSH calculations can end up with different values of dim(1,1) (no. of 
! trajectories) if some are rejected in only one. Check for this.
!-----------------------------------------------------------------------
      if (psitype .eq. 3) then
         if (dim(1,1) .ne. dimtst(1,1)) then
            write(idat,*)
            write(idat,'(a)') '**** INITIAL CHECK FAILED ****'
            write(idat,'(a)') 'Different no. of trajectories on tests'
            hpassed = .false.
            spassed = .false.

            go to 500
         endif
      endif
! Allocate memory
! ecorr
      zeig1=1
! etot
      zeig2=zeig1+nstate
! spop
      zeig3=zeig2+nddstate
! dicht3
      zeig4=zeig3+nddstate
! ggp
      zeig5=zeig4+d3matdim
! qdq
      zeig6=zeig5+gwpinfodim
! end
      vvdim=zeig6+4*ndof*nstate
! norm and trace not compared as only used for density matrix calcs

      allocate(vv1(vvdim))
      allocate(vv2(vvdim))
      lrdchk(:)=.true.

      if (filever(itst) .ge. 11.0_dop) rewind(itst)
      if (filever(icmp) .ge. 11.0_dop) rewind(icmp)
      do 
         vv1(:)=0.0_dop
         vv2(:)=0.0_dop
         call rdchk(itst,vv1(zeig1),vv1(zeig2),vv1(zeig3),vv1(zeig4),&
              vv1(zeig5),vv1(zeig6),norm,trace)
         if (lend) exit
         call rdchk(icmp,vv2(zeig1),vv2(zeig2),vv2(zeig3),vv2(zeig4),&
              vv2(zeig5),vv2(zeig6),norm,trace)
         if (lend) exit

!.....Find maximum and build average
         do i = 1, vvdim
            error=abs(vv1(i)-vv2(i))
       !     print*,i,vv1(i),vv2(i),error
            averr=averr+error
            maxerr=max(error,maxerr)
         enddo
         iaver = iaver + vvdim
      enddo
      
      averr=averr/dble(iaver)

      if (maxerr .gt. herrlim) hpassed=.false.
      if (maxerr .gt. serrlim) spassed=.false.
      write(idat,*)
      write(idat,'(a)') '**** CHECK CHECK ****'
      write(idat,'(a,i5)') 'Number of data  : ',iaver
      write(idat,'(2(a,e20.8))')&
           'Av. error = ',averr,'     Max. error = ',maxerr
      close(itst)
      close(icmp)

      deallocate(vv1)
      deallocate(vv2)

!-----------------------------------------------------------------------
! compare gridpop files
!-----------------------------------------------------------------------
  200 continue
      chkdvr=1
      chkgrd=1
      chkdat=0
!     --- read the headers of gridpop files          
      open(itst,file=name(1:ilbl)//'/gridpop',status='old',&
           form='unformatted')
      read(itst) filever(itst), i
      lgpel = int2log(i)
      call rdrhoinfo(itst,chkdvr,chkgrd,chkdat)
      open(icmp,file=cname(1:ilblc)//'/gridpop',status='old',&
           form='unformatted')
      read(icmp) filever(icmp), i
      call rdrhoinfo(icmp,chkdvr,chkgrd,chkdat)
      
      iaver=0
      averr=0.0_dop
      maxerr=0.0_dop
      nst=nstate
      if(.not.lgpel) nst=1

! Allocate memory, vs1 is sp workspace
      vsdim=ortdim*nddstate*npacktot

      allocate(vs1(2*maxgdim))
      allocate(gpop1(vsdim))
      allocate(gpop2(vsdim))
      allocate(gpop1c(vsdim))
      allocate(gpop2c(vsdim))

      if (filever(itst) .ge. 11.0_dop) rewind(itst)
      if (filever(icmp) .ge. 11.0_dop) rewind(icmp)
      do 
         call rdgridpop(itst,vs1,gpop1,gpop2)
         if (lend) exit
         call rdgridpop(icmp,vs1,gpop1c,gpop2c)
         if (lend) exit

!.....Find maximum and build average
         do i = 1, vsdim
            error=abs(gpop1(i)-gpop1c(i))
            averr=averr+error
            maxerr=max(error,maxerr)

            error=abs(gpop2(i)-gpop2c(i))
            averr=averr+error
            maxerr=max(error,maxerr)
         enddo
         iaver = iaver + 2*vsdim
      enddo

      averr=averr/dble(iaver)

      if (maxerr .gt. herrlim) hpassed=.false.
      if (maxerr .gt. serrlim) spassed=.false.
      write(idat,*)
      write(idat,'(a)') '**** GRIDPOP CHECK ****'
      write(idat,'(a,i5)') 'Number of data  : ',iaver
      write(idat,'(2(a,e20.8))')&
           'Av. error = ',averr,'     Max. error = ',maxerr
      close(itst)
      close(icmp)

      deallocate(vs1)
      deallocate(gpop1,gpop2,gpop1c,gpop2c)

!-----------------------------------------------------------------------
! ADPOP
!-----------------------------------------------------------------------
      filename=name(1:ilbl)//'/adp'
      inquire(file=filename,exist=lex)
      if (lex) then
         open(itst,file=filename,status='old',form='formatted',err=5)
         filename=cname(1:ilblc)//'/adp'
         open(icmp,file=filename,status='old',form='formatted')

         call companal(itst,icmp,5,maxerr,averr,iaver)

         if (maxerr .gt. herrliman) hpassed=.false.
         if (maxerr .gt. serrliman) spassed=.false.
         write(idat,*)
         write(idat,'(a)') '**** ADPOP CHECK ****'
         write(idat,'(a,i5)') 'Number of data  : ',iaver
         write(idat,'(2(a,e20.8))')&
              'Av. error = ',averr,'     Max. error = ',maxerr
         close(itst)
         close(icmp)
      endif
 
!-----------------------------------------------------------------------
! AUTOSPEC
!-----------------------------------------------------------------------
      filename=name(1:ilbl)//'/spectrum.pl'
      inquire(file=filename,exist=lex)
      if (lex) then
         open(itst,file=filename,status='old',form='formatted',err=5)
         filename=cname(1:ilblc)//'/spectrum.pl'
         open(icmp,file=filename,status='old',form='formatted')

         call companal(itst,icmp,4,maxerr,averr,iaver)

         if (maxerr .gt. herrliman) hpassed=.false.
         if (maxerr .gt. serrliman) spassed=.false.
         write(idat,*)
         write(idat,'(a)') '**** AUTOSPEC CHECK ****'
         write(idat,'(a,i5)') 'Number of data  : ',iaver
         write(idat,'(2(a,e20.8))')&
              'Av. error = ',averr,'     Max. error = ',maxerr
         close(itst)
         close(icmp)
      endif
 
!-----------------------------------------------------------------------
! SHOWD1D
!-----------------------------------------------------------------------
      filename=name(1:ilbl)//'/den1d_f1'
      inquire(file=filename,exist=lex)
      if (lex) then
         open(itst,file=filename,status='old',form='formatted',err=5)
         filename=cname(1:ilblc)//'/den1d_f1'
         open(icmp,file=filename,status='old',form='formatted')

         call companal(itst,icmp,3,maxerr,averr,iaver)

         if (maxerr .gt. herrliman) hpassed=.false.
         if (maxerr .gt. serrliman) spassed=.false.
         write(idat,*)
         write(idat,'(a)') '**** SHOWD1D CHECK ****'
         write(idat,'(a,i5)') 'Number of data  : ',iaver
         write(idat,'(2(a,e20.8))')&
              'Av. error = ',averr,'     Max. error = ',maxerr
         close(itst)
         close(icmp)
      endif
 
!-----------------------------------------------------------------------
! STATEPOP
!-----------------------------------------------------------------------
      filename=name(1:ilbl)//'/spops'
      inquire(file=filename,exist=lex)
      if (lex) then
         open(itst,file=filename,status='old',form='formatted',err=5)
         filename=cname(1:ilblc)//'/spops'
         open(icmp,file=filename,status='old',form='formatted')

         call companal(itst,icmp,3,maxerr,averr,iaver)

         if (maxerr .gt. herrliman) hpassed=.false.
         if (maxerr .gt. serrliman) spassed=.false.
         write(idat,*)
         write(idat,'(a)') '**** STATEPOP CHECK ****'
         write(idat,'(a,i5)') 'Number of data  : ',iaver
         write(idat,'(2(a,e20.8))')&
              'Av. error = ',averr,'     Max. error = ',maxerr
         close(itst)
         close(icmp)
      endif
 
!-----------------------------------------------------------------------
! RDGPOP
!-----------------------------------------------------------------------
      filename=name(1:ilbl)//'/gpop.pl'
      inquire(file=filename,exist=lex)
      if (lex) then
         open(itst,file=filename,status='old',form='formatted',err=5)
         filename=cname(1:ilblc)//'/gpop.pl'
         open(icmp,file=filename,status='old',form='formatted')

         call companal(itst,icmp,5,maxerr,averr,iaver)

         if (maxerr .gt. herrliman) hpassed=.false.
         if (maxerr .gt. serrliman) spassed=.false.
         write(idat,*)
         write(idat,'(a)') '**** RDGPOP CHECK ****'
         write(idat,'(a,i5)') 'Number of data  : ',iaver
         write(idat,'(2(a,e20.8))')&
              'Av. error = ',averr,'     Max. error = ',maxerr
         close(itst)
         close(icmp)
      endif
 
!-----------------------------------------------------------------------
! FLUX
!-----------------------------------------------------------------------
      filename=name(1:ilbl)//'/flux'
      inquire(file=filename,exist=lex)
      if (lex) then
         open(itst,file=filename,status='old',form='formatted',err=5)
         filename=cname(1:ilblc)//'/flux'
         open(icmp,file=filename,status='old',form='formatted')

         call companal(itst,icmp,4,maxerr,averr,iaver)

         if (maxerr .gt. herrliman) hpassed=.false.
         if (maxerr .gt. serrliman) spassed=.false.
         write(idat,*)
         write(idat,'(a)') '**** FLUX CHECK ****'
         write(idat,'(a,i5)') 'Number of data  : ',iaver
         write(idat,'(2(a,e20.8))')&
              'Av. error = ',averr,'     Max. error = ',maxerr
         close(itst)
         close(icmp)
      endif
 
!-----------------------------------------------------------------------
! write status of checks
!-----------------------------------------------------------------------
  500 continue
      if (.not. lrun) then
         write(idat,'(/a)') '***  TEST RUN FAILED ***'
      else 
         if (hpassed) then
            write(idat,'(/a)') '***  Passed hard limit? : YES ***'
         else 
            write(idat,'(/a)') '***  Passed hard limit? : NO  ***'
         endif
         if (spassed) then
            write(idat,'(/a)') '***  Passed soft limit? : YES ***'
         else 
            write(idat,'(/a)') '***  Passed soft limit? : NO  ***'
         endif
      endif

      write(idat,*)
      write(idat,'(80a1)') ('#',i=1,80)

      if (hpassed .and. lrun) then
         write(isum,'(2a,f12.4,a,f12.4)') &
              name(1:ilbl), ' : Passed hard limit.   Time ratio: ',&
                         sys2/sys1,'  Memory ratio: ',mem2/mem1
      else if (spassed .and. lrun) then
         write(isum,'(2a,f12.4,a,f12.4)') &
              name(1:ilbl),' : Passed *SOFT* limit. Time ratio: ',&
                         sys2/sys1,'  Memory ratio: ',mem2/mem1
      else
         write(isum,'(2a,f12.4,a,f12.4)') &
              name(1:ilbl), ' : FAILED.              Time ratio: ',&
                         sys2/sys1,'  Memory ratio: ',mem2/mem1
      endif

      close(idat)
      close(isum)

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_psidef
      call dealloc_dynwf
      if (allocated(dimtst)) deallocate (dimtst)

      stop
  600 print*,'ERROR: No comparison data found. Has the elk_test_gen been run?'
      print*
      stop

      contains

!#######################################################################
! CMPOVER
! reads an existing overlap file (old format and new format)
!#######################################################################

      subroutine cmpover(unit,averr,maxerr,icount)

      implicit none

      integer(long), intent(in)  :: unit
      integer(long), intent(out) :: icount
      logical(kind=4)            :: newr
      real(dop)                  :: time,error,nsq,n1sq,reovlp,imovlp
      real(dop), intent(out)     :: averr,maxerr
      character(len=4)           :: string

      newr=.false.
      icount=0
      averr=0.0_dop
      maxerr=0.0_dop
      read(unit,'(a)',end=999,err=999) string
      if( string(1:3) .eq. '#-#') newr=.true.

  9   read(unit,'(a)',end=999,err=999) string
      if( string(1:1) .eq. '#') goto 9
      backspace unit
      if(newr) goto 20

   10 continue
      read(unit,*,end=999) time,error
      icount=icount+1
      averr=averr+abs(error)
      maxerr=max(abs(error),maxerr)
      go to 10

   20 continue
      read(unit,*,end=999) time,reovlp,imovlp,nsq,n1sq
      icount=icount+1
      error=sqrt(abs(nsq+n1sq-2.0_dop*reovlp))
      if(error .lt. 3.0e-7_dop) error=0.0_dop
      averr=averr+error
      maxerr=max(error,maxerr)
      go to 20

  999 continue

      return
      end subroutine cmpover

!#######################################################################
! Compare output from analysis programs
!
! IN
! itst: unit with test data
! icmp: unit with comparison data
! nnum: no. of data per line to be read
!
! OUT:
! maxerr: max difference between data abs(itst-icmp)
! averr:  Av. error
! ndata:  no. data used in average
!
!#######################################################################

      subroutine companal(itst,icmp,nnum,maxerr,averr,ndata)

      implicit none

      integer(long), intent(in) :: itst,icmp,nnum
      integer(long), intent(out):: ndata
      integer(long)             :: i,n,n1
      real(dop), intent(out)    :: maxerr,averr
      real(dop), dimension(:,:), allocatable :: tnum,cnum

!-----------------------------------------------------------------------
! find no. of sets of data
!-----------------------------------------------------------------------
      rewind(itst)
      ndata = 0
      do
        read(itst,'(a)',end=20) string
        if (string(1:1) .eq. '#') cycle
        if (string(1:20) .eq. '                    ') cycle
        if (string(1:3) .eq. 'set') cycle
        if (string(1:4) .eq. 'plot') cycle
        if (string(1:1) .eq. 'e') cycle
        if (string(1:10) .eq. '----------') cycle
        ndata = ndata+1
      enddo
   20 continue

      allocate(tnum(nnum,ndata))
      allocate(cnum(nnum,ndata))

!-----------------------------------------------------------------------
! read test data
!-----------------------------------------------------------------------
      rewind(itst)
      n=0
      do
        read(itst,'(a)',end=40) string
        if (string(1:1) .eq. '#') cycle
        if (string(1:20) .eq. '                    ') cycle
        if (string(1:3) .eq. 'set') cycle
        if (string(1:4) .eq. 'plot') cycle
        if (string(1:1) .eq. 'e') cycle
        if (string(1:10) .eq. '----------') cycle
        n = n+1
        backspace(itst)
        read(itst,*) (tnum(i,n),i=1,nnum)
      enddo
   40 continue

!-----------------------------------------------------------------------
! read comparison data
!-----------------------------------------------------------------------
      rewind(icmp)
      n=0
      do
        read(icmp,'(a)',end=60) string
        if (string(1:1) .eq. '#') cycle
        if (string(1:20) .eq. '                    ') cycle
        if (string(1:3) .eq. 'set') cycle
        if (string(1:4) .eq. 'plot') cycle
        if (string(1:1) .eq. 'e') cycle
        if (string(1:10) .eq. '----------') cycle
        n = n+1
        backspace(icmp)
        read(icmp,*) (cnum(i,n),i=1,nnum)
      enddo
   60 continue

!-----------------------------------------------------------------------
! calculate average and maximum error
!-----------------------------------------------------------------------
      averr=0.0_dop
      maxerr=0.0_dop
      do n=1,ndata
         do n1=1,nnum
            error=dabs(tnum(n1,n) - cnum(n1,n))
            averr=averr+error
            maxerr=max(error,maxerr)
         enddo
      enddo
! change ndata from no. lines to no. data in average
      ndata=ndata*nnum
      averr=averr/dble(ndata)

      deallocate(tnum,cnum)

      return
      end subroutine companal

      end program compare


