!***********************************************************************c
!                                                                       c
!                         AUSWERT concat program                        c
!                                                                       c
! Concatenation of two MCTDH wave functions, namely                     c
! |Psi>=|Psi_1>|Psi_2>, where |Psi_i> is an MCTDH-type WF               c
!                                                                       c
!     05/2016 QM  and HDM                                               c
!                                                                       c
!***********************************************************************c

      program concat

use decimal, only: long,dop,sip
use maxv
use global
use channels
use dvrdatmod
use griddatmod
use psidef
use datenmod
use compdatmod
use basismod
use versions
use logdat

use iodvrdef
use iogrddef
use iopsidef
use einsbasmod
use iorst
use compdat

      implicit none

      integer(long) :: maxpsidim,tmaxdim,tmaxsta
      integer(long) :: chkdvr,chkgrd,chkpsi,chkprp,iflag,idum,laein,ilbl
      integer(long) :: iarg, myiargc,izpsi,jzpsi,kzpsi,npsi,ierr
      integer(long) :: f,n,m,i,ii,j,jj,jjs,s
      integer(long) :: ctotpdim(2),idum1(1),idum2(1)
      real(dop)     :: t,rdum(1)
      character(len=80) :: buf
      character(len=15) :: stat
      character(len=c5) :: filepsi(2),filename
      logical           :: ldum,lerr,ldir,cleb(2)
      logical           :: lrundvr

      integer(long), dimension(:,:,:), allocatable   :: cipbp
      real(dop), dimension(:,:,:), allocatable       :: crpbp
      character(len=c1), dimension(:,:), allocatable :: cmodelab
      complex(dop), dimension(:), allocatable        :: psi1,psi2,psi
      complex(dop), dimension(1)                     :: cdum,cdum1

!-----------------------------------------------------------------------
! Initialize variables and set defaults
!-----------------------------------------------------------------------
      call default
      call adefault
      ncomp=2
      stat = 'new'
      ldir=.false.
      lrundvr = .false.
      name = './'
      do i=1,ncomp
        filepsi(i) = ' '
      enddo

!-----------------------------------------------------------------------
! read options
!-----------------------------------------------------------------------
      iarg = myiargc()
      if (iarg .le. 0) goto 200
      call mygetarg(0,options)
      optlaenge  = index(options,' ')
      do i = 1, iarg
         call mygetarg(i,buf)
         ilbl  = index(buf,' ')
         ilbl = min(ilbl,100-optlaenge)
         if(ilbl.gt.0) options = options(1:optlaenge)//buf(1:ilbl)
         optlaenge = optlaenge + ilbl
      end do

      i = 0
      buf=' '
 100  i=i+1
      if(iarg .ge. 1) call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if(buf .eq. '-D') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
            ldir=.true.
         else if( buf .eq. '-dvr' ) then
            lrundvr=.true.
         else if( buf .eq. '-ver' ) then
            call wrversion(6)
            stop
         else if( buf(1:2) .eq. '-w' ) then
            lovwr=.true.
            stat='unknown'
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,2a,/1x,a)') &
      'Purpose: Build the (Hartree) product of two MCTDH', &
               ' wavefunctions.', &
      'Usage : concat [-ver -D -dvr -w -h -?] psi1 psi2'
            write(6,'(/,a)') 'Options :'
            write(6,'(a)') &
                     ' -ver    : Version information about the program'
            write(6,'(a,/10x,a,/10x,a)') &
                   ' -D DIR  : Data are stored in directory DIR,', &
                   ' rather than in the current directory.', &
                   ' The directory DIR will be made, if not existing.'
            write(6,'(2a)') &
                     ' -dvr    : The dvr-file for the new WF', &
                     ' will be build.'
            write(6,'(a)') &
                     ' -w      : Existing files are overwritten.'
            write(6,'(a,/a)') &
                     ' -h      : Print this help text.', &
                     ' -?      : Print this help text.'
            write(6,'(2a/2a)') ' The arguments psi1, psi2 are the', &
                               ' names of the restart-files to be', &
                               ' to be concatenated,', &
                               ' i.e. psi = psi1*psi2 .'
            write(6,'(2a,/2a,/a,/a)') ' If no argument is given,', &
               ' the user is prompted for the missing argument.', &
               ' The first WF may have electronic states', &
               '  (in single- or multi-set),', &
               ' but the second WF must not have electronic states.', &
               ' Packets are not allowed.'
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
 200  continue

!-----------------------------------------------------------------------
! Check for existence of output-file and open log-file
!-----------------------------------------------------------------------
      laenge=index(name,' ')-1
      call abspath(name,laenge)

      if(.not.lovwr) then
         filename=name(1:laenge)//'/restart'
         inquire(FILE=filename, EXIST=ldum)
         if(ldum) then
            routine='concat'
            message='The output-file '//filename(1:laenge+8)// &
            ' already exist. Use option -w if you want to overwrite.'
            call errormsg
         endif
      endif

      if(ldir) then
         open (ilog,file=name(1:laenge)//'/scratch',err=3000)
         close(ilog,status='delete')
         goto 30
 3000    close(ilog,status='delete')
         write(6,'(2a,/a)') 'Name-directory: ',name(1:laenge), &
                            'does not exists. I will make it!'
         inptit='mkdir '//name(1:laenge)
         call excmd2(ierr, inptit)
         if (ierr.ne.0) then
            write(6,'(2a)') '### ERROR: ', &
            'Could not make the name-directory. ###'
            stop
         endif
      endif
 30   filename=name(1:laenge)//'/cc.log'
      open (ilog,file=filename,form='formatted',status=stat,err=2000)
      logisopen=.true.
      call fpstartup(filename)
      write(ilog,'(a80)') filename
      call wrversion(ilog)

!-----------------------------------------------------------------------
! Read arguments
!-----------------------------------------------------------------------
      npsi=0
 2    continue
      if(iarg .lt. i) goto 300
      npsi=npsi+1
      call mygetarg(i,filepsi(npsi))
      i=i+1
      call mygetarg(i,buf)
      goto 2
 300  continue
      if (npsi .gt. ncomp) then
        write(6,'(a,i5)') &
         'Only two wavefunctions are allowed in the concat program:'
        stop
      endif
!.....In case, no arguments were provided.
      if(npsi .eq. 0) then
        do n=1,ncomp
          write(6,'(a,i3,a)') ' Filename containing wave function',n,' ?'
          read(5,'(a)') filepsi(n)
        enddo
      endif

!.....Write options to log-file.
      if( optlaenge .le. 60 ) then
         write(ilog,'(2a/)')   ' Options : ', options(1:optlaenge)
      else
         write(ilog,'(a,/a/)') ' Options : ', options(1:optlaenge)
      end if

!.....Make file-name absolute.
      do n=1,ncomp
        laein=index(filepsi(n),' ')-1
        call abspath(filepsi(n),laein)
!.......Test if "restart" should be appened.
        if(filepsi(n)(laein-7:laein) .ne. '/restart'.and. &
           filepsi(n)(laein-6:laein-3) .ne. '/rst') then
           open(irst,file=filepsi(n),form='unformatted',status='old',err=52)
           close(irst)
           goto 51
  52       filepsi(n)=filepsi(n)(1:laein)//'/restart'
           laein=laein+8
           close(irst,status='delete')
  51       continue
        endif
        write(ilog,'(a,i1,a)') ' Reading ',n,'. restart-file from:'
        write(ilog,'(1x,a/)') filepsi(n)(1:laein)
      enddo
      write(ilog,'(a,i1,a)') ' Concatenated WF is written to:'
      write(ilog,'(1x,a/)') name(1:laenge)//'/restart'
      write(ilog,'(78a1)') ('-',i=1,78)

!-----------------------------------------------------------------------
! get memory sizes and allocate memory
!-----------------------------------------------------------------------
      tmaxdim = 0
      tmaxsta = 0
      do n=1,ncomp
         open(irst,file=filepsi(n),form='unformatted',status='old')
         call rdmemdim(irst)
         tmaxdim = tmaxdim+maxdim
         tmaxsta = max(maxsta,tmaxsta)
         close(irst)
      enddo
      maxdim = tmaxdim
      maxsta = tmaxsta
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef
      call alloc_compdat

      allocate(cipbp(mbaspar,maxdim,2))
      allocate(crpbp(mbaspar,maxdim,2))
      allocate(cmodelab(maxdim,2))

print*,'RUNNING 1'
!-----------------------------------------------------------------------
! read the restart-info from input-files and store the data
!-----------------------------------------------------------------------
      chkdvr = 1           ! dvrinfo is read and stored
      chkgrd = 1           ! grdinfo is read and stored
      chkpsi = 1           ! psiinfo is read and stored
      chkprp = 0           ! propinfo is read but not stored
      iflag  = 0           ! read wavefunction only

      do npsi=1,ncomp
        filename=filepsi(npsi)
print*,'READING',npsi,filename
        open(irst,file=filename,form='unformatted',&
                  status='old',err=1000)
        write(ilog,'(i3,a)') npsi,'. input WF'
        call rstinfo(ldum,lerr,chkdvr,chkgrd,chkpsi,chkprp)
print*,'DONE'
        if(psitype .ne. 0) then
           routine='concat'
           message='Only standard MCTDH WFs are allowed!'
           write(6,*) 'psitype =',psitype
           call errormsg
        endif
        if(npacket.gt.1 .or. npackts.gt.1) then
           routine='concat'
           message='Packets are not allowed!'
           write(6,*) 'npsi,npacket, npackts',npsi,npacket,npackts
           call errormsg
        endif

print*,'DONE 1'
        call psidat
        write(ilog,'(2a)')' ndof nmode nstate leb     ', &
                          ' adim  totphidim     dgldim'
        write(ilog,'(3i5,l6,3i11)') ndof,nmode,nstate,leb, &
                                    adim,totphidim,dgldim
        write(ilog,'(78a1)') ('-',i=1,78)

print*,'DONE 2'
        call savecsys(npsi)
        call savecpsi(npsi)
print*,'DONE 3'
!.......Some variables are still not saved, so save them by hand.
        ctotpdim(npsi)=totphidim
        cleb(npsi)=leb
        do f=1,ndof
           cmodelab(f,npsi)=modelabel(f)
           do i=1,mbaspar
              cipbp(i,f,npsi)=ipbaspar(i,f)
              crpbp(i,f,npsi)=rpbaspar(i,f)
           enddo
        enddo
print*,'DONE 4'

        close(irst)
      enddo  ! of do npsi=1,ncomp

print*,'RUNNING 2'
! Check that modelables are all different.
      ii=0
 66   do f=1,cndof(1)
         do i=1,cndof(2)
            if(cmodelab(f,1).eq.cmodelab(i,2)) then
              if(ii.eq.0) then
                write(ilog,*)
                write(6,'(/20(1x,a))')(trim(cmodelab(j,1)),j=1,cndof(1))
                write(6,'( 20(1x,a))')(trim(cmodelab(j,2)),j=1,cndof(2))
              endif
              write(6,'(2a)') &
                       'modelabels not unique: ',trim(cmodelab(i,2))
              write(ilog,'(2a)') &
                       'modelabels not unique: ',trim(cmodelab(i,2))
              cmodelab(f,1)=trim(cmodelab(f,1))//'-1'
              cmodelab(i,2)=trim(cmodelab(i,2))//'-2'
              write(6,'(4a)') 'Changed to: ', &
                      trim(cmodelab(f,1)),' and ',trim(cmodelab(i,2))
              write(ilog,'(4a)') 'Changed to: ', &
                      trim(cmodelab(f,1)),' and ',trim(cmodelab(i,2))
              ii=ii+1
              goto 66
            endif
         enddo
      enddo
      if(ii.ge.2) then
         write(6,'(/a)') ' New modelabels:'
         write(6,'(20(1x,a))')(trim(cmodelab(j,1)),j=1,cndof(1))
         write(6,'(20(1x,a))')(trim(cmodelab(j,2)),j=1,cndof(2))
      endif

! Get maxpsidim 
      maxpsidim=cadim(1)*cadim(2)+ctotpdim(1)+ctotpdim(2)

!.....Check electronic states
      if(cnstate(2) .ne. 1  .or. cleb(2)) then
         routine='concat'
         message='Sorry, concat requests that the second WF '// &
         ' has no electronic states,                        '// &
         ' neither single-set nor multi-set.'
         call errormsg
      endif

!-----------------------------------------------------------------------
! Allocate memory
!-----------------------------------------------------------------------
      allocate(psi(maxpsidim))
      allocate(psi1(maxpsidim))
      allocate(psi2(maxpsidim))

!-----------------------------------------------------------------------
! After checks are done, now actually read the input restart files
!-----------------------------------------------------------------------
      lwrlog=.false. ! This is to avoid dublicate output to log file.
      do npsi=1,ncomp
        filename=filepsi(npsi)
        open(irst,file=filename,form='unformatted',status='old',err=1000)
        if(npsi.eq.1)then
          call rdrst(t,psi1,ldum,chkdvr,chkgrd,chkpsi,chkprp, &
                     cdum,idum,rdum,iflag,idum1,cdum1,idum2)
        elseif(npsi.eq.2)then
          call rdrst(t,psi2,ldum,chkdvr,chkgrd,chkpsi,chkprp, &
                     cdum,idum,rdum,iflag,idum1,cdum1,idum2)
        endif
        close(irst)
      enddo
      lwrlog=.true.  !reset lwrlog

!-----------------------------------------------------------------------
! Build the data for the concatenated WF
!-----------------------------------------------------------------------
      nstate=cnstate(1)
      ndof=cndof(1)+cndof(2)
      nmode=cnmode(1)+cnmode(2)
      leb=cleb(1)
      meb=cmeb(1)
      feb=cfeb(1)

! Set subdim, nspfdof, spfdof, and dim for the new WF
      do m=1,cnmode(1)
         subdim(m)=csubdim(m,1)
         nspfdof(m)=cnspfdof(m,1)
         do s=1,nstate
            dim(m,s)=cdim(m,s,1)
         enddo
         do n=1,cnspfdof(m,1)
            spfdof(n,m)=cspfdof(n,m,1)
         enddo
      enddo
      do m=1,cnmode(2)
         subdim(cnmode(1)+m)=csubdim(m,2)
         nspfdof(cnmode(1)+m)=cnspfdof(m,2)
         do s=1,nstate
            dim(cnmode(1)+m,s)=cdim(m,1,2)
         enddo
         do n=1,cnspfdof(m,2)
           spfdof(n,cnmode(1)+m)=cspfdof(n,m,2)+cndof(1)
         enddo
      enddo

! Set gdim,modelabel,dofspf,basis,ipbaspar,rpbaspar for the new WF
      do f=1,cndof(1)
         gdim(f)=cgdim(f,1)
         modelabel(f)=cmodelab(f,1)
         dofspf(f)=cdofspf(f,1)
         if(f.eq.feb) dofspf(f)=meb
         basis(f)=cbasis(f,1)
         do i=1,mbaspar
            ipbaspar(i,f)=cipbp(i,f,1)
            rpbaspar(i,f)=crpbp(i,f,1)
         enddo
      enddo
      do f=1,cndof(2)
         gdim(cndof(1)+f)=cgdim(f,2)
         modelabel(cndof(1)+f)=cmodelab(f,2)
         dofspf(cndof(1)+f)=cdofspf(f,2)+cnmode(1)
         basis(cndof(1)+f)=cbasis(f,2)
         do i=1,mbaspar
            ipbaspar(i,cndof(1)+f)=cipbp(i,f,2)
            rpbaspar(i,cndof(1)+f)=crpbp(i,f,2)
         enddo
      enddo

      call dvrdat  ! to set ldvr
      call griddat
      call psidat
      write(ilog,'(/a)') 'Concatenated WF:'
      write(ilog,'(2a)')' ndof nmode nstate leb     ', &
                        ' adim  totphidim     dgldim'
      write(ilog,'(3i5,l6,3i11)') ndof,nmode,nstate,leb, &
                                  adim,totphidim,dgldim
      write(ilog,'(78a1)') ('-',i=1,78)


!-----------------------------------------------------------------------
! Compute the A-vector
! A_{j_1 ...j_f_1,k_1...k_f_2} = A_{j_1 ...j_f_1} * A_{k_1...k_f_2}
!-----------------------------------------------------------------------
      kzpsi=0
      do jzpsi=1,cadim(2)
         do izpsi=1,cadim(1)
            kzpsi=kzpsi+1
            psi(kzpsi)=psi1(izpsi)*psi2(jzpsi)
         enddo
      enddo

! Load the SPFs, copy the SPFs of PSI1 to PSI
      do j=1,ctotpdim(1)
         psi(adim+j)=psi1(cadim(1)+j)
      enddo
! Loding the SPFs of PSI2 is more complicated, as PSI1 may
! have (multi-set) electronic states.
      jjs=adim+ctotpdim(1)
      jj=cadim(2)
      do m=1,cnmode(2)
         do s=1,nstate
            do j=1,csubdim(m,2)*cdim(m,1,2)
              jjs=jjs+1
              psi(jjs)=psi2(jj+j)
            enddo
         enddo
         jj=jj+csubdim(m,2)*cdim(m,1,2)
      enddo

! Write the restart-file of the concatenated wavefunction
      ldum=.true.
      t=0.d0
      call wrrst(t,psi,ldum,cdum,idum,rdum,rdum,idum1,cdum,idum1)
      close(irst)

! Write some information about the concatenated WF to log-file.
      write(ilog,'(/a/)')"***** PROPERTIES OF THE NEW RESTART WF *****"
      write(ilog,'(a,50(1x,a8))')" Modelabel:", (modelabel(f),f=1,ndof)
      write(ilog,'(a,50(1x,a8))')" DVR-Basis:",(cbas(basis(f)),f=1,ndof)
      write(ilog,'(a,50i9)') " gdim:", (gdim(f),f=1,ndof)
      write(ilog,'(/a,50i6)')" subdim:", (subdim(m),m=1,nmode)
      write(ilog,*)
      write(ilog,'(a,20i9)')" 1. block:", (cblock(s,1),s=1,nstate)
      write(ilog,'(a,20i9)')" 2. block:", (cblock(s,2),s=1,nstate)
      write(ilog,'(a,20i9)')"    block:", (block(s),s=1,nstate)
      write(ilog,*)
      write(ilog,'(a,i8)')" Number of SPF:"
      write(ilog,*)
      write(ilog,'(a,i8)')"    State(s)   Mode(m)   dim(m,s)"
      write(ilog,'(44a1)')  ('-',i=1,44)
      do s=1,nstate
         do m=1,nmode
            write(ilog,'(3(i8,3x))') s,m,dim(m,s)
         end do
      end do
      write(ilog,'(/a)') '   m, subdim, spfdof(n,m)'
      do m=1,nmode
         write(ilog,'(i4,i7,2x,12i3)') m,subdim(m), &
                 (spfdof(n,m),n=1,nspfdof(m))
      enddo
      write(ilog,'(/a)') '   f,   m, modelabel(f)'
      do f=1,ndof
         write(ilog,'(i4,i5,3x,a)') f,dofspf(f),trim(modelabel(f))
      enddo
      write(ilog,*)
      write(ilog,'(a)')"********************************************"


      write(ilog,*)
      write(ilog,'(a,i8)')"1.   State(s)   Mode(m)   dim(m,s)"
      write(ilog,'(44a1)')  ('-',i=1,44)
      do s=1,cnstate(1)
         do m=1,cnmode(1)
            write(ilog,'(3(i8,3x))') s,m,cdim(m,s,1)
         end do
      end do

      write(ilog,*)
      write(ilog,'(a,i8)')"2.  State(s)   Mode(m)   dim(m,s)"
      write(ilog,'(44a1)')  ('-',i=1,44)
      do s=1,cnstate(2)
         do m=1,cnmode(2)
            write(ilog,'(3(i8,3x))') s,m,cdim(m,s,2)
         end do
      end do

      write(ilog,'(/a)') '   m   s    zetf(m,s)'
      do m=1,cnmode(1)
         do s=1,cnstate(1)
            write(ilog,'(2i4,i8)') m,s,czetf(m,s,1)
         enddo
      enddo
      write(ilog,'(/a)') '   m   s    zetf(m,s)'
      do m=1,nmode
         do s=1,nstate
            write(ilog,'(2i4,i8)') m,s,zetf(m,s)
         enddo
      enddo

!-----------------------------------------------------------------------
! Build the dvr-file if option -dvr is set.
!-----------------------------------------------------------------------
      if (lrundvr) then
     !    neu=.true.
         dname=name
         dlaenge=laenge
         filename=name(1:laenge)//'/dvr'
         ilbl=index(filename,' ')-1
         write(ilog,'(/,72a1)')  ('#',i=1,72)
         write(ilog,'(/,a)') ' Creating DVR file does not work '
         write(ilog,'(72a1)')  ('#',i=1,72)
     !    write(ilog,'(/,72a1)')  ('#',i=1,72)
     !    write(ilog,'(/,a)') ' Creating DVR file: '//filename(1:ilbl)
     !    write(ilog,'(72a1)')  ('#',i=1,72)
     !    call zeigdvr
     !    call memdvr
     !    dynbytes(1) = complexsize()*(mcdim/8+1)
     !    dynbytes(2) = realsize()   *(mrdim/8+1)
     !    dynbytes(3) = integersize()*(midim/8+1)
     !    dynbytes(4) = logicalsize()*(mldim/8+1)
     !    call calldvr(dynbytes)
     !    call logdvr(ilog)
         write(ilog,'(78a1)')  ('-',i=1,78)
         call logsbas
         write(ilog,'(78a1)')  ('-',i=1,78)
      endif

!-----------------------------------------------------------------------
! DeAllocate memory
!-----------------------------------------------------------------------
      deallocate(psi,psi1,psi2)
      deallocate(cipbp,crpbp,cmodelab)

      stop
 1000 write(6,'(/a)') ' Cannot open file for reading : '
      write(6,'(a)')  trim(filename)
      stop
 2000 write(6,'(/a)')' Cannot open log-file. Set overwrite option -w :'
      write(6,'(a)')  trim(filename)
      stop

      end program concat
