!#######################################################################
!
! Interconverts a direct-dynamics database between binary (DB) and 
! ascii (DBA)
!
!#######################################################################

      program convdb

use decimal, only: sip
use global
use versions

use channels
use dirdyn
use dbtools

      implicit none

      integer(long)     :: ierr,iend,nignore,nrdfiles,nmaxrec
      character(len=10) :: cerr
      integer(long)     :: nmdof,socdim
      integer(long), external :: myiargc
      integer(long)           :: nrec,s,s1,ndof1,n,n1,interp,i,iarg,&
                                 j,joff
      character(len=c1)       :: binstat,ascstat,filepos
      character(len=120)      :: string
      character(len=1)        :: atmp
      character(len=10)       :: buf

      integer(long), dimension(:), allocatable    :: gwpsymrep
      real(dop), dimension(:), allocatable        :: gpoint,nmpoint
      real(dop), dimension(:,:), allocatable      :: v,diabtrans
      real(dop), dimension(:,:), allocatable      :: cv
      real(dop), dimension(:,:,:), allocatable    :: deriv1
      real(dop), dimension(:,:,:,:), allocatable  :: deriv2
      real(dop), dimension(:,:,:), allocatable    :: dip
      real(dop), dimension(:,:), allocatable      :: av
      real(dop), dimension(:,:), allocatable      :: aderiv1
      real(dop), dimension(:,:,:), allocatable    :: aderiv2
      real(dop), dimension(:,:,:), allocatable    :: adip
      real(sip), dimension(:,:), allocatable      :: mocoeff
      real(dop), dimension(:,:), allocatable      :: nact
      logical(kind=4)                             :: a2b,check,checka,lbatch,&
                                                     lclean,lasciidbold
      complex(dop), dimension(:,:), allocatable   :: casoc

! defaults
! a2b: if true ascii -> binary. false binary -> ascii
! lbatch: if true no output ot screen (batch job)
      a2b = .false.
      lbatch = .false.
      lclean = .false.
      ddname = './'
      nignore = 0
      nmaxrec = 0

! default
      nbasis=-999
      nddstate=-999
      nsocstate=-999
      dercpdim = 0

! read options
      iarg = myiargc()

      i=0
      do while (i .le. iarg)
         i=i+1
         call mygetarg(i,buf)
         if ( buf .eq. '-ver' ) then
            call wrversion(6)
            stop
         else if ( buf .eq. '-i' ) then
            i=i+1
            call mygetarg(i,ddname)
         else if ( buf .eq. '-batch' ) then
            lbatch = .true.
         else if ( buf .eq. '-c' ) then
            lclean = .true.
         else if ( buf .eq. '-r' ) then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*) nignore
         else if ( buf .eq. '-mx' ) then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*) nmaxrec
         else if ( buf .eq. '-h' .or. buf(1:2) .eq. '-?') then
            write(6,'(/,78a1,/)')  ('*',i=1,78)
            write(6,'(a)') ' Analyse program "convdb" '
            write(6,'(1x,a)')&
       'Purpose : Converts a DD binary DB to ascii OR an ascii DB to binary'
            write(6,'(1x,2a)') 'Usage   : convdb ',&
       '[-batch -i NAME -c -ver -h -?] '
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(/,a)') ' Options : '
            write(6,'(a)') &
            ' -i NAME : The DB is in directory NAME '
            write(6,'(a)') &
            ' -c      : The DB is cleaned by removing points from failed calculations '
            write(6,'(a)') &
            ' -batch  : Batch mode - no output to the screen '
            write(6,'(a)') &
            ' -r nignore : The DB entry nignore is removed '
            write(6,'(a)') &
            ' -mx nmaxrec : Write only the first nmaxrec records to the DB '
            write(6,'(a)')&
            ' -ver    : Version information about the program.'
            write(6,'(a,/a)') &
            ' -h      : Print this help text. ',&
            ' -?      : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else if (buf(1:1) .eq. "-") then
            write(6,'(2a)') ' unknown option : ', buf
            stop

         endif
      end do

!
! get full path
!
      ddlaenge=index(ddname,' ')-1
      call abspath(ddname,ddlaenge)

!-----------------------------------------------------------------------
! set up file names and see which DB type and files to be processed
!-----------------------------------------------------------------------
      call chkdb(ierr)
      if (ierr .eq. 1) stop

      if (lasciidb) then
         a2b = .true.
      else
         a2b = .false.
      endif

! set status of files depending on conversion
      if (a2b) then
         ascstat = 'old'
         binstat = 'new'
      else 
         ascstat = 'new'
         binstat = 'old'
      endif
      filepos='rewind'

!-----------------------------------------------------------------------
! check correct formatting will be performed
!-----------------------------------------------------------------------
      if (.not. lbatch) then
         if (a2b) then
            write(6,'(a)') 'Converting from ascii to binary'
            write(6,'(a)') 'Existing ascii DB files will be destroyed'
         else 
            write(6,'(a)') 'Converting from binary to ascii'
            write(6,'(a)') 'Existing binary DB files will be destroyed'
         endif
         if (nignore .gt. 0) then
            write(6,'(a,i7)') 'Ignoring Record : ',nignore
         endif
         if (nmaxrec .gt. 0) then
            write(6,'(a,i7)') 'Truncating after Record : ',nmaxrec
         endif

         write(6,'(a)') 'Continue ? <y / n> '
         read(5,*) atmp
         if (atmp .eq. 'n' .or. atmp .eq. 'N') stop

      endif

!-----------------------------------------------------------------------
! open files
!-----------------------------------------------------------------------
! save lascii db and close both sets of files
      lasciidbold=lasciidb
      lasciidb=.true.
      call opendbold(ascstat,filepos)
      lasciidb=.false.
      call opendbold(binstat,filepos)
!restore lasciidb flag
      lasciidb=lasciidbold

!-----------------------------------------------------------------------
! get data from refdb.dat file
!-----------------------------------------------------------------------
   10 continue
      read(irefdb,'(a)',end=20) string
      if (string(1:23) .eq. '# Natm, Nroot, Nddstate') then
         read(irefdb,*) natm,nroot,nddstate
      else if (string(1:14) .eq. '# Natm, Nstate') then
         read(irefdb,*) natm,nddstate
      else if (string(1:8) .eq. '# Nbasis') then
         read(irefdb,*) nbasis
       else if (string(1:10) .eq. '# Dercpdim')then
         read(irefdb,*) dercpdim
      else if (string(1:18) .eq. '# Complex Energies') then
         lsocbas = .true.
      else if (string(1:18) .eq. '# Pntgp') then
         read(irefdb,*) dbpntgp,nsymop
      else if (string(1:11) .eq. '# Nsocstate') then
         read(irefdb,*) nsocstate
      endif
      go to 10

20    continue
      if (nsocstate .lt. 0) nsocstate=nddstate  ! Not defined in old refdb

! 
! get sizes and assign arrays
!
      ndof1 = natm*3
      if (lnmdb)then
         if(natm.gt.2)then
            nmdof = ndof1-6
         else
            nmdof = ndof1-5
         endif
      endif

      maxdim = 3*natm+3
      maxsta = max(nddstate,nsocstate)
      dercpdim = nddstate*(nddstate-1)/2
      socdim = nsocstate*(nsocstate-1)/2
      cerr = 'gpoint'
      allocate(gpoint(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'v'
      allocate(v(maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      if(lcpesdb)then
         cerr = 'cv'
         allocate(cv(maxsta,maxsta),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if(lgradb)then
         cerr = 'deriv1'
         allocate(deriv1(maxdim,maxsta,maxsta),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if(lhesdb)then
         cerr = 'deriv2'
         allocate(deriv2(maxdim,maxdim,maxsta,maxsta),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      cerr = 'av'
      allocate(av(maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'aderiv1'
      allocate(aderiv1(maxdim,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'aderiv2'
      allocate(aderiv2(maxdim,maxdim,maxsta),stat=ierr)
      if (ldipdb) then
         cerr = 'dip'
         allocate(dip(3,maxsta,maxsta),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'adip'
         allocate(adip(3,maxsta,maxsta),stat=ierr)
         if (ierr > 0) go to 1000
      else
         allocate(dip(1,1,1),stat=ierr)
      endif
      if(lnmdb)then
         cerr = 'nmpoint'
         allocate(nmpoint(maxdim),stat=ierr)
         if(ierr .gt.0) goto 1000
      endif
      if(lmocodb)then
         if(nbasis.lt.0)then
            write(6,'(a)')'Need # Nbasis entry in refdb'
            stop
         else
            allocate(mocoeff(nbasis,nbasis))
         endif
      else
         allocate(mocoeff(1,1))
      endif
      if(lnactdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(nact(ndof1,dercpdim))
         endif
      else
         allocate(nact(1,1))
      endif
      if(ltransdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(diabtrans(nddstate,nddstate))
         endif
      else
         allocate(diabtrans(1,1))
      endif
      if(lcasocdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(casoc(nddstate,nddstate))
         endif
      else
         allocate(casoc(1,1))
      endif
      if(lsymdb)then
         allocate(gwpsymrep(nsymop))
      else
         allocate(gwpsymrep(1))
      endif

!
! read and write DB records
! nrdfiles is a check that all files in DB are read and converted
!
      if (a2b) then

  100    continue
         nrdfiles=0

         read(igeodba,'(14x,i15,12x,i5)',end=200) nrec,interp
         if ((nmaxrec .gt. 0) .and. (nrec .gt. nmaxrec)) go to 200
         nrdfiles=nrdfiles+1
         joff = 0
         do i = 1, ndof1/3
            read(igeodba,'(3f20.12)',err=101) (gpoint(joff+j),j=1,3)
  101       continue
            joff = joff + 3
         enddo

         if (lpesdb) then
            read(ipesdba,*) 
            read(ipesdba,'(4f20.12)',err=102)&
               ((v(s1,s),s1=s,nddstate),s=1,nddstate)
  102       continue
            nrdfiles=nrdfiles+1
         endif
         if (lgradb) then
            read(igradba,*) 
            read(igradba,'(3f20.12)',err=103)&
               (((deriv1(n,s1,s),n=1,ndof1),s1=s,nddstate),s=1,nddstate)
  103       continue
            nrdfiles=nrdfiles+1
         endif
         if (lhesdb) then
            read(ihesdba,*) 
            read(ihesdba,'(3f20.12)',err=104)&
               ((((deriv2(n,n1,s1,s),n=1,ndof1),n1=1,ndof1),&
                             s1=s,nddstate),s=1,nddstate)
  104       continue
            nrdfiles=nrdfiles+1
         endif
         if (lapesdb) then
            read(iapesdba,*) 
            read(iapesdba,'(4f20.12)',err=105) (av(s,s),s=1,nddstate)
  105       continue
            nrdfiles=nrdfiles+1
         endif
         if (lagradb) then
            read(iagradba,*) 
            read(iagradba,'(3f20.12)',err=106)&
               ((aderiv1(n,s),n=1,ndof1),s=1,nddstate)
  106       continue
            nrdfiles=nrdfiles+1
         endif
         if (lahesdb) then
            read(iahesdba,*,iostat=iend) 
            read(iahesdba,'(3f20.12)',iostat=iend,err=107)&
               (((aderiv2(n,n1,s),n=1,ndof1),n1=1,ndof1),s=1,nddstate)
  107       continue
! may only have 1 Hessian if doing diabatisation of data
            if ((iend .ne. 0) .and. (nrec .eq. 1)) then
               write(6,'(a)') 'ERROR : adhes.dba file empty'
               stop
            else if (iend .ne. 0) then
               aderiv2=0.0
            endif
            nrdfiles=nrdfiles+1
         endif
         if (ldipdb) then
            read(idipdba,*) 
            read(idipdba,'(3f20.12)',err=108)&
               ((dip(n,s,s),n=1,3),s=1,nddstate)
  108       continue
            nrdfiles=nrdfiles+1
         endif
         if (ladipdb) then
            read(iadipdba,*) 
            read(iadipdba,'(3f20.12)',err=109)&
               ((adip(n,s,s),n=1,3),s=1,nddstate)
  109       continue
            nrdfiles=nrdfiles+1
         endif
         if(lmocodb) then
            read(imocodba,*) 
            read(imocodba,'(3f20.12)',err=110)&
               ((mocoeff(n,s),n=1,nbasis),s=1,nbasis)
  110       continue
            nrdfiles=nrdfiles+1
         endif
         if(lnactdb) then
            read(inactdba,*)
            do s = 1,dercpdim
               joff = 0
               do n = 1, ndof1/3
                  read(inactdba,'(3f20.12)',err=111) (nact(j+joff,s),j=1,3)
  111       continue
                  joff = joff + 3
               enddo
            enddo
            nrdfiles=nrdfiles+1
         endif
         if(ltransdb) then
            read(itransdba,*) 
            read(itransdba,'(2f20.12)',err=112)&
               ((diabtrans(n,s),n=1,nddstate),s=1,nddstate)
  112       continue
            nrdfiles=nrdfiles+1
         endif
         if(lnmdb)then
            read(inmdba,*)
            read(inmdba,'(4f20.12)',err=113) (nmpoint(j),j=1,nmdof)
  113       continue
            nrdfiles=nrdfiles+1
         endif
         if(lcasocdb) then
            read(icasocdba,*) 
            read(icasocdba,'(2f20.12)',err=120)&
                    ((casoc(s1,s),s1=s+1,nsocstate),s=1,nsocstate)
  120       continue
            nrdfiles=nrdfiles+1
         endif
         if (lsymdb) then
            read(isymdba,*,err=118) (gwpsymrep(n),n=1,nsymop)
  118       continue
            nrdfiles=nrdfiles+1
         endif
         if (lcpesdb) then
            read(icpesdba,*) 
            read(icpesdba,'(4f20.12)',err=121)&
               ((cv(s1,s),s1=s,nddstate),s=1,nddstate)
  121       continue
            nrdfiles=nrdfiles+1
         endif

! check all files read
         if (nrdfiles .ne. ndbfiles) then
            write(6,'(2(a,i5))') 'ERROR: Not all DB files read. ',&
                 nrdfiles,' of ',ndbfiles
            stop
         endif

! if cleaning DB do not write interpolated points
         if (lclean .and. interp .eq. -1) go to 100

! if DB entry to be removed do not write 
         if (nrec .eq. nignore) go to 100

         if (.not. lbatch) write(6,'(a,i10)') 'Writing record : ',nrec

         write(igeodb) interp,(gpoint(i),i=1,ndof1)
         if (lpesdb) write(ipesdb) ((v(s1,s),s1=s,nddstate),s=1,nddstate)
         if (lcpesdb) write(icpesdb) ((cv(s1,s),s1=s,nddstate),s=1,nddstate)
         if (lgradb) then
            write(igradb) (((deriv1(n,s1,s),n=1,ndof1),s1=s,nddstate),&
                       s=1,nddstate)
         endif
         if (lhesdb) then
             write(ihesdb) ((((deriv2(n,n1,s1,s),n=1,ndof1),n1=1,ndof1),&
                       s1=s,nddstate),s=1,nddstate)
         endif
         if (lapesdb) write(iapesdb) (av(s,s),s=1,nddstate)
         if (lagradb) then
            write(iagradb) ((aderiv1(n,s),n=1,ndof1),s=1,nddstate)
         endif
         if (lahesdb) then
            write(iahesdb) (((aderiv2(n,n1,s),n=1,ndof1),n1=1,ndof1),&
                          s=1,nddstate)
         endif
         if (ldipdb) then
            write(idipdb) ((dip(n,s,s),n=1,3),s=1,nddstate)
         endif
         if (ladipdb) then
            write(iadipdb) ((adip(n,s,s),n=1,3),s=1,nddstate)
         endif
         if (lmocodb) then
            write(imocodb) ((mocoeff(n,s),n=1,nbasis),s=1,nbasis)
         endif
         if (ltransdb) then
            write(itransdb) ((diabtrans(s,n),s=1,nddstate),n=1,nddstate)
         endif
         if (lnactdb) then
            write(inactdb) ((nact(n,s),n=1,ndof1),s=1,dercpdim)
         endif
         if(lnmdb) write(inmdb) (nmpoint(i),i=1,nmdof)
         if (lcasocdb) then
            write(icasocdb) ((casoc(s1,s),s1=s+1,nsocstate),s=1,nsocstate)
         endif
         if (lsymdb) write(isymdb) (gwpsymrep(n),n=1,nsymop)

! get next record
         go to 100

      else

         nrec = 0
  150    continue
         nrdfiles = 0

         read(igeodb,end = 200) interp,(gpoint(i),i=1,ndof1)
         nrdfiles=nrdfiles+1
         nrec = nrec+1
         if ((nmaxrec .gt. 0) .and. (nrec .gt. nmaxrec)) go to 200
         if (lpesdb) then
            read(ipesdb) ((v(s1,s),s1=s,nddstate),s=1,nddstate)
            nrdfiles=nrdfiles+1
         endif
         if (lcpesdb) then
            read(icpesdb) ((cv(s1,s),s1=s,nddstate),s=1,nddstate)
            nrdfiles=nrdfiles+1
         endif
         if (lgradb) then
            read(igradb) (((deriv1(n,s1,s),n=1,ndof1),s1=s,nddstate),&
                          s=1,nddstate)
            nrdfiles=nrdfiles+1
         endif
         if (lhesdb) then
            read(ihesdb) ((((deriv2(n,n1,s1,s),n=1,ndof1),n1=1,ndof1),&
                          s1=s,nddstate),s=1,nddstate)
            nrdfiles=nrdfiles+1
         endif
         if (lapesdb) then
            read(iapesdb) (av(s,s),s=1,nddstate)
            nrdfiles=nrdfiles+1
         endif
         if (lagradb) then
            read(iagradb) ((aderiv1(n,s),n=1,ndof1),s=1,nddstate)
            nrdfiles=nrdfiles+1
         endif
         if (lahesdb) then
            read(iahesdb) (((aderiv2(n,n1,s),n=1,ndof1),n1=1,ndof1),&
                          s=1,nddstate)
            nrdfiles=nrdfiles+1
         endif
         if (ldipdb) then
            read(idipdb) ((dip(n,s,s),n=1,3),s=1,nddstate)
            nrdfiles=nrdfiles+1
         endif
         if (ladipdb) then
            read(iadipdb) ((adip(n,s,s),n=1,3),s=1,nddstate)
            nrdfiles=nrdfiles+1
         endif
         if (lmocodb) then
            read(imocodb) ((mocoeff(n,s),n=1,nbasis),s=1,nbasis)
            nrdfiles=nrdfiles+1
         else
            mocoeff=0.0
         endif
         if (lnactdb) then
            read(inactdb) ((nact(n,s),n=1,ndof1),s=1,dercpdim)
            nrdfiles=nrdfiles+1
         else
            nact=0.0_dop
         endif
         if (ltransdb) then
            read(itransdb) ((diabtrans(s,n),s=1,nddstate),n=1,nddstate)
            nrdfiles=nrdfiles+1
         else
            diabtrans=0.0_dop
         endif
         if (lnmdb) then
            read(inmdb) (nmpoint(i),i=1,nmdof)
            nrdfiles=nrdfiles+1
         endif
         if (lcasocdb) then
            read(icasocdb) ((casoc(s1,s),s1=s+1,nsocstate),s=1,nsocstate)
            nrdfiles=nrdfiles+1
         endif

         if (lsymdb) then
            read(isymdb) (gwpsymrep(n),n=1,nsymop)
            nrdfiles=nrdfiles+1
         endif

! check all files read
        if (nrdfiles .ne. ndbfiles) then
           write(6,'(2(a,i5))') 'ERROR: Not all DB files read. ',nrdfiles,' of ',ndbfiles
           stop
         endif

! if cleaning DB do not write interpolated points
         if (lclean .and. interp .eq. -1) go to 150

! if DB entry to be removed do not write 
         if (nrec .eq. nignore) go to 150

         if (.not. lbatch) write(6,'(a,i10)') 'Writing record : ',nrec

         write(igeodba,'(a,i15,a,i5)') '#Record No. : ',nrec,'   Status : ',interp
         joff = 0
         do i = 1, ndof1/3
            write(igeodba,'(3f20.12)') (gpoint(joff+j),j=1,3)
            joff = joff + 3
         enddo

         if (lpesdb) then
            write(ipesdba,'(a,i15)') '#Record No. : ',nrec
            write(ipesdba,'(4f20.12)')&
               ((v(s1,s),s1=s,nddstate),s=1,nddstate)
         endif
         if (lcpesdb) then
            write(icpesdba,'(a,i15)') '#Record No. : ',nrec
            write(icpesdba,'(4f20.12)')&
               ((cv(s1,s),s1=s,nddstate),s=1,nddstate)
         endif
         if (lgradb) then
            write(igradba,'(a,i15)') '#Record No. : ',nrec
            write(igradba,'(3f20.12)')&
               (((deriv1(n,s1,s),n=1,ndof1),s1=s,nddstate),s=1,nddstate)
         endif
         if (lhesdb) then
            write(ihesdba,'(a,i15)') '#Record No. : ',nrec
            write(ihesdba,'(3f20.12)') &
               ((((deriv2(n,n1,s1,s),n=1,ndof1),n1=1,ndof1),&
                          s1=s,nddstate),s=1,nddstate)
         endif
         if (lapesdb) then
            write(iapesdba,'(a,i15)') '#Record No. : ',nrec
            write(iapesdba,'(4f20.12)') (av(s,s),s=1,nddstate)
         endif

         if (lagradb) then
            write(iagradba,'(a,i15)') '#Record No. : ',nrec
            write(iagradba,'(3f20.12)') ((aderiv1(n,s),n=1,ndof1),s=1,nddstate)
         endif

         if (lahesdb) then
            write(iahesdba,'(a,i15)') '#Record No. : ',nrec
            write(iahesdba,'(3f20.12)') &
               (((aderiv2(n,n1,s),n=1,ndof1),n1=1,ndof1),&
                          s=1,nddstate)
         endif
         if (ldipdb) then
            write(idipdba,'(a,i15)') '#Record No. : ',nrec
            write(idipdba,'(3f20.12)') &
                ((dip(n,s,s),n=1,3),s=1,nddstate)
         endif
         if (ladipdb) then
            write(iadipdba,'(a,i15)') '#Record No. : ',nrec
            write(iadipdba,'(3f20.12)') &
                ((adip(n,s,s),n=1,3),s=1,nddstate)
         endif
         if (lmocodb) then
            write(imocodba,'(a,i15)') '#Record No. : ',nrec
            write(imocodba,'(3f20.12)') &
                ((mocoeff(n,s),n=1,nbasis),s=1,nbasis)
         endif
         if (lnactdb) then
            write(inactdba,'(a,i15)') '#Record No. : ',nrec
            do s = 1, dercpdim
               joff = 0
               do n=1,ndof1/3
                  write(inactdba,'(3f20.12)') (nact(j+joff,s),j=1,3)
                  joff = joff + 3
               enddo
            enddo
         endif
         if (ltransdb) then
            write(itransdba,'(a,i15)') '#Record No. : ',nrec
            write(itransdba,'(2f20.12)') &
                ((diabtrans(n,s),n=1,nddstate),s=1,nddstate)
         endif
         if(lnmdb)then
            write(inmdba,'(a,i15)') '#Record No. : ',nrec
            write(inmdba,'(4f20.12)') (nmpoint(i),i=1,nmdof)
         endif
         if(lcasocdb) then
            write(icasocdba,*) '#Record No. : ',nrec
            write(icasocdba,'(2f20.12)')&
                    ((casoc(s1,s),s1=s+1,nsocstate),s=1,nsocstate)
         endif
         if (lsymdb) then
            write(isymdba,*) (gwpsymrep(n),n=1,nsymop)
         endif

!
! get next record
!
         go to 150

      endif

  200 continue

!
! delete old DB and keep new files
!
      if (a2b) then
         ascstat = 'delete'
         binstat = 'keep'
      else 
         ascstat = 'keep'
         binstat = 'delete'
      endif
    
      if (.not. lbatch) write(6,'(a)') 'Deleting old DB files '

! save lascii db and close both sets of files
      lasciidbold=lasciidb
      lasciidb=.true.
      call closedb(ascstat)
      lasciidb=.false.
      call closedb(binstat)
!restore lasciidb flag
      lasciidb=lasciidbold

! Deallocation
      if(allocated(gpoint))deallocate(gpoint)
      if(allocated(v))deallocate(v)
      if(allocated(cv))deallocate(cv)
      if(allocated(deriv1))deallocate(deriv1)
      if(allocated(deriv2))deallocate(deriv2)
      if(allocated(av))deallocate(av)
      if(allocated(aderiv1))deallocate(aderiv1)
      if(allocated(aderiv2))deallocate(aderiv2)
      if (allocated(dip)) deallocate(dip)
      if (allocated(adip)) deallocate(adip)
      if (allocated(mocoeff)) deallocate(mocoeff)
      if (allocated(diabtrans)) deallocate(diabtrans)
      if (allocated(nact)) deallocate(nact)
      if (allocated(nmpoint)) deallocate(nmpoint)
      if (allocated(casoc)) deallocate(casoc)
      if (allocated(gwpsymrep)) deallocate(gwpsymrep)

      stop
 1000 write(6,'(a)') 'Error allocating memory in CONVDB :',cerr
      write(6,'(a)') 'Error Code :',ierr
      stop

      end program convdb
