!#######################################################################
!
! Interconverts a direct-dynamics database from an SQLite3 format to
! an ascii (DBA) 
!
!#######################################################################

      program convdbfromsql

use decimal, only: sip
use global ! lsocbas
use versions, only: progver,wrversion
use sqlite
use db_sql
use dbtools
use dirdyn
use versions

      implicit none

      integer(long)     :: ierr,nignore,nmaxrec,numrecs
      character(len=10) :: cerr
      integer(long)     :: nmdof
      integer(long), external :: myiargc
      integer(long)           :: nrec,s,s1,ndof1,n,n1,i,iarg,&
                                 j,joff, timestamp,ns
      integer(long)           :: socdim
      real(dop)               :: quantver
      !Version of the sql database 
      integer(long)           :: sql_vers
      character(len=120)      :: ascstat,filepos
      character(len=1)        :: atmp
      character(len=10)       :: buf
      character(len=c5)       :: indbfile

      integer(long), dimension(1)                   :: interp
      integer(long), dimension(:,:), allocatable    :: symrep
      real(dop), dimension(:,:), allocatable        :: gpoint,nmpoint
      real(dop), dimension(:,:,:), allocatable      :: v,diabtrans
      real(dop), dimension(:,:,:), allocatable      :: cv,diabctrans
      real(dop), dimension(:,:,:,:), allocatable    :: deriv1
      real(dop), dimension(:,:,:,:,:), allocatable  :: deriv2
      real(dop), dimension(:,:,:,:), allocatable    :: dip
      real(dop), dimension(:,:,:), allocatable      :: av
      real(dop), dimension(:,:,:,:), allocatable    :: aderiv1
      real(dop), dimension(:,:,:,:,:), allocatable  :: aderiv2
      real(dop), dimension(:,:,:,:), allocatable    :: adip
      real(sip), dimension(:,:,:), allocatable      :: mocoeff
      real(dop), dimension(:,:,:), allocatable      :: nact
      complex(dop), dimension(:,:,:), allocatable   :: asoc
      character(len=30), dimension(:,:,:), allocatable :: ciconfig
      real(dop), dimension(:,:,:,:), allocatable    :: cicoeff

      logical(kind=4)                             :: lbatch,lclean,lkeep
      real(dop)                                   :: tt1,tt2

      type(SQLITE_DATABASE)                       :: db

! defaults
! lbatch: if true no output ot screen (batch job)
      call default
      lbatch = .false.
      ddname = './'
      indbfile = 'database.sql'
      lkeep = .false.

      lasciidb=.true.

! default
      nbasis=-999
      nddstate=-999
      nignore = 0
      nmaxrec = 0

! read options
      iarg = myiargc()

      i=0
      do while (i .le. iarg)
         i=i+1
         call mygetarg(i,buf)
         if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else if ( buf .eq. '-i' ) then
            i=i+1
            call mygetarg(i,ddname)
         else if ( buf .eq. '-batch' ) then
            lbatch = .true.
         else if ( buf .eq. '-c' ) then
            lclean = .true.
         else if ( buf .eq. '-r' ) then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*) nignore
         else if ( buf .eq. '-mx' ) then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*) nmaxrec
         else if ( buf .eq. '-k' ) then
            lkeep = .true.
         else if ( buf .eq. '-binary' ) then
            lasciidb = .false.
         else if ( buf .eq. '-h' .or. buf(1:2) .eq. '-?') then
            write(6,'(/,78a1,/)')  ('*',i=1,78)
            write(6,'(a)') ' Analyse program "convdbfromsql" '
            write(6,'(1x,a)')&
       'Purpose : Converts an SQL DB to ascii (or binary) flat file format'
            write(6,'(1x,2a)') 'Usage   : convdbfromsql ',&
       '[-batch -i NAME -c -ver -h -?] '
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(/,a)') ' Options : '
            write(6,'(a)') &
            ' -i NAME : The DB is in directory NAME '
            write(6,'(a)') &
            ' -c      : The DB is cleaned by removing points from failed calculations '
            write(6,'(a)') &
            ' -batch  : Batch mode - no output to the screen '
            write(6,'(a)') &
            ' -r nignore : The DB entry nignore is removed '
            write(6,'(a)') &
            ' -mx nmaxrec : Write only the first nmaxrec records to the DB '
            write(6,'(a)')&
            ' -k      : The DB file is not deleted (default: it is)'
            write(6,'(a)') &
            ' -binary : The DB is binary '
            write(6,'(a)') &
            ' -ver    : Version information about the program.'
            write(6,'(a,/a)') &
            ' -h      : Print this help text. ',&
            ' -?      : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if (buf(1:1) .eq. "-") then
            write(6,'(2a)') ' unknown option : ', buf
            stop

         endif
      end do

!-----------------------------------------------------------------------
! Start timer
!-----------------------------------------------------------------------
      call cpu_time(tt1)

!
! get full path
!
      ddlaenge=index(ddname,' ')-1
      call abspath(ddname,ddlaenge)
!
! Open SQL database
!
! read-only
      !As read only version, info and file are all dummies
      call dbopen(ddname(1:ddlaenge)//'/'//indbfile,db,sqlvers,quanticsver,&
           infoarray, flagarray, 0)
      call dbgetallversion(db, sql_vers,quantver)
      call dbrdrefdb(db, natm, nroot, nddstate, nbasis, dde0, dimref, &
                     nfam, dercpdim,dbpntgp,nsymop,nsocstate,nddmult)
      call dbrdrefdbgeom(db, natm, mref, atnam, xref)
      ndof1 = natm*3
      if (dbpntgp .gt. 0) then
         call dbrdrefdbmaps(db, natm, nsymop, dbsymmap)
      endif
!
! see what's in the SQL db
!
      call dbgetentries( db, 6, lpesdb, lgradb, lhesdb, lapesdb, lagradb,&
           lahesdb, ldipdb, ladipdb, lmocodb, lnactdb, ltransdb, lnmdb, &
           lcasocdb, lsymdb, lcpesdb, lctransdb, lcivecdb, ltemplate, &
           lqcfreq)

! If CIvec to be read, multiplicity data is needed
      if (lcivecdb) then
         nsmult=count(nddmult .gt. 0)
         allocate(idxmult(nsmult))
         allocate(imultblk(nsmult))
         allocate(nciconf(nsmult))
         call dbrdcidat(db, nsmult, idxmult,imultblk,nciconf)
         maxmultblk = maxval(imultblk)
         maxcivec = maxval(nciconf)
      endif


! check correct formatting will be performed
      if (.not. lbatch) then
         if (lasciidb) then
            write(6,'(a)') 'Converting from SQL to ascii files'
         else
            write(6,'(a)') 'Converting from SQL to binary files'
         endif
         write(6,'(a)') 'Continue ? <y / n> '
         read(5,*) atmp
         if (atmp .eq. 'n' .or. atmp .eq. 'N') stop
      endif
!
! get sizes and assign arrays
!
      ndof1 = natm*3
      if (lnmdb)then
         if(natm.gt.2)then
            nmdof = ndof1-6
         else
            nmdof = ndof1-5
         endif
      endif

      maxdim = 3*natm+3
      maxsta = max(nddstate,nsocstate)
      if(dercpdim.le.0) dercpdim = nddstate*(nddstate-1)/2
      socdim = nsocstate*(nsocstate-1)/2

      cerr = 'gpoint'
      allocate(gpoint(maxdim,1),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'v'
      allocate(v(maxsta,maxsta,1),stat=ierr)
      if (ierr > 0) go to 1000
      if(lcpesdb)then
         cerr = 'cv'
         allocate(cv(maxsta,maxsta,1),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if(lgradb)then
         cerr = 'deriv1'
         allocate(deriv1(maxdim,maxsta,maxsta,1),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if(lhesdb)then
         cerr = 'deriv2'
         allocate(deriv2(maxdim,maxdim,maxsta,maxsta,1),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      cerr = 'av'
      allocate(av(maxsta,maxsta,1),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'aderiv1'
      allocate(aderiv1(maxdim,maxsta,maxsta,1),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'aderiv2'
      allocate(aderiv2(maxdim,maxdim,maxsta,maxsta,1),stat=ierr)
      if (ldipdb) then
         cerr = 'dip'
         allocate(dip(3,maxsta,maxsta,1),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'adip'
         allocate(adip(3,maxsta,maxsta,1),stat=ierr)
         if (ierr > 0) go to 1000
      else
         allocate(dip(1,1,1,1),stat=ierr)
      endif
      if(lnmdb)then
         cerr = 'nmpoint'
         allocate(nmpoint(maxdim,1),stat=ierr)
         if(ierr .gt.0) goto 1000
      endif
      if(lmocodb)then
         if(nbasis.lt.0)then
            write(6,'(a)')'Need # Nbasis entry in refdb'
            stop
         else
            allocate(mocoeff(nbasis,nbasis,1))
         endif
      else
         allocate(mocoeff(1,1,1))
      endif
      if(lnactdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(nact(ndof1,dercpdim,1))
         endif
      else
         allocate(nact(1,1,1))
      endif
      if(ltransdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(diabtrans(nddstate,nddstate,1))
         endif
      else
         allocate(diabtrans(1,1,1))
      endif
      if(lctransdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(diabctrans(nddstate,nddstate,1))
         endif
      else
         allocate(diabctrans(1,1,1))
      endif
      if(lcasocdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(asoc(nsocstate,nsocstate,1))
         endif
      else
         allocate(asoc(1,1,1))
      endif
      if (lsymdb) then
         cerr = 'symrep'
         allocate(symrep(maxsymop,1),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if (lcivecdb) then
         cerr = 'civec'
         allocate(ciconfig(maxcivec,nsmult,1),stat=ierr)
         allocate(cicoeff(maxcivec,maxmultblk,nsmult,1),stat=ierr)
         if (ierr > 0) go to 1000
      endif

!-----------------------------------------------------------------------
! open files
!-----------------------------------------------------------------------
! save lascii db and close both sets of files
      call setfilenames
      ascstat = 'new'
      filepos='rewind'
      call opendbold(ascstat,filepos)
!
! write refdb.dat
!
      call wrrefdborig
!
! write cidb.dat
!
      call wrcidbdat
!
! read and write DB records
!
      call dbgetnrec( db, numrecs )

      if (nmaxrec .gt. 0) numrecs = nmaxrec

      do nrec = 1, numrecs

         call dbrdentry( db, nrec, -1, interp, timestamp )
         call dbrdgeo( db, natm, nrec, -1, gpoint)
         if (lpesdb) call dbrdpes(db, nddstate, nrec, -1, v)
         if (lcpesdb) call dbrdcpes(db, nddstate, nrec, -1, cv)
         if (lgradb) call dbrdgra(db, nddstate, natm, nrec, -1, deriv1 )
         if (lhesdb) call dbrdhes(db, nddstate, natm, nrec, -1, deriv2)
         if (lapesdb) call dbrdapes(db, nddstate, nrec, -1, av)
         if (lagradb) call dbrdagra(db, nddstate, natm, nrec, -1, aderiv1)
         if (lahesdb) call dbrdahes(db, nddstate, natm, nrec, -1, aderiv2)
         if (ldipdb) call dbrddip( db, nrec, -1, dip )
         if (ladipdb) call dbrdadip( db, nrec, -1, adip )
         if (lmocodb) call dbrdmoco(db, nbasis, nrec, -1, mocoeff)
         if (lnactdb) call dbrdnact(db, natm, dercpdim, nrec, -1, nact)
         if (ltransdb) call dbrdtrans(db,nddstate, nrec, -1, diabtrans)
         if (lctransdb) call dbrdctrans(db,nddstate, nrec, -1, diabctrans)
         if (lnmdb) call dbrdnm( db, nmdof, nrec, -1, nmpoint )
         if (lcasocdb) call dbrdcasoc( db, nsocstate, nrec, -1, asoc )
         if (lsymdb) call dbrdsymrep( db, nsymop, nrec, -1, symrep )
         if (lcivecdb) call dbrdcivec( db, nsmult, imultblk, nrec, -1, &
            ciconfig,cicoeff)

! if cleaning DB do not write interpolated points
         if (lclean .and. interp(1) .eq. -1) continue

! if DB entry to be removed do not write 
         if (nrec .eq. nignore) continue

         if (.not. lbatch) write(6,'(a,i10)') 'Writing record : ',nrec

         if (lasciidb) then  ! ascii DB (default)
            write(igeodba,'(a,i15,a,i5)') '#Record No. : ',nrec,'   Status : ',interp
            joff = 0
            do i = 1, ndof1/3
               write(igeodba,'(3f20.12)') (gpoint(joff+j,1),j=1,3)
               joff = joff + 3
            enddo

            if (lpesdb) then
               write(ipesdba,'(a,i15)') '#Record No. : ',nrec
               write(ipesdba,'(4f20.12)')&
                  ((v(s1,s,1),s1=s,nddstate),s=1,nddstate)
            endif
            if (lcpesdb) then
               write(icpesdba,'(a,i15)') '#Record No. : ',nrec
               write(icpesdba,'(4f20.12)')&
                  ((cv(s1,s,1),s1=s,nddstate),s=1,nddstate)
            endif
            if (lgradb) then
               write(igradba,'(a,i15)') '#Record No. : ',nrec
               write(igradba,'(3f20.12)')&
                  (((deriv1(n,s1,s,1),n=1,ndof1),s1=s,nddstate),s=1,nddstate)
            endif
            if (lhesdb) then
               write(ihesdba,'(a,i15)') '#Record No. : ',nrec
               write(ihesdba,'(3f20.12)') &
                  ((((deriv2(n,n1,s1,s,1),n=1,ndof1),n1=1,ndof1),&
                             s1=s,nddstate),s=1,nddstate)
            endif
            if (lapesdb) then
               write(iapesdba,'(a,i15)') '#Record No. : ',nrec
               write(iapesdba,'(4f20.12)') (av(s,s,1),s=1,nddstate)
            endif
            if (lagradb) then
               write(iagradba,'(a,i15)') '#Record No. : ',nrec
               write(iagradba,'(3f20.12)') &
                  ((aderiv1(n,s,s,1),n=1,ndof1),s=1,nddstate)
            endif
            if (lahesdb) then
               write(iahesdba,'(a,i15)') '#Record No. : ',nrec
               write(iahesdba,'(3f20.12)') &
                  (((aderiv2(n,n1,s,s,1),n=1,ndof1),n1=1,ndof1),&
                             s=1,nddstate)
            endif
            if (ldipdb) then
               write(idipdba,'(a,i15)') '#Record No. : ',nrec
               write(idipdba,'(3f20.12)') &
                   ((dip(n,s,s,1),n=1,3),s=1,nddstate)
            endif
            if (ladipdb) then
               write(iadipdba,'(a,i15)') '#Record No. : ',nrec
               write(iadipdba,'(3f20.12)') &
                   ((adip(n,s,s,1),n=1,3),s=1,nddstate)
            endif
            if (lmocodb) then
               write(imocodba,'(a,i15)') '#Record No. : ',nrec
               write(imocodba,'(3f20.12)') &
                   ((mocoeff(n,s,1),n=1,nbasis),s=1,nbasis)
            endif
            if (lnactdb) then
               write(inactdba,'(a,i15)') '#Record No. : ',nrec
               do s = 1, dercpdim
                  joff = 0
                  do n=1,ndof1/3
                     write(inactdba,'(3f20.12)') (nact(j+joff,s,1),j=1,3)
                     joff = joff + 3
                  enddo
               enddo
            endif
            if (ltransdb) then
               write(itransdba,'(a,i15)') '#Record No. : ',nrec
               write(itransdba,'(2f20.12)') &
                   ((diabtrans(s,n,1),s=1,nddstate),n=1,nddstate)
            endif
            if (lctransdb) then
               write(ictransdba,'(a,i15)') '#Record No. : ',nrec
               write(ictransdba,'(2f20.12)') &
                   ((diabctrans(s,n,1),s=1,nddstate),n=1,nddstate)
            endif
            if(lnmdb)then
               write(inmdba,'(a,i15)') '#Record No. : ',nrec
               write(inmdba,'(4f20.12)') (nmpoint(i,1),i=1,nmdof)
            endif
            if(lcasocdb) then
               write(icasocdba,*) '#Record No. : ',nrec
               write(icasocdba,'(2f20.12)')&
                    ((asoc(s1,s,1),s1=s+1,nsocstate),s=1,nsocstate)
            endif
            if(lsymdb)then
               write(isymdba,*) '#Record No. : ',nrec
               write(isymdba,*) (symrep(j,1),j=1,nsymop)
            endif
            if(lcivecdb)then
               write(icidba,*) '#Record No. : ',nrec
               do ns=1,nsmult
                  write(icidba,*) '#Multiplicity : ',idxmult(ns)
                  do i=1,nciconf(ns)
                     write(icidba,'(a20,7f20.12)') ciconfig(i,ns,1),&
                              (cicoeff(i,s,ns,1),s=1,imultblk(ns))
                  enddo
               enddo
            endif

         else ! binary DB

            write(igeodb) interp,(gpoint(i,1),i=1,ndof1)
            if (lpesdb) write(ipesdb) ((v(s1,s,1),s1=s,nddstate),s=1,nddstate)
            if (lcpesdb) write(icpesdb) &
                         ((cv(s1,s,1),s1=s,nddstate),s=1,nddstate)
            if (lgradb) write(igradb) &
                       (((deriv1(n,s1,s,1),n=1,ndof1),s1=s,nddstate),&
                          s=1,nddstate)
            if (lhesdb) write(ihesdb) &
                        ((((deriv2(n,n1,s1,s,1),n=1,ndof1),n1=1,ndof1),&
                          s1=s,nddstate),s=1,nddstate)
            if (lapesdb) write(iapesdb) (av(s,s,1),s=1,nddstate)
            if (lagradb) write(iagradb) &
                         ((aderiv1(n,s,s,1),n=1,ndof1),s=1,nddstate)
            if (lahesdb) write(iahesdb) &
                         (((aderiv2(n,n1,s,s,1),n=1,ndof1),n1=1,ndof1),&
                           s=1,nddstate)
            if (ldipdb) write(idipdb) ((dip(n,s,s,1),n=1,3),s=1,nddstate)
            if (ladipdb) write(iadipdb) ((adip(n,s,s,1),n=1,3),s=1,nddstate)
            if (lmocodb) write(imocodb) &
                         ((mocoeff(n,s,1),n=1,nbasis),s=1,nbasis)
            if (ltransdb) write(itransdb) &
                          ((diabtrans(s,n,1),s=1,nddstate),n=1,nddstate)
            if (lctransdb) write(ictransdb) &
                          ((diabctrans(s,n,1),s=1,nddstate),n=1,nddstate)
            if (lnactdb) write(inactdb) ((nact(n,s,1),n=1,ndof1),s=1,dercpdim)
            if (lnmdb) write(inmdb) (nmpoint(i,1),i=1,nmdof)
            if (lcasocdb) write(icasocdb) &
                          ((asoc(s1,s,1),s1=s+1,nsocstate),s=1,nsocstate)
            if (lsymdb) write(isymdb) (symrep(n,1),n=1,nsymop)
            if(lcivecdb)then
               do ns=1,nsmult
                  do i=1,nciconf(ns)
                     write(icidba) ciconfig(i,ns,1),&
                              (cicoeff(i,s,ns,1),s=1,imultblk(ns))
                  enddo
               enddo
            endif

         endif

      enddo

!-----------------------------------------------------------------------
! keep new files
!-----------------------------------------------------------------------
      ascstat = 'keep'
      call closedb(ascstat)

!-----------------------------------------------------------------------
! close SQL DB and delete it
!-----------------------------------------------------------------------
      call dbclose(db)
      if (.not. lkeep) then
         open(isqldb, file=ddname(1:ddlaenge)//'/'//indbfile, status='old')
         close(isqldb, status='delete')
      endif

!-----------------------------------------------------------------------
! Stop timer
!-----------------------------------------------------------------------
      call cpu_time(tt2)
      write(6,'(/,a,f10.3,a)') 'Total time to convert DB  ',tt2-tt1,'s'

! Deallocation
      if (allocated(gpoint)) deallocate(gpoint)
      if (allocated(v)) deallocate(v)
      if (allocated(cv)) deallocate(cv)
      if (allocated(deriv1)) deallocate(deriv1)
      if (allocated(deriv2)) deallocate(deriv2)
      if (allocated(av)) deallocate(av)
      if (allocated(aderiv1)) deallocate(aderiv1)
      if (allocated(aderiv2)) deallocate(aderiv2)
      if (allocated(dip)) deallocate(dip)
      if (allocated(adip)) deallocate(adip)
      if (allocated(mocoeff)) deallocate(mocoeff)
      if (allocated(diabtrans)) deallocate(diabtrans)
      if (allocated(diabctrans)) deallocate(diabctrans)
      if (allocated(nact)) deallocate(nact)
      if (allocated(nmpoint)) deallocate(nmpoint)
      if (allocated(asoc)) deallocate(asoc)
      if (allocated(symrep)) deallocate(symrep)
      if (allocated(ciconfig)) deallocate(ciconfig)
      if (allocated(cicoeff)) deallocate(cicoeff)
      if (allocated(idxmult)) deallocate(idxmult)
      if (allocated(imultblk)) deallocate(imultblk)
      if (allocated(nciconf)) deallocate(nciconf)

      stop
 1000 write(6,'(a)') 'Error allocating memory in convdbtosql :',cerr
      write(6,'(a)') 'Error Code :',ierr
      stop

      end program convdbfromsql

