!#######################################################################
!
! Interconverts a direct-dynamics database between binary (DB) or
! ascii (DBA) to the new SQLite3 format.
!
!#######################################################################

      program convdbtosql

use decimal, only: sip
use global ! lsocbas
use versions
use sqlite
use db_sql
use dbtools

use dirdyn, only: natm,nroot,nbasis,dde0,dimref,nfam,mref,atnam,xref,&
                  dercpdim,lpesdb,lgradb,lhesdb,lapesdb,lagradb,&
                  lahesdb,ldipdb,ladipdb,lmocodb,ltransdb,lnactdb,lnmdb,&
                  lcasocdb,lcpesdb,dbpntgp,nsymop,dbsymmap,&
                  lsymdb,maxsymop,sqlvers,makeflagarray,makeinfoarray,&
                  lddsoc, qcmethod, qcprog, qcbasis
                  
      implicit none

      integer(long)     :: ierr,iend,nignore,nrdfiles,nmaxrec
      character(len=10) :: cerr
      integer(long)     :: nmdof
      integer(long), external :: myiargc
      integer(long)           :: nrec,s,s1,ndof1,n,n1,interp,i,iarg,&
                                 j,joff,rowid, timestamp,ns
      character(len=c1)       :: ascstat,filepos
      character(len=1)        :: atmp
      character(len=10)       :: buf
      character(len=c5)       :: outdbfile

      integer(long), dimension(:), allocatable    :: symrep
      real(dop), dimension(:), allocatable        :: gpoint,nmpoint
      real(dop), dimension(:,:), allocatable      :: v,diabtrans
      real(dop), dimension(:,:), allocatable      :: cv,diabctrans
      real(dop), dimension(:,:,:), allocatable    :: deriv1
      real(dop), dimension(:,:,:,:), allocatable  :: deriv2
      real(dop), dimension(:,:,:), allocatable    :: dip
      real(dop), dimension(:,:), allocatable      :: av
      real(dop), dimension(:,:,:), allocatable    :: aderiv1
      real(dop), dimension(:,:,:,:), allocatable  :: aderiv2
      real(dop), dimension(:,:,:), allocatable    :: adip
      real(sip), dimension(:,:), allocatable      :: mocoeff
      real(dop), dimension(:,:), allocatable      :: nact
      complex(dop), dimension(:,:), allocatable   :: casoc
      logical(kind=4)                             :: ascii,check,lbatch,&
                                                     lclean
      character(len=30), dimension(:,:), allocatable :: ciconfig
      real(dop), dimension(:,:,:), allocatable    :: cicoeff

      real(dop) :: tt1,tt2

      type(SQLITE_DATABASE)                       :: db

! defaults
! ascii: if true ascii -> SQL. false binary -> SQL
! lbatch: if true no output ot screen (batch job)
      call default
      ascii = .false.
      lbatch = .false.
      ddname = './'
      outdbfile = 'database.sql'
      ddtrfile = ' '

! default
      nbasis=-999
      nddstate=-999
      nignore = 0
      nmaxrec = 0

! read options
      iarg = myiargc()

      i=0
      do while (i .le. iarg)
         i=i+1
         call mygetarg(i,buf)
         if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else if ( buf .eq. '-i' ) then
            i=i+1
            call mygetarg(i,ddname)
         else if ( buf .eq. '-f' ) then
            i=i+1
            call mygetarg(i,outdbfile)
         else if ( buf .eq. '-batch' ) then
            lbatch = .true.
         else if ( buf .eq. '-c' ) then
            lclean = .true.
         else if ( buf .eq. '-r' ) then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*) nignore
         else if ( buf .eq. '-mx' ) then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*) nmaxrec
         else if ( buf .eq. '-dbver' ) then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*) sqlvers
         else if ( buf .eq. '-h' .or. buf(1:2) .eq. '-?') then
            write(6,'(/,78a1,/)')  ('*',i=1,78)
            write(6,'(a)') ' Analyse program "convdbtosql" '
            write(6,'(1x,a)')&
       'Purpose : Converts a flat file binary or ascii DB to SQL format'
            write(6,'(1x,2a)') 'Usage   : convdbtosql ',&
       '[-batch -i NAME -f NAME -c -ver -h -?] '
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(/,a)') ' Options : '
            write(6,'(a)') &
            ' -i NAME : The DB is read from directory NAME '
            write(6,'(a)') &
            ' -f NAME : The DB is written to file NAME '
            write(6,'(a)') &
            ' -c      : The DB is cleaned by removing points from failed calculations '
            write(6,'(a)') &
            ' -batch  : Batch mode - no output to the screen '
            write(6,'(a)') &
            ' -r nignore : The DB entry nignore is removed '
            write(6,'(a)') &
            ' -mx nmaxrec : Write only the first nmaxrec records to the DB '
            write(6,'(a)') &
            ' -dbver I : DB version use is I (=1,2) '
            write(6,'(a)')&
            ' -ver    : Version information about the program.'
            write(6,'(a,/a)') &
            ' -h      : Print this help text. ',&
            ' -?      : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if (buf(1:1) .eq. "-") then
            write(6,'(2a)') ' unknown option : ', buf
            stop

         endif
      end do

!
! get full path
!
      ddlaenge=index(ddname,' ')-1
      call abspath(ddname,ddlaenge)

!-----------------------------------------------------------------------
! Check if conversion needed
!-----------------------------------------------------------------------
      check=.false.
      inquire(file=ddname(1:ddlaenge)//'/'//outdbfile,exist=check)
      if (check) then
         Write(6,'(a)') 'SQL database exists. Nothing to convert'
         stop
      endif

!-----------------------------------------------------------------------
! Start timer
!-----------------------------------------------------------------------
      call cpu_time(tt1)

!-----------------------------------------------------------------------
! set up file names and see which DB type and files to be processed
!-----------------------------------------------------------------------
      call chkdb(ierr)
      if (ierr .eq. 1) stop
      ascii = lasciidb

!-----------------------------------------------------------------------
! open files
!-----------------------------------------------------------------------
! save lascii db and close both sets of files
      ascstat = 'old'
      filepos='rewind'
      if (ascii) then
         lasciidb=.true.
         call opendbold(ascstat,filepos)
      else
         lasciidb=.false.
         call opendbold(ascstat,filepos)
      endif

!
! get data for memory sizes from refdb.dat file
!
      call rdrefdbmemorig
      ndof1 = natm*3

      if (allocated(mref)) then
         deallocate(mref)
      endif
      allocate(mref(natm))
      if (allocated(atnam)) then
         deallocate(atnam)
      endif
      allocate(atnam(natm))
      if (allocated(xref)) then
         deallocate(xref)
      endif
      allocate(xref(3,natm))
      if (nsymop .gt. 0) then
         if (allocated(dbsymmap)) then
            deallocate(dbsymmap)
         endif
         allocate(dbsymmap(natm,nsymop))
      endif

!
! Read refdb.dat
!
      call rdrefdborig
      if (lnactdb .and. (dercpdim .le. 0)) &     ! dercpdim not always 
           dercpdim = nddstate*(nddstate-1)/2    ! in REFDB

! If CIvec to be read, multiplicity data is needed
      if (lcivecdb) then
         nsmult=count(nddmult .gt. 0)
         allocate(idxmult(nsmult))
         allocate(imultblk(nsmult))
         allocate(nciconf(nsmult))
         call rdcidbdat
         maxmultblk = maxval(imultblk)
         maxcivec = maxval(nciconf)
      endif

! check correct formatting will be performed
      if (.not. lbatch) then
         if (ascii) then
            write(6,'(a)') 'Converting from ascii to SQL'
            write(6,'(a)') 'Existing ascii DB files will be destroyed'
         else
            write(6,'(a)') 'Converting from binary to SQL'
            write(6,'(a)') 'Existing binary DB files will be destroyed'
         endif
         write(6,'(a)') 'Continue ? <y / n> '
         read(5,*) atmp
         if (atmp .eq. 'n' .or. atmp .eq. 'N') stop
      endif
!
! Open SQL database
!
! read-write, create
      !Needed for NEWSQL
      call makeflagarray
      call makeinfoarray(nddstate)
      call dbopen(ddname(1:ddlaenge)//'/'//outdbfile,db,sqlvers,quanticsver,&
           infoarray,flagarray, 2)
      call dbwrrefdb(db, natm, nroot, nddstate, nbasis, dde0, dimref, &
                     nfam, mref, atnam, xref, dercpdim, dbpntgp, &
                     nsymop, nsocstate, dbsymmap, nddmult, &
                     qcprog, qcmethod, qcbasis,ddtrfile,ddname,ddlaenge)

      if (lcivecdb) call dbwrcidat(db,nsmult,idxmult,imultblk,nciconf)
! 
! get sizes and assign arrays
!
      ndof1 = natm*3
      if (lnmdb)then
         if(natm.gt.2)then
            nmdof = ndof1-6
         else
            nmdof = ndof1-5
         endif
      endif

      maxdim = 3*natm+3
      maxsta = max(nddstate,nsocstate)
      if(dercpdim.le.0) dercpdim = nddstate*(nddstate-1)/2
      cerr = 'gpoint'
      allocate(gpoint(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'v'
      allocate(v(maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      if (lcpesdb) then
         cerr = 'cv'
         allocate(cv(maxsta,maxsta),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if(lgradb)then
         cerr = 'deriv1'
         allocate(deriv1(maxdim,maxsta,maxsta),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if(lhesdb)then
         cerr = 'deriv2'
         allocate(deriv2(maxdim,maxdim,maxsta,maxsta),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      cerr = 'av'
      allocate(av(maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'aderiv1'
      allocate(aderiv1(maxdim,maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'aderiv2'
      allocate(aderiv2(maxdim,maxdim,maxsta,maxsta),stat=ierr)
      if (ldipdb) then
         cerr = 'dip'
         allocate(dip(3,maxsta,maxsta),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'adip'
         allocate(adip(3,maxsta,maxsta),stat=ierr)
         if (ierr > 0) go to 1000
      else
         allocate(dip(1,1,1),stat=ierr)
      endif
      if(lnmdb)then
         cerr = 'nmpoint'
         allocate(nmpoint(maxdim),stat=ierr)
         if(ierr .gt.0) goto 1000
      endif
      if(lmocodb)then
         if(nbasis.lt.0)then
            write(6,'(a)')'Need # Nbasis entry in refdb'
            stop
         else
            allocate(mocoeff(nbasis,nbasis))
         endif
      else
         allocate(mocoeff(1,1))
      endif
      if(lnactdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(nact(ndof1,dercpdim))
         endif
      else
         allocate(nact(1,1))
      endif
      if(ltransdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(diabtrans(nddstate,nddstate))
         endif
      else
         allocate(diabtrans(1,1))
      endif
      if(lctransdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(diabctrans(nddstate,nddstate))
         endif
      else
         allocate(diabctrans(1,1))
      endif
      if(lcasocdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(casoc(nsocstate,nsocstate))
         endif
      else
         allocate(casoc(1,1))
      endif
      if (lsymdb) then
         cerr = 'symrep'
         allocate(symrep(maxsymop),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if (lcivecdb) then
         cerr = 'civec'
         allocate(ciconfig(maxcivec,nsmult),stat=ierr)
         allocate(cicoeff(maxcivec,maxmultblk,nsmult),stat=ierr)
         if (ierr > 0) go to 1000
      else
         allocate(ciconfig(1,1))
         allocate(cicoeff(1,1,1))
      endif

!-----------------------------------------------------------------------
! LOOP OVER RECORDS
! read and write DB records
! nrdfiles is a check that all files in DB are read and converted
!-----------------------------------------------------------------------
         nrec = 0
 rddata: do

         nrdfiles=0

         if (ascii) then

            read(igeodba,'(14x,i15,12x,i5)',end=200) nrec,interp
            if ((nmaxrec .gt. 0) .and. (nrec .gt. nmaxrec)) go to 200
            nrdfiles=nrdfiles+1
            joff = 0
            do i = 1, ndof1/3
               read(igeodba,'(3f20.12)',err=101) (gpoint(joff+j),j=1,3)
     101       continue
               joff = joff + 3
            enddo

            if (lpesdb) then
               read(ipesdba,*) 
               read(ipesdba,'(4f20.12)',err=102)&
                  ((v(s1,s),s1=s,nddstate),s=1,nddstate)
     102       continue
               nrdfiles=nrdfiles+1
            endif
            if (lcpesdb) then
               read(icpesdba,*) 
               read(icpesdba,'(4f20.12)',err=202)&
                  ((cv(s1,s),s1=s,nddstate),s=1,nddstate)
     202       continue
               nrdfiles=nrdfiles+1
            endif
            if (lgradb) then
               read(igradba,*) 
               read(igradba,'(3f20.12)',err=103)&
                  (((deriv1(n,s1,s),n=1,ndof1),s1=s,nddstate),s=1,nddstate)
     103       continue
               nrdfiles=nrdfiles+1
            endif
            if (lhesdb) then
               read(ihesdba,*) 
               read(ihesdba,'(3f20.12)',err=104)&
                  ((((deriv2(n,n1,s1,s),n=1,ndof1),n1=1,ndof1),&
                                s1=s,nddstate),s=1,nddstate)
     104       continue
               nrdfiles=nrdfiles+1
            endif
            if (lapesdb) then
               read(iapesdba,*) 
               read(iapesdba,'(4f20.12)',err=105) (av(s,s),s=1,nddstate)
     105       continue
               nrdfiles=nrdfiles+1
            endif
            if (lagradb) then
               read(iagradba,*) 
               read(iagradba,'(3f20.12)',err=106)&
                  ((aderiv1(n,s,s),n=1,ndof1),s=1,nddstate)
     106       continue
               nrdfiles=nrdfiles+1
            endif
            if (lahesdb) then
               read(iahesdba,*,iostat=iend) 
               read(iahesdba,'(3f20.12)',iostat=iend,err=107)&
                  (((aderiv2(n,n1,s,s),n=1,ndof1),n1=1,ndof1),s=1,nddstate)
     107       continue
! may only have 1 Hessian if doing diabatisation of data
               if ((iend .ne. 0) .and. (nrec .eq. 1)) then
                  write(6,'(a)') 'ERROR : adhes.dba file empty'
                  stop
               else if (iend .ne. 0) then
                  aderiv2=0.0
               endif
               nrdfiles=nrdfiles+1
            endif
            if (ldipdb) then
               read(idipdba,*) 
               read(idipdba,'(3f20.12)',err=108)&
                  ((dip(n,s,s),n=1,3),s=1,nddstate)
     108       continue
               nrdfiles=nrdfiles+1
            endif
            if (ladipdb) then
               read(iadipdba,*) 
               read(iadipdba,'(3f20.12)',err=109)&
                  ((adip(n,s,s),n=1,3),s=1,nddstate)
     109       continue
               nrdfiles=nrdfiles+1
            endif
            if(lmocodb) then
               read(imocodba,*) 
               read(imocodba,'(3f20.12)',err=110)&
                  ((mocoeff(n,s),n=1,nbasis),s=1,nbasis)
     110       continue
               nrdfiles=nrdfiles+1
            endif
            if(lnactdb) then
               read(inactdba,*)
               do s = 1,dercpdim
                  joff = 0
                  do n = 1, ndof1/3
                     read(inactdba,'(3f20.12)',err=111) (nact(j+joff,s),j=1,3)
     111       continue
                     joff = joff + 3
                  enddo
               enddo
               nrdfiles=nrdfiles+1
            endif
            if(ltransdb) then
               read(itransdba,*) 
               read(itransdba,'(2f20.12)',err=112)&
                  ((diabtrans(s,n),s=1,nddstate),n=1,nddstate)
     112       continue
               nrdfiles=nrdfiles+1
            endif
            if(lctransdb) then
               read(ictransdba,*) 
               read(ictransdba,'(2f20.12)',err=212)&
                  ((diabctrans(s,n),s=1,nddstate),n=1,nddstate)
     212       continue
               nrdfiles=nrdfiles+1
            endif
            if(lnmdb)then
               read(inmdba,*)
               read(inmdba,'(4f20.12)',err=113) (nmpoint(j),j=1,nmdof)
     113       continue
               nrdfiles=nrdfiles+1
            endif
            if(lcasocdb)then
               read(icasocdba,*) 
               read(icasocdba,'(2f20.12)',err=115)&
                    ((casoc(s1,s),s1=s+1,nsocstate),s=1,nsocstate-1)
               do s = 1,nddstate-1
                  do s1 = s+1,nddstate
                     casoc(s,s1) = dconjg(casoc(s1,s))
                  enddo
               enddo
     115       continue
               nrdfiles=nrdfiles+1
            endif
            if (lsymdb) then
               read(isymdba,*)
               read(isymdba,*,err=118) (symrep(n),n=1,nsymop)
     118       continue
               nrdfiles=nrdfiles+1
            endif
            if(lcivecdb)then
               read(icidba,*) 
               do ns=1,nsmult
                  read(icidba,*) 
                  do i=1,nciconf(ns)
                     read(icidba,*) ciconfig(i,ns),&
                              (cicoeff(i,s,ns),s=1,imultblk(ns))
                  enddo
               enddo
               nrdfiles=nrdfiles+1
            endif

         else

            read(igeodb,end = 200) interp,(gpoint(i),i=1,ndof1)
            nrdfiles=nrdfiles+1
            nrec = nrec+1
            if ((nmaxrec .gt. 0) .and. (nrec .gt. nmaxrec)) go to 200
            if (lpesdb) then
               read(ipesdb) ((v(s1,s),s1=s,nddstate),s=1,nddstate)
               nrdfiles=nrdfiles+1
            endif
            if (lcpesdb) then
               read(icpesdb) ((cv(s1,s),s1=s,nddstate),s=1,nddstate)
               nrdfiles=nrdfiles+1
            endif
            if (lgradb) then
               read(igradb) (((deriv1(n,s1,s),n=1,ndof1),s1=s,nddstate),&
                             s=1,nddstate)
               nrdfiles=nrdfiles+1
            endif
            if (lhesdb) then
               read(ihesdb) ((((deriv2(n,n1,s1,s),n=1,ndof1),n1=1,ndof1),&
                             s1=s,nddstate),s=1,nddstate)
               nrdfiles=nrdfiles+1
            endif
            if (lapesdb) then
               read(iapesdb) (av(s,s),s=1,nddstate)
               nrdfiles=nrdfiles+1
            endif
            if (lagradb) then
               read(iagradb) ((aderiv1(n,s,s),n=1,ndof1),s=1,nddstate)
               nrdfiles=nrdfiles+1
            endif
            if (lahesdb) then
               read(iahesdb) (((aderiv2(n,n1,s,s),n=1,ndof1),n1=1,ndof1),&
                             s=1,nddstate)
               nrdfiles=nrdfiles+1
            endif
            if (ldipdb) then
               read(idipdb) ((dip(n,s,s),n=1,3),s=1,nddstate)
               nrdfiles=nrdfiles+1
            endif
            if (ladipdb) then
               read(iadipdb) ((adip(n,s,s),n=1,3),s=1,nddstate)
               nrdfiles=nrdfiles+1
            endif
            if (lmocodb) then
               read(imocodb) ((mocoeff(n,s),n=1,nbasis),s=1,nbasis)
               nrdfiles=nrdfiles+1
            else
               mocoeff=0.0_dop
            endif
            if (lnactdb) then
               read(inactdb) ((nact(n,s),n=1,ndof1),s=1,dercpdim)
               nrdfiles=nrdfiles+1
            else
               nact=0.0_dop
            endif
            if (ltransdb) then
               read(itransdb) ((diabtrans(s,n),s=1,nddstate),n=1,nddstate)
               nrdfiles=nrdfiles+1
            else
               diabtrans=0.0_dop
            endif
            if (lctransdb) then
               read(ictransdb) ((diabctrans(s,n),s=1,nddstate),n=1,nddstate)
               nrdfiles=nrdfiles+1
            else
               diabctrans=0.0_dop
            endif
            if (lnmdb) then
               read(inmdb) (nmpoint(i),i=1,nmdof)
               nrdfiles=nrdfiles+1
            endif
            if (lcasocdb) then
               read(icasocdb) ((casoc(s1,s),s1=s+1,nsocstate),s=1,nsocstate)
               do s = 1,nsocstate-1
                  do s1 = s+1,nsocstate
                     casoc(s,s1) = dconjg(casoc(s1,s))
                  enddo
               enddo
               nrdfiles=nrdfiles+1
            endif
            if (lsymdb) then
               read(isymdb) (symrep(n),n=1,nsymop)
               nrdfiles=nrdfiles+1
            endif
            if(lcivecdb)then
               read(icidba) 
               do ns=1,nsmult
                  read(icidba) 
                  do i=1,nciconf(ns)
                     read(icidba) ciconfig(i,ns),&
                              (cicoeff(i,s,ns),s=1,imultblk(ns))
                  enddo
               enddo
               nrdfiles=nrdfiles+1
            endif

         endif

! check all files read
         if (nrdfiles .ne. ndbfiles) then
            write(6,'(2(a,i5))') 'ERROR: Not all DB files read. ',&
                 nrdfiles,' of ',ndbfiles
            stop
         endif

! if cleaning DB do not write interpolated points
         if (lclean .and. interp .eq. -1) cycle

! if DB entry to be removed do not write 
         if (nrec .eq. nignore) cycle

! write record to SQL database
         if (.not. lbatch) write(6,'(a,i10)') 'Writing record : ',nrec

         ! START TRANSACTION
         call dbstarttrans ( db )
         ! write entry and get new rowid
         rowid = 0
         call dbwrentry(db, rowid, interp, timestamp)
         call dbwrgeo( db, rowid, natm, gpoint )
         if (lpesdb) call dbwrpes( db, rowid, nddstate, v )
         if (lcpesdb) call dbwrcpes( db, rowid, nddstate, cv )
         if (lgradb) call dbwrgra( db, rowid, natm, nddstate, deriv1 )
         if (lhesdb) call dbwrhes( db, rowid, natm, nddstate, deriv2 )
         if (lapesdb) call dbwrapes( db, rowid, nddstate, av )
         if (lagradb) call dbwragra( db, rowid, natm, nddstate, aderiv1 )
         if (lahesdb) call dbwrahes( db, rowid, natm, nddstate, aderiv2 )
         if (ldipdb) call dbwrdip( db, rowid, nddstate, dip )
         if (ladipdb) call dbwradip( db, rowid, nddstate, adip )
         if (lmocodb) call dbwrmoco( db, rowid, nbasis, mocoeff )
         if (lnactdb) call dbwrnact( db, rowid, natm, nddstate, nact )
         if (ltransdb) call dbwrtrans( db, rowid, nddstate, diabtrans )
         if (lctransdb) call dbwrctrans( db, rowid, nddstate, diabctrans )
         if (lnmdb) call dbwrnm( db, rowid, nmdof, nmpoint )
         if (lcasocdb) call dbwrcasoc( db, rowid, nsocstate, casoc )
         if (lsymdb) call dbwrsymrep( db, rowid, nsymop, symrep )
         if (lcivecdb) call dbwrcivec( db, rowid, nsmult, maxcivec, &
                            maxmultblk, nciconf, imultblk, ciconfig, cicoeff)

         ! COMMIT TRANSACTION
         call dbcommittrans ( db )
      enddo rddata

  200 continue ! exit loop

!-----------------------------------------------------------------------
! delete old DB and keep new files
!-----------------------------------------------------------------------
      if (.not. lbatch) write(6,'(a)') 'Deleting old DB files '

      ascstat = 'delete'
      call closedb(ascstat)

!-----------------------------------------------------------------------
! close sql DB
!-----------------------------------------------------------------------
      call dbclose(db)

!-----------------------------------------------------------------------
! Stop timer
!-----------------------------------------------------------------------
      call cpu_time(tt2)
      write(6,'(/,a,f10.3,a)') 'Total time to convert DB  ',tt2-tt1,'s'

! Deallocation
      if (allocated(gpoint)) deallocate(gpoint)
      if (allocated(v)) deallocate(v)
      if (allocated(deriv1)) deallocate(deriv1)
      if (allocated(deriv2)) deallocate(deriv2)
      if (allocated(av)) deallocate(av)
      if (allocated(aderiv1)) deallocate(aderiv1)
      if (allocated(aderiv2)) deallocate(aderiv2)
      if (allocated(dip)) deallocate(dip)
      if (allocated(adip)) deallocate(adip)
      if (allocated(mocoeff)) deallocate(mocoeff)
      if (allocated(diabtrans)) deallocate(diabtrans)
      if (allocated(diabctrans)) deallocate(diabctrans)
      if (allocated(nact)) deallocate(nact)
      if (allocated(nmpoint)) deallocate(nmpoint)
      if (allocated(casoc)) deallocate(casoc)
      if (allocated(symrep)) deallocate(symrep)
      if (allocated(ciconfig)) deallocate(ciconfig)
      if (allocated(cicoeff)) deallocate(cicoeff)
      if (allocated(idxmult)) deallocate(idxmult)
      if (allocated(imultblk)) deallocate(imultblk)
      if (allocated(nciconf)) deallocate(nciconf)

      stop
 1000 write(6,'(a)') 'Error allocating memory in convdbtosql :',cerr
      write(6,'(a)') 'Error Code :',ierr
      stop

      end program convdbtosql
! vi: set sw=3 ts=3:
