!***********************************************************************
!
!                       CROSSPEC
!
!  Computes the spectrum from the cross-correlation function 
! 
!  (HDM 03/01)
!
!***********************************************************************

      program crosspec

use decimal, only:dop,long
use keyunitsmod
use global
use aglobal
use versions

      implicit none

      integer(long), external              :: slen
      integer(long)                        :: maxtp
      integer(long)                        :: i,j,icross,epoints,ierr
      real(dop)                            :: fac, eau, emina, emaxa,&
                                              a,b,t,t0,dt,delt,pia,tcut,&
                                              e,emin,emax,eoff,dele,phcorr

      complex(dop)                         :: sum,cc,cr0
      complex(dop), dimension(:), allocatable :: cross
      logical(kind=4)                      :: lsin,lcos,ldc,lold,lresol
      character(len=80)                    :: buffer
      character(len=200)                   :: gnucmd
      character(len=16)                    :: unit
      character(len=8)                     :: lab

!-----------------------------------------------------------------------
!     Set defaults. Set up the input.
!-----------------------------------------------------------------------
      epoints = 1000
      call setunits
      call cspecinp(emin,emax,unit,eoff,epoints,tcut,lsin,lcos,ldc,&
                    lold,lresol,phcorr)

      write(6,'(a,2g10.4E1,1x,a)') ' Emin, Emax : ',emin,emax,unit 
      write(6,'(a,f9.6,a,f8.3,a)') &
           ' E-offset   : ', eoff,' au,   ', eoff*ev,' eV'
      write(6,'(a,i6)') ' No. of plot-points: ', epoints 

!-----------------------------------------------------------------------
!     Determine the number of t-points on cross-file..
!-----------------------------------------------------------------------
      maxtp=1
 3    read(20,*,end=1,err=32) buffer
      maxtp=maxtp+1
      goto 3
 1    continue
      rewind 20
      allocate(cross(maxtp))

!-----------------------------------------------------------------------
!     Read the crosscorrelation function.
!-----------------------------------------------------------------------
      dt = fs
      i=0
      if(lresol .and. filein.eq.' ') goto 37
 25   read(20,900,end=31,err=32) buffer
      if( buffer(1:9) .ne. '#    time' ) goto 25
      read(20,*,end=31,err=32) t0, a, b
      cr0 = dcmplx(a,b)
      read(20,*,end=31,err=32) t, a, b
      cross(1) = dcmplx(a,b)*exp(dcmplx(0.d0,phcorr*t*t))
      dt = (t-t0)*fs
      tcut = tcut - 0.99*(t-t0)

      icross = 1
 34   read(20,*,end=37,err=33) t, a, b
      icross = icross+1
      if(icross .gt. maxtp) goto 37
      cross(icross) = dcmplx(a,b)*exp(dcmplx(0.d0,phcorr*t*t))
      if( t .gt. tcut ) goto 37
      goto 34

 33   backspace 20            ! Check if the unreadable line starts
      read(20,'(a)') buffer   ! with a '#' or with a blank.
      if( buffer(1:1) .eq. '#' ) goto 34
      if( buffer      .eq. ' ' ) goto 34
      goto 32                 ! Goto error message

 37   if(lresol) then
         if(i.le.1) i=maxtp 
         cr0 = 1.0_dop
         do j = 1, i
            cross(j) = 1.0_dop*exp(dcmplx(0.0_dop,phcorr*t*t))
            icross = j
            t = j*dt/fs
            if( t .gt. tcut ) goto 30
         enddo
      endif
!-----------------------------------------------------------------------
!

 30   continue
!.....fs is set to 1.0 if unit='no'. Otherweise fs = 41.3414
      if( fs .gt. 40.0_dop ) then
         write(6,'(a,i6,a,f10.3,a)') &
          ' No. of t-points =', icross,',   t-final =', t,' fs'
      else
         write(6,'(a,i6,a,f10.3,a)') &
          ' No. of t-points =', icross,',   t-final =', t,' (no units)'
      endif

!-----------------------------------------------------------------------
! Remove the DC component (-dc option)
!-----------------------------------------------------------------------
      if(ldc) then
         sum=cr0
         do i = 1, icross
            sum = sum + cross(i)
         enddo
         sum=sum/dble(icross+1)
         do i = 1, icross
            cross(i) = cross(i) - sum
         enddo
         write(6,'(a,1p2e12.4)') ' DC component: ', sum
      endif

!-----------------------------------------------------------------------
! calculate fourier transform / spectrum
!-----------------------------------------------------------------------
      pia   = pi/dble(2*icross+2)
      dele  = (emax-emin)/dble(epoints)
      delt  = abs(dt)/(2.0_dop*pi)
      emaxa = emax
      emina = emin
      call keylowc(unit)
      call keyeunits(emaxa,unit,ierr)
      call keyeunits(emina,unit,ierr)
      if(ierr .gt. 0) then
         write(6,*)' unknown unit: ', unit
         stop
      end if
      fac  = abs(emaxa-emina)*delt
      if(fac .ge. 1.0_dop) write(6,800) 1.0_dop/delt, ev/delt, fac 
      
      if(fs.gt.40.0_dop) then
         if(lold) then
            write(22,930) unit
         else
            write(22,931) unit
         endif 
      else
         if(lold) then
            write(22,930) 'no unit'
         else
            write(22,931) 'no unit'
         endif
      endif
      
      do j = 0, epoints
         e = emin + j*dele
         eau = e
         call keyeunits(eau,unit,ierr)
         eau = eau - eoff
         sum = 0.5_dop*cr0
         if(lsin)  sum = sum*sin(pia)
         do i = 1, icross
            t = i*dt
            cc   =  exp(dcmplx(0.0_dop,eau*t))*cross(i) 
            if(lsin)  cc = cc*sin(pia*(2*i+1))
            if(lcos)  cc = cc*cos(pia*i)
            sum = sum + cc
         end do
         sum = sum - 0.5_dop*cc
         sum = sum*dt           ! atuospec:  sum = sum*dt/pi
         eau = eau + eoff
         if(lold) then 
            write(22,920) e, abs(sum), dble(sum), dimag(sum)
         else
            write(22,920) e,dble(sum)/pi,dimag(sum)/pi,abs(sum)/pi
         endif
      end do


!-----------------------------------------------------------------------
!          call gnuplot     
!-----------------------------------------------------------------------
      if(lagnu) then
         write(22,'(/a/)') '#   END '
         rewind 22              ! This is to ensure that the file is
  60     read(22,'(a8)') lab    ! written before read by gnuplot.
         if( lab .ne. '#   END ') goto 60 
!!   This may cause problems when running on Mac
!!         gnucmd = 'gnuplot -geometry +0+0 -persist 2>/dev/null "'&
         gnucmd = 'gnuplot -persist 2>/dev/null "'&
          //fileout(1:slen(fileout))//'"'
         call excmd(gnucmd)
      endif

      stop


 800  format(' Warning: Energy interval too large for time-',&
           'resolution.',/,' max-del-E =', 1pe11.3,' au, ',&
           1pe11.3,' eV',/,' You will see the spectrum',&
           0pf7.2,' times'/)
 900  format(a80)
 920  format(4(1x,1pe17.8))
 930  format('#    ',a16,/,'#',8x,'Energy',11x,'Abs(FT)',12x,&
             'Re(FT)',12x,'Im(FT)')
 931  format('#    ',a16,/,'#',8x,'Energy',11x,'Re(FT)',12x,&
             'Im(FT)',12x,'Abs(FT)')

 31   write(6,'(a)') ' No data. Check the input (cross) file.'
      stop
 32   write(6,'(/a,/a,/a)') &
         ' Error occurred when reading the input (cross) file.', &
         ' The wrong line reads : ',trim(buffer)
      stop
      contains
      
!#######################################################################

      subroutine cspecinp(emin,emax,unit,eoff,epoints,tcut,&
                          lsin,lcos,ldc,lold,lresol,phcorr) 

      implicit none

      integer(long), external        :: myiargc, slen
      integer(long)                  :: i,iarg,ierr
      integer(long), intent(out)     :: epoints
      real(dop), intent(out)         :: emin,emax,eoff,tcut,phcorr
      character(len=80)              :: buf
      character(len=16)              :: unite, label
      character(len=16), intent(out) :: unit
      logical(kind=4)                :: lnorm,lall,lreim,lre,lim
      logical(kind=4), intent(out)   :: lsin,lcos,ldc,lold,lresol
!-----------------------------------------------------------------------

      filein = '  '
      name   = './'
      fileout = '  '

      label=','
      eoff = 0.0_dop
      tcut = 9.9e+49_dop
      phcorr=0.0_dop
      unit = 'au'
      lsin = .false.
      lcos = .false.
      lnorm= .false.
      lgnu = .false.
      lagnu= .false.
      ldc  = .false.
      lreim= .false.
      lre  = .false.
      lim  = .false.
      lold = .false.
      lall = .false.
      lresol = .false.


      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf(1:3) .eq. '-n ' ) then
            lnorm = .true.
         else if( buf(1:3) .eq. '-t ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) tcut
            if (ierr .ne. 0) then
               write(6,*)'Argument for -t must be real number. Not :'
               write(6,*) buf
               stop
            end if
         else if( buf .eq. '-g' ) then 
            lgnu = .true.
         else if( buf .eq. '-reim' ) then
            lreim = .true.
         else if( buf .eq. '-re' ) then
            lre = .true.
         else if( buf .eq. '-im' ) then
            lim = .true.
         else if( buf .eq. '-all' ) then
            lall = .true.
         else if( buf .eq. '-a' ) then
            lgnu  = .true.
            lagnu = .true.
         else if( buf .eq. '-sin' ) then
            lsin = .true.
            label=label(1:slen(label))//' sin'
         else if( buf .eq. '-cos' ) then
            lcos = .true.
            label=label(1:slen(label))//' cos'
         else if( buf .eq. '-nofs' ) then
            fs = 1.d0
         else if( buf(1:3) .eq. '-G ' ) then
            lpgrid = .true.
         else if( buf .eq. '-old' ) then
            lold = .true.
            label=label(1:slen(label))//' old'
         else if( buf .eq. '-dc' ) then
            ldc = .true.
            label=label(1:slen(label))//' dc'
         else if( buf .eq. '-r' ) then
            lresol=.true.
         else if( buf(1:4) .eq. '-ph ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) phcorr
            if (ierr .ne. 0) then
               write(6,*)'Argument for -ph must be real number. Not :'
               write(6,*) buf
               stop
            end if
         else if( buf .eq. '-p' .or. buf .eq. '-q') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) epoints
            if (ierr .ne. 0) then
               write(6,*)'Argument for -p must be real number. Not :'
               write(6,*) buf
               stop
            end if
         else if( buf(1:2) .eq. '-e' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) eoff
            if (ierr .ne. 0) then
               write(6,*)'Argument for -e must be real number. Not :'
               write(6,*) buf
               stop
            end if
            i = i+1
            call mygetarg(i,unite)
            call keylowc(unite)
            call keyeunits(eoff,unite,ierr)
            if(ierr .gt. 0) then
               write(6,*)' unknown eoff-unit: ', unite
               stop
            end if
            unite = 'au'
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(/,a/,10x,a,/10x,a,/10x,a,/10x,a)')&
       'Purpose: Computation of the spectrum by fourier-transforming',&
       'the cross-correlation function.',&
       'Output is: Energy, Re(FT), Im(FT), Abs(FT),'&
       ,'where FT denotes the Fourier-transform (prefactor Pi^-1)' &
       ,'of the cross-correlation function.'

            write(6,'(/,2a,/19x,a)')&
            'Usage : crosspec [-o -e -p -q -a -g -all -reim',&
            ' -re -im -sin -cos -dc -t',&
            ' -nofs -G -ph -r -old -h ] [emin emax unit file].'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a/,9x,a)') &
               ' -o FILE : The spectrum is written to',&
               ' file FILE.pl','  rather than to ./cspec.pl',&
               '  The string FILE may be a relative or full path-name.'
            write(6,'(2a)') &
               ' -e R Unit : the offset energy is set to R in ',&
               'units Unit; e.g. 0.3 ev'
            write(6,'(2a,i5/,9x,a)') &
               ' -p I    : The no. of plot-points is I rather',&
               ' than', epoints,&
               '  Use "-p 2000" for final plots'
            write(6,'(a)') ' -q I    : Same as -p I'
            write(6,'(2a)') &
               ' -g      : GNUplot command lines are written to ',&
               'the output file.'
            write(6,'(2a)') &
                ' -a      : GNUplot is called automatically.',&
                ' (sets "-g").'
            write(6,'(2a,/11x,a)')&
                ' -all    : Re, Im and Abs parts (rather than Abs)',&
                ' are plotted.','(ignored if neither -g or -a are set).'
            write(6,'(2a,/11x,a)')&
                ' -reim   : Real and Imaginary parts (rather than Abs)',&
                ' are plotted.','(ignored if neither -g or -a are set).'
            write(6,'(a,/11x,a)')&
                ' -re     : Real part (rather than Abs) is plotted.',&
                '(ignored if neither -g or -a are set).'
            write(6,'(a,/11x,a)')&
                ' -im     : Imag. part (rather than Abs) is plotted.',&
                '(ignored if neither -g or -a are set).'
           write(6,'(2a)')&
               ' -sin    : The crosscorrelation function is multiplied',&
               ' with sin(pi*t/T).'
           write(6,'(2a)')&
               ' -cos    : The crosscorrelation function is multiplied',&
               ' with cos(pi*t/2T).'
           write(6,'(2a)')&
               ' -dc     : A DC component is subtracted from the ',&
               'cross-corr before FT.'
           write(6,'(a)')&
               ' -r      : Plot the resolution rather than the spectrum'
            write(6,'(a)')&
               ' -ph  ph : Use phase correction exp(i*ph*1.0e-6_dop*t^2)'
           write(6,'(2a)') &
               ' -t tcut : The crosscorrelation function is read only',&
               ' up to t=tcut'
            write(6,'(2a)') &
               ' -nofs   : No transformation to fs. Use when',&
               ' "time-not-fs" was set in mctdh.'
           write(6,'(2a)') &
               ' -G      : Plot gridlines',&
               ' (meaningfull only if -g or -a is set).'
           write(6,'(a,/11x,a)')&
               ' -old    : The old output format is used.',&
               '(Different order of colums, output times Pi.)'
            write(6,'(a)')&
               ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
               ' -h      : Print this help text. ',&
               ' -?      : Print this help text. '
            write(6,'(2a,/,11x,a/,a/,2a/,2a/,2a)') &
               'Following the options one may provide the ',&
               'arguments (separated by blank):',&
               'Emin, Emax, Unit file',&
               'where file denotes the path of the crosscorr file.',&
               'If not enough arguments are given, the user is ',&
               'prompted for the missing ','arguments. ', &
               "Use the Unit 'no' if the propagation is run with the", &
               "keywords: 'time-not-fs','energy-not-ev'." 
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            if(ichar(buf(2:2)).ge.46 .and. ichar(buf(2:2)).le.57) &
                 goto 111     ! Found a negative number, not an option.
            write(6,*) 'unknown option : ', buf(1:50)
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      
 111  read(buf,*,iostat=ierr) emin
      if (ierr .ne. 0 .or. abs(emin) .gt. 1.0e55_dop) then
         write(6,*)&
         'The first argument (emin)  must be real number. Not :'
         write(6,*) buf(1:60)
         stop
      end if

      i=i+1
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) emax
      if (ierr .ne. 0) then
         write(6,*)&
         'The second argument (emax)  must be real number. Not :'
         write(6,*) buf
         stop
      end if

      i=i+1
      if (iarg .lt. i) goto 300
      call mygetarg(i,buf)
      unit = trim(buf)
!.....unit = 'no' is to be used, when the keywords 'time-not-fs' and
!.....'energy-not-ev' are set during the propagation. 
      if(unit .eq. 'no' ) then
         unit = 'au'
         fs = 1.0_dop
      endif

      i=i+1
      if (iarg .lt. i) goto 300
      call mygetarg(i,buf)
      filein=buf

      goto 500

!-----------------------------------------------------------------------
!  Prompting for missing input
!-----------------------------------------------------------------------
 200  write(6,'(a)') ' E-min, E-max, unit =? (e.g:  2., 5., ev)'
      read(5,*) emin, emax, unit

 300  if(lresol .and. tcut.le.1.0e+49_dop) then
         filein='-----'
      else 
         write(6,'(a)') ' cross-corr-file = ? '
         read(5,*,err=500,end=500) filein
      endif

 500  continue

!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
! Otherwise take name from filein
!-----------------------------------------------------------------------
      laein=index(filein,' ')-1
      call abspath(filein,laein)
      
      i=laein
 10   continue
      if (filein(i:i) .eq. '/') then
         laenge=i-1
      else
         i=i-1
         go to 10
      endif
      name=filein(1:laenge)

      open(20,file=filein(1:laein),form='formatted',&
              status='old',err=98)
      if(laein .le. 58 ) then
      write(6,'(2a)') ' The  inputfile is : ',filein(1:laein)
      else
      write(6,'(a)') ' The inputfile is : '
      write(6,'(a)')   filein(1:laein)
      end if

 95   if (fileout .eq. ' ') then
         fileout='cspec.pl'
         laeout=index(fileout,' ')-1
      else
         laeout=index(fileout,' ')-1
      endif
      if (fileout(laeout:laeout) .eq. '/') then 
         fileout = fileout(1:laeout)//'cspec.pl'
         laeout  = laeout + 11
      else if( fileout(laeout-2:laeout) .ne. '.pl' ) then
         fileout = fileout(1:laeout)//'.pl'
         laeout  = laeout + 3
      end if
      call abspath(fileout,laeout)
      if (.not. linter) then
         open(22,file=fileout(1:laeout),form='formatted',&
              status='unknown',err=99)
         if(laeout .lt. 55 ) then
            write(6,'(2a)') ' The outputfile is : ',fileout(1:laeout)
         else
            write(6,'(a)') ' The outputfile is : '
            write(6,'(a)')   fileout(1:laeout)
         end if
         if(lgnu) write(6,'(a,i2)') &
            ' GNUPLOT command lines written ' 
      endif

      if( lgnu ) then
         i = index(unit,' ') - 1
         write(22,'(a)') 'set style data lines' 
         write(22,'(a)') 'set nologscale' 
         write(22,'(a)') 'set xzeroaxis' 
         if(lpgrid) write(22,'(a)') 'set grid'
         write(22,'(a,e14.7,a,e14.7,a)') 'set xr [',emin,':',emax,']' 
!........fs is set to 1.0 if unit='no'. Otherweise fs = 41.3414
         if(fs.gt.40.0_dop) then
            write(22,'(3a)') 'set xlabel "Energy[',unit(1:i),']"'
         else
            write(22,'(3a)') 'set xlabel "Energy"'
         endif
         write(22,'(4a)') &
                 'set title "C-Spectrum, ', filein(1:laein-4),&
                 label(1:slen(label)),'"' 
         if(lold) then
            if(lre) then
               write(22,'(3a)') 'plot "',fileout(1:laeout),&
                                '" us 1:3  tit "Re"'
            elseif(lreim) then
               write(22,'(3a)') 'plot "',fileout(1:laeout),&
                   '" us 1:3  tit "Re", "-" us 1:4 tit "Im"'
            elseif(lall) then
               write(22,'(3a,a1,/3a,a1,/3a)')&
               'plot "',fileout(1:laeout),'" us 1:3 tit "Re" ','\\',&
                  ', "',fileout(1:laeout),'" us 1:4 tit "Im" ','\\',&
                  ', "',fileout(1:laeout),'" us 1:2 tit "Abs" '
            elseif(lall) then
               write(22,'(4a)') 'plot "',fileout(1:laeout),&
               '" us 1:3  tit "Re", "-" us 1:4 tit "Im",',&
               ' "-" us 1:2 tit "Abs"'
            else
               write(22,'(3a)') 'plot "',fileout(1:laeout),&
                                '" us 1:2 tit "Abs"'
            endif
         else
            if(lre) then
               write(22,'(3a)') 'plot "',fileout(1:laeout),&
                                '" us 1:2  tit "Re"'
            elseif(lim) then
               write(22,'(3a)') 'plot "',fileout(1:laeout),&
                                '" us 1:3  tit "Im"'
            elseif(lreim) then
               write(22,'(3a)') 'plot "',fileout(1:laeout),&
               '" us 1:2  tit "Re", "-" us 1:3 tit "Im"'
            elseif(lall) then
               write(22,'(3a,a1,/3a,a1,/3a)')&
               'plot "',fileout(1:laeout),'" us 1:2 tit "Re" ','\\',&
                  ', "',fileout(1:laeout),'" us 1:3 tit "Im" ','\\',&
                  ', "',fileout(1:laeout),'" us 1:4 tit "Abs" '
            else
               write(22,'(3a)') 'plot "',fileout(1:laeout),&
                                '" us 1:4 tit "Abs"'
            endif

         endif
         write(22,'(a)') ' ' 
      end if
      phcorr = phcorr*1.0e-6_dop

      return 
      
 98   if(lresol .and. tcut.lt.1.0e+49_dop) then
         filein='     '
         laein=5
         goto 95
      endif
      write(6,*)' Cannot open input (cross) file : '
      write(6,*) filein
      stop
      
 99   write(6,*)' Cannot open output (cspec) file : '
      write(6,*) fileout
      stop

      end subroutine cspecinp
 
      end program crosspec
