!***********************************************************************
!                                                                     
!              Quantics module   DBTOOLS
!
! Module containing routines for analysing a DB. In particular, routines
! for using the flat file structure DBs, both binary and ascii.
!
!***********************************************************************

      module dbtools

      use decimal, only : long, dop
      use global, only : c1, c2, c3, c4,c5
      use global, only : ddname, ddlaenge
      use versions, only: progver
      use channels, only: ilog
      use logdat
      
      use griddatmod, only: nddstate
      use dirdyn
      use readqc, only: getinam, getmass
      use symoplib, only: fnsymop

      implicit none

!-----------------------------------------------------------------------
! flags to control DB files
!
! ndbfiles : no. of files in DB (not including refdb.dat)
!
!-----------------------------------------------------------------------
      integer(long), save   :: ndbfiles = 0
      character(len=c5)     :: refdbfile,cidbfile,&
                               geofile,pesfile,grafile,hesfile,&
                               geofilea,pesfilea,grafilea,hesfilea,&
                               apesfile,agrafile,ahesfile,&
                               apesfilea,agrafilea,ahesfilea,&
                               dipfile,dipfilea,&
                               adipfile,adipfilea,&
                               mocofile,mocofilea, &
                               transfile,transfilea,&
                               nactfile,nactfilea,&
                               nmfile,nmfilea,&
                               coofile,coofilea,&
                               socfile,socfilea,&
                               asocfile,asocfilea,&
                               csocfile,csocfilea,&
                               casocfile,casocfilea,&
                               cifile,cifilea,&
                               adpesfile,adpesfilea,&
                               symfile,symfilea, &
                               cpesfile,cpesfilea, &
                               ctransfile,ctransfilea

      logical(kind=4), save :: lasciidb

!-----------------------------------------------------------------------
! Flags for old SOCs  
! 
!-----------------------------------------------------------------------
      logical(kind=4), save :: lsocdb = .false.,&
                               lasocdb = .false.,&
                               lcoorddb = .false.

!-----------------------------------------------------------------------
!   Files for direct dynamics database 
! 200  irefdb   : refdb.dat
! 201  igeodb   : geo.db
! 202  ipesdb   : pes.db
! 203  igradb   : gra.db
! 204  ihesdb   : hes.db
! 205  iapesdb  : apes.db
! 206  iagradb  : agra.db
! 207  iahesdb  : ahes.db
! 208  idipdb   : dip.db
! 209  iadipdb  : adip.db
! 210  imocodb  : moco.db
! 211  itransdb : trans.db Adiabatic-diabatic transformation database
! 212  inactdb  : nact.db  Non-adiabatic couplings database
! 213  inmdb    : nmode.db Normal modes database
! 214  isocdb   : soc.db  Diabatic spin-orbit couplings database
! 215  iasocdb  : adsoc.db  Adiabatic spin-orbit couplings database
! 216  icidb    : civec.db  Adiabatic CI vector database
! 217  icoodb   : coord.db Coordinates database (for Cartesians)
! 218  isymdb   : symrep.db Symmetry mapping
! 219  icsocdb  : csoc.db  Diabatic complex spin-orbit couplings database
! 220  icasocdb : cadsoc.db  Adiabatic complex spin-orbit couplings database
! 221  icpesdb  : cpes.db  Diabatic complex pes database
! 222  ictransdb : ctrans.db  Complex ADT database
! 223  icidatdb  : cidb.dat Data for CI vectorfile
!
! 231  igeodba   : geo.dba
! 232  ipesdba   : pes.dba
! 233  igradba   : gra.dba
! 234  ihesdba   : hes.dba
! 235  iapesdba  : apes.dba
! 236  iagradba  : agra.dba
! 237  iahesdba  : ahes.dba
! 238  idipdba   : dip.dba
! 239  iadipdba  : adip.dba
! 240  imocodba  : moco.dba
! 241  itransdba : trans.dba Adiabatic-diabatic transformation database
! 242  inactdba  : nact.dba  Non-adiabatic couplings database
! 243  inmdba    : nmode.dba Normal modes database
! 244  isocdba   : soc.dba  Diabatic spin-orbit couplings database
! 245  iasocdba  : adsoc.dba  Adiabatic spin-orbit couplings database
! 246  icidba    : civec.dba  Adiabatic CI vector database
! 247  icoodba   : coord.dba  Coordinates database (for Cartesians)
! 248  isymdba   : symrep.dba Symmetry mapping
! 249  icsocdba  : csoc.dba  Diabatic complex spin-orbit couplings database
! 250  icasocdba : cadsoc.dba  Adiabatic complex spin-orbit couplings database
! 251  icpesdba  : cpes.dba  Diabatic complex pes database
! 252  ictransdba : ctrans.dba  Complex ADT database
!
! 300  isqldb    : SQL DB 
!
! coorddb,csocdb are obsolete 
!-----------------------------------------------------------------------
      integer(long), parameter :: irefdb =  200, &
                                  igeodb =  201 , &
                                  ipesdb =  202 , &
                                  igradb =  203 , &
                                  ihesdb =  204 , &
                                  iapesdb = 205 , &
                                  iagradb = 206 , &
                                  iahesdb = 207 , &
                                  idipdb =  208 , &
                                  iadipdb = 209 , &
                                  imocodb = 210 , &
                                  itransdb= 211 , &
                                  inactdb = 212 , &
                                  inmdb   = 213 , &
                                  isocdb  = 214 , &
                                  iasocdb = 215 , &
                                  icidb   = 216 , &
                                  icoodb  = 217 , &
                                  isymdb  = 218 , &
                                  icsocdb  = 219 , &
                                  icasocdb = 220 , &
                                  icpesdb  = 221 , &
                                  ictransdb  = 222 , &
                                  icidatdb  = 223 , &
                                  igeodba =  231 , &
                                  ipesdba =  232 , &
                                  igradba =  233 , &
                                  ihesdba =  234 , &
                                  iapesdba = 235 , &
                                  iagradba = 236 , &
                                  iahesdba = 237 , &
                                  idipdba =  238 , &
                                  iadipdba = 239 , &
                                  imocodba = 240 , &
                                  itransdba= 241 , &
                                  inactdba = 242 , &
                                  inmdba   = 243 , &
                                  isocdba  = 244 , &
                                  iasocdba = 245 , &
                                  icidba   = 246 , &
                                  icoodba  = 247 , &
                                  isymdba  = 248 , &
                                  icsocdba = 249 , &
                                  icasocdba = 250 , &
                                  icpesdba  =  251 , & 
                                  ictransdba  = 252 

      integer(long), parameter :: isqldb = 300  ! only used for file deletion

      public

contains

!#######################################################################
!
! Routine to set filenames used for DB.
!
!#######################################################################

      subroutine setfilenames

      implicit none

! name for DB reference data file
      refdbfile=ddname(1:ddlaenge)//'/refdb.dat'
! name for CI vec data file
      cidbfile=ddname(1:ddlaenge)//'/cidb.dat'

! Names for flat file structure DB
! binary files
      geofile=ddname(1:ddlaenge)//'/geo.db'
      pesfile=ddname(1:ddlaenge)//'/pes.db'
      grafile=ddname(1:ddlaenge)//'/gra.db'
      hesfile=ddname(1:ddlaenge)//'/hes.db'
      apesfile=ddname(1:ddlaenge)//'/adpes.db'
      agrafile=ddname(1:ddlaenge)//'/adgra.db'
      ahesfile=ddname(1:ddlaenge)//'/adhes.db'
      dipfile=ddname(1:ddlaenge)//'/dip.db'
      adipfile=ddname(1:ddlaenge)//'/addip.db'
      mocofile=ddname(1:ddlaenge)//'/moco.db'
      transfile=ddname(1:ddlaenge)//'/trans.db'
      nactfile=ddname(1:ddlaenge)//'/nact.db'
      nmfile=ddname(1:ddlaenge)//'/nmode.db'
      coofile=ddname(1:ddlaenge)//'/coord.db'
      socfile=ddname(1:ddlaenge)//'/soc.db'
      asocfile=ddname(1:ddlaenge)//'/adsoc.db'
      casocfile=ddname(1:ddlaenge)//'/cadsoc.db'
      cifile=ddname(1:ddlaenge)//'/civec.db'
      symfile=ddname(1:ddlaenge)//'/symrep.db'
      cpesfile=ddname(1:ddlaenge)//'/cpes.db'

! ascii files
      geofilea=ddname(1:ddlaenge)//'/geo.dba'
      pesfilea=ddname(1:ddlaenge)//'/pes.dba'
      grafilea=ddname(1:ddlaenge)//'/gra.dba'
      hesfilea=ddname(1:ddlaenge)//'/hes.dba'
      apesfilea=ddname(1:ddlaenge)//'/adpes.dba'
      agrafilea=ddname(1:ddlaenge)//'/adgra.dba'
      ahesfilea=ddname(1:ddlaenge)//'/adhes.dba'
      dipfilea=ddname(1:ddlaenge)//'/dip.dba'
      adipfilea=ddname(1:ddlaenge)//'/addip.dba'
      mocofilea=ddname(1:ddlaenge)//'/moco.dba'
      transfilea=ddname(1:ddlaenge)//'/trans.dba'
      nactfilea=ddname(1:ddlaenge)//'/nact.dba'
      nmfilea=ddname(1:ddlaenge)//'/nmode.dba'
      coofilea=ddname(1:ddlaenge)//'/coord.dba'
      socfilea=ddname(1:ddlaenge)//'/soc.dba'
      asocfilea=ddname(1:ddlaenge)//'/adsoc.dba'
      casocfilea=ddname(1:ddlaenge)//'/cadsoc.dba'
      cifilea=ddname(1:ddlaenge)//'/civec.dba'
      symfilea=ddname(1:ddlaenge)//'/symrep.dba'
      cpesfilea=ddname(1:ddlaenge)//'/cpes.dba'

      end subroutine setfilenames

!#######################################################################
!
! routine to check flat file DB to see which type it is (ascii or 
! binary) and which files are present
!
! ierr = 1 if not an old DB
!#######################################################################

      subroutine chkdb(ierr)

      implicit none

      integer(long), intent(out) :: ierr
      logical(kind=4)            :: check,checka

      call setfilenames

!-----------------------------------------------------------------------
! check for existence of redb.dat file
!-----------------------------------------------------------------------
      check = .false.
      inquire(file=refdbfile,exist=check)

      ierr=0
      if (.not. check) then
         routine = 'chkdb (dbtools.f9090)'
         message = 'ERROR: No REFDB.DAT file! Is this a DB?'
         call warnmsg
         ierr = 1
         return
      endif

!-----------------------------------------------------------------------
! check existence of DB and type of DB (ascii or binary)
!-----------------------------------------------------------------------
      check = .false.
      inquire(file=geofile,exist=check)

      checka = .false.
      inquire(file=geofilea,exist=checka)

      if (check .and. checka) then
         routine = 'chkdb (dbtools.f9090)'
         message = 'Both binary and ascii DB present ! Sort out which.'
         call errormsg
      else if (checka) then
         lasciidb = .true.
         lgeodb=.true.
      else if (check) then
         lasciidb = .false.
         lgeodb=.true.
      endif

! geo file found, continue
      if (lgeodb) then
         ndbfiles = ndbfiles+1
      else
         write(ilog,'(/,a)') 'Empty DB.'
         return
      endif

! see if pes file present
      lpesdb = .false.
      if (lasciidb) then
         inquire(file=pesfilea,exist=lpesdb)
      else
         inquire(file=pesfile,exist=lpesdb)
      endif
      if (lpesdb) ndbfiles = ndbfiles+1

! see if complex pes file present
      lcpesdb = .false.
      if (lasciidb) then
         inquire(file=cpesfilea,exist=lcpesdb)
      else
         inquire(file=cpesfile,exist=lcpesdb)
      endif
      if (lcpesdb) ndbfiles = ndbfiles+1

! see if gra file present
      lgradb = .false.
      if (lasciidb) then
         inquire(file=grafilea,exist=lgradb)
      else
         inquire(file=grafile,exist=lgradb)
      endif
      if (lgradb) ndbfiles = ndbfiles+1

! see if hes file present
      lhesdb = .false.
      if (lasciidb) then
         inquire(file=hesfilea,exist=lhesdb)
      else
         inquire(file=hesfile,exist=lhesdb)
      endif
      if (lhesdb) ndbfiles = ndbfiles+1

! see if apes file present
      lapesdb = .false.
      if (lasciidb) then
         inquire(file=apesfilea,exist=lapesdb)
      else
         inquire(file=apesfile,exist=lapesdb)
      endif
      if (lapesdb) ndbfiles = ndbfiles+1

! see if agra file present
      lagradb = .false.
      if (lasciidb) then
         inquire(file=agrafilea,exist=lagradb)
      else
         inquire(file=agrafile,exist=lagradb)
      endif
      if (lagradb) ndbfiles = ndbfiles+1

! see if ahes file present
      lahesdb = .false.
      if (lasciidb) then
         inquire(file=ahesfilea,exist=lahesdb)
      else
         inquire(file=ahesfile,exist=lahesdb)
      endif
      if (lahesdb) ndbfiles = ndbfiles+1

! see if dipole file present
      ldipdb = .false.
      if (lasciidb) then
         inquire(file=dipfilea,exist=ldipdb)
      else
         inquire(file=dipfile,exist=ldipdb)
      endif
      if (ldipdb) ndbfiles = ndbfiles+1

! see if addipole file present
      ladipdb = .false.
      if (lasciidb) then
         inquire(file=adipfilea,exist=ladipdb)
      else
         inquire(file=adipfile,exist=ladipdb)
      endif
      if (ladipdb) ndbfiles = ndbfiles+1

! see if MO coeff file present
      lmocodb = .false.
      if (lasciidb) then
         inquire(file=mocofilea,exist=lmocodb)
      else
         inquire(file=mocofile,exist=lmocodb)
      endif
      if (lmocodb) ndbfiles = ndbfiles+1

! see if Adiab/Diab trans file present
      ltransdb = .false.
      if (lasciidb) then
         inquire(file=transfilea,exist=ltransdb)
      else
         inquire(file=transfile,exist=ltransdb)
      endif
      if (ltransdb) ndbfiles = ndbfiles+1

! see if complex Adiab/Diab trans file present
      lctransdb = .false.
      if (lasciidb) then
         inquire(file=ctransfilea,exist=lctransdb)
      else
         inquire(file=ctransfile,exist=lctransdb)
      endif
      if (lctransdb) ndbfiles = ndbfiles+1

! see if non-adiab. CT file present
      lnactdb = .false.
      if (lasciidb) then
         inquire(file=nactfilea,exist=lnactdb)
      else
         inquire(file=nactfile,exist=lnactdb)
      endif
      if (lnactdb) ndbfiles = ndbfiles+1

! see if normal mode file exists
      lnmdb = .false.
      if (lasciidb) then
         inquire(file=nmfilea,exist=lnmdb)
      else
         inquire(file=nmfile,exist=lnmdb)
      endif
      if (lnmdb) ndbfiles = ndbfiles+1

! see if coordinate file exists
      lcoorddb = .false.
      if (lasciidb) then
         inquire(file=coofilea,exist=lcoorddb)
      else
         inquire(file=coofile,exist=lcoorddb)
      endif
      if (lcoorddb) ndbfiles = ndbfiles+1

! see if SOC files (adiabatic and diabatic) present
      lasocdb = .false.
      if (lasciidb) then
         inquire(file=asocfilea,exist=lasocdb)
      else
         inquire(file=asocfile,exist=lasocdb)
      endif
      if (lasocdb) ndbfiles = ndbfiles+1

      lsocdb = .false.
      if (lasciidb) then
         inquire(file=socfilea,exist=lsocdb)
      else
         inquire(file=socfile,exist=lsocdb)
      endif
      if (lsocdb) ndbfiles = ndbfiles+1

! see if complex SOC files (adiabatic and diabatic) present
      lcasocdb = .false.
      if (lasciidb) then
         inquire(file=casocfilea,exist=lcasocdb)
      else
         inquire(file=casocfile,exist=lcasocdb)
      endif
      if (lcasocdb) ndbfiles = ndbfiles+1

      lcsocdb = .false.
      if (lasciidb) then
         inquire(file=csocfilea,exist=lcsocdb)
      else
         inquire(file=csocfile,exist=lcsocdb)
      endif
      if (lcsocdb) ndbfiles = ndbfiles+1

! see if CI vector files (adiabatic and diabatic) present
      lcivecdb = .false.
      if (lasciidb) then
         inquire(file=cifilea,exist=lcivecdb)
      else
         inquire(file=cifile,exist=lcivecdb)
      endif
      if (lcivecdb) then
         check = .false.   ! check cidb.dat file exists
         inquire(file=cidbfile,exist=check)
         ierr=0
         if (.not. check) then
            routine = 'chkdb (dbtools.f9090)'
            message = 'ERROR: CI Vector DB but No CIDB.DAT file!'
            call warnmsg
            ierr = 1
            return
         endif
         ndbfiles = ndbfiles+1
      endif

! see if symmetry mapping file present
      lsymdb = .false.
      if (lasciidb) then
         inquire(file=symfilea,exist=lsymdb)
      else
         inquire(file=symfile,exist=lsymdb)
      endif
      if (lsymdb) ndbfiles = ndbfiles+1

      end subroutine chkdb

!#######################################################################
!
! Open the old style DB files
!
! filestat: status, e.g. new,old
! filepos:  position, e.g. rewind,append
!#######################################################################

      subroutine opendbold(filestat,filepos)

      implicit none

      integer(long)     :: ilbl
      character(len=c5) :: filename
      character(len=c1) :: filestat,filepos

      open(irefdb,file=refdbfile,status=filestat,form='formatted',err=1040)
      if (lcivecdb) then
          open(icidatdb,file=cidbfile,status=filestat,form='formatted',err=1050)
      endif

      if (lasciidb) then
         filename=geofilea
     !    write(*,*)'filename ', filename,' status', filestat,' pos', filepos
         open(igeodba,file=filename,status=filestat,form='formatted',&
              position=filepos,err=1020)
         filename=pesfilea
         if (lpesdb) open(ipesdba,file=filename,status=filestat,form='formatted',&
              position=filepos,err=1020)
         filename=cpesfilea
         if (lcpesdb) open(icpesdba,file=filename,status=filestat,form='formatted',&
              position=filepos,err=1020)
         filename=grafilea
         if (lgradb) &
              open(igradba,file=filename,status=filestat,form='formatted',&
              position=filepos,err=1020)
         filename=hesfilea
         if (lhesdb) &
              open(ihesdba,file=filename,status=filestat,form='formatted',&
              position=filepos,err=1020)
         filename=apesfilea
         if (lapesdb) open(iapesdba,file=filename,status=filestat,form='formatted',&
              position=filepos,err=1020)
         filename=agrafilea
         if (lagradb) &
              open(iagradba,file=filename,status=filestat,form='formatted',&
              position=filepos,err=1020)
         filename=ahesfilea
         if (lahesdb) &
              open(iahesdba,file=filename,status=filestat,form='formatted',&
              position=filepos,err=1020)
         filename=dipfilea
         if (ldipdb) open(idipdba,file=filename,status=filestat,form='formatted',&
              position=filepos,err=1020)
         filename=adipfilea
         if (ladipdb) open(iadipdba,file=filename,status=filestat,form='formatted',&
              position=filepos,err=1020)
         filename=mocofilea
         if (lmocodb) open(imocodba,file=filename,status=filestat,&
              position=filepos,form='formatted',err=1020)
         filename=nactfilea
         if (lnactdb) open(inactdba,file=filename,status=filestat,&
              position=filepos,form='formatted',err=1020)
         filename=transfilea
         if (ltransdb) open(itransdba,file=filename,status=filestat,&
              position=filepos,form='formatted',err=1020)
         filename=ctransfilea
         if (lctransdb) open(ictransdba,file=filename,status=filestat,&
              position=filepos,form='formatted',err=1020)
         filename=nmfilea
         if (lnmdb) open(inmdba,file=filename,status=filestat,&
              position=filepos,form='formatted',err=1020)
         filename=coofilea
         if (lcoorddb) open(icoodba,file=filename,status=filestat,&
              position=filepos,form='formatted',err=1020)
         filename=socfilea
         if (lsocdb) open(isocdba,file=filename,status=filestat,&
              position=filepos,form='formatted',err=1020)
         filename=asocfilea
         if (lasocdb) open(iasocdba,file=filename,status=filestat,&
              position=filepos,form='formatted',err=1020)
         filename=csocfilea
         if (lcsocdb) open(icsocdba,file=filename,status=filestat,&
              position=filepos,form='formatted',err=1020)
         filename=casocfilea
         if (lcasocdb) open(icasocdba,file=filename,status=filestat,&
              position=filepos,form='formatted',err=1020)
         filename=cifilea
         if (lcivecdb) open(icidba,file=filename,status=filestat,&
              position=filepos,form='formatted',err=1020)
         filename=symfilea
         if (lsymdb) open(isymdba,file=filename,status=filestat,&
              position=filepos,form='formatted',err=1020)

      else

         filename=geofile
         open(igeodb,file=filename,status=filestat,form='unformatted',&
              position=filepos,err=1010)
         filename=pesfile
         if (lpesdb) open(ipesdb,file=filename,status=filestat,form='unformatted',&
              position=filepos,err=1010)
         filename=cpesfile
         if (lcpesdb) open(icpesdb,file=filename,status=filestat,form='unformatted',&
              position=filepos,err=1010)
         filename=grafile
         if (lgradb) open(igradb,file=filename,status=filestat,form='unformatted',&
              position=filepos,err=1010)
         filename=hesfile
         if (lhesdb) open(ihesdb,file=filename,status=filestat,form='unformatted',&
              position=filepos,err=1010)
         filename=apesfile
         if (lapesdb) open(iapesdb,file=filename,status=filestat,form='unformatted',&
              position=filepos,err=1010)
         filename=agrafile
         if (lagradb) open(iagradb,file=filename,status=filestat,form='unformatted',&
              position=filepos,err=1010)
         filename=ahesfile
         if (lahesdb) open(iahesdb,file=filename,status=filestat,form='unformatted',&
              position=filepos,err=1010)
         filename=dipfile
         if (ldipdb) open(idipdb,file=filename,status=filestat,form='unformatted',&
              position=filepos,err=1010)
         filename=adipfile
         if (ladipdb) open(iadipdb,file=filename,status=filestat,form='unformatted',&
              position=filepos,err=1010)
         filename=mocofile
         if (lmocodb) open(imocodb,file=filename,status=filestat,&
              position=filepos,form='unformatted',err=1010)
         filename=nactfile
         if (lnactdb) open(inactdb,file=filename,status=filestat,&
              position=filepos,form='unformatted',err=1010)
         filename=transfile
         if (ltransdb) open(itransdb,file=filename,status=filestat,&
              position=filepos,form='unformatted',err=1010)
         filename=ctransfile
         if (lctransdb) open(ictransdb,file=filename,status=filestat,&
              position=filepos,form='unformatted',err=1010)
         filename=nmfile
         if (lnmdb) open(inmdb,file=filename,status=filestat,&
              position=filepos,form='unformatted',err=1010)
         filename=coofile
         if (lcoorddb) open(icoodb,file=filename,status=filestat,&
              position=filepos,form='unformatted',err=1010)
         filename=socfile
         if (lsocdb) open(isocdb,file=filename,status=filestat,&
              position=filepos,form='unformatted',err=1010)
         filename=asocfile
         if (lasocdb) open(iasocdb,file=filename,status=filestat,&
              position=filepos,form='unformatted',err=1010)
         filename=csocfile
         if (lcsocdb) open(icsocdb,file=filename,status=filestat,&
              position=filepos,form='unformatted',err=1010)
         filename=casocfile
         if (lcasocdb) open(icasocdb,file=filename,status=filestat,&
              position=filepos,form='unformatted',err=1010)
         filename=cifile
         if (lcivecdb) open(icidb,file=filename,status=filestat,&
              position=filepos,form='unformatted',err=1010)
         filename=symfile
         if (lsymdb) open(isymdb,file=filename,status=filestat,&
              position=filepos,form='unformatted',err=1010)

      endif

      return
 1010 ilbl=index(filename,' ')-1
      write(6,'(2a)') 'ERROR opening binary DB file: ',filename(1:ilbl)
      stop
 1020 ilbl=index(filename,' ')-1
      write(6,'(2a)') 'ERROR opening ascii DB file: ',filename(1:ilbl)
      stop
 1040 write(6,'(a)') 'ERROR opening Refdb.dat file'
      stop
 1050 write(6,'(a)') 'ERROR opening Cidb.dat file'
      stop

      end subroutine opendbold

!#######################################################################
!
! Close the DB files
!
! filestat: status, e.g. delete, keep
!#######################################################################

      subroutine closedb(filestat)

      implicit none

      character(len=c1) :: filestat

      close(irefdb,status=filestat)
      if (lcivecdb) close(icidatdb,status=filestat)

      if (lasciidb) then
         close(igeodba,status=filestat)
         if (lpesdb) close(ipesdba,status=filestat)
         if (lcpesdb) close(icpesdba,status=filestat)
         if (lgradb) close(igradba,status=filestat)
         if (lhesdb) close(ihesdba,status=filestat)
         if (lapesdb) close(iapesdba,status=filestat)
         if (lagradb) close(iagradba,status=filestat)
         if (lahesdb) close(iahesdba,status=filestat)
         if (ldipdb) close(idipdba,status=filestat)
         if (ladipdb) close(iadipdba,status=filestat)
         if (lmocodb) close(imocodba,status=filestat)
         if (lnactdb) close(inactdba,status=filestat)
         if (ltransdb) close(itransdba,status=filestat)
         if (lctransdb) close(ictransdba,status=filestat)
         if (lnmdb) close(inmdba,status=filestat)
         if (lcoorddb) close(icoodba,status=filestat)
         if (lsocdb) close(isocdba,status=filestat)
         if (lasocdb) close(iasocdba,status=filestat)
         if (lcsocdb) close(icsocdba,status=filestat)
         if (lcasocdb) close(icasocdba,status=filestat)
         if (lcivecdb) close(icidba,status=filestat)
         if (lsymdb) close(isymdba,status=filestat)

      else
         close(igeodb,status=filestat)
         if (lpesdb) close(ipesdb,status=filestat)
         if (lcpesdb) close(icpesdb,status=filestat)
         if (lgradb) close(igradb,status=filestat)
         if (lhesdb) close(ihesdb,status=filestat)
         if (lapesdb) close(iapesdb,status=filestat)
         if (lagradb) close(iagradb,status=filestat)
         if (lahesdb) close(iahesdb,status=filestat)
         if (ldipdb) close(idipdb,status=filestat)
         if (ladipdb) close(iadipdb,status=filestat)
         if (lmocodb) close(imocodb,status=filestat)
         if (lnactdb) close(inactdb,status=filestat)
         if (ltransdb) close(itransdb,status=filestat)
         if (lctransdb) close(ictransdb,status=filestat)
         if (lnmdb) close(inmdb,status=filestat)
         if (lcoorddb) close(icoodb,status=filestat)
         if (lsocdb) close(isocdb,status=filestat)
         if (lasocdb) close(iasocdb,status=filestat)
         if (lcsocdb) close(icsocdb,status=filestat)
         if (lcasocdb) close(icasocdb,status=filestat)
         if (lcivecdb) close(icidb,status=filestat)
         if (lsymdb) close(isymdb,status=filestat)

      endif

      return
      end subroutine closedb

!#######################################################################
!
! Create refdb.dat
!
!#######################################################################

      subroutine wrrefdborig

      implicit none

      integer(long)                           :: j,i,ilbl
      character(len=c5)                       :: filename 
      logical(kind=4)                         :: check
      
!-----------------------------------------------------------------------
! look for reference geometry in refdb.dat file. If needed write file.
!-----------------------------------------------------------------------
      check = .false.
      inquire(irefdb,opened=check)
      filename=ddname(1:ddlaenge)//'/refdb.dat'
      if (.not. check) then
         open(irefdb,file=filename,status='old',form='formatted', &
              err=1002)                                         
      else
         rewind(irefdb)
      endif
                
      write(irefdb,'(f20.12)') progver
      write(irefdb,'(a)') '# Natm, Nroot, Nddstate'
      write(irefdb,'(i10,i5,i5)') natm,nroot,nddstate
      if (lmocodb) then
         write(irefdb,'(a)') '# Nbasis'
         write(irefdb,'(i10)') nbasis
      endif
      if (ldiabdb) then
         write(irefdb,'(a)') '# Dercpdim'
         write(irefdb,'(i10)') dercpdim
      endif
      write(irefdb,'(a)') '# Nsocstate'
      write(irefdb,'(i10)') nsocstate
      if (nsocstate .ne. nddstate) then
         write(irefdb,'(a)') '# Nddmult'
         write(irefdb,'(7(i4))') (nddmult(i),i=1,7)
      endif
      write(irefdb,'(a)') '# E0'
      write(irefdb,'(f20.12)') dde0
      write(irefdb,'(a)') '# Symmetry nos.'
      write(irefdb,'(2i7)') dimref,nfam
      write(irefdb,'(a)') '# Mass '
      write(irefdb,'(4f20.12)') (mref(i),i=1,natm)
      write(irefdb,'(a)') '# Name, Xcoo'
      do i=1,natm
         write(irefdb,'(a10,3f20.12)') atnam(i),(xref(j,i),j=1,3)
      enddo

!-----------------------------------------------------------------------
! write symmetry information
!-----------------------------------------------------------------------
      write(irefdb,'(a)') '# Pntgp'
      write(irefdb,'(i10,i5)') dbpntgp, nsymop
      if (nsymop .gt. 12) then
         routine = 'Wrrefdb'
         message = 'Change format for dbsymmap to write all symmetry maps'
         call errormsg
      endif
      if (dbpntgp .gt. 0) then
         write(irefdb,'(a)') '# Symmaps'
         do i=1,natm
            write(irefdb,'(12i5)') (dbsymmap(i,j),j=1,nsymop)
         enddo
      endif

      close(irefdb)

      return
 1002 ilbl = index(filename,' ')-1
      routine='Wrrefdb (dd_db.f90)'
      write(message,'(2a)') 'Error opening: ', filename(1:ilbl)
      call errormsg

      end subroutine wrrefdborig

!#######################################################################
!              
! Read an old Refdb.dat file
!     
!#######################################################################
      
      subroutine rdrefdborig

      implicit none

      integer(long)              :: i,j,ilbl
      character(len=c5)          :: filename,string
      real(dop)                  :: filever
      logical                    :: lfound=.false., check=.false.

! nsocstate not in older refdb files.
      nsocstate =-999
      nddmult(:) = 0

      check = .false.
      inquire(irefdb,opened=check)
      filename=ddname(1:ddlaenge)//'/refdb.dat'
      if (.not. check) then
         open(irefdb,file=filename,status='old',form='formatted', &
              err=1002)                                         
      else
         rewind(irefdb)
      endif
      
      read(irefdb,*,err=1003) filever
  100 continue
      read(irefdb,'(a)',end=500) string
      if (string .eq. '# Natm, Nstate') then
         read(irefdb,*) natm,nroot
         if (nddstate .ne. nroot) nddstate = nroot
                                                   
      else if (string .eq. '# Natm, Nroot, Nddstate') then
         read(irefdb,*) natm,nroot,nddstate        

      else if (string .eq. '# Nbasis') then
         read(irefdb,*) nbasis

      else if (string .eq. '# Dercpdim')then
         read(irefdb,*) dercpdim

      else if (string .eq. '# Nsocstate') then
         read(irefdb,*) nsocstate        

      else if (string .eq. '# Nddmult') then
         read(irefdb,*) (nddmult(i),i=1,7)

      else if (string .eq. '# E0') then
         read(irefdb,*) dde0

      else if (string .eq. '# Symmetry nos.') then
         read(irefdb,*) dimref,nfam

      else if (string .eq. '# Mass') then
         read(irefdb,*) (mref(i),i=1,natm)

      else if (string .eq. '# Name, Xcoo') then
         lfound=.true.
         do i=1,natm
            read(irefdb,*) atnam(i),(xref(j,i),j=1,3)
         enddo

      else if (string .eq. '# Pntgp') then
         read(irefdb,*) dbpntgp, nsymop

      else if (string .eq. '# Symmaps') then
         do i=1,natm
            read(irefdb,*) (dbsymmap(i,j),j=1,nsymop)
         enddo

      endif
      go to 100

  500 continue

      if (.not. lfound) then
         routine='Rdrefdborig'
         message='Coordinates not found. Error in REFDB.DAT?'
         call errormsg
      endif

!-----------------------------------------------------------------------
! Take default nsocstate if not in file (no SOC included)
! Set np. of multiplicities, nsmult
!-----------------------------------------------------------------------
      nsmult = count(nddmult .gt. 0)
      if (nsmult .eq. 0) then   ! ddmult not given
         nsocstate=nddstate
         nddmult(1) = nddstate
         nsmult = 1
      endif

      return
 1002 ilbl = index(filename,' ')-1
      routine='Rdrefdborig (dbtools.f90)'
      write(message,'(2a)') 'Error opening: ', filename(1:ilbl)
      call errormsg
 1003 ilbl = index(filename,' ')-1
      routine='Rdrefdborig (dbtools.f90)'
      write(message,'(2a)') 'Error in version: ', filename(1:ilbl)
      call errormsg
      stop
      end subroutine rdrefdborig

!#######################################################################
!
! Create cidb.dat
!
!#######################################################################

      subroutine wrcidbdat

      implicit none

      integer(long)       :: ns
      character(len=c5)   :: filename 
      logical(kind=4)     :: check
      
!-----------------------------------------------------------------------
! Write reference data for CI vector in cidb.dat
!-----------------------------------------------------------------------
      check = .false.
      inquire(icidatdb,opened=check)
      filename = ddname(1:ddlaenge)//'/cidb.dat'
      if (.not. check) then
         open(icidatdb,file=filename,status='old',form='formatted', &
              err=1002)                                         
      else
         rewind(icidatdb)
      endif
                
      write(icidatdb,'(f20.12)') progver
      write(icidatdb,'(a)') '# Imult Imultblk Nconfig'
      do ns=1,nsmult
         write(icidatdb,'(i5,i8,i8)') idxmult(ns),imultblk(ns),nciconf(ns)
      enddo

      return
 1002 routine='Wrcidbdat (dd_db.f90)'
      write(message,'(2a)') 'Error opening: ', trim(filename)
      call errormsg

      end subroutine wrcidbdat

!#######################################################################
!
! Read cidb.dat
!
!#######################################################################

      subroutine rdcidbdat

      implicit none

      integer(long)           :: ns
      real(dop)               :: filever
      character(len=c5)       :: filename,string
      logical(kind=4)         :: check
      
!-----------------------------------------------------------------------
! Read reference data for CI vector from cidb.dat
!-----------------------------------------------------------------------
      check = .false.
      inquire(icidatdb,opened=check)
      filename = ddname(1:ddlaenge)//'/cidb.dat'
      if (.not. check) then
         open(icidatdb,file=filename,status='old',form='formatted', &
              err=1002)                                         
      else
         rewind(icidatdb)
      endif
                
      read(icidatdb,*,err=1003) filever
  100 continue
      read(icidatdb,'(a)',end=500) string
      if (string .eq. '# Imult Imultblk Nconfig') then
         do ns=1,nsmult
            read(icidatdb,'(i5,i8,i8)') idxmult(ns),imultblk(ns),nciconf(ns)
         enddo

      endif
      go to 100

  500 continue

      return
 1002 routine='Rdcidbdat (dd_db.f90)'
      write(message,'(2a)') 'Error opening: ', trim(filename)
      call errormsg
 1003 routine='Rdcidbdat (dbtools.f90)'
      write(message,'(2a)') 'Error in version: ', trim(filename)
      call errormsg

      end subroutine rdcidbdat

!#######################################################################
!
! Get the no. of records in a DB by reading the geo.db file
!        
!#######################################################################
      
      subroutine getdbnrecorig(nrec)

      implicit none
                                         
      integer(long), intent(out)     :: nrec
      logical(kind=4)                :: fexist
      character(len=c5)              :: geofile

!-----------------------------------------------------------------------
! get filestem from ddname variable and open database file with 
! geometries
!-----------------------------------------------------------------------
      geofile=ddname(1:ddlaenge)//'/geo.db'
         
      inquire(exist=fexist,file=geofile)
         
      nrec=0
      
      if(fexist)then
         open(igeodb,file=geofile,form='unformatted',err=1001)
!-----------------------------------------------------------------------
! count records in file
!-----------------------------------------------------------------------
         do
            read(igeodb,end=100,err=1000)
            nrec=nrec+1 
         end do
      endif
         
 100  continue
         
      close(igeodb)

      return
 1000 routine='Getdbnrec (dbtools.f90)'
      write(message,'(2a)') &
         'ERROR reading (in getdbnrec) DB file  ',geofile          
      call errormsg   
 1001 routine = 'Getdbnrec (dbtools.f90)'
      write(message,'(2a)') 'ERROR opening DB file  ',geofile
      call errormsg

      end subroutine getdbnrecorig

!#######################################################################
! read refdb.dat file for system sizes that can be used to allocate memory
!#######################################################################

      subroutine rdrefdbmemorig

      implicit none

      integer(long)              :: ilbl
      character(len=c5)          :: filename,string
      real(dop)                  :: filever
      logical                    :: check=.false.    

      nsocstate = -999
      
      inquire(irefdb,opened=check)
      filename=ddname(1:ddlaenge)//'/refdb.dat'      
      if (.not. check) then                             
         open(irefdb,file=filename,status='old',form='formatted', &
              err=1002)
      else
         rewind(irefdb)
      endif
      
      read(irefdb,*,err=1003) filever
  100 continue
      read(irefdb,'(a)',end=500) string              
      if (string .eq. '# Natm, Nstate') then
         read(irefdb,*) natm,nroot                   
         if (nddstate .ne. nroot) nddstate = nroot   
      
      else if (string .eq. '# Natm, Nroot, Nddstate') then
         read(irefdb,*) natm,nroot,nddstate

      else if (string .eq. '# Nbasis') then
         read(irefdb,*) nbasis
         lmocodb = .true.

      else if (string .eq. '# Nsocstate') then
         read(irefdb,*) nsocstate        

      else if (string(1:7) .eq. '# Pntgp') then
         read(irefdb,*) dbpntgp, nsymop

      endif
      go to 100
      
  500 continue

      if (nsocstate .lt. 0) nsocstate=nddstate

      close(irefdb)

      return
 1002 ilbl = index(filename,' ')-1
      routine='Rdrefdbmem (dbtools.f90)'
      write(message,'(2a)') 'Error opening: ', filename(1:ilbl)
      call errormsg
 1003 ilbl = index(filename,' ')-1
      routine='Rdrefdb (dbtools.f90)'
      write(message,'(2a)') 'Error reading: ', filename(1:ilbl)
      call errormsg
      stop
      end subroutine rdrefdbmemorig


      end module dbtools
