      program ddpes

use decimal, only: long,dop,sip
use global
use globmemmod
use versions
use aglobal
use keyunitsmod
use dvrdatmod
use griddatmod
use operdef
use psidef
use channels
use datenmod
use dirdyn
use compdatmod
use plot
use gapmod

use maxv, only: maxdim
use logdat
use iofile, only: rdpsiinfo
use rddvrmod, only: dvrinfo
use rdopermod, only: operinfo
use zeigauswmod, only: zeigausw
use memhopsmod, only: memhops
use dd_db, only: ddq2x,distdb,getdbnrec

      implicit none

      integer(long)                                     :: chkdvr,chkgrd,chkdat,&
                                                           chkpsi,ilbl

      character(len=c5)                                 :: filename

      logical(kind=4)                                   :: lerr

      call default ! Initialise variables
      call adefault

      filename=rname(1:dlaenge)//'/restart'
      ilbl=index(filename,' ')-1
      open(irst,file=filename,form='unformatted',status='old',err=1000)
      call rdmemdim(irst)
      close(irst)

      ldbsave=.true.

      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      call alloc_psidef
      call alloc_dirdyn
      call alloc_dddb
      call alloc_dercp
      call alloc_gapkernel

      filename=dname(1:dlaenge)//'/dvr'
      open(idvr,file=filename,form='unformatted',status='old',err=1000)
      ilbl=index(filename,' ')-1
      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      if (lerr) go to 999
      close(idvr)

      filename=operfile
      open(ioper,file=filename,form='unformatted',status='old',&
           err=1000)
      ilbl=index(filename,' ')-1
      chkdvr=2
      chkgrd=1
      call operinfo(lerr,chkdvr,chkgrd)
      if (lerr) go to 999
      close(ioper)
      if (loptcontract) then
         write(6,'(a)') &
    'Warning: Operator contains terms that mix KE and PE '
         stop
      endif

      chkdvr=1
      chkgrd=1
      chkdat=1
      chkpsi=1

      open(ipsi,file='psi',form='unformatted',status='old')
      call rdmemdim(ipsi)
      read(ipsi) filever(ipsi)
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      close(ipsi)

      call zeigausw
      call memausw

      miplt2ort=midim
      midim=miplt2ort+ortdim
      miort2plt=midim
      midim=miort2plt+ortdim

      miworki = midim

      call memhops

if(allocated(mi))deallocate(mi)

allocate(mi(midim))

      call setpes(mi(mifftfak))

deallocate(mi)
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_psidef
      call dealloc_dirdyn
      call dealloc_dddb
      call dealloc_dercp
      call dealloc_gapkernel

      stop
  999 continue
      message= 'Error reading :'//filename(1:c5-15)
      write(6,'(a)') message
      stop
 1000 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop

      contains

!#######################################################################

!     SETPES

!#######################################################################

      subroutine setpes(fftfak)

use global
use aglobal
use dvrdatmod
use griddatmod
use operdef
use psidef
use channels
use datenmod
use dirdyn
use compdatmod
use plot

use gwplib, only: gwp_cmom,gwp_ovl,gwp_param
use plotutils
use rdopermod, only: rdoper
use rddvrmod
use psiutil, only: getweight

      implicit none

      integer(long)                                     :: chkdvr,chkgrd,&
                                                           ierr
      integer(long), dimension(1)                       :: idum
      integer(long), dimension(*), intent(out)          :: fftfak

      real(dop), dimension(1)                           :: rdum
      real(dop), dimension(hopsdim)                     :: hops
      real(dop), dimension(ortdim)                      :: ort,weight
      real(dop), dimension(dvrdim)                      :: trafo
      real(dop), dimension(:,:,:), allocatable          :: srf

      complex(dop), dimension(1)                        :: zdum
      complex(dop), dimension(fftdim)                   :: rueck
      complex(dop), dimension(expdim)                   :: exphin

!-----------------------------------------------------------------------
! read DVR and operator data
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1

      dvrdata(1)=.true.      ! ort
      dvrdata(2)=.true.      ! trafo
      if(lfft) then
         dvrdata(6)=.true.   ! rueck
         dvrdata(7)=.true.   ! fftfak
         dvrdata(11)=.true.  ! exphin
      endif
      call rddvr(ort,trafo,rdum,rdum,zdum,rueck,fftfak,exphin,zdum,&
                 idum,idum,rdum,chkdvr)

      operfile=operfile
      hops=0.0d0
      call rdoper(hops,chkdvr,chkgrd)

!-----------------------------------------------------------------------
! allocate arrays for plotting
!-----------------------------------------------------------------------
      call alloc_plot

      call getplgdim

      weight=0.0_dop

      ddmindb = 999.0d0

      call getweight(trafo,weight,ort,.true.)

      plnham=1

      zaxunitlab='ev'
      zaxunit = 1.0_dop
      call keyunits(zaxunit,zaxunitlab,ierr)
      
      call pesdiab(hops)

      end subroutine setpes

!#######################################################################

!     PESDIAB
!     Sets up grid defined by cuts specified in input file.
!     

!#######################################################################

      subroutine pesdiab(hops)

use global
use aglobal
use maxv, only: maxdim,maxsta
use logdat
use griddatmod
use channels
use dvrdatmod
use psidef
use operdef
use compdatmod
use plot

use potevalmod, only: calcdiab
use rdopermod, only: rdoper

      implicit none

      integer(long)                                   :: i,j,k,&
                                                         lg1,rg1,lg2,rg2,&
                                                         glength1,glength2,&
                                                         nsta,idb,xco,ntimes,&
                                                         x,y,tsteps,&
                                                         gnum,chkdvr,chkgrd,nrec
      integer(long), dimension(maxdim)                :: igpoint
      integer(long), dimension(:,:), allocatable      :: grid

      real(dop)                                       :: sca1,sca2,grddist,&
                                                         intv
      real(dop), dimension(maxdim)                    :: gpoint
      real(dop), dimension(ndofddpes)                 :: xgpoint
      real(dop), dimension(:), allocatable            :: distvec
      real(dop), dimension(hopsdim), intent(inout)       :: hops
      real(dop), dimension(:,:,:), allocatable        :: coords,asrf,ggrid
      real(dop), dimension(:,:,:,:), allocatable      :: srf
      real(dop), dimension(:,:), allocatable          :: darr,grddists
      real(dop), dimension(:,:,:,:,:), allocatable    :: dgtraj,adgtraj
      real(dop), dimension(nspfdof(1))                :: cut1,cut2
      real(dop)                                       :: mindist,mindisp
      real(dop), dimension(:), allocatable            :: times
      real(dop), dimension(:,:,:), allocatable        :: qcent,pcent,dtrajp,&
                                                         adtrajp
      real(dop), dimension(4)                         :: dgridp,adgridp
      real(dop), dimension(:,:,:), allocatable        :: gpops

      complex(dop), dimension(maxsta,maxsta)          :: rotmat

      logical(kind=4)                                 :: lpltraj,ldiab,&
                                                         lanim,lgauss,lerr
      logical(kind=4), dimension(gwpdim(1,1))         :: plgwp

      call usrinput(lg1,rg1,lg2,rg2,glength1,glength2,sca1,sca2,&
                    lpltraj,lgauss,plgwp,cut1,cut2,nsta,tsteps)

      allocate(coords(glength1,glength2,nspfdof(1)))
      allocate(srf(glength1,glength2,nsta,nsta))
      allocate(asrf(glength1,glength2,nsta))
      allocate(darr(glength1,glength2))
      allocate(grid(glength1*glength2,2))
      srf=0.0d0
      coords=0.0d0

      grid=0
      do i=1,glength1
         do j=1,glength2
            coords(i,j,:)=(dble(-lg1+i-1)*cut1(:)*sca1)+(dble(-lg2+j-1)*cut2(:)*sca2)
            grid(((i-1)*glength2)+j,1)=-lg1+i-1
            grid(((i-1)*glength2)+j,2)=-lg2+j-1
         enddo
      enddo

      chkdvr=1
      chkgrd=1
      if (lrediab) then
         call rdoper(hops,chkdvr,chkgrd)
         call getraw
      endif
      call getdbnrec(nrec)
      allocate(distvec(nrec))

      gpoint=0.0d0
      igpoint=0
      darr=0.0d0
      do i=1,glength1
         do j=1,glength2
            gpoint(:nspfdof(1))=coords(i,j,:)
            do k=1,nspfdof(1)
               if (cut1(k).ne.0) then
                  igpoint(k)=i
               else if (cut2(k).ne.0) then
                  igpoint(k)=j
               endif
            enddo
            call calcdiab(hops,asrf(i,j,:),srf(i,j,:,:),rotmat,igpoint,gpoint,plnham)
            call ddq2x(gpoint,xgpoint)
            call distdb(xgpoint,idb,distvec,mindist,mindisp,nrec)
            darr(i,j)=mindist
         enddo
      enddo

      !Write the adiabatic and diabatic surface files for gnuplot

      open(1,file='./ddpesout/pesd1',form='formatted',status='replace')
      open(2,file='./ddpesout/pesd2',form='formatted',status='replace')
      open(3,file='./ddpesout/pesad1',form='formatted',status='replace')
      open(4,file='./ddpesout/pesad2',form='formatted',status='replace')
      open(5,file='./ddpesout/pesd_coup',form='formatted',status='replace')

      xco = grid(1,1)
      do i=1,glength1
         do j=1,glength2
            write(1,'(2I5.1, 2F30.20)') grid(((i-1)*glength2)+j,1),&
                      grid(((i-1)*glength2)+j,2),srf(i,j,1,1),darr(i,j)
            write(2,'(2I5.1, 2F30.20)') grid(((i-1)*glength2)+j,1),&
                      grid(((i-1)*glength2)+j,2),srf(i,j,2,2),darr(i,j)
            write(3,'(2I5.1, 2F30.20)') grid(((i-1)*glength2)+j,1),&
                      grid(((i-1)*glength2)+j,2),asrf(i,j,1),darr(i,j)
            write(4,'(2I5.1, 2F30.20)') grid(((i-1)*glength2)+j,1),&
                      grid(((i-1)*glength2)+j,2),asrf(i,j,2),darr(i,j)
            write(5,'(2I5.1, 2F30.20)') grid(((i-1)*glength2)+j,1),&
                      grid(((i-1)*glength2)+j,2),srf(i,j,1,2),darr(i,j)
            if (((i-1)*glength2)+j+1.lt.glength1*glength2) then
               if (grid(((i-1)*glength2)+j+1,1).ne.xco) then
                  write(1,*)
                  write(2,*)
                  write(3,*)
                  write(4,*)
                  write(5,*)
                  xco=grid(((i-1)*glength2)+j+1,1) 
               endif
            endif   
         enddo
      enddo 

      do i=1,5
         close(i)
      enddo

      if (lpltraj) then
         call settraj
         call gettimes(ntimes)
         allocate(gpops(ntimes,nddstate,gwpdim(1,1)))
         call getgwpweights(ntimes,gpops)
 
         allocate(times(ntimes))
         allocate(qcent(ntimes,nspfdof(1),gwpdim(1,1)))
         allocate(pcent(ntimes,nspfdof(1),gwpdim(1,1)))
         allocate(dtrajp(ntimes,gwpdim(1,1),4))
         allocate(adtrajp(ntimes,gwpdim(1,1),4))
         allocate(grddists(ntimes,gwpdim(1,1)))
         allocate(dgtraj(ntimes,gwpdim(1,1),2*glength1-1,2*glength2-1,nddstate))
         allocate(adgtraj(ntimes,gwpdim(1,1),2*glength1-1,2*glength2-1,nddstate))
         allocate(ggrid(2*glength1-1,2*glength2-1,2))

         call rdtraj(ntimes,times,qcent,pcent) 

         intv=0.5d0
         
         do i=1,ntimes
            do j=1,gwpdim(1,1)
               if (plgwp(j)) then
                  !
                  ! gnum used to determine whether ggrid has been populated by 
                  ! call to calcgauss2d - prevents setting grid twice (efficiency)
                  !
                  call gdist(qcent(i,:,j),srf,asrf,dgridp,adgridp,coords,grddist)
                  dtrajp(i,j,:)=dgridp
                  x=int(dtrajp(i,j,1))
                  y=int(dtrajp(i,j,2))
                  dtrajp(i,j,1)=grid(((x-1)*glength2)+y,1)
                  dtrajp(i,j,2)=grid(((x-1)*glength2)+y,2)
                  
                  if (lgauss) then
                     gnum=0
                     call calcgauss2d(lg1,lg2,glength1,glength2,&
                                      gpops(i,:,j),0.2d0,dtrajp(i,j,:),&
                                      dgtraj(i,j,:,:,:),ggrid(:,:,:),gnum,intv) 
                     gnum=1
                  endif  
             
                  adtrajp(i,j,:)=adgridp
                  x=int(adtrajp(i,j,1))
                  y=int(adtrajp(i,j,2))
                  adtrajp(i,j,1)=grid(((x-1)*glength2)+y,1)
                  adtrajp(i,j,2)=grid(((x-1)*glength2)+y,2)
 
                  if (lgauss) then
                     call calcgauss2d(lg1,lg2,glength1,glength2,&
                                      gpops(i,:,j),0.2d0,adtrajp(i,j,:),&
                                      adgtraj(i,j,:,:,:),ggrid(:,:,:),gnum,intv)
                  endif            
   
                  grddists(i,j)=grddist
               endif
            enddo
         enddo

         do i=1,gwpdim(1,1)
            if (plgwp(i)) then
               ldiab=.true.
               call wrtrajgwp(i,dtrajp,adtrajp,ldiab,times,grddists,tsteps,gpops)
               if (lgauss) then
                  call wrgtraj(i,dgtraj,ldiab,times,grddists,tsteps,ggrid,&
                               srf,asrf,intv)
               endif

               ldiab=.false.
               call wrtrajgwp(i,dtrajp,adtrajp,ldiab,times,grddists,tsteps,gpops)
               if (lgauss) then
                  call wrgtraj(i,adgtraj,ldiab,times,grddists,tsteps,ggrid,&
                               srf,asrf,intv)
               endif
            endif
         enddo

         ldiab=.true.
         lanim=.true.
         if (lgauss) then
            call wrgauss(ldiab,lanim,plgwp)
         else
            call wrgnugwp(ldiab,lanim,plgwp)
         endif
         lanim=.false.
         if (lgauss) then
            call wrgauss(ldiab,lanim,plgwp)
         else
            call wrgnugwp(ldiab,lanim,plgwp)
         endif
         ldiab=.false.
         lanim=.true.
         if (lgauss) then
            call wrgauss(ldiab,lanim,plgwp)
         else
            call wrgnugwp(ldiab,lanim,plgwp)
         endif
         lanim=.false.
         if (lgauss) then
            call wrgauss(ldiab,lanim,plgwp)
         else
            call wrgnugwp(ldiab,lanim,plgwp)
         endif

      else
         ldiab=.true.
         call wrgnu(ldiab)
         ldiab=.false.
         call wrgnu(ldiab)
      endif
      
      call wrgnucoup

      end subroutine pesdiab

!#######################################################################

      subroutine usrinput(lg1,rg1,lg2,rg2,glength1,glength2,sca1,sca2,&
                          lpltraj,lgauss,plgwp,cut1,cut2,nsta,tsteps)

      implicit none

      integer(long)                                   :: i,c,mode,&
                                                         gwpindx,opt
      integer(long), intent(out)                      :: lg1,rg1,lg2,rg2,&
                                                         glength1,glength2,&
                                                         nsta,tsteps

      real(dop)                                       :: sc1,sc2
      real(dop), intent(out)                          :: sca1,sca2
      real(dop), dimension(nspfdof(1)),intent(out)    :: cut1,cut2

      logical(kind=4)                                 :: ex,llog
      logical(kind=4), intent(out)                    :: lpltraj,lgauss
      logical(kind=4), dimension(gwpdim(1,1)),intent(out) :: plgwp

      character(len=150)                              :: iline,lline,dline,buf,&
                                                         inpfile,flog

      i=1
      call mygetarg(i,buf)
      do while (buf(1:1).eq.'-')
         if (buf(1:2).eq.'-h') then
            opt=1
            call wroptions(opt)
            stop
         else if (buf(1:2).eq.'-i') then
            i=i+1
            call mygetarg(i,buf)
            inpfile=buf
         else if (buf(1:4).eq.'-ver') then
            call wrversion(6)
            stop
         else if (buf(1:4).eq.'-ex1') then
            opt=2
            call wroptions(opt)
            stop
         else if (buf(1:4).eq.'-ex2') then
            opt=3
            call wroptions(opt)
            stop
         endif
         i=i+1
         call mygetarg(i,buf)
      enddo

      call system('mkdir -p ddpesout')

      inquire(file=inpfile, exist=ex)

      if (ex) then
         open(1,file=inpfile,form='formatted',status='old')
      else
         STOP "Input file not provided"
      endif

      mode=0
      lg1=0
      rg1=0
      lg2=0
      rg2=0
      sca1=1.0d0
      sca2=1.0d0
      tsteps=10
      lpltraj=.false.
      plgwp=.false.
      lgauss=.false.
      cut1=0.0d0
      cut2=0.0d0
      llog=.false.
      flog="log"
      lrediab=.false.

      do
         read(1,'(a)',end=10) iline

         if (iline(1:5).eq.'cuts1') then
            read(iline(8:16),*) mode
            read(iline(17:),*) cut1(mode)

         else if (iline(1:5).eq.'cuts2') then
            read(iline(8:16),*) mode
            read(iline(17:),*) cut2(mode)

         else if (iline(1:3).eq.'lg1') then
            read(iline(6:),*) lg1

         else if (iline(1:3).eq.'rg1') then
            read(iline(6:),*) rg1

         else if (iline(1:3).eq.'lg2') then
            read(iline(6:),*) lg2

         else if (iline(1:3).eq.'rg2') then
            read(iline(6:),*) rg2

         else if (iline(1:7).eq.'nstates') then
            read(iline(10:),*) nsta

         else if (iline(1:6).eq.'scale1') then
            read(iline(9:),*) sca1

         else if (iline(1:6).eq.'scale2') then
            read(iline(9:),*) sca2

         else if (iline(1:5).eq.'scale') then
            write(*,*) "Keyword 'scale' is redundant: please specify &
                        &'scale1' and 'scale2' independently"
            stop

! keyword 'log' gets dc and gd vectors from log file

         else if (iline(1:7).eq.'pathlog') then
            flog=trim(adjustl(iline(10:)))

         else if (iline(1:3).eq.'log') then
            llog=.true.

         else if (iline(1:8).eq.'plottraj') then
            lpltraj=.true.

         else if (iline(1:9).eq.'trajsteps') then
            read(iline(12:),*) tsteps

         else if (iline(1:6).eq.'gwpall') then
            plgwp=.true.

         else if (iline(1:4).eq.'gwp ') then
            read(iline(5:),*) gwpindx
            plgwp(gwpindx)=.true.

         else if (iline(1:9).eq.'gaussians') then
            lgauss=.true.

         else if (iline(1:5).eq.'rawad') then
            lrediab=.true.

         else if ((iline.ne.'').and.(iline(1:1).ne.'#')) then
            write(*,*) "ERROR: Keyword '"//trim(iline(1:))//"' is not recognised"

         endif

      enddo

   10 close(1)

      if (llog) then
         open(2,file=flog,form='formatted',status='old')
         do
            read(2,'(a)',end=11) lline
            if (lline(40:47).eq.'Der Coup') then
               c=1
               do
                  read(2,'(a)',end=11) dline
                  read(dline(40:50),*) cut1(c)
                  read(dline(60:70),*) cut2(c)
                  if (c.eq.nspfdof(1)) exit
                  c=c+1
               enddo
               exit
            endif
         enddo
      endif

   11 close(2)

      sc1=0.0d0
      sc2=0.0d0
      do i=1,nspfdof(1)
         sc1=sc1+abs(cut1(i))
         sc2=sc2+abs(cut2(i))
      enddo

      if ((sc1.eq.0.0d0).and.(sc2.eq.0.0d0)) then
         if (llog) then
            write(*,*) "ERROR: Vectors are zero vectors &
                  &- does the log file contain the dc and gd vectors?"
            stop
         else
            write(*,*) "ERROR: No vector input detected in input file."
            stop
         endif
      endif

      glength1=lg1+rg1+1
      glength2=lg2+rg2+1

      if ((glength1.le.0).or.(glength2.le.0)) then
         write(*,*) "ERROR: Error reading gridlength - choose positive &
                     &integers for lg1/2 and rg1/2"
         stop
      endif

      if (DOT_PRODUCT(cut1,cut2).gt.1.0d-8) then
         write(*,*) "ERROR: Input vectors are not orthogonal. Dot product &
                     &greater than threshold."
         stop
      endif

      return

      end subroutine


!#######################################################################

      subroutine settraj()

      implicit none

      integer(long)                                     :: chkdvr,chkgrd,&
                                                           chkpsi,chkdat
      logical(kind=4)                                   :: lerr

      open(ipsi,file='psi',form='unformatted',status='old')
      
      call rdmemdim(ipsi)

      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef

      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      read(ipsi) filever(ipsi)
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      
      chkdvr=2
      open(idvr,file='dvr',form='unformatted',status='old')
      call dvrinfo(lerr,chkdvr)
      close(idvr)
      
      lpsi=.true.
      call zeigausw
      call memausw
      
      end subroutine settraj

!#######################################################################

      subroutine rdtraj(ntimes,times,qcenters,pcenters)

      use rddvrmod
      use rdfilesmod, only: rdpsi
      use gwplib, only: gwp_param,gwp_cmom

      implicit none

      integer(long)                                     :: ndata,&
                                                           psicount,chkdvr,&
                                                           chkgrd,chkpsi,&
                                                           chkdat
      integer(long), intent(in)                         :: ntimes
      integer(long), dimension(gwpinfodim)              :: trajst
      integer(long), dimension(adim)                    :: jindx
      integer(long), dimension(gwpinfodim)              :: gwpdep

      real(dop)                                         :: time
      real(dop), dimension(qcentdim)                    :: qcenter,pcenter
      real(dop), dimension(ntimes), intent(out)         :: times
      real(dop), dimension(ntimes,nspfdof(1),gwpdim(1,1)), intent(out) :: qcenters,pcenters

      complex(dop), dimension(dgldim)                   :: psi
      complex(sip), dimension(spsidim)                  :: spsi
      complex(dop), dimension(agmatdim)                 :: agmat
      complex(dop), dimension(gsmatdim)                 :: adgwp
      complex(dop), dimension(gfgwpdim)                 :: gfgwp
      complex(dop), dimension(aigwpdim)                 :: aigwp
      complex(dop), dimension(zgwpdim)                  :: zgwp

      ncut=999999
      nskip=0
      
      time=(tinit-out2)/fs
      ndata=1
      rewind(ipsi)
      chkdvr=0
      chkgrd=0
      chkpsi=0
      chkdat=0
      psicount=-nskip
      read(ipsi) filever(ipsi)
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      
      do while (ndata.le.ntimes)
         call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
         if (lend) goto 14
         psicount=psicount+1
         time=time+out2/fs
         call gwp_param(psi,aGmat,gfgwp,aigwp,zgwp,qcenter,1)
         call gwp_cmom(psi,pcenter)
         times(ndata)=time
         call septraj(qcenter(:),qcenters(ndata,:,:))
         call septraj(pcenter(:),pcenters(ndata,:,:))
         ndata=ndata+1
      enddo

   14 return

      end subroutine rdtraj
      
!#######################################################################

      subroutine gettimes(ntimes)

      implicit none

      integer(long), intent(out)                        :: ntimes

      real(dop)                                         :: maxtime

      character(len=150)                                :: iline

       open(1,file='output',form='formatted',status='old')

      do
         read(1,'(a)',end=13) iline
         if (iline(2:5).eq.'Time') then
            read(iline(9:19),*) maxtime
         endif
      enddo
   13 close(1)

      ntimes=int(maxtime/(tinit+out2/fs))+1

      return

      end subroutine gettimes

!#######################################################################

      subroutine septraj(arr,arr1)

      implicit none

      real(dop), dimension(nspfdof(1),gwpdim(1,1)), intent(in) :: arr
      real(dop), dimension(nspfdof(1),gwpdim(1,1)), intent(out) :: arr1

      arr1=arr
      
      return

      end subroutine septraj

!#######################################################################

      subroutine gdist(npoint,surf,adsurf,dgridp,adgridp,geom,rmin)

      implicit none

      integer(long)                                     :: ax1,ax2,i,j
      integer(long), dimension(4)                       :: surfdim

      real(dop), dimension(nspfdof(1)), intent(in)      :: npoint
      real(dop), dimension(:,:,:,:), intent(in)         :: surf
      real(dop), dimension(:,:,:), intent(in)           :: adsurf
      real(dop), dimension(:,:,:), intent(in)           :: geom
      real(dop), dimension(ndofddpes)                   :: xnpoint,xgeom,dvec
      real(dop), dimension(4), intent(out)              :: dgridp,adgridp
      real(dop), intent(out)                            :: rmin
      real(dop)                                         :: r

      real(dop)                                         :: dnrm2 !BLAS norm func

      surfdim=shape(surf)
      ax1=surfdim(1)
      ax2=surfdim(2)

      rmin=999.0_dop
      do i=1,ax1
         do j=1,ax2
            call ddq2x(npoint,xnpoint)
            call ddq2x(geom(i,j,:),xgeom)
            dvec = xnpoint-xgeom
            r=dnrm2(ndofddpes,dvec,1)
            if (r.lt.rmin) then
               rmin=r
               dgridp(:)=(/dble(i),dble(j),surf(i,j,1,1),surf(i,j,2,2)/)
               adgridp(:)=(/dble(i),dble(j),adsurf(i,j,1),adsurf(i,j,2)/)
            endif
         enddo
      enddo

      return

      end subroutine

!#######################################################################

      subroutine wrtrajgwp(ngwp,dtraj,adtraj,ldiab,times,distar,steps,gpops)

      implicit none

      integer(long), intent(in)                         :: ngwp,steps
      integer(long)                                     :: i,nrec,c
      integer(long), dimension(3)                       :: tshape

      real(dop), dimension(:,:,:), intent(in)           :: dtraj,adtraj
      real(dop), dimension(:,:), intent(in)             :: distar
      real(dop), dimension(:), intent(in)               :: times
      real(dop), dimension(:,:,:), intent(in)           :: gpops

      character(len=150)                                :: gwpfile
      logical(kind=4), intent(in)                       :: ldiab

      if (ldiab) then
         write(gwpfile,'(A6,I0.2)') "dplgwp",ngwp
      else
         write(gwpfile,'(A7,I0.2)') "adplgwp",ngwp
      endif

      gwpfile=trim(gwpfile)

      tshape=shape(dtraj)
      nrec=tshape(1)

      open(1,file='./ddpesout/'//gwpfile,form='formatted',status='replace')

      c=0
      do i=1,nrec
         if ((c.eq.steps).or.(c.eq.0)) then
            if (ldiab) then
               write(1,'(8F30.20)') anint(times(i)*100.0)/100.0,dtraj(i,ngwp,:),distar(i,ngwp),&
                                    10.0d0*gpops(i,1,ngwp),10.0d0*gpops(i,2,ngwp)
            else
               write(1,'(8F30.20)') anint(times(i)*100.0)/100.0,adtraj(i,ngwp,:),distar(i,ngwp),&
                                    10.0d0*gpops(i,1,ngwp),10.0d0*gpops(i,2,ngwp)
            endif
            write(1,*) " "
            write(1,*) " "
            c=1
         else
            c=c+1
         endif
      enddo

      close(1)

      end subroutine wrtrajgwp

!#######################################################################

      subroutine calcgauss2d(lg1,lg2,gl1,gl2,&
                             ggp,gwidth,gcent,gwp,ggrid,&
                             n,intv)

      implicit none

      integer(long)                                     :: i,j,k
      integer(long), intent(in)                         :: lg1,lg2,&
                                                           gl1,gl2,n

      real(dop)                                         :: gauss
      real(dop), dimension(2*gl1-1,2*gl2-1,2), intent(inout) :: ggrid
      real(dop), dimension(2*gl1-1,2*gl2-1,nddstate), intent(out) :: gwp
      real(dop), intent(in)                             :: gwidth,intv
      real(dop), dimension(nddstate), intent(in)               :: ggp,gcent
      
      if (n.eq.0) then
         i=-lg1
         j=-lg2

         do i=1,2*gl1-1
            do j=1,2*gl2-1
               ggrid(i,j,1)=-dble(lg1)+dble((i-1))*intv
               ggrid(i,j,2)=-dble(lg2)+dble((j-1))*intv
            enddo
         enddo
      endif

      do i=1,2*gl1-1
         do j=1,2*gl2-1
            do k=1,nddstate
               call gauss2d(dble(ggrid(i,j,1)),gcent(1),&
                            dble(ggrid(i,j,2)),gcent(2),&
                            ggp(k),gwidth,gauss)
               gwp(i,j,k)=gauss
            enddo
         enddo
      enddo

      return

      end subroutine calcgauss2d

!#######################################################################

      subroutine gauss2d(x,x0,y,y0,amp,sigma,gauss)

      implicit none

      real(dop), intent(in)                             :: x,x0,y,y0,amp,sigma
      real(dop), intent(out)                            :: gauss
      
      gauss=amp*exp(-(((x-x0)**2)+((y-y0)**2))/(2*sigma**2))

      return

      end subroutine gauss2d

!#######################################################################

      subroutine wrgtraj(ngwp,gtraj,ldiab,times,distar,steps,&
                         ggrid,dsurf,adsurf,intv)

      implicit none

      integer(long), intent(in)                         :: ngwp,steps
      integer(long)                                     :: i,j,k,nrec,&
                                                           tgl1,tgl2,c
      integer(long), dimension(5)                       :: tshape

      real(dop), dimension(:,:,:), intent(in)           :: ggrid,adsurf
      real(dop), dimension(:,:,:,:), intent(in)         :: dsurf
      real(dop), dimension(:,:,:,:,:), intent(in)       :: gtraj
      real(dop), dimension(:,:), intent(in)             :: distar
      real(dop), dimension(:), intent(in)               :: times
      real(dop), intent(in)                             :: intv

      character(len=150)                                :: gwpfile
      logical(kind=4), intent(in)                       :: ldiab
      logical(kind=4)                                   :: ltraj

      if (ldiab) then
         write(gwpfile,'(A7,I0.2)') "dgplgwp",ngwp
      else
         write(gwpfile,'(A8,I0.2)') "adgplgwp",ngwp
      endif

      gwpfile=trim(gwpfile)

      tshape=shape(gtraj)
      nrec=tshape(1)
      tgl1=tshape(3)
      tgl2=tshape(4)

      ltraj=.false.

      open(1,file='./ddpesout/'//gwpfile,form='formatted',status='replace')
      
      c=-1
      do i=1,nrec
         c=c+1
         if (mod(c,steps).ne.0) cycle
         do j=1,tgl1
            do k=1,tgl2
               if (gtraj(i,ngwp,j,k,1).gt.1.0d-10.or.gtraj(i,ngwp,j,k,2).gt.1.0d-10) then
                  if (ldiab) then
                     write(1,'(6F30.20)') anint(times(i)*100.0)/100.0,&
                       ggrid(j,k,1),ggrid(j,k,2),&
                       1.0d0*abs(gtraj(i,ngwp,j,k,1))+&
                       dsurf(nint(intv*j),nint(intv*k),1,1),&
                       -1.0d0*abs(gtraj(i,ngwp,j,k,2))+&
                       dsurf(nint(intv*j),nint(intv*k),2,2),&
                       distar(i,ngwp)
                  else
                     write(1,'(6F30.20)') anint(times(i)*100.0)/100.0,&
                       ggrid(j,k,1),ggrid(j,k,2),&
                       abs(gtraj(i,ngwp,j,k,1)+adsurf(nint(intv*j),nint(intv*k),1)),&
                       abs(gtraj(i,ngwp,j,k,2)+adsurf(nint(intv*j),nint(intv*k),2)),&
                       distar(i,ngwp)
                  endif
                  ltraj=.true.
               endif
            enddo
         enddo
         if (.not.ltraj) then
            write(1,'(6F30.20)') anint(times(i)*100.0)/100.0,0.0d0,0.0d0,0.0d0,&
              0.0d0,0.0d0
         endif
         write(1,*)
         write(1,*)
         ltraj=.false.
      enddo

      close(1)

      end subroutine wrgtraj

!#######################################################################

      subroutine getgwpweights(ntimes,gpops)
 
      use rddvrmod
      use rdfilesmod, only: rdpsi
      use iofile, only: rdpsiinfo
      use gwplib, only: gwp_param, gwp_overlap

      implicit none

      integer(long)                                     :: chkdvr,chkgrd,&
                                                           chkpsi,chkdat,gwpdim1,&
                                                           gs2dim2,&
                                                           i,j,k,k1,k2
      integer(long), intent(in)                         :: ntimes
      integer(long), dimension(gwpinfodim)              :: trajst
      integer(long), dimension(adim)                    :: jindx
      integer(long), dimension(gwpinfodim)              :: gwpdep

      real(dop), dimension(ntimes,nddstate,gwpdim(1,1)), & 
                                            intent(out) :: gpops
      real(dop), dimension(qcentdim)                    :: qcenter
      real(dop), dimension(gwpinfodim)                  :: gsval

      complex(dop), dimension(dgldim)                   :: psi
      complex(sip), dimension(spsidim)                  :: spsi
      complex(dop), dimension(agmatdim)                 :: agmat
      complex(dop), dimension(gsmatdim)                 :: gsinv,adgwp
      complex(dop), dimension(gfgwpdim)                 :: gfgwp
      complex(dop), dimension(aigwpdim)                 :: aigwp
      complex(dop), dimension(zgwpdim)                  :: zgwp
      complex(dop), dimension(:,:,:,:), allocatable     :: gs2
      complex(dop), dimension(gsmatdim)                 :: gsvek
      complex(dop), dimension(gwpdim(1,1))              :: w

      gpops=0.0d0

      gwpdim1=gwpdim(1,1)

      gs2dim2=nspfdof(1)+1

      allocate(gs2(gwpdim1,gwpdim1,gs2dim2,gs2dim2))
      
      gs2=0.0d0

      open(ipsi,file='./psi',form='unformatted',status='old')

      call rdmemdim(ipsi)
 
      read(ipsi) filever(ipsi)

      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1

      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      
      do i=1,ntimes
         call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
         call gwp_param(psi,aGmat,gfgwp,aigwp,zgwp,qcenter,1)
         call gwp_overlap(gs2,gsinv,gsval,gsvek,gfgwp,aigwp,zgwp,3)
         do j=1,nddstate
            do k=1,gwpdim(1,1)
               w(k)=psi(k+(j-1)*gwpdim(1,1))
            enddo
            do k1=1,gwpdim(1,1)
               do k2=1,gwpdim(1,1)
                  gpops(i,j,k1)=gpops(i,j,k1)+&
                        dble(conjg(w(k2))*w(k1)*gs2(k2,k1,1,1)) !gs2 index
               enddo
            enddo
         enddo
      enddo

      return

      end subroutine getgwpweights

!#######################################################################

      subroutine getraw

      implicit none

      integer(long)  :: i,f,f1,s

      dbener = 0.0_dop
      dbgrad = 0.0_dop
      dbhess = 0.0_dop
      do i=1,dbnrec
         do s=1,nddstate
            dbener(s,s,i) = dbadener(s,s,i)

            do f=1,ndofddpes
               dbgrad(f,s,s,i) = dbadgrad(f,s,s,i)

               do f1=1,ndofddpes
                  dbhess(f1,f,s,s,i) = dbadhess(f1,f,s,s,i)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine getraw

!#######################################################################

      subroutine wrgnu(ldiab)

      implicit none

      character(len=150)                                :: filnam

      logical(kind=4), intent(in)                       :: ldiab

      if (ldiab) then
         filnam='pesd.p'
      else
         filnam='pesad.p'
      endif

      open(1,file=filnam,form='formatted',status='replace')

      write(1,*) "# Gnuplot script file for plotting DD-vMCG PES"
      write(1,*) "# - written for Gnuplot 5.0.0"
      write(1,*) "set style fill transparent solid 0.60 border"

      if (ldiab) then
         write(1,*) "stats './ddpesout/pesd1' using 4 name 'PES1' nooutput"
         write(1,*) "stats './ddpesout/pesd2' using 4 name 'PES2' nooutput"
      else
         write(1,*) "stats './ddpesout/pesad1' using 4 name 'PES1' nooutput"
         write(1,*) "stats './ddpesout/pesad2' using 4 name 'PES2' nooutput"
      endif

      write(1,*) "set cbrange[PES1_min:PES1_max+PES1_min+PES2_max+PES2_min]"
      write(1,*) "set format cb '%.2f'"
      write(1,*) "unset key"

      write(1,*) "set palette defined (PES1_min 'black',\"
      write(1,*) "            0.33*PES1_max+PES1_min 'red',\"
      write(1,*) "            0.66*PES1_max+PES1_min 'yellow',\"
      write(1,*) "            PES1_max+PES1_min 'white',\" 
      write(1,*) "            PES2_min+PES1_max+PES1_min 'white',\"
      write(1,*) "            PES2_min+PES1_max+PES1_min 'black',\"
      write(1,*) "            0.33*PES2_max+PES2_min+PES1_max+PES1_min 'magenta',\" 
      write(1,*) "            0.66*PES2_max+PES2_min+PES1_max+PES1_min 'blue',\"
      write(1,*) "            PES2_max+PES2_min+PES1_max+PES1_min 'white')"

      write(1,*) "set pm3d depthorder border linetype -1 linewidth 0.4"
      
      if (ldiab) then
         write(1,*) "splot './ddpesout/pesd1' with pm3d palette,\" 
         write(1,*) "      './ddpesout/pesd2' u 1:2:3:($4+PES2_min+PES1_max+PES1_min) &
                    &with pm3d palette"
      else
         write(1,*) "splot './ddpesout/pesad1' with pm3d palette,\"
         write(1,*) "      './ddpesout/pesad2' u 1:2:3:($4+PES2_min+PES1_max+PES1_min) &
                     &with pm3d palette"
      endif

      close(1)

      end subroutine wrgnu

!#######################################################################

      subroutine wrgnucoup()

      implicit none

      character(len=150)                                :: filnam

      filnam='pesdcoup.p'

      open(1,file=filnam,form='formatted',status='replace')

      write(1,*) "# Gnuplot script file for plotting DD-vMCG diabatic coupling"
      write(1,*) "# - written for Gnuplot 5.0.0"
      write(1,*) "set style fill transparent solid 0.60 border"

      write(1,*) "set pm3d depthorder border linetype -1 linewidth 0.4"

      write(1,*) "splot './ddpesout/pesd_coup' u 1:2:3 with pm3d"
      
      close(1)

      end subroutine wrgnucoup

!#######################################################################

      subroutine wrgauss(ldiab,lanim,pgwp)

      implicit none

      integer(long)                                     :: i,fgwp,nplt

      logical(kind=4), intent(in)                       :: ldiab,lanim
      logical(kind=4), dimension(gwpdim(1,1)),intent(in) :: pgwp

      character(len=150)                                :: gfile
      character(len=9), dimension(48)                   :: colour = &
         [        '#000000FF','#00FF0000','#00006400','#00D2691E','#009932CC',&
                 &'#00E9967A','#00696969','#0000BFFF','#00FFD700','#004B0082','#00ADFF2F',&
                 &'#00CD5C5C','#00E6E6FA','#00ADD8E6','#0020B2AA','#00FF00FF','#00191970',&
                 &'#00808000','#00FF4500','#00FFDAB9','#00DDA0DD','#00B0E0E6','#00F4A460',&
                 &'#00C0C0C0','#00008080','#0040E0D0','#00EE82EE','#00FFFF00','#00FF6347',&
                 &'#0000FF7F','#008B4513','#00BC8F8F','#00800080','#00FFA500','#00FFDEAD',&
                 &'#00C71585','#00800000','#007CFC00','#00008000','#00B22222','#00FF1493',&
                 &'#008FBC8F','#0000FFFF','#00FF7F50','#005F9EA0','#00000000','#007FFFD4',&
                 &'#00A52A2A']
 
      if (ldiab) then
         open(1,file='pesd.p',form='formatted',status='replace')
      else
         open(1,file='pesad.p',form='formatted',status='replace')
      endif

      write(1,*) "# Gnuplot script file for plotting DD-vMCG PES"
      write(1,*) "# - written for Gnuplot 5.0.0"

      write(1,*) "set print '-'"
      write(1,*) "GWP_min=100"
      write(1,*) "GWP_max=0"

      do i=1,gwpdim(1,1)
         gfile=''
         if (ldiab.and.pgwp(i)) then
            write(gfile,'(A7,I0.2)') "dgplgwp",i
         else if (pgwp(i)) then
            write(gfile,'(A8,I0.2)') "adgplgwp",i
         endif
         gfile=trim(gfile)
         if (gfile.ne.'') then
            fgwp=i
            if (ldiab) then
               write(1,'(A19,A9,A37,I0.2,A10)') " stats './ddpesout/",gfile,&
                       "' using (abs($4)):(abs($5)) name 'GWP",i,"' nooutput"
            else
               write(1,'(A19,A10,A37,I0.2,A10)') " stats './ddpesout/",gfile,&
                       "' using (abs($4)):(abs($5)) name 'GWP",i,"' nooutput"
            endif
            write(1,'(A8,I0.2,A30,I0.2,A7)') " if (GWP",i,"_min_x < GWP_min) &
                       &{GWP_min=GWP",i,"_min_x}"
            write(1,'(A8,I0.2,A30,I0.2,A7)') " if (GWP",i,"_min_y < GWP_min) &
                       &{GWP_min=GWP",i,"_min_y}"
            write(1,'(A8,I0.2,A30,I0.2,A7)') " if (GWP",i,"_max_x > GWP_max) &
                       &{GWP_max=GWP",i,"_max_x}"
            write(1,'(A8,I0.2,A30,I0.2,A7)') " if (GWP",i,"_max_y > GWP_max) &
                       &{GWP_max=GWP",i,"_max_y}"
         endif
      enddo

      if (ldiab) then
         write(1,*) "stats './ddpesout/pesd1' using 4 name 'PES1' nooutput"
         write(1,*) "stats './ddpesout/pesd2' using 4 name 'PES2' nooutput"
      else
         write(1,*) "stats './ddpesout/pesad1' using 4 name 'PES1' nooutput"
         write(1,*) "stats './ddpesout/pesad2' using 4 name 'PES2' nooutput"
      endif

      write(1,*) "set cbrange[PES1_min:PES1_max+PES1_min+PES2_max+PES2_min+&
                      &GWP_max+GWP_min]"
      write(1,*) "set format cb '%.2f'"
      write(1,*) "set style fill transparent solid 0.60 border"

      write(1,*) "set palette defined (PES1_min 'black',\"
      write(1,*) "            0.33*PES1_max+PES1_min 'red',\"
      write(1,*) "            0.66*PES1_max+PES1_min 'yellow',\"
      write(1,*) "            PES1_max+PES1_min 'white',\"
      write(1,*) "            PES2_min+PES1_max+PES1_min 'white',\"
      write(1,*) "            PES2_min+PES1_max+PES1_min 'black',\"
      write(1,*) "            0.33*PES2_max+PES2_min+PES1_max+PES1_min 'magenta',\"
      write(1,*) "            0.66*PES2_max+PES2_min+PES1_max+PES1_min 'blue',\"
      write(1,*) "            PES2_max+PES2_min+PES1_max+PES1_min 'white',\"
      write(1,*) "            PES1_max+PES1_min+PES2_max+PES2_min+GWP_min 'white',\"
      write(1,*) "            PES1_max+PES1_min+PES2_max+PES2_min+GWP_min 'blue',\"
      write(1,*) "            PES1_max+PES1_min+PES2_max+PES2_min+GWP_max+GWP_min &
                              &'green')"

      write(1,*) "set pm3d explicit depthorder border linetype -1 linewidth 0.4"
      write(1,*) "unset key"

      if (ldiab) then
         write(1,*) "stats './ddpesout/pesd1' using 3 name 'PES1e' nooutput"
         write(1,*) "stats './ddpesout/pesd2' using 3 name 'PES2e' nooutput"
      else
         write(1,*) "stats './ddpesout/pesad1' using 3 name 'PES1e' nooutput"
         write(1,*) "stats './ddpesout/pesad2' using 3 name 'PES2e' nooutput"
      endif

      write(1,*) "if (PES1e_max >= PES2e_max) {"
      write(1,*) "   PES_max=PES1e_max"
      write(1,*) "} else {"
      write(1,*) "   PES_max=PES2e_max"
      write(1,*) "}"
      write(1,*) "if (PES1e_min <= PES2e_min) {"
      write(1,*) "   PES_min=PES1e_min"
      write(1,*) "} else {"
      write(1,*) "   PES_min=PES2e_min"
      write(1,*) "}"

      write(1,*) "set zrange [PES_min-0.1:PES_max]"
      write(1,*) "set format z '%.2f'"

      if (lanim) then
         if (ldiab) then
            write(1,*) "system('mkdir -p animationd')"
         else
            write(1,*) "system('mkdir -p animationad')"
         endif
      endif

      write(1,'(A20,I0.2,A13)') " do for [i=0:int(GWP",fgwp,"_blocks)-1] {"

      if (lanim) then
         if (ldiab) then
            write(1,*) "outfile=sprintf('animationd/time%02.0f.png',i)"
         else
            write(1,*) "outfile=sprintf('animationad/time%02.0f.png',i)"
         endif
         write(1,*) "set output outfile"
      else
         write(1,*) "   print i"
      endif

      write(1,*) "   set dgrid3d 25,25 gauss 0.4,0.4"
      write(1,*) "   set xrange [GWP_min:GWP_max]"
      write(1,*) "   set yrange [GWP_min:GWP_max]"

      nplt=1
      do i=1,gwpdim(1,1)
         gfile=''
         if (ldiab.and.pgwp(i)) then
            write(gfile,'(A7,I0.2)') "dgplgwp",i
         else if (pgwp(i)) then
            write(gfile,'(A8,I0.2)') "adgplgwp",i
         endif
         if (gfile.ne.'') then
            if (ldiab) then
               write(1,'(A19,I0.2,A5)') "   set table 'data",nplt,".dat'"
               write(1,'(A22,A9,A17)') "    splot './ddpesout/",gfile,"' index i u &
                    &2:3:4"
               write(1,*) "   unset table"
               nplt=nplt+1
               write(1,'(A19,I0.2,A5)') "   set table 'data",nplt,".dat'"
               write(1,'(A22,A9,A17)') "    splot './ddpesout/",gfile,"' index i u &
                    &2:3:5"
               write(1,*) "   unset table"
               nplt=nplt+1
            else
               write(1,'(A19,I0.2,A5)') "    set table 'data",nplt,".dat'"
               write(1,'(A22,A10,A17)') "    splot './ddpesout/",gfile,"' index i u &
                    &2:3:4"
               write(1,*) "   unset table"
               nplt=nplt+1
               write(1,'(A19,I0.2,A5)') "    set table 'data",nplt,".dat'"
               write(1,'(A22,A10,A17)') "    splot './ddpesout/",gfile,"' index i u &
                    &2:3:5"
               write(1,*) "   unset table"
               nplt=nplt+1
            endif
         endif
      enddo

      write(1,*) "   unset xrange"
      write(1,*) "   unset yrange"
      write(1,*) "   unset dgrid3d"

      do i=1,nplt-1
         if (i.eq.1) then
            write(1,'(A15,I0.2,A18,A9,A3)') "    splot 'data",i,".dat' w l lc rgb '",&
                                       &trim(colour(1)),"',\"
         else
            if (mod(i,2).eq.0) then
               write(1,'(A15,I0.2,A18,A9,A3)') "          'data",i,".dat' w l lc rgb '",&
                                       &colour(int(i/2)),"',\"
            else
               write(1,'(A15,I0.2,A18,A9,A3)') "          'data",i,".dat' w l lc rgb '",&
                                       &colour(int(i-((i-1)/2))),"',\"
            endif
         endif
      enddo

      if (ldiab) then
         write(1,*) "         './ddpesout/pesd1' with pm3d palette,\"
         write(1,*) "         './ddpesout/pesd2' u 1:2:3:($4+PES1_max+PES1_min+PES2_min) &
                               &with pm3d palette"
      else
         write(1,*) "         './ddpesout/pesad1' with pm3d palette,\"
         write(1,*) "         './ddpesout/pesad2' u 1:2:3:($4+PES1_max+PES1_min+PES2_min) &
                               &with pm3d palette"
      endif

      if (.not.lanim) write(1,*) "   pause -1"

      write(1,*) "}"

      close(1)
 
      end subroutine wrgauss

!#######################################################################

      subroutine wrgnugwp(ldiab,lanim,pgwp)

      implicit none

      integer(long)                                     :: i,nplt,fgwp

      logical(kind=4), intent(in)                       :: ldiab,lanim
      logical(kind=4), dimension(gwpdim(1,1)), intent(in) :: pgwp
 
      character(len=9)                                  :: gfile

      if (ldiab) then
         if (lanim) then
            open(1,file='pesdanim.p',form='formatted',status='replace')
         else 
            open(1,file='pesd.p',form='formatted',status='replace')
         endif
      else
         if (lanim) then
            open(1,file='pesadanim.p',form='formatted',status='replace')
         else
            open(1,file='pesad.p',form='formatted',status='replace')
         endif
      endif

      write(1,*) "# Gnuplot script file for plotting DD-vMCG PES"
      write(1,*) "# - written for Gnuplot 5.0.0"

      if (lanim) write(1,*) "set terminal pngcairo"

      write(1,*) "set print '-'"
      write(1,*) "GWP_min=100"
      write(1,*) "GWP_max=0"

      do i=1,gwpdim(1,1)
         gfile=''
         if (ldiab.and.pgwp(i)) then
            write(gfile,'(A6,I0.2)') "dplgwp",i
         else if (pgwp(i)) then
            write(gfile,'(A7,I0.2)') "adplgwp",i
         endif
         gfile=trim(gfile)
         if (gfile.ne.'') then
            fgwp=i
            if (ldiab) then
               write(1,'(A19,A8,A19,I0.2,A10)') " stats './ddpesout/",gfile,"' using 6 &
                     &name 'GWP",i,"' nooutput"
            else
               write(1,'(A19,A9,A19,I0.2,A10)') " stats './ddpesout/",gfile,"' using 6 &
                     &name 'GWP",i,"' nooutput"
            endif
            write(1,'(A8,I0.2,A28,I0.2,A5)') " if (GWP",i,"_min < GWP_min) {GWP_min=GWP",i,"_min}"
            write(1,'(A8,I0.2,A28,I0.2,A5)') " if (GWP",i,"_max > GWP_max) {GWP_max=GWP",i,"_max}"
         endif
      enddo
      
      if (ldiab) then      
         write(1,*) "stats './ddpesout/pesd1' using 4 name 'PES1' nooutput"
         write(1,*) "stats './ddpesout/pesd2' using 4 name 'PES2' nooutput"
      else
         write(1,*) "stats './ddpesout/pesad1' using 4 name 'PES1' nooutput"
         write(1,*) "stats './ddpesout/pesad2' using 4 name 'PES2' nooutput"
      endif

      write(1,*) "set cbrange[PES1_min:PES1_max+PES1_min+PES2_max+PES2_min+GWP_max+GWP_min]"
      write(1,*) "set format cb '%.2f'"
      write(1,*) "set style fill transparent solid 0.60 border"

      write(1,*) "set palette defined (PES1_min 'black',\"
      write(1,*) "            0.33*PES1_max+PES1_min 'red',\"
      write(1,*) "            0.66*PES1_max+PES1_min 'yellow',\"
      write(1,*) "            PES1_max+PES1_min 'white',\"
      write(1,*) "            PES2_min+PES1_max+PES1_min 'white',\"
      write(1,*) "            PES2_min+PES1_max+PES1_min 'black',\"
      write(1,*) "            0.33*PES2_max+PES2_min+PES1_max+PES1_min 'magenta',\"
      write(1,*) "            0.66*PES2_max+PES2_min+PES1_max+PES1_min 'blue',\"
      write(1,*) "            PES2_max+PES2_min+PES1_max+PES1_min 'white',\"
      write(1,*) "            PES1_max+PES1_min+PES2_max+PES2_min+GWP_min 'white',\"
      write(1,*) "            PES1_max+PES1_min+PES2_max+PES2_min+GWP_min 'blue',\"
      write(1,*) "            PES1_max+PES1_min+PES2_max+PES2_min+GWP_max+GWP_min &
                              &'green')"

      write(1,*) "set pm3d depthorder border linetype -1 linewidth 0.4"
      write(1,*) "unset key"

      if (lanim) then
         if (ldiab) then
            write(1,*) "system('mkdir -p animationd')"
         else
            write(1,*) "system('mkdir -p animationad')"
         endif
      endif

      write(1,'(A20,I0.2,A13)') " do for [i=0:int(GWP",fgwp,"_blocks)-1] {"

      if (lanim) then
         if (ldiab) then
            write(1,*) "outfile=sprintf('animationd/time%02.0f.png',i)"
         else
            write(1,*) "outfile=sprintf('animationad/time%02.0f.png',i)"
         endif
         write(1,*) "set output outfile"
      else
         write(1,*) "   print i-1"
      endif
      
      nplt=1
      do i=1,gwpdim(1,1)
         gfile=''
         if (ldiab.and.pgwp(i)) then
            write(gfile,'(A6,I0.2)') "dplgwp",i
         else if (pgwp(i)) then
            write(gfile,'(A7,I0.2)') "adplgwp",i
         endif
         if (gfile.ne.'') then
            if (nplt.eq.1) then
               if (ldiab) then
                  write(1,'(A22,A8,A104)') "    splot './ddpesout/",gfile,&
                        "' index i u 2:3:5:8:&
                         &(PES1_max+PES1_min+PES2_max+PES2_min+GWP_min+$6) &
                         &w points pt 7 ps variable palette,\"
                  write(1,'(A22,A8,A104)') "          './ddpesout/",gfile,&
                        "' index i u 2:3:4:7:(PES1_max+PES1_min+PES2_max+PES2_min+GWP_min+$6) &
                         &w points pt 7 ps variable palette,\"
               else
                  write(1,'(A22,A9,A104)') "    splot './ddpesout/",gfile,&
                        "' index i u 2:3:5:7:&
                         &(PES1_max+PES1_min+PES2_max+PES2_min+GWP_min+$6) &
                         &w points pt 7 ps variable palette,\"
                  write(1,'(A22,A9,A104)') "          './ddpesout/",gfile,&
                        "' index i u 2:3:4:8:(PES1_max+PES1_min+PES2_max+PES2_min+GWP_min+$6) &
                         &w points pt 7 ps variable palette,\"
               endif
            else
               if (ldiab) then
                  write(1,'(A22,A8,A104)') "          './ddpesout/",gfile,&
                        "' index i u 2:3:5:8:&
                         &(PES1_max+PES1_min+PES2_max+PES2_min+GWP_min+$6) &
                         &w points pt 7 ps variable palette,\"
                  write(1,'(A22,A8,A104)') "          './ddpesout/",gfile,&
                        "' index i u 2:3:4:7:(PES1_max+PES1_min+PES2_max+PES2_min+GWP_min+$6) &
                         &w points pt 7 ps variable palette,\"
               else
                  write(1,'(A22,A9,A104)') "          './ddpesout/",gfile,&
                        "' index i u 2:3:5:7:&
                         &(PES1_max+PES1_min+PES2_max+PES2_min+GWP_min+$6) &
                         &w points pt 7 ps variable palette,\"
                  write(1,'(A22,A9,A104)') "          './ddpesout/",gfile,&
                        "' index i u 2:3:4:8:(PES1_max+PES1_min+PES2_max+PES2_min+GWP_min+$6) &
                         &w points pt 7 ps variable palette,\"
               endif
            endif
            nplt=nplt+1
         endif
      enddo

      if (ldiab) then
         write(1,*) "         './ddpesout/pesd1' with pm3d palette,\"
         write(1,*) "         './ddpesout/pesd2' u 1:2:3:($4+PES1_max+PES1_min+PES2_min) &
                               &with pm3d palette"
      else
         write(1,*) "         './ddpesout/pesad1' with pm3d palette,\"
         write(1,*) "         './ddpesout/pesad2' u 1:2:3:($4+PES1_max+PES1_min+PES2_min) &
                               &with pm3d palette"
      endif

      if (.not.lanim) write(1,*) "         pause -1"

      write(1,*) "}"

      close(1)

      end subroutine wrgnugwp

!#######################################################################

      subroutine wroptions(option)
      
      implicit none

      integer(long), intent(in)                         :: option
      integer(long)                                     :: i

      write(6,'(78a1)') ('-',i=1,78)

      if (option.eq.1) then
         write(6,'(1x,a,/a)')&
     "Purpose: Plots the diabatic and adiabatic potential energy surfaces.",&
     "          And also plots the GWP trajectories on the PES."
         write(6,'(1x,a)') "Usage:   ddpes[-i -ex1 -ex2 -ver -h]"
         write(6,'(/,a)') "Options : "
         write(6,'(a)') " -i     : Specify the input file to be read for arguments."
         write(6,'(a)') " -ex1   : Example input file using log keyword."
         write(6,'(a)') " -ex2   : Example input file using cuts1 and cuts2 &
                                   &keywords."
         write(6,'(a)') " -ver   : Version information about this program."
         write(6,'(a)') " -h     : Print this help text."
         write(6,'(/2a)') "Program also takes an input file with options as below."
         write(6,'(/,a)') "Program input file options (file pesinp): "
         write(6,'(/,a)') "Required arguments : "
         write(6,'(a)') " lg1     : Number of steps along vector 1 in negative &
                                    &direction."
         write(6,'(a)') " rg1     : Number of steps along vector 1 in positive &
                                    &direction."
         write(6,'(a)') " lg2     : Number of steps along vector 2 in negative &
                                    &direction."
         write(6,'(a)') " rg2     : Number of steps along vector 2 in positive &
                                    &direction."
         write(6,'(a)') " nstates : The number of states in the system."
         write(6,'(/,a)') "Optional arguments : "
         write(6,'(a)') " scale1     : scaling factor for the weightings of the &
                                      &vector mode 1."
         write(6,'(a)') " scale2     : scaling factor for the weightings of the &
                                      &vector mode 2."
         write(6,'(a)') " cuts1     : This should be input n times, where n is &
                                      &the number of normal modes in the system."
         write(6,'(a)') "             cuts1 is followed by the mode number (1 to n) &
                                      &and a weighting coefficient."
         write(6,'(a)') " cuts2     : Must be used in conjunction with cuts1. &
                                      &Implementation as for cuts1."
         write(6,'(a)') " trajsteps : Number of time intervals after which to &
                                      &plot new trajectory point."
         write(6,'(/,a)') "Optional keywords : "
         write(6,'(a)') " pathlog  : Specify the path to the log file from which &
                                     &to read vectors - useful for propagation &
                                     &diabatisation calculations."
         write(6,'(a)') " log      : Derivative coupling and gradient difference &
                                     &vectors are read from the log file "
         write(6,'(a)') "            and used as vectors 1 and 2 respectively."
         write(6,'(a)') " plottraj : GWP trajectories are plotted on the PES."
         write(6,'(a)') "            Specify which GWP using gwp keyword."
         write(6,'(a)') " gwp      : Followed by integer specifying GWP &
                                     &trajectory to plot."
         write(6,'(a)') "            Keyword can be used up to m times, where m &
                                     &is the number of GWPs." 
         write(6,'(a)') " gwpall   : All GWP trajectories are plotted."
         write(6,'(a)') "            Identical to using gwp m times."
         write(6,'(a)') " rawad    : Plots raw (un-adiabatised) adiabatic surfaces - &
                                     &from electronic structure calculations."
         write(6,'(/,a)') "For example input files, run program with option -ex1 &
                           &or -ex2."
         write(6,'(/,a)') "Using gnuplot to visualise surfaces : "
         write(6,'(a)') " 1) Ensure gnuplot 5.0.0 or newer is loaded."
         write(6,'(a)') " 2) In gnuplot terminal, enter 'load <file>' where file "
         write(6,'(a)') "    is pesd.p or pesad.p for diabatic/adiabatic surfaces."
         write(6,'(/,a)') "Note that the output gnuplot files are only compatible &
                         &with gnuplot 5.0.0 onwards."
      else if (option.eq.2) then
         write(6,'(a)') "Program example input file reading log for vectors."
         write(6,'(/,a)') "#DDPES INPUT FILE"
         write(6,'(a)') "lg1 = 4"
         write(6,'(a)') "rg1 = 4"
         write(6,'(a)') "lg2 = 4"
         write(6,'(a)') "rg2 = 4"
         write(6,'(a)') "nstates = 2"
         write(6,'(a)') "scale1 = 100"
         write(6,'(a)') "scale2 = 100"
         write(6,'(a)') "pathlog = /home/ds3012/fulvene-dynamics/fulv_24gwp/log"
         write(6,'(a)') "log"
         write(6,'(a)') "plottraj"
         write(6,'(a)') "trajsteps = 10"
         write(6,'(a)') "gwp 1"
         write(6,'(a)') "gwp 3"
         write(6,'(a)') "gwp 21"
         write(6,'(a)') "gwp 22"
      else if (option.eq.3) then
         write(6,'(a)') "Program example input file with user-specified vectors."
         write(6,'(/,a)') "#DDPES INPUT FILE"
         write(6,'(a)') "lg1 = 15"
         write(6,'(a)') "rg1 = 15"
         write(6,'(a)') "lg2 = 15"
         write(6,'(a)') "rg2 = 15"
         write(6,'(a)') "nstates = 2"
         write(6,'(a)') "scale1 = 0.2"
         write(6,'(a)') "scale2 = 0.2"
         write(6,'(/,a)') "cuts1    3        -1.93823576"
         write(6,'(a)') "cuts1    6        -2.65482587"
         write(6,'(a)') "cuts1    7        -3.98798854"
         write(6,'(/,a)') "cuts2    5        -1.75857733"
         write(6,'(a)') "cuts2    13        0.55460097"
         write(6,'(a)') "cuts2    15        0.33206331"
         write(6,'(a)') "cuts2    17        0.42682307"
         write(6,'(a)') "cuts2    20       -1.11016540"
         write(6,'(a)') "cuts2    21        0.75326162"
         write(6,'(a)') "cuts2    22       -0.25071183"
         write(6,'(a)') "cuts2    24       -1.68888395"
         write(6,'(a)') "cuts2    25        0.11551169"
         write(6,'(a)') "cuts2    27       -0.00537052"
         write(6,'(a)') "cuts2    30       -0.01008006"
         write(6,'(/,a)') "plottraj"
         write(6,'(a)') "trajsteps = 10"
         write(6,'(a)') "gwpall"
      endif

      write(6,'(78a1)') ('-',i=1,78)

      end subroutine wroptions

      end program ddpes

