
      program ddtraj

use decimal, only:long,dop,sip
use global
use globmemmod
use versions
use aglobal
use logdat
use memdimmod
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use compdatmod
use dirdyn
use plot
use channels

use op1lib, only: cpvxd, cpvxz, cuttxd2
use mtlib, only: qtxxzz
use iofile, only: rdpsiinfo
use rddvrmod, only: dvrinfo
use rdopermod, only: operinfo
use zeigauswmod, only: zeigausw
use memhopsmod, only: memhops

use module_interface

use dd_db, only: ddq2x,gwpcentres
use readqc, only: getinam

      implicit none

      integer(long)                        :: chkdvr,chkgrd,chkpsi,chkdat
      character(len=c5)                    :: filename
      logical(kind=4)                      :: lerr

      integer(long)                        :: ilbl,jlbl
      integer(long)                        :: i,j,ilddtrans
      character(len=c2)                    :: rdtag
      logical(kind=4)                      :: int2log
      logical(kind=4)                      :: lwronly=.false., &
                                              lxyz=.false., &
                                              lqtrans =.true.


!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      lovwr = .true.

!-----------------------------------------------------------------------
! read input options
!-----------------------------------------------------------------------
     call ddtrajinput

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(ipsi)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_dirdyn
      call alloc_dddb
      call alloc_psidef

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='ddtraj.log',form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare files to be read
! set flag to .true. if file is to be read
!
!     lrddvr    DVR file
!     lrdoper   OPER file. 
!-----------------------------------------------------------------------
      lrddvr=.false.
      lrdoper=.false.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare resources required (other than those needed to read a file)
!
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! ltrafo : FBR/DVR trafo matrices
! lfft   : FFT arrays rueck and fftfak
!-----------------------------------------------------------------------
      ncomp=0
      lpsi=.true.
      lpsi1=.false.
      lpsigrd=.false.
      ldmat=.false.
      ldmat1=.false.
      lort=.true.
      lgpop=.false.
      ltrafo=.false.
      lfft=.false.

      ldd = .true.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! open input and output data files
!
! use standard channels if possible
!  irho : gridpop file
!  ipsi : psi file
!  iaut : auto file
!  ichk : check file
!  iaus : output file
!-----------------------------------------------------------------------
! psifile is opened in ddtrajinput 
!     filename='psi'
!     open(ipsi,file=filename,form='unformatted',status='old',err=1000)

!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
! read information from system files
!-----------------------------------------------------------------------
      if (lrddvr) then
         chkdvr=2
         filename=dname(1:dlaenge)//'/dvr'
         open(idvr,file=filename,form='unformatted',status='old',&
              err=1000)
         call dvrinfo(lerr,chkdvr)
         close(idvr)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif
      if (lrdoper) then
         chkdvr=2
         chkgrd=2
         filename=operfile
         open(ioper,file=filename,form='unformatted',status='old',&
              err=1000)
         call operinfo(lerr,chkdvr,chkgrd)
         close(ioper)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif

!-----------------------------------------------------------------------
! get ndofddpes + natm from oper file
!-----------------------------------------------------------------------
      filename=operfile
      open(ioper,file=filename,form='unformatted',status='old',&
           err=1000)
      rewind(ioper)
      do
   10    continue
         read(ioper,err=10,end=20) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'ndofddpes') then
            read(ioper) ndofddpes
         else if (rdtag(ilbl:jlbl) .eq. 'ndofdd') then
            read(ioper) ndofdd
         else if (rdtag(ilbl:jlbl) .eq. 'natm') then
            read(ioper) natm
         else if (rdtag(ilbl:jlbl) .eq. 'xatnam') then
            read(ioper) (xatnam(i),i=1,natm)
         else if (rdtag(ilbl:jlbl) .eq. 'lddtrans') then
            read(ioper) ilddtrans
            lddtrans=int2log(ilddtrans)
         else if (rdtag(ilbl:jlbl) .eq. 'ddxcoo0') then
            read(ioper) (ddxcoo0(i),i=1,ndofddpes)
         else if (rdtag(ilbl:jlbl) .eq. 'ddtransf') then
            read(ioper) ((ddtransf(j,i),j=1,ndofddpes),i=1,ndofdd)
         else if (rdtag(ilbl:jlbl) .eq. 'ddtransb') then
            read(ioper) ((ddtransb(j,i),j=1,ndofdd),i=1,ndofddpes)
         else if (rdtag(ilbl:jlbl) .eq. 'end oper info') then
            exit
         endif
      enddo
   20 continue
      close(ioper)
      
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
! First read file version number
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      filename='psi'
      read(ipsi,err=999) filever(ipsi)
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)

      if (.not. (lgwp .or. ltraj)) then
         message='No GWP or trajectory mode present'
         call errormsg
      endif

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      if (ltraj) then
         zcent(1,1) = 1
         qcentdim = nspfdof(1)*dim(1,1)
      endif
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
! here assign any further memory arrays
!-----------------------------------------------------------------------
      mrpcent=mrdim
      mrdim=mrpcent+qcentdim

!-----------------------------------------------------------------------
! assign memory for operator
!-----------------------------------------------------------------------
      if (lrdoper) call memhops

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim/2,&
       ' real*4     : ', mfdim/2,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)
if(allocated(gfgwp))deallocate(gfgwp)
if(allocated(aigwp))deallocate(aigwp)
if(allocated(zgwp))deallocate(zgwp)
if(allocated(agmat))deallocate(agmat)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))
allocate(gfgwp(gfgwpdim))
allocate(aigwp(aigwpdim))
allocate(zgwp(zgwpdim))
allocate(agmat(agmatdim))

      call ddtraj1(mc(mcpsi),ms(mspsi),mi(mijindx),&
           agmat,mr(mrqcent),mr(mrpcent),gfgwp,aigwp,zgwp,&
           mc(mcadgwp),mi(migwpdep))

deallocate(mc,mr,mi,ml,ms,mf,gfgwp,aigwp,zgwp,agmat)
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_dirdyn
      call dealloc_dddb
      call dealloc_psidef
      call dealloc_plot

      stop
  999 continue
      message= 'Error reading :'//filename(1:c5-15)
      write(6,'(a)') message
      stop
 1000 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop
      contains

!#######################################################################

      subroutine ddtraj1(psi,spsi,jindx,&
                 agmat,qcenter,pcenter,&
                 gfgwp,aigwp,zgwp,adgwp,gwpdep)

use global
use aglobal
use griddatmod
use psidef
use datenmod
use compdatmod
use operdef
use dirdyn
use plot
use channels

use rdfilesmod, only: rdpsi
use gwplib, only: gwp_param,gwp_cmom

      implicit none

! standard variables for interactive plots
      integer(long)                :: ndata,nplots,psicount,m,chkdvr,&
                                      chkgrd,n,s,chkpsi,chkdat,nt,ns,e,f,f1
      integer(long)                :: plottyp
      integer(long), dimension(1)                              :: idum
      integer(long), dimension(gwpinfodim), intent(out)        :: gwpdep
      integer(long), dimension(adim), intent(out)              :: jindx
      real(dop)                                                :: time
      real(dop), dimension(1)                                  :: xdum,ydum

      complex(sip), dimension(spsidim), intent(out)  :: spsi
      complex(dop), dimension(dgldim), intent(out)   :: psi
      character(len=240)                             :: string

! for GWP plotting
      real(dop), dimension(qcentdim), intent(out)    :: qcenter,pcenter
      real(dop), allocatable, dimension(:,:,:)       :: allcent
      real(dop), allocatable, dimension(:,:)         :: avcent,ggp
      real(dop), dimension(nddstate)                 :: norm
      real(dop)                                      :: totggp
      complex(dop), dimension(agmatdim), intent(out) :: agmat
      complex(dop), dimension(gsmatdim), intent(out) :: adgwp
      complex(dop), dimension(gfgwpdim), intent(out) :: gfgwp
      complex(dop), dimension(aigwpdim), intent(out) :: aigwp
      complex(dop), dimension(zgwpdim), intent(out)  :: zgwp
      character(len=c5) :: fileout1,fileout2
      character(len=25) :: buf
      character(len=2)  :: tag

!-----------------------------------------------------------------------
! following are needed for plots
!-----------------------------------------------------------------------
      integer(long)       :: i

!-----------------------------------------------------------------------
! zero dummy variables
!-----------------------------------------------------------------------
      xdum(1)=0.0
      ydum(1)=0.0

!-----------------------------------------------------------------------
! interactive plot defaults
!-----------------------------------------------------------------------
      ndata=1

!-----------------------------------------------------------------------
! open scratch files
!-----------------------------------------------------------------------
      open(unit=itmpin,file='scratch',form='unformatted')
      open(unit=idump,file='scratch1',form='unformatted')

!-----------------------------------------------------------------------
! Allocate array for all geometries
!-----------------------------------------------------------------------
      allocate(allcent(ndofdd,totgp,nstate))
      allocate(avcent(ndofdd,nddstate+1))
      allocate(ggp(totgp,nddstate))

!-----------------------------------------------------------------------
! Determine the number of data-sets (times).
! and write data (qcenter and pcenter) to scratch
!-----------------------------------------------------------------------
      time=(tinit-out2)/fs
      ndata=0
      rewind(ipsi)
      chkdvr=0
      chkgrd=0
      chkpsi=0
      chkdat=0
      psicount=-nskip
      read(ipsi) filever(ipsi)
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)

!-----------------------------------------------------------------------
! If simulation used Cartesian, no transformation needed (or possible)
!-----------------------------------------------------------------------
      if (.not. lqtrans .and. .not. lddtrans) then
         print*,'Coordinates cannot be transformed. Ignoring notr option'
         lqtrans = .true.
      endif

      if (ltraj) gwpdim = dim

      do  
         call rdpsi(ipsi,psi,spsi,jindx,agmat,idum,adgwp,gwpdep)

         if (lend) exit
         psicount=psicount+1
         if(psicount.le.0) cycle

         time=time+out2/fs
         if (lgwp) then
            call gwp_param(psi,aGmat,gfgwp,aigwp,zgwp,qcenter,1)
            call gwp_cmom(psi,pcenter)
         else if (ltraj) then
            call extr(psi(zetf(1,1)),qcenter,pcenter,nspfdof(1),dim(1,1))
         endif

         do s=1,nstate
            do m=1,nmode
               if (.not. gwpm(m)) cycle
               call gwpcentres(m,allcent(1,1,s),qcenter(zcent(m,s)),&
                    ndofdd,nspfdof(m),vdimgp(m,s),dimgp(m,s),ndimgp(m,s))
            enddo
! add coordinates of point DVRs
            if (lpntdvr) then
               f1=0
               do f=1,ndof
                  if (f .eq. feb) cycle
                  f1=f1+1
                  if (basis(f) .eq. 19) allcent(f1,:,s) = rpbaspar(1,f)
               enddo
            endif
         enddo

! get GGPs
         ggp=0.0
         call getggp(ggp,gfgwp,psi,norm)

! calculate average coordinates
         avcent=0.0
         if (leb) then
            do s=1,nddstate
               do e=1,totgp
                  avcent(:,s) = avcent(:,s) + allcent(:,e,1)*ggp(e,s)
               enddo
            enddo
         else
            do s=1,nstate
               do e=1,nsgp(s)
                  avcent(:,s) = avcent(:,s) + allcent(:,e,s)*ggp(e,s)
               enddo
            enddo
         endif

! get overall average
         do s=1,nddstate
            avcent(:,nddstate+1) = avcent(:,nddstate+1) + avcent(:,s)
         enddo

! normalise state specific averages
         do s=1,nddstate
            avcent(:,s) = avcent(:,s)/norm(s)
         enddo

         write(itmpin) time,&
             (((allcent(i,e,s),i=1,ndofdd),e=1,totgp),s=1,nstate),&
             ((avcent(i,s),i=1,ndofdd),s=1,nddstate+1)
         write(idump) time,((ggp(e,s),e=1,totgp),s=1,nddstate)

         ndata=ndata+1
      enddo

      nplots = ndata
      call myflush(itmpin)

   
   50 continue
      write(6,'(a)') 'Plot trajectory?' 
      nt = 0
      ns = 0
      plottyp = 0
      write(6,'(a,i7,a)') 'Enter: Trajectory No. (Max: ',nsgp(1:nstate),')' 
      write(6,'(a)') '       or "av" to show average structure ' 
      write(6,'(a)') '       or "avS" to show average structure for one state' 
      write(6,'(a)') '       or "all" to print all trajectory files' 
      write(6,'(a)') '       or 0 to stop' 
   60 continue
      read(5,'(a)') buf
      if (buf .eq. '  ') then
         go to 60
      else if (buf .eq. 'av ') then
         plottyp = 2
         nt = totgp+1
      else if (buf(1:2) .eq. 'av') then
         plottyp = 3
         nt = totgp+1
      else if (buf .eq. 'all') then
         plottyp = 4
         nt = 1
         ns = 1
      else 
         read(buf,*) nt
         plottyp = 1
      endif

! nothing selected. Stop.
      if (nt .eq. 0) then
         go to 100
      endif  

! if a single trajectory selected, select state if multi-set
      if (plottyp .eq. 1 .and. nstate .gt. 1) then
         write(6,'(a,i7,a)') 'Enter State No. (Max: ',nstate,')'
         read(5,*) ns
      else
         ns=1
      endif
! check selected trajectory allowed
      if (plottyp .eq. 1 .and. nt .gt. nsgp(ns)) then
         print*,'No. too large. Try again'
         print*
         go to 50
      endif

! if single state average selected, select state
      if (plottyp .eq. 3) then
         write(6,'(a,i7,a)') 'Enter State No. (Max: ',nddstate,')'
         read(5,*) ns
         if (ns .gt. nddstate) then
            print*,'No. too large. Try again'
            print*
            go to 50
         endif
      endif

   80 continue
      rewind(itmpin)
      !ilbl=index(fileout,'.')-1
      ilbl=scan(fileout,'.', .TRUE.)-1  !So finds last . not first (if using hidden dir)
      jlbl=index(fileout,' ')-1
      tag = ' '
      if (.not. lqtrans) tag = 'q'
      if (plottyp .eq. 1 .or. plottyp .eq. 4) then
         write(fileout1,'(3a,i5,a)') fileout(1:ilbl),tag,"_",nt,&
             fileout(ilbl+1:jlbl)
         call rmblnk(fileout1)
         ilbl=index(fileout1,' ')-1
         open(iaus,file=fileout1(1:ilbl),form='formatted',&
              status='unknown',err=1000)
      else if (plottyp .eq. 2) then
         write(fileout1,'(4a)') fileout(1:ilbl),tag,"_av",fileout(ilbl+1:jlbl)
         call rmblnk(fileout1)
         ilbl=index(fileout1,' ')-1
         open(iaus,file=fileout1(1:ilbl),form='formatted',&
              status='unknown',err=1000)
      else if (plottyp .eq. 3) then
         write(fileout1,'(3a,i5,a)') fileout(1:ilbl),tag,"_av",ns,&
               fileout(ilbl+1:jlbl)
         call rmblnk(fileout1)
         ilbl=index(fileout1,' ')-1
         open(iaus,file=fileout1(1:ilbl),form='formatted',&
              status='unknown',err=1000)
      endif

      write(6,'(2a)') ' The plot file is : '//fileout1(1:ilbl)
      if (lqtrans) then
! Output Cartesians
         do n=1,ndata
            read(itmpin) time, &
                (((allcent(i,e,s),i=1,ndofdd),e=1,totgp),s=1,nstate),&
                ((avcent(i,s),i=1,ndofdd),s=1,nddstate+1)
            if (plottyp .eq. 1 .or. plottyp .eq. 4) then
               if (lxyz) then
                  call plddtrajxyz(allcent(1,nt,ns),time)
               else
                  call plddtraj(allcent(1,nt,ns))
               endif
            else if (plottyp .eq. 2) then
               if (lxyz) then
                  call plddtrajxyz(avcent(1,nddstate+1),time)
               else
                  call plddtraj(avcent(1,nddstate+1))
               endif
            else if (plottyp .eq. 3) then
               if (lxyz) then
                  call plddtrajxyz(avcent(1,ns),time)
               else
                  call plddtraj(avcent(1,ns))
               endif
            endif
         enddo
      else
! Output Nmodes
         write(iaus,'(a)') '# Datablocks for plotting'
         do f=1,ndofdd
            write(buf,'(a,i5)') '$Data_',f
            call rmblnk(buf)
            ilbl=index(buf,' ')-1
            write(buf(ilbl+1:ilbl+8),'(a)') ' << EOD'
            write(iaus,'(a)') buf

            rewind(itmpin)
            do n=1,ndata
               read(itmpin) time, &
                   (((allcent(i,e,s),i=1,ndofdd),e=1,totgp),s=1,nstate),&
                   ((avcent(i,s),i=1,ndofdd),s=1,nddstate+1)
               if (plottyp .eq. 1 .or. plottyp .eq. 4) then
                  write(iaus,'(f18.8,f20.12)') time,allcent(f,nt,ns)
               else if (plottyp .eq. 2) then
                  write(iaus,'(f18.8,f20.12)') time,avcent(f,nddstate+1)
               else if (plottyp .eq. 3) then
                  write(iaus,'(f18.8,f20.12)') time,avcent(f,ns)
               endif
            enddo
            write(iaus,'(a)') 'EOD '
         enddo
            
      endif

!-----------------------------------------------------------------------
! Print out GGPs
!-----------------------------------------------------------------------
      write(fileout2,'(a,i5,a)') "ggp_",nt,'.dat'
      call rmblnk(fileout2)
      ilbl=index(fileout2,' ')-1
      open(idat,file=fileout2(1:ilbl),form='formatted',&
           status='unknown',err=1000)
      write(idat,'(a)') '# GGPs. First colum is time (fs), last column total GGP.'
      write(idat,'(a)') '# Middles columns state resolved GGPs. '
      rewind(idump)
      do n=1,ndata
         read(idump) time,((ggp(e,s),e=1,totgp),s=1,nddstate)

         totggp = 0.0
         do s =1,nddstate
            totggp = totggp+ggp(nt,s)
         enddo
         write(idat,*) time,(ggp(nt,s),s=1,nddstate),totggp
      enddo
      close(idat)

!-----------------------------------------------------------------------
! loop to plot all files
!-----------------------------------------------------------------------
      if (plottyp .eq. 4) then
         if (nt .lt. nsgp(ns)) then
            nt = nt+1
            go to 80
         else if (nt .eq. nsgp(ns)) then
            nt = 1
            ns = ns+1
            if (ns .le. nstate) go to 80
         endif
      endif

!-----------------------------------------------------------------------
! Animate Cartesians using jmol
! OR plot Nmodes using gnuplot
! OR write xyz files
!-----------------------------------------------------------------------
      if (.not. lwronly .and. plottyp .ne. 4) then
         if (lqtrans) then
            call myflush(iaus)

            ilbl=index(fileout1,' ')-1
            write(string,'(2a)') "jmol ",fileout1(1:ilbl)
            call excmd(string)

         else
            if (plottyp .eq. 1 .or. plottyp .eq. 4) then
               if (nstate .gt. 1) then
                  write(string,'(a,2i5,a)') &
                        'set title "Normal Modes. GWP: ',nt,ns,'"'
               else
                  write(iaus,'(a,i5,a)') &
                        'set title "Normal Modes. GWP: ',nt,'"'
               endif
            else if (plottyp .eq. 2) then
               write(iaus,'(a)') 'set title "Normal Modes. Average " '
            else if (plottyp .eq. 3) then
               write(iaus,'(a,i5,a)') &
                     'set title "Normal Modes. State Average: ',ns,'"'
            endif
            write(iaus,'(a)') 'set tics out '
            write(iaus,'(a)') 'set nogrid '
            write(iaus,'(a)') 'set output '
            write(iaus,'(a)') 'set terminal x11 '
            write(iaus,'(a)') 'set key out'
            write(iaus,'(a)') 'set xlabel "Time [fs]" '
            write(iaus,'(a)') 'set ylabel "Q " '
            write(iaus,'(a)') 'set xrange [*:*] noreverse '
            write(iaus,'(a)') 'set yrange [*:*] noreverse '
            write(iaus,'(a)') 'set size 1.,1.'
            write(iaus,'(a)') 'set origin 0.,0.'
            write(iaus,'(3a)') 'plot $Data_1 w l title "',&
                      trim(modelabel(1)),'"'
            do f=2,ndofdd
               write(buf,'(a,i5)') '$Data_',f
               call rmblnk(buf)
               ilbl=index(buf,' ')-1
               write(iaus,'(5a)') 'replot ',buf(1:ilbl),' w l title "',&
                         trim(modelabel(f)),'"'
            enddo
            write(iaus,'(a)') 'pause -1'
            call myflush(iaus)

            ilbl=index(fileout1,' ')-1
            string='gnuplot -p '//fileout1(1:ilbl)
            call excmd(string)
         endif
      endif

!-----------------------------------------------------------------------
! Finish plotting and return for new command
!-----------------------------------------------------------------------
      close(iaus)
      go to 50

!-----------------------------------------------------------------------
! Stop called
!-----------------------------------------------------------------------
  100 continue
 
      close(itmpin,status='delete')
      close(idump,status='delete')
      deallocate(allcent)

      return
 1000 write(6,*)' Cannot open output file : '
      ilbl=index(filename,' ')-1
      write(6,*) filename(1:ilbl)

      end subroutine ddtraj1

!#######################################################################

      subroutine getggp(ggp,gs,psi,norm)

      integer(long)    :: zeig,zeig1,zeig2,i,m,s
      real(dop), dimension(totgp,nddstate), intent(out) :: ggp
      real(dop), dimension(nddstate), intent(out)       :: norm
      real(dop), dimension(adim)                        :: dtmp
      complex(dop), dimension(adim), intent(in)         :: psi
      complex(dop), dimension(gfgwpdim), intent(in)     :: gs
      complex(dop), dimension(3*adim)                   :: ctmp

      if (leb) then
         zeig1=1
         zeig2=block(1)+1
         call cpvxz(psi(1),ctmp(zeig2),block(1))
         do m=1,nmode
            if (.not. gwpm(m)) cycle
            zeig=zeig1
            zeig1=zeig2
            zeig2=zeig
            call qtxxzz(gs(zgfgwp(m,1,1)),ctmp(zeig1),ctmp(zeig2),&
                 vdim(m,1),dim(m,1),ndim(m,1))
         enddo
         do i=1,block(1)
            ctmp(zeig1+i-1) = dconjg(psi(i))*ctmp(zeig2+i-1) 
            dtmp(i) = dble(ctmp(zeig1+i-1))
         enddo
         do s=1,nddstate
            call cuttxd2(dtmp,ggp(1,s),vdim(meb,1),dim(meb,1),ndim(meb,1),s)
         enddo

         norm=0.0
         do s=1,nddstate
            do i=1,totgp
               norm(s) = norm(s)+ggp(i,s)
            enddo
         enddo

      endif

      return
      end subroutine getggp

!#######################################################################

      subroutine extr(psi,qcenter,pcenter,nspfdof,dim)

      integer(long)                                    :: e,n
      integer(long), intent(in)                        :: nspfdof,dim
      real(dop), dimension(nspfdof,dim), intent(out)   :: qcenter,pcenter
      complex(dop), dimension(nspfdof,dim), intent(in) :: psi

      do e=1,dim
         do n=1,nspfdof
            qcenter(n,e) = dble(psi(n,e))
            pcenter(n,e) = dimag(psi(n,e))
         enddo
      enddo

      return
      end subroutine extr

!#######################################################################

      subroutine plddtraj(gpoint)

      implicit none

      integer(long)                                :: n,i,izero,gpsize
      integer(long), dimension(natm)               :: iatnam,iatiso
      real(dop), dimension(ndofddpes)              :: xcenter
      real(dop), dimension(ndofdd), intent(in)     :: gpoint

      izero=0
      do n = 1, natm
         call getinam(xatnam(n),iatnam(n),iatiso(n),ilog)
      enddo

      write(iaus,'(/,a)') ' Entering Gaussian System  '

!     Transform
      if (lddtrans) then
         call ddq2x(gpoint,xcenter)
      else
         call cpvxd(gpoint,xcenter,ndofddpes)
      endif

! convert to Angst
      do n=1,ndofddpes
         xcenter(n)=xcenter(n)*0.52917725_dop
      enddo

      write(iaus,'(/,a)')&
            '                          Input orientation:   '
      write(iaus,'(2a)')&
            ' ----------------------------------------------',&
            '-----------------------'
      write(iaus,'(2a)')&
            ' Center     Atomic     Atomic             ',&
            ' Coordinates (Angstroms)'
      write(iaus,'(2a)')&
            ' Number     Number      Type              ',&
            'X           Y           Z '
      write(iaus,'(2a)')&
            ' ----------------------------------------------',&
            '-----------------------'

      i=1
      do n=1,natm
         write(iaus,'(i5,i11,i14,4x,3f12.6)')&
             n,iatnam(n),izero,xcenter(i),xcenter(i+1),&
             xcenter(i+2)
         i=i+3
      enddo
      write(iaus,'(2a)')&
            ' ----------------------------------------------',&
            '-----------------------'

      return
      end subroutine plddtraj

!#######################################################################

      subroutine plddtrajxyz(gpoint,time)

      implicit none

      integer(long)                                :: n,i,izero,gpsize
      integer(long), dimension(natm)               :: iatnam,iatiso
      real(dop), dimension(ndofddpes)              :: xcenter
      real(dop), dimension(ndofdd), intent(in)     :: gpoint
      real(dop), intent(in)                        :: time

      izero=0
      do n = 1, natm
         call getinam(xatnam(n),iatnam(n),iatiso(n),ilog)
      enddo

!     Transform
      if (lddtrans) then
         call ddq2x(gpoint,xcenter)
      else
         call cpvxd(gpoint,xcenter,ndofddpes)
      endif

! convert to Angst
      do n=1,ndofddpes
         xcenter(n)=xcenter(n)*0.52917725_dop
      enddo

      write(iaus,'(a)') ' '
      write(iaus,'(i5)') natm
      write(iaus,'(a,f8.3)') ' Time (fs) = ',time
      i=1
      do n=1,natm
         write(iaus,'(i11,4x,3f12.6)')&
             iatnam(n),xcenter(i),xcenter(i+1),&
             xcenter(i+2)
         i=i+3
      enddo

      return
      end subroutine plddtrajxyz

!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------

      subroutine ddtrajinput

use global
use aglobal
use griddatmod
use psidef
use datenmod
use compdatmod
use plot
use channels

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: iarg,ilbl,ierr,i,psilaenge
      character(len=80)       :: buf
      character(len=c5)       :: filename

      psifile  = ' '
      name  = './'
      fileout = 'ddtraj.pl'
      iarg = myiargc() 
      nskip=0

      i = 0
 100  i=i+1
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if(buf .eq. '-i') then
            i = i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf(1:3) .eq. '-f ') then
            i = i+1
            call mygetarg(i,buf)
            psifile = buf
         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
            laeout=index(fileout,' ')-1
            fileout = fileout(1:laeout)//'.pl '

         else if( buf .eq. '-wr ') then
            lwronly=.true.

         else if( buf .eq. '-xyz ') then
            lxyz=.true.

         else if( buf .eq. '-notr ') then
            lqtrans = .false.
            
         else if( buf(1:5) .eq. '-skip ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nskip
            if (nskip .lt. 0) then
               write(6,'(/a)')&
               'Argument of -skip must be positive integer'
               stop
            endif
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -skip'
               stop
            endif

         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(/1x,a)')&
       'Purpose: Enables plotting of the GWP trajectories.'
            write(6,'(1x,2a)')&
       'Usage:   ddtraj [-i -f -o -wr -xyz -notr -skip -ver -h -?]'
            write(6,'(/,a)') 'Options : '
            write(6,'(a)') ' -i DIR  : data is stored in directory DIR'
            write(6,'(2a,/,9x,a)')&
                     ' -f FILE : the wavefunction is read '&
                    ,'from FILE rather than from ./psi',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a)') &
                     ' -o FILE : The ouput is written to file ',&
                     'FILE.pl rather than to ./ddtraj.pl',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(a)')&
                     ' -wr     : the output is written but no animation made'
            write(6,'(a)')&
                     ' -xyz    : Output coordinate sets in xyz format'
            write(6,'(a)')&
                     ' -notr   : No transformation made to Cartesians'
            write(6,'(a)')&
                     ' -skip m : the first m WFs are skipped.'
            write(6,'(a)')&
                     ' -ver    : Version information about the program.'
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(/2x,a,/2x,2a,/2x,2a)')&
            'The program is menu driven. Just type "ddtraj"' ,&
            'and then follow ','the menu.',&
            '-----------------------------',&
            '---------------------------------------------------'

            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         goto 100
      end if
      
  200 continue
      nopt=i-1

      laenge=index(name,' ')-1
      call abspath(name,laenge)

      if (psifile .eq. ' ') psifile=name(1:laenge)//'/psi'
      psilaenge=index(psifile,' ')-1
      call abspath(psifile,psilaenge)

      laeout=index(fileout,' ')-1
      call abspath(fileout,laeout)

!-----------------------------------------------------------------------
! XYZ format output is write only
!-----------------------------------------------------------------------
      if (lxyz) then
         lwronly = .true.
         fileout = 'ddtraj.xyz'
         lqtrans = .true.     ! must output cartesians
      endif

!-----------------------------------------------------------------------
!  Opening the psi file.
!-----------------------------------------------------------------------
      filename=psifile
      open(ipsi,file=filename,form='unformatted',status='old',&
           err=1000)
      ilbl=index(psifile,' ')-1
      write(6,'(2a)') ' The psi file is : '//psifile(1:ilbl)

      return 
      
 1000 write(6,*)' Cannot open psi file : '
      ilbl=index(filename,' ')-1
      write(6,*) filename(1:ilbl)

      stop
      end subroutine ddtrajinput


      end program ddtraj

