!#######################################################################
!
! Transforms the wavefunction in a psi file to the adiabatic picture.
! GW 05/02
!#######################################################################

      program di2ad

use decimal, only:dop,long,sip
use global
use globmemmod
use versions
use aglobal
use logdat
use memdimmod
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use compdatmod
use compdat1mod
use plot
use channels

use iofile, only: rdpsiinfo,wrpsiinfo
use rddvrmod, only: dvrinfo
use rdopermod, only: operinfo
use zeigauswmod, only: zeigausw
use memhopsmod, only: memhops
use zeighpsimod, only: operflag,zeighpsi

use module_interface


      implicit none

      integer(long)                        :: chkdvr,chkgrd,chkpsi,chkdat,ilbl
      character(len=c5)                    :: filename
      logical(kind=4)                      :: lerr

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      lovwr = .true.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare files to be read
! set flag to .true. if file is to be read
!
!     lrddvr    DVR file
!     lrdoper   OPER file. 
!-----------------------------------------------------------------------
      lrddvr=.true.
      lrdoper=.true.

!-----------------------------------------------------------------------
! read input options
!-----------------------------------------------------------------------
      call di2adinput

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare resources required (other than those needed to read a file)
!
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! ltrafo : FBR/DVR trafo matrices
!-----------------------------------------------------------------------
      ncomp=0
      lpsi=.true.
      lpsi1=.false.
      lpsigrd=.false.
      ldmat=.false.
      ldmat1=.false.
      lort=.true.
      lgpop=.false.
      ltrafo=.false.

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(ipsi)

      open(ioper,file=operfile,form='unformatted',status='old',err=1010)
      call rdmemdim(ioper)
      close(ioper)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      call alloc_psidef

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! open input and output data files
!
! use standard channels if possible
!  irho : gridpop file
!  ipsi : psi file
!  iaut : auto file
!  ichk : check file
!  iaus : output file
!-----------------------------------------------------------------------
! psifile is opened in pesinput 
!     filename='psi'
!     open(ipsi,file=filename,form='unformatted',status='old',err=1000)

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
! First read file version number
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      leb=.false.
      lmult=.false.
      read(ipsi,err=999) filever(ipsi)
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)

      if (psitype .ne. 1) then
         routine='Di2ad'
         message=&
            'Need full grid representation of psi. Run psi2ex first'
         call errormsg
      endif

      if (nstate .eq. 1) then
         routine='Di2ad'
         message=&
            'Already in adiabatic picture - single state'
         call errormsg
      endif

!-----------------------------------------------------------------------
! write psi information to new file
!-----------------------------------------------------------------------
      write(ipsi1) progver
      call wrpsiinfo(ipsi1)

!-----------------------------------------------------------------------
! read information from system files
!-----------------------------------------------------------------------
      if (lrddvr) then
         chkdvr=2
         filename=dname(1:dlaenge)//'/dvr'
         open(idvr,file=filename,form='unformatted',status='old',&
              err=1000)
         call dvrinfo(lerr,chkdvr)
         close(idvr)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif

      if (lrdoper) then
         chkdvr=2
         chkgrd=2
         filename=operfile
         open(ioper,file=filename,form='unformatted',status='old',&
              err=1010)
         call operinfo(lerr,chkdvr,chkgrd)
         close(ioper)
         call chksyserr(filename,lerr)
         if (lerr) go to 999

         if (loptcontract) then
            write(6,'(a)') &
       'Warning: Operator contains terms that mix KE and PE '
            stop
         endif

      endif

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
! here assign any further memory arrays
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      workcdim=max(workcdim,dgldim)

      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
! assign memory for operator
!-----------------------------------------------------------------------
      if (lrdoper) call memhops

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

      call subdi2ad(mr(mrhops),mc(mcpsi),mc(mcworkc),ms(mspsi),&
           mi(mijindx),mr(mrort))

deallocate(mc,mr,mi,ml,ms,mf)

      stop
  999 continue
      message= 'Error reading :'//trim(filename)
      write(6,'(a)') message
      stop
 1000 continue

      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//trim(filename)
      call errormsg
      stop
 1010 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open oper file.'
      call errormsg
      stop
      contains

!#######################################################################

      subroutine subdi2ad(hops,psi,psiad,spsi,jindx,ort)

use global
use aglobal
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod
use operdef
use channels
use operdef
use dirdyn

use rddvrmod
use rdopermod, only: rdoper
use rdfilesmod, only: rdpsi

      implicit none

      integer(long)                                 :: chkdvr,chkgrd,d
      integer(long), dimension(1)                   :: idum
      integer(long), dimension(adim), intent(out)   :: jindx
      real(dop)                                     :: time
      real(dop), dimension(1)                       :: rdum
      real(dop), dimension(hopsdim), intent(out)    :: hops
      real(dop), dimension(ortdim), intent(out)     :: ort
      complex(sip), dimension(spsidim), intent(out) :: spsi
      complex(dop), dimension(dgldim), intent(out)  :: psi,psiad
      complex(dop), dimension(1)                    :: cdum

      character(len=c2)                             :: wrtag

!-----------------------------------------------------------------------
! read DVR and operator data
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      dvrdata(1)=.true.      ! ort
      call rddvr(ort,rdum,rdum,rdum,cdum,cdum,idum,cdum,cdum,&
                 idum,idum,rdum,chkdvr)

      chkdvr=1
      chkgrd=1
      call rdoper(hops,chkdvr,chkgrd)

      time=tinit/fs
      do d=1,adim
         psiad(d)=1.0_dop
      enddo

      countpsi=0
      do
         call rdpsi(ipsi,psi,spsi,jindx,cdum,idum,cdum,idum)
         if (lend) exit

         call trans2ad(hops,psi(zetf(1,1)),psiad(zetf(1,1)),subdim(1),&
              nstate,ort)

         time=time+out2/fs

         countpsi=countpsi+1

         write(wrtag,'(a9,i10,a1)') '<data set',countpsi,'>'
         write(ipsi1) wrtag
         wrtag='<psi>'
         write(ipsi1) wrtag

         if (lpsiopt(1)) then
            write(ipsi1) (cmplx(psiad(d)), d = 1,dgldim)
         else
            write(ipsi1) (psiad(d), d = 1,dgldim)
         endif
         write(wrtag,'(a13,i10,a1)') '<end data set',countpsi,'>'
         write(ipsi1) wrtag
      enddo

      return
      end subroutine subdi2ad

!#######################################################################

      subroutine trans2ad(hops,psi,psiad,subdim1,nstate1,ort)

use global
use aglobal
use maxv, only:maxdim,maxsta
use dvrdatmod
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod
use operdef

use potevalmod, only: calcdiab

      implicit none

      integer(long)                                         :: f,n,p,p1,s,s1,g,nham
      integer(long), intent(in)                             :: subdim1,nstate1
      integer(long), dimension(maxdim)                      :: igpoint
      real(dop), dimension(hopsdim), intent(in)             :: hops
      real(dop), dimension(ortdim), intent(in)              :: ort
      real(dop), dimension(maxdim)                          :: gpoint
      real(dop), dimension(maxsta)                          :: adiab
      real(dop), dimension(maxsta,maxsta)                   :: diab
      complex(dop), dimension(subdim1,nstate1), intent(in)  :: psi
      complex(dop), dimension(subdim1,nstate1), intent(out) :: psiad
      complex(dop), dimension(maxsta,maxsta)                :: rotmat

! default first point for DOFs not in grid
      do f=1,ndof
         igpoint(f)=1
         gpoint(f) = ort(zort(f))
      enddo

      do p=1,subdim(1)
         p1=p-1
         do n=1,nspfdof(1)
            f=spfdof(n,1)
            g=p1
            p1=p1/gdim(f)
            g=g - p1*gdim(f) + 1
            igpoint(f)=g
            gpoint(f) = ort(zort(f)-1+g)
         enddo                                                                  

         nham=1
         call calcdiab(hops,adiab,diab,rotmat,igpoint,gpoint,nham)

         do s=1,nstate1
            psiad(p,s)=0.0_dop
            do s1=1,nstate1
               psiad(p,s) = psiad(p,s) + dconjg(rotmat(s1,s))*psi(p,s1)
            enddo
         enddo

      enddo

      return
      end subroutine trans2ad

!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------

      subroutine di2adinput

use global
use aglobal
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod
use plot
use channels

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: iarg,ilbl,i,psilaenge
      character(len=80)       :: buf
      character(len=c5)       :: filename

      operfile  = ' '
      psifile  = ' '
      name  = './'
      fileout = 'psi.ad'
      iarg = myiargc()

      i = 0
 100  i=i+1
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if(buf .eq. '-i') then
            i = i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf(1:3) .eq. '-f ') then
            i = i+1
            call mygetarg(i,buf)
            psifile = buf
         else if( buf(1:3) .eq. '-p ') then
            i = i+1
            call mygetarg(i,buf)
            operfile = buf
         else if( buf(1:2) .eq. '-o' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf(1:3) .eq. '-w ' ) then
            lovwr=.true.
            
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(a)')&
        '--------------------------------------------------------------'
            write(6,'(1x,a)')&
       'Purpose: Converts a diabatic wavefunction to adiabatic.'
            write(6,'(1x,2a)')&
       'Usage:   di2ad [-i -f -p -o -w -ver -h -?]'
            write(6,'(/,a)') 'Options : '
            write(6,'(a)') ' -i DIR  : data is stored in directory DIR'
            write(6,'(2a,/,9x,a)') &
                     ' -f FILE : the wavefunction is read '&
                    ,'from FILE rather than from ./psi',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a)') &
                     ' -p FILE : the PES is read '&
                    ,'from FILE rather than from ./oper',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a)')&
                     ' -o FILE : The ouput is written to file ',&
                     'FILE rather than to ./psi.ad',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(2a)')&
                     ' -w      : An existing output file is ',&
                     'overwritten.'
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(/2a,/a)') 'The wavefunction and the oper-file',&
                     ' must be in "exact" format.',&
                     'One may use psi2ex to transform the WF.'
            write(6,'(a)')&
        '--------------------------------------------------------------'

            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         goto 100
      end if
      
  200 continue
  
!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='di2ad.log',form='formatted')
      logisopen=.true.

      laenge=index(name,' ')-1
      call abspath(name,laenge)

      if (operfile .eq. ' ') operfile=name(1:laenge)//'/oper'
      opflaenge=index(operfile,' ')-1
      call abspath(operfile,opflaenge)

      if (psifile .eq. ' ') psifile=name(1:laenge)//'/psi'
      psilaenge=index(psifile,' ')-1
      call abspath(psifile,psilaenge)

      dname=name
      dlaenge=index(dname,' ')-1
      call abspath(dname,dlaenge)

      laeout=index(fileout,' ')-1
      call abspath(fileout,laeout)

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      filename=psifile
      ilbl=index(filename,' ')-1
      open(ipsi,file=filename,form='unformatted',status='old',&
              err=1000)
      write(6,'(2a)') ' The psi file is : '//psifile(1:ilbl)

      filename=fileout(1:laeout)
      if (lovwr) then
         open (ipsi1,file=filename,form='unformatted',status='unknown',&
                 err=1000)
      else
         open (ipsi1,file=filename,form='unformatted',status='new',&
                 err=1010)
      endif
      write(6,'(2a)') ' The outputfile is : ',fileout(1:laeout)
      return
      
 1000 write(6,*)' Cannot open file : '
      ilbl=index(filename,' ')-1
      write(6,*) filename(1:ilbl)
      stop
 1010 write(6,*)' File already exists: '
      ilbl=index(filename,' ')-1
      write(6,*) filename(1:ilbl)
      stop
 
      end subroutine di2adinput


      end program di2ad

