
! **********************************************************************
! *                                                                    *
! *                               DIAG                                 *
! *                                                                    *
! * This routine diagonalizes the Hamiltonian matrix calculated by     *
! * program "fmat". Needs the overlap and Hamiltonian matrix files.    *
! *                                                                    *
! * SS 02.02i                                                          *
! *                                                                    *
! **********************************************************************


      program diag

use decimal, only:long,dop,sip
use global
use globmemmod, only: mcdim,mrdim,midim,mldim,msdim,mfdim
use channels
use diagmod

use op1lib, only: cpqxz
use xvlib, only: mvtxzz
use mmlib, only: mmtxzz, qdxxzz, dqxxzz, mmaxzz, dqxxdz, mmxxzz
use svdmod, only: svdz, svdhz
use logdat
use keyunitsmod
use versions

use module_interface

      implicit none

      integer(long)                        :: s,t1,t2,ilbl
      real(dop)                            :: tcpu,tsys,ttot
      real(dop), external                  :: fcputime,fsystime
      character(len=100)                   :: buffer

complex(dop), dimension(:), allocatable    :: mc
real(dop), dimension(:), allocatable       :: mr
integer(long), dimension(:), allocatable   :: mi
logical(kind=4), dimension(:), allocatable :: ml
complex(sip), dimension(:), allocatable    :: ms
real(sip), dimension(:), allocatable       :: mf


! --- READ INPUT DATA ---

      call default
      call diaginput

      call fpstartup(buffer)
      write(ilog,'(/a)') ' --- diag start ---'
      write(ilog,'(a80)') trim(buffer)

! --- COMPUTE ARRAY ADDRESSES ---

      call dallocate

! --- DYNAMICALLY ALLOCATE ARRAYS ---

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

call receiver(mi,mr,mc,ml,ms,mf)

deallocate(mc,mr,mi,ml,ms,mf)

! --- WRITE MEMORY ALLOCATION INFORMATION TO THE LOG-FILE ---

      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'(a,i10,/a,i10)')&
       ' workcdim   : ', workcdim,&
       ' workrdim   : ', workrdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)')&
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim/2,&
       ' real*4     : ', mfdim/2,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'

! --- GET TIMIMG AND WRITE TO SCREEN AND "LOG" FILE ---

      tcpu = fcputime(0)
      tsys = fsystime(0)
      ttot = tcpu+tsys
      t1   = int(ttot/3600)
      ttot = ttot - 3600*t1
      t2   = int(ttot/60)
      ttot = ttot - 60*t2

      write(ilog,'(/a,/)') '***** Overall Timing *****'
      write(ilog,'(a,1x,f12.2)') 'Total cpu time     [s] : ',tcpu
      write(ilog,'(a,1x,f12.2)') 'Total system time  [s] : ',tsys
      write(ilog,900) 'Total time     [h:m:s] : ',t1,t2,ttot
      write(6   ,900) 'Total time     [h:m:s] : ',t1,t2,ttot
  900 format( /a,i4,' :',i3,' :',f5.1)

      call fpstartup(buffer)
      write(ilog,'(/a)') ' -- diag terminated normally --'
      write(ilog,'(a80)') trim(buffer)
      write(6,'(80a)') ('*',s=1,80)

      close(ilog)

      contains

!-----------------------------------------------------------------------
!                          SUBROUTINE RECEIVER
!
! This routine is called from "caller" and continues the Diag program
! after having dynamically allocated the memory.
!-----------------------------------------------------------------------

      subroutine receiver (mi, mr, mc, ml, ms, mf)

use global
use channels
use diagmod

      implicit none

      complex(dop), dimension(1:mcdim), intent(inout)  :: mc
      real(dop), dimension(1:mrdim), intent(inout)     :: mr
      integer(long), dimension(1:midim), intent(in)    :: mi
      logical(kind=4), dimension(mldim), intent(in)    :: ml
      complex(sip), dimension(msdim), intent(in)       :: ms
      real(sip), dimension(mfdim), intent(in)          :: mf

      integer(long)       :: i,inum,ierr
      real(dop)           :: en,width,intens,en0

      routine='Receiver'

! --- READ AND MIRROR THE OVERLAP AND HAMILTONIAN MATRICES FROM 
!     MTT & MTT_HAM FILES
! --- INTENSITY VECTOR IS COMPUTED FROM OVERLAP MATRIX

      if (.not. ltneg) then
         call readmtt(iover,filein(1:laein),mc(mcover),mc(mcintensity),&
                   klo,khi,dim,mstep,product,0)
         call readmtt(iham,hamfile(1:lham),mc(mcham),mc(mcwork),&
                   klo,khi,dim,mstep,product,1)
      else
         call readmtt(iover,filein(1:laein),mc(mcwork),mc(mcwork+dim**2)&
                      ,klo,khi,dim,mstep,product,1)
         call readmtt(iover2,filein2(1:laein2),mc(mcwork+dim**2+dim),&
                      mc(mcwork+2*dim**2+dim),klo,khi,dim,mstep,&
                      product_2,1)
         call diagmirror(mc(mcwork),mc(mcwork+dim**2+dim),mc(mcover),&
                         mc(mcintensity),dim,0)

         call readmtt(iham,hamfile(1:lham),mc(mcwork),mc(mcwork+dim**2),&
                      klo,khi,dim,mstep,product,1)
         call readmtt(iham2,hamfile2(1:lham2),mc(mcwork+dim**2+dim),&
                      mc(mcwork+2*dim**2+dim),klo,khi,dim,mstep,&
                      product_2,1)
         call diagmirror(mc(mcwork),mc(mcwork+dim**2+dim),mc(mcham),&
                         mc(mcintensity),dim,1)

         dim=2*dim-1
      endif

! --- COMPUTE EIGENENERGIES AND EIGENVECTORS ---

      call dvariation(mc(mcover),mc(mcham),dim,epscut,&
                      hdim,mc(mceigenvec),mc(mceigenval),&
                      mr(mrwork),mc(mcwork),product)

      write(ilog,'(/a,i6/)') 'Final dimension for diagonalization, dim='&
                             ,hdim

! --- COMPUTE INTENSITIES ---

      call dcalcintensity(mc(mcintensity),mc(mceigenvec),dim,hdim,&
                          mc(mcwork))

! --- WRITE ENERGIES AND INTENSITIES TO OUTPUT FILE ---

      if (laeout .ne. 0) then
          inum=iout
      else
          inum=6
      endif

      en0 =1.0_dop
      call keyeunits(en0,eunit,ierr)

      do i = 1,hdim
         en     = (dble(mc(mceigenval+i-1)) + eoffset )/en0
         width  = -2.0_dop*dimag(mc(mceigenval+i-1))/en0
         intens = dble(mc(mcintensity+i-1))
         if (product .eq. 1) then
            write(inum,'(1p3e25.15e3)') en,width,intens
         else
            write(inum,'(1p2e25.15e3)') en,intens
         endif
      enddo
      close(iout)

      return
      end subroutine receiver



! ----------------------------------------------------------------------
!                          SUBROUTINE DVARIATION
!
! INPUT PARAMETERS:
!     h0,h1: Hamiltonian matrices H^(0) and H^(1)
!     hamdim: size of Hamiltonian matrices
!     epscut: cut-off parameter for eigenvalues in SVD
!     workdim: size of complex workspace
! OUTPUT PARAMETERS:
!     hdim: size of reduced Hamiltonian matrices (= no. of eigenvalues)
!     eigenvec: eigenvector matrix of Hamiltonian
!     eigenval: vector containing eigenenergies of Hamiltonian
!     error: error code having the meaning as defined above
! OTHER PARAMETERS:
!     work: complex workspace of length workdim
! ----------------------------------------------------------------------

      subroutine dvariation (h0,h1,hamdim,epscut,hdim,eigenvec,eigenval,&
                             workr,workc,product)

use global

      implicit none

      integer(long)                                           :: error
      integer(long), intent(in)                               :: hamdim,product
      integer(long), intent(out)                              :: hdim
      real(dop), intent(in)                                   :: epscut
      real(dop), dimension(1:workrdim), intent(out)           :: workr
      complex(dop), dimension(1:hamdim,1:hamdim), intent(in)  :: h0,h1
      complex(dop), dimension(1:hamdim,1:hamdim), intent(out) :: eigenvec
      complex(dop), dimension(1:hamdim), intent(out)          :: eigenval
      complex(dop), dimension(1:workcdim), intent(out)        :: workc

      integer(long)       :: wrdim1,zeigr1,zeigc1,wcdim1,&
                             wcdim2,wrdim,wcdim,i

      routine='Dvariation'

! --- INITIALISE VARIABLES ---

      error = 0

      zeigr1 = 1+hamdim
      wrdim1 = 4*hamdim
      wrdim = zeigr1+wrdim1-1
      if (wrdim .gt. workrdim) then
         message='Not enough real workspace'
         call errormsg
      endif

      zeigc1 = 1+hamdim**2
      wcdim1 = hamdim**2+2*hamdim
      wcdim2 = hamdim**2+4*hamdim
      wcdim = zeigc1+wcdim2-1
      if (wcdim .gt. workcdim) then
         message='Not enough complex workspace'
         call errormsg
      endif

! --- SAVE THE HAMILTONIAN MATRICES ---

      call cpqxz(h0,workc(1),hamdim)
      call cpqxz(h1,eigenvec,hamdim)

! --- SOLVE GENERALISED EIGENVALUE PROBLEM ---

      if (product .eq. 1) then
          call svdz(eigenvec,workc(1),hamdim,epscut,wcdim2,eigenval,&
                    hdim,error,workc(zeigc1))
      else
          call svdhz(eigenvec,workc(1),hamdim,epscut,workr(1),&
                  hdim,workr(zeigr1),workc(zeigc1),&
                  wrdim1,wcdim1,error)
         do i=1,hdim
            eigenval(i) = dcmplx(workr(i),0.0_dop)
         enddo
      endif 

      if (error .ne. 0) then
         message='Error in solving generalized eigenvalue problem'
         call errormsg
      endif

      return
      end subroutine dvariation

! ----------------------------------------------------------------------
!                        SUBROUTINE CALC INTENSITY
!
!  This routine 'dcalcintensity' is identical to 'ccalcintensity'
!  on source/fd_cmplx/ccalculate.F
!
! PURPOSE:
!     Computes the intensities.
! INPUT PARAMETERS:
!     intensity: intensity vector as determined by "calcintensvec"
!     coeff: coefficient matrix (B in the PhD thesis)
!     hamdim: size of Hamiltonian matrices
!     hdim: second dimension of coefficient matrix
! OUTPUT PARAMETERS:
!     intensity: vector containing the intensities
! OTHER PARAMETERS:
!     work: complex workspace; must be >= hdim
! ----------------------------------------------------------------------

      subroutine dcalcintensity (intensity, coeff, hamdim, hdim, work)

      implicit none

      integer(long)                                        :: i
      integer(long), intent(in)                            :: hamdim,hdim
      complex(dop), dimension(1:hamdim), intent(inout)     :: intensity
      complex(dop), dimension(1:hamdim,1:hdim), intent(in) :: coeff
      complex(dop), dimension(1:hdim), intent(out)         :: work


! --- DETERMINE OVERLAPS OF INITIAL WAVEFUNCTION AND EIGENFUNCTIONS ---

      call mvtxzz(coeff,intensity,work,hamdim,hdim)

! --- COMPUTE SPECTRAL INTENSITIES ---

      do i = 1,hdim
         intensity(i) = dcmplx(abs(work(i))**2,0.0_dop)
      enddo

      return
      end subroutine dcalcintensity


!***********************************************************************

      subroutine diaginput

use global
use channels
use diagmod

      implicit none

      integer(long), external  :: myiargc
      integer(long)            :: i,khi1,khi2,khi3,khi4,klo1,klo2,klo3,klo4,&
                                  product1,product2,product3,product4,length,&
                                  ierr,iarg,ilbl,step1,step2,step3,step4,dim1,error
      real(dop)                :: e0
      character(len=240)       :: buf,filename
      character(len=10)        :: eoffunit
      logical(kind=4)          :: lexist


      routine='Diaginput'
 
! --- SET DEFAULT VALUES ---

      name    = '.'
      dname   = '.'
      oname   = '.'
      laenge  = 1
      dlaenge = 1
      olaenge = 1
      fileout = './'
      diagfile  = 'diag'
      logfile   = 'diag.log'
      ldiag  = index(diagfile,' ')-1
      llog   = index(logfile,' ')-1

      eunit='au'
      epscut=1.0e-10_dop
      eoffset=0.0_dop
      lovwr=.false.
      ltneg=.false.

      filein=' '
      laein=1
      hamfile=' '
      lham =1 
      filein2=' '
      laein2=1
      hamfile=' '
      lham2=1
      mlow =1
      mhigh=99999
      mstep=1

      iover=10
      iover2=11
      iham=12
      iham2=13
      ilog=14
      iout=15

      iarg  = myiargc()
      if (iarg .le. 0) goto 200
      call mygetarg(0,options)
      optlaenge  = index(options,' ')
      do i = 1, iarg
         call mygetarg(i,buf)
         ilbl  = index(buf,' ')
         ilbl = min(ilbl,100-optlaenge)
         if(ilbl.gt.0) options = options(1:optlaenge)//buf(1:ilbl)
         optlaenge = optlaenge + ilbl
      end do


! --- READ THE OPTIONS ---
 200  continue
      i=0
 10   continue
      i=i+1
      call mygetarg(i,buf)
      if (buf(1:1) .eq. ' ') go to 100

      if (buf(1:1) .eq. '-') then
         if (buf(2:3) .eq. 'd ') then
            i=i+1
            call mygetarg(i,buf)
            length=index(buf,' ')-1
            read(buf(1:length),'(a)') fileout
         else if (buf(2:3) .eq. 'o ') then
            i=i+1
            call mygetarg(i,buf)
            length=index(buf,' ')-1
            read(buf(1:length),'(a)') diagfile
            ldiag = length
         else if (buf(2:3) .eq. 'lo') then
            i=i+1
            call mygetarg(i,buf)
            length=index(buf,' ')-1
            read(buf(1:length),*) mlow
         else if (buf(2:3) .eq. 'hi') then
            i=i+1
            call mygetarg(i,buf)
            length=index(buf,' ')-1
            read(buf(1:length),*) mhigh
         else if (buf(2:3) .eq. 'n ') then
            i=i+1
            call mygetarg(i,buf)
            length=index(buf,' ')-1
            read(buf(1:length),*) mstep
         else if (buf(2:3) .eq. 't ') then
            ltneg=.true.
         else if (buf(2:4) .eq. 'es ') then
            i=i+1
            call mygetarg(i,buf)
            length=index(buf,' ')-1
            read(buf(1:length),*) epscut
         else if (buf(2:3) .eq. 'e ') then
            i=i+1
            call mygetarg(i,buf)
            length=index(buf,' ')-1
            if (buf(1:length) .eq. 'lsth') then
               eoffset = 4.747459669_dop
               eoffunit='ev'
               call keyeunits(eoffset,eoffunit,ierr)
            elseif (buf(1:length) .eq. 'co2') then 
               eoffset = -2534.5298_dop
               eoffunit='cm-1'
               call keyeunits(eoffset,eoffunit,ierr)
            else
               read(buf(1:length),*) eoffset
               i=i+1
               call mygetarg(i,buf)
               length=index(buf,' ')-1
               read(buf(1:length),'(a)') eoffunit
               call keyeunits(eoffset,eoffunit,ierr)
            endif
            if (ierr .ne. 0) then
               write(6,'(a)') &
               'Illegal parameters are given with option "-e".'
               stop
            endif
         else if (buf(2:3) .eq. 'u ') then
            i=i+1
            call mygetarg(i,buf)
            length=index(buf,' ')-1
            read(buf(1:length),'(a)') eunit
            e0=1.0_dop
            call keyeunits(e0,eunit,ierr)
            if (ierr .ne. 0) then
                write(6,'(a)') 'Undefined energy unit is given.'
               stop
            endif
         else if (buf(2:3) .eq. 'w ') then
            lovwr=.true.
         else if (buf(2:4) .eq. 'ver') then
            call wrversion(6)
            stop
         else if (buf(2:3) .eq. 'h ' .or. buf(2:3) .eq. '? ') then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(2x,2a)')&
      'Purpose : Diagonalizes the Hamiltonian matrix calculated by ',&
                 'fmat.'
            write(6,'(2x,2a,/,19x,a)')&
                 'Usage   : diag<ver> [-d -o -lo -hi -n -t -es -e -u ',&
                 '-w -ver -h -?] mtt_ovl ',&
                 'mtt_ham [[mtt_ovl_2] [mtt_ham_2]].'
            write(6,'(/,a)') ' Options  : '
            write(6,'(2a,/,12x,a/,12x,a)')&
                     ' -d PATH  : The output is directed to PATH ',&
                     'rather than to the current directory.',&
            'The string PATH may be a relative or a full path-name.',&
            'If PATH=no, i.e. "-d no", no output files will be opened.'
            write(6,'(2a)') ' -o NAME  : Change the name of output ',&
                     'file from "diag" to NAME.'
            write(6,'(2a)') ' -lo klo  : Take matrix elements in Mtt ',&
                     'files only from the klo-th index. '
            write(6,'(2a)') ' -hi khi  : Take matrix elements in Mtt ',&
                     'files only up to the khi-th index. '
            write(6,'(2a)') ' -n step  : Take only every n-th step ',&
                     'of the Mtt matrix elements. '
            write(6,'(3a/,12x,2a/,12x,a)') ' -t       : Expand the ',&
                     'nXn mtt-matrix ',&
                     'to (2n-1)X(2n-1) one including negative ',&
                     'times. In this case the second set of overlap ',&
                     'and Hamiltonian ','matrices are needed.'
            write(6,'(2a)') ' -es eps  : Set cut-off parameter ',&
                     'for eigenvalues. Default is eps=1.e-10'
            write(6,'(2a,/,12x,a,/18x,a)') ' -e E unit: The offset ',&
                     'energy is set to E unit.',&
                    'Note: "-e lsth" is equivalent to "-e 4.74746 ev".',&
                     '"-e co2" is equivalent to "-e -2534.5298 cm-1".'
            write(6,'(2a)') ' -u UNIT  : Set the output energy unit ',&
                      'to UNIT. Default is au.'
            write(6,'(2a)')  ' -w       : Overwrite an existing "diag"',&
                            ' file.'
            write(6,'(a)')&
                    ' -ver     : Version information about the program '
            write(6,'(a,/a)')&
                     ' -h       : Print this help text. ',&
                     ' -?       : Print this help text. '
            write(6,'(2a,/2a,/2a,/a/)') ' The arguments "mtt_ovl" and ',&
                     '"mtt_ham" are the Overlap- and Hamiltonian ',&
                     ' matrix files calculated by the',&
                     ' "fmat" program. If "-t" option is used, ',&
                     ' the second set of Overlap-',&
                     ' and  Hamiltonian matrices with different ',&
                     ' scalar product is needed.'
            write(6,'(2a)')  ' Examples  : diag<ver> -es 1.0e-8_dop -hi 60 ',&
                            ' mtt_h mtt_H1_h '
            write(6,'(2a)')  '           : diag<ver> -hi 40 -n 2 ',&
                            '-t mtt_h mtt_H1_h  mtt_s mtt_H1_s'
            write(6,'(2a,/,17x,2a)') ' Important note: Both overlap ',&
                     'and Hamiltonian-matrices should be calculated ',&
                     'either as hermitian or as symmetric ',&
                     'scalar products.'
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else
            write(6,'(2a)') ' unknown option : ', buf
            stop
         endif
      else
         length=index(buf,' ')-1
         if (filein(1:1) .eq. ' ') then
            read(buf(1:length),'(a)') filein
            laein=length
            call abspath(filein,laein)
         elseif (hamfile(1:1) .eq. ' ') then
            read(buf(1:length),'(a)') hamfile
            lham=length
            call abspath(hamfile,lham)
         elseif (filein2(1:1) .eq. ' ') then
            read(buf(1:length),'(a)') filein2
            laein2=length
            call abspath(filein2,laein2)
         else
            read(buf(1:length),'(a)') hamfile2
            lham2=length
            call abspath(hamfile2,lham2) 
         endif
      endif

      goto 10
 100  continue   

! --- Check the variables --- 

      error = 0

      if (filein(1:1) .eq. ' ') then
         message='No overlap file for input is given.'
         error=1
      elseif (hamfile(1:1) .eq. ' ') then
         message='No Hamiltonian file for input is given'
         error=1
      endif

      if (ltneg) then
         if (filein2(1:laein2) .eq. ' ') then 
            message='Second overlap-matrix file is missed.'
            error=1
         elseif (hamfile2(1:lham2) .eq. ' ') then
            message='Second Hamiltonian-matrix file missed.'
            error=1
         endif
      endif 

      if (mhigh .lt. mlow) then
         message=' khi must be greater than klo'
         error=1
      endif
      if (epscut .lt. 0) then
         message='epscut must be a positive integer'
         error=1
      endif
      if (mlow .lt. 0 .or. mhigh .lt. 0) then
         message='klo and khi must be positive integers'
         error=1
      endif
      if (mstep .le. 0) then
         message='step should be defined as a positive integer'
         error=1
      endif

      if (ltneg .and. mlow.ne.1) then
         message='The option "-t" should be used only with klo=1'
         error=1
      endif

      if (error .ne. 0) then
         call errormsg
      endif

! ----------------------------  

      if (fileout(1:2) .eq. 'no') then
         laeout = 0
      else
         laeout=index(fileout,' ')-1
         if (fileout(laeout:laeout) .ne. '/') then
            fileout = fileout(1:laeout)//'/'
            laeout = laeout + 1
         endif
         logfile = fileout(1:laeout)//logfile(1:llog)
         llog = laeout + llog
         call abspath(logfile,llog)
         fileout = fileout(1:laeout)//diagfile(1:ldiag)
         laeout = laeout + ldiag
         call abspath(fileout,laeout)
      endif
 

! --- READ INFORMATION FROM MTT & MTT_HAM FILES ---

      call mttinfo(iover,filein,klo1,khi1,step1,product1)
      call mttinfo(iham,hamfile,klo2,khi2,step2,product2)
      if (klo1 .ne. klo2 .or. khi1 .ne. khi2 .or. step1 .ne. step2) then
         message='Dimensions for overlap and Hamiltonian '//&
                 'matrices are different'
         call errormsg
      endif
      if (product1 .ne. product2) then
         message='Both overlap and Hamiltonian matrices should '//&
                 'be calculated  either as hermitian or as '//&
                 'symmetric scalar products.'
         call errormsg
      endif
      dim1=int((khi1-klo1)/step1)+1

      if (ltneg) then
         call mttinfo(iover2,filein2,klo3,khi3,step3,product3) 
         call mttinfo(iham2,hamfile2,klo4,khi4,step4,product4)
         if (klo3.ne.klo4 .or. khi3.ne.khi4 .or. step3.ne.step4) then
            message='Dimensions for the 2-nd set of overlap and '//&
                    'Hamiltonian-matrices are different.'
            call errormsg
         else if (product3 .ne. product4) then
            message='Second set of Overlap and Hamiltonian-matrices '//&
                    'should be calculated either as '//&
                    'symmetric or as hermitian scalar products.'
            call errormsg
         endif
         if (klo3.ne.klo1 .or. khi3.ne.khi1 .or. step3.ne.step1) then
            message='Dimension error between two sets of Overlap '//&
                    'and Hamiltonian matrices.'
            call errormsg
         else if (product3 .eq. product1) then
            message='Two sets of Overlap and Hamiltonian matrices '//&
                    'should be different scalar products.'
            call errormsg
         endif
         product_2=product3
      endif 

      klo=max(klo1,mlow)
      khi=min(dim1,mhigh)
      dim=int((khi-klo)/mstep)+1
      product=product1


! --- WRITE STARTING INFORMATIONS TO THE SCREEN AND TO THE "LOG" FILE ---

      write(6,'(80a)') ('*',i=1,80)
      write(6,'(2a/)')   ' Options : ', options(1:optlaenge)
      write(6,'(2a)') ' The overlap matrix file is     : ',&
                      filein(1:laein)
      write(6,'(2a)') ' The hamiltonian matrix file is : ',&
                      hamfile(1:lham)
      if (laeout .ne. 0) then
         write(6,'(2a)') ' The output file is             : ',&
                       fileout(1:laeout)
      else
         write(6,'(2a)') ' No outputfiles written '
      endif
      write(6,'(a)')  ' In mtt-files : '
      write(6,'(a,i6,a,i6,a,i4,a,i6)') '         klo =', klo1,&
                      ',  khi =',khi1,',  step =',step1,',  dim =',dim1
      write(6,'(a)')  ' In Diagonalization : '
      write(6,'(a,i6,a,i6,a,i4,a,i6)') '         klo =', klo,&
                      ',  khi =',khi,',  step =',mstep,',  dim =',dim
      if (product .eq. 1) then
         write(6,'(a)') ' Symmetric scalar product is used.'
      else
         write(6,'(a)') ' Hermitian scalar product is used.'
      endif
      if (ltneg) then
         write(6,'(2a,i6)') ' Matrices are extended including negative',&
                            ' times. dim=',2*dim-1
      endif

      if( laeout .ne. 0) then 
         filename = logfile(1:llog)
         open (unit=ilog,file=filename(1:llog),form='formatted',err=13)
         logisopen=.true.

         write(ilog,'(2a/)')   ' Options : ', options(1:optlaenge)
         write(ilog,'(2a)') ' The overlap matrix file is     : ',&
                            filein(1:laein)
         write(ilog,'(2a)') ' The hamiltonian matrix file is : ',&
                            hamfile(1:lham)
         write(ilog,'(2a)') ' The output file is             : ',&
                            fileout(1:laeout)
         write(ilog,'(a)')  ' In mtt-files : '
         write(ilog,'(a,i6,a,i6,a,i4,a,i6)') '         klo =', klo1,&
                      ',  khi =',khi1,',  step =',step1,',  dim =',dim1
         write(ilog,'(a)')  ' In Diagonalization : '
         write(ilog,'(a,i6,a,i6,a,i4,a,i6)') '         klo =', klo,&
                      ',  khi =',khi,',  step =',mstep,',  dim =',dim
         if (product .eq. 1) then
            write(ilog,'(a)') ' Symmetric scalar product is used.'
         else
            write(ilog,'(a)') ' Hermitian scalar product is used.'
         endif
         if (ltneg) then 
            write(ilog,'(2a,i6)') ' Matrices are extended including ',&
                               'negative times. dim=',2*dim-1
         endif
      else
         open (unit=ilog,status='scratch',form='formatted')
      endif

! --- Open and write some information to the output file ---
      if (laeout .ne. 0) then
         filename = fileout(1:laeout)
         inquire(file=filename(1:laeout),exist=lexist)
         if (lexist .and. .not.lovwr) then 
            write(6,'(/,a,/,3a,/a,/a)')&
            '#########################################################',&
            ' The output file: ',filename(1:laeout),' already exists!',&
            ' overwrite disabled? (Use -w option to enable overwrite)',&
            '#########################################################'
            stop
         endif
         open(unit=iout,file=filename(1:laeout),form='formatted',err=13)
         write(iout,'(2a/)') '# Options : ', options(1:optlaenge)
         write(iout,'(a,i6,a,i6,a,i4,a,i6)') '# klo =', klo,',  khi =',&
               khi,',  step =',mstep,',  dim =',dim
         if (ltneg) then
            write(iout,'(2a,i6)') '# Matrices are extended including ',&
                               'negative times. dim=',2*dim-1
         endif
         if (product .eq. 1) then
            write(iout,'(a)') '# Symmetric scalar product is used.'
            write(iout,'(/2a/)') '#  Energy                   Width',&
                  '                     Intensity'
         else
            write(iout,'(a)') '# Hermitian scalar product is used.'
            write(iout,'(/a/)') '#  Energy                   Intensity'
         endif
      else
         if (product .eq. 1) then
            write(6,'(/2a/)') '   Energy                   Width',&
                  '                     Intensity' 
         else
            write(6,'(/a/)') '   Energy                   Intensity'
         endif
      endif
      return

 13   write(6,'(a)') ' Cannot open file : ' 
      write(6,'(a)')  filename
      stop

      end subroutine diaginput

!#######################################################################
!  Subroutine Dallocate
!
!  Computes pointers to the addresses of the integer, real and complex 
!  arrays nedded for diag routine
!
!  Input:  dim - dimension of the overlap and Hamiltonian matrices
!  Output: midim - length of the integer array
!          mrdim - length of the real array
!          mcdim - length of the complex array
!          Different pointers to intensity, eigenvector and
!                    eigenvalue arrays
!  All data are given via common blocks
!#######################################################################

      subroutine dallocate

use global
use diagmod

      implicit none

      integer(long)       :: dim1

      if (ltneg) then
         dim1=2*dim-1
      else
         dim1=dim
      endif

! --- SET POINTERS TO MEMORY ADDRESSES OF INTEGER ARRAYS ---

      midim = 1

! --- SET POINTERS TO MEMORY ADDRESSES OF REAL ARRAYS ---

      mrdim = 1
      mrwork = mrdim
      workrdim = 5*dim1
      mrdim = mrdim+workrdim-1

! --- ...
      msdim = 1
      mldim = 1
      mfdim = 1

! --- SET POINTERS TO MEMORY ADDRESSES OF COMPLEX ARRAYS ---

      mcdim = 1
      mcover = mcdim
      mcdim = mcdim+dim1*dim1
      mcham = mcdim
      mcdim = mcdim+dim1*dim1
      mcintensity = mcdim
      mcdim = mcdim+dim1
      mceigenvec = mcdim
      mcdim = mcdim+dim1*dim1
      mceigenval = mcdim
      mcdim = mcdim+dim1
      mcwork = mcdim
      workcdim = 1
! for subroutine dvariation and for the case of ltneg (for negative times)
      workcdim = 2*dim1**2+4*dim1
! for subroutine dcalcintensity
!      workcdim = max(workcdim,dim1)
      mcdim = mcdim+workcdim-1

      return
      end subroutine dallocate

!#######################################################################
!  Subroutine Mttinfo
!
!  Reads information from Mtt file
!  On output: klo,khi - lower and higher bounds of the overlap and
!                       Hamiltonian matrices
!             product = 1, if symmetric scalar product is used
!                     = 2, if hermitian scalar product is used
!
!#######################################################################

      subroutine mttinfo(imtt,mttfile,klo,khi,step,product1)

use global

      implicit none


      integer(long)                 :: lmtt,dummy
      integer(long), intent(in)     :: imtt
      integer(long), intent(out)    :: klo,khi,step,product1
      character(len=c3)             :: string
      character(len=2)              :: flag
      character(len=c5), intent(in) :: mttfile

      routine='Mttinfo'

      lmtt=index(mttfile,' ')-1
      open(imtt,file=mttfile(1:lmtt),status='old',err=10)

 100  continue
      read(imtt,'(a)',err=10,end=30) string
      if (string(1:7) .eq. '#   klo') then 
         read(imtt,'(a,i5,3i7)',err=10,end=30) flag,klo,khi,dummy,step
         read(imtt,'(a)',err=10,end=30) string
         if (string(1:26) .eq. '# symmetric scalar product') then
             product1=1
         elseif (string(1:26) .eq. '# hermitian scalar product') then
             product1=2
         else
             go to 35
         endif
      else
          goto 100
      endif

      if (khi .lt. klo) then
         message='khi should be greater than klo'
         call errormsg
      endif

      close(imtt)
      goto 40
 
 10   message='Cannot open file '//mttfile(1:lmtt)
      call errormsg
 30   message='End of file '//mttfile(1:lmtt)//' has been reached.'
      call errormsg
 35   message='Overlap or Hamiltonian matrices should be '//&
              'calculated either as symmetric or as hermitian '//&
              'scalar products.'
      call errormsg

 40   return
      end subroutine mttinfo


!#######################################################################
!  Subroutine Readmtt
!
!  Reads overlap or Hamiltonian matrices from Mtt file
!#######################################################################

      subroutine readmtt(imtt,mttfile,matrix,intensity,klo,khi,dim,&
                         mstep,product,num)

use global

      implicit none

      integer(long)                                 :: lmtt,i,n,s,s1
      integer(long), intent(in)                     :: imtt,klo,khi,dim,num,product,mstep
      real(dop)                                     :: re,im,dummy
      complex(dop), dimension(dim), intent(out)     :: intensity
      complex(dop), dimension(dim,dim), intent(out) :: matrix
      character(len=c4)                             :: string
      character(len=c5), intent(in)                 :: mttfile

      routine='Readmtt'

      lmtt=index(mttfile,' ')-1
      open(imtt,file=mttfile(1:lmtt),status='old',err=10)

 100  continue
      read(imtt,'(a)',err=10,end=30) string
      if (string(1:7) .ne. '#  Time') goto 100

      do n=1,klo-1
         do i=1,n
            read(imtt,*,err=10,end=30) dummy,dummy,re,im 
         enddo
      enddo
      do n=klo,khi
         do i=1,n
            read(imtt,*,err=10,end=30) dummy,dummy,re,im
            if (mod(n-klo,mstep) .eq. 0 .and. i .ge. klo .and.&
                mod(i-klo,mstep) .eq. 0) then
                s =int((n-klo)/mstep)+1
                s1=int((i-klo)/mstep)+1
                matrix(s1,s) = dcmplx(re,im) 
                if (s1 .ne. s) then
                   if (product .eq. 1) then
                       matrix(s,s1) = matrix(s1,s)
                   else
                       matrix(s,s1) = dconjg(matrix(s1,s))
                   endif
                endif  
            endif
         enddo
      enddo

      if (num .eq. 0) then
         do i=1,dim
            intensity(i) = matrix(1,i)
         enddo
      endif

      goto 200

 10   message='Cannot open file '//mttfile(1:lmtt)
      call errormsg
 30   message='End of file '//mttfile(1:lmtt)//' has been reached.'
      call errormsg

 200  continue
      return
      end subroutine readmtt


!#######################################################################
!  Subroutine Diagmirror
!
!  Expands the Hamiltonian matrix to negative times.
!#######################################################################


      subroutine diagmirror(h1,h2,h,intensity,dim,num)

      implicit none

      integer(long)                                         :: i,j
      integer(long), intent(in)                             :: dim,num
      complex(dop), dimension(2*dim-1), intent(out)         :: intensity
      complex(dop), dimension(dim,dim), intent(in)          :: h1,h2
      complex(dop), dimension(2*dim-1,2*dim-1), intent(out) :: h


      do i=1,dim
         do j=1,dim
            h(i,j)=dconjg(h1(dim-i+1,dim-j+1))
         enddo
      enddo

      do i=1,dim
         do j=2,dim
            h(i,j+dim-1)=h2(dim-i+1,j)
         enddo
      enddo 

      do i=2,dim
         do j=1,dim
            h(i+dim-1,j)=dconjg(h2(i,dim-j+1))
         enddo
      enddo

      do i=2,dim
         do j=2,dim
            h(i+dim-1,j+dim-1)=h1(i,j)
         enddo
      enddo  

      if (num .eq. 0) then
         do i=1,2*dim-1
            intensity(i)=h(1,i)
         enddo
      endif

      return
      end subroutine diagmirror

      end program diag

