!***********************************************************************
!
!                         ANALYSIS subroutine DIFFMAP
!
! Routine to calculate a diffusion map of the wavefunction. Based on the
! analysis program PSI2EX.
! GWR 02/20
!***********************************************************************

      module diffmapmod

use decimal, only: long,dop,sip

      real(dop), save       :: thrsmax,thrsmin
      logical(kind=4), save :: lsp,lerr,labs,labs2,lascii,lrst,linwf,&
           lwght,lthrsmax,lsobol

      real(dop), parameter :: bohr=0.52917721_dop

! Derived type linked list to hold sampled DVR points 
! (number of wavefunctions to be sampled is unknown to start with). 
      type dmsamp
         real(dop) :: time
         type(dmsamp), pointer :: next,prev
!         real(dop), dimension(:), allocatable :: jac
         real(dop), dimension(:,:), allocatable :: dmsort
      end type dmsamp

      type(dmsamp), pointer :: psisamp,psisamp2
      type(dmsamp), target :: ipsisamp

! Other variables.
      integer(long), save   :: nsample=50,st=1,imethod=1,kpar=1
      real(dop), save :: kerscal=1.0_dop,alpha=1.0_dop,epsilon=1.0_dop

      end module diffmapmod
      
      program diffmap

use decimal, only: long,dop,sip,vlong
use global
use globmemmod
use aglobal
use versions
use dvrdatmod
use griddatmod
use memdimmod
use psidef
use channels
use datenmod
use compdatmod
use compdat1mod
use dirdyn
use operdef
use gapmod

use op1lib, only: cpvxz, initvxi
use mtlib, only: mtxxzz
use logdat
use compdat, only: savecsys,savecpsi,getcpsi
use iofile, only: rdpsiinfo,wrpsiinfo,rdchkinfo
use iogrddef, only: griddat
use iopsidef, only: psidat
use rddvrmod, only: dvrinfo
use einsbasmod, only: einexact
use auswutil, only: savecdat
use zeigauswmod, only: zeigausw
use iorst, only: rstinfo
use rdopermod, only: operinfo

use module_interface
use diffmapmod

      implicit none

      logical(kind=4)      :: lopened
      integer(long)        :: m,ilbl,chkdvr,chkgrd,chkpsi,chkdat,f,chkprp,&
           check
      real(dop)            :: totmem

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      lsp=.true.
!      lpop2=.false.
      lswap=.false.
      labs=.false.
      labs2=.false.
      lascii=.false.
      lwght=.false.
      lrst=.false.
      call diffmapinput

!-----------------------------------------------------------------------
! Files to be read
!-----------------------------------------------------------------------
!      lrddvr=lpop2
      lrddvr=.false.

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (ilog,file='diffmap.log',form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      if (lrst) then
         call rdmemdim(irst)
      else
         call rdmemdim(ipsi)
      endif

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef
!      call alloc_compdat

!-----------------------------------------------------------------------
!  Read Check file info.
!-----------------------------------------------------------------------
      read(ichk) filever(ichk)
      if (filever(ichk) .lt. 11.0_dop) then
         write(6,'(a)') '****************************************'
         write(6,'(2a)')   &
          'For check files older than version 11.0 use appropriate ', &
          'Rdcheck program'
         write(6,'(a,f8.4)') ' Check file version =', filever(ichk)
         write(6,'(a)') '****************************************'
         stop
      endif
      check = 1
      call rdchkinfo(ichk,check,check,check,check)
      tinit = tinit/fs
      tfinal = tfinal/fs
      out1  = out1/fs

!-----------------------------------------------------------------------
! read data from psi file
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      if (lrst) then
         irst=ipsi
         chkprp=0
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         read(irst,err=998) tinit
      else
         read(ipsi) filever(ipsi)
         call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      endif
!      if (psitype .eq. 1) then
!         routine='Diffmap'
!         message = 'Wavefunction already in exact representation'
!         call errormsg
!      endif
      if (chkerr(1) .ne. 0) go to 999

!-----------------------------------------------------------------------
! read data from dvr file
!-----------------------------------------------------------------------
      lort=.true.
      open(idvr,file=dname(1:dlaenge)//'/dvr',form='unformatted',&
           status='old',err=997)
      call dvrinfo(lerr,chkdvr)
      if (lerr) then 
         write(6,*) ' Error reading: dvr'
         stop
      end if
      close(idvr)

!-----------------------------------------------------------------------
! read data from oper file
!-----------------------------------------------------------------------
      if(ldd.or.lddmctdh)then
         open(ioper,file=dname(1:dlaenge)//'/oper',form='unformatted',&
              status='old',err=996)
         call rdmemdim(ioper)
         call alloc_operdef
         call alloc_dirdyn
         call alloc_dddb
         call alloc_dercp
         call alloc_gapkernel
         call operinfo(lerr,chkdvr,chkgrd)
         close(ioper)
      endif

!-----------------------------------------------------------------------
! set up and save exact psi information
!-----------------------------------------------------------------------
      lpsiopt(1)=lsp
      lpsiopt(2)=.false.
      lpsiopt(3)=.false.
      lpsiopt(4)=.false.
      griddim=nstate
      do m=1,nmode
         if(griddim.lt.huge(1_long)/subdim(m))then
            griddim=griddim*subdim(m)
         else
            griddim=huge(1_long)
            write(*,'(a)')'Total grid dimension overflow. Set to huge(1_long).'
            exit
         endif
      enddo

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
!-----------------------------------------------------------------------
      lpsi=.true.
!      lpsigrd=.true.
      lpsigrd=.false.
      ltrafo=lpop2.or.lwght
      lfft=.false.
!     is any of the degrees of freedom of fft or expDVR type?
      do f=1,ndof
         if(basis(f).eq.4 .or. basis(f).eq.5) lfft=.true.
      end do
      lfft=lpop2.and.lfft  ! don't set lfft if not needed

      call zeigausw

!-----------------------------------------------------------------------
! assign memory arrays
!-----------------------------------------------------------------------
      call memausw
      mrweight=mrdim
      if(lwght .or. lpop2) then
         mrdim=mrweight+ortdim
      else
         mrdim=mrweight+1
      endif

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
!      workcdim=max(workcdim,2*griddim+cdgldim(1))

      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim 
      msdim   = msworks+worksdim

      mfworkf = mfdim 
      mfdim   = mfworkf+workfdim
      
!-----------------------------------------------------------------------
! call conversion routine
!-----------------------------------------------------------------------
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      totmem = 16.0_dop*dble(mcdim) + 8.0_dop*dble(mrdim) + 4.0_dop*dble(midim)&
             +  4.0_dop*dble(mldim) + 8.0_dop*dble(msdim) + 4.0_dop*dble(mfdim) 

      call wrversion(ilog)
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', totmem/1048576.0_dop,' MB'

      if(imethod.eq.1)then
         write(ilog,'(a)') 'Using diffusion map.'
         write(ilog,'(a,i4)') 'State of interest: ',st
         write(ilog,'(a,f8.4)') 'Kernel scaling parameter: ',kerscal
         write(ilog,'(a,f8.4)') 'Alpha parameter: ',alpha
         write(ilog,'(a,f8.4)') 'Min. WF density factor: ',thrsmin
         write(ilog,'(a,f8.4)') 'Max. WF density factor: ',thrsmax
         write(ilog,'(a,i4)') 'Timesteps between WF sampling: ',step
         write(ilog,'(a,i4)') 'No. of WFs skipped before first sample: ',nskip
         write(ilog,'(a,i4)') 'No. of gridpoints sampled for each WF: ',nsample
         if(lsobol)then
            write(ilog,'(a)') 'Sampling method: Sobol'
         else
            write(ilog,'(a)') 'Sampling method: Random'
         endif
      else if(imethod.eq.2)then
         write(ilog,'(a)') 'Using e-isomap.'
         write(ilog,'(a,f8.4)') 'Distance parameter: ',epsilon
      else if(imethod.eq.3)then
         write(ilog,'(a)') 'Using K-isomap.'
         write(ilog,'(a,i6)')'No. of nearest neighbours: ',kpar
      else
         write(ilog,'(a)') 'Unrecognised method.'
         stop
      endif
!      write(ilog,'(a,f8.4)') 'Density sampling threshold: ',thrs
      write(ilog,'(a,i10)') 'Total grid dimension:',griddim

      call myflush(ilog)

      if( abs(dble(ilbl)-totmem) .gt. 1.0_dop ) then
         routine='Diffmap'
         write(message,'(3a,f9.1,a)')&
           'Integer overflow in memory assignment. ',&
           'You are asking for too much memory.    ',&
           'Total Memory asked for =',totmem/1048576.0_dop,' MB'  
         call errormsg
      endif
 
if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

!-----------------------------------------------------------------------
! recall psi data for length of arrays
!-----------------------------------------------------------------------

!      call subdiffmap(mc(mcpsi),ms(mspsi),mc(mcworkc),mi(miworki),&
!           mi(mijindx),dgldim,mr(mrort),mr(mrweight),&
!           mr(mrtrafo),mc(mcexphin),mc(mcrueck),mi(mifftfak))

      call subdiffmap(mc(mcpsi),ms(mspsi),&
           mi(mijindx),dgldim,mr(mrort),mr(mrweight),&
           mr(mrtrafo),mc(mcexphin),mc(mcrueck),mi(mifftfak))

deallocate(mc,mr,mi,ml,ms,mf)
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_psidef
      call dealloc_operdef
      call dealloc_dirdyn
      call dealloc_dddb
      call dealloc_dercp
      call dealloc_gapkernel
!      call dealloc_compdat
      inquire(ichk,opened=lopened)
      if(lopened) close(ichk)
      inquire(ilog,opened=lopened)
      if(lopened) close(ilog)
      inquire(ipsi,opened=lopened)
      if(lopened) close(ipsi)

      stop
  996 write(6,'(a,/a)') 'Cannot open oper-file:',dname(1:dlaenge)//'/oper'
      stop 1
  997 write(6,'(a,/a)') 'Cannot open dvr-file:',dname(1:dlaenge)//'/dvr'
      stop 1
  998 continue
      write(6,'(a)') 'Error reading time from restart-file.'
      stop 1
  999 continue
      write(6,'(a)') 'Error reading psi-file.'
      stop
      contains

!***********************************************************************
!     calculates the exact wavefunction
!***********************************************************************

      subroutine subdiffmap (psi1,spsi,jindx,dgldim1,&
                            ort,weight,trafo,exphin,rueck,fftfak)

use compdat, only: getcpsi,getcsys
use rdfilesmod, only: rdpsi,rdchk,lrdchk
use rddvrmod
use psiutil, only: showpop2, getweight, calcpsis
use diffmapmod
use dd_db, only: ddq2x

      implicit none

      integer(long)                                   :: i,j,k,d,iz,chkdvr
      integer(long)                                   :: m,n,f,ndof1,nmode1
      integer(long), dimension(maxdim)                :: igpoint
      integer(long), dimension(maxdim)               :: findx
      integer(long), intent(in)                       :: dgldim1
      integer(long), dimension(1)                     :: idum
      integer(long), dimension(adim), intent(out)     :: jindx
!      integer(long), dimension(workidim), intent(out) :: worki
      integer(long), dimension(fftdim), intent(out)   :: fftfak
      real(dop)                                       :: time,dt,now,spent
      real(dop), dimension(1)                         :: rdum
      real(dop), dimension(gwpinfodim)                :: rdum1
      real(dop), dimension(maxsta)                    :: rdum2
      real(dop), dimension(d3matdim)                  :: rdum3
      real(dop), dimension(dvrdim), intent(out)       :: trafo
      real(dop), dimension(ortdim), intent(out)       :: ort,weight
      real(dop), external                             :: fcputime,fsystime
      complex(sip), dimension(spsidim), intent(out)   :: spsi
      complex(dop), dimension(dgldim1), intent(out)   :: psi1
!      complex(dop), dimension(workcdim), intent(out)  :: workc
      complex(dop), dimension(fftdim), intent(out)    :: rueck
      complex(dop), dimension(expdim), intent(out)    :: exphin
      complex(dop), dimension(1)                      :: zdum
      character(len=80)                               :: actual
      real(dop), save                                 :: previous = 0.0_dop

      integer(long)                                   :: countpsi
      character(len=c2)                               :: wrtag

! Diffusion map variables.
      character(len=256) :: formt
      character(len=256), dimension(maxdim) :: mlabel
      logical(kind=4) :: lendpsi,lendchk,lopen
      logical(kind=4), allocatable, dimension(:) :: lselect
      integer(long) :: ierr,nsample1,isamp,totsamp,itmp,idx,jdx,lwork,&
           p,gpnt,mgpnt,imaxeigv,gsample,npnt,ido,ncv,nev,koff
      integer(long), dimension(0:nsample) :: idxsamp
      integer(long), dimension(maxdim) :: idxf 
      integer(long), dimension(5) :: ieigv
      integer(long), dimension(11) :: iparam
      integer(long), dimension(14) :: ipntr
      integer(long), dimension(:,:), allocatable :: grange
      real(dop) :: dlamch
      real(dop) :: xrand,psidia,psiarg,maxeigv,norm,trace,dij,tsampinv,dtol,&
           tmin,tmax,swidth
      real(dop), dimension(:,:,:), allocatable :: qdq
      real(dop), dimension(:,:), allocatable :: diffarr,vr,qdqr,hmat,tmpmat
      real(dop), dimension(:), allocatable   :: vnorm,renorm,wr,wi,vl,workl,&
           xgpoint,workd,resid,workev,xsobol
      real(dop), dimension(:), pointer :: coord1,coord2
      real(dop), dimension(maxdim) :: gpoint,rvec,rvecavg
      complex(dop) :: psipnt

!-----------------------------------------------------------------------
! read DVR data
!-----------------------------------------------------------------------
      chkdvr=1                ! dvrinfo read and stored in system data
      dvrdata(1) = lort       ! ort
      dvrdata(2) = ltrafo     ! trafo
      dvrdata(11)= lfft       ! exphin
      dvrdata(6) = lfft       ! rueck
      dvrdata(7) = lfft       ! fftfak
      call rddvr(ort,trafo,rdum,rdum,zdum,rueck,fftfak,exphin,&
           zdum,idum,idum,rdum,chkdvr)

!-----------------------------------------------------------------------
! Determine DVR weights, if required. Note sqrt of weights is returned
!-----------------------------------------------------------------------
      if(lwght .or. lpop2) call getweight(trafo,weight,ort,.false.)

!-----------------------------------------------------------------------
! calculate time step for data
!-----------------------------------------------------------------------
      iz = 0
      dt   = out2/fs*step
      time = tinit/fs

! --- skip the first nskip WFs
      if(allocated(qdq))deallocate(qdq)
      allocate(qdq(4,ndof,nstate))
      lrdchk = .false.
      lrdchk(3) = .true.
      do i=1,nskip
         call rdpsi(ipsi,psi1,spsi,jindx,zdum,idum,zdum,idum)
         lendpsi = lend
         call rdchk(ichk,rdum2,rdum2,rdum2,rdum3,rdum1,qdq,norm,trace)
         lendchk = lend
         if (lendpsi.and.lendchk) goto 77
         time=time + out2/fs
      enddo

! Initialise the linked list for sampling.
! First get number of active DOFs
      nmode1=nmode
      if(nstate.gt.1) nmode1=nmode+1
      i=0
      do m=1,nmode!1
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            i=i+1
            findx(i) = f
            mlabel(i) = modelabel(f)
         enddo
      enddo
      ndof1=i  ! this is necessary to remove dummy DOFs

! Allocate array to hold active DOF wavefunction centres and widths.
      if(allocated(qdqr))deallocate(qdqr)
      allocate(qdqr(2,ndof1))
! Allocate array to hold grid sampling limits.
      if(allocated(grange))deallocate(grange)
      allocate(grange(4,ndof1))

! Now set up the list:
      if(associated(psisamp))nullify(psisamp)
      allocate(ipsisamp%dmsort(ndof1,nsample),stat=ierr)
!      allocate(ipsisamp%jac(nsample),stat=ierr)
      if(ierr.gt.0)goto 1001
      psisamp => ipsisamp
      nullify(psisamp%next)
      nullify(psisamp%prev)

      countpsi=0

! Initialise the random number generator.
      if(.not.lsobol)then
         call init_random_seed
      else
         if(allocated(xsobol))deallocate(xsobol)
         allocate(xsobol(ndof1))
      endif

   10 continue

! Converts the current MCTDH wavefunction to an exact form.
      call rdpsi(ipsi,psi1,spsi,jindx,zdum,idum,zdum,idum)
      lendpsi = lend
      call rdchk(ichk,rdum2,rdum2,rdum2,rdum3,rdum1,qdq,norm,trace)
      lendchk = lend
      if (lendpsi.and.lendchk) goto 77

! Allocate the next member of the linked list.
      if(countpsi.ne.0)then
         allocate(psisamp%next,stat=ierr)
         if(ierr.gt.0)goto 1000
         if(allocated(psisamp%next%dmsort))deallocate(psisamp%next%dmsort)
!         if(allocated(psisamp%next%jac))deallocate(psisamp%next%jac)
         allocate(psisamp%next%dmsort(ndof1,nsample),stat=ierr)
!         allocate(psisamp%next%jac(nsample),stat=ierr)
         if(ierr.gt.0)goto 1001
         psisamp%next%prev => psisamp
         psisamp => psisamp%next
         nullify(psisamp%next)
      endif

! Copy the wavepacket centre and width.
      if(abs(time).lt.1.0e-8_dop)then
         swidth=1.0_dop
      else
         swidth=3.0_dop
      endif
      i=0
      grange=0
      gsample=1
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            i=i+1
            qdqr(1:2,i) = qdq(1:2,f,st)

! Find the gridpoints nearest the WP centre and extremities along each DOF,
! and the number of gridpoints within the range.            
            do j = 1,gdim(f)
               if(qdqr(1,i)-swidth*qdqr(2,i).le.ort(zort(f)+j-1) &
                   .and. grange(1,i).eq.0) grange(1,i)=j-1
               if(qdqr(1,i)+swidth*qdqr(2,i).le.ort(zort(f)+j-1) &
                    .and. grange(2,i).eq.0) grange(2,i)=j
               if(qdqr(1,i).lt.ort(zort(f)+j-1) &
                    .and. grange(3,i).eq.0) then
                  if(abs(qdqr(1,i)-ort(zort(f)+j-1))&
                       .lt.abs(qdqr(1,i)-ort(zort(f)+j-2)))then
                     grange(3,i)=j
                  else
                     grange(3,i)=j-1
                  endif
               endif
            enddo
            if(grange(1,i).eq.0) grange(1,i)=1
            if(grange(2,i).eq.0) grange(2,i)=gdim(f)

            grange(4,i) = grange(2,i)-grange(1,i)+1

            if(gsample.lt.(huge(1_long)/grange(4,i)))then
               gsample=gsample*grange(4,i)
            else
               gsample=huge(1_long)
               write(ilog,'(a)')'Grid dimension overflow. Set to huge(1_long).'
            endif
         enddo
      enddo

! determine actual time and CPU-time spent since last call
      call fzeit(actual)
      i = index(actual,':')
      actual=actual(i-9:i+6)
      now=fcputime(0)+fsystime(0)
      spent=now-previous
      previous=now

      write(ilog,'(a,f8.3,a,f15.1,f15.2,4x,a16)') &
           'Writing WF : ',time,' fs, ',now,spent,actual
      call myflush(ilog)
! --- Handle multi-set electronic states
!      nmode1=nmode
!      if(nstate.gt.1) nmode1=nmode+1

! Carry out the sampling of gridpoints.
      nsample1 = 0
      idxsamp = 0
      idxf = 1
      gpoint = 0.0_dop

! Sobol sampling.
      if(lsobol)then
         call sobol_init(ndof1)
      endif

      do i = 1,gsample
! Sample from the subset of gridpoints within (just over) 3 widths of the WP centre.
         j = 0
         isamp = 0
         if(lsobol)&
              call sobol(ndof1,xsobol,.false.)
         do m = 1,nmode
            do n = 1,nspfdof(m)
               f = spfdof(n,m)
               j = j+1
               if(i.eq.1)then
!                  idxf(f) = (grange(2,j)+grange(1,j))/2
                  idxf(f) = grange(3,j)
               else
                  if(lsobol)then
!                     call sobol(ndof1,xsobol,.false.)
                     idxf(f) = floor(grange(4,j)*xsobol(j))+grange(1,j)                    
                  else
                     call random_number(xrand)
                     idxf(f) = floor(grange(4,j)*xrand)+grange(1,j)
                  endif
               endif

               if(psitype.eq.1)then
                  if(j.eq.1)then
                     isamp = isamp + idxf(f)
                  else
                     isamp = isamp + (idxf(f)-1)*vgdim(f)
                  endif
               endif
            enddo
         enddo

         do f = 1,ndof1
            gpoint(findx(f)) = ort(zort(findx(f))+idxf(findx(f))-1)
            if(abs(gpoint(findx(f))).lt.1.0e-12_dop) gpoint(findx(f))=0.0_dop
         enddo

! MCTDH or Exact.
         if(psitype.ne.1)then
            call calcpsis(psi1,psidia,psiarg,psipnt,idxf,st)
         else
            psidia = dble(dconjg(psi1(isamp+zetf(1,st)-1))*psi1(isamp+zetf(1,st)-1))
         endif

! Set the wavefunction sampling threshold.
         if(i.eq.1)then
            write(ilog,'(a,d12.4)') 'Density at wavefunction centre: ',sqrt(psidia)
            tmin=thrsmin*sqrt(psidia)
            if(lthrsmax)then 
               tmax=thrsmax*sqrt(psidia)
            else
               tmax = 1.0_dop
            endif
            write(ilog,'(a,d12.4)') 'Minimum density sampling threshold: ',tmin
            write(ilog,'(a,d12.4)') 'Maximum density sampling threshold: ',tmax
         endif

         if(sqrt(psidia).ge.tmin .and. sqrt(psidia).le.tmax)then
            nsample1 = nsample1 + 1
            do f = 1,ndof1
               psisamp%dmsort(f,nsample1) = gpoint(findx(f))
            enddo
            if(nsample1.eq.nsample)exit
         endif
      enddo

      if(nsample1.ne.nsample)then
         write(ilog,'(a,f12.3)')'Not enough sampled points at time :',time
         write(ilog,'(a,i6)')'Requested = ',nsample,'Actual = ',nsample1
         write(ilog,'(a,f12.6)')'Reduce threshold from: ',thrsmin
         stop
      else if(lsobol)then
         call sobol(ndof1,xsobol,.true.)
      endif

! ---          remove weights from WF if required
!         if(lwght) then   ! if(findx(i).ne.feb) then
!            do i=1,ndof1
!               if(findx(i).ne.feb) then
!                  workc(d) = workc(d)/weight(zort(findx(i))+igpoint(i)-1)
!               endif
!            enddo
!         endif

!      enddo     !  d=griddim

      psisamp%time = time
      countpsi = countpsi + 1

      time=time + dt
      iz=iz+1
      if(lrst) goto 77
      if(iz.ge.ncut) goto 77

!-----------------------------------------------------------------------
! if more data, go to beginning of loop
!-----------------------------------------------------------------------
      do i=2,step
         call rdpsi(ipsi,psi1,spsi,jindx,zdum,idum,zdum,idum)
         lendpsi = lend
         call rdchk(ichk,rdum2,rdum2,rdum2,rdum3,rdum1,qdq,norm,trace)
         lendchk = lend
         if (lendpsi.and.lendchk) goto 77
      enddo

      goto 10

 77   continue
      close(ipsi)
      close(ipsi1)

! Now at the end of the psi file, we can start calculating the diffusion map.
! First allocate the diffusion kernel array.
      if(allocated(diffarr))deallocate(diffarr)
      totsamp = countpsi*nsample
      allocate(diffarr(totsamp,totsamp),stat=ierr)
      if(ierr.gt.0)then
         write(ilog,'(a,i6)')'Error allocating diffusion kernel array: ',ierr
         stop
      endif

! Rewind the sample list.
      do while(associated(psisamp%prev))
         psisamp => psisamp%prev
      enddo
      psisamp2 => psisamp

! Run over the list, taking the grid points and calculating the distance kernel array. 
      idx = 1
      do
         do i = 1,nsample
            coord1 => psisamp%dmsort(:,i)
            jdx = 1
            do
               do j = 1,nsample
                  coord2 => psisamp2%dmsort(:,j)
                  dij = dot_product(coord1-coord2,coord1-coord2)
! Elements of the matrix depend on the method chosen.
                  if(imethod.eq.1)then
                     diffarr(idx,jdx) = exp(-dij/kerscal**2)
                  else
                     diffarr(idx,jdx) = dij
                  endif
                  jdx = jdx + 1
               enddo
               if(.not.associated(psisamp2%next))exit
               psisamp2 => psisamp2%next
            enddo
            do while(associated(psisamp2%prev))
               psisamp2 => psisamp2%prev
            enddo
            
            idx = idx + 1
         enddo
         if(.not.associated(psisamp%next))exit
         psisamp => psisamp%next
      enddo

! Diffusion map.
      if(imethod.eq.1)then
! Allocate and evaluate normalisation and renormalisation vectors (v and Z).
         if(allocated(vnorm))deallocate(vnorm)
         allocate(vnorm(totsamp))
         if(allocated(renorm))deallocate(renorm)
         allocate(renorm(totsamp))

! Normalisation vector.
         do i = 1,totsamp
            vnorm(i) = diffarr(i,1)
            do j = 2,totsamp
               vnorm(i) = vnorm(i) + diffarr(i,j)
            enddo
         enddo

! Renormalisation vector.
         do i = 1,totsamp
            renorm(i) = diffarr(i,1)/(vnorm(i)*vnorm(1))**alpha
            do j = 2, totsamp
               renorm(i) = renorm(i)+diffarr(i,j)/(vnorm(i)*vnorm(j))**alpha
            enddo
            renorm(i) = 1.0_dop/renorm(i)
         enddo

! Form the diffusion operator.
         do i = 1,totsamp
            do j = 1,totsamp
               diffarr(i,j) = diffarr(i,j)*renorm(i)/(vnorm(i)*vnorm(j))**alpha
            enddo
         enddo
! Isomap.
      else
! e-isomap.         
         if(imethod.eq.2)then
            do i = 1,totsamp
               do j = i+1,totsamp
                  if(diffarr(j,i) .ge. epsilon) then
                     diffarr(j,i) = 0.5_dop*huge(1.0_dop)
                     diffarr(i,j) = diffarr(j,i)
                  endif
               enddo
            enddo
! K-isomap.
         else
            do i = 1,totsamp
               do j = totsamp,kpar+2,-1
                  jdx = maxloc(diffarr(1:totsamp,i),1)
                  diffarr(jdx,i) = -0.5_dop*huge(1.0_dop)
               enddo
            enddo
            do i = 1,totsamp
               do j = 1,totsamp
                  if(diffarr(i,j).lt.0.0_dop) diffarr(i,j) = -diffarr(i,j)
               enddo
            enddo
         endif

! Find the minimum distance path from i to j.
         do i = 1,totsamp
            do j = 1,totsamp
               do k = 1,totsamp
                  diffarr(i,j) = min(diffarr(i,j),diffarr(i,k)+diffarr(k,j))
               enddo
            enddo
         enddo

         if(allocated(hmat))deallocate(hmat)
         allocate(hmat(totsamp,totsamp))
  
         tsampinv = -1.0_dop/dble(totsamp)
         do i = 1,totsamp
            do j = 1,totsamp
               diffarr(i,j) = diffarr(i,j)*diffarr(i,j)
               hmat(i,j) = tsampinv
            enddo
            hmat(i,i) = hmat(i,i)+1.0_dop
         enddo
         diffarr = matmul(hmat,diffarr)
         diffarr = matmul(diffarr,hmat)
         diffarr = -0.5_dop*diffarr
      endif

! Timing of diagonalisation.
      call fzeit(actual)
      i = index(actual,':')
      actual=actual(i-9:i+6)
      now=fcputime(0)+fsystime(0)
      spent=now-previous
      previous=now

! ARPACK
      ido = 0
      dtol = dlamch('e')
      iparam = 0
!         iparam(1)=0
      iparam(3)=300
      iparam(4)=1
      iparam(7)=1
      nev=min(6,totsamp)
      ncv=min(100,totsamp)
      lwork = 3*ncv*ncv+6*ncv
      if(allocated(workl))deallocate(workl)
      if(allocated(workd))deallocate(workd)
      if(allocated(resid))deallocate(resid)
      if(allocated(vr))deallocate(vr)
      allocate(workl(lwork))
      allocate(workd(3*totsamp))
      allocate(resid(totsamp))
      allocate(vr(totsamp,ncv))

      workd=0.0_dop
      
!50       call dnaupd( ido, bmat, n, which, nev, tol, resid, ncv, v, ldv, iparam, &
!              ipntr, workd, workl, lworkl, info )
         
50    call dnaupd(ido,'I',totsamp,'LM',nev,dtol,resid,ncv,vr,totsamp,iparam, &
           ipntr,workd,workl,lwork,ierr)

      if(ido.eq.-1 .or. ido.eq.1)then
         workd(ipntr(2):ipntr(2)+totsamp-1) &
              = matmul(diffarr,workd(ipntr(1):ipntr(1)+totsamp-1))
         goto 50
      endif
      
      if(ierr.lt.0)then
         write(*,*)'Error in dnaupd: ',ierr
         stop
      endif
         
      if(allocated(lselect))deallocate(lselect)
      if(allocated(wr))deallocate(wr)
      if(allocated(wi))deallocate(wi)
!      if(allocated(vr))deallocate(vr)
      if(allocated(workev))deallocate(workev)
      allocate(lselect(ncv))
      allocate(wr(nev+1))
      allocate(wi(nev+1))
!      allocate(vr(totsamp,nev+1))
      allocate(workev(3*ncv))

      call dneupd (.true.,'A',lselect,wr,wi,vr,totsamp, &
           rdum,rdum, workev,'I',totsamp,'LM',nev,dtol, & 
           resid,ncv,vr,totsamp,iparam,ipntr,workd,workl, &
           lwork,ierr)

      if(ierr.ne.0)then
         write(*,*)'Error in dneupd: ',ierr
         stop
      endif

! Timing of diagonalisation.
      call fzeit(actual)
      i = index(actual,':')
      actual=actual(i-9:i+6)
      now=fcputime(0)+fsystime(0)
      spent=now-previous
      previous=now

      write(ilog,'(a,f15.2,a,i8)') &
           'Diagonalisation walltime: ',spent,' s for total sample: ',totsamp
      call myflush(ilog)


! Rewind the sample list.
      do while(associated(psisamp%prev))
         psisamp => psisamp%prev
      enddo

! Write the eigenvalues of the diffusion matrix.
      open(unit=400,file='diffeval.dat',status='replace')
      write(400,'(x,a)')'Index   real part   imag part'
!      do i = 1,totsamp
      do i = 1,nev
         write(400,'(i6,2f12.6)')i,wr(i),wi(i)
      enddo
      inquire(unit=400,opened=lopen)
      if(lopen) close(400,status='keep')

! Write the sample time, eigenvectors with the five largest, non-trivial eigenvalues 
! and the sampled coordinates associated with each element.  
      if(imethod.eq.1)then
         jdx=2
      else
         jdx=1
         do j = 1,5
            if(wr(j).lt.0.0_dop)then
               write(*,'(a,i6)')'Negative eigenvalue of kernel matrix: ',j
               stop
            endif
            vr(1:totsamp,j) = sqrt(wr(j))*vr(1:totsamp,j)
         enddo
      endif

! If doing DD in normal modes, want to print the Cartesians.      
      open(unit=401,file='diffvect.dat',status='replace')
      if((ldd.or.lddmctdh) .and. ddcoo.eq.1)then
         open(unit=402,file='diffcart.dat',status='replace')
         if(allocated(xgpoint))deallocate(xgpoint)
         allocate(xgpoint(ndofddpes))
         write(formt,'(a,i4,a)')"(a,2(3x,a),68x,a,",16*(ndof1-1)+3,"x,a)"
         write(401,fmt=trim(formt))'#','Time',' Eigenvectors',&
              'Sample coord.','Cartesian coord.'
         write(formt,'(a,i4,a)') "(a,90x",ndof1,"a16)"
         write(401,fmt=trim(formt))'#',(mlabel(f),f=1,ndof1)
      else
         write(401,'(a,2(3x,a),68x,a)')'#','Time',' Eigenvectors','Sample coord.'
         write(formt,'(a,i4,a)') "(a,90x",ndof1,"a16)"
         write(401,fmt=trim(formt))'#',(mlabel(f),f=1,ndof1)
      endif

      do i = 1,totsamp
         if((ldd.or.lddmctdh) .and. ddcoo.eq.1)then
            write(formt,'(a,i4,a)')"(f8.2,5e16.6,",ndof1+ndofddpes,"e16.6)"
            gpoint=0.0_dop
            do f = 1,ndof1
               gpoint(findx(f)) = psisamp%dmsort(f,mod((i-1),nsample)+1)
            enddo
            call ddq2x(gpoint,xgpoint)
!            call ddq2x(psisamp%dmsort(:,mod((i-1),nsample)+1),xgpoint)
            write(401,fmt=trim(formt)) psisamp%time,vr(i,jdx:jdx+4),&
!                 (psisamp%dmsort(findx(f),mod((i-1),nsample)+1),f=1,ndof1),&
                 (psisamp%dmsort(f,mod((i-1),nsample)+1),f=1,ndof1),&
                 xgpoint(1:ndofddpes)!,psisamp%jac(mod((i-1),nsample)+1)

            write(402,'(i6)') ndofddpes/3
            write(402,'(i6,f12.6)') i,psisamp%time
            koff=0
            do j = 1,ndofddpes/3
               write(402,'(a,3f12.6)')xatnam(j),(xgpoint(koff+k)*bohr,k=1,3)
               koff = koff+3
            enddo
         else
            write(formt,'(a,i4,a)')"(f8.2,5e16.6,",ndof1,"e16.6)"
            write(401,fmt=trim(formt))psisamp%time,vr(i,jdx:jdx+4),&
                 (psisamp%dmsort(f,mod((i-1),nsample)+1),f=1,ndof1)
!                 (psisamp%dmsort(findx(f),mod((i-1),nsample)+1),f=1,ndof1)
         endif
         if(mod(i,nsample).eq.0.and.i.ne.totsamp) psisamp => psisamp%next
      enddo
      inquire(unit=401,opened=lopen)
      if(lopen) close(401,status='keep')
      inquire(unit=402,opened=lopen)
      if(lopen) close(402,status='keep')

! Final deallocations.
      if(allocated(qdq))deallocate(qdq)
      if(allocated(qdqr))deallocate(qdqr)
      if(allocated(grange))deallocate(grange)
      if(allocated(diffarr))deallocate(diffarr)
      if(allocated(vnorm))deallocate(vnorm)
      if(allocated(renorm))deallocate(renorm)
      if(allocated(workl))deallocate(workl)
      if(allocated(workd))deallocate(workd)
      if(allocated(workev))deallocate(workev)
      if(allocated(resid))deallocate(resid)
      if(allocated(lselect))deallocate(lselect)
      if(allocated(wr))deallocate(wr)
      if(allocated(wi))deallocate(wi)
      if(allocated(vr))deallocate(vr)
      if(allocated(vl))deallocate(vl)
      if(allocated(hmat))deallocate(hmat)
      if(allocated(xgpoint))deallocate(xgpoint)
      if(allocated(xsobol))deallocate(xsobol)
! debug
!      if(allocated(tmpmat))deallocate(tmpmat)
! debug
      if(associated(psisamp2))nullify(psisamp2)
      do while(associated(psisamp%next))
         psisamp => psisamp%next
      enddo
      do while (associated(psisamp%prev))
         if(allocated(psisamp%dmsort))deallocate(psisamp%dmsort)
         psisamp => psisamp%prev
         nullify(psisamp%next%prev)
         deallocate(psisamp%next)
         nullify(psisamp%next)
      enddo
      nullify(psisamp)
      if(allocated(ipsisamp%dmsort))deallocate(ipsisamp%dmsort)

      return

1000  write(ilog,'(a)') 'Error allocating sample list'
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
1001  write(ilog,'(a)') 'Error allocating sample array'
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
1002  write(ilog,'(a)') 'Error allocating eigenfunction arrays'
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop

      end subroutine subdiffmap

!-----------------------------------------------------------------------
!***********************************************************************
!
! DIFFMAPINPUT
!
! get input parameters and file names for calculation
!
!-----------------------------------------------------------------------

      subroutine diffmapinput
      
use global
use aglobal
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use compdat1mod
use diffmapmod

      implicit none

      integer(long), external         :: myiargc
      integer(long)                   :: iarg,ierr,i
      character(len=80)               :: buf
      character(len=c5)               :: filename
      character(len=12)               :: oformat
      logical(kind=4)                 :: lfileout
      real(dop)                       :: ddum

!-----------------------------------------------------------------------

      nskip=0
      ncut = 999999
      lthrsmax=.false.
      thrsmax=1.0_dop
      thrsmin=0.2_dop
      filein  = './'
      filechk = './'
!      fileout = 'psi.ex'
      lfileout=.false.
      lsobol=.true.
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:3) .eq. '-f ' .or.  buf(1:3) .eq. '-i ') then
            i = i+1
            call mygetarg(i,buf)
            filein = buf
!         else if( buf(1:3) .eq. '-o ' ) then
!            i = i+1
!            call mygetarg(i,buf)
!            fileout = buf
!            lfileout=.true.
         else if( buf(1:5) .eq. '-stp ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:6) .eq. '-thrs ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) thrsmin
            if (ierr .ne. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -thrs',&
                 ' must be a real number. Not : ', buf
               stop
            end if
         else if( buf(1:9) .eq. '-thrsmax ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) thrsmax
            if (ierr .ne. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -thrsmax',&
                 ' must be a real number. Not : ', buf
               stop
            end if
            lthrsmax = .true.
         else if( buf(1:9) .eq. '-thrsmin ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) thrsmin
            if (ierr .ne. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -thrsmin',&
                 ' must be a real number. Not : ', buf
               stop
            end if
         else if( buf(1:4) .eq. '-st ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) st
            if (ierr .ne. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -st',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:3) .eq. '-n ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ncut
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -n'
               stop
            endif
         else if( buf(1:9) .eq. '-nsample ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nsample
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -nsample'
               stop
            endif
         else if( buf(1:9) .eq. '-kerscal ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) kerscal
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -kerscal'
               stop
            endif
         else if( buf(1:7) .eq. '-alpha ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) alpha
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -alpha'
               stop
            endif
         else if( buf(1:5) .eq. '-eps ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) epsilon
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -eps'
               stop
            endif
         else if( buf(1:3) .eq. '-k ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) kpar
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -k'
               stop
            endif
         else if( buf(1:5) .eq. '-skip ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nskip
            if (nskip .lt. 0) then
               write(6,'(/a)')&
               'Argument of -skip must be positive integer'
               stop
            endif
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -skip'
               stop
            endif
         else if( buf(1:8) .eq. '-random ' ) then
            lsobol=.false.
         else if( buf(1:7) .eq. '-sobol ' ) then
            lsobol=.true.
         else if( buf(1:3) .eq. '-w ' ) then
            lovwr=.true.
         else if( buf(1:4) .eq. '-dp ' ) then
            lsp=.false.
!         else if( buf(1:6) .eq. '-pop2 ' ) then
!            lpop2=.true.
!         else if( buf(1:6) .eq. '-plot ' ) then
!            lswap=.true.
         else if( buf(1:6) .eq. '-abs  ' ) then
            labs=.true.
         else if( buf(1:6) .eq. '-abs2 ' ) then
            labs2=.true.
         else if( buf(1:4) .eq. '-W  ' ) then
            lwght=.true.
         else if( buf(1:6) .eq. '-ascii' ) then
            lascii=.true.
         else if( buf(1:6) .eq. '-rst  ' ) then
            lrst=.true.
         else if( buf(1:6) .eq. '-diff ' ) then
            imethod=1
         else if( buf(1:6) .eq. '-eiso ' ) then
            imethod=2
         else if( buf(1:6) .eq. '-kiso ' ) then
            imethod=3
         else if( buf(1:3) .eq. '-h ' .or. buf(1:3) .eq. '-? ' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,a,7(/10x,a))') &
         'Purpose: Analyses an MCTDH or exact wavefunction in a psi file ', &
         'using diffusion map or isomap methods. Works by sampling gridpoints ', &
         'of the wavefunction at each selected time, locating points where the ', &
         'density is some proportion of the density at the centre of the wavefunction. ',&
         'Selected points are then analysed using eith diffusion map (default) or isomap ',&
         'in order to provide insight into the actual modes which most well describe ',&
         'the dynamics. For more details see Richings and Habershon, Molecules, 2021, 26, 7621.'
            write(6,'(/1x,2a,/19x,a)') 'Usage  : diffmap ', &
              '[-f -i -stp -n -skip -thrs -thrsmax -thrsmin -ascii -W',&
              ' -abs -abs2 -rst -w -dp -diff -eiso -kiso -eps -k -ver -h -?]'

            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a)') &
                     ' -f FILE : The wavefunction is read '&
                    ,'from file FILE rather than from ./psi'&
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(a)') ' -i FILE : Same as -f FILE.'
!            write(6,'(2a,/,11x,a)') &
!                     ' -o FILE : The output is written to file ',&
!                     'FILE rather than to ./psi.ex',&
!             'The string FILE may be a relative or a full path-name.'
            write(6,'(2a)') &
                     ' -rst    : A restart file rather than a psi file is read', &
                     ' as input.'
            write(6,'(2a)') ' -stp step: Compute the exact WF', &
                            ' only every step-th output.'
            write(6,'(2a)')&
                     ' -n num  : only the first num WFs are read '&
                    ,' from psi-file.'
            write(6,'(2a)')&
                     ' -nsample nsample  : number of DVR points sampled '&
                    ,' from psi-file. Default is 50.'
            write(6,'(2a)')&
                     ' -kerscal kerscal  : diffusion length scale. '&
                    ,' Default is 1.0.'
            write(6,'(2a)')&
                     ' -alpha alpha  : diffusion type parameter. '&
                    ,' Default is 1.0.'
            write(6,'(2a)')&
                     ' -eps epsilon  : distance parameter for e-isomap. '&
                    ,' Default is 1.0.'
            write(6,'(2a)')&
                     ' -k kpar  : no. of nearest neighbours for K-isomap. '&
                    ,' Default is 1.'
            write(6,'(2a)')&
                     ' -st st  : state of interest. '&
                    ,' Default is 1.'
            write(6,'(a)')&
                     ' -skip m : the first m WFs are skipped.'
            write(6,'(a)')&
                     ' -random : use random sampling.'
            write(6,'(a)')&
                     ' -sobol : use Sobol sampling (default).'
!            write(6,'(a)')&
!                     ' -pop2   : exact WF is given in second population'
!            write(6,'(a,/a,/a,/a,/a,/a)')&
!            ' -plot   : only meaningful if -pop2 is activated',&
!            '           and some dof is of fft type. ',&
!            '           For each dof of fft type, the x-axis is',&
!            '           swapped in a convenient way for further',&
!            '           plotting. NB: the resulting WF cannot be',&
!            '           used in a subsequent MCTDH calculation.'
            write(6,'(a,/11x,a,/11x,a)') &
            ' -W      : The DVR-weights are removed from the WF.',&
                      'Useful for plotting the WF.',&
                      '-W requires that -ort is set, otherwise ignored.'
            write(6,'(a,/11x,a)') &
            ' -thrs thrsmin : The WF and their coordinates are sampled ', &
                    'if(abs(WF).GE.thrsmin*abs(WF(centre)). Default 0.2.'
            write(6,'(a,/11x,a)') &
            ' -thrsmin thrsmin : The WF and their coordinates are sampled ', &
                    'if(abs(WF).GE.thrsmin*abs(WF(centre)). Default 0.2.'
            write(6,'(a,/11x,a)') &
            ' -thrsmax thrsmax : The WF and their coordinates are sampled ', &
                    'if(abs(WF).LE.thrsmax*abs(WF(centre)). Default 1.0.'
            write(6,'(2a,/11x,a,/11x,a)') &
            ' -ascii  : The output file is in ascii rather than binary', &
                       ' format.', &
                       'In this case the (dummy) A-vector and psiinfo ', &
                 'are not written, i.e. the file contains the WF only.'
            write(6,'(a,/11x,a,/11x,a)') &
            ' -abs    : The absolute value of the exact WF is written.', &
                       'In this case the (dummy) A-vector and psiinfo ', &
                 'are not written, i.e. the file contains abs(WF) only.'
            write(6,'(a,/11x,a)') &
            ' -abs2   : Similar to -abs, but the absolute value squared' &
                       ,'of the exact WF is written.' 
            write(6,'(2a)') &
                     ' -dp     : The file is written double precision.'
            write(6,'(2a)') &
                     ' -diff   : Use diffusion map. (Default).'
            write(6,'(2a)') &
                     ' -eiso   : Use e-isomap.'
            write(6,'(2a)') &
                     ' -kiso   : Use K-isomap.'
            write(6,'(2a)') &
                     ' -w      : An existing output file is ',&
                     'overwritten.'
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      else
         write(6,'(2a)') ' unknown argument : ', buf
         stop
      end if
      
 200  continue

      if(kerscal.lt.0.0_dop.or.abs(kerscal).lt.1.0e-8_dop)then
         write(ilog,'(a)')'Small or negative kernel scale'
         stop
      endif

      if(thrsmax.lt.thrsmin .and. lthrsmax)then
         write(ilog,'(a)')'Re-ordering max. and min. thresholds.'
         ddum = thrsmax
         thrsmax = thrsmin
         thrsmin = ddum
      endif

      lswap=lswap.or.(labs.and.lpop2).or.(labs2.and.lpop2)
! --- If lswap is true, swapfft will be called in showpop2.
      laein=index(filein,' ')-1
      if(laein.gt.6) then
         if (filein(laein-6:laein) .eq. 'restart' ) lrst = .true.
         if (filein(laein-2:laein) .eq. 'psi' )     lrst = .false.
      endif
      if (filein(laein:laein) .eq. '/') then
         if(lrst) then
            irst = ipsi
            filein = filein(1:laein)//'restart'
            laein  = laein + 7
         else
            filein = filein(1:laein)//'psi'
            laein  = laein + 3
         endif
      end if
      call abspath(filein,laein)

      laechk=index(filechk,' ')-1
      if (filechk(laechk:laechk) .eq. '/') then
         filechk = filechk(1:laechk)//'check'
         laechk  = laechk + 5
      end if
      call abspath(filechk,laechk)

!      laeout=index(fileout,'  ')-1
!      call abspath(fileout,laeout)
!      if(lpop2 .and. .not.lfileout) then
!         fileout=fileout(1:laeout)//'.pop2'
!         laeout=laeout+5
!      endif
!      if(labs .and. .not.lfileout) then
!         fileout=fileout(1:laeout)//'.abs'
!         laeout=laeout+4
!      endif
!      if(labs2 .and. .not.lfileout) then
!         fileout=fileout(1:laeout)//'.abs2'
!         laeout=laeout+5
!      endif

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      write(6,'(81a)') '#',('*',i=1,80)
      filename=filein(1:laein)
      open(ipsi,file=filename,form='unformatted',status='old',&
           err=1000)
      write(6,'(2a)') '# The inputfile is  : ',filein(1:laein)

      filename=filechk(1:laechk)
      open(ichk,file=filename,form='unformatted',status='old',err=1000)

!      oformat='unformatted'
!      if(lascii) oformat='formatted'
!      filename=fileout(1:laeout)
!      if (lovwr) then
!         open (ipsi1,file=filename,form=oformat,status='unknown',&
!                 err=1000)
!      else
!         open (ipsi1,file=filename,form=oformat,status='new',&
!                 err=1010)
!      endif
!      write(6,'(2a)') '# The outputfile is : ',fileout(1:laeout)
!      write(6,'(2a)') '# The log file is   :  ./diffmap.log'

      return 
      
 1000 write(6,*)' Error opening file : '
      write(6,*) filename
      stop
 1010 write(6,*)' File already exists: '
      write(6,*) filename
      stop
      
      end subroutine diffmapinput


!***********************************************************************
!
!                RDPSIGRID2
!
! Called by the various ANALYSE programs in order to read the
! psi file data, and transform the wavefunction to the full grid
! (primitive basis) representation
! 
! As opposed to its sister routine rdpsigrid, rdpsigrid2 returns
! the wave function to the full grid IN SECOND POPULATION
!
! OV 12/05
!***********************************************************************

      subroutine rdpsigrid2(unit,psi,spsi,jindx,workc,&
                            trafo,exphin,rueck,fftfak,lrst)

use global
use aglobal
use dvrdatmod
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use compdat1mod

use psiutil, only: showpop2
use rdfilesmod, only: rdpsi

      implicit none

      integer(long)                              :: zeig1,zeig2,zgrd,f,m,s,&
                                                    block1,swapzeig,vdim1,i
      integer(long), intent(in)                                         :: unit
      integer(long), dimension(1)                                       :: idum
      integer(long), dimension(adim), intent(out)                       :: jindx
      integer(long), dimension(*), intent(in)                           :: fftfak
      real(dop), dimension(dvrdim), intent(in)                          :: trafo
      complex(sip), dimension(dgldim), intent(out)                      :: spsi
      complex(dop), dimension(dgldim), intent(out)                      :: psi
      complex(dop), dimension(dgldim+2*(griddim/nstate)), intent(inout) :: workc
      complex(dop), dimension(1)                                        :: zdum
      complex(dop), dimension(fftdim), intent(in)                       :: rueck
      complex(dop), dimension(expdim), intent(in)                       :: exphin
      logical(kind=4)                          :: lrst

      if(dgldim+2*(griddim/nstate).gt.workcdim) then
         routine='rdpsigrid2'
         message='workcdim is too small'
         write(6,*) 'workcdim =                 ', workcdim
         write(6,*) 'dgldim+2*(griddim/nstate) =',&
                     dgldim+2*(griddim/nstate)
         call errormsg
      endif 
      zeig1=dgldim+1
      zeig2=zeig1+griddim/nstate

! read mctdh-wfn into array psi or spsi
      if(lrst) then
         read(unit,err=999) (psi(i),i=1,dgldim)
      else
         call rdpsi(unit,psi,spsi,jindx,zdum,idum,zdum,idum)
      endif

! transform mctdh-wfn to second population
!     each time showpop2 is called it transforms a single
!     mode, then, loop through all modes
!     careful, the last arguments in showpop2 are dof's, not modes!
!     The bases phifbr, el, sphfbr, k, and external are not transformed.
      do f=1,ndof
         if(basis(f).gt.0 .and. basis(f).ne.6 .and.&
            basis(f).ne.8 .and. basis(f).ne.13) &
            call showpop2(psi,trafo,exphin,rueck,fftfak,f,f)
      end do

! copy the transformed wavefunction into array workc
      call cpvxz(psi(1),workc(1),dgldim)
      
! psi wavefunction is written to the primitive grid
      zgrd=1
      do s=1,nstate
         call cpvxz(workc(zpsi(s)),workc(zeig2),block(s))

         vdim1=1
         do m=1,nmode
            swapzeig=zeig1
            zeig1=zeig2
            zeig2=swapzeig
            call mtxxzz (workc(zetf(m,s)),workc(zeig1),&
                 workc(zeig2),vdim1,dim(m,s),ndim(m,s),subdim(m))
            block1=vdim1*subdim(m)*ndim(m,s)
            vdim1=vdim1*subdim(m)
         enddo

         call cpvxz(workc(zeig2),psi(zgrd),griddim/nstate)
         zgrd=zgrd+griddim/nstate
      enddo

      return
 999  write(6,*) ' ERROR reading restart file (-rst)'
      stop
      end subroutine rdpsigrid2

!##########################################################################
! Random seed routine.
! From:
! https://gcc.gnu.org/onlinedocs/gfortran/RANDOM_005fSEED.html#RANDOM_005fSEED
!########################################################################## 
      subroutine init_random_seed()
      implicit none
      integer(long), allocatable :: seed(:)
      integer(long) :: i, n, un, istat, dt(8), pid
      integer(vlong) :: t
        
      call random_seed(size = n)
      allocate(seed(n))
      call system_clock(t)
      if (t == 0) then
         call date_and_time(values=dt)
         t = (dt(1) - 1970) * 365_vlong * 24 * 60 * 60 * 1000 &
              + dt(2) * 31_vlong * 24 * 60 * 60 * 1000 &
              + dt(3) * 24_vlong * 60 * 60 * 1000 &
              + dt(5) * 60 * 60 * 1000 &
              + dt(6) * 60 * 1000 + dt(7) * 1000 &
              + dt(8)
      end if
      do i = 1, n
         seed(i) = lcg(t)
      end do
      call random_seed(put=seed)
      end subroutine init_random_seed     

!#########################################################################
! Subroutine to initialise a Sobol sequence of points for use in 
! sampling configuration space.
! GWR 05/18
!#########################################################################
      subroutine sobol_init(ndof1)

use gapmod, only: dirnno

      implicit none 

      integer(long), intent(in) :: ndof1

      logical(kind=4) :: lsmall
      integer(long) :: i,idxp,j,k,l

      if(ndof1.gt.maxsob)then
         routine = 'sobol_init'
         message = 'Too many DOFs for current implementation'
         call errormsg
      endif

      idxn = 0
      
      if(allocated(dirnno)) deallocate(dirnno)
      allocate(dirnno(ndof1,maxb))
      dirnno = 0
      
      if(dirninit(1+idnpnt(ndof1)) .ne. 1) then
         routine = 'sobol_init'
         message = 'First initialising value not 1'
         call errormsg
      endif

      sobfac = 0.5_dop**maxb
      
      do k = 1,ndof1
         do j = 1, polydeg(ndof1)
            dirnno(k,j) = dirninit(j+idnpnt(k))*2**(maxb-j)
         enddo
         
         do j = polydeg(k)+1,maxb
            idxp = ipoly(k)
            i = dirnno(k,j-polydeg(k))
            i = ieor(i,i/2**polydeg(k))
            do l = polydeg(k)-1,1,-1
               if(iand(idxp,1).ne.0) i = ieor(i,dirnno(k,j-l))
               idxp = idxp/2
            enddo
            dirnno(k,j) = i
         enddo
      enddo

      nchk = 0
      intx = 0

      return

      end subroutine sobol_init

!#########################################################################
! Subroutine to calculate a Sobol sequence of points for use in 
! sampling configuration space.
! GWR 05/18
!#########################################################################
      subroutine sobol(ndof1,xsobol,llast)

use gapmod, only: dirnno

      implicit none 

      logical(kind=4), intent(in)              :: llast 
      integer(long), intent(in)                :: ndof1
      real(dop), dimension(ndof1), intent(out) :: xsobol

      logical(kind=4) :: lsmall
      integer(long) :: i,imin,j,k,jdx

      if(nchk.eq.0)then
         nchk = ndof1
      else if(ndof1.ne.nchk)then
         routine = 'sobol'
         message = 'No. of dimensions changed between initialisations.'
         call errormsg
      endif
      
      imin = idxn
      lsmall = .true.
      do j = 1,maxb
         if(iand(imin,1).eq.0)lsmall=.false.
         if(.not.lsmall)then
            jdx = j
            exit
         endif
         imin = imin/2
      enddo

      if(lsmall)then
         routine = 'sobol'
         message = 'Maximum no. of bits too small'
         call errormsg
      endif

      imin = (jdx-1)*ndof1
      do k = 1,ndof1
         intx(k) = ieor(intx(k),dirnno(k,jdx))
         xsobol(k) = dble(intx(k))
      enddo
      xsobol = sobfac*xsobol
      idxn = idxn + 1
      
      if(llast)then
         if(allocated(dirnno))deallocate(dirnno)
      endif

      return

      end subroutine sobol

!############################################################################
!      contains
        ! This simple PRNG might not be good enough for real work, but is
        ! sufficient for seeding a better PRNG.
      function lcg(s)
      integer(long) :: lcg
      integer(vlong) :: s
      if (s == 0) then
         s = 104729
      else
         s = mod(s, 4294967296_vlong)
      end if
      s = mod(s * 279470273_vlong, 4294967291_vlong)
      lcg = int(mod(s, int(huge(0), vlong)), kind(0))
      end function lcg


      end program diffmap

