!***********************************************************************
!
!             Analyse program EXPECT
!
! This program reads the psi-file and a specified operator and
! computes the expectation value o(t) = <psi(t)|operator|psi(t)>.
! It outputs or plots o(t).
!
!     11/96 MB
!
!
! GW TO DO add trexpectation. 
!***********************************************************************
      program expect

use decimal, only:long,dop,sip
use global
use globmemmod
use versions
use aglobal
use memdimmod
use keyunitsmod
use dvrdatmod
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod
use operdef
use dirdyn
use gapmod
use hpsimod
use channels
use runpropmod
use op1lib, only: zerovxz
use logdat
use iorst, only: rstinfo
use iofile, only: rdpsiinfo
use rddvrmod
use rdopermod
use zeigauswmod, only: zeigausw
use memhopsmod, only: memhops
use zeighpsimod, only: operflag,zeighpsi,d2zeighpsi,d1zeighpsi
use psiutil, only: maxmatwk
use multilayer, only: setoperml
use wftree,  only: new_wftree,dgldim_wftree
use wftree1, only: dispose_wftree
use inputmlmod, only: inputml
use daten, only: speedout
use wftree, only: wftree_set_pointers

use module_interface

use rdfilesmod, only: rdpsi
use tdkoeffs, only: calctdk
use expectval, only: wfexpectvalue,d1expectvalue,d2expectvalue,&
                     trexpectvalue,lnwexpect

      implicit none

      integer(long), external              :: slen
      integer(long)                        :: chkdvr,chkgrd,chkpsi,chkdat,chkprp,ilbl,n,nham 
      character(len=c5)                    :: filename
      character(len=10)                    :: stat
      logical(kind=4)                      :: lerr,lexist,linwf

!-----------------------------------------------------------------------
! Initialize variables and set defaults
!-----------------------------------------------------------------------
      call default
      call adefault

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare files to be read
! set flag to .true. if file is to be read
!
!     lrddvr    DVR file
!     lrdoper   OPER file
!-----------------------------------------------------------------------
      lrddvr=.true.
      lrdoper=.true.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare resources required (other than those needed to read a file)
!
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! ltrafo : FBR/DVR trafo matrices
!-----------------------------------------------------------------------
      ncomp=0
      lpsi=.true.
      lpsi1=.false.
      lpsigrd=.false.
      ldmat=.true.
      ldmat1=.false.
      lort=.true.
      lgpop=.false.
      ltrafo=.true.

!-----------------------------------------------------------------------
! read input parameters and options
! open files
!-----------------------------------------------------------------------
      call expinput(stat)

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! open input and output data files
!
! use standard channels if possible
!  irho : gridpop file
!  ipsi : psi file
!  iaut : auto file
!  ichk : check file
!  iaus : output file
!-----------------------------------------------------------------------
      filename=filein
      open(ipsi,file=filename,form='unformatted',status='old',err=1000)
      if (fileout .ne. 'no') then
         filename=fileout
         open(iaus,file=filename,form='formatted',&
             status=stat,err=1000)
      else
         iaus=6
      endif

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(ipsi)
      if (lrdoper) then
         filename=operfile
         ilbl=index(filename,' ')-1
         open(ioper,file=filename,form='unformatted',status='old',err=900)
         call rdmemdim(ioper)
         close(ioper)
      endif

!-----------------------------------------------------------------------
! Allocate memory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      call alloc_psidef
!      call alloc_hpsimod  ! Need to allocate this later on, when nmode has been defined
      call alloc_runpropmod

!-----------------------------------------------------------------------
! write gnuplot info to file
!-----------------------------------------------------------------------
      if (lgnu) then
         write(iaus,'(3a)') 'set title "',fileout(1:laeout),'"'
         write(iaus,'(a)')  'set style data lines'
         write(iaus,'(a)')  'set nologscale'
         if(fs.gt.10.0_dop) then
            write(iaus,'(a)')  'set xlabel "Time [fs]"'
         else
            write(iaus,'(a)')  'set xlabel "Time"'
         endif
         write(iaus,'(5a)') 'set ylabel "<',operator(1:slen(operator)),&
                         '> [',unitlab(1)(1:slen(unitlab(1))),']"'
         if( xmax .gt. xmin ) then
            write(iaus,'(a,f8.3,a,f8.3,a)')&
                  'set xrange [',xmin,':',xmax,']'
         endif
         if(lpgrid) write(iaus,'(a)') 'set grid'
         write(iaus,'(3a)') 'plot "-" using 1:2 notitle'
         write(iaus,'(a)')  '   '
      end if

!-----------------------------------------------------------------------
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
! First read file version number
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      chkprp=0
      filename=filein
      if (irst.eq.ipsi) then
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         if (citype .gt. 0) then
            routine='expect'
            message='citype .gt.0 not implemented for restart files.'
            call errormsg
         endif
         filename='restart (-rst option)'
         if (lerr) goto 999
      else
         rewind(ipsi)
         read(ipsi,err=999) filever(ipsi)
         call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
         filename='psi'
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif

!      if (psitype.eq.5) then
!         ! set correct dgldim for ML wfn. NO LONGER REQUIRED - set in psidat.
!         tree => new_wftree(tape) ! tape was read by rdpsiinfo/rstinfo
!         dgldim = dgldim_wftree(tree)
!         call dispose_wftree(tree)
!         ! set ML flag
!         lrunml = .true.
!      endif

!-----------------------------------------------------------------------
! Allocate extra space if DD or DD-GB.
!-----------------------------------------------------------------------
      call alloc_hpsimod

      if (ldd.or.lddmctdh) then
         call alloc_dercp
         call alloc_dirdyn
         call alloc_dddb
         call alloc_gapkernel
      endif

!-----------------------------------------------------------------------
! read information from system files
!-----------------------------------------------------------------------
      if (lrddvr) then
         chkdvr=2
         filename=dname(1:dlaenge)//'/dvr'
         open(idvr,file=filename,form='unformatted',status='old',&
              err=1000)
         call dvrinfo(lerr,chkdvr)
         close(idvr)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif
      if (lrdoper) then
         chkdvr=2
         chkgrd=2
         filename=operfile
         open(ioper,file=filename,form='unformatted',status='old',&
              err=1000)
         call operinfo(lerr,chkdvr,chkgrd)
         close(ioper)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif
         
!-----------------------------------------------------------------------
! get operator name
!-----------------------------------------------------------------------
      if (operator .eq. ' ') then
         write(6,*) 'No operator specified! Use -h option to see usage.'
         write(6,'(1x,a)') 'Name of operator ? Choose from:'
         do n=1,hamdim
            write(6,'(2x,a)') hamlab(n)
         enddo
         read(5,'(a)') operator
         if (operator .eq. ' ') stop
      endif

!-----------------------------------------------------------------------
! Get operator number
!-----------------------------------------------------------------------
 20   nham=0
      do n=1,hamdim
         if (hamlab(n) .eq. operator) nham=n
      enddo

      if (nham .eq. 0) then
         write(6,'(4a)')'Operator "',&
           operator(1:slen(operator)),'" not found in ',&
           operfile(1:slen(operfile))
         write(6,'(1x,a)') 'Name of operator ? Choose from:'
         do n=1,hamdim
            write(6,'(2x,a)') hamlab(n)
         enddo
         read(5,'(a)') operator
         if (operator .eq. ' ') stop
         goto 20
      endif

      if (logisopen) then
         write(ilog,'(3a,i4)') ' Operator: ',operator(1:slen(operator)),&
                               ',  nham =',nham
         write(ilog,'(a,i8,a,i8)') ' k-max =',zham(nham)+kzahl(nham),&
                                   ', maxkoe =', maxkoe

         write(ilog,'(/a)') ' Wavefunction'
         write(ilog,'(a,i8,1pe12.3)') ' adim      =',adim,dble(adim)
         write(ilog,'(a,i8,1pe12.3)') ' totphidim =',totphidim,&
                                      dble(totphidim)
         write(ilog,'(a,i8,1pe12.3)') ' dgldim    =',dgldim,dble(dgldim)

         if(nodiag(nham)) then
            write(ilog,'(a/)') &
           ' ###  WARNING: Operator is "nodiag" but should be "usediag".'
         endif

      endif

!-----------------------------------------------------------------------
! Turn off direct dynamics if it is not the energy that is being 
! evaluated
!-----------------------------------------------------------------------
      if (nham .ne. 1) ldd=.false.

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
! set up various flags and array information needed to use an operator
!-----------------------------------------------------------------------
      do n=1,hamdim
         call operflag(n)
         if (dentype.eq.0) then
            call zeighpsi(n)
         else if (dentype.eq.1) then
            call d1zeighpsi(n)
         else if (dentype.eq.2) then
            call d2zeighpsi(n)
         endif
      enddo
        
      mcpsi1=mcdim
      mcdim=mcpsi1+dgldim
      mchpsi=mcdim
      mcdim=mchpsi+hpsidim
      if (lgwp) then  
         mcxp    = mcdim
         mcdim   = mcxp + xpmatdim + 1
      else 
         mcxp    = mcdim
         mcdim   = mcxp + 1
      endif

      if (lgwp) then
         mrqcent = mrdim
         mrdim   = mrqcent + qcentdim + 1
      else
         mrqcent = mrdim
         mrdim   = mrqcent + 1
      endif                                              

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! assign work arrays
! NB work arrays must be at end of mc, mr, mi, ml arrays.
!-----------------------------------------------------------------------
      call maxmatwk(nham)

!-----------------------------------------------------------------------
!  assign memory for operator
! workcdim etc needed for legacy
!-----------------------------------------------------------------------
      workcdim=1
      mcworkc=mcdim
      workidim=1
      miworki=midim
      workrdim=1
      mrworkr=mrdim
      workldim=1
      mlworkl=mldim
      if (lrdoper) call memhops

!-----------------------------------------------------------------------
! call analysis routine
!-----------------------------------------------------------------------
      if (logisopen) then
         call wrversion(ilog)
         ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
         write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
          ' complex*16 : ', mcdim,&
          ' real*8     : ', mrdim,&
          ' integer    : ', midim,&
          ' logical    : ', mldim,&
          ' complex*8  : ', msdim,&
          ' real*4     : ', mfdim,&
          ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
         call myflush(ilog)
      endif

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)
if(allocated(gfgwp))deallocate(gfgwp)
if(allocated(aigwp))deallocate(aigwp)
if(allocated(zgwp))deallocate(zgwp)
if(allocated(hteil))deallocate(hteil)
if(allocated(gh))deallocate(gh)
if(allocated(gs2))deallocate(gs2)
if(allocated(gs2))deallocate(gsvek)
if(allocated(hung))deallocate(hung)
if(allocated(agmat))deallocate(agmat)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))
allocate(gfgwp(gfgwpdim))
allocate(aigwp(aigwpdim))
allocate(zgwp(zgwpdim))
allocate(hteil(pmatdim))
allocate(gh(ghmatdim))
allocate(gs2(gs2matdim))
allocate(gsvek(gsmatdim))
allocate(hung(hungdim))
allocate(agmat(agmatdim))

      call subexpect(mc(mcpsi),mc(mcpsi1),mr(mrort),mc(mchin),&
           mc(mcrueck),mi(mifftfak),mr(mrhops),ms(mspsi),mi(mijindx),&
           mc(mchpsi),hteil,agmat,gfgwp,aigwp,zgwp,&
           mr(mrqcent),gh,gs2,hung,mc(mcxp),mc(mcadgwp),mi(mitrajst),&
           mi(migwpdep),mr(mrgsval),gsvek)

deallocate(mc,mr,mi,ml,ms,mf, &
           gfgwp,aigwp,zgwp,hteil,gh,gs2,gsvek,agmat)
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_psidef
      call dealloc_hpsimod
      call dealloc_dirdyn
      call dealloc_dddb
      call dealloc_dercp
      call dealloc_gapkernel
      call dealloc_runpropmod

      stop
  900 continue
      routine='Expect'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg

  999 continue
      write(6,'(2a)') 'Error reading :', filename
      stop
 1000 continue
      ilbl=index(filename,' ')-1
      inquire(file=filename(1:ilbl),exist=lexist)
      if (lexist) then
           write(6,'(2a,/a)') &
          ' The output file: ',filename(1:ilbl),' already exists!'&
         ,' overwrite disabled? (Use -w option to enable overwrite)'
      else
         write(6,'(2a)') 'File not found : ',filename(1:ilbl)
      endif
      stop

      contains


!***********************************************************************
!
!     HERE IS THE SUBROUTINE TO DO THE ANALYSIS
!
!***********************************************************************
      subroutine subexpect (psi,dtpsi,ort,hin,rueck,fftfak,hops,&
                 spsi,jindx,hpsi,hteil,&
                 agmat,gfgwp,aigwp,zgwp,qcenter,gh,gs2,hung,&
                 xpmat,adgwp,trajst,gwpdep,gsval,gsvek)

      implicit none

      complex(dop)                                   :: expval,aovl
      complex(dop), dimension(1)                     :: zdum,zdum1
      complex(dop), dimension(dgldim), intent(out)   :: psi,dtpsi
      complex(dop), dimension(fftdim), intent(out)   :: hin,rueck
      complex(dop), dimension(hpsidim), intent(out)  :: hpsi
      complex(dop), dimension(pmatdim), intent(out)  :: hteil
      complex(dop), dimension(gsmatdim), intent(out) :: adgwp
      complex(dop), dimension(agmatdim), intent(out) :: agmat
      complex(dop), dimension(gfgwpdim), intent(out) :: gfgwp
      complex(dop), dimension(aigwpdim), intent(out) :: aigwp
      complex(dop), dimension(zgwpdim), intent(out)  :: zgwp
      complex(dop), dimension(ghmatdim), intent(out) :: gh
      complex(dop), dimension(gs2matdim), intent(out):: gs2
      complex(dop), dimension(hungdim), intent(out)  :: hung
      complex(dop), dimension(xpmatdim), intent(out) :: xpmat
      complex(sip), dimension(spsidim), intent(out)  :: spsi

      real(dop), external                            :: fcputime,fsystime
      real(dop)                                      :: tcpu,tsys,ttot,time,&
                                                        spent,timeold
      real(dop), dimension(1)                        :: rdum,rdum1,rdum2,rdum3
      real(dop), dimension(qcentdim), intent(out)    :: qcenter
      real(dop), dimension(ortdim), intent(out)      :: ort
      real(dop), dimension(hopsdim), intent(out)     :: hops

      integer(long), external                         :: slen
      integer(long)                                   :: t1,t2,i,chkdvr,chkgrd,&
                                                         n,nham,ns,nz
      integer(long), dimension(1)                     :: idum,idum1
      integer(long), dimension(fftdim), intent(out)   :: fftfak
      integer(long), dimension(adim), intent(out)     :: jindx
      integer(long), dimension(*), intent(out)        :: trajst
      complex(dop), dimension(gsmatdim), intent(out)        :: gsvek
      real(dop), dimension(gwpinfodim), intent(out)         :: gsval
      integer(long), dimension(gwpinfodim),intent(out)      :: gwpdep

      character(len=120)                              :: gnucmd
      logical(kind=4)                                 :: lexist,limag

      logical(kind=4), dimension(1)                   :: ldum

! not sure where this is set but it may be printed.
      aovl=0.0
      ns=0
      nz=0

!-----------------------------------------------------------------------
! for GWP integrals use symmetric LHA as default and energy only
!-----------------------------------------------------------------------
      if (lgwp) then
         gwpintm(:)=2
         gwpintordm(:) = 0
         gwpcxm(:) = 0
      endif
       
!-----------------------------------------------------------------------
! set up ML
!-----------------------------------------------------------------------
      if (lrunml) then
         call inputml(tape)
         call wftree_set_pointers(tree,dgldim)
      endif

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! read data from dvr file if desired
! edit which data (set dvrdata(x) to .true. and substitute dummy array)
!-----------------------------------------------------------------------
      if (lrddvr) then
         dvrdata(1)=.true.     ! ort
         dvrdata(2)=.false.    ! trafo
         dvrdata(3)=.false.    ! dvrmat (dif1mat, dif2mat)
         dvrdata(4)=.false.    ! fftp (fft momentum)
         dvrdata(5)=.true.     ! hin (fft)
         dvrdata(6)=.true.     ! rueck (fft)
         dvrdata(7)=.true.     ! fftfak
         dvrdata(8)=.false.    ! jsph
         dvrdata(9)=.false.    ! msph
         dvrdata(10)=.false.   ! kinsph
         dvrdata(11)=.true.    ! exphin
         dvrdata(12)=.true.    ! exprueck
         chkdvr=1
         call rddvr(ort,rdum,rdum1,rdum2,hin,&
           rueck,fftfak,zdum,zdum1,idum,idum1,rdum3,chkdvr)
      endif

!-----------------------------------------------------------------------
! read data from oper file.
!-----------------------------------------------------------------------
      if (lrdoper) then
         chkdvr=1
         chkgrd=1
         call rdoper(hops,chkdvr,chkgrd)
      endif

!-----------------------------------------------------------------------
! Get operator number
!-----------------------------------------------------------------------
      nham=0
      do n=1,hamdim
         if (hamlab(n) .eq. operator) nham=n
      enddo

      if (logisopen) then
         write(ilog,'(a,i3,a,l2,3x,a)')  ' nham =',nham,&
              '   nodiag =',nodiag(nham),operator
         if(.not. nodiag(nham)) then
            write(ilog,'(/a/)')&
           ' ###  WARNING: Operator is "usediag" but should be "nodiag".'
         endif
      endif
      if(.not. nodiag(nham)) then
         write(6,'(/a/)') &  
        ' ###  WARNING: Operator is "usediag" but should be "nodiag".'
      endif

      write(iaus,'(/,2a)') '# Wavefunction is read from file : ',&
            filein(1:laein)
      write(iaus,'(2a)') '# Expectation value of operator : ',&
            operator(1:slen(operator))
      write(iaus,'(2a)') '#    Taken from file :',&
            operfile(1:slen(operfile))
      write(iaus,'(2a)') '#    Units : ',unitlab(1)
      if(lnwexpect) then
         write(iaus,'(a)') '#  ---  Not normalised!  ---'
         write(iaus,'(/,a11,2a22)')&
              '# Time[fs] ','  Real part','Imaginary part'
      else
         if(fs.gt.10.d0) then
            write(iaus,'(/,a11,2a22,a21)')&
              '# Time[fs] ','  Real part','Imaginary part','A-Norm^2'
         else
            write(iaus,'(/,a11,2a22,a21)')&
              '# Time[fs] ','  Real part','Imaginary part','A-Norm^2'
         endif
      end if
      call myflush(iaus)

      time  = tinit
      limag = .false.

!-----------------------------------------------------------------------
! allocate space for operator-related matrices in the multilayer modules
!-----------------------------------------------------------------------
      if (lrunml) then
         if (lgwp) then
            call setoperml(nham,.false.,.false.)
         else
            call setoperml(nham,.false.,.true.)
         endif
      endif

!-----------------------------------------------------------------------
! loop over steps
!-----------------------------------------------------------------------
      if (irst.ne.ipsi.and.nskip.gt.0)then
         do n=1,nskip
            call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
            if (lend) go to 77
            time=time+out2
         enddo
      endif
      nz=0
      ns=1
      if (logisopen) then
         ispeed=ilog
         spent=fcputime(0)+fsystime(0)
         call speedout(-out2*step)
      endif

   10 continue
      if ( irst .eq. ipsi ) then
         read(ipsi,err=990) time
         read(ipsi,err=990) (psi(i),i=1,dgldim)
      else
         call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
         if (lend) goto 77
      endif

      call zerovxz(dtpsi,dgldim)

!-----------------------------------------------------------------------
! calculate time-dependent coefficients
!-----------------------------------------------------------------------
      ldum = .false.
      if (ltdh(nham)) call calctdk(time,nham,psi)

      if (ltraj) then
         call trexpectvalue(psi,operator,expval,trajst,dim(1,1),nspfdof(1))

      else if (dentype.eq.0) then
!    --- wave functions
         print*,'Time = ',time/fs
         call wfexpectvalue(psi,nham,& ! input
              expval,hpsi,hteil,  &
              hops,jindx,ort, &       ! various other input 
              agmat,gfgwp,aigwp,zgwp,qcenter,gh,ghderiv1,gs2,&  ! GWP info
              hung,time,gsval,gsvek,aovl)

      else if (dentype.eq.1) then
!    --- density operators of type I
         call d1expectvalue(psi,nham,expval,hpsi)

      else if (dentype.eq.2) then         
!    --- density operators of type II
         call d2expectvalue(psi,nham,expval,hpsi,hteil,&  
              agmat,gfgwp,aigwp,zgwp,qcenter,gh,ghderiv1,gs2,&  
              hung,time,gsval,gsvek)

      endif

      if(nz.eq.0) then
         spent=fcputime(0)+fsystime(0)-spent
         if(spent*step .lt. 1.d0)  ns=10
         if(spent*step .lt. 1.d-1) ns=100
      endif

!-----------------------------------------------------------------------
! write values, continue or terminate the loop over data-sets.
!-----------------------------------------------------------------------

      if(lnwexpect) then
         write(iaus,'(f10.3,1x,2f22.14)') &
              time/fs,dble(expval)*eunit,dimag(expval)*eunit
      else
         write(iaus,'(f10.3,1x,2f22.14,6x,f15.12)') time/fs, &
              dble(expval)*eunit,dimag(expval)*eunit,dble(aovl) 
      end if
      if(mod(nz,ns) .eq. 0 .and. logisopen) call speedout(time)
      if(abs(dimag(expval)*eunit) .gt. 1.0e-12_dop ) limag = .true.
      timeold=time
      time=time+out2
      nz=nz+1
      call myflush(iaus)
      if (irst .eq. ipsi) go to 77

      do n=1,step-1
         call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
         if (lend) go to 77
         time=time+out2
      enddo
      if (time .gt. tfin*fs+0.1*out2) go to 77

      go to 10


 77   continue
      if(mod(nz,ns) .ne. 1 .and. logisopen) call speedout(timeold)
      close(ipsi)
      close(iaus)
      if(limag) write(6,'(a)') &
               ' Warning: The expectation value is complex!'

!-----------------------------------------------------------------------
! Plot and/or print the data.
!-----------------------------------------------------------------------
      if(lprint) then
         inquire(file="printplot",exist=lexist)
         open(iaus,file="printplot",form='formatted',status='unknown')
         if(.not.lexist) then
            open(iaus,file="printplot",form='formatted')
            write(iaus,'(a)') 'set terminal postscript'
            write(iaus,'(a)')'set output "| lpr" '
            write(6,'(a)')'File printplot created.'
         end if
         rewind iaus
         read(iaus,'(a1)') gnucmd
         read(iaus,'(a30)') gnucmd
         close(iaus)
         write(6,'(a)')'Press enter twice to print. Ctrl C to abort.'
         write(6,'(3a)')'Abort and edit printplot if you printcommand',&
                        ' is not: '//gnucmd(11:29)
      end if
      if(lagnu) then
!!      This may cause problems when running on Mac
!!        gnucmd = 'gnuplot -geometry +0+0 -persist "' &
        gnucmd = 'gnuplot -persist "' &
          //trim(fileout)//'"'
         call excmd(gnucmd)
      end if
      if(lprint) then
         gnucmd = 'gnuplot -persist '&
         //'"printplot"  "'//trim(fileout)//'"'
         call excmd(gnucmd)
      end if

!-----------------------------------------------------------------------
! Get overall timing 
!-----------------------------------------------------------------------
      if(fileout .eq. 'no') return   ! skip timing if output to screen
      tcpu = fcputime(0)
      tsys = fsystime(0)
      ttot = tcpu+tsys
      t1   = nint(ttot/3600)
      ttot = ttot - 3600*t1
      t2   = nint(ttot/60)
      ttot = ttot - 60*t2

      if (logisopen) then
         write(ilog,'(//,a,/)') ' ***** Overall Timing *****'
         write(ilog,'(a,1x,f12.2)') ' Total cpu time     [s] : ',tcpu
         write(ilog,'(a,1x,f12.2)') ' Total system time  [s] : ',tsys
         write(ilog, 900) ' Total time     [h:m:s] : ',t1,t2,ttot
         write(ilog,*)
      endif
      write(6,'(//,a,/)') '***** Overall Timing *****'
      write(6,'(a,1x,f12.2)') 'Total cpu time     [s] : ',tcpu
      write(6,'(a,1x,f12.2)') 'Total system time  [s] : ',tsys
      write(6, 900) 'Total time     [h:m:s] : ',t1,t2,ttot
      write(6,*)

      if (logisopen) then
         call fpstartup(inputfile)
         write(ilog,'(/a)') inputfile(1:slen(inputfile))
      endif
      
      return

  900 format( a,i4,' :',i3,' :',f5.1)
      
  990 write(6,'(/a)')    'Error while reading WF from RESTART file.'
      if (logisopen) &
         write(ilog,'(/a)') 'Error while reading WF from RESTART file.'
      stop

      end subroutine subexpect

!***********************************************************************
!-----------------------------------------------------------------------
! EXPINPUT reads the options and the argument.
!-----------------------------------------------------------------------
      subroutine expinput(stat)

use global
use aglobal
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod
use operdef
use runpropmod
use hpsimod
use channels

use runpropmod

      implicit none

      integer(long), external      :: myiargc
      integer(long)                :: iarg,i,ierr
      character(len=80)            :: buf
      character(len=*), intent(out):: stat
      character*(c5) filename

!-----------------------------------------------------------------------
! provide default input and output filenames 
!-----------------------------------------------------------------------
      filein  = ' '
      fileout  = ' '
      operfile  = ' '
      operator = ' '
      unitlab(1)='a.u.'
      name='./'

!-----------------------------------------------------------------------
! set default flags
!
! stat : status of output file   
!-----------------------------------------------------------------------
      stat   = 'new'
      lgnu   = .false.
      step   = 1
      nskip  = 0
      eunit  = 1.0
      lgnu   = .false.
      lagnu  = .false.
      lprint = .false.
      lnwexpect = .false.
      xmax   = 0.0_dop
      xmin   = 1.0_dop
      tfin   = 1.0e+50_dop

!-----------------------------------------------------------------------
! read options 
!-----------------------------------------------------------------------
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:4) .eq. '-op ') then
            i = i+1
            call mygetarg(i,buf)
            operfile = buf
         else if(buf(1:3) .eq. '-f ') then
            i = i+1
            call mygetarg(i,buf)
            filein = buf
         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if(buf(1:3) .eq. '-i ') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if (buf(1:3) .eq. '-r ' ) then
            lnwexpect = .true.
         else if (buf(1:5) .eq. '-rst ' ) then
            irst = ipsi
         else if (buf(1:3) .eq. '-t ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*) tfin

         else if( buf(1:4) .eq. '-DB ' ) then
            i = i+1
            call mygetarg(i,buf)
            ddpath = buf

         else if (buf(1:3) .eq. '-u ' ) then
            i = i+1
            call mygetarg(i,buf)
            call keylowc(buf)
            eunit = 1.0
            call keyunits(eunit,buf,ierr)
            if (ierr .ne. 0) then
               write(6,'(a)') 'Unknown Units with -u :'//buf
               stop
            endif
            unitlab(1)=buf(1:c1)
            eunit=1.0/eunit
         else if( buf(1:3) .eq. '-x ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=998) xmin
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=998) xmax
         else if( buf(1:3) .eq. '-n ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:5) .eq. '-skip' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nskip
            if (ierr .ne. 0  .or.  nskip .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -skip',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:6) .eq. '-nofs ' ) then
            fs = 1.0_dop
         else if( buf(1:3) .eq. '-G ' ) then
            lpgrid = .true.
         else if( buf(1:2) .eq. '-w' ) then
            lovwr=.true.
         else if( buf(1:2) .eq. '-g' ) then
            lgnu = .true.
         else if( buf(1:3) .eq. '-a ' ) then
            lgnu  = .true.
            lagnu = .true.
            stat  = 'unknown'
         else if( buf(1:3) .eq. '-p ' ) then
            lgnu   = .true.
            lprint = .true.
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,2a,/1x,2a,/17x,a)')&
       'Purpose: ',&
       'Calculates the time evolution of an expectation value.',&
       'Usage  : ',&
       'expect [-op -f -i -o -nofs -g -a -r -rst -x -t -p -u -n -skip',&
               '-MC -Macc -G -w -ver -h -?] [operator].'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a)') &
                     ' -op FILE : The operator is read '&
                    ,'from file FILE rather than from ./oper'
            write(6,'(2a,/,9x,a)') &
                     ' -f FILE : The wavefunction is read '&
                    ,'from file FILE rather than from ./psi'&
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,11x,a/,11x,a/,11x,a)')&
                     ' -o FILE : The output is written to file ',&
                     'FILE.pl rather than to ./expect.pl',&
             'The string FILE may be a relative or a full path-name.',&
             'If FILE=no, i.e. "-o no", no output file will be opened,',&
             'and the output is directed to screen.'
            write(6,'(a)') ' -i DIR : data is stored in directory DIR'
            write(6,'(2a)') 
            write(6,'(2a)')&
                     ' -nofs   : No transformation to fs.',&
                     " Use when 'time-not-fs' was set in mctdh."
             write(6,'(2a)')&
                     ' -g      : GNUplot comand lines are written to ',&
                     'the output file(s).'
            write(6,'(2a)')&
                     ' -a      : GNUplot is called automatically.',&
                     ' (sets -g and -w).'
            write(6,'(a)')&
                     ' -DB PATH : Sets path to DD DB relative to current location.'
            write(6,'(a)')&
                     ' -p      : Print GNUplot to lpr (sets -g). '
            write(6,'(a)')&
             ' -MC     : Use Monte-Carlo integration (useful for G-MCTDH evaluation). '
            write(6,'(a)')&
             ' -MCacc R: Set accuracy of Monte-Carlo integration to R (default R=0.01). '
            write(6,'(2a)')&
                     ' -r      : Do not divide the expectation values',&
                     ' by norm^2.'
            write(6,'(2a)')&
                     ' -rst    : Use restart rather than psi file.'
            write(6,'(2a)')&
                     ' -x xi xf: Set the abscissa length.'
            write(6,'(2a)')&
                     ' -t tfin : The expectation values are computed',&
                     ' only up to time=tfin.'
            write(6,'(2a)')&
                     ' -u UNIT : The unit used is UNITs',&
                     ' (Default is a.u.).'
            write(6,'(2a)') ' -n step : Compute the property',&
                            ' only every step-th output.'
            write(6,'(2a)') ' -skip nskip : Skip the first nskip',&
                            ' wavefunctions.'
            write(6,'(2a)') &
                     ' -G      : Plot gridlines',&
                     ' (meaningfull only if -g or -a is set).'
            write(6,'(2a)')&
                     ' -w      : An existing property file is ',&
                     'overwritten.'
            write(6,'(a)')&
                     ' -ver    : Version information about the program.'
            write(6,'(a,/a)')&
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(/2a/,2a)') ' The argument operator is the name',&
                            ' of the operator, as specified in an',&
                            ' HAMILTONIAN-SECTION_xxx, for which the',&
                            ' expectation value is required.'
            write(6,'(2a)') ' If no argument is given, ',&
                 'the user is prompted for the missing argument.'
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      if (iarg .lt. i) goto 200

!-----------------------------------------------------------------------
! get compulsory argument.
!-----------------------------------------------------------------------
      if (iarg .eq. i) call mygetarg(i,operator)

!-----------------------------------------------------------------------
! End of loop over options and arguments.
!-----------------------------------------------------------------------
 200  continue
 
!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      if(fileout .ne. 'no' ) then
         filename=fileout
         ilbl=index(filename,' ')-1
         if(filename(ilbl-2:ilbl) .eq. '.pl') then
            filename=filename(1:ilbl-3)
         endif
         filename=trim(filename)//'.log'
         open (unit=ilog,file=filename,form='formatted')
         logisopen=.true.
      else
         logisopen=.false.
      endif

!-----------------------------------------------------------------------
! set status variable to overwrite files
!-----------------------------------------------------------------------
      if (lovwr) stat='unknown'

!-----------------------------------------------------------------------
! convert filenames to absolute
!-----------------------------------------------------------------------
      laenge=index(name,' ')-1
      call abspath(name,laenge)

      if( filein .eq. ' ') then
         if(ipsi.eq.irst) then
            filein=name(1:laenge)//'/restart'
         else
            filein=name(1:laenge)//'/psi'
         endif
      endif
      laein=index(filein,' ')-1
      if (filein(max(1,laein-6):laein) .eq. 'restart') irst = ipsi
      if (filein(laein:laein)   .eq. '/') then
         if(ipsi.eq.irst) then
            filein = filein(1:laein)//'restart'
            laein  = laein + 7
         else
            filein = filein(1:laein)//'psi'
            laein  = laein + 3
         endif
      end if

      call abspath(filein,laein)

      dname=name
      dlaenge=laenge

      if (operfile .eq. '  ') then
         operfile=name(1:laenge)//'/oper'
         opflaenge=index(operfile,' ')-1
      else
         opflaenge=index(operfile,' ')-1
         call abspath(operfile,opflaenge)
      endif

      if (fileout .eq. '  ') then
         fileout=name(1:laenge)//'/expect.pl'
         laeout=index(fileout,'  ')-1
      else if (fileout(1:3) .ne. 'no ') then
         laeout=index(fileout,'  ')-1
         call abspath(fileout,laeout)
      endif

      return

  998 continue
      write(6,*) 'Two real numbers expected after -x  option '
      stop
      end subroutine expinput


      end program expect

