!-----------------------------------------------------------------------

      program fdcheck

!-----------------------------------------------------------------------
! Analysis the output of the program "fdcheck", i.e. checks whether a
! state has been found in all comparison calculations and computes the
! internal error. See on-line manual for details.
!-----------------------------------------------------------------------

      implicit none

      integer, parameter :: long  = selected_int_kind(9)    
      integer, parameter :: dop = selected_real_kind(15,307) ! real*8 and complex*16

      integer(long), parameter :: maxenergies = 5000, maxdatasets = 20
      real(dop), parameter     :: missing = -1.0e+99_dop

      logical(kind=4)                                    :: ok
      integer(long)                                      :: iargc
      integer(long)                                      :: datasets,line,lines,set,found,exactflag,set2,i
      real(dop)                                          :: meane,derive,meani,derivi,ave,ee 
      real(dop), dimension(1:maxdatasets)                :: energy, intensity
      character(len=512)                                 :: argument
      character(len=512), dimension(1:maxdatasets)       :: inputline

!-----------------------------------------------------------------------
!     Get number of data sets from command line
!-----------------------------------------------------------------------

      call getarg(1,argument)
      if(argument .eq. '-h'  .or. argument .eq. ' ' ) goto 678  
      if (iargc() .ne. 2) goto 666
      call getarg(1,argument)
      read(argument,*,err=777) datasets
      if (datasets .le. 1) goto 777
      if (datasets .gt. maxdatasets) then
         write(6,'(/a)') 'ERROR! Too many data sets.'
         write(6,'(a,i3,a,i3)') 'datasets =', datasets,&
              ',  maxdatasets =',maxdatasets
         stop
      endif 
      call getarg(2,argument)
      if(argument .eq. '-h') goto 678  
      read(argument,*,err=777) exactflag
      if ((exactflag .ne. 0) .and. (exactflag .ne. 1)) goto 777

!-----------------------------------------------------------------------
!     Write headline
!-----------------------------------------------------------------------
      if (exactflag .eq. 1) then
         write(6,'(a,11x,a,11x,a,8x,a,8x,a,10x,a,10x,a,8x,a)') &
              '#      Eex','E','DE','E-Eex','Iex','I','DI','I-Iex'
      endif

!-----------------------------------------------------------------------
!     Read data from stdin
!-----------------------------------------------------------------------

      lines = 0
      i = 0
      ave = 0.0_dop
 100  continue
      read(5,'(a)',end=200) inputline(lines+1)
      lines = lines+1
      read(inputline(lines)(1:18),*,err=100,end=100) ee
      ave=ave+abs(ee)
      i=i+1
      goto 100
 200  continue
      ave=ave/dble(10*i+1)

!-----------------------------------------------------------------------
!     Check each input line and output the results
!-----------------------------------------------------------------------

      do line = 1,lines
         do set = 1,datasets
            if(inputline(line)(2+(set-1)*36:35+(set-1)*36) .eq.&
                 '                                  ' ) then
               energy(set)    = missing
               intensity(set) = missing
            else
               read(inputline(line)(1+(set-1)*36:36+(set-1)*36),*,&
                    err=300) energy(set), intensity(set)
               goto 400
 300           continue
               write(6,'(a,2i4)') &
                    ' ERROR reading inputline, line, set =',line,set
               energy(set)    = missing
               intensity(set) = missing
 400           continue
            endif
         enddo
         found = 0
         do set = exactflag+1,datasets
            if (energy(set) .ne. missing) found = found+1
         enddo
         if (found .eq. datasets-exactflag) then
            meane  = 0.0_dop
            meani  = 0.0_dop
! ----------------------------------------------------------
! In "M. Beck, PhD thesis", the last set of energies (which was obtained
! using the 1/H variational principle) was not taken into account for
! the computation of the mean values.
!            do set = exactflag+1,datasets-1
! ----------------------------------------------------------
            do set = exactflag+1,datasets
               if (energy(set) .ne. missing) then
                  meane = meane+energy(set)
                  meani = meani+intensity(set)
               endif
            enddo
! See comment above.
!            meane = meane/dble(found-1)
!            meani = meani/dble(found-1)
            meane = meane/dble(found)
            meani = meani/dble(found)
            derive = 0.0_dop
            derivi = 0.0_dop
! ----------------------------------------------------------
! See comment above.
!            do set = exactflag+1,datasets-1
! ----------------------------------------------------------
            do set = exactflag+1,datasets
               if (energy(set) .ne. missing) then
                  derive = derive+(energy(set)-meane)**2
                  derivi = derivi+(intensity(set)-meani)**2
               endif
            enddo
! ----------------------------------------------------------
! See comment above.
!            derive = sqrt(derive/dble(found-1))
!            derivi = sqrt(derivi/dble(found-1))
! ----------------------------------------------------------
            derive = sqrt(derive/dble(found))
            derivi = sqrt(derivi/dble(found))
         endif
         ok = found .eq. (datasets-exactflag)
! ----------------------------------------------------------
! In "M. Beck, PhD thesis", only those values were accepted where the
! difference of the energies was less than 1/1000 of the mean value.
! ave is 1/10 of the average energy of the first set. This is to avoid
! problems when E is close to zero.
! ----------------------------------------------------------
         do set = 1,datasets-1
            do set2 = set+1,datasets
               if (abs(energy(set)-energy(set2)) .gt.&
                   1.0e-3_dop*max(abs(meane),ave)) ok = .false.
               if (meani .lt. 1.0e-6_dop) ok = .false.
            enddo
         enddo
! ----------------------------------------------------------
         if (exactflag .eq. 1) then
            if (energy(1) .ne. missing) then
               if (ok) then
                  write(6,'(4f12.3,4(1pe12.2e2))')&
                     energy(1), meane, derive, abs(energy(1)-meane),&
                     intensity(1), meani, derivi,&
                     abs(intensity(1)-meani)
               else
                  write(6,'(f12.3,36x,1pe12.2e2)')&
                     energy(1), intensity(1)
               endif
            else
               if (ok) then
                  write(6,'(12x,2f12.3,24x,2(1pe12.2e2))')&
                     meane, derive, meani, derivi
               endif
            endif
         else
            if (ok) then
               write(6,'(2f15.6,2(1pe15.5e2))')&
                  meane, derive, meani, derivi
            endif
         endif
      enddo
      write(6,*)

      stop

!-----------------------------------------------------------------------
!     Output error message
!-----------------------------------------------------------------------

 666  continue
      write(6,'(a)') 'ERROR: illegal number of arguments specified.'
      stop

 678  continue
      write(6,'(/2a,/9x,a)') 'Purpose: Checks whether a state ',&
              'has been found in all comparison ',&
              'calculations  and computes the internal error.'

      write(6,'(/a)') 'Usage  : fdcheck k exact datasets '
      write(6,'(/5x,a)') 'k = number of datasets.'
      write(6,'( 5x,a)') &
      'exact = 0 if if there is no exact comaprison data set, =1 else.'
      write(6,'( 5x,a)') 'datasets = data as created by fdmatch.'
      write(6,'( 5x,a)') 'Example: fdcheck 4 0 < list'
      write(6,'( 5x,a)') &
      'where list contains the output of fdmatch, or:'
      write(6,'( 5x,a)') &
      'fdmatch f1.eig f2.eig f3.eig | sort -n | fdcheck 3 0'
      write(6,*)
      stop


 777  continue
      write(6,'(a)') 'ERROR: illegal argument specified.'
      stop

      end program fdcheck










