!***********************************************************************
!
!                     ANALYSE ROUTINE FINDCOIN
!
!     This routine optimises the MECI in a potential
!
!***********************************************************************

      module findcoinmod

use decimal, only: long,dop,sip
use maxv, only: maxdim,maxsta
use global, only:pi, fs, ev

use potevalmod, only: calcdiab,calcdiabder

! states for CoIn search
      integer(long), save       :: s1=1, s2=2
      integer(long), save       :: maxiter=10

      real(dop), save                            :: lambda_lagrange,sigma
      real(dop), dimension(:), allocatable, save :: q_ci_low,q_ci_0,&
                                                    gd_low,dcp_low,dq_ci
      integer(long) :: eqdim
! specify system Hamiltonian
      integer(long) :: nham = 1
! arrays with operators and coordinates
      real(dop), dimension(:), allocatable :: hops,ort

      end module findcoinmod

!#######################################################################

      program findcoin

use global
use globmemmod
use aglobal
use dvrdatmod
use griddatmod
use operdef
use psidef
use channels
use datenmod
use compdatmod
use versions
use keyunitsmod

use op1lib, only: cpvxi, cpvxd
use xvlib, only: xvixddo, vvtxdd
use logdat, only: errormsg
use rddvrmod, only: dvrinfo
use rdopermod, only: operinfo
use zeigauswmod, only: zeigausw
use memhopsmod, only: memhops
use findcoinmod
use dirdyn
use dd_db, only: ddq2x

use module_interface

use rdopermod, only: rdoper
use rddvrmod

      implicit none

      integer(long)                     :: chkdvr,chkgrd,chkdat,ilbl,chkpsi
      character(len=c5)                 :: filename
      logical(kind=4)                   :: lerr

      logical(kind=4)   :: check=.false.
      character(len=240)      :: host

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare files to be read
! set flag to .true. if file is to be read
!
!     lrddvr    DVR file
!     lrdoper   OPER file.
!-----------------------------------------------------------------------
      lrddvr=.true.
      lrdoper=.true.

!-----------------------------------------------------------------------
! read input options
!-----------------------------------------------------------------------
      call coininput

!-----------------------------------------------------------------------
! get array dimensions 
!-----------------------------------------------------------------------
      inquire(ioper,opened=check)
      if (check) close(ioper)
      filename=name(1:laenge)//'/oper'
      ilbl=index(filename,' ')-1
      open(ioper,file=filename(1:ilbl),form='unformatted',status='old',err=1000)
      call rdmemdim(ioper)
      close(ioper)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      call alloc_dirdyn
      call alloc_dddb
      call alloc_dercp

!-----------------------------------------------------------------------
!  Read the system information
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!  Opening the DVR file.
!-----------------------------------------------------------------------
      write(6,'(/,a80)') host
      filename=dname(1:dlaenge)//'/dvr'
      open(idvr,file=filename,form='unformatted',status='old',err=1000)
      ilbl=index(filename,' ')-1
      write(6,'(2a)') ' The DVR file is : ',filename(1:ilbl)
      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      if (lerr) go to 1010
      close(idvr)

!-----------------------------------------------------------------------
!  Opening the oper and output files.
!-----------------------------------------------------------------------
      filename=operfile
      open(ioper,file=filename,form='unformatted',status='old',err=1000)
      ilbl=index(filename,' ')-1
      write(6,'(2a)') ' The operator file is : ',filename(1:ilbl)
      chkdvr=2
      chkgrd=1
      call operinfo(lerr,chkdvr,chkgrd)
      if (lerr) go to 1010
      close(ioper)

      if (.not. lgwp) then
         write(6,*)
         write(6,*) 'ERROR: Findcoin needs an analytic PES. '
         write(6,*) 'Regenerate oper file with GWP primitve basis'
         stop
      endif

!-----------------------------------------------------------------------
!  open output file
!-----------------------------------------------------------------------
      if (fileout .eq. ' ') fileout=name(1:laenge)//'/coin'
      laeout=index(fileout,' ')-1
      call abspath(fileout,laeout)
      filename=fileout
      write(6,'(2a)') ' The outputfile is : ',fileout(1:laeout)
      if(lovwr) then
         open(iaus,file=filename,form='formatted',status='unknown',&
              err=1020)
      else
         open(iaus,file=filename,form='formatted',status='new',&
              err=1020)
      endif

      write(iaus,*)
      write(iaus,'(a80)') host
      write(iaus,'(1x,a,/)') name(1:laenge)

      if(s1.gt.nddstate .or. s2.gt.nddstate) then
         write(6,'(/,a,i3,a,i3,a,i3)') ' There are only ',nddstate,&
            ' states, but you are requesting Coin between =',s1,',',s2 
         stop   
      else
         write(6,'(/,a,i3,a,i3,a,i3)') ' There are ',nddstate,&
            ' states. Requesting Coin between =',s1,',',s2 
      endif

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare resources required (other than those needed to read a file)
!
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! ltrafo : FBR/DVR trafo matrices
!-----------------------------------------------------------------------
      ncomp=0
      lpsi=.false.
      lpsi1=.false.
      lpsigrd=.false.
      ldmat=.false.
      ldmat1=.false.
      lort=.true.
      lgpop=.false.
      ltrafo=.false.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! open input and output data files
!
! use standard channels if possible
!  irho : gridpop file
!  ipsi : psi file
!  iaut : auto file
!  ichk : check file
!  iaus : output file
!-----------------------------------------------------------------------
! psifile is opened in pesinput if required
!     filename='psi'
!     open(ipsi,file=filename,form='unformatted',status='old',err=1000)

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
! First read file version number
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1

!-----------------------------------------------------------------------
! read information from system files
!-----------------------------------------------------------------------
      if (lrddvr) then
         chkdvr=2
         filename=dname(1:dlaenge)//'/dvr'
         open(idvr,file=filename,form='unformatted',status='old',&
              err=1000)
         call dvrinfo(lerr,chkdvr)
         close(idvr)
         call chksyserr(filename,lerr)
         if (lerr) go to 1010
      endif
      if (lrdoper) then
         chkdvr=2
         chkgrd=2
         filename=operfile
         open(ioper,file=filename,form='unformatted',status='old',&
              err=1000)
         call operinfo(lerr,chkdvr,chkgrd)
         close(ioper)
         call chksyserr(filename,lerr)
         if (lerr) go to 1010
      endif
      
!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      mcworkc = mcdim
      mcdim   = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
! assign memory for operator
!-----------------------------------------------------------------------
      call memhops

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

      call optcoin

deallocate(mc,mr,mi,ml,ms,mf)
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_dirdyn
      call dealloc_dddb
      call dealloc_dercp

      stop
 1000 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop
 1010 continue
      ilbl=index(filename,' ')-1
      message = 'Error reading file: '//filename(1:ilbl)
      call errormsg
      stop
 1020 write(6,*)' Cannot open the output file : '
      ilbl=index(filename,' ')-1
      write(6,*) filename(1:ilbl)
      write(6,*) 'Use option -w to overwrite the output file.'
      stop
      contains


!#######################################################################

      subroutine optcoin

      implicit none

      integer(long), external                          :: slen
      integer(long)                                    :: n
      integer(long)                                    :: i,j,chkdvr,chkgrd
      integer(long)                                    :: t1,t2
      integer(long), dimension(1)                      :: idum,idum1,idum2
      integer(long), dimension(maxdim)                 :: point
      real(dop), external                              :: fcputime,fsystime
      real(dop)                                        :: tcpu,tsys,ttot
      real(dop)                                        :: qnorm,dqnorm
      real(dop), dimension(1)                          :: rdum,rdum1,rdum2,rdum3
      complex(dop), dimension(1)                       :: zdum,zdum1,zdum2,zdum3
      character(len=80)                                :: buffer
      character(len=10)                                :: atnamup

      integer(long)                :: iter
      real(dop)                    :: fret1,tol,norm_low,&
                                      norm_1,norm_2,citol,lb
      real(dop)                    :: E1,E2
      real(dop), dimension(3,natm) :: xcoo
      real(dop), dimension(maxsta,maxsta)    :: pesdia
      real(dop), dimension(maxsta)           :: pesad
      complex(dop), dimension(maxsta,maxsta) :: rotmatz

      allocate(ort(ortdim))
      allocate(hops(hopsdim))

!-----------------------------------------------------------------------
! read DVR and operator data
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1

      dvrdata(1)=.true.
      call rddvr(ort,rdum,rdum1,rdum2,zdum,zdum1,idum,zdum2,zdum3,idum1,&
                 idum2,rdum3,chkdvr)

      operfile=operfile
      call rdoper(hops,chkdvr,chkgrd)

! No. equantions = ndof + 1   to include lagrange multiplier
      eqdim = ndof+1

! System Hamiltonian
      nham = 1

! Cannot use ldbsmall for calculating surfaces
      if (ldd .and. ldbsmall) then
         ldbsmall = .false.
      endif

      write(ilog,'(a,i5)') 'No. DOF =',ndof

!-----------------------------------------------------------------------
! allocate memory
!-----------------------------------------------------------------------
      if (allocated(q_ci_low)) deallocate(q_ci_low)
      if (allocated(q_ci_0)) deallocate(q_ci_0)
      if (allocated(gd_low)) deallocate(gd_low)
      if (allocated(dcp_low)) deallocate(dcp_low)
      if (allocated(dq_ci)) deallocate(dq_ci)
      allocate(q_ci_low(eqdim))
      allocate(q_ci_0(eqdim))
      allocate(dq_ci(eqdim))
      allocate(gd_low(maxdim))
      allocate(dcp_low(maxdim))

      write(iaus,*) '#################################################'
      write(iaus,*)  'Calculating positions of conical intersections   '
      write(iaus,*) '#################################################'

      write(iaus,'(/,a,2i3)') '           States ',s1,s2
      write(iaus,*) '          the lowest conical intersection        '
      write(iaus,*) '================================================='
      write(iaus,*) '                                                 '
      write(iaus,*) ' n      E1        E2       delta    norm(Q)   '//&
           ' |Grad|    '
      write(iaus,*) '-------------------------------------------------------'

      q_ci_low(:) = 0.0         ! initial coordinate is FC point.
      q_ci_low(ndof+1) = 0.0   ! the langrangian multiplier
      point(:) = 0  ! "grid point" not needed for analytical functions

! Initial Energies 
      call calcdiab(hops,pesad,pesdia,rotmatz,point,q_ci_low,nham)
      E1 = pesad(s1)
      E2 = pesad(s2)
      call df_ci0(q_ci_low,dq_ci)

      n=0
      call normvxd(q_ci_low,qnorm,ndof)
      call normvxd(dq_ci,dqnorm,ndof)
      write(iaus,'(i4,5F10.5)') n,E1,E2,E2-E1,qnorm,dqnorm
      call myflush(iaus)

      write(iaus,*) '-------------------------------------------------------'

      citol = 1.0d-3
      tol=1.0e-10
! First push to near the seam minimising energy gap
      do j=1,maxiter
         write(iaus,'(a)') 'Moving to Seam'
         iter = 100

         lb = -999.0_dop
         call frprmn(q_ci_low,ndof,tol,iter,fret1,f_ci0,df_ci0,ilog,&
              lb,1)

         call calcdiab(hops,pesad,pesdia,rotmatz,point,q_ci_low,nham)
         E1 = pesad(s1)
         E2 = pesad(s2)
         call df_ci0(q_ci_low,dq_ci)

         n=iter
         call normvxd(dq_ci,dqnorm,ndof)
         call normvxd(q_ci_low,qnorm,ndof)
         write(iaus,'(i4,5F10.5)') n,E1,E2,E2-E1,qnorm,dqnorm

         if (ldd) then
            if (lddtrans) then
               call ddq2x(q_ci_low,xcoo)
            else
               call cpvxd(q_ci_low,xcoo,ndofddpes)
            endif

            write(iaus,'(/,a)') 'Coordinates at Seam (Angstrom)'
            do n=1,natm
               atnamup=atnam(n)
               call keyupc(atnamup)  ! convert to upper case
               write(iaus,'(a5,3f12.6)') atnamup,(xcoo(i,n)*angstroem,i=1,3)
            enddo
         endif

         write(iaus,*) '-------------------------------------------------------'
         call myflush(iaus)

! Now optimise along seam, projecting out branching space
         write(iaus,'(a)') 'Optimising CoIn'
         iter = 100

         lb = -999.0_dop

         call frprmn(q_ci_low,ndof,tol,iter,fret1,f_ci1,df_ci1,ilog,&
              lb,1)

         call calcdiab(hops,pesad,pesdia,rotmatz,point,q_ci_low,nham)
         E1 = pesad(s1)
         E2 = pesad(s2)
         call df_ci1(q_ci_low,dq_ci)

         n=iter
         call normvxd(q_ci_low,qnorm,ndof)
         call normvxd(dq_ci,dqnorm,ndof)
         write(iaus,'(i4,5F10.5)') n,E1,E2,E2-E1,qnorm,dqnorm
         call myflush(iaus)

         write(iaus,*) '-------------------------------------------------------'

! If we have strayed from seam, repeat
         if (E2-E1 .lt. citol) exit
      enddo

! final energy at MECI
      call calcdiab(hops,pesad,pesdia,rotmatz,point,q_ci_low,nham)
      E2 = pesad(s2)

! DCP/GD at MECI
      gd_low(:)=0.0_dop
      dcp_low(:)=0.0_dop
      if (iter .gt. 0) then   !  Calculate GD/DCP only if CoIn found
! If diabatic = adiabatic then H12 = 0
! If not, need to get rotation matrix and rotate gradients. TO BE DONE.
         if (abs(pesdia(1,2)) .gt. 0.0001) then
             write(iaus,'(a,f10.6)') 'Warning: E_coup',pesdia(1,2)
             write(iaus,'(a)') 'Need to rotate diabatic basis. Not yet implemented'
         endif
         call coin_gddcp(s1,s2,q_ci_low,gd_low,dcp_low)
      endif

      gd_low = gd_low*uev
      dcp_low = dcp_low*uev
      call normvxd(q_ci_low,norm_low,ndof)
      call normvxd(gd_low,norm_1,ndof)
      call normvxd(dcp_low,norm_2,ndof)

      write(iaus,'(/,a)') ' --- Coordinates --- '
      write(iaus,'(4x,a)') &
                  'mode            Q          GD (eV)     DCP (eV) ' 
      write(iaus,*) '-------------------------------------------------------'
      do i=1,ndof
         if (i .eq. feb) cycle  
         write(iaus,'(2x,i3,1x,a7,3f12.5)') i,modelabel(i),q_ci_low(i),&
                      gd_low(i),dcp_low(i)
      enddo
      write(iaus,*) '-------------------------------------------------------'
      write(iaus,'(a,3F12.5)')'   ||Q|| =   ',norm_low,norm_1,norm_2 
      write(iaus,'(a,F12.5,a)') '   E_ci =    ',E2*ev,' eV'
      write(iaus,*) '-------------------------------------------------------'

      if (ldd) then
         if (lddtrans) then
            call ddq2x(q_ci_low,xcoo)
         else
            call cpvxd(q_ci_low,xcoo,ndofddpes)
         endif

         write(iaus,'(/,a)') 'Coordinates at MECI (Angstrom)'
         do n=1,natm
            atnamup=atnam(n)
            call keyupc(atnamup)  ! convert to upper case
            write(iaus,'(a5,3f12.6)') atnamup,(xcoo(i,n)*angstroem,i=1,3)
         enddo
      endif

      deallocate(q_ci_low,q_ci_0,gd_low,dcp_low,dq_ci)
      deallocate(ort,hops)

!-----------------------------------------------------------------------
! Get overall timing 
!-----------------------------------------------------------------------
      tcpu = fcputime(0)
      tsys = fsystime(0)
      ttot = tcpu+tsys
      t1   = int(ttot/3600)
      ttot = ttot - 3600*t1
      t2   = int(ttot/60)
      ttot = ttot - 60*t2

      write(iaus,'(//,a,/)') '***** Overall Timing *****'
      write(iaus,'(a,1x,f12.2)') 'Total cpu time     [s] : ',tcpu
      write(iaus,'(a,1x,f12.2)') 'Total system time  [s] : ',tsys
      write(iaus, 900) 'Total time     [h:m:s] : ',t1,t2,ttot
      write(iaus,*)

! --- WRITE DATE AND MACHINE  ---
      call fpstartup(buffer)
      write(iaus,*)
      write(iaus,'(a80)') buffer
      write(iaus,'(1x,a)') name(1:laenge)
      write(6,*)
      write(6,'(a80)') buffer
      write(6,'(1x,a,/)') name(1:laenge)

  900 format( a,i4,' :',i3,' :',f5.1)
      return

      return
      end subroutine optcoin

!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------

      subroutine coininput

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: i,iarg,ierr,tmplen
      character(len=240)      :: buf

      character(len=c5)  :: trajectory, compare
      logical(kind=4) ::  ltrajectory, lcompare, lstartup
      common /trajcomp/ trajectory,compare,ltrajectory,lcompare,lstartup

      operfile = ' '
      fileout  = ' '
      name     = './'
      ddpath = '..'
      iarg = myiargc()
      unitlab(1) = 'ev' 
      step  = 100
      lwrsrf = .false.

! --- WRITE DATE AND MACHINE  ---
      write(6,'(78a1)')  ('-',i=1,78)
      call fpstartup(host)

      i = 0
 100  i=i+1
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if(buf .eq. '-i') then
            i = i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf(1:3) .eq. '-p ') then
            i = i+1
            call mygetarg(i,buf)
            operfile = buf
         else if( buf(1:2) .eq. '-o' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         elseif(buf .eq. '-w') then
            lovwr = .true.
         else if( buf(1:4) .eq. '-DB ' ) then
            i = i+1
            call mygetarg(i,buf)
            ddpath = buf
         else if( buf(1:4) .eq. '-nit ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*) maxiter
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(1x,a)')&
       'Purpose: Sarches for a CoIn between states s1 and s2. Default &
       & s1=1, s2=2 '
            write(6,'(1x,2a)')&
       'Usage:   findcoin [ -i -p -o -w -nit -DB -ver -h -?] [s1 s2]' 
            write(6,'(/,a)') 'Options  : '
            write(6,'(2a)') ' -i DIR  : DIR is taken as',&
                            '  name-directory rather than ./'
            write(6,'(2a,/,9x,a)') &
                     ' -p FILE : the PES is read '&
                    ,'from FILE rather than from ./oper',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a,/,9x,a)') &
                     ' -o FILE : The ouput is written to file ',&
                     'name/FILE rather than to name/coin',&
            '  The string FILE may be a relative or a full path-name.',&
            '  If FILE=no, i.e. -o no, no output file will be written.'
            write(6,'(2a)') &
                     ' -w      : Overwrite the output-file coin.'
            write(6,'(2a)') &
                     ' -nit n  : No. of iterations (default n = 10).'
            write(6,'(a,/,12x,a)') &
            ' -DB PATH : DB is in directory with relative path PATH.'
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         goto 100
      end if

      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) s1
      if (ierr .ne. 0) then
         write(6,*)'The first state must be an integer. Not :'
         write(6,*) buf
         stop
      end if
     
      i=i+1
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) s2
      if (ierr .ne. 0) then
         write(6,*)'The second state must be an integer. Not :'
         write(6,*) buf
         stop
      end if
     
  200 continue

      if (s1 .gt. s2) then
         i=s2
         s2=s1
         s1=i
      endif

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='coin.log',form='formatted')
      logisopen=.true.
      
      laenge=index(name,' ')-1
      call abspath(name,laenge)

      if (operfile .eq. ' ') operfile=name(1:laenge)//'/oper'
      opflaenge=index(operfile,' ')-1
      call abspath(operfile,opflaenge)

      dname=name
      dlaenge=index(dname,' ')-1
      call abspath(dname,dlaenge)

      if (ltrajectory) then
         tmplen = index(trajectory,' ')-1
         call abspath(trajectory,tmplen)
      end if

      if (lcompare) then
         tmplen = index(compare,' ')-1
         call abspath(compare,tmplen)
      end if

      return 
      stop
      
      end subroutine coininput

!#######################################################################
! Gets GD and DCP for CoIn between states s1 and s2
!#######################################################################

      subroutine coin_gddcp(s1,s2,q,gd,dcp)

      implicit none

      integer(long), intent(in) :: s1,s2
      integer(long)             :: i
      real(dop), dimension(maxdim), intent(in) :: q
      real(dop), dimension(maxdim), intent(out) :: gd,dcp
      real(dop), dimension(maxsta,maxsta,maxdim) :: derdia
      real(dop), dimension(maxsta,maxsta,maxdim) :: derad
      complex(dop), dimension(maxsta,maxsta)     :: rotmatz

      call calcdiabder(hops,derad,derdia,rotmatz,q,nham)

      do i=1,ndof
         if (i .eq. feb) cycle
         gd(i) = derdia(s2,s2,i) - derdia(s1,s1,i)
         dcp(i) = derdia(s1,s2,i) 
      enddo

      return
      end subroutine coin_gddcp

!############################################################
!
!############################################################

      real(dop) function f_ci0(q)

      implicit none

      real(dop), intent(in)        :: q(maxdim)

      integer(long), dimension(maxdim) :: point
      real(dop), dimension(maxsta,maxsta)    :: pesdia
      real(dop), dimension(maxsta)           :: pesad
      complex(dop), dimension(maxsta,maxsta) :: rotmatz

      real(dop) :: E1,E2,delta

      point(:) = 0
      call calcdiab(hops,pesad,pesdia,rotmatz,point,q,nham)

      E2 = pesad(s2)
      E1 = pesad(s1)
      delta = abs(E2-E1)

      f_ci0 = delta

      return
      end function f_ci0

!---------------------------------------------------------

      subroutine df_ci0(q,dq)

      implicit none

      integer(long)                                 :: n,idum
      real(dop)                                     :: h,err
      real(dop), dimension(4*maxdim*maxdim*2)       :: work
      real(dop), dimension(*), intent(in)           :: q
      real(dop), dimension(*), intent(out)          :: dq
      real(dop), external :: numder1r

      idum=0
      dq(1:ndof+1) = 0.0
      do n=1,ndof
         if (n .eq. feb) cycle
         h=0.001
         dq(n)=numder1r(f_ci0,q,ndof,n,h,err,work,idum)
      enddo

      return
      end subroutine df_ci0

!############################################################
!
!############################################################

      real(dop) function f_ci1(q)

      implicit none

      real(dop), intent(in)        :: q(maxdim)

      integer(long), dimension(maxdim) :: point
      real(dop), dimension(maxsta,maxsta)    :: pesdia
      real(dop), dimension(maxsta)           :: pesad
      complex(dop), dimension(maxsta,maxsta) :: rotmatz

      real(dop) :: E1,E2

      point(:) = 0
      call calcdiab(hops,pesad,pesdia,rotmatz,point,q,nham)

      E2 = pesad(s2)
      E1 = pesad(s1)

      f_ci1 = E2 

      return
      end function f_ci1

!---------------------------------------------------------

      subroutine df_ci1(q,dq)

      implicit none

      integer(long)                                 :: n,idum
      real(dop)                                     :: h,err
      real(dop), dimension(4*maxdim*maxdim*2)       :: work
      real(dop), dimension(*), intent(in)           :: q
      real(dop), dimension(*), intent(out)          :: dq
      real(dop), external :: numder1r

      real(dop), dimension(maxdim)           :: gd,dcp
      real(dop)                              :: dnorm

      idum=0
      do n=1,ndof
         if (n .eq. feb) cycle
         h=0.001
         dq(n)=numder1r(f_ci1,q,ndof,n,h,err,work,idum)
      enddo

! Get GD and dcp. Normalise vectors.
      call coin_gddcp(s1,s2,q,gd,dcp)
      call normvxd(gd,dnorm,ndof)
      call xvixddo(dnorm,gd,ndof)
      call normvxd(dcp,dnorm,ndof)
      call xvixddo(dnorm,dcp,ndof)

! Project out gradiennt in branching space
      call vvtxdd(gd,dq,dnorm,ndof)
      do n=1,ndof
         dq(n) = dq(n) - dnorm*gd(n)
      enddo
      call vvtxdd(dcp,dq,dnorm,ndof)
      do n=1,ndof
         dq(n) = dq(n) - dnorm*dcp(n)
      enddo

      return
      end subroutine df_ci1

!#######################################################################
      end program findcoin

