!***********************************************************************
!
!                         FLX2UT
!
!     Set of routines needed for flux.
!     Contents: operin, fullflx, fdrvflx
!
!
!***********************************************************************

      module flx2ut

use versions, only: progver
use decimal, only: long,dop,sip

use global
use globmemmod
use aglobal
use maxv
use dvrdatmod
use griddatmod
use psidef
use datenmod
use operdef
use rddvrmod
use rdopermod
use channels
use fluxmod
use flxutils, only: tout,fullgth0,fullgtheta,getstart
use flxreduce
use logopmod, only: buffterm

use op1lib, only: cpvxz
use xvlib, only: vvaxzz, dvxxdz
use mtlib, only: dtxxdz
use fftmod

      implicit none

      private
      public :: operin,fullflx,fdrvflx
      contains

      subroutine operin(zcap,zortf)

!***********************************************************************
!     reads DVR and oper
!***********************************************************************

      implicit none

      integer(long) :: chkdvr,chkgrd,typ,i,h,k,f,zcap,zortf,strlen
      integer(long) :: idum(1)
      real(dop)     :: rdum(1,1)
      complex(dop)  :: zdum(1)
      logical   normal
      parameter (strlen=240)
      character(len=strlen) :: string
      character(len=5) :: caplab

!-----------------------------------------------------------------------
! Read DVR data needed  (see rddvr.f for details)
!-----------------------------------------------------------------------
      dvrdata(:) = .false.
      dvrdata(1) = .true.    ! read ort
      dvrdata(2) = .false.   ! read trafo
      dvrdata(3) = .false.   ! read dvrmat (dif1mat, dif2mat)
      dvrdata(4) = .false.   ! read fftp   (fft momentum)
      dvrdata(5) = .false.   ! read hin    (fft)
      dvrdata(6) = .false.   ! read rueck  (fft)
      dvrdata(7) = .false.   ! read fftfak (fft factors)
      dvrdata(8) = .false.   ! read jsph
      dvrdata(9) = .false.   ! read msph
      dvrdata(10)= .false.   ! read kinsph
      dvrdata(11)= .false.   ! read exphin   (PLeg, exp-DVR)
      dvrdata(12)= .false.   ! read exprueck (PLeg, exp-DVR)
      chkdvr=1
      if(loper .or. lproj .or. lderiv) then
         dvrdata(2) = .true.    ! read trafo
         dvrdata(5) = .true.    ! read hin   (fft)
         dvrdata(6) = .true.    ! read rueck (fft)
         dvrdata(7) = .true.    ! read fftfak
         dvrdata(8) = .true.    ! read jsph
         dvrdata(9) = .true.    ! read msph
         dvrdata(11)= .true.    ! read exphin
         dvrdata(12)= .true.    ! read exprueck
!   rddvr(ort,trafo,dvrmat,fftp,hin,rueck,fftfak,exphin,
!         exprueck,jsph,msph,kinsph,chkdvr)
         if (lderiv) then
            dvrdata(4) = .true. ! read fftp  (fft momentum)
            call rddvr(mr(mrort),mr(mrtrafo),rdum,mr(mrfftp),mc(mchin), &
               mc(mcrueck),mi(mifftfak),mc(mcexphin),mc(mcexprueck), &
               mi(mijsph),mi(mimsph),rdum,chkdvr)
         else
            call rddvr(mr(mrort),mr(mrtrafo),rdum,rdum,mc(mchin), &
               mc(mcrueck),mi(mifftfak),mc(mcexphin),mc(mcexprueck), &
               mi(mijsph),mi(mimsph),rdum,chkdvr)
         endif
      else
         call rddvr(mr(mrort),rdum,rdum,rdum,zdum,zdum, &
              idum,zdum,zdum,mi(mijsph),mi(mimsph),rdum,chkdvr)
      endif

!---- Determine caplab
      if(captype.eq.1) then
         caplab='left'
      elseif(captype.eq.2) then
         caplab='right'
      elseif(captype.eq.3) then
         caplab='both'
      else
         write(6,'(a)') 'Unknown captype:', captype
         stop
      endif

!-----------------------------------------------------------------------
! Read data needed by the operator
!-----------------------------------------------------------------------
      zcap=0
      zortf=0
      chkdvr=2
      chkgrd=1
      normal=.false.
      if(hcap.eq.0) normal=.true.
      call rdoper(mr(mrhops),chkdvr,chkgrd)

      do 100 k=1,khtot(1)
         if( ki(k) .ne. kf(k) ) goto 100
         if( ki(k).lt.nst1 .or. ki(k).gt.nst2 ) goto 100
         if(normal) then      ! Normal flow, search for CAP.
            f = abs(autocap(k))
            if(f.eq.0) goto 100
            h = hamilton(f,k)
            typ=htmtype(h)
            if (typ .ne. 3) goto 100
         else                   ! h-cap is given directly.
            f = htmdof(hcap)
            h = hamilton(f,k)
            if( h .ne. hcap ) goto 100
            fcap = f
            if(fcap .eq. 0) goto 100
            typ  = htmtype(h)
            if (typ .ne. 3) goto 100
            mcap = dofspf(fcap)
         endif
         if (ltest) then
!........Test run
            zcap  = mrhops+zhop(h)-1
            zortf = mrort + zort(f) - 1
            fcap  = f
            mcap  = dofspf(fcap)
            sdmc  = subdim(mcap)
            if(nspfdof(mcap) .ne. 1) then
               write(6,'(a,/a,i3)') ' The CAP is in a combined mode!',&
                    ' nspfdof(mcap) =', nspfdof(mcap)
            end if
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(a,i3,a,i3,4x,a,a,i1)') ' CAP FOUND!  mode =', &
               mcap, '   dof =', fcap,trim(modelabel(fcap)),'captype=',captype
            write(6,'(a,i5,a,i5,a,i3,a,i7)') ' k =',k,' , h =', &
                 h,' , state =',ki(k),' , zhop =', zhop(h)
            call buffterm(h,string,strlen,i)
            write(ilog,'(2a)') ' hoplab: ', string(1:i)
            if(k.le.kzahl(1)) write(6,'(a)') &
                 ' ### WARNING : CAP seems to be correlated! ###'
            write(6,'(a,f6.2,a,f11.6,a,f6.2,a,f11.6,a)') &
                 ' W^gam(',mr(zortf),') =', -mr(zcap)*ev, &
                 ' eV;     W^gam(',mr(zortf+gdim(fcap)-1), &
                 ') =', -mr(zcap+gdim(fcap)-1)*ev,' eV.'
            write(6,'(78a1)')  ('-',i=1,78)
         else
!........Regular run
            if(zhop(h).gt.0 .and. mcap*(fcap-f).eq.0) then
               zcap  = mrhops+zhop(h)-1
               fcap  = f
               hcap  = h
               if(mcap .eq. 0 ) mcap = dofspf(fcap)
               sdmc = subdim(mcap)
!              ignore the CAP if it is not right or left as required
               if( captype.eq.1 .and. -mr(zcap).lt.1.d-9 ) goto 100
               if( captype.eq.2 .and. -mr(zcap+gdim(fcap)-1) .lt.1.d-9 ) goto 100
               zortf = mrort + zort(f) - 1
               write(6,'(a,i5,a,i3,a,i3,4x,a,a,i1)') &
                ' CAP FOUND!  h =', h,'   mode =', mcap, '   dof =', &
                fcap, trim(modelabel(fcap)),' captype=',captype
               write(6,'(a,f6.2,a,f11.6,a,f6.2,a,f11.6,a)') &
                ' W^gam(',mr(zortf),') =', -mr(zcap)*ev, &
                ' eV;     W^gam(',mr(zortf+gdim(fcap)-1), &
                ') =', -mr(zcap+gdim(fcap)-1)*ev,' eV.'
               write(6,'(78a1)')  ('-',i=1,78)
               write(ilog,'(a,i3,a,i3,4x,a)') ' CAP:  mode =', &
                 mcap, '   dof =', fcap, trim(modelabel(fcap))
               write(ilog,'(a,i5,a,i5,a,i3,a,i7)') ' k =',k, &
                   ' , h =',h,' , state =',ki(k),' , zhop =',zhop(h)
               call buffterm(h,string,strlen,i)
               write(ilog,'(2a)') ' hoplab: ', string(1:i)
               if(k.le.kzahl(1)) write(6,'(a)') &
                    ' ### WARNING : CAP seems to be correlated! ###'
               write(ilog,'(a,f6.2,a,f11.6,a,f6.2,a,f11.6,a)') &
                 ' W^gam(',mr(zortf),') =', -mr(zcap)*ev, &
                 ' eV;     W^gam(',mr(zortf+gdim(fcap)-1), &
                 ') =', -mr(zcap+gdim(fcap)-1)*ev,' eV.'
               goto 101
            endif
         endif
 100  continue

      if( zortf .eq. 0 ) then
         write(6,'(/a/)') ' No CAP found!  Stop.'
         if(zcap.gt.0) write(6,'(a,i5,a,i3,a,i3,4x,a,a,i1,/a/a/)') &
         ' h =', h,'   mode =', mcap, '   dof =', &
           fcap, trim(modelabel(fcap)),' captype=',captype, &
         ' Probably, this is a CAP of wrong captype (right/left)!', &
         ' Run "flux85 -t" to check for CAPs.'
         stop
      end if

 101  if (ltest) stop          !  Stop here if  Test Run

      return
      end subroutine operin

!***********************************************************************
!     calculates the matrix elements <fullpsi(t)|W|fullpsi(t')>
!     where fullpsi is the fully reduced 1D wavefunction.
!     The wavefunction fullpsi is kept in memory for all out-times.
!     Operators are not allowed when fully projecting.
!***********************************************************************

      subroutine fullflx(projector,wgam,theta,ort,wmat,gtau)

      implicit none

      integer(long) :: k1,k2,kk1,kk2,kk,s,slen,tm1
      complex(dop)  :: workc(workcdim)
      complex(dop)  :: wmat(laewmat),gtau(2*nout2)
      complex(dop)  :: projector(prjdim),ovl,ovlth
      real(dop)     :: wgam(sdmc),theta(sdmc),ort(sdmc),time
      real(dop)     ::fcputime,fsystime,tcpu,tsys,norm2,rwtt,rwtt1,irwtt
      external   fcputime,fsystime

      complex(sip), allocatable  ::  spsi(:)
      complex(dop), allocatable  ::  psi1(:),psi2(:),fullpsi(:),dover(:)

      allocate(psi1(dgldim),psi2(dgldim),dover(doverdim))
      allocate(fullpsi(fulldim),spsi(spsidim))

!---------------------------------------------------------------------*
! Definitions:                                                        *
! mcap = mode of the CAP; fcap = degree of freedom of the CAP.        *
! sdmc = subdim(mcap)                                                 *
! wgam = W^gam = CAP.                                                 *
! wtt  = W_tt = <psi(t)|W^gam|psi(t)>                                 *
! IWtt = 2*Int <psi(t)|W^gam|psi(t)> dt + <psi(t)|Theta|psi(t)>       *
! wmat = W_tt'=<psi(t)|W^gam|psi(t')>; t=(k1-1)*out2, t'=(k2-1)*out2  *
! wmat is in symmetric storage mode.                                  *
!---------------------------------------------------------------------*

!-----------------------------------------------------------------------
! Get some additional arguments
!-----------------------------------------------------------------------
      time  = 0.d0
      tovl  = 0.d0
      rwtt1 = 0.d0
      irwtt = 0.d0
      ovlth = 0.d0
      kk    = 0

      call getstart(wgam)  ! Detemine start1,start2,start

!------------------------------------------------------------------------
!     Build Theta. start1 is the starting point of a left-hand side
!     cap, start2 the one of a right-hand side cap.
!------------------------------------------------------------------------
      do k1 = 1, start1
            theta(k1) = 1.d0
      enddo
      do k1 = start1+1, start2-1
            theta(k1) = 0.d0
      enddo
      do k1 = start2, gdim(fcap)
            theta(k1) = 1.d0
      enddo
      if(start .le. gdim(fcap) ) then
         ostart = ort(start)
         if(.not.lwtt) write(iflx,'(a,f11.4)') '# ort(start) =',ostart
         write(iIwtt,'(a,f11.4)') '# ort(start) =',ostart
      else
         ostart = 0.d0
         if(.not.lwtt) write(iflx,'(a)') '# ort = No g_theta evaluation'
         write(iIwtt,'(a)') '# ort = No g_theta evaluation'
      endif

!-----------------------------------------------------------------------
! Write w^gam to log file.
!-----------------------------------------------------------------------
      write(ilog,'(/a,6x,a)') '           ort        w^gam[eV] ', &
                              '           ort        w^gam[eV]'
      do k1 = 1, (gdim(fcap)+1)/2
         k2 = k1 + gdim(fcap)/2
         write(ilog,'(i5,f12.4,f15.8,6x,i5,f12.4,f15.8)') &
              k1, ort(k1), -wgam(k1)*ev, k2, ort(k2), -wgam(k2)*ev
      enddo
      write(ilog,'(a)')
      call unixzeit(tm1)
      call myflush(ilog)

!-----------------------------------------------------------------------
! Write head lines to Wtt, IWtt and flux file.
!-----------------------------------------------------------------------
      if(laeout.gt.0) then
         write(iwtt,'(/a)') '#   klo    khi   kmax   step  start'
         write(iwtt,'(a2,i5,4i7)') '# ',klo,khi,kmax,step,start
         write(iwtt,'(/a,12x,a,16x,a,13x,a)') '#  Time[fs]', &
             'W_tt','norm**2','W_tt/norm**2'
         write(iIwtt,'(/a)') '#   klo    khi   kmax   step  start'
         write(iIwtt,'(a2,i5,4i7)') '# ',klo,khi,kmax,step,start
         if(loper) then
            write(iIwtt,'(2a)') &
                 '# operator = ',operator(1:slen(operator))
         endif
         write(iIwtt,'(/a,12x,a,16x,a,10x,a)') '#  Time[fs]', &
             'IW_tt','Theta','IW_tt + Theta'
         if(.not.lwtt) then
            write(iflx,'(/a)') '#   klo  khi step start'
            write(iflx,'(a2,4i5)') '#%',klo,khi,step,start
            call myflush(iflx)
         endif
      endif

!-----------------------------------------------------------------------
! Call reduce to compute the reduced wavefunction.
! States will be reordered by reduce and then: nstate=nst2-nst1+1
!-----------------------------------------------------------------------
      if(lproj) then
         call reduce(projector,psi2,psi1,spsi,mi(mijindx),dover,workc,fullpsi)
      endif

!-----------------------------------------------------------------------
! Step over evaluation of g_w when -v option is set.
!-----------------------------------------------------------------------
      if(lgtt) goto 200

!-----------------------------------------------------------------------
! Start the loops over time-steps.
!-----------------------------------------------------------------------
      do k1=1,khi
         if(mod(k1-klo,25).eq.0 ) call tout(k1,klo,time,tm1)
         time = (k1-1)*out2
         kk1  = (k1-1)*sdmc*nstate + 1
         kk2  = 1

!------------------------------------------------------------------------
!     Multiply FullPsi(t) by W^gam and store into Psi1
!------------------------------------------------------------------------
         do s=1,nstate
            if(nspfdof(mcap) .eq. 1 ) then ! un-combined
               call dvxxdz(wgam,fullpsi(kk1),psi1(kk2),sdmc)
            else                           !  combined
               call dtxxdz(wgam,fullpsi(kk1),psi1(kk2),vgdim(fcap), &
                    gdim(fcap),ngdim(fcap))
            endif
            kk1 = kk1 + sdmc
            kk2 = kk2 + sdmc
         enddo

!------------------------------------------------------------------------
!   If (lwtt) evaluate Theta term and jump over time off-diagonal terms
!------------------------------------------------------------------------
         if(lwtt) then
            kk1  = (k1-1)*sdmc*nstate + 1
            call vvaxzz (fullpsi(kk1),psi1,ovl,sdmc*nstate)
            rwtt = dble(-ovl)
            call vvaxzz (fullpsi(kk1),fullpsi(kk1),ovl,sdmc*nstate)
            norm2 = dble(ovl)
            call fullgth0 (fullpsi(kk1),psi1,theta,ovlth)
         else
            do k2=1,k1
               kk  = kk + 1
               kk2 = (k2-1)*sdmc*nstate + 1
               call vvaxzz (fullpsi(kk2),psi1,ovl,sdmc*nstate)
               wmat(kk) = -ovl
            end do                 ! k2-loop  (psi)
            rwtt = dble(-ovl)
            call vvaxzz (fullpsi(kk2),fullpsi(kk2),ovl,sdmc*nstate)
            norm2 = dble(ovl)
            call fullgth0 (fullpsi(kk2),psi1,theta,ovlth)
         endif

         if(laeout.gt.0) then
            write(iwtt,'(f11.3,3e22.12)') time/fs,rwtt,norm2,rwtt/norm2
            irwtt=irwtt+(rwtt+rwtt1)*out2
            rwtt1=rwtt
            write(iIwtt,'(f11.3,3e22.12)') &
              time/fs,irwtt,dble(ovlth),irwtt+dble(ovlth)
         endif
      end do                    ! k1-loop   (psi1)

      call tout(k1,klo,time,tm1)

!-----------------------------------------------------------------------
! Get the residual g_theta
!-----------------------------------------------------------------------
 200  call fullgtheta(fullpsi,theta,gtau)
      call tout(zeroi,klo,time,tm1)

!-----------------------------------------------------------------------
! Write start and ostart=ort(start) to screen and log-file.
!-----------------------------------------------------------------------
      if(captype .eq. 3 ) then
         write(6,'(a,2i4,a,f11.4)') &
              ' start =',start1,start2,',  ort =', ostart
      else
         write(6,'(a,i4,a,f11.4)')  ' start =',start,',  ort =', ostart
      endif
      write(ilog,'(/a,2i4,a,i4,a,f11.4)') &
       ' start1,2 =',start1,start2,',  start =',start,',  ort =',ostart


!-----------------------------------------------------------------------
! Get timing. Will be written in main program.
!-----------------------------------------------------------------------
      tcpu = fcputime(0)
      tsys = fsystime(0)
      tovl = tcpu+tsys

      deallocate(spsi,psi1,psi2,dover,fullpsi)

      return

      end subroutine fullflx



!************************************************************************
      subroutine fdrvflx(projector,wgam,ort,hin,rueck, &
                         fftfak,fftp,xmat,gtau)
!************************************************************************
!
! For the fully projected 1D wavefunction fullpsi, computes
! X_tt' = -i/4m [ fullpsi(R0,t)^* fullpsi'(R0,t') - fullpsi'(R0,t)^* fullpsi(R0,t') ]
! in order to compute the flux from the spatial derivative of the WF.
!
! fullpsi is computed from psi and projector.
! The evaluation point R0 is determined by grid point start.
! m is the reduced mass of the collisional/dissociative coordinate.
! xmat contains the output (similar to wmat from fullflx).
! gtau will be set to zero.
!
!************************************************************************

      implicit none

      integer(long) :: k1,k2,kk1,kk2,kk,s,slen,g
      complex(dop)  :: workc(workcdim)
      complex(dop)  :: xmat(laewmat),gtau(2*nout2)
      complex(dop)  :: projector(prjdim),ovl
      complex(dop)  :: hin(fftdim),rueck(fftdim)
      integer(long) ::  fftfak(fftdim)    
      real(dop)     ::  fftp(fftdim)      
      real(dop)     ::  ort(sdmc),wgam(sdmc)
      real(dop)     ::  tcpu,tsys,dx
      real(dop), external :: fcputime,fsystime

      complex*8, allocatable  ::  spsi(:)
      complex*16,allocatable  ::  psi1(:),psi2(:),dover(:),fullpsi(:)

      allocate(psi1(dgldim),psi2(dgldim),dover(doverdim))
      allocate(fullpsi(fulldim),spsi(spsidim))

!---------------------------------------------------------------------*
! Definitions:                                                        *
! mcap = mode of the CAP; fcap = degree of freedom of the CAP.        *
! sdmc = subdim(mcap)                                                 *
! wgam = W^gam = CAP.                                                 *
! t=(k1-1)*out2, t'=(k2-1)*out2                                       *
! xmat = X_tt' (see above). xmat is in symmetric storage mode.        *
!---------------------------------------------------------------------*

! ----------------------------------------------------------------------
!     Some sanity checks.
! ----------------------------------------------------------------------
      routine = 'fdrvflx'
      if (.not.full) then
         message = 'Derivative Flux only implemented for fully '// &
                   'projected wavefunctions.'
         call errormsg
      endif
      if (nspfdof(mcap).ne.1) then
         message = 'Derivative Flux not implemented for combined '// &
                   'dissociation DOF.'
         call errormsg
      endif
      if (basis(fcap).ne.4) then
         message = 'Derivative Flux only implemented for FFT grid.'
         call errormsg
      endif
      if (lrunml) then
         message='Derivative Flux not implemented for ML-wavefunctions.'
         call errormsg
      endif

! ----------------------------------------------------------------------
!     Determine evaluation point:
!     If start is set, use it; otherwise find the start of the CAP and
!     go two grid points in.
! ----------------------------------------------------------------------
      if (start.le.0) then
         call getstart(wgam)
         if (captype.eq.1) then ! left CAP
            start = start+2
         else ! right CAP, or both
            start = start-2
         endif
      endif
      if (start.le.0 .or. start.gt.gdim(fcap)) then
         routine = 'fdrvflx'
         message = 'Invalid evaluation grid point (start)'
         call errormsg
      endif
      ostart = ort(start)

! ----------------------------------------------------------------------
!     Write head lines to Wtt, IWtt and flux file.
! ----------------------------------------------------------------------
      if(laeout.gt.0) then
         write(iwtt,'(/a)') '#   klo    khi   kmax   step  start'
         write(iwtt,'(a2,i5,4i7)') '# ',klo,khi,kmax,step,start
         write(iwtt,'(/a,12x,a,16x,a,13x,a)') '#  Time[fs]', &
             'W_tt','norm**2','W_tt/norm**2'

         write(iIwtt,'(a,f11.4)') '# ort(start) =',ostart
         write(iIwtt,'(/a)') '#   klo    khi   kmax   step  start'
         write(iIwtt,'(a2,i5,4i7)') '# ',klo,khi,kmax,step,start
         if(loper) then
            write(iIwtt,'(2a)') &
                 '# operator = ',operator(1:slen(operator))
         endif
         write(iIwtt,'(/a,12x,a,16x,a,10x,a)') '#  Time[fs]', &
             'IW_tt','Theta','IW_tt + Theta'

         if(.not.lwtt) then
            write(iflx,'(a,f11.4)') '# ort(start) =',ostart
            write(iflx,'(/a)') '#   klo  khi step start'
            write(iflx,'(a2,4i5)') '#%',klo,khi,step,start
            call myflush(iflx)
         endif
      endif

! ----------------------------------------------------------------------
!     Write w^gam to log file.
! ----------------------------------------------------------------------
      write(ilog,'(/a,6x,a)') '           ort        w^gam[eV] ', &
                              '           ort        w^gam[eV] '
      do k1 = 1, (gdim(fcap)+1)/2
         k2 = k1 + gdim(fcap)/2
         write(ilog,'(i5,f12.4,f15.8,6x,i5,f12.4,f15.8)') &
              k1, ort(k1), -wgam(k1)*ev, k2, ort(k2), -wgam(k2)*ev
      enddo
      write(ilog,'(a)')
      call myflush(ilog)

! ----------------------------------------------------------------------
!     Call reduce to compute the reduced wavefunction.
! ----------------------------------------------------------------------
      if(lproj) then
         call reduce(projector,psi2,psi1,spsi,mi(mijindx),dover,workc,fullpsi)
      endif

! ----------------------------------------------------------------------
!     Calculate spatial derivative at evaluation point, for all times.
! ----------------------------------------------------------------------
      do k1=1,khi ! khi is actually kmax now
         kk1 = (k1-1)*sdmc*nstate + 1
         do s=1,nstate
            call cpvxz(fullpsi(kk1),psi1(kk1),sdmc)
            call fft(psi1(kk1),workc,rueck(zfft(fcap)), &
                 fftfak(zfft(fcap)),sdmc,1,1,.true.)
            do g=1,sdmc
               psi1(kk1+g-1) = psi1(kk1+g-1) * &
                  dcmplx(0.d0, fftp(zfft(fcap)+g-1)) / sdmc
            enddo
            call fft(psi1(kk1),workc,hin(zfft(fcap)), &
                 fftfak(zfft(fcap)),sdmc,1,1,.false.)
            kk1 = kk1 + sdmc
         enddo
      enddo

! ----------------------------------------------------------------------
!     Compute X_tt' matrix
! ----------------------------------------------------------------------
      kk=1
      do k1=1,khi ! t=(k1-1)*out2
         do k2=1,k1 ! t'=(k2-1)*out2
            kk1 = (k1-1)*sdmc*nstate
            kk2 = (k2-1)*sdmc*nstate
            ovl = (0.d0,0.d0)
            do s=1,nstate
               ovl = ovl + &
                   fullpsi(kk1+start)*dconjg(psi1(kk2+start)) - &
                   dconjg(fullpsi(kk2+start))*psi1(kk1+start)
               kk1 = kk1 + sdmc
               kk2 = kk2 + sdmc
            enddo
            xmat(kk) = ovl
            kk = kk+1
         enddo
      enddo
!     have to divide by the DVR weight, which is the same as the
!     grid spacing for FFT
      dx = dabs(xend(2,fcap)-xend(1,fcap))/dble(sdmc-1)
      call zscal(laewmat,dcmplx(0.d0,0.25d0/mass(fcap)/dx),xmat,1)

! ----------------------------------------------------------------------
!     Set gtau (here, this means g_theta(tau)) to zero, because in this
!     mode of flux evaluation, there is no theta part. The actual gtau
!     will be computed from X_tt in routine dblint.
! ----------------------------------------------------------------------
      gtau(:) = 0.0_dop

! ----------------------------------------------------------------------
!     Get timing. Will be written in main program.
! ----------------------------------------------------------------------
      tcpu = fcputime(0)
      tsys = fsystime(0)
      tovl = tcpu+tsys

      deallocate(spsi,psi1,psi2,dover,fullpsi)

      return
      end subroutine fdrvflx

      end module flx2ut

