      module flxgth

use decimal
use global 
use globmemmod
use aglobal 
use dvrdatmod 
use griddatmod 
use psidef 
use fluxmod 
use flxutils
use hpsimod 
use operdef 
use channels 
use compdat 
use memhopsmodmod 
use datenmod
use iofile, only: rdpsiinfo
use hphimod, only: hphi,hunphi,hunphi1ms
use phihphimod, only: phihphi
use calchamod, only: calcha
use rdfilesmod, only: rdpsi
use psiutil, only: spfovl

use wftree
use wftree1
use multilayer, only: get_moduletree
use mlanalysis, only: croco_tree,overlap_1    

use op1lib, only: cpvxz
use xvlib, only: vvaxzz
use mmlib, only: dmxxdz,mmaxzz
use mtlib, only: dtxxdz

      implicit none

      private
      public :: gtheta,gth0,mtrxelhun,mtrxelmnt,multpsi
      contains

!********wftree_set_pointers********************************************
!
!                         FLXGTH
!
!     Set of routines needed for flux.
!     Contents: gtheta, gth0, mtrxelmnt, mtrxelhun, multpsi
!
! HDM Sep 99, HDM Nov 00, moved to F90 to include ML HDM Oct 2016
! HDM Jan 2022, new subroutines added, operation on CAP mode possible
!
!***********************************************************************

    subroutine gtheta(psi2,psi1,time,spsi,dover,theta,ort,&
               gtau,hunpsi,hpsi,hteil,hops,dtpsi,mcap_phi,jindx)

!***********************************************************************
!   calculates the (residual) g_theta(tau) = <psi(T-tau)|Theta|psi(T)>
!   Theta denotes the Heaviside's step function, being one where the
!   flux is measured. Theta is switched on at grid-point "start".
!***********************************************************************

    implicit none

    integer(long)              ::  i,k1,kk,chk,modus
    integer(long), dimension(1)                     :: idum1,idum2
    integer(long), dimension(adim), intent(inout)   :: jindx
    real(dop)                  ::  time
    real(dop),intent(in)       ::  theta(gdmc),ort(ortdim),hops(hopsdim)
    complex(sip),intent(inout) ::  spsi(spsidim)
    complex(dop), dimension(1)                      :: zdum1,zdum2
    complex(dop)            ::  ovl,mcap_phi(capphidim)
    complex(dop)            ::  dover(doverdim)
    complex(dop)            ::  hunpsi(totphidim),hpsi(hpsidim)
    complex(dop)            ::  hteil(pmatdim),dtpsi(dgldim)
    complex(dop)            ::  psi1(dgldim),psi2(dgldim)
    complex(dop)            ::  gtau(2*nout2)
    type(wfnode_t),pointer :: flxtree

    if (lrunml) then
       call get_moduletree(flxtree)
    endif

    chk  = 0
    gtau(:) = 0.0_dop
    if(start.gt.gdim(fcap)) then
       write(ilog,'(a)')  'No g_theta(tau) evaluated.'
       return
    endif

!-----------------------------------------------------------------------
! Get psi1(T_final).
!-----------------------------------------------------------------------
    rewind (ipsi)
    read (ipsi)
    if (filever(ipsi) .lt. 11.0_dop) call rdpsiinfo(ipsi,chk,chk,chk,chk)

    do k1=1,khi-1
       call rdpsi(ipsi,psi1,spsi,jindx,zdum1,idum1,zdum2,idum2)
    enddo
    dgldim=dgldimold
    call rdpsi(ipsi,psi1,spsi,jindx,zdum1,idum1,zdum2,idum2)

!------------------------------------------------------------------------
! Expand psi1 to full MCTDH-length in case of identical particles.
!------------------------------------------------------------------------
    if(lid) call mexpand(psi1)

!-----------------------------------------------------------------------
! Read psi2(t).
!-----------------------------------------------------------------------
    kk = kmax + 1
    rewind (ipsi)
    read (ipsi)
    if (filever(ipsi) .lt. 11.0_dop) call rdpsiinfo(ipsi,chk,chk,chk,chk)

    do k1=1,klo-1
       call rdpsi(ipsi,psi2,spsi,jindx,zdum1,idum1,zdum2,idum2)
    enddo
    do k1=klo,khi,step
       kk = kk - 1
       dgldim=dgldimold
       call rdpsi(ipsi,psi2,spsi,jindx,zdum1,idum1,zdum2,idum2)
       if (lend) goto 77
       do i = 2, min(step,nout2-k1)
          call rdpsi(ipsi,psi2,spsi,jindx,zdum1,idum1,zdum2,idum2)
          if (lend) goto 77
       enddo

!------------------------------------------------------------------------
! Expand psi2 to full MCTDH-length in case of identical particles.
!------------------------------------------------------------------------
       if(lid) call mexpand(psi2)
       call save_CAPspf(psi2,mcap_phi,.true.)

!-----------------------------------------------------------------------
! Apply operator and do the overlap.  ovl = <theta*psi2|O|psi1>
!-----------------------------------------------------------------------
       if(loper .and. .not.lcorr) then
          call multpsi(psi2,theta)
          call mtrxelhun(psi2,psi1,ovl,time,hunpsi,dover,mcap_phi,&
               flxtree)
       elseif(loper .and. lcorr) then
          call multpsi(psi2,theta)
          if(lrunml) then
             ovl = croco_tree(nham,time,tree,psi2,psi1)

          else
             call mtrxelmnt(psi2,psi1,time,ovl,dtpsi,hteil,&
                            hpsi,hops,ort,jindx)
          endif
       else ! no operators
          call multpsi(psi2,theta)
          if (lrunml) then
             call spfovl(dover,psi2,psi1,zdum1,zdum2,modus)
             call overlap_1(psi2,psi1,flxtree,ovl,dover)
          else
             call ovlp(ovl,psi2,psi1)
          endif
       endif

       gtau(kk+nout2) = 0.5d0*ovl

       if(lopsym) then
          call save_CAPspf(psi2,mcap_phi,.false.)
          if(loper .and. .not.lcorr) then
             call multpsi(psi2,theta)
             call mtrxelhun(psi1,psi2,ovl,time,hunpsi,dover,mcap_phi,flxtree)
          elseif(loper .and. lcorr) then
             call multpsi(psi1,theta)
             if(lrunml) then
                ovl = croco_tree(nham,time,tree,psi1,psi2)
             else
                call mtrxelmnt(psi1,psi2,time,ovl,dtpsi,hteil,&
                               hpsi,hops,ort,jindx)
             endif ! runml
          endif    ! loper

          gtau(kk+nout2) = 0.5d0*(gtau(kk+nout2) + dconjg(0.5d0*ovl))

       endif       ! lopsym

    enddo  ! k1=klo,khi,step

    return

 77 continue
    write(6   ,'(a)') 'gtheta: EOF found when reading the psi-file!'
    write(ilog,'(a)') 'gtheta: EOF found when reading the psi-file!'
    stop

    end subroutine gtheta

!***********************************************************************

    subroutine gth0(psi2,psi1,theta,wgam,dover,time,hunpsi,&
                    hpsi,hteil,hops,dtpsi,spsi,ort,mcap_phi,jindx)

!***********************************************************************
!   Calculates ovlth = <psi(t)|Theta|psi(t)> for wtt and iwtt file.
!   Theta denotes the Heaviside's step function, being one where the
!   flux is measured. Theta is switched on at grid-point "start".
!***********************************************************************

    implicit none

    integer(long)   ::  i,k1,chk,modus
    integer(long), dimension(1)                     :: idum1,idum2
    integer(long), dimension(adim), intent(inout)   :: jindx
    complex(dop)    ::  psi1(dgldim),psi2(dgldim)
    complex(dop)    ::  dover(doverdim),mcap_phi(capphidim)
    complex(dop)    ::  hunpsi(totphidim),hpsi(hpsidim)
    complex(dop)    ::  hteil(pmatdim),dtpsi(dgldim)
    complex(dop)    ::  ovl,ovlth,ovlo,ovln
    complex(dop), dimension(1)                      :: zdum1,zdum2
    real(dop)       ::  rwtt,irwtt,rwtt1,rovlo,rovlth,norm2,time
    complex(sip)    ::  spsi(spsidim)
    real(dop),intent(in) ::  hops(hopsdim)
    real(dop),intent(in) :: ort(ortdim),theta(gdmc),wgam(gdmc)
    type(wfnode_t),pointer :: flxtree

    if (lrunml) then
       call get_moduletree(flxtree)
    endif

    irwtt = 0.d0
    rwtt1 = 0.d0
    chk   = 0

!-----------------------------------------------------------------------
! Start the loops over time-steps.
!-----------------------------------------------------------------------
    rewind (ipsi)
    read (ipsi)
    call rdpsiinfo(ipsi,chk,chk,chk,chk)
    time=0.d0

    do k1=1,klo-1
       read (ipsi)
    enddo

!------------------------------------------------------------------------
! Read psi1(t=t_k1)
!------------------------------------------------------------------------
    do k1=klo,khi,step
       time = (k1-1)*out2
       dgldim=dgldimold
       call rdpsi(ipsi,psi1,spsi,jindx,zdum1,idum1,zdum2,idum2)
       if (lend) goto 77
       do i = 2, min(step,nout2-k1)
          read(ipsi,end=77)
       enddo

!------------------------------------------------------------------------
! Expand psi1 to full MCTDH-length in case of identical particles.
! Copy of psi1 to psi2, save the SPFs of the CAP mode.
!------------------------------------------------------------------------
       if(lid) call mexpand(psi1)
       call cpvxz (psi1,psi2,dgldim)
       call save_CAPspf(psi2,mcap_phi,.true.)

!------------------------------------------------------------------------
!      Compute ovln = <psi1|psi1>,  norm2=dble(ovln)
!------------------------------------------------------------------------
       if(lrunml) then
          call spfovl(dover,psi2,psi1,zdum2,zdum1,modus)
          call overlap_1(psi2,psi1,flxtree,ovln,dover)
       else
          call ovlp(ovln,psi2,psi1)
       endif
       norm2 = dble(ovln)

!------------------------------------------------------------------------
!      Initially psi and psi1 are identical!
!      compute ovlo = <psi2|O|psi1>
!------------------------------------------------------------------------
       if(loper .and. .not.lcorr) then
          call mtrxelhun(psi2,psi1,ovlo,time,hunpsi,dover,mcap_phi,&
               flxtree)
       elseif(loper .and. lcorr) then
          if(lrunml) then
             ovlo = croco_tree(nham,time,tree,psi2,psi1)
          else
             call mtrxelmnt(psi2,psi1,time,ovlo,dtpsi,hteil,&
                            hpsi,hops,ort,jindx)
          endif
       else ! no operators
          ovlo=0.d0
       endif
       rovlo=dble(ovlo)*ufac ! transform to unit given by -u

!------------------------------------------------------------------------
!      Compute ovl = <wgam*psi2|O|psi1>
!------------------------------------------------------------------------
       call save_CAPspf(psi2,mcap_phi,.false.)
       if(loper .and. .not.lcorr) then
          call multpsi(psi2,wgam)
          call mtrxelhun(psi2,psi1,ovl,time,hunpsi,dover,mcap_phi,&
               flxtree)
       elseif(loper .and. lcorr) then
          call multpsi(psi2,wgam)
          if(lrunml) then
             ovl = croco_tree(nham,time,tree,psi2,psi1)
          else
             call mtrxelmnt(psi2,psi1,time,ovl,dtpsi,hteil,&
                            hpsi,hops,ort,jindx)
          endif
       else ! no operator
          call multpsi(psi2,wgam)
          if(lrunml) then
             call spfovl(dover,psi2,psi1,zdum2,zdum1,modus)
             call overlap_1(psi2,psi1,flxtree,ovl,dover)
          else
             call ovlp(ovl,psi2,psi1)
          endif
       endif
       rwtt = dble(-ovl)*ufac

!------------------------------------------------------------------------
!      Compute ovlth = <theta*psi2|O|psi1>
!------------------------------------------------------------------------
       call save_CAPspf(psi2,mcap_phi,.false.)
       if(loper .and. .not.lcorr) then
          call multpsi(psi2,theta)
          call mtrxelhun(psi2,psi1,ovlth,time,hunpsi,dover,mcap_phi,&
               flxtree)
       elseif(loper .and. lcorr) then
          call multpsi(psi2,theta)
          if(lrunml) then
             ovlth = croco_tree(nham,time,tree,psi2,psi1)
          else
             call mtrxelmnt(psi2,psi1,time,ovlth,dtpsi,hteil,&
                            hpsi,hops,ort,jindx)
          endif
       else ! no operators
          call multpsi(psi2,theta)
          if(lrunml) then
             call spfovl(dover,psi2,psi1,zdum2,zdum1,modus)
             call overlap_1(psi2,psi1,flxtree,ovlth,dover)
          else
             call ovlp(ovlth,psi2,psi1)
          endif
       endif
       rovlth=dble(ovlth)*ufac

       if(laeout.gt.0) then
          irwtt=irwtt+(rwtt+rwtt1)*out2*step
          rwtt1=rwtt
          write(iIwtt,'(f11.3,3e22.12)')                   &
                time/fs,irwtt,rovlth,irwtt+rovlth
          if(loper) then
             write(iwtt,'(f11.3,1p5e18.9)')                &
                time/fs,rwtt,norm2,rwtt/max(norm2,1.d-20), &
                rovlo,rovlo/max(norm2,1.d-20)
          else
             write(iwtt,'(f11.3,1p3e18.9)')                &
                time/fs,rwtt,norm2,rwtt/max(norm2,1.d-20)
          endif
       endif

    enddo  ! k1

    return

 77 continue
    write(6   ,'(a)') ' gth0: EOF found when reading the psi-file!'
    write(ilog,'(a)') ' gth0: EOF found when reading the psi-file!'
    stop

    end subroutine gth0
!***************************************************************************

!   mtrxelmnt computes ovl = <psi|O|psi1> for correlated operator (nodiag)

    subroutine mtrxelmnt(psi2,psi1,time,ovl,dtpsi,hteil,hpsi,&
                         hops,ort,jindx)

    implicit none

    integer(long) ::  s,ktype
    integer(long), dimension(adim), intent(in)      :: jindx
    complex(dop)  ::  psi1(dgldim),psi2(dgldim)
    complex(dop)  ::  hpsi(hpsidim)
    complex(dop)  ::  hteil(pmatdim),dtpsi(dgldim)
    complex(dop)  ::  ovl,sumov
    real(dop),intent(in) :: hops(hopsdim),ort(ortdim),time

    ktype=0
    dtpsi(:) = 0.0_dop
    call hphi(psi1,hpsi,ktype,nham)
    call phihphi(psi2,gh,hpsi,hteil,nham,.false.)
    call calcha(psi1,dtpsi,hteil,hops,psi1(zetf(1,1)), &
                nham,psi2(zetf(1,1)),jindx,ort)

    ovl=(0.0d0,0.d0)
    do s=nst1,nst2
       call vvaxzz(psi2(zpsi(s)),dtpsi(zpsi(s)),sumov,block(s))
       ovl=ovl+sumov
    enddo

    return
    end subroutine mtrxelmnt

!#######################################################################
!
!   mtrxelhun computes ovl = <psi2|O|psi1> for uncorrelated operator 
!   terms (usediag)
!
!#######################################################################

    subroutine mtrxelhun(psi2,psi1,ovl,time,hunpsi,dover,mcap_phi,&
               flxtree)

    implicit none

    integer(long)  ::  s,hktype,m,m1,i,modus
    real(dop)      ::  aa,bb
    complex(dop)   ::  sumov
    complex(dop), dimension(1)   ::  zdum1,zdum2
    complex(dop)   ::  mcap_phi(capphidim)
    complex(dop)   ::  dover(doverdim),hunpsi(totphidim)
    complex(dop)   ::  psi2(dgldim),psi1(dgldim)
    complex(dop)   ::  psi01(dgldim) ! tmp array
    real(dop),    intent(in)  ::  time
    complex(dop),intent(out) ::  ovl
    type(wfnode_t),pointer :: flxtree

    ovl=(0.0d0,0.d0)
    hktype=0

!!  outside mtrxelhun: call multpsi(psi2,wgam)

    do m = 1, nmode
       if(.not.mfound(m)) cycle
       do s = nst1, nst2
          if(khzahl(m,s,nham) .gt. 0 ) then ! Multiply psi1 with uncorr. operator
             call cpvxz(psi1(zetf(m,s)),psi01(zetf(m,s)),dim(m,s)*subdim(m)) ! save SPF(m,s)
             if(i0s.gt.0) then  ! This if-block is for p% projectors.
                aa=1.d0/dble(start-1-i0s)
                do i=0,i0s
                   psi1(zetf(m,s)+i)=0.d0
                enddo
                do i=i0s+1,start-2
                   bb=pi*(start-1-i)*aa  ! Like g5 filter
                   psi1(zetf(m,s)+i)=psi1(zetf(m,s)+i)*(((i-i0s)*aa)*cos(bb)+sin(bb)/pi)
                enddo
             endif
             call hunphi1ms(psi1(zetf(m,s)),hunpsi(zetf(m,s)-adim),&
                  m,s,dim(m,s),subdim(m),hktype,nham)
             call cpvxz (hunpsi(zetf(m,s)-adim), &
                         psi1(zetf(m,s)),dim(m,s)*subdim(m))
          endif ! khzahl
       enddo  ! s

       do s = nst1, nst2    ! restore SPF(m,s) from psi01 to psi1
          do m1 = 1,nmode
             call mmaxzz(psi2(zetf(m1,s)),psi1(zetf(m1,s)),&
                  dover(dmat(m1,s)),subdim(m1),dim(m1,s),dim(m1,s))
          enddo
       enddo
       if(lrunml) then
          call spfovl(dover,psi2,psi1,zdum2,zdum1,modus)
          call overlap_1(psi2,psi1,flxtree,sumov,dover)
       else
          call ovlp(sumov,psi2,psi1)
       endif
       ovl=ovl+sumov

       do s = nst1, nst2    ! restore SPF(m,s) from psi01 to psi1
          if(khzahl(m,s,nham) .gt. 0 ) then
             call cpvxz(psi01(zetf(m,s)),psi1(zetf(m,s)),dim(m,s)*subdim(m))
          endif
       enddo
    enddo  ! m

!    Not necessary, psi2 is allowed to change when leaving this subroutine.
!    call save_CAPspf(psi2,mcap_phi,.false.) ! restore SPF(mcap,s) of psi2

    return
    end subroutine mtrxelhun

!#######################################################################
!   multpsi computes thwgam|psi> where thwgam = theta or wgam

    subroutine multpsi(psi,thwgam)

    implicit none

    integer(long)   ::  s,e,pt
    complex(dop), intent(inout) ::  psi(dgldim)
    real(dop),intent(in)        ::  thwgam(gdmc)  ! gdmc = gdim(fcap)


    if(nspfdof(mcap) .eq. 1 ) then  ! un-combined
       do s = 1, nstate
          call dmxxdz(thwgam,psi(zetf(mcap,s)),psi(zetf(mcap,s)), &
                      gdmc,dim(mcap,s))
       enddo
    else                         ! combined CAP mode
       do s = 1, nstate
          do e = 1,dim(mcap,s)
             pt = (e-1)*sdmc + zetf(mcap,s)
             call dtxxdz(thwgam,psi(pt),psi(pt),vgdim(fcap), &
                  gdmc,ngdim(fcap))
          enddo
       enddo
    endif

    return
    end subroutine multpsi

!#######################################################################

    subroutine save_CAPspf(psi,mcap_phi,forward)

    implicit none

    logical(kind=4),intent(in)  ::  forward
    integer(long)               ::  pt, s
    complex(dop), intent(inout) ::  psi(dgldim),mcap_phi(capphidim)


    if(forward) then
       pt=1 ! save SPF(mcap,s) to mcap_phi
       do s = nst1, nst2
          call cpvxz(psi(zetf(mcap,s)),mcap_phi(pt),dim(mcap,s)*sdmc)
          pt=pt+dim(mcap,s)*sdmc
       enddo

    else
       pt=1 ! restore SPF(mcap,s) from mcap_phi
       do s = nst1, nst2
          call cpvxz(mcap_phi(pt),psi(zetf(mcap,s)),dim(mcap,s)*sdmc)
          pt=pt+dim(mcap,s)*sdmc
       enddo
    endif

    return
    end subroutine save_CAPspf

!#######################################################################

      end module flxgth
