!***********************************************************************
!
!                        FLXINPUT
!
!     This routine handles the input of flux.
!     Contents: flxinput, rdflxinpf, rdoptions
!
! HDM Sep 99, HDM Nov 00
!
!
!***********************************************************************

      module flxinputmod

use decimal, only:long,dop
use keyunitsmod
use logdat

use global
use aglobal
use versions
use fluxmod
use dvrdatmod
use griddatmod
use psidef
use channels
use datenmod
use basismod
use operdef

use iofile, only: rdpsiinfo
use rddvrmod, only: dvrinfo
use rdopermod, only: operinfo
use iopsidef, only: psidat
use rdfilesmod, only: rdpsindat

use module_interface

      implicit none
      private
      public :: flxinput,flxrdinfo
      contains

      subroutine flxinput(nthreads)

      implicit none

      integer(long) :: iarg, myiargc, ilbl, i, j, ierr
      integer(long) :: slen
      integer(long) :: dvrlaenge
      integer(long), intent(in) :: nthreads
      logical(kind=4)   :: lexist, lgtau, lmnd, lopsym_in
      character(len=c4) :: buf, cgtau
      character(len=c5) :: filename
      character(len=10) :: stat

!-----------------------------------------------------------------------

      name    = '.'
      dname   = '.'
      oname   = '.'
      laenge  = 1
      dlaenge = 1
      olaenge = 1
      nppar   = 0
      nprj    = 0
      start   = 0
      start1  = 0
      i0s     = 0
      captype = 2
      nst1    = 1
      nst2    = 0
      dipolem = 1.d0
      do i = 1,maxprj
         prjmode(i) = 0
         prjname(i) = ' '
         prjlab(i)  = ' '
         do j = 1,maxprj
            prjpar(i,j) = -999
         enddo
      enddo

      filein  = './'
      fileout = './'
      operfile= 'oper'
      operator= ' '
      edstrf  = 'enerd'
      dvrfile = 'dvr'
      capdof  = '  '
      unit    = 'au'
      unite   = 'au'
      unitf   = 'au'
      loper = .false.
      lcorr = .false.
      lproj = .false.
      ltest = .false.
      lwtt  = .false.
      lgtt  = .false.
      lgtau = .false.
      lread = .false.
      lderiv= .false.
      full  = .false.
      ladd  = .false.
      lc3d  = .false.
      ltonly= .false.
      lmb   = .false.
      lid   = .false.
      lmnd  = .false.
      lopsym  = .false.
      lopsym_in  = .false.
      l2short = .false.
      lsmooth = .false.
      lprojonly=.false.
      klo   = 1
      khi   = 0
      mcap  = 0
      fcap  = 0
      ufac  = 1.0d0
      ostart= 0.0d0
      emin  = -1.1d+18
      emax  = -1.1d+18
      epoints = 500
      iarg  = myiargc()
      if (iarg .le. 0) goto 200
      call mygetarg(0,options)
      optlaenge  = index(options,' ')
      do i = 1, iarg
         call mygetarg(i,buf)
         ilbl  = index(buf,' ')
         ilbl = min(ilbl,100-optlaenge)
         if(ilbl.gt.0) options = options(1:optlaenge)//buf(1:ilbl)
         optlaenge = optlaenge + ilbl
      end do

!........Look if there is an .inp file.
      call mygetarg(iarg,inputfile)
      inplaenge = index(inputfile,' ')-1
      if(inputfile(max(1,inplaenge-3):inplaenge) .ne. '.inp') then
         inputfile = inputfile(1:inplaenge)//'.inp'
         inplaenge = inplaenge + 4
      endif
      inquire(file=inputfile,exist=lexist)
      if(lexist) then
         iarg = iarg - 1
         call abspath(inputfile,inplaenge)
         call rdflxinpf(lmnd,lopsym_in)
      else
         inputfile = ' '
         inplaenge = 0
      endif

!........Read the options
 200  call rdoptions(iarg,lmnd,lopsym_in)
      laein=index(filein,' ')-1
      if (filein(laein:laein) .eq. '/') then
         filein = filein(1:laein)//'psi'
         laein = laein + 3
      end if
      call abspath(filein,laein)

      opflaenge=index(operfile,' ')-1
      if (operfile(opflaenge:opflaenge) .eq. '/') then
         operfile = operfile(1:opflaenge)//'oper'
         opflaenge = opflaenge + 4
      end if
      call abspath(operfile,opflaenge)

      dvrlaenge=index(dvrfile,' ')-1
      if (dvrfile(dvrlaenge:dvrlaenge) .eq. '/') then
         dvrfile = dvrfile(1:dvrlaenge)//'dvr'
         dvrlaenge = dvrlaenge + 3
      end if
      call abspath(dvrfile,dvrlaenge)
      dname=dvrfile(1:dvrlaenge)
      dlaenge=dvrlaenge

      edlaenge=index(edstrf,' ')-1
      if (edstrf(edlaenge:edlaenge) .eq. '/') then
         edstrf = edstrf(1:edlaenge)//'enerd'
         edlaenge = edlaenge + 5
      end if
      call abspath(edstrf,edlaenge)

!-----------------------------------------------------------------------
!  make the flux-directory if -om option or keyword make-outdir is set.
!-----------------------------------------------------------------------
      if(lmnd) then
         open(itmpin,file=trim(fileout)//'/scratch',err=666)
         close(itmpin,status='delete')
         write(6,'(/2a)') 'Flux-directory already exists: ', &
                                trim(fileout)
         lmnd=.false.  ! inhibits the deletion of the fileout file
         goto 500      ! if an error occurs

 666     inptit='mkdir '//trim(fileout)
         call excmd2(ierr, inptit)
         if (ierr.ne.0) then
            write(6,'(2a)') '### ERROR: ', &
            'Invalid path for flux-directory (-om option). ###'
            stop
         endif
         write(6,'(/2a)') 'Created flux-directory: ',trim(fileout)
      endif


 500  if (fileout(1:3) .eq. 'no ' .or. ltest .or. lprojonly) then
         laeout = 0
      else
         laeout=index(fileout,' ')-1
         call abspath(fileout,laeout)
         open(99,file=fileout(1:laeout)//'/test-&_%-tset',err=56)
         goto 57
 56      close(99,status='DELETE')
         write(6,'(/a,/a,/a,/a,/a)') &
              '######################################################', &
              ' ERROR: The output directory:', fileout(1:laeout), &
              ' does not exist (or is not writeable)!', &
              '######################################################'
         stop
 57      close(99,status='DELETE')
      end if

      write(6,'(/,80a)') ('*',i=1,80)
      if(inplaenge .gt. 0 ) &
        write(6,'(2a)') ' The input-file is : ',inputfile(1:inplaenge)

!-----------------------------------------------------------------------
!  Determine the name of gtau file.
!-----------------------------------------------------------------------
      cgtau='/gtau'
      if(loper) cgtau='/gtau_'//operator(1:slen(operator))
      if(lproj) cgtau=cgtau(1:slen(cgtau))//'.' &
              //prjname(1)(1:slen(prjname(1)))
      if(nprj.ge.2) cgtau=cgtau(1:slen(cgtau))//'.' &
              //prjname(2)(1:slen(prjname(2)))

!-----------------------------------------------------------------------
!  Open the Log-file.
!-----------------------------------------------------------------------
      if(laeout .eq. 0) then
         write(6,'(2a)') ' No outputfiles written '
         open (ilog,status='scratch',form='formatted')
      else
         filename=fileout(1:laeout)//'/flux.log'
         open (ilog,file=filename,form='formatted',err=1000)
         if( .not.lovwr .or. lwtt ) then
 54         read(ilog,'(a1)',end=55,err=55) buf
            goto 54
 55         backspace ilog
            write(ilog,'(//,80a1,/)') ('*',i=1,80)
         else
            endfile (ilog)
            backspace (ilog)
         end if
         logisopen=.true.
         write(ilog,'(/a)') trim(cstartup)
         call wrversion(ilog)
         if(nthreads.gt.0) write(ilog,'(a,i4)') &
             'OpenMP: no. of threads =',nthreads
      end if

!-----------------------------------------------------------------------
!  Look whether gtau file exist.
!-----------------------------------------------------------------------
      if((.not.lovwr .and. .not.ltest .and. .not.lwtt) .or. lgtt) then
         j=start
         filename=fileout(1:laeout)//cgtau
         inquire( file=filename, exist=lexist)
         if(lexist)  then
            lgtau = .true.
            open (unit=iaus,file=filename,form='formatted',err=1000)
            read(iaus,'(a1)',end=35,err=35)
            read(iaus,'(2x,i5,3i7,f15.5,i7,f15.5)',end=35,err=35) &
                 klo, khi, kmax, step, out2,start,ostart
            read(iaus,'(a1)',end=35,err=35)
            read(iaus,'(a1)',end=35,err=35)
            write(6,'(2a)') ' gtau-file exists! : ', &
                 filename(1:slen(filename))
            write(6,'(2a)') ' No overlaps will be calculated but', &
             ' gtau will be read from gtau file.'
            write(6,'(2a,/2a)')' Use the option -w to overwrite or', &
            ' use the option -o PATH',' to write to another directory', &
            ' if you want to re-do the overlaps.'
            write(6,'(a,/a,i3,a,i6,a,i5,a,i2,a,f8.3,a/a,i6,f13.5,a/a)') &
                 ' Read from gtau file: ', &
                 ' klo =',klo,',  khi =',khi,',  kmax =',kmax, &
                 ',  step =',step,',  out2 =',out2/fs,'fs', &
                 ' start =',start, ostart,'au', &
         '-------------------------------------------------------------'
            if(laeout .gt. 0)  write(ilog, &
                 '(a,/a,i3,a,i6,a,i5,a,i2,a,f8.3,a,/a,i6,f13.5,a,/a)') &
                 ' Read from gtau file: ', &
                 ' klo =',klo,',  khi =',khi,',  kmax =',kmax, &
                 ',  step =',step,',  out2 =',out2/fs,'fs', &
                 ' start =',start, ostart,'au', &
         '-------------------------------------------------------------'
            close(iaus)
            if(lgtt .and. j.gt.0) start=j
         else
            lovwr = .true.
            lgtt  = .false.
            lgtau = .false.
            if(lread) then
               write(6,'(/a,/a/)') &
              ' The gtau file does not exist! (Required by -r option).', &
              ' STOP'
               stop
            endif
         end if
      end if
      goto 36
 35   write(6,'(a)') ' Error occurred when reading gtau. Ignoring gtau.'
      lovwr = .true.
      lgtt  = .false.
      lgtau = .false.
      close(iaus,status='delete')

 36   lovwr = lovwr .and. .not.ltest
      kmax  = (khi-klo+step)/step

      if(ltest) write(6,'(2a)') & 
         ' Test run. Checking files, searching for CAP''s.'

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      filename=filein(1:laein)
      open(ipsi,file=filename,form='unformatted', &
                status='old',err=1001)
      write(6,'(2a)') ' The psi-file   is : ',filein(1:laein)
      filename=operfile(1:opflaenge)
      open(ioper,file=filename,form='unformatted', &
                status='old',err=1000)
      write(6,'(2a)') ' The oper-file  is : ',operfile(1:opflaenge)

      if(laeout .gt. 0) then
         if (lovwr) then
            stat='unknown'
         else 
            stat='new'
         endif

         write(6,'(2a)')' The output dir is : ',fileout(1:laeout)
         if(.not.lwtt) then
            filename=fileout(1:laeout)//cgtau
            open (unit=iaus,file=filename,status=stat, &
                            form='formatted',err=1000)
            filename=fileout(1:laeout)//'/flux'
            open (unit=iflx,file=filename,status=stat, &
                            form='formatted',err=1000)
         else
            filename=fileout(1:laeout)//'/wtt'
            open (unit=iwtt,file=filename,status=stat, &
                            form='formatted',err=1000)
            filename=fileout(1:laeout)//'/iwtt'
            open (unit=iIwtt,file=filename,status=stat, &
                            form='formatted',err=1000)
         endif
      end if

      filename=dvrfile
      open(idvr,file=dvrfile,form='unformatted',status='old',err=1000)
      write(6,'(2a)') ' The DVR-file   is : ',dvrfile(1:dlaenge)
      write(6,'(2a)') ' The edstr-file is : ',edstrf(1:edlaenge)

!-----------------------------------------------------------------------
!  Check for existence of 'enerd' file, open it and determine its length.
!-----------------------------------------------------------------------
      maxendim = 0
      inquire(file=edstrf,exist=lexist)
      if(lexist) then
         filename=edstrf
         open(ienerd,file=edstrf,form='formatted',status='old',err=1000)
 50      read (ienerd,*,end=51)
         maxendim = maxendim + 1
         goto 50
 51      rewind (ienerd)
      else
         edlaenge = 0
         write(6   ,'(a)') ' No energy-distribution file!'
         write(ilog,'(a)') ' No energy-distribution file!'
      end if


      return

 1000 write(6,'(a)') ' Cannot open file : '
      write(6,'(10x,a)')  trim(filename)
      if (.not. lovwr) write(6,'(a)') ' If file already exists, use -w'
      stop
 1001 write(6,'(a)') ' Cannot open psi file : '
      if(lmnd) call excmd('rm -rf '//trim(fileout)) ! Remove the newly
! -----      created output directory if the psi file cannot be opened.
      write(6,'(a)')  trim(filename)
      stop

      end subroutine flxinput

!#######################################################################

      subroutine flxrdinfo

      implicit none

      integer(long), external :: myiargc,slen
      integer(long)           :: ilbl,i,j,ierr,m,n,f,s,ll
      integer(long)           :: chkdvr,chkgrd,chkpsi,chkdat
      real(dop)               :: emina,emaxa,delt,fac
      logical(kind=4)         :: lerr,lgtau
      character(len=c4)       :: cgtau
      character(len=c5)       :: filename,mdlbl
      character(len=2*c5)     :: nspf

!-----------------------------------------------------------------------
!  Read DVR, psi-file and oper-file information.
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      call dvrinfo(lerr,chkdvr)
      close(idvr)
      chkdvr=2
      rewind(ipsi)
      read(ipsi) filever(ipsi)
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      call chksyserr(filein,lerr)
      if (lerr) goto 999
      if(lovwr .or. ltest) then
         chkgrd=2
         call operinfo(lerr,chkdvr,chkgrd)
         close(ioper)
         if (lerr) goto 888

         write(6,'(a,f5.1,a,4x,f5.1,a)')            &
            ' program-version =',                   &
            filever(ipsi), ' (psi-file)',           &
            filever(ioper), ' (oper-file)'
         write(6,'(78a1)')  ('-',i=1,78)
         write(6,'(a,i4)') ' Number of degrees of freedom:', ndof
         write(6,'(a,i4)') ' Number of combined modes    :', nmode
         write(6,'(a,i4)') ' Number of electronic states :', nstate
         write(6,'(a,i9)') ' Size of WF : dgldim =', dgldim
         if( nst2 .le. 0 ) then
            nst2 = nstate
         else
            write(6,'(a,2i4)') ' Flux evaluated for states   :', &
                                 nst1, nst2
         endif

         write(6,'(/,a)') ' mode dof  dim   DVR     modelabel'
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               write(6,'(1x,i3,1x,i4,i5,3x,a8,a10)') &
                    m, f, gdim(f), cbas(basis(f)), modelabel(f)
            enddo
         enddo

!-----------------------------------------------------------------------
! write mode information.
!-----------------------------------------------------------------------
         write(6,'(/,a)') ' mode: No. of spf''s per state : modelabels'
         do m=1,nmode
            ll = 1
            mdlbl='                                                       '
            nspf ='                                                       '
            do  n=1,nspfdof(m)
               ilbl  = index(modelabel(spfdof(n,m)),' ')
               mdlbl = mdlbl(1:ll)//modelabel(spfdof(n,m))(1:ilbl)
               ll    = ll + ilbl
            end do
            write(nspf,'(80i3)') (dim(m,s),s=1,nstate)
            write(6,'(i4,4a)') m,' :',nspf(1:3*nstate),' : ', &
                  mdlbl(1:ll)
         enddo
      else
         close(ipsi)
      end if      !   if(lovwr .or. ltest) then

!-----------------------------------------------------------------------
! Determine the new dgldim and load zetf_f in case of identical modes.
!-----------------------------------------------------------------------
      dgldimold=dgldim
      do m=1,nmode
         do s=1,nstate
            zetf_f(m,s)=zetf(m,s) ! save old zetf, needed to expand the WF
         enddo
         if(idmode(m).gt.0) then
            lid=.true.
            idmode(m)=0
         endif
      enddo
      call psidat ! re-computes zetf and dgldim
      dgldimnew=dgldim

!-----------------------------------------------------------------------
! Determine the dof of the CAP.
!-----------------------------------------------------------------------
      if((lovwr.or.ltest) .and. .not.lprojonly) then  ! Here starts the IF loop.
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            if( capdof .eq. modelabel(f) ) then
                mcap = m
                fcap = f
                if(nspfdof(m) .ne. 1 ) then
                  write(6,'(/a)') &
                   ' The CAP is in a combined mode!'
                  write(6,'(a,3i5)')' m, f, nspfdof(m)',m,f,nspfdof(m)
               end if
            end if
         enddo
      enddo

      ierr = 0
      hcap = 0
      if( fcap .eq. 0 )   read(capdof,*,iostat=ierr) fcap
      if( capdof .ne. '0' .and. (fcap .eq. 0 .or. ierr .ne. 0) ) then
         write(6,'(/a)') &
        'Cannot determine the degree of freedom for the CAP.'
         write(6,'(a,a12)') 'Illegal input reads : ', capdof
         stop
      end if
      if( fcap .lt. 0 ) then
         write(6,'(/a,a,i6)') &
        'Illegal degree of freedom for the CAP specified.', &
        '   fcap =',fcap
         stop
      elseif( fcap .gt. ndof ) then
         hcap = fcap
         fcap = htmdof(hcap)
         write(6,'(/a,i6)') &
         ' h-term rather then f-cap assumed. h =', hcap
      else
         write(6,*)
      end if
      mcap=0
      if( fcap .gt. 0 ) mcap = dofspf(fcap)
      if( mcap .gt. 0 ) then
         gdmc=gdim(fcap)
         sdmc = subdim(mcap)
      else           ! sdim=max(subdim).
         sdmc = sdim ! sdmc must be set when allocating memory.
         gdmc = sdim
      endif
      if(ltest) then
         continue
      else if( fcap.eq.0 ) then
         write(6,'(a,/a)') &
        ' No degree of freedom for the CAP specified.', &
        ' The first CAP found on the oper file will be taken.'
      else
         write(6,'(a,i3,a,i3,a,3x,a)') &
       ' CAP-mode =', mcap,',  CAP-dof =',fcap,',',trim(modelabel(fcap))
      end if


!-----------------------------------------------------------------------
! Determine number of data sets.
!-----------------------------------------------------------------------
      nout2 = 0
      call rdpsindat(ipsi,nout2)           

      tfinal = out2*(nout2-1) + tinit 
      write(6,'(/a,i6,a)') ' There are', nout2, &
                          ' data sets on the psi file.'
      write(6,'(a,f7.2,a,f9.2,a,f8.2,a)') &
        ' tinit =', tinit/fs, ' fs,  t-final =', tfinal/fs, &
        ' fs,  tpsi =', out2/fs, ' fs.'
      if(.not.lgtau .and. .not.lwtt) &
          write(6,'(a)') ' The gtau file will be computed.'

      if(khi .eq. 0) khi = nout2
      if( (khi.ne.nout2 .or. klo.ne.1) .and. .not.lgtau ) then
         write(6,'(/a,/a,i6,a,f11.2,a,/a,i6,a,f11.2,a)') &
       ' times for integration are changed by -lo and/or -hi option to:' &
      ,' first data point, klo =', klo,', at:', &
                     ((klo-1)*out2+tinit)/fs,'fs' &
      ,' last  data point, khi =', khi,', at:', &
                     ((khi-1)*out2+tinit)/fs,'fs'
      end if

      if( nst1 .gt. nst2 .or. nst2 .gt. nstate ) then
         write(6,'(/a,/a,2i4,/a,i4,/a)') &
               '######################################################' &
              ,' Illegal value of (option -es) nst1,nst2 =', nst1,nst2 &
              ,' 1 .le. nst1 .le. nst2 .le.', nstate &
              ,'######################################################' 
         stop
      end if
      if( khi .gt. nout2 ) then
         write(6,'(/a,/a,i5,/a,i5,/a)') &
               '######################################################' &
              ,' Illegal value of (option -hi) khi =', khi &
              ,' khi must be lower/equal  nout2 =', nout2 &
              ,'######################################################'
         stop
      end if
      if( klo .ge. khi ) then
         write(6,'(/a,/a,/a,i5,a,i5,/a)') &
               '######################################################' &
              ,' Illegal values of (options -lo, -hi) klo and khi' &
              ,' klo =',klo,' must be lower than  khi =', khi &
              ,'######################################################'
         stop
      end if

      kmax  = (khi-klo+step)/step
      laewmat = (kmax*(kmax+1))/2
      if(ltest) then
         laewmat = 0
         khi = 0
      end if


      else    ! This is the else  of "If(lovwr)"
!################################################################
!........These are dummy values for lovwr=.false.
         khi = 4001
         klo = 1
         laewmat = 10
         nout2 = kmax + 100
         if(lprojonly) then
            nout2 = 0
            nout2 = 0
            call rdpsindat(ipsi,nout2)           
            write(6,'(/a,i6,a)') ' There are', nout2, &
                          ' data sets on the psi file.'
            khi = nout2
            kmax = nout2
            step =1
         endif
!################################################################
      end if  ! This is the endif of "If(lovwr)"

!.....On input dipolem = Norm of D*Psi (see MCTDH log file).
      if(lmb)  dipolem=2.689093d0*dipolem*dipolem
!.....2.689093d0=pi*e^2*r0^2/(3*c*eps0*hbar*Mb).
!.....On output dipolem = conversion factor to Mb. 1Mb = 10^-18 cm^2

!.....Write to screen if step, lwtt, or lmb are set.
      if(start.gt.0) write(6,'(a,i4,4x,a)') &
           ' start =', start, '(grid point of flux surface)'
      if( step .ne. 1) &
            write(6,'(a,i3)') ' CAUTION : step =',step
      if(lgtt) write(6,'(a)') ' CAUTION : -v option is set!'
      if(lmb) write(6,'(2a)') ' CAUTION : -Mb option is set!', &
       '  Flux is multiplied by E and dipole moment.'

!.....Write to log-file.
      if(inplaenge .gt. 0 ) call cpflxinp
      if( optlaenge .le. 60 ) then
         write(ilog,'(2a/)')   '### Options : ', options(1:optlaenge)
      else
         write(ilog,'(a,/a/)') '### Options : ', options(1:optlaenge)
      end if

      filename=fileout(1:laeout)//cgtau
      if(inplaenge .gt. 0 ) write(ilog,'(2a)') &
                      ' The input-file is : ',inputfile(1:inplaenge)
      write(ilog,'(2a)') ' The psi-file   is : ',filein(1:laein)
      write(ilog,'(2a)') ' The oper-file  is : ',operfile(1:opflaenge)
      write(ilog,'(2a)') ' The output dir is : ',fileout(1:laeout)
      if(.not.lwtt) write(ilog,'(2a)') ' The gtau file  is : ', &
                                     filename(1:slen(filename))

      write(ilog,'(2a)') ' The DVR-file   is : ',dvrfile(1:dlaenge)
      write(ilog,'(2a)') ' The edstr-file is : ',edstrf(1:edlaenge)

      write(ilog,'(/a,i4,a,i6,a,i4,a,i5)') ' klo =', klo, &
         ',  khi =', khi,',  step =', step,',  start =', start

      write(ilog,'(a,i5)') ' Energy points = ',epoints
      write(ilog,'(a,a)') ' CAP = ',trim(capdof)
      if(loper) write(ilog,'(2a)') ' Operator = ',trim(operator)
      do i = 1, nprj
         do j = maxprj , 1 , -1
            if(prjpar(j,i) .ne. -999) then
               ll = j
               goto 66
            endif
         enddo
 66      if(prjlab(i) .eq. ' ' ) then
            write(ilog,'(a,i2,a,a12,2x,8i5)') &
              ' Projector(',prjmode(i),'):', &
              prjname(i)(1:slen(prjname(i))), &
              (prjpar(j,i),j=1,ll)
         else
            write(ilog,'(a,i2,a,2a12,8i5)') &
              ' Projector(',prjmode(i),'):', &
              prjname(i)(1:slen(prjname(i))), &
              prjlab(i)(1:slen(prjlab(i))),(prjpar(j,i),j=2,ll)
         endif
      enddo
      if(lwtt .or. lprojonly) goto 81
      write(ilog,'(2a,f12.4)') ' unitf: ',unitf, ufac

      emina = emin
      emaxa = emax
      call keylowc(unit)
      call keyeunits(emaxa,unit,ierr)
      call keyeunits(emina,unit,i)
      if(ierr+i .gt. 0) then
         write(6,'(2a)') ' unknown energy unit: ', unit
         stop
      end if
      write(6,'(/a,f11.4,a,f11.4,1x,a,5x,a,2f11.6,a)') ' Emin =', &
         emin,',  Emax =',emax,trim(unit),'  (',emina,emaxa,' au)'
      write(ilog,'(/a,f11.4,a,f11.4,1x,a,5x,a,2f11.6,a)')' Emin =', &
         emin,',  Emax =',emax,trim(unit),'  (',emina,emaxa,' au)'

!.....Check if energy interval is consistent with time interval.
      delt = step*out2/(2.d0*pi)
      fac  = abs(emaxa-emina)*delt
      if(fac .ge. 1.d0) write(6,800) 1.d0/delt, ev/delt, fac

      if( eoff .ne. 0.d0 ) then
         write(6,'(a,f12.6,1x,a,22x,a,f22.12,a)') &
              ' Eoff =', eoff1,trim(unite), '(',eoff,' au)'
         write(ilog,'(a,f12.6,1x,a,22x,a,f22.12,a)') &
              ' Eoff =', eoff1,trim(unite), '(',eoff,' au)'
      end if
      call myflush(ilog)

      if(ltest) write(6,'(/a,i5,a)') ' enerd :',maxendim, ' lines.'

 81   write(6,'(78a1)')  ('-',i=1,78)


!.....Set logical "full" which is true for full projection, leaving 1D WF.
      if( nmode - nprj .eq. 1 ) full = .true.
      if( nmode .le.  nprj ) then
         write(6,'(a,/a,i4,a,i4)') &
              'ERROR:  Too many projectors, more than nmode-1', &
              '  nmode =',nmode,',   nprj =',nprj
         stop
      endif
      full = full.and.lovwr.and.(nmode.gt.1)
      if(full .and. loper) then
         write(6,'(a)') &
              'ERROR  Operators are not allowed for full projection'
         stop
      endif
      if(full .and. lrunml) full=.false. ! A ML WF is reduced without reducing
  ! the number of primitive modes. The grid length or reduced modes is set to 1.


!.....Print logicals to log-file.
      write(ilog,'(/2a,/l3,13l6)') ' ltest lovwr lproj loper  lwtt', &
        '  lgtt lgtau lderiv full lrunml lopsym ladd lc3d', &
         ltest,lovwr,lproj,loper,lwtt,lgtt,lgtau,lderiv,full, &
         lrunml,lopsym,ladd,lc3d,lprojonly

      if(lprojonly .and. .not.lproj) then
         write(6,'(a)') &
          'ERROR  lproj must be true if lprojonly is true.'
         stop
      endif

      return

 800  format(/,' Warning: Energy interval too large for time-', &
           'resolution.',/,' max-del-E =', 1pe11.3,' au, ', &
           1pe11.3,' eV',/,' You will see the spectrum', &
           0pf7.2,' times'/)

 888  continue
      message= 'Error reading oper-info from:'//operfile(1:opflaenge)
      write(6,'(a)') message
      write(ilog,'(/a)') message
      stop

 999  continue
      message= 'Error reading psi-info from:'//filein(1:laein)
      write(6,'(a)') message
      write(ilog,'(/a)') message
      stop

      end subroutine flxrdinfo

!***********************************************************************
! Read the flux input file.
!***********************************************************************

      subroutine rdflxinpf(lmnd,lopsym_in)

      implicit none

      integer(long) :: ic,iz,iz0,ierr,ilbl,i,j
      logical(kind=4), intent(out) :: lmnd,lopsym_in

      open(iin,file=inputfile,status='old',form='formatted',err=90)

      routine = 'rdflxinpf'
      iz0 = 0
      iz  = 0
      lwarn = .false.
      lopsym_in=.false.

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (keyword(1).eq.'end-input') goto 20
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue

      if (keyword(i) .eq. 'overwrite')  then
            lovwr=.true.
      else if (keyword(i) .eq. 'test')  then
            ltest=.true.
      else if (keyword(i) .eq. 'nofs')  then
            fs=1.d0
      else if (keyword(i) .eq. 'noev')  then
            ev=1.d0
      else if( keyword(i) .eq. 'operfile' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with operfile keyword'
            call warnmsg
         else
            i=i+2
            operfile=keyword(i)
         endif
      else if( keyword(i) .eq. 'psi' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with psifile keyword'
            call warnmsg
         else
            i=i+2
            filein=keyword(i)
         endif
      else if( keyword(i) .eq. 'outdir' .or. &
               keyword(i) .eq. 'outpath' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with outdir keyword'
            call warnmsg
         else
            i=i+2
            fileout=keyword(i)
         endif
      elseif( keyword(i) .eq. 'dvr' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with dvr keyword'
            call warnmsg
         else
            i=i+2
            dvrfile=keyword(i)
         endif
      elseif( keyword(i) .eq. 'edstr' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with edstr keyword'
            call warnmsg
         else
            i=i+2
            edstrf=keyword(i)
            if(keyword(i+1) .eq. '+' ) then
               i=i+1
               ladd=.true.
               lc3d=.false.
               if(keyword(i+1) .eq. 'd' ) then
                  i=i+1
                  lc3d=.true.
               endif
            elseif(keyword(i+1) .eq. 'd' ) then
               i=i+1
               ladd=.false.
               lc3d=.true.
               if(keyword(i+1) .eq. '+' ) then
                  i=i+1
                  ladd=.true.
               endif
            endif
         endif
      elseif (keyword(i) .eq. 'flux-unit') then
         if (keyword(i+1) .ne. '=') then
            message = 'No argument specified with edstr keyword'
            call warnmsg
         else
            i=i+2
            read(keyword(i),*,iostat=ierr) ufac
            if (ierr .ne. 0) then
               ierr  = 0
               unitf=trim(keyword(i))
               ufac  = 1.d0
               call keylowc(unitf)
               call keyunits(ufac,unitf,ierr)
               if(ierr .ne. 0) then
                  message = 'Illegal flux-unit: '//unitf
                  call warnmsg
               end if
               ufac = 1.d0/ufac
            endif
         endif
      elseif (keyword(i)(1:2) .eq. 'hi') then
         if (keyword(i+1) .ne. '=') then
            message = 'No argument specified with high keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) khi
            if (ierr .ne. 0 .or. khi .le. 0) then
               message = 'Positive integer argument required for high. Not:'
               ilbl    = index(message,':')+2
               message = message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
         endif
      elseif (keyword(i)(1:2) .eq. 'lo') then
         if (keyword(i+1) .ne. '=') then
            message = 'No argument specified with low keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) klo
            if (ierr .ne. 0 .or. klo .le. 0) then
               message = 'Positive integer argument required for low. Not:'
               ilbl    = index(message,':')+2
               message = message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
         endif
      else if (keyword(i) .eq. 'make-outdir' .or. &
               keyword(i) .eq. 'make_outdir')  then
         lmnd=.true.
      else if (keyword(i) .eq. 'symmetrized-operator' .or. &
               keyword(i) .eq. 'symmetrized_operator')  then
         lopsym_in=.true.
      else if (keyword(i) .eq. 'theta-only')  then
         ltonly=.true.
      else if (keyword(i) .eq. 'use-deriv') then
         lderiv=.true.
      elseif (keyword(i)  .eq. 'step') then
         if (keyword(i+1) .ne. '=') then
            message = 'No argument specified with step keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) step
            if (ierr .ne. 0 .or. step .le. 0) then
               message = 'Positive integer argument required for step. Not:'
               ilbl    = index(message,':')+2
               message = message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
         endif
      elseif (keyword(i)  .eq. 'start') then
         if (keyword(i+1) .ne. '=') then
            message = 'No argument specified with start keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) start
            if (ierr .ne. 0 .or. start .le. 0) then
               message = 'Positive integer argument required for start. Not:'
               ilbl    = index(message,':')+2
               message = message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
         endif
      elseif (keyword(i) .eq. 'epoints') then
         if (keyword(i+1) .ne. '=') then
            message = 'No argument specified with epoints keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) epoints
            if (ierr .ne. 0 .or. epoints .le. 0) then
               message = 'Positive integer argument required for epoints. Not:'
               ilbl    = index(message,':')+2
               message = message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
         endif
      elseif (keyword(i) .eq. 'wtt-only') then
         lwtt  = .true.
         lovwr = .true.
      elseif (keyword(i) .eq. 'gtau-only') then
         lgtt  = .true.
         lovwr = .true.
      elseif (keyword(i) .eq. 'smooth') then
         lsmooth  = .true.
      elseif(keyword(i) .eq. 'Mb' ) then
         lmb=.true.
         if (keyword(i+1) .ne. '=') then
            message = 'No argument specified with Mb keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) dipolem
            if (ierr .ne. 0) then
               message = 'Real argument required for Mb. Not:'
               ilbl    = index(message,':')+2
               message = message(1:ilbl)//trim(keyword(i+2))
               call warnmsg
            endif
            i=i+2
         endif
      elseif (keyword(i) .eq. 'el-state') then
         if (keyword(i+1) .ne. '=') then
            message = 'No argument specified with el-state keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) nst1
            if (ierr .ne. 0 .or. nst1 .le. 0) then
               message = 'Positive integer argument required for el-staes. Not:'
               ilbl    = index(message,':')+2
               message = message(1:ilbl)//trim(keyword(i+2))
               call warnmsg
            endif
            i=i+2
            if(keyword(i) .eq. ',') then
               read(keyword(i+1),*,iostat=ierr) nst2
               if (ierr .ne. 0 .or. nst2 .lt. nst1) then
                  message = &
                      'Positive integer argument required for el-staes. Not:'
                  ilbl    = index(message,':')+2
                  message = message(1:ilbl)//trim(keyword(i+2))
                  call warnmsg
               endif
               i=i+1
            else
               nst2 = nst1
            endif
         endif
      elseif (keyword(i)(1:4) .eq. 'eoff') then
         if (keyword(i+1) .ne. '=') then
            message = 'No argument specified with eoff keyword'
            call warnmsg
         else
            if(keyword(i+2) .eq. 'lsth' ) then
               eoff  = 4.74746d0
               unite = 'ev'
               eoff1 = eoff
               call keyeunits(eoff,unite,ierr)
            else
               read(keyword(i+2),*,iostat=ierr) eoff
               eoff1 = eoff
               if (ierr .ne. 0 ) then
                  message = 'Real number argument required for eoff. Not:'
                  ilbl    = index(message,':')+2
                  message = message(1:ilbl)//trim(keyword(i+2))
                  call warnmsg
               endif
               i=i+2
               if(keyword(i+1).eq.',' ) then
                  if(i+2.le.ic) then
                     unite=trim(keyword(i+2))
                     call keylowc(unite)
                     call keyeunits(eoff,unite,ierr)
                  else
                     message = 'No argument followed comma! keyword : '
                     call wrline(i,iz,iz0)
                  endif
                  i=i+2
               endif
            endif
         endif

      elseif (keyword(i) .eq. 'energy') then
         if (keyword(i+1) .ne. '=') then
            message = 'No argument specified with energy keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) emin
            if (ierr .ne. 0 ) then
               message = 'Real number argument required for emin. Not:'
               ilbl    = index(message,':')+2
               message = message(1:ilbl)//trim(keyword(i+2))
               call warnmsg
            endif
            i=i+2
            if(keyword(i+1).eq.',' ) then
               if(i+2.le.ic) then
                  read(keyword(i+2),*,iostat=ierr) emax
                  if (ierr .ne. 0 ) then
                     message = 'Real number argument required for emax. Not:'
                     ilbl    = index(message,':')+2
                     message = message(1:ilbl)//trim(keyword(i+2))
               call warnmsg
            endif
               else
                  message = 'No argument followed comma! keyword : '
                  call wrline(i,iz,iz0)
               endif
               i=i+2
            endif
            if(keyword(i+1).eq.',' ) then
               if(i+2.le.ic) then
                  unit = trim(keyword(i+2))
                  call keylowc(unit)
               else
                  message = 'No argument followed comma! keyword : '
                  call wrline(i,iz,iz0)
               endif
               i=i+2
            endif
         endif
      else if( keyword(i) .eq. 'cap' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with outpath keyword'
            call warnmsg
         else
            i=i+2
            capdof=trim(keyword(i))
         endif
      else if( keyword(i) .eq. 'captype' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with outpath keyword'
            call warnmsg
         else
            i=i+2
            if(keyword(i) .eq. 'left') then
               captype=1
            elseif(keyword(i) .eq. 'right') then
               captype=2
            elseif(keyword(i) .eq. 'both') then
               captype=3
            else
            message = 'Wrong argument for captype. Must be both, left, or right'
            call warnmsg
            endif
         endif
      else if( keyword(i) .eq. 'operator' )  then
         loper = .true.
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with operator keyword'
            call warnmsg
         else
            i=i+2
            operator=trim(keyword(i))
         endif
      else if( keyword(1) .eq. 'projector' )  then
         lproj = .true.
         nppar = 0
         nprj  = nprj + 1
         if (nprj .gt. maxprj) then
            write(6,*)'Too many projectors. maxprj =', maxprj
            stop
         endif
         read(keyword(2),*,iostat=ierr) prjmode(nprj)
         if (ierr .ne. 0 .or. prjmode(nprj) .le. 0) then
            message = 'Positive integer argument required for prjmode. Not:'
            ilbl    = index(message,':')+2
            message = message(1:ilbl)//trim(keyword(i+2))
            call warnmsg
         endif
         do j=1,nprj-1
            if(prjmode(j) .eq. prjmode(nprj)) then
               message = 'There must not be two projectors '// &
                         'projecting on the same mode!'
               call warnmsg
            endif
         enddo
         if(ic .lt. 4) then
            message = 'Not enough arguments for projector'
            call warnmsg
         endif
         prjname(nprj) = trim(keyword(3))
         j=3
 50      j=j+1
         if(keyword(j)(1:1) .eq. '%' .or. j .gt. ic) goto 10
         nppar = nppar + 1
         if( nppar .gt. maxprj) then
            write(6,*) 'Too many parameters! maxprj =',maxprj
            stop
         endif
         read(keyword(j),*,iostat=ierr) prjpar(nppar,nprj)
         if(nppar.eq.1  .and.  ierr.ne.0 ) then
            prjpar(nppar,nprj) = -999
            prjlab(nprj) = trim(keyword(j))
            ierr = 0
         end if
         if (ierr .ne. 0 ) then
            message = 'Integer argument required for prjpar. Not:'
            ilbl    = index(message,':')+2
            message = message(1:ilbl)//trim(keyword(j))
            call warnmsg
         endif
         goto 50

      else
         message = ' unknown keyword : '
         call wrline(i,iz,iz0)
      end if

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

 20   continue

      close (iin)

      if(lwarn) then
         message='Finished with input error. See log file.'
         call errormsg
      endif
      return

 90   continue
      write(6,*)
      write(6,'(a)') 'ERROR in subroutine rdflxinpf:'
      write(6,'(a)') 'Cannot open input file!'
      write(6,'(2a)') 'inputfile: ', inputfile
      write(6,*)
      stop

      end subroutine rdflxinpf


!***********************************************************************
! Read the options.
!***********************************************************************

      subroutine rdoptions(iarg,lmnd,lopsym_in)

      implicit none

      integer(long)                 :: i,j,ierr
      integer(long), intent(in)     :: iarg
      character(len=80)             :: buf
      logical(kind=4)               :: lopno
      logical(kind=4), intent(out)  :: lmnd,lopsym_in

      lopno=.false.

      if(iarg .le. 0) goto 200

!........Read the options
      i = 0
 100  i=i+1
      if (iarg .lt. i .and. ltest) goto 400
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)

      if( buf(1:1) .eq. '-' ) then
         j = 1
 99      j = j + 1
         if( buf(j:j) .eq. ' ' ) goto 100
!........Options of more than one character length without arguments.
         if( buf(j:j+2) .eq. 'ver' ) then
            call wrversion(6)
            stop
         else if( buf(j:j+3) .eq. 'wtt ' ) then
            lwtt  = .true.
            j = j + 2
            goto 99
         else if( buf(j:j+2) .eq. 'sm ' ) then
            lsmooth  = .true.
            j = j + 1
            goto 99
         else if( buf(j:j+4) .eq. 'nofs ' ) then
            fs  = 1.d0
            j = j + 3
            goto 99
         else if( buf(j:j+4) .eq. 'noev ' )  then
            ev = 1.d0
            j = j + 3
            goto 99
         else if( buf(j:j+8) .eq. 'projonly ' )  then
            lprojonly=.true.
            j = j + 7
            goto 99

!........Options of more than one character length with arguments.
         else if( buf(j:j+2) .eq. 'ed ' ) then
            i = i+1
            call mygetarg(i,edstrf)
            call mygetarg(i+1,buf)
            if( buf .eq. '+' ) then
               i=i+1
               ladd=.true.
               lc3d=.false.
               call mygetarg(i+1,buf)
               if( buf .eq. 'd' ) then
                  i=i+1
                  lc3d=.true.
               endif
            elseif( buf .eq. 'd' ) then
               i=i+1
               ladd=.false.
               lc3d=.true.
               call mygetarg(i+1,buf)
               if( buf .eq. '+' ) then
                  i=i+1
                  ladd=.true.
               endif
            else
               ladd=.false.
               lc3d=.false.
            endif
         else if( buf(j:j+3) .eq. 'dvr ' ) then
            i = i+1
            call mygetarg(i,dvrfile)
         else if( buf(j:j+2) .eq. 'lo ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) klo
            if (ierr .ne. 0  .or.  klo .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -lo', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(j:j+2) .eq. 'hi ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) khi
            if (ierr .ne. 0  .or.  khi .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -hi', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(j:j+2) .eq. 'es ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nst1
            nst2 = nst1
            if (ierr .ne. 0  .or.  nst1 .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -es', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(j:j+2) .eq. 'Mb ' ) then
            i = i+1
            lmb=.true.
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) dipolem
            if (ierr .ne. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -Mb', &
                 ' must be a real number. Not : ', buf
               stop
            end if

!........Options of one character length without arguments
         else if( buf(j:j+1) .eq. 't ' ) then
            ltest=.true.
            capdof='0'
            emin=0.d0
            emax=0.d0
            goto 99
         elseif( buf(j:j+1) .eq. 'w ' ) then
            lovwr=.true.
            goto 99
         elseif( buf(j:j+1) .eq. 'r ' ) then
            lread=.true.
            goto 99
         elseif( buf(j:j+1) .eq. 'v ' ) then
            lovwr=.true.
            lgtt =.true.
            goto 99
         elseif( buf(j:j+1) .eq. 'x ' ) then
            ltonly=.true.
            goto 99
         elseif( buf(j:j+1) .eq. 'd ' ) then
            lderiv=.true.
            goto 99
         elseif( buf(j:j+1) .eq. 'h ' .or. buf(j:j+1) .eq. '? ' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,a,/10x,a)') &
       'Purpose: Computes the quantum flux.', &
           'See review Eqs.(197-201) for a definition of the output'  
            write(6,'(/,2a,/8x,2a)')                                &
       'Usage : flux [-f -i -o -om -ed -dvr -t -n -lo -hi -p -P ',   &
                 '-O -Os -e -x -d -v -w -r -c -exp -u ',             &
             '-sm -es -wtt -nofs -noev -DB -ver -h -?] ',           &
             '[emin emax unit CAP-dof CAP-type] [.inp-file].'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a)') &
                     ' -f FILE : The wavefunction is read '&
                    ,'from file FILE rather than from ./psi'&
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a)') &
                     ' -i FILE : The operator is read '&
                    ,'from file FILE rather than from ./oper'&
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a)') &
                     ' -dvr FL : The DVR information is read ' &
                    ,'from file FL rather than from ./dvr'&
            ,'  The string FL may be a relative or a full path-name.'
            write(6,'(2a/,9x,a/,9x,a/,9x,a/,9x,2a/,9x,a/9x,a)') &
                     ' -ed FL  : The energy distribution is read '&
                    ,'from file FL rather than from ./enerd'&
            ,'  The string FL may be a relative or a full path-name.'&
            ,'  One may use a flux-file as energy-distribution file.'&
            ,'  In this case, the flux-column will be taken as edstr.'&
            ,'  If a "d" follows the file-name, the DELTA(E) column'&
            ,' will be taken.'&
            ,'  If a "+" follows the file-name, the present flux will '&
            ,'  be added to the edstr input.'
            write(6,'(11x,a,/,11x,a,/,11x,a,/,11x,a,/,11x,a,/,11x,a)')&
             'Alternatively, one may use a spectrum.pl (autospec)',&
             'or a cspec.pl (crosspec) file as energy distribution.',&
             'The options "d" and "+" work here as well. With "d" ',&
             'the third (rather than the second) column of the ed-file',&
             'is used, and with "+" the data is added to DELTA(E)',&
             'rather than overwriting it.'
            write(6,'(2a,/,11x,a/,11x,a)') &
                     ' -o PATH : The output is directed to PATH ',&
                     'rather than to the current directory.',&
            'The string PATH may be a relative or a full path-name.',&
            'If PATH=no, i.e. "-o no", no output-files will be opened.'
            write(6,'(2a,/,11x,a)') &
                ' -om PATH: Similar to -o, but in addition the ', &
                'output directory', &
                'will be created (like -mnd in mctdh).'
            write(6,'(2a)') ' -n step : Compute Wtt'' only', &
                            ' every step-th psi-output.'
            write(6,'(2a)') ' -es nst : Compute the flux for the', &
                            ' electronic state nst only.'
            write(6,'(2a)') ' -lo klo : Compute Wtt'' only', &
                            ' from the klo-th psi-output onwards.'
            write(6,'(2a)') ' -hi khi : Compute Wtt'' only', &
                            ' up to the khi-th psi-output.'
            write(6,'(2a,i5,a)') &
               ' -p I    : The no. of energy-points is I rather',&
               ' than', epoints,'.'
            write(6,'(2a,/11x,a,/11x,2a,/11x,2a)') &
               ' -e R U  : The offset energy is set to R in ',&
               'units U; e.g. "-e 0.3 ev". ',&
               'Note: "-e lsth" is equivalent to "-e 4.74746 ev".',&
               'Give here the negative of "Energy shift (E_total - ',&
                'E_kinetic)",','which was printed to log-file. ',&
                '(correction=dia).'
            write(6,'(2a/11x,a)') ' -s strt : Set the starting ',&
               'point of the flux evaluation to strt.',&
               'Default is the grid-point where the cap starts.'
            write(6,'(2a)') ' -t      : Test run. ',&
               ' Check of the data sets. No output files opened.'
            write(6,'(2a,/10x,2a,/10x,a,/10x,a)') &
                ' -P      : Use projectors.',&
                ' -P is followed by the mode number, the projector',&
                ' name and up to 8 parameters.',&
                ' The input is closed by a "%".',&
                ' There may be several projectors. Example:', &
                ' -P 3 KLeg 2 1 % -P 2 eigenf vib 6 %'
            write(6,'(2a)') ' -O oper : Apply operator ',&
                            '"oper" before measuring the flux.'
            write(6,'(2a,/11x,2a,/11x,a)') &
                 ' -Os oper: Apply the operator ', &
                 'in a symmetric way. I.e 0.5*(OF+FO).',  &
                 'Works for correlated and uncorrelated ', &
                 'operators which may ', &
                 'operate on the CAP-mode.'
            write(6,'(a,/,11x,a,/,11x,a)') &
                 ' -u unit : Transform flux to unit "unit".',&
                 '(useful when a operator is applied).',&
                 'Note: Rather than a unit one may give a number.'
            write(6,'(2a,/11x,a)') &
            ' -w      : The gtau file may be overwritten. Without -w ',&
            'option','gtau is read from the gtau file (if it exist).'
            write(6,'(2a)')&
            ' -r      : Enforce reading the gtau file. Stop if gtau',&
            ' does not exist.'
            write(6,'(a,/11x,a)') &
            ' -v      : Re-compute only the theta part of gtau.',&
            '(useful in combination with the -s option).'
            write(6,'(a,/11x,a)') &
            ' -x      : Evaluate flux only for the theta part of gtau.',&
            '(useful for tests in combination with the -s -v options).'
            write(6,'(a)') ' -d      : '// &
            'Compute gtau from spatial derivative of wavefunction.'
            write(6,'(a)')  ' -c icos : Cosine**icos damping of gtau.'
            write(6,'(a)')  ' -exp n tau: exp(-(t/tau)**n) damping of gtau'
            write(6,'(2a)') ' -sm     : smooth Delta(E) by averaging',&
                            ' the enerd data over 5 points.'
            write(6,'(2a,/a)') ' -Mb norm: Flux spectrum in Mb.', &
             ' norm is ||D*Psi|| (in au)', &
             '           where D is the Dipole moment.'
            write(6,'(a/10x,a)')&
             ' -wtt    : Only wtt and iwtt are computed.', &
             ' Note: The arguments', &
             ' "Emin, Emax, unit" must not be given when -wtt is set.'
            write(6,'(a/10x,a)') &
             ' -nofs   : No transformation to fs.', &
             ' Use when "time-not-fs" was set in mctdh.'
            write(6,'(a/10x,a)') &
             ' -noev   : No transformation to eV.', &
             ' Use when "energy-not-ev" was set in mctdh.'
            write(6,'(a)')&
                 ' -DB PATH : The path to a direct dynamics DB.'
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(a,/a,/)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(2a,/a)')' The arguments specify the energy', &
             ' interval and',' the degree of freedom of the CAP.'
            write(6,'(a,/2a)')&
             ' The CAP-dof may be a number or a modelabel.',&
             ' The CAP-type may "right", "left" or "both".',&
             ' "right" is the default.'
            write(6,'(2a,/2a)')&
             ' If the number given as CAP-dof is larger than the',&
             ' number of DoFs,',' this number is interpreted as',&
             ' the H-term, h, of the CAP.'
            write(6,'(a,/a,/a)') &
                       ' Example: flux -lo 25 -w 0.3 2.5 ev rd', &
                       '          flux 1000 3000 cm-1  x left',  &
                       '          flux -wtt -s 21  y'
            write(6,'(2a)') ' If no argument is given, ',&
                 'the user is prompted for the missing argument(s).'
            write(6,'(2a,/a)') ' The path of an input file ',&
                    'may be given as the very last argument.',&
                    ' Options will overwrite the input-file.'

            write(6,'(78a1)')  ('-',i=1,78)
            stop

!........Options of one character length with arguments.
         else if( buf(j:j+1) .eq. 'f ' ) then
            i = i+1
            call mygetarg(i,filein)
         else if( buf(j:j+1) .eq. 'i ' ) then
            i = i+1
            call mygetarg(i,operfile)
         else if( buf(j:j+2) .eq. 'om ' ) then
            i = i+1
            lmnd=.true.
            call mygetarg(i,fileout)
         else if( buf(j:j+1) .eq. 'o ' ) then
            lmnd=.false.
            i = i+1
            call mygetarg(i,fileout)
         else if( buf(j:j+1) .eq. 'n ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -n', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(j:j+1) .eq. 's ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) start
            if (ierr .ne. 0  .or.  start .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -s', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(j:j+1) .eq. 'S ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) i0s
            if (ierr .ne. 0  .or.  i0s .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -S', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(j:j+1) .eq. 'u ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ufac
            if (ierr .ne. 0) then
               ierr  = 0
               unitf = trim(buf)
               ufac  = 1.d0
               call keylowc(unitf)
               call keyunits(ufac,unitf,ierr)
               if(ierr .ne. 0) then
                  write(6,'(/,2a,/)') &
                      '### Illegal unit (-u option) unitf = ',unitf
                  stop
               end if
               ufac = 1.d0/ufac
            endif
         else if( buf(j:j+1) .eq. 'e ' ) then
            i = i+1
            call mygetarg(i,buf)
            if( buf .eq. 'lsth' ) then
               eoff  = 4.74746d0
               eoff1 = eoff
               unite = 'ev'
               call keyeunits(eoff,unite,ierr)
            else
               read(buf,*,iostat=ierr) eoff
               if (ierr .ne. 0) then
                  write(6,*)'Argument for -e must be real number. Not :'
                  write(6,*) buf
                  stop
               end if
               i = i+1
               call mygetarg(i,unite)
               eoff1 = eoff
               call keylowc(unite)
               call keyeunits(eoff,unite,ierr)
               if(ierr .gt. 0) then
                  write(6,*)' -e option, unknown eoff-unit: ', unite
                  stop
               end if
            end if
         else if( buf(j:j+1) .eq. 'p ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) epoints
            if (ierr .ne. 0) then
               write(6,*)'Argument for -p must be a real number. Not :'
               write(6,*) buf
               stop
            end if
         else if( buf(j:j+1) .eq. 'P ' ) then
            lproj=.true.
            nppar = 0
            nprj = nprj + 1
            if (nprj .gt. maxprj) then
               write(6,*)'Too many projectors. maxprj =', maxprj
               stop
            endif
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) prjmode(nprj)
            if (ierr .ne. 0) then
               write(6,*)'Projector mode must be an integer. Not :'
               write(6,*) buf
               stop
            end if
            do j=1,nprj-1
               if(prjmode(j) .eq. prjmode(nprj)) then
               write(6,'(2a)') 'There must not be two projectors ', &
                               'projecting on the same mode!'
               stop
               endif
            enddo
            i = i+1
            call mygetarg(i,buf)
            call keylowc(buf)
            prjname(nprj) = trim(buf)
 110        i = i+1
            if(nppar .gt. maxprj) then
               write(6,*)'Too many proj-parameters. maxprj =', maxprj
               stop
            endif
            call mygetarg(i,buf)
            if( buf(1:1) .ne. '%' ) then
               nppar = nppar + 1
               ierr=11
               if(ichar(buf(1:1)).ne.47 .and. ichar(buf(1:1)).lt.60) &
                   read(buf,*,iostat=ierr) prjpar(nppar,nprj)
               if(nppar.eq.1  .and.  ierr.gt.0 ) then
                  prjpar(nppar,nprj) = -999
                  prjlab(nprj) = trim(buf)
                  ierr = 0
               end if
               goto 110
            end if
            if (ierr .ne. 0) then
               write(6,*) 'Error occurred while reading projector-parameters'
               stop
            endif
         else if( buf(j:j+1) .eq. 'O ' ) then
            loper=.true.
            lopno=.true.
            i = i+1
            call mygetarg(i,operator)
         else if( buf(j:j+2) .eq. 'Os' ) then
            loper=.true.
            lopsym =.true.
            i = i+1
            call mygetarg(i,operator)
         else
            if(ichar(buf(2:2)).ge.46 .and. ichar(buf(2:2)).le.57) &
                 goto 111     ! Found a negative number, not an option.
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
          if (iarg .le. i .and. ltest) goto 400
          if (iarg .le. i) goto 200
         goto 100
      end if

      if (iarg .lt. i .and. ltest) goto 400
      if (iarg .lt. i) goto 200

 111  if(lwtt) then
         if(iarg.ge.i+3) then
            i=i+3
            call mygetarg(i,buf)
         endif
         capdof = trim(buf)
         if(iarg .le. i ) goto 400
         i=i+1
         call mygetarg(i,buf)
         if( buf .eq. 'left' ) then
            captype = 1
         elseif( buf .eq. 'right' ) then
            captype = 2
         elseif( buf .eq. 'both' ) then
            captype = 3
         else
            write(6,'(a,/a,a)') &
           ' The CAP-type must be "right", "left" or "both".', &
           ' Not : ', buf(1:60)
            stop
         endif
         goto 400
      endif

      read(buf,*,iostat=ierr) emin
      if (ierr .ne. 0 .or. abs(emin) .gt. 1.d55) then
         write(6,*) 'The first argument (emin)  must be real number. Not :'
         write(6,*) buf(1:60)
         stop
      end if

      i=i+1
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) emax
      if (ierr .ne. 0) then
         write(6,*) 'The second argument (emax)  must be real number. Not :'
         write(6,*) buf
         stop
      end if

      i=i+1
      if (iarg .lt. i) goto 300
      call mygetarg(i,unit)
      call keylowc(unit)

      i=i+1
      if (iarg .lt. i) goto 300
      call mygetarg(i,capdof)
      if(iarg .le. i ) goto 400
      i=i+1
      call mygetarg(i,buf)
      if( buf .eq. 'left' ) then
         captype = 1
      elseif( buf .eq. 'right' ) then
         captype = 2
      elseif( buf .eq. 'both' ) then
         captype = 3
      else
         write(6,'(a,/a,a)') &
        ' The CAP-type must be "right", "left" or "both".', &
        ' Not : ', buf(1:60)
         stop
      endif

!-----------------------------------------------------------------------
!  Prompting for missing input (energies ignored if lwtt=.true.)
!-----------------------------------------------------------------------
 200  if(lprojonly) goto 600
      if(lwtt) goto 300
      if(emin .gt. -1.0d+18 .and. emax .gt. -1.0d+18) goto 300
      write(6,'(a)') ' E-min, E-max, unit =? (e.g:  2., 5., ev).'
      read(5,*) emin, emax, unit
      call keylowc(unit)

 300  if( capdof .ne. '  ' ) goto 400
      write(6,'(2a)') ' CAP-dof = ? ', &
                      ' (give a number or a modelabel).'
      write(6,'(2a)') ' 0 -> search for dof of CAP.', &
                      ' (useful, if there is only one CAP).'
      read(5,'(a)') capdof
      if( capdof(1:9) .eq. '         ' ) capdof = '0'
!-----------------------------------------------------------------------

 400  if(lopno.and.lopsym) then
         write(6,'(a)') ' There must appear only one of the options', &
                        ' "-O" or "-Os".'
         stop
      endif
      if(.not.(lopno .or. lopsym)) then
         ! Take the data from input-file as not set as options.
         lopsym=lopsym_in
      endif

 600  return
      end subroutine rdoptions


!***********************************************************************
! Copy the flux input file.
!***********************************************************************

      subroutine cpflxinp

      implicit none

      integer(long) :: i

      open(iin,file=inputfile,status='old',form='formatted')
      write(ilog,'(a)') 'Input file:'
 70   read(iin,'(a)') inptit
      call rmlblnk(inptit)  ! remove leading blanks
      if(inptit(1:9) .eq. 'end-input') goto 75
      if(inptit(1:1).ne.'#' .and. inptit(1:9).ne.'         ') &
             write(ilog,'(1x,a)') trim(inptit)
      goto 70

 75   write(ilog,'(80a1)') ('-',i=1,80)
      close(iin)
      return
      end subroutine cpflxinp

      end module flxinputmod  


