!***********************************************************************
      module flxprj

use versions, only: progver
use decimal, only: long,dop,sip
use op1lib, only: zerovxz, initvxl, cpvxz, unitqxz
use mtlib, only: mtxxzz
use mmlib, only: mmxtzz, mmxxzz
use xvlib, only: mvtxzz, mvxxzz
use lalib, only: normspf
use logdat

      implicit none
      private
      public :: projin
      contains

!***********************************************************************
!
!                         AUSWERT MODULE FLXPRJ
!
!     This file contains all routine needed for the use of projectors.
!     Contens: projin, reduce, newvndim, 
!              prj_eigenf, prj_read, prj_sph, prj_diff prj_diff2
!              prj_pop2, prj_leg, prj_KLeg, prj_PLeg, prj_kleg2
!              prj_Wigner
!
! HDM Nov 00
!
!
!***********************************************************************

      subroutine projin(hops,ort,jsph,msph,projector,trafo)

!***********************************************************************
!   PROJIN loads the projectors.
!   Note: The array "projector" actually contains the complex conjugate 
!         of the projecting function.
!***********************************************************************

use global
use aglobal
use dvrdatmod
use griddatmod
use fluxmod

      implicit none

      complex(dop), dimension(prjdim), intent(out)   :: projector
      real(dop), dimension(*), intent(in)            :: hops
      real(dop), dimension(ortdim), intent(in)       :: ort
      real(dop), dimension(dvrdim), intent(in)       :: trafo
      integer(long), external                        :: slen
      integer(long), dimension(*), intent(in)        :: jsph,msph
      integer(long)                                  :: np,m,sdm,f,f2,bsf,bsf2,&
                                                        gdm,gdm2,ggdim,nsm,dk,dm
      real(dop),allocatable  ::   weight1(:),weight2(:)

       do np = 1, nprj
          m   = prjmode(np)
          if( m .le. 0  .or. m .gt. nmode ) then
             write(6,'(3a,2i3)')'Illegal mode given for projector:  ',&
             prjname(np)(1:slen(prjname(np))), ' , mode =', m, nmode
             stop
          endif
          sdm = subdim(m)
          f   = spfdof(1,m)
          nsm = nspfdof(m)
          bsf = basis(f)
          gdm = gdim(f)
          if(prjname(np) .eq. 'eigenf') then
             if (.not.ldvr(f)) then
                write(6,'(a)') &
                ' Sorry, prj_eigenf works only for regular DVRs.'
                stop
             endif 
             call prj_eigenf(np,m,f,sdm,nsm,hops,projector(zprj(np)))

          elseif(prjname(np) .eq. 'read') then
             call prj_read(m,prjpar(3,np),sdm,prjpar(2,np),&
                           prjlab(np),projector(zprj(np)))

          elseif(prjname(np) .eq. 'diff'  .or. &
                 prjname(np) .eq. 'diff1') then
             call prj_diff(prjpar(1,np),m,f,sdm,gdm,bsf,nsm,pi,&
                           ort(zort(f)),projector(zprj(np)))

          elseif(prjname(np) .eq. 'diff2') then
             f2   = spfdof(2,m)
             bsf2 = basis(f2)
             gdm2 = gdim(f2)
             call prj_diff2(prjpar(1,np),prjpar(2,np),m,f,f2,sdm,&
                            gdm,gdm2,bsf,bsf2,nsm,pi,ort(zort(f)),&
                            ort(zort(f2)),projector(zprj(np)))

          elseif(prjname(np) .eq. 'sph') then
             call prj_sph(prjpar(1,np),prjpar(2,np),m,sdm,f,bsf,nsm,&
                  gdm,jsph(zsph(f)),msph(zsph(f)),projector(zprj(np)))

          elseif(prjname(np) .eq. 'pop1') then
             call prj_pop1(prjpar(1,np),m,f,sdm,projector(zprj(np)))

          elseif(prjname(np) .eq. 'pop2') then
             ggdim = gdm
             if(bsf .eq. 12 .or. bsf .eq. 20 .or. bsf .eq. 21) &
                ggdim = ipbaspar(3,f)
             call prj_pop2(prjpar(1,np),m,f,sdm,gdm,ggdim,bsf,nsm,&
                  trafo(zdvr(f)),projector(zprj(np)),ldvr(f))

          elseif(prjname(np) .eq. 'leg') then
             ggdim = gdim(f)
             if(bsf .eq. 12) ggdim = ipbaspar(3,f)
             call prj_leg(prjpar(1,np),f,m,sdm,gdm,ggdim,bsf,&
                  ipbaspar(1,f),ipbaspar(2,f),&
                  trafo(zdvr(f)),projector(zprj(np)))

          elseif(prjname(np) .eq. 'kleg') then
             dk = max(ipbaspar(5,f+1),1)
             ggdim = gdm
             if(bsf .eq. 20) ggdim = ipbaspar(3,f)
             call prj_kleg(prjpar(1,np),prjpar(2,np),m,sdm,gdm,ggdim,&
                  gdim(f+1),bsf,basis(f+1),nsm,&
                  ipbaspar(1,f),ipbaspar(1,f+1),dk,&
                  trafo(zdvr(f)),projector(zprj(np)))

          elseif(prjname(np) .eq. 'pleg') then
              ggdim = gdim(f)
              if(bsf .eq. 21) ggdim = ipbaspar(3,f)
             call prj_pleg(prjpar(1,np),prjpar(2,np),m,sdm,gdm,ggdim,&
                  gdim(f+1),bsf,basis(f+1),nsm,&
                  ipbaspar(1,f),ipbaspar(1,f+1),ort(zort(f+1)),&
                  trafo(zdvr(f)),projector(zprj(np)))

          elseif(prjname(np) .eq. 'kleg2') then
             call prj_kleg2(prjpar(1,np),prjpar(2,np),prjpar(3,np),&
                  prjpar(4,np),prjpar(5,np),m,gdim(f),gdim(f+1),&
                  gdim(f+2),gdim(f+3),trafo(zdvr(f)),trafo(zdvr(f+2)),&
                  projector(zprj(np)))

          elseif(prjname(np) .eq. 'wigner') then
             if (basis(f+1).eq.8) then
                dk = max(ipbaspar(5,f+1),1)
             else
                dk = 1
             endif
             if (basis(f+2).eq.8) then
                dm = max(ipbaspar(5,f+2),1)
             else
                dm = 1
             endif
             call prj_Wigner(prjpar(1,np),prjpar(2,np),prjpar(3,np),m, &
                  sdm,gdm,gdim(f+1),gdim(f+2),bsf,basis(f+1),basis(f+2), &
                  nsm,ipbaspar(1,f),ipbaspar(2,f),ipbaspar(6,f),dk,dm, &
                  trafo(zdvr(f)),ort(zort(f+1)),ort(zort(f+2)), &
                  projector(zprj(np)))

          elseif(prjname(np) .eq. 'p-1d') then
             allocate(weight1(gdm))
             call dvrweights(trafo(zdvr(f)),ort(zort(f)),gdm,gdm, &
                             weight1,bsf,rpbaspar(1,f),ipbaspar(1,f), &
                             ipbaspar(1,f),.false.,.false.)
             call prj_p1D(prjpar(1,np),prjpar(2,np),m,f,sdm,gdm,bsf, &
                          nsm,ort(zort(f)),projector(zprj(np)),weight1)
             deallocate(weight1)

          elseif(prjname(np) .eq. 'p-2d') then
             f2   = spfdof(2,m)
             bsf2 = basis(f2)
             gdm2 = gdim(f2)
             allocate(weight1(gdm),weight2(gdm2))
             call dvrweights(trafo(zdvr(f)),ort(zort(f)),gdm,gdm, &
                             weight1,bsf,rpbaspar(1,f),ipbaspar(1,f), &
                             ipbaspar(1,f),.false.,.false.)
             call dvrweights(trafo(zdvr(f2)),ort(zort(f2)),gdm2,gdm2, &
                             weight2,bsf2,rpbaspar(1,f2),ipbaspar(1,f2), &
                             ipbaspar(1,f2),.false.,.false.)
             call prj_p2D(prjpar(1,np),prjpar(2,np),prjpar(3,np), &
                          m,f,f2,sdm,gdm,gdm2,bsf,bsf2,nsm, &
                          ort(zort(f)),ort(zort(f2)), &
                          projector(zprj(np)),weight1,weight2)
             deallocate(weight1,weight2)

          else
             write(6,'(2a)') ' Unknown Projector ', prjname(np)
             stop
          endif
       enddo

       return
       end subroutine projin

!***********************************************************************

      subroutine prj_eigenf(np,m,f,sdm,nsm,hops,projector)
!-----------------------------------------------------------------------
!     The projector is on an eigenfunction of a 1D-operator.
!     The operator must be defined in the HAMILTONIAN-SECTION.
!     pop numbers the eigenstates (pop=1 -> ground-state). 
!-----------------------------------------------------------------------

use global
use dvrdatmod
use griddatmod
use operdef
use fluxmod

use genphimod, only: veigen

      implicit none
      
      integer(long)                                   :: n,ipot,pop,isym,s,dmf,g
      integer(long), intent(in)                       :: m,np,sdm,f,nsm
      real(dop), dimension(1)                         :: eeigen
      real(dop), dimension(*), intent(in)             :: hops
      real(dop), dimension(2)                         :: rdum
      real(dop)                                       :: e0eigen
      complex(dop), dimension(sdm), intent(out)       :: projector 

      call zerovxz(projector,sdm)        

      pop = prjpar(2,np)
      if(pop .le. 0 .or. pop .gt. sdm) then
         write(6,'(a,i3)') 'Illegal population. pop =', pop
         stop
      endif
      if (nsm .ne. 1) then
         write(6,'(a)') &
              'Sorry, prj_eigenf works only for uncombined modes.'
         stop
      endif

!-----------------------------------------------------------------------
! Determine the operator to be diagonalized.
! ipot is the hamiltonian number nham.
!-----------------------------------------------------------------------
      ipot = 0
      do n=1,hamdim
         if (prjlab(np) .eq. hamlab(n)) ipot=n
      enddo
      if (ipot .eq. 0) then
         write(6,'(a,/a,i3,2a)')&
              ' Hamiltonian required by eigenf not defined.',&
              ' Proj. No.',np,', op-label = ',prjlab(np)
         stop         
      endif

!-----------------------------------------------------------------------
! Call veigen (geninwf/genphi1.F) which computes the eigenvectors.
!-----------------------------------------------------------------------

      dmf=1
      s=1
      isym=0
      rdum(1)=0.0_dop
      call veigen(projector,gdim(f),dmf,rdum,pop,isym,hops,s,f,m,ipot,&
                  modelabel(f),eeigen,e0eigen,.false.)

!.....Note: The array "projector" actually contains the complex conjugate
!....       of the projecting function. Thus:
      do g = 1, gdim(f)
         projector(g) = dconjg(projector(g))
      enddo

      return
      end subroutine prj_eigenf


!***********************************************************************

      subroutine prj_read(mode,state,sdm,pop,rstfile,projector)
!-----------------------------------------------------------------------
!     The projector (SPF) is read from a restart file. 
!-----------------------------------------------------------------------

use global
use griddatmod
use psidef
use channels

use logdat, only: errormsg
use compdat, only: getcpsi
use iorst, only: rstinfo

      implicit none

      integer(long)                             :: g,j,lang,chkdvr,chkgrd,chkpsi,chkprp
      integer(long), intent(in)                 :: mode,sdm
      integer(long), intent(inout)              :: state,pop
      real(dop)                                 :: time
      character(len=64), intent(in)             :: rstfile
      character(len=c5)                         :: filename
      complex(dop)                              :: dum
      complex(dop), dimension(sdm), intent(out) :: projector
      logical(kind=4)                           :: linwf,lerr

!.....Set defaults, if pop and state are unset.
      if(pop  .eq.-999)   pop=1      
      if(state.eq.-999) state=1

      filename=rstfile
      lang=index(filename,' ')-1 
      if(lang.lt. 0 .or. lang.ge.64) then
         routine='prj_read (flxprj.F)'
         message='Too long dir-name for projector "read":        '// &
         '              '//rstfile//'          '//                   &
         '                  No more than 63 characters allowed!'
         call errormsg
      elseif(lang.eq.0) then
         routine='prj_read (flxprj.F)'
         message='Empty dir-name for projector "read".'
         call errormsg
      endif
      call abspath(filename,lang)
      filename=filename(1:lang)//'/restart'
      lang=lang+8

      open(irst,file=filename,form='unformatted',status='old',err=999)
      chkdvr=2  ! Check dvrdef 
      chkgrd=2  ! Check grddef
      chkpsi=1  ! Overwrite psidef
      chkprp=0  ! Ignore propagation info
      call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
      if(lerr) then
         routine='prj_read (flxprj.F)'
         message='ERROR in rstinfo'      
         call errormsg
      endif
      call chksyserr(filename,lerr)
      if (dentype .eq. 1) then
         routine='prj_read (flxprj.F)'
         message='Cannot read SPFs from density I restart file.'
         call errormsg
      endif
      if(pop .gt. dim(mode,state)) then
         routine='prj_read (flxprj.F)'
         write(message,'(a,i3,a,i3)') &
               'pop',pop,' .gt. dim',dim(mode,state)
         call errormsg
      endif

!-----------------------------------------------------------------------
! read wavefunction from restart file.
!-----------------------------------------------------------------------
      read (irst,err=998) time
      write(ilog,'(2a)')' rst-file: ',filename(1:lang)
      write(ilog,'(a,i3,a,i3,a,i3,a,f11.3,a)')' prj_read:  mode =',mode,&
         ',  state =',state,',  pop =', pop,',  Time =',  time/fs, ' fs'
      read (irst,err=998) (dum,g=1,zetf(mode,state)-1),&
                          ((projector(g),g=1,sdm),j=1,pop)
!.....Note: The array "projector" actually contains the complex conjugate
!....       of the projecting function. Thus:
      do g = 1, sdm
         projector(g)=dconjg(projector(g))
      enddo

      call getcpsi(1) ! Restore original pointers (overwritten by rstinfo)

      return

  998 routine='prj_read (flxprj.F)'
      message='Error reading restart file'
      call errormsg

  999 routine='prj_read (flxprj.F)'
      message='Cannot open '//filename(1:lang)  
      call errormsg

      end subroutine prj_read

!***********************************************************************

      subroutine prj_sph(qnj,qnm,m,sdm,f,bsf,nsm,gdim,&
                         jsph,msph,projector)

!-----------------------------------------------------------------------
!     The projector is on a (j,m)-state of a sphFBR. 
!-----------------------------------------------------------------------

      implicit none
      
      integer(long)                             :: g
      integer(long), intent(in)                 :: m,sdm,f,bsf,nsm,gdim,qnj,qnm
      integer(long), dimension(sdm), intent(in) :: jsph,msph
      complex(dop), dimension(sdm), intent(out) :: projector


!-----------------------------------------------------------------------
! sdm  = subdim(m)
! gdim = gdim(f) ;   f=dof of theta
! bsf  = basis(f)
! nsm  = nspfdof(m)
! qnj, qnm = quantum-number j, m
!-----------------------------------------------------------------------
      call zerovxz(projector,sdm)        

      if(qnj .lt. 0 ) then
         write(6,'(a,i3)') ' Negative quantum number j. j =', qnj
         stop
      endif
      if(abs(qnm) .gt. qnj ) then
         write(6,'(a,2i3)') ' |m| > j . j, m =', qnj, qnm
         stop
      endif
      if(bsf .ne. 6) then
         write(6,'(a)') ' Wrong mode. Not a spherical harmonics FBR'
         write(6,'(a,3i3)') ' m, f, basis(f) :', m,f,bsf
         stop
      endif
      if(nsm .ne. 2) then
         write(6,'(a)') ' There must be exactly two combined modes:'
         write(6,'(a,i3)')' theta and phi of the sphFBR. nspfdof =',nsm
         stop
      endif
      if(sdm .ne. gdim) then
         write(6,'(a)') ' subdim must be equal gdim for sphFBR.'
         write(6,'(a,2i3)') ' subdim, gdim :', sdm, gdim
         stop
      endif

      g=0
 10   g=g+1
      if (g.gt.sdm) then
         write(6,'(a)')&
              ' Wrong quantum number for spherical harmonics FBR'
         write(6,'(a,i3,a,i3,a,i3)') &
              ' j =',qnj,',  m =',qnm, ',  mode =', m
         stop
      endif
      
      if ((jsph(g).eq.qnj).and.(msph(g).eq.qnm)) then
         projector(g)=(1.0_dop,0.0_dop)
         goto 20
      else
         goto 10
      endif

 20   continue
      
      return
      end subroutine prj_sph


!***********************************************************************

      subroutine prj_diff(qnn,m,f,sdm,gdim,bsf,nsm,pi,ort1,projector)

!-----------------------------------------------------------------------
!     The projector is on a 1D plane-wave. Diffractive scattering.
!-----------------------------------------------------------------------

      implicit none
      
      integer(long)                              :: g
      integer(long), intent(in)                  :: m,sdm,gdim,f,bsf,nsm,qnn
      real(dop)                                  :: a
      real(dop), intent(in)                      :: pi
      real(dop), dimension(gdim), intent(in)     :: ort1
      complex(dop), dimension(gdim), intent(out) :: projector

!-----------------------------------------------------------------------
! sdm  = subdim(m)
! gdim = gdim(f)
! bsf  = basis(f)
! nsm  = nspfdof(m)
! qnn  = quantum-number n
!-----------------------------------------------------------------------
      call zerovxz(projector,sdm)        

      if (abs(qnn) .gt. (gdim+1)/2) then
         write(6,'(a,i3)') &
              ' Illegal value of quantum-number n. n =',qnn
         stop
      endif
      if (nsm .ne. 1) then
         write(6,'(a,/a)') &
              ' Sorry, prj_diff works only for uncombined modes.',&
              ' Use prj_diff2 for 2D combined modes.'
         stop
      endif
      if(bsf .ne. 4  .and.  bsf .ne. 5 ) then
         write(6,'(a)') ' Wrong mode. Not a FFT or exp-DVR'
         write(6,'(a,3i3)') ' m, f, basis(f) :', m,f,bsf
         stop
      endif
      if(sdm .ne. gdim) then
         write(6,'(a)')  ' subdim must be equal gdim for diff.'
         write(6,'(a,2i4)') ' subdim, gdim :', sdm, gdim
         stop
      endif

      a = ort1(gdim) + ort1(2) - 2.0_dop*ort1(1)
      
      do g = 1, gdim
         projector(g) = exp(dcmplx(0.0_dop,-2.0_dop*pi*qnn*ort1(g)/a))&
                         /sqrt(a)
      enddo 
!.....Note: The array "projector" actually contains the complex conjugate 
!....        of the projecting function.
 
      call normspf(projector,sdm,1)      

      return
      end subroutine prj_diff

!***********************************************************************

      subroutine prj_diff2(qnn,qnm,m,f1,f2,sdm,gdim1,gdim2,&
                           bsf1,bsf2,nsm,pi,ort1,ort2,projector)

!-----------------------------------------------------------------------
!     The projector is on a 2D plane-wave. Diffractive scattering.
!-----------------------------------------------------------------------

      implicit none
      
      integer(long)                             :: g,g1,g2
      integer(long), intent(in)                 :: m,sdm,f1,f2,qnn,qnm,gdim1,&
                                                   gdim2,bsf1,bsf2,nsm
      real(dop)                                 :: a,b,c
      real(dop), intent(in)                     :: pi
      real(dop), dimension(gdim1), intent(in)   :: ort1
      real(dop), dimension(gdim2), intent(in)   :: ort2
      complex(dop), dimension(sdm), intent(out) :: projector



!-----------------------------------------------------------------------
! sdm   = subdim(m)
! gdim1 = gdim(f1)
! gdim2 = gdim(f2)
! bsf1  = basis(f1)
! bsf2  = basis(f2)
! nsm   = nspfdof(m)
! qnn   = quantum-number n
! qnm   = quantum-number m
!-----------------------------------------------------------------------
      call zerovxz(projector,sdm)        

      if (abs(qnn) .gt. (gdim1+1)/2) then
         write(6,'(a,i3)')&
              ' Illegal value of quantum-number n. n =',qnn
         stop
      endif
      if (abs(qnm) .gt. (gdim2+1)/2) then
          write(6,'(a,i3)')&
              ' Illegal value of quantum-number m. m =',qnm
         stop
      endif
      if (nsm .ne. 2) then
          write(6,'(a,/a)')&
              ' Sorry, prj_diff2 works only for 2D combined modes.',&
              ' Use prj_diff for uncombined modes.'
         stop
      endif
      if(bsf1 .ne. 4  .and.  bsf1 .ne. 5 ) then
         write(6,'(a,/a,3i3)')&
              ' Wrong DVR of first mode. Not a FFT or exp-DVR',&
              ' m, f1, basis(f1) :', m,f1,bsf1
         stop
      endif
      if(bsf2 .ne. 4  .and.  bsf2 .ne. 5 ) then
         write(6,'(a,/a,3i3)')&
              ' Wrong DVR of second mode. Not a FFT or exp-DVR',&
              ' m, f2, basis(f2) :', m,f2,bsf2
         stop
      endif
      if(gdim1*gdim2 .ne. sdm) then
         write(6,'(a,/a,3i4)')&
              ' Mode Error: gdim(f1)*gdim(f2) .ne. subdim(m)',&
              ' gdim(f1),gdim(f2),subdim(m)', gdim1,gdim2,sdm
         stop
      endif

      a = ort1(gdim1) + ort1(2) - 2.0_dop*ort1(1)
      b = ort2(gdim2) + ort2(2) - 2.0_dop*ort2(1)

      g = 0
      do g1 = 1, gdim1
         do g2 = 1, gdim2
            g = g + 1
            c = qnn*ort1(g1)/a + qnm*ort2(g2)/b 
            projector(g) = exp(dcmplx(0.0_dop,-2.0_dop*pi*c))/sqrt(a*b)
         enddo 
      enddo 
!.....Note: The array "projector" actually contains the complex conjugate 
!....        of the projecting function.

      call normspf(projector,sdm,1)      

      return
      end subroutine prj_diff2

!***********************************************************************


      subroutine prj_pop1(qn,m,f,sdm,projector)

!-----------------------------------------------------------------------
!     The projector is the n-th (FBR) basis function underlying the DVR.
!-----------------------------------------------------------------------

      implicit none

      integer     m,f,sdm,qn
      complex*16  projector(sdm)

!-----------------------------------------------------------------------
!   m  = mode (particle)
!   f  = degree of freedom (DOF)
! sdm  = subdim(m)
! qn   = number of selected grid-point
!-----------------------------------------------------------------------

      if (qn .le. 0  .or. qn .gt. sdm) then
         write(6,'(a,2i4,i6)') ' prj_pop1: m, f, sdm =',m,f,sdm
         write(6,'(a,i5)') ' prj_pop1: Illegal value of grid-point =',qn
         stop
      endif

      call zerovxz(projector,sdm)
      projector(qn) = 1.d0

      return
      end subroutine prj_pop1

!***********************************************************************


      subroutine prj_pop2(qn,m,f,sdm,gdim,ggdim,bsf,nsm,trafo,&
                          projector,ldvr1)

!-----------------------------------------------------------------------
!     The projector is the n-th (FBR) basis function underlying the DVR.
!-----------------------------------------------------------------------

      implicit none
      
      integer(long)                                :: g
      integer(long), intent(in)                    :: m,sdm,gdim,ggdim,f,bsf,nsm,qn
      real(dop), dimension(ggdim,gdim), intent(in) :: trafo
      complex(dop), dimension(gdim), intent(out)   :: projector
      logical(kind=4), intent(in)                  :: ldvr1
!-----------------------------------------------------------------------
!   m  = mode (particle)
!   f  = degree of freedom (DOF)
! sdm  = subdim(m)
! gdim = gdim(f)
! ggdim= 'long' gdim for Leg/R
! bsf  = basis(f)
! nsm  = nspfdof(m)
! qn   = number of selected basis function
!-----------------------------------------------------------------------

      if (qn .le. 0  .or. qn .gt. ggdim) then
         write(6,'(a,2i4,i6)') ' prj_pop2: m, f, sdm =',m,f,sdm
         write(6,'(a,i4)') ' prj_pop2: Illegal value of quantum-number =',qn
         stop
      endif
      if (nsm .ne. 1) then
         write(6,'(a)') &
              ' Sorry, prj_pop2 works only for uncombined modes.'
         stop
      endif
      if(.not.ldvr1) then
         write(6,'(a)') 'Sorry, Projector pop2 works only with DVR'
         write(6,'(a,3i4)') ' m, f, basis(f) :', m,f,bsf
         stop
      endif
      if(sdm .ne. gdim) then
         write(6,'(a)')  ' subdim must be equal gdim for pop2.'
         write(6,'(a,2i4)') ' subdim, gdim :', sdm, gdim
         stop
      endif

      do g = 1, gdim
         projector(g) = trafo(qn,g)
      enddo
 
      return
      end subroutine prj_pop2


!***********************************************************************
      
      subroutine prj_leg(qnj,f,m,sdm,gdim,ggdim,bsf,blz,ibleg,&
                         trafo,projector)

!-----------------------------------------------------------------------
!     The projector is on a 1D Leg-DVR.
!-----------------------------------------------------------------------

      implicit none

      integer(long)                                :: g,jmin,jmax,dj,ij
      integer(long), intent(in)                    :: f,m,sdm,gdim,ggdim,bsf, &
                                                      qnj,blz,ibleg
      real(dop), dimension(ggdim,gdim), intent(in) :: trafo
      complex(dop), dimension(gdim), intent(out)   :: projector

!-----------------------------------------------------------------------
!   m   = mode (particle)
!   f   = DOF  (Degree of freedom)
! sdm   = subdim(m)
! gdim  = gdim(f)   
! ggdim = 'long' gdim for Leg/R
! bsf   = basis(f)
! blz   = m-quantum-number (l_z) of the Leg primitive basis.
! ibleg = Symmetry parameter. 0 -> no sym, 1->odd, 2->even 
! qnj   = quantum number j 
!-----------------------------------------------------------------------
      call zerovxz(projector,sdm)

      if (ibleg.eq.0) then
         jmax = gdim-1+blz
         jmin = blz
         dj   = 1
      else
         jmin = blz + mod(blz+ibleg,2)
         jmax = 2*(ggdim-1) + jmin
         dj   = 2
      endif

      if( bsf.ne.2 .and. bsf.ne.12 ) then 
         write(6,'(2a,i3,a,i3)')  'Sorry, primitive basis must',&
              '  be Leg or Leg/R for DOF=',f,',  mode=',m
         stop
      endif

      if (sdm .ne. gdim) then
         write(6,'(2a,2i4,/a,i3,a,i3)') &
              'Leg: ERROR in grid dimensions.',&
              ' subdim .ne. gdim :', sdm, gdim,&
              'Seems to be a combined mode. mode=',m,', DOF=',f
         stop
      endif

      if (qnj.lt.jmin .or. qnj.gt.jmax .or. mod(qnj-jmin,dj).ne.0) then
         write(6,'(a,i3)')&
              'Leg: Illegal value of quantum-number j. j =',qnj
         stop
      endif

!.....Transform quantum numbers into indices.
      ij = (qnj-jmin)/dj + 1

      do g = 1, gdim
         projector(g) = trafo(ij,g)
      enddo

      return
      end subroutine prj_leg


!***********************************************************************
      
      subroutine prj_kleg(qnj,qnm,m,sdm,gdim1,ggdim1,gdim2,bsf1,bsf2,&
                          nsm,ibleg,kmin,dk,trafo,projector)

!-----------------------------------------------------------------------
!     The projector is on a 2D KLeg-DVR.
!-----------------------------------------------------------------------

      implicit none

      integer(long)                                        :: g,jmin,jmax,dj,im,ij
      integer(long), intent(in)                            :: m,sdm,gdim1,ggdim1,gdim2,bsf1,bsf2,&
                                                              nsm,qnj,qnm,ibleg,kmin,dk
      real(dop), dimension(ggdim1,gdim1,gdim2), intent(in) :: trafo
      complex(dop), dimension(gdim1,gdim2), intent(out)    :: projector

!-----------------------------------------------------------------------
!   m   = mode (particle)
! sdm   = subdim(m)
! gdim1 = gdim(f)   (Leg)
! ggdim1 = 'long' gdim for KLeg/R
! ggdi2 = gdim(f+1) (K)
! bsf   = basis(f)
! nsm   = nspfdof(m)
! qnj   = quantum number j 
! qnm   = quantum number m
! dk    = delta-k (step in qn m)
!-----------------------------------------------------------------------
      call zerovxz(projector,sdm)

      if (ibleg.eq.0) then
         jmax = gdim1-1+abs(qnm)
         jmin = abs(qnm)
         dj   = 1
      else
         jmin = abs(qnm) + mod(abs(qnm)+ibleg,2)
         jmax = 2*(ggdim1-1) + jmin
         dj   = 2
      endif

      if( bsf1.ne.7  .and. bsf1.ne.20 .and.  bsf2.ne.8) then 
         write(6,'(2a,i3)')  'Sorry, primitive basis must',&
              '  be KLeg or KLeg/r for mode', m
         stop
      endif
      if (nsm .ne. 2) then
         write(6,'(a)')&
              'Sorry, projector KLeg works only for 2D combined modes.'
         stop
      endif
      if (qnj.lt.jmin .or. qnj.gt.jmax .or. mod(qnj-jmin,dj).ne.0) then
         write(6,'(a,i3)')&
              'KLeg: Illegal value of quantum-number j. j =',qnj
         stop
      endif
      if (abs(qnm) .gt. jmax  .or.  qnm .lt. kmin  .or. &
          abs(qnm) .gt. qnj   .or.  qnm .gt. (kmin+gdim2-1)) then
         write(6,'(a,i3)')&
              'KLeg: Illegal value of quantum-number m. m =',qnm
         stop
      endif
      if (sdm .ne. gdim1*gdim2) then
         write(6,'(2a,3i4)') 'KLeg: ERROR in grid dimensions.', &
         ' subdim, gdim1, gdim2 :', sdm, gdim1, gdim2
         stop
      endif

!.....Transform quantum numbers into indices.
      im = (qnm-kmin)/dk + 1
      ij = (qnj-jmin)/dj + 1

      do g = 1, gdim1
         projector(g,im) = trafo(ij,g,im)
      enddo

      return
      end subroutine prj_kleg


!***********************************************************************
      
      subroutine prj_pleg(qnj,qnm,m,sdm,gdim1,ggdim1,gdim2,bsf1,bsf2,&
                          nsm,ibleg,kmin,ort,trafo,projector)

!-----------------------------------------------------------------------
!     The projector is on a 2D PLeg-DVR.
!-----------------------------------------------------------------------

      implicit none

      integer(long)                                        :: g,k,jmin,jmax,dj,im,ij
      integer(long), intent(in)                            :: m,sdm,gdim1,ggdim1,gdim2,bsf1,bsf2,&
                                                              nsm,qnj,qnm,ibleg,kmin
      real(dop), dimension(gdim2), intent(in)              :: ort
      real(dop), dimension(ggdim1,gdim1,gdim2), intent(in) :: trafo
      complex(dop), dimension(gdim1,gdim2), intent(out)    :: projector

!-----------------------------------------------------------------------
!   m   = mode (particle)
! sdm   = subdim(m)
! gdim1 = gdim(f)   (Leg)
! ggdim1 = 'long' gdim for PLeg/R
! ggdi2 = gdim(f+1) (K)
! bsf   = basis(f)
! nsm   = nspfdof(m)
! qnj   = quantum number j 
! qnm   = quantum number m
!-----------------------------------------------------------------------
      call zerovxz(projector,sdm)

      if (ibleg.eq.0) then
         jmax = gdim1 - 1 + abs(qnm)
         jmin = abs(qnm)
         dj   = 1
      else
         jmin = abs(qnm) + mod(abs(qnm)+ibleg,2)
         jmax = 2*(ggdim1-1) + jmin
         dj   = 2
      endif

      if( bsf1.ne.9 .and. bsf1.ne.21 .and.  bsf2.ne.5) then 
         write(6,'(2a,i3)')  'Sorry, primitive basis must', &
              ' be PLeg for mode', m
         stop
      endif
      if (nsm .ne. 2) then
         write(6,'(a)')&
              'Sorry, projector PLeg works only for 2D combined modes.'
         stop
      endif
      if (qnj.lt.jmin .or. qnj.gt.jmax .or. mod(qnj-jmin,dj).ne.0) then
         write(6,'(a,i3)')&
              'PLeg: Illegal value of quantum-number j. j =',qnj
         stop
      endif
      if (abs(qnm) .gt. jmax  .or.  qnm .lt. kmin  .or. &
          abs(qnm) .gt. qnj   .or.  qnm .gt. (kmin+gdim2-1)) then
         write(6,'(a,i3)')&
              'PLeg: Illegal value of quantum-number m. m =',qnm
         stop
      endif
      if (sdm .ne. gdim1*gdim2) then
         write(6,'(2a,3i4)') ' PLeg: ERROR in grid dimensions.', &
         ' subdim, gdim1, gdim2 :', sdm, gdim1, gdim2
         stop
      endif
      
!.....Transform quantum numbers into indices.
      im = (qnm-kmin)    + 1
      ij = (qnj-jmin)/dj + 1 

      do k = 1, gdim2
         do g = 1, gdim1
            projector(g,k) = trafo(ij,g,im)*&
                 exp(dcmplx(0.0_dop,-qnm*ort(k)))/sqrt(dble(gdim2))
         enddo
      enddo
!.....Note: The array "projector" actually contains the complex conjugate 
!....        of the projecting function.

      return
      end subroutine prj_pleg


!***********************************************************************
!
! PRJ_KLEG2: projection on KLeg/K/KLeg/K
!
! j1,m1: quantum numbers for first angle
! j2,m2: quantum numbers for second angle
! isym : > 0 : symmetrize state
!        < 0 : antisymmetrize state
!        = 0 : do nothing
! m    : mode (particle) number
! gdim1: gdim(first KLeg DOF)
! kdim1: gdim(first K DOF)
! gdim2: gdim(second KLeg DOF)
! kdim2: gdim(second K DOF)
! trafo1,trafo2: trafo matrices
! projector: holds the result
!
! FO 06/2006
!
!***********************************************************************

      subroutine prj_kleg2(j1,m1,j2,m2,isym,m,&
                           gdim1,kdim1,gdim2,kdim2,&
                           trafo1,trafo2,projector)

!-----------------------------------------------------------------------
!     The projector is on a 4D mode (KLeg/K/Kleg/K).
!-----------------------------------------------------------------------

use global
use dvrdatmod
use griddatmod

use rdopermod, only: get_nth_kleg

      implicit none

      integer(long), intent(in)                                     :: j1,m1,j2,m2,isym,m,gdim1,&
                                                                       kdim1,gdim2,kdim2
      real(dop), dimension(gdim1,gdim1,kdim1), intent(in)           :: trafo1
      real(dop), dimension(gdim2,gdim2,kdim2), intent(in)           :: trafo2
      complex(dop), dimension(gdim1,kdim1,gdim2,kdim2), intent(out) :: projector

      integer(long)       :: sdm,f1,f2,ibleg1,ibleg2,i,&
                             j1min,j1max,j2min,j2max,dj1,dj2,&
                             k1min,k1max,k2min,k2max,dk1,dk2,&
                             ij1,ij2,im1,im2,g1,g2
      logical(kind=4)     :: lchkerr
      real(dop)           :: fac,sign

      sdm = subdim(m)
      call get_nth_kleg(f1,1,m)
      call get_nth_kleg(f2,2,m)

!     copied from PRJ_KLEG ...
      ibleg1 = ipbaspar(1,f1)
      ibleg2 = ipbaspar(1,f2)
      if (ibleg1.eq.0) then
         j1max = gdim1 - 1 + abs(m1)
         j1min = abs(m1)
         dj1   = 1
      else
         j1min = abs(m1) + mod(abs(m1)+ibleg1,2)
         j1max = 2*(gdim1-1) + j1min
         dj1   = 2
      endif
      if (ibleg2.eq.0) then
         j2max = gdim2 - 1 + abs(m2)
         j2min = abs(m2)
         dj2   = 1
      else
         j2min = abs(m2) + mod(abs(m2)+ibleg2,2)
         j2max = 2*(gdim2-1) + j2min
         dj2   = 2
      endif

      k1min = ipbaspar(1,f1+1)
      k1max = ipbaspar(2,f1+1)
      dk1   = max(ipbaspar(5,f1+1),1)
      k2min = ipbaspar(1,f2+1)
      k2max = ipbaspar(2,f2+1)
      dk2   = max(ipbaspar(5,f2+1),1)

!     some checks
      if (basis(f1).ne.7 .and. basis(f1).ne.20 .and. &
          basis(f1+1).ne.8) then
         write (6,'(a,i3)')&
            'Sorry, primitive basis must be KLeg for DOF ',f1
         stop
      endif
      if (basis(f2).ne.7 .and. basis(f2).ne.20 .and. &
          basis(f2+1).ne.8) then
         write (6,'(a,i3)')&
            'Sorry, primitive basis must be KLeg for DOF ',f2
         stop
      endif

      if (nspfdof(m).ne.4) then
         write (6,'(a)')&
            'Sorry, projector KLeg2 only works for 4D modes.'
         stop
      endif

!     more checks, copied from PRJ_KLEG ...
      if (j1.lt.j1min .or. j1.gt.j1max .or. mod(j1-j1min,dj1).ne.0) then
         write(6,'(a,i3,a,i3)')&
              'KLeg2: Illegal value of quantum-number j =',j1,&
              ' for DOF ',f1
         stop
      endif
      if (j2.lt.j2min .or. j2.gt.j2max .or. mod(j2-j2min,dj2).ne.0) then
         write(6,'(a,i3,a,i3)')&
              'KLeg2: Illegal value of quantum-number j =',j2,&
              ' for DOF ',f2
         stop
      endif

      if (abs(m1).gt.j1 .or. m1.lt.k1min .or. m1.gt.k1max .or.&
          mod(m1-k1min,dk1).ne.0) then
         write(6,'(a,i3)')&
              'KLeg2: Illegal value of quantum-number m =',m1,&
              ' for DOF ',f1+1
         stop
      endif
      if (abs(m2).gt.j2 .or. m2.lt.k2min .or. m2.gt.k2max .or.&
          mod(m2-k2min,dk2).ne.0) then
         write(6,'(a,i3)')&
              'KLeg2: Illegal value of quantum-number m =',m2,&
              ' for DOF ',f2+1
         stop
      endif

      if (sdm .ne. gdim1*kdim1*gdim2*kdim2) then
         write(6,'(a/a,5i4)') 'KLeg2: ERROR in grid dimensions.', &
         'subdim,gdim1,kdim2,gdim2,kdim2 :',sdm,gdim1,kdim1,gdim2,kdim2
         stop
      endif

!     for (anti)symmetrization, the primitive basis must match
      if (isym.ne.0) then
         lchkerr=.false.
         if (gdim1.ne.gdim2 .or.&
             kdim1.ne.kdim2 .or.&
             ibleg1.ne.ibleg2) lchkerr=.true.
         do i=1,mbaspar
            if (ipbaspar(i,f1).ne.ipbaspar(i,f2) .or.&
                ipbaspar(i,f1+1).ne.ipbaspar(i,f2+1)) lchkerr=.true.
         enddo
         if (lchkerr) then
            write (6,'(a)') 'KLeg2: the primitive basis for the '//&
               'two KLegs do not match.'
            stop
         endif
      endif

!     for antisymmetrization, the projector state must not be symmetric
      if (isym.lt.0) then
         if (j1.eq.j2 .and. m1.eq.m2) then
            write (6,'(a)') 'KLeg2: projector state is already '//&
               'symmetric and cannot be antisymmetrized.'
            stop
         endif
      endif

!     transform quantum numbers into indices
      im1 = (m1-k1min)/dk1 + 1
      ij1 = (j1-j1min)/dj1 + 1
      im2 = (m2-k2min)/dk2 + 1
      ij2 = (j2-j2min)/dj2 + 1

!     calculate projector
      call zerovxz(projector,sdm)
      if (isym.eq.0 .or.&
          (j1.eq.j2 .and. m1.eq.m2)) then
!        no symmetry, or already symmetric
         do g1=1,gdim1
            do g2=1,gdim2
               projector(g1,im1,g2,im2) =&
                    trafo1(ij1,g1,im1) * trafo2(ij2,g2,im2)
            enddo
         enddo

      else
!        (anti)symmetrize
         if (isym.gt.0) then
            sign = 1.0_dop
         else
            sign = -1.0_dop
         endif
         fac = sqrt(0.5_dop)
         do g1=1,gdim1
            do g2=1,gdim2
               projector(g1,im1,g2,im2) = projector(g1,im1,g2,im2) +&
                    trafo1(ij1,g1,im1) * trafo2(ij2,g2,im2) * fac
               projector(g1,im2,g2,im1) = projector(g1,im2,g2,im1) +&
                    trafo1(ij1,g2,im1) * trafo2(ij2,g1,im2) * fac * sign
            enddo
         enddo

      endif

      return
      end subroutine prj_kleg2 ! of prj_kleg2

!***********************************************************************
!         PRJ_WIGNER
!
!     The projector is on a 3D Wigner-DVR.
!-----------------------------------------------------------------------
!   m   = mode (particle)
! sdm   = subdim(m)
! gdim1 = gdim(f)   (Leg)
! ggdi2 = gdim(f+1) (K or exp)
! ggdi3 = gdim(f+2) (K or exp)
! bsf1  = basis(f)
! bsf2  = basis(f+1)
! bsf3  = basis(f+2)
! nsm   = nspfdof(m)
! qnj   = quantum number j
! qnk   = quantum number k
! qnm   = quantum number m
! dk    = delta-k (step in qn k)
! dm    = delta-m (step in qn m)
!-----------------------------------------------------------------------

      subroutine prj_Wigner(qnj,qnk,qnm,m,sdm,gdim1,gdim2,gdim3,bsf1, &
                            bsf2,bsf3,nsm,ibleg,kmin,mmmin,dk,dm, &
                            trafo,ort2,ort3,projector)

      implicit none

      integer(long) ::    m,sdm,gdim1,gdim2,gdim3,g,bsf1,bsf2,bsf3,nsm
      integer(long) ::    qnj,qnk,qnm,kmin,mmmin,jmin,jmax,dj,dk,dm
      integer(long) ::    ibleg,im,ik,ij,mormm,k,mm
      real(dop), dimension(gdim1,gdim1,gdim2,gdim3) :: trafo
      real(dop), dimension(gdim2)                   :: ort2
      real(dop), dimension(gdim3)                   :: ort3
      complex(dop), dimension(gdim1,gdim2,gdim3)    :: projector
      complex(dop)                                  :: e1,e2

      call zerovxz(projector,sdm)

      mormm = max(abs(qnk),abs(qnm))
      if (ibleg.eq.0) then
         jmax = gdim1-1+mormm
         jmin = mormm
         dj   = 1
      else
         jmin = mormm + mod(mormm+ibleg,2)
         jmax = 2*(gdim1-1) + jmin
         dj   = 2
      endif

      if (bsf1.ne.33) then
         write(6,'(2a,i3)')  'Primitive basis must','  be Wigner for mode', m
         stop
      endif
      if (nsm.ne.3) then
         write(6,'(a)') 'Projector Wigner works only for 3D combined modes.'
         stop
      endif
      if (qnj.lt.jmin .or. qnj.gt.jmax .or. mod(qnj-jmin,dj).ne.0) then
         write(6,'(a,i3)') 'Wigner: Illegal value of quantum-number j. j =',qnj
         stop
      endif
      if (abs(qnk).gt.qnj .or. qnk.lt.kmin .or. qnk.gt.(kmin+dk*(gdim2-1))) then
         write(6,'(a,i3)') 'Wigner: Illegal value of quantum-number k. k =',qnk
         stop
      endif
      if (abs(qnm).gt.qnj .or. qnm.lt.mmmin .or. qnm.gt.(mmmin+dm*(gdim3-1))) then
         write(6,'(a,i3)') 'Wigner: Illegal value of quantum-number m. m =',qnm
         stop
      endif
      if (sdm .ne. gdim1*gdim2*gdim3) then
         write(6,'(2a,4i4)') 'Wigner: ERROR in grid dimensions.', &
            ' subdim, gdim1, gdim2 :', sdm, gdim1, gdim2, gdim3
         stop
      endif

!.....Transform quantum numbers into indices.
      ik = (qnk-kmin)/dk + 1
      im = (qnm-mmmin)/dm + 1
      ij = (qnj-jmin)/dj + 1

!.....Note: The array "projector" actually contains the complex conjugate
!....        of the projecting function for exp DVR (bsf=5).
      if (bsf2.eq.5 .and. bsf3.eq.5) then
         do mm = 1, gdim3
            e1 = exp(dcmplx(0.d0,-qnm*ort3(mm)))/sqrt(dble(gdim3))
            do k = 1, gdim2
               e2 = exp(dcmplx(0.d0,-qnk*ort2(k)))/sqrt(dble(gdim2))
               do g = 1, gdim1
                  projector(g,k,mm) = trafo(ij,g,ik,im)*e1*e2
               enddo
            enddo
         enddo
      elseif (bsf2.eq.5 .and. bsf3.eq.8) then
         do k = 1, gdim2
            e2 = exp(dcmplx(0.d0,-qnk*ort2(k)))/sqrt(dble(gdim2))
            do g = 1, gdim1
               projector(g,k,im) = trafo(ij,g,ik,im)*e2
            enddo
         enddo
      elseif (bsf2.eq.8 .and. bsf3.eq.5) then
         do mm = 1, gdim3
            e1 =  exp(dcmplx(0.d0,-qnm*ort3(mm)))/sqrt(dble(gdim3))
            do g = 1, gdim1
               projector(g,ik,mm) = trafo(ij,g,ik,im)*e1
            enddo
         enddo
      elseif (bsf2.eq.8 .and. bsf3.eq.8) then
         do g = 1, gdim1
            projector(g,ik,im) = trafo(ij,g,ik,im)
         enddo
      else
         write(6,'(2a,i3)')  'Primitive basis must','  be K or exp for mode', m
         stop
      endif

      return
      end subroutine prj_Wigner

!***********************************************************************

      subroutine prj_p1d(ip1,ip2,m,f,sdm,gdim,bsf,nsm,ort1,projector,weight)

!-----------------------------------------------------------------------
!     The projector is on a 1D plane-wave. Projection on momentum p.
!-----------------------------------------------------------------------

      implicit none

      integer(long) :: m, sdm, gdim, g, f, bsf,nsm, ip1, ip2
      real(dop)     :: ort1(gdim), p, weight(gdim)
      complex(dop)  :: projector(gdim)

!-----------------------------------------------------------------------
! sdm  = subdim(m)
! gdim = gdim(f)
! bsf  = basis(f)
! nsm  = nspfdof(m)
!-----------------------------------------------------------------------
      call zerovxz(projector,sdm)

      if (nsm .ne. 1) then
         write(6,'(a,/a)') &
              ' Sorry, prj_p1D works only for uncombined modes.', &
              ' Use prj_p-2D for 2D combined modes.'
         stop
      endif
      if(bsf.eq.0 .or. bsf.eq.2 .or. (bsf.ge.6 .and. bsf.le.9) .or. &
                                                     bsf.eq.33) then
         write(6,'(2a)') ' prj_p1D . Wrong DVR: ', &
          'el, Leg, sphfbr, kleg, k, pleg, Wigner are not allowed!'
         write(6,'(a,3i3)') ' m, f, basis(f) :', m,f,bsf
         stop
      endif
      if(sdm .ne. gdim) then
         write(6,'(a)')  ' subdim must be equal gdim for p-1D.'
         write(6,'(a,2i4)') ' subdim, gdim :', sdm, gdim
         stop
      endif

      p=dble(ip1)/dble(ip2)
      do g = 1, gdim
         projector(g) = exp(dcmplx(0.d0,-p*ort1(g)))*weight(g)
      enddo
!.....Note: The array "projector" actually contains the complex conjugate
!....        of the projecting function.

      call normspf(projector,sdm,1)

      return
      end subroutine prj_p1d

!***********************************************************************

      subroutine prj_p2d(ip1,ip2,ip3,m,f1,f2,sdm,gdim1,gdim2, &
                         bsf1,bsf2,nsm,ort1,ort2,projector,   &
                         weight1,weight2)

!-----------------------------------------------------------------------
!     The projector is on a 2D plane-wave. Projection on momentum (p1,p2)
!-----------------------------------------------------------------------

      implicit none

      integer     m,sdm,g,g1,g2,f1,f2,gdim1,gdim2,bsf1,bsf2,nsm
      integer     ip1,ip2,ip3
      real*8      c, ort1(gdim1),ort2(gdim2), p1, p2
      real*8      weight1(gdim1), weight2(gdim2)
      complex*16  projector(sdm)

!-----------------------------------------------------------------------
! sdm   = subdim(m)
! gdim1 = gdim(f1)
! gdim2 = gdim(f2)
! bsf1  = basis(f1)
! bsf2  = basis(f2)
! nsm   = nspfdof(m)
!-----------------------------------------------------------------------
      call zerovxz(projector,sdm)
      if (nsm .ne. 2) then
          write(6,'(a,/a)') &
              ' Sorry, prj_p2D works only for 2D combined modes.', &
              ' Use prj_p1D for uncombined modes.'
         stop
      endif
      if(bsf1 .ne. 4  .and.  bsf1 .ne. 5 ) then
         write(6,'(a,/a,3i3)') &
              ' Wrong DVR of first mode. Not a FFT or exp-DVR', &
              ' m, f1, basis(f1) :', m,f1,bsf1
         stop
      endif
      if(bsf2 .ne. 4  .and.  bsf2 .ne. 5 ) then
         write(6,'(a,/a,3i3)') &
              ' Wrong DVR of second mode. Not a FFT or exp-DVR', &
              ' m, f2, basis(f2) :', m,f2,bsf2
         stop
      endif
      if(gdim1*gdim2 .ne. sdm) then
         write(6,'(a,/a,3i4)') &
              ' Mode Error: gdim(f1)*gdim(f2) .ne. subdim(m)', &
              ' gdim(f1),gdim(f2),subdim(m)', gdim1,gdim2,sdm
         stop
      endif

      p1=dble(ip1)/dble(ip3)
      p2=dble(ip2)/dble(ip3)
      g = 0
      do g1 = 1, gdim1
         do g2 = 1, gdim2
            g = g + 1
            c = p1*ort1(g1) + p2*ort2(g2)
            projector(g) = exp(dcmplx(0.d0,-c))*weight1(g1)*weight2(g2)
         enddo
      enddo
!.....Note: The array "projector" actually contains the complex conjugate
!....        of the projecting function.

      call normspf(projector,sdm,1)

      return
      end subroutine prj_p2d


      end module flxprj
