!***********************************************************************
!
!                         AUSWERT MODULE FLXREDUCE
!
!     This file contains reduce routines needed for the use of projectors.
!     These subroutines are used in flux85 and wfproj85
!
!     Contens: newvndim, reduce, reduceml
!
! HDM Jul 2020
!
!***********************************************************************

     module flxreduce

use versions, only: progver
use decimal, only: long,dop,sip

use global 
use aglobal 
use maxv
use griddatmod 
use psidef 
use datenmod 
use compdatmod 
use compdat1mod 
use operdef 
use fluxmod 
use channels 

use compdat, only: getcpsi,getcsys
use iofile, only: rdpsiinfo,wrpsiinfo
use rdfilesmod, only: rdpsi
use zeighpsimod, only: operflag,zeighpsi
use auswutil, only: getcdat
use flxutils, only: mexpand

use op1lib, only: initvxl,cpvxz,unitqxz,zerovxz
use xvlib, only: mvxxzz,mvtxzz
use mmlib, only: mmxxzz,mmxtzz
use mtlib, only: mtxxzz

      use wftree
      use mlanalysis
      use multilayer

      implicit none
      private
      public :: reduce,newvndim,reduceml
      contains

!***********************************************************************

      subroutine reduce(projector,psi,psi1,spsi,jindx,dover,&
                        workc,fullpsi)

!***********************************************************************
!              REDUCE
! Computes the projected (reduced) wavefunction and writes it to file.
! Special treatment for 1D and 2D reduced wawefunctions.
!***********************************************************************

      implicit none

      integer(long)                                 :: n,m,s,ss,prvdim,&
                                                       prndim,zeig,mm,h,k,&
                                                       i,k1,k2,kk2,nndof, &
                                                       np,sdm,chk,m1,m2,&
                                                       cmcap=0
      integer(long), dimension(1)                    :: idum,idum1
      integer(long), dimension(maxdim)               :: mold
      integer(long), dimension(maxsta)               :: zeige
      integer(long), dimension(adim), intent(out)    :: jindx
      complex(dop), dimension(1)                     :: zdum,zdum1
      complex(sip), dimension(spsidim), intent(out)  :: spsi
      complex(dop), dimension(dgldim), intent(out)   :: psi1,psi
      complex(dop), dimension(doverdim), intent(out) :: dover
      complex(dop), dimension(workcdim), intent(out) :: workc
      complex(dop), dimension(prjdim), intent(in)    :: projector
      complex(dop), dimension(fulldim), intent(out)  :: fullpsi
      logical(kind=4), dimension(maxdim)             :: todo
      character(len=c2)                              :: wrtag

!-----------------------------------------------------------------------
! Consistency Check.
!-----------------------------------------------------------------------
         do np  = 1, nprj
            if( prjmode(np) .eq. mcap ) then
               write(6,'(a,/,a,3i5)') &
                    'Projection onto the CAP-mode is not allowed!', &
                    'np, prjmode, mcap =', np, prjmode(np), mcap
               stop
            endif
         enddo

!-----------------------------------------------------------------------
! New pointers need not to be determined if full.
!-----------------------------------------------------------------------
       cblock(1,2)=1
       if(full .and. nstate.eq.1) goto 120

!-----------------------------------------------------------------------
! Determine the new pointers.
! phidim(m) is length of spf vectors for mode m
!-----------------------------------------------------------------------
      call initvxl(todo,nmode)
      do np = 1, nprj
         todo(prjmode(np)) = .false.
      enddo
      do s = 1, nstate
         zeige(s) = 1
      enddo
      mm   = 0
      nndof= 0
      do m = 1, nmode
         do s = 1, nstate
            cdim(m,s,2)   = 0
            cvdim(m,s,2)  = 0
            cndim(m,s,2)  = 0
         enddo
         cdofspf(m,2)  = 0
         cnspfdof(m,2) = 0
         csubdim(m,2)  = 0
         phidim(m)     = 0
         do n=1,nspfdof(m)
            cspfdof(n,m,2)=0
         enddo
         if(todo(m)) then
            mm = mm + 1
            mold(mm) = m
            if(m.eq.mcap) cmcap=mm
            cnspfdof(mm,2) = nspfdof(m)
            csubdim(mm,2)  = subdim(m)
            do n=1,nspfdof(m)
               nndof=nndof+1
               cspfdof(n,mm,2)=spfdof(n,m)
               cdofspf(spfdof(n,m),2)= mm
            enddo
            ss=0
            do s = nst1, nst2
               ss=ss+1
               cvdim(mm,ss,2) = zeige(ss)
               zeige(ss)=zeige(ss)*dim(m,s)
               cdim(mm,ss,2) = dim(m,s)
            enddo
            if(loper) then
               ss=0
               do s = nst1, nst2
                  ss=ss+1
                  khzahl(mm,ss,nham) = khzahl(m,s,nham) 
                  zhun  (mm,ss,nham) = zhun  (m,s,nham) 
               enddo
               do k = 1, ktot
                  hlsym(mm,k) = hlsym(m,k)
                  hsym (mm,k) = hsym (m,k)
                  diag (mm,k) = diag (m,k)
               enddo 
               do h = 1, htmdim
                  if(htmmode(h) .eq. m) htmmode(h) = mm
               enddo 
            endif
         endif
      enddo

      cnmode(2)  = mm
      cndof(2)   = nndof
      cadim(2)   = 0
      cnstate(2) = nst2-nst1+1
      zeig=1
      ss=0
      do ss = 1, cnstate(2)
         cblock(ss,2) = zeige(ss)
         cadim(2)     = cadim(2) +  zeige(ss)
         czpsi(ss,2)  = zeig
         zeig = zeig + cblock(ss,2)
      enddo

      if(full) goto 120

!.....Special case: 2D reduced system. (Do nothing for equal dim's).
      if(cnmode(2) .eq. 2 .and. nstate.eq.1) then
         s=1
         if(cdim(1,s,2) .lt. cdim(2,s,2)) then
            l2short = .true.
            cdim(2,s,2) = cdim(1,s,2)
         elseif(cdim(1,s,2) .gt. cdim(2,s,2)) then
            l2short = .true.
            cdim(1,s,2) = cdim(2,s,2)
         endif
         cvdim(2,s,2) = cdim(1,s,2)
         zeig         = cdim(1,s,2)*cdim(2,s,2)
         cadim(2)     = zeig 
         cblock(s,2)  = zeig
      endif

      do ss = 1, cnstate(2)
         do mm=1,cnmode(2)
            cndim(mm,ss,2)=cblock(ss,2)/(cdim(mm,ss,2)*cvdim(mm,ss,2))
         enddo
      enddo

      zeig = cadim(2) + 1
      do mm = 1, cnmode(2)
         do ss = 1, nst2-nst1+1
            czetf(mm,ss,2) = zeig
            zeig = zeig + csubdim(mm,2)*cdim(mm,ss,2)
            phidim(mm) = phidim(mm) + csubdim(mm,2)*cdim(mm,ss,2)
         enddo
      enddo
      cdgldim(2) = zeig - 1 
 
      zeig=1
      do ss = 1, cnstate(2)
         do mm=1,cnmode(2)
            dovmat(mm,ss)=zeig 
            zeig=zeig+cdim(mm,ss,2)*cdim(mm,ss,2)
         enddo
      enddo

      filever(ipsi1) = progver 
      clpsiopt(1,2)  = .false.  ! Not in single precision.
      clpsiopt(2,2)  = .false.  ! Not in compact mode.
      clpsiopt(3,2)  = .false.  ! Not in natorb  mode.
      clpsiopt(4,2)  = .false.  ! Not in select  mode.

      call getcsys(2)           ! Load the new pointers.
      call getcpsi(2)          
      call getcdat(2)
      if(laeout.eq.0) then
         laeout=1 
         fileout(1:1)='.'
      endif    
      open(ipsi1,file=fileout(1:laeout)//'/prjpsi',&
           status='unknown',form='unformatted')      
      write(ipsi1) filever(ipsi1)
      call wrpsiinfo(ipsi1)

!-----------------------------------------------------------------------
! Read psi(t).
!-----------------------------------------------------------------------
      call getcsys(1)           ! Restore the old pointers.
      call getcpsi(1)        
      call getcdat(1)
 120  k2  = 0                   ! Shortcut for If(full)
      chk = 0
      rewind (ipsi)
      read (ipsi)
      if (filever(ipsi) .lt. 11.0_dop) call rdpsiinfo(ipsi,chk,chk,chk,chk)
      
      do k1=1,klo-1
         call rdpsi(ipsi,psi,spsi,jindx,zdum,idum,zdum1,idum1)
      enddo
      do k1=klo,khi,step
         dgldim=dgldimold
         call rdpsi(ipsi,psi,spsi,jindx,zdum,idum,zdum1,idum1)
         k2 = k2 + 1
         if (lend) goto 77

!------------------------------------------------------------------------
! Expand psi1 to full MCTDH-length in case of identical particles.
!------------------------------------------------------------------------
         if(lid) call mexpand(psi)

!-----------------------------------------------------------------------
! Copy A-vector of psi into psi1 and initialize logical todo.
!-----------------------------------------------------------------------
         call cpvxz(psi,psi1,adim)
         call initvxl(todo,nmode)

!-----------------------------------------------------------------------
! Reduce A-vector mode after mode.
!-----------------------------------------------------------------------
         do np  = 1, nprj
            m   = prjmode(np)
            sdm = subdim(m)
            ss=0 
            do s = nst1, nst2
               ss=ss+1

!..compute projector-vor-dim and projector-nach-dim (prvdim,prndim).
               call newvndim(prvdim,prndim,m,todo,s)

!..compute the overlap <projector|psi> for mode m.
!..Note: The array projector contains the complex-conjugate of the function.
               call mvtxzz(psi(zetf(m,s)),projector(zprj(np)),&
                    dover,sdm,dim(m,s))

!..Reduction of the A-vector. The reduced vector is written to workc.
               call mtxxzz(dover,psi1(zpsi(s)),workc,prvdim,&
                           dim(m,s),prndim,onei)

!.....Copy reduced A-vector to psi1.
               if(np .eq. nprj) then
                  call cpvxz(workc,psi1(czpsi(ss,2)),prvdim*prndim)
               else
                  call cpvxz(workc,psi1(zpsi(s)),prvdim*prndim)
               endif

            enddo               ! end do-loop over states.
         enddo                  ! end do-loop over np, i.e. the projectors.

!-----------------------------------------------------------------------
! Store the reduced wavefunction into psi1 and write do disc.
! If(full) store (after multiplication with A) into fullpsi.
!-----------------------------------------------------------------------
         if(full) then
!...1D reduced WF
            ss=0
            kk2 = (k2-1)*sdmc*(nst2-nst1+1) + 1
            do s = nst1, nst2
               ss=ss+1
               call mvxxzz(psi(zetf(mcap,s)),psi1(czpsi(ss,2)),&
                           fullpsi(kk2),sdmc,dim(mcap,s))
               kk2 = kk2 + sdmc
            enddo 
!...Reset nstate. Only the relevant states are stored in fullpsi.
            nstate = cnstate(2)

         elseif(l2short) then
!...2D reduced WF (with dim(m1) .ne. dim(m2)  .and. nstate=1)
            m1 = mold(1)
            m2 = mold(2)
            s  = 1
            call unitqxz(psi1,cdim(1,s,2)) ! Load unit-matrix into psi1
            if(dim(m1,s).lt.dim(m2,s)) then 
               call cpvxz(psi(zetf(m1,s)),psi1(czetf(1,s,2)),&
                    subdim(m1)*dim(m1,s))  
               call mmxtzz(psi(zetf(m2,s)),workc,psi1(czetf(2,s,2)),&
                    subdim(m2),dim(m2,s),dim(m1,s))
            elseif(dim(m1,s).gt.dim(m2,s)) then
               call mmxxzz(psi(zetf(m1,s)),workc,psi1(czetf(1,s,2)),&
                    subdim(m1),dim(m1,s),dim(m2,s))
               call cpvxz(psi(zetf(m2,s)),psi1(czetf(2,s,2)),&
                    subdim(m2)*dim(m2,s))  
            endif

            write(wrtag,'(a9,i10,a1)') '<data set',k1,'>'
            write(ipsi1) wrtag
            wrtag='<psi>'
            write(ipsi1) wrtag
            write(ipsi1) (psi1(i), i=1,cdgldim(2))
            write(wrtag,'(a13,i10,a1)') '<end data set',countpsi,'>'
            write(ipsi1) wrtag

         else
!...general case
            ss=0 
            do s = nst1, nst2
               ss=ss+1
               do mm = 1, cnmode(2)
                  m = mold(mm)
                  call cpvxz(psi(zetf(m,s)),psi1(czetf(mm,ss,2)),&
                       subdim(m)*dim(m,s))  
               enddo
            enddo

            write(wrtag,'(a9,i10,a1)') '<data set',k1,'>'
            write(ipsi1) wrtag
            wrtag='<psi>'
            write(ipsi1) wrtag
            write(ipsi1) (psi1(i), i=1,cdgldim(2))
            write(wrtag,'(a13,i10,a1)') '<end data set',countpsi,'>'
            write(ipsi1) wrtag
         endif

         do i = 2, min(step,nout2-k1)   ! step over missed points
            call rdpsi(ipsi,psi,spsi,jindx,zdum,idum,zdum1,idum1)
            if (lend) goto 77
         enddo
         
      enddo                     ! end do-loop over k1, i.e. psi(t)

!-----------------------------------------------------------------------
! Set all parameters for the reduced wavefunction.
!-----------------------------------------------------------------------
      if(full) then
         write(ilog,'(a,i4,/)') ' 1D reduced system. Subdim =',sdmc
      elseif(cnmode(2) .eq. 2 .and. nstate.eq.1) then
         write(ilog,'(a,i4,a,L2,/)') ' 2D reduced system. Dim =',&
              cdim(1,1,2), ', l2short =', l2short
      endif

      close (ipsi)              ! Close old wavefunction.
      khi  = kmax
      klo  = 1
      out2 = out2*step
      cout2(2) = cout2(2)*step
      step = 1
      nst1 = 1
      nst2 = cnstate(2) 
      dgldimold=cdgldim(2)
      dgldimnew=cdgldim(2)
      lid=.false.
      if(.not.full) then
         mcap = cmcap           ! Set mcap according to new pointers.
         ipsi = ipsi1           ! The reduced wavefunction is now THE WF. 
         call getcsys(2)        ! Load the new pointers.
         call getcpsi(2)  
         call getcdat(2)
         if(loper) then         ! set new pointers for the operator.
            call operflag(nham)
            call zeighpsi(nham)
         end if
      end if

      return

 77   continue
      write(6,'(a)') ' reduce: EOF found when reading the psi-file!'
      stop
      end subroutine reduce

!************************************************************************

      subroutine newvndim(prvdim,prndim,prm,todo,s)

!************************************************************************
! compute current value of prvdim, prndim
! for a given projector-mode prm, consider only those modes which
! have not been included in the summation up to now.
!************************************************************************

use global
use maxv, only: maxdim
use griddatmod
use psidef

      implicit none

      integer(long)                                     :: m
      integer(long), intent(in)                         :: prm,s
      integer(long), intent(out)                        :: prvdim,prndim
      logical(kind=4), dimension(maxdim), intent(inout) :: todo

      prvdim=1
      prndim=1

      do m=1,prm-1
         if (todo(m)) prvdim=prvdim*dim(m,s)
      enddo

      do m=prm+1,nmode
         if (todo(m)) prndim=prndim*dim(m,s)
      enddo
      
      todo(prm) = .false.

      return
      end subroutine newvndim

!************************************************************************

   subroutine reduceml(projector,psi,spsi,dover,tree)

!***********************************************************************
!              REDUCEML
! Computes the projected (reduced) ML-wavefunction and writes it to file.
!***********************************************************************

      implicit none

   integer(long)               :: i,m,e,k1,np,sdm,chk,jindx(1)
   integer(long), dimension(1) :: idum,idum1
   complex(dop),intent(inout)  :: psi(dgldim),dover(doverdim)
   complex(dop),intent(in)     :: projector(prjdim)
   complex(sip) ,intent(inout) :: spsi(spsidim)
   complex(dop), dimension(1)  :: zdum,zdum1
   type(wfnode_t),intent(in)   :: tree
   character(len=c2)           :: wrtag

!-----------------------------------------------------------------------
! Consistency Check.
!-----------------------------------------------------------------------
   do np=1,nprj
      if(prjmode(np).eq.mcap) then
         write(6,'(a,/,a,3i5)') &
              'Projection onto the CAP-mode is not allowed!',&
              'np, prjmode, mcap =', np, prjmode(np), mcap
         stop
      endif
   enddo

!-----------------------------------------------------------------------
!  Write header of projected psi file
!-----------------------------------------------------------------------
   filever(ipsi1) = progver
   open(ipsi1,file=fileout(1:laeout)//'/prjpsi',&
        status='unknown',form='unformatted')
   write(ipsi1) filever(ipsi1)
   call wrpsiinfo(ipsi1)

!-----------------------------------------------------------------------
! Read psi(t).
!-----------------------------------------------------------------------
   chk = 0
   rewind (ipsi)
   read (ipsi)
   call rdpsiinfo(ipsi,chk,chk,chk,chk)

   do k1=1,klo-1
      call rdpsi(ipsi,psi,spsi,jindx,zdum,idum,zdum1,idum1)
   enddo

   do k1=klo,khi,step
      call rdpsi(ipsi,psi,spsi,jindx,zdum,idum,zdum1,idum1)
      if (lend) goto 77

!-----------------------------------------------------------------------
! Reduce the bottom SPFs by applying the projector
!-----------------------------------------------------------------------
      do np  = 1, nprj
         m   = prjmode(np)
         sdm = subdim(m)

!..compute the overlap <projector|psi> for mode m.
!..Note: The array projector contains the complex-conjugate of the function.
         call mvtxzz(psi(zetf(m,1)),projector(zprj(np)),dover,sdm,dim(m,1))

!..Zero the projected SPFs and give their first grid point the overlap value.
         call zerovxz(psi(zetf(m,1)),sdm*dim(m,1))
         do e = 1, dim(m,1)
            psi(zetf(m,1)+(e-1)*sdm)=dover(e)
         enddo
      enddo
      write(wrtag,'(a9,i10,a1)') '<data set',k1,'>'
      write(ipsi1) wrtag
      wrtag='<psi>'
      write(ipsi1) wrtag
      write(ipsi1) (psi(i), i=1,dgldim)
      write(wrtag,'(a13,i10,a1)') '<end data set',countpsi,'>'
      write(ipsi1) wrtag

      do i = 2, min(step,nout2-k1) ! step over missed points
         call rdpsi(ipsi,psi,spsi,jindx,zdum,idum,zdum1,idum1)
      enddo
   enddo            ! end do-loop over k1, i.e. psi(t)

   close (ipsi)     ! Close old wavefunction.
   ipsi = ipsi1     ! The reduced wavefunction is now THE WF.
   khi  = kmax
   klo  = 1
   out2 = out2*step
   step = 1

!  Reset the z-pointers, such that subdim(m)=1 is assumed in ML.
   do np  = 1, nprj
      m   = prjmode(np)
      write(ilog,'(a,i4)') 'Reset z-pointers for mode', m
      call reset_pointer_z(tree,m)
   enddo
   write(ilog,*)

   return

77 write(6,'(a)') 'reduceml: EOF found when reading the psi-file!'
   stop

   end subroutine reduceml

   end module flxreduce
