!***********************************************************************
!
!                         FLXUTILS
!
!     Set of routines needed for flux.
!     Contents: ovlp, dblint, fullgtheta, getstart, rdenerd, rdff
!               tout, fullgth0, mexpand
!
! HDM Sep 99, HDM Nov 00
!
!
!***********************************************************************

      module flxutils

use decimal, only:long,dop,sip
use mltypes
use op1lib, only: cpvxz, zerovxz
use mtlib, only: dtxxdz, mtxxzz
use mmlib, only: mmaxzz, dmxxdz
use xvlib, only: vvaxzz, dvxxdz
use logdat
use keyunitsmod

use global
use aglobal
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use compdatmod
use datenmod
use channels
use fluxmod

use hpsimod
use iofile, only: rdpsiinfo
use hphimod, only: hphi,hunphi
use phihphimod, only: phihphi
use calchamod, only: calcha
use rdfilesmod, only: rdpsi
use wftree
use wftree1
use mlanalysis

      implicit none
      private
      public :: ovlp,dblint,getstart,tout,mexpand,fullgth0,fullgtheta
      contains


      subroutine ovlp(ovl,psi,psi1)

      implicit none

      integer(long)                                  :: s,m,zeig1,zeig2,&
                                                        swapzeig,i,ii,j
      complex(dop)                                   :: zdum
      complex(dop), intent(out)                      :: ovl
      complex(dop), dimension(dgldim), intent(in)    :: psi1,psi
      complex(dop), dimension(doverdim)              :: dover
      complex(dop), dimension(2*maxblock)            :: workc
      logical(kind=4)                                :: lpsicm,lpsicm1

      lpsicm=lpsiopt(2)
      lpsicm1=lpsiopt(2)

!-----------------------------------------------------------------------
!  calculate density matrices for each degree of freedom defined as
!  overlap of the single-particle functions rho^m_ij=<phi_i,2|phi_j,1>
!-----------------------------------------------------------------------
      do s=nst1,nst2
         do m=1,nmode
            call mmaxzz(psi(zetf(m,s)),psi1(zetf(m,s)), &
                    dover(dovmat(m,s)),subdim(m),dim(m,s),dim(m,s))
         enddo
      enddo

!-----------------------------------------------------------------------
! Shortcut for l2short (Special case for 2D reduced systems).
!-----------------------------------------------------------------------
      if(l2short) then
         ovl = (0.d0,0.d0)
         do s=nst1,nst2
            do j = 1, dim(1,s)
               do i = 1, dim(1,s)
                  ii = (i-1)*dim(1,s) + j - 1
                  ovl=ovl+dover(dovmat(1,s)+ii)*dover(dovmat(2,s)+ii)
               enddo
            enddo
         enddo
         return
      endif

!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J,2|Phi_L,1>A_L,1 within each state
!-----------------------------------------------------------------------
      ovl = (0.d0,0.d0)
      do s=nst1,nst2
         zeig1=1
         zeig2=maxblock+1
         call cpvxz(psi1(zpsi(s)),workc(zeig2),block(s))

         do m=1,nmode
            swapzeig=zeig1
            zeig1=zeig2
            zeig2=swapzeig
            call mtxxzz(dover(dovmat(m,s)),workc(zeig1), &
                    workc(zeig2),vdim(m,s),dim(m,s),ndim(m,s), &
                    dim(m,s))
         enddo

!-----------------------------------------------------------------------
! calculate Sum_J A_J,2^* Sum_L <Phi_J,1|Phi_L,2>A_L,2 within each state
!-----------------------------------------------------------------------
         call vvaxzz(psi(zpsi(s)),workc(zeig2),zdum,block(s))
         ovl = ovl + zdum
      enddo

      return
      end subroutine ovlp

!#######################################################################
!-----------------------------------------------------------------------
      subroutine dblint(wmat,gtau,energad,edstr,splinit,work)

      implicit none

!-----------------------------------------------------------------------*
!                SUBROUTINE  DBLINT                                     *
! This routine evaluates the double integral Eqs.(197-199)              *
! Physics Reports 324, 1 (2000).                                        *
!-----------------------------------------------------------------------*
! Definitions:                                                          *
! mcap = mode of the CAP; fcap = degree of freedom of the CAP.          *
! sdmc = subdim(mcap)                                                   *
! wgam = W^gam = CAP.                                                   *
! wtt  = W_tt = <psi(t)|W^gam|psi(t)>                                   *
! wmat = W_tt' = <psi(t)|W^gam|psi(t')>; t=(k1-1)*out2, t'=(k2-1)*out2  *
! wmat is in symmetric storage mode.                                    *
! The correlation function g(tau) is defined by Eq.(199) of the         *
! MCTDH-review (Physics Reports (1999)).                                *
! gtau(k) = g(delt*(k-1)).                                              *
!-----------------------------------------------------------------------*
! energad(n) = energy points read from file 'enerd'.                    *
! edstr(n)   = adiabatically corrected energy distribution              *
!              DELTA(energad(n))   read from file 'enerd'.              *
!   Note: This data can alternatively be read fron another flux-file.   *
!-----------------------------------------------------------------------*

      integer(long)       :: j,j1,k,kk,ierr,energdim
      real(dop)           :: dele,delea,emaxa,emina,energy,eau,delt,t,&
                             flux,sum,sume,eev,yedstr,yp1,ypn,y1,&
                             eminb,emaxb,fl,efac,enj1,sumed

      real(dop), dimension(maxendim), intent(out)   :: energad,edstr,work
      real(dop), dimension(maxendim), intent(in)    :: splinit
      complex(dop), dimension(laewmat), intent(in)  :: wmat
      complex(dop), dimension(2*nout2), intent(out) :: gtau
      character(len=8)                              :: buf
      character(len=6)                              :: unitrd
      logical(kind=4) ::     lrdff

!-----------------------------------------------------------------------
! Initialize, transform energies to au.
!-----------------------------------------------------------------------
      lrdff = .false.
      delt  = out2*step
      dele  = (emax-emin)/dble(epoints)
      emaxa = emax
      emina = emin
      delea = dele
      call keyeunits(emaxa,unit,ierr)
      call keyeunits(emina,unit,ierr)
      call keyeunits(delea,unit,ierr)
      if(ierr .gt. 0) then
         write(6,'(2a)')' dblint : unknown unit: ', unit
         stop
      end if

!-----------------------------------------------------------------------
! Read the energy distribution from file 'enerd', if existing;
! and determine spline coefficients.
! maxendim: maximal entries for edstr, energdim: actual length of edstr
!-----------------------------------------------------------------------
      if(maxendim .gt. 0) then
         yp1=1d30
         ypn=1d30
         if(edlaenge .gt. 4 ) then
            if (edstrf(edlaenge-4:edlaenge).eq.'enerd' .or. &
                edstrf(edlaenge-4:edlaenge).eq.'adwkb') then
               eminb = (emina - delea - eoff)*ev
               emaxb = (emaxa + delea - eoff)*ev
               call rdenerd(energad,edstr,energdim,eminb,emaxb)
               lrdff=.false.
               unitrd='ev'
            else
               call rdff(energad,edstr,energdim,unitrd)
               lrdff=.true.
               if(unitrd .eq. '      ') then
                  write(6,'(3(/a),/)') &
                 '### Cannot determine the energy unit on the', &
                 ' flux or spectrum file given by -ed option.', &
                 ' Assuming identical unit as given in present input.' 
                  write(ilog,'(3(/a),/)') &
                 '### Cannot determine the energy unit on the', &
                 ' flux or spectrum file given by -ed option.', &
                 ' Assuming identical unit as given in present input.'
               elseif(unitrd .ne. unit(1:6)) then
                  efac=1.d0  ! Transform energad to the current energy unit.
                  call keyunits(efac,unit,ierr)
                  efac=1.d0/efac
                  if(unit .eq. 'nmwl') then
                     do j=1,(energdim+1)/2
                        call keyunits(energad(j),unitrd,ierr)
                        energad(j)=energad(j)*efac
                        energad(j)=1.d0/energad(j)
                        j1=energdim+1-j
                        if(j.ge.j1) exit
                        call keyunits(energad(j1),unitrd,ierr)
                        energad(j1)=energad(j1)*efac
                        enj1=1.d0/energad(j1)
                        energad(j1)=energad(j) ! Reverse the order of
                        energad(j)=enj1        ! the energies (for nmwl)
                        enj1=edstr(j1)
                        edstr(j1)=edstr(j)
                        edstr(j)=enj1
                     enddo
                  else
                     do j=1,energdim
                        call keyunits(energad(j),unitrd,ierr)
                        energad(j)=energad(j)*efac
                     enddo
                  endif
                  if(unitrd .eq. 'nmwl') then
                     do j=1,(energdim+1)/2
                        j1=energdim+1-j
                        if(j.ge.j1) exit
                        enj1=energad(j1)
                        energad(j1)=energad(j) ! Reverse the order of
                        energad(j)=enj1        ! the energies (for nmwl)
                        enj1=edstr(j1)
                        edstr(j1)=edstr(j)
                        edstr(j)=enj1
                     enddo
                  endif
                  if(ierr .gt. 0) then
                      write(6,'(2a)')' dblint : unknown unit: ', unitrd
                      stop
                  end if
               endif  ! if(unitrd .ne. unit)
            endif ! edlaenge .gt. 4 (rdenerd or rdff)
         endif    ! maxendim .gt. 0
         write(ilog,'(a,i5,a,i5)') &
              'maxendim =',maxendim,',   energdim =',energdim
         call splinenm(energad,edstr,energdim,yp1,ypn,splinit,work)
         close(ienerd)
         sumed=0.d0
         do j=1,energdim
            sumed=sumed+edstr(j)**2
         enddo
         sumed=sqrt(sumed/dble(energdim))
      end if
!-----------------------------------------------------------------------
! Read gtau(k) from gtau file, provided gtau exist and -w option is not set.
! -w = lovwr , -v = lgtt
!-----------------------------------------------------------------------
      if( .not.lovwr .or. lgtt) then
         j=start
         y1=ostart
         rewind (iaus)
         read(iaus,'(a8)') buf
         read(iaus,'(2x,i5,3i7,f15.5,i7,f15.5)') &
              klo,khi,kmax,step,out2,start,ostart
         read(iaus,'(a8)') buf
         read(iaus,'(a8)') buf
         do k = 1, kmax
            read(iaus,'(30x, 2e25.15)') gtau(k)
         end do
         if(lgtt .and. j.gt.0) then
            start=j
            ostart=y1
         endif
      else
!-----------------------------------------------------------------------
! Compute gtau(k). (W-part)  (lovwr=.true.  .and.  lgtt=.false.)
! g(k) = Sum(j) w(j,k+j-1)
!-----------------------------------------------------------------------
         do k = 1, kmax
            do j = 1, kmax - k + 1
               kk = ( (k+j-2)*(k+j-1) )/2
               gtau(k) = gtau(k) + wmat(kk+j)
            end do
!....Trapezoidal rule (Dieter 28.01.11)
!            j=(k*(k-1))/2 +1
!            kk=((kmax-1)*kmax )/2 + kmax - k + 1
!            gtau(k) = gtau(k) - 0.5d0*(wmat(j)+wmat(kk))
!....Found not to be useful, hence outcommented.
            gtau(k) = gtau(k)*delt
         end do
      endif

!-----------------------------------------------------------------------
! Read gtau(k) (theta-part) from gtau file.
!-----------------------------------------------------------------------
      if(.not.lovwr .and. .not.lgtt) then
         read(iaus,'(a)')
         read(iaus,'(a)')
         do k = 1, kmax
            read(iaus,'(30x, 2e25.15)') gtau(k+nout2)
         end do
      endif

!-----------------------------------------------------------------------
! Write gtau to gtau-file. (lovwr=.true.))
!-----------------------------------------------------------------------
      if(lovwr .and. (laeout.gt.0)) then
         rewind (iaus)
         write(iaus,'(2a)') '#   klo    khi', &
              '   kmax   step        out2     start     ort(start)'
         write(iaus,'(a2,i5,3i7,f15.5,i7,f15.5)') &
              '# ',klo,khi,kmax,step,out2,start,ostart
         write(iaus,'(/a,7x,a,17x,a,17x,a)') '#  tau[fs]', &
              'ABS(gtau)','RE(gtau)','Im(gtau)'

         do k = 1, kmax
            write(iaus,'(f10.3,e20.9,2e25.15)') &
                 step*(k-1)*out2/fs,abs(gtau(k)),gtau(k)
         end do

         write(iaus,'(/a,7x,a,17x,a,17x,a)') '#  tau[fs]', &
              'ABS(gtau)','RE(gtau)','Im(gtau)'
         do k = 1, kmax
            write(iaus,'(f10.3,e20.9,2e25.15)') step*(k-1)*out2/fs, &
                 abs(gtau(k+nout2)),gtau(k+nout2)
         enddo
      end if

      close(iaus)

!-----------------------------------------------------------------------
! Sum gtau[W] + gtau[theta]. ( if(ltonly) use only theta part)
!-----------------------------------------------------------------------
      if(ltonly) then
         do k = 1, kmax
            gtau(k) = gtau(k+nout2)
         enddo
      else
         do k = 1, kmax
            gtau(k) = gtau(k) + gtau(k+nout2)
         enddo
      endif

!-----------------------------------------------------------------------
! Write headline to flux file.
!-----------------------------------------------------------------------
      k = index(unit,' ')-1
      if(laeout.gt.0) then
         write(iflx,'(/3a,10x,a,14x,a,11x,a)') &
         '# Energy[',unit(1:k),']','flux','DELTA(E)','Probability'
      endif

!-----------------------------------------------------------------------
! Check that energy interval o ed-file is suffifient.
!-----------------------------------------------------------------------
      if(maxendim.gt.1) then
        if(energad(1) .gt. emin) then
           write(6,'(/a,1pe10.3,/a,1pe10.3,/a)') &
           'The lowest energy, emin =',emin, &
           'is lower than the lowest energy on the ed-file:', &
           energad(1),'This may lead to interpolation problems.'
        endif
        if(energad(energdim) .lt. emax) then
           write(6,'(/a,1pe10.3,/a,1pe10.3,/a)') &
           'The highest energy, emax =',emax, &
           'is higher than the highest energy on the ed-file:', &
           energad(energdim), &
           'This may lead to interpolation problems.'
        endif
        write(6,*)
      endif

!-----------------------------------------------------------------------------
! Compute the flux, defined by Eq.(198) MCTDH-review (Physics Reports 324
! (2000) 1-105). Note that the term  pi*|delta(e)|**2  is omitted here.
!-----------------------------------------------------------------------------
      sum  = 0.d0
      sume = 0.d0
      y1   = 0.d0
      do j = 0, epoints
         flux = 0.5d0*dble(gtau(1))   !  0.5 because of trapezoidal rule.
         energy = emin + j*dele
         eau = energy    ! eau is needed for Fourier transform
         call keyeunits(eau,unit,ierr)
         eau = eau - eoff

         if(maxendim .gt. 0) then
            eev = eau*ev
! --------  determine the energy-distribution for 'energy' via spline fit.
            call splintnm(energad,edstr,splinit,energdim,eev,yedstr)
! --------  avoid division by zero when evaluating flux/yedstr
            if(yedstr .lt. 1.d-7*sumed) yedstr=1.d-4*sumed
        end if

         do k = 2, kmax
            t = (k-1)*delt
            fl= dble( exp(dcmplx(0.d0,eau*t))*gtau(k) )
            flux = flux + fl
         end do
         flux = 2.d0*delt*flux*ufac/pi

         if(lmb) then   !  -Mb (mega-barn) is set
            flux=flux*eau*dipolem
         endif

         if(laeout.gt.0) then
            if(maxendim .gt. 0 .and. .not.ladd ) then
               write(iflx,'(g12.6,1x,2g20.9,3x,g20.12)') &
                    energy, flux, yedstr, flux/yedstr
            elseif(maxendim .gt. 0 .and. ladd ) then 
               write(iflx,'(g12.6,1x,2g20.9,3x,g20.12)') &
                    energy, flux, yedstr+flux, flux/(yedstr+flux)
            else
!.............no energy distribution (file enerd) available.
              yedstr = 0.d0
               write(iflx,'(g12.6,1x,2g20.9,3x,g8.3)') &
                    energy, flux, yedstr, yedstr
            end if
         end if

         if( unit .eq. 'nmwl' ) then
            sum = sum + flux*dele*(eau+eoff)/energy
            sume = sume + yedstr*dele*(eau+eoff)/energy
!     This allows to use the unit "nmwl" (nanometer wavelength).
         else
            sum  = sum + flux*delea
            sume = sume + yedstr*delea
         end if
      end do

      if(sum .gt. 999.9d0) then
         write(ilog,'(/,a,f14.3,3x,a)') &
          ' 2*Integral(W_tt)  =', 2.d0*dble(gtau(1))*ufac, &
          '(total outgoing flux)'
         write(ilog,'(a,f14.3,3x,a)') &
          '   Integral(flux)  =', sum, '(total outgoing flux)'
      else
         write(ilog,'(/,a,f14.9,3x,a)') &
          ' 2*Integral(W_tt)  =', 2.d0*dble(gtau(1))*ufac, &
          '(total outgoing flux)'
         write(ilog,'(a,f14.9,3x,a)') &
          '   Integral(flux)  =', sum, '(total outgoing flux)'
      endif
      write(ilog,'(a,f14.9,3x,a)') &
       '   Integral(DELTA) =', sume,'(total incoming flux)'
      write(ilog,'(/2a,/a)') ' The total outgoing flux should be 1, ', &
       'if there is only one channel', &
       ' and no operators/projectors are applied.'
      if(sum .gt. 999.9d0) then
         write(6,'(a,f11.1,5x,a)') '    2*Integral(W_tt)  =', &
                  2.d0*dble(gtau(1))*ufac,'(total outgoing flux)'
         write(6,'(a,f11.1,5x,a)') &
           '      Integral(flux)  =',sum, '(total outgoing flux)'
      elseif(sum .gt. 9.9d0) then
         write(6,'(a,f11.5,5x,a)') '    2*Integral(W_tt)  =', &
                  2.d0*dble(gtau(1))*ufac,'(total outgoing flux)'
         write(6,'(a,f11.5,5x,a)') &
           '      Integral(flux)  =',sum, '(total outgoing flux)'

      else
         write(6,'(a,f11.5,5x,a)') ' 2000*Integral(W_tt)  =', &
                  2.d3*dble(gtau(1))*ufac,'(total outgoing flux)'
         write(6,'(a,3pf11.5,5x,a)') &
           ' 1000*Integral(flux)  =',sum, '(total outgoing flux)'
      endif
      write(6,'(a,3pf11.5,5x,a)') &
           ' 1000*Integral(DELTA) =',sume,'(total incoming flux)'
      write(6,'(78a1)')  ('-',k=1,78)

      if(laeout .gt. 0) close(iflx)

      return
      end subroutine dblint

!#######################################################################

      subroutine fullgtheta(fullpsi,theta,gtau)

!***********************************************************************
!   calculates the (residual) g_theta(tau) = <psi(T-tau)|Theta|psi(T)>
!   for the fully reduced WF fullpsi. (See  subroutine gtheta)
!***********************************************************************

      implicit none

      integer(long)                                   :: k1,kk,kk1,kk2,s
      complex(dop)                                    :: ovl
      complex(dop), dimension(2*nout2), intent(out)   :: gtau
      complex(dop), dimension(fulldim), intent(inout) :: fullpsi 
      real(dop), dimension(sdmc), intent(in)          :: theta

      call zerovxz(gtau,2*nout2)
      if(start.gt.gdim(fcap)) then
         write(ilog,'(a)')  'No g_theta(tau) evaluated.'
         return
      endif

!------------------------------------------------------------------------
!     Multiply Psi(T_final) by Theta (=W^gam) ; sdmc=subdim(mcap)
!------------------------------------------------------------------------
      kk2 = (khi-1)*sdmc*nstate + 1
      do s=1,nstate
         if(nspfdof(mcap) .eq. 1 ) then ! un-combined
            call dvxxdz(theta,fullpsi(kk2),fullpsi(kk2),sdmc)
         else
            call dtxxdz(theta,fullpsi(kk2),fullpsi(kk2),vgdim(fcap), &
                        gdim(fcap),ngdim(fcap))
         endif
         kk2 = kk2 + sdmc
      enddo

!-----------------------------------------------------------------------
! Do the overlaps.
!-----------------------------------------------------------------------
      kk2 = (khi-1)*sdmc*nstate + 1
      kk  = khi + 1
      do k1=1,khi
         kk = kk - 1
         kk1 = (k1-1)*sdmc*nstate + 1
         call vvaxzz(fullpsi(kk1),fullpsi(kk2),ovl,sdmc*nstate)
         gtau(kk+nout2) = 0.5d0*ovl
      enddo

      return
      end subroutine fullgtheta


!***********************************************************************

      subroutine getstart(wgam)

!------------------------------------------------------------------------
!     Determine start1, start2, and start.
!     start1 is the starting point of the left-hand cap, start2 of the
!     right-hand side cap. For captype=3 (both), the -s option acts only
!     on start2, i.e. on the rhs cap.
!------------------------------------------------------------------------

      implicit none

      integer(long)                             :: k1 
      real(dop), dimension(maxgdim), intent(in) :: wgam

      if(captype .eq. 3) then     !  both
         start2=gdim(fcap)+1
         start1=0
         do k1 = 1, gdim(fcap)
            if( abs(wgam(k1)) .lt. 1.d-14 ) then
               start2=k1
            elseif(start2 .gt. k1 ) then
               start1=k1+1
            endif
         enddo
         if(start.gt.0) then
            start2 = start
         else
            start = start2
         endif
         goto 30
      endif

      if(captype .eq. 2) then     !  right CAP
         start1=0
         if(start.gt.0) then
            start2=start
            goto 30
         endif
         do k1 = 3, gdim(fcap)
            if( abs(wgam(k1)) .gt. 1.d-14 ) then
               start2 = k1 - 1
               start  = start2
               goto 30
            end if
         enddo

      elseif(captype .eq. 1) then !  left CAP
         start2=gdim(fcap)+1
         if(start.gt.0) then
            start1=start
            goto 30
         endif
         do k1 = gdim(fcap)-2, 1, -1
            if( abs(wgam(k1)) .gt. 1.d-14 ) then
               start1 = k1 + 1
               start  = start1
               goto 30
            end if
         enddo
      endif

 30   return
      end subroutine getstart



!***********************************************************************

      subroutine rdenerd(energad,edstr,energdim,eminb,emaxb)

!------------------------------------------------------------------------
!  RDADWKB  reads the enerd-file which contains the energy-distribution.
!------------------------------------------------------------------------

      implicit none

      integer(long)                               :: g,n,subdim
      integer(long), intent(out)                  :: energdim
      real(dop)                                   :: rdum1,rdum2,de,dd,edstr1,e1
      real(dop), dimension(maxendim), intent(out) :: energad,edstr
      real(dop), intent(in)                       :: eminb,emaxb

      routine='rdenerd'

      rewind (ienerd)
      read (ienerd,*,end=40,err=40) subdim
      do g=1,subdim
         read (ienerd,*,err=20) rdum1,rdum2
      enddo

      n=0
 50   n=n+1
      if (n.gt.maxendim) goto 30
      read (ienerd,*,end=10,err=20) energad(n),edstr(n)
      energdim=n
      if(energad(n) .le. emaxb) goto 50

 10   continue

!.....Give delat(E) a dummy value for very low energies.
      if( energad(1) .gt. eminb ) then
         de = energad(2)-energad(1)
         dd = max(0.d0,edstr(1)-edstr(2))
         g  = int((energad(1)-eminb)/de) + 1
         e1 = energad(1) - g*de
         edstr1 = min(max(edstr(1),0.1d0),30.d0)
         energdim = min(energdim+g-1,maxendim)
         do n = energdim, g, -1
            energad(n) = energad(n-g+1)
            edstr(n)   = edstr(n-g+1)
         enddo
         do n = 1, g-1
            energad(n) = e1 + n*de
            edstr(n)   = edstr1 + (g-n)*dd
         enddo
      endif

!.....Smooth.
      if(lsmooth) then
         do n = 3, energdim-2
            edstr(n) = (edstr(n-2)+edstr(n-1)+edstr(n)+edstr(n+1) &
                 +edstr(n+2))*0.2d0
         enddo
         do n = energdim-2, 3, -1
            edstr(n) = (edstr(n-2)+edstr(n-1)+edstr(n)+edstr(n+1) &
                 +edstr(n+2))*0.2d0
         enddo
      endif


      RETURN

 20   write(6,'(i5,a)') energdim,' lines read before error occured.'
      message='Error reading "enerd" file'
      call errormsg
      stop

 30   message='There are more than maxendim energy grid points.'
      call errormsg

 40   message=' enerd file is empty?'
      call errormsg
      end subroutine rdenerd


!
!***********************************************************************

      subroutine rdff(energad,edstr,energdim,unitrd)

!------------------------------------------------------------------------
!  RDFF  reads a flux-file and takes the data as energy-distribution.
!------------------------------------------------------------------------

      implicit none

      integer(long)                               :: n,j,nz
      integer(long), intent(out)                  :: energdim
      real(dop)                                   :: dum
      real(dop), dimension(maxendim), intent(out) :: energad,edstr
      character(len=16)                           :: lab
      character(len=6)                            :: unitrd

      rewind (ienerd)

      n=0
      nz=0
      energdim=0
      unitrd='      '

 15   read (ienerd,'(a)',end=41,err=50) lab
      if(lab(1:8) .eq. '# Energy') then
         j=index(lab,']')-1
         if(j.ge.10) unitrd=lab(10:j)
         call keylowc(unitrd)
      elseif( lab(1:1) .eq. '#' ) then
         goto 15
      elseif( lab .eq. '                ' ) then
         goto 15
      else
         backspace ienerd
      endif

 20   n=n+1
      if (n.gt.maxendim) goto 40
      if(lc3d) then
         read (ienerd,*,end=40,err=21) energad(n),dum,edstr(n)
      else
         read (ienerd,*,end=40,err=21) energad(n),edstr(n)
      endif
      nz=0
      energdim=n
      goto 20

 21   backspace ienerd
 25   read (ienerd,'(a)',end=40,err=50) lab
      if( lab(1:1) .eq. '#' )           goto 25
      if( lab .eq. '                ' ) goto 25
      backspace ienerd
      n=n-1
      nz=nz+1
      if(nz.gt.100) goto 50
      goto 20

 40   RETURN

 41   write(6,'(a)') ' Empty edstr-file (except blank and #-lines)'
      stop
 50   write(6,'(a)')' ERROR while reading flux/spectrum as edstr-file.'
      write(6,'(i5,a)') energdim,' lines read before error occured.'
      stop

      end subroutine rdff

!***********************************************************************

      subroutine tout(k1,klo,time,tm1)

!------------------------------------------------------------------------
!  TOUT  writes timing information to flux.log
!------------------------------------------------------------------------

      implicit none

      integer(long)             :: i,tm2
      integer(long), intent(in) :: k1,klo,tm1
      real(dop)                 :: cpu,spent
      real(dop), external       :: fcputime,fsystime
      real(dop), intent(in)     :: time
      character(len=80)         :: actual

      real(dop), save :: previous = 0.0_dop

      if(k1.eq.klo ) then
         write(ilog,'(5x,a,3x,a,2x,a,6x,a,4x,a,6x,a,4x,a)') &
            'k1', 'k1-klo','time[fs]', 'CPU[s]', 'Deta-CPU', &
            'Real-time','wall-time[s]'
         previous=fcputime(0)+fsystime(0)
         return
      endif

!----------------------------------------------------------------------
! determine actual time and CPU-time spent since last call
!----------------------------------------------------------------------
      call fzeit(actual)
      i = index(actual,':')
      actual=actual(i-9:i+6)
      call unixzeit(tm2)
      cpu=fcputime(0)+fsystime(0)
      spent=cpu-previous
      previous=cpu

      write(ilog,'(2i7,3f12.2,3x,a,i9)') &
            k1-1,k1-klo-1,time/fs,cpu,spent,trim(actual),tm2-tm1
      call myflush(ilog)

      return
      end subroutine tout

!***********************************************************************

      subroutine fullgth0(fullpsi,psi,theta,ovlth)

!***********************************************************************
!   calculates ovlth = <fullpsi(T)|Theta|fullpsi(T)>
!   and psi = Theta|fullpsi(T)>
!   for the fully reduced WF fullpsi. (See  subroutine gth0)
!***********************************************************************

      implicit none

      integer(long) :: kk,s
      complex(dop), dimension(sdmc*nstate), intent(in)  :: fullpsi
      complex(dop), dimension(sdmc*nstate), intent(out) :: psi
      complex(dop), intent(out)                         :: ovlth
      real(dop), dimension(sdmc), intent(in)            :: theta

!------------------------------------------------------------------------
!     Multiply Psi by Theta ; sdmc=subdim(mcap)
!------------------------------------------------------------------------
      kk =  1
      do s=1,nstate
         if(nspfdof(mcap) .eq. 1 ) then ! un-combined
            call dvxxdz(theta,fullpsi(kk),psi(kk),sdmc)
         else
            call dtxxdz(theta,fullpsi(kk),psi(kk),vgdim(fcap), &
                        gdim(fcap),ngdim(fcap))
         endif
         kk = kk + sdmc
      enddo

!-----------------------------------------------------------------------
! Do the overlaps.
!-----------------------------------------------------------------------
      call vvaxzz(psi,psi,ovlth,sdmc*nstate)

      return
      end subroutine fullgth0

!***********************************************************************


      subroutine mexpand(psi)

!------------------------------------------------------------------------
! Expand psi to full MCTDH-length in case of identical particles.
!------------------------------------------------------------------------

      implicit none

      integer(long) :: m,s
      complex(dop), dimension(dgldimnew), intent(out)   :: psi
      complex(dop), dimension(dgldimnew)                :: workc

      call cpvxz(psi(adim+1),workc,dgldimold-adim)
      do m=1,nmode
         do s=1,nstate
            call cpvxz(workc(zetf_f(m,s)-adim),psi(zetf(m,s)), &
                             dim(m,s)*subdim(m))
         enddo
      enddo
      dgldim=dgldimnew
      return
      end subroutine mexpand

!***********************************************************************

      end module flxutils
