!########################################################
!
!  To produce the guess field for the first propagation
!  17, Mar, 2005, Luxia Wang
!  Markus Schroeder and Alex Brown.
!
!########################################################

      program guessfld

use decimal, only:dop,long,sip

      implicit none

      integer(long), external           :: myiargc
      integer(long)                     :: i,iarg,nfin
      integer(long)                     :: mrdim,midim,mcdim,mldim,msdim,mfdim
      real(dop)                         :: fld_strgth,omega_fld,factor,tfinal,tout
      character(len=16)                 :: buf,units
     
      real(dop), parameter :: fs=41.3414_dop, ev=27.2114_dop, icm=219474.6313_dop

      common /parameters/  tfinal, tout, fld_strgth, omega_fld, nfin


      iarg = myiargc()
      
      call mygetarg(1,buf) 
      if( buf .eq. '-h' ) then
         write(6,'(78a1)')  ('-',i=1,78)
         write(6,'(1x,a,/10x,a)')&
       'Purpose: Calculates the initial guess field for optimal control'&
           ,'The pulse is a Gaussian times sin(omega_field*t)'
            write(6,'(/1x,2a)')&
       'Usage  : guessfld tfinal tout fld_strgth omega_fld ',&
                 '[frequency unit]' 
         write(6,'(/2a,/,2a)') ' The times tfinal, tout,',&
             ' are in fs. The field-strength,',&
             ' fld_strgth, and the ',&
             ' field-frequency, omega_fld, '
          write(6,'(2a,/,2a)') ' are in eV, au or cm-1.',&
             ' The frequency unit can be',&
             ' given with an additional optional string.',&
             ' The default unit is au. '
         write(6,'(78a1)')  ('-',i=1,78)
         stop
      endif
      if( myiargc() .eq. 0 ) then
         write(6,'(/2a,/a,i4,a)') ' guessfld requires command line arguments ',&
         'Use guessfld -h for help.'
         stop 1
      endif
      if( myiargc() .gt. 5 ) then
         write(6,'(/2a,/a,i4,a)') ' guessfld expects 4 or 5 ',&
         'command line arguments',' There are',myiargc(),' arguments.'
         stop 1
      endif 

      
      call mygetarg(1,buf)
      read(buf,*) tfinal
      call mygetarg(2,buf) 
      read(buf,*) tout
      call mygetarg(3,buf)
      read(buf,*) fld_strgth
      call mygetarg(4,buf)
      read(buf,*) omega_fld
      factor=1.0_dop
      if (iarg .eq. 5) then
         call mygetarg(5, units)
         if ( units(1:5) .eq. 'eV   ') then
            factor=1.0_dop/ev
         else if ( units(1:5) .eq. 'cm-1 ') then
            factor=1.0_dop/icm
         else if ( units(1:5) .eq. 'au   ') then
            factor=1.0_dop
         else
            write(6,*) 'guessfld.f: Unknown unit: ', units
            write(6,*) 'Expecting one of: "au" "cm-1" or "eV"'
            stop 1
         end if
      else
         units='au'
      end if
      

!      call mygetarg(5,buf)
!      read(buf,*) pmax      
!      call mygetarg(6,buf)
!      read(buf,*) plength

      write(6,*) "guessfld:      "
      write(6,*) " tfinal        ",tfinal, "fs"
      write(6,*) " tout          ",tout,   "fs"
      write(6,*) " fld_strgth    ",fld_strgth, units
      write(6,*) " omega_fld     ",omega_fld, units

 
 
      tfinal = tfinal*fs
      tout   = tout*fs
      
      fld_strgth  = fld_strgth*factor
      omega_fld     = omega_fld*factor
      

      nfin  = tfinal/tout+0.1_dop
      nfin  = nfin

      call mkfld

      contains

!#######################################################################

      subroutine mkfld

      implicit none

      real(dop)                  :: tfinal, tout, fld_strgth, omega_fld,time
      integer(long)              :: i,iout,nfin
      real(dop), dimension(nfin) :: fld

      common /parameters/  tfinal, tout, fld_strgth, omega_fld, nfin

      real(dop), parameter :: PI=3.141592653589793116_dop

      iout=9

      do i =1, nfin
         time = i*tout
         fld(i) = fld_strgth*dcos(omega_fld*(time-tfinal/2.0_dop))&
                   *dsin( PI*dble(i)/dble(nfin) )**2 
      end do 


      open(iout, file='fieldf.0',err=100)
      write(iout,500) -2.0_dop*tout,  0.0_dop,  0.0_dop, 0.0_dop
      write(iout,500)      -tout,  0.0_dop,  0.0_dop, 0.0_dop
      write(iout,500)       0.0_dop,  0.0_dop,  0.0_dop, 0.0_dop
      do i = 1, nfin
         write(iout,500) dble(i)*tout, fld(i), fld(i), fld(i) 
      enddo
      write(iout,500)      tout+time,  0.0_dop,  0.0_dop, 0.0_dop
      write(iout,500) 2.0_dop*tout+time,  0.0_dop,  0.0_dop, 0.0_dop
      close(9)

      open(9, file='fieldb.0',err=101)
      write(iout,500) -2.0_dop*tout,  0.0_dop,  0.0_dop, 0.0_dop
      write(iout,500)      -tout,  0.0_dop,  0.0_dop, 0.0_dop
      do i = 1, nfin
         write(iout,500) dble(i-1)*tout, fld(nfin-i+1), fld(nfin-i+1),&
              fld(nfin-i +1) 
      enddo
      write(iout,500)           time,  0.0_dop,  0.0_dop, 0.0_dop
      write(iout,500)      tout+time,  0.0_dop,  0.0_dop, 0.0_dop
      write(iout,500) 2.0_dop*tout+time,  0.0_dop,  0.0_dop, 0.0_dop
 
      close(iout)


 500  format(4e24.16)

      return
      
 100  write(6,*) "Guessfld,mkfld: Unable to open file fieldf.0"
      stop 1

 101  write(6,*) "Guessfld,mkfld: Unable to open file fieldb.0"
      stop 1



      end subroutine mkfld


      end program guessfld
