
      module gwptrajmod

use decimal, only: long,dop,sip

      integer(long), save                  :: plottask,singleplot,scan

      logical(kind=4), save                :: ltrjfile = .false.

      end module gwptrajmod

!#######################################################################

      program gwptraj

use decimal, only: long,dop,sip
use global
use globmemmod
use versions
use aglobal
use dvrdatmod
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use plot
use gwptrajmod

use op1lib, only: zerovxd
use logdat
use iofile, only: rdpsiinfo
use rddvrmod, only: dvrinfo
use rdopermod, only: operinfo
use zeigauswmod, only: zeigausw
use memhopsmod, only: memhops

use plotutils, only : mode_lab_corr

use module_interface

      implicit none

      integer(long)                        :: chkdvr,chkgrd,chkpsi,chkdat,ilbl
      character(len=c5)                    :: filename
      logical(kind=4)                      :: lerr
      character(len=c1)                    :: moderep
!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      lovwr = .true.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare files to be read
! set flag to .true. if file is to be read
!
!     lrddvr    DVR file
!     lrdoper   OPER file. 
!-----------------------------------------------------------------------
      lrddvr=.true.
      lrdoper=.false.

!-----------------------------------------------------------------------
! read input options
!-----------------------------------------------------------------------
      call gwptrajinput

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='gwptraj.log',form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(ipsi)

!-----------------------------------------------------------------------
! Allocate memory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare resources required (other than those needed to read a file)
!
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! ltrafo : FBR/DVR trafo matrices
! lfft   : FFT arrays rueck and fftfak
!-----------------------------------------------------------------------
      ncomp=0
      lpsi=.true.
      lpsi1=.false.
      lpsigrd=.false.
      ldmat=.false.
      ldmat1=.false.
      lort=.true.
      lgpop=.false.
      ltrafo=.false.
      lfft=.false.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! open input and output data files
!
! use standard channels if possible
!  irho : gridpop file
!  ipsi : psi file
!  iaut : auto file
!  ichk : check file
!  iaus : output file
!-----------------------------------------------------------------------
! psifile is opened in gwptrajinput 
!     filename='psi'
!     open(ipsi,file=filename,form='unformatted',status='old',err=1000)

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
! First read file version number
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      filename='psi'
      read(ipsi,err=999) filever(ipsi)
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)

      if (.not. (lgwp .or. ltraj)) then
         routine = 'Gwptraj'
         message='No GWP mode present'
         call errormsg
      endif

!-----------------------------------------------------------------------
! read information from system files
!-----------------------------------------------------------------------
      if (lrddvr) then
         chkdvr=2
         filename=dname(1:dlaenge)//'/dvr'
         open(idvr,file=filename,form='unformatted',status='old',&
              err=1000)
         call dvrinfo(lerr,chkdvr)
         close(idvr)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif
      if (lrdoper) then
         chkdvr=2
         chkgrd=2
         filename=operfile
         open(ioper,file=filename,form='unformatted',status='old',&
              err=1000)
         call operinfo(lerr,chkdvr,chkgrd)
         close(ioper)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      if (ltraj) then
         zcent(1,1) = 1
         qcentdim = nspfdof(1)*dim(1,1)
      endif
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
! here assign any further memory arrays
!-----------------------------------------------------------------------
      mrweight=mrdim
      mrdim=mrweight+qcentdim
      mrpcent=mrdim
      mrdim=mrpcent+qcentdim

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      workcdim=max(workcdim,adim)
      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
! assign memory for operator
!-----------------------------------------------------------------------
      if (lrdoper) call memhops

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim/2,&
       ' real*4     : ', mfdim/2,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)
if(allocated(gfgwp))deallocate(gfgwp)
if(allocated(aigwp))deallocate(aigwp)
if(allocated(zgwp))deallocate(zgwp)
if(allocated(dicht2))deallocate(dicht2)
if(allocated(agmat))deallocate(agmat)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))
allocate(gfgwp(gfgwpdim))
allocate(aigwp(aigwpdim))
allocate(zgwp(zgwpdim))
allocate(dicht2(dmatdim))
allocate(agmat(agmatdim))

      call plottraj(mc(mcpsi),&
           mc(mcworkc),ms(mspsi),mi(mijindx),&
           mr(mrweight),agmat,mr(mrqcent),&
           mr(mrpcent),mi(miworki),gfgwp,aigwp,zgwp,&
           dicht2,mr(mrort),mc(mcadgwp),mi(mitrajst))

deallocate(mc,mr,mi,ml,ms,mf,gfgwp,aigwp,zgwp,dicht2,agmat)
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_psidef

      stop
  999 continue
      message= 'Error reading :'//filename(1:c5-15)
      write(6,'(a)') message
      stop
 1000 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop
      contains

!#######################################################################

      subroutine plottraj(psi,workc,spsi,jindx,&
                 weight,agmat,qcenter,pcenter,&
                 worki,gfgwp,aigwp,zgwp,dicht2,ort,adgwp,trajst)

use global
use aglobal
use maxv, only: maxdim
use dvrdatmod
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use plot
use operdef
use gwptrajmod

use rddvrmod
use rdfilesmod, only: rdpsi
use plotutils, only: plotdefs,initplot,timebnds,inmodebnds,plotmenu,dotask
use gwplib, only: gwp_param,gwp_cmom

      implicit none

! standard variables for interactive plots
      integer(long)       :: ndata,task,nsets,nplots,plotdim,psicount,&
                             f,m,chkdvr,chkgrd,n,xdim,ydim,chkpsi,&
                             chkdat,nplots0

      integer(long), dimension(maxdim)                         :: idum 
      integer(long), dimension(gwpinfodim), intent(out)        :: trajst
      integer(long), dimension(adim), intent(out)              :: jindx
      integer(long), dimension(*), intent(out)                 :: worki
      integer(long), dimension(gwpinfodim)                     :: gwpdep
      real(dop)                                                :: time
      real(dop), dimension(1)                                  :: xdum,ydum,edum,rdum
      real(dop), dimension(:), allocatable                     :: pltime
      real(dop), dimension(ortdim), intent(out)                :: ort
      real(dop), dimension(qcentdim), intent(out)              :: weight
      real(dop), dimension(:,:,:), allocatable                 :: pldata,pldata1
      complex(dop), dimension(1)                               :: zdum
      complex(sip), dimension(spsidim), intent(out)            :: spsi
      complex(dop), dimension(dgldim), intent(out)             :: psi
      complex(dop), dimension(workcdim), intent(out)           :: workc

! to correctly assign axes etc after changing plottask
      integer(long)       :: plottaskold

! for GWP plotting
      real(dop), dimension(qcentdim), intent(out)    :: qcenter,pcenter
      complex(dop), dimension(agmatdim), intent(out) :: agmat
      complex(dop), dimension(gsmatdim), intent(out) :: adgwp
      complex(dop), dimension(dmatdim), intent(in)   :: dicht2
      complex(dop), dimension(gfgwpdim), intent(out) :: gfgwp
      complex(dop), dimension(aigwpdim), intent(out) :: aigwp
      complex(dop), dimension(zgwpdim), intent(out)  :: zgwp
      logical(kind=4)                                :: lmom
      
!-----------------------------------------------------------------------
! following are needed for plots
!-----------------------------------------------------------------------
      integer(long), external :: slen
      integer(long)       :: f1,f2,i

!-----------------------------------------------------------------------
! zero dummy variables
!-----------------------------------------------------------------------
      xdum(1)=0.0
      ydum(1)=0.0

!-----------------------------------------------------------------------
! read DVR data (ort)
!-----------------------------------------------------------------------
      chkdvr=1
      dvrdata(1)=.true.      ! ort
      call rddvr(ort,rdum,rdum,rdum,zdum,zdum,idum,zdum,zdum,&
                  idum,idum,rdum,chkdvr)

!-----------------------------------------------------------------------
! allocate arrays for plotting
! dvrdat and grddat needed for label arrays
!-----------------------------------------------------------------------
      if (nmode .eq. 1 .and. nspfdof(1) .eq. 1) then
         nplottask=3
      else
         nplottask=5
      endif
      call alloc_plot

!-----------------------------------------------------------------------
! interactive plot defaults
!-----------------------------------------------------------------------
      call plotdefs

!-----------------------------------------------------------------------
! open trajectory file
!-----------------------------------------------------------------------
      open(unit=itmpin,file='trajectory',form='formatted')

!-----------------------------------------------------------------------
! Determine the number of data-sets (times).
! and write data (qcenter and pcenter) to scratch
!-----------------------------------------------------------------------
      time=(tinit-out2)/fs
      ndata=0
      rewind(ipsi)
      chkdvr=0
      chkgrd=0
      chkpsi=0
      chkdat=0
      psicount=-nskip
      read(ipsi) filever(ipsi)
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)

      if (ltraj) gwpdim = dim

   12 continue
      call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)

      if (lend) goto 13
      psicount=psicount+1
      time=time+out2/fs
      if (lgwp) then
         call gwp_param(psi,aGmat,gfgwp,aigwp,zgwp,qcenter,1)
         call gwp_cmom(psi,pcenter)
      else if (ltraj) then
         call extr(psi(zetf(1,1)),qcenter,pcenter,nspfdof(1),dim(1,1))
      endif
      write(itmpin,*) time,(qcenter(i),i=1,qcentdim),&
                      (pcenter(i),i=1,qcentdim)
      if(psicount.le.0) goto 12 
      ndata=ndata+1
      if(psicount.lt.ncut)  goto 12
   13 nplots = ndata

!-----------------------------------------------------------------------
! if only writing the trajectory file, stop now
!-----------------------------------------------------------------------
      if (ltrjfile) stop

!-----------------------------------------------------------------------
! allocate arrays for plotting
!-----------------------------------------------------------------------
      maxdata = ndata
      maxdata1 = ndata

      allocate(pltime(maxdata))    
      allocate(pldata(maxdata,maxspf,nstate))
      allocate(pldata1(maxdata1,maxspf,nstate))

      pltime=0.0_dop
      weight=0.0_dop

      rewind(itmpin)
      do n=1,ndata
         read(itmpin,*) pltime(n)
      enddo

!-----------------------------------------------------------------------
! interactive plotting
! open file for data to be plotted
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/tmpplot.pl'
      open(iaus,file=filename,status='unknown')

!-----------------------------------------------------------------------
! menu defaults
! set lplcoo2tim at start to set up time dof
!-----------------------------------------------------------------------
      if (plottask .eq. 1 .or. plottask .eq. 2) then
         plotdim=1
         lpltimcoo=.true.
      else if (plottask .eq. 3) then
         plotdim=-1
         lplcootim=.true.
      else if (plottask .eq. 4 .or. plottask .eq. 5) then
         if (nmode .eq. 1 .and. nspfdof(1) .eq. 1) then
            print*,'Only 1 DOF - do ask for a 2 DOF trajectory'
            stop
         endif
         plotdim=-1
         lplcoo2tim=.true.
         f=axis(1)
         icut(f)=(gdim(f)/2)+1
         rcut(f)=ort(zort(f)-1+icut(f))
         f=axis(2)
         if (f .gt. 0) then
            icut(f)=(gdim(f)/2)+1
            rcut(f)=ort(zort(f)-1+icut(f))
         endif

      endif
! GWPs depend on state only for multi-set
      lstate=lmult
      call initplot(filename,plotdim)


      call timebnds(pltime,ndata)
      write(6,'(a,i6,a,2f11.3,a)') 'There are',nplots,&
             ' WFs read in. Times:',bound(1,ftime),bound(2,ftime),' fs' 
      nplots0 = nplots

!-----------------------------------------------------------------------
! set plot tasks
!-----------------------------------------------------------------------
      if (lgwp) then
         cplottask(1)='GWP Center with time'
         cplottask(2)='GWP Momentum with time'
         cplottask(3)='GWP Phase-space trajectory'
         if (nmode .gt. 1 .or. nspfdof(1) .gt. 1) then
            cplottask(4)='GWP X1, X2 trajectory'
            cplottask(5)='GWP P1, P2 trajectory'
         endif
      else if (ltraj) then
         cplottask(1)='Coordinate with time'
         cplottask(2)='Momentum with time'
         cplottask(3)='Phase-space trajectory'
         if (nmode .gt. 1 .or. nspfdof(1) .gt. 1) then
            cplottask(4)='X1, X2 trajectory'
            cplottask(5)='P1, P2 trajectory'
         endif
      endif
      plottaskold=plottask

      do i=1,nplottask
         cptlen(i)=slen(cplottask(i)) 
      enddo

!-----------------------------------------------------------------------
! calculate initial mode bounds
!-----------------------------------------------------------------------
      call inmodebnds(ort,plotdim)
      call timebnds(pltime,ndata)

!-----------------------------------------------------------------------
! initialise scan
!-----------------------------------------------------------------------
      if (scan .ne. 0) then
         task = 1
         plottask = scan
         axis(2) = 0
         lrecalc=.true.
      endif

!-----------------------------------------------------------------------
! call menu and select task
!-----------------------------------------------------------------------
    1 continue
      lpltimcoo=.false.
      lplcoo=.false.
      lplcoo2=.false.
      if (plottask .eq. 1 .or. plottask .eq. 2) then
         plotdim = 1
         lpltimcoo=.true.
! if plottask changed delete old coordinate selection and choose new 
         if (plottaskold .ne. 1 .and. plottaskold .ne. 2) then
            axis(1) = ftime
            axis(2) = spfdof(1,1)
            f=axis(2)
            if (f .gt. 0) then
               icut(f)=(gdim(f)/2)+1
               rcut(f)=ort(zort(f)-1+icut(f))
            endif
            plottaskold=plottask
            task = 26
            go to 110
         endif
        
      else if (plottask .eq. 3) then
         plotdim = -1
         lplcoo=.true.
         axis(2) = axis(1)
! if plottask changed delete old coordinate selection and choose new 
         if (plottask .ne. plottaskold) then
            f=axis(1)
            icut(f)=(gdim(f)/2)+1
            rcut(f)=ort(zort(f)-1+icut(f))
            plottaskold=plottask
            task = 25
            axis(1) = spfdof(1,1)
            axis(2) = axis(1)
            go to 110
         endif
      else if (plottask .eq. 4 .or. plottask .eq. 5) then
         plotdim = -1
         lplcoo2=.true.
! if plottask changed delete old coordinate selection and choose new 
         if (plottaskold .ne. 4 .and. plottaskold .ne. 5) then
            axis(1) = spfdof(1,1)
            axis(2) = spfdof(1,2)
            f=axis(1)
            icut(f)=(gdim(f)/2)+1
            rcut(f)=ort(zort(f)-1+icut(f))
            f=axis(2)
            if (f .gt. 0) then
               icut(f)=(gdim(f)/2)+1
               rcut(f)=ort(zort(f)-1+icut(f))
            endif
            plottaskold=plottask
            task = 20
            go to 110
         endif
      endif
      if (singleplot .eq. 1) then
         task = 1
      endif

  500 continue    ! start of scan loop
      if (scan .gt. 0 .and. axis(2).lt.ndof) then
         axis(2) = axis(2)+1                 ! go to next dof
         if (axis(2) .eq. feb) go to 500     ! do not plot feb

         write(6,'(2a)') 'Plotting mode : ',modelabel(axis(2))
      else if (scan .gt. 0 .and. axis(2).eq.ndof) then
         go to 501  ! end
      else
         call plotmenu(task,plottask,plotdim)
      endif

!-----------------------------------------------------------------------
! calculate trajectory if required
!-----------------------------------------------------------------------
      if (lrecalc) then
         lrecalc  = .false.
         lrebound = .true.

         if (plottask .ge. 0) then
            rewind(itmpin)
            n=0
   10       continue
            read(itmpin,*,end=77) time,(qcenter(i),i=1,qcentdim),&
                      (pcenter(i),i=1,qcentdim)
            if(time.lt.bound(1,ftime)-tiny) goto 10 
            if(time.gt.bound(2,ftime)+tiny) goto 77

            n=n+1
            if (plottask .eq. 1) then
               f=axis(2)
               lmom = .false.
               call gettraj(f,lmom,pldata,qcenter,pcenter,n)
            else if (plottask .eq. 2) then
               f=axis(2)
               lmom = .true.
               call gettraj(f,lmom,pldata,qcenter,pcenter,n)
            else if (plottask .eq. 3) then
               f=axis(1)
               lmom = .false.
               call gettraj(f,lmom,pldata,qcenter,pcenter,n)
               lmom = .true.
               call gettraj(f,lmom,pldata1,qcenter,pcenter,n)
            else if (plottask .eq. 4) then
               f=axis(1)
               lmom = .false.
               call gettraj(f,lmom,pldata,qcenter,pcenter,n)
               f=axis(2)
               lmom = .false.
               call gettraj(f,lmom,pldata1,qcenter,pcenter,n)
            else if (plottask .eq. 5) then
               f=axis(1)
               lmom = .true.
               call gettraj(f,lmom,pldata,qcenter,pcenter,n)
               f=axis(2)
               lmom = .true.
               call gettraj(f,lmom,pldata1,qcenter,pcenter,n)
            endif

            go to 10

   77       continue
         endif
      endif

!-----------------------------------------------------------------------
! get data bounds
!-----------------------------------------------------------------------
      if (lrebound) then
         f1=axis(1)
         f2=axis(2)
         if (plottask .eq. 3) then
            f2=nanmode+axis(1)
         endif
      endif

  110 continue
!-----------------------------------------------------------------------
! do task
! TASK=0 STOP  else return to menu
!-----------------------------------------------------------------------
      if (task .gt. 0) then
         if (plottask .eq. 1 .or. plottask .eq. 2) then
            f2=axis(2)
            m=dofspf(f2)
            xdim=ndata
            ydim=1
            nsets=gwpdim(m,1)
            nplots=1
            axislab(1)='Time [fs]'
            if (plottask .eq. 1) then
               plttitle='GWP centre coordinate'
               call mode_lab_corr(modelabel(f2),moderep)
               axislab(3)=moderep(1:slen(moderep))//&
                    ' ['//axunitlab(f2)(1:slen(axunitlab(f2)))//'] '
            else
               plttitle='GWP centre momentum'
               call mode_lab_corr(modelabel(f2),moderep)
               axislab(3)='Mom '//moderep(1:slen(moderep))//&
                    ' ['//axunitlab(f2)(1:slen(axunitlab(f2)))//'] '
            endif
            call dotask(task,plottask,plotdim,pltime,&
                 ydum,pldata,xdim,ydim,nsets,nplots,&
                 maxdata,1,maxspf,1,ort,ortdim,pltime,ndata,edum,1,&
                 pldata1,maxdata,1,maxspf,1,idum)

         else if (plottask .eq. 3) then
            f1=axis(1)
            m=dofspf(f1)
            xdim=ndata
            ydim=1
            nsets=gwpdim(m,1)
            nplots=1
            plttitle='GWP trajectory in phase-space'
            call mode_lab_corr(modelabel(f1),moderep)
            axislab(1)=moderep(1:slen(moderep))//&
                    ' ['//axunitlab(f1)(1:slen(axunitlab(f1)))//'] '
            call mode_lab_corr(modelabel(f1),moderep)      
            axislab(3)='Mom '//moderep(1:slen(moderep))
            call dotask(task,plottask,plotdim,xdum,&
                 ydum,pldata,xdim,ydim,nsets,nplots,&
                 maxdata,1,maxspf,1,ort,ortdim,pltime,ndata,edum,1,&
                 pldata1,maxdata,1,maxspf,1,idum)

         else if (plottask .eq. 4 .or. plottask .eq. 5) then
            f1=axis(1)
            f2=axis(2)
            m=dofspf(f1)
            xdim=ndata
            ydim=1
            nsets=gwpdim(m,1)
            nplots=1
            if (plottask .eq. 4) then
               plttitle='GWP trajectory in coordinate space'
               call mode_lab_corr(modelabel(f2),moderep)  
               axislab(1)=moderep(1:slen(moderep))//&
                    ' ['//axunitlab(f1)(1:slen(axunitlab(f1)))//'] '
               call mode_lab_corr(modelabel(f2),moderep)  
               axislab(3)=moderep(1:slen(moderep))//&
                    ' ['//axunitlab(f2)(1:slen(axunitlab(f2)))//'] '
            else
               plttitle='GWP trajectory in momentum space'
               call mode_lab_corr(modelabel(f1),moderep) 
               axislab(1)='Mom '//moderep(1:slen(moderep))//&
                    ' ['//axunitlab(f1)(1:slen(axunitlab(f1)))//'] '
               call mode_lab_corr(modelabel(f2),moderep) 
               axislab(3)='Mom '//moderep(1:slen(moderep))//&
                    ' ['//axunitlab(f2)(1:slen(axunitlab(f2)))//'] '
            endif
            call dotask(task,plottask,plotdim,pltime,&
                 ydum,pldata,xdim,ydim,nsets,nplots,&
                 maxdata,1,maxspf,1,ort,ortdim,pltime,ndata,edum,1,&
                 pldata1,maxdata,1,maxspf,1,idum)
         endif
         if ((task .eq. 1 .and. singleplot .eq. 1) .and. .not. lrecalc) &
               then
            continue
         else if (scan .gt. 0) then
            if (axis(2) .eq. ndof) then
               continue
            else
                lrecalc = .true.
               go to 500   ! next plot
            endif
         else
            goto 1
         endif
      endif

  501 continue
! delete trajectory scratch file
      close(unit=itmpin,status='delete')
! deallocate memory
      deallocate(pltime,pldata,pldata1)

      return
      end subroutine plottraj

!#######################################################################

      subroutine extr(psi,qcenter,pcenter,nspfdof,dim)

      integer(long)                                    :: e,n
      integer(long), intent(in)                        :: nspfdof,dim
      real(dop), dimension(nspfdof,dim), intent(out)   :: qcenter,pcenter
      complex(dop), dimension(nspfdof,dim), intent(in) :: psi

      do e=1,dim
         do n=1,nspfdof
            qcenter(n,e) = dble(psi(n,e))
            pcenter(n,e) = dimag(psi(n,e))
         enddo
      enddo

      return
      end subroutine extr

!#######################################################################

      subroutine gettraj(f,lmom,pldata,qcenter,pcenter,ndata)

use global
use aglobal
use griddatmod
use psidef
use compdatmod
use plot
use operdef

      implicit none

      integer(long)             :: m,g,s,n,s1
      integer(long), intent(in) :: f,ndata
      real(dop), dimension(maxdata,maxspf,nstate), intent(out) :: pldata
      real(dop), dimension(qcentdim), intent(in)               :: qcenter,pcenter
      logical(kind=4), intent(in)                              :: lmom

      m=dofspf(f)
      g=0
      do n=1,nspfdof(m)
         if (spfdof(n,m) .eq. f) g=n
      enddo
      if (lmom) then
         do s1=1,nplstates
            s=plstate(s1)
            call gettraj1(pcenter(zcent(m,s)),nspfdof(m),gwpdim(m,s),&
                 pldata,ndata,s1,g,maxdata,maxspf,nstate)
         enddo
      else
         do s1=1,nplstates
            s=plstate(s1)
            call gettraj1(qcenter(zcent(m,s)),nspfdof(m),gwpdim(m,s),&
                 pldata,ndata,s1,g,maxdata,maxspf,nstate)
         enddo
      endif

      return
      end subroutine gettraj

!#######################################################################

      subroutine gettraj1(qcenter,gdof,dim1,pldata,ndata,s,g,&
                 maxdata,maxspf,nstate)

      implicit none
     
      integer(long)                :: e
      integer(long), intent(in)    :: dim1,gdof,ndata,s,g,&
                                      maxdata,maxspf,nstate
      real(dop), dimension(gdof,dim1), intent(in)              :: qcenter
      real(dop), dimension(maxdata,maxspf,nstate), intent(out) :: pldata


      do e=1,dim1
         pldata(ndata,e,s)=qcenter(g,e)
      enddo

      return
      end subroutine gettraj1

!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------

      subroutine gwptrajinput

use global
use aglobal
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use plot
use gwptrajmod

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: iarg,ilbl,ierr,i,psilaenge
      character(len=80)       :: buf
      character(len=c5)       :: filename

      psifile  = ' '
      name  = './'
      fileout = 'gwptraj.pl'
      iarg = myiargc() 
      ncut = 999999
      nskip=0
      plottask = 1
      singleplot = 0
      scan = 0

      i = 0
 100  i=i+1
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if(buf .eq. '-i') then
            i = i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf(1:3) .eq. '-f ') then
            i = i+1
            call mygetarg(i,buf)
            psifile = buf
         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
            laeout=index(fileout,' ')-1
            fileout = fileout(1:laeout)//'.pl '
            
         else if( buf(1:5) .eq. '-skip ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nskip
            if (nskip .lt. 0) then
               write(6,'(/a)')&
               'Argument of -skip must be positive integer'
               stop
            endif
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -skip'
               stop
            endif

         else if( buf(1:5) .eq. '-task ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) plottask
            if (plottask .lt. 0) then
               write(6,'(/a)')&
               'Argument of -task must be positive integer'
               stop
            endif
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -task'
               stop
            endif

         else if( buf(1:4) .eq. '-1pl ') then
            singleplot=1

         else if( buf(1:4) .eq. '-trj ') then
            ltrjfile=.true.

         else if( buf(1:5) .eq. '-scan ') then
            scan = 1

         else if( buf .eq. '-gnupl4 ') then
            lgnu4 = .true.                 ! gnuplot ver < 5

         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(/1x,a)')&
       'Purpose: Enables 2D plotting of the GWP trajectories.'
            write(6,'(1x,2a)')&
       'Usage:   gwptraj [-i -f -o -skip -gnupl4 -ver -h -?]'
            write(6,'(/,a)') 'Options : '
            write(6,'(a)') ' -i DIR  : data is stored in directory DIR'
            write(6,'(2a,/,9x,a)')&
                     ' -f FILE : the wavefunction is read '&
                    ,'from FILE rather than from ./psi',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a)') &
                     ' -o FILE : The ouput is written to file ',&
                     'FILE.pl rather than to ./gwptraj.pl',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(a)')&
                     ' -skip m : the first m WFs are skipped.'
            write(6,'(a)')&
                     ' -task m : On startup plottask is m.'
            write(6,'(a)')&
                     ' -1pl    : Do single plot and finish.'
            write(6,'(a)')&
                     ' -trj : Write a trajectory file only.'
            write(6,'(a)')&
                     ' -scan : Plot coordinate trajectories of each &
                   &mode in turn.'
            write(6,'(a)')&
               ' -gnupl4  : Use gnuplot version < 5 for plotting '
            write(6,'(a)')&
                     ' -ver    : Version information about the program.'
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(/2x,a,/2x,2a,/2x,2a)')&
            'The program is menu driven. Just type "gwptraj"' ,&
            'and then follow ','the menu.',&
            '-----------------------------',&
            '---------------------------------------------------'

            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         goto 100
      end if
      
  200 continue
      nopt=i-1

      laenge=index(name,' ')-1
      call abspath(name,laenge)

      if (psifile .eq. ' ') psifile=name(1:laenge)//'/psi'
      psilaenge=index(psifile,' ')-1
      call abspath(psifile,psilaenge)

      dname=name
      dlaenge=index(dname,' ')-1

      laeout=index(fileout,' ')-1
      call abspath(fileout,laeout)

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      filename=psifile
      open(ipsi,file=filename,form='unformatted',status='old',&
           err=1000)
      write(6,'(2a)') ' The psi file is : '//psifile

      return 
      
 1000 write(6,*)' Cannot open file : '
      ilbl=index(filename,' ')-1
      write(6,*) filename(1:ilbl)

      stop
      end subroutine gwptrajinput


      end program gwptraj

