

module linkedlist_m


  type linkedlist_t
     double precision, allocatable  :: ddata(:)
     integer, allocatable           :: idata(:)
     complex*16, allocatable        :: zdata(:)

     integer :: ddim, idim, zdim

     type(linkedlist_t), pointer :: next
     type(linkedlist_t), pointer :: prev
  end type linkedlist_t


  ! constructor like overloading
  ! of type name with a function
  interface linkedlist_t
     module procedure new_list_element
  end interface linkedlist_t


contains


  function new_list_element(ddata, idata, zdata) result (newelem)
    implicit none
    double precision, intent(IN), optional :: ddata(:)
    integer, intent(IN), optional          :: idata(:)
    complex*16, intent(IN), optional       :: zdata(:)

    type(linkedlist_t), pointer            :: newelem

    allocate(newelem)

    if (present(ddata)) then
       newelem%ddim = size(ddata)
       allocate(newelem%ddata(newelem%ddim))
       newelem%ddata = ddata
    else
       newelem%ddim = 0
    end if

    if (present(idata)) then
       newelem%idim = size(idata)
       allocate(newelem%idata(newelem%idim))
       newelem%idata = idata
    else
       newelem%idim = 0
    end if

    if (present(zdata)) then
       newelem%zdim = size(zdata)
       allocate(newelem%zdata(newelem%zdim))
       newelem%zdata = zdata
    else
       newelem%zdim = 0
    end if

    nullify(newelem%prev)
    nullify(newelem%next)

  end function new_list_element





  ! insert a new element into the linked list
  ! where self is one element in an already
  ! existing list.
  subroutine insert_after(self, ddata, idata, zdata)
    implicit none
    type(linkedlist_t), pointer, intent(INOUT) :: self
    double precision, intent(IN), optional :: ddata(:)
    integer, intent(IN), optional          :: idata(:)
    complex*16, intent(IN), optional       :: zdata(:)


    type(linkedlist_t), pointer :: newelem, bef,aft

    newelem => new_list_element(ddata, idata, zdata)



    bef => self%prev
    aft => self%next

    newelem%prev => self
    self%next => newelem

    newelem%next => aft
    if (associated(aft)) then
       aft%prev => newelem
    end if


  end subroutine insert_after


  subroutine insert_bef(self, ddata, idata, zdata)
    implicit none
    type(linkedlist_t), pointer, intent(INOUT) :: self
    double precision, intent(IN), optional :: ddata(:)
    integer, intent(IN), optional          :: idata(:)
    complex*16, intent(IN), optional       :: zdata(:)


    type(linkedlist_t), pointer :: newelem, bef,aft

    newelem => linkedlist_t(ddata, idata, zdata)


    bef => self%prev
    aft => self%next

    newelem%next => self
    self%prev => newelem

    newelem%prev => bef
    if (associated(bef)) then
       bef%next => newelem
    end if

  end subroutine insert_bef



  subroutine free_list(self)
    implicit none
    type(linkedlist_t), pointer, intent(INOUT) :: self
    type(linkedlist_t), pointer :: current, next
    integer ::zipcount


    current => self


    ! zip to start of list (let's hope its not a circle)
    zipcount = 0
    do while (associated(current%prev))
       current => current%prev
       zipcount = zipcount +1
       if (zipcount .gt. 1000000) then
          print*, "There seems to be a circular list. Stopping."
          print*, "File source/analyse/linkedlist.f90"
          stop 1
       end if
    end do


    do while (associated(current))
       next => current%next
       if (allocated(current%ddata)) deallocate(current%ddata)
       if (allocated(current%idata)) deallocate(current%idata)
       if (allocated(current%zdata)) deallocate(current%zdata)
       deallocate(current)
       nullify(current)
       current => next
    end do


  end subroutine free_list






end module linkedlist_m




