#!/usr/bin/env python3
"""
-------------------------------------------------------------------------------
Purpose   : Reads the lownatpop file and prints the maximum over time
            of the populations for all listed nodes and modes.
            The four highest and four lowest populations will be marked.

Usage     : lownp.py [-h, -f, -t, -last, -min, -max]
Options   :  -h  Print this help-text (--help and -? also work).
             -f <path>      "path" is the path to the file to be analyzed.
                            If the -f option is not given,
                            path=./lownatpop is assumed.
             -tmin <ti>     Analyze only data with time >= ti.
             -tmax <tx>     Analyze only data with time <= tx.
             -last          Analyze only data of the last time point.
             -min <minpop>  Mark all populations which
                            are < mimpop with '+'.
             -max <maxpop>  Mark all populations which
                            are > maxpop with '*'.
             -skip          Skip the first two lines.
                            This is useful when electronic states are present.

Examples  : lownp.py -f ../run2/lownatpop
            lownp.py -max 1.e-4 | grep '*'

This script has been tested with Python version 2.7.16
HDM 02/20
-------------------------------------------------------------------------------
"""

import sys

filename='lownatpop'
start=0.0
stop=9999999999.9
last=0
ii=1
bb=1
minpop=-1.0
maxpop=2.0

while  len(sys.argv) > ii:
#  -h then print help
    if sys.argv[ii] == "-h" or sys.argv[1] == "-?" or sys.argv[1] == "--help":
        sys.exit(__doc__)
    elif sys.argv[ii] == "-f":
        filename=sys.argv[ii+1]
        ii=ii+2
    elif sys.argv[ii] == "-tmin":
        start=float(sys.argv[ii+1])
        ii=ii+2
    elif sys.argv[ii] == "-tmax":
        stop=float(sys.argv[ii+1])
        ii=ii+2
    elif sys.argv[ii] == "-min":
        minpop=float(sys.argv[ii+1])
        ii=ii+2
    elif sys.argv[ii] == "-max":
        maxpop=float(sys.argv[ii+1])
        ii=ii+2
    elif sys.argv[ii] == "-last":
        last=1
        ii=ii+1
    elif sys.argv[ii] == "-skip":
        bb=3
        ii=ii+1
    else:
        print('Unknown option:', sys.argv[ii])
        sys.exit()

infile = open(filename, 'r')
infile.readline()  # skip first line

filestring = infile.readline()
layer=filestring.split()
filestring = infile.readline()
node=filestring.split()
filestring = infile.readline()
mode=filestring.split()
wrdlen=len(mode)
mxpop=[0.0]*wrdlen
tim=['  ']*wrdlen
mark=['    ']*wrdlen

for line in infile:
    if '#' not in line[0:1]:
       words=line.split()
       wordsf=[float(w) for w in words]
       if start <= wordsf[0] <= stop:
           tfin=words[0]
           for n in range(1,wrdlen):
               mxpop[n]=max(mxpop[n],wordsf[n])
               if mxpop[n] == wordsf[n]:
                   tim[n]=words[0]

print('Reading    file:', filename, 'Time :', tfin)

if last==1:
    mxpop=wordsf
    tim=[words[0]]*wrdlen

maxmax=0.0
n1=0
for n in range(bb,wrdlen):
    maxmax=max(mxpop[n],maxmax)
    if mxpop[n] == maxmax:
        n1=n
mark[n1]='####'

maxmax=0.0
n2=0
for n in range(bb,wrdlen):
    if n != n1:
        maxmax=max(mxpop[n],maxmax)
        if mxpop[n] == maxmax:
            n2=n
mark[n2]='### '

maxmax=0.0
n3=0
for n in range(bb,wrdlen):
    if n != n1 and n != n2:
        maxmax=max(mxpop[n],maxmax)
        if mxpop[n] == maxmax:
            n3=n
mark[n3]='##  '

maxmax=0.0
n4=0
for n in range(bb,wrdlen):
    if n != n1 and n != n2 and n != n3:
        maxmax=max(mxpop[n],maxmax)
        if mxpop[n] == maxmax:
            n4=n
mark[n4]='#   '


minmin=2.0
n1=0
for n in range(bb,wrdlen):
    minmin=min(mxpop[n],minmin)
    if mxpop[n] == minmin:
        n1=n
mark[n1]='----'

minmin=2.0
n2=0
for n in range(bb,wrdlen):
    if n != n1:
        minmin=min(mxpop[n],minmin)
        if mxpop[n] == minmin:
            n2=n
mark[n2]='--- '

minmin=2.0
n3=0
for n in range(bb,wrdlen):
    if n != n1 and n != n2:
        minmin=min(mxpop[n],minmin)
        if mxpop[n] == minmin:
            n3=n
mark[n3]='--  '

minmin=2.0
n4=0
for n in range(bb,wrdlen):
    if n != n1 and n != n2 and n != n3:
        minmin=min(mxpop[n],minmin)
        if mxpop[n] == minmin:
            n4=n
mark[n4]='-   '


format='%9s %9s %7s  time = %7s   max-pop = %.3E %s %s'
for n in range(bb,wrdlen):
    mk=' '
    if mxpop[n] > maxpop:
        mk='*'
    if mxpop[n] < minpop:
        mk='+'
    print(format % (layer[n+1],node[n],mode[n],tim[n],mxpop[n],mark[n],mk))
infile.close()

quit()

