!
!

      program maxpe

use decimal, only: long,dop,sip
use global
use maxv
use globmemmod
use aglobal
use versions
use griddatmod
use operdef
use psidef
use channels
use datenmod
use compdatmod
use compdat1mod

use rddvrmod
use rdopermod, only: operinfo
use zeigauswmod, only: zeigausw
use memhopsmod, only: memhops

use module_interface

      implicit none

      integer(long), external           :: myiargc 
      integer(long)                     :: ilbl,chkdvr,chkgrd
      character(len=c5)                 :: filename
      logical(kind=4)                   :: lerr,check

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault

!-----------------------------------------------------------------------
! Get name of data directory 'name/some_data'
!-----------------------------------------------------------------------
      if (myiargc().lt.1) then
         write(6,*) 'Name of directory   :'
         read(5,'(a)') name
         write(6,*)
      else
         call mygetarg(1,name)
         if( name(1:2).eq.'-h' .or. name(1:2).eq.'-?') then
            write(6,'(a,/,1x,a/,1x,a,/,1x,a/,1x,a,/,1x,a)') &
        '-------------------------------------------------------------',&
        'Purpose : Calculates the minimum and maximum value of a PES',&
        'Usage   : maxpe [-h -ver ] name-directory',&
        'Generate an oper file for an exact calculation and then run',&
        ' maxpe with the path to that directory as argument.'
            write(6,'(a,/a,/a)')&
        ' -ver    : Version information about the program ',&
        ' -h      : Print this help text ',&
        'NB: The routine vminmax is a more powerful alternative which',&
        'does not require an operator in exact, but in pes format',&
        '-------------------------------------------------------------'
           stop
         else if( name(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         endif
      endif
      laenge=index(name,' ')-1
      call abspath(name,laenge)

!-----------------------------------------------------------------------
! get array dimensions 
!-----------------------------------------------------------------------
      inquire(irst,opened=check)
      if (check) close(irst)
      filename=name(1:laenge)//'/restart'
      ilbl=index(filename,' ')-1
      open(irst,file=filename(1:ilbl),form='unformatted',status='old',err=1000)
      call rdmemdim(irst)
      close(irst)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='maxpe.log',form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
! read info-data from dvr and oper files.
!-----------------------------------------------------------------------
      dname=name
      dlaenge=laenge
      filename=dname(1:dlaenge)//'/dvr'
      open(idvr,file=filename,form='unformatted',status='old',err=1000)
      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      write(ilog,'(a)') ' The  DVR file is: ',filename(1:dlaenge+4)
      close(idvr)

      oname=name
      olaenge=laenge
      filename=oname(1:olaenge)//'/oper'
      open(ioper,file=filename,form='unformatted',status='old',err=1000)
      chkdvr=2
      chkgrd=1
      call operinfo(lerr,chkdvr,chkgrd)
      write(ilog,'(a)') ' The OPER file is: ',filename(1:olaenge+5)
      close(ioper)

!-----------------------------------------------------------------------
! assign standard arrays required
!-----------------------------------------------------------------------
      lort=.true.
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
! here assign any further memory arrays
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
! assign memory for operator
!-----------------------------------------------------------------------
      call memhops

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

      call submaxpe(mr(mrhops),mr(mrort))

deallocate(mc,mr,mi,ml,ms,mf)


      stop
 1000 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      write(6,'(a)') message

      contains

!#######################################################################

      subroutine submaxpe(hops,ort)

use global
use aglobal
use maxv, only: maxdim
use griddatmod
use operdef

use rddvrmod, only: rddvr
use rdopermod, only: rdoper

      implicit none

      integer(long)                              :: k,f,h,zeig,k1,s,g,i,chkdvr,chkgrd
      integer(long)                              :: m,n
      integer(long), dimension(1)                :: idum
      real(dop)                                  :: maxval,minval,maxpnt,minpnt
      real(dop), dimension(1)                    :: rdum
      real(dop), dimension(maxdim)               :: coord
      real(dop), dimension(ortdim), intent(out)  :: ort
      real(dop), dimension(hopsdim), intent(out) :: hops
      complex(dop), dimension(1)                 :: zdum

      chkdvr=1
      chkgrd=1
      minval=0.0_dop
      maxval=0.0_dop

      dvrdata(1)=.true.
      call rddvr(ort,rdum,rdum,rdum,zdum,zdum,idum,zdum,zdum,idum,&
                 idum,rdum,chkdvr)

      operfile=oname(1:olaenge)//'/oper'
      call rdoper(hops,chkdvr,chkgrd)

      if(nmode .gt. 1) then
         write(6,*) 'There is more than 1 mode, so this is probably not an &
                     &operator in exact format!' 
         write(6,*) 'As an alternative, ',&
                    'generate a PES file and use vminmax.' 
      endif  

      write(6,'(a,i8)') 'Dimension of grid :', subdim(1)
      write(6,*)
      
      do n=1,nspfdof(1)
         f=spfdof(n,1)
         zeig=zort(f)-1
         if (basis(f).eq.2 .or. basis(f).eq.12) then
            write(6,'(2a,2f12.6)') modelabel(f)(1:10),&
              'Grid min / max (deg) = ',&
              ort(zeig+1)*(180.0/pi),ort(zeig+gdim(f))*(180.0/pi)
         else
            write(6,'(2a,2f12.6)') modelabel(f)(1:10),&
            'Grid min / max (au) = ',ort(zeig+1),ort(zeig+gdim(f))
         endif
      enddo

      maxpnt=0
      minpnt=-999
      m=1
      coord = 0.0
      do s=1,nstate
         if (nstate .gt. 1) then
            write(6,'(/,a,i0)') ' *** State ',s     
            write(6,*)
         endif
         do k1=1,khzahl(m,s,1)
            k=zhun(m,s,1)+k1-1
            f=spfdof(1,1)
            h=hamilton(f,k)
            if (htmtype(h) .eq. 2 .and. htmmode(h) .eq. 1) then
               minval= 999999.0
               maxval=-999999.0
               zeig=zhop(h)-1
               do g=1,subdim(1)
                  if (hops(zeig+g) .gt. maxval) then
                     maxval=hops(zeig+g)
                     maxpnt=g
                  endif
                  if (hops(zeig+g) .lt. minval) then
                     minval=hops(zeig+g)
                     minpnt=g
                  endif
               enddo
               
               do n=1,nspfdof(1)
                  f=spfdof(n,1)
                  zeig=zort(f)-1
                  i=nint(maxpnt)-1
                  g=i/vgdim(f) - (i/(vgdim(f)*gdim(f)))*gdim(f)
                  g=g+1
                  coord(f)=ort(zeig+g)
                  if (basis(f).eq.2 .or. basis(f).eq.12) &
                       coord(f)=coord(f)*(180.0/pi)
               enddo
               write(6,'(/a,f20.8)') 'Maximum PE value (eV) = ',maxval*ev
               write(6,'(a5)') 'at'
               write(6,'(6f12.6)') (coord(f),f=1,ndof)
               
               do n=1,nspfdof(1)
                  f=spfdof(n,1)
                  zeig=zort(f)-1
                  i=nint(minpnt)-1
                  g=i/vgdim(f) - (i/(vgdim(f)*gdim(f)))*gdim(f)
                  g=g+1
                  coord(f)=ort(zeig+g)
                  if (basis(f).eq.2 .or. basis(f).eq.12) &
                       coord(f)=coord(f)*(180.0/pi)
               enddo
               write(6,'(/a,f20.8)') 'Minimum PE value (eV) = ',minval*ev
               write(6,'(a5)') 'at'
               write(6,'(6f12.6)') (coord(f),f=1,ndof)
               
            endif
         
         enddo
      enddo
      
      return
      end subroutine submaxpe

      end program maxpe

      
