!***********************************************************************
!
!          MCTDH   subroutine MEMAUSW
!
! called from almost all analyse routines
!
! calculate pointers and dimensions for standard arrays.
!
!
! 10/96 aJ
!
!
!***********************************************************************

      subroutine memausw

use decimal, only:long,dop
use global
use globmemmod
use aglobal
use dvrdatmod
use griddatmod
use psidef
use operdef
use datenmod
use compdatmod
use compdat1mod
use dirdyn

use compdat, only: getcsys,getcpsi
use auswutil, only: getcdat

      implicit none

!-----------------------------------------------------------------------
! with the parameter "pdum" the pointers to arrays that are not
! used are initialized with the value one in order to avoid an abortion
! of the program when array bounds are checked (compiler option "-c").
!-----------------------------------------------------------------------
      integer(long), parameter :: pdum = 1

!-----------------------------------------------------------------------
! if comparison data sets sets are used, set standard data to set 1
!-----------------------------------------------------------------------
      if (ncomp .gt. 0) then
         call getcsys(1)
         call getcpsi(1)
         call getcdat(1)
      endif

!-----------------------------------------------------------------------
! assign standard arrays to memory arrays
! mcdim, mrdim, midim and mldim are final addresses assigned in mc, 
!  mr, mi and ml. 
!-----------------------------------------------------------------------
! set pointers to complex arrays in memory array mc 
!-----------------------------------------------------------------------
      mcdim=1
      if (lpsi .and. .not. lpsigrd) then
         mcpsi=mcdim
         mcdim=mcdim+dgldim
         if (lgwp .and. psitype .eq. 0) then
            mcadgwp = mcdim
            mcdim = mcadgwp+gsmatdim
         else
            mcadgwp = pdum
         endif

      else if (lpsi .and. lpsigrd) then
         mcpsi=mcdim
         mcdim=mcdim+griddim
         mcadgwp = pdum

      else 
         mcpsi=pdum
         mcadgwp = pdum
      endif
      if (lpsi1 .and. .not. lpsigrd) then
         mcpsi1=mcdim
         mcdim=mcdim+cdgldim(2)
         mcadgwp1=mcdim
         mcdim=mcdim+cgsmatdim(2)
      else if (lpsi1 .and. lpsigrd) then
         mcpsi1=mcdim
         mcdim=mcdim+max(cdgldim(2),griddim)
         mcadgwp1=pdum
      else
         mcpsi1=pdum
         mcadgwp1=pdum
      endif

      if (lfft) then
         mchin=mcdim
         mcdim=mchin+fftdim
         mcrueck=mcdim
         mcdim=mcrueck+fftdim
         mcexphin=mcdim
         mcdim=mcexphin+expdim
         mcexprueck=mcdim
         mcdim=mcexprueck+expdim
      else
         mchin=mcdim
         mcdim=mchin+2
         mcrueck=mcdim
         mcdim=mcrueck+2
         mcexphin=mcdim
         mcdim=mcexphin+2
         mcexprueck=mcdim
         mcdim=mcexprueck+2
      endif

!-----------------------------------------------------------------------
! set pointers to real arrays in memory array mr 
!-----------------------------------------------------------------------
      mrdim=1
      if (lgwp) then
         mrgsval = mrdim
         mrdim   = mrgsval + gwpinfodim + 1
      else
         mrgsval = pdum
      endif

      if (lort) then
         mrort=mrdim
         mrdim=mrort+ortdim
         if (lgwp .or. ltraj) then
            mrqcent = mrdim
            mrdim   = mrqcent + qcentdim + 1
         else
            mrqcent = pdum
         endif

      else
         mrort=pdum
         mrqcent = pdum
      endif
      if (lgpop) then
         mrgpop1=mrdim
         mrdim=mrgpop1+ortdim*maxsta
         mrgpop2=mrdim
         mrdim=mrgpop2+ortdim*maxsta
         mrweight=mrdim
         mrdim=mrweight+maxgdim
      else
         mrgpop1=pdum
         mrgpop2=pdum
         mrweight=pdum
      endif
      if (ltrafo) then
         mrtrafo=mrdim
         mrdim=mrtrafo+dvrdim
      else
         mrtrafo=pdum
      endif
      if (lsphfbr) then 
         mrkinsph=mrdim
         mrdim=mrkinsph+3*sphdim
      else
         mrkinsph=pdum
      endif

!-----------------------------------------------------------------------
! set pointers to integer arrays in memory array mi 
!-----------------------------------------------------------------------
      midim   = 1
      if (lpsi .and. lpsiopt(2)) then
         mijindx = midim
         midim = mijindx+adim
      else
         mijindx = pdum
      endif
      if (lpsi1 .and. clpsiopt(2,2)) then
         mijindx1 = midim
         midim = mijindx1+cadim(2)
      else
         mijindx1 = pdum
      endif
      if (lpsi .and. ltraj) then
         mitrajst = midim
         midim = mitrajst + gwpinfodim
      else
         mitrajst = pdum
      endif
      if (lgwp) then
         migwpdep = midim
         midim = migwpdep + gwpinfodim + 1
      else
         migwpdep = pdum
      endif
      if (lsphfbr) then 
         mijsph=midim
         midim=mijsph+sphdim
         mimsph=midim
         midim=mimsph+sphdim
      else
         mijsph=pdum
         mimsph=pdum
      endif
      if (lfft) then
         mifftfak=midim
         midim=mifftfak+fftdim
      else
         mifftfak=midim
         midim=mifftfak+2
      endif


!----------------------------------------------------------------------
! set pointers to logical arrays in memory array ml 
!-----------------------------------------------------------------------
      mldim   = 1

!----------------------------------------------------------------------
! set pointers to single precision complex arrays in memory array ms 
!-----------------------------------------------------------------------
      msdim   = 1
      if (lpsi) then
         mspsi   = msdim
         msdim   = mspsi+spsidim
      else
         mspsi   = pdum
      endif

!----------------------------------------------------------------------
! set pointers to single precision real arrays in memory array mf 
!-----------------------------------------------------------------------
      mfdim   = 1
      if (lgpop) then
         mfgpop = mfdim
         mfdim = mfgpop+2*maxgdim
      else
         mfgpop = pdum
      endif

      return
      end subroutine memausw

