!#######################################################################
!
! Merges two direct-dynamics databases (DB only)
!
!#######################################################################

      program mergedb


use decimal, only: sip
use global
use versions

!use channels
use dirdyn
use db_sql
use dd_db, only: distdb

      implicit none
      integer(long)     :: ierr,dbpoint
      integer(long)     :: atmchk,rootchk,statechk,basischk,formchk
      integer(long)     :: ndof1,i,iarg
      integer(long),dimension(2)     :: nrecin

      integer(long), external :: myiargc

      character(len=10)   :: cerr,buf
      character(len=c5)  :: ddname1,ddname2
      integer(long)      :: ddlaenge1,ddlaenge2
      character(len=c5)  :: dbname1,dbname2

      integer(long), dimension(1)                   :: interp
      integer(long), dimension(:,:), allocatable    :: symrep
      real(dop), dimension(:,:), allocatable        :: gpoint,nmpoint
      real(dop), dimension(:,:,:), allocatable      :: v,diabtrans
      real(dop), dimension(:,:,:), allocatable      :: cv,diabctrans
      real(dop), dimension(:,:,:,:), allocatable    :: deriv1
      real(dop), dimension(:,:,:,:,:), allocatable  :: deriv2
      real(dop), dimension(:,:,:,:), allocatable    :: dip
      real(dop), dimension(:,:,:), allocatable      :: av
      real(dop), dimension(:,:,:,:), allocatable    :: aderiv1
      real(dop), dimension(:,:,:,:,:), allocatable  :: aderiv2
      real(dop), dimension(:,:,:,:), allocatable    :: adip
      real(sip), dimension(:,:,:), allocatable      :: mocoeff
      real(dop), dimension(:,:,:), allocatable      :: nact
      complex(dop), dimension(:,:,:), allocatable   :: casoc

      real(dop), dimension(:), allocatable        :: r1

      real(dop), dimension(:), allocatable        :: mref1,mref2
      character(len=c1), dimension(:), allocatable:: atnam1,atnam2
      real(dop), dimension(:,:), allocatable      :: xref1,xref2
      real(dop)                                   :: dde01,dde02,xdiff
      real(dop)                                   :: mindist,mindisp
      integer(long)                               :: idbloc
      integer(long)                               :: dimref1,nfam1,&
                                                     dimref2,nfam2,&
                                                     dercpdim2,j,&
                                                     dbpntgp2,nsymop2, timestamp

      logical(kind=4)     :: lbatch,lerror
      logical(kind=4)     :: checkrec
      type(SQLITE_DATABASE), dimension(2)         :: dbin
      integer(long)                               :: irec,rowid
      real(dop)                                   :: dist,mindb
      character(len=3)                            :: goahead
      logical(kind=4)                             :: fail=.false.
      integer(long)                               :: natm2,&
         nroot2,nbasis2,nddstate,nddstate2,nmdof,nsocstate2
      integer(long), dimension(7)                 :: nddmult2
      logical(kind=4)                             :: lpesdb2,lgradb2,&
        lhesdb2,lapesdb2,lagradb2,lahesdb2,ldipdb2,ladipdb2,lmocodb2,&
        lnactdb2,ltransdb2,lnmdb2,lcasocdb2,lsymdb2,lcpesdb2,&
        lctransdb2,lcivecdb2,ltemplate2,lqcfreq2

! Setting defaults
! lbatch: if true no output ot screen (batch job)
      
      lbatch = .false.
      ddname1 = './'
      ddname2 = './'
      checkrec=.false.
      dist = 0.05
! Set mindb to default used in creating DBs
      mindb = ddmindb
      dbpoint = 2
      lerror = .false.

! default
      nbasis=-999
      nbasis2=-999
      nddstate=-999
      nddstate2=-999
      nroot=-999
      nroot2=-999

! read options
      iarg = myiargc()

      i=0
! j is no. of options
      j=0
      do while (i .le. iarg)
         i=i+1
         call mygetarg(i,buf)
         if ( buf .eq. '-ver' ) then
            call wrversion(6)
            stop
         else if ( buf .eq. '-mindb' ) then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*) mindb
            j=j+2
         else if ( buf .eq. '-batch' ) then
            lbatch = .true.
            i=i+1
            j=j+1
         else if ( buf .eq. '-dbp' ) then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*) dbpoint
            j=j+2
         else if ( buf .eq. '-h' .or. buf(1:2) .eq. '-?') then
            write(6,'(/,78a1,/)')  ('*',i=1,78)
            write(6,'(a)') ' Analyse program "mergedb" '
            write(6,'(1x,a)')&
       'Purpose : Merges two DD DBs into one'
            write(6,'(1x,2a)') 'Usage   :  ',&
       '[-mindb R -dbp I -batch -h -?] DBold DBnew '
            write(6,'(a)')  'Where new records in DBnew are written to DBold'
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(/,a)') ' Options : '
            write(6,'(a,/,a,f5.3)') &
            ' -mindb R : A DB entry is not written if it is closer ',&
            '            than R to any entries in DBold. Default R = ',ddmindb
            write(6,'(a,2(/,a))')&
            ' -dbp   I : Change definition of distance.',&
            '            I=1 Use RMSD ',&
            '            I=2 Use maximum atom difference (Default) '
            write(6,'(a)') &
            ' -batch  : Batch mode - no output to the screen '
            write(6,'(a,/a)') &
            ' -h      : Print this help text. ',&
            ' -?      : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop

         endif
      end do

      if ((iarg - j) .ne. 2) then
         write(6,'(a)') 'ERROR: require two folders containing DBs.'
         write(6,'(a)') '       Second DB is merged in to first.'
         stop
      endif   

       call mygetarg(iarg-1,ddname1)
       call mygetarg(iarg,ddname2)

!
! get full path
!
      ddlaenge1=index(ddname1,' ')-1
      ddlaenge2=index(ddname2,' ')-1
      call abspath(ddname1,ddlaenge1)
      call abspath(ddname2,ddlaenge2)

      dbname1 = trim(ddname1) // '/database.sql'
      dbname2 = trim(ddname2) // '/database.sql'

!
!     open the input databases: first read-write,
!     second read-only.
!     !Needed for NEWSQL
      call makeflagarray
      call makeinfoarray(nddstate)
      call dbopen(dbname1, dbin(1),sqlvers,quanticsver,infoarray,flagarray, 1 )
      call dbopen(dbname2, dbin(2),sqlvers,quanticsver,infoarray,flagarray, 0 )

!
! get data from refdb.dat files
!
      call dbrdrefdb(dbin(1), natm, nroot, nddstate, nbasis, dde01,&
                     dimref1, nfam1, dercpdim, dbpntgp, nsymop, &
                     nsocstate, nddmult)
      call dbrdrefdbgeom(dbin(1), natm, mref1, atnam1, xref1)
      call dbrdrefdb(dbin(2), natm2, nroot2, nddstate2, nbasis2, dde02,&
                     dimref2, nfam2, dercpdim2, dbpntgp2, nsymop2, &
                     nsocstate2, nddmult2)
      call dbrdrefdbgeom(dbin(2), natm2, mref2, atnam2, xref2)

!
! Check that refdb files for the same molecule/same format
!
       atmchk=natm-natm2
       rootchk=nroot-nroot2
       statechk=nddstate-nddstate2
       basischk=nbasis-nbasis2
       formchk=abs(atmchk)+abs(rootchk)+abs(statechk)+abs(basischk)
       if (formchk .ne. 0) then
         write(6,'(a)')'ERROR refdb files of differing format.'
         write(6,'(a)')'Non-zero values below  are where the error occurs:'
         write(6,*) 'Natm',atmchk,'Nroot',rootchk,'Nddstate',statechk,&
                    'Nbasis',basischk 
         lerror = .true.
      endif
      if (dde01.ne.dde02) then
         write(6,'(a)')'WARNING: energy origins are different'
         lerror = .true.
      endif
      if (dimref1.ne.dimref2) then
         write(6,'(a)')'WARNING: symmetry(dimref)s are different'
         lerror = .true.
      endif
      if (nfam1.ne.nfam2) then
         write(6,'(a)')'WARNING: symmetry(nfam)s are different'
         lerror = .true.
      endif
      if (dercpdim.ne.dercpdim2) then
         write(6,'(a)')'WARNING: dercpdims are different'
         lerror = .true.
      endif
      if (.not.all(mref1==mref2)) then
         write(6,'(a)')'WARNING: masses are different'
         lerror = .true.
      endif
      if (.not.all(atnam1==atnam2)) then
         write(6,'(a)')'WARNING: names are different'
         lerror = .true.
      endif
      xdiff = 0.0_dop
      do i=1,natm
         do j=1,3
            xdiff = xdiff + (xref1(j,i)-xref2(j,i))**2
         enddo
      enddo
      if (xdiff .gt. tiny) then
         write(6,'(a)')'WARNING: Xcoo are different'
         lerror = .true.
      endif
      if (lerror) stop
      
! If databases differ in uses of symmetry, check it is OK to continue
      if (dbpntgp .ne. dbpntgp2) then
         write(6,'(a)') 'WARNING: One DB uses symmetry, the other does not'
         write(6,'(a)') 'Continue (y/n)?'
         read(5,'(a)') buf
         if (buf(1:1) .eq. "n") stop
      endif

! Find out what's in the databases
      write(6,'(/,a)') 'DB 1 Entries'
      call dbgetentries( dbin(1), 6, lpesdb, lgradb, lhesdb, lapesdb,&
           lagradb, lahesdb, ldipdb, ladipdb, lmocodb, lnactdb, ltransdb, &
           lnmdb, lcasocdb, lsymdb, lcpesdb, lctransdb,lcivecdb, ltemplate, &
           lqcfreq)
      write(6,'(/,a)') 'DB 2 Entries'
      call dbgetentries( dbin(2), 6, lpesdb2, lgradb2, lhesdb2, lapesdb2,&
           lagradb2, lahesdb2, ldipdb2, ladipdb2, lmocodb2, lnactdb2,&
           ltransdb2, lnmdb2, lcasocdb2, lsymdb2, lcpesdb2, lctransdb2, &
           lcivecdb2, ltemplate2, lqcfreq2)

      if     ( lpesdb .neqv.lpesdb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are energies'
        fail = .true.
      else if     ( lcpesdb .neqv.lcpesdb2 ) then
        write(6,'(a)') &
            'Input databases disagree whether there are complex energies'
        fail = .true.
      elseif ( lgradb.neqv.lgradb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are gradients'
        fail = .true.
      elseif ( lhesdb.neqv.lhesdb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are Hessians'
        fail = .true.
      elseif ( lapesdb.neqv.lapesdb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are apes'
        fail = .true.
      elseif ( lagradb.neqv.lagradb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are agradients'
        fail = .true.
      elseif ( lahesdb.neqv.lahesdb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are aHessians'
        fail = .true.
      elseif ( ldipdb.neqv.ldipdb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are dipoles'
        fail = .true.
      elseif ( ladipdb.neqv.ladipdb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are adipoles'
        fail = .true.
      elseif ( lmocodb.neqv.lmocodb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are MOs'
        fail = .true.
      elseif ( lnactdb.neqv.lnactdb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are&
            & diabat(nact) mats'
        fail = .true.
      elseif ( ltransdb.neqv.ltransdb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are&
            & diabat(trans) mats'
        fail = .true.
      elseif ( lctransdb.neqv.lctransdb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are&
            & complex diabat(trans) mats'
        fail = .true.
      elseif ( lnmdb.neqv.lnmdb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are normals'
        fail = .true.
      elseif ( lcasocdb.neqv.lcasocdb2 ) then
        write(6,'(a)') 'Input databases disagree whether there are aSOC'
        fail = .true.
      elseif ( lsymdb.neqv.lsymdb2 ) then
        write(6,'(a)') &
             'Input databases disagree whether there are symmetry replicas'
        fail = .true.
      endif
      if (fail) stop

! Get the no. of records from the geodb files

      call dbgetnrec( dbin(1), nrecin(1) )
      call dbgetnrec( dbin(2), nrecin(2) )

!     See if we want to go ahead
      if (.not. lbatch) then
!-----------------------------------------------------------------------
! Write info of merge to screen and ask before continuing
!-----------------------------------------------------------------------
         Write(6,'(/,2a)')   'Merging DB from new: ',ddname2(1:ddlaenge2)
         Write(6,'(2a)')     '             to old: ',ddname(1:ddlaenge)
         Write(6,'(a,i8)')   ' Nrec in new       : ',nrecin(2)
         Write(6,'(a,i8)')   ' Nrec in old       : ',nrecin(1)
         Write(6,'(a,f8.5)') ' MinDB parameter   : ',mindb
      
         Write(6,'(a)') ' Continue ? <Y/N> '
         read(5,*) goahead
         if (goahead .eq.'n'.or. goahead .eq. 'N') stop
      endif

!-----------------------------------------------------------------------
!get sizes and assign arrays
! Index of 1 on all DB arrays is needed for interface to SQL
!-----------------------------------------------------------------------
      ndof1 = natm*3
      if (lnmdb)then
         if(natm.gt.2)then
            nmdof = ndof1-6
         else
            nmdof = ndof1-5
         endif
      endif

      maxdim = 3*natm+3
      maxsta = max(nddstate,nsocstate)
      if(dercpdim.le.0) dercpdim = nddstate*(nddstate-1)/2
      cerr = 'gpoint'
      allocate(gpoint(maxdim,1),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'v'
      allocate(v(maxsta,maxsta,1),stat=ierr)
      if (ierr > 0) go to 1000
      if (lcpesdb) then
         cerr = 'cv'
         allocate(cv(maxsta,maxsta,1),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if(lgradb)then
         cerr = 'deriv1'
         allocate(deriv1(maxdim,maxsta,maxsta,1),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if(lhesdb)then
         cerr = 'deriv2'
         allocate(deriv2(maxdim,maxdim,maxsta,maxsta,1),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      cerr = 'av'
      allocate(av(maxsta,maxsta,1),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'aderiv1'
      allocate(aderiv1(maxdim,maxsta,maxsta,1),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'aderiv2'
      allocate(aderiv2(maxdim,maxdim,maxsta,maxsta,1),stat=ierr)
      if (ldipdb) then
         cerr = 'dip'
         allocate(dip(3,maxsta,maxsta,1),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'adip'
         allocate(adip(3,maxsta,maxsta,1),stat=ierr)
         if (ierr > 0) go to 1000
      else
         allocate(dip(1,1,1,1),stat=ierr)
      endif
      if(lnmdb)then
         cerr = 'nmpoint'
         allocate(nmpoint(maxdim,1),stat=ierr)
         if(ierr .gt.0) goto 1000
      endif
      if(lmocodb)then
         if(nbasis.lt.0)then
            write(6,'(a)')'Need # Nbasis entry in refdb'
            stop
         else
            allocate(mocoeff(nbasis,nbasis,1))
         endif
      else
         allocate(mocoeff(1,1,1))
      endif
      if(lnactdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(nact(ndof1,dercpdim,1))
         endif
      else
         allocate(nact(1,1,1))
      endif
      if(ltransdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(diabtrans(nddstate,nddstate,1))
         endif
      else
         allocate(diabtrans(1,1,1))
      endif
      if(lctransdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(diabctrans(nddstate,nddstate,1))
         endif
      else
         allocate(diabctrans(1,1,1))
      endif
      if(lcasocdb)then
         if(nddstate.lt.0)then
            write(6,'(a)')'Need # Nstate entry in refdb'
            stop
         else
            allocate(casoc(nddstate,nddstate,1))
         endif
      else
         allocate(casoc(1,1,1))
      endif
      if (lsymdb) then
         cerr = 'symrep'
         allocate(symrep(maxsymop,1),stat=ierr)
         if (ierr > 0) go to 1000
      endif

!-----------------------------------------------------------------------
! store geoms in DB1 in memory
!-----------------------------------------------------------------------
      ndofddpes = ndof1   ! needed for distdb 
      ldbsave = .true.
      cerr = 'dbgeo'
      allocate(dbgeo(maxdim,nrecin(1)),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'r1'
      allocate(r1(nrecin(1)),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dbignore'
      allocate(dbignore(nrecin(1)),stat=ierr)
      if (ierr > 0) go to 1000

      do irec = 1, nrecin(1)
         dbignore(irec) = 0
      enddo
      call dbrdgeo( dbin(1), natm, 1, nrecin(1), dbgeo )

! Start reading DB2, writing to DB1
! Each geometry is checked against the other database
! to see if it's already there, if so it's skipped.

      do irec = 1, nrecin(2)
         call dbrdgeo( dbin(2), natm, irec, -1, gpoint )

         call distdb(gpoint,idbloc,r1,mindist,mindisp,nrecin(1))

         if ( mindisp.lt.mindb ) then
            write(6,'(a, i0, a)')'Skipped record ', irec,&
                                    ' from database 2'
            cycle
         endif

! read new record
         call dbrdentry(dbin(2),irec,-1, interp, timestamp)
         if (lpesdb) call dbrdpes(dbin(2), nddstate, irec, -1, v )
         if (lgradb) call dbrdgra(dbin(2), nddstate,natm, irec, -1, deriv1 )
         if (lhesdb) call dbrdhes(dbin(2),nddstate,natm, irec, -1, deriv2 )
         if (lapesdb) call dbrdapes(dbin(2), nddstate, irec, -1, av )
         if (lagradb) call dbrdagra(dbin(2), nddstate,natm, irec, -1, aderiv1 )
         if (lahesdb) call dbrdahes(dbin(2), nddstate, natm, irec, -1, aderiv2 )
         if (ldipdb) call dbrddip(dbin(2), irec, -1, dip )
         if (ladipdb) call dbrdadip(dbin(2), irec, -1, adip )
         if (lmocodb) call dbrdmoco(dbin(2), nbasis, irec,-1,  mocoeff )
         if (lnactdb) call dbrdnact(dbin(2), natm, dercpdim, irec, -1, nact )
         if (ltransdb) call dbrdtrans(dbin(2),nddstate, irec, -1, diabtrans )
         if (lctransdb) call dbrdctrans(dbin(2),nddstate, irec, -1, diabctrans )
         if (lnmdb) call dbrdnm(dbin(2), nmdof, irec, -1, nmpoint )
         if (lcasocdb) call dbrdcasoc( dbin(2), nddstate, irec, -1, casoc )
         if (lsymdb) call dbrdsymrep( dbin(2), nsymop, irec, -1, symrep )
         if (lcpesdb) call dbrdpes(dbin(2), nddstate, irec, -1, v )

         ! Write the database record
         write(6,'(a, i0)')'Writing record ', irec
         ! START TRANSACTION
         call dbstarttrans ( dbin(1) )
         ! write entry and get new rowid
         rowid = 0
         call dbwrentry( dbin(1), rowid, interp(1), 1000 )
         call dbwrgeo( dbin(1), rowid, natm, gpoint(:,1) )
         if (lpesdb) call dbwrpes( dbin(1), rowid, nddstate, v(:,:,1) )
         if (lgradb) call dbwrgra( dbin(1), rowid, natm, nddstate, deriv1(:,:,:,1) )
         if (lhesdb) call dbwrhes( dbin(1), rowid, natm, nddstate, deriv2(:,:,:,:,1) )
         if (lapesdb) call dbwrapes( dbin(1), rowid, nddstate, av(:,:,1) )
         if (lagradb) call dbwragra( dbin(1), rowid, natm, nddstate, aderiv1(:,:,:,1) )
         if (lahesdb) call dbwrahes( dbin(1), rowid, natm, nddstate, aderiv2(:,:,:,:,1) )
         if (ldipdb) call dbwrdip( dbin(1), rowid, nddstate, dip(:,:,:,1) )
         if (ladipdb) call dbwradip( dbin(1), rowid, nddstate, adip(:,:,:,1) )
         if (lmocodb) call dbwrmoco( dbin(1), rowid, nbasis, mocoeff(:,:,1) )
         if (lnactdb) call dbwrnact( dbin(1), rowid, natm, nddstate, nact(:,:,1) )
         if (ltransdb) &
            call dbwrtrans( dbin(1), rowid, nddstate, diabtrans(:,:,1) )
         if (lctransdb) &
            call dbwrctrans( dbin(1), rowid, nddstate, diabctrans(:,:,1) )
         if (lnmdb) call dbwrnm( dbin(1), rowid, nmdof, nmpoint(:,1) )
         if (lcasocdb) call dbwrcasoc( dbin(1), rowid, nddstate, casoc(:,:,1) )
         if (lsymdb) call dbwrsymrep( dbin(1), rowid, nsymop, symrep(:,1) )
         if (lcpesdb) call dbwrcpes( dbin(1), rowid, nddstate, cv(:,:,1) )

         ! COMMIT TRANSACTION
         call dbcommittrans ( dbin(1) )

      enddo

! Deallocation
      if (allocated(gpoint))deallocate(gpoint)
      if (allocated(v))deallocate(v)
      if (allocated(cv))deallocate(cv)
      if (allocated(deriv1))deallocate(deriv1)
      if (allocated(deriv2))deallocate(deriv2)
      if (allocated(av))deallocate(av)
      if (allocated(aderiv1))deallocate(aderiv1)
      if (allocated(aderiv2))deallocate(aderiv2)
      if (allocated(dip)) deallocate(dip)
      if (allocated(adip)) deallocate(adip)
      if (allocated(mocoeff)) deallocate(mocoeff)
      if (allocated(diabtrans)) deallocate(diabtrans)
      if (allocated(diabctrans)) deallocate(diabctrans)
      if (allocated(nact)) deallocate(nact)
      if (allocated(nmpoint)) deallocate(nmpoint)
      if (allocated(casoc)) deallocate(casoc)
      if (allocated(symrep)) deallocate(symrep)
      if (allocated(dbgeo)) deallocate(dbgeo)
      if (allocated(r1)) deallocate(r1)
      if (allocated(dbignore)) deallocate(dbignore)

      print*,'DONE'
      stop
! Error messages

 1000 write(6,'(a)') 'Error allocating memory in MERGEDB :',cerr
      write(6,'(a)') 'Error Code :',ierr
      stop

      end program mergedb
