!***********************************************************************
!
!                           ANALYSIS PROGRAM: ml2mctdh
!
! Converts a psi file from a multilayer to an MCTDH representation
!
! Code built using psi2ex as example
! OV 03/10
!***********************************************************************

program ml2mctdh

use decimal
use global
use aglobal
use versions
use maxv
use dvrdatmod
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod
use channels
use logdat
use rdfilesmod
use iorst
use rdfilesmod
use iodvrdef
use iofile
use auswutil
use iogrddef
use iopsidef
use inputmlmod
use compdat, only: savecsys,savecpsi,getcsys,getcpsi
      use mlanalysis, only: ml_to_mctdh
      use wftree
      use wftree1

      implicit none

      integer  :: chkdvr,chkgrd,chkpsi,chkdat,chkprp
      logical  :: lsp,lmst,lrst,linwf,lerr

      type(wfnode_t),pointer :: mltree
      integer                :: i,jindx(1),d,iz,s,ierr
      integer                :: mldgldim1,dgldim1
      integer                :: log2int
      real(dop)              :: time,dt, &
                                previous,now,spent,fcputime,fsystime
      complex(sip), allocatable :: spsi(:)
      complex(dop),allocatable :: psi(:)
      complex(dop),allocatable :: psiout(:)
      complex(dop),allocatable :: agmat(:)
      complex(dop),allocatable :: adgwp(:)
      integer(long),allocatable :: gwpdep(:)
      integer(long),allocatable :: trajst(:)
      character(len=80)      :: actual
      character(len=100)     :: buffer
      character(len=c2)      :: wrtag
      external               :: fcputime,fsystime
      data                      previous /0.0d0/
      save                      previous

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      lsp=.true.
      lmst=.false.
      lrst=.false.
      call ml2mcinput(lsp,lmst,lrst)

  !    print*,'WARNING: Program memory allocation needs checking'
  !    stop
!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (ilog,file='ml2mctdh.log',form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(ipsi)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef
      call alloc_compdat

!-----------------------------------------------------------------------
! set no. of comparison data sets
!-----------------------------------------------------------------------
      ncomp=2

!-----------------------------------------------------------------------
! read data from psi file
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      if (lrst) then
         irst = ipsi
         chkprp = 0
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         read (irst,iostat=ierr) tinit
         if (ierr /= 0) then
            routine='ml2mctdh'
            message='error reading time from restart-file'
            call errormsg
         endif
      else
         read(ipsi) filever(ipsi)
         call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      endif

      if (psitype .ne. 5) then
         routine='ml2mctdh'
         select case (psitype)
            case (1)
               message = 'Wavefunction in exact representation'
            case (0)
               message = 'Wavefunction in MCTDH representation'
            case default
               message = 'Wavefunction not in ML-MCTDH representation'
         end select
         call errormsg
      endif

!-----------------------------------------------------------------------
! set up ML
!-----------------------------------------------------------------------
      if (lrunml) then
         call inputml(tape)
      endif

!-----------------------------------------------------------------------
! allocate data structures for ML
!-----------------------------------------------------------------------
      mltree    => new_wftree(tape)
      call wftree_set_pointers(mltree,dgldim)

!-----------------------------------------------------------------------
! Initialize the tree structure of the ML wavafunction
!   - tape was read by rdpsiinfo/rstinfo
!-----------------------------------------------------------------------
      mldgldim1 = dgldim_wftree(mltree)
      if (.not. lmst) then
         dgldim1   = dgldim_wftree_mctdh(mltree,2)
      else
         dgldim1   = dgldim_wftree_mctdh(mltree,3)
      end if

      write(ilog,'(a,i12,a)') &
           'Length of ML WF    : ',mldgldim1,' coeffs'
      write(ilog,'(a,i12,a)') &
           'Length of MCTDH WF : ',dgldim1,' coeffs'
      write(ilog,*) ' '

!-----------------------------------------------------------------------
! write ml and mctdh info in comparison sets 1 and 2, respectively
!-----------------------------------------------------------------------
      dgldim = mldgldim1
      call savecsys(1)
      call savecpsi(1)
      call savecdat(1)

      if (lmst) then  ! multi-set in psi
        nstate = dim(nmode,1)
        nmode = nmode - 1
        lmult = .true.
        leb = .false.
        do s=2,nstate
          dim(:,s) = dim(:,1)
        end do
        dofspf(ndof) = dofspf(ndof) + 1
      end if
      dgldim = dgldim1
      psitype=0
      lrunml=.false.
      lpsiopt(1)=lsp
      lpsiopt(2)=.false.
      lpsiopt(3)=.false.
      lpsiopt(4)=.false.
      call savecsys(2)
      call savecpsi(2)
      call savecdat(2)

      allocate(psi(mldgldim1))
      allocate(spsi(mldgldim1))
      allocate(psiout(dgldim1))
      allocate(agmat(agmatdim))
      allocate(adgwp(gsmatdim))
      allocate(trajst(gwpinfodim))
      allocate(gwpdep(gwpinfodim))

!-----------------------------------------------------------------------
! write mctdh psi information to new file
!-----------------------------------------------------------------------
      lpsi=.true.
      lpsigrd=.false.

      call getcsys(2)
      call getcpsi(2)
      call getcdat(2)
      dgldim = dgldim1
      write(ipsi1) progver
      if (lrst) then
         ! see wrrst
         call wrdvrdef(ipsi1)
         call wrgrddef(ipsi1)
         call wrpsidef(ipsi1)
         write(ipsi1) runtitle
         call fpstartup(buffer)
         write(ipsi1) buffer
         write(ipsi1) log2int(.true.)
         write(ipsi1) tinit/fs
      else
         call wrpsiinfo(ipsi1)
      endif

!-----------------------------------------------------------------------
! calculate time step for data
!-----------------------------------------------------------------------
      ! switch to set 1 (ML), needed for rdpsi
      psitype=5
      lrunml=.true.
      call getcsys(1)
      call getcpsi(1)
      call getcdat(1)
      dgldim = mldgldim1
      iz = 0
      dt   = out2/fs*step
      time = tinit/fs

! --- skip the first nskip WFs
      do i=1,nskip
         call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
         if (lend) exit
         time=time + out2/fs
      enddo

! --- read first psi to be precessed from psi file
      if (lrst) then
         read (ipsi,iostat=ierr) (psi(i), i=1,dgldim)
         if (ierr /= 0) then
            routine="ml2mctdh"
            message="error reading restart file"
            call errormsg
         endif
      else
         call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
      endif

! --- MAIN LOOP, transform one psi at a time
      countpsi = 0
MAIN: do
         if (lend) exit MAIN

! --- transform ml to mctdh
        if(.not. lmst) then
            call ml_to_mctdh(mltree,psi,psiout,2)
        else
            call ml_to_mctdh(mltree,psi,psiout,3)
        end if

! --- determine actual time and CPU-time spent since last call
        call fzeit(actual)
        i = index(actual,':')
        actual=actual(i-9:i+6)
        now=fcputime(0)+fsystime(0)
        spent=now-previous
        previous=now

        write(ilog,'(a,f8.3,a,f15.1,f15.2,4x,a16)') &
             'Writing WF : ',time,' fs, ',now,spent,actual
        call myflush(ilog)

        countpsi=countpsi+1

        write(wrtag,'(a9,i10,a1)') '<data set',countpsi,'>'
        write(ipsi1) wrtag

        if (lgwp) then
           if (gwptype .ge. 3) then
              wrtag='<agmat>'
              write(ipsi) wrtag
              write(ipsi) (agmat(d),d=1,agmatdim)
           endif
        endif

        wrtag='<psi>'
        write(ipsi1) wrtag
        if (lsp) then
           write(ipsi1) (cmplx(psiout(d)), d = 1,dgldim1)
        else
           write(ipsi1) (psiout(d), d = 1,dgldim1)
        endif

        write(wrtag,'(a13,i10,a1)') '<end data set',countpsi,'>'
        write(ipsi1) wrtag
        call myflush(ipsi1)

        time=time + dt
        iz=iz+1
        if (lrst) exit MAIN
        if(iz.ge.ncut) exit

! --- loop over non desired steps, default step=1
        do i=2,step
           call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
           if (lend) exit MAIN
        enddo

! --- read next psi to be transformed
        call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)

      end do MAIN

      close(ipsi)
      close(ipsi1)

      deallocate(psi)
      deallocate(spsi)
      deallocate(psiout)
      deallocate(agmat)
      deallocate(adgwp)
      deallocate(trajst)
      deallocate(gwpdep)

end program ml2mctdh


!***********************************************************************
!
! ML2MCTDH input
!
! get input parameters and file names for calculation
!
!-----------------------------------------------------------------------
      subroutine ml2mcinput(lsp,lmst,lrst)

use versions
use global
use aglobal
use maxv
use griddatmod
use psidef
use datenmod
use compdat
use compdat1mod
use channels
use logdat

      implicit none

      logical           :: lsp,lmst,lrst

      integer           :: iarg, myiargc, ierr,i
      character*80      :: buf
      character*(c5)    :: filename
      logical           :: lfileout
!-----------------------------------------------------------------------

      nskip=0
      ncut = 999999
      filein  = 'psi'
      lfilein = .false.
      fileout = 'psi.mctdh'
      lfileout=.false.
      iarg = myiargc()

      i = 0
 args:do
      if (iarg .lt. 1) exit
      if (i .ge. iarg) exit
      i=i+1
      call getarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:3) .eq. '-f ' .or.  buf(1:3) .eq. '-i ') then
            i = i+1
            call getarg(i,buf)
            filein = buf
            lfilein=.true.
         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call getarg(i,buf)
            fileout = buf
            lfileout=.true.
         else if( buf(1:3) .eq. '-r ' ) then
            lrst = .true.
            lsp = .false. ! restart files are always double-precision
            if (.not.lfilein)  filein ='restart'
            if (.not.lfileout) fileout='restart.mctdh'
         else if( buf(1:5) .eq. '-stp ' ) then
            i = i+1
            call getarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -n', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:3) .eq. '-n ') then
            i = i+1
            call getarg(i,buf)
            read(buf,*,iostat=ierr) ncut
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -n'
               stop
            endif
         else if( buf(1:5) .eq. '-skip ') then
            i = i+1
            call getarg(i,buf)
            read(buf,*,iostat=ierr) nskip
            if (nskip .lt. 0) then
               write(6,'(/a)') &
               'Argument of -skip must be positive integer'
               stop
            endif
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -skip'
               stop
            endif
         else if( buf(1:3) .eq. '-w ' ) then
            lovwr=.true.
         else if( buf(1:4) .eq. '-dp ' ) then
            lsp=.false.
         else if( buf(1:8) .eq. '-mstate ' ) then
            lmst=.true.
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,a,/10x,a)') &
         'Purpose: Converts a multilayer wavefunction in a psi/restart file '&
        ,'to an equivalent MCTDH form. Default is single precision.'
            write(6,'(/1x,2a)') 'Usage  : ml2mctdh ', &
           '[-f -i -o -r -stp -n -skip -w -dp -ver -h -?]'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a)') &
                     ' -f FILE : The wavefunction is read ' &
                    ,'from file FILE rather than from ./psi' &
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(a)') ' -i FILE : Same as -f FILE.'
            write(6,'(2a,/,11x,a)') &
                     ' -o FILE : The output is written to file ', &
                     'FILE rather than to ./psi.mctdh', &
             'The string FILE may be a relative or a full path-name.'
            write(6,'(a,/,a)') &
                     ' -r      : Read and write restart files instead of psi files.', &
                     '           This automatically sets -dp.'
            write(6,'(2a)') ' -stp step: Compute the exact WF', &
                            ' only every step-th output.'
            write(6,'(2a)') &
                     ' -n num  : only the first num WFs are read ' &
                    ,' from psi-file.'
            write(6,'(a)') &
                     ' -skip m : the first m WFs are skipped.'
            write(6,'(2a)')  &
                     ' -dp     : The file is written double precision.'
            write(6,'(2a)')  &
                     ' -mstate : A multi-state psi file is written. '
            write(6,'(2a)')  &
                     '           Only makes sense if the top layer has *only* two logical modes '
            write(6,'(2a)')  &
                     '           corresponding to the system and electronic degree of freedom.'
            write(6,'(2a)')  &
                     ' -w      : An existing output file is ', &
                     'overwritten.'
            write(6,'(a)')  &
                     ' -ver    : Version information about the program '
            write(6,'(a,/a)')  &
                     ' -h      : Print this help text. ', &
                     ' -?      : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
      else
         write(6,'(2a)') ' unknown argument : ', buf
         stop
      end if
      cycle
      end do args

      if (lrst) then
         routine = "ml2mcinput"
         if (nskip /= 0) then
            message = "There is nothing to skip in a restart file!" &
                   //" Resetting nskip to zero."
            call warnmsg
            nskip = 0
         endif
         if (step /= 1) then
            message = "There is only one WF in a restart file!" &
                   //" Resetting step to one."
            call warnmsg
            step = 1
         endif
      endif

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      laein=index(filein,' ')-1
      if (filein(laein:laein) .eq. '/') then
         filein = filein(1:laein)//'psi'
         laein = laein + 3
      end if
      call abspath(filein,laein)

      laeout=index(fileout,'  ')-1
      call abspath(fileout,laeout)

      write(6,'(80a)') ('*',i=1,80)
      filename=filein(1:laein)
      open(ipsi,file=filename,form='unformatted',status='old',&
           err=1000)
      write(6,'(2a)') ' The inputfile is  : ',filein(1:laein)

      filename=fileout(1:laeout)
      if (lovwr) then
         open (ipsi1,file=filename,form='unformatted',status='unknown',&
                err=1000)
      else
         open (ipsi1,file=filename,form='unformatted',status='new',&
                err=1010)
      endif
      write(6,'(2a)') ' The outputfile is : ',fileout(1:laeout)
      write(6,'(2a)') ' The log file is   :  ./ml2mctdh.log'

      write(runtitle,'(3a)') 'ml2mctdh("',trim(filein),'")'
      
      return

 1000 write(6,*)' Error opening file : '
      write(6,*) filename
      stop
 1010 write(6,*)' File already exists: '
      write(6,*) filename
      stop

      end subroutine ml2mcinput

