! Terry's f90 module dependence analyser.

! Reads a list of sources from stdin, and pulls out the module dependence
! for mctdh.  Currently analyses a single tree.  The 'graph' is file-based.

! To turn this into a proper dependency list edges from ancestors
! of ancestors should be removed, then dump the graph

! A file list can be genereted by, e.g.
!    ( find source -iname '*.f' ; find source -iname '*.f90' )

module tracker
implicit none
save
integer,parameter::MaxF=1000,MaxM=2000
integer,parameter::MaxDef=20,MaxUse=200
integer::Nf,Nm
character(len=80),dimension(MaxF)::file
character(len=80),dimension(MaxM)::module
 ! element 0 is the count:
integer,dimension(0:MaxDef,MaxF)::defines
integer,dimension(MaxM)::deffile
integer,dimension(MaxF)::order
!integer,dimension(0:MaxUse,MaxM)::uses
 ! if graph(i,j) is 1, then file j uses a module from file i
integer,dimension(MaxF,MaxF)::graph
logical,dimension(MaxF)::mctdh

contains

subroutine ReadFiles
integer::i,j,badness,curr,m
character(len=80)::inline,s1,s2
Nm=0
defines=0
deffile=0
!uses=0
graph=0
do i=1,MaxF
  read(*,'(a)',iostat=badness)file(i)
  if (badness/=0) exit
enddo
Nf=i-1
write(*,*)Nf,' files'
 ! first pass is for module definitions.  Second is for dependencies
write(*,*)'Collating module names'
do j=1,2
  do i=1,Nf
    open(unit=10,file=file(i))
    curr=0
    do
      read(10,'(a80)',iostat=badness)inline
      if (badness/=0) exit
       ! gobble empty lines
      if (inline=='') cycle
       ! ignore lines without whitespace
      if (index(trim(adjustl(inline)),' ')==0) cycle
       ! ignore lines with quotes
      if (index(trim(adjustl(inline)),'''')/=0) cycle
      if (index(trim(adjustl(inline)),'"')/=0) cycle
      read(inline,*)s1,s2
      s1=lower(trim(s1))
      s2=lower(trim(s2))
      if ((s1=='module').or.(s1=='program')) then
        curr=curr+1
        if (j==1) then
          Nm=Nm+1
          module(Nm)=s2
          defines(curr,i)=Nm
          defines(0,i)=curr
          deffile(Nm)=i
        endif
      elseif ((j==2).and.(s1=='use')) then
        m=index(s2,',')
        if (m>0) s2=s2(1:m-1)
        m=FindMod(trim(s2))
        m=FindFile(m)
        if (m/=i) graph(m,i)=1
      endif
    enddo
    close(10)
  enddo
  if (j==1) then
    write(*,*)Nm,' modules'
    write(*,*)'Compiling dependencies'
  endif
enddo
write(*,*)count(graph(1:Nf,1:Nf)==1),' uses'
end subroutine ReadFiles

integer function FindMod(str)
character(len=*),intent(in)::str
integer::i
do i=1,Nm
  if (module(i)==str) exit
enddo
FindMod=i
end function FindMod

integer function FindFile(m)
integer,intent(in)::m
integer::i
do i=1,Nf
  if (any(defines(1:,i)==m)) exit
enddo
FindFile=i
end function FindFile

function lower(str) result (R)
character(len=*),intent(in)::str
character(len=len(str))::R
integer::i,j
do i=1,len(str)
  j=iachar(str(i:i))
  if ((j>=65).and.(j<=90)) then
    R(i:i)=achar(j+32)
  else
    R(i:i)=str(i:i)
  endif
enddo
end function lower

subroutine SpanTree(a)
 ! starting from node a, follow all links and touch mctdh
integer,intent(in)::a
integer::i,j
logical,dimension(MaxF)::hit
order=0
mctdh=.false.
hit=.false.
mctdh(a)=.true.
order(a)=1
j=1
do while (any(mctdh.and..not.hit))
  i=first(mctdh.and..not.hit)
  j=j+1
  order(i)=j
  mctdh=mctdh.or.(graph(:,i)==1)
  hit(i)=.true.
enddo
write(*,*)count(mctdh),' in mctdh tree'
end subroutine SpanTree

logical function IsAncestor(me,him)
 ! true if him is an ancestor of me
integer,intent(in)::me,him
integer,dimension(MaxF**2)::Q
integer::N,i,j
logical,dimension(MaxF)::hit
logical::chill
N=1
Q(1)=me
hit=.false.
chill=.true.
do while (N>0)
  i=Q(N)
  if (.not.chill.and.(i==him)) then
    IsAncestor=.true.
    return
  endif
  N=N-1
  hit(i)=.true.
  do j=1,MaxF
    if (graph(j,i)==1) then
      if (all(Q(1:N)/=j)) then
        N=N+1
        Q(N)=j
      endif
    endif
  enddo
  chill=.false.
enddo
IsAncestor=.false.
end function IsAncestor

integer function first(v)
logical,dimension(:),intent(in)::v
integer::i
do i=1,size(v)
  if (v(i)) exit
enddo
first=i
end function first

end module tracker

program moduledep
use tracker
implicit none
integer::root,i,j
call ReadFiles
do root=1,Nm
  if (module(root)=='mctdh') exit
enddo
root=deffile(root)
call SpanTree(root)
write(*,*)'Checking for cycles 1'
if (sum(graph*transpose(graph))>0) then
  do i=1,Nf
    do j=1,Nf
      if (graph(i,j)*graph(j,i)>0) write(*,*)trim(file(i)),' ',trim(file(j))
    enddo
  enddo
  stop 'bummer'
endif
write(*,*)'Checking for cycles 2'
do i=1,Nf
  if (mctdh(i)) then
    if (IsAncestor(i,i)) then
      write(*,*)i
      write(*,*)trim(file(i))
    endif
  endif
enddo
end
