!***********************************************************************
!
!                         AUSWERT subroutine NORM
!
!     This subroutine calculates the norm and the A-norm of a wave function.
!
!     05/99 HDM
!
!
!***********************************************************************
      
      module normmod

      use decimal, only : long,dop

!-----------------------------------------------------------------------
! options for overlap calculation
!-----------------------------------------------------------------------
      integer(long), save    :: ebtype
      logical(kind=4), save :: lallov

      end module normmod

!#######################################################################

      program norm

use decimal, only: long,dop,sip
use global
use globmemmod
use versions
use logdat
use aglobal
use dglobal
use dvrdatmod
use griddatmod
use operdef
use psidef
use channels
use datenmod
use normmod
use plot

use mmlib, only: mmaxzzh, mmaxzz
use op1lib, only: cpvxz
use op2lib, only: swapsxxi
use mtlib, only: qtxxzz, mtxxzz, mttxzz
use xvlib, only: vvaxzz
use iorst, only: rstinfo
use iofile, only: rdpsiinfo
use zeigauswmod, only: zeigausw
use wftree1, only: dispose_wftree

use module_interface

!     ML-MCTDH modules
      use wftree
      use mlanalysis

      implicit none

      integer(long)                     :: zeig,zeig1,zeig2,m,s,chkdvr,chkgrd,&
                                           chkpsi,chkdat,chkprp
      integer(long)                     :: mcovr
      character(len=c5)                 :: filename
      logical(kind=4)                   :: lerr,linwf

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      call norminput

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      if (ebtype .lt. 0) then
         call rdmemdim(irst)
      else
         call rdmemdim(ipsi)
      endif

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      call alloc_psidef
      if (ldynspf) call alloc_dynwf
      
!-----------------------------------------------------------------------
! set nplottask and ncalcpar before allocating memory for plotting routines
!-----------------------------------------------------------------------
      nplottask=5
      call alloc_plot

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='norm.log',form='formatted')
      logisopen=.true.

      if( ebtype .lt. 0 ) then
         rewind(irst)
!-----------------------------------------------------------------------
! read data from restart file  (ebtype = -1  -> restart rather than psi)
!-----------------------------------------------------------------------
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkprp=1
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         if (linwf) write(6,'(a)') ' Initial wave function' 
         if (lerr) then 
            write(6,'(a)') ' ERROR reading restart file' 
            stop
         end if
      else
!-----------------------------------------------------------------------
! read data from psi file
!-----------------------------------------------------------------------
         rewind(ipsi)
         read(ipsi) filever(ipsi)
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkdat=1
         call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
         filename='psi'
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif

!     ML-MCTDH wavefunction
      if (psitype.eq.5) then
! ... ML wavefunction? Get dgldim and write header.
         !tree => new_wftree(tape,if_spfdof)
         !dgldim = dgldim_wftree(tree)
         if(lprint) then
            write (6,*)
            call print_wftree(6,tree,modelabel,.true.)
         endif
         write (6,*)
         call print_wftree(ilog,tree,modelabel,.false.)
         call dispose_wftree(tree)
      endif
!     Dynamical WF
      if (ldynspf) dglmax=dgldim

      if (nmode .eq. 1) then
         do s=1,nstate
            dim(1,s)=1
         enddo
      endif

!-----------------------------------------------------------------------
! write head-line.
!-----------------------------------------------------------------------
!     Not head-line when ML-MCTDH
      if (psitype .ne. 5) then
         if (.not. linter) then
            if (fs.gt.2.d0) then
               write(6,'(/,5a)') &
              '    Time[fs]','     ||psi||^2','       ||A||^2', &
                         '         ||psi||  ','       ||A||'
            else
               write(6,'(/,5a)') &
              '    Time[au]','     ||psi||^2','       ||A||^2', &
                         '         ||psi||  ','       ||A||'
            endif
         endif
      endif

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
!-----------------------------------------------------------------------
      lpsi=.true.
      lpsigrd=.false.
      call zeigausw

      zeig=1
      do s=1,nstate
         zeig1=block(s)
         do m=1,nmode
            zeig=zeig+dim(m,s)*dim(m,s)
            zeig2=zeig1
            zeig2=zeig2+zeig1
            workcdim=max(workcdim,zeig2)
         enddo
      enddo
      

!-----------------------------------------------------------------------
! assign memory arrays
!-----------------------------------------------------------------------
      call memausw
      mcovr=mcdim
      mcdim=mcovr+dmatdim

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      if (dentype.eq.2) then
         workcdim = max(workcdim,2*(maxblock**4))
      else
         workcdim = max(workcdim,2*maxblock)
      endif

      mcworkc = mcdim
      mcdim   = mcworkc+workcdim

!-----------------------------------------------------------------------
! call overlap routine
!-----------------------------------------------------------------------
if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)
if(allocated(agmat))deallocate(agmat)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))
allocate(agmat(agmatdim))

      call norm1(mc(mcpsi),ms(mspsi),mc(mcovr),mc(mcworkc),&
           mi(mijindx),agmat,mc(mcadgwp))

deallocate(mc,mr,mi,ml,ms,mf,agmat)

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_psidef
      call dealloc_dynwf
      call dealloc_plot

      stop
  999 continue
      message= 'Error reading :'//filename(1:c5-15)
      write(6,'(a)') message
      stop
      contains

!***********************************************************************
!     calculates the overlap
!***********************************************************************

      subroutine norm1 (psi,spsi,ovr,workc,jindx,agmat,adgwp)

use global
use aglobal
use griddatmod
use psidef
use channels
use datenmod
use normmod
use plot

use rdfilesmod, only: rdpsi,rdpsirst
use plotutils, only: plotdefs,initplot,timebnds,dotask,plotmenu,databnds

      implicit none

      integer(long), external                         :: slen
      integer(long)                                   :: i,k
      integer(long), dimension(adim), intent(out)     :: jindx
      integer(long), dimension(gwpinfodim)            :: gwpdep
      complex(sip), dimension(spsidim), intent(out)   :: spsi
      complex(dop), dimension(dgldim), intent(out)    :: psi
      complex(dop)                                    :: ovl,aovl
      complex(dop), dimension(dmatdim), intent(out)   :: ovr
      complex(dop), dimension(workcdim), intent(out)  :: workc
      complex(dop), dimension(agmatdim), intent(out)  :: agmat
      complex(dop), dimension(gsmatdim), intent(out)  :: adgwp
      real(dop)                                       :: time,dt,norm,normsq,anormsq,anorm
! interactive plot variables
      integer(long)                         :: ndata,plottask,task,plotdim,&
                                               dimx,dimy,nsets,nplots
      real(dop), dimension(1)               :: ydum,xdum,edum,rdum
      real(dop), dimension(:), allocatable, save     :: pltime
      real(dop), dimension(:,:), allocatable, save   :: pldata
      integer(long), dimension(:), allocatable, save :: idum
      character(len=c5)                              :: filename
      integer(long)                                  :: n,ilbl,jlbl
      integer(long)     ::  chkdvr,chkgrd,chkpsi,chkdat,chkprp

      character(len=c2)                              :: rdtag

!     ML-MCTDH pointers and variables
      integer                     :: j,f,l
      type(wfnode_t),pointer      :: no
      logical linwf,lerr

!-----------------------------------------------------------------------
! calculate time step for data
!-----------------------------------------------------------------------
      dt = out2/fs
      time = tinit/fs
      k    = 0
      ndata = 0

      if (ebtype .lt. 0) then
! reading restart file - only 1 datum
         ndata = 1
      else
! get no. of data sets in psi file
         rewind(ipsi)
         do
   10       continue
            read(ipsi,err=10,end=20) rdtag
            if (rdtag(1:1) .ne. '<') cycle

            ilbl=2
            jlbl=index(rdtag,'>')-1
            if (rdtag(ilbl:ilbl+7) .eq. 'data set') then
               read(rdtag(ilbl+8:jlbl),'(i10)') ndata
            endif
         enddo
   20    continue

         rewind(ipsi)
      endif

      maxdata = ndata
      allocate(pltime(maxdata))
      allocate(pldata(maxdata,5))
      allocate(idum(maxdata))

      idum(1) = 0

      j=1 ! j is a counter of the ML-nodes that have been printed

!     ML-MCTDH wavefunction
      if (psitype.eq.5) then
!        allocate data structures for ML
!         tree => new_wftree(tape,if_spfdof) ! tree is set up in rdpsiinfo
!        Process first node
         no => tree%swap_down(j)%p
!        Print node information (primitive and non-primitive) and header for ML
         call norm0_tree(iout,tree,modelabel,psi,j,time,anormsq,ovl,fs,0,&
              agmat)
      endif

  101 continue ! loop for ML-WF nodes
      do n=1, ndata
!-----------------------------------------------------------------------
! ebtype = -1   --> read restart rather than psi file.
!-----------------------------------------------------------------------
         if (ebtype .lt. 0) then
!-----------------------------------------------------------------------
! read restart wavefunction
!-----------------------------------------------------------------------
            read (irst,err=998) time
            time = time/fs
            call rdpsirst(irst,psi,agmat)

         else 
            if (ldynspf) dgldim=dglmax ! Dynamical WF
            call rdpsi(ipsi,psi,spsi,jindx,agmat,idum,adgwp,gwpdep)

         endif

         if( mod(n,step) .eq. 0 ) then
!        Calculate ML-MCTDH norm
            if (psitype.eq.5) then
               call norm0_tree(iout,tree,modelabel,psi,j,time,anormsq,ovl,fs,2,&
                    agmat)
            else
               call norm0(ovl,aovl,psi,ovr,workc,agmat)
               anormsq = dble(aovl)
            endif

            call norm0(ovl,aovl,psi,ovr,workc,agmat)

            if (j .eq. 1) then
               normsq  = dble(ovl)
               norm    = sqrt(normsq)
               anormsq = dble(aovl)
               anorm   = sqrt(anormsq)

               pltime(n)=time
               pldata(n,1)=normsq
               pldata(n,2)=anormsq
               pldata(n,3)=norm
               pldata(n,4)=anorm
               pldata(n,5)=normsq-anormsq
            endif
         endif

         time=time + dt

!-----------------------------------------------------------------------
! if more data, go to beginning of loop
!-----------------------------------------------------------------------
      enddo

!     ML-MCTDH wavefunction
!     Process remaining nodes until j < tree%tot_nodes
      if ((psitype.eq.5) .AND. (j.lt.tree%tot_nodes) .AND. lallov) then
!        j is a counter of the nodes that have been printed
         j=j+1
!        Process jth node
!        Print node information (primitive and non-primitive) and header for ML
         call norm0_tree(iout,tree,modelabel,psi,j,time,anormsq,ovl,fs,0,&
              agmat)
!        Reboot all relevant variables and go back to label "101"
         time = tinit/fs
         lend=.false.
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkdat=1
         chkprp=1
         if (ebtype.lt.0) then
           rewind(irst)
           call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         else
            rewind(ipsi)
            read(ipsi) filever(ipsi)
            call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
         endif
         goto 101
      endif

      close(ipsi)

      if (.not. linter) then
         if (.not.lallov .and. k.gt.3*step) then
            if (fs.gt.2.d0) then
                write(6,'(5a)')&
                '    Time[fs]','     ||psi||^2','      ||A||^2',&
                             '       ||psi||  ','      ||A||'
            else
                write(6,'(5a)')&
                '    Time[au]','     ||psi||^2','      ||A||^2',&
                             '       ||psi||  ','      ||A||'
            endif
         endif
         write(6,'(76a)') ('-',i=1,76)
         close(iaus)
         return
      endif


      if (.not. linter) then
         do n=1,ndata
            write(6,'(f12.4,1x,2f15.11,2x,2f13.9)') &
                pltime(n),(pldata(n,i),i=1,4)
         enddo
         write(6,'(76a)') '#',('-',i=1,75)
         close(iaus)
         return
      endif
      
!-----------------------------------------------------------------------
! interactive plotting
!-----------------------------------------------------------------------
      if (ndata .gt. maxdata) then
         routine='Norm'
         write(message,'(a,i7)')&
           'Increase parameter MAXDATA in plot.inc. Minimum : ',ndata
         call errormsg
      endif

!-----------------------------------------------------------------------
! interactive plotting
! open file for data to be plotted
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/tmpplot.pl'
      open(iaus,file=filename,status='unknown')

!-----------------------------------------------------------------------
! zero dummy variables
!-----------------------------------------------------------------------
      xdum(1)=0.0
      ydum(1)=0.0
      rdum(1)=0.0
      edum(1)=0.0

!-----------------------------------------------------------------------
! interactive plot defaults
!-----------------------------------------------------------------------
      call plotdefs

      plotdim=1
      dimx=ndata
      dimy=1
      nsets=1
      nplots=1
      lpltime=.true.
      call initplot(filename,plotdim)

      call timebnds(pltime,ndata)

!-----------------------------------------------------------------------
! set program specific data, e.g. plot tasks
!-----------------------------------------------------------------------
      plottask=1

      cplottask(1)='Total norm square'
      cplottask(2)='A-norm square'
      cplottask(3)='Total norm'
      cplottask(4)='A-norm'
      cplottask(5)='diff norm^2 - A-norm^2'
      do i=1,nplottask
         cptlen(i)=slen(cplottask(i))
      enddo

  250 continue
      call plotmenu(task,plottask,plotdim)

!-----------------------------------------------------------------------
! do task
! TASK=0 STOP  else return to menu
!-----------------------------------------------------------------------
      if (task .gt. 0) then
         if (plottask .eq. 1) then
            plttitle='Total norm square'
            axislab(1)='Time [fs]'
            axislab(3)='norm^2'
         else if (plottask .eq. 2) then
            plttitle='A-vector norm square'
            axislab(1)='Time [fs]'
            axislab(3)='A-norm^2'
         else if (plottask .eq. 3) then
            plttitle='Total norm'
            axislab(1)='Time [fs]'
            axislab(3)='norm'
         else if (plottask .eq. 4) then
            plttitle='A-vector norm'
            axislab(1)='Time [fs]'
            axislab(3)='A-norm'
         else if (plottask .eq. 5) then
            plttitle='Difference between total and A-vector norm^2'
            axislab(1)='Time [fs]'
            axislab(3)='Diff'
         endif
         if(fs.lt.2.d0) axislab(1)=axislab(1)(1:4)
         if (lrebound) then
            call databnds(pldata(1,plottask),dimx,dimy,nsets,nplots,&
                 ndata,1,1,1)
         endif
         call dotask(task,plottask,plotdim,pltime,ydum,&
              pldata(1,plottask),dimx,dimy,nsets,nplots,ndata,1,1,1,&
              xdum,1,pltime,ndata,edum,1,rdum,1,1,1,1,idum)

         goto 250
      endif

      close(iaus,status='delete')

      return
         
 998  continue
      write(6,'(a)') ' Error reading restart file'
      stop   
      end subroutine norm1

!#######################################################################

      subroutine norm0(ovl,aovl,psi,ovr,workc,agmat)

use global
use griddatmod
use psidef
use datenmod
use normmod

use gwplib, only: diagonal_moments
use lalib, only: simtran1bz
use op1lib, only: cpqxz

      implicit none

      integer(long)                                  :: s,t,m,zeig1,zeig2,swapzeig 
      complex(dop)                                   :: zdum
      complex(dop), intent(out)                      :: ovl,aovl
      complex(dop), dimension(1)                     :: cdum,cdum1
      complex(dop), dimension(dgldim), intent(in)    :: psi
      complex(dop), dimension(dmatdim), intent(out)  :: ovr
      complex(dop), dimension(workcdim), intent(out) :: workc
      complex(dop), dimension(agmatdim), intent(in)  :: agmat
      complex(dop), dimension(gsmatdim)              :: govr

      ovl=0.0_dop
      aovl=0.0_dop
!-----------------------------------------------------------------------
!     calculate density matrices for each degree of freedom defined as 
!     overlap of the single-particle functions rho^m_ij=<phi_i,2|phi_j
!     ,1>
!-----------------------------------------------------------------------
      if ((dentype.eq.0).or.(dentype.eq.2)) then
         do s=1,nstate
            do m=1,nmode
               if (gwpm(m)) then
                  call diagonal_moments(m,s,s,&
                       nspfdof(m),gwpdim(m,s),gwpdim(m,s),&
                       agmat(zaGmat(m,s)),agmat(zaGmat(m,s)),&
                       agmat(zxivec(m,s)),agmat(zxivec(m,s)),&
                       agmat(zphase(m,s)),agmat(zphase(m,s)),&
                       govr(gsmat(m,s)),cdum,cdum1,0,0)
                  if (psitype .eq. 0) then
                     call simtran1bz(govr(gsmat(m,s)),psi(zdgwp(m,s)), &
                          ovr(dmat(m,s)),gwpdim(m,s),dim(m,s))
                  else
                     call cpqxz(govr(gsmat(m,s)),ovr(dmat(m,s)), &
                          gwpdim(m,s))
                  endif

               else 
                  call mmaxzz(psi(zetf(m,s)),psi(zetf(m,s)),&
                       ovr(dmat(m,s)),subdim(m),dim(m,s),dim(m,s))
               endif
            enddo
         enddo
      else
         do s=1,nstate
            do t=1,nstate
               do m=1,nmode
                  call mmaxzzh(psi(dzetf(m,s,t)),&
                       psi(dzetf(m,s,t)),ovr(ddmat(m,s,t)),&
                       subdim(m),ddim(m,s,t))
               enddo
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J|Phi_L>A_L within each state 
!-----------------------------------------------------------------------
      if (dentype.eq.0) then
         ovl=(0.0_dop,0.0_dop)
         do s=1,nstate
            zeig1=1
            zeig2=block(s)+1
            call cpvxz(psi(zpsi(s)),workc(zeig2),block(s))
            do m=1,nmode
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig
               call qtxxzz(ovr(dmat(m,s)),workc(zeig1),workc(zeig2),&
                    vdim(m,s),dim(m,s),ndim(m,s))
            enddo

!-----------------------------------------------------------------------
! calculate Sum_J A_J^* Sum_L <Phi_J|Phi_L>A_L within each state 
!-----------------------------------------------------------------------
            call vvaxzz(psi(zpsi(s)),workc(zeig2),zdum,block(s))
            ovl=ovl+zdum
         enddo

      else if (dentype.eq.2) then

!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J|Phi_L>A_L within each state 
!-----------------------------------------------------------------------
         ovl=(0.0_dop,0.0_dop)

         do s=1,nstate
            do t=1,nstate
               zeig1=1
               zeig2=block(s)*block(t)+1
               call cpvxz(psi(dzpsi(s,t)),workc(zeig1),&
                    block(s)*block(t))
               do m=1,nmode
                  call mtxxzz(ovr(dmat(m,s)),workc(zeig1),workc(zeig2),&
                       vdim(m,s),dim(m,s),ndim(m,s)*block(t),dim(m,s))
                  call swapsxxi(zeig1,zeig2)
               enddo

               do m=1,nmode
                  call mttxzz(ovr(dmat(m,t)),workc(zeig1),workc(zeig2),&
                       block(s)*vdim(m,t),dim(m,t),ndim(m,t),dim(m,t))
                  call swapsxxi(zeig1,zeig2)
               enddo

!-----------------------------------------------------------------------
! calculate Sum_J A_J^* Sum_L <Phi_J|Phi_L>A_L within each state 
!-----------------------------------------------------------------------
               call vvaxzz(psi(dzpsi(s,t)),workc(zeig1),zdum,&
                    block(s)*block(t))
               ovl=ovl+zdum
            enddo
         enddo
      else if (dentype.eq.1) then
         ovl=(0.0_dop,0.0_dop)
         do s=1,nstate
            do t=1,nstate
               zeig1=1
               zeig2=dblock(s,t)+1
               call cpvxz(psi(dzpsi(s,t)),workc(zeig2),dblock(s,t))
               
               do m=1,nmode
                  call swapsxxi(zeig1,zeig2)
                  call qtxxzz(ovr(ddmat(m,s,t)),workc(zeig1),&
                       workc(zeig2),vddim(m,s,t),ddim(m,s,t),&
                       nddim(m,s,t))
               enddo

!-----------------------------------------------------------------------
! calculate Sum_J A_J^* Sum_L <Phi_J|Phi_L>A_L within each state 
!-----------------------------------------------------------------------
               call vvaxzz(psi(dzpsi(s,t)),workc(zeig2),zdum,&
                    dblock(s,t))
               ovl=ovl+zdum
            enddo
         enddo        
      endif

!-----------------------------------------------------------------------
! calculate Sum_J A_J^*  * A_J 
! A-norm has no meaning for exact Psi
!-----------------------------------------------------------------------
      if (psitype .eq. 1) then
         aovl = 1.0
      else
         call vvaxzz(psi,psi,aovl,adim)
      endif
         
      return
      end subroutine norm0

!#######################################################################
!-----------------------------------------------------------------------
      subroutine norminput

use global
use aglobal
use griddatmod
use psidef
use channels
use normmod

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: i,iarg,ierr
      character(len=80)       :: buf
!-----------------------------------------------------------------------

      filein  = ' '
      name   = './'
      ebtype  = 0
      lallov  = .false.
      lprint  = .true.
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf .eq. '-inter' ) then
            linter=.true.
         else if( buf .eq. '-all' ) then
            lallov=.true.

         else if( buf .eq. '-n' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf .eq. '-nofs' ) then
            fs = 1.d0
         else if( buf .eq. '-r' ) then
            irst   = ipsi
            ebtype = -1
            write(6,'(a)') ' *** Reading the RESTART file !  ***'
         else if( buf .eq. '-s') then
            lprint = .false.
         else if( buf .eq. '-nofs' ) then
            fs = 1.d0
         else if( buf .eq. '-gnupl4 ') then
            lgnu4 = .true.                 ! gnuplot ver < 5
         else if( buf .eq. '-h' .or. buf .eq. '-?' ) then
            write(6,'(1x,a)')&
       'Purpose: Calculates the norm and A-norm of a wavefunction.',&
       ' or density operator of type II.'
            write(6,'(1x,a)')&
       'Usage:   norm [-f -i -r -n -all -s -nofs -inter -gnupl4 -ver -h -?] .'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a/,9x,a)') &
                     ' -f FILE : The wavefunction (density operator) '&
                    ,'is read from file '&
                    ,'  FILE rather than from ./psi'&
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(a)') ' -i DIR  : data is stored in directory DIR'
            write(6,'(2a)') ' -n step : Compute the norm', &
           ' only every step-th output.'
            write(6,'(a)')&
           ' -all    : Print the norm of all ML-nodes, not only the top one.',&
           '           -all is ignored for MCTDH runs. It must not be used with -inter.'

            write(6,'(a)') &
           ' -r      : Take the restart file rather than the psi file.'
            write(6,'(a)')&
           ' -s      : Suppress printing information on ML-tree.'
            write(6,'(2a)') ' -nofs   : No transformation to fs.', &
                            ' Use when "time-not-fs" was set in mctdh.'
            write(6,'(a)')  ' -inter  : enables interactive plotting.'
            write(6,'(a)')&
               ' -gnupl4  : Use gnuplot version < 5 for plotting '
            write(6,'(a)')&
           ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(a)') ' Note: The output is to screen.'
            write(6,'(2a)') ' For ML-wavefunctions the norm of each layer is',&
                            ' evaluated, if "-all" is set.'
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      
      if (iarg .lt. i) goto 200

 200  continue

      if( step .gt.1  .and. ebtype .lt. 0) then
         write(6,'(a)') ' -n and -r options are incompatible. Set step=1'
         step = 1
      end if
      if(lallov .and. linter) then
         write(6,'(/2a/)') '#### -all and -inter are incompatible!',&
                           ' -all is unset. ####'
         lallov=.false.
      endif

      if( linter  .and. ebtype .lt. 0) then
         write(6,'(a)') &
       ' -inter and -r options are incompatible. Turn off interactive plotting'
         linter=.false.
      end if

!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
! Otherwise take name from filein
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         if (ebtype .ge. 0) then
            filein=name(1:laenge)//'/psi'
         else 
            filein=name(1:laenge)//'/restart'
         endif
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      open(ipsi,file=filein(1:laein),form='unformatted',&
           status='old',err=1000)
      write(6,'(2a)') ' The inputfile is : ',filein(1:laein)

      return 
      
 1000 write(6,*)' Cannot open file : '
      write(6,*) filein(1:laein)
      stop
      
      end subroutine norminput

      end program norm

