!***********************************************************************
!
!                         AUSWERT subroutine ORTHO
!
!     This subroutine checks the orthonormality of the SPFs.
!
!     05/99 HDM
!
!
!***********************************************************************
      
      module orthomod

      use decimal, only : long,dop

!-----------------------------------------------------------------------
!     logicals for ortho routines
!-----------------------------------------------------------------------
      logical(kind=4), save :: lrst,ltrace

      end module orthomod

!#######################################################################

      program ortho

use decimal, only: long,dop,sip
use global
use aglobal
use globmemmod
use versions
use dvrdatmod
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use compdat1mod
use orthomod
use logdat

use mmlib, only: mmaxzzh, qqaxzz, mmaxzz
use d2tens, only: chkherm, chkherm2
use op1lib, only: trqxz,cpqxz,trhxz,normvxz
use lalib, only: simtran1bz
use xvlib, only: vvxazz
use iorst, only: rstinfo
use iofile, only: rdpsiinfo
use zeigauswmod, only: zeigausw

use module_interface
use mltypes
use mlanalysis
use wftree1, only: dispose_wftree
use if_grid

use iorst, only: rdrst
use rdfilesmod, only: rdpsi,rdpsirst

      implicit none

      integer(long)                     :: ilbl,zeig,zeig1,zeig2,m,s,t,n,ll,&
                                           chkdvr,chkgrd,chkpsi,chkdat,chkprp
      integer(long)                     :: mcovr
      character(len=c5)                 :: filename, mdlbl
      character(len=80)                 :: nspf
      logical(kind=4)                   :: lerr,linwf

      integer(long)                                   :: i,k,idum
      integer(long), dimension(1)                     :: idum1
      complex(dop), dimension(1)                      :: cdum
      real(dop)                                       :: time,dt
      real(dop), dimension(1)                         :: rdum
      logical(kind=4)                                 :: ldum

! arrays to be used in analysis
      complex(dop), dimension(:), allocatable  :: psi,ovr,adgwp
      complex(dop), dimension(:), allocatable  :: workc
      complex(sip), dimension(:), allocatable  :: spsi
      integer(long), dimension(:), allocatable :: jindx,gwpdep
      character*10  buf

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      call orthoinput 

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='ortho.log',form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
! get memory sizes
!-----------------------------------------------------------------------
      call rdmemdim(ipsi)

!-----------------------------------------------------------------------
! Allocate mememory
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef

      if( lrst ) then
!-----------------------------------------------------------------------
! read data from restart file  (lrst=true --> read restart rathen than psi)
!-----------------------------------------------------------------------
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkprp=1
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         if (linwf) write(6,'(a)') ' Initial wave function' 
         if (lerr) then 
            write(6,'(a)') ' ERROR reading restart file' 
            stop
         end if
      else
!-----------------------------------------------------------------------
! read data from psi file
!-----------------------------------------------------------------------
         read(ipsi) filever(ipsi)
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkdat=1
         call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
         filename='psi'
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif

      if (nmode .eq. 1) then
         do s=1,nstate
            if (dentype.eq.1) then
               do t=1,nstate
                  ddim(1,s,t)=1
               enddo
            else
               dim(1,s)=1
            endif
         enddo
      endif
      
!-----------------------------------------------------------------------
! write mode information and head-line.
!-----------------------------------------------------------------------
    if (psitype .eq. 5) then
       write (6,*)
       call print_wftree(6,tree,modelabel,.true.)
       write (6,*)
       call dispose_wftree(tree)

    else
      write(6,'(/,a)') ' mode: No. of spf''s per state : modelabels'
      do m=1,nmode
         ll = 1
         mdlbl = '                                                       '
         nspf  = '                                                       '
         do  n=1,nspfdof(m)
            ilbl  = index(modelabel(spfdof(n,m)),' ')
            mdlbl = mdlbl(1:ll)//modelabel(spfdof(n,m))(1:ilbl)
            ll    = ll + ilbl
         end do
         if (dentype.eq.1) then
            t=1
            write(nspf,'(15i3)') (ddim(m,s,t),s=1,nstate)
         else
            write(nspf,'(15i3)') (dim(m,s),s=1,nstate)
         endif
            if(3*nstate+ll .le. 72 ) then
            write(6,'(i4,4a)') m,' :',nspf(1:3*nstate),' : ',&
                               mdlbl(1:ll)
         else
            write(6,'(i4,3a)') m,' :',nspf(1:3*nstate),' :'
            write(6,'(a,/)') mdlbl(1:ll)
         end if
      enddo
      if (dentype.eq.0) then
         write(6,'(/,2a)') ' Error = ', &
           'SQRT[ (1/n) sum_{j=1}^n sum_{k=1}^n |O_{jk}-delta_{jk}|^2 ]'
         write(6,'(/,a,33(a,i2,a))')&
              '# Time[fs]  state  total  ',(' mode(',m,') ',m=1,nmode)

      else if(dentype.eq.1) then
         write(6,'(/,a,33(a,i2,a))')&
              '# Time[fs]   states   total  ',&
              (' mode(',m,') ',m=1,nmode),' Hermiticity error'

      else if (dentype.eq.2) then
         write(6,'(/,a)') &
              '                 orthonormality of spf '//&
              '        Trace error         Hermiticity error'
         write(6,'(a,33(a,i2,a),33(a,i2,a),33(a,i2,a))')&
              '# Time[fs]  state  total  ',(' mode(',m,') ',m=1,nmode),&
              (' mode(',m,') ',m=1,nmode),(' mode(',m,') ',m=1,nmode) 

      endif
   endif

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
!-----------------------------------------------------------------------
  100 continue
      lpsi=.true.
      lpsigrd=.false.
      call zeigausw

      zeig=1
      if (dentype.eq.1) then
         do s=1,nstate
            do t=1,nstate
               zeig1=dblock(s,t)
               do m=1,nmode
                  zeig=zeig+ddim(m,s,t)*ddim(m,s,t)
                  zeig2=zeig1
                  zeig2=zeig2+zeig1
                  workcdim=max(workcdim,zeig2)
               enddo
            enddo
         enddo
      else
         do s=1,nstate
            zeig1=block(s)
            do m=1,nmode
               zeig=zeig+dim(m,s)*dim(m,s)
               zeig2=zeig1
               zeig2=zeig2+zeig1
               workcdim=max(workcdim,zeig2)
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! assign memory arrays
!-----------------------------------------------------------------------
      call memausw
!      if (psitype.eq.5) dmatdim=1

!-----------------------------------------------------------------------
! work array size
!-----------------------------------------------------------------------
      workcdim=max(workcdim,2*maxblock)
      do m=1,nmode
         workcdim = max(workcdim,2*subdim(m)**2)
      enddo

      if (psitype.eq.5) workcdim=1

!-----------------------------------------------------------------------
! call overlap routine
!-----------------------------------------------------------------------

allocate(psi(dgldim))
allocate(spsi(dgldim))
allocate(ovr(dmatdim))
allocate(workc(workcdim))
allocate(jindx(adim))
if (lgwp) then
   allocate(agmat(agmatdim))
   allocate(adgwp(gsmatdim))
   allocate(gwpdep(gwpinfodim))
endif

!-----------------------------------------------------------------------
! calculate time step for data
!-----------------------------------------------------------------------
      dt = out2/fs
      time = tinit/fs
      k    = 0
   10 continue
!-----------------------------------------------------------------------
! choose overlap calculation routine
!-----------------------------------------------------------------------

      if (lrst) then
!-----------------------------------------------------------------------
! read restart wavefunction  (lrst=true --> read restart rather than psi)
!-----------------------------------------------------------------------
         rewind(irst)
         chkgrd = 0
         call rdrst(time,psi,ldum,0,chkgrd,0,0,cdum,idum,rdum,0,jindx,&
              agmat,idum1)
         read (irst,err=998) time
         time = time/fs
         call rdpsirst(irst,psi,agmat)

      else
         call rdpsi(ipsi,psi,spsi,jindx,agmat,idum1,adgwp,gwpdep)
         if (lend) goto 77
      end if

      if (mod(k,step) .eq. 0) then
         call ortho0(time,psi,ovr,workc,agmat)
         if (ltrace) call ortho1(time,psi,ovr,workc,agmat)
      endif

      k = k + 1
      time=time + dt

!-----------------------------------------------------------------------
! if more data, go to beginning of loop
!-----------------------------------------------------------------------
      if (.not.lrst)  goto 10

 77   continue
       if(.not.lrst) then
         if (psitype.eq.3) then
            call headline_tree(iout,tree,.false.,'#   Time[fs]','     total',buf)
         else
            write(6,'(a,999(a,i3,a))') &
               '# Time[fs]  state  total  ',('mode(',m,') ',m=1,nmode)
         endif
      endif
      write(6,'(76a)') ('-',i=1,76)
      close(iaus)
      close(ipsi)
      
! deallocate memory
deallocate(psi,ovr,workc)
deallocate(spsi)
deallocate(jindx)
if (allocated(agmat)) deallocate(agmat)
if (allocated(adgwp)) deallocate(adgwp)
if (allocated(gwpdep)) deallocate(gwpdep)

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_psidef

      stop
  999 continue
      message= 'Error reading :'//filename(1:c2-14)
      write(6,'(a)') message
      stop
         
 998  continue
      write(6,'(a)') ' Error reading restart file'
      stop   

      contains

!#######################################################################

      subroutine ortho0(time,psi,ovr,workc,agmat)

use global
use aglobal
use maxv, only: maxdim
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod
use orthomod

use gwplib, only: fgaover

      implicit none

      integer(long)                                  :: s,t,m,e,e1,p,zeig1,zeig2 
      complex(dop)                                   :: trace
      complex(dop), dimension(dgldim), intent(in)    :: psi
      complex(dop), dimension(dmatdim), intent(out)  :: ovr
      complex(dop), dimension(gsmatdim)              :: govr
      complex(dop), dimension(workcdim), intent(out) :: workc
      complex(dop), dimension(agmatdim), intent(in)  :: agmat
      real(dop)                                      :: tot,herm

      real(dop), intent(in)              :: time
      real(dop), dimension(maxdim)       :: glo,hermerr,traceerr
      logical(kind=4)                    :: lpsicm

      lpsicm=lpsiopt(2)


!-----------------------------------------------------------------------
!     calculate density matrices for each degree of freedom defined as 
!     overlap of the single-particle functions rho^m_ij=<phi_i,2|phi_j
!     ,1>
!-----------------------------------------------------------------------
      if (dentype.eq.1) then
!        check Eq. 37 in Raab et al., JCP 112, 10718-10729 (2000) 
         do s=1,nstate
            do t=1,nstate
               tot=0.0_dop
               hermerr(1) = 0.0_dop
               do m=1,nmode
                  glo(m)=0.0_dop
                  do e=1,ddim(m,s,t)
                     do e1=1,ddim(m,s,t)
                        call qqaxzz(psi(dzetf(m,s,t)+(e-1)*subdim(m)),&
                             psi(dzetf(m,s,t)+(e1-1)*subdim(m)),&
                             workc,supgdim(m))
!                       calculate the trace
                        call trqxz(workc,trace,supgdim(m))

                        if (e1.eq.e) then
                           glo(m)=glo(m)+abs(trace-1.0_dop)**2
                        else
                           glo(m)=glo(m)+abs(trace)**2
                        endif
!                       calculate the hermiticity error
                        if (e1.eq.e) then
                           call chkherm2(psi(dzetf(m,s,t)&
                                +(e-1)*subdim(m)),&
                                psi(dzetf(m,t,s)+(e-1)*subdim(m)),&
                                herm,&
                                supgdim(m),supgdim(m))
                           hermerr(1) = hermerr(1) + herm
                        endif
                     enddo      ! of e1
                  enddo         ! of e
                  glo(m)=sqrt(glo(m))
                  tot=tot+glo(m)
               enddo            ! of m
               write(6,'(f12.4, 2i3, 1p10e10.2,e10.2)') time,s,t,tot&
                       ,(glo(m),m=1,nmode),hermerr(1)
            enddo               ! of t
         enddo                  ! of s

      else if (dentype.eq.2) then
         do s=1,nstate
            do t=1,nstate
               tot=0.0_dop
               do m=1,nmode
                  glo(m)      = 0.0_dop
                  traceerr(m) = 0.0_dop
                  hermerr(m)  = 0.0_dop
!                 scalar product
                  call mmaxzz(psi(zetf(m,s)),psi(zetf(m,s)),&
                       ovr(1),subdim(m),dim(m,s),dim(m,t))
                  do e=1,dim(m,s)
                     do e1=1,dim(m,t)
                        p=e1+(e-1)*dim(m,t)
                        if ((s.eq.t).and.(e1.eq.e)) then
                           glo(m)=glo(m)+abs(ovr(p)-1.0_dop)**2
                        else
                           glo(m)=glo(m)+abs(ovr(p))**2
                        endif
                        
                        call vvxazz(psi(zetf(m,s)+(e-1)*subdim(m)),&
                             psi(zetf(m,t)+(e1-1)*subdim(m)),&
                             workc,subdim(m))
                        call vvxazz(psi(zetf(m,t)+(e1-1)*subdim(m)),&
                             psi(zetf(m,s)+(e-1)*subdim(m)),&
                             workc(subdim(m)**2+1),subdim(m))

!                       calculate the trace error
                        call trqxz(workc,trace,subdim(m))
                        if ((s.eq.t).and.(e1.eq.e)) then
                           traceerr(m) = traceerr(m)+abs(trace-1.0_dop)
                        else
!                                   this trace is used twice  > 
!                           traceerr(m) = traceerr(m)+0.5*abs(trace)
                        endif

!                       calculate the hermiticity error
                        call chkherm(workc,herm,subdim(m))
                        if ((s.eq.t).and.(e.eq.e1)) then
                           hermerr(m) = hermerr(m) + abs(herm)
                        else
                           call chkherm2(workc,workc(subdim(m)**2+1),&
                                herm,subdim(m),subdim(m))
                           hermerr(m) = hermerr(m) + abs(herm)
                        endif

                     enddo      ! of e1
                  enddo         ! of e
                  glo(m)=sqrt(glo(m))                
                  tot=tot+glo(m)
               enddo            ! of m

               write(6,'(f12.4,i3,1p10e10.2,1p10e10.2,1p10e10.2)') &
                    time,s,tot, &
                    (glo(m),m=1,nmode),(traceerr(m),m=1,nmode),&
                    (hermerr(m),m=1,nmode)
            enddo
         enddo
      else
        do s=1,nstate
           tot=0.0_dop
           do m=1,nmode
              glo(m)=0.0_dop

              if (gwpm(m)) then
                 zeig1=zetf(m,s)
                 zeig2=zeig1+dim(m,s)
                 call fgaover(govr(gsmat(m,s)),agmat(zphase(m,s)), &
                      agmat(zphase(m,s)),agmat(zxivec(m,s)), &
                      agmat(zxivec(m,s)),agmat(zagmat(m,s)),&
                      agmat(zagmat(m,s)),gwpdim(m,s),gwpdim(m,s),nspfdof(m))
                 if (psitype .eq. 4) then
                    call cpqxz(ovr(dmat(m,s)),govr(gsmat(m,s)),dim(m,s))
                 else if (psitype .eq. 0 .or. psitype .eq. 5) then
                    call simtran1bz(govr(gsmat(m,s)),psi(zdgwp(m,s)),&
                         ovr(dmat(m,s)),gwpdim(m,s),dim(m,s))
                 endif

              else
                 call mmaxzzh(psi(zetf(m,s)),psi(zetf(m,s)),&
                            ovr(dmat(m,s)),subdim(m),dim(m,s))

              endif

              do e=1,dim(m,s)
                 do e1=1,dim(m,s)
                   p=dmat(m,s)-1+e1+(e-1)*dim(m,s)
                   if (e1.eq.e) then
                      glo(m)=glo(m)+abs(ovr(p)-1.0_dop)**2
                   else
                      glo(m)=glo(m)+abs(ovr(p))**2
                   endif
                enddo
             enddo
!             glo(m)=sqrt(glo(m))
             glo(m)=sqrt(glo(m)/dim(m,s)**2) 
             tot=tot+glo(m)
           enddo

! for ML-MCTDH calculate middlle layer values and output, else output
           if (psitype .eq. 5) then
              call ortho_tree(6,time,tree,psi,glo)
           else
              write(6,'(f12.4, i3, 1p999e10.2)') time,s,tot,(glo(m),m=1,nmode)
           endif
         enddo         
      endif

      if(nstate .gt. 1) write(6,*)
 
      return
      end subroutine ortho0

!#######################################################################

      subroutine ortho1(time,psi,ovr,workc,agmat)

use global
use aglobal
use maxv, only: maxdim
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod
use orthomod

use gwplib, only: fgaover

      implicit none

      integer(long)                                  :: s,t,m,e,e1,p,zeig1,zeig2 
      complex(dop)                                   :: trace
      complex(dop), dimension(dgldim), intent(in)    :: psi
      complex(dop), dimension(dmatdim), intent(out)  :: ovr
      complex(dop), dimension(gsmatdim)              :: govr
      complex(dop), dimension(workcdim), intent(out) :: workc
      complex(dop), dimension(agmatdim), intent(in)  :: agmat
      real(dop)                                      :: tot,herm

      real(dop), intent(in)              :: time
      real(dop), dimension(maxdim)       :: glo,hermerr,traceerr
      logical(kind=4)                    :: lpsicm


      if (dentype.ne.1) return

      lpsicm=lpsiopt(2)
!-----------------------------------------------------------------------
!     calculate density matrices for each degree of freedom defined as 
!     overlap of the single-particle functions rho^m_ij=<phi_i,2|phi_j
!     ,1>
!-----------------------------------------------------------------------
      do s=1,nstate
         do t=1,nstate
            do m=1,nmode
               write(6,'(a3i5)') ' Trace of SPDOs m,s,t : ',m,s,t
               call analspdo(psi(dzetf(m,t,s)),supgdim(m),ddim(m,t,s))
            enddo            ! of m
         enddo               ! of t
      enddo                  ! of s

      return
      end subroutine ortho1

!#######################################################################

      subroutine analspdo(spdo,supgdim1,dim1)

      implicit none

      integer(long), intent(in) :: supgdim1,dim1
      integer(long)             :: g,e,subdim1
      real(dop)                 :: trace,norm
      complex(dop), dimension(supgdim1,supgdim1,dim1), intent(in) :: spdo

      subdim1 = supgdim1*supgdim1
      do e=1,dim1
         call trhxz(spdo(1,1,e),trace,supgdim1)
         call normvxz(spdo(1,1,e),norm,subdim1)

         write(6,'(a,i5,2f12.8)') 'SPDO :',e,trace,norm
      enddo

      return
      end subroutine analspdo

!#######################################################################

      subroutine ortho_tree(iout,time,tree,psi,glo)

      implicit none
      integer,intent(in)          :: iout   ! output channel
      real(dop),intent(in)        :: time   ! time of wavefunction
      type(wfnode_t),intent(in)   :: tree   ! the tree
      complex(cdbl),intent(in)    :: psi(:) ! wavefunction
      real(dop), intent(in)       :: glo(maxdim)
      type(wfnode_t),pointer      :: no
      integer                     :: m
      real(dbl)                   :: o,otot

      otot = ZEROR
      write (iout, '(f12.4)', advance='no') time
      do m=1,tree%tot_nodes
         no => tree%swap_down(m)%p
         if (no%istop) cycle
         if (no%isprimitive) then
            o = glo(no%m)
         else
            call ortho_node(no,psi,o)
         endif
         otot = otot + o
         write (iout, '(es10.2)', advance='no') o
      enddo
      write (iout, '(es10.2)') otot
      return
      end subroutine ortho_tree

      subroutine ortho_node(cnode,psi,o)
      ! Measures how close the SPFs of a node are to orthonormality,
      ! assuming that the SPFs of the children are actually orthonormal.
      ! Evaluates:
      ! o = \sqrt{ \sum_{ij} | < \phi_i | \phi_j > - \delta_{ij} |^2 }

      implicit none
      type(wfnode_t),intent(in) :: cnode
      complex(cdbl),intent(in)  :: psi(:)
      real(dbl),intent(out)     :: o

      integer       :: i,j,z1i,z1f,z2i,z2f
      complex(cdbl) :: ovl

      o = 0.d0
      do i=1,cnode%navecs
         z1i = cnode%z(i,1)
         z1f = cnode%z(i,2)
         do j=i,cnode%navecs
            z2i = cnode%z(j,1)
            z2f = cnode%z(j,2)
            ovl = dot_product(psi(z1i:z1f),psi(z2i:z2f))
            if (i==j) then
               o = o + abs(ovl - CONE)**2
            else
               o = o + 2.d0 * abs(ovl)**2
            endif
         enddo
      enddo
      o = sqrt(o/dble(cnode%navecs))

      return
      end subroutine ortho_node

!#######################################################################
!-----------------------------------------------------------------------
      subroutine orthoinput

use global
use aglobal
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use compdat1mod
use orthomod

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: i,iarg,ierr
      character(len=80)       :: buf
!-----------------------------------------------------------------------

      filein  = '  '
      name    = './'
      lrst    = .false.
      ltrace  = .false.
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-f') then
            i = i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i = i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf .eq. '-n ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf .eq. '-nofs' ) then
            fs = 1.d0
         else if( buf .eq. '-tr' ) then
            ltrace = .true.
         else if( buf .eq. '-r' ) then
            irst = ipsi
            lrst = .true.
            write(6,'(a)') ' *** Reading the RESTART file !  ***'
         else if( buf .eq. '-h' .or. buf .eq. '-?' ) then
            write(6,'(1x,a)')&
       'Purpose: Checks the orthonormality of the SPFs (SPDOs).'
            write(6,'(1x,a)')&
       '         For density operators it checks also the'
            write(6,'(1x,a)')&
       '         hermiticity of the one-particle density operators.'
            write(6,'(1x,a)')&
       'Usage:   ortho [-f -i -n -r -nofs -ver -h -?] .'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a)') &
                ' -f FILE : The wavefunction (density operator) ',&
                'is read from file FILE rather than from ./psi',&
                '  The string FILE may be a relative or a full path-name.'
            write(6,'(a)') ' -i FILE : Same as -f FILE.'
            write(6,'(2a)') ' -n step : Compute the orthonormality', &
           ' error only every step-th output.'
            write(6,'(a)') &
           ' -r      : Take the restart file rather than the psi file.'
            write(6,'(2a)') &
            ' -nofs  : No transformation to fs. Use when', &
                       ' "time-not-fs" was set in mctdh.'
            write(6,'(a)') &
           ' -tr     : For pMCTDH(I), analyse the trace of the SPDOs'
            write(6,'(a)')&
           ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
           ' -h      : Print this help text. ',&
           ' -?      : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      
      if (iarg .lt. i) goto 200

 200  continue
      if( step .gt.1  .and. lrst ) then
         write(6,'(a)') ' -n and -r options are incompatible'
         stop
      end if

!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
! Otherwise take name from filein
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         if(lrst)then
            filein=name(1:laenge)//'/restart'
         else 
            filein=name(1:laenge)//'/psi'
         endif
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      open(ipsi,file=filein(1:laein),form='unformatted',&
           status='old',err=1000)
      write(6,'(78a1)')  ('*',i=1,78)
      write(6,'(2a)') ' Analyse program "ortho". ', &
                      'Check the orthonormality of the spf''s'
      write(6,'(2a)') ' The inputfile is : ',filein(1:laein)

      return 
      
 1000 write(6,*)' Cannot open file : '
      write(6,*) filein(1:laein)
      stop
      
      end subroutine orthoinput

      end program ortho

