!*********************************************************************
!
!                         AUSWERT subroutine OVERLAP
!
!     This subroutine calculates the overlap between the wave functions
!     of two different calculations. The wave functions may differ in
!     the number of spfs but not in the number of grid points of a
!     certain dof. When overlaping density operators, then in addition 
!     the combination schemes must be identical.
!     (If the combination schemes differ, the WF overlap will be slow!)
!     The Bra-state is called ps1 and goes with save-system(2).
!     The Ket-state is called ps  and goes with save-system(1).
!     The time-overlap table is saved as 'ovl_2' in the directory 
!     of the first calculation, i.e the directory of the Ket-state.
!     The 2 obove stands for the name of the second calculation,
!     i.e. the name-directory of the Bra-state.
!
!     11/96 MB
!
!
!***********************************************************************

      module overlapmod

      use decimal, only : long,dop
use mltypes
use wftree
use wftree1
use versions

!-----------------------------------------------------------------------
! options for overlap calculation
!-----------------------------------------------------------------------
      integer(long), save :: ovtype,ebtype,init,nst1,nst2,sbra,sket,&
                             ovmaxblock,idxbra,idxket

!-----------------------------------------------------------------------
!     pointers for overlap routine
!-----------------------------------------------------------------------
      integer(long), save :: doverdim,dgldim1,dgldim0,adim1,dmatdim1,&
                             agmatdim1,gsmatdim1,gdoverdim,gwpinfodim1, &
                             dmatdim2

      integer(long), dimension(:,:), allocatable, save   :: dovmat,gdovmat
      integer(long), dimension(:,:,:), allocatable, save :: ddovmat
      complex(dop), dimension(:), allocatable, save      :: agmat1

!-----------------------------------------------------------------------
! ML trees
!-----------------------------------------------------------------------
      type(wfnode_t), pointer :: kettree, bratree

!-----------------------------------------------------------------------
!     real variables for overlap routine
!-----------------------------------------------------------------------
      real(dop), save :: tend

!-----------------------------------------------------------------------
!     logical for overlap routine
!-----------------------------------------------------------------------
      logical(kind=4), save :: lrst,lrst1,lallov,lback,lcc,lsingle,lshort, &
                               lcrsspf,lcrsdiag,lpp,lmatrixel,lphasereal
      logical(kind=4), save :: pernode

      end module overlapmod

      program overlap

use decimal, only: long,dop,sip
use global
use globmemmod
use aglobal
use dglobal
use maxv, only: maxdim,maxsta
use dvrdatmod
use griddatmod
use psidef
use channels
use datenmod
use overlapmod
use compdatmod
use compdat1mod
use compdat

use gwplib, only: diagonal_moments
use mlanalysis, only: overlap_1,overlap_2,overlap_3

use rdfilesmod, only: rdpsi,rdpsigrid
use mltypes
use wftree
use mlanalysis
use dmatmod
use if_grid
use logging
use lazyflush_m

use lalib, only: simtran1bz, simtran2bz
use op1lib, only: cpvxz,normvxz,cpqxz,cpmxz
use mmlib, only: mmaxzz, mmaxzzh
use mtlib, only: qtxxzz, mtxxzz, mttxzz
use xvlib, only: vvaxzz, vvtxzz, vvxazz
use op2lib, only: swapsxxi
use d2tens, only: sorttens
use logdat
use compdat, only: savecsys,savecpsi
use iofile, only: rdpsiinfo
use auswutil, only: savecdat
use zeigauswmod, only: zeigausw

use module_interface

      implicit none

      integer(long)                     :: zeig,zeig1,zeig2,m,s,t,ilbl,&
                                           chkdvr,chkgrd,&
                                           chkpsi,chkdat,d1,d2,maxsubdim,maxspf1
      integer(long)                     :: mcdover,mcovr,mcovr1,psitype1
      character(len=c5)                 :: filename
      logical(kind=4)                   :: readsys,lerr

      integer(long)                              :: aerr
      character(len=20)                          :: cerr
      logical(kind=4), external                  :: int2log

      integer(long), external   :: integersize, realsize, logicalsize

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      call ovrinput

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (ilog,file='ovrlp.log',form='formatted')
      logisopen=.true.
      call fpstartup(filename)
      write(ilog,'(/a78)') filename
      write(ilog,'(2a)')    '# Options: ',options(1:optlaenge)

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(ipsi)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef
      if (ldynspf) call alloc_dynwf
      call alloc_compdat

!-----------------------------------------------------------------------
! set no. of comparison data sets
!-----------------------------------------------------------------------
      ncomp=2

!-----------------------------------------------------------------------
! read data from data directory  (ket)
!-----------------------------------------------------------------------
      rewind(ipsi1)
      read(ipsi1) filever(ipsi1)
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      call rdpsiinfo(ipsi1,chkdvr,chkgrd,chkpsi,chkdat)
      psitype1 = psitype
      filename='psi1'
      call chksyserr(filename,lerr)
      if (lerr) go to 999

!     Dynamical WF
      if (ldynspf) dglmax=dgldim

!-----------------------------------------------------------------------
! save data as set 2
!-----------------------------------------------------------------------
      call savecsys(2)
      call savecpsi(2)
      call savecdat(2)
      maxspf1=maxspf

!-----------------------------------------------------------------------
! read data from comparison psi file (bra)
!-----------------------------------------------------------------------
      rewind(ipsi)
      read(ipsi) filever(ipsi)
      chkdvr=5
      if(sbra.gt.0) chkdvr=6
      chkgrd=4
      chkpsi=1
      chkdat=1
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      if (psitype .ne. psitype1) then
         routine = 'Overlap'
         message = 'Wavefunctions are of different types'
         call errormsg
      endif
      if (chkerr(1) .ne. 0) go to 999
      maxspf=max(maxspf,maxspf1)
      if(sbra.gt.0) chkerr(2)=0

!     Dynamical WF
      if (ldynspf) dglmax=max(dgldim,dglmax)

      if (psitype .eq. 3 .or. cpsitype(2) .eq. 3) then
         routine='Overlap'
         message = 'Cannot run overlap on trajectory calculation'
         call errormsg
      endif

!-----------------------------------------------------------------------
! compare psi and psi1 data to decide on overlap calculation type
!-----------------------------------------------------------------------
      ovtype = 0
      ebtype = 0
      readsys = .false.
      if (chkerr(2) .ne. 0 .or. chkerr(3) .ne. 0) then
         if(lignore) then
            write(6,'(a,/a)') ' ############ WARNING #############',&
                              'Different primitive bases'
         else
            routine='Overlap'
            message='Different primitive bases'
            call errormsg
         endif
      endif
      if (chkerr(4) .ne. 0) then
         ovtype = 1
         if (dentype .ne. 0) then
            routine='Overlap'
            message='ovtype=1 not yet implemented for density operators'
            call errormsg
         endif
         readsys=.true.
      endif
      if (chkerr(5) .ne. 0  .and.  sbra.eq.0 ) then
         ebtype = 1
         readsys=.true.
      endif

!.....check for ML wavefunctions
      if (psitype.eq.5 .or. cpsitype(2).eq.5) then
         if (psitype.ne.cpsitype(2)) then
            routine='Overlap'

            message='overlap of ML with non-ML wavefunction not implemented, use ml2mctdh'
            call errormsg
         endif
         ovtype = 0
      endif

      write(ilog,'(a,i3,a,i3)') 'ovtype =',ovtype,' ,  ebtype =',ebtype
 
!-----------------------------------------------------------------------
! now read data for psi file
!-----------------------------------------------------------------------
      if (readsys) then
         rewind(ipsi)
         read(ipsi) filever(ipsi)
         chkdvr=5
         if(sbra.gt.0) chkdvr=6
         chkgrd=1
         chkpsi=1
         chkdat=1
         call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)      
         if (nmode .eq. 1) then
            do s=1,nstate
               dim(1,s)=1
               do t=1,nstate
                  ddim(1,s,t)=1
               enddo
            enddo
         endif
      endif

!-----------------------------------------------------------------------
! save data as set 1
!-----------------------------------------------------------------------
      call savecsys(1)
      call savecpsi(1)
      call savecdat(1)

!-----------------------------------------------------------------------
! Check if sbra is set (-s option). Only WFs and ovtype=0 are then allowed. 
!-----------------------------------------------------------------------
      if (sbra.gt.0) then
         if((cdentype(1).ne.0) .or. (cdentype(2).ne.0)) then
            routine='overlap'
            message='-s option is not allowed for density operators'
            call errormsg
         endif  
         if(nstate.eq.1) then
            routine='overlap'
            message='-s option requires multi-set electronic states.'//&
                    ' Here nstate=1.'
            call errormsg
         elseif(ovtype .ne. 0) then
            routine='overlap'
            message='-s option requires identical grids'//&
                    ' and combination schemes' 
            call errormsg
         endif  
      endif

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
!-----------------------------------------------------------------------
      if(allocated(dovmat))deallocate(dovmat)
      if(allocated(gdovmat))deallocate(gdovmat)
      if(allocated(ddovmat))deallocate(ddovmat)
      cerr = 'overlap0_dovmat  '
      allocate(dovmat(maxdim,maxsta),stat=aerr)
      allocmemory=allocmemory + maxdim*maxsta*realsize()
      if (aerr > 0) go to 1000
      cerr = 'overlap0_gdovmat  '
      allocate(gdovmat(maxdim,maxsta),stat=aerr)
      allocmemory=allocmemory + maxdim*maxsta*realsize()
      if (aerr > 0) go to 1000
      cerr = 'overlap0_ddovmat  '
      allocate(ddovmat(maxdim,maxsta,maxsta),stat=aerr)
      allocmemory=allocmemory + maxdim*maxsta*realsize()
      if (aerr > 0) go to 1000

      lpsi=.true.
      lpsi1=.true.
      lpsigrd=ovtype.eq.1
      call zeigausw

!-----------------------------------------------------------------------
!  Get work array dimensions 
!  NEEDS TIDYING - workc should be assigned in subroutines!
!-----------------------------------------------------------------------
      if (ovtype.eq.0) then
!    --- Both datasets are of density type I with the same mode combinations
!        (ovtype=0).
         if ((cdentype(1).eq.1).and.(cdentype(2).eq.1)) then
            do s=1,nstate
               do t=1,nstate
                  zeig1=cdblock(s,t,1)
                  do m=1,nmode
                     zeig2=zeig1
                     zeig1=zeig1*cddim(m,s,t,2)/cddim(m,s,t,1)
                     zeig2=zeig2+zeig1
                     maxblock=max(maxblock,zeig1)
                     workcdim=max(workcdim,zeig2)
                  enddo
               enddo
            enddo
            zeig=1
            do s=1,nstate
               do t=1,nstate
                  zeig1=cdblock(s,t,2)
                  do m=1,nmode
                     ddovmat(m,s,t)=zeig
                     zeig=zeig+ddim(m,s,t)*cddim(m,s,t,2)
                     zeig2=zeig1
                     zeig1=zeig1*cddim(m,s,t,1)/cddim(m,s,t,2)
                     zeig2=zeig2+zeig1
                     maxblock=max(maxblock,zeig1)
                     workcdim=max(workcdim,zeig2)
                  enddo
               enddo
            enddo
            workcdim=max(workcdim,2*maxblock)
            doverdim=zeig-1

!    --- The first dataset is of density type I, the second of density type II.
!        Both datasets have the same mode combinations (ovtype=0).    
         else if ((cdentype(1).eq.1).and.(cdentype(2).eq.2)) then
            zeig=1
            do s=1,nstate
               do t=1,nstate
                  zeig1=cblock(s,2)*cblock(t,2) + cdblock(s,t,1)
                  do m=1,nmode
                     ddovmat(m,s,t)=zeig
                     zeig=zeig+ddim(m,s,t)*cdim(m,s,2)*cdim(m,t,2)
                     zeig2=zeig1
                     zeig1=zeig1*&
                          cddim(m,s,t,1)/(cdim(m,s,2)*cdim(m,t,2))
                     zeig2=zeig2+zeig1
                     maxblock=max(maxblock,zeig1)
                     workcdim=max(workcdim,zeig2)
                  enddo
               enddo
            enddo
            workcdim=max(workcdim,2*maxblock)
            doverdim=zeig-1
            
!    --- The first  dataset is of density type II, the second of density type I.
!        This case is not implemented. Interchange instead the input!
         else if ((cdentype(1).eq.2).and.(cdentype(2).eq.1)) then
            write(6,*) ' Interchange the two types !' 
            stop

!    --- Both datasets are wavefunctions with same mode combinations.
!        The option -s is set.            
         elseif(sbra.gt.0) then 
            zeig1=max(cblock(sket,1),cblock(sbra,2))
            do m=1,nmode
               zeig2=zeig1
               zeig1=zeig1*cdim(m,sbra,2)/cdim(m,sket,1)
               maxblock=max(maxblock,zeig1)
               zeig2=zeig2+zeig1
               workcdim=max(workcdim,zeig2)
            enddo
            zeig=1
            zeig1=max(cblock(sbra,2),block(sket))
            do m=1,nmode
               dovmat(m,1)=zeig
               zeig=zeig+cdim(m,sket,1)*cdim(m,sbra,2)
               zeig2=zeig1
               zeig1=zeig1*dim(m,sket)/cdim(m,sbra,2)
               maxblock=max(maxblock,zeig1)
               zeig2=zeig2+zeig1
               workcdim=max(workcdim,zeig2)
            enddo
            workcdim=max(workcdim,2*maxblock)
            doverdim=zeig-1

!    --- Both datasets are wavefunctions or density operators of type II.
!        In both cases they have the same mode combinations.            
         else 
            do s=1,nstate
               zeig1=cblock(s,1)
               do m=1,nmode
                  zeig2=zeig1
                  zeig1=zeig1*cdim(m,s,2)/cdim(m,s,1)
                  zeig2=zeig2+zeig1
                  maxblock=max(maxblock,zeig1)
                  workcdim=max(workcdim,zeig2)
               enddo
            enddo
            zeig=1
            do s=1,nstate
               zeig1=cblock(s,2)
               do m=1,nmode
                  dovmat(m,s)=zeig
                  zeig=zeig+cdim(m,s,1)*cdim(m,s,2)
                  zeig2=zeig1
                  zeig1=zeig1*cdim(m,s,1)/cdim(m,s,2)
                  zeig2=zeig2+zeig1
                  maxblock=max(maxblock,zeig1)
                  workcdim=max(workcdim,zeig2)
               enddo
            enddo
            workcdim=max(workcdim,2*maxblock)
            doverdim=zeig-1
            zeig=1
            do s=1,nstate
               do m=1,nmode
                  gdovmat(m,s)=zeig
                  if (gwpm(m)) zeig=zeig+cgwpdim(m,s,1)*cgwpdim(m,s,2)
               enddo
            enddo
            gdoverdim=max(zeig-1,1)
         endif    

         griddim=1
         if (lrunml) workcdim = 1   ! workc not used for ML-MCTDH

      else if (ovtype .eq. 1) then ! ovtype=1 means, that the two datasets
         doverdim=1                ! hasn't the same mode combinations!
         griddim=nstate            ! Only implemented for wavefunctions. 
         do m=1,nmode
            griddim=griddim*subdim(m)
         enddo

      else
         message = 'unimplemented ovtype'
         call errormsg

      endif
      
      
!-----------------------------------------------------------------------
! assign memory arrays
!-----------------------------------------------------------------------
      call memausw

      mcovr=1
      mcovr1=1
      mcdover=1
      if (ovtype .eq. 0) then
         mcovr=mcdim
         mcdim=mcovr+cdmatdim(1)
         mcovr1=mcdim
         mcdim=mcovr1+cdmatdim(2)
         if (dentype .eq. 0) then
            mcdover=mcdim
            mcdim=mcdover+2*doverdim ! 2* is needed in wfd2overlap0
         else if (dentype .eq. 1) then
            mcdover=mcdim
            mcdim=mcdover+doverdim
         else if (dentype .eq. 2) then
            mcdover=mcdim
            mcdim=mcdover+2*doverdim
         endif
      else if (ovtype .eq. 1) then
         mcovr=1
         mcovr1=1
         mcdover=1
      endif

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      if (ovtype .eq. 0 .and. dentype .eq. 0) then
         workcdim=max(workcdim,2*maxblock)
      else if (ovtype .eq. 0 .and. dentype .eq. 1) then
         workcdim=max(workcdim,2*maxblock)
      else if (ovtype .eq. 0 .and. dentype .eq. 2) then
         workcdim=max(workcdim,2*maxblock*maxblock)
      else if (ovtype .eq. 1) then
         workcdim=max(workcdim,2*(griddim/cnstate(1))+cdgldim(1))
         workcdim=max(workcdim,2*(griddim/cnstate(2))+cdgldim(2))
      endif
      
      if (((cdentype(1).eq.1).and.(cdentype(2).eq.2))&
           .or.((cdentype(1).eq.2).and.(cdentype(2).eq.1))) then
         do s=1,nstate
            do t=1,nstate
               do m=1,nmode
                  maxsubdim=0
                  do d1=1,cdim(m,s,2)
                     do d2=1,cdim(m,t,2)
                        maxsubdim = maxsubdim + subdim(m)
                     enddo
                  enddo
                  workcdim = max(workcdim,maxsubdim)
               enddo
            enddo
         enddo
      endif

      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim 
      msdim   = msworks+worksdim

      mfworkf = mfdim 
      mfdim   = mfworkf+workfdim
      
!-----------------------------------------------------------------------
! call overlap routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

      call logmem('overlap')

!-----------------------------------------------------------------------
! get lengths of comparison data arrays
!-----------------------------------------------------------------------
      call getcpsi(2)
      dgldim1=dgldim
      adim1=adim
      dmatdim1=dmatdim
      dmatdim2=dmatdim
      agmatdim1=agmatdim
      gsmatdim1=gsmatdim
      gwpinfodim1=gwpinfodim
      call getcpsi(1)

if (lgwp) then
   allocate(agmat(agmatdim))
   allocate(agmat1(agmatdim1))
endif

      call ovrlap(mc(mcpsi),mc(mcpsi1),ms(mspsi),mc(mcdover),&
           mc(mcovr),mc(mcovr1),mc(mcworkc),mi(mijindx),&
           mi(mijindx1),agmat,agmat1,mc(mcadgwp),mc(mcadgwp1))

deallocate(mc,mr,mi,ml,ms,mf)
deallocate(dovmat,gdovmat,ddovmat)
if (allocated(agmat)) deallocate(agmat)
if (allocated(agmat1)) deallocate(agmat1)

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_psidef
      call dealloc_dynwf

      call dealloc_compdat

      stop
  999 continue
      ilbl=index(filename,' ')-1
      message= 'Error reading :'//filename(1:ilbl)
      write(6,'(a)') message
      stop

 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',aerr
      stop

      contains

!***********************************************************************
!     calculates the overlap
!***********************************************************************

      subroutine ovrlap (psi,psi1,spsi,dover,ovr,ovr1,workc,&
                 jindx,jindx1,agmat,agmat1,adgwp,adgwp1)

      implicit none

      integer(long)                                     :: nstep,nstep1,i,ncount
      integer(long)                                     :: m,s
      integer(long), dimension(1)                       :: idum
      integer(long), dimension(adim1), intent(out)      :: jindx1
      integer(long), dimension(adim), intent(out)       :: jindx
      integer(long), dimension(gwpinfodim)              :: gwpdep
      integer(long), dimension(gwpinfodim1)             :: gwpdep1
      complex(dop)                                      :: ovl1,ovl2,ovl
      complex(sip), dimension(spsidim), intent(out)     :: spsi
      complex(dop), dimension(dgldim1), intent(out)     :: psi1
      complex(dop), dimension(dgldim), intent(out)      :: psi
      complex(dop), dimension(*), intent(out)           :: dover
      complex(dop), dimension(dmatdim), intent(out)     :: ovr
      complex(dop), dimension(dmatdim1), intent(out)    :: ovr1
      complex(dop), dimension(workcdim), intent(out)    :: workc
      complex(dop), dimension(agmatdim), intent(out)    :: agmat
      complex(dop), dimension(agmatdim1), intent(out)   :: agmat1
      complex(dop), dimension(gsmatdim), intent(out)    :: adgwp
      complex(dop), dimension(gsmatdim1), intent(out)   :: adgwp1
      character(len=11)      :: nodebuf

      logical(kind=4) :: lover
      real(dop)       :: time,dt,error,hilbert,pherr,phcorrect,normsq,norm1sq

      integer                :: idelphi

!-----------------------------------------------------------------------
! allocate data structures for ML
!-----------------------------------------------------------------------
      ! bra
      if (cpsitype(1).eq.5) then
         bratree => new_wftree(ctape(:,1),if_spfdof)
         if (pernode) call alloc_dmat_wftree(bratree)
         call wftree_set_pointers(bratree,cdgldim(1))
      endif
      ! ket
      if (cpsitype(2).eq.5) then
         kettree => new_wftree(ctape(:,2),if_spfdof)
         if (pernode) call alloc_dmat_wftree(kettree)
         call wftree_set_pointers(kettree,cdgldim(2))
      endif

!-----------------------------------------------------------------------
! open data file for per-mode differences
!-----------------------------------------------------------------------
      if (pernode) then
         ! open data file
         call open_log(idelphi, 'modediff.dat')
         ! print header to data file
         write (idelphi,'(a,11(/,a))') &
            '# ----------------------------------------------------------------------',&
            '# Per-mode differences, computed by analyse program "overlap85".',&
            '# ----------------------------------------------------------------------',&
            '# The difference for one mode is computed via the expression',&
            '#   1 - tr(O^\dagger \rho^T O) / tr(\rho)',&
            '# where O is the overlap matrix between the mode-SPFs of the two',&
            '# wavefunctions, and \rho is the mode''s density matrix for one of',&
            '# the wavefunctions. As this expression treats the WFs asymmetrically,',&
            '# it is again evaluated with the role of the WFs reversed, and the',&
            '# arithmetic mean is taken.',&
            '# ----------------------------------------------------------------------',&
            '# The bra wavefunction tree is:'
         call print_wftree(idelphi,bratree,modelabel,.true.)
         write (idelphi,'(a,/,a)') &
            '# ----------------------------------------------------------------------',&
            '# The ket wavefunction tree is:'
         call print_wftree(idelphi,kettree,modelabel,.true.)
         write (idelphi,'(a)') &
            '# ----------------------------------------------------------------------'
         call headline_tree(idelphi,bratree,.false.,'#   Time[fs] Error^2','',nodebuf)
         call lazyflush(idelphi)
      endif

!-----------------------------------------------------------------------
! calculate time step for data
!-----------------------------------------------------------------------
      call getcpsi(1)
      lover=.false.
      if(lallov) cout2(2)=out2   ! -a option, set times equal. 
      if (out2.ge.cout2(2)) then
         if (mod(out2,cout2(2)).lt.tiny) then
            nstep1 = int((out2+tiny)/cout2(2))
            nstep = 1
         else
            lover=.true.
         endif
      else
         if (mod(cout2(2),out2).lt.tiny) then
            nstep = int((cout2(2)+tiny)/out2)
            nstep1 = 1
         else
            lover=.true.
         endif
      endif

      if (lover) then
!        12600 = 2*2*2*3*3*5*5*7
         if( out2 .ge. cout2(2) ) then
            nstep1 = int(12600.0_dop*out2/cout2(2)  + 0.5_dop)
            nstep  = 12600
         else
            nstep  = int(12600.0_dop*cout2(2)/out2  + 0.5_dop)
            nstep1 = 12600
         end if
         do  i = 10, 2, -1
            if (mod(nstep1,i).eq.0 .and. mod(nstep,i).eq.0) then
               nstep1 = nstep1/i
               nstep  = nstep/i
            end if
         end do
         do  i = 6, 2, -1
            if (mod(nstep1,i).eq.0 .and. mod(nstep,i).eq.0) then
               nstep1 = nstep1/i
               nstep  = nstep/i
            end if
         end do
         write(6,'(a,2f15.5)') ' output steps [fs]  : ',&
                                 out2/fs, cout2(2)/fs
         if(   abs(nstep*out2-nstep1*cout2(2)) .lt. &
            1.0e-6_dop*(nstep*out2+nstep1*cout2(2)) ) then
            write(6,'(a,f15.5)')&
                ' common output step : ',nstep*out2/fs
         else
            write(6,'(a)') ' output steps are incompatible '
            write(6,'(" out2:",2f19.8)') out2/fs,cout2(2)/fs
            write(6,'(1x,2f19.8,e12.3)') nstep*out2/fs,&
                 nstep1*cout2(2)/fs, nstep*out2-nstep1*cout2(2)
            stop
         endif
      endif

      nstep  = nstep *step
      nstep1 = nstep1*step
      dt     = out2/fs*nstep
      time   = tinit/fs

      write(ilog,'(/a,2f9.4,a,2i4,a,f9.5/)') ' tpsi:', &
         out2/fs,cout2(2)/fs,', nstep:',nstep,nstep1,', dt =',dt
      write(6,'(80a)') ('-',i=1,80)
      write(6,'(a12,4a11,1x,2a11)')&
        '    Time[fs]','      Error','    Hilbert',&
         '      Phase','   Ph. corr','  ||bra||^2','  ||ket||^2'
      
      ncount=0
      if (filever(ipsi) .ge. 11.0_dop) rewind(ipsi)
      if (filever(ipsi1) .ge. 11.0_dop) rewind(ipsi1)

      if(lpp) then
         if(fs.lt.2.0) then
            write(idat,'(a,99(3x,a,i2.2,1x,a,i2.2))') '#    Time  ',&
               (('m',m,'s',s,s=1,nstate),m=1,nmode)
         else
            write(idat,'(a,99(3x,a,i2.2,1x,a,i2.2))') '#   Time[fs]',&
               (('m',m,'s',s,s=1,nstate),m=1,nmode)
         endif
      endif

      if(lback) then
         if(nstep.ne.1 .or.  nstep1.ne.1) then
            write(6,'(a,2i3,/a)') 'WARNING: nstep, nstep1:',&
            nstep,nstep1,'Should be 1, because -b (back) is given.'
         endif
         call getcpsi(1)
   27    call rdpsi(ipsi,psi,spsi,jindx,agmat,idum,adgwp,gwpdep)
         ncount=ncount+1
         if (.not.lend) goto 27
      endif    

   10 continue
!-----------------------------------------------------------------------
! choose overlap calculation routine
!-----------------------------------------------------------------------
      if (ovtype .eq. 0) then
         call getcpsi(1)      ! read psi = Bra
!        Dynamical WF
         if (ldynspf) dgldim=dglmax
         if(lback) then  !  -b option, reverse order of bra-states.
            ncount=ncount-1
            if(ncount.eq.0) goto 77
            backspace ipsi
            backspace ipsi
         endif
         call rdpsi(ipsi,psi,spsi,jindx,agmat,idum,adgwp,gwpdep)
         if (lend) goto 77
         if(lcc) then     ! -c option, complex-symmetric scalar-product.
            do i = 1, dgldim
               psi(i) = dconjg(psi(i))
            enddo 
         endif
!        Dynamical WF
         if (ldynspf) then
            call savecsys(1)
            call savecpsi(1)
            call savecdat(1)
            if (lrunml) Then
               bratree => new_wftree(ctape(:,1),if_spfdof)
               if (pernode) call alloc_dmat_wftree(bratree)
               call wftree_set_pointers(bratree,cdgldim(1))
            endif
         endif

         call getcpsi(2)      ! read psi1 = Ket
!        Dynamical WF
         if (ldynspf) dgldim=dglmax

         call rdpsi(ipsi1,psi1,spsi,jindx1,agmat1,idum,adgwp1,gwpdep1)
         if (lend) goto 77

!        Dynamical WF
         if (ldynspf) then
            call savecsys(2)
            call savecpsi(2)
            call savecdat(2)
            if (lrunml) Then
               kettree => new_wftree(ctape(:,2),if_spfdof)
               if (pernode) call alloc_dmat_wftree(kettree)
               call wftree_set_pointers(kettree,cdgldim(2))
            endif
         endif

         call getcpsi(1)      ! restore data
         
         if ((cdentype(1).eq.0).and.(cdentype(2).eq.0)) then
            if( sbra .gt. 0 ) then
               call overlap0s(ovl,ovl1,ovl2,psi,psi1,dover,ovr,&
                              ovr1,workc,sbra,sket)
            else
              call overlap0(ovl,ovl1,ovl2,psi,psi1,dover,ovr,ovr1,workc,&
                   agmat,agmat1,time,dt)
            endif
         else if ((cdentype(1).eq.1).and.(cdentype(2).eq.1)) then
            call d1overlap0(ovl,ovl1,ovl2,psi,psi1,dover,ovr,ovr1,&
                 workc)
         else if ((cdentype(1).eq.2).and.(cdentype(2).eq.2)) then
!            call d2overlap0(ovl,psi,ovr,workc,1,dmatdim)
            call d2overlap0(ovl,psi,ovr,workc,1,dmatdim2)
            call d2overlap0(ovl1,psi1,ovr1,workc,2,dmatdim1)
            call d2overlap2(ovl2,psi1,psi,dover,workc)
         else if ((cdentype(1).eq.1).and.(cdentype(2).eq.2)) then
            
            call dooverlap0(ovl,ovl1,ovl2,psi,psi1,dover,ovr,ovr1,&
                 workc)

         else if ((cdentype(1).eq.2).and.(cdentype(2).eq.1)) then

            write(6,*) 'Not implemented yet!'
            write(6,*) 'Interchange the two different types! '
            stop
         else if ((cdentype(1).eq.0).and.(cdentype(2).eq.2)) then
            call wfd2overlap0(ovl,ovl1,ovl2,psi,psi1,dover,ovr,ovr1,&
                 workc)
         else if ((cdentype(1).eq.2).and.(cdentype(2).eq.0)) then
            write(6,*) 'Not implemented yet!'
            write(6,*) 'Interchange the two different types! '
            stop
         else    
            write(6,'(/2a)')  'Illegal combination of ',&
                   'wavefunction and density operator!'
            stop
         endif

      else if (ovtype .eq. 1) then
         call getcsys(1)
         call getcpsi(1)
         if(lback) then  !  -b option, reverse order of bra-states.
            ncount=ncount-1
            if(ncount.eq.0) goto 77
            backspace ipsi
            backspace ipsi
         endif
         call rdpsigrid(ipsi,psi,spsi,jindx,workc,lrst)
         if (lend) goto 77
         if(lcc) then     ! -c option, complex-symmetric sclar-product.
            do i = 1, griddim
               psi(i) = dconjg(psi(i))
            enddo 
         endif

         call getcsys(2)
         call getcpsi(2)
         call rdpsigrid(ipsi1,psi1,spsi,jindx1,workc,lrst)
         if (lend) goto 77

         call vvaxzz(psi,psi,ovl,griddim)
         call vvaxzz(psi1,psi1,ovl1,griddim)
         call vvaxzz(psi,psi1,ovl2,griddim)

      else
         message = 'not implemented'
         call errormsg

      endif

      normsq=dble(ovl)
      norm1sq=dble(ovl1)
      if(laeout.gt.0)  write(iaus,'(f12.4,2f23.16,1x,2f21.15)')&
                       time,ovl2,normsq,norm1sq
      
      error=normsq+norm1sq-2.0_dop*dble(ovl2)
      if (-error .gt. tiny) then
         write(6,*) ' WARNING: Error is negative ! error =', error
         error=0.0_dop
      endif
      error=sqrt(abs(error))

      hilbert=min(abs(ovl2)/dsqrt(normsq*norm1sq+1.0e-25_dop),1.0_dop)
!     hilbert=acos(hilbert)*(180.0_dop/pi)
      hilbert=acos(hilbert)

!     pherr=atan2(dimag(ovl2),dble(ovl2))*(180.0_dop/pi)
      pherr=atan2(dimag(ovl2),dble(ovl2))

      phcorrect=normsq+norm1sq-2.0_dop*abs(ovl2)
!      if (phcorrect .lt. 0.0_dop) phcorrect=0.0_dop
      phcorrect=sqrt(abs(phcorrect))

      write(6,'(f12.4,6f11.6)') time,error,hilbert,pherr,&
                                phcorrect,normsq,norm1sq 

      time=time + dt
      if(time-0.5*dt .gt. tend)  goto 77

!-----------------------------------------------------------------------
! if more data, go to beginning of loop
!-----------------------------------------------------------------------

      call getcpsi(2)
      do i=2,nstep1
         call rdpsi(ipsi1,psi1,spsi,jindx1,agmat1,idum,adgwp1,gwpdep1)
         if (lend) goto 77
      enddo
      call getcpsi(1)
      do i=2,nstep
         call rdpsi(ipsi,psi,spsi,jindx,agmat,idum,adgwp,gwpdep)
         if (lend) goto 77
      enddo

      goto 10

 77   continue
      write(6,'(a12,4a11,1x,2a11)')&
        '    Time[fs]','      Error','    Hilbert',&
         '      Phase','   Ph. corr','  ||bra||^2','  ||ket||^2'
      write(6,'(80a)') ('-',i=1,80)
      if(laeout.gt.0) close(iaus)
      close(ipsi)
      close(ipsi1)

!-----------------------------------------------------------------------
! deallocate ML data structures
!-----------------------------------------------------------------------
      if (cpsitype(1).eq.5) then
         call dispose_wftree(bratree)
      endif
      if (cpsitype(2).eq.5) then
         call dispose_wftree(kettree)
      endif
      call close_log(idelphi)

      return
      end subroutine ovrlap

!#######################################################################
!
! OVERLAP0
!
! Calculates the overlap between two wavefunctions in MCTDH form
!
!  ovl2 = <psi|psi1>,  ovl = <psi|psi>,   ovl1 = <psi1|psi1>
!
!#######################################################################

      subroutine overlap0(ovl,ovl1,ovl2,psi,psi1,dover,ovr,ovr1,workc,&
                 agmat,agmat1,time,dt)

      implicit none

      integer(long)                                    :: m,s,zeig1,zeig2,&
                                                          swapzeig,cs
      real(dop), intent(in)                            :: time,dt
      real(dop), dimension(maxdim,maxsta)              :: normpp
      complex(dop)                                     :: zdum
      complex(dop), dimension(1)                       :: cdum,cdum1
      complex(dop), intent(out)                        :: ovl1,ovl2,ovl
      complex(dop), dimension(dgldim1), intent(in)     :: psi1
      complex(dop), dimension(dgldim), intent(in)      :: psi
      complex(dop), dimension(doverdim), intent(out)   :: dover
      complex(dop), dimension(gdoverdim)               :: gdover
      complex(dop), dimension(dmatdim), intent(out)    :: ovr
      complex(dop), dimension(dmatdim1), intent(out)   :: ovr1
      complex(dop), dimension(gsmatdim)                :: govr
      complex(dop), dimension(gsmatdim1)               :: govr1
      complex(dop), dimension(workcdim), intent(out)   :: workc
      complex(dop), dimension(agmatdim), intent(in)    :: agmat
      complex(dop), dimension(agmatdim1), intent(in)   :: agmat1
      real*8 deltaphi(2*maxdim)

!-----------------------------------------------------------------------
!     calculate density matrices for each degree of freedom defined as 
!     overlap of the single-particle functions 
!     rho^m_ij=<phi_i,2|phi_j,1>
!-----------------------------------------------------------------------
      do s=1,nstate
         do m=1,nmode
            if (gwpm(m)) then
               call diagonal_moments(m,s,s,&
                    nspfdof(m),cgwpdim(m,s,2),cgwpdim(m,s,2),&
                    agmat1(czaGmat(m,s,2)),agmat1(czaGmat(m,s,2)),&
                    agmat1(czxivec(m,s,2)),agmat1(czxivec(m,s,2)),&
                    agmat1(czphase(m,s,2)),agmat1(czphase(m,s,2)),& 
                    govr1(cgsmat(m,s,2)),cdum,cdum1,0,0)
               if (psitype .eq. 0 .or. psitype .eq. 5) then
                  call simtran1bz(govr1(cgsmat(m,s,2)),psi1(czdgwp(m,s,2)), &
                       ovr1(cdmat(m,s,2)),cgwpdim(m,s,2),cdim(m,s,2))
               else
                  call cpqxz(govr1(cgsmat(m,s,2)),ovr1(cdmat(m,s,2)), &
                       cgwpdim(m,s,2))
               endif
               
               call diagonal_moments(m,s,s,&
                    nspfdof(m),cgwpdim(m,s,1),cgwpdim(m,s,1),&
                    agmat(czaGmat(m,s,1)),agmat(czaGmat(m,s,1)),&
                    agmat(czxivec(m,s,1)),agmat(czxivec(m,s,1)),&
                    agmat(czphase(m,s,1)),agmat(czphase(m,s,1)),& 
                    govr(cgsmat(m,s,1)),cdum,cdum1,0,0)
               if (psitype .eq. 0 .or. psitype .eq. 5) then
                  call simtran1bz(govr(cgsmat(m,s,1)),psi(czdgwp(m,s,1)), &
                       ovr(cdmat(m,s,1)),cgwpdim(m,s,1),cdim(m,s,1))
               else
                  call cpqxz(govr(cgsmat(m,s,1)),ovr(cdmat(m,s,1)), &
                       cgwpdim(m,s,1))
               endif
               
               call diagonal_moments(m,s,s,&
                    nspfdof(m),cgwpdim(m,s,1),cgwpdim(m,s,2),&
                    agmat(czaGmat(m,s,1)),agmat1(czaGmat(m,s,2)),&
                    agmat(czxivec(m,s,1)),agmat1(czxivec(m,s,2)),&
                    agmat(czphase(m,s,1)),agmat1(czphase(m,s,2)),& 
                    gdover(gdovmat(m,s)),cdum,cdum1,2,0)
               if (psitype .eq. 0 .or. psitype .eq. 5) then
                  call simtran2bz(psi(czdgwp(m,s,1)),gdover(gdovmat(m,s)), &
                       psi(czdgwp(m,s,2)),dover(dovmat(m,s)),&
                       cgwpdim(m,s,2),cgwpdim(m,s,1),cdim(m,s,2),cdim(m,s,1))
               else
                  call cpmxz(gdover(gdovmat(m,s)),dover(dovmat(m,s)), &
                       cgwpdim(m,s,1),cgwpdim(m,s,2))
               endif

            else
               call mmaxzzh(psi1(czetf(m,s,2)),psi1(czetf(m,s,2)),&
                    ovr1(cdmat(m,s,2)),subdim(m),cdim(m,s,2))
               call mmaxzzh(psi(czetf(m,s,1)),psi(czetf(m,s,1)),&
                    ovr(cdmat(m,s,1)),subdim(m),cdim(m,s,1))
               call mmaxzz(psi(czetf(m,s,1)),psi1(czetf(m,s,2)),&
                    dover(dovmat(m,s)),subdim(m),cdim(m,s,1),&
                    cdim(m,s,2))
               if(lpp) call normvxz(dover(dovmat(m,s)),normpp(m,s),&
                            cdim(m,s,1)*cdim(m,s,2))

            endif
         enddo
      enddo

      if(lpp) then
         if (dt .lt. 0.5d0) then
            write(idat,'(1x,f11.5,20f10.6)') time,&
              ((normpp(m,s)**2,s=1,nstate),m=1,nmode)
         else
            write(idat,'(1x,f11.2,20f10.6)') time,&
              ((normpp(m,s)**2,s=1,nstate),m=1,nmode)
         endif
      endif

      if (lrunml) then

         if (lcc) then
            cs = 1
         else
            cs = 0
         endif

         dmatdim = cdmatdim(1)
         dmat(:,:) = cdmat(:,:,1)
         call overlap_1(psi,psi,bratree,ovl,ovr)
         dmatdim = cdmatdim(2)
         dmat(:,:) = cdmat(:,:,2)
         call overlap_1(psi1,psi1,kettree,ovl1,ovr1)
         dmatdim = doverdim
         dmat(:,:) = dovmat(:,:)
         if (pernode) then
            call overlap_3(psi,psi1,bratree,kettree,ovl2,deltaphi,dover,cs)
         else
            call overlap_2(psi,psi1,bratree,kettree,ovl2,dover,cs)
         endif
! restore
         dmatdim = cdmatdim(1)
         dmat(:,:) = cdmat(:,:,1)

      else
!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J|Phi_L>A_L within each state for psi1
!-----------------------------------------------------------------------
         call getcpsi(2)
         ovl1=(0.0_dop,0.0_dop)
         do s=1,nstate
            zeig1=1
            zeig2=block(s)+1
            call cpvxz(psi1(zpsi(s)),workc(zeig2),block(s))

            do m=1,nmode
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig
               call qtxxzz(ovr1(dmat(m,s)),workc(zeig1),workc(zeig2),&
                           vdim(m,s),dim(m,s),ndim(m,s))
            enddo

!-----------------------------------------------------------------------
! calculate Sum_J A_J^* Sum_L <Phi_J|Phi_L>A_L within each state (psi1)
!-----------------------------------------------------------------------
            call vvaxzz(psi1(zpsi(s)),workc(zeig2),zdum,block(s))
            ovl1=ovl1+zdum
         enddo

!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J|Phi_L>A_L within each state (for psi2)
!-----------------------------------------------------------------------
         call getcpsi(1)
         ovl=(0.0_dop,0.0_dop)
         do s=1,nstate
            zeig1=1
            zeig2=block(s)+1
            call cpvxz(psi(zpsi(s)),workc(zeig2),block(s))
            do m=1,nmode
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig
               call qtxxzz(ovr(dmat(m,s)),workc(zeig1),workc(zeig2),&
                           vdim(m,s),dim(m,s),ndim(m,s))
            enddo

!-----------------------------------------------------------------------
! calculate Sum_J A_J^* Sum_L <Phi_J|Phi_L>A_L within each state (psi2)
!-----------------------------------------------------------------------
            call vvaxzz(psi(zpsi(s)),workc(zeig2),zdum,block(s))
            ovl=ovl+zdum
         enddo
         
!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J,2|Phi_L,1>A_L,1 within each state
!-----------------------------------------------------------------------
         call getcpsi(1)
         ovl2=(0.0_dop,0.0_dop)
         do s=1,nstate
            zeig1=1
            zeig2=maxblock+1
            call cpvxz(psi1(czpsi(s,2)),workc(zeig2),cblock(s,2))

            do m=1,nmode
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig
               call mtxxzz(dover(dovmat(m,s)),workc(zeig1),&
                       workc(zeig2),vdim(m,s),cdim(m,s,2),cndim(m,s,2),&
                       dim(m,s))
            enddo

!-----------------------------------------------------------------------
! calculate Sum_J A_J,2^* Sum_L <Phi_J,1|Phi_L,2>A_L,2 within each state
!-----------------------------------------------------------------------
            call vvaxzz(psi(zpsi(s)),workc(zeig2),zdum,block(s))
            ovl2=ovl2+zdum
         enddo

      endif


      return
      end subroutine overlap0

!#######################################################################
!
! OVERLAP0s
!
! Calculates the overlap between two wavefunctions in MCTDH form
! Special routine which performs the overlap <psi(s=s2)|psi1(s=s1).
! The numbers of the electronic states, s2 and s2, are to be given
! on the command line as option "-s <s1> <s2>".
!
!  ovl2 = <psi|psi1>,  ovl = <psi|psi>,   ovl1 = <psi1|psi1>
!
!#######################################################################

      subroutine overlap0s(ovl,ovl1,ovl2,psi,psi1,dover,ovr,&
                            ovr1,workc,s2,s1)

      implicit none

      integer(long)                                  :: m,zeig1,zeig2,swapzeig
      integer(long), intent(in)                      :: s1,s2
      complex(dop)                                   :: zdum
      complex(dop), intent(out)                      :: ovl1,ovl2,ovl
      complex(dop), dimension(dgldim1), intent(in)   :: psi1
      complex(dop), dimension(dgldim), intent(in)    :: psi
      complex(dop), dimension(doverdim), intent(out) :: dover
      complex(dop), dimension(dmatdim), intent(out)  :: ovr
      complex(dop), dimension(dmatdim1), intent(out) :: ovr1
      complex(dop), dimension(workcdim), intent(out) :: workc

!-----------------------------------------------------------------------
!     calculate density matrices for each degree of freedom defined as 
!     overlap of the single-particle functions rho^m_ij=<phi_i,2|phi_j
!     ,1>
!-----------------------------------------------------------------------
      do m=1,nmode
         call mmaxzzh(psi1(czetf(m,s1,2)),psi1(czetf(m,s1,2)),&
              ovr1(cdmat(m,s1,2)),subdim(m),cdim(m,s1,2))
         call mmaxzzh(psi(czetf(m,s2,1)),psi(czetf(m,s2,1)),&
              ovr(cdmat(m,s2,1)),subdim(m),cdim(m,s2,1))
         call mmaxzz(psi(czetf(m,s2,1)),psi1(czetf(m,s1,2)),&
              dover(dovmat(m,1)),subdim(m),cdim(m,s2,1),cdim(m,s1,2))
      enddo

!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J|Phi_L>A_L within each state for psi1,s1
!-----------------------------------------------------------------------
      call getcpsi(2)
      ovl1=(0.0_dop,0.0_dop)
      zeig1=1
      zeig2=block(s1)+1
      call cpvxz(psi1(zpsi(s1)),workc(zeig2),block(s1))
      
      do m=1,nmode
         swapzeig=zeig1
         zeig1=zeig2
         zeig2=swapzeig
         call qtxxzz(ovr1(dmat(m,s1)),workc(zeig1),workc(zeig2),&
              vdim(m,s1),dim(m,s1),ndim(m,s1))
      enddo

!-----------------------------------------------------------------------
! calculate Sum_J A_J^* Sum_L <Phi_J|Phi_L>A_L within each state (psi1,s1)
!-----------------------------------------------------------------------
      call vvaxzz(psi1(zpsi(s1)),workc(zeig2),zdum,block(s1))
      ovl1=ovl1+zdum

!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J|Phi_L>A_L within each state (for psi,s2)
!-----------------------------------------------------------------------
      call getcpsi(1)
      ovl=(0.0_dop,0.0_dop)
      zeig1=1
      zeig2=block(s2)+1
      call cpvxz(psi(zpsi(s2)),workc(zeig2),block(s2))

      do m=1,nmode
         swapzeig=zeig1
         zeig1=zeig2
         zeig2=swapzeig
         call qtxxzz(ovr(dmat(m,s2)),workc(zeig1),workc(zeig2),&
              vdim(m,s2),dim(m,s2),ndim(m,s2))
      enddo
      
!-----------------------------------------------------------------------
! calculate Sum_J A_J^* Sum_L <Phi_J|Phi_L>A_L within each state (psi,s2)
!-----------------------------------------------------------------------
      call vvaxzz(psi(zpsi(s2)),workc(zeig2),zdum,block(s2))
      ovl=ovl+zdum
         
!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J,2|Phi_L,1>A_L,1 within each state
!-----------------------------------------------------------------------
      call getcpsi(1)
      ovl2=(0.0_dop,0.0_dop)
      zeig1=1
      zeig2=maxblock+1
      call cpvxz(psi1(czpsi(s1,2)),workc(zeig2),cblock(s1,2))
      
      do m=1,nmode
         swapzeig=zeig1
         zeig1=zeig2
         zeig2=swapzeig
         call mtxxzz(dover(dovmat(m,1)),workc(zeig1),&
              workc(zeig2),vdim(m,s2),cdim(m,s1,2),cndim(m,s1,2),&
              dim(m,s2))
      enddo
      
!-----------------------------------------------------------------------
! calculate Sum_J A_J,2^* Sum_L <Phi_J,1|Phi_L,2>A_L,2 within each state
!-----------------------------------------------------------------------
      call vvaxzz(psi(zpsi(s2)),workc(zeig2),zdum,block(s2))
      ovl2=ovl2+zdum

      return
      end subroutine overlap0s

!#######################################################################
!
! D1OVERLAP0
!
! Calculates the overlap between type I density matrices.
!
!#######################################################################

      subroutine d1overlap0(ovl,ovl1,ovl2,psi,psi1,dover,ovr,ovr1,workc)

      implicit none

      integer(long)                                  :: s,t,m,zeig1,zeig2,swapzeig
      complex(dop)                                   :: zdum
      complex(dop), intent(out)                      :: ovl1,ovl2,ovl
      complex(dop), dimension(dgldim1), intent(in)   :: psi1
      complex(dop), dimension(dgldim), intent(in)    :: psi
      complex(dop), dimension(doverdim), intent(out) :: dover
      complex(dop), dimension(dmatdim), intent(out)  :: ovr
      complex(dop), dimension(dmatdim1), intent(out) :: ovr1
      complex(dop), dimension(workcdim), intent(out) :: workc


!-----------------------------------------------------------------------
!     calculate density matrices for each degree of freedom defined as
!     overlap of the single-particle functions rho^m_ij=<phi_i,2|phi_j
!     ,1>
!-----------------------------------------------------------------------
      do s=1,nstate
         do t=1,nstate
            do m=1,nmode
               call mmaxzzh(psi1(cdzetf(m,s,t,2)),psi1(cdzetf(m,s,t,2)),&
                    ovr1(cddmat(m,s,t,2)),subdim(m),cddim(m,s,t,2))
               call mmaxzzh(psi(cdzetf(m,s,t,1)),psi(cdzetf(m,s,t,1)),&
                    ovr(cddmat(m,s,t,1)),subdim(m),cddim(m,s,t,1))
               call mmaxzz(psi(cdzetf(m,s,t,1)),psi1(cdzetf(m,s,t,2)),&
                    dover(ddovmat(m,s,t)),subdim(m),cddim(m,s,t,1),&
                    cddim(m,s,t,2))
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J|Phi_L>A_L within each state for psi1
!-----------------------------------------------------------------------
      call getcpsi(2)
      ovl1=(0.0_dop,0.0_dop)
      do s=1,nstate
         do t=1,nstate
            zeig1=1
            zeig2=dblock(s,t)+1
            call cpvxz(psi1(dzpsi(s,t)),workc(zeig2),dblock(s,t))

            do m=1,nmode
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig
               call qtxxzz(ovr1(ddmat(m,s,t)),workc(zeig1),workc(zeig2),&
                    vddim(m,s,t),ddim(m,s,t),nddim(m,s,t))
            enddo

!-----------------------------------------------------------------------
! calculate Sum_J A_J^* Sum_L <Phi_J|Phi_L>A_L within each state (psi1)
!-----------------------------------------------------------------------
            call vvaxzz(psi1(dzpsi(s,t)),workc(zeig2),zdum,dblock(s,t))
            ovl1=ovl1+zdum
         enddo
      enddo

!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J|Phi_L>A_L within each state (for psi2)
!-----------------------------------------------------------------------
      call getcpsi(1)
      ovl=(0.0_dop,0.0_dop)
      do s=1,nstate
         do t=1,nstate
            zeig1=1
            zeig2=dblock(s,t)+1
            call cpvxz(psi(dzpsi(s,t)),workc(zeig2),dblock(s,t))
            do m=1,nmode
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig
               call qtxxzz(ovr(ddmat(m,s,t)),workc(zeig1),workc(zeig2),&
                    vddim(m,s,t),ddim(m,s,t),nddim(m,s,t))
            enddo

!-----------------------------------------------------------------------
! calculate Sum_J A_J^* Sum_L <Phi_J|Phi_L>A_L within each state (psi2)
!-----------------------------------------------------------------------
            call vvaxzz(psi(dzpsi(s,t)),workc(zeig2),zdum,dblock(s,t))
            ovl=ovl+zdum
         enddo
      enddo

!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J,2|Phi_L,1>A_L,1 within each state
!-----------------------------------------------------------------------
      call getcpsi(1)
      ovl2=(0.0_dop,0.0_dop)
      do s=1,nstate
         do t=1,nstate
            zeig1=1
            zeig2=maxblock+1
            call cpvxz(psi1(cdzpsi(s,t,2)),workc(zeig2),cdblock(s,t,2))

            do m=1,nmode
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig
               call mtxxzz(dover(ddovmat(m,s,t)),workc(zeig1),&
                    workc(zeig2),vddim(m,s,t),cddim(m,s,t,2),&
                    cnddim(m,s,t,2),ddim(m,s,t))
            enddo

!-----------------------------------------------------------------------
! calculate Sum_J A_J,2^* Sum_L <Phi_J,1|Phi_L,2>A_L,2 within each state
!-----------------------------------------------------------------------
            call vvaxzz(psi(dzpsi(s,t)),workc(zeig2),zdum,dblock(s,t))
            ovl2=ovl2+zdum
         enddo
      enddo

      return
      end subroutine d1overlap0


!***********************************************************************
!                               D2OVERLAP0
!
!     Computes tr( rho_1^\dagger rho_1 ) 
!
!***********************************************************************

!-----------------------------------------------------------------------
!
!   k: flag for the used dataset
!
!-----------------------------------------------------------------------
      subroutine d2overlap0(ovl,psi,ovr,workc,k,dimdmat)

      implicit none

      integer(long)                                             :: hzw,m,s,t
      integer(long), intent(in)                                 :: k,dimdmat
      integer(long), dimension(2)                               :: zw
      complex(dop)                                              :: x
      complex(dop), intent(out)                                 :: ovl
      complex(dop), dimension(dgldim1), intent(in)              :: psi
      complex(dop), dimension(dimdmat), intent(out)             :: ovr
      complex(dop), dimension(maxblock*maxblock,2), intent(out) :: workc



      do s = 1,nstate
         do m=1,nmode
            call mmaxzz(psi(czetf(m,s,k)),psi(czetf(m,s,k)),&
                 ovr(cdmat(m,s,k)),subdim(m),cdim(m,s,k),cdim(m,s,k))
         enddo
      enddo

      ovl  = (0.0_dop, 0.0_dop)
      do s = 1,nstate
         do t = 1, nstate

            zw(1) = 1
            zw(2) = 2
            call cpvxz(psi(cdzpsi(s,t,k)),workc(1,zw(1)),&
                 cblock(s,k)*cblock(t,k))

            do m=1,nmode
               call mtxxzz(ovr(cdmat(m,s,k)),workc(1,zw(1)),&
                    workc(1,zw(2)),cvdim(m,s,k),cdim(m,s,k),&
                    cndim(m,s,k)*cblock(t,k),cdim(m,s,k))
               hzw = zw(1)
               zw(1) = zw(2)
               zw(2) = hzw
            enddo

            do m=1,nmode
               call mttxzz(ovr(cdmat(m,t,k)),workc(1,zw(1)),&
                    workc(1,zw(2)),cblock(s,k)*cvdim(m,t,k),&
                    cdim(m,t,k),cndim(m,t,k),cdim(m,t,k))
               hzw = zw(1)
               zw(1) = zw(2)
               zw(2) = hzw
            enddo

            call vvaxzz(psi(cdzpsi(s,t,k)),workc(1,zw(1)),x,&
                 cblock(s,k)*cblock(t,k))
            ovl = ovl + x
         enddo
      enddo


      return
      end subroutine d2overlap0

!***********************************************************************
!                               D2OVERLAP2
!
!     Computes tr( rho_1^dagger  rho ) 
!
!***********************************************************************

      subroutine d2overlap2(ovl2,psi1,psi,dover,workc)

      implicit none

      integer(long)                                             :: hzw,m,s,t
      integer(long), dimension(2)                               :: zw
      complex(dop)                                              :: x
      complex(dop), intent(out)                                 :: ovl2
      complex(dop), dimension(dgldim1), intent(in)              :: psi1
      complex(dop), dimension(dgldim), intent(in)               :: psi
      complex(dop), dimension(doverdim,2), intent(out)          :: dover
      complex(dop), dimension(maxblock*maxblock,2), intent(out) :: workc
      

      do s = 1,nstate
         do m=1,nmode
            call mmaxzz(psi1(czetf(m,s,2)),psi(zetf(m,s)),&
                 dover(dovmat(m,s),1),subdim(m),cdim(m,s,2),dim(m,s))
            call mmaxzz(psi(zetf(m,s)),psi1(czetf(m,s,2)),&
                 dover(dovmat(m,s),2),subdim(m),dim(m,s),cdim(m,s,2))
         enddo
      enddo

      ovl2  = (0.0_dop, 0.0_dop)
      do s = 1,nstate
         do t = 1, nstate

            zw(1) = 1
            zw(2) = 2
            call cpvxz(psi(dzpsi(s,t)),workc(1,zw(1)),block(s)*block(t))

            do m=1,nmode
               call mtxxzz(dover(dovmat(m,s),1),workc(1,zw(1)),&
                    workc(1,zw(2)),cvdim(m,s,2),dim(m,s),&
                    ndim(m,s)*block(t),cdim(m,s,2))
               hzw = zw(1)
               zw(1) = zw(2)
               zw(2) = hzw
            enddo

            do m=1,nmode
               call mttxzz(dover(dovmat(m,t),2),workc(1,zw(1)),&
                    workc(1,zw(2)),cblock(s,2)*cvdim(m,t,2),dim(m,t),&
                    ndim(m,t),cdim(m,t,2))
               hzw = zw(1)
               zw(1) = zw(2)
               zw(2) = hzw
            enddo

            call vvaxzz(psi1(cdzpsi(s,t,2)),workc(1,zw(1)),x,&
                 cblock(s,2)*cblock(t,2))
            ovl2 = ovl2 + x
         enddo
      enddo


      return
      end subroutine d2overlap2



!#######################################################################
!
! DOOVERLAP0
!
! Calculates the overlap between type I and type II density matrices.
!
!
!  04/2002 CC
!
!#######################################################################
!
!     rho1 : type 1 =  A_M Sigma_M
!     rho2 : type 2 =  B_IJ |PHI_I> <PHI_J|
!#######################################################################

      subroutine dooverlap0(ovl1,ovl2,ovl12,rho1,rho2,dover,ovr,ovr1,&
           workc)

      implicit none

      integer(long)       :: s,t,m,d1,d2,zeig1,zeig2,swapzeig,&
                             dim1,dim2,dim3,dim4

      complex(dop)                                   :: zdum
      complex(dop), intent(out)                      :: ovl1,ovl2,ovl12
      complex(dop), dimension(dgldim1), intent(in)   :: rho2
      complex(dop), dimension(dgldim), intent(in)    :: rho1
      complex(dop), dimension(doverdim), intent(out) :: dover
      complex(dop), dimension(dmatdim), intent(out)  :: ovr
      complex(dop), dimension(dmatdim1), intent(out) :: ovr1
      complex(dop), dimension(workcdim), intent(out) :: workc
   

!-----------------------------------------------------------------------
!     calculate density matrices for each degree of freedom defined as
!     overlap of the single-particle functions rho^m_ij=<phi_i,2|phi_j
!     ,1>
!-----------------------------------------------------------------------
      do s=1,nstate
         do t=1,nstate
            do m=1,nmode
               call mmaxzzh(rho1(cdzetf(m,s,t,1)),rho1(cdzetf(m,s,t,1)),&
                    ovr(cddmat(m,s,t,1)),subdim(m),cddim(m,s,t,1))
            enddo
         enddo
         do m=1,nmode
            call mmaxzz(rho2(czetf(m,s,2)),rho2(czetf(m,s,2)),&
                 ovr1(cdmat(m,s,2)),&
                 supgdim(m),cdim(m,s,2),cdim(m,s,2))
         enddo
      enddo
    

!-------------------------------------------------------------------
! --- construct from rho2 (type II) a density operator of type I 
!-------------------------------------------------------------------

      
!     the spdo's
!     construct from | PHI >< PHI | the sigma matrices
      zeig1=0
      do s=1,nstate
         do t=1,nstate
            do m=1,nmode
               zeig1=0
               do d1=1,cdim(m,s,2)
                  do d2=1,cdim(m,t,2)
                     call vvxazz(rho2(czetf(m,s,2)+(d1-1)*supgdim(m)),&
                          rho2(czetf(m,t,2)+(d2-1)*supgdim(m)),&
                          workc(zeig1+1),supgdim(m))

!                    note: subdim(m) = supgdim(m) * supgdim(m)
                     zeig1=zeig1 + supgdim(m)*supgdim(m)
                  enddo
               enddo

               call mmaxzz(workc(1),rho1(cdzetf(m,s,t,1)),&
                    dover(ddovmat(m,s,t)),&
                    subdim(m),cdim(m,s,2)*cdim(m,t,2),cddim(m,s,t,1))

            enddo
         enddo
      enddo


!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J|Phi_L>A_L within each state for rho1 (type I)
!-----------------------------------------------------------------------
      call getcpsi(1)
      ovl1=(0.0_dop,0.0_dop)
      do s=1,nstate
         do t=1,nstate
            zeig1=1
            zeig2=dblock(s,t)+1
            call cpvxz(rho1(dzpsi(s,t)),workc(zeig2),dblock(s,t))

            do m=1,nmode
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig
               call qtxxzz(ovr(ddmat(m,s,t)),workc(zeig1),workc(zeig2),&
                    vddim(m,s,t),ddim(m,s,t),nddim(m,s,t))
            enddo

!-----------------------------------------------------------------------
! calculate Sum_J A_J^* Sum_L <Phi_J|Phi_L>A_L within each state (rho1)
!-----------------------------------------------------------------------
            call vvaxzz(rho1(dzpsi(s,t)),workc(zeig2),zdum,dblock(s,t))
            ovl1=ovl1+zdum
         enddo
      enddo
      


!-------------------------------------------------------------------------
! calculate Sum_L <Phi_J|Phi_L>A_L within each state (for rho2) ( type II)
!-------------------------------------------------------------------------
      call getcpsi(2)
      ovl2=(0.0_dop,0.0_dop)
      do s=1,nstate
         do t=1,nstate
            zeig1=1
            zeig2=cblock(s,2)*cblock(t,2) + 1
            call cpvxz(rho2(cdzpsi(s,t,2)),workc(zeig1),&
                 cblock(s,2)*cblock(t,2))

            do m=1,nmode
               call mtxxzz(ovr1(cdmat(m,s,2)),workc(zeig1),&
                    workc(zeig2),cvdim(m,s,2),cdim(m,s,2),&
                    cndim(m,s,2)*cblock(t,2),cdim(m,s,2))

               swapzeig = zeig1
               zeig1 = zeig2
               zeig2 = swapzeig
            enddo
            do m=1,nmode

               call mttxzz(ovr1(cdmat(m,t,2)),workc(zeig1),&
                    workc(zeig2),cblock(s,2)*cvdim(m,t,2),&
                    cdim(m,t,2),cndim(m,t,2),cdim(m,t,2))

               swapzeig = zeig1
               zeig1 = zeig2
               zeig2 = swapzeig
            enddo

            call vvaxzz(rho2(cdzpsi(s,t,2)),workc(zeig1),zdum,&
                 cblock(s,2)*cblock(t,2))
            ovl2 = ovl2 + zdum
         enddo
      enddo
      


!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J,2|Phi_L,1>A_L,1 within each state
!-----------------------------------------------------------------------
      call getcpsi(1)
      ovl12=(0.0_dop,0.0_dop)
      do s=1,nstate
         do t=1,nstate
            zeig1=1
            zeig2=max(cdblock(s,t,1),cblock(s,2)*cblock(t,2)) + 1
            call cpvxz(rho1(cdzpsi(s,t,1)),workc(zeig2),&
                 cdblock(s,t,1))

            dim1=cvddim(1,s,t,1) ! = 1 
            dim2=cddim(1,s,t,1)
            dim3=cnddim(1,s,t,1)
            dim4=cdim(1,s,2)*cdim(1,t,2)

            call swapsxxi(zeig1,zeig2)
            call mtxxzz(dover(ddovmat(1,s,t)),workc(zeig1),&
                 workc(zeig2),dim1,dim2,dim3,dim4)

            do m=2,nmode
               call swapsxxi(zeig1,zeig2)

               dim1 = dim1 * dim4
               dim2 = cddim(m,s,t,1) 
               dim3 = cnddim(m,s,t,1)
               dim4 = cdim(m,s,2)*cdim(m,t,2)

               call mtxxzz(dover(ddovmat(m,s,t)),workc(zeig1),&
                    workc(zeig2),dim1,dim2,dim3,dim4)
            enddo
               
! !!!       note: the tensor is now in a form like:
! !!!             A_(i1j1 i2j2 ... ifjf)



!-----------------------------------------------------------------------
! calculate Sum_J A_J,2^* Sum_L <Phi_J,1|Phi_L,2>A_L,2 within each state
!-----------------------------------------------------------------------

!      ---  transform the tensor in the "standard form":
!           A_(i1 i2 ... if j1 j2 ... jf)
            dim1 = cdim(1,s,2)   
            dim2 = cdim(1,t,2)
            dim3 = cdim(2,s,2)
            dim4 = (cblock(s,2)*cblock(t,2))/(dim1*dim2*dim3)
            call swapsxxi(zeig1,zeig2) 
            call sorttens(workc(zeig1),workc(zeig2),dim1,dim2,dim3,dim4)
            
            do m=3,nmode
               dim1 = dim1*dim3
               dim2 = dim2*cdim(m,t,2)
               dim3 = cdim(m,s,2)
               dim4 = dim4/(dim1*dim2*dim3)
               
               call swapsxxi(zeig1,zeig2) 
               call sorttens(workc(zeig1),workc(zeig2),&
                    dim1,dim2,dim3,dim4)
            enddo               ! of m

            call vvtxzz(rho2(cdzpsi(s,t,2)),workc(zeig2),&
                 zdum,cblock(s,2)*cblock(t,2))

            ovl12=ovl12+zdum
         enddo                  ! of t
      enddo                     ! of s


      return
      end subroutine dooverlap0                      ! of dooverlap0





!#######################################################################
!
! wfd2overlap0
!
! Calculates the overlap between wavefunction and type II density 
! matrices.
!
!
!  08/2003 CC
!
!#######################################################################
!
!     psi : wf     =  A_M |PHI_M>
!     rho : type 2 =  B_IJ |PHI_I> <PHI_J|
!#######################################################################

      subroutine wfd2overlap0(ovl1,ovl2,ovl12,psi,rho,dover,ovr,ovr1,&
           workc)

      implicit none

      integer(long)                                    :: s,t,m,zeig1,zeig2,swapzeig
      complex(dop)                                     :: zdum
      complex(dop), intent(out)                        :: ovl1,ovl2,ovl12
      complex(dop), dimension(dgldim1), intent(in)     :: rho
      complex(dop), dimension(dgldim), intent(in)      :: psi
      complex(dop), dimension(doverdim,2), intent(out) :: dover
      complex(dop), dimension(dmatdim), intent(out)    :: ovr
      complex(dop), dimension(dmatdim1), intent(out)   :: ovr1
      complex(dop), dimension(workcdim), intent(out)   :: workc


!-----------------------------------------------------------------------
!     calculate density matrices for each degree of freedom defined as
!     overlap of the single-particle functions rho^m_ij=<phi_i,2|phi_j
!     ,1>
!-----------------------------------------------------------------------
      do s=1,nstate
         do m=1,nmode
            call mmaxzzh(psi(czetf(m,s,1)),psi(czetf(m,s,1)),&
                         ovr(cdmat(m,s,1)),subdim(m),cdim(m,s,1))
            call mmaxzz(rho(czetf(m,s,2)),rho(czetf(m,s,2)),&
                 ovr1(cdmat(m,s,2)),&
                 subdim(m),cdim(m,s,2),cdim(m,s,2))
         enddo
      enddo
      
 
!-----------------------------------------------------------------------
! calculate Sum_L <Phi_J|Phi_L>A_L within each state for psi
!-----------------------------------------------------------------------
      call getcpsi(1)
      ovl1=(0.0_dop,0.0_dop)
      do s=1,nstate
         zeig1=1
         zeig2=block(s)+1
         call cpvxz(psi(zpsi(s)),workc(zeig2),block(s))

         do m=1,nmode
            swapzeig=zeig1
            zeig1=zeig2
            zeig2=swapzeig
            call qtxxzz(ovr(dmat(m,s)),workc(zeig1),workc(zeig2),&
                        vdim(m,s),dim(m,s),ndim(m,s))
         enddo
!-----------------------------------------------------------------------
! calculate Sum_J A_J^* Sum_L <Phi_J|Phi_L>A_L within each state (psi1)
!-----------------------------------------------------------------------
         call vvaxzz(psi(zpsi(s)),workc(zeig2),zdum,block(s))
         ovl1=ovl1+zdum
      enddo    
      
! --- square the norm, to compare with densities     
      ovl1=ovl1**2 





!-------------------------------------------------------------------------
! calculate Sum_L <Phi_J|Phi_L>A_L within each state (for rho2) ( type II)
!-------------------------------------------------------------------------
      call getcpsi(2)
      ovl2=(0.0_dop,0.0_dop)
      do s=1,nstate
         do t=1,nstate
            zeig1=1
            zeig2=cblock(s,2)*cblock(t,2) + 1
            call cpvxz(rho(cdzpsi(s,t,2)),workc(zeig1),&
                 cblock(s,2)*cblock(t,2))

            do m=1,nmode
               call mtxxzz(ovr1(cdmat(m,s,2)),workc(zeig1),&
                    workc(zeig2),cvdim(m,s,2),cdim(m,s,2),&
                    cndim(m,s,2)*cblock(t,2),cdim(m,s,2))
               swapzeig = zeig1
               zeig1 = zeig2
               zeig2 = swapzeig
            enddo
            do m=1,nmode
               call mttxzz(ovr1(cdmat(m,t,2)),workc(zeig1),&
                    workc(zeig2),cblock(s,2)*cvdim(m,t,2),&
                    cdim(m,t,2),cndim(m,t,2),cdim(m,t,2))
               swapzeig = zeig1
               zeig1 = zeig2
               zeig2 = swapzeig
            enddo
            call vvaxzz(rho(cdzpsi(s,t,2)),workc(zeig1),zdum,&
                 cblock(s,2)*cblock(t,2))
            ovl2 = ovl2 + zdum
         enddo
      enddo
      
  
!-------------------------------------------------------------
!
!-------------------------------------------------------------
      do s = 1,nstate
         do m=1,nmode
            call mmaxzz(rho(czetf(m,s,2)),psi(czetf(m,s,1)),&
                 dover(dovmat(m,s),2),subdim(m),cdim(m,s,2),&
                 dim(m,s))
            call mmaxzz(psi(czetf(m,s,1)),rho(czetf(m,s,2)),&
                 dover(dovmat(m,s),1),subdim(m),dim(m,s),&
                 cdim(m,s,2))

         enddo
      enddo

      ovl12  = 0.0_dop
      do s = 1,nstate
         do t = 1, nstate
            zeig1 = 1
            zeig2 = block(s)*block(t)+1
     
            do m=1,nmode
               call akcopy_wf2d2(psi(dzpsi(s,t)),workc(zeig1),&
                         cvdim(m,s,1),cdim(m,s,1),cdim(m,s,1),&
                         cndim(m,s,1))
            enddo  
     
            do m=1,nmode
               call mtxxzz(dover(dovmat(m,s),1),workc(zeig1),&
                    workc(zeig2),cvdim(m,s,2),dim(m,s),&
                    ndim(m,s)*block(t),cdim(m,s,2))
               call swapsxxi(zeig1,zeig2)
            enddo

            do m=1,nmode
               call mttxzz(dover(dovmat(m,t),1),workc(zeig1),&
                    workc(zeig2),cblock(s,2)*cvdim(m,t,2),&
                    dim(m,t),ndim(m,t),cdim(m,t,2))
                call swapsxxi(zeig1,zeig2)
            enddo

            call vvaxzz(rho(cdzpsi(s,t,2)),workc(zeig1),zdum,&
                 cblock(s,2)*cblock(t,2))
            ovl12 = ovl12 + zdum
         enddo
      enddo
      
!      ovl12=sqrt(ovl1*ovl2)


      return
      end subroutine wfd2overlap0   ! of wfd2overlap0	




!#######################################################################
!
! RDOVER
!
! reads an existing overlap file
!
!#######################################################################

      subroutine rdover(unit,ierr)

      implicit none

      integer(long)              :: i
      integer(long), intent(in)  :: unit
      integer(long), intent(out) :: ierr
      logical(kind=4)            :: newr
      real(dop)                  :: time,error,hilbert,pherr,phcorrect,&
                                    nsq,n1sq,reovlp,imovlp
      character(len=80)          :: string

      newr=.false.
      read(unit,'(a3)',end=1000) string
      if( string(1:3) .eq. '#-#') newr=.true.

  9   read(unit,'(a1)',end=1000) string
      if( string(1:1) .eq. '#') goto 9 
      backspace unit

      write(6,'(80a)') ('-',i=1,80)
      write(6,'(a)') ' ***********  Data read from file ! ***********' 
      write(6,'(a12,6a11)')&
        '    Time[fs]','      Error','    Hilbert',&
         '      Phase','   Ph. corr','  ||bra||^2','  ||ket||^2'

      if(newr) goto 20

   10 continue
      read(unit,*,end=999) time,error,hilbert,pherr,phcorrect,nsq,n1sq
      write(6,'(f12.4,6f11.6)') time,&
          error,hilbert,pherr,phcorrect,nsq,n1sq
      go to 10

   20 continue
      read(unit,*,end=999) time,reovlp,imovlp,nsq,n1sq
      error=nsq+n1sq-2.0_dop*reovlp
      if (-error .gt. 2.0e-8_dop) then
         write(6,*) ' WARNING: Error is negative ! error =', error
         error=0.0_dop
      endif
      error=sqrt(abs(error))
      hilbert=sqrt(reovlp**2+imovlp**2)/sqrt(nsq*n1sq+1.0e-25_dop)
      hilbert=acos(min(hilbert,1.0_dop))
      pherr=atan2(imovlp,reovlp)
      phcorrect=nsq+n1sq-2.0_dop*sqrt(reovlp**2+imovlp**2)
      phcorrect=sqrt(abs(phcorrect))
 
      write(6,'(f12.4,6f11.6)') time,&
          error,hilbert,pherr,phcorrect,nsq,n1sq
      go to 20

  999 continue

      write(6,'(a12,6a11)')&
        '    Time[fs]','      Error','    Hilbert',&
         '      Phase','   Ph. corr','  ||bra||^2','  ||ket||^2'
      write(6,'(a)') ' ***********  Data read from file ! ***********' 
      write(6,'(80a)') ('-',i=1,80)

      return

! empty overlap file - ignore
 1000 continue
      ierr=1
      return

      end subroutine rdover

!-----------------------------------------------------------------------
!***********************************************************************
!
! OVRINPUT
!
! get input parameters and file names for calculation
!
!-----------------------------------------------------------------------
      subroutine ovrinput

      implicit none

      integer(long), external :: myiargc,slen
      integer(long)           :: iarg,ilbl,llaenge,ierr,i,laepp
      character(len=80)       :: buf
      character(len=c5)       :: filename,flabel
!-----------------------------------------------------------------------

      lignore = .false.
      lallov  = .false.
      lback   = .false.
      lcc     = .false.
      lpp     = .false.
      sbra    = 0
      sket    = 0
      filein1 = 'psi'
      filein  = '  '
      optlaenge=1
      options = '   '
      tend=9.d99
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      do i = 1, iarg
         call mygetarg(i,buf)
         ilbl  = index(buf,' ')
         ilbl = min(ilbl,100-optlaenge)
         if(ilbl.gt.0) options = options(1:optlaenge)//buf(1:ilbl)
         optlaenge = optlaenge + ilbl
      end do

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:3) .eq. '-f ' .or.  buf(1:3) .eq. '-i ') then
            i = i+1
            call mygetarg(i,buf)
            filein1 = buf
         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf .eq. '-nofs' ) then
            fs = 1.d0
         else if( buf(1:3) .eq. '-a ' ) then
            lallov=.true.
         else if( buf(1:3) .eq. '-b ' ) then
            lback=.true.
         else if( buf(1:3) .eq. '-c ' ) then
            lcc=.true.
         else if( buf(1:3) .eq. '-p ' ) then
            lpp=.true.
         else if( buf(1:3) .eq. '-s ' ) then 
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) sbra
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) sket
            if (ierr.ne.0  .or.  sbra.le.0 .or. sket.le.0) then
               write(6,'(2a/,a)')&
                 ' The  arguments following the option -s',&
                 ' must be a positive integers. Not : ', buf
               stop
            end if
         else if( buf(1:3) .eq. '-n ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:3) .eq. '-t ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) tend
            if (ierr .ne. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -t',&
                 ' must be a real number. Not : ', buf
               stop
            end if
         else if( buf(1:3) .eq. '-w ' ) then
            lovwr=.true.
         else if( buf(1:4) .eq. '-ig ' ) then
           lignore = .true.
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,a,/10x,a)')&
       'Purpose: Calculates the difference between two wavefunctions'&
       , 'or density operators. See review Eqs.(159-163) for a'&
       , '         definition of the output.'
            write(6,'(/1x,2a)') 'Usage  : overlap ',&
                  '[-f -i -o -a -b -c -t -n -nofs -w -ig -p -ver -h -?]  [Psi] .'
            write(6,'(/,a)') 'Options : '
            write(6,'(a,3(/,11x,a))') &
                     ' -f FILE : The wavefunction (density operator) '&
                    ,'is read from file FILE rather than from ./psi.'&
                    ,'If FILE is a directory containing psi it must end with a /'&
            ,'The string FILE may be a relative or a full path-name.'
            write(6,'(a)') ' -i FILE : Same as -f FILE.'
            write(6,'(2a,/,11x,a/,11x,a)') &
                     ' -o FILE : The overlaps are written to file ',&
                     'FILE.pl rather than to ./ovl_XXX',&
             'The string FILE may be a relative or a full path-name.',&
             'If FILE=no, i.e. "-o no", no ovl-file will be opened.'
            write(6,'(a,a)')&
                     ' -nofs   : No transformation to fs. Use when',&
                     ' "time-not-fs" was set in mctdh.'
            write(6,'(2a,/11x,a)') ' -a      : Compute the overlaps', &
                     ' according to their position on psi-file,',&
                     'i.e. times (tpsi) are ignored.'
            write(6,'(2a,/11x,a)') ' -b      : Compute the overlaps', &
                     ' <Psi(t_final-t)|Psi1(t)>,',&
                     'i.e. the bra-state runs backwards in time.'
            write(6,'(2a,/11x,a)') ' -c      : Compute the overlaps', &
                     ' <dconjg(Psi(t))|Psi1(t)>,',&
                 'i.e. the complex-symmetric scalar product is used.'
            write(6,'(2a,/11x,2a)') ' -p      : Compute in addition ',&
                 'the projections of the two sets of SPFs onto',&
                 'eachother, i.e. trace O^dagger O, where O is ',&
                 'the SPF-overlap matrix.'
            write(6, '(2a, /11x,2a)')' -s s s1 : Compute the overlaps',&
                     ' < Psi(t)_s|Psi1(t)_s1>,', 's and s1 denote ',&
            'the electronic states of bra- and ket-WF.'
            write(6,'(2a)') ' -t tend : Compute the overlap',&
           ' only up to t=tend.'
            write(6,'(2a)') ' -n step : Compute the overlap', &
           ' only every step-th output.'
            write(6,'(2a)') &
                     ' -w      : An existing overlap file is ',&
                     'overwritten.'
            write(6,'(a)') &
                     ' -ig     : Ignore "different bases" error-msg.'
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(a)') ' '
            write(6,'(2a,/,a)') ' The argument Psi (bra-states) is',&
                   ' the name of the file containing',&
                   ' the comparison wavefunction or density operator.'
            write(6,'(2a)') ' The ket-states are read from ./psi',&
                            ' or from the file given by the -f option.'
            write(6,'(2a)') ' If no argument is given, ',&
                 'the user is prompted for the missing argument.'
            write(6,'(2a/,a)') ' If one calculates the overlap between',&
                 ' density operators of different types,',&
                 ' [Psi] has to be the one of type II.'
            write(6,'(2a,/2a)') ' NB: The overlap is written to the',&
                               ' file ./ovl_XXX, the screen-output',&
                   ' shows the error=||Psi-Psi1||. See chapter 8.2',&
                   ' of the MCTDH review.'
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      
      if (iarg .lt. i) goto 200
      call mygetarg(i,filein)

 200  continue
      if (filein .eq. ' ') then
         write(6,*) 'Name of comparison file (bra-states)  :'
         read(5,'(a)') filein
         write(6,*)
      endif

      laein1=index(filein1,' ')-1
      if (filein1(laein1:laein1) .eq. '/') then 
         filein1 = filein1(1:laein1)//'psi'
         laein1 = laein1 + 3
      end if
      call abspath(filein1,laein1)
      laein=index(filein,' ')-1
      if (filein(laein:laein) .eq. '/') then 
         filein = filein(1:laein)//'psi'
         laein = laein + 3
      end if
      call abspath(filein,laein)

!-----------------------------------------------------------------------
! get stem of comparison name
! If file name is psi, use directory name.
! Else use file name
!-----------------------------------------------------------------------
      llaenge=laein
      ilbl=llaenge
   11 continue
      if (filein(ilbl:ilbl) .eq. '/') then
         continue
      else
        ilbl=ilbl-1
        go to 11
      endif

      if (filein(ilbl+1:laein) .eq. 'psi') then
         llaenge = ilbl
         ilbl=ilbl-1
   12    continue
         if (filein(ilbl:ilbl) .eq. '/') then
            continue
         else
           ilbl=ilbl-1
           go to 12
         endif
      endif
      flabel=filein(ilbl+1:llaenge)
      llaenge=index(flabel,' ' )-1

      if (fileout(1:2) .eq. 'no') then
         laeout = 0
         goto 203 
      else
         if (fileout .eq. ' ') then
            fileout = 'ovl_'//flabel(1:llaenge)
         endif
         laeout=index(fileout,'  ')-1
         call abspath(fileout,laeout)
      end if

!-----------------------------------------------------------------------
! check for existence of overlap file, and read if there
! if error while reading file, write over it
!-----------------------------------------------------------------------
      if (.not.lovwr) then
         filename=fileout(1:laeout)
         open(iaus,file=filename,form='formatted',&
                   status='old',err=203)
         ierr=0
         call rdover(iaus,ierr)
         if (ierr .eq. 0) then
            close (iaus)
            stop
         else
            rewind(iaus)
         endif
       endif
 203   continue

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      if (filein .eq. filein1) then
         print*
         print*,'ERROR: Cannot calculate overlap of psi with itself'
         print*
         stop
      endif

      write(6,'(80a)') ('*',i=1,80)
      filename=filein1(1:laein1)
      open(ipsi1,file=filename,form='unformatted',status='old',&
           err=1000)
      write(6,'(2a)') ' The input file (ket) is : ',filein1(1:laein1)

      filename=filein(1:laein)
      open(ipsi,file=filename,form='unformatted',status='old',&
           err=1000)
      write(6,'(2a)') ' The comp. file (bra) is : ',filein(1:laein)

      if(laeout .eq. 0) then
         write(6,'(2a)') ' No outputfile '
      else
         filename=fileout(1:laeout)
         open(iaus,file=filename,form='formatted',status='unknown',&
              err=1000)
         write(6,'(2a)') ' The outputfile is : ',fileout(1:laeout)
         if(lallov) write(6,'(2a)') ' The WFs are taken according to',&
                  ' their file-position. Times are ignored. (-a option)'
         if(lback) write(6,'(2a)') ' The Bra-states are read in',&
                                   ' inverse time order. (-b option)'
         if(lcc)   write(6,'(2a)') ' The Bra-states are complex',&
                                   ' conjugated. (-c option)'
         if(sbra.gt.0) write(6,'(a,i3,a,/a,i3,a)')&
               ' Electronic state',sbra,' for Bra-WF. (-s option)',&
               ' Electronic state',sket,' for Ket-WF. (-s option)'

         call fpstartup(filename)
         ilbl = slen(filename)-1
         write(iaus,'(2a)')    '#-# Overlap :  ',filename(14:ilbl)
         write(iaus,'(2a)')    '# Options: ',options(1:optlaenge)
         ilbl = max(1,laein-77)
         write(iaus,'(2a)')    '# Psi : ',filein(ilbl:laein)
         ilbl = max(1,laein1-77) 
         write(iaus,'(2a)')    '# Psi1: ',filein1(ilbl:laein1)
         write(iaus,'(a)')     '# '
         write(iaus,'(a12,8x,a,9x,a,11x,a,11x,a)') '#   Time[fs]',&
         'Re(<Psi|Psi1>)','Im(<Psi|Psi1>)','<Psi|Psi>','<Psi1|Psi1>' 
      end if

      if(lpp) then
         filename='projection'
         laepp=10
         call abspath(filename,laepp)
         open(idat,file=filename,form='formatted',status='unknown',&
              err=1000)
         write(6,'(2a)') ' The projection-file is : ',filename(1:laepp)
      endif

      return 
      
 1000 write(6,*)' Cannot open file : '
      write(6,*) filename
      stop
      
      end subroutine ovrinput
      
      
!#####################################################################
!
! This routine assigns an A-vector to another A-vector with different
! numbers of spfs. It reduces or expands the second A-vector to fit the
! original A-vector.
! The original A-vector is the A-vector of a wavefunction, while the new
! A-vector belongs to a density-operator of type II.
! Same as akcopy_wf_to_d2 of geninwf/getpsi.F
!
!  01/03 CC
!
!#####################################################################

      subroutine akcopy_wf2d2(psiold,psinew,vdim1,dim1,dim,ndim)

      implicit none

      integer(long)                                                         :: v,e,n,w,f,m,dim2
      integer(long), intent(in)                                             :: vdim1,dim1,dim,ndim
      complex(dop), dimension(vdim1,dim,ndim), intent(in)                   :: psiold
      complex(dop), dimension(vdim1,dim1,ndim,vdim1,dim1,ndim), intent(out) :: psinew

      dim2=min(dim,dim1)
      do n=1,ndim
         do m=1,ndim
            do e=1,dim2
               do f=1,dim2
                  do v=1,vdim1
                     do w=1,vdim1
                        psinew(v,e,n,w,f,m)=&
                             psiold(v,e,n)*dconjg(psiold(w,f,m))
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo

      if (dim1 .gt. dim) then
         do n=1,ndim
            do m=1,ndim
               do e=dim+1,dim1
                  do f=dim+1,dim1
                     do v=1,vdim1
                        do w=1,vdim1
                           psinew(v,e,n,w,f,m)=0.0_dop
                        enddo
                     enddo
                  enddo
               enddo
            enddo
         enddo
      endif

      return
      end subroutine akcopy_wf2d2                      ! of akcopy_wf2d2
      


      end program overlap

