!#######################################################################
!     lpltime   = xdata(t) v zdata(t,np,ns,np1)
!     lplener   =.false.
!     lplcootim =.false.
!     lpltimcoo =.false.
!     lplenetim =.false.
!     lplcoo    =.false.
!     lplcoo2   =.false.
!     lplcoo2tim=.false.
!
! A file on unit iaus is needed for interactive plotting (tmpplot.pl)
!#######################################################################

      module plotutils

use decimal, only: long,dop
use keyunitsmod
use logdat
use channels
use global
use aglobal
use dvrdatmod
use griddatmod
use plot
use psidef
use datenmod
use compdatmod
use compdat1mod
use psiutil, only: spfovl

use wftree
use mlanalysis
use multilayer
use dirdyn, only: xref,natm
use dd_db, only: rdrefdb

      implicit none
      private
      public :: plotdefs,getplgdim,initplot,timebnds,dotask,plotmenu,databnds,&
                enerbnds,inmodebnds,maport,maport1,find_nearest_gpoint, &
                calcd2d_1ml, calcd2d_2ml,mode_lab_corr
      contains

!#######################################################################
! PLOTDEFS
!
! sets up defaults for plotting
!
! Plotting is x v z for 1D and x +y v z for 2D
!#######################################################################

      subroutine plotdefs

      implicit none

!-----------------------------------------------------------------------
! initialise variables to direct plotting
!-----------------------------------------------------------------------
      lgnu      =.true.
      lpltime   =.false.
      lplener   =.false.
      lplcootim =.false.
      lpltimcoo =.false.
      lplenetim =.false.
      lplcoo    =.false.
      lplcoo2   =.false.
      lplcoo2tim=.false.
      lstate    =.false.
      ltslice   =.false.
      lstate    =.false.
      lshowpot  =.false.
      lshowsys  =.false.
      lshowd1d  =.false.

      lgrid   =.false.
      lkey    =.false.
      ltitle  =.true.
      lpoints =.false.
      lpointsonly = .false.
      lstick  =.false.
      lsmooth = .false.
      plotform= 0

      lmultipl = .false.
      multiname= 'Not selected '
      loverlay = .true.

      rcalcpar(:)=0.0
      acalcpar(:)=' '

      plttitle=' '

      lrecalc  = .true.
      lredo    = .false.
      lrebound = .true.
      lreplot  = .false.
      lreploto = .true.

      return
      end subroutine plotdefs

!#######################################################################
! INITPLOT
!
!
! initialises plotting options
!#######################################################################

      subroutine initplot(filename,plotdim)

      implicit none

      integer(long)                 :: a,m,f,ierr,mode(4),m1,m2,m3
      integer(long), intent(out)    :: plotdim
      character(len=c5), intent(in) :: filename

      lgnu   = .true.
!!      This may cause problems when running on Mac
!!      gnucmd = 'gnuplot -geometry +0+0 -persist '//trim(filename)
      gnucmd = 'gnuplot -persist '//trim(filename)
      pprcmd ='lpr '

!-----------------------------------------------------------------------
! find where continuum electronic state info is stored
!-----------------------------------------------------------------------
      if (lebcont) then
         fener=feb
      endif

!-----------------------------------------------------------------------
! ftime can be defined in operator but not used. If so, reset it.
!-----------------------------------------------------------------------
      if (ftime .gt. ndof) ftime = -1

!-----------------------------------------------------------------------
! get no. of "active" DOFs
!-----------------------------------------------------------------------
      nandof=ndof
      nanmode=nmode
      nactdof=0
      do m=1,nmode
         nactdof=nactdof+nspfdof(m)
      enddo
      if (meb .gt. 0) nactdof = nactdof-1

!-----------------------------------------------------------------------
! add time and energy degrees of freedom 
! if already defined (in operator) make sure they can be accessed. 
!-----------------------------------------------------------------------
      if (lplcootim .or. lpltime .or. lplcoo2tim .or. lplenetim &
          .or. lpltimcoo) then
         if (ftime .le. 0) then
            nandof=nandof+1
            ftime=nandof
            gdim(ftime)=1
            nanmode=nanmode+1
            m=nanmode
            nspfdof(m)=1
            spfdof(1,m)=ftime
            dofspf(ftime)=m
         endif
         modelabel(ftime)='Time'
      endif

      if (lplener .or. lplenetim) then
         if (fener .le. 0) then
            nandof=nandof+1
            fener=nandof
            gdim(fener)=1
            nanmode=nanmode+1
            m=nanmode
            nspfdof(m)=1
            spfdof(1,m)=fener
            dofspf(fener)=m
         endif
         modelabel(fener)='Energy'
      endif

      if (lplcoo2 .or. lplcootim .or. lplcoo2tim .or. lpltimcoo) then
         nandof = nandof + 1
         fcut = nandof
         gdim(fcut)=maxplgdim
         zort(fcut) = 1
         nanmode=nanmode+1
         m=nanmode
         nspfdof(m)=1
         spfdof(1,m)=fcut
         dofspf(fcut)=m
         modelabel(fcut)='Cut'
      endif
!
! set up axis information based on system and type of plot
!
      do a=1,3
         axis(a)=0
      enddo

      if (lplcootim .or. lpltimcoo .or. lplcoo  .or. lplcoo2 .or. &
            lplcoo2tim) then
         do a=1,4
            mode(a)=0
         enddo

         do a=1,min(3,nmode)
            mode(a)=a
         enddo
! if meb in list, move to end of list
         if ((meb .gt. 0) .and. (meb .lt. nmode)) then
            do a=meb,nmode-1
               mode(a)=mode(a+1)
            enddo
            mode(nmode) = meb
         endif

         m1=mode(1)
         m2=mode(2)
         m3=mode(3)
         if (nspfdof(m1) .ge. 3) then
            do a=1,3
               axis(a)=spfdof(a,m1)
            enddo
            plotdim=2
         else if (nspfdof(m1) .eq. 2) then
            do a=1,2
               axis(a)=spfdof(a,m1)
            enddo
            if (m2 .gt. 0) axis(3)=spfdof(1,m2)
            plotdim=2
         else if (nspfdof(m1) .eq. 1) then
            axis(1)=spfdof(1,m1)
            if (m2 .gt. 0) then
               if (nspfdof(m2) .ge. 2) then
                  do a=2,3
                     axis(a)=spfdof(a-1,m2)
                  enddo
               else
                  axis(2)=spfdof(1,m2)
                  if (m3 .gt. 0) then
                     axis(3)=spfdof(1,m3)
                  endif
               endif
               plotdim=2
            else
               plotdim=1
            endif
         endif

      endif

      if (lpltime) then
         axis(2)=max(axis(1),1)
         axis(1)=ftime
      else if (lplener) then
         axis(2)=max(axis(1),1)
         axis(1)=fener
      else if (lplcootim) then
         axis(2)=ftime
      else if (lpltimcoo) then
         axis(2)=max(axis(1),1)
         axis(1)=ftime
      else if (lplenetim) then
         axis(1)=fener
         axis(2)=ftime
      endif

!-----------------------------------------------------------------------
! for coo2tim plots, set time slice and plotform
!-----------------------------------------------------------------------
      if (lplcoo2tim) then
         ltslice=.true.
         plotform=1
      endif

!-----------------------------------------------------------------------
! default units
!-----------------------------------------------------------------------
      do f=1,ndof
         axunit(f)=1.0_dop
         axunitlab(f)='au'
      enddo
      if (ftime .ne. 0) then
         if (lplcootim .or. lpltime .or. lplcoo2tim .or. lpltimcoo) then
            axunit(ftime)=1.0_dop
            axunitlab(ftime)='fs'
         else if (lplener) then
            axunit(fener)=1.0_dop
            axunitlab(fener)='au'
         endif
      endif
      if (fener>0) then
         axunitlab(fener)='ev'
         call keyunits(axunit(fener),axunitlab(fener),ierr)
      endif
      if (fcut .gt. 0) then
         axunit(fcut)=1.0_dop
         axunitlab(fcut)='Structure'
      endif

!-----------------------------------------------------------------------
! initialise z-axis units - take from calling program for plot. Overlay
! units set to ev
!-----------------------------------------------------------------------
      zaxunit=1.0_dop
      call keyunits(zaxunit,zaxunitlab,ierr)
      if (ierr .gt. 0) then
         zaxunit=1.0_dop
         zaxunitlab='au'
      endif

!-----------------------------------------------------------------------
! defaults for contour plots
!-----------------------------------------------------------------------
      if (plotdim .eq. 1) then
         lcntr=.false.
      else if (plotdim .eq. 2) then
         lcntr=.true.
      else if (plotdim .eq. -1) then
         lcntr=.false.
      else
         routine='Initplot'
         message='Plotdim must be set first to plot dimensions (1 or 2)'
         call errormsg
      endif

      cctr(1)='linear'
      cctr(2)='logarithmic'

      nctr=21
      mctr=1
      nctro=21

!-----------------------------------------------------------------------
! defaults for 3D plots
!-----------------------------------------------------------------------
      lsrf=.false.
      lhid3d=.false.
      rview(1)=60.0
      rview(2)=30.0
      rview(3)=1.0
      rview(4)=1.0
      ticslevel=0.5
      autoz=.true.
      autozo=.true.
      lsrfo=.false.

!-----------------------------------------------------------------------
! defaults for plotting of states / data sets
!-----------------------------------------------------------------------
      plstate(1)=1
      nplstates=1

!-----------------------------------------------------------------------
! set initial zbounds
!-----------------------------------------------------------------------
      zbound(1)=-99999.0
      zbound(2)=99999.0
      zboundo(1)=-99999.0
      zboundo(2)=99999.0

      return
      end subroutine initplot

!#######################################################################
!
! DATABNDS
!
! finds bounds of plotting data
!
!
! zdata(dimx,dimy,nsets,nplots)  : input data
! dimx                          : no. of x data points
! dimx                          : no. of y data points
! nsets                         : no. of data sets for a plot
! nplots                        : no. of plots
! dim1, dim2, dim3, dim4        : physical dimensions of zdata
!
!#######################################################################

      subroutine databnds(zdata,dimx,dimy,nsets,nplots,dim1,dim2,dim3,&
                 dim4)

      implicit none

      integer(long)                                          :: g,g1,s,p
      integer(long), intent(in)                              :: dimx,dimy,nsets,nplots,dim1,&
                                                                dim2,dim3,dim4
      real(dop), dimension(dim1,dim2,dim3,dim4), intent(in)  :: zdata

      lrebound=.false.

      if (dim1 .lt. dimx) then
         write(6,*) 'Fewer data points for zdata(1) than xdata'
         return
      endif
      if (dim2 .lt. dimy) then
         write(6,*) 'Fewer data points for zdata(2) than ydata'
         return
      endif
      if (dim3 .lt. nsets) then
         write(6,*) 'Fewer data sets for zdata than nsets'
         return
      endif
      if (dim4 .lt. nplots) then
         write(6,*) 'Fewer plot sets for zdata than nplots'
         return
      endif

      zmax=-99999.0
      zmin=99999.0
      do p=1,nplots
         do s=1,nsets
            do g1=1,dimy
               do g=1,dimx
                  zmax=max(zmax,zdata(g,g1,s,p))
                  zmin=min(zmin,zdata(g,g1,s,p))
               enddo
            enddo
         enddo
      enddo
      zbound(1)=zmin
      zbound(2)=zmax

      return
      end subroutine databnds

!#######################################################################
! INMODEBNDS
!
! sets up mode bounds
!#######################################################################

      subroutine inmodebnds(ort,plotdim)

      implicit none

      integer(long)                            :: m,f,f1,f2,n
      integer(long), intent(out)               :: plotdim
      integer(long), external                  :: slen
      real(dop)                                :: lb,ub,x
      real(dop), dimension(ortdim), intent(in) :: ort
      real(dop), dimension(maxdim)             :: ddeq

! If DD in Cartesians, get equilibrium geometry from xref in refdb.dat
! natm also from DB
      if (ldd .and. ddcoo .eq. 2) then
         call rdrefdb
         f=0
         do n=1,natm
            f=f+1
            ddeq(f)=xref(1,n)
            f=f+1
            ddeq(f)=xref(2,n)
            f=f+1
            ddeq(f)=xref(3,n)
         enddo
      endif

      allmlabels='  '
      allmlen=2
      do m=1,nanmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
          !  if (f .eq. feb .and. lebcont) f=fener
            if (f .eq. ftime) then
               gdim(f)=1
               icut(f)=1
               rcut(f)=0.0_dop
            else if (f .eq. fcut) then
               gdim(f)=maxplgdim
               icut(f)=1
               rcut(f)=0.0_dop
            else
               if (ldd .and. ddcoo .eq. 2) then
                  call find_nearest_gpoint(ort(zort(f)),ddeq(f),&
                       icut(f),x,gdim(f))
                  rcut(f)=x
               else
                  icut(f)=(gdim(f)/2)+1
                  rcut(f)=ort(zort(f)-1+icut(f))
               endif
            endif

            modelen(f)=slen(modelabel(f))
            if (f .ne. ftime .and. f .ne. fcut) then
               allmlabels=allmlabels(1:allmlen)//&
                  modelabel(f)(1:modelen(f))//'  '
               allmlen=slen(allmlabels)+2
            endif

            if (f .eq. ftime) then
               ibound(1,f)=99999
               ibound(2,f)=-99999
               bound(1,f)=99999.00
               bound(2,f)=-99999.00
            else if (f .eq. fcut) then
               ibound(1,f)=1
               ibound(2,f)=maxplgdim
               bound(1,f)=0
               bound(2,f)=2.0
            else
               lb=ort(zort(f))
               ub=ort(zort(f)-1+gdim(f))
               if(lb.lt.ub) then
                  bound(1,f)=lb
                  bound(2,f)=ub
               else
                  bound(1,f)=ub
                  bound(2,f)=lb
               endif
               ibound(1,f)=1
               ibound(2,f)=gdim(f)
            endif
         enddo
      enddo
      if (nactdof.eq.1) then
         plotdim=1
         icut(1)=-1
         alabels=modelabel(spfdof(1,1))(1:modelen(spfdof(1,1)))//'=x  '
         alen=slen(alabels)
      else
         plotdim=2
         f1=axis(1)
         f2=axis(2)
         icut(f1)=-1
         icut(f2)=-2
         alabels=' '
         alen=0
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (f .eq. f1) then
                  write(alabels(alen+3:),'(2a)')&
                     modelabel(f1)(1:modelen(f1))//'=x  '
               else if (f .eq. f2) then
                  write(alabels(alen+3:),'(2a)')&
                     modelabel(f2)(1:modelen(f2))//'=y  '
               else if (f .ne. feb) then
! check required to see that striong fits into alabels array
                  if (alen+3 .lt. 950) then
                     write(alabels(alen+3:),'(a,f8.3)')&
                          modelabel(f)(1:modelen(f))//'=',rcut(f)*axunit(f)
                  else if (alen+3 .lt. 980) then
                     write(alabels(alen+3:),'(a)') ' ..XXX.. '
!                  else if (alen+3 .gt. 980) then
!                     alen = 996
!                  else if (alen+3 .lt. 2000) then
!                     write(alabels(alen+3:),'(a)') ' ..XXX.. '
               else if (lplcoo2 .or. lplcoo2tim) then
                  if(alen+11+modelen(f) .le. len(alabels)) then
                     write(alabels(alen+3:),'(a,f8.3)') &
                     modelabel(f)(1:modelen(f))//'=',rcut(f)*axunit(f)
                  else
                     write(6,'(2a)') 'Length of alabels too short.', &
                              ' Edit plot.inc'
                  endif

                  endif

               endif

               alenold=alen
               alen=slen(alabels)
               call rmblnk(alabels(alenold+3:alen))
               alen=slen(alabels)
               alabels(alen+1:alen+2)='  '
            enddo
         enddo
      endif

      return
      end subroutine inmodebnds

!#######################################################################
! TIMEBNDS
!
! sets up time bounds
!#######################################################################

      subroutine timebnds(timedat,tdim)

use global
use aglobal
use dvrdatmod
use griddatmod
use plot
use psidef
use datenmod
use compdatmod
use compdat1mod

      implicit none

      integer(long)                           :: f,i
      integer(long), intent(in)               :: tdim
      real(dop), dimension(tdim), intent(in)  :: timedat
      real(dop)                               :: mintemp,maxtemp

      mintemp = 9d99
      maxtemp = -9d99

      do f=1,nandof
         if (f .eq. ftime) then
            gdim(f)=tdim
            ibound(1,f)=1
            ibound(2,f)=tdim
            do i = 1, tdim
               maxtemp=max(maxtemp,timedat(i))
               mintemp=min(mintemp,timedat(i))
            enddo
!            bound(1,f)=timedat(1)
!            bound(2,f)=timedat(tdim)
            bound(1,f)=mintemp
            bound(2,f)=maxtemp
         endif
      enddo

      return
      end subroutine timebnds

!#######################################################################
! ENERBNDS
!
! sets up energy bounds. For plot add small margin at either end
!#######################################################################

      subroutine enerbnds(enerdat,edim)

use global
use aglobal
use dvrdatmod
use griddatmod
use plot
use psidef
use datenmod
use compdatmod
use compdat1mod

      implicit none

      integer(long)                          :: f,n
      integer(long), intent(in)              :: edim
      real(dop)                              :: margin
      real(dop), dimension(edim), intent(in) :: enerdat

      if (fener .le. 0) then
         print*,'ERROR: No Energy Data'
         stop
      endif     

      f=fener
      gdim(f)=edim
      bound(1,f) = 9999.0_dop
      bound(2,f) = -9999.0_dop

      do n=1,edim
         if (enerdat(n) .gt. bound(2,f)) then
            ibound(2,f)=n
            bound(2,f)=enerdat(n)
         endif
         if (enerdat(n) .lt. bound(1,f)) then
            ibound(1,f)=n
            bound(1,f)=enerdat(n)
         endif
      enddo

      margin = 0.01*(bound(2,f) - bound(1,f))
      bound(1,f) = bound(1,f) - margin
      bound(2,f) = bound(2,f) + margin

      return
      end subroutine enerbnds

!#######################################################################
! PLOTMENU
!
! provides a menu of possible tasks for plotting
!
!#######################################################################

      subroutine plotmenu(task,plottask,plotdim)

use global
use aglobal
use griddatmod
use operdef
use plot
use psidef
use datenmod
use compdatmod
use compdat1mod

      implicit none

      integer(long), external    :: slen
      integer(long)              :: i,ll,ierr,j
      integer(long), intent(in)  :: plottask,plotdim
      integer(long), intent(out) :: task
      character(len=c5)          :: line
      character(len=6)           :: word

!-----------------------------------------------------------------------
! plot commands
!-----------------------------------------------------------------------
      write(6,*)
    1 write(6,*)
      write(6,'(a)')   '  0 = stop,  q = stop,  quit = stop'
      write(6,'(a)')   '  1 = plot to screen'
      write(6,'(a)')   '  2 = print plot '
      write(6,'(a)')   '  3 = save plot to a postscript file'
      write(6,'(a)') &
         '  4 = save plot to a gnuplot file (use after 1 or 2)'
      write(6,'(a)') &
         '  5 = save data to an xyz file '
      if (lcntr .and. .not. lreplot) then
         write(6,'(3a)')   &
            '  9 = toggle re-plot (get new set of contours)'
      else if (lcntr .and. lreplot) then
         write(6,'(3a)')   &
            '  9 = toggle re-plot (use old set of contours)'
      endif

!-----------------------------------------------------------------------
! select plot and bounds
!-----------------------------------------------------------------------
      write(6,*)
      write(6,'(3a)')&
         ' 10 = change plot task (plot ',&
         cplottask(plottask)(1:cptlen(plottask)),')'
      if (lplcoo2 .or. lplcoo2tim) then
         if (lcutfile .or. lqcutfile) then
            ll=index(cutfile,' ')-1
            write(6,'(3a)')&
               ' 20 = change coordinate section (',cutfile(1:ll),')'
         else
            write(6,'(3a)')&
               ' 20 = change coordinate section (',alabels(1:alen),')'
         endif
      else if (lplcootim .or. lplcoo) then
         write(6,'(3a)')&
            ' 25 = change coordinate (',alabels(1:alen),')'
      else if (lpltimcoo) then
         write(6,'(3a)')&
            ' 26 = change coordinate (',alabels(1:alen),')'
      endif
      if (lplcootim .or. lplcoo2 .or. lplcoo .or. lplcoo2tim) then
         write(6,'(a)') ' 30 = change coordinate bounds'
         write(6,'(a)') ' 40 = show coordinate info'
      else if (lpltime .or. lpltimcoo) then
         write(6,'(a)') ' 30 = change time bounds'
         write(6,'(a)') ' 40 = show time info'
      else if (lplener) then
         write(6,'(a)') ' 35 = change energy bounds'
         write(6,'(a)') ' 40 = show energy info'
      else if (lplenetim) then
         write(6,'(a)') ' 30 = change energy / time bounds'
         write(6,'(a)') ' 40 = show energy / time info'
      endif
      if (lplcoo2 .or. lplcoo2tim) then
         write(6,'(a)') ' 50 = change a single coordinate'
      endif
      if (lstate) then
         write(line,'(i3)') plstate(1)
         do i=2,min(nplstates,20)
            ll=slen(line)
            write(line(ll+1:),'(i3)') plstate(i)
         enddo
         if (nplstates .gt. 20) then 
            ll=slen(line)
            write(line(ll+1:),'(a3)') "..."
            ll=slen(line)
            write(line(ll+1:),'(i3)') plstate(nplstates)
         endif
         if (nddstate .gt. 1) write(6,'(3a)')&
            ' 60 = change state selection (',line(1:slen(line)),')'

! lplsumst enables adding sum of properties over states
         if (lplsumst) then
            if (lsumst) then
               write(6,'(3a)')&
               ' 65 = Include sum over states? (on)'
            else
               write(6,'(3a)')&
               ' 65 = Include sum over states? (off)'
            endif
         endif

      endif
      if (autoz) then
         write(line,'(a)') 'auto'
      else
         write(line,'(2f8.2)') zbound(1)/zaxunit, &
              zbound(2)/zaxunit
      endif
      if (plotdim .eq. 1) then
         write(6,'(3a)')&
            ' 70 = set Y-range for plot (',line(1:slen(line)),')'
      else if (lsrf) then
         write(6,'(3a)')&
            ' 70 = set Z-range for 3D plot (',line(1:slen(line)),')'
      endif
      if (.not. autoz) then
         if (plotdim .eq. 1) then
            write(6,'(3a)') ' 75 = set auto Y-range'
         else if (lsrf) then
            write(6,'(3a)') ' 75 = set auto Z-range'
         endif
      endif
      if (lplener .or. lplenetim) then
         write(6,'(3a)') ' 80 = change energy units '
      else
         write(6,'(3a)') ' 80 = change coordinate units '
      endif
      if (lplenetim) then
         write(6,'(3a)') ' 85 = change time units '
      endif
      if (plotdim .eq. 1) then
         write(6,'(3a)') ' 90 = change Y-axis units (',&
              zaxunitlab(1:slen(zaxunitlab)),')'
      else if (plotdim .eq. 2) then
         write(6,'(3a)') ' 90 = change Z-axis units (',&
              zaxunitlab(1:slen(zaxunitlab)),')'
      endif

!-----------------------------------------------------------------------
! select type and form of plot 
!-----------------------------------------------------------------------
      write(6,*)
      if (lgnu) then

         if (lcntr) write(6,'(3a)')&
            '110 = toggle contour mode (',&
            cctr(mctr)(1:slen(cctr(mctr))),')'
         if (lcntr) write(6,'(a,i2,a)')&
            '120 = change number of contours (',nctr,')'

         if (lgrid) then
            write(6,'(a)') '150 = toggle grid lines (on)'
         else
            write(6,'(a)') '150 = toggle grid lines (off)'
         endif
         if (lsrf .and. plotdim .eq. 2) then
            write(6,'(a)') '160 = toggle surface (on)'
         else if (plotdim .eq. 2) then
            write(6,'(a)') '160 = toggle surface (off)'
         endif
         if (lcntr .and. plotdim .eq. 2) then
            write(6,'(a)') '170 = toggle contour lines (on)'
         else if (plotdim .eq. 2) then
            write(6,'(a)') '170 = toggle contour lines (off)'
         endif

         if (lsrf .and. plotdim .eq. 2) then
            write(line,'(2(a,f8.2))') &
               'rotx, rotz =',rview(1),',',rview(2)
            write(6,'(3a)')&
             '180 = change rotation of 3D plot (',line(1:slen(line)),')'

            write(line,'(2(a,f6.2))') &
               'scale, scalez =',rview(3),',',rview(4)
            write(6,'(3a)')&
               '190 = change scale of 3D plot (',line(1:slen(line)),')'

            write(line,'(f8.2)') ticslevel
            write(6,'(3a)') &
               '210 = Set height of plot above xy plane (',&
                      line(1:slen(line)),')'
            if (lhid3d) then
               write(6,'(a)') '220 = Toggle hidden 3D effect (on)'
            else
               write(6,'(a)') '220 = Toggle hidden 3D effect (off)'
            endif

            write(6,'(a)') '230 = Reset view parameters for 3D plot'

         endif
         if (plotdim .eq. 1 .or. &
                  ((lshowsys .or. lshowpot) .and. plotdim.eq.2)) then
            if (lkey) then
               write(6,'(a)') '240 = toggle key (on)'
            else
               write(6,'(a)') '240 = toggle key (off)'
            endif
         endif
         if (ltitle) then
            write(6,'(a)') '245 = toggle title (on)'
         else
            write(6,'(a)') '245 = toggle title (off)'
         endif

         if (lpoints) then
            write(6,'(a)') '250 = toggle show points (on)'
         else
            write(6,'(a)') '250 = toggle show points (off)'
         endif
         if (lpointsonly) then
            write(6,'(a)') '255 = toggle show points only (on)'
         else
            write(6,'(a)') '255 = toggle show points only (off)'
         endif
         if (lstick) then
            write(6,'(a)') '260 = toggle stick spectrum (on)'
         else
            write(6,'(a)') '260 = toggle stick spectrum (off)'
         endif
         if (lsmooth) then
            write(6,'(a)') '270 = toggle smooth curve (on)'
         else
            write(6,'(a)') '270 = toggle smooth curve (off)'
         endif

         if (ltslice) then
            if (plotform .eq. 0) then
               write(line,'(a)') '(single-plot)'
            else if (plotform .eq. 1) then
               write(line,'(a)') '(step-through)'
            else if (plotform .eq. 2) then
               write(line,'(a)') '(movie)'
            endif

            write(6,'(2a)') &
              '280 = change time-slice format, e.g. movie',&
                  line(1:slen(line))
            write(6,'(a,i3,a)')&
              '285 = change time-step (',step,')'
         endif
         if(lshowd1d .or. (lshowsys .and. plottask.gt.4)) then
            if (lnw) then
               write(6,'(a)') '290 = toggle no-weights (on)'
            else
               write(6,'(a)') '290 = toggle no-weigths (off)'
            endif
         endif
      endif

!-----------------------------------------------------------------------
! multiplot data
!-----------------------------------------------------------------------
      if(lshowsys .and. plottask.gt.4) then
      write(6,*)
      if (plotdim .eq. 2 .and. .not. lmultipl) then
         write(6,'(a)') '400 = Overlay plots (off)'
      else if (lmultipl) then
         write(6,'(a)') '400 = Overlay plots (on)'
         write(6,'(a)') '410 = File for overlay (.xyz)'
         write(6,'(10a,60a)') ' Present: ',multiname(1:slen(multiname))
         if (lcntr .and. .not. lreploto) then
            write(6,'(3a)')   &
            '420 = toggle overlay re-plot (get new set of contours)'
         else if (lcntr .and. lreploto) then
            write(6,'(3a)')   &
            '420 = toggle overlay re-plot (use old set of contours)'
         endif
         if (lcntr) write(6,'(a,i2,a)')&
            '430 = change number of contours (',nctro,')'

         if (autozo) then
            write(line,'(a)') 'auto'
         else
            write(line,'(2f8.2)') zboundo(1), zboundo(2)
         endif
         if (lsrf) then
           write(6,'(3a)')&
            '440 = set overlay Z-range for 3D plot (',&
                  line(1:slen(line)),')'
         endif
         if (.not. autoz) then
            if (lsrf) then
               write(6,'(3a)')&
               '445 = set auto overlay Z-range'
            endif
         endif
         if (lsrf .and. plotdim .eq. 2 .and. lsrfo) then
            write(6,'(a)') '450 = toggle overlay surface (on)'
         else if (lsrf .and. plotdim .eq. 2) then
            write(6,'(a)') '450 = toggle overlay surface (off)'
         endif


      endif
      endif

!-----------------------------------------------------------------------
! change natpot cut
!-----------------------------------------------------------------------
      if(lshowpot) then 
         write(6,'(a)') ' '      
         write(6,'(a,1pe11.4,1x,2a)') '500 = change natpot cut (',&
              natpotcut/zaxunit,zaxunitlab(1:slen(zaxunitlab)),')'
      endif

!-----------------------------------------------------------------------
! change operator to be plotted
!-----------------------------------------------------------------------
      if (lshowsys) then
         ll=slen(hamlab(plnham))
         write(line,'(3a)') '798 = Select Operator (', &
              hamlab(plnham)(1:ll),')'
         write(6,'(a)') line(1:slen(line))
      endif
         
!-----------------------------------------------------------------------
! change user-defined parameters used in analysis
!-----------------------------------------------------------------------
      if (ncalcpar .gt. 0) write(6,*)
      do i=1,ncalcpar
         j=799+i
         if (rcalcpar(i) .gt. 1e19) then
            write(line,'(i3,3a)') j,' = change ',&
               acalcpar(i)(1:slen(acalcpar(i))),' : infinity '
         else
            write(line,'(i3,3a,f8.3)') j,' = change ',&
               acalcpar(i)(1:slen(acalcpar(i))),' : ',rcalcpar(i)
         endif
         write(6,'(a)') line(1:slen(line))
      enddo

!-----------------------------------------------------------------------
! general commands
!-----------------------------------------------------------------------
      write(6,*)
      if (lgnu) then
         write(6,'(a)') '900 = toggle gnuplot output format (on)'
      else
         write(6,'(a)') '900 = toggle gnuplot output format (off)'
      endif
      write(6,'(3a)')&
            '910 = change printer (',pprcmd(1:slen(pprcmd)),')'

      write(6,'(a)')&
            '920 = change GNUplot command '

      write(6,*)

!-----------------------------------------------------------------------
! Read task
!-----------------------------------------------------------------------
      read(5,'(a)') word
      if((word .eq. 'q') .or. (word .eq. 'quit')) then
         task=0
      else
         read(word,*,iostat=ierr) task
         if (ierr.ne.0) then
            write(6,*)
            if (len(trim(word)).gt.0)  write(6,'(2a)') &
                   'ERROR: read error. What is: ',trim(word)
            goto 1
         endif
      endif

      return
      end subroutine plotmenu

!#######################################################################
!
! DOTASK
!
! routine to carry out tasks selected from plotting menu
!
! task          : integer specifying task
! plottask      : integer specifying plot task, i.e. which data are to 
!                 be plotted
! plotdim = 1   : 1D plot of xdata v zdata
! plotdim = 2   : 2D plot of xdata, ydata v zdata
! plotdim = -1  : 1D plot of zdata v zdata1. This plots the locus of
!                 a point with coords (z,z1). E.g. a trajectory in x1,x2
! 
! xdata(dimx)                   : input xdata
! ydata(dimy)                   : input ydata
! zdata(dim1,dim2,dim3,dim4)    : input zdata to be plotted
! dimx                          : no. of x data points
! dimx                          : no. of y data points
! nsets                         : no. of data sets for a plot
! nplots                        : no. of plots
! dim1 >= dimx , dim2 >= dimy , dim3 >= nsets, dim4 >= nplots 
!                               : physical dimensions of zdata
!
! ort(ortdim)   : system coordinate data
! timedat(tdim) : time data
! enerdat(edim) : energy data
!
! zdata1(dim11,dim12,dim13,dim14) : input zdata1 to be plotted
! dim11 >= dim1x , dim12 >= dim1y , dim13 >= nsets, dim14 >= nplots 
!                               : physical dimensions of zdata1
!
! If plotting:
! nplots of nsets of data are made, which may be 1D or 2D. 
! if nplots > 1 and if dim4=1 y axis used as time-slice.
!
!#######################################################################

      subroutine dotask(task,plottask,plotdim,xdata,ydata,zdata,&
                 dimx,dimy,nsets,nplots,dim1,dim2,dim3,dim4,&
                 ort,ortdim1,timedat,tdim,enerdat,edim,&
                 zdata1,dim11,dim12,dim13,dim14,ort2plt)

use global
use aglobal
use maxv, only:maxdim
use dvrdatmod
use griddatmod
use operdef
use plot
use psidef
use datenmod
use compdatmod
use compdat1mod


      implicit none

      integer(long), external                     :: slen
      integer(long)                               :: f1,ilb1,iub1,f2,&
                                                     ilb2,iub2,s,g,ierr,&
                                                      n,tlen,g1,i,m,f,a,len,&
                                                      max_nof_tokens,&
                                                      nof_tokens,mode,p,p1,p2,&
                                                      np,a1,ll
      integer(long), dimension(3)                  :: axis2
      integer(long), dimension(maxdim)             :: icut2
      integer(long), dimension(maxdim,2)           :: aof_tbounds
      integer(long), intent(inout)                 :: plottask,nsets
      integer(long), intent(inout)                 :: plotdim
      integer(long), intent(in)                    :: task,dim1,dim2,dim3,dim4,&
                                                      dimx,dimy,&
                                                      tdim,edim,dim11,dim12,&
                                                      dim13,dim14,ortdim1,nplots
      integer(long), dimension(ortdim), intent(in) :: ort2plt

      real(dop)                                                 :: unit,x,lb,ub
      real(dop), dimension(maxdim)                              :: rcut2
      real(dop), dimension(dimx), intent(in)                    :: xdata
      real(dop), dimension(dimy), intent(in)                    :: ydata
      real(dop), dimension(dim1,dim2,dim3,dim4), intent(in)     :: zdata
      real(dop), dimension(ortdim1), intent(in)                 :: ort
      real(dop), dimension(tdim), intent(in)                    :: timedat
      real(dop), dimension(edim), intent(in)                    :: enerdat
      real(dop), dimension(dim11,dim12,dim13,dim14), intent(in) :: zdata1
      character(len=c5)                                         :: token,&
                                                                   buffer,filename
      integer, parameter :: maxline=1400  ! should be larger than 200
      character(len=maxline)                                    :: line

      character(len=c2)                                         :: string
      logical(kind=4)                                           :: check

      lprint  = .false.
      lprint  = .false.
      if(9*ndof.ge.maxline) then
         write(6,'(2a,i7)') 'Increase parameter maxline in subroutine', &
             '  dotask (analyse/plotutils.f90) to at least:', &
             9*ndof+1
         stop  
      endif


!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: PLOT
! lredo allows an extra pass to recalculate before plotting - used
! in showsys??
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      if (task.eq.1 .or. task.eq.2 .or. task .eq. 3 .or. &
          task .eq. 5 .or. lredo) then

         if(lrecalc .and. (lshowsys.or.lshowpot)) then 
            lredo = .true.
            return
         endif
         lredo = .false.

         if(lshowsys .and. plotdim.eq.1 .and. lmultipl ) then
            write(6,'(a)') 'Overlay turned off for 1D plot' 
            lmultipl=.false.
            loverlay=.false.
         endif

         if (nplots .gt. 1 .and. dim4 .gt. 1) then
            if (nplots .gt. dim4) then
               write(6,*) 'ERROR: Too many plots requested'
               return
            endif
         else if (nplots .gt. 1 .and. dim4 .eq. 1) then
            if (nplots .gt. dim2) then
               write(6,*) 'ERROR: Too many plots requested'
               return
            endif
         endif

         if (task.eq.2) then
            lprint  = .true.
            prcmd='| '//pprcmd
         endif
         if (task .eq. 3) then
            lprint=.true.
    1       continue
            write(6,'(a)') 'Name for postscript file'
            read(5,'(a)') prcmd
            len=slen(prcmd)
            if (len .eq. 0) go to 1
            call abspath1(prcmd,len,name)
         endif
         if (task .eq. 5) then
            lprint=.true.
    2       continue
            write(6,'(a)') 'Name for data file'
            read(5,'(a)') prcmd
            len=slen(prcmd)
            if (len .eq. 0) go to 2
         endif

!......Writing to plot file

         f1=axis(1)
         if (ibound(1,f1) .lt. ibound(2,f1)) then
            ilb1=ibound(1,f1)
            iub1=ibound(2,f1)
         else 
            iub1=ibound(1,f1)
            ilb1=ibound(2,f1)
         endif
         f2=axis(2)
         if (f2 .gt. 0) then
            if (ibound(1,f2) .lt. ibound(2,f2)) then
               ilb2=ibound(1,f2)
               iub2=ibound(2,f2)
            else
               iub2=ibound(1,f2)
               ilb2=ibound(2,f2)
            endif
         else
            ilb2=1
            iub2=1
         endif
!
! map bounds to plot grid (do not map a cut)
!
         if (lmaport .and. f1 .ne. fcut) then
            iub1 = ort2plt(zort(f1)-1+iub1)
            ilb1 = ort2plt(zort(f1)-1+ilb1)
            if (f2 .gt. 0) then
               iub2 = ort2plt(zort(f2)-1+iub2)
               ilb2 = ort2plt(zort(f2)-1+ilb2)
            endif
         endif

         if (lgnu4) then   ! gnuplot versions < 5

            np=0
         do p=1,nplots
! if multiple datasets, ignore sets not in time bounds
               if (nplots .gt. 1) then
                  if ( p.lt.ibound(1,ftime) .or. p.gt.ibound(2,ftime) )&
                       goto 5
               endif
               if( plotform.eq.0  .and. p.gt.1 ) goto 5
               np=np+1

               if (lgnu .and. task .ne. 5) then
!                  if (p .gt. 1) then
                  if (lshowpsi) then
                     if(lpop2) then
                        write(plttitle,'(a,f8.3)') &
                             ' (pop2)  Time : ',timedat(p)
                     else
                        write(plttitle,'(a,f8.3)') 'Time : ',timedat(p)
                     endif
                  endif
                  call inclgnupl4(iaus,ierr,np,nsets,plotdim,timedat(p))
                  if (ierr .ne. 0) return
               endif
               do n=1,nsets
                  if (plotdim.eq.1) then
                     if (nplots .gt. 1) then
                        if (dim4 .gt. 1) then
                           p1=1
                           p2=p
                        else 
                           p1=p
                           p2=1
                        endif
                     else
                        p1=1
                        p2=1
                     endif
                     do g=ilb1,iub1
                        write(iaus,'(f18.8,e20.12)') &
                               xdata(g)/axunit(f1),&
                               zdata(g,p1,n,p2)/zaxunit
                     enddo
      
                  elseif (plotdim.eq.2) then
                     do g1=ilb2,iub2
                        do g=ilb1,iub1
                           write(iaus,'(2f18.8,e20.12)') &
                                  xdata(g)/axunit(f1),&
                                  ydata(g1)/axunit(f2),&
                                  zdata(g,g1,n,p)/zaxunit
                        enddo
                        write(iaus,*)
                     enddo

                  else if (plotdim .eq. -1) then
                     if (nplots .gt. 1) then
                        if (dim4 .gt. 1) then
                           p1=1
                           p2=p
                        else
                           p1=p
                           p2=1
                        endif
                     else
                        p1=1
                        p2=1
                     endif

                     do g=1,dimx
                        write(iaus,'(f15.8,e20.12)') &
                               zdata(g,p1,n,p2)/axunit(f1),&
                               zdata1(g,p1,n,p2)/axunit(f2)
                     enddo

                  endif

                  if (task .ne. 5) then
                     write(iaus,'(a)') 'e'
                  else if (task .eq. 5) then
                     if (nsets .gt. 1 .or. nplots .gt. 1) then
                        write(iaus,'(a)') ' '
                     endif
                  endif
               enddo
! add overlay to plot
               if (lgnu .and. lmultipl) then
                  call inclgnupl4(iaus,ierr,np,nsets,plotdim,0.0_dop)
                  if (ierr .ne. 0) return
               endif
       5       continue
            enddo
! --- Added by AP to enable the gnuplot-4 interactive features.
            if(task.ne.5) write(iaus,'(a)') 'pause -1'

         else   ! gnuplot version > 5
 
            np=0
            do p=1,nplots
! if multiple datasets, ignore sets not in time bounds
               if (nplots .gt. 1) then
                  if ( p.lt.ibound(1,ftime) .or. p.gt.ibound(2,ftime) ) cycle
               endif

! write data sets
               if (task .ne. 5) write(iaus,'(a)') '# Datablocks for plotting'
               do n=1,nsets

                  if (task .ne. 5) then
                     write(string,'(a,i5,a,i5)') "$Data",p,"_",n
                     call rmblnk(string)
                     ll=slen(string)
                     string=string(1:ll)//' << EOD '
                     ll=slen(string)
                     write(iaus,'(a)') string(1:ll)
                  endif

                  if (plotdim.eq.1) then
                     if (nplots .gt. 1) then
                        if (dim4 .gt. 1) then
                           p1=1
                           p2=p
                        else 
                           p1=p
                           p2=1
                        endif
                     else
                        p1=1
                        p2=1
                     endif
                     do g=ilb1,iub1
                        write(iaus,'(f18.8,e20.12)') &
                               xdata(g)/axunit(f1),&
                               zdata(g,p1,n,p2)/zaxunit
                     enddo
      
                  elseif (plotdim.eq.2) then
                     do g1=ilb2,iub2
                        do g=ilb1,iub1
                           write(iaus,'(2f18.8,e20.12)') &
                                  xdata(g)/axunit(f1),&
                                  ydata(g1)/axunit(f2),&
                                  zdata(g,g1,n,p)/zaxunit
                        enddo
                        write(iaus,*)
                     enddo

                  else if (plotdim .eq. -1) then
                     if (nplots .gt. 1) then
                        if (dim4 .gt. 1) then
                           p1=1
                           p2=p
                        else
                           p1=p
                           p2=1
                        endif
                     else
                        p1=1
                        p2=1
                     endif

                     do g=1,dimx
                        write(iaus,'(f15.8,e20.12)') &
                               zdata(g,p1,n,p2)/axunit(f1),&
                               zdata1(g,p1,n,p2)/axunit(f2)
                     enddo

                  endif

                  if (task .ne. 5) then
                     string='EOD '
                     ll=slen(string)
                     write(iaus,'(a)') string(1:ll)
                  else if (task .eq. 5) then
                     if (nsets .gt. 1 .or. nplots .gt. 1) then
                        write(iaus,'(a)') ' '
                     endif
                  endif
               enddo
            enddo

            if (lgnu .and. task .ne. 5) then
!               if (lshowpsi) then
!                  if(lpop2) then
!                     write(plttitle,'(a,f8.3)') &
!                          ' (pop2)  Time : ',timedat(nplots)
!                  else
!                     write(plttitle,'(a,f8.3)') 'Time : ',timedat(nplots)
!                  endif
!               endif
               call inclgnupl(iaus,ierr,nplots,nsets,plotdim,timedat)
               if (ierr .ne. 0) return
            endif

         endif   ! end of writing data

         call myflush(iaus)
         rewind(iaus)

         If(lgnu .and. task .ne. 5) then
            if (task .eq. 2) then
               write(6,'(2a)') 'Printing to : ',pprcmd(1:slen(pprcmd))
            else if (task .eq. 3) then
               write(6,'(2a)') 'Writing to : ',prcmd(1:slen(prcmd))
            endif
            if (.not.lredo  .and.  plotform.eq.1  .and.  nplots.gt.1) &
                 write(6,'(a,/a)') 'Calling GNUplot.',&
                 'Press RETURN to advance to the next time-step.'
            call excmd(gnucmd)
         else if (task .eq. 5) then
            write(6,'(2a)') 'Writing to : ',prcmd(1:slen(prcmd))
            buffer='tmpplot.pl'
            len=slen(buffer)
            call abspath1(buffer,len,name)
            line='cp -i '//buffer(1:len)//' '//prcmd(1:slen(prcmd))
            call excmd(line)
         end if
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: SAVE PLOT TO GNUPLOT FILE
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.4) then
         buffer='tmpplot.pl'
         len=slen(buffer)
         call abspath1(buffer,len,name)
         line='cp -i '//buffer(1:len)
         s = slen(line)

    4    continue
         write(6,'(a)') 'Name for gnuplot file'
         read(5,'(a)') buffer
         len=slen(buffer)
         if (len .eq. 0) go to 4
         call abspath1(buffer,len,name)
         line=line(1:s)//' '//buffer(1:len)
         write(6,'(a)') &
            'Saving plot as '//buffer(1:len) 
         call excmd(line)

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE CONTOUR REPLOT
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.9) then
         lreplot = .not. lreplot

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE PLOT TASK
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.10) then
         lrecalc=.true.
         lrebound=.true.
   10    write(6,*)
         do i=1,nplottask
            if(cptlen(i).gt.1)  write(6,'(i3,3a)')&
                 i,' = plot ',cplottask(i)(1:cptlen(i))
         enddo
         write(6,*)
         read(5,*,err=10) plottask
         if (plottask.lt.1 .or. plottask.gt.nplottask .or.&
              plottask.gt.nplottask) then
            write(6,'(/,a)') 'ERROR: invalid plot task'
            goto 10
         endif
         if(cptlen(plottask).le.1)  then
            write(6,'(/,a)') 'ERROR: invalid plot task'
            goto 10
         endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE PES SECTION
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      elseif (task.eq.20) then
         lrecalc=.true.
         lrebound=.true.
         autoz=.true.
   20    write(6,*)
         if (lshowsys .and. lshowpsi) then
            if(cplottask(7)(1:15) .eq. 'reduced density' .or.&
               cplottask(7)(1:24) .eq. 'diabatic reduced density') then
               if( plottask .eq. 7 ) then
                  write(6,'(2a)')&
                       'select a density plane from :'//&
                        allmlabels(1:allmlen)
               elseif( plottask .eq. 5 .or. plottask .eq. 6  ) then
                  write(6,'(2a)')&
                        'select a WF cut from :'//allmlabels(1:allmlen)
               endif
            else
               write(6,'(2a)')&
                       'select a pes cut from :'//allmlabels(1:allmlen)
            endif
         else
            write(6,'(2a)')&
                 'select a pes cut from :'//allmlabels(1:allmlen)
         endif

         line='current               :'
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (icut(f).eq.-1) then
                  line=line(1:slen(line))//'  x'
               else if (icut(f).eq.-2) then
                  line=line(1:slen(line))//'  y'
               else if (icut(f).eq.-9) then
                  line=line(1:slen(line))//'  min'
               else if (icut(f).eq.-8) then
                  line=line(1:slen(line))//'  max'
               else
                  write(token,'(f8.3)') rcut(f)/axunit(f)
                  call rmlblnk(token)
                  line=line(1:slen(line))//'  '//token(1:slen(token))
               endif
            enddo
         enddo
         write(6,'(a)') line(1:slen(line))
         write(6,*)
         write(6,'(a)') 'Enter: label of mode to change with x,y,eq,or VALUE'
         write(6,'(a)') '     : 0 or blank line to finish'
         write(6,'(a)') 'OR '
         write(6,'(a)') 'To read a cut defined in a file enter: file OR qfile'

! loop over input
      rdtokens: do 
         read(5,'(a)',err=20) line

         max_nof_tokens=maxdim
         call get_tokens(line,max_nof_tokens,nof_tokens,aof_tbounds)

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     token = 0 or ' ': inspect plotdim and return
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         if (aof_tbounds(1,1) .lt. 0) then
            return
         else if ((line(aof_tbounds(1,1):aof_tbounds(1,2)) .eq. '0') .or. &
           (line(aof_tbounds(1,1):aof_tbounds(1,2)) .eq. ' ')) then
            plotdim=0
            if (axis(1) .gt. 0) plotdim = plotdim+1
            if (axis(2) .gt. 0) plotdim = plotdim+1
            if (axis(1).eq.0) then
               write(6,*)
               write(6,'(a)') 'ERROR: you must specify "x"'
               goto 20
            endif
            if (plotdim.eq.2 .and. axis(2).eq.0) then
               write(6,*)
               write(6,'(a)') 'ERROR: for two dimensions you must '//&
                  'specify "x" and "y"!'
               goto 20
            endif
            if (plotdim.lt.1 .or. plotdim.gt.2) then
               write(6,*)
               write(6,'(a)') 'ERROR: you must at least specify '//&
                  '"x" and at most "x" and "y"!'
               goto 20
            endif
            return

         else if (line(aof_tbounds(1,1):aof_tbounds(1,2)) .eq. 'file') then
            print*,'Enter file with coordinates defining cut : '
            print*,'XYZ Format in Angstrom in order  '
            read(5,'(a)') cutfile

            len=index(cutfile,' ')-1
            cutfile = name(1:laenge)//'/'//cutfile(1:len)
            inquire(file=cutfile,exist=check)
            if (check) then
               print*,'Reading cut from :',cutfile(1:laenge+1+len)
               lcutfile = .true.
               plotdim=1
               axis = 0
               axis(1) = fcut
               icut(fcut)=-1

               return
            else
               print*,'File not found :',cutfile(1:laenge+1+len)
               go to 20
            endif

         else if (line(aof_tbounds(1,1):aof_tbounds(1,2)) .eq. 'qfile') then
            print*,'Enter file with coordinates defining cut : '
            print*,'Nmode Format in order  '
            read(5,'(a)') cutfile

            len=index(cutfile,' ')-1
            cutfile = name(1:laenge)//'/'//cutfile(1:len)
            inquire(file=cutfile,exist=check)
            if (check) then
               print*,'Reading cut from :',cutfile(1:laenge+1+len)
               lqcutfile = .true.
               plotdim=1
               axis = 0
               axis(1) = fcut
               icut(fcut)=-1

               return
            else
               print*,'File not found :',cutfile(1:laenge+1+len)
               go to 20
            endif

         else if (line(aof_tbounds(1,1):aof_tbounds(1,2)) .eq. 'nofile') then
            lcutfile = .false.
            if (axis(1) .eq. fcut) axis(1) = 1
            icut(fcut)=1
            return

         else if (nof_tokens.ne.2) then
            write(6,*)
            write(6,'(a,a)') 'ERROR: expecting 2 tokens here!'
            goto 20

         endif

! set new values of axes to unknown and cut to previous values
         do a=1,3
            axis2(a)=0
         enddo
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               icut2(f)=icut(f)
               rcut2(f)=rcut(f)
            enddo
         enddo

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     get DOF from first token. Stored as f1
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         f1=0
         i=1
         token=line(aof_tbounds(i,1):aof_tbounds(i,2))
         tlen=slen(token)
         do m=1,nmode
            if (m.eq.meb) cycle
            do n=1,nspfdof(m)
               f=spfdof(n,m)

               if (token .eq. modelabel(f)) f1 = f
            enddo
         enddo

         if (f1 .eq. 0) then
            write(6,*)
            write(6,'(a,a)') 'ERROR: DOF not in calculation!'
            goto 20
         endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     inspect second token: is it an x or a y or a number?
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         i=2
         token=line(aof_tbounds(i,1):aof_tbounds(i,2))
         tlen=slen(token)
         if (token.eq.'x') then
            axis2(1)=f1
            icut2(f1)=-1
         elseif (token.eq.'y') then
            axis2(2)=f1
            icut2(f1)=-2
         elseif (token.eq.'min') then
            axis2(3)=f1
            icut2(f1)=-9
         elseif (token.eq.'max') then
            axis2(3)=f1
            icut2(f1)=-8
         elseif (token.eq.'eq') then
            icut(f1) = (gdim(f1)/2)+1
            rcut(f1) = ort(zort(f1)-1+icut(f1))
         else
            read(token(1:tlen),*,iostat=ierr) rcut2(f1)
            rcut2(f1)=rcut2(f1)*axunit(f1)
            if (ierr.ne.0) then
               write(6,*)
               write(6,'(a)') 'ERROR: "'//token(1:tlen)//&
                '" is neither an "x" nor a "y" nor a number!'
               goto 20
            endif
! get grid point
            if (f1 .eq. ftime) then
               call find_nearest_gpoint(timedat,rcut2(f1),&
                    icut2(f1),x,tdim)
            else if (f1 .eq. fener) then
               call find_nearest_gpoint(enerdat,rcut2(f1),&
                    icut2(f1),x,edim)
            else 
               call find_nearest_gpoint(ort(zort(f1)),rcut2(f1),&
                    icut2(f1),x,gdim(f1))
            endif
            rcut2(f1)=x

         endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     valid selection encountered
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         if (icut2(f1) .gt. 0) then
! if f1 previously an axis reset it
            if (icut(f1) .lt. 0) then
               do a=1,3
                  if (axis(a) .eq. f1) axis(a) = 0
               enddo
            endif
! assign new value 
            icut(f1) = icut2(f1)
            rcut(f1) = rcut2(f1)

         else
            do a=1,3
               if ((axis2(a) .gt. 0) .and. (axis2(a) .ne. axis(a))) then
! set previous axis to default coordinate
                  f=axis(a)
                  if (f .gt. 0) then
                     icut(f) = (gdim(f)/2)+1
                     rcut(f) = ort(zort(f)-1+icut(f))
                  endif
! check if new axis was a previous axis
                  do a1=1,3
                     if (axis(a1) .eq. f1) axis(a1) = 0
                  enddo
! set new axis
                  axis(a) = axis2(a)
                  icut(f1) = icut2(f1)
               endif
            enddo
         endif

         alabels='  '
         alen=0
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (icut(f).eq.-1) then
                  alabels=alabels(1:alen+2)//&
                     modelabel(f)(1:modelen(f))//'=x'
               elseif (icut(f).eq.-2) then
                  alabels=alabels(1:alen+2)//&
                     modelabel(f)(1:modelen(f))//'=y'
               elseif (icut(f).eq.-9) then
                  alabels=alabels(1:alen+2)//&
                     modelabel(f)(1:modelen(f))//'=min'
               elseif (icut(f).eq.-8) then
                  alabels=alabels(1:alen+2)//&
                     modelabel(f)(1:modelen(f))//'=max'
               else if (lplcoo2 .or. lplcoo2tim) then
                  write(alabels(alen+3:),'(a,f8.3)')&
                   modelabel(f)(1:modelen(f))//'=',rcut(f)/axunit(f)
                  alenold=alen
                  alen=slen(alabels)
                  call rmblnk(alabels(alenold+3:alen))
               endif
               alen=slen(alabels)
               alabels(alen+1:alen+2)='  '
            enddo
         enddo
         call rmlblnk(alabels)
         alen=slen(alabels)

! get next input
      enddo rdtokens

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE X coordinate
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      elseif (task.eq.25) then
         lrecalc=.true.
         lrebound=.true.
         autoz=.true.
   25    write(6,*)
         write(6,'(2a)')&
              'select a new x coordinate from:'//allmlabels(1:allmlen)
         line='current                       :'
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (icut(f).eq.-1) then
                  line=line(1:slen(line))//'  '//&
                        modelabel(f)(1:modelen(f))
               endif
            enddo
         enddo
         write(6,'(a)') line(1:slen(line))
         write(6,*)
         read(5,'(a)',err=25) line
         if(line .eq. ' ') return
! old x-axis
         f1=axis(1)
         icut(f1)=(gdim(f1)/2)+1
         rcut(f1)=ort(zort(f1)-1+icut(f1))
! new x-axis
         f1=0
         do f=1,ndof
            if (line .eq. modelabel(f)) f1=f
         enddo
         
         if (f1.eq.0) then
            write(6,*)
            write(6,'(a)') 'ERROR: you must specify a modelabel'
            goto 25
         endif

         icut(f1)=-1
         axis(1)=f1

         alabels='  '
         alen=0

         do m=1,nanmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (icut(f).eq.-1) then
                  alabels=alabels(1:alen+2)//&
                     modelabel(f)(1:modelen(f))//'=x'
               elseif (icut(f).eq.-2) then
                  alabels=alabels(1:alen+2)//&
                     modelabel(f)(1:modelen(f))//'=y'
               endif
               alen=slen(alabels)
               alabels(alen+1:alen+2)='  '
            enddo
         enddo
         call rmlblnk(alabels)
         alen=slen(alabels)

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE Y coordinate
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      elseif (task.eq.26) then
         lrecalc=.true.
         lrebound=.true.
         autoz=.true.
   26    write(6,*)
         write(6,'(2a)')&
              'select a new y coordinate from:'//allmlabels(1:allmlen)
         line='current                       :'
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (icut(f).eq.-2) then
                  line=line(1:slen(line))//'  '//&
                        modelabel(f)(1:modelen(f))
               endif
            enddo
         enddo
         write(6,'(a)') line(1:slen(line))
         write(6,*)
         read(5,'(a)',err=26) line
         if(line .eq. ' ') return
! old y-axis
         f2=axis(2)
         icut(f2)=(gdim(f2)/2)+1
         rcut(f2)=ort(zort(f2)-1+icut(f2))
! new y-axis
         f2=0
         do f=1,ndof
            if (line .eq. modelabel(f)) f2=f
         enddo
         
         if (f2.eq.0) then
            write(6,*)
            write(6,'(a)') 'ERROR: you must specify a modelabel'
            goto 26
         endif

         icut(f2)=-2
         axis(2)=f2

         alabels='  '
         alen=0

         do m=1,nanmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (icut(f).eq.-1) then
                  alabels=alabels(1:alen+2)//&
                     modelabel(f)(1:modelen(f))//'=x'
               elseif (icut(f).eq.-2) then
                  alabels=alabels(1:alen+2)//&
                     modelabel(f)(1:modelen(f))//'=y'
               endif
               alen=slen(alabels)
               alabels(alen+1:alen+2)='  '
            enddo
         enddo
         call rmlblnk(alabels)
         alen=slen(alabels)

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE MODE BOUNDS
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.30) then
         lrecalc=.true.
         call show_mode_bounds
   30    write(6,*)
         if (lpltime .or. lpltimcoo) then
            write(6,'(a)') 'select lower '//&
               'and upper bound for time:'
            read(5,*,err=30) lb,ub
            mode=ftime
         else if (lplener .or. lplenetim) then
            write(6,'(a)') 'select lower '//&
               'and upper bound for energy:'
            read(5,*,err=30) lb,ub
            mode=fener
         else if (lplcoo .or. lplcootim .or. lplcoo2 .or. &
                  lplcoo2tim) then
            write(6,'(a)') 'select a mode and enter its lower '//&
               'and upper bound:'
            write(6,*)
            read(5,'(a)') buffer
            if(buffer .eq. ' ') return
            read(buffer,*,err=30,end=30) mode,lb,ub
         else
            write(6,*)
            write(6,'(a)') 'ERROR: unknown plot type'
            stop
         endif
         if (mode .eq. 0) then
            return
         else if(mode.lt.0 .or. mode.gt.nandof) then
            write(6,*)
            write(6,'(a)') 'ERROR: invalid mode selected'
            goto 30
         else if (lb.ge.ub) then
            write(6,*)
            write(6,'(a)') 'ERROR: we need lower_bound < upper_bound'
            goto 30
         endif

         lb=lb*axunit(mode)
         ub=ub*axunit(mode)
         m=dofspf(mode)
         if (mode .eq. ftime) then
            call find_nearest_gpoint(timedat,lb,ilb1,bound(1,mode),tdim)
            call find_nearest_gpoint(timedat,ub,iub1,bound(2,mode),tdim)
         else if (mode .eq. fener) then
            call find_nearest_gpoint(enerdat,lb,ilb1,bound(1,mode),edim)
            call find_nearest_gpoint(enerdat,ub,iub1,bound(2,mode),edim)
         else
            call find_nearest_gpoint(ort(zort(mode)),lb,ilb1,&
                 bound(1,mode),gdim(mode))
            call find_nearest_gpoint(ort(zort(mode)),ub,iub1,&
                 bound(2,mode),gdim(mode))
         endif
         if (ilb1.gt.iub1) then
            ibound(1,mode)=iub1
            ibound(2,mode)=ilb1
         else
            ibound(1,mode)=ilb1
            ibound(2,mode)=iub1
         endif
         call show_mode_bounds

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE ENERGY BOUNDS FOR SPECTRUM PLOT
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.35) then
         lrecalc=.true.
         call show_mode_bounds
   35    write(6,*)
         write(6,'(a)') 'select lower '//&
               'and upper bound for energy:'
         read(5,*,err=35) lb,ub
         mode=fener
         if (mode .eq. 0) then
            return
         else if(mode.lt.0 .or. mode.gt.nanmode) then
            write(6,*)
            write(6,'(a)') 'ERROR: invalid mode selected'
            goto 35
         else if (lb.ge.ub) then
            write(6,*)
            write(6,'(a)') 'ERROR: we need lower_bound < upper_bound'
            goto 35
         endif
         lb=lb*axunit(mode)
         ub=ub*axunit(mode)

         bound(1,mode)=lb
         bound(2,mode)=ub
         call show_mode_bounds

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: PRINT INFO
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.40) then
         if (lpltime) then
            write(6,*)
            write(6,'(a)') 'time bounds and number of selected points:'
         else if (lplcoo .or. lplcoo2 .or. lplcootim .or. &
                  lplcoo2tim) then
            write(6,*)
            write(6,'(a)') 'mode bounds and number of selected points:'
         endif
         call show_mode_bounds
         write(6,*)
         write(6,'(a)')&
               '**************************************************'//&
               '****************************'

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE A SINGLE COORDINATE VALUE
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.50) then
         lrecalc=.true.
         lrebound=.true.
   50    continue
         write(6,'(a)') 'Choose a coordinate to change (enter number):'
         do f=1,ndof
            if (dofspf(f) .gt. 0) then
               write(line,'(i5)') f
               line=line(1:slen(line)+5)//modelabel(f)
               if (icut(f).eq.-1) then
                  line=line(1:c1+10)//'  x'
               else if (icut(f).eq.-2) then
                  line=line(1:c1+10)//'  y'
               else if (icut(f).eq.-9) then
                  line=line(1:c1+10)//' min'
               else if (icut(f).eq.-8) then
                  line=line(1:c1+10)//' max'
               else
                  write(token,'(f8.3)') rcut(f)/axunit(f)
                  call rmlblnk(token)
                  line=line(1:c1+10)//'  '//token(1:slen(token))
               endif
               write(6,'(a)') line(1:slen(line))
            endif
         enddo

         f1=0
         f2=0
         write(6,*)
         read(5,*) f1

         do a=1,3
            axis2(a)=axis(a)
         enddo
         
         write(6,'(2a)') 'Enter x, y, or new value for ',modelabel(f1)
         write(6,'(2a)') 'For showpot you may also enter min or max'
         read(5,'(a)') token
         if(token .eq. ' ') return
         tlen=slen(token)
         if (token.eq.'x') then
            axis2(1)=f1
            icut2(f1)=-1

!           if (axis2(1) .ne. axis(1)) then
               f2=axis(1)
               write(6,'(a)') 'Enter new value for old x-axis'
               read(5,*) rcut2(f2)
               rcut2(f2)=rcut2(f2)*axunit(f2)
               if (f2 .eq. ftime) then
                  call find_nearest_gpoint(timedat,rcut2(f2),&
                       icut2(f2),x,tdim)
               else if (f2 .eq. fener) then
                  call find_nearest_gpoint(enerdat,rcut2(f2),&
                       icut2(f2),x,edim)
               else
                  call find_nearest_gpoint(ort(zort(f2)),rcut2(f2),&
                       icut2(f2),x,gdim(f2))
               endif
               rcut2(f2)=x
!           endif

         elseif (token.eq.'y') then
            axis2(2)=f1
            icut2(f1)=-2

            f2=axis(2)
            if (f2 .gt. 0 .and. axis2(2) .ne. axis(2)) then
               write(6,'(a)') 'Enter new value for old y-axis'
               read(5,*) rcut2(f2)
               rcut2(f2)=rcut2(f2)*axunit(f2)
               if (f2 .eq. ftime) then
                  call find_nearest_gpoint(timedat,rcut2(f2),&
                       icut2(f2),x,tdim)
               else if (f2 .eq. fener) then
                  call find_nearest_gpoint(enerdat,rcut2(f2),&
                       icut2(f2),x,edim)
               else
                  call find_nearest_gpoint(ort(zort(f2)),rcut2(f2),&
                       icut2(f2),x,gdim(f2))
               endif
               rcut2(f2)=x
            else
               plotdim=plotdim+1
            endif
         elseif (token.eq.'min') then
            axis2(3)=f1
            icut2(f1)=-9
         elseif (token.eq.'max') then
            axis2(3)=f1
            icut2(f1)=-8
         else
            read(token(1:tlen),*,iostat=ierr) rcut2(f1)
            rcut2(f1)=rcut2(f1)*axunit(f)
            if (ierr.ne.0) then
               write(6,*)
               write(6,'(a)') 'ERROR: "'//token(1:tlen)//&
                     '" is neither an "x" nor a "y" nor a number!'
               goto 50
            endif
            if (f1 .eq. ftime) then
               call find_nearest_gpoint(timedat,rcut2(f1),&
                    icut2(f1),x,tdim)
            else if (f1 .eq. fener) then
               call find_nearest_gpoint(enerdat,rcut2(f1),&
                    icut2(f1),x,edim)
            else
               call find_nearest_gpoint(ort(zort(f1)),rcut2(f1),&
                    icut2(f1),x,gdim(f1))
            endif
            rcut2(f1)=x

! if coordinate was x axis, select new axis. If y axis, make 1D plot
            if (f1 .eq. axis(1)) then
               write(6,'(a)') 'Select new x-axis'
               read(5,*) f2
               icut2(f2)=-1
               axis2(1)=f2
               if (f2 .eq. axis(2)) then
                  plotdim=plotdim-1
                  axis2(2)=0
               endif
            else if (f1 .eq. axis(2)) then
               plotdim=plotdim-1
               axis2(2)=0
            endif

         endif
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     valid selection encountered
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         alabels='  '
         alen=0
         do a=1,3
            axis(a)=axis2(a)
         enddo
         icut(f1)=icut2(f1)
         rcut(f1)=rcut2(f1)
         if (f2 .gt. 0) then
            icut(f2)=icut2(f2)
            rcut(f2)=rcut2(f2)
         endif
         do m=1,nmode
            f=spfdof(1,m)
            if (icut(f).eq.-1) then
               alabels=alabels(1:alen+2)//&
                  modelabel(f)(1:modelen(f))//'=x'
            elseif (icut(f).eq.-2) then
               alabels=alabels(1:alen+2)//&
                  modelabel(f)(1:modelen(f))//'=y'
            else if (lplcoo2 .or. lplcoo2tim) then
               write(alabels(alen+3:),'(a,f8.3)')&
                   modelabel(f)(1:modelen(f))//'=',rcut(f)/axunit(f)
               alenold=alen
               alen=slen(alabels)
               call rmblnk(alabels(alenold+3:alen))
            endif
            alen=slen(alabels)
            alabels(alen+1:alen+2)='  '
         enddo
         call rmlblnk(alabels)
         alen=slen(alabels)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     inspect plotdim
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         if (axis(1).eq.0) then
            write(6,*)
            write(6,'(a)') 'ERROR: you must specify "x"'
            goto 50
         endif
         if (plotdim.eq.2 .and. axis(2).eq.0) then
            write(6,*)
            write(6,'(a)') 'ERROR: for two dimensions you must '//&
               'specify "x" and "y"!'
            goto 50
         endif
         if (plotdim.lt.1 .or. plotdim.gt.2) then
            write(6,*)
            write(6,'(a)') 'ERROR: you must at least specify '//&
               '"x" and at most "x" and "y"!'
            goto 50
         endif

!-----------------------------------------------------------------------
! select states to plot
!-----------------------------------------------------------------------
      else if (task .eq. 60) then
         lrecalc=.true.
         lrebound=.true.
         autoz=.true.
   60    write(6,*)
         write(6,'(a,i5)')&
              'How many states will be plotted ? Maximum:',nddstate
         read(5,*) nplstates
         if (nplstates .gt. nddstate) then
            write(6,*)
            write(6,'(a,i5)')&
               'ERROR: No. of states must be < ',nddstate
            goto 60
         endif

   62    continue
         write(6,'(a)') 'Enter selection, "all" or numbers e.g. 2 3  '
         read(5,'(a)',err=62) line
         if(line .eq. ' ') return

         if (line(1:3) .eq. 'all') then
            do s=1,nplstates
               plstate(s)=s
            enddo
         else
            max_nof_tokens=nplstates
            call get_tokens(line,max_nof_tokens,nof_tokens,aof_tbounds)
            if (nof_tokens.ne.nplstates) then
               write(6,*)
               write(6,'(a,i3,a)') 'ERROR: expecting exactly ',nplstates,&
                  ' tokens here!'
               goto 62
            endif
            read(line,*) (plstate(s),s=1,nplstates)
         endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: SET Z-(Y-)RANGE
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.70) then
         if (plotdim .eq. 1) then
            write(6,'(2(a,f8.2))')&
               'Enter range (ymin ymax). Plot min/max: ',&
                 zmin/zaxunit,',',zmax/zaxunit
            read(5,*) zbound(1),zbound(2)
            zbound(1)=zbound(1)*zaxunit
            zbound(2)=zbound(2)*zaxunit
            autoz=.false.
         else if (plotdim .eq. 2) then
            write(6,'(2(a,f8.2))')&
               'Enter range (zmin zmax). Plot max/min: ',&
                 zmin/zaxunit,',',zmax/zaxunit
            read(5,*) zbound(1),zbound(2)
            zbound(1)=zbound(1)*zaxunit
            zbound(2)=zbound(2)*zaxunit
            autoz=.false.
         endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: SET SUM OVER STATES
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.65) then
         lsumst=.not. lsumst

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: SET AUTO Z-(Y-)RANGE
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.75) then
         autoz=.not. autoz

!-----------------------------------------------------------------------
! change coordinate units
!-----------------------------------------------------------------------
      else if (task .eq. 80) then
   80    continue
         if (lpltime) then
            write(6,'(a)') 'Choose a unit for time:'
         else if (lplener) then
            write(6,'(a)') 'Choose a unit for energy:'
         else if (lplenetim) then
            write(6,'(a)') 'Choose a unit for energy:'
         else
            write(6,'(a)') 'Choose a mode (0 selects all modes):'
         endif
         call show_mode_bounds
         if (lpltime) then
            f=ftime
            read(5,'(a)') buffer
            if (buffer .eq. 'fs ') then
               unit=1.0
               ierr=0
            else if (buffer .eq. 'au ') then
               buffer='fs'
               unit=1.0
               call keyunits(unit,buffer,ierr)
               unit=1.0/unit
               buffer='au '
            else
               write(6,'(a)') 'ERROR : unknown unit for time'
               go to 80
            endif
         else if (lplener .or. lplenetim) then
            f=fener
            read(5,'(a)') buffer
            unit=1.0
            call keyunits(unit,buffer,ierr)
         else
            read(5,*) f
            write(6,'(a,i5)') 'Choose a unit for mode:',f
            read(5,'(a)') buffer
            unit=1.0
            call keyunits(unit,buffer,ierr)
         endif
         if (f .lt. 0 .or. f .gt. nandof) then
            write(6,'(a)') 'ERROR : unknown mode'
            go to 80
         endif
         if (ierr .eq. 0) then
            if (f .eq. 0) then
               do i=1,nandof
                  axunitlab(i)=buffer(1:c1)
                  axunit(i)=unit
               end do
            else
               axunitlab(f)=buffer(1:c1)
               axunit(f)=unit
            end if
         else 
            write(6,'(a)') 'ERROR : unknown unit'
            go to 80
         endif

!-----------------------------------------------------------------------
! change time units for energy v time plots
!-----------------------------------------------------------------------
      else if (task .eq. 85) then
   85    continue
         if (lplenetim) then
            write(6,'(a)') 'Choose a unit for time:'
         endif
         call show_mode_bounds
         f=ftime
         read(5,'(a)') buffer
         if (buffer .eq. 'fs ') then
            unit=1.0
            ierr=0
         else if (buffer .eq. 'au ') then
            buffer='fs'
            unit=1.0
            call keyunits(unit,buffer,ierr)
            unit=1.0/unit
            buffer='au '
         else
            write(6,'(a)') 'ERROR : unknown unit for time'
            go to 85
         endif
         if (ierr .eq. 0) then
            if (f .eq. 0) then
               do i=1,nandof
                  axunitlab(i)=buffer(1:c1)
                  axunit(i)=unit
               end do
            else
               axunitlab(f)=buffer(1:c1)
               axunit(f)=unit
            end if
         else 
            write(6,'(a)') 'ERROR : unknown unit'
            go to 85
         endif
 
!-----------------------------------------------------------------------
! change zaxis units
!-----------------------------------------------------------------------
      else if (task .eq. 90) then
   90    continue
         write (6,'(2a)') 'Present unit : ',zaxunitlab
         write(6,'(a)') 'Choose a new unit or give a conversion factor:'
         read(5,'(a)') buffer
         read(buffer,*,err=91) unit
         unit=1.0_dop/unit
         ierr=0
         goto 92
   91    unit=1.0
         call keyunits(unit,buffer,ierr)
   92    if (ierr .eq. 0) then
            zaxunitlab=buffer(1:c1)
            zaxunit=unit 
         else 
            write(6,'(a)') 'ERROR : unknown unit'
            go to 90
         endif
      
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE CONTOUR MODE
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.110) then
         if (mctr.eq.1) then
            mctr=2
         else
            mctr=1
         endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE NUMBER OF CONTOURS
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.120) then
  120    write(6,*)
         write(6,'(a,i2,a)')&
              'enter number of contour lines (min=1,max=',&
              maxnctr,'):'
         write(6,*)
         read(5,*,err=120) nctr
         if (nctr.lt.1 .or. nctr.gt.maxnctr) then
            write(6,*)
            write(6,'(a,i2)') 'ERROR: we need 1 <= nctr <= ',&
                 maxnctr
            goto 120
         endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE GRID LINES
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.150) then
         lgrid=.not. lgrid

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE 3D SURFACE 
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.160) then
         lsrf=.not. lsrf

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE CONTOURS
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.170) then
         lcntr=.not. lcntr

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE ROTATION
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.180) then
         write(6,*)
         write(6,'(a)') 'Enter new rotx and rotz vales :'
         read(5,*) rview(1),rview(2)

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE SCALE
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.190) then
         write(6,*)
         if (lsrf) then
            write(6,'(a)') 'Enter new scale and scalez values :'
            read(5,*) rview(3),rview(4)
         else
            write(6,'(a)') 'Enter new scale value :'
            read(5,*) rview(3)
         endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: SET TICSLEVEL
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.210) then
         write(6,'(a)') &
            'Enter new factor for height of plot above plane '
         read(5,*) ticslevel

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: SET HIDDEN3D
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.220) then
         lhid3d= .not. lhid3d

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: RESET VIEW PARAMETERS
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.230) then
         rview(1)=60.0
         rview(2)=30.0
         rview(3)=1.0
         rview(4)=1.0
         lhid3d=.false.
         ticslevel=0.5
         autoz=.true.

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE KEY
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.240 .and. (plotdim.eq.1 .or. &
                    ((lshowsys.or.lshowpot) .and. plotdim.eq.2))) then
         lkey=.not. lkey

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE TITLE
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.245) then
         ltitle=.not. ltitle

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE POINTS
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.250) then
         lpoints=.not. lpoints
         if (lpoints) then
            lstick=.false.
            lpointsonly=.false.
         endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE POINTS
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.255) then
         lpointsonly=.not. lpointsonly
         if (lpointsonly) then
            lstick=.false.
            lpoints=.false.
         endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE IMPULSES
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.260) then
         lstick=.not. lstick
         if (lstick) then
            lpoints=.false.
            lpointsonly=.false.
         endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE SMOOTH CURVE
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.270) then
         lsmooth=.not. lsmooth

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE OUTPUT FORMAT FOR COORDINATE v TIME PLOT
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.280) then
         lrecalc=.true.
 280     write(6,*)
         write(6,'(a)') 'Choose one of following formats: '
!         if (.not. lplcoo2tim) write(6,'(a)') '     0 = single plot '
         write(6,'(a)') '     0 = single plot '
         write(6,'(a)') '     1 = step-through plot'
         write(6,'(a)') '     2 = movie '
         read(5,*,iostat=ierr) plotform
         if(plotform.lt.0 .or. plotform.gt.2 .or. ierr.ne.0) then
            write(6,'(a)') 'ERROR: Illegal value for plotform'
            goto 280
         endif
         if(plotform.eq.0 .and. lkey .and. lshowsys) write(6,*)&
             "Better use key=off (item 240) for 'single plot'"

      else if (task.eq.285) then
         lrecalc=.true.
 285     write(6,*)
         write(6,'(a,f8.2,a)')&
            'The time-step in fs is : step *',out2/fs,' fs' 
         write(6,'(a,i3)')&
            'Enter step (positive integer), current value=',step
         read(5,*,iostat=ierr) step
         if( step.le.0 .or. step.gt.999 .or. ierr.ne.0) then
            write(6,'(a)') 'ERROR: Illegal value for step'
            goto 285
          endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE NO-WEIGHTS
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.290 .and. (lshowd1d .or. &
               (lshowsys.and.plottask.gt.4))) then
         lnw=.not. lnw
         lrecalc=.true.

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE MULTIPLOT (OVERLAY)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.400 .and. lshowsys .and. plottask.gt.4) then
         lmultipl=.not. lmultipl
         if (lmultipl) then
            loverlay=.true.
            lkey=.false.
         endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: READ FILE WITH OVERLAY DATA 
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.410 .and. lshowsys) then
         write(6,*)
         write(6,'(a)') &
            'Enter file name containing (xyz) data for overlay'
         read(5,'(a)') multiname
         filename=name(1:laenge)//'/'//multiname(1:slen(multiname))
         call bndoverlay(filename,zmino,zmaxo)
         zboundo(1)=zmino
         zboundo(2)=zmaxo
         if (filename(1:9) .eq. 'Not found') multiname='Not found'
         lreploto=.false.

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE OVERLAY CONTOUR REPLOT
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.420 .and. lshowsys .and. plottask.gt.4) then
         lreploto = .not. lreploto

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE NUMBER OF OVERLAY CONTOURS
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.430 .and. lshowsys .and. plottask.gt.4) then
  430    write(6,*)
         write(6,'(a,i2,a)')&
              'enter number of contour lines (min=1,max=',&
              maxnctr,'):'
         write(6,*)
         read(5,*,err=430) nctro
         if (nctro.lt.1 .or. nctro.gt.maxnctr) then
            write(6,*)
            write(6,'(a,i2)') 'ERROR: we need 1 <= nctro <= ',&
                 maxnctr
            goto 430
         endif

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: SET OVERLAY Z-(Y-)RANGE
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.440 .and. lshowsys) then
         write(6,'(2(a,f8.2))')&
            'Enter range (zmin zmax). Plot max/min: ',&
                 zmino,',',zmaxo
         read(5,*) zboundo(1),zboundo(2)
         autozo=.false.

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: SET OVERLAY AUTO Z-(Y-)RANGE
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.445 .and. lshowsys) then
         autozo=.not. autozo

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE OVERLAY SURFACE
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.450 .and. lshowsys) then
         lsrfo=.not. lsrfo

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     Task: change natpotcut
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.500 .and. lshowpot) then
         write(6,'(a)') 'Enter a new value for the'
         write(6,'(a,1pe11.4,1x,a)') 'natpot cut = ',&
           natpotcut/zaxunit,zaxunitlab(1:slen(zaxunitlab))
         write(6,'(a,a)') 'Used unit: ',zaxunitlab
         write(6,'(a)') 'Change unit with menu item 90.'
         write(6,'(a)') ' '
         read(5,*) natpotcut
         natpotcut=natpotcut*zaxunit
         lrecalc=.true.
         
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
! Task: change operator
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.798 .and. lshowsys) then
  798    continue
         write(6,'(a)') 'Select from the following operators (using number)'
         do n=1,hamdim
            write(6,'(i5,1x,a)') n,hamlab(n)
         enddo
         read(5,*,err=798) plnham

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE USER-DEFINED PARAMETERS
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.ge.800 .and. task .le. 800+ncalcpar-1)then
         i=task-800+1
         write(6,'(2a)') 'Enter new value for ',&
              acalcpar(i)(1:slen(acalcpar(i)))
         read(5,*) rcalcpar(i)
         lrecalc=.true.
         lrebound=.true.

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: TOGGLE INCLUSION OF GNUPLOT COMMANDS
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.900)then
         lgnu=.not. lgnu

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE PRINT COMMAND
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.910) then
 910     write(6,*)
         write(6,'(a,/)')&
              'enter print command (e.g. "lpr -Pps2" or filename) :'
         read(5,'(a)',err=910) pprcmd
         write(6,'(a)') pprcmd

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: CHANGE GUPLOT COMMAND
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else if (task.eq.920) then
 920     write(6,*)
         write(6,'(a,/a,/a,/)')&
              'The GNUplot command currently reads:',&
                gnucmd(1:slen(gnucmd)),&
              'Give the new command: (input must be quoted!)'
         read(5,'(a)',err=920) gnucmd

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     TASK: UNKNOWN
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      else
         write(6,'(/,a)') 'ERROR: unknown task'
      endif

      return
      end subroutine dotask

!#######################################################################

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     writes gnuplot commands to the head of the outputfile connected
!     to `iounit'. the vector workf of length n contains the data to
!     be plotted.
!
!     iounit (i)
!     workf (i)
!     n (i)
!     icut (i)
!     rcut (i)
!     bound (i)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine inclgnupl(iounit,ierr,nplots,nsets,plotdim,tmdt)

use global
use aglobal
use dvrdatmod
use griddatmod
use plot
use psidef
use datenmod
use compdatmod
use compdat1mod

      implicit none

      integer(long), external               :: slen
      integer(long)                         :: i,ll,m,f1,f2,ic,ns,p,lena
      integer(long), intent(in)             :: iounit,nplots,plotdim
      integer(long), intent(out)            :: ierr,nsets
      real(dop)                             :: hmin,hmax,b,d,x,rf,zmin1,&
                                               zmax1,zaxunit1,nctr1
      real(dop), dimension(maxnctr,2), save :: ctr
      real(dop), dimension(nplots), intent(in) :: tmdt
      character(len=5000)                   :: line
      character(len=32)                     :: buf

! initialise variables
      ierr=0
      zmin1 = 0.0_dop
      zmax1 = 0.0_dop
      zaxunit1 = 0.0_dop
      nctr1 = 0.0_dop
      ns = 1

! correct  axislab(3)
      lena=slen(axislab(3))
      if(axislab(3)(lena-1:lena) .eq. ' .') lena=lena-1

!
! loverlay is toggled to plot plot or overlay
!        
      if (lmultipl) then
         loverlay = .not. loverlay
      else  
         loverlay=.false.
      endif 
         
      if(ltitle) then
         write(iounit,'(3a,a2,a3)') 'set title ', &
                '"',trim(plttitle),' \\','n \\'
         if(lfilein) then
            write(iounit,'(a)')  trim(psifile)
         elseif(lshowrst) then
            write(iounit,'(2a)') trim(name),'/restart"'
         elseif(lshowpsi) then
            write(iounit,'(2a)') trim(name),'/psi"'
         else
            write(iounit,'(2a)') trim(name),'"'
         endif
      endif

      if (nplots .gt. 1) go to 100
      if (loverlay) then
         if (lreploto) then
            go to 100
         else
            go to 29
         endif
      endif

!-----------------------------------------------------------------------
! write commands ...
!-----------------------------------------------------------------------
      write(iounit,'(a)') 'set tics out'

      if(lgrid) then
         write(iounit,'(a)')  'set grid'
      else
         write(iounit,'(a)')  'set nogrid'
      end if
      if(lprint) then
         write(iounit,'(a)')  'set terminal postscript'
         write(iounit,'(3a)') 'set output "',prcmd(1:slen(prcmd)),'"'
      else
         write(iounit,'(a)') 'set output'
         write(iounit,'(a)') 'set terminal x11'
      end if

      if (lkey) then
         write(iounit,'(a)')  'set key top right outside'
      else
         write(iounit,'(a)')  'set nokey'
      endif

      if (lmultipl) then
         write(iounit,'(a)') 'set bmargin 1'
         write(iounit,'(a)') 'set tmargin 1'
         write(iounit,'(a)') 'set lmargin 1'
         write(iounit,'(a)') 'set rmargin 1'
      endif

!-----------------------------------------------------------------------
! ... for a 1d plot (or a "trajectory" plot if plotdim = -1)
!-----------------------------------------------------------------------
  29  if (abs(plotdim).eq.1) then
         write(iounit,'(3a)') 'set xlabel "',&
            axislab(1)(1:slen(axislab(1))),'"'
         write(iounit,'(3a)')  'set ylabel "',&
            axislab(3)(1:slen(axislab(3))),'"'
         f1=axis(1)
         write(iounit,'(a,e16.9,a,e16.9,a)') 'set xrange [',&
               bound(1,f1)/axunit(f1),':',&
               bound(2,f1)/axunit(f1),'] noreverse'
         if (autoz) then
            write(iounit,'(a)')  'set yrange [*:*] noreverse'
         else 
            write(iounit,'(a,e16.9,a,e16.9,a)') 'set yrange [',&
                zbound(1)/zaxunit,':',&
                zbound(2)/zaxunit,'] noreverse'
         endif
         write(iounit,'(a)')  'set size 1.,1.'
         write(iounit,'(a)')  'set origin 0.,0.'

!-----------------------------------------------------------------------
! ... for a 2d plot
!-----------------------------------------------------------------------
      else
         if (lsrf .and. lcntr) then
            write(iounit,'(a)')  'set surface'
            write(iounit,'(a)')  'set contour'
         else if (lsrf .and. .not. lcntr) then
            write(iounit,'(a)')  'set surface'
            write(iounit,'(a)')  'set nocontour'
         else if (.not. lsrf .and. lcntr) then
            write(iounit,'(a)')  'set nosurface'
            write(iounit,'(a)')  'set contour'
         else if (.not. lsrf .and. .not. lcntr) then
            write(6,*)
            write(6,'(a)') &
            'ERROR: must select surface / contour before plotting'
            ierr=1
            return
         endif
        
         if(ltitle .or. lkey ) then
            write(iounit,'(a)')  'set clabel "%10.3e"'
            write(iounit,'(a)')  'set size 0.9,1.09'
            write(iounit,'(a)')  'set origin 0.03,-0.05'
         else
            write(iounit,'(a)')  'set size 1.05,1.3'
            write(iounit,'(a)')  'set origin 0.03,-0.06'
         endif

         if (lsrf) then
            write(line,'(2(a,f8.2),2(a,f6.2))') 'set view ',&
                rview(1),',',rview(2),',',rview(3),',',rview(4)
            if (lhid3d) then
               write(iounit,'(a)') 'set hidden3d'
            else
               write(iounit,'(a)') 'set nohidden3d'
            endif
            write (iounit,'(a,f8.2)') 'set ticslevel ',ticslevel
         else
            write(line,'(a)') 'set view 180,180,1,1'
         endif
         write(iounit,'(a)')  line(1:slen(line))
         
         f1=axis(1)
         if(lshowpot .or. .not.lsrf) then
         write(iounit,'(a,e16.9,a,e16.9,a)') 'set xrange [',&
               bound(1,f1)/axunit(f1),':',&
               bound(2,f1)/axunit(f1),'] reverse'
         else
         write(iounit,'(a,e16.9,a,e16.9,a)') 'set xrange [',&
               bound(1,f1)/axunit(f1),':',&
               bound(2,f1)/axunit(f1),'] noreverse'
         endif
         f2=axis(2)
         write(iounit,'(a,e16.9,a,e16.9,a)') 'set yrange [',&
               bound(1,f2)/axunit(f2),':',&
               bound(2,f2)/axunit(f2),'] noreverse'

         write(iounit,'(3a)') 'set xlabel "    ',&
            axislab(1)(1:slen(axislab(1))),'"'
         write(iounit,'(3a)') 'set ylabel "    ',&
            axislab(2)(1:slen(axislab(2))),"'"
         write(iounit,'(3a)') "set zlabel '    ",&
            axislab(3)(1:slen(axislab(3))),'"'
         write(iounit,'(a)')  'set cntrparam cubicspline'
         write(iounit,'(a)')  'set cntrparam points 5'

!-----------------------------------------------------------------------
! Do not recalculate contour data if not required
!-----------------------------------------------------------------------
         if(.not. lcntr .or. lreplot)  goto 100

!-----------------------------------------------------------------------
! For an overlay plot, have to calculate contour data twice. 
! First pass (p=1) is for overlay, second pass (p=2) for plot
! OTHERWISE
! p=1 for plot
!-----------------------------------------------------------------------
         if (lmultipl) then
            ns = 2
         else
            ns = 1
         endif

         do p =1,ns
            if (lmultipl .and. p .eq. 1) then
               write(6,'(a)')&
                 '--- Select contours for overlay. Units as original. '
               zmin1=zmin
               zmax1=zmax
               zaxunit1=zaxunit
               nctr1=nctr

               zmin=zmino
               zmax=zmaxo
               zaxunit=1.0_dop
               nctr=nctro

               ic = 2
            else
               write(6,'(2a)')&
                 '--- Select contours for plot. Units: ',&
                         zaxunitlab(1:slen(zaxunitlab))
               ic = 1
            endif

!-----------------------------------------------------------------------
! ... linearly spaced
!-----------------------------------------------------------------------
 31         if (mctr.eq.1) then
 33            write(6,*)
               write(6,'(a,e13.6e2,a,e13.6e2)')&
                  'zmin = ',zmin/zaxunit,&
                  '               zmax =',zmax/zaxunit
               write(6,'(/a,/a)')&
                  'level of lowest and highest contour line ?',&
                  '(1 = take zmin,zmax)'
               write(6,*)
               read(5,'(a)') buf
               if(buf .eq. ' ') return
               if(buf .eq. '1') then
                  hmin = real(zmin/zaxunit)
                  hmax = real(zmax/zaxunit)
                  goto 34
               endif
               read(buf,*,err=33,end=33) hmin,hmax
               if (hmax.le.hmin) then
                  write(6,*)
                  write(6,'(a)') 'ERROR: we need lowest_level < '//&
                     'highest_level'
                  goto 33
               endif
 34            if (nctr.gt.1) then 
                   d=(hmax-hmin)/(nctr-1) 
               else 
                  d=0. 
               end if 
               do i=1,nctr 
                  ctr(i,ic)=(i-1)*d+hmin 
               enddo

!-----------------------------------------------------------------------
! ... logarithmically spaced
!-----------------------------------------------------------------------
            else if (mctr.eq.2) then
 35            write(6,*)
               write(6,'(a,e13.6e2,a,e13.6e2)')&
                  'zmin = ',zmin/zaxunit,&
                  '               zmax =',zmax/zaxunit
               write(6,*)
               write(6,'(a)')&
                  'level of lowest and highest contour line ?'
               write(6,'(2a)')&
                  'reduction factor (0 < rf < 1) ? ',&
                  'The smaller rf, the more uneven is the spacing.'
               write(6,*)
               read(5,*,err=35) hmin,hmax,rf
               if (hmax.le.hmin) then
                  write(6,*)
                  write(6,'(a)') 'ERROR: we need lowest_level < '//&
                     'highest_level'
                  goto 35
               endif
               if (rf.le.0. .or. rf.ge.1.) then
                  write(6,*)
                  write(6,'(a)') 'ERROR: invalid reduction factor'
                  goto 35
               endif
                  
               d=hmax-hmin
               if (nctr.gt.1) then
                  b=exp(log(rf)/(nctr-1))
               else
                  b=1.
               endif
               x = real(d/(1.0_dop-rf))
               hmin = hmin -x*rf
               do i=nctr,1,-1
                  ctr(i,ic)=x+hmin
                  x=x*b
               enddo
            endif
      
!-----------------------------------------------------------------------
!     show contour levels
!-----------------------------------------------------------------------
   40       write(6,*)
            write(6,'(a,e13.6e2,a,e13.6e2)')&
                  'zmin = ',zmin/zaxunit,'               zmax =',zmax/zaxunit
            write(6,*)
            write(6,'(a)') 'contour line levels :'
            write(6,*)
            write(6,'(i2,a,e11.4e2)') (i,'   ',ctr(i,ic),i=nctr,1,-1)
            write(6,*)
            write(6,'(a)') 'ok? (yes=1,no=0)'
            write(6,*)
            read(5,*,err=40) m
            if (m.eq.0) then
               go to 31
            else if (m.ne.1) then
               go to 40
            endif

! save overlay contour data
            if (lmultipl .and. p .eq. 1) then
               zmino=zmin
               zmaxo=zmax
               nctro=nctr

               zmin=zmin1
               zmax=zmax1
               zaxunit=zaxunit1
               nctr=nint(nctr1)
            endif
         enddo

      endif ! end of set up

  100 continue

      if(lshowsys .and. lplcoo2tim) then 
         if(plotform.eq.0 .and. .not.loverlay) then
            nsets=(ibound(2,ftime)-ibound(1,ftime)+1)
         endif
      endif

!-----------------------------------------------------------------------
! write plot or splot command
!-----------------------------------------------------------------------
      line=' '
      if (abs(plotdim) .eq. 1) then
  
         do p=1,nplots
! write title for plot
            if (ltitle) then   
               write(iounit,'(3a,a2,a3,a,f11.3,a)') 'set title ',&
                  '"',plttitle(1:slen(plttitle)),' \\','n \\',&
                  ' Time :',tmdt(p),' fs "'
            endif
            do i=1,nsets
               write(line,'(2(a,i5))') "$Data",p,"_",i
               call rmblnk(line)
               ll=slen(line)
               if (i .eq. 1) then
                  line='plot '//line(1:ll)
               else
               !   line='replot '//line(1:ll)
                  line='     '//line(1:ll)
               endif

               ll=slen(line)
               if (lsmooth) then
                  ll=slen(line)
                  write(line(ll+1:),'(a)') ' smooth csplines '
               endif
               ll=slen(line)
               if (lpoints) then
                  write(line(ll+1:),'(a)') ' w lp '
               else if (lpointsonly) then
                  write(line(ll+1:),'(a)') ' w p '
               else if (lstick) then
                  write(line(ll+1:),'(a)') ' w impulses '
               else
                  write(line(ll+1:),'(a)') ' w l '
               endif
               ll=slen(line)
               write(line(ll+1:),'(a,i3,a)') ' title "',i,'"'
               ll=slen(line)
               if (i .ne. nsets) line=line(1:ll)//', \'
               write(iounit,'(a)')  line(1:slen(line))
            enddo

! add pause for "movies"
            if (plotform .le. 1) then
               write(iounit,'(a)')  "pause -1"
            else if (plotform .eq. 2) then
               write(iounit,'(a)')  "pause 0.25"
            endif

         enddo

!-----------------------------------------------------------------------
! 2D plots
!-----------------------------------------------------------------------
      else if (plotdim .eq. 2) then

         do p=1,nplots
! write title for plot
            if (ltitle) then   
               write(iounit,'(3a,a2,a3,a,f11.3,a)') 'set title ',&
                  '"',plttitle(1:slen(plttitle)),' \\','n \\',&
                  ' Time :',tmdt(p),' fs "'
            endif

! add an overlay file
            if (lmultipl) write(iounit,'(a)') 'set multiplot'

! write contour and zrange levels. 
            if (autoz) then
               write(iounit,'(a)') 'set autoscale z'
            else
               write(iounit,'(a,e13.6,a,e13.6,a)') 'set zrange [',&
                  zbound(1)/zaxunit,':',zbound(2)/zaxunit,']'
            endif
            if (lcntr) then
               ic=1
               write(line,'(a,e11.4e2)')&
                    'set cntrparam levels discrete ',ctr(1,ic)
               do i=2,nctr
                  ll=slen(line)
                  write(line(ll+1:),'(a,e13.6e2)') ',',ctr(i,ic)
               enddo
               write(iounit,'(a)') line(1:slen(line))
            endif

! plot data sets
            do i=1,nsets
               write(line,'(2(a,i5))') "$Data",p,"_",i

               call rmblnk(line)
               ll=slen(line)
               if (i .eq. 1) then
                  line='splot '//line(1:ll)
               else
                  line='      '//line(1:ll)
               endif

               ll=slen(line)
               if (lsmooth) then
                  ll=slen(line)
                  write(line(ll+1:),'(a)') ' smooth csplines '
               endif
               ll=slen(line)
               write(line(ll+1:),'(a)') ' w l '

               if (plotform .eq. 0) then
                  ll=slen(line)
                  write(line(ll+1:),'(a,i3,a)') ' title "',i,'"'
               else 
                  if (i .eq. ns) then
                     ll=slen(line)
                     write(line(ll+1:),'(a,f11.3,a)') ' title "',tmdt(p),&
                                ' fs"'
                  endif
               endif
               if (i .ne. nsets) line=line(1:ll)//', \'
               write(iounit,'(a)')  line(1:slen(line))
            enddo

            if (lmultipl) then
               write(iounit,'(a)')  'set nokey'
               write(iounit,'(a)')  'set noztics'
               if (.not. lsrfo) write(iounit,'(a)')  'set nosurface'
               if (lkey) write(iounit,'(a)')  'set key top right outside'
               if (.not. lsrfo .and. lsrf) write(iounit,'(a)')  'set surface'

! write contour and zrange levels. 
               if (autozo) then
                  write(iounit,'(a)') 'set autoscale z'
               else
                  write(iounit,'(a,e13.6,a,e13.6,a)') 'set zrange [',&
                     zboundo(1),':',zboundo(2),']'
               endif
               if (lcntr) then
                  ic=2
                  write(line,'(a,e11.4e2)')&
                       'set cntrparam levels discrete ',ctr(1,ic)
                  do i=2,nctr
                     ll=slen(line)
                     write(line(ll+1:),'(a,e13.6e2)') ',',ctr(i,ic)
                  enddo
                  write(iounit,'(a)') line(1:slen(line))
               endif
               line='splot '
               ll=slen(line)
               write(line(ll+1:),'(3a)') ' "',&
                     multiname(1:slen(multiname)),'" '
               if (lsmooth) then
                  ll=slen(line)
                  write(line(ll+1:),'(a)') ' smooth csplines '
               endif
               ll=slen(line)
               write(line(ll+1:),'(a)') ' w l notitle '

               write(iounit,'(a)')  line(1:slen(line))

! end overlay
               write(iounit,'(a)') 'unset multiplot'  
            endif

! add pause for "movies"
            if (plotform .le. 1) then
               write(iounit,'(a)')  "pause -1"
            else if (plotform .eq. 2) then
               write(iounit,'(a)')  "pause 0.25"
            endif

         enddo

      endif

      return
      end subroutine inclgnupl

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     writes gnuplot commands to the head of the outputfile connected
!     to `iounit'. the vector workf of length n contains the data to
!     be plotted.
!
! For Gnuplot versions < 5
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine inclgnupl4(iounit,ierr,nplot,nsets,plotdim,tmdt)

use global
use aglobal
use dvrdatmod
use griddatmod
use plot
use psidef
use datenmod
use compdatmod
use compdat1mod

      implicit none

      integer(long), external               :: slen
      integer(long)                         :: i,ll,m,f1,f2,ic,ns
      integer(long), intent(in)             :: iounit,nplot,plotdim
      integer(long), intent(out)            :: ierr,nsets
      real(dop)                             :: hmin,hmax,b,d,x,rf,zmin1,&
                                               zmax1,zaxunit1,nctr1
      real(dop), dimension(maxnctr,2), save :: ctr
      real(dop), intent(in)                 :: tmdt
      character(len=3000)                   :: line
      character(len=32)                     :: buf
      real(dop), save                       :: tmdt1 = 0.0_dop

! initialise variables
      ierr=0
      zmin1 = 0.0_dop
      zmax1 = 0.0_dop
      zaxunit1 = 0.0_dop
      nctr1 = 0.0_dop

!
! loverlay is toggled to plot plot or overlay
!
      if (lmultipl) then
         loverlay = .not. loverlay
      else
         loverlay=.false.
      endif
 
      if(ltitle) then   
         write(iounit,'(3a,a2,a3)') 'set title ',&
                '"',plttitle(1:slen(plttitle)),' \\','n \\'
         if(lshowrst) then
            write(iounit,'(2a)') name(1:slen(name)),'/restart"'
         elseif(lshowpsi) then
            write(iounit,'(2a)') name(1:slen(name)),'/psi"'
         else
            write(iounit,'(2a)') name(1:slen(name)),'"'
         endif
      endif
    
      if (nplot .gt. 1) go to 100
      if (loverlay) then
         if (lreploto) then
            go to 100
         else
            go to 29
         endif
      endif

!-----------------------------------------------------------------------
! write commands ...
!-----------------------------------------------------------------------
      write(iounit,'(a)') 'set tics out'

      if(lgrid) then
         write(iounit,'(a)')  'set grid'
      else
         write(iounit,'(a)')  'set nogrid'
      end if
      if(lprint) then
         write(iounit,'(a)')  'set terminal postscript'
         write(iounit,'(3a)') 'set output "',prcmd(1:slen(prcmd)),'"'
      else
         write(iounit,'(a)') 'set output'
         write(iounit,'(a)') 'set terminal x11'
      end if

      if (lkey) then
         write(iounit,'(a)')  'set key top right outside'
      else
         write(iounit,'(a)')  'set nokey'
      endif

      if (lmultipl) then
         write(iounit,'(a)') 'set bmargin 1'
         write(iounit,'(a)') 'set tmargin 1'
         write(iounit,'(a)') 'set lmargin 1'
         write(iounit,'(a)') 'set rmargin 1'
      endif

!-----------------------------------------------------------------------
! ... for a 1d plot (or a "trajectory" plot if plotdim = -1)
!-----------------------------------------------------------------------
  29  if (abs(plotdim).eq.1 .and. .not. loverlay) then
         write(iounit,'(3a)') 'set xlabel "',&
            axislab(1)(1:slen(axislab(1))),'"'
         write(iounit,'(3a)')  'set ylabel "',&
            axislab(3)(1:slen(axislab(3))),'"'
         f1=axis(1)
         write(iounit,'(a,e16.9,a,e16.9,a)') 'set xrange [',&
               bound(1,f1)/axunit(f1),':',&
               bound(2,f1)/axunit(f1),'] noreverse'
         if (autoz) then
            write(iounit,'(a)')  'set yrange [*:*] noreverse'
         else 
            write(iounit,'(a,e16.9,a,e16.9,a)') 'set yrange [',&
                zbound(1)/zaxunit,':',&
                zbound(2)/zaxunit,'] noreverse'
         endif
         write(iounit,'(a)')  'set size 1.,1.'
         write(iounit,'(a)')  'set origin 0.,0.'

!-----------------------------------------------------------------------
! ... for a 2d plot
!-----------------------------------------------------------------------
      else
         if (loverlay) goto 30
         if (lsrf .and. lcntr) then
            write(iounit,'(a)')  'set surface'
            write(iounit,'(a)')  'set contour'
         else if (lsrf .and. .not. lcntr) then
            write(iounit,'(a)')  'set surface'
            write(iounit,'(a)')  'set nocontour'
         else if (.not. lsrf .and. lcntr) then
            write(iounit,'(a)')  'set nosurface'
            write(iounit,'(a)')  'set contour'
         else if (.not. lsrf .and. .not. lcntr) then
            write(6,*)
            write(6,'(a)') &
            'ERROR: must select surface / contour before plotting'
            ierr=1
            return
         endif
        
         if(ltitle .or. lkey ) then
            write(iounit,'(a)')  'set clabel "%10.3e"'
            write(iounit,'(a)')  'set size 0.9,1.09'
            write(iounit,'(a)')  'set origin 0.,-0.05'
         else
            write(iounit,'(a)')  'set size 1.05,1.3'
            write(iounit,'(a)')  'set origin 0.0,-0.06'
         endif

         if (lsrf) then
            write(line,'(2(a,f8.2),2(a,f6.2))') 'set view ',&
                rview(1),',',rview(2),',',rview(3),',',rview(4)
            if (lhid3d) then
               write(iounit,'(a)') 'set hidden3d'
            else
               write(iounit,'(a)') 'set nohidden3d'
            endif
            write (iounit,'(a,f8.2)') 'set ticslevel ',ticslevel
         else
            write(line,'(a)') 'set view 180,180,1,1'
         endif
         write(iounit,'(a)')  line(1:slen(line))
         
         f1=axis(1)
         if(lshowpot .or. .not.lsrf) then
         write(iounit,'(a,e16.9,a,e16.9,a)') 'set xrange [',&
               bound(1,f1)/axunit(f1),':',&
               bound(2,f1)/axunit(f1),'] reverse'
         else
         write(iounit,'(a,e16.9,a,e16.9,a)') 'set xrange [',&
               bound(1,f1)/axunit(f1),':',&
               bound(2,f1)/axunit(f1),'] noreverse'
         endif
         f2=axis(2)
         write(iounit,'(a,e16.9,a,e16.9,a)') 'set yrange [',&
               bound(1,f2)/axunit(f2),':',&
               bound(2,f2)/axunit(f2),'] noreverse'

         write(iounit,'(3a)') 'set xlabel "    ',&
            axislab(1)(1:slen(axislab(1))),'"'
         write(iounit,'(3a)') 'set ylabel "    ',&
            axislab(2)(1:slen(axislab(2))),"'"
         write(iounit,'(3a)') "set zlabel '    ",&
            axislab(3)(1:slen(axislab(3))),'"'
         write(iounit,'(a)')  'set cntrparam cubicspline'
         write(iounit,'(a)')  'set cntrparam points 5'

  30     continue
!-----------------------------------------------------------------------
! Do not recalculate contour data if not required
!-----------------------------------------------------------------------
         if(.not. lcntr)  goto 100
         if(.not. loverlay .and. lreplot)  goto 100
         if(loverlay .and. lreploto)  goto 100

         if (loverlay) then
            write(6,'(a)')&
              '--- Select contours for overlay. Units as original. '
            zmin1=zmin
            zmax1=zmax
            zaxunit1=zaxunit
            nctr1=nctr

            zmin=zmino
            zmax=zmaxo
            zaxunit=1.0_dop
            nctr=nctro

            ic = 2
         else
            write(6,'(2a)')&
              '--- Select contours for plot. Units: ',&
                      zaxunitlab(1:slen(zaxunitlab))
            ic = 1
         endif

!-----------------------------------------------------------------------
! ... linearly spaced
!-----------------------------------------------------------------------
 31      if (mctr.eq.1) then
 33         write(6,*)
            write(6,'(a,e13.6e2,a,e13.6e2)')&
               'zmin = ',zmin/zaxunit,&
               '               zmax =',zmax/zaxunit
            write(6,'(/a,/a)')&
               'level of lowest and highest contour line ?',&
               '(1 = take zmin,zmax)'
            write(6,*)
            read(5,'(a)') buf
            if(buf .eq. ' ') return
            if(buf .eq. '1') then
               hmin = zmin/zaxunit
               hmax = zmax/zaxunit
               goto 34
            endif
            read(buf,*,err=33,end=33) hmin,hmax
            if (hmax.le.hmin) then
               write(6,*)
               write(6,'(a)') 'ERROR: we need lowest_level < '//&
                  'highest_level'
               goto 33
            endif
 34         if (nctr.gt.1) then 
               d=(hmax-hmin)/(nctr-1) 
            else 
               d=0. 
            end if 
            do i=1,nctr 
               ctr(i,ic)=(i-1)*d+hmin 
            enddo

!-----------------------------------------------------------------------
! ... logarithmically spaced
!-----------------------------------------------------------------------
         else if (mctr.eq.2) then
 35         write(6,*)
            write(6,'(a,e13.6e2,a,e13.6e2)')&
               'zmin = ',zmin/zaxunit,&
               '               zmax =',zmax/zaxunit
            write(6,*)
            write(6,'(a)')&
               'level of lowest and highest contour line ?'
            write(6,'(2a)')&
               'reduction factor (0 < rf < 1) ? ',&
               'The smaller rf, the more uneven is the spacing.'
            write(6,*)
            read(5,*,err=35) hmin,hmax,rf
            if (hmax.le.hmin) then
               write(6,*)
               write(6,'(a)') 'ERROR: we need lowest_level < '//&
                  'highest_level'
               goto 35
            endif
            if (rf.le.0. .or. rf.ge.1.) then
               write(6,*)
               write(6,'(a)') 'ERROR: invalid reduction factor'
               goto 35
            endif
               
            d=hmax-hmin
            if (nctr.gt.1) then
               b=exp(log(rf)/(nctr-1))
            else
               b=1.
            endif
            x = d/(1.0_dop-rf)
            hmin = hmin -x*rf
            do i=nctr,1,-1
               ctr(i,ic)=x+hmin
               x=x*b
            enddo
         endif
      
!-----------------------------------------------------------------------
!     show contour levels
!-----------------------------------------------------------------------
   40    write(6,*)
         write(6,'(a,e13.6e2,a,e13.6e2)')&
               'zmin = ',zmin/zaxunit,&
               '               zmax =',zmax/zaxunit
         write(6,*)
         write(6,'(a)') 'contour line levels :'
         write(6,*)
         write(6,'(i2,a,e11.4e2)') (i,'   ',ctr(i,ic),i=nctr,1,-1)
         write(6,*)
         write(6,'(a)') 'ok? (yes=1,no=0)'
         write(6,*)
         read(5,*,err=40) m
         if (m.eq.0) then
            go to 31
         else if (m.ne.1) then
            go to 40
         endif

         if (loverlay) then
            zmino=zmin
            zmaxo=zmax
            nctro=nctr

            zmin=zmin1
            zmax=zmax1
            zaxunit=zaxunit1
            nctr=nint(nctr1)
         endif

      endif

  100 continue

      if (lmultipl .and. .not. loverlay) &
           write(iounit,'(a)') 'set multiplot'

!-----------------------------------------------------------------------
! write plot or splot command
!-----------------------------------------------------------------------
      if(lshowsys .and. lplcoo2tim) then 
         if(plotform.eq.0 .and. .not.loverlay) then
            nsets=(ibound(2,ftime)-ibound(1,ftime)+1)
!         else
!            nsets=1
         endif
      endif

      if (abs(plotdim) .eq. 1) then
  
         line='plot'
         do i=1,nsets
            ll=slen(line)
            if (ll .gt. 2900) then
               Print*,'Too many data sets to plot. Reduce number.'
               Print*,'Plotting first ',i,'sets'
               exit
            endif
            write(line(ll+1:),'(a)') ' "-" '
            if (lsmooth) then
               ll=slen(line)
               write(line(ll+1:),'(a)') ' smooth csplines '
            endif
            ll=slen(line)
            if (lpoints) then
               write(line(ll+1:),'(a)') ' w lp '
            else if (lstick) then
               write(line(ll+1:),'(a)') ' w impulses '
            else
               write(line(ll+1:),'(a)') ' w l '
            endif
            ll=slen(line)
            if (lkey) then
               write(line(ll+1:),'(a,i3,a)') ' title "',i,'"'
            endif
            if (i .lt. nsets) then
               ll=slen(line)
               write(line(ll+1:),'(a)') ' , '
            endif
         enddo
         if (plotform .eq. 1) then
            ll=slen(line)
! GW Is this line required?
            write(line(ll+1:),'(a,f11.3,a)') '; pause -1 "',tmdt,' fs"'
         else if (plotform .eq. 2) then
            ll=slen(line)
            write(line(ll+1:),'(a)') '; pause 0.25'
         endif
         write(iounit,'(a)')  line(1:slen(line))

      else if (plotdim .eq. 2) then
!-----------------------------------------------------------------------
! write contour and zrange levels. Must be done with each splot to 
! allow multiplots
!-----------------------------------------------------------------------
         if (loverlay) then
            if (autozo) then
               write(iounit,'(a)') 'set autoscale z'
            else
               write(iounit,'(a,e13.6,a,e13.6,a)') 'set zrange [',&
                  zboundo(1),':',zboundo(2),']'
            endif
         else 
            if (autoz) then
               write(iounit,'(a)') 'set autoscale z'
            else
               write(iounit,'(a,e13.6,a,e13.6,a)') 'set zrange [',&
                  zbound(1)/zaxunit,':',zbound(2)/zaxunit,']'
            endif
         endif
         if (lcntr) then
            if (loverlay) then
               ic=2
            else
               ic=1
            endif
            write(line,'(a,e11.4e2)')&
                 'set cntrparam levels discrete ',ctr(1,ic)
            do i=2,nctr
               ll=slen(line)
               write(line(ll+1:),'(a,e13.6e2)') ',',ctr(i,ic)
            enddo
            write(iounit,'(a)') line(1:slen(line))
         endif

         line='splot' 
         if (loverlay) then
            ns=1
         else
            ns=nsets
         endif
         do i=1,ns
            ll=slen(line)
            if (loverlay) then
               write(line(ll+1:),'(3a)') ' "',&
                     multiname(1:slen(multiname)),'" notitle '
            else
               write(line(ll+1:),'(a)') ' "-" notitle '
            endif
            if (lsmooth) then
               ll=slen(line)
               write(line(ll+1:),'(a)') ' smooth csplines '
            endif
            ll=slen(line)
            write(line(ll+1:),'(a)') ' w l '
            if (i .lt. ns) then
               ll=slen(line)
               write(line(ll+1:),'(a)') ' , '
            endif
         enddo
!C         if (.not. lmultipl .or. (lmultipl .and. nplot .gt. 1)) then
         if (.not. lmultipl) then
            if (plotform .eq. 1 .and. .not. loverlay) then
               ll=slen(line)
               write(line(ll+1:),'(a,f11.3,a)') &
                     '; pause -1 "',tmdt,' fs"'
            else if (plotform .eq. 2 .and. .not. loverlay) then
               ll=slen(line)
               write(line(ll+1:),'(a)') '; pause 0.25'
            endif
         endif
         if (loverlay) then
            write(iounit,'(a)')  'set nokey'
            write(iounit,'(a)')  'set noztics'
            if (.not. lsrfo) write(iounit,'(a)')  'set nosurface'
            write(iounit,'(a)')  line(1:slen(line))
            if (lkey) write(iounit,'(a)')  'set key top right outside'
            write(iounit,'(a)')  'set ztics'
            if (.not. lsrfo .and. lsrf) &
                write(iounit,'(a)')  'set surface'
         else
            write(iounit,'(a)')  line(1:slen(line))
         endif

      endif

      if (lmultipl .and. loverlay) then
!         write(iounit,'(a)') 'set nomultiplot'
         write(iounit,'(a)') 'unset multiplot'  ! 12.03.07 HDM
         if (plotform .eq. 1) then              ! New stuff, HDM
            write(iounit,'(a,f11.3,a)')  &       ! see CC above
                 'pause -1 "',tmdt1,' fs"; pause 0.1'
         else if (plotform .eq. 2) then
            write(iounit,'(a)') 'pause 0.25'
         endif
      endif
      tmdt1=tmdt
         
      return
      end subroutine inclgnupl4

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     finds the index `point' and the corresponding location `xpoint' of
!     that grid point of mode `mode' which is nearest to x.
!
!     ort (i)
!     mode (i)
!     x (i)
!     point (o)
!     xpoint (o)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine find_nearest_gpoint(data,x,point,xpoint,dim1)

      implicit none

      integer(long)                          :: g
      integer(long), intent(in)              :: dim1
      integer(long), intent(out)             :: point
      real(dop)                              :: damin,da
      real(dop), intent(in)                  :: x
      real(dop), intent(out)                 :: xpoint
      real(dop), dimension(dim1), intent(in) :: data

      point=1
      damin=abs(data(1)-x)
      do g=2,dim1
         da=abs(data(g)-x)
         if (da.lt.damin) then
            damin=da
            point=g
         endif
      enddo

      xpoint=data(point)

      return
      end subroutine find_nearest_gpoint

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     prints a list of the available modes with their bounds and the
!     corresponding number of selected points on the terminal
!
!     bound (i)
!        array containing the bounds
!     ibound (i)
!        array containing the bounds in terms of grid point indices
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine show_mode_bounds

      implicit none

      integer(long)       :: f,modus,m,n
      
      modus=0
      if (nanmode .eq. 1 .and. nspfdof(1) .eq. 1) then
         f=spfdof(1,1)
         if (f .eq. ftime) modus=1
         if (f .eq. fener) modus=2
      endif
      if(lpltime .or. lpltimcoo) modus=1
      if(lplener) modus=2
      if(lplenetim) modus=3

      if (modus .eq. 0) then
         write(6,*)
         write(6,'(2a)')&
            '           mode           min         max         unit',&
            '         N '
         do m=1,nmode
            if (m .ne. meb) then
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  write(6,'(i3,a,e11.4,a,e11.4,a,i4)')               &
               f,'  '//modelabel(f)//' ',bound(1,f)/axunit(f),' ',   &
               bound(2,f)/axunit(f),                                 &
               '     '//axunitlab(f)(1:10)//'  ',                    &
               ibound(2,f)-ibound(1,f)+1
               enddo
            endif
         enddo

      elseif (modus .eq. 1) then
         f=ftime
         write(6,'(2a)')&
            '                       min         max         unit',&
            '         N '
         write(6,'(a,e11.4,a,e11.4,a,i4)')&
                '  '//modelabel(f)//' ',bound(1,f)/axunit(f),' ',&
                  bound(2,f)/axunit(f),&
                '     '//axunitlab(f)(1:10)//'  ',&
                  ibound(2,f)-ibound(1,f)+1
      elseif (modus .eq. 2) then
         f=fener
         write(6,'(2a)')&
            '                       min         max         unit',&
            '         N '
         write(6,'(a,e11.4,a,e11.4,a,i4)')&
                '  '//modelabel(f)//' ',bound(1,f)/axunit(f),' ',&
                  bound(2,f)/axunit(f),&
                '     '//axunitlab(f)(1:10)//'  ',&
                  ibound(2,f)-ibound(1,f)+1
      elseif (modus .eq. 3) then
         f=ftime
         write(6,'(2a)')&
            '                       min         max         unit',&
            '         N '
         write(6,'(a,e11.4,a,e11.4,a,i4)')&
                '  '//modelabel(f)//' ',bound(1,f)/axunit(f),' ',&
                  bound(2,f)/axunit(f),&
                '     '//axunitlab(f)(1:10)//'  ',&
                  ibound(2,f)-ibound(1,f)+1
         f=fener
         write(6,'(2a)')&
            '                       min         max         unit',&
            '         N '
         write(6,'(a,e11.4,a,e11.4,a,i4)')&
                '  '//modelabel(f)//' ',bound(1,f)/axunit(f),' ',&
                  bound(2,f)/axunit(f),&
                '     '//axunitlab(f)(1:10)//'  ',&
                  ibound(2,f)-ibound(1,f)+1
      endif

      return
      end subroutine show_mode_bounds
      
!#######################################################################

      subroutine bndoverlay(name,zmin,zmax)

      implicit none

      integer(long), external         :: slen
      real(dop)                       :: x,y,z
      real(dop), intent(out)          :: zmin,zmax
      character(len=*), intent(inout) :: name
      character(len=80)               :: string

      open(itmpin,file=name(1:slen(name)),status='old',err=999)

      zmin=99999.0_dop
      zmax=-99999.0_dop
   10 read(itmpin,'(a80)',end=900) string
      if (string(1:5) .eq. 'pause') goto 10
      if (string(1:1) .eq. '#') goto 10
      if (string .ne. '                                       ') then
         backspace(itmpin)
         read(itmpin,*) x,y,z
         zmax=max(zmax,z)
         zmin=min(zmin,z)
      endif
      go to 10

  900 continue
      close (itmpin)
      return
  999 write(6,*)
      write(6,'(2a)') 'ERROR: Overlay file not found : ',&
           name(1:slen(name))
      name='Not found'
      return
      end subroutine bndoverlay

!#######################################################################

      subroutine getplgdim

      implicit none

      integer(long)       :: m,n,f

      maxplgdim=0
      lmaport=.false.
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            if (gdim(f) .gt. maxplgrid) then
               lmaport=.true.
               maxplgdim=maxplgrid
            else
               maxplgdim=max(maxplgdim,gdim(f))
            endif
         enddo
      enddo
      maxplgdim2=maxplgdim*maxplgdim

      return
      end subroutine getplgdim    


!#######################################################################

      subroutine maport(plt2ort,ort2plt)

      implicit none

      integer(long)                                 :: m,n,f,delpnt,n1,n2,g,g1,g2,g3
      integer(long), dimension(maxplgrid)           :: intmiss
      integer(long), dimension(ortdim), intent(out) :: plt2ort,ort2plt
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            if (gdim(f) .gt. maxplgrid) then
!
! work out how many points to be missed in each interval. n1+1 points
! in the first n2 intervals, n1 after that.
!
               delpnt = gdim(f)-maxplgrid    ! no. points to ignore
               n1 = delpnt/maxplgrid
               n2 = mod(delpnt,maxplgrid)
               do g=1,n2
                  intmiss(g) = n1+1
               enddo
               do g=n2+1,maxplgrid
                  intmiss(g) = n1
               enddo
               
               g1=1
               do g=1,maxplgrid
                  plt2ort(zort(f)-1+g) = g1
                  g2 = g1 + intmiss(g) 
                  do g3 = g1,g2
                     ort2plt(zort(f)-1+g3) = g
                  enddo 
                  g1 = g2 + 1
               enddo

            else
               do g=1,gdim(f)
                  plt2ort(zort(f)-1+g) = g
                  ort2plt(zort(f)-1+g) = g
               enddo
            endif
         enddo
      enddo

      return
      end subroutine maport    

!#######################################################################

      subroutine maport1(plort1,plort2,plgdim1,plgdim2,ort1,&
              ort2,plt2ort1,plt2ort2,maxplgrid,gdim1,gdim2,ncutstruct)

      integer(long)                                   :: g,g1
      integer(long), intent(in)                       :: maxplgrid,gdim1,gdim2
      integer(long), intent(out)                      :: plgdim1,plgdim2
      integer(long), dimension(maxplgrid), intent(in) :: plt2ort1,plt2ort2
      real(dop), dimension(maxplgrid), intent(out)    :: plort1,plort2
      real(dop), dimension(gdim1), intent(in)         :: ort1
      real(dop), dimension(gdim2), intent(in)         :: ort2

      integer(long), intent(in)   :: ncutstruct
      integer(long)               :: ndiv,npntdiv,npnti,npntf,itmp,&
                                     i,j,nd,npnt,gpnt

! if reading a cut file set up axis labels
      if (lcutfile) then
         plgdim1 = maxplgrid
         plgdim2 = maxplgrid
         plort2 = 0

         ndiv = ncutstruct+1
         npntdiv = (maxplgrid - ncutstruct)/ndiv
         itmp = mod((maxplgrid - ncutstruct),ndiv)
         if (mod(itmp,2) .eq. 0) then
            npnti = npntdiv + itmp/2
            npntf = npntdiv + itmp/2
         else
            npnti = npntdiv + itmp/2 + 1
            npntf = npntdiv + itmp/2
         endif

! first division
         nd = 1
         j = 0
         do i = 1,npnti
            j=j+1
            plort1(j) = real(nd) - real(npnti+1-i)/real(npnti)
         enddo
         j=j+1
         plort1(j) = real(nd) 

!    middle divisions
         do nd = 1, ncutstruct-1
            do i = 1,npntdiv+1
               j=j+1
               plort1(j) = real(nd) + real(i)/real(npntdiv+1)
            enddo
         enddo

!    final division
         nd = ncutstruct
         do i = 1,npntf
            j=j+1
            plort1(j) = real(nd) + real(i)/real(npntf)
         enddo

! set bounds for plot
         bound(1,fcut) = plort1(1)
         bound(2,fcut) = plort1(maxplgrid)
         ibound(1,fcut) = 1
         ibound(2,fcut) = maxplgrid

! if reading a qcut file set up axis labels
      else if (lqcutfile) then
         plgdim1 = ncutstruct
         plgdim2 = ncutstruct
         plort2 = 0

! divide grid so that 0 is at centre and point lies 2/3 way along grid
         npnt = (ncutstruct-1)/2
         gpnt = 2*npnt/3
! midpoint
         npnt = npnt+1

         ndiv = gpnt-1

! middoint
         j=npnt
         plort1(j) = 0.0

         j=0
         do nd = npnt+1, ncutstruct
            j=j+1
            plort1(nd) = real(j)/real(ndiv+1)
         enddo

         j=0
         do nd = npnt-1, 1,-1
            j=j+1
            plort1(nd) = -real(j)/real(ndiv+1)
         enddo

! set bounds for plot
         bound(1,fcut) = plort1(1)
         bound(2,fcut) = plort1(ncutstruct)
         ibound(1,fcut) = 1
         ibound(2,fcut) = ncutstruct

      else
         plgdim1 = min(maxplgrid,gdim1)
         plgdim2 = min(maxplgrid,gdim2)
         do g=1,plgdim1
            g1 = plt2ort1(g)
            plort1(g) = ort1(g1)
         enddo
         do g=1,plgdim2
            g1 = plt2ort2(g)
            plort2(g) = ort2(g1)
         enddo
      endif

      return
      end subroutine maport1

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  calcd2d_1ml computes the (diagonal part of the) 2d reduced density  !
!  when both DOFs are in same combined mode.                           !
!  Used in showsys and dengen.                                         !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  subroutine calcd2d_1ml(psi,phi,vd,gda,md,gdb,nd,gd1,gd2,&
                         dim1,adml,swap,lnw,wfunc,mxgdm,&
                         weight1,weight2,ilb2,iub2,ilb1,iub1,agmat)


     implicit none

     integer(long),intent(in)   :: vd,gda,md,gdb,nd,dim1,mxgdm
     integer(long),intent(in)   :: ilb1,iub1,ilb2,iub2,gd1,gd2
     integer(long)              :: i,j,k,l,g1,g2
     real(dop),intent(in)       :: weight1(gd1),weight2(gd2)
     real(dop),intent(out)      :: wfunc(mxgdm,mxgdm),adml
     complex(dop),intent(inout) :: psi(dgldim)
     complex(dop),intent(inout) :: phi(vd,gda,md,gdb,nd,dim1)
     complex(dop)               :: sphi(vd,gda,md,gdb,nd,dim1)
     complex(dop),intent(in)    :: agmat(*)
     complex(dop)               :: norm2
     complex(dop)               :: overlap(dmatdim)
     logical(kind=4),intent(in) :: lnw,swap

     ! swap is true, if f2 comes before f1 in the combined mode m.
     ! gda=gd2 if swap is true, gda=gd1 if swap is false.

     !--Save the sets of SPFs to sphi and zero phi.
     sphi=phi
     phi=(0.d0,0.d0)

     do g2=ilb2,iub2
        do g1=ilb1,iub1

           if(swap) then
              do i=1,dim1
                 do j=1,nd
                    do k=1,md
                       do l=1,vd
                          phi(l,1,k,1,j,i) = sphi(l,g2,k,g1,j,i)
                       enddo
                    enddo
                 enddo
              enddo
           else
              do i=1,dim1
                 do j=1,nd
                    do k=1,md
                       do l=1,vd
                          phi(l,1,k,1,j,i) = sphi(l,g1,k,g2,j,i)
                       enddo
                    enddo
                 enddo
              enddo
           endif
           call spfovl(overlap,psi,psi,agmat,agmat,0)
           call normml(psi,norm2,overlap)
           adml=abs(norm2)
           if(adml.lt.1.d-12) adml=0.d0
           if(lnw) then
              wfunc(g1,g2)=adml
           else
              wfunc(g1,g2)=adml/(weight1(g1)*weight2(g2))
           endif

        enddo ! g1
     enddo    ! g2

     !--Re-set the sets of SPFs
     phi=sphi

  end subroutine calcd2d_1ml


!------------------------------------------------------------------------------


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  calcd2d_2ml computes the (diagonal part of the) 2d reduced density when !
!  both DOFs are in different modes. If f2=0 it computes the 1D density.   !                           !
!  Used in showsys and dengen.                                             !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  subroutine calcd2d_2ml(psi,phi1,phi2,vd1,gd1,nd1,vd2,gd2,nd2,f2, &
                         dim1,dim2,adml,lnw,wfunc,mxgdm,&
                         weight1,weight2,ilb2,iub2,ilb1,iub1,agmat)

     implicit none

     integer(long),intent(in) :: vd1,gd1,nd1,dim1,ilb1,iub1,mxgdm
     integer(long),intent(in) :: vd2,gd2,nd2,dim2,ilb2,iub2,f2
     integer(long)            :: i,j,k,g1,g2
     real(dop),intent(in)       :: weight1(gd1),weight2(gd2)
     real(dop),intent(out)      :: wfunc(mxgdm,mxgdm),adml
     complex(dop),intent(inout) :: psi(dgldim)
     complex(dop),intent(inout) :: phi1(vd1,gd1,nd1,dim1),phi2(vd2,gd2,nd2,dim2)
     complex(dop),intent(in)    :: agmat(*)
     complex(dop)               :: sphi1(vd1,gd1,nd1,dim1),sphi2(vd2,gd2,nd2,dim2)
     complex(dop)               :: overlap(dmatdim)
     complex(dop)               :: norm2
     logical(kind=4),intent(in) :: lnw

     !--Save the sets of SPFs to sphi and zero phi
     sphi1=phi1
     phi1=(0.d0,0.d0)
     if(f2.gt.0) then
        sphi2=phi2
        phi2=(0.d0,0.d0)
     endif

     do g2=ilb2,iub2
        do g1=ilb1,iub1

           do i=1,dim1
              do j=1, nd1
                 do k=1,vd1
                    phi1(k,1,j,i) = sphi1(k,g1,j,i)
                 enddo
              enddo
           enddo
           if(f2.gt.0) then
              do i=1,dim2
                 do j=1, nd2
                    do k=1,vd2
                       phi2(k,1,j,i) = sphi2(k,g2,j,i)
                    enddo
                 enddo
              enddo
           endif

           call spfovl(overlap,psi,psi,agmat,agmat,0)
           call normml(psi,norm2,overlap)
           adml=abs(norm2)
           if(adml.lt.1.d-12) adml=0.d0
           if(lnw) then
              wfunc(g1,g2)=adml
           else
              wfunc(g1,g2)=adml/weight1(g1)
              if (f2 .gt. 0) then
                 wfunc(g1,g2)=wfunc(g1,g2)/weight2(g2)
              endif
           endif

        enddo ! g1
     enddo    ! g2

     !--Re-set the sets of SPFs
     phi1=sphi1
     if(f2.gt.0) phi2=sphi2

  end subroutine calcd2d_2ml


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!------------------------------------------------------------------------------


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Sub to deal with Gnu's " issue                                           !
! replaces any " with ''                                                   !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   subroutine mode_lab_corr(cur_label, new_label)
      implicit none
      
      character(len=c1), intent(in)         :: cur_label
      !character, intent(in)                 :: prob_symbol, replace_symbol
      character(len=c1), intent(out)        :: new_label
      integer             :: primed_pos

      new_label = cur_label
      primed_pos = SCAN(new_label, '"')
      if (primed_pos .ne. 0) then
         new_label(primed_pos:primed_pos+2) = "''"
     endif


   end subroutine mode_lab_corr

      end module plotutils

