!***********************************************************************
!
!             Analyse program PRHOSUB
!
! This program reads the pdensity-file and writes out the reduced 
! density matrix on the primitive grid. The grid is 1D x 1D if
! the DOFs are uncombined. For combined modes, the grid dimension
! the square of the mode-grid. In this case, ort is not output, but
! rather the grid-point number. 
!
!     11/02  MN
!
!***********************************************************************
      module prhosubmod

      use decimal, only : long, dop

      integer(long), save  :: nm,ns   ! indices of mode and state
      integer(long), save  :: pts
      real(dop), save      :: pmin,pmax
      logical(kind=4),save :: lk

      end module prhosubmod
      
      program prhosub

use decimal, only: long,dop,sip
use global
use globmemmod
use aglobal
use dvrdatmod
use griddatmod
use psidef
use iopsidef
use channels
use datenmod
use prhosubmod
use logdat
use rddvrmod, only: rddvr,dvrdata,dvrinfo

use op1lib, only: zerovxz
use iofile, only: rdrhoinfo
use rdfilesmod, only: rdtodata
use zeigauswmod, only: zeigausw
use versions

use module_interface

      implicit none

      integer(long)      :: chkdvr,chkgrd,chkpsi,chkdat,ilbl,s,m,n,ll
      character(len=c5)  :: filename, mdlbl
      character(len=80)  :: nspf
      logical(kind=4)    :: lerr

      
!-----------------------------------------------------------------------
! Initialize variables and set defaults
!-----------------------------------------------------------------------
      call default
      call adefault

!-----------------------------------------------------------------------
! Declare files to be read
! set flag to .true. if file is to be read
!
!     lrddvr    DVR file
!     lrdoper   OPER file
!-----------------------------------------------------------------------
      lrddvr=.true.
      lrdoper=.false.

!-----------------------------------------------------------------------
! Declare resources required (other than those needed to read a file)
!
! ncomp   : no. of comparison data sets to be used.
!           See auswutil.F for details of use of comparison sets
! lpsi    : wavefunction
! lpsi1   : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat   : density matrices for psi
! ldmat1  : density matrices for psi1
! lort    : grid coordinates
! lgpop   : grid populations
! ltrafo  : FBR/DVR trafo matrices
!----------------------------------------------------------------------
      lpsi=.true.
      lpsi1=.false.
      lpsigrd=.false.
      ldmat=.false.
      ldmat1=.false.
      lort=.true.
      lgpop=.false.
      ltrafo=.true.

!-----------------------------------------------------------------------
! read input parameters and options
! open files
!-----------------------------------------------------------------------
      call prhoinput

!-----------------------------------------------------------------------
! open input and output data files
!
! use standard channels if possible
!  ipden : pdensity file
!  irho : gridpop file
!  ipsi : psi file
!  iaut : auto file
!  ichk : check file
!  iaus : output file
!-----------------------------------------------------------------------
      filename=filein
      open(ipden,file=filename,form='unformatted',status='old',err=1000)

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(ipden)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef

!-----------------------------------------------------------------------
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
! First read file version number
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      filename=filein
      rewind(ipden)
      read(ipden,err=999) filever(ipden)
      if (filever(ipden) .ge. 11.00_dop) then
         call rdrhoinfo(ipden,chkdvr,chkgrd,chkdat)
         call rdpsidef(ipden,chkpsi)
      else
         call rdrhoinfo(ipden,chkdvr,chkgrd,chkdat)
         read(ipden) nmode, nstate, ((dim(m,s),m=1,nmode),s=1,nstate)
      endif
      if(nm.gt.nmode) then
         write(6,'(2a/,2i6)') ' The  first argument (mode)',&
              ' must not be larger than nmode.', nm, nmode
         stop
      end if
      if(ns.gt.nstate) then
         write(6,'(2a/,2i6)') ' The  second argument (state)',&
              ' must not be larger than nstate.', ns,nstate
         stop
      end if

      dgldim = 1
      do s = 1, nstate
         do m = 1, nmode
           dgldim = max(dgldim,subdim(m)*dim(m,s)) 
        enddo
      enddo 

!-----------------------------------------------------------------------
! write mode information.
!-----------------------------------------------------------------------
      write(ilog,'(/,a)') ' mode: No. of spf''s per state : modelabels'
      m=nm
      ll = 1
      mdlbl = '                                                       '
      nspf  = '                                                       '
      do  n=1,nspfdof(m)
         ilbl  = index(modelabel(spfdof(n,m)),' ')
         mdlbl = mdlbl(1:ll)//modelabel(spfdof(n,m))(1:ilbl)
         ll    = ll + ilbl
      end do
      write(nspf,'(95i3)') (dim(m,s),s=1,nstate)
      if(3*nstate+ll .le. 72 ) then
         write(ilog,'(i4,4a)') m,' :',nspf(1:3*nstate),' : ',&
                            mdlbl(1:ll)
      else
         write(ilog,'(i4,3a)') m,' :',nspf(1:3*nstate),' :'
         write(ilog,'(a,/)') mdlbl(1:ll)
      end if
      
!-----------------------------------------------------------------------
! read information from system files
!-----------------------------------------------------------------------
      chkdvr=2
      filename=dname(1:dlaenge)//'/dvr'
      open(idvr,file=filename,form='unformatted',status='old',&
              err=1000)
      call dvrinfo(lerr,chkdvr)
      close(idvr)
      call chksyserr(filename,lerr)
      if (lerr) go to 999
  
!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call zeigausw
      call memausw

      if(.not.lk) pts=1
      mcdum=mcdim
      mcdim=mcdum+pts

      mrweight = mrdim
      mrdim=mrdim + maxgdim*ndof ! mrdim=mrdim+maxgdim*maxdata1*nstate
      

!-----------------------------------------------------------------------
! call analysis routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim/2,&
       ' real*4     : ', mfdim/2,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

      call subprhosub(mc(mcpsi),mr(mrort),mr(mrtrafo),&
                      mr(mrweight),mc(mcdum))

deallocate(mc,mr,mi,ml,ms,mf)
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_psidef

      stop
  999 continue
      write(6,'(2a)') 'Error reading :', filename
      stop
 1000 continue
      ilbl=index(filename,' ')-1
      write(6,'(2a)') ' Cannot open file: ',filename(1:ilbl)
      stop

      contains

!***********************************************************************
!
!     HERE IS THE SUBROUTINE TO DO THE ANALYSIS
!
!***********************************************************************

      subroutine subprhosub (psi,ort,trafo,weight,rhok)

      implicit none

      integer(long)       :: chkdvr,n,s,m,k,p1,&
                             kmin,kmax,kmin2,dk,f,g,ggdim,&
                             gdim1,dof,sphtyp,jmax,gj,mmax,&
                             mincr,mmin,gm,dj,j0,zeig,j,j2,j3
      integer(long)       :: mmmin,mmmax,mmmin2,dm,p2

      integer(long)                                :: ierr
      integer(long)                                :: ilbl,jlbl,ndata
      integer(long), dimension(1)                  :: idum,idum1,idum2
      complex(dop)                                 :: rhosub
      complex(dop), dimension(1)                   :: zdum,zdum1,zdum2,zdum3
      complex(dop), dimension(dgldim), intent(out) :: psi
      complex(dop), dimension(pts), intent(out)    :: rhok
      real(dop), dimension(:),allocatable          :: avec
      real(dop)                                    :: dt,time,dp,dx
      real(dop), dimension(1)                      :: rdum,rdum1,rdum2
      real(dop), dimension(ortdim), intent(out)    :: ort
      real(dop), dimension(dvrdim), intent(out)    :: trafo
      real(dop), dimension(maxgdim), intent(out)   :: weight
      character(len=2)                             :: fnum
      character(len=c2)                            :: rdtag
      character(len=c5)                            :: gnucmd, fileout

!-----------------------------------------------------------------------
! Open output file
!-----------------------------------------------------------------------
      write(fnum,'(I2.2)') nm
      if (lk) then       ! if momentum spectrum is required
        fileout="prhosub"//fnum//".p.dat"
        call zerovxz(rhok,pts) 
        dp=(pmax-pmin)/dble(pts-1)
      else
        fileout="prhosub"//fnum//".x.dat"
      endif
      if (lovwr) then
         open(iaus,file=trim(fileout),form="formatted",status="unknown")
      else
         open(iaus,file=trim(fileout),form="formatted",status="new",err=910)
      endif

! --- write the gnuplot commands, if desired, to the fileout-file
! note the '" \\' written a3 outputs only "\", but this is required
! as some compilers need "\\" to ouput "\"
      if(lgnu) then
         write(iaus,'(a)') 'set title " "'
         write(iaus,'(a)') "set tics out"
         write(iaus,'(a)') "set nogrid"
         write(iaus,'(a)') "set output"
         write(iaus,'(a)') "set terminal x11"
         write(iaus,'(a)') "set nokey"
         write(iaus,'(a)') "set nosurface"
         write(iaus,'(a)') "set contour"
         write(iaus,'(a)') 'set clabel "%10.3e"'
         write(iaus,'(a)') "set size 0.9,1.09"
         write(iaus,'(a)') "set origin 0.,-0.05"
         write(iaus,'(a)') "set view 180,180,1,1"
         write(iaus,'(a)') "set xrange [ *:* ] reverse"
         write(iaus,'(a)') "set yrange [ *:* ] noreverse"
         write(iaus,'(3a)') 'set xlabel "',trim(modelabel(nm)),' [au]"'
         write(iaus,'(3a)') 'set ylabel "',trim(modelabel(nm)),' [au]"'
         write(iaus,'(a)') "set cntrparam cubicspline"
         write(iaus,'(a)') "set cntrparam points 5"
      end if

!-----------------------------------------------------------------------
! read data from dvr file if desired
! edit which data (set dvrdata(x) to .true. and substitute dummy array)
!-----------------------------------------------------------------------
      if (lrddvr) then
         dvrdata(1)=.true.
         dvrdata(2)=.true.
         dvrdata(4)=.false.
         dvrdata(5)=.false.
         dvrdata(6)=.false.
         dvrdata(7)=.false.
         dvrdata(8)=.false.
         dvrdata(9)=.false.
         dvrdata(10)=.false.
         dvrdata(11)=.false.
         dvrdata(12)=.false.
         chkdvr=2
         call rddvr(ort,trafo,rdum,rdum1,zdum,zdum1,&
                    idum,zdum2,zdum3,idum1,idum2,rdum2,chkdvr)
      endif

      dt     = out1/fs
      time   = tinit/fs

!-----------------------------------------------------------------------
! compute maximum number of single-particle functions and allocate avec
!-----------------------------------------------------------------------
      maxspf=0
      do s=1,nstate
         do m=1,nmode
            maxspf=max(maxspf,dim(m,s))
         enddo
      enddo
      allocate(avec(maxspf),stat=ierr)
      if (ierr > 0) then
         routine = 'subprhosub'
         write(message,'(a,i5)') 'Cannot allocate memory for avec. Maxspf :',maxspf
         call errormsg
      endif

! New **************************************
!-----------------------------------------------------------------------
! Determine the DVR weights.
!-----------------------------------------------------------------------
      if (lrddvr) then
         if( (.not.lnw) .and. (nspfdof(nm).gt.1)) then 
           write(6,'(a)') "Program works only for uncombined mode "
           write(6,'(a)') "unless option '-nw' is set."
           stop
         endif
         f=spfdof(1,nm)
         dof=f
         ggdim = gdim(dof)
         gdim1 = gdim(dof)
         if(basis(dof) .eq. 12 .or. basis(dof) .eq. 20 .or. &
            basis(dof) .eq. 21) &
            ggdim = ipbaspar(3,dof)

!-----------------------------------------------------------------------
! Special handling for sphFBR
!-----------------------------------------------------------------------
         if( basis(f) .eq. 6 .or. basis(f) .eq. -1 ) then
           if(basis(f) .eq. -1 ) dof = f-1
           sphtyp = ipbaspar(1,dof)
           jmax   = ipbaspar(2,dof)
           gj     = ipbaspar(4,dof)
           mmax   = ipbaspar(1,dof+1)
           mincr  = ipbaspar(2,dof+1)
           mmin   = ipbaspar(3,dof+1)
           gm = (mmax-mmin)/mincr + 1
           dj = 1
           if( sphtyp.eq.2  .or. sphtyp.eq.3 ) dj = 2 
           j0 = mod(jmax,dj)
           if(basis(f) .eq. -1 ) then
             gdim1 = gm
             ggdim = gm
             do g = 1, gdim1
               ort(zort(dof)-1+g) = mmin + (g-1)*mincr
             end do
           else
             gdim1 = gj
             ggdim = gj
             do g = 1, gdim1
               ort(zort(dof)-1+g) = j0 + (g-1)*dj
             end do
           end if
         end if

!...Determine the appropriate 'column' of trafo(gdim,gdim,kdim) for KLeg/PLeg
!...and trafo(gdim,gdim,kdim,mmdim) for Wigner
         p1=0
         if( basis(f) .eq. 7 .or. basis(f) .eq. 20) then ! KLeg
           kmin=ipbaspar(1,f+1)
           kmax=ipbaspar(2,f+1)
           kmin2=ipbaspar(3,f+1)
           p1=(min(kmax,kmin2)-kmin)
           dk=max(ipbaspar(5,f+1),1)
           p1=(p1+dk-1)/dk
!k           p1=p1*gdim(f)**2
           p1=p1*ggdim**2
         elseif( basis(f) .eq. 9 .or. basis(f) .eq. 21) then           ! PLeg

           kmin=ipbaspar(1,f+1)
           kmax=ipbaspar(2,f+1)
           kmin2=ipbaspar(4,f+1)
!k           p1=(min(kmax,kmin2)-kmin)*gdim(f)**2
           p1=(min(kmax,kmin2)-kmin)*ggdim**2
         elseif( basis(f) .eq. 33 ) then          ! Wigner
           kmin=ipbaspar(2,f)
           kmax=ipbaspar(3,f)
           kmin2=ipbaspar(5,f)
           p2=min(kmax,kmin2)-kmin
           if (basis(f+1).eq.8) then
              dk=ipbaspar(5,f+1)
              p2=(p2+dk-1)/dk
           endif
           p2=p2*gdim(f)**2
           mmmin=ipbaspar(6,f)
           mmmax=ipbaspar(7,f)
           mmmin2=ipbaspar(9,f)
           p1=min(mmmax,mmmin2)-mmmin
           if (basis(f+2).eq.8) then
              dm=ipbaspar(5,f+2)
              p1=(p1+dm-1)/dm
           endif
           p1=p1*ipbaspar(4,f)*gdim(f)**2 + p2
         endif
            
         call dvrweights(trafo(zdvr(f)+p1),ort(zort(f)),gdim1,ggdim,&
                     weight,basis(f),rpbaspar(1,f),ipbaspar(1,f),&
                     ipbaspar(1,f+1),.false.,lnw)
      
      endif

!-----------------------------------------------------------------------
! Rewind data file
!-----------------------------------------------------------------------
      if (filever(ipden) .ge. 11.0_dop) then
         rewind(ipden)
         call rdtodata(ipden)    
      endif

!-----------------------------------------------------------------------
! loop over steps
!-----------------------------------------------------------------------
   10 continue
      if (filever(ipden) .ge. 11.00_dop) then
         do
   20       continue
            read(ipden,err=20,end=77) rdtag
            if (rdtag(1:1) .ne. '<') cycle

            ilbl=2
            jlbl=index(rdtag,'>')-1
            if (rdtag(ilbl:ilbl+8) .eq. 'data set') then
               read(rdtag(ilbl+9:jlbl),*,err=900) ndata

            else if (rdtag(ilbl:jlbl) .eq. 'mode state') then
               read(ipden,err=900) m,s

            else if (rdtag(ilbl:jlbl) .eq. 'dicht3') then
               if (m .eq. nm .and. s .eq. ns) &
                  read(ipden,err=900) (avec(k),k=1,dim(m,s))

            else if (rdtag(ilbl:jlbl) .eq. 'spfs') then
               if (m .eq. nm .and. s .eq. ns) &
                  read(ipden,err=900) (psi(k),k=1,subdim(m)*dim(m,s))

            else if (rdtag(ilbl:ilbl+12) .eq. 'end data set') then
               exit

            endif
         enddo
      else
         read(ipden,end=77) s,m, (avec(k),k=1,dim(m,s))
         read(ipden,end=77) (psi(k),k=1,subdim(m)*dim(m,s))
         if( m .ne. nm  ) goto 10
         if( s .ne. ns ) goto 10
      endif

      if (lk .and. pmin.gt.pmax) then  ! if momentum spectrum is required
        dx = (ort(zort(nm)-1+subdim(m))-ort(zort(nm)))/subdim(m)
        pmin = -2.0_dop/dx      ! minimum momentum
        pmax =  2.0_dop/dx      ! maximum momentum
      endif
      
      m=nm
      s=ns
!-----------------------------------------------------------------------
! new data set
!-----------------------------------------------------------------------
      write(iaus,'(a,f8.2,a)') '# Time ',time,' fs'
      write(6,'(a,f8.2,a)') 'Writing reduced density at time: ',time,' fs'

!-----------------------------------------------------------------------
! Gnuplot lines
!-----------------------------------------------------------------------
      if (lgnu) then
         write(iaus,'(a,f8.2,a)') 'set title " Time:  ',time,'fs "'
         write(iaus,'(a)') 'set autoscale z'
         write(iaus,'(a)') 'set cntrparam levels discrete '//&
            '0.000, 0.050, 0.100, 0.150, 0.200, 0.250, 0.300, '//&
            '0.350, 0.400, 0.450, 0.500, 0.550, 0.600, 0.650, '//&
            '0.700, 0.750, 0.800, 0.850, 0.900, 0.950, 1.000 '
         write(iaus,'(a)') 'splot "-" notitle w l; pause -1 " "'
     endif

!-----------------------------------------------------------------------
! write values, continue or terminate the loop over data-sets.
!-----------------------------------------------------------------------

1111  FORMAT (5(F16.10,2X))
      do j=1,subdim(m)
        do j2=1,subdim(m)
          rhosub = (0.0_dop, 0.0_dop)
          do j3=1,dim(m,s)
            zeig = (j3-1)*subdim(m)
            rhosub = rhosub + avec(j3)*dconjg(psi(zeig+j))*psi(zeig+j2)
            
          enddo
          if (lk) then
            dx = ort(zort(nm)-1+j)-ort(zort(nm)-1+j2) ! redefine dx=x-x'
            do k=1,pts
              rhok(k)=rhok(k)+rhosub*weight(j)*weight(j2)*&
                    exp(dcmplx(0.0_dop,dx*(pmin+dp*dble(k-1))))
            enddo
          else
            rhosub = rhosub/weight(j)/weight(j2)
            write(iaus,1111) ort(zort(nm)-1+j),ort(zort(nm)-1+j2),&
              dreal(rhosub),dimag(rhosub),cdabs(rhosub)
          endif
        enddo
        if (.not.lk) then 
          write(iaus,*) " " 
        endif
      enddo
      if (lk) then
        do k=1,pts
          write(iaus,1111) pmin+(k-1)*dp,&
              dreal(rhok(k)),dimag(rhok(k)),cdabs(rhok(k))
        enddo
      endif

!-----------------------------------------------------------------------
! end data set
!-----------------------------------------------------------------------
      write(iaus,'(a)') 'e'

      time=time+dt

      do n=1,step-1
 12      read(ipden,end=77) s,m, (avec(k),k=1,dim(m,s))
         read(ipden,end=77) (psi(k),k=1,subdim(m)*dim(m,s))
         if( m .ne. nm ) goto 12
         if( s .ne. ns ) goto 12
         time=time+dt
      enddo
      if (time .gt. tfin) go to 77

      go to 10


 77   continue
      close(ipden)
      close(iaus)
      if( time .eq. tinit/fs ) then
         write(6,'(a,i3,a)') ' No data sets for mode',nm,&
                             ' on pdensity file.'
         stop
      endif

      deallocate(avec)

!.....call gnuplot     
      if(lagnu) then
         gnucmd = 'gnuplot -persist 2>/dev/null "'//trim(fileout)//'"'
         call excmd(gnucmd)
      endif 

      return
 900  write(6,'(a)') 'ERROR: error reading pdensity file'
      stop
 910  write(6,'(a)') &
            'ERROR: Output file already exists. Use -w to overwrite.'
      stop
      end subroutine subprhosub

!***********************************************************************
!-----------------------------------------------------------------------
! PRHOINPUT reads the options and the argument.
!-----------------------------------------------------------------------
      subroutine prhoinput

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: iarg,i,ierr
      character(len=80)       :: buf

!-----------------------------------------------------------------------
! provide default input and output filenames 
!-----------------------------------------------------------------------
      filein  = 'pdensity'
      operfile = 'oper'
      fileout = 'pdensity.xxt'
      operator = ' '
      unitlab(1)='a.u.'

!-----------------------------------------------------------------------
! set default flags
!-----------------------------------------------------------------------
      ns  = 1
      nm   = 1
      step   = 1
      tfin   = 1.0e+50_dop
      lnw    = .false.
      lk     = .false.
      pmin   =  9.9e29_dop
      pmax   = -9.9e29_dop
      pts    = 501

!-----------------------------------------------------------------------
! read options 
!-----------------------------------------------------------------------
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if(buf(1:2) .eq. '-i') then
            i = i+1
            call mygetarg(i,buf)
            filein = buf
         else if( buf(1:3) .eq. '-n ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,2a,/1x,3a)')&
       'Purpose: ',&
       'Build and output reduced density from pdensity file.',&
       'Usage  : ',&
       'prhosub [-i -w -n -nw -k -q -g -a -ver -h -?] [mode [state]]'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a,/,9x,a)') &
                     ' -i FILE : The one particle density is read '&
                    ,'from file FILE rather'&
                    ,'  than from ./pdensity'&
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(a)') ' -w      : overwrite enabled.'
            write(6,'(2a)') ' -n step : Compute the density', &
                               ' only every step-th output.'
            write(6,'(2a)') &
                     ' -nw     : No weights are employed.',&
                     ' Write the "naked" DVR populations.'
            write(6,'(2a,/11x,2a)') &
                     ' -k min max: Write the momentum spectrum for ',&
                     'the interval min < p < max.',&
                     'If "-k d" is given, default ',&
                     'values for min and max will be taken.'
            write(6,'(2a,/11x,a,i6,/11x,a)') &
                     ' -q pts  : Use the number of pts points ',&
                     'for representing the momentum',&
                     'spectrum. Default: pts =',pts,&
                     'This option is ignored, when -k is not given.' 
            write(6,'(2a)') &
              ' -g      : GNUplot command lines are written to ',&
              'the output file(s).'
            write(6,'(2a)') & 
                ' -a      : GNUplot is called automatically.',&
                ' (sets "-g").'
            write(6,'(a)')&
                     ' -ver    : Version information about the program.'
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(/2a/,2a)') ' The argument "mode" is the mode',&
                                 ' for which the reduced density is',&
                                 ' calculated. "state" is the',&
                                 ' electronic state.' 
            write(6,'(2a)') ' If an argument is not given, ',&
                 'the default value 1 will be taken.'  
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else if( buf .eq. '-w' ) then
            lovwr  = .true.   
         else if( buf(1:4) .eq. '-nw ' ) then
            lnw = .true.
         else if( buf(1:3) .eq. '-k ' ) then
            lk  = .true.
            i = i+1
            call mygetarg(i,buf)
            if(buf .ne. 'd') then
               read(buf,*,iostat=ierr) pmin
               if (ierr .ne. 0) then
                   write(6,'(2a/1x,a)') ' The first argument following',&
                          ' the option -k is invalid : ', buf
                  stop 1
               end if
               i=i+1
               call mygetarg(i,buf)
               read(buf,*,iostat=ierr) pmax
               if (ierr .ne. 0) then
                  write(6,'(2a/1x,a)') ' The second argument following',&
                 ' the option -k is invalid : ', buf
               endif
            endif

         else if( buf(1:3) .eq. '-q ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) pts
            if (ierr .ne. 0 .or. pts.le.1 ) then
                write(6,'(2a/1x,a)') ' The argument following the ',&
                  'option -p must be a positve integer. Not : ',buf
            endif 

         else if( buf(1:4) .eq. '-g ' ) then
            lgnu = .true.
         else if( buf(1:4) .eq. '-a ' ) then
            lgnu = .true.
            lagnu = .true.

         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      if (iarg .lt. i) goto 200
      
!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='prhosub.log',form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
! get compulsory argument.
!-----------------------------------------------------------------------
      if (iarg .ge. i) then
         call mygetarg(i,buf)
         read(buf,*,iostat=ierr) nm
         if(ierr.ne.0 .or. nm.le.0 ) then
            write(6,'(2a/,a)') ' The argument (mode)',&
                 ' must be a positive integer. Not : ', buf
            stop
         end if
      end if

      i=i+1
      if (iarg .ge. i) then
         call mygetarg(i,buf)
         read(buf,*,iostat=ierr) ns
         if(ierr.ne.0 .or. ns.le.0 ) then
            write(6,'(2a/,a)') ' The argument (state)',&
                 ' must be a positive integer. Not : ', buf
            stop
         end if
      end if
      
 200  continue

!-----------------------------------------------------------------------
! convert filenames to absolute
!-----------------------------------------------------------------------
      laenge=index(filein,' ')-1
      call abspath(filein,laenge)

      opflaenge=index(operfile,' ')-1
      call abspath(operfile,opflaenge)

      if (fileout .ne. 'no') then
         laeout=index(fileout,'  ')-1
         call abspath(fileout,laeout)
      end if

      return 
      end subroutine prhoinput

!#######################################################################

      end program prhosub


