!***********************************************************************
!
!                         AUSWERT subroutine PSI2EX
!
! Converts a psi file from an MCTDH to an exact representation
! GW 05/02
! FO 06/2013: convert incoming single-set WF to multi-set exact WF
!***********************************************************************

      module psi2exmod

use decimal, only: long,dop,sip

      real(dop), save       :: thrs
      logical(kind=4), save :: lsp,lerr,labs,labs2,lascii,lrst,linwf,lwght

      end module psi2exmod
      
      program psi2ex

use decimal, only: long,dop,sip
use global
use globmemmod
use aglobal
use versions
use dvrdatmod
use griddatmod
use memdimmod
use psidef
use channels
use datenmod
use compdatmod
use compdat1mod

use op1lib, only: cpvxz, initvxi
use mtlib, only: mtxxzz
use logdat
use compdat, only: savecsys,savecpsi,getcpsi
use iofile, only: rdpsiinfo,wrpsiinfo
use iogrddef, only: griddat
use iopsidef, only: psidat
use rddvrmod, only: dvrinfo
use einsbasmod, only: einexact
use auswutil, only: savecdat
use zeigauswmod, only: zeigausw
use iorst, only: rstinfo

use module_interface
use psi2exmod

      implicit none

      integer(long)        :: m,ilbl,chkdvr,chkgrd,chkpsi,chkdat,f,chkprp
      real(dop)            :: totmem

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      lsp=.true.
      lpop2=.false.
      lswap=.false.
      labs=.false.
      labs2=.false.
      lascii=.false.
      lwght=.false.
      lrst=.false.
      call psi2exinput

!-----------------------------------------------------------------------
! Files to be read
!-----------------------------------------------------------------------
      lrddvr=lpop2

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (ilog,file='psi2ex.log',form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      if (lrst) then
         call rdmemdim(irst)
      else
         call rdmemdim(ipsi)
      endif

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef
      call alloc_compdat

!-----------------------------------------------------------------------
! set no. of comparison data sets
!-----------------------------------------------------------------------
      ncomp=2

!-----------------------------------------------------------------------
! read data from psi file
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      if (lrst) then
         irst=ipsi
         chkprp=0
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         if (filever(irst) .lt. 11.0) read(irst,err=998) tinit
      else
         read(ipsi) filever(ipsi)
         call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      endif
      if (psitype .eq. 1) then
         routine='Psi2ex'
         message = 'Wavefunction already in exact representation'
         call errormsg
      endif
      if (chkerr(1) .ne. 0) go to 999

!-----------------------------------------------------------------------
! read data from dvr file
!-----------------------------------------------------------------------
      if(lpop2 .or. lort) then
         open(idvr,file=dname(1:dlaenge)//'/dvr',form='unformatted',&
             status='old',err=997)
         call dvrinfo(lerr,chkdvr)
         if (lerr) then 
            write(6,*) ' Error reading: dvr'
            stop
         end if
         close(idvr)
      end if

!-----------------------------------------------------------------------
! save data as set 1
!-----------------------------------------------------------------------
      call savecsys(1)
      call savecpsi(1)
      call savecdat(1)

!-----------------------------------------------------------------------
! set up and save exact psi information
!-----------------------------------------------------------------------
      psitype=1
      lpsiopt(1)=lsp
      lpsiopt(2)=.false.
      lpsiopt(3)=.false.
      lpsiopt(4)=.false.
      griddim=nstate
      do m=1,nmode
         griddim=griddim*subdim(m)
      enddo
      write(ilog,'(a,i10)') 'Total grid dimension:',griddim

! Electronic basis: for exact, we always use multi-set (see
! EINSBAS). If the incoming WF is single-set, fix it.
      if (feb.gt.0 .and. .not.lmult) then
         lmult = .true.
         leb = .false.
         nstate = gdim(feb)
         meb = dofspf(feb)
         if (nspfdof(meb).eq.1) then
            nmode = nmode-1
         else
            nspfdof(meb) = nspfdof(meb)-1
         endif
      endif
      call einexact(1)
      call einexact(2)

      call psidat

! complete spfdof information with koeff and eb dofs
      fkoe=ndof+1
      mkoe=nmode+1
      if(feb.gt.0) then
         meb=nmode+2 ! cf. HEINGABE
      endif
      dofspf(fkoe)=mkoe
      spfdof(1,mkoe)=fkoe
      if(feb.gt.0) then
         dofspf(feb)=meb
         spfdof(1,meb)=feb
         nspfdof(meb)=1
      endif

      call savecsys(2)
      call savecpsi(2)
      call savecdat(2)

!-----------------------------------------------------------------------
! write exact psi information to new file
!-----------------------------------------------------------------------
      if(.not.labs .and. .not.labs2 .and. .not.lascii) then 
         write(ipsi1) progver 
         call wrpsiinfo(ipsi1)
      endif

      write(ilog,'(a,/25i4)') 'Order of DOFs in exact mode:',&
                               (cspfdof(f,1,2),f=1,cnspfdof(1,2))
      write(ilog,*)

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
!-----------------------------------------------------------------------
      lpsi=.true.
      lpsigrd=.true.
      ltrafo=lpop2.or.lwght
      lfft=.false.
!     is any of the degrees of freedom of fft or expDVR type?
      do f=1,ndof
         if(basis(f).eq.4 .or. basis(f).eq.5) lfft=.true.
      end do
      lfft=lpop2.and.lfft  ! don't set lfft if not needed

      call zeigausw

!-----------------------------------------------------------------------
! assign memory arrays
!-----------------------------------------------------------------------
      call memausw
      mrweight=mrdim
      if(lwght .or. lpop2) then
         mrdim=mrweight+ortdim
      else
         mrdim=mrweight+1
      endif

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      workcdim=max(workcdim,2*griddim+cdgldim(1))

      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim 
      msdim   = msworks+worksdim

      mfworkf = mfdim 
      mfdim   = mfworkf+workfdim
      
!-----------------------------------------------------------------------
! call conversion routine
!-----------------------------------------------------------------------
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      totmem = 16.0_dop*dble(mcdim) + 8.0_dop*dble(mrdim) + 4.0_dop*dble(midim)&
             +  4.0_dop*dble(mldim) + 8.0_dop*dble(msdim) + 4.0_dop*dble(mfdim) 

      call wrversion(ilog)
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', totmem/1048576.0_dop,' MB'
      call myflush(ilog)

      if( abs(dble(ilbl)-totmem) .gt. 1.0_dop ) then
         routine='Psi2ex'
         write(message,'(3a,f9.1,a)')&
           'Integer overflow in memory assignment. ',&
           'You are asking for too much memory.    ',&
           'Total Memory asked for =',totmem/1048576.0_dop,' MB'  
         call errormsg
      endif
 
if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

!-----------------------------------------------------------------------
! recall psi data for length of arrays
!-----------------------------------------------------------------------
      call getcpsi(1)

      call subpsi2ex(mc(mcpsi),ms(mspsi),mc(mcworkc),mi(miworki),&
           mi(mijindx),cdgldim(2),mr(mrort),mr(mrweight),&
           mr(mrtrafo),mc(mcexphin),mc(mcrueck),mi(mifftfak))

deallocate(mc,mr,mi,ml,ms,mf)
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_psidef
      call dealloc_compdat

      stop
  997 write(6,'(a,/a)') 'Cannot open dvr-file:',dname(1:dlaenge)//'/dvr'
      stop 1
  998 continue
      write(6,'(a)') 'Error reading time from restart-file.'
      stop 1
  999 continue
      write(6,'(a)') 'Error reading psi-file.'
      stop
      contains

!***********************************************************************
!     calculates the exact wavefunction
!***********************************************************************

      subroutine subpsi2ex (psi,spsi,workc,worki,jindx,dgldim1,&
                            ort,weight,trafo,exphin,rueck,fftfak)

use compdat, only: getcpsi,getcsys
use rdfilesmod, only: rdpsi,rdpsigrid
use rddvrmod
use psiutil, only: showpop2, getweight
use psi2exmod

      implicit none

      integer(long)                                   :: i,d,iz,chkdvr
      integer(long)                                   :: m,n,f,ndof1,nmode1
      integer(long), dimension(maxdim)                :: igpoint
      integer(long), dimension(maxgdim)               :: findx
      integer(long), intent(in)                       :: dgldim1
      integer(long), dimension(1)                     :: idum
      integer(long), dimension(adim), intent(out)     :: jindx
      integer(long), dimension(workidim), intent(out) :: worki
      integer(long), dimension(fftdim), intent(out)   :: fftfak
      real(dop)                                       :: time,dt,now,spent
      real(dop), dimension(1)                         :: rdum
      real(dop), dimension(dvrdim), intent(out)       :: trafo
      real(dop), dimension(ortdim), intent(out)       :: ort,weight
      real(dop), external                             :: fcputime,fsystime
      complex(sip), dimension(spsidim), intent(out)   :: spsi
      complex(dop), dimension(dgldim1), intent(out)   :: psi
      complex(dop), dimension(workcdim), intent(out)  :: workc
      complex(dop), dimension(fftdim), intent(out)    :: rueck
      complex(dop), dimension(expdim), intent(out)    :: exphin
      complex(dop), dimension(1)                      :: zdum
      character(len=80)                               :: actual
      real(dop), save                                 :: previous = 0.0_dop

      integer(long)                                   :: countpsi
      character(len=c2)                               :: wrtag
      
!-----------------------------------------------------------------------
! read DVR data
!-----------------------------------------------------------------------
      if(lpop2 .or. lort) then
         chkdvr=1                ! dvrinfo read and stored in system data
         dvrdata(1) = lort       ! ort
         dvrdata(2) = ltrafo     ! trafo
         dvrdata(11)= lfft       ! exphin
         dvrdata(6) = lfft       ! rueck
         dvrdata(7) = lfft       ! fftfak
         call rddvr(ort,trafo,rdum,rdum,zdum,rueck,fftfak,exphin,&
                    zdum,idum,idum,rdum,chkdvr)
      endif

!-----------------------------------------------------------------------
! Determine DVR weights, if required. Note sqrt of weights is returned
!-----------------------------------------------------------------------
      if(lwght .or. lpop2) call getweight(trafo,weight,ort,.false.)

!-----------------------------------------------------------------------
! calculate time step for data
!-----------------------------------------------------------------------
      call getcsys(1)
      call getcpsi(1)

      iz = 0
      dt   = out2/fs*step
      time = tinit/fs
! --- skip the first nskip WFs
      do i=1,nskip
         call rdpsi(ipsi,psi,spsi,jindx,zdum,idum,zdum,idum)
         if (lend) goto 77
         time=time + out2/fs
      enddo

      countpsi=0
   10 continue

      if(lpop2) then
         call rdpsigrid2(ipsi,psi,spsi,jindx,workc,&
                         trafo,exphin,rueck,fftfak,lrst)
      else
         call rdpsigrid(ipsi,psi,spsi,jindx,workc,lrst)
      end if

      if (lend) goto 77

! determine actual time and CPU-time spent since last call
      call fzeit(actual)
      i = index(actual,':')
      actual=actual(i-9:i+6)
      now=fcputime(0)+fsystime(0)
      spent=now-previous
      previous=now

      write(ilog,'(a,f8.3,a,f15.1,f15.2,4x,a16)') &
           'Writing WF : ',time,' fs, ',now,spent,actual
      call myflush(ilog)
      call getcsys(2)
      call getcpsi(2)
! --- Handle multi-set electronic states
      nmode1=nmode
      if(nstate.gt.1 .and. lort) nmode1=nmode+1

! "coefficients"
      do d=1,adim
         workc(d)=1.0_dop
      enddo
      call cpvxz(psi,workc(adim+1),griddim)
      if(lascii) then
         write(ipsi1,'(1x/,a,f9.4,/)') '# Time =', time
         if(lort) then
               i=0
               do m=1,nmode1
                  do n=1,nspfdof(m)
                     f=spfdof(n,m)
                     i=i+1
                     findx(i) = f
                  enddo
               enddo
               ndof1=i  ! this is necessary to remove dummy DOFs
            if(labs2) then
               write(ipsi1,'(a,3x,a,12x,55(a12,2x))') &
                   '#',' abs2(WF)',(modelabel(findx(i)),i=1,ndof1)
            elseif(labs) then
               write(ipsi1,'(a,4x,a,12x,55(a12,2x))') &
                   '#',' abs(WF)',(modelabel(findx(i)),i=1,ndof1)
            else
               write(ipsi1,'(a,6x,a,12x,55(a12,2x))') &
                   '#    Re(WF)','Im(WF)',(modelabel(findx(i)),i=1,ndof1)
            endif

            call initvxi(igpoint,maxdim) ! Set igpoint(i)=1
            igpoint(1)=0
            do d=adim+1,dgldim
! ---          determine set of grid-points
               i=0
               do m=1,nmode1
                  do n=1,nspfdof(m)
                     f=spfdof(n,m)
                     i=i+1
                     if (igpoint(i).lt.gdim(f)) then
                        igpoint(i)=igpoint(i)+1
                        goto 15
                     else
                        igpoint(i)=1
                     endif
                  enddo     ! n=1,nspfdof(m)
               enddo        ! m=1,nmode1
  15           continue
! ---          remove weights from WF if required
               if(lwght) then   ! if(findx(i).ne.feb) then
                  do i=1,ndof1
                     if(findx(i).ne.feb) then
                        workc(d) = workc(d)/weight(zort(findx(i))+igpoint(i)-1)
                     endif
                  enddo
               endif
! ---          write WF and grid points
               if(cdabs(workc(d)) .gt. thrs) then
                  if(labs2) then
                     write(ipsi1,995) cdabs(workc(d))**2, &
                       (ort(zort(findx(i))+igpoint(i)-1),i=1,ndof1)
                  elseif(labs) then
                     write(ipsi1,995) cdabs(workc(d)), &
                       (ort(zort(findx(i))+igpoint(i)-1),i=1,ndof1)
                  else
                     write(ipsi1,996) workc(d), &
                       (ort(zort(findx(i))+igpoint(i)-1),i=1,ndof1)
                  endif
               endif  !  .gt. thr
            enddo     !  d=adim+1,dgldim
         else         ! not lort
            if(labs2) then
               write(ipsi1,991) (cdabs(workc(d))**2,d=adim+1,dgldim)
            elseif(labs) then
               write(ipsi1,991) (cdabs(workc(d)),d=adim+1,dgldim)
            else
               write(ipsi1,992) (workc(d), d=adim+1,dgldim)
            endif
         endif
      else    ! not ascii
         countpsi=countpsi+1

         write(wrtag,'(a9,i10,a1)') '<data set',countpsi,'>'
         write(ipsi1) wrtag
         wrtag='<psi>'
         write(ipsi1) wrtag

         if(labs2) then
            if (lsp) then
               write(ipsi1) (real(cdabs(workc(d))**2),d=adim+1,dgldim)
            else
               write(ipsi1) (cdabs(workc(d))**2, d=adim+1,dgldim)
            endif
         elseif(labs) then
            if (lsp) then
               write(ipsi1) (real(cdabs(workc(d))), d=adim+1,dgldim)
            else
               write(ipsi1) (cdabs(workc(d)), d=adim+1,dgldim)
            endif
         else
            if (lsp) then
               write(ipsi1) (cmplx(workc(d)), d = 1,dgldim)
            else
               write(ipsi1) (workc(d), d = 1,dgldim)
            endif
         endif

         write(wrtag,'(a13,i10,a1)') '<end data set',countpsi,'>'
         write(ipsi1) wrtag

      endif

      time=time + dt
      iz=iz+1
      if(lrst) goto 77
      if(iz.ge.ncut) goto 77

      call getcsys(1)
      call getcpsi(1)

!-----------------------------------------------------------------------
! if more data, go to beginning of loop
!-----------------------------------------------------------------------
      do i=2,step
         call rdpsi(ipsi,psi,spsi,jindx,zdum,idum,zdum,idum)
         if (lend) goto 77
      enddo

      goto 10

 77   continue
      close(ipsi)
      close(ipsi1)

      return
 991  format(6f13.8)
 992  format(3(2f13.8,2x))
 995  format(f17.12,1x,33f14.5)
 996  format(2f14.8,2x,33f14.5)
      end subroutine subpsi2ex

!-----------------------------------------------------------------------
!***********************************************************************
!
! PSI2EXINPUT
!
! get input parameters and file names for calculation
!
!-----------------------------------------------------------------------

      subroutine psi2exinput
      
use global
use aglobal
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use compdat1mod
use psi2exmod

      implicit none

      integer(long), external         :: myiargc
      integer(long)                   :: iarg,ierr,i
      character(len=80)               :: buf
      character(len=c5)               :: filename
      character(len=12)               :: oformat
      logical(kind=4)                 :: lfileout

!-----------------------------------------------------------------------

      nskip=0
      ncut = 999999
      thrs=-1.d0
      filein  = './'
      fileout  = 'psi.ex'
      lfileout=.false.
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:3) .eq. '-f ' .or.  buf(1:3) .eq. '-i ') then
            i = i+1
            call mygetarg(i,buf)
            filein = buf
         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
            lfileout=.true.
         else if( buf(1:5) .eq. '-stp ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:6) .eq. '-thrs ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) thrs
            if (ierr .ne. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -thrs',&
                 ' must be a real number. Not : ', buf
               stop
            end if
         else if( buf(1:3) .eq. '-n ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ncut
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -n'
               stop
            endif
         else if( buf(1:5) .eq. '-skip ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nskip
            if (nskip .lt. 0) then
               write(6,'(/a)')&
               'Argument of -skip must be positive integer'
               stop
            endif
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -skip'
               stop
            endif
         else if( buf(1:3) .eq. '-w ' ) then
            lovwr=.true.
         else if( buf(1:4) .eq. '-dp ' ) then
            lsp=.false.
         else if( buf(1:6) .eq. '-pop2 ' ) then
            lpop2=.true.
         else if( buf(1:6) .eq. '-plot ' ) then
            lswap=.true.
         else if( buf(1:6) .eq. '-abs  ' ) then
            labs=.true.
         else if( buf(1:6) .eq. '-abs2 ' ) then
            labs2=.true.
         else if( buf(1:4) .eq. '-W  ' ) then
            lwght=.true.
         else if( buf(1:6) .eq. '-ort  ' ) then
            lort=.true.
            lascii=.true.
         else if( buf(1:6) .eq. '-ascii' ) then
            lascii=.true.
         else if( buf(1:6) .eq. '-rst  ' ) then
            lrst=.true.
         else if( buf(1:3) .eq. '-h ' .or. buf(1:3) .eq. '-? ' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,a,7(/10x,a))') &
         'Purpose: Converts the wavefunction in a psi file from MCTDH ', &
         'to numerically exact form. Default is single precision.', &
         'To inspect the WF, the output may also be in ASCII fromat', &
         'with or without printing coordinates (option -ort).',&
         'Default is showing the "naked" DVR populations, but',&
         'DVR-weights can be removed from the WF by setting the ',&
         'option -W (requires that -ort is also set). NB: -W does',&
         'the opposite of -nw used in other analysis routines.'

            write(6,'(/1x,2a,/19x,a)') 'Usage  : psi2ex ', &
              '[-f -i -o -stp -n -skip -thrs -ort -ascii -W -pop2',&
              ' -plot -abs -abs2 -rst -w -dp -ver -h -?]'

            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a)') &
                     ' -f FILE : The wavefunction is read '&
                    ,'from file FILE rather than from ./psi'&
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(a)') ' -i FILE : Same as -f FILE.'
            write(6,'(2a,/,11x,a)') &
                     ' -o FILE : The output is written to file ',&
                     'FILE rather than to ./psi.ex',&
             'The string FILE may be a relative or a full path-name.'
            write(6,'(2a)') &
                     ' -rst    : A restart file rather than a psi file is read', &
                     ' as input.'
            write(6,'(2a)') ' -stp step: Compute the exact WF', &
                            ' only every step-th output.'
            write(6,'(2a)')&
                     ' -n num  : only the first num WFs are read '&
                    ,' from psi-file.'
            write(6,'(a)')&
                     ' -skip m : the first m WFs are skipped.'
            write(6,'(a)')&
                     ' -pop2   : exact WF is given in second population'
            write(6,'(a,/a,/a,/a,/a,/a)')&
            ' -plot   : only meaningful if -pop2 is activated',&
            '           and some dof is of fft type. ',&
            '           For each dof of fft type, the x-axis is',&
            '           swapped in a convenient way for further',&
            '           plotting. NB: the resulting WF cannot be',&
            '           used in a subsequent MCTDH calculation.'
            write(6,'(2a,/11x,a)') &
            ' -ort    : The WF and their coordinates are written to', &
                       ' output file.', &
                       'Note that -ort sets -ascii.'
            write(6,'(a,/11x,a,/11x,a)') &
            ' -W      : The DVR-weights are removed from the WF.',&
                      'Useful for plotting the WF.',&
                      '-W requires that -ort is set, otherwise ignored.'
            write(6,'(a,/11x,a)') &
            ' -thrs thrs : The WF and their coordinates are written ', &
                    'only if(abs(WF).GT.thrs). Works only with -ort.'
            write(6,'(2a,/11x,a,/11x,a)') &
            ' -ascii  : The output file is in ascii rather than binary', &
                       ' format.', &
                       'In this case the (dummy) A-vector and psiinfo ', &
                 'are not written, i.e. the file contains the WF only.'
            write(6,'(a,/11x,a,/11x,a)') &
            ' -abs    : The absolute value of the exact WF is written.', &
                       'In this case the (dummy) A-vector and psiinfo ', &
                 'are not written, i.e. the file contains abs(WF) only.'
            write(6,'(a,/11x,a)') &
            ' -abs2   : Similar to -abs, but the absolute value squared' &
                       ,'of the exact WF is written.' 
            write(6,'(2a)') &
                     ' -dp     : The file is written double precision.'
            write(6,'(2a)') &
                     ' -w      : An existing output file is ',&
                     'overwritten.'
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      else
         write(6,'(2a)') ' unknown argument : ', buf
         stop
      end if
      
 200  continue
      if(lwght.and. .not.lort) write(6,'(a)') &
       'WARNING: Option -W is ignored because -ort is not set!'
      lwght=lwght.and.lort
      lswap=lswap.or.(labs.and.lpop2).or.(labs2.and.lpop2)
! --- If lswap is true, swapfft will be called in showpop2.
      laein=index(filein,' ')-1
      if(laein.gt.6) then
         if (filein(laein-6:laein) .eq. 'restart' ) lrst = .true.
         if (filein(laein-2:laein) .eq. 'psi' )     lrst = .false.
      endif
      if (filein(laein:laein) .eq. '/') then
         if(lrst) then
            irst = ipsi
            filein = filein(1:laein)//'restart'
            laein  = laein + 7
         else
            filein = filein(1:laein)//'psi'
            laein  = laein + 3
         endif
      end if
      call abspath(filein,laein)

      laeout=index(fileout,'  ')-1
      call abspath(fileout,laeout)
      if(lpop2 .and. .not.lfileout) then
         fileout=fileout(1:laeout)//'.pop2'
         laeout=laeout+5
      endif
      if(labs .and. .not.lfileout) then
         fileout=fileout(1:laeout)//'.abs'
         laeout=laeout+4
      endif
      if(labs2 .and. .not.lfileout) then
         fileout=fileout(1:laeout)//'.abs2'
         laeout=laeout+5
      endif

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      write(6,'(80a)') ('*',i=1,80)
      filename=filein(1:laein)
      open(ipsi,file=filename,form='unformatted',status='old',&
           err=1000)
      write(6,'(2a)') ' The inputfile is  : ',filein(1:laein)

      oformat='unformatted'
      if(lascii) oformat='formatted'

      filename=fileout(1:laeout)
      if (lovwr) then
         open (ipsi1,file=filename,form=oformat,status='unknown',&
                 err=1000)
      else
         open (ipsi1,file=filename,form=oformat,status='new',&
                 err=1010)
      endif
      write(6,'(2a)') ' The outputfile is : ',fileout(1:laeout)
      write(6,'(2a)') ' The log file is   :  ./psi2ex.log'

      return 
      
 1000 write(6,*)' Error opening file : '
      write(6,*) filename
      stop
 1010 write(6,*)' File already exists: '
      write(6,*) filename
      stop
      
      end subroutine psi2exinput


!***********************************************************************
!
!                RDPSIGRID2
!
! Called by the various ANALYSE programs in order to read the
! psi file data, and transform the wavefunction to the full grid
! (primitive basis) representation
! 
! As opposed to its sister routine rdpsigrid, rdpsigrid2 returns
! the wave function to the full grid IN SECOND POPULATION
!
! OV 12/05
!***********************************************************************

      subroutine rdpsigrid2(unit,psi,spsi,jindx,workc,&
                            trafo,exphin,rueck,fftfak,lrst)

use global
use aglobal
use dvrdatmod
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use compdat1mod

use psiutil, only: showpop2
use rdfilesmod, only: rdpsi
use iorst, only: rdrstpsi

      implicit none

      integer(long)                              :: zeig1,zeig2,zgrd,f,m,s,&
                                                    block1,swapzeig,vdim1,i
      integer(long), intent(in)                  :: unit
      integer(long), dimension(1)                :: idum
      integer(long), dimension(adim), intent(out):: jindx
      integer(long), dimension(*), intent(in)    :: fftfak
      real(dop), dimension(dvrdim), intent(in)   :: trafo
      complex(sip), dimension(dgldim), intent(out) :: spsi
      complex(dop), dimension(dgldim), intent(out) :: psi
      complex(dop), dimension(dgldim+2*(griddim/nstate)), intent(inout) :: workc
      complex(dop), dimension(1)                   :: zdum
      complex(dop), dimension(fftdim), intent(in)  :: rueck
      complex(dop), dimension(expdim), intent(in)  :: exphin
      real(dop)                                :: rdum
      logical(kind=4)                          :: lrst

      if(dgldim+2*(griddim/nstate).gt.workcdim) then
         routine='rdpsigrid2'
         message='workcdim is too small'
         write(6,*) 'workcdim =                 ', workcdim
         write(6,*) 'dgldim+2*(griddim/nstate) =',&
                     dgldim+2*(griddim/nstate)
         call errormsg
      endif 
      zeig1=dgldim+1
      zeig2=zeig1+griddim/nstate

! read mctdh-wfn into array psi or spsi
      if(lrst) then
         if (filever(irst) .lt. 11.0) then
            read(irst,err=999) (psi(i),i=1,dgldim)
         else
            call rdrstpsi(rdum,psi,jindx,zdum,idum)
         endif
      else
         call rdpsi(unit,psi,spsi,jindx,zdum,idum,zdum,idum)
      endif

! transform mctdh-wfn to second population
!     each time showpop2 is called it transforms a single
!     mode, then, loop through all modes
!     careful, the last arguments in showpop2 are DOFs, not modes!
!     The bases phifbr, el, sphfbr, k, and external are not transformed.
      do f=1,ndof
         if(basis(f).gt.0 .and. basis(f).ne.6 .and.&
            basis(f).ne.8 .and. basis(f).ne.13) &
            call showpop2(psi,trafo,exphin,rueck,fftfak,f,f)
      end do

! copy the transformed wavefunction into array workc
      call cpvxz(psi(1),workc(1),dgldim)
      
! psi wavefunction is written to the primitive grid
      zgrd=1
      do s=1,nstate
         call cpvxz(workc(zpsi(s)),workc(zeig2),block(s))

         vdim1=1
         do m=1,nmode
            swapzeig=zeig1
            zeig1=zeig2
            zeig2=swapzeig
            call mtxxzz (workc(zetf(m,s)),workc(zeig1),&
                 workc(zeig2),vdim1,dim(m,s),ndim(m,s),subdim(m))
            block1=vdim1*subdim(m)*ndim(m,s)
            vdim1=vdim1*subdim(m)
         enddo

         call cpvxz(workc(zeig2),psi(zgrd),griddim/nstate)
         zgrd=zgrd+griddim/nstate
      enddo

      return
 999  write(6,*) ' ERROR reading restart file (-rst)'
      stop
      end subroutine rdpsigrid2


      end program psi2ex

