!***********************************************************************
!
!                      AUSWERT subroutine PSI2RST
!
! Converts a MCTDH psi file to a restart file
! HDM 12/03
!***********************************************************************
      
      program psi2rst

use decimal, only: long,dop,sip
use global
use globmemmod
use aglobal
use versions
use dvrdatmod
use griddatmod
use operdef
use psidef
use channels
use datenmod
use memdimmod

use iofile, only: rdpsiinfo
use zeigauswmod, only: zeigausw

use module_interface

      implicit none

      integer(long)                        :: m,ilbl,ll,n,s,chkdvr,chkgrd,&
                                              chkpsi,chkdat
      character(len=c5)                    :: mdlbl,nspf,message

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      call psi2rstinput

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (ilog,file='psi2rst.log',form='formatted')
      logisopen=.true.

      call rdmemdim(ipsi)

      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      call alloc_psidef

!-----------------------------------------------------------------------
! read data from psi file
!-----------------------------------------------------------------------
      read(ipsi) filever(ipsi)
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      if (chkerr(1) .ne. 0) go to 999

!-----------------------------------------------------------------------
! write mode information.
!-----------------------------------------------------------------------
      write(6,'(/,a)') ' mode: No. of SPFs per state : modelabels'
      do m=1,nmode
         ll = 1
         mdlbl = '                                                    '
         nspf  = '                                                    '
         do  n=1,nspfdof(m)
            ilbl  = index(modelabel(spfdof(n,m)),' ')
            mdlbl = mdlbl(1:ll)//modelabel(spfdof(n,m))(1:ilbl)
            ll    = ll + ilbl
         end do
         write(nspf,'(95i3)') (dim(m,s),s=1,nstate)
         if(3*nstate+ll .le. 72 ) then
            write(6,'(i4,4a)') m,' :',nspf(1:3*nstate),' : ',&
                               mdlbl(1:ll)
         else
            write(6,'(i4,3a)') m,' :',nspf(1:3*nstate),' :'
            write(6,'(a,/)') mdlbl(1:ll)
         end if
      enddo

      write(6,'(/,a,f11.3,a,f11.3,a)') &
        ' tinit =', tinit/fs, 'fs,    tpsi =', out2/fs, 'fs' 

!-----------------------------------------------------------------------
! set up and save exact psi information
!-----------------------------------------------------------------------

      if (lpsiopt(2)) then
         write(6,'(a)') 'Cannot handle WF in compact format'
      endif
      if (lpsiopt(4)) then
         write(6,'(a)') 'Cannot handle WF in selected CI format'
      endif

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
!-----------------------------------------------------------------------
      lpsi=.true.
      lpsigrd=.false.

      call zeigausw

!-----------------------------------------------------------------------
! assign memory arrays
!-----------------------------------------------------------------------
      call memausw

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      workcdim=max(workcdim,dgldim)

      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim 
      msdim   = msworks+worksdim

      mfworkf = mfdim 
      mfdim   = mfworkf+workfdim
      
!-----------------------------------------------------------------------
! call conversion routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)
if(allocated(agmat))deallocate(agmat)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))
allocate(agmat(agmatdim))

      call subpsi2rst(mc(mcpsi),ms(mspsi),mc(mcworkc),mi(miworki),&
           mi(mijindx),dgldim,agmat,mi(mitrajst),mc(mcadgwp),mi(migwpdep))

deallocate(mc,mr,mi,ml,ms,mf,agmat)


      stop
  999 continue
      message= 'Error reading psi-file.'
      write(6,'(a)') message
      stop
      contains


!***********************************************************************
!     writes the new restart file 
!***********************************************************************

      subroutine subpsi2rst (psi,spsi,workc,worki,jindx,dgldim1,agmat,&
                 trajst,adgwp,gwpdep)

use global
use aglobal
use griddatmod
use psidef
use channels
use datenmod

use rdfilesmod, only: rdpsi
use iorst, only: wrrst,rstinfo

      implicit none

      integer(long)                                    :: i
      integer(long), intent(in)                        :: dgldim1
      integer(long), dimension(workidim), intent(in)   :: worki
      integer(long), dimension(adim), intent(out)      :: jindx
      integer(long), dimension(gwpinfodim),intent(out) :: trajst,gwpdep
      complex(sip), dimension(spsidim), intent(out)    :: spsi
      complex(dop), dimension(dgldim1), intent(out)    :: psi
      complex(dop), dimension(workcdim), intent(out)   :: workc
      complex(dop), dimension(agmatdim), intent(out)   :: agmat
      complex(dop), dimension(gsmatdim), intent(out)   :: adgwp
      real(dop)                                        :: time
      real(dop), dimension(1)                          :: rdum
      real(dop), dimension(2)                          :: beta
      real(dop), dimension(3)                          :: alpha
      logical(kind=4)                                  :: linwf

      alpha(1) = 0.0_dop
      alpha(2) = 0.0_dop
      beta(1)  = 0.0_dop
      beta(2)  = 0.0_dop
!-----------------------------------------------------------------------
! calculate time step for data
!-----------------------------------------------------------------------

      time   = tinit - out2
      linwf=.true. 

      do i=1,step
         time = time + out2
         call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
         if (lend) then
            write(6,'(/a,i4)') ' End of file found (WF). step =',i
            write(6,'(a,i4,a,f8.3,a)') ' There are',i-1,&
                ' WFs on file. Last time =', (time-out2)/fs,' fs'
            close(ipsi)
            goto 77
         endif
      enddo
      close(ipsi)

      write(ilog,'(a,f8.3)') ' Writing wavefunction at time : ',time/fs
      write(6,'(/a,f8.3)')   ' Writing wavefunction at time : ',time/fs
!.....Opening (for trial) the restart file).
      if (lovwr) then
         open (irst,file=fileout(1:laeout),form='unformatted',&
                 status='unknown',err=1000)
      else
         open (irst,file=fileout(1:laeout),form='unformatted',&
                 status='new',err=1010)
      endif
      write(6,'(2a)') ' The outputfile is : ',fileout(1:laeout)
      close(irst)

      rdum(1)=0.0_dop
      call wrrst(time,psi,linwf,workc,1,alpha,beta,jindx,agmat,trajst)

      close(irst)     
 
 77   write(6,'(80a)') ('-',i=1,80)

      return

 1000 write(6,*)' Error opening file : '
      write(6,*) fileout(1:laeout)
      stop
 1010 write(6,*)' File already exists (use -w): '
      write(6,*) fileout(1:laeout)
      stop

      end subroutine subpsi2rst

!-----------------------------------------------------------------------
!***********************************************************************
!
! PSI2RSTINPUT
!
! get input parameters and file names for calculation
!
!-----------------------------------------------------------------------
      subroutine psi2rstinput

use global
use aglobal
use griddatmod
use psidef
use channels
use datenmod

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: iarg,ierr,i
      character(len=80)       :: buf
      character(len=c5)       :: filename
!-----------------------------------------------------------------------

      filein  = 'psi'
      fileout = '.'
      name = '.'
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:3) .eq. '-i ') then
            i = i+1
            call mygetarg(i,buf)
            laein=index(buf,' ')-1
            filein = buf(1:laein)//'/psi'
         else if( buf(1:3) .eq. '-f ') then
            i = i+1
            call mygetarg(i,buf)
            laein=index(buf,' ')-1
            filein = buf(1:laein)
         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf(1:3) .eq. '-w ' ) then
            lovwr=.true.
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,2a,/10x,2a)')&
                 'Purpose: Converts the wavefunction of a psi file to ',&
                 'restart format.','The argument "step" is ',&
                 'the number of the WF to be converted.'
            write(6,'(/1x,a)')&
       'Usage  : psi2rst [-f -i -w  -ver -h -?]  step'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/10x,a)') &
                     ' -f FILE: The wavefunction is read ',&
                     'from file FILE rather than from ./psi',&
              'The string FILE may be a relative or a full path-name.'
            write(6,'(2a)') ' -i DIR : The wavefunction is read ',&
                     'from file DIR/psi rather than from ./psi'
            write(6,'(2a)') &
                     ' -o DIR : The output is written to file ',&
                     'DIR/restart rather than to ./restart'
            write(6,'(2a)') &
                     ' -w     : An existing output file is ',&
                     'overwritten.'
            write(6,'(a)')&
                     ' -ver   : Version information about the program '
            write(6,'(a,/a)') &
                     ' -h     : Print this help text. ',&
                     ' -?     : Print this help text. '
            write(6,'(/2a,/5x,a)')' NB: The output is written to ',&
                     'the file restart in the current directory,',&
                     'or to DIR/restart, if the option -o is given.'

            write(6,'(5x,2a,/5x,2a)') 'To receive some information ',&
                 'on the WF file, run "psi2rst nn", where',&
                 'nn is zero or a large number ',&
                 '(larger than the number of WFs on file).' 
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      
      read(buf,*,iostat=ierr) step
      if (ierr .ne. 0) then
         write(6,'(a)')&
              'The argument (step) must be an integer. Not :'
         write(6,'(a)') buf(1:60)
         stop
      endif
      goto 220

!-----------------------------------------------------------------------
!  Prompting for missing input
!-----------------------------------------------------------------------
 200  write(6,'(a)') ' Give the number of the WF to be converted'
      read(5,*,iostat=ierr) step
      if (ierr .ne. 0) then
         write(6,'(a)')&
              'The argument (step) must be an integer.'
         goto 200
      endif

 220  if( step .le. 0 ) step = 999999
!-----------------------------------------------------------------------
! Making path-names absolute 
!-----------------------------------------------------------------------
      laein=index(filein,' ')-1
      if (filein(laein:laein) .eq. '/') then 
         filein = filein(1:laein)//'psi'
         laein = laein + 3
      end if
      call abspath(filein,laein)

      laenge=index(name,' ')-1
      call abspath(name,laenge)   ! will be used in wrrst

      fileout=name(1:laenge)//'/restart'
      laeout=laenge+8

!-----------------------------------------------------------------------
!  Opening the psi file.
!-----------------------------------------------------------------------
      write(6,'(80a)') ('*',i=1,80)
      filename=filein(1:laein)
      open(ipsi,file=filename,form='unformatted',status='old',&
           err=1000)
      write(6,'(2a)') ' The inputfile is : ',filein(1:laein)

      return 
      
 1000 write(6,*)' Error opening file : '
      write(6,*) filename
      stop
      
      end subroutine psi2rstinput

      end program psi2rst

