"""
Simple Python script to test out new SQL schema
version 2
In this case one entry per point
Number of columns will vary depending on number of atoms
Takes version 1, makes version 2. Otherwise it will crash
"""

# some useful sql queries
# get number of columns on a table
# select count(*) from pragma_table_info('tablename')
# get table schema
# "SELECT sql FROM sqlite_schema WHERE name='tablename';"
# get info on tables in db
# "SELECT * FROM sqlite_master WHERE type='table';"

# import sqlite3
import argparse  # import for parsing of command line options
import python_sqlite as pysql

# import os   #for file operations


def parsing():
    """
    Function create the cmd line argument parsing
    returns the parsing function
    would make sense to go in its own library separate to gui things
    """

    # the - means optional argument, no dash means must be given and error is raised if not
    parse = argparse.ArgumentParser(
        description="Purpose: \
                                    Create inputs for Quantics calculations"
    )
    parse.add_argument(
        "-DB",
        metavar="FILE",
        type=str,
        default="database.sql",
        help="Path to find existing DB. Absolute",
    )
    parse.add_argument("-natm", type=int, help="Number of atoms")
    parse.add_argument(
        "-NDB",
        metavar="nFILE",
        type=str,
        default="ndatabase.sql",
        help="Path to create existing DB. Absolute and must not exist",
    )
    # parse.add_argument('-nst',
    #                     type=int,
    #                     help='Number of steps')
    # parse.add_argument('-si',
    #                     type=float,
    #                     help='Size of steps')
    return parse


if __name__ == "__main__":
    # if run on its own
    # can take CLI arguments with argparse

    parser = parsing()
    parsed_args = parser.parse_args()
    filename = parsed_args.DB
    nfilename = parsed_args.NDB

    lmoco = True  # are moco needed
    tmoco = False  # do moco table in db or as external file

    print(f"Connecting to {filename}")
    # Connect to sql database and get cursor
    _,con, cursor = pysql.connect_db(filename,0)

    # Get number of entries
    dbcount = pysql.select_existing(cursor, "SELECT count(id) FROM dbentry;")[0][0]
    print(dbcount)

    # get info from refdb
    natom, nstate, nbasis = pysql.select_existing(
        cursor, "SELECT Natm,Nroot, Nbasis FROM refdb"
    )[0]
    # set coord and norm mode info
    ncoords = natom * 3  # cartesians
    nmodes = ncoords - 6  # number of modes, assume non-linear
    # get state info
    nstate_sq = nstate * nstate
    state_lab = []
    for state in range(1, nstate + 1):
        state_lab.append(state)

    print(
        f"There are {natom} atoms, {ncoords} coordinates and {nmodes} modes"
    )

    # Get list of tables
    print(
        pysql.select_existing(
            cursor, "SELECT name FROM sqlite_master WHERE type='table';"
        )
    )

    # make new database
    print(f"Connecting to {nfilename} as new database")

    _,con_new, cursor_new = pysql.connect_db(nfilename,1)


    # CREATE TABLES

    # create dbentry
    # Contains the primary index
    # This might not be needed anymore?
    cursor_new.execute(
        "CREATE TABLE dbentry (\
                        id INTEGER PRIMARY KEY,\
                        status INTEGER NOT NULL,\
                        created INTEGER NOT NULL );"
    )

    # create versions
    # Contains version info for quantics and SQL
    # Size, single row
    # quanticsversion - which version of quantics is in use
    # dbversion - which version of the database is in use
    # dbentry - current entry in db
    cursor_new.execute(
        "CREATE TABLE versions (quanticsversion INTEGER NOT NULL,\
                         dbversion INTEGER NOT NULL,\
                        dbentry INTEGER NOT NULL);"
    )

    # create query string to insert versions
    sql_string = (
        "INSERT INTO versions(quanticsversion, dbversion, dbentry) values(1000, 2, 0);"
    )
    # apply SQL query
    cursor_new.execute(sql_string)

    # Read in template file

    with open("template.dat", "r") as infile:
        temp = infile.read()

    res = cursor_new.execute(
        "CREATE TABLE template (filename TEXT NOT NULL,\
                       contents TEXT NOT NULL);"
    )
    print(res)
    cursor_new.execute("SELECT name FROM sqlite_master")
    print(cursor_new.fetchone())
    sql_string = "LOAD DATA INFILE 'template.dat' INTO TABLE template; "
    sql_string = ".import template.dat template"

    # Use the quote and string symbol to make sure the string characters are escaped
    sql_string = "INSERT INTO template(filename, contents) values ('template.dat',quote('{0}'));".format(
        temp
    )
    cursor_new.execute(sql_string)

    # cursor_new.execute(
    #     "CREATE TABLE nmfile (filename TEXT NOT NULL, contents TEXT NOT NULL)"
    # )
    # nmfile = "ethanal_cas_14_13_freq_2.out"
    # with open(nmfile, "r") as infile:
    #     temp = infile.read()

    # sql_string = 'INSERT INTO nmfile(filename, contents) values (quote("{0}"),quote("{1}"));'.format(
    #     nmfile, temp
    # )
    # print(sql_string)
    # cursor_new.execute(sql_string)
    con_new.commit()
    # create refdb
    # refdb has key info on the database and calculation
    # SAME IN OLD AND NEW FORMAT!
    # Size, only a single entry
    # Natm - number of atoms in molecule
    # Nroot - number of states
    # Nddstate - number of states
    # Nbasis - number of basis functions in CAS
    # E0 - the zero of the energy scale
    # Dimref -
    # Nfam -
    # Dercpdim - Number of derivative coupling dimensions
    # DBpntgp - point group used in calculation
    # Nsynop - number of symmetry operations
    cursor_new.execute(
        "CREATE TABLE refdb (Natm INTEGER NOT NULL,\
                        Nroot INTEGER NOT NULL,\
                        Nddstate INTEGER NOT NULL,\
                        Nbasis INTEGER,\
                        E0 DOUBLE,\
                        Dimref INTEGER,\
                        Nfam INTEGER,\
                        Dercpdim INTEGER,\
                        DBpntgp INTEGER,\
                        Nsymop INTEGER);"
    )
    # In this case load in old refdb
    sql_string = "SELECT * FROM refdb"
    cursor.execute(sql_string)
    result = cursor.fetchall()
    for entry in result:
        print(entry)
        sql_string = "INSERT INTO refdb values{0}".format(entry)
        cursor_new.execute(sql_string)

    # create refdbrefgeom
    # This contains the reference geometry of the molecule
    # SAME IN OLD AND NEW FORMAT!
    # Size, each row is an atom
    # mass - mass of the atom
    # name - name/label of the atom
    # x,y,z - the x,y and z coordinates of the atom
    cursor_new.execute(
        "CREATE TABLE refdbrefgeom(atno INTEGER NOT NULL,\
                         mass DOUBLE NOT NULL,\
                         name TEXT NOT NULL,\
                         x DOUBLE NOT NULL,\
                         y DOUBLE NOT NULL,\
                         z DOUBLE NOT NULL);"
    )
    # In this case load in old refdbrefgeom
    sql_string = "SELECT * FROM refdbrefgeom"
    cursor.execute(sql_string)
    result = cursor.fetchall()
    for entry in result:
        print(entry)
        sql_string = "INSERT INTO refdbrefgeom values{0}".format(entry)
        # either above or below will work
        # sql_string = "INSERT INTO refdbrefgeom(atno, mass, name, x, y, z) values{0}".format(entry)
        cursor_new.execute(sql_string)

    # create refdbsymmaps table
    # Contains the symmetry relationships
    # Size, a row per symmetry operation
    # Column size changes with number of atoms
    # Symopno - The number of the symmetry operation
    # atmi - the mapping of atom number i for given symopno
    sql_string = "CREATE TABLE refdbsymmaps (\
                Symopno INTEGER PRIMARY KEY"
    for catm in range(1, natom + 1):
        sql_string = sql_string + ",atm{0} INTEGER".format(catm)
    sql_string = sql_string + ")"
    cursor_new.execute(sql_string)
    # In this case load in old refdbrefgeom
    for x in range(1, 3):
        sql_string = "SELECT * FROM refdbsymmaps WHERE Symopno = {0}".format(x)
        cursor.execute(sql_string)
        result = cursor.fetchall()
        sql_string = "INSERT INTO refdbsymmaps values({0}".format(x)
        for catm in range(0, natom):
            sql_string = sql_string + ",{0}".format(result[catm][2])
        sql_string = sql_string + ")"
        cursor_new.execute(sql_string)

    # TABLES FROM HERE ARE NOT FILLED UPON CREATION

    # create geo table
    # Contains geometries for each DB entry in cartesian
    # Size, a row for each DB entry
    # Column number changes based on number of atoms, for each atom have 3 columns
    # x_i,y_i,z_i - the x,y and z coordinates of the atom, i is the atom number
    sql_string = "CREATE TABLE geo (\
                id INTEGER NOT NULL"
    for coo in range(1, natom + 1):
        sql_string = (
            sql_string
            + ",\
                                x_{0} DOUBLE NOT NULL,\
                                y_{0} DOUBLE NOT NULL,\
                                z_{0} DOUBLE NOT NULL".format(
                coo
            )
        )
    sql_string = sql_string + ")"
    cursor_new.execute(sql_string)

    # create nact table (like geo is in cart)
    # Contains the nonadiabatic couplings for each DB entry
    # Size, a row for each DB entry
    # Column number changes based on number of atoms, for each atom have 3 columns
    # x_i,y_i,z_i - the x,y and z coordinates of the atom, i is the atom number
    sql_string = "CREATE TABLE nact (\
                id INTEGER NOT NULL,\
                dercp INTEGER NOT NULL"
    for nact in range(1, natom + 1):
        sql_string = (
            sql_string
            + ",\
                                x_{0} DOUBLE NOT NULL,\
                                y_{0} DOUBLE NOT NULL,\
                                z_{0} DOUBLE NOT NULL".format(
                nact
            )
        )
    sql_string = sql_string + ")"
    cursor_new.execute(sql_string)

    # make Nmode table
    # Contains geometries for each DB entry in normal modes
    # Size, a row for each DB entry
    # Column number changes based on number of atoms,
    # have 3N-6 (for nonlinear molecules) normal modes
    # nmdof_i - the normal mode coordinate i is the atom number
    sql_string = "CREATE TABLE nmode (\
                id INTEGER NOT NULL"
    for nmm in range(1, nmodes + 1):
        sql_string = sql_string + ",nmdof_{0} DOUBLE NOT NULL".format(nmm)
    sql_string = sql_string + ")"
    cursor_new.execute(sql_string)

    # make pes table
    # Contains the PES energies for each DB entry
    # Size, a row for DB entry
    # Column number changes based on number of electronic states
    # Have a column for each state and the coupling between them
    # eng_i_j -   the energy for states i & j (only i & j stored not j & i)
    sql_string = "CREATE TABLE pes (\
                id INTEGER NOT NULL,\
                compdim INTEGER NOT NULL"
    for state1 in range(1, nstate + 1):
        for state2 in range(state1, nstate + 1):
            sql_string = sql_string + ",eng_{0}_{1} DOUBLE NOT NULL".format(
                state1, state2
            )
    sql_string = sql_string + ")"
    # print(sql_string)
    cursor_new.execute(sql_string)

    # make apes table
    # Contains the adiabatic PES energies for each DB entry
    # Size, a row for DB entry
    # Column number changes based on number of electronic states
    # Have a column for each state
    # eng_i -   the energy for state i
    sql_string = "CREATE TABLE apes (\
                id INTEGER NOT NULL"
    for state in range(1, nstate + 1):
        sql_string = sql_string + ",eng_{0} DOUBLE NOT NULL".format(state)
    sql_string = sql_string + ")"
    cursor_new.execute(sql_string)

    # make trans table
    # Contains trans data
    # Size, a row for each DB entry
    # Column number changes based on number of electronic states
    # Have a column for each state and the coupling between them
    # tran_i_j -   the trans between states i & j (only i & j stored not j & i)
    sql_string = "CREATE TABLE trans (\
                id INTEGER NOT NULL,\
                compdim INTEGER NOT NULL"
    for state1 in range(1, nstate + 1):
        for state2 in range(1, nstate + 1):
            sql_string = sql_string + ",tran_{0}_{1} DOUBLE NOT NULL".format(
                state1, state2
            )
    sql_string = sql_string + ")"
    # print(sql_string)
    cursor_new.execute(sql_string)

    # make grad table
    # Contains PES gradient values in cartesian
    # Size, a row for each DB entry
    # Column number changes based on number of electronic states and atoms
    # Have a column for each state and coupling between them for each atom
    # x_i_j_k, y_i_j_k, z_i_j_k - the gradient in cartesian for states i & j with atom k
    sql_string = "CREATE TABLE gra (\
                id INTEGER NOT NULL"
    for state1 in range(1, nstate + 1):
        for state2 in range(state1, nstate + 1):
            for coo in range(1, natom + 1):
                sql_string = (
                    sql_string
                    + ",\
                                x_{0}_{1}_{2} DOUBLE NOT NULL,\
                                y_{0}_{1}_{2} DOUBLE NOT NULL,\
                                z_{0}_{1}_{2} DOUBLE NOT NULL".format(
                        state1, state2, coo
                    )
                )
    sql_string = sql_string + ")"
    # print(sql_string)
    cursor_new.execute(sql_string)

    # make agrad table
    # Contains adiabatic PES gradient values in cartesian
    # Size, a row for each DB entry
    # Column number changes based on number of electronic states and atoms
    # Have a column for each state for each atom
    # x_i_k, y_i_k, z_i_k - the gradient in cartesian for state i with atom k
    sql_string = "CREATE TABLE agra (\
                id INTEGER NOT NULL"
    for state1 in range(1, nstate + 1):
        for coo in range(1, natom + 1):
            sql_string = (
                sql_string
                + ",\
                                x_{0}_{1} DOUBLE NOT NULL,\
                                y_{0}_{1} DOUBLE NOT NULL,\
                                z_{0}_{1} DOUBLE NOT NULL".format(
                    state1, coo
                )
            )
    sql_string = sql_string + ")"
    # print(sql_string)
    cursor_new.execute(sql_string)

    # make hes table
    # Contains the Hessian values in cartesian
    # Size, a row for each DB entry
    # Column number changes based on number of electronic states and atoms
    # Have a column for each state for each atom (twice as second diff) and coupling
    # x1_i_j_k_l, y1_i_j_k_l, z1_i_j_k_l - first hessian entries for states i & j with atoms k & l
    # x2_i_j_k_l, y2_i_j_k_l, z2_i_j_k_l - second hessian entries for states i & j with atoms k & l
    # x3_i_j_k_l, y3_i_j_k_l, z3_i_j_k_l - third hessian entries for states i & j with atoms k & l
    # THIS IS BIG, TOO BIG needs to be split down - table for each state

    # Also create new hes table with one entry per electronic state
    # Contains the Hessian values in cartesian
    # Size, a row for each DB entry a table for each electronic state
    # Column number changes based on number of electronic states and atoms
    # Have a column for each state for each atom (twice as second diff) and coupling
    # x1_i_k_l, y1_i_k_l, z1_i_k_l - first hessian entries for states i with atoms k & l
    # x2_i_k_l, y2_i_k_l, z2_i_k_l - second hessian entries for states i with atoms k & l
    # x3_i_k_l, y3_i_k_l, z3_i_k_l - third hessian entries for states i with atoms k & l

    # sql_string = "CREATE TABLE hes (\
    # id INTEGER NOT NULL"
    count = 0
    for state1 in range(1, nstate + 1):
        sql_string_2 = "CREATE TABLE hes_{0} (\
                id INTEGER NOT NULL".format(
            state1
        )
        for state2 in range(state1, nstate + 1):
            for coo1 in range(1, natom + 1):
                for coo2 in range(1, natom + 1):
                    count = count + 1
                    # sql_string = sql_string + ",\
                    # x1_{0}_{1}_{2}_{3} DOUBLE NOT NULL,\
                    # y1_{0}_{1}_{2}_{3} DOUBLE NOT NULL,\
                    # z1_{0}_{1}_{2}_{3} DOUBLE NOT NULL,\
                    # x2_{0}_{1}_{2}_{3} DOUBLE NOT NULL,\
                    # y2_{0}_{1}_{2}_{3} DOUBLE NOT NULL,\
                    # z2_{0}_{1}_{2}_{3} DOUBLE NOT NULL,\
                    # x3_{0}_{1}_{2}_{3} DOUBLE NOT NULL,\
                    # y3_{0}_{1}_{2}_{3} DOUBLE NOT NULL,\
                    # z3_{0}_{1}_{2}_{3} DOUBLE NOT NULL \
                    #  ".format(state1,state2,coo1,coo2)

                    sql_string_2 = (
                        sql_string_2
                        + ",\
                                x1_{0}_{1}_{2} DOUBLE NOT NULL,\
                                y1_{0}_{1}_{2} DOUBLE NOT NULL,\
                                z1_{0}_{1}_{2} DOUBLE NOT NULL,\
                                x2_{0}_{1}_{2} DOUBLE NOT NULL,\
                                y2_{0}_{1}_{2} DOUBLE NOT NULL,\
                                z2_{0}_{1}_{2} DOUBLE NOT NULL,\
                                x3_{0}_{1}_{2} DOUBLE NOT NULL,\
                                y3_{0}_{1}_{2} DOUBLE NOT NULL,\
                                z3_{0}_{1}_{2} DOUBLE NOT NULL \
                                ".format(
                            state2, coo1, coo2
                        )
                    )
        sql_string_2 = sql_string_2 + ")"
        cursor_new.execute(sql_string_2)

    # sql_string = sql_string + ")"
    # cursor_new.execute(sql_string)

    # make ahes table
    # Contains the adiabatic Hessian values in cartesian
    # Size, a row for each DB entry
    # Column number changes based on number of electronic states and atoms
    # Have a column for each state for each atom (twice as second diff)
    # x1_i_k_l, y1_i_k_l, z1_i_k_l - first hessian entries for state i with atoms k & l
    # x2_i_k_l, y2_i_k_l, z2_i_k_l - second hessian entries for state i with atoms k & l
    # x3_i_k_l, y3_i_k_l, z3_i_k_l - third hessian entries for state i with atoms k & l
    # THIS IS BIG, TOO BIG needs to be split down - table for each state
    # sql_string = "CREATE TABLE ahes (\
    #             id INTEGER NOT NULL"
    for state1 in range(1, nstate + 1):
        sql_string_2 = "CREATE TABLE ahes_{0} (\
        id INTEGER NOT NULL".format(
            state1
        )
        for coo1 in range(1, natom + 1):
            for coo2 in range(1, natom + 1):
                # sql_string = sql_string + ",x1_{0}_{1}_{2} DOUBLE NOT NULL,\
                #             y1_{0}_{1}_{2} DOUBLE NOT NULL,\
                #             z1_{0}_{1}_{2} DOUBLE NOT NULL,\
                #             x2_{0}_{1}_{2} DOUBLE NOT NULL,\
                #             y2_{0}_{1}_{2} DOUBLE NOT NULL,\
                #             z2_{0}_{1}_{2} DOUBLE NOT NULL,\
                #             x3_{0}_{1}_{2} DOUBLE NOT NULL,\
                #             y3_{0}_{1}_{2} DOUBLE NOT NULL,\
                #             z3_{0}_{1}_{2} DOUBLE NOT NULL  ".format(state1,coo1,coo2)
                sql_string_2 = (
                    sql_string_2
                    + ",\
                            x1_{0}_{1} DOUBLE NOT NULL,\
                            y1_{0}_{1} DOUBLE NOT NULL,\
                            z1_{0}_{1} DOUBLE NOT NULL,\
                            x2_{0}_{1} DOUBLE NOT NULL,\
                            y2_{0}_{1} DOUBLE NOT NULL,\
                            z2_{0}_{1} DOUBLE NOT NULL,\
                            x3_{0}_{1} DOUBLE NOT NULL,\
                            y3_{0}_{1} DOUBLE NOT NULL,\
                            z3_{0}_{1} DOUBLE NOT NULL  ".format(
                        coo1, coo2
                    )
                )
        sql_string_2 = sql_string_2 + ")"
        cursor_new.execute(sql_string_2)
    # sql_string = sql_string + ")"
    # print(count)
    # cursor_new.execute(sql_string)
    if lmoco:
        if tmoco:
            # create new table to copy into
            sql_string = "CREATE TABLE moco (\
                            id INTEGER NOT NULL\
                            ,mo1 INTEGER NOT NULL\
                            ,mo2 INTEGER NOT NULL\
                            ,value DOUBLE NOT NULL);"
            cursor_new.execute(sql_string)
        else:
            # write moco file
            mfile = "moco"
            # create moco table of file pos
            sql_string = "CREATE TABLE moco (\
                            id INTEGER NOT NULL);"
            cursor_new.execute(sql_string)
            # create a moco table per MO
            for mo1 in range(1, nbasis + 1):
                sql_string = "CREATE TABLE moco_{0} (\
                            id INTEGER NOT NULL".format(
                    mo1
                )
                for mo2 in range(1, nbasis + 1):
                    sql_string = sql_string + ",mo_{0} INTEGER NOT NULL".format(mo2)
                sql_string = sql_string + ")"
                cursor_new.execute(sql_string)

            outfile = open(mfile, "w")  # this must be closed later!

        # make sure to commit changes
        con_new.commit()

    # read from old db into new
    for did in range(1, dbcount):
        # for did in range(1,2):
        # Set new entry in dbentry
        sql_string = "INSERT INTO dbentry(status, created) VALUES (1, 000)"
        cursor_new.execute(sql_string)

        cursor_new.execute("SELECT max(id) FROM dbentry")
        max_id = cursor_new.fetchone()[0]
        # cursor_new.execute("SELECT last_insert_rowid() FROM dbentry")
        # max_id = cursor
        print("Current database point is {0}".format(max_id))

        # now read in a geom and recast into single entry
        # Get coordinates from geometry table for id 208
        cursor.execute("SELECT value FROM geo WHERE id={0};".format(max_id))
        geo = cursor.fetchall()

        sql_string = "INSERT INTO geo values ({0}".format(max_id)
        for index in range(0, len(geo), 3):
            sql_string = sql_string + ",{0},{1},{2}".format(
                geo[index][0], geo[index + 1][0], geo[index + 2][0]
            )
            if index == len(geo) - 3:
                break
        sql_string = sql_string + ")"

        cursor_new.execute(sql_string)

        # now read in a nact and recast into single entry

        cursor.execute("SELECT dercp,value FROM nact WHERE id={0};".format(max_id))
        nact = cursor.fetchall()

        sql_string = "INSERT INTO nact values ({0}, {1}".format(max_id, nact[0][0])
        for index in range(0, len(nact), 3):
            sql_string = sql_string + ",{0},{1},{2}".format(
                nact[index][1], nact[index + 1][1], nact[index + 2][1]
            )
            if index == len(nact) - 3:
                break
        sql_string = sql_string + ")"

        cursor_new.execute(sql_string)

        # read old nmode
        cursor.execute("SELECT value FROM nmode WHERE id={0};".format(max_id))
        nmodes = cursor.fetchall()
        # print(nmodes)

        # read nmodes into new database
        sql_string = "INSERT INTO nmode values({0}".format(max_id)
        for index in range(0, len(nmodes)):
            sql_string = sql_string + ",{0}".format(nmodes[index][0])
        sql_string = sql_string + ")"

        cursor_new.execute(sql_string)

        # read in old pes
        cursor.execute("SELECT energy FROM pes WHERE id={0}".format(max_id))
        pes = cursor.fetchall()

        # insert into new database
        sql_string = "INSERT INTO pes values({0}, {1}".format(max_id, 1)
        for index in range(0, len(pes)):
            sql_string = sql_string + ",{0}".format(pes[index][0])
        sql_string = sql_string + ")"

        cursor_new.execute(sql_string)

        # read in old apes
        cursor.execute("SELECT energy FROM apes WHERE id={0}".format(max_id))
        apes = cursor.fetchall()

        # insert into new apes database
        sql_string = "INSERT INTO apes values({0}".format(max_id)
        for index in range(0, len(apes)):
            sql_string = sql_string + ",{0}".format(apes[index][0])
        sql_string = sql_string + ")"

        cursor_new.execute(sql_string)
        # read in old trans
        cursor.execute("SELECT value FROM trans WHERE id={0}".format(max_id))
        trans = cursor.fetchall()

        # insert into new database
        sql_string = "INSERT INTO trans values({0}, {1}".format(max_id, 1)
        for index in range(0, len(trans)):
            sql_string = sql_string + ",{0}".format(trans[index][0])
        sql_string = sql_string + ")"

        cursor_new.execute(sql_string)

        # read in old gra
        cursor.execute("SELECT value FROM gra WHERE id={0}".format(max_id))
        gra = cursor.fetchall()

        # insert into new database
        sql_string = "INSERT INTO gra values({0}".format(max_id)
        for index in range(0, len(gra)):
            sql_string = sql_string + ",{0}".format(gra[index][0])
        sql_string = sql_string + ")"

        cursor_new.execute(sql_string)

        # read in old agra
        cursor.execute("SELECT value FROM agra WHERE id={0}".format(max_id))
        agra = cursor.fetchall()

        # insert into new database
        sql_string = "INSERT INTO agra values({0}".format(max_id)
        for index in range(0, len(agra)):
            sql_string = sql_string + ",{0}".format(agra[index][0])
        sql_string = sql_string + ")"

        cursor_new.execute(sql_string)

        # read in old hes
        # cursor.execute("SELECT value FROM hes WHERE id={0}".format(max_id))
        # hes = cursor.fetchall()

        # insert into new database
        # sql_string = "INSERT INTO hes values({0}".format(max_id)
        # for index in range(0,len(hes)):
        #     sql_string = sql_string + ",{0}".format(hes[index][0])
        # sql_string = sql_string + ")"

        # cursor_new.execute(sql_string)

        # read in old hes state by state
        for state in range(1, nstate + 1):
            cursor.execute(
                "SELECT value FROM hes WHERE id={0} AND state1={1}".format(
                    max_id, state
                )
            )
            hes = cursor.fetchall()
            # insert into new database
            sql_string = "INSERT INTO hes_{0} values({1}".format(state, max_id)
            for index in range(0, len(hes)):
                sql_string = sql_string + ",{0}".format(hes[index][0])
            sql_string = sql_string + ")"
            cursor_new.execute(sql_string)

        # read in old ahes
        # cursor.execute("SELECT value FROM ahes WHERE id={0}".format(max_id))
        # ahes = cursor.fetchall()

        # #insert into new database
        # sql_string = "INSERT INTO ahes values({0}".format(max_id)
        # for index in range(0,len(ahes)):
        #     sql_string = sql_string + ",{0}".format(ahes[index][0])
        # sql_string = sql_string + ")"
        # cursor_new.execute(sql_string)
        # read in old ahes state by state
        for state in range(1, nstate + 1):
            cursor.execute(
                "SELECT value FROM ahes WHERE id={0} AND state1={1}".format(
                    max_id, state
                )
            )
            ahes = cursor.fetchall()
            # insert into new database
            sql_string = "INSERT INTO ahes_{0} values({1}".format(state, max_id)
            for index in range(0, len(ahes)):
                sql_string = sql_string + ",{0}".format(ahes[index][0])
            sql_string = sql_string + ")"
            cursor_new.execute(sql_string)

        # get moco if needed
        if lmoco:
            cursor.execute("SELECT * FROM moco WHERE id={0}".format(max_id))
            moco = cursor.fetchall()
            if tmoco:
                for entry in moco:
                    sql_string = "INSERT INTO moco values{0}".format(entry)
                    cursor_new.execute(sql_string)
            else:
                text = ""
                for entry in moco:
                    text = text + str(entry[3]) + ","
                text = text + "\n"
                outfile.writelines(text)
                sql_string = "INSERT INTO moco VALUES ({0})".format(outfile.tell())
                cursor_new.execute(sql_string)

                # read in old mo basis by basis
                for mo1 in range(1, nbasis + 1):
                    cursor.execute(
                        "SELECT value FROM moco WHERE id={0} AND mo1={1}".format(
                            max_id, mo1
                        )
                    )
                    mos = cursor.fetchall()
                    # insert into new DB
                    sql_string = "INSERT INTO moco_{0} values({1}".format(mo1, max_id)
                    for index in range(0, len(mos)):
                        sql_string = sql_string + ",{0}".format(mos[index][0])
                    sql_string = sql_string + ")"
                    cursor_new.execute(sql_string)
            print("Current MOCO point is {0}".format(max_id))
    # make sure to commit changes
    con_new.commit()
    # get moco if needed

    # if lmoco:
    # if tmoco:
    #     #create new table
    #     print("Getting MOCO")
    #     sql_string = "CREATE TABLE moco (\
    #                     id INTEGER NOT NULL\
    #                     ,mo1 INTEGER NOT NULL\
    #                     ,mo2 INTEGER NOT NULL\
    #                     ,value DOUBLE NOT NULL);"
    #     cursor_new.execute(sql_string)
    #     #read old table
    #     #get number of entries
    #     sql_string = "SELECT COUNT(*) FROM moco"
    #     cursor.execute(sql_string)
    #     nmoco = (cursor.fetchall()[0][0])
    #     print("Number of MOCO entries {0}".format(nmoco))
    #     #read in old moco
    #     for row in range(1, nmoco+1):
    #         #get current row
    #         cursor.execute("SELECT * FROM moco WHERE rowid={0}".format(row))
    #         moco = cursor.fetchall()
    #         #stick in new db
    #         for entry in moco:
    #             sql_string = "INSERT INTO moco values{0}".format(entry)
    #             cursor_new.execute(sql_string)
    #         print("Current MOCO point is {0}".format(row))

    #     #make sure to commit changes
    #     con_new.commit()
    # else:
    #     #write moco file
    #     mfile = "moco"
    #     #create moco table of file pos
    #     sql_string = "CREATE TABLE moco (\
    #                     id INTEGER NOT NULL);"
    #     cursor_new.execute(sql_string)

    #     with open(mfile, 'w') as outfile:
    #         #read old table
    #         #get number of entries
    #         sql_string = "SELECT COUNT(*) FROM moco"
    #         cursor.execute(sql_string)
    #         nmoco = (cursor.fetchall()[0][0])
    #         print("Number of MOCO entries {0}".format(nmoco))
    #         #read in old moco
    #         for row in range(1, dbcount):
    #             #get current row
    #             cursor.execute("SELECT * FROM moco WHERE id={0}".format(row))
    #             moco = cursor.fetchall()
    #         #stick in new db
    #             text = ""
    #             for entry in moco:
    #                 text = text + str(entry[3]) + ","
    #             text = text + "\t"
    #             outfile.writelines(text)
    #             sql_string = "INSERT INTO moco VALUES ({0})".format(outfile.tell())
    #             cursor_new.execute(sql_string)
    #             print("Current MOCO point is {0}".format(row))

    #     #make sure to commit changes
    #     con_new.commit()
    # clean up moco file

    if outfile:
        outfile.close()
    # clean up database connections
    if con:
        con.close()
    if con_new:
        con_new.close()
