"""
A module to provide connections to a
sqlite database

"""

import sqlite3
import os
import time

def query_existing(src_cursor):
    """
    Function to get table information from existing sqlite DB
    src_cursor - connection to DB
    Returns
    tables - list of tables in database
    """
    try:
        src_cursor.execute("SELECT name FROM sqlite_master WHERE type='table';")
        tables = src_cursor.fetchall()
    except sqlite3.Error as error:
        print(
            "Error while requesting \n SELECT name FROM sqlite_master WHERE type='table'; \n",
            error,
        )

    return tables

def query_table_schema(src_cursor, tablename):
    """
    Function to get table schemas
    src_cursor - connection to DB
    tablename - name of table to query schema from
    Returns
    Just prints list to screen
    """
    try:
        src_cursor.execute(f"SELECT sql FROM sqlite_master WHERE name='{tablename}';")
        print(f"Table {tablename} created with following schema:")
        tab_schema = src_cursor.fetchall()[0]
        print(tab_schema)
        return tab_schema
    except sqlite3.Error as error:
        print(
            f"Error while requesting \n SELECT sql FROM sqlite_schema WHERE name={tablename}; \n",
            error,
        )


def connect_db(db_name, db_create):
    """
    connect to target DB
    db_name string which is the path to the database

    returns
    connect which is the sqlite3 object
    curs which is the cursor to access the database
    error - is -1 if DB does not exis
    """
    try:
        # check the database exists or we want new
        if os.path.isfile(db_name) or (db_create == 1):
            connect = sqlite3.connect(db_name)
            curs = connect.cursor()
            error = 0
            return error, connect, curs
        else:
            error = -1
            return error, 0, 0

    except sqlite3.Error as error:
        print("Error while connecting to sqlite", error)

    return error, connect, curs


def select_existing(src_cursor, select_text):
    """
    Function to get information from existing sqlite DB
    src_cursor - connection to DB
    select_text - text to send to DB, must be a SELECT
    Returns
    tables - list of tables in database
    """
    try:
        # src_cursor.execute("SELECT name FROM sqlite_master WHERE type='table';")
        src_cursor.execute(select_text)
        tables = src_cursor.fetchall()
    except sqlite3.Error as error:
        print("Error while connecting to sqlite", error)

    return tables

#READ functions below here
def dbrdrefdb(src_cursor, dbversion):
    """
    Function to read info from refdb
    might be version dependent in future
    returns a dictionary of the values with column name as key
    """
    dict_db = {}
    if dbversion > 0:
        try:
            dblist = select_existing(src_cursor, "SELECT * FROM refdb")[0]
            for index, description in enumerate(src_cursor.description):
                dict_db[description[0]] = dblist[index]
            return dict_db
        except sqlite3.Error as error:
            print("Error while connecting to sqlite", error)


def dbrdgeo(src_cursor, dbversion):
    """
    Function to read in geometry from geo table
    is version dependent
    returns array of geoms
    """
    #Will test different ways of getting the data

    if dbversion == 2:
        #is version 2
        #get single version and loop
        # src_cursor.execute("SELECT count(id) FROM geo")
        # entry = src_cursor.fetchone()[0]
        # geo = []
        # t1 = time.time()
        # for id in range(1,entry):
        #     src_cursor.execute(f"SELECT * FROM geo WHERE id={id};")
        #     #print(src_cursor.fetchone())
        #     geo.append(src_cursor.fetchone()[2:])
        # t2 = time.time()
        # print("Looped pull",(t2-t1), "s")
        # #print(geo)
        t1 = time.time()
        src_cursor.execute("SELECT * FROM geo;")
        tgeo = src_cursor.fetchall()
        geo = []
        for entry in tgeo:
            geo.append(entry[2:])
        t2 = time.time()
        print("Fetch all pull",(t2-t1), "s")
        
        return geo