"""
Simple Python script to get info on an sql database
"""
import argparse  # import for parsing of command line options
import python_sqlite as pysql

def parsing():
    """
    Function create the cmd line argument parsing
    returns the parsing function
    would make sense to go in its own library separate to gui things
    """

    # the - means optional argument, no dash means must be given and error is raised if not
    parse = argparse.ArgumentParser(
        description="Purpose: \
                                    Create inputs for Quantics calculations"
    )
    parse.add_argument(
        "-DB",
        metavar="FILE",
        type=str,
        default="database.sql",
        help="Path to find existing DB. Absolute",
    )
    return parse


if __name__ == "__main__":
    # if run on its own
    # can take CLI arguments with argparse

    parser = parsing()
    parsed_args = parser.parse_args()
    filename = parsed_args.DB

    print(f"Connecting to {filename}")
    # Connect to existing sql database and get cursor
    dberror, con, cursor  = pysql.connect_db(filename,0)
    if dberror == 0:
        #Get DB version
        quant_Ver, db_Ver, dbtime = pysql.select_existing(cursor, "SELECT * FROM versions")[0]
        print(f"Quantics Version: {quant_Ver}")
        print(f"SQL version: {db_Ver}")
        # Get number of entries
        dbcount = pysql.select_existing(cursor, "SELECT count(id) FROM dbentry;")[0][0]
        print(f"Database contains {dbcount} entries")
        #Get info from DB
        dict_db = pysql.dbrdrefdb(cursor,db_Ver)
        #print that info
        for key, ent in dict_db.items():
            print(f"{key}: {ent}")

        #Get list of tables that are present
        #Uncomment following 4 lines
        # db_tabs = pysql.query_existing(cursor)
        # print("Database contains the following tables:")
        # for tab in db_tabs:
        #     print(tab[0])

        #If wanted query the SQL schema for a table
        #pysql.query_table_schema(cursor, "geo")
        #Example of how to access geometry
        geo = pysql.dbrdgeo(cursor,db_Ver)

        #close the DB connection
        cursor.close()
    else:
        print(f"Error connecting to {filename}")
