!***********************************************************************
!
!                      AUSWERT subroutine RDACOEFF
!
!     This subroutine prints the A-vector.
!
!     06/08 HDM
!
!
!***********************************************************************
      
      program rdacoeff

use decimal, only: long,dop,sip
use global
use globmemmod
use aglobal
use versions
use dvrdatmod
use griddatmod
use psidef
use operdef
use channels
use datenmod
use memdimmod

use iorst, only: rstinfo
use zeigauswmod, only: zeigausw
use iogrddef, only: griddat
use iopsidef, only: psidat
use iofile, only: rdpsiinfo
use logdat
use plot

use wftree
use wftree1, only: dispose_wftree
use mlanalysis

      implicit none

      integer(long)     ::  zeig,zeig1,zeig2,m,s,t,ilbl, &
                            chkdvr,chkgrd,chkpsi,chkdat,chkprp
      character(len=c5) :: filename
      logical(kind=4)   :: lerr,linwf

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      call rdacinput

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='rdacoeff.log',form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(ipsi)

!-----------------------------------------------------------------------
! Allocate memory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      if(lshowrst) call alloc_operdef
      call alloc_psidef

      if( lshowrst ) then
!-----------------------------------------------------------------------
! read data from restart file  rather than psi)
!-----------------------------------------------------------------------
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkprp=1
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         if (lerr) then 
            write(6,'(a)') ' ERROR reading restart file' 
            stop
         end if
      else
!-----------------------------------------------------------------------
! read data from psi file
!-----------------------------------------------------------------------
         read(ipsi) filever(ipsi)
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkdat=1
         call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
         filename='psi'
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif

      if (dentype .eq. 1 .and. nstate .gt. 1) then
         write(6,'(a)') 'Does not work for multi-set type I densities'
         stop
      endif

     ! if (nmode .eq. 1) then
     !    do s=1,nstate
     !       dim(1,s)=1
     !    enddo
     ! endif
      
!     ML-wavefunction
      if (psitype.eq.5) then
! ... ML wavefunction? Write header.
         write (6,*)
         call print_wftree(6,tree,modelabel,.true.)
         write (6,*)
         call dispose_wftree(tree)
      endif

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
!-----------------------------------------------------------------------
      lpsi=.true.
      lpsigrd=.false.
      call zeigausw

      if (dentype .eq. 1) then
         zeig=1
         s=1
         t=1
         zeig1=dblock(s,t)
         do m=1,nmode
            zeig=zeig+ddim(m,s,t)*ddim(m,s,t)
            zeig2=zeig1
            zeig2=zeig2+zeig1
            workcdim=max(workcdim,zeig2)
         enddo

      else
         zeig=1
         do s=1,nstate
            zeig1=block(s)
            do m=1,nmode
               zeig=zeig+dim(m,s)*dim(m,s)
               zeig2=zeig1
               zeig2=zeig2+zeig1
               workcdim=max(workcdim,zeig2)
            enddo
         enddo
      endif

      call griddat
      call psidat

!-----------------------------------------------------------------------
! assign memory arrays
!-----------------------------------------------------------------------
      call memausw

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      workcdim = max(workcdim,2*maxblock)

      mcworkc = mcdim
      mcdim   = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)
if(allocated(agmat))deallocate(agmat)
allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))
allocate(agmat(agmatdim))

      call rdac(mc(mcpsi),ms(mspsi),mc(mcworkc),mi(miworki), &
                mi(mijindx),agmat,mc(mcadgwp),mi(mitrajst))

deallocate(mc,mr,mi,ml,ms,mf,agmat)

      stop
  999 continue
      message= 'Error reading :'//filename(1:c5-15)
      write(6,'(a)') message
      stop

contains

!***********************************************************************
!    Does the work 
!***********************************************************************

      subroutine rdac(psi,spsi,workc,worki,jindx,agmat,adgwp,trajst)

use plot

use rdfilesmod, only: rdpsi
use psiutil, only: adecoder,a1decoder

      implicit none

      integer(long), dimension(workidim), intent(out) :: worki
      integer(long), dimension(adim), intent(out)     :: jindx
      integer(long)                                   :: i,k,j,nn1,nn2,m,&
                                                         ss,sindx,tindx
      integer(long), dimension(maxdim)                :: idx
      integer(long)                                   :: absi, cnt, uplo, s,&
                                                         jj,jx
      integer(long), dimension(1)                     :: idum
      integer(long), dimension(gwpinfodim), intent(out) :: trajst
      complex(sip), dimension(spsidim), intent(out)   :: spsi
      complex(dop), dimension(dgldim), intent(out)    :: psi
      complex(dop), dimension(workcdim), intent(out)  :: workc
      complex(dop), dimension(agmatdim), intent(out)  :: agmat
      complex(dop), dimension(gsmatdim), intent(out)  :: adgwp
      real(dop)                                       :: time,dt,thr,tm,atm,mx,&
                                                         msum, norm
      logical(kind=4)                                 :: lcnt,ldblout,luse,luplo
      character(len=6), dimension(5)                  :: ctype
      integer(long)                                   :: ilbl,jlbl
      character(len=c2)                               :: rdtag

      common /rdacin/ mx,thr,tm,atm,nn1,nn2,ss,absi,lcnt,ldblout,uplo,jj,jx
      data  ctype /' cmplx','  abs ',' abs^2','  Re  ','  Im  '/

!-----------------------------------------------------------------------
! calculate time step for data
!-----------------------------------------------------------------------
      dt = out2/fs
      time = tinit/fs
      tm=tm+dt*1.d-5
      atm=atm-dt*1.d-5
      k    = 0
      nn2 = min(nn2,adim)
      luplo = (uplo .ne. 0)

      if (.not. luplo) luse = .true.

! --- Set nn1 and nn2 if state index ss is given.
      if(ss.gt.0) then
         if(ss.gt.nstate) then
            write(6,'(a,i3,a,i3,a)') 'Illegal value for s =', ss, &
                  ' , (nstate =',nstate,')'
            stop
         endif
         nn1=max(nn1,zpsi(ss))
         nn2=min(nn2,zpsi(ss)+block(ss)-1)
         s = ss
      else
         s = 1
         t = 1
      endif

   10 continue  ! Loop

!-----------------------------------------------------------------------
! read restart rather than psi file.
!-----------------------------------------------------------------------
      if (lshowrst) then
!-----------------------------------------------------------------------
! read restart wavefunction
!-----------------------------------------------------------------------
         if (filever(irst) .lt. 11.0_dop) then
            read (irst,err=998) time
            if (citype .gt. 0) then
               read(irst,err=998) (jindx(i),i=1,adim)
            endif
            read (irst,err=998) (psi(i),i=1,dgldim)
         else
            rewind(irst)
            do
  20           continue
               read(irst,iostat=chkerr(1),err=20) rdtag
               if (rdtag(1:1) .ne. '<') cycle

               ilbl=2
               jlbl=index(rdtag,'>')-1

               if (rdtag(ilbl:jlbl) .eq. 'time') then
                  read (irst,err=998) time
               else if (rdtag(ilbl:jlbl) .eq. 'psi') then
                  read (irst,err=998) (psi(i),i=1,dgldim)
               else if (rdtag(ilbl:jlbl) .eq. 'end restart file') then
                  exit
               endif
            enddo
         endif
         time = time/fs
      
      else 
         call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,idum)
         if (lend) goto 77
         if( mod(k,step) .ne. 0 ) goto 11
         if(time .le. atm) goto 11
      endif

      nn2 = min(nn2,adim)
      write(*,*) "n1,n2, adim", nn1, nn2, adim

      if(fs .lt. 2.d0) then
         write(6,'(/a,f10.4,3a/)') '      ------- Time =',time ,&
               '  -------         (',ctype(absi+1),')'
      else
         write(6,'(/a,f10.4,3a/)') '      ------- Time =',time ,&
               ' fs -------         (',ctype(absi+1),')'
      endif

      if (psitype .ne. 5) then
         if(.not.lcnt) then
            if(nstate.gt.1) then
               if (luplo) then
                  write(6,'(8x,a,12x,a,14x,a)') &
                        'J','A','Len(J)        s,   j1 j2 j3 j4 j5 ...'
               else
                  write(6,'(8x,a,12x,a,14x,a)') &
                       'J','A','s   j1 j2 j3 j4 j5 ...'
               end if
            else
               if (luplo) then
                  write(6,'(8x,a,12x,a,13x,a)') &
                       'J','A',' Len(J)      j1 j2 j3 j4 j5 ...'
               else
                  write(6,'(8x,a,12x,a,13x,a)') &
                       'J','A','j1 j2 j3 j4 j5 ...'
               end if
            endif
            write(6,'(2a)') '  -----------------',&
                 '--------------------------------------------------------'
         endif
      endif

      cnt  = 0
      norm = 0.0d0
!     ML-wavefunction
      if (psitype.eq.5) then
!CDMT    To sort out later: option -uplo
         if (luplo) then
            write(6,*) 'Not implemented'
            stop
         end if
!        To sort out later: option -n1 -n2
!        nn1,nn2 is always 1
!        nn2 is always adim_wftree(tree)
!        allocate data structures for ML
!         tree => new_wftree(tape,if_spfdof) ! tape was read by rdpsiinfo
         call printA_tree(6,tree,psi,absi,thr,mx,lcnt)
      else

         do j=nn1,nn2
            if (dentype .eq. 1) then
               call a1decoder(j,idx,sindx,tindx)
            else
               call adecoder(j,idx,sindx)
            endif
            if(jj.gt.0) then
               if(idx(jj)+1.ne.jx) cycle
            endif

            if (luplo) then
               luse = .false.
               msum = 0.d0
               if (dentype .eq. 1) then
                  do m=1,nmode
                     msum = msum + dble(idx(m))/(ddim(m,s,t)-1)
                  end do
               else
                  do m=1,nmode
                     msum = msum + dble(idx(m))/(dim(m,s)-1)
                  end do
               endif
              if (uplo .gt. 0) then
                 if (msum.gt. 1.000001d0) luse = .true.
              else if  (uplo .lt. 0) then
                 if (msum.lt. 1.000001d0) luse = .true.
              end if
            end if

            if( abs(psi(j)) .gt. thr .and. abs(psi(j)) .le. mx .and. luse) then
               cnt = cnt + 1
                norm = norm + dble(psi(j))**2 + dimag(psi(j))**2
               if(lcnt) goto 111

               if (luplo)then

                 if(absi.eq.1) then
                     if(nstate.gt.1) then
                        write(6,'(i9,f21.12,2x,f12.8,2x,i5,", ",35i3)') &
                             j,abs(psi(j)),msum,sindx, &
                             (idx(m)+1,m=1,nmode)
                     else
                        write(6,'(i9,f21.12,2x,f12.8,2x,35i3)') &
                             j,abs(psi(j)),msum,(idx(m)+1,m=1,nmode)
                     endif
                  elseif(absi.eq.2) then
                     if(nstate.gt.1) then
                        write(6,'(i9,f21.12,2x,f12.8,2x,i5,", ",35i3)') &
                             j,abs(psi(j))**2,msum,sindx, &
                             (idx(m)+1,m=1,nmode)
                     else
                        write(6,'(i9,f21.12,2x,f12.8,2x,35i3)') &
                             j,abs(psi(j))**2,msum,(idx(m)+1,m=1,nmode)
                     endif
                  elseif(absi.eq.3) then
                     if(nstate.gt.1) then
                        if(abs(dble(psi(j))).gt.thr) &
                             write(6,'(i9,f21.12,2x,f12.8,'// &
                             '2x,i5,", ",35i3)') &
                             j,dble(psi(j)),msum,sindx, &
                             (idx(m)+1,m=1,nmode)
                     else
                        if(abs(dble(psi(j))).gt.thr) &
                             write(6,'(i9,f21.12,2x,f12.8,2x,35i3)') &
                             j,dble(psi(j)),msum,(idx(m)+1,m=1,nmode)
                     endif
                  elseif(absi.eq.4) then
                     if(nstate.gt.1) then
                        if(abs(dimag(psi(j))).gt.thr) &
                             write(6,'(i9,f21.12,2x,f12.8,'// &
                             '2x,i5,", ",35i3)') &
                             j,dimag(psi(j)),msum,sindx, &
                             (idx(m)+1,m=1,nmode)
                     else
                        if(abs(dimag(psi(j))).gt.thr) &
                             write(6,'(i9,f21.12,2x,f12.8,2x,35i3)') &
                             j,dimag(psi(j)),msum,(idx(m)+1,m=1,nmode)
                     endif
                  else
                     if(nstate.gt.1) then
                        write(6,'(i9,"  (",f9.6,",",f9.6,")"'// &
                             ',2x,f11.8,i5,", ",35i3)') &
                             j,psi(j),msum,sindx,(idx(m)+1,m=1,nmode)
                     else
                        write(6,'(i9,"  (",f9.6,",",f9.6,")"'// &
                             ',2x,f11.8,2x,35i3)') &
                             j,psi(j),msum,(idx(m)+1,m=1,nmode)
                     endif
                  endif

               else
                  if(absi.eq.1) then
                     if(nstate.gt.1) then
                        write(6,'(i9,f21.12,2x,i5,", ",35i3)') &
                             j,abs(psi(j)),sindx,(idx(m)+1,m=1,nmode)
                     else
                        write(6,'(i9,f21.12,2x,2x,35i3)') &
                             j,abs(psi(j)),(idx(m)+1,m=1,nmode)
                     endif
                  elseif(absi.eq.2) then
                     if(nstate.gt.1) then
                        write(6,'(i9,f21.12,2x,i5,", ",35i3)') &
                             j,abs(psi(j))**2,sindx,(idx(m)+1,m=1,nmode)
                     else
                        write(6,'(i9,f21.12,2x,2x,35i3)') &
                             j,abs(psi(j))**2,(idx(m)+1,m=1,nmode)
                     endif
                  elseif(absi.eq.3) then
                     if(nstate.gt.1) then
                        if(abs(dble(psi(j))).gt.thr) &
                             write(6,'(i9,f21.12,2x,i5,", ",35i3)') &
                             j,dble(psi(j)),sindx,(idx(m)+1,m=1,nmode)
                     else
                        if(abs(dble(psi(j))).gt.thr) &
                             write(6,'(i9,f21.12,2x,2x,35i3)') &
                             j,dble(psi(j)),(idx(m)+1,m=1,nmode)
                     endif
                  elseif(absi.eq.4) then
                     if(nstate.gt.1) then
                        if(abs(dimag(psi(j))).gt.thr) &
                             write(6,'(i9,f21.12,2x,i5,", ",35i3)') &
                             j,dimag(psi(j)),sindx,(idx(m)+1,m=1,nmode)
                     else
                        if(abs(dimag(psi(j))).gt.thr) &
                             write(6,'(i9,f21.12,2x,2x,35i3)') &
                             j,dimag(psi(j)),(idx(m)+1,m=1,nmode)
                     endif
                  else
                     if(nstate.gt.1) then
                        write(6,'(i9,"  (",f9.6,",",f9.6,")",'// &
                             'i5,", ",35i3)') &
                             j,psi(j),sindx,(idx(m)+1,m=1,nmode)
                     else
                        write(6,'(i9,"  (",f9.6,",",f9.6,")",2x,35i3)') &
                             j,psi(j),(idx(m)+1,m=1,nmode)
                     endif
                  endif
               end if ! luplo
 111           continue
            endif ! abs(psi(j)).gt.thr
         enddo    ! j=n1,n2

      endif     ! psitype.eq.5, else

 11   k = k + 1
      time=time + dt

!     ML-wavefunction
      if (psitype.ne.5) then
         write(6,'(/a,1p2e9.2," :",i8)')&
           ' Number of A-coeffs with absolute values between',thr,mx,cnt
         write(6,'(2(a,f12.8))') ' Norm of these A-coeffs (sqrt[sum|A|^2]):',&
           sqrt(norm), ',  norm^2 =', norm
      endif

!-----------------------------------------------------------------------
! if more data, go to begining of loop
!-----------------------------------------------------------------------
      if (.not. lshowrst .and. time.le.tm)  goto 10

 77   continue
      close(ipsi)

      write(6,'(76a)') ('-',i=1,76)
      
      return
         
 998  continue
      write(6,'(a)') ' Error reading restart file'
      stop 1
      end subroutine rdac

!#######################################################################

      subroutine rdacinput

      implicit none

      integer(long)     :: i,iarg,myiargc,ierr,nn1,nn2,ss,absi,uplo,jj,jx
      real(dop)         :: thr, tm, atm, mx
      logical(kind=4)   ::   lcnt, ldblout
      character(len=80) ::   buf

      common /rdacin/ mx,thr,tm,atm,nn1,nn2,ss,absi,lcnt,ldblout,uplo,jj,jx
!-----------------------------------------------------------------------

      filein  = ' '
      name   = './'
      lshowrst = .false.
      ss = 0
      absi=0
      thr = 1.d-8
      uplo = 0
      mx  = 1.d+8 
      nn1 = 1
      nn2 = 99999999
      jj=0
      jx=0
      atm = -1.d99
      tm  =  1.d99
      lcnt= .false.
      ldblout = .false.
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf

         else if( buf .eq. '-nofs' ) then
            fs=1.d0
         else if( buf .eq. '-abs' ) then
            absi=1
         else if( buf .eq. '-abs2' ) then
            absi=2
         else if( buf .eq. '-re' ) then
            absi=3
         else if( buf .eq. '-im' ) then
            absi=4
         else if( buf .eq. '-thr' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) thr
            if (ierr .ne. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -thr', &
                 ' must be a real number. Not : ', buf
               stop 1
            end if

         else if( buf .eq. '-max' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) mx
            if (ierr .ne. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -max', &
                 ' must be a real number. Not : ', buf
               stop 1
            end if

         else if( buf .eq. '-uplo' ) then
            i = i+1
            call mygetarg(i,buf)
            if (buf .eq. 'u' .or. buf .eq. 'U') then
               uplo = 1
            elseif (buf .eq. 'l' .or. buf .eq. 'L') then
               uplo = -1
            else
               write(6,'(2a/,a)') &
                 ' The  argument following the option -uplo', &
                 ' must be "U"/"u" or "L"/"l". Not : ', buf
               stop 1
            end if

         else if( buf .eq. '-n1' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nn1
            if (ierr .ne. 0  .or.  nn1 .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -n1', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if

         else if( buf .eq. '-n2' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nn2
            if (ierr .ne. 0  .or.  nn2 .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -n2', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if

         else if( buf .eq. '-t' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) tm
            if (ierr .ne. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -t', &
                 ' must be a real number. Not : ', buf
               stop
            end if

         else if( buf .eq. '-at' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) atm
            if (ierr .ne. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -at', &
                 ' must be a real number. Not : ', buf
               stop
            end if

         else if( buf .eq. '-n' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -n', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if

         else if( buf .eq. '-s' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ss
            if (ierr .ne. 0  .or.  ss .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -s', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if

         else if( buf .eq. '-jx' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) jj
            if (ierr .ne. 0  .or.  jj .le. 0) then
               write(6,'(2a/,a)') &
                 ' The first argument following the option -jx', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) jx
            if (ierr .ne. 0  .or.  jx .le. 0) then
               write(6,'(2a/,a)') &
                 ' The second argument following the option -jx', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if

         else if( buf .eq. '-count') then
            lcnt = .true.

         else if( buf .eq. '-r' .or. buf .eq. '-rst') then
            irst   = ipsi
            lshowrst = .true.
            write(6,'(a)') ' *** Reading the RESTART file !  ***'

         else if( buf .eq. '-dblout' )then
            ldblout = .true.

         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,2a)') &
            'Purpose: Prints  the A-vector of a wavefunction', &
               ' or restart file.'
            write(6,'(1x,2a,/20x,a)') &
            'Usage  : rdacoeff [-f -i -n -n1 -n2 -s -jx -max -thr -uplo -r ', &
                 '-abs -abs2','-re -im -t -at -count -nofs -ver -h -?] .'
            write(6,'(a)') ' Options:'
            write(6,'(2a,/,9x,a/,9x,a)') &
                     ' -f FILE : The wavefunction ' &
                    ,'is read from file ' &
                    ,'  FILE rather than from ./psi', &
           '  The string FILE may be a relative or a full path-name.' 
            write(6,'(a)') ' -i DIR  : data is read from directory DIR'
            write(6,'(2a)') ' -n step : Print the A-vector', &
           ' only every step-th output.'
            write(6,'(2a,/10x,a)') ' -thr thr: Print only', &
           ' coefficients whose absolute values', &
           ' are larger than thr (default: 10^-8).' 
            write(6,'(2a,/10x,a)') ' -max mx : Print only', &
           ' coefficients whose absolute values', &
           ' are smaller or equal than mx.'
            write(6,'(2a,/10x,a)') ' -uplo ul: Print only', &
           ' coefficients from the upper (ul="U") or lower (ul="L")', &
           ' "triangle" of the A-tensor.'
            write(6,'(11x,2a,/11x,2a)') 'If ul="L" is set, only ', &
                 'A_j1_j2... with', &
                 '(j1-1)/(n1-1) + (j2-1)/(n2-1) + ... <= 1', &
                 ' are taken into account,'
            write(6,'(11x,2a,/11x,2a)') 'if ul="U" is set, only ', &
                 'A_j1_j2... with', &
                 '(j1-1)/(n1-1) + (j2-1)/(n2-1) + ... > 1', &
                 ' are taken into account.'
            write(6,'(a)') ' -n1  n1 : Print only coefficients with J.ge.n1.'
            write(6,'(a)') ' -n2  n2 : Print only coefficients with J.le.n2.'
            write(6,'(a)') ' -t   tm : Print only WFs with time .le. tm.'
            write(6,'(a)') ' -at atm : Print only WFs with time .ge. atm.'
            write(6,'(a)') ' -s  s   : Print only coefficients for state s.'
            write(6,'(2a,/10x,a)') ' -jx m n : Print only', &
           ' coefficients when the index of the m-th mode ', &
           ' takes the value n.'
            write(6,'(a)') &
           ' -count  : Count the number of coeffs between thr and max.'
            write(6,'(a)') &
           ' -r      : Take the restart file rather than the psi file.'
            write(6,'(a)')  ' -rst    : Same as -r.'
            write(6,'(a)')  ' -abs    : Print abs(psi).'
            write(6,'(a)')  ' -abs2   : Print abs(psi)**2.'
            write(6,'(a)')  ' -re     : Print Re(psi).'
            write(6,'(a)')  ' -im     : Print Im(psi).'
            write(6,'(a/10x,a)') &
             ' -nofs   : No transformation to fs.',&
             ' Use when "time-not-fs" was set in mctdh.'
            write(6,'(a)')  ' -ver    : Version information about the program.'
            write(6,'(a)')  ' -dblout : Print out coefficients in full.'
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ', &
                     ' -?      : Print this help text. '
            write(6,'(/2a)') 'It is in general useful to pipe the', &
            ' output into less: rdacoeff | less' 
            write(6,'(2a)') 'or to re-direct it into a file:', &
                            '  rdacoeff > acoeff '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      
      if (iarg .lt. i) goto 200

 200  continue

      if( step .gt.1  .and. lshowrst) then
         write(6,'(a)') ' -n and -r options are incompatible'
         stop
      end if


!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
! Otherwise take name from filein
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         if (lshowrst) then
            filein=name(1:laenge)//'/restart'
         else 
            filein=name(1:laenge)//'/psi'
         endif
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      open(ipsi,file=filein(1:laein),form='unformatted',status='old',err=1000)
      write(6,'(2a)') ' The inputfile is : ',filein(1:laein)

      return 
      
 1000 write(6,*)' Cannot open file : '
      write(6,*) filein(1:laein)
      stop
      
      end subroutine rdacinput

      end program rdacoeff
