!***********************************************************************
!
!                        ANALYSE MODULE RDAUTO
!
! Reads the autocorrelation function from file "auto".
!
! 11/96 MB
!
!
!***********************************************************************

      program rdauto

use decimal, only: long,dop
use global
use aglobal
use versions
use maxv, only: maxdim
use dvrdatmod
use griddatmod
use channels
use plot

use logdat
use plotutils, only: plotdefs,initplot,timebnds,databnds,dotask,plotmenu

use module_interface

      implicit none

      integer(long), external :: myiargc,slen
      integer(long)           :: i,iarg,n
      real(dop)               :: reauto,imauto,absauto,pltime1 
      character(len=80)       :: headline, buf

! interactive plot variables
      integer(long)           :: ndata,plottask,task,plotdim,dimx,dimy,&
                                 nsets,nplots
      character(len=c5)       :: filename

      real(dop), dimension(1)                        :: ydum,xdum,zdum,edum
      integer(long), dimension(:), allocatable, save :: idum
      real(dop), dimension(:), allocatable, save     :: pltime
      real(dop), dimension(:,:), allocatable, save   :: pldata

      integer(long)     :: ierr
      character(len=10) :: cerr
      logical(kind=4)   :: lopen

!-----------------------------------------------------------------------
! Initialise variables
!-----------------------------------------------------------------------
      call default
      call adefault
      filein = '  '
      name   = './'

!-----------------------------------------------------------------------
! read options
!-----------------------------------------------------------------------
      iarg = myiargc()
      if (iarg .lt. 1) goto 2

      i = 0
   1  i=i+1
      if (myiargc() .ge. 1) then
         call mygetarg(i,buf)
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf .eq. '-inter' ) then
            linter=.true.
         else if( buf .eq. '-gnupl4 ') then
            lgnu4 = .true.                 ! gnuplot ver < 5
         else if( buf .eq. '-ver' ) then
            call wrversion(6)
            stop

         else if( buf .eq. '-h' .or.  buf .eq. '-?') then
            write(6,'(1x,a/,11x,a)') &
        'Purpose : Reads the autocorrelation from file "auto" ',&
        'Usage   : rdauto [-f -i -inter -gnupl4 -ver -h -?].'
            write(6,'(a)') ' Options : '
            write(6,'(2a,/,11x,a/11x,a)') &
              ' -f FILE : the data is read '&
              ,'from file FILE','rather than from ./auto'&
              ,'The string FILE may be a relative or full path-name.'
            write(6,'(a)') ' -i DIR : data is stored in directory DIR'
            write(6,'(a)') ' -inter : enables interactive plotting'
            write(6,'(a)')&
               ' -gnupl4  : Use gnuplot version < 5 for plotting '
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
              ' -h      : Print this help text. ',&
              ' -?      : Print this help text. '
            stop
         else
            write(6,'(2a)') ' Unknown option :', buf
            write(6,'(2a)') ' Use option -h'
            stop
         end if
         if (iarg .le. i) goto 2
         goto 1

      end if
         
    2 continue
!-----------------------------------------------------------------------
! If filein has not been set, look in name directory. 
! Otherwise take name from filein
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         filein=name(1:laenge)//'/auto'
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

!-----------------------------------------------------------------------
! Open and read file "auto" 
!-----------------------------------------------------------------------
      open(iaut,file=filein(1:laein),form='formatted',&
           status='old',err=666)
      read(iaut,'(a)') headline

! get no. of data sets
      ndata=0
      do
         read(iaut,*,end=50,err=666) pltime1,reauto,imauto,absauto

         ndata=ndata+1
      enddo
  50  continue

! no. of data sets to be plotted
      maxdata = ndata
! space in label arrays for Time etc.
      maxdim = 3
! allocate data arrays
      cerr = 'pltime'
      allocate(pltime(maxdata),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'pldata'
      allocate(pldata(maxdata,3),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'idum'
      allocate(idum(maxdim),stat=ierr)
      if (ierr > 0) go to 1000

! set defaults
      idum(1)=0

! read data
      rewind(iaut)
      read(iaut,'(a)') headline
      do n = 1,ndata
         read(iaut,*,err=666) pltime1,reauto,imauto,absauto
         pltime(n)=pltime1
         pldata(n,1)=reauto
         pldata(n,2)=imauto
         pldata(n,3)=absauto
      enddo

      close(iaut,err=666)

!-----------------------------------------------------------------------
! non-interactive plotting - write to screen
!-----------------------------------------------------------------------
      if (.not. linter) then
         write(6,'(76a)') '#',('-',i=1,75)

         do n=1,ndata
            write(6,'(f8.3,3e20.8)') pltime(n),pldata(n,1),pldata(n,2),pldata(n,3)
         enddo

         write(6,'(76a)') '#',('-',i=1,75)
         go to 999
      endif

!-----------------------------------------------------------------------
! interactive plotting
! open file for data to be plotted
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/tmpplot.pl'
      open(iaus,file=filename,status='unknown')

!-----------------------------------------------------------------------
! allocate arrays for plotting
! dvrdat and grddat needed for label arrays
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      nplottask=3
      call alloc_plot

!-----------------------------------------------------------------------
! zero dummy variables
!-----------------------------------------------------------------------
      xdum(1)=0.0
      ydum(1)=0.0
      zdum(1)=0.0

      call plotdefs

      plotdim=1
      dimx=ndata
      dimy=1
      nsets=1
      nplots=1
      lpltime=.true.
      call initplot(filename,plotdim)

      call timebnds(pltime,ndata)

!-----------------------------------------------------------------------
! set plot data, e.g. plot tasks
!-----------------------------------------------------------------------
      cplottask(1)='Real values'
      cplottask(2)='Imaginary values'
      cplottask(3)='Absolute values'
      do i=1,nplottask
         cptlen(i)=slen(cplottask(i))
      enddo

      plottask=3

      do 
         call plotmenu(task,plottask,plotdim)
         if (task .eq. 0) exit

!-----------------------------------------------------------------------
! do task
! TASK=0 STOP  else return to menu
!-----------------------------------------------------------------------
         if (plottask .eq. 1) then
            plttitle='Real autocorrelation function'
            axislab(1)='Time [fs]'
            axislab(3)='Re[C(t)]'
         else if (plottask .eq. 2) then
            plttitle='Imaginary autocorrelation function'
            axislab(1)='Time [fs]'
            axislab(3)='Im[C(t)]'
         else if (plottask .eq. 3) then
            plttitle='Absolute autocorrelation function'
            axislab(1)='Time [fs]'
            axislab(3)='Abs[C(t)]'
         endif
         if (lrebound) then
            call databnds(pldata(1,plottask),dimx,dimy,nsets,nplots,&
                    ndata,1,1,1)
         endif
         call dotask(task,plottask,plotdim,pltime,ydum,&
              pldata(1,plottask),dimx,dimy,nsets,nplots,ndata,1,1,1,&
              xdum,1,pltime,ndata,edum,1,zdum,1,1,1,1,idum)

      enddo

  999 continue
      inquire(unit=iaus,opened=lopen)
      if (lopen) close(iaus,status='delete')

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_plot
      if (allocated(pltime)) deallocate(pltime)
      if (allocated(pldata)) deallocate(pldata)
      if (allocated(idum)) deallocate(idum)

      stop

 666  continue
      write(6,*)
      write(6,'(a,/a)')&
         'ERROR in program rdauto: Cannot open or read auto file:'&
         , filein(1:laein)
      write(6,*)
      stop

 1000 write(ilog,'(a)') 'Error allocating memory in RDAUTO :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop

      end program rdauto

