!***********************************************************************
!
!                        ANALYSE MODULE RDCHECK
!
! Reads the data from file "check".
! State poulations are written to file "spop.pl", natural populations
! are written to file natpop.pl and coordinate expectation values and 
! widths are written to file qdq.pl
!
! 04/99 HDM  
!
!
!***********************************************************************
 
      program rdcheck

use decimal, only: long,dop
use global
use aglobal
use dglobal
use versions
use maxv, only: maxdim,maxsta
use keyunitsmod
use dvrdatmod
use griddatmod
use psidef
use datenmod
use memdimmod
use basismod
use channels

use op1lib, only: zeromxd
use iofile, only: rdchkinfo
use rdfilesmod, only: rdchk,lrdchk

use module_interface

      implicit none

      integer(long)       :: mxorb 
      integer(long)       :: s,t,i,imax,m,n,f,k,kk,ns,nmd,ndf,laenat,&
                             laeqdq,idt,gq,nstate2,ilbl,ll,check,laegg,&
                             dd,pnode
      integer(long), dimension(3)                     :: iargs
      real(dop)                                       :: time,dt,norm,trace,spm,etmp
      real(dop)                                       :: enav,enstd,normav,normstd
      real(dop), dimension(:), allocatable            :: spop,ecorr,etot
      real(dop), dimension(:), allocatable            :: tr2,dicht
      real(dop), dimension(:,:), allocatable          :: expn,dexpn,dmx
      real(dop), dimension(:,:,:), allocatable        :: qdq
      real(dop), dimension(:), allocatable            :: dicht3,ggp
      logical(kind=4)                                 :: lren,ltr2,lcheck, &
                                                         lentropy,llast,lml
      character(len=c5)                               :: filenat,fileqdq,fileggp
      character(len=c5)                               :: mdlbl
      character(len=20)                               :: mdlab
      character(len=80)                               :: nspf
      character(len=c1)                               :: enunit,cfs,cfs2

      integer(long)     :: ierr
      character(len=10) :: cerr

!-----------------------------------------------------------------------
!     Set defaults. Set up the input.
!-----------------------------------------------------------------------
      ns = 0
      nmd = 0
      ndf = 0

      write(6,'(/,78a1,/)')  ('*',i=1,78)
      write(6,'(2a)') ' Analyse program "rdcheck" ',&
                      '(Read check file)'
      iargs=0
      call default
      call adefault
      call setunits
      call chkinput(lren,ltr2,lentropy,llast,dt,gq,iargs,lml,enunit)

      cfs='[fs]'
      cfs2=' fs'
      if(fs.lt.2) cfs='[au]'
      if(fs.lt.2) cfs2=' '
      if(lml) then
         pnode=ns
         ns = 1
         if(.not.lrunml) then
            write(6,'(/2a/)') '### ERROR: ', &
              'The -ml option can be used only for ML-runs.'
            stop
         endif
         if(filever(ichk) .lt. 8.5011d0) then
            write(6,'(/3a/)') '### ERROR: ', &
              'The version of the check file is too old ', &
              'for usage with -ml option.'
            stop
         endif
      else
         pnode = 1
      endif


!-----------------------------------------------------------------------
!  Opening input file.
!-----------------------------------------------------------------------
      open(ichk,file=filein(1:laein),form='unformatted',status='old',err=97)
      if(laein .le. 58 ) then
         write(6,'(2a)') ' The inputfile is  : ',filein(1:laein)
      else
         write(6,'(a)')  ' The inputfile is  :' 
         write(6,'(a)')   filein(1:laein)
      end if

!-----------------------------------------------------------------------
!  Reading the file header
!
!  since version 8.2002 there are <q> and <dq> informations in the 
!  check - file
! 
!-----------------------------------------------------------------------
      read(ichk) filever(ichk)
      if (filever(ichk) .lt. 11.0_dop) then
         write(6,'(a)') '****************************************'
         write(6,'(2a)')   &
          'For check files older than version 11.0 use appropriate ', &
          'Rdcheck program'
         write(6,'(a,f8.4)') ' Check file version =', filever(20)
         write(6,'(a)') '****************************************'
         stop
      endif
         
!-----------------------------------------------------------------------
!  allocate memory
!-----------------------------------------------------------------------
      call rdmemdim(ichk)

      maxtape = 1000
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef
      if (ldynspf) call alloc_dynwf

!-----------------------------------------------------------------------
!  Read Check file info and print basis set information.
!-----------------------------------------------------------------------
      check = 1
      call rdchkinfo(ichk,check,check,check,check)
      tinit = tinit/fs
      tfinal = tfinal/fs
      out1  = out1/fs

!-----------------------------------------------------------------------
!  Prompting for missing input
!-----------------------------------------------------------------------
      if (lrdchk(2)) then
         if (nmode .gt. 1 .and. .not. leb .and. iargs(1) .eq. 0) then
            write(6,'(a)') ' Enter mode for analysis '
            read(5,*) nmd
         else
            nmd = max(1,iargs(1))
         endif
         if (nstate .gt. 1 .and. iargs(2) .eq. 0) then
            write(6,'(a)') ' Enter state for analysis '
            read(5,*) ns
         else
            ns = max(1,iargs(2))
         endif
         if (ns .gt. 0) then
            write(6,'(a,i3)') ' Analysis for electronic state: ns =',ns
         endif
         if (nmd .gt. 0) then
            write(6,'(a,i3)') ' Analysis for mode: nmd =',nmd
         endif
      endif
      if (lrdchk(3)) then
         if (ndof .gt. 1 .and. .not. leb .and. iargs(1) .eq. 0) then
            write(6,'(a)') ' Enter DOF for analysis '
            read(5,*) ndf
         else
            ndf = max(1,iargs(1))
         endif
         if (nstate .gt. 1 .and. iargs(2) .eq. 0) then
            write(6,'(a)') ' Enter state for analysis '
            read(5,*) ns
         else
            ns = max(1,iargs(2))
         endif
         if (ns .gt. 0) then
            write(6,'(a,i3)') ' Analysis for electronic state: ns =',ns
         endif
         if (ndf .gt. 0) then
            write(6,'(a,i3)') ' Analysis for dof: ndf =',ndf
         endif
      endif
      if (lrdchk(4)) then
         if (nmode .gt. 1 .and. .not. leb .and. iargs(1) .eq. 0) then
            write(6,'(a)') ' Enter mode for analysis '
            read(5,*) nmd
         else
            nmd = max(1,iargs(1))
         endif
         if (nstate .gt. 1 .and. iargs(2) .eq. 0) then
            write(6,'(a)') ' Enter state for analysis '
            read(5,*) ns
         else
            ns = max(1,iargs(2))
         endif
         if (ns .gt. 0) then
            write(6,'(a,i3)') ' Analysis for electronic state: ns =',ns
         endif
         if (nmd .gt. 0) then
            write(6,'(a,i3)') ' Analysis for mode: nmd =',nmd
         endif
      endif

!-----------------------------------------------------------------------
! GGP can onl be read for GWP calculations
! Check GGP requested for GWP mode
!-----------------------------------------------------------------------
      if (lrdchk(4)) then
         if (.not. lgwp) then
            write(6,'(a)') 'Error: Cannot plot GGPs for non-GWP calculation '
            stop
         endif
         if (.not. gwpm(nmd)) then
            write(6,'(a)') 'Error: Cannot plot GGPs for non-GWP mode '
            stop
         endif
      endif

!-----------------------------------------------------------------------
! natural populations can onl be read for MCTDH calculations
!-----------------------------------------------------------------------
      if (lrdchk(2) .and. psitype .ne.0 ) then
         write(6,'(a)') &
             'Error: Can only plot natural populations for MCTDH wavefunction '
         stop
      endif
      
!-----------------------------------------------------------------------
!  Check for legality of the arguments
!-----------------------------------------------------------------------
      if(ns .gt. nstate ) then
         write(6,'(/a,/a,i9,/a,i3,/a)')&
               '######################################################'&
              ,' Illegal value of ns =', ns&
              ,' ns must not be greater than nstate =',nstate    &
              ,'######################################################'
         stop 2
      end if
      if(nmd .gt. nmode ) then
         write(6,'(/a,/a,i9,/a,i3,/a)')&
               '######################################################'&
              ,' Illegal value of nmd =', nmd&
              ,' nmd must not be greater than nmode =',nmode    &
              ,'######################################################'
         stop 3
      end if
      if(ndf .gt. ndof ) then
         write(6,'(/a,/a,i9,/a,i3,/a)')&
               '######################################################'&
              ,' Illegal value of ndf =', ndf&
              ,' ndf must not be greater than ndof =',ndof    &
              ,'######################################################'
         stop 3
      end if

!-----------------------------------------------------------------------
!  Opening output files.
!-----------------------------------------------------------------------
      if (fileout(1:2) .eq. 'no') then
! switch output to screen
        iaus = 6
      else
         if (lrdchk(1)) then
            fileout=name(1:laenge)//'/spop.pl'
         else if (lrdchk(2)) then
            write(fileout,'(a,i5,a,i5,a)') name(1:laenge)//'/natpop_',nmd,'_',ns,'.pl'
            call rmblnk(fileout)
         else if (lrdchk(3)) then
            write(fileout,'(a,i5,a,i5,a)') name(1:laenge)//'/qdq_',ndf,'_',ns,'.pl'
            call rmblnk(fileout)
         else if (lrdchk(4)) then
            write(fileout,'(a,i5,a,i5,a)') name(1:laenge)//'/ggp_',nmd,'_',ns,'.pl'
            call rmblnk(fileout)
         else if (lrdchk(5)) then
            fileout=name(1:laenge)//'/etot.pl'
         endif

         if (any(lrdchk)) then
            laeout=index(fileout,' ')-1
            call abspath(fileout,laeout)
            open(iaus,file=fileout(1:laeout),form='formatted',&
                       status='unknown',err=98)
            write(6,'(2a)') ' The outputfile is : ',fileout(1:laeout)
         endif
      end if
      
      if (.not. any(lrdchk)) then

         write(6,'(78a1)')  ('-',i=1,78)
         write(6,'(a,i4)') ' Number of degrees of freedom:', ndof       
         write(6,'(a,i4)') ' Number of combined modes    :', nmode
         if (leb) then
            write(6,'(a,i4)') ' Singleset-formulation'
            write(6,'(a,i4)') ' Number of electronic states :', nddstate
         else
            write(6,'(a,i4)') ' Multiset-formulation'
            write(6,'(a,i4)') ' Number of electronic states :', nstate
         endif

         write(6,'(/,a)') ' mode dof  dim   DVR     modelabel'
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               write(6,'(1x,i3,1x,i4,i5,3x,a8,a10)') &
               m, f, gdim(f), cbas(basis(f)), modelabel(f)  
            enddo
         enddo

!-----------------------------------------------------------------------
! write mode information.
!-----------------------------------------------------------------------
         write(6,'(/,a)') ' mode: No. of spf''s per state : modelabels'
         do m=1,nmode
            ll = 1
            mdlbl = '                                                    '
            nspf  = '                                                    '
            do  n=1,nspfdof(m)
               ilbl  = index(modelabel(spfdof(n,m)),' ')
               mdlbl = mdlbl(1:ll)//modelabel(spfdof(n,m))(1:ilbl)
               ll    = ll + ilbl
            end do
            write(nspf,'(95i3)') (dim(m,s),s=1,nstate)
            if(3*nstate+ll .le. 72 ) then
               write(6,'(i4,4a)') m,' :',nspf(1:3*nstate),' : ',&
                                  mdlbl(1:ll)
            else
               write(6,'(i4,3a)') m,' :',nspf(1:3*nstate),' :'
               write(6,'(a,/)') mdlbl(1:ll)
            end if
         enddo

         write(6,'(/,3(a,f11.3,a))') &
             ' tinit =', tinit, 'fs','    tout =', out1, 'fs', &
                    '    tfinal =', tfinal, 'fs' 
      endif  ! (i6.ne.6)

! get maximum no. of spfs
      mxorb=0
      if(dentype .eq. 1 ) then
         do s=1,nstate
            do t=1,nstate
               do m=1,nmode
                  mxorb=max(mxorb,ddim(m,s,t))
               enddo
            enddo
         enddo
      else
         do s=1,nstate
            do m=1,nmode
               mxorb=max(mxorb,dim(m,s))
               if (gwpm(m)) mxorb=max(mxorb,gwpdim(m,s))
            enddo
         enddo
      endif

      cerr = 'spop'
      allocate(spop(nddstate),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ecorr'
      allocate(ecorr(maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'etot'
      allocate(etot(maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tr2'
      allocate(tr2(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dicht'
      allocate(dicht(mxorb),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dmx'
      allocate(dmx(maxdim,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'qdq'
      allocate(qdq(4,ndof,nstate),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'expn'
      allocate(expn(maxdim,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dexpn'
      allocate(dexpn(maxdim,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dicht3'
      allocate(dicht3(d3matdim),stat=ierr)
      if (ierr > 0) go to 1000
      if (lgwp) then
         cerr = 'ggp'
         allocate(ggp(gwpinfodim),stat=ierr)
         if (ierr > 0) go to 1000
      else
         cerr = 'ggp'
         allocate(ggp(1),stat=ierr)
         if (ierr > 0) go to 1000
      endif

      spop=0.0_dop
      ecorr=0.0_dop
      etot=0.0_dop
      tr2=0.0_dop
      dmx=0.0_dop
      qdq=0.0_dop
      expn=0.0_dop
      dexpn=0.0_dop
      dicht3=0.0_dop
      ggp=0.0_dop

      enav = 0.0_dop
      enstd = 0.0_dop
      normav = 0.0_dop
      normstd = 0.0_dop
      
! --- write the gnuplot commands, if desired, to the fileout-file
! note the '" \\' written a3 outputs only "\", but this is required
! as some compilers need "\\" to ouput "\"
      if(lgnu .and. lrdchk(1)) then
         write(iaus,'(3a)') 'set title "State populations,    '&
                          ,filein(1:laenge),'"'
         write(iaus,'(a)') 'set style data lines'
         write(iaus,'(a)') 'set nologscale'
         if( fs.ge.40.0_dop) then
            write(iaus,'(a)') 'set xlabel "time[fs]"'
         else
            write(iaus,'(a)') 'set xlabel "time"'
         endif
         write(iaus,'(a)') 'set yrange [0:1.0]'
         if ( nddstate .eq. 1 ) then
            write(iaus,'(3a)') 'plot "',fileout(1:laeout),'" us 1:2 notit'

         else if( nddstate .eq. 2 ) then
            write(iaus,'(2a,a3)') 'plot "',fileout(1:laeout),'" \\'
            write(iaus,'(2a)') '    us 1:2 title " State  1",',&
                             ' "" us 1:3 title " State  2" '
         else
            write(iaus,'(2a,a3)') 'plot "',fileout(1:laeout),'" \\'
            write(iaus,'(a32)') '    us 1: 2 title " State  1", \\'
            do i = 3, nddstate
               write(iaus,'(a,i2,a,i3,a4)')&
                    ' "" us 1:',i,' title " State',i-1,'", \\'
            end do
            write(iaus,'(a,i2,a,i3,a)')&
                 ' "" us 1:',nddstate+1,' title " State',nddstate,'"'
         end if
         write(iaus,'(a)')  'quit'
         write(iaus,'(a)')  '#--#--#'
         write(iaus,'(a,a5,2x,99(a,i2,a))')&
           '#  time',cfs,  ('  s-pop(',i,')  ',i=1,nddstate)
      end if

! --- write the gnuplot commands, if desired, to the filenat-file
      if(lgnu .and. lrdchk(2)) then
         write(iaus,'(a)') 'set style data lines'
         write(iaus,'(a)') 'set logscale y'
         if( fs.ge.40.0_dop) then
            write(iaus,'(a)') 'set xlabel "time[fs]"'
         else
            write(iaus,'(a)') 'set xlabel "time"'
         endif
         write(iaus,'(a)') 'set yrange [1e-6:1.0]'
         if(lren) then
            if (lml) then
               write(iaus,'(a,i2,a,i3,1x,a2)')&
                     'set title "Natural populations re-normalised, nodes =',&
                     ns,', mode =',nmd,' \\'
            else
               write(iaus,'(a,i2,a,i3,1x,a2)')&
                     'set title "Natural populations re-normalised, state =',&
                     ns,', mode =',nmd,' \\'
            endif
            write(iaus,'(2a)') filein(1:laenge),'"'
         else
            write(iaus,'(a,i2,a,i3,1x,a2)')&
             'set title "Natural populations, state =',&
                 ns,', mode =',nmd,' \\'
            write(iaus,'(2a)') filein(1:laenge),'"'
         end if
         write(iaus,'(3a)') 'plot "',fileout(1:laeout),'" us 1:2 notit'
         if(lml) then
            dd=30
         else
            dd=dim(nmd,ns)
         endif
         do i = 3, dd
            write(24,'(a,i3,a11)') ' "" us 1:',i,' notitle, \\'
         end do
         write(24,'(a,i3,a)') ' "" using 1:',dd+1,' notitle'
         write(iaus,'(a)')  'quit'
         write(iaus,'(a)')  '#'

         write(iaus,'(a,i3,5x,a,i3)') '# state =', ns,' n-mode =', nmd
         if(lren) then
            write(iaus,'(a,a5,4x,a)') '#  time',cfs,&
                  ' natural populations re-normalised'
         else
            write(iaus,'(a,a5,a)') '#  time',cfs,' natural populations '
         end if
      end if


! --- write the gnuplot commands, if desired to the fileqdq-file
      if(lgnu .and. lrdchk(3)) then
         write(iaus,'(3a,i3,a)') 'set title "<q>,    '&
                          ,filein(1:laenge),'  state =',ns,'"'
         write(iaus,'(a)') 'set style data lines'
         write(iaus,'(a)') 'set nologscale'
         if( fs.ge.40.0_dop) then
            write(iaus,'(a)') 'set xlabel "time[fs]"'
         else
            write(iaus,'(a)') 'set xlabel "time"'
         endif
         write(iaus,'(a)') 'set ylabel "<q> in a.u." '

         write(iaus,'(2a,a3)') 'plot "',fileout(1:laeout),'" \\'

         write(iaus,'(a,a10,a)') '   us 1: 2 title "',modelabel(ndf),'" '

! -----  gnuplot commands for <dq>

         write(iaus,'(a)')     'pause -1 "Hit return to continue"'
         write(iaus,'(a)')  'set ylabel "<dq> in a.u." '
         write(iaus,'(3a,I3,a)') 'set title "<dq>,    '&
                      ,filein(1:laenge),'  state =',ns,'"'
         write(iaus,'(2a,a3)') 'plot "',fileout(1:laeout),'" \\'
         write(iaus,'(a,a10,a)') '   us 1: 3 title "',modelabel(ndf),'" '
         write(iaus,'(a)') 'quit'

         write(iaus,'(a,i3)') '#  time[fs]      <q>        <dq>   for state s = ',ns
      end if

! --- write the gnuplot commands, if desired, to the fileggp-file
      if(lgnu .and. lrdchk(4)) then
         write(iaus,'(a)') 'set style data lines'
   !      write(iaus,'(a)') 'set logscale y'
         if( fs.ge.40.0_dop) then
            write(iaus,'(a)') 'set xlabel "time[fs]"'
         else
            write(iaus,'(a)') 'set xlabel "time"'
         endif
         write(iaus,'(a)') 'set yrange [*:*]'
         write(iaus,'(a,i2,a,i3,1x,a2)')&
             'set title "Gross Gaussian populations, state =',&
                 ns,', mode =',nmd,' \\'
         write(iaus,'(2a)') filein(1:laenge),'"'
         if ( gwpdim(nmd,ns) .eq. 1 ) then
            write(iaus,'(3a)') 'plot "',fileout(1:laeout),'" us 1:2 notit'

         else if( gwpdim(nmd,ns) .eq. 2 ) then
            write(iaus,'(2a,a3)') 'plot "',fileout(1:laeout),'" \\'
            write(iaus,'(2a)') '    us 1:2 title " GGP  1",',&
                             ' "" us 1:3 title " GGP  2" '
         else
            write(iaus,'(2a,a3)') 'plot "',fileout(1:laeout),'" \\'
            write(iaus,'(a30)') '    us 1: 2 title " GGP  1", \\'
            do i = 3, gwpdim(nmd,ns)
               write(iaus,'(a,i2,a,i3,a4)')&
                    ' "" us 1:',i,' title " GGP',i-1,'", \\'
            end do
            write(iaus,'(a,i2,a,i3,a)')&
                 ' "" us 1:',gwpdim(nmd,ns)+1,' title " GGP ',gwpdim(nmd,ns),'"'
         end if
         write(iaus,'(a)')  'quit'
         write(iaus,'(a)')  '#'

         write(iaus,'(a,i3,5x,a,i3)') '# state =', ns,' n-mode =', nmd
         write(iaus,'(a)') '#  time[fs]    Gross Gaussian populations '
      end if

      if(lgnu .and. lrdchk(5)) then
         write(iaus,'(3a)') 'set title "Total Energy,    '&
                          ,filein(1:laenge),'"'
         write(iaus,'(a)') 'set style data lines'
         write(iaus,'(a)') 'set nologscale'
         if( fs.ge.40.0_dop) then
            write(iaus,'(a)') 'set xlabel "time[fs]"'
         else
            write(iaus,'(a)') 'set xlabel "time"'
         endif
         write(iaus,'(a)') 'set yrange [*:*]'
         write(iaus,'(3a)') 'plot "',fileout(1:laeout),'" us 1:2 notit'

         write(iaus,'(a)')  'quit'
         write(iaus,'(a)')  '#--#--#'
         if (dentype .gt. 0) then
            write(iaus,'(4a)')&
              '#  time[fs]  ', '     Etot', '            Norm', '           Trace'
         else
            write(iaus,'(3a)')&
              '#  time[fs]  ', '     Etot', '            Norm'
         endif
      end if

      if(ltr2) then
         if(lml) then
            write(6,'(/a,i3)') 'Trace rho**2 for node =',pnode
            write(6,'(5x,a,6x,a,8(9x,a,i1),99(8x,a,i2))') 'Time', &
                     'm=1',('m=',i,i=2,6)
         else
            write(6,'(/a,i3)') 'Trace rho**2 for state =',max(1,ns)
            write(6,'(5x,a,6x,a,8(9x,a,i1),99(8x,a,i2))') 'Time', &
                     'm=1', ('m=',i,i=2,nmode)
         endif

      elseif(lentropy) then
         if(lml) then
            write(6,'(/a,i3)') 'Entropy for node =',pnode
            write(6,'(5x,a,6x,a,8(9x,a,i1),99(8x,a,i2))') 'Time', &
                     'm=1',('m=',i,i=2,6)
         else
            write(6,'(/a,i3)') 'Entropy for state =',max(1,ns)
            write(6,'(5x,a,6x,a,99(9x,a,i1))') 'Time','m=1',('m=',i,i=2,nmode)
         endif

      end if

!-----------------------------------------------------------------------
!     read the DATA section of the check file
!-----------------------------------------------------------------------
      rewind(ichk)
      if (filever(ichk) .lt. 11.0_dop) then
         read(ichk) filever(ichk)
         check = 1
         call rdchkinfo(ichk,check,check,check,check)
      endif
      lend=.false.
      k=0
      do 
         if (lml) then
            pnode=-1
            print*,'ML-MCTDH layer populations not yet implemneted'
            stop
    !        call rdmldicht(pnode,mldicht,mlddim,nmode_ml)
         else
            call rdchk(ichk,ecorr,etot,spop,dicht3,ggp,qdq,norm,trace)
         endif

         if (lend) exit

         k=k+1
         time = tinit +(k-1)*out1

!-----------------------------------------------------------------------
! output state populations
!-----------------------------------------------------------------------
         if (lrdchk(1)) then
            spm=spop(1)
            do s = 2, nstate
               spm=max(spm,spop(s))
            enddo
            if (leb) then
               if(spm.gt.999.d0) then
                  write(iaus,'(f11.3,599e20.10)') time,(spop(s),s=1,nddstate)
               else
                  write(iaus,'(f11.3,599f13.8)') time, (spop(s),s=1,nddstate)
               endif
            else
               if(spm.gt.999.d0) then
                  write(iaus,'(f11.3,599e20.10)') time,(spop(s),s=1,nstate)
               else
                  write(iaus,'(f11.3,599f13.8)') time, (spop(s),s=1,nstate)
               endif
            endif
         endif

!-----------------------------------------------------------------------
! output natural orbital populations
!-----------------------------------------------------------------------
         if (lrdchk(2)) then
            if ((dentype.eq.0).or.(dentype.eq.2)) then
               nstate2 = 1
            else
               nstate2 = nstate
            endif

            do i = 1, dim(nmd,ns)
               dicht(i) = dicht3(d3mat(nmd,ns)+i-1)
               if (dicht(i) .lt. 1.0d-16) dicht(i)=0.0_dop
            end do
            if (lren) then
               do i = 1, dim(nmd,ns)
                  dicht(i) = dicht(i)/(spop(ns)+1.0e-6_dop)
               end do
            end if

! calculate max populations
            do s = 1, nstate
               do t=1,nstate2

                  do m=1,nmode
                     if (llast) then
                        if(filever(ichk) .ge. 8.1009d0) then
                           dmx(m,s) = dicht3(d3mat(m,s)+dim(m,s)-1)
                        else
                           dmx(m,s) = dicht3(d3mat(m,1)+dim(m,s)-1)
                        endif
                     elseif( k.gt.nskip .and. k.le.ncut) then
                        if(filever(ichk) .ge. 8.1009_dop) then
                           if  ((dentype.eq.0).or.(dentype.eq.2)) then
                              dmx(m,s) = max(dmx(m,s),dicht3(d3mat(m,s)+dim(m,s)-1))
                           else
                              dmx(m,s) = max(dmx(m,s),dicht3(d3mat(m,s)+ddim(m,s,t)-1))
                           endif
                        else
                            print*,'Not sure of dentype = 1 natorb max plotting'
!                           dmx(m,s) = dmax1(dmx(m,s),dicht3(m,s,t))
                        endif
                     endif
                  enddo
               enddo
            enddo


            write(iaus,'(f11.3,1p599e11.3)') time,(dicht(i),i=1,dim(nmd,ns))

            if( dt .gt. 0.0_dop  .and.  time .gt. 0.5_dop*dt ) then
               idt = int((time+0.3_dop*out1)/dt)
               if( abs(time-idt*dt) .le. 0.3_dop*out1 )  then
                  write(6,'(78a1)')  ('-',i=1,78)
                  if(llast) then
                     write(6,'(a,f11.2,a)') &
                         ' Lowest nat.-weight;    time :', time,' fs'
                  else
                     write(6,'(2a,f11.2,a)') ' Maximum over time of', &
                        ' lowest nat.-weight;    time :', time,' fs'
                  endif
                  write(6,'(a,8(a,i2,2x))')' mode',('    s =',s,s=1,nstate)
                  do m = 1, nmode
                     write(6,'(i4,1x,1p8e11.3)') m,(dmx(m,s),s=1,nstate)
                  end do
               end if
            end if

            if (ltr2) then
               s=max(1,ns)
               do m = 1, nmode
                   tr2(m)=0.0_dop
                  do i = 1, dim(m,s)
                     tr2(m) = tr2(m) + dicht3(d3mat(m,s)+i-1)**2
                  enddo
                  tr2(m) = tr2(m)/(spop(s)**2+1.d-50)
               enddo
               write(6,'(f9.2,9f12.8)') time, (tr2(m),m=1,nmode)
            elseif (lentropy) then
               s=max(1,ns)
               do m = 1, nmode
                  tr2(m)=0.d0
                  do i = 1, dim(m,s)
                     tr2(m)=tr2(m) - dicht3(d3mat(m,s)+i-1)/(spop(s)+1.d-50) * &
                         log(max(1.d-50,dicht3(d3mat(m,s)+i-1)/(spop(s)+1.d-50)))
                  enddo
               enddo
               write(6,'(f9.2,9f12.6)') time, (tr2(m),m=1,nmode)
            endif
         endif

!-----------------------------------------------------------------------
!    write coordinate expectation values
!-----------------------------------------------------------------------
         if (lrdchk(3)) then
            write(iaus,'(f12.3,2(1x,f14.8))') time,qdq(1,ndf,ns),qdq(2,ndf,ns)
         endif

!-----------------------------------------------------------------------
!    write GGP values
!-----------------------------------------------------------------------
         if (lrdchk(4)) then
            do i = 1, gwpdim(nmd,ns)
! check for underflow
               dicht(i) = max(ggp(zgwpinfo(nmd,ns)+i-1),1.0d-8)
            end do
            write(iaus,'(f11.3,599f12.6)') time,&
                    (dicht(i),i=1,gwpdim(nmd,ns))
         endif

!-----------------------------------------------------------------------
! output total energy
!-----------------------------------------------------------------------
         if (lrdchk(5)) then

            etmp = 0.0_dop
            do s=1,nstate
               etmp = etmp + etot(s)
            enddo

            enav = enav + etmp
            enstd = enstd + etmp*etmp
            normav = normav + norm
            normstd = normstd + norm*norm
            if (dentype .gt. 0 .and. lrlx) etmp = etmp/trace
            if (dentype .gt. 0) then
               write(iaus,'(f11.3,3f16.9)') time, etmp*ev, norm, trace
            else
               write(iaus,'(f11.3,2f16.9)') time, etmp*ev, norm
            endif
         endif

      end do                    ! of k

      if (lrdchk(2) .and. (.not.(ltr2 .or. lentropy))) then
         write(6,'(78a1)')  ('-',i=1,78)
         if(llast) then
            write(6,'(a,f11.2,a)') ' Lowest nat.-weight;    time :', time,' fs'
         else
            write(6,'(2a,f11.2,a)') ' Maximum over time of', &
               ' lowest nat.-weight;    time :', time,' fs'
         endif
         k = 1
 80      kk = min(nstate,k+5)
         write(6,'(a,8(a,i2,2x))') ' mode',('    s =',s,s=k,kk)
         do m = 1, nmode
            write(6,'(i4,1x,1p8e11.3)') m,(dmx(m,s),s=k,kk)
         enddo
         if(kk .lt. nstate) then
            write(6,*)
            k = k + 6
            goto 80
         endif
      endif
      write(6,'(78a1)')  ('-',i=1,78)

      if (lrdchk(5)) then
         enav = enav / dble(k)
         enstd = enstd/dble(k)
         enstd = dsqrt(enstd - enav*enav)
         normav = normav / dble(k)
         normstd = normstd/dble(k)
         normstd = dsqrt(normstd - normav*normav)

         write(6,'(/,a,f20.8,a)') 'Average Energy :',enav*ev, 'eV'
         write(6,'(a,f20.8,a)') 'Energy STD     :',enstd*ev, 'eV'
         write(6,'(a,f20.8)') 'Average Norm   :',normav
         write(6,'(a,f20.8)') 'Norm STD       :',normstd
      endif

      deallocate(spop)
      deallocate(ecorr)
      deallocate(etot)
      deallocate(tr2)
      deallocate(dmx)
      deallocate(qdq)
      deallocate(expn)
      deallocate(dexpn)
      deallocate(dicht)
      deallocate(dicht3)
      deallocate(ggp)

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_psidef

! --- close all open files
      inquire(ichk,opened=lcheck)
      if (lcheck) close(ichk)
      inquire(iaus,opened=lcheck)
      if (lcheck) close(iaus)
     
      stop

 97   write(6,*)' Cannot open input (check) file : '
      write(6,*) filein
      stop
      
 98   write(6,*)' Cannot open output file : '
      write(6,*) fileout
      stop

 1000 write(ilog,'(a)') 'Error allocating memory in Rdcheck :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop

      contains

!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------
      subroutine chkinput(lren,ltr2,lentropy,llast,dt,gq,iargs,lml,enunit) 

use compdatmod
use compdat1mod

      implicit none

      integer(long), external        :: myiargc
      integer(long)                  :: iarg,ierr,i,laenat,laeqdq,itask
      integer(long), intent(out)     :: gq
      integer(long), dimension(3), intent(out)     :: iargs
      real(dop), intent(out)         :: dt
      character(len=80)              :: buf
      character(len=c1), intent(out) :: enunit
      logical(kind=4), intent(out)   :: lren,ltr2, lentropy, llast, lml

!-----------------------------------------------------------------------

      name ='./'
      laenge=1
      filein  = ' '
      fileout = '  '
      lgnu = .false.
      lren = .false.
      ltr2 = .false.
      lentropy= .false.
      llast   = .false.
      dt  = -1.0_dop
      gq  = 0
      ncut = 9999999
      nskip = 0
      lrdchk = .false.
      lml = .false.
      enunit = ' '

      iarg = myiargc() 
      if (iarg .lt. 1) then
         write(6,'(a)') 'Reading system information. To see options use -h'
         go to 200
      endif

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf(1:3).eq.'-o ') then 
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf(1:3) .eq. '-t ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) dt
            if (ierr .ne. 0  .or.  dt .le. 0.0_dop) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -t',&
                 ' must be a positive real number. Not : ', buf
               stop
            end if
         else if( buf .eq. '-ml' ) then
            lml=.true.
         else if( buf .eq. '-last' ) then
            llast=.true.
         else if( buf .eq. '-skip' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nskip
            if (ierr .ne. 0  .or.  nskip .lt. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -skip',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf .eq. '-cut' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ncut
            if (ierr .ne. 0  .or.  ncut .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -cut',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:3) .eq. '-r ' ) then
            lren = .true.
         else if( buf(1:4) .eq. '-eu ' ) then
            i=i+1
            call mygetarg(i,enunit)
         else if( buf(1:5) .eq. '-tr2 ' ) then
            ltr2 = .true.
         else if( buf(1:9) .eq. '-entropy ' ) then
            lentropy = .true.
         else if( buf(1:3) .eq. '-n ' ) then
            fs = 1.0_dop
            ev = 1.d0
            cfs2=' '
         else if( buf(1:5) .eq. '-nofs' ) then
            fs = 1.d0
            cfs2=' '
         else if( buf(1:5) .eq. '-noev' ) then
            ev = 1.d0
         else if( buf(1:3) .eq. '-g ' ) then
            lgnu = .true.
         else if( buf(1:3) .eq. '-gq' ) then
            lgnu = .true.
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) gq
            if (ierr .ne. 0  .or.  gq .lt. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -gq',&
                 ' must be a non-negative integer. Not : ', buf
               stop
            end if
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(/,78a1,/)')  ('*',i=1,78)
            write(6,'(a)') ' Analyse program "rdcheck" ' 
            write(6,'(1x,a/,11x,a)')&
       'Purpose : Reads the check-file.', &
       'Outputs and analyses the state population, natural weights etc..'
            write(6,'(1x,2a)') 'Usage   : rdcheck ',&
       '[-f -i -o -eu -g -gq -t -n -ver -h -?] task [ nmd / ndf  ns]'
            write(6,'(1x,a)') 'Exact usage depends on task chosen '
            write(6,'(1x,2a)') ' rdcheck : No task outputs system information to the screen'
            write(6,'(1x,2a)') ' rdcheck spop: State populations output to spop.pl'
            write(6,'(1x,a,/,a)') &
   ' rdcheck natpop [nmd ns]: Natural populations for mode nmd and state ns',&
   '                          written to natpop_nmd_ns.pl'
            write(6,'(2a/2a/a)') ' If the -ml option is set, then the ', &
             'first argument is ML-node for which data', &
             ' is written. The second argument is nmd, ', &
             'i.e. the runing number of the mode', &
             ' for which the natpops are written.'

            write(6,'(1x,a,/,a)') ' rdcheck qdq [ndf ns]:    Coordinate expection values for DOF ndf and state ns',&
                                  '                          written to qdq_ndf_ns.pl'
            write(6,'(1x,a,/,a)') ' rdcheck ggp [nmd ns]:    Gross Gaussian populations for mode nmd and state ns',&
                                  '                          written to ggp_nmd_ns.pl'
            write(6,'(1x,2a)') ' rdcheck etot: Total energy and norm output to etot.pl'
            write(6,'(15x,a)') ' For density matrices the trace is also written.'
            write(6,'(2a)') ' If not enough arguments given, ',&
                 'the user is prompted for the missing arguments.'  
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(/,a)') ' Options : '
            write(6,'(2a,/,9x,a)') &
            ' -f FILE : The data is read '&
           ,'from file FILE rather than from ./check'&
           ,'  The string FILE may be a relative or full path-name.'
            write(6,'(a)') ' -i DIR  : Data is read from file DIR/check'
            write(6,'(2a,/,9x,a/,9x,a/,9x,a)') &
            ' -o FILE: The output is written to',&
            ' file FILE.pl','  rather than the default.',&
            '  The string FILE may be a relative or full path-name.',&
            '  If FILE=no, output is to the screen.'
            write(6,'(2a,/,11x,2a,/,11x,a)') &
            ' -ml     : If natpop, read the natural populations of the non-bottom' &
            ,' nodes of','an ML-run. In this case the first argument' &
            ,' read is the','node-number rather than the state.'
            write(6,'(2a)') &
            ' -tr2    : Trace{rho^2} is written to screen. ',&
            '(MCTDH 1 part. dens.)'
            write(6,'(2a,/11x,a)') &
            ' -entropy: Entropy of natural populations', &
            ' is written to screen.', &
            'Do not use -tr2, or -entropy simultainiously.'
            write(6,'(2a)') &
            ' -g      : GNUplot command lines are written to ',&
            'the output file(s).'
            write(6,'(2a)')&
            ' -r      : Renormalise the natural weights via division',&
            ' by state population.'      
            write(6,'(a,/a)') &
            ' -eu unit   : Use "unit" for the energy.'
            write(6,'(a)') &
            ' -t dt   : Print the natural weights every dt fs.'
            write(6,'(a,/a)') &
            ' -last   : Take last WF when computing the ', &
            '           maximum over time of lowest nat.-weight.'
            write(6,'(a,/a)') &
            ' -skip n : Skip the first n WFs when computing the ',&
            '           maximum over time of lowest nat.-weight.'
            write(6,'(2a,/a)') &
            ' -cut n  : Consider only the first n WFs when computing ',&
            'the','           maximum over time of lowest nat.-weight.'
            write(6,'(2a)') " -nofs   : No time units.", &
            " Sets fs=1. Use, if 'time-not-fs' is set."
            write(6,'(a)')  " -n      : Sets both -nofs and -noev"
            write(6,'(2a)') " -noev   : No energy units.", &
            " Sets ev=1. Use, if 'energy-not-ev' is set."
            write(6,'(a)')&
            ' -ver    : Version information about the program.'
            write(6,'(a,/a)') &
            ' -h      : Print this help text. ',&
            ' -?      : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) then
! finish arguments - no options given
            goto 200
         else
! get next option
            goto 100
         endif
      else
! move on to arguments
         continue
      end if

! first argument is task
!
      call mygetarg(i,buf)
      if (buf .eq. 'spop') then
         lrdchk(1) = .true.
      else if (buf .eq. 'natpop') then
         lrdchk(2) = .true.
      else if (buf .eq. 'qdq') then
         lrdchk(3) = .true.
      else if (buf .eq. 'ggp') then
         lrdchk(4) = .true.
      else if (buf .eq. 'etot') then
         lrdchk(5) = .true.
      else if (buf .eq. 'stop') then
         stop
      else
         write(6,'(a)') 'Unknown task. Try again. Allowed options are:.'
         write(6,'(a)') 'spop, natpop, qdq, ggp, etot, stop'
         write(6,'(a)') 'NOT:'
         write(6,'(a)') buf
         stop
      endif

!
! optional arguments for mode/ dof + state
!
      i=i+1
      if (i .gt. iarg) go to 200
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) iargs(1)
      if (ierr .ne. 0) then
         write(6,'(a/,1x,a72)') &
        ' The argument must be int number. Not :',buf
         stop
      end if
   
      i=i+1
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) iargs(2)
      if (ierr .ne. 0) then
       write(6,'(1x,a/,a)')&
       'The second argument  must be int number. Not :', buf
       stop
      end if

 200  continue

!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
! laenge is length of name directory string
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         filein=name(1:laenge)//'/check'
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         if (filein(laein:laein) .eq. '/') then 
            filein = filein(1:laein)//'check'
            laein = laein + 5
         end if
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

      if (.not. lrdchk(2)) lml = .false.

!-----------------------------------------------------------------------
! Set energy unit
!-----------------------------------------------------------------------
      if(enunit.eq.' ') then
         enunit='[eV]'
         if(ev.lt.2) enunit='[au]'
      else
         write(6,'(1x,2a)') 'Energy-unit = ',enunit
         ev=1.d0
         call keylowc(enunit)
         call keyeunits(ev,enunit,i)
         ev=1.d0/ev
         if(i .gt. 0) then
            write(6,'(2a)') ' unknown energy unit: ', enunit
            write(6,'(2a)') ' Allowed units: ', &
                     'au mH ev mev cm-1 kcal/mol kj/mol nmwl'
            stop
         end if
         enunit ='['//trim(enunit)//']'
      endif

      pnode=-1
      if(lml) then
         pnode=ns
         if(.not.lrunml) then
            write(6,'(/2a/)') '### ERROR: ', &
              'The -ml option can be used only for ML-runs.'
            stop
         endif
         if(filever(ichk) .lt. 8.5011d0) then
            write(6,'(/3a/)') '### ERROR: ', &
              'The version of the check file is too old ', &
              'for usage with -ml option.'
            stop
         endif
      endif


      return 
      end subroutine chkinput

      end program rdcheck

