!#######################################################################
!
!#######################################################################

      program rddddb

use decimal, only: long,dop
use global ! lsocbas
use maxv, only: maxdim,maxsta
use keyunitsmod
use dbtools, only: igeodb, ipesdb, irefdb
use channels, only: iaus
use dirdyn, only: lddsoc,infoarray,flagarray
use versions
use sqlite
use readqc, only: getinam
use db_sql
                                                                        
      implicit none                                                     

      integer(long)       :: ndof1,i,n,izero,s,nrec,irec,ilbl,ilog

      real(dop), dimension(:), allocatable      :: xgpoint
      real(dop), dimension(:,:), allocatable    :: tmpxgpoint
      real(dop), dimension(:,:), allocatable    :: vpes
      real(dop), dimension(:,:,:), allocatable  :: tmpvpes
      character(len=80)                         :: fileout
      character(len=200)                        :: plotfile,filename
      logical(kind=4)                           :: lauto=.false.
      logical(kind=4)                           :: lpesdb,lgradb,lhesdb,&
         lapesdb,lagradb,lahesdb,ldipdb,ladipdb,lmocodb,lnactdb,ltransdb,&
         lnmdb,lasocdb,lddcsoc,lsymdb,lcsocdb,lcasocdb,&
         lcpesdb,lctransdb,lcivecdb,ltemplate,lqcfreq
      real(dop)                                 :: dde0
      integer(long)                             :: dercpdim,&
         dimref,natm,nroot,nddstate,nbasis,j,nfam,dbpntgp,nsymop,nsocstate
      integer(long), dimension(7)               :: nddmult
      character(len=c1), dimension(:), allocatable:: atnam
      integer(long), dimension(:), allocatable  :: iatnam,iatiso
      real(dop), dimension(:), allocatable        :: mref
      real(dop), dimension(:,:), allocatable      :: xref
      type(SQLITE_DATABASE)                     :: db
      integer(long)                             ::sql_vers = 2

      integer(long)                             :: plottask = 1
      integer(long), external        :: myiargc
      integer(long)                  :: iarg
      character(len=80)              :: buf,pltcmd

      izero = 0
      ilog = 6     ! log messages to screen

!-----------------------------------------------------------------------
! read options
!-----------------------------------------------------------------------

   ! default: all files are in current directory

      ddname = '.'
      ddlaenge=1
      fileout = 'plot.db'


      iarg = myiargc()
      !Read in cli arguments

      if (iarg .eq. 1) then
         i = 1
         call mygetarg(i,buf)
         if (buf .eq. '-h' .or. buf .eq. '-?') then
            !Get the help
            write(6,'(/,78a1,/)')  ('*',i=1,78)
            write(6,'(a)') ' Analyse program "rddddb" '
            write(6,'(1x,a/,11x,a)')&
            'Purpose : Reads the DB-files to plot geometeries.'
            write(6,'(1x,a)') 'Usage   : rddddb [-i -o -a -ver -h -?] task '
            write(6,'(1x,a)') ' rddddb DB : Reads geo.db and writes plot.db'
            write(6,'(1x,a)') &
            ' rddddb refdb : Reads refdb.dat and writes xref to plot.db'
            write(6,'(1x,a)') &
            'If no task is given, geo.db structures are plotted' 
            write(6,'(a)') '  '
            write(6,'(a)') ' -i DIR  : Use DIR as name-directory.'
            write(6,'(2a,/,11x,a/,11x,a,/11x,a)') &
                     ' -o FILE : The output is written to file',&
                     ' FILE rather than to the default name'
            write(6,'(a)') &
            ' -a  : Jmol is called to automatically plot structures.'
            write(6,'(a)') &
           ' -ver    : Version information about the program.'
            write(6,'(a,/a)') &
            ' -h      : Print this help text. ',&
            ' -?      : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if (buf .eq. '-ver') then
            call wrversion(6)
            stop
         else
            write(6,'(a)') "Unknown option please use -h for information"
         end if
      else if (iarg .gt. 0) then
         i = 1
         do while (i .lt. iarg)
            call mygetarg(i,buf)
            !-----------------------------------------------------------------------
            ! options start with -
            !-----------------------------------------------------------------------
            if (buf(1:1) .eq. '-') then
               if (buf .eq. '-i') then
                  !Get the name directory
                  i = i + 1
                  call mygetarg(i,buf)
                  ddname = buf
                  ddlaenge=index(ddname,' ')-1
               else if (buf .eq. "-o") then
                  !set output file name
                  i = i + 1
                  call mygetarg(i,buf)
                  fileout = buf
               else if (buf .eq. "-a") then
                  !automatically run jmol with file
                  lauto = .true.
               end if
            else
               !This is a task - not sure these will ever be called as refer to refdb?
               call mygetarg(i,buf)
               if (buf .eq. 'DB') then
                  plottask = 1 
               else if (buf .eq. 'refdb') then
                  plottask = 2 
               end if
            end if

            i = i + 1
         end do
      end if


!
! get full path of DB
!
      call abspath(ddname,ddlaenge)
      ddname = trim(ddname) // '/database.sql'
!
! Open SQL database
!
! read-only
      !Needed for NEWSQL
      !call makeflagarray
      !call makeinfoarray(nddstate)
      !As just opening can ignore info and flagaway
      call dbopen(ddname,db,sql_vers,quanticsver,infoarray,flagarray, 0)
      write(*,*)'db open'
      call dbgetnrec( db, nrec)
!
! read DB data
!
      call dbrdrefdb(db, natm, nroot, nddstate, nbasis, dde0, dimref, &
                     nfam, dercpdim, dbpntgp, nsymop, nsocstate, nddmult)
      maxdim=3*natm+3
      maxsta=nddstate
      ndof1 = natm*3
      call dbrdrefdbgeom(db, natm, mref, atnam, xref)

      allocate(iatnam(maxdim))
      allocate(iatiso(maxdim))
      do n = 1, natm
         call getinam(atnam(n),iatnam(n),iatiso(n),ilog)
      enddo
!
! get size of system
!

      call dbgetentries(db,6,lpesdb,lgradb,lhesdb,lapesdb,lagradb,&
                        lahesdb,ldipdb,ladipdb,lmocodb,lnactdb,&
                        ltransdb,lnmdb,lasocdb,lsymdb,lcpesdb,&
                        lctransdb,lcivecdb,ltemplate,lqcfreq)

      allocate(xgpoint(maxdim))
      allocate(tmpxgpoint(maxdim,1))
      allocate(vpes(maxsta,maxsta))
      allocate(tmpvpes(maxsta,maxsta,1))

!
! open plotfile
!                                                                       
      plotfile=ddname(1:ddlaenge)//'/'//fileout                         
      ilbl=index(plotfile,' ')-1

      filename=plotfile
      open(iaus,file=filename,status='unknown',form='formatted',&
          err=1003)   
      write(6,'(a)') '---------------------'
      write(6,'(a)') 'Writing output file: '
      write(6,'(5x,a)') plotfile(1:ilbl)
      write(6,'(a)') '---------------------'
!                                                                       
! read geometries and output
!                                                                       
      write(iaus,'(/,a)')&
            ' Entering Gaussian System  '

      if (plottask .eq. 1) then
         call dbgetnrec( db, nrec )
         do irec=1, nrec
            call dbrdgeo( db,natm,irec,-1,tmpxgpoint )
            xgpoint(:) = tmpxgpoint(:,1)

! convert to Angst
             do n=1,ndof1
                xgpoint(n)=xgpoint(n)*0.52917725_dop
             enddo

            call dbrdpes( db,nddstate,irec,-1,tmpvpes )
            vpes(:,:) = tmpvpes(:,:,1)
! this only gives us half of the symmetric matrix,
! but it seems that we are only interested in the diagonal below

             write(iaus,'(/,a)')&
                   '                          Input orientation:   '
             write(iaus,'(2a)')&
                   ' ----------------------------------------------',&
                   '-----------------------'
             write(iaus,'(2a)')&
                   ' Center     Atomic     Atomic             ',&
                   ' Coordinates (Angstroms)'
             write(iaus,'(2a)')&
                   ' Number     Number      Type              ',&
                   'X           Y           Z '
             write(iaus,'(2a)')&
                   ' ----------------------------------------------',&
                   '-----------------------'

             i=1
             do n=1,natm
                write(iaus,'(i5,i11,i14,4x,3f12.6)')&
                    n,iatnam(n),izero,xgpoint(i),xgpoint(i+1),xgpoint(i+2)
                i=i+3
             enddo
             write(iaus,'(2a)')&
                   ' ----------------------------------------------',&
                   '-----------------------'

             write(iaus,'(/,a,8f12.6)') 'Energy = ',(vpes(s,s),s=1,nddstate)
         enddo

      else if (plottask .eq. 2) then

! convert to Angst
         xref(:,:)=xref(:,:)*0.52917725_dop

         write(iaus,'(/,a)')&
               '                          Input orientation:   '
         write(iaus,'(2a)')&
               ' ----------------------------------------------',&
               '-----------------------'
         write(iaus,'(2a)')&
               ' Center     Atomic     Atomic             ',&
               ' Coordinates (Angstroms)'
         write(iaus,'(2a)')&
               ' Number     Number      Type              ',&
               'X           Y           Z '
         write(iaus,'(2a)')&
               ' ----------------------------------------------',&
               '-----------------------'

         do n=1,natm
            write(iaus,'(i5,i11,i14,4x,3f12.6)')&
                n,iatnam(n),izero,xref(1,n),xref(2,n),xref(3,n)
         enddo
         write(iaus,'(2a)')&
               ' ----------------------------------------------',&
               '-----------------------'

      endif

      deallocate(iatnam,iatiso,xgpoint,tmpxgpoint,vpes,tmpvpes)

      if (lauto) then
         call myflush(iaus)
         pltcmd = 'jmol '//plotfile
         call excmd(pltcmd)
      endif
!                                                                       
      stop                                                              
 1003 print*,'ERROR opening DB file  ',filename
      stop                                                              
      end program rddddb                      

