!***********************************************************************
!
!                        ANALYSE MODULE RDEIGVAL
!
! Reads the eigvalcorrelation function from file "eigval".
!
! 11/96 MB
!
!
!***********************************************************************

      program rdeigval

use decimal, only: long,dop
use global
use aglobal
use versions
use keyunitsmod
use dvrdatmod
use griddatmod
use plot
use channels

use logdat, only: errormsg
use plotutils, only: plotdefs,initplot,enerbnds,dotask,plotmenu,databnds

use module_interface

      implicit none

      integer(long), external :: myiargc,slen
      integer(long)           :: i,iarg,ieig
      real(dop)               :: eigval1,intens,eigerr 
      character(len=80)       :: headline, buf
! If set to true, set intesities of lines to all be 1.
      logical(kind=4)         :: lequal = .false.

! interactive plot variables
      integer(long)                                  :: ndata,plottask,task, &
                                                        plotdim,dimx,dimy,nsets, &
                                                        nplots,ilbl,n
      integer(long), dimension(:), allocatable, save :: idum
      character(len=c5)                              :: filename
      real(dop), dimension(1)                        :: ydum,xdum,zdum,tdum
      real(dop), dimension(:), allocatable, save     :: eigval
      real(dop), dimension(:,:), allocatable, save   :: pldata
      character(len=10)                              :: units

      integer(long)     :: ierr
      character(len=10) :: cerr

!-----------------------------------------------------------------------
! Initialise variables
!-----------------------------------------------------------------------
      call default
      call adefault
      filein = '  '
      name   = './'
      units = 'ev'
      call setunits

!-----------------------------------------------------------------------
! zero dummy variables
!-----------------------------------------------------------------------
      xdum(1)=0.0
      ydum(1)=0.0
      zdum(1)=0.0

!-----------------------------------------------------------------------
! read options
!-----------------------------------------------------------------------
      iarg = myiargc()
      if (iarg .lt. 1) goto 2

      i = 0
   1  i=i+1
      if (myiargc() .ge. 1) then
         call mygetarg(i,buf)
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if(buf .eq. '-u') then
            i=i+1
            call mygetarg(i,buf)
            units = buf(1:10)
         else if(buf .eq. '-I0') then
            lequal=.true.
         else if( buf .eq. '-inter' ) then
            linter=.true.
         else if( buf .eq. '-gnupl4 ') then
            lgnu4 = .true.                 ! gnuplot ver < 5
         else if( buf .eq. '-ver' ) then
            call wrversion(6)
            stop

         else if( buf .eq. '-h' .or.  buf .eq. '-?') then
            write(6,'(1x,a/,11x,a)') &
        'Purpose : Reads the eigvalues from file "eigval" ',&
        'Usage   : rdeigval [-f -i -inter -I0 -gnupl4 -ver -h -?].'
            write(6,'(a)') ' Options : '
            write(6,'(2a,/,11x,a/11x,a)') &
              ' -f FILE : the data is read '&
              ,'from file FILE','rather than from ./eigval'&
              ,'The string FILE may be a relative or full path-name.'
            write(6,'(a)') ' -i DIR : data is stored in directory DIR'
            write(6,'(a)') ' -inter : enables interactive plotting'
            write(6,'(a)') ' -I0    : set intesity of all lines to 1.0'
            write(6,'(a)') ' -u units : Define energy units for plot'
            write(6,'(a)')&
                     ' -gnupl4  : Use gnuplot version < 5 for plotting '
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
              ' -h      : Print this help text. ',&
              ' -?      : Print this help text. '
            stop
         else
            write(6,'(2a)') ' Unknown option :', buf
            write(6,'(2a)') ' Use option -h'
            stop
         end if
         if (iarg .le. i) goto 2
         goto 1

      end if
         
    2 continue
!-----------------------------------------------------------------------
! If filein has not been set, look in name directory. 
! Otherwise take name from filein
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         filein=name(1:laenge)//'/eigval'
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

!-----------------------------------------------------------------------
! Open and read file "eigval" generated in a constant mean-field
! calculation with adaptive stepsize. 
! Skip repetition steps and save data in arrays
!-----------------------------------------------------------------------
      open(iaut,file=filein(1:laein),form='formatted',status='old',err=666)
      read(iaut,'(a)') headline

      print*,filein(1:laein)
! get no. of data sets
      ndata=0
      do
         read(iaut,*,end=50,err=666) ieig,eigval1,intens,eigerr

         ndata=ndata+1
      enddo
  50  continue

! no. of data sets to be plotted
      maxdata = ndata
! space in label arrays for Time etc.
      maxdim = 3
! allocate data arrays
      cerr = 'pltime'
      allocate(eigval(maxdata),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'pldata'
      allocate(pldata(maxdata,3),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'idum'
      allocate(idum(maxdim),stat=ierr)
      if (ierr > 0) go to 1000

! set defaults
      idum(1)=0

! read data
      rewind(iaut)
      read(iaut,*)
      do n=1,ndata
         read(iaut,*,err=666) ieig,eigval1,intens,eigerr

! convert eigval to au
         eigval(n)=eigval1/ev
         if (lequal) then
            pldata(n,1)=1.0_dop
         else
            pldata(n,1)=intens
         endif
         pldata(n,2)=eigerr
      enddo

      close(iaut,err=666)

!-----------------------------------------------------------------------
! non-interactive plotting - write to screen
!-----------------------------------------------------------------------
      if (.not. linter) then
         write(6,'(76a)') '#',('-',i=1,75)

         do n=1,ndata
            write(6,'(i8,3x,3e20.8)') n,eigval(n),pldata(n,1),pldata(n,2)
         enddo

         write(6,'(76a)') '#',('-',i=1,75)
         stop
      endif

!-----------------------------------------------------------------------
! interactive plotting
! open file for data to be plotted
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/tmpplot.pl'
      open(iaus,file=filename,status='unknown')

!-----------------------------------------------------------------------
! allocate arrays for plotting
! dvrdat and grddat needed for label arrays
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      nplottask=2
      call alloc_plot

!-----------------------------------------------------------------------
! zero dummy variables
!-----------------------------------------------------------------------
      xdum(1)=0.0
      ydum(1)=0.0
      tdum(1)=0.0

      call plotdefs

      plotdim=1
      dimx=ndata
      dimy=1
      nsets=1
      nplots=1
      lplener=.true.
      call initplot(filename,plotdim)

      call enerbnds(eigval,ndata)

!-----------------------------------------------------------------------
! set program specific data, e.g. plot tasks
!-----------------------------------------------------------------------
      cplottask(1)='Stick spectrum'
      cplottask(2)='Errors'
      do i=1,nplottask
         cptlen(i)=slen(cplottask(i))
      enddo

      plottask=1

      eunit=1.0
      call keyeunits(eunit,units,ierr)
      if (ierr .eq. 0) then
         axunitlab(fener)=units
         axunit(fener)=eunit
      else
         write(6,*) 'Unknown energy unit',units
         stop
      endif

      call enerbnds(eigval,ndata)
  250 continue

      call plotmenu(task,plottask,plotdim)

!-----------------------------------------------------------------------
! do task
! TASK=0 STOP  else return to menu
!-----------------------------------------------------------------------
      if (task .gt. 0) then
         if (plottask .eq. 1) then
            lstick=.true.
            plttitle='Eigenvalue spectrum'
            ilbl = index(axunitlab(fener),' ')-1
            axislab(1)='Energy ['//axunitlab(fener)(1:ilbl)//']'
            axislab(3)='Intensity'
         else if (plottask .eq. 2) then
            plttitle='Eigenvalue Error '
            ilbl = index(axunitlab(fener),' ')-1
            axislab(1)='Energy ['//axunitlab(fener)(1:ilbl)//']'
            axislab(3)='Error'
            lpoints=.true.
         endif
         if (lrebound) then
            call databnds(pldata(1,plottask),dimx,dimy,nsets,nplots,&
                    ndata,1,1,1)
         endif
         call dotask(task,plottask,plotdim,eigval,ydum,&
              pldata(1,plottask),dimx,dimy,nsets,nplots,ndata,1,1,1,&
              xdum,1,tdum,1,eigval,ndata,zdum,1,1,1,1,idum)

         goto 250
      endif

      close(iaus,status='delete')
      stop

 666  continue
      write(6,*)
      write(6,'(a,/a)')&
         'ERROR in program rdeigval: Cannot open or read eigval file:'&
         , filein(1:laein)
      write(6,*)
      stop

 1000 write(ilog,'(a)') 'Error allocating memory in RDEIGVAL :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop

      end program rdeigval

