!***********************************************************************
!
!                        ANALYSE MODULE RDGPOP
!
! Reads the data from file "gridpop" and sums the grid occupation of the
! nz first and last grid points/FBR occupations.
! Data for ploting is written to the file "gpop.pl".
!
! 04/99 HDM
!
!
!***********************************************************************

      program rdgpop

use decimal, only: long,dop,sip
use global
use aglobal
use versions
use maxv, only: maxdim, maxsta
use memdimmod
use keyunitsmod
use dvrdatmod
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod

use iofile, only: rdrhoinfo
use rdfilesmod, only: rdgridpop
use logdat

use module_interface

use basismod
use channels, only: filever, irho

      implicit none

      integer(long), external :: slen
      integer(long)       :: mp,nst,npl
      integer(long)       :: mxgd,ierr 
      integer(long)       :: f,s,i,j,n,m,nz,gdi,gdi2,gd1,gd2,k,kmax,dof,idt,&
                             gj,gm,mmax,mmin,mincr,timedx
      real(dop), dimension(:), allocatable      :: g1,g2,f1,f2,g1m,g2m,&
                                                   f1m,f2m,g1e,g2e,f1e,f2e
      real(dop)                                 :: time,dt,amin,tskip,tcut
      real(sip), dimension(:),allocatable       :: fgpop
      real(dop), dimension(:,:),allocatable     :: grid,fbr
      real(dop), dimension(:,:),allocatable     :: gpop1,gpop2
      real(dop), dimension(:,:,:), allocatable  :: gall
      logical(kind=4)                           :: lplst,llast
      character(len=8)                          :: abasis
      character(len=14) :: cerr
      character(len=120)                        :: gnucmd

!-----------------------------------------------------------------------
!     Set defaults. Set up the input.
!-----------------------------------------------------------------------

      write(6,'(/,78a1,/)')  ('*',i=1,78)
      write(6,'(2a)') ' Analyse program "rdgpop" ', &
                      '(Read grid populations)'
      call default
      call adefault
      call setunits
      call popinput(dof,nz,abasis,dt,mp,lplst,llast)

!-----------------------------------------------------------------------
! get no. of separate "state" wavepackets to be read
!-----------------------------------------------------------------------
      if (lmult .or. .not. lgpel) then
         nst = nstate
      else
         nst = nddstate*npacktot
      endif

      cerr = 'g1'
      allocate(g1(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'g2'
      allocate(g2(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'f1'
      allocate(f1(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'f2'
      allocate(f2(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'g1m'
      allocate(g1m(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'g2m'
      allocate(g2m(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'f1m'
      allocate(f1m(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'f2m'
      allocate(f2m(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'g1e'
      allocate(g1e(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'g2e'
      allocate(g2e(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'f1e'
      allocate(f1e(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'f2e'
      allocate(f2e(maxdim),stat=ierr)
      if (ierr > 0) go to 1000

      tskip = (nskip-0.5d0)*out1 + tinit
      tcut  = (ncut -0.5d0)*out1 + tinit
      kmax = 0
      mxgd = 0
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            g1m(f) = 0.0_dop
            g2m(f) = 0.0_dop
            f1m(f) = 0.0_dop
            f2m(f) = 0.0_dop
            mxgd = max(mxgd,gdim(f))
         end do
      end do
      maxgdim=mxgd
      cerr = 'fgpop'
      allocate(fgpop(2*mxgd),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'grid'
      allocate(grid(mxgd,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'fbr'
      allocate(fbr(mxgd,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'gpop1'
      allocate(gpop1(ortdim,nddstate*npacktot),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'gpop2'
      allocate(gpop2(ortdim,nddstate*npacktot),stat=ierr)
      if (ierr > 0) go to 1000

      timedx = 0
      do while(.not.lend)
         timedx = timedx + 1
         call rdgridpop(irho,fgpop,gpop1,gpop2)        
      enddo
      rewind(irho)
      cerr = 'gall'
      allocate(gall(timedx,maxsta,5),stat=ierr)
      if (ierr > 0) go to 1000
    
      time = tinit - out1
      gall(:,:,:) = 0.0_dop

!-----------------------------------------------------------------------
! loop to read gridpop file
!-----------------------------------------------------------------------
 100  time = time + out1
! read grid populations
      call rdgridpop(irho,fgpop,gpop1,gpop2)
      if (lend) go to 200

      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            gdi  = gdim(f)
            gdi2 = gdim(f)
            if(basis(f) .eq. 4 ) then
               gd1 = gdim(f)/2 
               gd2 = gdim(f)/2 
            elseif(basis(f) .eq. 6 ) then
               gj     = ipbaspar(4,f)
               mmax   = ipbaspar(1,f+1)
               mincr  = ipbaspar(2,f+1)
               mmin   = ipbaspar(3,f+1)
               gm = (mmax-mmin)/mincr + 1

               gdi = gj
               gd1 = 0
               gd2 = gj
            elseif(basis(f) .eq. 12 .or. basis(f) .eq. 20 .or. &
                   basis(f) .eq. 21) then
               gdi2 = ipbaspar(3,f)
               gd1 = 0
               gd2 = gdi2 
            else if(basis(f) .eq. -1 ) then
               gdi = gm
               gd1 = gj
               gd2 = gj+gm+1
            else
               gd1 = 0
               gd2 = gdim(f) 
            end if

! load grid populations into arrays for DOF f
            grid(1:gdi,:) = gpop1(zort(f):zort(f)+gdi-1,:)
            fbr(1:gdi2,:) = gpop2(zort(f):zort(f)+gdi2-1,:)

            g1(f) = 0.0_dop
            g2(f) = 0.0_dop
            f1(f) = 0.0_dop
            f2(f) = 0.0_dop

! sum up grid pops
            do s = 1, nst
               do i = 1, min0(nz,gdi)
                  g1(f) = g1(f) + grid(i,s)
                  g2(f) = g2(f) + grid(gdi+1-i,s)
                  f1(f) = f1(f) + fbr(gd1+i,s)
                  f2(f) = f2(f) + fbr(gd2+1-i,s)
               enddo
            end do
            if(basis(f) .eq. 6 .or. basis(f) .eq. -1) then
               g1(f)  = f1(f)
               g2(f)  = f2(f) 
               f1(f)  = 1.0_dop
               f2(f)  = 1.0_dop 
            end if

            if(llast) then
               g1e(f) = g1(f)
               g2e(f) = g2(f)
               f1e(f) = f1(f)
               f2e(f) = f2(f)
            else if(time.gt.tskip .and. time.lt.tcut)  then
               g1m(f) = dmax1(g1m(f),g1(f))
               g2m(f) = dmax1(g2m(f),g2(f))
               f1m(f) = dmax1(f1m(f),f1(f))
               f2m(f) = dmax1(f2m(f),f2(f))
            endif

! store grid pops for plotting
            if(f .eq. dof) then
               k = 1 + int((time+0.1_dop*out1)/out1)
               kmax = max(k,kmax)

               if (lplst) then
                  do s = 1, nst
                     gall(k,s,1) = time
                     do i = 1, min0(nz,gdi)
                        gall(k,s,2) = gall(k,s,2) + grid(i,s)
                        gall(k,s,3) = gall(k,s,3) + grid(gdi+1-i,s)
                        gall(k,s,4) = gall(k,s,4) + fbr(gd1+i,s)
                        gall(k,s,5) = gall(k,s,5) + fbr(gd2+1-i,s)
                     enddo
                  enddo
               else
                  gall(k,1,1) = time
                  do s = 1, nst
                     do i = 1, min0(nz,gdi)
                        gall(k,1,2) = gall(k,1,2) + grid(i,s)
                        gall(k,1,3) = gall(k,1,3) + grid(gdi+1-i,s)
                        gall(k,1,4) = gall(k,1,4) + fbr(gd1+i,s)
                        gall(k,1,5) = gall(k,1,5) + fbr(gd2+1-i,s)
                     enddo
                  enddo
               end if
            end if

         end do
      end do

      if( dt .gt. 0.0_dop  .and.  time .gt. tinit+0.5_dop*dt ) then
         idt = int((time+0.3_dop*out1)/dt)
         if( abs(time-idt*dt) .le. 0.3_dop*out1 )  then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(a,f9.2,a)')&
                 ' Maximal values (up to time);   time:',time,' fs'
            write(6,'(2a)') '   #  dof     grid(begin)    grid(end)',&
                      '      basis(begin)   basis(end)'  
            do m=1,nmode
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  write(6,'(i4,2x,a8,f11.9,3f15.9)' ) &
                    f, modelabel(f)(1:8),g1m(f),g2m(f),f1m(f),f2m(f)
               end do
            end do
            write(6,'(78a1)')  ('-',i=1,78)
         end if
      end if

      goto 100


! 120  write(6,*) ' Error occurred while reading gridpop-file'
!      write(6,*) ' time = ', time,' fs'
!      write(6,*) ' e, f :', e, f

 200  continue

      if(llast)  then
         do f=1,ndof
            if(f.eq.feb) cycle
            g1m(f) = g1e(f)
            g2m(f) = g2e(f)
            f1m(f) = f1e(f)
            f2m(f) = f2e(f)
         enddo
         write(6,'(/,a,f9.2,a)') &
              ' Final values (summed over all states);  final time:' &
                            ,time-out1,' fs'
      else
         write(6,'(/,a,f9.2,a)')&
           ' Maximal values (all times and states);  final time:'&
                            ,time-out1,' fs'  
      endif
      write(6,'(2a)') '   #  dof     grid(begin)    grid(end)',&
                      '      basis(begin)   basis(end)'  
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            write(6,'(i4,2x,a8,f11.9,3f15.9)' ) &
               f, modelabel(f)(1:8),g1m(f),g2m(f),f1m(f),f2m(f)
         end do
      end do
      write(6,'(78a1)')  ('-',i=1,78)

!-----------------------------------------------------------------------
! write to file
!-----------------------------------------------------------------------
      npl=1
      if(lgnu  .and.  fileout(1:2) .ne. 'no') then
         write(22,'(a,i2,a,i2,a2)') &
                     'set title "Grid populations, nz='&
                     ,nz,', dof=',dof,' \'
         write(22,'(2a)') filein(1:laein-7),'"' 
         write(22,'(3a)') 'set lab "',modelabel(dof)(1:8),&
                           '" at graph 0.03,0.95'
         write(22,'(3a)') 'set lab "',abasis,'" at graph 0.16,0.95'
         write(22,'(a)') 'set style data lines' 
         write(22,'(a)') 'set logscale y' 
         write(22,'(a)') 'set xlabel "time[fs]"'
         write(22,'(a)') 'set yrange [1.e-7:0.1]'
         write(22,'(2a,a3)') 'plot \'
         write(22,'(a)') '"-"  us 1:2 tit "first grid point",\'    
         write(22,'(a)') '"-"  us 1:3 tit "last  grid point",\'
         if(basis(dof) .eq. 4 ) then
            write(22,'(a)') '"-"  us 1:4 tit "neg. mom. occ.",\'
            write(22,'(a)') '"-"  us 1:5 tit "pos. mom. occ."'
            npl=4
         else
            write(22,'(a)') '"-"  us 1:5 tit "basis occupation"'
            npl=3
         endif
         write(22,'(a)') '#'    
      end if

      if(fileout(1:2) .ne. 'no') then
         amin = 1.0e-16_dop
         if (lplst) then
            do s = 1, nst
               write(22,'(a,i3,5x,a,i3,2a,/a,i4,a,i4,5x,2a)') &
                    '# nz =',nz,'dof = ',dof,' : ',modelabel(dof),&
                    '# state =',s,' /',nst,'DVR = ',abasis
               do i=1,npl
                  write(22,'(2a)') '#   time[fs]   grid(begin)     ',&
                          'grid(end)       basis(begin)    basis(end)' 
                  do k = 1, kmax
                     write(22,'(f12.4,4f16.12)') (gall(k,s,j),j=1,5)
                  end do
                  write(22,'(a)')  'e'
               enddo
               write(22,'(78a1)')  ('#',i=1,78)

               if (s .lt. nst .and. lgnu) then
                  write(22,'(a)')  'pause -1'
                  write(22,'(2a,a3)') 'plot \'
                  write(22,'(a)') '"-"  us 1:2 tit "first grid point",\'    
                  write(22,'(a)') '"-"  us 1:3 tit "last  grid point",\'
                  if(basis(dof) .eq. 4 ) then
                     write(22,'(a)') '"-"  us 1:4 tit "neg. mom. occ.",\'
                     write(22,'(a)') '"-"  us 1:5 tit "pos. mom. occ."'
                     npl=4
                  else
                     write(22,'(a)') '"-"  us 1:5 tit "basis occupation"'
                     npl=3
                  endif
                  write(22,'(a)') '#'    
               endif
            end do
            close (22)
         else
            write(22,'(a,i3,5x,a,i3,2a,5x,2a)') &
                 '# nz =',nz,'dof = ',dof,' : ',modelabel(dof),&
                 'DVR = ',abasis
            do i=1,npl
               write(22,'(2a)') '#   time[fs]   grid(begin)     ',&
                       'grid(end)       basis(begin)    basis(end)' 
               do k = 1, kmax
                  write(22,'(f12.4,4f16.12)') (gall(k,1,j),j=1,5)
               end do
               write(22,'(a)')  'e'
            enddo
            write(22,'(78a1)')  ('#',i=1,78)
            close (22)
         endif
      end if

      if(lagnu) then
         gnucmd = 'gnuplot -persist "'//trim(fileout)//'"'
         call excmd(gnucmd)
      end if

      stop

 1000 write(6,'(a)') 'Error allocating memory in Rdgpop :',cerr
      write(6,'(a)') 'Error Code :',ierr
      stop

      contains

!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------
      subroutine popinput(dof,nz,abasis,dt,mp,lplst,llast) 

      implicit none

      logical(kind=4), external      :: int2log
      integer(long), external        :: myiargc
      integer(long)                  :: iarg,ierr,check,ilgpel,i,f,n,m
      integer(long), intent(out)     :: mp
      integer(long), intent(out)     :: dof,nz
      real(dop), intent(out)         :: dt 
      character(len=80)              :: buf
      character(len=8), intent(out)  :: abasis
      logical(kind=4),intent(out)    :: lplst,llast
      character(len=10)              :: stat

!-----------------------------------------------------------------------

      dof  = -10
      nz   = -10
      dt   = -1.0_dop
      ncut = 9999999
      nskip = 0
      filein  = ' '
      fileout = 'gpop'
      name='./'
      lgnu = .false.
      lagnu  = .false.
      lplst = .false.
      llast = .false.
      stat   = 'new'

      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf(1:2) .eq. '-w' ) then
            stat = 'unknown'
         else if( buf .eq. '-last' ) then
            llast=.true.
         else if( buf(1:3) .eq. '-g ' ) then
            lgnu = .true.
         else if( buf(1:3) .eq. '-a ' ) then
            lgnu  = .true.
            lagnu = .true.
            stat  = 'unknown'
         else if( buf(1:3) .eq. '-n ' ) then
            fs = 1.0_dop
         else if( buf(1:3) .eq. '-s ' ) then
            lplst = .true.
         else if( buf(1:3) .eq. '-t ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) dt
            if (ierr .ne. 0  .or.  dt .le. 0.0_dop) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -t',&
                 ' must be a positive real number. Not : ', buf
               stop
            end if
         else if( buf .eq. '-skip' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nskip
            if (ierr .ne. 0  .or.  nskip .lt. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -skip', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf .eq. '-cut' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ncut
            if (ierr .ne. 0  .or.  ncut .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -cut', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(1x,a)')&
       'Purpose : Analyses the grid population read from file gridpop.'
            write(6,'(1x,a)')&
       'Usage   : rdgpop [-f -i -last -g -a -s -o -t -n -w -ver -h -?] &
       &[nz  dof].'
            write(6,'(/,a)') ' Options : '
            write(6,'(2a,/,11x,a/11x,a)') &
              ' -f FILE : the grid populations (densities) are read '&
             ,'from file FILE','rather than from ./gridpop'&
             ,'The string FILE may be a relative or full path-name.'
            write(6,'(a)') ' -i FILE : Same as -f FILE'
            write(6,'(2a,/,11x,a/11x,a/11x,a)') &
              ' -o FILE : The grid populations are written to file ',&
              'FILE.pl','rather than to ./gpop.pl',&
              'The string FILE may be a relative or full path-name.',&
              'If FILE=no, i.e."-o no", no gpop.pl file will be opened.'
            write(6,'(2a,/2a,/a)') &
            ' -last   : Write only the grid populations at final time', &
            ' to screen.','           The options -t, -skip, -cut are', &
            '  ignored', &
            '           and nz=dof=0 is set, when this option is given.'
            write(6,'(2a)') &
            ' -t dt   : Print the maximal occupations every dt fs.'
            write(6,'(a,/a)') &
            ' -skip n : Skip the first n WFs when computing the ', &
            '           maximum over time of the grid populations.'
            write(6,'(a,/a)') &
            ' -cut n  : Consider only the first n WFs when computing the', &
            '           maximum over time of the grid populations'
            write(6,'(2a)') &
            " -n      : No units. To be used when 'time-not-fs' is set."
            write(6,'(2a)') &
              ' -g      : GNUplot command lines are written to ',&
              'the plot file.'
            write(6,'(2a)')&
                     ' -a      : GNUplot is called automatically.',&
                     ' (sets -g and -w).'
            write(6,'(2a)') &
              ' -s      : Grid populations for different states ',&
              'written separately to plot file.'
            write(6,'(2a)')&
                     ' -w      : An existing property file is ',&
                     'overwritten.'
            write(6,'(a)')&
              ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
              ' -h      : Print this help text. ',&
              ' -?      : Print this help text. '
            write(6,'(/a)') ' There are two arguments: '
            write(6,'(2a)') ' The first argument is nz, i.e. the ',&
                            'number of grid points sumed over.' 
            write(6,'(2a,/2a)') ' The second argument is dof, ',&
                         'i.e. the runing number of the degree ',&
                         ' of freedom to be considered.', &
                         ' (dof=0 -> no output to file gpop.pl).'
            write(6,'(2a)') ' If not enough arguments are given, ',&
                 'the user is prompted for the',' missing arguments.'  
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      endif
      
      read(buf,*,iostat=ierr) nz
      if (ierr.ne.0 .or. nz.lt.0) then
         write(6,'(2a/,a)') ' The first argument (nz)',&
          ' must be a positive integer number. Not :', buf
         stop
      end if
      if(nz.eq.0) then
         nz=1
         dof=0
      endif

      i=i+1
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) dof
      if (ierr.ne.0 .or. dof.lt.0) then
         write(6,'(2a/,a)') ' The second argument (dof)', &
          ' must be a non-negative integer number. Not :', buf
       stop
      end if

!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
! Otherwise take name from filein
!-----------------------------------------------------------------------
 200  if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         filein=name(1:laenge)//'/gridpop'
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

!-----------------------------------------------------------------------
!  Prompting for missing input
!-----------------------------------------------------------------------
      if(llast) then
         nz=1
         dof=0
         dt=-1.d0
         nskip=0
         ncut=9999999
      endif
      write(6,'(78a1)')  ('-',i=1,78)

 180  if(nz .lt. 0 ) then
         write(6,'(a)')&
            ' Number of grid-points to be summed over: nz =?'
      read(5,*) nz
      if(nz.eq.0) then
         nz=1
         dof=0
      endif
      if(nz.lt.0  .or.  nz.gt.maxdim)  goto 180
      else
         write(6,'(a,i3)')&
            ' Number of grid-points to be summed over: nz =',nz
      end if

      if(dof .lt. 0 ) then
         write(6,'(a)')' Runing-number of degree of freedom: dof =?' 
         write(6,'(a)')' dof = 0 -> Print only maximum over time'
         read(5,*) dof
      else
         write(6,'(a,i3)')&
            ' Runing-number of degree of freedom:     dof =',dof
         if( dof .eq. 0 )  write(6,'(a)')&
            ' dof = 0 -> Print only maximum over time'
      end if

      write(6,'(78a1)')  ('-',i=1,78)
      if( dof .le. 0 ) fileout = 'no'


!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      laein=index(filein,' ')-1
      if (filein(laein:laein) .eq. '/') then 
         filein = filein(1:laein)//'gridpop'
         laein = laein + 7
      end if
      call abspath(filein,laein)
      open(irho,file=filein(1:laein),form='unformatted',&
                   status='old',err=98)
      if(laein .le. 58 ) then
         write(6,'(2a)') ' The inputfile is  : ',filein(1:laein)
      else
         write(6,'(a)')  ' The inputfile is  :' 
         write(6,'(a)')   filein(1:laein)
      end if

      if(dof.gt.0 .and. fileout(1:2) .ne. 'no')  then
         laeout=index(fileout,' ')-1
         if (fileout(laeout:laeout) .eq. '/') then 
            fileout = fileout(1:laeout)//'gpop.pl'
            laeout = laeout + 7
         else if (fileout(laeout-2:laeout) .ne. '.pl') then
            fileout = fileout(1:laeout)//'.pl '
            laeout = laeout + 3
         end if 
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         fileout=name(1:laenge)//'/'//fileout(1:laeout)
         laeout=index(fileout,' ')-1
      
         open(22,file=fileout(1:laeout),form='formatted',&
              status=stat,err=99)
         if(laeout .le. 58 ) then
            write(6,'(2a)') ' The outputfile is : ',fileout(1:laeout)
         else
            write(6,'(a)')  ' The outputfile is :' 
            write(6,'(a)')   fileout(1:laeout)
         end if
      else
         write(6,'(a)') ' No outputfile (gpop.pl) written'
      end if


      call rdmemdim(irho)
      call alloc_dvrdat
      call alloc_grddat

!-----------------------------------------------------------------------
!  Reading the file header; printing basis set information.
!-----------------------------------------------------------------------

      rewind(irho)
      read(irho) filever(irho), ilgpel
      lgpel = int2log(ilgpel)
      write(6,'(a,f8.4)') ' Gridpop file version =', filever(irho)

      check = 1
      call rdrhoinfo(irho,check,check,check)

      write(6,'(78a1)')  ('-',i=1,78)
      write(6,'(a,i4)') ' Number of electronic states :', nddstate      
      write(6,'(a,i4)') ' Number of degrees of freedom:', ndof       
      write(6,'(a,i4)') ' Number of combined modes    :', nmode
      if (npacktot .gt. 1) then
         write(6,'(a,i4)') ' Number of wavepackets    :', npacktot
      endif

      write(6,'(/,a)') ' mode dof  dim   DVR     modelabel'

      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            write(6,'(1x,i3,1x,i4,i5,3x,a8,a10)') &
            m, f, gdim(f), cbas(basis(f)), modelabel(f)  
         enddo
      enddo

      tinit = tinit/fs
      out1  = out1/fs
      write(6,'(/,a,f8.3,a,f8.3,a)') &
        ' tinit =', tinit, 'fs,    tout =', out1, 'fs' 

      if(nstate .gt. 1) then
         write(6,'(78a1)')  ('-',i=1,78)
         if (lgpel) then
            write(6,'(a)') ' Electronic states are treated separately' 
         else
            write(6,'(a)') ' Sum over electronic states'
            nstate = 1
         end if
      endif

!-----------------------------------------------------------------------
!  set size for sum
!-----------------------------------------------------------------------
      mp=nz

!-----------------------------------------------------------------------
!  Check for legality of the arguments
!-----------------------------------------------------------------------

      if(nz .le. 0 ) then
         write(6,'(/a,/a,i5,/a,/a)')&
               '######################################################'&
              ,' Illegal value of nz =', nz&
              ,' nz must be positive and smaller than', mp     &
              ,'######################################################'
         stop
      end if
      dof = max(dof,0)
      if(dof .gt. ndof ) then
         write(6,'(/a,/a,i5,/a,i5,/a)')&
               '######################################################'&
              ,' Illegal value of dof =', dof&
              ,' dof must be smaller than ndof =', ndof    &
              ,'######################################################'
         stop
      end if 
      if(dof.gt.0 .and. dof.eq.feb .and. (.not.leb .or. lmult)) then
         write(6,'(/a,/a,i5,/a,/a)')&
             '#######################################################'&
            ,' Illegal value of dof =', dof&
            ,' This is the multi-state electronic degree of freedom!'&
            ,'#######################################################'
         stop
      end if 

!-----------------------------------------------------------------------
!  Set the character-string for the DVR-type in use.
!-----------------------------------------------------------------------
      if(dof .gt. 0) then
         abasis = cbas(basis(dof))
      else
         abasis = ' '
      endif


      return 
      
 98   write(6,*)' Cannot open input (gridpop) file : '
      write(6,*) filein
      stop
      
 99   write(6,*)' Cannot open output (pop.pl) file : '
      write(6,*) fileout
      write(6,*)' If file already exists use -w option to overwrite '
      stop

      end subroutine popinput


      end program rdgpop


