!***********************************************************************
!
!                  rdgrid
!
! This program reads the DVR grid (ort) from DVR file and prints
! it to screen.
!
!     04/40 HDM
!***********************************************************************
      
      program rdgrid

use decimal, only: long,dop
use global
use maxv
use aglobal
use versions
use dvrdatmod
use griddatmod
use channels
use basismod
use rddvrmod

use rddvrmod, only: dvrinfo,rddvr
use zeigauswmod, only: zeigausw

use module_interface

      implicit none

      integer(long)                        :: chkdvr,ilbl,dof,f,g,&
                                              nn,n2,n3,z,ggdim
      integer(long), dimension(1)          :: idum
      real(dop), dimension(1)              :: rdum
      real(dop), dimension(:), allocatable :: ort,weight
      real(dop), dimension(:), allocatable :: trafo
      complex(dop), dimension(1)           :: zdum
      character(len=c5)                    :: filename
      logical(kind=4)                      :: lerr,lwght,lsuppress,loc,&
                                              check,lpgrd
      integer(long)     :: ierr
      character(len=10) :: cerr

!-----------------------------------------------------------------------
! Initialize variables and set defaults
!-----------------------------------------------------------------------
      call default

!-----------------------------------------------------------------------
! Open log file channel (/dev/null)
!-----------------------------------------------------------------------
      open (unit=ilog,file='/dev/null',form='formatted')

!-----------------------------------------------------------------------
! read input parameters and options
!-----------------------------------------------------------------------
      call inrdgrid(dof,lwght,lsuppress,loc,lpgrd)

!-----------------------------------------------------------------------
! get array dimensions 
!-----------------------------------------------------------------------
      inquire(irst,opened=check)
      if (check) close(irst)
      filename=name(1:laenge)//'/restart'
      ilbl=index(filename,' ')-1
      open(irst,file=filename(1:ilbl),form='unformatted',status='old',err=900)
      call rdmemdim(irst)
      close(irst)

!-----------------------------------------------------------------------
! read information from system files
!-----------------------------------------------------------------------
      chkdvr=1
      filename=filein(1:laein)
      open(idvr,file=filename,form='unformatted',status='old',err=900)
      if(lpgrd .and. basis(dof).ne.4) then
         write(6,'(a)') 'Option -pgrd requires an FFT.'
         stop
      endif

      if (dof .le.0 .or. dof.gt.ndof) then
         write(6,'(a,i3,a)') 'No valid dof given. (dof=',dof,')'
         stop
      endif

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat

      call dvrinfo(lerr,chkdvr)
          
!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
!-----------------------------------------------------------------------
      call zeigausw

      cerr = 'ort'
      allocate(ort(ortdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'weight'
      allocate(weight(ortdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'trafo'
      allocate(trafo(dvrdim),stat=ierr)
      if (ierr > 0) go to 1000
 
!-----------------------------------------------------------------------
! read data from dvr file if desired
! edit which data (set dvrdata(x) to .true. and substitute dummy array)
!-----------------------------------------------------------------------
      if (filever(idvr) .ge. 11.0_dop) rewind(idvr)
      dvrdata(1) = .true.       ! read ort
      if (lwght) then
         dvrdata(2) = .true.       ! read trafo
      else
         dvrdata(2) = .false.      ! read trafo
      endif
      dvrdata(3) = .false.      ! read dvrmat (dif1mat, dif2mat)
      if (lpgrd) then
         dvrdata(4) = .true.      ! read fftp   (fft momentum)
      else
         dvrdata(4) = .false.      ! read fftp   (fft momentum)
      endif
      dvrdata(5) = .false.      ! read hin    (fft)
      dvrdata(6) = .false.      ! read rueck  (fft)
      dvrdata(7) = .false.      ! read fftfak
      dvrdata(8) = .false.      ! read jsph
      dvrdata(9) = .false.      ! read msph
      dvrdata(10)= .false.      ! read kinsph
      dvrdata(11)= .false.      ! read exphin   (expDVR)
      dvrdata(12)= .false.      ! read exprueck (expDVR)
      chkdvr=1
      if (lpgrd) then
         call rddvr(rdum,trafo,rdum,ort,zdum,zdum,&
              idum,zdum,zdum,idum,idum,rdum,chkdvr)
      else
         call rddvr(ort,trafo,rdum,rdum,zdum,zdum,&
              idum,zdum,zdum,idum,idum,rdum,chkdvr)
      endif

      if (lwght) then
         ggdim=gdim(dof)
         if(basis(dof) .eq. 12) ggdim = ipbaspar(3,dof)
         call dvrweights(trafo(zdvr(dof)),ort(zort(dof)),gdim(dof),&
                    ggdim,weight(zort(dof)),basis(dof),&
                    rpbaspar(1,dof),&
                    ipbaspar(1,dof),ipbaspar(1,dof+1),.true.,.false.)
      endif

      if (lpop2) then
         call getpop2(ort(zort(dof)),weight(zort(dof)),gdim(dof),dof)
      endif

!-----------------------------------------------------------------------
! Print  grid information
!-----------------------------------------------------------------------
      if(.not.lsuppress) then
         write(6,'(/a,/a)') '   f   label       N   DVR', &
              ' ------------------------------'
         do f=1,ndof
            write(6,'(i4,3x,a9,i4,3x,a8)') &
                 f, modelabel(f), gdim(f), cbas(basis(f))
         enddo
      endif

!-----------------------------------------------------------------------
! Print the grid
!-----------------------------------------------------------------------
      if(lsuppress) then
         write(6,'(a,i4,a,a)') '# dof =', dof,',  ',modelabel(dof)
      else
         write(6,'(/a,i4,a,a/)') '  dof =', dof,',  ',modelabel(dof)
      endif

      z=zort(dof)-1
      if(lpgrd) z=zfft(dof)-1
      if (lwght) then
         if(lsuppress) then
            write(6,'(a,9x,a,11x,a)') '#','Coordinate','Weight'
         else
            write(6,'(10x,a,11x,a)') 'Coordinate','Weight'
         endif
         do g=1, gdim(dof)
            write(6,'(i5,2G22.14)') g, ort(g+z), weight(g+z)
         enddo
      else if( gdim(dof) .le. 10 .or. loc) then   ! one column
         do g=1, gdim(dof)
            write(6,'(i5,G22.14)') g, ort(g+z)
         enddo
      else if(gdim(dof) .le. 80) then             ! two columns
         nn = (gdim(dof)+2)/2
         n2 = gdim(dof) - nn
         do g=1, nn
            write(6,'(i5,G22.14,i15,G22.14)') g,ort(g+z),g+n2,ort(g+n2+z)
         enddo
      else                                        ! three columns
         nn = (gdim(dof)+2)/3
         n2 = gdim(dof) - nn*2
         n3 = gdim(dof) - nn
         if(mod(gdim(dof),3).ne.0) n2 = n2+1
         do g=1, nn
            write(6,'(i5,G21.13,i5,G21.13,i5,G21.13)') &
                g,ort(g+z),g+n2,ort(g+n2+z),g+n3,ort(g+n3+z)
         enddo

      endif
      if(.not.lsuppress) write(6,'(78a1)')  ('-',g=1,78)

      deallocate(ort)
      deallocate(weight)
      deallocate(trafo)

      stop

!-----------------------------------------------------------------------
! Error handling
!-----------------------------------------------------------------------
  900 continue
      ilbl=index(filename,' ')-1
      write(6,'(2a)') 'Cannot open file: ',filename(1:ilbl)
      stop
 1000 write(ilog,'(a)') 'Error allocating memory in Statepop :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop

      contains

!#######################################################################
!-----------------------------------------------------------------------
! Input Routine
!-----------------------------------------------------------------------
      subroutine inrdgrid(dof,lwght,lsuppress,loc,lpgrd)

      implicit none

      integer(long), external      :: myiargc
      integer(long)                :: iarg,i,ierr
      integer(long), intent(out)   :: dof
      character(len=80)            :: buf
      logical(kind=4), intent(out) :: lwght,lsuppress,loc,lpgrd

!-----------------------------------------------------------------------
! provide default input and output filenames 
!-----------------------------------------------------------------------
      filein = '  '
      name   = './'
      dof=-1 
      lwght = .false.
      lsuppress = .false.
      loc = .false.
      lpop2 = .false.
      lpgrd = .false.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! read options 
! comment out or add options as required
!-----------------------------------------------------------------------
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if(buf .eq. '-s') then
            lsuppress = .true.
         else if(buf .eq. '-oc') then
            loc = .true.
         else if(buf .eq. '-w') then
            lwght = .true.
         else if(buf .eq. '-pop2') then
            lpop2 = .true.
         else if(buf .eq. '-pgrd') then
            lpgrd = .true.
         else if( buf .eq. '-ver' ) then
            call wrversion(6)
            stop
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(a,/a)')&
       'Purpose : Read the DVR grid from dvr file and print it.',&
       'Usage   : rdgrid [ -f -i -s -oc -w -pop2 -pgrd -h -? ]  dof .'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,10x,a/10x,a)')&
              ' -f FILE: the DVR file is read '&
              ,'from file FILE rather','than from ./dvr .'&
              ,'The string FILE may be a relative or full path-name.'
            write(6,'(a)') &
                     ' -i DIR : dvr file is read from directory DIR.'
            write(6,'(a)') &
                     ' -s     : Additional information is suppressed.'
            write(6,'(a)') &
                     ' -oc    : One-column output enforced.'
            write(6,'(a)') &
                     ' -w     : DVR weights are also printed.'
            write(6,'(a)') &
                     ' -pop2  : Grids of 2nd population are printed.'
            write(6,'(a)') &
                     ' -pgrd  : Momentum grid-points of a FFT are printed.'
            write(6,'(a)')&
                     ' -ver   : Version information about the program.'
            write(6,'(a,/a)') &
                     ' -h     : Print this help text. ',&
                     ' -?     : Print this help text. '
            write(6,'(/2a/,a)') ' The argument dof is the number of',&
                                ' the degree of freedom for which', &
                                ' the grid is printed.' 
            write(6,'(2a)')     ' If no argument is given, ',&
                 'the user is prompted for the missing argument.'  
            write(6,'(78a1)')  ('-',i=1,78)
            stop

         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) dof
      if (ierr .ne. 0) then
         write(6,'(1x,a/,a)')&
         'The  argument (dof)  must be int number. Not :', buf
         stop
      end if

!-----------------------------------------------------------------------
!  Prompting for missing input
!-----------------------------------------------------------------------
 200  continue
      if (dof .lt. 0) then
         write(6,'(a)') ' dof =? ' 
         read(5,*) dof
         if (dof.le.0) then
            write(6,*) 'The  argument (dof)  must be int number. Not :', dof
         endif
      endif

!-----------------------------------------------------------------------
!  Consistency checks
!-----------------------------------------------------------------------
      if(lpgrd .and. lpop2) then
         write(6,'(a)') 'Options -pgrd and -pop2 are incompatible.'
         stop
      endif          
      if(lpgrd .and. lwght) then
         write(6,'(a)') 'Options -pgrd and -w are incompatible.'
         stop
      endif          
            
!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
! Otherwise take name from filein
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         filein=name(1:laenge)//'/dvr'
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif
      
      dname=name(1:laenge)
      dlaenge=laenge
      if (.not. lsuppress) write(6,'(/2a)') ' The dvr file is: ', filein(1:laein)  

      return 
      end subroutine inrdgrid

!-----------------------------------------------------------------------
! Compute grids and weights for second population.
!-----------------------------------------------------------------------

      subroutine getpop2(ort,weight,gdm,f)

      implicit none

      integer(long), intent(in)           :: f, gdm
      integer(long) :: g,ibleg,ggdim,gj,gm
      real(dop)     :: dp
      real(dop), dimension(*), intent(out) :: weight(*),ort(*)

      do gj=1,gdm  ! gdm = gdim(f)
         weight(gj) = 1.d0
      enddo
! -  HO  -
      if( basis(f) .eq. 1  ) then
         do g = 1, gdm
            ort(g) = g-1
         end do
! -  Leg  Leg/R  Kleg  PLeg  Wigner -
      elseif(basis(f).eq.2 .or. basis(f).eq.7 .or. &
              basis(f).eq.9 .or. basis(f).eq.12 .or. &
              basis(f).eq.19) then
         ggdim = gdm
         if(basis(f) .eq. 12) ggdim = ipbaspar(3,f)
         if(basis(f).eq.2 .or. basis(f).eq.12) ibleg=ipbaspar(2,f)
         if(basis(f).eq.7 .or. basis(f).eq.9 .or. &
              basis(f).eq.19) ibleg=ipbaspar(1,f)
         if(ibleg .eq. 0 ) then
            do g = 1, ggdim
               ort(g) = g-1
            end do
         else
            do g = 1, ggdim
               ort(g) = g*2 - ibleg
            end do
         endif
! -  FFT -
      elseif( basis(f) .eq. 4  ) then
         dp = 2.d0*pi/( (ort(2)-ort(1))*gdm )
         gm = gdm/2 + 1
         do g = 1, gdm
            ort(g) = (g-gm)*dp
            weight(g) = dp
         end do
! -  exp -
      elseif( basis(f) .eq. 5  ) then
         dp = 2.d0*pi/( (ort(2)-ort(1))*gdm )
         do g = 1, gdm
            ort(g) = dp*(g-gdm/2-1)
            weight(g) = dp
         end do
! - rest -
      else
         do g = 1, gdm
            ort(g) = g
         end do
      end if

      return
      end subroutine getpop2

!#######################################################################

      end program rdgrid


