!***********************************************************************
!
!                         AUSWERT subroutine RDRSTART
!
!     This subroutine calculates the norm and the A-norm of a wave function.
!
!     05/99 HDM
!
!
!***********************************************************************

    program rdrestart

use decimal
use global
use dvrdatmod
use griddatmod
use psidef
use runpropmod
use runinwfmod
use daten
use operdef
use basismod
use channels
use versions
use iorst, only: rstinfo

!   ML-MCTDH modules
use wftree
use wftree1
use mlanalysis


    implicit none

    integer  chkdvr,chkgrd,chkpsi,chkprp,f
    logical  lerr,linwf,lall,lprint,lnth
    real*8   time

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
    call default
    call rdrstinput(lall,lprint,lnth)
    call rdmemdim(irst)
    call alloc_dvrdat
    call alloc_grddat
    call alloc_operdef
    call alloc_psidef

!-----------------------------------------------------------------------
! read data from restart file
!-----------------------------------------------------------------------
    chkdvr=1
    chkgrd=1
    chkpsi=1
    chkprp=1
    ilog=6
    if(lprint.or.lnth) lwrlog=.false.
    call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
    if (lerr) then
       write(6,'(a)') ' ERROR reading restart file'
       stop
    end if
    if(lnth) then
       write(6,'(1pe17.9)') normthermal
       stop
    elseif (lrunml .and. lprint) then
! . ML wavefunction. Get dgldim and write header.
       tree => new_wftree(tape)
       dgldim = dgldim_wftree(tree)
       write (6,*)
       call print_wftree(6,tree,modelabel,.true.)
       write (6,*)
       call dispose_wftree(tree)
       stop
    elseif (.not.lrunml .and. lprint) then
       write (6,'(a)') ' Sorry, this is not am ML wavefunction!'
       stop
    endif
    read(irst,err=99) time
    write(6,'(a,f9.5)') ' File version =',filever(irst)
    write(6,'(a,i4,a,l2)') ' psitype =', psitype,',  lrunml =',lrunml
    if(lrunml) write(6,'(a)') ' This is an ML-run.'
    if(linwf) then
       write(6,'(a)') ' Initial wave function (linwf=.true.)'
    else
       write(6,'(a)') ' Propagated/relaxed wave function (linwf=.false.)'
    endif
    if(lrlx) then
       write(6,'(a)') ' Ralaxation run (lrlx=.true.)'
    elseif(irunprop .eq. 1) then
       write(6,'(a)') ' Propagation run (lrunprop=.true.)'
    elseif(lrundiag) then
       write(6,'(a)') ' Diagonalisation run,Lanczos (lrundiag=.true.)'
    endif
    write(6,'(a,1pe17.9)') ' normthermal =',normthermal
    write(6,'(a,f12.6,a,f12.6)') ' out1[fs] =', out1/fs, ',  out2[fs] =', out2/fs
    write(6,'(a,f13.5,a,f13.6)') ' Time[au] =',time,',  Time[fs] =',time/fs
    write(6,'(a,i6,a,i6,a,i3,a,i6,a,i3)') ' ndof =',ndof,', nmode =',nmode,&
           ', nstate =',nstate,', maxdim =',maxdim,', maxsta =',maxsta
    write(6,'(a,2(f15.9,a))') ' E-start =', estart,'[eV],   ',estart/ev,'[au]'

   if(.not.lall) stop

   write(6,'(/6(a,i5))') ' npacket =',npacket,',  npackts =',npackts,&
      ',  feb =',feb,',  meb =',meb,',  fpb =', fpb,',  mpb =',mpb
!   if(lrunml) write(6,'(a,i8,a,i4,a,i8)') &
!    ' mladim =',mladim,',  mlatopdim =',mlatopdim,',  mltotphidim =',mltotphidim

   write(6,'(/a)') ' lconst  lfunkl  lfunkr  lexact lconstfix lconstvar lcmfadaphi'
   write(6,'(6(3x,l2,3x))') lconst,lfunkl,lfunkr,lexact,lconstfix,lconstvar,lcmfadaphi
   write(5,'(/a)') ' lnaturcmf  lopro    lcap   lreal  lrreal'
   write(5,'(6(3x,l2,3x))') lnaturcmf,lopro,lcap,lreal,lrreal
   write(6,'(/a)') ' lmlcmfsplit liork lrealphi lrhonrm limportho lenorb usehsym ldirmuld'
   write(6,'(3x,8(3x,l2,3x))') lmlcmfsplit,lrealphi,lrhonrm,limportho,liork,lenorb,usehsym,ldirmuld
!!   write(6,'(a,i6)')  'ktot =', ktot
   write(6,'(/a)') '    macheps         epsmat        epssvd'
   write(6,'(1p3e14.5)') macheps,epsmat,epssvd
   write(6,'(/a)') ' gdim(1) ... gdim(ndof)'
   write(6,'(10i7)') (gdim(f),f=1,ndof)
   write(6,'(/a)') ' modelabel(1) ... modelabel(ndof)'
   write(6,'(4x,10a7)') (modelabel(f),f=1,ndof)
   write(6,'(/a)') ' basisi(1) ... basis(ndof)'
   write(6,'(4x,10a7)') (cbas(basis(f)),f=1,ndof)

    stop
 99 write(6,'(a)') ' Cannot read time from restart file!'
    stop

contains
 
!#######################################################################
!-----------------------------------------------------------------------
    subroutine rdrstinput(lall,lprint,lnth)

    implicit none

    integer   i, iarg, myiargc
    logical   lall,lprint,lnth
    character*80   buf
!------------------------------------------------------------------

    rname  = ' '
    name   = './'
    lall   = .false.
    lprint = .false.
    lnth   = .false.
    iarg   = myiargc()
    if (iarg .lt. 1) goto 200

    i = 0
100 i=i+1
    call mygetarg(i,buf)
    if( buf(1:1) .eq. '-' ) then
       if( buf .eq. '-f') then
          i=i+1
          call mygetarg(i,buf)
          rname = buf
       else if( buf .eq. '-all' ) then
          lall = .true.
       else if( buf .eq. '-ml') then
          lprint = .true.
       else if( buf .eq. '-nth') then
          lnth = .true.

       else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
          write(6,'(1x,a)')&
         'Purpose: Reads information on a restart file.'
          write(6,'(1x,a)')&
         'Usage:   rdrestart [-f -nth -ml -all -ver -h -?] .'
          write(6,'(/,a)') 'Options : '
          write(6,'(2a,/,9x,a/,9x,a)')&
                   ' -f FILE : The restart file '&
                  ,'is read from file '&
                  ,'  FILE rather than from ./restart'&
          ,'  The string FILE may be a relative or a full path-name.'
          write(6,'(a)')&
         ' -nth    : Print only normthermal.',&
         ' -all    : Print more information (except ML-tree).',&
         ' -ml     : Print information on ML-tree (and nothing elese).'
          write(6,'(a)')&
         ' -ver    : Version information about the program '
          write(6,'(a,/a)')&
                   ' -h      : Print this help text. ',&
                   ' -?      : Print this help text. '
          write(6,'(a)') ' Note: The output is to screen.'
          write(6,'(78a1)')  ('-',i=1,78)
          stop
       else if( buf(1:4) .eq. '-ver' ) then
          call wrversion(6)
          stop
       else
          write(6,'(2a)') ' unknown option : ', buf
          stop
       end if
       if (iarg .le. i) goto 200
       goto 100
    end if

    if (iarg .lt. i) goto 200

200 continue


!-----------------------------------------------------------------------
! If rname has not been set, look in name directory.
! Otherwise take name from rname
!-----------------------------------------------------------------------
    if (rname .eq. ' ') then
       laenge=index(name,' ')-1
       call abspath(name,laenge)
       rname=name(1:laenge)//'/restart'
       rlaenge=index(rname,' ')-1
    else
       rlaenge=index(rname,' ')-1
       call abspath(rname,rlaenge)

       i=rlaenge
 10    continue
       if (rname(i:i) .eq. '/') then
          laenge=i-1
       else
          i=i-1
          go to 10
       endif
       name=rname(1:laenge)
    endif

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
    open(irst,file=rname(1:rlaenge),form='unformatted',status='old',err=88)
    if(.not.lnth) write(6,'(/2a)') ' The inputfile is : ',rname(1:rlaenge)

    return

 88 write(6,*)' Cannot open file : '
    write(6,*) rname(1:rlaenge)
    stop

    end subroutine rdrstinput

    end program rdrestart
