!***********************************************************************
!
!                        ANALYSE MODULE RDSTEPS
!
! Reads the integration steps from file "steps" and computes some
! statistics.
!
!   11/96 MB
!
!
!***********************************************************************

      program rdsteps

use decimal, only: long,dop
use global
use aglobal
use versions
use maxv
use op1lib, only:  zerovxi, zerovxd

use module_interface

      implicit none

      integer(long), parameter  :: channel = 1, irst = 3

      logical(kind=4)         :: lconst
      integer(long), external :: myiargc
      integer(long)       :: int1,int2,mode,m,maxordvmfabm,stepsvmfabm,&
                             maxordaabm,stepsaabm,colon,nmode,stepsasil,&
                             iterasil,maxordasil,sumordasil,stepsvmfbs,&
                             maxitervmfbs,sumitervmfbs,sumsmallvmfbs,stepsabs,&
                             maxiterabs,sumiterabs,sumsmallabs,inttyp,inttypea,&
                             inttypephi,inttypeex,i,iarg,stepsark5,&
                             rjctark5,stepsvmfrk5,rjctvmfrk5,stepsark8,stepsazvode,&
                             int2old,rjctazvode
      real(dop)           :: real1,real2,sumvmfabm,sumaabm,sumasil,sumvmfbs,sumabs,&
                             sumark5,sumvmfrk5,tstart,tm1,sumark8,sumvmfrk8,time,&
                             rjctark8,stepsvmfrk8,rjctvmfrk8,sumazvode

      integer(long), dimension(:), allocatable :: maxordphiabm,stepsphiabm,&
                                          stepsphibs,&
                                          maxiterphibs,sumiterphibs,sumsmallphibs,&
                                          stepsphirk5,rjctphirk5,stepsphirk8,&
                                          rjctphirk8,rjctphizvode,stepsphizvode
      real(dop), dimension(:), allocatable    :: sumphiabm,sumphibs,sumphirk5,&
                                          sumphirk8,sumphizvode
      integer(long)                    :: ilbl
      character(len=80)                :: line,data,dummy,buf,filename
      logical(kind=4)                  :: check

      integer(long)     :: ierr
      character(len=14) :: cerr

!-----------------------------------------------------------------------
! Initialise variables
!-----------------------------------------------------------------------
      lconst       =.false.
      lexact       =.false.
      mode         =0
      nmode        =0
      maxordvmfabm =0
      stepsvmfabm  =0
      maxordaabm   =0
      stepsaabm    =0
      stepsasil    =0
      iterasil     =0
      maxordasil   =0
      sumordasil   =0
      stepsvmfbs   =0
      maxitervmfbs =0
      sumitervmfbs =0
      sumsmallvmfbs=0
      stepsabs     =0
      maxiterabs   =0
      sumiterabs   =0
      sumsmallabs  =0
      stepsark5    =0
      rjctark5     =0
      stepsvmfrk5  =0
      rjctvmfrk5   =0
      stepsark8    =0
      rjctark8     =0
      stepsvmfrk8  =0
      rjctvmfrk8   =0
      stepsazvode  =0
      rjctazvode   =0
      inttyp       =0
      inttypea     =0
      inttypephi   =0
      inttypeex    =0
      sumvmfabm=0.0_dop
      sumaabm  =0.0_dop
      sumasil  =0.0_dop
      sumvmfbs =0.0_dop
      sumabs   =0.0_dop
      sumvmfrk5=0.0_dop
      sumark5  =0.0_dop
      sumvmfrk8=0.0_dop
      sumark8  =0.0_dop
      sumazvode=0.0_dop
      tfin     =9.9e99_dop
      tstart   =-1.0_dop

!-----------------------------------------------------------------------
! Get name of directory where the file "steps" is stored.
!-----------------------------------------------------------------------
      filein = '  '
      name   = './'

!-----------------------------------------------------------------------
! read options
!-----------------------------------------------------------------------
      iarg = myiargc()
      if (iarg .lt. 1) goto 2

      i = 0
   1  i=i+1
      if (myiargc() .ge. 1) then
         call mygetarg(i,buf)
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if(buf .eq. '-a') then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*) tstart
         else if(buf .eq. '-t') then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*) tfin

         else if( buf(1:2) .eq. '-h' .or.  buf(1:2) .eq. '-?') then
            write(6,'(1x,2a,/11x,a)')&
              'Purpose : Reads the step-file and evaluates ', &
              'statistic measures','of the integrator steps.'
            write(6,'(1x,a)')&
                 'Usage   : rdsteps [-f -i -a -t -ver -h -?].'
            write(6,'(/,a)') ' Options : '
            write(6,'(2a,/,11x,a/11x,a)') &
              ' -f FILE : the data is read '&
              ,'from file FILE','rather than from ./steps'&
              ,'The string FILE may be a relative or full path-name.'
            write(6,'(a)') ' -i DIR  : data is stored in directory DIR'
            write(6,'(2a)') ' -a tstrt: steps file is read ', &
                  'only for times larger than tstrt'
            write(6,'(a)') &
                  ' -t tfin : steps file is read only up to time tfin'
            write(6,'(a)')&
              ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
              ' -h      : Print this help text. ',&
              ' -?      : Print this help text. '
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else
            write(6,'(2a)') ' Unknown option :', buf
            write(6,'(2a)') ' Use option -h'
            stop
         end if
         if (iarg .le. i) goto 2
         goto 1

      end if

    2 continue

!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
! Otherwise take name from filein
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         filein=name(1:laenge)//'/steps'
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

!-----------------------------------------------------------------------
! get array dimensions 
!-----------------------------------------------------------------------
      inquire(irst,opened=check)
      if (check) close(irst)
      filename=name(1:laenge)//'/restart'
      ilbl=index(filename,' ')-1
      open(irst,file=filename(1:ilbl),form='unformatted',status='old',err=900)
      call rdmemdim(irst)
      close(irst)

      cerr = 'maxordphiabm'
      allocate(maxordphiabm(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'stepsphiabm'
      allocate(stepsphiabm(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'stepsphibs'
      allocate(stepsphibs(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'maxiterphibs'
      allocate(maxiterphibs(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'sumiterphibs'
      allocate(sumiterphibs(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'sumsmallphibs'
      allocate(sumsmallphibs(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'stepsphirk5'
      allocate(stepsphirk5(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'rjctphirk5'
      allocate(rjctphirk5(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'stepsphirk8'
      allocate(stepsphirk8(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'rjctphirk8'
      allocate(rjctphirk8(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'stepsphizvode'
      allocate(stepsphizvode(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'rjctphizvode'
      allocate(rjctphizvode(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'sumsphiabm'
      allocate(sumphiabm(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'sumsphibs'
      allocate(sumphibs(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'sumsphirk5'
      allocate(sumphirk5(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'sumsphirk8'
      allocate(sumphirk8(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'sumsphizvode'
      allocate(sumphizvode(maxdim),stat=ierr)
      if (ierr > 0) go to 1000

      call zerovxi(maxordphiabm,maxdim)
      call zerovxi(stepsphiabm,maxdim)
      call zerovxi(stepsphibs,maxdim)
      call zerovxi(maxiterphibs,maxdim)
      call zerovxi(sumiterphibs,maxdim)
      call zerovxi(sumsmallphibs,maxdim)
      call zerovxi(stepsphirk5,maxdim)
      call zerovxi(rjctphirk5,maxdim)
      call zerovxi(stepsphirk8,maxdim)
      call zerovxi(rjctphirk8,maxdim)
      call zerovxd(sumphiabm,maxdim)
      call zerovxd(sumphibs,maxdim)
      call zerovxd(sumphirk5,maxdim)
      call zerovxd(sumphirk8,maxdim) 
      call zerovxi(stepsphizvode,maxdim)
      call zerovxi(rjctphizvode,maxdim)
      call zerovxd(sumphizvode,maxdim)

!-----------------------------------------------------------------------
! Open steps file
!-----------------------------------------------------------------------
      open(channel,file=filein(1:laein),form='formatted',&
           status='old',err=666)

      write(6,'(78a1)')  ('-',i=1,78)
      if(laein .le. 58 ) then
         write(6,'(2a)') ' The inputfile is: ', filein(1:laein)
      else
         write(6,'(a)')  ' The inputfile is:' 
         write(6,'(a)')   filein(1:laein)
      end if
      write(6,'(78a1)')  ('-',i=1,78)
      if(tfin  .lt. 9.0e99_dop) write(6,'(a,f10.3)') ' tfin =',  tfin
      if(tstart .ge. 0.0_dop) write(6,'(a,f10.3)') ' tstart =',tstart

!-----------------------------------------------------------------------
! Check steps file
!-----------------------------------------------------------------------
      read(channel,'(a)',err=666,end=666) dummy
      read(channel,'(a)',err=666,end=666) dummy
      read(channel,'(a)',err=666,end=666) dummy
      rewind channel

!-----------------------------------------------------------------------
! Check for final time to exclude incomplete sets
!-----------------------------------------------------------------------
      if(tfin .gt. 9.0e99_dop) then
         time=0.0_dop
  15     read(channel,'(a)',err=666,end=25) line
         if(index(line,'Time in fs:').eq.1) then 
            tm1=time
            read(line,'(a12,f12.5)') dummy, time
            read(channel,'(a)',err=666,end=25) line
            read(channel,'(a)',err=666,end=25) line
         endif
         goto 15
  25     tfin = 0.2*(4.0_dop*tm1+time)
         if(time .gt. 9999.9_dop) then
            write(6,'(a,f9.1,a)') 'Final Time : ',time,' fs'
         else
            write(6,'(a,f9.3,a)') 'Final Time : ',time,' fs'
         endif       
         rewind channel
      endif
  
!-----------------------------------------------------------------------
! Look for next headline
!-----------------------------------------------------------------------
 100  continue
      int2old=1000000
      read(channel,'(a)',err=666,end=200) line
      if(index(line,'Time in fs:').eq.1) then
         read(line,'(a12,f12.5)') dummy, time
         if(time .gt. tfin+1.0e-9_dop) goto 200
      endif 
      if(time .lt. tstart) goto 100
      if ((index(line,'Complete') .eq. 0) .and.&
          (index(line,'A:') .eq. 0) .and. (index(line,'Mode') .eq. 0)&
          .and. (index(line,'Exact') .eq. 0))&
         goto 100
      read(channel,'(a)',err=666,end=200) dummy
      if (index(line,'Mode') .ne. 0) then
         colon = index(line,':')
         if (colon .gt. 0) read(line(5:colon-1),*) mode
      endif

!-----------------------------------------------------------------------
! Read and analyse data
!-----------------------------------------------------------------------
 300  continue
      read(channel,'(a)',err=666,end=200) data
      if (index(data,' ') .eq. 1) then
         read(data,*,err=666) int1,int2,real1,real2
         if (index(line,'Complete') .ne. 0) then
            lconst = .false.
            lexact = .false.
            if (index(line,'Adams-Bashforth-Moulton') .ne. 0) then
               inttyp = 1
               stepsvmfabm =stepsvmfabm+1
               maxordvmfabm=max(maxordvmfabm,int2)
               sumvmfabm   =sumvmfabm+real1
            elseif (index(line,'Bulirsch-Stoer') .ne. 0) then
               inttyp = 2
               if (int1 .eq. 1) then
                  stepsvmfbs=stepsvmfbs+1
                  sumvmfbs  =sumvmfbs+real1
               endif
               maxitervmfbs =max(maxitervmfbs,int1)
               sumitervmfbs =sumitervmfbs+1
               sumsmallvmfbs=sumsmallvmfbs+int2
            elseif (index(line,'Runge-Kutta order 5') .ne. 0) then
               inttyp = 4
               stepsvmfrk5 = stepsvmfrk5+1
               if (int2 .eq. 0) then
                  rjctvmfrk5 = rjctvmfrk5+1
               else
!                 only count stepsizes for accepted steps
                  sumvmfrk5 = sumvmfrk5+real1
               endif
            elseif (index(line,'Runge-Kutta order 8') .ne. 0) then
               inttyp = 5
               stepsvmfrk8 = stepsvmfrk8+1
               if (int2 .eq. 0) then
                  rjctvmfrk8 = rjctvmfrk8+1
               else
!                 only count stepsizes for accepted steps
                  sumvmfrk8 = sumvmfrk8+real1
               endif
            elseif (index(line,'Block-Davidson') .ne. 0)  then
               inttypea = 60
               if (real1 .gt. 0.0d0) then
                  if (int2 .lt. int2old) then
                     if(int2old .eq. 1000000 ) then
                        stepsasil =stepsasil+1
                        sumordasil=sumordasil+int2-1
                        iterasil  =iterasil+int2-1
                     endif
                  endif
                  iterasil  =iterasil+1
                  maxordasil=max(maxordasil,int2)
                  sumordasil=sumordasil+1
                  int2old=int2
               endif
            elseif (index(line,'Davidson') .ne. 0) then
               inttypea = 6
               if (real1 .gt. 0.0d0) then
                  if (int2 .eq. 1) then
                     stepsasil=stepsasil+1
                  endif
                  iterasil  =iterasil+1
                  maxordasil=max(maxordasil,int2)
                  sumordasil=sumordasil+1
               endif
            else
               goto 777
            endif
         elseif (index(line,'A:') .ne. 0) then
            lconst = .true.
            lexact = .false.
            if (index(line,'Adams-Bashforth-Moulton') .ne. 0) then
               inttypea = 1
               stepsaabm =stepsaabm+1
               maxordaabm=max(maxordaabm,int2)
               sumaabm   =sumaabm+real1
            elseif (index(line,'Bulirsch-Stoer') .ne. 0) then
               inttypea = 2
               if (int1 .eq. 1) then
                  stepsabs=stepsabs+1
                  sumabs  =sumabs+real1
               endif
               maxiterabs =max(maxiterabs,int1)
               sumiterabs =sumiterabs+1
               sumsmallabs=sumsmallabs+int2
            elseif (index(line,'Lanczos') .ne. 0) then
               inttypea = 3
               if (real1 .ge. 0.0_dop) then
                  if (int2 .eq. 1) then
                     stepsasil=stepsasil+1
                     sumasil  =sumasil+real1
                  endif
                  iterasil  =iterasil+1
                  maxordasil=max(maxordasil,int2)
                  sumordasil=sumordasil+1
               endif
            elseif (index(line,'Runge-Kutta order 5') .ne. 0) then
               inttypea = 4
               stepsark5 = stepsark5+1
               if (int2 .eq. 0) then
                  rjctark5 = rjctark5+1
               else
!                 only count stepsizes for accepted steps
                  sumark5 = sumark5+real1
               endif
            elseif (index(line,'Runge-Kutta order 8') .ne. 0) then
               inttypea = 5
               stepsark8 = stepsark8+1
               if (int2 .eq. 0) then
                  rjctark8 = rjctark8+1
               else
!                 only count stepsizes for accepted steps
                  sumark8 = sumark8+real1
               endif
            elseif (index(line,'Davidson') .ne. 0) then
               inttypea = 6 
               if (real1 .gt. 0.0_dop) then
                  if (int2 .eq. 1) then
                     stepsasil=stepsasil+1
                     sumasil  =sumasil+real1
                  endif
                  iterasil  =iterasil+1
                  maxordasil=max(maxordasil,int2)
                  sumordasil=sumordasil+1
               endif
            elseif (index(line,'ZVODE') .ne. 0) then
               inttypea = 9
               stepsazvode = stepsazvode+1
               if (int2 .eq. 0) then
                  rjctazvode = rjctazvode+1
               else
!                 only count stepsizes for accepted steps
                  sumazvode = sumazvode+real1
               endif
            else
               goto 777
            endif
         elseif (index(line,'Mode') .ne. 0) then
            lconst = .true.
            lexact = .false.
            if (index(line,'Adams-Bashforth-Moulton') .ne. 0) then
               inttypephi = 1
               stepsphiabm(mode) =stepsphiabm(mode)+1
               maxordphiabm(mode)=max(maxordphiabm(mode),int2)
               sumphiabm(mode)   =sumphiabm(mode)+real1
            elseif (index(line,'Bulirsch-Stoer') .ne. 0) then
               inttypephi = 2
               if (int1 .eq. 1) then
                  stepsphibs(mode)=stepsphibs(mode)+1
                  sumphibs(mode)  =sumphibs(mode)+real1
               endif
               maxiterphibs(mode) =max(maxiterphibs(mode),int1)
               sumiterphibs(mode) =sumiterphibs(mode)+1
               sumsmallphibs(mode)=sumsmallphibs(mode)+int2
            elseif (index(line,'Runge-Kutta order 5') .ne. 0) then
               inttypephi = 4
               stepsphirk5(mode) = stepsphirk5(mode)+1
               if (int2 .eq. 0) then
                  rjctphirk5(mode) = rjctphirk5(mode)+1
               else
!                 only count stepsizes for accepted steps
                  sumphirk5(mode) = sumphirk5(mode)+real1
               endif
            elseif (index(line,'Runge-Kutta order 8') .ne. 0) then
               inttypephi = 5
               stepsphirk8(mode) = stepsphirk8(mode)+1
               if (int2 .eq. 0) then
                  rjctphirk8(mode) = rjctphirk8(mode)+1
               else
!                 only count stepsizes for accepted steps
                  sumphirk8(mode) = sumphirk8(mode)+real1
               endif
            elseif (index(line,'ZVODE') .ne. 0) then
               inttypephi = 9
               stepsphizvode(mode) = stepsphizvode(mode)+1
               if (int2 .eq. 0) then
                  rjctphizvode(mode) = rjctphizvode(mode)+1
               else
!                 only count stepsizes for accepted steps
                  sumphizvode(mode) = sumphizvode(mode)+real1
               endif
            else
               goto 777
            endif
         elseif (index(line,'Exact wavepacket:') .ne. 0) then
            lconst = .false.
            lexact = .true.
            if (index(line,'Adams-Bashforth-Moulton') .ne. 0) then
               inttypeex = 1
               stepsaabm =stepsaabm+1
               maxordaabm=max(maxordaabm,int2)
               sumaabm   =sumaabm+real1
            elseif (index(line,'Bulirsch-Stoer') .ne. 0) then
               inttypeex = 2
               if (int1 .eq. 1) then
                  stepsabs=stepsabs+1
                  sumabs  =sumabs+real1
               endif
               maxiterabs =max(maxiterabs,int1)
               sumiterabs =sumiterabs+1
               sumsmallabs=sumsmallabs+int2
            elseif (index(line,'Lanczos') .ne. 0) then
               inttypeex = 3
               if (int2 .eq. 1) then
                  stepsasil=stepsasil+1
                  sumasil  =sumasil+real1
               endif
               iterasil  =iterasil+1
               maxordasil=max(maxordasil,int2)
               sumordasil=sumordasil+1
            elseif (index(line,'Runge-Kutta order 5') .ne. 0) then
               inttypeex = 4
               stepsark5 = stepsark5+1
               if (int2 .eq. 0) then
                  rjctark5 = rjctark5+1
               else
!                 only count stepsizes for accepted steps
                  sumark5 = sumark5+real1
               endif
            elseif (index(line,'Runge-Kutta order 8') .ne. 0) then
               inttypeex = 5
               stepsark8 = stepsark8+1
               if (int2 .eq. 0) then
                  rjctark8 = rjctark8+1
               else
!                 only count stepsizes for accepted steps
                  sumark8 = sumark8+real1
               endif
            elseif (index(line,'ZVODE') .ne. 0) then
               inttypeex = 9
               stepsazvode = stepsazvode+1
               if (int2 .eq. 0) then
                  rjctazvode = rjctazvode+1
               else
!                 only count stepsizes for accepted steps
                  sumazvode = sumazvode+real1
               endif
            else
               goto 777
            endif
         else
            write(6,*)
            write(6,'(a)')&
               'ERROR in program rdsteps: Unknown data in steps file.'
            write(6,'(2a)') 'line : ', line(1:70)
            write(6,'(2a)') 'data : ', data(1:70)
            write(6,'(2a)') 'dummy: ', dummy(1:70)
            write(6,*)
            stop
         endif
         goto 300
      else
         backspace(channel,err=666)
         goto 100
      endif
 200  continue
      close(channel,err=666)

!-----------------------------------------------------------------------
! Output statistics
!-----------------------------------------------------------------------
      write(6,'(a)') 'Analysis of integrator steps.'
      if (.not. lconst) then
         if (.not. lexact) then
            write(6,'(a)') 'Variable mean-field (VMF) mode.'
            write(6,*)
            if (inttyp .eq. 1) then
               write(6,'(a)') 'Complete ODE: ABM integrator.'
               write(6,'(a,f10.6)')&
                  'Mean stepsize (in fs): ',  sumvmfabm/stepsvmfabm
               write(6,'(a,i8)') 'Overall number of steps: ',stepsvmfabm
               write(6,'(a,i8)') '(Maximum) order:         ',&
                  maxordvmfabm
            elseif (inttyp .eq. 2) then
               write(6,'(a)') 'Complete ODE: BS integrator.'
               write(6,'(a,f10.6)')&
                  'Mean large stepsize (in fs): ',  sumvmfbs/stepsvmfbs
               write(6,'(a,i8)')&
                  'Overall number of large steps: ',stepsvmfbs
               write(6,'(a,i8)')&
                  'Overall number of iterations:  ',sumitervmfbs
               write(6,'(a,i8)')&
                  'Maximum number of iterations:  ',maxitervmfbs
               write(6,'(a,i8)')&
                  'Overall number of small steps: ',sumsmallvmfbs
            elseif (inttyp .eq. 4) then
               write(6,'(a)') 'Complete ODE: RK5 integrator.'
               write(6,'(a,i8)')&
                    'Total number of steps:          ',stepsvmfrk5
               write(6,'(a,i8)')&
                    'Total number of rejected steps: ',rjctvmfrk5
               write(6,'(a,f12.6)')&
                    'Mean stepsize [fs]:         ',&
                    sumvmfrk5/(stepsvmfrk5-rjctvmfrk5)
            elseif (inttyp .eq. 5) then
               write(6,'(a)') 'Complete ODE: RK8 integrator.'
               write(6,'(a,i8)')&
                    'Total number of steps:          ',stepsvmfrk8
               write(6,'(a,i8)')&
                    'Total number of rejected steps: ',rjctvmfrk8
               write(6,'(a,f12.6)')&
                    'Mean stepsize [fs]:         ',&
                    sumvmfrk8/(stepsvmfrk8-rjctvmfrk8)
            else
               goto 777
            endif
         else
            write(6,'(a)') 'Exact wavepacket propagation.'
            write(6,*)
            if (inttypeex .eq. 1) then
               write(6,'(a)') 'Complete ODE: ABM integrator.'
               write(6,'(a,f8.6)')&
                  'Mean stepsize (in fs):   ',sumaabm/stepsaabm
               write(6,'(a,i8)') 'Overall number of steps: ',stepsaabm
               write(6,'(a,i8)') '(Maximum) order:         ',maxordaabm
            elseif (inttypeex .eq. 2) then
               write(6,'(a)') 'Complete ODE: BS integrator.'
               write(6,'(a,f8.6)')&
                  'Mean large stepsize (in fs):   ',sumabs/stepsabs
               write(6,'(a,i8)')&
                  'Overall number of large steps: ',stepsabs
               write(6,'(a,i8)')&
                  'Overall number of iterations:  ',sumiterabs
               write(6,'(a,i8)')&
                  'Maximum number of iterations:  ',maxiterabs
               write(6,'(a,i8)')&
                  'Overall number of small steps: ',sumsmallabs
            elseif (inttypeex .eq. 3) then
               write(6,'(a)') 'MCTDH coefficients: SIL/CSIL integrator.'
               write(6,'(a,f8.3)') 'Mean order:                   ',&
                                   dble(sumordasil)/dble(stepsasil)
               write(6,'(a,i8)')&
                  'Maximum order:                ',maxordasil
               write(6,'(a,f12.6)')&
                  'Mean stepsize (in fs):    ',    sumasil/stepsasil
               write(6,'(a,i8)')&
                  'Overall number of steps:      ',stepsasil
               write(6,'(a,i8)')&
                  'Overall number of iterations: ',iterasil
            elseif (inttypeex .eq. 4) then
               write(6,'(a)') 'Complete ODE: RK5 integrator.'
               write(6,'(a,i8)')&
                    'Total number of steps:          ',stepsark5
               write(6,'(a,i8)')&
                    'Total number of rejected steps: ',rjctark5
               write(6,'(a,f12.6)')&
                    'Mean stepsize [fs]:         ',&
                    sumark5/(stepsark5-rjctark5)
            elseif (inttypeex .eq. 5) then
               write(6,'(a)') 'Complete ODE: RK8 integrator.'
               write(6,'(a,i8)')&
                    'Total number of steps:          ',stepsark8
               write(6,'(a,i8)')&
                    'Total number of rejected steps: ',rjctark8
               write(6,'(a,f12.6)')&
                    'Mean stepsize [fs]:         ',&
                    sumark8/(stepsark8-rjctark8)
            elseif (inttypeex .eq. 9) then
               write(6,'(a)') 'Complete ODE: ZVODE integrator.'
               write(6,'(a,i8)')&
                    'Total number of steps:          ',stepsazvode
               write(6,'(a,i8)')&
                    'Total number of rejected steps: ',rjctazvode
               write(6,'(a,f12.6)')&
                    'Mean stepsize [fs]:         ',&
                    sumazvode/(stepsazvode-rjctazvode)
            else
               goto 777
            endif
            write(6,*)
         endif
      else
         write(6,'(a)') 'Constant mean-field (CMF) mode.'
         write(6,*)
         if (inttypea .eq. 1) then
            write(6,'(a)') 'MCTDH coefficients: ABM integrator.'
            write(6,'(a,f10.6)')&
               'Mean stepsize (in fs): ',  sumaabm/stepsaabm
            write(6,'(a,i8)') 'Overall number of steps: ',stepsaabm
            write(6,'(a,i8)') '(Maximum) order:         ',maxordaabm
         elseif (inttypea .eq. 2) then
            write(6,'(a)') 'MCTDH coefficients: BS integrator.'
            write(6,'(a,f10.6)')&
               'Mean large stepsize (in fs): ',  sumabs/stepsabs
            write(6,'(a,i8)')&
               'Overall number of large steps: ',stepsabs
            write(6,'(a,i8)')&
               'Overall number of iterations:  ',sumiterabs
            write(6,'(a,i8)')&
               'Maximum number of iterations:  ',maxiterabs
            write(6,'(a,i8)')&
               'Overall number of small steps: ',sumsmallabs
         elseif (inttypea .eq. 3) then
            write(6,'(a)') 'MCTDH coefficients: SIL/CSIL integrator.'
            write(6,'(a,f8.3)') 'Mean order:                   ',&
                                dble(sumordasil)/dble(stepsasil)
            write(6,'(a,i8)')&
               'Maximum order:                ',maxordasil
            write(6,'(a,f12.6)')&
               'Mean stepsize (in fs):    ',    sumasil/stepsasil
            write(6,'(a,i8)')&
               'Overall number of steps:      ',stepsasil
            write(6,'(a,i8)')&
               'Overall number of iterations: ',iterasil
         elseif (inttypea .eq. 4) then
            write(6,'(a)') 'MCTDH coefficients: RK5 integrator.'
            write(6,'(a,i8)')&
                 'Total number of steps:          ',stepsark5
            write(6,'(a,i8)')&
                 'Total number of rejected steps: ',rjctark5
            write(6,'(a,f12.6)')&
                 'Mean stepsize [fs]:         ',&
                 sumark5/(stepsark5-rjctark5)
         elseif (inttypea .eq. 5) then
            write(6,'(a)') 'MCTDH coefficients: RK8 integrator.'
            write(6,'(a,i8)')&
                 'Total number of steps:          ',stepsark8
            write(6,'(a,i8)')&
                 'Total number of rejected steps: ',rjctark8
            write(6,'(a,f12.6)')&
                 'Mean stepsize [fs]:         ',&
                 sumark8/(stepsark8-rjctark8)
         elseif (inttypea .eq. 6) then
            write(6,'(a)') 'MCTDH coefficients: Dav integrator.'
            write(6,'(a,f8.3)') 'Mean order:                   ',&
                                dble(sumordasil)/dble(stepsasil)
            write(6,'(a,i8)')&
               'Maximum order:                ',maxordasil
            write(6,'(a,f12.6)')&
               'Mean stepsize (in fs):    ',    sumasil/stepsasil
            write(6,'(a,i8)')&
               'Overall number of steps:      ',stepsasil
            write(6,'(a,i8)')&
               'Overall number of iterations: ',iterasil
         elseif (inttypea .eq. 9) then
            write(6,'(a)') 'MCTDH coefficients: ZVODE integrator.'
            write(6,'(a,i8)')&
                 'Total number of steps:          ',stepsazvode
            write(6,'(a,i8)')&
                 'Total number of rejected steps: ',rjctazvode
            write(6,'(a,f12.6)')&
                 'Mean stepsize [fs]:         ',&
                 sumazvode/(stepsazvode-rjctazvode)
         else
            goto 777
         endif
         write(6,*)
         if (inttypephi .eq. 1) then
            do m=1,nmode
               write(6,'(a,i2,a)')&
                  'Single-particle functions of mode ',m,&
                  ': ABM integrator.'
               write(6,'(a,f10.6)') 'Mean stepsize (in fs): ',&
                                    sumphiabm(m)/stepsphiabm(m)
               write(6,'(a,i8)')&
                  'Overall number of steps: ',stepsphiabm(m)
               write(6,'(a,i8)')&
                  '(Maximum) order:         ',maxordphiabm(m)
               write(6,*)
            enddo
         elseif (inttypephi .eq. 2) then
            do m=1,nmode
               write(6,'(a,i2,a)') &
                  'Single-particle functions of mode ',m,&
                  ': BS integrator.'
               write(6,'(a,f10.6)') 'Mean large stepsize (in fs): ',&
                                    sumphibs(m)/stepsphibs(m)
               write(6,'(a,i8)')&
                  'Overall number of large steps: ',stepsphibs(m)
               write(6,'(a,i8)')&
                  'Overall number of iterations:  ',sumiterphibs(m)
               write(6,'(a,i8)')&
                  'Maximum number of iterations:  ',maxiterphibs(m)
               write(6,'(a,i8)') 'Overall number of small steps: ',&
                  sumsmallphibs(m)
               write(6,*)
            enddo
         elseif (inttypephi .eq. 4) then
            do m=1,mode
               write(6,'(a,i2,a)')&
                    'Single-particle functions of mode ',m,&
                    ': RK5 integrator'
               write(6,'(a,i8)')&
                    'Total number of steps:          ',stepsphirk5(m)
               write(6,'(a,i8)')&
                    'Total number of rejected steps: ',rjctphirk5(m)
               write(6,'(a,f12.6)')&
                    'Mean stepsize [fs]:         ',&
                    sumphirk5(m)/(stepsphirk5(m)-rjctphirk5(m))
               write(6,*)
            enddo
         elseif (inttypephi .eq. 5) then
            do m=1,mode
               write(6,'(a,i2,a)')&
                    'Single-particle functions of mode ',m,&
                    ': RK8 integrator'
               write(6,'(a,i8)')&
                    'Total number of steps:          ',stepsphirk8(m)
               write(6,'(a,i8)')&
                    'Total number of rejected steps: ',rjctphirk8(m)
               write(6,'(a,f12.6)')&
                    'Mean stepsize [fs]:         ',&
                    sumphirk8(m)/(stepsphirk8(m)-rjctphirk8(m))
               write(6,*)
            enddo
         elseif (inttypephi .eq. 9) then
            do m=1,mode
               write(6,'(a,i2,a)')&
                    'Single-particle functions of mode ',m,&
                    ': ZVODE integrator'
               write(6,'(a,i8)')&
                    'Total number of steps:          ',stepsphizvode(m)
               write(6,'(a,i8)')&
                    'Total number of rejected steps: ',rjctphizvode(m)
               write(6,'(a,f12.6)')&
                    'Mean stepsize [fs]:         ',&
                    sumphizvode(m)/(stepsphizvode(m)-rjctphizvode(m))
               write(6,*)
            enddo
         else
            goto 777
         endif
      endif
      write(6,'(78a1)')  ('-',i=1,78)

      stop

!-----------------------------------------------------------------------
! Echo error message
!-----------------------------------------------------------------------
 666  continue
      write(6,*)
      write(6,'(a,/a)')&
         'ERROR in program rdsteps: Cannot open or read steps file:'&
         , filein(1:laein)
      write(6,*)
      stop

  900 continue
      write(6,*)
      write(6,'(a,/a)')&
         'ERROR in program rdsteps: Cannot open file:'&
         , filename(1:ilbl)
      write(6,*)
      stop

 777  continue
      write(6,*)
      write(6,'(a)') 'ERROR in program rdsteps: Unknown integration type.'
      write(6,'(2a)') 'line : ', line(1:70)
      write(6,'(2a)') 'data : ', data(1:70)
      write(6,'(2a)') 'dummy: ', dummy(1:70)
      write(6,*)
      stop

 1000 write(6,'(a)') 'Error allocating memory in Rdsteps :',cerr
      write(6,'(a)') 'Error Code :',ierr
      stop

      end program rdsteps


