!***********************************************************************
!
!                        ANALYSE MODULE RDUPDATE
!
! Reads the update steps from file "update" and sends the data to the
! screen.
!
! 11/96 MB
!
!
!***********************************************************************

      program rdupdate

use decimal, only: long,dop
use global
use versions
use maxv
use aglobal
use plot
use channels

use logdat
use plotutils, only: initplot,plotdefs,timebnds,dotask,plotmenu,databnds

use dvrdatmod
use griddatmod

use module_interface

      implicit none

      integer(long), external :: myiargc,slen
      integer(long)           :: upstep,nextstep,i,iarg,ilbl
      real(dop)               :: update,erra,errphi,pltime1
      character(len=80)       :: headline,buf

! interactive plot variables
      integer(long)                          :: ndata,plottask,task,plotdim,&
                                                dimx,dimy,nsets,nplots
      integer(long), dimension(1)            :: idum
      real(dop), dimension(1)                :: ydum,xdum,zdum,edum
      real(dop), dimension(:), allocatable   :: pltime
      real(dop), dimension(:,:), allocatable :: pldata
      character(len=c5)                      :: filename
      logical(kind=4)                        :: check

      integer(long)     :: ierr
      character(len=10) :: cerr

!-----------------------------------------------------------------------
! Initialise variables
!-----------------------------------------------------------------------
      call default
      call adefault
      idum(1)=0
      filein = '  '
      name   = './'

!-----------------------------------------------------------------------
! read options
!-----------------------------------------------------------------------
      iarg = myiargc()
      if (iarg .lt. 1) goto 2

      i = 0
   1  i=i+1
      if (myiargc() .ge. 1) then
         call mygetarg(i,buf)
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf .eq. '-inter' ) then
            linter=.true.
         else if( buf .eq. '-gnupl4 ') then
            lgnu4 = .true.                 ! gnuplot ver < 5
         else if( buf .eq. '-ver' ) then
            call wrversion(6)
            stop

         else if( buf .eq. '-h' .or.  buf .eq. '-?') then
            write(6,'(1x,a/,11x,a)') &
        'Purpose : Reads the update steps from file "update" ',&
        'Usage   : rdupdate [-f -i -inter -gnupl4 -ver -h -?].'
            write(6,'(a)') ' Options : '
            write(6,'(2a,/,11x,a/11x,a)') &
              ' -f FILE : the data is read '&
              ,'from file FILE','rather than from ./update'&
              ,'The string FILE may be a relative or full path-name.'
            write(6,'(a)') ' -i DIR : data is stored in directory DIR'
            write(6,'(a)') ' -inter : enables interactive plotting'
            write(6,'(a)')&
                     ' -gnupl4  : Use gnuplot version < 5 for plotting '
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
              ' -h      : Print this help text. ',&
              ' -?      : Print this help text. '
            stop
         else
            write(6,'(2a)') ' Unknown option :', buf
            write(6,'(2a)') ' Use option -h'
            stop
         end if
         if (iarg .le. i) goto 2
         goto 1

      end if
         
    2 continue
!-----------------------------------------------------------------------
! If filein has not been set, look in name directory. 
! Otherwise take name from filein
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         filein=name(1:laenge)//'/update'
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

!-----------------------------------------------------------------------
! Open and read file "update" generated in a constant mean-field
! calculation with adaptive stepsize. 
! Skip repetition steps and save data in arrays
!-----------------------------------------------------------------------
      open(iupd,file=filein(1:laein),form='formatted',&
           status='old',err=666)
      read(iupd,'(a)') headline
      ndata=0
 100  continue
      read(iupd,*,end=200,err=666) upstep 
      read(iupd,*,end=200,err=666) nextstep
      backspace(iupd,err=666)
      if (upstep  .ne. nextstep) backspace(iupd,err=666)
      read(iupd,*,end=200,err=666) upstep,update,erra,errphi,pltime1

      if (linter) then
         ndata=ndata+1
      else
         write(6,'(f8.3,3e20.8)') pltime1,update,erra,errphi
      endif

      goto 100

 200  continue

      if (.not. linter) then
         write(6,'(76a)') ('-',i=1,76)
         close(iupd,err=666)
         stop
      endif

!-----------------------------------------------------------------------
! interactive plotting
! dvr data neded in interactive plotting
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
! get array dimensions 
!-----------------------------------------------------------------------
      inquire(irst,opened=check)
      if (check) close(irst)
      filename=name(1:laenge)//'/restart'
      ilbl=index(filename,' ')-1
      open(irst,file=filename(1:ilbl),form='unformatted',status='old',err=900)
      call rdmemdim(irst)
      close(irst)

      call alloc_dvrdat
      call alloc_grddat

! allocate memory and read data
      maxdata=ndata
      cerr = 'pltime'
      allocate(pltime(maxdata),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'pldata'
      allocate(pldata(maxdata,3),stat=ierr)
      if (ierr > 0) go to 1000

      rewind(iupd)
      read(iupd,'(a)') headline
      ndata=0
      do
         read(iupd,*,end=300,err=666) upstep 
         read(iupd,*,end=300,err=666) nextstep
         backspace(iupd,err=666)
         if (upstep  .ne. nextstep) backspace(iupd,err=666)
         read(iupd,*,end=300,err=666) upstep,update,erra,errphi,pltime1

         ndata=ndata+1
         pltime(ndata)=pltime1
         pldata(ndata,1)=update
         pldata(ndata,2)=erra
         pldata(ndata,3)=errphi

      enddo

 300  continue
      close(iupd,err=666)

!-----------------------------------------------------------------------
! interactive plotting
! open file for data to be plotted
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/tmpplot.pl'
      open(iaus,file=filename,status='unknown')

      nplottask=3
      call alloc_plot

!-----------------------------------------------------------------------
! zero dummy variables
!-----------------------------------------------------------------------
      xdum(1)=0.0
      ydum(1)=0.0
      zdum(1)=0.0

      call plotdefs

      plotdim=1
      dimx=ndata
      dimy=1
      nsets=1
      nplots=1
      lpltime=.true.
      print*,ftime
      call initplot(filename,plotdim)

      call timebnds(pltime,ndata)

!-----------------------------------------------------------------------
! set program specific data, e.g. plot tasks
!-----------------------------------------------------------------------
      plottask=1

      cplottask(1)='Update step size'
      cplottask(2)='Error of A-vector'
      cplottask(3)='Error of spfs'
      do i=1,nplottask
         cptlen(i)=slen(cplottask(i))
      enddo

  250 continue
      call plotmenu(task,plottask,plotdim)

!-----------------------------------------------------------------------
! do task
! TASK=0 STOP  else return to menu
!-----------------------------------------------------------------------
      if (task .gt. 0) then
         if (plottask .eq. 1) then
            plttitle='Update step sizes'
            axislab(1)='Time [fs]'
            axislab(3)='Step [fs]'
         else if (plottask .eq. 2) then
            plttitle='CMF error in A-vector'
            axislab(1)='Time [fs]'
            axislab(3)='Error'
         else if (plottask .eq. 3) then
            plttitle='CMF error in spfs'
            axislab(1)='Time [fs]'
            axislab(3)='Error'
         endif
         if (lrebound) then
            call databnds(pldata(1,plottask),dimx,dimy,nsets,nplots,&
                    ndata,1,1,1)
         endif
         call dotask(task,plottask,plotdim,pltime,ydum,&
                 pldata(1,plottask),dimx,dimy,nsets,nplots,ndata,&
                 1,1,1,xdum,1,pltime,ndata,edum,1,zdum,1,1,1,1,idum)

         goto 250
      endif

      close(iaus,status='delete')


      stop

 666  continue
      write(6,*)
      write(6,'(a,/a)')&
         'ERROR in program rdupdate: Cannot open or read update file:'&
         , filein(1:laein)
      write(6,*)
      stop
  900 continue
      routine='Rdupdate'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop
 1000 write(ilog,'(a)') 'Error allocating memory in Rdupdate :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop

      end program rdupdate

