!***********************************************************************
!
!                         AUSWERT subroutine REFLEX
!
!     This routine calculates the reflexion and transmision of a CAP.
!
!     11/99 HDM
!
!***********************************************************************
      
      program reflex

use decimal, only:long,dop
use global
use keyunitsmod

      implicit none

      integer(long)       :: n,i,j,ierr,nz,id
      real(dop)           :: length,beta,mass,e1,e2,delta,eu,ga,eta,&
                             g1,g2,at,bt,ct,et,bk,c0,trans,ref,&
                             e1au, e2au

      real(dop), dimension(4)       :: rrtt
      complex(dop)                  :: alpha, ff, ee
      character(len=12)             :: unite
      logical(kind=4)               :: lgnu,lmult

      common /in/ alpha,beta,e1,e2,e1au,e2au,mass,length,eta,g1,g2

!-----------------------------------------------------------------------
! Call input
!-----------------------------------------------------------------------
      call reflinput(lgnu,lmult,unite,n,id)

!-----------------------------------------------------------------------
! Compute reflexion
!-----------------------------------------------------------------------
      if(lmult) then
         nz = 4
         write(22,'(2a)')'#    Energy           eta          2*eta ',&
                         '         4*eta           8*eta' 
      else
         nz = 1
         write(22,'(a)')'#    Energy       Transmission    Reflection'
      end if
      delta   = (log(e2) - log(e1)) / 300
      do i = 0, 300
         eu = exp( log(e1) + i*delta )
         et = eu
         call keyeunits(et,unite,ierr)
         bk = sqrt(2.0_dop*mass*et)
         bt = bk*length
         c0 = eta/( et * bk**n )
         do j = 1, nz
            ct = c0* 2**(j-1)
            at = ct * bt**n
         
!.....Transmission
            ga = at/( 1.0_dop + g1*at**2 + g2*at**4 )**0.125_dop
            trans = exp( -min(id*ga*bt/dble(n+1),230.2_dop) )

!.....Reflection
            ff = alpha*ct/(1.0_dop + alpha*ct)
            ee = exp( (0.0_dop,2.0_dop)*(beta/dcmplx(0.0_dop,-ct)**(1.0_dop/dble(n))&
                 + 0.25_dop*pi ) )
            ref = abs( (ff-ee)/(1.0_dop-ff*ee) )**2

            rrtt(j) = ref + trans + sqrt(ref*trans)
         end do
         if(lmult) then
            write(22,'(5e15.5)') eu,rrtt(1),rrtt(2),rrtt(3),rrtt(4)
         else
            write(22,'(4e15.5)') eu, trans, ref, rrtt(1)
         end if
      end do
         
      stop
      contains

!#######################################################################

      subroutine reflinput(lgnu,lmult,unite,n,id)

      implicit none

      integer(long), external        :: myiargc
      integer(long)                  :: iarg,laenge,i,ierr
      integer(long), intent(out)     :: n
      integer(long), intent(out)     :: id
      real(dop)                      :: e1, e2, e1au, e2au, g1, g2 , eta, length, mass, beta
      real(dop)                      :: mass_orig
      complex(dop)                   :: alpha
      logical(kind=4)                :: lgrid,lsearch
      logical(kind=4), intent(out)   :: lgnu,lmult
      character(len=80)              :: buf
      character(len=12)              :: unitm,unitm_orig,unite_orig 
      character(len=12), intent(out) :: unite
      character(len=240)             :: fileout

      common /in/ alpha,beta,e1,e2,e1au,e2au,mass,length,eta,g1,g2

      real(dop)            :: fbeta (1:6) = (/0.66666666667_dop,0.785398163_dop,0.841309_dop,&
                                             0.874019_dop,0.895522_dop,0.910744_dop/) 

!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! provide default input and output filenames 
!-----------------------------------------------------------------------
      fileout = 'reflex.pl'
      pi = 4.0_dop*atan(1.0_dop)

!-----------------------------------------------------------------------
! set default flags
!-----------------------------------------------------------------------
      lgnu  = .false.
      lmult = .false.
      lgrid = .false.
      lsearch = .false.
      unitm = 'au'
      unite = 'au'
      id = 1

!-----------------------------------------------------------------------
! read options 
! comment out or add options as required
!-----------------------------------------------------------------------
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:2) .eq. '-o' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf(1:2) .eq. '-g' ) then
            lgnu = .true.
         else if( buf(1:2) .eq. '-G' ) then
            lgrid = .true.
         else if( buf(1:2) .eq. '-b' ) then
            lmult = .true.
         else if( buf(1:2) .eq. '-d' ) then
            id = 2
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(80a1)')  ('-',i=1,80)
            write(6,'(1x,a,/1x,a,/1x,a)')&
       'Purpose: Computes reflection and transmission of a CAP.',&
       'Usage  : reflex [ -o -g -G -b -h -?] ',  &
       '         [order eta length mass unit_m e1 e2 unit_e] .'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,11x,a)') &
                     ' -o FILE : The output is written to file ',&
                     'FILE.pl rather than to ./reflex.pl',&
             'The string FILE may be a relative or a full path-name.'
             write(6,'(2a)')  ' -g      : GNUplot comand lines',&
                     ' are written to the output file(s).'
             write(6,'(a)')   ' -G      : Grid lines are shown.'
             write(6,'(2a)')  ' -b      : Plot (refl. + transm.)',&
                   " for four eta's.  (eta, 2*eta, 4*eta, 8*eta)."
            write(6,'(a,a)')  ' -d      : Transmission for DVR. ',&
                              '(i.e. squared) '
            write(6,'(a,/a)') ' -h      : Print this help text. ',&
                              ' -?      : Print this help text. '
            write(6,'(2a/2a/a/a/a/2a)')'The arguments order, eta ',&
             'and length define the CAP.','mass is the (reduced) ',&
             'mass of the particle in unit unit_m.',&
             'If no mass-unit is given, au is assumed.',&
             'e1 and e2 give the energy range in unit unit_e.',&
             'CAP parameters eta and length are in au.', &
             'If eta=0 or eta=?, then the program searches for an ',&
             'optimal eta.' 
            write(6,'(2a)') 'If not all arguments are given, ',&
                 'the user is prompted for the missing arguments.'  
            write(6,'(2a)') 'Example : reflex -g 3 0.04 1.8 ',&
                            '1.5 H-mass 0.3 15 eV '
            write(6,'(80a1)')  ('-',i=1,80)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      if (iarg .lt. i) goto 200

!-----------------------------------------------------------------------
! get any compulsory arguments if in argument list
!-----------------------------------------------------------------------
      read(buf,*,iostat=ierr) n 
      if (ierr .ne. 0 .or. n.le.0  .or. n.gt.6 ) then
       write(6,*)'The first argument (order)  must be 1,2,...,6. Not :'
       write(6,*) buf
       stop
      end if

      i=i+1
      if (iarg .lt. i) goto 201
      call mygetarg(i,buf)
      if( buf .eq. '?' .or.  buf .eq. '*' ) then
         lsearch = .true.
      else
         read(buf,*,iostat=ierr) eta
         if ( ierr.eq.0 .and. eta.eq.0.0_dop ) then
            lsearch = .true.
         else if (ierr .ne. 0 .or. eta.lt.0.0_dop) then
            write(6,*)'The second argument (eta)  must be positive ',&
                 'real number. Not :'
            write(6,*) buf
            stop
         end if
      end if

      i=i+1
      if (iarg .lt. i) goto 202
      call mygetarg(i,buf) 
      read(buf,*,iostat=ierr) length
      if (ierr .ne. 0 .or. length.le.0.0_dop) then
       write(6,*)'The third argument (lenght)  must be positive ',&
               'real number. Not :'
       write(6,*) buf
       stop
      end if

      i=i+1
      if (iarg .lt. i) goto 203
      call mygetarg(i,buf) 
      read(buf,*,iostat=ierr) mass
      if (ierr .ne. 0 .or. mass.le.0.0_dop) then
       write(6,*)'The forth argument (mass)  must be positive ',&
               'real number. Not :'
       write(6,*) buf
       stop
      end if

      i=i+1
      if (iarg .lt. i) goto 203
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) e1
!.....if there is no mass unit, assume au for mass-unit.
      if(ierr .eq. 0 ) goto 104
      unitm = buf

      i=i+1
      if (iarg .lt. i) goto 204
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) e1 
 104  if (ierr .ne. 0 .or. e1.le.0.0_dop) then
       write(6,*)'The sixth argument (e1)  must be positive ',&
               'real number. Not :'
       write(6,*) buf
       stop
      end if

      i=i+1
      if (iarg .lt. i) goto 204
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) e2
      if (ierr .ne. 0 .or. e1.le.0.0_dop) then
       write(6,*)'The seventh argument (e2)  must be positive ',&
               'real number. Not :'
       write(6,*) buf
       stop
      end if

      i=i+1
      if (iarg .lt. i) goto 205
      call mygetarg(i,buf)
      unite = buf
      goto 300

!-----------------------------------------------------------------------
! fill in any arguments not given on command line
!-----------------------------------------------------------------------
 200  continue
      write(6,'(a)') ' CAP order n =?'
      read(5,*) n
      if ( n.le.0  .or. n.gt.6 ) then
         write(6,*) 'Order n must be 0<n<7 !  n =',n
         stop
      end if  
 201  write(6,'(a)') ' CAP strength eta =?  (eta=0 -> search for eta)'
      read(5,*) eta
      if ( eta.eq.0.0_dop ) then
         lsearch = .true.
      else if ( eta.lt.0.0_dop ) then
         write(6,*) 'CAP strength eta must be > 0 !   eta =',eta
         stop
      end if
 202  write(6,'(a)') ' CAP length =?      (add half a grid spacing)'
      read(5,*) length
      if ( length.le.0 ) then
         write(6,*) 'CAP length must be > 0 !   length =', length
         stop
      end if
 203  write(6,'(a)') ' mass, mass-unit =?'
      read(5,*) mass, unitm
      if ( mass.le.0 ) then
         write(6,*) 'mass must be > 0 !   mass =',mass
         stop
      end if

 204  write(6,'(a)') ' E1, E2, energy-unit =?'
      read(5,*) e1,e2, unite
      if ( e1.le.0 .or. e2.le.e1 ) then
         write(6,*)'Energies must satisfy 0<E1<E2 !  E1, E2 =',E1,E2
         stop
      end if
      goto 300
 205  write(6,'(a)') ' energy-unit =?'
      read(5,*) unite

 300  continue

!-----------------------------------------------------------------------
! convert mass and energy to au
!-----------------------------------------------------------------------
      unitm_orig = unitm
      mass_orig  = mass
      call keylowc(unitm)
      call keymunits(mass,unitm,ierr)
      if(ierr .gt. 0) then
         write(6,*)' unknown mass-unit: ', unitm
         stop
      end if

      e1au = e1
      e2au = e2
      unite_orig = unite
      call keylowc(unite)
      call keyeunits(e1au,unite,ierr)
      call keyeunits(e2au,unite,ierr)
      if(ierr .gt. 0) then
         write(6,*)' unknown energy-unit: ', unite
         stop
      end if

!-----------------------------------------------------------------------
! Print parameters
!-----------------------------------------------------------------------
      write(6,'(/,80a1)')  ('-',i=1,80)
      write(6,'(a,i2)')   ' CAP order      n =', n
      if(id.eq.1) then
         write(6,'(a,f10.6,1pe12.3)') ' CAP strength eta =', eta, eta
      else
         write(6,'(a,f10.6,6x,a)') ' CAP strength eta =', eta,&
                               '( d option is set).'
      end if
      write(6,'(a,f8.4,a)') ' CAP length     L =', length,&
          '  (add half a grid spacing to the length of the CAP)' 
      if(mass_orig .le. 99.9_dop ) then
      write(6,'(a,f8.4,1x,a,13x,a,f12.2,a)') ' reduced mass   m =',&
                  mass_orig, unitm_orig, '(',mass,' au )'
      else
      write(6,'(a,f10.1,1x,a,11x,a,f12.2,a)') ' reduced mass   m =',&
                  mass_orig, unitm_orig, '(',mass,' au )'
      end if
      if(e2 .le. 999.9_dop ) then
         write(6,'(2a,f8.3,a,f8.3,1x,a,3x,a,2f9.4,a)') ' Energies ', &
         'E1 : E2 =', e1,' :', e2, unite,'(',e1au,e2au,' au )'
      else
         write(6,'(2a,f9.1,a,f9.1,1x,a,1x,a,2f9.4,a)') ' Energies ', &
         'E1 : E2 =', e1,' :', e2, unite,'(',e1au,e2au,' au )'
      end if
      write(6,'(80a1)')  ('-',i=1,80)


!-----------------------------------------------------------------------
! Initialize alpha, beta, g1, g2
!-----------------------------------------------------------------------
      g1   = dble(n+1)/dble(3*n+1)
      g2   = 0.0625_dop*(dble(n+2)/dble(2*n+2))**8
      beta = fbeta(n)
      alpha = (0.0_dop,-0.25_dop)
      do i = 1, n
         alpha = alpha*(0.0_dop,0.5_dop)*dble(i)
      end do

!-----------------------------------------------------------------------
! Determine eta, if lsearch=.true.
!-----------------------------------------------------------------------
      if(lsearch) then
         call findeta(n,id)
         if(eta .le. 99.99_dop .and. eta .gt. 0.00001) then
            write(6,'(a,f12.7)') ' Computed eta =', eta
         else
            write(6,'(a,e12.5)') ' Computed eta =', eta
         end if
         write(6,'(80a1)')  ('-',i=1,80)
         lsearch = .false.
      end if

!-----------------------------------------------------------------------
! convert filenames to absolute
!-----------------------------------------------------------------------
      laenge=index(fileout,' ')-1
      call abspath(fileout,laenge)
      if (fileout(laenge:laenge) .eq. '/') then 
         fileout = fileout(1:laenge)//'reflex.pl'
         laenge = laenge + 9
      else if (fileout(laenge-2:laenge) .ne. '.pl') then 
         fileout = fileout(1:laenge)//'.pl '
         laenge = laenge + 3
      end if

      open(22,file=fileout(1:laenge),form='formatted',&
           status='unknown')

!-----------------------------------------------------------------------
! Write GNUPLOT command lines.
!-----------------------------------------------------------------------

      if( lgnu ) then
         i = index(unite_orig,' ') - 1
         write(22,'(a)') 'set nolabel'
         write(22,'(a)') 'set tit "CAP Reflection and Tranmission"'
         if(lgrid) then
            write(22,'(a)') 'set grid'
         else
            write(22,'(a)') 'set nogrid'
         end if
         write(22,'(a)') 'set key' 
         if(lmult) then
            write(22,'(a,i2,a)') 'set lab "n =',n,&
                                 '" at graph 0.15,0.95'
            write(22,'(a,f6.2,a)') 'set label "L =',length,&
                                   '" at graph 0.25,0.95'
            write(22,'(a,f9.6,a,f9.6,a)') 'set label "eta =',&
                 eta,' --', 8*eta, '" at graph 0.40,0.95'
            write(22,'(a,i6,a)') 'set label "m =',nint(mass),&
                                 '" at graph 0.70,0.95'
            if(id.eq.2) write(22,'(a)')&
                   'set label "-d" at graph 0.84,0.95' 
         else
            write(22,'(a,i2,a)') 'set lab "n =',n,&
                                 '" at graph 0.2,0.95'
            write(22,'(a,f6.2,a)') 'set label "L =',length,&
                                   '" at graph 0.3,0.95'
            write(22,'(a,f9.6,a)') 'set label "eta =',eta,&
                 '" at graph 0.45,0.95'
            write(22,'(a,i6,a)') 'set label "m =',nint(mass),&
                                 '" at graph 0.64,0.95'
             if(id.eq.2) write(22,'(a)') &
                    'set label "-d" at graph 0.78,0.95' 
         end if
         write(22,'(a)') 'set style data lines' 
         write(22,'(a)') 'set logscale xy' 
         write(22,'(a,e14.7,a,e14.7,a)') 'set xr [',e1,':',e2,']' 
         write(22,'(3a)') 'set xlabel "Energy[',unite_orig(1:i),']"'
         if(lmult) then
            write(22,'(2a,a3,/a)') 'plot "',fileout(1:laenge),'" \\',&
         'us 1:2 title "eta","" us 1:3 title "2*eta", &
         "" us 1:4 title "4*eta","" us 1:5 title "8*eta"'
         else
            write(22,'(2a,a3,/a)') 'plot "',fileout(1:laenge),'" \\',&
                 'us 1:2 title "R","" us 1:3 title "T","" us 1:4 title "R+T"'
         end if
      end if

      return 
      end subroutine reflinput


!#######################################################################

      subroutine findeta(n,id)

      implicit none

      integer(long)             :: i,j
      integer(long), intent(in) :: n,id
      real(dop)                 :: e1, e2, e1au, e2au, mass, length, eta, beta, g1, g2
      real(dop)          :: a2,b2,c1,c2,ga,ff,ee,spn,ref,trans,vv,vz
      complex(dop)       :: alpha
      common /in/ alpha,beta,e1,e2,e1au,e2au,mass,length,eta,g1,g2
!-----------------------------------------------------------------------

         spn = 2.0_dop*beta*sin(pi/dble(2*n))
         b2  = sqrt(2.0_dop*mass*e2au)*length
         eta = 70.0_dop*dble(n+1)*e2au/(b2*length**n)
         c1  = eta/( e1au * sqrt(2.0_dop*mass*e1au)**n )
         c2  = eta/( e2au * sqrt(2.0_dop*mass*e2au)**n )
         a2  = c2 * b2**n

         vv = 0.5_dop
         vz = 1.0_dop

         do i = 1, 5
            do j = 1, 50
          
!.....Transmission
               ga = a2/( 1.0_dop + g1*a2**2 + g2*a2**4 )**0.125_dop
               trans = exp( -min(id*ga*b2/dble(n+1),230.2_dop) )
            
!.....Reflection
               ff  = abs(alpha*c1/(1.0_dop + alpha*c1))
               ee  = exp( - spn*c1**(-1.0_dop/dble(n)) )
               ref = ( (ff+ee)/(1.0_dop+ff*ee) )**2
            
               if( vz*(ref - trans) .le. 0.0_dop ) goto 100
               eta = eta * vv
               a2  = a2  * vv
               c1  = c1  * vv
            end do

            write(6,*) ' ######## NO CONVERGENCE ! ###############'
            return

 100        vv = vv**(-0.2_dop)
            vz = -vz
            eta = eta * vv
            a2  = a2  * vv
            c1  = c1  * vv
         end do

         return
         end subroutine findeta


      end program reflex


