!***********************************************************************
!
!                      AUSWERT subroutine PSI2RST
!
! Converts a MCTD restart file to a psi file
! HDM 06/22
!***********************************************************************

      program rst2psi

use decimal
use global
use globmemmod, only: agmat,mi,mijindx,mitrajst,midim
use aglobal
use versions
use zeigauswmod
use dvrdatmod
use griddatmod
use operdef
use psidef
use runpropmod
use datenmod
use channels
use iorst, only: rstinfo,rdrstpsi
use iofile, only: wrpsiinfo
use psioutmod

      implicit none

      integer(long) ::  m,ilbl,i,d,ll,n,s,chkdvr,chkgrd,chkpsi,chkdat
      integer(long), dimension(1) :: idum
      real(dop)     ::   time
      character(len=c5) ::  mdlbl, nspf
      logical       ::  linwf,lerr
      complex(dop),allocatable :: psi(:)
      complex(dop), dimension(1) :: cdum

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      logisopen=.false.
      lwrlog=.false.
      call rst2psiinput

!-----------------------------------------------------------------------
! get memory sizes
!-----------------------------------------------------------------------
      call rdmemdim(irst)

!-----------------------------------------------------------------------
! Allocate mememory
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      call alloc_psidef
      call alloc_runpropmod

!-----------------------------------------------------------------------
! read data from rst file
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=0
      call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkdat)
      if (lerr) then 
         write(6,'(a)') ' ERROR reading restart file'
         stop
      end if
      if (ldynspf) then
         write(6,'(a)') ' Cannot read a WF with dynamic SPFs'
         stop
      endif

!-----------------------------------------------------------------------
! write mode information.
!-----------------------------------------------------------------------
      write(6,'(/,a)') ' mode: No. of SPFs per state : modelabels'
      do m=1,nmode
      ll = 1
      mdlbl = '                                                       '
      nspf  = '                                                       '
         do  n=1,nspfdof(m)
            ilbl  = index(modelabel(spfdof(n,m)),' ')
            mdlbl = mdlbl(1:ll)//modelabel(spfdof(n,m))(1:ilbl)
            ll    = ll + ilbl
         end do
         write(nspf,'(95i3)') (dim(m,s),s=1,nstate)
         if(3*nstate+ll .le. 72 ) then
            write(6,'(i4,4a)') m,' :',nspf(1:3*nstate),' : ', &
                               mdlbl(1:ll)
         else
            write(6,'(i4,3a)') m,' :',nspf(1:3*nstate),' :'
            write(6,'(a,/)') mdlbl(1:ll)
         end if
      enddo

      write(6,'(/,a,f11.3,a)') ' tinit =', tinit/fs, 'fs'
      write(6,'(a,i9,a,f8.4)') ' dgldim =',dgldim, &
                   ' ,   filever =',filever(irst)
      if(lswap) stop

      call zeigausw
      call memausw
      allocate(mi(midim))
      allocate(agmat(agmatdim))
      allocate(psi(dgldim))

!-----------------------------------------------------------------------
! read restart
!-----------------------------------------------------------------------
      if (filever(irst) .lt. 11.0_dop) then
         read (irst,err=999) time
         read (irst,err=999) (psi(d),d=1,dgldim)
         close(irst)
      else
         call rdrstpsi(time,psi,mi(mijindx),agmat,mi(mitrajst))
      endif

      write(6,'(/a,f8.3)')   ' Writing wavefunction at time : ',time/fs
      tinit=time
      out2=fs

!.....Opening the psi file.
      if (lovwr) then
         open (ipsi,file=fileout(1:laeout),form='unformatted', &
                 status='unknown',err=1000)
      else
         open (ipsi,file=fileout(1:laeout),form='unformatted', &
                 status='new',err=1010)
      endif
      write(6,'(2a)') ' The outputfile is : ',fileout(1:laeout)

      write(ipsi) filever(irst)
      call wrpsiinfo(ipsi)
      lpsiopt(:)=.false.
      call psiout(psi,cdum,time,mi(mijindx),agmat,mi(mitrajst),idum)
      close(ipsi)

      write(6,'(80a)') ('-',i=1,80)
      stop

  998 continue
      write(6,'(a)') 'Error reading rdrstinfo'
      stop 1
  999 write(6,*) 'Error reading restart file'
      stop
 1000 write(6,*)' Error opening file : '
      write(6,*) fileout(1:laeout)
      stop
 1010 write(6,*)' File already exists (use -w): '
      write(6,*) fileout(1:laeout)
      stop

      contains

!#######################################################################
!
! RST2PSIINPUT
!
! get input parameters and file names for calculation
!
!-----------------------------------------------------------------------
      subroutine rst2psiinput

      implicit none

      integer(long)     :: iarg, myiargc, i
      character(len=80) :: buf
      character(len=c5) :: filename
!-----------------------------------------------------------------------

      filein  = 'restart'
      fileout = 'psi'
      lswap=.false.
      iarg = myiargc()
      if (iarg .lt. 1) goto 200

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if(i.eq.1  .and. buf.eq.'0') then
         lswap=.true.
         goto 100
      endif
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:3) .eq. '-f ') then
            i = i+1
            call mygetarg(i,buf)
            laein=index(buf,' ')-1
            filein = buf(1:laein)
         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf(1:3) .eq. '-w ' ) then
            lovwr=.true.
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,a)') &
                 'Purpose: Converts a restart-file to psi-file format'
            write(6,'(/1x,a)') &
       'Usage  : rst2psi [-f -o -w  -ver -h -?]  step'
            write(6,'(/,a)') 'Options : ' 
            write(6,'(2a,/10x,a)') &
                     ' -f FILE: The restart is read ', &
                     'from file FILE rather than from ./restart', &
              'The string FILE may be a relative or a full path-name.'  
            write(6,'(2a)') &
                     ' -o DIR : The output is written to file FILE', &
                     ' rather than to ./psi'
            write(6,'(2a)') &
                     ' -w     : An existing output file is ', &
                     'overwritten.'
            write(6,'(a)') &
                     ' -ver   : Version information about the program '
            write(6,'(a,/a)') &
                     ' -h     : Print this help text. ', &
                     ' -?     : Print this help text. '

            write(6,'(5x,2a)') 'To receive some information ', &
                 'on the restart file, run "rst2psi 0 "'
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if

!-----------------------------------------------------------------------
! Making path-names absolute
!-----------------------------------------------------------------------
 200  laein=index(filein,' ')-1
      if (filein(laein:laein) .eq. '/') then
         filein = filein(1:laein)//'restart'
         laein = laein + 8
      end if
      call abspath(filein,laein)

      laeout=index(fileout,' ')-1
      call abspath(fileout,laeout)

!-----------------------------------------------------------------------
!  Opening the restart file.
!-----------------------------------------------------------------------
      write(6,'(80a)') ('*',i=1,80)
      filename=filein(1:laein)
      open(irst,file=filename,form='unformatted',status='old',err=1000)
      write(6,'(2a)') ' The inputfile is : ',filein(1:laein)

      return

 1000 write(6,*)' Error opening file : '
      write(6,*) filename
      stop

      end subroutine rst2psiinput

      end program rst2psi
