!***********************************************************************
!
!              Analyse program  SHOWD1D
!
!     11/99 GW, 03/00 HDM
!
!
!***********************************************************************
      
      module showd1dmod

use decimal, only: long,dop,sip

      integer(long), save :: dof,state,sel,pltpnt
      real(dop), save     :: factor,w2

      end module showd1dmod

!***********************************************************************

      program showd1d

use decimal, only: long,dop,sip
use global
use globmemmod
use aglobal
use versions
use dvrdatmod
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use showd1dmod
use plot

use op1lib, only: zerovxl
use logdat
use iofile, only: rdrhoinfo
use rddvrmod, only: dvrinfo
use rdopermod, only: operinfo
use zeigauswmod, only: zeigausw

use module_interface

      implicit none

      integer(long)                     :: chkdvr,chkgrd,chkpsi,chkdat,ilbl,ilgpel
      character(len=c5)                 :: filename
      character(len=10)                 :: stat
      logical(kind=4), external         :: int2log
      logical(kind=4)                   :: lerr,llgrthm

!-----------------------------------------------------------------------
! Initialize variables and set defaults
!-----------------------------------------------------------------------
      call default
      call adefault

!-----------------------------------------------------------------------
! Declare files to be read
! set flag to .true. if file is to be read
!
!     lrddvr    DVR file
!     lrdoper   OPER file
!-----------------------------------------------------------------------
      lrddvr=.true.
      lrdoper=.false.

!-----------------------------------------------------------------------
! Declare resources required (other than those needed to read a file)
!
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! llogrthm : logarithmic plot
!-----------------------------------------------------------------------
      ncomp=0
      lpsi=.false.
      lpsi1=.false.
      lpsigrd=.false.
      ldmat=.false.
      ldmat1=.false.
      lort=.true.
      lgpop=.true.
      ltrafo=.true.

!-----------------------------------------------------------------------
! read input parameters and options
! open files
!-----------------------------------------------------------------------
      call den1dinp(stat,llgrthm)

!-----------------------------------------------------------------------
! open input and output data files
!
! use standard channels if possible
!  irho : gridpop file
!  iaus : output file
!  ilog : log-file showd1d.log
!-----------------------------------------------------------------------
      if (fileout .ne. 'no') then
         filename=dirout(1:laeoutd)//'/showd1d.log'
         open (unit=ilog,file=filename,form='formatted')
         logisopen=.true.
      else
         ilog=6
      endif
      filename=filein
      open(irho,file=filename,form='unformatted',status='old',err=1000)
      if (.not. linter) then
         if (fileout .ne. 'no') then
            filename=fileout
            open(iaus,file=filename,form='formatted',&
                 status=stat,err=1000)
         else
            iaus=6
         endif
         
      else
         filename=name(1:laenge)//'/tmpplot.pl'
         open(iaus,file=filename,status='unknown',err=1000)

      endif

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(irho)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef

!-----------------------------------------------------------------------
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
! First read file version number
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      filename=filein
      rewind(irho)
      read(irho,err=999) filever(irho), ilgpel
      lgpel = int2log(ilgpel)
      call rdrhoinfo(irho,chkdvr,chkgrd,chkdat)

!-----------------------------------------------------------------------
! read information from system files
!-----------------------------------------------------------------------
      if (lrddvr) then
         chkdvr=2
         filename=dname(1:dlaenge)//'/dvr'
         open(idvr,file=filename,form='unformatted',status='old',&
              err=1000)
         call dvrinfo(lerr,chkdvr)
         close(idvr)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif
      if (lrdoper) then
         chkdvr=2
         chkgrd=2
         filename=oname(1:olaenge)//'/oper'
         open(ioper,file=filename,form='unformatted',status='old',&
              err=1000)
         call operinfo(lerr,chkdvr,chkgrd)
         close(ioper)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif

!-----------------------------------------------------------------------
! check input
!-----------------------------------------------------------------------
      if( state .gt. 1 .and. .not.lgpel ) then
         write(6,'(2a,/,2a,/2a)') 'You must set "gridpop=el" in the ',&
         'run-section to obtain separate information',&
         'for each electronic state. This gridpop file contains only ', &
         'grid populations','summed over all electronic states. ',&
         'Thus only s1 is allowed.'
         stop
      endif
      if( state .gt. nstate) then
         write(6,'(a,i3,a,i3)') 'The required state s =',state, &
         ' must not be larger than nstate = ', nddstate   
         stop
      endif
      if(basis(dof).eq.0 .and.(dof.ne.feb.or.lmult.or. .not.leb)) then 
         write(6,'(2a,i3)') 'There is no nuclear degree of', &
         ' freedom with number dof =', dof   
         stop
      endif
      if(.not.lgpel) nstate=1

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! write gnuplot info to file
!-----------------------------------------------------------------------
      if (lgnu .and. .not. linter) then
         ilbl=index(filein,' ')-9
         if(nstate.eq.1) then
            if(sel.eq.1 .and. .not.lnw) then
               write(iaus,'(a,3x,2a)') 'set title "1D Density',&
                                       filein(1:ilbl),'"'
            elseif(sel.eq.1 .and. lnw) then
               write(iaus,'(a,3x,2a)') 'set title "nw  1D Density',&
                                       filein(1:ilbl),'"'
            elseif(sel.eq.2 .and. .not.lnw) then
               write(iaus,'(a,3x,2a)') 'set title "1D Density (pop2)',&
                                        filein(1:ilbl),'"'
            elseif(sel.eq.2 .and. lnw) then
               write(iaus,'(a,3x,2a)') &
               'set title "nw  1D Density (pop2)', filein(1:ilbl),'"'
            endif
         else
            if(sel.eq.1 .and. .not.lnw) then 
              write(iaus,'(2a,i2,3x,2a)') 'set title "1D Density',&
                                   '  s =',state,filein(1:ilbl),'"'
            elseif(sel.eq.1 .and. lnw) then
               write(iaus,'(2a,i2,3x,2a)') 'set title "nw  1D Density',&
                                   '  s =',state,filein(1:ilbl),'"'
            elseif(sel.eq.2 .and. .not.lnw) then
               write(iaus,'(2a,i2,3x,2a)') &
                                   'set title "1D Density (pop2)',&
                                   '  s =',state,filein(1:ilbl),'"'
            elseif(sel.eq.2 .and. lnw) then
               write(iaus,'(2a,i2,3x,2a)') &
                                   'set title "nw  1D Density (pop2)',&
                                   '  s =',state,filein(1:ilbl),'"'
            endif
         endif
         write(iaus,'(a)') 'set style data lines'
         if(llgrthm) then
            write(iaus,'(a)') 'set logscale y'
         else
            write(iaus,'(a)') 'set nologscale'
         endif
         if(lpgrid) write(iaus,'(a)') 'set grid'
         if (outform .eq. 0 .or. outform .ge. 2) then
            if( xmax .gt. xmin ) then
               write(iaus,'(a,f8.3,a,f8.3,a)') &
                  'set xrange [',xmin,':',xmax,']'
            endif
            if( ymax .le. 0.0_dop ) then
               if(llgrthm) then
                  write(iaus,'(a)') 'set yrange [1e-8:*]'
               else
                  write(iaus,'(a)') 'set yrange [0:*]'
               endif
            else
               if(llgrthm) then
                  write(iaus,'(a,e11.4,a)') 'set yrange [1e-8:',ymax,']'
               else
                  write(iaus,'(a,e11.4,a)') 'set yrange [0:',ymax,']'
               endif
            end if
            write(iaus,'(a)') '#  '

         else if (outform .eq. 1) then  ! -T "3D time file"
            write(iaus,'(a)') 'set surface'
            write(iaus,'(a)') 'set contour'
            write(iaus,'(a)') 'set cntrparam levels auto'
            write(iaus,'(a)') '#set view 0,0,1,1'
            write(iaus,'(a)') 'set view 60.0,30.0,1,1'
            write(iaus,'(3a)') 'set xlabel "',modelabel(dof)(1:9),'"'
            write(iaus,'(a)') 'set ylabel "Time"'
            write(iaus,'(a)') 'set zlabel ""'
            write(iaus,'(a)') 'set origin 0.,0.'
            write(iaus,'(a)') 'set size 1.0, 1.0'
            if(lsmooth) then
               write(iaus,'(a)')  'set cntrparam cubicspline'
               write(iaus,'(a)')  'set cntrparam points 5'
            else
               write(iaus,'(a)')  'set cntrparam linear'
            end if
            if (sel.eq.1) then
               write(iaus,'(a)') 'splot "-" using 1:2:3 ; pause -1'
            else if (sel.eq.2) then
               write(iaus,'(a)') 'splot "-" using 1:2:4 ; pause -1'
            endif

         endif
         write(iaus,'(a)') '#  '
         if(outform .eq. 0) write(6,'(a)') &       
           'Press RETURN to see the next plot'  ! -S "Step through" (default)
      end if

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! assign work arrays
! NB work arrays must be at end of mc, mr, mi, ml arrays.
!-----------------------------------------------------------------------
      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
! call analysis routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      write(ilog,'(/,2a)') 'gridpop : ', filein(1:laein)
      write(ilog,'(2a)')   'output  : ', fileout(1:laeout)
      write(ilog,'(2a,/)') 'dname   : ', dname(1:dlaenge)
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

      call subden1d(mr(mrort),mr(mrtrafo),mr(mrweight))

deallocate(mc,mr,mi,ml,ms,mf)

      stop
  999 continue
      message= 'Error reading :'//trim(filename)
      write(6,'(a)') message
      stop
 1000 continue
      routine='showd1d'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//trim(filename)
      if(filename(1:ilbl) .eq. fileout(1:ilbl))&
      write(6,'(/a)') '###  Use -w option to overwrite! ###' 
      call errormsg
      stop
      contains

!***********************************************************************
!
!     HERE IS THE SUBROUTINE TO DO THE ANALYSIS
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! provide dummy arguments of arrays needed for analysis
!  e.g. psi to match argument mc(mcpsi)
!***********************************************************************

      subroutine subden1d (ort,trafo,weight)

use global
use aglobal
use maxv, only: maxdim
use griddatmod
use channels
use datenmod
use plot
use showd1dmod

use rdfilesmod, only: rdgridpop
use plotutils, only: dotask,plotmenu,inmodebnds,plotdefs,initplot,timebnds,databnds
use rddvrmod

      implicit none

      integer(long)                     :: chkdvr,chkpsi,chkgrd,chkdat,ilgpel
      integer(long), dimension(maxdim)  :: idum
      complex(dop), dimension(1)        :: zdum

      integer(long), external           :: slen
      integer(long)                     :: g,f,n,s,sc,ggdim,gdim1,i,s1,p1,&
                                           kmin,kmax,kmin2,dk,dm,ii,&
                                           sphtyp,jmax,j0,dj,gj,gm,mmax,mmin,&
                                           mincr,ibleg,mmmax,mmmin,mmmin2,p2
      character(len=6)                  :: fslab
      character(len=c5)                 :: filename
      character(len=64)                 :: ccc
      logical(kind=4)                   :: lexist
      logical(kind=4), external         :: int2log

      real(sip), dimension(2*maxgdim)                  :: fgpop
      real(dop), dimension(ortdim,nddstate*npacktot)   :: gpop1,gpop2
      real(dop), dimension(ortdim), intent(out)        :: ort
      real(dop), dimension(dvrdim), intent(out)        :: trafo
      real(dop), dimension(maxgdim), intent(out)       :: weight
      real(dop), dimension(1)                          :: edum,rdum
      real(dop)                                        :: time,dp,sumd

! standard variables for interactive plots
      integer(long)                 :: ndata,ntdata,task,plottask,f1,f2,ydim,&
                                       nsets,nplots,plotdim
      real(dop), dimension(1)                                    :: y 
      real(dop), dimension(:), allocatable     :: pltime
      real(dop), dimension(:,:,:), allocatable :: pldata
      integer(long)     :: ierr
      character(len=10) :: cerr

!-----------------------------------------------------------------------
! zero dummy variables
!-----------------------------------------------------------------------
      y(1)=0.0
      fslab = ' fs,  '
      w2=factor

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! read data from dvr file if desired
! edit which data (set dvrdata(x) to .true. and substitute duumy array)
!-----------------------------------------------------------------------
      if (lrddvr) then
         dvrdata(:) = .false.
         dvrdata(1)=.true.
         dvrdata(2)=.true.
         dvrdata(4)=.false.
         dvrdata(5)=.false.
         dvrdata(6)=.false.
         dvrdata(7)=.false.
         dvrdata(8)=.false.
         dvrdata(9)=.false.
         dvrdata(11)=.false.
         dvrdata(12)=.false. 
         chkdvr=1
         call rddvr(ort,trafo,rdum,rdum,zdum,&
              zdum,idum,zdum,zdum,idum,idum,rdum,chkdvr)
      endif

! --- Check for consistency
      if(outform.eq.5) then  ! -P "Point file"
         if(pltpnt.le.0 .or. pltpnt.gt.gdim(dof)) then
            routine='showd1d'
            message='The number following -P must be an integer'// &
                    ' .ge.1 and .le. gdim'
            call errormsg
         endif
      endif

!-----------------------------------------------------------------------
! get no. of data sets
!-----------------------------------------------------------------------
      ndata=0
      rewind(irho)
      do 
         call rdgridpop(irho,fgpop,gpop1,gpop2)
         if (lend) exit
         ndata = ndata+1
      enddo

! no. of data sets to be plotted
      ndata = (ndata-1)/step + 1
      write(6,'(a,i5)') 'No. datasets found for plotting :',ndata
      maxdata = ndata
! allocate data arrays
      cerr = 'pltime'
      allocate(pltime(maxdata),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'pldata'
      allocate(pldata(maxgdim,maxdata,nddstate+1),stat=ierr)
      if (ierr > 0) go to 1000

!-----------------------------------------------------------------------
! initialise interactive plotting
!-----------------------------------------------------------------------
      if (linter) then

         nplottask=2
         call alloc_plot

         call plotdefs
         lshowd1d=.true.
         plotdim=2
         lplcootim=.true.
         lstate=.true.

         filename=name(1:laenge)//'/tmpplot.pl'
         call initplot(filename,plotdim)

         lenunit=1.0
         ev=1.0

!-----------------------------------------------------------------------
! set program specific data, e.g. plot tasks
!-----------------------------------------------------------------------
         plottask=1
         if (sel .eq. 1) then
            cplottask(1)='3D grid populations'
            cplottask(2)='time-slices of grid populations'
         else if (sel .eq. 2) then
            cplottask(1)='3D FBR / Mom grid populations'
            cplottask(2)='time-slices of FBR / Mom grid populations'
         endif
         do i=1,nplottask
            cptlen(i)=slen(cplottask(i))
         enddo

         lsrf=.true.
         lcntr=.false.

!-----------------------------------------------------------------------
! calculate initial mode bounds
!-----------------------------------------------------------------------
         call inmodebnds(ort,plotdim)
      endif

!-----------------------------------------------------------------------
! call menu
!-----------------------------------------------------------------------
  200 continue
      if (linter) then
! Turn on possible summing of states (default to on)
         if (nplstates .gt. 1) then
            lplsumst = .true.
         endif

         call plotmenu(task,plottask,plotdim)

         if (.not. lrecalc) go to 250
      
         ndata=0
         ntdata=0
         dof=axis(1)

         if (plottask .eq. 1) then
            plotdim=2
            ltslice=.false.
         else if (plottask .eq. 2) then
            plotdim=1
            ltslice=.true.
         endif
      endif

      s=max(1,state)
      f=dof
      ggdim = gdim(f)
      gdim1 = gdim(f)
      if(basis(f) .eq. 12 .or. basis(f) .eq. 20 .or. basis(f) .eq. 21)&
                  ggdim = ipbaspar(3,f)

!-----------------------------------------------------------------------
! Special handeling for sphFBR
!-----------------------------------------------------------------------
      if( basis(dof) .eq. 6 .or. basis(dof) .eq. -1 ) then
         if(basis(dof) .eq. -1 ) f = dof-1
         sphtyp = ipbaspar(1,f)
         jmax   = ipbaspar(2,f)
         gj     = ipbaspar(4,f)
         mmax   = ipbaspar(1,f+1)
         mincr  = ipbaspar(2,f+1)
         mmin   = ipbaspar(3,f+1)
         gm = (mmax-mmin)/mincr + 1
         dj = 1
         if( sphtyp.eq.2  .or. sphtyp.eq.3 ) dj = 2 
         j0 = mod(jmax,dj)
         if( sel .eq. 2) then
            write(6,*) '-pop2 not allowed for shpFBR/phiFBR'
            stop
         end if   
         if(basis(dof) .eq. -1 ) then
            gdim1 = gm
            ggdim = gm
            do g = 1, gdim1
               ort(zort(f)-1+g) = mmin + (g-1)*mincr
            end do
         else
            gdim1 = gj
            ggdim = gj
            do g = 1, gdim1
               ort(zort(f)-1+g) = j0 + (g-1)*dj
            end do
         end if
      end if

!-----------------------------------------------------------------------
! Remove the fs label if nofs is set.
!-----------------------------------------------------------------------
      if( fs .lt. 2.0_dop ) fslab = '      '

!-----------------------------------------------------------------------
! Determine the DVR weights.
!...and trafo(gdim,gdim,kdim,mmdim) for Wigner
!-----------------------------------------------------------------------
!...Determine the appropriate 'column' of trafo(gdim,gdim,kdim) for KLeg/PLeg
      p1=0
      if( basis(f) .eq. 7 .or. basis(f) .eq. 20) then ! KLeg
         kmin=ipbaspar(1,f+1)
         kmax=ipbaspar(2,f+1)
         kmin2=ipbaspar(3,f+1)
         p1=(min(kmax,kmin2)-kmin)
         dk=max(ipbaspar(5,f+1),1)
         p1=(p1+dk-1)/dk
!k         p1=p1*gdim(f)**2
         p1=p1*ggdim**2
      elseif( basis(f) .eq. 9 .or. basis(f) .eq. 21) then           ! PLeg

         kmin=ipbaspar(1,f+1)
         kmax=ipbaspar(2,f+1)
         kmin2=ipbaspar(4,f+1)
!k         p1=(min(kmax,kmin2)-kmin)*gdim(f)**2
         p1=(min(kmax,kmin2)-kmin)*ggdim**2
      elseif( basis(f) .eq. 33 ) then          ! Wigner
         kmin=ipbaspar(2,f)
         kmax=ipbaspar(3,f)
         kmin2=ipbaspar(5,f)
         p2=min(kmax,kmin2)-kmin
         if (basis(f+1).eq.8) then
            dk=ipbaspar(5,f+1)
            p2=(p2+dk-1)/dk
         endif
         p2=p2*gdim(f)**2
         mmmin=ipbaspar(6,f)
         mmmax=ipbaspar(7,f)
         mmmin2=ipbaspar(9,f)
         p1=min(mmmax,mmmin2)-mmmin
         if (basis(f+2).eq.8) then
            dm=ipbaspar(5,f+2)
            p1=(p1+dm-1)/dm
         endif
         p1=p1*ipbaspar(4,f)*gdim(f)**2 + p2
      endif

      if (f .eq. febcont) then
         weight(:)=1.0_dop
      else
         call dvrweights(trafo(zdvr(f)+p1),ort(zort(f)),gdim1,ggdim,&
                         weight,basis(f),rpbaspar(1,f),ipbaspar(1,f),&
                         ipbaspar(1,f+1),.true.,lnw)
      endif

      if( abs(factor-1.0_dop) .gt. 1.0e-12_dop ) then
         do g = 1, gdim1
            weight(g) = weight(g)/factor
         enddo
      endif

!-----------------------------------------------------------------------
! Set x-label and re-set ort for -pop2 (sel=2)
!-----------------------------------------------------------------------
      if( sel .eq. 2 ) then
! -  HO  -
         if( basis(f) .eq. 1  ) then
            do g = 1, gdim1
               ort(zort(f)-1+g) = g-1
            end do
! -  Leg  Leg/R  Kleg Kleg/R -
         elseif( basis(f) .eq. 2 .or. basis(f) .eq. 7  &
            .or. basis(f) .eq. 9 .or. basis(f) .eq. 12 &
            .or. basis(f) .eq. 20 .or. basis(f) .eq. 21 &
            .or. basis(f) .eq. 33) then
            if(basis(f).eq.2 .or. basis(f).eq.12) ibleg=ipbaspar(2,f)
            if(basis(f).eq.7 .or. basis(f).eq.9 .or. basis(f) .eq. 20 & 
            .or. basis(f) .eq. 21 .or. basis(f) .eq. 33) &
                      ibleg=ipbaspar(1,f)
            if(ibleg .eq. 0 ) then
               do g = 1, ggdim
                  ort(zort(f)-1+g) = g-1
               end do
            else
               do g = 1, ggdim
                  ort(zort(f)-1+g) = g*2 - ibleg
               end do
            endif
! -  FFT -
         elseif( basis(f) .eq. 4  ) then
            dp = 2.0_dop*pi/( (ort(zort(f)+1)-ort(zort(f)))*gdim1 )
            do g = 1, gdim1/2
               ort(zort(f)-1+g) = (g-1)*dp
            end do
            do g = gdim1/2 + 1, gdim1
               ort(zort(f)-1+g) = (g-gdim1-1)*dp
            end do
            if(.not.lnw) w2=factor/dp
! -  exp -
         elseif( basis(f) .eq. 5  ) then
            dp = 2.0_dop*pi/( (ort(zort(f)+1)-ort(zort(f)))*gdim1 )
            do g = 1, gdim1
               ort(zort(f)-1+g) = dp*(g-gdim1/2-1)
            end do
            if(.not.lnw) w2=factor/dp
         else
            do g = 1, gdim1
               ort(zort(f)-1+g) = g
            end do
         end if

!-----------------------------------------------------------------------
! re-calculate initial mode bounds
!-----------------------------------------------------------------------
         if (lrebound) call inmodebnds(ort,plotdim)
         if (plottask .eq. 1) then
            plotdim = 2
         else if (plottask .eq. 2) then
            plotdim = 1
         endif
      end if

!-----------------------------------------------------------------------
! file header for non-interactive plotting
!-----------------------------------------------------------------------
      if (.not. linter) then
          call setxlab(axislab(1),sel,basis(dof))
          write(iaus,'(3a)') 'set xlabel "',&
                  axislab(1)(1:slen(axislab(1))),'"'
      
         if(lsmooth .and. sel .eq. 1) then
            ccc = ' using 1:2 smooth csplines  notitle ; pause -1'
         else if(lsmooth .and. sel .eq. 2) then
            ccc = ' using 1:3 smooth csplines  notitle ; pause -1'
         elseif(lstick .and. sel .eq. 1) then
            ccc = ' using 1:2 with impulses  notitle ; pause -1'
         else if(lstick .and. sel .eq. 2) then
            ccc = ' using 1:3 with impulses  notitle ; pause -1'
         else if(.not.lsmooth .and. sel .eq. 1) then
            ccc = ' using 1:2 notitle ; pause -1'
         else if(.not.lsmooth .and. sel .eq. 2) then
            ccc = ' using 1:3 notitle ; pause -1'
         end if
         if(outform .eq. 6) then  ! -E single-set state pops.
            i=0
            ii=max(3,laeout-15)
  34        i=ii
            ii=ii+index(fileout(i:laeout),'/')
            if(index(fileout(ii:laeout),'/') .ne. 0) goto 34
            ccc= '"'//fileout(ii:laeout)//'" using 1:'
         endif
         sc = slen(ccc)

         if (outform .eq. 3) then            ! -I "Index file"
            time = tinit/fs + indx*step*out1/fs
            write(iaus,'(a)') 'set nolabel'
            write(iaus,'(a,f6.1,3a)') 'set label "',time,fslab,&
               modelabel(dof)(1:12),'" at graph 0.01,0.95'
            write(iaus,'(a,i5,a)') 'plot "-"  index ',indx,ccc(1:sc)
         else if (outform .eq. 4) then       ! -Ma "Mathematica file"
            write(iaus,*) out2/fs
            write(iaus,*) gdim(dof)
            do g=1,gdim(dof)
               write(iaus,*) ort(zort(dof)-1+g)
            enddo
         else if (outform .eq. 5) then      ! -P "Point file"
            write(iaus,'(3a)') 'set xlabel "Time[fs]"'
            write(iaus,'(a)') 'set nolabel'
            if(sel .eq. 1 ) then
               write(iaus,'(a,i4,a,f7.2,a,2x,2a)')&
                    'set label "pltpnt=', pltpnt,&
                    ',  ort=',ort(zort(dof)-1+pltpnt),&
                    ',',modelabel(dof)(1:12),'" at graph 0.01,0.95'
            else if(basis(f).eq.4 .or. basis(f).eq.5) then
               write(iaus,'(a,i4,a,f7.2,a,2x,2a)')&
                    'set label "pltpnt=', pltpnt,&
                    ',  momentum=',ort(zort(dof)-1+pltpnt),&
                    ',',modelabel(dof)(1:12),'" at graph 0.01,0.95'
            else
               write(iaus,'(a,i4,a,2x,2a)')&
                    'set label "pltpnt=', pltpnt,&
                    ',',modelabel(dof)(1:12),'" at graph 0.01,0.95'
            endif
            write(iaus,'(2a)') 'plot "-" ',ccc(1:sc-11)

         else if (outform .eq. 6) then   ! -E single-set state pops.
            write(iaus,'(3a)') 'set xlabel "Time[fs]"'
            write(iaus,'(a)') 'set nolabel'
            write(iaus,'(a,2a)') &
             'set label "',modelabel(dof)(1:12),'" at graph 0.02,0.95'
            write(iaus,'(3a)') 'plot ',ccc(1:sc), '2 , \'
            do i = 3, gdim(f)
               if(i.le.9) then
                  write(iaus,'(a,i1,a)') ccc(1:sc),i,' , \'
               else
                  write(iaus,'(a,i2,a)') ccc(1:sc),i,' , \'
               endif
            enddo
            i=gdim(f)+1
            if(i.le.9) then
               write(iaus,'(a,i1,a)') ccc(1:sc),i,' ; pause -1'
            else
               write(iaus,'(a,i2,a)') ccc(1:sc),i,' ; pause -1'
            endif
         endif

      endif

!-----------------------------------------------------------------------
! now read data
!-----------------------------------------------------------------------
      ndata=0
      time=tinit/fs
      rewind(irho)
      chkdvr=0
      chkgrd=0
      chkpsi=0
      chkdat=0
      filename='gridpop'
      rewind(irho)
      read(irho) filever(irho), ilgpel
      lgpel = int2log(ilgpel)
      if (filever(irho) .lt. 11.0_dop) call rdrhoinfo(irho,chkdvr,chkgrd,chkdat)

      do n=1,nskip  ! skip the first nskip densities
         call rdgridpop(irho,fgpop,gpop1,gpop2)
         time=time+out1/fs
      enddo

   10 continue
      call rdgridpop(irho,fgpop,gpop1,gpop2)
      if (lend) go to 77

!-----------------------------------------------------------------------
! If state=0, then sum overall states.
!-----------------------------------------------------------------------
      if( state .eq. 0 ) then
         do s = 2, nddstate
            do g = 1, ggdim
               gpop1(zort(f)-1+g,1) = gpop1(zort(f)-1+g,1) +&
                                      gpop1(zort(f)-1+g,s)
               gpop2(zort(f)-1+g,1) = gpop2(zort(f)-1+g,1) +&
                                      gpop2(zort(f)-1+g,s)
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! Special handeling for sphFBR
!-----------------------------------------------------------------------
      s=max(1,state)
      if(basis(dof) .eq. 6 ) then
         do g = 1, gdim1
            gpop1(zort(f)-1+g,s)=gpop2(zort(f)-1+g,s)
         end do
      elseif(basis(dof) .eq. -1 ) then
         do g = 1, gdim1
            gpop1(zort(f)-1+g,s)=gpop2(zort(f)-1+gj+g,s)
         end do
      end if
      
      if( ggdim.gt.gdim1 ) then
         do g = gdim1+1, ggdim
            gpop1(zort(f)-1+g,s) = gpop1(zort(f)-1+gdim1,s)            
            weight(g) = weight(gdim1)
            if(sel.eq.1) ort(zort(f)-1+g) = ort(zort(f)-1+gdim1)
         end do
      end if

!-----------------------------------------------------------------------
! Re-normalize data, if required
!-----------------------------------------------------------------------
      if(lrenorm) then
         sumd=1.0d-25 ! to avoid devision by zero
         do g=1,ggdim
            sumd=sumd+gpop1(zort(f)-1+g,s)
         enddo
         do g=1,ggdim
            gpop1(zort(f)-1+g,s) = gpop1(zort(f)-1+g,s)/sumd
            gpop2(zort(f)-1+g,s) = gpop2(zort(f)-1+g,s)/sumd
         enddo
      endif

!-----------------------------------------------------------------------
! write out or store data
!-----------------------------------------------------------------------
      s=max(1,state)
      ndata=ndata+1
      ntdata=ntdata+1
      if (linter) then
         pltime(ndata)=time
         if (sel .eq. 1) then
            do s1=1,nplstates
               s=plstate(s1)
               do g=1,gdim(f)
                  pldata(g,ndata,s1)=gpop1(zort(f)-1+g,s)/weight(g)
               enddo
            enddo
         else if (sel .eq. 2) then
            do s1=1,nplstates
               s=plstate(s1)
               do g=1,gdim(f)
                  pldata(g,ndata,s1)=gpop2(zort(f)-1+g,s)*w2
               enddo
            enddo
         endif

! sum states and store in nplstates+1
         if (lsumst) then
            s1=nplstates+1
            pldata(:,ndata,s1)=0.0_dop
            do s=1,nplstates
               do g=1,gdim(f)
                  pldata(g,ndata,s1)=pldata(g,ndata,s1)+pldata(g,ndata,s)
               enddo
            enddo
         endif
      else if (outform .eq. 0) then
         write(iaus,'(a)') 'set nolabel'
         if(time.lt.99.9_dop) then
            write(iaus,'(a,f7.2,3a)') 'set label "',time,fslab,&
               modelabel(dof)(1:12),'" at graph 0.01,0.95'
         elseif(time.lt.9999.0d0) then
            write(iaus,'(a,f7.1,3a)') 'set label "',time,fslab, &
               modelabel(dof)(1:12),'" at graph 0.01,0.95'
         else
            write(iaus,'(a,i9,3a)') 'set label "',int(time+0.1d0), &
               fslab,modelabel(dof)(1:12),'" at graph 0.01,0.95'
         endif
         write(iaus,'(3a,f11.3,2a)') 'plot "-" ',ccc(1:sc),&
                                    ' "',time,fslab,'"'
         if(llgrthm) then
            do g=1,ggdim
               write(iaus,'(3(1x,f25.15))') ort(zort(f)-1+g), &
                  max(1.1d0*ymin,gpop1(zort(f)-1+g,s)/weight(g)), &
                  max(1.1d0*ymin,gpop2(zort(f)-1+g,s)*w2)
            enddo
         else
            do g=1,ggdim
               write(iaus,'(3(1x,f25.15))') ort(zort(f)-1+g), &
                  gpop1(zort(f)-1+g,s)/weight(g), gpop2(zort(f)-1+g,s)*w2
            enddo
         endif

      else if (outform .eq. 1) then
         do g=1,ggdim
            write(iaus,'(4(1x,f20.12))') ort(zort(f)-1+g),time,&
            gpop1(zort(f)-1+g,s)/weight(g), gpop2(zort(f)-1+g,s)*w2
         enddo

      else if (outform .eq. 2) then
         write(iaus,'(a)') 'set nolabel'
         if(time.lt.99.9_dop) then
            write(iaus,'(a,f7.2,3a)') 'set label "',time,fslab,&
               modelabel(dof)(1:12),'" at graph 0.01,0.95'
         elseif(time.lt.9999.0d0) then
            write(iaus,'(a,f7.1,3a)') 'set label "',time,fslab, &
               modelabel(dof)(1:12),'" at graph 0.01,0.95'
         else
            write(iaus,'(a,i9,3a)') 'set label "',int(time+0.1d0), &
               fslab,modelabel(dof)(1:12),'" at graph 0.01,0.95'
         endif
         write(iaus,'(2a)') 'plot "-" ',ccc(1:sc-2)//'0.25 '
         if(llgrthm) then
            do g=1,ggdim
               write(iaus,'(3(1x,f20.12))') ort(zort(f)-1+g), &
                  max(1.1d0*ymin,gpop1(zort(f)-1+g,s)/weight(g)), &
                  max(1.1d0*ymin,gpop2(zort(f)-1+g,s)*w2)
            enddo
         else
            do g=1,ggdim
               write(iaus,'(3(1x,f20.12))') ort(zort(f)-1+g), &
                  gpop1(zort(f)-1+g,s)/weight(g), gpop2(zort(f)-1+g,s)*w2
            enddo
         endif

      else if (outform .eq. 3) then
         do g=1,ggdim
            write(iaus,'(3(1x,f20.12))') ort(zort(f)-1+g),&
            gpop1(zort(f)-1+g,s)/weight(g), gpop2(zort(f)-1+g,s)*w2
         enddo
         write(iaus,'(/,a)')
      else if (outform .eq. 4) then
         do g=1,ggdim
            if (sel .eq. 1) then
               write(iaus,*) gpop1(zort(f)-1+g,s)/weight(g)
            else if (sel .eq. 2) then
               write(iaus,*) gpop2(zort(f)-1+g,s)*w2
            endif
         enddo
      else if (outform .eq. 5) then
         g = zort(f)-1+pltpnt
         write(iaus,'(3(1x,f25.15))') time, &
              max(1.1d0*ymin,gpop1(g,s)/weight(pltpnt)), &
              max(1.1d0*ymin,gpop2(g,s)*w2)
      else if (outform .eq. 6) then
         write(iaus,'(f12.5,99(1x,f15.9))') time, &
          (max(1.1d0*ymin,gpop1(zort(f)-1+g,s)/weight(g)),g=1,gdim(f))
      endif

      if (outform .eq. 0 .or. outform .eq. 2) then
         write(iaus,'(a)') 'e'
      else if (outform .eq. 1) then
         write(iaus,*)
      endif


      time=time+out1/fs

      do n=1,step-1
         call rdgridpop(irho,fgpop,gpop1,gpop2)
         if (lend) go to 77

         ntdata=ntdata+1
         time=time+out1/fs
      enddo

      if( time .le. tfinal) goto 10

   77 continue
      if (linter) go to 250
!-----------------------------------------------------------------------
! write file if non-interactive
!-----------------------------------------------------------------------
      close(irho)
      close(iaus)

      if(lprint) then
         inquire(file="printplot",exist=lexist) 
         open(iaus,file="printplot",form='formatted',status='unknown')
         if(.not.lexist) then
            open(iaus,file="printplot",form='formatted')
            write(iaus,'(a)') 'set terminal postscript'
            write(iaus,'(a)')'set output "| lpr" '
            write(6,'(a)')'File printplot created.'
         end if
         rewind iaus
         read(iaus,'(a1)') gnucmd 
         read(iaus,'(a30)') gnucmd 
         close(iaus)
         write(6,'(a)')'Press enter twice to print. Ctrl C to abort.'
         write(6,'(3a)')'Abort and edit printplot if you printcommand',&
                        ' is not: '//gnucmd(11:29)
      end if
      if(lagnu) then
         gnucmd = 'gnuplot -persist "'//trim(fileout)//'" 2>/dev/null'
         call excmd(gnucmd)
      end if
      if(lprint) then
         gnucmd = 'gnuplot -persist '//'"printplot"  "'&
               //fileout(1:slen(fileout))//'" 2>/dev/null'
         call excmd(gnucmd)
      end if
      return

  250 continue
!-----------------------------------------------------------------------
! interactive plotting
!-----------------------------------------------------------------------
      if (lrebound) then
         call timebnds(pltime,ndata)
         f1=axis(1)
         f2=axis(2)
         if (f2 .eq. 0) then
            ydim=1
         else
            ydim=gdim(f2)
         endif
         call databnds(pldata,gdim(f1),ydim,1,1,maxgdim,maxdata,nstate,1)
      endif

!-----------------------------------------------------------------------
! do task
! TASK=0 STOP  else return to menu
!-----------------------------------------------------------------------
      f1=axis(1)
      if (sel .eq. 1) then
         axislab(1)=modelabel(f1)
      else
         call setxlab(axislab(f1),sel,basis(f1))
      endif
      axislab(2)='Time [fs]'
      axislab(3)='Population'

      if (task .gt. 0) then
         if (sel .eq. 1) then
            plttitle='Grid population'
         else if (sel .eq. 2 .and. ldvr(f1)) then
            plttitle='FBR population'
         else if (sel .eq. 2 .and. basis(f1) .eq. 4) then
            plttitle='Momentum'
         endif
         if (plottask .eq. 1) then
            nsets=nplstates
            if (lsumst) nsets = nsets+1
            nplots=1
            f1=axis(1)
            call dotask(task,plottask,plotdim,ort(zort(f1)),pltime,&
                    pldata,gdim(f1),ndata,nsets,nplots,&
                    maxgdim,maxdata,nddstate,1,ort,ortdim,pltime,ndata,&
                    edum,1,rdum,1,1,1,1,idum)
         else if (plottask .eq. 2) then
            f1=axis(1)
            if (plotform .eq. 0) then
               if (nplstates .gt. 1) then
                  write(6,'(a)') &
            'WARNING : Only data for 1st selected state is displayed'
               endif
               nsets=ndata
               nplots=1
               call dotask(task,plottask,plotdim,ort(zort(f1)),y(1),&
                    pldata,gdim(f1),1,nsets,nplots,&
                    maxgdim,1,nsets,1,ort,ortdim,pltime,ndata,&
                    edum,1,rdum,1,1,1,1,idum)
            else
               nsets=nplstates
               if (lsumst) nsets = nsets+1
               nplots=ndata
               call dotask(task,plottask,plotdim,ort(zort(f1)),y(1),&
                    pldata,gdim(f1),1,nsets,nplots,&
                    maxgdim,maxdata,nstate,1,ort,ortdim,pltime,ndata,&
                    edum,1,rdum,1,1,1,1,idum)
            endif
         endif
         go to 200
      endif

      close(iaus,status='delete')

      return
 1000 write(ilog,'(a)') 'Error allocating memory in Statepop :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop

      end subroutine subden1d


!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------
      subroutine den1dinp(stat,llgrthm)

use global
use aglobal
use griddatmod
use datenmod
use plot
use showd1dmod

      implicit none

      integer(long), external       :: myiargc
      integer(long)                 :: iarg,ilbl,i,ierr
      character(len=80)             :: buf,string
      character(len=*), intent(out) :: stat
      logical(kind=4), intent(out)  :: llgrthm

!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! provide default input and output filenames 
!-----------------------------------------------------------------------
      filein  = ' '
      fileout = ' '
      dirout = ' '
      name    = './'

      dof=0
      state=1
      sel=1

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! set default flags
!
! stat : status of output file   
!-----------------------------------------------------------------------
      stat    = 'new'
      lnw     = .false.
      lgnu    = .false.
      lagnu   = .false.
      lprint  = .false.
      lpgrid  = .false.
      lsmooth = .false.
      lstick  = .false.
      llgrthm = .false.
      linter  = .false.
      outform = 0
      factor  = 1.0_dop
      step = 1
      xmax = 0.0_dop
      xmin = 1.0_dop
      ymax = 0.0_dop
      ymin = 0.0_dop
      tfinal = 9.9e+99_dop

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! read options 
! comment out or add options as required
!-----------------------------------------------------------------------
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf .eq. '-inter' ) then
            linter=.true.

         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf(1:3) .eq. '-D ' ) then
            i = i+1
            call mygetarg(i,buf)
            dirout = buf
         else if( buf(1:4) .eq. '-no ' ) then
            fileout='no'
         else if( buf(1:3) .eq. '-I ' ) then
            i = i+1
            outform = 3
            call mygetarg(i,buf)
            read(buf,*,err=999) indx
         else if( buf(1:3) .eq. '-S ' ) then
            outform = 0
         else if( buf(1:3) .eq. '-T ' ) then
            outform = 1
         else if( buf(1:3) .eq. '-M ' ) then
            outform = 2
         else if( buf(1:3) .eq. '-Ma' ) then
            outform = 4
         else if( buf(1:3) .eq. '-P ' ) then
            outform = 5
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=999) pltpnt
         else if( buf(1:3) .eq. '-E' ) then
            outform = 6
         else if( buf(1:3) .eq. '-G ' ) then
            lpgrid = .true.
         else if( buf(1:5) .eq. '-pop2' ) then
            sel=2
         else if( buf(1:3) .eq. '-n ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if(buf(1:4).eq.'-sm ' .or. buf(1:7).eq.'-smooth') then
            lgnu = .true.
            lsmooth = .true.
         else if(buf(1:7).eq.'-stick ') then
            lgnu = .true.
            lstick = .true.
         else if( buf(1:3) .eq. '-x ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=998) xmin
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=998) xmax
         else if( buf(1:3) .eq. '-y ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=996) ymax
         else if( buf(1:3) .eq. '-z ' ) then
            i = i+1
            call mygetarg(i,buf)
         else if( buf(1:3) .eq. '-t ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=996) tfinal
         else if( buf(1:4) .eq. '-fac' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=997) factor
         else if( buf(1:5) .eq. '-skip' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=997) nskip
         else if( buf(1:3) .eq. '-r' ) then
            lrenorm=.true.
         else if( buf(1:3) .eq. '-w ' ) then
            stat='unknown'
         else if( buf(1:6) .eq. '-nofs ' ) then
            fs = 1.0_dop
         else if( buf(1:4) .eq. '-nw ' ) then
            lnw = .true.
         else if( buf(1:3) .eq. '-l ' ) then
            llgrthm = .true.
            if (ymin.le.0d0) ymin=1.0d-10
         else if( buf(1:3) .eq. '-g ' ) then
            lgnu = .true.
         else if( buf(1:3) .eq. '-a ' ) then
            lgnu  = .true.
            lagnu = .true.
            stat  = 'unknown'
         else if( buf(1:3) .eq. '-p ' ) then
            lgnu   = .true.
            lprint = .true.
         else if( buf .eq. '-gnupl4 ') then
            lgnu4 = .true.                 ! gnuplot ver < 5
         else if( buf(1:3) .eq. '-h ' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,a,/1x,a,3(/15x,a))') &
       'Purpose: To selectively plot 1-dimensional densitites.',&
       'Usage  : showd1d [-T|-S|-I|-P|-E|-M|-Ma]',&
       '[-f -i -o -D -a -g -p -x -y -fac -n -no -r -t -skip',&
       '-nofs -l -sm -stick -w -nw -pop2 -G -inter -gnupl4 ',&
       '-ver -h -?] fx sx.'
            write(6,'(/,a)') 'Output Formats : '
            write(6,'(2a)') &
                    ' -S      : "Step through" by pressing RETURN.',&
                    ' (default).'
            write(6,'(a)') &
                    ' -M      : "Movie", one picture every second.'
            write(6,'(2a)') &
                    ' -I id   : "Index file", id=0 shows the first ',&
                    'picture, id=1 the second, etc.'
            write(6,'(2a)') &
                    ' -P pnt  : "Point file", shows the population ',&
                    'of the pnt-th grid-point over time.'
            write(6,'(2a,/11x,2a)') &
                    ' -E      : Similar to -P, but shows the ',&
                    'population of all grid-point over time.',&
                    'May be used to plot state populations of a ',&
                    'single-set run.'
            write(6,'(a)') &
                    ' -T      : "3D time file" (grid file).'
            write(6,'(a)') &
                    ' -Ma     : Mathematica file (for dynamic plots).'

            write(6,'(/,a)') 'Options  : '
            write(6,'(2a,/,9x,a)') &
                     ' -f FILE : The 1D density is read '&
                    ,'from file FILE rather than from ./gridpop'&
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(a)') ' -i DIR  : Use DIR as name-directory.'
            write(6,'(2a,/,11x,a/,11x,a,/11x,a)') &
                     ' -o FILE : The output is written to file',&
                     ' FILE_f* rather than','to ./den1d_f*',&
             'The string FILE may be a relative or a full path-name.',&
             'If FILE=no, i.e. "-o no", no output file will be opened.'
            write(6,'(2a,/,11x,a/,11x,a/,11x,a)') &
                ' -D DIR  : The output is written to directory DIR.' &
               ,' This is useful,' &
               ,'if one has no right to write to the name directory.' &
               ,'The directory DIR must exist.' &
               ,'The string DIR may be a relative or a full path-name.'
            write(6,'(2a)') ' -pop2   : 2nd population plotted ',&
                            '(i.e. basis set occupations).'
            write(6,'(2a)') &
                     ' -g      : GNUplot comand lines are written to ',&
                     'the output file(s).'
            write(6,'(2a)') &
                     ' -a      : GNUplot is called automatically.',&
                     ' (sets -g and -w).'
            write(6,'(2a)') &
                     ' -p      : Print GNUplot (sets -g). ',&
                                'Allowed only with -I or -T.' 
            write(6,'(2a)') &
                     ' -sm     : Smooth the plot-data by splines.',&
                     ' (Only for GNUplot, sets -g).'
            write(6,'(2a)') &
                     ' -stick  : Plot sticks rathet than lines.', &
                     ' (Only for GNUplot, sets -g).'
            write(6,'(2a)')&
                     ' -l      : Use logarithmic scale.',&
                     ' (Only for GNUplot).'
            write(6,'(2a)') ' -n step : Show only every', &
                  ' step-th picture. (Only useful for -S, -M).'
            write(6,'(2a)') ' -t tfinal : Show densities only up',&
                                      ' to t=tfinal.'
            write(6,'(a)') ' -skip n : Skip the first n densities.'
            write(6,'(a)') ' -r      : Re-normalize densities.'
            write(6,'(2a)') &
                     ' -nw     : No weights are employed.',&
                     ' Plot the "naked" DVR populations.'
            write(6,'(a)')&
                     ' -no     : No output file opened.'
            write(6,'(2a)')&
                     ' -nofs   : No transformation to fs.',&
                     " Use when 'time-not-fs' was set in mctdh."
            write(6,'(2a)') &
               ' -x xmin xmax : Set the abscissa length',&
               ' (default: auto). Ignored for -T, -Ma.'
            write(6,'(2a)') &
               ' -y ymax : Set the upper bound of the ordinate',&
               ' (default: auto). Ignored for -T, -Ma.'
            write(6,'(2a)') &
               ' -z ymin : Set the lower bound of the ordinate',&
               ' (default: 0 or 1d-10 (for -l)).'
            write(6,'(2a)') '  -x, -y, -z options are ignored when',&
                            ' -T or -Ma is set.'
            write(6,'(2a)') &
               ' -fac factor : Multiply the density values by factor.'
            write(6,'(a)') ' -G      : Plot gridlines.'
            write(6,'(2a)') &
                     ' -w      : An existing output file is ',&
                     'overwritten.'
            write(6,'(a)')&
               ' -gnupl4  : Use gnuplot version < 5 for plotting '
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(a)') &
                     ' -ver    : Version information about the program '
            write(6,'(/2a,/a,/a)') &
                  ' The arguments fx and sx select the dof and state',&
                  ' (x=integer).',&
             ' With the argument s0 a sum over all states is performed.',&
             ' If sx is not given, s1 is assumed.'
            write(6,'(2a)') ' If no argument is given, ',&
                 'the user is prompted for the missing argument.'  
            write(6,'(a)')  ' EXAMPLES: showd1d -a f1'
            write(6,'(2a)') '           showd1d -a -M -y 0.3 ',&
                            ' -n 2  f2 s1 '
            write(6,'(a)')  '           showd1d -w -p -I 20 f1'
            write(6,'(a)')  '           showd1d -a -p -T f3'
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      if (iarg .lt. i) goto 200

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! get any compulsory arguments if in argument list
!-----------------------------------------------------------------------
  150 continue
      call mygetarg(i,string)
      if (string(1:1) .eq. 'f' .or. string(1:1) .eq. 'F') then
         read(string(2:20),*) dof
      else if (string(1:1) .eq. 's' .or. string(1:1) .eq. 'S') then
         read(string(2:20),*) state
      else if (string(1:1) .eq. '-') then
         write(6,'(2a)')'Expecting "f" or "s", not "-".  ',trim(string)
         stop 1
      endif
      i=i+1
      if (i .le. iarg) goto 150

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! fill in any arguments not given on command line
!-----------------------------------------------------------------------
 200  continue
      if (linter) then
         dof=1
      else if (dof .eq. 0) then
         write(6,*) 'DOF  :'
         read(5,*,iostat=ierr) dof
         write(6,*)
         if(ierr.ne.0) then
            write(6,'(a)') 'Wrong input. Please give an integer only.'
            dof=0
            goto 200
         endif
      endif

!-----------------------------------------------------------------------
! Check for consistency.
!-----------------------------------------------------------------------
      if(lprint .and. (outform.eq.0 .or. outform.eq.2)) then
         write(6,'(a)') '-p allowed only with -I or -T. STOP!'
         stop
      end if

!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
!-----------------------------------------------------------------------
      laenge=index(name,' ')-1
      call abspath(name,laenge)
      dlaenge=laenge
      dname=name

      if (filein .eq. ' ') then
         filein=name(1:laenge)//'/gridpop'
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)
      endif

!-----------------------------------------------------------------------
! set up output file name
!-----------------------------------------------------------------------
      if (dirout .eq. ' ') then
         dirout=name
         laeoutd=laenge
      else
         laeoutd=index(dirout,' ')-1
         call abspath(dirout,laeoutd)
      endif

      if (fileout .eq. 'no' .or. linter) go to 500
      ilbl=index(fileout,' ')
      if (ilbl .eq. 1) then
         fileout=dirout(1:laeoutd)//'/den1d'
         ilbl=index(fileout,' ')
      endif
      fileout(ilbl:ilbl+1)='_f'
      ilbl=ilbl+2
      write(fileout(ilbl:ilbl+4),'(i5)') dof
      ilbl=ilbl+5

      if (state .ne. 1) then
         fileout(ilbl:ilbl+1)='_s'
         ilbl=ilbl+2
         write(fileout(ilbl:ilbl+4),'(i5)') state
      endif
      call rmblnk(fileout)
      
!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! convert filenames to absolute
!-----------------------------------------------------------------------
      laeout=index(fileout,'  ')-1
      call abspath(fileout,laeout)

 500  continue

      if (outform .eq. 4) then
         ymax   =  0.0_dop
         lgnu   = .false.
         lagnu  = .false.
         lprint = .false.
      end if

      if (linter) lagnu=.false.
      if (linter) lprint=.false.
      if (linter) outform=-1

      return 
 999  write(6,*) 'Integer expected after -I  option '
      stop
 998  write(6,*) 'Two real numbers expected after -x  option '
      stop
 997  write(6,*) 'Real number expected after -y  option '
      stop
 996  write(6,*) 'Real number expected after -fac  option '
      stop
      end subroutine den1dinp


!#######################################################################

      subroutine setxlab(xaxlab,sel,basis)

      implicit none

      integer(long), intent(in)     :: sel,basis
      character(len=*), intent(out) :: xaxlab

      if( sel .eq. 1 ) then
         if(basis.eq.6) then
            xaxlab='l'
         else if(basis.eq.-1) then
            xaxlab='m'
         else
            xaxlab='x'
         endif
      else
! -  HO  -
         if( basis .eq. 1  ) then
            xaxlab='n'
! -  Leg  Leg/R  Kleg  Wigner-
         elseif( basis .eq. 2 .or. basis .eq. 7&
            .or. basis .eq. 9 .or. basis .eq. 12&
            .or. basis .eq. 20 .or. basis .eq. 21&
            .or. basis .eq. 33) then
            xaxlab='j'
! -  FFT -
         elseif( basis .eq. 4  ) then
            xaxlab='p'
! -  exp -
         elseif( basis .eq. 5  ) then
            xaxlab='p'
         else
            xaxlab='n'
         end if
      end if

      return
      end subroutine setxlab


      end program showd1d

