      module showfieldmod

use decimal, only: long,dop,sip
use global
use maxv
use aglobal
use versions
use dvrdatmod
use griddatmod
use operdef
use psidef
use dirdyn
use channels
use datenmod
use plot
use compdatmod
use logdat

      integer(long),save   :: state,state1
! set no. or points to be plotted
      integer(long),save   :: npnts=221

      end module showfieldmod

      program showfield

use op1lib, only: zerovxd
use logdat, only: errormsg
use iofile, only: rdpsiinfo
use rddvrmod, only: dvrinfo
use rdopermod, only: operinfo,reinitext1d
use zeigauswmod, only: zeigausw
use memhopsmod, only: memhops
use showfieldmod
use iofile, only: rdchkinfo

use module_interface

use rdopermod, only: rdoper
use plotutils, only: plotdefs,initplot,timebnds,plotmenu,dotask
use potevalmod, only: poteval

      implicit none

      integer(long)                        :: chkdvr,chkgrd,chkpsi,chkdat,ilbl
      character(len=c5)                    :: filename
      logical(kind=4)                      :: lerr

      integer(long)     :: ierr
      character(len=10) :: cerr

! standard variables for interactive plots
      integer(long)                        :: ndata,task,plottask,nplots,&
                                              plotdim,n,nham,k,s,s1,h,&
                                              pottyp,nsets
      integer(long), dimension(:), allocatable :: idum
      real(dop)                                :: time,c,tdel
      real(dop), dimension(1)                  :: edum
      real(dop), dimension(:), allocatable     :: rdum,gpoint
      real(dop), dimension(:), allocatable     :: pltime
      real(dop), dimension(:), allocatable     :: efield
      real(dop), dimension(:), allocatable     :: hops

      complex(dop), dimension(:), allocatable  :: workc
      logical :: lopen,check
      logical :: lexist=.false.
      
!-----------------------------------------------------------------------
! following are needed for plots
!-----------------------------------------------------------------------
      integer(long), external :: slen
      integer(long)           :: i
!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      lovwr = .true.

!-----------------------------------------------------------------------
! read input options
!-----------------------------------------------------------------------
      call showfieldinput

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare resources required (other than those needed to read a file)
!
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! ltrafo : FBR/DVR trafo matrices
! lfft   : FFT arrays rueck and fftfak
!-----------------------------------------------------------------------
      ncomp=0
      lpsi=.false.
      lpsi1=.false.
      lpsigrd=.false.
      ldmat=.false.
      ldmat1=.false.
      lort=.false.
      lgpop=.false.
      ltrafo=.false.
      lfft=.false.

!-----------------------------------------------------------------------
! get array dimensions 
!-----------------------------------------------------------------------
      inquire(ioper,opened=check)
      if (check) close(ioper)
      filename=operfile
      ilbl=index(filename,' ')-1
      open(ioper,file=filename,form='unformatted',status='old',err=1000)
      call rdmemdim(ioper)
      close(ioper)

!-----------------------------------------------------------------------
! Allocate mememory
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      if (ldd) then
         call alloc_dirdyn
         call alloc_dddb
         call alloc_dercp
      endif

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! read information from system files
!-----------------------------------------------------------------------
      chkdvr=1
      filename=dname(1:dlaenge)//'/dvr'
      ilbl=index(filename,' ')-1
      write(6,'(2a)') ' The DVR file is : ',filename(1:ilbl)
      open(idvr,file=filename,form='unformatted',status='old',err=1000)
      call dvrinfo(lerr,chkdvr)
      close(idvr)
      call chksyserr(filename,lerr)
      if (lerr) go to 900

      chkdvr=2
      chkgrd=1
      filename=operfile
      ilbl=index(filename,' ')-1
      write(6,'(2a)') ' The operator file is : ',filename(1:ilbl)
      open(ioper,file=filename,form='unformatted',status='old',err=1000)
      call operinfo(lerr,chkdvr,chkgrd)
      close(ioper)
      call chksyserr(filename,lerr)
      if (lerr) go to 900

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
! maxdata1 is size of data set to be plotted. Set it 2 twice initial
! value to allow for resizing during plotting.
!-----------------------------------------------------------------------
      maxdata1=2*npnts
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
! set nplottask and ncalcpar before allocating memory for plotting routines
!-----------------------------------------------------------------------
      nplottask=1
      ncalcpar=3
      call alloc_plot

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      workcdim=1
      workcdim=max(workcdim,dgldim+maxspf*2)
      workcdim=max(workcdim,2*maxblock+maxspf*maxspf*2)

!-----------------------------------------------------------------------
! assign memory for operator
!-----------------------------------------------------------------------
      call memhops

!-----------------------------------------------------------------------
! write version
!-----------------------------------------------------------------------
      call wrversion(ilog)

!-----------------------------------------------------------------------
! allocate memory
!-----------------------------------------------------------------------
      cerr = 'pltime'
      allocate(pltime(maxdata1),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'efield'
      allocate(efield(maxdata1),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'hops'
      allocate(hops(hopsdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'gpoint'
      allocate(gpoint(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'rdum'
      allocate(rdum(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'idum'
      allocate(idum(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'workc'
      allocate(workc(workcdim),stat=ierr)
      if (ierr > 0) go to 1000

      rdum=0
      idum=0

!-----------------------------------------------------------------------
! interactive plotting
! open file for data to be plotted
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/tmpplot.pl'
      open(iaus,file=filename,status='unknown')

!-----------------------------------------------------------------------
! interactive plot defaults
!-----------------------------------------------------------------------
      call plotdefs

      ndata=1
      call zerovxd(pltime,maxdata1)

!-----------------------------------------------------------------------
! read operator data
!-----------------------------------------------------------------------
      operfile=operfile
      call rdoper(hops,chkdvr,chkgrd)

!-----------------------------------------------------------------------
! set up data for interpolation of time data from a file
!-----------------------------------------------------------------------
      call reinitext1d

!-----------------------------------------------------------------------
! Get operator number
!-----------------------------------------------------------------------
 20   nham=0
      do n=1,hamdim
         if (hamlab(n) .eq. operator) nham=n
      enddo

      if (nham .eq. 0) then
         write(6,'(4a)')'Operator "',&
           operator(1:slen(operator)),'" not found in ',&
           operfile(1:slen(operfile))
         write(6,'(1x,a)') 'Name of operator ? Choose from:'
         do n=1,hamdim
            write(6,'(2x,a)') hamlab(n)
         enddo
         read(5,'(a)') operator
         if (operator .eq. ' ') stop
         goto 20
      endif
      if (nham .ne. 1) write(6,'(2a)')'Operator selected :',operator

!-----------------------------------------------------------------------
! If propagation has been run get time limits
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/check'
      inquire(file=filename,exist=lexist)
      if (lexist) then
         open(ichk,file=filename,form='unformatted',status='old',err=1000)
         read(ichk) filever(ichk)

         chkdvr = 0
         chkgrd = 0
         chkpsi = 0
         chkdat = 1
         call rdchkinfo(ichk,chkdvr,chkgrd,chkpsi,chkdat)
! convert times to fs (default for plotting)
         tinit=tinit/fs
         tfinal=tfinal/fs
      endif

!-----------------------------------------------------------------------
! menu defaults
!-----------------------------------------------------------------------
      plotdim=1
      lpltime=.true.
      lstate=.true.
      filename=name(1:laenge)//'/tmpplot.pl'
      call initplot(filename,plotdim)

      nplstates=2
      plstate(1) = state
      plstate(2) = state1

!-----------------------------------------------------------------------
! calculate initial time points and bounds for plotting
!-----------------------------------------------------------------------
      ndata=npnts
      tdel= (tfinal-tinit)/(npnts-1)
      do n=1,ndata
         pltime(n)=tinit+(n-1)*tdel
      enddo
      ibound(1,ftime) = 1
      ibound(2,ftime) = npnts
      bound(1,ftime) = pltime(1)
      bound(2,ftime) = pltime(npnts)

!-----------------------------------------------------------------------
! set plot tasks
! do not exceed nplottask or ncalcpar
!-----------------------------------------------------------------------
      cplottask(1)='Field with time'
      plottask=1
      
      do i=1,nplottask
         cptlen(i)=slen(cplottask(i)) 
      enddo

!-----------------------------------------------------------------------
! add extra options for menus
!-----------------------------------------------------------------------
      if (linter) then
         acalcpar(1)= 'Tinit, (fs)'
         acalcpar(2)= 'Tfinal (fs)'
         acalcpar(3)= 'Npnts'
         rcalcpar(1)= tinit
         rcalcpar(2)= tfinal
         rcalcpar(3)= npnts
      endif

!-----------------------------------------------------------------------
! call menu and select task
!-----------------------------------------------------------------------
    1 continue
      if(.not.lredo) call plotmenu(task,plottask,plotdim)

!-----------------------------------------------------------------------
! get time points and data bounds
!-----------------------------------------------------------------------
      if (lrecalc) then
         tinit = rcalcpar(1)
         tfinal = rcalcpar(2)
         npnts = int(rcalcpar(3))
         if (npnts .gt. maxdata1) then
            npnts=maxdata1
            write(6,*)
            write(6,'(a,i5,a)') &
                 'Npnts must be =< ',maxdata1,' Resetting'
            write(6,*)
         endif
         ndata=npnts
         tdel= (tfinal-tinit)/(npnts-1)
         do n=1,ndata
            pltime(n)=tinit + (n-1)*tdel
         enddo
         call timebnds(pltime,ndata)
         lrebound=.false.
      endif

!-----------------------------------------------------------------------
! calculate field
!-----------------------------------------------------------------------
      if (plottask .eq. 1) then
         do n=1,ndata
            time=pltime(n)
            efield(n)=0.0_dop
            do k=zham(nham),zham(nham)+khtot(nham)-1
               s = kf(k)
               s1= ki(k)

               if (s .eq. plstate(1) .and. s1 .eq. plstate(2)) then

                  h=hamilton(fkoe,k)
                  if (htmtype(h) .eq. 101) then
                     gpoint(fkoe)=1.0_dop
! convert time to au
                     gpoint(ftime+1)=time*fs
                     pottyp=0
                     call poteval(h,c,gpoint,koeff(k),rdum,pottyp)

                     efield(n)=efield(n)+c
                  endif
               endif
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! write title and axis labels
!-----------------------------------------------------------------------
      plttitle=operfile
      axislab(2)='Field ['//zaxunitlab(1:slen(zaxunitlab))//'] '

!-----------------------------------------------------------------------
! do task
! TASK=0 STOP  else return to menu
!-----------------------------------------------------------------------
      if (task .gt. 0) then
  500    continue
         if (plottask .eq. 1) then
            nplots=1
            nsets = 1
            call dotask(task,plottask,plotdim,pltime,rdum,&
                 efield,ndata,1,nsets,nplots,&
                 ndata,1,1,1,rdum,ortdim,pltime,npnts,edum,1,&
                 rdum,1,1,1,1,idum)
         endif
         if (task.eq.30) then
            ndata = ibound(2,ftime)-ibound(1,ftime)+1
            rcalcpar(1)= bound(1,ftime)
            rcalcpar(2)= bound(2,ftime)
            rcalcpar(3)= ndata
         endif
         if(task.eq.60 .and. nplstates .ne. 2) then
            write(6,'(a)') 'Must choose a pair of states'
            task=60
            go to 500
         endif
         if (task .eq. 800) lrebound = .true.
         if (task .eq. 801) lrebound = .true.
         if (task .eq. 802) lrebound = .true.
!         if(task.eq.285) nplots = nplots0
         goto 1
      endif

! delete plot file
      inquire(unit=iaus,opened=lopen)
      if (lopen) close(iaus,status='delete')

! deallocate memory
      deallocate(pltime,efield,hops,gpoint)
      deallocate(workc,rdum,idum)
      call dealloc_plot

      stop
  900 continue
      ilbl=index(filename,' ')-1
      message= 'Error reading :'//filename(1:ilbl)
      write(6,'(a)') message
      stop
 1000 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg

      contains

!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------

      subroutine showfieldinput

      use showfieldmod

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: iarg,i,ierr
      logical(kind=4)         :: lnopes
      character(len=80)       :: buf

      operfile  = ' '
      name  = './'
      fileout = 'field.pl'
      ncut = 999999
      nskip=0

      lshowpsi=.true.
      lshowrst=.false.
      lpop2   =.false.
      lnopes  =.false.
      lnw     =.true.
      linter = .true.

      tinit=-100.0
      tfinal=200.0
      state =1
      state1 =1
      operator = 'system '

! default path to DB (required for a DD calc)
      ddpath = ".."

      iarg = myiargc() 
      
      i = 0
      if (iarg .lt. i) goto 200
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if(buf .eq. '-i') then
            i = i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf(1:3) .eq. '-p ') then
            i = i+1
            call mygetarg(i,buf)
            operfile = buf
         else if( buf(1:3) .eq. '-si ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*) state 
         else if( buf(1:3) .eq. '-sf ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*) state1
         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
            laeout=index(fileout,' ')-1
            fileout = fileout(1:laeout)//'.pl '
         else if( buf(1:4) .eq. '-ti ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) tinit
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -ti '
               stop
            endif
         else if( buf(1:4) .eq. '-tf ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) tfinal
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -tf '
               stop
            endif

         else if( buf(1:4) .eq. '-op ') then
            i = i+1
            call mygetarg(i,buf)
            operator = buf(1:32)

         else if( buf .eq. '-gnupl4 ') then
            lgnu4 = .true.                 ! gnuplot ver < 5

         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(/1x,a)')&
       'Purpose: Enables plotting of time-dependent field.'
            write(6,'(1x,a)')&
       'Usage:   showfield [-i -p -o -ti T0 -tf T -si S1 -sf S2 -gnupl4 -ver -h -?]'
            write(6,'(/,a)') 'Options : '
            write(6,'(a)') ' -i DIR  : data is stored in directory DIR'
            write(6,'(2a,/,9x,a)') &
                     ' -o FILE : The ouput is written to file ',&
                     'FILE.pl rather than to ./field.pl'
            write(6,'(a)') ' -ti T0 : Initial time is T0 (fs)'
            write(6,'(a)') ' -tf T  : Final time is T (fs)'
            write(6,'(a)') ' -si S1 : Initial State is S1'
            write(6,'(a)') ' -sf S2 : Final State is S1'
            write(6,'(a)') ' -op H: Evaluate operator H rather than system'
            write(6,'(a)')&
               ' -gnupl4  : Use gnuplot version < 5 for plotting '
            write(6,'(a)')&
                     ' -ver    : Version information about the program.'
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(2(/2x,2a),(/2x,a),/2a)')&
            'The program is menu driven. Just type "showfield" ',&
            'and then follow the menu. ',&
            'By default the field will ',&
            'be plotted from -100 to 200 fs',&
            'Use the options to choose the correct states and time.', &
            '-----------------------------',&
            '---------------------------------------------------'

            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         goto 100
      end if
      nopt=i-1
      
!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! fill in any arguments not given on command line
!-----------------------------------------------------------------------
 200  continue
      if (state .eq. 0) then
         write(6,*) 'State1  :'
         read(5,*) state
         write(6,*)
         write(6,*) 'State2  :'
         read(5,*) state1
         write(6,*)
      endif

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='showsys.log',form='formatted')
      logisopen=.true.
      
      laenge=index(name,' ')-1
      call abspath(name,laenge)

      dname=name
      dlaenge=index(dname,' ')-1
      call abspath(dname,dlaenge)

      laeout=index(fileout,' ')-1
      call abspath(fileout,laeout)

      if (operfile .eq. ' ') operfile=name(1:laenge)//'/oper'

      return 
      end subroutine showfieldinput

      end program showfield
