      program showsys

use decimal, only: long,dop,sip
use maxv
use global
use globmemmod
use aglobal
use versions
use keyunitsmod
use dvrdatmod
use griddatmod
use operdef
use psidef
use channels
use datenmod
use dirdyn
use compdatmod
use plot
use plotutils
use gapmod

use op1lib, only: initvxi, zerovxd
use logdat
use iorst, only: rstinfo
use iofile, only: rdpsiinfo
use rddvrmod, only: dvrinfo
use rdopermod, only: operinfo
use zeigauswmod, only: zeigausw
use memhopsmod, only: memhops

use dd_db, only: getdbnrec
use db_sql, only: dbopen

use module_interface

      implicit none

      integer(long)           :: chkdvr,chkgrd,chkdat,ilbl,chkpsi
      integer(long)           :: nrec
      character(len=c5)       :: filename
      logical(kind=4)         :: lerr,linwf,lexist
      integer(long)           :: slen

      character(len=c1)       :: moderep  !Quick fix for " error in gnuplot

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      lovwr = .true.
      sqldbfile = ' '

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare files to be read
! set flag to .true. if file is to be read
!
!     lrddvr    DVR file
!     lrdoper   OPER file. This is set true if oper exists and lnopes
!               false (set in showsysinput).
!-----------------------------------------------------------------------
      lrddvr=.true.
      lrdoper=.false.

!-----------------------------------------------------------------------
! read input options
!-----------------------------------------------------------------------
      call showsysinput

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
     if(laeoutd .eq. 0) then
         filename=name(1:laenge)//'/showsys.log'
      else
         filename=dirout(1:laeoutd)//'/showsys.log'
      endif
      open (unit=ilog,file=filename,form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      if (lshowrst .or. lshowpsi) then
         filename=name(1:laenge)//'/restart'
         open(irst,file=filename,form='unformatted',status='old',err=1000)
         call rdmemdim(irst)
         close(irst)
      endif

! dimensions from oper must be read second as the oper file may have
! been recreated at a later date with new operators
      if (lrdoper) then
         filename=name(1:laenge)//'/oper'
         open(ioper,file=filename,form='unformatted',status='old',err=1000)
         call rdmemdim(ioper)
         close(ioper)
      endif

! Cannot use ldbsmall for plotting surfaces 
      if (ldd .and. ldbsmall) then
         ldbsmall = .false.
      endif

!-----------------------------------------------------------------------
!  Open data file
!-----------------------------------------------------------------------
      if (lshowrst) then
         filename=psifile
         open(irst,file=filename(1:slen(filename)),form='unformatted', &
              status='old',err=1000)
         write(6,'(2a)') ' The restart file is : '//psifile(1:slen(psifile))
      elseif (lshowpsi) then
         filename=psifile
         open(ipsi,file=filename(1:slen(filename)),form='unformatted', &
              status='old',err=1000)
         write(6,'(2a)') ' The psi file is : '//psifile(1:slen(psifile))
      endif


      if (lrediab .and. .not. ldd) then
         write(6,'(a)') &
           'ERROR: Not a DD calculation so cannot plot raw surfaces'
         stop
      endif

!-----------------------------------------------------------------------
! Allocate memory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      if (ldd.or.lddmctdh) then
         call alloc_dirdyn
         call alloc_dddb
      else if (ltraj) then
         call alloc_dirdyn
      endif
      if (nbasis .eq. 0) nbasis=maxmoco
      if (lddmctdh) call alloc_dirdyn_ddmctdh
      if (ldd.or.lddmctdh) call alloc_dercp
      call alloc_gapkernel
      call alloc_psidef

!-----------------------------------------------------------------------
!  Read the system information
!-----------------------------------------------------------------------
      filename=dname(1:dlaenge)//'/dvr'
      open(idvr,file=filename,form='unformatted',status='old',&
           err=1000)
      ilbl=index(filename,' ')-1
      write(6,'(2a)') ' The DVR file is : ',filename(1:ilbl)
      chkdvr=1
      call dvrinfo(lerr,chkdvr)

      if (lerr) go to 999
      close(idvr)

      if (lrdoper) then
         filename=operfile
         open(ioper,file=filename,form='unformatted',status='old',&
              err=1000)
         ilbl=index(filename,' ')-1
         write(6,'(2a)') ' The operator file is : ',filename(1:ilbl)
         chkdvr=2
         chkgrd=1
         call operinfo(lerr,chkdvr,chkgrd)
         if (lerr) go to 999
         close(ioper)
         if (ldd .or. lddmctdh) then
            lexist=.false.
            inquire(file=ddname(1:ddlaenge)//'/geo.dba',exist=lexist)
            if (lexist) then
               write(6,'(2a)') ' the dd db is in ascii format.&
         &        first use convdbtosql'
               stop
            endif
            sqldbfile = ddname(1:ddlaenge)//'/database.sql'
            inquire(file=sqldbfile,exist=lexist)
            if (lexist) then
               write(6,'(2a)') ' the dd db is in : ',ddname(1:ddlaenge)&
                  //'/database.sql'
            else
               write(6,'(2a)') ' error: the dd db is not found'
               write(6,'(2a)') ' db asked for is : ',ddname(1:ddlaenge)&
                  //'/database.sql'
               write(6,'(a)') ' Use -DB keyord to specify path?'
               stop
            endif
         endif

         if (loptcontract) then
            write(6,'(a)') &
       'Warning: Operator contains terms that mix KE and PE '
            stop
         endif

! if reading raw adiabatic data and multi-state, need to set this 
! flag to read the non-adiabatic couplings
         if (lrediab .and. nddstate .gt. 1) ldiabdb = .true.
! allocate arrays to read raw data
         if (lrediab) then
            if (.not.the_db%db_open) then
               !Needed for NEWSQL
               call makeflagarray
               call makeinfoarray(nddstate)
               call dbopen(sqldbfile, the_db,sqlvers,quanticsver,&
                    infoarray,flagarray,1)  ! untested with SQL DB
            endif
            call getdbnrec(nrec)
            call alloc_dirdyn_rediab(nrec,nddstate,ndofddpes)
         endif
      endif

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare resources required (other than those needed to read a file)
!
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! ltrafo : FBR/DVR trafo matrices
! lfft   : FFT arrays rueck and fftfak
!-----------------------------------------------------------------------
      ncomp=0
      if (lshowpsi) then
         lpsi=.true.
      else
         lpsi=.false.
      endif
      lpsi1=.false.
      lpsigrd=.false.
      ldmat=.true.
      ldmat1=.false.
      lort=.true.
      lgpop=.false.
      ltrafo=.true.
      lfft=lpop2

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! open input and output data files
!
! use standard channels if possible
!  irho : gridpop file
!  ipsi : psi file
!  iaut : auto file
!  ichk : check file
!  iaus : output file
!-----------------------------------------------------------------------
! psifile is opened in showsysinput if required
!     filename='psi'
!     open(ipsi,file=filename,form='unformatted',status='old',err=1000)

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
! First read file version number
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      if (lshowrst) then
         filename='restart'
         read(irst,err=999) filever(irst)
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkdat)
      elseif (lshowpsi) then
         filename='psi'
         read(ipsi,err=999) filever(ipsi)
         call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      endif
      if(dentype .gt.1 .and. lshowpsi) then
         write(6,'(a)') 'Sorry, showsys cannot plot densityII-operators'
         stop
      endif

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
! here assign any further memory arrays
!-----------------------------------------------------------------------
      mrweight=mrdim
      mrdim=mrweight+ortdim
      mrpcent=mrdim
      mrdim=mrpcent+qcentdim

      miplt2ort=midim
      midim=miplt2ort+ortdim
      miort2plt=midim
      midim=miort2plt+ortdim

!-----------------------------------------------------------------------
! assign work arrays - redundant but needed for memeory allocation
!-----------------------------------------------------------------------
      mcworkc = mcdim
      mrworkr = mrdim
      miworki = midim
      mlworkl = mldim
      msworks = msdim
      mfworkf = mfdim

!-----------------------------------------------------------------------
! assign memory for operator
!-----------------------------------------------------------------------
      if (lrdoper) then
         call memhops
      else
         mrhops=1
      endif

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)
if(allocated(gfgwp))deallocate(gfgwp)
if(allocated(aigwp))deallocate(aigwp)
if(allocated(zgwp))deallocate(zgwp)
if(allocated(gs2))deallocate(gs2)
if(allocated(agmat))deallocate(agmat)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))
allocate(gfgwp(gfgwpdim))
allocate(aigwp(aigwpdim))
allocate(zgwp(zgwpdim))
allocate(gs2(gs2matdim))
allocate(agmat(agmatdim))

      call plotsys(mr(mrhops),mr(mrort),mc(mcpsi),&
           ms(mspsi),mi(mijindx),&
           mr(mrtrafo),mr(mrweight),agmat,mr(mrqcent),&
           mr(mrpcent),gfgwp,aigwp,zgwp,&
           gs2,mc(mcexphin),mc(mcrueck),&
           mi(mifftfak),mi(mitrajst),mc(mcadgwp),&
           mi(miplt2ort),mi(miort2plt),mi(migwpdep))

deallocate(mc,mr,mi,ml,ms,mf,gfgwp,aigwp,zgwp,gs2,agmat)
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_psidef
      call dealloc_operdef
      call dealloc_dirdyn
      call dealloc_dddb
      call dealloc_dercp
      call dealloc_dirdyn_ddmctdh
      call dealloc_gapkernel
      call dealloc_dercp
      if (lrediab) call dealloc_dirdyn_rediab
      call dealloc_plot

      stop
  999 continue
      message= 'Error reading :'//trim(filename)
      write(6,'(a)') message
      stop
 1000 continue
      message = 'Cannot open file: '//trim(filename)
      call errormsg
      stop

      contains


!#######################################################################

      subroutine plotsys(hops,ort,psi,spsi,jindx,&
                 trafo,weight,agmat,qcenter,pcenter,gfgwp,&
                 aigwp,zgwp,gs2,exphin,rueck,fftfak,trajst,&
                 adgwp,plt2ort,ort2plt,gwpdep)

use global
use aglobal
use dvrdatmod
use griddatmod
use operdef
use psidef
use channels
use datenmod
use dirdyn
use compdatmod
use plot

use gwplib, only: gwp_cmom,gwp_ovl,gwp_param
use plotutils
use rdopermod, only: rdoper
use rddvrmod
use rdfilesmod, only: rdpsi
use psiutil, only: getweight

      implicit none

! standard variables for interactive plots
      integer(long)                                     :: ndata,task,plottask, &
                                                           nsets,nplots,&
                                                           plotdim,psicount,f,m,&
                                                           chkdvr,chkgrd,&
                                                           ierr,ydim,chkpsi, &
                                                           chkdat,nplots0,plgdim1,&
                                                           plgdim2,n,ilbl,jlbl
      integer(long), dimension(1)                       :: idum,idum1
      integer(long), dimension(*), intent(out)          :: fftfak
      integer(long), dimension(:), allocatable          :: worki
      integer(long), dimension(adim), intent(out)       :: jindx
      integer(long), dimension(gwpinfodim), intent(out) :: trajst,gwpdep
      integer(long), dimension(ortdim), intent(out)     :: plt2ort,ort2plt

      real(dop)                                           :: time,zaxunit0
      real(dop), dimension(1)                             :: edum,rdum,rdum1,rdum2
      real(dop), dimension(:), allocatable                :: plort1,plort2
      real(dop), dimension(:), allocatable                :: pltime
      real(dop), dimension(hopsdim), intent(out)          :: hops
      real(dop), dimension(ortdim), intent(out)           :: ort,weight
      real(dop), dimension(dvrdim), intent(out)           :: trafo
      real(dop), dimension(:,:,:), allocatable            :: srf
      real(dop), dimension(:,:,:,:), allocatable          :: wfunc

      complex(sip), dimension(spsidim), intent(out)  :: spsi
      complex(dop), dimension(1)                     :: zdum,zdum1
      complex(dop), dimension(dgldim), intent(out)   :: psi
      complex(dop), dimension(:), allocatable        :: workc
      complex(dop), dimension(fftdim), intent(out)   :: rueck
      complex(dop), dimension(expdim), intent(out)   :: exphin
      complex(dop), dimension(gsmatdim), intent(out) :: adgwp

! for GWP plotting
      real(dop), dimension(qcentdim), intent(out)     :: qcenter,pcenter
      complex(dop), dimension(agmatdim), intent(out)  :: agmat
      complex(dop), dimension(gfgwpdim), intent(out)  :: gfgwp
      complex(dop), dimension(aigwpdim), intent(out)  :: aigwp
      complex(dop), dimension(zgwpdim), intent(out)   :: zgwp
      complex(dop), dimension(gs2matdim), intent(out) :: gs2

      logical(kind=4)                                 :: lerr,linwf
      character(len=16)                               :: zlab0
      character(len=c2)                               :: rdtag
      
! no. of structures in a cut file
      integer(long), save                             ::  ncutstruct=1

!-----------------------------------------------------------------------
! following are needed for plots
!-----------------------------------------------------------------------
      integer(long)           :: f1,f2,i
      integer(long), external :: slen
      character(len=c5)       :: line,tmptitle

!-----------------------------------------------------------------------
! read DVR and operator data
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1

      dvrdata(1)=.true.      ! ort
      dvrdata(2)=.true.      ! trafo
      if(lfft) then
         dvrdata(6)=.true.   ! rueck
         dvrdata(7)=.true.   ! fftfak
         dvrdata(11)=.true.  ! exphin
      endif
      call rddvr(ort,trafo,rdum,rdum1,zdum,rueck,fftfak,exphin,zdum1,&
                 idum,idum1,rdum2,chkdvr)

      if (lrdoper) then
         operfile=operfile
         call rdoper(hops,chkdvr,chkgrd)
      endif
      ddmindb = 999.0
      if (lrediab) call getraw
! Force use of local DB points for plotting if selected
      if (ldblocal) then
         lgausspot = .false.
         lddmctdh = .false.
      endif

!-----------------------------------------------------------------------
! allocate arrays for plotting
!-----------------------------------------------------------------------
      if (lshowpsi .and. lrdoper) then
         nplottask=10
      else if (lshowpsi) then
         nplottask=10
      else if (nddstate .gt. 1) then
         nplottask=4
      else 
         nplottask=1
      endif
      call alloc_plot

!-----------------------------------------------------------------------
! interactive plot defaults
!-----------------------------------------------------------------------
      zaxunit0=zaxunit ! Because zaxunit will be overwritten by initplot
      zlab0=zaxunitlab
      call plotdefs
      lshowsys=.true.

!-----------------------------------------------------------------------
! open scratch file
!-----------------------------------------------------------------------
      open(unit=itmpin,file='scratch',form='formatted')

!-----------------------------------------------------------------------
! get maximum grid sizes - may be different to gdim(f) if very long 
! grids have been used
!-----------------------------------------------------------------------
      call getplgdim

      allocate(plort1(maxplgrid))    
      allocate(plort2(maxplgrid))  

!-----------------------------------------------------------------------
! determine no. of data sets and allocate pldata arrays
!-----------------------------------------------------------------------
      if (lshowpsi .and. .not. lshowrst) then
         time=(tinit-out2)/fs
         ndata=0
         rewind(ipsi)
         chkdvr=0
         chkgrd=0
         chkpsi=0
         chkdat=0
         psicount=-nskip
         read(ipsi) filever(ipsi)
         call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)

         do      
            call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
            if(lend .and. psicount.le.0) then
               write(6,'(/a,i6,a,i6,a)') &
                     'There are only', psicount+nskip,&
                     ' psi functions on file, but you skip over',&
                     nskip,' functions.'
               stop
            endif
            if (lend) exit

            do n=1,step-1
               call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
               if (lend) exit
            enddo

            psicount=psicount+1

            if(psicount.le.0) cycle
            if(psicount.gt.ncut)  exit
            ndata=ndata+1
         enddo
         nplots = ndata
         maxdata = ndata
      else 
         nplots = 1
         maxdata = 1
      endif

      if (lshowpsi .or. lshowrst) then
         allocate(pltime(maxdata))
         allocate(wfunc(maxplgrid,maxplgrid,nddstate,maxdata),stat=ierr)
         if (ierr > 0) then
            write(ilog,'(a)') 'Error allocating memory for Wavefunction '
            write(ilog,'(a)') 'Too many waveunctions?? Use -skip, -step, -n to reduce'
            write(ilog,'(a)') 'Error Code :',ierr
            stop
         endif
         pltime=0.0_dop
         wfunc=0.0_dop
      else
         allocate(pltime(1))
         allocate(wfunc(1,1,1,1))
      endif

      if (lrdoper) then
         allocate(srf(maxplgrid,maxplgrid,nddstate))
         srf=0.0_dop
      endif

      weight=0.0_dop

!-----------------------------------------------------------------------
! work arrays
!-----------------------------------------------------------------------
      workcdim=max(workcdim,dgldim+maxspf*2)
      workcdim=max(workcdim,2*maxblock+maxspf*maxspf*2)
      if(lpop2)  workcdim=max(workcdim,totphidim)
      allocate (workc(workcdim))

      allocate (worki(workidim))

      call maport(plt2ort,ort2plt)

!-----------------------------------------------------------------------
! Determine the DVR weights.
!-----------------------------------------------------------------------
      call getweight(trafo,weight,ort,.true.)

!-----------------------------------------------------------------------
! set up vgdim arrays if nmulpot term present
!-----------------------------------------------------------------------
!     if (nmulpot(1) .gt. 0) then
!        zeig=1
!        do m=1,nmode
!           f=spfdof(1,m)
!           vgdim(f)=zeig
!           zeig=zeig*gdim(f)
!        enddo
!     endif

!-----------------------------------------------------------------------
! interactive plotting
! open file for data to be plotted
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/tmpplot.pl'
      open(iaus,file=filename,status='unknown')

!----------------------------------------------------------------------
! menu defaults
!-----------------------------------------------------------------------
      plotdim=2
      if (lshowpsi) then
         lplcoo2tim=.true.
      else
         lplcoo2=.true.
      endif
      lstate=.true.
      call initplot(filename,plotdim)

!-----------------------------------------------------------------------
! System Hamiltonian provides default PES
!-----------------------------------------------------------------------
      plnham=1

!-----------------------------------------------------------------------
! set plot tasks
!
! ?? CHECK: 5,6 plots WF or WF**2 ??
!-----------------------------------------------------------------------
      cplottask=' '
      if (.not. lshowpsi .and. .not. lrdoper) then
         write(6,'(a)') 'ERROR: Nothing to plot. '
         stop
      endif
      if (lshowpsi .and. lrdoper) then
         cplottask(1)='adiabatic pes'
         cplottask(2)='diabatic pes '
         cplottask(3)='adiabatic coupling '
         cplottask(4)='diabatic coupling '
         if (dentype .eq. 0) then
            cplottask(5)='adiabatic wavefunction (abs^2, cut)'
            cplottask(6)='diabatic wavefunction (abs^2, cut)'
         endif
         cplottask(7)='diabatic reduced density '
         plottask=7
         if (nddstate .eq. 1) then
            cplottask(2)='pes'
            if (dentype .eq. 0) cplottask(6)='wavefunction (abs^2, cut)'
            cplottask(7)='reduced density '
            plottask=7
         endif
         if (dentype .eq. 0) then
            cplottask(8)='wavefunction (argument, cut)'
            cplottask(9)='wavefunction (real part, cut)'
            cplottask(10)='wavefunction (imag part, cut)'
         endif
         if(lpop2all) plottask=6
      else if (lshowpsi) then
         cplottask(5)='adiabatic wavefunction (pes file required!)'
         cplottask(6)='diabatic wavefunction (abs^2, cut)'
         cplottask(7)='diabatic reduced density '
         if (nddstate .eq. 1) then
            cplottask(6)='wavefunction (abs^2, cut)'
            cplottask(7)='reduced density '
         endif
         cplottask(8)='wavefunction (argument, cut)'
         cplottask(9)='wavefunction (real part, cut)'
         cplottask(10)='wavefunction (imag part, cut)'
         plottask=7
         if(lpop2all) plottask=6
      else if (lrediab .and. nddstate .gt. 1) then
         cplottask(2)='adiabatic pes'
         cplottask(4)='adiabatic coupling '
         plottask=2
      else if (nddstate .gt. 1) then
         cplottask(1)='adiabatic pes'
         cplottask(2)='diabatic pes '
         cplottask(3)='adiabatic coupling '
         cplottask(4)='diabatic coupling '
         plottask=1
      else 
         cplottask(1)='pes'
         plottask=1
      endif
      do i=1,nplottask
         cptlen(i)=slen(cplottask(i)) 
      enddo

!-----------------------------------------------------------------------
! Determine the number of data-sets (times).
!-----------------------------------------------------------------------
      if (plottask .ge. 5 .and. plottask .le. 10) then
         rewind(ipsi)
         if(lshowrst) then
            ndata=1
            nplots=1
            pltime(ndata)=0.0_dop
         elseif(lshowpsi) then
            time=(tinit-out2)/fs
            ndata=0
            rewind(ipsi)
            psicount=-nskip
            read(ipsi) filever(ipsi)
            if (filever(ipsi) .lt. 11.0_dop) then
               chkdvr=0
               chkgrd=0
               chkpsi=0
               chkdat=0
               call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
            endif

 12         continue
            call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)

            if (lend) goto 13
            psicount=psicount+1
            time=time+out2/fs
            if (lgwp) then
               call gwp_param(psi,aGmat,gfgwp,aigwp,zgwp,qcenter,1)
               call gwp_cmom(psi,pcenter)
               write(itmpin,*) time,(qcenter(i),i=1,qcentdim),&
                   (pcenter(i),i=1,qcentdim)
            endif 
            if(psicount.le.0) goto 12 
            ndata=ndata+1
            pltime(ndata)=time
            do i = 1,step-1
               call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
               if (lend) go to 13
               time=time+out2/fs
            enddo
            if(psicount.lt.ncut)  goto 12
 13         nplots = ndata

            call timebnds(pltime,ndata)
            write(6,'(a,i6,a,2f11.3,a)') 'There are',nplots,&
             ' WFs read in. Times:',bound(1,ftime),bound(2,ftime),' fs' 

         endif
         call timebnds(pltime,ndata)
      endif
      nplots0 = nplots

!-----------------------------------------------------------------------
! calculate initial mode bounds
!-----------------------------------------------------------------------
      call inmodebnds(ort,plotdim)

!-----------------------------------------------------------------------
! set default PES unit to eV
!-----------------------------------------------------------------------
      if( zaxunit0 .eq. 0.0_dop ) then    ! i.e. not set by option.
         if (plottask .ge. 1 .and. plottask .le. 4) then
            zaxunitlab='ev'
            zaxunit = 1.0_dop
            call keyunits(zaxunit,zaxunitlab,ierr)
         else
            zaxunitlab='au'
            zaxunit = 1.0_dop
         endif
      else
         zaxunit=zaxunit0
         zaxunitlab=zlab0
      endif

!-----------------------------------------------------------------------
! call menu and select task
!-----------------------------------------------------------------------
    1 continue
      if (plottask .ge. 1 .and. plottask .le. 4) then
         lplcoo2tim=.false.
         lplcoo2=.true.
      else if (plottask .ge. 5 .and. plottask .le. 10) then
         lplcoo2tim=.true.
         lplcoo2=.false.
      endif
      if(.not.lredo) call plotmenu(task,plottask,plotdim)

!-----------------------------------------------------------------------
! calculate surface if required
!-----------------------------------------------------------------------
      if (lrecalc.and.lredo) then
         lrecalc  = .false.
         lredo    = .false.
         lrebound = .true.

         if (plottask .ge. 5 .and. plottask .le. 10) then
            if (plottask .eq. 7) write(6,'(a)') &
                 'Computing the densities. This may take a while.'
            if(lshowrst) then
               rewind(irst)
               read(irst) filever(irst)
               if (filever(irst) .lt. 11.0_dop) then
                  chkdvr=0
                  chkgrd=0
                  chkpsi=0
                  chkdat=0
                  call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkdat)
                  read (irst,err=999) time
                  if (citype .gt. 0) then
                     read(irst,err=999) (jindx(i),i=1,adim)
                  endif
                  read (irst,err=999) (psi(i),i=1,dgldim)
                  if (lgwp) then
                     read(irst,err=999) (agmat(i),i=1,agmatdim)
                     call gwp_param(psi,aGmat,gfgwp,aigwp,zgwp,qcenter,1)
                     write(itmpin,*) (qcenter(i),i=1,qcentdim)
                     call gwp_ovl(gs2,gfgwp)
                  endif
               else
                  do
   5                 continue
                     read(irst,iostat=chkerr(1),err=5,end=100) rdtag
                     if (rdtag(1:1) .ne. '<') cycle

                     ilbl=2
                     jlbl=index(rdtag,'>')-1
!-----------------------------------------------------------------------
! read wavefunction
!-----------------------------------------------------------------------
                     if (rdtag(ilbl:jlbl) .eq. 'time') then
                        read (irst,err=999) time
                     else if (rdtag(ilbl:jlbl) .eq. 'jindx') then
                        read(irst,err=999) (jindx(i),i=1,adim)
                     else if (rdtag(ilbl:jlbl) .eq. 'psi') then
                        read (irst,err=999) (psi(i),i=1,dgldim)
                     else if (rdtag(ilbl:jlbl) .eq. 'end restart file') then
                        exit

                     endif
                  enddo
  100             continue

               endif

               time=time/fs 
               ndata=1
               pltime(ndata)=time
               call getsrf(srf,wfunc(1,1,1,ndata),hops,psi,&
                    plottask,weight,trafo,exphin,rueck,fftfak,&
                    plt2ort,ort2plt,ort,agmat,gs2,trajst,ncutstruct)
            elseif(lshowpsi) then
               time=(tinit-out2)/fs
               ndata=0
               rewind(ipsi)
               read(ipsi) filever(ipsi)
               if (filever(ipsi) .lt. 11.0_dop) then
                  chkdvr=0
                  chkgrd=0
                  chkpsi=0
                  chkdat=0
                  call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
               endif
               do i = 1, nskip
                  call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
                  if (lend) go to 77
                  time=time+out2/fs
               enddo

   10          continue
               call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
               if (lend) go to 77
               time=time+out2/fs
               if(time.lt.(bound(1,ftime)-tiny)) goto 10 
               if(time.gt.(bound(2,ftime)+tiny)) goto 77
               ndata=ndata+1
               pltime(ndata)=time

               if (lgwp) then
                  call gwp_param(psi,aGmat,gfgwp,aigwp,zgwp,qcenter,1)
                  call gwp_cmom(psi,pcenter)
                  write(itmpin,*) time,(qcenter(i),i=1,qcentdim),&
                      (pcenter(i),i=1,qcentdim)
                  call gwp_ovl(gs2,gfgwp)
               endif

               call getsrf(srf,wfunc(1,1,1,ndata),hops,psi,&
                    plottask,weight,trafo,exphin,rueck,fftfak,&
                    plt2ort,ort2plt,ort,agmat,gs2,trajst,ncutstruct)
               do i = 1,step-1
                  call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
                  if (lend) go to 77
                  time=time+out2/fs
               enddo
               if( ndata.lt.nplots ) goto 10

   77          continue
            endif
            nplots=ndata
            if (plottask .eq. 7) write(6,'(a,i5,a)') &
                  'Densities done for',ndata,' WFs.'  
         else
            call getsrf(srf,wfunc,hops,psi,plottask,weight,&
                 trafo,exphin,rueck,fftfak,plt2ort,ort2plt,ort,&
                 agmat,gs2,trajst,ncutstruct)
         endif

         f1=axis(1)
         f2=axis(2)
         if (f2 .eq. 0) f2=f1
         call maport1(plort1,plort2,plgdim1,plgdim2,ort(zort(f1)),&
              ort(zort(f2)),plt2ort(zort(f1)),plt2ort(zort(f2)),&
              maxplgrid,gdim(f1),gdim(f2),ncutstruct)

      endif

!-----------------------------------------------------------------------
! get data bounds
!-----------------------------------------------------------------------
      if (lrebound) then
         f1=axis(1)
         f2=axis(2)
         if (f2 .eq. 0) f2=f1
         call maport1(plort1,plort2,plgdim1,plgdim2,ort(zort(f1)),&
              ort(zort(f2)),plt2ort(zort(f1)),plt2ort(zort(f2)),&
              maxplgrid,gdim(f1),gdim(f2),ncutstruct)

         if (f2 .eq. f1) then
            ydim=1
         else
            ydim=plgdim2
         endif
         if (plottask .ge. 1 .and. plottask .le. 2) then
            nsets=nplstates
         else if (plottask .ge. 3 .and. plottask .le. 4) then
            nsets=1
         else if (plottask .ge. 5 .and. plottask .le. 10) then
            nsets=nplstates
         endif

         if (plottask .ge. 1 .and. plottask .le. 4) then
            nplots=1
            call databnds(srf,plgdim1,ydim,nsets,nplots,maxplgrid,&
                 maxplgrid,nddstate,1)
         else if (plottask .ge. 5 .and. plottask .le. 10) then
            nplots=ndata
            call timebnds(pltime,ndata)
            call databnds(wfunc,plgdim1,ydim,nsets,nplots,maxplgrid,&
                 maxplgrid,nddstate,ndata)
         endif
      endif

!-----------------------------------------------------------------------
! write title and axis labels
!-----------------------------------------------------------------------
      plttitle=' '

      do m=1,nmode
         do n=1,min(nspfdof(m),4)
            f=spfdof(1,m)
            if (icut(f).gt.0 ) then
               write(line,'(2a,e11.4)')&
               modelabel(f)(1:slen(modelabel(f))),'=',rcut(f)/axunit(f)

               call rmblnk(line(1:slen(line)))
               if (slen(plttitle)+slen(line) .le. c5-2) then
                  write(tmptitle,'(a,2x,a)')&
                     plttitle(1:slen(plttitle)),line(1:slen(line))
                  plttitle = tmptitle
               endif
            endif
         enddo
      enddo

      f1=axis(1)
      call mode_lab_corr(modelabel(f1),moderep)
      f2=axis(2)
      write(axislab(1),'(4a)') moderep(1:slen(moderep)),&
            ' [',axunitlab(f1)(1:slen(axunitlab(f1))),'] '

      if (f2 .gt. 0) then
         call mode_lab_corr(modelabel(f2),moderep)
         write(axislab(2),'(4a)') moderep(1:slen(moderep)),&
            ' [',axunitlab(f2)(1:slen(axunitlab(f2))),'] '
      else
         axislab(2)=' '
      endif

!-----------------------------------------------------------------------
! do task
! TASK=0 STOP  else return to menu
!-----------------------------------------------------------------------
  900 continue
      if (task .gt. 0) then
         f1=axis(1)
         f2=axis(2)
         if (f2 .eq. 0) f2=f1
         call maport1(plort1,plort2,plgdim1,plgdim2,ort(zort(f1)),&
              ort(zort(f2)),plt2ort(zort(f1)),plt2ort(zort(f2)),&
              maxplgrid,gdim(f1),gdim(f2),ncutstruct)

         if (plottask .ge. 1 .and. plottask .le. 4) then
            write(axislab(3),'(3a)') &
              'V [',zaxunitlab(1:slen(zaxunitlab)),'] '
            nplots=1
            call dotask(task,plottask,plotdim,plort1,plort2,&
                 srf,plgdim1,plgdim2,nsets,nplots,maxplgrid,maxplgrid,&
                 nddstate,1,ort,ortdim,pltime,1,edum,1,&
                 rdum,1,1,1,1,ort2plt)
         else if (plottask .ge. 5 .and. plottask .le. 10) then
            if(plottask.eq.5) axislab(3)='WF (ad)'
            if(plottask.eq.6) axislab(3)='WF '
            if(plottask.eq.7) axislab(3)='Density '
            if(plottask.eq.8) axislab(3)='WF (arg) '
            if(plottask.eq.9) axislab(3)='WF (Re) '
            if(plottask.eq.10) axislab(3)='WF (Im) '
            call dotask(task,plottask,plotdim,plort1,plort2,&
                 wfunc,plgdim1,plgdim2,nsets,nplots,maxplgrid,maxplgrid,&
                 nddstate,ndata,ort,ortdim,pltime,ndata,&
                 edum,1,rdum,1,1,1,1,ort2plt)

         endif
         if (ftime .gt. 0) then
            if(task.eq.30) nplots = ibound(2,ftime)-ibound(1,ftime)+1
            if(task.eq.285) nplots = nplots0
         endif

!-----------------------------------------------------------------------
! check that combination is allowed
!-----------------------------------------------------------------------
         if (plottask .eq. 3 .or. plottask .eq. 4) then
            if (nplstates .ne. 2) then
               write(6,'(a)') &
                  'Must first select 2 states to plot coupling'
               task = 60
               go to 900
            endif
         endif

         goto 1
      endif

      if (allocated(pltime)) deallocate(pltime)
      if (allocated(wfunc)) deallocate(wfunc)
      if (allocated(srf)) deallocate(srf)
      if (allocated(plort1)) deallocate(plort1)
      if (allocated(plort2)) deallocate(plort2)

      return

  999 routine='plotsys  (showsys.F)'
      message='Error reading restart file'
      call errormsg

      end subroutine plotsys

!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------

      subroutine showsysinput

use global
use aglobal
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use plot

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: iarg,ierr,i,psilaenge
      logical(kind=4)         :: lnopes
      character(len=80)       :: buf

      operfile  = ' '
      psifile  = ' '
      name  = './'
      fileout = 'surface.pl'
      iarg = myiargc() 
      ncut = 999999
      nskip=0
      zaxunit = 0.0_dop   ! Used as switch, whether unit set by option.
      ddpath = '..'

      lshowpsi=.true.
      lshowrst=.false.
      lpop2   =.false.
      lpop2all=.false.
      lnopes = .false.
      ldbsave = .true.
      
      i = 0
 100  i=i+1
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if(buf .eq. '-i') then
            i = i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf(1:3) .eq. '-f ') then
            i = i+1
            call mygetarg(i,buf)
            psifile = buf
         else if( buf(1:3) .eq. '-p ') then
            i = i+1
            call mygetarg(i,buf)
            operfile = buf
         else if( buf(1:3) .eq. '-n ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ncut
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -n'
               stop
            endif
         else if( buf(1:5) .eq. '-skip ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nskip
            if (nskip .lt. 0) then
               write(6,'(/a)') &
               'Argument of -skip must be positive integer'
               stop
            endif
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -skip'
               stop
            endif
         else if( buf(1:5) .eq. '-step ') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (step .lt. 0) then
               write(6,'(/a)')&
               'Argument of -step must be positive integer'
               stop
            endif
            if (ierr .eq. 1) then
               write(6,'(/a)') 'Cannot read argument of -step'
               stop
            endif
         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
            laeout=index(fileout,' ')-1
            fileout = fileout(1:laeout)//'.pl '
         else if( buf(1:3) .eq. '-D ' ) then
            i = i+1
            call mygetarg(i,buf)
            dirout = buf
            laeoutd=index(dirout,' ')-1
            call abspath(dirout,laeoutd)

         else if( buf(1:3) .eq. '-u ' ) then
            i = i+1
            call mygetarg(i,buf)
            zaxunitlab = buf(1:c1)
            read(buf,*,err=91) zaxunit
            zaxunit=1.0_dop/zaxunit
            ierr=0
            goto 92
   91       zaxunit = 1.0_dop
            call keyunits(zaxunit,buf,ierr)
            if (ierr .gt. 0) then
               write(6,'(2a)') 'ERROR : unknown unit: ',buf(1:16)
               stop
            endif
   92       continue
         
         else if( buf .eq. '-pes') then
            lshowpsi=.false.
            ncut=1
         else if( buf .eq. '-nopes') then
            lnopes=.true.
         else if( buf .eq. '-ddraw') then
            lshowpsi=.false.
! by setting lrediab to true the raw data adiabatic DB files are
! read in place of the usual diabatic ones
            lrediab = .true.
         else if( buf(1:9) .eq. '-shepord ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*) dbinterord
         else if( buf .eq. '-ddlocal') then
            ldblocal=.true.
         else if( buf .eq. '-rst') then
            lshowrst=.true.
            ncut=1
         else if( buf .eq. '-pop2') then
            lpop2=.true.
            lnw=.true.
         else if( buf .eq. '-pop2all') then
            lpop2all=.true.
            lpop2=.true.
            lnw=.true.
         else if( buf .eq. '-nw'  ) then
            lnw=.true.
         else if( buf(1:6) .eq. '-nofs ' ) then
            fs = 1.0_dop
         else if( buf(1:7) .eq. '-plgrd ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*) maxplgrid

         else if( buf(1:4) .eq. '-DB ' ) then
            i = i+1
            call mygetarg(i,buf)
            ddpath = buf

         else if( buf(1:4) .eq. '-DBG ' ) then
            i = i+1
            call mygetarg(i,buf)
            ddpath = buf
            lgausspot = .true.

         else if( buf .eq. '-gnupl4 ') then
            lgnu4 = .true.                 ! gnuplot ver < 5

         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(/1x,a)')&
       'Purpose: Enables 2D plotting of the wavefunction or PES.'
            write(6,'(1x,a,2(/21x,a))')&
       'Usage:   showsys [-i -D -f -p -o -pes -nopes -ddraw -n -skip -step',&
                           '-rst -nw -nofs -u -pop2 -pop2all -plgrd -DB',&
                           '-shepord -ddlocal -gnupl4 -ver -h -?]'
            write(6,'(/,a)') 'Options : '
            write(6,'(a)') ' -i DIR  : data is stored in directory DIR'
            write(6,'(2a,/,9x,a)')&
                     ' -f FILE : the wavefunction is read '&
                    ,'from FILE rather than from ./psi',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(2a)')&
                     ' -n num  : only the first num WFs are read '&
                    ,'from psi-file.'
            write(6,'(a)')&
                     ' -skip m : the first m WFs are skipped.'
            write(6,'(2a,/,9x,a)') &
                     ' -p FILE : the PES is read '&
                    ,'from FILE rather than from ./oper',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a)') &
                     ' -o FILE : The ouput is written to file ',&
                     'FILE.pl rather than to ./surface.pl',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,11x,a/,11x,a/,11x,a)')&
            ' -D DIR  : The output is written to directory DIR.',&
            ' This is useful,',&
            'if one has no right to write to the name directory.',&
            'The directory DIR must exist.',&
            'The string DIR may be a relative or a full path-name.'
            write(6,'(2a)') &
                     ' -pes    : only the OPER file is read,  '&
                    ,'i.e. no wavefunction plotting.' 
            write(6,'(2a)') &
                     ' -nopes  : the OPER file is not read, '&
                    ,'no potential plotting, no dia->ad trafo.'
            write(6,'(2a)') &
                     ' -ddraw    : the raw adiabatic PES from a DD,  '&
                    ,'calculation is plotted.' 
            write(6,'(2a)')&
                     ' -rst    : the restart file is read, '&
                    ,'rather than the psi file.'
            write(6,'(a)')&
                     ' -n num  : only num WFs are processed.'
            write(6,'(a)')&
                     ' -skip m : the first m WFs are skipped.'
            write(6,'(2a)')&
                     ' -step step : only every step''s WF '&
                    ,'will be processed.'
            write(6,'(2a)')&
                     ' -nw     : No weights are employed.',&
                     ' Plot the "naked" DVR populations. (WF only).'
            write(6,'(2a)')&
                     ' -nofs   : No transformation to fs.',&
                     " Use when 'time-not-fs' was set in mctdh."
            write(6,'(2a)')&
                     ' -u UNIT : The energy unit UNIT is applied.',&
                     ' (default for pes: eV).'
            write(6,'(2a,/10x,2a,/10x,a)')&
                 ' -pop2   : the basis set occupations are ',&
                 'plotted rather than grid',&
                 ' populations. For FFT this implies momentum ',&
                 'space representation.',&
                 ' Note: -pop2 sets -nw.'
            write(6,'(2a,/10x,2a)')&
                 ' -pop2all: Similar to -pop2, but now all DOFs ',&
                 'are transformed to second population.',&
                 ' This is only useful for cuts.',&
                 ' Note: -pop2all sets -pop2 and -nw.'
            write(6,'(a)')&
                 ' -plgrd MAXPLGRID : The maximum number of points for a grid.'
            write(6,'(a)')&
                 ' -DB PATH : The path to a direct dynamics DB.'
            write(6,'(a)')&
                 ' -shepord I : Order I will be used for the Shephard interpolation.'
            write(6,'(a)')&
                 ' -ddlocal : The local rather than interpolated energy is plotted.'
            write(6,'(a)')&
               ' -gnupl4  : Use gnuplot version < 5 for plotting '
            write(6,'(a)')&
                     ' -ver    : Version information about the program.'
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(4(/2x,2a),/2x,a,3(/2x,2a),/2x,a,/2x,2a,/2x,a,/2x,2a,/2a)')&
            'The program is menu driven. Just type "showsys" or ',&
            '"showsys -pop2"',&
            'or "showsys -pes" or "showsys -rst" and then follow ', &
            'the menu.','If there are more WFs on psi file than you ',&
            'want to inspect, use the','options -n, -skip, or -step. ',&
            'Example: "showsys -skip 10 -n 2" will plot',&
            'the WFs no. 11 and 12 (i.e. t=10 and 11 if tpsi=1fs),',&
            'and "showsys -step 10 -n 2" will plot the WFs ',&
            'no. 1 and 11.',&
            'If one wants to plot a potential one should use ',&
            '"showsys -pes",','because otherwise the WF is read, ',&
            'which consumes a large amount of memory.', &
        'NB1: The option -step may also be set by the menu-point 285.',&
            '     The WFs to be processed may also be selected by ',&
            'setting time bounds','     (menu-point 30).', &
            'NB2: The option -pop2 and -pop2all are',&
            ' ignored for phifbr,sphfbr,k,external bases.',&
            '-----------------------------',&
            '---------------------------------------------------'

            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         goto 100
      end if
      
  200 continue
      nopt=i-1

!-----------------------------------------------------------------------
! check options
!-----------------------------------------------------------------------
      if(lshowrst .and. .not.lshowpsi) then
         routine='showsysinput'
         message='One must not give "-pes" AND "-rst" options'
         call errormsg
      endif
      laenge=index(name,' ')-1
      call abspath(name,laenge)

!-----------------------------------------------------------------------
! set up file names
!-----------------------------------------------------------------------
      if(.not.lnopes) then
         if (operfile .eq. ' ') operfile=name(1:laenge)//'/oper'
         opflaenge=index(operfile,' ')-1
         call abspath(operfile,opflaenge)
         inquire(file=operfile(1:opflaenge),exist=lrdoper)
      endif

      if (lshowrst) then
         if (psifile .eq. ' ') psifile=name(1:laenge)//'/restart'
         psilaenge=index(psifile,' ')-1
         call abspath(psifile,psilaenge)
         inquire(file=psifile(1:psilaenge),exist=lshowpsi)
         if(.not.lshowpsi) then
            routine='showsysinput'
            message='There is no RESTART file: '//psifile(1:psilaenge)
            call errormsg
         endif
      elseif (lshowpsi) then
         if (psifile .eq. ' ') psifile=name(1:laenge)//'/psi'
         psilaenge=index(psifile,' ')-1
         call abspath(psifile,psilaenge)
         inquire(file=psifile(1:psilaenge),exist=lshowpsi)
      endif

      dname=name
      dlaenge=index(dname,' ')-1
      call abspath(dname,dlaenge)

      laeout=index(fileout,' ')-1
      call abspath(fileout,laeout)

      
      return
      end subroutine showsysinput

!#######################################################################

      subroutine getsrf(srf,wfunc,hops,psi,plottask,weight,&
                        trafo,exphin,rueck,fftfak,plt2ort,ort2plt,&
                        ort,agmat,gs2,trajst,ncutstruct)

use global
use aglobal
use maxv, only: maxdim,maxsta
use griddatmod
use operdef
use psidef
use compdatmod
use plot

use psiutil, only: showpop2,calcpsi
use operutil, only: calcd2d
use potevalmod, only: calcdiab

      implicit none

      integer(long), intent(in) :: plottask
      integer(long)             :: f,zeig,m,g,s,s1,st,n,g1,f1,f2,&
                                   gg,gg1,ilb1,iub1,ilb2,iub2,ilbl,ierr
      integer(long)             :: gda,gdb,ff,m1,m2,md,nd,vd
      integer(long), dimension(ortdim), intent(in)     :: plt2ort,ort2plt
      integer(long), dimension(maxdim)                 :: igpoint
      integer(long), dimension(fftdim), intent(in)     :: fftfak
      integer(long), dimension(gwpinfodim), intent(in) :: trajst

      real(dop), dimension(2)                    :: trbin
      real(dop), dimension(maxdim)               :: gpoint
      real(dop), dimension(maxsta)               :: pesad,arg
      real(dop), dimension(maxsta,maxsta)        :: pesdia
      real(dop), dimension(maxsta)               :: psiad,psidia
      real(dop), dimension(ortdim), intent(in)   :: weight,ort
      real(dop), dimension(hopsdim), intent(in)  :: hops
      real(dop), dimension(dvrdim), intent(in)   :: trafo
      real(dop), dimension(maxplgrid,maxplgrid,nddstate), intent(out) :: srf
      real(dop), dimension(maxplgrid,maxplgrid,nddstate), intent(out) :: wfunc

      integer(long), intent(out)                 :: ncutstruct
      integer(long)                              :: natm1
      integer(long), dimension(:,:), allocatable :: icutcoo
      real(dop), dimension(:,:), allocatable     :: cutcoo
      real(dop), dimension(:,:), allocatable     :: tmpcutcoo
      real(dop)                                  :: x,y,z
      real(dop)                                  :: adml
      character(len=20)                          :: string
      character(len=3)                           :: atnm

      complex(dop), dimension(dgldim), intent(inout)    :: psi
      complex(dop), dimension(expdim), intent(in)       :: exphin
      complex(dop), dimension(fftdim), intent(in)       :: rueck
      complex(dop), dimension(agmatdim), intent(in)     :: agmat
      complex(dop), dimension(gs2matdim), intent(in)    :: gs2
      complex(dop), dimension(maxsta,maxsta)            :: rotmat
      complex(dop), dimension(maxsta)                   :: psipnt

      logical(kind=4) :: lddtrans0, ldd0, swap, lbohr
      integer(long)   :: ndof0

      if(lrunml .and. (plottask.eq.3 .or. plottask.eq.4)) then
         routine='showsys'
         message= &
          'wavefunction cuts not implemented for ML (but densities are)'
         call errormsg
      endif

! initialise variables
      gg = 0
      gg1 = 0

!      call initvxi(igpoint,ndof)
!      gpoint=0.0_dop
!      do m=1,nmode
!         do n=1,nspfdof(m)
!            f=spfdof(n,m)
!            if (icut(f).gt.0) then
!               igpoint(f)=icut(f)
!               gpoint(f) = ort(zort(f)-1+icut(f))
!            endif
!         enddo
!      enddo
! save original lddtrans (is reset when a cut file is used)
      lddtrans0 = lddtrans
      ldd0 = ldd
      ndof0 = ndof
! test
!      if(lgausspot.or.lddmctdh)then
!         do f = 1,ndof
!            if(f.eq.feb)cycle
!            if(basis(f).eq.19)&
!                 gpoint(f) = ort(zort(f))
!         enddo
!      endif
! test

      if (lcutfile) then
! do not transform DD as cut file contains cartesians
         lddtrans = .false.
         ldd = .true.

         if (natm .eq. 0) then
! calculation uses explicit MCTDH input rather than cartesian
            ndofddpes = ndof
            natm = ndofddpes/3
         else
            ndof = ndofddpes
         endif
! set bounds for a 1D plot
         ilb1=1
         iub1=maxplgrid
         ilb2=1
         iub2=1
! igpoint not used for DD calcs 
         igpoint=1

         allocate(cutcoo(ndofddpes,maxplgrid))
         allocate(tmpcutcoo(ndofddpes,maxplgrid))
         ilbl = index(cutfile,' ')-1
         open(idump,file=cutfile(1:ilbl),status='old',form='formatted',err=900)
         read(idump,'(a)') string
         lbohr=.false.
         if (string .eq. '#au') lbohr=.true.
         rewind(idump)

         ncutstruct = 0
         do
! gaps between structures
            do 
              read(idump,'(a)',end=20) string
              if (string(1:1) .ne. '!' .and. string(1:1) .ne. '#' .and. string .ne. ' ') exit
            enddo
            backspace(idump)

            read(idump,*,iostat=ierr) natm1
            if (ierr .ne. 0) then
               print*,'Error in reading file. Check Format.'
               stop
            endif
            if (natm .ne. natm1) then
               print*,'Wrong no. of atoms in xyz file'
               print*,'In system :',natm
               print*,'In file   :',natm1
               stop
            endif
            ncutstruct = ncutstruct + 1
! title
            do 
              read(idump,'(a)',end=20) string
              if (string(1:1) .ne. '!' .and. string(1:1) .ne. '#' .and. string .ne. ' ') exit
            enddo
            backspace(idump)
            f1=0
            if (lbohr) then
               do n=1,natm1
                  read(idump,*) atnm,x,y,z
                  tmpcutcoo(f1+1,ncutstruct) = x 
                  tmpcutcoo(f1+2,ncutstruct) = y 
                  tmpcutcoo(f1+3,ncutstruct) = z 
                  f1=f1+3
               enddo
            else
               do n=1,natm1
                  read(idump,*) atnm,x,y,z
                  tmpcutcoo(f1+1,ncutstruct) = x / 0.52917720859d0
                  tmpcutcoo(f1+2,ncutstruct) = y / 0.52917720859d0
                  tmpcutcoo(f1+3,ncutstruct) = z / 0.52917720859d0
                  f1=f1+3
               enddo
            endif
         enddo
   20    continue
         close(idump)
         call intercut(cutcoo,tmpcutcoo,maxplgrid,ncutstruct)
         deallocate(tmpcutcoo)
          
      else if (lqcutfile) then
! set bounds for a 1D plot
         ilb1=1
         iub1=maxplgrid
         ilb2=1
         iub2=1
         f1=ndof

         allocate(icutcoo(maxdim,maxplgrid))
         allocate(cutcoo(maxdim,maxplgrid))
         allocate(tmpcutcoo(maxdim,maxplgrid))
         ilbl = index(cutfile,' ')-1
         open(idump,file=cutfile(1:ilbl),status='old',form='formatted',err=900)

! title
         do 
           read(idump,'(a)',end=25) string
           if (string(1:1) .ne. '!' .and. string(1:1) .ne. '#' &
                .and. string .ne. ' ') exit
         enddo
   25    continue
         backspace(idump)
         do f=1,ndof
            if (f .eq. feb) cycle
            read(idump,*) tmpcutcoo(f,1),tmpcutcoo(f,2) 
         enddo
         close(idump)
         ncutstruct=iub1
         call qintercut(cutcoo,icutcoo,tmpcutcoo,ncutstruct,ort)
         iub1=ncutstruct
         deallocate(tmpcutcoo)
          
      else
         allocate(cutcoo(1,1))

         call initvxi(igpoint,ndof)
         call zerovxd(gpoint,ndof)
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (icut(f).gt.0) then
                  igpoint(f)=icut(f)
                  gpoint(f) = ort(zort(f)-1+icut(f))
               endif
            enddo
         enddo

         f1=axis(1)
         ilb1=ibound(1,f1)
         iub1=ibound(2,f1)
         f2=axis(2)
         if (f2 .gt. 0) then
            ilb2=ibound(1,f2)
            iub2=ibound(2,f2)
         else
            ilb2=1
            iub2=1
         endif
! get bin size for trajectories
         if (ltraj) then
            trbin(1) = ort(zort(f1)+1)- ort(zort(f1))
            if (f2 .gt. 0) trbin(2) = ort(zort(f2)+1)- ort(zort(f2))
         endif
!
! map bounds to plot grid
!
         iub1 = ort2plt(zort(f1)-1+iub1)
         ilb1 = ort2plt(zort(f1)-1+ilb1)
         if (f2 .gt. 0) then
            iub2 = ort2plt(zort(f2)-1+iub2)
            ilb2 = ort2plt(zort(f2)-1+ilb2)
         endif

         if (plottask.ge.5 .and. plottask.le.8) then
            if(lpop2all) then
               lswap = .true.
               do f=1,ndof
                  if(basis(f).gt.0 .and. basis(f).ne.6 .and. &
                     basis(f).ne.8 .and. basis(f).ne.13) &
                     call showpop2(psi,trafo,exphin,rueck,fftfak,f,f)
               end do
            elseif(lpop2) then
               lswap = .true.
               call showpop2(psi,trafo,exphin,rueck,fftfak,f1,f2)
               if(f2.gt.0) &
                 call showpop2(psi,trafo,exphin,rueck,fftfak,f2,f1)
            endif
         endif

      endif

      do g1=ilb2,iub2
         do g=ilb1,iub1
            if (lcutfile) then
               igpoint(f1) = g
               gpoint(1:ndofddpes) = cutcoo(1:ndofddpes,g)
            else if (lqcutfile) then
               igpoint(1:ndof) = icutcoo(1:ndof,g)
               gpoint(1:ndof) = cutcoo(1:ndof,g)
            else
               gg = plt2ort(zort(f1)-1+g)
               igpoint(f1) = gg
               gpoint(f1) = ort(zort(f1)-1+gg)
               if (f2 .gt. 0) then
                  gg1 = plt2ort(zort(f2)-1+g1)
                  igpoint(f2) = gg1
                  gpoint(f2) = ort(zort(f2)-1+gg1)
               endif
            endif

            if (plottask .ge. 1 .and. plottask .le. 4 .or. &
               (plottask .eq. 5 .and. nddstate .gt. 1) .or. &
               (plottask .eq. 6 .and. nddstate .gt. 1) .or. &
               (plottask .eq. 8 .and. nddstate .gt. 1) .or. &
               (plottask .eq. 9 .and. nddstate .gt. 1) .or. &
               (plottask .eq. 10 .and. nddstate .gt. 1)) then
               call calcdiab(hops,pesad,pesdia,rotmat,igpoint,gpoint,plnham)
            endif

            if (plottask .eq. 5 .or. plottask .eq. 6 .or. &
                    (plottask .ge. 8 .and. plottask .le. 10)) then
               zeig=dgldim+1
               call calcpsi(psi,psiad,psidia,arg,rotmat,igpoint,gpoint,&
                    agmat,psipnt,f1,f2,weight)
            endif
            if (plottask .eq. 7) then
!..............Calculate the 2D density.
               if (psitype .eq. 5) then ! ML-MCTDH

                  s=1
                  m1=dofspf(f1)
                  if(f2.eq.0) then
                     m2=m1
                     ff=f1
                  else
                     m2=dofspf(f2)
                     ff=f2
                  endif
                  if(m1.eq.m2 .and. f2.gt.0) then  ! both DOFs in same mode
                     if(vgdim(f1).gt.vgdim(f2)) then
                        swap=.true.  ! swap is true, if f2 is before f1 in mode
                        vd=vgdim(f2)
                        nd=ngdim(f1)
                        gda=gdim(f2)
                        gdb=gdim(f1)
                     else
                        swap=.false.
                        vd=vgdim(f1)
                        nd=ngdim(f2)
                        gda=gdim(f1)
                        gdb=gdim(f2)
                     endif
                     md=subdim(m1)/(vd*nd*gda*gdb)
                     call calcd2d_1ml(psi,psi(zetf(m1,s)),vd,gda,md,gdb,nd, &
                          gdim(f1),gdim(f2),dim(m1,s),adml,swap,lnw, &
                          wfunc,maxgdim,weight(zort(f1)),weight(zort(f2)), &
                          ilb2,iub2,ilb1,iub1,agmat)
                  else
                     call calcd2d_2ml(psi,psi(zetf(m1,s)),psi(zetf(m2,s)), &
                          vgdim(f1),gdim(f1),ngdim(f1),vgdim(ff),gdim(ff), &
                          ngdim(ff),f2,dim(m1,s),dim(m2,s),adml, &
                          lnw,wfunc,maxgdim,weight(zort(f1)), &
                          weight(zort(ff)),ilb2,iub2,ilb1,iub1,agmat)
                  endif ! m1.eq.m2

               else
                  call calcd2d(psi,psidia,f1,f2,igpoint,gpoint,agmat,&
                       gs2,trbin,trajst,weight)
               endif
            endif
            if (plottask .eq. 1) then
               do st=1,nplstates
                  s=plstate(st)
                  srf(g,g1,st)=pesad(s)
               enddo
            elseif (plottask .eq. 2) then
               do st=1,nplstates
                  s=plstate(st)
                  srf(g,g1,st)=pesdia(s,s)
               enddo
            elseif (plottask .eq. 3) then
               print*,'Adiabatic coupling not implemented'
            elseif (plottask .eq. 4) then
               s=plstate(1)
               s1=plstate(2)
               srf(g,g1,1)=pesdia(s,s1)
            elseif (plottask .eq. 5) then
               do st=1,nplstates
                  s=plstate(st)
                  if(lnw) then
                     wfunc(g,g1,st)=psiad(s)
                  else
                     wfunc(g,g1,st)=psiad(s)/weight(zort(f1)-1+gg)
                     if (f2 .gt. 0) then
                        wfunc(g,g1,st)=wfunc(g,g1,st)/&
                             weight(zort(f2)-1+gg1)
                     endif
                  endif
               enddo
            elseif (plottask .eq. 6) then
               do st=1,nplstates
                  s=plstate(st)
                  if(lnw) then
                     wfunc(g,g1,st)=psidia(s)
                  else
                     wfunc(g,g1,st)=psidia(s)/weight(zort(f1)-1+gg)
                     if (f2 .gt. 0) then
                        wfunc(g,g1,st)=wfunc(g,g1,st)/&
                             weight(zort(f2)-1+gg1)
                     endif
                  endif
               enddo
            elseif (plottask .eq. 7) then
               do st=1,nplstates
                  s=plstate(st)
                  if(lnw) then
                     wfunc(g,g1,st)=psidia(s)
                  else
                     wfunc(g,g1,st)=psidia(s)/weight(zort(f1)-1+gg)
                     if (f2 .gt. 0) then
                        wfunc(g,g1,st)=wfunc(g,g1,st)/&
                             weight(zort(f2)-1+gg1)
                     endif
                  endif
               enddo
            elseif (plottask .eq. 8) then
               do st=1,nplstates
                  s=plstate(st)
                  if(lnw) then
                     wfunc(g,g1,st)=arg(s)
                  else
                     wfunc(g,g1,st)=arg(s)/weight(zort(f1)-1+gg)
                     if (f2 .gt. 0) then
                        wfunc(g,g1,st)=wfunc(g,g1,st)/&
                             weight(zort(f2)-1+gg1)
                     endif
                  endif
               enddo
            elseif (plottask .eq. 9) then
               do st=1,nplstates
                  s=plstate(st)
                  if(lnw) then
                     wfunc(g,g1,st)=dble(psipnt(s))
                  else
                     wfunc(g,g1,st)=dble(psipnt(s))/weight(zort(f1)-1+gg)
                     if (f2 .gt. 0) then
                        wfunc(g,g1,st)=wfunc(g,g1,st)/&
                             weight(zort(f2)-1+gg1)
                     endif
                  endif
               enddo
            elseif (plottask .eq. 10) then
               do st=1,nplstates
                  s=plstate(st)
                  if(lnw) then
                     wfunc(g,g1,st)=dimag(psipnt(s))
                  else
                     wfunc(g,g1,st)=dimag(psipnt(s))/weight(zort(f1)-1+gg)
                     if (f2 .gt. 0) then
                        wfunc(g,g1,st)=wfunc(g,g1,st)/&
                             weight(zort(f2)-1+gg1)
                     endif
                  endif
               enddo
            endif

         enddo
      enddo

      if (allocated(cutcoo)) deallocate(cutcoo)
      if (allocated(icutcoo)) deallocate(icutcoo)
      lddtrans = lddtrans0
      ldd = ldd0
      ndof = ndof0

      return
  900 write(6,'(a)') 'ERROR opening cut file: ',cutfile(1:ilbl)
      stop
      end subroutine getsrf

!#######################################################################
!
! if plotting raw adiabatic data, copy data from adgra to gra etc.
!
!#######################################################################

      subroutine getraw

      implicit none

      integer(long)  :: i,f,f1,s,s1,kdx

      dbener = 0.0_dop
      dbgrad = 0.0_dop
      dbhess = 0.0_dop
      do i=1,dbnrec
         do s=1,nddstate
            dbener(s,s,i) = dbadener(s,s,i)

            do f=1,ndofddpes
               dbgrad(f,s,s,i) = dbadgrad(f,s,s,i)

               do f1=1,ndofddpes
                  dbhess(f1,f,s,s,i) = dbadhess(f1,f,s,s,i)
               enddo
            enddo
         enddo
! add nacts to gra
         if (ldiabdb) then
            do s = 1,nddstate-1
               do s1 = s+1,nddstate
                  kdx = (s1-1)*(s1-2)/2+s
                  dbgrad(:,s,s1,i) = dbdercp(:,kdx,i)
                  dbgrad(:,s1,s,i) = dbgrad(:,s,s1,i)
               enddo
            enddo
         endif

      enddo

      return
      end subroutine getraw

!#######################################################################
!
! interpolate betwwen structures in cut file
!
!#######################################################################

      subroutine intercut(cutcoo,tmpcutcoo,maxplgrid,ncutstruct)

      implicit none

      integer(long), intent(in)   :: maxplgrid,ncutstruct
      integer(long)               :: ndiv,npntdiv,npnti,npntf,itmp,&
                                     i,j,nd
      real(dop), dimension(ndofddpes,maxplgrid) :: cutcoo,tmpcutcoo
      real(dop), dimension(ndofddpes)           :: xvec

      ndiv = ncutstruct+1
      npntdiv = (maxplgrid - ncutstruct)/ndiv
      itmp = mod((maxplgrid - ncutstruct),ndiv)
      if (mod(itmp,2) .eq. 0) then
         npnti = npntdiv + itmp/2
         npntf = npntdiv + itmp/2
      else
         npnti = npntdiv + itmp/2 + 1
         npntf = npntdiv + itmp/2
      endif

! first division
      nd = 1
      xvec(1:ndofddpes) = tmpcutcoo(1:ndofddpes,nd+1) - tmpcutcoo(1:ndofddpes,nd)
      j = 0
      do i = 1,npnti
         j=j+1
         cutcoo(1:ndofddpes,j) = tmpcutcoo(1:ndofddpes,nd) - xvec(1:ndofddpes)*real(npnti+1-i)/real(npnti)
      enddo
      j=j+1
      cutcoo(1:ndofddpes,j) = tmpcutcoo(1:ndofddpes,nd)

! middle divisions
      do nd = 1, ncutstruct-1
         xvec(1:ndofddpes) = tmpcutcoo(1:ndofddpes,nd+1) - tmpcutcoo(1:ndofddpes,nd)
         do i = 1,npntdiv+1
            j=j+1
            cutcoo(1:ndofddpes,j) = tmpcutcoo(1:ndofddpes,nd) + xvec(1:ndofddpes)*real(i)/real(npntdiv+1)
         enddo
      enddo

! final division
      nd = ncutstruct
      xvec(1:ndofddpes) = tmpcutcoo(1:ndofddpes,nd) - tmpcutcoo(1:ndofddpes,nd-1)
      do i = 1,npntf
         j=j+1
         cutcoo(1:ndofddpes,j) = tmpcutcoo(1:ndofddpes,nd) + xvec(1:ndofddpes)*real(i)/real(npntf)
      enddo

!     open(idump,file='cut_show.xyz',form='formatted')
!     do j=1,maxplgrid
!        write(idump,'(i2)') 6
!        write(idump,'(a,i2)') '# Structure :',j
!        write(idump,'(i2,4x,3f15.9)') 6,cutcoo(1:3,j)*0.52917720859d0
!        write(idump,'(i2,4x,3f15.9)') 7,cutcoo(4:6,j)*0.52917720859d0
!        write(idump,'(i2,4x,3f15.9)') 1,cutcoo(7:9,j)*0.52917720859d0
!        write(idump,'(i2,4x,3f15.9)') 1,cutcoo(10:12,j)*0.52917720859d0
!        write(idump,'(i2,4x,3f15.9)') 8,cutcoo(13:15,j)*0.52917720859d0
!        write(idump,'(i2,4x,3f15.9)') 1,cutcoo(16:18,j)*0.52917720859d0
!        write(idump,'(a,i2)') ' '
!     enddo
!     close(idump)

      return
      end subroutine intercut

!#######################################################################

! iub1 is no. of grid points
      subroutine qintercut(cutcoo,icutcoo,tmpcutcoo,ncutstruct,ort)

      implicit none

      integer(long), intent(inout) :: ncutstruct
      integer(long)                :: ndiv,i,j,npnt,ii,gpnt,f
      integer(long), dimension(maxdim,maxplgrid), intent(out) :: icutcoo
      real(dop), dimension(maxdim,maxplgrid), intent(out)     :: cutcoo
      real(dop), dimension(maxdim,2), intent(in)          :: tmpcutcoo
      real(dop), dimension(maxdim)           :: qincr
      real(dop)                              :: x
      real(dop), dimension(ortdim), intent(in) :: ort
      logical(kind=4), dimension(maxplgrid)    :: ldelete 

! divide grid so that 0 is at centre and point lies 2/3 way along grid
      npnt = (maxplgrid-1)/2
      gpnt = 2*npnt/3
! midpoint
      npnt = npnt+1

      ndiv = gpnt-1

      qincr(:) = (tmpcutcoo(:,2) - tmpcutcoo(:,1))/ real(ndiv)

! start at midpoint
      ii = npnt
      cutcoo(:,ii) = tmpcutcoo(:,1)

      j=0
      do i=ii+1,maxplgrid
         j=j+1
         cutcoo(:,i) = cutcoo(:,ii) + j*qincr(:)
      enddo
      j=0
      do i=ii-1,1,-1
         j=j+1
         cutcoo(:,i) = cutcoo(:,ii) - j*qincr(:)
      enddo

      npnt = ncutstruct
! find nearest grid point to specified coordinates
      do i=1,npnt
         do f=1,ndof
            call find_nearest_gpoint(ort(zort(f)),cutcoo(f,i),&
                 icutcoo(f,i),x,gdim(f))
            cutcoo(f,i)=x
         enddo
      enddo

! mark points if repeats
      ldelete(:) = .true.
      do i=2,npnt
         do f=1,ndof
! ignore dof not in calculation
            if (dofspf(f) .eq. 0) cycle

! ignore modes that are not part of vector
            if (dabs(qincr(f)) .lt. tiny) cycle
     
            if (icutcoo(f,i) .ne. icutcoo(f,i-1)) then
               ldelete(i) = .false.
            endif
         enddo
      enddo

! discard repeats
      ii = 1
      do i=2,npnt
         if (ldelete(i)) then
            do j = ii+1,npnt-1
               icutcoo(:,j) = icutcoo(:,j+1)
               cutcoo(:,j) = cutcoo(:,j+1)
            enddo
         else
            ii = ii+1
         endif
      enddo

      ncutstruct = ii

      return
      end subroutine qintercut

      end program showsys

