!***********************************************************************
!
!                ANALYSE  program STATEPOP
!
! This program reads the state populations from the check file.
!
!     07/01 GW
!
!***********************************************************************
      
      program statepop

use decimal, only: long,dop,sip
use global
use globmemmod
use aglobal
use dglobal
use versions
use dvrdatmod
use griddatmod
use psidef
use channels
use datenmod
use compdatmod

use logdat
use iofile, only: rdchkinfo
use zeigauswmod, only: zeigausw

use module_interface
use maxv, only: maxdim,maxsta
use plot
use rdfilesmod, only: rdchk,lrdchk
use plotutils, only: plotdefs,initplot,timebnds,dotask,plotmenu,databnds
use rdopermod,only: rdoper
use rddvrmod, only: rddvr

      implicit none

      integer(long)                         :: chkdvr,chkgrd,chkdat,chkpsi
      integer(long)                         :: i,ilbl
      character(len=c5)                     :: filename
      character(len=10)                     :: stat
      character(len=80)                     :: host

!-----------------------------------------------------------------------
! Initialize variables and set defaults
!-----------------------------------------------------------------------
      call default
      call adefault

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare files to be read
! set flag to .true. if file is to be read
!
!     lrddvr    DVR file
!     lrdoper   OPER file
!-----------------------------------------------------------------------
      lrddvr=.false.
      lrdoper=.false.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare resources required (other than those needed to read a file)
!
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! ltrafo : FBR/DVR trafo matrices
! lsphfbr : jfbr, mfbr and kinfbr matrices.
!-----------------------------------------------------------------------
      ncomp=0
      lpsi=.false.
      lpsi1=.false.
      lpsigrd=.false.
      ldmat=.true.
      ldmat1=.false.
      lort=.false.
      lgpop=.false.
      ltrafo=.false.
      lsphfbr=.false.

!-----------------------------------------------------------------------
! read input parameters and options
! open files
!-----------------------------------------------------------------------
      call stpopinp(stat)
      
!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='ausw.log',form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! open input and output data files
!
! use standard channels if possible
!  irho : gridpop file
!  ipsi : psi file
!  iaut : auto file
!  ichk : check file
!  iout : output file (opened if not interactive plotting)
!-----------------------------------------------------------------------
      filename=filein(1:laein)
      open(ichk,file=filename,form='unformatted',status='old',err=1000)

     if (.not. linter) then
        if (fileout .eq. 'no') then
            laeout=2
            open(iout,file='/dev/null',form='formatted',status='unknown')
         else
            iout=13
            if (fileout .eq. ' ') fileout=name(1:laenge)//'/spops'
            laeout=index(fileout,' ')-1
            call abspath(fileout,laeout)
            filename=fileout
            if(lovwr) then
               open(iout,file=filename,form='formatted',status='unknown',&
                    err=1030)
            else
               open(iout,file=filename,form='formatted',status='new',&
                    err=1020)
            endif
         endif

! --- WRITE DATE AND MACHINE  ---
         write(iout,'(78a1)')  '#',('-',i=1,77)
         call fpstartup(host)
         write(iout,'(/a2,a78)') '# ',host
      endif

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(ichk)

maxtape = 1000

      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef
      if (ldynspf) call alloc_dynwf

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
! First read file version number
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      filename=filein
      read(ichk,err=999) filever(ichk)
      call rdchkinfo(ichk,chkdvr,chkdat,chkpsi,chkdat)

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! assign work arrays
! NB work arrays must be at end of mc, mr, mi, ml arrays.
!-----------------------------------------------------------------------
      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim


!-----------------------------------------------------------------------
! call analysis routine
!-----------------------------------------------------------------------
if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

      call substpop

deallocate(mc,mr,mi,ml,ms,mf)
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_psidef
      call dealloc_dynwf

      stop
  999 continue
      routine='Statepop'
      message= 'Error reading :'//filename(1:c5-15)
      write(6,'(a)') message
      stop
 1000 continue
      routine='Statepop'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
 1020 continue
      routine='Statepop'
      write(message(1:80),'(a)') &
                'Output file exists. Use option -w to overwrite.'
      ilbl=index(fileout,' ')-1
      write(message(81:c5),'(a)') 'Output file: '//fileout(1:ilbl)
      call errormsg
 1030 continue
      routine='Statepop'
      ilbl=index(fileout,' ')-1
      write(message(1:80),'(a)') 'Cannot open output file: '//fileout(1:ilbl)
      call errormsg

      stop
      contains

!#######################################################################
!***********************************************************************
!
!     HERE IS THE SUBROUTINE TO DO THE ANALYSIS
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! change name of subroutine (or else mutil gets confused)
! provide dummy arguments of arrays needed for analysis
!  e.g. psi to match argument mc(mcpsi)
!***********************************************************************

      subroutine substpop

      implicit none

      integer(long), external                     :: slen
      integer(long)                               :: n,i,s,nddst,s1,s2
      integer(long), dimension(maxdim)            :: idum
      real(dop)                                   :: time
      real(dop), dimension(1)                     :: rdum,edum
      real(dop), dimension(gwpinfodim)            :: ggp
      real(dop), dimension(4,ndof,nstate)         :: qdq
      real(dop), dimension(nddstate)              :: spop
      real(dop), dimension(maxsta)                :: ecorr,etot
      real(dop)                                   :: norm,trace
      real(dop), dimension(d3matdim)              :: dicht3
     
! interactive plot variables
      integer(long)                              :: ndata,plottask,task,plotdim,&
                                                    dimx,dimy,nsets,nplots
      real(dop), dimension(1)                    :: ydum,xdum
      real(dop), dimension(:), allocatable       :: pltime
      real(dop), dimension(:,:), allocatable     :: pldata,pldata1
      integer(long)     :: ierr
      character(len=10) :: cerr

!-----------------------------------------------------------------------
! get no. data sets
!-----------------------------------------------------------------------
      ndata=0
      do 
         call rdchk(ichk,ecorr,etot,spop,dicht3,ggp,qdq,norm,trace)
         if (lend) exit
         ndata=ndata+1
      enddo

! no. of data sets to be plotted
      maxdata = ndata
! space in label arrays for Time etc.
!      maxdim = maxdim+1
! allocate data arrays
      cerr = 'pltime'
      allocate(pltime(maxdata),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'pldata'
      allocate(pldata(maxdata,nddstate),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'pldata1'
      allocate(pldata1(maxdata,nddstate),stat=ierr)
      if (ierr > 0) go to 1000

! set defaults
      idum(1)=0
      plottask = 0

!-----------------------------------------------------------------------
! read check file
!-----------------------------------------------------------------------
      time=tinit
      lrdchk(1)=.true.
      lrdchk(2)=.true.
      rewind(ichk)

      do n=1,ndata
         call rdchk(ichk,ecorr,etot,spop,dicht3,ggp,qdq,norm,trace)
         pltime(n)=time/fs
         do s=1,nddstate
            pldata(n,s)=spop(s)
         enddo
         time=time+out1
      enddo

      close(ichk)

!-----------------------------------------------------------------------
! output data
!-----------------------------------------------------------------------
      if (.not. linter) then
         if(.not.lallcol)then
            nddst = (nddstate / 5)+1
            s1=1
            s2=min(s1+4,nddstate)
            do i=1,nddst
               write(iout,'(a,2i5)') '# State Populations : ',s1,s2
               
               do n=1,ndata
                  write(iout,'(f8.3,6e14.6)') pltime(n),(pldata(n,s),s=s1,s2)
               enddo
               write(iout,*)
               s1=s1+5
               s2=min(s1+4,nddstate)
            enddo
         else
            s1=1
            s2=nddstate
            write(iout,'(a,2i5)') '# State Populations : ',s1,s2
            do n=1,ndata
               write(iout,"(f8.3,99e14.6)") pltime(n),(pldata(n,s),s=s1,s2)
            enddo
            write(iout,*)
         endif
         write(iout,'(78a)') '# ',('-',i=1,7)
         return
      endif

!-----------------------------------------------------------------------
! interactive plotting
! open file for data to be plotted
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/tmpplot.pl'
      open(iaus,file=filename,status='unknown')

      nplottask=1
      call alloc_plot

!-----------------------------------------------------------------------
! zero dummy variables
!-----------------------------------------------------------------------
      xdum(1)=0.0
      ydum(1)=0.0

      call plotdefs

      plotdim=1
      dimx=ndata
      dimy=1
      nsets=1
      nplots=1
      lpltime=.true.
      lstate=.true.
      call initplot(filename,plotdim)
! plot all states by default
      nplstates = nddstate
      do s=1,nddstate
         plstate(s) = s
      enddo

      call timebnds(pltime,ndata)

!-----------------------------------------------------------------------
! set program specific data, e.g. plot tasks
!-----------------------------------------------------------------------
      plottask = 1
      cplottask(1)='State populations'
      do i=1,nplottask
         cptlen(i)=slen(cplottask(i))
      enddo

  250 continue
      call plotmenu(task,plottask,plotdim)

!-----------------------------------------------------------------------
! do task
! TASK=0 STOP  else return to menu
!-----------------------------------------------------------------------
      if (task .gt. 0) then
         if (plottask .ge. 1 .and. plottask .le. 5) then
            nsets=nplstates
            plttitle='State population'
            axislab(1)='Time [fs]'
            axislab(3)='Population'
! select states for plotting
            do n=1,ndata
               do s=1,nplstates
                  s1 = plstate(s)
                  pldata1(n,s)=pldata(n,s1)
               enddo
            enddo
         endif
         if (lrebound) then
            call databnds(pldata1,dimx,dimy,nsets,nplots,&
                    maxdata,1,nddstate,1)
         endif
         lrecalc=.false.
         call dotask(task,plottask,plotdim,pltime,ydum,&
                 pldata1,dimx,dimy,nsets,nplots,&
                 maxdata,1,maxsta,1,xdum,1,pltime,ndata,edum,1,&
                 rdum,1,1,1,1,idum)

         goto 250
      endif

      close(iaus,status='delete')

      return
 1000 write(ilog,'(a)') 'Error allocating memory in Statepop :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop

      end subroutine substpop

!-----------------------------------------------------------------------
!***********************************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! change name of subroutine (or else mutil gets confused)
!-----------------------------------------------------------------------
      subroutine stpopinp(stat)

      implicit none

      integer(long), external       :: myiargc
      integer(long)                 :: iarg,i,ierr
      character(len=80)             :: buf
      character(len=*), intent(out) :: stat

!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! provide default input and output filenames 
!-----------------------------------------------------------------------
      filein = '  '
      name   = './'
      fileout  = ' '

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! set default flags
!
! stat : status of output file   
!-----------------------------------------------------------------------
      stat = 'new'
      lgnu = .false.
      step = 1

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! read options 
! comment out or add options as required
!-----------------------------------------------------------------------
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf .eq. '-o' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf .eq. '-inter' ) then
            linter=.true.
         else if( buf .eq. '-allcol' ) then
            lallcol=.true.
         else if( buf .eq. '-ver' ) then
            call wrversion(6)
            stop

!         else if( buf(1:3) .eq. '-o ' ) then
!            i = i+1
!            call mygetarg(i,buf)
!            fileout = buf
         else if( buf(1:3) .eq. '-n ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:3) .eq. '-w ' ) then
            lovwr=.true.
         else if( buf(1:3) .eq. '-g ' ) then
            lgnu = .true.
         else if( buf .eq. '-gnupl4 ') then
            lgnu4 = .true.                 ! gnuplot ver < 5
         else if( buf(1:3) .eq. '-h ' .or. buf(1:3) .eq. '-? ' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,a,/1x,a)')&
       'Purpose: To extract the state populations from the check file   ',&
       'Usage  : statepop [-f -i -o -inter -ver -g -nofs -w -gnupl4 -h -?] .'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,11x,a/11x,a)')&
              ' -f FILE : the data is read '&
              ,'from file FILE','rather than from ./check'&
              ,'The string FILE may be a relative or full path-name.'
            write(6,'(a)') ' -i DIR  : data is stored in directory DIR'
            write(6,'(2a,/,9x,a,/,9x,a)') &
                     ' -o FILE : The ouput is written to file ',&
                     'FILE rather than to ./spops',&
            '  The string FILE may be a relative or a full path-name.',&
            '  If FILE=no, i.e. -o no, no output file will be written.'
            write(6,'(a)') ' -inter  : enables interactive plotting'
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
!            write(6,'(2a,/,11x,a/,11x,a)') 
!     +               ' -o FILE : The output is written to file ',
!     +               'FILE.pl rather than to ./XXX',
!     +       'The string FILE may be a relative or a full path-name.',
!     +       'If FILE=no, i.e. "-o no", no XXX-file will be opened.'
             write(6,'(2a)')& 
                     ' -g      : GNUplot comand lines are written to ',&
                     'the output file(s).'
!            write(6,'(2a)') ' -n step : Compute the property', 
!     +     ' only every step-th output.'
            write(6,'(2a)') &
                     ' -nofs   : No transformation to fs.', &
                     " Use when 'time-not-fs' was set in mctdh."
            write(6,'(2a)') &
                     ' -w      : An existing property file is ',&
                     'overwritten.'
            write(6,'(a)')&
                     ' -gnupl4  : Use gnuplot version < 5 for plotting '
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      if (iarg .lt. i) goto 200

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! get any compulsory arguments if in argument list
!-----------------------------------------------------------------------
!     call mygetarg(i,filein1)

 200  continue
!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! If filein has not been set, look in name directory.
! Otherwise take name from filein
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         filein=name(1:laenge)//'/check'
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! fill in any arguments not given on command line
!-----------------------------------------------------------------------
!     if (filein1 .eq. ' ') then
!        write(6,*) 'XXX   :'
!        read(5,'(a)') filein1
!        write(6,*)
!     endif

!-----------------------------------------------------------------------
! set status variable to overwrite files
!-----------------------------------------------------------------------
      if (lovwr) stat='unknown'

      return 
      end subroutine stpopinp

!#######################################################################

      end program statepop

