!***********************************************************************
!
!                     ANALYSE ROUTINE STPOPPSI
!
!     This routine calculates the population (norm of WF) of the  
!     electronic directly from the wavefunction in the psi file.
!
!***********************************************************************
!#######################################################################

      program stpoppsi

use decimal, only: long,dop,sip
use maxv, only: maxsta
use global
use globmemmod
use aglobal
use dvrdatmod
use griddatmod
use psidef
use channels
use datenmod
use memdimmod
use logdat
use versions
use gwptrj
use psiutil, only: spfovl

use rddvrmod, only: rddvr
use rdfilesmod, only: rdpsi

use op1lib, only: cpvxz, zerovxd, cpvxi, zerovxi,cpqxz,sum2vxz
use mtlib, only: mtxxzz,mtvxzz
use mmlib, only: mmvxzz,mmaxzz
use xvlib, only: qvaxzz1,qvxxzz,vvaxzz
use mtlib, only: qtxxzz
use rmlib, only: rmhxxxzz
use iorst, only: rstinfo
use iofile, only: rdpsiinfo
use zeigauswmod, only: zeigausw
use rddvrmod, only: dvrinfo

      implicit none

      integer(long)                        :: chkdvr,chkgrd,chkdat,chkpsi,&
                                              chkprp,ilbl
      logical(kind=4)                      :: lerr,linwf

      logical(kind=4), save                :: lrst = .false.
      real(dop),save                       :: tcut = -999.0
      real(dop),save                       :: dt = -999.0

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare files to be read
! set flag to .true. if file is to be read
!
!     lrddvr    DVR file. Set in adpopinfo.
!     lrdoper   OPER file.
!-----------------------------------------------------------------------
      lrddvr=.false.
      lrdoper=.false.
      
!-----------------------------------------------------------------------
! read input options
!-----------------------------------------------------------------------
      call stpoppsiinp
      
!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(ipsi)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare resources required (other than those needed to read a file)
!
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! ltrafo : FBR/DVR trafo matrices
!-----------------------------------------------------------------------
      lpsi=.true.
      lpsi1=.false.
      lpsigrd=.false.
      ldmat1=.false.
      lort=.false.
      ltrafo=.false.
      ldmat=.false.
      lgpop=.false.
      
!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! open input and output data files
!
! use standard channels if possible
!  irho : gridpop file
!  ipsi : psi file
!  iaut : auto file
!  ichk : check file
!  iaus : output file
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
! First read file version number
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      chkprp=0      

      if(lrst) then
         irst=ipsi
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
      else
         read(ipsi,err=999) filever(ipsi)
         call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      endif
      call chksyserr(filein,lerr)
      if (lerr) go to 999

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)
if(allocated(agmat))deallocate(agmat)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))
allocate(agmat(agmatdim))

      call stpopfrompsi(mc(mcpsi),ms(mspsi),mi(mijindx),agmat,&
           mc(mcadgwp),mi(migwpdep))

deallocate(mc,mr,mi,ml,ms,mf,agmat)

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_psidef

      stop
  999 continue
      routine='Stpoppsi'
      message= 'Error reading :'//filein(1:laein)
      call errormsg
      stop

      contains

!#######################################################################

      subroutine stpopfrompsi(psi,spsi,jindx,agmat,adgwp,gwpdep)

      implicit none

      integer(long)       :: m,i,s,e,t1,t2,g
      integer(long)       :: wfnum,zeig,zeig1,zeig2

      integer(long), dimension(adim), intent(out)           :: jindx
      integer(long), dimension(1)                           :: idum
      real(dop)                                             :: tcpu,tsys,ttot,&
                                                               time   
      real(dop), dimension(maxsta)                          :: spop
      complex(sip), dimension(spsidim), intent(out)         :: spsi
      complex(dop), dimension(dgldim), intent(out)          :: psi
      complex(dop), dimension(gsmatdim), intent(out)        :: adgwp
      integer(long), dimension(gwpinfodim),intent(out)      :: gwpdep
      complex(dop), dimension(agmatdim), intent(out)        :: agmat
      complex(dop), dimension(dmatdim)                      :: overlap
      complex(dop), dimension(dgldim)                       :: tmppsi
      complex(dop)                                          :: sum

      character(len=80)                                     :: buffer
      real(dop), external                                   :: fcputime,fsystime

      wfnum = 0

!-----------------------------------------------------------------------
! calculate time step for data
!-----------------------------------------------------------------------
      dt     = out2/fs
      time   = tinit/fs
 
!-----------------------------------------------------------------------
! find point for tcut. 
! ncut marks wanted data set, nskip marks data set before
!-----------------------------------------------------------------------
      if (tcut .gt. 0) then
         ncut=0
    5    continue
         if (abs(tcut-time) .lt. 1.0d-4) then
            ncut = ncut + 1
            nskip = ncut -1
         else if (tcut .lt. time) then
            write(6,*) ' '
            write(6,*) 'ERROR: Time specified not found', tcut
            write(6,*) ' '
            stop
         else 
            time = time+dt
            ncut = ncut + 1
            go to 5
         endif
            
         time   = tinit/fs
      endif

!-----------------------------------------------------------------------
! skip first nskip WFs
!-----------------------------------------------------------------------
      if (.not.lrst) then
         do i=1,nskip
            call rdpsi(ipsi,psi,spsi,jindx,agmat,idum,adgwp,gwpdep)
            if (lend) goto 77
            time = time + dt
            wfnum=wfnum+1
         enddo
      endif
      
!-----------------------------------------------------------------------
! loop over all time steps
!-----------------------------------------------------------------------
   10 continue
   
!-----------------------------------------------------------------------
! read WF
!----------------------------------------------------------------------- 
      if (lrst) then
         irst=ipsi
         read (irst,err=999) time
         time = time/fs
         read (irst,err=999) (psi(g),g=1,dgldim)

      else
         call rdpsi(ipsi,psi,spsi,jindx,agmat,idum,adgwp,gwpdep)
         if (lend) goto 77
      endif

!-----------------------------------------------------------------------
! calculate SPF overlap matrices
!-----------------------------------------------------------------------
      if (.not. lexact .and. .not. lBEL) then
         call spfovl(overlap,psi,psi,agmat,agmat,0)
      endif

!-----------------------------------------------------------------------
! calculate populations
!-----------------------------------------------------------------------       
      spop=0.0_dop
      if (lexact) then
         do s=1,nstate
            call sum2vxz(psi(zetf(1,s)),sum,subdim(1))
            spop(s)=dble(sum)
         enddo

      else if (lgwptrj) then
! gwptrj is single set
         do s=1,ngwptrjst
            do e=1,gwpdim(1,1)
               tmppsi(e) = psi(e)*gwptrjci(s,e)
            enddo

            zeig=1
            zeig1=gwpdim(1,1)+1
            call qtxxzz(overlap(dmat(1,1)),tmppsi(zeig),tmppsi(zeig1), &
                 vdim(1,1),dim(1,1),ndim(1,1))

            call vvaxzz(tmppsi(zeig),tmppsi(zeig1),sum,gwpdim(1,1))
            spop(s)=spop(s)+dble(sum)
         enddo

      else if (lBEL) then
        do s=1,nstate
            zeig=1
            zeig1=block(s)+1
            call cpvxz(psi(zpsi(s)),tmppsi(zeig),block(s))
            do m=1,nmode
              if (gwpm(m)) then
                call cpqxz(gs2(gs2mat(m,s)),overlap(dmat(m,s)),&
                           gwpdim(m,s))
             else
                call mmaxzz(psi(zetf(m,s)),psi(zetf(m,s)),&
                            overlap(dmat(m,s)),subdim(m),dim(m,s),dim(m,s))
             endif
               call qtxxzz(overlap(dmat(m,s)),tmppsi(zeig),tmppsi(zeig1), &
                    vdim(m,s),dim(m,s),ndim(m,s))
               zeig2=zeig
               zeig=zeig1
               zeig1=zeig2
            enddo
            call vvaxzz(psi(zpsi(s)),tmppsi(zeig),sum,block(s))
            spop(s)=dble(sum)
         enddo

      else
         do s=1,nstate
            zeig=1
            zeig1=block(s)+1
            call cpvxz(psi(zpsi(s)),tmppsi(zeig),block(s))

            do m=1,nmode
               call qtxxzz(overlap(dmat(m,s)),tmppsi(zeig),tmppsi(zeig1), &
                    vdim(m,s),dim(m,s),ndim(m,s))
               zeig2=zeig
               zeig=zeig1
               zeig1=zeig2
            enddo

            call vvaxzz(psi(zpsi(s)),tmppsi(zeig),sum,block(s))
            spop(s)=dble(sum)
         enddo

      endif
      
!-----------------------------------------------------------------------
! write results 
!-----------------------------------------------------------------------
      write(iaus,'(6x,f8.3,6x,99(f10.8,4x))')  time,&
           (spop(s),s=1,nstate)
      call myflush(iaus)

!-----------------------------------------------------------------------
! increment of time step
!-----------------------------------------------------------------------
      time=time + dt
      wfnum=wfnum+1
      
!-----------------------------------------------------------------------
! go out of loop if enough WFs read
!-----------------------------------------------------------------------
      if (ncut .ne. 0) then
         if (wfnum .ge. ncut) goto 77
      endif

!-----------------------------------------------------------------------
! skip WFs
!-----------------------------------------------------------------------
      if (.not.lrst) then
         do i=2,step
            call rdpsi(ipsi,psi,spsi,jindx,agmat,idum,adgwp,gwpdep)
            if (lend) goto 77
            time=time + dt
            wfnum=wfnum+1
         enddo
      endif

!-----------------------------------------------------------------------
! go to the beginning of loop
!-----------------------------------------------------------------------      
      if (.not.lrst) goto 10

 77   continue
      close(ipsi)
      close(idvr)


!-----------------------------------------------------------------------
! Get overall timing 
!-----------------------------------------------------------------------
      tcpu = fcputime(0)
      tsys = fsystime(0)
      ttot = tcpu+tsys
      t1   = int(ttot/3600)
      ttot = ttot - 3600*t1
      t2   = int(ttot/60)
      ttot = ttot - 60*t2

      write(ilog,'(//,a,/)') '# ***** Overall Timing *****'
      write(ilog,'(a,1x,f12.2)') '# Total cpu time     [s] : ',tcpu
      write(ilog,'(a,1x,f12.2)') '# Total system time  [s] : ',tsys
      write(ilog, 900) '# Total time     [h:m:s] : ',t1,t2,ttot
      write(ilog,*)

! --- WRITE DATE AND MACHINE  ---
      call fpstartup(buffer)
      write(iaus,*)
      write(iaus,'(a2,a78)') '# ',buffer

  900 format( a,i4,' :',i3,' :',f5.1)
      stop
      
  999 continue
      routine='population'
      message= '2: Error reading :'//filein(1:laein)
      call errormsg

      return
      end subroutine stpopfrompsi

!-----------------------------------------------------------------------
!***********************************************************************
! i is no. of arguments read, excluding DOFs for densities
!-----------------------------------------------------------------------

      subroutine stpoppsiinp

      implicit none

      integer(long)                                  :: i
      integer(long), external                        :: myiargc
      integer(long)                                  :: iarg,ilbl,ierr
      character(len=80)                              :: buf,host
      character(len=c5)                              :: filename,dvrfile
      logical(kind=4)                                :: lnoq

      operfile = ' '
      fileout  = ' '
      filein   = ' '
      name     = './'
      iarg     = myiargc()
      step     = 1
      nskip    = 0
      ncut     = 0
      dvrfile  = ' '
      lnoq = .false.
      
!***********************************************************************

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if(buf .eq. '-f') then
            i = i+1
            call mygetarg(i,buf)
            filein = buf
         else if( buf .eq. '-i' ) then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf .eq. '-o' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         elseif(buf .eq. '-r') then
            lrst = .true.
            filein = 'restart'
         elseif(buf .eq. '-w') then
            lovwr = .true.
         elseif(buf .eq. '-n') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ncut
            if (ierr .ne. 0 .or. ncut.le.0) then
               write(6,'(2a,/a)') 'Expecting a positive integer ',&
                    'after -n, not: ', buf
               stop
            endif
         elseif(buf .eq. '-skip') then
            i = i+1         
            call mygetarg(i,buf)         
            read(buf,*,iostat=ierr) nskip 
            if (ierr .ne. 0 .or. nskip.le.0) then
               write(6,'(2a,/a)') 'Expecting a positive integer ',&
                    'after -skip, not: ', buf
               stop         
            endif         
         elseif(buf .eq. '-step') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0 .or. step.le.0) then
               write(6,'(2a,/a)') 'Expecting a positive integer ',&
                    'after -step, not: ', buf
               stop
            endif
         elseif(buf .eq. '-tcut') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) tcut
            if (ierr .ne. 0 .or. tcut.le.0) then
               write(6,'(2a,/a)') 'Expecting a positive number ',&
                    'after -tcut, not: ', buf
               stop
            endif

         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(1x,a)')&
       'Purpose: Calculates the state populations from a psi file.'
            write(6,'(1x,a)')&
       'Usage:   stpoppsi [-i -f -d -o -r -n -skip -step -tcut -w  -ver -h -?]'
            write(6,'(/,a)') 'Options  : '
            write(6,'(a)') ' -i DIR  : data is stored in directory DIR'
            write(6,'(2a,/,9x,a)') &
                     ' -f FILE : the PSI is read from',&
                     ' FILE rather than from ./psi',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a)') &
                     ' -d FILE : the DVR is read from',&
                     ' FILE rather than from ./dvr',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a)') &
                     ' -o FILE : The ouput is written to file ',&
                     'FILE rather than to ./stpoppsi',&
            '  The string FILE may be a relative or a full path-name.',&
            '  If FILE=no, i.e. -o no, no output file will be written.'
            write(6,'(2a)') ' -r      : The PSI is read from',&
                             ' restart file rather than from ./psi'
            write(6,'(2a)')&
                     ' -n num  : only the first num WFs are read '&
                    ,' from psi-file.'
            write(6,'(a)')&
                     ' -skip m : the first m WFs are skipped.'
            write(6,'(2a)') ' -step step : Compute the population',&
                            ' only every step-th WF.'
            write(6,'(2a)') ' -tcut tcut : Compute the population',&
                            ' only at time tcut.'
            write(6,'(2a)') &
                     ' -w      : Overwrite the output-file stpoppsi.'
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         goto 100
      endif
      
      write(6,'(78a1)')  ('-',i=1,78)
!-----------------------------------------------------------------------
! Opening the PSI or RESTART file.
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         if (lrst) then
            filein = name(1:laenge)//'/restart'
         else
            filein = name(1:laenge)//'/psi'
         endif
         laein = index(filein,' ')-1
      else 
         laein=index(filein,' ')-1
         laenge=index(name,' ')-1
         filein=name(1:laenge)//'/'//filein(1:laein)
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)

      endif

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/stpoppsi.log'
      open (unit=ilog,file=filename,form='formatted')
      logisopen=.true.
      
!-----------------------------------------------------------------------
! Open psi file channel
!-----------------------------------------------------------------------
      filename=filein
      if (lrst) irst=ipsi
      open(ipsi,file=filename,form='unformatted',status='old',err=1000)
      if (lrst) then
         write(6,'(2a)') ' The restart file is  : '//filein(1:laein)
      else
         write(6,'(2a)') ' The psi file is  : '//filein(1:laein)
      endif

!-----------------------------------------------------------------------
!  Opening the output file
!-----------------------------------------------------------------------
      if (fileout .eq. 'no') then
         laeout=2
         open(iaus,file='/dev/null',form='formatted',status='unknown')
      else
         if (fileout .eq. ' ') fileout=name(1:laenge)//'/stpoppsi'
         laeout=index(fileout,' ')-1
         call abspath(fileout,laeout)
         filename=fileout
         if(lovwr) then
            open(iaus,file=filename,form='formatted',status='unknown',&
                 err=1030)
         else
            open(iaus,file=filename,form='formatted',status='new',&
                 err=1020)
         endif
         write(6,'(2a)') ' The output file is  : '//fileout(1:laeout)
      endif

      write(6,'(78a1)')  ('-',i=1,78)

! --- WRITE DATE AND MACHINE  ---
      call fpstartup(host)
      write(iaus,'(/a2,a78)') '# ',host

      return 
      
 1000 write(6,*)' Cannot open file : '
      ilbl=index(filename,' ')-1
      write(6,*) filename(1:ilbl)
      stop
 1020 write(6,*)' Cannot open the output file : '
      ilbl=index(filename,' ')-1
      write(6,*) filename(1:ilbl)
      write(6,*) 'Use option -w to overwrite the output file.'
      stop
 1030 write(6,*)'Cannot open the output file: '
      ilbl=index(filename,' ')-1
      write(6,*) filename(1:ilbl)
      stop
      
      end subroutine stpoppsiinp
      

      end program stpoppsi
