!***********************************************************************
!
!                        ANALYSE MODULE SUMSPEC
!
! Sums spectra (output from autospec).
!
! 11/96 MB 
!
! need to open files and count data to get maxdata
!***********************************************************************

      program sumspec

use decimal, only: long,dop
use global
use aglobal
use versions
use channels

use module_interface

      implicit none

      integer(long)            :: maxspec, maxdata

      integer(long), external :: myiargc,slen
      integer(long)           :: i,j,k,iarg,nspec,n,ierr,ierr1,ncos,&
                                 ndata,ndata1,ndatamax,ic,icolumn
      real(dop)               :: ener,emin,emax,shft,denr,fac 
      real(dop)               :: moment(0:2,19)

      real(dop), dimension(:), allocatable         :: facspec,expspec,spec
      real(dop), dimension(:), allocatable         :: plener
      real(dop), dimension(:,:), allocatable       :: pldata
      character(len=8)                             :: stat
      character(len=240)                           :: buf
      character(len=c5)                            :: gnucmd
      character(len=c5), dimension(:), allocatable :: filespec
      logical(kind=4)                              :: lgrid,lrdinp,lx,ly
      logical(kind=4)                              :: lmom, lproduct
      character(len=10) :: cerr

!-----------------------------------------------------------------------
! Initialise variables
!-----------------------------------------------------------------------
      call  default
      call adefault
      fileout = 'sumspec.pl'
      name    = ' '
      lgrid = .false.
      lmom  = .false.
      lproduct = .false.
      emin =  1.0e+22_dop
      emax = -1.0e+22_dop
      xmin =  99999.0_dop
      xmax =  -99999.0_dop
      ymin =  99999.0_dop
      ymax =  -99999.0_dop
      lx = .false.
      ly = .false.
      shft =  0.0_dop
      denr =  0.0_dop
      ncos = -1
      stat = 'new'
      ndatamax = 100000000
      fac     = 1.0d0
      moment(:,:) = 0.0_dop

!-----------------------------------------------------------------------
! read options
!-----------------------------------------------------------------------
      i = 0
      iarg = myiargc()
      if (iarg .lt. 1) goto 2

      buf=' '
   1  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf .eq. '-o' ) then
            i=i+1
            call mygetarg(i,fileout)
         else if( buf .eq. '-fac' ) then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) fac
            if (ierr .ne. 0) then
               write(6,*)'Argument for -fac must be a real number.'
               write(6,*)'Not :', buf
               stop
            end if
            write(6,'(a,e12.4)') 'Default factor =',fac
         else if( buf .eq. '-w' ) then
            lovwr  = .true.
            stat='unknown'
         else if( buf .eq. '-mom' ) then
            lmom  = .true.
         else if( buf .eq. '-prod' ) then
            lproduct  = .true.
         else if( buf .eq. '-G' ) then 
            lgrid = .true.
         else if( buf(1:2) .eq. '-g' ) then 
            lgnu = .true.
            if( buf .eq. '-g' ) then
               i = i+1
               call mygetarg(i,buf)
            else
               buf=buf(3:4) 
            endif  
            read(buf,*,iostat=ierr) ncos
            if (ierr .ne. 0) then
               write(6,*)&
               'Argument for -g must be a non-negative integer.'
               write(6,*)'Not :', buf
               stop
            end if
         else if( buf .eq. '-x' ) then 
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) xmin
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr1) xmax
            lx = .true.
            if (ierr .ne. 0 .or. ierr1 .ne. 0) then
               write(6,*)'Arguments for -x must be two real numbers.'
               write(6,*)'Not :', buf
               stop
            end if
         else if( buf .eq. '-y' ) then 
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ymin
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr1) ymax
            ly = .true.
            if (ierr .ne. 0 .or. ierr1 .ne. 0) then
               write(6,*)'Arguments for -x must be two real numbers.'
               write(6,*)'Not :', buf
               stop
            end if
         else if( buf .eq. '-s' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) shft
            if (ierr .ne. 0) then
               write(6,*)'Arguments for -s must be a real number.'
               write(6,*)'Not :', buf
               stop
            end if
         else if( buf .eq. '-S' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) denr
            if (ierr .ne. 0) then
               write(6,*)'Arguments for -S must be a real number.'
               write(6,*)'Not :', buf
               stop
            end if
         else if( buf .eq. '-a' ) then
            lgnu  = .true.
            lagnu = .true.
            if(ncos .lt. 0) ncos  = 1
         else if( buf .eq. '-n' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ndatamax
            if (ierr .ne. 0) then
               write(6,*)'Arguments for -n must be an integer.'
               write(6,*)'Not :', buf
               stop
            end if
         else if( buf .eq. '-ver' ) then
            call wrversion(6)
            stop

         else if( buf .eq. '-h' .or.  buf .eq. '-?') then
            write(6,'(/1x,a/11x,a/1x,2a)') &
        'Purpose : Sums spectral components (output from autospec) ',&
                  'Data files in plain xyz format',&
        'Usage   : sumspec [options]',&
                 ' file1 fac1 exp1 file2 fac2 exp2 [file3 fac3 exp3 ...]'
            write(6,'(/a)') ' Options : '
            write(6,'(a)') &
                 ' -i DIR  : pathes are relative to directory DIR'
            write(6,'(2a)') ' -o FILE : write to file FILE.',&
                 ' The default is: sumspec.pl'
            write(6,'(a)') &
               ' -fac fac  : Use fac as default factor rather than 1.'
            write(6,'(a)') ' -w      : overwrite enabled.'
            write(6,'(a,/11x,a)') ' -mom    : computation of moments.', &
                 'Only one input file allowed, no output file opened.'
            write(6,'(2a,/11x,2a,/11x,a)') &
               ' -g ncos : GNUplot command lines are written to ',&
               'the output file.','ncos = 0,1,2 is the exponent',&
               ' of the cosine damping function.',&
               'NB: This sets "using 1:(ncos+2)" '
            write(6,'(2a)') &
                ' -a      : GNUplot is called automatically.',&
                ' (sets "-g 1").'
            write(6,'(a)') &
                ' -G      : Plot gridlines. Requires -g or -a option.'
            write(6,'(2a,/16x,a)') &
                ' -x xmin xmax : The energy range is set to ', &
                '[xmin,xmax].','(ignored if neither -g nor -a is set).'
            write(6,'(2a,/16x,a)') &
                ' -y ymin ymax : The ordinate scale is set to ', &
                '[ymin,ymax].','(ignored if neither -g nor -a is set).'
            write(6,'(a,/16x,a)')&
                ' -s shift: Shift the energy scale by -shift ',&
                '(ignored if neither -g nor -a is set).'
            write(6,'(a,/11x,a)')&
                ' -S shift: Shift the energy scale by -shift ',&
                'Similar to -s, but this time the output file is modified.'
            write(6,'(a)') ' -G      : Plot gridlines.'
            write(6,'(a)') ' -n ndata: Read only the first ndata data-sets.'
             write(6,'(a)') &
                ' -prod   : The product rather then the sum is computed.'
            write(6,'(a)')&
                ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
              ' -h      : Print this help text. ',&
              ' -?      : Print this help text. '
            write(6,'(/,2a,//,2a,/,2a,11(/a))') &
              'Example: sumspec -a -G b2u/spectrum.pl 1.0 1.0 ',&
              '  b2g/spectrum.pl 1.0 1.0 '
              write(6,'(80a1)') ('-',k=1,80)
            stop
         else
            write(6,'(2a)') ' Unknown option :', buf
            write(6,'(2a)') ' Use option -h'
            stop
         end if
         if (iarg .le. i) goto 2
         goto 1

      end if
         
    2 continue
!-----------------------------------------------------------------------
! Read arguments. ic is no. of arguments with -
!-----------------------------------------------------------------------
      ic=max(i-1,0)
      nspec=(iarg-ic)/3

!.....In case, no arguments were provided.
      lrdinp=.false.
      if (nspec .eq. 0) then
         write(6,'(a)') 'Number of spectra to be summed ?'
         read (5,*) nspec
         if (nspec .eq. 0) stop
         lrdinp=.true.
      endif
      if(lmom .and. nspec.gt.1) then
         write(6,'(a)') 'Only the first file is used, if -mom is set!'
         nspec=1
      endif

!-----------------------------------------------------------------------
! Define maximum no. of spectra to be summed
!-----------------------------------------------------------------------
      maxspec=nspec

      cerr = 'filespec'
      allocate(filespec(maxspec),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'facspec'
      allocate(facspec(maxspec),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'expspec'
      allocate(expspec(maxspec),stat=ierr)
      if (ierr > 0) go to 1000

      do i=1,maxspec
         filespec(i) = '  '
         facspec(i)=fac
         expspec(i)=1.0d0
      enddo

      if (lrdinp) then
         do n=1,nspec
            write(6,'(a,i3,a)') ' Filename containing spectrum',n,' ?'
            read (5,'(a)') filespec(n)
            write(6,'(a,i3,a)') &
                ' Multiplicative factor for spectrum',n,' ?'
            read (5,*) facspec(n)
            write(6,'(a,i3,a)') &
                ' Exponent for spectrum',n,' ?'
            read (5,*) expspec(n)
         enddo

      else
         j=ic+1
         do nspec=1,maxspec
            call mygetarg(j,filespec(nspec))
            j=j+1
            call mygetarg(j,buf)
            read(buf,*,iostat=ierr) facspec(nspec)
            if (ierr .ne. 0) then
               if(fac .eq. 0.d0) then
                  write(6,'(a,i3,a)') 'Multiplicative Factor for data-set', &
                  nspec,', set to 1.0 by default.'
               endif
            else 
              j=j+1
            endif
            call mygetarg(j,buf)
            read(buf,*,iostat=ierr) expspec(nspec)
            if (ierr .ne. 0) then
               if(fac .eq. 0.d0) then
                  write(6,'(a,i3,a)') 'Exponent for data-set', &
                  nspec,', set to 1.0 by default.'
               endif
            else
               j=j+1
            endif
         enddo
      endif

!.....Make file-names absolute.
      do n=1,maxspec
         if (name .ne. ' ') then
            laenge=index(name,' ')-1
            laein=index(filespec(n),' ')-1
            call abspath1(filespec(n),laein,name)
         else
            laein=index(filespec(n),' ')-1
            call abspath(filespec(n),laein)
         endif
      enddo
      laeout=index(fileout,' ')-1
      if (name .ne. ' ') then
         call abspath1(fileout,laeout,name)
      else
         call abspath(fileout,laeout)
      endif

!-----------------------------------------------------------------------
! Open and read first spectrum file to get no. of columns
!-----------------------------------------------------------------------
      n=1
      laein=slen(filespec(n))
      open(iaut,file=filespec(n)(1:laein),form='formatted',&
              status='old',err=666)
      ndata=0
 108  read(iaut,'(a)',end=111,err=777) buf
      if( buf(1:1)  .eq. '#'  .or.  buf .eq. ' ') goto 108
      backspace iaut

! --- Determine number of columns
      read(iaut,'(a)',err=777) buf
      icolumn=0
      do 
         call rmlblnk(buf)
         i=index(trim(buf),' ')
         if(i.eq.0) exit
         icolumn=icolumn+1
         buf=buf(i:)
      enddo
      backspace iaut
      if(icolumn .le. 0) then
         write(6,'(a,i2)') ' No data column to sum!, icolumn=',icolumn
         stop
      endif
      write(6,'(a,i2)') 'Number of data columns: icolumn=',icolumn
      cerr = 'spec'
      allocate(spec(icolumn),stat=ierr)
      if (ierr > 0) go to 1000
      write(6,'(a,i4)') 'Number of spectra (data files): npec=',maxspec

!-----------------------------------------------------------------------
! Open and read spectrum files to get maxdata
!-----------------------------------------------------------------------
      maxdata=0
      do n=1,maxspec
         ndata=0
         laein=slen(filespec(n))
         open(iaut,file=filespec(n)(1:laein),form='formatted',&
              status='old',err=666)
         ndata=0
 110     read(iaut,'(a)',end=111,err=777) buf
         if( buf(1:1)  .eq. '#'  .or.  buf(1:50) .eq. &
             '                                                  ' ) goto 110
! ignore strings (gnuplot info)
         read(buf,*,err=110) ener
         backspace iaut
         do
            read(iaut,*,end=120) ener
            ndata=ndata+1
         enddo
 120     continue
         if (n .eq. 1) then
            maxdata=ndata
         else if (ndata .ne. maxdata) then
            write(6,'(a,/,a)') 'Spectrum has different no. of points :',&
               filespec(n)(1:laein)
            write(6,'(/,a,2i7)') 'ndata, maxdata =', ndata, maxdata
            stop
         endif
         close(iaut)
      enddo

      cerr = 'plener'
      allocate(plener(maxdata),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'pldata'
      allocate(pldata(maxdata,icolumn),stat=ierr)
      if (ierr > 0) go to 1000
      plener(:)=0.0
      pldata(:,:)=0.0

!-----------------------------------------------------------------------
! Open and read spectrum files to get first data
!-----------------------------------------------------------------------
      n=1
      ierr=0
      laein=slen(filespec(n))
      open(iaut,file=filespec(n)(1:laein),form='formatted',&
           status='old',err=666)
      write(6,'(a,2f12.6)') &
           'Reading : '//filespec(n)(1:laein),facspec(n),expspec(n)
 160  read(iaut,'(a)',end=111,err=777) buf
      if( buf(1:1)  .eq. '#'  .or.  buf(1:50) .eq. &
      '                                                  ' ) goto 160
      read(buf,*,err=160) ener
      backspace iaut

      do ndata=1,maxdata
         read(iaut,*,end=200,err=777) ener,(spec(i),i=1,icolumn)
         plener(ndata)=ener
         if (.not. lx) then
            xmin = min(xmin,ener)
            xmax = max(xmax,ener)
         endif

         do i=1,icolumn
            pldata(ndata,i)=facspec(n)*spec(i)**expspec(n)
         enddo
      enddo

 200  continue
      close(iaut)

      if(lmom) then
         do i=1,icolumn
            do j=1,ndata
               moment(0,i) = moment(0,i) + pldata(j,i)
               moment(1,i) = moment(1,i) + &
                            pldata(j,i)*(plener(j)-denr-shft)
               moment(2,i) = moment(2,i) + &
                             pldata(j,i)*(plener(j)-denr-shft)**2
            enddo
         enddo
         write(6,'(/a)') 'Original moments'
         write(6,'(a,1p19e12.5)') '   norm :',(moment(0,i),i=1,icolumn)
         write(6,'(a,1p19e12.5)') '1st mom :',(moment(1,i),i=1,icolumn)
         write(6,'(a,1p19e12.5)') '2nd mom :',(moment(2,i),i=1,icolumn)
         write(6,'(/a)') 'Normalized moments'
         do i=1,icolumn
            moment(1,i)=moment(1,i)/moment(0,i)
            moment(2,i)=moment(2,i)/moment(0,i)
         enddo
         write(6,'(a,1p19e12.5)') '1st mom :',(moment(1,i),i=1,icolumn)
         write(6,'(a,1p19e12.5)') '2nd mom :',(moment(2,i),i=1,icolumn)
         write(6,'(/a)') 'Central moments'
         do i=1,icolumn
            moment(2,i) = moment(2,i) - moment(1,i)**2
         enddo
         write(6,'(a,1p19e12.5)') 'variance:',(moment(2,i),i=1,icolumn)
         write(6,'(a,1p19e12.5)') '  width :', &
                                        (sqrt(moment(2,i)),i=1,icolumn)
         write(6,*)
         stop
      endif

!-----------------------------------------------------------------------
! Now sum or multiply spectra
!-----------------------------------------------------------------------
      do n=2,maxspec
         ierr=0
         laein=slen(filespec(n))
         open(iaut,file=filespec(n)(1:laein),form='formatted',&
              status='old',err=666)
         write(6,'(a,2f12.6)') &
              'Reading : '//filespec(n)(1:laein),facspec(n),expspec(n)
 260     read(iaut,'(a)',end=111,err=777) buf
         if( buf(1:1)  .eq. '#'  .or.  buf(1:50) .eq. &
         '                                                  ' ) goto 260
         read(buf,*,err=260) ener
         backspace iaut

         do ndata=1,maxdata
            read(iaut,*,end=300,err=777) ener,(spec(i),i=1,icolumn)
            if (abs(ener-plener(ndata)) .gt. tiny) then
               write(6,'(a,/,a)') 'Spectrum has different energy points '
               stop
            endif
            if (lproduct) then
               do i=1,icolumn
                  pldata(ndata,i) = &
                       pldata(ndata,i)*facspec(n)*spec(i)**expspec(n)
               enddo
            else
               do i=1,icolumn
                  pldata(ndata,i)= &
                       pldata(ndata,i) + facspec(n)*spec(i)**expspec(n)
               enddo
            endif
         enddo

 300     continue
         close(iaut)
      enddo

!-----------------------------------------------------------------------
! Get min and max values
!-----------------------------------------------------------------------
      if (.not. ly) then
         do ndata=1,maxdata
            ymin=min(ymin,pldata(ndata,ncos))
            ymax=max(ymax,pldata(ndata,ncos))
         enddo
      endif

      if(xmin.lt.xmax) then
         emin = xmin-denr-shft
         emax = xmax-denr-shft
      endif

!-----------------------------------------------------------------------
! Write to output file.  
!-----------------------------------------------------------------------
      open(iaus,file=fileout(1:laeout),form='formatted',&
                status=stat,err=667)

      if(lproduct) then
         write(iaus,'(a)')'#  -prod is set!'
      endif
      write(iaus,'(a,8x,a,8x,a)')'#      factor','exponent','file-name'

      do n=1,maxspec
         write(iaus,'(a,i4,1p2e14.6,3x,a)') &
           '#',n,facspec(n),expspec(n),trim(filespec(n))
      enddo
      write(iaus,'(80a1)') '#',('-',i=1,79)

      write(6,'(a)') 'Writing : '//fileout(1:laeout)

      if( lgnu ) then
         ncos = min(ncos,icolumn-1)
         if( ncos .lt. 0 .or.  ncos .gt. 2 ) then
            write(6,'(a)') '#########################################'
            write(6,'(a,i3)') ' illegal value of ncos =', ncos
            write(6,'(a,i3)') ' ncos must be 0, 1 or 2 '
            write(6,'(a)') '#########################################'
            stop
         end if
         write(buf,'(a,i2,a)') 'tit "n-cos =',ncos,'"'
         write(iaus,'(a)') 'set style data lines' 
         write(iaus,'(a)') 'set nologscale' 
         write(iaus,'(a)') 'set xzeroaxis' 
         if(lgrid) write(iaus,'(a)') 'set grid' 
         write(iaus,'(a,e14.7,a,e14.7,a)') 'set xr [',emin,':',emax,']'
         write(iaus,'(a,e14.7,a,e14.7,a)') 'set yr [',ymin,':',ymax,']' 
         write(iaus,'(3a)') 'set xlabel "Energy"'
         write(iaus,'(a,i2,a,3x,2a)') &
                 'set title "Summed Spectrum,  n-cos =', ncos,&
                 ',', filein(1:laein-4),'"' 
         if(shft.eq.0.0_dop) then
            write(iaus,'(3a,i1,a)') 'plot "',fileout(1:laeout),&
                                  '" us 1:',ncos+2,' notit'
         else
            write(iaus,'(3a,f11.7,a,i1,a)') 'plot "',fileout(1:laeout),&
                   '" us ($1-',shft,'):',ncos+2,' notit'
         endif
         write(iaus,'(a)') ' ' 
      end if

      do i=1,maxdata
         if(plener(i)-denr .ge. 0.0001d0 .and. &    
                     plener(i)-denr .le. 999.99d0) then
            write(iaus,'(1x,f17.12,9(1x,1pe17.8))') &
                  plener(i)-denr, (pldata(i,k),k=1,icolumn)
         else   
            write(iaus,'(10(1x,1pe17.8))') &
                  plener(i)-denr, (pldata(i,k),k=1,icolumn)
         endif
      enddo

      deallocate(filespec)
      deallocate(facspec)
      deallocate(spec)
      deallocate(plener)
      deallocate(pldata)

      close(iaus)

!.....call gnuplot     
      if(lagnu) then
         gnucmd = 'gnuplot -persist 2>/dev/null "'//trim(fileout)//'"'
         call excmd(gnucmd)
      endif

      stop

 111  continue
      write(6,'(a)') 'ERROR: No data on file: ',filespec(n)(1:laein)
      stop

 666  continue
      write(6,*)
      write(6,'(a,/a)')&
         'ERROR in program sumspec. Cannot open:', filespec(n)(1:laein)
      write(6,*)
      stop

 777  continue
      write(6,*)
      write(6,'(a,/a)')&
         'ERROR in program sumspec. Cannot read file:', filespec(n)(1:laein)
      write(6,*) ' Does it contain gnuplot command lines?'
      write(6,*) ' ndata =', ndata
      write(6,*)
      stop

 667  continue
      write(6,*)
      write(6,'(a,/a,/a)')&
         'ERROR in program sumspec. Cannot open output-file:'&
         , fileout(1:laeout)&
         ,'Overwrite disabled? (Use -w option to enable overwrite)' 
      write(6,*)
      stop

 1000 write(ilog,'(a)') 'Error allocating memory in Statepop :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop

      end program sumspec








