!***********************************************************************
!
!              Analyse program  TDPESPEC
!
!     11/99 GW, 03/00 HDM
!
!
!***********************************************************************
      
      program tdpespec

use decimal, only: long,dop,sip
use global
use globmemmod
use aglobal
use versions
use dvrdatmod
use griddatmod
use psidef
use channels
use datenmod
use plot
use compdatmod
use memdimmod
use logdat

use op1lib, only: zerovxl, cpvxd, zerovxd
use logdat, only: errormsg
use iofile, only: rdrhoinfo
use rddvrmod, only: dvrinfo
use rdopermod, only: operinfo
use zeigauswmod, only: zeigausw

use module_interface

      implicit none

      integer(long)                        :: chkdvr,chkgrd,chkdat,ilbl,chkpsi
      character(len=c5)                    :: filename
      character(len=10)                    :: stat
      logical(kind=4), external            :: int2log
      logical(kind=4)                      :: lerr,llgrthm,check
      integer(long)                        :: dof,state,pltpnt,ilgpel
      common /itdpesp/ dof,state,pltpnt

!-----------------------------------------------------------------------
! Initialize variables and set defaults
!-----------------------------------------------------------------------
      call default
      call adefault

!-----------------------------------------------------------------------
! Declare files to be read
! set flag to .true. if file is to be read
!
!     lrddvr    DVR file
!     lrdoper   OPER file
!-----------------------------------------------------------------------
      lrddvr=.true.
      lrdoper=.false.

!-----------------------------------------------------------------------
! Declare resources required (other than those needed to read a file)
!
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! llogrthm : logarithmic plot
!-----------------------------------------------------------------------
      ncomp=0
      lpsi=.false.
      lpsi1=.false.
      lpsigrd=.false.
      ldmat=.false.
      ldmat1=.false.
      lort=.true.
      lgpop=.true.
      ltrafo=.true.

!-----------------------------------------------------------------------
! read input parameters and options
! open files
!-----------------------------------------------------------------------
      call tdpespecinp(stat,llgrthm)

!-----------------------------------------------------------------------
! check for existence of name directory and pump only data
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/gridpop'
      inquire(file=filename,exist=check)
      if (.not. check) then
         message= 'ERROR: Pump-only directory does not exist '
         write(6,*)
         write(6,'(a)') message
         stop
      endif

!-----------------------------------------------------------------------
! open input and output data files
!
! use standard channels if possible
!  irho : gridpop file
!  iaus : output file
!  ilog : log-file showd1d.log
!-----------------------------------------------------------------------
      open(irho,file=filename,form='unformatted',status='old',err=920)
      read(irho,err=999) filever(irho), ilgpel

      open (unit=ilog,file='tdpespec.log',form='formatted')
      logisopen=.true.
      if (linter) then
         filename=name(1:laenge)//'/tmpplot.pl'
         open(iaus,file=filename,status='unknown',err=920)
      else
         if (fileout .ne. 'no') then
            filename=fileout
            open(iaus,file=filename,form='formatted',&
                 status=stat,err=920)
         else
            iaus=6
         endif
      endif

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(irho)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat

!-----------------------------------------------------------------------
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
! First read file version number
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      lgpel = int2log(ilgpel)
      call rdrhoinfo(irho,chkdvr,chkgrd,chkdat)

!-----------------------------------------------------------------------
! read information from system files
!-----------------------------------------------------------------------
      if (lrddvr) then
         chkdvr=2
         filename=dname(1:dlaenge)//'/dvr'
         open(idvr,file=filename,form='unformatted',status='old',err=920)
         call dvrinfo(lerr,chkdvr)
         close(idvr)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif
      if (lrdoper) then
         chkdvr=2
         chkgrd=2
         filename=oname(1:olaenge)//'/oper'
         open(ioper,file=filename,form='unformatted',status='old',err=920)
         call operinfo(lerr,chkdvr,chkgrd)
         close(ioper)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif

!-----------------------------------------------------------------------
! check input
!-----------------------------------------------------------------------
      if( state .gt. nddstate) then
         write(6,'(a,i3,a,i3)') 'The required state s =',state, &
         ' must not be larger than nddstate = ', nddstate   
         stop
      endif
      if(.not.lgpel) nddstate=1

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! write gnuplot info to file
!-----------------------------------------------------------------------
      if (lgnu .and. .not. linter) then
         ilbl=index(filein,' ')-9
         write(iaus,'(a,3x,2a)') 'set title "TD PE Spectrum',&
                                       filein(1:ilbl),'"'
         write(iaus,'(a)') 'set style data lines'
         if(llgrthm) then
            write(iaus,'(a)') 'set logscale y'
         else
            write(iaus,'(a)') 'set nologscale'
         endif
         if(lpgrid) write(iaus,'(a)') 'set grid'
         if (outform .eq. 0 .or. outform .ge. 2) then
            if( xmax .gt. xmin ) then
               write(iaus,'(a,f8.3,a,f8.3,a)') &
                  'set xrange [',xmin,':',xmax,']'
            endif
            if( ymax .le. 0.0_dop ) then
               if(llgrthm) then
                  write(iaus,'(a)') 'set yrange [1e-8:*]'
               else
                  write(iaus,'(a)') 'set yrange [0:*]'
               endif
            else
               if(llgrthm) then
                  write(iaus,'(a,e11.4,a)') 'set yrange [1e-8:',ymax,']'
               else
                  write(iaus,'(a,e11.4,a)') 'set yrange [0:',ymax,']'
               endif
            end if
            write(iaus,'(a)') '#  '

         else if (outform .eq. 1) then
            write(iaus,'(a)') 'set surface'
            write(iaus,'(a)') 'set contour'
            write(iaus,'(a)') 'set cntrparam levels auto'
            write(iaus,'(a)') '#set view 0,0,1,1'
            write(iaus,'(a)') 'set view 60.0,30.0,1,1'
            write(iaus,'(3a)') 'set xlabel "',modelabel(febcont)(1:9),'"'
            write(iaus,'(a)') 'set ylabel "Time"'
            write(iaus,'(a)') 'set zlabel ""'
            write(iaus,'(a)') 'set origin 0.,0.'
            write(iaus,'(a)') 'set size 1.0, 1.0'
            if(lsmooth) then
               write(iaus,'(a)')  'set cntrparam cubicspline'
               write(iaus,'(a)')  'set cntrparam points 5'
            else
               write(iaus,'(a)')  'set cntrparam linear'
            end if
            write(iaus,'(a)') 'splot "-" using 1:2:3 ; pause -1'

         endif
         write(iaus,'(a)') '#  '
         if(outform .eq. 0) write(6,'(a)') &
           'Press RETURN to see the next plot'
      end if

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
! set nplottask and ncalcpar before allocating memory for plotting routines
!-----------------------------------------------------------------------
      nplottask=4
      ncalcpar=1
      call alloc_plot

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! assign work arrays
! NB work arrays must be at end of mc, mr, mi, ml arrays.
!-----------------------------------------------------------------------
      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! If oper file read, comment in routine to assign memory for operator
!-----------------------------------------------------------------------
!     call memhops

!-----------------------------------------------------------------------
! call analysis routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      write(ilog,'(/,2a)') 'gridpop : ', filein(1:laein)
      write(ilog,'(2a)')   'output  : ', fileout(1:laeout)
      write(ilog,'(2a,/)') 'dname   : ', dname(1:dlaenge)
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

      call subtdpespec(mr(mrgpop1),mr(mrgpop2),mf(mfgpop),&
                    mr(mrort),mr(mrtrafo))

deallocate(mc,mr,mi,ml,ms,mf)


      stop
  999 continue
      ilbl=index(filename,' ')-1
      message= 'Error reading :'//filename(1:ilbl)
      write(6,'(a)') message
      stop
  920 continue
      routine='tdpespec'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      if(filename(1:ilbl) .eq. fileout(1:ilbl))&
      write(6,'(/a)') '###  Use -w option to overwrite! ###' 
      call errormsg
      stop
      contains


!***********************************************************************
!
!     HERE IS THE SUBROUTINE TO DO THE ANALYSIS
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! provide dummy arguments of arrays needed for analysis
!  e.g. psi to match argument mc(mcpsi)
!***********************************************************************

      subroutine subtdpespec (gpop1,gpop2,fgpop,ort,trafo)

use global
use aglobal
use maxv, only: maxdim,maxsta
use dvrdatmod
use griddatmod
use datenmod
use plot
use channels

use rdfilesmod, only: rdgridpop
use rddvrmod
use plotutils, only: plotdefs,plotmenu,timebnds,enerbnds,databnds,dotask,initplot,inmodebnds
use rddvrmod, only: rddvr
use op1lib, only: zerovxd,zerovxl,cpvxd

      implicit none

      integer(long)   :: dof,state,pltpnt
      real(dop)       :: factor,w2
      common /itdpesp/ dof,state,pltpnt
      common /rtdpesp/ factor, w2

      integer(long)                     :: chkdvr,chkpsi,chkgrd,chkdat,ilgpel,edim
      integer(long), dimension(maxdim)  :: idum,idum1,idum2
      real(dop), dimension(1)           :: rdum,rdum1,rdum2
      complex(dop), dimension(1)        :: zdum,zdum1,zdum2,zdum3

      integer(long), external           :: slen
      integer(long)                     :: g,s,sc,n,gdim1,i,ilbl,jlbl,spdim,s1
      character(len=64)                 :: ccc
      character(len=c5)                 :: filename
      character(len=6)                  :: fslab
      character(len=200)                :: string
      logical(kind=4), external         :: int2log
      logical(kind=4)                   :: lexist

      real(dop)                                                  :: time,eoff
      real(dop)                                                  :: sum,diff
      real(sip), dimension(2*maxgdim), intent(out)               :: fgpop
      real(dop), dimension(ortdim), intent(out)                  :: ort
      real(dop), dimension(dvrdim), intent(out)                  :: trafo
      real(dop), dimension(maxgdim)                              :: ener
      real(dop), dimension(ortdim,nddstate), intent(out)         :: gpop1,gpop2
      real(dop), dimension(ortdim,nddstate)                      :: gpopold
      real(dop), dimension(ortdim,nddstate)                      :: tmpgpop1,tmpgpop2
      real(dop), dimension(1)                                    :: y

      real(dop), dimension(:), allocatable                       :: tyield
      real(dop), dimension(:,:), allocatable                     :: spec
      real(dop), dimension(:,:,:), allocatable                   :: specs
      real(dop), dimension(maxgdim)                              :: spec0
      real(dop), dimension(maxgdim,maxsta)                       :: spec0s
      real(dop), dimension(:), allocatable                       :: pltime
      real(dop), dimension(:,:,:), allocatable                   :: pldata
! standard variables for interactive plots
      integer(long)                       :: ndata,task,plottask,f1,&
                                             nsets,nplots,plotdim
      integer(long)     :: ierr
      character(len=10) :: cerr


!-----------------------------------------------------------------------
! zero dummy variables
!-----------------------------------------------------------------------
      y(1)=0.0
      fslab = ' fs,  '
      eoff = 0.0_dop

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! read data from dvr file if desired
! edit which data (set dvrdata(x) to .true. and substitute duumy array)
!-----------------------------------------------------------------------
      dvrdata(:)=.false.
      if (lrddvr) then
         dvrdata(1)=.true.
         dvrdata(2)=.true.
         dvrdata(4)=.false.
         dvrdata(5)=.false.
         dvrdata(6)=.false.
         dvrdata(7)=.false.
         dvrdata(8)=.false.
         dvrdata(9)=.false.
         dvrdata(11)=.false.
         dvrdata(12)=.false. 
         chkdvr=0
         call rddvr(ort,trafo,rdum,rdum1,zdum,&
              zdum1,idum,zdum2,zdum3,idum1,idum2,rdum2,chkdvr)
      endif

!-----------------------------------------------------------------------
! get system info 
!-----------------------------------------------------------------------
      spdim=gdim(febcont)
      axis(1)=febcont

      edim=gdim(febcont)
      call cpvxd(ort(zort(febcont)),ener,gdim(febcont))

!-----------------------------------------------------------------------
! get pump only background spectrum
!-----------------------------------------------------------------------
      gpopold = 0.0
      do  
         call rdgridpop(irho,fgpop,tmpgpop1,tmpgpop2)
         if (lend) exit

         gpop1=tmpgpop1
         gpop2=tmpgpop2

!-----------------------------------------------------------------------
! read to end of file to get final state populations
! check convergence
!-----------------------------------------------------------------------
         sum=0.0_dop
         do s=1,nddstate
            if (.not. sebcont(s)) cycle
            do g=1,gdim(febcont)
               diff = (gpop1(zort(febcont)-1+g,s) - gpopold(zort(febcont)-1+g,s))**2
               sum = sum + dsqrt(diff)
            enddo
         enddo
       !  print*,sum
         call cpvxd(gpop1,gpopold,ortdim*nddstate)

      enddo

!     stop


!-----------------------------------------------------------------------
! sum up spectra
!-----------------------------------------------------------------------
      call zerovxd(spec0,maxgdim)
      call zerovxd(spec0s,maxgdim*nddstate)
      do s=1,nddstate
         if (sebcont(s)) then
            do g=1,gdim(febcont)
               spec0(g) = spec0(g) + gpop1(zort(febcont)-1+g,s)
               spec0s(g,s) = gpop1(zort(febcont)-1+g,s)
            enddo
         endif
      enddo

      close(irho)

!-----------------------------------------------------------------------
! list directories
!-----------------------------------------------------------------------
      string="ls -d1 --sort=time -r "//name(1:laenge)//"_* > tdpespec.ls"
      call excmd(string)

!-----------------------------------------------------------------------
! open list of directories and get no. of data sets
! read list and ignore files that have a type, e.g. .inp files
!-----------------------------------------------------------------------
      filename="tdpespec.ls"
      open(iin,file=filename,form="formatted",status="old",err=920)
      ndata=0
      do
         read(iin,'(a)',end=5) string
         ilbl = index(string,'.')
         if (ilbl .le. 0) ndata = ndata+1
      enddo
    5 continue
      write(6,'(a,i5)') 'No. datasets found for plotting :',ndata
      maxdata = ndata

! allocate data arrays
      cerr = 'pltime'
      allocate(pltime(maxdata),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'pldata'
      allocate(pldata(maxgdim,maxdata,nddstate),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'spec'
      allocate(spec(maxgdim,maxdata),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'specs'
      allocate(specs(maxgdim,maxdata,nddstate),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tyield'
      allocate(tyield(maxdata),stat=ierr)
      if (ierr > 0) go to 1000

!-----------------------------------------------------------------------
! get times from names of directories and read data
!-----------------------------------------------------------------------
      rewind(iin)
      ndata=0
      do 
         read(iin,'(a)',end=55) string

         ilbl = index(string,'.')
         if (ilbl .gt. 0) cycle
         ndata = ndata+1

         ilbl = index(string,' ')-1

! read times
         jlbl=ilbl-1
         do while (string(jlbl:jlbl) .ne. "_") 
            jlbl=jlbl-1
         enddo   
         jlbl=jlbl+1
! error if "time" is not a number and get next file name
         read(string(jlbl:ilbl),*,err=910) pltime(ndata)

! now read data
         string=string(1:ilbl)//'/gridpop'
         open(irho,file=string,form='unformatted',status='old',err=930)

         chkdvr=0
         chkgrd=0
         chkpsi=0
         chkdat=0
         filename='gridpop'
         read(irho) filever(irho), ilgpel
         lgpel = int2log(ilgpel)
         if (filever(irho) .lt. 11.0_dop) call rdrhoinfo(irho,chkdvr,chkgrd,chkdat)

         lend = .false.
         do 
            call rdgridpop(irho,fgpop,tmpgpop1,tmpgpop2)
            if (lend) exit
            gpop1=tmpgpop1
            gpop2=tmpgpop2
         enddo

         call zerovxd(spec(1,ndata),maxgdim)
         do s=1,nddstate
            if (sebcont(s)) then
               do g=1,gdim(febcont)
                  spec(g,ndata) = spec(g,ndata) + gpop1(zort(febcont)-1+g,s)
                  specs(g,ndata,s) = gpop1(zort(febcont)-1+g,s)
               enddo
            endif
         enddo
         tyield(ndata) = 0.0_dop
         do g=1,gdim(febcont)
            tyield(ndata) = tyield(ndata) + spec(g,ndata)
         enddo

         close(irho)
      enddo
! end of loop to read data

   55 continue

!-----------------------------------------------------------------------
! initialise interactive plotting
!-----------------------------------------------------------------------
      if (linter) then
         call plotdefs
         plotdim=2
         lplenetim=.true.
         lstate=.true.

         filename=name(1:laenge)//'/tmpplot.pl'
         call initplot(filename,plotdim)

!-----------------------------------------------------------------------
! set program specific data, e.g. plot tasks
! do not exceed nplottask or ncalcpar
! Select time slice as default
!-----------------------------------------------------------------------
         plottask=2
         ltslice=.true.

         cplottask(1)='3D Spectrum'
         cplottask(2)='Time-slices of spectrum'
         cplottask(3)='Total ion yield'
         cplottask(4)='Pump-only spectrum'
         do i=1,nplottask
            cptlen(i)=slen(cplottask(i))
         enddo

         lsrf=.true.
         lcntr=.false.

         s=slen(axunitlab(fener))
         acalcpar(1)= &
             'Ion excess energy zero ('//axunitlab(fener)(1:s)//')'
         rcalcpar(1)= 0.0_dop

!-----------------------------------------------------------------------
! calculate initial mode bounds
!-----------------------------------------------------------------------
         call inmodebnds(ort,plotdim)
      else
         dof=febcont
      endif

!-----------------------------------------------------------------------
! call menu
!-----------------------------------------------------------------------
  200 continue
      if (linter) then
! check and change unit of eoff for menu - it is stored in au 
         s=slen(axunitlab(fener))
         acalcpar(1)= &
             'Ion excess energy zero ('//axunitlab(fener)(1:s)//')'
         eoff=rcalcpar(1)
         rcalcpar(1)=eoff/axunit(fener)
         call plotmenu(task,plottask,plotdim)
         rcalcpar(1)=eoff

         if (.not. lrecalc) go to 250
         axis(1)=fener

! Added to allow 3D plots etc. GWR 09/12
         axunit(ftime)=1d0
!
         if (plottask .eq. 1) then
            plotdim=2
            ltslice=.false.
            lpltime=.false.
            lplenetim=.true.
            lplener=.false.
            axis(1)=fener
         else if (plottask .eq. 2) then
            plotdim=1
            ltslice=.true.
            lpltime=.false.
            lplenetim=.true.
            lplener=.false.
            axis(1)=fener
         else if (plottask .eq. 3) then
            plotdim=1
            ltslice=.false.
            lpltime=.true.
            lplenetim=.false.
            lplener=.false.
            axis(1)=ftime
         else if (plottask .eq. 4) then
            plotdim=1
            ltslice=.false.
            lpltime=.false.
            lplenetim=.false.
            lplener=.true.
            axis(1)=fener
         endif

         eoff = rcalcpar(1)
      endif

      s=max(1,state)
      gdim1 = edim

!-----------------------------------------------------------------------
! Remove the fs label if nofs is set.
!-----------------------------------------------------------------------
      if( fs .lt. 2.0_dop ) fslab = '      '

!-----------------------------------------------------------------------
! file header for non-interactive plotting
!-----------------------------------------------------------------------
      ccc = ' '
      sc = 0
      if (.not. linter) then
          write(iaus,'(3a)') 'set xlabel "',&
                  axislab(1)(1:slen(axislab(1))),'"'
      
         if(lsmooth) then
            ccc = ' using 1:2 smooth csplines  notitle ; pause -1'
         else if(.not.lsmooth) then
            ccc = ' using 1:2 notitle ; pause -1'
         end if
         sc = slen(ccc)

         if (outform .eq. 3) then
            time = tinit/fs + indx*step*out1/fs
            write(iaus,'(a)') 'set nolabel'
            write(iaus,'(a,f6.1,3a)') 'set label "',time,fslab,&
               modelabel(dof)(1:12),'" at graph 0.01,0.95'
            write(iaus,'(a,i5,a)') 'plot "-"  index ',indx,ccc(1:sc)
         else if (outform .eq. 4) then
            write(iaus,*) out2/fs
            write(iaus,*) gdim(dof)
            do g=1,gdim(dof)
               write(iaus,*) ort(zort(dof)-1+g)
            enddo
         else if (outform .eq. 5) then
            write(iaus,'(3a)') 'set xlabel "Time[fs]"'
            write(iaus,'(a)') 'set nolabel'
            write(iaus,'(a,i4,a,f7.2,a,2x,2a)')&
                    'set label "pltpnt=', pltpnt,&
                    ',  ort=',ort(zort(dof)-1+pltpnt),&
                    ',',modelabel(dof)(1:12),'" at graph 0.01,0.95'
            write(iaus,'(2a)') 'plot "-" ',ccc(1:sc-11)
         endif
      endif

!-----------------------------------------------------------------------
! store data
!-----------------------------------------------------------------------
      do s=1,nddstate
         if (sebcont(s)) then
            do g=1,gdim(febcont)
               spec(g,ndata) = spec(g,ndata) + gpop1(zort(febcont)-1+g,s)
               specs(g,ndata,s) = gpop1(zort(febcont)-1+g,s)
            enddo
         endif
      enddo

!-----------------------------------------------------------------------
! write out or store data
!-----------------------------------------------------------------------
      if (linter) then
         do n=1,ndata
            call zerovxd(pldata(1,n,1),maxgdim)
            do g=1,gdim(febcont)
               pldata(g,n,1) = spec(g,n) - spec0(g)
            enddo
            s1 = 1
! components are stored in pldata too.
            do s=1,nddstate
               if (sebcont(s)) then
                  s1 = s1+1
                  call zerovxd(pldata(1,n,s1),maxgdim)
                  do g=1,gdim(febcont)
                     pldata(g,n,s1) = specs(g,n,s) - spec0s(g,s)
                  enddo
               endif
            enddo
            nplstates = s1
         enddo

         edim=gdim(febcont)
         call cpvxd(ort(zort(febcont)),ener,gdim(febcont))
         if (abs(eoff) .gt. 1.0e-8_dop) then
            do g=1,gdim(febcont)
               ener(g) = eoff - ener(g)
            enddo
         endif

      else if (outform .eq. 0) then
         write(iaus,'(a)') 'set nolabel'
         if(time.lt.99.9_dop) then
            write(iaus,'(a,f7.2,3a)') 'set label "',time,fslab,&
               modelabel(dof)(1:12),'" at graph 0.01,0.95'
         else
            write(iaus,'(a,f7.1,3a)') 'set label "',time,fslab,&
               modelabel(dof)(1:12),'" at graph 0.01,0.95'
         endif
         write(iaus,'(3a,f11.3,2a)') 'plot "-" ',ccc(1:sc),&
                                    ' "',time,fslab,'"'
         do g=1,edim
            write(iaus,'(2(1x,f20.12))') ort(zort(febcont)-1+g),&
            spec(g,1) - spec0(g)
         enddo

      else if (outform .eq. 1) then
         do g=1,edim
            write(iaus,'(4(1x,f20.12))') ort(zort(febcont)-1+g),time,&
            gpop1(zort(febcont)-1+g,s), gpop2(zort(febcont)-1+g,s)*w2
         enddo

      else if (outform .eq. 2) then
         write(iaus,'(a)') 'set nolabel'
         if(time.lt.99.9_dop) then
            write(iaus,'(a,f7.2,3a)') 'set label "',time,fslab,&
               modelabel(dof)(1:12),'" at graph 0.01,0.95'
         else
            write(iaus,'(a,f7.1,3a)') 'set label "',time,fslab,&
               modelabel(dof)(1:12),'" at graph 0.01,0.95'         
         endif
         write(iaus,'(2a)') 'plot "-" ',ccc(1:sc-2)//'0.25 '
         do g=1,edim
            write(iaus,'(3(1x,f20.12))') ort(zort(febcont)-1+g),&
            gpop1(zort(febcont)-1+g,s), gpop2(zort(febcont)-1+g,s)*w2
         enddo

      else if (outform .eq. 3) then
         do g=1,edim
            write(iaus,'(3(1x,f20.12))') ort(zort(febcont)-1+g),&
            gpop1(zort(febcont)-1+g,s), gpop2(zort(febcont)-1+g,s)*w2
         enddo
         write(iaus,'(/,a)')
      else if (outform .eq. 4) then
         do g=1,edim
            write(iaus,*) gpop1(zort(febcont)-1+g,s)
         enddo
      else if (outform .eq. 5) then
         g = zort(febcont)-1+pltpnt
         write(iaus,'(3(1x,f20.12))') time,&
              gpop1(g,s), gpop2(g,s)*w2
      endif

      if (outform .eq. 0 .or. outform .eq. 2) then
         write(iaus,'(a)') 'e'
      else if (outform .eq. 1) then
         write(iaus,*)
      endif


      if (linter) go to 250

      close(iaus)

      if(lprint) then
         inquire(file="printplot",exist=lexist) 
         open(iaus,file="printplot",form='formatted',status='unknown')
         if(.not.lexist) then
            open(iaus,file="printplot",form='formatted')
            write(iaus,'(a)') 'set terminal postscript'
            write(iaus,'(a)')'set output "| lpr" '
            write(6,'(a)')'File printplot created.'
         end if
         rewind iaus
         read(iaus,'(a1)') gnucmd 
         read(iaus,'(a30)') gnucmd 
         close(iaus)
         write(6,'(a)')'Press enter twice to print. Ctrl C to abort.'
         write(6,'(3a)')'Abort and edit printplot if you printcommand',&
                        ' is not: '//gnucmd(11:29)
      end if
      if(lagnu) then
         gnucmd = 'gnuplot -persist "'//trim(fileout)//'"'
         call excmd(gnucmd)
      end if
      if(lprint) then
         gnucmd = 'gnuplot -persist '//'"printplot"  "'&
                  //fileout(1:slen(fileout))//'"'
         call excmd(gnucmd)
      end if
      return

  250 continue
!-----------------------------------------------------------------------
! interactive plotting
!-----------------------------------------------------------------------
      if (lrebound) then
         call timebnds(pltime,ndata)
         call enerbnds(ener,edim)
         call databnds(pldata,edim,ndata,1,1,maxgdim,maxdata,&
                 nddstate,1)
      endif

!-----------------------------------------------------------------------
! do task
! TASK=0 STOP  else return to menu
!-----------------------------------------------------------------------
      if (task .gt. 0) then
         if (abs(eoff) .gt. 1.0e-8_dop) then
            plttitle='Ion Excess Energy'
         else
            plttitle='Photo-Electron Energy'
         endif
         if (plottask .eq. 1) then
            nsets=nplstates
            nplots=1
            axislab(1)='Energy [eV]'
            axislab(2)='Time [fs]'
            axislab(3)='Intensity'
            call dotask(task,plottask,plotdim,ener,pltime,&
                    pldata,edim,ndata,nsets,nplots,&
                    maxgdim,maxdata,nddstate,1,ort,ortdim,pltime,ndata,&
                    ener,edim,rdum,1,1,1,1,idum)
         else if (plottask .eq. 2) then
            axislab(1)='Energy [eV]'
            axislab(2)='Time [fs]'
            axislab(3)='Intensity'
            if (plotform .eq. 0) then
               if (nplstates .gt. 1) then
                  write(6,'(a)') &
            'WARNING : Only data for 1st selected state is displayed'
               endif
               nsets=ndata
               nplots=1
               call dotask(task,plottask,plotdim,ener,y(1),&
                    pldata,edim,1,nsets,nplots,&
                    maxgdim,1,nsets,1,ort,ortdim,pltime,ndata,&
                    ener,edim,rdum,1,1,1,1,idum)
            else
               nsets=nplstates
               nplots=ndata
               call dotask(task,plottask,plotdim,ener,y(1),&
                    pldata,edim,1,nsets,nplots,&
                    maxgdim,maxdata,nddstate,1,ort,ortdim,pltime,ndata,&
                    ener,edim,rdum,1,1,1,1,idum)
            endif

         else if (plottask .eq. 3) then
            nsets=1
            f1=axis(1)
            nplots=1
            axislab(1)='Time [fs]'
            axislab(3)='Intensity'
            call dotask(task,plottask,plotdim,pltime,rdum,&
                    tyield,ndata,1,nsets,1,&
                    maxdata,1,1,1,ort,ortdim,pltime,ndata,&
                    ener,edim,rdum,1,1,1,1,idum)

         else if (plottask .eq. 4) then
            axislab(1)='Energy [eV]'
            axislab(3)='Intensity'
            nsets=1
            nplots=1
            call dotask(task,plottask,plotdim,ener,rdum,&
                 spec0,edim,1,nsets,nplots,&
                 maxgdim,1,nsets,1,ort,ortdim,pltime,ndata,&
                 ener,edim,rdum,1,1,1,1,idum)

         endif

!
! convert eoff to au
!
         if (task .eq. 800) then
            rcalcpar(1)=rcalcpar(1)*axunit(fener)
         endif
         go to 200
      endif

      !close(iaus,status='delete')
      !close(iin,status='delete')
      close(iaus)
      close(iin)

      return

  910 continue
      routine='Tdpespec'
      ilbl=index(string,' ')-1
      message = 'Directory name without time: '//string(1:ilbl)
      call errormsg
      stop
  920 continue
      routine='Tdpespec'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop
  930 continue
      routine='Tdpespec'
      ilbl=index(string,' ')-1
      message = 'Cannot open file: '//string(1:ilbl)
      call errormsg
      stop
 1000 write(ilog,'(a)') 'Error allocating memory in Statepop :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop

      end subroutine subtdpespec


!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------
      subroutine tdpespecinp(stat,llgrthm)

use global
use aglobal
use griddatmod
use datenmod
use plot

      implicit none

      integer(long) :: dof,state,pltpnt
      real(dop)     :: factor,w2
      common /itdpesp/ dof,state,pltpnt
      common /rtdpesp/ factor, w2

      integer(long), external       :: myiargc
      integer(long)                 :: iarg,ilbl,i,ierr
      character(len=80)             :: buf,string
      character(len=*), intent(out) :: stat
      logical(kind=4), intent(out)  :: llgrthm

!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! provide default input and output filenames 
!-----------------------------------------------------------------------
      filein  = ' '
      fileout = ' '
      name    = ' '

      dof=0
      state=1

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! set default flags
!
! stat : status of output file   
!-----------------------------------------------------------------------
      stat    = 'new'
      lnw     = .false.
      lgnu    = .false.
      lagnu   = .false.
      lprint  = .false.
      lpgrid  = .false.
      lsmooth = .false.
      llgrthm = .false.
      linter  = .false.
      outform = 0
      factor  = 1.0_dop
      step = 1
      xmax = 0.0_dop
      xmin = 1.0_dop
      ymax = 0.0_dop
      tfinal = 9.9e+99_dop

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! read options 
! comment out or add options as required
!-----------------------------------------------------------------------
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf .eq. '-inter' ) then
            linter=.true.

         else if( buf(1:3) .eq. '-o ' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf(1:4) .eq. '-no ' ) then
            fileout='no'
         else if( buf(1:3) .eq. '-I ' ) then
            i = i+1
            outform = 3
            call mygetarg(i,buf)
            read(buf,*,err=999) indx
         else if( buf(1:3) .eq. '-S ' ) then
            outform = 0
         else if( buf(1:3) .eq. '-T ' ) then
            outform = 1
         else if( buf(1:3) .eq. '-M ' ) then
            outform = 2
         else if( buf(1:3) .eq. '-Ma' ) then
            outform = 4
         else if( buf(1:3) .eq. '-P ' ) then
            outform = 5
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=999) pltpnt
         else if( buf(1:3) .eq. '-G ' ) then
            lpgrid = .true.
         else if( buf(1:3) .eq. '-n ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if(buf(1:4).eq.'-sm ' .or. buf(1:7).eq.'-smooth') then
            lgnu = .true.
            lsmooth = .true.
         else if( buf(1:3) .eq. '-x ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=998) xmin
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=998) xmax
         else if( buf(1:3) .eq. '-y ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=996) ymax
         else if( buf(1:3) .eq. '-t ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=996) tfinal
         else if( buf(1:4) .eq. '-fac' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,err=997) factor
         else if( buf(1:3) .eq. '-w ' ) then
            stat='unknown'
         else if( buf(1:6) .eq. '-nofs ' ) then
            fs = 1.0_dop
         else if( buf(1:4) .eq. '-nw ' ) then
            lnw = .true.
         else if( buf(1:3) .eq. '-l ' ) then
            llgrthm = .true.
         else if( buf(1:3) .eq. '-g ' ) then
            lgnu = .true.
         else if( buf(1:3) .eq. '-a ' ) then
            lgnu  = .true.
            lagnu = .true.
            stat  = 'unknown'
         else if( buf(1:3) .eq. '-p ' ) then
            lgnu   = .true.
            lprint = .true.
         else if( buf .eq. '-gnupl4 ') then
            lgnu4 = .true.                 ! gnuplot ver < 5
         else if( buf(1:3) .eq. '-h ' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,a,/1x,2a,/18x,a)')&
       'Purpose: To plot time-resolved photo-electron spectra.',&
       'Usage  : tdpespec [-T|-S|-I|-P|-Ma] [-f -i -o -a -g -p',&
       ' -x -y -fac -n -no',&
       ' -t -nofs -l -sm -w -nw -G -inter -gnupl4 -ver -h -?] [name].'
            write(6,'(/,a)') 'Output Formats : '
            write(6,'(2a)') &
                    ' -S      : "Step through" by pressing RETURN.',&
                    ' (default).'
            write(6,'(a)') &
                    ' -M      : "Movie", one picture every second.'
            write(6,'(2a)') &
                    ' -I id   : "Index file", id=0 shows the first ',&
                    'picture, id=1 the second, etc.'
            write(6,'(2a)') &
                    ' -P pnt  : "Point file", shows the population ',&
                    'of the pnt-th grid-point over time.'
            write(6,'(a)') &
                    ' -T      : "3D time file" (grid file).'
            write(6,'(a)') &
                    ' -inter  : Opens interactive plotting menu.'
            write(6,'(/,a)') 'Options  : '
            write(6,'(2a,/,9x,a)') &
                     ' -f FILE : The wavefunction is read '&
                    ,'from file FILE rather than from ./gridpop'&
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(a)') ' -i DIR  : Use DIR as name-directory.'
            write(6,'(2a,/,11x,a/,11x,a,/11x,a)') &
                     ' -o FILE : The output is written to file',&
                     ' FILE_f* rather than','to ./den1d_f*',&
             'The string FILE may be a relative or a full path-name.',&
             'If FILE=no, i.e. "-o no", no output file will be opened.'
            write(6,'(2a)') &
                     ' -g      : GNUplot comand lines are written to ',&
                     'the output file(s).'
            write(6,'(2a)') &
                     ' -a      : GNUplot is called automatically.',&
                     ' (sets -g and -w).'
            write(6,'(2a)') &
                     ' -p      : Print GNUplot (sets -g). ',&
                                'Allowed only with -I or -T.' 
            write(6,'(2a)') &
                     ' -sm     : Smooth the plot-data by splines.',&
                     ' (Only for GNUplot, sets -g).'
            write(6,'(2a)')&
                     ' -l      : Use logarithmic scale.',&
                     ' (Only for GNUplot).'
            write(6,'(2a)') ' -n step : Show only every', &
                  ' step-th picture. (Only useful for -S, -M).'
            write(6,'(2a)') ' -t tfinal : Show densities only up',&
                  ' to t=tfinal.'
            write(6,'(a)')&
                     ' -no     : No output file opened.'
            write(6,'(2a)')&
                     ' -nofs   : No transformation to fs.',&
                     " Use when 'time-not-fs' was set in mctdh."
            write(6,'(2a)') &
               ' -x xmin xmax : Set the abszisse length',&
               ' (default: auto). Ignored for -T, -Ma.'
            write(6,'(2a)') &
               ' -y ymax : Set the ordinate length',&
               ' (default: auto). Ignored for -T, -Ma.'
            write(6,'(2a)') &
               ' -fac factor : Multiply the density values by factor.'
            write(6,'(a)') ' -G      : Plot gridlines.'
            write(6,'(2a)') &
                     ' -w      : An existing output file is ',&
                     'overwritten.'
            write(6,'(a)')&
                     ' -gnupl4  : Use gnuplot version < 5 for plotting '
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(a)') &
                     ' -ver    : Version information about the program '
            write(6,'(/a)') &
  ' The argument "name" gives the filestem for the directories with data.'
            write(6,'(a)') &
  ' "name contains a pump-only calculation and "name_T" contain data '
            write(6,'(a)') &
  ' with a probe at varying times T. At least one T file is needed as '
            write(6,'(a)') &
  ' well as the pump only.'
            write(6,'(a)') &
  ' If no argument is given, the user is prompted for the missing argument.'  
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      if (iarg .lt. i) goto 200

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! get any compulsory arguments if in argument list
!-----------------------------------------------------------------------
  150 continue
      call mygetarg(i,string)
      name=string
      i=i+1
      if (i .le. iarg) goto 150

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! fill in any arguments not given on command line
!-----------------------------------------------------------------------
 200  continue
      if (name .eq. ' ') then
         write(6,*) 'Directory stem  :'
         read(5,*) name
         write(6,*)
      endif

!-----------------------------------------------------------------------
! Check for consistency.
!-----------------------------------------------------------------------
      if(lprint .and. (outform.eq.0 .or. outform.eq.2)) then
         write(6,'(a)') '-p allowed only with -I or -T. STOP!'
         stop
      end if

!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
!-----------------------------------------------------------------------
      laenge=index(name,' ')-1
      call abspath(name,laenge)

      dlaenge=laenge
      dname=name

!-----------------------------------------------------------------------
! set up output file name
!-----------------------------------------------------------------------
      if (fileout .eq. 'no' .or. linter) go to 500

      ilbl=index(fileout,' ')
      if (ilbl .eq. 1) then
         fileout=name(1:laenge)//'.tdps'
         ilbl=index(fileout,' ')
      endif
      
!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! convert filenames to absolute
!-----------------------------------------------------------------------
      laeout=index(fileout,'  ')-1
      call abspath(fileout,laeout)

 500  continue

      if (outform .eq. 4) then
         ymax   =  0.0_dop
         lgnu   = .false.
         lagnu  = .false.
         lprint = .false.
      end if

      if (linter) lagnu=.false.
      if (linter) lprint=.false.
      if (linter) outform=-1

      return 
 999  write(6,*) 'Integer expected after -I  option '
      stop
 998  write(6,*) 'Two real numbers expected after -x  option '
      stop
 997  write(6,*) 'Real number expected after -y  option '
      stop
 996  write(6,*) 'Real number expected after -fac  option '
      stop
      end subroutine tdpespecinp


      end program tdpespec
