!***********************************************************************
!
!                         AUSWERT template program
!
! This program can be used as a basis for analysis programs. By
! selection of options the memory can be allocated, and files
! read, making use of the structures of the MCTDH program.
!
!     11/96 MB
!
!
!***********************************************************************
      
!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! substitute "template" with program name and comment in
!-----------------------------------------------------------------------
!     program template

use decimal, only: long,dop,sip
use global
use globmemmod
use aglobal
use memdimmod
use maxv
use dvrdatmod
use griddatmod
use operdef
use psidef
use channels
use datenmod
use compdatmod
use compdat1mod

use rddvrmod

      implicit none

      integer(long)     :: chkdvr,chkgrd,chkpsi,chkdat,ilbl 
      character(len=c5) :: filename
      character(len=10) :: stat
      logical(kind=4)   :: lerr

!-----------------------------------------------------------------------
! Initialize variables and set defaults
!-----------------------------------------------------------------------
      call default
      call adefault

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare files to be read
! set flag to .true. if file is to be read
!
!     lrddvr    DVR file
!     lrdoper   OPER file
!-----------------------------------------------------------------------
      lrddvr=.false.
      lrdoper=.false.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare resources required (other than those needed to read a file)
!
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! ltrafo : FBR/DVR trafo matrices
! lsphfbr : jfbr, mfbr and kinfbr matrices.
!-----------------------------------------------------------------------
      ncomp=0
      lpsi=.false.
      lpsi1=.false.
      lpsigrd=.false.
      ldmat=.false.
      ldmat1=.false.
      lort=.false.
      lgpop=.false.
      ltrafo=.false.
      lsphfbr=.false.

!-----------------------------------------------------------------------
! read input parameters and options
! open files
!-----------------------------------------------------------------------
      call input(stat)

!-----------------------------------------------------------------------
! get memory sizes
! Comment in what is being used
!-----------------------------------------------------------------------
! choose with appropriate file unit if only a data file is being analysed
!      call rdmemdim(irho)

! choose if the wavefunction is being analysed
!      if (lshowrst .or. lshowpsi) then
!         filename=name(1:laenge)//'/restart'
!         open(irst,file=filename,form='unformatted',status='old',err=1000)
!         call rdmemdim(irst)
!         close(irst)
!      endif

! choose if the operator is being used
! dimensions from oper must be read second as the oper file may have
! been recreated at a later date with new operators
!      if (lrdoper) then
!         filename=name(1:laenge)//'/oper'
!         open(ioper,file=filename,form='unformatted',status='old',err=1000)
!         call rdmemdim(ioper)
!         close(ioper)
1      endif

!-----------------------------------------------------------------------
! Allocate memory
! Add appropriate alloc routines as required
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
!      call alloc_operdef
!      call alloc_psidef

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! open input and output data files
!
! use standard channels if possible
!  irho : gridpop file
!  ipsi : psi file
!  iaut : auto file
!  ichk : check file
!  iaus : output file
!-----------------------------------------------------------------------
      filename=filein
      open(irho,file=filename,form='unformatted',status='old',err=1000)
!     if (fileout .ne. 'no') then
!        filename=fileout
!        open(iaus,file=filename,form='formatted',
!    +       status=stat,err=1000)
!     else
!        iaus=6
!     endif

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
! First read file version number
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      filename=filein
!     read(ipsi,err=999) filever(ipsi)
!     call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
!     read(ichk,err=999) filever(ichk)
!     call rdchkinfo(ichk,chkdvr,chkgrd,chkpsi,chkdat)
!     read(iaut,err=999) filever(iaut)
!     call rdautinfo(iaut,chkdvr,chkgrd,chkpsi,chkdat)
      read(irho,err=999) filever(irho)
      call rdrhoinfo(irho,chkdvr,chkgrd,chkdat)

!-----------------------------------------------------------------------
! read information from system files
!-----------------------------------------------------------------------
      if (lrddvr) then
         chkdvr=2
         filename=dname(1:dlaenge)//'/dvr'
         open(idvr,file=filename,form='unformatted',status='old',&
              err=1000)
         call dvrinfo(lerr,chkdvr)
         close(idvr)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif
      if (lrdoper) then
         chkdvr=2
         chkgrd=2
         filename=oname(1:olaenge)//'/oper'
         open(ioper,file=filename,form='unformatted',status='old',&
              err=1000)
         call operinfo(lerr,chkdvr,chkgrd)
         close(ioper)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif
         
!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! assign work arrays
! NB work arrays must be at end of mc, mr, mi, ml arrays.
!-----------------------------------------------------------------------
      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! If oper file read, comment in routine to assign memory for operator
!-----------------------------------------------------------------------
      if (lrdoper) call memhops

!-----------------------------------------------------------------------
! call analysis routine
!-----------------------------------------------------------------------

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)
if(allocated(gfgwp))deallocate(gfgwp)
if(allocated(aigwp))deallocate(aigwp)
if(allocated(zgwp))deallocate(zgwp)
if(allocated(dicht2))deallocate(dicht2)
if(allocated(agmat))deallocate(agmat)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))
!allocate(gfgwp(gfgwpdim))
!allocate(aigwp(aigwpdim))
!allocate(zgwp(zgwpdim))
!allocate(dicht2(dmatdim))
!allocate(agmat(agmatdim))

      call subanalyse(  )

deallocate(mc,mr,mi,ml,ms,mf)

      stop
  999 continue
      message= 'Error reading :'//filename
      write(6,'(a)') message
      stop
 1000 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop

      contains

!***********************************************************************
!
!     HERE IS THE SUBROUTINE TO DO THE ANALYSIS
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! change name of subroutine 
! provide dummy arguments of arrays needed for analysis
!  e.g. psi to match argument mc(mcpsi)
!***********************************************************************

      subroutine subanalyse ( )

      implicit none

      integer(long)                :: chkdvr,chkgrd
      integer(long), dimension(1)  :: idum
      real(dop), dimension(1)      :: rdum
      complex(dop), dimension(1)   :: zdum

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! read data from dvr file if desired
! edit which data (set dvrdata(x) to .true. and substitute dummy array)
!-----------------------------------------------------------------------
      if (lrddvr) then
         dvrdata(1) = .false.    ! read ort
         dvrdata(2) = .false.    ! read trafo
         dvrdata(3) = .false.    ! read dvrmat (dif1mat, dif2mat)
         dvrdata(4) = .false.    ! read fftp   (fft momentum)
         dvrdata(5) = .false.    ! read hin    (fft)
         dvrdata(6) = .false.    ! read rueck  (fft)
         dvrdata(7) = .false.    ! read fftfak
         dvrdata(8) = .false.    ! read jsph
         dvrdata(9) = .false.    ! read msph
         dvrdata(10)= .false.    ! read kinsph
         dvrdata(11)= .false.    ! read exphin   (expDVR)
         dvrdata(12)= .false.    ! read exprueck (expDVR)
         chkdvr=2
         call rddvr(rdum,rdum,rdum,rdum,zdum,zdum,&
                    idum,zdum,zdum,idum,idum,rdum,chkdvr)
      endif
!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! read data from oper file if desired
! replace dummies rdum by the previously allocated hops(hopsdim), 
!   ddtransf(ddtransdim),ddtransb(ddtransdim),ddxcoo0(ndofddpes)
!-----------------------------------------------------------------------
      if (lrdoper) then
         chkdvr=1
         chkgrd=1
         call rdoper(rdum,chkdvr,chkgrd)
      endif

!
! DO THE ANALYSIS
!

      return
      end subroutine subanalyse

!-----------------------------------------------------------------------
!***********************************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! change name of subroutine 
!-----------------------------------------------------------------------
      subroutine input(stat)

      implicit none

      integer(long)                 :: iarg,myiargc,i,ierr
      character(len=80)             :: buf
      character(len=*), intent(out) :: stat

!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! provide default input and output filenames 
!-----------------------------------------------------------------------
      filein = '  '
      fileout= '  '
      name   = './'

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! set default flags
!
! stat : status of output file   
!-----------------------------------------------------------------------
      stat = 'new'
      lgnu = .false.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! read options 
! comment out or add options as required
!-----------------------------------------------------------------------
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf .eq. '-inter' ) then
            linter=.true.
         else if( buf .eq. '-ver' ) then
            call wrversion(6)
            stop

         else if( buf(1:2) .eq. '-o' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf(1:2) .eq. '-n' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:2) .eq. '-w' ) then
            lovwr=.true.
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,a,/10x,a)')&
       'Purpose: WHAT WILL CODE DO?   ',&
       'Usage  : NAME [-f -i -inter -ver -o -n -w -h -?]  [XXX] .'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,11x,a/11x,a)')&
              ' -f FILE : the data is read '&
              ,'from file FILE','rather than from ./XXX'&
              ,'The string FILE may be a relative or full path-name.'
            write(6,'(a)') ' -i DIR : data is stored in directory DIR'
            write(6,'(a)') ' -inter : enables interactive plotting'
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(2a,/,11x,a/,11x,a)') &
                     ' -o FILE : The difference is written to file ',&
                     'FILE.pl rather than to ./XXX',&
             'The string FILE may be a relative or a full path-name.',&
             'If FILE=no, i.e. "-o no", no XXX-file will be opened.'
            write(6,'(2a)') ' -n step : Compute the property', &
           ' only every step-th output.'
            write(6,'(2a)') &
                     ' -w      : An existing property file is ',&
                     'overwritten.'
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(2a/,a)') ' The argument psi2 is the name of',&
                            ' the file containing the comparison', &
                            ' wavefunction.' 
            write(6,'(2a)') ' If no argument is given, ',&
                 'the user is prompted for the missing argument.'  
            write(6,'(78a1)')  ('-',i=1,78)
            stop

         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      if (iarg .lt. i) goto 200
      
!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='ausw.log',form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! If filein has not been set, look in name directory.
! Set XXX to default name
! Otherwise take name from filein
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         filein=name(1:laenge)//'/XXXX'
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! get any compulsory arguments if in argument list
!-----------------------------------------------------------------------
!     call mygetarg(i,filein1)

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! fill in any arguments not given on command line
!-----------------------------------------------------------------------
 200  continue
!     if (filein1 .eq. ' ') then
!        write(6,*) 'XXX   :'
!        read(5,'(a)') filein1
!        write(6,*)
!     endif

!-----------------------------------------------------------------------
! set status variable to overwrite files
!-----------------------------------------------------------------------
      if (lovwr) stat='unknown'

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! convert filenames to absolute
!-----------------------------------------------------------------------
      if (fileout .ne. 'no') then
         laeout=index(fileout,'  ')-1
         call abspath(fileout,laeout)
      end if

      return 
      end subroutine input

!#######################################################################


      end program template
