!***********************************************************************
!
!                      AUSWERT subroutine TwoPDens
!
!     This routine computes the 2-particle density from the A-vector.
!
!     09/09 HDM
!
!
!***********************************************************************
      
      module twopdensmod

use decimal, only: long,dop,sip

      integer(long), save :: ss,noev,mm1,mm2,ebtype

      end module twopdensmod

      program twopdens

use decimal, only: long,dop,sip
use global
use globmemmod
use aglobal
use versions
use dvrdatmod
use griddatmod
use psidef
use channels
use datenmod
use logdat
use memdimmod

use iofile, only: rdpsiinfo
use zeigauswmod, only: zeigausw
use iogrddef, only: griddat
use iopsidef, only: psidat
use iorst, only: rstinfo
use rdfilesmod, only: rdpsi
use twopdensmod

      implicit none

      integer(long)     :: zeig,zeig1,zeig2,m,s,ilbl, &
                           chkdvr,chkgrd,chkpsi,chkdat,chkprp,mcovr
      character(len=c5) :: filename
      logical(kind=4)   :: lerr,linwf

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      call twopdinput

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='rdacoeff.log',form='formatted')
      logisopen=.true.

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      if (ebtype .lt. 0) then
         call rdmemdim(irst)
      else 
         call rdmemdim(ipsi)
      endif

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef

      if( ebtype .lt. 0 ) then
!-----------------------------------------------------------------------
! read data from restart file  (ebtype = -1  -> restart rather than psi)
!-----------------------------------------------------------------------
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkprp=1
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         if (lerr) then 
            write(6,'(a)') ' ERROR reading restart file' 
            stop
         end if
      else
!-----------------------------------------------------------------------
! read data from psi file
!-----------------------------------------------------------------------
         read(ipsi) filever(ipsi)
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkdat=1
         call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
         filename='psi'
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif

      if (nmode .eq. 1) then
         do s=1,nstate
            dim(1,s)=1
         enddo
      endif
      
!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
!-----------------------------------------------------------------------
      lpsi=.true.
      lpsigrd=.false.
      call zeigausw

      zeig=1
      do s=1,nstate
         zeig1=block(s)
         do m=1,nmode
            zeig=zeig+dim(m,s)*dim(m,s)
            zeig2=zeig1
            zeig2=zeig2+zeig1
            workcdim=max(workcdim,zeig2)
         enddo
      enddo

      call griddat
      call psidat
      
!-----------------------------------------------------------------------
! assign memory arrays
!-----------------------------------------------------------------------
      call memausw
      mcovr=mcdim
      mcdim=mcovr+dmatdim

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      workcdim = max(workcdim,sdim**4+2*sdim**2)
      workrdim = max(workrdim,4*sdim**2)

      mcworkc = mcdim
      mcdim   = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)
if(allocated(agmat))deallocate(agmat)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))
allocate(agmat(agmatdim))

      call twopd(mc(mcpsi),ms(mspsi),mc(mcovr),mc(mcworkc),mr(mrworkr),&
           mi(miworki),mi(mijindx),mi(mitrajst),agmat,mc(mcadgwp),mi(migwpdep))

deallocate(mc,mr,mi,ml,ms,mf,agmat)

      stop
  999 continue
      message= 'Error reading :'//filename(1:c5-15)
      write(6,'(a)') message
      stop 1

      contains

!***********************************************************************
!    Does the work 
!***********************************************************************

      subroutine twopd(psi,spsi,ovr,workc,workr,worki,jindx,trajst,agmat, &
                       adgwp,gwpdep)

!use plot

      implicit none

      integer(long), dimension(workidim), intent(out) :: worki
      integer(long), dimension(adim), intent(out)     :: jindx
      integer(long) :: i,dgl,k,s1,s2,s,m1,m2,zdim,dd,mm1a,mm1b
      integer(long), dimension(gwpinfodim), intent(out) :: trajst,gwpdep
      complex(sip), dimension(spsidim), intent(out)   :: spsi
      complex(dop), dimension(dgldim), intent(out)    :: psi
      complex(dop), dimension(dmatdim), intent(out)   :: ovr
      complex(dop), dimension(workcdim), intent(out)  :: workc
      complex(dop), dimension(gsmatdim), intent(out)  :: adgwp
      complex(dop), dimension(agmatdim), intent(out)  :: agmat
      real(dop), dimension(workrdim), intent(out)     :: workr
      real(dop) ::    time,dt

!-----------------------------------------------------------------------
! calculate time step for data
!-----------------------------------------------------------------------
      dt = out2/fs
      time = tinit/fs
      k    = 0
      if(ss.gt.0) then
         if(ss.gt.nstate) then
            write(6,'(a,i3,a,i3,a)') 'Illegal value for s =', ss, &
                                     ' , (nstate =',nstate,')'
            stop
         endif
      endif
   10 continue
!-----------------------------------------------------------------------
! ebtype = -1   --> read restart rather than psi file.
!-----------------------------------------------------------------------
      if (ebtype .lt. 0) then
!-----------------------------------------------------------------------
! read restart wavefunction
!-----------------------------------------------------------------------
         read (irst,err=998) time
         time = time/fs
         read (irst,err=998) (psi(dgl),dgl=1,dgldim)
      
      else 
         call rdpsi(ipsi,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
         if (lend) goto 77
         if(k.lt.nskip) goto 11
         if( mod(k-nskip,step) .ne. 0 ) goto 11
      endif

      if(mm1.ge.nmode) then
         write(6,'(a,2i4,a)')'mm1 must be .lt. mode', mm1, nmode,' STOP'
         stop
      endif
      if(mm2.gt.nmode) then
         write(6,'(a,2i4,a)')'mm2 must be .le. mode', mm2, nmode,' STOP'
         stop
      endif

      if(ss.gt.0) then
         s1=ss
         s2=ss
      else
         s1=1
         s2=nstate
      endif
      if(mm1.eq.0) then
         mm1a=1
         mm1b=nmode-1
      else
         mm1a=mm1
         mm1b=mm1
      endif

      do s = s1, s2
         do m1 = mm1a, mm1b
            do m2 = m1+1, nmode
               if(mm2*(m2-mm2) .eq. 0) then
                  write(6,'(a,i3,a,i3,a,i3,a,i4)') &
                   ' s =',s,',  m1 =',m1,',  m2 =',m2,',   k =',k+1
                  dd=dim(m1,s)*dim(m2,s)
                  zdim=block(s)/(vdim(m1,s)*dd*ndim(m2,s))
                  call calc2pd(psi(zpsi(s)),vdim(m1,s),zdim,ndim(m2,s), &
                               dim(m1,s),dim(m2,s),dd,noev, &
                               workc,workr,workc(dd*dd+1),workr(dd+1))
               endif
            enddo
         enddo
         write(6,'(76a)') ('-',i=1,76)
      enddo

 11   k = k + 1
      time=time + dt

!-----------------------------------------------------------------------
! if more data, go to begining of loop
!-----------------------------------------------------------------------
      if (ebtype.ge.0 .and. k.lt.ncut)  goto 10

 77   continue
      close(ipsi)

      write(6,'(76a)') ('-',i=1,76)
      
      return
         
 998  continue
      write(6,'(a)') ' Error reading restart file'
      stop   
      end subroutine twopd

!#######################################################################

      subroutine calc2pd(psi,vdim,zdim,ndim,dim1,dim2,dd,noev, &
                         mat,eigen,workc,workr)

      implicit none

      integer(long), intent(in) :: vdim,zdim,ndim,dim1,dim2,dd
      integer(long) :: i,j,k,l,ij,kl,v,z,n,ierr,noev
      complex(dop), dimension(vdim,dim1,zdim,dim2,ndim), intent(out) :: psi
      complex(dop), dimension(dd,dd), intent(out) :: mat
      complex(dop), dimension(2*dd), intent(out)  :: workc
      real(dop), dimension(3*dd), intent(out)  :: workr
      real(dop), dimension(dd), intent(out)  :: eigen
      real(dop) :: esum,ee,entr

      ij=0
      do i= 1, dim1
         do j = 1, dim2
            ij = ij + 1
            kl=0
            do k= 1, dim1
               do l = 1, dim2
                  kl = kl + 1
                  mat(ij,kl) = (0.d0,0.d0)
                  do v=1,vdim
                     do z=1,zdim
                        do n=1,ndim
                           mat(ij,kl) = mat(ij,kl) - &
                                        dconjg(psi(v,i,z,j,n))*psi(v,k,z,l,n)
                        enddo
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo
!-----we have computed the negative of rho2, such that the eigenvalues
!-----are correctly ordered (beginning with largest value).

      call zheev('N','U',dim1*dim2,mat,dd,eigen,workc,2*dd,workr,ierr)
      if(noev.eq.0) then
         write(6,'(3p5f15.7)') (-eigen(i),i=1,dd)
         write(6,*) 
      endif 
      esum = 0.d0
      entr = 0.d0
      do i=1,dd
         esum=esum-eigen(i)
      enddo
      do i=1,dd
         ee=max(1.d-50,-eigen(i)/(esum+1.d-30))
         entr=entr - ee*log(ee)
      enddo
      write(6,'(a,3pf13.7,a,0pf12.8/)') 'Sum of eigenvalues =', &
               esum,',    Entropy =', entr
      return
      end subroutine calc2pd

!#######################################################################

      subroutine twopdinput

      implicit none

      integer(long) :: i, iarg, myiargc, ierr
      character(len=80) :: buf

!-----------------------------------------------------------------------

      filein  = ' '
      name   = './'
      ebtype  = 0
      ss = 0
      mm1=0
      mm2=0
      nskip=0
      noev=0
      ncut=99999999
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-f') then
            i=i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf

         else if( buf .eq. '-n' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -n', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if

         else if( buf .eq. '-skip' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nskip
            if (ierr .ne. 0  .or.  nskip .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -skip', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if

         else if( buf .eq. '-cut' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ncut
            if (ierr .ne. 0  .or.  ncut .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -cut', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if

         else if( buf .eq. '-s' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ss
            if (ierr .ne. 0  .or.  ss .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -s', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if

         else if( buf .eq. '-m1' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) mm1
            if (ierr .ne. 0  .or.  mm1 .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -m1', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if

         else if( buf .eq. '-m2' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) mm2
            if (ierr .ne. 0  .or.  mm2 .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -m2', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if

         else if( buf .eq. '-r' .or. buf .eq. '-rst') then
            irst   = ipsi
            ebtype = -1
            write(6,'(a)') ' *** Reading the RESTART file !  ***'

         else if( buf .eq. '-noev' ) then
            noev = 1

         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,2a)') &
            'Purpose: Computes the 2-particle density in the basis of', &
               ' the SPFs.'
            write(6,'(1x,a)') &
            'Usage:   twopdens [-f -i -skip -cut -n -s -r -ver -h -?] .'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a/,9x,a)')               &
                     ' -f FILE : The wavefunction '    &
                    ,'is read from file '              &
                    ,'  FILE rather than from ./psi'   &
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(a)') ' -i DIR  : data is read from directory DIR'
            write(6,'(2a)')' -n step : Compute the 2-particle-density', &
           ' only every step-th output.'
            write(6,'(2a)') &
            ' -skip n : Skip the first n WFs when computing the ', &
                       '2-particle-density.'
            write(6,'(2a)') &
            ' -cut n  : Consider only the first n WFs when computing ', &
                        'the 2-p-density.'
            write(6,'(a)')  &
            ' -s  s   : Compute the 2-particle-density only for state s.'
            write(6,'(a)') ' -m1 mm1 : Set the first mode index to mm1.'
            write(6,'(a)') ' -m2 mm2 : Set the second mode index to mm2.'
            write(6,'(a)') ' -noev   : Do not print the eigenvalues.'
            write(6,'(a)') &
            ' -r      : Take the restart file rather than the psi file.'
            write(6,'(a)') ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ', &
                     ' -?      : Print this help text. '
            write(6,'(/2a)') 'It is in general useful to pipe the', &
            ' output into less:' 
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      
      if (iarg .lt. i) goto 200

 200  continue

      if( step .gt.1  .and. ebtype .lt. 0) then
         write(6,'(a)') ' -n and -r options are incompatible'
         stop
      end if

      if(mm1.gt.0 .and. mm2.gt.0 .and. mm1.ge.mm2) then
         write(6,'(a,2i4,a)') 'mm1 must be .lt. mm2', mm1, mm2,'  STOP'
         stop
      endif

!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
! Otherwise take name from filein
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         if (ebtype .ge. 0) then
            filein=name(1:laenge)//'/psi'
         else 
            filein=name(1:laenge)//'/restart'
         endif
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      open(ipsi,file=filein(1:laein),form='unformatted',status='old',err=1000)
      write(6,'(2a)') ' The inputfile is : ',filein(1:laein)

      return 
      
 1000 write(6,*)' Cannot open file : '
      write(6,*) filein(1:laein)
      stop
      
      end subroutine twopdinput

      end program twopdens
