!***********************************************************************
!
!                     ANALYSE ROUTINE VMINMAX
!
!     This routine determines some (num) highest and lowest  
!     potential energy values and their grid-position. 
!     It reads the pes file und then runs over the full primitive
!     product grid. Thus it will take some time, if the primitive 
!     product grid is large (more than 10^8 points, say).  
!     The maximal grid size is 2*10^9 points, because of integer
!     overflow (integer*4, 32-bit architecture).
! 
! HDM June 01
!
!***********************************************************************

      program vminmax

use decimal, only: long,dop,sip
use global
use globmemmod
use aglobal
use maxv, only: maxdim,maxsta,maxkoe
use dvrdatmod
use griddatmod
use operdef
use psidef
use channels
use datenmod
use compdatmod
use versions
use keyunitsmod

use op1lib, only: cpvxi
use logdat, only: errormsg
use rddvrmod, only: dvrinfo
use rdopermod, only: operinfo
use zeigauswmod, only: zeigausw
use memhopsmod, only: memhops

use module_interface

use rdopermod, only: rdoper
use rddvrmod
use auswutil, only: sortdown,sortup
use operutil, only: vcalcmlst,vcalc1st
use psiutil, only: decoder,f2mdecoder
use basismod

      implicit none

      integer(long)                     :: chkdvr,chkgrd,chkdat,ilbl,chkpsi,chkprp
      character(len=c5)                 :: filename
      logical(kind=4)                   :: lerr
      integer(long)                     :: sadia, sdia1, sdia2, nham, f
      common /vmmblk/ sadia, sdia1, sdia2, nham

      character(len=c5) :: trajectory, compare
      logical(kind=4)   :: ltrajectory, lcompare, lstartup
      logical(kind=4)   :: check=.false.
      common /trajcomp/ trajectory,compare,ltrajectory,lcompare,lstartup
      character(len=240)      :: host

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      trajectory = " "
      compare = " "
      ltrajectory = .false.
      lcompare = .false.
      call default
      call adefault

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare files to be read
! set flag to .true. if file is to be read
!
!     lrddvr    DVR file
!     lrdoper   OPER file.
!-----------------------------------------------------------------------
      lrddvr=.true.
      lrdoper=.true.

!-----------------------------------------------------------------------
! read input options
!-----------------------------------------------------------------------
      call vmminput

!-----------------------------------------------------------------------
! get array dimensions 
!-----------------------------------------------------------------------
      inquire(ioper,opened=check)
      if (check) close(ioper)
      filename=name(1:laenge)//'/oper'
      ilbl=index(filename,' ')-1
      open(ioper,file=filename(1:ilbl),form='unformatted',status='old',err=1000)
      call rdmemdim(ioper)
      close(ioper)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef

!-----------------------------------------------------------------------
!  Read the system information
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!  Opening the DVR file.
!-----------------------------------------------------------------------
      write(6,'(/,a80)') host
      filename=dname(1:dlaenge)//'/dvr'
      open(idvr,file=filename,form='unformatted',status='old',err=1000)
      ilbl=index(filename,' ')-1
      write(6,'(2a)') ' The DVR file is : ',filename(1:ilbl)
      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      if (lerr) go to 1010
      close(idvr)

!-----------------------------------------------------------------------
!  Opening the oper and output files.
!-----------------------------------------------------------------------
      filename=operfile
      open(ioper,file=filename,form='unformatted',status='old',err=1000)
      ilbl=index(filename,' ')-1
      write(6,'(2a)') ' The operator file is : ',filename(1:ilbl)
      chkdvr=2
      chkgrd=1
      call operinfo(lerr,chkdvr,chkgrd)
      if (lerr) go to 1010
      close(ioper)

      if (lwrsrf) then
         filename=name(1:laenge)//'/srf.asc'
         laeout=index(filename,' ')-1
         call abspath(filename,laeout)
         if (lovwr) then
            open(idat,file=filename,form='formatted',status='unknown')
         else
            open(idat,file=filename,form='formatted',status='new',&
                 err=1020)
         endif
      endif

      if (fileout .eq. 'no') then
         laeout=2
         open(iaus,file='/dev/null',form='formatted',status='unknown')
      else
         if (fileout .eq. ' ') fileout=name(1:laenge)//'/vmm'
         laeout=index(fileout,' ')-1
         call abspath(fileout,laeout)
         filename=fileout
         if(lovwr) then
            open(iaus,file=filename,form='formatted',status='unknown',&
                 err=1020)
         else
            open(iaus,file=filename,form='formatted',status='new',&
                 err=1020)
         endif
      endif

      write(iaus,*)
      write(iaus,'(a80)') host
      write(iaus,'(1x,a,/)') name(1:laenge)

!-----------------------------------------------------------------------
!  Printing basis set information.
!-----------------------------------------------------------------------
      write(6,'(2a)') ' The outputfile is : ',fileout(1:laeout)
      write(6,'(a,i4)') ' Number of degrees of freedom:', ndof       
      write(6,'(a,i4)') ' Number of electronic states :', nstate
      write(iaus,'(a,i4)') ' Number of degrees of freedom:', ndof       
      write(iaus,'(a,i4)') ' Number of electronic states :', nstate

      write(6,'(/,a)')    ' dof  dim   DVR     modelabel'
      write(iaus,'(/,a)') ' dof  dim   DVR     modelabel'
      do f=1,ndof
         write(6,'(1x,i3,i5,3x,a8,a10)') &
              f, gdim(f), cbas(basis(f)), modelabel(f)  
         write(iaus,'(1x,i3,i5,3x,a8,a10)') &
              f, gdim(f), cbas(basis(f)), modelabel(f)  
      enddo
      write(6,*)
      write(iaus,*)

      if(nstate.eq.1) then
         sadia=1
         sdia1=1
         sdia2=1
      endif
      if(sadia .gt. nstate) then
         write(6,'(a,i3,a,i3)') ' There are only nstate =',nstate,&
            ' electronic states, but you are requesting adia =',&
            sadia 
         stop   
      endif
      if(sdia1.gt.nstate .or. sdia2.gt.nstate) then
         write(6,'(a,i3,a,i3,a,i3)') ' There are only nstate =',nstate,&
            ' states, but you are requesting dia =',sdia1,',',sdia2 
         stop   
      endif

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Declare resources required (other than those needed to read a file)
!
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! ltrafo : FBR/DVR trafo matrices
!-----------------------------------------------------------------------
      ncomp=0
      lpsi=.false.
      lpsi1=.false.
      lpsigrd=.false.
      ldmat=.false.
      ldmat1=.false.
      lort=.true.
      lgpop=.false.
      ltrafo=.false.

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! open input and output data files
!
! use standard channels if possible
!  irho : gridpop file
!  ipsi : psi file
!  iaut : auto file
!  ichk : check file
!  iaus : output file
!-----------------------------------------------------------------------
! psifile is opened in pesinput if required
!     filename='psi'
!     open(ipsi,file=filename,form='unformatted',status='old',err=1000)

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Read system information from property file(s)
! chkflags = 0   read information and ignore
!            1   read information and store
!            2   read information and check
! First read file version number
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1

!-----------------------------------------------------------------------
! read information from system files
!-----------------------------------------------------------------------
      if (lrddvr) then
         chkdvr=2
         filename=dname(1:dlaenge)//'/dvr'
         open(idvr,file=filename,form='unformatted',status='old',&
              err=1000)
         call dvrinfo(lerr,chkdvr)
         close(idvr)
         call chksyserr(filename,lerr)
         if (lerr) go to 1010
      endif
      if (lrdoper) then
         chkdvr=2
         chkgrd=2
         filename=operfile
         open(ioper,file=filename,form='unformatted',status='old',&
              err=1000)
         call operinfo(lerr,chkdvr,chkgrd)
         close(ioper)
         call chksyserr(filename,lerr)
         if (lerr) go to 1010
      endif
      
      write(6,'(/,4x,a)')'zham   khtot   kzahl   nmulpot   nham'
      write(6,'(5i8,/)') zham(nham),khtot(nham),kzahl(nham),&
                         nmulpot(nham),nham 
      write(ilog,'(/,4x,a)') 'zham   khtot   kzahl   nmulpot   nham'
      write(ilog,'(5i8,/)') zham(nham),khtot(nham),kzahl(nham),&
                         nmulpot(nham),nham 
      write(iaus,'(/,4x,a)') 'zham   khtot   kzahl   nmulpot   nham'
      write(iaus,'(5i8,/)') zham(nham),khtot(nham),kzahl(nham),&
                           nmulpot(nham),nham 

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call zeigausw
      call memausw

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      mcworkc = mcdim
      mcdim   = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
! assign memory for operator
!-----------------------------------------------------------------------
      call memhops

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

      call findminmax(mr(mrhops),mr(mrort))

deallocate(mc,mr,mi,ml,ms,mf)
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef

      stop
 1000 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop
 1010 continue
      ilbl=index(filename,' ')-1
      message = 'Error reading file: '//filename(1:ilbl)
      call errormsg
      stop
 1020 write(6,*)' Cannot open the output file : '
      ilbl=index(filename,' ')-1
      write(6,*) filename(1:ilbl)
      write(6,*) 'Use option -w to overwrite the output file.'
      stop
      contains


!#######################################################################

      subroutine findminmax(hops,ort)

      implicit none

      integer(long), parameter :: maxvv = 250 ! here 'maxv' change to 'maxvv' 
                                              ! to remove conflict with 'maxv' 
                                              ! module!
      integer(long), external                          :: slen
      integer(long)                                    :: f,f1,zeig,m,i,j,&
                                                          chkdvr,chkgrd,ierr,&
                                                          totgrid,ndof1,t1,t2,&
                                                          iprint,isrf,igrd, &
                                                         sadia,sdia1,sdia2,nham
      integer(long)                                    :: iaus2
      integer(long), dimension(1)                      :: idum
      integer(long), dimension(maxdim)                 :: fpoint,mpoint,fsort
      integer(long), dimension(maxdim,maxvv)           :: minpoint,maxpoint
      real(dop), external                              :: fcputime,fsystime
      real(dop)                                        :: tcpu,tsys,ttot,atotgrd,&
                                                          v,vc
      real(dop)                                        :: diffmean, diffrms, &
                                                          pesvmean, pesvrms, &
                                                          vcmean, vcrms
      real(dop)                                        :: diffmin, diffmax, &
                                                          pesvmin, pesvmax, &
                                                          vcmin, vcmax
      real(dop)                                        :: invcm, uev
      real(dop), dimension(hopsdim), intent(out)       :: hops
      real(dop), dimension(ortdim), intent(out)        :: ort
      real(dop), dimension(1)                          :: rdum
      real(dop), dimension(maxvv)                      :: vvmin,vvmax
      real(dop), dimension(maxsta)                     :: adiab
      real(dop), dimension(maxsta,maxsta)              :: diab,rotmat
      complex(dop), dimension(1)                       :: zdum
      character(len=80)                                :: buffer

      common /vmmblk/ sadia, sdia1, sdia2, nham

      character(len=c5) :: trajectory, compare, buf
      logical(kind=4)   ::  ltrajectory, lcompare, lstartup, lstart
      common /trajcomp/ trajectory,compare,ltrajectory,lcompare,lstartup

      lstart = .true.
      buf = " "

!.... statistics
      pesvmean = 0.d0
      pesvrms = 0.d0
      diffmean = 0.d0
      diffrms = 0.d0
      vcmean = 0.d0
      vcrms = 0.d0
      lstartup = .true.

!.... additional I/O channels
      isrf = 71
      igrd = 72

      step = min(step,maxvv)

!-----------------------------------------------------------------------
! read DVR and operator data
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1

      dvrdata(1)=.true.
      call rddvr(ort,rdum,rdum,rdum,zdum,zdum,idum,zdum,zdum,idum,&
                 idum,rdum,chkdvr)

      if (lrdoper) then
         operfile=operfile
         call rdoper(hops,chkdvr,chkgrd)
      endif

!-----------------------------------------------------------------------
! set PES unit 
!-----------------------------------------------------------------------
      eunit = 1.0_dop
      call keyunits(eunit,unitlab(1),ierr)
      if(ierr.ne.0) then
         write(6,*) 'Unknown unit : ', unitlab(1)
         stop
      endif
      invcm = 1.d0
      call keyunits(invcm,"cm-1",ierr)
      if(ierr.ne.0) then
         write(6,*) 'Something went wrong while getting the '
         write(6,*) "numerical value of 'cm-1'. "
         stop
      endif
      uev = 1.d0
      call keyunits(uev,"ev",ierr)
      if(ierr.ne.0) then
         write(6,*) 'Something went wrong while getting the '
         write(6,*) "numerical value of 'ev'. "
         stop
      endif

!-----------------------------------------------------------------------
! Set defaults for vvmin, vvmax. 
!-----------------------------------------------------------------------
      do i = 1, maxvv
         vvmin(i) =  1.0e+9_dop - i
         vvmax(i) = -1.0e+9_dop + i
      enddo

!-----------------------------------------------------------------------
! Remove electronic degree of freedom by setting the mapping fsort. 
!-----------------------------------------------------------------------
      if(feb.gt.0) then
         ndof1=0  
         do f = 1, ndof
            if(f.ne.feb) then
               ndof1=ndof1+1
               fsort(ndof1) = f
            endif
         enddo
      else
         ndof1=ndof
         do f=1,ndof
            fsort(f)=f
         enddo
      endif 

!-----------------------------------------------------------------------
! determine total grid dimension of not a trajectory is used
!-----------------------------------------------------------------------
      totgrid = 1
      atotgrd = 1.0_dop
      if (.not. ltrajectory) then
         do f1 = 1, ndof1
            f=fsort(f1)
            atotgrd = atotgrd*gdim(f)
            if( atotgrd .gt. 2147483647.0_dop ) then
               write(6,'(a/,a/,a)')  'The primitive grid is too large!',&
               'Integer overflow. More than 2,147,483,647 grid points.',&
               'Use a coarser grid.'
               do i = f1+1, ndof1
                  f=fsort(i)
                  atotgrd = atotgrd*gdim(f)
               enddo
               write(6,*)    ' Total griddim = ', atotgrd
               stop
            endif
            totgrid = totgrid*gdim(f)
         enddo
         write(6,'(a,i11,e14.4)')     ' Total griddim = ',totgrid,atotgrd
         write(iaus,'(a,i11,e14.4,/)')' Total griddim = ',totgrid,atotgrd
         call myflush(iaus)
      endif

!-----------------------------------------------------------------------
! calculate surface 
!-----------------------------------------------------------------------
      i=0
      if (ltrajectory) then
         write(*,*)
         write(*,*) "Determine trajectory size..."
         write(*,*) "File: "//trim(trajectory)
         write(ilog,*) "File: "//trim(trajectory)
         open(igrd,file=trajectory,status='old',err=999)
         totgrid = 0
!........scan length of grid file
 11      continue
         read(igrd,'(a)',err=13,end=12) buf
         buf = adjustl(buf)
         if (buf(1:1) .eq. "#") then
            if (lstart) then
               buf(1:1) = " "
               lstart = .false.
               write(*,'(a)') "File info:"//trim(buf)
               write(ilog,'(a)') "File info:"//trim(buf)
            end if
            goto 11
         end if
         totgrid = totgrid + 1
         goto 11
 12      continue
         close(igrd)
         write(*,*) "No. of points: ##", totgrid
         write(ilog,*) "No. of points: ##", totgrid
         write(*,*)
         open(igrd,file=trajectory,status='old')
      end if
      if (lcompare) then
         write(*,*) "Enegies file: "//trim(compare)
         write(ilog,*) "Enegies file: "//trim(compare)
         open(isrf,file=compare,status="old")
         read(isrf,'(a)',err=14) buf
         buf = adjustl(buf)
         if (buf(1:1) .eq. "#") then
            buf(1:1) = " "
            write(*,'(a)') "File info:"//trim(buf)
            write(ilog,'(a)') "File info:"//trim(buf)
         end if
         write(*,*)
         rewind(isrf)
      end if

      iprint = 100000
      do i = 1, totgrid
         if (ltrajectory) then
 16         continue
            read(igrd,'(a)',err=13) buf
            buf = adjustl(buf)
            if (buf(1:1) .eq. "#") goto 16

            read(buf,*,err=13) (fpoint(j),j=1,ndof)

            do j=1,ndof
               if (fpoint(j).gt. gdim(j)) goto 15
               if (fpoint(j).lt. 1) goto 15
            end do
            !convert DOF index to mode index
            call f2mdecoder(fpoint,mpoint)

         else
            call decoder(i,mpoint,fpoint)
         end if

         if(nstate.eq.1 .and. nmulpot(nham) .eq. 0) then
            call vcalc1st(hops,v,mpoint,nham)
!...........vcalc1st is faster than vcalcmltst.
         else
            call vcalcmlst(hops,adiab,diab,rotmat,v,mpoint,fpoint)
         endif

         if (i .eq. 1) then
            pesvmin = v
            pesvmax = v
            if (.not. lcompare) lstartup = .false.
         else
            if (v .lt. pesvmin) pesvmin = v
            if (v .gt. pesvmax) pesvmax = v
         end if
         pesvmean = pesvmean + v
         pesvrms = pesvrms + v**2

         if (lcompare) then
17          continue
            read(isrf,'(a)',err=14) buf
            buf = adjustl(buf)
            if (buf(1:1) .eq. "#") goto 17
            read(buf,*) vc

            if (i .eq. 1) then
               vcmin = vc
               vcmax = vc
               diffmin = v-vc
               diffmax = v-vc
               lstartup = .false.
            else
               if (vc .lt. vcmin) vcmin = vc
               if (vc .gt. vcmax) vcmax = vc
            end if

            vcmean = vcmean + vc
            vcrms = vcrms + vc**2

            v = v - vc

            if (v .lt. diffmin) diffmin = v
            if (v .gt. diffmax) diffmax = v

            diffmean = diffmean + v
            diffrms = diffrms + v**2
         end if

!........always write energy in a.u. (the way readsrf expects it)
         if (lwrsrf) write(idat,*) v

         v = v/eunit

         if(v.gt.vvmax(1)) then
            vvmax(1) = v
            call cpvxi(fpoint,maxpoint,maxdim)
            call sortup(vvmax,maxpoint,maxdim,step)
         endif
         if(v.lt.vvmin(1)) then
            vvmin(1) = v
            call cpvxi(fpoint,minpoint,maxdim)
            call sortdown(vvmin,minpoint,maxdim,step)
         endif

         if( mod(i,iprint) .eq. 0 ) then
!.........determine actual time and CPU-time each 'ipoint' points.
            call fzeit(buffer)
            m = index(buffer,':')
            buffer=buffer(m-9:m+6)
            ttot=fcputime(0)+fsystime(0)
            write(iaus,'(a,i11,4x,a16,f18.1,a)') &
                 'No of points done:',i, buffer, ttot,' sec.'
            call myflush(iaus)
            if(i .eq.      10000000 ) then
               iprint =     5000000
            elseif(i .eq.  30000000 ) then
               iprint =    10000000
            elseif(i .eq.  60000000 ) then
               iprint =    20000000
            elseif(i .eq. 100000000 ) then
               iprint =    50000000
            elseif(i .eq. 300000000 ) then
               iprint =   100000000
            elseif(i .eq. 600000000 ) then
               iprint =   200000000
            endif
         endif

      enddo
      go to 20

 13   continue
      write(6,*) 'ERROR: Error while reading indices from file'
      write(6,*) trajectory
      write(6,'(a,i8,a)') "after ",i, " gridpoints. Writing output."
      close(igrd)
      goto 20
 14   continue
      write(6,*) 'ERROR: Error while reading srf from file'
      write(6,*) compare
      write(6,'(a,i8,a)') "after ",i, " values. Writing output."
      close(isrf)
      goto 20
 15   continue
      write(6,'(a,i8)') 'ERROR: Index out of range: sample ', i
      write(6,'(a,i8,a)') 'DOF ', j, ' of file'
      write(6,*) compare
      write(6,'(a)') "Writing output."
      close(isrf)
      goto 20

 20   continue
      close(idat)

      if(nstate.gt.1) then
         write(6,*)
         if(sdia1.gt.0) then
            write(6,'(a,i2,a,i2,a)') ' Analysing the (',&
                     sdia1,',',sdia2,') diabatic state.'
            write(iaus,'(/a,i2,a,i2,a)') ' Analysing the (',&
                     sdia1,',',sdia2,') diabatic state.'
         else
           write(6,'(a,i3,a)') ' Analysing the', sadia,&
                               '-th adiabatic state.'
           write(iaus,'(/a,i3,a)') ' Analysing the', sadia,&
                                 '-th adiabatic state.'
         endif
      endif
      write(6,*)
      if (lcompare) then
         write(6,*) ' MAXIMUM OF DIFFERENCE ', &
              '[', unitlab(1)(1:slen(unitlab(1))),']'
      else
         write(6,*) ' MAXIMUM OF PES [', &
              unitlab(1)(1:slen(unitlab(1))),']'
      end if
      write(6,'(6x,a6,7x,9(a7,1x))') 'Energy', &
               (modelabel(fsort(f1)),f1=1,ndof1)
      write(iaus,*)
      if (lcompare) then
         write(iaus,*) ' MAXIMUM OF DIFFERENCE ', &
              '[', unitlab(1)(1:slen(unitlab(1))),']'
      else
         write(iaus,*) ' MAXIMUM OF PES [', &
              unitlab(1)(1:slen(unitlab(1))),']'
      end if

      write(iaus,'(6x,a6,7x,9(a7,1x))') 'Energy', &
                  (modelabel(fsort(f1)),f1=1,ndof1)
      do i = 1, step
!         call decoder(maxpoint(i),mpoint,fpoint)
         call cpvxi(maxpoint(1,i),fpoint,maxdim)
         write(6,'(f14.6, 2x,9f8.3)') vvmax(i), &
              (ort(zort(fsort(f1))+fpoint(fsort(f1))-1),f1=1,ndof1)
         write(iaus,'(f14.6, 2x,9f8.3)') vvmax(i), &
              (ort(zort(fsort(f1))+fpoint(fsort(f1))-1),f1=1,ndof1)
      enddo
      write(iaus,'(6x,a6,7x,9(a7,1x))') 'Energy', &
               (modelabel(fsort(f1)),f1=1,ndof1)
      write(6,'(6x,a6,7x,9(a7,1x))') 'Energy', &
               (modelabel(fsort(f1)),f1=1,ndof1)

      write(6,*)
      if (lcompare) then
         write(6,*) ' MINIMUM OF DIFFERENCE ', &
              '[', unitlab(1)(1:slen(unitlab(1))),']'
      else
         write(6,*) ' MINIMUM OF PES [', &
              unitlab(1)(1:slen(unitlab(1))),']'
      end if
      write(6,'(6x,a6,7x,9(a7,1x))') 'Energy', &
               (modelabel(fsort(f1)),f1=1,ndof1)
      write(iaus,*)



      write(iaus,'(6x,a6,7x,9(a7,1x))') 'Energy', &
               (modelabel(fsort(f1)),f1=1,ndof1)
      if (lcompare) then
         write(iaus,*) ' MINIMUM OF DIFFERENCE ', &
              '[', unitlab(1)(1:slen(unitlab(1))),']'
      else
         write(iaus,*) ' MINIMUM OF PES [', &
              unitlab(1)(1:slen(unitlab(1))),']'
      end if
      do i = 1, step
!         call decoder(minpoint(i),mpoint,fpoint)
         call cpvxi(minpoint(1,i),fpoint,maxdim)
         write(6,'(f14.6, 2x,9f8.3)') vvmin(i), &
              (ort(zort(fsort(f1))+fpoint(fsort(f1))-1),f1=1,ndof1)
         write(iaus,'(f14.6, 2x,9f8.3)') vvmin(i), &
              (ort(zort(fsort(f1))+fpoint(fsort(f1))-1),f1=1,ndof1)
      enddo
      write(6,'(6x,a6,7x,9(a7,1x))') 'Energy', &
               (modelabel(fsort(f1)),f1=1,ndof1)
      write(iaus,'(6x,a6,7x,9(a7,1x))') 'Energy', &
               (modelabel(fsort(f1)),f1=1,ndof1)

!-----------------------------------------------------------------------
! Get overall timing 
!-----------------------------------------------------------------------
      tcpu = fcputime(0)
      tsys = fsystime(0)
      ttot = tcpu+tsys
      t1   = ttot/3600
      ttot = ttot - 3600*t1
      t2   = ttot/60
      ttot = ttot - 60*t2

      write(iaus,'(//,a,/)') '***** Overall Timing *****'
      write(iaus,'(a,1x,f12.2)') 'Total cpu time     [s] : ',tcpu
      write(iaus,'(a,1x,f12.2)') 'Total system time  [s] : ',tsys
      write(iaus, 900) 'Total time     [h:m:s] : ',t1,t2,ttot
      write(iaus,*)
      write(6,'(//,a,/)') '***** Overall Timing *****'
      write(6,'(a,1x,f12.2)') 'Total cpu time     [s] : ',tcpu
      write(6,'(a,1x,f12.2)') 'Total system time  [s] : ',tsys
      write(6, 900) 'Total time     [h:m:s] : ',t1,t2,ttot
      write(6,*)

      do i=1,2
         iaus2= iaus
         if (i.eq.1) iaus2 = 6

         write(iaus2,*) "***** Statistics *****"
         write(iaus2,*)
         write(iaus2,'(a)') "PES file:"
         write(iaus2,901) "  Minimum = ",pesvmin/invcm, &
              " cm-1, ", pesvmin/uev, " eV, ",pesvmin, " au"
         write(iaus2,901) "  Maximum = ",pesvmax/invcm, &
              " cm-1, ", pesvmax/uev, " eV, ",pesvmax, " au"
         write(iaus2,901) "  Mean =    ",pesvmean/dble(totgrid)/invcm, &
              " cm-1, ", pesvmean/dble(totgrid)/uev, &
              " eV, ", pesvmean/dble(totgrid), " au"
         write(iaus2,901) "  RMS  =    ",sqrt(pesvrms/dble(totgrid))/invcm, &
              " cm-1, ", sqrt(pesvrms/dble(totgrid))/uev, &
              " eV, ",  sqrt(pesvrms/dble(totgrid)), " au"
         write(iaus2,*)

         if (lcompare) then
            write(iaus2,'(a)') "External file:"
            write(iaus2,901) "  Minimum = ",vcmin/invcm, &
                 " cm-1, ", vcmin/uev, " eV, ",vcmin, " au"
            write(iaus2,901) "  Maximum = ",vcmax/invcm, &
                 " cm-1, ", vcmax/uev, " eV, ",vcmax, " au"
            write(iaus2,901) "  Mean =    ",vcmean/dble(totgrid)/invcm, &
                 " cm-1, ", vcmean/dble(totgrid)/uev, &
                 " eV, ", vcmean/dble(totgrid), " au"
            write(iaus2,901) "  RMS =     ",sqrt(vcrms/dble(totgrid))/invcm, &
                 " cm-1, ", sqrt(vcrms/dble(totgrid))/uev, &
                 " eV, ", sqrt(vcrms/dble(totgrid)), " au"
            write(iaus2,*)
            write(iaus2,'(a)') "Difference (PES - External):"
            write(iaus2,901) "  Minimum = ",diffmin/invcm, &
                 " cm-1, ", diffmin/uev, " eV, ",diffmin, " au"
            write(iaus2,901) "  Maximum = ",diffmax/invcm, &
                 " cm-1, ", diffmax/uev, " eV, ",diffmax, " au"
            write(iaus2,901) "  Mean =    ",diffmean/dble(totgrid)/invcm, &
                 " cm-1, ", diffmean/dble(totgrid)/uev, &
                 " eV, ",  diffmean/dble(totgrid), " au"
            write(iaus2,901) "  RMS =     ",sqrt(diffrms/dble(totgrid))/invcm, &
                 " cm-1, ", sqrt(diffrms/dble(totgrid))/uev, &
                 " eV, ", sqrt(diffrms/dble(totgrid)), " au"
            write(iaus2,*)

         end if
      enddo

! --- WRITE DATE AND MACHINE  ---
      call fpstartup(buffer)
      write(iaus,*)
      write(iaus,'(a80)') buffer
      write(iaus,'(1x,a)') name(1:laenge)
      write(6,*)
      write(6,'(a80)') buffer
      write(6,'(1x,a,/)') name(1:laenge)

  900 format( a,i4,' :',i3,' :',f5.1)
  901 format(a,f16.3,a,f16.8,a,f16.8,a)
      return

 999  continue
      write(iaus,'(a)') 'Unable to open file'
      write(iaus,'(a)') trajectory
      write(6,'(a)') 'Unable to open file'
      write(6,'(a)') trajectory
      stop 1

 998  continue
      write(iaus,'(a)') 'Unable to open file'
      write(iaus,'(a)') compare
      write(6,'(a)') 'Unable to open file'
      write(6,'(a)') compare
      stop 1

      return
      end subroutine findminmax

!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------

      subroutine vmminput

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: i,f,iarg,ilbl,chkdvr,chkgrd,ierr,&
                                 sadia,sdia1,sdia2,nham,tmplen
      logical(kind=4)         :: lerr
      character(len=240)      :: buf
      character(len=c5)       :: filename

      common /vmmblk/ sadia, sdia1, sdia2, nham

      character(len=c5)  :: trajectory, compare
      logical(kind=4) ::  ltrajectory, lcompare, lstartup
      common /trajcomp/ trajectory,compare,ltrajectory,lcompare,lstartup

      operfile = ' '
      fileout  = ' '
      name     = './'
      iarg = myiargc()
      unitlab(1) = 'ev' 
      step  = 100
      sadia = 1
      sdia1 = 0
      sdia2 = -1
      nham  = 1
      lwrsrf = .false.

! --- WRITE DATE AND MACHINE  ---
      write(6,'(78a1)')  ('-',i=1,78)
      call fpstartup(host)


      i = 0
 100  i=i+1
      if (iarg .lt. i) goto 200
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if(buf .eq. '-i') then
            i = i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf(1:3) .eq. '-p ') then
            i = i+1
            call mygetarg(i,buf)
            operfile = buf
         else if( buf(1:2) .eq. '-o' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if(buf .eq. '-c') then
            i = i+1
            lcompare = .true.
            call mygetarg(i,buf)
            compare = buf
         else if(buf .eq. '-t') then
            i = i+1
            ltrajectory = .true.
            call mygetarg(i,buf)
            trajectory = buf
         elseif(buf .eq. '-w') then
            lovwr = .true.
         elseif(buf .eq. '-u') then
            i = i+1
            call mygetarg(i,buf)
            unitlab(1) = buf
         elseif(buf .eq. '-s') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) sadia
            if (ierr .ne. 0 .or. sadia.le.0) then
               write(6,'(2a,/a)') 'Expecting a positive integer ',&
                    'after -s, not :', buf
               stop
            endif
        elseif(buf .eq. '-d') then
            sadia=0
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ilbl) sdia1
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) sdia2
            ierr=ierr+ilbl
            if (ierr .ne. 0 .or. sdia1.le.0 .or. sdia2.le.0) then
               write(6,'(2a,/a)') 'Expecting two positive integers ',&
                    'after -d, not :', buf
               stop
            endif
         elseif(buf .eq. '-ham') then
            i = i+1         
            call mygetarg(i,buf)         
            read(buf,*,iostat=ierr) nham 
            if (ierr .ne. 0 .or. nham.le.0) then
               write(6,'(2a,/a)') 'Expecting a positive integer ',&
                    'after -ham, not :', buf
               stop         
            endif         
         elseif(buf .eq. '-n') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0 .or. step.le.0) then
               write(6,'(2a,/a)') 'Expecting a positive integer ',&
                    'after -n, not :', buf
               stop
            endif
         elseif(buf .eq. '-srf') then
            lwrsrf = .true.
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(1x,a)')&
       'Purpose: Finds the "num" lowest and largest points on a PES.'
            write(6,'(1x,2a)')&
       'Usage:   vminmax [ -i -p -o -n -s -d -ham -u -w -c -t -ver -h -?]'
            write(6,'(/,a)') 'Options  : '
            write(6,'(2a)') ' -i DIR  : DIR is taken as',&
                            '  name-directory rather than ./'
            write(6,'(2a,/,9x,a)') &
                     ' -p FILE : the PES is read '&
                    ,'from FILE rather than from ./oper',&
            '  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a,/,9x,a)') &
                     ' -o FILE : The ouput is written to file ',&
                     'FILE rather than to name/vmm',&
            '  The string FILE may be a relative or a full path-name.',&
            '  If FILE=no, i.e. -o no, no output file will be written.'
            write(6,'(a,/,10x,a,i4,a)') &
                     ' -n num  : Print num maximal and minimal values.',&
                     ' The default is num =',step,' . Maximum is 250.'
            write(6,'(2a)')&
                     ' -s s    : Print maximal and minimal values for',&
                     ' adiabatic state s. (Def: s=1).'
            write(6,'(2a)')&
                     ' -d s1 s2: Print maximal and minimal values for',&
                     ' diabatic state (s1,s2).'
            write(6,'(2a)')&
                     ' -ham nhm: Use the Hamiltonian nhm.',&
                     ' Default is nhm=1 (system).'
            write(6,'(a/a)')&
                     ' -srf    : write full potential to ASCII file '&
                     //'"srf.asc" (one energy ',&
                     '           per line, for use as readsrf input).'
            write(6,'(a)') &
                     ' -u UNIT : Use energy unit UNIT. (Default: eV).'
            write(6,'(2a)') &
                     ' -w      : Overwrite the output-file vmm.'
            write(6,'(a/a)') &
                     ' -c srf  : Substract potential in file srf ' &
                     //'(one energy', &
                     '           per line, like readsrf input)' &
                     //' before evaluation.'
            write(6,'(a/a)') &
                     ' -t trj  : Do not run over the complete grid but ' &
                     //'use dvr indices', &
                     '           in file trj (n blank separated ' &
                     //'integers per line)'
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(/2a/2a/3a)') 'For a multi-state PES file the',&
                    ' lowest adiabatic surface is analysed by default.',&
                    'To analyse diabatic diagonal or off-diagonal',&
                    ' surfaces use the option -d., E.g. use "-d 1 1" ',&
                    'to analyse the 1st diabatic state, "-d 1 2" ',&
                    'refers to the coupling.'
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         goto 100
      end if
     
  200 continue
      nopt=i-1

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='vmm.log',form='formatted')
      logisopen=.true.
      
      laenge=index(name,' ')-1
      call abspath(name,laenge)

      if (operfile .eq. ' ') operfile=name(1:laenge)//'/oper'
      opflaenge=index(operfile,' ')-1
      call abspath(operfile,opflaenge)

      dname=name
      dlaenge=index(dname,' ')-1
      call abspath(dname,dlaenge)

      if (ltrajectory) then
         tmplen = index(trajectory,' ')-1
         call abspath(trajectory,tmplen)
      end if

      if (lcompare) then
         tmplen = index(compare,' ')-1
         call abspath(compare,tmplen)
      end if

      return 
      
      stop
      
      
      end subroutine vmminput

!#######################################################################
      end program vminmax

