!***********************************************************************
!
!                       AUSWERT ROUTINE WFPROJ
!
!   This routine calculates the expectation value of a projector
!   P = |phi><phi| where phi is a mode function.
!
!   Contents: wfproj,receiver,dvr_oper_in,wfp,fllwfp,wfpinput,rdoptn
!
! HDM 06/08
!
!
!***********************************************************************
      
      program wfproj

use decimal, only: long,dop,sip
use global
use globmemmod
use versions
use aglobal
use maxv, only: maxdim,maxsta
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use fluxmod
use operdef
use hpsimod
use basismod
use rddvrmod
use memdimmod

use op1lib, only: zerovxl
use xvlib, only: vvaxzz
use compdat, only: savecsys, savecpsi
use zeigauswmod, only: zeigausw
use rddvrmod, only: dvrinfo
use rdopermod, only: operinfo
use iopsidef, only: psidat
use auswutil, only: savecdat
use iofile, only: rdpsiinfo
use rdfilesmod, only: rdpsi, rdpsindat
use memhopsmod, only: memhops
use flxprj
use flxreduce, only: reduce
use flxutils, only: ovlp, tout
use mlanalysis, only: overlap_1
use psiutil, only: spfovl

      implicit none


      integer(long) :: zeig,m,s,t1,t2,sdm,np,ilbl
      real(dop) :: fcputime,fsystime,tcpu,tsys,ttot
      external fcputime,fsystime
      character(len=100) :: buffer

!-----------------------------------------------------------------------
! Initialize variables and read input
!-----------------------------------------------------------------------
      lrdoper=.false. ! may be set .true. in wfpinput
      lrddvr =.false. ! may be set .true. in wfpinput
      call default
      call adefault
      call wfpinput0

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(ipsi)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef
      call alloc_compdat

      allocate(zetf_f(maxdim,maxsta),dovmat(maxdim,maxsta))

!-----------------------------------------------------------------------
! Read remaining input
!-----------------------------------------------------------------------
      call wfpinput1

!-----------------------------------------------------------------------
! to allocate memory for psi and psi1
! 1. set no. of comparison data sets for 2 wavefunctions
! 2. save information in comparison data sets
!-----------------------------------------------------------------------
      ncomp=2
      call savecsys(1)
      call savecpsi(1)
      call savecdat(1)
      call savecsys(2)
      call savecpsi(2)
      call savecdat(2)


!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! ltrafo : FBR/DVR trafo matrices
! lsphfbr : jfbr, mfbr and kinfbr matrices.
!-----------------------------------------------------------------------
      ovtype  = 0
      lpsi    = .true.
      lpsi1   = .true.
      lpsigrd = .false.
      lort    = lrddvr
      ltrafo  = lrddvr 
      lsphfbr = lrddvr
      lfft    = lrddvr
      ldmat   = .true.
      ldmat1  = .false.
      lgpop   = .false.
      call zeigausw
      call memausw

      if (ovtype .eq. 0) then
         zeig=1
         do s=1,nstate
            do m=1,nmode
               dovmat(m,s)=zeig
               zeig=zeig+dim(m,s)*dim(m,s)
             enddo
          enddo
          doverdim=zeig-1
          griddim=1

      else if (ovtype .eq. 1) then
         doverdim=1
         griddim=nstate
         do m=1,nmode
            griddim=griddim*subdim(m)
         enddo
      endif

!-----------------------------------------------------------------------
! set up various flags and array information needed to use projectors
!-----------------------------------------------------------------------
      zprj(1) = 1
      do np = 2, nprj
         zprj(np) = zprj(np-1) + subdim(prjmode(np-1))
      enddo
      prjdim = zprj(nprj) + subdim(prjmode(nprj))
      mcprj  = mcdim
      mcdim  = mcprj + prjdim 
      do np = 1, nprj
         if(prjname(np) .eq. 'eigenf' ) then
            sdm = subdim(prjmode(np))
            workrdim = max(workrdim,2*sdm*sdm+sdm)
            workcdim = max(workcdim,2*sdm*sdm)
         endif
      enddo
      workcdim=max(workcdim,2*dgldim)  ! do we need this ?

!-----------------------------------------------------------------------
! set up array information needed for full projection (reduced 1D WF)
!-----------------------------------------------------------------------
      if(full)  then
         fulldim   = kmax*sdmc*(nst2-nst1+1)
      else
         fulldim   = 1
      end if
      mcfullpsi = mcdim
      mcdim     = mcfullpsi+fulldim

!-----------------------------------------------------------------------
! assign work arrays. The variables m?work? must be set for "memhops".
! NB work arrays must be at end of mc, mr, mi, ml, ms, mf arrays.
!-----------------------------------------------------------------------
      if (ovtype .eq. 0) then
         mcdover=mcdim
         mcdim=mcdover+doverdim
         workcdim=max(workcdim,2*maxblock)
      else if (ovtype .eq. 1) then
         mcdover=1
         workcdim=max(workcdim,3*griddim+dgldim)
      endif

      workrdim = max(workrdim,maxgdim*maxgdim)
      workidim = max(workidim,maxgdim)

      mcworkc = mcdim
      mcdim   = mcworkc+workcdim
 
      mrworkr = mrdim
      mrdim   = mrworkr+workrdim
      mrdim   = max(mrdim,4*maxendim)

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
! assign memory for operator
!-----------------------------------------------------------------------
      mrhops=1    ! default, if no operator is read
      if(lrdoper) call memhops

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
      call wrversion(ilog)
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)
if(allocated(agmat))deallocate(agmat)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))
allocate(agmat(agmatdim))

      if(lrdoper.or.lrddvr) call dvr_oper_in(mc,mr,mi)

      call projin(mr(mrhops),mr(mrort),mi(mijsph),mi(mimsph),mc(mcprj),mr(mrtrafo))

      if( .not.full ) call wfp(mc(mcpsi),mc(mcpsi1),ms(mspsi),mc(mcdover), &
                               mc(mcprj),mc(mcworkc),mi(mijindx),mc(mcfullpsi), &
                               agmat,mi(mitrajst),mc(mcadgwp),mi(migwpdep))

      if( full ) call fullwfp(mc(mcpsi),mc(mcpsi1),ms(mspsi),mc(mcdover), &
                              mc(mcprj),mc(mcworkc),mi(mijindx),mc(mcfullpsi))


deallocate(mc,mr,mi,ml,ms,mf)
deallocate(zetf_f,dovmat)

!-----------------------------------------------------------------------
! Get timing and write data to screen and "log" file
!-----------------------------------------------------------------------
      tcpu = fcputime(0)
      tsys = fsystime(0)
      ttot = tcpu+tsys
      t1   = int(ttot)/3600
      ttot = ttot - 3600*t1
      t2   = int(ttot)/60
      ttot = ttot - 60*t2

      write(ilog,'(/a,/)') '***** Overall Timing *****'
      write(ilog,'(a,1x,f12.2)') 'Total cpu time     [s] : ',tcpu
      write(ilog,'(a,1x,f12.2)') 'Total system time  [s] : ',tsys
      write(ilog,900) 'Total time     [h:m:s] : ',t1,t2,ttot
      write(6   ,900) 'Total time     [h:m:s] : ',t1,t2,ttot
  900 format( a,i4,' :',i3,' :',f5.1)

      call fpstartup(buffer)
      write(ilog,'(/a)') ' -- wfproj terminated normally --'
      write(ilog,'(a80)') buffer
      write(6,'(80a)') ('*',s=1,80)

      stop
      
      contains

!***********************************************************************
!    DVR_OPER_IN : reads DVR and oper
!***********************************************************************

      subroutine dvr_oper_in(mc,mr,mi)

use rddvrmod, only: rddvr
use rdopermod, only: rdoper

      implicit none

      complex(dop), dimension(*), intent(inout) :: mc
      complex(dop), dimension(1)                :: zdum1,zdum2,zdum3,zdum4
      real(dop), dimension(*), intent(inout)    :: mr
      real(dop), dimension(1)                   :: rdum1,rdum2,rdum3,rdum4
      integer(long), dimension(*), intent(inout) :: mi
      integer(long), dimension(1)                :: idum

      integer(long) :: chkdvr,chkgrd

!-----------------------------------------------------------------------
! Read DVR data needed  (see rddvr.f for details)
!-----------------------------------------------------------------------
      dvrdata(:)=.false.
      dvrdata(1) = .true.    ! read ort
      chkdvr=1
      if(lrddvr) then
         dvrdata(2) = .true.    ! read trafo
         dvrdata(5) = .true.    ! read hin (fft)
         dvrdata(6) = .true.    ! read rueck (fft)
         dvrdata(7) = .true.    ! read fftfak
         dvrdata(8) = .true.    ! read jsph
         dvrdata(9) = .true.    ! read msph
         dvrdata(11)= .true.    ! read exphin
         dvrdata(12)= .true.    ! read exprueck
         call rddvr(mr(mrort),mr(mrtrafo),rdum2,rdum3,mc(mchin), &
              mc(mcrueck),mi(mifftfak),mc(mcexphin),mc(mcexprueck), &
              mi(mijsph),mi(mimsph),rdum4,chkdvr)
      else
         call rddvr(mr(mrort),rdum1,rdum2,rdum3,zdum1,zdum2, &
              idum,zdum3,zdum4,mi(mijsph),mi(mimsph),rdum4,chkdvr)
      endif

!-----------------------------------------------------------------------
! Read data needed by the operator
!-----------------------------------------------------------------------
      if(lrdoper) then
         chkdvr=2
         chkgrd=1
         call rdoper(mr(mrhops),chkdvr,chkgrd)
      endif

      return
      end subroutine dvr_oper_in

!***********************************************************************

      subroutine wfp(psi,psi1,spsi,dover,projector,workc,jindx,fullpsi, &
                     agmat,trajst,adgwp,gwpdep)

!***********************************************************************
!     calculates the matrix elements <psi(t)|P|psi(t)> where P is
!     a projector onto SPFs, P=|phi><phi|, or a product of those.
!
! Calls to: reduce ovlp 
!***********************************************************************

      implicit none

      integer(long) :: i,k1,k3,kk,chk,tm1,modus
      integer(long), dimension(adim), intent(out)     :: jindx
      integer(long), dimension(gwpinfodim), intent(out) :: trajst,gwpdep
      complex(sip), dimension(spsidim), intent(inout) :: spsi
      complex(dop), dimension(dgldim), intent(inout)  :: psi
      complex(dop), dimension(dgldim), intent(inout)  :: psi1
      complex(dop), dimension(doverdim), intent(out)  :: dover
      complex(dop), dimension(workcdim), intent(out)  :: workc
      complex(dop), dimension(agmatdim), intent(out)  :: agmat
      complex(dop), dimension(gsmatdim), intent(out)  :: adgwp
!      complex(dop) :: ovl
      complex(dop)                                    :: ovl
      complex(dop), dimension(prjdim), intent(in)     :: projector
      complex(dop), dimension(fulldim), intent(out)   :: fullpsi
!      real(dop) :: fcputime,fsystime,tcpu,tsys,rwtt,time
      real(dop) :: fcputime,fsystime,tcpu,tsys,time
      real(dop)                                       :: rwtt
!-----------------------------------------------------------------------
! Get some additional arguments
!-----------------------------------------------------------------------
      tovl = 0.d0
      chk  = 0
      call unixzeit(tm1)

!-----------------------------------------------------------------------
! Write head lines to wtt file.
!-----------------------------------------------------------------------
      write(iwtt,'(/a)') '#   klo    khi   kmax   step'
      write(iwtt,'(a2,i5,3i7)') '# ', klo, khi, kmax, step
      write(iwtt,'(/a,12x,a)') '#  Time[fs]','WFProj'
      call myflush(iwtt)

!-----------------------------------------------------------------------
! Call reduce to compute the reduced wavefunction.
!-----------------------------------------------------------------------
      call reduce(projector,psi,psi1,spsi,jindx,dover,workc,fullpsi)

!-----------------------------------------------------------------------
! Start the loops over time-steps.
!-----------------------------------------------------------------------
      rewind (ipsi)
      read (ipsi)
      time=0.d0
      call rdpsiinfo(ipsi,chk,chk,chk,chk)
       
      do k1=1,klo-1
         read (ipsi)
      enddo

      kk = 0
      do k1=klo,khi,step
         if(mod(k1-klo,5*step).eq.0 ) call tout(k1,klo,time,tm1)
         time = (k1-1)*out2

!------------------------------------------------------------------------
! Read Psi(t=t_k1)
!------------------------------------------------------------------------
         call rdpsi(ipsi,psi1,spsi,jindx,agmat,trajst,adgwp,gwpdep)
         if (lend) goto 77
         do i = 2, min(step,nout2-k1)
            read (ipsi,end=77)
         enddo

!------------------------------------------------------------------------
!   evaluate overlap.
!------------------------------------------------------------------------
         if(lrunml) then
            modus=0
            call spfovl(dover,psi1,psi1,agmat,agmat,modus)
            call overlap_1(psi1,psi1,tree,ovl,dover)
         else
           call ovlp(ovl,psi1,psi1)
         endif

         rwtt = dble(ovl)
         if(laeout.gt.0) write(iwtt,'(f11.3,e22.12)') time/fs,rwtt
         k3 = k1+1

      end do  ! k1-loop   (psi1)

      call tout(k3,klo,time,tm1)    ! k3 = last k1 +1

!-----------------------------------------------------------------------
! Get timing. Will be written in main program.
!-----------------------------------------------------------------------
      tcpu = fcputime(0)
      tsys = fsystime(0)
      tovl = tcpu+tsys

      close(iwtt)
      if(lproj .and. ipsi.eq.ipsi1) then
         close(ipsi,status='delete') ! Delete the reduced WF file. 
      else
         close(ipsi,status='keep') 
      endif
      return

 77   continue
      write(6,'(a)') ' EOF found when reading the psi-file!'

      return
      end subroutine wfp

!***********************************************************************
!     calculates the matrix elements <fullpsi(t)|W|fullpsi(t')>
!     where fullpsi is the fully reduced 1D wavefunction.
!     The wavefunction fullpsi is kept in memory for all out-times.
!     Operators are not allowed when fully projecting.
!***********************************************************************

      subroutine fullwfp(psi,psi1,spsi,dover,projector,workc,jindx,fullpsi)

      implicit none

      integer(long) :: k1,kk2,kk
      integer(long), dimension(adim), intent(out)     :: jindx
      complex(sip), dimension(spsidim), intent(inout) :: spsi
      complex(dop), dimension(dgldim), intent(inout)  :: psi
      complex(dop), dimension(dgldim), intent(inout)  :: psi1
      complex(dop), dimension(doverdim), intent(out)  :: dover
      complex(dop), dimension(prjdim), intent(out)    :: projector
      complex(dop), dimension(fulldim), intent(out)   :: fullpsi
      complex(dop), dimension(workcdim), intent(out)  :: workc
      complex(dop)                                    :: ovl
      real(dop) :: fcputime,fsystime,tcpu,tsys,rwtt,time

!-----------------------------------------------------------------------*
! Definitions:                                                          *
! mcap = mode of the CAP; fcap = degree of freedom of the CAP.          *
! sdmc = subdim(mcap)                                                   *
!-----------------------------------------------------------------------*

!-----------------------------------------------------------------------
! Get some additional arguments
!-----------------------------------------------------------------------
      time = 0.d0
      tovl = 0.d0
      kk   = 0

!-----------------------------------------------------------------------
! Call reduce to compute the reduced wavefunction.
!-----------------------------------------------------------------------
      call reduce(projector,psi,psi1,spsi,jindx,dover,workc,fullpsi)

!-----------------------------------------------------------------------
! Start the loops over time-steps.
!-----------------------------------------------------------------------
      do k1=1,khi
         time = (k1-1)*out2
         kk2 = (k1-1)*sdmc*nstate + 1
         call vvaxzz (fullpsi(kk2),fullpsi(kk2),ovl,sdmc*nstate)
         rwtt = dble(ovl)
         if(laeout.gt.0) write(iwtt,'(f11.3,e22.12)') time/fs,rwtt
      end do                    ! k1-loop   (psi1)

!-----------------------------------------------------------------------
! Get timing. Will be written in main program.
!-----------------------------------------------------------------------
      tcpu = fcputime(0)
      tsys = fsystime(0)
      tovl = tcpu+tsys

      return

      end subroutine fullwfp

!-----------------------------------------------------------------------

      subroutine wfpinput0

      implicit none

      integer(long) :: iarg, myiargc, ilbl, i, j
      integer(long) :: dvrlaenge
      logical(kind=4) :: lgtau
      character(len=80) ::   buf
      character(len=c5) :: filename

!-----------------------------------------------------------------------

      dname   = '.'
      oname   = '.'
      laenge  = 1
      dlaenge = 1
      olaenge = 1
      nppar   = 0
      nprj    = 0
      start   = 0
      start1  = 0
      captype = 2
      nst1    = 1
      nst2    = 0
      do i = 1,maxprj
         prjmode(i) = 0
         prjname(i) = ' '
         prjlab(i)  = ' '
         do j = 1,maxprj
            prjpar(i,j) = -999
         enddo
      enddo

      filein  = './'
      fileout = './'        ! because of flxprj we must choose
      name    = './wfproj'  ! fileout as name-dir and name as output-file. 
      operfile= 'oper'
      operator= ' '
      edstrf  = 'enerd'
      dvrfile = 'dvr'
      capdof  = '  '
      unit    = 'au'
      unite   = 'au'
      unitf   = 'au'
      loper = .false.
      lcorr = .false.
      lproj = .false.
      ltest = .false.
      lwtt  = .true.
      lgtt  = .false.
      lgtau = .false.
      lread = .false.
      lcos  = .false.
      full  = .false.
      ladd  = .false.
      lc3d  = .false.
      ltonly= .false.
      l2short = .false.
      lsmooth = .false.
      klo   = 1
      khi   = 0
      mcap  = 0
      fcap  = 0
      icos  = 0
      ufac  = 1.0d0
      ostart= 0.0d0
      emin  = -1.1d+18
      emax  = -1.1d+18
      epoints = 500
      iarg  = myiargc()
      if (iarg .le. 0) goto 200
      call mygetarg(0,options)
      optlaenge  = index(options,' ')
      do i = 1, iarg
         call mygetarg(i,buf)
         ilbl  = index(buf,' ')
         ilbl = min(ilbl,100-optlaenge)
         if(ilbl.gt.0) options = options(1:optlaenge)//buf(1:ilbl)
         optlaenge = optlaenge + ilbl
      end do
   
!........Read the options
 200  call rdoptn(iarg,buf)
      laeout=index(fileout,' ')-1
      call abspath(fileout,laeout)
      laein=index(filein,' ')-1
      if (filein(laein:laein) .eq. '/') then
         filein = filein(1:laein)//'psi'
         laein = laein + 3
      end if
      call abspath(filein,laein)

      dvrlaenge=index(dvrfile,' ')-1
      if (dvrfile(dvrlaenge:dvrlaenge) .eq. '/') then
         dvrfile = dvrfile(1:dvrlaenge)//'dvr'
         dvrlaenge = dvrlaenge + 3
      end if
      call abspath(dvrfile,dvrlaenge)
      dname=dvrfile(1:dvrlaenge)
      dlaenge=dvrlaenge

      laenge=index(name,' ')-1
      call abspath(name,laenge)

      opflaenge=index(operfile,' ')-1
      if (operfile(opflaenge:opflaenge) .eq. '/') then
         operfile = operfile(1:opflaenge)//'oper'
         opflaenge = opflaenge + 4
      end if
      call abspath(operfile,opflaenge)

      write(6,'(80a)') ('*',i=1,80)
 
!-----------------------------------------------------------------------
!  Open the Log-file.
!-----------------------------------------------------------------------
      filename=fileout(1:laeout)//'/wfproj.log'
      open (ilog,file=filename,form='formatted',err=1000)
      logisopen=.true.
      call fpstartup(filename)
      write(ilog,'(a80)') filename
      call wrversion(ilog)

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      filename=filein(1:laein)
      open(ipsi,file=filename,form='unformatted',status='old',err=1000)
      write(6,'(2a)')   ' The psi-file    is : ',filein(1:laein)
      if(lrdoper) then
         filename=operfile(1:opflaenge)
         open(ioper,file=filename,form='unformatted',status='old',err=1000)
         write(6,'(2a)')' The oper-file   is : ',operfile(1:opflaenge)
      endif

      write(6,'(2a)')   ' The output file is : ',name(1:laenge)
      filename=name(1:laenge)
      if(lovwr) then
         open(unit=iwtt,file=filename,form='formatted',status='unknown',err=1000)
      else
         open(unit=iwtt,file=filename,form='formatted',status='new',err=1001)
      endif

      filename=dvrfile(1:dlaenge)
      open(idvr,file=filename,form='unformatted',status='old',err=1000)
      write(6,'(2a)')   ' The DVR-file    is : ',dvrfile(1:dlaenge)

      return

 1000 write(6,'(a)') ' Cannot open file : '
      write(6,'(a)')  filename
      stop

 1001 write(6,'(a)') ' Cannot open file : '
      write(6,'(a)')  name(1:laenge)
      write(6,'(a)') ' Set -w option to overwrite the output file.' 
      stop

      end subroutine wfpinput0

!-----------------------------------------------------------------------

      subroutine wfpinput1

      implicit none

      integer(long) :: ilbl, jlbl, i, j, m, n, f
      integer(long) :: chkdvr,chkgrd,chkpsi,chkdat, s, ll, slen
      logical(kind=4)   :: lerr
      character(len=80) :: nspf
      character(len=c5) :: mdlbl
      character(len=c2) :: rdtag

!-----------------------------------------------------------------------
!  Read DVR, psi-file and oper-file information.
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      call dvrinfo(lerr,chkdvr)
      close(idvr)
      chkdvr=2
      rewind(ipsi)
      read(ipsi) filever(ipsi)
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      if(psitype.eq.3) then
         routine='wfpinput'
         message='Sorry, wfproj cannot work with ML-wavefunctions.'
         call errormsg
      endif
      call chksyserr(filein,lerr)
      if (lerr) goto 999
      if (lrdoper) then
         chkgrd=2
         call operinfo(lerr,chkdvr,chkgrd)
         close(ioper)
         if (lerr) goto 888
      endif
 
      if (lrdoper) then
         write(6,'(a,f5.1,a,4x,f5.1,a)')            &
            ' program-version =',                   &
            filever(ipsi), ' (psi-file)',           &
            filever(ioper), ' (oper-file)'
      else
         write(6,'(a,f5.1,a)')                      &
            ' program-version =',                   &
            filever(ipsi),  ' (psi-file)'
      endif
      write(6,'(78a1)')  ('-',i=1,78)
      write(6,'(a,i4)') ' Number of degrees of freedom:', ndof
      write(6,'(a,i4)') ' Number of combined modes    :', nmode
      write(6,'(a,i4)') ' Number of electronic states :', nstate
      if( nst2 .le. 0 ) then
         nst2 = nstate
      else
         write(6,'(a,i4)') ' WFProj evaluated for state  :', nst1
      endif

      write(6,'(/,a)') ' mode dof  dim   DVR     modelabel'
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            write(6,'(1x,i3,1x,i4,i5,3x,a8,a10)') &
                       m, f, gdim(f), cbas(basis(f)), modelabel(f)
         enddo
      enddo

!-----------------------------------------------------------------------
! Determine the new dgldim and load zetf_f in case of identical modes.
!-----------------------------------------------------------------------
      dgldimold=dgldim
      do m=1,nmode
         do s=1,nstate
            zetf_f(m,s)=zetf(m,s) ! save old zetf, needed to expand the WF
         enddo
         if(idmode(m).gt.0) then
            lid=.true.
            idmode(m)=0
         endif
      enddo
      call psidat
      dgldimnew=dgldim

!-----------------------------------------------------------------------
!  Determine a mode, macp, without a projector. mcap is needed by flxprj.
!-----------------------------------------------------------------------
      if(nprj.le.0) then
         write(6,'(/a/)') '*** ERROR: No projector specified ***'
         write(2,'(/a/)') '*** ERROR: No projector specified ***'
         stop
      endif
      mcap=0
  333 mcap=mcap+1
      do m=1,nprj
         if (mcap .eq. prjmode(m)) goto 333
         if (prjmode(m).le.0 .or. prjmode(m).gt.nmode) then
            write(6,'(a)') 'ERROR: Illegal projector modes.'
            write(6,'(20i4)')  (prjmode(i),i=1,nprj)
         stop
         endif
      enddo
      if(mcap.gt.nmode) then
         write(6,'(a,i3,a,i3,/a,/a)') 'ERROR:  mcap=',mcap,            &
                 ' is greater than nmode=',nmode,                      &
             ' There must be at least one mode without a projector!',  &
             ' For a complete projection use crosscorr.' 
         stop
      endif
      sdmc=subdim(mcap)

!-----------------------------------------------------------------------
! write mode information.
!-----------------------------------------------------------------------
      write(6,'(/,a)') ' mode: No. of spf''s per state : modelabels'
      do m=1,nmode
         ll = 1
         mdlbl='  '
         nspf ='  '
         do  n=1,nspfdof(m)
            ilbl  = index(modelabel(spfdof(n,m)),' ')
            mdlbl = mdlbl(1:ll)//modelabel(spfdof(n,m))(1:ilbl)
            ll    = ll + ilbl
         end do
         write(nspf,'(15i3)') (dim(m,s),s=1,nstate)
         if(3*nstate+ll .le. 72 ) then
            write(6,'(i4,4a)') m,' :',nspf(1:3*nstate),' : ',mdlbl(1:ll)
         else
            write(6,'(i4,3a)') m,' :',nspf(1:3*nstate),' :'
            write(6,'(a,/)') mdlbl(1:ll)
         end if
      enddo


!-----------------------------------------------------------------------
! Determine number of data sets.
!-----------------------------------------------------------------------
      nout2 = 0
      call rdpsindat(ipsi,nout2)

      tfinal = out2*(nout2-1) + tinit
      write(6,'(/a,i6,a)') ' There are', nout2, &
                          ' data sets on the psi file.'
      write(6,'(a,f7.2,a,f9.2,a,f8.2,a)') &
        ' tinit =', tinit/fs, ' fs,  t-final =', tfinal/fs, &
        ' fs,  tpsi =', out2/fs, ' fs.'

      if(khi .eq. 0) khi = nout2
      if( khi.ne.nout2 .or. klo.ne.1 ) then
      write(6,'(/a,/a,i5,a,f11.2,a,/a,i5,a,f11.2,a)')                     &
      ' times for integration are changed by -lo and/or -hi option to:'   &
      ,' first data point, klo =', klo,', at:',                           &
                     ((klo-1)*out2+tinit)/fs,'fs'                         &
      ,' last  data point, khi =', khi,', at:',                           &
                     ((khi-1)*out2+tinit)/fs,'fs'
      end if

      kmax  = (khi-klo+step)/step
      laewmat = (kmax*(kmax+1))/2
      if(ltest) then
         laewmat = 0
         khi = 0
      end if


!.....Write to log-file.
      if( optlaenge .le. 60 ) then
         write(ilog,'(2a/)')   ' Options : ', options(1:optlaenge)
      else
         write(ilog,'(a,/a/)') ' Options : ', options(1:optlaenge)
      end if

      write(ilog,'(2a)') ' The psi-file    is : ',filein(1:laein)
      write(ilog,'(2a)') ' The output file is : ',fileout(1:laeout)
      write(ilog,'(2a)') ' The DVR-file    is : ',dvrfile(1:dlaenge)
      do i = 1, nprj
         ll=0
         do j = maxprj , 1 , -1
            if(prjpar(j,i) .ne. -999) then
               ll = j
               goto 66
            endif
         enddo
 66      if(prjlab(i) .eq. ' ' ) then
            write(ilog,'(a,i2,a,a12,2x,8i5)')   &
              ' Projector(',prjmode(i),'):',    &
              prjname(i)(1:slen(prjname(i))),   &
              (prjpar(j,i),j=1,ll)
         else
            write(ilog,'(a,i2,a,2a12,8i5)')    &
              ' Projector(',prjmode(i),'):',   &
              prjname(i)(1:slen(prjname(i))),  &
              prjlab(i)(1:slen(prjlab(i))),(prjpar(j,i),j=2,ll)
         endif
      enddo
      if(lid) write(ilog,'(a,2i9)') &
        ' Identical particles: dgldim old/new:', dgldimold, dgldimnew

      call myflush(ilog)
      write(6,'(78a1)')  ('-',i=1,78)

!.....Set logical "full" which is true for full projection, leaving 1D WF.
      if( nmode - nprj .eq. 1 ) full = .true.

      if( nmode - nprj .le. 0 ) then
         write(6,'(a)') 'One must not project onto all modes.'
         write(6,'(a)') 'Use crosscorr in this case.'
         stop
      endif
 
      return

 888  continue
      message= 'Error reading oper-info from:'//operfile(1:opflaenge)
      write(6,'(a)') message
      stop

 998  continue
      message= 'Error reading through:'//trim(filein)
      write(6,'(a)') message
      stop

 999  continue
      message= 'Error reading psi-info from:'//trim(filein)
      write(6,'(a)') message
      stop

      end subroutine wfpinput1

!***********************************************************************
! Read the options.
!***********************************************************************

      subroutine rdoptn(iarg,buf)

      implicit none

      integer(long), intent(in) :: iarg
      integer(long)             :: i, j, ierr
      character(len=80), intent(out) :: buf

      if(iarg .le. 0) goto 400

!........Read the options
      i = 0
 100  i=i+1
      if (iarg .lt. i) goto 400
      call mygetarg(i,buf)

      if( buf(1:1) .eq. '-' ) then
         j = 1
 99      j = j + 1
         if( buf(j:j) .eq. ' ' ) goto 100
!........Options of more than one character length without arguments.
         if( buf(j:j+2) .eq. 'ver' ) then
            call wrversion(6)
            stop

!........Options of more than one character length with arguments.
          else if( buf(j:j+3) .eq. 'dvr ' ) then
            i = i+1
            call mygetarg(i,dvrfile)
        else if( buf(j:j+2) .eq. 'lo ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) klo
            if (ierr .ne. 0  .or.  klo .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -lo', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(j:j+2) .eq. 'hi ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) khi
            if (ierr .ne. 0  .or.  khi .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -hi', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if


!........Options of one character length without arguments
         else if( buf(j:j) .eq. 'w' ) then
            lovwr=.true.
            goto 99
         else if( buf(j:j) .eq. 'h' .or. buf(j:j) .eq. '?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(2a,/8x,a)') &
            'Purpose: Computes the projection of a WF onto one mode,', &
            ' i.e.: <psi|P|psi>,',                                     &
            ' where P=|phi><phi| denotes a projector onto one mode.'
            write(6,'(/,2a,/8x,a)') &
           'Usage  : wfproj [-f -i -o -dvr -n -lo -hi -P ', &
                   '-s -w -ver -h -?] '
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a)')                           &
                     ' -f FILE : The wavefunction is read '  &
                    ,'from file FILE rather than from ./psi' &
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a)')                            &
                     ' -i FILE : The operator is read '       &
                    ,'from file FILE rather than from ./oper' &
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a)')                             &
                     ' -dvr FL : The DVR information is read ' &
                    ,'from file FL rather than from ./dvr'     &
            ,'  The string FL may be a relative or a full path-name.'
            write(6,'(2a,/,11x,a)')                               &
                     ' -o FILE : The output is written to FILE ', &
                     'rather than to ./wfproj .',                  &
            'The string FILE may be a relative or a full path-name.'
            write(6,'(2a)') ' -n step : Compute WFProj only', &
                            ' every step-th psi-output.'
            write(6,'(2a)') ' -s nst  : Compute the WFProj for the', &
                            ' electronic state nst only.'
            write(6,'(2a)') ' -lo klo : Compute WFProj only', &
                            ' from the klo-th psi-output onwards.'
            write(6,'(2a)') ' -hi khi : Compute WFProj only', &
                            ' up to the khi-th psi-output.'
            write(6,'(2a,/10x,2a,/10x,a,/10x,a,/10x,a,/10x,a)')     &
                ' -P      : Use projectors.',                       &
                ' -P is followed by the mode number, the projector', &
                ' name and up to 8 parameters.',                    &
                ' The input is closed by a "%".',                   &
                ' There may be several projectors. Example:',       &
                ' -P 3 KLeg 2 1 % -P 2 eigenf vib 6 %',             &
                ' Most useful is the projector "read".',            &
            ' See the documentation of flux for possible projectors.'
            write(6,'(a)') ' -w      : A WFProj file may be overwritten.'
            write(6,'(a)') ' -ver    : Version information about the program '
            write(6,'(a,/a,/)') &
                     ' -h      : Print this help text. ', &
                     ' -?      : Print this help text. '
            write(6,'(2a)')' Example: ',' wfproj -w -P 2 read ../inwf 1 1 %'
            write(6,'(78a1)')  ('-',i=1,78)
            stop

!........Options of one character length with arguments.
         else if( buf(j:j+1) .eq. 'f ' ) then
            i = i+1
            call mygetarg(i,filein)
         else if( buf(j:j+1) .eq. 'i ' ) then
            i = i+1
            call mygetarg(i,operfile)
         else if( buf(j:j+1) .eq. 'o ' ) then
            i = i+1
            call mygetarg(i,name)   ! because of flxprj we must choose
!                      "fileout" as name-dir and "name" as output-file.
         else if( buf(j:j+1) .eq. 'n ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -n', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(j:j+1) .eq. 's ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nst1
            nst2 = nst1
            if (ierr .ne. 0  .or.  nst1 .le. 0) then
               write(6,'(2a/,a)') &
                 ' The  argument following the option -s', &
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(j:j+1) .eq. 'P ' ) then
            lproj=.true.
            nppar = 0
            nprj = nprj + 1
            if (nprj .gt. maxprj) then
               write(6,*)'Too many projectors. maxprj =', maxprj
               stop
            endif
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) prjmode(nprj)
            if (ierr .ne. 0) then
               write(6,*)'Projector mode must be an integer. Not :'
               write(6,*) buf
               stop
            end if
            i = i+1
            call mygetarg(i,buf)
            call keylowc(buf)
            prjname(nprj) = trim(buf)
            if(buf .eq. 'eigenf') lrdoper=.true.
            if(buf.ne.'eigenf' .and. buf.ne.'read') lrddvr=.true.
 110        i = i+1
            if(nppar .gt. maxprj) then
               write(6,*)'Too many proj-parameters. maxprj =', maxprj
               stop
            endif
            call mygetarg(i,buf)
            if( buf(1:1) .ne. '%' ) then
               nppar = nppar + 1
               ierr=11
               if(ichar(buf(1:1)).ne.47 .and. ichar(buf(1:1)).lt.60) &
                   read(buf,*,iostat=ierr) prjpar(nppar,nprj)
               if(nppar.eq.1  .and.  ierr.gt.0 ) then
                  prjpar(nppar,nprj) = -999
                  prjlab(nprj) = trim(buf)
                  ierr = 0
               end if
               goto 110
            end if
            if (ierr .ne. 0) then
               write(6,*) 'Error occurred while reading projector-parameters'
               stop
            endif
         else
            if(ichar(buf(2:2)).ge.46 .and. ichar(buf(2:2)).le.57) &
                 goto 111     ! Found a negative number, not an option.
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
 111     if (iarg .le. i) goto 400
         goto 100
      end if

 400  return
      end subroutine rdoptn
 
!##############################################################################
      end program wfproj
