!***********************************************************************
      module zeigauswmod

use decimal, only: long,dop
use logdat

      implicit none
      private
      public :: zeigausw
      contains

!***********************************************************************
!
!          MCTDH   subroutine ZEIGAUSW
!
! called from almost all analyse routines.
! calls wkausw and getmaxblock (on this file).
! sets pointer for analyse routines. Usually called before memausw.
!***********************************************************************

      subroutine zeigausw

use global
use aglobal 
use dvrdatmod
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod

use compdat, only: getcsys,getcpsi
use iopsidef, only: gwpdat
use auswutil, only: getcdat

      implicit none

      logical(kind=4)       :: lpsisp,lpsicm

      routine='Zeigausw'

!-----------------------------------------------------------------------
! if comparison data sets are present, check that active data set is
!  first set
!-----------------------------------------------------------------------
      if (ncomp .gt. 0) then
         if (ncomp .gt. maxcomp) then
            routine='Zeigausw'
            write(message,'(a,i5)') &
               'Increase MAXCOMP in compdat.inc. Minimum : ',ncomp
            call errormsg
         endif
         call getcsys(1)
         call getcpsi(1)
         call getcdat(1)
      endif

      spsidim=0
      if (lpsi) then
         lpsisp=lpsiopt(1)
         lpsicm=lpsiopt(2)
         if (lpsisp .or. lpsicm) spsidim=max(spsidim,dgldim)
      endif
      if (lpsi1) then
         lpsisp=clpsiopt(1,2)
         lpsicm=clpsiopt(2,2)
         if (lpsisp .or. lpsicm) spsidim=max(spsidim,cdgldim(2))
      endif

!-----------------------------------------------------------------------
! if GWP modes, set up pointers for wavefunction
!-----------------------------------------------------------------------
      if (lpsi .and. lgwp) call gwpdat

      call wkausw

      call getmaxblock
      fmblock=maxblock

      return
      end subroutine zeigausw

!#######################################################################

      subroutine wkausw

use global
use aglobal
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod

      implicit none

!-----------------------------------------------------------------------
! SIZE OF COMPLEX WORK ARRAY
!-----------------------------------------------------------------------
      workcdim=1
! for rdpsigrid
      if (lpsigrd) then
         workcdim = max(workcdim,griddim)
! for rdpsifull
      else if (lpsi) then
         workcdim = max(workcdim,dgldim)
      endif
      if (lpsi1) then
         workcdim = max(workcdim,cdgldim(2))
      endif

!-----------------------------------------------------------------------
!   SIZE OF REAL WORK ARRAY
!-----------------------------------------------------------------------
      workrdim = 1
      if (lpsi .and. lpsiopt(2)) workrdim=max(workrdim,adim)
      if (lpsi1 .and. clpsiopt(2,2)) workrdim=max(workrdim,cadim(2))

!-----------------------------------------------------------------------
! SIZE OF INTEGER WORK ARRAY
!-----------------------------------------------------------------------
      workidim=1
      if (lpsi .and. citype .gt. 0) workidim=max(workidim,adim)
      if (lpsi1 .and. ccitype(2) .gt. 0) workidim=max(workidim,cadim(2))

!-----------------------------------------------------------------------
! SIZE OF LOGICAL WORK ARRAY
!-----------------------------------------------------------------------
      workldim = 1

      return
      end subroutine wkausw

!#######################################################################

      subroutine getmaxblock

use global
use aglobal
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod

      implicit none

      integer(long)       :: s,t

      maxblock=1

      if (lpsi .and. dentype .eq. 1) then
         do s=1,nstate
            do t=1,nstate
               maxblock=max(maxblock,dblock(s,t))
            enddo
         enddo
      else if (lpsi) then
         do s=1,nstate
            maxblock=max(maxblock,block(s))
         enddo
      endif
      if (lpsi1 .and. dentype .eq. 1) then
         do s=1,nstate
            do t=1,nstate
               maxblock=max(maxblock,cdblock(s,t,2))
            enddo
         enddo
      else if (lpsi1) then
         do s=1,nstate
            maxblock=max(maxblock,cblock(s,2))
         enddo
      endif

      return
      end subroutine getmaxblock


      end module zeigauswmod

