!***********************************************************************
!
!         SUBROUTINE   DIMSPHFBR
!
!  In case of spherical harmonics, gdim(m) must be calculated
!  according to the type of primitive basis used.
!  10/97 MCH
!
!***********************************************************************

      subroutine dimsphfbr (sphtyp,jmax,mmax,mincr,mmin,gj,gdim)

use decimal, only: long
use global
use logdat

      implicit none

      integer(long), intent(in)  :: sphtyp,jmax,mmax,mincr
      integer(long), intent(out) :: mmin,gdim,gj
      integer(long)              :: xx,i,m,j0,j
       
!-----------------------------------------------------------------------
! gj = number of j-points, j0 = smallest j, xx = j-increment
!-----------------------------------------------------------------------
      routine = 'dimsphfbr'

      if (sphtyp.eq.1) then
         gdim = (jmax+1)**2
         mmin = -mmax
         gj   = jmax+1

      elseif (sphtyp.eq.2) then
         mmin = -mmax

         if (mod(jmax,2).eq.0) &
              gdim=(1+jmax)*(1+jmax/2)
         if (mod(jmax,2).eq.1) &
              gdim=(2+jmax)*((1+jmax)/2)
         gj   = (jmax+2)/2

      elseif ((sphtyp.eq.3).or.(sphtyp.eq.4)) then

         if (jmax.lt.mmax) then
            Message = 'jmax must be larger then mmax'
            Call errormsg
         endif

         if (sphtyp.eq.3) then
            xx=2
            j0=mod(jmax,2)
         else
            xx=1
            j0=0
         endif
   
         mmin=mmax
         do while ( ( mmin-mincr ) .ge. -mmax )
            mmin=mmin-mincr
         enddo


         gdim=0
         gj  =0
         do j=j0,jmax,xx
            gj = gj + 1
            do m=-j,j,1
               if (iabs(m).le.mmax) then 
                  do i=mmin,mmax,mincr
                     if (i.eq.m) gdim=gdim+1
                  enddo
               endif
            enddo
         enddo

      else
         Message = 'Wrong type of spherical harmonics basis'
         Call errormsg
      endif

      return 
      end subroutine dimsphfbr



