!***********************************************************************
! 
!             MCTDH subroutine EINPBAS.F
!
! contains eindvr,einpbas,ho_inp,rho_inp,leg_inp,sin_inp,fft_inp,exp_inp,
!          cos_inp,sph_inp,phi_inp,kleg_inp,k_inp,pleg_inp,lagu_inp,
!          ext_inp,pnt_inp,tpnt_inp,gwp_inp,xloc_inp,sinc_inp,
!          wsinc_inp
!
!  01/98 AJ
!***********************************************************************

      module einpbasmod

use decimal, only: long, dop
use versions, only: progname
use maxv, only: maxdim,maxsta
use keyunitsmod
use dvrdatmod
use global 
use logdat
use op1lib,  only: zerovxl
use einingeomod, only: einingeo,einddmct
use iodvrdef, only:dvrdat
use dvrmod_interface
use channels

      implicit none

      integer(long), parameter     :: maxg = 75000
      
      private
      public :: eindvr,einpbas,dvrdefaults
      contains
  
!***********************************************************************
! read primitive-basis-section in input file.
! For direct dynamics an init_geom-section is alternative
!-----------------------------------------------------------------------

      subroutine eindvr

      implicit none

      integer(long)                              :: ic,iz,ierr
      logical(kind=4)                            :: linperr

!-----------------------------------------------------------------------
! set defaults
!-----------------------------------------------------------------------
      call dvrdefaults

! CHECK iin open
      rewind(iin)
      iz=0
      linperr=.false.
      readiin: do

         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .gt. 0) then
            routine = 'Eingabe, primitive-basis-section'
            message = inptit
            call errormsg
         endif

         if (keyword(1) .eq. 'primitive-basis-section' .or. &
              keyword(1) .eq. 'pbasis-section') then
            call einpbas(linperr,iz)
            exit

         else if (keyword(1) .eq. 'initial-geometry-section') then
            if (.not. ldd .and. .not. lgwptrj) then
               routine='Eingabe, primitive-basis-section'
               write(message,'(2a)') &
         'Seems to be a direct dynamics calculation, but DIRECT ', &
          'keyword not specified in RUN-SECTION'
               call errormsg
            endif

            call einingeo(linperr,iz,1)
            exit

         else if (keyword(1) .eq. 'end-input') then
            if (ldd) then
               routine='Eingabe, primitive-basis-section'
               write(message,'(2a)') 'No PRIMITIVE-BASIS-SECTION or ', &
                   'INITIAL-GEOMETRY-SECTION in input file'
               call warnmsg
            else
               routine='Eingabe, basis-section'
               
               call warnmsg
            endif
            exit

         endif

      enddo readiin

      if (linperr) then
         routine = 'Eindvr'
         message = "Error in input for Primitive Basis. See log file"
         call errormsg
      endif

      end subroutine eindvr

!***********************************************************************

      subroutine einpbas(linperr,iz)

      implicit none

      integer(long), intent(in)      :: iz
      integer(long)                  :: i,iz0
      integer(long)                  :: ic,ierr,ierr1,f,jmax,sphtyp,lunit,ic0
      logical(kind=4), intent(inout) :: linperr
      logical(kind=4)                :: lebonly = .false.

!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
      routine = 'read PBASIS-SECTION (einpbasmod.f90)'
      iz0 = 0

!-----------------------------------------------------------------------
! READ PARAMETERS DEFINING PRIMITIVE BASIS
!
! basis(f) : basis type for dof f. (dof=degree of freedom)
!     (0,1,2,3,4,5,6,7,8,9)->(el,HO,Leg,sin,FFT,exp,sphfbr,kleg,k,pleg)
!     (10) -> elcont
!     (-1,11,12,13) -> blank basis (phifbr), rHO, Leg/r, external-DVR
!     (14,15,16,17,18,19) -> cos, Lagu1, Lagu2, Lagu3, Lagu4, point,
!     (20,21,22,25,26,27,31,32) -> kleg/r,pleg/r,tpoint,gwp,xloc,traj,sinc,wsinc
!     (33,34) -> wigner,Lobatto
! gdim(f)  : number of grid points for dof f.
!
! other parameters, specific to basis type, are read and stored in
! ipbaspar(n,f) and rpbaspar(n,f).
! NB check that dimensions of these arrays (in main.inc and default.f) 
! are large enough for the number of arguments.
!-----------------------------------------------------------------------
      f=0
      ndof=0
  110 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      if (keyword(1) .eq. 'end-primitive-basis-section' .or. &
          keyword(1) .eq. 'end-pbasis-section' ) then
         goto 200
      endif

!-----------------------------------------------------------------------
! Get modelabel
!-----------------------------------------------------------------------
      f=f+1
      if(f.gt.maxdim) goto 110
      modelabel(f)=keyorig(1)(1:12)
      lunit=0
      do i=1,f-1
         if(modelabel(f) .eq. modelabel(i) ) then
            message='modelabel doubly defined. '//modelabel(f)
            call errormsg
         endif
      enddo

!-----------------------------------------------------------------------
! For electronic continuum DVR input is in energy units not length
! lunit=0 length units for input
!       1 energy units for input
!-----------------------------------------------------------------------
      if (keyword(1) .eq. 'elcont') then
         febcont=f
         lunit=1
      endif
      
!-----------------------------------------------------------------------
! Electronic Basis
! if normal electronic basis gdim(feb) = no. states
! if continuum states present gdim(feb) = no. grid points in continuum
! (set below).
!-----------------------------------------------------------------------
      if (keyword(2).eq.'el' .or. keyword(2).eq.'elgrid') then
         basis(f) = 0
         ldvr(f) = .false.
         feb = f
         read(keyword(3),*,iostat=ierr) gdim(f)
         if (ierr .eq. 1) then
            message = 'Cannot read gdim for electronic mode :'
            call wrline(zeroi,iz,iz0)
         endif

      else if (keyword(2).eq.'ho') then
         basis(f) = 1
         ldvr(f)  = .true.
         call ho_inp(iz,iz0,ic,f)
      elseif (keyword(2)(1:3).eq.'leg') then
         basis(f) = 2
         if(keyword(2)(4:5).eq.'/r')  basis(f) = 12
         ldvr(f)  = .true.
         call leg_inp(iz,iz0,f)
      else if( keyword(2) == 'sin' ) then
         basis(f) = 3
         ldvr(f)  = .true.
         call sin_inp(iz,iz0,ic,f,lunit)
      elseif (keyword(2).eq.'fft') then
         basis(f) = 4
         ldvr(f)  = .false.
         i=0
! i is not needed here, but is used when fft_inp is called from exp_inp
         call fft_inp(iz,iz0,ic,f,i)
      elseif (keyword(2).eq.'exp') then
         basis(f) = 5
         ldvr(f)  = .true.
         call exp_inp(iz,iz0,ic,f)
      elseif (keyword(2).eq.'sphfbr') then
         basis(f) = 6
         ldvr(f)  = .false.
         call sph_inp(iz,iz0,ic,f,jmax,sphtyp)
      elseif (keyword(2).eq.'phifbr') then
         basis(f) = -1
         ldvr(f)  = .false.
         call phi_inp(iz,iz0,ic,f,jmax,sphtyp)
      elseif (keyword(2)(1:4).eq.'kleg') then
         basis(f) = 7
         if(keyword(2)(5:6).eq.'/r')  basis(f) = 20
         ldvr(f)  = .true.
         call kleg_inp(iz,iz0,f)
      elseif (keyword(2)(1:2).eq.'k ') then
         basis(f) = 8
         ldvr(f)  = .true.
         call k_inp(iz,iz0,ic,f)
      elseif (keyword(2)(1:4).eq.'pleg') then
         basis(f) = 9
         if(keyword(2)(5:6).eq.'/r')  basis(f) = 21
         ldvr(f)  = .true.
         call pleg_inp(iz,iz0,f)

      else if (keyword(2).eq.'elcont') then
         basis(f)=10
         feb=f
         lebcont=.true.
         read(keyword(3),*,iostat=ierr) eltot
         read(keyword(4),*,iostat=ierr) elbnd
! gdim will be changed later to the continumm grid length
         gdim(f) = eltot
         ipbaspar(1,f) = eltot
         ipbaspar(2,f) = elbnd
         if (ierr .eq. 1) then
            message = 'Cannot read eltot for electronic mode :'
            call wrline(zeroi,iz,iz0)
         endif
         if (eltot.lt.1 .or. eltot.gt.maxsta) then
            write(6,'(/a,i4,i8)') 'f, eltot', f,eltot
            message = 'Illegal value of eltot for mode :'
            call wrline(onei,iz,iz0)
         endif

      else if (keyword(2).eq.'rho') then
         basis(f) = 11
         ldvr(f)  = .true.
         call rho_inp(iz,iz0,ic,f)
      elseif (keyword(2)(1:6).eq.'extern') then
         basis(f) = 13
         ldvr(f)  = .true.
         call ext_inp(iz,iz0,ic,f)
      elseif (keyword(2).eq.'cos') then
         basis(f) = 14 
         ldvr(f)  = .true.
         call cos_inp(iz,iz0,ic,f)
      elseif (keyword(2)(1:4).eq.'lagu') then
         basis(f) = 15
         ldvr(f)  = .true.
         if(    keyword(2) .eq. 'lagu1') then
            basis(f) = 15
         elseif(keyword(2) .eq. 'lagu2') then
            basis(f) = 16
         elseif(keyword(2) .eq. 'lagu3') then
            basis(f) = 17
         elseif(keyword(2) .eq. 'lagu4') then
            basis(f) = 18
         endif
         call lagu_inp(iz,iz0,ic,f)
      elseif (keyword(2).eq.'point') then
         basis(f) = 19
         ldvr(f)  = .false.
         lpntdvr=.true.
         call pnt_inp(iz,iz0,f)

      elseif (keyword(2).eq.'tpoint') then
         basis(f) = 22
         ldvr(f)  = .false.
         lpntdvr=.true.
         call tpnt_inp(iz,iz0,ic,f)


      elseif (keyword(2).eq.'gwp') then
         basis(f) = 25
         ic0 = 3
         lgwp=.true.
         call gwp_inp(iz,iz0,ic,f,ic0)
      elseif (keyword(2).eq.'gwpcl') then
         routine = 'Einpbas (einpbaspar.f90)'
         message(1:80) = &
           'Basis type GWPCL no longer used. To set classical GWPs use :'
         message(81:160) = &
           'gwp_model, gwp_model@m, gwpmodel@f keywords in integrator-section.'
         call errormsg
      elseif (keyword(2).eq.'x-local') then
         basis(f) = 26
         lxloc = .true.
         call xloc_inp(iz,iz0,f)
      elseif (keyword(2).eq.'classical') then
         basis(f) = 27
         ipbaspar(1,f) = 1
         lxloc = .true.
         ic0 = 3
         call gwp_inp(iz,iz0,ic,f,ic0)
 
      else if( keyword(2) == 'sinc' ) then
         basis(f) = 31
         ldvr(f)  = .true.
         call sinc_inp(iz,iz0,ic,f,lunit)
      else if( keyword(2) == 'wsinc' ) then
         basis(f) = 32
         ldvr(f)  = .true.
         call sinc_inp(iz,iz0,ic,f,lunit)
      elseif (keyword(2).eq.'wigner') then
         basis(f) = 33
         ldvr(f) = .true.
         call Wig_inp(iz,iz0,f)

      elseif (keyword(2).eq.'lobatto') then
         basis(f) = 34
         ldvr(f)  = .true.
         call lob_inp(iz,iz0,ic,f)

      else
         message='Unknown primitive basis'
         call wrline(zeroi,iz,iz0)
      endif
!-----------------------------------------------------------------------
! end of if loop over primitive bases
!-----------------------------------------------------------------------
      go to 110

  200 continue

!-----------------------------------------------------------------------
! check continuum basis 
! - has it been set?
! - large enough?
!
! then set gdim for the electronic DOF to be the continuum grid length
!
! Should check continuum grid starts at 0
!-----------------------------------------------------------------------
      if (febcont .gt. 0) then
         if (basis(febcont) .lt. 0) then
            message='Continuum basis asked for but not specified '
            call errormsg
         endif
        
      endif

!-----------------------------------------------------------------------
! check feb has been set up even for a 1-state system
!-----------------------------------------------------------------------
      if (progname(1:8) .eq. 'quantics') then
         if (feb .eq. 0) then
            f=f+1
            basis(f) = 0
            ldvr(f) = .false.
            feb = f
            gdim(f) = 1
            modelabel(f) = 'el'
         endif
      endif
      
!-----------------------------------------------------------------------
! set ndof variable
!-----------------------------------------------------------------------
      ndof=f

!-----------------------------------------------------------------------
! ndof <= MAXDIM 
!-----------------------------------------------------------------------
      if( ndof .gt. maxdim ) then
         write(message,'(a,i5)') &
          ' Error in assigning MAXDIM parameter'
         call errormsg
      endif

!-----------------------------------------------------------------------
! Flag a calculation with only an electronic basis
!-----------------------------------------------------------------------
      if (ndof .eq. 1 .and. feb .eq. 1) lebonly=.true.

!-----------------------------------------------------------------------
! for swarm of trajectories, force all modes to be classical, unless
! specified as a poiint basis, i.e. frozen
! (this is case when nswarm = N set in run-section)
!-----------------------------------------------------------------------
      if (ltraj) then
         do f=1,ndof
            if (f .eq. feb) cycle
            if ((basis(f) .gt. 0) .and. (basis(f) .ne. 19)) basis(f)=27
         enddo
      endif

!-----------------------------------------------------------------------
! If nswarm not set, but all DOFs classical, flag as a swarm of 
! trajectories
!-----------------------------------------------------------------------
      if (.not. lebonly) then
         ltraj=.true.
         do f=1,ndof
            if (f .eq. feb) cycle
            if (basis(f) .eq. 19) cycle
            if (basis(f) .ne. 27) ltraj = .false.
         enddo
      endif
       
!-----------------------------------------------------------------------
! calculate dvr related data 
!-----------------------------------------------------------------------
      call dvrdat

!-----------------------------------------------------------------------
! Check for Kleg/K  and PLeg/exp pairs, and Wigner/(exp or K) trios. 
!-----------------------------------------------------------------------
      if (progname(1:8) .eq. 'quantics') then
         do f = 1, ndof-1
            if((basis(f).eq.7 .or. basis(f).eq.20) .and. &
                basis(f+1).ne.8) then 
               ierr1   = 1
               message='"k-DVR" must be after "kleg-DVR"'
               call errormsg
            endif      
            if((basis(f).eq.9 .or. basis(f).eq.21) .and. &
                                   basis(f+1).ne.5) then 
               ierr1   = 1
               message='"exp-DVR" must be after "PLeg-DVR"'
               call errormsg
            endif
            if(basis(f).eq.9 .and. basis(f+1).eq.5) then
               if(abs((rpbaspar(2,f+1)-rpbaspar(1,f+1))* &
                    (1.d0+1.d0/(gdim(f+1)-1))-6.2831853d0) .gt. 1.d-6) then
                  ierr1   = 1
                  message='"exp-DVR" after "PLeg-DVR" must be on 2pi'
                  call errormsg
               endif
            endif
            if (basis(f).eq.6 .and. basis(f+1).ne.-1) then
               ierr1   = 1
               message='"phifbr" must be after "sphfbr"'
               call errormsg
            endif
            if (basis(f) .eq. 33) then
              if ((basis(f+1) .ne. 5) .and. (basis(f+1) .ne. 8)) then
                 ierr1   = 1
                 message='Either "exp" or "K" must be one after "Wig-DVR"'
                 call errormsg
              endif
              if ((basis(f+2) .ne. 5) .and. (basis(f+2) .ne. 8)) then
                 ierr1   = 1
                 message='Either "exp" or "K" must be two after "Wig-DVR"'
                 call errormsg
              endif
            endif
         enddo
      endif
         
!-----------------------------------------------------------------------
! Check for invalid primitive bases for density propagation. 
!-----------------------------------------------------------------------
      if(dentype .gt. 0 ) then
         do f = 1, ndof
            if (basis(f) .eq. 25) cycle
            if (basis(f) .eq. 0) cycle
            if (basis(f) .eq. 4) cycle

            if (.not.ldvr(f)) then
                write(message,'(a,i3,2a)') &
               'The dof',f,' is not a simple DVR, FFT or GWP. ', &
               'This is not allowed for densities.'
                call warnmsg
             endif
         enddo
      endif

!-----------------------------------------------------------------------
! If using DD-MCTDH then check for normal mode symmetries.
!-----------------------------------------------------------------------
      if(lddmctdh)call einddmct(linperr,iz0,1)

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr
      lwarn=.false.

      return
      end subroutine einpbas

!*****************************************************************************

      subroutine ho_inp(iz,iz0,ic,f)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,ic,f
      integer(long)                :: i,ierr,ierr1
      real(dop)                    :: xi,xf,homass,hofreq,hoxeq
      logical lfound

!-----------------------------------------------------------------------
! Harmonic oscillator DVR
! if hoptyp=0 three real parameters define DVR:
! hofreq: harmonic oscillator frequency
! hoxeq : harmonic oscillator eqilibrium position
! homass: harmonic oscillator mass
! if hoptyp=1 two real parameters define DVR:
! xi: coordinate of first grid point
! xi: coordinate of last grid point
!-----------------------------------------------------------------------
! ipbaspar(1) = 0 -> hoxeq, hofreq, homass; 1 -> xi, xf
! rpbaspar(1)=hoxeq     (rpbaspar(1)=xi)
! rpbaspar(2)=hofreq    (rpbaspar(2)=xf)
! rpbaspar(3)=homass
!-----------------------------------------------------------------------

      ierr1    = 0
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).le.1 .or. gdim(f) .gt. maxg) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)
      endif

      i=4
      if (keyword(i) .eq.  'xi-xf') then
         i=i+1
         read(keyword(i),*,iostat=ierr) xi
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xi,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'ho_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         i=i+1
         read(keyword(i),*,iostat=ierr) xf
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xf,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'ho_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         ipbaspar(1,f)=1
         rpbaspar(1,f)=xi
         rpbaspar(2,f)=xf
      else
         call find_parameter(keyorig(i),hoxeq,lfound)
         if(.not.lfound) then
         read(keyword(i),*,iostat=ierr) hoxeq
         if (ierr .ne. 0) ierr1=ierr
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keylunits(hoxeq,keyword(i),ierr)
               if (ierr .eq. 1) then
                  routine = 'ho_inp (einpbasmod.f90)'
                  message = 'no length unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
         endif
         i=i+1
         call find_parameter(keyorig(i),hofreq,lfound)
         if(.not.lfound) then
         read(keyword(i),*,iostat=ierr) hofreq
         if (ierr .ne. 0) ierr1=ierr
            if (keyword(i+1) .eq. ',' ) then
               i = i+2
               call keyeunits(hofreq,keyword(i),ierr)
               if (ierr .eq. 1) then
                  routine = 'ho_inp (einpbasmod.f90)'
                  message = 'no energy unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
         endif
         homass = 1.0_dop
         if (i.lt.ic) then
            i = i+1
            call find_parameter(keyorig(i),homass,lfound)
            if(.not.lfound) then
               read(keyword(i),*,iostat=ierr) homass
               if (ierr .ne. 0) ierr1=ierr
               if(keyword(i+1) .eq. ',' .and. i.lt.ic) then
                  i = i+2
                  call keymunits(homass,keyword(i),ierr)
                  if (ierr .eq. 1) then
                     routine = 'ho_inp (einpbasmod.f90)'
                     message = 'no mass unit after comma'
                     call wrline(zeroi,iz,iz0)
                  endif
               endif
            endif
         endif
         ipbaspar(1,f)=0
         rpbaspar(1,f)=hoxeq
         rpbaspar(2,f)=hofreq
         rpbaspar(3,f)=homass
      endif
      
      if (ierr1 .ne. 0) then
         routine = 'ho_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif
      
      return
      end subroutine ho_inp
      
!***********************************************************************

      subroutine rho_inp(iz,iz0,ic,f)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,ic,f
      integer(long)                :: i,ierr,ierr1
      real(dop)                    :: xi,xf,homass,hofreq,hoxeq,x0
      logical lfound

!-----------------------------------------------------------------------
! Radial harmonic oscillator DVR
! if ipbaspar=0 three real parameters define DVR:
! hofreq: harmonic oscillator frequency
! hoxeq : starting point of the interval
! homass: harmonic oscillator mass
! if ipbaspar=1 two real parameters define DVR:
! xi: coordinate of first grid point
! xi: coordinate of last grid point
! if ipbaspar=2 two real parameters define DVR:
! x0: coordinate of the boundary condiion phi(x0)=0.
! xi: coordinate of last grid point
!-----------------------------------------------------------------------
      ierr1    = 0
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).le.1 .or. gdim(f) .gt. maxg) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)
      endif
      i=4
      if (keyword(i) .eq.  'xi-xf') then
         i=i+1
         read(keyword(i),*,iostat=ierr) xi
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xi,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'rho_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         i=i+1
         read(keyword(i),*,iostat=ierr) xf
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xf,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'rho_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         ipbaspar(1,f)=1
         rpbaspar(1,f)=xi
         rpbaspar(2,f)=xf
      elseif (keyword(i) .eq.  'x0-xf') then
         i=i+1
         read(keyword(i),*,iostat=ierr) x0
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xi,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'rho_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         i=i+1
         read(keyword(i),*,iostat=ierr) xf
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xf,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'rho_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         ipbaspar(1,f)=2
         rpbaspar(1,f)=x0
         rpbaspar(2,f)=xf
      else
         call find_parameter(keyorig(i),hoxeq,lfound)
         if(.not.lfound) then
            read(keyword(i),*,iostat=ierr) hoxeq
            if (ierr .ne. 0) ierr1=ierr
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keylunits(hoxeq,keyword(i),ierr)
               if (ierr .eq. 1) then
                  routine = 'rho_inp (einpbasmod.f90)'
                  message = 'no length unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
         endif
         i=i+1
         call find_parameter(keyorig(i),hofreq,lfound)
         if(.not.lfound) then
            read(keyword(i),*,iostat=ierr) hofreq
            if (ierr .ne. 0) ierr1=ierr
            if (keyword(i+1) .eq. ',' ) then
               i = i+2
               call keyeunits(hofreq,keyword(i),ierr)
               if (ierr .eq. 1) then
                  routine = 'rho_inp (einpbasmod.f90)'
                  message = 'no energy unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
         endif
         homass = 1.0_dop
         if (i.lt.ic) then
            i = i+1
            call find_parameter(keyorig(i),homass,lfound)
            if(.not.lfound) then

               read(keyword(i),*,iostat=ierr) homass
               if (ierr .ne. 0) ierr1=ierr
               if(keyword(i+1) .eq. ',' .and. i.lt.ic) then
                  i = i+2
                  call keymunits(homass,keyword(i),ierr)
                  if (ierr .eq. 1) then
                     routine = 'rho_inp (einpbasmod.f90)'
                     message = 'no mass unit after comma'
                     call wrline(zeroi,iz,iz0)
                  endif
               endif
            endif
         endif
         ipbaspar(1,f)=0
         rpbaspar(1,f)=hoxeq
         rpbaspar(2,f)=hofreq
         rpbaspar(3,f)=homass
      endif
      
      if (ierr1 .ne. 0) then
         routine = 'rho_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      return
      end subroutine rho_inp

!****************************************************************************

      subroutine leg_inp(iz,iz0,f)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,f
      integer(long)                :: ierr,ierr1,blz,ibleg,n
      
!-----------------------------------------------------------------------
! Legendre DVR
!
! blz   : 'magnetic' quantum number of associated Legendre polynomials
! ibleg : integer to control even/odd options
!-----------------------------------------------------------------------
! ipbaspar(1)=blz
! ipbaspar(2)=ibleg
! ipbaspar(3)=gdim         (only for Leg/R)
! rpbaspar(1)=theta-end    (only for Leg/R)
! rpbaspar(2)=theta-begin  (only for Leg/R)
!-----------------------------------------------------------------------
      ierr1    = 0
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).le.1 .or. gdim(f).gt. maxg) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)
      endif

      if (keyword(4)(1:3) .eq. 'jbf') then
         blz = -9999
         do n=1,npar
            if (apar(n) .eq. keyorig(4)) then
               blz=nint(rpar(n))
            endif
         enddo
         if( blz .eq. -9999 ) then
            routine = 'leg_inp (einpbasmod.f90)'
            message='jbf is not set! Use "parameter-section".'
            call wrline(zeroi,iz,iz0)
            blz = 0
         endif
      else
         read(keyword(4),*,iostat=ierr) blz
         if (ierr .ne. 0) ierr1=ierr
      endif
      if(keyword(5) .eq. 'all' ) then
         ibleg = 0
      elseif(keyword(5) .eq. 'even' ) then
         ibleg = 2
      elseif(keyword(5) .eq. 'odd' ) then
         ibleg = 1
      else
         read(keyword(5),*,iostat=ierr) ibleg
         if (ierr .ne. 0) then
            ierr1=ierr
            routine = 'leg_inp (einpbasmod.f90)'
            message='expecting "even" or "odd" or "all" and not: ' &
                 //keyword(5)(1:32)
            call wrline(zeroi,iz,iz0)
         end if
      end if
      if (blz.lt.0.or.ibleg.lt.0.or.ibleg.gt.2) then
         ierr1   = 1
         routine = 'leg_inp (einpbasmod.f90)'
         message = 'Incorrect blz or ibleg parameters'
         call wrline(zeroi,iz,iz0)
      endif
      ipbaspar(1,f)=blz
      ipbaspar(2,f)=ibleg
      
      if(basis(f) .eq. 12 ) then
         ipbaspar(3,f)=gdim(f)
         read(keyword(6),*,iostat=ierr) rpbaspar(2,f)
         if (ierr .ne. 0) ierr1=ierr
         read(keyword(7),*,iostat=ierr) rpbaspar(1,f)
         if (ierr .ne. 0) ierr1=ierr 
         if(rpbaspar(1,f) .le. rpbaspar(2,f) ) then
            routine = 'leg_inp (einpbasmod.f90)'
            message = 'Leg/R :'// &
            ' first restriction number must be lower than second.'
            call wrline(zeroi,iz,iz0)
         endif
      endif
      
      
      if (ierr1 .ne. 0) then
         routine = 'leg_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif
      
      return
      end subroutine leg_inp

!****************************************************************************

      subroutine sin_inp(iz,iz0,ic,f,lunit)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,ic,f,lunit
      integer(long)                :: i,ierr,ierr1,pord
      real(dop)                    :: gdiff,xi,xf,br
      
!-----------------------------------------------------------------------
!     Sine DVR
!
! xi: coordinate of first grid point
! xf: coordinate of last grid point
!-----------------------------------------------------------------------
! rpbaspar(1)=xi
! rpbaspar(2)=xf
! ipbaspar(1)=0,1,2 ; 0=normal, 1=sdq first deriv., 2=spin
!-----------------------------------------------------------------------

      ierr1    = 0
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).le.1 .or. gdim(f).gt. maxg) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)

      endif

      i=4
      If(keyword(4)(1:3) .eq. '2pi' )  then
         if (keyword(4)(4:4) .eq. '/') then
            read(keyword(4)(5:5),*,iostat=ierr) pord
            if(ierr .ne. 0) ierr1=ierr
         endif
         br = 2.0_dop*pi/dble(pord)
         xi =      br/dble(gdim(f)+1)
         xf = br - br/dble(gdim(f)+1)
         i=i+1
         if (i .le. ic)  then
            if(keyword(i) .eq. 'sdq' ) then
               ipbaspar(1,f)=1
            else 
               routine = 'sin_inp (einpbasmod.f90), 2pi special input'
               message = 'Expecting "sdq" or nothing, but not : '
               call wrline(i,iz,iz0)
            endif
         end if
         goto 200
      endif

      i=4
      read(keyword(i),*,iostat=ierr) xi
      if (ierr .ne. 0) ierr1=ierr
      if (keyword(i+1) .eq. ',') then
         i=i+2
         if (lunit .eq. 0) then
            call keylunits(xi,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'sin_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         else if (lunit .eq. 1) then
            call keyeunits(xi,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'sin_inp (einpbasmod.f90)'
               message = 'no energy unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
      endif
      i=i+1
      read(keyword(i),*,iostat=ierr) xf
      if (ierr .ne. 0) ierr1=ierr
      if (keyword(i+1) .eq. ',') then
         i=i+2
         if (lunit .eq. 0) then
            call keylunits(xf,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'sin_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         else if (lunit .eq. 1) then
            call keyeunits(xf,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'sin_inp (einpbasmod.f90)'
               message = 'no energy unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
      endif
      
 100  i=i+1
      if (i .le. ic)  then
         if(keyword(i) .eq. 'sdq' ) then
             ipbaspar(1,f)=1
         elseif(keyword(i) .eq. 'spin' ) then
             ipbaspar(1,f)=2
         elseif(keyword(i) .eq. 'long' ) then
            gdiff = (xf-xi)/dble(gdim(f)+1)
            xf = xf - gdiff
            xi = xi + gdiff
         else if(keyword(i) .ne. 'short') then
            routine = 'sin_inp (einpbasmod.f90)'
            message = 'Expecting "short", "long", "sdq",'// &
                      ' or "spin", but not : '
            call wrline(i,iz,iz0)
         endif
      end if
      if(i.lt.ic) goto 100
      
 200  rpbaspar(1,f)=xi
      rpbaspar(2,f)=xf

      if (ierr1 .ne. 0) then
         routine = 'sin_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      if(ipbaspar(1,f) .eq. 2) then
         if(gdim(f).ne.2 .or. abs(xi+0.5_dop).gt.1.0e-9_dop .or. &
                              abs(xf-0.5_dop).gt.1.0e-9_dop ) then
            routine = 'sin_inp (einpbasmod.f90)'
            message = 'Incorrect input for "spin"'
            call wrline(zeroi,iz,iz0)
         endif
      endif

      return
      end subroutine sin_inp

!****************************************************************************

      subroutine sinc_inp(iz,iz0,ic,f,lunit)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,ic,f,lunit
      integer(long)                :: i,ierr,ierr1,pord
      real(dop)                    :: gdiff,xi,xf,br

!-----------------------------------------------------------------------
!     Sinc DVR
!
! xi: coordinate of first grid point
! xf: coordinate of last grid point
!-----------------------------------------------------------------------
! rpbaspar(1)=xi
! rpbaspar(2)=xf
! ipbaspar(1)=0,1,2 ; 0=normal, 1=sdq first deriv., 2=spin
!-----------------------------------------------------------------------

      ierr1    = 0
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).le.1 .or. gdim(f).gt. maxg) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)

      endif

      i=4
      If(keyword(4)(1:3) .eq. '2pi' )  then
         if (keyword(4)(4:4) .eq. '/') then
            read(keyword(4)(5:5),*,iostat=ierr) pord
            if(ierr .ne. 0) ierr1=ierr
         endif
         br = 2.d0*pi/dble(pord)
         xi =      br/dble(gdim(f)+1)
         xf = br - br/dble(gdim(f)+1)
         i=i+1
         if (i .le. ic)  then
            if(keyword(i) .eq. 'sdq' ) then
               ipbaspar(1,f)=1
            else
               routine = 'sin_inp (einpbas.F), 2pi special input'
               message = 'Expecting "sdq" or nothing, but not : '
               call wrline(i,iz,iz0)
            endif
         end if
         goto 200
      endif

      i=4
      read(keyword(i),*,iostat=ierr) xi
      if (ierr .ne. 0) ierr1=ierr
      if (keyword(i+1) .eq. ',') then
         i=i+2
         if (lunit .eq. 0) then
            call keylunits(xi,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'sinc/wsinc_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         else if (lunit .eq. 1) then
            call keyeunits(xi,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'sinc/wsinc_inp (einpbasmod.f90)'
               message = 'no energy unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
      endif
      i=i+1
      read(keyword(i),*,iostat=ierr) xf
      if (ierr .ne. 0) ierr1=ierr
      if (keyword(i+1) .eq. ',') then
         i=i+2
         if (lunit .eq. 0) then
            call keylunits(xf,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'sinc/wsinc_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         else if (lunit .eq. 1) then
            call keyeunits(xf,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'sinc/wsinc_inp (einpbasmod.f90)'
               message = 'no energy unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
      endif

 100  i=i+1
      if (i .le. ic)  then
         if(keyword(i) .eq. 'long' ) then
            gdiff = (xf-xi)/dble(gdim(f)+1)
            xf = xf - gdiff
            xi = xi + gdiff
         else if(keyword(i) .ne. 'short') then
            routine = 'sinc/wsinc_inp (einpbasmod.f90)'
            message = 'Expecting "short", "long",'// &
                      ' but not : '
            call wrline(i,iz,iz0)
         endif
      end if
      if(i.lt.ic) goto 100

 200  rpbaspar(1,f)=xi
      rpbaspar(2,f)=xf

      if (ierr1 .ne. 0) then
         routine = 'sinc/wsinc_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      if(ipbaspar(1,f) .eq. 2) then
         if(gdim(f).ne.2 .or. abs(xi+0.5d0).gt.1.0d-9 .or. &
                              abs(xf-0.5d0).gt.1.0d-9 ) then
            routine = 'sin_inp (einpbas.F)'
            message = 'Incorrect input for "spin"!  '// &
                      'Use "sin  2  -0.5  0.5  spin"'
            call wrline(zeroi,iz,iz0)
         endif
      endif

      return
      end subroutine sinc_inp


!****************************************************************************

      subroutine fft_inp(iz,iz0,ic,f,i)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(out)   :: i
      integer(long), intent(in)    :: iz,ic,f
      integer(long)                :: ierr,ierr1,pord
      real(dop)                    :: gdiff,xi,xf
      
!-----------------------------------------------------------------------
! FFT (and exp-DVR)
!
! rpbaspar(1)=xi ,  xi: coordinate of first grid point
! rpbaspar(2)=xf ,  xf: coordinate of last grid point
!-----------------------------------------------------------------------

      ierr1    = 0
      pord     = 1
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).le.1 .or. gdim(f).gt.20000) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)
      endif

      If(keyword(4)(1:3) .eq. '2pi' )  then
         i=4
         if (keyword(4)(4:4) .eq. '/') then
            read(keyword(4)(5:5),*,iostat=ierr) pord
            if(ierr .ne. 0) ierr1=ierr
         endif
         xi = 0.0_dop 
         xf = 2.0_dop*pi*dble(gdim(f)-1)/dble(gdim(f)*pord)
      else if(keyword(4)(1:5) .eq. 's-2pi' )  then
         i=4
         if (keyword(4)(6:6) .eq. '/') then
            read(keyword(4)(7:7),*,iostat=ierr) pord
            if(ierr .ne. 0) ierr1=ierr
         endif
         xi = pi/dble(gdim(f)*pord) 
         xf = (2.0_dop*pi/pord - xi)
      else if(keyword(4)(1:5) .eq. 'c-2pi' )  then
         i=4
         if (keyword(4)(6:6) .eq. '/') then
            read(keyword(4)(7:7),*,iostat=ierr) pord
            if(ierr .ne. 0) ierr1=ierr
         endif
         xf = pi*dble(gdim(f)-1)/dble(gdim(f)*pord)
         xi = -xf
      else
         i=4
         read(keyword(i),*,iostat=ierr) xi
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xi,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'fft_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         i=i+1
         read(keyword(i),*,iostat=ierr) xf
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xf,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'fft_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         if (i .lt. ic)  then
            i=i+1
            if(keyword(i) .eq. 'periodic' ) then
               xf = xf - (xf-xi)/dble(gdim(f))
            else if(keyword(i) .eq. 's-periodic') then
               gdiff = (xf-xi)/dble(2*gdim(f))
               xi = xi + gdiff
               xf = xf - gdiff
            else if(keyword(i) .ne. 'linear') then
               routine = 'fft_inp (einpbasmod.f90)'
               message = &
             'Expecting "linear", "periodic" or "s-periodic" and not : '
               call wrline(i,iz,iz0)
            endif
         end if
      endif
            
      rpbaspar(1,f)=xi
      rpbaspar(2,f)=xf

      if (i.lt.ic .and. ((basis(max(1,f-1)).ne.9 .and. &
                         basis(max(1,f-1)).ne.21))) then
         routine = 'fft_inp (einpbasmod.f90)'
         message='Expecting nothing after "periodic" but not :'
         call wrline(i+1,iz,iz0)
         ierr1=1
      endif
      if (ierr1 .ne. 0) then
         routine = 'fft_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      return
      end subroutine fft_inp

!****************************************************************************

      subroutine exp_inp(iz,iz0,ic,f)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: ic,f,iz
      integer(long)                :: i,ierr,ierr1,jmax,kmin,kmax,kvalue
      character(len=4)             :: knum

!-----------------------------------------------------------------------
! Exponential DVR   (calls fft_inp)
!
! ipbaspar(1)=kmin
! ipbaspar(2)=kmax
! ipbaspar(3)=kmax-kmin+1
! ipbaspar(4)=min(abs(kmin),abs(kmax)) .or. ipbaspar(4)=0 if(kmin*kmax<0)
! rpbaspar(1)=xi ,  xi: coordinate of first grid point (set by fft_inp).
! rpbaspar(2)=xf ,  xf: coordinate of last grid point  (set by fft_inp).
!-----------------------------------------------------------------------

      call fft_inp(iz,iz0,ic,f,i)

      ierr1    = 0
      if( mod(gdim(f),2).eq.0) then
         routine = 'exp_inp (einpbasmod.f90)'
         message = 'exp-DVR requires an odd number of grid points'
         call wrline(zeroi,iz,iz0)               
      endif
      

!C--If 'PLeg-DVR' or 'Wigner-DVR' is combined with 'Exp-DVR', set the 
! parameters for the K-grid.
! ipbaspar(1,f-1)=0,1,2 if no-symmetry / odd / even functions used for 
!                 PLeg basis
! ipbaspar(2,f-1)=jtot
!
! NB for PLeg/KLeg, ipbaspar for k grid stored in DOF f rather than f-1
! as was done up to mctdh85
      if (f.gt.1 .and. ((basis(max(1,f-1)).eq.9 .or.  &
                        basis(max(1,f-1)).eq.21) .or. &
                        (basis(max(1,f-1)).eq.33)) .or. &
           (f.gt.2 .and. (basis(max(1,f-2)).eq.33)) ) then

         if ((basis(max(1,f-1)).eq.9) &
             .or. (basis(max(1,f-1)).eq.21) &
             .or. (basis(max(1,f-1)).eq.33)) then
            if (ipbaspar(1,f-1).eq.0) then
               jmax=gdim(f-1)-1
            else
               jmax=gdim(f-1)*2 - ipbaspar(1,f-1)
            endif
            kvalue=min(min(ipbaspar(2,f-1),gdim(f)/2),jmax)
         else    ! 2nd DOF after Wigner
            if (ipbaspar(1,f-2).eq.0) then
               jmax=gdim(f-2)-1
            else
               jmax=gdim(f-2)*2 - ipbaspar(1,f-2)
            endif
            kvalue=min(min(ipbaspar(6,f-2),gdim(f)/2),jmax)
         endif

         i=i+1
         if (i.le.ic .and. keyword(i) .ne. 'k') then
            routine = 'exp_inp (einpbasmod.f90)'
            message='Expecting "k" (or nothing) but not :'
            call wrline(i,iz,iz0)
         endif
         if (keyword(i) .eq. 'k') then
            i=i+1
            if (i .le. ic) then
               if (keyword(i) .eq. '=') then
                  i=i+1
               else
                  routine = 'exp_inp (einpbasmod.f90)'
                  message='Incorrect pbasis definition for mode :'
                  call wrline(onei,iz,iz0)
               endif
               read(keyword(i),*,iostat=ierr) kmin
               i=i+1
               if (i .le. ic) then
                  if (keyword(i) .eq. ',') then
                     i=i+1
                  else
                     routine = 'exp_inp (einpbasmod.f90)'
                     message='Incorrect pbasis definition for mode :'
                     call wrline(onei,iz,iz0)
                  endif
                  read(keyword(i),*,iostat=ierr) kmax
                  if (ierr .ne. 0) ierr1=ierr
               else
                  if (kmin .lt. 0) then
                     routine = 'exp_inp (einpbasmod.f90)'
                     message='Incorrect pbasis definition for mode :'
                     call wrline(onei,iz,iz0)
                  endif
                  kmax=kmin
                  kmin=-kmin
               endif
            endif
         else
            kmin=-kvalue
            kmax=kvalue
         endif
         if (kmin .lt. -kvalue) then
            write(knum,'(I4.1)') -kvalue
            routine = 'exp_inp (einpbasmod.f90)'
            message = &
                 '"kmin in "Exp-DVR" will be increased to '//knum
            call warnmsg1
            write(6,*) message
            kmin=-kvalue
         endif
         if (kmax .gt. kvalue) then
            write(knum,'(I4.1)') kvalue
            routine = 'exp_inp (einpbasmod.f90)'
            message = &
                 '"kmax" in "Exp-DVR" will be decreased to '//knum
            call warnmsg1
            write(6,*) message
            kmax=kvalue
         endif
         if (kmax .lt. kmin) then
            routine = 'exp_inp (einpbasmod.f90)'
            message = &
            'In Exp-DVR "kmax" should be greater than "kmin"'
            call errormsg
         endif
         if (basis(max(1,f-1)).eq.33) then
            ipbaspar(2,f-1)=kmin
            ipbaspar(3,f-1)=kmax
            ipbaspar(4,f-1)=kmax-kmin+1
            if (kmin*kmax .lt. 0.) then
               ipbaspar(5,f-1)=0
            else
               ipbaspar(5,f-1)=min(abs(kmin),abs(kmax))
            endif
         else if (basis(max(1,f-1)).eq.9 .or. basis(max(1,f-1)) .eq. 21) then
            ipbaspar(1,f)=kmin
            ipbaspar(2,f)=kmax
            ipbaspar(3,f)=kmax-kmin+1
            if (kmin*kmax .lt. 0.) then
               ipbaspar(4,f)=0
            else
               ipbaspar(4,f)=min(abs(kmin),abs(kmax))
            endif
         else
            ipbaspar(6,f-2)=kmin
            ipbaspar(7,f-2)=kmax
            ipbaspar(8,f-2)=kmax-kmin+1
            if (kmin*kmax .lt. 0.) then
               ipbaspar(9,f-2)=0
            else
               ipbaspar(9,f-2)=min(abs(kmin),abs(kmax))
            endif
         endif

      elseif (i.lt.ic) then
         i=i+1
         routine = 'exp_inp (einpbasmod.f90)'
         message='Expecting "k" (or nothing) but not :'
         call wrline(i,iz,iz0)
      endif

      if (ierr1 .ne. 0) then
         routine = 'exp_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif
       
      return
      end subroutine exp_inp


!****************************************************************************

      subroutine cos_inp(iz,iz0,ic,f)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,ic,f
      integer(long)                :: i,ierr,ierr1,pord
      real(dop)                    :: xi,xf,x0,br

!-----------------------------------------------------------------------
! cos-DVR
!
! rpbaspar(1)=xi ,  xi: coordinate of first grid point
! rpbaspar(2)=xf ,  xf: coordinate of last grid point
! x0 and x0+br are the left and right coordinates of the "box"
!-----------------------------------------------------------------------

      ierr1    = 0
      pord     = 1
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).le.1 .or. gdim(f).gt. maxg) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)
      endif

      i=4
      If(keyword(4)(1:3) .eq. '2pi' )  then
         if (keyword(4)(4:4) .eq. '/') then
            read(keyword(4)(5:5),*,iostat=ierr) pord
            if(ierr .ne. 0) ierr1=ierr
         endif
         x0 = 0.0_dop
         br = 2.0_dop*pi/dble(pord)
         xi = x0 + br/(2.0_dop*gdim(f))
         xf = br - br/(2.0_dop*gdim(f))
      else
         read(keyword(i),*,iostat=ierr) xi
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xi,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'cos_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         i=i+1
         read(keyword(i),*,iostat=ierr) xf
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xf,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'cos_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
 200     continue
         if (i .lt. ic)  then
            i=i+1
            if(keyword(i) .eq. 'long' ) then
               x0 = xi
               br = xf
               xi = x0 + br/(2.0_dop*gdim(f))
               xf = br - br/(2.0_dop*gdim(f))
            else if(keyword(i) .eq. 'sdq') then
               continue ! dummy read for compatibility with sin-DVR
            else if(keyword(i) .ne. 'short') then
               routine = 'cos_inp (einpbasmod.f90)'
               message = &
             'Expecting "long" or "short" and not : '
               call wrline(i,iz,iz0)
            endif
            goto 200
         end if
      endif

      rpbaspar(1,f)=xi
      rpbaspar(2,f)=xf

      if (i.lt.ic .and. (basis(max(1,f-1)).ne.9 .and.  &
                         basis(max(1,f-1)).ne.21)) then
         routine = 'cos_inp (einpbasmod.f90)'
         message='Expecting nothing after "long|short" but not :'
         call wrline(i+1,iz,iz0)
         ierr1=1
      endif
      if (ierr1 .ne. 0) then
         routine = 'cos_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      return
      end subroutine cos_inp

      
!*****************************************************************************

      subroutine sph_inp(iz,iz0,ic,f,jmax,sphtyp)

      implicit none

      integer(long), intent(out)   :: jmax,sphtyp
      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,ic,f
      integer(long)                :: ierr,ierr1,joff
      real(dop)                    :: thrshld

!-----------------------------------------------------------------------
! Spherical harmonic FBR
!
! sphtyp  : type of spherical harmonics basis :  
!           1: j,m=all (abs(m)<= jmax)
!           2: j=sym and m=all (j=0,2.. or 1,3..)
!           3: j=sym and m=truncated
!                   (j=0,2.. or 1,3..) and (abs(m) <= mmax)
!           4: j=no-sym  and m=truncated  (abs(m) <= mmax)
! jmax    : maximum azimutal quantum number for spherical harmonics
! mmax    : maximum magnetic quantum number for truncated basis
! mincr   : increment of m's, starting from mmax, if m=truncated
!-----------------------------------------------------------------------
! ipbaspar(1,)=sphtyp
! ipbaspar(2)=jmax
! ipbaspar(3)=joff    (off-set)
! ipbaspar(4)=gj      (gj = number of j-points)
! rpbaspar(1)=thrshld
!-----------------------------------------------------------------------

      joff     = 6
      thrshld  = 1.0e-10_dop
      ierr1    = 0
      read(keyword(3),*,iostat=ierr) jmax
      if (ierr .ne. 0) ierr1=ierr
      if (keyword(4).eq.'nosym' .or. keyword(4).eq.'no-sym' .or.  &
           keyword(4).eq.'no_sym') then
         sphtyp=1 
      elseif (keyword(4).eq.'sym' ) then
         sphtyp=2 
      else
         routine = 'sph_inp (einpbasmod.f90)'
         message = &
              'Expecting "sym", or "nosym", but not : '
         call wrline(fouri,iz,iz0)               
      endif
      
      if ( ic.ge.5 ) then 
         read(keyword(5),*,iostat=ierr) thrshld
         if (ierr .ne. 0) ierr1=ierr
         if (ic.ge.6) then
            read(keyword(6),*,iostat=ierr) joff
            if (ierr .ne. 0) ierr1=ierr
         endif
      endif 
      
      if (ierr1 .ne. 0 ) then
         routine = 'sph_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)   
      endif

      ipbaspar(1,f)=sphtyp   !  May be re-set in phi_inp
      ipbaspar(2,f)=jmax
      ipbaspar(3,f)=joff
!     ipbaspar(4,f)=gj          This will be set in phi_inp
      rpbaspar(1,f)=thrshld

      return
      end subroutine sph_inp


!*****************************************************************************

      subroutine phi_inp(iz,iz0,ic,f,jmax,sphtyp)

      implicit none

      integer(long), intent(inout) :: sphtyp,iz0
      integer(long), intent(in)    :: iz,ic,f,jmax
      integer(long)                :: ierr,ierr1,mmax,mincr,mmin,gj
      
!-----------------------------------------------------------------------
! PHIFBR. basis for phi dof in a sphfbr basis. 
!-----------------------------------------------------------------------
! ipbaspar(1)=mmax
! ipbaspar(2)=mincr
! ipbaspar(3)=mmin
!-----------------------------------------------------------------------

      ierr1 = 0
      if ( ic.ge.3 ) then 
         sphtyp=5-sphtyp
         read(keyword(3),*,iostat=ierr) mmax
         if (ierr .ne. 0) ierr1=ierr
         if (ic.ge.4) then
            read(keyword(4),*,iostat=ierr) mincr
            if (ierr .ne. 0) ierr1=ierr
         else
            mincr=1
         endif
      else 
         mmax=jmax
         mincr=1
      endif
      
      if (ierr1 .ne. 0 ) then
         routine = 'phi_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)   
      endif
      if (basis(max(1,f-1)) .ne. 6 ) then
         routine = 'phi_inp (einpbasmod.f90)'
         message = 'phifbr must come after sphfbr'
         call wrline(zeroi,iz,iz0)
      endif
         
! calculation of gdim(f) according to the type of primitive basis used.
      call dimsphfbr(sphtyp,jmax,mmax,mincr,mmin,gj,gdim(f-1))

      gdim(f)=1        
      ipbaspar(1,f-1)=sphtyp
      ipbaspar(4,f-1)=gj
      ipbaspar(1,f)=mmax
      ipbaspar(2,f)=mincr
      ipbaspar(3,f)=mmin

      return
      end subroutine phi_inp


!*****************************************************************************

      subroutine kleg_inp(iz,iz0,f)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,f
      integer(long)                :: ierr,ierr1,ibleg

!-----------------------------------------------------------------------
! Extended Legendre DVR
!
! ibleg : integer to control even/odd options
!-----------------------------------------------------------------------
! ipbaspar(1)=ibleg
! ipbaspar(3)=gdim    only for kleg/r     
!-----------------------------------------------------------------------
      if(progname(1:6) .eq. 'potfit' .or. progname(1:6) .eq. 'mcpotf' ) then
         Write(6,'(/a,/2a,/2a,/2a,/)') '### WARNING ###', &
         'As (MC)POTFIT only needs the grid-points and not the ', &
         'derivative matrices,','it is recommended to replace "KLeg" ', &
         'with "Leg" and add there "0" for','the blz-parameter. ', &
         '(MC)POTFIT may crash with the present input.'
         Write(2,'(/a,/2a,/2a,/2a,/)') '### WARNING ###', &
         'As (MC)POTFIT only needs the grid-points and not the ', &
         'derivative matrices,','it is recommended to replace "KLeg" ', &
         'with "Leg" and add there "0" for','the blz-parameter. ', &
         '(MC)POTFIT may crash with the present input.'
      endif


      ierr1    = 0
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).le.1 .or. gdim(f).gt. maxg) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)
      endif

      if(keyword(4) .eq. 'all' ) then
         ibleg = 0
      elseif(keyword(4) .eq. 'even' ) then
         ibleg = 2
      elseif(keyword(4) .eq. 'odd' ) then
         ibleg = 1
      else
         read(keyword(4),*,iostat=ierr) ibleg
         if (ierr .ne. 0) then
            ierr1=ierr
            routine = 'kleg_inp (einpbasmod.f90)'
            message='expecting "even" or "odd" or "all" and not: ' &
                 //keyword(4)(1:32)
            call wrline(zeroi,iz,iz0)
         end if
      end if
      if (ibleg.lt.0 .or.ibleg.gt.2) then
         ierr1   = 1
         routine = 'kleg_inp (einpbasmod.f90)'
         message = 'Incorrect ibleg parameter'
         call wrline(zeroi,iz,iz0)
      endif
      
      ipbaspar(1,f)=ibleg

      if(basis(f) .eq. 20 ) then
         ipbaspar(3,f)=gdim(f)
         read(keyword(5),*,iostat=ierr) rpbaspar(2,f)
         if (ierr .ne. 0) ierr1=ierr
         read(keyword(6),*,iostat=ierr) rpbaspar(1,f)
         if (ierr .ne. 0) ierr1=ierr
         if(rpbaspar(1,f) .le. rpbaspar(2,f) ) then
            routine = 'kleg_inp (einpbasmod.f90)'
            message = 'KLeg/R :'// &
            ' first restriction number must be lower than second.'
            call wrline(zeroi,iz,iz0)
         endif
      endif

      
      if (ierr1 .ne. 0 ) then
         routine = 'kleg_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      return
      end subroutine kleg_inp

!*****************************************************************************

      subroutine k_inp(iz,iz0,ic,f)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,ic,f
      integer(long)                :: ierr,ierr1,kmin,kmax,dk

!----------------------------------------------------------------------
!  k DVR for completion Extended Legendre DVR
!----------------------------------------------------------------------
! ipbaspar(1)=kmin
! ipbaspar(2)=kmax
! ipbaspar(3)=min(abs(kmin),abs(kmax)) or ipbaspar(3)=0 if kmin*kmax<0
! ipbaspar(4)=max(abs(kmin),abs(kmax))
! ipbaspar(5)=dk    (delta k, default = 1)
!----------------------------------------------------------------------

      ierr1    = 0
      read(keyword(3),*,iostat=ierr) kmin
      if (ierr .ne. 0) ierr1=ierr
      read(keyword(4),*,iostat=ierr) kmax
      if (ierr .ne. 0) ierr1=ierr
      if (kmin.gt.kmax) then
         ierr1   = 1
         routine = 'k_inp (einpbasmod.f90)'
         message = 'kmax must be >= kmin'
         call wrline(zeroi,iz,iz0)
      endif
      
      if (ic .ge. 5) then
         if(keyword(5) .eq. 'all' ) then
            dk = 1
         elseif(keyword(5) .eq. 'even' ) then
            if (mod(kmin,2) .ne. 0) kmin=kmin+1
            dk = 2
         elseif(keyword(5) .eq. 'odd' ) then
            if (mod(kmin,2) .eq. 0) kmin=kmin+1
            dk = 2
         else
            read(keyword(5),*,iostat=ierr) dk
            if (ierr.ne.0 .or. dk.le.0) then
               ierr1=ierr
               routine = 'k_inp (einpbasmod.f90)'
               message='The step for K-DVR must be a positive integer.'
               call wrline(zeroi,iz,iz0)
            end if
         endif
      else
         dk = 1
      end if
      
      gdim(f)=(kmax-kmin)/dk+1
      kmax = kmin + (gdim(f)-1)*dk
      ipbaspar(1,f)=kmin
      ipbaspar(2,f)=kmax
      
      if (kmax*kmin .lt. 0.) then
         ipbaspar(3,f)=0
      else
         ipbaspar(3,f)=min(abs(kmin),abs(kmax))
      endif
      ipbaspar(4,f)=max(abs(kmin),abs(kmax))
      ipbaspar(5,f)=dk
      
! --- If K is combined with Wigner, set the Wigner parameters here

      if (f.gt.1 .and. basis(max(1,f-1)).eq.19) then
        ipbaspar(2,f-1)=ipbaspar(1,f)
        ipbaspar(3,f-1)=ipbaspar(2,f)
        ipbaspar(4,f-1)=gdim(f)
        ipbaspar(5,f-1)=ipbaspar(3,f)
      endif
      if (f.gt.2 .and. basis(max(1,f-2)).eq.19) then
        ipbaspar(6,f-2)=ipbaspar(1,f)
        ipbaspar(7,f-2)=ipbaspar(2,f)
        ipbaspar(8,f-2)=gdim(f)
        ipbaspar(9,f-2)=ipbaspar(3,f)
      endif

      if (ierr1 .ne. 0 ) then
         routine = 'k_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      return
      end subroutine k_inp

!*****************************************************************************

      subroutine pleg_inp(iz,iz0,f)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,f
      integer(long)                :: ierr,ierr1,ibleg

!-----------------------------------------------------------------------
! PLeg DVR
!
! ibleg : integer to control even/odd options
!-----------------------------------------------------------------------
! ipbaspar(1)=ibleg
! ipbaspar(3)=gdim  only for pleg/r
!-----------------------------------------------------------------------

      ierr1    = 0
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).le.1 .or. gdim(f).gt. maxg) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)
      endif

      if(keyword(4) .eq. 'all' ) then
         ibleg = 0
      elseif(keyword(4) .eq. 'even' ) then
         ibleg = 2
      elseif(keyword(4) .eq. 'odd' ) then
         ibleg = 1
      else
         read(keyword(4),*,iostat=ierr) ibleg
         if (ierr .ne. 0) then
            ierr1=ierr
            routine = 'pleg_inp (einpbasmod.f90)'
            message='expecting "even" or "odd" or "all" and not: ' &
                 //keyword(4)(1:32)
            call wrline(zeroi,iz,iz0)
         end if
      end if
      if (ibleg.lt.0 .or.ibleg.gt.2) then
         ierr1   = 1
         routine = 'pleg_inp (einpbasmod.f90)'
         message = 'Incorrect ibleg parameter'
         call wrline(zeroi,iz,iz0)
      endif
      
!     C--Note: The values of ipbaspar(2-4,f) are given in Exp-DVR section.
      ipbaspar(1,f)=ibleg
      ipbaspar(2,f)=9999

      if(basis(f) .eq. 21 ) then
         ipbaspar(3,f)=gdim(f)
         read(keyword(5),*,iostat=ierr) rpbaspar(2,f)
         if (ierr .ne. 0) ierr1=ierr
         read(keyword(6),*,iostat=ierr) rpbaspar(1,f)
         if (ierr .ne. 0) ierr1=ierr
         if(rpbaspar(1,f) .le. rpbaspar(2,f) ) then
            routine = 'pleg_inp (einpbasmod.f90)'
            message = 'PLeg/R :'// &
            ' first restriction number must be lower than second.'
            call wrline(zeroi,iz,iz0)
         endif
      endif
      
      if (ierr1 .ne. 0 ) then
         routine = 'pleg_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      return
      end subroutine pleg_inp
      

!*****************************************************************************

      subroutine lagu_inp(iz,iz0,ic,f)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,ic,f
      integer(long)                :: i,ierr,ierr1,icut
      real(dop)                    :: xi,xf,x0,breite

!-----------------------------------------------------------------------
!  Laguerre DVR
!-----------------------------------------------------------------------
!   ipbaspar(1)=0 -> default , 1 ->'xi-xf'    2->'x0-xf'
!   ipbaspar(2)=icut
!   rpbaspar(1)=x0,     xi,   x0
!   rpbaspar(2)=breite, xf,   xf
!-----------------------------------------------------------------------

      ierr1    = 0
      icut     = 0
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).le.1 .or. gdim(f).gt. maxg) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)
      endif

      i=4
      if (keyword(i) .eq.  'xi-xf') then
         i=i+1
         read(keyword(i),*,iostat=ierr) xi
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xi,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'lagu_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         i=i+1
         read(keyword(i),*,iostat=ierr) xf
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xf,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'lagu_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         if ( ic.gt.i ) then 
            i=i+1
            read(keyword(i),*,iostat=ierr) icut
            if (ierr .ne. 0 .or. icut.lt.0) ierr1=ierr
            if (ierr1 .ne. 0 ) then
               routine = 'lagu_inp (einpbasmod.f90)'
               message = 'Error reading icut (Lagu).'
               call wrline(i,iz,iz0)   
            endif
         endif 
         ipbaspar(1,f)=1
         ipbaspar(2,f)=icut
         rpbaspar(1,f)=xi
         rpbaspar(2,f)=xf
      elseif (keyword(i) .eq.  'x0-xf') then
         i=i+1
         read(keyword(i),*,iostat=ierr) x0
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(x0,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'lagu_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         i=i+1
         read(keyword(i),*,iostat=ierr) xf
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xf,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'lagu_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         if ( ic.gt.i ) then 
            i=i+1
            read(keyword(i),*,iostat=ierr) icut
            if (ierr .ne. 0 .or. icut.lt.0) ierr1=ierr
            if (ierr1 .ne. 0 ) then
               routine = 'lagu_inp (einpbasmod.f90)'
               message = 'Error reading icut (Lagu).'
               call wrline(i,iz,iz0)   
            endif
         endif 
         ipbaspar(1,f)=2
         ipbaspar(2,f)=icut
         rpbaspar(1,f)=x0
         rpbaspar(2,f)=xf
      else
         read(keyword(i),*,iostat=ierr) x0
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(x0,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'lagu_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         i=i+1
         read(keyword(i),*,iostat=ierr) breite
         if (ierr .ne. 0) ierr1=ierr
         if (keyword(i+1) .eq. ',' ) then
            i = i+2
            call keylunits(breite,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'lagu_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         if ( ic.gt.i ) then 
            i=i+1
            read(keyword(i),*,iostat=ierr) icut
            if (ierr.ne.0 .or. icut.lt.0) ierr1=ierr
            if (ierr1 .ne. 0 ) then
               routine = 'lagu_inp (einpbasmod.f90)'
               message = 'Error reading icut (Lagu).'
               call wrline(i,iz,iz0)   
            endif
         endif 
         ipbaspar(1,f)=0
         ipbaspar(2,f)=icut
         rpbaspar(1,f)=x0
         rpbaspar(2,f)=breite
      endif
      
      return
      end subroutine lagu_inp

!*****************************************************************************

      subroutine ext_inp(iz,iz0,ic,f)

use channels

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: ic,iz,f
      integer(long)                :: ii,ierr,ierr1,ifile,filetype,lfile
      real(dop)                    :: unitfactor
      character(len=c5)            :: filename

!-----------------------------------------------------------------------
! External DVR
!
! Both the grid points and DVR-matrices are read from files.
!-----------------------------------------------------------------------
! ipbaspar(1)=ifile        (channel number, ifile=iebas+f) iebas set 
!                           in default.f90.
! ipbaspar(2)=filetype     (1=ascii, 2=binary)
! rpbaspar(1)=unitfactor   (factor to modify ort)
!-----------------------------------------------------------------------
      ierr1 = 0
      filetype=1
      unitfactor = 1.0_dop
      ifile=-99
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f) .lt. 1) then
         routine = 'ext_inp (einpbasmod.f90)'
         message='Grid dimension should be a positive integer.'
         call wrline(zeroi,iz,iz0)
      endif
      write(ilog,*)
      if (ic.ge.4) then
         read(keyorig(4),'(a)',iostat=ierr) filename
         if (ierr .ne. 0) ierr1=ierr
         lfile=index(filename,' ')-1
         call abspath(filename,lfile)
         write(ilog,*) 'Read DVR file for dof ',modelabel(f)(1:10), &
              ': ',filename(1:lfile)
         ifile=iebas-1+f
         ii=4

 15      ii=ii+1
         if(ic.lt.ii) goto 14
         if(ii.gt.6) then
            ierr1=1
            routine = 'ext_inp (einpbasmod.f90)'
            message='There must not be more than 6 keywords '// &
                    'in an Extern-DVR line'
            call warnmsg
         endif
         if(keyword(ii)(1:5) .eq. 'ascii') then
            filetype=1
         elseif (keyword(ii)(1:3) .eq. 'bin') then
            filetype=2
         else
            call keylunits(unitfactor,keyword(ii),ierr)
            if (ierr .ne. 0) then 
               ierr1=1 
               routine = 'ext_inp (einpbasmod.f90)'
               message='Expecting "bin" or "ascii" or a '// &
                       'length-unit but not: '
               call wrline(ii,iz,iz0)
            endif
         endif
         goto 15
      else
         ierr1=1
      endif
      
 14   continue
      if (ierr1 .ne. 0 ) then
         routine = 'ext_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      if(filetype .eq. 1 ) then
         open(unit=ifile,file=filename(1:lfile),form='formatted', &
              status='old',err=13)
      elseif(filetype .eq. 2 ) then
         open(unit=ifile,file=filename(1:lfile),form='unformatted', &
              status='old',err=13)
      endif      
      ipbaspar(1,f)=ifile
      ipbaspar(2,f)=filetype
      rpbaspar(1,f)=unitfactor

      return
      
 13   continue
      routine = 'ext_inp (einpbasmod.f90)'
      message='Cannot open file '//filename(1:lfile)
      ierr1=1
      call warnmsg

      return
      end subroutine ext_inp
      
!*****************************************************************************

      subroutine Wig_inp(iz,iz0,f)

      implicit none

      integer iz,iz0,ierr,ierr1,f,ibleg

!-----------------------------------------------------------------------
! Wigner rotation matrix DVR
!
! ibleg : integer to control even/odd options
!-----------------------------------------------------------------------
! ipbaspar(1)=ibleg
! If exp DVR is used for the 2nd and/or 3rd combined DOFs, then
! the parameters 2-9 are set in exp_inp
! ipbaspar(2)=kmin
! ipbaspar(3)=kmax
! ipbaspar(4)=kmax-kmin+1
! ipbaspar(5)=min(abs(kmin),abs(kmax)) or ipbaspar(5)=0 if(kmin*kmax<0)
! ipbaspar(6)=mmmin
! ipbaspar(7)=mmmax
! ipbaspar(8)=mmmax-mmmin+1
! ipbaspar(9)=min(abs(mmmin),abs(mmmax)) or ipbaspar(5)=0 if(mmmin*mmmax<0)
!-----------------------------------------------------------------------

      if(progname(1:6) .eq. 'potfit' .or. progname(1:6) .eq. 'mcpotf' ) then
         Write(6,'(/a,/2a,/2a,/2a,/)') '### WARNING ###', &
         'As (MC)POTFIT only needs the grid-points and not the ', &
        'derivative matrices,','it is recommended to replace "Wigner" ', &
         'with "Leg" and add there "0" for','the blz-parameter. ', &
         '(MC)POTFIT may crash with the present input.'  
         Write(2,'(/a,/2a,/2a,/2a,/)') '### WARNING ###', &
         'As (MC)POTFIT only needs the grid-points and not the ', &
        'derivative matrices,','it is recommended to replace "Wigner" ', &
         'with "Leg" and add there "0" for','the blz-parameter. ', &
         '(MC)POTFIT may crash with the present input.'
      endif

      ierr1    = 0
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).le.1 .or. gdim(f).gt. maxg) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)
      endif

      if(keyword(4) .eq. 'all' ) then
         ibleg = 0
      elseif(keyword(4) .eq. 'even' ) then
         ibleg = 2
      elseif(keyword(4) .eq. 'odd' ) then
         ibleg = 1
      else
         read(keyword(4),*,iostat=ierr) ibleg
         if (ierr .ne. 0) then
            ierr1=ierr
            routine = 'Wig_inp (einpbas.F)'
            message='expecting "even" or "odd" or "all" and not: ' &
                    //keyword(4)(1:32)
            call wrline(zeroi,iz,iz0)
         end if
      end if
      if (ibleg.lt.0 .or.ibleg.gt.2) then
         ierr1   = 1
         routine = 'Wig_inp (einpbas.F)'
         message = 'Incorrect ibleg parameter'
         call wrline(zeroi,iz,iz0)
      endif

      ipbaspar(1,f)=ibleg
      ipbaspar(2,f)=9999
      ipbaspar(6,f)=9999

      if (ierr1 .ne. 0 ) then
         routine = 'Wig_inp (einpbas.F)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      return
      end subroutine Wig_inp

!*****************************************************************************

      subroutine pnt_inp(iz,iz0,f)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,f
      integer(long)                :: i,ierr,ierr1
      real(dop)                    :: xi

!-----------------------------------------------------------------------
!     Single point
!
! xi: coordinate of point
!-----------------------------------------------------------------------
! rpbaspar(1)=xi
!-----------------------------------------------------------------------

      ierr1    = 0
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).ne.1) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)

      endif

      i=4
      read(keyword(i),*,iostat=ierr) xi
      if (ierr .ne. 0) ierr1=ierr
      if (keyword(i+1) .eq. ',') then
         i=i+2
         call keylunits(xi,keyword(i),ierr)
         if (ierr .eq. 1) then
            routine = 'pnt_inp (einpbasmod.f90)'
            message = 'no length unit after comma'
            call wrline(zeroi,iz,iz0)
         endif
      endif

      rpbaspar(1,f)=xi

      if (ierr1 .ne. 0) then
         routine = 'pnt_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      return
      end subroutine pnt_inp

!*****************************************************************************

      subroutine tpnt_inp(iz,iz0,ic,f)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,ic,f
      integer(long)                :: i,ierr,ierr1

!-----------------------------------------------------------------------
!     Single point with a time-dependent value
!
! pntop: name of operator defining coordinate
! 
!-----------------------------------------------------------------------
! rpbaspar(1)=xi
!-----------------------------------------------------------------------

      ierr1    = 0
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).ne.1) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)
      endif

      if (ic .lt. 4) then
         ierr=1
         routine = 'tpnt_inp (einpbasmod.f90)'
         message = 'no operator specified for tpoint basis'
         call wrline(zeroi,iz,iz0)
      else
         i=4
         atpntop(f) = keyword(i)(1:c1)
         ltpntdvr=.true.
      endif

      if (ierr1 .ne. 0) then
         routine = 'tpnt_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      return
      end subroutine tpnt_inp

!****************************************************************************
!
! Read the optional keywords for a GWP basis, e.g. the parameters for
! the auxiliary grid used for analysis.
! 
! ic0 is the first keyword to read - this differs depending on which 
! type of input DVR or INGEOM
!
!****************************************************************************

      subroutine gwp_inp(iz,iz0,ic,f,ic0)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,ic,ic0,f
      integer(long)                :: i,ierr,ierr1,iperiod
      real(dop)                    :: xi,xf,xip,xfp
      
!-----------------------------------------------------------------------
! GWP
!
! GWPs do not need any grid information, but a grid is set up to 
! help analysis.
!
! optional parameters:
! gdim(f): no. of grid points
! rpbaspar(1)=xi ,  xi: coordinate of first grid point
! rpbaspar(2)=xf ,  xf: coordinate of last grid point
!
! ipbaspar(1) = 1  classical gwp
! ipbaspar(2) = iperiod  if iperiod > 0 coordinate is periodic with m the
!                  multiplicity of the rotation axis. Domain is 0:2pi/iperiod
!                  if iperiod < 0 coordinate is periodic with iperiod the
!                  multiplicity of the rotation axis. 
!                  Domain is -pi/iperiod:pi/iperiod    !pergauss
!
! default values:
! gdim(f): 101
! rpbaspar(1)= -10.0
! rpbaspar(2)= 10.0
!-----------------------------------------------------------------------
!
! Set default values
!
      gdim(f) = 101
      xi = -10.0
      xf = 10.0
      iperiod=0
      xip = 0.0_dop
      xfp = 0.0_dop

      ierr1=0
      i = ic0
!-----------------------------------------------------------------------
! periodic coordinate      !pergauss
!-----------------------------------------------------------------------
      if (ic .ge. i) then
         if (keyword(i)(1:3) .eq. '2pi') then
            iperiod=1
            lpergwp=.true.
            if (keyword(i)(4:4) .eq. '/') then
              read(keyword(i)(5:5),*,iostat=ierr) iperiod
              ierr1=ierr

              xip = 0.0_dop
              xfp = 2.0*pi/dble(iperiod)
            else
              xip = 0.0_dop
              xfp = 2.0*pi/dble(iperiod)
            endif

! point to next keyword if 2pi found
            i=i+1 

         else if (keyword(i)(1:5) .eq. 'c-2pi') then
            iperiod=1
            lpergwp=.true.
            if (keyword(i)(6:6) .eq. '/') then
              read(keyword(i)(7:7),*,iostat=ierr) iperiod
              ierr1=ierr

              xip = -pi/dble(iperiod)
              xfp = pi/dble(iperiod)
            else
              xip = -pi/dble(iperiod)
              xfp = pi/dble(iperiod)            
              iperiod=-iperiod
            endif
! point to next keyword if 2pi found
            i=i+1 

         else

            continue
         endif
      endif
!
! look for no. of grid points. 
!
      if (ic .ge. i) then
         read(keyword(i),*,iostat=ierr) gdim(f)
         ierr1=ierr
         if (gdim(f).le.1 .or. gdim(f).gt.20000) then
            write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
            message = 'Illegal value of gdim for mode :'
            call wrline(onei,iz,iz0)
         endif
      endif

!
! look for grid start and overwrite xi if found
!
      i = i+1
      if (ic .ge. i) then
         read(keyword(i),*,iostat=ierr) xi
         ierr1=ierr

         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xi,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'gwp_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
      endif

!
! look for grid end and overwrite xf if found
!
      i = i+1
      if (ic .ge. i) then
         read(keyword(i),*,iostat=ierr) xf
         ierr1=ierr

         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keylunits(xf,keyword(i),ierr)
            if (ierr .eq. 1) then
               routine = 'gwp_inp (einpbasmod.f90)'
               message = 'no length unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
         endif
      endif
            
      if (iperiod .ne. 0) then
         rpbaspar(1,f)=xip
         rpbaspar(2,f)=xfp
      else
         rpbaspar(1,f)=xi
         rpbaspar(2,f)=xf
      endif

      ipbaspar(2,f)=iperiod

      if (ierr1 .ne. 0) then
         routine = 'gwp_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      return
      end subroutine gwp_inp

!*****************************************************************************
!-----------------------------------------------------------------------
! Basis for x-local mode 
!-----------------------------------------------------------------------

      subroutine xloc_inp(iz,iz0,f)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,f
      integer(long)                :: i,ierr,ierr1
      real(dop)                    :: xi,xf

      ierr1=0
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr

      i=4
      read(keyword(i),*,iostat=ierr) xi
      if (ierr .ne. 0) ierr1=ierr
      if (keyword(i+1) .eq. ',') then
         i=i+2
         call keylunits(xi,keyword(i),ierr)
         if (ierr .eq. 1) then
            message = 'no length unit after comma'
            call wrline(zeroi,iz,iz0)
         endif
      endif
      i=i+1
      read(keyword(i),*,iostat=ierr) xf
      if (ierr .ne. 0) ierr1=ierr
      if (keyword(i+1) .eq. ',') then
         i=i+2
         call keylunits(xf,keyword(i),ierr)
         if (ierr .eq. 1) then
            message = 'no length unit after comma'
            call wrline(zeroi,iz,iz0)
         endif
      endif

      i=i+1
      read(keyword(i),*,iostat=ierr) ipbaspar(1,f)
      if (ierr .ne. 0) ierr1=ierr

      if (ierr1 .ne. 0) then
         routine = 'xloc_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      rpbaspar(1,f)=xi
      rpbaspar(2,f)=xf

      
      return
      end subroutine xloc_inp
      
!****************************************************************************

      subroutine lob_inp(iz,iz0,ic,f)

      implicit none

      integer(long), intent(inout) :: iz0
      integer(long), intent(in)    :: iz,ic,f
      integer(long)                :: ierr,ierr1,i
      real(dop)                    :: xi,xf
      
!-----------------------------------------------------------------------
! Lobatto "DVR"
!
! rpbaspar(1)=xi ,  xi: coordinate of first grid point
! rpbaspar(2)=xf ,  xf: coordinate of last grid point
!-----------------------------------------------------------------------

      ierr1    = 0
      read(keyword(3),*,iostat=ierr) gdim(f)
      if (ierr .ne. 0) ierr1=ierr
      if (gdim(f).le.1 .or. gdim(f).gt.20000) then
         write(6,'(/a,i4,i8)') 'f, gdim(f)', f,gdim(f)
         message = 'Illegal value of gdim for mode :'
         call wrline(onei,iz,iz0)
      endif

      i=4
      read(keyword(i),*,iostat=ierr) xi
      if (ierr .ne. 0) ierr1=ierr
      if (keyword(i+1) .eq. ',') then
         i=i+2
         call keylunits(xi,keyword(i),ierr)
         if (ierr .eq. 1) then
            routine = 'fft_inp (einpbasmod.f90)'
            message = 'no length unit after comma'
            call wrline(zeroi,iz,iz0)
         endif
      endif
      i=i+1
      read(keyword(i),*,iostat=ierr) xf
      if (ierr .ne. 0) ierr1=ierr
      if (keyword(i+1) .eq. ',') then
         i=i+2
         call keylunits(xf,keyword(i),ierr)
         if (ierr .eq. 1) then
            routine = 'fft_inp (einpbasmod.f90)'
            message = 'no length unit after comma'
            call wrline(zeroi,iz,iz0)
         endif
      endif
         
      rpbaspar(1,f)=xi
      rpbaspar(2,f)=xf

! may have up to 2 optional keywords
      if (i.lt.ic) then
         i = i+1 
         if (keyword(i) .eq. 'no-xi') then
            ipbaspar(1,f) = 1
         else if (keyword(i) .eq. 'no-xf') then
            ipbaspar(2,f) = 1
         endif
      endif
      if (i.lt.ic) then
         i = i+1 
         if (keyword(i) .eq. 'no-xi') then
            ipbaspar(1,f) = 1
         else if (keyword(i) .eq. 'no-xf') then
            ipbaspar(2,f) = 1
         endif
      endif

      if (i.lt.ic) then
         routine = 'lob_inp (einpbasmod.f90)'
         message='Expecting no more input. Unknown keyword :'
         call wrline(i+1,iz,iz0)
         ierr1=1
      endif
      if (ierr1 .ne. 0) then
         routine = 'lob_inp (einpbasmod.f90)'
         message = 'Incorrect pbasis definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      return
      end subroutine lob_inp

!#######################################################################
!
! Set default values for variables used in DVR generation
!
!#######################################################################

      subroutine dvrdefaults

use dvrdatmod

!-----------------------------------------------------------------------
! read by subroutine einpbas
!-----------------------------------------------------------------------
      basis(:) = 0
      gdim(:) = 0
      ipbaspar(:,:) = 0
      rpbaspar(:,:) = 0.0_dop
      ldvr(:) = .false.
      lpntdvr=.false.
      ltpntdvr=.false.
      lxloc = .false.
      lgwp  = .false.
      basis(:) = 999
      modelabel(:) = '--'
      atpntop(:) = '  '
      itpntop(:) = -999

      feb    = 0
      fpb    = 0
      eltot = 0
      elbnd = 0
      lebcont = .false.

      jtot=-999.0_dop
      jbf=-2.0_dop

      end subroutine dvrdefaults

      end module einpbasmod


      

