!
!
!                 MCTDH  module   INIT
!
! Loops over modes, works out desired basis for mode m and calls
! required subroutine to initialise DVRrepresentation data.
!
!  10/96 AJ  
!
!***********************************************************************
      module initmod

use decimal, only: dop, long
use logdat
use fftmod,  only: fftinit
use op1lib,  only: zerovxd
use global
use dvrdatmod
use channels
use iogrddef, only: griddat
use init1mod, only: initho,initexp,initrho,initsin,initsinc,initwsinc, &
                    initfft,initsphfbr,initlagu,initcos,initgwpgrid,&
                    initlob
use init2mod, only: initleg,initlegr,initkleg,initklegr,initk,initextern
use init3mod, only: initwigner


      implicit none
      private
      public :: init
      contains

!***********************************************************************

      subroutine init(trafo,dvrmat,ort,fftp,kinsph,jsph,&
                 msph,hin,rueck,fftfak,exphin,exprueck)

      implicit none

      integer(long)                                    :: f,i,typ,blz,ibleg,&
                                                          jmax,mmax,mincr,mmin,&
                                                          kmin,kmin2,kdim,dk,&
                                                          icut,filetype,ifile,&
                                                          mmmin,mmmin2,mmdim
      integer(long), dimension(sphdim), intent(out)    :: jsph,msph
      integer(long), dimension(fftdim), intent(out)    :: fftfak

      real(dop), dimension(dvrdim), intent(out)        :: trafo
      real(dop), dimension(ortdim), intent(out)        :: ort
      real(dop), dimension(fftdim), intent(out)        :: fftp
      real(dop), dimension(sphdim*3), intent(out)      :: kinsph
      real(dop), dimension(dvrdim,dvrord), intent(out) :: dvrmat

      real(dop)                                        :: hoxeq,hofreq,homass,hofm,x0,xi,xf,dx,b
      complex(dop), dimension(fftdim), intent(out)     :: hin,rueck
      complex(dop), dimension(expdim), intent(out)     :: exphin,exprueck

!-----------------------------------------------------------------------
!  Loop over modes and call the appropriate init-routines to set up the
!  matrices for the first and second derivatives: dif1mat and dif2mat.
!  Note that in case of the Legendre DVR these are defined as:
!  dif2mat := sin^-1(theta) * d/dtheta * sin(theta) * d/dtheta 
!              - m^2/sin^2(theta).  (N.B.: m = blz).
!  dif1mat := d/dtheta * sin(theta). 
!-----------------------------------------------------------------------
      do f=1,ndof

  outer: if (basis(f) .eq. -1) then
             continue

         elseif (basis(f).eq.0 .or. basis(f) .eq. 10) then outer ! el
            do i = 1, gdim(f)
               ort(i-1+zort(f)) = i 
            enddo

         elseif (basis(f).eq.1) then outer ! HO
            routine = 'initHO   (init1.F)'
            typ=ipbaspar(1,f)
            if (typ .eq. 1) then
               xi = rpbaspar(1,f)
               xf = rpbaspar(2,f)
               hoxeq  = (xi+xf)*0.5_dop
               homass = 1.0_dop
               hofreq = 1.0_dop
               dx=xf-xi
            else
               hoxeq  = rpbaspar(1,f)
               hofreq = rpbaspar(2,f)
               homass = rpbaspar(3,f)
               dx=0.0_dop
            endif

            hofm = homass*hofreq

            
            call initho(trafo(zdvr(f)),ort(zort(f))&
                 ,dvrmat(zdvr(f),2),dvrmat(zdvr(f),1)&
                 ,gdim(f),hofm,hoxeq,dx,typ,message)
!...........reset the parameters to standard form
            if(typ .eq. 1) then
               hofreq = hofm
               ipbaspar(1,f) = 0
               rpbaspar(1,f) = hoxeq
               rpbaspar(2,f) = hofreq
               rpbaspar(3,f) = homass
            end if
          
         elseif (basis(f).eq.11) then outer ! rHO
            routine = 'initRHO   (init1.F)'
            typ=ipbaspar(1,f)
            if (typ .eq. 1) then
               xi = rpbaspar(1,f)
               xf = rpbaspar(2,f)
               homass = 1.0_dop
               hofreq = 1.0_dop
               dx=xf-xi
            elseif (typ .eq. 2) then
               hoxeq  = rpbaspar(1,f)
               xf     = rpbaspar(2,f)
               homass = 1.0_dop
               hofreq = 1.0_dop
               dx=xf-hoxeq
            else
               hoxeq  = rpbaspar(1,f)
               hofreq = rpbaspar(2,f)
               homass = rpbaspar(3,f)
               dx=0.0_dop
            endif

            hofm = homass*hofreq

            call initrho(trafo(zdvr(f)),ort(zort(f))&
                 ,dvrmat(zdvr(f),2),dvrmat(zdvr(f),1)&
                 ,gdim(f),hofm,hoxeq,xi,dx,typ,message)
            if(typ .gt. 0) then
!...........reset the parameters to standard form
               hofreq = hofm
               ipbaspar(1,f) = 0
               rpbaspar(1,f) = hoxeq
               rpbaspar(2,f) = hofreq
               rpbaspar(3,f) = homass
            end if

         elseif (basis(f).eq.2 .or. basis(f).eq.12) then outer ! Leg, Leg/r
            blz=ipbaspar(1,f)
            ibleg=ipbaspar(2,f)
            routine = 'initLeg   (init2.F)'
            call initleg(trafo(zdvr(f)),ort(zort(f)),&
                 dvrmat(zdvr(f),2),dvrmat(zdvr(f),1),&
                 gdim(f),blz,ibleg,message)
            if(basis(f).eq.12) then
               call initlegr(trafo(zdvr(f)),ort(zort(f)),&
                    dvrmat(zdvr(f),2),dvrmat(zdvr(f),1),&
                    ipbaspar(3,f),gdim(f),rpbaspar(1,f),rpbaspar(2,f))
               call griddat
            end if

         elseif (basis(f).eq.3) then outer ! sin
            xi=rpbaspar(1,f)
            xf=rpbaspar(2,f)
            routine = 'initSin   (init1mod.f90)'
            call initsin(trafo(zdvr(f)),ort(zort(f)),&
                 dvrmat(zdvr(f),2),dvrmat(zdvr(f),1),&
                 gdim(f),xi,xf,ipbaspar(1,f))

         elseif (basis(f).eq.31) then outer ! sinc
            xi=rpbaspar(1,f)
            xf=rpbaspar(2,f)
            routine = 'initSinc   (init1mod.f90)'
            call initsinc(trafo(zdvr(f)),ort(zort(f)),&
                 dvrmat(zdvr(f),2),dvrmat(zdvr(f),1),&
                 gdim(f),xi,xf,ipbaspar(1,f))

         elseif (basis(f).eq.32) then outer ! wsinc
            xi=rpbaspar(1,f)
            xf=rpbaspar(2,f)
            routine = 'initWsinc   (init1mod.f90)'
            call initwsinc(trafo(zdvr(f)),ort(zort(f)),&
                 dvrmat(zdvr(f),2),dvrmat(zdvr(f),1),&
                 gdim(f),xi,xf,ipbaspar(1,f))

         elseif (basis(f).eq.4) then outer ! FFT
            xi=rpbaspar(1,f)
            xf=rpbaspar(2,f)
            call initfft(ort(zort(f)),fftp(zfft(f)),gdim(f),xi,xf)

         elseif (basis(f).eq.5) then outer ! exp
            xi=rpbaspar(1,f)
            xf=rpbaspar(2,f)
            if ((f .gt. 1) .and. &
                   (basis(max(f-1,1)).eq.9 .or. &
                    basis(max(f-1,1)).eq.21 .or. & 
                    basis(max(f-1,1)).eq.33)) then
               kmin=ipbaspar(1,f)
               kdim=ipbaspar(3,f)
            elseif ((f.gt.2) .and. (basis(max(f-2,1)).eq.33)) then
               kmin=ipbaspar(6,f-2)
               kdim=ipbaspar(8,f-2)
            else 
               kmin=-gdim(f)/2
               kdim=gdim(f)
            endif 
            routine = 'initExp  (init1mod.f90)'
            call initexp(trafo(zdvr(f)),ort(zort(f))&
                 ,dvrmat(zdvr(f),2),dvrmat(zdvr(f),1)&
                 ,exphin(zexp(f)),exprueck(zexp(f)),gdim(f),xi,xf&
                 ,kmin,kdim)

         elseif (basis(f).eq.6) then outer ! sphfbr
            typ   = ipbaspar(1,f)
            jmax  = ipbaspar(2,f)
            mmax  = ipbaspar(1,f+1)
            mincr = ipbaspar(2,f+1)
            mmin  = ipbaspar(3,f+1)
            call initsphfbr(gdim(f),kinsph(zsph(f)),&
                kinsph(zsph(f)+sphdim),kinsph(zsph(f)+2*sphdim),&
                jsph(zsph(f)),msph(zsph(f)),jmax,typ,&
                mmax,mincr,mmin,ilog)

         elseif (basis(f).eq.7 .or. basis(f).eq.20) then outer ! kleg
            ibleg=ipbaspar(1,f)
            kmin =ipbaspar(1,f+1)
            kmin2=ipbaspar(3,f+1)
            routine = 'initkleg   (init2.F)'
            call initkleg(trafo(zdvr(f)),ort(zort(f)),&
                 dvrmat(zdvr(f)+gdim(f+1)*gdim(f)**2,2),&
                 dvrmat(zdvr(f)+gdim(f+1)*gdim(f)**2,1),&
                 dvrmat(zdvr(f),2),dvrmat(zdvr(f),1),&
                 gdim(f),gdim(f+1),kmin,kmin2,ibleg,message)

            if(basis(f).eq.20) then
               call initklegr(trafo(zdvr(f)),ort(zort(f)),&
                 dvrmat(zdvr(f)+gdim(f+1)*gdim(f)**2,1),&
                 dvrmat(zdvr(f),2),dvrmat(zdvr(f),1),&
                 ipbaspar(3,f),gdim(f),gdim(f+1),rpbaspar(1,f),&
                 rpbaspar(2,f))
               call griddat
            end if

         elseif (basis(f).eq.8) then outer ! k
             kmin=ipbaspar(1,f)
             dk=ipbaspar(5,f)
             call initk(ort(zort(f)),gdim(f),kmin,dk)

         elseif (basis(f).eq.9 .or. basis(f).eq.21) then outer ! pleg
             if (basis(f+1).ne.5) then
                message='After"pleg or pleg/r-DVR" must come "exp-DVR"'
                call errormsg
             endif
             ibleg=ipbaspar(1,f)

             kmin =ipbaspar(1,f+1)
             kdim =ipbaspar(3,f+1)
             kmin2=ipbaspar(4,f+1)
             routine = 'initkleg   (init2.F)'
             call initkleg(trafo(zdvr(f)),ort(zort(f)),&
                  dvrmat(zdvr(f)+kdim*gdim(f)**2,2),&
                  dvrmat(zdvr(f)+kdim*gdim(f)**2,1),&
                  dvrmat(zdvr(f),2),dvrmat(zdvr(f),1),&
                  gdim(f),kdim,kmin,kmin2,ibleg,message)
             if(basis(f).eq.21) then
               call initklegr(trafo(zdvr(f)),ort(zort(f)),&
                 dvrmat(zdvr(f)+kdim*gdim(f)**2,1),&
                 dvrmat(zdvr(f),2),dvrmat(zdvr(f),1),&
                 ipbaspar(3,f),gdim(f),kdim,rpbaspar(1,f),&
                 rpbaspar(2,f))
               call griddat  
            end if

         elseif (basis(f).eq.14) then outer ! cos
            xi=rpbaspar(1,f)
            xf=rpbaspar(2,f)
            routine = 'initCos   (init1.F)'
            call initcos(trafo(zdvr(f)),ort(zort(f)),&
                 dvrmat(zdvr(f),2),dvrmat(zdvr(f),1),&
                 gdim(f),xi,xf,message)

         elseif (basis(f).ge.15 .and. basis(f).le.18) then outer ! Lagu
            routine = 'initLagu   (init1.F)'
            typ=ipbaspar(1,f)
            icut=ipbaspar(2,f)
            if (typ .eq. 1) then
               xi = rpbaspar(1,f)
               xf = rpbaspar(2,f)
               x0 = 0.0_dop
               b  = 1.0_dop
            elseif (typ .eq. 2) then
               x0 = rpbaspar(1,f)
               xf = rpbaspar(2,f)
               b  = 1.0_dop
            else
               x0 = rpbaspar(1,f)
               b  = rpbaspar(2,f)
            endif

            call initlagu(trafo(zdvr(f)),ort(zort(f)),&
                 dvrmat(zdvr(f),2),dvrmat(zdvr(f),1),&
                 gdim(f),x0,b,xi,xf,typ,icut,message,basis(f))
!...........reset the parameters to standard form
            if(typ .gt. 0) then
               ipbaspar(1,f) = 0
               rpbaspar(1,f) = x0
               rpbaspar(2,f) = b
            end if

         elseif (basis(f).eq.13) then outer ! external
             routine = 'initextrn  (init2.F)'
             ifile=ipbaspar(1,f)
             filetype=ipbaspar(2,f)
             call initextern(ifile,filetype,rpbaspar(1,f),ort(zort(f)),&
                       dvrmat(zdvr(f),1),dvrmat(zdvr(f),2),gdim(f))

! copy coordinate of selected point to ort
         elseif (basis(f).eq.19) then outer
             ort(zort(f)) = rpbaspar(1,f)

! set initial coordinate of tpoint DVR to 0.
         elseif (basis(f).eq.22) then outer
             ort(zort(f)) = 0.0_dop

         elseif (basis(f).eq.25 .or. basis(f) .eq. 27) then outer
            xi=rpbaspar(1,f)
            xf=rpbaspar(2,f)
            call initgwpgrid(ort(zort(f)),gdim(f),xi,xf)

         elseif (basis(f).eq.26) then outer
            call zerovxd(ort(zort(f)),gdim(f))

         elseif (basis(f).eq.33) then outer ! Wigner
            ibleg=ipbaspar(1,f)
            kmin=ipbaspar(2,f)
            kdim=ipbaspar(4,f)
            kmin2=ipbaspar(5,f)
            mmmin=ipbaspar(6,f)
            mmdim=ipbaspar(8,f)
            mmmin2=ipbaspar(9,f)
            routine = 'initwigner   (init3.F)'
            call initwigner(trafo(zdvr(f)),ort(zort(f)), &
                 dvrmat(zdvr(f)+mmdim*kdim*gdim(f)**2,2), &
                 dvrmat(zdvr(f)+mmdim*kdim*gdim(f)**2,1), &
                 dvrmat(zdvr(f),2),dvrmat(zdvr(f),1), &
                 gdim(f),gdim(f+1),gdim(f+2),kmin,mmmin, &
                 kmin2,mmmin2,ibleg,message)

         elseif (basis(f).eq.34) then outer ! Lobatto
            xi=rpbaspar(1,f)
            xf=rpbaspar(2,f)
            routine = 'initLob   (init1mod.f90)'
            call initlob(trafo(zdvr(f)),ort(zort(f)),&
                 dvrmat(zdvr(f),2),dvrmat(zdvr(f),1),&
                 gdim(f),xi,xf,ipbaspar(1,f),ipbaspar(2,f))

         else outer
            write(6,*) ' basis(f) =', basis(f),', f =', f
            routine = 'INIT'
            message = 'Wrong primitive basis identification number'
            call errormsg
         endif outer
      enddo
      write(ilog,*) 

!-----------------------------------------------------------------------
! store end grid points for log information
!-----------------------------------------------------------------------
      do f=1,ndof
         xend(1,f)=ort(zort(f))
         xend(2,f)=ort(zort(f)+gdim(f)-1)
      enddo

!-----------------------------------------------------------------------
! Factor decomposition and exponentials for FFT
!-----------------------------------------------------------------------
      do f=1,ndof
         if (basis(f) .eq. 4) then
            call fftinit(gdim(f),fftfak(zfft(f)),hin(zfft(f)),&
                 rueck(zfft(f)))
         endif
      enddo

      return
      end subroutine init

!-----------------------------------------------------------------------
      end module initmod
!-----------------------------------------------------------------------

