!#######################################################################
!
!
!                 MCTDH  module LOGDVR
!
!#######################################################################

      module logdvrmod

use global
use maxv
use dvrdatmod
use channels
use logdat
use rddvrmod, only: dvrinfo

      implicit none
      private
      public :: logdvr,logdvrdat
      contains

!#######################################################################

      subroutine logdvrdat

      implicit none

      logical(kind=4)           :: lerr
      character(len=c5)         :: filename
      integer(long)             :: ilbl,chkdvr

!-----------------------------------------------------------------------
! open dvr file
!-----------------------------------------------------------------------
      filename=dname(1:dlaenge)//'/dvr'
      ilbl=index(filename,' ')-1
      open(idvr,file=filename,form='unformatted',status='old',err=999)

!-----------------------------------------------------------------------
! get memory sizes from DVR file
!-----------------------------------------------------------------------
      call rdmemdim(idvr)

!-----------------------------------------------------------------------
! Allocate mememory and read system / DVR information
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat

!-----------------------------------------------------------------------
! read dvr file
!-----------------------------------------------------------------------
      rewind(idvr)
      chkdvr=1
      call dvrinfo(lerr,chkdvr)

      close(idvr)

!-----------------------------------------------------------------------
! log data
!-----------------------------------------------------------------------
      call logdvr(ilog)

      call dealloc_dvrdat

      return

  999 continue
      routine='Logdvr'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop

      end subroutine logdvrdat

!#######################################################################

      subroutine logdvr(iounit)

      implicit none

      integer(long)             :: f,gj,kmin,kmax,dk,blz,ibleg,typ,&
                                   jmax,mmax,mmin,mincr
      integer(long), intent(in) :: iounit
      real(dop)                 :: hofm,x1,x2,dx,dp

      character(len=16), dimension(4)       :: &
                      flag = (/'no-sym   m-all  ','symmetry  m-all ', &
                      'symmetry m-trnc.','no-sym   m-trnc.'/)
!
! write heading 
!
      write(iounit,900)
!
! write DVR grid info
!
      do f=1,ndof
         x1=xend(1,f)
         x2=xend(2,f)

         if (basis(f) .eq. 0) then
! Do not print electronic basis if only 1 state
            if (f .ne. feb .or. gdim(f) .gt. 1) then
               write(iounit,901) modelabel(f),f,'el ',gdim(f)
            endif

         else if (basis(f) .eq. 1) then
            hofm = rpbaspar(2,f)*rpbaspar(3,f)
            dx=abs(x2-x1)/dble(gdim(f)-1)
            dp=0.5_dop*(x2-x1)*hofm
            write(iounit,902) modelabel(f),f,'HO ',&
                  gdim(f),x1,x2,dx,dp,hofm

         else if (basis(f) .eq. 2) then
            blz=ipbaspar(1,f)
            ibleg=ipbaspar(2,f)
            dx=abs(x2-x1)/dble(gdim(f)-1)
            write(iounit,903) modelabel(f),f,'Leg',&
                  gdim(f),x1,x2,dx,blz,ibleg

         else if (basis(f) .eq. 3) then
            dx=abs(x2-x1)/dble(gdim(f)-1)
            dp=pi*gdim(f)/(dx*(gdim(f)+1))
            if(ipbaspar(1,f).ne.2) then
               write(iounit,902) modelabel(f),f,'sin',&
                  gdim(f),x1,x2,dx,dp
            else           ! spin (ipbaspar(1)=2)
               write(iounit,922) modelabel(f),f,'sin',&
                  gdim(f),x1,x2,dx,'spin'
            endif

         else if (basis(f) .eq. 4) then
            dx=abs(x2-x1)/dble(gdim(f)-1)
            dp=pi*(gdim(f)-1)/(dx*gdim(f))
            write(iounit,902) modelabel(f),f,'FFT',&
                  gdim(f),x1,x2,dx,dp

         else if (basis(f) .eq. 5) then
            dx=abs(x2-x1)/dble(gdim(f)-1)
            dp=pi*(gdim(f)-1)/(dx*gdim(f))
            write(iounit,902) modelabel(f),f,'exp',&
                  gdim(f),x1,x2,dx,dp

         else if (basis(f) .eq. 6) then
            typ   =ipbaspar(1,f)
            jmax  =ipbaspar(2,f)
            gj    =ipbaspar(4,f)
            mmax  =ipbaspar(1,f+1)
            mincr =ipbaspar(2,f+1)
            mmin  =ipbaspar(3,f+1)
            write(iounit,904) modelabel(f),f,'sphfbr',&
                  gdim(f),gj,jmax,flag(typ)
            write(iounit,908) modelabel(f+1),f+1,'phifbr',&
                  gdim(f),mmax,mmin,mincr

         else if (basis(f) .eq. 7) then
            ibleg=ipbaspar(1,f)
            dx=abs(x2-x1)/dble(gdim(f)-1)
            if (ipbaspar(1,f).eq.0) then
               jmax=gdim(f)-1
            else            
               jmax=gdim(f)*2 - ipbaspar(1,f)
            endif
            write(iounit,905) modelabel(f),f,'KLeg',&
                  gdim(f),x1,x2,dx,jmax,ibleg

         else if (basis(f) .eq. 8) then
            kmin=ipbaspar(1,f)
            kmax=ipbaspar(2,f)
            dk  =ipbaspar(5,f)
            write(iounit,906) modelabel(f),f,'k',gdim(f),kmin, &
                  kmax,dk

         else if (basis(f) .eq. 9) then
            ibleg=ipbaspar(1,f)
            dx=abs(x2-x1)/dble(gdim(f)-1)
            if (ipbaspar(1,f).eq.0) then
               jmax=gdim(f)-1   
            else
               jmax=gdim(f)*2 - ipbaspar(1,f)
            endif
            write(iounit,905) modelabel(f),f,'PLeg', &
                  gdim(f),x1,x2,dx,jmax,ibleg

         else if (basis(f) .eq. 10) then
            write(iounit,901) modelabel(f),f,'elcont ',gdim(f)

         else if (basis(f) .eq. 11) then
            hofm = rpbaspar(2,f)*rpbaspar(3,f)
            dx=abs(x2-x1)/dble(gdim(f)-1)
            dp=(x2-x1)*hofm
            write(iounit,902) modelabel(f),f,'RHO',&
                  gdim(f),x1,x2,dx,dp,hofm

         else if (basis(f).eq.12) then
            blz=ipbaspar(1,f)
            ibleg=ipbaspar(2,f)
            dx=abs(x2-x1)/dble(gdim(f)-1)
            write(iounit,907) modelabel(f),f,'Leg/R',&
                  gdim(f),ipbaspar(3,f),x1,x2,dx,blz,ibleg

         else if (basis(f) .eq. 13) then
            write(iounit,913) modelabel(f),f,'extern',gdim(f),x1,x2

         else if (basis(f) .eq. 14) then
            dx=abs(x2-x1)/dble(gdim(f)-1)
            dp=pi*gdim(f)/(dx*(gdim(f)+1))
            write(iounit,902) modelabel(f),f,'cos',&
                  gdim(f),x1,x2,dx,dp

         else if (basis(f).ge.15 .and. basis(f).le.18) then
            dx=abs(x2-x1)/dble(gdim(f)-1)
            dp=pi*gdim(f)/(dx*(gdim(f)+1))
            write(iounit,912) modelabel(f),f,'Lagu',basis(f)-14,&
                  gdim(f),x1,x2,dx,dp
            write(iounit,909) rpbaspar(1,f), rpbaspar(2,f), &
                             ipbaspar(2,f)

         else if (basis(f) .eq. 19) then
            x1=rpbaspar(1,f)
            write(iounit,914) modelabel(f),f,'point',gdim(f),x1

         else if (basis(f).eq.20) then
            ibleg=ipbaspar(1,f)
            dx=abs(x2-x1)/dble(gdim(f)-1)
            if (ipbaspar(1,f).eq.0) then
               jmax=gdim(f)-1
            else            
               jmax=gdim(f)*2 - ipbaspar(1,f)
            endif
            write(iounit,915) modelabel(f),f,'KLeg/R',&
                  gdim(f),ipbaspar(3,f),x1,x2,dx,jmax,ibleg

         else if (basis(f).eq.21) then
            ibleg=ipbaspar(1,f)
            dx=abs(x2-x1)/dble(gdim(f)-1)
            if (ipbaspar(1,f).eq.0) then
               jmax=gdim(f)-1
            else
               jmax=gdim(f)*2 - ipbaspar(1,f)
            endif
            write(iounit,915) modelabel(f),f,'PLeg/R', &
                  gdim(f),ipbaspar(3,f),x1,x2,dx,jmax,ibleg

         else if (basis(f) .eq. 22) then
            write(iounit,916) modelabel(f),f,'tpoint',gdim(f),atpntop(f)

         else if (basis(f) .eq. 25) then
            dx=abs(x2-x1)/dble(gdim(f)-1)
            if (ipbaspar(1,f) .eq. 1) then
               if (ipbaspar(2,f) .ne. 0) then
                  write(iounit,910) modelabel(f),f,'GWPCL', &
                        gdim(f),x1,x2,dx,'  Periodic'
               else
                  write(iounit,902) modelabel(f),f,'GWPCL', &
                        gdim(f),x1,x2,dx
               endif
            else
               if (ipbaspar(2,f) .ne. 0) then
                  write(iounit,910) modelabel(f),f,'GWP', &
                        gdim(f),x1,x2,dx,'  Periodic'
               else
                  write(iounit,902) modelabel(f),f,'GWP', &
                        gdim(f),x1,x2,dx
               endif
            endif

         else if (basis(f) .eq. 26) then
            write(iounit,901) modelabel(f),f,'x-loc',gdim(f)

         else if (basis(f) .eq. 27) then
            dx=abs(x2-x1)/dble(gdim(f)-1)
            write(iounit,902) modelabel(f),f,'class', &
                     gdim(f),x1,x2,dx

         else if (basis(f) .eq. 31) then
            dx=abs(x2-x1)/dble(gdim(f)-1)
            dp=pi*gdim(f)/(dx*(gdim(f)+1))
            write(iounit,902) modelabel(f),f,'sinc',&
                  gdim(f),x1,x2,dx,dp

         else if (basis(f) .eq. 32) then
            dx=abs(x2-x1)/dble(gdim(f)-1)
            dp=pi*gdim(f)/(dx*(gdim(f)+1))
            write(iounit,902) modelabel(f),f,'wsinc',&
                  gdim(f),x1,x2,dx,dp

         else if (basis(f) .eq. 33) then   ! Wigner
            ibleg=ipbaspar(1,f)
            dx=abs(x2-x1)/dble(gdim(f)-1)
            if (ipbaspar(1,f).eq.0) then
               jmax=gdim(f)-1
            else
               jmax=gdim(f)*2 - ipbaspar(1,f)
            endif
            write(iounit,905) modelabel(f),f,'Wigner',gdim(f),x1,x2,dx,jmax,ibleg
         else if (basis(f) .eq. 34) then
            write(iounit,902) modelabel(f),f,'Lob ',&
                  gdim(f),x1,x2

         endif

      enddo

      if (febcont .gt. 0) then
         write(iounit,'(/,a)') '** Continuum Electronic States **'
         write(iounit,'(2(a,i5))') 'Total no. electronic states =',&
                                   ipbaspar(1,feb), &
                                   '  No. bound states =',ipbaspar(2,feb)
      endif

      call myflush(ilog)

      return

 900  format(//'****** Primitive Basis *********'//, &
        ' mode      kappa    DVR    N',8x, &
        'xi         xf         dx       p-max'/)
 901  format(1x,a12,i4,1x,a6,i5,4x)
 902  format(1x,a12,i4,1x,a6,i5,2f11.3,f11.6,f11.3,f9.4)
 922  format(1x,a12,i4,1x,a6,i5,2f11.3,f11.6,3x,a4)
 903  format(1x,a12,i4,1x,a6,i5,2f11.3,f11.6,'  m=',i2,', sym=',i2)
 904  format(1x,a12,i4,1x,a6,i5,'   j_dim =',i3,'   jmax =',i3,3x,a)
 905  format(1x,a12,i4,1x,a6,i5,2f11.3,f11.6,', jmax=',i3,', sym=',i2) 
 906  format(1x,a12,i4,1x,a6,i5,8x,i3,8x,i3,6x,i3)
 907  format(1x,a12,i4,1x,a6,i3,'/',i3,f9.3,f11.3,f11.6, &
             '  m=',i4,', sym=',i2)
 908  format(1x,a12,i4,1x,a6,i5,'    mmax =',i3, &
              '   mmin =',i3,'   mincr =',i3)
 909  format(20x,'x0 =',f10.6,',  b =',f11.6,',  icut =',i3)
 910  format(1x,a12,i4,1x,a6,i5,2f11.3,f11.6,a10)
 912  format(1x,a12,i4,1x,a5,i1,i5,2f11.3,f11.6,f11.3)
 913  format(1x,a12,i4,1x,a6,i5,2f11.3,9x,'-',9x,'-')
 914  format(1x,a12,i4,1x,a6,i5,f11.3,10x,'-',9x,'-',9x,'-')
 915  format(1x,a12,i4,1x,a6,i3,'/',i3,f9.3,f11.3,f11.6, &
             '  jmax=',i3,', sym=',i2)
 916  format(1x,a12,i4,1x,a6,i5,1x,'TD function = ',a10)

      end subroutine logdvr

!-----------------------------------------------------------------------
      end module logdvrmod
!-----------------------------------------------------------------------



