!#######################################################################
!
!                 RUNDVR
!
! This routine installs the dvr file.
!
!#######################################################################

      subroutine rundvr

use versions, only: progname
use global
use globmemmod
use timing
use maxv
use dvrdatmod

use decimal,    only: long, dop
use channels,   only: ilog, itim, iin
use initmod,    only: init
use zeigdvrmod, only: zeigdvr
use op1lib,     only: zerovxd, zerovxl, zerovxi, zerovxz
use dvrmod_interface

use logdat
use einpbasmod, only: eindvr
use memdimmod, only: memdiminp
use logdvrmod, only: logdvr
use einparamod

      implicit none

      integer(long)                              :: i,ilbl,ierr
      character(len=c5)                          :: filename
      character(len=10)                          :: cerr
      character(len=80)                          :: actual
      character(len=16)                          :: progpart
      logical(kind=4)                            :: check

      integer(long), save                        :: tid=0 ! timer ID

      integer(long), external   :: integersize,realsize,complexsize,logicalsize

      ierr = 0

!-----------------------------------------------------------------------
! Initialise and Start timing 
!-----------------------------------------------------------------------
      call timinit
      call get_timer(tid, 'GenDVR')
      call start_timer(tid)

      call fzeit(actual)
      i = index(actual,':')
      actual=actual(i-9:i+6)
      filename=name(1:laenge)//'/dvr'
      ilbl=index(filename,' ')-1
      write(ilog,'(/,72a1)')  ('#',i=1,72)
      write(ilog,'(/,a)') ' Creating DVR file: '//filename(1:ilbl)
      write(ilog,'(57x,a)')  actual(1:15)
      write(ilog,'(72a1)')  ('#',i=1,72)

!-----------------------------------------------------------------------
! Check input file open for reading
!-----------------------------------------------------------------------
      inquire(iin,opened=check)
      if (check) close(iin)
      if (inplaenge .le. 0) inplaenge=index(inputfile,' ')-1
      filename=inputfile(1:inplaenge)
      open(iin,file=filename,status='old',form='formatted',err=900)

!-----------------------------------------------------------------------
! read in parameters
!-----------------------------------------------------------------------
      call einpara

!-----------------------------------------------------------------------
! get basic dimensions (maxdim, maxsta)
!-----------------------------------------------------------------------
      call memdiminp(iin)

!-----------------------------------------------------------------------
! allocate memory for DVR data
!-----------------------------------------------------------------------
      allocmemory=0.0_dop
      call alloc_dvrdat

!-----------------------------------------------------------------------
! read in DVR data
!-----------------------------------------------------------------------
      call eindvr

!-----------------------------------------------------------------------
! Close input file
!-----------------------------------------------------------------------
      close(iin)

!-----------------------------------------------------------------------
! allocate memory for DVR generation
!-----------------------------------------------------------------------
      call zeigdvr
      call memdvr

      if (lBEL) write(1000,*)"lBEL",lBEL,"source/gendvr/rundvr.F90"
      if(allocated(mc))deallocate(mc)
      if(allocated(mr))deallocate(mr)
      if(allocated(mi))deallocate(mi)
      if(allocated(ml))deallocate(ml)

      cerr = 'rundvr_mc '
      allocate(mc(mcdim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + mcdim*complexsize()
      cerr = 'rundvr_mr '
      allocate(mr(mrdim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + mrdim*realsize()
      cerr = 'rundvr_mi '
      allocate(mi(midim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + midim*integersize()
      cerr = 'rundvr_ml '
      allocate(ml(mldim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + mldim*logicalsize()

!-----------------------------------------------------------------------
! write memory usage to log
!-----------------------------------------------------------------------
      if (progname(1:6) .ne. 'chnpot') then
         progpart='rundvr'
         call logmem(progpart)
      endif

!-----------------------------------------------------------------------
! Zero memory arrays
!-----------------------------------------------------------------------
      routine = 'Rundvr'

      call zerovxz(mc,mcdim)
      call zerovxd(mr,mrdim)
      call zerovxi(mi,midim)
      call zerovxl(ml,mldim)

!-----------------------------------------------------------------------
! Calculate FFT, DVR, ... data for single-particle functions
!-----------------------------------------------------------------------
      call init(mr(mrtrafo),mr(mrdvrmat),mr(mrort), &
           mr(mrfftp),mr(mrkinsph),mi(mijsph),&
           mi(mimsph),mc(mchin),mc(mcrueck),mi(mifftfak),mc(mcexphin),&
           mc(mcexprueck))

!-----------------------------------------------------------------------
! write grid and trafo/fftp data to dvr file
!-----------------------------------------------------------------------
      call wrdvr(mr(mrort),mr(mrtrafo),mr(mrdvrmat),mr(mrfftp),&
                 mc(mchin),mc(mcrueck),mi(mifftfak),mc(mcexphin),&
                 mc(mcexprueck),mi(mijsph),mi(mimsph),mr(mrkinsph))

!-----------------------------------------------------------------------
! log dvr information
!-----------------------------------------------------------------------
      call logdvr(ilog)

      cerr = 'rundvr_...'
      deallocate(mc,mr,mi,ml,stat=ierr)
      if (ierr > 0) go to 1001

      call dealloc_dvrdat

!-----------------------------------------------------------------------
! finish timing
!-----------------------------------------------------------------------
      call stop_timer(tid)
      if (ltime) then
         call timout
         close(itim)
      endif

      return

  900 continue
      routine='Gendvr'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop

 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      stop

 1001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
      stop

      end subroutine rundvr


