!***********************************************************************
!
!                       ADIAB
!
! contains : adiab,adiab2,adiab3,vintrd2,vintrd3,vintrdmat2,vintrdmat3,
! adddiag2,adddiag3,hdiag,searchev,sumcoeff
!
!***********************************************************************

      module adiabmod

use decimal, only: long, dop
use logdat
use mmlib,   only: mmxxzz
use op1lib,  only: zerovxz, zerovxd, cpvxz

      implicit none

      private
      public :: adiab
      contains

!***********************************************************************

      subroutine adiab(tmf,psi,ort,exphin)

use global
use dvrdatmod
use griddatmod
use psidef
use operdef
use runinwfmod
use channels

      implicit none

      real(dop), parameter :: widnum = 8.0_dop

      integer(long)       :: g,p,gmin,gmax,nef2,slz3,sl03,mmin3,m1,zeig

      real(dop)                                        :: gxeq1,gwidth1
      real(dop), dimension(sdim), intent(out)          :: tmf
      real(dop), dimension(ortdim), intent(in)         :: ort
      complex(dop), dimension(dgldim), intent(out)     :: psi
      complex(dop), dimension(sdim)                    :: phi3
      complex(dop), dimension(expdim), intent(in)      :: exphin

!-----------------------------------------------------------------------
! Zero MCTDH expansion coefficients and translational adiabatic
! potential array 'tmf'.
!-----------------------------------------------------------------------
      call zerovxz(psi,adim)
      call zerovxd(tmf,sdim)

!-----------------------------------------------------------------------
! get spf basis dependent parameters
!-----------------------------------------------------------------------
      gxeq1   = rsbaspar(1,1,1)
      gwidth1 = 1.0_dop/rsbaspar(3,1,1)
!     nef2    = isbaspar(4,2,1)-1
      nef2    = 0
      mmin3   = 0

!.....Because the spf's are now re-ordered. First spf is the occupied one.
      if (sbasis(3).eq.2) then
         slz3   = isbaspar(1,3,1)
         sl03   = isbaspar(2,3,1)
         if (subdim(3).gt.gdim(3)) mmin3=isbaspar(2,4,1)
      elseif (sbasis(3).eq.5) then
         sl03   = isbaspar(1,3,1)
         slz3   = isbaspar(1,4,1) 
         mmin3  = isbaspar(2,4,1)
      endif

      if(abs(rsbaspar(4,1,1)) .gt. 1.0e-16_dop) then
         routine='adiab'
         message='complex gaussian exponents not yet allowed'
         call errormsg
      end if

!-----------------------------------------------------------------------
! Determine those translational grid points for which the initial 
! Gaussian wave packet is almost zero.
!-----------------------------------------------------------------------
      gmin=1
      gmax=gdim(1)
      do g=1,gdim(1)
         p=zort(1)+g-1
         if (ort(p).lt.gxeq1-widnum*gwidth1) gmin=g
      enddo
      do g=gdim(1),gmin,-1
         p=zort(1)+g-1
         if (ort(p).gt.gxeq1+2.0_dop*widnum*gwidth1) gmax=g
      enddo
      write (ilog,'(a,f5.1,a,2i4)')  &
           'Widnum parameter in subroutine ADIAB: ', &
           widnum, ' gmin, gmax', gmin, gmax 
 
!-----------------------------------------------------------------------
! Calculate translational adiabatic potentials and new MCTDH expansion
! coefficients.
!-----------------------------------------------------------------------
      if (nmode.eq.2) then
         call adiab2(psi,psi(zetf(1,1)),psi(zetf(2,1)),ort(zort(1)) &
              ,ort(zort(2)),tmf,gdim(1),gdim(2),gmin,gmax &
              ,dim(1,1),dim(2,1),nef2,addim,eeigen,lbkmp2)
      elseif (nmode.eq.3) then
         if (basis(3).eq.9 .or. basis(3).eq.21) then
            call mmxxzz(psi(zetf(3,1)),exphin(zexp(3)),phi3,gdim(3), &
                        gdim(3),ipbaspar(3,4))
         else
            call cpvxz(psi(zetf(3,1)),phi3,subdim(3))
         endif
         slz3 = 0
         sl03 = 0
         if (subdim(3).gt.gdim(3) .and. sbasis(3).eq.5) then
             m1=slz3+1-mmin3
             zeig=(m1-1)*gdim(3)+1
         else
             zeig=1
         endif
               
         call adiab3(psi,psi(zetf(1,1)),psi(zetf(2,1)) &
              ,phi3(zeig),ort(zort(1)),ort(zort(2)),ort(zort(3)) &
              ,tmf,gdim(1),gdim(2),gdim(3),gmin,gmax &
              ,dim(1,1),dim(2,1),dim(3,1),mass(1),nef2,sl03,slz3,addim, &
               eeigen,jtot)
      endif

!-----------------------------------------------------------------------
! Define translational adiabatic mean-field on all grid points.
!-----------------------------------------------------------------------
      do g=1,gmin-1
         tmf(g)=tmf(gmin)
      enddo
      do g=gmax+1,gdim(1)
         tmf(g)=tmf(gmax)
      enddo


      return
      end subroutine adiab

!***********************************************************************
!
!                     ADIAB2
!
!***********************************************************************

      subroutine adiab2(acoeff,phi1,phi2,ort1,ort2,tmf,gdim1,gdim2, &
                 gmin,gmax,dim1,dim2,nef,addim,eeigen, &
                 lbkmp2)

use global
use channels

      implicit none

      integer(long), parameter  :: basmax=10

      integer(long)             :: g,p1,p2,pmin,num,basdim
      integer(long), intent(in) :: gmin,gmax,gdim1,gdim2,dim1,dim2,&
                                   nef,addim

      real(dop)                                          :: rd
      real(dop), dimension(gdim1), intent(out)           :: tmf
      real(dop), dimension(gdim1), intent(in)            :: ort1
      real(dop), dimension(gdim2), intent(in)            :: ort2
      real(dop), dimension(*), intent(in)                :: eeigen
      real(dop), dimension(gdim2)                        :: vint
      real(dop), dimension(dim2*dim2)                    :: vim
      real(dop), dimension(addim+addim**2+3*addim-1)     :: workr

      complex(dop), dimension(gdim1,dim1), intent(in)    :: phi1
      complex(dop), dimension(gdim2,dim2), intent(in)    :: phi2
      complex(dop), dimension(dim1,dim2), intent(inout)  :: acoeff
      logical(kind=4), intent(in)                        :: lbkmp2


!-----------------------------------------------------------------------
! Initialize variables.
!-----------------------------------------------------------------------
      routine = 'ADIAB2'
      pmin = 0

!-----------------------------------------------------------------------
! Set upper limit on dimension of diabatic basis. Higher basis functions
! can cover regions which are not relevant for the adiabatic
! correction and can cause therefore problems.
!-----------------------------------------------------------------------
      basdim=min(addim,basmax)
      write (ilog,'(a,i5)') &
           'Maximum number of basis functions considered in ADIAB2: '&
           ,basmax
      if (nef+1.gt.basmax) then
         message = 'Increase parameter basmax'
         call errormsg
      endif

!-----------------------------------------------------------------------
! Loop over those translational grid points for which the initial 
! Gaussian wave packet is almost zero, calculate the adiabatic
! Hamiltonian representations for each grid point, diagonalize it
! to determine the adiabatic potential and the MCTDH representation
! of the adiabatic initial state.
!-----------------------------------------------------------------------
      do g=gmin,gmax
!      do g=gmax,gmin,-1

!-----------------------------------------------------------------------
! Calculate interaction potential at all 'r' grid points 
! for given R=ort(g).
!-----------------------------------------------------------------------
         rd=ort1(g)
         call vintrd2(rd,ort2,vint,gdim2,lbkmp2)

!-----------------------------------------------------------------------
! Calculate 1D matrix representation of interaction potential.
! Basis functions are the diabatic eigenfunctions
! corresponding to sl0(2)==0 and nef(2) (phi2).
!-----------------------------------------------------------------------
         call vintrdmat2(vim,vint,phi2,gdim2,dim2,basdim)

!-----------------------------------------------------------------------
! Add diabatic eigenvalues to 'vim'.
!-----------------------------------------------------------------------
         call adddiag2(vim,eeigen,basdim)

!-----------------------------------------------------------------------
! Diagonalize 1D adiabatic channel Hamiltonian to determine adiabatic
! eigenvectors and eigenvalues depending parametically on the
! translational coordinate.
!  workr: addim + addim**2 + addim*(addim+1)/2+addim
!-----------------------------------------------------------------------
         p1=1+basdim
         p2=p1+basdim**2
         call hdiag(vim,basdim,workr,workr(p1),nef,ilog)

!-----------------------------------------------------------------------
! Search eigenvector and eigenvalue corresponding to nef(2) 
!-----------------------------------------------------------------------
         call searchev(workr(p1),num,nef,basdim)
         if (num.eq.0) then
            pmin=g
            tmf(g)=0.0_dop
         else
            tmf(g)=workr(num)
         endif

!-----------------------------------------------------------------------
! Calculate adiabatically corrected MCTDH expansion coeffs.
!-----------------------------------------------------------------------
         p1=p1+(num-1)*basdim
         call sumcoeff(acoeff,workr(p1),phi1,g,gdim1,dim1,dim2,basdim)

      enddo

!-----------------------------------------------------------------------
! 
!-----------------------------------------------------------------------
      if (pmin.ge.gmin) then
         write(6,   '(a,3i5)') 'gmin, gmax, pmin ', gmin,gmax,pmin
         write(ilog,'(a,3i5)') 'gmin, gmax, pmin ', gmin,gmax,pmin
         message = 'pmin error! Could not find corresponding ad-evec.'
         call errormsg
      endif
 
      return
      end subroutine adiab2

!***********************************************************************
!
!                     ADIAB3
!
!***********************************************************************

      subroutine adiab3(acoeff,phi1,phi2,phi3,ort1,ort2,ort3,tmf &
           ,gdim1,gdim2,gdim3,gmin,gmax,dim1,dim2,dim3 &
           ,mass1,nef,sl0,slz,addim &
           ,eeigen,jtot)

use global
use channels

      implicit none

      integer(long)             :: g,p1,p2,pmin,num
      integer(long), intent(in) :: gmin,gmax,gdim1,gdim2,gdim3,dim1,dim2,&
                                   dim3,nef,sl0,slz,addim

      real(dop)                                          :: rd,jbf
      real(dop), intent(in)                              :: mass1,jtot
      real(dop), dimension(gdim1), intent(out)           :: tmf
      real(dop), dimension(gdim1), intent(in)            :: ort1
      real(dop), dimension(gdim2), intent(in)            :: ort2
      real(dop), dimension(gdim3), intent(in)            :: ort3
      real(dop), dimension(*), intent(in)                :: eeigen
      real(dop), dimension(gdim2,gdim3)                  :: vint
      real(dop), dimension(addim,addim)                  :: vim
      real(dop), dimension(addim+addim**2+3*addim-1)     :: workr

      complex(dop), dimension(gdim3), intent(in)             :: phi3
      complex(dop), dimension(gdim1,dim1), intent(in)        :: phi1
      complex(dop), dimension(gdim2,dim2), intent(in)        :: phi2
      complex(dop), dimension(dim1,dim2,dim3), intent(inout) :: acoeff
 
!-----------------------------------------------------------------------
! Initializ variables.
!-----------------------------------------------------------------------
      routine = 'ADIAB3'
      pmin = 0

!-----------------------------------------------------------------------
! Loop over those translational grid points for which the initial 
! Gaussian wave packet is not almost zero, calculate the adiabatic
! Hamiltonian representations for each grid point, diagonalize it
! to determine the adiabatic potential and the MCTDH representation
! of the adiabatic initial state.
!-----------------------------------------------------------------------
      do g=gmin,gmax

!-----------------------------------------------------------------------
! Calculate interaction potential at all 'rv' and 'theta' grid points
! for given Rd=ort(g).
!-----------------------------------------------------------------------
         rd=ort1(g)
         call vintrd3(rd,ort2,ort3,vint,gdim2,gdim3)

!-----------------------------------------------------------------------
! Calculate 1D matrix representation of interaction potential.
! Basis functions are the diabatic eigenfunctions
! corresponding to sl0(2) and nef(2) (phi2).
! It is assumed that all but one MCTDH coefficients are zero.
!-----------------------------------------------------------------------
         call vintrdmat3(vim,vint,phi2,phi3,gdim2,gdim3,dim2)

!-----------------------------------------------------------------------
! Add translational centrifugal term and diabatic eigenvalues to 'vim'
!-----------------------------------------------------------------------
         jbf = dble(slz)
         call adddiag3(vim,rd,mass1,sl0,dim2,eeigen,jtot,jbf)
  
!-----------------------------------------------------------------------
! Diagonalize 1D adiabatic channel Hamiltonian to determine 2D adiabatic
! ground state and new adiabatic MCTDH expansion coefficients as
! components of the ground state vector.
!  workr: addim + addim**2 + addim*(addim+1)/2+addim
!-----------------------------------------------------------------------
         p1=1+addim
         p2=p1+addim**2
         call hdiag(vim,addim,workr,workr(p1),nef,ilog)

!-----------------------------------------------------------------------
! Search eigenvector and eigenvalue corresponding to nef(2) 
!-----------------------------------------------------------------------
         call searchev(workr(p1),num,nef,dim2)
         if (num.eq.0) then
            pmin=g
            tmf(g)=0.0_dop
         else
            tmf(g)=workr(num)
         endif

!-----------------------------------------------------------------------
! Calculate adiabatically corrected MCTDH expansion coeffs.
!-----------------------------------------------------------------------
         p1=p1+(num-1)*addim
         call sumcoeff(acoeff,workr(p1),phi1,g,gdim1,dim1,dim2,dim2)

      enddo

      if (pmin.ge.gmin) then
         write(6   ,'(a,3i5)') ' gmin, gmax, pmin ',gmin,gmax,pmin
         write(ilog,'(a,3i5)') ' gmin, gmax, pmin ',gmin,gmax,pmin
         write(6   ,'(a)') ' Lower the parameter "widnum" to fix this'
         write(ilog,'(a)') ' Lower the parameter "widnum" to fix this'
         message = 'pmin error! Could not find corresponding ad-evec.'
         call errormsg
      endif
 
      return
      end subroutine adiab3

!***********************************************************************
!
!                           VINTRD2
!
!***********************************************************************
      
      subroutine vintrd2(rd,ort2,vint,gdim2,lbkmp2)

      implicit none

      integer(long), parameter :: imod=2,icoll=1,ideriv=0

      integer(long)                            :: g2
      integer(long), intent(in)                :: gdim2
      real(dop)                                :: rv,r1,r2,r3,v,rminh2,vpoth2
      real(dop), intent(in)                    :: rd
      real(dop), dimension(3)                  :: dv
      real(dop), dimension(gdim2), intent(in)  :: ort2
      real(dop), dimension(gdim2), intent(out) :: vint
      logical(kind=4), intent(in)              :: lbkmp2

      if (lbkmp2) then
         rminh2=0.2_dop
         do g2=1,gdim2
            rv=ort2(g2)
            r1=max(rminh2,rd-0.5_dop*rv)
            r2=max(rminh2,rv)
            r3=r1+r2
            call bkmp2(r1,r2,r3,v,dv,ideriv)
            call vh2(r2,vpoth2)
            vint(g2)=v-vpoth2
         enddo
      else
         rminh2=0.4_dop
         do g2=1,gdim2
            rv=ort2(g2)
            r1=max(rminh2,rd-0.5_dop*rv)
            r2=max(rminh2,rv)
            r3=r1+r2
            call lsth(r1,r2,r3,v,imod,icoll)
            vint(g2)=v
         enddo
      endif

      return
      end subroutine vintrd2

!***********************************************************************
!
!                           VINTRD3
!
!***********************************************************************
      
      subroutine vintrd3(rd,ort2,ort3,vint,gdim2,gdim3)

use global
use griddatmod
use psidef
use runinwfmod

      implicit none

      integer(long)                                  :: imod,icoll,ideriv,g2,g3
      integer(long), intent(in)                      :: gdim2,gdim3
      real(dop)                                      :: rv,cphi,r1,r2,r3,&
                                                        v,x,vpoth2,rminh2
      real(dop), intent(in)                          :: rd
      real(dop), dimension(3)                        :: dv
      real(dop), dimension(gdim2), intent(in)        :: ort2
      real(dop), dimension(gdim3), intent(in)        :: ort3
      real(dop), dimension(gdim2,gdim3), intent(out) :: vint

      imod=2
      icoll=0
      ideriv=0

      if (lbkmp2) then
         rminh2=0.2_dop
         do g3=1,gdim3
            cphi=cos(ort3(g3))
            do g2=1,gdim2
               rv=ort2(g2)
               x=tfac*rv*rv/4.0_dop+rd*rd
               r1=max(rminh2, &
                      sqrt(rd**2+(tfac*rv)**2-2.0_dop*tfac*rd*rv*cphi))
               r2=max(rminh2,rv)
               r3=max(rminh2,sqrt(rd**2+((1.0_dop-tfac)*rv)**2+ &
                      2.0_dop*(1.0_dop-tfac)*rd*rv*cphi))
               call bkmp2(r1,r2,r3,v,dv,ideriv)
               call vh2(r2,vpoth2)
               vint(g2,g3)=v-vpoth2
            enddo
         enddo
      else
         rminh2=0.4_dop
         do g3=1,gdim3
            cphi=cos(ort3(g3))
            do g2=1,gdim2
               rv=ort2(g2)
               x=tfac*rv*rv/4.0_dop+rd*rd
               r1=max(rminh2, &
                      sqrt(rd**2+(tfac*rv)**2-2.0_dop*tfac*rd*rv*cphi))
               r2=max(rminh2,rv)
               r3=max(rminh2,sqrt(rd**2+((1.0_dop-tfac)*rv)**2+ &
                      2.0_dop*(1.0_dop-tfac)*rd*rv*cphi))
               call lsth(r1,r2,r3,v,imod,icoll)
               vint(g2,g3)=v
            enddo
         enddo
      endif

      return
      end subroutine vintrd3

!***********************************************************************
!
!                         VINTRDMAT2
! 
!***********************************************************************

      subroutine vintrdmat2(vim,vint,phi2,gdim2,dim2,basdim)

      implicit none

      integer(long)                                    :: e2a,e2b,g2
      integer(long), intent(in)                        :: gdim2,dim2,basdim
      real(dop), dimension(gdim2), intent(in)          :: vint
      real(dop), dimension(basdim,basdim), intent(out) :: vim
      complex(dop), dimension(gdim2,dim2), intent(in)  :: phi2
 
!-----------------------------------------------------------------------
! Compute subdiagonal and diagonal matrix elements
!-----------------------------------------------------------------------
      do e2b=1,basdim
         do e2a=e2b,basdim
            vim(e2a,e2b)=0.0_dop
            do g2=1,gdim2
               vim(e2a,e2b)=vim(e2a,e2b)+ &
                    dble(phi2(g2,e2a))*vint(g2)*dble(phi2(g2,e2b))
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
!  Compute upper diagonal matrix elements
!-----------------------------------------------------------------------
      do e2b=1,basdim
         do e2a=e2b+1,basdim
            vim(e2b,e2a)=vim(e2a,e2b)
         enddo
      enddo

      return
      end subroutine vintrdmat2

!***********************************************************************
!
!                         VINTRDMAT3
! 
!***********************************************************************

      subroutine vintrdmat3(vim,vint,phi2,phi3,gdim2,gdim3,dim2)

      implicit none

      integer(long)                                   :: e2a,e2b,g2,g3
      integer(long), intent(in)                       :: gdim2,gdim3,dim2
      real(dop), dimension(dim2,dim2), intent(out)    :: vim
      real(dop), dimension(gdim2,gdim3), intent(in)   :: vint
      complex(dop), dimension(gdim3), intent(in)      :: phi3
      complex(dop), dimension(gdim2,dim2), intent(in) :: phi2
 
!-----------------------------------------------------------------------
! Compute subdiagonal and diagonal matrix elements
!-----------------------------------------------------------------------
      do e2b=1,dim2
         do e2a=e2b,dim2
            vim(e2a,e2b)=0.0_dop
            do g3=1,gdim3
               do g2=1,gdim2
                  vim(e2a,e2b)=vim(e2a,e2b)+dble(phi3(g3)) &
                       *dble(phi2(g2,e2a))*vint(g2,g3) &
                       *dble(phi2(g2,e2b))*dble(phi3(g3))
               enddo
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
!  Compute upper diagonal matrix elements
!-----------------------------------------------------------------------
      do e2b=1,dim2
         do e2a=e2b+1,dim2
            vim(e2b,e2a)=vim(e2a,e2b)
         enddo
      enddo

      return
      end subroutine vintrdmat3

!***********************************************************************
!
!                       ADDDIAG2
!
!***********************************************************************

      subroutine adddiag2(vim,eeigen,basdim)

      implicit none

      integer(long)                                      :: e2
      integer(long), intent(in)                          :: basdim
      real(dop), dimension(basdim), intent(in)           :: eeigen
      real(dop), dimension(basdim,basdim), intent(inout) :: vim

!-----------------------------------------------------------------------
! Add diabatic eigenvalues
!-----------------------------------------------------------------------
      do e2=1,basdim
         vim(e2,e2)=vim(e2,e2)+eeigen(e2)
      enddo

      return
      end subroutine adddiag2

!***********************************************************************
!
!                       ADDDIAG3
!
!***********************************************************************

      subroutine adddiag3(vim,rd,mass1,sl0,dim2,eeigen,jtot,jbf)

      implicit none

      integer(long)                                  :: e2
      integer(long), intent(in)                      :: sl0,dim2
      real(dop)                                      :: erot
      real(dop), intent(in)                          :: rd,mass1,jtot,jbf
      real(dop), dimension(dim2,dim2), intent(inout) :: vim
      real(dop), dimension(*), intent(in)            :: eeigen

!-----------------------------------------------------------------------
! Add translational centrifugal term and diabatic eigenvalues.
! Additionally, add coupled states approximation centrifugal potential
! term (12/97 AJ)
!-----------------------------------------------------------------------
      erot=(sl0*(sl0+1.0_dop)+jtot*(jtot+1.0_dop)-2.0_dop*jbf*jbf)/(2.0_dop*mass1*rd**2)

      do e2=1,dim2
         vim(e2,e2)=vim(e2,e2)+erot+eeigen(e2)
      enddo

      return
      end subroutine adddiag3

!***********************************************************************
!
!                         HDIAG
!
!***********************************************************************

      subroutine hdiag(matrix,a,eval,evec,nef,ilog)

      implicit none
 
      integer(long)                            :: ier,i,j,pcomp
      integer(long), intent(in)                :: a,ilog,nef
      real(dop), dimension(a), intent(out)     :: eval
      real(dop), dimension(a,a), intent(inout) :: matrix
      real(dop), dimension(a,a), intent(out)   :: evec

      do j = 1,a
         do i = 1,j
            evec(i,j) = matrix(i,j)
         enddo
      enddo


!-----------------------------------------------------------------------
! get all eigenvalues "eval" and the corresponding normalized
! eigenvectors "evec" of the real symmetric matrix "evec" 
!-----------------------------------------------------------------------
      j=a*a
      call dsyev('V','U',a,evec,a,eval,matrix,j,ier)
      if (ier .ne. 0) then
        write(ilog,*) &
           'Error on diagonalising in HDIAG'
        write(ilog,*) 'Error code : ',ier
        stop
      endif

!-----------------------------------------------------------------------
! Define component corresponding to rovibrational state.
!-----------------------------------------------------------------------
      pcomp=nef+1

!-----------------------------------------------------------------------
! Choose phase such that the 'pcomp'th component of each eigenvector 
! is positive.
!-----------------------------------------------------------------------
      do j=1,a
         if (evec(pcomp,j).lt.0.0_dop) then
            do i=1,a
               evec(i,j)=-evec(i,j)
            enddo
         endif
      enddo
      
      return
      end subroutine hdiag

!***********************************************************************
!
!                       SEARCHEV
!
! Find eigenvector for which the largest value of the absolute values
! of all components corresponds to the (ro)vibrational state defined
! by 'nef' and 'sl0'.
!***********************************************************************

      subroutine searchev(evec,num,nef,dim2)

      implicit none

      integer(long)              :: e1,e2,pcomp,maxcomp
      integer(long), intent(in)  :: dim2,nef
      integer(long), intent(out) :: num

      real(dop)                                   :: comp,modcomp
      real(dop), dimension(dim2,dim2), intent(in) :: evec

!-----------------------------------------------------------------------
! Define component corresponding to (ro)vibrational diabatic state.
!-----------------------------------------------------------------------
      pcomp=nef+1

!-----------------------------------------------------------------------
! Find eigenvectors for which the modulus of the component 'pcomp'
! is the largest. Of these eigenvectors choose that one with the
! largest 'pcomp' component.
!-----------------------------------------------------------------------
      modcomp=0.0_dop
      num=0
      do e2=1,dim2
         comp=abs(evec(1,e2))
         maxcomp=1
         do e1=2,dim2           
            if (abs(evec(e1,e2)).gt.comp) then 
               comp=abs(evec(e1,e2))
               maxcomp=e1
            endif 
         enddo
         if (maxcomp.eq.pcomp) then
            if (abs(evec(pcomp,e2)).gt.modcomp) then
               modcomp=abs(evec(pcomp,e2))
               num=e2
            endif
         endif
      enddo
            
      return
      end subroutine searchev

!***********************************************************************
!
!                         SUMCOEFF
!
!***********************************************************************

      subroutine sumcoeff(acoeff,advec,phi1,g,gdim1,dim1,dim2,basdim)

      implicit none
      
      integer(long)             :: e1,e2
      integer(long), intent(in) :: g,gdim1,dim1,dim2,basdim

      real(dop), dimension(dim2), intent(in)            :: advec
      complex(dop), dimension(gdim1,dim1), intent(in)   :: phi1
      complex(dop), dimension(dim1,dim2), intent(inout) :: acoeff

      do e2=1,dim2
         if (e2.le.basdim) then
            do e1=1,dim1
               acoeff(e1,e2)=acoeff(e1,e2) &
                    +advec(e2)*dconjg(phi1(g,e1))*phi1(g,1)
            enddo
         endif
      enddo

      return
      end subroutine sumcoeff

      end module adiabmod

