!
! contains:chkinwf,chkbuild
!      

      module chkinwfmod 

use decimal, only: long, dop
use logdat

      implicit none

      private
      public :: chkinwf,chkbuild
      contains
     
!#######################################################################
! Check the input of the INIT_WF-SECTION for consistency.
!#######################################################################

      subroutine chkinwf(ligeosec)

use global
use dvrdatmod
use griddatmod
use psidef
use runinwfmod
use runpropmod
use channels
use operdef
use multkleg

      implicit none

      integer(long)       :: i,p,f,f1,f2,n,m,m1,blz,ibleg,slz,isleg,ijtot,ijbf,&
                             s,mmin,mmax,j,jmax,msym,pqchk,kmin,kmax,t
      logical(kind=4)     :: lchkerr
      logical,intent(in)  :: ligeosec !whether initial-geometry-section was read in input file

      routine = 'Chkinwf  (Check for consistency)'

!-----------------------------------------------------------------------
! Warn about unimplemented features for ML.
! TODO: This is currently incomplete.
!-----------------------------------------------------------------------
      if (lrunml) then
         if(lblockspf .or. lblocka) then
            message = '"block-SPF" and "block-A" are not'// &
              ' implemented for multi-layer'
            call warnmsg
         endif
      endif

!-----------------------------------------------------------------------
! Check that "build" and "file" do not appear simultaniously.
! unless there is an initial-geometry-section and an init_wf-setion
! in which case the restart overrides the build
!-----------------------------------------------------------------------
      if(lrestart .and. lbuildwf) then
         if (.not. ligeosec) then
            message = 'A build block must not appear in a restart-run' &
                      //' (i.e. file keyword).'
            call warnmsg
         else
            lbuildwf = .false.
         endif
      endif
      if((.not.lrestart).and.(.not.lbuildwf).and.&
           (.not.ligeosec).and.(.not.lblockspf)) then
         message = &
          'A build block must be given or a restart file specified'  
         call warnmsg
      endif
      if(lrestart .and. lblockspf) then
         message = 'There cannot be both a "block-SPF" and'&
                   //' a "file" keyword (restart-run).'
         call warnmsg
      endif
      if(lrestart .and. lblocka) then
         message = 'There cannot be both a "block-A" and'&
                   //' a "file" keyword (restart-run).'
         call warnmsg
      endif
      if(lblocka .and. lautoblk) then
         message = 'There can be either an "autoblock" '&
                   //'or a "read-inwf" keyword, but not both.'
         call warnmsg
      endif

      if(citype.gt.0) then
         if(inwfop(1) .gt. 0) then
            message = 'S-MCTDH and operate are incompatible.'
            call warnmsg
         endif 
         if(lrestart) then
            message = 'S-MCTDH requires "build", not "file".'
            call warnmsg
         endif 
      endif

      if (lrlxt) then
         if (all(dimfran(1:ndof) .eq. 0)) then
            write(message(1:70),'(a)') &
                   'Thermal relaxation asked for but no DOFs selected.'
            write(message(80:139),'(a)') &
               'Has a density operator ansatz been selected?'
!            write(message(140:),'(a)') &
!               'Use "ran" or "all_ran / no_ran" keywords in init_wf-section'
            call warnmsg
         endif
      endif

!-----------------------------------------------------------------------
! Check that nodiag is set for "operate".
!-----------------------------------------------------------------------
      do j = 1, 15
         if(inwfop(j).gt.0) then
            if(.not.nodiag(inwfop(j))) then
               write(message,'(a,i3,2a)')&
               'The operator',j,' used for "operate" must be ',&
               'generated with the "nodiag" keyword'
               call warnmsg
            endif
         endif
      enddo

!-----------------------------------------------------------------------
! flag any complete spf bases
! GWP basis can be over-complete so ignore
!-----------------------------------------------------------------------
      complete(:)=.false.
      if (dentype .eq. 1) then
         do t=1,nstate
            do s=1,nstate
               do m=1,nmode
                  if (ddim(m,s,t) .ge. subdim(m)) complete(m)=.true.
               enddo
            enddo
         enddo
      else
         do s=1,nstate
            do m=1,nmode
               if (dim(m,s) .ge. subdim(m)) complete(m)=.true.
            enddo
         enddo
      endif

      do m=1,nmode
         if (gwpm(m)) complete(m)=.false.
      enddo

!-----------------------------------------------------------------------
! check that 4n+1 points available if wigner=symmetric used
!-----------------------------------------------------------------------
      if (pqdistr .eq. 2) then
         do m=1,nmode
            f=spfdof(1,m)
            if (basis(f) .eq. 26 .or. basis(f) .eq. 27) then  
               do s=1,nstate
                  pqchk=dim(m,s)-1
                  pqchk=mod(pqchk,4) 
                  if (pqchk .ne. 0) then
                     pqchk = dim(m,s) - pqchk
                     if (nmode .gt. 1) then
                        write(ilog,'(2(a,i5))') &
            '      For mode:',m,' state: ',s
                     endif
                     write(ilog,'(a)') &
            'Need 4n+1 points for trajectory modes in symmetric Wigner'
                     routine = 'Chkinwf (chkinwfmod.f90) '
                     message = 'Selecting non-symmetric Wigner '
                     call warnmsg1

                     pqdistr = 1
                  endif
               enddo
            endif
         enddo
      endif

!-----------------------------------------------------------------------
! check that 4n+1 points available if symmetric PQ distribution of GWPs used
!-----------------------------------------------------------------------
      if (lgwp) then
         do s=1,nstate
            do m=1,nmode
               if (.not. gwpm(m)) cycle
               do f=1,nspfdof(m)
                  if (isbaspar(1,f,s) .eq. 7) then
                     n = dim(m,s) - 1
                     n = mod(n,4)
                     if (n .ne. 0) then
                        message(1:79)= &
                'For symmetrical PQ selection of GWPs, dim = 4n+1'
                        message(80:160)= &
                'Change no. GWPs or select random_pq for mode: '//modelabel(f)
                        call warnmsg
                     endif
                  endif
               enddo
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! load particular parameters from operator (e.g. jtot) needed for
! initial wavefunction generation
!
! if jbf is set, check that slz and blz are set to this value
!-----------------------------------------------------------------------
      ijtot=nint(abs(jtot))     ! If jtot is not set, it is -999.0
      ijbf=nint(jbf)

      do f = 1,ndof
         if ( (basis(f).eq.2 .or. basis(f).eq.12)  &
                        .and. sbasis(f).eq.2 ) then
            if (isbaspar(1,f,1) .ne. ipbaspar(1,f)) then
               message =  &
                  'slz not equal to blz for mode :'//modelabel(f)
               call warnmsg
            endif
         endif

         if (basis(f).eq.7 .or. basis(f).eq.20) then
            if (ipbaspar(1,f).eq.0) then
               jmax=gdim(f)-1
            else            
               jmax=gdim(f)*2 - ipbaspar(1,f)
            endif 
            mmin=ipbaspar(1,f+1)
            mmax=ipbaspar(2,f+1)
!            if (max(abs(mmin),abs(mmax)).gt.ijtot .and. .not.lft) then
!               write(6,*) ' mmin, mmax, jtot =', mmin, mmax, ijtot
!               message=
!     +         'KLeg: abs(mmin) or abs(mmax) is greater than jtot'
!               call errormsg
!            endif
            if (max(abs(mmin),abs(mmax)) .gt. jmax) then
               write(6,*) ' mmin, mmax, jmax =', mmin, mmax, jmax
               message= &
               'KLeg: abs(mmin) or abs(mmax) is greater than jmax'
               call errormsg
            endif
         endif
      
         if (basis(f).eq.9 .or. basis(f).eq.21) then
            if (ipbaspar(1,f).eq.0) then
               jmax=gdim(f)-1
            else            
               jmax=gdim(f)*2 - ipbaspar(1,f)
            endif 
            mmin=ipbaspar(1,f+1)
            mmax=ipbaspar(2,f+1)
!            if (max(abs(mmin),abs(mmax)) .gt. ijtot) then
!               write(6,*) ' mmin, mmax, jtot =', mmin, mmax, ijtot
!               message=
!     +         'PLeg: abs(mmin) or abs(mmax) is greater than jtot'
!               call errormsg
!            endif
            if (max(abs(mmin),abs(mmax)) .gt. jmax) then
               write(6,*) ' mmin, mmax, jmax =', mmin, mmax, jmax
               message= &
               'PLeg: abs(mmin) or abs(mmax) is greater than jmax'
               call errormsg
            endif
         endif

         if (basis(f).eq.7 .or. basis(f).eq.9) then
            if(dofspf(f) .ne. dofspf(f+1)) then
               write(6,'(/2x,a8,1x,a8,2x,a,2i4,2x,a,2i4)') &
                    modelabel(f),modelabel(f+1),'f =',f, f+1, &
                    ',   m =',dofspf(f), dofspf(f+1)
               message='The two DOFs of KLeg or PLeg must be combined.'
               call errormsg
            endif
         endif

         if (basis(f).eq.33) then
            if (ipbaspar(1,f).eq.0) then
               jmax=gdim(f)-1
            else
               jmax=gdim(f)*2 - ipbaspar(1,f)
            endif
            kmin=ipbaspar(2,f)
            kmax=ipbaspar(3,f)
            mmin=ipbaspar(6,f)
            mmax=ipbaspar(7,f)
            if (max(abs(kmin),abs(kmax)) .gt. jmax) then
               write(6,*) ' kmin, kmax, jmax =', kmin, kmax, jmax
               message= &
               'Wigner: abs(kmin) or abs(kmax) is greater than jmax'
               call errormsg
            endif
            if (max(abs(mmin),abs(mmax)) .gt. jmax) then
               write(6,*) ' mmin, mmax, jmax =', mmin, mmax, jmax
               message= &
               'Wigner: abs(mmin) or abs(mmax) is greater than jmax'
               call errormsg
            endif
            if ((dofspf(f) .ne. dofspf(f+1)) .or. &
                     (dofspf(f) .ne. dofspf(f+2))) then
               write(6,'(/2x,a8,1x,a8,1x,a8,2x,a,3i4,2x,a,3i4)') &
                 modelabel(f),modelabel(f+1),modelabel(f+2),'f =',f, &
                 f+1,f+2,',   m =',dofspf(f),dofspf(f+1),dofspf(f+2)
               message='The three DOFs of Wigner must be combined.'
               call errormsg
            endif
         endif

      enddo

      if( ijbf .ge. 0 ) then 
         do f = 1,ndof
            if ( (basis(f).eq.2 .or. basis(f).eq.12) &
                           .and. sbasis(f).eq.2 ) then
               if( ijtot.ge.0 .and. ijtot.lt.ijbf) then
                  message = 'jtot is smaller than jbf'
                  call warnmsg
               end if
            endif
            if ( (basis(f).eq.2 .or. basis(f).eq.12)  &
                        .and. ipbaspar(1,f).ne.ijbf) then
               message = &
              'blz not equal to jbf for mode :'//modelabel(f)
               call warnmsg
            endif
            if (sbasis(f).eq.2 .and. isbaspar(1,f,1).ne.ijbf) then
               message = &
                    'slz not equal to jbf for mode :'//modelabel(f)
               call warnmsg
            endif
         enddo
      endif

!-----------------------------------------------------------------------
!     Check for consistency
!-----------------------------------------------------------------------
      routine = 'Chkinwf  (Check for consistency)'
!      if (leb .and. lmulpack) then
!         message = 'Single-set and multi-packet do not mix'
!         call warnmsg
!      endif
      if (ladiab .and. lmulpack) then
         message = &
            'Multi-packet with adiabatic corrections not implemented'
         call warnmsg
      endif

      do p = 1,npacket
         do f = 1,ndof
            if ( (basis(f).eq.2 .or. basis(f).eq.12) &
                          .and. sbasis(f).eq.2 ) then
               blz=ipbaspar(1,f)
               ibleg=ipbaspar(2,f)
               slz=isbaspar(1,f,p)
               isleg=isbaspar(3,f,p)
               if (ibleg.ne.0 .and. isleg.ne.ibleg) then
                  message = 'isleg and ibleg are inconsistent'
                  call warnmsg
               endif
               if (slz.ne.blz) then
                  message = 'slz must equal blz for Leg-DVR'
                  call warnmsg
               endif
            endif
         enddo
      enddo

      do m = 1, nmode
         do n = 1, nspfdof(m)    
            f = spfdof(n,m)
            if(basis(f) .eq. 6) then
               if(nspfdof(m).ne.2) then
                  if(lexact .or. lrealwp) write(6,'(/a)') &
                  ' ERROR: "exact" not possible with sphFBR'
                  write(6,'(a,3i5)') ' n, m, nspfdof =',n,m,nspfdof(m)
                  message= &
                  'sphFBR must operate on a 2D-mode (alpha,beta)'
                  call errormsg
               endif
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
!     Check msymmtr  (keywords: asym2d, sym2d, asymmcoef, symmcoeff)
! m=200 -> symcoeff,  f=200 -> sym3d,  f=201 -> sym2kleg, f=202 -> nsym2d
!-----------------------------------------------------------------------
      lsymmultkleg(:)=.false.
      do j = 1, 2*maxdim
         if(msymmtr(j).eq.0)  goto 35
         lchkerr=.false.
         msym=mod(abs(msymmtr(j)),65536)
         if(msym.eq.51200) then
            do s=1, nstate
               do m=1, nmode-1
                  do m1=m+1, nmode
                     if (symcv(m).eq.symcv(m1) .and. m1.ne.mpb) then
                        if(dim(m,s).ne.dim(m1,s)) then
                           write(6,'(/a,/a,i3,a,2i3,a,i3,a,i3)')&
                           '----------------------------------------',&
                           's =',s,', m,m1 =',m,m1,&
                           ', dim(m,s) =',dim(m,s),&
                           ', dim(m1,s) =',dim(m1,s)
                           write(2,'(/a,/a,i3,a,2i3,a,i3,a,i3)')&
                           '----------------------------------------',&
                           's =',s,', m,m1 =',m,m1,&
                           ', dim(m,s) =',dim(m,s),&
                           ', dim(m1,s) =',dim(m1,s)
                           message='Cannot (a)symmetrize '//&
                                   "coefficients, n's not equal."
                           call errormsg

                        endif
                     endif
                  enddo
               enddo
            enddo
         elseif (msym.gt.0 .and. &
                (mod(msym,256).eq.0 .or. mod(msym,256).eq.202)) then
            m =msym/256
            f1=spfdof(1,m)
            f2=spfdof(2,m)
            if( nspfdof(m).ne.2  .or.  gdim(f1).ne.gdim(f2) .or. &
                            basis(f1).ne.basis(f2) ) lchkerr=.true.
            do i=1,mbaspar
               if(ipbaspar(i,f2).ne.ipbaspar(i,f1)) lchkerr=.true.
               if(abs(rpbaspar(i,f2)-rpbaspar(i,f1)) &
                                         .gt. tiny) lchkerr=.true.
            enddo
            if(lchkerr) then
               write(6,'(a,2i3,a,2i4)') 'mode,nspfdof:',m,nspfdof(m), &
                      ' ; f1,f2:', f1,f2
               write(6,'(a,2i5,a,2i3)') 'gdim1,gdim2:',gdim(f1), &
                  gdim(f2),' ; basis1,basis2:',basis(f1),basis(f2)
               message= &
              'Cannot (a)symmetrize this mode ((a)sym2d). Bases differ.'
               call errormsg
            endif 
         elseif(msym.gt.0  .and.  mod(msym,256).eq.200) then
            m =msym/256
            f1=spfdof(1,m)
            f2=spfdof(2,m)
            f =spfdof(3,m)
            if( nspfdof(m).ne.3  .or.  gdim(f1).ne.gdim(f2) .or. &
                gdim(f1).ne.gdim(f) .or. basis(f1).ne.basis(f2)  &
                .or. basis(f1).ne.basis(f)   ) lchkerr=.true.
            do i=1,mbaspar
               if(ipbaspar(i,f2).ne.ipbaspar(i,f1)) lchkerr=.true.
               if(ipbaspar(i,f ).ne.ipbaspar(i,f1)) lchkerr=.true.
               if(abs(rpbaspar(i,f2)-rpbaspar(i,f1))&
                                         .gt. tiny) lchkerr=.true.
               if(abs(rpbaspar(i,f )-rpbaspar(i,f1))&
                                         .gt. tiny) lchkerr=.true.
            enddo 
            if(lchkerr) then
               write(6,'(a,2i3,a,3i4)') 'mode,nspfdof:',m,nspfdof(m),&
                      ' ; f1,f2,f3:', f1,f2,f
               write(6,'(a,3i5,a,3i3)')&
                  'gdim1,gdim2,gdim3:',   gdim(f1),gdim(f2),gdim(f),&
               ' ; basis1,basis2,basis3:',basis(f1),basis(f2),basis(f)
               message=&
               'Cannot symmetrize this mode (sym3d). Bases differ.'
               call errormsg
            endif
         elseif(msym.gt.0 .and. mod(msym,256).eq.201) then
            m =msym/256
            f1=spfdof(1,m)
            f2=spfdof(3,m)
            if( nspfdof(m).ne.4 .or.&
                gdim(f1).ne.gdim(f2) .or.&
                gdim(f1+1).ne.gdim(f2+1) .or.&
                basis(f1).ne.7 .and. basis(f1).ne.20 .or.&
                basis(f2).ne.7 .and. basis(f2).ne.20) lchkerr=.true.
            do i=1,mbaspar
               if(ipbaspar(i,f2).ne.ipbaspar(i,f1)) lchkerr=.true.
               if(ipbaspar(i,f2+1).ne.ipbaspar(i,f1+1)) lchkerr=.true.
               if(abs(rpbaspar(i,f2)-rpbaspar(i,f1))&
                                         .gt. tiny) lchkerr=.true.
               if(abs(rpbaspar(i,f2+1)-rpbaspar(i,f1+1))&
                                         .gt. tiny) lchkerr=.true.
            enddo
            if(lchkerr) then
               write(6,'(a,2i3,a,4i4)') 'mode,nspfdof:',m,nspfdof(m),&
                      ' ; f1,k1,f2,k2:', f1,f1+1,f2,f2+1
               write(6,'(a,2i5,a,2i5,a,2i3)')&
                  'gdim1,gdim2:',gdim(f1),gdim(f2),&
                  ' ; kdim1,kdim2:',gdim(f1+1),gdim(f2+1),&
                  ' ; basis1,basis2:',basis(f1),basis(f2)
               message='Cannot (a)symmetrize this mode ((a)sym2kleg).'&
                  //' Not a KLeg/K/KLeg/K mode, or bases differ.'
               call errormsg
            endif 
            lsymmultkleg(m)=.true.
         endif
      enddo
 35   continue

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true. Stop now if error.
!-----------------------------------------------------------------------
      if (lwarn) then
         write(ilog,'(a)') '****** Error in initial wavefunction definition. STOP ******'
         write(6,'(a)') '****** Error in initial wavefunction definition. STOP ******'
         stop
      endif

      return
      end subroutine chkinwf

!#######################################################################

      subroutine chkbuild

use global
use dvrdatmod
use griddatmod
use psidef
use runinwfmod

      implicit none

      integer(long) :: p,m,n,f,s,p1,p2,i,ns

      routine = 'Chkbuild  (Check for consistency)'

!-----------------------------------------------------------------------
! check that a basis has been specified for each mode (state=1).
!-----------------------------------------------------------------------
      if (lmulpack .and. nstate .eq. 1) then
         sbfound(feb,1)=.true.
      endif

      do m=1,nmode          
         do n=1,nspfdof(m)          
            f=spfdof(n,m)           
            if (.not. sbfound(f,1)) then
               routine='Eininwf (chkbuild)'
               write(message,'(2a,i5)') 'Initial orbital not ',&
                    'specified for first state/packet for dof :',f
               call warnmsg
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! copy spf basis parameters for 1st state to all packets
! then copy 1st state to other states
!-----------------------------------------------------------------------
      if (lmult) then
         ns = nddstate
      else
         ns = 1
      endif
      do p = 2,npacket
         p1=ns*(p-1)+1
         do f=1,ndof
            if (.not. sbfound(f,p1)) then
               do i=1,sbaspar
                  isbaspar(i,f,p1) = isbaspar(i,f,1)
                  rsbaspar(i,f,p1) = rsbaspar(i,f,1)
                  sbfound(f,p1) = sbfound(f,1)
               enddo
            endif
         enddo
      enddo

      do p = 1,npacket
         p1=ns*(p-1)+1
         p2=ns*(p-1)+ns
         do s=p1+1,p2
            do f=1,ndof
               if (.not. sbfound(f,s)) then
                  do i=1,sbaspar
                     isbaspar(i,f,s) = isbaspar(i,f,p1)
                     rsbaspar(i,f,s) = rsbaspar(i,f,p1)
                     sbfound(f,s) = sbfound(f,p1)
                  enddo
               endif
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! check that electronic dof has an electronic (discrete) spf basis
! sbasis should be zero, but for single-set it may be 12 (readspf)
!-----------------------------------------------------------------------
      if (feb .gt. 0 .and. febcont .eq. 0)  then
         if(sbasis(feb) .ne. 0 .and. sbasis(feb) .ne. 10) then
            if(lmult .or. sbasis(feb) .ne. 12) then

               write(6,*) 'feb, basis, sbasis :',&
                           feb, basis(feb), sbasis(feb) 
               routine='Eininwf (chkbuild)'
               message=&
              'Incorrect Spf basis type for electronic dof.'
               call warnmsg
            endif
         endif
      endif

!-----------------------------------------------------------------------
! check that a basis has been specified for each mode and state.
!-----------------------------------------------------------------------
      if (lmult) then
         ns = nddstate
      else
         ns = 1
      endif
      do p = 1,npacket
         do s=1,ns
            p1=ns*(p-1)+s
            do m=1,nmode          
               do n=1,nspfdof(m)          
                  f=spfdof(n,m)           
                  if (.not. sbfound(f,p1)) then
                     routine='Eininwf (chkbuild)'
                     write(message,'(2a,2i3,a,i5)') &
                     'Initial orbital must be specified for ',&
                     'state/packet :',s,p,' and for dof :',f
                     call warnmsg
                  endif
               enddo
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! for trajectory driven GWPs the initial functions must not be
! optimised to fit initial WP.
!-----------------------------------------------------------------------
      if (lgwptrj) gwpopit = 0

      return
      end subroutine chkbuild

      end module chkinwfmod

