
!!  Computes the dtems (i.e. hterms for operate)
!!
!!  In parts copied from ml-mctdh/mlcore/eqofmotion.f90
!!


module dterms

    use wftree
    use multilayer
    use htermmod
    use linear
    use timing
    use mlop_eom
    use mlanalysis
    use global
    use eqofmotion
    use psidef, only: block, dmatdim, zetf, dmat, dim, dgldim
    use griddatmod, only: tree, nmode, subdim
    use mmlib, only: mmaxzz

    private
    public  :: compute_dterms_tree,  &
               compute_dfields_tree, &
               noniterative_tree,    &
               iterative_tree

    logical,save   :: debug = .false.

    contains


! -----------------------------------------------------------------------------
! TREE COMPUTATION PROCEDURES
!
! subroutines and functions that work at the tree level. They contain usually
! calls to node routines, which are the ones doing the real work.
! -----------------------------------------------------------------------------
    subroutine compute_dterms_tree(psi1,psi2,koeff,maxkoe,nham)
        implicit none

        complex(dop)                :: psi1(dgldim),psi2(dgldim),koeff(maxkoe)
        integer, intent(in)         :: nham
        integer                     :: maxkoe
        integer                  :: i
        type(wfnode_t),pointer   :: cnode
        integer, save            :: tid = 0 ! timer ID

        call get_timer(tid, 'dterms_tree')
        call start_timer(tid)
        do i=1,tree%tot_nodes
            cnode => tree%swap_up(i)%p
            call compute_dterms_node(cnode,psi1,psi2,nham)
            call noniterative_node(cnode,psi1,psi2,koeff,nham)
        end do
        call stop_timer(tid)
        if(debug) write(6,*) ' END DTERMS TREE'
    end subroutine compute_dterms_tree

    subroutine noniterative_tree(psi1,psi2,koeff,maxkoe,mactive,nham)
        implicit none
        complex(dop),intent(in)     :: psi2(:),koeff(maxkoe)
        complex(dop),intent(inout)  :: psi1(:)
        integer, intent(in)      :: nham
        integer,intent(in)           :: maxkoe,mactive
        integer                  :: i,numparent
        type(wfnode_t),pointer   :: cnode
        integer, save            :: tid = 0 ! timer ID

        call get_timer(tid, 'operate-ml')
        call start_timer(tid)
        numparent=0
        do i=1,tree%tot_nodes
            cnode => tree%swap_up(i)%p
            call compute_dterms_node(cnode,psi1,psi2,nham)
            if(cnode%isprimitive) then
               if(cnode%m.ne.mactive) cycle
            elseif(cnode%num.ne.numparent) then
               if(mactive.ne.0) cycle
            endif
            call noniterative_node(cnode,psi1,psi2,koeff,nham)
            if(i.lt.tree%tot_nodes) numparent=cnode%parent%num
        end do
        call stop_timer(tid)
    end subroutine noniterative_tree


    subroutine iterative_tree(psi1,psi2,psi3,koeff,maxkoe,kshift,&
               OPnit,OPtol,nham)
        implicit none
        complex(dop),intent(in)     :: psi2(:)
        complex(dop),intent(inout)  :: psi1(dgldim),psi3(dgldim),&
                                       koeff(maxkoe)
        integer,intent(in)           :: maxkoe,kshift,OPnit
        real(dop),intent(in)         :: OPtol
        integer                  :: i,j
        real(dop)                :: OPerr
        type(wfnode_t),pointer   :: cnode
        integer, intent(in)      :: nham
        integer, save            :: tid = 0 ! timer ID

        call get_timer(tid, 'operate-ml')
        call start_timer(tid)
        do i=1,tree%tot_nodes
            cnode => tree%swap_up(i)%p
            call compute_dterms_node(cnode,psi1,psi2,nham)
            call noniterative_node(cnode,psi1,psi2,koeff,nham)
        end do

        psi3(:)=psi2(:); i=0
        call operate_error(OPerr,psi1,psi2,psi3,tree,i,koeff,kshift)

        do i=1,opnit
           psi3(:)=psi1(:)
           call compute_dfields_tree(tree,psi1,psi2,nham)
           do j=1,tree%tot_nodes
               cnode => tree%swap_up(j)%p
               call compute_dterms_node(cnode,psi1,psi2,nham)
               call iterative_node(cnode,psi1,psi2,koeff,nham)
            enddo
           call operate_error(OPerr,psi1,psi2,psi3,tree,i,koeff,kshift)
           if(OPerr<OPtol) exit
        enddo
        call stop_timer(tid)
    end subroutine iterative_tree


    subroutine operate_error(OPerr,psi1,psi2,psi3,tree,i,koeff,kshift)
        implicit none
        type(wfnode_t), pointer      :: tree
        integer,intent(in)           :: i,kshift
        complex(dop),intent(inout)  :: koeff(:)
        real(dop),intent(out)        :: OPerr
        complex(dop),intent(in)     :: psi1(dgldim),psi2(dgldim),&
                                       psi3(dgldim)
        complex(dop)                :: ovl
        real(dop),save               :: normq1,normq3

        integer :: m
        complex(dop), dimension(dmatdim) :: ovlmat

!$OMP PARALLEL
        if(i.eq.0) then
!$OMP WORKSHARE
           normq3=dble(dot_product(psi3(1:block(1)),psi3(1:block(1))))
!$OMP END WORKSHARE
        endif
!$OMP WORKSHARE
        normq1=dble(dot_product(psi1(1:block(1)),psi1(1:block(1))))
!$OMP END WORKSHARE
!$OMP END PARALLEL

!NEED SPF OVERLAPS FIRST AND THEN CALL OVERLAP_1
        do m=1,nmode
           call mmaxzz(psi1(zetf(m,1)),psi3(zetf(m,1)), &
                ovlmat(dmat(m,1)),subdim(m),dim(m,1),dim(m,1))
        enddo
        call overlap_1(psi1,psi3,tree,ovl,ovlmat)
        OPerr = 1.d0 - 2.d0*dble(ovl)/(normq1+normq3)
        normq3=normq1

        if(kshift.gt.0) then
           if(i.gt.0) then
!NEED SPF OVERLAPS FIRST AND THEN CALL OVERLAP_1
              do m=1,nmode
                 call mmaxzz(psi2(zetf(m,1)),psi1(zetf(m,1)), &
                      ovlmat(dmat(m,1)),subdim(m),dim(m,1),dim(m,1))
              enddo
              call overlap_1(psi2,psi1,tree,ovl,ovlmat)
              koeff(kshift) = koeff(kshift) - ovl
           endif
           write(2,'(i5,1pe13.3,1pe18.9,a,1pe13.6,1pe14.6,a,1pe10.2)')&
                i, OPerr, sqrt(normq1),'  (',-koeff(kshift),') ' !, dble(ovl)
           call myflush(2)
        else
           write(2,'(i5,1x,1p2e15.6)') i, OPerr, sqrt(normq1)
           call myflush(2)
        endif
    end subroutine operate_error

    subroutine compute_dfields_tree(tree,psi1,psi2,nham)
        implicit none
        type(wfnode_t),intent(inout) :: tree
        complex(dop),intent(in)      :: psi1(:),psi2(:)
        integer, intent(in)          :: nham 
        integer                      :: i
        type(wfnode_t),pointer       :: cnode
        integer, save                :: tid = 0 ! timer ID
        if(debug) write(6,*) ' DFIELDS TREE'
        call get_timer(tid, 'dfields_tree')
        call start_timer(tid)
        do i=1,tree%tot_nodes
           cnode => tree%swap_down(i)%p
           call compute_dfields_node(cnode,psi1,psi2,nham)
        end do
        call stop_timer(tid)
        if(debug) write(6,*) ' END DFIELDS TREE'
    end subroutine compute_dfields_tree


! -----------------------------------------------------------------------------
! NODE COMPUTATION PROCEDURES
!
! subroutines and functions that work at the node level
! -----------------------------------------------------------------------------

    subroutine compute_dterms_node(t,psi1,psi2,nham)
    ! Eq.(33), Manthe,[JCP,128,164116,(2008)]
        implicit none
        type(wfnode_t),intent(in)    :: t
        complex(dop),intent(in)     :: psi1(:),psi2(:)
        type(wfnode_t),pointer       :: childf
        complex(dop),allocatable    :: work(:)
        integer                      :: ndim,subdim,k,f,i,j
        integer                      :: z0,zf,ki,kf
        integer, intent(in)          :: nham 

        if(t%isprimitive) return

        ki = zham(nham)
        kf = zham(nham)+kzahl(nham)-1

        ! loop over modes
        do f=1,t%nmodes
            childf => t%submodes(f)%p
            if (childf%tid_hterms == 0) then
               call init_node_timer(childf, childf%tid_hterms, 'dterms')
            endif
            call start_timer(childf%tid_hterms)
            ndim   =  t%ndim(f)
            subdim =  product(childf%ndim)

!$OMP       parallel default(shared) private(work,k,i,z0,zf,j)
            allocate(work(subdim))
!$OMP       do schedule(runtime)
            do k=ki,kf ! loop over h_k oper products (k-terms)
                do i=1,ndim ! loop over elements h_f^k(i,j)
                    z0 = childf%z(i,1)
                    zf = childf%z(i,2)
                    ! compute h_k|phi_i>
                    if(childf%isprimitive) then
                        call hcall(childf%m,k,work,psi2(z0:zf),subdim)

                    else
                        work = psi2(z0:zf)
                        call hprod_phi(childf%hterms(k,:),work)
                    end if
                    ! do matrix elements <phi_j|h|phi_i>.
                    ! No symmetry (hermitian, antihermitian,...) is assumed.
                    do j=1,ndim
                        z0 = childf%z(j,1)
                        zf = childf%z(j,2)
                        t%hterms(k,f)%p%matrix(j,i) = phi_phi(psi1(z0:zf),work)
                    end do
                end do
            end do
!$OMP       end do
            deallocate(work)
!$OMP       end parallel

            call stop_timer(childf%tid_hterms)
        end do

        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        if(debug) then
            do f=1,t%nmodes
                ndim = t%ndim(f)
                do k=ki,kf
                    if(t%isunit(k,f)) cycle ! skip h_f^k
                    write(6,*) 'dterm: nodenum,f,k', t%num,f,k
                    do j=1,ndim
                        write(6,10) (t%hterms(k,f)%p%matrix(j,i),i=1,ndim)
                    end do
                end do
            end do
        end if
        10 format(8(2x,'(',1PE11.4,1x,1PE11.4,')'))
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    end subroutine compute_dterms_node


!------------------------------------------------------------------------
!  ML-operate, non-iterative procedure.  A = [d]*\tilde{A},  HDM 08/2020
!------------------------------------------------------------------------
    subroutine noniterative_node(t,psi1,psi2,koeff,nham)
        implicit none
        type(wfnode_t),intent(inout) :: t
        type(hterm_tp),pointer       :: hk(:)
        complex(dop)                :: psi1(:),psi2(:),koeff(:)
        complex(dop),allocatable    :: work(:)
        integer                      :: subdim,k,r,ierr,navcs,i
        integer                      :: z0,zf,ki,kf
        integer,intent(in)           :: nham 
        real(dop)                    :: norm

        navcs  = t%navecs
        subdim = product(t%ndim)
        allocate(work(subdim))

        ki = zham(nham)
        kf = zham(nham)+kzahl(nham)-1

        if(t%isprimitive) then
           do r=1,navcs
               z0 = t%z(r,1)
               zf = t%z(r,2)
               psi1(z0:zf)=CZERO
               do k=ki,kf ! loop over oper products
                  if(koeff(k)==0.d0) cycle
                  call hcall(t%m,k,work,psi2(z0:zf),subdim)

                  if(koeff(k)/=1.d0) work=work*koeff(k)
                  psi1(z0:zf)=psi1(z0:zf)+work
              enddo
           enddo

        else

           do r=1,navcs
               z0 = t%z(r,1)
               zf = t%z(r,2)
               psi1(z0:zf)=CZERO
               do k=ki,kf ! loop over oper products
                   if(koeff(k)==0.d0) cycle
                   hk => t%hterms(k,:)
                   work(:) = psi2(z0:zf)
                   call hprod_phi(hk,work)
                   if(koeff(k)/=1.d0) work=work*koeff(k)
                   psi1(z0:zf)=psi1(z0:zf)+work
               enddo
           enddo

        endif

        deallocate(work)

        if(t%istop) return ! Top A-vector is not orthogonalized

        r=0
        z0 = t%z(1,1)
        zf = t%z(navcs,2)
  30    call gramschmidt(psi1(z0:zf),subdim,navcs,ierr)
        if(ierr.gt.r) then
           r=ierr
           do i=t%z(r,1),t%z(r,2)
              call random_number(norm)
              psi1(i) = dcmplx(norm)
           enddo
           write(2,'(2a,i4,a,i4)') 'noniterative_node: orthogonalization fixed.',&
               ' Node number =', t%num,',  SPF =', r
           write(6,'(2a,i4,a,i4)') 'noniterative_node: orthogonalization fixed.',&
               ' Node number =', t%num,',  SPF =', r
           goto 30
        elseif(ierr.ne.0) then
           write(6,'(/a)') 'noniterative_node: Cannot orthogonalize!'
           write(6,'(a,i3,a,i3)') 'Node number =',t%num,', SPF =',ierr
           write(2,'(/a)') 'noniterative_node: Cannot orthogonalize!'
           write(2,'(a,i3,a,i3)') 'Node number =',t%num,', SPF =',ierr
           stop
        endif

        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    end subroutine noniterative_node


!------------------------------------------------------------------------
!  ML-operate, iterative procedure.  A = [d]*\tilde{A},  HDM 08/2020
!------------------------------------------------------------------------
    subroutine iterative_node(t,psi1,psi2,koeff,nham)
        implicit none
        type(wfnode_t),intent(inout) :: t
        type(hterm_tp),pointer       :: hk(:)
        complex(dop),pointer        :: mfparent(:,:)
        complex(dop)                :: psi1(:),psi2(:),koeff(:)
        complex(dop),allocatable    :: work(:),work1(:,:),work2(:,:)
        integer                      :: subdim,k,r,s,ierr,navcs,ns,i
        integer                      :: z0,zf,ki,kf
        integer, intent(in)          :: nham
        real(dop)                    :: norm

        ns     = t%nsubcoord
        navcs  = t%navecs
        subdim = product(t%ndim)
        
        allocate(work1(subdim,navcs))
        allocate(work2(subdim,navcs))
        work1 = CZERO
        work2 = CZERO
        ki = zham(nham)
        kf = zham(nham)+kzahl(nham)-1

!$OMP   parallel default(shared) private(k,mfparent,hk,r,z0,zf,work,s)
        allocate(work(subdim))

!$OMP   do reduction(+:work1,work2)
        do k=ki,kf ! loop over oper products
           if(koeff(k)==0.d0) cycle
           if(.not.t%istop) mfparent => t%parent%mfields(k,ns)%p%matrix
           hk => t%hterms(k,:)
           do r=1,navcs
              z0 = t%z(r,1)
              zf = t%z(r,2)
              if(t%isprimitive) then
                  call hcall(t%m,k,work,psi2(z0:zf),subdim)
               else
                  work(:) = psi2(z0:zf)
                  call hprod_phi(hk,work(:))
               endif
               if(koeff(k)/=1.d0) work(:)=work(:)*koeff(k)
               work1(:,r) = work1(:,r) + work(:)
               if(.not.t%istop) then
                  do s=1,navcs
                     work2(:,s) = work2(:,s) + mfparent(s,r)*work(:)
                  enddo
               endif
           enddo ! r=1,navcs
        enddo    ! k=ki,kf
!$OMP   end do
        deallocate(work)
!$OMP   end parallel


        if(t%istop) then
!$OMP   parallel do private(r,z0,zf)
           do r=1,navcs
              z0 = t%z(r,1)
              zf = t%z(r,2)
              psi1(z0:zf) = work1(:,r)
           enddo
!$OMP   end parallel do

           deallocate(work1)
           deallocate(work2)
           return   ! Top A-vector is not orthogonalized
        endif

!$OMP   parallel do private(r,z0,zf,norm)
        do r=1,navcs
           z0 = t%z(r,1)
           zf = t%z(r,2)
           psi1(z0:zf) = work2(:,r)
           norm=dble(dot_product(psi1(z0:zf),psi1(z0:zf)))
           if(norm.gt.1.d-18) then
              norm=1/sqrt(norm)
              psi1(z0:zf) = norm*psi1(z0:zf)
           else
              if(r.lt.navcs) then
                 psi1(z0:zf) = work1(:,r)
              else   ! this is for starting with a Hartree product
                 psi1(z0:zf) = psi2(t%z(1,1):t%z(1,2))
              endif
           endif
        enddo
!$OMP   end parallel do
        deallocate(work1)
        deallocate(work2)

        r=0
        z0 = t%z(1,1)
        zf = t%z(navcs,2)
 30     call gramschmidt(psi1(z0:zf),subdim,navcs,ierr)
        if(ierr.gt.r) then
           r=ierr
           do i=t%z(r,1),t%z(r,2)
              call random_number(norm)
              psi1(i) = dcmplx(norm)
           enddo
           write(2,'(2a,i4,a,i4)') 'iterative_node: orthogonalization fixed.',&
               ' Node number =', t%num,',  SPF =', r
           goto 30
        elseif(ierr.ne.0) then
           write(6,'(/a)') 'iterative_node: Cannot orthogonalize!'
           write(6,'(a,i3,a,i3)') 'Node number =',t%num,', ierr=',ierr
           write(2,'(/a)') 'iterative_node: Cannot orthogonalize!'
           write(2,'(a,i3,a,i3)') 'Node number =',t%num,', ierr=',ierr
           stop
        endif

        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    end subroutine iterative_node



    subroutine compute_dfields_node(t,psi1,psi2,nham)
    ! Eq.(36), Manthe,[JCP,128,164116,(2008)]
        implicit none
        type(wfnode_t),intent(inout) :: t
        complex(dop),intent(in)     :: psi1(:),psi2(:)

        type(hterm_tp),pointer       :: hk(:)
        complex(dop),pointer        :: mfparent(:,:),mfcurrent(:,:)
        integer                      :: k,f,p,r,ns
        complex(dop),allocatable    :: work(:)
        complex(dop),allocatable    :: work2(:,:)
        complex(dop),allocatable    :: mftmp(:,:)
        integer                      :: z0,zf,plen,ki,kf
        integer, intent(in)          :: nham 

        if(t%isprimitive) return

        if (t%tid_mfields == 0) &
            call init_node_timer(t, t%tid_mfields, 'dfields')
        call start_timer(t%tid_mfields)

        plen = product(t%ndim)
        ki = zham(nham)
        kf = zham(nham)+kzahl(nham)-1

        if(t%istop) then ! NORMAL MCTDH mean fields

        z0 = t%z(1,1)
        zf = t%z(1,2)

!$OMP   parallel default(shared) private(k,hk,f,mfcurrent,work)
        allocate(work(plen))
!$OMP   do schedule(runtime)
        do k=ki,kf ! loop over oper products
            hk => t%hterms(k,:)
            do f=1,t%nmodes ! loop over modes
                mfcurrent => t%mfields(k,f)%p%matrix
                work = psi2(z0:zf)
                call hprod_phi(hk,work,f)
                call compute_dmat(mfcurrent,f,t%ndim,psi1(z0:zf),work)
            end do
        end do
!$OMP   end do
        deallocate(work)
!$OMP   end parallel

        else ! MULTILAYER mean fields

        ns=t%nsubcoord
!$OMP   parallel default(shared) private(k,hk,mfparent,work2,r,z0,zf,p,f,mftmp,mfcurrent,work)
        allocate(work2(plen,t%navecs))
        allocate(work(plen))
!$OMP   do schedule(runtime)
        do k=ki,kf ! loop over oper products
            hk => t%hterms(k,:)
            mfparent => t%parent%mfields(k,ns)%p%matrix
            ! Apply parent MF to list of A-vectors.
            work2 = CZERO
            do r=1,t%navecs
                z0 = t%z(r,1)
                zf = t%z(r,2)
                do p=1,t%navecs
                    work2(:,p) = work2(:,p) + mfparent(p,r)*psi2(z0:zf)
                enddo
            enddo
            do f=1,t%nmodes ! loop over modes
                allocate(mftmp(t%ndim(f),t%ndim(f)))
                mfcurrent => t%mfields(k,f)%p%matrix
                mfcurrent = CZERO
                do r=1,t%navecs
                    z0 = t%z(r,1)
                    zf = t%z(r,2)
                    work(:) = work2(:,r)
                    call hprod_phi(hk, work, f)
                    call compute_dmat(mftmp, f, t%ndim, psi1(z0:zf), work)
                    mfcurrent = mfcurrent + mftmp
                enddo
                deallocate(mftmp)
            end do
        end do
!$OMP   end do
        deallocate(work)
        deallocate(work2)
!$OMP   end parallel

        end if
        call stop_timer(t%tid_mfields)

        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        if(debug) then
            do f=1,t%nmodes
                do k=ki,kf
                    if(t%isuncor(k)) cycle
                    mfcurrent => t%mfields(k,f)%p%matrix
                    write(6,*) 'mean-field: nodenum,f,k', t%num,f,k
                    do r=1,t%ndim(f)
                      write(6,10) (mfcurrent(r,p),p=1,t%ndim(f))
                    end do
                end do
            end do
        end if
        10 format(8(2x,'(',1PE11.4,1x,1PE11.4,')'))
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    end subroutine compute_dfields_node


end module dterms

