!***********************************************************************
! 
!             MCTDH subroutine EININWFMOD
!
! contains: eininwf,inwf_meigenf,einbuild,eb_init_state,eb_left_state,
!           eb_right_state,eb_temperature,eb_pack,eb_state,eb_pop,
!           eb_periodic,eb_leg,eb_ho,eb_gauss,eb_eigenf,eb_sphfbr,eb_kleg,
!           eb_k,eb_map,eb_wigner,eb_gwp,eb_hogwp,einreadwf,
!           einingeowf,rdebnumd
!
! Add new options to isbaspar and rsbaspar arrays. List all defined
! options for each initial spf type together at the end of the
! relevant keyword, so that it is easy to see what is what.
!   isbaspar(4,f,p) is reserved for pop
!----------------------------------------------------------------------
! basis(f) : pbasis type for dof f. (dof=degree of freedom)
! el,  HO, Leg, sin, FFT, exp, sphfbr, kleg,   k, pleg, rHO, Leg/r external
!  0    1    2    3    4    5      6     7     8    9    11   12     13
!  point, kleg/r, pleg/r
!   19     20     21
!  -1 is for blank basis (phifbr). (15,16,17,18) -> Lagu1-4
! Wigner
!   33
!-----------------------------------------------------------------------
! sbasis(f) : sbasis type for dof f. (dof=degree of freedom)
!(phifbr), el, HO,  gauss, Leg, eigenf, sphfbr, kleg,  k  map Wigner  
!   -1      0   1     1     2     3      4       5     6   7    8       
!  GWP/ho-gwp  Euclid  tdpoint  readspf  3deigf
!     9          10      11       12       30
!***********************************************************************

      module eininwfmod

use decimal, only: long, dop
use global
use maxv
use logdat
use keyunitsmod
use dvrdatmod
use einingeomod, only: einingeo
use chkinwfmod, only: chkinwf,chkbuild
use datenmod
use griddatmod
use psidef
use runinwfmod
use runopermod
use runpropmod
use operdef
use gwplib, only: gwp_distin
use channels


use op1lib, only: zerovxi,zerovxd,zeromxi,initvxz

      implicit none
      private
      public :: eininitwf
      contains

!***********************************************************************

      subroutine eininitwf

      implicit none

      integer(long)                              :: ic,iz,ierr
      logical(kind=4)                            :: linperr
      logical                                    :: liwfsec,ligeosec
      logical                                    :: check=.false.

! CHECK iin open
      inquire(iin,opened=check)
      if (check) close(iin)
      open(iin,file=inputfile(1:inplaenge),status='old',&
           form='formatted',err=900)

      rewind(iin)
      iz=0
      linperr=.false.
      liwfsec=.false.
      ligeosec=.false.

      call initwfdefaults

      readiin: do
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .gt. 0) then
            routine = 'Eingabe, initwf-section'
            message = inptit
            call errormsg
         else if (keyword(1) .eq. 'init_wf-section') then
            call eininwf(linperr,iz)
            liwfsec=.true.
         else if (keyword(1) .eq. 'initial-geometry-section') then
            call einingeo(linperr,iz,3)
            ligeosec=.true.
         else if (keyword(1) .eq. 'end-input') then
            if (liwfsec .or. ligeosec) then
               exit
            else
               routine = 'Eingabe, initwf-section'
               message='No initial wavefunction specified'
               call warnmsg
            endif
         endif
      enddo readiin

      if (liwfsec .or. ligeosec) call chkinwf(ligeosec)

!-----------------------------------------------------------------------
! fill in blanks in parameters needed for build
!-----------------------------------------------------------------------
      if (lbuildwf) call chkbuild

      return
  900 continue
      routine='Eininitwf (eininwfmod.f90)'
      message = 'Cannot open file: '//inputfile(1:inplaenge)
      call errormsg

      end subroutine eininitwf

!***********************************************************************

      subroutine eininwf(linperr,iz)

      implicit none

      integer(long)         :: i,iz0,ic,ierr,instate,j,n,p,m,lngth,&
                               symmode,symdof,nsym,ipersist,symcount,iacount,&
                               ralaenge,s,jop
      integer(long), intent(inout)   :: iz

      logical(kind=4), intent(inout)                  :: linperr
      logical(kind=4)                                 :: l100,lrd,lexist
      character(len=c5)                               :: raname,filename

!-----------------------------------------------------------------------
! do not read input if maxdim needs to be redimensioned
!-----------------------------------------------------------------------
      if (ndof .gt. maxdim) then
         routine    = 'read INIT_WF-SECTION  (eininwf)'
         write(message,'(a,i4,a,i4,a)') 'ndof =',ndof, &
         ' is larger than maxdim =', maxdim,'. Enlarge maxdim!' 
         call errormsg
      endif

!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
      nsym       = 0
      iz0        = 0
      jop        = 0
      instate    = 1
      ftr        = 1
      statetr    = 0
      symcount   = 0
      iorthowf   = 380
      iacount    = 0
      equalfile  = 0
      azahl      = 0

!-----------------------------------------------------------------------
! set wavefunction type according to known info
! 0 usual MCTDH, 
! 2 S-MCTDH (set by keyword) 
! 4 G-MCTDH 
! 
!
! GAW
!! vMCG calculations use a standard MCTDH form SUM_J A_J g_J
!! Use G-MCTDH 
!-----------------------------------------------------------------------
      if (lexact) psitype = 1
      if (lrealwp)psitype = 11
      if (ltraj)  psitype = 3
      if (lgwp) then
         if (nmode .eq. 1 .or. (nmode .eq. 2 .and. leb)) then
            lvmcg = .true.
            psitype = 4
         endif
      endif 

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'read INIT_WF-SECTION  (eininwf)'
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (keyword(1).eq.'end-init_wf-section' ) then
         goto 20

      elseif (keyword(i) .eq. 'realpsi')  then
         lrealpsi2=.true.

      elseif (keyword(i) .eq. 'file')  then
         if (lrestart) then
            routine = 'read INIT_WF-SECTION'
            write(message,'(a)') &
             'Restart file to read already specified before "file" keyword'
            call errormsg
         endif

         lrestart = .true.
         if (keyword(i+1) .eq. '=') then
            i = i+2
            rname = keyorig(i)
            rlaenge = lc(i)
            call abspath(rname,rlaenge)

            filename=rname(1:rlaenge)//'/restart'
            inquire(file=filename,exist=lexist)

! In optimal control calculation, restart files (e.g. for initial and
! target states may be relative to the directory above. Try this.
            if (.not. lexist) then
               rname = '../'//keyorig(i)
               rlaenge = index(rname,' ')-1
               call abspath(rname,rlaenge)

               filename=rname(1:rlaenge)//'/restart'
               inquire(file=filename,exist=lexist)
            endif

! If restart file not found, stop.
            if (.not. lexist) then
               routine = 'read INIT_WF-SECTION'
               write(message,'(2a)') 'restart file does not exist : ', &
                    filename(1:rlaenge+8)
               call errormsg
            endif

            readwffile(1) = filename
            nreadwf = 1

  70        if (keyword(i+1) .eq. ',') then
               i = i+2
               if (keyword(i) .eq. 'orthopsi') then
                  lorthopsi = .true.
               elseif (keyword(i) .eq. 'noorthopsi') then
                  lorthopsi = .false.
               elseif (keyword(i) .eq. 'ignore') then
                  lignore = .true.
               elseif (keyword(i) .eq. 'realpsi') then
                  lrealpsi = .true.
               elseif (keyword(i) .eq. 'g-mctdh') then
                  psitype = 4
               elseif (keyword(i) .eq. 'normalise') then
                  lrstnorm = .true.
               elseif (keyword(i) .eq. 'nonormalise') then
                  lrstnorm = .false.
               elseif (keyword(i) .eq. 'incoherent') then
                  lrstcoh = .false.
               else
                  routine='Eininwf'
                  message = &
                  'Expecting "orthopsi", "noorthopsi", "realpsi" or '//  &
                  '"ignore", not '
                  call wrline(i,iz,iz0)
               endif
            endif
            if (keyword(i+1) .eq. ',') goto 70
         endif

      elseif (keyword(i) .eq. 'select') then
         if(.not.lrestart) then
            routine='eininwf;  geninwf/eininwf'
            message=' "select" must come after "file=..."'
            call wrline(i,iz,iz0)
         endif
         s=1
         m=1
         j=0
         allocate(selectspf(maxdim,maxdim,maxsta))
         call zeromxi(selectspf,maxdim*maxdim,maxsta)
 17      if (keyword(i+1)(1:1) .eq. 's') then
            read(keyword(i+1)(2:),*,iostat=ierr) s
            if (ierr.ne.0 .or. s.le.0 .or. s.gt.nstate) then
               routine='Eininwf'
               message = 'A positive integer between 1 and nstate '//&
                         'must follow "s". Not :'
               call wrline(i+1,iz,iz0)
            endif
         elseif (keyword(i+1)(1:1) .eq. 'm') then
            read(keyword(i+1)(2:),*,iostat=ierr) m
            if (ierr.ne.0 .or. m.le.0 .or. m.gt.nmode) then
               routine='eininwf'
               message = 'A positive integer between 1 and nmode '//&
                         'must follow "m". Not :'
               call wrline(i+1,iz,iz0)
            endif
         elseif (keyword(i+1)(1:1) .eq. ',') then
            continue
         else
            j=j+1
            if (j .gt. maxreadspf) then
               routine='Eininwf'
               message=&
            'No. of SPfs read must not be larger than maxreadspf in selectspf.   '//&
            '      If necessary increase this number in runinwfmod.f90'
               call errormsg
            endif
            read(keyword(i+1),*,iostat=ierr) selectspf(j,m,s)
            if (ierr.ne.0) then
               routine='Eininwf'
               message = 'Error reading numbers for select :'
               call wrline(i+1,iz,iz0)
            endif
         endif
         if(i+1.lt.ic) then
            i=i+1
            goto 17
         endif
         if(j.ne.dim(m,s)) then
            routine='Eininwf'
            message='There must be exactly dim(m,s) integers in select'
            call wrline(0,iz,iz0)
         endif

      elseif (keyword(i) .eq. 'block-spf' .or.&
              keyword(i) .eq. 'block_spf')  then
         lblockspf = .true.
         if (keyword(i+1) .eq. '=') then
            i = i+2
            rname = keyorig(i)
            rlaenge = lc(i)
            call abspath(rname,rlaenge)
  72        if (keyword(i+1) .eq. ',') then
               i = i+2
               if (keyword(i) .eq. 'orthopsi') then
                  lorthopsi = .true.
               elseif (keyword(i) .eq. 'noorthopsi') then
                  lorthopsi = .false.
               elseif (keyword(i) .eq. 'realpsi') then
                  lrealpsi = .true.
               else
                  routine='eininwf;  geninwf/eininwfmod.f90 '
                  message =&
                  'Expecting "orthopsi", "noorthopsi", "realpsi", not :'
                  call wrline(i,iz,iz0)
               endif
            endif
            if (keyword(i+1) .eq. ',') goto 72
         else
            routine='eininwf;  geninwf/eininwfmod.f90'
            message='Missing "=" after "block-spf" '
            call wrline(0,iz,iz0)
         endif

      elseif (keyword(i) .eq. 'block-a' .or.&
              keyword(i) .eq. 'block_a')  then
         lblocka = .true.
         if (keyword(i+1) .eq. '=') then
 73         i = i+2
            raname = keyorig(i)
            ralaenge = lc(i)
            iacount=iacount+1
            call abspath(raname,ralaenge)

            readwffile(iacount) = raname
            if (keyword(i+1) .eq. ',') goto 73
         else
            routine='eininwf;  geninwf/eininwfmod.f90 '
            message='Missing "=" after "block-A" '
            call wrline(0,iz,iz0)
         endif

      else if (keyword(1) .eq. 'select_state') then
         if(.not.lrestart) then
            routine='eininwf;  geninwf/eininwf'
            message=' "select_state" must come after "file=..."'
            call wrline(i,iz,iz0)
         endif
         if (keyword(i+1) .eq. '=') then
            i = i+2
            read(keyword(i),*,iostat=ierr) s
            if (ierr.ne.0 .or. s.le.0 .or. s.gt.gdim(feb)) then
               routine='eininwf'
               message = 'A positive integer between 1 and nstate '//&
                         'must follow "select_state". Not :'
               call wrline(i+1,iz,iz0)
            endif
            amapwf(1,1,1) = 1
            amapwf(1,2,1) = s
            
         else
            routine='eininwf;  geninwf/eininwfmod.f90 '
            message='Missing "=" after "select_state" '
            call wrline(0,iz,iz0)
         endif

      elseif (keyword(i) .eq. 'noorthopsi') then
         lorthopsi = .false.

      else if( keyword(1) .eq. 'read-inwf' ) then
         if (lrestart) then
            routine = 'read INIT_WF-SECTION'
            write(message,'(a)') &
             'Restart file to read already specified before "read-inwf" block'
            call errormsg
         endif

         call einreadwf(iz,iz0)  ! lrestart set in einreadwf if file specified

      else if( keyword(1) .eq. 'build' ) then
         lbuildwf = .true.
         l100=.true.
  100    continue
         call einbuild(sbfound,iz,iz0,instate,l100)
         if(l100) goto 100

         if (keyword(1) .eq. 'end-build' ) then
!........Set sbasis for (single-set) packet dof
            if (fpb.gt.0) then
               if (sbasis(fpb).eq.999) then
                  sbasis(fpb) = 0
                  isbaspar(4,fpb,1) = 1
                  sbfound(fpb,1)=.true.
               endif
            endif
!........Set sbasis in case there was no 'init_state' keyword
            if (feb.gt.0) then
               if (sbasis(feb).eq.999) then
                  sbasis(feb) = 0
                  isbaspar(4,feb,1) = 1
                  sbfound(feb,1)=.true.
               endif
            endif
            goto 10
         else if (keyword(1) .eq. 'end-input' .or. &
                 keyword(1) .eq. 'end-init_wf-section') then
            routine='geninwf/eininwfmod.f90'
            message = 'No end-build keyword at end of build'
            call warnmsg
            linperr=lwarn.or.linperr
            lwarn=.false.
            RETURN
         else 
            routine='eininwf, einbuild (END Problem)'
            message = inptit
            call errormsg
         endif

      else if (keyword(1) .eq. 'gwpopit') then
         lgwpop0 = .true.
         if (keyword(2) .eq. '=') then
            read(keyword(3),*,iostat=ierr) gwpopit
            if (ierr .ne. 0) then
               routine='geninwf/eininwfmod.f90'
               message = 'Incorrect argument for keyword : '
               call wrline(onei,iz,iz0)
            endif
         endif

      else if( keyword(1) .eq. 'autoblock' ) then
         lautoblk=.true.

      else if( keyword(1) .eq. 'a-coeff' )  then
         do
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            routine='geninwf/eininwfmod.f90'
            message = inptit
            call cntrl(keyword(1),lc(1),ierr)
            
            if (keyword(1) .eq. 'end-a-coeff'   )  exit
            if (keyword(1) .eq. 'end-init_wf-section'   .or. &
                keyword(1) .eq. 'end-input' )    then
               routine='geninwf/eininwfmod.f90'
               message = ' "end-A-coeff" is missing '
               call wrline(zeroi,iz,iz0)
               linperr=lwarn.or.linperr
               lwarn=.false.
               return
            end if

            azahl = azahl + 1
         enddo

      else if( keyword(i) .eq. 'correction' ) then
         if (lexact) then
            routine='geninwf/eininwfmod.f90'
            message='Correction not implemented for exact propagation.'
            call errormsg
         else if (dentype .ne. 0) then
            routine='geninwf/eininwfmod.f90'
            message='Correction not implemented for densities.'
            call errormsg
         endif         
         ladiab = .true.
         if (keyword(i+1) .eq. '=' ) then
            i = i + 2
            if (keyword(i) .eq. 'edstr') then
               ledstr= .true.
            elseif (keyword(i) .eq. 'ad'   ) then
               lad   = .true.
            elseif (keyword(i) .eq. 'dia'  ) then
                ldia  = .true.
            elseif (keyword(i)(1:3) .eq. 'hh2'  ) then
               lhh2  = .true.
               if (keyword(i)(4:9) .eq. '-bkmp2') lbkmp2 = .true.
            else
               routine='eininwf;  geninwf/eininwf.F line 322'
               message = 'Unknown keyword :'
               call wrline(i,iz,iz0)
               linperr=lwarn.or.linperr
            endif
            if (keyword(i+1) .eq. ','    ) then
               i = i + 2
               if (keyword(i) .eq. 'edstr') then
                  ledstr= .true.
               elseif (keyword(i) .eq. 'ad'   ) then
                  lad   = .true.
               elseif (keyword(i) .eq. 'dia'  ) then
                   ldia  = .true.
               elseif (keyword(i)(1:3) .eq. 'hh2'  ) then
                  lhh2  = .true.
                  if (keyword(i)(4:9) .eq. '-bkmp2') lbkmp2 = .true.
               else
                  routine='eininwf;  geninwf/eininwf.F '
                  message = 'Unknown keyword :'
                  call wrline(i,iz,iz0)
                  linperr=lwarn.or.linperr
               endif
            endif
         end if
      elseif (keyword(i).eq.'tfac') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after tfac '
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) tfac
            if (ierr.ne.0) then
               routine='geninwf/eininwfmod.f90'
               message &
                    ='A real number must be argument of tfac. Not :'
               call wrline(i+2,iz,iz0)
            endif
         endif
         i=i+2

! Left for compatibility. Use G-MCTDH keyword in run-section
      else if (keyword(i)(1:9) .eq. 'gwp_nomix') then
         if (lgwp) psitype = 4

      else if (keyword(i)(1:9) .eq. 'gwp_plain') then
         if (lgwp) psitype = -4

      else if (keyword(i)(1:9) .eq. 's-mctdh') then
         psitype = 2
         citype = 1
         if ( keyword(i+1) .eq. '=' ) then
            i = i + 2
            read(keyword(i),*,iostat=ierr) cicut
            cicut=cicut+tiny
            if (ierr .ne. 0) then
               routine='geninwf/eininwfmod.f90'
               message = &
            'expecting a number as first argument for ci-select'
                  call wrline(zeroi,iz,iz0)
            endif
            if (cicut .lt. 1.0) then
               cicut=1.0_dop
               routine='geninwf/eininwfmod.f90'
               message='cicut < 1. Assuming cicut=1.0 '
               call warnmsg
            endif
            if (cicut .ge. nmode) then
               psitype = 0
               citype = 0
               cicut=nmode
               write (ilog,'(a)')  &
                   'cicut > nmode. Assuming normal MCTDH '
            endif

         else
            cicut=2.0
            citype=1
            write(ilog,'(a)') &
          'No arguments for s-mctdh. Assuming "singles-doubles" CI'
         endif

      elseif (keyword(i).eq.'trans') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after translation '
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) ftr
            if (ierr.ne.0) then
               routine='geninwf/eininwfmod.f90'
               message &
                    ='A integer must be argument of trans. Not :'
               call wrline(i+2,iz,iz0)
            endif
            i=i+2
            if (keyword(i+1).eq.',') then
               read(keyword(i+2),*,iostat=ierr) statetr
               if (ierr.ne.0) then
                  routine='geninwf/eininwfmod.f90'
                  message &
                    ='A integer must be 2. argument of trans. Not :'
                  call wrline(i+2,iz,iz0)
               endif
            endif
         endif
         i=i+2

      elseif (keyword(i).eq.'meigenf') then
         call inwf_meigenf(iz,iz0,i)

      elseif (keyword(i).eq.'operate') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after operate '
            call warnmsg
         else
 60         jop=jop+1
            i=i+2
            do n=1,hamdim
               if (keyorig(i) .eq. hamlab(n)) inwfop(jop)=n
            enddo
            if (inwfop(jop) .eq. 0) then
               routine='geninwf/eininwfmod.f90'
               message='Hamiltonian required by operate not defined :'
               call wrline(i,iz,iz0)
            endif
            if (ic.ge.i+1 .and. keyorig(i+1) .eq. ',' ) then
               if( jop .eq. 16 ) then
                  routine='geninwf/eininwfmod.f90'
                  message= &
                  'Not more than 15 operators allowed for operate'
                  call wrline(i+2,iz,iz0)
               else
                  goto 60
               endif
            endif
         endif

      elseif (keyword(i).eq.'operate_iter') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after operate_iter '
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) inwfopnit
            if (ierr.ne.0) then
               routine='geninwf/eininwfmod.f90'
               message &
                    ='A integer must be argument of operate_init. Not :'
               call wrline(i+2,iz,iz0)
            endif
         endif
         i=i+2

      elseif (keyword(i).eq.'operate_tol') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after operate_tol '
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) inwfoptol
            if (ierr.ne.0) then
               routine='geninwf/eininwfmod.f90'
               message &
                    ='A integer must be argument of operate_tol. Not :'
               call wrline(i+2,iz,iz0)
            endif
         i=i+2
         endif

      elseif (keyword(i).eq.'operate_no-norm') then
         lnonorm=.true.

      elseif (keyword(i).eq.'operate_ortho') then
         loportho=.true.

      elseif (keyword(i).eq.'operate_no-direct') then
         lnodirect = .true.
      elseif (keyword(i).eq.'operate_direct') then
         lnodirect = .false.

      elseif (keyword(i)(1:8) .eq.'orthogon') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after orthogonalise '
            call warnmsg
         else
   31       iorthowf = iorthowf + 1
            inptit=keyorig(i+2)
            lngth=lc(i+2)
            call abspath(inptit,lngth)
            if(rlaenge.eq.lngth) then
               if(rname(1:rlaenge).eq.inptit(1:lngth)) then
                  equalfile=iorthowf
                  write(ilog,'(2a,/2a)') 'Using open channel "irst"',&
                             ' for "orthogonalise" ',&
                      'file: ',inptit(1:lngth)
                  goto 37
               endif
            elseif(rlaenge+8.eq.lngth) then
               if(rname(1:rlaenge)//'/restart'.eq.inptit(1:lngth)) then
                  equalfile=iorthowf
                  write(ilog,'(2a,/2a)') 'Using open channel "irst"',&
                             ' for "orthogonalise" ',&
                      'file: ',inptit(1:lngth)
                  goto 37
               endif
            elseif(rlaenge.eq.lngth+8) then
               if(rname(1:rlaenge).eq.inptit(1:lngth)//'/restart') then
                  equalfile=iorthowf
                  write(ilog,'(2a,/2a)') 'Using open channel "irst"',&
                             ' for "orthogonalise" ',&
                      'file: ',inptit(1:lngth)//'/restart'
                  goto 37
               endif
            endif
            inptit=inptit(1:lngth)//'/restart' 
            lngth=lngth+8         
            open(iorthowf,file=inptit(1:lngth),form='unformatted', &
                 status='old',err=34)
            goto 36 
   34       lngth=lngth-8 ! Long form failed, try not to append 'restart'       
            open(iorthowf,file=inptit(1:lngth),form='unformatted', &
                 status='old',err=35)
   36       write(ilog,'(a,i4,a,/2a)') 'Opened channel', iorthowf, &
                             ' for "orthogonalise" ', &
                      'file: ',inptit(1:lngth)
   37       i=i+2
            if(keyword(i+1) .eq. ',' ) goto 31
         endif

      elseif (keyword(i).eq.'symorb') then
         lsymorb=.true.
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after symorb'
            call wrline(i,iz,iz0)
         else
            read(keyword(i+2),*,iostat=ierr) symorb1
            if (ierr.ne.0 .or. symorb1.le.0 .or. symorb1.gt.nmode) then
               routine='geninwf/eininwfmod.f90'
               message = 'A positive integer number must be '// &
                         'argument of symorb. Not :'
               call wrline(i+2,iz,iz0)
            endif
         endif
         i=i+2
         if (keyword(i+1).ne.',') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "," after first argument of symorb'
            call wrline(i,iz,iz0)
         else
            read(keyword(i+2),*,iostat=ierr) symorb2
            if (ierr.ne.0 .or. symorb2.le.0 .or. symorb2.gt.nmode) then
               routine='geninwf/eininwfmod.f90'
               message = 'A positive integer number must be '// &
                         'argument of symorb. Not :'
               call wrline(i+2,iz,iz0)
            endif
         endif
         i=i+2

      elseif (keyword(i).eq.'symcoeff') then
         symcount=symcount+1
         nsym=nsym+1
         msymmtr(nsym) = 51200
         lrd=.false.
         if(keyword(i+1) .eq. '=') then
  71        if(keyword(i+2)(1:4) .eq. 'pers')  then
               i=i+2
               msymmtr(nsym) = msymmtr(nsym) + 65536
            elseif(keyword(i+2) .eq. 'dav')  then
               i=i+2
               msymmtr(nsym) = msymmtr(nsym) + 2*65536
            else
               i=i+2
               read(keyword(i),*,iostat=ierr) j
               if (j.le.0 .or. j.gt.nmode) then
                  routine='geninwf/eininwfmod.f90'
                  message='A number given as argument to symcoeff'//&
                          ' must be gt 0 and le nmode but not : '
                  call wrline(i,iz,iz0)
               else
                  if(symcv(j).ne.0) then
                  write(6,'(a,20i3)') 'symcv :',(symcv(j),j=1,nmode)
                  routine='geninwf/eininwfmod.f90'
                  message='A particle number (mode number) must'//&
                          ' not be given twice for (a)symcoeff : '
                  call wrline(i,iz,iz0)
                  endif
                  symcv(j)=symcount
                  lrd=.true.
               endif
               if (ierr.ne.0) then
                  routine='eininwf'
                  message='Expecting "persist" or "dav" or a number'//&
                          ' as argument to symcoeff but not : '
                  call wrline(i,iz,iz0)
               endif
            endif
            if(keyword(i+1) .eq. ',') goto 71
         endif
         if(.not.lrd) then ! if no number is given, set all particles.
            if(symcount .ne. 1) then
               write(6,'(a,20i3)') 'symcv :',(symcv(j),j=1,nmode)
               routine='eininwf'
               message='A particle number (mode number) must'//&
                       ' not be given twice for (a)symcoeff : '
               call wrline(i,iz,iz0)
            endif
            do j=1,nmode
               symcv(j)=1
            enddo
         endif
         if(lrd .and. msymmtr(nsym).ge.51200+2*65536) then
            routine='eininwf'
            message='Partial symmetrisation is not implemented for dav'
            call warnmsg
         endif

      elseif (keyword(i).eq.'sym1d') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after sym1d '
            call wrline(i,iz,iz0)
         else
   81       read(keyword(i+2),*,iostat=ierr) symdof
            nsym=nsym+1
            if (ierr.ne.0) then
               routine='geninwf/eininwfmod.f90'
               message = 'An integer number or "persist" '//&
                         'must be argument of sym1d. Not :'
               call wrline(i+2,iz,iz0)
            endif
            i=i+2
            ipersist=0
            if(keyword(i+2) .eq. 'persist')  then
               i=i+2
               ipersist=65536
            endif
            symmode=dofspf(symdof)
            msymmtr(nsym) = 256*symmode + symdof + ipersist
            if (keyword(i+1).eq.',') goto 81
         endif

      elseif (keyword(i).eq.'parity') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after parity'
            call wrline(i,iz,iz0)
         else
   91       read(keyword(i+2),*,iostat=ierr) symdof
            nsym=nsym+1
            if (ierr.ne.0) then
               routine='geninwf/eininwfmod.f90'
               message = 'An integer number or "persist" '//&
                         'must be argument of parity. Not :'
               call wrline(i+2,iz,iz0)
            endif
            i=i+2
            ipersist=0
            if(keyword(i+2) .eq. 'persist')  then
               i=i+2
               ipersist=65536
            endif
            symmode=dofspf(symdof)
            msymmtr(nsym) = 256*(symmode+100) + symdof + ipersist
            if (keyword(i+1).eq.',') goto 91
         endif

      elseif (keyword(i).eq.'asymcoeff') then
         symcount=symcount+1
         nsym=nsym+1
         msymmtr(nsym) = -51200
         lrd=.false.
         if(keyword(i+1) .eq. '=') then
   61       if(keyword(i+2)(1:4) .eq. 'pers')  then
               i=i+2
               msymmtr(nsym) = msymmtr(nsym) - 65536
            elseif(keyword(i+2) .eq. 'dav')  then
               i=i+2
               msymmtr(nsym) = msymmtr(nsym) - 2*65536
            else
               i=i+2
               read(keyword(i),*,iostat=ierr) j
               if(j.le.0 .or. j.gt.nmode) then
                  routine='geninwf/eininwfmod.f90'
                  message='A number given as argument to asymcoeff'//&
                          ' must be gt 0 and le nmode but not : '
                  call wrline(i,iz,iz0)
               else
                  if(symcv(j).ne.0) then
                  write(6,'(a,20i3)') 'symcv :',(symcv(j),j=1,nmode)
                  routine='geninwf/eininwfmod.f90'
                  message='A particle number (mode number) must'//&
                          ' not be given twice for (a)symcoeff: '
                  call wrline(i,iz,iz0)
                  endif
                  symcv(j)=symcount
                  lrd=.true.
               endif
               if (ierr.ne.0) then
                  routine='geninwf/eininwfmod.f90'
                  message='Expecting "persist" or "dav" or a number'//&
                          ' as argument to asymcoeff but not : '
                  call wrline(i,iz,iz0)
               endif
            endif
            if(keyword(i+1) .eq. ',') goto 61
         endif
         if(.not.lrd) then      ! if no number is given, set all particles.
            if(symcount .ne. 1) then
               write(6,'(a,20i3)') 'symcv :',(symcv(j),j=1,nmode)
               routine='eininwf'
               message='A particle number (mode number) must'//&
                       ' not be given twice for (a)symcoeff : '
                call wrline(i,iz,iz0)
            endif
            do j=1,nmode
               symcv(j)=1
            enddo
         endif
         if(lrd .and. -msymmtr(nsym).ge.51200+2*65536) then
            routine='eininwf'
            message='Partial asymmetrisation is not implemented for dav'
            call warnmsg
         endif

      elseif (keyword(i).eq.'asym1d') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after asym1d '
            call wrline(i,iz,iz0)
         else
   82       read(keyword(i+2),*,iostat=ierr) symdof
            nsym=nsym+1
            if (ierr.ne.0) then
               routine='geninwf/eininwfmod.f90'
               message = 'An integer number or "persist" '//&
                         'must be argument of asym1d. Not :'
               call wrline(i+2,iz,iz0)
            endif
            i=i+2
            ipersist=0
            if(keyword(i+2) .eq. 'persist')  then
               i=i+2
               ipersist=65536
            endif
            symmode=dofspf(symdof)
            msymmtr(nsym) = -(256*symmode + symdof + ipersist)
            if (keyword(i+1).eq.',') goto 82
         endif

      elseif (keyword(i).eq.'sym2d') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after sym2d '
            call wrline(i,iz,iz0)
         else
   83       read(keyword(i+2),*,iostat=ierr) symmode
            nsym=nsym+1
            if (ierr.ne.0) then
               routine='geninwf/eininwfmod.f90'
               message = 'An integer number or "persist" '//&
                         'must be argument of sym2d. Not :'
               call wrline(i+2,iz,iz0)
            endif
            i=i+2
            ipersist=0
            if(keyword(i+2) .eq. 'persist')  then
               i=i+2
               ipersist=65536
            endif
            msymmtr(nsym) = 256*symmode + ipersist
            if (keyword(i+1).eq.',') goto 83
         endif

      elseif (keyword(i).eq.'asym2d') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after asym2d '
            call wrline(i,iz,iz0)
         else
   84       read(keyword(i+2),*,iostat=ierr) symmode
            nsym=nsym+1
            if (ierr.ne.0) then
               routine='geninwf/eininwfmod.f90'
               message = 'An integer number or "persist" '//&
                         'must be argument of asym2d. Not :'
               call wrline(i+2,iz,iz0)
            endif
            i=i+2
            ipersist=0
            if(keyword(i+2) .eq. 'persist')  then
               i=i+2
               ipersist=65536
            endif
            msymmtr(nsym) = -256*symmode-ipersist
            if (keyword(i+1).eq.',') goto 84
         endif

      elseif (keyword(i).eq.'sym3d') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after sym3d '
            call wrline(i,iz,iz0)
         else
   85       read(keyword(i+2),*,iostat=ierr) symmode
            nsym=nsym+1
            if (ierr.ne.0) then
               routine='geninwf/eininwfmod.f90'
               message = 'An integer number or "persist" '//&
                         'must be argument of sym3d. Not :'
               call wrline(i+2,iz,iz0)
            endif
            i=i+2
            ipersist=0
            if(keyword(i+2) .eq. 'persist')  then
               i=i+2
               ipersist=65536
            endif
            msymmtr(nsym) = 256*symmode+200+ipersist
            if (keyword(i+1).eq.',') goto 85
         endif

      else if (keyword(i) .eq. 'wigner') then
         if ( keyword(i+1) .eq. '=' ) then
            i = i + 2
            if (keyword(i) .eq. 'sym') then
               pqdistr=2
            else if (keyword(i) .eq. 'nosym') then
               pqdistr=1
            else if (keyword(i) .eq. 'off') then
               pqdistr=0
            else
               routine='geninwf/eininwfmod.f90'
               message =  'Unknown Wigner distribution type'
               call warnmsg
            endif

         else
            pqdistr=1

         endif

      elseif (keyword(i).eq.'sym2kleg') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message = 'missing "=" after sym2kleg'
            call wrline(i,iz,iz0)
         else
   86       read(keyword(i+2),*,iostat=ierr) symmode
            nsym=nsym+1
            if (ierr.ne.0) then
               routine='geninwf/eininwfmod.f90'
               message = 'The mode number must be argument of '//&
                         '"sym2kleg", not:'
               call wrline(i+2,iz,iz0)
            endif
            i=i+2
            ipersist=0
!           persist for sym2kleg not implemented at the moment
            msymmtr(nsym) = 256*symmode+201+ipersist
            if (keyword(i+1).eq.',') goto 86
         endif

      elseif (keyword(i).eq.'asym2kleg') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message = 'missing "=" after asym2kleg'
            call wrline(i,iz,iz0)
         else
   87       read(keyword(i+2),*,iostat=ierr) symmode
            nsym=nsym+1
            if (ierr.ne.0) then
               routine='geninwf/eininwfmod.f90'
               message = 'The mode number must be argument of '//&
                         '"asym2kleg", not:'
               call wrline(i+2,iz,iz0)
            endif
            i=i+2
            ipersist=0
!           persist for asym2kleg not implemented at the moment
            msymmtr(nsym) = -(256*symmode+201+ipersist)
            if (keyword(i+1).eq.',') goto 87
         endif
! symmetrization and antisymmetrizaion with "nsym2d" keyword
      elseif (keyword(i).eq.'nsym2d') then
         if (keyword(i+1).ne.'=') then
            routine='geninwf/eininwfmod.f90'
            message =  'missing "=" after nsym2d '
            call wrline(i,iz,iz0)
         else
   88       read(keyword(i+2),*,iostat=ierr) symmode
            nsym=nsym+1
            if (ierr.ne.0) then
               routine='geninwf/eininwfmod.f90'
               message = 'An integer number or "persist" '//&
                         'must be argument of nsym2d. Not :'
               call wrline(i+2,iz,iz0)
            endif
            i=i+2
            ipersist=0
            if(keyword(i+2) .eq. 'persist')  then
               i=i+2
               ipersist=65536
            endif
            msymmtr(nsym) = 256*symmode+202+ipersist
            if (keyword(i+1).eq.',') goto 88
         endif

      else if (keyword(i) .eq. 'no-redundancy-check') then
         lredundant = .false.

      else if (keyword(i) .eq. 'redundancy-check') then
         lredundant = .true.

! error if unknown keyword
      else
         routine='geninwf/eininwfmod.f90'
         message = ' unknown keyword : '
         call wrline(i,iz,iz0)

         if (keyword(i+1) .eq. '=') then
            i=i+2
            do
               if (keyword(i+1) .eq. ',') then
                  i=i+2
               else
                  exit
               endif
            enddo
         endif

      end if

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

   20 continue
               
!-----------------------------------------------------------------------
! Set feb sbasis in case there was no 'init_state' or 'select_state' keyword
!-----------------------------------------------------------------------
       if (feb.gt.0) then
          if (sbasis(feb).eq.999) then
             sbasis(feb) = 0
             isbaspar(4,feb,1) = 1
             sbfound(feb,1)=.true.
          endif
       endif

!-----------------------------------------------------------------------
! check consistency 
!-----------------------------------------------------------------------
!      call chkinwf  !chkinwf is called in eininitwf
      if(lblocka  .and. iacount.ne.npackts) then
         routine='eininwf'
         message = 'block-A : The number of files specified must equal '&
                   //'exactly the number of packets!'
         call errormsg
      endif

!-----------------------------------------------------------------------
! Turn off G-MCTDH if no GWPs
!-----------------------------------------------------------------------
      if (psitype .eq. 4 .and. .not. lgwp) psitype = 0

!-----------------------------------------------------------------------
! GWPs and exact wavefunction cannot be Schmidt-orthonormalised
!-----------------------------------------------------------------------
      if (lgwp .or. lexact .or. lrealwp) lorthopsi=.false.

!-----------------------------------------------------------------------
! if G-MCTDH wavefunction, check Nspf = Ngwp
!-----------------------------------------------------------------------
      if (psitype .eq. 4) then
         do m=1,nmode
            if (.not. gwpm(m)) cycle

            do s=1,nstate
               if (gwpdim(m,s) .ne. dim(m,s)) then
                  routine='eininwf'
                  message = &
     'Cannot specify different nos. of SPFs and GWPs for a G-MCTDH calculation'
                  call errormsg
               endif
                  
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! rho-mctdh(II) with gwps must by psitype=0
!-----------------------------------------------------------------------
      if (dentype .eq. 2 .and. lgwp) then
         if (psitype .eq. 4) then
            routine='eininwf'
            write(message,'(2a)') &
             ' Density matrices cannot be propagated with G-MCTDH form.',&
             ' Remove g-mctdh keyword'
            call errormsg
         endif
      endif

!-----------------------------------------------------------------------
! if 1ML-vMCG calculation set no. of SPFs = no. of states
! vMCG uses psitype = 4
!-----------------------------------------------------------------------
!     if (lvmcg .and. psitype .eq. 0) then
!        if (leb .and. dim(1,1) .ne. nddstate) then
!           dim(1,1) = nddstate
!           routine = 'Eininitwf (eininwfmod.F90)'
!           message = &
!   'Adjusting no. of SPFs in a 1ML-vMCG calculation to equal the no. of states'
!           call warnmsg1
!        else if (lmult) then
!           ladjust = .false.
!           do s=1,nstate
!              if (dim(1,s) .ne. 1) then
!                 ladjust = .true.
!                 dim(1,s) = 1
!              endif
!           enddo
!           if (ladjust) then
!              routine = 'Eininitwf (eininwfmod.F90)'
!              message = &
!   'Only need 1 SPF for each state in a 1ML-vMCG calculation'
!              call warnmsg1
!           endif
!        endif
!     endif

!-----------------------------------------------------------------------
! For a multi-packet run, parameters for packets needed.
!-----------------------------------------------------------------------
      if (dentype.eq.0 .and. lmulpack .and. lmult) then
         do p = nddstate+1,nstate
            do m = 1,nmode
               dim(m,p) = dim(m,mod(p-nddstate-1,nddstate)+1)
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! Check correct number of trajectories in swarm calculation
!-----------------------------------------------------------------------
      if (ltraj .and. pqdistr .eq. 2) then
         n = ntraj - 1
         n = mod(n,4)
         if (n .ne. 0) then
            routine = 'Eininitwf (eininwfmod.F90)'
            message(1:79)= &
        'For symmetrical Wigner selection of trajectories swarm = 4n+1'
            message(80:120)= &
        'Change swarm or select wigner = nosym'
            call errormsg
         endif
      endif

      if (statetr .eq. 0 ) statetr = instate

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true. Stop now if error in init_wf-section.
!-----------------------------------------------------------------------
      if (lwarn) then
         write(ilog,'(a)') '****** Error in INIT_WF-SECTION. STOP ******'
         write(6,'(a)') '****** Error in INIT_WF-SECTION. STOP ******'
         stop
      endif

      return

 35   routine='eininwf'
      message='Cannot open '//inptit(1:lngth)//'                     '&
      //'     Possible problem: FORTRAN cannot open a file twice.   '&
      //'                      Also try "ls '//inptit(1:lngth)//'" .'
      call errormsg

      call errormsg 
      end subroutine eininwf


!-----------------------------------------------------------------------


      subroutine inwf_meigenf(iz,iz0,i)

! Reading "meigenf" of the INIT_WF-SECTION
 
      implicit none

      integer(long)                :: ierr,n
      integer(long), intent(in)    :: iz
      integer(long), intent(inout) :: i,iz0

      medim=0
      menum=0
      meham=0
      meselect=0
      mewrite=0
      meran=0
      routine = 'inwf_meigenf;  geninwf/eininwf.F'
      if (keyword(i+1).ne.'=') then
         routine='Inwf_meigen (Eininwf) '
         message =  'missing "=" after meigenf'
         call warnmsg
      else
         read(keyword(i+2),*,iostat=ierr) memode
         if (ierr.ne.0 .or. memode .le. 0) then
            routine='Inwf_meigen (Eininwf) '
            message = 'A pos. int. (mode) must be ' &
                 //'1st argument of meigenf. Not :'
            call wrline(i+2,iz,iz0)
         endif
         if (memode .gt. nmode) then
            routine='Inwf_meigen (Eininwf) '
            message = 'Illegal mode (.gt.nmode) '
            call wrline(zeroi,iz,iz0)
         endif
         if (keyword(i+3).ne.',') then
            routine='Inwf_meigen (Eininwf) '
            message='Expecting "," not :'
            call wrline(i+3,iz,iz0)
         endif
         do n=1,hamdim
            if (keyorig(i+4) .eq. hamlab(n)) meham=n
         enddo
         if (meham .eq. 0) then
            routine='Inwf_meigen (Eininwf) '
            message='Hamiltonian required by meigenf not defined :'
            call wrline(i+4,iz,iz0)
         endif
         i=i+4
         if (keyword(i+1).eq.',') then
            if (keyword(i+2).eq.'follow') then
               menum = 920
               i=i+2
            else
               read(keyword(i+2),*,iostat=ierr) menum
               if(ierr.ne.0 .or. menum.lt.0 .or. menum.gt.900) then
                  routine='Inwf_meigen (Eininwf) '
                  message = '"follow" or integer argument .ge.0'// &
                       ' and .le.900 required as 3rd arg. Not: '
                  call wrline(i+2,iz,iz0)
               endif
               i=i+2
            endif
            if (menum.ge.0 .and. keyword(i+1).eq.',') then
               if (keyword(i+2).eq.'full') then
                  menum = menum + 10000
               elseif(keyword(i+2).eq.'select') then
                  meselect=1
               elseif(keyword(i+2).eq.'noselect') then
                  meselect=-1
               elseif(keyword(i+2).eq.'write') then
                  mewrite=1
               else
                  read(keyword(i+2),*,iostat=ierr) medim
                  if (ierr.ne.0 .or. medim.le.0) then
                     routine='Inwf_meigen (Eininwf) '
                     message='full, write, (no)select or integer' &
                       //' .gt.0 required as 4th argument. Not: '
                     call wrline(i+2,iz,iz0)
                  endif
               endif
               i=i+2
               if (keyword(i+1).eq.',') then
                  if (keyword(i+2).eq.'full') then
                     menum = menum + 10000
                  elseif(keyword(i+2).eq.'select') then
                     meselect=1
                  elseif(keyword(i+2).eq.'noselect') then
                     meselect=-1
                  elseif(keyword(i+2).eq.'write') then
                     mewrite=1
                  else
                     read(keyword(i+2),*,iostat=ierr) medim
                     if (ierr.ne.0 .or. medim.le.0) then
                        routine='Inwf_meigen (Eininwf) '
                        message='"full, (no)select, write or integer' &
                           //' .gt.0 required as 5th argument. Not: '
                        call wrline(i+2,iz,iz0)
                     endif
                  endif
                  i=i+2
                  if (keyword(i+1).eq.',') then
                     if (keyword(i+2).eq.'full') then
                        menum = menum + 10000
                     elseif(keyword(i+2).eq.'select') then
                        meselect=1
                     elseif(keyword(i+2).eq.'noselect') then
                        meselect=-1
                     elseif(keyword(i+2).eq.'write') then
                        mewrite=1
                     else
                        read(keyword(i+2),*,iostat=ierr) medim
                        if (ierr.ne.0 .or. medim.le.0) then
                           routine='Inwf_meigen (Eininwf) '
                           message='full,(no)select,write or integer' &
                           //' .gt.0 required as 6th argument. Not: '
                           call wrline(i+2,iz,iz0)
                        endif
                     endif
                     i=i+2
                     if (keyword(i+1).eq.',') then
                        if (keyword(i+2).eq.'full') then
                           menum = menum + 10000
                        elseif(keyword(i+2).eq.'select') then
                           meselect=1
                        elseif(keyword(i+2).eq.'noselect') then
                           meselect=-1
                        elseif(keyword(i+2).eq.'write') then
                           mewrite=1
                        elseif(keyword(i+2).eq.'ran') then
                           if (keyword(i+3).eq.'=') then
                              read(keyword(i+4),*,iostat=ierr) meran
                              i=i+3
                           else
                              meran=gdim(memode)-1
                           endif
                        else
                           read(keyword(i+2),*,iostat=ierr) medim
                           if (ierr.ne.0 .or. medim.le.0) then
                              routine='Inwf_meigen (Eininwf) '
                              message='full,(no)select,write or integer' &
                              //' .gt.0 required as 7th argument. Not: '
                              call wrline(i+2,iz,iz0)
                           endif
                        endif
                        i=i+2
                        if (keyword(i+1).eq.',') then
                           routine='Inwf_meigen (Eininwf) '
                           message='There should be no 8th argument'
                           call wrline(zeroi,iz,iz0)
                        endif
                     endif
                  endif
               endif
            endif
         endif
         if(medim.eq.0)  medim = subdim(memode)
         if(medim.gt.subdim(memode)) then
            write(6,'(a,i3,a,/a,/a,i4,a,i4)') &
                 '## WARNING meigenf   mode =',memode,' ##', &
                 ' dimension must not be larger than subdim.', &
                 ' medim =',medim,'  set to subdim =',subdim(memode)
            medim = subdim(memode)
         endif
      endif
      mez = mez+1
      if (mez.gt.maxme) then
         routine='Inwf_meigen (Eininwf) '
         message='Too many meigenf. Increase maxme.'
         call errormsg
      endif
      if(meselect.eq.0) then    ! Setting default values for meselect
         if(mod(menum/10000,2).eq.1 .and. &
              medim.eq.subdim(memode)) then
            meselect=-1
         else
            meselect=1
         endif
      endif
      mearray(mez,1) = memode
      mearray(mez,2) = meham
      mearray(mez,3) = menum
      mearray(mez,4) = medim
      mearray(mez,5) = meselect
      mearray(mez,6) = mewrite
      mearray(mez,7) = meran
      
      return
      end subroutine inwf_meigenf
 
!-----------------------------------------------------------------------


      subroutine einbuild(found,iz,iz0,instate,l100)

!-----------------------------------------------------------------------
! Reading the "build" part of the INIT_WF-SECTION
!-----------------------------------------------------------------------

      implicit none

      integer(long)                :: i,ic,f,f1,ierr,p,parainc,lunit,&
                                      f1m,f1mm,febcount
      integer(long), intent(in)    :: iz
      integer(long), intent(out)   :: instate
      integer(long), intent(inout) :: iz0

      logical(kind=4), intent(inout) :: l100
      logical(kind=4)                                        :: exitflag
      logical(kind=4), dimension(maxdim,maxsta), intent(out) :: found


      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine='einbuild (eininwfmod.f90)'
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)
      f1=0
            
      if (keyword(1)(1:4) .eq. 'end-' ) then
         l100=.false.
         return
      endif
      
!-----------------------------------------------------------------------
! Search for "a-coeff", "meigenf", "operate"
!-----------------------------------------------------------------------
      if (keyword(1) .eq. 'a-coeff') then
         routine='einbuild (eininwfmod.f90)'
         message='The A-coeff block must not be within the build-block'
         call errormsg
      endif
      if (keyword(1) .eq. 'meigenf') then
         routine='einbuild (eininwfmod.f90)'
         message='meigenf must not be within the build-block'
         call errormsg
      endif
      if (keyword(1)(1:7) .eq. 'operate') then
         routine='einbuild (eininwfmod.f90)'
         message='operate must not be within the build-block'
         call errormsg
      endif
      
!-----------------------------------------------------------------------
! Search for "print"
!-----------------------------------------------------------------------
      if (keyword(1) .eq. 'print') then
         lprintmode=.true.
         return
      endif

!-----------------------------------------------------------------------
! Search for "pack" keyword that specifies the wavepacket
!-----------------------------------------------------------------------
      parainc = 0
      p = 1
      call eb_pack(f1,parainc,p,ic,iz,iz0,exitflag)
      if (exitflag) return
      
!-----------------------------------------------------------------------
! Search for "init_state"
!-----------------------------------------------------------------------
      if (keyword(1) .eq. 'init_state') then
         febcount=0
         do f=1,ndof
            if(basis(f).eq.0) febcount=febcount+1
         enddo
         if(febcount.gt.1) then
            write(message,'(a,i3,a,9x,2a)') 'There are',febcount, &
             ' electronic bases defined.','Use "euclid" rather than', &
             ' "init_state" if there is more than one el-basis.'
            routine='einbuild on eininwf.F'
            call errormsg
         endif
         call eb_init_state(found,instate,p,iz,iz0)
         return
      endif
      
!-----------------------------------------------------------------------
! Search for "all_ran" for thermalised relaxation
!-----------------------------------------------------------------------
      if (keyword(1) .eq. 'all_ran') then
         dimfran(:)=gdim(:)-1
         return
      endif

!-----------------------------------------------------------------------
! For density operators search for: *_state" and "temperature"
!-----------------------------------------------------------------------
      if (keyword(1) .eq. 'left_state') then
         call eb_left_state(found,iz,iz0)
         return
      endif

      if (keyword(1) .eq. 'right_state') then
         call eb_right_state(iz,iz0)
         return
      endif

      if (keyword(1)(1:11) .eq. 'temperature') then
         call eb_temperature(iz,iz0)
         if (keyword(1)(12:17) .eq. '_readf') ltemp_mfsc = .true.
         return
      endif

!-----------------------------------------------------------------------
! For density operators type I may need a real initial psi
!-----------------------------------------------------------------------
      if (keyword(1) .eq. 'realpsi') then
         lrealpsi = .true.
         return
      endif

      f1=0
      do f = 1, ndof
         if(keyorig(1) .eq. modelabel(f) ) f1=f
      enddo
      if (f1.eq.0) then
         routine='einbuild (eininwfmod.f90)'
         message='unknown keyword or mode-label in build block : '
         call wrline(1,iz,iz0)
         return
      endif

!-----------------------------------------------------------------------
! lunit=0 length units for input
!       1 energy units for input (for continuum electronic basis)
!-----------------------------------------------------------------------
      if ((f1 .eq. febcont) .or. (f1 .eq. feb .and. febcont .gt. 0)) then
         f1 = febcont
         lunit=1
      else
         lunit=0
      endif

!-----------------------------------------------------------------------
! Search for "state" keyword that specifies the state
! Must be made after search for pack keyword
!-----------------------------------------------------------------------
      call eb_state(f1,parainc,p,ic,iz,iz0,exitflag)
      if (exitflag) go to 100

!-----------------------------------------------------------------------
! flag that basis set has been found for f1,p
!-----------------------------------------------------------------------
      found(f1,p)=.true.

!-----------------------------------------------------------------------
! Search for "pop" keyword that specifies the populated spf
! NB isbaspar(4,f,p) is reserved for pop.
! NB isbaspar(6,f,p) is used for "sym" and "check".
!-----------------------------------------------------------------------
      isbaspar(4,f1,p) = 1
      isbaspar(6,f1,p) = 0
      call eb_pop(f1,parainc,p,ic,iz,iz0)

!-----------------------------------------------------------------------
! Search for "periodic" keyword that specifies that the grid is
! interpreted as being periodic.
!-----------------------------------------------------------------------

!not for pergauss
      isbaspar(5,f1,p) = 0
      if (basis(f1) .ne. 25 .and. sbasis(f1) .ne. 9) then
         call eb_periodic(f1,parainc,p,ic,iz,iz0)
      endif

!-----------------------------------------------------------------------
! Search for "ran" keyword that specifies that the DOF will be
! thermalized
!-----------------------------------------------------------------------
      call eb_ran(f1,parainc,ic,iz,iz0)

!-----------------------------------------------------------------------
! Run over sbasis-types: Leg, HO, Gauss, eigenf, KLeg, K, map, Wigner
!-----------------------------------------------------------------------
      if (keyword(2).eq.'discrete') then
         sbasis(f1) = 0

      else if (keyword(2).eq.'leg') then
         call eb_leg(f1,p,iz,iz0)

      elseif (keyword(2).eq.'ho') then
         call eb_ho(f1,parainc,p,ic,iz,iz0)
         
      elseif (keyword(2).eq.'gauss') then
         call eb_gauss(f1,p,iz,iz0,lunit)

      elseif (keyword(2).eq.'eigenf') then
         call eb_eigenf(f1,p,iz,iz0)
   
      elseif (keyword(2).eq.'sphfbr') then
         call eb_sphfbr(f1,p,iz,iz0)

      elseif (keyword(2).eq.'phifbr') then
         call eb_phifbr(f1,p,iz,iz0)

      elseif (keyword(2).eq.'kleg') then
         call eb_kleg(f1,parainc,p,ic,iz,iz0)

      elseif (keyword(2).eq.'k') then
         call eb_k(f1,parainc,p,ic,iz,iz0)

      elseif (keyword(2).eq.'map') then
         call eb_map(f1,p,ic,iz,iz0)

      elseif (keyword(2).eq.'wigner') then
         call eb_Wigner(f1,parainc,p,ic,iz,iz0)

      elseif (keyword(2).eq.'gwp') then
         call eb_gwp(f1,p,iz,iz0,ic,parainc,0)

      elseif (keyword(2).eq.'gwpp') then
         call eb_gwp(f1,p,iz,iz0,ic,parainc,1)

      elseif (keyword(2).eq.'ho-gwp') then
         call eb_hogwp(f1,p,iz,iz0,0)

      elseif (keyword(2).eq.'ho-gwpp') then
         call eb_hogwp(f1,p,iz,iz0,1)

      elseif (keyword(2)(1:4).eq.'eucl' .or. keyword(2)(1:4).eq.'eukl') then
         call eb_euclid(f1,p,iz,iz0)

      elseif (keyword(2).eq.'tdpoint') then
         call eb_tdpoint(f1,p)

      elseif (keyword(2).eq.'readspf') then
         call eb_readspf(f1,p,ic,iz,iz0)

      elseif (keyword(2).eq.'3deigf') then 
         sbasis(f1) = 30
         lbuildrestart = .true.
         i = 2
         if (keyword(i+1) .eq. '=') then
            i = i+2
            rname = keyorig(i)
            rlaenge = lc(i)
            call abspath(rname,rlaenge)
  70        if (keyword(i+1) .eq. ',') then
               i = i+2
               if (keyword(i) .eq. 'orthopsi') then
                  lorthopsi = .true.
               elseif (keyword(i) .eq. 'noorthopsi') then
                  lorthopsi = .false.
               elseif (keyword(i) .eq. 'ignore') then
                  lignore = .true.
               elseif (keyword(i) .eq. 'realpsi') then
                  lrealpsi = .true.
               else
                  routine='einbuild (eininwfmod.f90)'
                  message = &
                  'Expecting "orthopsi", "noorthopsi", "realpsi" or '// &
                  '"ignore", not '
                  call wrline(i,iz,iz0)
               endif
            endif
            if (keyword(i+1) .eq. ',') goto 70
         endif
         return
      else
         routine='einbuild (eininwfmod.f90)'
         message='Unknown initial wavepacket type'
         call wrline(zeroi,iz,iz0)    
      endif

 100  l100 =.true.

      if(lrlxt .and. f1.gt.0) then
         if(dimfran(f1).gt.0) then
            f1m=max(1,f1-1)
            f1mm=max(1,f1-2)
            if( (sbasis(f1).le.0) .or. (sbasis(f1).ge.4 .and. sbasis(f1) &
                 .le.9) .or. (basis(f1).eq.7) .or. (basis(f1).eq.8) &
                 .or.(basis(f1).eq.9) .or. (basis(f1).eq.19)  .or. &
                     (basis(f1m).eq.7) .or. (basis(f1m).eq.9) .or. &
                     (basis(f1m).eq.19) .or. (basis(f1mm).eq.19) ) then  
               write(6,'(a,i5,a,i4,a,i4)')' f =',f1,',  sbasis(f) =', &
                             sbasis(f1),',  basis(f) =',basis(f1)
               routine='einbuid on eininwf.F'
               message='Thermalization with keyword ran is only '// &
               'allowed for       build-types: HO, gauss, Leg, '// &
               'eigenf, meigenf, and euclid.        '// &
               'And also not for the DVRs: KLeg, PLeg, and Wigner.'
               call errormsg
            endif
         else
            dimfran(f1)=0
         endif
      endif

      end subroutine einbuild


! ======================================================================
!     Subroutines for ein_build.
!     factored out of the originally monolithic ein_build routine
!     by FO, 11/2004
! ======================================================================

      subroutine eb_init_state(found,instate,p,iz,iz0)

      implicit none

      integer(long)                 :: ierr
      integer(long), intent(in)     :: p,iz
      integer(long), intent(out)    :: instate
      integer(long), intent(inout)  :: iz0

      logical(kind=4), dimension(maxdim,maxsta), intent(out) :: found

      if (keyword(2) .ne. '=') then
         routine='eb_init_state (eininwf_util.F)'
         message='No arguments specified with "init_state" keyword'
         call wrline(zeroi,iz,iz0)
      else
         read(keyword(3),*,iostat=ierr) instate
         if (ierr .ne. 0) then
            routine='eb_init_state (eininwf_util.F)'
            message = 'Incorrect argument for keyword : '
            call wrline(onei,iz,iz0)
         endif
         if(feb.gt.0 .and. instate.gt.gdim(feb)) then
            routine='eb_init_state (eininwf_util.F line 57)'
            message='init_state must not be larger than '//&
                    'the number of electronic states!'
            call wrline(0,iz,iz0)
         endif
      endif
      if (feb .gt. 0) then
         sbasis(feb) = 0
         isbaspar(4,feb,p) = instate
         found(feb,p)=.true.
      endif

      return
      end subroutine eb_init_state! of eb_init_state

! ----------------------------------------------------------------------

      subroutine eb_left_state(found,iz,iz0)

      implicit none

      integer(long)                :: ierr
      integer(long), intent(in)    :: iz
      integer(long), intent(inout) :: iz0

      logical(kind=4), dimension(maxdim,maxsta), intent(out) :: found

      if (keyword(2) .ne. '=') then
         routine='eb_pack (eininwf_util.F)'
         message='No arguments specified with "left_state" keyword'
         call wrline(zeroi,iz,iz0)
      else
         read(keyword(3),*,iostat=ierr) dlstate
         if (ierr .ne. 0) then
            routine='eb_pack (eininwf_util.F)'
            message = 'Incorrect argument for keyword : '
            call wrline(onei,iz,iz0)
         endif
      endif
      if (feb .gt. 0) then
         sbasis(feb) = 0
         isbaspar(4,feb,1) = dlstate
         found(feb,1)=.true.
      endif
      return
      end subroutine eb_left_state! of eb_left_state

! ----------------------------------------------------------------------

      subroutine eb_right_state(iz,iz0)

      implicit none

      integer(long)                :: ierr
      integer(long), intent(in)    :: iz
      integer(long), intent(inout) :: iz0

      if (keyword(2) .ne. '=') then
         routine='eb_right_state (eininwf_util.F)'
         message = 'No arguments specified with "right_state" keyword'
         call wrline(zeroi,iz,iz0)
      else
         read(keyword(3),*,iostat=ierr) drstate
         if (ierr .ne. 0) then
            routine='eb_right_state (eininwf_util.F)'
            message = 'Incorrect argument for keyword : '
            call wrline(onei,iz,iz0)
         endif
      endif
      return
      end subroutine eb_right_state! of eb_right_state

! ----------------------------------------------------------------------

      subroutine eb_temperature(iz,iz0)

      implicit none

      integer(long)                :: ierr
      integer(long), intent(in)    :: iz
      integer(long), intent(inout) :: iz0

      if (temperature0 .ge. 0.0_dop) then
         routine='eb_temperature (eininwf_util.F)'
         message= &
            'Temperature already set by relaxation times. Remove keyword from build section'
         call errormsg
      endif

      if (keyword(2) .ne. '=') then
         routine='eb_temperature (eininwf_util.F)'
         message='No arguments specified with "temperature" keyword'
         call wrline(zeroi,iz,iz0)
      else if (keyword(3) .eq. "infinity") then
         temperature0 = 99999999.0_dop
      else
         read(keyword(3),*,iostat=ierr) temperature0
         if (ierr .ne. 0) then
            routine='eb_temperature (eininwf_util.F)'
            message = 'Incorrect argument for keyword : '
            call wrline(onei,iz,iz0)
         endif
         if(keyword(4) .eq. ',') then
            call keyeunits(temperature0,keyword(5),ierr)
            if (ierr .ne. 0) then
               routine='eb_temperature (eininwf_util.F)'
               message = 'Incorrect unit for temperature : '
               call wrline(onei,iz,iz0)
            endif
         endif
      endif

      return
      end subroutine eb_temperature! of eb_temperature

! ----------------------------------------------------------------------

      subroutine eb_pack(f1,parainc,p,ic,iz,iz0,exitflag)

      implicit none

      integer(long), intent(in)    :: f1,ic,iz
      integer(long), intent(inout) :: parainc,p,iz0
      integer(long)                :: i,ierr,pack
      logical(kind=4), intent(out) :: exitflag

      do i = 1,ic-2
         if (keyword(i) .eq. 'pack') then
            if (dentype .ne. 0) then
               routine='eb_pack (eininwf_util.F)'
               message='Pack not implemented for densities.'
               call errormsg
            endif
            
            if (keyword(i+1) .ne. '=') then
               routine='eb_pack (eininwf_util.F)'
               message='No arguments specified with "pack" keyword'
               call wrline(zeroi,iz,iz0)
            else
               read(keyword(i+2),*,iostat=ierr) pack
               if ((ierr .ne. 0) .or. (pack .lt. 1)) then
                  routine='eb_pack (eininwf_util.F)'
                  message='Illegal or no packet specified for mode ' &
                       //modelabel(f1)
                  call warnmsg
               endif
               if (pack .gt. npacket) then
                  exitflag = .true.
                  goto 99
               endif
               p = nddstate * (pack-1) + 1
               parainc = parainc+3
            endif
         endif
      enddo
      exitflag = .false.
 99   return
      end subroutine eb_pack! of eb_pack

! ----------------------------------------------------------------------

      subroutine eb_state(f1,parainc,p,ic,iz,iz0,exitflag)

      implicit none

      integer(long)                 :: state,i,ierr
      integer(long), intent(in)     :: f1,ic,iz
      integer(long), intent(inout)  :: parainc,p,iz0
      logical(kind=4), intent(out)  :: exitflag

      state = 1
      do i = 1,ic-2
         if (keyword(i) .eq. 'state') then
            if (keyword(i+1) .ne. '=') then
               routine = 'eb_state (eininwf_util.F)'
               message = 'No arguments specified with "state" keyword'
               call wrline(zeroi,iz,iz0)
            else
               read(keyword(i+2),*,iostat=ierr) state
               if ((ierr .ne. 0) .or. (state .lt. 1)) then
                  routine = 'eb_state (eininwf_util.F)'
                  message = 'Illegal or no state specified for mode ' &
                       //modelabel(f1)
                  call warnmsg
               endif
               if (state .gt. nddstate) then
                  exitflag = .true.
                  goto 99
               endif
               p = p + state - 1
               parainc = parainc + 3
            endif
         endif
      enddo
      exitflag = .false.
 99   return
      end subroutine eb_state! of eb_state

! ----------------------------------------------------------------------

      subroutine eb_pop(f1,parainc,p,ic,iz,iz0)

      implicit none

      integer(long)                 :: i,isym,ierr,m,pop
      integer(long), intent(in)     :: iz,f1,p,ic
      integer(long), intent(inout)  :: parainc,iz0

      isym = 0
      do i = 1,ic-2
         if (keyword(i) .eq. 'pop') then
            if (dentype .eq. 1) then
               routine = 'eb_pop (eininwf_util.F)'
               message='Pop not implemented for density1.'
               call errormsg
            endif
            if (keyword(i+1) .ne. '=') then
               routine = 'eb_pop (eininwf_util.F)'
               message = 'No arguments specified with "pop" keyword'
               call wrline(zeroi,iz,iz0)
            else
               m=dofspf(f1)
               read(keyword(i+2),*,iostat=ierr) pop
               if ((ierr .ne. 0) .or. (pop .le. 0) .or. &
                    (pop .gt. gdim(f1)) ) then
                  routine = 'eb_pop (eininwf_util.F)'
                  message = &
                       'Illegal or no population specified for mode ' &
                       //modelabel(f1)
                  call wrline(zeroi,iz,iz0)
               endif
               parainc = parainc+3
            endif
            if(keyword(i+3) .eq. ',') then
               if(keyword(i+4) .eq. 'sym') then
                  isym = 1
                  parainc = parainc+2
               else
                  routine = 'eb_pop (eininwf_util.F)'
                  message = 'Expecting "sym" but not :'
                  call wrline(i+4,iz,iz0)
               endif
            endif
            if(keyword(i+5) .eq. ',') then
               if(keyword(i+6) .eq. 'check') then
                  isym = isym + 10
                  parainc = parainc+2
               else
                  routine = 'eb_pop (eininwf_util.F)'
                  message = 'Expecting "check" but not :'
                  call wrline(i+6,iz,iz0)
               endif
            endif
            if( isym .eq. 10 ) then
               routine = 'eb_pop (eininwf_util.F)'
               message = &
                    ' "check" is allowed only if "sym" is given first.'
               call wrline(zeroi,iz,iz0)
            endif
            isbaspar(4,f1,p) = pop
            isbaspar(6,f1,p) = isym
         endif
      enddo
      return
      end subroutine eb_pop! of eb_pop

! ----------------------------------------------------------------------

      subroutine eb_periodic(f1,parainc,p,ic,iz,iz0)

      implicit none

      integer(long)                 :: i
      integer(long), intent(in)     :: iz,f1,p,ic
      integer(long), intent(inout)  :: iz0,parainc

      do i = 1,ic
         if (keyword(i) .eq. 'periodic') then
            if (basis(f1).ne.4 .and. basis(f1).ne.5) then
! Not for gwp : .lpergwp. carries the periodicity for gaussians !pergauss
               routine = 'eb_periodic (eininwf_util.F)'
               message='"periodic" allowed only for FFT or exp.'
               call wrline(zeroi,iz,iz0)
            endif
            isbaspar(5,f1,p) = 1
            parainc          = parainc+1
         endif
      enddo
      return
      end subroutine eb_periodic! of eb_periodic

! ----------------------------------------------------------------------

      subroutine eb_ran(f1,parainc,ic,iz,iz0)
!     DMT 06/2018
      implicit none

!     For reading and processing the input
      integer f1,parainc,ic,iz,iz0,i,ierr

      do i = 1,ic
         if (keyword(i) .eq. 'ran') then
            if (keyword(i+1) .ne. '=') then
!              Default settings
               dimfran(f1)=gdim(f1)-1
               parainc = parainc + 1
            else
!              User settings
               read(keyword(i+2),*,iostat=ierr) dimfran(f1)
               if ((ierr .ne. 0) .or. (dimfran(f1) .le. 0) .or.&
                    (dimfran(f1) .gt. gdim(f1)) ) then
                  routine = 'eb_ran (eininwf_util.F)'
                  message = 'Illegal or no ran specified for mode '&
                       //modelabel(f1)
                  call wrline(zeroi,iz,iz0)
               endif
               parainc = parainc + 3
            endif
         else if (keyword(i) .eq. 'no_ran') then
            dimfran(f1)=0
            parainc = parainc + 1
         endif
      enddo

      return
      end subroutine eb_ran! of eb_ran

!-----------------------------------------------------------------------
!       Leg
! isbaspar(1)=slz
! isbaspar(2)=sl0
! isbaspar(3)=isleg
! isbaspar(4)=sl0-slz+1        if(isleg.eq.0), no. of in. pop. spf
! isbaspar(4)=(sl0-slz)/2 + 1  if(isleg.gt.0), no. of in. pop. spf
!-----------------------------------------------------------------------

      subroutine eb_leg(f1,p,iz,iz0)

      implicit none

      integer(long)                 :: ierr,ierr1,slz,sl0,n,isleg
      integer(long), intent(in)     :: iz,f1,p
      integer(long), intent(inout)  :: iz0

      sbasis(f1) = 2
      ierr1=0

      if (keyword(3)(1:3) .eq. 'jbf') then
         slz = -9999
         do n=1,npar
            if (apar(n) .eq. keyorig(3)) then
               slz=nint(rpar(n))
            endif
         enddo
         if( slz .eq. -9999 ) then
            routine='eb_leg (eininwf_util.F)'
            message='jbf is not set! Use e.g. "alter-parameters".'
            call wrline(zeroi,iz,iz0)
            slz = 0
         endif

      else if(keyword(3) .eq. 'blz') then
         slz = ipbaspar(1,f1)

      else if(keyword(3)(1:3) .eq. 'slz') then
         slz = -9999
         do n=1,npar
            if (apar(n) .eq. keyorig(3)) then
               slz=nint(rpar(n))
            endif
         enddo
         if( slz .eq. -9999 ) then
            routine='eb_leg (eininwf_util.F)'
            message='slz is not set! Use e.g. "alter-parameters".'
            call wrline(zeroi,iz,iz0)
            slz = 0
         endif

      else
         read(keyword(3),*,iostat=ierr) slz
         if (ierr .ne. 0) ierr1=ierr
      endif

      if (keyword(4)(1:3) .eq. 'sl0') then
         sl0 = -9999
         do n=1,npar
            if (apar(n) .eq. keyorig(3)) then
               sl0=nint(rpar(n))
            endif
         enddo
         if( sl0 .eq. -9999 ) then
            routine='eb_leg (eininwf_util.F)'
            message='sl0 is not set! Use e.g. "alter-parameters".'
            call wrline(zeroi,iz,iz0)
            sl0 = 0
         endif

      else
         read(keyword(4),*,iostat=ierr) sl0
         if (ierr .ne. 0) ierr1=ierr
      endif

      if(keyword(5) .eq. 'sym' ) then
         isleg = 2-mod(sl0,2)
      elseif( keyword(5) .eq. 'nosym'  .or.  &
              keyword(5) .eq. 'no_sym' .or. &
              keyword(5) .eq. 'no-sym') then
         isleg = 0
      else
         read(keyword(5),*,iostat=ierr) isleg
         if (ierr .ne. 0) then
            ierr1=ierr
            routine='eb_leg (eininwf_util.F)'
            message='expecting "sym" or "no-sym" and not: ' &
                 //keyword(5)(1:32)
            call wrline(zeroi,iz,iz0)
         endif
      endif

      if (isleg.lt.0 .or. isleg.gt.2) then
         ierr1   = 1
         routine='eb_leg (eininwf_util.F)'
         message = 'isleg must be 0, 1, or 2'
         call wrline(zeroi,iz,iz0)
      endif
      if (isleg.ne.0 .and. mod(sl0-isleg,2).ne.0) then
         ierr1   = 1
         routine='eb_leg (eininwf_util.F)'
         message = 'sl0 is not consistent with isleg'
         write(6,*) 'slz =', slz,' , isleg =', isleg
         call wrline(zeroi,iz,iz0)
      endif
      if ( sl0 .lt. isleg*(2-isleg) ) then
         ierr1   = 1
         routine='eb_leg (eininwf_util.F)'
         message = 'sl0 is negative or not consistent with isleg'
         write(6,*) 'slz =', slz,' , sl0 =', isleg
         call wrline(zeroi,iz,iz0)
      endif
      if ( slz.lt.0  .or. slz.gt.sl0 ) then
         ierr1   = 1
         write(6,*) 'slz =', slz,' , sl0 =', sl0
         routine='eb_leg (eininwf_util.F)'
         message = 'slz is negative or greater than sl0'
         call wrline(zeroi,iz,iz0)
      endif
         
      if (ierr1.ne.0) then
         routine='eb_leg (eininwf_util.F)'
         message = 'Incorrect build definition for mode :'
         call wrline(onei,iz,iz0)
      endif
         
      isbaspar(1,f1,p)=slz
      isbaspar(2,f1,p)=sl0
      isbaspar(3,f1,p)=isleg
! isbaspar(4) set to no. of initially populated spf. Replaces pop.
      if (isleg.eq.0) then
         isbaspar(4,f1,p)=sl0-slz+1
      else
         isbaspar(4,f1,p)=(sl0-slz)/2 + 1
      endif

      return
      end subroutine eb_leg! of eb_leg

!------------------------------------------------------------------
!         HO
! isbaspar(1)=typ  (0,1,2 = all,odd,even)
! rsbaspar(1)=gxeq
! rsbaspar(2)=gmom
! rsbaspar(3)=rwidth
! rsbaspar(4)=cwidth
! rsbaspar(5)=gmass
!------------------------------------------------------------------

      subroutine eb_ho(f1,parainc,p,ic,iz,iz0)

      implicit none

      integer(long)                 :: ierr,ierr1,i,typ,i0
      integer(long), intent(in)     :: iz,f1,parainc,p,ic
      integer(long), intent(inout)  :: iz0

      real(dop)          :: rfreq,cfreq,gxeq,gmom,rwidth,cwidth,gmass
      complex(dop)       :: z
      logical lfound,lfound1

      rfreq   = 0.0_dop
      cfreq   = 0.0_dop
      sbasis(f1) = 1
      ierr1=0
      i = 3
      typ = 0
      if (keyword(i).eq.'odd') then
         typ = 1
         i=i+1
      else if (keyword(i).eq.'even') then
         typ = 2
         i=i+1
      endif
         
      read(keyword(i),*,iostat=ierr) gxeq
      if (ierr .ne. 0) ierr1=ierr
      i = i+1
      if(keyword(i) .eq. ',') then
         i = i+1
         call keylunits(gxeq,keyword(i),ierr)
         if (ierr .ne. 0) ierr1=ierr
         i = i+1
      endif
      read(keyword(i),*,iostat=ierr) gmom
      if (ierr .ne. 0) ierr1=ierr
      i = i+1
      if(keyword(i) .eq. ',') then
         i = i+1
         call keypunits(gmom,keyword(i),ierr)
         if (ierr .ne. 0) ierr1=ierr
         i = i+1
      endif
      i0=i
      call find_parameter(keyorig(i),rfreq,lfound)
      i = i+1
      if (keyword(i) .eq. ',' ) then
         i = i+1
         call find_parameter(keyorig(i),cfreq,lfound1)
         i = i+1
         if(lfound.neqv.lfound1) then
            message='If cfreq is given, rfreq and cfreq must be '// &
                    'both numbers or both parameters.'
           routine='eb_ho on eininwf_util.F'
            call errormsg
         endif
      endif
      if(.not.lfound) then
         i=i0
         read(keyword(i),*,iostat=ierr) rfreq
         if (ierr .ne. 0) ierr1=ierr
         i = i+1
         if (keyword(i) .eq. ',' ) then
            i = i+1
            call keyeunits(rfreq,keyword(i),ierr)
            if(ierr.gt.0) read(keyword(i),*,iostat=ierr) cfreq
            ierr1=ierr
            i = i+1
         endif
         if (keyword(i) .eq. ',' ) then
            i = i+1
            call keyeunits(rfreq,keyword(i),ierr)
            call keyeunits(cfreq,keyword(i),ierr)
            if (ierr .ne. 0) ierr1=ierr
            i = i+1
         endif
      endif
      gmass = 1.d0
      if (i .le. ic-parainc) then
         call find_parameter(keyorig(i),gmass,lfound)
         if(lfound) then
            i=i+1
         else
            read(keyword(i),*,iostat=ierr) gmass
            if (ierr.ne.0) ierr1=ierr
            i = i+1
            if(keyword(i) .eq. ',') then
               i = i+1
               call keymunits(gmass,keyword(i),ierr)
               if (ierr .ne. 0) ierr1=ierr
            endif
         endif
         rfreq = rfreq*gmass
         cfreq = cfreq*gmass
      end if

         
      if (ierr1.ne.0) then
         routine='eb_ho (eininwf_util.F)'
         message = 'Incorrect build definition for mode :'
         call wrline(onei,iz,iz0)               
      endif
      
!.... Transform to the convention used in genphi1.f for both HO and Gauss.
! sigma^2 = 1/rwidth
      z = dcmplx(rfreq,cfreq)
      z = sqrt(2.0_dop*z)
      rwidth = dble(z)
      cwidth = dimag(z)
         
      rsbaspar(1,f1,p)=gxeq
      rsbaspar(2,f1,p)=gmom
      rsbaspar(3,f1,p)=rwidth
      rsbaspar(4,f1,p)=cwidth
      rsbaspar(5,f1,p)=gmass
         
      isbaspar(1,f1,p)=typ

!-----------------------------------------------------------------------
! set defaults if GWPs
! - HO-gwp with shell, ovl=0.5, gwpwidth = rwidth
!-----------------------------------------------------------------------
      if (basis(f1) .eq. 25) then
         sbasis(f1)=9
         isbaspar(1,f1,p)=3
         isbaspar(2,f1,p)=0
         isbaspar(3,f1,p)=1
         rsbaspar(5,f1,p)=0.7
         rsbaspar(6,f1,p)=rwidth
      endif

      return
      end subroutine eb_ho! of eb_ho

!-----------------------------------------------------------------------
      subroutine eb_gauss(f1,p,iz,iz0,lunit)

      implicit none

      integer(long)                 :: ierr,ierr1,i,typ
      integer(long), intent(in)     :: iz,f1,p,lunit
      integer(long), intent(inout)  :: iz0

      real(dop)          :: rwidth,cwidth,gxeq,gmom
      complex(dop)       :: z

      rwidth  = 0.0_dop
      cwidth  = 0.0_dop
      sbasis(f1) = 1 
      ierr1=0
      i = 3
      typ = 0
      if (keyword(i).eq.'odd') then
         typ = 1
         i=i+1
      else if (keyword(i).eq.'even') then
         typ = 2
         i=i+1
      endif
         
      read(keyword(i),*,iostat=ierr) gxeq
      if (ierr .ne. 0) ierr1=ierr
      i = i+1
      if(keyword(i) .eq. ',') then
         i = i+1
         if (lunit .eq. 0) then
            call keylunits(gxeq,keyword(i),ierr)
         else if (lunit .eq. 1) then
            call keyeunits(gxeq,keyword(i),ierr)
         endif
         i = i+1
      endif

!     read momentum - also look in parameter list
      call rdebnumd(i,gmom,ierr)
      if (ierr .ne. 0) ierr1=ierr
      i = i+1
      read(keyword(i),*,iostat=ierr) rwidth
      if (ierr .ne. 0) ierr1=ierr
      i = i+1
      if (keyword(i) .eq. ',' ) then
         i = i+1
         if (lunit .eq. 0) then
            call keylunits(rwidth,keyword(i),ierr)
         else if (lunit .eq. 1) then
            call keyeunits(rwidth,keyword(i),ierr)
         endif
         if(ierr.gt.0) read(keyword(i),*,iostat=ierr) cwidth
         ierr1=ierr
         i = i+1
      endif
      if (keyword(i) .eq. ',' ) then
         i = i+1
         if (lunit .eq. 0) then
            call keylunits(rwidth,keyword(i),ierr)
            call keylunits(cwidth,keyword(i),ierr)
         else if (lunit .eq. 1) then
            call keyeunits(rwidth,keyword(i),ierr)
            call keyeunits(cwidth,keyword(i),ierr)
         endif
         if (ierr .ne. 0) ierr1=ierr
         i = i+1
      endif
         
      if (ierr1.ne.0) then
         routine='eb_gauss (eininwf_util.F)'
         message = 'Incorrect build definition for mode :'
         call wrline(onei,iz,iz0)         
      endif
            
!.... Transform to the convention used in genphi1.f for both HO and Gauss.
! sigma^2 = 1/rwidth
      z = dcmplx(rwidth,cwidth)
      if (abs(z).lt.1d-12) then
         routine='eb_gauss (eininwf_util.F)'
         message = "Gaussian width is too small :"
         call wrline(5,iz,iz0)
      endif
      z = 1.0_dop/z
      rwidth = dble(z)
      cwidth = dimag(z)
         
      rsbaspar(1,f1,p)=gxeq
      rsbaspar(2,f1,p)=gmom
      rsbaspar(3,f1,p)=rwidth
      rsbaspar(4,f1,p)=cwidth
      rsbaspar(5,f1,p)=1.0_dop
         
      isbaspar(1,f1,p)=typ

!-----------------------------------------------------------------------
! set defaults for GWPs
! - HO-gwp with shell, ovl=0.5, gwpwidth = rwidth
!-----------------------------------------------------------------------
      if (basis(f1) .eq. 25) then
         sbasis(f1)=9
         isbaspar(1,f1,p)=3
         isbaspar(2,f1,p)=0
         rsbaspar(5,f1,p)=0.5
         rsbaspar(6,f1,p)=rwidth
      endif

      return
      end subroutine eb_gauss! of eb_gauss

!-----------------------------------------------------

      subroutine eb_euclid(f1,p,iz,iz0)

      implicit none

      integer f1,p,iz,iz0,point,ierr

      sbasis(f1)  = 10

      read(keyword(3),*,iostat=ierr) point
      if (ierr.ne.0 .or. point.le.0 .or. point.gt.gdim(f1)) then
         routine = 'eb_euclid (eininwf_util.F)'
         message = 'The argument for euclid must be a positive integer'&
                 //' .le. gdim.             Incorrect build for mode :'
         call wrline(onei,iz,iz0)
      endif

      isbaspar(1,f1,p)=point

      return
      end subroutine eb_euclid! of eb_euclid

!-----------------------------------------------------
!               eigenf
! isbaspar(1)=ipot
!-----------------------------------------------------

      subroutine eb_eigenf(f1,p,iz,iz0)

      implicit none

      integer(long)                 :: m,ipot,n
      integer(long), intent(in)     :: iz,f1,p
      integer(long), intent(inout)  :: iz0

      if (basis(f1) .eq. 25) then
         routine='eb_eigenf (eininwf_util.F)'
         message= &
          'Eigenf build requires a grid. Cannot use with a GWP basis.'
         call wrline(0,iz,iz0)         
      endif
      sbasis(f1)  = 3
      m=dofspf(f1)
      ipot=0
      do n=1,hamdim
         if (keyorig(3) .eq. hamlab(n)) ipot=n
      enddo
      if (ipot .eq. 0) then
         routine='eb_eigenf (eininwf_util.F)'
         message='Hamiltonian required by eigenf not defined :'
         call wrline(3,iz,iz0)         
      endif
         
      isbaspar(1,f1,p)=ipot
      return
      end subroutine eb_eigenf! of eb_eigenf

!--------------------------------------------------------
!        sphfbr  (phifbr)
! isbaspar(1)=qnl0
! isbaspar(2)=qnm0
!--------------------------------------------------------

      subroutine eb_sphfbr(f1,p,iz,iz0)

      implicit none

      integer(long), intent(in)    :: f1,p,iz
      integer(long)                :: qnl0,ierr
      integer(long), intent(inout) :: iz0

      sbasis(f1) = 4
      read(keyword(3),*,iostat=ierr) qnl0
      if (ierr.ne.0) then
         routine = 'Eb_sphfbr (eininwfmod.f90)'
         message = 'Incorrect build definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      isbaspar(1,f1,p)=qnl0
      return

      end subroutine eb_sphfbr ! of eb_sphfbr


      subroutine eb_phifbr(f1,p,iz,iz0)

      implicit none

      integer(long), intent(in)     :: iz,f1,p
      integer(long), intent(inout)  :: iz0
      integer(long)                 :: qnm0,ierr

      sbasis(f1) = -1
      if (sbasis(f1-1) .ne. 4) then
         routine='eb_phifbr (eininwf_util.F)'
         message = '"phiFBR" must directly follow "sphFBR"'
         call wrline(zeroi,iz,iz0)     
      endif

      read(keyword(3),*,iostat=ierr) qnm0
      if (ierr.ne.0) then
         routine='eb_phifbr (eininwf_util.F)'
         message = 'Incorrect build definition for mode :'
         call wrline(onei,iz,iz0)     
      endif
         
      isbaspar(2,f1-1,p)=qnm0
      return
      end subroutine eb_phifbr ! of eb_phifbr

!-----------------------------------------------------------------
!       KLeg
! isbaspar(1) = sl0
! isbaspar(2) = isleg
! isbaspar(3) = iex
! isbaspar(4) = 1
! isbaspar(5) = ilogprint
! isbaspar(6) = nspf
! NB isbaspar(4) is reserved for pop. pop is not allowed for KLeg.
!-----------------------------------------------------------------

      subroutine eb_kleg(f1,parainc,p,ic,iz,iz0)

      implicit none

      integer(long)                 :: i,ierr,ierr1,iex,ilogprint,sl0,n,isleg,nspf
      integer(long), intent(in)     :: iz,f1,parainc,p,ic
      integer(long), intent(inout)  :: iz0

      sbasis(f1) = 5
      ierr1=0
      iex=1
      ilogprint=0
      nspf=0
         
      i=3
!     first parameter is l (numeric) or sl0XXX
      if (keyword(i)(1:3) .eq. 'sl0') then
         sl0 = -9999
         do n=1,npar
            if (apar(n) .eq. keyword(i)) then
               sl0=nint(rpar(n))
            endif
         enddo
         if( sl0 .eq. -9999 ) then
            routine='eb_kleg (eininwf_util.F)'
            message='sl0 is not set! Use e.g. "alter-parameters".'
            call wrline(zeroi,iz,iz0)
            sl0 = 0
         endif
      else
         read(keyword(i),*,iostat=ierr) sl0
         if (ierr .ne. 0) ierr1=ierr
      endif
      i=i+1
         
!     second parameter is symmetry
      if (keyword(i) .eq. 'sym' ) then
         isleg = 2-mod(sl0,2)
      elseif (keyword(i) .eq. 'nosym'  .or. &
              keyword(i) .eq. 'no_sym' .or. &
              keyword(i) .eq. 'no-sym') then
         isleg = 0
      else
         read(keyword(i),*,iostat=ierr) isleg
         if (ierr .ne. 0) ierr1=ierr
      endif
      i=i+1
         
!     further parameters: print, excite, nspf
   10 continue
      if (keyword(i) .eq. ' ') then
         goto 100

      elseif (keyword(i) .eq. ',') then
         i=i+1

      elseif (keyword(i) .eq. 'print') then 
         ilogprint=1
         i=i+1

      elseif (keyword(i) .eq. 'pack') then
         i=i+3

      elseif (keyword(i) .eq. 'excite' ) then
         if (keyword(i+1).eq.'=') then
            if (keyword(i+2).eq.'j') then
               iex=1
            elseif (keyword(i+2).eq.'m') then
               iex=2
            elseif (keyword(i+2).eq.'m-old') then
               iex=3
            else
               routine='eb_kleg (eininwf_util.F)'
               message='Excitation parameter must be "j" '// &
                       '"m", or "m-old", and not :'
               ierr1 = 1
               call wrline(i+2,iz,iz0)
            endif
            i=i+3
         else
            routine='eb_kleg (eininwf_util.F)'
            message='After "excite" must come "="'
            ierr1 = 1
            call wrline(zeroi,iz,iz0)
            i=i+1
         endif

      elseif (keyword(i) .eq. 'nspf') then
         if (keyword(i+1) .eq. '=') then
            read (keyword(i+2),*,iostat=ierr) nspf
            if (ierr.ne.0) ierr1=ierr
            i=i+3
         else
            routine='eb_kleg (eininwf_util.F)'
            message='After "nspf" must come "="'
            ierr1 = 1
            call wrline(zeroi,iz,iz0)
            i=i+1
         endif

      else
         routine='eb_kleg (eininwf_util.F)'
         message='Unknown parameter in KLeg InWF definition :'
         ierr1 = 1
         call wrline(i,iz,iz0)
         i=i+1
      endif
      if (i.le.ic-parainc) goto 10
         
  100 continue
      if (isleg.ne.0 .and. mod(sl0+isleg,2).ne.0) then
         ierr1   = 1
         routine='eb_kleg (eininwf_util.F)'
         message = 'sl0 is not consistent with isleg'
         call wrline(zeroi,iz,iz0)
      endif
      if ( sl0 .lt. isleg*(2-isleg) ) then
         ierr1   = 1
         routine='eb_kleg (eininwf_util.F)'
         message = 'sl0 is negative or not consistent with isleg'
         call wrline(zeroi,iz,iz0)
      endif
      if ( nspf .lt. 0) then
         ierr    = 1
         routine='eb_kleg (eininwf_util.F)'
         message = 'nspf must not be negative'
         call wrline(zeroi,iz,iz0)
      endif

      if (ierr1.ne.0) then
         routine='eb_kleg (eininwf_util.F)'
         message = 'Incorrect build definition for mode :'
         call wrline(onei,iz,iz0)
      endif
         
      isbaspar(1,f1,p) = sl0
      isbaspar(2,f1,p) = isleg
      isbaspar(3,f1,p) = iex
      isbaspar(4,f1,p) = 1      ! pop not for KLeg
      isbaspar(5,f1,p) = ilogprint
      isbaspar(6,f1,p) = nspf

      return
      end subroutine eb_kleg ! of eb_kleg

!---------------------------------------------------------------
!               K
! isbaspar(1) = slz
! isbaspar(2) = mmin
! isbaspar(3) = mmax
! isbaspar(4) = 1
! isbaspar(5) = dm
! NB isbaspar(4) is reserved for pop. pop is not allowed for K.
!---------------------------------------------------------------

      subroutine eb_k(f1,parainc,p,ic,iz,iz0)

      implicit none

      integer(long)                 :: ierr,ierr1,kmin,kmax,mmin,mmax,dm,&
                                       sl0,slz,n,bas1,bas2,slz1,slz2
      integer(long), intent(in)     :: iz,f1,parainc,p,ic
      integer(long), intent(inout)  :: iz0

      sbasis(f1)=6
      ierr1 = 0 
      bas1=f1-1 ! bas1 is the (Kleg,Pleg,Wigner) DOF combined with K
      bas2=f1+1 ! bas2 is the other K DOF combined with K (Wigner only)
      kmin=-999
      kmax=-999
      sl0=-999
      if (basis(f1-1).eq.7 .or. basis(f1-1).eq.20) then
         kmin = ipbaspar(1,f1)
         kmax = ipbaspar(2,f1)
      elseif (basis(f1-1).eq.9 .or. basis(f1-1).eq.21) then
         kmin = ipbaspar(1,f1)
         kmax = ipbaspar(2,f1)
      elseif (basis(f1-1).eq.33) then
         kmin = ipbaspar(2,f1-1)
         kmax = ipbaspar(3,f1-1)
      elseif (f1.gt.2 .and. basis(max(1,f1-2)).eq.33) then
         bas1 = f1-2  ! Index of Wigner DOF in 'Wigner+other K+this K'
         bas2 = f1-1  ! Index of other K DOF in 'Wigner+other K+this K'
         kmin = ipbaspar(6,f1-2)
         kmax = ipbaspar(7,f1-2)
      else
         kmin=ipbaspar(1,f1)
         kmax=ipbaspar(2,f1)
         dm  =ipbaspar(5,f1)
      endif
      if (sbasis(f1-1).eq.2) sl0 = isbaspar(2,f1-1,p)
      if (sbasis(f1-1).eq.5) sl0 = isbaspar(1,f1-1,p)
      if (sbasis(bas1).eq.8) sl0 = isbaspar(1,bas1,p)
         
      if (keyword(3)(1:3) .eq. 'slz') then
         slz = -9999
         do n=1,npar
            if (apar(n) .eq. keyword(3)) then
               slz=nint(rpar(n))
            endif
         enddo
         if ( slz .eq. -9999 ) then
            routine='eb_k (eininwf_util.F)'
            message='slz is not set! Use e.g. "alter-parameters".'
            call wrline(zeroi,iz,iz0)
            slz = 0
         endif
      elseif (keyword(3)(1:3) .eq. 'jbf') then
         slz = -9999
         do n=1,npar
            if (apar(n) .eq. keyword(3)) then
               slz=nint(rpar(n))
            endif
         enddo
         if( slz .eq. -9999 ) then
            routine='eb_k (eininwf_util.F)'
            message='jbf is not set! Use e.g. "alter-parameters".'
            call wrline(zeroi,iz,iz0)
            slz = 0
         endif
      else
         read(keyword(3),*,iostat=ierr) slz
         if (ierr .ne. 0) ierr1=ierr
      endif
         
      if (ic-parainc .ne. 3) then
         read(keyword(4),*,iostat=ierr) mmin
         if (ierr .ne. 0) ierr1=ierr
         if (ic-parainc .ne. 4) then
            read(keyword(5),*,iostat=ierr) mmax
            if (ierr .ne. 0) ierr1=ierr
            if (ic-parainc .ne. 5) then
               read(keyword(6),*,iostat=ierr) dm
               if (ierr .ne. 0) ierr1=ierr
            else
               dm = 1
            endif
         else
            mmax = kmax
            dm   = 1
         endif
      else
         mmin = kmin
         mmax = kmax
         dm   = 1
      endif

      if (sbasis(f1-1).eq.2 .or. sbasis(f1-1).eq.5 .or. &
            sbasis(bas1).eq.8) then
         if (slz.gt.sl0 ) then
            ierr1   = 1
            routine='eb_k (eininwf_util.F)'
            message = 'slz is  greater than sl0'
            call wrline(zeroi,iz,iz0)
         endif
      endif

      if (dm.lt.0) then
         ierr1   = 1
         routine='eb_k (eininwf_util.F)'
         message = 'dm must be greater than 0'
         call wrline(zeroi,iz,iz0)
      endif
         
      if ((mmin.lt.kmin) .or. (mmax.gt.kmax)) then
         ierr1  = 1
         routine='eb_k (eininwf_util.F)'
         message = '[mmin,mmax] is out of range'
         call wrline(zeroi,iz,iz0)
      endif

      if (sbasis(bas1).eq.5 .and. basis(bas1).eq.33) then
! --- If Kleg+K+K inwf is used with Wigner+K+K DVR, the slz value of
!     the Kleg must come from the FIRST K-inwf input. The slz of the
!     second K-inwf must then be zero.
         if (bas2.lt.f1 .and. slz.ne.0) then
            ierr1  = 1
            routine='eb_k (eininwf_util.F)'
            message = 'If Kleg+K+K inwf is used with Wigner+_+K DVR,'// &
                      ' then slz is defined by the first K line and '// &
                      'slz in the second K line must be set to zero.'
            call wrline(zeroi,iz,iz0)
         endif
      endif

      if (sbasis(bas1).eq.2) then
! --- If Leg+K inwf is used for Kleg+K or Wigner+K p-basis, then the
!     values of [kmin,kmax] for the K inwf must match those of the K
!     p-basis. This is due to the fact that BOTH are used to index
!     trafo: the p-basis values are used in genphi() and genaleg(),
!     while the inwf values are used in kclearspf()
         if (kmin.ne.mmin .or. kmax.ne.mmax) then
            ierr1  = 1
            routine='eb_k (eininwf_util.F)'
            message = 'For Leg+K inwf, [kmin,kmax] must match DVR'
            call wrline(zeroi,iz,iz0)
         endif
         slz1 = isbaspar(1,bas1,p)
         if (basis(bas1).eq.7 .or. &
            (basis(bas1).eq.33 .and. basis(bas2).ne.8)) then
! --- When using Leg+K inwf for Kleg+K or Wigner+K p-basis, slz is set
!     for both Leg inwf AND K inwf. The values should be consistent.
            if (slz.ne.slz1) then
               ierr1 = 1
               routine='eb_k (eininwf_util.F)'
               message='slz values from Leg-inwf and K-inwf inputs'// &
                       ' are not consistent.'
               call wrline(zeroi,iz,iz0)
            endif
         elseif (basis(bas1).eq.33 .and. basis(bas2).eq.8) then
! --- If Leg+K+K inwf is used for Wigner+K+K p-basis, there are THREE
!     slz values: one from Leg inwf, and one from each of two K-inwfs.
!     In this case one of the K-slz values should match the one
!     from Leg inwf input, and the other K-slz value should be zero.
!     This can be tested only after both K-slz values have been read.
            if (bas2.eq.f1-1) then ! bas2 is first K, f1 is the 2nd K
               slz2 = isbaspar(1,bas2,p)
               if (.not. ((slz2.eq.slz1 .and. slz.eq.0) .or. &
                          (slz.eq.slz1 .and. slz2.eq.0))) then
                  ierr1 = 1
                  routine='eb_k (eininwf_util.F)'
                  message='If Leg+K+K inwf is used with Wigner+K+K '// &
                          'DVR, the slz value from one K-inwf must '// &
                          'match the slz value from Leg-inwf, and '// &
                          'the slz value from the other K-inwf '// &
                          'must be zero.'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
         endif
! --- Finally, make sure slz is in the k-range
         if (slz.gt.mmax .or. slz.lt.mmin) then
            ierr1  = 1
            routine='eb_k (eininwf_util.F)'
            message = 'slz is out of range [kmin,kmax]'
            call wrline(zeroi,iz,iz0)
         endif
      endif

      if (ierr1.ne.0) then
         routine='eb_k (eininwf_util.F)'
         message = 'Incorrect build definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      isbaspar(1,f1,p) = slz
      isbaspar(2,f1,p) = mmin
      isbaspar(3,f1,p) = mmax
      isbaspar(4,f1,p) = 1      ! pop not for K
      isbaspar(5,f1,p) = dm
      
      return
      end subroutine eb_k! of eb_k

!--------------------------------------------------------------
!               MAP
! isbaspar(1..3) = f2   (DOF to be mapped)
!--------------------------------------------------------------

      subroutine eb_map(f1,p,ic,iz,iz0)

      implicit none

      integer(long)                :: f,f2,i,j,ierr
      integer(long), intent(in)    :: f1,p,ic,iz
      integer(long), intent(inout) :: iz0

      i=0
      f2=0
      do f = 1, ndof
         if(keyorig(3) .eq. modelabel(f) ) f2=f
      enddo
      if(f2.eq.0) then
         routine = "eb_map"
         message = "Unknown modelabel "//keyorig(3)
         call errormsg
      endif

  10  i=i+1
      if(i.ge.4) then
         routine = "eb_map"
         message = "There must be not more than three map lines for "&
                 //"one DOF."
         call errormsg
      endif
      if(isbaspar(i,f1,p).gt.0) goto 10
      isbaspar(i,f1,p) = f2
      sbasis(f1) = 7
      j=4
      if(keyorig(j).eq.',') j=j+1
      if(ic.ge.j) then
         read(keyorig(j),*,iostat=ierr) rsbaspar(i,f1,p)
         if(ierr.gt.0) then
            routine='eb_map (eininwf_util.F)'
            message='Cannot read map factor (re)'
            call wrline(zeroi,iz,iz0)
         endif
      else
         rsbaspar(i,f1,p)=1.d0
      endif
      j=j+1
      if(keyorig(j).eq.',') j=j+1
      if(ic.ge.j) then
         read(keyorig(j),*,iostat=ierr) rsbaspar(i+3,f1,p)
         if(ierr.gt.0) then
            routine='eb_map (eininwf_util.F)'
            message='Cannot read map factor (im)'
            call wrline(zeroi,iz,iz0)
         endif
      else
         rsbaspar(i+3,f1,p)=0.d0
      endif

      return
      end subroutine eb_map! of eb_map

!-----------------------------------------------------------------
! Readspf
!-----------------------------------------------------------------

      subroutine eb_readspf(f1,p,ic,iz,iz0)

      implicit none

      integer f1,p,ic,iz,iz0

      if (nreadspf.eq.maxreadspf) then
         routine = "eb_readspf"
         message = "too many readspf, increase maxreadwf in " &
                 //"runinwf.inc"
         call errormsg
      endif

      nreadspf = nreadspf+1
      readspffile(nreadspf) = keyorig(3)
      isbaspar(1,f1,p) = nreadspf
      isbaspar(2,f1,p) = 0
      sbasis(f1) = 12
      if(ic.gt.3) then
         if(keyword(4)(1:8).eq.'add-weig') then
            isbaspar(2,f1,p) = 1
         else
            routine = "eb_readspf"
            message = " unknown keyword : "
            call wrline (4,iz,iz0)
         endif
      endif

      return
      end subroutine eb_readspf ! of eb_readspf

!-----------------------------------------------------------------
!       Wigner
! isbaspar(1) = sl0
! isbaspar(2) = isleg
! isbaspar(3) = iex
! isbaspar(4) = 1
! isbaspar(5) = ilogprint
! NB isbaspar(4) is reserved for pop. pop is not allowed for Wigner.
!
! Note: symmetry is not yet implemented here
!------------------------------------------------------------------

      subroutine eb_Wigner(f1,parainc,p,ic,iz,iz0)

      implicit none

      integer(long), intent(in)    :: f1,parainc,p,ic,iz
      integer(long), intent(inout) :: iz0
      integer(long)                :: ierr,ierr1,i,iex,ilogprint,sl0,n,isleg

      sbasis(f1) = 8
      ierr1=0
      iex=1
      ilogprint=0

      i=3
!     first parameter is l (numeric) or sl0XXX
      if (keyword(i)(1:3) .eq. 'sl0') then
         sl0 = -9999
         do n=1,npar
            if (apar(n) .eq. keyword(i)) then
               sl0=nint(rpar(n))
            endif
         enddo
         if( sl0 .eq. -9999 ) then
            routine='eb_wigner (eininwf_util.F)'
            message='sl0 is not set! Use e.g. "alter-parameters".'
            call wrline(zeroi,iz,iz0)
            sl0 = 0
         endif
      else
         read(keyword(i),*,iostat=ierr) sl0
         if (ierr .ne. 0) ierr1=ierr
      endif
      i=i+1

!     second parameter is symmetry
      if (keyword(i) .eq. 'sym' ) then
         isleg = 2-mod(sl0,2)
         routine='eb_wigner (eininwf_util.F)'
         message='g/u symmetry is not yet implemented for Wigner.'
         call wrline(zeroi,iz,iz0)
         ierr1 = 1
      elseif (keyword(i) .eq. 'nosym'  .or. &
              keyword(i) .eq. 'no_sym' .or. &
              keyword(i) .eq. 'no-sym') then
         isleg = 0
      else
         read(keyword(i),*,iostat=ierr) isleg
         if (ierr .ne. 0) ierr1=ierr
      endif
      i=i+1

!     further parameters: print and excite
   10 continue
      if (keyword(i) .eq. ' ') then
         goto 100

      elseif (keyword(i) .eq. ',') then
         i=i+1

      elseif (keyword(i) .eq. 'print') then
         ilogprint=1
         i=i+1

      elseif (keyword(i) .eq. 'excite' ) then
         if (keyword(i+1).eq.'=') then
            if (keyword(i+2).eq.'mkj') then
               iex=1
            elseif (keyword(i+2).eq.'kmj') then
               iex=2
            else
               routine='eb_wigner (eininwf_util.F)'
               message='Excitation parameter must be "mkj" '// &
                       'or "kmj", and not :'
               ierr1 = 1
               call wrline(i+2,iz,iz0)
            endif
            i=i+3
         else
            routine='eb_wigner (eininwf_util.F)'
            message='After "excite" must come "="'
            ierr1 = 1
            call wrline(zeroi,iz,iz0)
            i=i+1
         endif
      else
         routine='eb_wigner (eininwf_util.F)'
         message='Unknown parameter in Wigner InWF definition :'
         ierr1 = 1
         call wrline(i,iz,iz0)
         i=i+1
      endif
      if (i.le.ic-parainc) goto 10

  100 continue
      if (isleg.ne.0 .and. mod(sl0+isleg,2).ne.0) then
         ierr1   = 1
         routine='eb_wigner (eininwf_util.F)'
         message = 'sl0 is not consistent with isleg'
         call wrline(zeroi,iz,iz0)
      endif
      if ( sl0 .lt. isleg*(2-isleg) ) then
         ierr1   = 1
         routine='eb_wigner (eininwf_util.F)'
         message = 'sl0 is negative or not consistent with isleg'
         call wrline(zeroi,iz,iz0)
      endif
      if (ierr1.ne.0) then
         routine='eb_wigner (eininwf_util.F)'
         message = 'Incorrect build definition for mode :'
         call wrline(onei,iz,iz0)
      endif

      isbaspar(1,f1,p) = sl0
      isbaspar(2,f1,p) = isleg
      isbaspar(3,f1,p) = iex
      isbaspar(4,f1,p) = 1      ! pop not for Wigner
      isbaspar(5,f1,p) = ilogprint

      return
      end subroutine eb_Wigner

!-----------------------------------------------------------------------
!  GWP wavepacket with gwp basis
! 
!  rsbaspar(1)=gxeq
!  rsbaspar(2)=gmom
!  rsbaspar(3)=rwidth   Width of WP
!  rsbaspar(4)=cwidth
!  rsbaspar(5)=gshift
!  rsbaspar(6)=gwpwidth  Width of GWPs
!  rsbaspar(7)=scwidth

!  isbaspar(1)=typ
!  isbaspar(2)=typ1
!  isbaspar(3)=typp
!  isbaspar(4) is pop
!  isbaspar(5)=nq  Grid dimension for grid distribution
!  isbaspar(6)=np
!  isbaspar(7)=1 if pergauss
!  isbaspar(8)=noq0
!-----------------------------------------------------------------------

      subroutine eb_gwp(f1,p,iz,iz0,ic,parainc,typp)

      implicit none

      integer(long)                 :: ierr,ierr1,i,typ,typ1,np,nq,&
                                       typp,noq0
      integer(long), intent(in)     :: f1,p,ic,parainc,iz
      integer(long), intent(inout)  :: iz0

      real(dop)          :: rwidth,cwidth,gxeq,gmom,gshift,scwidth,gwpwidth
      complex(dop)       :: z

      rwidth  = 0.0_dop
      cwidth  = 0.0_dop
      np = 0
      nq = 0
! Beginning of modification (Benjamin, 14 Feb 2007)
      if(sbasis(f1) .eq. 10) then
         write(6,'(/a)') 'ERROR: check GWP-function build block!'
         routine='eb_gwp (eininwf_util.F)'
         message='The GWP-function build block contains a mode-label'&
                 //' already set as GWP in the standard build block:'
         call wrline(1,iz,iz0)
         call errormsg
      else
         sbasis(f1) = 9
      endif
! End of modification (Benjamin, 14 Feb 2007)
      ierr1=0
      i = 3

! Read type of distribution
      typ = 0
      np = 0
      nq = 0

      call gwp_distin(i,typ,np,nq,noq0)
      i = i+1  ! move on to next keyword
      if (typ .eq. 0) then
         routine='eb_gwp (eininwf_util.F)'
         message='Unknown gwp keyword '//keyword(i)
         call warnmsg
      endif

! Read defining parameters
      read(keyword(i),*,iostat=ierr) gxeq
      if (ierr .ne. 0) ierr1=ierr
      i = i+1
      if(keyword(i) .eq. ',') then
         i = i+1
         call keylunits(gxeq,keyword(i),ierr)
         i = i+1
      endif
      read(keyword(i),*,iostat=ierr) gmom
      if (ierr .ne. 0) ierr1=ierr
      i = i+1
      read(keyword(i),*,iostat=ierr) rwidth
      if (ierr .ne. 0) ierr1=ierr
      i = i+1
      if (keyword(i) .eq. ',' ) then
         i = i+1
         call keylunits(rwidth,keyword(i),ierr)
         if(ierr.gt.0) read(keyword(i),*,iostat=ierr) cwidth
         ierr1=ierr
         i = i+1
      endif
      if (keyword(i) .eq. ',' ) then
         i = i+1
         call keylunits(rwidth,keyword(i),ierr)
         call keylunits(cwidth,keyword(i),ierr)
         if (ierr .ne. 0) ierr1=ierr
         i = i+1
      endif

      gshift=0.7
      typ1 = 0
      scwidth=0.0_dop
      gwpwidth=-999.0_dop
         
! read in optional keywords

      do while (i .le. ic-parainc) 
         if (keyword(i) .eq. 'ovl') then
            typ1 = 0
            i = i+1
            if(keyword(i) .eq. '=') then
               i = i+1
               read(keyword(i),*,iostat=ierr) gshift
               if (ierr .ne. 0) ierr1=ierr
               i = i+1
            else
               gshift=0.5
            endif

         else if (keyword(i) .eq. 'step') then
            typ1 = 1
!            read(keyword(i),*,iostat=ierr) gshift
!            if (ierr .ne. 0) ierr1=ierr
            i = i+1
            if(keyword(i) .eq. '=') then
               i = i+1
               read(keyword(i),*,iostat=ierr) gshift
               if (ierr .ne. 0) ierr1=ierr
               i = i+1
               if(keyword(i) .eq. ',') then
                  i = i+1
                  call keylunits(gshift,keyword(i),ierr)
                  if (ierr .ne. 0) ierr1=ierr
                  i = i+1
               endif
            else
               gshift=0.5
            endif

         elseif (keyword(i) .eq. 'scw') then
            i = i+1
            if(keyword(i) .eq. '=') then
               i = i+1
               read(keyword(i),*,iostat=ierr) scwidth
               if (ierr .ne. 0) ierr1=ierr
               i = i+1
            endif

         else if (keyword(i) .eq. 'gwpwidth') then
           if (keyword(i+1)/='=') then
              ierr1=1
           else
              i = i+2
              read(keyword(i),*,iostat=ierr) gwpwidth
              if (ierr .ne. 0) ierr1=ierr
              i = i+1
              if(keyword(i) .eq. ',') then
                i = i+1
                call keylunits(gwpwidth,keyword(i),ierr)
                i = i+1
              endif
           endif
!---
         else if (keyword(i).eq.'periodic') then
             i=i+1
             isbaspar(7,f1,p) = 1
!---             
         else
            i=i+1
            ierr1 = 1
            routine='EB_GWP (eininwf.F90)'
            write(message,'(2a)') 'Unknown keyword :',keyword(i)
            call warnmsg

         endif

      enddo ! end of optional parameter loop

      if (ierr1.ne.0) then
         routine='eb_gwp (eininwf_util.F)'
         message = 'Incorrect build definition for mode :'
         call wrline(onei,iz,iz0)         
      endif
            
!.... Transform to the convention used in genphi1.f for both HO and Gauss.
      z = dcmplx(rwidth,cwidth)
      z = 1.0_dop/z
      rwidth = dble(z)
      cwidth = dimag(z)
      if (gwpwidth .gt. 0.0) then
         gwpwidth=1.0/gwpwidth
      else
         gwpwidth=rwidth
      endif
         
! This list has to be the same in eb_hogwp and einingeom
      rsbaspar(1,f1,p)=gxeq
      rsbaspar(2,f1,p)=gmom
      rsbaspar(3,f1,p)=rwidth
      rsbaspar(4,f1,p)=cwidth
      rsbaspar(5,f1,p)=gshift
      rsbaspar(6,f1,p)=gwpwidth
      rsbaspar(7,f1,p)=scwidth

      isbaspar(1,f1,p)=typ
      isbaspar(2,f1,p)=typ1
      isbaspar(3,f1,p)=typp
! isbaspar(4) is pop
      isbaspar(5,f1,p)=nq
      isbaspar(6,f1,p)=np
! isbaspar(7)=1 if pergauss
      isbaspar(8,f1,p)=noq0
      

      return
      end subroutine eb_gwp

!-----------------------------------------------------------------------

      subroutine eb_hogwp(f1,p,iz,iz0,typp)

      implicit none

      integer(long)                 :: ierr,ierr1,i,typ,typ1,np,nq,noq0
      integer(long), intent(in)     :: f1,p,iz,typp
      integer(long), intent(inout)  :: iz0

      real(dop)          :: rwidth,cwidth,rfreq,cfreq,gxeq,gmom,gmass,gshift,&
                            gwpfreq,gwpwidth
      complex(dop)       :: z

      rfreq  = 0.0_dop
      cfreq  = 0.0_dop
      np = 0
      nq = 0
      sbasis(f1) = 9 
      ierr1=0
      i = 3

! Read type of distribution
      typ = 0
      typ1 = 0
      np = 0
      nq = 0

      call gwp_distin(i,typ,np,nq,noq0)
      i = i+1  ! move on to next keyword
      if (typ .eq. 0) then
         routine='eb_hogwp (eininwf_util.F)'
         message='Unknown gwp keyword '//keyword(i)
         call warnmsg
      endif

! if reading parameters no need for more input
      if (typ .eq. 11) go to 900
         
      read(keyword(i),*,iostat=ierr) gxeq
      if (ierr .ne. 0) ierr1=ierr
      i = i+1
      if(keyword(i) .eq. ',') then
         i = i+1
         call keylunits(gxeq,keyword(i),ierr)
         i = i+1
      endif
      read(keyword(i),*,iostat=ierr) gmom
      if (ierr .ne. 0) ierr1=ierr
      i = i+1
      read(keyword(i),*,iostat=ierr) rfreq
      if (ierr .ne. 0) ierr1=ierr
      i = i+1
      if (keyword(i) .eq. ',' ) then
         i = i+1
         call keyeunits(rfreq,keyword(i),ierr)
         if(ierr.gt.0) read(keyword(i),*,iostat=ierr) cfreq
         ierr1=ierr
         i = i+1
      endif
      if (keyword(i) .eq. ',' ) then
         i = i+1
         call keyeunits(rfreq,keyword(i),ierr)
         call keyeunits(cfreq,keyword(i),ierr)
         if (ierr .ne. 0) ierr1=ierr
         i = i+1
      endif
      read(keyword(i),*,iostat=ierr) gmass
      if (ierr.ne.0) ierr1=ierr
      i = i+1
      if(keyword(i) .eq. ',') then
         i = i+1
         call keymunits(gmass,keyword(i),ierr)
         if (ierr .ne. 0) ierr1=ierr
         i = i+1
      endif
      rfreq = rfreq*gmass
      cfreq = cfreq*gmass

      gwpfreq=-999.0_dop
      gshift = 0.7
      typ1 = 0
      if (keyword(i) .eq. 'ovl') then
         typ1 = 0
         i = i+1
         if(keyword(i) .eq. '=') then
            i = i+1
            read(keyword(i),*,iostat=ierr) gshift
            if (ierr .ne. 0) ierr1=ierr
            i = i+1
         else
            gshift=0.5
         endif
      else if (keyword(i) .eq. 'step') then
         typ1 = 1
         i = i+1
         if(keyword(i) .eq. '=') then
            i = i+1
            read(keyword(i),*,iostat=ierr) gshift
            if (ierr .ne. 0) ierr1=ierr
            i = i+1
            if(keyword(i) .eq. ',') then
               i = i+1
               call keylunits(gshift,keyword(i),ierr)
               if (ierr .ne. 0) ierr1=ierr
               i = i+1
            endif
         else
            gshift=0.5
         endif

      else if (keyword(i) .eq. 'gwpfreq') then
         if(keyword(i) .eq. '=') then
            i = i+1
            read(keyword(i),*,iostat=ierr) gwpfreq
            if (ierr .ne. 0) ierr1=ierr
            i = i+1
            if(keyword(i) .eq. ',') then
               i = i+1
               call keylunits(gwpfreq,keyword(i),ierr)
               if (ierr .ne. 0) ierr1=ierr
               i = i+1
            endif
         endif

         gwpfreq = gwpfreq*gmass
      endif

900   continue
      if (ierr1.ne.0) then
         routine='eb_hogwp (eininwf_util.F)'
         message = 'Incorrect build definition for mode :'
         call wrline(onei,iz,iz0)         
      endif
            
!.... Transform to the convention used in genphi1.f for both HO and Gauss.
      z = dcmplx(rfreq,cfreq)
      z = sqrt(2.0_dop*z)
      rwidth = dble(z)
      cwidth = dimag(z)

      if (gwpfreq .gt. 0.0) then
         gwpwidth=sqrt(2.0_dop*gwpfreq)
      else
         gwpwidth=rwidth
      endif

! This list has to be the same in eb_gwp and einingeom
      rsbaspar(1,f1,p)=gxeq
      rsbaspar(2,f1,p)=gmom
      rsbaspar(3,f1,p)=rwidth
      rsbaspar(4,f1,p)=cwidth
      rsbaspar(5,f1,p)=gshift
      rsbaspar(6,f1,p)=gwpwidth
! do not use width scaling
      rsbaspar(7,f1,p)=0.0

      isbaspar(1,f1,p)=typ
      isbaspar(2,f1,p)=typ1
      isbaspar(3,f1,p)=typp
! isbaspar(4) is pop
      isbaspar(5,f1,p)=nq
      isbaspar(6,f1,p)=np
! isbaspar(7)=1 if pergauss
      isbaspar(8,f1,p)=noq0

      return
      end subroutine eb_hogwp

!-----------------------------------------------------------------------
!
! isbaspar(1) = nham  the operator driving the coordinate
!
!-----------------------------------------------------------------------

      subroutine eb_tdpoint(f1,p)

      implicit none

      integer(long), intent(in)     :: f1,p
      integer(long)                 :: s,m,n
      character(c1)                 :: fdrive

      sbasis(f1) = 11

! get name of driving function
      fdrive = keyword(3)(1:c1)
      n=0
      isbaspar(1,f1,p)=0
      do while (n .le. hamdim)
         n=n+1
         if (fdrive .eq. hamlab(n)) then
            isbaspar(1,f1,p)=n
            exit
         endif
      enddo
!
! check specified operator exists
!
      if (isbaspar(1,f1,p) .eq. 0) then
         routine='Eb_tdpoint (eininwf) '
         write(message,'(2a)') &
     'Operator specified for TDpoint SPF does not exist : ',fdrive
         call warnmsg
      endif
!
! check DOF is not in a combined mode, and if SPFs have been defined
! check only 1
!
      m=dofspf(f1)
      if (m .eq. 0) return

      if (nspfdof(m) .gt. 1) then
         routine='Eb_tdpoint (eininwf) '
         message = &
     'TDpoint SPF must not be in a combined mode '
         call warnmsg
      endif
!
! Only 1 SPF required
!
      do s=1,nstate
         dim(m,s)=1
      enddo

      return
      end subroutine eb_tdpoint

!--------------------------------------------------------------
!              EINREADWF
!--------------------------------------------------------------

      subroutine einreadwf(iz,iz0)

      implicit none

      integer(long)                          :: i,ic,ispf,iavec,ierr,ier1,&
                                                j,k,instate,nwf
      integer(long), intent(in)              :: iz
      integer(long), intent(inout)           :: iz0
      integer(long), dimension(maxsta)       :: icount
      real(dop)                              :: rewt,imwt
      logical(kind=4)                        :: check

      instate=0
      icount(:)=0
      ispf=-999
      iavec=-999
      write(ilog,'(/a)') ' ***  READ-INWF  *** '

      check=.false.
      inquire(irst,opened=check)
      if (check) close(irst)

      nwf = 0
 100  call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine='einreadwf (eininwfmod.f90)'
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      if (keyword(1) .eq. 'end-read-inwf' ) then
         if (lmult) then
            do i = 1, nstate
               if(icount(i).eq.0 .and. .not.(lexact .or. lrealwp)) then
                  routine='einreadwf (eininwfmod.f90)'
                  write(message,'(a,i3)') 'No SPF definition for state',i
                  call errormsg
               elseif(icount(i).gt.1) then
                  routine='einreadwf (eininwfmod.f90)'
                  write(message,'(a,i3)') &
                  'Multiple SPF definition for state',i
                  call errormsg
               endif
            enddo
         else if (leb) then
            if(icount(1).gt.0) then
               routine='einreadwf (eininwfmod.f90)'
               write(message,'(a,i3)') &
                     'No SPF definition required for single-set basis'
               call errormsg
            endif
         endif

         nreadwf = nwf   ! store no. of files to be read
         return

      else if (keyword(1) .eq. 'end-input' .or. &
              keyword(1) .eq. 'end-init_wf-section') then
         routine='eininwf'
         message='No end-read-inwf keyword at end of read-inwf'
         call warnmsg
         return

!-----------------------------------------------------------------------
! Search for "file" keyword
!-----------------------------------------------------------------------
      else if(keyword(1) .eq. 'file') then
         lrestart = .true.
         if(keyword(2) .eq. '=') then
            rname = keyorig(3)
            rlaenge = lc(3)
            call abspath(rname,rlaenge)
            ispf=0
            iavec=0
            if(nwf.gt.maxreadwf) then
               routine='einreadwf (eininwfmod.f90)'
               message= &
            'Too many files! Increase maxreadspf in runinwfmod.f90'
               call errormsg
            endif

            rname=rname(1:rlaenge)//'/restart'
            open(irst,file=rname,form='unformatted',status='old',err=34)
            goto 36
! --- Long form failed, try not to append 'restart'
  34        rname=rname(1:rlaenge)//'        '
            open(irst,file=rname,form='unformatted',status='old',err=999)
  36        continue

            nwf = nwf+1
            readwffile(nwf) = rname
            close(irst)

         else
            routine='einreadwf (eininwfmod.f90)' 
            message='Expecting "=" after "file", not :' 
            call wrline(2,iz,iz0)
         endif
         if(ic.gt.3) then
            routine='einreadwf (eininwfmod.f90)'
            message='Nothing must follow the file-name! '//&
                    '(put noorthopsi on a separate line)'
            call wrline(0,iz,iz0)
         endif

      elseif(keyword(1) .eq. 'spf') then
         if(keyword(3).eq.'-' .and. keyword(4).eq.'>') then
            read(keyword(2),*,iostat=ierr) k 
            read(keyword(5),*,iostat=ier1) j 
            if(ierr.ne.0 .or. ier1.ne.0 .or. k.le.0 .or. j.le.0) then
               routine='einreadwf (eininwfmod.f90)'  
               message='Expecting positive integers after "SPF" ' 
               call wrline(0,iz,iz0)
            endif
            if(ispf.ge.maxsta) then
               routine='einreadwf (eininwfmod.f90)'
               message='Too many SPF lines! Maximum is maxsta.'
               call errormsg
            endif
            ispf=ispf+1
            smapwf(ispf,1,nwf)=k
            smapwf(ispf,2,nwf)=j
            icount(min(j,maxsta))=icount(min(j,maxsta))+1
         else
            routine='einreadwf (eininwfmod.f90)'
            message='Expecting "->" , not :'
            call wrline(3,iz,iz0)
         endif
         i=6
         if(keyword(i).eq.',') then
  50        read(keyword(i+1),*,iostat=ierr) j
            if(ierr.ne.0 .or. j.le.0) then
               routine='einreadwf (eininwfmod.f90)'
               message='Expecting positive integers after "SPF", not : '
               call wrline(i+1,iz,iz0)
            endif
            if(ispf.ge.maxsta) then
               routine='einreadwf (eininwfmod.f90)'
               message='Too many SPF lines! Maximum is maxsta.'
               call errormsg
            endif
            ispf=ispf+1
            smapwf(ispf,1,nwf)=k
            smapwf(ispf,2,nwf)=j
            icount(min(j,maxsta))=icount(min(j,maxsta))+1
            i=i+2
            if(keyword(i).eq.',') goto 50
          endif

      elseif(keyword(1) .eq. 'a') then
         if(keyword(3).eq.'-' .and. keyword(4).eq.'>') then
            iavec=iavec+1
            read(keyword(2),*,iostat=ierr) k
            read(keyword(5),*,iostat=ier1) j
            if(ierr.ne.0 .or. ier1.ne.0 .or. k.le.0 .or. j.le.0) then
               routine='einreadwf (eininwfmod.f90)'
               message='Expecting positive integers after "A" '
               call wrline(0,iz,iz0)
            endif
            if(iavec.gt.maxsta) then
               routine='einreadwf (eininwfmod.f90)'
               message='Too many A lines! Maximum is maxsta.'
               call errormsg
            endif
            amapwf(iavec,1,nwf)=k
            amapwf(iavec,2,nwf)=j
            if(ic.gt.6) then    ! read aweight
               if(keyword(6) .eq. ',') then
                  continue         ! No weights, A -> j,k .. input
               elseif(keyword(6).eq.'weight'.and.keyword(7).eq.'=') then
                  read(keyword(8),*,iostat=ierr) rewt
                  imwt=0.0_dop
                  if(keyword(9) .eq. ',')&
                    read(keyword(10),*,iostat=ier1) imwt
                  aweight(j)=dcmplx(rewt,imwt)
                  if(ierr.ne.0 .or. ier1.ne.0) then
                     routine='einreadwf (eininwfmod.f90)'
                     message='Cannot read weight (c-number)'
                     call wrline(0,iz,iz0)
                  endif
               else
                  routine='einreadwf (eininwfmod.f90)'
                  message='Expecting "weight = <c-number>" , not :'
                  call wrline(6,iz,iz0)
               endif
            endif

         else
            routine='einreadwf (eininwfmod.f90)'
            message='Expecting "->" , not :'
            call wrline(0,iz,iz0)
         endif
         i=6
         if(keyword(i).eq.',') then
  51        read(keyword(i+1),*,iostat=ierr) j
            if(ierr.ne.0 .or. j.le.0) then
               routine='einreadwf (eininwfmod.f90)'
               message='Expecting positive integers after "A", not : '
               call wrline(i+1,iz,iz0)
            endif
            if(iavec.ge.maxsta) then
               routine='einreadwf (eininwfmod.f90)'
               message='Too many A lines! Maximum is maxsta.'
               call errormsg
            endif
            iavec=iavec+1
            amapwf(iavec,1,nwf)=k
            amapwf(iavec,2,nwf)=j
            i=i+2
            if(keyword(i).eq.',') goto 51
         endif

      elseif(keyword(1) .eq. 'hartree') then
         lhartreepsi = .true.
      elseif(keyword(1) .eq. 'orthopsi') then
         lorthopsi = .true.
      elseif(keyword(1) .eq. 'noorthopsi') then
         lorthopsi = .false.
      elseif(keyword(1) .eq. 'realpsi') then
         lrealpsi = .true.
      elseif(keyword(1) .eq. 'ignore') then
         lignore = .true.
      elseif(keyword(1) .eq. 'normalise') then
         lrstnorm = .true.
      elseif(keyword(1) .eq. 'nonormalise') then
         lrstnorm = .false.
      elseif(keyword(1) .eq. 'incoherent') then
         lrstcoh = .false.

      elseif(keyword(1) .eq. 'coherence') then
         if(keyword(2).eq.'=') then
            read(keyword(3),*,iostat=ierr) dlstate
            read(keyword(5),*,iostat=ierr) drstate
         else
            routine='einreadwf (eininwfmod.f90)'
            message='Expecting "=" after "coherence , not :'
            call wrline(2,iz,iz0)
         endif
         if(ierr.ne.0) then
            routine='einreadwf (eininwfmod.f90)'
            message='Error reading states for coherence'
            call wrline(3,iz,iz0)
         endif

      else
         routine='einreadwf (eininwfmod.f90)'
         message='Expecting keywords such as "file" or "SPF" or "A"'// &
                 ', but not : '
         call wrline(1,iz,iz0)
      
      endif

      goto 100

 999  routine='einreadwf (eininwfmod.f90)'
      message='Cannot open file:'//rname(1:rlaenge+8)
      call errormsg

      end subroutine einreadwf         ! end of einreadwf

!#######################################################################

      subroutine rdebnumd(i,num,ierr)

      implicit none

      integer(long)             :: n
      integer(long),intent(in)  :: i
      integer(long),intent(out) :: ierr
      real(dop),intent(out)     :: num
      logical(kind=4)           :: found

      found=.false.
      ierr = 0
      do n=1,npar
         if (apar(n) .eq. keyword(i)) then
            num = rpar(n)
            found = .true.
         endif
      enddo
      if (.not. found) read(keyword(i),*,iostat=ierr) num

      return
      end subroutine rdebnumd

!#######################################################################
!
! Set default values for variables used in DVR generation
!
!#######################################################################

      subroutine initwfdefaults

      implicit none

!-----------------------------------------------------------------------
! read by subroutine eininwf
!-----------------------------------------------------------------------
      symcv(:)=0
      msymmtr(:)=0
      citype=0
      cicut=2.0_dop
      ciweight(:)=0.0_dop

      call zerovxi(inwfop,16)
      isbaspar(:,:,:) = 0
      rsbaspar(:,:,:) = 0.0_dop
      sbasis(:) = 999
      maxspf = 0
      dimfran(:) = 0

      dlstate = 1
      drstate = 1
      intrace = 1.0_dop
      tfac    = 0.5_dop
      inctr   = (1.0_dop,0.0_dop)

      sbfound(:,:) = .false.
      lprintmode= .false.
      ladiab    = .false.
      lad       = .false.
      ldia      = .false.
      ledstr    = .false.
      lhh2      = .false.
      lbkmp2    = .false.
      lignore   = .false.
      lnodirect = .true.
      lnonorm   = .false.
      loportho   = .false.
      inwfopnit = 10
      inwfoptol = 1.0d-8

      if (ldd .and. ltraj) then
         pqdistr=1
      else
         pqdistr=2
      endif
!      pqseed=21867

      AZahl    = 0
      iza      = 0

      e0eigen   = 0.0_dop
      memode = 0
      menum = -2
      meham = 0
      medim = 0
      mez   = 0
      mearray(:,:) = 0
      aweight(:) = 1.0_dop
      selectspf(:,:,:) = 0
      smapwf(:,:,:)=0
      amapwf(:,:,:)=0

      end subroutine initwfdefaults

      end module eininwfmod

