!!***********************************************************************
! contains: gencoeff,rdkoeff,genjindx,thermalize2,multfac,nsym2d_fixa
!
!***********************************************************************
      module gencoeffmod

use decimal, only: long, dop
use logdat
use global
use maxv, only: maxdim,maxsta
use dvrdatmod
use griddatmod
use operdef
use psidef
use runinwfmod
use channels

use gwpspfmod, only: gwpspf
use getpsimod, only: getcoeff3d
use psiutil, only: adecoder, mltop_decoder
use gwplib, only: fgaover1,ffgaover
use iopsidef, only: decompj
use genphi1, only: excite2

use d2lib, only: d2retrace

use mtlib,   only: dtxxzzo,dtxxddo
use xvlib,   only: xvixdzo,xvxxdzo
use op1lib,  only: normvxz, unitqxz, zerovxi, initvxz, zerovxz,trhxz
use linear,  only: ijk_shape
use mltypes
use wftree1, only: allocate_mtraces_matrices2,deallocate_mtraces_matrices,&
                  set_hartree_avec

      implicit none
      private
      public :: gencoeff,rdkoeff
      contains

!***********************************************************************
!
!           MCTDH   subroutine GENCOEFF
!
!
!  Define MCTDH expansion coefficients depending on input.
!  If azahl.ne.0, i.e. if the expansion coefficients are explicitly
!  given, nothing is done.
!  If azahl.eq.0 one expansion coefficient is set to one.
!
!  12/96 AJ                     
!
!***********************************************************************

      subroutine gencoeff(psi,jindx,agmat)

      implicit none

      integer(long)                               :: s,instate,p,s1,e,&
                                                     m,f,meb1
      integer(long), dimension(3)                 :: ijk
      integer(long), dimension(jindxdim), intent(out) :: jindx
      real(dop)                                   :: norm,phase

      complex(dop), dimension(dgldim), intent(out)     :: psi
      complex(dop), dimension(d3matdim)                :: spfwei
      complex(dop), dimension(agmatdim), intent(inout) :: agmat

      type(wfnode_t),pointer                           :: childf

      routine='Gencoeff'

!-----------------------------------------------------------------------
! zero coefficients
!-----------------------------------------------------------------------
      call zerovxz(psi,adim)

      if ((dentype.eq.0).or.(dentype.eq.1)) then

!-----------------------------------------------------------------------
! if selected CI, generate vector of allowed indices
! then assign vdims and ndims
!-----------------------------------------------------------------------
         if (citype .gt. 0) then
            call genjindx(jindx,cicut,vdim,dim,ndim,jvdim,jdim, &
                 jindxdim,zpsi,ciweight)
         endif

!-----------------------------------------------------------------------
! read in coefficients 
!-----------------------------------------------------------------------
         if (azahl.ne.0) then
! for ML set vectors for intermdiate layers
            if (psitype .eq. 5) call set_hartree_avec(psi,tree)
            call rdkoeff(psi,agmat)

!-----------------------------------------------------------------------
! or generate coefficients and set first coefficient of populated 
! state to 1.0
! 1. calculate weight for each spf (for non-GWPs weight=1.0 for first 
!    spf)
! 2. multiply out psi array
!-----------------------------------------------------------------------
         else if (dentype .eq. 0 .and. lrunml) then
            if (.not. tree%istop) then
               routine = 'Genrho1 (genrho1mod.f90)'
               message = 'Electronic DOF not at top of tree '
               call errormsg
            endif

            call set_hartree_avec(psi,tree)

         else if (dentype .eq. 0) then
            do s=1,nstate
               do m=1,nmode
                  f=spfdof(1,m)
                  if (basis(f) .eq. 25 .and. abs(psitype) .eq. 4) then
                     call gwpspf(agmat(zphase(m,s)),agmat(zxivec(m,s)), &
                          agmat(zagmat(m,s)),psi(zxi(m,s)),gwpdim(m,s), &
                          nspfdof(m),m,s,spfwei(d3mat(m,s)))     

                  else if (basis(f) .eq. 25) then
                     call gwpspf(agmat(zphase(m,s)),agmat(zxivec(m,s)), &
                          agmat(zagmat(m,s)),psi(zxi(m,s)),gwpdim(m,s), &
                          nspfdof(m),m,s,psi(zdgwp(m,s)))     
                     call zerovxz(spfwei(d3mat(m,s)),dim(m,s))
                     spfwei(d3mat(m,s))=1.0_dop

                  else if (basis(f) .eq. 26) then
                     do e=1,dim(m,1)
                        spfwei(d3mat(m,s)+e-1) = &
                                dimag(psi(zetf(m,1)+(e-1)*3+2))
                        phase=dble(psi(zetf(m,1)+(e-1)*3+2))
                        psi(zetf(m,1)+(e-1)*3+2)=dcmplx(phase,0.0_dop)
                     enddo
                  else
                     call zerovxz(spfwei(d3mat(m,s)),dim(m,s))
                     spfwei(d3mat(m,s))=1.0_dop
                  endif
               enddo
            enddo

            if (lgwpphase) then
               do s=1,nstate
                  do m=1,nmode
                     if(gwpm(m)) psi(zphi(m,s):zphi(m,s)+gwpdim(m,s)-1) = &
                          agmat(zphase(m,s):zphase(m,s)+gwpdim(m,s)-1)
                  enddo
               enddo
            endif

            if(lreflex) then
               psi(1)=1.0_dop
               psi(adim/2+1)=1.0_dop
            else
               do p=1,npacket
                  if (lmult) then
! look for initial state information
                     s1 = nddstate*(p-1) + 1
                     instate = isbaspar(4,feb,s1)
                     s = max(instate,1)
                  else
                     s=1  ! for single-set initial state is set
                  endif   ! by pop in genpsi.F 

                  s1=(p-1)*(nstate/npacket) + s
                  if(s1.gt.nstate)then
                     routine = 'gencoeff'
                     message = 'Initial state greater than no. of states.'
                     call errormsg
                  endif
                  call initvxz(psi(zpsi(s1)),block(s1))

                  do m=1,nmode
                     call dtxxzzo(spfwei(d3mat(m,s1)),psi(zpsi(s1)),&
                          vdim(m,s1),dim(m,s1),ndim(m,s1))
                  enddo
               enddo
            endif

         endif

      else if (dentype.eq.2 .and. lrunml) then
         if (.not. tree%istop) then
            routine = 'Gencoeff (gencoeffmod.f90)'
            message = 'Electronic DOF not at top of tree '
            call errormsg
         endif

         if (temperature0 .gt. tiny) then
            call thermalize2(psi,1,1)

         else
! pure state (Hartree product) generation
            call set_hartree_avec(psi,tree)

            if (dlstate .ne. drstate) then
                do f=1,tree%nmodes
                   tz => tree%swap_down(1)%p
                   childf => tz%submodes(f)%p
                   if (childf%m .eq. meb) meb1=f
                enddo
                ijk = ijk_shape(meb1,tree%ndim)
                call seteb_den2(psi,ijk(2),ijk(1),ijk(3))
            endif

         endif

         call normvxz(psi,norm,adim)
         call xvixdzo(norm,psi,adim)

      else if (dentype.eq.2) then

!-----------------------------------------------------------------------
! Choose initial state
!-----------------------------------------------------------------------
         if ((dlstate.lt.1) .or. (dlstate.gt.nddstate)) then
            message = 'Choose correct init_state.'
            call ErrorMsg
         endif
         if ((drstate.lt.1) .or. (drstate.gt.nddstate)) then
            message = 'Choose correct ground_state.'
            call ErrorMsg
         endif
         
         if (lmult) then
            if (temperature0 .lt. tiny) then
               if (dlstate .ne. drstate) then
! form a 50/50 mix of dlstate and drstate  
                  psi(dzpsi(dlstate,dlstate))=0.5d0
                  psi(dzpsi(drstate,drstate))=0.5d0

                  psi(dzpsi(dlstate,drstate))=0.5_dop
                  psi(dzpsi(drstate,dlstate)) =psi(dzpsi(dlstate,drstate))
               else
                  psi(dzpsi(drstate,drstate))=(1.0_dop,0.0_dop)
               endif
            else
               if (dlstate .ne. drstate) then
                  call thermalize2(psi,dlstate,drstate)
                  call thermalize2(psi,drstate,dlstate)
                  call thermalize2(psi,drstate,drstate)
                  call thermalize2(psi,dlstate,dlstate)
               else
                  call thermalize2(psi,drstate,drstate)
               endif
            endif
         else
            if (temperature0 .lt. tiny) then
               if (dlstate .ne. drstate) then
! form a 50/50 mix of dlstate and drstate  
                  call seteb_den2(psi(dzpsi(1,1)),dim(meb,1),vdim(meb,1),&
                       ndim(meb,1))
               else
                  psi(1)=(1.0_dop,0.0_dop)
               endif

            else
               s=1
               s1=1
               call thermalize2(psi,s,s1)
            endif
         endif           

         call normvxz(psi,norm,adim)
         call xvixdzo(norm,psi,adim)

      endif

      if (lbuildrestart) then
         call getcoeff3d(psi,jindx)
      endif
        
      return
      end subroutine gencoeff

!***********************************************************************
!
!            MCTDH   subroutine   RDKOEFF
!
! called from runinwf or gencoeff ! Modification (Benjamin, 15 Oct 2007)
!
! reads specified wavefunction coefficients from the .inp file
!
! variables passed:
!  psi: wavefunction coefficients
!
!  V6.0 AJ 
!
!***********************************************************************

      subroutine rdkoeff(psi,agmat) ! Modification (Benjamin, 14 Feb 2007)

      implicit none
      
      integer(long)                      :: b,s,t,i,k,m,mblock,j,j1,b1,&
                                            e,e1,gdof,p,pp,sindx,nm
      integer(long)                      :: ic,iz,ierr,ilbl
      integer(long), dimension(maxdim)   :: indx
      real(dop)                          :: norm

      complex(dop)                                 :: acoeff,ove
      complex(dop), dimension(dgldim),intent(out)  :: psi
      complex(dop), dimension(agmatdim),intent(in) :: agmat
      logical(kind=4)                              :: lshort,ltest,lopen
      character(len=c5)                            :: filename

!-----------------------------------------------------------------------
! If azahl > 0, read and normalise desired wavefunction coefficients
!-----------------------------------------------------------------------
      if (azahl .gt. adim ) then
         routine='Rdkoeff (gencoeff.f90)'
         message='Reading more A-coefficients than allowed'
         call errormsg
      endif

!-----------------------------------------------------------------------
! Zero A-coefficients
!-----------------------------------------------------------------------
      psi(1:adim) = 0.0_dop

!-----------------------------------------------------------------------
! Open input file for reading
!-----------------------------------------------------------------------
      lopen=.false.
      inquire(iin,opened=lopen)
      if (lopen) close(iin)

      filename=inputfile(1:inplaenge)
      open(iin,file=filename,status='old',form='formatted',err=900)

! read down to input
      rewind(iin)
      do 
         read(iin,'(a)',end=20) InpTit(1:60)
         call rmlblnk(inptit)
         call keylowc(inptit)
         if (inptit(1:8) .eq. 'a-coeff ') exit
      end do

! look for keywords
      read(iin,'(a)',end=25) InpTit(1:60)
      call rmlblnk(inptit)
      call keylowc(inptit)
      if (inptit(1:11) .eq. 'equalweight ') then
         s=isbaspar(4,feb,1)
         if (lmult) then
            psi(zpsi(s):zpsi(s)+block(s)-1) = 1.0_dop
         else
            call eqweight(psi,vdim(meb,1),dim(meb,1),ndim(meb,1))
         endif

! else read input
      else
         backspace(iin)

! If ML only deal with the modes in the top layer
        if (psitype .eq. 5) then
           nm = tree%nmodes
        else
           nm = nmode
        endif
      
!......Test for long or short form of input.
         lshort = .true.
         if((nm*max(1,dentype) .le. 32)) then
! get no. of entries on line
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            backspace(iin)

! remove koeff value from count (if x,y then complex nos. input)
            if (keyword(ic-1) .eq. ',') then
               ic = ic-3
            else
               ic = ic-1
            endif

! get no. of indices
            k = nm
            if (lmult) k = k + 1 
            if (dentype.eq.1 .and. lmult)  k = k + 1
            if (dentype.eq.2)  k = 2*k
            if (ic .eq. k) lshort = .false.
         endif

         do i=1,azahl
            ltest=.false.
            s=1
            t=1
            if (lmult .and. lshort) then
               if (dentype.eq.0)  then
                  read(iin,*,err=30) b,s,acoeff
               else
                  read(iin,*,err=30) b,s,t,acoeff
               endif
            elseif (lshort) then
               read(iin,*,err=30) b,acoeff
            elseif (lmult) then
               if (dentype.eq.0)  then
                  read(iin,*,err=30) (indx(m),m=1,nm),s,acoeff
               elseif (dentype.eq.1)  then
                  read(iin,*,err=30) (indx(m),m=1,nm),s,t,acoeff
               elseif (dentype.eq.2)  then
                  read(iin,*,err=30) (indx(m),m=1,2*nm),s,t,acoeff
               endif
            else
              if (dentype.eq.0 .or. dentype.eq.1)  then
                  read(iin,*,err=30) (indx(m),m=1,nm),acoeff
               elseif (dentype.eq.2)  then
                  read(iin,*,err=30) (indx(m),m=1,2*nm),acoeff
               endif
            endif

            if(.not.lshort) then
               k = 1
               b = indx(1)
               if (psitype .eq. 5) then
                  if(indx(1).gt.tree%ndim(1) .or. indx(1).le.0) ltest=.true.
               else
                  if(indx(1).gt.dim(1,s) .or. indx(1).le.0) ltest=.true.
               endif
               do m = 2, nm
                  if (psitype .eq. 5) then
! no need to differentiate for dentype
                     k = k*tree%ndim(m-1)
                  else
                     if ((dentype.eq.0).or.(dentype.eq.2)) then
                        k = k*dim(m-1,s)
                     else if (dentype.eq.1) then
                        k = k*ddim(m-1,s,t)
                     endif
                  endif
                  b = b + (indx(m)-1)*k
                  if (psitype .eq. 5) then
                     if(indx(m).gt.tree%ndim(m) .or. indx(m).le.0) ltest=.true.
                  else
                     if(indx(m).gt.dim(m,s) .or. indx(m).le.0) ltest=.true.
                  endif
               enddo
               if (dentype.eq.2) then
!    Not sure correct here - hence error. If encountered, check and remove
!    error message
                  routine = 'Gencoeff'
                  message = 'Check code for reading A-coeffs dentyp = 2'
                  call errormsg
                  if (psitype .eq. 5) then
                     k = k*tree%ndim(nm)
                     b = b + (indx(nm)-1)*k
                     do m = 1,nm-1
                        k = k*tree%ndim(m)
                        b = b + (indx(nm+m)-1)*k
                     enddo
                  else
                     k = k*dim(nm,s)
                     b = b + (indx(nm)-1)*k
                     do m = 1,nm-1
                        k = k*dim(m,s)
                        b = b + (indx(nm+m)-1)*k
                     enddo
                  endif
               endif
            endif

            mblock = 0
            if (dentype.eq.0) then
               mblock = block(max(1,min(s,nstate)))
            else if (dentype.eq.1) then
               mblock=dblock(max(1,min(s,nstate)),max(1,min(t,nstate)))
            else if (dentype.eq.2) then
               mblock = block(max(1,min(s,nstate)))
               mblock=mblock*mblock
            endif

            if(b.le.0 .or. s.le.0 .or. s.gt.nstate .or. t.le.0 .or. &
                 t.gt.nstate .or. b.gt.mblock .or. ltest) then
               write(6,*) b,s,t,acoeff,lshort
               routine='Rdkoeff'
               message = 'Wrong input for A-coefficients. Check indices!'
               call errormsg
            endif
            if (dentype .eq. 0) then
               b=zpsi(s)+b-1
            else 
               b=dzpsi(s,t)+b-1
            endif

            psi(b) = acoeff

         enddo
      endif ! end of read A-coeffs

! normalise new coefficients
      if(lreflex) then
         do p=1,2
            pp = (p-1)*adim/2 + 1
            call normvxz(psi(pp),norm,adim/2) ! compute norm
            if(norm.le.1.d-6) then
               routine='rdkoeff (gencoeff.F line 155)'
               write(message,'(a,i2,a,e11.4)')&
               'The norm of the initial WF (el.st.=',p,&
               ') is too small. norm =',norm
            call errormsg
            endif
            call xvixdzo(norm,psi(pp),adim/2) ! normalise
         enddo

      else if(.not. lgwp) then
         call normvxz(psi,norm,adim)  ! compute norm
         if(lmulpack) norm=norm/sqrt(dble(npacket))
         if(mpb.gt.0) norm=norm/sqrt(dble(npackts))
         if(norm.le.1.d-6) then
            routine='rdkoeff (gencoeff.F line 168)'
            write(message,'(a,e11.4)')&
            'The norm of the initial WF is too small. norm =',norm
         call errormsg
         endif
         call xvixdzo(norm,psi,adim)  ! normalise

      elseif(dentype .eq. 0) then
         norm=0.0_dop
         do s=1,nstate
            do j=zpsi(s),zpsi(s)+block(s)-1
               do j1=zpsi(s),zpsi(s)+block(s)-1
                  b=j-zpsi(s)
                  b1=j1-zpsi(s)
                  ove=dcmplx(1.0_dop,0.0_dop)
                  do m=1,nmode
                     e=mod(b,dim(m,s))+1
                     b=(b-e+1)/dim(m,s)
                     e1=mod(b1,dim(m,s))+1
                     b1=(b1-e1+1)/dim(m,s)
                     if(gwpm(m)) then
                        gdof=nspfdof(m)
                        ove=ove*fgaover1(&
                            agmat(zphase(m,s)+e-1),&
                            agmat(zphase(m,s)+e1-1),&
                            agmat(zxivec(m,s)+gdof*(e-1)),&
                            agmat(zxivec(m,s)+gdof*(e1-1)),&
                            agmat(zagmat(m,s)+gdof**2*(e-1)),&
                            agmat(zagmat(m,s)+gdof**2*(e1-1)),&
                            gdof)
                     else
                        if(e .ne. e1) ove=dcmplx(0.0_dop,0.0_dop)
                     endif
                  enddo
                  norm=norm+dble(dconjg(psi(j))*ove*psi(j1))
               enddo
            enddo
         enddo
         norm=dsqrt(norm)
         if(lmulpack) norm=norm/sqrt(dble(npacket))
         call xvixdzo(norm,psi,adim)   ! normalise

      else
         routine='Rdkoeff'
         write(6,'(/a)') 'ERROR: density operator not allowed here!'
         message='An A-coefficient build block for GWP-functions'&
                 //' is not compatible with a density operator run.'
         call errormsg
      endif

      write(ilog,*)
      write(ilog,'(a)') '**** A-coefficients taken from input ****'
      do i = 1, adim
         if( abs(psi(i)) .gt. tiny ) then
            if (psitype .eq. 5) then
               call mltop_decoder(tree,i,indx,sindx)
               nm = tree%nmodes
            else
               call adecoder(i,indx,sindx)
               nm = nmode
            endif
            if(nstate.gt.1) then
               write(ilog,'(i6,"  (",f9.6,",",f9.6,")",a,i3,", ",35i3)')&
                   i,psi(i),'   s =',sindx,(indx(m),m=1,nm)
            else
               write(ilog,'(i6,"  (",f9.6,",",f9.6,")",2x,35i3)')&
                   i,psi(i),(indx(m),m=1,nm)
            endif
         endif
      enddo
      write(ilog,*)

      Call MyFlush(ILog)
      close(iin)

      return

   20 routine='Rdkoeff'
      message= 'End of file found before A-coeff block'
      call errormsg

   25 routine='Rdkoeff'
      message= &
         'Missing "end-a-coeff"? End of file found while reading A-coeff block'
      call errormsg

   30 routine='Rdkoeff'
      message='Error reading A-coefficients'
      backspace iin
      read(iin,'(a)') InpTit(1:60)
      i = index(InpTit(1:60),'#')
      write(6,'(a)') ' ----------------------------------------------'
      if(i.gt.0) write(6,'(a)') 'No comment lines within A-coeff block!'
      write(6,'(2a)') 'Wrong line: ', InpTit(1:60)
      write(6,'(a,l2,a,l2)') ' lshort =',lshort,'   lmult =',lmult
      if(lshort) then
         write(6,'(a,3i4,1x,2e11.3)')&
                 'nmode,b,s,acoeff:',nmode,b,s,acoeff
      else
         write(6,'(a,2i4,1x,2e11.3)')&
                 'nmode,s,acoeff:',nmode,s,acoeff
         write(6,'(a,(20i4/))')&
                 'indx:',(indx(i),i=1,nmode)
      endif

      call errormsg

  900 routine='Rdkoeff (gencoeffmod.F90)'
      ilbl=index(filename,' ')-1
      message = 'Error opening file: '//filename(1:ilbl)
      call errormsg

      end subroutine rdkoeff

!#######################################################################

      subroutine eqweight(psi,vdim1,dim1,ndim1)

      implicit none

      integer(long), intent(in) :: vdim1,dim1,ndim1
      complex(dop), dimension(vdim1,dim1,ndim1), intent(out) :: psi
      integer(long) :: v,n

! initially populated state is in "state = 1" due to swap of electronic
! spfs
      do v = 1,vdim1
         do n = 1,ndim1
            psi(v,1,n) = 1.0_dop
         enddo
      enddo

      return
      end subroutine eqweight

!#######################################################################
!
! GENJINDX
!
! generates the index vector for an S-MCTDH calculation
!
! jindx(b) : index for b th A-coefficient
! jvalue   : index value
!
!#######################################################################

      subroutine genjindx(jindx,cicut,vdim,dim,ndim,jvdim,jdim, &
                 jindxdim,zpsi,ciweight)

      implicit none

      integer(long)                                        :: jvalue,v,e,n,m,&
                                                              mindx,s
      integer(long), intent(in)                            :: jindxdim
      integer(long), dimension(jindxdim),intent(out)       :: jindx
      integer(long), dimension(maxsta)                     :: b
      integer(long), dimension(maxdim)                     :: j
      integer(long), dimension(maxdim,maxsta), intent(in)  :: dim
      integer(long), dimension(maxdim,maxsta), intent(out) :: vdim,ndim
      integer(long), dimension(maxdim), intent(in)         :: jvdim,jdim
      integer(long), dimension(maxsta), intent(in)         :: zpsi

      real(dop)                               :: weight
      real(dop), intent(in)                   :: cicut
      real(dop), dimension(maxsta)            :: jsum
      real(dop), dimension(maxdim),intent(in) :: ciweight

      call zerovxi(j,nmode)
      do s=1,nstate
         b(s)=zpsi(s)-1
         do m=1,nmode
            vdim(m,s)=0
            ndim(m,s)=0
         enddo
      enddo

      jvalue=0

   10 continue
      jvalue=jvalue+1
      do s=1,nstate
         jsum(s)=0.0_dop
      enddo
      do m=1,nmode
         do s=1,nstate
            if (j(m) .lt. dim(m,s)) then
               if (dim(m,s) .gt. 1) then
                  weight=dble(ciweight(m)/(dim(m,s)-1))
               else
                  weight=1.0
               endif
               jsum(s)=jsum(s)+weight*j(m)
            else
               jsum(s)=jsum(s)+2.0*cicut
            endif
         enddo
      enddo

      do s=1,nstate
         if (jsum(s) .le. cicut) then
            b(s)=b(s)+1
            jindx(b(s))=jvalue

            do m=1,nmode
               call decompj(jvalue,v,e,n,jvdim(m),jdim(m))
               if (n .eq. 1 .and. e .eq. 1) then
                  vdim(m,s)=vdim(m,s)+1
               endif
               if (v .eq. 1 .and. e .eq. 1) then
                  ndim(m,s)=ndim(m,s)+1
               endif
            enddo
         endif
      enddo

      mindx=1
   20 continue
      if (j(mindx) .lt. (jdim(mindx)-1)) then
         j(mindx)=j(mindx)+1
         go to 10
      else if (mindx .lt. nmode) then
         j(mindx)=0
         mindx=mindx+1
         go to 20
      endif

      return
      end subroutine genjindx

!***********************************************************************
!
!     DMCTDH   subroutine THERMALIZE
!
!     Thermalizes the initial coefficients for given temperature, only
!     for HO or gauss valid !!!
!
!
!*********************************************************************** 

      subroutine thermalize2(psi,s,s1)

      implicit none

      integer(long), intent(in)             :: s,s1
      integer(long)                         :: f,m,n,i,j,ex,np,ilbl,f1
      integer(long), dimension(maxdim)      :: indx
      real(dop)                             :: energy,gmass
      real(dop), dimension(d3matdim)        :: factor
      real(dop), dimension(maxdim)          :: freq
      logical(dop), dimension(maxdim)       :: lfound 

      complex(dop)                                 :: width
      complex(dop), dimension(dgldim), intent(out) :: psi
      logical(kind=4), save  :: firstcall = .true.

      if (firstcall) then
         write(ilog,*) 
         write(ilog,'(50a)') ('*',i=1,50)
         write(ilog,'(a)') '**** Thermalization'
         write(ilog,*)
         write(ilog,'(a,f20.15)') 'Temperature [a.u.] = ',temperature0
         write(ilog,*)
         firstcall = .false.

         temperature_psi = temperature0
      endif

      do m=1,nmode
         if (dim(m,s) .ne. dim(m,s1)) then
            routine='thermalize'
            message='Numbers of SPFs for right_state and '// &
                 'left_state must be equal.'
            call errormsg
         endif
      enddo

!-----------------------------------------------------------------------
! infinite temp.
!-----------------------------------------------------------------------
      if (temperature0 .gt. 99999990.0) then

         if (lrunml) call set_hartree_avec(psi,tree)
         call unitqxz(psi(dzpsi(s,s1)),block(s))
 
         go to 900
      endif

      freq(:) = 0.0_dop
      lfound(:) = .false.
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            if (f .eq. feb) cycle

!     Check basis set
            if (sbasis(f).ne.1) then
               message = 'T>0 implemented only for Gauss or HO'
               call errormsg
            endif

! get frequency of SPF basis 
! width is sqrt(2*mass*freq)
            width = dcmplx(rsbaspar(3,f,1),rsbaspar(4,f,1))      
            gmass = rsbaspar(5,f,1)
            freq(f)  = 0.5_dop * dble(width)**2 /gmass

            if (ltemp_mfsc) then
               do np=1,npar
                  if (apar(np)(1:5) .eq. 'freq0') then
                     ilbl=index(apar(np),' ')-1
                     read(apar(np)(7:ilbl),*,err=901) f1
                     if (f1 .eq. f) then
                        freq(f) = rpar(np) * freq(f)
                        lfound(f) = .true.
                     endif
                  endif
               enddo
               if (.not. lfound(f)) then
                  routine = 'Thermalize2 (gencoeffmod.f90)'
                  write(message,'(a,i5)') &
                      'Frequency not found in parameters for DOF: ',f
                  call errormsg
               endif
            endif
         enddo
      enddo

      write(ilog,*)
      do m=1,nmode
         do n=1,nspfdof(m)
            f = spfdof(n,m)
            write(ilog,'(a,i5,f10.6)') 'Frequency [a.u.] for DOF ',f,freq(f)
         enddo
      enddo
      write(ilog,*)

      f = 1
      do m=1,nmode

! special case: Electronic grid populates first state only.
   !     f = spfdof(1,m)
   !     if (f .eq. febgrid) then
   !        factor(d3mat(m,1)) = 1.0
   !        do i=2,dim(m,s)
   !           factor(d3mat(m,1)-1+i) = 0.0
   !        enddo
   !        cycle
   !     endif

        write(ilog,'(a,i4)') 'MCTDH Mode : ',m
        write(ilog,'(3x,a,f10.8)')'spf =  1 , thermalization-factor = ', &
              1.0_dop 

!     Calculate weights and thermalize, note that the groundstate-
!     energy is divided out
         if (nspfdof(m).eq.1) then
            f = spfdof(1,m)
!     Uncontracted mode
            factor(d3mat(m,1)) = 1.0_dop
            do i=2,dim(m,s)
               factor(d3mat(m,1)-1+i) = exp( - (i-1)*freq(f)/temperature0 )
               write(ilog,'(3x,a,i2,a,f10.8)') 'spf = ',i, &
                    ' , thermalization-factor = ',factor(d3mat(m,1)-1+i)
            enddo
         else
!     Contracted mode
            ex = 0
            do j=1,nspfdof(m)
               indx(j) = 1
            enddo
            factor(d3mat(m,1)) = 1.0_dop
            do i=2,dim(m,s)
               call excite2(nspfdof(m),ex,indx)               
               energy = 0.0_dop
               do j=1,nspfdof(m)
                  f = spfdof(j,m)
                  energy = energy + (indx(j)-1)*freq(f)
               enddo
               factor(d3mat(m,1)-1+i) = exp( -energy/temperature0 )
               write(ilog,'(3x,a,i2,a,f10.8)') 'spf = ',i, &
                    ' , thermalization-factor = ',factor(d3mat(m,1)-1+i)
            enddo
         endif
      enddo

      if (lrunml) then
! use type(I) trace vectors as storage for thermalization factors. 
         call allocate_mtraces_matrices2(tree,factor)
         call thermalize2ml(psi,tree)
         call deallocate_mtraces_matrices(tree)

      else if (lmult) then
!     Initialize A matrix
         call unitqxz(psi(dzpsi(s,s1)),block(s))
         if (dlstate .ne. drstate) &
                  call xvxxdzo(0.5d0,psi(dzpsi(s,s1)),block(s)*block(s1))

         do m=1,nmode
            do i=1,dim(m,s)
               call multfac(psi(dzpsi(s,s1)),vdim(m,s), &
                    dim(m,s),ndim(m,s),i,factor(d3mat(m,1)-1+i))
            enddo
         enddo

      else if (leb) then
!     Initialize A matrix, s=s1=1
         call unitqxz(psi(dzpsi(s,s1)),block(s))
         do m=1,nmode
            do i=1,dim(m,s)
               call multfac(psi(dzpsi(s,s1)),vdim(m,s), &
                    dim(m,s),ndim(m,s),i,factor(d3mat(m,1)-1+i))
            enddo
         enddo
         call seteb_den2t(psi(dzpsi(1,1)),dim(meb,1),vdim(meb,1),ndim(meb,1))

      endif 

  900 continue
!     Make hermitian
      if (s .ne. s1) then
         do i=0,(block(s)*block(s1)-1)
            psi(dzpsi(s1,s)+i) = psi(dzpsi(s,s1)+i)
         enddo
      endif

      write(ilog,*) 
      write(ilog,'(40a)') ('*', i=1,40)
      write(ilog,*)

      return
  901 routine='Thermalize2'
      message = "Frequencies required in format freq0_f "
      message = message(1:39)//"where f is in integer"
      call errormsg
      end subroutine thermalize2

!***********************************************************************

      subroutine multfac(psi,vdim,dim,ndim,n,factor)

      implicit none

      integer(long)             :: i,j
      integer(long), intent(in) :: vdim,dim,ndim,n
      real(dop), intent(in)     :: factor

      complex(dop),dimension(vdim,dim,ndim,vdim,dim,ndim),intent(inout) :: psi

      do j=1,ndim
         do i=1,vdim
            psi(i,n,j,i,n,j) = psi(i,n,j,i,n,j)*factor
         enddo
      enddo

      return
      end subroutine multfac

!#######################################################################
! 
! set up coherence between 2 states: dlstate and drstate 
! 
!#######################################################################

      subroutine seteb_den2(psi,dim1,vdim1,ndim1)

      implicit none

      integer(long), intent(in)   :: vdim1,dim1,ndim1
      complex(dop), dimension(vdim1,dim1,ndim1,vdim1,dim1,ndim1) , &
                    intent(out)   :: psi

      psi(1,dlstate,1,1,dlstate,1) = 0.5_dop
      psi(1,drstate,1,1,drstate,1) = 0.5_dop
      psi(1,dlstate,1,1,drstate,1) = 0.5_dop
      psi(1,drstate,1,1,dlstate,1) = 0.5_dop

      return
      end subroutine seteb_den2

      subroutine seteb_den2t(psi,dim1,vdim1,ndim1)

      implicit none

      integer(long), intent(in)   :: vdim1,dim1,ndim1
      integer(long)               :: s,s1
      complex(dop), dimension(vdim1,dim1,ndim1,vdim1,dim1,ndim1) , &
                    intent(out)   :: psi

! zero out all the coefficients for non-populated states.
      if (dlstate .ne. drstate) then
         psi = 0.5d0*psi
         do s=1,gdim(feb)
            if (s .ne. dlstate .and. s .ne. drstate) psi(:,s,:,:,s,:) = 0.0
            do s1=1,s-1
               if (s .ne. dlstate .or. s1 .ne. drstate) then
                  psi(:,s1,:,:,s,:) = 0.0
                  psi(:,s,:,:,s1,:) = 0.0
               endif
            enddo
         enddo

       else
         do s=2,gdim(feb)
            psi(:,s,:,:,s,:) = 0.0
            do s1=1,s-1
               psi(:,s1,:,:,s,:) = 0.0
               psi(:,s,:,:,s1,:) = 0.0
            enddo
         enddo

       endif

      return
      end subroutine seteb_den2t

! ----------------------------------------------------------------------------

    subroutine thermalize2ml(psi,tree)
        implicit none
        ! Compute thermalisation factors for a type II density matix,
        ! set up initial layer vectors
        ! the factors for the primitive layer SPFs are already
        ! calculated and stored in the cnode%mtrace vectors
        complex(cdbl),intent(out)       :: psi(:)
        type(wfnode_t),intent(inout)    :: tree
        type(wfnode_t),pointer          :: cnode
        integer                         :: i

        do i=1,tree%tot_nodes
            cnode => tree%swap_up(i)%p
            if(cnode%isprimitive) cycle

            call compute_tfactors_node(psi,cnode)
        end do

    end subroutine thermalize2ml

      subroutine compute_tfactors_node(psi,cnode)
        implicit none
        complex(cdbl),intent(out)       :: psi(:)
        type(wfnode_t),intent(inout)    :: cnode

        type(wfnode_t),pointer          :: child
        integer,allocatable             :: indx(:)
        real(dbl),allocatable           :: weights(:)
        integer                         :: j,f,avdim,f1,g
        integer(long), dimension(3)     :: ijk

        do f=1,cnode%nmodes
           child  => cnode%submodes(f)%p
           if (child%isprimitive) cycle

           avdim = product(child%ndim)
           allocate(weights(avdim))
           allocate(indx(avdim))
           weights(:) = 1.0_dop

           do f1=1,child%nmodes
              ijk = ijk_shape(f1,child%ndim)
              call dtxxddo(child%mtraces(f1)%p%rvector,weights,ijk(1),ijk(2),ijk(3))

            end do

! sort in order of weights to select most important
            call ibubble_down(weights,indx,avdim)
! store child spf weights in cnode
            do j=1,child%navecs
               cnode%mtraces(f)%p%rvector(j) = weights(j)
            enddo

! set spfs of child to be most important configuration
            psi(child%z(1,1):child%z(child%navecs,2)) = CZERO
            do j=1,child%navecs
               g=indx(j)
               psi(child%z(j,1)+g-1) = CONE
            enddo
                     
            deallocate(weights,indx)
        end do

! For top node now fill the top layer density matrix coefficients
! No need to sort weights as most important configs are in SPFs
        if (cnode%istop) then
           avdim = product(cnode%ndim)
           allocate(weights(avdim))
           allocate(indx(avdim))
           weights(:) = 1.0_dop
           do f=1,cnode%nmodes
              ijk = ijk_shape(f,cnode%ndim)
              call dtxxddo(cnode%mtraces(f)%p%rvector,weights,ijk(1),ijk(2),ijk(3))
            end do

            call filldiag(psi,weights,block(1))

            deallocate(weights,indx)

        endif
      end subroutine compute_tfactors_node

      subroutine filldiag(psi,weights,block1)

      implicit none

      integer(long)                           :: e
      integer(long), intent(in)               :: block1
      real(dbl),dimension(block1), intent(in) :: weights
      complex(dbl),dimension(block1,block1), intent(out) :: psi

      do e =1,block1
         psi(e,e) = weights(e)
      enddo

      end subroutine filldiag

      end module gencoeffmod

