!***********************************************************************
! 
!            MCTDH MODULE GENPHI1
!                
! contains : gendisc,gengauss,xlocgauss,clgauss,gengwp,gwppq,rdgwp,genaleg,
! genkleg,check,excite3,excite4,excite5,genk,genkgwp,getgpara,enum1,enum,
! gensphfbr,genhigher,excite1,excite2,multspf,swapsopf,ddhigher

! not in use gwppq,swapsopf
!
!-----------------------------------------------------------------------
! basis(f) : basis type for dof f. (dof=degree of freedom)
!(phifbr), el, HO, Leg, sin, FFT, exp, sphfbr, kleg, k, pleg, rHO, Leg/r
!   -1      0   1    2    3    4    5      6    7    8    9    11   12  
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
! external, cos, Lagu1, Lagu2, Lagu3, Lagu4 Point KLeg/R PLeg/R Wigner
!    13      14    15     16     17     18   19    20     21      33
!***********************************************************************

      module genphi1

use decimal,         only: long, dop
use global
use logdat
use keyunitsmod
use xvlib,           only: xvixdzo, xvixddo, xvxxdzo, vvaxzz, xvxxzzr, qvxxzz
use schmidtorthomod, only: schmidtortho
use op1lib,          only: sum2vxd, cpqxz, cpvxz, zerovxz, zerovxi, zeromxz, &
                           normvxz, cpmxz, cpvzdz, cpvidz, cpmxd
use mtlib,   only: dtxxdz,mttxzz
use gwptrjmod, only: gwptrj_pq
use gwplib, only: pqtoxivec
use channels
use dvrdatmod, only: ipbaspar, rpbaspar
use d2tens, only: mkherm

      implicit none

      private
      public :: gendisc,gengauss,xlocgauss,clgauss,gengwp,rdgwp,genaleg,genkleg,&
                check,genkgwp,genrgwp,gentgwp,gensphfbr,genhigher,excite1,&
                excite2,multspf,ddhigher,swapspf,genwigner,kclearspf,gentdpoint,&
                inventspfs,mgenhigher,kgenhigher,kgenhigher2,genrdgwp,&
                genreadspf,geneuclid,genran

      contains
!***********************************************************************
! 
!            MCTDH subroutine GENDISC
!                
! 
! sets up single particle functions for a discrete basis set. 
! The spfs are a complete set of orthonormal vectors. 
! This routine is used to set up the electronic basis. 
!
! variables passed for one mode:
!  psi: single particle functions (states)
!  dim: number of spfs (states)
!  gdim: as dim
!
!***********************************************************************

      subroutine gendisc (psi,dim,gdim)

      implicit none

      integer(long)                                  :: e
      integer(long), intent(in)                      :: dim,gdim
      complex(dop), dimension(gdim,dim), intent(out) :: psi
      
      psi(:,:)=(0.0_dop,0.0_dop)
      do e=1,min(dim,gdim)
         psi(e,e)=(1.0_dop,0.0_dop)
      enddo

      return
      end subroutine gendisc 

!***********************************************************************
! GENREADSPF
!***********************************************************************

      subroutine genreadspf(psi,gdim,dim,fname,basis,ort,trafo,&
                           ipbaspar,ipbaspar1,rpbaspar,swtch,f)
      implicit none

      integer(long), intent(in)                  :: gdim,dim,swtch,&
                                                    basis,f
      integer(long), dimension(*), intent(in)    :: ipbaspar,ipbaspar1
      real(dop),dimension(gdim), intent(in)      :: ort
      real(dop),dimension(gdim,gdim), intent(in) :: trafo
      real(dop),dimension(gdim)                  ::  weight
      real(dop),dimension(6), intent(in)         ::  rpbaspar
      complex(dop), dimension(gdim,dim), intent(out) :: psi
      character(len=c5)                          :: fname
      character*8    buf
      integer(long) :: ierr,g,next,power,isym,dim1,ggdim,k,kk
      real(dop)     :: re,im,shift,gxeq,norm

      routine = "genreadspf"

      open(idump,file=trim(fname),form='formatted',status='old',iostat=ierr)
      if (ierr.ne.0) then
         message = 'error opening file "'//trim(fname)//'"'
         call errormsg
      endif

      k=0
 25   k=k+1
      do g=1,gdim
         read(idump,*,err=100,end=200) re,im
         psi(g,k) = DCMPLX(re,im)
      enddo
      kk=k
      if(kk.eq.dim) goto 20
      read(idump,*,err=20,end=20) buf
      call rmlblnk(buf)
      if(buf .eq. 'next') goto 25
 20   continue
      write(ilog,'(a,i4,a,i4)') 'readspf:',kk,' SPFs read for DOF =',f

!-----Multiply the SPF with weight^{1/2} if "add-weight" is set.
      if(swtch.eq.1) then
         ggdim=gdim
         if (basis.eq.12) ggdim=ipbaspar(3)
         call dvrweights(trafo,ort,gdim,ggdim,weight,basis,&
                  rpbaspar,ipbaspar,ipbaspar1,.false.,.false.)
         do k=1,kk
            do g=1,gdim
               psi(g,k) = psi(g,k)*weight(g)
            enddo
         enddo
      endif

      if(kk.eq.1) then
         call normvxz(psi,norm,gdim) ! compute norm
         call xvixdzo(norm,psi,gdim) ! normalise
      else
         call schmidtortho(psi,gdim,kk,ierr)
      endif

!-----------------------------------------------------------------------
! Set up orthonormal higher initial single-particle functions:
!  next is a parameter to say that 2nd spf is first function to be
!  generated
!-----------------------------------------------------------------------
      dim1=min(dim,gdim)
      if(basis .eq. 7 .or. basis .eq. 9 .or. basis .eq. 33) then
         dim1=(dim1+3)/4    ! Kleg, Pleg, Wigner. Higher functions
      endif                 ! done in kgenhigher or wiggenhigher
      if(kk.ge.dim1) goto 30
      power=1
      next=kk+1
      isym=0
      shift=0.d0
      gxeq=(ort(1)+ort(gdim))*0.5d0
      call genhigher(psi,gdim,dim1,next,ort,basis,ipbaspar,power,&
                     gxeq,shift,isym)
      call schmidtortho(psi,gdim,dim1,ierr)
      if(ierr.ne.0) then
         write(6,*) 'genreadspf: Generation of higher SPFs failed.'
         write(6,*) 'ierr =', ierr
         write(ilog,*) 'genreadspf: Generation of higher SPFs failed.'
         write(ilog,*) 'ierr =', ierr
         stop
      endif

  30  close(idump)
      return

 100  message = 'error reading initial SPF from file '//trim(fname)//'"'
      routine='genreadspf (genphi1.F)'
      call errormsg

 200  message = 'premature EOF when reading initial SPF from file "'//&
                trim(fname)//'"'
      routine='genreadspf (genphi1.F)'
      call errormsg

      end subroutine genreadspf

!***********************************************************************
!
!             MCTDH  subroutine  GENEUCLID
!
!  05/19  HDM
!
!***********************************************************************

      subroutine geneuclid(psi,gdim,dim,point)

      implicit none

      integer(long), intent(in) :: gdim,dim,point
      integer(long)             :: sft,gg,e
      complex(dop), dimension(gdim,dim), intent(out) :: psi

      call  zeromxz(psi,gdim,dim)
      psi(point,1)=1.d0

      sft=0
      do e=2,dim
         if(sft.ge.0) sft=sft+1
         gg=point+sft
         if(gg.le.0)    gg=gg+gdim
         if(gg.gt.gdim) gg=gg-gdim
         psi(gg,e)=1.d0
         sft=-sft
      enddo

      return
      end subroutine geneuclid

!***********************************************************************
!
!             MCTDH  subroutine  GENRAN
!
!     Generation of a MCTDH thermal state
!     See "thermal" keyword
!
!  06/18  DMT
!
!***********************************************************************

      subroutine genran(psi,gdim,dimfr,mlseed)

      implicit none

!     Wavefunction
      integer(long)              ::    e,g
      integer(long), intent(in)  ::    gdim,dimfr
      complex(dop), dimension(gdim,dimfr), intent(out) :: psi
!     Flags and data from outside
      integer(long), intent(in)  :: mlseed
!     Random number generator
      integer(long)              :: iran
      integer(long), parameter   :: nseed=64
      integer(long)              :: rsize,n
      integer(long), dimension(nseed) :: rseed
      real(dop)                  :: ran
      logical(kind=4), save      :: firstcall=.true.

!-----------------------------------------------------------------------
!     Initialise random seed
!-----------------------------------------------------------------------
!     mlseed default value is set in mdefault.F
!     but can be overwriten with "thermal" keyword
!     (read RUN-SECTION)
      if (firstcall) then
         firstcall=.false.
         call random_seed(size=rsize)
         if (rsize .gt. nseed) then
            write(*,'(a)') "ERROR: subroutine gencoeff"
            write(*,'(a)') "       parameter 'nseed' is too small."
            write(*,'(a,i8)') "       must be at least:,",  rsize
            stop 1
         end if
         call random_seed(get=rseed(1:rsize))
         do n=1,rsize
            rseed(n)= mlseed + n
         end do
         call random_seed(put=rseed(1:rsize))
      endif

!-----------------------------------------------------------------------
!     Average over states
!-----------------------------------------------------------------------
      do e=2,dimfr
         call random_number(ran)
         iran=mod(FLOOR(1.0d3*ran),2)
         do g=1,gdim
            psi(g,1)=psi(g,1)+psi(g,e)*(-1)**iran
         enddo
      enddo

      return
      end subroutine genran

!***********************************************************************
!
!             MCTDH  subroutine  GENGAUSS
!
!  11/96  AJ
!
!***********************************************************************

      subroutine gengauss(psi,gdim,dim,trafo,ort,basis,rpbaspar, &
                 ipbaspar,ipbaspar1,rsbaspar,isbaspar,ggdim,f,nn,isym)

      implicit none

      integer(long)                          :: e,g,next,blz,ibleg,dim1,dim2,typ,power,ierr
      integer(long), intent(in)              :: gdim,dim,basis,ggdim,f,nn,isym
      integer(long), dimension(*),intent(in) :: ipbaspar,ipbaspar1,isbaspar

      real(dop)                                      :: gxeq,gmom,norm,homass,&
                                                        hofreq,hoxeq,ortg,shift
      real(dop), dimension(gdim), intent(inout)      :: ort
      real(dop), dimension(gdim)                     :: weight
      real(dop), dimension(ggdim,gdim), intent(in)   :: trafo
      real(dop), dimension(*), intent(in)            :: rpbaspar,rsbaspar
      complex(dop)                                   :: gwidth
      complex(dop), dimension(gdim,dim), intent(out) :: psi

!-----------------------------------------------------------------------
! get primitive basis (DVR) parameters
!-----------------------------------------------------------------------
      if (basis .eq. 1 .or. basis .eq. 11) then
         hoxeq  = rpbaspar(1)
         hofreq = rpbaspar(2)
         homass = rpbaspar(3)
      else if (basis .eq. 2 .or. basis .eq. 12) then
         blz=ipbaspar(1)
         ibleg=ipbaspar(2)
      else if (basis .eq. 7 .or. basis .eq.20) then
         blz=ipbaspar1(3)
         ibleg=ipbaspar(1)
      else if (basis .eq. 9 .or. basis .eq.21) then
         blz=ipbaspar1(4)
         ibleg=ipbaspar(1)
      else if (basis .eq. 33) then
         ibleg=ipbaspar(1)
      endif

!-----------------------------------------------------------------------
! get DVR weights
!-----------------------------------------------------------------------
      call dvrweights(trafo,ort,gdim,ggdim,weight,basis, &
                      rpbaspar,ipbaspar,ipbaspar1,.false.,.false.)

!-----------------------------------------------------------------------
! get type (i.e. all or only odd / even functions)
!-----------------------------------------------------------------------
      typ=isbaspar(1)
      if(typ.eq.0  .and.  isym.lt.0) typ=1
      if(typ.eq.0  .and.  isym.gt.0) typ=2

!-----------------------------------------------------------------------
! get spf basis parameters
! gwidth = 1/sigma^2 where sigma is STD of density
!-----------------------------------------------------------------------
      gxeq   = rsbaspar(1)
      gmom   = rsbaspar(2)
      gwidth = dcmplx(rsbaspar(3),rsbaspar(4))
      shift  = 0.0_dop 
      if(isbaspar(5).eq.1) shift = ort(2) + ort(gdim) - 2.0_dop*ort(1)

!-----------------------------------------------------------------------
! Multiply initial Gaussian by DVR weights
!-----------------------------------------------------------------------
      do g=1,gdim
         ortg = ort(g)-gxeq
         if( isbaspar(5) .eq. 1 ) then
            if(abs(ortg) .gt. abs(ortg-shift)) ortg = ortg-shift
            if(abs(ortg) .gt. abs(ortg+shift)) ortg = ortg+shift
            if(abs(ortg) .gt. abs(ortg-shift)) ortg = ortg-shift
         endif

         psi(g,1) = weight(g)*exp(-0.25_dop*(ortg*gwidth)**2)
         !To deal with cases where number is very small but should be zero
         if (REAL(psi(g,1),dop) .lt. 10.0_dop*macheps) then
            psi(g,1) = (0.0_dop,0.0_dop)
         endif
         if (typ .eq. 1) then
!.....if odd functions requested, set up x*psi
            psi(g,1)=ortg*psi(g,1)
         endif
      enddo

!-----------------------------------------------------------------------
! Normalise initial gaussian
!-----------------------------------------------------------------------
      call normvxz(psi,norm,gdim)
      if (abs(norm) .lt. tiny) then
         routine='Gengauss(Genphi1)'
         write(message,'(a,i5)') &
              'Generated Gaussian SPF is zero. Mode:', f
         call errormsg
      endif
   
      call xvixdzo(norm,psi,gdim)

!-----------------------------------------------------------------------
! Set up orthonormal higher initial single-particle functions:
!  next is a parameter to say that 2nd spf is first function to be
!  generated
!-----------------------------------------------------------------------
      power=1
      if (typ .gt. 0) power=2
      next=2
      dim1=min(dim,gdim,gdim/power)
      dim2=min(dim1,2+dim/nn)
      if(basis .eq. 7 .or. basis .eq. 9 .or. basis .eq. 33) then
         dim1=(dim1+2)/2    ! Kleg, Pleg, Wigner. Higher functions
         dim2=(dim2+4)/3    ! done in kgenhigher or wiggenhigher
      endif
      call genhigher(psi,gdim,dim1,next,ort,basis,ipbaspar,power, &
                      gxeq,shift,isym)
      call schmidtortho(psi,gdim,dim1,ierr)

      ! DEBUG
!      open(unit=50,file='gengauss',form='formatted',status='unknown')
!      do e=1,dim1
!         do g=1,gdim
!            write(50,'(2i4,g30.16)') e,g,dble(psi(g,e))
!         enddo
!      enddo
!      close(50)
      ! END DEBUG

!-----------------------------------------------------------------------
!     If the gauss is too localized, it is impossible to create a large
!     number of linearly independent spf's. schmidtortho then returns
!     with ierr=number of function that could not be orthonormalized.
!     The failed function is then set equal to its coordinate.
!-----------------------------------------------------------------------
      if( ierr .gt. 0 ) then
         next = ierr+1
         do g=1,gdim
            psi(g,ierr)=(ort(g)-gxeq)**typ
         enddo
         call normvxz(psi(1,ierr),norm,gdim)
         call xvixdzo(norm,psi(1,ierr),gdim)
         call genhigher(psi,gdim,dim1,next,ort,basis,ipbaspar, &
                        power,gxeq,shift,isym)
         call schmidtortho(psi,gdim,dim1,ierr)
         if( ierr .gt. 0 ) then
            if(ierr .gt. dim2 ) goto 50 
            write(6,'(/a,/a,i4,a,/a,i3,a,i4,a,i3,a,i3)') &
           '############# ERROR in gengauss (genphi1.F) #############',  &
           'Cannot Schmidt-orthogonalize the',ierr,'-th spf (build).', &
           ' basis =',basis,',  gdim =', gdim,', dim =',dim,', f =',f

            write(2,'(/a,/a,i4,a,/a,i3,a,i4,a,i3,a,i3)')  &
           '############# ERROR in gengauss (genphi1.F) #############',  &
           'Cannot Schmidt-orthogonalize the',ierr,'-th spf (build).', &
           ' basis =',basis,',  gdim =', gdim,', dim =',dim,', f =',f  
            call stopnow
         end if
      end if



!-----------------------------------------------------------------------
! Multiply by factor for initial momentum 
!-----------------------------------------------------------------------
 50   do e=1,dim
         do g=1,gdim
            psi(g,e)=psi(g,e) &
                 *exp(dcmplx(0.0_dop,gmom*(ort(g)-gxeq)))
         enddo
      enddo

      return
      end subroutine gengauss

!***********************************************************************

      subroutine xlocgauss(psi,subdim,dim,rsbaspar,ort,rpbaspar,ipbaspar)

      implicit none

      integer(long)                          :: e,ndiv
      integer(long), intent(in)              :: subdim,dim
      integer(long), dimension(*),intent(in) :: ipbaspar
      real(dop)                              :: dx,xi,xf
      real(dop), dimension(*), intent(in)    :: rsbaspar,rpbaspar
      real(dop)                              :: gxeq, gmom, norm, pi, xdiff,&
                                                gwidth,hofm,width,deltax

      real(dop), dimension(dim), intent(out)           :: ort
      real(dop), dimension(dim,dim)                    :: workr

      complex(dop), dimension(subdim,dim), intent(out) :: psi

      pi=4.0_dop*atan(1.0_dop)

!-----------------------------------------------------------------------
! get spf basis parameters
!-----------------------------------------------------------------------
      gxeq   = rsbaspar(1)
      gmom   = rsbaspar(2)
!     gwidth = dcmplx(rsbaspar(3),rsbaspar(4))
      gwidth = rsbaspar(3)
      hofm=0.5_dop*gwidth*gwidth

!     hotyp=0
      dx=0.0_dop
      xi=rpbaspar(1)
      xf=rpbaspar(2)
      ndiv=ipbaspar(1)
!
! position and momentum
!
      if (dim .eq. 1) then
         ort(1)=gxeq
         psi(1,1)=ort(1)
         deltax=1
      else
         xdiff=(xf-xi)/(dim-1)
         deltax=xdiff/ndiv
         do e=1,dim
            ort(e)=xi+(e-1)*xdiff
            psi(1,e)=ort(e)
         enddo
      endif
      do e=1,dim
         width=pi/deltax
         psi(2,e)=dcmplx(gmom,width)
      enddo
!
! store weight as imag of psi(3), phase as real part. 
!
      do e=1,dim
         workr(e,1)=pi*mod(e,2)
         workr(e,1)=workr(e,1)+gmom*ort(e)
      enddo
      do e=1,dim
         psi(3,e)=workr(e,1)
      enddo
         
      norm=0.0_dop
      do e=1,dim
         workr(e,1)=exp(-0.25_dop*((ort(e)-gxeq)*gwidth)**2)
         norm=norm+workr(e,1)
      enddo
      do e=1,dim
         workr(e,1)=workr(e,1)/norm
      enddo

      do e=1,dim
         psi(3,e)=psi(3,e)+(0,1)*workr(e,1)
      enddo

      return
      end subroutine xlocgauss

!***********************************************************************

      subroutine clgauss(psi,dim,rsbaspar,pqdistr)

      use randommod, only: randgaussian

      implicit none

      integer(long)                       :: e,n
      integer(long), intent(in)           :: dim
      integer(long), intent(in)           :: pqdistr
      real(dop)                           :: gxeq,gmom,ort,mom,xsigma,psigma,&
                                             gwidth,dpsi,impsi
      real(dop), dimension(*), intent(in) :: rsbaspar
      complex(dop)                        :: gpair
      complex(dop), dimension(1,dim), intent(out) :: psi

!-----------------------------------------------------------------------
! get spf basis parameters
! sigma^2 = 1/gwidth  where sigma is STD of density
!-----------------------------------------------------------------------
      gxeq   = rsbaspar(1)
      gmom   = rsbaspar(2)
      gwidth = rsbaspar(3)

!-----------------------------------------------------------------------
! Scaling for Gaussian width
!-----------------------------------------------------------------------
      xsigma=1.0_dop/gwidth
      psigma=0.5_dop*gwidth

!-----------------------------------------------------------------------
! Set up initial Gaussian 
!-----------------------------------------------------------------------
      if (pqdistr .eq. 0) then
! no random sampling - useful for testing
         
         psi(1,1)=dcmplx(gxeq,gmom)
         n=0
         do e=2,dim,2
            n=n+1
            gpair=0.2_dop*n

            ort=xsigma*dble(gpair)
            mom=psigma*dimag(gpair)

            dpsi=ort+gxeq
            impsi=mom+gmom
            psi(1,e)=dcmplx(dpsi,impsi)
            if (e+1 .le. dim) psi(1,e+1)=-dcmplx(dpsi,impsi)
         enddo

      else if (pqdistr .eq. 1) then
         
         psi(1,1)=dcmplx(gxeq,gmom)
         do e=2,dim
            gpair=randgaussian()

            ort=xsigma*real(gpair)
            mom=psigma*aimag(gpair)

            dpsi=ort+gxeq
            impsi=mom+gmom
            psi(1,e)=dcmplx(dpsi,impsi)
         enddo

      else if (pqdistr .eq. 2) then
         
         psi(1,1)=dcmplx(gxeq,gmom)
         do e=2,dim-3,4
            gpair=randgaussian()

            ort=xsigma*real(gpair)
            mom=psigma*aimag(gpair)

            dpsi=ort+gxeq
            impsi=mom+gmom
            psi(1,e)=dcmplx(dpsi,impsi)
            dpsi=-ort+gxeq
            impsi=mom+gmom
            psi(1,e+1)=dcmplx(dpsi,impsi)
            dpsi=ort+gxeq
            impsi=-mom+gmom
            psi(1,e+2)=dcmplx(dpsi,impsi)
            dpsi=-ort+gxeq
            impsi=-mom+gmom
            psi(1,e+3)=dcmplx(dpsi,impsi)
         enddo

      endif

      return
      end subroutine clgauss

!***********************************************************************
!
!             MCTDH  subroutine  GENGWP
!
!
!    Generates a set of 1D Gaussian wavepackets for a given
!    degree of freedom
!
!***********************************************************************

      subroutine gengwp(psi,dim,rsbaspar,isbaspar,f1)

      use gwplib, only: fgaover1,ffgaover
      use randommod, only: randgaussian

      implicit none

      real(dop), parameter :: tiny=1.0e-8_dop
      real(dop)            :: pi

      integer(long), intent(in)               :: f1
      integer(long)                           :: k,typ,next,idim,typ1,&
                                                 np,nq,typp,iperiod
      integer(long), intent(in)               :: dim
      integer(long), dimension(*), intent(in) :: isbaspar

      real(dop), dimension(*), intent(in) :: rsbaspar
      real(dop)                           :: gxeq, gmom, gshift, norm, gx, &
                                             pshift,gp,xrange,xi,xf,q0,p0
      complex(dop)                                :: gwidth
      complex(dop), dimension(3)                  :: param
      complex(dop), dimension(3,dim), intent(out) :: psi

      pi=4.0_dop*atan(1.0_dop)

!-----------------------------------------------------------------------
! set up of GWPs for a gwptrj run done in genpsi / gentgwp
!-----------------------------------------------------------------------
      if (lgwptrj) return

!-----------------------------------------------------------------------
! get generation type. 
!
! typ = 1    seqex
!       2    simex
!       3    shell
!       4    readmd - done in genpsi
!       5    wigner (random_pq) - done in genpsi
!       6    random
!       7    wigner_sym (symmetric_pq) - done in genpsi
!       8    pqgrid
!
! typp = 0  distribution in coordinates
!        1  distribution in momentum
!-----------------------------------------------------------------------
      typ    = isbaspar(1)
      typp   = isbaspar(3)

! Some distributions are generated on mdGWPs and do not come here.
! If explicitely reading parameters for MD GWPs, this is done in genpsi
      if (typ .eq. 4) then
         return
! If random sampling, this is done in genpsi for the whole multi-D GWP
      else if (typ .ge. 5 .and. typ .le. 7) then
         return
      endif

! pqgrid
      if (typ .eq. 8) then
         nq = isbaspar(5)
         np = isbaspar(6)
         if (dim .ne. np*nq) then
            routine = 'Gengwp'
            write(message,'(a,i7)') &
                 'Nspf must be pqgrid dimension np*nq :',np*nq
            call errormsg
         endif

! ignore typp setting as this is not relevant for a grid.
         typp = 0

      else
         nq = dim
         np = 1
      endif

!-----------------------------------------------------------------------
! check for periodic GWPs
!-----------------------------------------------------------------------
      iperiod = ipbaspar(2,f1)
      xi = rpbaspar(1,f1)
      xf = rpbaspar(2,f1)
      xrange = xf - xi

!-----------------------------------------------------------------------
! get spf basis parameters
! width is passed as 1/sigma to keep same convention as in gengauss
!-----------------------------------------------------------------------
      gxeq   = rsbaspar(1)
      gmom   = rsbaspar(2)
      if (rsbaspar(6) .gt. 0) then
         gwidth = dcmplx(rsbaspar(6),0.0_dop)
      else
         gwidth = dcmplx(rsbaspar(3),rsbaspar(4))
      endif
      gwidth = -0.25_dop*gwidth**2
      
!-----------------------------------------------------------------------
! get shift for new GWP. 
! typ1 = 0 : gshift defined by overlap
!        1 : gshift as given in input
!-----------------------------------------------------------------------
      typ1   = isbaspar(2)
      gshift = -999.0_dop
      pshift = -999.0_dop
      if (typp .eq. 0) then
         if (typ1 .eq. 0) then
            gshift = 2.0_dop*dlog(rsbaspar(5))/dble(gwidth)
            gshift = dsqrt(gshift)
         else if (typ1 .eq. 1) then
            gshift = rsbaspar(5)
         endif
      else if (typp .eq. 1) then
         if (typ1 .eq. 0) then
            pshift = 8.0_dop*dble(gwidth)*dlog(rsbaspar(5))
            pshift = dsqrt(pshift)
         else if (typ1 .eq. 1) then
            pshift = rsbaspar(5)
         endif
      else
         routine = 'GENGWP (genphi1.f90)'
         write(message,'(a,i5)') 'Unknown typp:',typp
         call errormsg
      endif

! for PQgrid set up shift in second axis.
      if (typ .eq. 8) then
         if (typ1 .eq. 0) then
            pshift = 4.0_dop*dble(gwidth)*dlog(rsbaspar(5))
            pshift = dsqrt(gshift)
         else if (typ1 .eq. 1) then
            pshift = rsbaspar(5)
         endif
      endif

!-----------------------------------------------------------------------
! Check periodic GWPS in range
!-----------------------------------------------------------------------
      if (iperiod .ne. 0) then
         if ((gxeq .lt. xi) .or. (gxeq .gt. xf)) then
            routine = 'GENGWP (genphi1.f90)'
            write(message,'(a)') 'Initial GWP outside range for a periodic GWP'
            call errormsg
         endif
      endif

!-----------------------------------------------------------------------
!     generate 3-element 'param' array from spf basis parameters;
!     first, corresponding to 'ground state' Gaussian       
! param(1) = width
! param(2) = q + ip      Start at gxeq,gmom unless q0 not included
! param(3) = phase
!
!-----------------------------------------------------------------------
      param(1) = gwidth

      q0=gxeq
      p0=gmom
      param(2) = dcmplx(q0,p0)

      norm = pi/(-2.0_dop*dble(gwidth))  
      norm = dsqrt(norm)
      norm = -0.5_dop*dlog(norm)+1.0_dop*dble(gwidth)*gxeq**2
!     the imag part of the phase parameter is set to zero; the real 
!     part results from normalization

      param(3) = dcmplx(norm,0.0_dop)

!     store in spf1d : 

      do k=1,3
         psi(k,1) = param(k)
      enddo

!    'excited' configurations for a given degree of freedom: generate
!     via shift in center coordinate

      next = 1

      gx = 0.0
      gp = 0.0
      do idim=2,min(dim,nq)
         if (typp .eq. 0) then
            gx=gxeq + dble(next)*gshift
            gp=gmom
         else if (typp .eq. 1) then
            gx=gxeq 
            gp=gmom + dble(next)*pshift
         endif

         if ((iperiod .ne. 0) .and. (gx .lt. xi)) gx = gx + xrange
         if ((iperiod .ne. 0) .and. (gx .gt. xi)) gx = gx - xrange

         param(2) = dcmplx(gx,gp)
         norm = dsqrt(pi/(-2.0_dop*dble(gwidth))) 
         norm = -0.5_dop*dlog(norm)+1.0_dop*dble(gwidth)*gx**2
         param(3) = dcmplx(norm,0.0_dop)
         psi(:,idim) = param(:)


! ...   implement such that alternate +/- shifts are done 
         if(next.gt.0)then
            next=-next
         else 
            next=abs(next)+1
         endif
      enddo

      if (typ .eq. 8) call genpqgrid(psi,nq,np,pshift)

      return
      end subroutine gengwp

!#######################################################################

      subroutine genpqgrid(psi,nq,np,pshift)

      implicit none

      integer(long), intent(in)                       :: nq,np
      integer(long)                                   :: m,n,next
      real(dop), intent(in)                           :: pshift
      real(dop)                                       :: gp, gp0,gq
      complex(dop), dimension(3,nq,np), intent(inout) :: psi

! get initial momenta from psi
      gp0 = dimag(psi(2,1,1))

      do m=1,nq
         next = 1
         do n=2,np
! generate momenta for grid
            gp = gp0 + dble(next)*pshift

! ...   implement such that alternate +/- shifts are done 
            if(next.gt.0)then
               next=-next
            else 
               next=abs(next)+1
            endif

            psi(1,m,n) = psi(1,m,1)
            gq = dble(psi(2,m,1))
            psi(2,m,n) = dcmplx(gq,gp)
            psi(3,m,n) = psi(3,m,1)
         enddo
      enddo

      return
      end subroutine genpqgrid

! Beginning of modification (Benjamin, 14 Feb 2007)
!***********************************************************************
!
!            MCTDH   subroutine   RDGWP
!
! called from genphi
!
! reads specified 1D GWP-functions from the .inp file
!
! variables passed:
!  spf1d: 1D GWP parameters
!
!***********************************************************************

      subroutine rdgwp(spf1d)

use global
use dvrdatmod
use griddatmod
use psidef
use runinwfmod
use channels

      implicit none

      integer(long)                                  :: mcount,gwpcount,m,f,s,i,s1,ic,iz,ierr,f1,n,e1,b
      real(dop)                                      :: gxeq,gmom,rwidth,cwidth,norm
      complex(dop)                                   :: gwidth
      complex(dop), dimension(spf1ddim), intent(out) :: spf1d

      routine='read build_gwp-func (rdgwp, genphi1.F)'

!-----------------------------------------------------------------------
! Count number of GWP-modes involved in GWP-function build block
! Check consistency of 1D GWP-modes (DOFs) with multi-D GWP-modes
!-----------------------------------------------------------------------
      if(gwpzahl .gt. spf1ddim) then
         write(6,'(/a)') 'ERROR: check GWP-function build block!'
         message='Reading more GWP-functions than allowed'
         call errormsg
      endif
      mcount=0
      gwpcount=0
      do m=1,nmode
         if(sbasis(spfdof(1,m)) .eq. 9) then
            if(nspfdof(m) .gt. 2) then
               do f=1,nspfdof(m)
                  if(sbasis(spfdof(f,m)) .ne. 9) then
                     write(6,'(/a)') 'ERROR: check GWP-function build' &
                                     //' block!'
                     write(message,'(a,i3,a,i3)') '1D mode',spfdof(f,m), &
                                              ' is not consistent with' &
                                              //' multi-D GWP-mode',m
                     call errormsg
                  endif
               enddo
            endif
            mcount=mcount+1
            do s=1,nstate
               gwpcount=gwpcount+dim(m,s)*nspfdof(m)
            enddo
         else
            if(nspfdof(m) .gt. 2) then
               do f=1,nspfdof(m)
                  if(sbasis(spfdof(f,m)) .eq. 9) then
                     write(6,'(/a)') 'ERROR: check GWP-function build' &
                                     //' block!'
                     write(message,'(a,i3,a,i3)') '1D mode',spfdof(f,m), &
                                              ' is not consistent with' &
                                              //' multi-D GWP-mode',m
                     call errormsg
                  endif
               enddo
            endif
         endif
      enddo
      if(gwpcount .ne. gwpzahl) then
         write(6,'(/a)') 'ERROR: check GWP-function build block!'
         message='The number of GWP-functions is not consistent'
         call errormsg
      endif

!-----------------------------------------------------------------------
! Locate GWP-function build block in input file
!-----------------------------------------------------------------------
      rewind(iin)
      do i=1,izgwp
         read(iin,'(a)') InpTit(1:60)
      end do

!-----------------------------------------------------------------------
! Read parameters defining 1D SPFs in GWP-function build block:
!      a = -0.25/sigma^2
!      xi = -2*a*q + i*p
!      the imaginary part of the phase parameter is set to zero
!      the real part results from normalization
! Store in spf1d
!-----------------------------------------------------------------------
      iz=izgwp
      do i=1,gwpzahl
         s1=1
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         call cntrl(keyword(1),lc(1),ierr)
         f1=0
         do f=1,ndof
            if(keyorig(1) .eq. modelabel(f)) f1=f
         enddo
         n=2
         read(keyword(n),*) gxeq
         n=n+1
         if(keyword(n) .eq. ',') then
            n=n+1
            call keylunits(gxeq,keyword(n),ierr)
            n=n+1
         endif
         read(keyword(n),*) gmom
         n=n+1
         read(keyword(n),*) rwidth
         n=n+1
         if(keyword(n) .eq. ',') then
            n=n+1
            call keylunits(rwidth,keyword(n),ierr)
            if(ierr.gt.0) then
               read(keyword(n),*) cwidth
            else
               cwidth=0.0_dop
            endif
            n=n+1
         endif
         if(keyword(n) .eq. ',') then
            n=n+1
            call keylunits(rwidth,keyword(n),ierr)
            call keylunits(cwidth,keyword(n),ierr)
            n=n+1
         endif
         read(keyword(n),*) e1
         n=n+1
         if(lmult) then
            read(keyword(n),*) s1
         endif
         gwidth=dcmplx(rwidth,cwidth)
         gwidth=1.0_dop/gwidth
         gwidth=-0.25_dop*gwidth**2
         b=zspf1d(f1,s1)+(e1-1)*3
         spf1d(b)=gwidth
         spf1d(b+1)=dcmplx(-2.0_dop*dble(gwidth)*gxeq, &
                          (-2.0_dop*dimag(gwidth)*gxeq+gmom))
         norm=pi/(-2.0_dop*dble(gwidth))
         norm=dsqrt(norm)
         norm=-0.5_dop*dlog(norm)+1.0_dop*dble(gwidth)*gxeq**2
         spf1d(b+2)=dcmplx(norm,0.0_dop)
      enddo

      return
      end subroutine rdgwp! of rdgwp
! End of modification (Benjamin, 14 Feb 2007)

!***********************************************************************
!
!             MCTDH  subroutine  GENALEG
!
!
!    Generates a set of the first 'dim' normalized associated
!    Legendre polynomials P_l^n for fixed slz
!
!    10/96 AJ
!
!***********************************************************************

      subroutine genaleg(psi,gdim,dim,nn,trafo,ort,basis,ggdim, &
                         rpbaspar,ipbaspar,ipbaspar1,isbaspar,message)

use lalib,  only: normspf

      implicit none

      integer(long)             :: e,g,dim1,slz,sl0,isleg,ibleg,j,isleg1,ierr
      integer(long)             :: kmin,kmax,kmin2,mmmin,mmmax,mmmin2,usemk
      integer(long), intent(in) :: gdim,dim,basis,ggdim,nn

      integer(long), dimension(*), intent(in)        :: ipbaspar,ipbaspar1,isbaspar
      real(dop), dimension(*), intent(in)            :: rpbaspar
      real(dop), dimension(gdim), intent(in)         :: ort
      real(dop), dimension(gdim)                     :: weight
      real(dop), dimension(ggdim,gdim), intent(in)   :: trafo
      real(dop), external                            :: dlegendre
      complex(dop), dimension(gdim,dim), intent(out) :: psi
      character(len=*), intent(out)                  :: message
      logical(kind=4)                                :: lerr

!-----------------------------------------------------------------------
! get primitive basis (DVR) parameters
!-----------------------------------------------------------------------
      if (basis .eq. 2  .or.  basis .eq. 12) then
         ibleg = ipbaspar(2)
      elseif (basis .eq. 7 .or.  basis .eq. 20) then
         ibleg = ipbaspar(1)
      elseif (basis .eq. 9 .or. basis .eq.21) then
         ibleg = ipbaspar(1)
      elseif (basis .eq. 33) then
         ibleg = ipbaspar(1)
         kmin=ipbaspar(2)
         kmax=ipbaspar(3)
         kmin2=ipbaspar(5)
         mmmin=ipbaspar(6)
         mmmax=ipbaspar(7)
         mmmin2=ipbaspar(9)
      endif

!-----------------------------------------------------------------------
! get spf basis parameters
!-----------------------------------------------------------------------
      slz   = isbaspar(1)
      sl0   = isbaspar(2)
      isleg = isbaspar(3)

!-----------------------------------------------------------------------
! reduce effective dim for combined modes (nn=number of DOFs in mode)
!-----------------------------------------------------------------------
      dim1=max(3,(dim+2)/nn)
      if ((basis.eq.2 .or. basis.eq.12) .and. ibleg.eq.0 &
                                        .and. isleg.gt.0 ) then
         dim1=max(dim1,(gdim+isleg-1)/2)
      else
         dim1=max(dim1,gdim)
      endif
      dim1=min(dim,dim1)

!-----------------------------------------------------------------------
! Determine how Legendre inwf is obtained for Wigner p-basis
!-----------------------------------------------------------------------

      usemk=-999
      if (basis.eq.33) then
         if (slz.ge.mmmin .and. slz.le.mmmax .and. kmin2.eq.0) then
            usemk=1  ! take inwf from trafo, normal phase
         elseif (slz.ge.kmin .and. slz.le.kmax .and. mmmin2.eq.0) then
            usemk=-1 ! take inwf from trafo, reverse phase
         elseif (slz.eq.0) then
            usemk=0  ! calculate inwf using dlegendre function
         else
            usemk=9999 ! slz does not agree with k,m-ranges; call error
         endif
      endif

!-----------------------------------------------------------------------
! get DVR weigts
!-----------------------------------------------------------------------
      call dvrweights(trafo,ort,gdim,ggdim,weight,basis, &
                      rpbaspar,ipbaspar,ipbaspar1,.false.,.false.)

!-----------------------------------------------------------------------
! Check parameters
!-----------------------------------------------------------------------
      lerr = .false.
      message='Illegal input parameters for generation of Leg. func.'
! --- Error if isleg is not 0,1,2, or if pop exceeds 'effective' dim
      if (isleg.eq.0) then
         if (sl0-slz+1.gt.dim1)      lerr = .true.
      elseif (isleg.eq.1) then
         if ((sl0-slz)/2+1.gt.dim1)  lerr = .true.
      elseif (isleg.eq.2) then
         if ((sl0-slz)/2+1.gt.dim1)  lerr = .true.
      else
         lerr = .true.
      endif
      if (lerr) then
         write(6,'(/a,3i4)') '###  sl0,slz,dim1 :',sl0,slz,dim1
         message=message(1:53)//'  (1)'
         call ErrorMsg
      endif

! --- Error if number of spfs requested exceeds pbasis size
      if (dim1.gt.gdim) then
         lerr = .true.
         write(6,'(/a,3i4)') '###  dim,dim1,gdim :',dim,dim1,gdim
         message=message(1:53)//'  (2)'
         call ErrorMsg
      endif

! --- Error if odd pbasis + even sbasis .or. odd sbasis + even pbasis
      if (ibleg.gt.0 .and. ibleg.ne.isleg)             lerr = .true.
! --- Error if slz is incompatible with pbasis
      if (basis.ne.2 .and. basis.ne.7 .and. basis .ne. 12 .and. &
            basis .ne. 33 .and. slz.ne.0)  lerr = .true.
! --- Error if Legendre sbasis is incompatible with pbasis
      if ((basis.le.0 .or. basis.gt.5) .and. (basis.ne.7 .and. &
           basis.ne.9 .and. basis.ne.12 .and.  &
           basis .ne. 20 .and. basis .ne. 21 .and. &
           basis .ne. 33)) lerr = .true. 
! --- Error if j-values in s-basis, using sym, exceed the pbasis range
      if ((basis.eq.2 .or. basis.eq.12) .and. ibleg.eq.0 .and. &
           isleg.gt.0  .and. (2*dim1+1-isleg).gt.gdim) lerr = .true.
      if (lerr) then
         write(6,'(/a,6i4)') '###  ibleg,isleg,slz,basis,gdim,dim1 :', &
                 ibleg,isleg,slz,basis,gdim,dim1
         message=message(1:53)//'  (3)'
         call ErrorMsg
      endif

! --- Error if slz is incompatible with Wigner k and mm ranges
      if (basis.eq.33 .and. usemk.eq.9999) lerr = .true.
      if (lerr) then
         write(6,'(/a,7i4)') '#kmin,kmax,mmmin,mmmax,kmin2,mmmin2,slz:', &
                 kmin,kmax,mmmin,mmmax,kmin2,mmmin2,slz
         message=message(1:53)//'  (4)'
         call ErrorMsg
      endif

      if(basis.eq.2.or.basis.eq.7.or.basis.eq.9.or.basis.eq.12.or. &
         basis.eq.20 .or. basis.eq.21 .or. &
         (basis.eq.33 .and. abs(usemk).eq.1)) then

!-----------------------------------------------------------------------
! Take the initial spf from trafo (DVR.eq.Leg)
!-----------------------------------------------------------------------
         do e=1,dim1
            j = e 
            if(ibleg.eq.0 .and. isleg.gt.0 ) j = 2*e-1+mod(slz+ibleg,2)
            do g=1,gdim
               psi(g,e) = trafo(j,g)
               if (basis.eq.33) psi(g,e) = psi(g,e)*usemk**slz
            end do
         end do

         call schmidtortho(psi,gdim,dim1,ierr)

      else

!-----------------------------------------------------------------------
! Get Legendre polynomials multiplied by their DVR weights (DVR.ne.Leg)
!-----------------------------------------------------------------------
         do g=1,gdim
            do e=1,dim1
               if (isleg.eq.0) then
                  psi(g,e)=weight(g)*dlegendre(e-1,ort(g))
               else 
                  isleg1 = 2-mod(isleg+slz,2)
                  psi(g,e)=weight(g)*dlegendre(2*e-isleg1,ort(g))
               endif
            enddo
         enddo
      
!-----------------------------------------------------------------------
! Normalize single-particle functions
!-----------------------------------------------------------------------
         call normspf(psi,gdim,dim1)
      end if

      return
      end subroutine genaleg

!***********************************************************************
!
!             MCTDH  subroutine  GENKLEG
!
!
!    Generates a set of the normalized associated
!    Legendre polynomials P_l^m for the given set of (l,m) pairs.
!
!    Calls:
!           Subroutine excitations3:  Calculates the set of the
!           most probable (l,m) pairs.
!
!    03/99 SS
!***********************************************************************

      subroutine genkleg(psi,gdim,kdim,dim,trafo,basis,ggdim, &
                 ipbaspar1,ipbaspar2,isbaspar1,isbaspar2,f,mode,mlab, &
                 ilog,message)

use runinwfmod
use multkleg

use op1lib, only: zerovxz

      implicit none

      integer(long)             :: slz,sl0,isleg,ibleg,kmin,ierr,e,g,j,dj,j1,&
                                   m,m1,dm,mmin,mmax,jmin,iex,iprint,failed,&
                                   mmmin2
      integer(long), intent(in) :: f,mode,gdim,ggdim,dim,basis,ilog,kdim

      integer(long), dimension(*), intent(in) :: ipbaspar1,ipbaspar2,isbaspar1,isbaspar2
      character(len=*), intent(in)            :: mlab
      character(len=*), intent(out)           :: message
      logical(kind=4)                         :: lerr,ok

      real(dop), dimension(ggdim,ggdim,kdim), intent(in)  :: trafo
      complex(dop), dimension(gdim,kdim,dim), intent(out) :: psi

!-----------------------------------------------------------------------
! zero the psi array
!-----------------------------------------------------------------------
      call zerovxz(psi,gdim*kdim*dim)

!-----------------------------------------------------------------------
! get primitive basis (DVR) parameters
!-----------------------------------------------------------------------
      ibleg = ipbaspar1(1)
      mmmin2 = 0
      kmin=-999
      if (basis.eq.7 .or. basis.eq.20 .or. basis.eq.9 &
           .or. basis.eq.21) then
         kmin  = ipbaspar2(1)
      elseif (basis.eq.33) then
         kmin  = ipbaspar1(2)
         mmmin2 = ipbaspar1(9)
      endif

!-----------------------------------------------------------------------
! get spf basis parameters
!-----------------------------------------------------------------------
      sl0   = isbaspar1(1)
      isleg = isbaspar1(2)
      iex   = isbaspar1(3)
      iprint= isbaspar1(5)
      slz   = isbaspar2(1)
      mmin  = isbaspar2(2)
      mmax  = isbaspar2(3)
      dm    = isbaspar2(5)
      if(dm.eq.0) dm=1
! NB isbaspar(4,f,p) is reserved for pop.

!-----------------------------------------------------------------------
! Check parameters
!-----------------------------------------------------------------------
      lerr = .false.

      if (dim.gt.gdim*kdim) lerr = .true.
      if (ibleg.gt.0 .and. ibleg.ne.isleg ) lerr = .true.
      if (basis.ne.7 .and. basis.ne.9 .and. basis.ne.20 .and. &
          basis.ne.21 .and. basis .ne. 33) lerr = .true.
      if(ibleg.eq.0 .and. isleg.gt.0 .and. (2*dim-isleg).ge.gdim) then
          write(6,'(/a)') ' dim too large for gdim, sym is used!'
          write(6,'(a,i4,a,i4)') ' gdim =',gdim,'  <  2*dim-isleg =',2*dim-isleg
          lerr=.true.
      endif
      if (mmmin2.ne.0) then
         write(6,'(/a)') 'Kleg-inwf is only compatible with Wigner-DVR '
         write(6,'(a,i4)') 'for mmmin2 = 0, but here mmmin2 = ', mmmin2
         lerr = .true.
      endif
      if(lerr) then
         write(6,'(/a)') '   dim, gdim, kdim, basis, ibleg, isleg'
         write(6,'(6i6)') dim,gdim,kdim,basis,ibleg,isleg
        message='Illegal input parameters for generation of KLeg. func.'
         call ErrorMsg
      endif

!-----------------------------------------------------------------------
! Take the initial spf from trafo (DVR.eq.KLeg)
!-----------------------------------------------------------------------
      dj=1
      j=sl0
      m=slz

      m1=m+1-kmin
      if (ibleg.eq.0) then
         j1=j-abs(m)+1
      else
         jmin = abs(m) + mod(abs(m)+ibleg,2)
         j1 = (j-jmin)/2 + 1
      endif

      if (iprint.eq.1) then
         write (ilog,'(/2a,i3,a,a/)') '*** The (j,m) pairs ', &
                       'for the KLEG InWF,  dof =',f,',  ',trim(mlab)
         write (ilog,'(a,i3,a,i3)') '   1)  j=',j,'   m=',m
      endif
      do g=1,gdim
         psi(g,m1,1) = trafo(j1,g,m1)
      enddo
      if (lsymmultkleg(mode)) then
         multkleg_j(f,1)=j
         multkleg_m(f,1)=m
      endif

      do e=2,dim
         failed=0
 30      continue
         if (iex .eq. 1) then    !  excite=j
            call excite3(j,m,sl0,slz,dj,dm)
         else if (iex.eq.2) then !  excite=m
            call excite5(j,m,sl0,slz,mmin,mmax)
         else if (iex.eq.3) then !  excite=m-old
            call excite4(j,m,sl0,slz,mmin,mmax)
         endif
         call check(j,m,mmin,mmax,ibleg,isleg,gdim,ok)
         if (.not.ok) failed=failed+1
         if(failed.gt.500) then
            write(message,'(a,i4,40x,a)') &
            'Cannot create the e-th KLeg function. e =',e, &
            'Too many SPFs for the size of the KLeg grid.'
            call errormsg
         endif 
         if (.not.ok) goto 30

         m1=m+1-kmin
         if (ibleg.eq.0) then
            j1=j-abs(m)+1
         else
            jmin = abs(m) + mod(abs(m)+ibleg,2)
            j1 = (j-jmin)/2 + 1
         endif

         if (iprint.eq.1) &
            write (ilog,'(i4,a,i3,a,i3)') e,')  j=',j,'   m=',m
         if (lsymmultkleg(mode)) then
            multkleg_j(f,e)=j
            multkleg_m(f,e)=m
         endif

         do g=1,gdim
            psi(g,m1,e) = trafo(j1,g,m1)
         enddo
      enddo

      if (iprint.eq.1) write (ilog,*)
  
      call schmidtortho(psi,gdim*kdim,dim,ierr)

      return
      end subroutine genkleg

!***********************************************************************

      subroutine check(j,m,mmin,mmax,ibleg,isleg,gdim,ok)

      implicit none

      integer(long)                :: jmax
      integer(long), intent(in)    :: j, m, mmin, mmax, ibleg, isleg, gdim
      logical(kind=4), intent(out) :: ok

      ok = .true.
!........j >= 0
      if (j .lt. 0) then
         ok = .false.
         return
      endif
!........jmin <= j <= jmax
      if (isleg.eq.0) then
          jmax=abs(m)+gdim-1
      else
          jmax=abs(m)+2*(gdim-1)
      endif
      if (j.lt.abs(m) .or. j.gt.jmax) then
          ok = .false.
          return
      endif
!........mmax <= m <= mmin
      if (m.lt.mmin .or. m.gt.mmax) then
         ok = .false.
         return
      endif
!......---symmetry (j is even or odd)---
      if (ibleg .ne. 0) then
         if (mod(j+ibleg,2).ne.0) then
            ok = .false.
            return
         elseif (isleg.gt.0) then
            if (mod(j+isleg,2).ne.0) then
               ok = .false.
               return
            endif 
         endif
      endif

      return
      end subroutine check

!***********************************************************************
      
      subroutine excite3(j,m,j0,m0,dj,dm)
!     For KLeg excite (j,m). Excite j-states preferentially (excite=j). 

      implicit none

      integer(long)                :: del, delj, delm, sj, sm
      integer(long), intent(in)    :: j0, m0, dj, dm
      integer(long), intent(inout) :: j, m

!     Calculate auxiliary values
      sj = 1
      if (j .lt. j0)  sj = -1
      sm = 1
      if (m .lt. m0)  sm = -1
      delj = abs(j - j0)/dj
      delm = abs(m - m0)/dm
      del = delm + delj

!     Determine next configuration

!     Try to flip spin
      if ((delm .gt. 0) .and. (sm .eq. 1)) then
         sm = -1
         goto 10
      endif

!     Try to flip 'logical spin' for j
      if ((delj .gt. 0) .and. (sj .eq. -1)) then
         sm = 1
         sj = 1
         goto 10
      endif

!     Try to move excitation from j to m
      if (delm .lt. del) then
         sm = 1
         sj = -1
         delm = delm + 1
         delj = delj -1
         goto 10
      endif

!     Choose 'logical' initial configuration for del+1
      del = del + 1
      sm = 1
      sj = -1
      delm = 0
      delj = del

 10   continue

!     Calculate new configuration
      j = j0 + delj*sj*dj
      m = m0 + delm*sm*dm

      return
      end subroutine excite3

!***********************************************************************

      subroutine excite4(j,m,j0,m0,mmin,mmax)
!     For KLeg excite (j,m). Excite m-states preferentially (excite=m). 
!
      implicit none

      integer(long), intent(in)    :: j0,m0,mmin,mmax
      integer(long), intent(inout) :: j,m


      if ((m.gt.mmax .and. 2*m0-m.lt.mmin) .or. (m.lt.mmin .and. &
           2*m0-m.gt.mmax)) then
         if (j0.eq.0 .or. j.eq.j0) then
            j=j+1
         else
  10        j=2*j0-j
            if (j.gt.j0) j=j+1
            if (j.lt.0) goto 10
         endif
         m=m0
      else
         if (m.eq.m0) then
             m=m+1
         else
             m=2*m0-m
             if (m.gt.m0) m=m+1
         endif
      endif

      return
      end subroutine excite4

!***********************************************************************

      subroutine excite5(j,m,j0,m0,mmin,mmax)
!     for KLeg excite (j,m), excite m-states preferentially (excite=m),
!     and go preferentially downwards for j.
!     06/2006 FO

      implicit none

      integer(long), intent(in)    :: mmin,mmax,j0,m0
      integer(long), intent(inout) :: j,m

      if ((m.gt.mmax .and. 2*m0-m.lt.mmin) .or. &
          (m.lt.mmin .and. 2*m0-m.gt.mmax)) then
!        all m-states for this j have been done, go to next j
         if (j0.eq.0) then
            j=j+1
         else if (j.eq.j0) then
            j=j-1
         else
  10        j=2*j0-j
            if (j.lt.j0) j=j-1
            if (j.lt.0) goto 10
         endif
         m=m0
      else
!        go to next m-state
         if (m.eq.m0) then
            if (m0.le.0) then
               m=m+1
            else
               m=m-1
            endif
         else
            m=2*m0-m
            if (m0.le.0) then
               if (m.gt.m0) m=m+1
            else
               if (m.lt.m0) m=m-1
            endif
         endif
      endif

      return
      end subroutine excite5

!***********************************************************************
!
!             MCTDH  subroutine  GENWIGNER
!
!    Generates a set of the normalized Wigner functions
!    d_j^(m,k) for the given set of (j,m,k) values.
!
!    Calls:
!      Subroutine excite6():  Calculates the list of
!      most probable (j,m,k) sets.
!
!    Note: symmetry is not yet implemented here
!
!    01/13 PST
!***********************************************************************

      subroutine genwigner(psi,gdim,kdim,mmdim,dim,trafo,basis, &
                           ipbaspar1,isbaspar1,isbaspar2,isbaspar3, &
                           f,mlab,ilog,message)

      implicit none

      integer(long), intent(in)               :: gdim,dim,basis
      integer(long), intent(in)               :: f,ilog
      integer(long), dimension(*), intent(in) :: isbaspar1,isbaspar2,isbaspar3
      integer(long), dimension(*), intent(in) :: ipbaspar1
      integer(long)   :: slz,sl0,slm,isleg,ibleg,pdim
      integer(long)   :: kmin,kmax,kdim,mmmin,mmmax,mmdim,dj,dk,dm,kminp,mminp
      integer(long)   :: e,g,j,j1,mm,m1,k,k1,mormm,iex,iprint,k1dim,m1dim
      logical(kind=4) :: lerr
      real(dop), dimension(gdim,gdim,kdim,mmdim),intent(in)     :: trafo
      complex(dop), dimension(gdim,kdim,mmdim,dim), intent(out) :: psi
      character(len=*), intent(in)  :: mlab
      character(len=*), intent(out) :: message

!-----------------------------------------------------------------------
! primitive basis (DVR) parameters
!-----------------------------------------------------------------------
      ibleg = ipbaspar1(1)
      kminp = ipbaspar1(2)
      mminp = ipbaspar1(6)

!-----------------------------------------------------------------------
! spf basis parameters
!-----------------------------------------------------------------------
      sl0   = isbaspar1(1)
      isleg = isbaspar1(2)
      iex   = isbaspar1(3)
      iprint= isbaspar1(5)
      slz   = isbaspar2(1)
      kmin  = isbaspar2(2)
      kmax  = isbaspar2(3)
      dk    = isbaspar2(5)
      slm   = isbaspar3(1)
      mmmin = isbaspar3(2)
      mmmax = isbaspar3(3)
      dm    = isbaspar3(5)
!-----------------------------------------------------------------------
! Check parameters
!-----------------------------------------------------------------------
      lerr = .false.

      k1dim=(kmax-kmin+dk)/dk
      m1dim=(mmmax-mmmin+dm)/dm
      pdim=gdim*k1dim*m1dim
      if (dim.gt.pdim) lerr = .true.
      if (dim.gt.gdim*kdim*mmdim) lerr = .true.
      if (ibleg.gt.0 .and. ibleg.ne.isleg ) lerr = .true.
      if (basis.ne.33) lerr = .true.
      if(ibleg.eq.0 .and. isleg.gt.0 .and. (2*dim-isleg).ge.gdim) then
          write(6,'(/a)') ' dim too large for gdim, sym is used!'
          write(6,'(a,i4,a,i4)') ' gdim =',gdim,'  <  2*dim-isleg =',2*dim-isleg
          lerr=.true.
      endif
      if(lerr) then
         write(6,'(/a)') '  dim, gdim, kdim, mmdim, basis, ibleg, isleg'
         write(6,'(7i6)') dim,gdim,kdim,mmdim,basis,ibleg,isleg
         if(dim.gt.pdim) then
            write(6,'(/2a)') ' dim, gdim*k1dim*m1dim, gdim,', &
                      ' k1dim, m1dim, dk,  dm'
            write(6,'(i4,"  >",i5,6x,";",2x,5i6)') &
               dim, pdim, gdim, k1dim, m1dim,dk,dm
         endif
         message='Illegal input parameters for Wigner initial SPFs.'
         call ErrorMsg
      endif

!-----------------------------------------------------------------------
! Take the initial spf from trafo (DVR.eq.Wigner)
!-----------------------------------------------------------------------
      j=sl0
      k=slz
      mm=slm
      mormm=max(abs(k),abs(mm))
      k1=k+1-kminp
      m1=mm+1-mminp

      j1=-999
      if (ibleg.eq.0) then
         j1=j+1-mormm
         dj=1  ! Symmetry not implemented, so dj is set to 1 for now...
      else
         message='Alas, symmetry still is not implemented for Wigner.'
         call ErrorMsg
      endif

      if (iprint.eq.1) then
         write (ilog,'(/2a,i3,a,a/)') '*** The (j,k,m) sets ', &
                       'for the Wigner InWF,  dof =',f,',  ',trim(mlab)
         write (ilog,'(a,i3,a,i3,a,i3)') '   1)  j=',j,'   k=',k, &
                       '   m=', mm
      endif
      do g=1,gdim
         psi(g,k1,m1,1) = trafo(j1,g,k1,m1)
      enddo

! --- Higher configurations ---

      do e=2,dim
        if (iex .eq. 1) then    !  excite=mkj
           call excite6(j,k,mm,sl0,slz,slm,kmin,mmmin,kmax,mmmax,dj,dk,dm,message)
        else if (iex.eq.2) then !  excite=kmj (swap all k and m args)
           call excite6(j,mm,k,sl0,slm,slz,mmmin,kmin,mmmax,kmax,dj,dm,dk,message)
        endif
        mormm=max(abs(k),abs(mm))
        k1=k+1-kminp
        m1=mm+1-mminp
        if (ibleg.eq.0) then
          j1=j+1-mormm
        else
          message='I TOLD you already, no symmetry for Wigner!'
          call ErrorMsg
        endif

        if (iprint.eq.1) &
          write (ilog,'(i4,a,i3,a,i3,a,i3)') e,')  j=',j,'   k=',k,'   m=',mm

        do g=1,gdim
          psi(g,k1,m1,e) = trafo(j1,g,k1,m1)
        enddo
      enddo
      if (iprint.eq.1) write (ilog,*)

      return
      end subroutine genwigner

!***********************************************************************
!                      MCTDH  subroutine  EXCITE6
!     For Wigners excite (j,k,m):
!     m-states preferentially, then k, then j
!     To do k-states, then m, then j: swap (k,k0,kmin,kmax,dk) with
!       (m,m0,mmin,mmax,dm) in the calling subroutine
!     01/13 PST
!***********************************************************************

      subroutine excite6(j,k,m,j0,k0,m0,kmin,mmin,kmax,mmax,dj,dk,dm,message)

      implicit none

      integer(long), intent(inout)  :: j,k,m
      integer(long), intent(in)     :: j0,k0,m0,kmin,mmin,kmax,mmax,dj,dk,dm
      integer(long)                 :: delj,delk,delm
      logical(kind=4)               :: jbad,kbad,kgood,mbad,mgood
      character(len=*), intent(out) :: message

! --- First, error checking
      if (j .lt. 0) then
         message='excite6(): j < 0'
         call errormsg
      endif
      if (abs(k) .gt. j) then
         message='excite6(): |k| > j'
         call errormsg
      endif
      if (abs(m) .gt. j) then
         message='excite6(): |m| > j'
         call errormsg
      endif
      if (kmin .gt. kmax) then
         message='excite6(): kmin > kmax'
         call errormsg
      endif
      if (mmin .gt. mmax) then
         message='excite6(): mmin > mmax'
         call errormsg
      endif
      if ((k .lt. kmin) .or. (k .gt. kmax)) then
         message='excite6(): k is outside of range [kmin,kmax]'
         call errormsg
      endif
      if ((m .lt. mmin) .or. (m .gt. mmax)) then
         message='excite6(): m is outside of range [mmin,mmax]'
         call errormsg
      endif

      kbad = .false.
      kgood = .false.
      mbad = .false.
      mgood = .false.

! --- Secondly, determine next m
      do while ( (.not. mbad) .and. (.not. mgood) )
        delm = m - m0
        if (delm .eq. 0) then  ! If m=m0, reduce it by dm
          m = m - dm
        else if (delm .lt. 0) then  ! If m<m0, go to m>m0
          m = m - 2*delm
        else  ! If m>m0, go to next m<m0
          m = m - 2*delm - dm
        endif
        if (m .lt. mmin) then  ! m is outside [mmin,mmax]
          if (2*m0-m .gt. mmax) mbad = .true.
        else if (m .gt. mmax) then  ! m is outside [mmin,mmax]
          if (2*m0-m .lt. mmin) mbad = .true.
        else  ! If m is inside of j-range, then it is good
          if ( (m.ge.-j) .and. (m.le.j) ) mgood = .true.
        endif
      enddo
! --- If new value of m is bad, find next one compatible with j...
      if (mbad) then
        m = m0
        do while ( (abs(m).gt.j) .and. (.not.kbad) )
          if (m .lt. 0) then
             if ( (m*(m+dm).lt.0) .and. (abs(m+dm).gt.j) ) then
               kbad = .true.  ! If m changes sign and still |m| > j
             else             ! then there are no more good m-values
               m = m + dm     ! so just go to next j-value
             endif
          else
             if ( (m*(m-dm).lt.0) .and. (abs(m-dm).gt.j) ) then
               kbad = .true.
             else
               m = m - dm
             endif
          endif
        enddo
! --- ... and determine next k that is also compatible with j
        do while ( (.not. kbad) .and. (.not. kgood) )
          delk = k - k0
          if (delk .eq. 0) then
            k = k - dk
          else if (delk .lt. 0) then
            k = k - 2*delk
          else
            k = k - 2*delk - dk
          endif
          if (k .lt. kmin) then
            if (2*k0-k .gt. kmax) kbad = .true.
          else if (k .gt. kmax) then
            if (2*k0-k .lt. kmin) kbad = .true.
          else
            if ( (k.ge.-j) .and. (k.le.j) ) kgood = .true.
          endif
        enddo
      endif
! --- Determine next j if k and m are finished for current j
      if (kbad) then
        m = m0
        k = k0
        jbad = .true.
        do while (jbad)
          kbad = .false.
          mbad = .false.
          delj = j - j0
          if (delj .eq. 0) then  ! Move from initial j
            if (j .lt. dj) then  ! Move j=0 -> j=dj
              j = j + dj
            else                 ! If j>0, go to lower j first
              j = j - dj
            endif
          else if (delj .lt. 0) then  ! j<j0 done, now move to j>j0
            j = j - 2*delj
          else          ! j>j0 done
            j = j + dj  ! Go to higher j by default
            if (j-j0 .le. j0) j = 2*j0-j  ! But do lower j if allowed
          endif
! ---  If |k| or |m| is too large for next j, fix them
          do while ( (abs(k).gt.j) .and. (.not.kbad) )
            if (k .lt. 0) then
               if ( (k*(k+dk).lt.0) .and. (abs(k+dk).gt.j) ) then
                 kbad = .true.
               else
                 k = k + dk
               endif
            else
               if ( (k*(k-dk).lt.0) .and. (abs(k-dk).gt.j) ) then
                 kbad = .true.
               else
                 k = k - dk
               endif
            endif
          enddo
          do while ( (abs(m).gt.j) .and. (.not.mbad) )
            if (m .lt. 0) then
               if ( (m*(m+dm).lt.0) .and. (abs(m+dm).gt.j) ) then
                 mbad = .true.
               else
                 m = m + dm
               endif
            else
               if ( (m*(m-dm).lt.0) .and. (abs(m-dm).gt.j) ) then
                 mbad = .true.
               else
                 m = m - dm
               endif
            endif
          enddo
! --- But if an acceptable k and m cannot be found, go to next j
          if ( (k.ge.kmin) .and. (k.le.kmax) .and. &
               (m.ge.mmin) .and. (m.le.mmax) .and. &
               (.not. kbad) .and. (.not. mbad) ) then
            jbad = .false.
          else
            k = k0
            m = m0
          endif
        enddo
      endif

      return
      end subroutine excite6

!***********************************************************************
!                 MCTDH subroutine KCLEARSPF
! For K INWF, this clears spfs for all K values not equal to slz.
! This is used for INWFs where K is combined with DOFs other than
! kleg or Wigner (e.g. Leg+K or Gauss+K)
!***********************************************************************

      subroutine kclearspf(psi,isbaspar,vgdim,gdim,ngdim,nspf,dim)

      implicit none

      integer(long), intent(in) :: vgdim,gdim,ngdim,nspf,dim
      integer(long)             :: n,v,g,slz,mmin,m1
      integer(long), dimension(*), intent(in)                    :: isbaspar
      complex(dop), dimension(vgdim,gdim,ngdim,dim), intent(out) :: psi

! --- get spf basis parameters (for K DOF)

      slz  = isbaspar(1)
      mmin = isbaspar(2)
      m1   = slz-mmin+1

! --- Set to zero all spfs except those with k = slz

      do n=1,ngdim
         do g=1,gdim
            do v=1,vgdim
               if (g.ne.m1) psi(v,g,n,nspf)=(0.0d0,0.0d0)
            enddo
         enddo
      enddo

      return
      end subroutine kclearspf

!***********************************************************************
!
!            MCTDH  subroutine GENKGWP
!
!     Generates the IWF for the combined mode containing "k" inwf
!     for a GWP-type particle.       
!
! NB pgmat contains either P-matrix or width matrix
! for diagonal form, gwptype = 3, dgmat is a vector with width.
!
!***********************************************************************

      subroutine genkgwp(phigmat,xigmat,pgmat,dgmat,agmat,phase, &
                 spf1d,dimf1d,gwpdim1,gdof,m,s)

use global
use maxv, only: maxdim
use griddatmod
use psidef
use runinwfmod

      implicit none

      integer(long)                          :: fg,index,nex,f,shell,&
                                                nexg,minex,maxex,eg
      logical                                :: endmark,premature
      integer(long), intent(in)              :: m,s,gdof,gwpdim1,&
                                                dimf1d
      integer(long), dimension(maxdim)       :: iexc
      integer(long), dimension(2*gwpdim1)    :: stack

      real(dop)                              :: norm
      integer(long)                          :: extype
      real(dop)                              :: scwidth

      complex(dop)                                         :: newwidth 
      complex(dop), dimension(3)                           :: param
      complex(dop), dimension(spf1ddim), intent(in)        :: spf1d
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(out) :: pGmat,agmat
      complex(dop), dimension(gdof,gwpdim1), intent(out)      :: xigmat,dgmat
      complex(dop), dimension(gwpdim1), intent(out)           :: phigmat,phase

      logical(kind=4) :: lnoq0

!     note:  gdof = nspfdof(m)      number of dof's in combined gwp 
!            dim                    number of gwp's       
      
!-----------------------------------------------------------------------
! 'excitation' type (extype): i.e., construction of higher
! configurations via shift in the center coordinate, either:

! extype = 1    seqex
!     sequentially do a positive and negative shift for each dof
!     indicidually. No combined shifts are made.
!          2    simex
!     simultaneously in all dof's, move along a diagonal (useful for
!     test purposes to avoid overlap between initial Gaussians).
!          3    shell
!                4    readmd - done in genrdgwp
!                5    wigner (random_pq) - done in genrgwp
!                6    random - done in genrgwp
!                7    wigner_sym (symmetric_pq) - done in genrgwp
!          8    pqgrid. Set up mdGWP as shell.
!
! typp = 0  distribution in coordinates
!        1  distribution in momentum

!-----------------------------------------------------------------------
      f = spfdof(1,m)
      extype = isbaspar(1,f,1)
      if (extype .eq. 8) extype = 3
      scwidth = rsbaspar(7,f,1)

! check all DOFs have same excitation types
      do fg=2,gdof
         f=spfdof(fg,m)
         if (isbaspar(1,f,1) .ne. extype) then
            routine = 'Genkgwp (genhi1.f90)'
            write(message,'(a,i5)')  &
                'Make sure all DOFs have same excitation in GWP for mode :',m
            call errormsg
         endif
      enddo

! see if Q0 not included.
      lnoq0 = .false.
      do fg=1,gdof
         f=spfdof(fg,m)
         if (isbaspar(8,f,1) .eq. 1) lnoq0 = .true.
      enddo


! ... initialize : 

! zero parameter arrays      
      agmat(:,:,:) = 0.0_dop
      phase(:) = 0.0_dop
      xigmat(:,:) = 0.0_dop

! ... for the full width and diagonal forms : initialize pGmat or dGmat
      if (gwptype .le. 1) then
         pgmat(:,:,:) = 0.0_dop
      else if (gwptype .eq. 2) then
         dgmat(:,:) = 0.0_dop
      endif
      if (lgwpphase) phigmat(:) = 0.0_dop

! iexc(i) is the function to take for DOF i
      iexc(1:gdof) = 0

! nex is the excitation level (i.e. sum of excitations)
      nex = 1
! stack is the DOFs being excited
      stack(:) = 0
      stack(1) = 1

      if (extype.eq.3) then
! start variables for various shell option variants
         shell = 1
         nexg = 1
         minex=1
         maxex=2
      endif

      index = 0
      eg = 0
      do while (eg .lt. gwpdim1) !loop over GWP to be generated
         index = index+1  

         if (index .eq. 1 .and. lnoq0) go to 100   ! do not store GWP

         eg = eg + 1  ! next GWP
         do fg = 1,gdof

! get the parameters for Gaussian of desired dof and excitation
            f=spfdof(fg,m)
            call getgpara(spf1d(zspf1d(f,s)),param,dimf1d,iexc(fg))

!     -- load parameters into psi vector

!     -- on-diagonal width parameter --
            if(index.eq.1.or.(abs(scwidth-0.0_dop) .lt. tiny))then 
               agmat(fg,fg,eg) = param(1)

            elseif(index.gt.1.and.scwidth.ne.0.0_dop)then 
               newwidth = param(1)*(1.0_dop+scwidth*dble(index-1)) 
               norm = pi/(-2.0_dop*dble(newwidth))
               norm = dsqrt(norm)
               norm = -0.5_dop*( dlog(norm)  &
                   -0.5_dop*dble(param(2))**2/dble(newwidth) )
               param(3) = dcmplx(norm,0.0_dop)

!     -- width parameter --
               agmat(fg,fg,eg) = newwidth

            endif 

            if (gwptype .le. 1) then
               pGmat(fg,fg,eg) = param(1)
            else if (gwptype .eq. 2) then
               dGmat(fg,eg) = param(1)
            endif

!     -- linear coefficients --
            xigmat(fg,eg) = param(2)

!     -- phase parameter --
            phase(eg) = phase(eg)+param(3)
            if (lgwpphase) phigmat(eg) = phase(eg)

         end do

     100 continue   ! skip function

! ... now consider higher configurations :
         if(extype.eq.1)then
            iexc(1:gdof)=0

            fg = (index-1)/2+1
            nex = ((fg-1)/gdof) 
            fg = mod(fg-1,gdof)+1
            iexc(fg)=(-mod(index,2)+2)+2*nex

         elseif(extype.eq.2) then
            iexc(1:gdof)=iexc(1:gdof)+1
  
         elseif(extype.eq.3) then

!(this implementation still seems a bit over complicated)
!(there should be a way to make enum1, enum and the all thing simpler 
! and not use array stack)

            premature = .true.
            do while (premature)
               endmark = .false.
               premature = .false.

               call enum1(gdof,nex,nexg,minex,maxex,endmark,iexc,stack, &
                    premature)

               if(endmark) then
                  if(nex .lt. 2*nexg*shell) then
                     nex = nex+1
                  else if(nexg .lt. gdof) then
                     nexg = nexg+1
                     nex = (2*shell-1) + (nexg-1)
                  else 
                     shell=shell+1
                     maxex=2*shell
                     minex=2*shell-1
                     nexg = 1
                     nex = minex
                  endif
                  stack(1:nex)=1
               endif
  
            end do 
  
         else
            routine = 'Genkgwp (genphi1.F90)'
            write(message,'(a,i5)') &
                 'Excitation type not supported here ',extype
            call errormsg
         endif 

      end do

      return
      end subroutine genkgwp

!***********************************************************************
!
!            MCTDH  subroutine GENRGWP
!
!     Generates the IWF for a GWP-type particle using random sampling.       
!
!     note:  gdof = nspfdof(m)      number of dof's in combined gwp 
!            gwpdim1                number of gwp's       
!            typ = 5  Wigner
!                  6  random (p=mom0)
!                  7  symmetric Wigner
!***********************************************************************

      subroutine genrgwp(phigmat,xigmat,pgmat,dgmat,agmat,phase, &
                 typ,gwpdim1,gdof,m,s)

use global
use griddatmod
use psidef
use runinwfmod
use gwplib, only: fgaover1,ffgaover
use randommod, only: randgaussian


      implicit none

      integer(long)                          :: f,g,i,iattempt,idim
      integer(long), intent(in)              :: typ,m,s,gdof,gwpdim1

      real(dop)                              :: norm, phase0, gshift, ort1, mom1
      real(dop), dimension(gdof)             :: gxeq, gmom, &
                                                ort,mom,xsigma,psigma
      complex(dop), dimension(gdof)          :: gwidth
      complex(dop)                           :: over,gpair
      real(dop)                              :: maxover

      complex(dop), dimension(3)                           :: param
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(out) :: pGmat,agmat
      complex(dop), dimension(gdof,gwpdim1), intent(out)      :: xigmat,dgmat
      complex(dop), dimension(gwpdim1), intent(out)           :: phigmat,phase

      
      pi=4.0_dop*atan(1.0_dop)

!-----------------------------------------------------------------------
! get spf basis parameters
! width is passed as 1/sigma to keep same convention as in gengauss
!-----------------------------------------------------------------------
      gshift=999.0
      do g=1,gdof
         f=spfdof(g,m)
         gxeq(g)  = rsbaspar(1,f,s)
         gmom(g)  = rsbaspar(2,f,s)
         if (rsbaspar(6,f,s) .gt. 0) then
            gwidth(g) = dcmplx(rsbaspar(6,f,s),0.0_dop)
         else
            gwidth(g) = dcmplx(rsbaspar(3,f,s),rsbaspar(4,f,s))
         endif
         gwidth(g) = -0.25_dop*gwidth(g)**2

         gshift = rsbaspar(5,f,s)

!-----------------------------------------------------------------------
! get scaling for domain for random generation (GS Wigner function)
! rsbaspar(3) is 1/sigma where sigma is std dev of WP
!-----------------------------------------------------------------------
         xsigma(g) = 1.0_dop/rsbaspar(3,f,s)
         psigma(g) = rsbaspar(3,f,s)
      enddo

!-----------------------------------------------------------------------
!     generate 3-element 'param' array from spf basis parameters;
!     first, corresponding to 'ground state' Gaussian
!-----------------------------------------------------------------------
      phase(:)=0.0_dop
      do g=1,gdof
         param(1) = gwidth(g)

! use q + ip for linear parameters
         param(2) = dcmplx(gxeq(g),gmom(g))

         norm = pi/(-2.0_dop*dble(gwidth(g)))
         norm = dsqrt(norm)
         norm = -0.5_dop*dlog(norm)+1.0_dop*dble(gwidth(g))*gxeq(g)**2

!     the imag part of the phase parameter is set to zero; the real
!     part results from normalization

         param(3) = dcmplx(norm,0.0_dop)

!     store in spf :

!     -- width - same for all GBFs --
         do i=1,gwpdim1
            agmat(g,g,i) = param(1)
            if (gwptype .le. 1) then
               pGmat(g,g,i) = param(1)
            else if (gwptype .eq. 2) then
               dGmat(g,i) = param(1)
            endif
         enddo

!     -- linear coefficients --
         xigmat(g,1) = param(2)

!     -- phase parameter --
         phase(1) = phase(1)+param(3)
         if (lgwpphase) phigmat(1) = phase(1)
      enddo

! configurations generated by random selection
      if (typ .eq. 5 .or. typ .eq. 6) then
         do idim=2,gwpdim1
            iattempt = 0
   10       continue
            if (iattempt .eq. 100) then
               routine = 'Genrgwp'
               write(message,'(a,i5)') &
        'Not able to place GBF after 100 attempts. GBF:',idim
               write(message(71:120),'(a,i5)') &
        'Extending range of search space by 10%:'
               call warnmsg1

               xsigma = xsigma*1.1
               psigma = psigma*1.1

            endif

            phase(idim)=0.0_dop
            do g=1,gdof
               gpair=randgaussian()

               ort(g)=xsigma(g)*real(gpair)
               mom(g)=psigma(g)*aimag(gpair)

               ort1 = gxeq(g) + ort(g)
               if (typ .eq. 5) then
                  mom1=gmom(g) + mom(g)
               else
                  mom1=gmom(g)
               endif

!     -- linear coefficients --
               param(2) = dcmplx(ort1,mom1)
               xigmat(g,idim) = param(2)

!     -- phase parameter --
               norm = dsqrt(pi/(-2.0_dop*dble(gwidth(g))))
               norm = -0.5_dop*dlog(norm)+1.0_dop*dble(gwidth(g))*ort1**2
               phase0 = -mom1*ort1
               param(3) = dcmplx(norm,phase0)

               phase(idim) = phase(idim)+param(3)
               if (lgwpphase) phigmat(idim) = phase(idim)
            enddo

! check not too close to previous functions
            maxover=0.0
            do i = 1,idim-1
               over=ffgaover(phase(idim),phase(i),xigmat(1,idim),&
                    xigmat(1,i),agmat(1,1,idim),agmat(1,1,i),gdof)
               maxover=max(maxover,dble(over))
            enddo
            if (abs(maxover) .gt. gshift) then
               iattempt = iattempt+1
               go to 10
            endif

         enddo

      else if (typ .eq. 7) then
         do idim=2,gwpdim1-3,4
            iattempt = 0
   30       continue
            if (iattempt .eq. 100) then
               routine = 'Genrgwp'
               write(message,'(a,i5)') &
        'Not able to place GBF after 100 attempts. GBF:',idim
               write(message(71:120),'(a,i5)') &
        'Extending range of search space by 10%:'
               call warnmsg1

               xsigma = xsigma*1.1
               psigma = psigma*1.1

            endif

            phase(idim)=0.0_dop
            do g=1,gdof
               gpair=randgaussian()

               ort(g)=xsigma(g)*real(gpair)
               mom(g)=psigma(g)*aimag(gpair)

               ort1 = gxeq(g) + ort(g)
               mom1=gmom(g) + mom(g)

!     -- linear coefficients --
               param(2) = dcmplx(ort1,mom1)
               xigmat(g,idim) = param(2)

!     -- phase parameter --
               norm = dsqrt(pi/(-2.0_dop*dble(gwidth(g))))
               norm = -0.5_dop*dlog(norm)+1.0_dop*dble(gwidth(g))*ort1**2
               phase0 = -mom1*ort1
               param(3) = dcmplx(norm,phase0)

               phase(idim) = phase(idim)+param(3)
               if (lgwpphase) phigmat(idim) = phase(idim)
            enddo

! check not too close to previous functions
            maxover=0.0
            do i = 1,idim-1
               over=ffgaover(phase(idim),phase(i),xigmat(1,idim),&
                    xigmat(1,i),agmat(1,1,idim),agmat(1,1,i),gdof)
               maxover=max(maxover,dble(over))
            enddo
            if (abs(maxover) .gt. gshift) then
               iattempt = iattempt+1
               go to 30
            endif

            phase(idim+1) = 0.0_dop
            phase(idim+2) = 0.0_dop
            phase(idim+3) = 0.0_dop
            do g=1,gdof
               ort1 = gxeq(g) - ort(g)
               mom1 = gmom(g) + mom(g)
               param(2) = dcmplx(ort1,mom1)
               norm = dsqrt(pi/(-2.0_dop*dble(gwidth(g))))
               norm = -0.5_dop*dlog(norm)+1.0_dop*dble(gwidth(g))*ort1**2
               phase0 = -mom1*ort1
               param(3) = dcmplx(norm,phase0)
               xigmat(g,idim+1) = param(2)
               phase(idim+1) = phase(idim+1)+param(3)
               if (lgwpphase) phigmat(idim+1) = phase(idim+1)

               ort1 = gxeq(g) + ort(g)
               mom1 = gmom(g) - mom(g)
               param(2) = dcmplx(ort1,mom1)
               norm = dsqrt(pi/(-2.0_dop*dble(gwidth(g))))
               norm = -0.5_dop*dlog(norm)+1.0_dop*dble(gwidth(g))*ort1**2
               phase0 = -mom1*ort1
               param(3) = dcmplx(norm,phase0)
               xigmat(g,idim+2) = param(2)
               phase(idim+2) = phase(idim+2)+param(3)
               if (lgwpphase) phigmat(idim+2) = phase(idim+2)

               ort1 = gxeq(g) - ort(g)
               mom1 = gmom(g) - mom(g)
               param(2) = dcmplx(ort1,mom1)
               norm = dsqrt(pi/(-2.0_dop*dble(gwidth(g))))
               norm = -0.5_dop*dlog(norm)+1.0_dop*dble(gwidth(g))*ort1**2
               phase0 = -mom1*ort1
               param(3) = dcmplx(norm,phase0)
               xigmat(g,idim+3) = param(2)
               phase(idim+3) = phase(idim+3)+param(3)
               if (lgwpphase) phigmat(idim+3) = phase(idim+3)
            enddo

         enddo
      endif

      return
      end subroutine genrgwp

!***********************************************************************
!
!            MCTDH  subroutine GENRDGWP
!
!     Generates the IWF for a GWP-type particle reading explicit 
!     parameters
!
!     note:  gdof = nspfdof(m)      number of dof's in combined gwp 
!            gwpdim1                number of gwp's       
!***********************************************************************

      subroutine genrdgwp(phigmat,xigmat,pgmat,dgmat,agmat,phase, &
                 gwpdim1,gdof,m1,s1)

use global
use griddatmod
use psidef
use runinwfmod
use gwplib, only: fgaover1,ffgaover


      implicit none

      integer(long)                          :: n,e,g,m,s
      integer(long), intent(in)              :: m1,s1,gdof,gwpdim1

      real(dop)                              :: norm
      real(dop)                              :: gx, gp, width, gwidth

      complex(dop), dimension(gdof,gdof,gwpdim1), intent(out) :: pGmat,agmat
      complex(dop), dimension(gdof,gwpdim1), intent(out)      :: xigmat,dgmat
      complex(dop), dimension(gwpdim1), intent(out)           :: phigmat,phase

      integer(long)     :: ic,ierr,iz
      logical(kind=4)   :: check
      character(len=c5) :: filename

      pi=4.0_dop*atan(1.0_dop)
      agmat = 0.0_dop
      phase = 0.0_dop
      if (gwptype .le. 1) then
         pGmat = 0.0_dop
      else if (gwptype .eq. 2) then
         dGmat = 0.0_dop
      endif
      if (lgwpphase) phigmat=0.0_dop

      inquire(iin,opened=check)
      if (check) then
         rewind(iin)
      else
         filename=inputfile(1:inplaenge)
         open(iin,file=filename,status='old',form='formatted',err=900)
      endif

!-----------------------------------------------------------------------
!!READ KEYWORDS:
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'genrdgwp  (eininwf)'
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      if (keyword(1) .ne. "gwp-define-section") go to 10

      m=1
      s=1
      n=0
   20 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'genrdgwp  (eininwf)'
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      if (keyword(1) .eq. "end-gwp-define-section") then
         continue
      else if (keyword(1) .eq. "define") then
         read(keyword(2),*) e
         if (ic .ge. 3) read(keyword(3),*) m
         if (ic .eq. 4) read(keyword(4),*) s

         if (m .ne. m1) go to 20
         if (s .ne. s1) go to 20
         if (e .gt. gwpdim1) go to 20

         n=n+1
         do g=1,gdof
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            routine = 'read INIT_WF-SECTION  (eininwf)'
            message = inptit
            call cntrl(keyword(1),lc(1),ierr)

            if (ic .lt. 3) then
               routine = "Genrdgwp"
               write(message,'(a,i5,i5)') &
            'Need 3 parameters per DOF to specify GWP parameters'
               call errormsg
            endif
            read(keyword(1),*) width
            read(keyword(2),*) gx
            read(keyword(3),*) gp
         
! width is sigma (std dev of Gaussian)
            gwidth = 0.25d0*(1.0d0/width)**2
            agmat(g,g,e) = -gwidth
            if (gwptype .le. 1) then
               pGmat(g,g,e) = -gwidth
            else if (gwptype .eq. 2) then
               dGmat(g,e) = -gwidth
            endif

            xigmat(g,e) = dcmplx(gx,gp)

            norm = pi/(2.0_dop*gwidth)
            norm = dsqrt(norm)
            phase(e) = phase(e) -0.5_dop*dlog(norm) - gwidth*gx**2
         enddo

         if (lgwpphase) phigmat(e) = phase(e)

         go to 20

      else
         go to 20
      endif

      close(iin)

      if (n .lt. gwpdim1) then
         routine = "Genrdgwp"
         write(message,'(a,i5,i5)') &
            'Not enough sets of GWP parameters specified for mode : ',m1,s1
         write(message(72:100),'(a,i5)') ' Need at least : ',gwpdim1
         call errormsg
      endif

      return
  900 continue
      routine='Genrdgwp'
      message = 'Cannot open input file '
      call errormsg

      end subroutine genrdgwp

!***********************************************************************
!
!             MCTDH  subroutine  GENGWP
!
!
!    Generates initial GWPs when driven along pre-computed trajectories
!
!***********************************************************************

      subroutine gentgwp(agmat,xivec,phase,xi,phi,gwpdim1,gdof,rsbaspar1)

use global
use griddatmod
use psidef
use runinwfmod

      implicit none

      integer(long)                          :: fg,ig
      integer(long), intent(in)              :: gdof,gwpdim1

      real(dop), dimension(*), intent(in)    :: rsbaspar1
      real(dop)                              :: time
      complex(dop)                           :: gwidth
      real(dop), dimension(gdof,gwpdim1)     :: qcenter

      complex(dop), dimension(gdof,gdof,gwpdim1), intent(out) :: agmat
      complex(dop), dimension(gdof,gwpdim1), intent(out)      :: xivec
      complex(dop), dimension(gwpdim1), intent(out)           :: phase
      complex(dop), dimension(gdof,gwpdim1), intent(out)      :: xi
      complex(dop), dimension(gwpdim1), intent(out)           :: phi

      pi=4.0_dop*atan(1.0_dop)

      if (gwptype .ne. 3) then
         routine = "Gentgwp"
         message = 'Trajectory GWPs must be frozen'
         call errormsg
      endif

!-----------------------------------------------------------------------
! width is passed as 1/sigma to keep same convention as in gengauss
!-----------------------------------------------------------------------
      if (rsbaspar1(6) .gt. 0) then
         gwidth = rsbaspar1(6)
      else
         gwidth = dcmplx(rsbaspar1(3),rsbaspar1(4))
      endif
      gwidth = -0.25_dop*gwidth**2

      agmat = 0.0_dop
      do ig=1,gwpdim1
         do fg = 1,gdof
            agmat(fg,fg,ig) = gwidth
         enddo
      enddo

!-----------------------------------------------------------------------
! get initial point on each trajectory to be centre of GWPs
! scalar parameter is set so that real part keeps GWP normalised
! and imaginary part is integral of E_tot
!-----------------------------------------------------------------------
      time=0.0_dop
      call gwptrj_pq(time,agmat,xivec,phase,xi,phi,qcenter,gdof,gwpdim1)
 
      return
      end subroutine gentgwp

!#######################################################################
!
! returns parameters for one Gaussian at a selected excitation (iexc)
!

      subroutine getgpara(spf1d,param,dimf1d,iexc)

      implicit none

      integer(long), intent(in)                     :: iexc,dimf1d
      complex(dop), dimension(3), intent(out)       :: param
      complex(dop), dimension(3,dimf1d), intent(in) :: spf1d

      param(:)=spf1d(:,iexc+1)

      return
      end subroutine getgpara

!***********************************************************************      

      subroutine enum1(n,n1,ng,minex,maxex,endmark,v,stack,premature)

      integer(long)                              :: total,i,nmin,nmax
      integer(long), intent(in)                  :: n,n1,ng,maxex,minex
      logical, intent(out)                       :: endmark,premature
      integer(long), dimension(n), intent(out)   :: v
      integer(long), dimension(:), intent(inout) :: stack
      logical                                    :: reject

      premature=.false.
      do
         reject = .false.
         call enum(n,n1,endmark,v,stack)

! reject if sum no. of dofs excited > required,
!        or if any dof excited more than wanted
!        or if all dofs under minex
         total=0
         nmin=0
         nmax=0
         do i=1,n
            if (v(i) .gt. 0) total=total+1
            if (v(i) .gt. maxex) nmax = nmax+1
            if (v(i) .ge. minex) nmin = nmin+1
         enddo
         if ((total.ne.ng).or.(nmin.eq.0).or.(nmax.gt.0)) reject = .true.

! if incorrect no. of excited DOFs, get next sequence, exit otherwise
         if ((.not.reject) .or. endmark) exit
      end do

! flag if not possible
      if(reject) premature = .true.

      return
      end subroutine enum1

!***********************************************************************      
! n = no. dofs
! n1 = degree of excitation

      subroutine enum(n,n1,endmark,v,stack)

      implicit none

      integer(long)                                  :: i1,i2,tmp
      integer(long), intent(in)                      :: n,n1
      logical, intent(out)                           :: endmark
      integer(long), dimension(n), intent(out)       :: v
      integer(long), dimension(:), intent(inout)     :: stack

      endmark=.false.
      v(:)=0
      do i1=1,n1
         tmp=stack(i1)
         v(tmp)=v(tmp)+1
      enddo

      if(stack(n1).eq.n) endmark = .true.

      do i1=1,n1
         if(stack(i1).lt.n) then
            stack(i1)=stack(i1)+1
            do i2=1,i1-1
               stack(i2)=stack(i1)
            enddo
            exit
         endif
      enddo

      return
      end subroutine enum

!#######################################################################
!
!            MCTDH subroutine GENSPHFBR
!
!
! sets up single particle functions for FBR-spherical harmonics basis
! set.
!
! variables passed for one mode:
!  psi: single particle functions (states)
!  dim: number of spfs (states)
!  gdim: nb of basis functions
!  jsph,msph  see subroutine initfbr in init1.f
!  qnl0,qnm   see main.inc
!
! 08/97 MCH
!
!***********************************************************************

      subroutine gensphfbr(psi,gdim,dim,jsph,msph, &
                           isbaspar,ilog,routine,message)

      implicit none

      character(len=64), intent(out)  :: routine
      character(len=240), intent(out) :: message

      integer(long), intent(in)                  :: gdim,dim,ilog
      integer(long), dimension(*),intent(in)     :: isbaspar
      integer(long)                              :: istart,e,i,linf,qnl0,qnm0
      integer(long), dimension(gdim), intent(in) :: jsph,msph

      complex(dop), dimension(gdim,dim), intent(out) :: psi

!-----------------------------------------------------------------------
! get spf basis parameters
!-----------------------------------------------------------------------
      qnl0 = isbaspar(1)
      qnm0 = isbaspar(2)

!    identification of the index istart of the basis function
!    corresponding to the initial wavepacket.

      i=0
 10   i=i+1
      if (i.gt.gdim) then
        routine = 'genphi1.F  GENSPHFBR'
        message = 'Wrong quantum number for spherical harmonics spf'
        call errormsg
      endif

      if ((jsph(i).eq.qnl0).and.(msph(i).eq.qnm0)) then
            istart=i
            goto 20
      else 
            goto 10
      endif

 20   continue 

!    initial unoccupied spf are distributed around the initial 
!    occupied spf, as symetrically as possible
!    linf = first unoccupied spf.

      linf=max(istart-dim/2,1) 

      if (linf+dim-1.gt.gdim) linf=gdim-dim+1
 
      if (linf.le.0) then
        routine = 'GENSPHFBR'
        message = 'Wrong number of spherical harmonics spf'
        call errormsg
      endif

! output to log
            write(ilog,'(/,a,/,a,i3,",",i3,/)') &
       '*** Initial wave packet  ***** ', &
       'FBR basis : spherical harmonics (j,mj): ',qnl0,qnm0

!  set psi(g,e) to zero.
      call  zeromxz(psi,gdim,dim)

      psi(istart,1)=(1.0_dop,0.0_dop)
     
      do e=2,dim
       if ( linf.eq.istart ) linf=linf+1
       psi(linf,e)=(1.0_dop,0.0_dop)
       linf=linf+1
      enddo
         
      return 
      end subroutine gensphfbr

!#######################################################################

!-----------------------------------------------------------------------
! Calls to: genhigher, mgenhigher, wiggenhigher, kgenhigher, kgenhigher2
! m     - (primitive) mode
! phi   - SPFs for this mode (one state)
! nspf0 - number of SPFs already set
! nspf  - total number of SPFs
! sdm   - size of each SPF
!-----------------------------------------------------------------------

      subroutine inventspfs(m,phi,nspf0,nspf,sdm,ort,exphin,exprueck)

use global
use maxv, only: maxdim
use dvrdatmod
use griddatmod
use psidef
use compdatmod

use rdopermod, only: get_num_klegs, get_num_wigs

      implicit none

      integer(long), intent(in)                      :: m,nspf0,nspf,sdm
      integer(long)                                  :: next,f,power,f1,&
                                                        j,n,nk,nw,kdim
      complex(dop), dimension(sdm,nspf)              :: tmpphi
      complex(dop), dimension(sdm,nspf), intent(out) :: phi
      real(dop), dimension(ortdim), intent(in)       :: ort
      real(dop)                                      :: shift,gxeq
      complex(dop), dimension(expdim), intent(in)    :: exphin,exprueck

      next=nspf0+1
      if (nspfdof(m) .gt. 1 .or. dentype .eq. 1) then
         call get_num_klegs(nk,m)
         call get_num_Wigs(nw,m)
         if (nk.eq.0 .and. nw.eq.0) then
            if (idmode(m).eq.0) &
               call mgenhigher(phi,sdm,nspf,next,m,cmsymmtr(1,1))
         elseif (nw.gt.0) then
            call wiggenhigher(phi,ort,sdm,nspf,next,m)
         elseif (nk.eq.1) then
            call kgenhigher(phi,ort,sdm,nspf,next,m)

            do n = 1, nspfdof(m)
               f = spfdof(n,m)
               if(basis(f) .eq. 9 ) then
                  kdim=ipbaspar(4,f)
                  do j=1,nspf0  ! p -> k
                    call mttxzz(exphin(zexp(f+1)),phi(1,j),tmpphi(1,j),&
                         vgdim(f+1),gdim(f+1),ngdim(f+1),kdim)
                  enddo
                  basis(f)=7  ! Temporary set basis to KLeg/K
                  basis(f+1)=8

                  call kgenhigher(tmpphi,ort,sdm,nspf,next,m)

                  do j=1,nspf ! k -> p
                    call mttxzz(exprueck(zexp(f+1)),tmpphi(1,j),phi(1,j),&
                         vgdim(f+1),kdim,ngdim(f+1),gdim(f+1))
                  enddo
                  basis(f)=9  ! Reset basis to PLeg/exp
                  basis(f+1)=5

               elseif(basis(f) .eq. 7 ) then
                  call kgenhigher(phi,ort,sdm,nspf,next,m)
               endif
! GW This goto means that the loop only ever runs to n=1??
               goto 122
            enddo

         else
            call kgenhigher2
         endif
  122    continue

      else
         f = spfdof(1,m)
         shift= 0.d0
         gxeq = 0.d0
         power= 1
         f1   = 0
         do j=1,2*maxdim ! Search for sym1d, asym1d
            if(msymmtr(j).eq.0) exit
            if(f.eq.mod(abs(msymmtr(j)),256)) then
               f1=msymmtr(j)
               power=2
               gxeq=0.5d0*(ort(zort(f))+ort(zort(f)+gdim(f)-1))
               exit
            endif
         enddo
         call genhigher(phi,sdm,nspf,next,ort(zort(f)),basis(f), &
                        ipbaspar(1,f),power,gxeq,shift,f1)
      endif

      end subroutine inventspfs

!#######################################################################
! GENHIGHER
!
! called by gen1, gen2, rstpsi
! calls to overlap, subcomp, normphi
!
! given an initial set of functions (1 or more), a higher set (starting
!  with the 'next' function) of Schmidt orthogonalised functions are
!  generated. 
!
! psi: single particle functions
! gdim: number of grid points
! dim: number of single particle functions
! next: first undefined spf
! ort: grid positions
!
! V4.0
!#######################################################################

      subroutine genhigher (psi,gdim,dim,next,ort,basis,ipbaspar, &
                            power,gxeq,shift,isym)

      implicit none

      integer(long)                           :: e,g,e1,ibleg,iv,sft
      integer(long), intent(in)               :: gdim,dim,next,basis,power,isym
      integer(long), dimension(*), intent(in) :: ipbaspar

      complex(dop)                                    :: s
      complex(dop), dimension(gdim,dim),intent(inout) :: psi

      real(dop)                              :: norm,x
      real(dop), intent(in)                  :: gxeq,shift
      real(dop), dimension(gdim), intent(in) :: ort

      ibleg=-999
      if (basis .eq. 2  .or. basis .eq. 12) then
         ibleg=ipbaspar(2)
         elseif(basis .eq. 7 .or. basis .eq. 9 .or. basis .eq. 20 .or. &
                basis .eq. 21 .or. basis .eq. 33) then
         ibleg=ipbaspar(1)
      endif

!-----------------------------------------------------------------------
!  1. Preceeding function is multiplied by co-ordinate (or function 
!     thereof)
!  2. Components of all preceeding functions removed by Schmidt-Orthog.
!  3. Function normalised
!  NB: shift is for periodic grids.
!-----------------------------------------------------------------------  

! --- Special treatment for grids with two grid points (like spin).
      if(gdim.eq.2  .and.  next.eq.2) then
         psi(1,2) =  psi(2,1)
         psi(2,2) = -psi(1,1)
         return
      endif

      do e=next,dim
         if((basis.eq.2 .or. basis.eq.7 .or. basis.eq.9 .or. &
             basis.eq.12 .or. basis.eq.20 .or. basis.eq.21 .or. &
             basis.eq.33) .and. ibleg.eq.0) then
            do g=1,gdim
               psi(g,e)=cos(ort(g))*psi(g,e-1)
            enddo
         else if((basis.eq.2 .or. basis.eq.7 .or. basis.eq.9 .or. &
                  basis.eq.12 .or. basis.eq.20 .or. basis.eq.21 .or. &
                  basis .eq. 33) .and. ibleg.gt.0 ) then
            do g=1,gdim
               psi(g,e)=(cos(ort(g))**2)*psi(g,e-1)
            enddo
         else 
            do g=1,gdim
               x=ort(g)-gxeq
               if(shift .ne. 0.0_dop ) then ! shift is for periodic grids.
                  if(abs(x) .gt. abs(x-shift)) x = x-shift
                  if(abs(x) .gt. abs(x+shift)) x = x+shift
                  if(abs(x) .gt. abs(x-shift)) x = x-shift
               endif
               x=x**power
               psi(g,e)=x*psi(g,e-1)
            enddo
         end if
! ---    Symmetrize
         if(isym.ne.0) then
            iv=isym/abs(isym)  ! iv=1,-1 for sym,asym
            do g = 1, (gdim+1)/2
               psi(g,e) = 0.5*(psi(g,e)+iv*psi(gdim+1-g,e))
               psi(gdim+1-g,e) = iv*psi(g,e)
            enddo
         endif

         sft=0
! ---    Schmidt-Orthogonalize
 123     do e1=1,e-1
            call vvaxzz(psi(1,e1),psi(1,e),s,gdim)
            call xvxxzzr(s,psi(1,e1),psi(1,e),gdim)
         enddo
         call normvxz(psi(1,e),norm,gdim)
         if( norm .lt. 1.d-9 ) then ! Shift the last SPF to create a new one.
            if(sft.ge.0) sft=sft+1
            call shiftspf(psi(1,e-1),psi(1,e),sft,gdim)
            sft=-sft
            if(sft.ge.-(gdim-1)/2 .and. sft.le.gdim/2) then
               goto 123
            else ! As nothing has worked, try random numbers.
               call randomspf(psi(1,e),gdim)
               do e1=1,e-1
                  call vvaxzz(psi(1,e1),psi(1,e),s,gdim)
                  call xvxxzzr(s,psi(1,e1),psi(1,e),gdim)
               enddo
               call normvxz(psi(1,e),norm,gdim)
               if(norm .lt. 1.d-10) norm=1.d40 ! zero WF as nothing had worked
            endif
         endif
         call xvixdzo(norm,psi(1,e),gdim) !normalize
         do e1=1,e-1                      ! Schmidt-Orthogonalize again
            call vvaxzz(psi(1,e1),psi(1,e),s,gdim)
            call xvxxzzr(s,psi(1,e1),psi(1,e),gdim)
         enddo
         call normvxz(psi(1,e),norm,gdim)
         if(norm .lt. 1.d-9) return ! Orthonormalization failed
         call xvixdzo(norm,psi(1,e),gdim)
      enddo   ! e=next,dim

      return
      end subroutine genhigher

!#######################################################################

      subroutine shiftspf(psiin,psiout,shift,gdim)

      implicit none

      integer(long), intent(in) :: shift,gdim
      integer(long)             :: i,j
      complex(dop), dimension(gdim), intent(in)   :: psiin
      complex(dop), dimension(gdim), intent(out)  :: psiout

      do i=1,gdim
         j=i+shift
         if(j.gt.gdim) j=j-gdim
         if(j.le.0)    j=j+gdim
         psiout(j)=psiin(i)
      enddo

      return
      end subroutine shiftspf

!#######################################################################

      subroutine randomspf(psi,gdim)

      implicit none

      integer(long), intent(in) ::  gdim
      integer(long)             ::  i
      complex(dop), dimension(gdim), intent(out) :: psi
      real(dop), dimension(2)   :: reim

      do i=1,gdim
         call random_number(reim)
         psi(i)=dcmplx(reim(1)-0.5d0, reim(2)-0.5d0)
      enddo

      return
      end subroutine randomspf

!#######################################################################
! MGENHIGHER
!
! called rstpsi. Generates higher multidimensional SPFs.
! HDM 06/04
!-----------------------------------------------------------------------

      subroutine mgenhigher(psi,sdm,dim1,next,m,msymmtr)

use global
use maxv, only: maxdim
use dvrdatmod
use griddatmod

      implicit none

      integer(long)                           :: n,f,e,e1,j,nspfdf,&
                                                 nndim,nc
      integer(long), intent(in)               :: sdm,dim1,m
      integer(long), intent(inout)            :: next
      integer(long), dimension(*), intent(in) :: msymmtr

      complex(dop)                                    :: ovl
      complex(dop), dimension(sdm,dim1),intent(inout) :: psi
      real(dop)                                       :: norm
      real(dop), dimension(sdm)                       :: workr

      nc=0
      do n = 1, nspfdof(m)
         f = spfdof(n,m)
         j=basis(f)
         if(j.eq.-1 .or. j.eq.6 .or. j.eq.0)  nc=nc+1
      enddo
      if(nc.ge.nspfdof(m)) then
         routine='mgenhigher (getpsiutil.F line 221)'
         message='Cannot create higher SPFs for el or sphFBR'
         call errormsg
      endif

!.... In case of 2D or 3D (a)symmetrization set nspfdof(m) to 1
!.... in order to avoid creating linearly dependent symmetrized SPFs. 
      nspfdf=nspfdof(m)
      do j = 1, 2*maxdim
         if( msymmtr(j) .eq. 0 ) then
            continue
         else
            if(abs(mod(msymmtr(j),65536)/256).eq.m) nspfdf=1 
         endif
      enddo 

!--------------------------------------------------------------------
!  Generate higher SPFs by multiplying the 1D functions of a mode
!  with a function of the grid-points. 
!--------------------------------------------------------------------
      e=next-1
      next=max(1,next-nspfdof(m))
 200  next=next+1
      do n = 1, nspfdf
         f = spfdof(n,m)
         j=basis(f)
         if(j.eq.-1 .or. j.eq.6 .or. j.eq.0)  then
            write(6,'(2a)') 'Warning: mgenhigher ', &
              'cannot create higher SPFs for el or sphFBR'
            cycle
         endif
         e=e+1
         if(basis(f).eq.4 .or. basis(f).eq.5) then ! periodic bc.
            do j=1,gdim(f)
               workr(j)=sin(2.0_dop*pi*j/dble(gdim(f)))
            enddo
         else
            do j=1,gdim(f)
               workr(j)=(2*j-1)/dble(gdim(f)) - 1.0_dop
            enddo
         endif
!.........psi(e) = x*psi(next-1)
         if (dentype .eq. 1) then
            nndim = ngdim(f)*supgdim(m)
            call dtxxdz(workr,psi(1,next-1),psi(1,e), &
                 vgdim(f),gdim(f),nndim)
         else
            call dtxxdz(workr,psi(1,next-1),psi(1,e), &
                 vgdim(f),gdim(f),ngdim(f))
         endif
         do j=1,2 ! Orthogonalise twice.
            call normvxz(psi(1,e),norm,sdm)
            call xvixdzo(norm,psi(1,e),sdm) ! normalise
!.........Schmidt-orthonormalise
            do e1=1,e-1
               call vvaxzz(psi(1,e1),psi(1,e),ovl,sdm)
               call xvxxzzr(ovl,psi(1,e1),psi(1,e),sdm)
            enddo

            if (dentype .eq. 1) call mkherm(psi(1,e1),supgdim(m))
         enddo
         call normvxz(psi(1,e),norm,sdm)
         if( norm .lt. 1.0e-3_dop ) then
!...ignore generated function, if norm after orthogonalisation is small.
            e=e-1
            write(2,'(a,e12.4,/a,i3,a,i3,a,3i4)')&
                 'mgenhigher: linear dependence. Norm =',norm,&
                 'f=',f,',  m=',m,',  next, e:',next,e
         else
            call xvixdzo(norm,psi(1,e),sdm) ! normalise
         endif
         if(e.ge.dim1)  goto 100
      enddo 
  
      if(e.lt.next) then
         routine='mgenhigher (geninwf/getpsi.F)'
         message='Cannot generate higher SPFs due to linear dependence.'
         call errormsg
      endif
   
      goto 200

 100  return

       end subroutine mgenhigher

!#######################################################################
! KGENHIGHER
!
! Generates higher SPFs by multiplying the 2D KLEG-functions with cosine
! or sine on the theta-grid and shifting in by (+ -)m on the K-grid.
!
! Note: For non-Kleg dofs the maximum excitation number is set to emax.
!       It can be changed depending on the problem (the role of non-Kleg
!       excitation).
!
! For 2 dofs in the mode the excitation reads:
!     
!  (r)  (j,m)
!   1    0,0
!   1    1,0
!   1    1,-1
!   1    1,1
!   2    0,0
!   2    1,0
!    ....
!
! 06.04 SS
!
!-----------------------------------------------------------------------
! psi : SPFs for this mode (and state)
! ort : DVR grid points
! sdm : number of grid points for this mode (subdim)
! dim1: number of SPFs for this mode
! next: number of next SPF to be generated
!       (SPFs 1,...,next-1 are already generated)
! m   : number of mode
! v   : workspace
!-----------------------------------------------------------------------

      subroutine kgenhigher(psi,ort,sdm,dim1,next,m)

use global
use dvrdatmod
use griddatmod

use operutil, only: vshiftk3
use rdopermod, only: get_nth_kleg,iskleg

      implicit none

      integer(long)                :: n,f,e,e1,e2,k,f1,l,p,g,vg,ng,emax,ibleg,&
                                      enext,enok,ibleg1,zeig,power,le,dkmax
      integer(long), intent(in)    :: sdm,dim1,m
      integer(long), intent(inout) :: next

      complex(dop)                                     :: s
      complex(dop), dimension(sdm,dim1), intent(inout) :: psi
      real(dop)                                        :: norm,theta,x
      real(dop), dimension(ortdim), intent(in)         :: ort
      real(dop), dimension(sdm)                        :: v

      if(next.gt.dim1) return    ! nothing to be done

!-----emax = Maximal excitation of other dofs in the mode.
      emax=2 ! TODO: make configurable

      call get_nth_kleg(f1,1,m) ! f1 is DOF of first (and only) KLeg or PLeg
      ibleg=ipbaspar(1,f1)      ! in mode m
      dkmax=min(gdim(f1+1)-1,gdim(f1+1)/2+1)  ! Allowed range of k-shift

      next=next-1
      enext=next
      enok=0
      l=0
 200  l=l+1
      if (ibleg.gt.0) l=l+1
      le=0
 220  le=le+1
      if (ibleg.gt.0) le=le+1
      if(nspfdof(m).eq.2 .and. iskleg(spfdof(1,m),basis)) le=l ! Dieter
      e=next
      do n = 1, nspfdof(m)
         f = spfdof(n,m)
         if (iskleg(f,basis)) then   ! KLeg or PLeg
            p=0
  10        e=e+1
            p=p+1
            k=p/2
            if (mod(p,2).eq.0) k=-k
            do vg=1,vgdim(f)
               do g=1,gdim(f)
                  do ng=1,ngdim(f+1)
                     theta=ort(zort(f)+g-1)
                     zeig=vg+(g-1)*vgdim(f)+(ng-1)*vgdim(f+1)
                     v(zeig)=cos(theta)**(l-abs(k)) * &
                             sin(theta)**(abs(k))
                     if (mod(p,2).eq.0) v(zeig)=-v(zeig)
                  enddo
               enddo
            enddo
            call vshiftk3(v,k,psi(1,enext),psi(1,e),vgdim(f+1),&
                          gdim(f+1),ngdim(f+1))
            call normvxz(psi(1,e),norm,sdm)
!            write(6,'(a,3i4,a,2i5,1pe11.2)')'k, l, le',k,l,le,
!     +          ' , enext e, norm',enext,e,norm ! Dieter
            if(norm.le.1.d-7) then
               if(e.gt.next+1) e=e-1
               goto 100
            endif
            if (e.ge.dim1)  goto 100 ! Last spf reached->orthogonalize
            if (k.ge.dkmax) goto 100 ! max k-shift reached->orth'gol'iz
            if (p.lt.2*le+1) goto 10 ! more k-shifts to go->keep going
            if (le.lt.l)    goto 220 ! go to next le value
         elseif (basis(f).ne.8) then  ! not K
            e=e+1                     ! For exciting dofs
            enok=enok+1               ! other than KLeg or K in mode.
            if (enok.gt.emax) then
               e=e-1
               goto 20
            endif
            if (basis(f).eq.2 .or. basis(f).eq.12) then ! Leg, Leg/r
               ibleg1=ipbaspar(2,f)
               power=1
               if (ibleg1.gt.0) power=2
               do g=1,gdim(f)
                  v(g)=cos(ort(zort(f)+g-1))**power
               enddo
            else
               power=1
               do g=1,gdim(f)
                  x=ort(zort(f)+g-1)
                  v(g)=x**power
               enddo
            endif
            call dtxxdz(v,psi(1,enext),psi(1,e),&
                        vgdim(f),gdim(f),ngdim(f))
            enext=e
         endif
  20     if (e.ge.dim1)  goto 100
      enddo   

!--------------------------------------------------------------------
!  Schmidt-orthonormalize
!--------------------------------------------------------------------

 100  do e2 = next+1, e
         do e1=1,e2-1
            call vvaxzz(psi(1,e1),psi(1,e2),s,sdm)
            call xvxxzzr(s,psi(1,e1),psi(1,e2),sdm)
         enddo
         call normvxz(psi(1,e2),norm,sdm)
         if(norm .le. 1.d-10) then
            write(6,'(/a,i3,a,e9.2,a,i3)') &
                 'kgenhigher:  mode =',m,',  norm =',norm,',  e2 =',e2
            write(2,'(/a,i3,a,e9.2,a,i3)') &
                 'kgenhigher:  mode =',m,',  norm =',norm,',  e2 =',e2
            routine='kgenhigher'
            message='Cannot generate enough new SPFs'
            call errormsg
         endif
         call xvixdzo(norm,psi(1,e2),sdm)   ! normalize
      enddo

      do e2 = next+1, e  ! Do normalization twice
         do e1=1,e2-1
            call vvaxzz(psi(1,e1),psi(1,e2),s,sdm)
            call xvxxzzr(s,psi(1,e1),psi(1,e2),sdm)
         enddo
         call normvxz(psi(1,e2),norm,sdm)
         call xvixdzo(norm,psi(1,e2),sdm)   ! normalize
      enddo

      enext=next+1
      next=e
      if (next.lt.dim1) goto 200

      return
      end subroutine kgenhigher

!#######################################################################
! KGENHIGHER2
!
! Like KGENHIGHER but with the possibility of having several KLegs
! in one mode.
!
! E.g. for 2 Klegs and 1 other DOF in the mode the excitation reads:
!     
!  (r)  (j1,m1)  (j2,m2)
!   1    0,0      0,0
!   1    1,0      0,0
!   1    0,0      1,0
!   1    1,-1     0,0
!   1    0,0      1,1
!   1    1,1      0,0
!   1    0,0      1,-1
!
!   2    0,0      0,0
!   2    1,0      0,0
!   ... (as above)
!   2    0,0      1,-1
!
!   2    1,0      1,0
!   2    1,-1     1,1
!   2    1,1      1,-1
!   2    1,-1     1,0
!   2    1,0      1,1
!   2    1,1      1,0
!   2    1,0      1,-1
!   2    1,-1     1,-1
!   2    1,1      1,1
!
!   2    2,0      0,0
!   ...
!
! 05/2006 FO
!
!-----------------------------------------------------------------------
! psi : SPFs for this mode (and state)
! ort : DVR grid points
! sdm : number of grid points for this mode
! dim1: number of SPFs for this mode
! next: number of next SPF to be generated
!       (SPFs 1,...,next-1 are already generated)
! m   : number of mode
! v   : workspace
!-----------------------------------------------------------------------

      subroutine kgenhigher2
!      subroutine kgenhigher2(psi,ort,sdm,dim1,next,m)

use global
use griddatmod

      implicit none

!     integer(long), intent(in)                    :: sdm,dim1,next,m
!     complex(dop), dimension(sdm,dim1),intent(in) :: psi
!     real(dop), dimension(ortdim), intent(in)     :: ort

! Hm, this is difficult.
! Idea:
! First create a (long) list of possible configurations,
! then sort them by the following criteria:
! - lowest overall excitation
! - prefer KLeg excitations
! - when several KLegs are excited, minimize Delta(k1+k2)
! - ...
! This involves the sorting of a complex data structure, so
! it is best done from C (thus we can use qsort).

      write(6,'(/a)') ' KGENHIGHER2 not implemented yet! Sorry.'
     call stopnow

      end subroutine kgenhigher2 ! of kgenhigher2

!#######################################################################
! WIGGENHIGHER
!
! Generates higher SPFs by generating the appropriate Wigner or Kleg
! function
!
! Note: non-Kleg and non-Wigner DOFs are excited only up to emax.
!       This can be changed depending on the problem (the role of
!       non-Kleg or non-Wigner excitation).
!
! For 5 dofs in the mode the excitation reads:
!
!  (r)  (j,m)  (j,k,m)    (r)  (j,m)  (j,k,m)
!   1    0,0    0,0,0      2    1,0    1,0,0
!   2    0,0    0,0,0      2    0,0    1,1,0
!   1    1,0    0,0,0      1    1,1    0,0,0
!   1    0,0    1,0,0      1    1,-1   1,0,0
!   2    1,0    0,0,0      1    1,0    1,1,0
!   2    0,0    1,0,0      1    0,0    1,0,-1
!   1    1,-1   0,0,0      2    1,1    0,0,0
!   1    1,0    1,0,0      2    1,-1   1,0,0
!   1    0,0    1,1,0      2    1,0    1,1,0
!   2    1,-1   0,0,0      2    0,0    1,0,-1
!                               ...
! 03/13 PT
!
!-----------------------------------------------------------------------
! psi   : SPFs for this mode (and state)
! ort   : DVR grid points
! sdm   : number of grid points for this mode (subdim)
! dim1  : number of SPFs for this mode
! next  : number of next SPF to be generated
!         (SPFs 1,...,next-1 are already generated)
! m     : number of mode
! workr : workspace
!-----------------------------------------------------------------------

      subroutine wiggenhigher(psi,ort,sdm,dim1,next,m)

use global
use maxv, only:maxdim
use dvrdatmod
use griddatmod

use operutil, only: vshiftk3,wshiftk3
use rdopermod, only: iskleg,iswig

      implicit none

      integer(long), intent(in) :: sdm,dim1,next,m
      integer(long) :: n,f,e,j,k,mm,g,vg,ng,emax
      integer(long) :: ibleg,zeig,power,i,wigtocalc,l2,n1,n2,f1,e1,e2
      integer(long) :: klwmax,nnok,prod,nspfdf,ll,llm,ulm,nngdim
      integer(long), dimension(maxdim,dim1) :: wptr,AV
      integer(long), dimension(dim1)        :: lnz
      integer(long), dimension(maxdim)      :: dkmax,wmax
      integer(long), dimension(3,dim1)      :: inds
      complex(dop), dimension(sdm,dim1)     :: psi
      complex(dop)                          :: s
      real(dop), dimension(sdm)             ::  ort,workr
      real(dop), dimension(sdm,dim1)        ::  wignrset
      real(dop)                             ::  norm,x

! Hm, this is even more difficult. Since MCTDH does have the INWF block
! from the previous run, it not "know" which k- and m- states were
! previously generated, so it does not know where to begin building the
! new spfs. Therefore, the danger exists of either generating new spfs
! that are too low, thereby regenerating old spfs (which will cause
! problems when the spfs are Schmidt orthoginalized) or of starting
! with new spfs that are too high, resulting in "gaps" in the INWF.
! This subroutine will have to wait for an ambitious programmer with
! lots of time...

      write(6,'(a)') "Error in geninwf/getpsi.F: wiggenhigher()"
      write(6,'(a)') "Subroutine WIGGENHIGHER is not yet implemented!"
      call stopnow

!.....emax = Maximal excitation of non-kleg and non-Wigner DOFs in mode.
      emax=2 ! TODO: make configurable.

      nspfdf = nspfdof(m)

! --- First, count the non-Kleg/Pleg/Wigner/K DOFS; also initialize the
!     pointers for spf excitation ('AV'), k and m-shifts ('dkmax'), the
!     Wigners themselves ('wptr'), and the number of Wigner spfs
!     generated so far ('wmax')
      nnok=0
      do n=1,nspfdf
         f = spfdof(n,m)
         if (isWig(f,basis)) then    ! Wigner: calc max k- and m-shifts
            dkmax(n+1)=min(ipbaspar(4,f)-1,ipbaspar(4,f)/2+1)
            dkmax(n+2)=min(ipbaspar(8,f)-1,ipbaspar(8,f)/2+1)
         elseif (iskleg(f,basis)) then ! Kleg/Pleg: calc max k-shift
            if (basis(f).eq.7) then
               dkmax(n+1)=min(gdim(f+1)-1,gdim(f+1)/2+1)
            elseif (basis(f).eq.9) then
               dkmax(n+1)=min(ipbaspar(4,f)-1,ipbaspar(4,f)/2+1)
            endif
         elseif (basis(f).ne.8) then
            nnok=nnok+1  ! Count non-Kleg/Pleg/Wigner/K DOFs in mode
         endif
         AV(n,next-1)=1  ! Excitation pointers for spfs
         wptr(n,1)=1     ! Pointers for Kleg/Wigner spfs to be used
         wmax(n)=1       ! Max number of Kleg/Wigner spfs to calc
      enddo

! --- Next, determine the number of product spfs involving either Klegs
!     or Wigners ('klwmax'). Since the Wigner code also generates Kleg
!     spfs (they are Wigner functions with k=0), we will use it to
!     generate Wigners and Klegs simultaneously, stopping when the
!     product of the spfs exceeds klwmax

      klwmax=(dim1-next+1)/(nnok**emax)
      prod=1
      j=0
      k=0
      mm=0
      i=1
      do while (prod.lt.klwmax)
         call nextwig(j,k,mm)
         i=i+1
         prod=1
         do n=1,nspfdf
           f = spfdof(n,m)
           if (isWig(f,basis)) then
              ! Use this Wigner only if k and m are within the max shift
              if (abs(k).lt.dkmax(n+1) .and. abs(mm).lt.dkmax(n+2)) then
                 wmax(n)=wmax(n)+1
                 wptr(n,wmax(n))=i
              endif
              ! Use this Kleg (k=0) only if m is within the max shift
           elseif (iskleg(f,basis)) then
              if (k.eq.0 .and. abs(mm).lt.dkmax(n+1)) then
                 wmax(n)=wmax(n)+1
                 wptr(n,wmax(n))=i
              endif
           endif
           prod=prod*wmax(n)
         enddo
      enddo
! --- The last value of the counter 'i' is the number of Wigners that
!     need to be generated
      wigtocalc=i

! --- Calculate the excitation scheme for the spfs (non-K DOFs only),
!     beginning with the "next"th spf
      e=next
      llm=e-1
      ulm=e-1
      lnz(e-1)=1
      do
         ll=1
         do n1=llm,ulm  ! Loop over previously calculated spfs
            l2=lnz(n1)
            do n2=l2,nspfdf  ! Loop over remaining DOF to excite
               f=spfdof(n2,m)
               if (basis(f).eq.8) CYCLE    ! Skip Ks
               ! emax to be exceeded for non-Kleg or non-Wigner: skip
               if (basis(f).ne.7 .and. basis(f).ne.9 .and. &
                   basis(f).ne.33 .and. AV(n2,n1).eq.emax) CYCLE
               lnz(e)=n2
               do n=1,nspfdf
                  f1=spfdof(n,m)
                  if (basis(f1).eq.8) CYCLE ! Skip Ks
                  AV(n,e)=AV(n,n1)
                  if (n.eq.n2) AV(n,e)=AV(n,e)+1 ! Excite the spf
               enddo
               ! Multiply by function of x (non-Klegs and non-Wigners)
               if (basis(f).eq.2 .or. basis(f).eq.12) then
                  ibleg=ipbaspar(2,f)
                  power=1
                  if (ibleg.gt.0) power=2
                  do g=1,gdim(f)
                     workr(g)=cos(ort(zort(f)+g-1))**power
                  enddo
               elseif (basis(f).eq.4 .or. basis(f).eq.5) then
                  do g=1,gdim(f)
                     workr(g)=sin(2.d0*pi*g/dble(gdim(f)))
                  enddo
               elseif (basis(f).ne.7 .and. basis(f).ne.9 .and. &
                       basis(f).ne.33) then
                  power=1
                  do g=1,gdim(f)
                     x=ort(zort(f)+g-1)
                     workr(g)=x**power
                  enddo
               endif
               call dtxxdz(workr,psi(1,n1),psi(1,e),vgdim(f),gdim(f),ngdim(f))
               ll=ll+1
               e=e+1
               if (e.gt.dim1) GOTO 100
            enddo
         enddo
         llm=ulm+1
         ulm=ulm+ll
      enddo

 100  continue

! --- Now that all of the non-Kleg and non-Wigner spfs are generated,
!     populate a list of Kleg/Wigner spfs, and use the indices stored
!     in 'AV' and 'wptr' to extract the correct function

      do n=1,nspfdf
         f=spfdof(n,m)
         if (iskleg(f,basis) .or. isWig(f,basis)) then
            call wigset(inds,ort(zort(f)),gdim(f),wigtocalc,wignrset)
            if (isWig(f,basis)) then
               nngdim=ngdim(f+2)
            elseif (iskleg(f,basis)) then
               nngdim=ngdim(f+1)
            endif
            do e=next,dim1
               do vg=1,vgdim(f)
                  do g=1,gdim(f)
                     do ng=1,nngdim
                        zeig=vg+(g-1)*vgdim(f)+(ng-1)*vgdim(f+1)
                        workr(zeig)=wignrset(g,wptr(n,AV(n,e)))
                     enddo
                  enddo
               enddo
               k=inds(2,wptr(n,AV(n,e)))
               mm=inds(3,wptr(n,AV(n,e)))
               if (isWig(f,basis)) then
                  if (basis(f+1).ne.8) k=0
                  if (basis(f+2).ne.8) mm=0
                  call wshiftk3(workr,k,mm,psi(1,next-1),psi(1,e), &
                              vgdim(f+1),gdim(f+1),gdim(f+2),ngdim(f+2))
               elseif (iskleg(f,basis)) then
                  if (basis(f+1).ne.8) mm=0
                  call vshiftk3(workr,mm,psi(1,next-1),psi(1,e), &
                              vgdim(f+1),gdim(f+1),ngdim(f+1))
               endif
            enddo
         endif
      enddo

! --- Finally, Schmidt-orthonormalize

      do e2 = next,dim1
         do e1=1,e2-1
            call vvaxzz(psi(1,e1),psi(1,e2),s,sdm)
            call xvxxzzr(s,psi(1,e1),psi(1,e2),sdm)
         enddo
         call normvxz(psi(1,e2),norm,sdm)
         if(norm .le. 1.d-10) then
            write(6,'(/a,i3,a,e9.2,a,i3)') &
            'wiggenhigher:  mode =',m,',  norm =',norm,',  e2 =',e2
            write(2,'(/a,i3,a,e9.2,a,i3)') &
            'wiggenhigher:  mode =',m,',  norm =',norm,',  e2 =',e2
            routine='wiggenhigher'
            message='Cannot generate enough new SPFs'
            call errormsg
         endif
         call xvixdzo(norm,psi(1,e2),sdm)   ! normalize
      enddo

      return
      end subroutine wiggenhigher

!#######################################################################
!
!               WIGSET
!
!           PT 03/14/2013
!
!  Generates the set of 1D Wigner spfs for Wiggenhigher() using the
!   sequence determined by nextwig()
!
!#######################################################################

      subroutine wigset(inds,x,gdim,dim1,wignrset)

use global

      implicit none

      integer(long), intent(in)                     :: dim1,gdim
      integer(long)                                 :: j,k,m,g,n
      integer(long), dimension(3,dim1), intent(out) :: inds
      real(dop), dimension(gdim), intent(in)        :: x
      real(dop), dimension(gdim)                    :: cx
      real(dop), dimension(gdim,dim1), intent(out)  :: wignrset
      real(dop)                                     :: djmk

! --- Transform angle beta --> cos(beta)
      do g=1,gdim
         cx(g)=cos(x(g))
      enddo

! --- Fill the wignrset array with Wigner functions
      j=0
      k=0
      m=0
      do n = 1, dim1
         do g=1,gdim
            wignrset(g,n) = djmk(j,m,k,cx(g))
         enddo
         inds(1,n)=j
         inds(2,n)=k
         inds(3,n)=m
         call nextwig(j,k,m)
      enddo

      return
      end subroutine wigset

!#######################################################################
!
!               NEXTWIG
!
!           PT  03/01/2013
!
!  Determines j,k,m values of next Wigner (d^j_mk) function
!    in the sequence:
!
!  (j) (k) (m)     (j) (k) (m)     (j) (k) (m)     (j) (k) (m)
!   0   0   0       2   0   0       2   1  -1       2  -1   2
!   1   0   0       2   1   0       2  -1  -1       2   2   2
!   1   1   0       2   0  -1       2  -1   1       2   2  -2
!   1   0  -1       2  -1   0       2   2   1       2  -2  -2
!   1  -1   0       2   0   1       2  -1  -2       2  -2   2
!   1   0   1       2   2   0       2  -2  -1       3   0   0
!   1   1   1       2   0  -2       2   1   2       3   1   0
!   1   1  -1       2  -2   0       2   2  -1       3   0  -1
!   1  -1  -1       2   0   2       2   1  -2       3  -1   0
!   1  -1   1       2   1   1       2  -2   1          ...
!
!#######################################################################

      subroutine nextwig(j,k,m)

      implicit none

      integer(long), intent(inout) :: j,k,m
      integer(long)                :: tmp,kf,mf,extot

! --- First, determine symmetry rule
      kf = -1
      mf = -1
      if (abs(m).ge.abs(k)) then
         kf = -kf
         if (m.lt.k .and. abs(m).ne.abs(k)) mf = -mf
      endif

! --- Swap k and m (and signs as needed)
      tmp = m
      m = k*mf
      k = tmp*kf

      if (k.ge.0 .and. m.ge.0 .and. k.ge.m) then  ! do next m,k pair
         extot = k + m
         k = k - 1
         m = m + 1
         if (m.gt.k) then  ! increase total m,k excitation level
            extot = extot + 1
            k = extot
            m = 0
            do while (k.gt.j)  ! if new k is too large, redistribute
               k = k - 1       !   quanta k --> m
               m = m + 1
               if (m.gt.j) then  ! if m becomes too large, do next j
                  j = j + 1
                  k = 0
                  m = 0
               endif
            enddo
         endif
      endif

      return
      end subroutine nextwig

!#######################################################################
!                   EXCITE1
! This routine generates the excitation scheme for multi-mode SPFs.
! This routine is used when symmetries are used.
! For e.g. a three-mode combination the excitation scheme reads:
!   1)    1  1  1
!   2)    2  1  1
!   3)    2  2  1
!   4)    2  2  2
!   5)    3  1  1
!   6)    3  2  1
!   7)    3  2  2
!   8)    3  3  1
!   9)    3  3  2
!  10)    3  3  3
!  11)    4  1  1
!  12)    4  2  1
!  13)    4  2  2
!  14)    4  3  1
!  15)    4  3  2
!  16)    4  3  3
!  17)    4  4  1
!  18)    4  4  2
!      etc.
!------------------------------------------------------------------------

      subroutine excite1(f,ex,b)


      implicit none

      integer(long)                             :: i
      integer(long), intent(in)                 :: f
      integer(long), intent(inout)              :: ex
      integer(long), dimension(f),intent(inout) :: b
!.....f=number of DOFs in mode, ex=index to be excited, b=index-array

!.....Increase the first index, if all indices are equal.
      if( b(1) .eq. b(f) ) then
         b(1)=b(1)+1
         ex=1
         do i = 2, f
            b(i) = 1
         enddo
     
!.....Increase the last index, which differs. Set the rest to 1.
      elseif(ex.eq.0) then
         do i = f, 2, -1
            ex=i
            if(b(i-1).gt.b(i)) goto 12
         enddo
  12     b(ex) = b(ex)+1
         do i = ex+1, f
            b(i) = 1
         enddo  
         if(ex.eq.f)  ex=0

!.....determine next configuration (regular step).
      else
         ex=ex+1
         b(ex)=b(ex)+1
         if(ex.eq.f)  ex=0
      endif

      return
      end subroutine excite1

!#######################################################################
!                   EXCITE2
! This routine generates the excitation scheme for multi-mode SPFs.
! For e.g. a three-mode combination the excitation scheme reads:
!   1)    1  1  1
!   2)    2  1  1
!   3)    1  2  1
!   4)    1  1  2
!   5)    3  1  1
!   6)    2  2  1
!   7)    2  1  2
!   8)    1  3  1
!   9)    1  2  2
!  10)    1  1  3
!  11)    4  1  1
!  12)    3  2  1
!  13)    3  1  2
!  14)    2  3  1
!  15)    2  2  2
!  16)    2  1  3
!  17)    1  4  1
!  18)    1  3  2
!  19)    1  2  3
!  20)    1  1  4
!  21)    5  1  1
!  22)    4  2  1
!  23)    4  1  2
!  24)    3  3  1
!  25)    3  2  2
!      etc.
!------------------------------------------------------------------------

      subroutine excite2(f,ex,b)

use maxv, only: maxdim

      implicit none

      integer(long)                             :: i,fh
      integer(long), dimension(maxdim)          :: c
      integer(long), intent(in)                 :: f
      integer(long), intent(inout)              :: ex
      integer(long), dimension(f),intent(inout) :: b
!     f=number of DOFs in mode, ex=degree of excitation, b=index-array

      if (f.gt.1) then
!     initialize c-vector
         c(1) = ex
         do i=2,(f-1)
            c(i) = c(i-1) - b(i-1) + 1
         enddo
         c(f) = b(f) - 1
         fh = f

!     determine next configuration
 10   continue
         c(fh) = c(fh) + 1
         if (c(fh) .gt. c(fh-1)) then

!     all configurations for ex are already calculated, set
!     ex = ex+1 and start with first higher configuration
            if (fh .eq. 2) then
               ex = ex + 1
               b(1) = ex + 1
               do i=2,f
                  b(i) = 1
               enddo
               return
            endif

            c(fh) = 0
            fh = fh - 1
            goto 10
         endif

!     calculate next configuration for ex
         do i=1,(f-1)
            b(i) = c(i) - c(i+1) + 1
         enddo
         b(f) = c(f) + 1
      else  ! simple case: only one DOF to excite
         b(1) = b(1) + 1
      endif

      return 
      end subroutine excite2

!***********************************************************************
! Multiply the multi-dimensional SPF for one mode with a single-DOF
! one-dimensional SPF.
!
! spfs: array of 1D SPFs
! psi : array of multi-D mode SPFs
! indx: index of the 1D SPF with which we wish to multiply
! nspf: index of the multi-D SPF to be multiplied
! gdim: (as usual) size of the 1D grid
! vgdim: (as usual) vor-dimension
! ngdim: (as usual) nach-dimension
! dim: (as usual) number of 1D SPFs altogether (isn't really used here)
!
! WARNING: documentation not by original author! (FO 02/2006)
!***********************************************************************

      subroutine multspf(spfs,psi,indx,vgdim,gdim,ngdim,nspf,dim)

      implicit none

      integer(long)             :: n,v,g
      integer(long), intent(in) :: vgdim,gdim,ngdim,nspf,dim,indx

      complex(dop), dimension(vgdim,gdim,ngdim,dim), intent(inout) :: psi
      complex(dop), dimension(gdim,dim), intent(in)                :: spfs

      do n=1,ngdim
         do g=1,gdim
            do v=1,vgdim
               psi(v,g,n,nspf)=psi(v,g,n,nspf)*spfs(g,indx)
            enddo
         enddo
      enddo

      return
      end subroutine multspf

!#######################################################################
! 
! if initally populated spf is not the first in the list of generated
! functions, swap it with the first spf.
! remaining functions are then listed: n, n-1, n+1, n-2, n+2 ....
!
!#######################################################################

      subroutine swapspf(psi,inspf,gdim,dim)

      implicit none

      integer(long)                                    :: e,e1,n
      integer(long), intent(in)                        :: gdim,dim,inspf
      complex(dop), dimension(gdim,dim), intent(inout) :: psi
      complex(dop), dimension(gdim,dim)                :: workc

      call cpvxz(psi(1,inspf),workc(1,1),gdim)
      e=inspf
      e1=inspf
      n=1
   10 continue
      e=e-1
      e1=e1+1
      if (e .ge. 1) then
         n=n+1
         call cpvxz(psi(1,e),workc(1,n),gdim)
      endif
      if (e1 .le. dim) then
         n=n+1
         call cpvxz(psi(1,e1),workc(1,n),gdim)
      endif
      if (n .lt. dim) go to 10

      call cpmxz(workc,psi,gdim,dim)
      
      return
      end subroutine swapspf

!#######################################################################

      subroutine ddhigher(xi,phi,gdof,gwpdim1,state,agmat,xivec,phase)

use global
use griddatmod
use psidef
use channels

use maxv,   only: maxdim
use gwplib, only: center, normgwp

      implicit none

      integer(long)             :: i,iz,iz0,natm,n,j,e1,g,ic,ierr
      integer(long), intent(in) :: gdof,gwpdim1,state

      real(dop)                         :: step,norm,fac
      real(dop), dimension(3)           :: xcoo
      real(dop), dimension(maxdim)      :: dcp,force,qcenter

      complex(dop), dimension(maxdim)                           :: qcenter1
      complex(dop), dimension(gwpdim1), intent(out)             :: phi
      complex(dop), dimension(gdof,gwpdim1), intent(inout)      :: xi
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(inout) :: agmat
      complex(dop), dimension(gdof,gwpdim1), intent(inout)      :: xivec
      complex(dop), dimension(gwpdim1), intent(inout)           :: phase

      character(len=10)                 :: aname
      logical(kind=4)                   :: check

      inquire(iin,opened=check)
      if (.not. check) then
         routine='DDhigher'
         message='Input file not open'
         call errormsg
      endif

      natm=gdof/3
      step=2.0_dop

      rewind(iin)
      iz0=0
!-----------------------------------------------------------------------
! FIND SECTION
!-----------------------------------------------------------------------
      routine = 'read INITDD-SECTION  (ddhigher)'
      iz=0
   10 continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         message = inptit
         call errormsg
      endif

      if (keyword(1).eq.'initdd-section') then
         go to 12
      elseif (keyword(1).eq.'end-input') then
         message='No INITDD-SECTION in input file'
         call errormsg
      else
         go to 10
      endif

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
   12 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (keyword(i).eq.'derivative-coupling') then
         j=0
         do n=1,natm
            read(iin,*,err=999) aname,(xcoo(i),i=1,3)
            do i=1,3
               j=j+1
               dcp(j)=xcoo(i)
            enddo
          enddo

      elseif (keyword(i).eq.'initial-force') then
         j=0
         do n=1,natm
            read(iin,*,err=999) aname,(xcoo(i),i=1,3)
            do i=1,3
               j=j+1
               force(j)=xcoo(i)
            enddo
         enddo

      elseif (keyword(i).eq.'step') then

         if (keyword(i+1) .ne. '=') then
            message = 'No argument specified with step keyword'
            call warnmsg
         else
            i=i+2
            read(keyword(i),*,iostat=ierr) step
         endif

      else if (keyword(i).eq.'end-initdd-section') then
         go to 20
      else
         message = ' unknown keyword : '
         call wrline(i,iz,iz0)
      end if

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 12
      endif

 20   continue

!-----------------------------------------------------------------------
! normalise force and dcp vectors
!-----------------------------------------------------------------------
      call sum2vxd(force,norm,gdof)
      norm = dsqrt(norm)
      call xvixddo(norm,force,gdof)
 
      call sum2vxd(dcp,norm,gdof)
      norm = dsqrt(norm)
      call xvixddo(norm,dcp,gdof)

!-----------------------------------------------------------------------
! get center of initial packet
!-----------------------------------------------------------------------
      call center(gdof,1,xi,qcenter)

!-----------------------------------------------------------------------
! generate new functions
!-----------------------------------------------------------------------
      e1 = 1
   50 continue
      if (state .eq. 2) then
         if (e1 .eq. 1) then
            e1 = e1+1
            go to 50
         else if (e1 .eq. 2) then
            do g=1,gdof
               qcenter1(g) = qcenter(g) + step*force(g)
            enddo
         else if (e1 .eq. 3) then
            do g=1,gdof
               qcenter1(g) = qcenter(g) - step*force(g)
            enddo
         else if (e1 .eq. 4) then
            do g=1,gdof
               qcenter1(g) = qcenter(g) - step*dcp(g)
            enddo
         else if (e1 .eq. 5) then
            do g=1,gdof
               qcenter1(g) = qcenter(g) + step*dcp(g)
            enddo
         endif

      else if (state .eq. 1) then
         if (e1 .eq. 1) then
            do g=1,gdof
               qcenter1(g) = qcenter(g) - step*dcp(g)
            enddo
         else if (e1 .eq. 2) then
            do g=1,gdof
               qcenter1(g) = qcenter(g) + step*dcp(g)
            enddo
         else if (e1 .eq. 3) then
            do g=1,gdof
               qcenter1(g) = qcenter(g) + step*force(g)
            enddo
         else if (e1 .eq. 4) then
            do g=1,gdof
               qcenter1(g) = qcenter(g) - step*force(g)
            enddo
         else if (e1 .eq. 5) then
            do g=1,gdof
               qcenter1(g) = qcenter(g) 
            enddo
         endif

      endif

      call cpqxz(agmat(1,1,1),agmat(1,1,e1),gdof)
!
! get new linear GWP parameter
!
      call qvxxzz(agmat(1,1,e1),qcenter1,xivec(1,e1),gdof)
      fac=-2.0_dop
      call xvxxdzo(fac,xivec(1,e1),gdof)
      xi(:,e1) = xivec(:,e1)

      if (e1 .lt. gwpdim1) then
         e1 = e1+1
         go to 50
      endif
!
! normalise new functions
!
      call zerovxz(phase,gwpdim1)
      call normgwp(gdof,gwpdim1,agmat,xivec,phase)
      if (lgwpphase) phi = phase
      
      return
  999 message='Error reading input file'
      call errormsg
      end subroutine ddhigher

!***********************************************************************
! 
!            MCTDH subroutine GENTDPOINT
!                
! 
! sets up single particle functions for a driven (TDpoint) basis set. 
! There is 1 spf populated at the relevant point
!
! variables passed for one mode:
!  psi: single particle functions (states)
!  gdim: No. of grid points
!
!***********************************************************************

      subroutine gentdpoint (f1,s1,psi,gdim1,isbaspar,ort)

use global
use dvrdatmod
use griddatmod
use datenmod, only: tinit
use tdkoeffs, only: calctdp1

      implicit none

      integer(long)                               :: nham,m
      integer(long), intent(in)                   :: f1,s1,gdim1
      integer(long), dimension(*),intent(in)      :: isbaspar
      real(dop), dimension(gdim1), intent(in)     :: ort
      complex(dop), dimension(gdim1), intent(out) :: psi
      
!-----------------------------------------------------------------------
! Check f not in a combined mode
!-----------------------------------------------------------------------
      m = dofspf(f1)
      if (nspfdof(m) .gt. 1) then
         routine = 'Gentdpoint (genphi1) '
         write(message,'(a,i5,a)') &
               'TDPoint DOF ',f1,' cannot be in a combined mode'
         call warnmsg
      endif

!-----------------------------------------------------------------------
! Check primitive basis suitable
!-----------------------------------------------------------------------
      if (basis(f1) .lt. 3 .or. basis(f1) .gt. 5) then
         routine = 'Gentdpoint (genphi1) '
         message = 'For a driven point, primitive basis must be '// &
                   'equidistant, i.e. sin, FFT or exp'
         call warnmsg
      endif

!-----------------------------------------------------------------------
! TO ADD: Check operator suitable (no KE for tdpoint in system, 
! nham is only a TD function)
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! evaluate function at tinit
!-----------------------------------------------------------------------
      nham = isbaspar(1)
      call calctdp1(tinit,s1,nham,psi,ort,gdim1)

      return
      end subroutine gentdpoint

      end module genphi1

