!***********************************************************************
! contains : genphi,vmcgingeo
!
!***********************************************************************

      module genphimod

use decimal, only: long, dop
use logdat
use global
use dvrdatmod
use griddatmod
use runinwfmod
use dirdyn
use channels
use psidef
use operdef
use datenmod

use op1lib,  only: zerovxz, zerovxd, cpvxd, zerovxi, cpmxd, cpvidz,&
                   cpvzdz
use schmidtorthomod, only: schmidtortho,schmidtortho2
use psiutil, only: plegrueck
use genphi1, only: clgauss,gendisc,rdgwp,genkleg,gengauss,genaleg,&
                   gengwp,xlocgauss,gensphfbr,genwigner,gentdpoint,&
                   genran,genreadspf,geneuclid,swapspf
      implicit none
      private
      public :: genphi,veigen

      contains
!***********************************************************************
!
!               MCTDH   subroutine  GENPHI
!
!  11/96 AJ      
!  12/01 HDM merged with dgenphi 
!
! basis(f) : pbasis type for dof f. (dof=degree of freedom)
!(phifbr), el, HO, Leg, sin, FFT, exp, sphfbr, kleg, k, pleg, rHO, Leg/r
!   -1      0   1    2    3    4    5      6    7    8    9    11   12  
! external, cos, Lagu1, Lagu2, Lagu3, Lagu4, point, kleg/r, pleg/r
!    13     14    15     16     17     18     19     20      21
! Wigner
!    33
!-----------------------------------------------------------------------
! sbasis(f) : sbasis type for dof f. (dof=degree of freedom)
!(phifbr), el, HO,  Leg, eigenf, sphfbr, kleg,  k  map Wigner,readspf
!   -1      0   1    2     3      4       5     6   7    8      9   
! euclid
!   10
!***********************************************************************

      subroutine genphi(spf1d,trafo,ort,jsph,msph,hops,exprueck)

      implicit none

      integer(long)       :: s,m,p1,p2,f,f1,ff,kmin,kmin2,kmax,dk,slz,&
                             ilbl,isym,t,i,e,k,k0,k1,ipot,ggdim,dmf,z1d,&
                             nst1,ishift(3),nn,j,tmpdim,zeigk,mmmin,mmmin2, &
                             mmmax,dm,kdim,nr,ierr,pop,i0,ii,dmfm,point

      integer(long), dimension(sphdim), intent(in)  :: jsph,msph
      real(dop), dimension(dvrdim), intent(in)      :: trafo
      real(dop), dimension(ortdim), intent(inout)   :: ort
      real(dop), dimension(hopsdim), intent(in)     :: hops
      real(dop)                                     :: thrs,nrm
      complex(dop),dimension(spf1ddim), intent(out) :: spf1d
      complex(dop),dimension(:), allocatable        :: tmpspf1d
      complex(dop),dimension(expdim), intent(in)    :: exprueck

      character(len=c5)       :: filename

!-----------------------------------------------------------------------
! for a Cartesian vMCG wavepacket initial coordinates have already been
! read  from iin
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
! Generate a set of orthonormal single-particle functions for each
! state and mode
!
! loop over DOFs using index ff allows info from a different DOF
! to be used in SPF generation (e.g. continuum electronic basis set
! where bound state and continuum state info in different "DOFs")
!-----------------------------------------------------------------------
      nst1 = nstate
      do s=1,nstate
         if(dentype .ne. 1) nst1 = s  
         do t=s,nst1
         do ff=1,ndof
            m=dofspf(ff)
            if(m.eq.0) then
               do f1=1,ndof
                  if(sbasis(f1) .eq. 7) then
                     m=dofspf(f1) ! Generate dummy DOFs if used in map.
                     if(isbaspar(1,f1,1).eq.f .or. &
                        isbaspar(2,f1,1).eq.f .or. &
                        isbaspar(3,f1,1).eq.f) goto 30
                  endif
               enddo
               goto 100 ! this is for dummy DOFs (ghost modes).
            endif

   30       continue
            if ((ldd .and. ltraj .and. ddcoo .eq. 2) &
                 .and. m .ne. meb) cycle ! Cartesian trajectories for DD
                                                          ! generated in genswarm

            f=ff
            if(dentype .eq. 1) then
               dmf = ddimf(f,s,t)
               dmfm = dmf
               z1d = dzspf1d(f,s,t)
            else
               dmf = dimf(f,s)
               dmfm = max(dimf(f,s),dimfran(f))
               z1d = zspf1d(f,s)
            endif
! ---       Set 1D symmetry parameter isym
            isym=0
inner:      do j=1,2*maxdim     ! Search for sym1d, asym1d
               if(msymmtr(j).eq.0) exit inner
               if(f.eq.mod(abs(msymmtr(j)),256)) then
                  isym=msymmtr(j)/abs(msymmtr(j))
                  exit inner
               endif 
            enddo inner
            
            routine = 'GENPHI'

! skip if no spfs required (sbasis=999 -> not set in build,
!                           sbasis= -1 -> phifbr)
            if (dmf.eq.0 .or. sbasis(f).eq.-1 .or. &
                 sbasis(f) .eq. 30 .or. sbasis(f).eq.999) then
               continue  

!   ---- electronic state ----
            else if (basis(f).eq.0 .or. basis(f) .eq. 10) then
               if (sbasis(f) .eq. 0) then
                  call gendisc(spf1d(z1d),dmf,gdim(f))
               else if (sbasis(f).eq.9) then
                  routine = "genphi (readspf)"
                  nr = isbaspar(1,f,s)
                  call genreadspf(spf1d(z1d),gdim(f),dmf, &
                  readspffile(nr),basis(f),ort(zort(f)),trafo(zdvr(f)), &
                  ipbaspar(1,f),ipbaspar(1,f+1),rpbaspar(1,f), &
                  isbaspar(2,f,s),f)
               else 
                  call zerovxz(spf1d(z1d),gdim(f))
                  spf1d(z1d)=1.0_dop
               endif

!   ---- Not sphfbr/phifbr or K primitive basis  ----
!   ---- or K primitive basis (when part of Kleg or Wigner) ----
            else if ((basis(f).ge.1 .and. basis(f).le.5) .or. &
                     basis(f).eq.7  .or. basis(f).eq.9 .or.  &
                     basis(f).eq.11 .or. basis(f).eq.12 .or.  &
                     basis(f).eq.20 .or. basis(f).eq.21 .or. &
                     basis(f) == 31 .or. basis(f) == 32 .or. &
                     basis(f) == 34 .or. &
                     (basis(f).ge.14 .and. basis(f).le.18) .or. &
                     (basis(f).eq.8 .and. basis(max(1,f-1)) .ne.7 &
                           .and. basis(max(1,f-1)).ne.9) .or. &
                     basis(f).eq.33 .or. &
                    (basis(f).eq.8 .and. basis(max(1,f-1)).ne. 7 .and. &
                        basis(max(1,f-1)).ne.33 .and. basis(max(1,f-2)).ne.33) &
                         ) then

!   ---- exp primitive basis part of Pleg or Wigner ----
               if (basis(f).eq.5 .and. (basis(max(1,f-1)).eq.9 .or. &
                        basis(max(1,f-1)).eq.33 .or. &
                        basis(max(1,f-2)).eq.33) ) then
                  if (basis(max(1,f-2)).eq.33) then
                     f1=max(f-2,1) ! exp is 3rd DOF in Wigner DVR
                  else
                     f1=max(f-1,1) ! exp is 2nd DOF in pleg/Wigner DVR
                  endif
                  if (sbasis(f1).eq.5 .or. sbasis(f1).eq.8) then
                     if (sbasis(f).eq.6) then
                        if (sbasis(f1).eq.5 .and. (f-2).eq.f1) then
                           write(6,'(a,3i4)') 'f1, DVR(f1), inwf(f1):', &
                           f1,basis(f1),sbasis(f1)
                           write(6,'(a,3i4)') ' f, DVR( f), inwf( f):', &
                           f,basis(f),sbasis(f)
                           message = 'If DVR is Wigner+_+exp '// &
                           'and inwf is Kleg+K+inwf(f), '// &
                           'then K-inwf is not allowed for f.'
                           call errormsg
                        else
                           goto 100
                        endif
                     else if (sbasis(f1).eq.5 .and. (f-2).eq.f1) then
                        continue  ! Kleg+k+X-inwf + Wigner+exp+exp-DVR
                     else
                        write(6,'(a,3i4)') 'f1, DVR(f1), inwf(f1):', &
                        f1,basis(f1),sbasis(f1)
                        write(6,'(a,3i4)') ' f, DVR( f), inwf( f):', &
                        f,basis(f),sbasis(f)
                        message = 'If DVR is [Pleg or Wigner]+exp '// &
                        'and inwf is [Kleg or Wigner]+inwf(f), then '// &
                        'inwf(f) must be K.'
                        call errormsg
                     endif
                  else if (sbasis(f).eq.6) then
                     write(6,'(a,3i4)') 'f1, DVR(f1), inwf(f1):', &
                     f1,basis(f1),sbasis(f1)
                     write(6,'(a,3i4)') ' f, DVR( f), inwf( f):', &
                     f,basis(f),sbasis(f)
                     message = 'If DVR is [Pleg or Wigner]+exp '// &
                     'and inwf is [not (Kleg or Wigner)]+inwf(f), '// &
                     'then K-inwf is not allowed for f.'
                     call errormsg
                  endif
               endif
!   ---- Gauss and HO and Lagu ----
!   --- discrete basis
               if (sbasis(f) .eq. 0) then
                  call gendisc(spf1d(z1d),dmf,gdim(f))

               else if (sbasis(f).eq.1) then 
                  routine = 'GENGAUSS (genphi1.f)'
                  p1=0
                  ggdim=gdim(f)
                  if (basis(f).eq.12 .or. basis(f).eq.20 .or. & 
                         basis(f).eq.21) ggdim=ipbaspar(3,f)
!...Determine the appropriate 'column' of trafo(gdim,gdim,kdim) for KLeg/PLeg
                  if (basis(f).eq.7 .or. basis(f).eq.20) then  ! (to get the weights)

                     kmin=ipbaspar(1,f+1)
                     kmax=ipbaspar(2,f+1) 
                     kmin2=ipbaspar(3,f+1)
                     p1=(min(kmax,kmin2)-kmin)
                     dk=ipbaspar(5,f+1)
                     p1=(p1+dk-1)/dk
                     p1=p1*ggdim**2
                  else if (basis(f).eq.9 .or. basis(f).eq.21) then

                     kmin=ipbaspar(1,f+1)
                     kmax=ipbaspar(2,f+1)
                     kmin2=ipbaspar(4,f+1)
                     p1=(min(kmax,kmin2)-kmin)*ggdim**2
!...or the 'column' of trafo(gdim,gdim,kdim,mmdim) for Wigner (to get the weights)
                  else if (basis(f).eq.33) then
                     kmin=ipbaspar(2,f)
                     kmax=ipbaspar(3,f)
                     kmin2=ipbaspar(5,f)
                     p2=min(kmax,kmin2)-kmin
                     if (basis(f+1).eq.8) then
                        dk=ipbaspar(5,f+1)
                        p2=(p2+dk-1)/dk
                     endif
                     p2=p2*gdim(f)**2
                     mmmin=ipbaspar(6,f)
                     mmmax=ipbaspar(7,f)
                     mmmin2=ipbaspar(9,f)
                     p1=min(mmmax,mmmin2)-mmmin
                     if (basis(f+2).eq.8) then
                        dm=ipbaspar(5,f+2)
                        p1=(p1+dm-1)/dm
                     endif
                     p1=p1*ipbaspar(4,f)*gdim(f)**2 + p2
                  endif
                  nn=nspfdof(m) ! number of dof in mode
                  call gengauss(spf1d(z1d),gdim(f),dmfm, &
                        trafo(zdvr(f)+p1),ort(zort(f)),basis(f), &
                        rpbaspar(1,f),ipbaspar(1,f),ipbaspar(1,f+1), &
                        rsbaspar(1,f,s),isbaspar(1,f,s),ggdim,f,nn,isym)

!   ---- associated Legendre polynomials P_l^m for fixed m=slz ----
               elseif (sbasis(f).eq.2) then
                  routine = 'GENALEG (genphi1.F)'
                  nn=nspfdof(m) ! number of dof in mode
                  p1=0
                  ggdim = gdim(f)
                  if(basis(f).eq.20 .or. basis(f).eq.21) &
                                    ggdim=ipbaspar(3,f)
                  if (basis(f).eq.7 .or. basis(f).eq.20) then
                     slz = isbaspar(1,f,1)
                     kmin= ipbaspar(1,f+1)
                     dk  = ipbaspar(5,f+1)
                     dk  = max(1,dk)
                     p1  = ((slz-kmin)/dk)*ggdim**2
                  endif
                  if (basis(f).eq.9 .or. basis(f).eq.21) then
                     slz =isbaspar(1,f,1)
                     kmin=ipbaspar(1,f+1)
                     p1=(slz-kmin)*ggdim**2
                  endif
! --- Wigner functions d^j_m0 (or d^j_0k) as Legendre polynomials---
                  if (basis(f).eq.33) then
                     slz = isbaspar(1,f,1)
                     kmin=ipbaspar(2,f)
                     kmax=ipbaspar(3,f)
                     kdim=ipbaspar(4,f)
                     kmin2=ipbaspar(5,f)
                     if (basis(f+1).eq.8) then
                        dk=ipbaspar(5,f+1)
                     else
                        dk=1
                     endif
                     mmmin=ipbaspar(6,f)
                     mmmax=ipbaspar(7,f)
                     mmmin2=ipbaspar(9,f)
                     if (basis(f+2).eq.8) then
                        dm=ipbaspar(5,f+2)
                     else
                        dm=1
                     endif
! --- If kmin2 is zero and slz is in the range [mmmin,mmmax], or
! --- if mmmin2 is zero and slz is in the range [kmin,kmax], then the
! --- Legendre initial-wf can be taken directly from the Wigner trafo;
! --- otherwise, slz must equal zero and the Legendre code is used
                     if (slz.ge.mmmin .and. slz.le.mmmax .and. &
                         kmin2.eq.0) then ! pick trafo for k=0, mm=slz
                        p2=(min(kmax,kmin2)-kmin+dk-1)/dk
                        p1=(slz-mmmin)*kdim*gdim(f)**2 + p2*gdim(f)**2
                     elseif (slz.ge.kmin .and. slz.le.kmax .and. &
                         mmmin2.eq.0) then ! use trafo for mm=0, k=slz
                        p2=(min(mmmax,mmmin2)-mmmin+dm-1)/dm
                        p1=(slz-kmin)*gdim(f)**2 + p2*kdim*gdim(f)**2
                     endif
                  endif
                  ggdim = gdim(f)
                  if(basis(f).eq.12) ggdim=ipbaspar(3,f)
                  call genaleg(spf1d(z1d),gdim(f),dmfm,nn, &
                          trafo(zdvr(f)+p1),ort(zort(f)),basis(f), &
                          ggdim,rpbaspar(1,f),ipbaspar(1,f), &
                          ipbaspar(1,f+1),isbaspar(1,f,s),message)

!   ---- eigenf ----
               elseif (sbasis(f).eq.3) then
                  minedim=max(minedim,dmfm)
                  if (dmfm.gt.maxedim) then
                     routine = 'VEIGEN (genphi1.F)'
                     do k = 1, nstate
                        write(6,*) 's =',k,',  dimf : ', &
                             (dimf(i,k),i=1,ndof) 
                     enddo
                     write(message,'(a,i5)') &
                       'Increase maxedim to at least',dmfm
                     call errormsg
                  endif
                  ipot=isbaspar(1,f,s)
                  m=dofspf(f)

                  if (lveigen) then
                     ilbl=index(modelabel(f),' ')-1
                     filename=name(1:laenge)//'/veigen_'// &
                              modelabel(f)(1:ilbl)
                     open(iveig,file=filename,form='formatted', &
                              status='unknown')
                  endif

                  call veigen (spf1d(z1d),gdim(f),dmfm,ort(zort(f)), &
                        isbaspar(4,f,s),isbaspar(6,f,s),hops, &
                        s,f,m,ipot,modelabel(f),eeigen,e0eigen,lveigen)
                  if (lveigen) then
                     close(iveig)
                  endif

!   ---- ass. Leg. polynomials P_l^m for the given set of (l,m) pairs. ----
               elseif (sbasis(f).eq.5) then
                  if (basis(f).eq.7 .or. basis(f).eq.20) then
                     routine = 'GENKLEG (genphi1.F)'
                     dmf=min(dimf(f,s),dim(m,s)) ! NEWKLEG, see zeiginwf0
                     ggdim = gdim(f)
                     if (basis(f).eq.20) ggdim=ipbaspar(3,f)
                     call genkleg(spf1d(z1d),gdim(f),gdim(f+1), &
                             dmf,trafo(zdvr(f)),basis(f), &
                             ggdim,ipbaspar(1,f),ipbaspar(1,f+1), &
                             isbaspar(1,f,s),isbaspar(1,f+1,s),f,m, &
                             modelabel(f),ilog,message)
                  else if (basis(f).eq.9 .or. basis(f).eq.21) then
                     routine = 'GENKLEG PLeg (genphi1.F)'
                     ggdim = gdim(f)
                     if (basis(f).eq.21) ggdim=ipbaspar(3,f)
                     tmpdim = gdim(f)*ipbaspar(3,f+1)*dim(m,s)
                     if (allocated(tmpspf1d)) deallocate(tmpspf1d)
                     allocate(tmpspf1d(tmpdim))
                     call genkleg(tmpspf1d,gdim(f),ipbaspar(3,f+1),&
                             dim(m,s),trafo(zdvr(f)),basis(f),&
                             ggdim,ipbaspar(1,f),ipbaspar(1,f+1),&
                             isbaspar(1,f,s),isbaspar(1,f+1,s),f,m,&
                             modelabel(f),ilog,message)
                     call plegrueck(tmpspf1d,exprueck(zexp(f+1)),spf1d(z1d)&
                              ,gdim(f),gdim(f+1),1,ipbaspar(3,f+1), &
                               dim(m,s))
                     if (allocated(tmpspf1d)) deallocate(tmpspf1d)
                  else if (basis(f).eq.33) then
                     routine = 'GENKLEG Wigner (genphi1.F)'
! --- For Kleg+K-inwf in combination with Wigner-DVR, select the
! --- correct 'column' of trafo(gdim,gdim,kdim,mmdim)
                     kdim=ipbaspar(4,f)
                     mmmin=ipbaspar(6,f)
                     mmmax=ipbaspar(7,f)
                     mmmin2=ipbaspar(9,f)
                     ggdim = gdim(f)
                     if (basis(f+2).eq.8) then
                        dm=ipbaspar(5,f+2)
                     else
                        dm=1
                     endif
                     p1=(min(mmmax,mmmin2)-mmmin+dm-1)/dm
                     p1=p1*kdim*gdim(f)**2
                     if (basis(f+1).eq.5) then  ! Wigner+exp p-basis
! --- Wigner requires space for two arrays: gdim(f)*kdim*mmdim*dim(m,s)
!     plus gdim(f)*gdim(f+1)*mmdim*dim(m,s) for the case where both
!     k and m need to be transformed K-->Exp
!
                        tmpdim = &
           gdim(f)*(gdim(f+1)+ipbaspar(4,f))*ipbaspar(8,f)*dimf(f,s)
                        if (allocated(tmpspf1d)) deallocate(tmpspf1d)
                        allocate(tmpspf1d(tmpdim))
                        call genkleg(tmpspf1d,gdim(f),ipbaspar(4,f), &
                                dim(m,s),trafo(zdvr(f)+p1),basis(f), &
                                ggdim,ipbaspar(1,f),ipbaspar(1,f+1), &
                                isbaspar(1,f,s),isbaspar(1,f+1,s), &
                                f,m,modelabel(f),ilog,message) 
                        call plegrueck(tmpspf1d,exprueck(zexp(f+1)), &
                                 spf1d(z1d),gdim(f),gdim(f+1), &
                                 1,ipbaspar(4,f),dim(m,s))
                        if (allocated(tmpspf1d)) deallocate(tmpspf1d)
                     else if (basis(f+1).eq.8) then  ! Wigner+K p-basis
                        dmf=min(dimf(f,s),dim(m,s))
                        call genkleg(spf1d(z1d),gdim(f),gdim(f+1), &
                                dmf,trafo(zdvr(f)+p1),basis(f), &
                                ggdim,ipbaspar(1,f),ipbaspar(1,f+1), &
                                isbaspar(1,f,s),isbaspar(1,f+1,s), &
                                f,m,modelabel(f),ilog,message)
                     endif

                  else 
                     write(message,'(a,i4)') &
                     '1 Wrong spf basis identification number, f =',f
                     call errormsg
                  endif 

!   ----  Wigner d^j_(m,k) for given set of (j,m,k)
               elseif (sbasis(f).eq.8) then
! --- Wigner requires space for two arrays: gdim(f)*kdim*mmdim*dim(m,s)
!     plus gdim(f)*gdim(f+1)*mmdim*dim(m,s) for the case where both
!     k and m need to be transformed K-->Exp
                  tmpdim = gdim(f)*(gdim(f+1)+ipbaspar(4,f))* &
                                      ipbaspar(8,f)*dimf(f,s)
                  if (allocated(tmpspf1d)) deallocate(tmpspf1d)
                  allocate(tmpspf1d(tmpdim))
                  if (basis(f).eq.33) then
                     dmf=min(dimf(f,s),dim(m,s))
                     routine = 'GENWIGNER (genphi1.F)'
                     if (basis(f+1).eq.5 .and. basis(f+2).eq.5) then
                        zeigk = 1+gdim(f)*ipbaspar(4,f)*ipbaspar(8,f)
                        call genwigner(tmpspf1d,gdim(f),ipbaspar(4,f), &
                             ipbaspar(8,f),dmf,trafo(zdvr(f)), &
                             basis(f),ipbaspar(1,f),isbaspar(1,f,s), &
                             isbaspar(1,f+1,s),isbaspar(1,f+2,s), &
                             f,modelabel(f),ilog,message)
                        call plegrueck(tmpspf1d,exprueck(zexp(f+1)), &
                             tmpspf1d(zeigk),gdim(f),gdim(f+1), &
                             ipbaspar(8,f),ipbaspar(4,f),dmf)
                        call plegrueck(tmpspf1d(zeigk),exprueck(zexp(f+2)), &
                             spf1d(z1d),gdim(f)*gdim(f+1),gdim(f+2), &
                             1,ipbaspar(8,f),dmf)
                     elseif (basis(f+1).eq.5 .and. basis(f+2).eq.8) then
                        call genwigner(tmpspf1d,gdim(f),ipbaspar(4,f), &
                             ipbaspar(8,f),dmf,trafo(zdvr(f)), &
                             basis(f),ipbaspar(1,f),isbaspar(1,f,s), &
                             isbaspar(1,f+1,s),isbaspar(1,f+2,s), &
                             f,modelabel(f),ilog,message)
                        call plegrueck(tmpspf1d,exprueck(zexp(f+1)), &
                             spf1d(z1d),gdim(f),gdim(f+1), &
                             gdim(f+2),ipbaspar(4,f),dmf)
                     elseif (basis(f+1).eq.8 .and. basis(f+2).eq.5) then
                        call genwigner(tmpspf1d,gdim(f),ipbaspar(4,f), &
                             ipbaspar(8,f),dmf,trafo(zdvr(f)), &
                             basis(f),ipbaspar(1,f),isbaspar(1,f,s), &
                             isbaspar(1,f+1,s),isbaspar(1,f+2,s), &
                             f,modelabel(f),ilog,message)
                        call plegrueck(tmpspf1d,exprueck(zexp(f+2)), &
                             spf1d(z1d),gdim(f)*gdim(f+1),gdim(f+2), &
                             1,ipbaspar(8,f),dmf)
                     elseif (basis(f+1).eq.8 .and. basis(f+2).eq.8) then
                        call genwigner(spf1d(z1d),gdim(f),ipbaspar(4,f), &
                             ipbaspar(8,f),dmf,trafo(zdvr(f)), &
                             basis(f),ipbaspar(1,f),isbaspar(1,f,s), &
                             isbaspar(1,f+1,s),isbaspar(1,f+2,s), &
                             f,modelabel(f),ilog,message)
                     else
                        write(message,'(a)') &
                        'Only exp/K can complement Wigner' 
                        call errormsg
                     endif
                  else
                     write(message,'(a,i4)') &
                     '2 Wrong spf basis identification number, f =',f
                     call errormsg
                  endif
                  if (allocated(tmpspf1d)) deallocate(tmpspf1d)

!   ---- exp-DVR and K (PLeg), (do nothing) ----
               elseif (basis(f).eq.5 .and. sbasis(f).eq.6) then
                  continue

!   ---- MAP, taking SPFs from anoter DOF. ----
               elseif (sbasis(f).eq.7) then 
                  routine = 'genphi.F (map)'
                  thrs=abs(ort(zort(f)+1)-ort(zort(f)))*1.d-4
                  do ii=1,3
                     if(isbaspar(ii,f,s).eq.0) goto 102
                     f1 = isbaspar(ii,f,s)
                     if( sbasis(f1) .eq. 999 ) then
                        write(message,'(a,i3,a,i3,2a)') 'Cannot map:',&
                        f1,' ->',f,'. Initial orbitals not defined : ',&
                        modelabel(f1)
                        call errormsg
                     endif
                     if(abs(ort(zort(f1))-ort(zort(f))) .lt. thrs) then
                        ishift(ii) = 0
                        goto 101
                     elseif(ort(zort(f1))-ort(zort(f)) .gt. thrs) then
                        do i = 1, abs(gdim(f)-gdim(f1))
                           if(abs(ort(zort(f)+i)-ort(zort(f1)))&
                                .lt. thrs) then
                              ishift(ii) = i
                              goto 101
                           endif
                        enddo
                     elseif(ort(zort(f))-ort(zort(f1)) .gt. thrs) then
                        do i = 1, abs(gdim(f)-gdim(f1))
                           if(abs(ort(zort(f))-ort(zort(f1)+i))&
                                .lt. thrs) then
                              ishift(ii) = -i
                              goto 101
                           endif
                        enddo
                     endif
                     write(6,*) 'No ishift found! ii,f,f1',ii,f,f1
                     routine = "genphi (MAP, line 425)"
                     message = 'Cannot find a shift for map'
                     call errormsg

 101                 do i0 = 1, min(gdim(f),gdim(f)-ishift(ii),&
                                    gdim(f1),gdim(f1)+ishift(ii))
                        i=i0+max(0,-ishift(ii))
                        if(abs(ort(zort(f)+i+ishift(ii)-1) - &
                               ort(zort(f1)+i-1)).gt. thrs) then
                           write(6,'(a,2i3,i5,i3,a,2i5,/a,2f17.9)') &
                             ' f, f1, i, ishift :',f,f1,i,ishift(ii),&
                             ' ,   gdim :', gdim(f),gdim(f1),' ort :',&
                           ort(zort(f)+i+ishift(ii)-1),ort(zort(f1)+i-1)
                           write(message,'(a,i3,a,i3,a)')&
                                'When mapping DOFs,',f1,' ->',f,&
                                ' they must have common grid-points.'
                           call errormsg
                        endif
                     enddo
                  enddo ! ii=1,3
 102              continue

!   ---- euclid ----
               elseif (sbasis(f).eq.10) then
                  call geneuclid(spf1d(z1d),gdim(f),dmfm,isbaspar(1,f,s))
 
!   ---- k (if not already in KLeg/Wigner) ----
               elseif (sbasis(f).eq.6) then
                  point=(isbaspar(1,f,s)-isbaspar(2,f,s))
                  point=point/isbaspar(5,f,s) + 1
                  call geneuclid(spf1d(z1d),gdim(f),dmfm,point)

! -- Time-dependent point
               else if (sbasis(f) .eq. 11) then
                  call gentdpoint(f,s,spf1d(z1d),gdim(f),isbaspar(1,f,s),&
                       ort(zort(f)))
                  ltdp = .true.

!   ---- READSPF ----
               elseif (sbasis(f).eq.12) then
                  routine = "genphi (readspf)"
                  nr = isbaspar(1,f,s)
                  call genreadspf(spf1d(z1d),gdim(f),dmf,&
                  readspffile(nr),basis(f),ort(zort(f)),trafo(zdvr(f)),&
                  ipbaspar(1,f),ipbaspar(1,f+1),rpbaspar(1,f),&
                  isbaspar(2,f,s),f)

               else
                  routine = 'genphimod.f90 (general)'
                  write(message,'(a,i4)')  &
                  '3 Wrong spf basis identification number, f =',f
                  call errormsg
               endif

!   ---- sph-FBR ----
            elseif (basis(f).eq.6) then
               routine = 'genphi.F (sph-FBR)'
               if (sbasis(f).eq.4) then
               call gensphfbr(spf1d(z1d),gdim(f),dmf, &
                        jsph(zsph(f)),msph(zsph(f)),isbaspar(1,f,s), &
                        ilog,routine,message)

               else
                  write(message,'(a,i4)')  &
                  '4 Wrong spf basis identification number, f =',f
                  call errormsg
               endif

!   ---- K (when combined with Kleg or Wigner) ----
            elseif (basis(f).eq.8) then
                 if (basis(max(1,f-2)).eq.33) then
                    f1=max(f-2,1) ! second K in Wigner+K.or.exp+K DVR
                 else
                    f1=max(f-1,1)
                 endif
                 if (sbasis(f1).ne.1 .and. sbasis(f1).ne.2 .and. &
                     sbasis(f1).ne.5 .and. sbasis(f1).ne.30) then
                    write(6,'(a,i4,a9)') ' f  =',f,trim(modelabel(f))
                    write(6,'(a,i4,a9)') ' f1 =',f1,trim(modelabel(f1))
                    message =  &
                    'Wrong spf basis identification number for K:'// &
                    ' K can only follow HO, Leg, Kleg, or Wigner'

                    call errormsg
                 endif 
                 if (sbasis(f) .ne. 6) then
                    message = 'The corresponding inwf for k-dvr '// &
                              'in a Kleg or Wigner set should be k-inwf.'
                    call errormsg
                 endif

!   --- GWPs ----
            elseif(basis(f).eq.25)then
               if(sbasis(f).eq.9)then
                  call gengwp(spf1d(zspf1d(f,s)),dimf(f,s),&
                       rsbaspar(1,f,s),isbaspar(1,f,s),f)
               endif 

            elseif(basis(f).eq.26)then
               if(sbasis(f).eq.1)then
                  call xlocgauss(spf1d(zspf1d(f,s)),3,dimf(f,s),rsbaspar(1,f,s),&
                       ort(zort(f)),rpbaspar(1,f),ipbaspar(1,f))
               endif 

            elseif(basis(f) .eq. 27)then
               if(sbasis(f).eq.1)then
                  call clgauss(spf1d(zspf1d(f,s)),dimf(f,s), &
                       rsbaspar(1,f,s),pqdistr)
               else
                  message =&
                     'Not allowed SPF type for a classical DOF'
                  call errormsg
               endif 

            else
               write(6,*) ' sbasis(f) =', sbasis(f), &
                         ',  basis(f) =', basis(f),', f =', f
               message =  &
                     'Wrong primitive basis identification number'
               call errormsg               
            endif

!   ---- Thermal ----
!   DMT 06/2018
            if (dimfran(f).ne.0) then
               call genran(spf1d(z1d),gdim(f),dimfran(f),mlseed)
               call schmidtortho(spf1d(z1d),gdim(f),dmf,ierr)
            endif

 100     continue
         enddo                  ! do f=1,ndof
         enddo                  ! do t=1,nst1
      enddo                     ! do s=1,nstate

!  ---- Electronic continuum states.
! for bound states set up SPFS as a delta-function on first grid point 
! (zero energy state). If a discrete basis was set up, now make sure all
! states in continuum are included.
       if (febcont .gt. 0) then
          if (lmult) then
             do s=1,elbnd
                z1d = zspf1d(febcont,s)
                call zerovxz(spf1d(z1d),gdim(febcont))
                spf1d(z1d)=1.0_dop
             enddo
             do s=elbnd+1,eltot
                z1d = zspf1d(febcont,s)
                call elbndspfms(spf1d(z1d),dimf(febcont,s),&
                     gdim(febcont),sbasis(febcont))
             enddo
          else 
             call elbndspf(spf1d(zspf1d(febcont,1)),dimf(febcont,1),&
                  gdim(febcont),sbasis(febcont),elbnd)
          endif
      endif


! Beginning of modification (Benjamin, 14 Feb 2007)
      if (gwpzahl .ne. 0) then
         call rdgwp(spf1d)
      endif
! End of modification (Benjamin, 14 Feb 2007)

!.....Mapping of initial SPFs
      do s=1,nstate
         do f=1,ndof
            if (sbasis(f).eq.7) then
               call zerovxz(spf1d(zspf1d(f,s)),dimf(f,s)*gdim(f))
               do ii=1,3
                  if(isbaspar(ii,f,s) .gt. 0) then
                     f1 = isbaspar(ii,f,s)
                      write(ilog,'(a,i3,a,i3,a,i3,a,i4)')&
                     'Mapping s =', s,' ; ',f1, ' ->',f,&
                     ' , ishift =',ishift(ii)
                     pop=isbaspar(4,f1,s)
                     if (pop.ne.1 .and. dimf(f1,s).gt.0 .and.&
                                        sbasis(f1).ne.3) then
                        call swapspf(spf1d(zspf1d(f1,s)),&
                                     pop,gdim(f1),dimf(f1,s))
                     endif
                     do e = 1, dimf(f,s)
                        k0 = zspf1d(f,s) +(e-1)*gdim(f)+ishift(ii)-1
                        k1 = zspf1d(f1,s)+(e-1)*gdim(f1)-1
                        do i0 = 1, min(gdim(f),gdim(f)-ishift(ii),&
                                      gdim(f1),gdim(f1)+ishift(ii))
                           i=i0+max(0,-ishift(ii))
                           spf1d(k0+i) = spf1d(k0+i) + spf1d(k1+i)*&
                           dcmplx(rsbaspar(ii,f,s),rsbaspar(ii+3,f,s))
                        enddo ! i0, i
                     enddo ! e
                  endif ! isbaspar>0
               enddo ! ii=1,3
               call schmidtortho2(spf1d(zspf1d(f,s)),gdim(f),dimf(f,s),&
                                  ierr,nrm)
               if(ierr.ne.0) then
                  write(6,   '(/a,i4,e10.2)') 'ierr, norm:',ierr,nrm
                  write(ilog,'(/a,i4,e10.2)') 'ierr, norm:',ierr,nrm
                  routine='genphi'
                  message='Cannot orthonormalize the 1D-SPFs of a '//&
                          ' mapped DOF: '//modelabel(f)
                  call errormsg
               endif
            endif ! (sbasis(f).eq.7)
         enddo ! f
      enddo ! s

      return
      end subroutine genphi

!***********************************************************************
!
!              MCTDH    Subroutine    VEIGEN
!
!    Calculates eigenvectors and eigenvalues of a 1D Hamiltonian
!
!***********************************************************************

      subroutine veigen(psi,gdim1,dim1,ort,pop,isym,hops,s1,f1,m1,ipot,lab, &
                        eeigen,e0eigen,lwrite)

      implicit none

      integer(long)             :: e,g,np,nm,nn,n,ier,sym1,sym2,&
                                   nsym,nsym1,nsym2,k,h,ll,ilbl
      integer(long), intent(in) :: f1,m1,s1,gdim1,dim1,ipot,isym
      integer(long)             :: f,m,h1,nh

      integer(long), intent(inout)                 :: pop
      integer(long), dimension(gdim1)               :: worki

      real(dop)                                    :: sum,sum1
      real(dop), intent(out)                       :: e0eigen
      real(dop), dimension(gdim1), intent(in)       :: ort
      real(dop), dimension(gdim1)                   :: eval
      real(dop), dimension(dim1), intent(out)       :: eeigen 
      real(dop), dimension(hopsdim), intent(in)     :: hops
      real(dop), dimension(gdim1,gdim1)              :: work

      complex(dop), dimension(gdim1,gdim1)             :: cevec,workc
      complex(dop), dimension(gdim1,dim1), intent(out) :: psi
      character(len=16), intent(in)                  :: lab
      logical(kind=4), intent(in)                    :: lwrite
      logical(kind=4)                                :: loned

!-----------------------------------------------------------------------
!  Initialisation and checks. 
!-----------------------------------------------------------------------
      ilbl=index(hamlab(ipot),' ')-1
      ll = index(lab,' ')-1

      if (.not. ldvr(f1)) then
         write(message,'(a,i5)') 'Must use a DVR basis for DOF : ',f1
         call errormsg
      endif

!-----------------------------------------------------------------------
! find 1D operators on DOF f1
!-----------------------------------------------------------------------
      loned=.true.
      do k=zham(ipot),zham(ipot)+khtot(ipot)-1
         do m=1,nmode
            if (m .eq. m1) then
               f=spfdof(1,m)
               h=hamilton(f,k)
               if (htmform(h) .eq. 0) then
!  look for a 1D operator
                  do n=1,nspfdof(m)
                     f=spfdof(n,m)
                     h=hamilton(f,k)

                     if (f .ne. f1 .and. htmtype(h) .ne. 1) loned =.false.
                     if (f .eq. f1) then

                        if ((htmtype(h) .ne. 2) .and. &
                            (htmtype(h) .ne. 8) .and. &
                            (htmtype(h) .ne. 9) .and. &
                            (htmtype(h) .ne. 10)) loned = .false.
                     endif
                  enddo

               else
! product mode operators
                  do nh=1,htmfac(h)
                     h1=hterm(nh,h)
                     if (h1 .gt. 0) then
                        f=htmdof(h1)
                        if (f .eq. f1) then
!    operators in DOF must be hermitian
                           if ((htmtype(h1) .ne. 2) .and. &
                               (htmtype(h1) .ne. 8) .and. &
                               (htmtype(h1) .ne. 9) .and. &
                               (htmtype(h1) .ne. 10)) loned =.false.
                        else
!    other DOF in mode must be unit
                           if (h1 .ne. f) loned = .false.
                        endif
                     endif
                  enddo
               endif

            else
!    other Modes must be unit
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  h=hamilton(f,k)

                  if (htmtype(h) .ne. 1) loned =.false.
               enddo
            endif
         enddo
      enddo
      if (.not. loned) then
         write(message,'(3a,i5)') 'Hamiltonian ',hamlab(ipot)(1:ilbl), &
              ' is not a 1D operator for DOF: ',f1
         call errormsg
      endif

      write(ilog,'(/,a,1x,4a,i3)')  &
           '*** VEIGEN ***  operator: ', &
           hamlab(ipot)(1:ilbl),' , DOF = ',lab(1:8), &
           ' state =', s1

!-----------------------------------------------------------------------
! set up operator in complex matrix cevec
!-----------------------------------------------------------------------
      cevec=0.0_dop
      g=gdim1*gdim1
      do k=zham(ipot),zham(ipot)+khtot(ipot)-1
         f=spfdof(1,m1)
         h=hamilton(f,k)
         if (htmform(h) .eq. 0) then
            h=hamilton(f1,k)
         endif

         if (htmtype(h).eq. 2) then
            call addhvec(cevec,koeff(k),hops(zhop(h)),vgdim(f1), &
                     gdim1,ngdim(f1))

         else if(htmtype(h) .eq. 8) then
            call addhmat(cevec,koeff(k),hops(zhop(h)),gdim1)

         else if (htmtype(h) .eq. 9) then
            call addhmati(cevec,koeff(k),hops(zhop(h)),gdim1)

         else if (htmtype(h) .eq. 10 ) then
            call addhmatz(cevec,koeff(k),hops(zhop(h)),hops(zhop(h)+g),gdim1)

         endif

      enddo

!-----------------------------------------------------------------------
! Diagonalise operator
! get all eigenvalues "eval" and the corresponding normalized
! eigenvectors "evec"
! of the real symmetric matrix "evec"
! or the complex hemritian "cevec"
!-----------------------------------------------------------------------
      call zheev('V','U',gdim1,cevec,gdim1,eval,workc,g,work,ier)

      if (ier.ne.0) then
         write (ilog,*) 'Error in subroutine VEIGEN'
         write (ilog,*) 'Error diagonalising the Hamiltonian matrix'
         write (ilog,*) 'Error code : ',ier
         call stopnow
      endif

!-----------------------------------------------------------------------
! If isym.gt.0 => symmetry, if isym.gt.9 => check
! Symmetry means: take every second eigenfunction. 
!-----------------------------------------------------------------------
      if (isym.gt.0) then
         sym1=2
         sym2=-mod(pop,2)
         pop = (pop+1)/2
      else
         sym1=1
         sym2=0
      endif

!-----------------------------------------------------------------------
! Zero the integer work array.
! Save eigenvalue corresponding to eigenstate 'pop'.
!-----------------------------------------------------------------------
      call zerovxi(worki,gdim1)
      e0eigen=eval(pop)
 
!-----------------------------------------------------------------------
! Transfer real evec data to complex psi array. The order of the SPF's is:
! pop, pop-1, pop-2, pop+1, pop-3, pop-4, pop+2, ...
!-----------------------------------------------------------------------      
      nsym=0
      nsym1=0
      nm = pop 
      np = pop - 1
      do e=1,min(dim1,gdim1)
         if( pop .le. 1 )  then
            nn = e
         else
            if(((mod(e,3).eq.1).or.(nm .eq. 1)) .and. np.lt.gdim1) then 
               np = np + 1
               nn = np
            else
               nm = nm - 1
               nn = nm
            end if
         end if
         nsym2=nsym1    ! Save already occupied indices
         nsym1=nsym     ! to avoid double assignments.
         nsym=sym1*nn + sym2
         if(nsym.eq.nsym1  .or. nsym.eq.nsym2) nsym=nsym+1
         if(nsym.eq.nsym1  .or. nsym.eq.nsym2) nsym=nsym+1
         nsym=mod(nsym+gdim1-1,gdim1) + 1
!........Check symmetry
         if(isym.gt.9) then
 300        sum  = 0.0_dop
            sum1 = 0.0_dop
            do g = 1, gdim1
               sum = sum + dble(dconjg(cevec(g,nsym))*cevec(gdim1+1-g,nsym))
            enddo
            sum = sum*(1.0_dop+2.0_dop*sym2)
            if(sum.gt.0.0_dop) then
               nsym=max(1,nsym-1)                 ! one down
               if(nsym.eq.nsym1  .or. nsym.eq.nsym2) nsym=nsym+2
               if(nsym.eq.nsym1  .or. nsym.eq.nsym2) nsym=nsym+1
               sum1 = 0.0_dop
               do g = 1, gdim1
                  sum1 = sum1 + &
                       dble(dconjg(cevec(g,nsym))*cevec(gdim1+1-g,nsym))
               enddo
               sum1 = sum1*(1.0_dop+2.0_dop*sym2)
               if(sum1.gt.0.0_dop) then 
                  nsym=min(gdim1,nsym+2)              !  one up   
                  if(nsym.eq.nsym1  .or. nsym.eq.nsym2) nsym=nsym+1
                  goto 300
               endif
            endif 
         endif
         if(worki(nsym).ne.0)  then
            write(6,'(/a/a,i4,a,i4,a,i4)') ' ERROR in veigen: ',&
             'Overwriting the',worki(nsym),'-th SPF with the',&
             e,'-th one. Eigenfunction no.:',nsym 
            write(ilog,'(/a/a,i4,a,i4,a,i4)') ' ERROR in veigen: ', &
             'Overwriting the',worki(nsym),'-th SPF with the', &
             e,'-th one. Eigenfunction no.:',nsym 
            call stopnow
         endif
         worki(nsym)=e 
         eeigen(e)=eval(nsym)
         do g=1,gdim1
            psi(g,e)=cevec(g,nsym)
         enddo
      enddo

!-----------------------------------------------------------------------
! Change the phases of the eigenvectors.
! The phase convention is: Sum_j [j*psi(j)] > 0.
!-----------------------------------------------------------------------
      do e=1,dim1
         sum=0.0_dop
         do g=1,gdim1
            sum = sum + dble(g*psi(g,e))
         enddo  
         if(sum.lt.0.0_dop) then
            do g=1,gdim1
               psi(g,e) = -psi(g,e)
            enddo
         endif
      enddo
 
!-----------------------------------------------------------------------
! Write eigenvalues in [eV] to log file
!-----------------------------------------------------------------------
      write(ilog,*)
      do e=1,min(max(sym1*dim1+1,np,(gdim1+4)/3),gdim1)
         if(worki(e) .gt. 0 ) then
            write(ilog,'(i4,a,f16.9,a,f18.12,2a,i4,a)') &
            e,'. Eigenvalue',eval(e)*ev,'[eV]',eval(e),'[au]',&
            ' ,',worki(e),'. spf'  
         else
            write(ilog,'(i4,a,f16.9,a,f18.12,a)')  &
                 e,'. Eigenvalue',eval(e)*ev,'[eV]',eval(e),'[au]'
         endif
      enddo
      write (ilog,*)
      call myflush(ilog)

!-----------------------------------------------------------------------
! if requested, write eigenvalues and vectors to veigen file
!-----------------------------------------------------------------------
      if (lwrite) then
         write(iveig,'(a,i3,2a,/a,i3,2a,//a)') &
               '# 1D-Hamiltonian:',ipot,', ',hamlab(ipot)(1:ilbl), &
               '# DoF:',f1,', ',lab(1:ll), &
               '# **** Eigenvalues in [eV]  and  [au] ****'
         do e=1,gdim1
            write(iveig,'(i5,2f23.13)') e,eval(e)*ev,eval(e)
         enddo
 
         do e=1,gdim1
            write(iveig,'(/a,i4,a,e15.7,a)') &
               '#',e,'-th eigenvector.  E =',eval(e),' au'
            do g=1,gdim1
               write(iveig,'(f22.12,3x,2f19.12)') ort(g),cevec(g,e)
            enddo
         enddo
      endif

      return
      end subroutine veigen

!#######################################################################

      subroutine addhvec(cevec,koeff,hops,vgdim1,gdim1,ngdim1)

      implicit none

      integer(long), intent(in) :: vgdim1,gdim1,ngdim1
      real(dop), intent(in), dimension(vgdim1,gdim1,ngdim1) :: hops
      complex(dop), intent(inout), dimension(gdim1,gdim1) :: cevec
      complex(dop), intent(in)                         :: koeff
      integer(long) :: g

      do g=1,gdim1
         cevec(g,g) = cevec(g,g) + koeff*hops(1,g,1)
      enddo

      return
      end subroutine addhvec

      subroutine addhmat(cevec,koeff,hops,gdim1)

      implicit none

      integer(long), intent(in) :: gdim1
      real(dop), intent(in), dimension(gdim1,gdim1) :: hops
      complex(dop), intent(inout), dimension(gdim1,gdim1) :: cevec
      complex(dop), intent(in)                         :: koeff
      integer(long) :: g,g1

      do g=1,gdim1
         do g1=1,gdim1
            cevec(g1,g) = cevec(g1,g) + koeff*hops(g1,g)
         enddo
      enddo

      return
      end subroutine addhmat

      subroutine addhmati(cevec,koeff,hops,gdim1)

      implicit none

      integer(long), intent(in) :: gdim1
      real(dop), intent(in), dimension(gdim1,gdim1) :: hops
      complex(dop), intent(inout), dimension(gdim1,gdim1) :: cevec
      complex(dop), intent(in)                         :: koeff
      integer(long) :: g,g1

      do g=1,gdim1
         do g1=1,gdim1
            cevec(g1,g) = cevec(g1,g) + koeff*(0,1.0)*hops(g1,g)
         enddo
      enddo

      return
      end subroutine addhmati

      subroutine addhmatz(cevec,koeff,hopsd,hopsi,gdim1)

      implicit none

      integer(long), intent(in) :: gdim1
      real(dop), intent(in), dimension(gdim1,gdim1) :: hopsd
      real(dop), intent(in), dimension(gdim1,gdim1) :: hopsi
      complex(dop), intent(inout), dimension(gdim1,gdim1) :: cevec
      complex(dop), intent(in)                         :: koeff
      integer(long) :: g,g1

      do g=1,gdim1
         do g1=1,gdim1
            cevec(g1,g) = cevec(g1,g) + koeff*dcmplx(hopsd(g1,g),hopsi(g1,g))
         enddo
      enddo

      return
      end subroutine addhmatz

!#######################################################################

      subroutine elbndspfms(spf1d,dimf1,gdim1,sbasis1)

      implicit none

      integer(long), intent(in)                          :: gdim1,dimf1,&
                                                            sbasis1
      integer(long)                                      :: e,e1
      real(dop)                                          :: norm
      complex(dop), dimension(gdim1,dimf1),intent(inout) :: spf1d

      if (sbasis1 .ne. 0) return

!-----------------------------------------------------------------------
! For sbasis = 0
! Include in first SPF all states not yet added.
!-----------------------------------------------------------------------
      e1 = 1
      do e=dimf1+1,gdim1     
         spf1d(e,e1) = 1.0_dop
      enddo
! normalise
      norm = dble(gdim1-dimf1+1)
      norm = 1.0_dop/dsqrt(norm)
      do e=1,gdim1     
         spf1d(e,e1) = norm*spf1d(e,e1)
      enddo
 
      return
      end subroutine elbndspfms

!#######################################################################

      subroutine elbndspf(spf1d,dimf1,gdim1,sbasis1,nbnd)

      implicit none

      integer(long), intent(in)                          :: gdim1,dimf1,&
                                                            sbasis1,nbnd
      integer(long)                                      :: e,e1,ierr
      real(dop)                                          :: norm
      complex(dop), dimension(gdim1,dimf1),intent(inout) :: spf1d

!-----------------------------------------------------------------------
! For sbasis = 0
! A discrete set of delta functions on first dimf1 points have been
! created. Now include remaining states in first SPF.
!-----------------------------------------------------------------------
      if (sbasis1 .eq. 0) then
         e1 = nbnd+1
         do e=dimf1+1,gdim1     
            spf1d(e,e1) = 1.0_dop
         enddo
! normalise
         norm = dble(gdim1-dimf1+1)
         norm = 1.0_dop/dsqrt(norm)
         do e=1,gdim1     
            spf1d(e,e1) = norm*spf1d(e,e1)
         enddo
 
      else
!-----------------------------------------------------------------------
! For sbasis > 0
! move all SPFs created up by 1, and make 1st SPF a delta function on
! the first grid point (E=0)
!-----------------------------------------------------------------------
         do e=dimf1,2,-1
            spf1d(:,e) = spf1d(:,e-1)
         enddo
         spf1d(:,1) = 0.0_dop
         spf1d(1,1) = 1.0_dop

         call schmidtortho(spf1d,gdim1,dimf1,ierr)
      endif

      return
      end subroutine elbndspf

      end module genphimod

