!***********************************************************************
!
!               MCTDH   subroutine  GENPSI
!
! contains : genpsi,exactwf,genmultspf,meigen,funkme,metransform,
!            genmultkleg,gensymmultkleg,sym2kleg_excite1,sym2kleg_excite2,
!            symorbitals,genswarm,genswarm1
! 
!  11/96 AJ            
!
!***********************************************************************

      module genpsimod

use decimal, only: long, dop
use global
use keyunitsmod
use globmemmod
use maxv, only: maxdim
use logdat
use xvlib,   only: xvxxzzo
use schmidtorthomod, only: schmidtortho, schmidtortho3
use op1lib,  only: cpvxz, cpmxz, zerovxz, zerovxi, initvxz, unitqxz
use gwplib, only: renormgwpa

use dvrdatmod
use griddatmod
use psidef
use operdef
use runinwfmod
use lctmod
use multkleg

use rdopermod, only: iskleg,get_num_klegs
use genphi1
use gencoeffmod, only: gencoeff
use getpsimod, only: getpsi3d
use gwplib, only: gwptophi,renormdgwp1,diagonal_moments,pqtoxivec
use rdopermod, only: iskleg,get_num_klegs, get_nth_kleg

use dirdyn, only: ddtrfile,ddxcoo0,ddtransf,ddtransb,ndofddpes,&
                  iatsym,ntr
use ioqc, only: getintyp,getxcoo,getnm,nm2xmat
use xvlib, only: xvixddo,xvxxddo
use randommod, only: randgaussian

use hphimod, only: hunphi1ms
use sillib, only: sildiag,silerrormsg
use channels
use dirdyn

      implicit none
      private
      public :: genpsi,exactwf,meigen,symorbitals,genswarm,funkme
      contains 

!***********************************************************************
!
! modus = 0 : run full generation
!       = 1 : generate coefficients (+ SPFs from GWPs) only
!
!***********************************************************************

      subroutine genpsi(psi,spf1d,ort,jindx,agmat,modus)

      implicit none

      integer(long)       :: s,m,f,f1,pop,n,next,nk

      integer(long), intent(in)                        :: modus
      integer(long), dimension(jindxdim), intent(out)  :: jindx
      real(dop), dimension(ortdim), intent(in)         :: ort
      complex(dop), dimension(dgldim), intent(inout)   :: psi
      complex(dop), dimension(spf1ddim), intent(inout) :: spf1d
      complex(dop), dimension(dgldim)                  :: workc
      complex(dop), dimension(agmatdim), intent(out)   :: agmat
      complex(dop), dimension(maxdof*maxspf*maxspf)        :: z
      complex(dop), dimension(maxdof*maxdof*maxspf*maxspf) :: ai
      complex(dop), dimension(gs2matdim)                   :: gs2

      if (modus .ne. 1) then ! if equal to 1 skip to coefficient generation

!-----------------------------------------------------------------------
! move initially populated spf to be 1st in list
!-----------------------------------------------------------------------
      do s=1,nstate
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
! for multi-set calcs, populated state set by the coefficients in gencoeff
               if (f .eq. feb .and. lmult) cycle

               ! if single-set and 50/50 mix of 2 states do not re-order 
               ! electronic basis
               if (f .eq. feb .and. dentype .eq. 2) then
                  if (leb .and. dlstate .ne. drstate) cycle
               endif

               pop=isbaspar(4,f,s)
               if (pop.gt. dimf(f,s)) then
                  routine = 'Genpsi (genpsimod.f90)'
                  message = 'Requested population greater than N SPFs'
                  call errormsg

               else if (pop.gt.1 .and. dimf(f,s).gt.0 .and. sbasis(f).ne.3)  &
                    then
                  call swapspf(spf1d(zspf1d(f,s)),pop,gdim(f), &
                       dimf(f,s))
               else if (pop .lt. 1) then
                  routine = 'Genpsi (genpsimod.f90)'
                  write(message,'(a,3i5)') &
                       'Illegal value of isbaspar(4)',pop,f,s
                  call errormsg
               endif
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! transfer spfs to psi, multiplying out multi-D spfs
!-----------------------------------------------------------------------
      do s=1,nstate
         do m=1,nmode
            f=spfdof(1,m)
!    ---    Count the number of K DOFs in the mode
            nk=0
            do n=1,nspfdof(m)
               f1=spfdof(n,m)
               if (sbasis(f1).eq.6) nk=nk+1
            enddo
! read SPFs from restart and copy to psi(zetf(m,s))
            if (sbasis(f) .eq. 30) then
             call getpsi3d(psi,ort,workc,jindx,agmat,m)

! Straight copy for 
!    ---    Mode with only 1 DOF (unless GWP), or with only 2D sphfbr+phifbr, or
!    ---    only 2D kleg+K, or only 3D Wigner+K+K
            else if ((nspfdof(m).eq.1 .and. sbasis(f) .ne. 9) .or. &
               (sbasis(f).eq.4 .and. nspfdof(m).eq.2) .or. &
               (sbasis(f).eq.5 .and. nspfdof(m).eq.2) .or. &
               (sbasis(f).eq.8 .and. nspfdof(m).eq.3)) then
               call cpmxz(spf1d(zspf1d(f,s)),psi(zetf(m,s)),subdim(m), &
                    dim(m,s))

            else if (nk.gt.0 ) then
!    ---    Modes with products involving klegs, Wigners, or Ks
               call initvxz(psi(zetf(m,s)),subdim(m)*dim(m,s))
               if (lsymmultkleg(m)) then
                  call gensymmultkleg(spf1d,psi,m,s)
               else
                  call genmultkleg(spf1d,psi,m,s)
               endif

! GWPs
            elseif (sbasis(f).eq.9) then 
               if (lgwptrj) then
                  call gentgwp(agmat(zagmat(m,s)),agmat(zxivec(m,s)),&
                       agmat(zphase(m,s)),psi(zxi(m,s)),psi(zphi(m,s)),&
                       gwpdim(m,s),nspfdof(m),rsbaspar(1,f,1))
               else if (isbaspar(1,f,s) .eq. 4) then
                  call genrdgwp(psi(zphi(m,s)),psi(zxi(m,s)),psi(zpg(m,s)), &
                       psi(zpg(m,s)),agmat(zagmat(m,s)), &
                       agmat(zphase(m,s)),gwpdim(m,s),nspfdof(m),m,s)

                  call pqtoxivec(nspfdof(m),gwpdim(m,s),agmat(zagmat(m,s)), &
                       psi(zxi(m,s)),agmat(zxivec(m,s)))

               else if &
                    (isbaspar(1,f,s) .ge. 5 .and. isbaspar(1,f,s) .le. 7) then
                  call genrgwp(psi(zphi(m,s)),psi(zxi(m,s)),psi(zpg(m,s)), &
                       psi(zpg(m,s)),agmat(zagmat(m,s)), &
                       agmat(zphase(m,s)),isbaspar(1,f,1), &
                       gwpdim(m,s),nspfdof(m),m,s)

                  call pqtoxivec(nspfdof(m),gwpdim(m,s),agmat(zagmat(m,s)), &
                       psi(zxi(m,s)),agmat(zxivec(m,s)))

               else
                  call genkgwp(psi(zphi(m,s)),psi(zxi(m,s)),psi(zpg(m,s)), &
                       psi(zpg(m,s)),agmat(zagmat(m,s)), &
                       agmat(zphase(m,s)),spf1d,dimf(f,s),&
                       gwpdim(m,s),nspfdof(m),m,s)

                  call pqtoxivec(nspfdof(m),gwpdim(m,s),agmat(zagmat(m,s)), &
                       psi(zxi(m,s)),agmat(zxivec(m,s)))
               endif

            else
!    ---    Modes with products of 1D spfs
               call initvxz(psi(zetf(m,s)),subdim(m)*dim(m,s))
               call genmultspf(spf1d,psi,m,s)
            endif

            if (iskleg(f,basis)) then  
               if( (basis(f).eq.7 .or. basis(f).eq.20) .and.  &
                   sbasis(f).eq.1 .and. sbasis(f+1).eq.6 ) then
!.....Re-determine the higher SPFs for KLeg (gauss,K).
                  next=2
                  call kgenhigher(psi(zetf(m,s)),ort,subdim(m),dim(m,s),next,m)
               endif
            endif

         enddo
      enddo

      endif
!-----------------------------------------------------------------------
! starting point for coefficient generation
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! generate coefficients
! if SPFs read from file read coefficients and reorder
!-----------------------------------------------------------------------
      call gencoeff(psi,jindx,agmat)

!-----------------------------------------------------------------------
! Set up SPF representation in GWP basis
! First get GWP overlap matrices (gs2), 
! then trafos for GWP -> SPF (dgwp and adgwp - dgwp stored in psi)
!-----------------------------------------------------------------------
      if (lgwp .and. (psitype .eq. 0 .or. psitype .eq. 5)) then
          do s=1,nstate
             do m=1,nmode
                if (.not. gwpm(m)) cycle
                call pqtoxivec(nspfdof(m),gwpdim(m,s),agmat(zagmat(m,s)), &
                     psi(zxi(m,s)),agmat(zxivec(m,s)))
                call diagonal_moments(m,s,s,nspfdof(m),gwpdim(m,s), &
                     gwpdim(m,s),agmat(zagmat(m,s)),agmat(zagmat(m,s)), &
                     agmat(zxivec(m,s)),agmat(zxivec(m,s)),agmat(zphase(m,s)),&
                     agmat(zphase(m,s)),gs2(gs2mat(m,s)),ai,z,0,0)
             enddo
         enddo

         call gwptophi(gs2,psi)
    !     do s=1,nstate
    !        do m=1,nmode
    !           if (.not. gwpm(m)) cycle
    !           call unitqxz(unitmat,dim(m,s))
    !           call renormdgwp1(psi(zdgwp(m,s)),gs2(gs2mat(m,s)),unitmat,&
    !                gwpdim(m,s),dim(m,s))
    !        enddo
    !     enddo

      else if (lgwp) then
         do s=1,nstate
            do m=1,nmode
               if (.not. gwpm(m)) cycle
               call pqtoxivec(nspfdof(m),gwpdim(m,s),agmat(zagmat(m,s)), &
                    psi(zxi(m,s)),agmat(zxivec(m,s)))
               call diagonal_moments(m,s,s,nspfdof(m),gwpdim(m,s), &
                    gwpdim(m,s),agmat(zagmat(m,s)),agmat(zagmat(m,s)), &
                    agmat(zxivec(m,s)),agmat(zxivec(m,s)),agmat(zphase(m,s)),&
                    agmat(zphase(m,s)),gs2(gs2mat(m,s)),ai,z,0,0)
            enddo
         enddo

         call renormgwpa(psi,gs2)

         if (lBEL) then
           do s=1,nstate
             do m=1,nmode
               write(1000,*)'m',m,'gwpm(m)',gwpm(m),'dim',dim(m,s),"source/geninwf/genpsimod.f90"
               if (.not. gwpm(m)) then
                 dimred(m,s)=dim(m,s) 
               else
                 dimred(m,s)=1
               endif
             enddo
           enddo
         endif
      endif

!----------------------------------------------------------------------
! copying psi to memory for LCT vibrational projector
!----------------------------------------------------------------------
      if (lctvibpop) then
         do n=1,dgldim
            lctpsi(n)=dcmplx(0.0_dop)
         enddo
         do s=1,nstate
           do m=1,nmode
             call  cpmxz(psi(zetf(m,s)),lctpsi(zetf(m,s)), &
             subdim(m),dim(m,s))
             do n=1,nspfdof(m)
                f=spfdof(n,m)
                pop=isbaspar(4,f,s)
             enddo 
             if (pop .gt. 1) then
                if (nviblct .eq. 1) then
                   nviblct=pop
                endif
             endif
           enddo
         enddo
      endif
      return
      end subroutine genpsi

!#######################################################################

      subroutine exactwf(psi)

      implicit none

      integer(long)                                  :: s,t
      complex(dop), dimension(dgldim), intent(inout) :: psi

!-----------------------------------------------------------------------
! include the coefficient into the spf.
!-----------------------------------------------------------------------
      if (dentype .eq. 1) then
         do s=1,nstate
            do t=1,nstate
               call xvxxzzo(psi(dzpsi(t,s)),psi(dzetf(1,t,s)),subdim(1))
               psi(dzpsi(t,s)) = 1.0_dop
            enddo
         enddo
      else
         do s=1,nstate
            call xvxxzzo(psi(zpsi(s)),psi(zetf(1,s)),subdim(1))
            psi(zpsi(s)) = 1.0_dop
         enddo
      endif

      return
      end subroutine exactwf

!######################################################################
! This routine builds the combined-modes SPFs from the 1D SPFs.
! To inspect the excitation scheme, set "print" in build-block 
! of the Init_WF-Section.
!----------------------------------------------------------------------

      subroutine genmultspf(spf1d,psi,m,s)

      implicit none

      integer(long)                                 :: j,n,f,f1,nspf,&
                                                       ex,isym,gdm,ngt10
      integer(long), dimension(maxdim)              :: indx
      integer(long), intent(in)                     :: m,s
      complex(dop), dimension(spf1ddim), intent(in) :: spf1d
      complex(dop), dimension(dgldim), intent(inout):: psi

!.....Check, if mode-sym is set.
      isym=0
      do j = 1, 2*maxdim
         if( msymmtr(j) .eq. 0 ) goto 20
         n=mod(abs(msymmtr(j)),65536)
         f=mod(n,256)
         n=n/256
         if(n.eq.m) then 
            if( (f.eq.0) .or. (f.eq.200) ) isym=1
            if( msymmtr(j) .lt. 0 ) isym=-isym
            if( isym .ne. 0 ) goto 20
         endif
      enddo
      isym=0
  20  if(isym.ne.0) then
!.....Do not use symm, if the initial 1D-spf differ.
         f1=spfdof(1,m)
         if(sbasis(f1).eq.7) f1=isbaspar(1,f1,s)
         do n=2,nspfdof(m)
            f=spfdof(n,m)
            if(sbasis(f).eq.7) f=isbaspar(1,f,s)
            do j=1,sbaspar
               if(isbaspar(j,f,s).ne.isbaspar(j,f1,s)) isym=0
               if(abs(rsbaspar(j,f,s)-rsbaspar(j,f1,s)) &
                    .gt. tiny) isym=0
            enddo 
         enddo 
      endif
      if(lprintmode) write(2,'(/a,i2,a,i3,a,i2,/a)')  &
          'State =',s, ', Mode =', m,', sym =',isym, &
          ' spf  1D-functions'  

!     Initial configuration
      ex = 0
      nspf = 1
      do n=1,nspfdof(m)
         indx(n) = 1
         if(isym.eq.-1) indx(n) = nspfdof(m)+1-n 
         f=spfdof(n,m)
         call multspf(spf1d(zspf1d(f,s)),psi(zetf(m,s)),indx(n), &
              vgdim(f),gdim(f),ngdim(f),nspf,dim(m,s))
      enddo
      if(lprintmode)write(2,'(i4,2x,99i3)')nspf,(indx(n),n=1,nspfdof(m))

!     Higher configurations
      do nspf=2,dim(m,s)
         ngt10=0 ! counter to avoid an infinite loop
   10    continue
         if(isym.ne.0) then  
   11       call excite1(nspfdof(m),ex,indx)
            if(isym.eq.-1 .and. indx(1).eq.indx(2)) goto 11
         else
            call excite2(nspfdof(m),ex,indx)
         endif
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            gdm=gdim(f)   ! Below is for odd/even of HO or gauss
            if(sbasis(f).eq.1 .and. isbaspar(1,f,s).gt.0) gdm=gdm/2
            if(nspfdof(m).gt.2) gdm=2*gdm/3+1 !To avoid high single exc.
            if(indx(n) .gt. gdm) then
               ngt10=ngt10+1
               if(ngt10 .gt. subdim(m)+dim(m,s)) then ! avoid an inf loop
                  write(6,'(/6(a,i4))') ' s=',s,', f=',f,', m=',m,&
                        ', ex=',ex,', gdm=',gdm,', dim=',dim(m,s)
                  write(2,'(/6(a,i4))') ' s=',s,', f=',f,', m=',m,&
                        ', ex=',ex,', gdm=',gdm,', dim=',dim(m,s)
                  message='Cannot create combined initial SPF'
                  routine='genpsi.F, genmultspf, line 216'
                  call errormsg
               endif
               goto 10
            endif
         enddo
         if(lprintmode) write(2,'(i4,2x,99i3)')  &
                        nspf,(indx(n),n=1,nspfdof(m))
         
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            call multspf(spf1d(zspf1d(f,s)),psi(zetf(m,s)),indx(n), &
                 vgdim(f),gdim(f),ngdim(f),nspf,dim(m,s))
         enddo
      enddo

      return
      end subroutine genmultspf

!######################################################################
! This routine diagonalizes a mode-operator using Lanczos (SIL).
! The eigenfunctions are taken as initial (combined) modes.
! Operator must be hermitian.
! HDM 09/03
!----------------------------------------------------------------------

      subroutine meigen(psi,workr,sdm,dim1)

      implicit none

      integer(long)                                    :: hktype,error,&
                                                          steps,order,pt
      integer(long), intent(in)                        :: dim1,sdm
      real(dop)                                        :: time,accuracy
      real(dop), dimension(*), intent(out)             :: workr

      complex(dop), dimension(sdm,2)                   :: hpsi
      complex(dop), dimension(sdm,dim1), intent(inout) :: psi

      hktype = 1
      call hunphi1ms (psi,hpsi,memode,mestate,1,sdm,hktype,meham)
     
      call cpvxz(psi(1,1),hpsi(1,2),sdm)  ! The second column of hpsi will 
                                          ! be used in metransform

      accuracy = 1.0e-10_dop
      steps    = memode 
      pt       = medim+1
      order    = 2*dim1+3
      order    = min(2*dim1+3,sdm/4+1)
      write(istp,'(/a,8x,a,10x,a,13x,a,17x,a )') '   mode', &
                            'Order','Stepsize','Error','Time'
      call sildiag(psi,hpsi,sdm,medim,accuracy,.false.,steps, &
                   mc(mcintpsi),order,error,menum,time,.true., &
                   funkme,workr,mr(mrintr),mr(mrintr+pt), &
                   mr(mrintr+2*pt),mr(mrintr+3*pt))

      call metransform(sdm,dim1,steps,order,mc(mcintpsi),workr,psi,hpsi)

      if (error .ne. 0) then
         routine = 'sildiag'
         call silerrormsg(error,message)
         call errormsg
      endif

      return
      end subroutine meigen

!-----------------------------------------------------------------------
! The routine is called from SIL.
!----------------------------------------------------------------------

      subroutine funkme(time,psi,hpsi)

      implicit none

      integer(long)                             :: dim1,sdm,hktype
      real(dop), intent(out)                    :: time
      complex(dop), dimension(*), intent(inout) :: psi,hpsi

      hktype = 1
      time = 0.0_dop
      dim1 = 1
      sdm  = subdim(memode)
      call hunphi1ms (psi,hpsi,memode,mestate,dim1,sdm,hktype,meham)
     
      return
      end subroutine funkme

!-----------------------------------------------------------------------
! The routine builds the eigenfunctions from the Krylov space.
!----------------------------------------------------------------------

      subroutine metransform(sdm,dim1,q,order,Krylov,EigenVector, &
                             psi,hpsi)
 
      implicit none

      integer(long)             :: q1,e,g,p,ilbl,iz,izz
      integer(long), intent(in) :: dim1,sdm,q,order

      complex(dop), dimension(sdm,dim1), intent(inout)    :: psi
      complex(dop), dimension(sdm,2), intent(in)          :: hpsi
      complex(dop), dimension(sdm,2:medim), intent(in)    :: Krylov
      real(dop)                                           :: sum
      real(dop), dimension(0:medim,0:medim+2), intent(in) :: EigenVector

      character(len=16)       :: filename
      character(len=8)        :: slab(4) = (/'noselect','select  ',&
                                             '        ','full    '/)
      e  = 1
      q1 = 0
      iz = 0
      izz= 0
      ilbl=index(hamlab(meham),' ')-1
      write(ilog,'(/a,i3,a,i2,a,i4,2x,a)')' Mode-eigenfunctions:  m =', &
               memode,' ,  state =',mestate, &
          ' ,  operator =',meham,hamlab(meham)(1:ilbl)
      write(ilog,'(a,i4,a,i6,a,i1,2a)') ' menum =', &
              mod(menum,10000),',  maxdim =',medim,',  write=', &
              mewrite,slab(3+menum/9999),slab((3+meselect)/2)
      write(ilog,'(2a,i5)')'  spf    Q    eigenvalue[eV]     ', &
                        'eigenvalue[au] , order =', order+1
      write(ilog,'(2i5,f18.12,f22.15)') e,Q,EigenVector(Q,medim+2)*eV, &
                                        EigenVector(Q,medim+2)

! --- Change the phases of the eigenvectors.
! --- The phase convention is: Sum_j [j*psi(j)] > 0.
      sum=0.0_dop
      do g=1,sdm
         sum = sum + g*dble(psi(g,e))
      enddo
      if(sum.lt.0.0_dop) then
         do g=1,sdm
            psi(g,e) = -psi(g,e)
         enddo
      endif
! ---    Writing of first eigenvectors to file
      if(mewrite.eq.1) then
         write(filename,'(a,i3,a,i2)') '/meigen_',memode,'_',mestate
         call rmblnk(filename)
         open(unit   = iveig, &
              file   = name(1:laenge)//filename, &
              form   = 'formatted', &
              status = 'unknown')
         write(iveig,'(/2a)') '#  SPF,  Q, subdim,  eigenvalue[au]', &
                              ' / eigenvector (real part)'
         write(iveig,'(3i5,f20.12)') e,Q,sdm,EigenVector(Q,medim+2)
         write(iveig,'(4f20.12)') (dble(Psi(g,e)),g=1,sdm)
      endif 

      do e = 2, dim1
 111     if( Q1 .eq. Q ) Q1 = Q1 + 1
! ----   Remove eigenvectors with vanishing intensity
! ----   meselect is 1 (selection) or -1 (no selection)
         if( (EigenVector(0,Q1)**2 .lt. 1.0e-9_dop .and. Q1.le.36) ) then
             write(irlx,'(a,i5,f15.9,e12.4)') ' Small            :', &
                   Q1,EigenVector(Q1,medim+2)*eV,EigenVector(0,Q1)**2
         endif
         if( (EigenVector(0,Q1)**2 .lt. meselect*1.0e-13_dop) ) then
             izz = izz + 1
             write(irlx,'(a,i5,f15.9,e12.4)') ' Removed by select:', &
                   Q1,EigenVector(Q1,medim+2)*eV,EigenVector(0,Q1)**2
             if( Q1 .lt. order-dim1+e-2 ) then
                Q1 = Q1 + 1
                iz = iz + 1
                goto 111
             endif
         endif
         write(ilog,'(2i5,f18.12,f22.15)') e, Q1, &
                    EigenVector(Q1,medim+2)*eV, EigenVector(Q1,medim+2)

!  ----  Build eigenvectors from Lanczos EigenVector and Krylov-vectors.
!  ----  HPsi(.,1) <= dtpsi,  HPsi(.,2) <= psi
         Do g = 1,sdm
            Psi(g,e) = EigenVector(0,Q1)*HPsi(g,2) +  &
                       EigenVector(1,Q1)*HPsi(g,1)
         EndDo
         Do P = 2,order
            Do g = 1,sdm
               Psi(g,e) = Psi(g,e) + EigenVector(P,Q1)*Krylov(g,P)
            EndDo
         EndDo
! ---    Phase correction     
         sum=0.0_dop
         do g=1,sdm
            sum = sum + g*dble(psi(g,e))
         enddo
         if(sum.lt.0.0_dop) then
            do g=1,sdm
               psi(g,e) = -psi(g,e)
            enddo
         endif
! ---    Writing of eigenvectors to file
         if(mewrite.eq.1) then
            write(iveig,'(/2a)')'#  SPF,  Q, subdim,  eigenvalue[au]', &
                                ' / eigenvector (real part)'
            write(iveig,'(3i5,f20.12)') e,Q1,sdm,EigenVector(Q1,medim+2)
            write(iveig,'(4f20.12)') (dble(Psi(g,e)),g=1,sdm)
         endif 
         Q1 = Q1 + 1
      enddo    !   do e = 2, dim1

! ---    Writing to log-file
      if(iz.gt.0) then
         write(ilog,'(/a,i4,2a,i3,a/)')  '##',iz,  &
          ' states with zero overlap have been removed by select.', &
          ' (meigenf, m =', memode,')'
      endif
      if(mewrite.eq.1) close(iveig) 
      if(izz.gt.iz) then
         write(ilog,'(a,i3,a,/a,i4,a,/a,i4,a/)') &
           '####  WARNING (meigenf, m =',memode,')  ####',  &
           ' There are',izz,' states with vanishing overlap,', &
           ' but only ',iz,' states could be removed by select.'
         write(6,'(/a,i3,a,/a,i4,a,/a,i4,a/)') &
           '####  WARNING (meigenf, m =',memode,')  ####', &
           ' There are',izz,' states with vanishing overlap,', &
           ' but only ',iz,' states could be removed by select.'
      endif

      return
      end subroutine metransform

!######################################################################
! Builds combined-mode SPFs if the KLeg or Wigner or K mode is present
! in the combined-mode.
!
! Note: Here the excitations for the not KLeg-dofs up to emax are
!       considered. This restriction can be modified depending the
!       problem.
!
! Modified to allow for multiple KLegs in one mode. (FO 04/2006)
! Modified to print out configurations if desired. (FO 07/2006)
! Modified to allow Wigners and ___+K sets in mode (PT 02/2013)
!----------------------------------------------------------------------

      subroutine genmultkleg(spf1d,psi,m,s)

      implicit none

      integer(long)                                 :: n,f,nspf,ex,i,emax,&
                                                       nk,ncount
      integer(long), intent(in)                     :: m,s
      integer(long), dimension(maxdim)              :: indx,indx1
      complex(dop), dimension(spf1ddim), intent(in) :: spf1d
      complex(dop), dimension(dgldim), intent(inout):: psi
      logical(kind=4)                               :: lprint

      emax=2 ! will be adjusted automatically
      
! --- Check for "print" statement --------------------------------------
      lprint = .false.
      do n=1,nspfdof(m)
         f=spfdof(n,m)
         if ((sbasis(f).eq.5 .or. sbasis(f).eq.8) .and. &
             isbaspar(5,f,s).ne.0) lprint=.true.
      enddo
      if (lprint) then
!        write header
         write (ilog,'(/a,i3,/,a$)') &
            '*** The configurations for combined mode ',m, &
            '    #spf    '
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            if (sbasis(f).ne.6) write (ilog,'(a$)') modelabel(f)(1:8)
         enddo
         write (ilog,*)
      endif

! --- Initial configuration --------------------------------------------
  123 ex = 0
      nspf = 1
      do n=1,nspfdof(m)
         indx1(n) = 1
         f=spfdof(n,m)
         if (sbasis(f) .eq. 5) then
!           multiply with the 2D (KLeg+K) function
            call multspf(spf1d(zspf1d(f,s)),psi(zetf(m,s)),indx1(n), &
                         vgdim(f),gdim(f)*gdim(f+1),ngdim(f+1), &
                         nspf,dim(m,s))
         elseif (sbasis(f).eq.8) then
!           multiply with the 3D (Wigner+K+K) function
            call multspf(spf1d(zspf1d(f,s)),psi(zetf(m,s)),indx1(n), &
                         vgdim(f),gdim(f)*gdim(f+1)*gdim(f+2), &
                         ngdim(f+2),nspf,dim(m,s))
         elseif (sbasis(f-1).eq.5 .or. sbasis(f-1).eq.8 .or. &
                 sbasis(f-2).eq.8) then
!           if previous DOF was KLeg, this one must be K -> skip
            continue
         elseif (sbasis(f).eq.6) then
!           K s-basis anywhere else: clear all but k = slz
            call kclearspf(psi(zetf(m,s)),isbaspar(1,f,s),vgdim(f), &
                           gdim(f),ngdim(f),nspf,dim(m,s))
         else
!           otherwise simply multiply with the 1D SPF
            call multspf(spf1d(zspf1d(f,s)),psi(zetf(m,s)),indx1(n),&
                         vgdim(f),gdim(f),ngdim(f),nspf,dim(m,s))
         endif
      enddo
      if (lprint) then
         write (ilog,'(i7,a$)') nspf,')'
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            if (sbasis(f).ne.6) write (ilog,'(i8,$)') indx1(n)
         enddo   
         write (ilog,*)
      endif

! --- Higher configurations --------------------------------------------
      ex=0 ! was ex=1, but why? F.O.
!     determine number of K DOFS to skip
      nk=0
      do n=1,nspfdof(m)
         f=spfdof(n,m)
         if (sbasis(f).eq.6) nk=nk+1
      enddo

      do nspf=2,dim(m,s)
         ncount=0
   10    continue
         ncount=ncount+1
         if(ncount .ge. nspfdof(m)) then
            emax=emax+1
            call initvxz(psi(zetf(m,s)),subdim(m)*dim(m,s))
            if(emax .le. dim(m,s)) goto 123
            write(message,'(a,i4,2a)') &
            'Cannot create higher excited states for mode m =',m, &
            '                    Decrease numer of SPFs (dim),', &
            ' or increase numbers of grid points (gdim).'
            routine='genmultkleg (genpsi.F Line 648)'
            call errormsg
         endif

!        indx1 is an array that holds the excitation of the DOFs
!        of which the mode is made up. Successive calls to excite2
!        yield the higher configurations from which to create the
!        mode-SPFs.
!        With KLegs, we need to omit all the K DOFs because, for
!        this purpose, the 2D (KLeg+K) DOFs are treated like 1D
!        modes. Thus we only consider excitations of the KLeg DOFs
!        and then skip over the K DOFs.

!        UPDATE: need also to skip K dofs which are part of Wigner
!        or combined with other DOFs (e.g. Leg+K inwf in conjunction
!        with Kleg+K DVR) (PT 02/2013)

         call excite2(nspfdof(m)-nk,ex,indx1)

!        indx is a remapped version of the indx1 array. While indx1
!        doesn't contain the K DOFs, indx has "holes" at the
!        corresponding locations, i.e. indx(n) is meaningless when
!        the n-th DOF of the mode is a K DOF.
         i=0
         call zerovxi(indx,nspfdof(m))

         do n=1,nspfdof(m)
            f=spfdof(n,m)
!           skip over K DOF
            if (sbasis(f).ne.6) then
!              map indx1 to indx
               i=i+1
               indx(n)=indx1(i)
!              check whether the configuration is ok
               if (sbasis(f).eq.5 .or. sbasis(f).eq.8 .or. &
                   (nspfdof(m)-nk.eq.1)) then
!                 for KLeg or Wigner: discard this configuration if the 
!                 excitation is higher than the number of 1D SPFs
!                 available.
                  if (indx(n).gt.dimf(f,s)) goto 10
               else
!                 for non-KLeg/non-Wigner: discard if excitation is 
!                 higher than size of primitive basis, or if higher 
!                 than emax.
                  if (indx(n).gt.min(emax,gdim(f))) goto 10
               endif
            endif
         enddo

!        create mode-SPF by multiplying the DOF-SPFs
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            if (sbasis(f).eq.5) then
               call multspf(spf1d(zspf1d(f,s)),psi(zetf(m,s)),indx(n), &
                            vgdim(f),gdim(f)*gdim(f+1),ngdim(f+1), &
                            nspf,dim(m,s))
            elseif (sbasis(f).eq.8) then
               call multspf(spf1d(zspf1d(f,s)),psi(zetf(m,s)),indx(n), &
                            vgdim(f),gdim(f)*gdim(f+1)*gdim(f+2), &
                            ngdim(f+2),nspf,dim(m,s))
            elseif (sbasis(f-1).eq.5 .or. sbasis(f-1).eq.8 .or. &
                    sbasis(f-2).eq.8) then
               continue
            elseif (sbasis(f).eq.6) then
               call kclearspf(psi(zetf(m,s)),isbaspar(1,f,s),vgdim(f), &
                              gdim(f),ngdim(f),nspf,dim(m,s))
            else
               call multspf(spf1d(zspf1d(f,s)),psi(zetf(m,s)),indx(n), &
                            vgdim(f),gdim(f),ngdim(f),nspf,dim(m,s))
            endif
         enddo
         if (lprint) then
            write (ilog,'(i7,a$)') nspf,')'
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (sbasis(f).ne.6) write (ilog,'(i8,$)') indx(n)
            enddo   
            write (ilog,*)
         endif
      enddo

      return
      end subroutine genmultkleg! of genmultkleg

!***********************************************************************
!
! GENSYMMULTKLEG: generates multi-KLeg SPFs from the 2D (KLeg/K) SPFs
! in case of (anti)symmetrization. At the moment this works only
! for 4D modes of the type KLeg/K/KLeg/K.
!
! The 2D SPFs are already computed (array spf1d) and their (j,m) values
! have been stored in arrays multkleg_[jm] (in genkleg). The excitation
! scheme is determined by a subroutine (sym2kleg_excite...). From the
! configurations returned by this subroutine, we omit all the combinations
! whose (j1,m1,j2,m2) values have (symmetrically) already been encountered.
! This also works in case that the two KLegs do not have the same initial
! state.
!
! The symmetrization of the 4D SPFs actually happens later
! (near the end of runinwf), here we just make sure that this
! symmetrization doesn't run into linear dependencies.
!
! FO 06/2006
!
!***********************************************************************

      subroutine gensymmultkleg(spf1d,psi,m,s)

      implicit none

      integer(long)                                      :: isym,j,m1,f1,&
                                                            f2,e,extotal,n,nspf
      integer(long), dimension(2)                        :: f,ex
      integer(long), intent(in)                          :: m,s
      integer(long), dimension(2,maxklegspf)             :: save_j,save_m
      integer(long), external                            :: slen ! external function

      complex(dop), dimension(spf1ddim), intent(in)      :: spf1d
      complex(dop), dimension(dgldim), intent(inout)     :: psi
      logical(kind=4)                                    :: lprint

      routine='gensymmultkleg'

! --- general checks ---------------------------------------------------

      if (dim(m,s).gt.maxklegspf) then
         write (message,'(a,i4)') &
            'Increase maxklegspf [maxkleg.f90] to at least ',dim(m,s)
         call errormsg
      endif

! --- find requested symmetry ------------------------------------------

      isym=0
      do j=1,2*maxdim
         if (msymmtr(j).eq.0) goto 20 ! end of msymmtr reached
         m1=mod(abs(msymmtr(j)),65536) ! remove persist flag
         f1=mod(m1,256) ! decode
         m1=m1/256
         if (m1.eq.m) then
!           found a symmetry statement for this mode
            if (f1.eq.201) isym=1 ! 201=sym2kleg
            if (msymmtr(j).lt.0) isym=-isym
            if (isym.ne.0) goto 20
         endif
      enddo
 20   continue
      if (isym.eq.0) then
!        oops, no symmetry for this mode after all
!        use unsymmetrized genmultkleg
         write (ilog,'(a/,a/,a,i2,/,a/,a/,a)') &
            '* WARNING ****************************************', &
            '  Internal inconsistency detected:', &
            '  gensymmultkleg called for mode ',m, &
            '  but no sym2kleg statement found for this mode.', &
            '  Using genmultkleg instead.', &
            '* WARNING ****************************************'
         call genmultkleg(spf1d,psi,m,s)
         return
      endif

! --- check for "print" statement --------------------------------------

      call get_nth_kleg(f1,1,m)
      call get_nth_kleg(f2,2,m)
      f(1)=f1
      f(2)=f2
      lprint=.false.
      do n=1,2
         if (isbaspar(5,f(n),s).ne.0) lprint=.true.
      enddo
      if (lprint) then
!        write header
         write (ilog,'(/a,i3,a/,a)') &
            '*** The configurations for multi-KLeg SPFs for mode ',m, &
            ' :','    #spf    j1    m1    j2    m2'
      endif

! --- checks for first SPF ---------------------------------------------

      if (isym.lt.0) then
!        antisym: initial states of the KLegs must not be the same
         if (multkleg_j(f1,1).eq.multkleg_j(f2,1) .and. &
             multkleg_m(f1,1).eq.multkleg_m(f2,1)) then
            message= &
               'Initial states for DOFs ('// &
               modelabel(f1)(1:slen(modelabel(f1)))//','// &
               modelabel(f1+1)(1:slen(modelabel(f1+1)))//') and ('// &
               modelabel(f2)(1:slen(modelabel(f2)))//','// &
               modelabel(f2+1)(1:slen(modelabel(f2+1)))//') must '// &
               'not be the same when antisymmetrization is used.'
            call errormsg
         endif
      endif

! --- generate configurations ------------------------------------------

      ex(1)=1
      ex(2)=1
      do nspf=1,dim(m,s)

!        create 4D SPF by multiplying the 2D SPFs
         do n=1,2
            call multspf(spf1d(zspf1d(f(n),s)),psi(zetf(m,s)),ex(n), &
                         vgdim(f(n)),gdim(f(n))*gdim(f(n)+1), &
                         ngdim(f(n)+1),nspf,dimf(f(n),s))
!           save (j,m) values
            save_j(n,nspf) = multkleg_j(f(n),ex(n))
            save_m(n,nspf) = multkleg_m(f(n),ex(n))
         enddo

         if (lprint) then
!           print (j,m) values
            write (ilog,'(i7,a1,4i6)') nspf,')', &
               save_j(1,nspf),save_m(1,nspf), &
               save_j(2,nspf),save_m(2,nspf)
         endif

!        leave loop early when finished
         if (nspf.eq.dim(m,s)) goto 500

!        calculate next excitation numbers
! 100    call sym2kleg_excite1(ex)
  100    call sym2kleg_excite2(ex,dimf(f1,s),dimf(f2,s))

!        check if something is going wrong
         extotal=ex(1)+ex(2)
         if (extotal.ge.10000) then
            message='extotal running high, looks like an infinite ' &
                  //'loop. Aborting. (KLeg nspf too low? Check log' &
                  //' file.)'
            call errormsg
         endif

!        check that KLeg excitations are not too high
!        (dimf(f,s) is the number of 2D SPFs that have been created)
         do n=1,2
            if (ex(n).gt.dimf(f(n),s)) goto 100
         enddo

!        if antisymmetrical, (j1,m1) and (j2,m2) must differ
         e=0
         if (isym.lt.0 .and. &
             multkleg_j(f1,ex(1)).eq.multkleg_j(f2,ex(2)) .and. &
             multkleg_m(f1,ex(1)).eq.multkleg_m(f2,ex(2))) goto 200

!        check if a symmetrical configuration was already used
         do e=1,nspf
            if (multkleg_j(f1,ex(1)).eq.save_j(2,e) .and. &
                multkleg_m(f1,ex(1)).eq.save_m(2,e) .and. &
                multkleg_j(f2,ex(2)).eq.save_j(1,e) .and. &
                multkleg_m(f2,ex(2)).eq.save_m(1,e)) goto 200
         enddo

!        configuration is ok
         goto 210

 200     continue
!        print why this configuration was omitted
         if (lprint) then
            write (ilog,'(a8,4i6,a,$)') ' ', &
               multkleg_j(f1,ex(1)),multkleg_m(f1,ex(1)), &
               multkleg_j(f2,ex(2)),multkleg_m(f2,ex(2)), &
               '    omitted '
            if (e.eq.0) then
               write (ilog,'(a)') '(j1=j2 & m1=m2)'
            else
               write (ilog,'(a,i3,a)') '(= spf #',e,')'
            endif
         endif
!        try next configuration
         goto 100

 210     continue
      enddo

 500  continue
      return
      end subroutine gensymmultkleg! of gensymmultkleg

!***********************************************************************
!
! SYM2KLEG_EXCITE1: implements the following excitation scheme:
!
!  #4d-SPF  #2d-SPF(1)  #2d-SPF(2)  (extotal = overall excitation)
!     1         1          1           2
!     2         1          2           3
!     3         2          1           3
!     4         2          2           4 
!     5         3          1           4
!     6         1          3           4
!     7         2          3           5
!     8         3          2           5
!     9         1          4           5
!    10         4          1           5
!    11         3          3           6
!    12         4          2           6
!    13         2          4           6
!    14         5          1           6
!    15         1          5           6
!    ...
!
! i.e. increasing overall excitation, and preferring the same excitation
! level for the 2 KLegs.
!
! FO 07/2006
!
!***********************************************************************

!     subroutine sym2kleg_excite1(ex)

!     implicit none

!     integer(long)                              :: extotal,i,j
!     integer(long), dimension(2), intent(inout) :: ex

!!    figure out current total excitation and index (i)
!     extotal=ex(1)+ex(2)
!     j=ex(1)-extotal/2
!     i=2*abs(j)
!     if (j.gt.0) i=i-1
!!    goto next index
!     i=i+1
!!    last index for this extotal reached?
!     if (i.gt.extotal-2) then
!        extotal=extotal+1
!        i=0
!     endif
!!    calculate new excitation numbers
!     j=(i+1)/2
!     if (mod(i,2).eq.0) j=-j
!     ex(1)=extotal/2+j
!     ex(2)=extotal-ex(1)

!     return
!     end subroutine sym2kleg_excite1 ! of sym2kleg_excite1

!***********************************************************************
!
! SYM2KLEG_EXCITE2: implements the following excitation scheme
! (assuming dimf1=6,dimf2=4)
!
!  #4d-SPF  #2d-SPF(1)  #2d-SPF(2)
!     1         1          1
!     2         2          1 
!     3         1          2
!     4         3          1
!     5         1          3
!     6         4          1
!     7         1          4
!     8         5          1
!     9         6          1
!    10         2          2
!    11         3          2
!    12         2          3
!    13         4          2
!    14         2          4
!    15         5          2
!    16         6          2
!    17         3          3
!    ...
! 
! i.e. we try to use all of 2d SPFs as early as possible without
! favoring one or the other KLeg.
!
! FO 07/2006
!
!***********************************************************************

      subroutine sym2kleg_excite2(ex,dimf1,dimf2)

      implicit none

      integer(long)                              :: tmp
      integer(long), intent(in)                  :: dimf1,dimf2
      integer(long), dimension(2), intent(inout) :: ex

      if (ex(1).gt.ex(2)) then
!        swap excitations
         tmp=ex(1)
         ex(1)=ex(2)
         ex(2)=tmp
!        check for 1d spf exhaustion
         if (ex(2).gt.dimf2) then
!           swap back, increase ex1
            tmp=ex(1)
            ex(1)=ex(2)+1
            ex(2)=tmp
!           check
            if (ex(1).gt.dimf1) goto 20
         endif
      else
!        swap back and increase one ex
         tmp=ex(1)
         ex(1)=ex(2)+1
         ex(2)=tmp
!        check
         if (ex(1).gt.dimf1) then
!           try swapping
            tmp=ex(1)
            ex(1)=ex(2)
            ex(2)=tmp
!           check
            if (ex(2).gt.dimf2) goto 20
         endif
      endif
      return

!     if 2d SPFs are exhausted, use next combination
   20 tmp=min(ex(1),ex(2))
      ex(1)=tmp+1
      ex(2)=ex(1)
      return

      end subroutine sym2kleg_excite2 ! of sym2kleg_excite2

!***********************************************************************
! SYMORBITALS symmetrises two sets of (multi-dimensional) SPFs by
! taking the 1st SPF from the fists set, then the 1st SPF from the 2nd
! set, then the 2nd SPF from the first set, etc. Both modes have then
! identical sets of SPFs and one may symmetrise the WF with the aid of
! A-coeff. 
! HDM 10/06
!-----------------------------------------------------------------------

      subroutine symorbitals(spf1,spf2,dim1,dim2,subdim1,subdim2,m1,m2,s) 

      implicit none

      integer(long)             :: i1,i2,e,zeig,dim3
      integer(long), intent(in) :: m1,m2,s,dim1,dim2,subdim1,subdim2
      real(dop)                 :: norm

      complex(dop), dimension(subdim1*dim1)                :: tmpspf
      complex(dop), dimension(subdim1)                     :: tmp
      complex(dop), dimension(subdim1,dim1), intent(inout) :: spf1
      complex(dop), dimension(subdim2,dim2), intent(inout) :: spf2

      write(ilog,'(/a,i3,a,i4,a,i4)')  &
                 'Symmetrise orbital sets for state s =',s, &
                 '  and modes m1 =',m1,' ,  m2 =',m2       

      if(dim1.ne.dim2 .or. subdim1.ne.subdim2) then
            write(6,'(a,2i8)') ' dim1,dim2 =',dim1,dim2
            write(6,'(a,2i8)') ' subim1,subdim2 =',subdim1,subdim2
            routine='symorbitals'
            message='The two sets of SPFs to be symmetrised must '// &
                    'have identical dim, subdim, and gdim parameters.'
            call errormsg  
      endif

      call cpvxz(spf1,tmpspf,subdim1) ! copy first SPF of first set to tmpspf
      i1=1
      i2=0
      zeig=1+subdim1
      dim3=1
      norm=1.0_dop
      write(ilog,901) dim3,i1,norm
      do e = 2, dim1
        if(i1.le.i2) then
  11       i1=i1+1
           call cpvxz(spf1(1,i1),tmp,subdim1)
           call schmidtortho3(tmpspf,tmpspf(zeig),subdim1,dim3,norm)
           if(norm.le.0.1_dop) goto 11
           write(ilog,901) dim3+1,i1,norm
           if(norm.le.0.8_dop) &
           call schmidtortho3(tmpspf,tmpspf(zeig),subdim1,dim3,norm)
           zeig=zeig+subdim1
           dim3=dim3+1
        else
  12       i2=i2+1
           call cpvxz(spf2(1,i2),tmpspf(zeig),subdim1)
           call schmidtortho3(tmpspf,tmpspf(zeig),subdim1,dim3,norm)
           if(norm.le.0.1_dop) goto 12
           write(ilog,902) dim3+1,i2,norm
           if(norm.le.0.8_dop) &
           call schmidtortho3(tmpspf,tmpspf(zeig),subdim1,dim3,norm)
           zeig=zeig+subdim1
           dim3=dim3+1
        endif
      enddo

      call cpvxz(tmpspf,spf1,subdim1*dim1)
      call cpvxz(tmpspf,spf2,subdim1*dim1)

      return

 901  format(i4,")",i6,"-th SPF of  first set,  norm =",f7.4) 
 902  format(i4,")",i6,"-th SPF of second set,  norm =",f7.4) 

      end subroutine symorbitals

!#######################################################################

      subroutine genswarm(psi,spf1d,trajst)

      implicit none

      integer(long)                                    :: s,m,f,n,instate,nst
      integer(long), dimension(gwpinfodim),intent(out) :: trajst
      complex(dop), dimension(dgldim), intent(out)     :: psi
      complex(dop), dimension(spf1ddim), intent(in)    :: spf1d

!-----------------------------------------------------------------------
! get electronic basis
!-----------------------------------------------------------------------
      if (feb .gt. 0) then
         instate = isbaspar(4,feb,1)
         nst = gdim(feb)

! electronic basis set 
         call cpmxz(spf1d(zspf1d(feb,1)),psi(zetf(meb,1)),gdim(feb),&
                    dim(meb,1))

      else
         instate = 1
         nst = 1
      endif

!-----------------------------------------------------------------------
! set up initial conditions for trajectories. If DD in Cartesian coords
! use wigner sampling of full space using QM frequency file to define 
! intial WP. 
! Else use 1D wigner sampling
!-----------------------------------------------------------------------
      if (ldd .and. ddcoo .eq. 2) then
         call genswarm2(psi(zetf(1,1)),trajst,dim(1,1),psi(1),&
              instate,pqdistr)
    
      else
         m=1
         s=1
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            call genswarm1(n,psi(zpsi(s)),psi(zetf(m,s)),&
                 spf1d(zspf1d(f,s)),dim(m,s),nspfdof(m),nst,trajst,instate)
         enddo
      endif


      return
      end subroutine genswarm

!#######################################################################

      subroutine genswarm1(n,psiel,psi,spf1d,dim,nspfdof,nst,trajst, &
                 instate)

      implicit none 

      integer(long)                                     :: e
      integer(long), intent(in)                         :: dim,nspfdof,n,&
                                                           nst,instate
      integer(long), dimension(dim),intent(out)         :: trajst
      complex(dop), dimension(dim), intent(in)          :: spf1d
      complex(dop), dimension(nspfdof,dim), intent(out) :: psi
      complex(dop), dimension(nst), intent(out)         :: psiel

! electronic state coefficients
      call zerovxz(psiel,nst)
      psiel(instate) = 1.0_dop
! trajectory swarm assigned to initial state
      do e=1,dim
         psi(n,e)=spf1d(e)

         trajst(e) = instate
      enddo

      return
      end subroutine genswarm1

!#######################################################################

      subroutine genswarm2(psi,trajst,ntraj1,psiel,instate,pqdistr)

      implicit none

      integer(long), intent(in)                        :: ntraj1,instate,&
                                                          pqdistr
      integer(long)                                    :: s,f,n
      integer(long), dimension(ntraj1),intent(out)     :: trajst
      complex(dop), dimension(ndoftsh,ntraj1), intent(out) :: psi

      complex(dop), dimension(ntraj1,nddstate), intent(out)  :: psiel
      real(dop)                                              :: weight

      integer(long)                            :: ndofnm
      integer(long)                            :: i,ilbl,ifiletyp,&
                                                    e,g,g1,iau
      integer(long), dimension(maxdim)         :: iatiso
      real(dop), dimension(maxdim)             :: freq
      character(len=c5)                        :: filename
      character(len=c1)                        :: fileorient
      character(len=c1), dimension(maxdim)     :: nmlab

      real(dop)                                :: gwidth,xsigma,psigma
      real(dop), dimension(ndofddpes,ntraj1)   :: xcoo,qcoo,xmom,qmom
      complex(dop)                             :: gpair

      logical(kind=4)                          :: lcheck

      write(ilog,'(/,2a)') 'Generating Wigner Distribution'

!-----------------------------------------------------------------------
! trajectory swarm assigned to initial state
!-----------------------------------------------------------------------
      do e=1,ntraj1
         trajst(e) = instate
      enddo

!-----------------------------------------------------------------------
! Assign initial "coefficients"
!-----------------------------------------------------------------------
      psiel(:,:)=0.0_dop

      weight=dble(ntraj1)
      weight = 1.0_dop/dsqrt(weight)
      do e=1,ntraj1
         s = instate
         psiel(e,s) = weight
      enddo

!-----------------------------------------------------------------------
! Ndof for nmodes: ntr = no. trivial modes (6 by default)
!-----------------------------------------------------------------------
      ndofnm = ndofddpes - ntr

      ilbl = index(ddtrfile,' ')-1
      filename = ddname(1:ddlaenge)//'/'//ddtrfile(1:ilbl)

      open(idat,file=filename,status='old',err=999)
      ilbl = index(filename,' ')-1
      write(ilog,'(/,2a)') 'Opening trafo file: ',filename(1:ilbl)

!-----------------------------------------------------------------------
! get information from file
!-----------------------------------------------------------------------
      call getintyp(idat,ifiletyp,filename,ilog)
      write(ilog,'(15x,a,i5)') 'Ab initio type:',ifiletyp
      iau = 0
      fileorient='input'
      call getxcoo(idat,ddxcoo0,ifiletyp,iau,fileorient,iatsym, &
           iatiso,ndofddpes,nddstate,ndofnm,ilog)

      iau = 0
      call getnm(idat,ifiletyp,nmlab,ddtransf,freq,iatsym,iau,&
           maxdim,ndofddpes,nddstate,ndofnm,ilog,1,.true.)

! log frequencies found
      write(ilog,'(a)') ' Frequencies read (in cm-1)'
      write(ilog,'(8f10.2)') (freq(i)*invcm,i=1,ndofnm)

! make any negative frequencies positive
      freq(1:ndofnm)=abs(freq(1:ndofnm))

      close(idat)

! create the transformation matrix
      iau = 0
      call nm2xmat(ddtransf,ddtransb,iatsym,freq,iau,ndofddpes,maxdim,&
           ndofnm,ilog)

!-----------------------------------------------------------------------
! rsbaspar(1,f) : initial geom
! rsbaspar(2,f) : initial momentum
!-----------------------------------------------------------------------
      lcheck = .false.
      do g=1,ndofddpes
         if (abs(rsbaspar(1,g,1)-ddxcoo0(g)) .gt. 1.0d-6) lcheck = .true.
      enddo
      if (lcheck) then
         routine = 'Genswarm2 (genpsimod.f90)'
         message = "Frequency file given does not match intial geometry"
         call errormsg
      endif

! first point at cente of WP
      qcoo(:,1)=0.0_dop
      qmom(:,1)=0.0_dop
      gpair=0.0_dop

! Sample Wigner distribution
      n=0
      do e=2,ntraj1
         if (pqdistr .eq. 0) then
! constant distribution - for testing
            qcoo(:,e)=0.0_dop
            qmom(:,e)=0.0_dop
            if (mod(e-2,ndofnm) .eq. 0) then
               n = n+1
               gpair=0.2*n
            endif
            g = mod(e-2,ndofnm) + 1
! Scaling for Gaussian width
            gwidth = rsbaspar(3,g,1)
            xsigma=1.0_dop/gwidth
            psigma=0.5_dop*gwidth

            qcoo(g,e)=xsigma*real(gpair)
            qmom(g,e)=psigma*aimag(gpair)
         else
            do g=1,ndofnm
               if (basis(g) .eq. 19) cycle ! do not sample frozen DOFs

! Scaling for Gaussian width
               gwidth = rsbaspar(3,g,1)
               xsigma=1.0_dop/gwidth
               psigma=0.5_dop*gwidth

               gpair=randgaussian()

               qcoo(g,e)=xsigma*real(gpair)
               qmom(g,e)=psigma*aimag(gpair)
            enddo
         endif
      enddo

! transform to Cartesians
      do e=1,ntraj1
         do g=1,ndofddpes
            xcoo(g,e)=ddxcoo0(g)
            xmom(g,e)=rsbaspar(2,g,1)
            do g1=1,ndofnm
               xcoo(g,e)=xcoo(g,e)+ddtransf(g,g1)*qcoo(g1,e)
               xmom(g,e)=xmom(g,e)+ddtransf(g,g1)*qmom(g1,e)
            enddo
         enddo
      enddo

!transfer to psi array
      do e=1,ntraj1
         g=0
         do f=1,ndof
            if (basis(f) .eq. 19) cycle
            if (f .eq. feb) cycle
            g=g+1
            psi(g,e) = dcmplx(xcoo(f,e),xmom(f,e))
         enddo
      enddo

      return
  999 routine='Rdddtrfile (wroper.F)'
      ilbl=index(filename,' ')-1
      write(message,'(2a)') 'Error opening file: ',filename(1:ilbl)
      call errormsg

      end subroutine genswarm2

      end module genpsimod


