!***********************************************************************
!
!               MCTDH   subroutine  GENRHO1
!
! contains: genrho1,genspdos,cntrex,multspf1,multspf2,crefi,crese,
!           thermalize1,thermfirst 
!
!***********************************************************************
      module genrho1mod

use decimal, only: dop, long
use logdat

use global
use maxv, only: maxdim
use dvrdatmod
use griddatmod
use psidef
use runinwfmod
use operdef

use genphi1, only: swapspf,excite2
use gencoeffmod, only: rdkoeff
use schmidtorthomod, only: schmidtortho
use wftree1, only: set_hartree_avec

use xvlib,   only: xvixdzo,vvxazz,xqxxzz,xqxxzzo
use op1lib,  only: initvxz, normvxz, cpvxz, unitqxz, offdqxz, offiqxz
use d2tens, only: mkherm

      implicit none
      private
      public :: genrho1
     contains

!***********************************************************************

      subroutine genrho1(psi,spf1d,trafo,ort,agmat)

      implicit none

      integer(long)                                    :: s,t,p,m,f,inspf
      real(dop), dimension(dvrdim), intent(in)         :: trafo
      real(dop), dimension(ortdim), intent(in)         :: ort
      complex(dop), dimension(dgldim), intent(inout)   :: psi
      complex(dop), dimension(spf1ddim), intent(inout) :: spf1d
      complex(dop), dimension(agmatdim), intent(inout) :: agmat

!-----------------------------------------------------------------------
! move initially populated spf to be 1st in list
! Do not do this on the electronic basis for a mixed electronic excitation
!-----------------------------------------------------------------------
      do s=1,nstate
         do t=s,nstate
            p = 1
            do f=1,ndof
               inspf=isbaspar(4,f,p)
               if ((f .eq. feb) .and. (dlstate .ne. drstate)) cycle

               if (inspf .ne. 1 .and. ddimf(f,s,t) .gt. 0) then
                  call swapspf(spf1d(dzspf1d(f,s,t)),inspf, &
                       gdim(f),ddimf(f,s,t))
               endif
            enddo
         enddo
      enddo

!     Generate SPDOs
      do s=1,nstate
         do t=s,nstate
            do m=1,nmode
               if (m .eq. meb) then
                  if (s .gt. 1) then
                     routine='Genrho1 (genrho1mod)'
                     message = 'Single-set SPDOs asked for but multi-set calculation?'
                     call errormsg
                  endif
                  call genspdosel(spf1d(dzspf1d(feb,1,1)),psi(dzetf(m,s,t)),gdim(feb))
               else
                  call initvxz(psi(dzetf(m,s,t)),subdim(m)*ddim(m,s,t))
                  call genspdos(spf1d,psi,m,s,t)
                  if (temperature0 .gt. tiny) then
                     call thermalize1(psi(dzetf(m,s,t)),trafo,ort,&
                          ddim(m,s,t),subdim(m),supgdim(m),m,s,t)
                  endif                 
               endif                 
            enddo
         enddo
      enddo
      do s=1,nstate
         do t=s+1,nstate
            do m=1,nmode
!     Note that all initial SPDOs are hermitian
               call cpvxz(psi(dzetf(m,s,t)),psi(dzetf(m,t,s)),&
                    subdim(m)*ddim(m,s,t))
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! generate coefficients
!-----------------------------------------------------------------------
      if (azahl.eq.0) then
! first set layer expansions in ML-DMCTDH
         if (lrunml) then
            if (.not. tree%istop) then
               routine = 'Genrho1 (genrho1mod.f90)'
               message = 'Electronic DOF not at top of tree '
               call errormsg
            endif

            call set_hartree_avec(psi,tree)

            psi(1:adim) = 0.0_dop  ! set top layer below
         endif

! set coefficient for selected state 
         if ((dlstate.lt.1) .or. (dlstate.gt.nddstate)) then
            routine = 'Genrho1 (genrho1mod.f90)'
            message = 'Choose correct left_state.'
            call ErrorMsg
         endif
         if ((drstate.lt.1) .or. (drstate.gt.nddstate)) then
            routine = 'Genrho1 (genrho1mod.f90)'
            message = 'Choose correct right_state.'
            call ErrorMsg
         endif
         
         if (lmult) then
            if (dlstate .eq. drstate) then
               psi(dzpsi(dlstate,drstate))=(1.0_dop,0.0_dop)   
            else
               psi(dzpsi(dlstate,dlstate))= 0.5_dop
               psi(dzpsi(drstate,drstate))= 0.5_dop
               psi(dzpsi(dlstate,drstate))= 0.5_dop
               psi(dzpsi(drstate,dlstate))=psi(dzpsi(dlstate,drstate))
            endif        
         else if (leb) then
            if (dlstate .eq. drstate) then
               psi(dzpsi(1,1))=(1.0_dop,0.0_dop)   
            else
               call aselectrho1(psi(dzpsi(1,1)),vddim(meb,1,1),&
                    nddstate,nddim(meb,1,1))
            endif
         endif

      else
! read coefficients
         call rdkoeff(psi,agmat)

      endif

      return
      end subroutine genrho1

!#######################################################################
!
!     GENSPDOS: Generates an orthonormal set of hermitian SPDOs
!
!     10/99 AR
!#######################################################################


      subroutine genspdos(spf1d,psi,m,s,t)

      implicit none

      integer(long)       :: n, f, nspf,ex1, ex2, n1, n2, zeigp

      integer(long), intent(in)                     :: m, s, t
      integer(long), dimension(maxdim)              :: indx1, indx2
      complex(dop), dimension(spf1ddim), intent(in) :: spf1d
      complex(dop), dimension(dgldim), intent(inout):: psi
      complex(dop), dimension(sdim)                 :: workc

!     Initial configuration
      ex1 = 0
      ex2 = 0
      n1 = 1
      n2 = 1
!     Generate first hermitian SPDO: |1><1|
      nspf = 1
      do n=1,nspfdof(m)
         f=spfdof(n,m)

         indx1(n) = 1
         call multspf1(spf1d(dzspf1d(f,s,t)),psi(dzetf(m,s,t)),indx1(n), &
              vgdim(f),gdim(f),ngdim(f)*supgdim(m),ddim(m,s,t))
         indx2(n) = 1
         call multspf2(spf1d(dzspf1d(f,s,t)),psi(dzetf(m,s,t)),indx2(n), &
              supgdim(m)*vgdim(f),gdim(f),ngdim(f),ddim(m,s,t))

      enddo
      if (nspf .ge. ddim(m,s,t)) go to 30

!     Higher configurations
 20   continue
      call cntrex(nspfdof(m),indx1,indx2,n1,n2,ex1,ex2) 

      if (n1 .eq. n2) then
!     Generate one hermitian SPDO: |n1><n1|
         nspf = nspf + 1
         zeigp = dzetf(m,s,t) + (nspf-1)*subdim(m)
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            call multspf1(spf1d(dzspf1d(f,s,t)),psi(zeigp),indx1(n), &
                 vgdim(f),gdim(f),ngdim(f)*supgdim(m),ddim(m,s,t))
            call multspf2(spf1d(dzspf1d(f,s,t)),psi(zeigp),indx2(n), &
                 supgdim(m)*vgdim(f),gdim(f),ngdim(f),ddim(m,s,t))
         enddo
         if (nspf .lt. ddim(m,s,t)) then
            goto 20
         else
            goto 30
         endif
      else
!     Generate two hermitian SPDOs
         call initvxz(workc,subdim(m))
!     Generate auxiliary SPDO: |n1><n2|
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            call multspf1(spf1d(dzspf1d(f,s,t)),workc,indx1(n), &
                 vgdim(f),gdim(f),ngdim(f)*supgdim(m),ddim(m,s,t))
            call multspf2(spf1d(dzspf1d(f,s,t)),workc,indx2(n), &
                 supgdim(m)*vgdim(f),gdim(f),ngdim(f),ddim(m,s,t))
         enddo

         nspf = nspf + 1
         zeigp = dzetf(m,s,t) + (nspf-1)*subdim(m)

!     |n1><n2|  ->  1/sqrt{2} ( |n1><n2| + |n2><n1| ) 
         call crefi(workc,psi(zeigp),supgdim(m))
         if (nspf .eq. ddim(m,s,t)) goto 30

! do not create imaginary SPDO if realpsi wanted
         if (.not. lrealpsi) then
            nspf = nspf + 1
            zeigp = dzetf(m,s,t) + (nspf-1)*subdim(m)
!     |n1><n2|  ->  i/sqrt{2} ( |n1><n2| - |n2><n1| ) 
            call crese(workc,psi(zeigp),supgdim(m))
         endif

         if (nspf .lt. ddim(m,s,t)) goto 20
      endif         

 30   continue

      return
      end subroutine genspdos

!#######################################################################
!
!     GENSPDOS: Generates a hermitian set of hermitian SPDOs
!     similar to Pauli matices, e.g. for 2 states:
!     1 0     0   0.5    0   -0.5i   0  0
!     0 0     0.5 0      0.5i 0      0  1
!
! the factor sqrt(2) means the operators are normalised by 1/sqrt(2)
!
!     1/19 GW
!#######################################################################

      subroutine genspdosel(spf1d,psi,nst)

      implicit none

      integer(long)       :: s,s1
      complex(dop)        :: x = (0,1.0_dop)
      complex(dop)        :: y = dsqrt(2.0_dop)

      integer(long), intent(in)                     :: nst
      complex(dop), dimension(nst,nst), intent(in)  :: spf1d
      complex(dop), dimension(nst,nst,nst,nst), intent(out) :: psi

      do s=1,nst
         call vvxazz(spf1d(1,s),spf1d(1,s),psi(1,1,s,s),nst)
         do s1=s+1,nst
            call vvxazz(spf1d(1,s1),spf1d(1,s),psi(1,1,s1,s),nst)
            call xqxxzz (x,psi(1,1,s1,s),psi(1,1,s,s1),nst)
            call mkherm(psi(1,1,s1,s),nst)
            call mkherm(psi(1,1,s,s1),nst)
            call xqxxzzo (y,psi(1,1,s1,s),nst)
            call xqxxzzo (y,psi(1,1,s,s1),nst)
         enddo
      enddo

      return
      end subroutine genspdosel

!#######################################################################
!
! ASELECTRHO1
!
! For single-set type I density matrices.
! Select and set the coefficients for an initial mixed electronic state
!
! NB ddim(meb,1,1) = nddstate*nddstate hence coefficent vector
! psi(vddim,ddim,nddim) = psi(vddim,nst,nst,nddim)
! 
!#######################################################################

      subroutine aselectrho1(psi,vddim1,nst,nddim1)

      implicit none

      integer(long), intent(in) :: vddim1,nst,nddim1
      complex(dop), dimension(vddim1,nst,nst,nddim1), intent(out) :: psi

      psi(1,dlstate,dlstate,1)= 0.5_dop
      psi(1,drstate,drstate,1)= 0.5_dop
! basis function (s1,s) with s < s1 is the real part of the hermitian SPDOs
! This ensures a real initial density matrix
      if (dlstate .gt. drstate) then
         psi(1,dlstate,drstate,1)= 1.0/sqrt(2.0_dop)
         psi(1,drstate,dlstate,1)= 0.0_dop
      else
         psi(1,drstate,dlstate,1)= 1.0/sqrt(2.0_dop)
         psi(1,dlstate,drstate,1)= 0.0_dop
      endif

      return
      end subroutine aselectrho1

!#######################################################################
!
!     Auxiliary subroutines for genmultspf I: Creation of configurations
!
!     cntrex:  take care that  n1 >= n2 for |n1><n2|
!     excite2: -> genphi1.F
!
!#######################################################################

      subroutine cntrex(dim,b1,b2,n1,n2,ex1,ex2)

      implicit none

      integer(long)                               :: i
      integer(long), intent(in)                   :: dim
      integer(long), intent(inout)                :: n1, n2, ex1, ex2
      integer(long), dimension(dim),intent(inout) :: b1, b2

      if (n1 .gt. n2) then
!     |n1><n2| -> |n1>|n2+1|
         n2 = n2 + 1
         if (dim .eq. 1) then
            ex2 = ex2 + 1
            b2(1) = b2(1) + 1
         else
            call excite2(dim,ex2,b2)
         endif
      else
!     |n1><n1| -> |n1+1><1|          
         n1 = n1 + 1
         if (dim .eq. 1) then
            ex1 = ex1 + 1
            b1(1) = b1(1) + 1
         else
            call excite2(dim,ex1,b1)
         endif
         n2 = 1
         ex2 = 0
         do i=1,dim
            b2(i) = 1
         enddo
      endif
      
      return
      end subroutine cntrex

!#######################################################################
!
!     Auxiliary subroutines for genmultspf II: 
!         Realisation of the configurations
!
!#######################################################################

      subroutine multspf1(spfs,psi,indx,vgdim,gdim,ngdim,dim)

      implicit none

      integer(long)             :: n,v,g
      integer(long), intent(in) :: vgdim,gdim,ngdim,dim,indx

      complex(dop), dimension(gdim,dim), intent(in)            :: spfs
      complex(dop), dimension(vgdim,gdim,ngdim), intent(inout) :: psi

      do n=1,ngdim
         do g=1,gdim
            do v=1,vgdim
               psi(v,g,n)=psi(v,g,n)*spfs(g,indx)
            enddo
         enddo
      enddo

      return
      end subroutine multspf1

!#######################################################################

      subroutine multspf2(spfs,psi,indx,vgdim,gdim,ngdim,dim)

      implicit none

      integer(long)             :: n,v,g
      integer(long), intent(in) :: vgdim,gdim,ngdim,dim,indx

      complex(dop), dimension(gdim,dim), intent(in)            :: spfs
      complex(dop), dimension(vgdim,gdim,ngdim), intent(inout) :: psi

      do n=1,ngdim
         do g=1,gdim
            do v=1,vgdim
               psi(v,g,n)=psi(v,g,n)*conjg(spfs(g,indx))
            enddo
         enddo
      enddo

      return
      end subroutine multspf2

!#######################################################################
!
!     Auxiliary subroutines for genmultspf III: 
!         Make two hermitian SPDOs out of one
!
!#######################################################################

      subroutine crefi(a,b,dim)

      implicit none

      integer(long)                                 :: i, j
      integer(long), intent(in)                     :: dim
      complex(dop)                                  :: fac
      complex(dop), dimension(dim,dim), intent(in)  :: a
      complex(dop), dimension(dim,dim), intent(out) :: b

      fac = (1.0_dop,0.0_dop) / sqrt(2.0_dop)
      do i=1,dim
         do j=i,dim
            b(i,j) = fac*( a(i,j) + conjg(a(j,i)) )
            b(j,i) = conjg(b(i,j))
         enddo
      enddo
      
      return
      end subroutine crefi

!#######################################################################

      subroutine crese(a,b,dim)

      implicit none

      integer(long)                                 :: i, j
      integer(long), intent(in)                     :: dim
      complex(dop)                                  :: fac
      complex(dop), dimension(dim,dim), intent(in)  :: a
      complex(dop), dimension(dim,dim), intent(out) :: b
      
      fac = (0.0_dop,1.0_dop) / sqrt(2.0_dop)
      do i=1,dim
         do j=i,dim
            b(i,j) = fac*( a(i,j) - conjg(a(j,i)) )
            b(j,i) = conjg(b(i,j))
         enddo
      enddo
      
      return
      end subroutine crese

!#######################################################################
!
!     Thermalizes the first SPDO and Schmidt-orthonormalizes the set
!     afterwards
!
!#######################################################################

      subroutine thermalize1(psi,trafo,ort,dim1,subdim1,supgdim1,m,s,t)

      implicit none

      integer(long)             :: f,n,ierr,ilbl,np,f1,i
      integer(long), intent(in) :: m,s,t,dim1,subdim1,supgdim1

      real(dop)                                            :: norm,freqfac,factor
      real(dop), dimension(dvrdim), intent(in)             :: trafo
      real(dop), dimension(ortdim), intent(in)             :: ort
      complex(dop), dimension(subdim1,dim1), intent(inout) :: psi
      logical(kind=4)                                      :: lfound
      logical(kind=4), save  :: firstcall = .true.

      if (firstcall) then
         write(ilog,*)
         write(ilog,'(50a)') ('*',i=1,50)
         write(ilog,'(a)') '**** Thermalization'
         write(ilog,*)
         write(ilog,'(a,f20.15)') 'Temperature [a.u.] = ',temperature0
         write(ilog,*)
         firstcall = .false.

         temperature_psi = temperature0
      endif

!-----------------------------------------------------------------------
! infinite temp.
!-----------------------------------------------------------------------
      if (temperature0 .gt. 99999990.0) then

         call unitqxz(psi(1,1),supgdim1)
         if (dim1 .ge. 2) call offdqxz(psi(1,2),supgdim1)
         if (dim1 .ge. 3) call offiqxz(psi(1,3),supgdim1)

!     Normalise 
         do i=1,max(dim1,3)
            call normvxz(psi(1,i),norm,subdim(m))
            call xvixdzo(norm,psi(1,i),subdim(m))
         enddo

         go to 900
      endif

!     Check basis set
      do n=1,nspfdof(m)
         f=spfdof(n,m)
         if (sbasis(f).ne.1) then
            routine = "Thermalize1 (genrho1mod.F90)"
            message = 'T>0 implemented only for Gauss or HO'
            call errormsg
         endif
      enddo

!     Thermalize first SPDO
      call initvxz(psi(1,1),subdim(m))
      factor=1.0d0
      do n=1,nspfdof(m)
         f=spfdof(n,m)

         freqfac = 1.0_dop
         if (ltemp_mfsc) then
            lfound=.false.
            do np=1,npar
               if (apar(np)(1:5) .eq. 'freq0') then
                  ilbl=index(apar(np),' ')-1
                  read(apar(np)(7:ilbl),*,err=901) f1
                  if (f1 .eq. f) then
                     freqfac = rpar(np) 
                     lfound = .true.
                  endif
               endif
            enddo
            if (.not. lfound) then
               routine = 'Thermalize1 (genrhomod.f90)'
               write(message,'(a,i5)') 'Frequency not found in parameters for DOF: ',f
               call errormsg
            endif
         endif

         call thermfirst(psi(1,1),temperature0,vgdim(f),gdim(f),ngdim(f),&
              trafo(zdvr(f)),ort(zort(f)),&
              basis(f),rpbaspar(1,f),ipbaspar(1,f),&
              rsbaspar(1,f,1),freqfac,factor)
      enddo
!     Normalise 
      call normvxz(psi(1,1),norm,subdim(m))
      call xvixdzo(norm,psi,subdim(m))
      write(ilog,'(3x,a,i2,a,f10.8)') 'mode = ',m, &
                ' , thermalization-factor = ',factor

  900 continue

      call schmidtortho(psi,subdim(m),ddim(m,s,t),ierr)
      if( ierr .gt. 0 ) then
         message='Cannot Schmidt-orthogonalize SPDO''s.'
         call errormsg
      end if

      
      return
  901 routine='Thermalize1'
      message = "Frequencies required in format freq0_f "
      message = message(1:39)//"where f is in integer"
      call errormsg
      end subroutine thermalize1

!#######################################################################

      subroutine thermfirst(psi,temperature0,vgdim,gdim,ngdim,trafo,ort, &
                 basis,rpbaspar,ipbaspar,rsbaspar,freqfac,factor)

      implicit none

      integer(long)                              :: blz,ibleg,i,j,k,l,m
      integer(long), intent(in)                  :: vgdim,gdim,ngdim,basis
      integer(long), dimension(*), intent(in)    :: ipbaspar
      real(dop)                                  :: gxeq,gmom,gfreq,gmass,wei1,&
                                                    wei2,alpha,homass,hofreq,hoxeq
      real(dop), intent(in)                      :: freqfac
      real(dop), intent(in)                      :: temperature0
      real(dop), intent(inout)                   :: factor
      real(dop), dimension(*),intent(in)         :: rpbaspar,rsbaspar
      real(dop), dimension(gdim),intent(in)      :: ort
      real(dop), dimension(gdim,gdim),intent(in) :: trafo

      complex(dop)                               :: gwidth
      complex(dop), dimension(vgdim,gdim,ngdim*vgdim,gdim,ngdim), intent(inout) :: psi


!     Get primitive basis (DVR) parameters
      hoxeq = -999.0_dop
      hofreq = -999.0_dop
      homass = -999.0_dop
      blz=-999
      ibleg =-999
      if (basis .eq. 1 .or. basis .eq. 11) then
         hoxeq  = rpbaspar(1)
         hofreq = rpbaspar(2)
         homass = rpbaspar(3)
      else if (basis .eq. 2) then
         blz=ipbaspar(1)
         ibleg=ipbaspar(2)
      endif

!     Get spf basis parameters
      gxeq   = rsbaspar(1)
      gmom   = rsbaspar(2)
      gwidth = dcmplx(rsbaspar(3),rsbaspar(4))      
      gmass  = rsbaspar(5)
      gfreq  = 0.5_dop * dble(gwidth)**2 / gmass
! multiply spf frequency by mode frequency if mass-frequency scaled coords used.
      alpha  = tanh( 0.5_dop*(gfreq*freqfac)/temperature0 )
      factor = factor*exp(-(gfreq*freqfac)/temperature0)

!     Thermalize
      do m=1,ngdim
         do l=1,gdim

!     Choose first weight
            if (basis.eq.1) then
               wei1 = trafo(1,l) &
                    /exp(-0.5_dop*homass*hofreq*(ort(l)-hoxeq)**2)
            elseif (basis.eq.11) then
               wei1 = trafo(1,l)/(ort(l)-hoxeq) &
                    /exp(-0.5_dop*homass*hofreq*(ort(l)-hoxeq)**2)
            elseif (basis.eq.2) then
               if (blz.eq.0) then
                  wei1 = trafo(1,l)
               elseif (blz.gt.0) then
                  wei1 = trafo(1,l)/sin(ort(l))**blz
               else
                  write(6,*) ' GENGAUSS, wrong blz: ', blz
                  stop
               endif
               if(ibleg.eq.1)  wei1 = wei1/cos(ort(l))
            elseif (basis.ge.3 .and. basis.le.5 ) then
               wei1 = 1.0_dop
            elseif (basis.eq.9 .or. basis.eq.21) then
               wei1 = 1.0_dop
            else
               write(6,*) ' GENGAUSS, wrong basis no: ', basis
               stop
            endif

            do k=1,ngdim*vgdim
               do j=1,gdim

!     Choose second weight
                  if (basis.eq.1) then
                     wei2 = trafo(1,j) &
                          /exp(-0.5_dop*homass*hofreq*(ort(j)-hoxeq)**2)
                  elseif (basis.eq.11) then
                     wei2 = trafo(1,j)/(ort(j)-hoxeq) &
                          /exp(-0.5_dop*homass*hofreq*(ort(j)-hoxeq)**2)
                  elseif (basis.eq.2) then
                     if (blz.eq.0) then
                        wei2 = trafo(1,j)
                     elseif (blz.gt.0) then
                        wei2 = trafo(1,j)/sin(ort(j))**blz
                     else
                        write(6,*) ' GENGAUSS, wrong blz: ', blz
                        stop
                     endif
                     if(ibleg.eq.1)  wei2 = wei2/cos(ort(j))
                  elseif (basis.ge.3 .and. basis.le.5 ) then
                     wei2 = 1.0_dop
                  elseif (basis.eq.9 .or. basis.eq.21) then
                     wei2 = 1.0_dop
                  else
                     write(6,*) ' GENGAUSS, wrong basis no: ', basis
                     stop
                  endif

                  do i=1,vgdim
                     psi(i,j,k,l,m) = psi(i,j,k,l,m)*wei1*wei2* &
                          exp( -0.25_dop*gmass*gfreq* ( &
                               (ort(j)+ort(l)-2.0_dop*gxeq)**2 *alpha &
                               + (ort(j)-ort(l))**2 /alpha ) ) &
                        * exp(dcmplx(0.0_dop,gmom*(ort(j)-ort(l))))
                  enddo ! of i
               enddo    ! of j
            enddo       ! of k
         enddo          ! of l
      enddo             ! of m

      return
      end subroutine thermfirst

      end module genrho1mod


