!#######################################################################
! MCTDH-module GETPSI
!
! reads in an old wavefunction and tailors it to new requirements
!
! getpsi: reads and alters wavefunction for a restart run
! akcopy, psicopy: subroutines needed by restpsi
!
! contains: getpsi_mult,getpsi_sing,getex,akcopy,getpsi3d,getcoeff3d,
!           getblk,getblka,
!           akcopy_wf_to_d2,psi2ex1,mgenhigher,kgenhigher,
!           kgenhigher2,gwp2sep,sep2gwp
!
! getpsi_mult : for multi-set WFs
! getpsi_sing : for single-set WFs
! getex   : similar to getpsi, but for exact Wf.
! getblka : for block-A keyword.
! getblk  : for block-SPF keyword.
!
! akcopy, realpsi, psi2ex1, mgenhigher, kgenhigher, kgenhigher2, wkgetpsi
! wiggenhigher, wigset, nextwig
!
!#######################################################################

      module getpsimod

use decimal, only: long, dop
use logdat
use mmlib,   only: mqxczz, mqxtzz
use mtlib,   only: mtxxzz,qtxxzz,qtaxzz
use xvlib,   only: vvaxzz, xvxxzzr, xvixdzo, xvxxzzo
use schmidtorthomod, only: schmidtortho, schmidtortho2
use op1lib,  only: zeromxz, zerovxz, normvxz, cpmxz, cpvxz, trhxz,&
                   cuttxz2, cuttxz3

use global
use globmemmod
use mltypes
use mlbase
use dvrdatmod
use griddatmod
use iopsidef
use inputmlmod
use genphi1
use multilayer
use psidef
use linear
use wftree
use wftree1
use ctensor
use htucker
use treemod
use if_grid
use eqofmotion1, only: compute_dmat_tree
use psiutil
use gwplib, only: renormgwpa,diagonal_moments
use compdatmod
use compdat, only: getcsys,getcpsi,savecsys,savecpsi
use runinwfmod
use channels

use densitymod, only: density
use iorst
use gwplib, only: gwp_param, gwp_overlap

use iogrddef, only: griddat
use iopsidef, only: psidat
use rdopermod, only: get_num_klegs, get_num_wigs
use rddvrmod, only: dvrinfo
use trafonatmod, only: trafonat,reorthopsi
use datenmod
use dmatmod, only: alloc_dmat_wftree
use wftrafo
use mlanalysis
use getrho1mod
use d2tens, only: mkherm,cuttens2,filltens2,filltens2c
use d1tens, only: cuttens1,filltens1,filltens1c

      implicit none
      private
      public :: getinwf,resizeml,getpsi3d,getcoeff3d,&
                wkgetpsi,getblka,getblk,realpsi
! unused routines
      public :: psi2ex1

      contains

      subroutine getinwf(psi)

      implicit none

      complex(dop), dimension(dgldim), intent(out) :: psi

      write (ilog,*)
      write(ilog,'(a)') '***** Restart calculation *****'
      Write(ilog,*)

      write(ilog,'(a)')  &
          'Restart file read from :  ',rname(1:rlaenge)//'/restart'
      if (lorthopsi) then
         write(ilog,'(a)') &
            'The initial wavefunction is Schmidt-orthogonalized.'
      else
         write(ilog,'(a)') &
            'The initial wavefunction is not Schmidt-orthogonalized.'
      endif
      if(lrealpsi) write(ilog,'(a)')'Real part of Psi taken. '

!-----------------------------------------------------------------------
! Select appropriate subroutine according to WF type
!-----------------------------------------------------------------------
      if (lexact) then
         if (dentype .eq. 0) then
            call getex(psi,mi(mijindx))
         else if (dentype .eq. 1) then
            call getrhoex(psi,mi(mijindx))
         else if (dentype .eq. 2) then
            routine='Getinwf (runinwf.f90)'
            message='Exact density matrices cannot be type II. Use density1'
            call errormsg
         endif
         
      else if (lmult) then
         if (dentype .eq. 0 .or. dentype .eq. 2) then
            call getpsi_mult(psi,mr(mrort),mi(mijindx),agmat,mc(mchin),&
                 mc(mcrueck))
         else if (dentype .eq. 1) then
            call getrho1(psi,dicht1,dicht2, &
                 dicht3,dicht4,mc(mcworkc), &
                 mi(mijindx))
         endif

      else   ! Single set. Includes ML
         call getpsi_sing(psi,mr(mrort),mi(mijindx),agmat,&  
              mc(mcexphin),mc(mcexprueck))
      endif

      return
      end subroutine getinwf

!#######################################################################
! GETPSI_MULT (based on previous getinwf)
!
! called from runinwf
! calls akcopy,phicopy,calcnorm,normpsi
!
! for a restart calculation, the initial wavefunction is read from the
! restart file. It is then possible to alter the number of functions
! used in the MC expansion, and if required higher functions are
! generated
!
! This is for multi-set WFs when altering SPFs / A-coeffs between states
!
! psi: initial wavefunction
! ort: grid positions
!
!#######################################################################

      subroutine getpsi_mult (psi,ort,jindx,agmat,exphin,exprueck)

      implicit none

      integer(long)       :: m,s,s1,s2,j,k,n,is,js,mblock,ierr,&
                             chkdvr,chkgrd,chkpsi,chkprp,idum,iflag,dim2,p,pp,&
                             zeig,zeig1,zeig2,i,npacket2,npackstates,t1,t2,dim3
      integer(long)       :: nwf

      integer(long), dimension(1)                     :: idum1
      integer(long), dimension(jindxdim), intent(out) :: jindx
      real(dop)                                       :: shift,norm,told
      real(dop), dimension(1)                         :: rdum
      real(dop), dimension(ortdim), intent(in)        :: ort
      real(dop), dimension(:), allocatable            :: olddicht3

      complex(dop), dimension(1)                      :: cdum
      complex(dop), dimension(dgldim), intent(out)    :: psi
      complex(dop), dimension(:), allocatable         :: oldpsi
      complex(dop), dimension(:), allocatable         :: oldagmat
      complex(dop), dimension(:), allocatable         :: tmppsi
      complex(dop), dimension(:), allocatable         :: olddicht1,olddicht2,&
                                                         olddicht4
      complex(dop), dimension(expdim), intent(in)     :: exphin,exprueck
      complex(dop), dimension(agmatdim), intent(out)  :: agmat
      complex(dop), dimension(maxdof*maxspf*maxspf)        :: z
      complex(dop), dimension(maxdof*maxdof*maxspf*maxspf) :: ai
      complex(dop), dimension(gs2matdim)                   :: goverlap

      logical(kind=4)                                 :: ldum,lmult2, &
                                                         leb2,lmulpack2
      logical(kind=4)                                 :: lerr,linwf
      character(len=c5)                               :: filename
      character(len=20)                               :: cerr

      routine='Getpsi_mult (for Multi-set WFs)'
      if(citype.gt.0) then
         message='Getpsi_mult is not implemented for selected-CI.'
         call errormsg
      endif
      if(lignore)  write(ilog,'(a)')'### Read-Inwf: ignore is set! ###'

      call zerovxz(psi,dgldim)

      npackstates = nstate/npacket 

!-----------------------------------------------------------------------
! save structure-data of psi, because rdrst will overwrite
!-----------------------------------------------------------------------
      call psidat
      call savecsys(1)
      call savecpsi(1)
      lmult2=lmult
      leb2=leb
      lmulpack2=lmulpack
      npacket2=npacket

!-----------------------------------------------------------------------
! Do-Loop over all restart files defined in read-inwf block
!-----------------------------------------------------------------------
      do nwf = 1, nreadwf

!-----------------------------------------------------------------------
! open old WF file
!-----------------------------------------------------------------------
         rname = readwffile(nwf)
         rlaenge = index(rname,' ')-1
         open(irst,file=rname,form='unformatted',status='old',err=900)

!-----------------------------------------------------------------------
! read information about psi on restart file. This must be done before
! reading the file, as the old wavefunction may be different from new.
!-----------------------------------------------------------------------
         chkdvr=6
         chkgrd=40
         chkpsi=1
         chkprp=0
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         routine='Getinwf'
         if (lerr) then
            message='Error reading RESTART file'
            call errormsg
         endif

!-----------------------------------------------------------------------
! If only 1 restart fie read, check that A-coeff mapping was defined in 
! input. If not (file keyword), define now. Note that in this case the
! restart file needs the same no. of states as the new calculation as the
! mapping is a simple 1-to-1.
!-----------------------------------------------------------------------
         if ((nreadwf .eq. 1) .and. (amapwf(1,1,1) .eq. 0)) then        
             if (nstate .ne. cnstate(1)) then  
                message = &
           'Restart file has different no. of states from calculation'
                call errormsg
             else
                do s = 1,nstate
                   amapwf(s,1,1) = s
                   amapwf(s,2,1) = s
                   smapwf(s,1,1) = s
                   smapwf(s,2,1) = s
                 enddo
             endif
         endif

!-----------------------------------------------------------------------
! Allocate memory for psi read from restart file
!-----------------------------------------------------------------------
         cerr = 'getinwf_oldpsi   '
         allocate(oldpsi(dgldim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'getinwf_oldagmat '
         allocate(oldagmat(agmatdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'getinwf_olddicht1 '
         allocate(olddicht1(dmatdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'getinwf_olddicht2 '
         allocate(olddicht2(dmatdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'getinwf_olddicht3 '
         allocate(olddicht3(d3matdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'getinwf_olddicht4 '
         allocate(olddicht4(dmatdim),stat=ierr)
         if (ierr > 0) go to 1000

!-----------------------------------------------------------------------
! read old psi into oldpsi. tinit read from .inp file, not from restart.
!-----------------------------------------------------------------------
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkprp=0
         idum=1
         iflag=0
         call rdrst(told,oldpsi,ldum,chkdvr,chkgrd,chkpsi,chkprp,&
           cdum,idum,rdum,iflag,jindx,oldagmat,idum1)
         close(irst)
         call griddat
         call psidat

         if(lrealpsi .or. lorthopsi) then  ! transform to natural orbitals.
            call density(oldpsi,tree,olddicht1,olddicht2,olddicht3,olddicht4,&
                 jindx,cdum)
            call trafonat(oldpsi,tree,olddicht4,jindx)
         endif

         if(lrealpsi) then   ! take real-part of psi if desired
            call realpsi(oldpsi,olddicht3)
            call reorthopsi(oldpsi,tree,olddicht1,olddicht2,olddicht3,&
                 olddicht4,jindx,gs2)
         endif
 
!-----------------------------------------------------------------------
! To copy old coefficients to new wavefunction, it is neccesary to
! correct for any difference in the number of MC functions.
! s1 = from-state,  s2 = to-state
!-----------------------------------------------------------------------
         do j = 1, maxsta
            s1 = amapwf(j,1,nwf)  ! state in old psi
            s2 = amapwf(j,2,nwf)  ! state in new psi
            if (s1 .eq. 0) exit   ! end of mapping
      
            write(ilog,'(a,i3,a,i3,a,i3,a," (",f11.6,",",f11.6,")")')  &
         'A   ',s1,' ->',s2,',  chn:',irst,',  weight =',aweight(s2)
            if (s1.gt.nstate) then
               write(message,'(2(a,i3))') &
                  'Not enough states on old psi restart, s1 =',s1, &
                       ' nstate =', nstate
               call errormsg
            endif
            if (s2.gt.cnstate(1)) then
               write(message,'(2(a,i3))') &
                  'Not enough states on new psi restart, s2 =',s2, &
                       ' nstate =', cnstate(1)
               call errormsg
            endif

            if (dentype .eq. 2) then
               do k = 1, maxsta
                  t1 = amapwf(k,1,nwf)  ! state in old psi
                  t2 = amapwf(k,2,nwf)  ! state in new psi
                  if (t1 .eq. 0) exit   ! end of mapping

!-----------------------------------------------------------------------
! calculate maximum size of block and allocate tmppsi array
!-----------------------------------------------------------------------
                  mblock=1
                  do m=1,nmode
                     dim2=max(dim(m,t1),cdim(m,t2,1))
                     dim3=max(dim(m,s1),cdim(m,s2,1))
                     mblock=max(mblock,&
                       cvdim(m,t2,1)*dim2*ndim(m,t1)*cvdim(m,s2,1)*dim3*ndim(m,s1))
                  enddo

                  cerr = 'getpsi_tmppsi   '
                  allocate(tmppsi(2*mblock),stat=ierr)
                  if (ierr > 0) go to 1000

                  zeig = 1
                  zeig1 = mblock+1
                  if (dentype.eq.2) then
                     call cpvxz(oldpsi(dzpsi(t1,s1)),tmppsi(zeig),&
                          block(t1)*block(s1))
                  else if (dentype.eq.0) then
                     m=1
                     call akcopy_wf_to_d2(oldpsi(zpsi(t1)),oldpsi(zpsi(s1)),&
                          tmppsi(zeig),vdim(m,t1),dim(m,t1),ndim(m,t1),&
                          vdim(m,s1),dim(m,s1),ndim(m,s1))
                  else
                     routine='getpsi_mult'
                     message='This transformation is not possible.'
                     call errormsg
                  endif

                 do m=1,nmode
                     call akcopy2(tmppsi(zeig),tmppsi(zeig1),cvdim(m,t2,1),&
                          cdim(m,t2,1),dim(m,t1),ndim(m,t1),cvdim(m,s2,1),&
                          cdim(m,s2,1),dim(m,s1),ndim(m,s1))
                     zeig2=zeig
                     zeig=zeig1
                     zeig1=zeig2
                  enddo               !    m=1,nmode
                  call cpvxz(tmppsi(zeig),psi(cdzpsi(t2,s2,1)),&
                       cblock(t2,1)*cblock(s2,1))

                  if(abs(aweight(s2)-1.d0) .gt. 1.d-9) then
                     call xvxxzzo(aweight(s2),psi(cdzpsi(t2,s2,1)),  &
                          cblock(t2,1)*cblock(s2,1))
                  endif
                  deallocate(tmppsi)
               enddo

            else
!-----------------------------------------------------------------------
! calculate maximum size of block and allocate tmppsi array
!-----------------------------------------------------------------------
               mblock=1
               do m=1,nmode
                  dim2=max(dim(m,s1),cdim(m,s2,1))
                  mblock=max(mblock,cvdim(m,s2,1)*dim2*ndim(m,s1))
               enddo

               cerr = 'getpsi_tmppsi   '
               allocate(tmppsi(2*mblock),stat=ierr)
               if (ierr > 0) go to 1000

               zeig = 1
               zeig1 = mblock+1
               call cpvxz(oldpsi(zpsi(s1)),tmppsi(zeig),block(s1))
               
               do m=1,nmode
                  call akcopy(tmppsi(zeig),tmppsi(zeig1),cvdim(m,s2,1), &
                              cdim(m,s2,1),dim(m,s1),ndim(m,s1))
                  zeig2=zeig
                  zeig=zeig1
                  zeig1=zeig2
               enddo               !    m=1,nmode
               call cpvxz(tmppsi(zeig),psi(czpsi(s2,1)),cblock(s2,1))
               if(abs(aweight(s2)-1.d0) .gt. 1.d-9) then
                  call xvxxzzo(aweight(s2),psi(czpsi(s2,1)),  &
                       cblock(s2,1))
               endif
               deallocate(tmppsi)
            endif
         enddo                  !    j=1,nstate

!-----------------------------------------------------------------------
! Copy old SPFs to new wavefunction, s1 = from-state,  s2 = to-state
!-----------------------------------------------------------------------
         do j = 1,maxsta 
            s1 = smapwf(j,1,nwf)
            s2 = smapwf(j,2,nwf) 
            if(s1.eq.0) exit

            write(ilog,'(a,i3,a,i3,a,i3)') &
                 'SPF ',s1, ' ->',s2,',  chn:',irst
            if (s1.gt.nstate) then
               write(message,'(2(a,i3))') &
                  'Not enough states on old psi restart, s1 =',s1, &
                       ' nstate =', nstate
               call errormsg
            endif
            if (s2.gt.cnstate(1)) then
               write(message,'(2(a,i3))') &
                  'Not enough states on new psi restart, s2 =',s2, &
                       ' nstate =', cnstate(1)
               call errormsg
            endif

            shift = 0.0_dop
            do m=1,nmode
               if( cidmode(m,1) .ne. 0 ) cycle  ! Do nothing for identical modes

               if (gwpm(m)) then     ! GWP mode
                  call getgwps(m,s1,s2,oldagmat,agmat,oldpsi,psi,agmatdim,&
                       cagmatdim(1),dgldim,cdgldim(1))

               else                  ! normal SPFs
                  if (selectspf(1,m,s2).gt.0) then ! take selected SPFs
                     write(ilog,'(a,i2,a,i3,a,22i3)') &
                      ' selected SPFs: s=',s2,'  m=',m, &
                      '  SPFs =', (selectspf(js,m,s2),js=1,maxreadspf)
                     n=0
                     ! copy all SPFs that we have ...
                     do k=1,cdim(m,s2,1)
                        i=selectspf(k,m,s2)
                        if(i.eq.0) exit
                        n=n+1

                        is=(i-1)*subdim(m)
                        js=(k-1)*subdim(m)
                        call cpvxz(oldpsi(zetf(m,s1)+is), &
                                   psi(czetf(m,s2,1)+js),subdim(m))
                    enddo
                    if (n .lt. cdim(m,s,1)) then
                       ! ... and generate more
                       call inventspfs(m,psi(czetf(m,s,1)),dim(m,s1), &
                            cdim(m,s,1),subdim(m),ort,exphin,exprueck)
                    endif

                  else if (cdim(m,s2,1) .gt. dim(m,s1)) then
                     ! copy all SPFs that we have ...
                     call cpmxz(oldpsi(zetf(m,s1)),psi(czetf(m,s2,1)), &
                          subdim(m),dim(m,s1))
                     ! ... and generate more
                     call inventspfs(m,psi(czetf(m,s2,1)),dim(m,s1), &
                          cdim(m,s2,1),subdim(m),ort,exphin,exprueck)
                  else if (cdim(m,s2,1) .le. dim(m,s1)) then
                     ! copy as many SPFs as possible, discard the rest
                     call cpmxz(oldpsi(zetf(m,s1)),psi(czetf(m,s2,1)), &
                          subdim(m),cdim(m,s2,1))
                  endif         !    cdim .gt. dim
               endif
            enddo               !    m=1,nmode
         enddo                  !    j=1,nstate

!-----------------------------------------------------------------------
! Deallocate memory
!-----------------------------------------------------------------------
         if (allocated(oldpsi)) deallocate(oldpsi,stat=ierr)
         if (ierr > 0) go to 1001
         if (allocated(oldagmat)) deallocate(oldagmat,stat=ierr)
         if (ierr > 0) go to 1001
         if (allocated(olddicht1)) deallocate(olddicht1,stat=ierr)
         if (ierr > 0) go to 1001
         if (allocated(olddicht2)) deallocate(olddicht2,stat=ierr)
         if (ierr > 0) go to 1001
         if (allocated(olddicht3)) deallocate(olddicht3,stat=ierr)
         if (ierr > 0) go to 1001
         if (allocated(olddicht4)) deallocate(olddicht4,stat=ierr)
         if (ierr > 0) go to 1001
         if (allocated(tmppsi)) deallocate(tmppsi,stat=ierr)
         if (ierr > 0) go to 1001

      enddo  ! end of loop over restart files

!-----------------------------------------------------------------------
! restore data for psi
!-----------------------------------------------------------------------
      call getcsys(1)
      call getcpsi(1)
      lmult=lmult2
      leb=leb2
      lmulpack=lmulpack2
      npacket=npacket2
      filename=dname(1:dlaenge)//'/dvr'
      open(idvr,file=filename,form='unformatted',status='old',err=999)
      chkdvr=1    !  Read DVR info again, as overwritten by rdrst.
      call dvrinfo(ldum,chkdvr)
      close(idvr)
      call griddat
      call psidat

!-----------------------------------------------------------------------
! Schmidt-orthogonalize initial single-particle functions
!-----------------------------------------------------------------------
      if(lorthopsi) then
         do s=1,nstate
            do m=1,nmode
               if (idmode(m) .eq. 0) then
                  call schmidtortho(psi(zetf(m,s)),subdim(m),dim(m,s),ierr)
               endif
               if( ierr .gt. 0 ) then
                  routine='Getinwf (getpsi.F)'
                  write(message,'(a,i3,a,i3,a,i3)')&
                       'Cannot Schmidt-orthogonalize SPFs for s =',s,&
                       ', m =',m,', e =',ierr
                  call errormsg
               end if
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! Set initial WP to be a sum of HArtree products
!-----------------------------------------------------------------------
      if (lhartreepsi) then
         do s=1,nstate
            i=zpsi(s)+1
            j=zpsi(s)+block(s)-1
            psi(i:j) = 0.0_dop
         enddo

!-----------------------------------------------------------------------
! Normalise A-vector 
!-----------------------------------------------------------------------
      else if (psitype .eq. 4) then
         do s=1,nstate
            do m=1,nmode
               if (.not. gwpm(m)) cycle
               call diagonal_moments(m,s,s,nspfdof(m),gwpdim(m,s), &
                    gwpdim(m,s),agmat(zagmat(m,s)),agmat(zagmat(m,s)), &
                    agmat(zxivec(m,s)),agmat(zxivec(m,s)),agmat(zphase(m,s)),&
                    agmat(zphase(m,s)),goverlap(gs2mat(m,s)),ai,z,0,0)
            enddo
         enddo

         call renormgwpa(psi,goverlap)

      elseif(lmulpack) then
         do p=1,npacket
            s=(p-1)*(nstate/npacket) + 1
            call normvxz(psi(zpsi(s)),norm,adim/npacket) ! compute norm
            if(norm.le.1.d-6) then
               routine='Getinwf (getpsi.F)'
               write(message,'(a,i2,a,e11.4)')             &
                  'The norm of the initial WF (pack=',p,   &
                  ') is too small. norm =',norm
               call errormsg
            endif
            call xvixdzo(norm,psi(zpsi(s)),adim/npacket) ! normalise
            write(ilog,'(2a,i2,a,d18.10)') 'Norm of initial WF ',    &
               '(pack=',p,') before re-normalisation:', norm
         enddo

      elseif(lreflex) then
         do p=1,npackts*2
            pp = (p-1)*adim/(2*npackts) + 1
            call normvxz(psi(pp),norm,adim/(2*npackts)) ! compute norm
            if(norm.le.1.d-6) then
               routine='Getinwf (getpsi.F)'
               write(message,'(a,i2,a,e11.4)')                    &
                  'The norm of the initial WF (pack=',p,          &
                  ') is too small. norm =',norm
               call errormsg
            endif
            call xvixdzo(norm,psi(pp),adim/(2*npackts)) ! normalise
            write(ilog,'(2a,i2,a,d18.10)') 'Norm of initial WF ',  &
               '(pack=',p,') before re-normalisation:', norm
         enddo

      elseif(mpb.gt.0) then
         do p=1,npackts
            pp = (p-1)*adim/npackts + 1
            call normvxz(psi(pp),norm,adim/npackts) ! compute norm
            if(norm.le.1.d-6) then
               routine='Getinwf (getpsi.F)'
               write(message,'(a,i2,a,e11.4)')                &
                  'The norm of the initial WF (pack=',p,      &
                  ') is too small. norm =',norm
               call errormsg
            endif
            call xvixdzo(norm,psi(pp),adim/npackts) ! normalise
            write(ilog,'(2a,i2,a,d18.10)') 'Norm of initial WF ',   &
               '(pack=',p,') before re-normalisation:', norm
         enddo

      else
         call normvxz(psi,norm,adim) ! compute norm
         if(norm.le.1.d-6) then
            routine='Getinwf (getpsi.F)'
            write(message,'(2a,e11.4)') 'The norm of the initial ',   &
                                     'WF is too small. norm =',norm
            call errormsg
         endif
         call xvixdzo(norm,psi,adim) ! normalise

      endif

      return

  900 continue
      routine = 'Getinwf'
      message = 'Cannot open restart file: '//rname(1:rlaenge)
      call errormsg
  999 continue
      i=index(filename,' ')-1
      routine = 'Getinwf'
      message = 'Cannot open file: '//filename(1:i)
      call errormsg
 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      stop
 1001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
      stop

      end subroutine getpsi_mult

!#######################################################################

      subroutine getgwps(m,s1,s2,oldagmat,agmat,oldpsi,psi,agmatdim1,&
                 agmatdim2,dgldim1,dgldim2)

      implicit none

      integer(long), intent(in) :: m,s1,s2,agmatdim1,agmatdim2,dgldim1,dgldim2
      integer(long)             :: gdof,gwpdim2,dim2
      complex(dop), dimension(agmatdim1), intent(in)  :: oldagmat
      complex(dop), dimension(dgldim1), intent(in)    :: oldpsi
      complex(dop), dimension(agmatdim2), intent(out) :: agmat
      complex(dop), dimension(dgldim2), intent(out)   :: psi

      gdof=nspfdof(m)
      gwpdim2=min(gwpdim(m,s1),cgwpdim(m,s2,1))
      dim2=gwpdim2

! copy SPF
       if (psitype .eq. 0) then
          call cpmxz(oldpsi(zdgwp(m,s1)),psi(czdgwp(m,s2,1)),gwpdim2,dim2)
       endif

! copy agmat, xivec, phase
       call cpmxz(oldagmat(zagmat(m,s1)),agmat(czagmat(m,s2,1)),gdof*gdof,gwpdim2)
       call cpmxz(oldagmat(zxivec(m,s1)),agmat(czxivec(m,s2,1)),gdof,gwpdim2)
       call cpvxz(oldagmat(zphase(m,s1)),agmat(czphase(m,s2,1)),gwpdim2)

       if (lgwpphase) call cpvxz(oldpsi(zphi(m,s1)),psi(czphi(m,s2,1)),gwpdim2)
       call cpmxz(oldpsi(zxi(m,s1)),psi(czxi(m,s2,1)),gdof,gwpdim2)

! if old psi is thawed GWP
       if (gwptype .le.1 .and. cgwptype(1) .le. 1) then
          call cpmxz(oldpsi(zpg(m,s1)),psi(czpg(m,s2,1)),gdof*gdof,gwpdim2)

       else if (gwptype .le.1 .and. cgwptype(1) .gt. 1) then
          routine = 'Getpsi'
          message =  'GWP conversion not allowed'// &
              ' thawed <--> separable / frozen'
          call errormsg

! if old psi is separable
       else if (gwptype .eq. 2) then
          if (cgwptype(1) .le. 1) then
             call sep2gwp(oldpsi(zpg(m,s1)),psi(czpg(m,s2,1)),gdof,gwpdim2)
          else if (cgwptype(1) .eq. 2) then
             call cpmxz(oldpsi(zpg(m,s1)),psi(czpg(m,s2,1)),gdof,gwpdim2)
          endif

! if old psi is diagonal FG or uncorrelated vMCG
      else if (gwptype .eq. 3) then
         if (cgwptype(1) .le. 1) then
            call cpmxz(oldagmat(zagmat(m,s1)),psi(czpg(m,s2,1)),gdof*gdof,gwpdim2)
         else if (cgwptype(1) .eq. 2) then
            call gwp2sep(oldagmat(zagmat(m,s1)),psi(czpg(m,s2,1)),gdof,gwpdim2)
         else if (cgwptype(1) .eq. 3) then
            call cpmxz(oldagmat(zagmat(m,s1)),agmat(czagmat(m,s2,1)),gdof*gdof,&
                 gwpdim2)
         else if (cgwptype(1) .eq. 4) then
            routine = 'Getpsi'
            message =  &
               'Pointless to convert diagonal FG to'// &
               ' non-diagonal'
            call errormsg
         endif

! if old psi is non-diagonal FG
      else if (gwptype .eq. 4) then
         if (cgwptype(1) .le. 1) then
            call cpmxz(oldagmat(zagmat(m,s1)),psi(czpg(m,s2,1)),gdof*gdof,gwpdim2)
         else if (cgwptype(1) .eq. 2) then
            routine = 'Getpsi'
            message = &
               'Conversion from non-diagonal FG to'// &
               ' separable not possible'
            call errormsg
         else if (cgwptype(1) .eq. 3) then
            routine = 'Getpsi'
            message = &
               'Conversion from non-diagonal FG to'// &
               ' diagonal FG not possible'
            call errormsg
         endif

      endif

      if (cdim(m,s2,1) .gt. dim2) then
         if (dim(m,s1) .eq. 1 .and. ldd .and. cnstate(1) .gt. 1) then
            call ddhigher(psi(czxi(m,s2,1)),psi(czphi(m,s2,1)), &
                 gdof,cgwpdim(m,s2,1),s2,agmat(czagmat(m,s2,1)), &
                 agmat(czxivec(m,s2,1)),agmat(czphase(m,s2,1)))

         else
            routine = 'Getpsi'
            message = 'Cannot generate extra functions for GWPs '
            call errormsg
         endif
      endif

      return
      end subroutine getgwps

!#######################################################################
! GETPSI_SING
!
! called from runinwf  (file=.. keyword)
! calls:
! rdrst,inventspfs,griddat,psidat,density,schmidtortho,akcopy
!
! for a restart calculation, the initial wavefunction is read from the
! restart file. It is then possible to alter the number of functions 
! used in the MC expansion, and if required higher functions are 
! generated. Note, this is for the ordinary "file=" statement.
!
! psi: initial wavefunction
! ort: grid positions
!   
! If removing functions from a GWP calculation, this should be 
! improved as the WP should be projected onto new space to 
! ensure the least important GWPs are removed. (use GGPs??)
!#######################################################################

      subroutine getpsi_sing (psi,ort,jindx,agmat,exphin,exprueck)

      implicit none

      integer(long)    :: s1,s2,t1,t2,m,s,ilbl,mblock,dim2,&
                          zeig,zeig1,zeig2,ierr,chkdvr,&
                          chkgrd,chkpsi,chkprp,idum,iflag,&
                          p,pp,i,j,k,is,js,n,nwf,nblock,vdim1,dim1,ndim1

      integer(long), dimension(1)                     :: idum1
      integer(long), dimension(3)                     :: ijk
      integer(long), dimension(jindxdim), intent(out) :: jindx
      integer(long), dimension(maxdim)                :: nspf

      real(dop)                                       :: norm,told,norm1
      real(dop), dimension(1)                         :: rdum
      real(dop), dimension(ortdim), intent(in)        :: ort

      complex(dop)                                    :: weight
      complex(dop), dimension(1)                      :: cdum
      complex(dop), dimension(dgldim), intent(inout)  :: psi
      complex(dop), dimension(expdim), intent(in)     :: exphin,exprueck
      complex(dop), dimension(:), allocatable         :: tmppsi
      complex(dop), dimension(:), allocatable         :: oldpsi
      complex(dop), dimension(:), allocatable         :: olddicht1,olddicht2,&
                                                         olddicht4
      complex(dop), dimension(:), allocatable         :: oldgfgwp
      complex(dop), dimension(:), allocatable         :: oldaigwp
      complex(dop), dimension(:), allocatable         :: oldagmat
      complex(dop), dimension(:), allocatable         :: oldzgwp
      complex(dop), dimension(:), allocatable         :: oldgs2
      complex(dop), dimension(:), allocatable         :: oldgsinv
      complex(dop), dimension(:), allocatable         :: oldgsvek
      real(dop), dimension(:), allocatable            :: olddicht3
      real(dop), dimension(:), allocatable            :: oldgsval
      real(dop), dimension(:), allocatable            :: oldqcent
      integer(long), dimension(:), allocatable        :: oldjindx

      complex(dop), dimension(agmatdim), intent(out)  :: agmat
      complex(dop), dimension(gs2matdim)                   :: goverlap
      complex(dop), dimension(maxdof*maxspf*maxspf)        :: z
      complex(dop), dimension(maxdof*maxdof*maxspf*maxspf) :: ai

      logical(kind=4)                                 :: ldum,lerr,&
                                                         linwf
      character(len=c5)                               :: filename
      character(len=20)                               :: cerr
      type(wfnode_t),pointer      :: oldtree,newtree,node

      routine='Getpsi'

      if(lignore)  write(ilog,'(a)') '### file: ignore is set! ###'
      write(ilog,*)

      if (nreadwf .gt. 1) then
         routine='Getpsi'
         message='Can only read a single restart file to set up a single-set WF'
         call errormsg
      endif
      nwf = 1

      call zerovxz(psi,dgldim)
      oldtree => null()
      newtree => null()

!-----------------------------------------------------------------------
! save data for new psi
!-----------------------------------------------------------------------
      call psidat
      call savecsys(1)
      call savecpsi(1)
      if (psitype .eq. 5) newtree => tree

!-----------------------------------------------------------------------
! open old WF file
!-----------------------------------------------------------------------
      rname = readwffile(1)
      rlaenge = index(rname,' ')-1
      open(irst,file=rname,form='unformatted',status='old',err=999)

!-----------------------------------------------------------------------
! read information about psi on restart file. This must be done before
! reading the file, as the old wavefunction may be different from new.
!-----------------------------------------------------------------------
      chkdvr=6
      chkgrd=40
      chkpsi=1
      chkprp=0
      call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
      if (lerr) then
         routine='Getpsi'
         message='Error reading RESTART file'
         call errormsg
      endif
! now read old data
      rewind(irst)
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkprp=0
      call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)

!-----------------------------------------------------------------------
! save data for old psi
!-----------------------------------------------------------------------
      call psidat
      call savecsys(2)
      call savecpsi(2)
      if (psitype .eq. 5) oldtree => tree

!-----------------------------------------------------------------------
! Check that A-coeff mapping was defined in 
! input. If not (file keyword), define now. Note that in this case the
! restart file needs the same no. of states as the new calculation as the
! mapping is a simple 1-to-1.
!-----------------------------------------------------------------------
      if (amapwf(1,1,1) .eq. 0) then        
         if (gdim(feb) .ne. cgdim(feb,1)) then  
             message = &
           'Restart file has different no. of states from calculation'
             call errormsg
         endif
      endif

!-----------------------------------------------------------------------
! Allocate memory for psi read from restart file
!-----------------------------------------------------------------------
      cerr = 'getpsi_oldpsi   '
      allocate(oldpsi(dgldim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'getinwf_olddicht1 '
      allocate(olddicht1(dmatdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'getinwf_olddicht2 '
      allocate(olddicht2(dmatdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'getinwf_olddicht3 '
      allocate(olddicht3(d3matdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'getinwf_olddicht4 '
      allocate(olddicht4(dmatdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'getinwf_oldjindx '
      allocate(oldjindx(jindxdim),stat=ierr)
      if (ierr > 0) go to 1000
      if (lgwp) then
         cerr = 'getpsi_oldjindx '
         allocate(oldgs2(gs2matdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'getpsi_oldagmat '
         allocate(oldagmat(agmatdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'runprop_oldgfgwp  '
         allocate(oldgfgwp(gfgwpdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'runprop_oldaigwp  '
         allocate(oldaigwp(aigwpdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'runprop_oldzgwp  '
         allocate(oldzgwp(zgwpdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'runprop_gsvek '
         allocate(gsvek(gsmatdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'runprop_oldgsinv  '
         allocate(oldgsinv(gsmatdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'runprop_oldgsvek '
         allocate(oldgsvek(gsmatdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'runprop_oldgsval '
         allocate(oldgsval(gwpinfodim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'runprop_oldqcent '
         allocate(oldqcent(gwpinfodim),stat=ierr)
         if (ierr > 0) go to 1000
      endif

      if (psitype .eq. 5) then
         ! We need to allocate the old tree insofar as the eigenvectors
         ! of its density matrices will be needed.
         call allocate_work(oldtree)
         call allocate_wftree(oldtree,.true.,olddicht1,olddicht2,&
                 olddicht3,olddicht4)
      endif

!-----------------------------------------------------------------------
! read old psi into oldpsi. tinit read from .inp file, not from restart.
!-----------------------------------------------------------------------
      rewind(irst)
      chkdvr=0
      chkgrd=0
      chkpsi=0
      chkprp=0
      idum=1
      iflag=0
      call rdrst(told,oldpsi,ldum,chkdvr,chkgrd,chkpsi,chkprp,&
           cdum,idum,rdum,iflag,jindx,oldagmat,idum1)
      if (equalfile .eq. 0) close(irst)

!-----------------------------------------------------------------------
! Check WF types compatible
!-----------------------------------------------------------------------
      if (psitype .eq. 0 .and. cpsitype(1) .eq. 5) then     
         nspf(:)=dim(:,1)
         call mctdh2ml(psi,newtree,oldpsi,nspf)
         psitype=5
      else if (psitype .eq. 5 .and. cpsitype(1) .eq. 0) then     
         routine='Getpsi'
         message= &
            'Old WF ML-MCTDH, New WF MCTDH. First convert using ml2mctdh'
         call errormsg
      endif

!-----------------------------------------------------------------------
!    transform to natural orbitals
!-----------------------------------------------------------------------
      if (lorthopsi) then  ! transform to natural orbitals.
                           ! ML transformed in resizepsi
         if (lgwp) then
            call gwp_param(oldpsi,oldagmat,oldgfgwp,oldaigwp,oldzgwp,&
                 oldqcent,2)
            call gwp_overlap(oldgs2,oldgsinv,oldgsval, &
                 oldgsvek,oldgfgwp,oldaigwp,oldzgwp,0)
         endif

         call density(oldpsi,oldtree,olddicht1,olddicht2,olddicht3,olddicht4,&
              oldjindx,oldgs2)
         call trafonat(oldpsi,oldtree,olddicht4,oldjindx)
      endif

!-----------------------------------------------------------------------
!    take real-part of psi if desired
!-----------------------------------------------------------------------
      if (lrealpsi) then   
         call realpsi(oldpsi,olddicht3)
         call reorthopsi(oldpsi,oldtree,olddicht1,olddicht2,olddicht3,&
              olddicht4,jindx,gs2)
      endif

!-----------------------------------------------------------------------
!    copy single particle functions to relevant part of psi array, 
!    discarding or generating higher phi functions as required
!-----------------------------------------------------------------------
      if (psitype .eq. 5) then    ! ML-form: truncate or extend
         call resize_tree(oldtree,oldpsi,newtree,psi)

      else if (psitype .eq. 0 .or. psitype .eq. 4) then
         ! MCTDH or G-MCTDH forms
         s=1
         do m=1,nmode
            if(cidmode(m,1) .ne. 0) cycle ! Do nothing for identical modes

            if (gwpm(m)) then     ! GWP mode
               call getgwps(m,s,s,oldagmat,agmat,oldpsi,psi,agmatdim,&
                    cagmatdim(1),dgldim,cdgldim(1))

            else                  ! normal SPFs
               if (selectspf(1,m,s).gt.0) then ! take selected SPFs
                  write(ilog,'(a,i2,a,i3,a,22i3)') &
                   ' selected SPFs: s=',s,'  m=',m, &
                   '  SPFs =', (selectspf(js,m,s),js=1,maxreadspf)
                  n=0
                  ! copy all SPFs that we have ...
                  do j=1,cdim(m,s,1)
                     i=selectspf(j,m,s)
                     if(i.eq.0) exit
                     n=n+1

                     is=(i-1)*subdim(m)
                     js=(j-1)*subdim(m)
                     call cpvxz(oldpsi(zetf(m,s)+is), &
                                psi(czetf(m,s,1)+js),subdim(m))
                 enddo
                 if (n .lt. cdim(m,s,1)) then
                    ! ... and generate more
                    call inventspfs(m,psi(czetf(m,s,1)),dim(m,s), &
                         cdim(m,s,1),subdim(m),ort,exphin,exprueck)
                 endif

               else if (cdim(m,s,1) .gt. dim(m,s)) then
                  ! copy all SPFs that we have ...
                  call cpmxz(oldpsi(zetf(m,s)),psi(czetf(m,s,1)), &
                             subdim(m),dim(m,s))
                  ! ... and generate more
                  call inventspfs(m,psi(czetf(m,s,1)),dim(m,s), &
                       cdim(m,s,1),subdim(m),ort,exphin,exprueck)
                  ! TODO

               else if (cdim(m,s,1) .le. dim(m,s)) then
                     ! copy as many SPFs as possible, discard the rest
                     call cpmxz(oldpsi(zetf(m,s)),psi(czetf(m,s,1)), &
                                subdim(m),cdim(m,s,1))
               endif     ! cdim .le. dim
            endif     
         enddo        ! m=1,nmode

!-----------------------------------------------------------------------
!    To copy old coefficients to new wavefunction, it is neccesary to
!    correct for any difference in the number of SPFs.

!    calculate maximum size of block
!-----------------------------------------------------------------------
         if (dentype .eq. 2) then
            mblock=1
             do m=1,nmode
                dim2=max(dim(m,1),cdim(m,1,1))
                mblock=max(mblock,&
                  cvdim(m,1,1)*dim2*ndim(m,1)*cvdim(m,1,1)*dim2*ndim(m,1))
             enddo

             cerr = 'getpsi_tmppsi   '
             allocate(tmppsi(2*mblock),stat=ierr)
             if (ierr > 0) go to 1000

             zeig = 1
             zeig1 = mblock+1
             if (dentype.eq.2) then
                call cpvxz(oldpsi(zpsi(1)),tmppsi(zeig),block(1)*block(1))
             else if (dentype.eq.0) then
                m=1
                call akcopy_wf_to_d2(oldpsi(zpsi(1)),oldpsi(zpsi(1)),&
                     tmppsi(zeig),vdim(m,1),dim(m,1),ndim(m,1),&
                     vdim(m,1),dim(m,1),ndim(m,1))
             else
                routine='getpsi_sing'
                message='This transformation is not possible.'
                call errormsg
             endif

            do m=1,nmode
                call akcopy2(tmppsi(zeig),tmppsi(zeig1),cvdim(m,1,1),&
                     cdim(m,1,1),dim(m,1),ndim(m,1),cvdim(m,1,1),&
                     cdim(m,1,1),dim(m,1),ndim(m,1))
                zeig2=zeig
                zeig=zeig1
                zeig1=zeig2
             enddo               
             call cpvxz(tmppsi(zeig),psi(czpsi(1,1)),cblock(1,1)*cblock(1,1))

             deallocate(tmppsi)

         else
            mblock=1
            do m=1,nmode
               dim2=max(dim(m,1),cdim(m,1,1))
               mblock=max(mblock,cvdim(m,1,1)*dim2*ndim(m,1))
            enddo
            cerr = 'getpsi_tmppsi   '
            allocate(tmppsi(2*mblock),stat=ierr)
            if (ierr > 0) go to 1000

            zeig=1
            zeig1=zeig+mblock
            tmppsi(:) = 0.0_dop
            call cpvxz(oldpsi(zpsi(1)),tmppsi(zeig),block(1))

            do m=1,nmode
               call akcopy(tmppsi(zeig),tmppsi(zeig1),cvdim(m,s,1),cdim(m,1,1), &
                 dim(m,1),ndim(m,1))
               zeig2=zeig
               zeig=zeig1
               zeig1=zeig2
            enddo
            call cpvxz(tmppsi(zeig),psi(czpsi(1,1)),cblock(1,1))

            deallocate(tmppsi)
         endif

      else
         routine = 'getpsi_sing'
         write(message,'(a,i3)') 'Incorrect initial WF type : ', psitype
         call errormsg

      endif

!-----------------------------------------------------------------------
! restore data for new psi
!-----------------------------------------------------------------------
      call getcsys(1)
      call getcpsi(1)
      call griddat
      call psidat
      if (psitype .eq. 5) tree => newtree

!-----------------------------------------------------------------------
! Map state coefficients if required
!-----------------------------------------------------------------------
      if (amapwf(1,1,nwf) .gt. 0) then

!-----------------------------------------------------------------------
! allocate tmppsi
!-----------------------------------------------------------------------
         cerr = 'getpsi_tmppsi   '
         allocate(tmppsi(2*adim),stat=ierr)
         if (ierr > 0) go to 1000

!-----------------------------------------------------------------------
! Get tensor dimensions for A-vector for meb
!-----------------------------------------------------------------------
         if (psitype .eq. 0 .or. psitype .eq. 4) then
            vdim1=vdim(meb,1)
            dim1=dim(meb,1)
            ndim1=ndim(meb,1)

         else if (psitype .eq. 5) then
            do m=1,tree%nmodes               ! loop over branches of top layer
               node => tree%submodes(m)%p    ! node numder 
               if (node%isprimitive .and. node%m .eq. meb) then  ! is node meb?
                  ijk=ijk_shape(m,newtree%ndim)
                  vdim1=ijk(1)
                  dim1=ijk(2)
                  ndim1=ijk(3)
               endif
            enddo
         endif

!-----------------------------------------------------------------------
! copy psi to tmppsi, transforming to state basis
!-----------------------------------------------------------------------
         call qtxxzz(psi(zetf(meb,1)),psi(1),tmppsi,vdim1,dim1,ndim1)
         psi(1:adim) = 0.0_dop

!-----------------------------------------------------------------------
! Now map states
!
! s is state of new psi
! s1 is state of old psi
!-----------------------------------------------------------------------
         do j=1,gdim(feb)
            s1 = amapwf(j,1,nwf)  ! state in old psi
            s2 = amapwf(j,2,nwf)  ! state in new psi
            if (s1 .eq. 0) exit   ! end of mapping

            write(ilog,'(a,i3,a,i3,a,i3,a," (",f11.6,",",f11.6,")")')  &
               'A   ',s1,' ->',s2,',  chn:',irst,',  weight =',aweight(s2)
            if (s1.gt.gdim(feb)) then
               write(message,'(2(a,i3))') &
                  'Not enough states on old psi restart, s1 =',s1, &
                       ' nstate =', nstate
               call errormsg
            endif
            if (s2.gt.cgdim(feb,1)) then
               write(message,'(2(a,i3))') &
                  'Not enough states on new psi restart, s2 =',s2, &
                       ' nstate =', cnstate(1)
               call errormsg
            endif

!-----------------------------------------------------------------------
! Map coefficients. Extract A-vector for s1 and copy to s2
!-----------------------------------------------------------------------
            if (dentype .eq. 0) then 
               zeig = adim+1
               nblock=adim
               if (meb .gt. 0) nblock=nblock/dim1

               call cuttxz2(tmppsi(1),tmppsi(zeig),vdim1,dim1,ndim1,s1)
               weight = aweight(s2)
               if(abs(weight-1.d0) .gt. 1.d-9) then
                  call xvxxzzo(weight,tmppsi(zeig),nblock)
               endif
               call cuttxz3(tmppsi(zeig),psi(zpsi(1)),vdim1,dim1,ndim1,s2)

            else if (dentype .eq. 1) then 
               do k=1,gdim(feb)
                  t1 = amapwf(k,1,nwf)  ! left state in old psi
                  t2 = amapwf(k,2,nwf)  ! left state in new psi
                  if (t1 .eq. 0) exit   ! end of mapping
               
                  if ( .not.(lrstcoh) .and. (s2 .ne. t2)) cycle

                  zeig = adim+1
                  nblock=adim
                  if (meb .gt. 0) nblock=nblock/dim1

                  call cuttens1(tmppsi(1),tmppsi(zeig),t1,s1,vdim1,gdim(feb),ndim1)
                  weight = aweight(s2)*aweight(t2)
                  if(abs(weight-1.d0) .gt. 1.d-9) then
                     call xvxxzzo(weight,tmppsi(zeig),nblock)
                  endif
                  call filltens1(tmppsi(zeig),psi(zpsi(1)),t2,s2,vdim1,gdim(feb),&
                       ndim1)
                  if (t2 .ne. s2) then
                     call filltens1c(tmppsi(zeig),psi(zpsi(1)),s2,t2,vdim1,&
                          gdim(feb),ndim1)
                     endif
               enddo

            else if (dentype .eq. 2) then 
               do k=1,gdim(feb)
                  t1 = amapwf(k,1,nwf)  ! left state in old psi
                  t2 = amapwf(k,2,nwf)  ! left state in new psi
                  if (t1 .eq. 0) exit   ! end of mapping
             
                  if ( .not.(lrstcoh) .and. (s2 .ne. t2)) cycle
               
                  zeig = adim+1
                  nblock=adim
                  if (meb .gt. 0) nblock=nblock/dim1

                  call cuttens2(tmppsi(1),tmppsi(zeig),t1,s1,vdim1,dim1,ndim1)
                  weight = aweight(s2)*aweight(t2)
                  if(abs(weight-1.d0) .gt. 1.d-9) then
                     call xvxxzzo(weight,tmppsi(zeig),nblock)
                  endif
                  call filltens2(tmppsi(zeig),psi(zpsi(1)),t2,s2,vdim1,dim1,&
                       ndim1)
                  if (t2 .ne. s2) then
                     call filltens2c(tmppsi(zeig),psi(zpsi(1)),s2,t2,vdim1,dim1,&
                          ndim1)
                  endif
               enddo

            endif

         enddo

!-----------------------------------------------------------------------
! transform psi back to spf basis
!-----------------------------------------------------------------------
         tmppsi(1:adim) = psi(1:adim)
         call qtaxzz(psi(zetf(meb,1)),tmppsi,psi(1),vdim1,dim1,ndim1)

         deallocate(tmppsi)

      endif ! end of mapping

!-----------------------------------------------------------------------
! Schmidt-orthogonalize SPFs of initial wave function if desired
!-----------------------------------------------------------------------
      if(lorthopsi) then  
         call reorthopsi(psi,tree,dicht1,dicht2,dicht3,dicht4,jindx,gs2)
      endif

!-----------------------------------------------------------------------
! Set initial WP as sum of Hartree products
!-----------------------------------------------------------------------
      if (lhartreepsi) then
         call sethp_sing(psi,vdim(meb,1),dim(meb,1),ndim(meb,1)) 

!-----------------------------------------------------------------------
! Normalize A-vectors for G-MCTDH
!-----------------------------------------------------------------------
      else if (psitype .eq. 4) then
         s=1
         do m=1,nmode
            if (.not. gwpm(m)) cycle
            call diagonal_moments(m,s,s,nspfdof(m),gwpdim(m,s), &
                 gwpdim(m,s),agmat(zagmat(m,s)),agmat(zagmat(m,s)), &
                 agmat(zxivec(m,s)),agmat(zxivec(m,s)),agmat(zphase(m,s)),&
                 agmat(zphase(m,s)),goverlap(gs2mat(m,s)),ai,z,0,0)
         enddo
         
         call renormgwpa(psi,goverlap)

      else if (lmulpack) then
         routine='Getpsi_sing (getpsimod.f90)'
         message = 'Mulpack not yet implemented for Single-set'
         call errormsg

!-----------------------------------------------------------------------
! Normalize A-vectors in case of reflex
!-----------------------------------------------------------------------
      else if(lreflex) then
         do p=1,npackts*2
            pp = (p-1)*adim/(2*npackts) + 1
            call normvxz(psi(pp),norm,adim/(2*npackts)) ! compute norm
            if(norm.le.1.d-6) then
               routine='Getpsi (getpsi.F)'
               write(message,'(a,i2,a,e11.4)')&
                  'The norm of the initial WF (pack=',p,&
                  ') is too small. norm =',norm
               call errormsg
            endif
            call xvixdzo(norm,psi(pp),adim/(2*npackts)) ! normalise
            write(ilog,'(2a,i2,a,f14.9)') 'Norm of initial WF ',&
               '(pack=',p,') before re-normalisation:', norm
         enddo

!-----------------------------------------------------------------------
! normalise new wavefunction to be same as old
!-----------------------------------------------------------------------
      else if (.not. lrstnorm) then
         call normvxz(oldpsi(1),norm,cadim(2))
         call normvxz(psi(1),norm1,adim)

         write(ilog,'(a,f14.9,",    squared:",f14.9)') &
            'Retaining Old norm: ',norm, norm**2
         if (abs(norm-0.0_dop) .gt. tiny) then
            norm1=norm1/norm
            call xvixdzo(norm1,psi(1),adim)
         else
            routine='Getpsi_sing (getpsi.F)'
            write(message,'(2a,e11.4)') 'The norm of the initial ',   &
                                     'WF is too small. norm =',norm
            call errormsg
         endif

!-----------------------------------------------------------------------
! normalise new wavefunction 
!-----------------------------------------------------------------------
      else
         call normvxz(psi(1),norm,adim)
         if(norm.le.1.d-6) then
            routine='Getpsi_sing (getpsi.F)'
            write(message,'(2a,e11.4)') 'The norm of the initial ',   &
                                     'WF is too small. norm =',norm
            call errormsg
         endif

         call xvixdzo(norm,psi(1),adim)

      endif

      cerr = 'getpsi_oldpsi   '
      if (allocated(oldpsi)) deallocate(oldpsi,stat=ierr)
      if (ierr > 0) go to 1001
      cerr = 'getpsi_oldagmat '
      if (allocated(oldagmat)) deallocate(oldagmat,stat=ierr)
      if (ierr > 0) go to 1001
      cerr = 'getpsi_olddicht1'
      if (allocated(olddicht1)) deallocate(olddicht1,stat=ierr)
      if (ierr > 0) go to 1001
      cerr = 'getpsi_olddicht2'
      if (allocated(olddicht2)) deallocate(olddicht2,stat=ierr)
      if (ierr > 0) go to 1001
      cerr = 'getpsi_olddicht3'
      if (allocated(olddicht3)) deallocate(olddicht3,stat=ierr)
      if (ierr > 0) go to 1001
      cerr = 'getpsi_olddicht4'
      if (allocated(olddicht4)) deallocate(olddicht4,stat=ierr)
      if (ierr > 0) go to 1001
      cerr = 'getpsi_oldjindx'
      if (allocated(oldjindx)) deallocate(oldjindx,stat=ierr)
      if (ierr > 0) go to 1001

      if (lgwp) then
         cerr = 'getpsi_oldgfgwp'
         if (allocated(oldgfgwp)) deallocate(oldgfgwp,stat=ierr)
         if (ierr > 0) go to 1001
         cerr = 'getpsi_oldaigwp'
         if (allocated(oldaigwp)) deallocate(oldaigwp,stat=ierr)
         if (ierr > 0) go to 1001
         cerr = 'getpsi_oldzgwp'
         if (allocated(oldzgwp)) deallocate(oldzgwp,stat=ierr)
         if (ierr > 0) go to 1001
         cerr = 'getpsi_oldgs2'
         if (allocated(oldgs2)) deallocate(oldgs2,stat=ierr)
         if (ierr > 0) go to 1001
         cerr = 'getpsi_oldgsinv'
         if (allocated(oldgsinv)) deallocate(oldgsinv,stat=ierr)
         if (ierr > 0) go to 1001
         cerr = 'getpsi_oldgsvek'
         if (allocated(oldgsvek)) deallocate(oldgsvek,stat=ierr)
         if (ierr > 0) go to 1001
         cerr = 'getpsi_oldgsval'
         if (allocated(oldgsval)) deallocate(oldgsval,stat=ierr)
         if (ierr > 0) go to 1001
         cerr = 'getpsi_oldqcent'
         if (allocated(oldqcent)) deallocate(oldqcent,stat=ierr)
         if (ierr > 0) go to 1001
      endif

      if (cpsitype(2) .eq. 5) call dispose_wftree(oldtree)

      return
  999 continue
      ilbl=index(filename,' ')-1
      routine = 'Getpsi'
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      stop
 1001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
      stop

      end subroutine getpsi_sing
      
!#######################################################################

      subroutine sethp_sing(psi,vdim1,dim1,ndim1)

      implicit none

      integer(long), intent(in) :: vdim1,dim1,ndim1
      complex(dop), dimension(vdim1,dim1,ndim1),intent(out) :: psi

      psi(1,:,2:ndim1) = 0.0d0
      psi(2:vdim1,:,1:ndim1) = 0.0d0

      return
      end subroutine sethp_sing

!#######################################################################

    subroutine resizeml(oldtree,oldpsi,newtree,old_dgldim,psi,&
               new_dgldim)

       implicit none

       integer,intent(in)          :: old_dgldim,new_dgldim
       complex(dop),intent(inout)  :: oldpsi(old_dgldim)
       complex(dop),intent(out)    :: psi(new_dgldim)
       type(wfnode_t),intent(inout):: oldtree
       type(wfnode_t),intent(in)   :: newtree
       type(wfnode_t),pointer      :: oldnode
       integer                     :: m
       ! We need to allocate the old tree insofar as the eigenvectors
       ! of its density matrices will be needed.
       call allocate_work(oldtree)
       do m=1,oldtree%tot_nodes
          oldnode => oldtree%swap_up(m)%p
          call allocate_wfnode(oldnode,.true.,dicht1,dicht2,dicht3,dicht4)
       enddo

       call resize_tree(oldtree,oldpsi,newtree,psi)
       call dispose_wftree(oldtree)
    end subroutine resizeml

!#######################################################################
!-----------------------------------------------------------------------
subroutine resize_tree(oldtree,oldpsi,tree,psi)
!-----------------------------------------------------------------------
   type(wfnode_t),intent(inout) :: oldtree
   complex(dop),intent(inout)   :: oldpsi(:)
   type(wfnode_t),intent(in)    :: tree
   complex(dop),intent(out)     :: psi(:)
   integer                      :: match
   logical                      :: truncated,enlarged

   match = compare_wftrees(oldtree,tree)
   if (match==0) then
      call stop_code("wftrafo_m","resize_tree",&
         "trees have different structure")

   elseif (match==2) then
      psi = oldpsi

   else
      ! Do all necessary truncations on the old tree.
      truncated = truncate_tree(oldtree,oldpsi,tree)

      ! Next, copy the avecs to the new tree.
      call copy_avecs_tree(oldtree,oldpsi,tree,psi)

      ! Next, create additional new SPFs, where necessary.
      enlarged = enlarge_tree(oldtree,tree,psi)

   endif

end subroutine resize_tree

!#######################################################################
!-----------------------------------------------------------------------
function enlarge_tree(tree,tree1,psi1) result(enlarged)
!-----------------------------------------------------------------------

   type(wfnode_t),intent(in)   :: tree,tree1
   complex(dop),intent(inout)  :: psi1(:)
   logical                     :: enlarged
   type(wfnode_t),pointer      :: node,node1
   integer                     :: m,nspf,nspf1,sdm1,i,z0,zf,next
   enlarged = .false.
   do m=1,tree%tot_nodes
      node => tree%swap_up(m)%p
      node1 => tree1%swap_up(m)%p
      nspf = node%navecs
      nspf1 = node1%navecs
      sdm1 = product(node1%ndim)
      if (nspf1 <= nspf) cycle    ! top node cycles here
      ! Create SPFs nspf+1 ... nspf1.
      if (node%isprimitive) then
         ! For primitive modes, reuse the routine from standard
         ! MCTDH that creates higher SPFs.
         z0 = node1%z(1,1)
         zf = node1%z(nspf1,2)
         if (dentype .eq. 1) then
            next = nspf+1
            call mgenhigher(psi1(z0:zf),sdm1,nspf1,next,node1%m,msymmtr)
         else
            call inventspfs(node%m,psi1(z0:zf),nspf,nspf1,sdm1,mr(mrort),&
                 mc(mcexphin),mc(mcexprueck))
         endif
      else
         ! For higher modes, do something similar to set_hartree_avec.
         do i=nspf+1,nspf1
            z0 = node1%z(i,1)
            zf = node1%z(i,2)
            psi1(z0:zf) = CONE
            psi1(z0+i-nspf) = -CONE
         enddo
         ! These new SPFs are neither orthogonal nor normalized, but
         ! as they are unoccupied, this is of no immediate concern.
         ! However, one has to orthonormalize afterwards.
      endif
      enlarged = .true.
   enddo
end function enlarge_tree

!#######################################################################

!-----------------------------------------------------------------------
function truncate_tree(tree,psi,tree1) result(truncated)
!-----------------------------------------------------------------------
! For each node in tree, truncates the avecs according to the
! corresponding node of tree1, i.e. if any children of tree1's node
! have less SPFs than the corresponding children of tree's node,
! then zero the extraneous elements of tree's node's avecs.
! Returns .true. if any such truncation was performed.
!-----------------------------------------------------------------------
   type(wfnode_t),intent(in)   :: tree,tree1
   complex(cdbl),intent(inout) :: psi(:)
   logical                     :: truncated
   type(wfnode_t),pointer      :: node,node1
   integer                     :: mm,m,i,z0,zf
   integer                     :: ijk(3)
   truncated = .false.
   do mm=1,tree%tot_nodes
      node => tree%swap_up(mm)%p
      node1 => tree1%swap_up(mm)%p
      do m=1,node%nmodes
         if (node1%ndim(m) >= node%ndim(m)) cycle
         ijk = ijk_shape(m,node%ndim)
         do i=1,node%navecs
            z0 = node%z(i,1)
            zf = node%z(i,2)
            call truncate_avec(psi(z0:zf),ijk(1),ijk(2),ijk(3),node1%ndim(m))
         enddo
         truncated = .true.
      enddo
   enddo
end function truncate_tree

!-----------------------------------------------------------------------

   subroutine mctdh2ml(mlpsi,tree,psi,nspf)

      implicit none
      complex(dop),intent(out)  :: mlpsi(:)
      complex(dop),intent(in)   :: psi(:)
      integer,intent(in)        :: nspf(:)
      type(wfnode_t),intent(in) :: tree

      type(wfnode_t),pointer    :: tree1,node,child
      complex(dop),allocatable  :: mlpsi1(:)
      integer,allocatable       :: navecs(:)
      integer                   :: n,m,num,plen,c,adim,zphi,z0,zf
      integer                   :: dgldim1
      integer                   :: num1,num2,minavecs
      type(ctensor_t)           :: avec
      type(htensor_t)           :: ht
      type(tree_t),pointer      :: baretree
      real(dop)                 :: norm,sqerr


      ! Compute the norm.
      adim = product(nspf)
      norm = sqrt(dble(dot_product(psi(1:adim),psi(1:adim))))

      ! How many avecs per node? The system tree (based on user's input)
      ! specifies the desired number for each node. But the given MCTDH
      ! wavefunction may not be big enough to support this many avecs...
      allocate(navecs(tree%tot_nodes))
      do n=1,tree%tot_nodes
         node => tree%swap_postorder(n)%p
         if (node%isprimitive) then
            ! For primitive modes, use as many avecs as there are
            ! SPFs in the MCTDH WF.
            m = node%m
            num = node%num
            navecs(num) = nspf(m)
         else
            ! For other modes, use the maximum number of avecs that
            ! the underlying layer supports, or the number requested
            ! by the user, whichever is smaller.
            plen = 1
            do c=1,node%nmodes
               child => node%submodes(c)%p
               num = child%num
               plen = plen * navecs(num)
            enddo
            num = node%num
            navecs(num) = min(plen,node%navecs)
         endif
      enddo
      ! If the top node has only two children, then these two children must
      ! use the same number of avecs.
      node => tree%topnode
      if (node%nmodes==2) then
         num1 = node%submodes(1)%p%num
         num2 = node%submodes(2)%p%num
         minavecs = min( navecs(num1), navecs(num2) )
         navecs(num1) = minavecs
         navecs(num2) = minavecs
      endif

      ! Hierarchical SVD of the A-vector.
      write(ilog,'(a)') & 
          "Conversion MCTDH -> ML-MCTDH wavefunction via truncated hierarchical SVD"
      baretree => tree_from_wftree(tree)
      call ctensor_from1d(avec,psi(1:adim))
      call reshape_ctensor(avec,nspf)
      sqerr = 0.d0
      call hiersvd_ltr(ht,avec,baretree,sqerr,navecs)
      write(ilog,'(a,es10.3)') &
         "Estimated Truncation Error <= ",sqrt(sqerr)

      ! Make a new, empty tree.
      tree1 => wftree_from_tree(baretree)
      ! Set information about primitive modes (from system data).
      call wftree_init_pmodes(tree1,if_gdim,if_dofnums)
      ! Set the number of avecs for all nodes.
      call wftree_set_navecs(tree1,ht%nbasis)
      ! Compute pointers and allocate storage for new WF.
      call wftree_set_pointers(tree1,dgldim1)
      call alloc_dmat_wftree(tree1,full=.true.) ! needed for resize_tree
      allocate(mlpsi1(dgldim1))

      ! Copy the SPFs into this tree.
      zphi = adim+1
      do n=1,tree1%tot_nodes
         node => tree1%swap_postorder(n)%p
         z0 = node%z(1,1)
         zf = node%z(node%navecs,2)
         if (node%isprimitive) then
            ! Primitive mode SPFs are somewhere in the MCTDH psi.
            plen = node%navecs * product(node%ndim)
            mlpsi1(z0:zf) = psi(zphi:zphi+plen-1)
            zphi = zphi + plen
         else
            ! Higher-mode SPFs are in the transfer tensors of the
            ! HT decomposition of the A-vector.
            num = node%num
            mlpsi1(z0:zf) = ht%tt(num)%p%data(:)
         endif
      enddo

      ! Because of the truncation during the HierSVD the norm
      ! of the new ML-WF might have become smaller. Fix that.
      call renorm_tree(tree1,mlpsi1,norm)

      ! Now resize this tree into the full system tree.
      call resize_tree(tree1,mlpsi1,tree,mlpsi)

      ! Clean up.
      deallocate(mlpsi1)
      call dispose_wftree(tree1)
      call dispose_htensor(ht)
      call dispose_tree(baretree)
      deallocate(navecs)
   end subroutine mctdh2ml

!#######################################################################

      subroutine getex (psi,jindx)

      implicit none

      integer(long)       :: ilbl,chkdvr,chkgrd,chkpsi,chkprp,idum,iflag
      integer(long)       :: ierr,s,s1,s2
      integer(long)       :: nwf

      integer(long), dimension(1)                    :: idum1
      integer(long), dimension(jindxdim), intent(out):: jindx
      real(dop)                                      :: t1,norm
      real(dop), dimension(1)                        :: rdum
      complex(dop), dimension(1)                     :: cdum,cdum1
      complex(dop), dimension(dgldim), intent(out)   :: psi
      complex(dop), dimension(:), allocatable        :: oldpsi,tmppsi
      logical(kind=4)                                :: ldum,lerr,linwf
      character(len=c5)                              :: filename
      character(len=16)                              :: cerr

      routine='Getex'

!-----------------------------------------------------------------------
! save structure-data of psi, because rdrst will overwrite
!-----------------------------------------------------------------------
      call psidat
      call savecsys(1)
      call savecpsi(1)

!-----------------------------------------------------------------------
! Do-Loop over all restart files defined in read-inwf block
!-----------------------------------------------------------------------
      do nwf = 1, nreadwf

!-----------------------------------------------------------------------
! open old WF file
!-----------------------------------------------------------------------
         rname = readwffile(nwf)
         rlaenge = index(rname,' ')-1
         open(irst,file=rname,form='unformatted',status='old',err=900)
         write(ilog,'(2a)')&
             'Restart file read from :  ',rname(1:rlaenge)//'/restart'

!-----------------------------------------------------------------------
! read information about psi on restart file. This must be done before
! reading the file, as the old wavefunction may be different from new.
!-----------------------------------------------------------------------
         chkdvr=6
         chkgrd=40
         chkpsi=1
         chkprp=0
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         routine='Getinwf'
         if (lerr) then
            message='Error reading RESTART file'
            call errormsg
         endif

!-----------------------------------------------------------------------
! If only 1 restart fie read, check that A-coeff mapping was defined in
! input. If not (file keyword), define now. Note that in this case the
! restart file needs the same no. of states as the new calculation as the
! mapping is a simple 1-to-1.
!
! Else if amap has been set, but not smap, then transfer amap info 
!-----------------------------------------------------------------------
         if ((nreadwf .eq. 1) .and. (amapwf(1,1,1) .eq. 0)) then
             if (nstate .ne. cnstate(1)) then
                message = &
           'Restart file has different no. of states from calculation'
                call errormsg
             else
                do s = 1,nstate
                   amapwf(s,1,1) = s
                   amapwf(s,2,1) = s
                   smapwf(s,1,1) = s
                   smapwf(s,2,1) = s
                 enddo
             endif
         else if ((nreadwf .eq. 1) .and. (smapwf(1,1,1) .eq. 0)) then
             do s = 1,nstate
                smapwf(s,1,1) = amapwf(s,1,1)
                smapwf(s,2,1) = amapwf(s,2,1)
             enddo
         endif

!-----------------------------------------------------------------------
! Allocate memory for psi read from restart file
!-----------------------------------------------------------------------
         cerr = 'getinwf_oldpsi  '
         allocate(tmppsi(dgldim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'getinwf_tmppsi  '
         allocate(oldpsi(cdgldim(1)),stat=ierr)
         if (ierr > 0) go to 1000

!-----------------------------------------------------------------------
! read old psi into oldpsi. tinit read from .inp file, not from restart.
!-----------------------------------------------------------------------
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkprp=0
         idum=1
         iflag=0
         call rdrst(t1,tmppsi,ldum,chkdvr,chkgrd,chkpsi,chkprp, &
              cdum,idum,rdum,iflag,jindx,cdum1,idum1)
         close(irst)

!-----------------------------------------------------------------------
! convert old wavefunction if needed
!-----------------------------------------------------------------------
         if (psitype .eq. 0) then
            write(ilog,'(a)')'Psi is transformed from MCTDH to exact.'

            call psi2ex1(tmppsi,oldpsi,dgldim,cdgldim(1))
         else
            oldpsi(:)=tmppsi(:)
         endif

!-----------------------------------------------------------------------
! set up new wavefunction
!-----------------------------------------------------------------------
         do s = 1, nstate
            s1 = smapwf(s,1,nwf)
            s2 = smapwf(s,2,nwf)
            if(s1.eq.0) exit

            write(ilog,'(a,i3,a,i3,a,i3)') &
                 'SPF ',s1, ' ->',s2,',  chn:',irst
            if (s1.gt.nstate) then
               write(message,'(2(a,i3))') &
                  'Not enough states on old psi restart, s1 =',s1, &
                       ' nstate =', nstate
               call errormsg
            endif
            if(s2.gt.cnstate(1)) then
               write(message,'(2(a,i3))') &
                  'Not enough states on new psi restart, s2 =',s2, &
                       ' nstate =', cnstate(1)
               call errormsg
            endif
            call cpvxz(oldpsi(zetf(1,s1)),psi(czetf(1,s2,1)),subdim(1))
         enddo                  !    s=1,nstate

!-----------------------------------------------------------------------
! Deallocate memory
!-----------------------------------------------------------------------
         if (allocated(oldpsi)) deallocate(oldpsi,stat=ierr)
         if (allocated(tmppsi)) deallocate(tmppsi,stat=ierr)
         if (ierr > 0) go to 1001

      enddo  ! end of loop over restart files

!-----------------------------------------------------------------------
! restore data for new psi
!-----------------------------------------------------------------------
      call getcsys(1)
      call getcpsi(1)
      call griddat
      call psidat

      do s2=1,nstate
         psi(zpsi(s2))=1.0_dop
      enddo

!-----------------------------------------------------------------------
! Normalise WP
!-----------------------------------------------------------------------
      call normvxz(psi(zetf(1,1)),norm,dgldim-nstate) ! compute norm
      call xvixdzo(norm,psi(zetf(1,1)),dgldim-nstate) ! normalise
      write(ilog,'(2a,d18.10)') 'Norm of initial WF ',    &
                             'before re-normalisation:', norm

      return
  900 continue
      ilbl=index(filename,' ')-1
      routine = 'getex (geninwf/getpsi.F)'
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      stop
 1001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
      stop

      end subroutine getex

!#######################################################################
! GETPSI3D
!
! called from genpsi
! calls akcopy,phicopy,calcnorm,normpsi
!
! for a restart calculation, the initial wavefunction is read from the
! restart file. It is then possible to alter the number of functions 
! used in the MC expansion, and if required higher functions are 
! generated
!
! psi: initial wavefunction
! ort: grid positions
! workc: complex workspace to store old wavefunction
!   
!#######################################################################

      subroutine getpsi3d (psi,ort,workc,jindx,agmat,mode)

      implicit none

      integer(long)       :: f,f1,s,j,nk,s1,ilbl,dim2,zeig,next,&
                             power,gdof,nstate1,chkdvr,chkgrd,chkpsi,&
                             chkprp,idum,iflag,zagmatold,agmatdim1,gwptype1,&
                             ndof1,m,m1,nmode1,n,n1,gwpdim2
      integer(long), intent(in) :: mode

      integer(long), dimension(1)                   :: idum1
      integer(long), dimension(maxsta)              :: block1,zpsi1
      integer(long), dimension(maxdim)              :: mmode,mmode1,nspfdof1
      integer(long), dimension(jindxdim),intent(out):: jindx
      integer(long), dimension(2*maxdim)            :: msymm1
      integer(long), dimension(maxdim,maxsta)       :: dim1,zetf1,vdim1,&
                                                       ndim1,zagmat1,zpg1,&
                                                       zxi1,zphi1,zdgwp1,&
                                                       gwpdim1,zxivec1,&
                                                       zphase1 ! phase mod yapol
      integer(long), dimension(maxdim,maxdim)       :: spfdof1

      real(dop)                                      :: t1,gxeq,shift
      real(dop), dimension(1)                        :: rdum
      real(dop), dimension(ortdim), intent(in)       :: ort
      complex(dop), dimension(1)                     :: cdum
      complex(dop), dimension(dgldim), intent(inout) :: psi
      complex(dop), dimension(workcdim), intent(out) :: workc
      complex(dop), dimension(agmatdim), intent(out) :: agmat
      logical(kind=4)                                :: ldum,neweb,lerr,linwf
      character(len=c5)                              :: filename
      character(len=c1), dimension(maxdim)           :: modelabel1


      routine='Getpsi3d'

      write(ilog,'(/a/)') '***** Restart calculation *****'

      filename=rname(1:rlaenge)//'/restart'
      open(irst,file=filename,form='unformatted',status='old',err=999)
      write(ilog,'(2a)')  &
          'Restart file read from :  ',rname(1:rlaenge)//'/restart'
      if (lorthopsi) then
         write(ilog,'(a)') &
            'The initial wavefunction is Schmidt-orthogonalized.'
      else
         write(ilog,'(a)') &
            'The initial wavefunction is not Schmidt-orthogonalized.'
      endif

!-----------------------------------------------------------------------
! save data for new psi
!-----------------------------------------------------------------------
      ndof1=ndof
      do f=1,ndof
         modelabel1(f) = modelabel(f)
      enddo
      nmode1=nmode
      do m=1,nmode
         nspfdof1(m)=nspfdof(m)
         do n=1,nspfdof(m)
            spfdof1(n,m)=spfdof(n,m)
         enddo
      enddo
      nstate1=nstate
      do s=1,nstate1
         block1(s)=block(s)
         zpsi1(s)=zpsi(s)
         do m=1,nmode
            dim1(m,s)=dim(m,s)
            vdim1(m,s)=vdim(m,s)
            ndim1(m,s)=ndim(m,s)
            zetf1(m,s)=zetf(m,s)
            msymm1(m)=msymmtr(m)
            msymm1(m+nmode)=msymmtr(m+nmode)
         enddo
      enddo

      gwptype1=0
      if (lgwp) then
         gwptype1=gwptype
         agmatdim1=agmatdim
         do s=1,nstate
            do m=1,nmode
               gwpdim1(m,s)=gwpdim(m,s)
               zagmat1(m,s)=zagmat(m,s)
               zxivec1(m,s)=zxivec(m,s)
               zphase1(m,s)=zphase(m,s)
               zpg1(m,s)=zpg(m,s)
               zxi1(m,s)=zxi(m,s)
               zphi1(m,s)=zphi(m,s)
               zdgwp1(m,s)=zdgwp(m,s)
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! read information about psi on restart file. This must be done before
! reading the file, as the old wavefunction may be different from new.
!-----------------------------------------------------------------------
      neweb = .false.

      chkdvr=3
      chkgrd=3
      chkpsi=1
      chkprp=0
      call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
      routine='Getpsi3d'
      if (lerr) then
         message='Error reading RESTART file'
         call errormsg
      endif

      if (chkdvr .eq. -5) neweb=.true.
      zagmatold=dgldim+1

!-----------------------------------------------------------------------
! read old psi into workc. tinit read from .inp file, not from restart.
!-----------------------------------------------------------------------
      rewind(irst)
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkprp=0
      idum=1
      iflag=0
      call rdrst(t1,workc,ldum,chkdvr,chkgrd,chkpsi,chkprp, &
           cdum,idum,rdum,iflag,jindx,workc(zagmatold),idum1)

      close(irst)

!-----------------------------------------------------------------------
! map dofs and modes from old to new psi
!-----------------------------------------------------------------------
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            do m1=1,nmode1
               do n1=1,nspfdof1(m1)
                  f1=spfdof1(n1,m1)
                  if (modelabel(f) .eq. modelabel1(f1)) then
                     mmode(m)=m1
                     mmode1(m1)=m
                  endif
               enddo
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! copy single particle functions to relevant part of psi array, 
! discarding or generating higher phi functions as required
!-----------------------------------------------------------------------
      shift = 0.0_dop
      do s=1,nstate1
         if (neweb) then
            s1=1
         else
            s1 = s
         endif

         do m=1,nmode
          if (mmode(m).eq.mode) then
             dim2=min(dim(m,s1),dim1(mode,s))

! GWP mode
            if (gwpm(m)) then
               gwpdim2=min(gwpdim(m,s1),gwpdim1(mode,s))
               gdof=nspfdof(m)
               call cpmxz(workc(zxi(m,s1)),psi(zxi1(m,s)), &
                       gdof,gwpdim2)

! copy agmat, xivec, phase
               call cpmxz(workc(zagmatold+zagmat(m,s1)-1),agmat(zagmat1(m,s)), &
                    gdof*gdof,gwpdim2)
               call cpmxz(workc(zagmatold+zxivec(m,s1)-1),agmat(zxivec1(m,s)), &
                    gdof,gwpdim2)
               call cpvxz(workc(zagmatold+zphase(m,s1)-1),agmat(zphase1(m,s)), &
                    gwpdim2)

               if (lgwpphase) call cpvxz(workc(zphi(m,s1)),psi(zphi1(m,s)),gwpdim2)
               call cpmxz(workc(zxi(m,s1)),psi(zxi1(m,s)),gdof,gwpdim2)

               if (psitype .eq. 0) then
                  call cpmxz(workc(zdgwp(m,s1)),psi(zdgwp1(m,s)), &
                             gwpdim2,dim2)
               endif

! if old psi is thawed GWP
               if (gwptype .le. 1) then
                  if (gwptype1 .le. 1) then
                     call cpmxz(workc(zpg(m,s1)),psi(zpg1(m,s)), &
                          gdof*gdof,gwpdim2)
                  else 
                     routine = 'Getpsi3d'
                     message =  &
                         'GWP conversion only allowed'// &
                         ' thawed <--> non-diagonal frozen'
                     call errormsg
                  endif

! if old psi is separable
               else if (gwptype .eq. 2) then
                  if (gwptype1 .le. 1) then
                     call sep2gwp(workc(zpg(m,s1)),psi(zpg1(m,s)), &
                          gdof,gwpdim2)
                  else if (gwptype1 .eq. 2) then
                     call cpmxz(workc(zpg(m,s1)),psi(zpg1(m,s)), &
                          gdof,gwpdim2)
                  endif

! if old psi is diagonal FG or uncorrelated vMCG
               else if (gwptype .eq. 3) then
                  zeig=zagmatold+zagmat(m,s1)-1
                  if (gwptype1 .le. 1) then
                     call cpmxz(workc(zeig),psi(zpg1(m,s)), &
                          gdof*gdof,gwpdim2)
                  else if (gwptype1 .eq. 2) then
                     call gwp2sep(workc(zeig),psi(zpg1(m,s)), &
                          gdof,gwpdim2)
                  else if (gwptype1 .eq. 4) then
                     routine = 'Getpsi3d'
                     message =  &
                        'Pointless to convert diagonal FG to'// &
                        ' non-diagonal'
                     call errormsg
                  endif

! if old psi is non-diagonal FG
               else if (gwptype .eq. 4) then
                  zeig=zagmatold+zagmat(m,s1)-1
                  if (gwptype1 .le. 1) then
                     call cpmxz(workc(zeig),psi(zpg1(m,s)), &
                          gdof*gdof,gwpdim2)
                  else if (gwptype1 .eq. 2) then
                     routine = 'Getpsi3d'
                     message = &
                        'Conversion from non-diagonal FG to'// &
                        ' separable not possible'
                     call errormsg
                  else if (gwptype1 .eq. 3) then
                     routine = 'Getpsi3d'
                     message = &
                        'Conversion from non-diagonal FG to'// &
                        ' diagonal FG not possible'
                     call errormsg
                  endif

               endif

               if (gwpdim1(m,s) .gt. gwpdim2) then
                  if (gwpdim(m,s1) .eq. 1 .and. ldd .and. nstate1 .gt. 1) then
                     call ddhigher(psi(zxi1(m,s)),psi(zphi1(m,s)), &
                          gdof,gwpdim1(m,s),s,agmat(zagmat1(m,s)), &
                          agmat(zxivec(m,s)),agmat(zphase(m,s)))
                  else
                     routine = 'Getpsi3d'
                     message = &
            'Cannot generate extra functions for GWPs '
                     call errormsg
                  endif
               endif

! normal SPFs
            else if (.not. gwpm(m)) then

               next=dim(m,s)+1
               if (dim1(mode,s) .gt. dim(m,s)) then
                  call cpmxz(workc(zetf(m,s)),psi(zetf1(mode,s)), &
                        subdim(m),dim(m,s))

                  if(nspfdof(m) .eq. 1 ) then
                     f = spfdof(1,m)
                     gxeq = 0.0_dop
                     power= 1
                     f1   = 0
                     do j=1,2*maxdim ! Search for sym1d, asym1d
                        if(msymmtr(j).eq.0) goto 40
                        if(f.eq.mod(abs(msymmtr(j)),256)) then
                           f1=msymmtr(j)
                           power=2
                           gxeq= 0.5_dop*(ort(zort(f))+ort(zort(f)+gdim(f)-1)) 
                           goto 40
                        endif
                     enddo
 40                  call genhigher(psi(zetf1(mode,s)),subdim(m),dim1(mode,s), &
                            next,ort(zort(f)),basis(f),ipbaspar(1,f), &
                            power,gxeq,shift,f1)
                  else
                     call get_num_klegs(nk,m)
                     if (nk.eq.0) then
                        if(idmode(m).eq.0) &
                          call mgenhigher(psi(zetf1(mode,s)),subdim(m), &
                                      dim1(mode,s),next,m,msymm1)
                     elseif (nk.eq.1) then
                        call kgenhigher(psi(zetf1(mode,s)),ort,subdim(m), &
                                        dim1(mode,s),next,m)
                     else
                        call kgenhigher2
                     endif
                  endif
               else 
                   call cpmxz(workc(zetf(m,s)),psi(zetf1(mode,s)), &
                        subdim(m),dim2)
               endif
            endif
          endif
         enddo
      enddo

!-----------------------------------------------------------------------
! restore data for new psi
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/restart'
      open(irst,file=filename,form='unformatted',status='old',err=999)
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkprp=0
      call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
      routine='Getpsi3d'
      if (lerr) then
         message='Error reading RESTART file'
         call errormsg
      endif
      close(irst)
      call griddat
      call psidat

!-----------------------------------------------------------------------
      return
  999 continue
      ilbl=index(filename,' ')-1
      routine = 'Getpsi3d'
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      end subroutine getpsi3d       
      
!#######################################################################
! GETCOEFF3D
!
! called from gencoeff
! calls akcopy,phicopy,calcnorm,normpsi
!
! for a restart calculation, the initial wavefunction is read from the
! restart file. It is then possible to alter the number of functions 
! used in the MC expansion, and if required higher functions are 
! generated
!
! psi: initial wavefunction
! ort: grid positions
! workc: complex workspace to store old wavefunction
!   
!
! This routine looks dodgy - psi from restart file overwrites system 
! info that is not restored at end??
!#######################################################################

      subroutine getcoeff3d(psi,jindx)

      implicit none

      integer(long)       :: f,f1,s,ilbl,mblock,dim2,zeig,zeig1,zeig2,ierr,&
                             nstate1,chkdvr,chkgrd,chkpsi,chkprp,idum,iflag,&
                             instate,agmatdim1,gwptype1,ndim2,&
                             ndof1,m,m1,nmode1,n,n1,mtmp

      integer(long), dimension(1)                    :: idum1
      integer(long), dimension(maxsta)               :: block1,zpsi1
      integer(long), dimension(maxdim)               :: mmode,mmode1,nspfdof1
      integer(long), dimension(jindxdim), intent(out):: jindx
      integer(long), dimension(2*maxdim)             :: msymm1
      integer(long), dimension(maxdim,maxsta)        :: dim1,zetf1,vdim1,&
                                                        ndim1,zagmat1,zpg1,&
                                                        zxi1,zphi1,vdim2,&
                                                        zxivec1,zphase1 
      integer(long), dimension(maxdim,maxdim)        :: spfdof1

      real(dop)                                      :: t1
      real(dop), dimension(1)                        :: rdum
      complex(dop), dimension(1)                     :: cdum
      complex(dop), dimension(dgldim), intent(inout) :: psi
      complex(dop), dimension(:), allocatable        :: oldpsi
      complex(dop), dimension(:), allocatable        :: oldagmat
      complex(dop), dimension(:), allocatable        :: tmppsi
      logical(kind=4)                                :: ldum,neweb,lerr,linwf
      character(len=c5)                              :: filename
      character(len=c1), dimension(maxdim)           :: modelabel1
      character(len=16)                               :: cerr

      routine='Getcoeff3d'

      write(ilog,'(/a/)') '***** Restart calculation *****'

      filename=rname(1:rlaenge)//'/restart'
      open(irst,file=filename,form='unformatted',status='old',err=999)
      write(ilog,'(2a)')  &
          'Restart file read from :  ',rname(1:rlaenge)//'/restart'
      if (lorthopsi) then
         write(ilog,'(a)') &
            'The initial wavefunction is Schmidt-orthogonalized.'
      else
         write(ilog,'(a)') &
            'The initial wavefunction is not Schmidt-orthogonalized.'
      endif

!-----------------------------------------------------------------------
! save data for new psi
!-----------------------------------------------------------------------
      ndof1=ndof
      do f=1,ndof
         modelabel1(f) = modelabel(f)
      enddo
      nmode1=nmode
      do m=1,nmode
         nspfdof1(m)=nspfdof(m)
         do n=1,nspfdof(m)
            spfdof1(n,m)=spfdof(n,m)
         enddo
      enddo

      nstate1=nstate
      do s=1,nstate1
         block1(s)=block(s)
         zpsi1(s)=zpsi(s)
         do m=1,nmode
            dim1(m,s)=dim(m,s)
            vdim1(m,s)=vdim(m,s)
            ndim1(m,s)=ndim(m,s)
            zetf1(m,s)=zetf(m,s)
            msymm1(m)=msymmtr(m)
            msymm1(m+nmode)=msymmtr(m+nmode)
         enddo
      enddo

      if (lgwp) then
         gwptype1=gwptype
         agmatdim1=agmatdim
         do s=1,nstate
            do m=1,nmode
               zagmat1(m,s)=zagmat(m,s)
               zxivec1(m,s)=zxivec(m,s)
               zphase1(m,s)=zphase(m,s)
               zpg1(m,s)=zpg(m,s)
               zxi1(m,s)=zxi(m,s)
               zphi1(m,s)=zphi(m,s)
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! read information about psi on restart file. This must be done before
! reading the file, as the old wavefunction may be different from new.
!-----------------------------------------------------------------------
      chkdvr=3
      chkgrd=3
      chkpsi=1
      chkprp=0
      call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
      routine='Getcoeff3d'
      if (lerr) then
         message='Error reading RESTART file'
         call errormsg
      endif

      neweb = .false.
      if (chkdvr .eq. -5) neweb=.true.

!-----------------------------------------------------------------------
! Allocate memory for psi read from restart file
!-----------------------------------------------------------------------
      cerr = 'getpsi_oldpsi   '
      allocate(oldpsi(dgldim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'getpsi_oldagmat '
      allocate(oldagmat(agmatdim),stat=ierr)
      if (ierr > 0) go to 1000

!-----------------------------------------------------------------------
! calculate maximum size of block and allocate tmppsi array
!-----------------------------------------------------------------------
      mtmp = 0
      do s=1,nstate1
         mblock=1
         do m=1,nmode
            dim2=max(dim(m,s),dim1(m,s))
            mblock=mblock*dim2
         enddo
         mtmp=max(mtmp,2*mblock)
      enddo
      mtmp=max(mtmp,dgldim)

      cerr = 'getpsi_tmppsi   '
      allocate(tmppsi(mtmp),stat=ierr)
      if (ierr > 0) go to 1000

!-----------------------------------------------------------------------
! read old psi into workc. tinit read from .inp file, not from restart.
!-----------------------------------------------------------------------
      rewind(irst)
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkprp=0
      idum=1
      iflag=0
            
      call rdrst(t1,oldpsi,ldum,chkdvr,chkgrd,chkpsi,chkprp, &
           cdum,idum,rdum,iflag,jindx,oldagmat,idum1)
      close(irst)

!-----------------------------------------------------------------------
! map dofs and modes from old to new psi
!-----------------------------------------------------------------------
      mmode1(:)=0
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            do m1=1,nmode1
               do n1=1,nspfdof1(m1)
                  f1=spfdof1(n1,m1)
                  if (modelabel(f) .eq. modelabel1(f1)) then
                     mmode(m)=m1
                     mmode1(m1)=m
                  endif
               enddo
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! see if old psi is from relaxation on single-state to be excited
!    into manifold (chkdvr = -5 returned from rdrst)
! if isbaspar(4,feb,1) not set (initial state has not been set by 
!     change_state keyword), assume lowest state is populated.
!-----------------------------------------------------------------------
      if (neweb) then
         if (isbaspar(4,feb,1) .eq. 0) isbaspar(4,feb,1)=1
         instate = isbaspar(4,feb,1)
      endif

!-----------------------------------------------------------------------
! To copy old coefficients to new wavefunction, it is neccesary to
! first correct for any difference in the number of MC functions.
!-----------------------------------------------------------------------
      do s=1,nstate
         mblock=1
         do m=1,nmode
            m1=mmode(m)
            dim2=max(dim(m,s),dim1(m1,s))
            mblock=mblock*dim2
         enddo
         zeig=1
         zeig1=zeig+mblock
         call cpvxz(oldpsi(zpsi(s)),tmppsi(zeig),block(s))

         vdim2(1,s) = vdim(1,s)
         ndim2=1
         do m=1,nmode
            m1=mmode(m)
            call akcopy(tmppsi(zeig),tmppsi(zeig1),vdim2(m,s),dim1(m1,s), &
                 dim(m,s),ndim(m,s))
            zeig2=zeig
            zeig=zeig1
            zeig1=zeig2
            vdim2(m+1,s)=vdim2(m,s)*dim1(m1,s)
            ndim2=ndim2*dim1(m1,s)
!            dim(m,s) = dim1(m1,s) 
         enddo
!-----------------------------------------------------------------------
! make sure final result is at dgldim+1
!-----------------------------------------------------------------------
         if (zeig1 .lt. zeig) then
            call cpvxz(tmppsi(zeig),tmppsi(zeig1),mblock)
            zeig=dgldim+1
         endif
         
!-----------------------------------------------------------------------
! workc(zeig) is old wavefunction with new wavefunction indices
!-----------------------------------------------------------------------
         zeig1=zeig+block1(s)
         
!         ndim2=dim(1,s)*ndim(1,s)
         do m1=1,nmode1
            m=mmode1(m1)
            if (m .eq. 0) then
               call akcopypsi3d(tmppsi(zeig),tmppsi(zeig1), &
                    vdim1(m1,s),dim1(m1,s),ndim2)
               zeig2=zeig
               zeig=zeig1
               zeig1=zeig2
            else
               ndim2=ndim(m,s)
            endif
         enddo
         call cpvxz(tmppsi(zeig),psi(zpsi1(s)),block1(s))

      enddo

      filename=name(1:laenge)//'/restart'
      open(irst,file=filename,form='unformatted',status='old',err=999)
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkprp=0
      call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
      routine='Getpsi3d'
      if (lerr) then
         message='Error reading RESTART file'
         call errormsg
      endif
      close(irst)
      call griddat
      call psidat

      cerr = 'getpsi_oldpsi   '
      deallocate(oldpsi,stat=ierr)
      if (ierr > 0) go to 1001
      cerr = 'getpsi_oldagmat '
      deallocate(oldagmat,stat=ierr)
      if (ierr > 0) go to 1001
      cerr = 'getpsi_tmppsi   '
      deallocate(tmppsi,stat=ierr)
      if (ierr > 0) go to 1001

!-----------------------------------------------------------------------
      return
  999 continue
      ilbl=index(filename,' ')-1
      routine = 'Getcoeff3d'
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      stop
 1001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
      stop

      end subroutine getcoeff3d

!#######################################################################
! GETBLKA
!
! called from runinwf, handels the "block-A" block.
! reads A-vectors from restart files and builds a block-A-vector
! for block improved relaxation.
!
! psi: initial wavefunction
! workc: complex workspace to store old wavefunction
!
!#######################################################################

      subroutine getblka (psi,workc,jindx,agmat)

      implicit none

      integer(long)  ::    m,s,i,mblock,ierr,irst0,zeig,zeig1,zeig2,adim1, &
                           block1,chkdvr,chkgrd,chkpsi,chkprp,idum,iflag,  &
                           dim2,npackts2,ia

      integer(long), dimension(1)                     :: idum1
      integer(long), dimension(jindxdim), intent(out) :: jindx
      real(dop), dimension(1)                         :: rdum
      real(dop)                                       :: norm,t1
      complex(dop), dimension(dgldim), intent(out)    :: psi
      complex(dop), dimension(workcdim),  intent(out) :: workc
      complex(dop), dimension(agmatdim), intent(out)  :: agmat
      complex(dop), dimension(1)                      :: cdum
      logical(kind=4)                                 :: ldum,lmult2,leb2,&
                                                         lmulpack2,cerror
      character(len=c5)                               :: filename

      routine='Getblka (for block-A)'
      write(ilog,'(/a)') '***** Restart calculation (block-A) *****'
      if(lorthopsi) write(ilog,'(a,/2a)')                            &
         '### noorthopsi is NOT set! ###',                           &
         'If block-SPF is used to read the SPFs, noorthopsi must ',  &
         'be set in  block-SPF.'
      if(lrealpsi) write(ilog,'(a)') '### block A: realpsi is set! ###'

      call zerovxz(psi,adim)
      irst0=irst

      if(mpb.le.0 .or. npackts.le.1) then
         message='This is not a multi-packet single-set run. '//    &
          'Use "read-inwf" or "file" to define the initial WF.'
         call errormsg 
      endif

!-----------------------------------------------------------------------
! save structure-data of psi, because rdrst will overwrite
!-----------------------------------------------------------------------
      irst0=irst
      call psidat
      call savecsys(1)
      call savecpsi(1)
      lmult2=lmult
      leb2=leb
      lmulpack2=lmulpack
      npackts2=npackts

!-----------------------------------------------------------------------
! Do-Loop over all restart files specified in a block-A block
!-----------------------------------------------------------------------
      do ia = 1, npackts2
         irst = 591+ia

!-----------------------------------------------------------------------
! read the restart file defined in the read-inwf block into workc.
!-----------------------------------------------------------------------
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkprp=0
         idum=1
         iflag=0
         call rdrst(t1,workc,ldum,chkdvr,chkgrd,chkpsi,chkprp,&
           cdum,idum,rdum,iflag,jindx,agmat,idum1)
         close(irst)
         call griddat
         call psidat
 
!-----------------------------------------------------------------------
! check consistency.
!-----------------------------------------------------------------------
         cerror=.false.
         if(nstate.ne. cnstate(1))  cerror=.true.
         if(nmode .ne. cnmode(1)-1) cerror=.true.
         do m=1,nmode
            if(subdim(m) .ne. csubdim(m,1)) cerror=.true.
         enddo
         if(cerror) then
            write(6,'(a,2i5)')    'irst =', irst, ia
            write(ilog,'(a,2i5)') 'irst =', irst, ia
            message='Inconsistent data structure (nmode, subdim) on '//  &
                    'foreign restart file.'
            call errormsg 
         endif

!-----------------------------------------------------------------------
! To copy old coefficients to new wavefunction, it is neccesary to
! correct for any difference in the number of SPFs.
!-----------------------------------------------------------------------
         if(lrealpsi) then
            do i=1, adim
               workc(i)=dble(workc(i))
            enddo
         endif

         do s=1,nstate
            block1=cblock(s,1)/npackts2
            adim1=cadim(1)/npackts2
            mblock=1
            do m=1,nmode
               dim2=max(dim(m,s),cdim(m,s,1))
               mblock=mblock*dim2
            enddo
            zeig=dgldim+1
            zeig1=zeig+mblock
            call cpvxz(workc(zpsi(s)),workc(zeig),block(s))

            do m=1,nmode
               call akcopy(workc(zeig),workc(zeig1),cvdim(m,s,1),   &
                           cdim(m,s,1),dim(m,s),ndim(m,s))
               zeig2=zeig
               zeig=zeig1
               zeig1=zeig2
            enddo            !    m=1,nmode
            call cpvxz(workc(zeig),psi(czpsi(s,1)+(ia-1)*block1),block1)
         enddo               !    s=1,nstate

         write(ilog,'(a,i3,a,i4,a,2i8,a)') 'Stored A-vector no.',ia,  &
          ' to block-A.  chn:',irst,'   adim =',adim, adim1,' (old/new)'
         call myflush(ilog)

      enddo                  !    ia=1,npackts

!-----------------------------------------------------------------------
! restore data for psi
!-----------------------------------------------------------------------
      call getcsys(1)
      call getcpsi(1)
      lmult=lmult2
      leb=leb2
      lmulpack=lmulpack2
      irst=irst0
      filename=dname(1:dlaenge)//'/dvr'
      open(idvr,file=filename,form='unformatted',status='old',err=999)
      chkdvr=1    !  Read DVR info again, as overwritten by rdrst.
      call dvrinfo(ldum,chkdvr)
      close(idvr)
      call griddat
      call psidat

!-----------------------------------------------------------------------
! Schmidt-orthogonalize the A-vector block
!-----------------------------------------------------------------------
      if(.not.lmult2) then
      call schmidtortho2(psi,adim/npackts,npackts,ierr,norm)
      write(ilog,'(a,e12.4)')    &
      'Schmidt-orthogonalization of A-vector block. Minimal norm =',norm
      if(ierr .gt. 0) then
         write(6,'(a,i3)')       &
         'getblka: Cannot Schmidt-orthogonalize block-A. ierr =',ierr
         write(ilog,'(a,i3)')    &
         'getblka: Cannot Schmidt-orthogonalize block-A. ierr =',ierr
         call stopnow
      endif
      endif

      return

  999 continue
      i=index(filename,' ')-1
      routine = 'Getblka'
      message = 'Cannot open file: '//filename(1:i)
      call errormsg
      end subroutine getblka


!#######################################################################
! GETBLK
!
! called from runinwf, handels the "block-SPF" block.
! reads the SPFs from a restart file and adds the "electronic"
! SPFs for block improved relaxation.
!
! psi: initial wavefunction
! ort: grid positions
! workc: complex workspace to store old wavefunction
!
!#######################################################################

      subroutine getblk (psi,ort,dicht1,dicht2,dicht3,dicht4,  &
                         workc,jindx,agmat,exphin,exprueck)

      implicit none

      integer(long)   :: m,s,ierr,irst0,i,  &
                         chkdvr,chkgrd,chkpsi,chkprp,idum,iflag    

      integer(long), dimension(1)                       :: idum1
      integer(long), dimension(jindxdim), intent(out)   :: jindx
      real(dop), dimension(ortdim), intent(in)          :: ort
      real(dop), dimension(d3matdim), intent(out)       :: dicht3
      real(dop), dimension(1)                           :: rdum
      real(dop)                                         :: shift,t1
      complex(dop), dimension(dgldim), intent(out)      :: psi
      complex(dop), dimension(dmatdim), intent(out)     :: dicht1,dicht2,dicht4
      complex(dop), dimension(workcdim), intent(out)    :: workc
      complex(dop), dimension(agmatdim), intent(out)    :: agmat
      complex(dop), dimension(expdim), intent(in)       :: exphin,exprueck
      complex(dop), dimension(1)                        :: cdum
      logical(kind=4)                                   :: ldum,ladone,lmult2,leb2,lmulpack2,cerror
      character(len=c5)                                 :: filename

      routine='getpsi.F Getblk (block-SPF)'
      write(ilog,'(/a)') '***** Restart calculation (block-SPF) *****'
      if(.not.lorthopsi) write(ilog,'(a)') '### noorthopsi is set! ###'
      if(lorthopsi) write(ilog,'(a,/2a)')        &
              '### noorthopsi is NOT set! ###',  &
              'If block-A is used to read the A-vector(s) one must ',  &
              'set noorthopsi in block-SPF'
      if(lrealpsi) write(ilog,'(a)') '### block-SPF: realpsi is set! ###'

      call zerovxz(psi,dgldim)
      irst0=irst
      ladone=.false.

      if(mpb.le.0 .or. npackts.le.1) then
         message='This is not a multi-packet single-set run. '//   &
          'Use "read-inwf" or "file" to define the initial WF.'
         call errormsg
      endif

!-----------------------------------------------------------------------
! save structure-data of psi, because rdrst will overwrite
!-----------------------------------------------------------------------
      irst0=irst
      call psidat
      call savecsys(1)
      call savecpsi(1)
      lmult2=lmult
      leb2=leb
      lmulpack2=lmulpack
!-----------------------------------------------------------------------
! read the restart file defined by the blk-SPF keyword into workc.
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkprp=0
      idum=1
      iflag=0
      irst=591
      call rdrst(t1,workc,ldum,chkdvr,chkgrd,chkpsi,chkprp,&
           cdum,idum,rdum,iflag,jindx,agmat,idum1)
      close(irst)

      call griddat
      call psidat

!-----------------------------------------------------------------------
! check consistency.
!-----------------------------------------------------------------------
      cerror=.false.
      if(nmode .eq. cnmode(1)) nmode=nmode-1  ! reading a block-rst file
      if(nmode .ne. cnmode(1)-1) cerror=.true.
      if(nstate.ne. cnstate(1))  cerror=.true.
      do m=1,nmode
         if(subdim(m) .ne. csubdim(m,1)) cerror=.true.
      enddo
      if(cerror) then
         routine='getblk on getpsi.F'
         message='Inconsistent data structure (nstate, nmode, subdim)'//  &
                  ' on foreign restart file. (block-SPF)'
         call errormsg
      endif

      if (lrealpsi) then   ! take real-part of psi if desired
         call realpsi(workc,rdum)
      endif

!-----------------------------------------------------------------------
! Schmidt-orthogonalize initial wave function if desired
!    1. compute density matrices
!    2. transform to natural orbital basis
!    3. Schmidt-orthogonalize single-particle functions
!-----------------------------------------------------------------------
      if(lorthopsi) then
         call density(workc,tree,dicht1,dicht2,dicht3,dicht4,jindx,cdum)
         do s=1,nstate
            do m=1,nmode
               call mqxczz(workc(zetf(m,s)),dicht4(dmat(m,s)),         &
                           workc(zetf(m,s)+dgldim),subdim(m),dim(m,s))
               call cpmxz(workc(zetf(m,s)+dgldim),workc(zetf(m,s)),    &
                          subdim(m),dim(m,s))
               call schmidtortho(workc(zetf(m,s)),subdim(m),           &
                                 dim(m,s),ierr)
               if( ierr .gt. 0 ) then
                  routine='getblk on getpsi.F'
                  message='Cannot Schmidt-orthogonalize spf''s.'
                  call errormsg
               end if
            enddo
         enddo

      endif

!-----------------------------------------------------------------------
! Copy old SPFs to new wavefunction
!-----------------------------------------------------------------------
      do s = 1, cnstate(1)
         shift = 0.d0
         do m=1,cnmode(1)-1 ! last mode specifies the block
            if(cidmode(m,1) .eq. 0) then ! Do nothing for identical modes
               if (cdim(m,s,1) .gt. dim(m,s)) then
                  ! copy all SPFs that we have ...
                  call cpmxz(workc(zetf(m,s)),psi(czetf(m,s,1)), &
                       subdim(m),dim(m,s))
                  ! ... and generate more
                  call inventspfs(m,psi(czetf(m,s,1)),dim(m,s), &
                       cdim(m,s,1),subdim(m),ort,exphin,exprueck)
                  ! TODO
               else if (cdim(m,s,1) .le. dim(m,s)) then
                  ! copy as many SPFs as possible, discard the rest
                  call cpmxz(workc(zetf(m,s)),psi(czetf(m,s,1)),  &
                             subdim(m),cdim(m,s,1))
               endif
            endif     ! idmode
         enddo               !    m=1,nmode
      enddo                  !    s=1,nstate

!-----------------------------------------------------------------------
! restore data for psi
!-----------------------------------------------------------------------
      call getcsys(1)
      call getcpsi(1)
      lmult=lmult2
      leb=leb2
      lmulpack=lmulpack2
      irst=irst0
      filename=dname(1:dlaenge)//'/dvr'
      open(idvr,file=filename,form='unformatted',status='old',err=999)
      chkdvr=1    !  Read DVR info again, as overwritten by rdrst.
      call dvrinfo(ldum,chkdvr)
      close(idvr)
      call griddat
      call psidat

!-----------------------------------------------------------------------
! Schmidt-orthogonalize initial single-particle functions
!-----------------------------------------------------------------------
      if(lorthopsi) then
         do s=1,nstate
            do m=1,nmode-1
               call schmidtortho(psi(zetf(m,s)),subdim(m),dim(m,s),ierr)
               if( ierr .gt. 0 ) then
                  routine='Getblk (getpsi.F)'
                  write(message,'(a,i3,a,i3,a,i3)')                    &
                       'Cannot Schmidt-orthogonalize SPFs for s =',s,  &
                       ', m =',m,', e =',ierr
                  call errormsg
               end if
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! Generate SPFs for the package mode (m=nmode)
!-----------------------------------------------------------------------
      do s = 1, nstate
         call gendisc(psi(zetf(nmode,s)),dim(nmode,s),subdim(nmode))
      enddo

      return

  999 continue
      i=index(filename,' ')-1
      routine = 'Getblk'
      message = 'Cannot open file: '//filename(1:i)
      call errormsg
      end subroutine getblk


!#######################################################################
!
! PSI2EX1
!
! converts MCTDH wavefunction to exact wavefunction
!
! data for MCTDH wavefunction in comparison set 1
! data for exact wavefunction in comparison set 2
!
! information in common blocks for exact wavefunction on return
!
!#######################################################################

      subroutine psi2ex1(psi,psiex,dgldim1,dgldimex)

      implicit none

      integer(long)                                  :: s,zeig1,zeig2,vdim1,&
                                                        m,swapzeig,block1
      integer(long), intent(in)                      :: dgldim1,dgldimex
      complex(dop), dimension(dgldim1), intent(in)   :: psi
      complex(dop), dimension(dgldimex), intent(out) :: psiex
      complex(dop), dimension(dgldimex)              :: tmppsi

      do s=1,nstate
         zeig1=1
         zeig2=csubdim(1,1)+1
         call cpvxz(psi(zpsi(s)),tmppsi(zeig2),block(s))

         vdim1=1
         do m=1,nmode
            if (dim(m,s) .gt. 0) then
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig
               call mtxxzz (psi(zetf(m,s)),tmppsi(zeig1), &
                    tmppsi(zeig2),vdim1,dim(m,s),ndim(m,s),subdim(m))
               block1=vdim1*subdim(m)*ndim(m,s)
               vdim1=vdim1*subdim(m)
            endif
         enddo

         call cpvxz(tmppsi(zeig2),psiex(czetf(1,s,1)),csubdim(1,1))
      enddo

!-----------------------------------------------------------------------
! load information for exact (new) wavefunction 
!-----------------------------------------------------------------------
      call getcsys(1)
      call getcpsi(1)

      do s=1,nstate
         psiex(zpsi(s))=0.0_dop
      enddo
      if (feb .ne. 0) then
         s = isbaspar(4,feb,1)
         s = max(s,1)
         psiex(zpsi(s))=1.0_dop
      else
         psiex(zpsi(1))=1.0_dop
      endif

      return
      end subroutine psi2ex1

!#####################################################################

      subroutine realpsi(psi,dicht3)

      implicit none

      real(dop), dimension(d3matdim), intent(inout)  :: dicht3
      complex(dop), dimension(dgldim), intent(inout) :: psi

      if (dentype .eq. 0 .or. dentype .eq. 2) then
         call realpsi0(psi)
      else if (dentype .eq. 1) then
         call realpsi1(psi,dicht3)
      endif

      return
      end subroutine realpsi
 
!#####################################################################

      subroutine realpsi0(psi)

      implicit none

      integer(long) :: i
      real(dop)     :: norm
      complex(dop), dimension(dgldim), intent(inout) :: psi


      do i=1, dgldim
         psi(i)=dble(psi(i))
      enddo

      call normvxz(psi,norm,adim) ! compute norm
      if (lmulpack) norm=norm/sqrt(dble(npacket))
      if (mpb.gt.0) norm=norm/sqrt(dble(npackts))
      if (lreflex)  norm=norm/sqrt(2.d0)
      write(ilog,'(a,f14.9,/a)') 'Norm after taking real part only:',norm, &
          'The wavefunction is re-normalized'

      call xvixdzo(norm,psi,adim) ! normalise

      return
      end subroutine realpsi0
 
!#####################################################################

      subroutine realpsi1(psi,dicht3)

      implicit none

      integer(long) :: i,s,t,m,ierr,z0,zf
      real(dop)     :: norm
      complex(dop), dimension(dgldim), intent(inout) :: psi
      real(dop), dimension(d3matdim), intent(inout)  :: dicht3
      type(wfnode_t),pointer   :: node

! coeffs
      do i=1, adim
         psi(i)=dble(psi(i))
      enddo

! middle layers
      if (psitype .eq. 5) then
         do m=1,tree%tot_nodes               ! loop over branches of top layer
            node => tree%swap_down(m)%p    ! node numder
            if (node%isprimitive .or. node%istop) cycle

            z0 = node%z(1,1)
            zf = node%z(node%navecs,2)
            do i=z0,zf
               psi(i)=dble(psi(i))
            enddo
         enddo
      endif

! SPDOs
      do s=1,nstate
         do t=1,nstate
            do m=1,nmode
               call realspdo(dicht3(dd3mat(m,s,t)),psi(dzetf(m,s,t)),&
                    subdim(m),supgdim(m),ddim(m,s,t),ierr)
               if( ierr .eq. 1 ) then
                  routine='realpsi1 (getpsiutil.F)'
                  write(message,'(a,i3,a,i3,a,i3,a,i3)') &
                       'Density cannot be made real (traceless SPDOs)'
                  call warnmsg1
               end if
            enddo
         enddo
      enddo

      call normvxz(psi,norm,adim) ! compute norm
      if (lmulpack) norm=norm/sqrt(dble(npacket))
      if (mpb.gt.0) norm=norm/sqrt(dble(npackts))
      if (lreflex)  norm=norm/sqrt(2.d0)
      write(ilog,'(a,f14.9,/a)') 'Norm after taking real part only:',norm, &
          'The wavefunction is re-normalized'

      call xvixdzo(norm,psi,adim) ! normalise

      return
      end subroutine realpsi1
 
!#######################################################################

      subroutine realspdo(dicht3,spdo,subdim1,supgdim1,dim1,ierr)

      implicit none

      integer(long), intent(in)  :: subdim1,supgdim1,dim1
      integer(long), intent(out) :: ierr
      integer(long)              :: e,g
      real(dop)                  :: trace
      real(dop), dimension(dim1), intent(in) :: dicht3
      complex(dop), dimension(subdim1,dim1), intent(inout) :: spdo

      ierr = 0
      do e = 1,dim1
         if (dicht3(e) .gt. tiny) then
            call trhxz(spdo(1,e),trace,supgdim1)
            if (abs(trace) .gt. tiny) then     ! trace can be < 0
               do g=1,subdim1
                  spdo(g,e)=dble(spdo(g,e))
               enddo
            else
               ierr = 1
            endif
         endif
      enddo

      return
      end subroutine realspdo


!#######################################################################
!
! GWP2SEP
!
! converts the width matrix from a diagonal FG to separable form
!
!#######################################################################

      subroutine gwp2sep(agmat,dmat,gdof,dim)

      implicit none

      integer(long)                                       :: e,g
      integer(long), intent(in)                           :: gdof,dim
      complex(dop), dimension(gdof,dim), intent(out)      :: dmat
      complex(dop), dimension(gdof,gdof,dim), intent(in)  :: agmat

      do e=1,dim
         do g=1,gdof
            dmat(g,e)=agmat(g,g,e)
         enddo
      enddo

      return
      end subroutine gwp2sep

!#######################################################################
!
! SEP2GWP
!
! converts the width matrix from a separable GWP to that for a thawed 
! or frozen form
!
!#######################################################################

      subroutine sep2gwp(dmat,agmat,gdof,dim)

      implicit none

      integer(long)                                       :: e,g
      integer(long), intent(in)                           :: gdof,dim
      complex(dop), dimension(gdof,dim), intent(in)       :: dmat
      complex(dop), dimension(gdof,gdof,dim), intent(out) :: agmat

      call zeromxz(agmat,gdof*gdof,dim)
      do e=1,dim
         do g=1,gdof
            agmat(g,g,e)=dmat(g,e)
         enddo
      enddo

      return
      end subroutine sep2gwp

!#######################################################################
!
!               WKGETPSI
!
! Here the workspace required for the module is assigned
!
! workcdim, workrdim, workidim and workldim are passed via a common
! block in global.inc, and are checked that they are large enough for
! the module.
!
!#######################################################################

      subroutine wkgetpsi(dgldim1,mblock,mablock,agmatdim1,mxmxblk)

      implicit none

      integer(long)             :: f
      integer(long), intent(in) :: dgldim1,mblock,agmatdim1,mablock,mxmxblk

!-----------------------------------------------------------------------
! assign workspace used in module
!-----------------------------------------------------------------------
      if (lexact .or. lrealwp) then
         workcdim=max(workcdim,dgldim1+2*subdim(1))
      else
         workcdim=max(workcdim,dgldim1+2*mblock+agmatdim1)
         workcdim=max(workcdim,dgldim1+2*mablock+agmatdim1)
         workcdim=max(workcdim,dgldim)
          if (dentype.eq.2) then
            workcdim=max(workcdim,dgldim1+2*mblock*mblock)
         endif
         if (lorthopsi) then
            workcdim=max(workcdim,2*dgldim1+mxmxblk)  !  for trafo to natorbs
         endif
      endif
! for getting part of the psi in getcoeff3d
      do f=1,ndof
         if (sbasis(f) == 30) workcdim=workcdim+mblock
      enddo

      workrdim=workrdim
      workidim=workidim
      workldim=workldim

      return
      end subroutine wkgetpsi

!#######################################################################

      end module getpsimod

