!#######################################################################
! 
! MCTDH-module GETRHO1
!
!#######################################################################

     module getrho1mod 

use global
use maxv,            only: maxdim,maxsta
use dvrdatmod
use griddatmod
use psidef
use runinwfmod
use channels

use iorst,           only: rdrst
use iopsidef,        only: psidat
use densitymod,      only: density
use logdat
use genphi1,         only:genhigher,mgenhigher
use psiutil,         only: akcopy,maxmatwk
use densitymod,      only: density1
use mmlib,           only: mqxczz, mqxtzz
use xvlib,           only: xvixdzo
use schmidtorthomod, only: schmidtortho
use op1lib,          only: cpvxz, normvxz, cpmxz
use compdat, only: getcsys,getcpsi,savecsys,savecpsi
use iogrddef, only: griddat

      implicit none
      private
      public :: getrho1, getrhoex
      contains

!#######################################################################
!
! GETRHO1
!
! Reads in a type I density matrix
!
!#######################################################################

      subroutine getrho1 (psi,dicht1,dicht2,dicht3,dicht4,workc,jindx)

      implicit none

      integer(long)       :: dgldim1,next,m,s,t,adim1,dmatdim1,d3matdim1,&
                             ilbl,mdblock,dim2,zeig,zeig1,zeig2,ierr,chkdvr,&
                             chkgrd,chkpsi,chkprp,idum,iflag

      integer(long),dimension(maxdim,maxsta,maxsta)      :: &
                                                           ddim1,dzetf1,vddim1,&
                                                           nddim1,ddmat1,d3mat1

      integer(long), dimension(maxsta,maxsta)        :: dblock1,dzpsi1
      integer(long), dimension(1)                    :: idum1
      integer(long), dimension(jindxdim), intent(out):: jindx

      real(dop)                                     :: norm,t1,norm1,gxeq,shift
      real(dop), dimension(1)                       :: rdum
      real(dop), dimension(d3matdim), intent(out)   :: dicht3

      complex(dop), dimension(1)                     :: cdum,cdum1
      complex(dop), dimension(dgldim), intent(out)   :: psi
      complex(dop), dimension(dmatdim), intent(out)  :: dicht2
      complex(dop), dimension(dmatdim), intent(out)  :: dicht1
      complex(dop), dimension(dmatdim), intent(inout):: dicht4
      complex(dop), dimension(workcdim), intent(out) :: workc

      logical(kind=4)                                :: ldum
      character(len=c5)                              :: filename

      routine='Getrho1'

      write (ilog,*)
      write(ilog,'(a)') '***** Restart calculation *****'
      Write(ilog,*)

      filename=rname(1:rlaenge)//'/restart'
      open(irst,file=filename,form='unformatted',status='old',err=999)
      write(ilog,'(a)')  &
          'Restart file read from :  ',rname(1:rlaenge)//'/restart'
      if (lorthopsi) then
         write(ilog,'(a)') &
            'The initial wavefunction is Schmidt-orthogonalized.'
      else
         write(ilog,'(a)') &
            'The initial wavefunction is not Schmidt-orthogonalized.'
      endif

!-----------------------------------------------------------------------
! save data for new psi
!-----------------------------------------------------------------------
      call psidat
      call savecsys(1)
      call savecpsi(1)

      dgldim1=dgldim
      adim1=adim
      dmatdim1=dmatdim
      d3matdim1=d3matdim
      do s=1,nstate
         do t=1,nstate
            dblock1(s,t)=dblock(s,t)
            dzpsi1(s,t)=dzpsi(s,t)
            do m=1,nmode
               ddim1(m,s,t)=ddim(m,s,t)
               vddim1(m,s,t)=vddim(m,s,t)
               nddim1(m,s,t)=nddim(m,s,t)
               dzetf1(m,s,t)=dzetf(m,s,t)
               ddmat1(m,s,t)=ddmat(m,s,t)
               d3mat1(m,s,t)=dd3mat(m,s,t)
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! read old psi into workc. tinit read from .inp file, not restart.
!-----------------------------------------------------------------------
      chkdvr=2
      chkgrd=2
      chkpsi=1
      chkprp=0
      idum=1
      iflag=0
      call rdrst(t1,workc,ldum,chkdvr,chkgrd,chkpsi,chkprp,&
           cdum,idum,rdum,iflag,jindx,cdum1,idum1)
      close(irst)

!-----------------------------------------------------------------------
! To copy old coefficients to new wavefunction, it is neccesary to 
! correct for any difference in the number of MC functions. 
!-----------------------------------------------------------------------
      do s=1,nstate
         do t=1,nstate
            mdblock=1
            do m=1,nmode
               dim2=max(ddim(m,s,t),ddim1(m,s,t))
               mdblock=mdblock*dim2
            enddo
            zeig=dgldim+1
            zeig1=zeig+mdblock
            call cpvxz(workc(dzpsi(s,t)),workc(zeig),dblock(s,t))

            do m=1,nmode
               call akcopy(workc(zeig),workc(zeig1),vddim1(m,s,t),&
                    ddim1(m,s,t),ddim(m,s,t),nddim(m,s,t))
               zeig2=zeig
               zeig=zeig1
               zeig1=zeig2
            enddo
            call cpvxz(workc(zeig),psi(dzpsi1(s,t)),dblock1(s,t))

!-----------------------------------------------------------------------
! normalise new wavefunction to be same as old
!-----------------------------------------------------------------------
            call normvxz(workc(dzpsi(s,t)),norm,dblock(s,t))
            call normvxz(psi(dzpsi1(s,t)),norm1,dblock1(s,t))
            if (nstate .eq. 1) then
               write(ilog,'(a,f12.8)') &
                    'Old norm                       : ',norm
               write(ilog,'(a,f12.8)') &
                    'new norm (before normalisation): ',norm1
            else
               write(ilog,'(a,2i3,a,f12.8)') &
                    'Old norm                       , state',s,t,' : '&
                    ,norm
               write(ilog,'(a,2i3,a,f12.8)') &
                    'New norm (before normalisation), state',s,t,' : '&
                    ,norm1
            endif
            if (abs(norm-0.0_dop) .gt. tiny) then
               norm1=norm1/norm
               call xvixdzo(norm1,psi(dzpsi1(s,t)),dblock1(s,t))
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! copy single particle functions to relevant part of psi array, 
! discarding or generating higher phi functions as required
!-----------------------------------------------------------------------
      shift = 0._dop
      do s=1,nstate
         do t=1,nstate
            do m=1,nmode
               gxeq = 0.0_dop
               if (gwpm(m)) then
                  routine = 'Getrho1 (getfho1mod.f90) '
                  message = &
                       'Adding GWPs after reading initial rho(0) not yet implemented'
                  call errormsg
               else if (ddim1(m,s,t) .gt. ddim(m,s,t)) then
                  call cpmxz(workc(dzetf(m,s,t)),psi(dzetf1(m,s,t)), &
                       subdim(m),ddim(m,s,t))
                  next=ddim(m,s,t)+1
                  call mgenhigher(psi(dzetf1(m,s,t)),subdim(m), &
                       ddim1(m,s,t),next,m,msymmtr)
               else if (ddim1(m,s,t) .le. ddim(m,s,t)) then
                  call cpmxz(workc(dzetf(m,s,t)),psi(dzetf1(m,s,t)), &
                       subdim(m),ddim1(m,s,t))
               endif
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! restore data for new psi
!-----------------------------------------------------------------------
      call getcsys(1)
      call getcpsi(1)
      call griddat
      call psidat

!-----------------------------------------------------------------------
! Schmidt-orthogonalize initial wave function if desired
!    1. compute density matrices
!    2. transform to natural orbital basis
!    3. Schmidt-orthogonalize single-particle functions
!-----------------------------------------------------------------------
      if (lorthopsi) then
         call density1(psi,dicht1,dicht2,dicht3,dicht4)
         do s=1,nstate
            do t=1,nstate
               do m=1,nmode
                  call mqxczz(psi(dzetf(m,s,t)),dicht4(ddmat(m,s,t)), &
                       workc(dzetf(m,s,t)),subdim(m),ddim(m,s,t))
                  call cpmxz(workc(dzetf(m,s,t)),psi(dzetf(m,s,t)), &
                       subdim(m),ddim(m,s,t))
                  call schmidtortho(psi(dzetf(m,s,t)),subdim(m), &
                       ddim(m,s,t),ierr)
                  if( ierr .gt. 0 ) then
                     routine='Getrho1'
                     message='Cannot Schmidt-orthogonalize spdo''s.'
                     call errormsg
                  end if
               enddo
            enddo
         enddo
       
         do s=1,nstate
            do t=1,nstate
               do m=1,nmode
                  call mqxtzz(psi(dzetf(m,s,t)),dicht4(ddmat(m,s,t)), &
                       workc(dzetf(m,s,t)),subdim(m),ddim(m,s,t))
                  call cpmxz(workc(dzetf(m,s,t)),psi(dzetf(m,s,t)), &
                       subdim(m),ddim(m,s,t))
               enddo
            enddo
         enddo
      endif

      return
  999 continue
      ilbl=index(filename,' ')-1
      routine = 'Getrho1'
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      end subroutine getrho1            
      
!#######################################################################
!
! GETRHOEX
!
! Reads in an exact density matrix
!
!#######################################################################

      subroutine getrhoex (psi,jindx)

      implicit none

      integer(long)       :: ilbl,chkdvr,chkgrd,chkpsi,chkprp,idum,iflag

      integer(long), dimension(1)                     :: idum1
      integer(long), dimension(jindxdim), intent(out) :: jindx

      real(dop)                                       :: t1
      real(dop), dimension(1)                         :: rdum

      complex(dop), dimension(1)                      :: cdum,cdum1
      complex(dop), dimension(dgldim), intent(out)    :: psi
      complex(dop), dimension(dgldim)                 :: workc

      logical(kind=4)                                 :: ldum
      character(len=c5)                               :: filename

      routine='Getrho1'

      filename=rname(1:rlaenge)//'/restart'
      open(irst,file=filename,form='unformatted',status='old',err=999)

!-----------------------------------------------------------------------
! save data for new psi
!-----------------------------------------------------------------------
      call savecsys(2)
      call savecpsi(2)

!-----------------------------------------------------------------------
! read old psi into workc. tinit read from .inp file, not restart.
!-----------------------------------------------------------------------
      chkdvr=5
      chkgrd=20
      chkpsi=1
      chkprp=0
      idum=1
      iflag=0
      call rdrst(t1,workc,ldum,chkdvr,chkgrd,chkpsi,chkprp,&
           cdum,idum,rdum,iflag,jindx,cdum1,idum1)
      close(irst)

!-----------------------------------------------------------------------
! save data for old psi as set 1
!-----------------------------------------------------------------------
      call savecsys(1)
      call savecpsi(1)

!-----------------------------------------------------------------------
! set up new wavefunction
!-----------------------------------------------------------------------
      call cpvxz(workc,psi,dgldim)

!-----------------------------------------------------------------------
! restore data for new psi
!-----------------------------------------------------------------------
      call getcsys(2)
      call getcpsi(2)
      call griddat

      return
  999 continue
      ilbl=index(filename,' ')-1
      routine = 'Getrhoex'
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      end subroutine getrhoex
      
!#######################################################################

      end module getrho1mod 
