!***********************************************************************
! 
!            MCTDH MODULE GWPSPFMOD
!                
! contains : gwpspf
!
!***********************************************************************
      module fgwpspfmod

use decimal, only: long, dop
use logdat

      integer(long) :: optgdof,optdim1

! arrays to pass GWP basis parameters to function
      complex(dop), dimension(:,:,:), allocatable :: optag
      complex(dop), dimension(:,:), allocatable   :: optxi
      complex(dop), dimension(:), allocatable     :: optphi

! arrays to pass GWP basis functions weights to function
      real(dop), dimension(:), allocatable        :: optwei

! arrays to pass initial WP basis parameters to function
      complex(dop), dimension(1)                  :: phi0
      complex(dop), dimension(:,:), allocatable   :: xi0
      complex(dop), dimension(:,:,:), allocatable :: a0


      end module fgwpspfmod

!#######################################################################

      module f1com1

use decimal, only: long, dop

      INTEGER(long) :: ncom
      INTEGER(long), parameter  :: NMAX = 50000
      REAL(dop), dimension(NMAX) :: pcom,xicom

      save ncom,pcom,xicom

      end module f1com1

!#######################################################################

      module gwpspfmod

use decimal,         only: long, dop
use op1lib,          only: zeromxz, zerovxz
use logdat
use gwplib
use xvlib, only: qvxxzz

      implicit none
      private
      public :: gwpspf,dgwpspf

      contains
!***********************************************************************
!
!            MCTDH  subroutine GWPSPF
!
!***********************************************************************

      subroutine gwpspf(phase,xivec,agmat,xigmat,gwpdim1,gdof,m,s,wei)

use global
use griddatmod
use runinwfmod
use psidef
use channels

use fgwpspfmod

      implicit none
      
      integer(long)            :: f,f1,e
      integer(long),intent(in) :: m,s,gdof,gwpdim1

      complex(dop), dimension(gwpdim1), intent(inout)        :: phase
      complex(dop), dimension(gdof,gwpdim1), intent(inout)   :: xivec
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(in) :: aGmat
      complex(dop), dimension(gdof,gwpdim1), intent(out)     :: xigmat
      complex(dop), dimension(gwpdim1), intent(out)          :: wei

      complex(dop)                                    :: spfwidth
      real(dop)                                       :: gxeq,gmom,norm

      integer                                         :: iter,nvar
      real(dop), dimension(gdof,gwpdim1)              :: optpar
      real(dop)                                       :: fret,tol,lb

      logical(kind=4) :: expand

      call zerovxz(wei,gwpdim1)

!-----------------------------------------------------------------------
! see if expansion neccesary
! rsbaspar(3) is width of initial wavepacket
! rsbaspar(6) is width of GWPs
!-----------------------------------------------------------------------
      expand=.false.
      do f=1,gdof
         f1=spfdof(f,m)
         if (abs(rsbaspar(6,f1,s) - rsbaspar(3,f1,s)) .gt. tiny) then
            expand=.true.
         endif
      enddo

      if (.not. expand) then
         wei(1)=1.0_dop
         return
      endif

      write(ilog,'(a,2i3)') &
         '*** Fitting initial WP with GWPs, m, s: ',m,s

      allocate(a0(gdof,gdof,1))
      allocate(xi0(gdof,1))

!-----------------------------------------------------------------------
! set up initial WP (in A,B,C representation)
!-----------------------------------------------------------------------
      call zeromxz(a0,gdof,gdof)
      phi0=0.0_dop
      do f=1,gdof
         f1=spfdof(f,m)
         gxeq   = rsbaspar(1,f1,s)
         gmom   = rsbaspar(2,f1,s)
         spfwidth = dcmplx(rsbaspar(3,f1,s),rsbaspar(4,f1,s))
         spfwidth = -0.25_dop*spfwidth**2

         xi0(f,1)=-2.0_dop*spfwidth*gxeq + (0,1)*gmom
         a0(f,f,1)=spfwidth

         norm = pi/(-2.0_dop*dble(spfwidth))
         norm = dsqrt(norm)
         norm = -0.5_dop*dlog(norm)+1.0_dop*dble(spfwidth)*gxeq**2

 

         phi0(1)=phi0(1)+dcmplx(norm,-gmom*gxeq)
      enddo

!-----------------------------------------------------------------------
! optimise positions and weights to optimally describe initial wavepacket
!-----------------------------------------------------------------------
      nvar = gdof*gwpdim1
      optgdof = gdof
      optdim1 = gwpdim1
      allocate(optag(gdof,gdof,gwpdim1))
      allocate(optxi(gdof,gwpdim1))
      allocate(optphi(gwpdim1))
      allocate(optwei(gwpdim1))

      do e=1,gwpdim1
         optphi(e) = dble(phase(e))
         do f=1,gdof
            optxi(f,e) = xivec(f,e)
            optag(f,f,e) = dble(agmat(f,f,e))

            optpar(f,e) = dble(xivec(f,e))
         enddo
      enddo

      fret=fgwpspf(optpar)

      if (lgwpop0) then
         write(ilog,'(/,a)') '*** Optimise GWPs ***'
         write(ilog,'(/,a)') 'Starting CONJUGATE GRADIENT'
         write(ilog,'(a,e12.6)')'Initial Value of |Psi0 - Psi|^2:  ',fret

         iter=gwpopit
         tol = 1.0d-5
         lb=tiny
         call cgmin(optpar,nvar,tol,iter,fret,fgwpspf,dgwpspf,ilog,lb,1)
         fret=fgwpspf(optpar)
         write(ilog,'(a,e12.6)')'Final Value of |Psi0 - Psi|^2:  ',fret
         
         
         do e=1,gwpdim1
            wei(e) = optwei(e)
            phase(e) = optphi(e)
            do f=1,gdof
               gmom=dimag(xivec(f,e))
               xivec(f,e) = dcmplx(optpar(f,e),gmom)
               gxeq = optpar(f,e)/(-2.0_dop*dble(agmat(f,f,e)))
               xigmat(f,e) = dcmplx(gxeq,gmom)
            enddo
         enddo
      else
         write(ilog,'(a,e12.6)')'Value of |Psi0 - Psi|^2:  ',fret
         do e=1,gwpdim1
            wei(e) = optwei(e)
            phase(e) = optphi(e)
         enddo
      endif

      deallocate(optag,optxi,optwei,optphi,a0,xi0)

      return
      end subroutine gwpspf

!-----------------------------------------------------------------------

      real(dop) function fgwpspf(optpar)

use global
use channels

use gwplib, only: fgaover,ffgaover
use fgwpspfmod
!use d2tens, only: filldens

      implicit none
      
      integer(long)            :: m,f,e,e1,e2,s,s1,gwpdim2

      complex(dop)                                    :: overlap

      real(dop)                                      :: gxeq,gmom,&
                                                        gwidth,norm,&
                                                        dxi,ixi,epsmat
      real(dop), dimension(optgdof,optdim1), intent(in) :: optpar

      complex(dop), dimension(optdim1,optdim1)          :: gsinv
      complex(dop), dimension(optdim1)                  :: ovl0,wei
      complex(dop), dimension(optdim1,optdim1)          :: gs
      complex(dop), dimension(optgdof,optdim1,1)        :: z
      complex(dop), dimension(optgdof,optgdof,optdim1,1):: ai

      integer(long) :: lwork,lda,info,nneg
      real(dop), dimension(optdim1) :: gsval
      real(dop), dimension(3*optdim1) :: work
      complex(dop), dimension(optdim1,optdim1) :: workc,evek


!-----------------------------------------------------------------------
! get linear parameter of GWPs 
!-----------------------------------------------------------------------
      do e=1,optdim1
         do f=1,optgdof
            dxi = optpar(f,e)
            ixi = dimag(optxi(f,e))
            optxi(f,e) = dcmplx(dxi,ixi)
         enddo
      enddo

!-----------------------------------------------------------------------
! get scalar parameter of GWPs
!-----------------------------------------------------------------------
      do e=1,optdim1
         optphi(e)=0.0d0

         do f=1,optgdof
            gwidth = -dble(optag(f,f,e))
            gxeq = dble(optxi(f,e))/(2.0_dop*gwidth)
            gmom = dimag(optxi(f,e))

            optphi(e)=optphi(e) - gwidth*gxeq**2 &
                       - (0.0_dop,1.0_dop)*gmom*gxeq &
                       - 0.5_dop*dlog(dsqrt(pi/(2.0_dop*dble(gwidth))))
         enddo
      enddo

!-----------------------------------------------------------------------
! get GWP basis overlap matrix
!-----------------------------------------------------------------------
      s = 1
      call diagonal_moments(m,s,s,optgdof,optdim1,optdim1,optag,optag, &
           optxi,optxi,optphi,optphi,gs,ai,z,0,0)

 !     do e=1,optdim1
 !     do e1=e+1,optdim1
 !        print*,e,e1,gs(e,e1)
 !        print*,e1,e,gs(e1,e)
 !     enddo
 !     enddo
!-----------------------------------------------------------------------
! invert overlap matrix of basis (direct routine)
!-----------------------------------------------------------------------
    !  gs2=gs
    !  gsinv=gs2
    !  call zpotrf('L',optdim1,gsinv,optdim1,ierr)
    !  call zpotri('L',optdim1,gsinv,optdim1,ierr)
    !  if (ierr .ne. 0) then
    !     routine='invert_ovr'
    !     write(message,'(a,i5)') 'Error inverting matrix. Error:',ierr
    !     call errormsg
    !  endif
    !  call filldens(gsinv,optdim1) 

      gsval=0.0_dop
      lda = optdim1
      lwork=optdim1*optdim1
      evek = gs
      call zheev('V','U',optdim1,evek,lda,gsval,workc,lwork,work,info)
      if (info .ne. 0) then
         routine = 'invert_ovr'
         message = 'Cannot diagonalize overlap matrix'
         call errormsg
      endif

      nneg = count(gsval .lt. 0.0)
      if (nneg .gt. 0) then
         routine = 'Fgwpspf'
         write(message,'(a,i5,a)') &
            'GBF Overlap matrix has ',nneg,' negative eigenvalues '
         call errormsg
      endif
      nneg = count(gsval .lt. tiny)
      if (nneg .gt. 0) then
         routine = 'Fgwpspf'
         write(message,'(a,i5,a,e8.3)') &
            'GBF Overlap matrix has ',nneg,' eigenvalues less than ',tiny
         call warnmsg1
      endif

!-----------------------------------------------------------------------
! regularize eigenvalues
!-----------------------------------------------------------------------
      epsmat = tiny
      do e=1,optdim1
         if (gsval(e) .lt. 0_dop*epsmat) then
             gsval(e) = epsmat
         else if (gsval(e) .lt. 64.0_dop*epsmat) then
            gsval(e)=gsval(e)+epsmat*exp(-gsval(e)/epsmat)
         endif
      enddo

!-----------------------------------------------------------------------
! form inverse of overlap matrix gsinv
!-----------------------------------------------------------------------
      gsinv = 0.0_dop
      do e=1,optdim1
         do e1=e,optdim1
            do e2=1,optdim1
               gsinv(e1,e)=gsinv(e1,e) +&
                  dconjg(evek(e,e2))*evek(e1,e2)/gsval(e2)
            enddo
         enddo
      enddo
      do e=1,optdim1
         gsinv(e,e)=dble(gsinv(e,e))
         do e1=e+1,optdim1
            gsinv(e,e1)=dconjg(gsinv(e1,e))
         enddo
      enddo

!-----------------------------------------------------------------------
! get overlap of initial WP with GWP basis
!-----------------------------------------------------------------------
      gwpdim2 = 1
      s = 1
      s1=s+1
      call diagonal_moments(m,s,s1,optgdof,optdim1,gwpdim2,optag,a0, &
           optxi,xi0,optphi,phi0,ovl0,ai,z,0,0)

!-----------------------------------------------------------------------
! project initial WP on to GWP basis
!-----------------------------------------------------------------------
      call qvxxzz(gsinv,ovl0,wei,optdim1)
      optwei=dble(wei)

! normalise new WP
      overlap=0.0_dop
      do e=1,optdim1
         do e1=1,optdim1
            overlap=overlap+optwei(e)*gs(e,e1)*optwei(e1)
         enddo
      enddo
      norm=dble(overlap)

      norm=1.0_dop/dsqrt(norm)
      do e=1,optdim1
         optwei(e)=optwei(e)*norm
      enddo

      overlap=0.0_dop
      do e=1,optdim1
         overlap=overlap+optwei(e)*ovl0(e)
      enddo

      fgwpspf = 2.0 - 2.0*dble(overlap)

      return
      end function fgwpspf

!#######################################################################

      subroutine dgwpspf(optpar,doptpar,nvar)

use global

      implicit none

      integer(long)                                 :: n
      integer(long), intent(in)                     :: nvar
      real(dop), dimension(nvar), intent(in)        :: optpar
      real(dop), dimension(nvar), intent(out)       :: doptpar
      real(dop)                                     :: err,h
      real(dop), dimension(nvar*2)                  :: work

      do n=1,nvar
         h=0.001
         doptpar(n)=dfnummc(fgwpspf,optpar,nvar,n,h,err,work)
      enddo

      return
      end subroutine dgwpspf

!#######################################################################
!
! iprn is a print flag
!      0 work silently   1 print 
!
      SUBROUTINE cgmin(p,n,ftol,ITMAX,fret,func,dfunc,ilog,lb,iprn)

use f1com1

      INTEGER(long), intent(in) :: n,ilog,iprn
      INTEGER(long) :: iter
      INTEGER(long), intent(in) :: ITMAX
      REAL(dop), intent(in) :: ftol,lb
      REAL(dop), intent(inout) :: fret
      REAL(dop), external :: func
      external :: dfunc
      REAL(dop), dimension(n), intent(inout) :: p
      REAL(dop), parameter :: EPS=1.e-10
      REAL(dop)  :: maxxi
      INTEGER(long) :: its,j
      REAL(dop) :: dgg,fp,gam,gg
      REAL(dop), dimension(NMAX) :: g,h,xi

      if (ITMAX .eq. 0) return
      if (n .gt. nmax) then
         write(ilog,'(a,i5)') 'Redimension NMAX in cgmin. Minimum :',n
         stop
      endif
      if (iprn .eq. 1) then
         write(ilog,'(a,i5,5x,a,e16.8)') 'Niter = ',ITMAX,'Tol = ',ftol
      endif
      fp=func(p)
      call dfunc(p,xi,n)
      maxxi=0.0
      do 6 j=1,n
         maxxi=max(maxxi,abs(xi(j)))
 6    continue
      if(iprn .eq. 1 .and. maxxi.le.ftol) then
         write(ilog,'(a)') 'Zero gradients at start of Optimisation '
         return
      endif

      do 11 j=1,n
        g(j)=-xi(j)
        h(j)=g(j)
        xi(j)=h(j)
11    continue
      do 14 its=1,ITMAX
        iter=its
        if (iprn .eq. 1) then
           write(ilog,'(a,i7)') '+++++++++++ calculating iteration: ',iter
        endif
        
        call linmincg(p,xi,n,fret,func)
        if (fret .lt. lb) then
           if (iprn .eq. 1) then
              write(ilog,'(a)') 'Under lower bounds for function '
           endif

           return
        endif

        call dfunc(p,xi,n)
        maxxi=0.0
        do 21 j=1,n
           maxxi=max(maxxi,abs(xi(j)))
21      continue
        if(maxxi.le.ftol) then
           if (iprn .eq. 1) then
              write(ilog,'(a)') 'Optimisation converged '
           endif

           return
        endif

        if(2.*abs(fret-fp).le.ftol*(abs(fret)+abs(fp)+EPS)) then
           if (iprn .eq. 1) then
              write(ilog,'(/,2a)') &
          'Optimisation: gradients above tolerance at convergence of', &
          ' function'
              write(ilog,'(a,2e16.8)') 'Max. Grad/ tolerance : ', &
                 maxxi,ftol
              !!write(ilog,'(a)') &
          !!'Try different optimisation method? e.g. optimiser = simplex'

           endif

           return
        endif

        fp=func(p)

        gg=0.
        dgg=0.
        do 12 j=1,n
          gg=gg+g(j)**2
          dgg=dgg+(xi(j)+g(j))*xi(j)
12      continue

        if(gg.eq.0.)return
        gam=dgg/gg
        do 13 j=1,n
          g(j)=-xi(j)
          h(j)=g(j)+gam*h(j)
          xi(j)=h(j)
13      continue
        if (iprn .eq. 1) then
           write(ilog,'(a,e16.8)') '  function value = ',fret
        endif
        call myflush(ilog)
14    continue
      if (iprn .eq. 1) then
         write(ilog,'(a)') 'Maximum optimisation iterations exceeded'
      endif

      return
      END subroutine cgmin

!#######################################################################
!
! LINMIN
! line minimiser
!
! p : point (coordinates)
! xi : gradient at p
! n : no. of coordinates
! fret: returned function
! func: routine for calculating function to be minimised
!
!#######################################################################

      SUBROUTINE linmincg(p,xi,n,fret,func)

use f1com1

      INTEGER(long), intent(in) :: n
      REAL(dop), intent(inout) :: fret
      REAL(dop), dimension(n), intent(inout) :: p,xi
      REAL(dop), parameter :: TOL=1.e-4
      REAL(dop), external :: func
      REAL(dop) :: ratio
      INTEGER(long) :: j
      REAL(dop) :: ax,bx,fa,fb,fx,xmin,xx

      if (n .gt. nmax) then
         write(6,'(a,i5)') 'Redimension NMAX in linmin. Minimum :',n
         write(6,'(a)') 'Check dimensions in f1dim'
         stop
      endif

      ax=0.
      fa = func(p)
!
! xx is step size. New func is calculated at pcom + xx*xicom
! Get scale of coordinates. Choose xx so that xx*xicom = 0.1*pcom
! 
      xx=1.
      ncom=n
      do 11 j=1,n
        pcom(j)=p(j)
        xicom(j)=xi(j)
        if(pcom(j) .eq. 0.0) then
            ratio=abs(0.1/xicom(j))
        else
            ratio=abs(0.1*pcom(j)/xicom(j))
        endif
        xx = min(xx,ratio)

11    continue

! check that fx is not too large
 8    fx = f1dim(xx,func)
      if (abs(fx) .gt. abs(10.0*fa)) then
         xx = 0.5*xx
         go to 8
      endif

      call mnbrakcg(ax,xx,bx,fa,fx,fb,f1dim,func)
      fret=brentcg(ax,xx,bx,f1dim,TOL,xmin,func)
      do 12 j=1,n
        xi(j)=xmin*xi(j)
        p(j)=p(j)+xi(j)
12    continue

      return
      END subroutine linmincg

!#######################################################################

      real(dop) FUNCTION brentcg(ax,bx,cx,f1dim,tol,xmin,func)

      REAL(dop), intent(in) :: ax,bx,cx,tol
      REAL(dop), intent(out) :: xmin
      REAL(dop), external :: f1dim,func
      real(dop), parameter :: CGOLD=.3819660,ZEPS=1.0e-10
      integer(long), parameter :: ITMAX=100
      INTEGER(long) :: iter
      REAL(dop) :: a,b,d,e,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm 

      a=min(ax,cx)
      b=max(ax,cx)
      v=bx
      w=v
      x=v
      d=0.
      e=0.
      fx=f1dim(x,func)
      fv=fx
      fw=fx
      do 11 iter=1,ITMAX
        xm=0.5*(a+b)
        tol1=tol*abs(x)+ZEPS
        tol2=2.*tol1
        if(abs(x-xm).le.(tol2-.5*(b-a))) goto 3
        if(abs(e).gt.tol1) then
          r=(x-w)*(fx-fv)
          q=(x-v)*(fx-fw)
          p=(x-v)*q-(x-w)*r
          q=2.*(q-r)
          if(q.gt.0.) p=-p
          q=abs(q)
          etemp=e
          e=d
          if(abs(p).ge.abs(.5*q*etemp).or.p.le.q*(a-x).or.p.ge.q*(b-x)) goto 1
          d=p/q
          u=x+d
          if(u-a.lt.tol2 .or. b-u.lt.tol2) d=sign(tol1,xm-x)
          goto 2
        endif
1       if(x.ge.xm) then
          e=a-x
        else
          e=b-x
        endif
        d=CGOLD*e
2       if(abs(d).ge.tol1) then
          u=x+d
        else
          u=x+sign(tol1,d)
        endif
        fu=f1dim(u,func)
        if(fu.le.fx) then
          if(u.ge.x) then
            a=x
          else
            b=x
          endif
          v=w
          fv=fw
          w=x
          fw=fx
          x=u
          fx=fu
        else
          if(u.lt.x) then
            a=u
          else
            b=u
          endif
          if(fu.le.fw .or. w.eq.x) then
            v=w
            fv=fw
            w=u
            fw=fu
          else if(fu.le.fv .or. v.eq.x .or. v.eq.w) then
            v=u
            fv=fu
          endif
        endif
11    continue
      stop 'brent exceed maximum iterations'
3     xmin=x
      brentcg=fx

      return
      END function brentcg

!#######################################################################

      SUBROUTINE mnbrakcg(ax,bx,cx,fa,fb,fc,f1dim,func)

      REAL(dop), intent(inout) :: ax,bx,cx,fa,fb,fc
      REAL(dop), external :: func,f1dim
      real(dop), PARAMETER :: GOLD=1.618034, GLIMIT=100., TINY=1.e-20
      REAL(dop) :: dum,fu,q,r,u,ulim

      if(fb.gt.fa)then
        dum=ax
        ax=bx
        bx=dum
        dum=fb
        fb=fa
        fa=dum
      endif
      cx=bx+GOLD*(bx-ax)
      fc=f1dim(cx,func)
1     if(fb.ge.fc)then
        r=(bx-ax)*(fb-fc)
        q=(bx-cx)*(fb-fa)
        u=bx-((bx-cx)*q-(bx-ax)*r)/(2.*sign(max(abs(q-r),TINY),q-r))
        ulim=bx+GLIMIT*(cx-bx)
        if((bx-u)*(u-cx).gt.0.)then
          fu=f1dim(u,func)
          if(fu.lt.fc)then
            ax=bx
            fa=fb
            bx=u
            fb=fu
            return
          else if(fu.gt.fb)then
            cx=u
            fc=fu
            return
          endif
          u=cx+GOLD*(cx-bx)
          fu=f1dim(u,func)
        else if((cx-u)*(u-ulim).gt.0.)then
          fu=f1dim(u,func)
          if(fu.lt.fc)then
            bx=cx
            cx=u
            u=cx+GOLD*(cx-bx)
            fb=fc
            fc=fu
            fu=f1dim(u,func)
          endif
        else if((u-ulim)*(ulim-cx).ge.0.)then
          u=ulim
          fu=f1dim(u,func)
        else
          u=cx+GOLD*(cx-bx)
          fu=f1dim(u,func)
        endif
        ax=bx
        bx=cx
        cx=u
        fa=fb
        fb=fc
        fc=fu
        goto 1
      endif

      return
      END subroutine mnbrakcg

!#######################################################################

      real(dop) FUNCTION f1dim(x,func)

use f1com1

      REAL(dop), external :: func
      REAL(dop), intent(in) :: x
!U    USES func
      INTEGER(long) :: j
      REAL(dop), dimension(NMAX) :: xt

      do 11 j=1,ncom
        xt(j)=pcom(j)+x*xicom(j)
11    continue

      f1dim=func(xt)

      return
      END function f1dim

!#######################################################################
!
! numerical derivatives

      real(dop) function dfnummc(func,x,nvar,n,h,err,work)

      integer(long), parameter :: ntab=10
      real(dop), parameter     :: con=1.4_dop,con2=con*con, &
                                  big=1.0e30_dop,safe=2.0_dop

      integer(long)                             :: i,j
      integer(long), intent(in)                 :: n,nvar
      real(dop), dimension(nvar,2), intent(out) :: work
      real(dop), dimension(nvar), intent(in)    :: x
      real(dop), dimension(ntab,ntab)           :: a
      real(dop)                                 :: errt,fac,hh
      real(dop), intent(in)                     :: h
      real(dop), intent(out)                    :: err
      real(dop), external                       :: func

      dfnummc=0.0_dop
      if (h.eq.0) stop 'h must be nonzero in dfridr'
      do i=1,nvar
         work(i,1)=x(i)
         work(i,2)=x(i)
      enddo

      hh=h
      work(n,1)=x(n)+hh
      work(n,2)=x(n)-hh
      a(1,1)=(func(work(1,1))-func(work(1,2)))
      a(1,1)=a(1,1)/(2.0*hh)
      err=big
      do i=2,ntab
         hh=hh/con
         work(n,1)=x(n)+hh
         work(n,2)=x(n)-hh
         a(1,i)=func(work(1,1)) - func(work(1,2))
         a(1,i)=a(1,i)/(2.0*hh)
         fac=con2
         do j=2,i
            a(j,i)=(a(j-1,i)*fac-a(j-1,i-1))/(fac-1.0)
            fac=con2*fac
            errt=max(abs(a(j,i)-a(j-1,i)),abs(a(j,i)-a(j-1,i-1)))
            if (errt .le. err) then
               err=errt
               dfnummc=a(j,i)
            endif
         enddo
         if(abs(a(i,i)-a(i-1,i-1)) .ge. safe*err) return
      enddo

      return
      end function dfnummc

      end module gwpspfmod

