!***********************************************************************
!
!                 MCTDH  subroutine   LOGINWF
!
!***********************************************************************

      module loginwfmod

use decimal, only: long, dop
use global
use dglobal
use globmemmod
use dvrdatmod
use griddatmod
use psidef
use runinwfmod
use operdef
use dirdyn
use channels
use logdat
use iorst, only: rstinfo
use rdopermod, only: operinfo

      implicit none
      private
      public :: loginwf,loginwfdat
      contains

!#######################################################################
!  LOGINWFDAT  reads the restart file and writes information on the 
!  wavefunction to the log file
!#######################################################################

      subroutine loginwfdat

      implicit none

      logical(kind=4)           :: lerr,linwf
      character(len=c5)         :: filename
      integer(long)             :: ilbl,chkdvr,chkgrd,chkpsi,chkprp

!-----------------------------------------------------------------------
! open restart file
!-----------------------------------------------------------------------
      filename=rname(1:rlaenge)//'/restart'
      ilbl=index(filename,' ')-1
      open(irst,file=filename,form='unformatted',status='old',err=900)

!-----------------------------------------------------------------------
! get memory sizes from oper file
!-----------------------------------------------------------------------
      call rdmemdim(irst)

!-----------------------------------------------------------------------
! Allocate mememory and read system / oper information
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      if (ldd .or. lgwptrj .or. lddmctdh .or. lddtrans .or. ltraj) then
         call alloc_dirdyn
         call alloc_dddb
         call alloc_dercp
      endif
      call alloc_psidef
      if (ldynspf) call alloc_dynwf
      call alloc_runinwfmod

!-----------------------------------------------------------------------
! read restart file
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkprp=0
      call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
      close(irst)

!-----------------------------------------------------------------------
! Read data from oper file
!-----------------------------------------------------------------------
      filename=oname(1:olaenge)//'/oper'
      ilbl=index(filename,' ')-1
      open(ioper,file=filename,form='unformatted',status='old',err=900)
      chkdvr=2
      chkgrd=2
      call operinfo(lerr,chkdvr,chkgrd)
      close(ioper)

!-----------------------------------------------------------------------
! log data
!-----------------------------------------------------------------------
      call loginwf

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_dirdyn
      call dealloc_dddb
      call dealloc_psidef
      call dealloc_dynwf
      call dealloc_runinwfmod

      return

  900 continue
      routine='Loginwfdat (loginwfmod.f90)'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop

      end subroutine loginwfdat

!#######################################################################

!#######################################################################

      subroutine loginwf

      implicit none

      integer(long)       :: f,ff,p1,s,p,ilbl,ipot,pop,m,i,isym,slz,&
                             isleg,sl0,typ,l0,m0,mmin,mmax,dm,nf,np,nm,&
                             gdof,shell,subshell,mshell,npnts,mfact,ntmp,&
                             npntsold,idx
      real(dop)           :: gxeq,rwidth,cwidth,gmom,gwidth,xi,xf,gshift,sigma
      logical(kind=4)     :: first
      character(len=22)   :: string

      if (dentype .eq. 1) then
         write(ilog,915)
      else
         write(ilog,900)
      endif

      do p=1,npacket
         if (lmult) then
            if(nstate .eq. npacket) then
               write(ilog,*)
            else
               p1=(nstate/npacket)*(p-1)+1
               write(ilog,'(/a,i5)') 'Initial State = ', &
                    isbaspar(4,feb,p1) 
            endif
         endif

         do s=1,nstate/npacket
            if (npacket .gt. 1 .and. nstate .gt. 1) then
               write(ilog,'(/,a,i5,a,i5,a)') & 
                  '*** SPFs for Packet = ',p,' , state = ',s,' ***'
            else if (npacket .gt. 1 .and. nstate .eq. 1) then
               write(ilog,'(/,a,i5,a)')  &
                  '*** SPFs for Packet = ',p,' ***'
            else if (npacket .eq. 1 .and. nstate .gt. 1) then
               write(ilog,'(/,a,i5,a)')  &
                  '*** SPFs for State = ',s,' ***'
            endif

            p1=(nstate/npacket)*(p-1)+s
            do ff=1,ndof
               f = ff
! do not output if not used
               if (dofspf(f) .eq. 0) go to 100

! if elcont basis and s denotes a continuum state, use appropriate dof for info
               if ((f .eq. feb) .and. (febcont .gt. 0) .and. &
                                           (s .gt. ipbaspar(2,f))) f=febcont

               if (sbasis(f) .eq. 0) then
                  if(mpb.gt.0) then
                     write(ilog,901) f,modelabel(f)
                  else if (nstate .gt. 1 .and. isbaspar(4,f,p1) .eq. s) then
                     write(ilog,901) f,modelabel(f),' Initial state '
                  else if (nstate .eq. 1) then
                     write(ilog,901) f,modelabel(f),'initial state:', &
                                  isbaspar(4,f,p1)
                  else 
                     write(ilog,901) f,modelabel(f)
                  endif

               else if (sbasis(f) .eq. 1) then
                  gxeq=rsbaspar(1,f,p1)
                  gmom=rsbaspar(2,f,p1)
                  rwidth=rsbaspar(3,f,p1)
                  cwidth=rsbaspar(4,f,p1)

                  typ=isbaspar(1,f,p1)
                  pop=isbaspar(4,f,p1)
                  if (typ .eq. 0) then
                     string=' gauss '
                     ilbl=7
                  else if (typ .eq. 1) then
                     string='gs-odd '
                     ilbl=7
                  else if (typ .eq. 2) then
                     string='gs-even'
                     ilbl=7
                  endif

                  if (abs(cwidth -0.0) .lt. tiny) then
                     write(ilog,902) f,modelabel(f),string(1:ilbl), &
                     pop,' p0 =',gmom,' x0 =',gxeq,' dx =',rwidth
                  else
                     write(ilog,903) f,modelabel(f),string(1:ilbl), &
                     pop,' p0 =',gmom,' x0 =',gxeq, &
                        ' dx =',rwidth,',',cwidth
                  endif

               else if (sbasis(f) .eq. 2) then
                  slz=isbaspar(1,f,p1)
                  sl0=isbaspar(2,f,p1)
                  isleg=isbaspar(3,f,p1)
                  pop=isbaspar(4,f,p1)
                  write(ilog,904) f,modelabel(f),'Leg', &
                  pop,'slz =',slz,'      sl0 =',sl0,'   isleg =',isleg

               else if (sbasis(f) .eq. 3) then
                  ipot=isbaspar(1,f,p1)
                  pop=isbaspar(4,f,p1)
                  isym=isbaspar(6,f,p1)
                  write(ilog,911) f,modelabel(f),'eigenf  pop =', &
                     pop,'  sym =',isym,'ham =',ipot,trim(hamlab(ipot))

               else if (sbasis(f) .eq.-1) then
                  continue
               else if (sbasis(f) .eq. 4) then
                  l0=isbaspar(1,f,p1)
                  m0=isbaspar(2,f,p1)
                  write(ilog,905) f,modelabel(f),'sphFBR', &
                      ' l0 =',l0,'       m0 =',m0

               else if (sbasis(f) .eq. 5) then
                  sl0=isbaspar(1,f,p1)
                  isleg=isbaspar(2,f,p1)
                  write(ilog,905) f,modelabel(f),'KLeg', &
                      'sl0 =',sl0,'    isleg =', isleg

               else if (sbasis(f) .eq. 6) then
                  slz =isbaspar(1,f,p1)
                  mmin=isbaspar(2,f,p1)
                  mmax=isbaspar(3,f,p1)
                  dm  =isbaspar(5,f,p1)
                  write(ilog,905) f,modelabel(f),'K', &
                      'slz =',slz,'     mmin =', mmin,  &
                      '  mmax =',mmax,'   dm =', dm 

               else if (sbasis(f) .eq. 7) then
                  l0=isbaspar(1,f,p1)
                  write(ilog,905) f,modelabel(f),'map',&
                      ' f-map =',l0

               else if (sbasis(f) .eq. 8) then
                  sl0=isbaspar(1,f,p1)
                  isleg=isbaspar(2,f,p1)
                  write(ilog,905) f,modelabel(f),'Wigner', &
                      'sl0 =',sl0,'    isleg =', isleg

               else if (sbasis(f) .eq. 9) then
                  typ=isbaspar(2,f,p1)
                  gxeq   = rsbaspar(1,f,p1)
                  if (rsbaspar(6,f,p1) .gt. 0) then
!                    gwidth = dcmplx(rsbaspar(6,f,p1),0.0_dop)
                     gwidth = rsbaspar(6,f,p1)
                  else
!                    gwidth = dcmplx(rsbaspar(3,f,p1),rsbaspar(4,f,p1))
                     gwidth = rsbaspar(3,f,p1)
                  endif
                  sigma = 1.0_dop/gwidth
                  gwidth = -0.25_dop*gwidth**2
             
                  if (typ .eq. 4) then
                     gshift = 2.0_dop*dlog(rsbaspar(5,f,p1))/gwidth
                     gshift = dsqrt(gshift)
                  else
                     gshift = rsbaspar(5,f,p1)
                  endif  
                  nf = dimf(f,s)-1
                  nm = nf/2
                  np = nf/2 + mod(nf,2)
                  
                  xi = gxeq - nm*gshift
                  xf = gxeq + np*gshift
                     
                  write(ilog,912) f,modelabel(f),'GWP', &
                    'sigma =',sigma, &
                    'xi =',xi,'xf =',xf,'dx =',gshift
!                  dm  = isbaspar(5,f,p1)
!                  write(ilog,*) 'dm =',dm

               else if (sbasis(f) .eq. 10) then
                  write(ilog,913) f, modelabel(f), 'Euclid'

! End of modification (Benjamin, 14 Feb 2007)
               else if (sbasis(f) .eq. 11) then
                  write(ilog,914) f, modelabel(f),&
                        'Coordinate driven by function: ',hamlab(isbaspar(1,f,p1))

               else 
                  write(ilog,910) f, modelabel(f), &
                      'No Log Output for basis type : ',sbasis(f)

               endif
 100           continue
            enddo
         enddo
      enddo

      if (lgwp) then
         if (gwptype .eq. 0) then
            write(ilog,'(/,a)') '*** GWPs in width matrix form ***'
         else if (gwptype .eq. 1) then
            write(ilog,'(/,a)') '*** GWPs in PZ form ***'
         else if (gwptype .eq. 2) then
            write(ilog,'(/,a)') '*** GWPs in diagonal form ***'
         else if (gwptype .eq. 3) then
            write(ilog,'(/,a)') '*** Frozen GWPs (diagonal) ***'
         else if (gwptype .eq. 4) then
            write(ilog,'(/,a)') '*** Frozen GWPs (non-diagonal) ***'
         endif
         if (gwpphase .eq. 0) then
            write(ilog,'(/,a)') '    GWP scalar parameters fixed '
         else if (gwpphase .eq. 1) then
            write(ilog,'(/,a)') '    GWPs remain normalised '
         else if (gwpphase .eq. 2) then
            write(ilog,'(/,a)') '    GWPs remain normalised '
            write(ilog,'(a)') &
                '    GWPs phase includes uncorrelated energy'
         endif

! work out which shells are filled
         idx = 0
         first=.true.
         do s=1,nstate
            do m=1,nmode
               f=spfdof(1,m)
               if (sbasis(f) .eq. 9 .and. isbaspar(1,f,s) .eq. 3) then 
                  if (first) then
                     write(ilog,'(/,a)') &
                  '**** Initial GWP arranged in shells **** '
                     write(ilog,'(a)') 'mode state   shell subshell'
                     first=.false.
                  endif

                  shell=0 
                  subshell=0
                  gdof=nspfdof(m)
                  npnts=1
                  npntsold = 0
  120             continue
                  if (dim(m,s) .eq. npnts) then
                     write(ilog,'(2i3,6x,2i5,a)') &
                          m,s,shell,subshell,'   Filled'
                  else if (dim(m,s) .lt. npnts) then
                     write(ilog,'(2i3,6x,2i5,a,i5,a,i5,a)') &
                        m,s,shell,subshell,&
                      '   Partially Filled. Filled =',npnts, &
                      '   (Nspf = ',dim(m,s),')'
                  else if (dim(m,s) .gt. npnts) then
                     if (shell .eq. 0) then
                        shell=1
                        subshell=1
                     else if (subshell .lt. gdof) then 
                        subshell=subshell+1
                     else if (subshell .eq. gdof) then 
                        shell=shell+1
                        subshell=1
                     endif

                     mshell=2*shell
                     mfact=1
                     do i=gdof-subshell+1,gdof
                        mfact=mfact*i
                     enddo
                     do i=1,subshell
                        mfact=mfact/i
                     enddo
                     
                     ntmp=mshell**subshell - (mshell-2)**subshell
                     npntsold = npnts
                     npnts=npnts + mfact*ntmp
                     idx = idx + 1
                     go to 120
                  endif
               endif
            enddo
         enddo
      endif

      if (dentype .eq. 1 .and. dlstate .ne. drstate) then
         write(ilog,'(/,a)') &
        '*** Initial density matrix is a coherent superposition ***'
         write(ilog,'(a,i5,i5)') &
        'States populated 50/50 :',dlstate,drstate
      endif

      return

 900  format(//'****** Initial SPF Functions *********')
 901  format(i3,2x,a9,1x,a14,3x,i5)
 902  format(i3,2x,a9,1x,a7,i3,3(1x,a,f10.5))
 903  format(i3,2x,a9,1x,a7,i3,4(1x,a,f10.5))
 904  format(i3,2x,a9,1x,a6,i3,4(1x,a,i4))
 905  format(i3,2x,a9,1x,a6,3x,4(1x,a,i4))
 910  format(i3,2x,a9,1x,a6,3x,i5,1x,a)
 911  format(i3,2x,a9,1x,a13,i3,a7,i3,5x,a6,i4,2x,a)
 912  format(i3,2x,a9,1x,a3,4(1x,a,f12.4))
 913  format(i3,2x,a9) 
 914  format(i3,2x,a9,1x,a31,a15) 
 915  format(//'****** Initial SPDO Functions *********')

      end subroutine loginwf


      end module loginwfmod
