!***********************************************************************
!
!               MCTDH   module OPWF
!
! This module enables an operator to act on a wavefunction 
!
!  opwfex: operator x psi  for numerically exact WF
!  opwf  : operator x psi  for MCTDH WF, non-iterative procedure.
!  opwfit: operator x psi  for MCTDH WF, iterative procedure.
!
! contains: operate,opwfex,opwfit,sumdqdz,chkphase,phasespf,opwf,d2opwf,detspfs,
!           addd2s1,addd2s2,addd2s3,d1opwfit,d1opwfit,d1opwfit,d1tracetens,d2brakettens
!           d1calchaloch2,d2calchunaloch2,d2calchaloch2
!           d2phihphi1m,xlocphihphi,gh2hteil1,gh2hteil,phihunphi,phihphi1m,phihphi
!***********************************************************************
      module opwfmod

use global
use decimal,         only: long, dop
use globmemmod
use logdat
use mmlib,           only: mmaxzz, mqxczz, mmxtzza, mmaxzzh, mmtczz     
use mtlib,           only: qtxxzz, dtxxzzo, qttxzz
use ttlib,           only: d1ttxx, d2ttxx1221
use xvlib,           only: xvixdzo, vvaxzz, xqxxzzo, xvxxzzo, xvxxzza,&
                           xvxxzzr
use schmidtorthomod, only: schmidtortho
use rmlib,           only: rm1hxxxzz, rmqxxxzz
use op1lib,          only: cpvxz, zerovxz, normvxz, cpvcz, cpmaz, cpmxz, cpqxz, &
                           trmmaxzz, trhxz
use op2lib,          only: subvxxzo1, addmxazo1, swapsxxi, addvxxzo
use timing

use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod
use runinwfmod
use runpropmod, only: epsmat
use datenmod
use channels

use gwplib, only: gwp_param,gwp_overlap
use gh_elementsmod
use hphimod, only: hphi,hunphi,hunphi1ms,hphi1mk
use phihphimod, only: phihphi,phihunphi
use calchamod, only: calcha,calchuna
use mfieldsmod, only: mfields
use hlochphimod, only: hlochphi
use addhunphimod, only: addhunphi
use densitymod

use trafonatmod, only: trafonat
use tdkoeffs, only: calctdk
use projectmod, only: project
use dicht1phimod, only: dicht1phi
use d2calchamod, only: d2calcha,d2calchuna,d2calchaloch2,d2calchunaloch2
use evaluate, only: prefactor
use d1hphimod, only: d1hphi,d1hunphi
use d1phihphimod, only: d1phihphi,d1phihunphi
use psiutil, only: d1regular
use trafonatmod, only: d1trafonat,d2trafonat
use d1calchamod, only: d1calchaloch2
use d1lib, only: trspdoc
use d2tens,      only: chkherm, mkherm, chkherm2, mkherm2

use wftree
use multilayer
use dterms

use inputmlmod
use mlanalysis, only: croco_tree

      implicit none
      private
      public :: operate, wkoperate, opwfex, d2opwf
      contains

!***********************************************************************
!     SUBROUTINE OPERATE                                                
!                                                                       
! This routine acts as caller for all routines belonging to operations  
! on initial wavefunctions or density operators                         
!                                                                       
! 12/2001 CC                                                            
!                                                                       
! nham is Hamiltonian operator
! lopnorm = .true. WF is normlaised
!***********************************************************************

      subroutine operate(psi,nham,lopnorm)

      implicit none

      complex(dop), dimension(dgldim), intent(inout) :: psi
      complex(dop), dimension(dgldim)                :: oldpsi
      integer(long)                                  :: ilbl,m,f,k,h,s,kshift
      integer(long), external                        :: slen
      integer(long), intent(in)                      :: nham
      logical(kind=4), intent(in)                    :: lopnorm
      logical(kind=4)                                :: ltest

      integer(long), save                              :: tid=0 ! timer ID

      call get_timer(tid, 'Operate_WF')
      call start_timer(tid)

      routine='operate'
      if(.not.nodiag(nham)) then
         ilbl=index(hamlab(nham),' ')-1
         message='The Hamiltonian '//hamlab(nham)(1:ilbl)// &
              ' must be of "nodiag" type for "operate".'
         call errormsg
      endif

! --- CHECK FOR UNIT OPERATOR WHICH IS USED IF loporth IS SET ---
      ltest=.true.
      kshift=0
      do k=zham(nham),zham(nham)+kzahl(nham)-1
         if( abs(koeff(k)-1.234d-9) .gt. 1.d-19 .or. &
                                    ki(k).ne.kf(k) ) goto 20
         ltest=.true.
         do m = 1,nmode
            f=spfdof(1,m)
            h=hamilton(f,k)
            if(htmtype(h).ne.1) then
               ltest=.false.  ! non-unit operator found
               goto 20
            endif
         enddo  ! m
         kshift=k
         if(ltest) goto 30  ! unit operator found
  20  enddo ! k

      if(loportho .and. .not.lexact) then
         routine='opwfit on geninwf/opwf.F'
         message='There must be a operator line contaning a unit '// &
                 'operator                                       '// &
                 'when operate_ortho is set.                     '// &
               'Add the line " 1.234d-9 | 1 " to the operator tableau.'
         call errormsg
      else
         goto 50
      endif

  30  koeff(kshift)=0.d0
      if(nstate.gt.1 .and. k+nstate.le.zham(nham)+kzahl(nham)) then
         do s=1,nstate-1
            koeff(kshift+s)=0.d0
         enddo
      endif

      if(loportho .and. .not.lexact) then
         write(ilog,'(2a,i5)') 'operate_ortho is set!', &
            ' unit operator found at k =', kshift
      endif

   50 continue
!-----------------------------------------------------------------------
! calculate time-dependent coefficients
!-----------------------------------------------------------------------
      if (ltdh(nham)) call calctdk(tinit,nham,psi)

      operatenorm=1.0_dop

!.....Copy psi to psiold. psiold will remain unchanged.
      call cpvxz(psi,oldpsi,dgldim)

! exact wavefunction
      if (psitype .eq. 1) then
         write(ilog,'(/,4a)') '*** Operating on numerically exact', &
              ' initial wavepacket Using: ',trim(hamlab(nham)),' ***'
         call myflush(ilog)
         call opwfex(oldpsi,psi,nham,lopnorm,.false.)

      elseif(psitype .eq. 5) then
         if (dentype .eq. 0) then
            write(ilog,'(/,3a)') &
               '*** Operating on ML-MCTDH initial wavepacket. Using: ', &
                    trim(hamlab(nham)),' ***'
            call myflush(ilog)
            call mloperate(oldpsi,psi,nham,inwfopnit,inwfoptol,kshift)
         endif

! MCTDH wavefunction
      else
         if (dentype .eq. 0) then
            write(ilog,'(/,3a)') &
            '*** Operating on MCTDH form initial wavepacket Using: ', &
                 trim(hamlab(nham)),' ***'
            call myflush(ilog)

! Do opwf only, if lnodirect is false (default).
! opwf works correctly only for nstate=1.
            if (nstate.eq.1 .and. .not.lnodirect)  &
                 call opwf(oldpsi,psi,nham,lopnorm)
            call opwfit(oldpsi,psi,nham,inwfopnit,inwfoptol,lopnorm,kshift)

         else if (dentype.eq.1) then
            call d1opwfit(oldpsi,psi,nham,inwfopnit,&
                 inwfoptol,lopnorm)
!            call cpvxz(oldpsi(adim+1),psi(adim+1),totphidim)
            call cpvxz(psi,oldpsi,dgldim)
            call cpvxz(psi,mc(mcintpsi),dgldim)

         else if (dentype .eq. 2) then
            call d2opwfit(oldpsi,psi,nham,inwfopnit,&
                 inwfoptol,lopnorm)
!            call cpvxz(oldpsi(adim+1),psi(adim+1),totphidim)
         else
            message = 'dentype = 0,1,2 only possible!'
            call errormsg
         endif
      endif

      call stop_timer(tid)

      return
      end subroutine operate      

!***********************************************************************
!
!                            SUBROUTINE OPWFEX
!
! An operator acts on a vector. The initial vector is represented in
! a "numerically exact" form.
!
! INPUT:
! psiold: original vector
! nham:   No. of operator to be used.
!
! OUTPUT:
! psinew: operator x psiold. Vector is normalised if lopnorm=.true.
!
!***********************************************************************

      subroutine opwfex(psiold,psinew,nham,lopnorm,swtch)

      implicit none

      integer(long)             :: s,hktype,k,s1,k1,k2
      integer(long), intent(in) :: nham
      real(dop)                 :: norm
      logical(kind=4), intent(in) :: lopnorm,swtch

      complex(dop)                                 :: ovl
      complex(dop), dimension(dgldim), intent(in)  :: psiold
      complex(dop), dimension(dgldim), intent(out) :: psinew
      complex(dop), dimension(dgldim)              :: tmphpsi


! --- INITIALISE VARIABLES ---
      call zerovxz(psinew,dgldim)

! --- CALCULATE SEPARABLE OPERATOR ACTION ---

      do s=1,nstate
         hktype=0
         call hunphi1ms(psiold(zetf(1,s)),psinew(zetf(1,s)),&
              1,s,dim(1,s),subdim(1),hktype,nham)
      enddo

!-----------------------------------------------------------------------
! If multi-state calculation, correlation hamiltonian couples states
!-----------------------------------------------------------------------
      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
      do k=k1,k2
         s1=ki(k)
         s=kf(k)
         call hphi1mk(psiold(zetf(1,s1)),tmphpsi,1,k,dim(1,s1),subdim(1))

         if (hamilton(fkoe,k) .eq. 1) then
            call addvxxzo(tmphpsi,psinew(zetf(1,s)),subdim(1))
         else
            call xvxxzza(koeff(k),tmphpsi,psinew(zetf(1,s)),subdim(1))
         endif

      enddo

      if(swtch) then   ! this is for crosscorr
         do s=1,nstate
            psinew(zpsi(s))=psiold(zpsi(s))
         enddo
         return
      endif

      if (loportho) then
         call vvaxzz(psiold(zetf(1,1)),psinew(zetf(1,1)),ovl,totphidim)
         call xvxxzzr(ovl,psiold(zetf(1,1)),psinew(zetf(1,1)),totphidim)
         write(ilog,'(a,1p2e13.6,a)') &
               ' operate_ortho is set. dshift = (', ovl,')'
      endif

!-----------------------------------------------------------------------
! now normalise new wavefunction
!-----------------------------------------------------------------------
      call normvxz(psinew(zetf(1,1)),norm,totphidim)
      if (lopnorm) call xvixdzo(norm,psinew(zetf(1,1)),totphidim)
      write(ilog,'(1x,a,1pe16.9,3x,a)')&
           'Operate-Norm = ',norm, '(exact WF)'
      operatenorm = operatenorm*norm
      call myflush(ilog)

!-----------------------------------------------------------------------
! write "MCTDH expansion coefficients" for new wavefunction
!-----------------------------------------------------------------------
      do s=1,nstate
         psinew(zpsi(s))=1.0_dop
      enddo

      call chkphase(psiold,psinew)

      return
      end subroutine opwfex

!***********************************************************************
!
!                            SUBROUTINE OPWFIT
!
! An operator acts on a vector. The initial and final vectors are
! in MCTDH form.
!
! INPUT:
! psiold: original vector
! nham:   No. of operator to be used.
!
! OUTPUT:
! psinew: operator x psiold. Vector is normalised.
!
!***********************************************************************

      subroutine opwfit(psiold,psinew,nham,wfopnit,wfoptol,lopnorm,kshift)

      implicit none

      integer(long)                                  :: s,hktype,ktype,&
                                                        ierr,zeig,m,nit,nz
      integer(long)                                  :: modgh
      integer(long), intent(in)                      :: nham,wfopnit,kshift
      real(dop)                                      :: norm,error,sum,sum1
      real(dop), intent(in)                          :: wfoptol
      real(dop)                                      :: time
      complex(dop)                                   :: dshift
      complex(dop), dimension(dgldim), intent(inout) :: psiold,psinew
      complex(dop), dimension(dgldim)                :: tmppsi
      logical(kind=4), intent(in) :: lopnorm

! --- INITIALISE VARIABLES ---

      call zerovxz(tmppsi,dgldim)
      nit=0
      ierr=0
      time=0.0_dop

      write(ilog,'(2a,i3)')  &
        'Starting iterative improvement of initial wavefunction.', &
        ' nham =', nham
      write(ilog,'(a,i3,a,1pe9.2,/)') 'Max. Iterations : ', &
           wfopnit, ' ,  Convergence tolerance : ',wfoptol
      call myflush(ilog)

!-----------------------------------------------------------------------
! GWP overlap matrices and operators
!-----------------------------------------------------------------------
      if (lgwp) then
         call gwp_param(psiold,agmat,gfgwp,aigwp,zgwp,mr(mrqcent),2)
         call gwp_overlap(gs2,gsinv,mr(mrgsval),gsvek,gfgwp,aigwp,&
              zgwp,0)
         modgh = 0
         call gh_elements(gh,hung,ghderiv1,agmat,gfgwp,aigwp,zgwp,&
              mr(mrqcent),1,modgh,gs2,time)
      endif

!-----------------------------------------------------------------------
! transform psi-old to natural orbitals
!-----------------------------------------------------------------------
      call density(psiold,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      call trafonat(psiold,tree,dicht4,mi(mijindx))
      call cpvxz(psiold,psinew,dgldim)

!-----------------------------------------------------------------------
! operate on spfs of initial WP, ie. on  psi-old
!-----------------------------------------------------------------------
      ktype=0
      call hphi(psiold,mc(mchpsi),ktype,nham)
      hktype=0
      call hunphi(psiold,mc(mchunpsi),hktype,nham)

!-----------------------------------------------------------------------
! calculate new A coefficients
!-----------------------------------------------------------------------
      hktype=0
      call zerovxz(psinew,adim)  ! initially psinew was set to psiold

      call phihphi(psinew,gh,mc(mchpsi),hteil,nham,.false.)
      call phihunphi(psinew,mc(mchunpsi),hung,gh,hteil,nham)

      call calcha(psiold,psinew,hteil,mr(mrhops),psiold(zetf(1,1)),&
           nham,psinew(zetf(1,1)),mi(mijindx),mr(mrort))
      call calchuna(psiold,psinew,hteil,nham,mi(mijindx))

      if(loportho) then
         call vvaxzz(psiold,psinew,dshift,adim) !compute 
                                                !dshift=<psiold|D|psiold>
         write(ilog,'(a,1p2e13.6,a)') ' initial dshift = (', dshift,')'
         koeff(kshift)=-dshift
         if(nstate.gt.1) then
            do s=1,nstate-1
               koeff(kshift+s)=-dshift
            enddo
         endif
      endif

!-----------------------------------------------------------------------
! normalise A-vector of new wavefunction
!-----------------------------------------------------------------------
      if (lopnorm) then
         call normvxz(psinew,norm,adim)
         if (norm .eq. 0.d0) then
            routine = "opwfit"
            message = "Division by zero. Are you operating"&
                 //" onto an unpopulated state?"
            call errormsg
         elseif (abs(norm) .lt. tiny) then
            routine = "opwfit"
            message = "Division by (almost) zero. Are you operating"&
                 //" onto an unpopulated state? Will try to continue"&
                 //" but the result may be garbage."
            call warnmsg
            lwarn = .false.
         end if
         call xvixdzo(norm,psinew,adim)
      endif

!-----------------------------------------------------------------------
! GWP overlap matrices and operators
!-----------------------------------------------------------------------
      if (lgwp) then
         call gwp_param(psinew,agmat,gfgwp,aigwp,zgwp,mr(mrqcent),2)
         call gwp_overlap(gs2,gsinv,mr(mrgsval),gsvek,gfgwp,aigwp,&
              zgwp,0)
         modgh = 0
         call gh_elements(gh,hung,ghderiv1,agmat,gfgwp,aigwp,zgwp,&
              mr(mrqcent),1,modgh,gs2,time)
      endif

!-----------------------------------------------------------------------
! transform to natural orbitals
!-----------------------------------------------------------------------
      call density(psinew,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      call trafonat(psinew,tree,dicht4,mi(mijindx))
!..chkphase is no longer needed, as trafonat accounts now for correct phases

!-----------------------------------------------------------------------
! Start Iteration Loop
!-----------------------------------------------------------------------
      if( wfopnit .le. 0 )  return    ! return, if iteration not wanted
   10 continue

!-----------------------------------------------------------------------
! Calculate required spfs
!-----------------------------------------------------------------------
      call zerovxz(tmppsi,dgldim)

      call phihphi(psinew,gh,mc(mchpsi),hteil,nham,.false.)
      call phihunphi(psinew,mc(mchunpsi),hung,gh,hteil,nham)
      call mfields(psiold,tmppsi,nham,psinew,1,.false.)
      call hlochphi(psiold,tmppsi,mc(mchpsi),mc(mchloch),nham,0)

      call addhunphi(tmppsi,mc(mchunpsi))

!-----------------------------------------------------------------------
! orthonormalise required spfs
!-----------------------------------------------------------------------
      do s=1,nstate
         do m=1,nmode
            nz = 0
 100        nz = nz + 1
            if(nz .gt. dim(m,s)+2) then
               routine='Opwfit'
               write(message,'(a,i5,a,i5)') &
               'Error in Schmidt orthonormalisation of SPFs. n =', &
                ierr,' mode =',m
               call errormsg
            endif
            call schmidtortho(tmppsi(zetf(m,s)),subdim(m),dim(m,s), &
                 ierr)
            if (ierr .gt. 0) then
! if new basis not complete, replace functions from old basis
               zeig=zetf(m,s)+subdim(m)*(ierr-1)
               call cpvxz(psinew(zeig),tmppsi(zeig),subdim(m))
               goto 100
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! calculate new A coefficients
!-----------------------------------------------------------------------
      call zerovxz(tmppsi,adim)

      call phihphi(tmppsi,gh,mc(mchpsi),hteil,nham,.false.)
      call phihunphi(tmppsi,mc(mchunpsi),hung,gh,hteil,nham)
      call calcha(psiold,tmppsi,hteil,mr(mrhops),psiold(zetf(1,1)),&
           nham,tmppsi(zetf(1,1)),mi(mijindx),mr(mrort))
      call calchuna(psiold,tmppsi,hteil,nham,mi(mijindx))

!-----------------------------------------------------------------------
! normalise A-vector of new wavefunction
!-----------------------------------------------------------------------
      if (lopnorm) then
         call normvxz(tmppsi,norm,adim)
         if (norm .eq. 0.d0) then
            routine = "opwfit"
            message = "Division by zero. Are you operating"&
                 //" on an unpopulated state?"
            call errormsg
         elseif (abs(norm) .lt. tiny) then
            routine = "opwfit"
            message = "Division by (almost) zero. Are you operating"&
                 //" onto an unpopulated state? Will try to continue"&
                 //" but the result may be garbage."
            call warnmsg
            lwarn = .false.
         end if
         call xvixdzo(norm,tmppsi,adim)
      endif
      
!-----------------------------------------------------------------------
! GWP overlap matrices and operators
!-----------------------------------------------------------------------
      if (lgwp) then
         call gwp_param(tmppsi,agmat,gfgwp,aigwp,zgwp,mr(mrqcent),2)
         call gwp_overlap(gs2,gsinv,mr(mrgsval),gsvek,gfgwp,aigwp,&
              zgwp,0)
         modgh = 0
         call gh_elements(gh,hung,ghderiv1,agmat,gfgwp,aigwp,zgwp,&
              mr(mrqcent),1,modgh,gs2,time)
      endif

!-----------------------------------------------------------------------
! transform to natural orbitals
!-----------------------------------------------------------------------
      call density(tmppsi,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      call trafonat(tmppsi,tree,dicht4,mi(mijindx))

!-----------------------------------------------------------------------
! calculate error
!-----------------------------------------------------------------------
      error=0.0_dop
      do m = 1, nmode
         sum = 1.0_dop
         do s = 1, nstate
!.....Generate overlap-matrix between last and before-last iteration.
            call mmaxzzh(psinew(zetf(m,s)),tmppsi(zetf(m,s)), &
                 mc(mcworkc),subdim(m),dim(m,s))
!.....Sum the squared overlap matrix-elements multiplied with
!.....the (diagonal) density matrix.
            call sumdqdz(mr(d3mat(m,s)),mc(mcworkc),sum1,dim(m,s))
            sum = sum - sum1
         enddo
         error=max(error,abs(sum))
      enddo

!-----------------------------------------------------------------------
! see if calculation is converged
!-----------------------------------------------------------------------
      call cpvxz(tmppsi,psinew,dgldim)

      call normvxz(psinew,norm,adim)
      nit=nit+1
      write(ilog,'(1x,a,i5,1pe14.6,5x,a,1pe16.9)') &
           'Iteration ',nit,error,'Norm = ',norm
      call myflush(ilog)

      if (error.gt.wfoptol .and. nit.lt.wfopnit ) goto 10

!     ! In case, if there are more than one operate calls.
      operatenorm = operatenorm*norm

      return
      end subroutine opwfit

!***********************************************************************
!
!                            SUBROUTINE MLOPERATE
!
! An operator acts on a vector. The initial and final vectors are
! in MCTDH form.
!
! INPUT:
! psiold:  original vector
! nham:    No. of operator to be used.
! opnit:   Max. No. of Iterations
! optol:   Convergence tolerance
! kshift:  k-value of the extra operator line for operate_ortho
!
! tmppsi:    Space to store an intermediate WF
!
! OUTPUT:
! psinew: operator x psiold. Vector is normalised.
!
!***********************************************************************

      subroutine mloperate(psiold,psinew,nham,opnit,optol,kshift)

      implicit none


      integer                     :: mactive,m,f,h,k
      integer,intent(in)          :: opnit,nham,kshift
      real(dop),intent(in)        :: optol
      complex(dop),intent(in)     :: psiold(dgldim)
      complex(dop),intent(inout)  :: psinew(dgldim)
      complex(dop)                :: tmppsi(dgldim)
      complex(dop)                :: dshift
      real(dop)                   :: time,norm

!-----------------------------------------------------------------------
! Check if the operator operates on one mode exclusively
!-----------------------------------------------------------------------
      mactive=-1
 k_l: do k=zham(nham),zham(nham)+kzahl(nham)-1
 f_l:    do f=1,ndof
            m=dofspf(f)
            h=hamilton(f,k)
            if(htmtype(h).ne.1) then
               if(mactive.eq.-1) then
                  mactive=m
               elseif(mactive.ne.m) then
                  mactive=0
                  exit k_l
               endif
            endif
         enddo  f_l
      enddo k_l

      if(mactive.eq.0) then
         write(ilog,'(2a,i3)') &
          'Starting iterative improvement of initial ML-wavefunction.',&
          ' nham =', nham
         write(ilog,'(a,i3,a,1pe9.2)') 'Max. Iterations:',&
              opnit, ' ,  Convergence tolerance:',optol
      else
         write(ilog,'(a,i3,a,i3)') 'Non-iterative operation on mode',&
               mactive,', operator: nham =', nham
      endif
      call myflush(ilog)

!-----------------------------------------------------------------------
! Compute the shift for operate_ortho
!-----------------------------------------------------------------------
      if(loportho) then
         time=0.d0
         dshift = croco_tree(nham,time,tree,psiold,psiold)
         koeff(kshift)=-dshift
      endif

      if(mactive.gt.0) then
         if(loportho) then
            write(ilog,'(a,1p2e13.6,a)') &
                 'operate_ortho:  initial dshift = (', dshift,')'
            call myflush(ilog)
         end if
         call noniterative_tree(psinew,psiold,koeff,maxkoe,mactive,nham)
      else
         if(loportho) then
            write(ilog,'(/,a,10x,a,19x,a)') 'iteration  error','norm','dshift'
         else
            write(ilog,'(/,a)') 'iteration  error          norm'
         endif
         call iterative_tree(psinew,psiold,tmppsi,&
                             koeff,maxkoe,kshift,opnit,optol,nham)
      endif

!-----------------------------------------------------------------------
!     swap back to system hamiltonian (see line 57)
!-----------------------------------------------------------------------
!      call swap_moduleauxtreesarray(nham)

!-----------------------------------------------------------------------
! normalise the new wavefunction
!-----------------------------------------------------------------------
      call normvxz(psinew,norm,block(1))
      if (norm .eq. 0.d0) then
         routine = "mloperate"
         message = "Division by zero. Are you operating on an unpopulated state?"
         call errormsg
      elseif (abs(norm) .lt. tiny) then
         routine = "mloperate"
         message = "Division by (almost) zero. Are you operating on an "&
         //"unpopulated state? Will try to continue but the result may be garbage."
         call warnmsg
         lwarn = .false.
      end if
      call xvixdzo(norm,psinew,block(1))

!     In case, if there are more than one operate calls.
      operatenorm = operatenorm*norm

      end subroutine mloperate

!#######################################################################

      subroutine sumdqdz(vec,mat,sum,dim)

      implicit none

      integer(long)                                :: i,j
      integer(long), intent(in)                    :: dim
      real(dop), intent(out)                       :: sum
      real(dop), dimension(dim), intent(in)        :: vec
      complex(dop), dimension(dim,dim), intent(in) :: mat

      sum = 0.0_dop
      do j=1,dim
         do i=1,dim
            sum = sum + vec(j)*dble(mat(i,j)*dconjg(mat(i,j)))
         enddo
      enddo

      return
      end subroutine sumdqdz

!#######################################################################

      subroutine chkphase(psi1,psi2)

      implicit none

      integer(long)                                  :: m,s
      complex(dop), dimension(dgldim), intent(in)    :: psi1
      complex(dop), dimension(dgldim), intent(inout) :: psi2
      complex(dop), dimension(maxspf)                :: workc

      do s=1,nstate
         do m=1,nmode
!.....Change phases of SPF's of psi2 to agree with those of psi1
            call phasespf(psi1(zetf(m,s)),psi2(zetf(m,s)),subdim(m), &
                 dim(m,s),workc)
!.....Change phases of A-vector of psi1 accordingly.
            call dtxxzzo(workc,psi2(zpsi(s)),vdim(m,s),dim(m,s), &
                 ndim(m,s))
         enddo
      enddo

      return
      end subroutine chkphase

!#######################################################################
!                     SUBROUTINE PHASESPF
!
! The phases of the SPF's of psi2 are chosen such, that the overlap
! with the corresponding SPF of psi1 is real-positive. The pasefactors
! are used in subroutine chkphase to change the A-vector accordingly.
!-----------------------------------------------------------------------
      subroutine phasespf(psi1,psi2,subdim,dim,ovl)

      implicit none

      integer(long)                                      :: e
      integer(long), intent(in)                          :: dim,subdim
      complex(dop), dimension(subdim,dim), intent(in)    :: psi1
      complex(dop), dimension(subdim,dim), intent(inout) :: psi2
      complex(dop), dimension(dim), intent(out)          :: ovl

      do e=1,dim
         call vvaxzz(psi1(1,e),psi2(1,e),ovl(e),subdim)
         if (abs(ovl(e)) .gt. 1.0e-8_dop) then
            ovl(e) = dconjg(ovl(e))/abs(ovl(e))
            if( abs(ovl(e)-1.0_dop) .gt. 1.0e-8_dop ) then
               call xvxxzzo(ovl(e),psi2(1,e),subdim)
            else
               ovl(e)=(1.0_dop,0.0_dop)
            endif
         else
            ovl(e)=(1.0_dop,0.0_dop)
         endif
      enddo

      return
      end subroutine phasespf

!***********************************************************************
!
!                            SUBROUTINE OPWF
!
! An operator acts on a vector. The initial and final vectors are
! in MCTDH form. No iteration.
!
! INPUT:
! psiold: original vector
! nham:   No. of operator to be used.
!
! OUTPUT:
! psinew: operator x psiold. Vector is normalised.
!
!***********************************************************************

      subroutine opwf(psiold,psinew,nham,lopnorm)

      implicit none

      integer(long)             :: hktype,ktype,s,m,zeig1,zeig2,zeig3
      integer(long)             :: modgh
      integer(long), intent(in) :: nham
      real(dop)                 :: norm
      logical(kind=4), intent(in) :: lopnorm

      complex(dop), dimension(dgldim), intent(inout) :: psiold
      complex(dop), dimension(dgldim), intent(out)   :: psinew
      real(dop)                                      :: time

      write(ilog,'(a)')'Direct (non iterative) calcalation done first.'

! initialise variables
      time=0.0_dop
      call zerovxz(psinew,dgldim)

!-----------------------------------------------------------------------
! GWP overlap matrices and operators
!-----------------------------------------------------------------------
      if (lgwp) then
         call gwp_param(psiold,agmat,gfgwp,aigwp,zgwp,mr(mrqcent),2)
         call gwp_overlap(gs2,gsinv,mr(mrgsval),gsvek,gfgwp,aigwp,&
              zgwp,0)
         modgh = 0
         call gh_elements(gh,hung,ghderiv1,agmat,gfgwp,aigwp,zgwp,&
              mr(mrqcent),1,modgh,gs2,time)
      endif

! compute 'derivative'
      call density(psiold,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      ktype=0
      call hphi(psiold,mc(mchpsi),ktype,nham)
      call phihphi(psiold,gh,mc(mchpsi),hteil,nham,.false.)
      call mfields(psiold,psinew,nham,psiold,0,.false.)
      call hlochphi(psiold,psinew,mc(mchpsi),mc(mchloch),nham,0)
      call dicht1phi(psinew,dicht1)

      hktype=0
      call hunphi(psiold,mc(mchunpsi),hktype,nham)
      call addhunphi(psinew,mc(mchunpsi))
      call phihunphi(psiold,mc(mchunpsi),hung,gh,hteil,nham)
      call project(psiold,psinew)
      call calchuna(psiold,psinew,hteil,nham,mi(mijindx))

! determine pointers
      zeig1 = 1
      zeig2 = 1
      zeig3 = 1
      do s=1,nstate
         do m=1,nmode
            dims(m,s) = dim(m,s)*2
            zeig1 = zeig1 + dims(m,s)*subdim(m)
            dmats(m,s) = zeig2
            zeig2 = zeig2 + dims(m,s)*dims(m,s)
            d3mats(m,s) = zeig3
            zeig3 = zeig3 + dims(m,s)*2
         enddo
      enddo
      dmatsdim = dmatdim*4
      d3matsdim = d3matdim*2

      workcdims = workcdim - dmatdim - 3*dmatsdim
      workrdims = workrdim - d3matsdim

      call detspfs(psiold,psinew)

! compute new A-vector
      call zerovxz(psinew,adim)
      call phihphi(psinew,gh,mc(mchpsi),hteil,nham,.false.)
      call phihunphi(psinew,mc(mchunpsi),hung,gh,hteil,nham)
      call calcha(psiold,psinew,hteil,mr(mrhops),psiold(zetf(1,1)),&
           nham,psinew(zetf(1,1)),mi(mijindx),mr(mrort))
      call calchuna(psiold,psinew,hteil,nham,mi(mijindx))

! normalise new wavefunction
      if (lopnorm) then
         call normvxz(psinew,norm,adim)
         call xvixdzo(norm,psinew,adim)
      endif

      return
      end subroutine opwf

!***********************************************************************
!
!                            SUBROUTINE D2OPWF
!
! An operator acts on a vector. The initial and final vectors are
! in MCTDH form. No iteration.
!
! INPUT:
! psiold: original vector
! nham:   No. of operator to be used.
!
! OUTPUT:
! psinew: operator x psiold. Vector is normalised.
!
!  09/2001 CC
!
!***********************************************************************

      subroutine d2opwf(psiold,psinew,nham,lopnorm)

      implicit none

      integer(long)             :: ktype,hktype,s
      integer(long), external   :: slen
      integer(long), intent(in) :: nham
      real(dop)                 :: norm
      logical(kind=4), intent(in) :: lopnorm
      complex(dop), dimension(1)                     :: cdum,cdum1 
      complex(dop), dimension(dgldim), intent(inout) :: psiold
      complex(dop), dimension(dgldim), intent(out)   :: psinew

! --- INITIALISE VARIABLES ---
      routine = 'D2opwf'

      s=slen(hamlab(nham))
      write(ilog,'(/,3a)') &
         '*** Operating on MCTDH form initial density operator Using: ', &
             hamlab(nham)(1:s),' ***'

      call density(psiold,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)

      ktype=0

      call hphi(psiold,mc(mchpsi),ktype,nham)

      call phihphi(psiold,cdum(1),mc(mchpsi),hteil,nham,.false.)

      hktype=0
      call hunphi(psiold,mc(mchunpsi),hktype,nham)
      call phihunphi(psiold,mc(mchunpsi),cdum,cdum1,hteil,nham)

      call zerovxz(psinew,dgldim)

      call d2calchuna(psiold,psinew,hteil,nham)

      call d2calcha(psiold,psinew,hteil,nham)

      call cpvxz(psiold(zetf(1,1)),psinew(zetf(1,1)),totphidim)

      call cpvxz(psiold,psinew,dgldim)

      call prefactor(psinew,adim)

      call normvxz(psinew,norm,adim)


!     if norm=zero then operator commutes with system Hamiltonian
      if (norm.le.tiny) then
         write(6,'(/,a,/)') ' Norm is zero! Maybe the operator '// &
              'commutes with system Hamiltonian!'
         message = ' norm = zero '
         call errormsg
      endif

      if (lopnorm) call xvixdzo(norm,psinew,adim)

      call cpvxz(psinew,psiold,adim)


      return
      end subroutine d2opwf

!***********************************************************************
!     DETSPFS
!***********************************************************************

      subroutine detspfs(psiold,psinew)

      implicit none

      integer(long)       :: m,m1,s,ierr,dimnew,zeig1,hdim
      real(dop)           :: norm,trace,epsrel

      real(dop), dimension(2*d3matdim)                 :: dicht3s
      complex(dop), dimension(dgldim), intent(in)      :: psiold
      complex(dop), dimension(dgldim), intent(inout)   :: psinew
      complex(dop), dimension(dmatdim)                 :: trafos
      complex(dop), dimension(4*dmatdim)               :: dicht1s,dicht2s,&
                                                          dicht4s
      complex(dop), dimension(workcdims)               :: workc

! determine trafos
      do s=1,nstate
         do m=1,nmode
            call cpmxz(psinew(zetf(m,s)),workc,subdim(m),dim(m,s))

            call schmidtortho(workc,subdim(m),dim(m,s),ierr)
            if (ierr .gt. 0) then
               dimnew=ierr-1
            else
               dimnew=dim(m,s)
            endif
! if new basis not complete, take remaining functions from old basis
            if (dimnew .lt. dim(m,s)) then
               zeig1 = subdim(m)*dimnew + 1
               hdim  = subdim(m)*(dim(m,s)-dimnew)
               call cpvxz(psiold(zetf(m,s)),workc(zeig1),hdim)
               call schmidtortho(workc,subdim(m),dim(m,s),ierr)
               if (ierr .gt. 0) then
                  routine='detspfs'
                  write(message,'(a,i5,a,i5)')&
             'Error in Schmidt orthonormalisation of SPFs. n =',ierr,&
             ' mode =',m
                  call errormsg
               endif
            endif

            call mmaxzz(psinew(zetf(m,s)),workc,trafos(dmat(m,s)),&
                 subdim(m),dim(m,s),dim(m,s))
            call cpmxz(workc,psinew(zetf(m,s)),subdim(m),dim(m,s))
         enddo
      enddo

! determine new density
      call zerovxz(dicht2s,dmatsdim)
      do s=1,nstate
         do m=1,nmode
            call rm1hxxxzz(psinew(zpsi(s)),workc,vdim(m,s),&
                 dim(m,s),ndim(m,s))
            call addd2s1(dicht2s(dmats(m,s)),workc,dims(m,s),dim(m,s))
         enddo
      enddo

      do s=1,nstate
         do m=1,nmode
            call qttxzz(trafos(dmat(m,s)),psiold(zpsi(s)),workc,&
                 vdim(m,s),dim(m,s),ndim(m,s))

            zeig1 = block(s) + 1
            call rmqxxxzz(psinew(zpsi(s)),workc,workc(zeig1),&
                 vdim(m,s),dim(m,s),ndim(m,s))
            call addd2s2(dicht2s(dmats(m,s)),workc(zeig1),dims(m,s),&
                 dim(m,s))

            call rm1hxxxzz(workc,workc(zeig1),vdim(m,s),dim(m,s),&
                 ndim(m,s))
            call addd2s3(dicht2s(dmats(m,s)),workc(zeig1),dims(m,s),&
                 dim(m,s))

            do m1=1,nmode
               if (m1 .ne. m) then
                  call rm1hxxxzz(workc,workc(zeig1),vdim(m1,s),&
                       dim(m1,s),ndim(m1,s))
                  call addd2s1(dicht2s(dmats(m1,s)),workc(zeig1),&
                       dims(m1,s),dim(m1,s))

               endif
            enddo

         enddo
      enddo

! compute norm and take a relative regularization parameter
      norm=0.0_dop
      do s=1,nstate
         call trhxz(dicht2s(dmats(1,s)),trace,dims(1,s))
         norm=norm+trace
      enddo
      epsrel=norm*epsmat

! regularize and invert density matrices for each mode and state
      norm = 1.0_dop
      do s=1,nstate
        do m=1,nmode
           call denmat(dicht1s(dmats(m,s)),dicht2s(dmats(m,s)),&
                dicht3s(d3mats(m,s)),dicht4s(dmats(m,s)),dims(m,s),&
                epsrel)
        enddo
      enddo

! compute new spfs in natural orbital representation and copy the
! first dim(m,s) spfs
      do s=1,nstate
         do m=1,nmode

            call cpvxz(psiold(zetf(m,s)),workc,subdim(m)*dim(m,s))
            zeig1 = subdim(m)*dim(m,s) + 1
            call cpvxz(psinew(zetf(m,s)),workc(zeig1),&
                 subdim(m)*dim(m,s))

            zeig1 = subdim(m)*dims(m,s) + 1
            call mqxczz(workc,dicht4s(dmats(m,s)),workc(zeig1),&
                 subdim(m),dims(m,s))
            call cpmxz(workc(zeig1),psinew(zetf(m,s)),&
                 subdim(m),dim(m,s))
            call schmidtortho(psinew(zetf(m,s)),subdim(m),dim(m,s),ierr)

         enddo
      enddo

      return
      end subroutine detspfs

!***********************************************************************
!     ADDD2S1
!***********************************************************************

      subroutine addd2s1(mat1,mat2,dim1,dim2)

      implicit none

      integer(long)                                     :: i,j
      integer(long), intent(in)                         :: dim1,dim2
      complex(dop), dimension(dim1,dim1), intent(inout) :: mat1
      complex(dop), dimension(dim2,dim2), intent(in)    :: mat2

      do j=1,dim2
         do i=1,dim2
            mat1(i,j) = mat1(i,j) + mat2(i,j)
         enddo
      enddo

      return
      end subroutine addd2s1

!***********************************************************************
!     ADDD2S2
!***********************************************************************

      subroutine addd2s2(mat1,mat2,dim1,dim2)

      implicit none

      integer(long)                                     :: i,j
      integer(long), intent(in)                         :: dim1,dim2
      complex(dop), dimension(dim1,dim1), intent(inout) :: mat1
      complex(dop), dimension(dim2,dim2), intent(in)    :: mat2

      do j=1,dim2
         do i=1,dim2
            mat1(i,j+dim2) = mat1(i,j+dim2) + mat2(i,j)
            mat1(j+dim2,i) = conjg(mat1(i,j+dim2))
         enddo
      enddo

      return
      end subroutine addd2s2

!***********************************************************************
!     ADDD2S3
!***********************************************************************

      subroutine addd2s3(mat1,mat2,dim1,dim2)

      implicit none

      integer(long)                                     :: i,j
      integer(long), intent(in)                         :: dim1,dim2
      complex(dop), dimension(dim1,dim1), intent(inout) :: mat1
      complex(dop), dimension(dim2,dim2), intent(in)    :: mat2

      do j=1,dim2
         do i=1,dim2
            mat1(i+dim2,j+dim2) = mat1(i+dim2,j+dim2) + mat2(i,j)
         enddo
      enddo

      return
      end subroutine addd2s3

!***********************************************************************
!                                                                       
!                            SUBROUTINE D1OPWFIT                        
!                                                                       
! An operator acts on a vector. The initial and final vectors are       
! in MCTDH form.                                                        
!                                                                       
! INPUT:                                                                
! psiold: original vector                                               
! nham:   No. of operator to be used.                                   
!                                                                       
! OUTPUT:                                                               
! psinew: operator x psiold. Vector is normalised.                      
!                                                                       
!                                                                       
!  12/01  CC                                                            
!                                                                       
!***********************************************************************

      subroutine d1opwfit(psiold,psinew,nham,wfopnit,wfoptol,lopnorm)

      implicit none

      complex(dop), dimension(dgldim), intent(inout) :: psiold
      complex(dop), dimension(dgldim), intent(out)   :: psinew
      complex(dop), dimension(dgldim)                :: tmppsi
      complex(dop), dimension(maxblock*maxblock,3)   :: workc

      integer(long), intent(in)                 :: nham,wfopnit
      real(dop), intent(in)                     :: wfoptol
      logical(kind=4), intent(in) :: lopnorm

      complex(dop),dimension(maxspf)            :: trace
      real(dop)                            :: norm,error,sum,sum1
      integer(long)                        :: k,k1,k2,nit,s,t,pworkc,&
                                              d,m,mloch,nz,z,ierr,&
                                              zeig,hktype,ktype
      integer(long), external              :: slen
      integer(long), dimension(2)          :: zw

! --- INITIALISE VARIABLES ---
      routine = 'D1opwfit'
!
      if (nstate.gt.1) then
         message = ' nstate > 1 ! Not tested ! '//&
              ' Maybe not correctly implemented ! ' 
         call errormsg
      endif

      s=slen(hamlab(nham))
      write(ilog,'(/,3a)')&
         '*** Operating on MCTDH form initial density operator Using: ',&
             hamlab(nham)(1:s),' ***'

      call zerovxz(tmppsi,dgldim)      

      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
      ierr = 0
      nit=0
      zw(1)=1
      zw(2)=2
      pworkc=mcworkc + maxblock*maxblock*3


!     --- this routine works (until now) only for one electronic state
      s=1
      t=1


!--------------------------------------------------------------------
! transform psiold to natural orbits, requires density matrix
!-------------------------------------------------------------------
      call density(psinew,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      call trafonat(psinew,tree,dicht4,mi(mijindx))

      call density(psiold,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      call trafonat(psiold,tree,dicht4,mi(mijindx))

!--------------------------------------------------------------------
! operate on spfs of initial WF
!--------------------------------------------------------------------
      ktype=0
      call d1hphi(psiold,mc(mchpsi),ktype,nham)
      hktype=0
      call d1hunphi(psiold,mc(mchunpsi),hktype,nham)

!--------------------------------------------------------------------
! calculate new A coefficients
!--------------------------------------------------------------------

!     --- store psinew in tmppsi(1,1) (=r.h.s. of iteration)
      call cpvxz(psinew,tmppsi,dgldim)

      call d1phihphi(tmppsi,mc(mchpsi),hteil,nham)
      call d1phihunphi(tmppsi,mc(mchunpsi),hteil,nham)

      call zerovxz(workc(1,zw(1)),adim)
      call d1calchaloch2(psiold,workc(1,zw(1)),hteil,nham)

      call cpvxz(workc(1,zw(1)),workc(dzpsi(s,t),zw(2)), &
           dblock(s,t))

      call cpvxz(workc(1,zw(2)),tmppsi,adim)

      call zerovxz(workc(1,zw(1)),adim)
      call d1calchaloch2(psiold,workc(1,zw(1)),hteil,nham)

      call cpvcz(workc(1,zw(1)),workc(dzpsi(s,t),zw(2)),dblock(s,t))

      call subvxxzo1(tmppsi,workc(1,zw(2)),adim)
!     multiply with -I
      call xvxxzzo((0.0_dop,-1.0_dop),tmppsi,adim)      


!     --- normalise new wavefunction (A-vector)
      call normvxz(tmppsi,norm,adim)      
      if (norm.le.tiny) then
         message = ' Norm is zero! Maybe the operator '// &
              'commutes with system Hamiltonian!' 
         write(6,'(/,a,/)') message
         call errormsg
      endif
      if (lopnorm) call xvixdzo(norm,tmppsi,adim)
!     -----------------------------------------


      
!####################################################################
!     === iteration loop                                         ====
!     --- use in this loop pworkc instead of mcworkc !!          ----
!####################################################################
 10   continue
      nit = nit + 1
      write(ilog,'(80(a1))') ('-',k=1,80)
      write(ilog,'(a,i5)') 'Iteration Step = ',nit
      write(ilog,*) ' '

!--------------------------------------------------------------------
! transform to natural orbits, requires density matrix
!-------------------------------------------------------------------
      call density(tmppsi,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      call trafonat(tmppsi,tree,dicht4,mi(mijindx))

      call zerovxz(workc(1,2),maxblock*maxblock)      


!--------------------------------------------------------------------
! calculate required spfs
!-------------------------------------------------------------------- 
      ktype=0
      call d1hphi(psiold,mc(mchpsi),ktype,nham)
      hktype=0
      call d1hunphi(psiold,mc(mchunpsi),hktype,nham)
      call d1phihphi(tmppsi,mc(mchpsi),hteil,nham)
      call d1phihunphi(tmppsi,mc(mchunpsi),hteil,nham)
      call zerovxz(psinew(adim+1),totphidim)

      do mloch=1,nmode
!        --- correlated terms
         do k=k1,k2
            call zerovxz(workc(1,zw(1)),adim)
            
            call d1tracetens(psiold(adim+1),mc(mchpsi),psiold, &
                 workc(1,2),mloch,k,mc(pworkc))

!           --- tmppsi is real
            call d1ttxx(tmppsi(dzpsi(s,t)),workc(dzpsi(s,t),2), &
                 workc(1,1), &
                 vddim(mloch,s,t),ddim(mloch,s,t),nddim(mloch,s,t))
            
!           mulitiply with -I
            call xqxxzzo((0.0_dop,-1.0_dop),workc(1,1),ddim(mloch,s,t))
            
!           multiply with koeff(k)
!            h=hamilton(fkoe,k)
!            if (htmtype(h).ne.1) then
            call xqxxzzo(koeff(k),workc(1,1),ddim(mloch,s,t))
!            endif
            
            call mmxtzza(mc(mchpsi+dzhpsi(mloch,k,t)-1),workc(1,1), &
                    psinew(dzetf(mloch,s,t)), &
                    subdim(mloch),ddim(mloch,s,t),ddim(mloch,s,t))
         enddo                  ! of k, i.e. correlated terms


!    ------ uncorrelated terms------------------------------------
         call zerovxz(workc(1,zw(1)),adim)
         
         call d1tracetens(psiold(adim+1),mc(mchunpsi),psiold, &
                 workc(1,2),mloch,0,mc(pworkc))

!           --- tmppsi is real
         call d1ttxx(tmppsi(dzpsi(s,t)),workc(dzpsi(s,t),2), &
              workc(1,1), &
              vddim(mloch,s,t),ddim(mloch,s,t),nddim(mloch,s,t))
            
!        --- mulitiply with -I  ---
         call xqxxzzo((0.0_dop,-1.0_dop),workc(1,1),ddim(mloch,s,t))
         call xqxxzzo(koeff(k),workc(1,1),ddim(mloch,s,t))


         call mmxtzza(mc(mchunpsi+dzetf(mloch,s,t)-1),workc(1,1), &
              psinew(dzetf(mloch,s,t)), &
              subdim(mloch),ddim(mloch,s,t),ddim(mloch,s,t))
 

!        -----------------------------------------
!           add the adjoint to each sigma
!        -----------------------------------------
         z=0
         do d=1,ddim(mloch,s,t)
            call cpqxz(psinew(dzetf(mloch,s,t)+z),workc(1,1), &
                 supgdim(mloch))
            call addmxazo1(psinew(dzetf(mloch,s,t)+z), &
                 workc(1,1),supgdim(mloch),supgdim(mloch))
            z=z+subdim(mloch)
         enddo


      enddo                     ! of mloch



!------------------------------------------------------------------------
! orthonormalise required spfs  
!------------------------------------------------------------------------
      do m=1,nmode
!        first care for non-negative traces         
         call trspdoc(psinew(dzetf(m,s,t)),trace,ddim(m,s,t),subdim(m), &
              supgdim(m))
         z=dzetf(m,s,t)
         do d=1,ddim(m,s,t)
            if ((dble(trace(d)).lt.0.0_dop).and. &
                 (abs(dimag(trace(d))).lt.tiny)) then
               call xvxxzzo ((-1.0_dop,0.0_dop),psinew(z),subdim(m))
            endif
            z=z+subdim(m)
         enddo

         nz = 0
 100     nz = nz + 1
         if(nz .gt. ddim(m,s,t)+2) then
            routine='d1opwfit'
            write(message,'(a,i5,a,i5)') &
                 'Error in Schmidt orthonormalisation of SPFs. n =', &
                 ierr,' mode =',m
            call errormsg
         endif
         
         call schmidtortho(psinew(dzetf(m,s,t)),subdim(m),ddim(m,s,t), &
              ierr)

         if (ierr .gt. 0) then
! if new basis not complete, replace functions from old basis
            write(ilog,'(a)') 'Basis not complete.'
            zeig=dzetf(m,s,t)+subdim(m)*(ierr-1)
            call cpvxz(tmppsi(zeig),psinew(zeig),subdim(m))
            goto 100
         endif
      enddo


!--------------------------------------------------------------------
! calculate new A coefficients
!--------------------------------------------------------------------

!     --- store psinew in tmppsi(1,1) (=r.h.s. of iteration)
      ktype=0
      call d1hphi(psiold,mc(mchpsi),ktype,nham)
      hktype=0
      call d1hunphi(psiold,mc(mchunpsi),hktype,nham)
      call d1phihphi(tmppsi,mc(mchpsi),hteil,nham)
      call d1phihunphi(tmppsi,mc(mchunpsi),hteil,nham)

      call zerovxz(workc(1,zw(1)),adim)
      call d1calchaloch2(psiold,workc(1,zw(1)),hteil,nham)

      call cpvxz(workc(1,zw(1)),workc(dzpsi(s,t),zw(2)),dblock(s,t))

      call cpvxz(workc(1,zw(2)),psinew,adim)

      call zerovxz(workc(1,zw(1)),adim)
      call d1calchaloch2(psiold,workc(1,zw(1)),hteil,nham)

      call cpvcz(workc(1,zw(1)),workc(dzpsi(s,t),zw(2)),dblock(s,t))

      call subvxxzo1(psinew,workc(1,zw(2)),adim)
!     multiply with -I
      call xvxxzzo((0.0_dop,-1.0_dop),psinew,adim) 
  

!     --- normalise new wavefunction (A-vector)
      call normvxz(psinew,norm,adim)      
      if (norm.le.tiny) then
         message = ' Norm is zero! Maybe the operator '// &
              'commutes with system Hamiltonian!' 
         write(6,'(/,a,/)') message
         call errormsg
      endif
      if (lopnorm) call xvixdzo(norm,psinew,adim)
!     -----------------------------------------


!     --- calculate the complex part of A-vector
      error=0.0_dop
      write(ilog,'(1x,a,f12.6,5x,a,f12.6)') 'Norm of A-Tensor = ',norm, &
           'Real-error of A-vector = ',error
      write(ilog,*) ' '


!-----------------------------------------------------------------------
! calculate error of A-Vector,i.e., the change of the A-Vector  
!-----------------------------------------------------------------------
      call density(psinew,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      error=0.0_dop
      do m=1,nmode
         sum = 1.0_dop
!         do s = 1, nstate
!.....Generate overlap-matrix between last and before-last iteration.
            call mmaxzzh(psinew(dzetf(m,s,t)),tmppsi(dzetf(m,s,t)), &
                 mc(pworkc),subdim(m),ddim(m,s,t)) 
!.....Sum the squared overlap matrix-elements multiplied with 
!.....the (diagonal) density matrix.
            call sumdqdz(mr(dd3mat(m,s,t)),mc(pworkc),sum1,ddim(m,s,t))
            sum = sum - sum1
!         enddo
         error=max(error,abs(sum))        
      enddo


!-----------------------------------------------------------------------
! see if calculation is converged
!-----------------------------------------------------------------------
      write(ilog,800) error
      write(6,800) error
      if (error .gt. wfoptol) then
         if (nit .ge. wfopnit) then 
            write(ilog,'(a)') ' Maximum iteration has been reached!    '
            write(ilog,'(a)') ' d1opwfit maybe not have been converged!'
            write(ilog,'(a)') ' Possible reasons are :                 '
            write(ilog,'(a)') ' - not enough spfs                      '
            write(ilog,'(a)') ' - operate_tol too small                '
            write(ilog,'(a)') ' - bug in subroutine d1opwfit           '
            write(ilog,'(a)') '   ( I do not hope so ;-) )             '
            write(6,'(a)') ' Maximum iteration has been reached!    '
            write(6,'(a)') ' d1opwfit maybe not have been converged!'
            write(6,'(a)') ' Possible reasons are :                 '
            write(6,'(a)') ' - not enough spfs                      '
            write(6,'(a)') ' - operate_tol too small                '
            write(6,'(a)') ' - bug in subroutine d1opwfit           '
            write(6,'(a)') '   ( I do not hope so ;-) )             '


            go to 999         ! abort iteration, max iteration count has
                              ! been reached
         else                 
            call cpvxz(psinew,tmppsi,dgldim)
            go to 10          ! go to next iteration loop
         endif
      endif

 800  format(1x,'======>>>  Error = ',e12.6)

 999  continue
      

      call density(psinew,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      call trafonat(psinew,tree,dicht4,mi(mijindx))
      call d1regular(psinew,.true.)


      do m=1,nmode
         nz = 0
 1000    nz = nz + 1
         if(nz .gt. ddim(m,s,t)+2) then
            routine='d1opwfit'
            write(message,'(a,i5,a,i5)') &
                 'Error in Schmidt orthonormalisation of SPFs. n =', &
                 ierr,' mode =',m
            call errormsg
         endif
         
         call schmidtortho(psinew(dzetf(m,s,t)),subdim(m),ddim(m,s,t), &
              ierr)

         if (ierr .gt. 0) then
! if new basis not complete, replace functions from old basis
            write(ilog,'(a)') 'Basis not complete.'
            zeig=dzetf(m,s,t)+subdim(m)*(ierr-1)
            call cpvxz(tmppsi(zeig),psinew(zeig),subdim(m))
            goto 1000
         endif
      enddo
      
      operatenorm = operatenorm*norm

      write(ilog,*) ' '
      write(ilog,'(80(a1))') ('=',k=1,72)
      write(ilog,*) ' '

      return
      end subroutine d1opwfit

!***********************************************************************
!                                                                       
!                            SUBROUTINE D2OPWFIT                        
!                                                                       
! An operator acts on a vector. The initial and final vectors are       
! in MCTDH form.                                                        
!                                                                       
! INPUT:                                                                
! psiold: original vector                                               
! nham:   No. of operator to be used.                                   
!                                                                       
! OUTPUT:                                                               
! psinew: operator x psiold. Vector is normalised.                      
!                                                                       
!                                                                       
!  08/01  CC                                                            
!                                                                       
!***********************************************************************

      subroutine d2opwfit(psiold,psinew,nham,wfopnit,wfoptol,lopnorm)

      implicit none

!     
      complex(dop), dimension(1)                     :: cdum,cdum1
      complex(dop), dimension(dgldim), intent(inout) :: psiold,psinew
      complex(dop), dimension(dgldim)                :: tmppsi
      complex(dop), dimension(maxblock*maxblock,3)   :: workc
      logical(kind=4), intent(in) :: lopnorm

      integer(long), intent(in)                :: nham,wfopnit
      real(dop), intent(in)                    :: wfoptol
!     
!     external                  :: mmaxzz,mmtczz
      integer(long), external   :: slen
      integer(long)             :: nz,i,ierr,k,k1,k2,nit,ktype,hktype,m,s,t,&
                                    mloch,zeig,pworkc

      real(dop)                                 :: error,norm,sum,sum1
      real(dop), dimension(maxspf,maxspf)       :: spfcutoff

! --- INITIALISE VARIABLES ---
      routine = 'd2opwfit'

      if (nstate.gt.1) then
         message = ' nstate > 1 ! Not tested ! '//&
              ' Maybe not correct implemented ! ' 
         call errormsg
      endif


      s=slen(hamlab(nham))
      write(ilog,'(/,72a1)')  ('#',i=1,72)
      write(ilog,'(/,3a)')&
         '*** Operating on MCTDH form initial density operator Using: ',&
             hamlab(nham)(1:s),' ***'            
      write(ilog,'(a,i5,a,1pe12.4,/)') 'Max. Iterations : ',&
           wfopnit, ' ,  Convergence tolerance : ',wfoptol

      write(ilog,'(a)')  ' '
      write(ilog,'(72a1)')  ('#',i=1,72)

      call zerovxz(tmppsi,dgldim)     

      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
      nit=0     
      pworkc = mcworkc + maxblock*maxblock*3
      ierr = 0


!--------------------------------------------------------------------
! transform psiold to natural orbits, requires density matrix
!--------------------------------------------------------------------
      call density(psinew,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      call trafonat(psinew,tree,dicht4,mi(mijindx))

      call density(psiold,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      call trafonat(psiold,tree,dicht4,mi(mijindx))

!--------------------------------------------------------------------
! operate on spfs of initial WF
!--------------------------------------------------------------------
      ktype=0
      call hphi(psiold,mc(mchpsi),ktype,nham)
      hktype=0
      call hunphi(psiold,mc(mchunpsi),hktype,nham)


!--------------------------------------------------------------------
! calculate new A coefficients
!--------------------------------------------------------------------
      call cpvxz(psinew,tmppsi,dgldim)

      call phihphi(tmppsi,cdum,mc(mchpsi),hteil,nham,.false.)
      call phihunphi(tmppsi,mc(mchunpsi),cdum,cdum1,hteil,nham)

!     --- second part
!     -----------------------      
      call cpvxz(psiold,workc(1,2),adim)
      call zerovxz(workc(1,1),adim)  
      call d2calchunaloch2(workc(1,2),workc(1,1),hteil,0,nham)
      call d2calchaloch2(workc(1,2),workc(1,1),hteil,0,1,nham)
      do s=1,nstate
         do t=1,nstate
            call cpmxz(workc(dzpsi(s,t),1),workc(dzpsi(t,s),2),&
                 block(s),block(t))
         enddo
      enddo
      call xvxxzzo((0.0_dop,-1.0_dop),workc(1,2),adim)
      call cpvxz(workc(1,2),tmppsi,adim) 
!     --- end of second part                                         --
!     -----------------------------------------------------------------


!     --- third part                             ----------------------
!     ---         
      call cpvxz(psiold,workc(1,1),adim)
      call zerovxz(workc(1,2),adim)  
      call d2calchunaloch2(workc(1,1),workc(1,2),hteil,0,nham)  
      call d2calchaloch2(workc(1,1),workc(1,2),hteil,0,1,nham)
      call d2brakettens(tmppsi(adim+1),psiold(adim+1),workc(1,2),&
           workc(1,1),0,mc(pworkc),mmtczz)
      do s=1,nstate
         do t=1,nstate
            call cpmaz(workc(dzpsi(s,t),1),workc(dzpsi(t,s),2),&
                 block(s),block(t))
         enddo
      enddo 
      call xvxxzzo((0.0_dop,1.0_dop),workc(1,2),adim)
      call addvxxzo(workc(1,2),tmppsi,adim)     
!     --- end of third part                                          --
!     -----------------------------------------------------------------


!     - calculate barrier for "small" spfs
      do s=1,nstate
         do mloch=1,nmode
            call d2ttxx1221(tmppsi(dzpsi(s,s)),tmppsi(dzpsi(s,s)),&
                 workc(1,1),block(s),vdim(mloch,s),dim(mloch,s),&
                 ndim(mloch,s))
            call normvxz(workc(1,1),spfcutoff(mloch,s),dim(mloch,s)**2)
         enddo
      enddo

      
!     --- normalise new wavefunction (A-vector)
      call normvxz(tmppsi,norm,adim)      
      if (norm.le.tiny) then
         message = ' Norm is zero! Maybe the operator '//&
              'commutes with system Hamiltonian!' 
         write(6,'(/,a,/)') message
         call errormsg
      endif
      if (lopnorm) call xvixdzo(norm,tmppsi,adim)
!     -----------------------------------------


!     --- care for hermiticity
      error=0.0_dop
      do s=1,nstate
         call chkherm(tmppsi(dzpsi(s,s)),error,block(s))
         call mkherm(tmppsi(dzpsi(s,s)),block(s))
         do t=(s+1),nstate
            call chkherm2(tmppsi(dzpsi(s,t)),tmppsi(dzpsi(t,s)),&
                 error,block(s),block(t))
            call mkherm2(tmppsi(dzpsi(s,t)),tmppsi(dzpsi(t,s)),&
                 block(s),block(t))            
         enddo
      enddo

!--------------------------------------------------------------------
! transform tmppsi to natural orbits, requires density matrix
!--------------------------------------------------------------------
      call density(tmppsi,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      call trafonat(tmppsi,tree,dicht4,mi(mijindx))


!####################################################################
!     === iteration loop                                         ====
!     --- use in this loop pworkc instead of mcworkc !!          ----
!####################################################################
      pworkc=mcworkc + maxblock*maxblock*3
 10   continue      
      nit = nit + 1

      write(ilog,'(80(a1))') ('-',k=1,72)
      write(ilog,'(a,i5)') 'Iteration Step = ',nit
      write(ilog,*) ' '

      call zerovxz(workc(1,2),maxblock*maxblock)


!--------------------------------------------------------------------
! calculate required spfs
!--------------------------------------------------------------------
      ktype=0
      call hphi(psiold,mc(mchpsi),ktype,nham)
      hktype=0
      call hunphi(psiold,mc(mchunpsi),hktype,nham)
      call phihphi(tmppsi,cdum(1),mc(mchpsi),hteil,nham,.false.)
      call phihunphi(tmppsi,mc(mchunpsi),cdum,cdum1,hteil,nham)
      call zerovxz(psinew(adim+1),totphidim)


      do mloch=1,nmode
!     --- third  part                          ----------------
!     -----      
         ktype=0
         call hphi(psiold,mc(mchpsi),ktype,nham)
         hktype=0
         call hunphi(psiold,mc(mchunpsi),hktype,nham)
         call phihphi(tmppsi,cdum(1),mc(mchpsi),hteil,nham,.false.)
         call phihunphi(tmppsi,mc(mchunpsi),cdum,cdum1,hteil,nham)

!        --- correlated terms ---         
         do k=k1,k2
            call cpvxz(psiold,workc(1,1),adim)
            call zerovxz(workc(1,2),adim)  
            call d2brakettens(tmppsi(adim+1),psiold(adim+1),psiold,&
                 workc(1,1),0,mc(mcworkc+maxblock*maxblock*3),mmtczz)

            call d2calchaloch2(workc(1,1),workc(1,2),hteil,mloch,2,nham,k)
            do s=1,nstate
               do t=1,nstate
                  call d2ttxx1221(tmppsi(dzpsi(t,s)),workc(dzpsi(s,t),2)&
                       ,workc(1,1),block(t),vdim(mloch,s),dim(mloch,s)&
                       ,ndim(mloch,s))
               enddo

               call xqxxzzo(koeff(k),workc(1,1),dim(mloch,s))

!              mulitiply with -I
               call xqxxzzo((0.0_dop,-1.0_dop),workc(1,1),dim(mloch,s))
 
!               call xqxxzzo((0.0_dop,0.0_dop),workc(1,1),dim(mloch,s))
               call mmxtzza(mc(mchpsi+zhpsi(mloch,k)-1),workc(1,1),&
                    psinew(zetf(mloch,s)),&
                    subdim(mloch),dim(mloch,s),dim(mloch,s)) 

            enddo

         enddo  ! of k, i.e. the correlated terms
      

!        --- uncorrelated terms
         call cpvxz(psiold,workc(1,1),adim)
         call zerovxz(workc(1,2),adim)  
         call d2brakettens(tmppsi(adim+1),psiold(adim+1),psiold,&
              workc(1,1),0,mc(mcworkc+maxblock*maxblock*3),mmtczz)
         call d2calchunaloch2(workc(1,1),workc(1,2),hteil,mloch,nham)  

         do s=1,nstate
            do t=1,nstate
               call d2ttxx1221(tmppsi(dzpsi(t,s)),workc(dzpsi(s,t),2)&
                    ,workc(1,1),block(t),vdim(mloch,s),dim(mloch,s)&
                    ,ndim(mloch,s))
            enddo
            call xqxxzzo(koeff(k),workc(1,1),dim(mloch,s))

!           mulitiply with -I
            call xqxxzzo((0.0_dop,-1.0_dop),workc(1,1),dim(mloch,s))

               
            call mmxtzza(mc(mchunpsi+zetf(mloch,s)-1),workc(1,1),&
                 psinew(zetf(mloch,s)),&
                 subdim(mloch),dim(mloch,s),dim(mloch,s)) 
         enddo
!        --- end of uncorrelated terms

!        --- end of third part                                        --
!        ---------------------------------------------------------------



!     --- 4th part                                ----------------------
!     ---                                                             --
         call cpvcz(tmppsi,workc(1,1),adim)

         ktype=0
         call hphi(tmppsi,mc(mchpsi),ktype,nham)
         hktype=0
         call hunphi(tmppsi,mc(mchunpsi),hktype,nham)

         call phihphi(psiold,cdum(1),mc(mchpsi),hteil,nham,.false.)
         call phihunphi(psiold,mc(mchunpsi),cdum,cdum1,hteil,nham)

         call zerovxz(workc(1,2),adim)  

         call d2brakettens(psiold(adim+1),tmppsi(adim+1),workc(1,1),&
              workc(1,2),mloch,mc(pworkc),mmtczz)

         call zerovxz(workc(1,1),adim)  
         call d2calchunaloch2(workc(1,2),workc(1,1),hteil,0,nham)  
         call d2calchaloch2(workc(1,2),workc(1,1),hteil,0,1,nham)
         
         do s=1,nstate
            do t=1,nstate
               call d2ttxx1221(workc(dzpsi(s,t),1),psiold,&
                    workc(1,2),&
                    block(t),vdim(mloch,s),dim(mloch,s),ndim(mloch,s))
            enddo

!           mulitiply with +I
            call xqxxzzo((0.0_dop,1.0_dop),workc(1,2),dim(mloch,s))    
            call mmxtzza(psiold(zetf(mloch,s)),workc(1,2),&
                 psinew(zetf(mloch,s)),&
                 subdim(mloch),dim(mloch,s),dim(mloch,s))  

         enddo
!        --- end of 4th part                                        --
!        -------------------------------------------------------------


      enddo  ! of mloch
         
!------------------------------------------------------------------------
! orthonormalise required spfs  
!------------------------------------------------------------------------
      do s=1,nstate
         do m=1,nmode
            nz = 0
 100        nz = nz + 1
            if(nz .gt. dim(m,s)+2) then
               routine='d2opwfit'
               write(message,'(a,i5,a,i5)') &
               'Error in Schmidt orthonormalisation of SPFs. n =', &
                ierr,' mode =',m
               call errormsg
            endif

            call schmidtortho(psinew(zetf(m,s)),subdim(m),dim(m,s), &
                 ierr)
            if (ierr .gt. 0) then
! if new basis not complete, replace functions from old basis
               zeig=zetf(m,s)+subdim(m)*(ierr-1)
               call cpvxz(tmppsi(zeig),psinew(zeig),subdim(m))
               goto 100
            endif
         enddo
      enddo

!--------------------------------------------------------------------
! calculate new A coefficients
!--------------------------------------------------------------------
!     --- calc.  < psi_k | D | psiold>            
      ktype=0
      call hphi(psiold,mc(mchpsi),ktype,nham)
      hktype=0
      call hunphi(psiold,mc(mchunpsi),hktype,nham)
      call phihphi(tmppsi,cdum(1),mc(mchpsi),hteil,nham,.false.)
      call phihunphi(tmppsi,mc(mchunpsi),cdum,cdum1,hteil,nham)

!     --- second part                            --------------------
!     ---        
      call d2brakettens(tmppsi(adim+1),psiold(adim+1),psiold, &
           workc(1,1),0,mc(mcworkc+maxblock*maxblock*3),mmtczz)
!        --- transpose A-tensor 
      do s=1,nstate
         do t=1,nstate
            call cpmxz(workc(dzpsi(s,t),1),workc(dzpsi(t,s),2), &
                 block(s),block(t))
         enddo
      enddo

      call zerovxz(workc(1,1),adim)  
      call d2calchunaloch2(workc(1,2),workc(1,1),hteil,0,nham)  
      call d2calchaloch2(workc(1,2),workc(1,1),hteil,0,1,nham)
         
      do s=1,nstate
         do t=1,nstate
            call cpmxz(workc(dzpsi(s,t),1),workc(dzpsi(t,s),2),&
                 block(s),block(t))
         enddo
      enddo
      
      call xvxxzzo((0.0_dop,-1.0_dop),workc(1,2),adim)

      call cpvxz(workc(1,2),psinew,adim) 
!     --- end of second part                                         --
!     -----------------------------------------------------------------


!     --- third part                             ----------------------
!     ---         
      call cpvxz(psiold,workc(1,1),adim)

      call zerovxz(workc(1,2),adim)  
      call d2calchunaloch2(workc(1,1),workc(1,2),hteil,0,nham)  
      call d2calchaloch2(workc(1,1),workc(1,2),hteil,0,1,nham)

      call d2brakettens(tmppsi(adim+1),psiold(adim+1),workc(1,2),&
           workc(1,1),0,mc(pworkc),mmtczz)

      do s=1,nstate
         do t=1,nstate
            call cpmaz(workc(dzpsi(s,t),1),workc(dzpsi(t,s),2),&
                 block(s),block(t))
         enddo
      enddo 
      
      call xvxxzzo((0.0_dop,1.0_dop),workc(1,2),adim)
      
      call addvxxzo(workc(1,2),psinew,adim)     
!     --- end of third part                                          --
!     -----------------------------------------------------------------



!     - calculate barrier for "small" spfs
      do s=1,nstate
         do mloch=1,nmode
            call d2ttxx1221(tmppsi(dzpsi(s,s)),tmppsi(dzpsi(s,s)),&
                 workc(1,1),block(s),vdim(mloch,s),dim(mloch,s),&
                 ndim(mloch,s))
            call normvxz(workc(1,1),spfcutoff(mloch,s),dim(mloch,s)**2)
         enddo
      enddo


!     --- normalise new wavefunction (A-vector)
      call normvxz(psinew,norm,adim)      
      if (norm.le.tiny) then
         message = ' Norm is zero! Maybe the operator '// &
              'commutes with system Hamiltonian!' 
         write(6,'(/,a,/)') message
         call errormsg
      endif      
!      spfcutoff = wfoptol/(norm**2)
      if (lopnorm) call xvixdzo(norm,psinew,adim)
!     -----------------------------------------



!     --- care for hermiticity
      error=0.0_dop
      do s=1,nstate
         call chkherm(psinew(dzpsi(s,s)),error,block(s))
         call mkherm(psinew(dzpsi(s,s)),block(s))
         do t=(s+1),nstate
            call chkherm2(psinew(dzpsi(s,t)),psinew(dzpsi(t,s)), &
                 error,block(s),block(t))
            call mkherm2(psinew(dzpsi(s,t)),psinew(dzpsi(t,s)), &
                 block(s),block(t))            
         enddo
      enddo
!     ------------------------


      write(ilog,'(1x,a,f12.6,5x,a,f12.6)') 'Norm of A-Tensor = ',norm, &
           'Hermiticity Error = ',error
      write(ilog,*) ' '

!-----------------------------------------------------------------------
! calculate error of A-Vector,i.e., the change of the A-Vector  
!-----------------------------------------------------------------------
      call density(psinew,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      error=0.0_dop
      do m=1,nmode
         sum = 1.0_dop
         do s = 1, nstate
!.....Generate overlap-matrix between last and before-last iteration.
            call mmaxzzh(psinew(zetf(m,s)),tmppsi(zetf(m,s)), &
                 mc(pworkc),subdim(m),dim(m,s)) 
!.....Sum the squared overlap matrix-elements multiplied with 
!.....the (diagonal) density matrix.
            call sumdqdz(mr(d3mat(m,s)),mc(pworkc),sum1,dim(m,s))
            sum = sum - sum1
         enddo
         error=max(error,abs(sum))        
      enddo

!-----------------------------------------------------------------------
! see if calculation is converged
!-----------------------------------------------------------------------
      write(ilog,800) error
      if (error .gt. wfoptol) then
         if (nit .ge. wfopnit) then 
            write(ilog,'(a)') ' Maximum iteration has been reached!'
            write(ilog,'(a)') ' d2opwfit maybe not have been converged!'
            write(ilog,'(a)') ' Possible reasons are :                 '
            write(ilog,'(a)') ' - not enough spfs                      '
            write(ilog,'(a)') ' - operate_tol to small                 '
            write(ilog,'(a)') ' - operate_iter to small                '
            write(ilog,'(a)') ' - bug in subroutine d2opwfit           '
            write(ilog,'(a)') '   ( I do not hope so ;-) )             '
            write(6,'(a)') ' Maximum iteration has been reached!'
            write(6,'(a)') ' d2opwfit maybe not have been converged!'
            write(6,'(a)') ' Possible reasons are :                 '
            write(6,'(a)') ' - not enough spfs                      '
            write(6,'(a)') ' - operate_tol to small                 '
            write(6,'(a)') ' - operate_iter to small                '
            write(6,'(a)') ' - bug in subroutine d2opwfit           '
            write(6,'(a)') '   ( I do not hope so ;-) )             '

            go to 999         ! abort iteration, max iteration count has
                              ! been reached
         else                 
            call cpvxz(psinew,tmppsi,dgldim)
            go to 10          ! go to next iteration loop
         endif
      endif


! --- finished, iteration hopefully converged

 800  format(1x,'======>>>  Error = ',e12.6)

 999  continue

      operatenorm = operatenorm*norm
      write(ilog,*) ' '
      write(ilog,'(80(a1))') ('=',k=1,72)
      write(ilog,*) ' '

      return
      end subroutine d2opwfit    ! of d2opwfit


!#######################################################################
!
!               WKOPERATE
!
! Here the workspace required for the module is assigned
!
! workcdim, workrdim, workidim and workldim are passed via a common
! block in global.inc, and are checked that they are large enough for
! the module.
!
!#######################################################################
      subroutine wkoperate

use psiutil, only: maxmatwk
use mfieldsmod, only: wkmfields
use zeighpsimod, only: d1maxmatwk

      implicit none

      integer(long)       :: j

!-----------------------------------------------------------------------
! assign workspace used in module
!-----------------------------------------------------------------------
      workrdim=workrdim
      workidim=workidim
      workldim=workldim

      if (dentype .eq. 0) then
         do j = 1, 15
            if(inwfop(j).gt.0) call maxmatwk(inwfop(j))
         enddo
         call wkmfields
      else if (dentype.eq.1) then
         do j = 1, 15
            if(inwfop(j).gt.0) call d1maxmatwk(inwfop(j))
         enddo
      else if (dentype .eq. 2) then
         do j = 1, 15
            if(inwfop(j).gt.0) call maxmatwk(inwfop(j))
         enddo
      endif

      return
      end subroutine wkoperate

!**********************************************************************
!
! d1tracetens
!
!
! 01/2002 CC
!**********************************************************************
      subroutine d1tracetens(sigma,dsigma,tens1,tens2,loch,k,workc)

      implicit none

      integer(long)                     :: s,t,m,zm,z1,z2,d1,d2
      integer(long), dimension(2)       :: zw
      integer(long), intent(in)         :: loch,k

      complex(dop)                                              :: trace
      complex(dop), dimension(adim+1:dgldim), intent(in)        :: sigma
      complex(dop), dimension(hpsidim), intent(in)              :: dsigma
      complex(dop), dimension(dgldim), intent(in)               :: tens1
      complex(dop), dimension(dgldim), intent(out)              :: tens2
      complex(dop), dimension(maxblock*maxblock,3), intent(out) :: workc


      routine = 'd1tracetens'

      call cpvxz(tens1,workc(1,2),adim)

!     calculate all trace{sigma(d1) h sigma(d2)}; 
!     the result is stored in workc(1,1),i.e. m  ddim(m,s,t)**2 arrays
      do s=1,nstate
         do t=1,nstate
            zm=1
            do m=1,nmode
               z1=dzetf(m,s,t)
               do d1=1,ddim(m,s,t)
                  if (k.ne.0) then ! k.ne.0 : correlated term
                     z2=dzhpsi(m,k,t)
                  else             ! k.eq.0 : uncorrelated term
                     z2=dzetf(m,s,t)-adim
                  endif
                  do d2=1,ddim(m,s,t)
                     call trmmaxzz(sigma(z1),dsigma(z2),trace,&
                          supgdim(m),supgdim(m))
                     workc(zm-1+d2+(d1-1)*ddim(m,s,t),1) = trace
                     z2=z2+subdim(m)
                  enddo         ! of d2
                  z1=z1+subdim(m)
               enddo            ! of d1
               zm=zm + ddim(m,s,t)**2
            enddo               ! of m
         enddo                  ! of t


         do t=1,nstate
            zm=1
            zw(1)=2
            zw(2)=3
            do m=1,nmode
               if (m.ne.loch) then
                  call qtxxzz(workc(zm,1),workc(dzpsi(s,t),zw(1)),&
                       workc(dzpsi(s,t),zw(2)),&
                       vddim(m,s,t),ddim(m,s,t),nddim(m,s,t))
                  call swapsxxi(zw(1),zw(2))
               endif
               zm = zm + ddim(m,s,t)**2
            enddo               ! of m
         enddo                  ! of t

      enddo                     ! of s

      call cpvxz(workc(1,zw(1)),tens2,adim)

      return
      end subroutine d1tracetens

!********************************************************************
!
!
! 01/02 CC
!
!********************************************************************
      subroutine d2brakettens(bra,ket,tens1,tens2,loch,workc,sclproduct)

      implicit none

      integer(long)                     :: s,t,m,zm
      integer(long), dimension(2)       :: zw
      integer(long), intent(in)         :: loch

      external                                                 :: sclproduct
      complex(dop), dimension(adim), intent(in)                :: tens1
      complex(dop), dimension(adim), intent(out)               :: tens2
      complex(dop), dimension(maxblock*maxblock,3), intent(out):: workc
      complex(dop), dimension(adim+1:dgldim), intent(in)       :: bra,ket


      routine = 'd2brakettens'

      call cpvxz(tens1,workc(1,2),adim)

      do s=1,nstate
         zm=1
         do m=1,nmode
            call sclproduct(bra(zetf(m,s)),ket(zetf(m,s)),workc(zm,1), &
                 subdim(m),dim(m,s),dim(m,s))
            zm = zm + dim(m,s)**2
         enddo

         
         do t=1,nstate
            zm=1
            zw(1)=2
            zw(2)=3
            do m=1,nmode
               if (m.ne.loch) then
                  call qtxxzz(workc(zm,1),workc(dzpsi(t,s),zw(1)), &
                       workc(dzpsi(t,s),zw(2)), &
                       block(t)*vdim(m,s),dim(m,s),ndim(m,s))
                  call swapsxxi(zw(1),zw(2))
               endif

               zm = zm +dim(m,s)**2
               
            enddo
         enddo
      enddo

      call cpvxz(workc(1,zw(1)),tens2,adim)

      return
      end subroutine d2brakettens

!#######################################################################

      end module opwfmod

