!#######################################################################
!
!                 RUNINWF
!
!#######################################################################

      subroutine runinwf

use decimal,     only: long, dop
use global
use dglobal
use globmemmod
use logdat
use memdimmod, only: memdimpack
use maxv
use dvrdatmod
use griddatmod
use operdef
use dirdyn
use gwptrj
use gwptrjmod
use datenmod
use compdatmod
use runinwfmod
use hpsimod
use channels
use timing
use gapmod
use inwfmod_interface
use einsbasmod,  only: eingrd, logsbas, genmlgrd, chkbasis
use eininwfmod,  only: eininitwf
use rdopermod,   only: operinfo,setoperpsi
use realwpmod
use multkleg
use einparamod
use inputmlmod
use multilayer, only: mkdotml, setoperml
use wftree1, only: allocate_wftree
use loginwfmod, only: loginwf

use zeiginwfmod, only: zeiginwf
use iorst,       only: wrrst,splitrst,wrrstrw
use rddvrmod,    only: rddvr,dvrinfo,dvrdata
use rdopermod,   only: rdoper
use genrho1mod,  only: genrho1
use genphimod,   only: genphi
use genphi1,     only: genran
use getpsimod,   
use genpsimod,   only: symorbitals,genpsi,meigen,genswarm,exactwf
use schmidtorthomod, only: schmidtortho,schmidtortho2
use densitymod
use symlib,      only: parity,sym1d,asym1d,sym2d,asym2d,sym3d,nsym2d,&
                       symcoeff,asymcoeff,nsym2d_fixa
use opwfmod,     only: operate
use orthowfmod,  only: orthowf
use adiabwkbmod, only: adiabwkb,adiabwkbhh2
use xvlib,       only: xvxxdzo
use op1lib,      only: zerovxz, zerovxd, zerovxi, zerovxl, cpvxz, cpmxi

use iopsidef

      implicit none

      complex(dop), dimension(:), allocatable    :: psi
      real(dop), dimension(:), allocatable       :: psit1,psit2

      integer(long)                              :: i,ierr
      character(len=80)                          :: actual
      character(len=20)                          :: cerr

      complex(dop), dimension(1)                 :: cdum
      complex(dop), dimension(:,:), allocatable  :: ranspfs
      real(dop)                                  :: time,mnorm
      real(dop), dimension(1)                    :: rdum,rdum1,rdum2
      real(dop), dimension(maxdim)               :: prplt,prfluxt
      integer(long)                              :: chkdvr,chkgrd,idum,j, &
                                                    jj,m,f,symcount,s, &
                                                    chan
      character(len=16)                          :: progpart

      logical(kind=4)                            :: linwf,lremove,lopnorm
      logical(kind=4)                            :: check

      integer(long), dimension(:), allocatable   :: tmptape
      integer(long)                              :: ilbl
      logical(kind=4)                            :: lerr
      character(len=c5)                          :: filename
      integer, save                              :: tid = 0 ! timer ID
      integer, save                              :: tid1 = 0 ! timer ID
      real(dop), dimension(maxspf)               :: nsym2dnorm

      integer(long), external   :: integersize,realsize,complexsize,logicalsize

      ierr = 0

!-----------------------------------------------------------------------
! Initialise timing arrays
!-----------------------------------------------------------------------
      call timinit
      call get_timer(tid, 'Init_WF')
      call start_timer(tid)

      call fzeit(actual)
      i = index(actual,':')
      actual=actual(i-9:i+6)
      write(ilog,'(/,72a1)')  ('#',i=1,72)
      write(ilog,'(/,a)') 'Creating RESTART file '
      write(ilog,'(57x,a)')  actual(1:15)
      write(ilog,'(72a1)')  ('#',i=1,72)

!-----------------------------------------------------------------------
! Check input file open for reading
!-----------------------------------------------------------------------
      inquire(iin,opened=check)
      if (check) close(iin)
      filename=inputfile(1:inplaenge)
      open(iin,file=filename,status='old',form='formatted',err=900)

!-----------------------------------------------------------------------
! get array dimensions 
!-----------------------------------------------------------------------
      inquire(ioper,opened=check)
      if (check) close(ioper)
      filename=oname(1:olaenge)//'/oper'
      ilbl=index(filename,' ')-1
      open(ioper,file=filename(1:ilbl),form='unformatted',status='old',err=900)
      call rdmemdim(ioper)
      close(ioper)

!-----------------------------------------------------------------------
! get memory sizes for multi-packet calculations
!-----------------------------------------------------------------------
      inquire(iin,opened=check)
      if (check) close(iin)
      ilbl=inplaenge
      filename=inputfile(1:ilbl)
      open(iin,file=filename,status='old',form='formatted',err=900)
      call memdimpack(iin)
      close(iin)

!-----------------------------------------------------------------------
! get memory sizes for gwp trajectory calculations
!-----------------------------------------------------------------------
      if (lgwptrj) call memgwptrj

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      if (lgwptrj) call alloc_gwptrj
      if (ldd .or. lgwptrj .or. lddmctdh .or. lddtrans .or. ltraj) then
         call alloc_dirdyn
         call alloc_dddb
         call alloc_dercp
      endif
      if(lddmctdh.or.lgausspot)then
!         call alloc_gapop
         if(ldiabdb) call alloc_dirdyn_ddmctdh
      endif
      call alloc_psidef
      if (ldynspf) call alloc_dynwf
      if (lrealwp) call alloc_realwpmod
      call alloc_runinwfmod
      call alloc_compdat
      call alloc_hpsimod

!-----------------------------------------------------------------------
! read in parameters
!-----------------------------------------------------------------------
      call einpara

!-----------------------------------------------------------------------
! Read system / DVR information
!-----------------------------------------------------------------------
      filename=dname(1:dlaenge)//'/dvr'
      ilbl=index(filename,' ')-1
      open(idvr,file=filename,form='unformatted',status='old',err=900)
      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      close(idvr)

!-----------------------------------------------------------------------
! Read data from oper file
!-----------------------------------------------------------------------
      filename=oname(1:olaenge)//'/oper'
      ilbl=index(filename,' ')-1
      open(ioper,file=filename,form='unformatted',status='old',err=900)
      chkdvr=2
      chkgrd=1
      call operinfo(lerr,chkdvr,chkgrd)
      close(ioper)

!-----------------------------------------------------------------------
! Read / generate SPF data
!-----------------------------------------------------------------------
      call genmlgrd
      call eingrd(2)

!-----------------------------------------------------------------------
! Log SPF data
!-----------------------------------------------------------------------
      if (nmode .gt. 0) call logsbas

!-----------------------------------------------------------------------
! ML Input
!-----------------------------------------------------------------------
      if (lrunml) then
         call inputml(tape)

         if (lgraphviz) then
            filename=name(1:laenge)//'/mltree.dot'
            open(unit=idump,file=filename,form='formatted',&
                 status='unknown')
            call mkdotml(idump)
            call myflush(idump)

            filename='dot -Tpdf '//name(1:laenge)//'/mltree.dot > '// &
                      name(1:laenge)//'/mltree.pdf'
            call excmd(trim(filename))

            close(idump)
         end if

      end if

!-----------------------------------------------------------------------
! Set up operator for calculation dependant on WF settings
!-----------------------------------------------------------------------
      call setoperpsi

!-----------------------------------------------------------------------
! Read initial WF information
!-----------------------------------------------------------------------
      call eininitwf

!-----------------------------------------------------------------------
! Check basis is reasonable 
!-----------------------------------------------------------------------
      if (.not. ltraj) call chkbasis

!-----------------------------------------------------------------------
! Close input file
!-----------------------------------------------------------------------
      close(iin)

!-----------------------------------------------------------------------
! Allocate memory
!-----------------------------------------------------------------------
      call zeiginwf
      call meminwf

      if (lBEL) write(1000,*)"lBEL",lBEL,"source/geninwf/runinwf.F90"
      if(allocated(psi))deallocate(psi)
      if(allocated(mc))deallocate(mc)
      if(allocated(mr))deallocate(mr)
      if(allocated(mi))deallocate(mi)
      if(allocated(ml))deallocate(ml)
      if(allocated(multkleg_j))deallocate(multkleg_j)
      if(allocated(multkleg_m))deallocate(multkleg_m)
      if(allocated(hteil))deallocate(hteil)
      if(allocated(dicht1))deallocate(dicht1)
      if(allocated(dicht2))deallocate(dicht2)
      if(allocated(dicht3))deallocate(dicht3)
      if(allocated(dicht4))deallocate(dicht4)
      if(allocated(agmat))deallocate(agmat)
      if(allocated(gfgwp))deallocate(gfgwp)
      if(allocated(aigwp))deallocate(aigwp)
      if(allocated(zgwp))deallocate(zgwp)
      if(allocated(hung))deallocate(hung)
      if(allocated(gh))deallocate(gh)
      if(allocated(gs2))deallocate(gs2)
      if(allocated(gsinv))deallocate(gsinv)
      if(allocated(gsvek))deallocate(gsvek)

      cerr = 'runinwf_psi '
      allocate(psi(dgldim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + dgldim*complexsize()
      cerr = 'runinwf_mc  '
      allocate(mc(mcdim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + mcdim*complexsize()
      cerr = 'runinwf_mr  '
      allocate(mr(mrdim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + mrdim*realsize()
      cerr = 'runinwf_mi  '
      allocate(mi(midim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + midim*integersize()
      cerr = 'runinwf_ml  '
      allocate(ml(mldim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + mldim*logicalsize()
      if (any (lsymmultkleg .eqv. .true.)) then
         cerr = 'runinwf_multkleg_j  '
         allocate(multkleg_j(maxdim,maxklegspf),stat=ierr)
         if (ierr > 0) go to 1000
         allocmemory=allocmemory + maxdim*maxklegspf*integersize()
         cerr = 'runinwf_multkleg_m  '
         allocate(multkleg_m(maxdim,maxklegspf),stat=ierr)
         if (ierr > 0) go to 1000
         allocmemory=allocmemory + maxdim*maxklegspf*integersize()
      endif
      cerr = 'runinwf_hteil  '
      allocate(hteil(pmatdim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + pmatdim*complexsize()
      cerr = 'runinwf_dicht1  '
      allocate(dicht1(dmatdim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + dmatdim*complexsize()
      cerr = 'runinwf_dicht2  '
      allocate(dicht2(dmatdim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + dmatdim*complexsize()
      cerr = 'runinwf_dicht3  '
      allocate(dicht3(d3matdim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + d3matdim*realsize()
      cerr = 'runinwf_dicht4  '
      allocate(dicht4(dmatdim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + dmatdim*complexsize()
      if (lgwp) then
         cerr = 'runinwf_agmat  '
         allocate(agmat(agmatdim),stat=ierr)
         if (ierr > 0) go to 1000
         allocmemory=allocmemory + agmatdim*complexsize()
         cerr = 'runinwf_gfgwp  '
         allocate(gfgwp(gfgwpdim),stat=ierr)
         allocmemory=allocmemory + gfgwpdim*complexsize()
         if (ierr > 0) go to 1000
         cerr = 'runinwf_aigwp  '
         allocate(aigwp(aigwpdim),stat=ierr)
         allocmemory=allocmemory + aigwpdim*complexsize()
         if (ierr > 0) go to 1000
         cerr = 'runinwf_zgwp  '
         allocate(zgwp(zgwpdim),stat=ierr)
         allocmemory=allocmemory + zgwpdim*complexsize()
         if (ierr > 0) go to 1000
         if (inwfop(1) .gt. 0) then   ! operate
            cerr = 'runinwf_gh  '
            allocate(gh(ghmatdim),stat=ierr)
            allocmemory=allocmemory + ghmatdim*complexsize()
            if (ierr > 0) go to 1000
            cerr = 'runinwf_gs2 '
            allocate(gs2(gs2matdim),stat=ierr)
            allocmemory=allocmemory + gs2matdim*complexsize()
            if (ierr > 0) go to 1000
            cerr = 'runinwf_gsinv  '
            allocate(gsinv(gsmatdim),stat=ierr)
            allocmemory=allocmemory + gsmatdim*complexsize()
            if (ierr > 0) go to 1000
            cerr = 'runinwf_gsvek '
            allocate(gsvek(gsmatdim+1),stat=ierr)
            allocmemory=allocmemory + gsmatdim*complexsize()
            if (ierr > 0) go to 1000
            cerr = 'runinwf_hung  '
            allocate(hung(hungdim),stat=ierr)
            allocmemory=allocmemory + hungdim*complexsize()
            if (ierr > 0) go to 1000

         endif
      endif

!-----------------------------------------------------------------------
! allocate space for operator-related matrices in the multilayer modules
! This required for density matrix allocation (readinwf) + operate*psi.
!-----------------------------------------------------------------------
      if(lrunml) then
! allocate density matrices
         call allocate_wftree(tree,.true.,dicht1,dicht2,dicht3,dicht4)  

! allocate hterm and mfield matrices
         if (inwfop(1) .gt. 0) then
            call setoperml(1,.true.,.true.)
         else
            call setoperml(0,.true.,.true.)
         endif
      end if

!-----------------------------------------------------------------------
! write memory usage to log
!-----------------------------------------------------------------------
      progpart='runinwf'
      call logmem(progpart)

!-----------------------------------------------------------------------
! log initial wavefunction information
!-----------------------------------------------------------------------
      if(lbuildwf) call loginwf

!-----------------------------------------------------------------------
! fake entries just to avoid "used before set" messages
!-----------------------------------------------------------------------
      idum=1
      rdum(1)=1.0_dop
      cdum(1)=(1.0_dop,1.0_dop)
      
!-----------------------------------------------------------------------
! Zero memory arrays
!-----------------------------------------------------------------------
      call zerovxz(psi,dgldim)
      call zerovxz(mc,mcdim)
      call zerovxd(mr,mrdim)
      call zerovxi(mi,midim)
      call zerovxl(ml,mldim)
      call zerovxz(mc(mcspf1d),spf1ddim)
      call zerovxz(hteil,pmatdim)
      call zerovxz(dicht1,dmatdim)
      call zerovxz(dicht2,dmatdim)
      call zerovxz(dicht4,dmatdim)

      routine = 'runinwf'

!-----------------------------------------------------------------------
! Prepare structures in ML modules
!-----------------------------------------------------------------------
      if(lrunml) then
         if(ladiab .or. (iorthowf.gt.380) .or. lblockspf .or. &
            lblocka) goto 500
      endif

!-----------------------------------------------------------------------
! Read DVR data needed to build wavefunction (see rddvr.f for details)
!-----------------------------------------------------------------------
      dvrdata(:)=.false.
      dvrdata(1)=.true.
      dvrdata(2)=.true.
      dvrdata(5)=.true.
      dvrdata(6)=.true.
      dvrdata(7)=.true.
      dvrdata(8)=.true.
      dvrdata(9)=.true.
      dvrdata(11)=.true.
      dvrdata(12)=.true.
      chkdvr=1
      call rddvr(mr(mrort),mr(mrtrafo),rdum,rdum1,mc(mchin),mc(mcrueck), &
           mi(mifftfak),mc(mcexphin),mc(mcexprueck),mi(mijsph), &
           mi(mimsph),rdum2,chkdvr)

!-----------------------------------------------------------------------
! save tape - it may be overwritten with partial tape in oper file
!-----------------------------------------------------------------------
      if (lrunml) then
         allocate(tmptape(tapelength()))
         tmptape(1:tapelength()) = tape(1:tapelength()) 
      endif

!-----------------------------------------------------------------------
! Read data needed by the operator
!-----------------------------------------------------------------------
      chkdvr=2
      chkgrd=1
      call rdoper(mr(mrhops),chkdvr,chkgrd)

!-----------------------------------------------------------------------
! restore tape
!-----------------------------------------------------------------------
      if (lrunml) then
         tape(1:tapelength()) = tmptape(1:tapelength()) 
         deallocate(tmptape)
      endif

!-----------------------------------------------------------------------
! Set up operator again for calculation dependant on WF settings
! Add back in packets (removed when reading oper)
!-----------------------------------------------------------------------
      if (npacket .gt. 1) nstate = nstate*npacket
      call setoperpsi

!-----------------------------------------------------------------------
! get data for gwp trajectory calculations
!-----------------------------------------------------------------------
      if (lgwptrj) call gwptrjfit

!-----------------------------------------------------------------------
! wavefunction read or built here
!-----------------------------------------------------------------------
      if (lblockspf) then
         call getblk(psi,mr(mrort),dicht1,dicht2,   &
              dicht3,dicht4,mc(mcworkc), &
              mi(mijindx),agmat,mc(mchin),mc(mcrueck))

      elseif (lrestart) then
         call getinwf(psi)

!-----------------------------------------------------------------------
! Generate initial 1-dimensional spfs
!-----------------------------------------------------------------------
      else if (lbuildWF) then
         call genphi(mc(mcspf1d),mr(mrtrafo),mr(mrort),mi(mijsph),mi(mimsph), &
              mr(mrhops),mc(mcexprueck))

!-----------------------------------------------------------------------
! write restart file to save system information if reading
! spfs from a subset-restart file
!-----------------------------------------------------------------------
         if (lbuildrestart) then
            time=0.0_dop
            linwf=.true.
            call wrrst(time,psi,linwf,cdum,idum,rdum,rdum1,mi(mijindx), &
                 agmat,mi(mitrajst))
         endif

!-----------------------------------------------------------------------
! generate psi from spf1d functions
!
! dimred for BELMCG is set as 1 in genpsi
!-----------------------------------------------------------------------
         if (dentype .eq. 1) then
            call genrho1(psi,mc(mcspf1d),mr(mrtrafo),mr(mrort),agmat)
         else if (ltraj) then
            call genswarm(psi,mc(mcspf1d),mi(mitrajst))
         else
            call genpsi(psi,mc(mcspf1d),mr(mrort),mi(mijindx),agmat,0)
         endif

      endif

!-----------------------------------------------------------------------
! if exact wavefunction, set unpopulated "spfs" to zero
!-----------------------------------------------------------------------
      if (lexact .or. lrealwp) call exactwf(psi)

      if (lblocka) then
         call getblka(psi,mc(mcworkc),mi(mijindx),agmat)
      endif
 
      call myflush(ilog)

!-----------------------------------------------------------------------
! generate mode-eigenfunctions (meigenf)
!-----------------------------------------------------------------------
      if( mez .gt. 0 ) then
         call get_timer(tid1, 'meigen')
         call start_timer(tid1)
         open(irlx,file=name(1:laenge)//'/meigenf_info', &
                 status='unknown',form='formatted',err=190)
         if(lintstep) open(istp,file=name(1:laenge)//'/meigenf_steps', &
                 status='unknown',form='formatted',err=191)
         do j = 1,mez
            memode   = mearray(j,1)
            meham    = mearray(j,2)
            menum    = mearray(j,3)
            medim    = mearray(j,4)
            meselect = mearray(j,5)
            mewrite  = mearray(j,6)
            meran    = mearray(j,7)
            do mestate = 1, nstate
               allocate(ranspfs(subdim(memode),subdim(memode)))
               
               write(irlx,'(/a,i3,a,i2,a,i5,a,i3,a)')&
               '#### mode =',memode,' ,   state =',mestate,&
               ' ,  subdim =',subdim(memode),' ,  dim =',&
               dim(memode,mestate),'  ####'
               if ((lrlxt).and.(meran.ne.0)) then
!                 Generation of a thermal state
!                 "thermal" keyword (lrlxt=.true.)
                  call cpvxz(psi(zetf(memode,mestate)),ranspfs,&
                       subdim(memode)*dim(memode,mestate))
                  call meigen(ranspfs,&
                       mr(mrworkr),subdim(memode),subdim(memode))
                  call genran(ranspfs,subdim(memode),&
                       meran,mlseed)
                  call schmidtortho(ranspfs,subdim(memode),&
                       dim(memode,mestate),ierr)
                  call cpvxz(ranspfs,psi(zetf(memode,mestate)),&
                       subdim(memode)*dim(memode,mestate))
               else
                  call meigen(psi(zetf(memode,mestate)),&
                       mr(mrworkr),subdim(memode),dim(memode,mestate))
                  write(irlx,*)
               endif

               deallocate(ranspfs)
            enddo
         enddo
         close(irlx)
         if(lintstep) close(istp)
         call stop_timer(tid1)
      endif

!-----------------------------------------------------------------------
! Symmetrise two sets of SPFs
!-----------------------------------------------------------------------
      if(lsymorb) then
         do s=1,nstate
            call symorbitals(psi(zetf(symorb1,s)),psi(zetf(symorb2,s)), &
                  dim(symorb1,s),dim(symorb2,s),subdim(symorb1), &
                  subdim(symorb2),symorb1,symorb2,s)
         enddo
      endif

!-----------------------------------------------------------------------
!  Symmetrize DOFs and 2D combined modes
!  m=200.and.f=0 -> symcoeff,  f=200 -> sym3d, f=0 -> sym2d,
!  f=201 -> sym2kleg, f=202 -> nsym2d, m>100.and.f>0 -> parity.
!-----------------------------------------------------------------------
      if( msymmtr(1) .ne. 0 ) write(ilog,*)
      symcount=0
      do j = 1, 2*maxdim
 201     lremove=.true. ! if persist, then don't remove
         if( msymmtr(j) .eq. 0 ) then
            goto 200
            
         elseif( msymmtr(j) .gt. 0 ) then
            if(msymmtr(j) .gt. 65536) then
               msymmtr(j) = msymmtr(j) - 65536
               lremove=.false.
            endif
            m=mod(msymmtr(j),65536)/256
            f=mod(msymmtr(j),256)

            if(m.eq.200 .and. f.eq.0 .and. .not.lautoblk) then
               if(lremove .and. msymmtr(j).lt.65536 ) then
                  write(ilog,'(a)') 'Symmetrize coefficients'
               elseif(msymmtr(j).eq.116736) then
                  write(ilog,'(a)') 'Symmetrize coefficients, dav'
               elseif(msymmtr(j).eq.51200) then
                  write(ilog,'(a)') 'Symmetrize coefficients, persist'
               elseif(msymmtr(j).eq.182272) then
                  write(ilog,'(2a)') 'Symmetrize coefficients,',    &
                                     ' persist, dav'
               endif
               call myflush(ilog)
               symcount=symcount+1
               if(mpb.gt.0) then
                  do s=1,npackts
                     call symcoeff(psi(1+(s-1)*adim/npackts),       &
                         s,.false.,symcount)
                  enddo
                  call schmidtortho2(psi,adim/npackts,npackts,ierr,mnorm)
                  write(ilog,'(a,i4,a,1pe11.4)') '#0: ierr =', &
                        ierr,',  min-norm =',mnorm
               else
                  call symcoeff(psi,0,.false.,symcount)
               endif

            elseif(m.eq.200 .and. f.eq.0 .and. lautoblk) then
! ---- if autoblock is set, do not symmetrize because the A-vector
! ---- is generated by autoblock at a later stage.
               if(lremove) then      ! restore the old values for
                  lremove=.false.    ! autoblock
               else
                  msymmtr(j) = msymmtr(j) + 65536
               endif

            elseif (m.gt.100 .and. f.gt.0) then
               m=m-100
               if(lremove) then
                  write(ilog,'(a,i0,a,i0)') 'Parity symmetrize DOF',f,  &
                                            '  in mode',m
               else
                  write(ilog,'(a,i0,a,i0,a)')'Parity symmetrize DOF',f, &
                                             '  in mode',m,' ,  persist'
               endif
               do s = 1, nstate
                  call parity(vgdim(f),ngdim(f),gdim(f),dim(m,s),psi(zetf(m,s)))
               enddo

            elseif(f.eq.200) then
               if(lremove) then
                  write(ilog,'(a,i0)') 'Symmetrize 3-mode',m
               else
                  write(ilog,'(a,i0,a)') 'Symmetrize 3-mode',       &
                                          m,' ,  persist'
               endif
               do s = 1, nstate
                  call  sym3d(gdim(spfdof(1,m)),dim(m,s),psi(zetf(m,s)))
               enddo

            elseif(f.eq.0) then 
               if(lremove) then
                  write(ilog,'(a,i0)') 'Symmetrize 2D-mode',m
               else
                  write(ilog,'(a,i0,a)') 'Symmetrize 2D-mode',       &
                                               m,' ,  persist'
               endif
               do s = 1, nstate
                  call  sym2d(gdim(spfdof(1,m)),dim(m,s),psi(zetf(m,s)))
               enddo

            elseif (f.eq.201) then
               if(lremove) then
                  write(ilog,'(a,i0)') 'Symmetrize 4D-KLeg-mode ',m
               else
                  write(ilog,'(a,i0,a)') 'Symmetrize 4D-KLeg-mode ', &
                                          m,' ,  persist'
               endif
               do s = 1,nstate
                  call sym2d(gdim(spfdof(1,m))*gdim(spfdof(2,m)),    &
                             dim(m,s),psi(zetf(m,s)))
               enddo

            elseif (f.eq.202) then
               if(lremove) then
                  write(ilog,'(a,i0)') 'nsym2d: Sym and asym the 2D-mode',m
               else
                  write(ilog,'(a,i0,a)')                 &
                  'nsym2d: Sym and asym the 2D-mode',m,' ,  persist'
               endif
               do s = 1, nstate
                  call nsym2d(gdim(spfdof(1,m)),dim(m,s),       &
                              psi(zetf(m,s)),nsym2dnorm)
               enddo
!              if nsym2d is used, the A-vetcor must be fixed
               call nsym2d_fixa(psi,m,nsym2dnorm)

            else
               if(lremove) then
                  write(ilog,'(a,i0,a,i0)') 'Symmetrize DOF',f,'  in mode',m
               else
                  write(ilog,'(a,i0,a,i0,a)') 'Symmetrize DOF',f,       &
                                      '  in mode',m,' ,  persist'
               endif
               do s = 1, nstate
                  call  sym1d(vgdim(f),ngdim(f),gdim(f),dim(m,s),psi(zetf(m,s)))
               enddo
            endif

         elseif( msymmtr(j) .lt. 0 ) then
            if(msymmtr(j) .lt. -65536) then
               msymmtr(j) = msymmtr(j) + 65536
               lremove=.false.
            endif
            m=mod(-msymmtr(j),65536)/256
            f=mod(-msymmtr(j),256)

            if(m.eq.200 .and. f.eq.0 .and. .not.lautoblk) then
               if(lremove .and. -msymmtr(j).lt.65536 ) then
                  write(ilog,'(a)') 'Asymmetrize coefficients'
               elseif(-msymmtr(j).eq.116736) then
                  write(ilog,'(a)') 'Asymmetrize coefficients, dav'
               elseif(-msymmtr(j).eq.51200) then
                  write(ilog,'(a)') 'Asymmetrize coefficients, persist'
               elseif(-msymmtr(j).eq.182272) then
                  write(ilog,'(2a)') 'Asymmetrize coefficients,',   &
                                     ' persist, dav'
               endif
               call myflush(ilog)
               symcount=symcount+1
               if(mpb.gt.0) then
                  do s=1,npackts
                     call symcoeff(psi(1+(s-1)*adim/npackts),    &
                          s,.true.,symcount)
                  enddo
                  call schmidtortho2(psi,adim/npackts,npackts, &
                                     ierr,mnorm)
                  write(ilog,'(a,i4,a,1pe11.4)') '#0: ierr =', &
                        ierr,',  min-norm =',mnorm
               else
                  call symcoeff(psi,0,.true.,symcount)
               endif

            elseif(m.eq.200 .and. f.eq.0 .and. lautoblk) then
! ---- if autoblock is set, do not symmetrize because the A-vector
! ---- is generated by autoblock at a later stage.
               if(lremove) then      ! restore the old values for
                  lremove=.false.    ! autoblock
               else
                  msymmtr(j) = msymmtr(j) - 65536
               endif

            elseif(f.eq.0) then
               if(lremove) then
                  write(ilog,'(a,i0)') 'Asymmetrize 2D-mode',m
               else
                  write(ilog,'(a,i0,a)') 'Asymmetrize 2D-mode',   &
                                                m,' ,  persist'
               endif
               do s = 1, nstate
                  call asym2d(gdim(spfdof(1,m)),dim(m,s),psi(zetf(m,s)))
               enddo

            elseif(f.eq.201) then
               if(lremove) then
                  write(ilog,'(a,i0)') 'Asymmetrize 4D-KLeg-mode ',m
               else
                  write(ilog,'(a,i0,a)') 'Asymmetrize 4D-KLeg-mode ',  &
                                          m,' ,  persist'
               endif
               do s = 1,nstate
                  call asym2d(gdim(spfdof(1,m))*gdim(spfdof(2,m)),    &
                              dim(m,s),psi(zetf(m,s)))
               enddo

            else
               if(lremove) then
                  write(ilog,'(a,i0,a,i0)') 'Asymmetrize DOF',f,    &
                                            '  in mode',m
               else
                  write(ilog,'(a,i0,a,i0,a)') 'Asymmetrize DOF',f,  &
                                       '  in mode',m,' ,  persist'
               endif
               do s = 1, nstate
                  call asym1d(vgdim(f),ngdim(f),gdim(f),dim(m,s),psi(zetf(m,s)))
               enddo
            endif  ! f,m check

         endif

         if(lremove) then       ! remove the j-th entrance
            do jj=j,2*maxdim-1
               msymmtr(jj)=msymmtr(jj+1)
            enddo
            goto 201
         endif 
      enddo
 200  continue

!-----------------------------------------------------------------------
! operate on initial wavepacket/density-operator by a defined operator
!-----------------------------------------------------------------------
      if (inwfop(1) .gt. 0) then
         write(ilog,'(//a)') '********  OPERATE  ********'
         operatenorm = 1.0_dop
         lopnorm=.true.
         do j = 1, 15
            if (inwfop(j) .eq. 0)  exit
            call operate(psi,inwfop(j),lopnorm)
         enddo

         write(ilog,'(/a,1pe16.9,a,1pe16.9,/a)') &
          ' Operate-Norm =',operatenorm,',  Norm^2 =',operatenorm**2, &
          ' (Operate-Norm may be used in plspec, autospec, or flux)'
         if(lnonorm) then
            call xvxxdzo(operatenorm,psi,adim)
            write(ilog,'(a)') ' Normalisation removed (operate).'
         endif
      endif
      call myflush(ilog)

!-----------------------------------------------------------------------
! orthogonalise initial wavepacket aginst foreign restart files
!-----------------------------------------------------------------------
      if(iorthowf.gt.380) then
        write(ilog,'(//a)') '********  ORTHOGONALIZATION  ********'
        lwrlog=.false. ! suppress some printing
        do chan = 381, iorthowf
          if(chan.eq.equalfile) then
           call orthowf(irst,psi,mc(mcpsiref),mc(mcccdover))
           close(irst)  ! close restart file
          else
           call orthowf(chan,psi,mc(mcpsiref),mc(mcccdover))
           close(chan)  ! close orthogonalization restart file
          endif
        enddo
        lwrlog=.true.
        call myflush(ilog)
      endif

!-----------------------------------------------------------------------
! Get adiabatically corrected initial wave packet
!-----------------------------------------------------------------------
      if (ladiab) then
         if (lhh2) then
            call adiabwkbhh2(psi,mr(mrort),mc(mcexphin))
         else
            call adiabwkb(psi,mr(mrort))
         endif
      endif

!-----------------------------------------------------------------------
! Take the real part of the WF only, if the keyword realpsi is given
!-----------------------------------------------------------------------
      if (lrealpsi2) then
         call density(psi,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
         call realpsi(psi,dicht3)
      endif

!-----------------------------------------------------------------------
! Set T to 0 if it has not been specified.
!-----------------------------------------------------------------------
      if (temperature0 .lt. 0.0_dop) temperature0 = 0.0_dop

!-----------------------------------------------------------------------
! Write the restart file.
! lrlx and tinit not known here. Will be reset in runprop
!-----------------------------------------------------------------------
      time=0.0_dop
      linwf=.true.
      if (lrealwp) then
         cerr = 'runinwf_psit1 '
         allocate(psit1(dgldim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'runinwf_psit2 '
         allocate(psit2(dgldim),stat=ierr)
         if (ierr > 0) go to 1000

         psit1=dble(psi)
         psit2=dimag(psi)
         prplt=0.0_dop
         prfluxt=0.0_dop

         call wrrstrw(time,psit1(zetf(1,1)),psit2(zetf(1,1)),prplt, &
              prfluxt,linwf)

         if (allocated(psit1)) deallocate(psit1)
         if (allocated(psit2)) deallocate(psit2)
!      elseif ((lBEL) .and. (iruninwf .eq.2)) then
!         call wrini(time,psi,linwf,cdum,idum,rdum,rdum,mi(mijindx), &
!              agmat,mi(mitrajst))
      else
         call wrrst(time,psi,linwf,cdum,idum,rdum,rdum1,mi(mijindx), &
              agmat,mi(mitrajst))
      endif

!-----------------------------------------------------------------------
! set rname to name as restart file has been written to that directory
!-----------------------------------------------------------------------
      rname=name
      rlaenge=laenge

! --- For block-Davidson split the RESTART file ---
      if(lsplitrst .and. frunlevel .eq. 3 .and. mpb.gt.0) then
         j=dgldim
         call splitrst(psi,mc(mcworkc),j,time,mi(mijindx), &
              agmat,mi(mitrajst))
      endif

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_dirdyn
      call dealloc_dddb
      call dealloc_dirdyn_ddmctdh
      call dealloc_dercp
!      call dealloc_gapop
      call dealloc_gwptrj
      call dealloc_psidef
      call dealloc_dynwf
      call dealloc_runinwfmod
      call dealloc_compdat
      call dealloc_hpsimod

      cerr = 'runinwf_...'
      deallocate(psi,mc,mr,mi,ml,hteil,dicht1,dicht2,dicht3,dicht4,&
                 stat=ierr)
      if (allocated(agmat)) deallocate(agmat,stat=ierr)
      if (allocated(multkleg_j)) deallocate(multkleg_j,stat=ierr)
      if (allocated(multkleg_m)) deallocate(multkleg_m,stat=ierr)
      if (ierr > 0) go to 1001

!-----------------------------------------------------------------------
! finish timing
!-----------------------------------------------------------------------
      call stop_timer(tid)

      if (ltime) then
         call timout
         close(itim)
      endif

      write(ilog,'(/a)')' GENINWF terminated normally.'

      return

!-----------------------------------------------------------------------
  190 continue
      routine = 'runinwf'
      message = 'Cannot open data file: meigenf_info'
      call errormsg
  191 continue
      routine = 'runinwf'
      message = 'Cannot open data file: meigenf_steps'
      call errormsg

!-----------------------------------------------------------------------

 500  if(iorthowf.gt.380) then
         routine = 'runinwf'
         message = '"orthogonalise" is not allowed for ML-MCTDH'
         call errormsg
      endif

      if(ladiab) then
         routine = 'runinwf'
         message = '"correction" is not allowed for ML-MCTDH'
         call errormsg
      endif

      if(lblockspf .or. lblocka) then
         routine = 'runinwf'
         message = ' "block-SPF" and "block-A" are not allowed for ML-MCTDH'
         call errormsg
      endif

      if(lmult) then
         routine = 'runinwf'
         message = '"multi-set" is not allowed for ML-MCTDH'
         call errormsg
      endif

!-----------------------------------------------------------------------

  900 continue
      routine='Runinwf'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop

 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      write (6,*) 'Workcdim',workcdim
      stop

 1001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
      stop

      end subroutine runinwf

