!***********************************************************************
!                         transmf
!  Computes diabatic translational mean-field
!  Restrictions : . translationnal mode must not be a combined
!                 . potential operators for the translational mode
!                   must be a real vector ( or unit ) , hoptype = 1 or 2
!                 . CAPs are never taken into account
!                 . azahl must be equal to 0, intial WF must be Hartree 
!
!  2/98 MCH  (07/01 HDM, s>1 allowed) (06/2003 HDM, potgen1d)
!
! contains : transmf,transmfhh2,transmf2,transmf2lin,transmf3,diabtmf3
!
!***********************************************************************
      module transmfmod

use decimal, only: long, dop
use global
use xvlib,   only: vvaxzz, xvxxzza
use op1lib,  only: zerovxd, zerovxz, initvxi, initvxz, initvxd
use mmlib,   only: ddxxdd
use op2lib,  only: addvxxdo

      implicit none
      private
      public :: transmf,transmfhh2
      contains
       
!***********************************************************************

      subroutine transmf(tmf,psi,ort,s,mtr,e0intern,e0trans,e0trpot)

use logdat
use maxv, only: maxdim
use dvrdatmod
use griddatmod
use psidef
use runinwfmod
use operdef
use channels

use hphimod, only: hcall

      implicit none

      integer(long)                                :: m,k,i,k1,h,k2,j,m1
      integer(long), intent(in)                    :: s,mtr
      integer(long), dimension(maxdim)             :: ind,indz
      real(dop), intent(out)                       :: e0intern,e0trans,e0trpot
      real(dop), dimension(sdim), intent(out)      :: tmf

      complex(dop), dimension(dgldim), intent(in):: psi
      complex(dop), dimension(sdim)              :: vpsi,ctmf,unitv
      complex(dop)                               :: vmfm,vmfk,cinten,ctrans,csum
      real(dop), dimension(*), intent(in)        :: ort
      complex(dop)                               :: z
      real(dop)                                  :: znorm

!   the potential operators  for the translational mode 
!   must be a real vector ( or unit ) , hoptype = 1 or 2

      do k=1,kzahl(1)
         h=hamilton(ftr,k)
         if ( htmtype(h) .gt. 2 ) then
             routine = 'transmf'
             message = &
             'operator of translational mode is not a real array'
             call errormsg
         endif
      enddo

      call zerovxd(tmf,subdim(mtr))
      call zerovxz(ctmf,subdim(mtr))

!  computation of internal energy (cinten=e0intern).
!  Sum over all uncorrelated terms of state s, except those of the 
!  translational dof and the CAP terms.
!  ctrans=e0trans is the separable energy of the translational mode.
!  e0trpot is the separable potential energy at the last grid point.
!  The translational mean field, tmf, is initialised with the 
!  separable potential of the translational DOF.

      cinten=(0.0_dop,0.0_dop)
      ctrans=(0.0_dop,0.0_dop)

      call initvxi(indz,nmode)
      do i=1,adim
         z=psi(zpsi(s)+i-1)
         znorm = dble(z)*dble(z) + dimag(z)*dimag(z)
         if (znorm .lt. 10.0_dop*macheps) goto 22
         if (indz(mtr).gt.1) then
            routine = 'transmf'
            message =&
            'No of particle in translational mode should be 1'
            call errormsg
         endif
         call initvxi(ind,nmode)
         do j=1,adim
            z=psi(zpsi(s)+j-1)
            znorm = dble(z)*dble(z) + dimag(z)*dimag(z)
            if (znorm .lt. 10.0_dop*macheps) goto 21
            do m=1,nmode
               k1=zhun(m,s,1)
               k2=zhun(m,s,1)+khzahl(m,s,1)-1
               do k=k1,k2
                  if(autocap(k).eq.0 .and. ki(k).eq.s &
                                     .and. kf(k).eq.s) then
                     call hcall(m,k,vpsi, &
                          psi(zetf(m,s)+(ind(m)-1)*subdim(m)),subdim(m))
                     call vvaxzz(psi(zetf(m,s)+(indz(m)-1)*subdim(m)),&
                          vpsi,vmfm,subdim(m))
                     vmfm=vmfm*koeff(k)*&
                          dconjg(psi(zpsi(s)+i-1))*psi(zpsi(s)+j-1)
                     if (m.ne.mtr) then
                        do m1=1,nmode
                           if (m1.ne.m) then
                              if (ind(m1).ne.indz(m1)) goto 31
                           endif
                        enddo
                        cinten=cinten+vmfm
  31                    continue
                     else
                        do m1=1,nmode
                           if (ind(m1).ne.indz(m1)) goto 32
                        enddo
                        ctrans=ctrans+vmfm
  32                    continue
!                       the following should only be done once for all
!                       i,j -- but keeping it here is just inefficient,
!                       not wrong.
                        h=hamilton(ftr,k)
                        call potgen1d(h,ort,gdim(ftr),tmf)
                        e0trpot=tmf(gdim(ftr))
                     endif
                  endif
               enddo
            enddo

  21        do m=1,nmode
               if (ind(m).lt.dim(m,s)) then
                  ind(m)=ind(m)+1
                  goto 6
               else
                  ind(m)=1
               endif
            enddo
  6         continue
         enddo ! loop over j

  22     do m=1,nmode
            if (indz(m).lt.dim(m,s)) then
               indz(m)=indz(m)+1
               goto 7
            else
               indz(m)=1
            endif
         enddo
  7      continue
      enddo ! loop over i

      if (dimag(cinten).gt.tiny) then
         write(ilog,*) 'internal energy is complex! - stop'
         stop
      else
         e0intern=dble(cinten)
      endif
      if (dimag(ctrans).gt.tiny) then
         write(ilog,*) 'translational energy is complex! - stop'
         stop
      else
         e0trans=dble(ctrans)
      endif

!  return if only energy distribution is required, without diabatic correction
      if (.not.ldia) return

!  contribution from all the correlated terms to the mean field 
!  integration is done over all dof except the translational dof.

      do 50 k=1,kzahl(1)
         if(ki(k).ne.s .or. kf(k).ne.s) goto 50
         csum=(0.0_dop,0.0_dop)
         call initvxi(indz,nmode)
         do i = 1,adim
            z=psi(zpsi(s)+i-1)
            znorm = dble(z)*dble(z) + dimag(z)*dimag(z)
            if (znorm .lt. 10.0_dop*macheps) goto 12
            call initvxi(ind,nmode)
            do j = 1,adim
               z=psi(zpsi(s)+i-1)
               znorm = dble(z)*dble(z) + dimag(z)*dimag(z)
               if (znorm .lt. 10.0_dop*macheps) goto 11
               vmfk=koeff(k)
               do m=1,nmode
                  if(m.ne.mtr) then
                     call hcall(m,k,vpsi,&
                          psi(zetf(m,s)+(ind(m)-1)*subdim(m)),subdim(m))
                     call vvaxzz(psi(zetf(m,s)+(indz(m)-1)*subdim(m)),&
                          vpsi,vmfm,subdim(m))
                     vmfk=vmfk*vmfm
                  endif
               enddo
               csum=csum+vmfk*dconjg(psi(zpsi(s)+i-1))*psi(zpsi(s)+j-1)

  11           do m=1,nmode
                  if (ind(m).lt.dim(m,s)) then
                     ind(m)=ind(m)+1
                     goto 1
                  else
                     ind(m)=1
                  endif
               enddo
  1            continue
            enddo ! loop over j

  12        do m=1,nmode
               if (indz(m).lt.dim(m,s)) then
                  indz(m)=indz(m)+1
                  goto 2
               else
                  indz(m)=1
               endif
            enddo
  2         continue
         enddo ! loop over i

         call initvxz(unitv,subdim(mtr))
         call hcall(mtr,k,vpsi,unitv,subdim(mtr))
         call xvxxzza(csum,vpsi,ctmf,subdim(mtr))
  50  continue ! end loop over k

! add internal energy and separable translational potential.
         
      do i=1,subdim(mtr)
         ctmf(i)=ctmf(i) + cinten 
            if (dimag(ctmf(i)).gt.tiny) then
               write(ilog,*) &
               'translational mean-field is complex! - stop '
               stop
            endif
         tmf(i)=tmf(i)+dble(ctmf(i))
      enddo

       end subroutine transmf
!#######################################################################
!
!                     TRANSMF HH2
!
!    04/97 AJ
!
!#######################################################################

      subroutine transmfhh2(tmf,psi,ort)

use dvrdatmod
use griddatmod
use psidef
use runinwfmod

      implicit none

      integer(long)                               :: nef
      real(dop), dimension(ortdim), intent(in)    :: ort
      real(dop), dimension(sdim), intent(out)     :: tmf
      complex(dop), dimension(dgldim), intent(in) :: psi

      if (ldia) then
         if (nmode.eq.2) then
            if (azahl.eq.0) then
               nef= isbaspar(4,1,1)-1
               call transmf2(tmf,psi(zetf(2,1)),ort(zort(1)) &
                    ,ort(zort(2)),subdim(1),subdim(2),dim(2,1),nef,tfac)
            else
               call transmf2lin(tmf,psi,psi(zetf(2,1)),ort(zort(1)) &
                    ,ort(zort(2)),subdim(1),subdim(2),dim(1,1),dim(2,1))
            endif
         elseif (nmode.eq.3) then
            if (azahl.ne.0) stop
            call transmf3(tmf,psi,psi(zetf(2,1)),psi(zetf(3,1)) &
                 ,ort(zort(1)),ort(zort(2)),ort(zort(3)),subdim(1) &
                 ,subdim(2),subdim(3),dim(1,1),dim(2,1),dim(3,1))
         endif
      elseif (lad) then
         stop
      endif

      return
      end subroutine transmfhh2

!***********************************************************************
!
!                     TRANSMF2
!
!***********************************************************************

      subroutine transmf2(tmf,phi2,ort1,ort2,subdim1,subdim2,dim2,nef, &
                 tfac)


      implicit none

      integer(long), parameter :: imod=2,icoll=1
      real(dop), parameter     :: rminh2=0.4_dop

      integer(long)             :: g1,g2,e0
      integer(long), intent(in) :: subdim1,subdim2,dim2,nef
      real(dop)                 :: rd,rv,r1,r2,r3,v

      real(dop), intent(in)                             :: tfac
      real(dop), dimension(subdim1), intent(in)         :: ort1
      real(dop), dimension(subdim1), intent(out)        :: tmf
      real(dop), dimension(subdim2), intent(in)         :: ort2
      complex(dop), dimension(subdim2,dim2), intent(in) :: phi2

      e0=nef+1

      do g1=1,subdim1
         tmf(g1)=0.0_dop
         rd=ort1(g1)
         do g2=1,subdim2
            rv=ort2(g2)
            r1=max(rminh2,rd-tfac*rv)
            r2=max(rminh2,rv)
            r3=r1+r2
            call lsth(r1,r2,r3,v,imod,icoll)
            tmf(g1)=tmf(g1)+dble(dconjg(phi2(g2,e0))*v*phi2(g2,e0))
         enddo
      enddo

      return
      end subroutine transmf2


!***********************************************************************
!
!                     TRANSMF2LIN
!
!***********************************************************************

      subroutine transmf2lin(tmf,acoeff,phi2,ort1,ort2,subdim1,&
                             subdim2,dim1,dim2)

use griddatmod
use psidef
use runinwfmod

      implicit none

      integer(long), parameter :: imod=2,icoll=1
      real(dop), parameter     :: rminh2=0.4_dop

      integer(long)             :: e2a,e2b,g1,g2
      integer(long), intent(in) :: subdim1,subdim2,dim1,dim2
      real(dop)                 :: rd,rv,r1,r2,r3,v

      real(dop), dimension(subdim1), intent(in)         :: ort1
      real(dop), dimension(subdim1), intent(out)        :: tmf
      real(dop), dimension(subdim2), intent(in)         :: ort2
      complex(dop), dimension(dim1,dim2), intent(in)    :: acoeff
      complex(dop), dimension(subdim2,dim2), intent(in) :: phi2
      complex(dop)                                      :: ztmp

      do g1=1,subdim1
         tmf(g1)=0.0_dop
         rd=ort1(g1)
         do g2=1,subdim2
            rv=ort2(g2)
            r1=max(rminh2,rd-tfac*rv)
            r2=max(rminh2,rv)
            r3=r1+r2
            call lsth(r1,r2,r3,v,imod,icoll)
            do e2b=1,dim2
               do e2a=1,dim2
                  ztmp=dconjg(acoeff(1,e2a))&
                       *dconjg(phi2(g2,e2a))*v&
                       *acoeff(1,e2b)*phi2(g2,e2b)
                  tmf(g1)=tmf(g1)+dble(ztmp)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine transmf2lin

!***********************************************************************
!
!                        TRANSMF3
!
!    INPUT:  phi2,phi3,ort1,ort2,ort3,
!            subdim2,subdim3 (standard arrays, see calling 
!                                         routine)
!    OUTPUT: tmf (energy shifted diabatic translational mean-field)
!
!
!  Note: It is assumed that the single-particle functions of the
!        vibrational and rotational mode are real -- though they 
!        are declared as complex and that exactly one MCTDH expansion 
!        coefficient A1ij equals one and that all others are zero.
!
!    12/96 AJ 
!
!***********************************************************************

      subroutine transmf3(tmf,coeff,phi2,phi3,ort1,ort2,ort3 &
           ,subdim1,subdim2,subdim3,dim1,dim2,dim3)

      implicit none

      integer(long)             :: e2,e3,e20,e30
      integer(long), intent(in) :: subdim1,subdim2,subdim3,dim1,dim2,dim3

      real(dop), dimension(subdim1), intent(in)           :: ort1
      real(dop), dimension(subdim1), intent(out)          :: tmf
      real(dop), dimension(subdim2), intent(in)           :: ort2
      real(dop), dimension(subdim3), intent(in)           :: ort3
      complex(dop), dimension(subdim2,dim2), intent(in)   :: phi2
      complex(dop), dimension(subdim3,dim3), intent(in)   :: phi3
      complex(dop), dimension(dim1,dim2,dim3), intent(in) :: coeff

!-----------------------------------------------------------------------
! Get non-zero MCTDH expansion coefficient.
!-----------------------------------------------------------------------
      e20 = 0
      e30 = 0
      do e3=1,dim3
         do e2=1,dim2
            if (abs(coeff(1,e2,e3)).gt.0.0_dop) then
               e20=e2
               e30=e3
               goto 10
            endif
         enddo
      enddo
      if( e20.eq.0  .or. e30.eq.0 ) then
         write(6,*) 'e20.or.e30 = 0, No coefficient found '
         write(2,*) 'e20.or.e30 = 0, No coefficient found '
         stop
      endif

!-----------------------------------------------------------------------
! Compute diabatic translational mean-field.
!-----------------------------------------------------------------------
 10   continue
      call diabtmf3(tmf,phi2(1,e20),phi3(1,e30),ort1,ort2,ort3 &
           ,subdim1,subdim2,subdim3)


      return
      end subroutine transmf3

!***********************************************************************
!
!                         DIABTMF3
!
!  Computes diabatic translational mean-field.
!
!***********************************************************************

      subroutine diabtmf3(tmf,phi2,phi3,ort1,ort2,ort3,subdim1 &
                          ,subdim2,subdim3)

use griddatmod
use psidef
use runinwfmod

      implicit none

      integer(long), parameter :: imod=2,icoll=0
      real(dop), parameter     :: rminh2=0.4_dop

      integer(long)                                :: g1,g2,g3
      integer(long), intent(in)                    :: subdim1,subdim2,subdim3
      real(dop)                                    :: rd,rv,cphi,r1,r2,r3,x,v
      real(dop), dimension(subdim1), intent(in)    :: ort1
      real(dop), dimension(subdim1), intent(out)   :: tmf
      real(dop), dimension(subdim2), intent(in)    :: ort2
      real(dop), dimension(subdim3), intent(in)    :: ort3
      complex(dop), dimension(subdim2), intent(in) :: phi2
      complex(dop), dimension(subdim3), intent(in) :: phi3


      do g1=1,subdim1
         tmf(g1)=0.0_dop
         rd=ort1(g1)
         do g3=1,subdim3
            cphi=cos(ort3(g3))
            do g2=1,subdim2
               rv=ort2(g2)
               x=rv*rv/4.0_dop+rd*rd
            r1=max(rminh2,sqrt(rd**2+(tfac*rv)**2-2.0_dop*tfac*rd*rv*cphi))
            r2=max(rminh2,rv)
            r3=max(rminh2, &
              sqrt(rd**2+((1.0_dop-tfac)*rv)**2+2.0_dop*(1.0_dop-tfac)*rd*rv*cphi))
               call lsth(r1,r2,r3,v,imod,icoll)               
               tmf(g1)=tmf(g1) &
                    +dble(phi2(g2))*dble(phi2(g2)) &
                    *dble(phi3(g3))*dble(phi3(g3))*v
            enddo
         enddo
      enddo
                           
      return
      end subroutine diabtmf3

!#######################################################################
!                potgen1d
!
! This subroutine generates 1D potential operators.
! The operator can be a sum of products of factors.
! Factors which are not a potential will be zeroed.
! This routine is used in transmf (geninwf/transmf.F).
! The output is the real matrix hopd.
! HDM 6/2003
!------------------------------------------------------------------------

      subroutine potgen1d(h,ort,gdim1,hopd)

use dvrdatmod
use griddatmod
use operdef
use runopermod

use callanldmod, only: callanld

      implicit none

      integer(long)                                :: f,nh,hl,zeig,zeig1,&
                                                      zeig2,zeig3,g,pottyp
      integer(long), intent(in)                    :: h,gdim1
      real(dop)                                    :: v
      real(dop), dimension(gdim1), intent(out)     :: hopd
      real(dop), dimension(1)                      :: rdum
      real(dop), dimension(gdim1), intent(in)      :: ort
      real(dop), dimension(3*maxgdim)              :: vlocal
      zeig=1
      zeig1=zeig+gdim1
      zeig2=zeig1+gdim1

      call zerovxd(hopd,gdim1)
      call zerovxd(vlocal,3*maxgdim)

!-----------------------------------------------------------------------
! Load the first term (nh=1) into work(zeig).
!-----------------------------------------------------------------------

      f=htmdof(h)
      pottyp=0
      hl=hterm(1,h)
      if (hopfile(hl).le.300 .or. hopfile(hl).ge.400) goto 50
      do g=1,gdim1
         call callanld(vlocal(g),ort(g),hopilab(hl),hopfile(hl), &
              hopipar(1,hl),hoprpar(1,hl),rdum(1),pottyp,hoplab(hl),mass(f))
      enddo

!-----------------------------------------------------------------------
! multiply out terms and sum terms (when hl=0).
!-----------------------------------------------------------------------

 50   do nh=2,htmfac(h)
         hl=hterm(nh,h)
!........hl=0 separates two products which are to be summed.
         if (hl .eq. 0) then
            call addvxxdo(vlocal(zeig),hopd,gdim1)
            call initvxd(vlocal(zeig),gdim1)
         else
            call zerovxd(vlocal(zeig1),gdim1)
            if (hopfile(hl).gt.300 .and. hopfile(hl).lt.400) then
               do g=1,gdim1
                  call callanld(v,ort(g),hopilab(hl),hopfile(hl), &
                       hopipar(1,hl),hoprpar(1,hl),rdum(1),pottyp, &
                       hoplab(hl),mass(f))
                  vlocal(zeig1+g-1)=v
               enddo
               call ddxxdd(vlocal(zeig),vlocal(zeig1),vlocal(zeig2), &
                    gdim1)
            else
               call zerovxd(vlocal,3*maxgdim)
            endif

            zeig3=zeig
            zeig=zeig2
            zeig2=zeig3
         endif
      enddo

      call addvxxdo(vlocal(zeig),hopd,gdim1)
      
      !Added MAP 21/10/21 so splines can be used with energd file
      if(allocated(splpaths))deallocate(splpaths)
      if(allocated(splunits))deallocate(splunits)

      return
      end subroutine potgen1d

      end module transmfmod


