!#######################################################################
!  zeiginwf. Zeiger (i.e. pointer) routines for geninwf step.
!
!  calls zeiginwf0 for wavepacket propagation (dentype=0)
!  calls zeiginwf1 for density Type I         (dentype=1)    
!  calls zeiginwf0 for density Type II        (dentype=2)
!  
!  04/2001 CC
!            
! contains: zeiginwf,zeiginwf0,zeiginwf1,wkinwf,wkinwf1
!#######################################################################
      module zeiginwfmod

use global
use dglobal
use decimal, only: long, dop

use dvrdatmod
use griddatmod
use psidef
use compdatmod
use hpsimod
use runpropmod
use channels
use multkleg
use runinwfmod
use operdef

use iopsidef, only: psidat,gwpdat
use iorst, only: rstinfo,chkrsttyp
use iogrddef, only: griddat
use compdat, only: savecsys,savecpsi,getcsys,getcpsi
use zeighpsimod
use rdopermod, only: num_kleg_sbas,num_wig_sbas
use orthowfmod, only: zeigowf
use psiutil, only: maxmatwk

use logdat
use runinwfmod
use runpropmod
use channels

use orthowfmod, only: zeigowf
use wftree, only: adim_wftree

use getpsimod, only: wkgetpsi
use psiutil, only: maxmatwk
use opwfmod, only: wkoperate

    
      implicit none
      private
      public :: zeiginwf
      contains

!#######################################################################

      subroutine zeiginwf

      implicit none

      intrdim = 1           
      intpsidim = 1        
      if (dentype.eq.0) then
         call zeiginwf0
         if(iorthowf.gt.80) call zeigowf  ! orthogonalize WF (impr.relax.)
      else if (dentype .eq. 1) then
         call zeiginwf1
      else if (dentype.eq.2) then
         call zeiginwf0
      else
         write(6,'(a,I1)') ' Unkown dentype! dentype = ',dentype
         message = 'error!'
         call errormsg
      endif

      return
      end subroutine zeiginwf

!#######################################################################
!  zeiginwf0 for wavepacket propagation (dentype=0) and density Type II
!#######################################################################

      subroutine zeiginwf0

      implicit none

      integer(long)       :: s,m,dgldimold,ilbl,dim2,chkdvr,chkgrd,chkpsi,&
                             chkprp,mblock,mblock1,ipop,f,f1,zeig,fadim,&
                             dentypeold,irst0,mxdm,nk,n0,emax,nw,i,&
                             agmatdimold,dmatdim1,d3matdim1,mablock,&
                             mxmxblk,dimfmax

      integer(long), dimension(maxsta)              :: fblock
      integer(long), dimension(nmode,nstate)        :: dim1
      real(dop)                                     :: rtmp
      character(len=c5)                             :: filename
      logical(kind=4)                               :: linwf,lerr,lmult2,&
                                                       leb2,lmulpack2,check

      routine='Zeiginwf0'
      agmatdimold=0
      dgldimold=0
      dmatdim1=0
      d3matdim1=0
      intrdim=1         
      intpsidim=1
      mblock=0
      mablock=0
      mxmxblk=0

! --- Save System data.
      call savecsys(1)
      call savecpsi(1)
      lmult2=lmult
      leb2=leb
      lmulpack2=lmulpack
      irst0=irst

      inquire(irst,opened=check)
      if (check) close(irst)

! --- file: read restart File for size of old wavefunction ---
      if (lbuildrestart) then
         dentypeold = dentype
         filename=rname(1:rlaenge)//'/restart'
         open(irst,file=filename,form='unformatted',status='old',err=34)
         goto 36
 34      filename=rname(1:rlaenge)
         open(irst,file=filename,form='unformatted',status='old',err=99)

 36      continue
         call chkrsttyp
         if (psitype .eq. 1 .or. psitype == 11) then  ! exact WF/realwp
            call savecsys(1)
            call savecpsi(1)

            chkdvr=2
            chkgrd=1
            chkpsi=1
            chkprp=0
            call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)

            dgldimold=dgldim
            agmatdimold=0

            mblock=0
            do s=1,nstate
               mblock=max(mblock,block(s))
            enddo

            call getcsys(1)
            call getcpsi(1)
            call griddat

         else
            call savecsys(1)
            call savecpsi(1)

!           nstate1=nstate
            do s=1,nstate
               do m=1,nmode
                  dim1(m,s)=dim(m,s)
               enddo
            enddo

           do f=1,ndof 
             
!              if (sbasis(f) .eq. 30 .or. sbasis(f).eq.1) then 
              if (lbuildrestart) then 
                chkdvr=3
                chkgrd=3
                chkpsi=1
                chkprp=0
                call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
              else
                chkdvr=6
                chkgrd=4
                chkpsi=1
                chkprp=0
                call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
              endif
            enddo

            dgldimold=dgldim
            agmatdimold=agmatdim

            mblock=0
            do s=1,cnstate(1)
               mblock1=1
               do m=1,nmode
                  dim2=max(dim(m,s),cdim(m,s,1))
                  mblock1=mblock1*dim2
               enddo
               mblock=max(mblock,mblock1)
            enddo

            call getcsys(1)
            call getcpsi(1)
            call griddat
         endif 
         
!        the calls to subroutine rstinfo changes the dentype
         if (dentypeold.ne.dentype) then
            if ((dentypeold.eq.2).and.(dentype.eq.0)) then
               dentype = 2
            else
               routine = 'zeiginwf0'
               message = 'This wavefunction to density-operator '// &
                    'transformation is not possible (yet) !'
               call errormsg
            endif
         endif

         if (equalfile .eq. 0) close(irst)

      elseif(lblockspf.or.lblocka) then  ! keywords "block-SPF", "block-A"
!.....Note that lblockspf and lblocka may be both set.
!.....That is why we have the following if statement.
        if(lblockspf) then   ! keyword "block-SPF"
           if(mpb.ne.nmode) then
              routine='zeiginwf'
              message='This is not a multi-packet single-set run!'//&
                      ' Use read-inwf rather than block-SPF.'
              call errormsg
           endif
           irst0=irst
           irst=591
           filename=rname(1:rlaenge)//'/restart'
           open(irst,file=filename,form='unformatted',status='old',err=35)
           goto 37
 35        filename=rname(1:rlaenge)
           open(irst,file=filename,form='unformatted',status='old',err=99)

 37        continue
           mblock=0
           mxdm=0
           chkdvr=6
           chkgrd=6
           chkpsi=1
           chkprp=0
           call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
           if (lerr) then
              routine='zeiginwf'
              message='Inconsistent restart file used for'//&
                      ' block-SPF. See log file.'
              call errormsg
           endif
           call psidat
           dmatdim1=max(dmatdim1,dmatdim)
           d3matdim1=max(d3matdim1,d3matdim)
           write(ilog,'(a,i7)')'rst-file block-SPF.  dgldim =',dgldim
           dgldimold=max(dgldim,dgldimold)
           do s=1,nstate
              mblock1=1
              do m=1,nmode
                 mxdm=max(dim(m,s)*subdim(m),mxdm)
                 dim2=max(dim(m,s),cdim(m,s,1))
                 mblock1=mblock1*dim2
              enddo
              mblock=max(mblock,mblock1)
           enddo               ! s=1,nstate

           mblock=mblock+mxdm
        else

           mblock=0
        endif      ! if(lblockspf)

! --- block-A: read restart File(s) for size of old wavefunction ---
! --- block-A restart file(s) have been already opened in eininwf ---
        if(lblocka) then
           mablock=0
           do irst = 592, 591+npackts
              chkdvr=6
              chkgrd=6
              chkpsi=1
              chkprp=0
              call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
              if (lerr) then
                 write(ilog,'(a,i4)') 'block-A : irst =', irst
                 write(ilog,'(a,10i3)') 'chkerr:',(chkerr(s),s=1,10)
                 routine='zeiginwf'
                 message='Inconsistent restart file used in'//&
                         ' block-A block. See log file.'
                 call errormsg
              endif
              call psidat
              dmatdim1=max(dmatdim1,dmatdim)
              d3matdim1=max(d3matdim1,d3matdim)
              write(ilog,'(a,i4,a,i7)')'rst-file',irst,'  dgldim =',dgldim
              dgldimold=max(dgldim,dgldimold)
              do s=1,cnstate(1)
                 mblock1=1
                 do m=1,nmode
                    dim2=max(dim(m,s),cdim(m,s,1))
                    mblock1=mblock1*dim2
                 enddo
                 mablock=max(mablock,mblock1)
              enddo               ! s=1,nstate
           enddo                  ! irst=592,591+npackts

        else

           mablock=0
        endif      ! if(lblocka)

      else
         mblock=0
         mablock=0
         dgldimold=0
         agmatdimold=0
      endif

!........Reset system data.
      call getcsys(1)
      call getcpsi(1)
      lmult=lmult2
      leb=leb2
      lmulpack=lmulpack2
      irst=irst0
      if (psitype .eq. 5) lrunml = .true.
      call griddat

!-----------------------------------------------------------------------
! calculate dim,vdim and ndim for each mode.
! size of block (number of MCTDH coefficients)
! and pointers for wavefunction
! maxmatwk(0) computes maxblock
!-----------------------------------------------------------------------
      call psidat
      dmatdim=max(dmatdim1,dmatdim)
      d3matdim=max(d3matdim1,d3matdim)
      maxblock=0
      call maxmatwk(0)

!-----------------------------------------------------------------------
! get maximum block of new wavefunction for a build restart 
!-----------------------------------------------------------------------
      if (lbuildrestart) then
         do s=1,nstate
            mblock=max(mblock,block(s))
         enddo
      endif

!-----------------------------------------------------------------------
! pointers for GWP arrays
!-----------------------------------------------------------------------
      call gwpdat

!-----------------------------------------------------------------------
! calculate data for spf1d array, which holds 1d "spfs"
!-----------------------------------------------------------------------
      do s=1,nstate
         do f=1,ndof
            ipop=isbaspar(4,f,s)
            m=dofspf(f)
            if (m .eq. 0 .or. m .gt. nmode) then
               dimf(f,s)=0
               dimfran(f)=0
            else
! check that complete discrete basis can be generated
               if (sbasis(f) .eq. 0) then
                  if (f .ne. febcont) then
                     dimf(f,s)=max(gdim(f),dim(m,s))
                  endif
! if k dof in a kleg function, no spfs are to be generated
               else if (sbasis(f) .eq. 6) then
                  if(basis(max(f-1,1)).eq.7 .or. basis(max(f-1,1)) &
                         .eq.33 .or. basis(max(f-2,1)).eq.33) then
                     dimf(f,s)=1
                  else
                     dimf(f,s)=dim(m,s)
                  endif
! if kleg or Wigner dof:
               else if  (sbasis(f).eq.5 .or. sbasis(f).eq.8) then
                  call num_kleg_sbas(nk,m)
                  call num_Wig_sbas(nw,m)
                  if (nk.eq.1 .and. nw.eq.0) then
!                    Only one kleg in mode.
                     dimf(f,s)=max(gdim(f),gdim(f+1),dim(m,s))
                  elseif (nk.eq.0 .and. nw.eq.1) then
!                    Only a Wigner and nothing else.
                     dimf(f,s)=max(gdim(f),gdim(f+1),gdim(f+2),dim(m,s))
                  else
!                    more than one kleg/Wigner in mode: genmultkleg will
!                    create the spfs for the combined mode from the
!                    kleg/Wigner spfs in an asymmetrical way, so lets
!                    try to generate only few 1D spfs.
!                    since genmultkleg only considers excitations of the
!                    other dofs in this mode up to emax=2, solve:
!                      emax**n0 * dimf**(nk+nw) >= dim
!                    where n0 is the number of non-kleg/non-Wigner
!                    dofs in the mode
                     emax=2 ! TODO: make configurable
                     n0=nspfdof(m)-2*nk-3*nw
                     rtmp=(dim(m,s)/dble(emax**n0))**(1.d0/(nk+nw))
                     dimf(f,s)=int(rtmp+1.d0-tiny) ! NEWKLEG
!                    but of course dimf can't be bigger than the grid
                     if (sbasis(f).eq.5 .and. &
                         dimf(f,s).gt.gdim(f)*gdim(f+1)) then
                        dimf(f,s)=gdim(f)*gdim(f+1)
                     elseif (sbasis(f).eq.8 .and. &
                         dimf(f,s).gt.gdim(f)*gdim(f+1)*gdim(f+2)) then
                        dimf(f,s)=gdim(f)*gdim(f+1)*gdim(f+2)
                     endif
!                    however, in case of symmetrization, the multi-KLeg
!                    SPFs must be (a)symmetrical with respect to the
!                    exchange of two KLegs, so not all combinations of
!                    1D spfs are allowed; thus we need more 1D spfs.
                     if (sbasis(f).eq.5 .and. lsymmultkleg(m)) then
                        dimf(f,s)=min(gdim(f)*gdim(f+1),maxklegspf)
                     endif
!                    or let the user override it in the InitWF-Section
                     if (sbasis(f).eq.5 .and. isbaspar(6,f,s).ne.0) then
                        dimf(f,s)=min(gdim(f)*gdim(f+1),isbaspar(6,f,s))
                     endif
                  endif

               else if (basis(f) .eq. 25) then   ! GWPs
                  dimf(f,s)=max(ipop,gwpdim(m,s))
               else if (basis(f) .eq. 26 .or. basis(f) .eq. 27) then ! trajectories
                  dimf(f,s)=max(ipop,dim(m,s))

               else
                  dimf(f,s)=min(gdim(f),max(ipop,dim(m,s))) ! Default value
               endif
            endif ! if(m.eq.0) else
         enddo ! f
      enddo  ! s


! set dimf for mapping dof (f1=isbaspar(1,f,s)), keyword 'map'.
      do s=1,nstate
         do f=1,ndof
            if(sbasis(f).eq.7) then
               do i=1,3
                  f1=isbaspar(i,f,s)
                  if(f1.gt.0) then
                     dimf(f1,s)=max(dimf(f,s),dimf(f1,s))
                     dimf(f1,s)=min(gdim(f),dimf(f1,s))
                  endif
               enddo
            endif
         enddo
      enddo

!  DMT 06/2018
!  Generation of a MCTDH thermal state (see "thermal" keyword in RUN-SEC.)
!  dimfran(f) was set in subroutine eb_ran called from eininwf when
!  reading the input file (see INIT_WF-SECTION, keyword "ran")

!-----------------------------------------------------------------------
! set up the pointers zspf1d for the array spf1d. Set spf1ddim.
! compute maximum number of single-particle functions to be generated (maxspf)
!-----------------------------------------------------------------------
      zeig=1
      do s=1,nstate
         do f=1,ndof
            dimfmax=max(dimf(f,s),dimfran(f))
            maxspf=max(maxspf,dimfmax)
            zspf1d(f,s)=zeig
            if (sbasis(f) .eq. 5) then
               zeig=zeig+gdim(f)*gdim(f+1)*dimfmax
            elseif (sbasis(f) .eq. 8) then
               zeig=zeig+gdim(f)*gdim(f+1)*gdim(f+2)*dimfmax
            else
               zeig=zeig+gdim(f)*dimfmax
            endif
         enddo
      enddo
      spf1ddim=zeig-1

!-----------------------------------------------------------------------
! Set up space for hpsi operation if operate option is used
!-----------------------------------------------------------------------
      if (inwfop(1) .gt. 0) then
         do i=1,hamdim
            call operflag(i)
            call zeighpsi(i)
         enddo
         intpsidim = max(intpsidim,dgldim)
      endif

!-----------------------------------------------------------------------
! (A)Diabatic modus.
!-----------------------------------------------------------------------
      addim  = 1
      if (ladiab) then
         if (nmode .eq. 2) then
            addim = max(addim,dim(2,1))
         else if (nmode .eq. 3) then
            addim = max(addim,dim(2,1))
         endif
      endif

!-----------------------------------------------------------------------
! for Selected CI calculation, calculate full configuration space size 
!-----------------------------------------------------------------------
      fmblock=0
      if (citype .ne. 0) then
         fadim=0
         do s=1,nstate
            fblock(s)=1
            do m=1,nmode
               fblock(s)=fblock(s)*dim(m,s)
            enddo
            fadim=fadim+fblock(s)
            fmblock=max(fmblock,fblock(s))
         enddo
      endif

!-----------------------------------------------------------------------
! write length of psi vector to log file
!-----------------------------------------------------------------------
      write(ilog,*)
      write(ilog,'(a)') '***** Length of psi vector *****'
      write(ilog,*)
      if (citype .eq. 1) then
         write(ilog,'(a,f8.2)') 'Static S-MCTDH. Cutoff :',cicut
      endif

      if (lmult) then
         if (citype .ne. 0) then
            do s=1,nstate
               write(ilog,'(a,i2,2(a,i9))') &
                  'Number of MCTDH coefficients for state ',s,': ', &
                  block(s),' / ',fblock(s)
            enddo
            write(ilog,*)
         else
            do s=1,nstate
               write(ilog,'(a,i2,a,i9)') &
                  'Number of MCTDH coefficients for state ',s,': ', &
                  block(s)
            enddo
            write(ilog,*)
         endif
         write(ilog,*)
      endif

      if (citype .gt. 0) then
         write(ilog,'(2(a,i9))') &
            'Overall length of A-vector        :', adim,' / ',fadim
      else if (psitype .eq. 5) then
         mladim  = adim_wftree(tree)
         write(ilog,'(a,i9)') &
            'Overall length of A-vector        :', mladim
         write(ilog,'(a,i9)') &
            'Length of top layer A-vector      :', adim
      else 
         write(ilog,'(a,i9)') &
            'Overall length of A-vector        :', adim
      endif

      do m = 1,nmode
         if(idmode(m).eq.0) then
            write(ilog,'(a,i4,a,i9)') &
            'Length of phi vector for mode ', m, ':', phidim(m)
         else
            write(ilog,'(a,i4,a,i9,a,i2,a)') &
            'Length of phi vector for mode ', m, ':', phidim(m), &
            ' (none, as set identical to mode',idmode(m),')'
         endif 
      enddo
      write(ilog,'(34x,a)') '--------'
      write(ilog,'(a,i9)') &
         'Length of wavefunction vector   :', dgldim

      call wkinwf(dgldimold,mblock,mablock,agmatdimold,mxmxblk)
      if(iorthowf.gt.380) call zeigowf  ! orthogonalize WF (impr.relax.)
      dmatdim=max(dmatdim1,dmatdim)
      d3matdim=max(d3matdim1,d3matdim)

      write(ilog,'(/a,i10,/a,i10,/a,i10)') &
         'workcdim   :', workcdim, &
         'workrdim   :', workrdim, &
         'workidim   :', workidim

      return
!-----------------------------------------------------------------------
! write error message to log file
!-----------------------------------------------------------------------
  99  continue
      ilbl=index(filename,' ')-1
      message='Cannot open file:'//filename(1:ilbl)//'                '&
       //'     Check file exists.'
      call errormsg
      end subroutine zeiginwf0

!#######################################################################
!   zeiginwf1 for density Type I (dentype=1)
!#######################################################################
      subroutine zeiginwf1

      implicit none

      integer(long)       :: s,t,p,m,dgldimold,ilbl,dim2,&
                             chkdvr,chkgrd,chkpsi,chkprp,&
                             mblock,mblock1,ipop,f,zeig,j,irst0

      integer(long), dimension(nmode,nstate,nstate)        :: ddim1
      character(len=c5)                                    :: filename
      logical(kind=4)                                      :: linwf,lerr
      logical(kind=4)                               :: lmult2,leb2,lmulpack2


      routine='Zeiginwf1'

! --- Save System data.
      call savecsys(1)
      call savecpsi(1)
      lmult2=lmult
      leb2=leb
      lmulpack2=lmulpack
      irst0=irst

! --- read restart File for size of old wavefunction ---
! first save dim (overwritten in rstinfo)
      if (lrestart) then
         filename=rname(1:rlaenge)//'/restart'
         open(irst,file=filename,&
                 form='unformatted',status='old',err=999)

         call chkrsttyp
         if (psitype .eq. 1 .or. psitype == 11) then
            chkdvr=6
            chkgrd=6
            chkpsi=1
            chkprp=0
            call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)

            dgldimold=dgldim

            mblock=0
            do s=1,nstate
               do t=1,nstate
                  mblock=max(mblock,dblock(s,t))
               enddo
            enddo

         else
            do s=1,nstate
               do t=1,nstate
                  do m=1,nmode
                     ddim1(m,s,t)=ddim(m,s,t)
                  enddo
               enddo
            enddo
      
            chkdvr=6
            chkgrd=6
            chkpsi=1
            chkprp=0
            call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)

            dgldimold=dgldim

            mblock=0
            do s=1,nstate
               do t=1,nstate
                  mblock1=1
                  do m=1,nmode
                     dim2=max(ddim(m,s,t),ddim1(m,s,t))
                     mblock1=mblock1*dim2
                  enddo
                  mblock=max(mblock,mblock1)
               enddo
            enddo

            do s=1,nstate
               do t=1,nstate
                  do m=1,nmode
                     ddim(m,s,t)=ddim1(m,s,t)
                  enddo
               enddo
            enddo

         endif
         close(irst)

      else
         mblock=0
         dgldimold=0
      endif

!........Reset system data.
      call getcsys(1)
      call getcpsi(1)
      lmult=lmult2
      leb=leb2
      lmulpack=lmulpack2
      irst=irst0
      if (psitype .eq. 5) lrunml = .true.
      call griddat

!-----------------------------------------------------------------------
! calculate dim,vdim and ndim for each mode.
! size of block (number of MCTDH coefficients)
!-----------------------------------------------------------------------
      call psidat

!-----------------------------------------------------------------------
! calculate data for spf1d array, which holds 1d "spfs"
!-----------------------------------------------------------------------
      do s=1,nstate
         do t=1,nstate
            p = (t-1)*nstate + s
            do f=1,ndof
               ipop=isbaspar(4,f,p)
               m=dofspf(f)
               if (m .eq. 0) then
                  ddimf(f,s,t)=0
               else
                  ddimf(f,s,t)=max(ipop,ddim(m,s,t))
!     check that complete discrete basis can be generated
                  if (sbasis(f) .eq. 0) &
                       ddimf(f,s,t)=max(ddim(f,s,t),gdim(f))
!     if k dof in a kleg function, no spfs are to be generated
                  if (sbasis(f) .eq. 6) ddimf(f,s,t)=1
               endif
            enddo
         enddo
      enddo

      zeig=1
      do s=1,nstate
         do t=s,nstate
            do f=1,ndof
               dzspf1d(f,s,t)=zeig
               if (sbasis(f) .eq. 5) then
                  zeig=zeig+gdim(f)*gdim(f+1)*ddimf(f,s,t)
               else
                  zeig=zeig+gdim(f)*ddimf(f,s,t)
               endif
            enddo
         enddo
      enddo
      spf1ddim=zeig-1


!-----------------------------------------------------------------------
! Set up space for hpsi operation if operate option is used
!-----------------------------------------------------------------------
      do j = 1, 15
         if (inwfop(j) .eq. 0) goto 20
         call d1operflag(inwfop(j))
         call d1zeighpsi(inwfop(j))
      enddo
 20   continue

!-----------------------------------------------------------------------
! compute maximum number of single-particle functions to be generated
!-----------------------------------------------------------------------
      do s=1,nstate
         do t=1,nstate
            do f=1,ndof
               maxspf=max(maxspf,ddimf(f,s,t))
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! write length of psi vector to log file
!-----------------------------------------------------------------------
      write(ilog,*)
      write(ilog,'(a)') '***** Length of psi vector *****'
      write(ilog,*)
      if (lmult) then
         do s=1,nstate
            do t=1,nstate
               write(ilog,'(a,i2,a,i2,a,i8)') &
                    'Number of MCTDH coefficients for state (',s,',', &
                    t,'):',dblock(s,t)
            enddo
         enddo
         write(ilog,*)
      endif
      if (psitype .eq. 5) then
         mladim  = adim_wftree(tree)
         write(ilog,'(a,i9)') &
            'Overall length of A-vector        :', mladim
         write(ilog,'(a,i9)') &
            'Length of top layer A-vector      :', adim
      else
         write(ilog,'(a,i9)') &
            'Overall length of A-vector        :', adim
      endif
      do m = 1,nmode
         write(ilog,'(a,i2,a,i9)') &
         'Length of phi vector for mode ', m, ':', phidim(m)
      enddo
      write(ilog,'(34x,a)') '--------'
      write(ilog,'(a,i9)') &
         'Length of wavefunction vector   :', dgldim

      call wkinwf1(dgldimold,mblock)

      return
!-----------------------------------------------------------------------
! write error message to log file
!-----------------------------------------------------------------------
  999 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      end subroutine zeiginwf1

!#######################################################################

      subroutine wkinwf(dgldim1,mblock,mablock,agmatdim1,mxmxblk)

      implicit none

      integer(long)             :: f
      integer(long), intent(in) :: dgldim1,mblock,agmatdim1,mablock,mxmxblk

      workcdim=1
      workrdim=1
      workidim=1
      workldim=1

      do f=1,ndof
         if ((iruninwf .eq. 1) .or. sbasis(f) .eq. 30) then
            call wkgetpsi(dgldim1,mblock,mablock,agmatdim1,mxmxblk)
         endif
      enddo
      
      if (inwfop(1) .gt. 0) then
         call  wkoperate
      else
         call maxmatwk(0)
         workcdim=max(workcdim,maxblock)
      endif

      return
      end subroutine wkinwf

!#######################################################################

      subroutine wkinwf1(dgldim1,mblock)

      implicit none

      integer(long)             :: f
      integer(long), intent(in) :: dgldim1,mblock

!-----------------------------------------------------------------------
! SIZE OF COMPLEX WORK ARRAY
!
! when the mean-fields are held constant the routine "funk3" uses
! the "intpsi" array as complex work array. For that reason "intpsi"
! must be large enough to keep three times the largest "block" (which is
! normally already the case), whereas for "workc" a size of twice the
! largest "block" suffices.
!-----------------------------------------------------------------------
      workcdim=1
! for getpsi (rstpsi)
      do f=1,ndof  
         if ((iruninwf .eq. 1) .or. sbasis(f) .eq. 30) then
            workcdim=max(workcdim,dgldim1+2*mblock)
            workcdim=max(workcdim,dgldim)
         endif
      enddo
! for genrho1
      workcdim=max(workcdim,sdim)
      workcdim=max(workcdim,maxgdim*maxspf)
! for genpleg
      do f=1,ndof
         if (basis(f).eq.9) then            ! Pleg
          workcdim=max(workcdim,gdim(f)*ipbaspar(3,f+1)*dimf(f,1))
         elseif (basis(f).eq.21) then        ! Pleg/R
          workcdim=max(workcdim,ipbaspar(3,f)*ipbaspar(3,f+1)*dimf(f,1))
         endif
      enddo

!-----------------------------------------------------------------------
!   SIZE OF REAL WORK ARRAY
!-----------------------------------------------------------------------
      workrdim = 1
! for genphi(veigen)
      do f=1,ndof
         if (sbasis(f).eq.3) then
            workrdim = max(workrdim,2*gdim(f)**2+gdim(f))
         endif
      enddo
      if (lad) then
         if (ndof.eq.2) then
            workrdim=max(workrdim,maxgdim+gdim(2)+addim**2+addim &
                         +addim**2+3*addim-1)
         elseif (ndof.eq.3 .or. ndof.eq.4) then
            workrdim=max(workrdim,maxgdim+gdim(2)*gdim(3)+addim**2 &
                         +addim+addim**2+3*addim-1)
         endif
      else if (ldia) then
         workrdim=max(workrdim,2*sdim)
      endif
! for getpsi (call to density)
      if (lorthopsi) workrdim=max(workrdim,3*maxspf)

!-----------------------------------------------------------------------
! SIZE OF INTEGER WORK ARRAY
!-----------------------------------------------------------------------
      workidim=1

!-----------------------------------------------------------------------
! SIZE OF LOGICAL WORK ARRAY
!-----------------------------------------------------------------------
      workldim = 1

      if (inwfop(1) .gt. 0) call  wkoperate


      return
      end subroutine wkinwf1

!#######################################################################

      end module zeiginwfmod

