!#######################################################################
!
! ASSIGNHOP
!
! assigns the type of operator
!
! hopz:    operator matrix
!
! hoptype =htmtype(h) : type of matrix, from folowing list
!
! --- 1D types ---
! 0 = zero
! 1 = unit
! 2 = real vector (diagonal matrix)
! 3 = imaginary vector (imaginary part stored as real)
! 4 = complex vector
! 5 = FFT, real vector
! 6 = FFT, imaginary vector
! 7 = FFT, complex vector
! 8 = real matrix
! 9 = imaginary matrix (imaginary part stored as real)
! 10 = complex matrix
! 11 = (obsolete)
! 12 = real, side-diagonal matrix (stored as vector)
! 13 = imaginary, side-diagonal matrix (stored as vector)
! 14 = complex, side-diagonal matrix (stored as vector)
!
! --- 2D combined-mode types ---
! 15 = real 3D tensor (for dth1_kleg, dth2_kleg)
! 16 = real 3D tensor (for jpm)
! 17 = real 3D tensor (for j^2kleg)
! 18 = real 3D tensor (for j_p or sJm or sJmk)
! 19 = real 3D tensor (for j_m or sJp or sJpk)
! 20 = real 3D tensor (for jp^2)
! 21 = real 3D tensor (for jm^2)
! 22 = real 3D tensor (for jpjm)
! 23 = real 3D tensor (for jmjp)
!
! 25 = readmap
!
! --- 3D combined-mode types ---
! 40 = real 4D tensor (for j^2wig)
! 41 = real 4D tensor (for j_pwig)
! 42 = real 4D tensor (for j_mwig)
! 43 = real 4D tensor (for jp2wig)
! 44 = real 4D tensor (for jm2wig)
! 45 = real 4D tensor (for jpmwig)
! 46 = real 4D tensor (for j_pswig)
! 47 = real 4D tensor (for j_mswig)
! 48 = real 4D tensor (for jp2swig)
! 49 = real 4D tensor (for jm2swig)
! 50 = real 4D tensor (for jpmswig)
! 51 = real 4D tensor (for jpjmwig)
! 52 = real 4D tensor (for jmjpwig)
! 53 = real 4D tensor (for jpjpswig)
! 54 = real 4D tensor (for jmjpswig)
! 55 = real 4D tensor (for jpjzwig)
! 56 = real 4D tensor (for jmjzwig)
! 57 = real 4D tensor (for jzjpwig)
! 58 = real 4D tensor (for jzjmwig)
! 59 = real 4D tensor (for jpjzswig)
! 60 = real 4D tensor (for jmjzswig)
! 61 = real 4D tensor (for jzjpswig)
! 62 = real 4D tensor (for jzjmswig)
! 63-69 (unassigned Wigner operators)
!
! --- 4D types for combined KLegs ---
! 30 = (cjpm)
!
! --- analytic functions used for GWPs and on-the-fly ---
! 101 = mode real diagonal analytic 
! 102 = gwp and classical momentum 
! 103 = analytical unit operator
! 104 = LHA of a multi-dimensional potential 
! 105 = LHA of a mode real potential
! 106 = GWP flux operator
! 107 = GWP step operator
! 108 = GWP CAP evaluated using saddle-point approx
! 109 = GWP operators evaluated using saddle-point approx
!
! --- local control function ---
! 151 = local control function
!
! --- function of expectation value of other operator---
! 161 = function of expectation
!
! contains: assignzhtm,assigndhtm,assdzhtm,asszdiag,assignhlab, &
!           asshtmlab,asshtmsym
!
!#######################################################################

      module asshop

use decimal, only: long, dop
use global
use logdat
use op1lib,  only: zerovxi, zerovxd
use operdef
use runopermod
use operutil, only: findnath
use maxv
use dvrdatmod
use griddatmod
use dirdyn
use channels

use callgrdmod, only: defgrd
use callanldmod, only: defanld
use callanlzmod, only: defanlz
use callmuldmod, only: defmuld

use rdopermod, only: iskleg

      implicit none
      private
      public :: assignzhtm,assigndhtm,assdzhtm,asszdiag,&
                assignhlab,asshtmsym,asshtmlab
      contains

!#######################################################################
!
! ASSIGNZHTM
!
! assigns operator type for a complex matrix operator
!
!#######################################################################

      subroutine assignzhtm(hopz,hoptype,ishift,gdim)

      implicit none

      integer(long)              :: g,g1,k
      integer(long), intent(in)  :: gdim
      integer(long), intent(out) :: hoptype,ishift
      real(dop)                  :: maxre, maxim, dmaxre, dmaxim, trshld,&
                                    smaxre, smaxim, sdmaxre, sdmaxim

      complex(dop), dimension(gdim,gdim), intent(in) :: hopz
      logical(kind=4)                                :: di,im,re,sd

      im=.false.
      re=.false.
      di=.false.
      sd=.false.

      hoptype=-1

      maxre=0.0_dop
      maxim=0.0_dop
      dmaxre=0.0_dop
      dmaxim=0.0_dop
!.....Maximum of off-diagonal terms
      do g=2,gdim
         do g1=1,g-1
            maxre = max(maxre,abs(dble( hopz(g1,g))), &
                              abs(dble( hopz(g,g1))) ) 
            maxim = max(maxim,abs(dimag(hopz(g1,g))), &
                              abs(dimag(hopz(g,g1))) )
         enddo
      enddo
!.....Maximum of diagonal terms
      do g=1,gdim
            dmaxre = max(dmaxre,abs(dble( hopz(g,g))))
            dmaxim = max(dmaxim,abs(dimag(hopz(g,g))))
      enddo
!.....Check for zero operator
      if( (maxre+maxim+dmaxre+dmaxim) .lt. 1.0e-12_dop ) then
         hoptype=0
         nremoved=nremoved+1
         goto 40
      endif

      trshld = (maxre+maxim+dmaxre+dmaxim)*1.0e-10_dop
      if( (maxim+dmaxim) .lt. trshld ) re=.true.
      if( (maxre+dmaxre) .lt. trshld ) im=.true.
      if( (maxre+maxim)  .lt. trshld ) di=.true.    

!.....Check for side-diagonal matrix
      if (.not.di) then
!        loop over all lower side diagonals
         k=1
 10      continue
!        get maximum of all elements not on this side diagonal
         smaxre =0.0_dop
         smaxim =0.0_dop
         sdmaxre=0.0_dop
         sdmaxim=0.0_dop
         do g=1,gdim
            do g1=1,gdim
               if (g-g1.ne.k) then
                  smaxre = max(smaxre,abs(dble(hopz(g,g1))))
                  smaxim = max(smaxim,abs(dimag(hopz(g,g1))))
                  if ((smaxre+smaxim).gt.trshld) goto 15
               else
                  sdmaxre = max(sdmaxre,abs(dble(hopz(g,g1))))
                  sdmaxim = max(sdmaxim,abs(dimag(hopz(g,g1))))
               endif
            enddo
         enddo
!        if we got here, the other elements are all small
         sd = .true.
         if ((smaxim+sdmaxim).lt.trshld) re=.true.
         if ((smaxre+sdmaxre).lt.trshld) im=.true.
         goto 20
!        else continue with next side diagonal
 15      if (k.gt.0) then
            k=-k
            goto 10
         else
            k=-k+1
            if (k.lt.gdim) goto 10
         endif
      endif
      k=0
 20   continue
      ishift=k

      if (di) then
         if (re) then
            hoptype=2
         else if (im) then
            hoptype=3
         else 
            hoptype=4
         endif
      else if (sd) then
         if (re) then
            hoptype=12
         else if (im) then
            hoptype=13
         else
            hoptype=14
         endif
      else
         if (re) then
            hoptype=8
         else if (im) then
            hoptype=9
         else 
            hoptype=10
         endif
      endif

      if (hoptype .eq. 2) then
         do g=1,gdim
            if (abs(hopz(g,g)-1.0_dop) .gt. tiny) goto 40
         enddo
         hoptype=1
      endif

  40  return
      end subroutine assignzhtm

!#######################################################################
!
! ASSIGNDHTM
!
! assigns real diagonal hamiltonian terms
!
!#######################################################################
 
      subroutine assigndhtm(h,hopd,subdim1,ctff)

      implicit none

      integer(long)                             :: g,hoptype,n,imlop
      integer(long), intent(in)                 :: subdim1,h
      real(dop), intent(in)                     :: ctff
      real(dop), dimension(subdim1), intent(in) ::  hopd
      logical(kind=4)                           :: unit,zero
 
      unit=.true.
      zero=.true.
      hoptype=-1
 
!-----------------------------------------------------------------------
! type mode operators
!-----------------------------------------------------------------------
      do g=1,subdim1
         if (abs(hopd(g)-1.0_dop) .gt. tiny) unit=.false.
         if (abs(hopd(g)) .gt. ctff) zero=.false. 
      enddo
 
      if (zero) then
         hoptype=0
         nremoved=nremoved+1
         call findnath(h,n,imlop)
         if (n.ne.0) ncutted(n)=ncutted(n)+1
      else if (unit) then
         hoptype=1
      else
         hoptype=2
      endif
 
      if (hoptype .ge. 0) htmtype(h)=hoptype
 
      return
      end subroutine assigndhtm

!#######################################################################
!
! ASSDZHTM
!
! assigns complex diagonal hamiltonian terms.
! If real, data is copied to hopd.
!
!#######################################################################
 
      subroutine assdzhtm(h,hopdz,hopd,subdim1,lz)

      implicit none

      integer(long)                                :: g,hoptype
      integer(long), intent(in)                    :: subdim1,h
      real(dop)                                    :: dmaxre,dmaxim,trshld
      real(dop), dimension(subdim1), intent(out)   :: hopd
      complex(dop), dimension(subdim1), intent(in) :: hopdz
      logical(kind=4)                              :: unit,re,im
      logical(kind=4), intent(out)                 :: lz
 
      re=.false.
      im=.false.
      unit=.false.
      hoptype=-1
      if(htmtype(h).gt.4 .or. .not.lz ) return

!-----------------------------------------------------------------------
! type mode operators
!-----------------------------------------------------------------------
!.....Maximum of diagonal terms
      dmaxre=0.0_dop
      dmaxim=0.0_dop
      do g=1,subdim1
            dmaxre = max(dmaxre,abs(dble( hopdz(g))))
            dmaxim = max(dmaxim,abs(dimag(hopdz(g))))
      enddo
!.....Check for zero operator
      if( (dmaxre+dmaxim) .lt. 1.0e-12_dop ) then
         hoptype=0
         lz=.false.
         nremoved=nremoved+1
         goto 40
      endif

      trshld = (dmaxre+dmaxim)*1.0e-10_dop
      if( dmaxim .lt. trshld ) re=.true.
      if( dmaxre .lt. trshld ) im=.true.

      if(re) then
         do g=1,subdim1
            if (abs(hopdz(g)-1.0_dop) .gt. tiny) goto 50
         enddo
         unit=.true.
      endif

  50  if( re ) then
         hoptype=2
         lz=.false.
         do g=1,subdim1
            hopd(g) = dble(hopdz(g))
         enddo
         if(unit) hoptype=1 
      elseif( im ) then
         hoptype=3
      else
         hoptype=4
      endif 
 
  40  if (hoptype .ge. 0) htmtype(h)=hoptype
      
      return
      end subroutine assdzhtm

!#######################################################################

      subroutine asszdiag(hopz,hopd,sdm,lim,lz,lzero)

      implicit none
      integer(long)                            :: g
      integer(long), intent(in)                :: sdm
      complex(dop), dimension(sdm), intent(in) :: hopz
      real(dop)                                :: maxre,maxim,trshld
      real(dop), dimension(sdm), intent(out)   :: hopd
      logical(kind=4), intent(out)             :: lim,lz,lzero

      maxre=0.0_dop
      maxim=0.0_dop
      lzero=.false.
      do g=1,sdm
         maxre=max(maxre,abs(dble(hopz(g))))
         maxim=max(maxim,abs(dimag(hopz(g))))
      enddo
      if(maxre+maxim .lt. 1.d-12) then ! zero operator (Dieter 8.1.20)
         lzero=.true.
         return
      endif
      trshld=(maxre+maxim)*1.0e-12_dop
      if (maxim.lt.trshld) then
         lim=.false.
         lz =.false.
         do g=1,sdm
            hopd(g)=dble(hopz(g))
         enddo
      elseif (maxre.lt.trshld) then
         lim=.true.
         lz =.false.
         do g=1,sdm
            hopd(g)=dimag(hopz(g))
         enddo
      else
         lim=.false.
         lz =.true.
      endif

      return
      end subroutine asszdiag

!#######################################################################
! New version (Benjamin, December 2008)
!#######################################################################

      subroutine assignhlab(loperr)

      implicit none

      integer(long)                  :: hl,n,h,ierr
      logical(kind=4), intent(inout) :: loperr

      routine='Assignhlab'

      write (ilog,'(/,a,/)') 'Assigning Operators From Library'

!-----------------------------------------------------------------------
! special labels may not have an associated file
!-----------------------------------------------------------------------
      do hl=1,min(hopdim,maxhop)
         if (hoplab(hl)(1:7) .eq. 'srffile') hopfile(hl)=-999
         if (hoplab(hl)(1:6) .eq. 'natpot')  hopfile(hl)=-999
         if (hoplab(hl)(1:6) .eq. 'potfit')  hopfile(hl)=-999
         if (hoplab(hl)(1:4) .eq. 'mlpf')    hopfile(hl)=-999
         if (hoplab(hl)(1:7) .eq. 'cpdfile') hopfile(hl)=-999
         if (hoplab(hl)(1:8) .eq. 'hcpdfile') hopfile(hl)=-999
      enddo

!-----------------------------------------------------------------------
! now assign function and file numbers 
!-----------------------------------------------------------------------
      do hl=1,min(hopdim,maxhop)
         if (hopfile(hl) .ne. 0) go to 99

         hopilab(hl)=0
         call zerovxi(hopipar(1,hl),maxhpar)
         call zerovxd(hoprpar(1,hl),maxhpar)

!-----------------------------------------------------------------------
! search grid (non-local and read) operators
!-----------------------------------------------------------------------
         call defgrd(hopilab(hl),hoplab(hl),hopfile(hl), &
              hopipar(1,hl),hoprpar(1,hl),maxhpar,rdlab,rdtype,nrdfunc, &
              hoppar)
         if (hopfile(hl) .ne. 0) go to 99

!-----------------------------------------------------------------------
! search analytic 1D functions
!-----------------------------------------------------------------------
         call defanld(hopilab(hl),hoplab(hl),hopfile(hl),hopipar(1,hl), &
              hoprpar(1,hl),maxhpar,hoppar,apar,rpar,npar,ierr, &
              zpesopts,pesopts,hamlab,hamdim)
         if (hopfile(hl) .ne. 0) go to 99

!-----------------------------------------------------------------------
! search complex 1D functions
!-----------------------------------------------------------------------
         call defanlz(hopilab(hl),hoplab(hl),hopfile(hl), &
              hopipar(1,hl),hoprpar(1,hl),maxhpar,hoppar)
         if (hopfile(hl) .ne. 0) go to 99

!-----------------------------------------------------------------------
! search multi-dimensional surfaces
! store grid length in hopipar(1)
!-----------------------------------------------------------------------
         call defmuld(hopilab(hl),hoplab(hl),hopfile(hl),pesopts, &
              zpesopts,hopipar(1,hl),hoprpar(1,hl),maxhpar,pesmin, &
              pesmax,lcutpes,ierr,ndof,hoppar)

         if (hopfile(hl) .ne. 0) go to 99

         if (ierr .ne. 0) then
            routine='Assignhlab'
            call warnmsg
         endif
   99    continue
      enddo

!-----------------------------------------------------------------------
! check that all labels used have been defined
!-----------------------------------------------------------------------
      do h=1,min(htmdim,maxhtm)
         do n=1,min(maxfac,htmfac(h))
            hl=hterm(n,h)
            if (hl .gt. 0 .and. hopfile(hl) .eq. 0) then
               routine='Assignhlab '
               message='Operator label not found : '//hoplab(hl)
               call warnmsg
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! If multi-dimensional term not yet assigned, assign to full grid
! This is necessary if a construct like |1&2&3 is not used.
!-----------------------------------------------------------------------
      do h=1,min(htmdim,maxhtm)
         do n=1,min(maxfac,htmfac(h))
            hl=hterm(n,h)
            if (hl .ne. 0) then
               if(hopfile(hl).gt.200 .and. hopfile(hl).lt.300) then
                  if(htmmuld(h).eq.0 .and. htmmode(h).eq.0 .and. .not. lddmctdh) then
                     if(hoplab(hl)(1:6) .eq. 'read1d') then
                       continue
                     elseif(nmode .eq. 1) then
                        htmdof(h)=0
                        htmmode(h)=1
                     else
                        htmdof(h)=0
                        htmmuld(h)=1
                     endif
                  endif
               endif
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      loperr=lwarn.or.loperr
      lwarn=.false.

      return
      end subroutine assignhlab

!#######################################################################
!
! Subroutine asshtmlab
!
! assigns the operator type if basis / label dependent 
!   e.g. FFT or KLEG operators
!---------------------------------------------------------------------

      subroutine asshtmlab(h,f1)

      implicit none

      integer(long)             :: n,hl,hoptype,m,i,kdim,ilbl,icount,mmdim
      integer(long), intent(in) :: h,f1
      logical(kind=4)           :: lmode
!      logical, external         :: iskleg ! external function

      hoptype=-1
      lmode=.false.
      m=dofspf(f1)

!-----------------------------------------------------------------------
! flag natural potentials and cpd as  type 2
! flag hcpd as type 8
! flag MLPF leaf potentials as type 2
! flag reflection operators Rf and Rfm as type 31 and 32
!-----------------------------------------------------------------------
      hl=hterm(1,h)
      if (htmfac(h) .eq. 1) then
         if (hoplab(hl)(1:4) .eq. 'npot') then
             hoptype=2
             go to 100
         elseif (hoplab(hl)(1:4) .eq. 'mlnp') then
             hoptype=2
             go to 100
         elseif (hoplab(hl)(1:5) .eq. 'cpdtm') then
             hoptype=2
             go to 100
          elseif (hoplab(hl)(1:6) .eq. 'hcpdtm') then
             hoptype=8
             go to 100
          elseif (hoplab(hl)(1:7) .eq. 'readmap') then
             hoptype=25
             go to 100
         elseif (hoplab(hl) .eq. 'Rf') then
             hoptype=31
             go to 100
         elseif (hoplab(hl) .eq. 'Rfm') then
             hoptype=32
             go to 100
         endif
      endif

!-----------------------------------------------------------------------
! electronic continuum basis 
!-----------------------------------------------------------------------
      hl=hterm(1,h)
      ilbl=index(hoplab(hl),'&')
      if (hoplab(hl)(1:2) .eq. 'CE' .and. ilbl .gt. 0 ) then
         hoptype=2
         go to 100
      
      else if (hoplab(hl)(1:2) .eq. 'CS' .and. ilbl .gt. 0 ) then
         hoptype=8
         go to 100
!-Alice
!     else if (hoplab(hl)(1:7) .eq. 'CS_bath' .and. ilbl .gt. 0 ) then
!        hoptype=8
!        go to 100
      else if (hoplab(hl)(1:2) .eq. 'CZ' .and. ilbl .gt. 0 ) then
         hoptype=8
         go to 100
      endif

!-----------------------------------------------------------------------
! local control operators
!-----------------------------------------------------------------------
      if (hoplab(hl)(1:3) .eq. 'lct') then
         hoptype = 151
         go to 100
      endif

!-----------------------------------------------------------------------
! local control operators
!-----------------------------------------------------------------------
      if (hoplab(hl)(1:6) .eq. 'expect') then
         hoptype = 161
         go to 100
      endif

!-----------------------------------------------------------------------
! now look at basis dependent types
!-----------------------------------------------------------------------
      if (basis(f1) .eq. 4) then     ! FFT
         icount=0
         do n=1,htmfac(h)
            hl=hterm(n,h)
            if (hl.ne.0) then
               if (hoplab(hl) .eq. 'KEfft') then
                  hoptype=5
               else if (hoplab(hl)(1:4) .eq. 'pfft') then
                  hoptype=5
               else if (hoplab(hl)(1:2) .eq. 'dq') then
                  read(hoplab(hl)(7:7),'(i1)') i
                  if( mod(i,2) .eq. 0 ) then
                     hoptype=5
                  else
                     hoptype=6
                  endif
               else if (hoplab(hl) .eq. 'I') then
                     icount=icount+1
               endif
            endif
         enddo
         if(mod(icount,2).eq.1) then
            if(hoptype.eq.5) hoptype=6
            if(hoptype.eq.6) hoptype=5
         endif

      else if (basis(f1) .eq. 6) then    ! sphFBR
         do n=1,htmfac(h)
            hl=hterm(n,h)
            if (hl.ne.0) then
               if (hoplab(hl) .eq. 'j^2sph') then
                  hoptype=2
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jzsph') then
                  hoptype=2
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jzzsph') then
                  hoptype=2
                  lmode=.true.
               endif
            endif
         enddo

      ! NEWKLEG
      else if (htmform(h).eq.0 .and. iskleg(f1,basis)) then ! KLeg/PLeg
         do n=1,htmfac(h)
            hl=hterm(n,h)
            if (hl.ne.0) then
               if (basis(f1).eq.7 .or. basis(f1).eq.20) then
                  kdim=gdim(f1+1)
               else
                  kdim=ipbaspar(3,f1+1)
               endif
               if (hoplab(hl) .eq. 'dth1_kleg') then
                  hoptype=15
                  lmode=.true.
               else if (hoplab(hl) .eq. 'dth2_kleg') then
                  hoptype=15
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jpm' .and. kdim.gt.1) then
                  hoptype=16
                  if (lrealwp) hoptype=2
                  lmode=.true.
               else if (hoplab(hl) .eq. 'j^2kleg') then
                  hoptype=17
                  if (lrealwp) hoptype=2
                  lmode=.true.
               else if (hoplab(hl) .eq. 'j_p' .and. kdim.gt.1) then
                  hoptype=18
                  if (lrealwp) hoptype=2
                  lmode=.true.
               else if (hoplab(hl) .eq. 'j_m' .and. kdim.gt.1) then
                  hoptype=19
                  if (lrealwp) hoptype=2
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jp^2' .and. kdim.gt.1) then
                  hoptype=20
                  if (lrealwp) hoptype=2
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jm^2' .and. kdim.gt.1) then
                  hoptype=21
                  if (lrealwp) hoptype=2
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jpjm' .and. kdim.gt.1) then
                  hoptype=22
                  if (lrealwp) hoptype=2
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jmjp' .and. kdim.gt.1) then
                  hoptype=23
                  if (lrealwp) hoptype=2
                  lmode=.true.
               else if (hoplab(hl) .eq. 'sJp' .and. kdim.gt.1) then
                  hoptype=19
                  lmode=.true.
               else if (hoplab(hl) .eq. 'sJm' .and. kdim.gt.1) then
                  hoptype=18
                  lmode=.true.
               else if (hoplab(hl) .eq. 'sJpk' .and. kdim.gt.1) then
                  hoptype=19
                  lmode=.true.
               else if (hoplab(hl) .eq. 'sJmk' .and. kdim.gt.1) then
                  hoptype=18
                  lmode=.true.
               else if (hoplab(hl) .eq. 'cjpm') then
!                 DOFs must be KLeg/K/KLeg/K                  
                  if (.not.iskleg(f1+2,basis)) then
                      routine='asshtmlab'
                      message='cjpm operator only works for '// &
                         'KLeg/K/KLeg/K combinations.'
                      call errormsg
                  endif
                  hoptype=30
                  lmode=.true.
               endif
            endif
         enddo

      else if (htmform(h).eq.0 .and. basis(f1).eq.33) then ! Wigner
         do n=1,htmfac(h)
            hl=hterm(n,h)
            if (hl.ne.0) then
               kdim=ipbaspar(4,f1)
               mmdim=ipbaspar(8,f1)
               if (hoplab(hl) .eq. 'j^2wig') then
                  hoptype=40
                  lmode=.true.
               else if (hoplab(hl) .eq. 'j_pwig' .and. kdim.gt.1) then
                  hoptype=41
                  lmode=.true.
               else if (hoplab(hl) .eq. 'j_mwig' .and. kdim.gt.1) then
                  hoptype=42
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jp2wig' .and. kdim.gt.1) then
                  hoptype=43
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jm2wig' .and. kdim.gt.1) then
                  hoptype=44
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jpmwig' .and. kdim.gt.1) then
                  hoptype=45
                  lmode=.true.
               else if (hoplab(hl) .eq. 'j_pswig' .and. mmdim.gt.1) then
                  hoptype=46
                  lmode=.true.
               else if (hoplab(hl) .eq. 'j_mswig' .and. mmdim.gt.1) then
                  hoptype=47
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jp2swig' .and. mmdim.gt.1) then
                  hoptype=48
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jm2swig' .and. mmdim.gt.1) then
                  hoptype=49
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jpmswig' .and. mmdim.gt.1) then
                  hoptype=50
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jpjmwig' .and. kdim.gt.1) then
                  hoptype=51
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jmjpwig' .and. kdim.gt.1) then
                  hoptype=52
                  lmode=.true.
               else if (hoplab(hl).eq.'jpjmswig' .and. mmdim.gt.1) then
                  hoptype=53
                  lmode=.true.
               else if (hoplab(hl).eq.'jmjpswig' .and. mmdim.gt.1) then
                  hoptype=54
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jpjzwig' .and. kdim.gt.1) then
                  hoptype=55
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jmjzwig' .and. kdim.gt.1) then
                  hoptype=56
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jzjpwig' .and. kdim.gt.1) then
                  hoptype=57
                  lmode=.true.
               else if (hoplab(hl) .eq. 'jzjmwig' .and. kdim.gt.1) then
                  hoptype=58
                  lmode=.true.
               else if (hoplab(hl).eq.'jpjzswig' .and. mmdim.gt.1) then
                  hoptype=59
                  lmode=.true.
               else if (hoplab(hl).eq.'jmjzswig' .and. mmdim.gt.1) then
                  hoptype=60
                  lmode=.true.
               else if (hoplab(hl).eq.'jzjpswig' .and. mmdim.gt.1) then
                  hoptype=61
                  lmode=.true.
               else if (hoplab(hl).eq.'jzjmswig' .and. mmdim.gt.1) then
                  hoptype=62
                  lmode=.true.
               endif
            endif
         enddo

      else if (basis(f1) .eq. 25) then
         if (htmfac(h) .eq. 1) then
            hl=hterm(1,h)
            if (hoplab(hl)(1:2) .eq. '1 ') hoptype=103
         endif

         do n=1,htmfac(h)
            hl=hterm(n,h)
            if (hl.ne.0) then
               if (hoplab(hl)(1:5) .eq. 'dqgwp') then
                  hoptype=102
               else if (hoplab(hl)(1:5) .eq. 'KEgwp') then
                  hoptype=102
               else if (hoplab(hl)(1:4) .eq. 'flux') then
                  hoptype=106
               else if (hoplab(hl)(1:4) .eq. 'step' &
                        .or. hoplab(hl)(1:5) .eq. 'rstep' &
                        .or. hoplab(hl)(1:3) .eq. 'box') then
                  hoptype=107
! CAPS
               else if (hopfile(hl) .eq. 101 .and. hopilab(hl) .eq. 5) then
                  hoptype=108
               else if (hopfile(hl) .eq. 101 .and. hopilab(hl) .eq. 4) then
                  hoptype=108
! special (non-potential) qchem operators
               else if (hopfile(hl) .eq. 207) then
                  hoptype=109
               endif
            endif
         enddo

      else if (basis(f1) .eq. 27) then
         do n=1,htmfac(h)
            hl=hterm(n,h)
            if (hl.ne.0) then
               if (hoplab(hl)(1:5) .eq. 'KE_cl') then
                  hoptype=102
               else if (hoplab(hl)(1:5) .eq. 'p2_cl') then
                  hoptype=102
               else if (hoplab(hl)(1:4) .eq. 'p_cl') then
                  hoptype=102
               else if (hoplab(hl)(1:7) .eq. 'flux_cl') then
                  hoptype=102
               endif
            endif
         enddo

      endif

  100 continue
      if (hoptype .ne. -1) htmtype(h)=hoptype
      if (lmode) then
         htmdof(h)=0
!xy      htmmuld(h)=0
         htmmode(h)=m
      endif

      return
      end subroutine asshtmlab

!#######################################################################
!
! ASSHTMSYM
!
! assigns symmetry type for a complex matrix operator
!
!           1   hermitian
! symtype = 0   general
!          -1   anti-hermitian
!#######################################################################

      subroutine asshtmsym(hopz,symtype,gdim,zero)

      implicit none

      integer(long)                                  :: g,g1,n1,n2,n3
      integer(long), intent(in)                      :: gdim
      integer(long), intent(out)                     :: symtype
      complex(dop)                                   :: x
      complex(dop), dimension(gdim,gdim), intent(in) :: hopz
      logical(kind=4)                                :: diim,dire,anti,herm
      logical(kind=4), intent(out)                   :: zero

      symtype=0

      dire=.false.
      diim=.false.
      anti=.false.
      herm=.false.
      zero=.false.

!-----------------------------------------------------------------------
! see if matrix has real and/or imaginary diagonal.
!-----------------------------------------------------------------------
      do g=1,gdim
         if (abs(dimag(hopz(g,g))) .gt. tiny) diim=.true.
         if (abs(dble(hopz(g,g))) .gt. tiny) dire=.true.
      enddo

!-----------------------------------------------------------------------
! check whether matrix is hermitian / anti-hermitian
!-----------------------------------------------------------------------
      n1=0
      n2=0
      do g=1,gdim
         do g1=g+1,gdim
            x=dconjg(hopz(g1,g))
            if (abs(hopz(g,g1)+x) .lt. tiny) n1=n1+1
            if (abs(hopz(g,g1)-x) .lt. tiny) n2=n2+1
         enddo
      enddo
      n3=gdim*gdim-gdim
      n3=n3/2
      if (n1.eq.n3) anti=.true.
      if (n2.eq.n3) herm=.true.

!-----------------------------------------------------------------------
! a hermitian matrix must have a real diagonal
! an anti- hermitian matrix must have an imaginary diagonal
!-----------------------------------------------------------------------
      if (diim) herm=.false.
      if (dire) anti=.false.

!-----------------------------------------------------------------------
! select symmetry type
!-----------------------------------------------------------------------
      if (anti .and. herm) then  ! Operator is zero
         zero=.true.
      else if (herm) then
         symtype=1
      else if (anti) then
         symtype=-1
      endif

      return
      end subroutine asshtmsym

!#######################################################################

      end module asshop

