!             MCTDH subroutine EINDIRDYN.F
!
! this routine reads kewords for direct dynamics. Previous to 90.45
! this was in integrator-section and in .op file
!
! contains: eindirdyn,chkdirdyn,cvmcgop,eingwptrj
!
!***********************************************************************
      module eindirdynmod

use decimal, only: long,dop
use global
use keyunitsmod
use logdat
use maxv
use griddatmod
use operdef
use datenmod
use dirdyn
use gwptrj
use channels
use paths
use dvrdatmod
use griddatmod
use operdef
use runopermod
use dd_db, only: preparedb
use gapmod
use funcgapmod, only: get_nconfig

use heinutil, only: muldnum
use heingabemod, only:genopdof
use asshop, only: assignhlab
use matchlabmod, only: matchlab
use heinsort, only: sorteb
use operutil, only: mvkend
use openmpmod, only: lompthread
use gapmod, only: ddmorder,lgapderiv,lddmopt,ddmoptord,lddgbtendec

use symoplib

      implicit none
      private
      public :: ddoper,eindirdyn,chkdirdyn,cvmcgop,eingwptrj,&
                chkgwptrj,cddmctop,ddmctoper
      contains

!***********************************************************************
! some options are required by others so cannot always be controlled
! explicitly
! ldbsave1, lupdhes1  : set when requred by selected options
! ldbsave2, lupdhes2  : turned on/off explicitely by keyword
!
! lstmap. If true statemap keyword used.
! Warnmsg only called when pass=2 to prevent repeated warnings in log 
! file.
!***********************************************************************

      subroutine eindirdyn(pass,linperr,iz)

      implicit none

      integer(long), intent(in)      :: pass,iz
      integer(long)                  :: ic,ierr,i,iz0,j,n,k,maxmult,tmpi,&
                                        ilbl,s,s1,sp
      integer(long)                  :: nddspst
      logical(kind=4), intent(inout) :: linperr
      logical(kind=4)                :: ldbsave1=.false.,ldbsave2=.true.
      logical(kind=4)                :: lupdhes1=.false.,lupdhes2=.true.
      logical(kind=4)                :: lorbread00=.false.
      real(dop)                      :: tmpa

!-----------------------------------------------------------------------
! set initial values
! Note: In order that the variable 'name' is specified, 'einfile' must 
! have been called before 'einoper'. 
!-----------------------------------------------------------------------
      routine  = 'read DIRDYN-SECTION  (eindirdyn)'
      iz0      = 0

      if (.not. (ldd.or.lddmctdh)) return

!-----------------------------------------------------------------------
! zero multiplicity data
!-----------------------------------------------------------------------
      nddspst = 0 
      nddmult(:) = 0 

!-----------------------------------------------------------------------
! Assume RW  DB will be used unless otherwise specified
! DB left as binary by default
!-----------------------------------------------------------------------
      lddwrdb = .true.

!----------------------------------------------------------------------- 
! no. of off-diagonal states = no. of NACT elements
!----------------------------------------------------------------------- 
      nactdim = nddstate*(nddstate-1)/2

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (keyword(i) .eq. 'end-dirdyn-section' &
           .or. keyword(i) .eq. 'end-dirdyn-gb-section') then
         goto 20

      else if (keyword(i) .eq. 'qcprogram') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            qcprog = keyword(i)(1:c2)
         else
            message = 'No argument specified with qcprogram keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'qcbasis') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            qcbasis = keyword(i)(1:c2)
         else
            message = 'No argument specified with qcbasis keyword'
            if (pass .eq. 2) call warnmsg
         endif
      else if (keyword(i) .eq. 'qcscratch') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            qcscr = keyword(i)(1:c2)
         else
            message = 'No argument specified with qcscratch keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'subcmd') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            ddextcmd = keyorig(i)(1:c3)
         else
            message = 'No argument specified with subcmd keyword'
            if (pass .eq. 2) call warnmsg
         endif
         if (keyword(i+1) .eq. ',' ) then
            i = i+2
            read(keyword(i),*,iostat=ierr) qcsubform
            if (ierr .eq. 1) then
               message = 'Error reading no. of arguments for SUBCMD'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         if (keyword(i+1) .eq. ',') then
            i = i+2
            read(keyword(i),*,iostat=ierr) qcnproc
            if (ierr .eq. 1) then
               message = 'Error reading no of processors for SUBCMD'
               call wrline(zeroi,iz,iz0)
            endif
         endif

      else if (keyword(i) .eq. 'simulate') then
         lqcsimulate = .true.

      else if (keyword(i) .eq. 'simulate0') then
! simulation of QC as a single point so make it a test run.
         lqcsimulate = .true.
         test = .true.

      else if (keyword(i) .eq. 'qcsaveall') then
         lqcsaveall = .true.

      else if (keyword(i) .eq. 'qcsaveout') then
         lqcsaveout = .true.

      else if (keyword(i) .eq. 'method') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            qcmethod = keyword(i)(1:c2)
            if (index(qcmethod,'cas') .ne. 0) lmocodb=.true.
         else
            message = 'No argument specified with method keyword'
            if (pass .eq. 2) call warnmsg
         endif

         if(keyword(i+1) .eq. ',')then
            i=i+2
            solmethod = keyword(i)(1:c2)
            lsolvent = .true.
         endif
 
      else if (keyword(i) .eq. 'ener0') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) dde0
            if (ierr .ne. 0) then
               message = 'Error reading ener0 keyword'
               if (pass .eq. 2) call warnmsg
            endif
            if (keyword(i+1) .eq. ',' ) then
               i = i+2
               call keyeunits(dde0,keyword(i),ierr)
               if (ierr .eq. 1) then
                  message = 'no energy unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif

         else
            message = 'No argument specified with ener0 keyword'
            if (pass .eq. 2) call warnmsg
         endif
 
      else if (keyword(i) .eq. 'nroot') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) nroot
            if (ierr .ne. 0) then
               message = 'Error reading nroot keyword'
               if (pass .eq. 2) call warnmsg
            endif

         else
            message = 'No argument specified with nroot keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'statemap') then
         routine = 'Eindirdyn (eindirdynmod.f90)'
         message = &
              'Statemap and multmap keywords deprecated. Use state_mult.'
         call errormsg

      else if (keyword(i) .eq. 'multmap') then
         routine = 'Eindirdyn (eindirdynmod.f90)' 
         message = &
              'Statemap and multmap keywords deprecated. Use state_mult.'
         call errormsg

      else if (keyword(i) .eq. 'state_mult') then
         if (keyword(i+1) .ne. '=') then
            message = 'No argument specified with qcmult keyword'
            if (pass .eq. 2) call warnmsg
         endif
         sp = 0
         do 
            i=i+2
            sp = sp+1
            read(keyword(i),*) nddmult(sp) 
            nddspst = nddspst+1
            if (keyword(i+1) .ne. ',') exit
         enddo

      else if (keyword(i) .eq. 'energy') then    
         message = &
            'Energy keyword for MOLCAS control dprecated.' &
             //' Use e.g. qcmethod = CAS/PT2' 
         if (pass .eq. 2) call warnmsg 
         i=i+2  

      else if (keyword(i) .eq. 'fchk') then
         lfchk = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            fchkcmd = keyword(i)(1:c3)
         endif

      else if (keyword(i) .eq. 'data') then
! ddname0(1:ddlaenge0) is directory of data as given in .inp file. It is 
!    relative to input file unless given as absolute path (starts with / or 
!    pointer to oppath)
! ddname(1:ddlaenge) is directory including path. ddname .ne. ddname0 unless
!    an absolute path is given in .inp
! dbdlength is length of directory name w/o path. Usually dbdlength = ddlaenge0
!    unless an absolute path is given in .inp
         if (keyword(i+1) .eq. '=') then
            i=i+2
            if (keyword(i+1) .eq. ',') then
               if (keyword(i+2) .eq. 'oppath') then
                  ddname0 = oppath(1:opplaenge)//'/'//keyorig(i)
                  ddlaenge0 = opplaenge+lc(i)+1
                  ddname = ddname
                  ddlaenge = ddlaenge
                  i=i+2
               endif
            else
               ddname0=keyorig(i)
               ddlaenge0 = lc(i)
               ddname=ddname0
               ddlaenge=ddlaenge0
               if (ddname(1:1) .ne. '/') call abspath(ddname,ddlaenge)
            endif
         endif
         dbdlength=ddlaenge-index(ddname,'/',back=.true.) 

      else if (keyword(i) .eq. 'db') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            if (keyword(i) .eq. 'rdwr') then
               lddwrdb = .true.
               lddrddb = .true.
            else if (keyword(i) .eq. 'read') then
               lddwrdb = .false.
               lddrddb = .true.
            else if (keyword(i) .eq. 'only') then
               lddwrdb = .false.
               lddrddb = .true.
            else if (keyword(i) .eq. 'write') then
               lddwrdb = .true.
               lddrddb = .false.
! if DB not used must store PES in memory
               ldbsave1 = .true.
            else if (keyword(i) .eq. 'none') then
               lddwrdb = .false.
               lddrddb = .false.
! if DB not used must store PES in memory
               ldbsave1 = .true.
            else 
               routine = 'Eindirdyn '
               message = 'For DB argument '// &
                         'choose among rdwr/read/only'
                  call errormsg
            endif

         else
            message = 'No argument specified with DB keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'localdb') then
         ldbsmall = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) numrec
            if (ierr .ne. 0) then
               message = 'Error reading LOCALDB = NUMREC keyword'
               if (pass .eq. 2) call warnmsg
            endif
         endif
      else if (keyword(i) .eq. 'no_localdb') then
         ldbsmall = .false.

      else if (keyword(i) .eq. 'dbnconf') then
         routine = 'Eindirdyn'
         message = 'Use db_shepard keyword to set confidence radius.'
         call errormsg

      else if (keyword(i) .eq. 'dipole') then
         ldipdb = .true.

!-----------------------------------------------------------------------
! If true convert DB to ascii on exit
! REDUNDANT option after introduction of SQL DB
!-----------------------------------------------------------------------
      else if (keyword(i) .eq. 'ascii_db') then
         routine = 'Eindirdyn'
         message = 'Ascii_db: Obsolete keyword. Use convdbfromsql to ' &
                    //'create ascii DB'
         if (pass .eq. 2) call warnmsg1

!-----------------------------------------------------------------------
! update options for direct dynamics potential
! ddupdate = 0  check DB and always update 
!            1  check DB and update if required and if past set time
!            2  never update - take from DB whatever
!            3  Error checking (equivalent to dbvar)
!-----------------------------------------------------------------------
      else if( keyword(i) .eq. 'update' )  then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            if (keyword(i) .eq. 'always') then
               ddupdate = 0
            else if (keyword(i) .eq. 'tout') then
               ddupdate = 1
               dduptim = out1
            else if (keyword(i) .eq. 'time') then
               ddupdate = 1
               if (keyword(i+1) .eq. ',') then
                  i=i+2
                  read(keyword(i),*,iostat=ierr) dduptim
               endif
            else if (keyword(i) .eq. 'never') then
               ddupdate = 2
            else if (keyword(i) .eq. 'error') then
               ddupdate = 3
               ldbvar = .true.
               if (keyword(i+1) .eq. ',') then
                  i=i+2
                  read(keyword(i),*,iostat=ierr) dddbvar
               endif
            else
               message =  &
                   'Unknown argument specified with UPDATE keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif

      else if (keyword(i) .eq. 'dbmin') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) ddmindb
            if (ierr .ne. 0) then
               message = 'Error reading DBMIN keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with DBMIN keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'db_point_group') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            dbpntgp0 = fpntgp(keyword(i))
            if (dbpntgp0 .eq. -99) then
               write(message,'(3a)') 'Error reading DB_POINT_GROUP keyword. ', &
                   'Unknown point group : ',keyword(i)(1:10)
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with DB_POINT_GROUP keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'dbminwgt') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) dbminwgt
            if (ierr .ne. 0) then
               message = 'Error reading DBMINWGT keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with DBMINWGT keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'dbdisp') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) dbdisp
            if (ierr .ne. 0) then
               message = 'Error reading DBDISP keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with DBDISP keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'dbdegen') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) dbdegen
            if (ierr .ne. 0) then
               message = 'Error reading DBDEGEN keyword'
               if (pass .eq. 2) call warnmsg
            endif
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keyunits(dbdegen,keyword(i),ierr)
               if (ierr .eq. 1) then
                  message = 'no unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif 
         else
            message = 'No argument specified with DBDEGEN keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'ignore_degen0') then
         lnodegen0 = .true.

      else if (keyword(i) .eq. 'no_ignore_degen0') then
         lnodegen0 = .false.

      else if (keyword(i) .eq. 'ignore_degen') then
         lnodegen = .true.

      else if (keyword(i) .eq. 'no_ignore_degen') then
         lnodegen = .false.

      else if (keyword(i) .eq. 'check_dcp') then
         lchkdcp = .true.

      else if (keyword(i) .eq. 'no_check_dcp') then
         lchkdcp = .false.

      else if (keyword(i) .eq. 'max_st_interest') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) mxsinterest
            if (ierr .ne. 0) then
               message = 'Error reading DBDEGEN keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with MAX_ST_INTEREST keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'dbmaxe') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) dbmaxe
            if (ierr .ne. 0) then
               message = 'Error reading DBMAXE keyword'
               if (pass .eq. 2) call warnmsg
            endif
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keyunits(dbmaxe,keyword(i),ierr)
               if (ierr .eq. 1) then
                  message = 'no unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif 
         else
            message = 'No argument specified with DBMAXE keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'dbadreg') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) dbadreg
            if (ierr .ne. 0) then
               message = 'Error reading DBADREG keyword'
               if (pass .eq. 2) call warnmsg
            endif
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keyunits(dbadreg,keyword(i),ierr)
               if (ierr .eq. 1) then
                  message = 'no unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif 
         else
            message = 'No argument specified with DBADREG keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'dbinterord') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) dbinterord
            if (ierr .ne. 0) then
               message = 'Error reading DBINTERORD keyword'
               if (pass .eq. 2) call warnmsg
            endif
            if(keyword(i+1).eq.',')then
               i=i+2
               read(keyword(i),*,iostat=ierr) dbinterord1
               if (ierr .ne. 0) then
                  message = 'Error reading DBINTERORD1 keyword'
                  if (pass .eq. 2) call warnmsg
               endif
            endif
         else
            message = 'No argument specified with DBINTERORD keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'db_shepard') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) dbnconf
            if (ierr .ne. 0) then
               message = 'Error reading DBNCONF keyword'
               if (pass .eq. 2) call warnmsg
            endif
            if(keyword(i+1).eq.',')then
               i=i+2
               read(keyword(i),*,iostat=ierr) dbinterord
               if (ierr .ne. 0) then
                  message = 'Error reading DBINTERORD keyword'
                  if (pass .eq. 2) call warnmsg
               endif
            endif
            if(keyword(i+1).eq.',')then
               i=i+2
               read(keyword(i),*,iostat=ierr) dbinterord1
               if (ierr .ne. 0) then
                  message = 'Error reading DBINTERORD1 keyword'
                  if (pass .eq. 2) call warnmsg
               endif
            endif
         else
            message = 'No argument specified with DB_SHEPARD keyword'
            if (pass .eq. 2) call warnmsg
         endif


      else if (keyword(i) .eq. 'mindv') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) dddvmin
            if (ierr .ne. 0) then
               message = 'Error reading MINDV keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with MINDV keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'rediabatise') then
         lrediab = .true.
         if (keyword(i+1) .eq. '=') then
            do
               i=i+2
               if (keyword(i) .eq. 'nomap') then
                  lrediabmap = .false.
               else if (keyword(i) .eq. 'nodcp_swap') then
                  lrediabswap = .false.
               else if (keyword(i) .eq. 'map') then
                  lrediabmap = .true.
               else if (keyword(i) .eq. 'dcp_swap') then
                  lrediabswap = .true.
               else
                  ilbl=index(keyword(i),' ')-1
                  write(message,'(2a)') &
                      'Unknown argument specified with rediabatise keyword :',&
                       keyword(i)(1:ilbl)
                  if (pass .eq. 2) call warnmsg
               endif
               if (i .ge. ic) exit
               if (keyword(i+1) .ne. ',') exit
            enddo
         endif

      else if (keyword(i) .eq. 'db_to_gausspot') then
         ldb2gapot = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            ddnamedb1=keyorig(i)
            ddlaengedb1 = lc(i)
            if (ddnamedb1(1:1) .ne. '/') call abspath(ddnamedb1,ddlaengedb1)
         else
            ilbl=index(keyword(i),' ')-1
            write(message,'(a)') &
                'No DB specified with db_to_gausspot'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'shepgrad') then
         lshepgrad = .true.

      else if (keyword(i) .eq. 'gausspot') then
         lgausspot = .true.
         lhaorder = 2
         gapupdate = 1

         if (keyword(i+1) .eq. '=') then
            i=i+2
            if(keyword(i) .eq. 'fit' .or. keyword(i).eq.'test')then
               lgaptest = .true.
            else
               message = 'Unknown argument specified with keyword : '
               call wrline(i,iz,iz0)
            endif
            if(keyword(i+1).eq.',')then
               i=i+2
               if(keyword(i).eq.'salicyl')then
                  iddmtest=1
               else if(keyword(i).eq.'malon')then
                  iddmtest=2
               else if(keyword(i).eq.'double')then
                  iddmtest=3
               else if(keyword(i).eq.'single')then
                  iddmtest=4
               else if(keyword(i).eq.'but')then
                  iddmtest=5
               else if(keyword(i).eq.'form')then
                  iddmtest=6
               else
                  message = 'Unknown argument specified with keyword : '
                  call wrline(i,iz,iz0)
               endif
            endif
         endif

      else if (keyword(i) .eq. 'pestest') then
         lpestest = .true.
         lhaorder = 2

         if (keyword(i+1) .eq. '=') then
            i=i+2
            if(keyword(i).eq.'malon')then
               idpestest=1
            else
               message = 'Unknown argument specified with keyword : '
               call wrline(i,iz,iz0)
            endif
         endif

! DD-MCTDH operator keywords.
      else if (keyword(i) .eq. 'ddmorder') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr)ddmorder
            if (ierr .ne. 0) then
               message = &
                 'Argument for ddmorder must be an integer. Not :'
               j=index(message,':')+2
               message=message(1:j)//keyword(i)
               if (pass .eq. 2) call warnmsg
            endif
         else
            message='ddmorder must be followed by ='
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'ddmoptorder') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr)ddmoptord
            if (ierr .ne. 0) then
               message = &
                 'Argument for ddmoptorder must be an integer. Not :'
               j=index(message,':')+2
               message=message(1:j)//keyword(i)
               if (pass .eq. 2) call warnmsg
            endif
         else
            message='ddmoptorder must be followed by ='
            if (pass .eq. 2) call warnmsg
         endif
         lddmopt = .true.

      else if (keyword(i) .eq. 'ddgbsvd')then
         ddmoptord = 2
         lddgbtendec = .true.
         if(keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr)ddgbsing
            if (ierr .ne. 0) then
               message = &
                 'Argument for ddgbsvd (intra-state) must be a number. Not :'
               j=index(message,':')+2
               message=message(1:j)//keyword(i)
               if (pass .eq. 2) call warnmsg
            endif
            ddgbsingss1=ddgbsing
            if(keyword(i+1) .eq. ',')then
               i = i+2
               read(keyword(i),*,iostat=ierr)ddgbsingss1
               if (ierr .ne. 0) then
                  message = &
                       'Conv. param. for ddgbsvd (inter-state) must be a number. Not :'
                  j=index(message,':')+2
                  message=message(1:j)//keyword(i)
                  call warnmsg
               endif
            endif
         endif

      else if (keyword(i) .eq. 'ddgbtensdec')then
         lddgbtendec = .true.
         ddmoptord = 2
         if(keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr)ddmoptord
            if (ierr .ne. 0) then
               message = &
                 'Argument for ddgbtensdec must be a number. Not :'
               j=index(message,':')+2
               message=message(1:j)//keyword(i)
               if (pass .eq. 2) call warnmsg
            endif
            if(keyword(i+1) .eq. ',')then
               i = i+2
               read(keyword(i),*,iostat=ierr)ddgbsing
               if (ierr .ne. 0) then
                  message = &
                       'Conv. param. for ddgbtensord must be a number. Not :'
                  j=index(message,':')+2
                  message=message(1:j)//keyword(i)
                  if (pass .eq. 2) call warnmsg
               endif
               ddgbsingss1=ddgbsing
            endif
            if(keyword(i+1) .eq. ',')then
               i = i+2
               read(keyword(i),*,iostat=ierr)ddgbtens
               if (ierr .ne. 0) then
                  message = &
                       'Conv. param. for ddgbtensord must be a number. Not :'
                  j=index(message,':')+2
                  message=message(1:j)//keyword(i)
                  if (pass .eq. 2) call warnmsg
               endif
            else if(ddmoptord.eq.3)then
               ddgbtens = ddgbsing
            endif
         endif

      else if( keyword(i) .eq. 'gap_update' )  then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            if (keyword(i) .eq. 'tout') then
               gapupdate = 1
               gapuptim = out1
            else if (keyword(i) .eq. 'time') then
               gapupdate = 1
               if (keyword(i+1) .eq. ',') then
                  i=i+2
                  read(keyword(i),*,iostat=ierr) gapuptim
                  gapuptim = gapuptim*fs
               endif
            else if (keyword(i) .eq. 'never') then
               gapupdate = 2
            else
               message =  &
                    'Unknown argument specified with GAP_UPDATE keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif

      else if (keyword(i) .eq. 'gapderiv')then
         lgapderiv = .true.

      else if (keyword(i) .eq. 'gapusesym')then
         lgapusesym = .true.

      else if (keyword(i) .eq. 'gapsobol')then
         lgapsobol = .true.

      else if (keyword(i) .eq. 'gaprandom')then
         lgapsobol = .false.

      else if (keyword(i) .eq. 'gapallgrd')then
         lgapallgrd = .true.
         lgapsobol = .false.

      else if (keyword(i) .eq. 'gapvarmax')then
         lgapvarmax = .true.

      else if (keyword(i) .eq. 'gapparopt')then
         lgapparopt = .true.

      else if (keyword(i) .eq. 'gapuseinwf')then
         lgapuseinwf = .true.

      else if (keyword(i) .eq. 'gapvarprint')then
         lgapvarprint = .true.

      else if (keyword(i) .eq. 'gapadiabwf')then
         lgapadiabwf = .true.

      else if(keyword(i) .eq. 'gapker')then
         if (keyword(i+1) .eq. '=') then
            i=i+2

            if(keyword(i) .eq. '{') then
               i=i+1
               j=1
               do 
                  if(keyword(i)(1:3).eq.'gau')then
                     gapker(j) = 1
                  else if(keyword(i)(1:3).eq.'exp')then
                     gapker(j) = 2
                  else if(keyword(i)(1:4).eq.'sgau')then
                     gapker(j) = 3
                  else
                     message = 'Unrecognised kernel keyword'
                     if (pass .eq. 2) call warnmsg
                  endif

                  if(trim(keyword(i+1)).eq.'}')then
                     i=i+1
                     exit
                  else if(keyword(i+1).ne.',')then
                     message = 'Error in GAPKER keyword list'
                     if (pass .eq. 2) call warnmsg
                  endif
                  i=i+2
                  j=j+1
                  if(j.gt.size(gapker))then
                     message = 'GAPKER input array too long'
                     call errormsg
                  endif
               enddo
            else if(keyword(i) .eq. '[')then
               i=i+1
               k=0
               do
                  k=k+1
                  if(k.gt.size(gapker))then
                     message = 'GAPKER input array too long'
                     call errormsg
                  endif
                  
                  if(keyword(i).eq.'all')then
                     if(keyword(i+1).eq.'=')then
                        i=i+2
                        if(keyword(i)(1:3).eq.'gau')then
                           tmpi = 1
                        else if(keyword(i)(1:3).eq.'exp')then
                           tmpi = 2
                        else if(keyword(i)(1:4).eq.'sgau')then
                           tmpi = 3
                        else
                           tmpi = 0
                           message = 'Unrecognised kernel keyword: all'
                           if (pass .eq. 2) call warnmsg
                        endif                       
                        gapker(1:)=tmpi
                     else
                        message = 'No argument specified with GAPKER/all keyword'
                        if (pass .eq. 2) call warnmsg
                     endif
                  else if(trim(keyword(i)).eq.']')then
                     i=i+1
                     exit
                  else
                     if(keyword(i).eq.',')i=i+1
                     read(keyword(i),*,iostat=ierr) j 
                     if (ierr .ne. 0) then
                        message = 'Error reading gapker: mode index'
                        if (pass .eq. 2) call warnmsg
                     endif
                     if(j.gt.size(gapker))then
                        message = 'GAPKER/mode no. out of bounds'
                        call errormsg
                     endif
 
                     if(keyword(i+1).eq.'=')then
                        i=i+2
                        tmpi=0
                        if(keyword(i)(1:3).eq.'gau')then
                           tmpi = 1
                        else if(keyword(i)(1:3).eq.'exp')then
                           tmpi = 2
                        else if(keyword(i)(1:4).eq.'sgau')then
                           tmpi = 3
                        else
                           tmpi = 0
                           write(message,'(a,i6)') 'Unrecognised kernel keyword: mode ',j
                           if (pass .eq. 2) call warnmsg
                        endif                       
                        gapker(j)=tmpi
                     else
                        write(message,'(3a)') 'No argument specified with ',&
                             'GAPKER/mode no. keyword: ',keyword(i+1)
                        if (pass .eq. 2) call warnmsg
                     endif
                  endif
                  i=i+1
               enddo
            else
               tmpi=0
               if(keyword(i)(1:3).eq.'gau')then
                  tmpi = 1
               else if(keyword(i)(1:3).eq.'exp')then
                  tmpi = 2
               else if(keyword(i)(1:4).eq.'sgau')then
                  tmpi = 3
               else
                  tmpi = 0
                  message = 'Unrecognised kernel type'
                  if (pass .eq. 2) call warnmsg
               endif
               gapker(1:) = tmpi
            endif
         else
            message = 'No argument specified with GAPKER keyword'
            if (pass .eq. 2) call warnmsg
         endif      

      else if (keyword(i) .eq. 'gapalpha') then
         if (keyword(i+1) .eq. '=') then
            i=i+2

            if(keyword(i) .eq. '{') then
               i=i+1
               j=1
               do 
                  read(keyword(i),*,iostat=ierr) gapalpha(j)
                  if (ierr .ne. 0) then
                     message = 'Error reading gapalpha keyword'
                     if (pass .eq. 2) call warnmsg
                  endif
                  if(trim(keyword(i+1)).eq.'}')then
                     i=i+1
                     exit
                  else if(keyword(i+1).ne.',')then
                     message = 'Error in GAPALPHA keyword list'
                     if (pass .eq. 2) call warnmsg
                  endif
                  i=i+2
                  j=j+1
                  if(j.gt.size(gapalpha))then
                     message = 'GAPALPHA input array too long'
                     call errormsg
                  endif
               enddo
            else if(keyword(i) .eq. '[')then
               i=i+1
               k=0
               do
                  k=k+1
                  if(k.gt.size(gapalpha))then
                     message = 'GAPALPHA input array too long'
                     call errormsg
                  endif
                  
                  if(keyword(i).eq.'all')then
                     if(keyword(i+1).eq.'=')then
                        i=i+2
                        read(keyword(i),*,iostat=ierr)tmpa
                        if (ierr .ne. 0) then
                           message = 'Error reading gapalpha keyword'
                           if (pass .eq. 2) call warnmsg
                        endif
                        gapalpha(1:)=tmpa
                     else
                        message = 'No argument specified with GAPALPHA/all keyword'
                        if (pass .eq. 2) call warnmsg
                     endif
                  else if(trim(keyword(i)).eq.']')then
                     i=i+1
                     exit
                  else
                     if(keyword(i).eq.',')i=i+1
                     read(keyword(i),*,iostat=ierr) j 
                     if (ierr .ne. 0) then
                        message = 'Error reading gapalpha: mode index'
                        if (pass .eq. 2) call warnmsg
                     endif
                     if(j.gt.size(gapalpha))then
                        message = 'GAPALPHA/mode no. out of bounds'
                        call errormsg
                     endif
 
                     if(keyword(i+1).eq.'=')then
                        i=i+2
                        read(keyword(i),*,iostat=ierr)tmpa
                        if (ierr .ne. 0) then
                           message = 'Error reading gapalpha value'
                           if (pass .eq. 2) call warnmsg
                        endif
                        gapalpha(j)=tmpa
                     else
                        write(message,'(3a)') 'No argument specified with ',&
                             'GAPALPHA/mode no. keyword: ',keyword(i+1)
                        if (pass .eq. 2) call warnmsg
                     endif
                  endif
                  i=i+1
               enddo
            else
               read(keyword(i),*,iostat=ierr) tmpa
               if (ierr .ne. 0) then
                  message = 'Error reading gapalpha keyword'
                  if (pass .eq. 2) call warnmsg
               endif
               gapalpha(1:) = tmpa
            endif
         else
            message = 'No argument specified with GAPALPHA keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'optgapalpha') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) optgapalpha
            if (ierr .ne. 0) then
               message = 'Error reading optgapalpha keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with OPTGAPALPHA keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'gapsigma') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) gapsigma
            if (ierr .ne. 0) then
               message = 'Error reading gapsigma keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with GAPSIGMA keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'gapkfac') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) gapkfac
         else
            message = 'No argument specified with GAPKFAC keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'gapvartol') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) gapvartol
            if (ierr .ne. 0) then
               message = 'Error reading gapvartol keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with GAPVARTOL keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'gapsdsamp') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) gapsd
            if (ierr .ne. 0) then
               message = 'Error reading gapsdsamp keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with GAPSDSAMP keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'gapsample') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) gapsample
            if (ierr .ne. 0) then
               message = 'Error reading gapsample keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with GAPSAMPLE keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'gapethresh') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) gapethresh
            if (ierr .ne. 0) then
               message = 'Error reading gapethresh keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with GAPETHRESH keyword'
            if (pass .eq. 2) call warnmsg
         endif

         if (keyword(i+1) .eq. ',' ) then
            i = i+2
            call keyeunits(gapethresh,keyword(i),ierr)
            if (ierr .eq. 1) then
               message = 'No energy unit after comma: gapethresh'
               call wrline(zeroi,iz,iz0)
            endif
         endif
         lgapethresh = .true.

      else if (keyword(i) .eq. 'gapeshift') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) gapeshift
            if (ierr .ne. 0) then
               message = 'Error reading gapeshift keyword'
               if (pass .eq. 2) call warnmsg
            endif
         else
            message = 'No argument specified with GAPESHIFT keyword'
            if (pass .eq. 2) call warnmsg
         endif

         if (keyword(i+1) .eq. ',' ) then
            i = i+2
            call keyeunits(gapeshift,keyword(i),ierr)
            if (ierr .eq. 1) then
               message = 'No energy unit after comma: gapeshift'
               call wrline(zeroi,iz,iz0)
            endif
         endif

      else if(keyword(i) .eq. 'gapcompare')then
         lgapcomp = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) ngapcomp
         else
            message = 'No argument specified with GAPCOMPARE keyword'
            if (pass .eq. 2) call warnmsg
         endif
            
      else if (keyword(i) .eq. 'cicut') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) cicocut
         else
            message = 'No argument specified with cicut keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'casdecoup') then
         ldecoup = .true.

      else if(keyword(i) .eq. 'malonpf')then
         lmalon = .true.

      else if (keyword(i) .eq. 'dbvar') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) dddbvar
         endif
         ldbvar = .true.

      else if (keyword(i) .eq. 'dbhar') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) dddbhar
         endif
         ldbhar = .true.

      else if (keyword(i) .eq. 'dd_adiab') then
         dddiab = 0

      else if (keyword(i) .eq. 'dd_diab') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            if (keyword(i) .eq. 'regularized') then
               dddiab = 1
            else if (keyword(i) .eq. 'local') then
               dddiab = 2
               lnactdb = .true.
            else if (keyword(i) .eq. 'global_old') then
               write(message,'(2a)') 'Deprecated option for dd_diab :', &
                     keyword(i)(1:lc(i))
               call wrline(i,iz,iz0)
            else if (keyword(i) .eq. 'global') then
               dddiab = 4
               lupdhes1 = .true.
               ldbsave1 = .true.
            else if (keyword(i) .eq. 'robertson') then
               dddiab = 5
               lupdhes1 = .true.
               ldbsave1 = .true.
            else if (keyword(i) .eq. 'projection') then
               dddiab = 5
               lupdhes1 = .true.
               ldbsave1 = .true.
            else if (keyword(i) .eq. 'procrustes')then
               dddiab = 6
               ldbsave = .true.              
               lupdhes1 = .true.
               ldbsave1 = .true.
            else if (keyword(i) .eq. 'none')then
               dddiab = 0
               if (nddstate .gt. 1) lnactdb = .true.
            else
               write(message,'(2a)') 'Unknown option for dd_diab: ', &
                     keyword(i)(1:lc(i))
               call wrline(i,iz,iz0)
            endif
         else
            dddiab = 4
            lupdhes1 = .true.
            ldbsave1 = .true.
         endif

      else if (keyword(i) .eq. 'diab_integrate') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            if (keyword(i) .eq. 'original') then
               diabint = 0
            else if (keyword(i) .eq. 'linear') then
               diabint = 1
            else if (keyword(i) .eq. 'shepard') then
               diabint = 2
            else
               write(message,'(2a)') 'Unknown option for dd_diab: ', &
                     keyword(i)(1:lc(i))
               call wrline(i,iz,iz0)
            endif
         else
            write(message,'(a)') 'No option given for diab_integrate '
            call wrline(i,iz,iz0)
         endif

      else if (keyword(i) .eq. 'socbas')then
         lsocbas = .true.

      else if (keyword(i) .eq. 'spinbas')then
         lsocbas = .false.

      else if (keyword(i) .eq. 'socdegen')then
         lsocdegen = .true.

      else if (keyword(i) .eq. 'no_socdegen')then
         lsocdegen = .false.

      else if (keyword(i) .eq. 'sacas') then
         lsacas = .true.

      else if (keyword(i) .eq. 'no_sacas') then
         lsacas = .false.

      else if (keyword(i) .eq. 'ddtrans') then
         lddtrans = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) ndofddpes
            if (ndofddpes .gt. maxdim-2) then
               write(message,'(a,i5)') 'Redimension MAXDIM in '// &
                    'maxdim.inc. Minimum: ',ndofddpes+2
               call wrline(i,iz,iz0)
            endif
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif

      else if (keyword(i) .eq. 'transfile') then
         if (lddtrans .and. ddtrfile .eq. 'inp') then
            routine = 'Eindirdyn, direct-section (Eindirdynmod.f90)'
            message = 'DDTRANS-SECTION present and transfile specified'
            call errormsg
         endif

         lddtrans = .true.
! if TSH, coords must be transformed too
         if (ltraj .and. ddcoo.eq. 1) then
            ltshtrans = .true.
            tshtrfile = 'lvc '
         endif
         if (keyword(i+1) .eq. '=') then
            i=i+2
            ddtrfile = keyorig(i)(1:c4)
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif
         if (keyword(i+1) .eq. ',') then
            i=i+2
            if (keyword(i) .eq. 'e0read') then
               lrde0 = .true.
               trfiletyp = 1
            else
               message = 'Unknown 2nd argument with keyword : '
               call wrline(i-4,iz,iz0)
            endif
            if (keyword(i+1) .eq. ',') then
               i=i+2
               if (keyword(i) .eq. 'rhf') then
                  trfiletyp = 1
               else if (keyword(i) .eq. 'mp2') then
                  trfiletyp = 2
               else if (keyword(i) .eq. 'mp3') then
                  trfiletyp = 3
               else if (keyword(i) .eq. 'mp4') then
                  trfiletyp = 4
               else if (keyword(i) .eq. 'uhf') then
                  trfiletyp = 5
               else if (keyword(i) .eq. 'cas') then
                  trfiletyp = 12
               else if (keyword(i) .eq. 'pt2') then
                  trfiletyp = 13
               else if (keyword(i) .eq. 'dft') then
                  trfiletyp = 14
               else
                  message = 'Unknown 3nd argument with keyword : '
                  call wrline(i-4,iz,iz0)
               endif
            endif
         endif

      else if (keyword(i) .eq. 'ntr') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*) ntr
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif

      else if (keyword(i) .eq. 'coinfile') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            coinfile = keyword(i)(1:c4)
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif
         if (keyword(i+1) .eq. ',') then
            i=i+2
            if (keyword(i) .eq. 'cas') then
               coinfiletyp = 12
            else if (keyword(i) .eq. 'pt2') then
               coinfiletyp = 13
            else
               message = 'Unknown 3nd argument with keyword : '
               call wrline(i-4,iz,iz0)
            endif
         else
            message = 'Coin File type must be specified : '
            call wrline(i-4,iz,iz0)
         endif

      else if (keyword(i) .eq. 'dreffile') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            dreffile = keyword(i)(1:c4)
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif

      else if (keyword(i) .eq. 'order_by_mass') then
         lddreord = .true.

      else if (keyword(i) .eq. 'no_order_by_mass') then
         lddreord = .false.

      else if (keyword(i) .eq. 'dbave' ) then
         if (keyword(i+1) .eq. '=') then
            i=i+2
         endif
         message = 'Dbave keyword obsolete. Ignored. '
         if (pass .eq. 2) call warnmsg

      else if (keyword(i) .eq. 'propcas' ) then
         lsacas = .true.

!-----------------------------------------------------------------------
! control reading initial orbitals for a CAS calculation
! lorbread = .false. CAS guess generated at each point
! lorbread = .true. CAS guess generated from previous point
! lorbread0 as lorbread, but for initial DB calculation only
!-----------------------------------------------------------------------
      else if (keyword(i) .eq. 'orbread' ) then
         lorbread = .true.

      else if (keyword(i) .eq. 'no_orbread' ) then
         lorbread = .false.
         lorbread0 = .false.

      else if (keyword(i) .eq. 'orbread0' ) then
! force lorbread0 to be true if set false bu no_orbread
         lorbread00 = .true.

      else if (keyword(i) .eq. 'no_orbread0' ) then
         lorbread0 = .false.
!-----------------------------------------------------------------------

      else if (keyword(i) .eq. 'ddlog') then
         ddlog = .true.

      else if (keyword(i) .eq. 'noddlog') then
         ddlog = .false.

      else if (keyword(i) .eq. 'ddlog_full') then
         ddlogfull = .true.

      else if (keyword(i) .eq. 'noddlog_full') then
         ddlogfull = .false.

      else if (keyword(i) .eq. 'logop_full') then
         loplogfull=.true.

      else if (keyword(i) .eq. 'dbsave') then
         ldbsave2 = .true.

      else if (keyword(i) .eq. 'nodbsave') then
         ldbsave2 = .false.
      
      else if (keyword(i) .eq. 'hess_upd') then
         lupdhes2 = .true.
         ldbsave1 = .true.

      else if (keyword(i) .eq. 'nohess_upd') then
         lupdhes2 = .false.

      else if (keyword(i) .eq. 'hess0') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            if (keyword(i) .eq. 'calculate') then
                hess0 = 0
            else if (keyword(i) .eq. 'gs') then
                hess0 = 1
            else
               message = 'Unknown argument specified with hess0 keyword'
               if (pass .eq. 2) call warnmsg
            endif

         else
            message = 'No argument specified with hess0 keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'nbasis') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) nbasis
            if (ierr .ne. 0) then
               message = 'Error reading NBASIS keyword'
               if (pass .eq. 2) call warnmsg
            endif

         else
            message = 'No argument specified with NBASIS keyword'
            if (pass .eq. 2) call warnmsg
         endif
         lnbasis = .true.

      !See if different sql_version than default is needed
      elseif(keyword(i) .eq. 'sql_version') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) sqlvers
            if (ierr .ne. 0) then
               message = 'Error reading SQL_VERSION keyword'
               if (pass .eq. 2) call warnmsg
            endif

         else
            message = 'No argument specified with SQL_VERSION keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'nocc') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) nocc
            if (ierr .ne. 0) then
               message = 'Error reading NOCC keyword'
               if (pass .eq. 2) call warnmsg
            endif

         else
            message = 'No argument specified with NOCC keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'nclosed') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) nclosed
            if (ierr .ne. 0) then
               message = 'Error reading NCLOSED keyword'
               if (pass .eq. 2) call warnmsg
            endif

         else
            message = 'No argument specified with NCLOSED keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if (keyword(i) .eq. 'nelectron') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) nelectron
            if (ierr .ne. 0) then
               message = 'Error reading NELECTRON keyword'
               if (pass .eq. 2) call warnmsg
            endif

         else
            message = 'No argument specified with NELECTRON keyword'
            if (pass .eq. 2) call warnmsg
         endif

      else if(keyword(i) .eq. 'logop_full')then
         message = 'Logop_full keyword moved to OPERATOR-SECTION.'
         call errormsg
            
      else

         message = ' unknown keyword : '
         call wrline(i,iz,iz0)
      end if

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         goto 15
      else
         goto 10
      endif

   20 continue

!-----------------------------------------------------------------------
! If full logging asked for, turn on normal logging
!-----------------------------------------------------------------------
      if (ddlogfull) ddlog = .true.

!----------------------------------------------------------------------- 
! Set state multiplicity data
! nddspst : No. of spin states (mutiplicities)
! nddmult(s) : No. of states of multiplicity s.
!
! Default: all singlets
!----------------------------------------------------------------------- 
      if (nddspst .eq. 0) then 
         nddspst = 1
         nddmult(1) = nddstate
      endif

!----------------------------------------------------------------------- 
! check state multiplicity input
!
! Nddstate = Sum(Nstates*mult)
!----------------------------------------------------------------------- 
      n = 0
      if (lsocdegen) then
         do sp=1,nddspst
            n = n + nddmult(sp)
         enddo
      else
         do sp=1,nddspst
            n = n + nddmult(sp)*sp
         enddo
      endif
      if (n .ne. nddstate) then
         routine = 'Eindirdyn (eindirdynmod.f90)'
         write(message(1:80),'(a)') &
          'Inconsistent no. of states. Check state_mult and primitive el basis.'
         write(message(81:160),'(2(a,i0))') &
                  'Nddstate = ',nddstate,'  NQCstates = ',n
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! Set up spin blocks in set of states
! QC states are degenerate
! DD states may be degenerate or multiplicities split into components
!
! nsmult: No. of multiplicities
! idxmult(sb): Multiplicty of spin block (sb)
! imultblk(sb): No. QC (unsplit) states in spin block
! imultoff(sb): Offset of spin block in list of QC states
! stoff(sb): Offset of spin block in list of split states
! stblk(sb): No of split states in spin block 
! nsocstate: Total no. of split states
!-----------------------------------------------------------------------
      nsmult = 0
      idxmult = 0
      imultblk = 0
      imultoff = 0
      stoff = 0
      stblk = 0

! Set up spin blocks
      do sp = 1,nddspst
         if (nddmult(sp) .gt. 0) then
            nsmult = nsmult+1
            idxmult(nsmult) = sp
            imultblk(nsmult) = nddmult(sp)
         endif
      enddo
! Get no. of states and spin block offset in state list
      do sp = 1,nsmult
         stblk(sp) = idxmult(sp)*imultblk(sp)
      enddo
      do sp = 2,nsmult
         imultoff(sp) = imultoff(sp-1) + imultblk(sp-1)
         stoff(sp) = stoff(sp-1) + stblk(sp-1)
      enddo
      nsocstate = stoff(nsmult) + stblk(nsmult)
      maxmult = idxmult(nsmult)   ! maximum multiplicity
      maxmultblk = maxval(imultblk(1:nsmult))

      maxqcsta = nsocstate

! block offsets and sizes in DD calculation
      if (lsocdegen) then
         do sp = 1,nsmult
            ddstblk(sp) = imultblk(sp)
            ddstoff(sp) = imultoff(sp)
         enddo
      else
         do sp = 1,nsmult
            ddstblk(sp) = stblk(sp)
            ddstoff(sp) = stoff(sp)
         enddo
      endif

!-----------------------------------------------------------------------
! imultqc(s): multiplicity of state s in QC list
! imultmap(s): multiplicity of state s in DD list 
!-----------------------------------------------------------------------
      s = 0
      s1 = 0
      do sp=1,nsmult
         do i=1,imultblk(sp)
            s = s+1
            imultqc(s) = idxmult(sp)
            if (lsocdegen) then
               s1 = s1+1
               imultmap(s1) = idxmult(sp)
            else
               do j=1,idxmult(sp)
                  s1 = s1+1
                  imultmap(s1) = idxmult(sp)
               enddo
            endif
         enddo
      enddo
      if (nddstate .ne. s1) then
         routine = 'Eindirdyn (eindirdynmod.f90)'
         message = &
       'Inconsistent no. of states. Check state_mult and socdegen'
         call warnmsg
      endif

!-----------------------------------------------------------------------
! dercpblk(sb): No. of derivative couplings for spin block sb
! dercpoff(sb): offset of derivative couplings for spin block sb
!-----------------------------------------------------------------------
      dercpoff(:) = 0
      do sp = 1,nsmult 
         dercpblk(sp) = imultblk(sp)*(imultblk(sp)-1)/2
         if (sp .gt. 1) dercpoff(sp) = dercpoff(sp-1)+dercpblk(sp-1)
      enddo

!-----------------------------------------------------------------------
! SOC required between state components or between spin blocks
!-----------------------------------------------------------------------
      if (nsmult .gt. 1) then
         lddsoc = .true.
      else if (nsmult .eq. 1) then
         if (idxmult(1) .gt. 1 .and. .not. lsocdegen) lddsoc = .true.
      endif

!-----------------------------------------------------------------------
! State mapping DD <-> QC
!-----------------------------------------------------------------------
      if (lsocdegen) then
         do s=1,nddstate
            istatemap(s) = s
         enddo
      else
         s = 0   ! QC state index
         s1 = 0  ! DD state index (nodegen)
         do sp = 1,nsmult
            do i = 1,imultblk(sp)
               s = s+1     
               do j = 1,idxmult(sp)
                  s1 = s1+1
                  istatemap(s1) = s
               enddo
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! Set the number of active space orbitals.
!-----------------------------------------------------------------------
      ncas = nocc - nclosed

!-----------------------------------------------------------------------
! Check that the GAP sampling keywords are consistent.
!-----------------------------------------------------------------------
      if(lgapsobol.and.lgapallgrd)then
         message = 'May only select one of gapsobol and gapallgrd.'
         if (pass .eq. 2) call warnmsg
      endif

      if(lgapallgrd)lgapusesym=.false.

!-----------------------------------------------------------------------
! Printing the GAP variances is only valid for exact calculations.
!-----------------------------------------------------------------------
      if(lgapvarprint.and..not.lexact)then
         message = 'Printing GAP variances is only valid for exact calcs.'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! Give a warning if CAS calculation requested but no nbasis keyword.
!-----------------------------------------------------------------------
      if(lmocodb .and. .not. lnbasis) then
         message = 'CAS calculation requested but no nbasis keyword'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! force initial guess for CAS to be read from previous calculation,
! even though no_ordread has been set to false
!-----------------------------------------------------------------------
      if (lorbread00 .and. (.not. lorbread)) lorbread0 = .true.

!-----------------------------------------------------------------------
! Check for clash in ldbsave (i.e. not asking to turn off when option
! must be on).
!-----------------------------------------------------------------------
      if (ldbsave1 .and. .not. ldbsave2) then
         message = &
    'Other selected options require storing DB in memory. Ignoring nodbsave.'
         if (pass .eq. 2) call warnmsg1
      else
         ldbsave1 = ldbsave2
      endif
      ldbsave = ldbsave1
         
!-----------------------------------------------------------------------
! Check Hessian updating possible
! Check for clash in lupdhes (i.e. not asking to turn off when option
! must be on).
! Cannot use updating if not using DB
!-----------------------------------------------------------------------
      if (lupdhes1 .and. .not. lupdhes2) then
         message = &
    'Other selected options require storing DB in memory. Ignoring nohess_upd.'
         if (pass .eq. 2) call warnmsg1
      else
         lupdhes1 = lupdhes2
      endif
      lupdhes = lupdhes1

      lupdhes = lupdhes .and. .not.(lddmctdh.or.lgausspot)

      if (lupdhes .and. .not. lddrddb ) lupdhes = .false.

!-----------------------------------------------------------------------
! Check for clash in saveall options
!-----------------------------------------------------------------------
      if (lqcsaveall .and. lqcsaveout ) then
         message = 'Ask for only one of QCSAVEALL or QCSAVEOUT '
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! Single state calcs use no diabatisation
!-----------------------------------------------------------------------
      if (nddstate .eq. 1) dddiab = 0

!-----------------------------------------------------------------------
! Check for consistency of rediabatisation request.
!-----------------------------------------------------------------------
      if(lrediab .and. (dddiab .eq. 1 .or. dddiab .eq. 2))then
         message = 'Rediabatisation requested but wrong diabatisation type'
         if (pass .eq. 2) call warnmsg
      endif
      if(lrediab .and. .not.ldbsave)then
         message = 'Rediabatisation requested but not saving DB to memory'
         if (pass .eq. 2) call warnmsg
      endif
      if(dddiab.eq.1 .and. nddstate .ne. 2)then
         message = 'Regularisation diabatisation only works for 2-states'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! Check that only one of ldbvar and ldbhar are set true.
!-----------------------------------------------------------------------
      if(ldbvar .and. ldbhar)then
         message = 'Requested both dbvar and dbhar. Only one please.'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! Check that using DB in memory when using Gaussian approximation to
! the potential.
!-----------------------------------------------------------------------
      if((lgausspot.or.lddmctdh) .and. .not. ldbsave)then
         message = 'Requested Gaussian approximation potential but not dbsave'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! If DB is not used Hessian updating is not possible
!-----------------------------------------------------------------------
      if (lupdhes .and. (.not. lddwrdb .and. .not. lddrddb)) then
         message = 'Cannot use Hessian updating with db=none'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! If DB is not used cannot use propagation diabatisation
!-----------------------------------------------------------------------
      if (dddiab .ge. 3 .and. (.not. lddwrdb .and. .not. lddrddb)) then
         message = 'Cannot use propagation diabatisation with db=none'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! If DB is not used cannot use symmetry
!-----------------------------------------------------------------------
      if (dbpntgp0 .gt. 0 .and. (.not. lddwrdb .and. .not. lddrddb)) then
         message = 'Cannot use db_point_group Symmetry with db=none'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! Check that GAP with derivatives is only used with full-D GAP
!-----------------------------------------------------------------------
      if(lgapderiv.and.ddmorder.ne.0)then
         message = 'Requested GAP with derivatives for additive GAP'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! Check that the order of the optimised GAP is sensible.
!-----------------------------------------------------------------------
      if(lddmopt .and. ddmoptord.lt.1)then
         message = 'Requested OPTGAP with order less than 1'
         if (pass .eq. 2) call warnmsg
      endif
      if(lddmopt .and. ddmoptord.gt.3)then
         message = 'Requested OPTGAP with order greater than 3'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! Check that only one of DDMOPT and DDGBSVD have been chosen.
!-----------------------------------------------------------------------
      if(lddmopt.and.lddgbtendec)then
         message = 'Requested both OPTGAP and DDGBSVD/DDGBTENDEC'
         if (pass .eq. 2) call warnmsg
      endif

      if(lddgbtendec .and. ddmorder.ne.0 .and. ddmoptord.gt.ddmorder)then
         message = 'Requested OPT/TENS GAP with order > underlying GAP'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! Check that gapcomp is only used with a read DB.
!-----------------------------------------------------------------------
      if(lgapcomp .and. .not. lddrddb)then
         message = 'Can only do a GAP comparison on a read run'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! Check that gapadiabwf is only used with a read DB.
!-----------------------------------------------------------------------
      if(lgapcomp .and. .not. lddrddb)then
         message = 'Can only convert to an adiabatic WF with GAP on a read run'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! Check that the order of the tensor decomposition of the GAP is okay.
!-----------------------------------------------------------------------
      if(lddgbtendec.and.ddmoptord.lt.2)then
         message = 'Requested GAP SVD with order less than 2'
         if (pass .eq. 2) call warnmsg
      endif
      if(lddgbtendec.and.ddmoptord.gt.3)then
         message = 'Requested GAP SVD with order greater than 3'
         if (pass .eq. 2) call warnmsg
      endif

!-----------------------------------------------------------------------
! Only makes sense to use the SOC basis if there are states of different 
! multiplicities present.
!-----------------------------------------------------------------------
      if(lsocbas.and. .not.lddsoc)then
         message = 'SOC basis calculation requested but only states '//&
              'one multiplicity present'
         if (pass .eq. 2) call warnmsg
      endif         

!-----------------------------------------------------------------------
! Use of single state SOCs with DD-GB SVD and tensor fitting not 
! implemented yet. 
!-----------------------------------------------------------------------
      if (lddsoc) then
         if(lsocdegen .and. lddgbtendec)then
            message = 'Use of degenerate SOCs (magnitudes) and secondary '//&
                 'GAP fitting (ddgbsvd or ddgbtendec) in DD-GB not '//&
                 'implemented yet.'
            call warnmsg
         endif
      endif

!-----------------------------------------------------------------------
! Use of single state SOCs with DD-GB only implemented for Procrustes
! diabatisation so far. 
!-----------------------------------------------------------------------
      if (lddsoc) then
         if(lsocdegen .and. lddmctdh .and. (dddiab .ne. 6) )then
            message = 'Use of degenerate SOCs (magnitudes) and diabatisations '//&
                 'other than Procrustes not implemented with DD-GB yet.'
            call warnmsg
         endif
      endif

!-----------------------------------------------------------------------
! Need dbsave if OMP??
!-----------------------------------------------------------------------
!      if (lompthread .and. .not. ldbsave) then
!         message = 'Need dbsave for a DD calculation using OMP'
!         if (pass .eq. 2) call warnmsg
!      endif

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr
      lwarn=.false.

      return
      end subroutine eindirdyn

!***********************************************************************
!
! Input specific to GWPTRJ calculations
!
!***********************************************************************

      subroutine eingwptrj(linperr,iz)

      implicit none

      integer(long)                  :: ic,ierr,i,iz0
      integer(long), intent(in)      :: iz
      logical(kind=4), intent(inout) :: linperr

!-----------------------------------------------------------------------
! set initial values
! Note: In order that the variable 'name' is specified, 'einfile' must 
! have been called before 'einoper'. 
!-----------------------------------------------------------------------
      routine  = 'read GWPTRJ-SECTION  (eindirdyn)'
      iz0      = 0

      if (.not. lgwptrj) return

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (keyword(i) .eq. 'end-gwptrj-section') then
         goto 20

      else if (keyword(i) .eq. 'gwptrjdir') then
         if (keyword(i+1).ne.'=') then
            gwptrjdir = ' '
            gwptrjlaenge = 0
         else
            gwptrjdir0 = keyorig(i+2)
            gwptrjlaenge0 = index(gwptrjdir0,' ')-1
            gwptrjdir = gwptrjdir0
            gwptrjlaenge = gwptrjlaenge0
            if (gwptrjdir(1:1) .ne. '/') call abspath(gwptrjdir,gwptrjlaenge)
            i=i+2
         endif

      else if (keyword(i) .eq. 'ener0') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) dde0
            if (ierr .ne. 0) then
               message = 'Error reading ener0 keyword'
               call warnmsg
            endif
            if (keyword(i+1) .eq. ',' ) then
               i = i+2
               call keyeunits(dde0,keyword(i),ierr)
               if (ierr .eq. 1) then
                  message = 'no energy unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif

         else
            message = 'No argument specified with ener0 keyword'
            call warnmsg
         endif
 
      else if (keyword(i) .eq. 'transfile') then
         if (lddtrans .and. ddtrfile .eq. 'inp') then
            routine = 'Eindirdyn, eingwptrj-section (Eindirdynmod.f90)'
            message = 'DDTRANS-SECTION present and transfile specified'
            call errormsg
         endif

         lddtrans = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            ddtrfile = keyword(i)(1:c4)
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif

      else

         message = ' unknown keyword : '
         call wrline(i,iz,iz0)
      end if

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         goto 15
      else
         goto 10
      endif

   20 continue

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr
      lwarn=.false.

      return
      end  subroutine eingwptrj

!#######################################################################
!
! check DD input correct and any information required is present
!
!#######################################################################

      subroutine chkdirdyn(linperr)

      implicit none

      logical(kind=4), save          :: check=.true.
      logical(kind=4), intent(inout) :: linperr
      character(len=240)             :: filename

      routine = 'Chkdirdyn (eindirdyn.F)'

!-----------------------------------------------------------------------
! check trafo file specified
! Cartesian coordinates do not need a trafo file
!-----------------------------------------------------------------------
      if (ddcoo .eq. 0) then
         lddtrans = .true.
         ddtrfile = 'inp'
      else if (ddcoo .eq. 1) then
         if (.not. lddtrans) then
            message = 'Transformation file or matrix not specified' 
            call warnmsg
         endif
      else if (ddcoo .eq. 2) then
         if (.not. lddtrans .and. lrunshzagreb) then
            message = &
             'No frequency transfile given for sampling of initial conditions'
            call warnmsg
         endif
         lddtrans = .false.
      endif

!-----------------------------------------------------------------------
! check that template file for direct dynamics present
!-----------------------------------------------------------------------
      if (check .and. .not.lgaptest) then
         filename=ddname(1:ddlaenge)
         filename(ddlaenge+1:ddlaenge+13)='/template.dat'
         inquire(file=filename,exist=check)
         if (.not. check) then
            message = ' There is no direct dynamics template.dat file! '// &
                  '                      '//filename(1:ddlaenge+13)
            call warnmsg
         endif
      endif

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr
      lwarn=.false.

      return
      end subroutine chkdirdyn

!#######################################################################
!
! check GWPTRJ input correct and any information required is present
!
!#######################################################################

      subroutine chkgwptrj(linperr)

      implicit none

      logical(kind=4), intent(inout) :: linperr

      routine = 'Chkgwptrj (eindirdyn.F)'

!-----------------------------------------------------------------------
! check trajectory directory is set
!-----------------------------------------------------------------------
      if (gwptrjlaenge .eq. 0) then
         gwptrjdir0 = 'trj_data'
         gwptrjlaenge0 = 8

         gwptrjdir = gwptrjdir0
         gwptrjlaenge = gwptrjlaenge0
         if (gwptrjdir(1:1) .ne. '/') call abspath(gwptrjdir,gwptrjlaenge)
      endif

!-----------------------------------------------------------------------
! check trafo file specified
! Cartesian coordinates do not need a trafo file
!-----------------------------------------------------------------------
      if (ddcoo .eq. 1) then
         if (.not. lddtrans) then
            message = 'Transformation file or matrix not specified' 
            call warnmsg
         endif
      else if (ddcoo .eq. 2) then
         lddtrans = .false.
      else if (ddcoo .lt. 1 .or. ddcoo .gt. 2) then
         message = 'Coordinate type not allowed for GWPTRJ calculation'
         call warnmsg
      endif

!-----------------------------------------------------------------------
! check that data directory exists
!-----------------------------------------------------------------------
!      filename=ddname(1:ddlaenge)
!      filename(ddlaenge+1:ddlaenge+13)='/template.dat'
!      inquire(file=filename,exist=check)
!      if (.not. check) then
!         message = ' There is no direct dynamics template.dat file! '// &
!               '                      '//filename(1:ddlaenge+13)
!         call warnmsg
!      endif

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr
      lwarn=.false.

      return
      end subroutine chkgwptrj

!#######################################################################

      subroutine cvmcgop(loperr)

      implicit none

      integer(long)                   :: m,f,h,k,s,s1,hopfreq,f1,hopfeb, &
                                         htmke,htmpe,htmfeb,i,hopqc,n
      integer(long), external         :: slen
      logical(kind=4), intent(inout)  :: loperr
      character(len=30)               :: string

      real(dop), dimension(npar)         :: rpar1
      character(len=c1), dimension(npar) :: apar1
      integer(long)                          :: gf1,mode,muld
      integer(long), dimension(maxdim)       :: grid

! need to know how many parameters were read in before entering routine
      integer(long), save   :: npar0
      logical(kind=4), save :: initial = .true.

!     integer getiatsym
!     character*10 fatnam(maxdim)

!-----------------------------------------------------------------------
! have to start counter for parameters with no. read in from files
! Due to iterative operator set up this must reset.
!-----------------------------------------------------------------------
      if (initial) then
         npar0=npar
         initial = .false.
      endif
      npar = npar0

!-----------------------------------------------------------------------
! allocate no. of DOFs in operator, and set up space for koeffs,
! electronic basis etc.
!-----------------------------------------------------------------------
      do m=1,min(nmode,maxdim)
         laddmode(m,1) = .true.
      enddo
      call genopdof

      if (lgwptrj .and. nddstate .gt. 1) then
         routine = 'Cvmcgop'
         message = 'GWPTRJ calculation must be on a single surface'
         call errormsg
      endif

!-----------------------------------------------------------------------
! assign no. of Hamiltonians
! assign minimum no. of factors needed in operators
!-----------------------------------------------------------------------
      hamdim = 1
      minfac = max(minfac,2)
!
!*** Set up labels
!-----------------------------------------------------------------------
! if hoplab needs to be initialised then:
! place unit operator at beginning of list for each degree of freedom
! place +1, -1 at beginning of list for coefficients
!-----------------------------------------------------------------------
      hoplab(1)='1'
      hoplab(2)='-1'
      hoplab(3)='I'
      if (ltraj) then
         hoplab(4)='KE_cl'
      else
         hoplab(4)='KEgwp'
      endif
      hopdim=4
!
! set up mode frequencies as parameters if nmode coordinates
!
      hopfreq=0
      if (ddcoo .eq. 1) then
         do n=1,npar
            apar1(n)=apar(n)
            rpar1(n)=rpar(n)
         enddo
         if (allocated(apar)) deallocate(apar)
         if (allocated(rpar)) deallocate(rpar)
         allocate(apar(npar+ndof))
         allocate(rpar(npar+ndof))
         do n=1,npar
            apar(n)=apar1(n)
            rpar(n)=rpar1(n)
         enddo

         hopfreq=hopdim
         do f=1,ndof
            if (f .eq. feb) cycle

            write(string,'(a6,i5)')  'freq0_',f
            call rmblnk(string)

            hopdim = hopdim+1
            hoplab(hopdim) = string
         enddo
      endif
!
! set up QC operators. hopqc is pointer to start of these.
!
      if (lgwptrj) then
         hopqc=hopdim
         write(string,'(a8)')  'qchemtrj'
         call rmblnk(string)
         hopdim = hopdim+1
         hoplab(hopdim) = string
      else if (nddstate .gt. 1) then
         hopqc=hopdim
         do s=1,nddstate
            write(string,'(a5,i5)')  'qchem',s
            call rmblnk(string)
            hopdim = hopdim+1
            hoplab(hopdim) = string
            do s1=s+1,nddstate
               write(string,'(a5,i5,a1,i5)')  'qchem',s,'&',s1
               call rmblnk(string)
               hopdim = hopdim+1
               hoplab(hopdim) = string
            enddo
         enddo
      else
         hopqc=hopdim
         write(string,'(a5)')  'qchem'
         call rmblnk(string)
         hopdim = hopdim+1
         hoplab(hopdim) = string
      endif
!
! set up electronic operators if single-set. hopfeb is pointer to start of
! electronic state operators
!
      if (nddstate .gt. 1) then
         hopfeb=hopdim
         do s=1,nddstate
            do s1=s,nddstate
               write(string,'(a1,i5,a1,i5)')  'S',s,'&',s1
               call rmblnk(string)
               hopdim = hopdim+1
               hoplab(hopdim) = string
            enddo
         enddo
      else
         hopfeb=0
      endif
!
!*** Now set up operators
!
     htmdim=0
!
! set up unit operators
!
      do f=1,nopdof
         htmdim=htmdim+1
         h=min(htmdim,maxhtm)
         hterm(1,h)=1
         htmfac(h)=1
         htmdof(h)=f
         htmadof(h)=f
         htmmode(h)=0
         htmmuld(h)=0
         htmtype(h)=1
         htmsym(h)=1
      enddo

!
! set up KE terms
! For Jacobi Cartesians and Cartesians KEgwp
! For Normal Modes freq(f)*KEgwp
!
      k = 0
      htmke=htmdim
      do f = 1,ndof
         if (f .eq. feb) cycle

         htmdim=htmdim+1
         h=min(htmdim,maxhtm)
         htmfac(h)=1
         htmdof(h)=f
         htmadof(h)=f
         htmmode(h)=0
         htmmuld(h)=0
         hterm(1,h)=4
         if (ddcoo .eq. 1) then
            htmfac(h)=2
            n=min(2,maxfac)
            hterm(n,h)=hopfreq+f
         endif
      enddo
!
! set up potential terms. 
!
      htmpe=htmdim
      i=hopqc
      do s=1,nddstate
         do s1=s,nddstate
            htmdim=htmdim+1
            h=min(htmdim,maxhtm)
            htmfac(h)=1
            htmdof(h)=0
            htmadof(h)=0
! flag V as mode or muld operator
            if ((leb .and. nmode .eq. 2) .or. (nmode .eq. 1)) then
               htmmode(h)=1
               htmmuld(h)=0
            else
               htmmode(h)=0
               htmmuld(h)=1
            endif
            !f1 = spfdof(1,1)
            !htmadof(h)=f1
            i=i+1
            hterm(1,h)=i
         enddo
      enddo

!
! set up electronic terms. 
!
      if (nddstate .gt. 1) then
         htmfeb=htmdim
         i=hopfeb
         do s=1,nddstate
            do s1=s,nddstate
               htmdim=htmdim+1
               h=min(htmdim,maxhtm)
               htmfac(h)=1
               htmdof(h)=feb
               htmadof(h)=feb
               htmmode(h)=0
               htmmuld(h)=0
               i=i+1
               hterm(1,h)=i
            enddo
         enddo
      else
         htmfeb=0
      endif

!-----------------------------------------------------------------------
!*** Finally set up Hamiltonian
!-----------------------------------------------------------------------
!
! first count terms to check available array sizes
! KE terms
!
      k = 0
      do f = 1,ndof
         if (f .eq. feb) cycle
         k = k+1
      enddo
!
! potential terms 
!
      if (nddstate .gt. 1) then
         do s=1,nddstate
            do s1=s,nddstate
               k=k+1
            enddo
         enddo
      else
         k=k+1
      endif
      minkoe=max(minkoe,k)
      if (minkoe .gt. maxkoe) return
!
! set up KE terms
!
      k = 0
      do f = 1,ndof
         if (f .eq. feb) cycle
         if (dofspf(f) .eq. 0) cycle   ! ignored DOFs
         k = k+1
         hamilton(f,k) = htmke+f
         hamilton(fkoe,k) = fkoe
         kact(k)   = 1
         ktwin(k)  = k
      enddo
!
! set up potential terms - one for each state operator. 
! Store under first non-frozen DOF.
!
      if (nddstate .gt. 1) then
         i=0
         do s=1,nddstate
            do s1=s,nddstate
               k=k+1
               i=i+1
               f1=spfdof(1,1)
               hamilton(f1,k) = htmpe+i
               hamilton(fkoe,k) = fkoe
               hamilton(feb,k) = htmfeb+i
               kact(k)   = 1
               ktwin(k)  = k
            enddo
         enddo
      else
         k=k+1
         f1=spfdof(1,1)
         hamilton(f1,k) = htmpe + 1
         hamilton(fkoe,k) = fkoe
         kact(k)   = 1
         ktwin(k)  = k
      endif
      ktot=k

!-----------------------------------------------------------------------
! expand muld operator (need to set up "grid"first)
!-----------------------------------------------------------------------
      if ((leb .and. nmode .gt. 2) .or. (.not. leb .and. nmode .gt. 1)) then
         grid = 0
         gf1=0
         do f=1,ndof
            if ((f .eq. ftime) .or. (f .eq. feb)) cycle 
            gf1=gf1+1
            grid(gf1)=f
         enddo
         call muldnum(muld,grid,mode)

! set ndef = 1 to force matchlab to process qchem muld term
         ndef=1
         call matchlab(loperr)
         ndef=0
      endif

!-----------------------------------------------------------------------
! fill in blanks in Hamiltonian with unit operators
!-----------------------------------------------------------------------
      do k=1,min(ktot,maxkoe)
         do f=1,nopdof
            if (hamilton(f,k) .eq. 0) then
               hamilton(f,k)=f
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! set other flags
!-----------------------------------------------------------------------
      do k=1,min(ktot,maxkoe)
         hamnum(k) = 1
      enddo

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      loperr=lwarn.or.loperr
      lwarn=.false.

      return
      end subroutine cvmcgop

!#######################################################################

      subroutine cddmctop(pass,loperr)

      implicit none

      integer(long), intent(in)       :: pass
      integer(long)                   :: m,f,h,k,s,s1,hopfreq,f1,hopfeb,l, &
                                         htmke,htmpe,htmfeb,i,hopqc,n,k1,&
                                         ktot1,j,i1,hoplin,htmlin,f2,p,poffs,&
                                         ioffs,joffs,nf,n1,n2
      integer(long), external         :: slen
      logical(kind=4), intent(inout)  :: loperr
      character(len=30)               :: string

      real(dop), dimension(npar)         :: rpar1
      character(len=c1), dimension(npar) :: apar1
      integer(long)                      :: gf1,mode,muld,hamidx
      integer(long), dimension(maxdim)   :: grid
      logical(kind=4), dimension(maxkoe) :: kignore
      integer(long), dimension(:), allocatable :: htmpe2,pesorder
      integer(long), dimension(:,:), allocatable :: hamnum1

! need to know how many parameters were read in before entering routine
      integer(long), save   :: npar0
      logical(kind=4), save :: initial = .true.

      routine='cddmctop'

      htmpe = 0
      htmlin = 0
      htmlin = 0
      hoplin = 0

!-----------------------------------------------------------------------
! Different orders depending on whether we use standard GAP or optimised
! potential based on GAP
!-----------------------------------------------------------------------
      if(allocated(htmpe2))deallocate(htmpe2)
      allocate(htmpe2(max(ddmoptord,ddmorder)))

!-----------------------------------------------------------------------
! have to start counter for parameters with no. read in from files
! Due to iterative operator set up this must reset.
!-----------------------------------------------------------------------
      if (initial) then
         npar0=npar
         initial = .false.
      endif
      npar = npar0

!-----------------------------------------------------------------------
! allocate no. of DOFs in operator, and set up space for koeffs,
! electronic basis etc.
!-----------------------------------------------------------------------
      do m=1,min(nmode,maxdim)
         laddmode(m,1) = .true.
      enddo
      call genopdof

!-----------------------------------------------------------------------
! assign no. of Hamiltonians
! assign minimum no. of factors needed in operators
! Need an auxiliary Hamiltonian if using OPTGAP; it defines the 
! underlying GAP used to give the global representation.
!-----------------------------------------------------------------------
      if(lddmopt .or. lddgbtendec)then
         hamdim = 2
         allocate(pesorder(hamdim))
         allocate(hamnum1(2,hamdim))
         pesorder(1) = ddmoptord
         pesorder(2) = ddmorder
      else
         hamdim = 1
         allocate(pesorder(hamdim))
         allocate(hamnum1(1,hamdim))
         pesorder(1) = ddmorder
      endif
      hamnum1 = 0
!      minfac = max(minfac,2)
!      minfac = max(minfac,nmode)

! Which DOFs are used?
      nf = 0
      grid = 0
      do f = 1,ndof
         if(basis(f).eq.19 .or. dofspf(f).eq.0 .or. f.eq.feb)cycle
         nf = nf + 1
         grid(nf) = f
      enddo

      if(ddmorder.eq.0)then
         minfac = max(minfac,nf,ddmoptord,2)
      else
         minfac = max(minfac,ddmorder,ddmoptord,2)
      endif

!
!*** Set up labels
!-----------------------------------------------------------------------
! if hoplab needs to be initialised then:
! place unit operator at beginning of list for each degree of freedom
! place +1, -1 at beginning of list for coefficients
!-----------------------------------------------------------------------
      hoplab(1)='1'
      hoplab(2)='-1'
      hoplab(3)='I'
      hoplab(4)='KEdvr'
      hoplab(5)='KEfft'
      hopdim=5
!
! set up mode frequencies as parameters if nmode coordinates
!
      hopfreq=0
      if (ddcoo .eq. 1) then
         do n=1,npar
            apar1(n)=apar(n)
            rpar1(n)=rpar(n)
         enddo
         if (allocated(apar)) deallocate(apar)
         if (allocated(rpar)) deallocate(rpar)
         allocate(apar(npar+ndof))
         allocate(rpar(npar+ndof))
         do n=1,npar
            apar(n)=apar1(n)
            rpar(n)=rpar1(n)
         enddo

         hopfreq=hopdim
         do f=1,ndof
            if (f .eq. feb) cycle

            write(string,'(a6,i5)')  'freq0_',f
            call rmblnk(string)
            npar = npar+1
            apar(npar) = string(1:c1)
            rpar(npar) = abs(hoppar(1,f))

            hopdim = hopdim+1
            hoplab(hopdim) = string
         enddo
      endif

!
! set up Qchem operators. hopqc is pointer to start of these.
!
      hopqc=hopdim
      if (nddstate .gt. 1) then
         do n = 1,nf
            f = grid(n)

            do s=1,nddstate
               write(string,'(a5,i5,a1,i5)')  'qchem',s,'_',f
               call rmblnk(string)
               hopdim = hopdim+1
               hoplab(hopdim) = string
               do s1=s+1,nddstate
                  write(string,'(a5,i5,a1,i5,a1,i5)')  'qchem',s,'&',s1,'_',f
                  call rmblnk(string)
                  hopdim = hopdim+1
                  hoplab(hopdim) = string
               enddo
            enddo
         enddo
      else
         do n = 1,nf
            f = grid(n)
 
            write(string,'(a6,i5)')  'qchem_',f
            call rmblnk(string)
            hopdim = hopdim+1
            hoplab(hopdim) = string
         enddo
      endif

!
! Set up linear terms if using derivatives with GAP
!
      if(lgapderiv)then
         hoplin = hopdim
         do n = 1,nf
            f = grid(n)

            write(string,'(a6,i5,a2)') 'qchem_',f,'*q'
            call rmblnk(string)
            hopdim=hopdim+1
            hoplab(hopdim) = string
         enddo
      endif

!
! set up electronic operators if single-set. hopfeb is a pointer to the 
! start of the electronic state operators
!
      if (nddstate .gt. 1) then
         hopfeb=hopdim
         do s=1,nddstate
            do s1=s,nddstate
               write(string,'(a1,i5,a1,i5)')  'S',s,'&',s1
               call rmblnk(string)
               hopdim = hopdim+1
               hoplab(hopdim) = string
            enddo
         enddo
      else
         hopfeb=0
      endif

!
!*** Now set up operators
!
      htmdim=0
!
! set up unit operators
!
      do f=1,nopdof
         htmdim=htmdim+1
         h=min(htmdim,maxhtm)
         hterm(1,h)=1
         htmfac(h)=1
         htmdof(h)=f
         htmadof(h)=f
         htmmode(h)=0
         htmmuld(h)=0
         htmtype(h)=1
         htmsym(h)=1
      enddo

!
! set up KE terms
! For Jacobi Cartesians and Cartesians KEdvr/fft
! For Normal Modes freq(f)*KEdvr/fft
! NB 1/mass is included in routine kedvrterm later on
!
      k = 0
      htmke=htmdim
      do f = 1,ndof
         if (f .eq. feb) cycle

         htmdim=htmdim+1
         h=min(htmdim,maxhtm)
         htmfac(h)=1
         htmdof(h)=f
         htmadof(h)=f
         htmmode(h)=0
         htmmuld(h)=0

         if(basis(f) .eq. 4)then
            hterm(1,h)=5
         else
            hterm(1,h)=4
         endif

         if (ddcoo .eq. 1) then
            htmfac(h)=2
            n=min(2,maxfac)
            hterm(n,h)=hopfreq+f
         endif
      enddo

!
! set up potential terms. 
!
      do hamidx = 1,hamdim
         if(pesorder(hamidx).eq.0)then
            htmpe=htmdim
            i=hopqc
            do n = 1,nf
               f = grid(n)

               do s=1,nddstate
                  do s1=s,nddstate
                     htmdim = htmdim+1
                     h=min(htmdim,maxhtm)
                     htmfac(h)=1
                     htmdof(h)=f
                     htmmode(h)=0
                     htmadof(h)=f
                     htmmuld(h)=0
                     htmsym(h)=0
                     i=i+1
                     hterm(1,h)=i
                     if(lddsoc)then
                        htmtype(h)=204
                     else
                        htmtype(h)=202
                     endif
                  enddo
               enddo
            enddo
            if(lgapderiv)then
               htmlin=htmdim
               i=hoplin
               do n = 1,nf
                  f = grid(n)
                  
                  do s=1,nddstate
                     do s1=s,nddstate
                        htmdim = htmdim+1
                        h=min(htmdim,maxhtm)
                        htmfac(h)=1
                        htmdof(h)=f
                        htmmode(h)=0
                        htmadof(h)=f
                        htmmuld(h)=0
                        htmsym(h)=0
                        i=i+1
                        hterm(1,h)=i
                     enddo
                  enddo
               enddo
            endif
         else
            htmpe2(1)=htmdim
            i = hopqc
            do n = 1,nf
               f = grid(n)
               
               do s=1,nddstate
                  do s1=s,nddstate
                     htmdim = htmdim+1
                     h=min(htmdim,maxhtm)
                     htmfac(h)=1
                     htmdof(h)=f
                     htmmode(h)=0
                     htmadof(h)=f
                     htmmuld(h)=0
                     htmsym(h)=0
                     i=i+1
                     hterm(1,h)=i
                     if(lddsoc)then
                        htmtype(h)=204
                     else
                        htmtype(h)=202
                     endif
!                  htmform(h)=0
                  enddo
               enddo
            enddo
         endif
      enddo

!
! set up electronic terms. 
!
      if (nddstate .gt. 1) then
         htmfeb=htmdim
         i=hopfeb
         do s=1,nddstate
            do s1=s,nddstate
               htmdim=htmdim+1
               h=min(htmdim,maxhtm)
               htmfac(h)=1
               htmdof(h)=feb
               htmadof(h)=feb
               htmmode(h)=0
               htmmuld(h)=0
               i=i+1
               hterm(1,h)=i
            enddo
         enddo
      else
         htmfeb=0
      endif

!-----------------------------------------------------------------------
!*** Finally set up Hamiltonian
!-----------------------------------------------------------------------
!
! first count terms to check available array sizes
! KE terms
!
      k = 0
      do f = 1,ndof
         if (f .eq. feb) cycle
         k = k+1
      enddo
!
! potential terms 

      do hamidx = 1,hamdim
         hamnum1(1,hamidx) = k+1
         if(pesorder(hamidx).eq.0)then
            do s=1,nddstate
               do s1=s,nddstate
                  k=k+1
                  if(lgapderiv)then
                     do n = 1,nf
                        k = k+1
                     enddo
                  endif
               enddo
            enddo
         else
            do n = 1,nf
               do s = 1,nddstate
                  do s1 = s,nddstate
                     k = k+1
                  enddo
               enddo
               if(pesorder(hamidx).gt.1)then
                  do n1 = n+1,nf

                     do s = 1,nddstate
                        do s1 = s,nddstate
                           k = k+1
                        enddo
                     enddo
! test ddm3
                     if(pesorder(hamidx).gt.2)then
                        do n2 = n1+1,nf

                           do s = 1,nddstate
                              do s1 = s,nddstate
                                 k = k+1
                              enddo
                           enddo
                        enddo
                     endif
! test ddm3
                  enddo
               endif
            enddo
         endif

         if(lddmopt .or. lddgbtendec)hamnum1(2,hamidx) = k
      enddo

      minkoe=max(minkoe,k)
      if (minkoe .gt. maxkoe) return

!
! set up KE terms
!
      k = 0
      do f = 1,ndof
         if (f .eq. feb) cycle
         k = k+1
         hamilton(f,k) = htmke+f
         hamilton(fkoe,k) = fkoe
         kact(k)   = 1
         ktwin(k)  = k
      enddo
      ktot = k

!
! set up potential terms - one for each state operator. 
! Store under first non-frozen DOF.
!
      do hamidx = 1,hamdim
         if(pesorder(hamidx).eq.0)then

            i=0
            do n = 1,nf
               f = grid(n)

               k1 = k
               if (nddstate .gt. 1) then
                  j=0
                  do s=1,nddstate
                     do s1=s,nddstate
                        k1=k1+1
                        i=i+1
                        j=j+1
                        hamilton(f,k1) = htmpe+i
                        hamilton(fkoe,k1) = fkoe
                        hamilton(feb,k1) = htmfeb+j
                        kact(k1)   = 1
                        ktwin(k1)  = k1
                     enddo
                  enddo
               else
                  k1 = k1 + 1
                  i = i + 1
                  hamilton(f,k1) = htmpe + i
                  hamilton(fkoe,k1) = fkoe
                  kact(k1)   = 1
                  ktwin(k1)  = k1
               endif
            enddo
            k=k1
            ktot=k
            
            if(lgapderiv)then
               i = 0
               k1 = k
               do n = 1,nf
                  f = grid(n)
                  if (nddstate .gt. 1) then
                     j=0
                     do s=1,nddstate
                        do s1=s,nddstate
                           k1=k1+1
                           i=i+1
                           j=j+1
                           hamilton(f,k1) = htmpe+i
                           hamilton(fkoe,k1) = fkoe
                           hamilton(feb,k1) = htmfeb+j
                           kact(k1)   = 1
                           ktwin(k1)  = k1
                        enddo
                     enddo
                  else
                     k1 = k1 + 1
                     i = i + 1
                     hamilton(f,k1) = htmlin + i
                     hamilton(fkoe,k1) = fkoe
                     kact(k1)   = 1
                     ktwin(k1)  = k1
                     j=0
                     do n1 = 1,nf
                        f1 = grid(n1)
                        j = j + 1
                        if(f1.eq.f)cycle
                        hamilton(f1,k1) = htmpe+j
                     enddo
                  endif
               enddo
               k=k1
               ktot=k
            endif
            
         else
            i=0
            k1=k
            do n = 1,nf
               f = grid(n)

               if(nddstate.gt.1)then
                  j=0
                  do s=1,nddstate
                     do s1=s,nddstate
                        k1=k1+1
                        i=i+1
                        j=j+1
                        hamilton(f,k1) = htmpe2(1)+i
                        hamilton(fkoe,k1) = fkoe
                        hamilton(feb,k1) = htmfeb+j
                        kact(k1)   = 1
                        ktwin(k1)  = k1                    
                     enddo
                  enddo
               else
                  k1 = k1+1
                  i=i+1
                  hamilton(f,k1) = htmpe2(1)+i
                  kact(k1)   = 1
                  ktwin(k1)  = k1
               endif
            enddo
            k=k1
            ktot=k
            if(pesorder(hamidx).gt.1)then
               do n = 1,nf
                  f = grid(n)

                  if(nddstate.gt.1)then
                     j = n*(nddstate+1)*nddstate/2
                     ioffs = (n-1)*(nddstate+1)*nddstate/2
                     do n1 = n+1,nf
                        f1 = grid(n1)

                        i=0
                        do s=1,nddstate
                           do s1=s,nddstate
                              k1 = k
                              k1 = k1+1
                              i=i+1
                              j=j+1
                              hamilton(f,k1) = htmpe2(1)+ioffs+i
                              hamilton(f1,k1) = htmpe2(1)+j
                              hamilton(fkoe,k1) = fkoe
                              hamilton(feb,k1) = htmfeb+i
                              kact(k1)   = 1
                              ktwin(k1)  = k1
                              k=k1
                           enddo
                        enddo
!                        i1=i
                     enddo
                  else
!                     i=i+1
!                     i=n
                     j=n
                     do n1 = n+1,nf
                        f1 = grid(n1)

                        k1 = k
                        k1 = k1+1
                        j=j+1
                        hamilton(f,k1) = htmpe2(1)+n
                        hamilton(f1,k1) = htmpe2(1)+j
                        hamilton(fkoe,k1) = fkoe
                        kact(k1)   = 1
                        ktwin(k1)  = k1
                        k=k1
                     enddo
                  endif
               enddo
               k=k1
               ktot = k
            endif
! test ddm3
            if(pesorder(hamidx).gt.2)then
               ioffs=0
               joffs=0
               poffs=0
               i=0
               j=0
               do n = 1,nf
                  f = grid(n)

                  if(nddstate.gt.1)then
                     ioffs = (n-1)*(nddstate+1)*nddstate/2

                     do n1 = n+1,nf
                        f1 = grid(n1)

                        joffs = (n1-1)*(nddstate+1)*nddstate/2

                        do n2 = n1+1,nf
                           f2 = grid(n2)

                           poffs = (n2-1)*(nddstate+1)*nddstate/2

                           i=ioffs
                           j=joffs
                           p=poffs
                           l=0                    
                           do s =1,nddstate
                              do s1=s,nddstate
                                 k1 = k
                                 k1 = k1+1
                                 i=i+1
                                 j=j+1
                                 p=p+1
                                 l=l+1
                                 hamilton(f,k1) = htmpe2(1)+i
                                 hamilton(f1,k1) = htmpe2(1)+j
                                 hamilton(f2,k1) = htmpe2(1)+p
                                 hamilton(fkoe,k1) = fkoe
                                 hamilton(feb,k1) = htmfeb+l
                                 kact(k1)   = 1
                                 ktwin(k1)  = k1
                                 k=k1
                              enddo
                           enddo
                        enddo
                        i1=i
                     enddo
!                     ioffs=ioffs+(nddstate+1)*nddstate/2

!                  if(nddstate.gt.1)then
!!                  j=j+(nddstate+1)*nddstate/2
!                     joffs=joffs+(nddstate+1)*nddstate/2
!                     j = joffs
!!                  i1=i
!                     i1=ioffs
!                     do n1 = n+1,nf
!                        f1 = grid(n1)
!                        i=i1
!                        poffs = joffs+(nddstate+1)*nddstate/2 
!                        p=poffs
!                        do n2 = n1+1,nf
!                           f2 = grid(n2)
!                           l=0                    
!                           do s =1,nddstate
!                              do s1=s,nddstate
!                                 k1 = k
!                                 k1 = k1+1
!                                 i=i+1
!                                 j=j+1
!                                 p=p+1
!                                 l=l+1
!                                 hamilton(f,k1) = htmpe2(1)+i
!                                 hamilton(f1,k1) = htmpe2(1)+j
!                                 hamilton(f2,k1) = htmpe2(1)+p
!                                 hamilton(fkoe,k1) = fkoe
!                                 hamilton(feb,k1) = htmfeb+l
!                                 kact(k1)   = 1
!                                 ktwin(k1)  = k1
!                                 k=k1
!                              enddo
!                           enddo
!                        enddo
!                        i1=i
!                     enddo
!                     ioffs=ioffs+(nddstate+1)*nddstate/2
                  else
                     i=i+1
                     j=i
                     do n1 = n+1,nf
                        f1 = grid(n1)

                        j=j+1
                        l = j
                        do n2 = n1+1,nf
                           f2 = grid(n2)

                           l = l + 1
                           k1 = k
                           k1 = k1+1
                           hamilton(f,k1) = htmpe2(1)+i
                           hamilton(f1,k1) = htmpe2(1)+j
                           hamilton(f2,k1) = htmpe2(1)+l
                           hamilton(fkoe,k1) = fkoe
                           kact(k1)   = 1
                           ktwin(k1)  = k1
                           k=k1
                        enddo
                     enddo
                  endif
               enddo
               k=k1
               ktot = k
            endif
! test ddm3
         endif        
      enddo

!-----------------------------------------------------------------------
! expand muld operator (need to set up "grid" first)
!-----------------------------------------------------------------------
      grid = 0
      gf1=0
      do f=1,ndof
         if ((f .eq. ftime) .or. (f .eq. feb)) cycle 
         gf1=gf1+1
         grid(gf1)=f
      enddo
      call muldnum(muld,grid,mode)

! set ndef = 1 to force matchlab to process qchem muld term
!      ndef=1
!      call matchlab(loperr)
!      ndef=0

!-----------------------------------------------------------------------
! fill in blanks in Hamiltonian with unit operators
!-----------------------------------------------------------------------
      do k=1,min(ktot,maxkoe)
         do f=1,nopdof
            if (hamilton(f,k) .eq. 0) then
               hamilton(f,k)=f
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! set other flags
!-----------------------------------------------------------------------
      do k=1,min(ktot,maxkoe)
         hamnum(k) = 1
      enddo

      if(lddmopt .or. lddgbtendec)then
         do hamidx = 1,hamdim
            do k = hamnum1(1,hamidx),hamnum1(2,hamidx)
               hamnum(k) = hamidx
            enddo
         enddo
      endif

      if(pass.eq.2) then
         nodiag(1:hamdim) = .false.
         if(lddmopt .or. lddgbtendec)hamlab(2) = 'GPR Ref.'
      endif

!-----------------------------------------------------------------------
! Delete any operators involving inactive modes.
!-----------------------------------------------------------------------
      kignore = .false.
      do k = 1,min(ktot,maxkoe)
         do f = 1,ndof
            if(dofspf(f).ne.0)cycle
            if(f.eq.feb)cycle
            h=hamilton(f,k)
            if(h.ne.f)kignore(k)=.true.
         enddo
      enddo
      ktot1=ktot
      k1=1
      do k=1,ktot1
         if (kignore(k)) then
            call mvkend(k1)
            ktot=ktot-1
          else
            k1=k1+1
         endif
      enddo

!-----------------------------------------------------------------------
! Turn on the use of complex integrators when using SOCs.
!-----------------------------------------------------------------------
      lcap = lddsoc

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      loperr=lwarn.or.loperr
      lwarn=.false.

      if(allocated(htmpe2))deallocate(htmpe2)
      if(allocated(pesorder))deallocate(pesorder)

      return
      end subroutine cddmctop

!#######################################################################

      subroutine ddoper

      implicit none

      integer(long)       :: n,h,hl,natmold,idbrd,ns

!-----------------------------------------------------------------------
! see if direct dynamics calculation
!-----------------------------------------------------------------------
      do h=1,htmdim
         if (htmform(h) .eq. 1) cycle

         hl=hterm(1,h)
         if (hl .eq. 0) cycle
         if (hopfile(hl) .eq. 205) ldd=.true.
      enddo

      if (.not. (ldd .or. lgwptrj)) return

!-----------------------------------------------------------------------
! count no. of DOFs in system (exclude electronic basis etc.)
!   NB nspfdof(1) contains no. of dofs in MCTDH
!-----------------------------------------------------------------------
      if (.not. (lgwp .or. ltraj)) then
         routine = 'DDoper (eindirdynmod.f90)'
         message = 'Direct dynamics calculation needs vMCG or Trajectories'
         call errormsg
      endif

      natmold = natm
      ndofdd = ndof
      if (feb .gt. 0) ndofdd = ndof-1
      if (ndofddpes .eq. 0) then
         if (ddcoo .eq. 0) then
            ndofddpes = ndofdd+3
         else if (ddcoo .eq. 1) then
            if (ndofdd .eq. 1) then
               ndofddpes = 6 !diatomics
            else
               ndofddpes = ndofdd+ntr
            endif
         else if (ddcoo .eq. 2) then
            ndofddpes = ndofdd
         endif
      endif
      natm=max(ndofddpes/3,1)

!-----------------------------------------------------------------------
! if natm defined previously (e.g. in einingeom) check same no. is
! obtained
!-----------------------------------------------------------------------
      if (natmold .gt. 0 .and. natm .ne. natmold) then
         routine = 'DDoper (eindirdynmod.f90)'
         write(message,'(a,2i5)')'No. Atoms in DD calculation has changed!?',&
              natmold,natm
         call errormsg
      endif

      if (.not. ldd) return

!-----------------------------------------------------------------------
! Get real size of derivative coupling matrix
!-----------------------------------------------------------------------
      if (ldiabdb.and.lgausspot)then
         dercpdim = sum(dercpblk(1:nsmult))
      endif

!-----------------------------------------------------------------------
! For Procrustes get no. of Configs in CI.
! nconf(ns,1): no. of alpha el configs in multiplicity ns
! nconf(ns,2): no. of beta el configs in multiplicity ns
! nciconf(ns) : no. of configs for multiplicity ns
! maxcivec: max no. of configs for all multiplicities
!-----------------------------------------------------------------------
      if (dddiab .eq. 6) then
         call get_nconfig
         lcivec = .true.
         maxcivec=1
         do ns=1,nsmult
            nciconf(ns) = nconf(ns,1)*nconf(ns,2)
            maxcivec=max(maxcivec,nciconf(ns))
         enddo
      else
         maxcivec=1
      endif

!-----------------------------------------------------------------------
! open DB files
! No need to load DB into memory unless rediabatisation 
!-----------------------------------------------------------------------
      if (lrediab) then
         idbrd = 2
      else
         idbrd = 0
      endif

      call preparedb(idbrd,0)

      if ((dbpntgp0 .ne. dbpntgp) .and. .not. lrediab) then
         routine = 'Ddoper (eindirdyn.F90)'
         write(message(1:80),'(2a)') &
            'Point group specified in input different from DB.', &
            ' Taking value from DB.'
         write(message(81:150),'(a)') &
            'To change point group use rediabatisation'
         call warnmsg1
      endif

!-----------------------------------------------------------------------
! If atoms have not been specified in einingeom, specify from refdb
!-----------------------------------------------------------------------
      if (natmold .eq. 0) then
         do n=1,natm
            xatnam(n) = atnam(n)
            xmass(n) = mref(n)
         enddo
      endif


      return
      end subroutine ddoper

!#######################################################################

      subroutine ddmctoper

      implicit none

      integer(long)       :: n,h,hl,natmold,f,idbrd

!-----------------------------------------------------------------------
! see if direct dynamics calculation
!-----------------------------------------------------------------------
      do h=1,htmdim
         if (htmform(h) .eq. 1) cycle

         hl=hterm(1,h)
         if (hl .eq. 0) cycle
         if (hopfile(hl).eq.208 .or. hopfile(hl).eq.209) lddmctdh=.true.
      enddo

      if (.not. lddmctdh) return

!-----------------------------------------------------------------------
! count no. of DOFs in system (exclude electronic basis etc.)
!   NB nspfdof(1) contains no. of dofs in MCTDH
!-----------------------------------------------------------------------
      if (.not. lddmctdh) then
         routine = 'DDmctoper (eindirdynmod.f90)'
         message = 'Direct dynamics MCTDH calculation needs QChem'
         call errormsg
      endif

      natmold = natm
      ndofdd = ndof
      if (feb .gt. 0) ndofdd = ndof-1
      if (ndofddpes .eq. 0) then
         if (ddcoo .eq. 0) then
            ndofddpes = ndofdd+3
         else if (ddcoo .eq. 1) then
            if (ndofdd .eq. 1) then
               ndofddpes = 6 !diatomics
            else
               ndofddpes = ndofdd+6
            endif
         else if (ddcoo .eq. 2) then
            ndofddpes = ndofdd
         endif
      endif
      natm=max(ndofddpes/3,1)

!-----------------------------------------------------------------------
! if natm defined previously (e.g. in einingeom) check same no. is 
! obtained
!-----------------------------------------------------------------------
      if (natmold .gt. 0 .and. natm .ne. natmold) then
         routine = 'DDmctoper (eindirdynmod.f90)'
         write(message,'(a,2i5)')'No. Atoms in DD calculation has changed!?',&
              natmold,natm
         call errormsg
      endif

      if (.not. lddmctdh) return

!-----------------------------------------------------------------------
! get real size of derivative coupling matrix
!-----------------------------------------------------------------------
      if ((ldd .or. lgwptrj .or. lddmctdh .or. lddtrans) .and. ldiabdb)then
         dercpdim = sum(dercpblk(1:nsmult))
      endif

!-----------------------------------------------------------------------
! open DB files and check for presence of ref.db
!-----------------------------------------------------------------------
      if (ldb2gapot) then
         idbrd = 4
      else if (lrediab) then
         idbrd = 2
      else if (ldbsave) then
         idbrd = 1
      else
         idbrd = 0
      endif
      call preparedb(idbrd,0)

!-----------------------------------------------------------------------
! If atoms have not been specified in einingeom, specify from refdb
!-----------------------------------------------------------------------
      if (natmold .eq. 0) then
         do n=1,natm
            xatnam(n) = atnam(n)
            xmass(n) = mref(n)
         enddo
      endif

!-----------------------------------------------------------------------
! If a scaled Gaussian kernel has been specified, set up the scaling
! coefficients for the necessary modes.
! Comparing the second derivatives, at 0, of a harmonic potential,
! h(x)=0.5*w*x**2 and that of a Morse curve, m(x)=D(1-exp(-ax))**2
! we see that for them to be equal 2Da**2=w i.e. a=sqrt(0.5*w/D). The 
! required coefficient is 'a', which we get by using a representative 
! D=0.140739 (this gives an 'a' which is the average of that needed for
! D's of 200kjmol-1 and 900kjmol-1; the range of typical bond energies).
!-----------------------------------------------------------------------
      do f = 1,ndof
         if(gapker(f).eq.3)&
              gapkerscl(f) = sqrt(0.5_dop*hoppar(1,f)/0.140739_dop)
      enddo

      return
      end subroutine ddmctoper

      end module eindirdynmod

