! New version (Benjamin, December 2008)
!
! contains: einoper,findlabels,predefined
!
!#######################################################################

      module einopermod

use decimal, only: long,dop
use global
use keyunitsmod
use logdat
use maxv
use dirdyn
use gwptrj
use dvrdatmod
use griddatmod
use operdef
use channels 
use runopermod
use paths
use gapmod

use eindirdynmod
use einingeomod, only: einingeo,einddmct
use heingabemod,  only: heingabe
use heintermmod,  only: hintgen
use heinutil, only: inc_rdfiles
use heinsort, only: sorteb
use zeigopermod, only: zeigrdf
use asshop, only: assignhlab

use opermod_interface

      implicit none
      private
      public :: inoper,einoper,predefined
      contains

!***********************************************************************

      subroutine inoper

      implicit none

      integer(long)                              :: ic,iz,ierr,jlbl
      integer(long)                              :: slen
      logical(kind=4)                            :: linperr,lopen,check
      character(len=c5)                          :: filename

      opname   = '     '
      oplaenge = 0

!-----------------------------------------------------------------------
! find out where Hamiltonian is stored and name of iop / ipar files
!
! If there is an operator-section read settings
!-----------------------------------------------------------------------
      inquire(iin,opened=lopen)
      if (.not. lopen) then
         filename=inputfile(1:inplaenge)
         open(iin,file=filename,status='old',form='formatted',err=900)
      endif

      rewind(iin)
      linperr=.false.
      iz=0
      readiin: do
         iz=iz+1
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .gt. 0) then
            routine = 'Inoper, operator-section'
            message = inptit
            call errormsg
         endif

         if (keyword(1) .eq. 'operator-section') then
            call einop(linperr,iz)
! if opname not defined in operator-section carry on looking
            if (oplaenge .eq. 0) cycle
            exit
         else if (keyword(1) .eq. 'hamiltonian-section') then
            iop=iin
            exit
         else if (keyword(1) .eq. 'initial-geometry-section') then
            iop=iin
            exit
         else if (keyword(1) .eq. 'dd-gb-section') then
            iop=iin
            exit
         else if (keyword(1) .eq. 'end-input') then
            exit
         endif
      enddo readiin

!-----------------------------------------------------------------------
! get spline fit paths and no. of spline fit operator names
!-----------------------------------------------------------------------
      call getsplnames(1)

!-----------------------------------------------------------------------
! open operator text file
! if iop=iin, input taken from input file
! else, first look in startup directory, then look in oppath.
!
! Logic had been changed in 100.262 so oppath searched first for .op
! file. However, local file should have precedence so logic reverted.
!-----------------------------------------------------------------------
      if (iop .ne. iin) then
         jlbl=0
         check=.false.

         filop=opname(1:oplaenge)//'.op'
         jlbl=slen(filop)
         call abspath(filop,jlbl)
         inquire(file=filop,exist=check)
         if (check) call getpath(filop,oppath,opplaenge)

         if (.not. check) then
            filop=oppath(1:opplaenge)//'/'//opname(1:oplaenge)//'.op'
            inquire(file=filop,exist=check)
            if (check) jlbl=slen(filop)
         endif

         if (.not. check) then
            routine='Heingabe'
            message='Op file not found: '//opname(1:oplaenge)//'.op' &
                  //' neither in current dir nor in: '&
                  //oppath(1:slen(oppath))
            call errormsg
         endif

         write(ilog,'(/,a)') 'Opening operator file:'//filop(1:jlbl)
         open(unit=iop,iostat=ierr,file=filop(1:jlbl),status='old',&
              form='formatted')
         if (ierr.ne.0) then
            routine='Heingabe'
            message = 'Error opening operator : '//filop(1:jlbl)
            call errormsg
         endif

!-----------------------------------------------------------------------
! append .op file to input file in data directory
!-----------------------------------------------------------------------
         if (.not. liopt) call cpopinp

      endif

      return
  900 routine='Inoper'
      jlbl=slen(filename)
      write(message,'(2a)') 'Error opening file : ',filename(1:jlbl)
      call errormsg
 
      end subroutine inoper

!***********************************************************************

      subroutine einoper(pass)

      implicit none

      integer(long), intent(inout)               :: pass
      integer(long)                              :: ic,iz,ierr
      integer(long)                              :: f,f1,ilbl,n,n1
      integer(long)                              :: k,h,nh,minfackoe
      logical(kind=4)                            :: linperr

      linperr=.false.
!-----------------------------------------------------------------------
! set defaults
!-----------------------------------------------------------------------
      call operdefaults

!-----------------------------------------------------------------------
! Check if parameter name is allowed.
!-----------------------------------------------------------------------
      do n=1,npar
         do n1=1,npre
            if (apar(n) .eq. prelabel(n1)) then
               routine='Einoper'
               message= &
                'A parameter-name must not coincide with a pre-defined' &
                //" operator label! See the User's Guide for a list" &
                //" of pre-defined labels. Label : "//apar(n)
               call errormsg
            endif
         enddo
         if(index(apar(n),':') .gt. 0) then
            routine='Einoper'
            message='A parameter-name must not contain a colon (:).' &
            //' Parameter = '//apar(n)
            call errormsg
         endif
      enddo

!-----------------------------------------------------------------------
! load any special parameters into appropriate variables
!-----------------------------------------------------------------------
      do n=1,npar
         if (apar(n)(1:5) .eq. 'mass_') then
            ilbl=index(apar(n),' ')-1
            f=0
            do f1=1,ndof
               if (modelabel(f1).eq.apar(n)(6:ilbl)) f=f1
            enddo
            if( f .gt. 0 ) then
               mass(f)=rpar(n)
            endif
         else if (apar(n) .eq. 'jtot') then
            jtot=rpar(n)
         else if (apar(n) .eq. 'jbf') then
            jbf=rpar(n)
         endif
      enddo

!-----------------------------------------------------------------------
! Read in spline names and operators
!-----------------------------------------------------------------------
      call getsplnames(2)

!-----------------------------------------------------------------------
! read direct dynamics info if present 
! read gwptrj info if present 
! look for DD trafo section specifying KEO -> PES coords
!-----------------------------------------------------------------------
      rewind(iin)
      iz=0
 430  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .gt. 0) then
         routine = 'Eingabe, direct-section'
         message = inptit
         call errormsg
      endif

      if (keyword(1) .eq. 'dirdyn-section' &
           .or. keyword(1) .eq. 'dirdyn-gb-section') then
         if (.not. (ldd .or. lddmctdh)) then
            routine='Einoper (einopermod.F90)'
            message = 'Dirdyn-section present but not a Direct-dynamics Calculation!'
            call errormsg
         endif
         call eindirdyn(pass,linperr,iz)
      else if (keyword(1) .eq. 'gwptrj-section') then
         if (.not. lgwptrj) then
            routine='Einoper (einopermod.F90)'
            message = 'GWPtrj-section present but not a trajctory GWP Calculation!'
            call errormsg
         endif
         call eingwptrj(linperr,iz)
      else if (keyword(1) .eq. 'ddtrans-section') then
         if (lddtrans .and. ddtrfile .ne. 'inp') then
            routine = 'Einoper, direct-section'
            message = 'DDTRANS-SECTION present and transfile specified'
            call errormsg
         endif
      else if (keyword(1) .eq. 'diabref-section') then
         lrefdbrd = .true.

      else if (keyword(1) .eq. 'end-input') then
         go to 440
      endif
      go to 430
  440 continue

!-----------------------------------------------------------------------
! read operator info if oper file designated
!
! if irunoper .eq. 1 then read operator-section and then H from .op file 
! unless:
! 1. if Hamiltonian-section read H from input file
! 2. if direct dynamics, generate operator if not specified
! 3. if pre-defined trajectories are used ignore
!-----------------------------------------------------------------------
! CHECK iin open
      rewind(iin)
      iz=0
      readiin: do

         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .gt. 0) then
            routine = 'Eingabe, operator-section'
            message = inptit
            call errormsg
         endif

         if (keyword(1) .eq. 'operator-section' .and. oplaenge .gt. 0) then
            call heingabe(pass,linperr)
            exit
         else if (keyword(1) .eq. 'hamiltonian-section') then
            iop=iin
            call heingabe(pass,linperr)
            exit
         else if (keyword(1) .eq. 'end-input') then
            if (ldd .or. lgwptrj .or. lddmctdh) then
               rewind(iin)
               iz=0

               if(.not.lddmctdh)then
                  do
                     call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                          maxkey)
! need initial-geometry input for automatic DD operator generation - read masses
                     if (keyword(1) .eq. 'initial-geometry-section') then
                        call einingeo(linperr,iz,2)
                        exit
                     else if (keyword(1) .eq. 'end-input') then
                        exit
                     endif
                  enddo

                  call cvmcgop(linperr)
               else
                  do
                     call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
                     if (keyword(1) .eq. 'dd-gb-section') then
                        call einddmct(linperr,iz,2)
                        exit
                     else if (keyword(1) .eq. 'end-input') then
                        exit
                     endif
                  enddo

                  call cddmctop(pass,linperr)

               endif

               rewind(iin)
               iz=0
425            continue
               call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
! add in any operators defined in input file
               if (keyword(1)(1:20) .eq. 'hamiltonian-section_') then
                  iop=iin
                  call heingabe(pass,linperr)
                  exit
               else if (keyword(1) .eq. 'end-input') then
                  exit
               else
                  go to 425
               endif

            else
               routine = 'Inoper'
               message = 'Operator not in .op file or .inp file'
               call errormsg
            endif
            exit

         endif

      enddo readiin

!-----------------------------------------------------------------------
! generate operators needed for run-time analysis
!-----------------------------------------------------------------------
      call hintgen

!-----------------------------------------------------------------------
! sort electronic states
!-----------------------------------------------------------------------
      if (leb .or. lmult) call sorteb(pass,linperr)

!-----------------------------------------------------------------------
! if second pass 
! 1. set up pointers and labels for operators to be read from files
! 2. search library for functions needed to set up operators
!-----------------------------------------------------------------------
      if (pass .eq. 2) then
         call zeigrdf
         call assignhlab(linperr)
      endif

!-----------------------------------------------------------------------
! if first pass set array dimensions
! maxkoe = minkoe+1 to allow space for swapping terms
! maxfac needs to have space to merge koe terms with operators
!-----------------------------------------------------------------------
      if (pass .eq. 1) then
         minfackoe=1
         do k=1,min(maxkoe,ktot)
            h=hamilton(fkoe,k)
            nh = min(maxfac,htmfac(h))
            if (h .gt. 0) minfackoe=max(minfackoe,htmfac(h))
         enddo
         minfac=minfac+minfackoe

         maxkoe=minkoe+maxsta*hamdim+1
         maxham=hamdim
         maxhop=max(hopdim,nhlabmax)
         maxhtm=htmdim
! +1 needed in gh_elements to search past end of factors
         maxfac=max(maxfac,minfac+1)
! +1 needed in maxrdf to search past last file
         maxrdf=nrdf+1
         maxdef=ndef
         maxhpar=max(nhparmax,9)
         maxmuld=32
         maxLMR=120
         maxmoco=max(nbasis,1)
      endif

      if (linperr) then
         routine = 'Einoper'
         message = "Error in input for Operator. See log file"
         call errormsg
      endif

!-----------------------------------------------------------------------
! check DD input. 
!-----------------------------------------------------------------------
      if (ldd .or. lddmctdh) then
         call chkdirdyn(linperr)
      endif

!-----------------------------------------------------------------------
! check GWPTRJ input. 
!-----------------------------------------------------------------------
      if (lgwptrj) then
         call chkgwptrj(linperr)
      endif

      if (linperr) then
         routine = 'Einoper'
         message = "Error in input for Operator generation. See log file"
         call errormsg
      endif

      return
      end subroutine einoper

!**********************************************************************
! 
!             MCTDH subroutine EINOPER.F
!
!  01/98 AJ
!
!***********************************************************************

      subroutine einop(loperr,iz)

      implicit none

      integer(long)                      :: i,iz0,ic,ierr,n,&
                                            ncutn0,ii,np
      integer(long), external            :: slen
      integer(long), intent(in)          :: iz
      logical(kind=4), intent(inout)     :: loperr

!-----------------------------------------------------------------------
! set initial values
! Note: In order that the variable 'name' is specified, 'einfile' must 
! have been called before 'einoper'. 
!-----------------------------------------------------------------------
      routine  = 'read OPERATOR-SECTION  (einoper)'
      iz0      = 0

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
 
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      if(iop.ne.iin)call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (keyword(i) .eq. 'end-operator-section') then
         goto 20

      else if( keyword(i) .eq. 'opname' )  then
         if (keyword(i+1).eq.'=') then
            opname = keyorig(i+2)
            oplaenge = lc(i+2)
            i=i+2
         else 
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif

      elseif (keyword(i).eq.'oppath') then
         if (keyword(i+1).eq.'=') then
            if (keyorig(i+2) .eq. "$QUANTICS_DIR/operators") then
               oppath = oppathdef
               opplaenge=index(oppathdef,' ')-1
            else
               oppath = keyorig(i+2)
               opplaenge = lc(i+2)
               call abspath(oppath,opplaenge)
            endif
            i=i+2
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif

      else if (keyword(i)(1:10) .eq. 'print-npot' .or. &
               keyword(i)(1:10) .eq. 'print_npot') then
         lwrnpot=.true.

      else if (keyword(i) .eq. 'logop_full') then
         loplogfull=.true.

      else if( keyword(i) .eq. 'closed'   )  then
         ldissin = .false.

      else if( keyword(i) .eq. 'open'   )  then
         ldissin = .true.

      else if( keyword(i) .eq. 'projection'   )  then
         ldproj = .true.

      else if( keyword(i) .eq. 'no_projection'   )  then
         ldproj = .false.

      else if (keyword(i) .eq. 'analytic_pes') then
         lanalpes=.true.

      else if (keyword(i) .eq. 'analytic_deriv') then
         lanalderiv=.true.

      else if (keyword(i) .eq. 'no_analytic_deriv') then
         lanalderiv=.false.

      else if (keyword(i) .eq. 'noanalder') then
         lanalderiv=.false.

      else if (keyword(i) .eq. 'cdvr') then
         lanalpes=.true.

      else if (keyword(i) .eq. 'tddvr') then
         lanalpes=.true.

      else if (keyword(i) .eq. 'optcontract') then
         loptcontract=.true.

      else if( keyword(1) .eq. 'alter-parameters' )  then
! read to end of alter-parameters - parameters read in einpara
         do
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            message = inptit
            call cntrl(keyword(1),lc(1),ierr)
            if( keyword(1) .eq. 'end-alter-parameters' )  exit
         enddo

      else if( keyword(i) .eq. 'parfile' )  then
         if (keyword(i+1) .eq. '=') then
            parfile=keyorig(i+2)
            parlaenge = lc(i+2)
            if (parfile(parlaenge:parlaenge) .eq. '/') then
               parfile(parlaenge:parlaenge)=' '
               parlaenge=parlaenge-1
            endif
            i=i+2
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif

      else if( keyword(1) .eq. 'alter-labels' )  then
! read to end of alter-labels - labels read in heingabe
         do
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            message = inptit
            call cntrl(keyword(1),lc(1),ierr)
            if( keyword(1) .eq. 'end-alter-labels' )  exit
         enddo

      elseif ((keyword(i).eq.'v') .or. (keyword(i).eq.'v_soft') .or. &
              (keyword(i).eq.'v_all') .or. (keyword(i).eq.'v_allsoft')) then
         if (keyword(i).eq.'v') lcutpes = .true.
         if (keyword(i).eq.'v_all') lcutpesall = .true.
         if (keyword(i).eq.'v_soft') lsoftcutpes = .true.
         if (keyword(i).eq.'v_allsoft') lsoftpesall = .true.

         if (keyword(i+1).eq.'='.or.keyword(i+1).eq.'<') then
            i=i+2
            read(keyword(i),*,iostat=ierr) pesmax
            if (ierr.ne.0) then
               message &
                    = 'A real number must be argument of "v". Not :'
               call wrline(i,iz,iz0)
            endif
            if (keyword(i+1).eq.',' ) then
               i=i+2
               call keyeunits(pesmax,keyword(i),ierr)
               if (ierr .eq. 1) then
                  message = 'no energy unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
         else if (keyword(i+1).eq.'>') then
            i=i+2
            read(keyword(i),*,iostat=ierr) pesmin
            if (ierr.ne.0) then
               message &
                    = 'A real number must be argument of "v". Not :'
               call wrline(i,iz,iz0)
            endif
            if (keyword(i+1).eq.',' ) then
               i=i+2
               call keyeunits(pesmin,keyword(i),ierr)
               if (ierr .eq. 1) then
                  message = 'no energy unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif

      else if (keyword(i).eq.'reduce-pf') then
         if (keyword(i+1) .ne. '=') then
            routine = 'einoper'
            message = 'Expecting "=" after "reduce-pf" not :'
            call wrline(i+1,iz,iz0)
         endif
         i=i+2
         read(keyword(i),*,iostat=ierr) np
         if (ierr .ne. 0 .or. np.le.0) then
            routine = 'einoper'
            message = 'The fist argument of "reduce-pf" must '//&
                  'be a positive integer (np=number of natpot). Not: '
            call wrline(i,iz,iz0)
            call stopnow
         endif
         if (np .gt. maxrdf) call inc_rdfiles
         do ii=1,maxdim
            i=i+1
            if(keyword(i) .eq. ',' ) i=i+1
            read(keyword(i),*,iostat=ierr,end=33) wpf(ii,np)
            if (ierr .ne. 0 .or. wpf(ii,np) .lt. 0.d0) then
               routine = 'einoper'
               message = 'The arguments (except first one) of '//&
                    '"reduce-pf" must be non-negative real numbers. '//&
                      'NB: The contraced mode should have weight 0.'
               call wrline(zeroi,iz,iz0)
            endif
         enddo
 33      continue

      else if (keyword(i).eq.'cutoff') then
         i=i+1
         if (keyword(i).eq.'=') then
            i=i+1
            read(keyword(i),*,iostat=ierr) cutoff
            if (ierr.ne.0) then
               message = &
               'A real number must be argument of "cutoff". Not :'
               call wrline(i,iz,iz0)
            endif
            i=i+1
            if (keyword(i) .eq. ',') then
               i=i+1
               call keyeunits(cutoff,keyword(i),ierr)
               if (ierr .eq. 1) then
                  message = 'no energy unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
         else
            message = 'No value for cutoff is given.'
            call wrline(zeroi,iz,iz0)
         endif

      else if (keyword(i).eq.'natpotcut') then
         i=i+1
         ncutn0=ncutn
         if (keyword(i)(1:1).eq.'{' .and. .not.lncutall) then
 11         i=i+1
            ncutn=ncutn+1
            if (ncutn .gt. maxrdf) call inc_rdfiles
            read(keyorig(i),*,iostat=ierr) ncutlabel(ncutn)
            if (ierr.ne.0) then
                message='Cannot read label for natpotcut'
                call wrline(i,iz,iz0)
            endif
            if (ncutlabel(ncutn).eq.'all') then
               lncutall=.true.
               i=i+1
               go to 12
            endif
            i=i+1
            if (keyword(i).eq.',') then
                go to 11
            elseif (keyword(i).ne.'}') then
                call wrline(i,iz,iz0)
            endif
         elseif (keyword(i).eq.'='.or.keyword(i).eq.'<') then
            lncutall=.true.
            i=i-1
         endif
 12      i=i+1
         if (keyword(i).eq.'='.or.keyword(i).eq.'<') then
            i=i+1
            read(keyword(i),*,iostat=ierr) natpotcut
            if (ierr.ne.0) then
               message = &
               'A real number must be argument of "natpotcut". Not :'
               call wrline(i,iz,iz0)
            endif
            i=i+1
            if (keyword(i) .eq. ',') then
               i=i+1
               call keyeunits(natpotcut,keyword(i),ierr)
               if (ierr .eq. 1) then
                  routine = 'einoper, line 235'
                  message = 'no energy unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
            do n=ncutn0+1,ncutn
               ncutrlabel(n)=natpotcut
            enddo
         else
            routine = 'einoper, line 244'
            message = 'No value for natpotcut is given.'
            call wrline(zeroi,iz,iz0)
         endif
         write(ilog,'(a,1pe10.3)') 'natpotcut =', natpotcut

!-----------------------------------------------------------------------
! Define highest order of moments used in GWP integrals
! gwpintorder is old keyword
!-----------------------------------------------------------------------
      else if (keyword(i) .eq. 'lhaorder' .or. &
               keyword(i) .eq. 'gwpintorder') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) lhaorder

         else
            message = 'No order specified with lhaorder keyword'
            call warnmsg

         endif

!-----------------------------------------------------------------------
! are any paths for spline files defined? 
!-----------------------------------------------------------------------
      else if (keyword(i)(1:10) .eq. 'splinepath') then
         if (keyword(i+1) .ne. '=') then
            message = 'No path specified with splinepath keyword'
            call warnmsg

         else
            nsplpaths = nsplpaths+1
            i=i+2

         endif

!-----------------------------------------------------------------------
! fast keyword has been moved to run-section (einrun) in 11.0 as it
! defines how the operators are used not set up and depends on dim(m,s)
!-----------------------------------------------------------------------
      else if (keyword(i).eq.'fast') then
          routine= 'Einoper'
          message= &
    'Keyword "fast" has been moved to the RUN-SECTION as "Natfast"'
          call errormsg

!-----------------------------------------------------------------------
! Zagreb SH code uses Cartesians. If a non-Cartesian (analytic model) 
! operator is used, a trafo is needed between coordinate systems.
! Read from input file.
!-----------------------------------------------------------------------
      else if (keyword(i) .eq. 'shztrans') then
         if (ldd) then
            routine= 'Einoper'
            message = 'Cannot specify a transformation for DD-ZSH calcs '
            call errormsg
         endif

         ltshtrans = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            if (keyword(i) .eq. 'read ') then
               tshtrfile = 'inp '
               if (keyword(i+1) .eq. ',') then
                  i=i+2
                  read(keyword(i),*,iostat=ierr) natmtsh
               endif
            else if (keyword(i) .eq. 'lvc ') then
               tshtrfile = 'lvc '
            else
               routine= 'Einoper'
               write(message,'(2a)') &
         'Unknown argument specified with shztrans keyword : ',keyword(i)(1:15)
               call wrline(i,iz,iz0)
            endif
         else
            routine= 'Einoper'
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif

!-----------------------------------------------------------------------
! following keywords are redundant (direct dynamics keywords moved to
! direct-dynamics-section after 90.55). kept here for backward 
! compatibility
!-----------------------------------------------------------------------
      else if (keyword(i) .eq. 'dd_adiab') then
         dddiab = 0

      else if (keyword(i) .eq. 'dd_diab') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            if (keyword(i) .eq. 'none') then
               dddiab = 0
            else if (keyword(i) .eq. 'regularized') then
               dddiab = 1
            else if (keyword(i) .eq. 'local') then
               dddiab = 2
            else if (keyword(i) .eq. 'global') then
               dddiab = 3
            endif
         else
            dddiab = 2
         endif

      else if (keyword(i) .eq. 'sacas') then
         lsacas = .true.

      else if (keyword(i) .eq. 'no_sacas') then
         lsacas = .false.

      else if (keyword(i) .eq. 'ddtrans') then
         lddtrans = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) ndofddpes
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif
!-----------------------------------------------------------------------
! end of direct dynamics redundant keywords
!-----------------------------------------------------------------------

      else

         message = ' unknown keyword : '
         call wrline(i,iz,iz0)
      end if

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         goto 15
      else
         goto 10
      endif

   20 continue  ! 'end-operator-section' found

!-----------------------------------------------------------------------
! Move path of opname to oppath if necessary
!-----------------------------------------------------------------------
      if(opplaenge.eq.0) then
         i=index(opname(2:oplaenge),'/')
         if(i.gt.0) then
            call getpath(opname,oppath,opplaenge)
            opname=opname(opplaenge+2:oplaenge)
            oplaenge=oplaenge-opplaenge-1
         endif
      endif

!-----------------------------------------------------------------------
! check and adjust keywords 
!-----------------------------------------------------------------------
      if (lgwp .or. ltraj) lanalpes = .true.
      if (lanalderiv .and. .not. (lgwp .or. ltraj)) lanalderiv = .false.

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      loperr=lwarn.or.loperr
      lwarn=.false.

      return
      end subroutine einop

!
! Set default values for variables used in OPER generation
!
!#######################################################################

      subroutine operdefaults

      implicit none

!-----------------------------------------------------------------------
! read by subroutine heingabe
!-----------------------------------------------------------------------
      hoplab0=' + '
      fkoe=0
      mkoe=0
      ftime=-1
      nopdof=0
      nopmode=0
      nremoved=0

      diag(:,:)=.true.
      kinop(:,:)=0
      hsym(:,:)=0
      hlsym(:,:)=0

      minkoe = 0
      minfac = 1
      hamilton(:,:)=0
      kact(:)=0
      ktwin(:)=0
      ldproj  = .true.
      hamLMR(:,:)=0
      hmlop(:) = 0
      kfirst(:,:)=0
      klha(:)=0
      iLMR=0
      h00=0

!-----------------------------------------------------------------------
! set first hamiltonian to be system H
! set that all operators should use diag flags
!-----------------------------------------------------------------------
      hamdim=1
      hamlab(:) = ' '
      hamlab(1)='system'

      nodiag(:)=.true.
      nodiag(1)=.false.
      ldirmuld = .false.

!-----------------------------------------------------------------------
! read by subroutine einoper
!-----------------------------------------------------------------------
      mass(:) = -999.0_dop
      htmshift(:) = 0

      hopdim=0
      hoplab(:) = ' '
      hopilab(:) = 0
      hopfile(:) = 0
      hoprpar(:,:) = 0.0_dop
      hopipar(:,:) = 0
      hopidef(:) = 0
      laddmode(:,:) = .false.

      htmdim = 0
      hterm(:,:) = 0
      htmform(:) = 0
      htmfac(:) = 0
      htmdof(:)=0
      htmmode(:)=0
      htmadof(:)=0
      htmndof(:)=0
      htmmuld(:)=0
      htmtype(:)=-1
      htmsym(:)=0
      htmderiv(:)=0
      hamnum(:)=0
      zham(:)=0
      fterm(:)=0

      mulddim=0
      nmulddof(:)=0
      muldsize(:)=0
      mulddof(:,:)=0
      mulddof2(:,:)=0

      nhoppar=0
      npesopt=0
      hoppar(:,:) = 0.0_dop
      hopidef(:) = 0
      pesopts(:) = ' '
      zpesopts(:,:)=0
      nrdf=0

      lnpot    = .false.
      parfile  = '   '
      parlaenge= 0
      iqconst(:)=0
      rqconst(:)=0.0_dop

      ndef = 0

! ..... variables need by FT and natpotcut
      mmodc(:)=0
      npotcut(:)=tiny
      ncutted(:)=0
      vmdim(:,:)=0
      mdim(:,:)=0
      nmdim(:,:)=0
      wpf(:,:)=0

!----------------------------------------------------------------------
! automatic activation of the CAP
!----------------------------------------------------------------------
      autocap(:)=0
      nstatecap(:)=0
      gpopcap(:)=10000.D0

      timecap(:) = -999.9e+9_dop  ! ACAP

!-----------------------------------------------------------------------
! direct dynamics control flags
!
! by default:
!     use a DB
!     SA-CAS for multi-state
!
! Flag defaults also set in include/dirdyn.f90
!-----------------------------------------------------------------------
      qcprog = ' '
      qcmethod = ' '
      fchkcmd = '  '
      ddextcmd = '  '
      qcsubform = 1
      lddreord = .false.
      lrde0 = .false.
      dduptim = 0.5*fs

      lddwrdb = .true.
      nbasis = 0
      nelectron = 1
      nocc = 1
      nclosed = 1
      lnbasis = .false.
      cicocut = 1.0e-8_dop
      ldecoup = .false.
      if(lddmctdh .or. lgausspot)then
         ddmorder = 0
         ddmoptord = 2
         lddmopt = .false.
         lddgbtendec = .false.
         lgapderiv = .false.
         lgapusesym = .false.
         lgapuseinwf = .false.
         lgapsobol = .true.
         lgapadiabwf = .false.
         lgapvarmax = .false.
         lgapcomp = .false.
         lmalon = .false.
         gapalpha = 0.5_dop
         gapker = 1
         gapkerscl = 0.0_dop
         gapsigma = 1.0e-8_dop
         gapkfac = 1.0_dop
         gapvartol = 1.0e-6_dop
         gapsd = 3.0_dop
         gapsample = 100
         gapupdate = 1
         gapuptim = 1.0*fs
         gapuptim0 = -99999.0_dop
         optgapalpha = 1.0_dop       
         ngapcomp = 1
      endif

!-----------------------------------------------------------------------
! for MLOps - reset variables
!-----------------------------------------------------------------------
      nmlop = 0
      
      end subroutine operdefaults

!######################################################################
!
!     SUBROUTINE PREDEFINED
!
! Loads array "prelabel" with pre-defined operator labels. 
! Pre-defined labels which contain a colon (:) or brackets ([]) 
! should not be listed here.
!----------------------------------------------------------------------

      subroutine predefined

      implicit none

      npre=84         ! Increase npre when adding new symbols.

      if (allocated(prelabel)) deallocate(prelabel)
      allocate(prelabel(npre)) 

      prelabel( 1)='1'
      prelabel( 2)='I'
      prelabel( 3)='q'
      prelabel( 4)='dq'
      prelabel( 5)='dqdvr'
      prelabel( 6)='dqfft'
      prelabel( 7)='KE'
      prelabel( 8)='KEdvr'
      prelabel( 9)='KEfft'
      prelabel(10)='p'
      prelabel(11)='sin'
      prelabel(12)='cos'
      prelabel(13)='tan'
      prelabel(14)='sinh'
      prelabel(15)='cosh'
      prelabel(16)='tanh'
      prelabel(17)='asin'
      prelabel(18)='acos'
      prelabel(19)='atan'
      prelabel(20)='exp'
      prelabel(21)='Exp'
      prelabel(22)='gauss'
      prelabel(23)='c_p'
      prelabel(24)='c_m'
      prelabel(25)='j^2'
      prelabel(26)='jz'
      prelabel(27)='j_p'
      prelabel(28)='j_m'
      prelabel(29)='jpm'
      prelabel(30)='sJp'
      prelabel(31)='sJm'
      prelabel(32)='sJpk'
      prelabel(33)='sJmk'
      prelabel(34)='Jp'
      prelabel(35)='Jm'
      prelabel(36)='Jx'
      prelabel(37)='Jy'
      prelabel(38)='dth1'
      prelabel(39)='dth2'
      prelabel(40)='qdq'
      prelabel(41)='cdq'
      prelabel(42)='sdq'
      prelabel(43)='udq'
      prelabel(44)='cjpm'
      prelabel(45)='absin'
      prelabel(46)='udq2'
      prelabel(47)='sdq2'
      prelabel(48)='uqdq'
      prelabel(49)='csdq'
      prelabel(50)='cdq2'
      prelabel(51)='crea'
      prelabel(52)='anni'
      prelabel(53)='Rf'
      prelabel(54)='dqR'
      prelabel(55)='dq2R'
      prelabel(56)='hKEh'
      prelabel(57)='hFRh'
      prelabel(58)='hdqh'
      prelabel(59)='hdqRh'
      prelabel(60)='Rfm'
      prelabel(60)='jp^2'
      prelabel(61)='jm^2'
      prelabel(62)='jpjm'
      prelabel(63)='jmjp'
      prelabel(64)='wig'
      prelabel(65)='j_ps'
      prelabel(66)='j_ms'
      prelabel(67)='jp^2s'
      prelabel(68)='jm^2s'
      prelabel(69)='jpms'
      prelabel(70)='jpjms'
      prelabel(71)='jmjps'
      prelabel(72)='jpjz'
      prelabel(73)='jmjz'
      prelabel(74)='jzjp'
      prelabel(75)='jzjm'
      prelabel(76)='jpjzs'
      prelabel(77)='jmjzs'
      prelabel(78)='jzjps'
      prelabel(79)='jzjms'
      prelabel(80)='AA'
      prelabel(81)='AD'
      prelabel(82)='AP'
      prelabel(83)='AM'
      prelabel(84)='NN'

      return
      end subroutine predefined

!#######################################################################
!
! search spline fit directories and get names
!
! NB the spline.ls file in the opdatadef/splines directory already
! exists and is left (users should not write to $QUANTICS_DIR
! directories). Other spline.ls files in local directories are deleted
! at end
!
! ipass: 1 - get no. splpaths and splnames to assign memory
!        2 - read in data
!
!#######################################################################

      subroutine getsplnames(ipass)

      implicit none

      integer(long), intent(in)    :: ipass
      integer(long)                :: ic,iz,ierr
      integer(long)                :: nspl,ilbl,jlbl,n,i
      character(len=c5)            :: filename,testname
      character(len=c2)            :: string
      character(len=240)           :: extcmd
      character(len=20)            :: cerr
      integer(long),external       :: slen

!-----------------------------------------------------------------------
! allocate memory for paths (this is redone in alloc_operdef during 
! full allocation)
!-----------------------------------------------------------------------
      if (allocated(splpaths)) deallocate(splpaths)
      cerr = 'splpaths '
      allocate(splpaths(nsplpaths),stat=ierr)
      if (ierr > 0) go to 999

!-----------------------------------------------------------------------
! default path name
!-----------------------------------------------------------------------
      nspl = 0
      nspl=nspl+1
      ilbl=slen(opdatadef)
      splpaths(nspl) = opdatadef(1:ilbl)//'/splines'

!-----------------------------------------------------------------------
! paths defined in file
!-----------------------------------------------------------------------
      if (nsplpaths .gt. 1) then
         rewind(iin)
         keyword(1) = ' '
         do while (keyword(1) .ne. 'operator-section')
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         enddo
         i=1
         do while (keyword(i) .ne. 'end-operator-section')
            do while (i .lt. ic)
               if (keyword(i) .eq. 'splinepath' .and. keyword (i+1) .eq. '=') &
                      then

                  nspl = nspl+1
                  splpaths(nspl) = keyorig(i+2)

                  ilbl=slen(splpaths(nspl))
                  call abspath(splpaths(nspl),ilbl)
! test existence - must be a different name from a file in the directory
! Hence random(ish) string
                  testname=splpaths(nspl)(1:ilbl)//'/tstXxYy_spl123'
                  open(idump,file=testname,status='unknown',iostat=ierr)
                  if (ierr .ne. 0) then
                     write(message,'(2a)') &
                         'Cannot find spline data directory :', &
                         splpaths(nspl)(1:ilbl)
                     call warnmsg
                     nspl = nspl - 1
                  else
                     close(idump,status='delete')
                  endif

                  i = i+2

               else
                  i = i+1
               endif
            enddo
            i=1
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         enddo
      endif
! update number of exisiting paths
      nsplpaths=nspl

!-----------------------------------------------------------------------
! count number of spline files
!-----------------------------------------------------------------------
      nsplnames=0
      do nspl=1,nsplpaths
         ilbl=slen(splpaths(nspl))
         jlbl=slen(strpid)

         if (nspl .eq. 1) then
            filename=splpaths(nspl)(1:ilbl)//'/spline.ls'
            ilbl=slen(filename)
            open(idump,file=filename,status='old',form='formatted',err=905)
         else
            filename=splpaths(nspl)(1:ilbl)//'/spline.ls-'//strpid(1:jlbl)
            extcmd = &
               "ls "//splpaths(nspl)(1:ilbl)//" > "//filename(1:ilbl+jlbl+11)
            call excmd(extcmd)
            ilbl=slen(filename)
            open(idump,file=filename,status='old',form='formatted',err=900)
         endif

         rewind(idump)
         do while (.true.)
            read(idump,'(a32)',end=10,err=910) string
            jlbl=slen(string)
            if (jlbl .lt. 3) cycle
            if (string(jlbl-2:jlbl) .eq. 'spl') nsplnames=nsplnames+1
         enddo
   10    continue

         close(idump)
      enddo

!-----------------------------------------------------------------------
! If initial pass, return after getting no. of names
!-----------------------------------------------------------------------
      if (ipass .eq. 1) return

!-----------------------------------------------------------------------
! set default units to au
!-----------------------------------------------------------------------
      splunits(:,:) = 'au'

!-----------------------------------------------------------------------
! read in and store names
!-----------------------------------------------------------------------
      n=0
      do nspl=1,nsplpaths
         ilbl=slen(splpaths(nspl))
         jlbl=slen(strpid)
         if (nspl .eq. 1) then
            filename=splpaths(nspl)(1:ilbl)//'/spline.ls'
         else
            filename=splpaths(nspl)(1:ilbl)//'/spline.ls-'//strpid(1:jlbl)
         endif
         ilbl=slen(filename)
         open(idump,file=filename,status='old',form='formatted',err=900)

         do while (.true.)
            read(idump,'(a32)',end=20,err=910) string
            jlbl=slen(string)
            if (jlbl .lt. 3) cycle
            if (string(jlbl-2:jlbl) .eq. 'spl') then
               n=n+1
               write(splnames(n),'(i5,a)') nspl,string(1:jlbl)
            endif
         enddo
   20    continue

         if (nspl .eq. 1) then
            close(idump)
         else
            close(idump,status='delete')
         endif
      enddo

      return
  900 routine='Getsplnames'
      write(message,'(2a)') 'Error opening spline list file : ',filename(1:ilbl)
      call errormsg
  905 routine='Getsplnames'
      write(message,'(2a)') 'Error opening spline list file : ',filename(1:ilbl)
      write(message(ilbl+37:ilbl+70),'(a)') 'Re-install Quantics to generate'
      call errormsg
  910 routine='Getsplnames'
      write(message,'(2a)') 'Error reading spline list file : ',filename(1:ilbl)
      call errormsg
  999 write(message,'(3a,i5)') 'Error in Memory Allocation of ',cerr, &
          ' Error code is ',ierr
      call errormsg
      end subroutine getsplnames

!#######################################################################
! *** V5.0 MB ************> SUBROUTINE CPOPINP <*************************
!                          ~~~~~~~~~~~~~~~~~~~

! Appends a copy of the .op file to the input file in the name directory.

      subroutine cpopinp

      implicit None

      integer(long), external   :: slen
      integer(long)             :: i,ilbl,lfile
      logical(kind=4)           :: lexist
      character(len=360)        :: filename

! --- check an inputfile already exists, make it rwm and open it.
      filename=name(1:laenge)//'/input'
      lfile = slen(filename)
      inquire(file=filename,exist=lexist)
      if (.not. lexist) then
         routine = 'Cpopinp (einopermod)'
         message = 'No name/input file present to copy .op file into'
         call errormsg
      endif
      call excmd('chmod 664 '//filename(1:lfile))

      open (idat,file=filename,status='old',position='append',form='formatted',&
           action='write',err=13)

! --- copy .op file
      rewind(iop)
      write(idat,'(16x,a,/)')'### ------ OPERATOR FILE ------ ###'
      do
         read(iop,'(a)',err=11,end=21) inptit
         if (inptit(1:1) .ne. '#' .and. slen(inptit).gt.0)  &
               write(idat,'(a)',err=11) inptit(1:slen(inptit))
         call keylowc(inptit)
         ilbl=index(inptit,'end-operator')
         if (ilbl .gt. 0) exit
      enddo
      write(idat,'(1x,/,80a1)') '#',('*',i=1,78),'#'

! --- close input file and make it read only
      endfile(idat,err=14)
      close(idat,err=14)
      call excmd('chmod 444 '//filename(1:lfile))

      return
 11   routine = 'cpopinp'
      message = 'Error copying ~.op file to name/input file'
      call errormsg
 13   routine = 'cpopinp'
      message = 'Cannot open file: '//filename(1:lfile)
      call errormsg
 14   routine = 'cpopinp'
      message = 'Error closing file: '//filename(1:lfile)
      call errormsg
 21   routine = 'cpopinp'
      message = 'end of ~.op file found. END-OPER keyword not present?'
      call errormsg

      end subroutine cpopinp


      end module einopermod

