!#######################################################################
!
! FINDKNUM
!
! sorts out hamiltonian into correlated, over-dimensional, and 
! uncorrelated terms, and finds kzahl, nmulpot, khzal etc.
!
! defines various flags for operator terms. Any new flags must be 
! added to mvkend(heinutil.F90) for correct sorting of terms.
! and also to setoperpsi(rdopermod.F90)  to be copied for multi-packet 
! calculations
!
!#######################################################################

      module findknummod

use decimal, only: long
use global
use logdat
use maxv
use dvrdatmod
use griddatmod
use operdef
use runopermod
use lctmod
use rdopermod

use realwpmod

use operutil, only: mvkend
use op1lib,  only: zeromxi, zerovxi, zeromxl

use logopmod, only:logkterms
use genmlham, only: mlop_setk

      implicit none
      private
      public :: findknum, setkmlop
      contains

!#######################################################################

      subroutine findknum

      implicit none

      integer(long)       :: ktotold,pointer,k,k1,k2,s,s1,h,i,ilbl,jlbl,&
                             klbl,n,m,hl,nh,f,nham,kl,kh,m1,gf,space,n1,&
                             nuncorr,zeig,muld,h1,nkmlop,iord
      integer(long), dimension(maxkoe)       :: muncorr,suncorr
      character(len=c2)                      :: label

      call zeromxi(khzahl,maxdim,maxsta*maxham)
      call zerovxi(muncorr,maxkoe)
      call zerovxi(suncorr,maxkoe)
      call zerovxi(ki,maxkoe)
      call zerovxi(kf,maxkoe)
      call zeromxl(diag,maxdim,maxkoe)
      nmulpot(:)=0

!-----------------------------------------------------------------------
! loop over Hamiltonians
!-----------------------------------------------------------------------
      nham=0
      ktotold=ktot
      pointer=1
   10 continue
      nham=nham+1
      zham(nham)=pointer

!-----------------------------------------------------------------------
! ktot=ktotold+nstate so that blank spaces left in ham. list at ktot+1
!-----------------------------------------------------------------------
      n=0
      if(nham.lt.hamdim) then
         ktot=ktotold+nstate
      else
         ktot=ktotold+1
      endif
      if(ktot.gt.maxkoe) then
         routine='findknum'
         write(message,'(a,i7)') &
          'Redimension maxkoe in maxkoe.inc. Minimum:',ktot+1
         call errormsg
      endif
      if (nham .lt. hamdim) then
         k1=zham(nham)
         do k=zham(nham),ktotold
            if (hamnum(k1) .gt. nham) then
               call mvkend(k1)
               n=n+1
            else
               k1=k1+1
            endif
         enddo
      endif              ! ktot is currently set to the number
      ktot=ktotold-n     ! of terms up to Hamiltonian nham
      khtot(nham)=ktot-zham(nham)+1

!-----------------------------------------------------------------------
! set state flags
!
! If electronic dof feb.gt.0 and get state labels for term.
! For multi-set, ki,kf are state-labels. For single-set no labels 
!    required
! For direct dyanmics, kidd,kfdd are state-labels even for single-set
!-----------------------------------------------------------------------
      do k=zham(nham),ktot
         if (feb .gt. 0) then
            h=hamilton(feb,k)
!     check dissipative operators
            if (kact(k).gt.1) then
               if (h.eq.feb) then
                  message = &
               'Specify state action for each dissipative operator.'
                  call errormsg
               endif
            endif

            if (htmform(h) .eq. 1) h=hterm(1,h)

            hl=hterm(1,h)
!
! get state numbers for the terms
!
            ilbl=0
            if (hoplab(hl)(1:1) .eq. '1') then
               ilbl=0
            else if (hoplab(hl)(1:1) .eq. 'S') then
               ilbl=2
            else if (hoplab(hl)(1:1) .eq. 'Z') then
               ilbl=2
            endif
            if (ilbl .gt. 0) then
               jlbl=index(hoplab(hl),'&')
               klbl=index(hoplab(hl),' ')-1
               read(hoplab(hl)(ilbl:jlbl-1),*) s
               read(hoplab(hl)(jlbl+1:klbl),*) s1
            else
               s=1
               s1=1
            endif

         else
            s=1
            s1=1
         endif

         ksf(k)=s
         ksi(k)=s1
         if (lmult) then
            kf(k)=s
            ki(k)=s1
         else
            kf(k)=1
            ki(k)=1
         endif

         if (ldd) then
            if ((nmode .eq. 1) .or. (nmode .eq. 2 .and. leb)) then
               f = spfdof(1,1)
               h = hamilton(f,k)
               if (htmtype(h) .eq. 101 .or. htmtype(h) .eq. 105) then
                  ddsf(h)=s
                  ddsi(h)=s1
               endif
            else
               f = fkoe
               h = hamilton(fkoe,k)
               if (htmtype(h) .eq. 104) then
                  ddsf(h)=s
                  ddsi(h)=s1
               endif
            endif
         endif

         if(lddmctdh)then
!            f = spfdof(1,1)
            do f = 1,ndof
!               if(f.eq.feb)cycle
               if(basis(f).eq.19)cycle
               h = hamilton(f,k)
               if(htmtype(h).eq.202 .or. htmtype(h).eq.204 .or. htmtype(h).eq.8 &
                    .or. htmtype(h).eq.2)then
                  ddsf(h)=s
                  ddsi(h)=s1
                  if(htmfac(h).gt.1)then
                     do nh = 1,htmfac(h)
                        h1 = hterm(nh,h)
                        ddsf(h1)=s
                        ddsi(h1)=s1
                     enddo
                  endif
               endif
            enddo
         endif

      enddo

!     check consistency of state action
      if (lmult .and. dentype .ne. 1) then
         do k=zham(nham),ktot
            if ((kact(k).eq.2) .and. (k.lt.ktwin(k))) then
               k1 = ktwin(k)
               if (ki(k).ne.kf(k1)) then
                  message = 'State action is not consistent for ' &
                  //'pure-left acting dissipative operators.'
                  call errormsg
               endif
            else if ((kact(k).eq.3) .and. (k.lt.ktwin(k))) then
               k1 = ktwin(k)
               if (ki(k).ne.kf(k1)) then
                  message = 'State action is not consistent for ' &
                  //'pure-right acting dissipative operators.'
                  call errormsg
               endif
            endif
         enddo
      endif
  
!-----------------------------------------------------------------------
! set diag flags
!-----------------------------------------------------------------------
      if (.not. nodiag(nham)) then
         do k=zham(nham),ktot
            if (ki(k) .eq. kf(k)) then
               do m=1,nopmode
                  diag(m,k)=.true.

                  do n=1,nspfdof(m)
                     f=spfdof(n,m)
                     h=hamilton(f,k)
                     if(h.gt.0) then   ! LMR
                        if (htmtype(h) .ne. 1 .and. htmtype(h) .ne. 103) & 
                            diag(m,k)=.false.
                     else
                        diag(m,k)=.false.
                     endif
                  enddo
               enddo

            else
               do m=1,nopmode
                  diag(m,k)=.false.
               enddo
            endif
         enddo
      endif

!-----------------------------------------------------------------------
! set diag for muld-potentials and
! move mulpot terms to end of hamilton list
! fterm(k) flags where a muld operator is stored in term
!-----------------------------------------------------------------------
      k1=zham(nham)
      do k=zham(nham),ktot
         if (fterm(k1) .eq. 0) then
            k1=k1+1
            cycle
         endif

         f=fterm(k1)  
         h=hamilton(f,k1)  
         muld=htmmuld(h)
         if (muld.gt.0 .and. htmmode(h).eq.0) then
            do m=1,nmode
               diag(m,k1)=.true.
            enddo
            do gf=1,nmulddof(muld)
               f=mulddof(gf,muld)
               if (f .eq. 0) cycle
               m=dofspf(f)
               diag(m,k1)=.false.
            enddo
            call mvkend(k1)
            nmulpot(nham)=nmulpot(nham)+1
         endif
      enddo
      kzahl(nham)=khtot(nham)-nmulpot(nham)

!-----------------------------------------------------------------------
! find which modes are uncorrelated in uncorrelated terms
! if multi-set, on-diagonal terms may be uncorrelated. Off-diagonal not.
!-----------------------------------------------------------------------
      nuncorr=0
      kl=zham(nham)
      kh=zham(nham)+kzahl(nham)-1
outer:do k=kl,kh
         if (kact(k) .ne. 1) cycle outer

         ! Flag terms used by MLOps as correlated.
         if (hmlop(k).ne.0) then
            muncorr(k)=0
            suncorr(k)=0
            cycle
         endif
         i=0
         s=ki(k)
         if (lmult) then
            if (kf(k) .ne. ki(k)) i=i+2
         endif
         do m=1,nmode
            if (laddmode(m,nham) .and. .not. diag(m,k)) then
               i=i+1
               muncorr(k)=m
               suncorr(k)=s
            endif
         enddo
         if (i .eq. 1) then
            nuncorr=nuncorr+1
         else
            muncorr(k)=0
            suncorr(k)=0
         endif

      enddo outer

!-----------------------------------------------------------------------
! move uncorr. terms to the end of the list, one mode/state at a time
!-----------------------------------------------------------------------
      zeig=zham(nham)+kzahl(nham)+nmulpot(nham)-nuncorr
      do m=1,nopmode
         if ( laddmode(m,nham) ) then
            do s=1,nstate
               khzahl(m,s,nham)=0
               zhun(m,s,nham)=zeig
               k1=zham(nham)
               kl=zham(nham)
               kh=zham(nham)+kzahl(nham)-1
               do k=kl,kh
                  if (muncorr(k1) .eq. m .and. suncorr(k1) .eq. s) then
                     call mvkend(k1)
                     muncorr(ktot+1)=muncorr(k1)
                     suncorr(ktot+1)=suncorr(k1)
                     do k2=k1,ktot
                        muncorr(k2)=muncorr(k2+1)
                        suncorr(k2)=suncorr(k2+1)
                     enddo
                     khzahl(m,s,nham)=khzahl(m,s,nham)+1
                  else
                     k1=k1+1
                  endif
               enddo
               zeig=zeig+khzahl(m,s,nham)
            enddo
         endif
      enddo
      kzahl(nham)=kzahl(nham)-nuncorr

!-----------------------------------------------------------------------
! move MLOp terms to the end of the list
!-----------------------------------------------------------------------
      kl=zham(nham)
      kh=zham(nham)+kzahl(nham)-1
      nkmlop = 0
      k1=kl
      do k=kl,kh
         if (hmlop(k1).ne.0) then
            call mvkend(k1)
            nkmlop=nkmlop+1
         else
            k1=k1+1
         endif
      enddo
      ! Adjust the pointers.
      do m=1,nopmode
         if (laddmode(m,nham)) then
            do s=1,nstate
               zhun(m,s,nham)=zhun(m,s,nham)-nkmlop
            enddo
         endif
      enddo
      ! Don't include the MLOp kterms in the regular Hamiltonian.
      kzahl(nham)=kzahl(nham)-nkmlop
      khtot(nham)=khtot(nham)-nkmlop

!-----------------------------------------------------------------------
! Set lklha if G-MCTDH with correlated LHA.
! Loop over correlated terms and if found reset diag flag (always
! working in GWP basis)
!-----------------------------------------------------------------------
      kl=zham(nham)
      kh=zham(nham)+kzahl(nham)-1
      klha(:) = 0
      lklha = .false.
      do k=kl,kh
         h = hamilton(fkoe,k)
         if (htmtype(h) .eq. 104) then
            lklha = .true.

            hl = hterm(1,h)
            label = hoplab(hl)

            ilbl = index(label,'%')
            read(label(ilbl+1:ilbl+1),*) iord
            klha(k) = iord+1
         endif
      enddo

!-----------------------------------------------------------------------
! flag complex terms (with a + sign for autocap, - for cap)
!
! usually CAPs are uncorrelated as they are in a single channel,
! but (especially in G-MCTDH) can be in correlated terms.
!-----------------------------------------------------------------------
      do m=1,nopmode
        do s=1,nstate
            kl=zhun(m,s,nham)
            kh=zhun(m,s,nham)+khzahl(m,s,nham)-1
            do k=kl,kh
               h=hamilton(fkoe,k)  ! Set lcap if coeff is not real
               if((htmtype(h).eq.3 .or. htmtype(h).eq.4 .or. &
                       htmtype(h) .eq. 108).and.nham.eq.1) &
                    lcap=.true.
              
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  h=max(1,hamilton(f,k))   ! LMR
                  if (htmtype(h) .eq. 3 .or. htmtype(h) .eq. 4 .or. &
                      htmtype(h)  .eq. 108) then
                     lcap = .true.
                     do nh=1,htmfac(h)
                        hl=hterm(nh,h)
                        if (hl .ne. 0) then
! CAP on a specific or ALL electronic state! 
                           if (hoplab(hl)(1:4) .eq. 'CAP[') then
                              ilbl=index(hoplab(hl),'[')+1
                              jlbl=index(hoplab(hl),']')-1
                              read(hoplab(hl)(ilbl:jlbl),*) n1
                              nstatecap(k) = nint(hoppar(5,n1))
                              if (nstatecap(k) .eq. 0) then 
                                 autocap(k)=-f 
                              else if (s .eq. nstatecap(k) ) then
                                 autocap(k)=-f
                              endif
                           endif
                           if (hoplab(hl)(1:5) .eq. 'ACAP[')  then
                              ilbl=index(hoplab(hl),'[')+1
                              jlbl=index(hoplab(hl),']')-1
                              read(hoplab(hl)(ilbl:jlbl),*) n1
                              autocap(k)=f
                              nstatecap(k) = nint(hoppar(6,n1))
                           endif
                        endif
                     enddo     
                  endif
               enddo
            enddo
         enddo
      enddo

      kl=zham(nham)
      kh=zham(nham)+kzahl(nham)-1
      do k=kl,kh
         do f=1,nmode
            h=hamilton(f,k)
            if (htmtype(h) .eq. 3 .or. htmtype(h) .eq. 4 .or. &
                htmtype(h)  .eq. 108) lcap = .true.
         enddo
      enddo

!-----------------------------------------------------------------------
! flag real terms (with a + sign for autoabs and autoflux)
!-----------------------------------------------------------------------
   if (lrealwp) then
      do m=1,nopmode
        do s=1,nstate
            kl=zhun(m,s,nham)
            kh=zhun(m,s,nham)+khzahl(m,s,nham)-1
      inner:do k=kl,kh
               autoabs(k)=0
               autoflux(k)=0
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  h=max(1,hamilton(f,k))   ! LMR
                  if (htmform(h) == 0 .and. htmtype(h) == 2) then
                     do nh=1,htmfac(h)
                        hl=hterm(nh,h)
                        if (hl /= 0) then
                           if (hoplab(hl)(1:5) == 'DAMP[') then
                              autoabs(k)=f
                              cycle inner
                           else if (hoplab(hl)(1:7) == 'fluxws[') then
                              autoflux(k)=f
                              cycle inner
                           endif
                        endif
                     enddo
                  endif
               enddo
            enddo inner
         enddo
      enddo
   endif

!-----------------------------------------------------------------------
! set kinop flag
!
! Definition of non-diagonal single-particle operators:
!  =1: fully non-diagonal
!  =0: diagonal
!  <0: side-diagonal. the value encodes the shift k:
!      k>0: kinop=-2*k
!      k<0: kinop= 2*k+1
!-----------------------------------------------------------------------
      kl=zham(nham)
      kh=zham(nham)+khtot(nham)-1

      do k=kl,kh
         kinop(:,k)=0
         h=max(1,hamilton(fkoe,k))  ! Set lcap if coeff is not real ! LMR
         if((htmtype(h).eq.3 .or. htmtype(h).eq.4 .or. &
                 htmtype(h) .eq. 108).and.nham.eq.1) lcap=.true.

         do m=1,nmode
            if (m .eq. meb) cycle
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (f .eq. febcont) cycle
               h=hamilton(f,k)
               if(h.gt.0) then  ! LMR
                  if (htmtype(h) .ne. 1 .and. htmtype(h) .ne. 2 .and. &
                      htmtype(h) .ne. 101 .and. htmtype(h) .ne. 103 .and. &
                      htmtype(h) .ne. 104 .and. htmtype(h) .ne. 105 .and. &
                      htmtype(h) .ne. 109 .and. htmtype(h) .ne. 202 .and. &
                      htmtype(h) .ne. 204) then
                     kinop(m,k)=1
                  endif
               else
                  kinop(m,k)=1 ! LMR operators treated as full non-diagonal
               endif
       !        if ( (dentype.eq.1) .and. (htmdof(h) .ne. m)) then
       !           kinop(m,k)=1
       !        endif
            enddo

!           check for side-diagonal mode operator
            if (kinop(m,k).ne.0) then
               f=spfdof(1,m)
               h=max(1,hamilton(f,k))   ! LMR
               if ( htmtype(h).eq.12 .and. &
                   (nspfdof(m).eq.1 .or. htmmode(h).eq.m)) then
                  if (htmshift(h).ge.0) then
                     kinop(m,k)=-2*htmshift(h)
                  else
                     kinop(m,k)= 2*htmshift(h)+1
                  endif
               endif
!              TODO also implement this for densities...
               if (dentype.ne.0) kinop(m,k)=1
!              kinop(m,k)=1 ! Debug: uncomment for testing
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! flag Hamiltonian if time-dependent terms present
! and look for local-control and functions of expectations
!-----------------------------------------------------------------------
      ltdh(nham)=.false.
      if (ftime .gt. 0) then
         kl=zham(nham)
         kh=zham(nham)+khtot(nham)-1
         do k=kl,kh
            h=hamilton(fkoe,k)
            if (htmtype(h) .eq. 101) then
               ltdh(nham)=.true.
            else if (htmtype(h) .eq. 151) then
               ltdh(nham)=.true.
               lcontrol = .true.
            else if (htmtype(h) .eq. 161) then
               ltdh(nham)=.true.
            endif
         enddo
      endif

!-----------------------------------------------------------------------
! see if any more Hamiltonians need to be sorted
! move blank spaces at ktot+space+1 to end of list
! The space 'space' between the operators is needed for pmat(m,k+s)
!-----------------------------------------------------------------------
      if (nham .lt. hamdim) then
! ---    compute space which may be necessary between operators.
         space=nstate
         do s=1,nstate
            do m=1,nmode
               space=space-khzahl(m,s,nham)
            enddo
         enddo
         space=max(0,space)

! ---    Set dummy values
         do k1=ktot+1,ktot+space
            hamnum(k1)=nham
            ktwin(k1)=k1
            ki(k1)=1
            kf(k1)=1
            do f=1,nopdof
               hamilton(f,k1)=f
            enddo
            do m=1,nopmode
               diag(m,k1)=.true.
            enddo
         enddo

! ---    move blank spaces at ktot+space+1 to end of list
         k1=ktot+space+1
         ktot=ktotold+nstate   ! This is for mvkend
         do i=space+1,nstate
            call mvkend(k1)
         enddo

         pointer=pointer+khtot(nham)+nkmlop+space
         ktotold=ktotold+space
         go to 10
      endif
!-######################################################################
! End of implicit loop (goto 10). End of sorting Hamiltonians (nham).
!-######################################################################

!-----------------------------------------------------------------------
! find symmetry of operators used in Hamiltonians
!-----------------------------------------------------------------------
      do k=1,ktot
         do m=1,nopmode
            f=spfdof(1,m)
            h=hamilton(f,k)
            if(h.lt.0) then   ! LMR
               hsym(m,k)=0
            else
! multiply symmetries
               hsym(m,k)=htmsym(h)
               do n=2,nspfdof(m) 
                  f=spfdof(n,m)
                  h=hamilton(f,k)
                  hsym(m,k)=hsym(m,k)*htmsym(h)
               enddo
            endif
         enddo

! symmetry not to be used if non-adiabatic operator
         if( lmult .and. (ki(k).ne.kf(k)) ) then
            do m=1,nopmode
               hsym(m,k)=0
            enddo
         endif

! symmetry not to be used for dissipative operators
         if( kact(k).ne.1 ) then
            do m=1,nopmode
               hsym(m,k)=0
            enddo
         endif

! symmetry not to be used for GWP mode operators unless
! full analytic potential terms
   !      do m=1,nopmode
   !         if (gwpm(m)) hsym(m,k) = 0

   !         f=spfdof(1,m)
   !         h=hamilton(f,k)

   !         if (htmtype(m,k) .eq. 101) hsym(m,k) = 1
   !      endif

      enddo

!-----------------------------------------------------------------------
! find symmetry of hloch operator matrices
!-----------------------------------------------------------------------
      do k=1,ktot
         do m=1,nmode
            hlsym(m,k)=1
            do m1=1,nmode
               if (m1 .ne. m) hlsym(m,k)=hlsym(m,k)*hsym(m1,k)
            enddo
            hlsym(m,k)=0   !  this is for test
         enddo
      enddo

      return
      end subroutine findknum

!=======================================================================

      subroutine setkmlop
!-----------------------------------------------------------------------
! After the Hamiltonian lines have been summed, rearranged, etc.
! the numbering of these lines is final. Now we can go back to the
! multi-layer operators and set the references to the k-terms for
! the leaves.
!-----------------------------------------------------------------------

      implicit none

      integer k,f,h,nh,n,hl,h1,nh1,n1
      logical lhamr,lhamt

!     Go through the Hamiltonian and set k for all terms that include
!     an MLPF leaf potential (hopfile=9).
      do k=1,ktot
         if (hmlop(k)==0) cycle
         do f=1,ndof
            h=hamilton(f,k)
            if (h.le.0) goto 500
            nh=htmfac(h)
            if (htmform(h).eq.0) then
               do n=1,nh
                  hl=hterm(nh,h)
                  if (hl.le.0) goto 500
                  if (hopfile(hl).eq.9) &
                     call mlop_setk(k,hopilab(hl),hopipar(1,hl), &
                          hopipar(2,hl))
               enddo
            else
               do n=1,nh
                  h1=hterm(n,h)
                  if (h1.le.0) goto 500
                  nh1=htmfac(h1)
                  do n1=1,nh1
                     hl=hterm(nh1,h1)
                     if (hl.le.0) goto 500
                     if (hopfile(hl).eq.9) &
                        call mlop_setk(k,hopilab(hl),hopipar(1,hl), &
                             hopipar(2,hl))
                  enddo
               enddo
            endif
         enddo
      enddo
      return

  500 write (*,'(a,i0)') "f=",f
      call logkterms(6,k,k,lhamr,lhamt)
      routine = "setkmlop"
      message = "unexpected structure of Hamiltonian term"
      call errormsg

      end subroutine setkmlop


      end module findknummod

