!#######################################################################
! New version (Benjamin, December 2008)
!#######################################################################
!
!            Subroutine      HEINLAB
!
! Reads the LABELS-SECTION, i.e. reads the data for any
! Hamiltonian terms defined "external" to the program.
! Called from: heingabe, einoper, matchlab
! Calls to:  rdinpf, cntrl, rmblnk, rdfactors, mkhpar, warnmsg
!
! 11/99 GW   (error messages etc, 05/01 HDM)
!
!#######################################################################

      subroutine heinlab(ichan,loperr,iz,pass)

use decimal, only: long,dop
use global
use logdat
use maxv, only: maxhpar,maxrdf
use griddatmod
use operdef
use runopermod
use channels
use heinutil, only: inc_rdfiles

use operutil, only: rdfactors,mkhpar


      implicit none

      integer(long)                        :: i,iz0,ic,ierr,j,n,n1,np,&
                                              nfac,nopt,nhpar,maxlen,&
                                              ilbl,jlbl
      integer(long), intent(in)            :: iz,ichan,pass
      integer(long), dimension(24)         :: optyp
      real(dop)                            :: x
      logical(kind=4)                      :: lprint
      logical(kind=4), intent(inout)       :: loperr
      character(len=c2)                    :: buffer
      character(len=c2),dimension(24)      :: factor

      iz0=0
      if (pass .eq. 2) then
         if (ichan .eq. iin) then
            routine = 'Heinlab (alter-labels,  *.inp file)'
            write(ilog,'(/a)') 'Operator labels taken from input file'
         else if (ichan .eq. iop) then
            routine = 'Heinlab (Labels-Section,  *.op file)'
            write(ilog,'(/2a)') 'Operator labels taken from .op file '
         endif
      endif

      nhparmax=0
      nhlabmax=max(hopdim,0)
      maxlen=0
      lprint=.true.

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
 10   continue
      call rdinpf(ichan,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)
      i=1

 15   continue
      if (keyword(i) .eq. 'end-labels-section') then
         goto 20
      else if (keyword(i) .eq. 'end-alter-labels') then
         goto 20
      endif

!-----------------------------------------------------------------------
! read labels and definitions. Check for legality.
!-----------------------------------------------------------------------
 16   do n1=1,npre
         if (keyorig(i) .eq. prelabel(n1)) then
            write(6,'(/a80)') inptit
            routine='heinlab'
            message='A new label must not coincide with a pre-defined'// &
            " one! See the User's Guide for a list of pre-defined"// &
            " labels. Label : "//keyorig(i)(1:50)
         call errormsg 
         endif
      enddo
      do n1=1,npar
         if (keyorig(i) .eq. apar(n1)) then
            write(6,'(/a80)') inptit
            routine='heinlab. LABELS-SECTION'
            message = 'Label already used as parameter : '//keyorig(i)(1:46)// &
                '  A new label must not coincide with a parameter name!'
            call errormsg 
         endif
      enddo
      if(index(keyorig(i),':') .gt. 0) then
         write(6,'(/a80)') inptit
         routine='heinlab. LABELS-SECTION'
         message='A new operator label must not contain a colon (:).' &
         //' Label = '//keyorig(i)(1:50)
         call errormsg
      endif

!-----------------------------------------------------------------------
! If label is already assigned, do not reassign.
!-----------------------------------------------------------------------
      n=0
      do n1=1,min(ndef,maxdef)
         if (keyorig(i) .eq. deflabel(n1)) then
             n=n1
             exit
         endif
      enddo
      if (n .ne. 0) then
         message = ' Label already assigned : '//keyorig(i)
         write(ilog,'(a)') message
!....Move on to next assignment
         i=i+3
         if(keyword(i) .eq. '{') then
            do j = i, ic
               if(keyword(j) .eq. '}') then
                  i=j+1
                  exit
               endif
            enddo
         endif
         do j = i, ic
            if(keyword(j) .eq. '=') then
            i = j-1
            goto 16
            endif
         enddo
         go to 10
      else
!....Define new operator label         
         ndef=ndef+1
         if (ndef .le. maxdef) then
            n=ndef
         else
            n=maxdef
         endif
         deflabel(n)=trim(keyorig(i))
      endif

      if (keyword(i+1) .ne. '=') then
         if (lprint) then
            write(6,910)
            write(ilog,910)
            lprint=.false.
         endif
         message='No operator defined by label : '
         call wrline(i,iz,iz0)
         go to 100
      endif

      i=i+2
!-----------------------------------------------------------------------
! Check for unallowed operations.
!-----------------------------------------------------------------------
      do j = 1, lc(i)
         if(keyword(i)(j:j).eq.'+' .or. keyword(i)(j:j).eq.'-' .or. &
            keyword(i)(j:j).eq.'*' .or. keyword(i)(j:j).eq.'/') then
            if (lprint) then
               write(6,910)
               write(ilog,910)
               lprint=.false.
            endif
            message='Forbidden operation : '
            call wrline(i,iz,iz0)
            go to 10
         endif
      enddo

!-----------------------------------------------------------------------
! see if operator has any associated parameters
!-----------------------------------------------------------------------
      if (keyword(i+1) .eq. '[') then
         nhoppar=nhoppar+1
         np=nhoppar
         write(buffer,'(2a,i7,a)') keyorig(i)(1:lc(i)),'[',np,']'
         call rmblnk(buffer)

         i=i+2
         nhpar=0
 35      continue
         if (keyword(i) .eq. ']') then
! add power to operator
            if (keyword(i+1)(1:1) .eq. '^') then
               ilbl=index(buffer,' ')-1
               i=i+1
               buffer=buffer(1:ilbl)//keyword(i)(1:lc(i)) 
            endif
            go to 99
         else if (keyword(i) .eq. ',') then
            i=i+1
            go to 35
         else
            nhpar=nhpar+1

            if ((nhpar .le. maxhpar) .and. (np .lt. maxhop)) then
               call rdfactors(keyorig(i),nfac,factor,optyp,lc(i))
               call mkhpar(x,nfac,factor,optyp)
               hoppar(nhpar,np)=x
            else
               nhparmax=max(nhparmax,nhpar)
               nhlabmax=max(nhlabmax,np)
            endif

            i=i+1
            go to 35
         endif
           
      else if (keyword(i+1) .eq. '{') then
         npesopt=npesopt+1
         if (npesopt .gt. maxrdf) call inc_rdfiles
         np=npesopt
         write(buffer,'(2a,i7,a)') keyorig(i)(1:lc(i)),'{',np,'}'
         call rmblnk(buffer)

         do nopt=1,min(nhparmax,2*maxhpar)
            zpesopts(nopt,np)=0
         enddo

         i=i+2
         nopt=0
         ilbl=1
 45      continue
         if (keyword(i) .eq. '}') then
            go to 99
         else if (keyword(i) .eq. ',') then
            i=i+1
            go to 45
         else
            nopt=nopt+1
            jlbl=ilbl+lc(i)-1
            if (nopt .le. 2*maxhpar) then
               zpesopts(nopt,np)=jlbl
               if (jlbl .lt. c5) then
                  pesopts(np)(ilbl:jlbl)=keyorig(i)(1:lc(i))
               else
                  maxlen=max(maxlen,jlbl)
               endif
               ilbl=jlbl+1
            else
               nhparmax=max(nhparmax,nopt/2+1)
            endif

         endif

! Get next keyword. If argument goes onto next line, read that
         if (i .eq. ic) then
            call rdinpf(ichan,keyword,keyorig,inptit,lc,ic,iz, &
                    ierr,maxkey)
            message = inptit
            call cntrl(keyword(1),lc(1),ierr)
            i=1
         else
            i=i+1
         endif
         go to 45

      else
         buffer=trim(keyorig(i))
      endif

 99   continue
!-----------------------------------------------------------------------
! store operator
!-----------------------------------------------------------------------
      deflabop(n)=trim(buffer)

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
 100  continue
      if (i.lt.ic) then
         i=i+1
         goto 15
      else
         goto 10
      endif

 20   continue
     
      if (maxlen .gt. 0) then
         write(message,'(a,i5)')&
           'Increase length of pesopts in operdef.inc. Minimum: ',maxlen
         call warnmsg
      endif

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      loperr=lwarn.or.loperr
      lwarn=.false.

      return

 910  format(/'******************************************************',/&
             'ERROR  ERROR  ERROR  ERROR  ERROR  ERROR  ERROR  ERROR',/&
             'No arithmetic among operators is allowed in the ',/&
             'LABELS-SECTION except exponentiation.',/&
             'I.e. no "+", "-", "*" or "/" operations are allowed,',/&
             'except within the parameter brackets.',/&
             '******************************************************',/)

      end subroutine heinlab

