!#######################################################################
! 
! SORTEB
!
! sorts out electronic basis in Hamiltonian e.g. :
! for single-set calculation set nstate=1
! for multi-set calculation, expand unit operator, 1, to s1&1, s2&2 etc.
!                          , add adjoint to off-diagonal terms s1&2 etc.
!
! contains:sorteb,delzero,delke,delkepnt,sortktms,sumhtms,combhtm1d,
!          combhtms,sortmode,sortmuld,sortanal,delkg,rmdq
!
!#######################################################################

      module heinsort

use decimal, only: long, dop
use global
use maxv, only: maxkoe,maxfac
use dvrdatmod
use griddatmod
use operdef
use logdat
use runopermod
use channels

use heinutil, only: hlabnum,htmnum,flaguncorr
use op1lib,  only: zeromxi, zerovxi, zerovxl, zeromxl
use operutil, only: findnath,findread,mvkend
use htmsetupmod, only: checkjm
use realwpmod
use deranld, only: fdefderd
use funcxlocmod, only: ddefxloc
use logopmod

      implicit none
      private
      public :: sorteb,delzero,delke,delkepnt,sortktms,sumhtms,sortmode,&
                sortmuld,sortanal,delkg,sortlha
      contains
!#######################################################################
! pass = 1 memory allocation runs
!        2 operator creation run
!#######################################################################

      subroutine sorteb(pass,loperr)

      implicit none

      integer(long), intent(in) :: pass
      integer(long)             :: k,k1,s,ilbl,jlbl,klbl,s1,h,k2,f,&
                                   ktot1,hl,ierr,ierr1,nham,nf,nf1
      integer(long), dimension(maxfac)       :: iterm
      integer(long), dimension(maxkoe)       :: optyp
      logical(kind=4), intent(out)           :: loperr
      character(len=c2)                      :: buffer

! initialise variables
      routine='Sorteb'
      ilbl=0

!-----------------------------------------------------------------------
! if pass = 2 memory allocation finished so full checks can be run 
! operator stored in hamilton(feb,k)
!-----------------------------------------------------------------------
      if (pass .eq. 2) then

         do k=1,ktot
            h=hamilton(feb,k)

            if (htmfac(h) .gt. 1) then
               write(message,'(a,i5)')&
          'Only 1 factor allowed for electronic Hamiltonian term : ',k
               call warnmsg
            endif

            hl=hterm(1,h)
            if (hl .eq. 0) cycle
            if (hoplab(hl)(1:1) .eq. 'S' .or. hoplab(hl)(1:1) .eq. 'Z'&
! continuum operators
                .or. hoplab(hl) .eq. '1' .or. hoplab(hl)(1:2) .eq. 'CE'&
                .or. hoplab(hl)(1:2) .eq. 'CS' &
                .or. hoplab(hl)(1:2) .eq. 'CZ' & !cycle
!-Alice-tb operators
                .or. hoplab(hl)(1:4) .eq. 'tb1d' &
                .or. hoplab(hl)(1:6) .eq. 'tbdiag' &
                .or. hoplab(hl)(1:7) .eq. 'CS_bath'  &
! Cap also allowed
                .or. hoplab(hl)(1:3) .eq. 'CAP') cycle

            message=&
          'Unknown Hamiltonian term for electronic degree of freedom : '//&
                hoplab(hl)
            call warnmsg
         enddo

         if (lwarn) then
            loperr=.true.
            lwarn=.false.
            return
         endif
      endif

!-----------------------------------------------------------------------
! Run through S and Z matrices to check indices and determine optyp(k).
! optyp = 1: 1
!         2: Ss&s1
!         3: Ss&s1
!         4: Zs&s1
!         5: CSs&s1
!         6: CZs&s1
!   Alice-7: CS_baths&s1  - tight binding decay function
!-----------------------------------------------------------------------
      do k=1,min(ktot,maxkoe)
         optyp(k)=0
         h=hamilton(feb,k)
         hl=hterm(1,h)
         if (hl .eq. 0) cycle

         nham=hamnum(k)
         ! determine operator type
         if (hl .eq. 1) then
            optyp(k)=1
         else 
! first read state labels
            if (hoplab(hl)(1:1) .eq. 'S' .or.  &
                  hoplab(hl)(1:1) .eq. 'Z') then
               ilbl=2

            else if (hoplab(hl)(1:2) .eq. 'CE' .or. &
                    (hoplab(hl)(1:2) .eq. 'CS' .and. hoplab(hl)(3:3) .ne. '_') .or. &
                    hoplab(hl)(1:2) .eq. 'CZ') then
               ilbl=3  

            else if (hoplab(hl)(1:7) .eq. 'CS_bath') then
!              print *, 'CSbath time_update'
               ilbl=8

            else
              cycle
            endif

            jlbl=index(hoplab(hl),'&')
            klbl=index(hoplab(hl),' ')-1
            read(hoplab(hl)(ilbl:jlbl-1),*,iostat=ierr) s
            read(hoplab(hl)(jlbl+1:klbl),*,iostat=ierr1) s1
            if(ierr.gt.0 .or. ierr1.gt.0) then
               message= &
               'Electronic indices are not numbers: '//hoplab(hl)
               routine='sorteb (heinsort.F)'
               call errormsg
            endif

! now set optype and reform operator if required
            if (hoplab(hl)(1:1) .eq. 'S' .and. s .ne. s1) then
               optyp(k)=2

            elseif (hoplab(hl)(1:1) .eq. 'S') then
               optyp(k)=3

            elseif (hoplab(hl)(1:1) .eq. 'Z') then
               if(s.eq.s1) then
                  message= &
                  'Z must carry different indices: '//hoplab(hl)
                  routine='sorteb (heinsort.F)'
                  call errormsg
               endif
               optyp(k)=4

            elseif (hoplab(hl)(1:2) .eq. 'CE') then
               if (s .ne. s1) then
                  routine='sorteb (heinsort.F)'
                  write(message,'(a,2i5)') &
                     'CE operator must be applied to single continuum state. NOT :',s,s1
                  call errormsg
               endif
               if (.not. sebcont(s) .and. .not. sebcont(s1)) then
                  routine='sorteb (heinsort.F)'
                  write(message,'(a,i5)') &
                     'CE operator must be applied to continuum state. NOT :',s
                  call errormsg
               endif

! same type as Ss&s1
               optyp(k)=3

! Change CEs&s1 to  Ss&s1 * CE
               write(buffer,'(a)') 'CE '
               call rmblnk(buffer)
               call hlabnum(hl,buffer)
               iterm(1) = hl
               nf = 1
               call htmnum(h,iterm,nf,febcont,0,0)

               hamilton(febcont,k) = h

               write(buffer,'(2(a,i5))') 'S',s,'&',s1
               call rmblnk(buffer)
               call hlabnum(hl,buffer)
               iterm(1) = hl
               nf = 1
               call htmnum(h,iterm,nf,feb,0,0)

               hamilton(feb,k) = h

            elseif (hoplab(hl)(1:2) .eq. 'CS' .and. hoplab(hl)(3:3) .ne. '_') then
               if (s .eq. s1) then
                  routine='sorteb (heinsort.F)'
                  write(message,'(a,2i5)') &
                     'CS operator must be applied different states. NOT just :',s
                  call errormsg
               endif
               if (.not. sebcont(s) .and. .not. sebcont(s1)) then
                  routine='sorteb (heinsort.F)'
                  write(message,'(a,2i5)') &
                     'CS operator must be applied to continuum state. NOT :',s,s1
                  call errormsg
               endif

               optyp(k)=5

! Change CSs&s1 to  Zs&s1 * CZs&s1 ( + Zs1&s * CZs1&s to be added below)
               write(buffer,'(2(a,i5))') 'CZ',s,'&',s1
               call rmblnk(buffer)
               call hlabnum(hl,buffer)
               nf = 1
               h=hamilton(febcont,k)
               if (h .ne. febcont) then  ! not unit operator
                  nf = nf+htmfac(h)
                  if (sebcont(s)) then    ! continuum on left
                     iterm(1:nf-1) = hterm(1:nf-1,h)
                     iterm(nf) = hl
                  else                    ! continuum on right
                     iterm(1) = hl
                     iterm(2:nf) = hterm(1:nf-1,h)
                  endif
               else
                  iterm(nf) = hl
               endif
               call htmnum(h,iterm,nf,febcont,0,0)

               hamilton(febcont,k) = h

               write(buffer,'(2(a,i5))') 'Z',s,'&',s1
               call rmblnk(buffer)
               call hlabnum(hl,buffer)
               iterm(1) = hl
               nf = 1
               call htmnum(h,iterm,nf,feb,0,0)

               hamilton(feb,k) = h

            elseif (hoplab(hl)(1:2) .eq. 'CZ') then
               if (s .eq. s1) then
                  routine='sorteb (heinsort.F)'
                  write(message,'(a,2i5)') &
                     'CZ operator must be applied different states. NOT just :',s
                  call errormsg
               endif
               if (.not. sebcont(s) .and. .not. sebcont(s1)) then
                  routine='sorteb (heinsort.F)'
                  write(message,'(a,2i5)') &
                     'CZ operator must be applied to continuum state. NOT :',s,s1
                  call errormsg
               endif

               optyp(k)=6

! Change CZs&s1 to  Zs&s1 * CZs&s1
               write(buffer,'(2(a,i5))') 'CZ',s,'&',s1
               call rmblnk(buffer)
               call hlabnum(hl,buffer)
               nf=0
               h=hamilton(febcont,k)
               if (h .ne. febcont) then  ! not unit operator
                  nf = htmfac(h)
                  iterm(1:nf) = hterm(1:htmfac(h),h)
               endif
               nf = nf+1
               iterm(nf) = hl
               if (h .ne. febcont) then  ! not unit operator
                  nf1 = nf+1
                  nf = nf+htmfac(h)
                  iterm(nf1:nf) = hterm(1:htmfac(h),h)
               endif

               call htmnum(h,iterm,nf,febcont,0,0)

               hamilton(febcont,k) = h
               write(buffer,'(2(a,i5))') 'Z',s,'&',s1
               call rmblnk(buffer)
               call hlabnum(hl,buffer)
               iterm(1) = hl
               nf = 1
               call htmnum(h,iterm,nf,feb,0,0)

               hamilton(feb,k) = h

!-Alice-CS_bath
            elseif (hoplab(hl)(1:7) .eq. 'CS_bath') then
               if (.not. sebcont(s) .and. .not. sebcont(s1)) then
                  routine='sorteb (heinsort.F)'
                  write(message,'(a,2i5)') &
                     'CS_bath operator must be applied to continuum state. NOT :',s,s1
                  call errormsg
               endif

               optyp(k)=7

! Change CS_baths&s1 to  Ss&s1 * CS_bath
               hamilton(febcont,k) = hamilton(feb,k)
               h = hamilton(febcont,k)
               htmdof(h) = febcont

               write(buffer,'(2(a,i5))') 'S',s,'&',s1
               call rmblnk(buffer)
               call hlabnum(hl,buffer)
               iterm(1) = hl
               nf = 1
               call htmnum(h,iterm,nf,feb,0,0)

               hamilton(feb,k) = h

            endif
        endif

      enddo

!-----------------------------------------------------------------------
! change operators as required for single-set or multi-set depending
! on type of operator 
! optyp = 1: 1
!         2: Ss1&s
!         3: Ss&s
!         4: Zs1&s
!   Alice-7: CS_baths1&s  - tight binding decay function
!-----------------------------------------------------------------------

      if (leb) then
!-----------------------------------------------------------------------
! for single-set calculation, reset nstate to 1, and return.
! For optyp = 5 (CSs&s1) need to add explicit Zs1&s * CZs1&s
!-----------------------------------------------------------------------
         nstate=1

         do k=1,min(ktot,maxkoe)
            if (optyp(k) .eq. 5) then
               minkoe=minkoe+1
            endif
         enddo
         if (minkoe .gt. maxkoe) return

         ktot1=ktot
         k1 = 1
         do k=1,ktot1
            if (optyp(k) .eq. 5) then
               call mvkend(k1)
!
! For Ss&s1 terms mirror to Ss1&s
!
               h    = hamilton(feb,ktot)
               hl   = hterm(1,h)
               ilbl = 2
               jlbl = index(hoplab(hl),'&')
               klbl = index(hoplab(hl),' ')-1
               read(hoplab(hl)(ilbl:jlbl-1),*) s
               read(hoplab(hl)(jlbl+1:klbl),*) s1

               k2 = ktot+1

               write(buffer,'(2(a,i5))') 'CZ',s1,'&',s
               call rmblnk(buffer)
               call hlabnum(hl,buffer)

               nf = 1
               h    = hamilton(febcont,ktot)
               if (htmfac(h) .gt. 1) then
                  nf = nf+htmfac(h)
                  if (sebcont(s1)) then    ! continuum on left
                     iterm(1:nf-1) = hterm(1:nf-1,h)
                     iterm(nf) = hl
                  else                    ! continuum on right
                     iterm(1) = hl
                     iterm(2:nf) = hterm(1:nf-1,h)
                  endif
               else
                  iterm(nf) = hl
               endif
               call htmnum(h,iterm,nf,febcont,0,0)

               hamilton(febcont,k2) = h

               write(buffer,'(2(a,i5))') 'Z',s1,'&',s
               call rmblnk(buffer)
               call hlabnum(hl,buffer)
               iterm(1) = hl
               nf = 1
               call htmnum(h,iterm,nf,feb,0,0)

               hamilton(feb,k2) = h

               hamnum(k2)=hamnum(ktot)
               kact(k2)  = kact(ktot)
               ktwin(k2) = ktwin(ktot)
               s  = s +nstate
               s1 = s1+nstate
               do f = 1,nopdof
                  if (f .ne. feb .and. f .ne. febcont) then
                     hamilton(f,k2) = hamilton(f,ktot)
                  endif
               enddo
               ktot = ktot+1

            else
               k1=k1+1
            endif

         enddo

      else
!-----------------------------------------------------------------------
! for multi-set calculation, 
!    if unit operator, 1, expand in electronic basis S1&1 etc. optyp=1
!    if off-diagonal matrix term, add conjugate. optyp=2
!
! check maxkoe large enough to continue
! optyp = 1 is unit operator that needs to be expanded to all nstates. 
! optyp = 2 is off-diagonal that need to be reflected
! other types need no further space.
!-----------------------------------------------------------------------
         do k=1,min(ktot,maxkoe)
            if (optyp(k) .eq. 1) then  
               minkoe=minkoe+nstate-1
            else if (optyp(k) .eq. 2 .or. optyp(k) .eq. 5) then
               minkoe=minkoe+1
            endif
         enddo
         if (minkoe .gt. maxkoe) return

         ktot1=ktot
         k1 = 1
         do k=1,ktot1
            if (optyp(k) .eq. 1) then  

! Expand in electronic basis for all states, i.e. 1 -> 1 & 1, 2 & 2, ...

               call mvkend(k1)

               s=0
               do k2=ktot,ktot+nstate-1
                  s=s+1
                  write(buffer,'(2(a,i5))') 'S',s,'&',s
                  call rmblnk(buffer)

                  call hlabnum(hl,buffer)
                  iterm(1)=hl
                  nf=1
                  call htmnum(h,iterm,nf,feb,0,0)

                  hamilton(feb,k2)=h
                  hamnum(k2)=hamnum(ktot)
                  kact(k2)  = kact(ktot)
                  ktwin(k2) = ktwin(ktot)
               enddo

               do k2=ktot+1,ktot+nstate-1
                  do f=1,nopdof
                     if (f .ne. feb) then
                        hamilton(f,k2)=hamilton(f,ktot)
                     endif
                  enddo
               enddo

               ktot=ktot+nstate-1

            else if (optyp(k) .eq. 2) then
               call mvkend(k1)
!
! For Ss&s1 terms mirror to Ss1&s
!
               h    = hamilton(feb,ktot)
               hl   = hterm(1,h)
               ilbl = 2
               jlbl = index(hoplab(hl),'&')
               klbl = index(hoplab(hl),' ')-1
               read(hoplab(hl)(ilbl:jlbl-1),*) s
               read(hoplab(hl)(jlbl+1:klbl),*) s1

               k2 = ktot+1
               write(buffer,'(2(a,i5))') 'S',s1,'&',s
               call rmblnk(buffer)
               call hlabnum(hl,buffer)
               iterm(1) = hl
               nf = 1
! store new term
               call htmnum(h,iterm,nf,feb,0,0)
               hamilton(feb,k2) = h
               hamnum(k2)=hamnum(ktot)
               kact(k2)  = kact(ktot)
               ktwin(k2) = ktwin(ktot)
               s  = s +nstate
               s1 = s1+nstate
               do f = 1,nopdof
                  if (f .ne. feb) then
                     hamilton(f,k2) = hamilton(f,ktot)
                  endif
               enddo
               ktot = ktot+1

            else if (optyp(k) .eq. 5) then
               call mvkend(k1)
!
! For CSs&s1 terms mirror to Ss1&s
!
               h    = hamilton(feb,ktot)
               hl   = hterm(1,h)
               ilbl = 2
               jlbl = index(hoplab(hl),'&')
               klbl = index(hoplab(hl),' ')-1
               read(hoplab(hl)(ilbl:jlbl-1),*) s
               read(hoplab(hl)(jlbl+1:klbl),*) s1

               k2 = ktot+1

               write(buffer,'(2(a,i5))') 'CZ',s1,'&',s
               call rmblnk(buffer)
               call hlabnum(hl,buffer)

               h    = hamilton(febcont,ktot)
               nf = 1
               if (htmfac(h) .gt. 1) then
                  nf = htmfac(h)
                  if (sebcont(s1)) then    ! continuum now on left
                     iterm(1:nf-1) = hterm(2:nf,h) 
                     iterm(nf) = hl
                  else                    ! continuum now on right
                     iterm(1) = hl
                     iterm(2:nf) = hterm(1:nf-1,h)
                  endif
               else
                  iterm(nf) = hl
               endif
               call htmnum(h,iterm,nf,febcont,0,0)

               hamilton(febcont,k2) = h

               write(buffer,'(2(a,i5))') 'Z',s1,'&',s
               call rmblnk(buffer)
               call hlabnum(hl,buffer)
               iterm(1) = hl
               nf = 1
               call htmnum(h,iterm,nf,feb,0,0)

               hamilton(feb,k2) = h

               hamnum(k2)=hamnum(ktot)
               kact(k2)  = kact(ktot)
               ktwin(k2) = ktwin(ktot)
               s  = s +nstate
               s1 = s1+nstate
               do f = 1,nopdof
                  if (f .ne. feb .and. f .ne. febcont) then
                     hamilton(f,k2) = hamilton(f,ktot)
                  endif
               enddo
               ktot = ktot+1

            else
               k1=k1+1
            endif

         enddo
      endif

      return
      end subroutine sorteb

!#######################################################################
!
! DELZERO
!
! deletes any terms with a zero coefficient (which could be included
! in an operator
!
!#######################################################################

      subroutine delzero

      implicit none

      integer(long)                       :: k,k1,f,h,ktot1
      logical(kind=4), dimension(maxkoe)  :: delete

      routine='Delzero'

!-----------------------------------------------------------------------
! flag terms with zero operator/coefficient
!-----------------------------------------------------------------------
      do k=1,ktot
         delete(k)=.false.
      enddo
      do k=1,ktot
         if((dentype.eq.0) .or. (k.le.ktwin(k))) then
            do f=1,nopdof
               h=max(1,hamilton(f,k))
               if (htmtype(h) .eq. 0) then 
                  delete(k)=.true.
                  if (dentype.ne.0) delete(ktwin(k))=.true.
               endif
            enddo
         endif
      enddo

!-----------------------------------------------------------------------
! delete any terms that have been flagged
!-----------------------------------------------------------------------
      ktot1=ktot
      k1=1
      do k=1,ktot1
         if (delete(k)) then
            call mvkend(k1)
            ktot=ktot-1
         else
            k1=k1+1
         endif
      enddo

      return
      end subroutine delzero
               
!#######################################################################
!
! DELKE
!
! deletes any terms which are not real diagonal (i.e. potentials)
!
!#######################################################################

      subroutine delke

      implicit none

      integer(long)                      :: k,k1,f,h,ktot1
      logical(kind=4)                    :: diagtm
      logical(kind=4), dimension(maxkoe) :: delete

      routine='Delke'

!-----------------------------------------------------------------------
! flag terms which are not potential operators.
!-----------------------------------------------------------------------
      do k=1,ktot
         delete(k)=.false.
         do f=1,ndof
            if (f .ne. feb .and. f .ne. fpb .and. f .ne. febcont) then
               h=hamilton(f,k)
               diagtm=.false.
               if (htmtype(h) .eq. 1)   diagtm=.true.
               if (htmtype(h) .eq. 2)   diagtm=.true.
               if (htmtype(h) .eq. 101) diagtm=.true.
               if (htmtype(h) .eq. 103) diagtm=.true.
               if (htmtype(h) .eq. 104) diagtm=.true.
               if (htmtype(h) .eq. 105) diagtm=.true.
               if (htmtype(h) .eq. 202) diagtm=.true.

               if (.not. diagtm) delete(k)=.true.
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! delete any terms that have been flagged
!-----------------------------------------------------------------------
      ktot1=ktot
      k1=1
      do k=1,ktot1
         if (delete(k)) then
            call mvkend(k1)
            ktot=ktot-1
         else
            k1=k1+1
         endif
      enddo

      return
      end subroutine delke
               
!#######################################################################
!
! DELKEPNT
!
! deletes any terms which are not real diagonal (i.e. potentials)
! involving a point DVR
!
!#######################################################################

      subroutine delkepnt

      implicit none

      integer(long)                        :: k,k1,f,h,ktot1
      logical(kind=4)                      :: diagtm
      logical(kind=4), dimension(maxkoe)   :: delete

      routine='Delke'

!-----------------------------------------------------------------------
! flag terms which are not potential operators.
!-----------------------------------------------------------------------
      do k=1,ktot
         delete(k)=.false.
         diagtm=.true.
         do f=1,ndof
            if (basis(f) .eq. 19 .or. basis(f) .eq. 22) then
               h=hamilton(f,k)
               if (htmtype(h) .ne. 1 .and. htmtype(h) .ne. 2 .and. &
                   htmtype(h) .ne. 101 .and. htmtype(h) .ne. 103 .and. &
                   htmtype(h) .ne. 104) then
                  diagtm=.false.
               endif
            endif
         enddo
         if (.not. diagtm) delete(k)=.true.
      enddo

!-----------------------------------------------------------------------
! delete any terms that have been flagged
!-----------------------------------------------------------------------
      ktot1=ktot
      k1=1
      do k=1,ktot1
         if (delete(k)) then
            call mvkend(k1)
            ktot=ktot-1
         else
            k1=k1+1
         endif
      enddo

      return
      end subroutine delkepnt
               
!#######################################################################
!
! SORTKTMS
!
! packs hamiltonian term coefficients into operators, if possible
!
! GW,  updated version 02/2016 Falk Richter
!#######################################################################

      subroutine sortktms

      implicit none

      integer(long)       :: i,k,k1,n,n1,f,f1,h,hkoe,nh,h1,m

      routine='Sortktms'

!-----------------------------------------------------------------------
! pack any non-unit coefficients into 1d operators, if operator used
! only once, not unit operator,not Rf, Rfm, not feb/fpb, ...
! and not GWP dof.
! and not readmap operators.
!-----------------------------------------------------------------------
      do k=1,ktot
         if (hmlop(k).ne.0) cycle

         hkoe=hamilton(fkoe,k)
         if (htmtype(hkoe) .eq. 2) then ! coefficient differs from 1
            do m=1,nmode
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  h=hamilton(f,k)
                  if(h.le.0) then
                     goto 12  ! LMR term, exit k1-loop, next f
                  endif
                  if((h.ne.f).and.(f.ne.feb).and.(f.ne.fpb) &
                    .and.(htmtype(h).ne.31).and.(htmtype(h).ne.32) & ! Rf, Rfm
                    .and. (htmtype(h).ne.25) ) then   ! readmap
                     do k1=1,ktot
                        if(k1.eq.k) goto 11 !  cycle
                        if (hamilton(f,k1).eq.h) goto 12 ! h reappars, next f
                        do n1=1,nspfdof(m) ! maybe h hides in one of
                           f1=spfdof(n1,m) ! the LMR terms for that mode
                           h1=hamilton(f1,k1)
                           if (h1.lt.0) then ! there are LMR terms for that mode
                              do i=1,3 ! lets see if h hides anywhere here
!                              ! if identical h-term is found, try next f
                                 if (hamLMR(-h1,i).eq.h) goto 12
                              enddo
                           endif
                         enddo
 11                  enddo  ! k1
                  else ! h is LMR term or other unsuitable
                     goto 12  ! exit k1-loop, next f
                  endif

!  no identical operator term found, multiply coefficient to operator
                  if (htmform(h) .eq. 0) then
                     do i=1,htmfac(hkoe)
                        htmfac(h)=htmfac(h)+1
                        nh=htmfac(h)
                        hterm(nh,h)=hterm(i,hkoe)
                     enddo
                     hamilton(fkoe,k)=fkoe
                  else
                     htmfac(h)=htmfac(h)+1
                     nh=htmfac(h)
                     hterm(nh,h)=hkoe
                     hamilton(fkoe,k)=fkoe
                  endif
                  goto 13 ! exit f-loop, next k

 12            enddo  ! n (->f)
            enddo  ! m
         endif  ! htmtype
 13   enddo  ! k

      return
      end subroutine sortktms
               

!#######################################################################
!
! SUMHTMS
!
! sums up hterms, taking into account the type of operator.
!
! pass = 1 : only 1D (uncorrelated) operators to be summed
! pass = 2 : all products of 1D operators to be summed if they differ 
!            on only one mode
! pass = 3 : all operators to be summed if they differ on only one mode
!#######################################################################

      subroutine sumhtms(pass)

      implicit none

      integer(long)                                 :: k,f,h,m,n1,n,h1,nh,&
                                                       k1,ktot1,s,s1,hl,&
                                                       nh1,hl1,imlop,nrd
      integer(long), intent(in)                     :: pass
      integer(long), dimension(maxdim,maxkoe)       :: sumtyp
      integer(long), dimension(maxkoe)              :: funcorr,nonunit
      logical(kind=4), dimension(maxkoe)            :: delete
      logical(kind=4)                               :: lpot

!-----------------------------------------------------------------------
! flag terms that can be summed. 
! If sumtyp(f,k) = 0 then the operator hamilton(f,k) will not be summed 
! with other operators, i.e. Hamiltonian terms cannot be contracted by
! summing these operators. In general, only operators of the same type 
! are summed, hence sumtype = htmtype (see asshop for htmtype types)
!
! sumtyp(f,k) > 200 for real side-diagonal terms (encodes htmshift)
!-----------------------------------------------------------------------
      sumtyp=0
      h=0

      do k=1,ktot
         do f=1,nopdof
            h=hamilton(f,k)
! do not sum:
            if (h.lt.0) cycle                 ! LMR
            if (lreflex .and. f.eq.feb) cycle ! El DOF for reflex calc.

! find if h containes a natpot term
            call findnath(h,n,imlop) 

! set sumtype to equal htmtype. Treat unit operators as real diagonal.
            if (htmtype(h) .eq. 1) then
               sumtyp(f,k) = 2
            else if (htmtype(h) .eq. 2) then
               sumtyp(f,k) = htmtype(h)
            else if (htmtype(h) .ge. 3 .and. htmtype(h) .le. 4) then
               sumtyp(f,k) = 3
            else if (htmtype(h) .ge. 5 .and. htmtype(h) .le. 6) then
               sumtyp(f,k) = 5
            else if (htmtype(h) .ge. 7 .and. htmtype(h) .le. 10) then
               sumtyp(f,k) = htmtype(h)
            else if (htmtype(h) .ge. 15 .and. htmtype(h) .le. 23) then
               sumtyp(f,k) = htmtype(h)
            else if (htmtype(h) .eq. 30) then
               sumtyp(f,k) = htmtype(h)
            else if (htmtype(h) .eq. 12) then    !  real side-diagonal
               if (htmshift(h).lt.0) then
                  sumtyp(f,k)=200-2*htmshift(h)-1 ! 201,203,...
               elseif (htmshift(h).gt.0) then
                  sumtyp(f,k)=200+2*htmshift(h)   ! 202,204,...
               endif

            else if (htmtype(h) .eq. 101) then
               sumtyp(f,k) = htmtype(h)
            else if (htmtype(h) .eq. 103) then
               sumtyp(f,k) = htmtype(h)

            else if (htmtype(h) .lt. 0) then
               message='There is an unset htmtype.'
               routine='sumhtms'
               call errormsg

            endif

! special check for damping / flux functions or CAPS - do not sum
            if (htmtype(h) == 2) then
               if (htmdof(h) .eq. 0 ) cycle
               if (htmform(h) .eq. 1 ) cycle

               do nh=1,htmfac(h)
                  hl=hterm(nh,h)
                  if (hl .eq. 0) cycle

                  if (hoplab(hl)(1:5) == 'DAMP[' .or. &
                              hoplab(hl)(1:7) == 'fluxws[') then 
                      sumtyp(f,k)=0
                      exit 
                  endif
               enddo 
            else if (htmtype(h).eq.3) then       
               do nh=1,htmfac(h)
                  hl=hterm(nh,h)
                  if (hl .ne. 0) then
                     if (hoplab(hl)(1:4) .eq. 'CAP[') sumtyp(f,k)=0
                     if (hoplab(hl)(1:5) .eq. 'ACAP[') sumtyp(f,k)=0
                  endif
               enddo
            endif

            if (htmtype(h) .eq. 8) then
               call findread(h,nrd) ! check for read operators of mode type
               if(nrd.gt.0) sumtyp(f,k)=19
            endif

         enddo
      enddo

!-----------------------------------------------------------------------
! flag potential for lrealwp
!-----------------------------------------------------------------------
      if (lrealwp) then
         do k=1,ktot
            if (hamnum(k).eq.1) then
               lpot=.true.
               do m=1,nopmode
                  do n=1,nspfdof(m)
                     f=spfdof(n,m)
                     h=hamilton(f,k)
                     if (htmtype(h) .ne. 1 .and. htmtype(h) .ne. 2)  &
                          lpot=.false.
                  enddo
                  f=spfdof(1,m)
                  h=max(1,hamilton(1,k))   !!! LMR
                  if (htmmode(h) .ne. 1) lpot=.false.
               enddo
               if (lpot) then
                  do n=1,nspfdof(1)
                     f=spfdof(n,1)
                     if (htmadof(h) == 1) sumtyp(1,k)=0
                  enddo
               endif
            endif       !  hamnum(k)
         enddo          !  k-loop
      endif

! for real wavepacket propagation
      if (lrealwp) then
         do k=1,ktot
    outer:  do f=1,nopdof
               h=hamilton(f,k)
               if (hamnum(k) == 1 .and. sumtyp(f,k) /= 0) then
           innerr:do nh=1,htmfac(h)
                     hl=hterm(nh,h)
                     if (hl /= 0) then
                        if (htmform(h) == 0) then
                           if (hoplab(hl)(1:2) == 'KE') then
                              sumtyp(f,k)=0
                              exit outer
                           endif
                        else if (htmform(h) == 1) then
                           do nh1=1,htmfac(hl)
                              hl1=hterm(nh1,hl)
                              if (hoplab(hl1)(1:2) == 'KE') then
                                 sumtyp(f,k)=0
                                 exit outer
                              endif
                           enddo
                        endif
                     endif
                  enddo innerr
               endif
            enddo  outer
         enddo
      endif

!-----------------------------------------------------------------------
! If optimal contraction of terms wanted, sum real matrices and diagonal
! matrices
!-----------------------------------------------------------------------
      if (loptcontract) then
         do k=1,ktot
            do f=1,nopdof
               if (sumtyp(f,k) .eq. 8) sumtyp(f,k) = 2
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! look at coefficients and see if only one non-unit factor is present
! nonunit(k) then flags which dof contains constant.
! If more than one constant, nonunit(k)=0
! If total constant=1, nonunit(k)=fkoe
!-----------------------------------------------------------------------
      call zerovxi(nonunit,maxkoe)
      do k=1,ktot
         m=mkoe
         n1=0
         h=hamilton(fkoe,k)
! ignore if complex coefficient
         if (htmtype(h) .eq. 3) cycle
! ignore if LHA coefficient
         if (htmtype(h) .eq. 104) cycle
         if (htmform(h) .eq. 1) then
            nonunit(k)=fkoe
            do n=1,htmfac(h)
               h1=hterm(n,h)
               f = htmdof(h1)
               if (h1 .ne. f) nonunit(k)=0
            enddo
         else 
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               h=hamilton(f,k)
               if (h .ne. f) then
                  n1=n1+1
                  nonunit(k)=f
               endif
            enddo
            if (n1 .gt. 1) nonunit(k)=0
            if (n1 .eq. 0) nonunit(k)=fkoe
            if (ftime .gt. 0 .and. nonunit(k) .eq. ftime) nonunit(k)=0
         endif
      enddo

!-----------------------------------------------------------------------
! if first pass, only allow uncorrelated terms to be summed
!-----------------------------------------------------------------------
      call zerovxl(delete,maxkoe)
      if (pass .eq. 1) then
         do s=1,nstate
            do s1=1,nstate
               call flaguncorr(funcorr,s,s1)
               do f=1,ndof
                  call combhtm1d(f,funcorr,sumtyp,delete,nonunit)
               enddo
            enddo
         enddo
      else
         call combhtms(sumtyp,delete,nonunit,pass)
      endif
                 
!-----------------------------------------------------------------------
! delete any terms that have been summed
!-----------------------------------------------------------------------
      ktot1=ktot
      k1=1
      do k=1,ktot1
         if (delete(k)) then
            call mvkend(k1)
            ktot=ktot-1
         else
            k1=k1+1
         endif
      enddo

      return
      end subroutine sumhtms

!#######################################################################
!
! COMBHTM1d
!
! auxilliary routine to sum up 1D hamiltonian terms
!
! fcon : dof for which 1D terms will be summed
! funcorr(k) : array flagging 1D terms
! sumtyp(f,k) : types of operators - only same types are summed
!
!#######################################################################

      subroutine combhtm1d(fcon,funcorr,sumtyp,delete,nonunit)

      implicit none

      integer(long)                                       :: k,k1,n,nhf,nhf0,&
                                                             i,fk,h
      integer(long), intent(in)                           :: fcon
      integer(long), dimension(:), allocatable            :: iterm
      integer(long), dimension(maxkoe), intent(in)        :: funcorr,nonunit
      integer(long), dimension(maxdim,maxkoe), intent(in) :: sumtyp
      logical(kind=4), dimension(maxkoe), intent(inout)   :: delete
      logical(kind=4), dimension(maxkoe)                  :: sumhtm

! do not sum on electronic dof if multi-set
      if (lmult .and. fcon .eq. feb) return

outer:do k=1,ktot-1
         if (delete(k) .or. nonunit(k) .eq. 0) cycle outer
         if (funcorr(k) .ne. fcon) cycle outer
! combine hamiltonian terms only
         if (kact(k).ne.1) cycle outer

         call zerovxl(sumhtm,maxkoe)
         if (sumtyp(fcon,k) .eq. 0) cycle outer

! loop over terms, and see which can be summed. Sum if same sumtype on
! dof of interest
inner:   do k1=k+1,ktot
            if (delete(k1) .or. nonunit(k1) .eq. 0) cycle inner
! check action type
            if (kact(k) .ne. kact(k1)) cycle inner

! check if summation is allowed
            sumhtm(k1)=.true.
            if (funcorr(k1) .ne. fcon) sumhtm(k1)=.false.
            if (sumtyp(fcon,k1) .eq. 0) sumhtm(k1)=.false.
            if (sumtyp(fcon,k) .ne. sumtyp(fcon,k1))  &
                sumhtm(k1)=.false.
            if (hamnum(k) .ne. hamnum(k1)) sumhtm(k1)=.false.
            if (sumhtm(k1)) sumhtm(k)=.true.

         enddo inner

         if (.not. sumhtm(k)) cycle outer
! now sum terms
! first count "factors"
         nhf=0
         do k1=k,ktot
            if (sumhtm(k1)) then
               h=hamilton(fcon,k1)
               nhf0=nhf+1
               nhf=nhf+htmfac(h)
! multiply by coeff, if not 1.0
               fk=nonunit(k1)
               h=hamilton(fk,k1)
               if (fcon .ne. fk .and. h .ne. fk) then
                  nhf0=nhf+1
                  nhf=nhf+htmfac(h)
               endif

! add zero as plus marker
               nhf=nhf+1
            endif
         enddo

         allocate(iterm(nhf))

         nhf=0
         do k1=k,ktot
            if (sumhtm(k1)) then
               h=hamilton(fcon,k1)
               nhf0=nhf+1
               nhf=nhf+htmfac(h)
               i=0
               do n=nhf0,nhf
                  i=i+1
                  iterm(n)=hterm(i,h)
               enddo

! multiply by coeff, if not 1.0
               fk=nonunit(k1)
               h=hamilton(fk,k1)
               if (fcon .ne. fk .and. h .ne. fk) then
                  nhf0=nhf+1
                  nhf=nhf+htmfac(h)
                  i=0
                  do n=nhf0,nhf
                     i=i+1
                     iterm(n)=hterm(i,h)
                  enddo
                  hamilton(fk,k1)=fk
               endif

! add zero as plus marker
               nhf=nhf+1
               iterm(nhf)=0

               if (k1 .ne. k) delete(k1)=.true.
            endif
         enddo
         nhf=nhf-1

         call htmnum(h,iterm,nhf,fcon,0,0)
         hamilton(fcon,k)=h

         if (allocated(iterm)) deallocate(iterm)

      enddo outer

      return
      end subroutine combhtm1d
               
!#######################################################################
!
! COMBHTMS
!
! auxilliary routine to sum up hamiltonian terms
!
! if pass=2, only htmform(h)=0 present
! if pass=3, change to htmform(h)=1 if summed
!
! fcon = DOF for contraction, i.e. the DOF where the hamilton-line differs
!#######################################################################

      subroutine combhtms(sumtyp,delete,nonunit,pass)

      implicit none

      integer(long)       :: k,k1,n,f,fcon,nhf,nhf0,i,f1,htyp,shift,&
                             m,ndiff,h,hk,h1,h2,ndof1,&
                             maxsum,muld,nunit
      integer(long), intent(in)                           :: pass
      integer(long), dimension(:), allocatable            :: iterm
      integer(long), dimension(maxdim,maxkoe), intent(in) :: sumtyp
      integer(long), dimension(maxdim)                    :: nsum
      integer(long), dimension(maxkoe), intent(in)        :: nonunit
      logical(kind=4), dimension(maxkoe), intent(inout)   :: delete
      logical(kind=4), dimension(maxkoe)                  :: constant
      logical(kind=4), dimension(maxdim,maxkoe)           :: sumhtm
      integer(long)                                       :: deleted

      if(lddmctdh)return

! initialise variables
      shift = 0

! loop over this code until no more lines can be deleted (FO 11/2007)
   5  deleted=0

! flag any constants
      do k=1,ktot
         constant(k)=.false.
         nunit=0
         ndof1=0
         do m=1,nmode
            f=spfdof(1,m)
            h=hamilton(f,k)
            if(h.lt.0) goto 124  !!!!!!!!!! LMR
            if (htmform(h) .eq. 1) then
               do n=1,htmfac(h)
                  h1=hterm(n,h)
                  if (h1 .ne. 0) then
                     f1=htmdof(h1)
                     if (h1 .eq. f1) nunit=nunit+1
                     ndof1=ndof1+1
                  else
                     nunit=nunit+ndof
                  endif
               enddo
            else
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  ndof1=ndof1+1
                  if (hamilton(f,k) .eq. f) nunit=nunit+1
               enddo
            endif
         enddo
         if (nunit .eq. ndof1) then
            constant(k)=.true.
         endif
 124     continue
      enddo

      k=1
!-----------------------------------------------------------------------
! start loop over Hamiltonian terms
!-----------------------------------------------------------------------
   10 continue
      if (delete(k) .or. nonunit(k) .eq. 0) go to 100
      if (hmlop(k).ne.0) goto 100
      if (constant(k) .and. nodiag(hamnum(k)) .and. &
           hamilton(fkoe,k).eq.h00) goto 100

! combine hamiltonian terms only
      if (kact(k).ne.1) goto 100

! initialise sumhtm array
      call zeromxl(sumhtm,maxdim,maxkoe)
      call zerovxi(nsum,maxdim)
      do f=1,nopdof
         sumhtm(f,k)=.true.
      enddo

! loop over terms, and see which can be summed
outer:do k1=k+1,ktot
         if (delete(k1) .or. nonunit(k1) .eq. 0) cycle outer
         if (hmlop(k1).ne.0) cycle outer
         if (hamnum(k) .ne. hamnum(k1)) cycle outer
         if (constant(k) .neqv. constant(k1)) cycle outer
         if (kact(k) .ne. kact(k1)) cycle outer
         if (constant(k1) .and. nodiag(hamnum(k1)) .and. &
                         hamilton(fkoe,k1).eq.h00) then
            do f=1,ndof
               sumhtm(f,k1)=.false.
            enddo
            cycle outer
         endif

! count no. of differences between terms
         ndiff=0
         fcon=0
         do f=1,ndof
            if (hamilton(f,k) .ne. hamilton(f,k1)) then
               ndiff=ndiff+1
               fcon=f
            endif
         enddo

! if only terms are coeffs, contract on fkoe. 
         if (ndiff .eq. 0) then
            fcon=fkoe
            ndiff=1
         endif

! check if contraction is allowed
         if (lmult .and. fcon .eq. feb) cycle outer
         if (sumtyp(fcon,k) .ne. sumtyp(fcon,k1)) cycle outer
         if (sumtyp(fcon,k) .eq. 0) cycle outer

         h=hamilton(fcon,k)
         h1=hamilton(fcon,k1)
         if(h.lt.0 .or. h1.lt.0) cycle outer  !!! LMR

! check if coordinates are the same
         if (htmdof(h)  .ne. htmdof(h1)) cycle outer
         if (lrealwp) then
            continue
         else
            if (htmadof(h) .ne. htmadof(h1)) cycle outer
         endif
         if (htmndof(h) .ne. htmndof(h1)) cycle outer
         if (htmmode(h) .ne. htmmode(h1)) cycle outer
         if (htmmode(h).eq.0 .and. htmmode(h1).eq.0 .and. &
            (htmmuld(h) .ne. htmmuld(h1))) cycle outer

! readmap
         if(htmtype(h).eq.25) cycle outer

         if (ndiff .eq. 1) then
            sumhtm(fcon,k1)=.true.
            nsum(fcon)=nsum(fcon)+1

            if (fcon.eq.fkoe) then             ! If lines are identical
               do f=1,ndof                     ! one may sum on any DOF.
                  if( (hamilton(f,k1).ne.f) &  ! Do not sum on unit op.
                      .and. (f.ne.feb)&        ! Do not sum on state op.
                      .and. (f.ne.fpb) ) then  ! Do not sum on packet op.
                     sumhtm(f,k1)=.true.       ! HDM Okt 2006
                     nsum(f)=nsum(f)+1
                  endif
               enddo
            endif
         endif

      enddo outer    !     k1=k+1,ktot

! find which dof has maximum contraction
      fcon=0
      maxsum=0
      do f=1,nopdof
         if (nsum(f) .gt. maxsum) then
            fcon=f
            maxsum=nsum(f)
         endif
      enddo
      if (maxsum .eq. 0) go to 100

! now sum terms - first count maximum number of "factors"
      nhf=0
      do k1=k,ktot
         if (sumhtm(fcon,k1)) then
            h=hamilton(fcon,k1)
            if (pass .eq. 2 .or. &
                  (pass .eq. 3 .and. htmform(h) .eq. 1)) then
               nhf0=nhf+1
               nhf=nhf+htmfac(h)
            else if (pass .eq. 3 .and. htmform(h) .eq. 0) then
               nhf=nhf+1
            endif

! multiply by coeff, if not 1.0
            f=nonunit(k1)
            hk=hamilton(f,k1)
            if (fcon .ne. f .and. hk .ne. f) then
               if (pass .eq. 3) then
                  nhf=nhf+1
               else if (pass .eq. 2) then
                  nhf0=nhf+1
                  nhf=nhf+htmfac(hk)
               endif
            endif

! add zero as plus marker
            nhf=nhf+1
         endif ! sumhtm(fcon,k1)
      enddo    ! k1=k,ktot

      allocate(iterm(nhf))

! now sum terms
      nhf=0
      htyp=-2 ! HACK
      do k1=k,ktot
         if (sumhtm(fcon,k1)) then
            h=hamilton(fcon,k1)
            if(htmsym(hamilton(fkoe,k1)).eq.0) htmtype(h)=0 !re-compute htmtype
            if(htmsym(hamilton(fkoe,k1)).eq.-1) then  ! "I" in coefficient
               if(htmtype(h).eq.2) then               !
                  htmtype(h)=3                        !
               elseif(htmtype(h).eq.5) then           ! Added by HDM
                  htmtype(h)=6                        ! 11 Oct 2023
               elseif(htmtype(h).eq.8) then           !
                  htmtype(h)=9                        !
               elseif(htmtype(h).eq.12) then          !
                  htmtype(h)=13                       !
               else                                   !
                  htmtype(h)=0 !re-compute htmtype    !
               endif                                  !
            endif
!           if all summands have same htmtype and htmshift, inherit it (ok?)
            if (htyp.eq.-2) then ! first term
               htyp = htmtype(h)
               shift = htmshift(h)
            elseif (htyp.eq.htmtype(h) .and. shift.eq.htmshift(h)) then
               continue  ! do nothing
            else
               htyp = -1 ! flag: don't inherit type/shift
            endif

            if (pass .eq. 2 .or. &
                  (pass .eq. 3 .and. htmform(h) .eq. 1)) then
               nhf0=nhf+1
               nhf=nhf+htmfac(h)
               i=0
               do n=nhf0,nhf
                  i=i+1
                  iterm(n)=hterm(i,h)
               enddo
            else if (pass .eq. 3 .and. htmform(h) .eq. 0) then
               nhf=nhf+1
               iterm(nhf)=h
            endif

! multiply by coeff, if not 1.0
            f=nonunit(k1)
            hk=hamilton(f,k1)
            if (fcon .ne. f .and. hk .ne. f) then
               if (pass .eq. 3) then
                  nhf=nhf+1
                  iterm(nhf)=hk
                  hamilton(f,k1)=f
               else if (pass .eq. 2) then
                  nhf0=nhf+1
                  nhf=nhf+htmfac(hk)
                  i=0
                  do n=nhf0,nhf
                     i=i+1
                     iterm(n)=hterm(i,hk)
                  enddo
                  hamilton(fkoe,k1)=fkoe
               endif
            endif

! add zero as plus marker
            nhf=nhf+1
            iterm(nhf)=0

            if (k1 .ne. k) then
               delete(k1)=.true.
               deleted=deleted+1
            endif
         endif
      enddo
      nhf=nhf-1

! get new operator number
! coordinates must match original operator h
      h=hamilton(fcon,k)
      f=htmadof(h)
      m=htmmode(h)
      muld=htmmuld(h)
      call htmnum(h2,iterm,nhf,f,m,muld)
      if (htyp.gt.0) then
         htmtype(h2)=htyp
         htmshift(h2)=shift
      endif
      if (pass .eq. 2) then
         htmform(h2)=0
      else if (pass .eq. 3) then
         htmform(h2)=1
      endif
      hamilton(fcon,k)=h2

      if (allocated(iterm)) deallocate(iterm)

!-----------------------------------------------------------------------
! end loop over Hamiltonian terms
!-----------------------------------------------------------------------
  100 continue
      if (k .le. ktot-1) then
         k=k+1
         go to 10
      endif

! end outer loop
      if (deleted.ne.0) goto 5

      return
      end subroutine combhtms
               
!#######################################################################
!
! SORTMODE
!
! sorts out mult-dimensional hamiltonian terms (for combined modes).
!
! If all dofs in a mode have (side-)diagonal operators, multiply out as
! multi-dimensional (side-)diagonal operator (i.e. on full mode grid).
!
! sums up any terms if they differ on only one mode.
!
!#######################################################################

      subroutine sortmode

      implicit none

      integer(long)                                 :: k,n,f,m,h,f1,h1,&
                                                       nunit,m1,f2,nf,hl,&
                                                       nh,muld,shift
      integer(long), dimension(maxfac)              :: iterm
      integer(long), dimension(maxdim,maxkoe)       :: sumtyp
      logical(kind=4)                               :: ljm,lpot,lsd,lunit,&
                                                       lmuld

      routine='sortmode'
      call zeromxi(sumtyp,maxdim,maxkoe)
      f1=0
      shift=0
      
!-----------------------------------------------------------------------
! flag terms that can be treated as product terms.
! sumtyp(m,k) = 5  if side-diagonal term
! sumtyp(m,k) = 4  if muld potential
! sumtyp(m,k) = 3  if diagonal term in a full multi-d analytic potential
! sumtyp(m,k) = 2  if kleg * diagonal term
! sumtyp(m,k) = 1  if diagonal (potential) term
! sumtyp(m,k) = 0  otherwise
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! first mode operators, i.e. products that fall within one mode
! products of mode operators with unit operators are not to be 
! multiplied
!-----------------------------------------------------------------------
      do k=1,ktot
inner:   do m=1,nopmode
            if (nspfdof(m) .eq. 1) cycle inner
            sumtyp(m,k)=1

            nunit=0
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               h=hamilton(f,k)
               if (h .eq. f) then
                  nunit=nunit+1
               else
                  f1=f
               endif
            enddo
! don't multiply if merely product of a mode operator with unit operators
            if (nunit .eq. nspfdof(m)-1) then
               h=hamilton(f1,k)
               if(h.gt.0) then    !!!  LMR
                  m1=htmmode(h)
               else
                  m1=1
               endif
               if (m1 .ne. 0) sumtyp(m,k)=0
            endif

            do n=1,nspfdof(m)
               f=spfdof(n,m)
               h=hamilton(f,k)
               if(h.lt.0) then   !!! LMR
                  sumtyp(m,k)=0
                  cycle inner
               endif
               if (htmtype(h).lt.1 .or. (htmtype(h).gt.4 .and. &
                     htmtype(h).lt.150)) then
                  sumtyp(m,k)=0
               elseif (htmtype(h).eq.2) then
!..............Do not sum damping function or flux
                  do nh=1,htmfac(h)
                     hl=hterm(nh,h)
                     if (hl .ne. 0) then
                        if (hoplab(hl)(1:5) .eq. 'DAMP[' .or. &
                           hoplab(hl)(1:7) .eq. 'fluxws[') sumtyp(m,k)=0
                     endif
                  enddo
               elseif (htmtype(h).eq.3 .or. htmtype(h).eq.4) then
!..............Do not sum CAPs
                  do nh=1,htmfac(h)
                     hl=hterm(nh,h)
                     if (hl .ne. 0) then
                        if (hoplab(hl)(1:4) .eq. 'CAP[')&
                           sumtyp(m,k)=0
                        if (hoplab(hl)(1:5) .eq. 'ACAP[')&
                           sumtyp(m,k)=0
                     endif
                  enddo
               endif
            enddo

            if (nspfdof(m) .eq. 1) sumtyp(m,k)=0

         enddo inner
      enddo

!-----------------------------------------------------------------------
! flag products of kleg operators with a "potential"
!-----------------------------------------------------------------------
      do m=1,nopmode
         do n=1,nspfdof(m)
            f1=spfdof(n,m)
            if (basis(f1) .eq. 7 .or. basis(f1) .eq. 20) then
               f2=f1+1
               do k=1,ktot
                  h=hamilton(f1,k)
                  h1=hamilton(f2,k)
                  if(h.gt.0 .and. h1.gt.0) then  !!! LMR
                     call checkjm(h,ljm)
                     if (ljm) then
                        if (htmtype(h1).eq.2) sumtyp(m,k)=2
                     endif
                  endif
               enddo
            endif
         enddo
      enddo

!-----------------------------------------------------------------------------
! jpm, j_p, j_m, jp^2 and jm^2 treat as special operator separately for realwp
!-----------------------------------------------------------------------------
     if(lrealwp) then
outer:  do k=1,ktot
          do f=1,ndof
             if (basis(f) .eq. 7 .or. basis(f) .eq. 20) then
                h=hamilton(f,k)
                do nh=1,htmfac(h)
                   hl=hterm(nh,h)
                   if (hl .ne. 0) then
                      if (hoplab(hl)(1:3) == 'jpm' .or. &
                          hoplab(hl)(1:3) == 'j_p' .or. &
                          hoplab(hl)(1:3) == 'j_m' .or. &
                          hoplab(hl)(1:4) == 'jm^2' .or.&
                          hoplab(hl)(1:4) == 'jp^2') then
                          sumtyp(1,k)=6
                          cycle outer
                      endif
                   endif
                enddo
             endif
          enddo
       enddo outer
     endif

!-----------------------------------------------------------------------
! flag muld-potentials. If any potential in a term is a muld pot, then
! the whole term is treated as a muld.
!-----------------------------------------------------------------------
outer1: do k=1,ktot
         lmuld = .false.
         do f=1,ndof
            h=hamilton(f,k)
            if(h.lt.0) cycle outer1    !!! LMR
            muld=htmmuld(h)
            if (muld .gt. 0) lmuld = .true.
         enddo
         if (lmuld) sumtyp(:,k)=4
      enddo outer1

!-----------------------------------------------------------------------
! flag potential products for analytic surface (CDVR)
!-----------------------------------------------------------------------
!     if (lanalpes .and. (.not. lgwp .and. .not. ltraj)) then
!        do k=1,ktot
!           if (hamnum(k).eq.1) then
!              lpot=.true.
!              do m=1,nopmode
!                 do n=1,nspfdof(m)
!                    f=spfdof(n,m)
!                    h=hamilton(f,k)
!                    if (htmtype(h) .ne. 1 .and. htmtype(h) .ne. 2)  &
!                        lpot=.false.
!                 enddo
!                 f=spfdof(1,m)
!                 h=max(1,hamilton(1,k))   !!! LMR
!                 if (htmmuld(h) .ne. 0) lpot=.false.
!              enddo
!              if (lpot) then
!                 sumtyp(1,k)=3
!              endif
!           endif       !  hamnum(k)
!        enddo          !  k-loop

!-----------------------------------------------------------------------
!!if product multi-dimensional potential, multiply out in first dof
!-----------------------------------------------------------------------
!        do k=1,ktot
!           if (sumtyp(1,k) .eq. 3) then
!              nf=0
!              lunit=.true.
!              do m=1,nmode
!!do not include unit operators in product, unless the term is 
!!a multi-dimensional unit operator, in which case lunit=.true.
!                 do n=1,nspfdof(m)
!                    f=spfdof(n,m)
!                    h=hamilton(f,k)
!                    if (h .ne. f) then
!                       lunit=.false.
!                       nf=nf+1
!                       iterm(nf)=hamilton(f,k)
!                    endif
!                 enddo
!              enddo
!              if (lunit) then
!                 nf = 1
!                 f=spfdof(1,1)
!                 iterm(1) = f
!              endif

!              if (nmode .eq. 1) then
!                 call htmnum(h,iterm,nf,0,1,0)
!              else
!                 call htmnum(h,iterm,nf,0,0,1)
!              endif
!              htmform(h)=1

!              f=spfdof(1,1)
!              hamilton(f,k)=h
!              do f1=1,ndof
!                 if (f1 .ne. f) hamilton(f1,k)=f1
!              enddo
!           endif   !  sumtyp(1,k)
!        enddo      !  k-loop

!!check that fterm flag is set for newly summed potentials
!        do k=1,ktot
!           f=spfdof(1,1)
!           h=hamilton(f,k)
!           if (htmmuld(h) .ne. 0 .and. fterm(k) .eq. 0) then
!              fterm(k)=f
!              htmtype(h) = 2
!           endif
!        enddo

!     endif         !  lanalpes

!-----------------------------------------------------------------------
! flag products of side-diagonal operators
!-----------------------------------------------------------------------
      do k=1,ktot
inner1:  do m=1,nopmode
!           do nothing if already something else applies
            if (sumtyp(m,k).ne.0) cycle inner1
!           do nothing if only one dof in mode
            if (nspfdof(m).eq.1) cycle inner1
!           clear side-diagonal flag
            lsd=.false.
            nunit=0
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               h=hamilton(f,k)
               if(h.lt.0) cycle inner1 !!! LMR
!              unit operators are ok
               if (h.eq.f .or. htmtype(h).eq.1) then
                  nunit=nunit+1
                  continue
!              diagonal operators are ok
               elseif (htmtype(h).eq.2) then
                  continue
!              real side-diagonal operators are ok
               elseif (htmtype(h).eq.12) then
                  lsd=.true.
!              multiplication is also possible for imaginary/complex
!              side-diagonal matrices, but this is not implemented yet
               else
                  cycle inner1 
               endif
            enddo
!           check if there was really a side-diagonal operator
            if (lsd) then
               sumtyp(m,k)=5
            elseif (nunit.lt.nspfdof(m)-1) then
               write (message,'(a,a,i6,a,i3)') &
                  'Strange: found potential operator too late. ', &
                  'k =',k,', m =',m
               call warnmsg
               sumtyp(m,k)=1
            endif
         enddo inner1
      enddo

!-----------------------------------------------------------------------
! if product particle potential, multiply out in first dof of mode
!-----------------------------------------------------------------------
      do k=1,ktot
         do m=1,nopmode
!            if(lddmctdh)cycle
            if (sumtyp(m,k).eq.1 .or. sumtyp(m,k).eq.5) then
               nf=0
! do not include unit operators in product
! This is change from 8.3 in which they are explicitly included. This is
! unecessary and cumbersome if many dofs are present
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  h=hamilton(f,k)
                  if (h .ne. f .or. f .eq. fkoe) then
                     nf=nf+1
                     iterm(nf)=hamilton(f,k)
                  endif
               enddo
! special case of unit multi-D operator
               if (nf .eq. 0) then
                  nf=1
                  f=spfdof(1,m)
                  iterm(nf)=f
               endif

! nf is no. non-unit factors in term. For compatability with 8.3, need
! following

               if ((m.ne.mkoe .and. nspfdof(m) .gt. 1)  &
                   .or. nf .gt. 1) then
                  if (sumtyp(m,k).eq.5) then
!                    For products of side-diagonal operators we
!                    also need to know the resulting overall htmshift,
!                    so we can later sum terms with the same htmshift.
                     shift=0
                     do n=nspfdof(m),1,-1
                        f=spfdof(n,m)
                        h1=hamilton(f,k)
                        shift=gdim(f)*shift+htmshift(h1)
                     enddo
                  endif
! flag dofs after first as unit operators
                  call htmnum(h,iterm,nf,0,m,0)
                  htmform(h)=1
                  f=spfdof(1,m)
                  hamilton(f,k)=h
                  do n=2,nspfdof(m)
                     f=spfdof(n,m)
                     hamilton(f,k)=f
                  enddo
                  if (sumtyp(m,k).eq.5) then
!                    Set the htmtype already now. We need to flag
!                    products of side-diagonal operators somehow,
!                    because the creation of the product operator
!                    is too different from the case of diagonal
!                    operators and can't be done in the same routine.
                     htmtype(h)=12
                     htmshift(h)=shift
                  endif
               endif !  nf

            elseif (sumtyp(m,k).eq.2) then
!              different strategy for (kleg-op)x(diagonal):
!              only multiply these two terms, but leave the rest of
!              the mode untouched
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  if (basis(f).eq.7 .or. basis(f).eq.20) then
!                    check if the operator on kleg-theta is one of the
!                    special 2d-kleg operators
                     h=hamilton(f,k)
                     call checkjm(h,ljm)
                     if (ljm) then
!                       check if the operator on kleg-k is not unity
!                       but really diagonal
                        f1=f+1
                        h1=hamilton(f1,k)
                        if (h1.ne.f1) then
!                          create the product term h*h1 -> h
                           iterm(1)=h
                           iterm(2)=h1
                           nf=2
                           call htmnum(h,iterm,nf,f,m,0)
                           htmform(h)=1
!                          update the entries in the hamilton-array
                           hamilton(f,k)=h
                           hamilton(f1,k)=f1
                        endif
                     endif
                  endif
               enddo

            endif    !  sumtyp(m,k) 
         enddo       !  m-loop
      enddo          !  k-loop

!--------------------------------------------------------------
! coriolis terms for real wavepacket
!--------------------------------------------------------------
   if (lrealwp) then
      do k=1,ktot
         do m=1,nopmode
            if (sumtyp(m,k) == 6) then
               nf=0
               f1=0
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  h=hamilton(f,k)
                  if (h /= f .or. f == fkoe) then
                     if (basis(f) == 7 .or. basis(f) == 20) f1=f
                     nf=nf+1
                     iterm(nf)=hamilton(f,k)
                  endif
               enddo
               f=f1
               call htmnum(h,iterm,nf,0,m,0)
               htmform(h)=1
!              update the entries in the hamilton-array
               hamilton(f,k)=h
               do n=1,nspfdof(m)
                  f1=spfdof(n,m)
                  if (f1 /= f) hamilton(f1,k)=f1
               enddo
            endif
         enddo
      enddo
   endif
               
   return
   end subroutine sortmode

!#######################################################################
!
! checks and sorts muld terms except LHA-muld. 
!
!#######################################################################

      subroutine sortmuld

      implicit none

      integer(long)       :: muld,k,h,h1,f,m,m1,m2,gf
      logical(kind=4)     :: others,lmuld

outer:do k=1,ktot
         muld=0
         lmuld=.false.
         do f=1,ndof
            h1=hamilton(f,k)

            if(h1.lt.0) cycle outer   ! LMR
            if (htmmuld(h1) .gt. 0) then
               lmuld = .true.
               muld=htmmuld(h1)
               h=h1
            endif
         enddo

! not a muld
         if (.not. lmuld) cycle outer 

   ! muld operator found
! ---   Check if operator wrongly combined with other operators
         do gf=1,nmulddof(muld)
            f=mulddof2(gf,muld)
            if(hamilton(f,k).ne.h .and. hamilton(f,k).ne.f) then
               write(6,'(/a,4i5)') 'k, h, muld, f :',k,h,muld,f
               write(6,'(a,40i4)') 'hamilton:',(hamilton(m,k),m=1,ndof)
               write(6,'(a,2i4,2x,a)') 'hl, hopfile, hoplab ',  &
                    hterm(1,h), hopfile(hterm(1,h)), hoplab(hterm(1,h))
               routine='sortmuld (genoper/heinsort.F)'
               message='There must be no other operator operating '// &
                       'on the muld-DOFs.'
               call logop
               call errormsg
            endif
         enddo

! ---   Look to see if operator lies within one mode: set m1 and m2  ---
         m1=maxdim  ! lowest mode no. in operator
         m2=1

         do gf=1,nmulddof(muld)
            f=mulddof2(gf,muld)
            hamilton(f,k)=f  ! set all muld-dof to unit operator. Operator stored 
                             ! in correct place below
            if (basis(f) .eq. 19) cycle  ! ignore point DVRs

            m=dofspf(f)
            m1=min(m1,m)
            m2=max(m2,m)
         enddo

! Store potential as muld- or mode-operator.
! Note that if a mode operator, both htmmode and htmmuld are set as
! ordering of DOFs in operator differ from ordering in SPF. If they
! were the same it would have been flagged as a mode operator
! in heinterm and not passed to this routine.

         if(m1.eq.m2) then      !  mode operator
            htmmode(h)=m1
            f=spfdof(1,m1)   ! first DOF in mode
            htmadof(h)=f     ! set active DOF
            fterm(k) = f     ! set where muld term is stored in H
            hamilton(f,k)=h
         else if (fterm(k) .eq. 0) then
            f=mulddof(1,muld)   ! first DOF in dynamical system
            htmadof(h)=f     ! set active DOF
            fterm(k) = f     ! set where muld term is stored in H
            hamilton(f,k)=h
         else
            f=fterm(k)
            htmadof(h)=f     ! set active DOF
            hamilton(f,k)=h     !  muld operator
         endif
         htmform(h)=0

! ---   Other operators not allowed with a muld term: Check ---
         if (htmmode(h) .eq. 0) then
            others=.false.
            do f=1,ndof
               if (f.eq.feb) cycle
               if (f.eq.fterm(k)) cycle
               if(hamilton(f,k).ne.f) others=.true.
            enddo

            if(others) then
               routine='sortmuld'
               message="There must be no other operators in a "// &
                 "Hamiltonian line (except multiset state operators) "// &
                 "together with a muld potential!"
               call errormsg
            endif
         endif
      enddo outer            ! k-loop

      return
      end subroutine sortmuld


!#######################################################################
!
! If analytic potential is required, flag all mode and grid terms as 
! analytic 
!
! time-dependent terms are also flagged
!
! GWP modes require analytic potentials
!#######################################################################

      subroutine sortanal

      implicit none

      integer(long)       :: h,n,h1,m,f,f1,sym,md,&
                             found,maxn,hl,nh
      logical(kind=4)     :: lunit

      if (lanalpes) then
         do h=1,min(htmdim,maxhtm)
            if (htmmuld(h) .ne. 0) htmtype(h)=101
            if (htmmode(h) .ne. 0) then
               m = htmmode(h)
               if (gwpm(m)) htmtype(h)=101
            endif
         enddo
      endif

      sym=1
      if (ftime .ne. 0) then
         do h=1,min(htmdim,maxhtm)
            if (htmmode(h) .eq. mkoe) then
               do n=1,htmfac(h)
                  h1=hterm(n,h)
                  if (h1 .ne. 0 .and. htmdof(h1) .eq. ftime) then
                     if (htmtype(h1) .eq. 151) then
                        htmtype(h)=151
                     else if (htmtype(h1) .eq. 161) then
                        htmtype(h)=161
                     else if (htmtype(h1) .ne. 1) then
                        if (htmtype(h1) .eq. 2) then
                           htmsym(h1)=1
                        else if (htmtype(h1) .eq. 3) then
                           htmsym(h1)=-1
                        else if (htmtype(h1) .eq. 4) then
                           htmsym(h)=0
                        endif
                        htmtype(h1)=101
                        htmtype(h)=101
                        sym=sym*htmsym(h1)
                        htmsym(h)=sym
                     endif
                  else if (h1 .ne. 0) then
                     sym=sym*htmsym(h1)
                  endif
               enddo
            endif
         enddo
      endif

! for gwp and classical modes, flag potentials as analytic
!  Note that for GWPs, unit operators are type 101 as operators are not 
!  diagonal 
      if (lgwp) then
         do h=1,htmdim
            f=htmdof(h)
            m=htmmode(h)
            md=htmmuld(h)

! get dof where operator is stored
            if (m .gt. 0 .and. f .eq. 0) then
               f=spfdof(1,m)
            else if (md .gt. 0) then
               f=1
            endif
            if (basis(f) .ne. 25) cycle
            if (f .eq. fkoe) cycle

! multi-D potentials
            if (md .ne. 0) then
               htmtype(h)=104

            else 
! product mode operators
               if (htmform(h) .eq. 1) then
! see if operator is single term which is a product of unit operators
                lunit = .true.
                do nh=1,htmfac(h)
                   h1=hterm(nh,h)
                   if (h1 .eq. 0) then
                      lunit=.false.
                   else if (h1 .ne. htmdof(h1)) then
                      lunit=.false.
                   endif
                enddo
                if (lunit) then
                   htmtype(h)=103
                else
                   htmtype(h)=101
                endif

                  htmsym(h)=1
! single DOF potentials
               else if (htmtype(h) .eq. 1) then
                  htmtype(h)=103
               else if (htmtype(h) .eq. 2) then
                  htmtype(h)=101
                  htmsym(h)=1
               endif

            endif
         enddo
      endif
      if (ltraj .or. lxloc) then
         do h=1,htmdim
! cannot have multi-D for a trajectory
            f=htmdof(h)
            m=htmmode(h)

! get dof where operator is stored
            if (m .gt. 0 .and. f .eq. 0) then
               f=spfdof(1,m)
            endif
            if (basis(f) .ne. 27) cycle
            if (f .eq. fkoe) cycle

            if (htmform(h) .eq. 1) then

! see if operator is single term which is a product of unit operators
                lunit = .true.
                do nh=1,htmfac(h)
                   h1=hterm(nh,h)
                   if (h1 .eq. 0) then
                      lunit=.false.
                   else if (h1 .ne. htmdof(h1)) then
                      lunit=.false.
                   endif
                enddo
                if (lunit) then
                   htmtype(h)=103
                else
                   htmtype(h)=101
                endif

             else if (htmtype(h) .eq. 2) then
                htmtype(h)=101
             else if (htmtype(h) .eq. 1) then
                htmtype(h)=103
             endif
         enddo
      endif

!-----------------------------------------------------------------------
! look to see if potential has analytic derivatives
!
! only simple 1D functions (no products of functions) 
! loop scans 1D hterms and checks that each term contains only 1 function
! with an analytic derivative (can have any number of constants in file 
! 301).
!
! momentum terms in trajectory calcs always with analytic derivatives
!-----------------------------------------------------------------------
      do h=1,htmdim
         htmderiv(h)=0
         f=htmdof(h)
    
         if (f .ne. 0) then
            n=0
            maxn=0
            do nh=1,htmfac(h)
               hl=hterm(nh,h)
               if (hl .ne. 0) then
                  if (hopfile(hl) .eq. 302) then
                     call fdefderd(hoplab(hl),found)
                     if (lanalderiv) n=n+found
                  else if (hopfile(hl) .eq. 307) then
                     call ddefxloc(hoplab(hl),found)
                     n=n+found
                  else if (hopfile(hl) .ne. 301) then
                     n=n+999
                  endif

               else
                  maxn=max(maxn,n)
                  n=0
               endif
            enddo
            maxn=max(maxn,n)
            if (maxn .eq. 1) htmderiv(h)=1
         endif
      enddo

! now look at product hterms
! factor must have analytic derivative, or else is a coefficient
      do h=1,htmdim
         if (htmform(h) .eq. 1) then
            f=0
            m=htmmode(h)
            if (m .eq. 0) cycle
            f=spfdof(1,m)
            if (f .ne. 0 .and. gwpm(m)) then
               htmderiv(h)=1
               do nh=1,htmfac(h)
                  h1=hterm(nh,h)
                  if (h1 .ne. 0) then
                      f1 = htmdof(h1)
                      if (htmderiv(h1) .eq. 0) then
                         if (f1 .ne. fkoe) htmderiv(h) = 0
                      endif
                  endif
               enddo
            endif
         endif
      enddo
!
! If a DD or Shepard PES flag as analytic derivative
! Also POTLIB NH3 surface in Cartesians
!  (2nds are actually by differencing firsts,
!  but we currently have no general code for this case).
!
      do h=1,htmdim
         if (htmform(h) .eq. 1) cycle

         m=htmmode(h)
         md=htmmuld(h)

         if (m .ne. 0 .or. md .ne. 0) then
            do nh=1,htmfac(h)
               hl=hterm(nh,h)
               if (hl .ne. 0) then
                  if (hopfile(hl) .eq. 205) htmderiv(h)=1
                  if (hopfile(hl) .eq. 206) htmderiv(h)=1
                  if ((hopfile(hl)==204).and.(hopilab(hl)==19)) htmderiv(h)=1
                  if (hopfile(hl) .eq. 208) htmderiv(h)=1
                  if (hopfile(hl) .eq. 209) htmderiv(h)=1
               endif
            enddo
         endif
      enddo
 
!
! if numerical derivatives explicitly asked for, 
!
      if (.not. lanalderiv) then
         do h=1,htmdim
            if (htmform(h) .eq. 1) cycle
            m=htmmode(h)

            if (m .ne. 0) then
               do nh=1,htmfac(h)
                  hl=hterm(nh,h)
                  if (hl .ne. 0) then
                     if (hopfile(hl) .eq. 205) htmderiv(h)=1
                     if (hopfile(hl) .eq. 206) htmderiv(h)=1
                     if ((hopfile(hl)==204).and.(hopilab(hl)==19)) htmderiv(h)=0
                     if (hopfile(hl) .eq. 208) htmderiv(h)=1
                     if (hopfile(hl) .eq. 209) htmderiv(h)=1
                  endif
               enddo
            endif
         enddo
      endif


      return
      end subroutine sortanal


!#######################################################################
!
! DELKG(f1,f2)
!
! deletes any terms which do not contain a differential operator
! in both DOF f1 and DOF f2. The differential operators are then
! replaced by unit operators.
! HDM 10/06
!#######################################################################

      subroutine delkg(f1,f2)

      implicit none

      integer(long)                      :: k,k1,f,h,ktot1
      integer(long), intent(in)          :: f1,f2
      logical(kind=4)                    :: diagtm,diftm1,diftm2
      logical(kind=4), dimension(maxkoe) :: delete

      routine='Delkg'

      write(ilog, '(/a,2i5,4x,a8,2x,a8)') 'DOFs used by gengmat:', &
                  f1,f2,modelabel(f1),modelabel(f2)
      if(f1.le.0 .or. f2.le.0 .or. f1.gt.ndof .or. f2.gt.ndof) then
         message='Illegal values for the DOFs used by gengmat.'
         call errormsg
      endif

!-----------------------------------------------------------------------
! flag terms which do not contain a differential operator in f1 and f2.
!-----------------------------------------------------------------------
      do k=1,ktot
         delete(k)=.false.
         diagtm=.true.
         diftm1=.false.
         diftm2=.false.
         if(f1.eq.f2) diftm2=.true.
         do f=1,ndof
            h=hamilton(f,k)
            if (f .eq. feb) then
               continue
            elseif(f.eq.f1) then
               if(htmtype(h).ge.5 .and. htmtype(h).le.10) then
                  diftm1=.true.
               endif
            elseif(f.eq.f2) then
               if(htmtype(h).ge.5 .and. htmtype(h).le.10) then
                  diftm2=.true.
               endif 
            else
               if(htmtype(h).ne.1 .and. htmtype(h).ne.2 .and.  &
               htmtype(h).ne.11 .and. htmtype(h).ne.101) diagtm=.false.
            endif
         enddo
         if (.not.(diagtm.and.diftm1.and.diftm2)) delete(k)=.true.
      enddo

!-----------------------------------------------------------------------
! delete any terms that have been flagged
!-----------------------------------------------------------------------
      ktot1=ktot
      k1=1
      do k=1,ktot1
         if (delete(k)) then
            call mvkend(k1)
            ktot=ktot-1
         else
            k1=k1+1
         endif
      enddo

!-----------------------------------------------------------------------
! remove differential operator in f1 and f2.
!-----------------------------------------------------------------------
      do k=1,ktot
         diftm1=.false.
         diftm2=.false.
         do f=1,ndof
            h=hamilton(f,k)
            if (f .eq. feb) then
               continue
            elseif(f.eq.f1 .or. f.eq.f2) then
               if(htmtype(h).ge.5 .and. htmtype(h).le.10) then
                  call rmdq(f,h)
                  htmtype(h)=2
               endif
            endif
         enddo
      enddo

      return
      end subroutine delkg

!#######################################################################
!     Removes dq and dq^2 operators. For GMAT.
!#######################################################################

      subroutine rmdq(f,h)


      implicit none

      integer(long)             :: hl,n
      integer(long), external   :: slen
      integer(long), intent(in) :: f,h

      do n=1,htmfac(h)
         hl=hterm(n,h)
         if (hl .ne. 0) then
            if(hoplab(hl)(1:4) .eq. 'sdq2') then
               hoplab(hl)='sin^2'
               hopipar(1,hl)=2
               hopilab(hl)=4
               hopfile(hl)=302
            elseif(hoplab(hl)(1:4) .eq. 'cdq2') then
               hoplab(hl)='cos^2'
               hopipar(1,hl)=2
               hopilab(hl)=6
               hopfile(hl)=302
            elseif(hoplab(hl)(1:4) .eq. 'csdq') then
               hoplab(hl)='sincos'
               hopipar(1,hl)=1
               hoprpar(2,hl)=1.0_dop
               hoprpar(3,hl)=0.0_dop
               hoprpar(4,hl)=1.0_dop
               hoprpar(5,hl)=0.0_dop
               hopilab(hl)=40
               hopfile(hl)=302
            elseif(hoplab(hl)(1:4) .eq. 'uqdq') then
               hoplab(hl)='usincos'
               hopipar(1,hl)=1
               hoprpar(2,hl)=1.0_dop
               hopilab(hl)=41
               hopfile(hl)=302
            elseif(hoplab(hl)(1:4) .eq. 'udq2') then
               hoplab(hl)='qs^2'
               hopipar(1,hl)=2
               hoprpar(1,hl)=2.0_dop
               hoprpar(2,hl)=1.0_dop
               hopilab(hl)=19
               hopfile(hl)=302
            elseif(hoplab(hl)(1:3) .eq. 'sdq') then
               hoplab(hl)='sin'
               hopipar(1,hl)=1
               hopilab(hl)=4
               hopfile(hl)=302
            elseif(hoplab(hl)(1:3) .eq. 'cdq') then
               hoplab(hl)='cos'
               hopipar(1,hl)=1
               hopilab(hl)=6
               hopfile(hl)=302
            elseif(hoplab(hl)(1:3) .eq. 'qdq') then
               hoplab(hl)='q'
               hopipar(1,hl)=1
               hoprpar(1,hl)=1.0_dop
               hoprpar(2,hl)=0.0_dop
               hopilab(hl)=3
               hopfile(hl)=302
            elseif(hoplab(hl)(1:3) .eq. 'udq') then
               hoplab(hl)='qs'
               hopipar(1,hl)=1
               hoprpar(1,hl)=1.0_dop
               hoprpar(2,hl)=1.0_dop
               hopilab(hl)=19
               hopfile(hl)=302
            elseif(hoplab(hl)(1:2) .eq. 'KE') then
               hoplab(hl)='i_mass_'//modelabel(f)(1:slen(modelabel(f)))
               hopipar(1,hl)=1
               hopilab(hl)=2
               hopfile(hl)=301
               hoprpar(1,hl)= -1.0_dop/(2.0_dop*mass(f))
            elseif(hoplab(hl)(1:2) .eq. 'dq') then
               hoplab(hl)='1'
               hopipar(1,hl)=1
               hopilab(hl)=1
               hopfile(hl)=302
            endif
         endif
      enddo

      return
      end subroutine rmdq

!#######################################################################

      subroutine sortlha

      implicit none

      integer(long)                    :: m,k,f,h,h1,hl,itmp,ipower,n,nh,nh1
      integer(long), dimension(maxdim) :: itmpf

!-----------------------------------------------------------------------
! for each Hamiltonian term get moment and check if .le. to lhaorder
! If it is the integral is evaluated exactly, if not by LHA
! power of q^n is stored in hopipar(1,hl)
!-----------------------------------------------------------------------
      do m=1,nmode
         if (.not. gwpm(m)) cycle

         do k=1,ktot
            ipower=0

            f=spfdof(1,m)
            h=hamilton(f,k)

! only look at GWP potentials
            if (htmtype(h) .ne. 101) cycle

            if (htmmuld(h) .ne. 0) then
               ipower = 0

            else if (htmform(h) .eq. 0) then

               ipower = 0
               itmp = 0
               hl = 0
               do nh=1,htmfac(h)+1
                  if (nh .le. htmfac(h)) hl=hterm(nh,h)
                  if (hl .eq. 0 .or. nh .gt. htmfac(h)) then
                     ipower=max(ipower,itmp)
                     itmp = 0
                  else
                     if (hopfile(hl) .eq. 301) then ! parameter
                        continue
                     else if (hopfile(hl) .eq. 302) then 
                        if (hopilab(hl) .eq. 1) then ! 1
                           continue
                        else if (hopilab(hl) .eq. 2) then ! -1
                           continue
                        else if (hopilab(hl) .eq. 3) then ! q^n
                           itmp=itmp+hopipar(1,hl)
                        else ! other operator
                           itmp=itmp+lhaorder+1
                        endif
                     else    ! other operator
                        itmp=itmp+lhaorder+1

                     endif

                  endif
               enddo

            else if (htmform(h) .eq. 1) then

               ipower = 0
               itmpf = 0
               hl = 0
               do nh=1,htmfac(h)+1
                  if (nh .le. htmfac(h)) h1=hterm(nh,h)

                  if (h1 .eq. 0 .or. nh .gt. htmfac(h)) then
                     itmp=0
                     do n=1,nspfdof(m)
                        f=spfdof(n,m)
                        itmp = itmp + itmpf(f)
                     enddo
                     ipower=max(ipower,itmp)
                     itmpf = 0
                  else
                     itmp=0
                     do nh1=1,htmfac(h1)+1
                        if (nh1 .le. htmfac(h1)) hl = hterm(nh1,h1)
                        f = htmdof(h1)
                        if (hl .eq. 0 .or. nh1 .gt. htmfac(h1)) then
                           itmpf(f)=max(itmpf(f),itmp)
                           itmp=0
                        else
                           if (hopfile(hl) .eq. 301) then ! parameter
                              continue
                           else if (hopfile(hl) .eq. 302) then 
                              if (hopilab(hl) .eq. 1) then ! 1
                                 continue
                              else if (hopilab(hl) .eq. 2) then ! -1
                                 continue
                              else if (hopilab(hl) .eq. 3) then ! q^n
                                 itmp=itmp+hopipar(1,hl)
                              else ! other operator
                                 itmp=itmp+lhaorder+1
                              endif
                           else    ! other operator
                              itmp=itmp+lhaorder+1
                           endif
                        endif

                     enddo

                  endif
               enddo
            endif

! Muld potentials always use LHA
! Else LHA if order of term higher than lhaorder
            if (htmmuld(h) .ne. 0) then
               htmtype(h) = 104

            else if (ipower .gt. lhaorder) then
               htmtype(h) = 105
            endif
         enddo
      enddo

      return
      end subroutine sortlha

!#######################################################################
      end module heinsort


