!
!              MCTDH  module HEINTERM
!
! MCTDH module to read and sort the data specifying the Hamiltonian
!
!            Subroutines:   heinterms, deinterms, renterm, storetm,
!
! Reads the Hamiltonian terms.
! Called from: heingabe
! Calls to: warnmsg, errormsg
!
! 03/97 HDM
!
! kignore is a logical for the 1D terms, that if the term includes a ghost mode
!  it is ignored.
! mham: mode no. in hamiltonian file. Needed if certain modes defined in
!       hamiltonian are ignored or reordered in calculation
!       mh=mham(m) means that mode m in the calculation is mode mh in the
!       Hamiltonian.
!
!#######################################################################

      module heintermmod

use decimal, only: long, dop
use logdat
use op1lib,  only: zerovxi, zerovxl

use global
use maxv, only: maxdim,maxkoe,maxhtm
use dvrdatmod
use griddatmod
use operdef
use runopermod
use channels

use heinutil, only: muldnum,hlabnum,htmnum,gethnum
use operutil, only: rdfactors,mvkend


      implicit none
      private
      public :: heinterms,deinterms,storetm,renterm,hintgen
      contains

!#######################################################################

      subroutine heinterms(iunit,loperr,iz,nham)

      implicit none

      integer(long)        :: ic,ierr,i,k,k1,n,m,mh,mh1,f,f1,nhmode,&
                              gf,gf1,ktot1,nfac,h,ilbl,jlbl,muld,mode,&
                              j2,klbl,mh2,llbl,iorder,j,j1,iz0
      integer(long)        :: mm,ff,nLMR,mmLMR,kkLMR

      integer(long), intent(in)              :: iz
      integer(long), intent(in)              :: nham,iunit
      integer(long), dimension(24)           :: optyp
      integer(long), dimension(maxdim)       :: mham,grid
      character(len=c2), dimension(24)       :: factor
      logical(kind=4)                        :: lnum,lnonum,lreadcoeff
      logical(kind=4), dimension(maxkoe)     :: kignore
      logical(kind=4), intent(out)           :: loperr

      routine = 'Heinterms (reading operator-file)'
      lwarn = .false.

      iz0 = 0
      mmLMR=0
      kkLMR=0

      call zerovxl(kignore,maxkoe)
      call zerovxi(mham,maxdim)    ! mapping of system DOFs to Hamiltonian
      call zerovxi(grid,maxdim)    ! set of coords for muld operators
      do m=1,nmode
         laddmode(m,nham) = .true.
      enddo

!-----------------------------------------------------------------------
! if hoplab needs to be initialised then:
! place unit operator at beginning of list for each degree of freedom
! place +1, -1 at beginning of list for coefficients
!-----------------------------------------------------------------------
      if (hopdim .eq. 0) then
         hoplab = ' '

         hopdim=3
         if (maxhop .ge. 3) then
            hoplab(1)='1'
            hoplab(2)='-1'
            hoplab(3)='I'
         endif

         do f=1,nopdof
            htmdim=htmdim+1
            h=min(htmdim,maxhtm)
            hterm(1,h)=1
            htmfac(h)=1
            htmdof(h)=f
            htmadof(h)=f
            htmtype(h)=1
            htmsym(h)=1
         enddo

         k=0
      else
         k=ktot
         do k1=k+1,maxkoe
            do f=1,maxdim
               hamilton(f,k1)=0
            enddo
         enddo
      endif

      mh=0
      nhmode=0
      lreadcoeff=.false.

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
 12   continue
      call rdinpf(iunit,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'heinterms, heinterm.F'
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)
      i=1

 14   continue
      if (keyword(i) .eq. 'modes') then
         lreadcoeff=.false.
         goto  13

!-----------------------------------------------------------------------
! look for nodiag keyword
!-----------------------------------------------------------------------
      else if (keyword(i) .eq. 'nodiag') then
         nodiag(nham) = .true.
      else if (keyword(i) .eq. 'usediag') then
         nodiag(nham) = .false.

!-----------------------------------------------------------------------
! look for addmode keyword
!-----------------------------------------------------------------------
      else if( keyword(i) .eq. 'addmode' ) then
         i=i+1
         if( keyword(i) .eq. '=' ) i=i+1
 17      f=0
         do f1 = 1, ndof
            if( keyorig(i) .eq. modelabel(f1) ) f = f1
         enddo
         if( f .eq. 0 ) then
            message='modelabel not found!'
            call wrline(zeroi,iz,iz0)
         else
            nopmode=nopmode+1
            m=nopmode
            nspfdof(m)=1
            spfdof(1,m)=f
            dofspf(f)=m
            laddmode(m,nham)=.true.
         endif
         if( keyword(i+1) .eq. ',' ) then
            i=i+2
            goto 17
         endif
      endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 14
      else
         go to 12
      endif           

!-----------------------------------------------------------------------
! READ MODE KEYWORDS:
!-----------------------------------------------------------------------
 10   continue
      call rdinpf(iunit,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      routine = 'heinterms, heinterm.F line 160'
      call cntrl(keyword(1),lc(1),ierr)
! --- Check for continuation line "&&&"
      if(keyword(1) .eq. '&&&') then
         j=2
         if (keyword(j)(1:1) .ne. '|') then
            routine='heinterms (reading operator-file, line 183)'
            message='Wrong format of continuation line! '// &
                    'Expecting " &&&  |....."'
            call errormsg
         endif
         if(.not.lreadcoeff) then
            routine='heinterms (reading operator-file, line 191)'
            message='There is no line to continue! '// &
                    'Only operator lines can be continued.'
            call errormsg
         endif
         goto 15
      endif

      lnum = .false.
      lnonum = .false.

!-----------------------------------------------------------------------
! first read mode labels
!-----------------------------------------------------------------------
 13   if (keyword(1) .eq. 'modes') then
         j=2
         do i=2,ic-1,2
            if (keyword(j)(1:1) .eq. '|') then
               nhmode=nhmode+1

! add time dof if required (was set to -1 in mdefault.F)
               f=0
               if (keyorig(j+1) .eq. 'Time' .and. ftime .lt. 0) then
                  nopdof=nopdof+1
                  ftime=nopdof
                  modelabel(ftime)='Time'
                  gdim(ftime)=1
                  f=ftime
                  nspfdof(mkoe)=nspfdof(mkoe)+1
                  n=nspfdof(mkoe)
                  spfdof(n,mkoe)=ftime
                  dofspf(ftime)=mkoe

                  htmdim=htmdim+1
                  h=min(htmdim,maxhtm)
                  hterm(1,h)=1
                  htmfac(h)=1
                  htmdof(h)=f
                  htmadof(h)=f

               else if (keyorig(j+1) .eq. 'Time'  &
                          .and. ftime .gt. 0) then
                  f=ftime

               else
                  do f1=1,ndof
                     if (keyorig(j+1) .eq. modelabel(f1)) f=f1
                  enddo
               endif

               if (f .eq. 0) then
                  message= &
           'Operator mode not found in system :'//keyorig(j+1)
                  call wrline(zeroi,iz,iz0)
               else
                  mham(nhmode)=f
               endif

               j=j+2
            else
               message='Wrong structure of Mode keyword'
               call wrline(zeroi,iz,iz0)
            endif
         enddo
      
         go to 10
      endif

!-----------------------------------------------------------------------
! check that modes have been defined 
!-----------------------------------------------------------------------
      if (nhmode .eq. 0) then
         routine='heinterms (reading operator-file)'
         message='Operator mode order not defined'
         call wrline(zeroi,iz,iz0)
      endif

!-----------------------------------------------------------------------
! set up full grid in operator dofs
!-----------------------------------------------------------------------
      gf1=0
      do gf=1,nhmode
         f1=mham(gf)
         if (f1 .eq. ftime) f1=0
         if (f1 .eq. feb) f1=0
         if (f1 .ne. 0) then
            gf1=gf1+1
            grid(gf1)=f1
         endif
      enddo
      call muldnum(muld,grid,mode)

      backspace(iunit)
!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
 100  continue
      call rdinpf(iunit,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      routine = 'heinterms, heinterm.F line 160'
      call cntrl(keyword(1),lc(1),ierr)
! --- Check for continuation line "&&&"
      if(keyword(1) .eq. '&&&') then
         j=2
         if (keyword(j)(1:1) .ne. '|') then
            routine='heinterms (reading operator-file, line 183)'
            message='Wrong format of continuation line! '// &
                    'Expecting " &&&  |....."'
            call errormsg
         endif
         if(.not.lreadcoeff) then
            routine='heinterms (reading operator-file, line 191)'
            message='There is no line to continue! '// &
                    'Only operator lines can be continued.'
            call errormsg
         endif
         goto 15
      endif

      lnum = .false.
      lnonum = .false.

!-----------------------------------------------------------------------
! Check for '[' or '{' . Only simple operators are allowed here.
!-----------------------------------------------------------------------
      routine = 'Heinterms (reading operator-file)'
      j1=index(inptit,'[')
      j2=index(inptit,'{')
      if( max(j2,j1) .gt. 1 ) then
         if(j1 .le. 0 ) j1 = j2
         if(j2 .le. 0 ) j2 = j1
         j2=min(j1,j2)
         j1=index(inptit,'#')
         if( j1 .le. 0 .or.  j1.gt.j2 ) then
            message='No brackets allowed in Hamiltonian-Section!'// &
                    ' Use Labels-Section.'
            call wrline(zeroi,iz,iz0) 
         endif
      endif

!-----------------------------------------------------------------------
! read operator terms
!-----------------------------------------------------------------------
      if (keyword(1) .eq. 'end-hamiltonian-section') goto 20

      j=1
   15 continue
      if (j.eq.1 .and. keyword(j)(1:1) .ne. '|') then

!.....Read coefficient
         if (k .lt. maxkoe) then
            k=k+1
            minkoe=max(minkoe,k)
            hamnum(k)=nham
         else
            minkoe=minkoe+1
            k=maxkoe
            hamnum(k)=nham
            hamilton(:,k)=0
         endif
         mh=0
         mh1=0
         lreadcoeff=.true.
         call rdfactors(keyorig(j),nfac,factor,optyp,lc(j))
         f=fkoe
         muld=0
         mode=0
         call storetm(factor,nfac,optyp,f,h,muld,mode)
         If(nodiag(nham)) then
            if(factor(1) .eq. '1.234d-9') then
               h00=h ! This is for inhibiting summation of this term
            endif
         endif
         hamilton(f,k)=h
         htmtype(h)= 2
         htmsym(h) = 1
         kact(k)   = 1
         ktwin(k)  = k
!........Unit coefficient
         if(nfac.eq.1 .and. factor(1).eq.'1' .and. optyp(1).eq.3 )  &
              htmtype(h)=1

!........Look for "I". iorder is the number of I's
         iorder=0
         do j1 = 1, nfac
            if(factor(j1) .eq. 'I' ) then
               iorder=iorder+1
            elseif(factor(j1)(1:2) .eq. 'I^' ) then
               ilbl=index(factor(j1),' ')-1
               read(factor(j1)(3:ilbl),*,iostat=ierr) j2
               if(ierr.ne.0) then
                  message= &
         'Exponent of I must be integer (in coefficient column) :'
                  call wrline(onei,iz,iz0)
               endif
               iorder = iorder+j2
            endif
            do j2 = 2, nfac
               if(optyp(j2).gt.2) then
                  routine='heinterms (reading coefficients)'
                  message= &
                  'No summation, if I is part of a coefficient:'
                  call wrline(j,iz,iz0)
               endif
            enddo
         enddo
         htmsym(h) = (-1)**iorder
         if(htmsym(h).eq.-1) htmtype(h)= 3
            
         if (keyword(j+1)(1:1) .ne. '|') then
            routine='heinterms (reading operator-file)'
            message='The wrong keyword reads: '
            j=j+1
            message= &
       'There must be no blanks or brackets in the coefficient column.'
            call wrline(j,iz,iz0)
         endif
!.....End read coefficients         

      else if (keyword(j)(1:1) .eq. '|') then
!.....Look for L,M,R extension
         nLMR = 0
         if(lc(j) .gt. 2) then
            if(keyorig(j)(lc(j):lc(j)) .eq. 'L') then
               nLMR=3
            elseif(keyorig(j)(lc(j):lc(j)) .eq. 'M') then
               nLMR=2
            elseif(keyorig(j)(lc(j):lc(j)) .eq. 'R') then
               nLMR=1
            endif
            if(nLMR.gt.0) then  ! remove the L,M,R extension
               keyword(j)(lc(j):lc(j))=' '
               keyorig(j)(lc(j):lc(j))=' '
               lc(j)=lc(j)-1
            endif
         endif

!.....Read "|" and determine its number mh
         call zerovxi(grid,maxdim)
         jlbl=index(keyword(j),'&')
         klbl=index(keyword(j),'-')
!....Only a number, no & or -
         if (lc(j) .gt. 1 .and. jlbl .eq. 0 .and. klbl .eq. 0) then  
            lnum = .true.
            read (keyword(j)(2:lc(j)),*,iostat=ierr) mh1
            if(ierr.gt.0 ) then
               message='Expecting a number (or blank) after "|" : '
               routine = 'Heinterms'
               call wrline(j,iz,iz0)
               call stopnow
            elseif(mh1.le.0 .or. mh1.gt.nopdof-1) then
               write(message,'(2a,i5)') 'CHECK OPERATOR.',&
                         'DOF-label must be .gt.0 and .le. : ',ndof
               routine = 'Heinterms'
               call errormsg
            endif
            mh=mh1
            muld=0
            mode=0

         else if (lc(j) .gt. 1 .and.  &
              (jlbl .gt. 0 .or. klbl .gt. 0)) then   ! At least one & or -
            mh1=mh
            gf=0
            ilbl=2
   50       continue
            if (jlbl .eq. 0) then
               llbl=klbl
            else if (klbl .eq. 0) then
               llbl=jlbl
            else 
               llbl=min(klbl,jlbl)
            endif
            gf=gf+1
            if( keyword(j)(ilbl:llbl) .eq. '&' ) then
               mh1 = mh1 + 1
               lnonum = .true.
            else
               lnum = .true.
               read (keyword(j)(ilbl:llbl-1),*,iostat=ierr) mh1
               if(ierr.gt.0 .and. ilbl.eq.2 ) then
                  message='Expecting a number (or blank) after "|" : '
                  routine = 'Heinterms'
                  call wrline(j,iz,iz0)
                  call stopnow
               elseif(ierr.gt.0 .and. ilbl.gt.2 ) then
                  message='Expecting a number after "&" : '
                  routine = 'Heinterms'
                  call wrline(j,iz,iz0)
               endif
            endif
            if(mh1.le.0 .or. mh1.gt.nopdof-1) then
               write(message,'(2a,i5)') 'CHECK OPERATOR.',&
                         'DOF-label must be .gt.0 and .le. : ',ndof
               routine = 'Heinterms'
               call errormsg
            endif

            if (gf .eq. 1) mh=mh1

! if next symbol a - fill in grid
            if (llbl .eq. klbl) then
               ilbl=klbl+1
               if (jlbl .ne. 0) then
                  read (keyword(j)(ilbl:jlbl-1),*,iostat=ierr) mh2
               else
                  read (keyword(j)(ilbl:lc(j)),*,iostat=ierr) mh2
               endif
               if(ierr.gt.0 ) then
                  message='Expecting a number after "-" : '
                  routine = 'Heinterms'
                  call wrline(j,iz,iz0)
                  mh1 = 1
               elseif(mh2.le.0 .or. mh2.gt.nopdof-1) then
                  write(message,'(2a,i5)') 'CHECK OPERATOR.',&
                         'DOF-label must be .gt.0 and .le. : ',ndof
                  routine = 'Heinterms'
                  call wrline(j,iz,iz0)
               endif

               do m=mh1,mh2
                  grid(gf)=mham(m)
                  gf=gf+1
               enddo
               gf=gf-1
            else if (llbl .eq. jlbl) then
               grid(gf)=mham(mh1)
            endif

            llbl=jlbl                 ! at least one &
            ilbl=jlbl+1
            jlbl=index(keyword(j)(ilbl:lc(j)),'&')  ! a second &
            if (jlbl .ne. 0) then
               jlbl=jlbl+ilbl-1       ! Because of searching in sub-string
               klbl=index(keyword(j)(ilbl:lc(j)),'-')
               if (klbl .gt. 0) klbl=klbl+ilbl-1       
               go to 50

            else if (llbl .gt. 0) then    
               klbl=index(keyword(j)(ilbl:lc(j)),'-')
               if (klbl .gt. 0) then
                  klbl=klbl+ilbl-1       
                  go to 50        
               endif

               gf=gf+1
               if(ilbl .gt. lc(j)) then
                  mh1=mh1+1
               else
                  read (keyword(j)(ilbl:lc(j)),*,iostat=ierr) mh1
               endif 

               if(ierr.gt.0 ) then
                  message='Expecting a number after "&" : '
                  routine = 'Heinterms'
                  call wrline(j,iz,iz0)
                  mh1 = 1
               elseif(mh1.le.0 .or. mh1.gt.nopdof-1) then
                  write(message,'(2a,i5)') 'CHECK OPERATOR.',&
                         'DOF-label must be .gt.0 and .le. : ',ndof
                  routine = 'Heinterms'
                  call errormsg
               endif
               grid(gf)=mham(mh1)
            endif

!-----------------------------------------------------------------------
! check that all dofs in grid are defined. If spfdof(f) = 0 DOF is not 
! included in dynamics. Make sure it is defined as a point DVR.
!-----------------------------------------------------------------------
            do f1=1,gf
               f=grid(f1)
               if (dofspf(f) .eq. 0 .and. basis(f) .ne. 19) then
                  routine = 'Heinterm (heintermmod.f90)'
                  message = 'Non-dynamical coordinates used that are defined '// &
                      ' in a multiD potential. Define using a point DVR.'
                  call errormsg
               endif
            enddo

            call muldnum(muld,grid,mode)

         else                                            !  No number, no & 
            lnonum = .true.
            mh=mh+1
            mh1=mh
            muld=0
            mode=0
         endif

         if (k .eq. 0) then
            message='Forgotten "modes" keyword in mode definition?'
            call errormsg
         endif

         j=j+1
         if (j.gt.ic .or. keyword(j)(1:1) .eq. '|') then
            write(6,'(/a)') trim(inptit)
            routine='heinterms (reading operator-file)'
            write(message,'(a,i4)') 'Missing operator term for DOF',mh
            call warnmsg
         endif

!-----------------------------------------------------------------------
! loop over dofs in keyword and
! 1. change any basis dependent keywords
! 2. get terms/factors
! 3. if ghost mode is involved, ignore term
! 4. find h in hoplab list, and assign to hamilton array
!
! for direct dynamics, do not ignore potential functions even if ghost
! modes present
!-----------------------------------------------------------------------
         f=mham(mh)
         if (f .eq. 0) then
            message='Operator for undefined mode. Missing modes in "mode" definition?'
            print*,"mode",mh
            call errormsg
         endif
         if( keyorig(j) .eq. '1' ) then
!...........Avoid building muld unit operators.
            muld=0
            mode=0
         endif
         if (dofspf(f).eq.0 .and. (muld .gt. 0 .or. mode .gt. 0)) then
            routine='Heinterms'
            message = 'Ghost mode in muld or mode term. Check?'
            call warnmsg1
         endif
         if (dofspf(f).eq.0) then
!           lc(j)=index(keyorig(j),' ')-1
            call rdfactors(keyorig(j),nfac,factor,optyp,lc(j))
            if (nfac .gt. 1 .or. factor(1) .ne. '1') then
               kignore(k)=.true.
            endif
            go to 90
         else
!           lc(j)=index(keyorig(j),' ')-1
            call rdfactors(keyorig(j),nfac,factor,optyp,lc(j))
            do n=1,nfac 
               if ((n.eq.1 .and. optyp(n).ne.3) .or.  &
                   (n.gt.1 .and. optyp(n).ne.1) ) then
                  routine='heinterms (reading operator-file)'
                  message='The wrong keyword reads: '
                  call wrline(j,iz,iz0)
                  message=&
                 'No "+" or "-" or "/" allowed in the operator tableau'
                  call warnmsg
               endif
               call renterm(factor(n),basis(f),loperr,&
                           ipbaspar(1,f),ipbaspar(2,f))
            enddo
         endif

         call storetm(factor,nfac,optyp,f,h,muld,mode)
         if (muld .ne. 0) then
            f=mulddof2(1,muld)
            fterm(k)=f
         else if (mode .ne. 0) then
            f=spfdof(1,mode)
         endif

         if(nLMR.gt.0) then      ! a LMR term is found
            mm=dofspf(f)
            ff=spfdof(1,mm)
            if(mm.ne.mmLMR .or. k.ne.kkLMR) then  ! new LMR term
               iLMR=iLMR+1
               if(iLMR.gt.maxLMR) then
                  routine='heinterms (reading operator-file,4)'
                  message='Too many LMR terms. Increase maxLMR on operdef.inc'
                  call errormsg
               endif
               mmLMR=mm
               kkLMR=k
               hamilton(ff,k)=-iLMR ! a neg. value indicates LMR term
               do n=2,nspfdof(mm)
                  ff=spfdof(n,mm)
                  hamilton(ff,k)=ff
               enddo
            endif
            if(hamLMR(iLMR,nLMR) .gt. 0) then
               routine='heinterms (reading operator-file,3)'
               write(message,'(a,i3,a,i3,a,i5,a,i3,3a,i3,a,i3)') &
               'LMR-operator-term already defined for DOF =',f,', mh =', &
               mh,',  k =',k,',  h =',hamilton(f,k),',  ', &
                keyorig(j)(1:8),', iLMR =',iLMR,', nLMR =',nLMR
               call warnmsg
            endif
            hamLMR(iLMR,nLMR)=h
         else                ! Normal procedure   
            if(hamilton(f,k) .gt. 0) then
               routine='heinterms (reading operator-file,2)'
               write(message,'(a,i3,a,i3,a,i5,a,i3,2a)') &
               'Operator-term already defined for DOF =',f,', mh =',mh, &
               ',  k =',k,',  h =',hamilton(f,k),',  ', keyorig(j)(1:8)
               call warnmsg
            endif
            hamilton(f,k)=h
         endif

         if(j.lt.ic .and. keyorig(j+1)(1:1) .ne. '|'  &
                    .and. keyorig(j+1)(1:1) .ne. '#')  then
            routine='heinterms (reading operator-file)'
            message='The wrong keyword reads: '
            j1=j+1
            call wrline(j1,iz,iz0)
            message=&
         'There must be no blanks or brackets within an operator symbol'
            call warnmsg
         endif

   90    continue
         mh=mh1                 ! This is for muld terms

      endif

      if (j.lt.ic) then
         j=j+1
         go to 15
      else
         if (lnum.and.lnonum) then
            loperr=.true.
            routine = 'Heinterms (read Hamiltonian-Section)'
            message='"|" with numbers and "|" without numbers'//&
                     ' must not be mixed in one line.'
            call wrline(zeroi,iz,iz0)
         endif
         if (mh.gt.nhmode) then
            loperr=.true.
            message='More modes in term       than allowed'
            write(message(20:24),'(i5)') k
            call wrline(zeroi,iz,iz0)
         endif
         go to 100
      endif

   20 continue

!-----------------------------------------------------------------------
! Set ktot to the numbers of lines of the operator tableau. 
!-----------------------------------------------------------------------
      ktot=k

!-----------------------------------------------------------------------
! remove ghost terms from hamilton 
!-----------------------------------------------------------------------
      ktot1=ktot
      k1=1
      do k=1,ktot1
         if (kignore(k)) then
            call mvkend(k1)
            ktot=ktot-1
          else
            k1=k1+1
         endif
      enddo

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      loperr=lwarn.or.loperr
      lwarn=.false.

      return
      end subroutine heinterms

!#######################################################################
!
!     DEINTERMS 
!
!     DEINTERMS is a subroutine analogous to heinterms, reading now
!     the dissipative part of the operator file.
!
!     10/99 AR
!
!#######################################################################

      subroutine deinterms(iunit,loperr,iz,nham)

      implicit none

      integer(long)        :: ic,ierr,ilbl,k,f,f1,nhmode,i,&
                              k1,ktot1,n,h,iorder,ndm,k2,m1,&
                              m2,j2,jt,f2,lclab,il1,il2,il3,&
                              il4,s1,s2,s3,s4,acttype,j,j1,iz0

      integer(long), intent(in)                 :: iz
      integer(long), intent(in)                 :: nham,iunit
      integer(long), dimension(24)              :: optyp
      integer(long), dimension(2*maxdim)        :: mham
      integer(long)                             :: nfac
      character(len=c2), dimension(24)          :: factor
      character(len=2*c3)                       :: clab
      logical(kind=4)                           :: initial
      logical(kind=4), dimension(maxkoe)        :: kignore
      logical(kind=4), intent(out)              :: loperr


! initialise variables
      routine = 'Deinterms (reading dissipative-section)'

      iz0 = 0
      call zerovxl(kignore,maxkoe)
      call zerovxi(mham,2*maxdim)
      acttype = 0
      ndm = 0
      k2 = 0

!-----------------------------------------------------------------------
! if hoplab needs to be initialised then:
! place unit operator at beginning of list for each degree of freedom
! place +1, -1 at beginning of list for coefficients
!-----------------------------------------------------------------------
      if (hopdim .eq. 0) then
         hoplab(1)='1'
         hoplab(2)='-1'
         hoplab(3)='I'
         hopdim=3

         do f=1,nopdof
            htmdim=htmdim+1
            h=min(htmdim,maxhtm)
            hterm(1,h)=1
            htmfac(h)=1
            htmdof(h)=f
            htmadof(h)=f
         enddo

         k=0
      else
         k=ktot
      endif

      nhmode=0
      initial=.true.

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iunit,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      routine = 'deinterms, heinterm.F line 627'
      call cntrl(keyword(1),lc(1),ierr)

!-----------------------------------------------------------------------
! first read mode labels
!-----------------------------------------------------------------------
      if (keyword(1) .eq. 'modes') then
         j=2
         do i=2,ic-1,2
            if (keyword(j)(1:1) .eq. '|') then
               nhmode=nhmode+1

! add time dof if required (was set to -1 in mdefault.F)
               f=0
               if (keyorig(j+1) .eq. 'Time' .and. ftime .lt. 0) then
                  nopdof=nopdof+1
                  ftime=nopdof
                  modelabel(ftime)='Time'
                  gdim(ftime)=1
                  f=ftime
                  nspfdof(mkoe)=nspfdof(mkoe)+1
                  n=nspfdof(mkoe)
                  spfdof(n,mkoe)=ftime
                  dofspf(ftime)=mkoe

                  htmdim=htmdim+1
                  h=min(htmdim,maxhtm)
                  hterm(1,h)=1
                  htmfac(h)=1
                  htmdof(h)=f
                  htmadof(h)=f

               else if (keyorig(j+1) .eq. 'Time'  &
                          .and. ftime .gt. 0) then
                  f=ftime

               else
                  do f1=1,ndof
                     if (keyorig(j+1) .eq. modelabel(f1)) f=f1
                  enddo
               endif

               if (f .eq. 0) then
                  message= &
           'Operator mode not found in system :'//keyorig(j+1)
                  call wrline(zeroi,iz,iz0)
               else
                  mham(nhmode)=f
               endif

               j=j+2
            else
               message='Wrong structure of Mode keyword'
               call warnmsg
            endif
         enddo
      
         go to 10
      endif

!-----------------------------------------------------------------------
! check that modes have been correctly defined and
! label any modes not included in sbasis
!-----------------------------------------------------------------------
      if (initial) then
         if (nhmode .eq. 0) then
            message='Operator mode order not defined'
            call errormsg
         endif

         do f=1,nhmode
            f1=mham(f)
            if (dofspf(f1) .eq. 0) mham(f)=0
         enddo

         if (mod(nhmode,2).eq.0) then
            ndm = nhmode / 2
         elseif (mod(nhmode,2).eq.1 .and. ftime .gt. 0) then
            ndm = (nhmode-1) / 2
         else
            message='Unbalanced mode definition in dissipative section'
            call errormsg
         endif

         do m1=1,ndm
            m2 = m1 + ndm
            if (mham(m1) .ne. mham(m2)) then 
               message='Check mode order in dissipative section'
               call errormsg
            endif
         enddo

         initial = .false.
      endif

!-----------------------------------------------------------------------      
! Check for '[' or '{' . Only simple operators are allowed here.
!-----------------------------------------------------------------------      
      routine = 'Deinterms (reading operator-file, dissipative-section)'
      j1=index(inptit,'[')
      j2=index(inptit,'{')
      if( max(j2,j1) .gt. 1 ) then
         if(j1 .le. 0 ) j1 = j2
         if(j2 .le. 0 ) j2 = j1
         j2=min(j1,j2)
         j1=index(inptit,'#')
         if( j1 .le. 0 .or.  j1.gt.j2 ) then
            message='No brackets allowed in Dissipative-Section!'// &
                    ' Use Lables-Section.'
            call wrline(zeroi,iz,iz0) 
         endif
      endif

!-----------------------------------------------------------------------
! read operator terms
!-----------------------------------------------------------------------
      if (keyword(1) .eq. 'end-dissipative-section') goto 20

!     Determine action type
      if (keyword(1) .eq. 'pure-left') then
         acttype = 2
         goto 10
      else if (keyword(1) .eq. 'pure-right') then
         acttype = 3
         goto 10
      else if (keyword(1) .eq. 'left-right') then
         acttype = 4
         goto 10
      endif
      
!     Take care that acttype is defined
      if (acttype .eq. 0) then
            message='Line    : Action type not specified ' // &
                    '(dissipative section)'
            write(message(6:8),'(i3)') iz
            call errormsg
      endif
         
!     One term in one line only
      if (keyword(1)(1:1) .eq. '|') then
            message='Line     is corrupt (dissipative section)'
            write(message(6:8),'(i3)') iz
            call errormsg
      endif

!     Read coefficient 
      if (k .lt. maxkoe) then
         k=k+1
         minkoe=k
      else
         minkoe=minkoe+1
         k=maxkoe
      endif
      call zerovxi(hamilton(1,k),maxdim)
      call rdfactors(keyorig(1),nfac,factor,optyp,lc(1))
      do j1 = 1, nfac
         if( factor(j1) .eq. 'I' ) then
            message='No "I" allowed in coefficient column!'
            call wrline(zeroi,iz,iz0) 
         endif
      enddo
      call storetm(factor,nfac,optyp,fkoe,h,0,0)
      hamilton(fkoe,k)=h
      
!     Define additional coefficient '1' for second term

      if ((dentype .eq. 1 .and. acttype.eq.4) .or. dentype .eq. 2) then
         if (k .lt. maxkoe) then
            k=k+1
            minkoe=k
         else
            minkoe=minkoe+1
            k=maxkoe
         endif
         call zerovxi(hamilton(1,k),maxdim)
         call rdfactors('1',nfac,factor,optyp,1)
         call storetm(factor,nfac,optyp,fkoe,h,0,0)
         hamilton(fkoe,k)=h
         htmtype(h)= 2
         htmsym(h) = 1
!........Unit coefficient
         if(nfac.eq.1 .and. factor(1).eq.'1' .and. optyp(1).eq.3 ) &
              htmtype(h)=1

!........Look for "I". iorder is the number of I's
         iorder=0
         do j1 = 1, nfac
            if(factor(j1) .eq. 'I' ) then
               iorder=iorder+1
            elseif(factor(j1)(1:2) .eq. 'I^' ) then
               ilbl=index(factor(j1),' ')-1
               read(factor(j1)(3:ilbl),*) j2
               iorder = iorder+j2
            endif
            do j2 = 2, nfac
               if(optyp(j2).gt.2) then
                  routine='heinterms (reading coefficients)'
                  message= &
                  'No summation, if I is part of a coefficient:'
                  call wrline(j,iz,iz0)
               endif
            enddo
         enddo
         htmsym(h) = (-1)**iorder
         if(htmsym(h).eq.-1) htmtype(h)= 3

         if (keyword(2)(1:1) .ne. '|') then
            routine='Deinterms (reading dissipative-section)'
            message='The wrong keyword reads: '
            j=2
            message= &
       'There must be no blanks or brackets in the coefficient column.'
            call wrline(j,iz,iz0)
         endif

!     Save action type
         k2=max(k-1,1)
         kact(k2)   = acttype
         kact(k)     = acttype
         ktwin(k2)  = k
         ktwin(k)    = k-1
         hamnum(k2) = nham
         hamnum(k)   = nham

         if (dentype .eq. 1) kact(k)   = 5
      else if (dentype .eq. 1) then
         kact(k)   = acttype
         ktwin(k)  = k
         hamnum(k) = nham
      endif

!     Check number of modes in term
      f = 0
      do j=2,ic
        if (keyword(j)(1:1) .eq. '|') then
            if (lc(j) .gt. 1) then
               read (keyword(j)(2:ic),*) f
            else 
               f = f+1
            endif
         endif
      enddo   
      if (f.gt.nhmode) then
         loperr=.true.
         message='More modes in line       than allowed '//&
                 '(dissipative section)'
         write(message(20:24),'(i5)') iz
         call warnmsg
      endif

!-----------------------------------------------------------------------
! loop over dofs in keyword 
!-----------------------------------------------------------------------
      f1 = 0
      do j=2,ic
!     Find 1D operator
         if (keyword(j)(1:1) .eq. '|') then
            ilbl=index(keyword(j),'&')
            if(ilbl.gt.0) then
               loperr=.true.
               routine = 'Deinterms'
               message=&
               'No non-1D operators allowed in dissipative section'
               call warnmsg
            endif
            if (lc(j) .gt. 1) then
               read (keyword(j)(2:ic),*) f1
            else 
               f1 = f1+1
            endif
            f=mham(f1)

            if ((dentype .eq. 1 .and. acttype.eq.4) .or. &
                 dentype .eq. 2) then
               lc(j+1) = index(keyorig(j+1),' ')-1
               clab    = keyorig(j+1)(1:lc(j+1))
               lclab   = lc(j+1)
               if (f1 .le. ndm) then
                  k2 = max(k-1,1)
               else
                  k2 = k
               endif

!     Merge pure-left and pure-right operators for type I
            else if (dentype .eq. 1) then
               if (f1.gt.ndm) goto 90

               jt=0
               f2=f1
               do j2=j+1,ic
                  if (keyword(j2)(1:1) .eq. '|') then
                     if (lc(j2) .gt. 1) then
                        read (keyword(j2)(2:ic),*) f2
                     else 
                        f2 = f2+1
                     endif
                     if (mham(f2).eq.f) then
                        jt=j2
                     endif
                  endif
               enddo
               if (jt.eq.0) then
                  message='Line       is corrupt '//&
                       '(dissipative section)'
                  write(message(5:10),'(i5)') iz
                  call errormsg
               endif
               if (f.eq.feb) then
                  il1=index(keyorig(j+1),'&')
                  il2=index(keyorig(j+1),' ')-1
                  read(keyorig(j+1)(2:il1-1),*) s1
                  read(keyorig(j+1)(il1+1:il2),*) s2
                  il3=index(keyorig(jt+1),'&')
                  il4=index(keyorig(jt+1),' ')-1
                  read(keyorig(jt+1)(2:il3-1),*) s3
                  read(keyorig(jt+1)(il3+1:il4),*) s4
                  if (s2.ne.s3) then
                     message='Line       is corrupt '//&
                          '(dissipative section)'
                     write(message(5:10),'(i5)') iz
                     call errormsg
                  endif
                  clab = keyorig(j+1)(1:il1)//keyorig(jt+1)(il3+1:il4)
                  lclab = il1 + il4 - il3
               else
                  lc(j+1)  = index(keyorig(j+1),' ')-1
                  lc(jt+1) = index(keyorig(jt+1),' ')-1
                  if ((keyorig(jt+1)(1:lc(jt+1)) .eq. '1') .or.&
                       (keyorig(jt+1)(1:lc(jt+1)) .eq. '1.0') .or.&
                       (keyorig(jt+1)(1:lc(jt+1)) .eq. '1.00')) then
                     clab   = keyorig(j+1)(1:lc(j+1))
                     lclab  = lc(j+1)
                  else if ((keyorig(j+1)(1:lc(j+1)) .eq. '1') .or.&
                          (keyorig(j+1)(1:lc(j+1)) .eq. '1.0') .or.&
                          (keyorig(j+1)(1:lc(j+1)) .eq. '1.00')) then
                     clab   = keyorig(jt+1)(1:lc(jt+1))
                     lclab  = lc(jt+1)
                  else
                     clab   = keyorig(j+1)(1:lc(j+1))//'%'&
                          //keyorig(jt+1)(1:lc(jt+1))
                     lclab  = lc(j+1)+lc(jt+1)+1
                  endif
               endif
               k2 = k
            endif

!     Save 1d operator
            if (f.eq.0) then
               call rdfactors(clab,nfac,factor,optyp,lclab)
               if (nfac .gt. 1 .or. factor(1) .ne. '1') then
                  kignore(k2)=.true.
                  go to 90
               endif
            else
               call rdfactors(clab,nfac,factor,optyp,lclab)
               do n=1,nfac 
                  call renterm(factor(n),basis(f),loperr,&
                               ipbaspar(1,f),ipbaspar(2,f))
               enddo
            endif
            do n=1,nfac
               if (optyp(n) .eq. 2) then
                  ilbl=index(factor(n),' ')
                  factor(n)(ilbl:ilbl+2)='^-1'
                  optyp(n)=1
               endif
            enddo
            call storetm(factor,nfac,optyp,f,h,0,0)
            hamilton(f,k2)=h

 90         continue
         elseif(j.lt.ic .and. keyorig(j+1)(1:1) .ne. '|'  &
                 .and. keyorig(j+1)(1:1) .ne. '#')  then
            routine='Deinterms (reading dissipative-section)'
            message='The wrong keyword reads: '
            j1=j+1 
            message=&
       'There must be no blanks or brackets within an operator symbol'
            call wrline(j1,iz,iz0)
         endif

      enddo

      goto 10
   20 continue

!-----------------------------------------------------------------------
! remove ghost terms from hamilton 
!-----------------------------------------------------------------------
      ktot=k
      ktot1=ktot
      k1=1
      do k=1,ktot1
         if (kignore(k)) then
            call mvkend(k1)
            ktot=ktot-1
          else
            k1=k1+1
         endif
      enddo
         
!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      loperr=lwarn.or.loperr
      lwarn=.false.

      return
      end subroutine deinterms

!#######################################################################
!
! RENTERM
!
! renames any terms read from the op file that depend on basis etc.
! e.g. KE -> KEfft if basis(f)=4
!
! Check for wrong input (e.g. sdq must go with cos- or sin-DVR).
!
!#######################################################################

      subroutine renterm(kword,basis,loperr,ipar,ibleg)

      implicit none

      integer(long)                   :: ipower,jlbl
      integer(long), intent(in)       :: basis,ipar,ibleg
      character(len=*), intent(inout) :: kword
      logical(kind=4), intent(out)    :: loperr
      logical(kind=4)                 :: dosdq

      routine='renterm'
      dosdq=.true.

!-----------------------------------------------------------------------
! alter any terms that are basis set dependent
!-----------------------------------------------------------------------
      if (kword(1:2) .eq. 'dq') then
         ipower=0
         if (kword(3:3) .eq. '^') then
            jlbl=index(kword,' ')-1
            read(kword(4:jlbl),*) ipower
         elseif (kword(3:3) .eq. ' ') then
            ipower = 1
         endif
         if (ipower .eq. 0) then
            write(message,'(2a)')  'Zero power detected ??? Term: ',&
                  kword
            call errormsg
         endif

         if (ipower .gt. 2 .and. basis .ne. 4) then
            message=&
          'Derivatives higher than 2nd order only defined for FFT.'
            call warnmsg
         endif

         if (basis .eq. 4) then
            if(ipower .gt. 0 ) then
               write(kword,'(a,i3)') 'dqfft^',ipower
               call rmblnk(kword)
            endif
         else if (basis .eq. 25) then
            write(kword,'(a,i3)') 'dqgwp^',ipower
            call rmblnk(kword)
         else if (basis .eq. 27) then
            if (ipower .eq. 2) then
               kword='p2_clm'
            else if (ipower .eq. 1) then
               message= &
          'Simple first derivative not defined for classical. ip ?'
               call warnmsg
            endif
         else if (basis .eq. 2 .or. basis .eq. 12) then
            message= &
          'Simple derivatives not defined for Legendre DVR. dth, j^2 ?'
            call warnmsg
         else if (basis .eq. 11 .and. ipower .eq. 1 ) then
            message= &
          'Simple first derivative not defined for Radial HO DVR. qdq ?'
            call warnmsg
         else if (basis .eq. 14 .and. ipower .eq. 1 ) then
            message= &
          'Simple first derivative not defined for cos DVR. sdq ?'
            call warnmsg
         else if (basis .eq. 3 .and. ipar .eq. 1 ) then
            message='Simple first derivative not defined '// &
                    'for sin DVR when sdq is set'
            call warnmsg
         else
            write(kword,'(a,i3)') 'dqdvr^',ipower
            call rmblnk(kword)
         endif

      else if (kword .eq. 'j^2') then
         if (basis .eq. 2 .or. basis .eq. 12 .or. basis .eq. 19 &
            .or. basis .eq. 22) then
            continue
         elseif(basis .eq. 7 .or. basis .eq. 9 .or. basis .eq. 20 .or. &
                 basis .eq. 21) then
            kword='j^2kleg'
         elseif(basis .eq. 6) then 
            kword='j^2sph'
         elseif(basis .eq. 27) then 
            kword='p2_cl'
         elseif(basis .eq. 33) then
            kword='j^2wig'
         elseif(basis .eq. 4) then
            loperr=.true.
            message='FFT cannot handle j^2 operation'
            call warnmsg
         else 
            loperr=.true.
            message= 'j^2 is defined for sphFBR, Leg, Leg/R, KLeg, KLeg/R, '//  &
               'PLeg and Wigner only!'
            call warnmsg
         endif

      else if (kword(1:3) .eq. 'j_p'  .or. &
               kword(1:3) .eq. 'j_m'  .or.  &
               kword(1:3) .eq. 'jpm'  .or. &
               kword(1:4) .eq. 'jp^2' .or. &
               kword(1:4) .eq. 'jm^2' .or. &
               kword(1:4) .eq. 'jpjm' .or. &
               kword(1:4) .eq. 'jmjp' .or. &
               kword(1:4) .eq. 'jpjz' .or. &
               kword(1:4) .eq. 'jmjz' .or. &
               kword(1:4) .eq. 'jzjp' .or. &
               kword(1:4) .eq. 'jzjm' .or. &
               kword(1:4) .eq. 'cjpm' ) then 
         if (basis .ne. 7 .and. basis .ne. 9 .and. basis .ne. 20 .and.  &
                 basis .ne. 21 .and. basis .ne. 33) then
            loperr=.true.
            message='j_p,j_m,jpm,jp^2,jm^2 and cjpm operators'// &
                  ' allowed for KLeg-, KLeg/R-, PLeg, PLeg/R and Wigner-DVR only.'
            call warnmsg
         endif

         if ((kword .eq. 'j_ps' .or. kword .eq. 'j_ms' .or. &
              kword .eq. 'jp^2s' .or. kword .eq. 'jm^2s' .or. &
              kword .eq. 'jpms' .or. kword .eq. 'jpjms' .or. &
              kword .eq. 'jmjps' .or. kword .eq. 'jpjzs' .or. &
              kword .eq. 'jmjzs' .or. kword .eq. 'jzjps' .or. &
              kword .eq. 'jzjms' .or. kword .eq. 'jpjz' .or. &
              kword .eq. 'jmjz' .or. kword .eq. 'jzjp' .or. &
              kword .eq. 'jzjm' ) .and. basis .ne. 33) then
            loperr=.true.
            message='j_ps, j_ms, jpms, jp^2s, jm^2s, jpjms, '// &
                    'jmjps, jpjzs, jmjzs, jzjps, jzjms, ' // &
                    'jpjz, jmjz, jzjp, and jzjm ' // &
                    'operators are allowed for Wigner-DVR only.'
            call warnmsg
         endif
         if (basis .eq. 33) then
            if (kword .eq. 'j_p') then
               kword='j_pwig'
            else if (kword .eq. 'j_m') then
               kword='j_mwig'
            else if (kword .eq. 'jpm') then
               kword='jpmwig'
            else if (kword .eq. 'jp^2') then
               kword='jp2wig'
            else if (kword .eq. 'jm^2' ) then
               kword='jm2wig'
            elseif (kword .eq. 'j_ps') then
               kword='j_pswig'
            else if (kword .eq. 'j_ms') then
               kword='j_mswig'
            else if (kword .eq. 'jpms') then
               kword='jpmswig'
            else if (kword .eq. 'jp^2s') then
               kword='jp2swig'
            else if (kword .eq. 'jm^2s' ) then
               kword='jm2swig'
            else if (kword .eq. 'jpjm') then
               kword='jpjmwig'
            else if (kword .eq. 'jmjp') then
               kword='jmjpwig'
            else if (kword .eq. 'jpjms') then
               kword='jpjmswig'
            else if (kword .eq. 'jmjps') then
               kword='jmjpswig'
            else if (kword .eq. 'jpjzs') then
               kword='jpjzswig'
            else if (kword .eq. 'jmjzs') then
               kword='jmjzswig'
            else if (kword .eq. 'jzjps') then
               kword='jzjpswig'
            else if (kword .eq. 'jzjms') then
               kword='jzjmswig'
            else if (kword .eq. 'jpjz') then
               kword='jpjzwig'
            else if (kword .eq. 'jmjz') then
               kword='jmjzwig'
            else if (kword .eq. 'jzjp') then
               kword='jzjpwig'
            else if (kword .eq. 'jzjm') then
               kword='jzjmwig'
            endif
         endif

      else if (kword .eq. 'sJp'  .or.&
               kword .eq. 'sJm'  .or.&
               kword .eq. 'sJpk' .or.&
               kword .eq. 'sJmk' .or. &
               kword(1:4) .eq. 'cjpm') then
         if (basis .ne. 7 .and. basis .ne. 9) then
            loperr=.true.
            message='cjpm,sJp,sJm,sJpk,sJmk operators allowed for KLeg'//&
                    ' and PLeg only.'
            call warnmsg
         endif

      else if (kword.eq.'qdq') then
         if (basis .eq. 11) then
            kword='qdq:rHO'
         elseif (basis .eq. 14) then
            message='qdq derivative not defined for cos DVR. sdq ?'
            call warnmsg
         else if (basis .eq. 3 .or. basis == 31 .or. basis == 32 .and. ipar .eq. 1 ) then
            message='qdq derivative not defined '//&
                       'for sin or Sinc or Wsinc DVR when sdq is set'
            call warnmsg
          elseif (basis .eq. 4) then
            message='qdq derivative not allowed for FFT.'
            call warnmsg
         endif

      else if (kword.eq.'sdq') then
         if (basis .eq. 14 ) then
             kword='sdq:cos'
         elseif (basis .eq. 3  .and. ipar .ne. 1) then
            kword='sdq:sin'
            if(dosdq) then
               message='You may want to set "sdq" in pbasis-section'//&
                       ' for sdq operation with sin-DVR'
               call warnmsg1
               dosdq=.false.
            endif
          elseif (basis .eq. 4) then
            message='sdq derivative not allowed for FFT.'
            call warnmsg
         endif

      else if (kword.eq.'cdq') then
         if (basis .eq. 11) then
            message='cdq derivative not allowed for radial HO DVR.?'
            call warnmsg
         elseif (basis .eq. 14) then
            message='cdq derivative not allowed for cos DVR. sdq ?'
            call warnmsg
         else if (basis .eq. 3 .and. ipar .eq. 1 ) then
            message='cdq derivative not allowed '//&
                       'for sin DVR when sdq is set'
            call warnmsg
          elseif (basis .eq. 4) then
            message='cdq derivative not allowed for FFT.'
            call warnmsg
         endif

      else if (kword.eq.'dth1') then
         if(basis .eq. 7  .or.  basis .eq. 9 .or. basis .eq. 20 .or.  &
             basis .eq. 21)  &
            kword='dth1_kleg' 
         if (basis.ne.2 .and. basis.ne.7 .and. basis.ne.9 &
                        .and. basis.ne.12 .and. basis.ne.20 & 
                        .and. basis.ne.21) then
            loperr=.true.
            message='dth1 operation allowed only for Leg,KLeg,Kleg/R '// &
                     'PLeg,PLeg/r or Leg/r'
            call warnmsg
         else if (ibleg.gt.0) then
            loperr=.true.
            message='dth1 operation allowed only for "no symmetry".'
            call warnmsg
         endif
 
      else if (kword.eq.'dth2') then
         if(basis .eq. 7  .or.  basis .eq. 9 .or. basis .eq. 20 .or. &
            basis .eq. 21)  &
            kword='dth2_kleg'
         if (basis.ne.2 .and. basis.ne.7 .and. basis.ne.9 &
                        .and. basis.ne.12 .and. basis.ne.20 .and. &
                              basis.ne.21) then
            loperr=.true.
            message='dth2 operation allowed only for Leg, KLeg, Kleg/R, '//  &
                     'Leg/r,PLeg or PLeg/R'
            call warnmsg
         else if (ibleg.eq.0) then
            loperr=.true.
            message='dth2 operation allowed only for "symmetry".'
            call warnmsg
         endif

      else if (kword.eq.'d2th1') then
         if (basis.ne.2 .and. basis.ne.7 .and. basis.ne.9 &
                        .and. basis.ne.12 .and. basis.ne.20 .and. &
                              basis.ne.21) then
            loperr=.true.
            message='d2th1 operation allowed only for Leg, KLeg,Kleg/R, '//  &
                     'Leg/r,PLeg or PLeg/R'
            call warnmsg
         endif

      else if (kword.eq.'KE') then
         if (basis .eq. 2 .or. basis .eq. 12) then
            loperr=.true.
            message= &
               'Legendre DVR cannot handle KE operation. j^2 ?'
            call warnmsg
         else if (basis .eq. 4) then
            kword='KEfft'
         else if (basis .eq. 6) then
            loperr=.true.
            message= &
               'sphFBR cannot handle KE operation. j^2 ?'
            call warnmsg
         else if (basis .eq. 7 .or. basis .eq. 9  &
                       .or. basis .eq. 20 .or. basis .eq. 21) then
            loperr=.true.
            message= &
          'KLeg,KLeg/R (PLeg,PLeg/R) cannot handle KE operation. j^2 ?'
            call warnmsg
         else if (basis .eq. 25) then
            kword='KEgwp'
         else if (basis .eq. 26) then
            kword='KE_xloc'
         else if (basis .eq. 27) then
            kword='KE_cl'
         else if (basis .eq. 33) then
            loperr=.true.
            message= 'Wigner cannot handle KE operation. j^2 ?'
            call warnmsg

         else
            kword='KEdvr'
         endif

      else if (basis.eq.-1 .and. kword(1:2).eq.'jz') then 
            loperr=.true.
            message='For sphFBR jz and jz^2 are 2D operators and '// &
                'must appear under theta rather than under phi mode.'
            call warnmsg

      else if (kword.eq.'jz') then
         if (basis .eq. 6) then
            kword='jzsph'
         else if (basis .eq. 8) then
            kword='q'
         elseif (basis .eq. 2 .or. basis .eq. 12) then
            loperr=.true.
            message= &
               'Legendre DVR cannot handle jz operation.'
            call warnmsg
         else if (basis .eq. 4) then
            loperr=.true.
            message= &
               'FFT cannot handle jz operation.'
            call warnmsg
         else if (basis .eq. 5) then
            loperr=.true.
            message= &
               'If you are using Pleg, use dq (or p) rather than jz.'
            call warnmsg
        else if (basis .eq. 7 .or. basis .eq. 20) then
            loperr=.true.
            message='For KLeg (KLeg/R) jz is a 1D operator and must '// &
               'be placed under phi rather than theta.'
            call warnmsg
        else if (basis .eq. 9 .or. basis .eq. 21) then
            loperr=.true.
            message='Use dq (or p) rather than jz and '// &
               'place it under phi rather than theta.'
            call warnmsg
         else if (basis .eq. 33) then
            loperr=.true.
            message='For Wigner jz is a 1D operator. '// &
               'If using k representation for gamma,'// &
               ' place jz there instead of under beta. '// &
               'If using gamma representation directly, '// &
               ' use dq and place it there instead of under beta.'
            call warnmsg
         else
            loperr=.true.
            message= &
               'DVR cannot handle jz operation.'
            call warnmsg
         endif

      else if (kword.eq.'jz^2') then
         if (basis .eq. 6) then
            kword='jzzsph'
         else if (basis .eq. 8) then
            kword='q^2'
         elseif (basis .eq. 2 .or. basis .eq. 12) then
            loperr=.true.
            message= &
               'Legendre DVR cannot handle jz^2 operation.'
            call warnmsg
         else if (basis .eq. 4) then
            loperr=.true.
            message= &
               'FFT cannot handle jz^2 operation.'
            call warnmsg
         else if (basis .eq. 5) then
            loperr=.true.
            message= &
              'If you are using Pleg, use (-1) dq^2 rather than jz^2.'
            call warnmsg
        else if (basis .eq. 7 .or. basis .eq. 20) then
            loperr=.true.
            message='For KLeg (KLeg/R)jz^2 is a 1D operator and must '// &
                    'be placed under phi rather than theta.'
            call warnmsg
        else if (basis .eq. 9 .or. basis .eq. 21) then
            loperr=.true.
            message='Use (-1) dq^2 rather than jz^2 and '// &
               'place it under phi rather than theta.'
            call warnmsg
         else if (basis .eq. 33) then
            loperr=.true.
            message='For Wigner jz^2 is a 1D operator. '// &
               'If using k representation for gamma,'// &
               ' place jz^2 there instead of under beta. '// &
               'If using gamma representation directly, '// &
               ' use -dq^2 and place it there instead of under beta.'
            call warnmsg
         else
            loperr=.true.
            message= &
               'DVR cannot handle jz^2 operation.'
            call warnmsg
         endif

      else if (kword(1:2).eq.'p ' .or. kword(1:2).eq.'p^' &
                                  .or. kword(1:2).eq.'p%' ) then
         ipower=0
         if (kword(2:2) .eq. '^') then
            jlbl=index(kword,' ')-1
            read(kword(3:jlbl),*) ipower
         elseif (kword(2:2) .eq. ' ') then
            ipower = 1
         elseif (kword(2:2) .eq. '%') then
            goto 700
         endif
         if(ipower.lt.0) then
            message= 'Negative exponents on p are not allowed.'
            call warnmsg
         elseif(ipower .eq. 0) then
            write(kword,'(a)') '1'
         elseif(ipower .gt. 0 ) then
            if (basis .eq. 4) then
               write(kword,'(a,i3)') 'pfft^',ipower
               call rmblnk(kword)
            else
               if (ipower .eq. 1) then
                  kword='pdvr'
               else
                  message='Exponents on p are only allowed for FFT.'
                  call warnmsg
               endif
            endif
         endif
 700     if (kword(2:2) .eq. '%') then
            jlbl=index(kword,' ')-1
            read(kword(3:jlbl),*) ipower
            if (basis .eq. 4) then
               write(kword,'(a,i4)') 'pfft%',ipower
               call rmblnk(kword)
            else
               message='p% is only allowed for FFT.'
               call warnmsg
            endif
         endif

      else if (kword.eq.'j_z' .or. kword.eq.'j_z^2') then
         loperr=.true.
         message='Do you mean jz or jz^2 rather than j_z or j_z^2 ?'
         call warnmsg

      else if (kword.eq.'hKEh' .or. kword.eq.'hFRh' .or.&
               kword.eq.'hdqh' .or. kword.eq.'hdqRh') then
         if(basis .ne. 3 .or. basis /= 31 .or. basis /= 32) then
            loperr=.true.
            message='hKEh, hFRh, hdqh, and hdqRh require sin or Sinc or Wsinc-DVR'
            call warnmsg
         endif

      endif

      return
      end subroutine renterm

!#######################################################################
! 
! STORETM
!
! stores factors read from .op file in hterm array, and returns
! number of hterm, h
! optyp denotes the operation type. *=1, /=2, +=3, -=4
!
!#######################################################################

      subroutine storetm(factor,nfac,optyp,dof,h,muld,mode)

      implicit none

      integer(long)                                 :: n,hl,nhf,ilbl,jlbl,ipower
      integer(long), intent(in)                     :: nfac,dof,muld,mode
      integer(long), intent(out)                    :: h
      integer(long), dimension(nfac), intent(inout) :: optyp
      integer(long), dimension(:), allocatable      :: iterm

      real(dop)                                     :: power
      character(len=*), dimension(nfac), intent(out):: factor

      nhf=0

!.....First change "/" to a negative power.
! and work out length required for iterm
      do n=1,nfac
         if (optyp(n) .eq. 2) then
            ilbl=index(factor(n),'^')+1
            jlbl=index(factor(n),' ')-1
            if (ilbl .ne. 1) then
               read(factor(n)(ilbl:jlbl),*) power
               power=-power
            else
               ilbl=jlbl+1
               write(factor(n)(ilbl:ilbl),'(a1)') '^'
               ilbl=ilbl+1
               power=-1.0
            endif
            ipower = nint(power)
            if (abs(dble(ipower)-power) .lt. tiny) then
               write(factor(n)(ilbl:ilbl+2),'(i3)') ipower
            else
               write(factor(n)(ilbl:ilbl+6),'(f7.3)') power
            endif
            call rmblnk(factor(n))
            optyp(n)=1
         endif

         nhf=nhf+1
         if (nhf .eq. 1) then
            if (optyp(n) .eq. 4) then
               nhf=nhf+1
            endif
         else
            if (optyp(n) .eq. 3) then
               nhf=nhf+1
            else if (optyp(n) .eq. 4) then
               nhf=nhf+2
            endif
         endif
      enddo

!
! allocate temp iterm array
!
      allocate(iterm(nhf))
!
!.....Store the label factor(n) into hoplab and return its number hl.
!     and set up iterm
!
      nhf=0
      do n=1,nfac
         call hlabnum(hl,factor(n))

         nhf=nhf+1
         if (nhf .eq. 1) then
            if (optyp(n) .eq. 3) then
               iterm(nhf)=hl
            else if (optyp(n) .eq. 4) then
               iterm(nhf)=2
               iterm(nhf+1)=hl
               nhf=nhf+1
            endif
         else
            if (optyp(n) .eq. 1) then
               iterm(nhf)=hl
            else if (optyp(n) .eq. 3) then
               iterm(nhf)=0
               iterm(nhf+1)=hl
               nhf=nhf+1
            else if (optyp(n) .eq. 4) then
               iterm(nhf)=0
               iterm(nhf+1)=2
               iterm(nhf+2)=hl
               nhf=nhf+2
            endif
         endif
      enddo

!-----------------------------------------------------------------------
! find number of term in the hterm list
!-----------------------------------------------------------------------
      call htmnum(h,iterm,nhf,dof,mode,muld)
 
      if (allocated(iterm)) deallocate(iterm)

      return
      end subroutine storetm

!#######################################################################
!
!            Subroutine      HINTGEN
!
! Generates operators required by Quantics during run-time. 
! Not presently used.
!
!#######################################################################

      subroutine hintgen

      integer(long)      :: k

! Set term number to end of presently defined terms
      k = ktot
! First internal operator placed after user defined operators
      nhamint0= hamdim+1

      return
      end subroutine hintgen


      end module heintermmod

