!######################################################################
!
! MODULE  HEINUTIL.F
!
! CONTAINS: mvkend,compterm,htmnum,hlabnum,gethnum,
!           rdfnum,muldnum,flaguncorr,flagstates
!
!#######################################################################
      module heinutil

use decimal, only: long, dop
use global
use maxv
use dvrdatmod
use griddatmod
use operdef
use runopermod
use logdat
use op1lib,  only: zerovxi
use channels

      implicit none
      private
      public :: htmnum,hlabnum,gethnum,rdfnum,muldnum,flaguncorr,&
                muldnum2,inc_rdfiles
      contains

!#######################################################################
!
! COMPTERM
!
! compares two sets of integers (e.g. hterm terms), and returns
! .true. if they are identical
!
!#######################################################################

      logical(kind=4) function compterm(term1,term2,nfac1,nfac2)

      implicit none

      integer(long)                               :: n
      integer(long), intent(in)                   :: nfac1,nfac2
      integer(long), dimension(nfac1), intent(in) :: term1
      integer(long), dimension(nfac2), intent(in) :: term2

      if (nfac1 .eq. nfac2) then
         compterm=.true.
         do n=1,nfac1
            if (term1(n) .ne. term2(n)) compterm=.false.
         enddo
      else
         compterm=.false.
      endif

      end function compterm

!#######################################################################
!
! HTMNUM
!
! for input Hamiltonian term, searches hterm list and returns
! number of term in the list. If it is not listed, the term is
! added to the list.
!
! iterm(nitm) : input Hamiltonian term
! f           : degree of freedom relevant for term
! m           : mode for term if mode operator
!
! nhtm        : number of term in list
!
!#######################################################################

      subroutine htmnum(nhtm,iterm,itmfac,f,m,muld)

      implicit none

      integer(long)                                :: h,h1,n,f1,maxitm
      integer(long), intent(in)                    :: f,m,muld
      integer(long), intent(in)                    :: itmfac
      integer(long), intent(out)                   :: nhtm
      integer(long), dimension(itmfac), intent(in) :: iterm
      logical(kind=4)                              :: same

      h1=0
      do h=1,min(htmdim,maxhtm)
         same=.false.
         if (htmadof(h) .eq. f .and. htmmode(h) .eq. m .and. &
             htmmuld(h) .eq. muld) then
            same=compterm(iterm,hterm(:,h),itmfac,htmfac(h))
         endif

         if (same) h1=h
      enddo

!-----------------------------------------------------------------------
! add to list
!-----------------------------------------------------------------------
      if (h1 .eq. 0) then

!-----------------------------------------------------------------------
! if a mode operator, top of list of factors is maxhtm, else top of
! list is maxhop
!-----------------------------------------------------------------------
         if (m.gt.0) then
            maxitm=maxhtm
         else
            maxitm=maxhop
         endif

         htmdim=htmdim+1
         minfac = max(minfac,itmfac+1)

         if (htmdim .gt. maxhtm) call inc_hterm
         h1 = htmdim
         htmfac(h1)=min(itmfac,maxfac)

         if (m.ne.0 .or. muld.ne.0) then
!           by convention, htmdof=0 for mode operators
            f1=0
         else
            f1=f
         endif
         htmdof(h1)=f1
         htmadof(h1)=f
         htmmode(h1)=m
         htmmuld(h1)=muld

         do n=1,htmfac(h1)
            hterm(n,h1)=min(iterm(n),maxitm)
         enddo
         n=min(maxfac,htmfac(h1)+1)             ! **
         hterm(n,h1)=0              ! **
      endif

      nhtm=h1

      end subroutine htmnum

!#######################################################################

      subroutine inc_hterm

      implicit none

      integer(long)     :: ierr,i,j,oldhtm,oldfac
      character(len=10) :: cerr

      integer(long), dimension(:), allocatable   :: tmpdof,tmpadof,tmpmode,&
                                                    tmpmuld,tmpfac,tmpform,&
                                                    tmpsym,tmpshift,tmptype,&
                                                    tmpndof,tmpderiv
      integer(long), dimension(:,:), allocatable :: tmphterm
!
! allocate temp arrays
!
      cerr = 'tmphterm'
      allocate(tmphterm(maxfac,maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpdof'
      allocate(tmpdof(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpadof'
      allocate(tmpadof(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpmode'
      allocate(tmpmode(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpmuld'
      allocate(tmpmuld(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpfac'
      allocate(tmpfac(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpform'
      allocate(tmpform(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpsym'
      allocate(tmpsym(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpshift'
      allocate(tmpshift(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmptype'
      allocate(tmptype(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpndof'
      allocate(tmpndof(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpderiv'
      allocate(tmpderiv(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
!
! copy hterm arrays
!
      do i=1,maxhtm
         do j=1,maxfac
            tmphterm(j,i) = hterm(j,i)
         enddo
         tmpdof(i) = htmdof(i)
         tmpadof(i) = htmadof(i)
         tmpmode(i) = htmmode(i)
         tmpmuld(i) = htmmuld(i)
         tmpfac(i) = htmfac(i)
         tmpform(i) = htmform(i)
         tmpsym(i) = htmsym(i)
         tmpshift(i) = htmshift(i)
         tmptype(i) = htmtype(i)
         tmpndof(i) = htmndof(i)
         tmpderiv(i) = htmderiv(i)
      enddo

!
! allocate new hterm arrays
!
      oldhtm=maxhtm
      oldfac=maxfac
      maxhtm = htmdim
      maxfac = max(maxfac,minfac)

      call alloc_hterm

      hterm(:,:) = 0
      htmdof(:) = 0
      htmadof(:) = 0
      htmmode(:) = 0
      htmmuld(:) = 0
      htmfac(:) = 0
      htmform(:) = 0
      htmsym(:) = 0
      htmshift(:) = 0
      htmtype(:) = -1
      htmndof(:) = 0
      htmderiv(:) = 0
!
! copy hterm values to new arrays
!
      do i=1,oldhtm
         do j=1,oldfac
            hterm(j,i) = tmphterm(j,i)
         enddo
         htmdof(i) = tmpdof(i)
         htmadof(i) = tmpadof(i)
         htmmode(i) = tmpmode(i)
         htmmuld(i) = tmpmuld(i)
         htmfac(i) = tmpfac(i)
         htmform(i) = tmpform(i)
         htmsym(i) = tmpsym(i)
         htmshift(i) = tmpshift(i)
         htmtype(i) = tmptype(i)
         htmndof(i) = tmpndof(i)
         htmderiv(i) = tmpderiv(i)
      enddo
!
! deallocate tmp arrays
!
      deallocate(tmphterm,tmpdof,tmpadof,tmpmode,tmpmuld,tmpfac,tmpform,&
                 tmpsym,tmpshift,tmptype,tmpndof,tmpderiv)

      return
 1000 write(ilog,'(a)') 'Error allocating memory in INC_HTERM :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop

      end subroutine inc_hterm

!#######################################################################
!
! HLABNUM
!
! for input Hamiltonian label, searches hoplab list and returns
! number of label in the list. If it is not listed, the label is
! added to the list.
!
! label   : input Hamiltonian label
! nlab    : number of label in list
! 
!#######################################################################

      subroutine hlabnum(nlab,label)

      implicit none

      integer(long)                 :: h,h1
      integer(long), intent(out)    :: nlab
      character(len=c2), intent(in) :: label

      h1=0
      do h=1,min(hopdim,maxhop)
         if (label .eq. hoplab(h)) h1=h
      enddo
      if (h1 .eq. 0) then
         hopdim=hopdim+1
         if (hopdim .gt. maxhop) call inc_hlab
         h1 = hopdim

         hoplab(h1)=label
      endif

      nlab=h1

      return
      end subroutine hlabnum

!#######################################################################

      subroutine inc_hlab

      implicit none

      integer(long)     :: ierr,i,j
      character(len=10) :: cerr

      character(len=c2), dimension(:), allocatable :: tmphoplab
      integer(long), dimension(:), allocatable     :: tmpfile,tmpilab,tmpidef
      integer(long), dimension(:,:), allocatable   :: tmpipar
      real(dop), dimension(:,:), allocatable       :: tmprpar,tmppar
!
! allocate temp arrays
!
      cerr = 'tmphoplab'
      allocate(tmphoplab(maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpfile'
      allocate(tmpfile(maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpilab'
      allocate(tmpilab(maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpidef'
      allocate(tmpidef(maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpipar'
      allocate(tmpipar(maxhpar,maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmprpar'
      allocate(tmprpar(maxhpar,maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmppar'
      allocate(tmppar(maxhpar,maxhop),stat=ierr)
      if (ierr > 0) go to 1000
!
! copy hoplab array
!
      do i=1,maxhop
         tmphoplab(i) = hoplab(i)
         tmpfile(i) = hopfile(i)
         tmpilab(i) = hopilab(i)
         tmpidef(i) = hopidef(i)
         do j=1,maxhpar
            tmpipar(j,i) = hopipar(j,i)
            tmprpar(j,i) = hoprpar(j,i)
            tmppar(j,i) = hoppar(j,i)
         enddo
      enddo
!
! allocate new hterm arrays
!
      maxhop = hopdim
      call alloc_hoplab

      hoplab(:) = ' '
      hopfile(:) = 0
      hopilab(:) = 0
      hopidef(:) = 0
      hopipar(:,:) = 0
      hoprpar(:,:) = 0.0_dop
      hoppar(:,:) = 0.0_dop
!
! copy hterm values to new arrays
!
      do i=1,maxhop-1
         hoplab(i) = tmphoplab(i)
         hopfile(i) = tmpfile(i)
         hopilab(i) = tmpilab(i)
         hopidef(i) = tmpidef(i)
         do j=1,maxhpar
            hopipar(j,i) = tmpipar(j,i)
            hoprpar(j,i) = tmprpar(j,i)
            hoppar(j,i) = tmppar(j,i)
         enddo
      enddo
!
! deallocate tmp arrays
!
      deallocate(tmphoplab,tmpfile,tmpilab,tmpidef,tmpipar,tmprpar,tmppar)

      return
 1000 write(ilog,'(a)') 'Error allocating memory in INC_HLAB :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop

      end subroutine inc_hlab

!#######################################################################
!
! GETHNUM
!
! for input Hamiltonian label, searches hamlab list and returns
! number of label in the list. If it is not listed, the label is
! added to the list.
!
! label   : input Hamiltonian label
! nham    : number of label in list
!
!#######################################################################

      subroutine gethnum(nham,label)

      implicit none

      integer(long)                :: h,h1
      integer(long), intent(out)   :: nham
      character(len=*), intent(in) :: label

      h1=0
      do h=1,min(hamdim,maxham)
         if (label .eq. hamlab(h)) h1=h
      enddo
      if (h1 .eq. 0) then
         hamdim=hamdim+1
         h1 = min(hamdim,maxham)
         hamlab(h1)=label
      endif

      nham=h1

      return
      end subroutine gethnum

!#######################################################################
!
! for natpot or rdfile file-names, searches rdfiles list and returns
! number of label in the list. If it is not listed, the label (i.e.
! filename) is added to the list.
! 
!#######################################################################

      subroutine rdfnum(nlab,label)

      implicit none

      integer(long)                        :: n,n1
      integer(long), intent(out)           :: nlab
      character(len=c5), intent(in)        :: label

      n1=0
      do n=1,min(nrdf,maxrdf)
         if (label .eq. rdfiles(n)) n1=n
      enddo
      
      if (n1 .eq. 0) then
         nrdf=nrdf+1
         if (nrdf .gt. maxrdf) call inc_rdfiles
         n1=nrdf

         rdfiles(n1)=label
      endif

      nlab=n1

      return
      end subroutine rdfnum

!#######################################################################
!
! INC_RDFILES
!
! Routine to increase size of arrays holding info on files read in
! and operator labels. Required to provide dynamic allocation
!
!#######################################################################

      subroutine inc_rdfiles

      implicit none

      integer(long)     :: ierr,i,j
      character(len=15) :: cerr

      character(len=c5), dimension(:), allocatable :: tmprdfiles
      real(dop), dimension(:), allocatable         :: tmpnpotcut
      integer(long), dimension(:), allocatable     :: tmpnpignore
      integer(long), dimension(:,:), allocatable   :: tmpnpmuld
      integer(vlong), dimension(:), allocatable    :: tmpnplocal
      integer(long), dimension(:), allocatable     :: tmpncutted

      integer(long), dimension(:), allocatable     :: tmpmmodc
      integer(long), dimension(:,:), allocatable   :: tmpvmdim
      integer(long), dimension(:,:), allocatable   :: tmpmdim
      integer(long), dimension(:,:), allocatable   :: tmpnmdim

      integer(long), dimension(:,:), allocatable   :: tmpzpesopts
      integer(long), dimension(:), allocatable     :: tmpnopts
      character(len=c5), dimension(:), allocatable :: tmppesopts

      real(dop), dimension(:), allocatable         :: tmpncutrlabel
      character(len=c2), dimension(:), allocatable :: tmpncutlabel
      real(dop), dimension(:,:), allocatable       :: tmpwpf
!
! allocate temp arrays
!
      cerr = 'tmprdfiles'
      allocate(tmprdfiles(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpnpotcut'
      allocate(tmpnpotcut(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpnpignore'
      allocate(tmpnpignore(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpnpmuld'
      allocate(tmpnpmuld(maxrdf,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpnplocal'
      allocate(tmpnplocal(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpncutted'
      allocate(tmpncutted(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpmmodc'
      allocate(tmpmmodc(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpvmdim'
      allocate(tmpvmdim(maxdim,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpmdim'
      allocate(tmpmdim(maxdim,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpnmdim'
      allocate(tmpnmdim(maxdim,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000

      cerr = 'tmpzpesopts'
      allocate(tmpzpesopts(2*maxhpar,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpnopts'
      allocate(tmpnopts(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmppesopts'
      allocate(tmppesopts(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000

      cerr = 'tmpncutrlabel'
      allocate(tmpncutrlabel(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpncutlabel'
      allocate(tmpncutlabel(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tmpwpf'
      allocate(tmpwpf(maxdim,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
!
!
! copy arrays
!
      do i=1,maxrdf
         tmprdfiles(i) = rdfiles(i)
         tmpnpotcut(i) = npotcut(i)
         tmpnpignore(i) = npignore(i)
         do j=1,maxrdf
            tmpnpmuld(j,i) = npmuld(j,i)
         enddo
         tmpnplocal(i) = nplocal(i)
         tmpncutted(i) = ncutted(i)
         tmpmmodc(i) = mmodc(i)
         tmpvmdim(:,i) = vmdim(:,i)
         tmpmdim(:,i) = mdim(:,i)
         tmpnmdim(:,i) = nmdim(:,i)

         tmpzpesopts(:,i) = zpesopts(:,i)
         tmpnopts(i) = nopts(i)
         tmppesopts(i) = pesopts(i)

         tmpncutrlabel(i) = ncutrlabel(i)
         tmpncutlabel(i) = ncutlabel(i)
         tmpwpf(:,i) = wpf(:,i)
      enddo
!
! allocate new arrays
!
      maxrdf = maxrdf+1
      call alloc_rdfiles

      rdfiles(:) = ' '
      npotcut(:) = 0.0_dop
      npignore(:) = 0
      npmuld(:,:) = 0
      nplocal(:) = 0
      ncutted(:) = 0 
      mmodc(:) = 0
      vmdim(:,:) = 0
      mdim(:,:) = 0
      nmdim(:,:) = 0

      zpesopts(:,:) = 0
      nopts(:) = 0
      pesopts(:) = ' '

      ncutrlabel(:) = 0.0_dop
      ncutlabel(:) = ' '
      wpf(:,:) = 0.0_dop
!
! copy values to new arrays
!
      do i=1,maxrdf-1
         rdfiles(i) = tmprdfiles(i)
         npotcut(i) = tmpnpotcut(i)
         npignore(i) = tmpnpignore(i)
         do j=1,maxrdf-1
            npmuld(j,i) = tmpnpmuld(j,i)
         enddo
         nplocal(i) = tmpnplocal(i)
         ncutted(i) = tmpncutted(i)
         mmodc(i) = tmpmmodc(i)
         vmdim(:,i) = tmpvmdim(:,i)
         mdim(:,i) = tmpmdim(:,i)
         nmdim(:,i) = tmpnmdim(:,i)

         zpesopts(:,i) = tmpzpesopts(:,i)
         nopts(i) = tmpnopts(i)
         pesopts(i) = tmppesopts(i)

         ncutrlabel(i) = tmpncutrlabel(i)
         ncutlabel(i) = tmpncutlabel(i)
         wpf(:,i) = tmpwpf(:,i)
      enddo
!
! deallocate tmp arrays
!
      deallocate(tmprdfiles,tmpnpotcut,tmpnpignore,tmpnpmuld,tmpnplocal)
      deallocate(tmpncutted,tmpmmodc,tmpvmdim,tmpmdim,tmpnmdim)
      deallocate(tmpzpesopts,tmpnopts,tmppesopts)
      deallocate(tmpncutrlabel,tmpncutlabel,tmpwpf)

      return
 1000 write(ilog,'(a)') 'Error allocating memory in INC_RDFILES :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop

      end subroutine inc_rdfiles

!#######################################################################
! MULDNUM
!
! match muld information to list of mulds. If not present, set up new
! muld. If muld matches mode, return mode no. Called from heinterms.
!
! mulddof2(n,k) lists DOF n for grid k
! mulddof(n,k)  is an ordered version of mulddof2
!#######################################################################

      subroutine muldnum(muld,grid,mode)

      implicit none

      integer(long)                                :: gf,f,m,gm,muld1,m1,&
                                                      nmulddof1,&
                                                      nmuldmode1,n,gf2
      integer(long), intent(out)                   :: muld,mode
      integer(long), dimension(maxdim)             :: muldmode1
      integer(long), dimension(maxdim), intent(in) :: grid
      logical(kind=4)                              :: same,found

      muld=0
      mode=0

! see if grid is in the list
      do muld1=1,mulddim
         same=.true.
         do gf=1,ndof
            if (mulddof2(gf,muld1) .ne. grid(gf)) same=.false.
         enddo
         if (same) muld=muld1
      enddo

      if (muld .ne. 0) return

! count no. of dofs and find which modes are in grid
      nmulddof1=0
      nmuldmode1=0
      call zerovxi(muldmode1,maxdim)
      do gf=1,ndof
         f=grid(gf)
         if (f .ne. 0) then
            nmulddof1=nmulddof1+1

            m=dofspf(f)
            if (m .eq. 0) cycle
            found=.false.
            do gm=1,nmuldmode1
               m1=muldmode1(gm)
               if (m .eq. m1) found=.true.
            enddo
            if (.not. found) then
               nmuldmode1=nmuldmode1+1
               gm=nmuldmode1
               muldmode1(gm)=m
            endif
         endif
      enddo

! see if muld matches a single mode
      mode=0
      if (nmuldmode1 == 1) then
         mode=muldmode1(1)
         m=mode
         do n=1,nspfdof(m)
            if (spfdof(n,m) .ne. grid(n)) mode=0
         enddo
      endif
      if (mode /= 0) then
         muld=0
         return
! muldnum2 may later set muld for a potfit operating on a mode.
      endif

! if grid not yet assigned, add to list
      mulddim=mulddim+1
      muld=min(mulddim,maxmuld)
      nmulddof(muld)=nmulddof1
      do gf=1,nmulddof1
         mulddof2(gf,muld)=grid(gf)
      enddo

!-----------------------------------------------------------------------
! Set MULDDOF, the form of mulddof2 with DOFs ordered is as in dynamics
! calculation rather than as in the Hamiltonian.
! Note that dofs not included in dynamics are not included in mulddof
!-----------------------------------------------------------------------
      gf2=0
      do m=1,nopmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            do gf=1,nmulddof(muld)
               if( f.eq.mulddof2(gf,muld) ) then
                  gf2=gf2+1
                  mulddof(gf2,muld) = f
               endif   
            enddo
         enddo
      enddo


! calculate grid size. Cutoff size if too large
      muldsize(muld)=1
      do gf=1,nmulddof(muld)
         f=mulddof2(gf,muld)
         if (muldsize(muld) .le. 199999999/gdim(f)) then
            muldsize(muld)=muldsize(muld)*gdim(f)
         else
            muldsize(muld)=199999999
         endif
      enddo

      return
      end subroutine muldnum

!#######################################################################
! MULDNUM2
!
! Like MULDNUM this routine match muld information to list of mulds.
! If not present, set up new muld. This routine is for natpots with
! ignore. If those match a mode, mulddof information is still needed.
! Called from matchlab.
!
! grid(*)=spfdof(*,m), nmulddof1=nspfdof(m)
!
!#######################################################################

      subroutine muldnum2(muld,grid,nmulddof1)

      implicit none

      integer(long)                                :: gf,f,m,muld1,n,gf2
      integer(long), intent(out)                   :: muld
      integer(long), intent(in)                    :: nmulddof1
      integer(long), dimension(maxdim), intent(in) :: grid
      logical(kind=4)                              :: same

      muld=0

! see if grid is in the list
      do muld1=1,mulddim
         same=.true.
         do gf=1,ndof
            if (mulddof2(gf,muld1) .ne. grid(gf)) same=.false.
         enddo
         if (same) muld=muld1
      enddo

      if (muld .ne. 0) return

! if grid not yet assigned, add to list
      mulddim=mulddim+1
      muld=min(mulddim,maxmuld)
      nmulddof(muld)=nmulddof1
      do gf=1,nmulddof1
         mulddof2(gf,muld)=grid(gf)
      enddo

! Set mulddof, the ordered form of mulddof2
      gf2=0
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            do gf=1,nmulddof(muld)
               if( f.eq.mulddof2(gf,muld) ) then
                  gf2=gf2+1
                  mulddof(gf2,muld) = f
               endif
            enddo
         enddo
      enddo

! calculate grid size. Cutoff size if too large
      muldsize(muld)=1
      do gf=1,nmulddof(muld)
         f=mulddof2(gf,muld)
         if (muldsize(muld) .le. 199999999/gdim(f)) then
            muldsize(muld)=muldsize(muld)*gdim(f)
         else
            muldsize(muld)=199999999
         endif
      enddo

      return
      end subroutine muldnum2


!#######################################################################
!
! FLAGUNCORR
!
! finds out which Hamiltonian terms are uncorrelated 1D operators.
! needs as input/output the array funcorr(maxkoe)
! funcorr(k)=0 : correlated operator term
! funcorr(k)=f : the only non-unit operator is in DoF f.
! funcorr(k)=fkoe : all operators are unit operators (constant term).
!
!#######################################################################

      subroutine flaguncorr(funcorr,state,state1)

      implicit none

      integer(long)                                 :: nuncorr,k,h,s,s1,f
      integer(long), intent(in)                     :: state,state1
      integer(long), dimension(maxkoe), intent(out) :: funcorr

      do k=1,ktot
         if (hmlop(k).ne.0) then
            funcorr(k)=0
            cycle
         endif

         nuncorr=0
         do f=1,ndof
            h=hamilton(f,k)
            if(h.lt.0) then   ! LMR
               funcorr(k)=0   ! LMR
               nuncorr=2      ! LMR
               goto 123       ! LMR
            endif
            if (lmult .and. f.eq. feb) then
               continue
            else if (h .ne. f) then
               nuncorr=nuncorr+1
               funcorr(k)=f
! if mode operator, cannot be uncorrelated
               if (htmdof(h) .eq. 0) then
                  nuncorr=nuncorr+2
               endif
            endif
 123        continue
         enddo

         if (lmult) then
            call flagstates(k,s,s1)
            if (s .ne. state) nuncorr=nuncorr+2
            if (s1 .ne. state1) nuncorr=nuncorr+2
            if (s .ne. s1) nuncorr=nuncorr+2
         endif

         if (nuncorr .gt. 1) then
            funcorr(k)=0
         else if (nuncorr .eq. 0) then
            funcorr(k)=fkoe
         endif

      enddo

      return
      end subroutine flaguncorr

!#######################################################################
!
! FLAGSTATES
!
! for multi-set, find out which states Hamiltonian term belongs to
!
!#######################################################################

      subroutine flagstates(k,state,state1)

      implicit none

      integer(long)              :: h,hl,ilbl,jlbl,klbl
      integer(long), intent(in)  :: k
      integer(long), intent(out) :: state,state1

      ilbl=0
      h=hamilton(feb,k)
      hl=hterm(1,h)
      if (hoplab(hl)(1:1) .eq. 'S' .or. hoplab(hl)(1:1) .eq. 'Z') then
         ilbl=2
      else if (hoplab(hl)(1:2) .eq. 'CE' .or.  &
             (hoplab(hl)(1:2) .eq. 'CS'.and. hoplab(hl)(3:3) .ne. '_') .or. &
             hoplab(hl)(1:2) .eq. 'CZ') then
          ilbl=3
!-Alice-cs_bath
      else if (hoplab(hl)(1:7) .eq. 'CS_bath') then
         ilbl=8
      endif
      if (ilbl .gt. 0) then
         jlbl=index(hoplab(hl),'&')
         klbl=index(hoplab(hl),' ')-1
         read(hoplab(hl)(ilbl:jlbl-1),*) state
         read(hoplab(hl)(jlbl+1:klbl),*) state1
      endif

      return
      end subroutine flagstates


      end module heinutil


