!#######################################################################
! contains:htmsetup,htmprodpot,htmprodsd,htmgen1d,htmgen1m,gen1dhtm2,genmodehtm2,checkmodejm,
! checkjm,htmpot,getoptype,wrscratch,rddiag,rdzdiag,rdzsdiag,wrdiag,potgen1d,genhtm2kleg,htmgensubp
!#######################################################################

      module htmsetupmod

use decimal, only: long, dop
use logdat

use openmpmod

use global
use maxv, only: maxdim,maxhtm, maxfac
use dvrdatmod
use griddatmod
use operdef
use runopermod
use combined
use channels

use rdopermod, only: assocdof
use logopmod, only: outhtm
use operutil, only: findnath,getvgn,findcpdh,findhcpdh,findread,findreadmap
use asshop, only: assdzhtm,assigndhtm,asshtmsym,assignzhtm,asszdiag,asshtmlab
use fbrintmod, only: fbrint
use potevalmod, only: poteval
use callanldmod, only: callanld
use callgrdmod, only: callgrd,callreadmap
use callanlzmod, only: callanlz
use funcsrf, only: read1dinfo,vpoint
use funcklegmod, only: funckleg
use funcwigmod, only: funcwig
use func2klegmod, only: func2kleg
use funcanldmod, only: funcanld

use mmlib,   only: ddxxdd, qdxxdd, qqxxdd, qqxxzz, qdxxzz
use mtlib,   only: dtxxzzo, dtxxddo
use xvlib,   only: xqxxdd, xvxxzzo, xvxxddo
use op1lib,  only: cpvxz, initvxl, cpvxzd, initvxz, zerovxz, unitqxd, zerovxd, &
                   zeromxz, initvxd, cpvxdz, unitqxz, cpvxd

use sdlib,   only: ooxxzzt
use op2lib,  only: addvxxzo, addmxxzo, addvxxdo

      implicit none
      private
      public :: htmsetup,htmgen1d,checkjm,rdzdiag,gethtmtype,checkmodejm
      contains
      
!#######################################################################
! HTMSETUP
!
! Set up hterms, i.e. generate all neccesary operators.
! The results are written to the scratch file on unit itmpop.
!
!#######################################################################

      subroutine htmsetup(ort,dvrmat,fftp,kinsph,rdop,jsph,msph,trafo,done)

      implicit none

      integer(long)       :: f,h,m,muld,jmax,k,f1,m1,zdim,f2,irmvd,&
                             j25,jq,sdm,htmoptyp,kdim,nnat,maxd,&
                             ncpd,nrd,nmap,imlop,ifunc,nel,nhcpd
      integer(long)       :: gdim1,gdim2,kdim1,kdim2,i,n,f3,mmdim

      integer(long), dimension(sphdim), intent(in)   :: jsph, msph
      integer(long), dimension(maxhop), intent(inout):: done
      real(dop)                                      :: ctff
      real(dop), dimension(ortdim), intent(inout)    :: ort
      real(dop), dimension(dvrdim,dvrord), intent(in):: dvrmat
      real(dop), dimension(dvrdim), intent(in)       :: trafo
      real(dop), dimension(fftdim), intent(in)       :: fftp
      real(dop), dimension(sphdim), intent(in)       :: kinsph
      real(dop), dimension(rdopdim), intent(in)      :: rdop
      logical(kind=4)                                :: lz,ldum,lzero
      logical(kind=4), dimension(maxhtm)             :: htmcalc

      character(len=20)                          :: cerr
      integer(long)                              :: ierr

      complex(dop), dimension(:), allocatable    :: hopz
      complex(dop), dimension(:), allocatable    :: hopz1
      real(dop), dimension(:), allocatable       :: hopd

      routine='Htmsetup'
      j25 = 0
      irmvd=0

      ierr=0
      if(allocated(hopz))deallocate(hopz,stat=ierr)
      if(allocated(hopz1))deallocate(hopz1,stat=ierr)
      if(allocated(hopd))deallocate(hopd,stat=ierr)
      if (ierr > 0) go to 1001

!-----------------------------------------------------------------------
! see if it is neccesary to calculate a hterm:
!  only calculate operators that are present in Hamiltonian. 
!             Always calculate 1D unit operators (h .le. nopdof).
!-----------------------------------------------------------------------
      call initvxl(htmcalc,maxhtm)
      do h=1,htmdim
         if (h .le. nopdof) then
            htmcalc(h)=.true.
         elseif (htmtype(h) .gt. 100 .and. htmtype(h).lt.200) then ! GWP and on-the-fly
            htmcalc(h)=.false.
         elseif (htmtype(h) .eq. 202 .or. htmtype(h).eq.204) then
            htmcalc(h)=.false.
         else
            htmcalc(h)=.false.
            do k=1,ktot
               do f=1,nopdof
                  if (h .eq. hamilton(f,k)) then
                     htmcalc(h)=.true.
                     goto 111
                  endif
               enddo
            enddo
            do i=1,iLMR
               do n=1,3
                  if (h .eq. hamLMR(i,n)) then
                     htmcalc(h)=.true.
                     goto 111
                  endif
               enddo
            enddo
  111       continue
         endif
      enddo

!-----------------------------------------------------------------------
!    set up hamiltonian operators
!-----------------------------------------------------------------------
      do h=1,htmdim
         if (.not. htmcalc(h) .and. lrealwp) go to 200 ! this line is included
         if (.not. htmcalc(h)) go to 100

!-----------------------------------------------------------------------
! get info on associated dof/mode
!-----------------------------------------------------------------------
         call assocdof(h,f,m,muld,f1,m1,sdm)

!-----------------------------------------------------------------------
! get operator type and natpot-number (nnat=0 if h is not a natpot)
! (nnat=0 if h is not a natpot)
! (imlop=0 if h is not part of an MLOp)
!-----------------------------------------------------------------------
         call getoptype(htmoptyp,h,f,m,muld,f1)
         lz=.false.
         call findnath(h,nnat,imlop)
         call findcpdh(h,ncpd)
         call findhcpdh(h,nhcpd)
         call findread(h,nrd)
         call findreadmap(h,nmap)
!         if(nmap.gt.0) htmtype(h)=25  ! already done in asshtmlab

!-----------------------------------------------------------------------
! check that operator allowed
!-----------------------------------------------------------------------
         if (htmoptyp .eq. 1 .and. muld .ne. 0) then
            write(message,'(a,i5)') &
                'Muld operator cannot use general 1D routines:',h
            call errormsg
         endif

!-----------------------------------------------------------------------
! generate operator
!-----------------------------------------------------------------------
! set f3 - required for realwp + j^2 operators
         f3=max(1,htmadof(h))
         if (htmtype(h) .eq. 25) then ! readmap operator
            nel = rdtype(hopilab(hterm(1,h)))-100
            sdm = 3*nel+1
            cerr = 'hopd '
            allocate(hopd(sdm),stat=ierr)
            if (ierr > 0) go to 1000
            ifunc=hopilab(hterm(1,h))
            call callreadmap(rdop(zrdop(ifunc)),hopd,gdim(htmdof(h)), &
                             hoplab(hterm(1,h)))

         else if (htmoptyp .eq. 0) then ! Analytical
            continue

         else if (htmoptyp .eq. 1) then ! all 1D operators
            cerr = 'hopz '
            allocate(hopz(sdm*sdm),stat=ierr)
            if (ierr > 0) go to 1000
            cerr = 'hopz1 '
            allocate(hopz1(sdm),stat=ierr)
            if (ierr > 0) go to 1000
            cerr = 'hopd '
            allocate(hopd(sdm),stat=ierr)
            if (ierr > 0) go to 1000

            if ((f1 .eq. fkoe) .or. (f1 .eq. ftime)) then
! koefficient DOF has no grid or DVR - pass arrays but they are not used
               call htmgen1d(h,f1,hopz,ort,dvrmat, &
                    dvrmat,fftp,kinsph, &
                    rdop,sdm,hopd,lz,'htmsetup    ',trafo,done)
            else
               call htmgen1d(h,f1,hopz,ort(zort(f1)),dvrmat(zdvr(f1),1), &
                    dvrmat(zdvr(f1),2),fftp(zfft(f1)),kinsph(zsph(f1)), &
                    rdop,sdm,hopd,lz,'htmsetup    ',trafo(zdvr(f1)),done)
            endif
            if(htmtype(h).eq.25) cycle  ! readmap operator
            if (lz) then
               call rdzdiag(hopz,hopz1,sdm)
               call cpvxz(hopz1,hopz,sdm)
            elseif (htmshift(h).ne.0) then
               call rdzsdiag(hopz,hopz1,htmshift(h),sdm)
               call cpvxz(hopz1,hopz,sdm)
               call asszdiag(hopz,hopd,sdm,ldum,lz,lzero)
               if(lzero) then
                  htmtype(h)=0
                  nremoved=nremoved+1
                  call outhtm(h,ilog)
               endif
            endif


         else if (htmoptyp .eq. 2) then ! all muld operators

!           if combined (natpot/cpd) operators are contained and
!           the operator is a normal multi-dimensional function
            if ( (htmform(h).eq.0) .and. (nnat.gt.0.or.ncpd .gt.0 &
                 .or.nhcpd .gt.0))then
               if (htmtype(h).eq.11 .and. nspfdof(m1).gt.2) then
                  f2 = htmadof(h) ! NEWKLEG
                  sdm=vgdim(f2+1)*ngdim(f2+1)
               endif
               cerr = 'hopd '
               allocate(hopd(sdm),stat=ierr)
               if (ierr > 0) go to 1000
               call htmgen1m(h,rdop,sdm,hopd)

!           if combined read operators are contained and
!           the operator is a real multi-dimensional matrix from a
!           hcpd file
            elseif( (htmform(h).eq.0) .and. (nrd.gt.0))then
               call htmgenreadm(h,rdop,sdm,hopz)

!           for j^2kleg and jpm,j_p,j_m,jp^2,jm^2 in real wavepacket
            else if (lrealwp .and. (basis(f3) == 7.or.basis(f3) == 20 .or. &
                     basis(f3) == 9 .or. basis(f3) == 21)) then
               if (basis(f3).eq.7 .or. basis(f3).eq.20) then
                  kdim=gdim(f3+1)
               else if (basis(f3).eq.9 .or. basis(f3).eq.21) then
                  kdim=ipbaspar(3,f3+1)
               endif
               cerr = 'hopd '
               allocate(hopd(sdm),stat=ierr)
               if (ierr > 0) go to 1000
!--allocating memmory for hopz which is not used here!!!
               cerr = 'hopz '
               allocate(hopz(1),stat=ierr)
               if (ierr > 0) go to 1000
               if (htmform(h) == 0) then
                  call gen1dhtm2rw(h,f3,hopd,ort(zort(f3)),&
                       dvrmat(zdvr(f3),1),dvrmat(zdvr(f3),2),gdim(f3),kdim,sdm,1)
               else if (htmform(h) == 1) then
                  call genmodehtm2rw(h,hopd,ort,dvrmat,fftp,kinsph,&
                       rdop,gdim(f3),kdim,sdm,done)
               endif

!           if MLOp is contained and
!           the operator is a normal multi-dimensional function
            elseif ( (htmform(h).eq.0) .and. (imlop.gt.0) ) then
               cerr = 'hopd '
               allocate(hopd(sdm),stat=ierr)
               if (ierr > 0) go to 1000
               call htmgenmlop(h,sdm,hopd)

!           if combined (natpot) operators are NOT contained and
!           the operator is a normal multi-dimensional function
            else if (htmform(h) .eq. 0) then
               cerr = 'hopd '
               allocate(hopd(sdm),stat=ierr)
               if (ierr > 0) go to 1000
               cerr = 'hopz '
               allocate(hopz(sdm),stat=ierr)
               if (ierr > 0) go to 1000
               call htmpot(h,ort,hopd,hopz,sdm,lz)

!           a product form operator is being summed
            else if (htmform(h) .eq. 1) then
               maxd=max(maxgdim2,maxsubpdim)
               cerr = 'hopd '
               allocate(hopd(sdm),stat=ierr)
               if (ierr > 0) go to 1000
               cerr = 'hopz '
               allocate(hopz(sdm),stat=ierr)
               if (ierr > 0) go to 1000
               if (htmtype(h).eq.12) then
                  call htmprodsd(h,hopz,hopd,ort,dvrmat,fftp,kinsph, &
                       rdop,sdm,lz,done)
               else
                  call htmprodpot(h,hopz,hopd,ort,dvrmat,fftp,kinsph, &
                       rdop,sdm,lz,maxd,done)
               endif
            endif

         else if (htmoptyp .eq. 3) then ! some mode operators (others are=1)
            if(j25.eq.0) write(ilog,'(/,a,1pe10.3,a,i3)')&
                ' ### sphFBR-Integrals threshold :',rpbaspar(1,f1),&
                ',  j-offset =',ipbaspar(3,f1)
            jmax=ipbaspar(2,f1)
            j25 = jmax + 25
            jq  = j25**2 + 1
            cerr = 'hopz '
            allocate(hopd(1),stat=ierr) ! not used but must be allocated
            allocate(hopz(sdm*sdm),stat=ierr)
            if (ierr > 0) go to 1000
            call fbrint(h,jsph(zsph(f1)),msph(zsph(f1)),jmax,hopz,sdm,f1,j25)
            call assignzhtm(hopz,htmtype(h),htmshift(h),sdm)
            if(htmtype(h).eq.0) call outhtm(h,ilog) ! write removed operators

         else if (htmoptyp .eq. 4) then ! 3D tensors jpm,jm^2,j_p,j_m,jpjm etc. & 
                                        ! for KLeg and PLeg
            f1=htmadof(h) ! NEWKLEG
            if (basis(f1).eq.7 .or. basis(f1).eq.20) then
                kdim=gdim(f1+1)
            elseif (basis(f1).eq.9 .or. basis(f1).eq.21) then
                kdim=ipbaspar(3,f1+1)
            endif
            zdim=gdim(f1)**2*kdim
            cerr = 'hopd '
            allocate(hopd(zdim),stat=ierr)
            if (ierr > 0) go to 1000
!--allocating memmory for hopz which is not used here!!!
            cerr = 'hopz '
            allocate(hopz(1),stat=ierr)
            if (ierr > 0) go to 1000
            if (htmform(h) .eq. 0) then
               call gen1dhtm2(h,f1,hopd,ort(zort(f1)),&
                    dvrmat(zdvr(f1),1),dvrmat(zdvr(f1),2),gdim(f1),kdim)

            else if (htmform(h) .eq. 1) then ! j^2, j_p,.. for combined modes.
               call genmodehtm2(h,hopd,ort(zort(f1)),dvrmat,fftp,kinsph,&
                    rdop,gdim(f1),kdim,done)

            endif

         else if (htmoptyp .eq. 5) then ! combined KLeg types
            f1=htmadof(h)
            gdim1=gdim(f1)
            kdim1=gdim(f1+1)
            gdim2=gdim(f1+2)
            kdim2=gdim(f1+3)
            zdim = (kdim1+kdim2) + (gdim1*gdim1*kdim1) +  (gdim2*gdim2*kdim2)
            cerr = 'hopd '
            allocate(hopd(zdim),stat=ierr)
            if (ierr > 0) go to 1000
!--allocating memmory for hopz which is not used here!!!
            cerr = 'hopz '
            allocate(hopz(1),stat=ierr)
            if (ierr > 0) go to 1000
            if (basis(f1).eq.9 .or. basis(f1).eq.21) then
               message = 'operators for combined KLegs (cjpm etc.) ' &
                  //'not implemented for PLeg.'
               call errormsg
            endif
            if (.not.((basis(f1).eq.7 .or. basis(f1).eq.20).and. &
                 (basis(f1+2).eq.7 .or. basis(f1+2).eq.20))) then
               message = 'operators for combined KLegs (cjpm etc.) ' &
                  //'require KLeg/K/KLeg/K combination.'
               call errormsg
            endif
            call genhtm2kleg(h,dvrmat,hopd)

         else if (htmoptyp .eq. 6) then ! Wigner operators
            f1=htmadof(h)
            kdim=ipbaspar(4,f1)
            mmdim=ipbaspar(8,f1)
            zdim=gdim(f1)**2*kdim*mmdim
            cerr = 'hopd '
            allocate(hopd(zdim),stat=ierr)
!--allocating memmory for hopz which is not used here!!!
            cerr = 'hopz '
            allocate(hopz(1),stat=ierr)
            if (ierr > 0) go to 1000
            if (htmform(h) .eq. 0) then ! Single mode Wigners
               call gen1dhtm3(h,f1,hopd,ort(zort(f1)), &
                    dvrmat(zdvr(f1),1),dvrmat(zdvr(f1),2), &
                    gdim(f1),kdim,mmdim)
            else
               message = 'Operators for Wigner functions must not be ' &
                       //'multiplied with other operators ' &
                       //'(feature not implemented)'
               call errormsg
            endif

         else 
            routine = "htmsetup"
            write(message,'(2(a,i5))') &
                  'Unknown htmoptyp: ',htmoptyp,' Operator: ',h
            call errormsg

         endif

!-----------------------------------------------------------------------
! check type of potential operators. (htmtype=0,1,2 for zero, unit, r-diag)
!-----------------------------------------------------------------------
         if (lz .and. htmtype(h).ne.14) then    ! complex side-diagonal
            call assdzhtm(h,hopz,hopd,sdm,lz)
            if (htmtype(h).eq.0) call outhtm(h,ilog) ! write removed operators
         elseif (htmtype(h) .eq. 2 .or. htmtype(h).eq.202) then ! real diagonal
            ctff=cutoff
!           for natpot-terms: use a different cutoff
            if (nnat.ne.0) ctff=npotcut(nnat)
            call assigndhtm(h,hopd,sdm,ctff) ! TODO
!    Print the removed operators to op.log file
!    add ".and. nnat.eq.0" if you don't want the removed natpots printed
            if(htmtype(h).eq.0) then
               if (irmvd .eq. 0) then
                 write(ilog,'(/a)') '#####  Removed operators (h-terms)  #####'
                 write(ilog,'(2x,8a4,a8,5x,a)') '   h ','   f','  af','  nf', &
                    '   m','  md',' Typ',' Sym',' Shift',' Term '
               endif
               irmvd=irmvd+1

               if(irmvd.lt.250) then
                  call outhtm(h,ilog)
               elseif(irmvd.eq.250) then
                  write(ilog,'(2a)') '----- Stopped printing removed', &
                  ' operators. (Too many lines) -----'
               endif
            endif
         endif

!-----------------------------------------------------------------------
! calculate symmetry of operators
!-----------------------------------------------------------------------
! htmtype: type of matrix, from folowing list
! 1D types  (0-14)
! 0 = zero
! 1 = unit
! 2 = real vector (diagonal matrix)
! 3 = imaginary vector (imaginary part stored as real)
! 4 = complex vector
! 5 = FFT, real vector
! 6 = FFT, imaginary vector
! 7 = FFT, complex vector
! 8 = real matrix
! 9 = imaginary matrix (imaginary part stored as real)
! 10 = complex matrix
! 11 = real vector + shifting in 2-nd variable (for npotk)
! 12 = real side-diagonal matrix
! 13 = imaginary side-diagonal matrix
! 14 = complex side-diagonal matrix
!
! 2D combined-mode types (15-23)
! 15 = real 3D tensor (for dth1_kleg, dth2_kleg)
! 16 = real 3D tensor (for jpm)
! 17 = real 3D tensor (for j^2kleg)
! 18 = real 3D tensor (for j_p)
! 19 = real 3D tensor (for j_m)
! 20 = real 3D tensor (for jp^2)
! 21 = real 3D tensor (for jm^2)
! 22 = real 3D tensor (for jpjm)
! 23 = real 3D tensor (for jmjp)
!
! 25 = readmap: mapping operator (SQR)
! 30 = cjpm
!
! 3D combined-mode types (40-59)
! 40 = real 4D tensor (for j^2wig)
! 41 = real 4D tensor (for j_pwig)
! 42 = real 4D tensor (for j_mwig)
! 43 = real 4D tensor (for jp2wig)
! 44 = real 4D tensor (for jm2wig)
! 45 = real 4D tensor (for jpmwig)
! 46 = real 4D tensor (for j_pswig)
! 47 = real 4D tensor (for j_mswig)
! 48 = real 4D tensor (for jp2swig)
! 49 = real 4D tensor (for jm2swig)
! 50 = real 4D tensor (for jpmswig)
! 51 = real 4D tensor (for jpjmwig)
! 52 = real 4D tensor (for jmjpwig)
! 53 = real 4D tensor (for jpjmswig)
! 54 = real 4D tensor (for jmjpswig)
! 55 = real 4D tensor (for jpjzwig)
! 56 = real 4D tensor (for jmjzwig)
! 57 = real 4D tensor (for jzjpwig)
! 58 = real 4D tensor (for jzjmwig)
! 59 = real 4D tensor (for jpjzswig)
! 60 = real 4D tensor (for jmjzswig)
! 61 = real 4D tensor (for jzjpswig)
! 62 = real 4D tensor (for jzjmswig)
! 63-69 (unassigned Wigner operators)
!
! htmsym=1 -> hermitian, htmsym=-1 -> anti-hermitian, htmsym=0 -> no symmetry
!

  100    continue
         if (htmtype(h) .eq. 1) then
            htmsym(h)=1
         else if (htmtype(h) .eq. 2) then
            htmsym(h)=1
         else if (htmtype(h) .eq. 3) then
            htmsym(h)=-1
         else if (htmtype(h) .eq. 4) then
            htmsym(h)=0
         else if (htmtype(h) .eq. 5) then
            htmsym(h)=1
         else if (htmtype(h) .eq. 6) then
            htmsym(h)=-1
         else if (htmtype(h) .eq. 7) then
            htmsym(h)=0
         else if (htmtype(h) .eq. 11) then
            htmsym(h)=0
         else if (htmtype(h) .eq. 12) then
            htmsym(h)=0
         else if (htmtype(h) .eq. 13) then
            htmsym(h)=0
         else if (htmtype(h) .eq. 14) then
            htmsym(h)=0
         else if (htmtype(h) .eq. 15) then
            if(htmfac(h) .eq. 1 ) then
               htmsym(h)=-1
            else
               htmsym(h)=0
            endif
         else if (htmtype(h) .eq. 16) then
            if(htmfac(h) .eq. 1 ) then
               htmsym(h)=1
            else
               htmsym(h)=0
            endif
         else if (htmtype(h) .eq. 17) then
            if(htmfac(h) .eq. 1 ) then
               htmsym(h)=1
            else
               htmsym(h)=0
            endif
         else if (htmtype(h) .ge. 18 .and. htmtype(h) .le. 24 ) then
            htmsym(h)=0
         else if (htmtype(h) .eq. 25) then
            htmsym(h)=0
         else if (htmtype(h) .eq. 30) then
            htmsym(h)=0 ! in theory it's Hermitian but we can't use that
         else if (htmtype(h) .eq. 31 .or. htmtype(h) .eq. 32) then
            htmsym(h)=1 ! reflex
         else if (htmtype(h) .eq. 40) then ! j^2wig
            if(htmfac(h) .eq. 1 ) then
               htmsym(h)=1
            else
               htmsym(h)=0
            endif
         else if ((htmtype(h) .ge. 41 .and. htmtype(h) .le. 44) .or. &
                  (htmtype(h) .ge. 46 .and. htmtype(h) .le. 49) .or. &
                  (htmtype(h) .ge. 51 .and. htmtype(h) .le. 69)) then
            htmsym(h)=0
         else if (htmtype(h).eq.45 .or. htmtype(h).eq.50) then ! jpmwig
            if(htmfac(h) .eq. 1 ) then
               htmsym(h)=1
            else
               htmsym(h)=0
            endif
         else if (htmtype(h) .eq. 101 .or. htmtype(h) .eq. 103) then
            htmsym(h)=1
         else if (htmtype(h) .eq. 102) then
            htmsym(h)=1
         else if (htmtype(h) .ge. 104 .and. htmtype(h) .le. 106) then
            htmsym(h)=1
         else if (htmtype(h) .eq. 108) then
            htmsym(h)=-1
         else if (htmtype(h) .eq. 151) then
            htmsym(h)=1
         else if (htmtype(h) .eq. 161) then
            htmsym(h)=1
         else if (htmtype(h) .eq. 202) then
            htmsym(h)=1
         else if (htmtype(h) .eq. 204) then
            htmsym(h)=0
         else if (htmtype(h) .gt. 0) then
            if  (htmcalc(h)) then
               call asshtmsym(hopz,htmsym(h),sdm,ldum)
               if(ldum) then       ! operator is found to be zero
                  htmtype(h)=0
                  nremoved=nremoved+1
                  irmvd=irmvd+1
                  if(irmvd.lt.250) then
                     call outhtm(h,ilog) ! Print removed operators to log
                  elseif(irmvd.eq.250) then
                     write(ilog,'(2a)') '----- Stopped printing', &
                      ' removed operators. (Too many lines) -----'
                  endif
               endif

            else
               htmsym(h)=0
            endif
         endif
         if (.not. htmcalc(h)) go to 200

         if (htmtype(h) .gt. 0 .and. htmtype(h) .lt. 100) &
              call wrscratch(itmpop,h,hopz,hopd,lz)

         write(cerr,'(a,i5)') 'htmsetup_...',h
         if(allocated(hopz))deallocate(hopz,stat=ierr)
         if(allocated(hopz1))deallocate(hopz1,stat=ierr)
         if(allocated(hopd))deallocate(hopd,stat=ierr)
         if (ierr > 0) go to 1001

  200    continue
      enddo

      if(irmvd.gt.0) write(ilog,'(a,i7)') &
                    ' --- Number of removed terms =', irmvd

      return

 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      stop

 1001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
      stop

      end subroutine htmsetup

!#######################################################################
!
! ortdim1, gdim1, and zort1 are to enable the use of grid points other
! than those in the standard ort array.
!#######################################################################
!
!                htmprodpot
!
! Diagonal 1D operators are multiplied to give a multi-dimensional (mode)
! diagonal operator. hops = real diagonal, hopdz = complex diagonal.
! lz is true, if there are complex terms.
!
! 10/2006 FO: allow sub-particle ops instead of just 1D ops.
!------------------------------------------------------------------------

      subroutine htmprodpot(h,hopdz,hops,ort,dvrmat,fftp,kinsph, &
                 rdop,subdim1,lz,maxd,done)

      implicit none

      integer(long)             :: f,nh,h1,f1,nh1,hl,vd,gd,nd
      integer(long), intent(in) :: h,subdim1,maxd
      integer(long), dimension(maxhop), intent(inout)          :: done

      real(dop), dimension(1)                                  :: rdum
      real(dop), dimension(ortdim), intent(in)                 :: ort
      real(dop), dimension(subdim1), intent(out)               :: hops
      real(dop), dimension(subdim1)                            :: hops1,workr
      real(dop), dimension(fftdim), intent(in)                 :: fftp 
      real(dop), dimension(sphdim), intent(in)                 :: kinsph
      real(dop), dimension(rdopdim), intent(in)                :: rdop
      real(dop), dimension(dvrdim,dvrord), intent(in)          :: dvrmat
      complex(dop), dimension(subdim1), intent(out)            :: hopdz
      complex(dop), dimension(maxd)                            :: hopz1
      complex(dop), dimension(subdim1)                         :: hopz2
      complex(dop), dimension(3*maxgdim2+subdim1)              :: workc
      logical(kind=4)                                          :: lfft
      logical(kind=4), intent(out)                             :: lz

      call zerovxd(hops,subdim1)
      call initvxd(workr,subdim1)
      call zerovxz(hopdz,subdim1)
      call initvxz(workc,subdim1)

!.....Determine if the product-operator is real or complex.
      lz = .false.
      if (htmform(h) .eq. 0) then
         do nh=1,htmfac(h)
            hl=hterm(nh,h)
            if((hopfile(hl).gt.100 .and. hopfile(hl).lt.200)&
                 .or.hopfile(hl).eq.209) lz=.true.
         enddo
      else
         do nh1=1,htmfac(h)
            h1=hterm(nh1,h)
            if (h1 .gt. 0) then
               do nh=1,htmfac(h1)
                  hl=hterm(nh,h1)
                  if(hl .gt. 0) then
                     if((hopfile(hl).gt.100 .and.  &
                        hopfile(hl).lt.200) .or.hopfile(hl).eq.209) lz=.true.
                   endif
               enddo
            endif
         enddo
      endif

      do nh=1,htmfac(h)
         h1=hterm(nh,h)
         if (h1 .eq. 0) then
            if(lz) then
               call addvxxzo(workc,hopdz,subdim1)
               call initvxz(workc,subdim1)
            else
               call addvxxdo(workr,hops,subdim1)
               call initvxd(workr,subdim1)
            endif
         else
            f=htmdof(h1)
            if (f.gt.0) then
               if (f.eq.fkoe .or. f .eq. ftime) then
                  f1=1
               else
                  f1=f
               endif
!              generate matrix for 1d operator (in hopz1/
!              real diagonal elements in hops1)
               call htmgen1d(h1,f,hopz1,ort(zort(f1)), &
                    dvrmat(zdvr(f1),1),dvrmat(zdvr(f1),2), &
                    fftp(zfft(f1)),kinsph(zsph(f1)), &
                    rdop,gdim(f),hops1,lfft,'htmprodpot  ',rdum,done)
               if(lz) then
!                 get diagonal elements (overwrite hopz1)
                  call rdzdiag(hopz1,hopz2,gdim(f))
                  call cpvxz(hopz2,hopz1,gdim(f))
!                 multiply
                  if (f.eq.fkoe .or. f.eq.ftime) then
                     call xvxxzzo(hopz1(1),workc,subdim1)
                  else
                     call dtxxzzo(hopz1,workc,vgdim(f),gdim(f),ngdim(f))
                  endif
               else
                  if (f.eq.fkoe .or. f.eq.ftime) then
                     call xvxxddo(hops1(1),workr,subdim1)
                  else
                     call dtxxddo(hops1,workr,vgdim(f),gdim(f),ngdim(f))
                  endif
               endif

            elseif (htmmode(h1).gt.0) then
               if (htmndof(h1).gt.0) then
!                 sub-particle operator: get diagonal elements (in
!                 hopz1)
                  call getvgn(h1,vd,gd,nd)
                  call htmgensubp(h1,hopz1,rdop,gd)
!                 mutiply
                  if (lz) then
                     call dtxxzzo(hopz1,workc,vd,gd,nd)
                  else
                     call cpvxzd(hopz1,hops1,gd)
                     call dtxxddo(hops1,workr,vd,gd,nd)
                  endif
               else
!                 multi-dimensional analytic potential 
                  call htmpot(h1,ort,workr,workc,subdim1,lz)
               endif

            else
               write(6,*) 'h=',h,',  h1=',h1,',  f=',f
               write(6,*) 'h: f,m,d',htmdof(h), htmmode(h), htmmuld(h)
               write(6,*) 'h1:f,m,d',htmdof(h1),htmmode(h1),htmmuld(h1)
               write(6,*) 'htmtype(h)',htmtype(h),'htmtype(h1)',htmtype(h1)
               routine='htmprodpot (htmsetup.F)'
               message='Neither htmdof nor htmmode is set!'
               call errormsg  
            endif

         endif
      enddo
      if(lz) then
         call addvxxzo(workc,hopdz,subdim1)
      else
         call addvxxdo(workr,hops,subdim1)
      endif

      
      if(lfft) then
         routine='htmprodpot  (htmsetup.F)'
         message='FFT cannot be a factor, FFT must not be called here!'
         call errormsg
      endif

      return
      end subroutine htmprodpot


!***********************************************************************
! 
! HTMPRODSD: multiplies 1D side-diagonal operators to create a
! multi-D side-diagonal operator.
!
! In:
!   h: Hamiltonian term
!   (+ lots of stuff which gets passed to htmgen1d)
! Out:
!   hops: array of side-diagonal elements (if real/imaginery)
!   hopz: array of side-diagonal elements (if complex)
!   lz  : if true, output is complex (use hopz, not hops)
! Workspace:
!   workc: 2*subdim1 for us, plus 3*maxgdim2 for htmgen1d
!          (we pretend this is workc(subdim1,3) and pass workc(1,3) to
!          htmgen1d, although the dimensions don't match...)
!   hopz1: for the result of htmgen1d
!
! 08/2006 FO
! 10/2006 FO: operators can also be sub-particle instead of just 1D
!------------------------------------------------------------------------

      subroutine htmprodsd(h,hopz,hops,ort,dvrmat,fftp,kinsph,rdop,subdim1,&
                           lz,done)

      implicit none

      integer(long)       :: f,nh,h1,g,vd,gd,nd,nskip,&
                             buf,dim1,k1,k2,k3,m,nf,nh1

      integer(long), intent(in)                       :: h,subdim1
      integer(long), dimension(maxhop), intent(inout) :: done
      real(dop), dimension(1)                         :: rdum 
      real(dop), dimension(ortdim), intent(in)        :: ort
      real(dop), dimension(fftdim), intent(in)        :: fftp
      real(dop), dimension(dvrdim,dvrord), intent(in) :: dvrmat
      real(dop), dimension(sphdim), intent(in)        :: kinsph 
      real(dop), dimension(subdim1), intent(out)      :: hops
      real(dop), dimension(subdim1)                   :: hops1
      real(dop), dimension(rdopdim), intent(in)       :: rdop
      complex(dop), dimension(subdim1*subdim1)        :: hopz1
      complex(dop), dimension(subdim1)                :: hopz2
      complex(dop), dimension(subdim1), intent(out)   :: hopz
      complex(dop), dimension(subdim1,3)              :: workc
      logical(kind=4), intent(out)                    :: lz
      logical(kind=4)                                 :: lim,lfft,lzero

      m=htmmode(h)
      if (m.eq.0) then
         routine='htmprodsd'
         message='FIXME: m=0. Why am I here?'
         call errormsg
      endif

!     zero result vector
      call zerovxz(hopz,subdim1)
!     start with first term
      nh1=1

!     init: start with a 1x1 unit matrix with shift k1=0
  5   buf =1
      dim1=1
      k1  =0
      workc(1,buf)=(1.0_dop,0.0_dop)

!     loop over all factors
      nf=1
      nskip=0
      do nh=nh1,htmfac(h)
         h1=hterm(nh,h)
         if (h1.eq.0) then
!           This signals the start of a new summand, i.e. the
!           multiplication is over for now. So let's get out of here!
            nh1=nh+1 ! start of next summand
            goto 20
         else
            f=htmadof(h1)
            if (f.eq.fkoe .or. f.eq.ftime) then
!              A coefficient, multiply the current result with it.
!              It's really easy to get the value of a simple
!              coefficient. See:
               call htmgen1d(h1,f,hopz1,ort(zort(1)), &
                    dvrmat(zdvr(1),1),dvrmat(zdvr(1),2), &
                    fftp(zfft(1)),kinsph(zsph(1)), &
                    rdop,gdim(f),hops1,lfft,'htmprodsd   ',rdum,done)
               call xvxxzzo(hopz1(1),workc(1,buf),dim1)
!              the rest can be skipped
               goto 12
            endif
!           1d operators must come in the order of the mode's dofs
            if (f.ne.spfdof(nf,m)) then
               write(6,*) 'Debug: h =',h,' h1 =',h1,' nf =',nf,' f =',f
               routine='htmprodsd'
               message='FIXME: 1d operators not in dof-order'
               call errormsg
            endif
            if (nskip.gt.0) then
!              check if we only skip unit operators
               if (h1.ne.f) then
                  routine='htmprodsd'
                  message='FIXME: skipping a non-unit operator!'
                  call warnmsg
               endif
               nskip=nskip-1
               goto 10
            endif
!           only (side-)diagonal operators can be multiplied here
            if (.not.((htmtype(h1).ge.1 .and. htmtype(h1).le.4) &
                .or. (htmtype(h1).ge.12 .and. htmtype(h1).le.14))) then
               routine='htmprodsd'
               message='FIXME: non-(side-)diagonal operator in product'
               write(6,*) 'h=',h,'  h1=',h1,'  h1typ=',htmtype(h1)
               call errormsg
            endif
            if (htmndof(h1).le.1) then
!              generate matrix for 1d operator (in hopz1)
               call htmgen1d(h1,f,hopz1,ort(zort(f)), &
                    dvrmat(zdvr(f),1),dvrmat(zdvr(f),2), &
                    fftp(zfft(f)),kinsph(zsph(f)), &
                    rdop,gdim(f),hops1,lfft,'htmprodsd   ',rdum,done)
!              get the side-diagonal elements (overwrite hopz1)
               k2=htmshift(h1)
               call rdzsdiag(hopz1,hopz2,k2,gdim(f))
               call cpvxz(hopz2,hopz1,gdim(f))
               gd=gdim(f)
            else
!              get the diagonal elements of the sub-particle operator
               call getvgn(h1,vd,gd,nd)
               call htmgensubp(h1,hopz1,rdop,gd)
               nskip=htmndof(h1)-1
               k2=0
            endif
!           multiply
            call ooxxzzt(workc(1,buf),k1,hopz1,k2,workc(1,3-buf),k3, &
                         dim1,gd)
!           update management variables
            buf =3-buf ! exchanges 1 and 2
            dim1=dim1*gd
            k1  =k3
 10         nf  =nf+1
 12         continue
         endif
      enddo
!     check consistency
 20   if (dim1.ne.subdim1 .or. nf-1.ne.nspfdof(m)) then
         routine='htmprodsd'
         message='FIXME: dimensions do not match'
         call errormsg
      endif
      if (k1 .ne. htmshift(h)) then
         routine='htmprodsd'
         message='FIXME: overall htmshift inconsistent!'
         call errormsg
      endif

!     add result
      do g=1,subdim1-abs(k1)
         hopz(g)=hopz(g)+workc(g,buf)
      enddo
!     more to do?
      if (nh.lt.htmfac(h)) goto 5

!     finally check for pure real/imaginary
      call asszdiag(hopz,hops,subdim1,lim,lz,lzero)
!     set type
      if (lzero) then
         htmtype(h)=0
         nremoved=nremoved+1
         call outhtm(h,ilog)
      elseif (lz) then
         htmtype(h)=14
      elseif (lim) then
         htmtype(h)=13
      else
         htmtype(h)=12
      endif

      return
      end subroutine htmprodsd! of htmprodsd


!#######################################################################
!                htmgen1d
!
! This subroutine generates all 1D operators.
! The operator can be a sum of products of factors.
! The output is the complex matrix hopz.
! The real, diagonal part of hopz is stored in hopd.
!------------------------------------------------------------------------


      subroutine htmgen1d(h,f,hopz,ort,dif1mat,dif2mat,fftp, &
                 kinsph,rdop,gdim1,hopd,lfft,scall,trafo,done)

      implicit none

      integer(long)                          :: nh,hl,zeig,zeig1,zeig2,zeig3, &
                                                g,gg,ncount,pottyp,htmfac1
      integer(long), intent(in)              :: h,f,gdim1
      integer(long), dimension(maxfac)       :: hterm1
      integer(long), dimension(maxhop), intent(inout)            :: done

      real(dop)                                                  :: v
      real(dop), dimension(1)                                    :: rdum
      real(dop), dimension(rdopdim), intent(in)                  :: rdop
      real(dop), dimension(gdim1), intent(in)                    :: ort,fftp,kinsph
      real(dop), dimension(gdim1), intent(out)                   :: hopd
      real(dop), dimension(gdim1,gdim1), intent(in)              :: dif1mat,dif2mat, &
                                                                    trafo
      complex(dop), dimension(gdim1,gdim1), intent(out)          :: hopz
      complex(dop), dimension(3*maxgdim2+2*maxgdim)              :: workc
      logical(kind=4), intent(out)                               :: lfft
      character(len=12), intent(in)                              :: scall 

      zeig=1
      zeig1=zeig+gdim1*gdim1
      zeig2=zeig1+gdim1*gdim1

      call zeromxz(hopz,gdim1,gdim1)
      call zerovxz(workc,3*maxgdim2)
      lfft =.false.
      ncount=1

!-----------------------------------------------------------------------
! If htmform(h)=1, entries in hterm point to other entries in hterm.
! These could be resolved into entries in hoplab, but that's not
! trivial. Anyway, it shouldn't happen at all, so for the moment we
! just abort with an error message.
!-----------------------------------------------------------------------
      if (htmform(h).eq.1) then
!        if you want to implement the resolution hterm->hoplab, put
!        the result into hterm1/htmfac1
         routine='htmgen1d'
         message='FIXME: cannot handle htmform=1'
         call errormsg
      else
!        simply copy the hterm entries
         htmfac1 = htmfac(h)
         do nh=1,htmfac1
            hterm1(nh)=hterm(nh,h)
         enddo
      endif

!-----------------------------------------------------------------------
! Load the first term (nh=1) into work(zeig).
! lfft = .true. if diagonal is to be taken from hmat
!-----------------------------------------------------------------------
      hl=hterm1(1)
      if (hopfile(hl).gt.0 .and. hopfile(hl).le.100) then
         if(hopfile(hl).eq.3 .and. hopilab(hl).le.3) lfft=.true.
         call callgrd(workc(zeig),hopilab(hl),hopfile(hl),&
              hopipar(1,hl),hoprpar(1,hl),dif1mat,dif2mat,&
              kinsph,fftp,ort,rdop,zrdop,f,workc(3*maxgdim2+1),&
              gdim(f),trafo,mass(f))
        if(hopfile(hl) .eq. 12) return  !readmap operator
      else if (hopfile(hl).gt.100 .and. hopfile(hl).le.200) then
         if(hopfile(hl).eq.101 .and. hopilab(hl).gt.3 .and. &
                                                f.eq.ftime) then
            workc(zeig)=(1.11_dop,1.11_dop) ! dummy for time-dependent operator
         else
            do g=1,gdim1
               gg = zeig + g*gdim1 + g - gdim1 -1
               call callanlz(workc(gg),ort(g),hopilab(hl),&
                    hopfile(hl),hopipar(1,hl),hoprpar(1,hl))
            enddo
         endif
      else if (hopfile(hl).gt.300 .and. hopfile(hl).le.400) then
         if(hopfile(hl).eq.302 .and. hopilab(hl).gt.2 .and.& 
                                               f.eq.ftime) then
            workc(zeig)=(1.11_dop,0.0_dop) ! dummy for time-dependent operator
         else
            do g=1,gdim1
               gg = zeig + g*gdim1 + g - gdim1 -1
               call callanld(v,ort(g),hopilab(hl),hopfile(hl),&
                    hopipar(1,hl),hoprpar(1,hl),rdum(1),pottyp,&
                    hoplab(hl),mass(f))
               workc(gg)=v
            enddo
         endif
      else if(hopfile(hl).eq.201 .and. hoplab(hl)(1:6).eq.'read1d') then
         call read1dinfo(h,hl,done)
         rewind(hopipar(1,hl))
         do g=1,gdim1
            gg = zeig + g*gdim1 + g - gdim1 -1
            call vpoint(v,ort,ndof,hopilab(hl),&
                        hopipar(1,hl),hoprpar(1,hl),hoplab(hl))
            workc(gg)=v
         enddo

      else if((hopfile(hl).eq.205 .or. hopfile(hl).eq.208 &
           .or. hopfile(hl).eq.209) .and. hoplab(hl)(1:5).eq.'qchem')then
         workc(zeig) = (1.0_dop,0.0_dop) ! Dummy for time-dependent operator.
      else
         routine='htmgen1d in htmsetup.F called from: '//scall
         write(message,'(a,56x,a,5i6,24x,a,i6,2x,a)') &
             'unallowed hopfile (nh=1)',&
             'f, h, hl, hopfile, htmform',f, h, hl, hopfile(hl), htmform(h), &
             'hopilab, hoplab',hopilab(hl),hoplab(hl)
         call errormsg
      endif

!-----------------------------------------------------------------------
! multiply out terms and sum terms (when hl=0).
!-----------------------------------------------------------------------
      do nh=2,htmfac1
         hl=hterm1(nh)
!........hl=0 separates two products which are to be summed.
         if (hl .eq. 0) then
            call addmxxzo(workc(zeig),hopz,gdim1,gdim1)
            call unitqxz(workc(zeig),gdim1)
            ncount=ncount+2
         else
            call zerovxz(workc(zeig1),gdim1*gdim1)
            if (hopfile(hl).gt.0 .and. hopfile(hl).le.100) then
               if(hopfile(hl).eq.3 .and. hopilab(hl).le.3) lfft=.true.
               call callgrd(workc(zeig1),hopilab(hl),hopfile(hl), &
                    hopipar(1,hl),hoprpar(1,hl),dif1mat,dif2mat, &
                    kinsph,fftp,ort,rdop,zrdop,f,workc(3*maxgdim2+1), &
                    gdim(f),trafo,mass(f))
               call qqxxzz(workc(zeig),workc(zeig1),workc(zeig2), &
                    gdim1)

            else if (hopfile(hl).gt.100 .and. hopfile(hl).le.200) then
               if(hopfile(hl).eq.101 .and. hopilab(hl).gt.3 .and.  &
                                                      f.eq.ftime) then 
                  workc(zeig1)=(1.11_dop,1.11_dop) ! dummy for time-dep. op.
               else
                  if(hopfile(hl).eq.101 .and. hopilab(hl).le.3)&
                        ncount=ncount+1
                  do g=1,gdim1
                     call callanlz(workc(zeig1+g-1),ort(g),hopilab(hl), &
                          hopfile(hl),hopipar(1,hl),hoprpar(1,hl))
                  enddo
               endif
               call qdxxzz(workc(zeig),workc(zeig1),workc(zeig2), &
                    gdim1)

            else if (hopfile(hl).gt.300 .and. hopfile(hl).le.400) then
               if(hopfile(hl).eq.302 .and. hopilab(hl).gt.2 .and.  &
                                                      f.eq.ftime) then 
                  workc(zeig1)=(1.11_dop,0.0_dop) ! dummy for time-dep. op.
               else 
                  if(hopfile(hl).eq.301 .or. (hopfile(hl).eq.302 .and. &
                       hopilab(hl).le.2) ) ncount=ncount+1
                  do g=1,gdim1
                     call callanld(v,ort(g),hopilab(hl),hopfile(hl), &
                          hopipar(1,hl),hoprpar(1,hl),rdum(1),pottyp, &
                          hoplab(hl),mass(f))
                     workc(zeig1+g-1)=v
                  enddo
               endif
               call qdxxzz(workc(zeig),workc(zeig1),workc(zeig2), &
                    gdim1)

            else if(hopfile(hl).eq.201 .and.  &
                     hoplab(hl)(1:6).eq.'read1d') then
               call read1dinfo(h,hl,done)
               rewind(hopipar(1,hl))
               do g=1,gdim1
                  call vpoint(v,ort,ndof,hopilab(hl), &
                              hopipar(1,hl),hoprpar(1,hl),hoplab(hl))
                  workc(zeig1+g-1)=v
               enddo
               call qdxxzz(workc(zeig),workc(zeig1),workc(zeig2), &
                    gdim1)

            else if((hopfile(hl).eq.205 .or. hopfile(hl).eq.208 &
                 .or. hopfile(hl).eq.209) .and. hoplab(hl)(1:5).eq.'qchem')then
               workc(zeig1) = (1.0_dop,0.0_dop) ! dummy for time-dependent op

            else
               write(6,'(a,5i6)') 'f, h, nh, hl, hopfile', &
                    f, h, nh, hl, hopfile(hl)
               message='unallowed hopfile'
               routine='htmgen1d in htmsetup.F called from: '//scall
               call errormsg
            endif

            zeig3=zeig
            zeig=zeig2
            zeig2=zeig3
         endif
      enddo

      if(lfft .and. htmfac1.gt.ncount) then
         write(6,*) 'FFT ERROR; h,htmfac,ncount:',h,htmfac1,ncount
         write(6,'(a,14i5)') 'hterm: ',(hterm1(nh),nh=1,htmfac1)
         write(6,'(7(a10,1x))') (hoplab(hterm1(nh)),nh=1,htmfac1)
         write(6,'(a,14i5)') 'hopfile:', &
              (hopfile(hterm1(nh)),nh=1,htmfac1)
         write(2,*) 'FFT ERROR; h,htmfac,ncount:',h,htmfac1,ncount
         write(2,*) 'hterm: ',(hterm1(nh),nh=1,htmfac1)
         write(2,'(7(a10,1x))') (hoplab(hterm1(nh)),nh=1,htmfac1)
         write(2,'(a,14i5)') 'hopfile:', &
              (hopfile(hterm1(nh)),nh=1,htmfac1)
         routine='htmgen1d (htmsetup.F) called from: '//scall
         message='FFT cannot do several factors, e.g. cos*dq'
         call errormsg
      endif

      call addmxxzo(workc(zeig),hopz,gdim1,gdim1)

!-----------------------------------------------------------------------
! store diagonal in real array (used if required)
!-----------------------------------------------------------------------
      call rddiag(hopz,hopd,gdim1)

      return
      end subroutine htmgen1d

!#######################################################################
!...This routine is for combined natural potentials only.
!...FG 05/02

      subroutine htmgen1m(h,rdop,sdm,hopd)

      implicit none

      integer(long)                             :: hl,i,j
      integer(long), intent(in)                 :: h,sdm
      real(dop)                                 :: fac
      real(dop), dimension(rdopdim), intent(in) :: rdop
      real(dop), dimension(sdm), intent(out)    :: hopd


!-----------------------------------------------------------------------
! store combined natpot/cpd potential into  hopd.
!-----------------------------------------------------------------------
      fac=1.0_dop
      if(htmfac(h).ne.1) then
         do j=2,htmfac(h)
            hl=hterm(j,h)
            if(hopfile(hl).eq.301) then
!...........Multiply natpot/cpd with constant
               fac=fac*hoprpar(1,hl)
            elseif(hopfile(hl).eq.302 .and. hopilab(hl).eq.1) then
               continue   ! 1 operator
            elseif(hopfile(hl).eq.302 .and. hopilab(hl).eq.2) then
               fac=-fac   ! -1 operator
            else
               write(6,'(/a,i5,a,2i5)') 'ERROR. hopfile =', &
                      hopfile(hl), '    j, hl',j, hl
               write(6,'(a,i5,2x,a8)')'hopilab, hoplab ', &
                      hopilab(hl),hoplab(hl)
               write(6,'(a,2i5,/a,9i5)') 'h, htmfac(h)',h,htmfac(h), &
                                'hl:',  (hterm(i,h),i=1,htmfac(h))
               write(6,'(9(a,2x))') 'hoplab:', &
                 (trim(hoplab(max(1,hterm(i,h)))),i=1,htmfac(h))

               message='htmfac(h).ne.1 for combined nat-pot/cpd.'
               routine='htmgen1m in htmsetup.F'
               call errormsg
            endif
         enddo
      endif

      hl=hterm(1,h)
      do i=1,sdm
         hopd(i)=rdop(zrdop(hopilab(hl))+i-1)*fac
      enddo

      return
      end subroutine htmgen1m

!=======================================================================

      subroutine htmgenmlop(h,sdm,pot)

!-----------------------------------------------------------------------
! This is for "simple" (htmform=0) operator terms containing MLPF
! leaf potentials (hoplab="mlnp...").
! FO 06/2013
!-----------------------------------------------------------------------
      implicit none

      integer(long), intent(in)              :: h,sdm
      real(dop), dimension(sdm), intent(out) ::  pot
      integer(long)                          ::  hl

      if (htmfac(h).ne.1) then
         routine = "htmgenmlop"
         message = "FIXME: htmfac>1 not implemented"
         call errormsg
      endif
      hl = hterm(1,h)
      call callmlop(pot,sdm,hopilab(hl),hopipar(1,hl),hopipar(2,hl))

      end subroutine htmgenmlop

!#######################################################################
!...This routine is for matrix read operators for combined modes only.
!...HDM 06/19

      subroutine htmgenreadm(h,rdop,sdm,hopz)

      implicit none

      integer(long) :: hl,sdm,i,j,hl0
      integer(long), intent(in)                     :: h
      real(dop)                                     ::  fac
      real(dop), dimension(rdopdim), intent(in)     :: rdop
      complex(dop), dimension(sdm*sdm), intent(out) :: hopz(sdm*sdm)

!-----------------------------------------------------------------------
! store combined read operator (real matrix) into  hopz.
!-----------------------------------------------------------------------
      fac=1.d0
      hl0=0
      call zerovxz(hopz,sdm*sdm)
      do j=1,htmfac(h)
         hl=hterm(j,h)
         if(hl.eq.0) then
            if(hl0.eq.0) then
               write(6,'(3(a,i5))')' h =',h,',  hl =',hl,',  hl0 =',hl0
               message='There is no operator (matrix) to sum.'
               routine='htmgenreadm in htmsetup.F'
               call errormsg
            endif
            do i=1,sdm*sdm
               hopz(i) = hopz(i) + rdop(zrdop(hopilab(hl0))+i-1)*fac
            enddo
            hl0=0
            fac=1.d0
         elseif(hoplab(hl)(1:4).eq.'read') then
            if(hl0.ne.0) then
               write(6,'(3(a,i5))')' h =',h,',  hl =',hl,',  hl0 =',hl0
               message='Cannot do products etc of read matrices.'
               routine='htmgenreadm in htmsetup.F'
               call errormsg
            endif
            hl0=hl
         elseif(hopfile(hl).eq.301) then
!........Multiply read-operator with constant
            fac=fac*hoprpar(1,hl)
         elseif(hopfile(hl).eq.302 .and. hopilab(hl).eq.1) then
            continue   ! 1 operator
         elseif(hopfile(hl).eq.302 .and. hopilab(hl).eq.2) then
            fac=-fac   ! -1 operator
         else
            write(6,'(/a,i5,a,2i5)') 'ERROR. hopfile =', &
                   hopfile(hl), '    j, hl',j, hl
            write(6,'(a,i5,2x,a8)')'hopilab, hoplab ', &
                   hopilab(hl),trim(hoplab(hl))
            write(6,'(a,2i5,/a,12i5)') 'h, htmfac(h)',h,htmfac(h), &
                             'hl:',  (hterm(i,h),i=1,htmfac(h))
            write(6,'(9(a,2x))') 'hoplab:', &
                (trim(hoplab(max(1,hterm(i,h)))),i=1,htmfac(h))  
            message='htmfac(h).ne.1 for combined read.'
            routine='htmgenreadm in htmsetup.F'
            call errormsg
         endif
      enddo
      if(hl0.eq.0) then
         write(6,'(3(a,i5))')' h =',h,',  hl =',hl,',  hl0 =',hl0
         message='There is no operator (matrix) to sum.'
         routine='htmgenreadm in htmsetup.F (line 1141)'
         call errormsg
      endif

      do i=1,sdm*sdm
         hopz(i) = hopz(i) + rdop(zrdop(hopilab(hl0))+i-1)*fac
      enddo

      return
      end subroutine htmgenreadm

!#######################################################################

!-----------------------------------------------------------------------
!                     Subroutine gen1dhtm2
!
! Calculates operators containing  j^2kleg, j_p, j_m, jpm, jp^2, jm^2,
! (see checkjm)                    dth1_kleg, dth2_kleg.
!-----------------------------------------------------------------------

      subroutine gen1dhtm2(h,f,hops2,ort,dif1mat,dif2mat,gdim1,kdim)

      implicit none

      integer(long)       :: nh,hl,zeig,zeig1,zeig2,zeig3,g,k,&
                             kmin,gdim2,zdim,pottyp

      integer(long), intent(in)                             :: h,f,gdim1,kdim
      real(dop)                                             :: v
      real(dop), dimension(1)                               :: rdum
      real(dop), dimension(gdim1), intent(in)               :: ort
      real(dop), dimension(gdim1,gdim1,kdim), intent(out)   :: hops2
      real(dop), dimension(gdim1,gdim1,kdim), intent(in)    :: dif2mat
      real(dop), dimension(gdim1,gdim1,2*kdim), intent(in)  :: dif1mat
      real(dop), dimension(3*gdim1*gdim1*kdim)              :: workr

      gdim2=gdim1*gdim1
      zdim=gdim1*gdim1*kdim
      zeig=1
      zeig1=zeig+zdim
      zeig2=zeig1+zdim
      if (basis(f).eq.7 .or. basis(f).eq.20) then
         kmin=ipbaspar(1,f+1)
      elseif (basis(f).eq.9 .or. basis(f).eq.21) then
         kmin=ipbaspar(1,f+1)
      endif

      call zerovxd(hops2,zdim)
      do k=1,kdim
         call unitqxd(workr((k-1)*gdim2+1),gdim1)
      enddo

      do nh=1,htmfac(h)
         hl=hterm(nh,h)
         if (hl .eq. 0) then
            call addvxxdo(workr(zeig),hops2,zdim)
            do k=1,kdim
               call unitqxd(workr((k-1)*gdim2+zeig),gdim1)
            enddo
         else
            if (hopfile(hl).gt.300 .and. hopfile(hl).le.400) then
               do g=1,gdim1
                  call callanld(v,ort(g),hopilab(hl),hopfile(hl), &
                      hopipar(1,hl),hoprpar(1,hl),rdum(1),pottyp, &
                      hoplab(hl),mass(f))
                  workr(zeig1+g-1)=v
               enddo
               do k=1,kdim
                  call qdxxdd(workr(zeig+(k-1)*gdim2),workr(zeig1), &
                              workr(zeig2+(k-1)*gdim2),gdim1)
               enddo
            else if (hopfile(hl) .eq. 5) then
               call zerovxd(workr(zeig1),zdim)

               call funckleg(workr(zeig1),gdim1,hopilab(hl), &
                    dif1mat,dif2mat,kdim,jtot,kmin)

               do k=1,kdim
                  call qqxxdd(workr(zeig+(k-1)*gdim2), &
                       workr(zeig1+(k-1)*gdim2), &
                       workr(zeig2+(k-1)*gdim2),gdim1)
               enddo
            else
               routine='Gen1dhtm2'
               message= &
                 'Operator type not allowed for Kleg mode: '//hoplab(hl)
               call errormsg
            endif

            zeig3=zeig
            zeig= zeig2
            zeig2=zeig3
         endif

       enddo
       call addvxxdo(workr(zeig),hops2,zdim)

       return
       end subroutine gen1dhtm2

!-----------------------------------------------------------------------
!                     Subroutine gen1dhtm2rw
!
! Real wave packet propagation
! Calculates operators containing  j^2kleg, j_p, j_m, jpm, jp^2, jm^2,
! (see checkjm)                    dth1_kleg, dth2_kleg.
!-----------------------------------------------------------------------

      subroutine gen1dhtm2rw(h,f,hops2,ort,dif1mat,dif2mat,gdim1,kdim,subdim1,ip)

      implicit none

      integer(long)       :: nh,hl,zeig,zeig1,zeig2,zeig3,g,k,&
                             kmin,gdim2,zdim,pottyp

      integer(long), intent(in)                             :: h,f,gdim1,kdim,subdim1,ip
      real(dop)                                             :: v
      real(dop), dimension(1)                               :: rdum
      real(dop), dimension(gdim1), intent(in)               :: ort
      real(dop), dimension(subdim1), intent(out)            :: hops2
      real(dop), dimension(gdim1,gdim1,kdim)                :: hops3
      real(dop), dimension(gdim1,gdim1,kdim), intent(in)    :: dif2mat
      real(dop), dimension(gdim1,gdim1,2*kdim), intent(in)  :: dif1mat
      real(dop), dimension(3*gdim1*gdim1*kdim)              :: workr
      real(dop), dimension(gdim1*kdim)                      :: workr1
      real(dop), dimension(subdim1)                         :: workr2

      call zerovxd(workr1,gdim1*kdim)
      call initvxd(workr2,subdim1)

      gdim2=gdim1*gdim1
      zdim=gdim1*gdim1*kdim
      zeig=1
      zeig1=zeig+zdim
      zeig2=zeig1+zdim
      if (basis(f).eq.7 .or. basis(f).eq.20) then
         kmin=ipbaspar(1,f+1)
      elseif (basis(f).eq.9 .or. basis(f).eq.21) then
         kmin=ipbaspar(1,f+1)
      endif

      call zerovxd(hops2,subdim1)
      call zerovxd(hops3,zdim)
      do k=1,kdim
         call unitqxd(workr((k-1)*gdim2+1),gdim1)
      enddo

      do nh=1,htmfac(h)
         hl=hterm(nh,h)
         if (hl .eq. 0) then
            call addvxxdo(workr(zeig),hops3,zdim)
            do k=1,kdim
               call unitqxd(workr((k-1)*gdim2+zeig),gdim1)
            enddo
         else
            if (hopfile(hl).gt.300 .and. hopfile(hl).le.400) then
               do g=1,gdim1
                  call callanld(v,ort(g),hopilab(hl),hopfile(hl), &
                      hopipar(1,hl),hoprpar(1,hl),rdum(1),pottyp, &
                      hoplab(hl),mass(f))
                  workr(zeig1+g-1)=v
               enddo
               do k=1,kdim
                  call qdxxdd(workr(zeig+(k-1)*gdim2),workr(zeig1), &
                              workr(zeig2+(k-1)*gdim2),gdim1)
               enddo
            else if (hopfile(hl) .eq. 5) then
               call zerovxd(workr(zeig1),zdim)

               call funckleg(workr(zeig1),gdim1,hopilab(hl), &
                    dif1mat,dif2mat,kdim,jtot,kmin)

               do k=1,kdim
                  call qqxxdd(workr(zeig+(k-1)*gdim2), &
                       workr(zeig1+(k-1)*gdim2), &
                       workr(zeig2+(k-1)*gdim2),gdim1)
               enddo
            else
               routine='Gen1dhtm2rw'
               message= &
                 'Operator type not allowed for Kleg mode: '//hoplab(hl)
               call errormsg
            endif

            zeig3=zeig
            zeig= zeig2
            zeig2=zeig3
         endif

       enddo
       call addvxxdo(workr(zeig),hops3,zdim)

       if (ip == 1) then
             call diagten(hops3,workr1,gdim1,kdim)
             call dtxxddo(workr1,workr2,vgdim(f),gdim1*kdim,ngdim(f+1))
             call cpvxd(workr2,hops2,subdim1) ! subdim1=totphidim
       else if (ip == 2) then
             call cpvxd(hops3,hops2,subdim1) !here subdim1=zdim
       endif


       return
       end subroutine gen1dhtm2rw

!-----------------------------------------------------------------------
!                Subroutine genmodehtm2
!
! Calculates operators containing  j^2kleg, j_p, j_m, jpm, jp^2, jm^2,
! terms for combined modes.        dth1_kleg, dth2_kleg.
!-----------------------------------------------------------------------

      subroutine genmodehtm2(h,hops2,ort,dvrmat,fftp,kinsph,rdop,gdim1,kdim,&
                             done)

      implicit none

      integer(long)       :: f,f1,nh,h1,hl,zdim,gdim2,k,kmin,&
                             zeig,zeig1,zeig2,zeig3
      integer(long), dimension(maxhop), intent(inout)       :: done

      integer(long), intent(in)                             :: h,gdim1,kdim
      real(dop), dimension(1)                               :: rdum
      real(dop), dimension(gdim1), intent(in)               :: ort
      real(dop), dimension(kdim)                            :: ortk
      real(dop), dimension(fftdim), intent(in)              :: fftp
      real(dop), dimension(sphdim), intent(in)              :: kinsph
      real(dop), dimension(rdopdim), intent(in)             :: rdop
      real(dop), dimension(dvrdim,dvrord), intent(in)       :: dvrmat
      real(dop), dimension(gdim1*gdim1*kdim), intent(out)   :: hops2
      real(dop), dimension(3*gdim1*gdim1*kdim)              :: workr
      complex(dop), dimension(maxgdim2)                     :: hopz
      logical(kind=4)                                       :: ljm,lfft


      gdim2=gdim1*gdim1
      zdim=gdim1*gdim1*kdim
      zeig =1
      zeig1=zeig+zdim
      zeig2=zeig1+zdim

      call zerovxd(hops2,zdim)
      do k=1,kdim
         call unitqxd(workr((k-1)*gdim2+1),gdim1)
      enddo

      do nh=1,htmfac(h)
         h1=hterm(nh,h)
         if (h1 .eq. 0) then
            call addvxxdo(workr(zeig),hops2,zdim)
            do k=1,kdim
               call unitqxd(workr((k-1)*gdim2+zeig),gdim1)
            enddo
         else
            f=htmadof(h1) ! NEWKLEG
            if (f.eq.fkoe .or. f .eq. ftime) then
                f1=1
            else
                f1=f
            endif

            call checkjm(h1,ljm)
            kmin=0
            if (ljm) then
               call gen1dhtm2(h1,f1,workr(zeig1),ort, &
                    dvrmat(zdvr(f1),1),dvrmat(zdvr(f1),2),gdim1,kdim)
               do k=1,kdim
                  call qqxxdd(workr(zeig+(k-1)*gdim2), &
                       workr(zeig1+(k-1)*gdim2), &
                       workr(zeig2+(k-1)*gdim2),gdim1)
               enddo
            else
               hl=hterm(1,h1)
               if ((hopfile(hl).gt.300 .and. hopfile(hl).le.400) .or. &
                    hopfile(hl).eq.1 .or.  hopfile(hl).eq.9) then
                  if (basis(f1).eq.8) then
                     if (basis(f1-1).eq.7 .or. basis(f1-1).eq.20) then 
                        kmin=ipbaspar(1,f1)
                     elseif(basis(f1-1).eq.9.or.basis(f1-1).eq.21) then
                        kmin=ipbaspar(1,f1)
                     endif
                     do k=1,kdim
                        ortk(k)=k-1+kmin
                     enddo
                     call htmgen1d(h1,f,hopz,ortk, &
                          dvrmat(zdvr(f1),1),dvrmat(zdvr(f1),2), &
                          fftp(zfft(f1)),kinsph(zsph(f1)), &
                          rdop,kdim,workr(zeig1),lfft,'genmodehtm2 ',rdum,done)
                     do k=1,kdim
                        call xqxxdd(workr(zeig1+k-1), &
                                    workr(zeig+(k-1)*gdim2), &
                                    workr(zeig2+(k-1)*gdim2),gdim1)
                     enddo
                  else
                     
                  endif
               else
                  routine='Genmodehtm2'
                  message='Operator type not allowed for Kleg mode: '// &
                          hoplab(hl)
                  call errormsg
               endif
            endif

            zeig3=zeig
            zeig =zeig2
            zeig2=zeig3
          endif
      enddo
      call addvxxdo(workr(zeig),hops2,zdim)

      return
      end subroutine genmodehtm2

!-----------------------------------------------------------------------
!                Subroutine genmodehtm2rw
!
! Real wave packet propagation
! Calculates operators containing  j^2kleg, j_p, j_m, jpm, jp^2, jm^2,
! terms for combined modes.        dth1_kleg, dth2_kleg.
!-----------------------------------------------------------------------

      subroutine genmodehtm2rw(h,hops,ort,dvrmat,fftp,kinsph,rdop,gdim1,&
                 kdim,subdim1,done)

      implicit none

      integer(long)       :: f,f1,nh,h1,hl,zdim,gdim2,k,kmin,&
                             zeig,zeig1,zeig2,zeig3

      integer(long), intent(in)                             :: h,gdim1,kdim,&
                                                               subdim1
      integer(long), dimension(maxhop), intent(inout)       :: done
      real(dop), dimension(1)                               :: rdum
      real(dop), dimension(ortdim), intent(in)              :: ort
      real(dop), dimension(kdim)                            :: ortk
      real(dop), dimension(fftdim), intent(in)              :: fftp
      real(dop), dimension(sphdim), intent(in)              :: kinsph
      real(dop), dimension(rdopdim), intent(in)             :: rdop
      real(dop), dimension(dvrdim,dvrord), intent(in)       :: dvrmat
      real(dop), dimension(subdim1), intent(out)            :: hops
      real(dop), dimension(3*gdim1*gdim1*kdim)              :: workr
      complex(dop), dimension(maxgdim2)                     :: hopz
      logical(kind=4)                                       :: ljm,lfft
      real(dop), dimension(gdim1*kdim)                      :: workr1
      real(dop), dimension(subdim1)                         :: workr2,hops1

      call zerovxd(hops,subdim1)
      call zerovxd(workr1,gdim1*kdim)
      call initvxd(workr2,subdim1)


      gdim2=gdim1*gdim1
      zdim=gdim1*gdim1*kdim
      zeig =1
      zeig1=zeig+zdim
      zeig2=zeig1+zdim

      do k=1,kdim
         call unitqxd(workr((k-1)*gdim2+1),gdim1)
      enddo

      do nh=1,htmfac(h)
         h1=hterm(nh,h)
         if (h1 .eq. 0) then
            call addvxxdo(workr2,hops,subdim1)
            call initvxd(workr2,subdim1)
            do k=1,kdim
               call unitqxd(workr((k-1)*gdim2+zeig),gdim1)
            enddo
         else
            f=htmadof(h1) ! NEWKLEG
            if (f.eq.fkoe .or. f .eq. ftime) then
                f1=1
            else
                f1=f
            endif

            call checkjm(h1,ljm)
            kmin=0
            if (ljm) then
               call gen1dhtm2rw(h1,f1,workr(zeig1),ort(zort(f1)), &
                    dvrmat(zdvr(f1),1),dvrmat(zdvr(f1),2),gdim1,kdim, &
                    gdim2*kdim,2)
               do k=1,kdim
                  call qqxxdd(workr(zeig+(k-1)*gdim2), &
                       workr(zeig1+(k-1)*gdim2), &
                       workr(zeig2+(k-1)*gdim2),gdim1)
               enddo
               call diagten(workr(zeig2),workr1,gdim1,kdim)
               call dtxxddo(workr1,workr2,vgdim(f1),gdim1*kdim,ngdim(f1+1))
               zeig3=zeig
               zeig =zeig2
               zeig2=zeig3
            else
               hl=hterm(1,h1)
               if ((hopfile(hl).gt.300 .and. hopfile(hl).le.400) .or. &
                    hopfile(hl).eq.1) then
                  if (basis(f1).eq.8) then
                     if (basis(f1-1).eq.7 .or. basis(f1-1).eq.20) then
                        kmin=ipbaspar(1,f1)
                     elseif(basis(f1-1).eq.9.or.basis(f1-1).eq.21) then
                        kmin=ipbaspar(1,f1)
                     endif
                     do k=1,kdim
                        ortk(k)=k-1+kmin
                     enddo
                     call htmgen1d(h1,f,hopz,ortk, &
                          dvrmat(zdvr(f1),1),dvrmat(zdvr(f1),2), &
                          fftp(zfft(f1)),kinsph(zsph(f1)), &
                          rdop,kdim,hops1,lfft,'genmodehtm2r',rdum,done)
                        if (f.eq.fkoe .or. f.eq.ftime) then
                           call xvxxddo(hops1(1),workr2,subdim1)
                        else
                           call dtxxddo(hops1,workr2,vgdim(f),gdim(f),ngdim(f))
                        endif
                  else 
                     call htmgen1d(h1,f,hopz,ort(zort(f1)),dvrmat(zdvr(f1),1), &
                          dvrmat(zdvr(f1),2),fftp(zfft(f1)),kinsph(zsph(f1)), &
                          rdop,gdim(f),hops1,lfft,'genmodehtm2r',rdum,done)
                        if (f.eq.fkoe .or. f.eq.ftime) then
                           call xvxxddo(hops1(1),workr2,subdim1)
                        else
                           call dtxxddo(hops1,workr2,vgdim(f),gdim(f),ngdim(f))
                        endif
                  endif
               else
                  routine='Genmodehtm2'
                  message='Operator type not allowed for Kleg mode: '// &
                          hoplab(hl)
                  call errormsg
               endif
            endif
         endif
      enddo
      call addvxxdo(workr2,hops,subdim1)


      return
      end subroutine genmodehtm2rw

!-----------------------------------------------------------------------
!                     Subroutine gen1dhtm3
!
! Calculates Wigner operators containing j^2wig, j_pwig, j_mwig, jpmwig,
! jp2wig, jm2wig, j_pswig, j_mswig, jpmswig, jp2swig, jm2swig, jpjmwig,
! jmjpwig, jpjmswig, jmjpswig, jpjzwig, jmjzwig, jzjpwig, jzjmwig,
! jpjzswig, jmjzswig, jzjpswig, jzjmswig
!-----------------------------------------------------------------------

      subroutine gen1dhtm3(h,f,hops2,ort,dif1mat,dif2mat,gdim1,kdim,mmdim)

      implicit none

      integer(long), intent(in) ::  h,f,kdim,mmdim,gdim1
      integer(long) :: nh,hl,zeig,zeig1,zeig2,zeig3,g,k,mm,pottyp
      integer(long) :: kmin,gdim2,zdim,mmmin
      real(dop), dimension(gdim1,gdim1,kdim,mmdim), intent(out) :: hops2
      real(dop), dimension(gdim1), intent(in)                   :: ort
      real(dop), dimension(gdim1,gdim1,kdim,mmdim), intent(in)  :: dif1mat, &
                                                                   dif2mat
      real(dop), dimension(3*gdim1*gdim1*kdim*mmdim)            :: workr
      real(dop)                                                 :: v

      gdim2=gdim1*gdim1
      zdim=gdim1*gdim1*kdim*mmdim
      zeig=1
      zeig1=zeig+zdim
      zeig2=zeig1+zdim
      kmin=ipbaspar(2,f)
      mmmin=ipbaspar(6,f)

      call zerovxd(hops2,zdim)
      do mm=1,mmdim
        do k=1,kdim
          call unitqxd(workr(zeig+gdim2*(k-1+kdim*(mm-1))),gdim1)
        enddo
      enddo

      do nh=1,htmfac(h)
         hl=hterm(nh,h)
         if (hl .eq. 0) then
            call addvxxdo(workr(zeig),hops2,zdim)
            do mm=1,mmdim
              do k=1,kdim
                call unitqxd(workr(zeig+gdim2*(k-1+kdim*(mm-1))),gdim1)
              enddo
            enddo
         else
            if (hopfile(hl).gt.300 .and. hopfile(hl).le.400) then
               pottyp=0
               do g=1,gdim1
                  call callanld(v,ort(g),hopilab(hl),hopfile(hl),hopipar(1,hl),&
                       hoprpar(1,hl),0.0_dop,pottyp,hoplab(hl),mass(f))
                  workr(zeig1+g-1)=v
               enddo
               do mm=1,mmdim
                 do k=1,kdim
                   call qdxxdd(workr(zeig+gdim2*(k-1+kdim*(mm-1))), &
                           workr(zeig1), &
                           workr(zeig2+gdim2*(k-1+kdim*(mm-1))),gdim1)
                 enddo
               enddo
            else if (hopfile(hl) .eq. 8) then
               call zerovxd(workr(zeig1),zdim)

               call funcwig(workr(zeig1),gdim1,hopilab(hl), &
                    dif1mat,dif2mat,kdim,jtot,kmin,mmdim,mmmin)

               do mm=1,mmdim
                 do k=1,kdim
                   call qqxxdd(workr(zeig+gdim2*(k-1+kdim*(mm-1))), &
                       workr(zeig1+gdim2*(k-1+kdim*(mm-1))), &
                       workr(zeig2+gdim2*(k-1+kdim*(mm-1))),gdim1)
                 enddo
               enddo

            else
               routine='Gen1dhtm3'
               message= &
               'Operator type not allowed for Wigner mode: '//hoplab(hl)
               call errormsg
            endif

            zeig3=zeig
            zeig= zeig2
            zeig2=zeig3
         endif

       enddo
       call addvxxdo(workr(zeig),hops2,zdim)

       return
       end subroutine gen1dhtm3

!----------------------------------------------------------------------
! Subroutine checkmodejm
!
! Checks if combined-mode operator contains 
! j^2kleg, j_p, j_m, jpm, jp^2, jm^2 terms.
!---------------------------------------------------------------------
      subroutine checkmodejm(h,ljmmode)

      implicit none

      integer(long)                :: n,h1
      integer(long), intent(in)    :: h
      logical(kind=4)              :: ljm
      logical(kind=4), intent(out) :: ljmmode

      ljmmode=.false.
      do n=1,htmfac(h)
         h1=hterm(n,h)
         if (h1.ne.0) then
            call checkjm(h1,ljm)
            if (ljm) then
                ljmmode=.true.
                return
            endif
         endif
      enddo

      return
      end subroutine checkmodejm

!----------------------------------------------------------------------
! Subroutine checkjm
!
! Checks if operator contains j^2kleg,j_p,j_m,jpm,jp^2,jm^2,jpjm,jmjp terms.
!--------------------------------------------------------------------
      subroutine checkjm(h,ljm)

      implicit none

      integer(long)                :: n,hl
      integer(long), intent(in)    :: h
      logical(kind=4), intent(out) :: ljm

      ljm=.false.
      do n=1,htmfac(h)
         hl=hterm(n,h)
         if (hl.ne.0) then
            if (hoplab(hl)(1:7) .eq. 'j^2kleg'   .or. &
                hoplab(hl)(1:3) .eq. 'j_p'       .or. &
                hoplab(hl)(1:3) .eq. 'j_m'       .or. &
                hoplab(hl)(1:3) .eq. 'jpm'       .or. &
                hoplab(hl)(1:9) .eq. 'dth1_kleg' .or. &
                hoplab(hl)(1:9) .eq. 'dth2_kleg' .or. &
                hoplab(hl)(1:4) .eq. 'jpjm'      .or. &
                hoplab(hl)(1:4) .eq. 'jmjp'      .or. &
                hoplab(hl)(1:4) .eq. 'jp^2'      .or. &
                hoplab(hl)(1:4) .eq. 'jm^2')     then

               ljm=.true.
               return
            endif
         endif
      enddo

      return
      end subroutine checkjm

!#######################################################################
! 
! zpot is used if potential is complex - but must be fixed
! manually - see below
!
!#######################################################################

      subroutine htmpot(h,ort,pot,zpot,muldsize1,lz)

      implicit none

      integer(long)       :: p,mgpnt,gpnt,n,f,muld,mode,&
                             nf,hl,nh,nh1,h1,idum
      integer(long), intent(in)                   :: h,muldsize1
      integer(long), dimension(maxdim)            :: grid
      real(dop)                                   :: v
      real(dop), dimension(maxdim)                :: rdum
      real(dop), dimension(maxdim)                :: gpoint
      real(dop), dimension(ortdim),intent(in)     :: ort
      real(dop), dimension(muldsize1),intent(out) :: pot
      complex(dop)                                :: vz
      complex(dop), dimension(muldsize1), intent(out)      :: zpot
      logical(kind=4)                             :: lqc
      logical(kind=4), intent(out)                :: lz

!-----------------------------------------------------------------------
! get information on dofs from muld or mode information
!-----------------------------------------------------------------------
      muld=htmmuld(h)
      mode=htmmode(h)
      nf=0
      if (muld .ne. 0) then
         nf=nmulddof(muld)
         do n=1,nf
            grid(n)=mulddof(n,muld)
         enddo
      else if (mode .ne. 0) then
         nf=nspfdof(mode)
         do n=1,nf
            grid(n)=spfdof(n,mode)
         enddo
      endif

! if direct dynamics potential, return
      lqc = .false.
      do nh=1,htmfac(h)
         hl=hterm(nh,h)
         if(hl .gt. 0) then
            if(hopfile(hl).eq.205 .or. hopfile(hl).eq.208 &
                 .or. hopfile(hl).eq.209) lqc =.true.
         endif
      enddo
      if (lqc) return

!.....Check, if there is a complex potential
      lz=.false. 
      if (htmform(h) .eq. 0) then
         do nh=1,htmfac(h)
            hl=hterm(nh,h)
            if(hl .gt. 0) then
               if(hopfile(hl).gt.100 .and. hopfile(hl).lt.200) lz=.true. 
            endif
         enddo
      else
         do nh1=1,htmfac(h)
            h1=hterm(nh1,h)
            if (h1 .gt. 0) then
               do nh=1,htmfac(h1)
                  hl=hterm(nh,h1)
                  if(hl .gt. 0) then
                     if(hopfile(hl).gt.100 .and.  &
                        hopfile(hl).lt.200) lz=.true. 
                  endif
               enddo
            endif
         enddo
      endif
      if(lz) then
         message='Complex multi-dimensional analytic potentials '// &
                 'not allowed (see note in htmpot).'
         routine='htmpot (htmsetup.F)'
         call errormsg
      endif

      do p=1,muldsize1
         gpnt=p-1
         do n=1,nf
            f=grid(n)
            mgpnt=gpnt
            gpnt=gpnt/gdim(f)
            mgpnt=mgpnt-gpnt*gdim(f)
            gpoint(f)=ort(zort(f)+mgpnt)  
         enddo

         call poteval(h,v,gpoint,vz,rdum,idum)

         pot(p)=v
         zpot(p)=vz
      enddo
  
      return
      end subroutine htmpot

!#######################################################################
!
! get operator type
!
!#######################################################################

      subroutine getoptype(htmoptyp,h,f,m,muld,f1)

      implicit none

      integer(long)       :: n,hl,hh,fh
      integer(long), intent(in) :: h,f,m,muld,f1
      integer(long), intent(out) :: htmoptyp
    
      htmoptyp=0

!-----------------------------------------------------------------------
! 1. all 1D operators are optyp=1
! 2. all muld operators are optyp=2
! 3. mode operators are more complicated and depend on basis/operator
! 4. analytic operators do not need to be calculated
!-----------------------------------------------------------------------
      if (htmtype(h) .gt. 100 .and. htmtype(h).lt.200) then
          htmoptyp=0

      else if (f .ne. 0) then
          htmoptyp=1

      else if (muld .ne. 0) then
          htmoptyp=2

!-----------------------------------------------------------------------
! sphFBR basis
! product operators are FBR integrals, optyp=3 unless unit
! non-product operators will be optyp 1 if e.g. unit, j^2sph, jzsph, jzzsph).
!-----------------------------------------------------------------------
      else if (basis(f1) .eq. 6) then

         if (htmform(h) .eq. 1) then
            htmoptyp=2
            do n=1,htmfac(h)
               hh=hterm(n,h)
               fh=htmdof(hh)
               if (hh .ne. fh) htmoptyp=3
            enddo
           
         else if (htmform(h) .eq. 0) then
            htmoptyp=3

            hl=hterm(1,h)
            if (hoplab(hl) .eq. 'j^2sph' .or. &
                hoplab(hl) .eq. 'jzsph'  .or. &
                hoplab(hl) .eq. 'jzzsph' .or. &
                hoplab(hl) .eq. '1') then
                htmoptyp=1
            endif

         endif

!-----------------------------------------------------------------------
! KLEG/PLEG  operators are type 4. 3D-Tensors:         (15-29) 
!  15=dth1,dth2, 16=jpm, 17=j^2kleg, 18=j_p, 19=j_m, 20=jp^2, 21=jm^2
!  22=jpjm, 23=jmjp
!-----------------------------------------------------------------------
      else if (htmtype(h) .ge. 15  .and.  htmtype(h) .le. 29) then
         htmoptyp=4

!-----------------------------------------------------------------------
! combined KLEG operators
! 30=cjpm
!-----------------------------------------------------------------------
      else if (htmtype(h) .eq. 30) then
         htmoptyp=5

!-----------------------------------------------------------------------
! Wigner operators are type 6. 4D-Tensors: (40-50)
! 40=j^2wig, 41=j_pwig, 42=j_mwig, 43=jp2wig, 44=jm2wig, 45=jpmwig
! 46=j_pswig, 47=j_mswig, 48=jp2swig, 49=jm2swig, 50=jpmswig, 51=jpjmwig
! 52=jmjpwig, 53=jpjmswig, 54=jmjpswig, 55=jpjzwig, 56=jmjzwig,
! 57=jzjpwig, 58=jzjmwig, 59=jpjzswig, 60=jmjzswig, 61=jzjpswig,
! 62=jzjmswig, 63-69 (Wigner unassigned)
!-----------------------------------------------------------------------
      else if (htmtype(h) .ge. 40  .and.  htmtype(h) .le. 69) then
         htmoptyp=6

!-----------------------------------------------------------------------
! remaining mode operators should be type 2
!-----------------------------------------------------------------------
      else if (m .ne. 0) then
          htmoptyp=2

      endif

      return
      end subroutine getoptype

!#######################################################################
!     WRSCRATCH writes operators to scratch file. The scratch file will
!     be read in  subroutine wroper.
!-----------------------------------------------------------------------

      subroutine wrscratch(iwr,h,hopz,hopd,lz)

      implicit none

      integer(long)                                 :: g,f,m,muld,f1,m1,sdm,&
                                                       kdim,f2,mmdim,nel
      integer(long), intent(in)                     :: h,iwr
      real(dop), dimension(*), intent(in)           :: hopd
      complex(dop), dimension(*), intent(in)        :: hopz
      logical(kind=4), intent(in)                   :: lz


!-----------------------------------------------------------------------
! get info on associated dof/mode
!-----------------------------------------------------------------------
      call assocdof(h,f,m,muld,f1,m1,sdm)
      write(iwr) h

      if(lz) then
            write(iwr) 10
            write(iwr) (hopz(g),g=1,sdm)
      else
         if (htmtype(h) .eq. 1 .or. htmtype(h) .eq. 2 .or. &
             htmtype(h) .eq. 12 .or. htmtype(h) .eq. 13 .or. &
             htmtype(h) .eq. 202 ) then
            write(iwr) 1
            write(iwr) (hopd(g),g=1,sdm)
         else if (htmtype(h) .eq. 11) then
            if (m1.ne.0) then
               f2=htmadof(h)
               sdm=vgdim(f2+1)*ngdim(f2+1)
            endif
            write(iwr) 1
            write(iwr) (hopd(g),g=1,sdm)

         else if (htmtype(h) .ge. 15  .and.  htmtype(h) .le. 24) then
            write(iwr) 2
            f1=htmadof(h) ! NEWKLEG
            if (basis(f1).eq.7 .or. basis(f1).eq.20) then
               kdim=gdim(f1+1)
            elseif(basis(f1).eq.9 .or. basis(f1).eq.21) then
               kdim=ipbaspar(3,f1+1)
            else
               kdim=0
            endif
            sdm=gdim(f1)*gdim(f1)*kdim
            write(iwr) (hopd(g),g=1,sdm)
         else if (htmtype(h) .eq. 25) then
            nel=nint(hopd(1))
            write(iwr) 25
            write(iwr) (hopd(g),g=1,3*nel+1)
         else if (htmtype(h) .eq. 30) then
            write(iwr) 3
            f1=htmadof(h)
            f2=f1+2
            kdim=gdim(f1+1)
            sdm =gdim(f1)*gdim(f1)*kdim + kdim
            kdim=gdim(f2+1)
            sdm =sdm + gdim(f2)*gdim(f2)*kdim + kdim
            write(iwr) (hopd(g),g=1,sdm)
         else if (htmtype(h) .ge. 40 .and. htmtype(h) .le. 69) then
            write(iwr) 4
            f1=htmadof(h)
            kdim=ipbaspar(4,f1)
            mmdim=ipbaspar(8,f1)
            sdm=gdim(f1)*gdim(f1)*kdim*mmdim
            write(iwr) (hopd(g),g=1,sdm)
         else   ! htmtype= 3,4..,10
            write(iwr) 0
            write(iwr) (hopz(g),g=1,sdm*sdm)
         endif
      endif

      call myflush(iwr)

      return
      end subroutine wrscratch

!#######################################################################

      subroutine rddiag(a,b,dim)

      implicit none
      integer(long)                                :: i
      integer(long), intent(in)                    :: dim
      real(dop), dimension(dim),  intent(out)      :: b
      complex(dop), dimension(dim,dim), intent(in) :: a

      do i=1,dim
         b(i)=dble(a(i,i))
      enddo

      return
      end subroutine rddiag

!#######################################################################

      subroutine rdzdiag(a,b,dim)

      implicit none

      integer(long)                                :: i
      integer(long), intent(in)                    :: dim
      complex(dop), dimension(dim),  intent(out)   :: b
      complex(dop), dimension(dim,dim), intent(in) :: a

      do i=1,dim
         b(i)=a(i,i)
      enddo

      return
      end subroutine rdzdiag

!#######################################################################
      
      subroutine rdzsdiag(a,b,k,dim)

      implicit none
      integer(long)                                :: i
      integer(long), intent(in)                    :: k,dim
      complex(dop), dimension(dim),  intent(out)   :: b
      complex(dop), dimension(dim,dim), intent(in) :: a

      if (k.ge.0) then
         do i=1,dim-k
            b(i)=a(i+k,i)
         enddo
      else
         do i=1,dim+k
            b(i)=a(i,i-k)
         enddo
      endif
      do i=dim-abs(k)+1,dim
         b(i)=(0.0_dop,0.0_dop)
      enddo
      
      return
      end subroutine rdzsdiag

!***********************************************************************
!
! GENHTM2KLEG: calculates operators for combined KLegs (cjpm...)
!
! h - number of Hamiltonian term
! dvrmat - DVR derivative matrices
! hmat - result
!
! 06/2006 FO
!
!***********************************************************************

      subroutine genhtm2kleg(h,dvrmat,hmat)

      implicit none

      integer(long)       :: hl,hmatdim,ifunc,i,n,f1,f2,gdim1,gdim2,&
                             kdim1,kdim2,kmin1,kmin2

      integer(long), intent(in)                       :: h
      real(dop), dimension(dvrdim,dvrord), intent(in) :: dvrmat
      real(dop)                                       :: coeff,v
      real(dop), dimension(*), intent(out)            :: hmat

      routine = 'genhtm2kleg'

!     for now, we just make sure that no features are used that probably
!     won't work, and then call func2kleg.

      if (htmform(h).ne.0) then
         message = 'operators for combined KLegs (cjpm etc.) ' &
            //'must not be multiplied with other operators ' &
            //'(feature not implemented)'
         call errormsg
      endif

      coeff=1.0_dop
      do n=1,htmfac(h)
         hl=hterm(n,h)
         if (hopfile(hl).eq.7) then
!           generate matrix representation of operator
            ifunc=hopilab(hl)
            f1=htmadof(h) ! first KLeg
            f2=f1+2       ! second KLeg must follow directly
            gdim1=gdim(f1)
            kdim1=gdim(f1+1)
            kmin1=ipbaspar(1,f1+1)
            gdim2=gdim(f2)
            kdim2=gdim(f2+1)
            kmin2=ipbaspar(1,f2+1)

            hmatdim = (kdim1+kdim2) + &
                      (gdim1*gdim1*kdim1) +  &
                      (gdim2*gdim2*kdim2)
            call func2kleg(hmat,hmatdim,ifunc,jtot, &
                           gdim1,kdim1,kmin1,dvrmat(zdvr(f1),1), &
                           gdim2,kdim2,kmin2,dvrmat(zdvr(f2),1))

         elseif (hopfile(hl).eq.301) then
!           coefficient
            coeff=coeff*hoprpar(1,hl)

         elseif (hopfile(hl).eq.302) then
!           yes, "-1" is an analytic 1D function! 
            ifunc=hopilab(hl)
            if (ifunc.lt.1 .or. ifunc.gt.2) then
               message = 'operators for combined KLegs (cjpm etc.) ' &
                  //'must not be multiplied with analytic 1D functions ' &
                  //'(feature not implemented)'
               call errormsg
            else
               call funcanld(v,0.0_dop,ifunc,hopipar(1,hl),hoprpar(1,hl))
               coeff=coeff*v
            endif
         else
!           unsupported
            write (message,'(a,i4,a)') 'hopfile =',hopfile(hl), &
               'not supported.'
            call errormsg
         endif
      enddo ! loop over hterm factors

!     multiply C+ part with coefficient
      do i=1,kdim1+kdim2
         hmat(i)=hmat(i)*coeff
      enddo

      return
      end subroutine genhtm2kleg 

!***********************************************************************
!
! HTMGENSUBP
!                This is for sub-particle operators.
!
! 10/2006 FO
!
!***********************************************************************

      subroutine htmgensubp(h,hdiag,rdop,sdm)

      implicit none

      integer(long)                                    :: hl, ifunc
      integer(long), intent(in)                        :: h,sdm
      complex(dop), dimension(maxsubpdim), intent(out) :: hdiag
      real(dop), dimension(rdopdim), intent(in)        :: rdop
      real(dop), dimension(rdopdim)                    :: hopd

      routine='htmgensubp'

!     products not implemented
      if (htmform(h).ne.0 .or. htmfac(h).ne.1) then
         message='Implement me: product of sub-particle operators'
         call errormsg
      endif
      hl=hterm(1,h)

      if (hopfile(hl).eq.1) then
!        copy data from read-file
         ifunc=hopilab(hl)
         call cpvxdz(rdop(zrdop(ifunc)),hdiag,sdm)

      elseif (hopfile(hl).eq.9) then
!        MLPF leaf potentials
         ifunc=hopilab(hl)
         call callmlop(hopd,sdm,ifunc,hopipar(1,hl),hopipar(2,hl))
         call cpvxdz(hopd,hdiag,sdm)

      else
!        others not implemented
         write (message,'(a,i6)') 'Implement me: hopfile =',hopfile(hl)
         call errormsg
      endif

      return
      end subroutine htmgensubp
   
!#######################################################################
!
! GETHTMTYPE
!
! finds types of hterms
! NB, potential operators of htmtype(h) = 2 will be re-typed after
! setting up the operator in htmsetup.
!
!#######################################################################

      subroutine gethtmtype(ort,dvrmat,fftp,kinsph,rdop,trafo,done)

      implicit none

      integer(long)       :: h,f,m,muld,sdm,f1,m1,n,nh,hl,h1,hts,&
                             icmplx,iorder,shift
      integer(long), dimension(maxhop), intent(inout) :: done

      real(dop), dimension(ortdim), intent(in)        :: ort
      real(dop), dimension(fftdim), intent(in)        :: fftp
      real(dop), dimension(sphdim), intent(in)        :: kinsph
      real(dop), dimension(rdopdim), intent(in)       :: rdop
      real(dop), dimension(dvrdim), intent(in)        :: trafo
      real(dop), dimension(dvrdim,dvrord), intent(in) :: dvrmat
      logical(kind=4)                                 :: ljm,lfft,lsd
      complex(dop), dimension(:), allocatable  :: hopz
      real(dop), dimension(:), allocatable     :: hopd

      character(len=20)                          :: cerr
      integer(long)                              :: ierr

      routine='gethtmtype'

!-----------------------------------------------------------------------
! start loop over hterms
!-----------------------------------------------------------------------
      h=1
   10 continue

!-----------------------------------------------------------------------
!  only type operators which need to be 
!-----------------------------------------------------------------------
      if (htmtype(h) .ge. 0) go to 100

!-----------------------------------------------------------------------
! get info on associated dof/mode
! f,m,muld are non-zero if DOF, mode or muld operator
! if mode operator, f1,m1 refer to spfdof(1,m) and m
!-----------------------------------------------------------------------
      call assocdof(h,f,m,muld,f1,m1,sdm)

!  Special treatment for coefficient.
      if (f.eq.fkoe) then
         hts=0
         icmplx=0
         htmtype(h)=2
         htmsym(h) =1
         do nh=1,htmfac(h)
            hl=hterm(nh,h)
            if(hl.gt.0) then
!           hl=0 -> separation of two summands.
               if(hoplab(hl)(1:1).eq.'I') then
                  iorder=1
                  if(hoplab(hl)(1:2).eq.'I^') &
                  read(hoplab(hl)(3:4),*) iorder
                  htmsym(h) = htmsym(h)*(-1)**iorder
               endif 
            else
               if(hts.ne.0 .and. hts.ne.htmsym(h)) then
                  icmplx=1
               else
                  hts=htmsym(h)
                  htmsym(h)=1
               endif
            endif
         enddo
         if(htmsym(h).eq.-1) htmtype(h)=3 

         if(hts.ne.0 .and. (icmplx.gt.0 .or. hts.ne.htmsym(h))) then
!        Summation of real and imaginary coefficient -> complex coef.
            htmtype(h)=4
            htmsym(h) =0
         endif

         goto 100
      endif      ! End special treatment for coefficients

!  Special treatment for Time operators
      if (f.eq.ftime) then
         if (htmfac(h) .eq. 1 .and. hterm(1,h) .eq. 1) then
            htmtype(h)=1
         else
            htmtype(h)=2
         endif
      endif

! product operators. May be kleg*operator or potential or side-diagonal
! if complex or imaginary operator in product, make complex
      if (htmform(h) .eq. 1) then
         htmtype(h)=2

         lsd=.false.
         do n=1,htmfac(h)
            h1=hterm(n,h)
            if (h1 .ne. 0) then
               call checkjm(h1,ljm)
               if (ljm) then
                  htmtype(h)=htmtype(h1)
                  htmadof(h)=htmadof(h1) ! NEWKLEG
               endif
               if (htmtype(h1).eq.12) then
                  lsd=.true.
               endif

               if (htmtype(h1).eq.3 .or. htmtype(h1) .eq. 4) then
                  htmtype(h)=4
               endif

               if (htmtype(h1) .eq. 151) then
                  htmtype(h)=151
               endif

               if (htmtype(h1) .eq. 202) then
                  htmtype(h)=202
               endif

               if (htmtype(h1) .eq. 204) then
                  htmtype(h)=204
               endif

            endif
         enddo

!        figure out overall htmshift
         if (lsd) then
            shift=0
            n=htmfac(h)
  12        h1=hterm(n,h)
            if (h1.eq.0) goto 13
            f1=htmadof(h1)
            shift=gdim(f1)*shift+htmshift(h1)
            n=n-1
            if (n.gt.0) goto 12
  13        htmtype(h)=12
            htmshift(h)=shift
         endif

         go to 100
      endif

! see if special operator
      call asshtmlab(h,f1)
      if (htmtype(h) .ge. 0) go to 100

! guess potential term
      htmtype(h)=2
      do nh=1,htmfac(h)
         hl=hterm(nh,h)
         if (hl .ne. 0) then
            if (hopfile(hl) .gt. 0 .and. hopfile(hl) .lt. 200)  &
                htmtype(h)=-1
         endif
      enddo
      if (htmtype(h) .ge. 0) go to 100

! guess imaginary potential term (CAP) - stored in file 101 (may be multiplied
! by real terms)
      htmtype(h)=3
      do nh=1,htmfac(h)
         hl=hterm(nh,h)
         if (hl .ne. 0) then
            if (hopfile(hl) .gt. 0 .and. hopfile(hl) .lt. 101)  &
                htmtype(h)=-1
            if (hopfile(hl) .gt. 101 .and. hopfile(hl) .lt. 200)  &
                htmtype(h)=-1
         endif
      enddo
      if (htmtype(h) .ge. 0) go to 100

! calculate operator and see
      cerr = 'hopz '
      allocate(hopz(sdm*sdm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'hopd '
      allocate(hopd(sdm),stat=ierr)
      if (ierr > 0) go to 1000
      call htmgen1d(h,f,hopz,ort(zort(f1)),dvrmat(zdvr(f1),1), &
           dvrmat(zdvr(f1),2),fftp(zfft(f1)),kinsph(zsph(f1)), &
           rdop,sdm,hopd,lfft,'gethtmtype  ',trafo(zdvr(f1)),done)

      call assignzhtm(hopz,htmtype(h),htmshift(h),sdm)
      if (htmtype(h).eq.0) call outhtm(h,ilog) ! write removed operators

      write(cerr,'(a,i5)') 'gethtmtype_...',h
      if(allocated(hopz))deallocate(hopz,stat=ierr)
      if(allocated(hopd))deallocate(hopd,stat=ierr)
      if (ierr > 0) go to 1001

!-----------------------------------------------------------------------
! end loop over hterms
!-----------------------------------------------------------------------
  100 continue

!-----------------------------------------------------------------------
! carry on if more terms to check
!-----------------------------------------------------------------------
      if (h .lt. min(htmdim,maxhtm)) then
         h=h+1
         go to 10
      endif

      return

 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      stop

 1001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
      stop

      end subroutine gethtmtype

!-----------------------------------------------------------------------
! c(i,j) = a(j,j,i)
!-----------------------------------------------------------------------
      subroutine diagten(ten,mat,gdim1,kdim)

      implicit none

      integer(long)                                         :: k,g

      integer(long), intent(in)                             :: gdim1,kdim
      real(dop), dimension(gdim1,gdim1,kdim), intent(in)    :: ten
      real(dop), dimension(gdim1,kdim), intent(out)         :: mat

      do k=1,kdim
            do g=1,gdim1
               mat(g,k)=ten(g,g,k)
         enddo
      enddo

    return
    end subroutine diagten

!------------------------------------------------------------------------


      end module htmsetupmod

