!#######################################################################
! contain: logop,buffterm,buffmdtm,breakstr,outhtm
!
!#######################################################################

      module logopmod

use decimal, only: long, dop
use mltypes
use logdat
use op1lib,  only: zerovxi, zerovxl
use operdef

use global
use maxv
use dvrdatmod
use dirdyn
use gwptrj
use psidef
use griddatmod
use runopermod
use paths
use channels
use gapmod

use compdat, only: getcsys,savecsys
use funcsrf, only: surfinfo
use einsbasmod, only: einexact
use operutil, only: findnath
use rdopermod, only: operinfo
use rddvrmod, only: dvrinfo
use einparamod, only: einpara
use iopsidef, only: psidat

use llist_int
use mlham, only: mlham_get
use optree, only: optree2txt
use rdopermod, only: lopsetprop

      implicit none

      private
      public :: logop,logopdat,buffterm,outhtm,logkterms,chkmuld
      contains

!#######################################################################
!  LOGOPDAT  reads the oper file and writes information on the operator 
!  to the op.log file
!#######################################################################

      subroutine logopdat

      implicit none

      logical(kind=4)           :: lerr
      character(len=c5)         :: filename
      integer(long)             :: ilbl,chkdvr,chkgrd

!-----------------------------------------------------------------------
! open oper file
!-----------------------------------------------------------------------
      filename=oname(1:olaenge)//'/oper'
      ilbl=index(filename,' ')-1
      open(ioper,file=filename,form='unformatted',status='old',err=900)

!-----------------------------------------------------------------------
! get memory sizes from oper file
!-----------------------------------------------------------------------
      call rdmemdim(ioper)

!-----------------------------------------------------------------------
! Allocate mememory and read system / oper information
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      if (lgwptrj) call alloc_gwptrj
      if (ldd .or. lgwptrj .or. lddmctdh .or. lddtrans .or. ltraj) then
         call alloc_dirdyn
         call alloc_dddb
         call alloc_dercp
      endif
      if(lddmctdh.or.lgausspot)call alloc_gapkernel

!-----------------------------------------------------------------------
! Read system / DVR information
!-----------------------------------------------------------------------
      filename=dname(1:dlaenge)//'/dvr'
      ilbl=index(filename,' ')-1
      open(idvr,file=filename,form='unformatted',status='old',err=900)
      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      close(idvr)

!-----------------------------------------------------------------------
! Read operator / system parameters
!-----------------------------------------------------------------------
      call einpara

!-----------------------------------------------------------------------
! read oper file
!-----------------------------------------------------------------------
      chkdvr=2
      chkgrd=1
      call operinfo(lerr,chkdvr,chkgrd)
      close(ioper)

!-----------------------------------------------------------------------
! log data
!-----------------------------------------------------------------------
      call logop

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_dirdyn
      call dealloc_dddb
      call dealloc_gwptrj

      return

  900 continue
      routine='Logop'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop

      end subroutine logopdat

!#######################################################################
!  LOGOP  writes information on the operator to the op.log file
!#######################################################################

      subroutine logop


      implicit none
     
      integer(long)       :: m,n,f,s,h,k,ilbl,jlbl,np,hl,k1,k2,nham,&
                             muld,gf,fg,i,ncorr,h1,kl,kh,j,hsymtot,&
                             nlines,ns,n1,vdim1,imag
      integer(long), external :: slen

      character(len=c1), dimension(ndof)         :: label
      character(len=8)                           :: labdiag
      character(len=4)                           :: pestr
      character(len=4), dimension(3)             :: bff
      character(len=8*maxdim+16)                 :: buffer
      logical(kind=4)                            :: lmmuld,lfirst, &
                                                    lherm,usedncut,&
                                                    lhamr,lhamt,&
                                                    lhermm,lherma,lherm0,&
                                                    lhermuncor,&
                                                    lrealuncor,&
                                                    luncorr=.false.
      logical(kind=4), dimension(maxhtm)         :: lwrite
      logical(kind=4)                            :: lqctm1 = .true. 

      integer(long), dimension(:), allocatable   :: done

!-----------------------------------------------------------------------
! --- OPEN OP.LOG FILE ---  (iolog=6 for test-run : -o option)
!-----------------------------------------------------------------------
      if(lrunpes) then
         buffer=name(1:laenge)//'/op.log.pes'
         i=laenge+11
      else if(lopsetprop) then
         buffer=name(1:laenge)//'/op.log.prp'
         i=laenge+11
      else
         buffer=name(1:laenge)//'/op.log'
         i=laenge+7
      endif

      if (iolog.ne.6 .and. iolog .ne. ilog) &
         open(iolog,file=buffer(1:i),status='unknown',form='formatted',err=35)

      jlbl=0

      bff(1)='imag'
      bff(2)='cmpx'
      bff(3)='real'

      write(ilog,'(/,a,/)') '***** Operator details *****' 
      write(ilog,'(a)') 'DOF no.    label            Mass'
      do f=1,ndof
         write(ilog,'(i5,7x,a10,f12.4)') f,modelabel(f),mass(f)
      enddo

      write(ilog,'(/,a)') 'Mode     DOFs'
      do m=1,nmode
         if( nspfdof(m) .le. 6 ) then
            write(ilog,'(i4,2x,6(i3,a1,a8))') m, (spfdof(n,m), &
                 '=',modelabel(spfdof(n,m))(1:8), n=1,nspfdof(m))
         else
            j=min(14,nspfdof(m))
            write(ilog,'(i4,2x,14(i5))') m,(spfdof(n,m),n=1,j)
            if (j .lt. nspfdof(m)) &
                write(ilog,'(6x,14i5)') & 
                   (spfdof(n,m),n=15,nspfdof(m))
         endif
      enddo
      if (lmult .and. gdim(feb) .gt. 1) &
          write(ilog,'(a)') '  Multi-set Operator '
      
      if (lanalpes) write(ilog,'(/,a)')  'Analytic PES'
      if (ldirmuld) write(ilog,'(/,a)')&
           'Direct algorithm used for multi-dimensional potentials'
      if (lgwp) &
      write(ilog,'(/,a,i2)')&
           'GWP integrals set up with moments to order: ',lhaorder

      if (lpntdvr) then
         write(ilog,'(/,a,/)') 'A cut through the full PES is taken'
         write(ilog,'(a)') 'The following coordinates are fixed:'
         do n=1,nspfdof(mkoe)
            f=spfdof(n,mkoe)
            if (basis(f) .eq. 19) then
               write(ilog,'(2a,f20.8)') modelabel(f),' = ',xend(1,f)
            endif
         enddo
      endif

      write(ilog,'(/,a,/)')  '***** Paths for spline data files *****'
      do n=1,nsplpaths
         ilbl=slen(splpaths(n))
         write(ilog,'(i3,1x,a)') n,splpaths(n)(1:ilbl)
      enddo

      if (ldd.or.lddmctdh) then
         write(ilog,'(/,a,/)')  &
                '***** Direct Dynamics Calculation *****' 

         ilbl=index(qcprog,' ')-1
         buffer=qcprog
         call keyupc(buffer)
         write(ilog,'(2a)') 'Quantum Chemistry Program : ', &
                buffer(1:ilbl)
         ilbl=index(qcmethod,' ')-1
         buffer=qcmethod
         call keyupc(buffer)
         write(ilog,'(2a)') 'Quantum Chemistry Method : ', &
                buffer(1:ilbl)
         ilbl=index(ddextcmd,' ')-1
         if (qcsubform .eq. 1) then
            write(ilog,'(3a)') 'Submission Command : ', &
                ddextcmd(1:ilbl),'  $job'
         else if (qcsubform .eq. 2) then
            write(ilog,'(3a)') 'Submission Command : ', &
                ddextcmd(1:ilbl),'  $job    $jobdir'
         endif

         buffer = 'States included: '
         ilbl = 17
         do ns=1,nsmult
            if (idxmult(ns) .eq. 1) then
               write(buffer(ilbl+1:ilbl+5),'(i0,a)') imultblk(ns),'S'
            else if (idxmult(ns) .eq. 2) then
               write(buffer(ilbl+1:ilbl+5),'(i0,a)') imultblk(ns),'D'
            else if (idxmult(ns) .eq. 3) then
               write(buffer(ilbl+1:ilbl+5),'(i0,a)') imultblk(ns),'T'
            else if (idxmult(ns) .eq. 4) then
               write(buffer(ilbl+1:ilbl+5),'(i0,a)') imultblk(ns),'Q'
            endif
            ilbl = ilbl+5
         enddo
         write(ilog,'(a)') buffer(1:ilbl)

         if (nsmult .gt. 1) then
            if (lsocdegen) then
               write(ilog,'(a)') 'Spin states treated as a degenerate set '
            else
               write(ilog,'(a)') 'Spin states treated as a full SOC split set '
            endif
         endif
               
         if (lddreord) then
            write(ilog,'(a)') 'Atoms in DB reordered in mass families'
         endif

         write(ilog,'(a,i10)') 'Maximum no. of records in DB: ',dbmemdim

         write(ilog,'(/,a)') '*** Atom data *** '
         write(ilog,'(a)') 'Name          Mass    '
         do n=1,natm
            write(ilog,'(a8,f12.4)') xatnam(n),xmass(n)
         enddo

         if (ddcoo .eq. 0) then
            write(ilog,'(/,a)') 'Jacobi Coordinates Used.'
         else if (ddcoo .eq. 1) then
            write(ilog,'(/,a)') 'Normal mode Coordinates Used.'
            ilbl=index(ddtrfile,' ')-1
            write(ilog,'(2a)') 'Trafo File: ',ddtrfile(1:ilbl)
         else if (ddcoo .eq. 2) then
            write(ilog,'(/,a)') 'Cartesian Coordinates Used.'
         endif
      endif 

!.....Overwrite the "Removed operators" lines, when no ops removed. 
      if(nremoved.eq.0 .and. iolog.ne.6) then
         backspace iolog
         backspace iolog
         backspace iolog
      endif

      write(iolog,'(/,a,/)') '******* Array-Dimensions *******'
      write(iolog,'(a,i8)') ' k   :  maxkoe', maxkoe
      write(iolog,'(a,i8)') ' h   :  maxhtm', maxhtm
      write(iolog,'(a,i8)') ' hl  :  maxhop', maxhop
      write(iolog,'(a,i8)') ' nh  :  maxfac', maxfac
      write(iolog,'(a,i8)') ' n   :  maxdef', maxdef
      write(iolog,'(a,i8)') ' n   :  maxpar', maxpar
      write(iolog,'(a,i8)') ' n   :  maxrdf', maxrdf
      write(iolog,'(a,i8)') ' nham:  maxham', maxham
      write(iolog,'(a,i8)') ' muld: maxmuld', maxmuld
      write(iolog,'(a,i8)') ' LMR :  maxLMR', maxLMR

      if(gmatf1.gt.0) write(iolog,'(/a,2i5,4x,a,2x,a/)')  &
            'DOFs used by gengmat:',gmatf1,gmatf2, &
            modelabel(gmatf1)(1:slen(modelabel(gmatf1))), &
            modelabel(gmatf2)(1:slen(modelabel(gmatf2)))

      write(iolog,'(/,a,/)') '***** Operator details *****'

      write(iolog,'(2(a,i4))') 'ndof   =',ndof  ,' ,  nmode   =',nmode
      write(iolog,'(2(a,i4))') 'nopdof =',nopdof,' ,  nopmode =',nopmode
      write(iolog,'(a)') 
      write(iolog,'(5(a,i3))') 'feb =',feb,',  meb =',meb,&
           ',   fpb =',fpb,',  mpb =',mpb,',  mkoe =',mkoe
      write(iolog,'(a,l2,a,l2,a,l2)')&
           'leb =',leb,',  lmult =',lmult,',  lmulpack =',lmulpack
      write(iolog,'(a)') 'DOF no.    label            mass'
      do f=1,nopdof
         write(iolog,'(i5,7x,a10,f12.4)') f,modelabel(f),mass(f)
         if(f.eq.ndof) write(iolog,'(a)')'- - - - - - - - - - - - - - -'
      enddo

      write(iolog,'(/,a)') 'Mode     DOFs'
      do m=1,nopmode
         if( nspfdof(m) .le. 6 ) then 
            write(iolog,'(i4,2x,6(i3,a1,a8))') m, (spfdof(n,m), &
                 '=',modelabel(spfdof(n,m))(1:8), n=1,nspfdof(m))
         else
            j=min(14,nspfdof(m))
            write(iolog,'(i4,2x,14(i5))') m,(spfdof(n,m),n=1,j)
            if (j .lt. nspfdof(m)) &
                write(iolog,'(6x,14i5)')  &
                      (spfdof(n,m),n=15,nspfdof(m))
         endif
         if(m.eq.nmode)write(iolog,'(a)')'- - - - - - - - - - - - - - -'
      enddo

      if (hamdim .gt. 1) then
         write( ilog,'(/a,i5)')'Number of Hamiltonians defined: ',hamdim
         write(iolog,'(/a,i5)')'Number of Hamiltonians defined by user   : ',&
            nhamint0-1
         if (nhamint0 .le. hamdim) &
            write(iolog,'(a,i5)')'Number of Hamiltonians defined internally: ',&
                hamdim-nhamint0+1
         do nham=1,hamdim
            if(nodiag(nham)) then
               labdiag = ' nodiag'
            else
               labdiag = 'usediag'
            endif
            write(ilog,'(i5,5x,a,a)')  &
                 nham,hamlab(nham)(1:21),labdiag
            write(iolog,'(i5,5x,a,a)') &
                 nham,hamlab(nham)(1:21),labdiag
         enddo
      endif

      if(mulddim .gt. 1) then
         write(iolog,'(/3x,a)') 'Info on muld'
         do muld=1,mulddim
            write(iolog,'(i3,2x,a/,(20i4))') muld, 'mulddof :',&
                 (mulddof(f,muld),f=1,nmulddof(muld))
            write(iolog,'(i3,2x,a/,(20i4))') muld, 'mulddof2:',&
                 (mulddof2(f,muld),f=1,nmulddof(muld))
         enddo
      endif

      if(allocated(done))deallocate(done)
      allocate(done(maxhop))
      done = 0

      do nham=1,hamdim
         if (nham .eq. 1 .and. hamdim .gt. 1) then
            buffer='Information for Hamiltonian : System'
         else
            buffer='Information for Hamiltonian : '//trim(hamlab(nham))
         endif
         write(iolog,'(/,a80,/)') trim(buffer)
         if (ltdh(nham)) then
            write(iolog,'(/,a)') 'Time-dependent coefficients'
         endif

         write(iolog,'(a,i7)') 'Total no. of terms      :',khtot(nham)
         ncorr=kzahl(nham)+nmulpot(nham)
         write(iolog,'(a,i7,/)') 'No. of correlated terms :',ncorr

         do m=1,nmode
            if (lmult) then
               do s=1,nstate
                  write(iolog,'(2(a,i3),a,i6)') &
                       'No. of uncorrelated terms for mode ', m,&
                       '  state',s,':',khzahl(m,s,nham)
               enddo
            else
               write(iolog,'(a,i3,a,i6)')    &
                  'No. of uncorrelated terms for mode ', m,':',&
                  khzahl(m,1,nham)
            endif
         enddo

! search for potential energy surfaces
         k1=zham(nham)
         k2=zham(nham)+khtot(nham)-1
!         allocate(done(maxhop))
!         done(:) = 0
         do k=k1,k2
            do f=1,ndof
               h=hamilton(f,k)
               if (htmform(h) .eq. 1) cycle
               hl=hterm(1,h)
               if(hopfile(hl).gt.200 .and.&
                  hopfile(hl).lt.300) call surfinfo(h,hl,done)
            enddo
         enddo
!         if(allocated(done))deallocate(done)
      enddo
      if(allocated(done))deallocate(done)

      if (lanalpes) write(iolog,'(/,a)')  'Analytic PES'

      write(iolog,'(/,a)') '  Parameters  '
      do n=1,npar
         if(rpar(n) .lt. 1.0e-5_dop .or. rpar(n) .gt. 1.0e6_dop) then
            write(iolog,'(a,1pe22.12)') apar(n),rpar(n)
         else
            write(iolog,'(a,f22.12)') apar(n),rpar(n)
         endif
      enddo

!----  Removed operators --------------
      if (nremoved .gt. 0) &
         write(iolog,'(/a,i6,",  ",a,1pe9.2)')  &
           'Number of operators removed =',nremoved,'cutoff =',cutoff   

!----  Parameters for natpotcut --------------
      usedncut=.false.
      do n=1,nrdf
         if (npotcut(n).ne.tiny) usedncut=.true.
      enddo

      if (usedncut) then
         write(iolog,'(/a)') 'Natpotcut is used:'
         write(iolog,'(a)') ' natpot no.  |  natpotcut (au)   |'//&
           ' cutted terms |  wpf(1)  wpf(2)  wpf(3)  wpf(4)  wpf(5)'
         do n=1,nrdf
            write(iolog,'(2x,i4,6x,"|",f15.10," |",5x,i8," |",20f8.5)') &
                 n,npotcut(n),ncutted(n),(wpf(m,n),m=1,nmode)
         enddo
      elseif(nrdf.gt.0) then
         write(iolog,'(/a,e10.5,a)') 'Default natpotcut = ',tiny,&
                                     ' au  is used.'
         write(iolog,'(2a)') ' natpot no. | cutted terms |  reduce-pf'
         do n=1,nrdf
            write(iolog,'(2x,i4,6x,"|",2x,i9,3x,"|",20f8.5)') &
                  n,ncutted(n),(wpf(m,n),m=1,nmode)
         enddo
      endif

!-----------------------------------------------------------------------
! find hterms of relevance
!-----------------------------------------------------------------------
      call zerovxl(lwrite,maxhtm)
! look through Hamiltonian
      do k=1,ktot
         do f=1,nopdof
            h = hamilton(f,k)
            lwrite(h)=.true.
         enddo
      enddo

      do h=1,htmdim
! add any other operators that have been set up
         if (zhop(h) .gt. 0) lwrite(h)=.true.

! add parts of product operators
         if (lwrite(h) .and. htmform(h) .eq. 1) then
            do n=1,htmfac(h)
               h1=hterm(n,h)
               if (h1 .gt. 0) lwrite(h1)=.true.
            enddo
         endif   
      enddo

      write(iolog,'(/,a,4x,a)') '  Operator Labels',  &
           '[hl,hoplab,(deflabel),hopilab,hopfile]'
      write(iolog,'(a)')  &
          '    hl  Label                       function no.  file no.'
      do hl=1,hopdim
         if (hopidef(hl).gt.maxdef) then
! Hack to log old oper files as hopidef was not saved
            write(iolog,'(i6,2x,a20,1x,a14,i6,i10)')  &
               hl,hoplab(hl),'Not Known',hopilab(hl),hopfile(hl)
         else if (hopidef(hl).gt.0) then
!        Print user given labels like "V" or "CAP_r"
            np=hopidef(hl)
            write(iolog,'(i6,2x,a20,1x,a14,i6,i10)')  &
               hl,hoplab(hl),deflabel(np),hopilab(hl),hopfile(hl)
         else 
            if(.not.lwrnpot .and. (hoplab(hl)(1:4).eq.'npot')) then
!...........Supress printing of npot lines (if lwrnpot=.false.)
! print first and last only
               if(hoplab(hl)(1:6).eq.'npot1,') then
                  continue
               elseif(hl .eq. hopdim .and. hoplab(hl)(1:4).eq.'npot') then
                  write(iolog,'(a,32x,a)') '   ...  ......','...       ...'
               elseif(hoplab(hl+1)(1:4).ne.'npot') then
                  write(iolog,'(2a)')  '   ...  ......', &
                   '  Set "print-npot" to see all   ...       ...'
               else
                  goto 16  ! Do not print
               endif
            endif

            if(.not.lwrnpot .and. (hoplab(hl)(1:5).eq.'cpdtm')) then
!...........Supress printing of cpd lines (if lwrnpot=.false.)
               if(hoplab(hl)(1:6).eq.'cpdtm1,') then
                  continue
               elseif(hoplab(hl+1)(1:5).ne.'cpdtm') then
                  write(iolog,'(2a)')  '   ...  ......', &
                   '  Set "print-npot" to see all   ...       ...'
               else
                  goto 16  ! Do not print
               endif
            endif
            if(.not.lwrnpot .and. (hoplab(hl)(1:6).eq.'hcpdtm')) then
!...........Supress printing of hcpd lines (if lwrnpot=.false.)
               if(hoplab(hl)(1:7).eq.'hcpdtm1,') then
                  continue
               elseif(hoplab(hl+1)(1:6).ne.'hcpdtm') then
                  write(iolog,'(2a)')  '   ...  ......', &
                   '  Set "print-npot" to see all   ...       ...'
               else
                  goto 16  ! Do not print
               endif
            endif

            write(iolog,'(i6,2x,a30,5x,i6,i10)')  &
               hl,hoplab(hl),hopilab(hl),hopfile(hl)
16          continue
         endif
      enddo

!      if (lanalpes) then
!         call savecsys(2)
!         call einexact(1)
!      endif

      write(iolog,'(a)') ' '
      if (loptcontract) then
         write(iolog,'(a)') &
           'Careful: KE and PE terms contracted where possible.'
      endif
      write(iolog,'(2a,/28x,a)') 'Hamiltonian Operator Terms [h,',&
                   'htmdof,htmadof,htmmode,htmmuld,htmtype,',&
                   'htmsym,string]'

      write(iolog,'(2x,7a4,5x,a)') ' No. ','   f','  af', &
            '   m','  md',' Typ',' Sym',' Term '
      do h=1,htmdim
         if (.not. lwrite(h)) cycle
         call outhtm(h,iolog)
      enddo
      if (any(htmshift(:) .gt. 0)) then
         write(iolog,'(a)') ' Side-diagonal Operator Shifts '
         write(iolog,'(a)') ' Operator  Shift '
         do h=1,htmdim
            if (.not. lwrite(h)) cycle
            write(iolog,'(i8,i8)') h,htmshift(h)
         enddo
      endif


!      if (lanalpes) then
!         call getcsys(2)
!      endif

      if (nhoppar .gt. 0) then
         write(iolog,*)
         write(iolog,'(a)') 'Arguments for operators'
         write(iolog,'(a)') '  No.    Arguments'
         n1=0
         do n=1,nhoppar+npesopt
            if(index(deflabop(n),'{') .eq. 0) then
               n1=n1+1
               write(iolog,'(i5,2x,a,"=",a)') n1, trim(deflabel(n)), &
                 trim(deflabop(n))
               write(iolog,'(i5,4G17.8)') n1,(hoppar(np,n1),np=1,4)
               if((abs(hoppar(5,n1)) + abs(hoppar(6,n1))).gt.1.d-12) &
               write(iolog,'(5x,4G17.8)') (hoppar(np,n1),np=5,maxhpar)
            endif
         enddo
      endif

      if (npesopt .gt. 0) then
         n1=0
         write(iolog,*)
         write(iolog,'(a)') 'Arguments for potentials'
         write(iolog,'(a)') '  No.  Label/PES      Arguments'
         do n=1,nhoppar+npesopt
            if(index(deflabop(n),'{') .gt. 0) then
               n1=n1+1
               write(iolog,'(i5,2x,a,"=",a,3x,a)') n1, &
                 trim(deflabel(n)),trim(deflabop(n)),trim(pesopts(n1))
            endif
         enddo
      endif

      if (lmuld) then
         call psidat
         write(iolog,'(/a)') &
             'Coordinate sets for multi-dimensional muld operators'
         write(iolog,'(2a)') 'Label      nham    k    h muld  vd  ', &
                             '    Size    nd    Dofs'
         do nham=1,hamdim
            do k = zham(nham)+kzahl(nham), &
                   zham(nham)+kzahl(nham)-1+nmulpot(nham)
               h=hamilton(1,k)
               muld=htmmuld(h)
               if(muld.gt.0  .and.  htmmode(h).eq.0) then
                  lfirst=.true.
                  s=ki(k)
                  vdim1=1
                  do m=1,nmode
                     if (diag(m,k)) then    ! Does not work for nodiag
                        vdim1=vdim1*dim(m,s)
                     else
                        if(lfirst) then
                           lfirst=.false.
                           vdmuld(muld)=vdim1
                        endif
                        vdim1=vdim1*subdim(m)
                     endif
                  enddo
                  ndmuld(muld)=vdim1/(vdmuld(muld)*muldsize(muld))
                  do gf=1,nmulddof(muld)
                     f=mulddof2(gf,muld)
                     label(gf)=modelabel(f)
                  enddo
                  i=1
 44               hl=hterm(i,h)
                  if(hopfile(hl).lt.200 .or. hopfile(hl).gt.300) then
                     i=i+1
                     goto 44
                  endif
                  write(iolog,'(a12,i3,2i5,i3,i6,i10,i6,12(2x,a))') &
                     hoplab(hl),nham,k,h,muld, &
                     vdmuld(muld),muldsize(muld),ndmuld(muld), &
                     (label(gf)(1:slen(label(gf))),gf=1,nmulddof(muld))
                  call chkmuld(nham,h,muld)
               endif         ! muld
            enddo            ! k-loop
         enddo               ! nham-loop
      endif

      if (lanalderiv) then
         write(iolog,'(/,a)')  &
               'Analytic derivatives used if htmderiv=1 :'
         write(iolog,'(a)') 'hterms   htmderiv'
         do h=1,htmdim
            if (lwrite(h)) write(iolog,'(i5,2x,i5)') h,htmderiv(h)
         enddo
      endif
!
! Output coordinate sets of multi-D operators
!
      lmuld=.false.
      lmmuld=.false.
      do h=1,htmdim
         if (lwrite(h)) then
            if (htmmuld(h).gt.0 .and. htmmode(h).eq.0) lmuld=.true.
            if (htmmuld(h).gt.0 .and. htmmode(h).gt.0) lmmuld=.true.
         endif
      enddo

      if (lmmuld .or. lmuld) then
         write(iolog,'(/a)')  &
             'Coordinate sets for multi-dimensional operators'
         write(iolog,'(2a)') '   Set        Dofs'
         do muld=1,mulddim
            do gf=1,nmulddof(muld)
               fg=mulddof2(gf,muld)
               label(gf)=modelabel(fg)
            enddo
            write(iolog,'(i5,5x,10(2x,a))') muld, &
                  (label(gf)(1:slen(label(gf))),gf=1,min(10,nmulddof(muld)))
            ! write remaining labels in lines of max(10).
            nlines = (nmulddof(muld)-1)/10
            do n=1,nlines
               i=n*10    
               j = min(nmulddof(muld),i+10)
               i = i+1    
               write(iolog,'(10x,10(2x,a))')  &
                     (label(gf)(1:slen(label(gf))),gf=i,j)
            enddo
            
         enddo

         write(iolog,'(/,2a)') '   hl  Label              Coord Set '
         do h=1,htmdim
            if ( lwrite(h) ) then
               muld=htmmuld(h)
               if (muld.gt.0) then
                  do i=1,htmfac(h)
                     hl=hterm(i,h)
                     if (hl .eq. 0) cycle
                     if(hopfile(hl).lt.200 .or. hopfile(hl).gt.300) cycle
                     write(iolog,'(i5,1x,a20,i5)') hl,hoplab(hl), muld
                  enddo
               endif   ! muld
            endif      ! lwrite
         enddo         ! h 
      endif            ! lmmuld

      if (nrdf .gt. 0) then
         write(iolog,'(/a)') 'Files containing operators'
         write(ilog,'(/a)') 'Files containing operators'
         write(iolog,'(a)') '  No.        Name'
         write(ilog,'(a)') '  No.        Name'
         do n=1,nrdf
            write(iolog,'(i5,5x,a70)') n,rdfiles(n)
            write(ilog,'(i5,5x,a70)') n,rdfiles(n)
         enddo
      endif
!
! print info if EXTERNAL programs used for potential
!
      do hl=1,hopdim
         if (hoplab(hl) .eq. 'qchemtrj') then
            write(iolog,'(/,a)') &
               'Precomputed trajectories provide potential'
         else if (hoplab(hl)(1:5) .eq. 'qchem') then
! only print for first term found
            if (.not. lqctm1) cycle

            write(iolog,'(/,a)') &
                        'QChem program used for potential'
            if (hopilab(hl) .eq. 1) &
               write(ilog,'(a)') 'Quantum chemistry program : GAUSSIAN'
            if (hopilab(hl) .eq. 5) &
               write(ilog,'(a)') 'Quantum chemistry program : MOLCAS'
            if (hopilab(hl) .eq. 2) &
               write(ilog,'(a)') 'Quantum chemistry program : QoMMMa'
            if (hopipar(2,hl) .eq. 0) then
                write(iolog,'(a)') 'Electronic representation : '// &
                                   'adiabatic'
            else if (hopipar(2,hl) .eq. 1 .or. hopipar(2,hl) .eq. 2) then
               write(iolog,'(a)') 'Electronic representation : '// &
                                  'quasi-diabatic'
               write(iolog,'(a)') '  local harmonic approximation'
               write(iolog,'(a)') '  0.5:0.5 state averaging'
            endif
            write(iolog,'(2a)') 'External command : ',ddextcmd
            write(iolog,'(a,f17.10)') 'Energy origin in hartree :',dde0
            write(iolog,'(a,i3)') 'Number of electronic states :', nddstate

            if (hopipar(7,hl) .eq. 1) then
               write(iolog,'(a)') 'Quantum chemistry method : RHF'
            else if (hopipar(7,hl) .eq. 12) then
               write(iolog,'(a)') 'Quantum chemistry method : CAS'
            else if (hopipar(7,hl) .eq. 19) then
               write(iolog,'(a)') 'Quantum chemistry method : DFT'
            endif

            write(iolog,'(a,i3)') 'Highest electronic state :', hopipar(8,hl)
            if (lfchk) then
               write(iolog,'(a)') 'GAUSSIAN output files : fchk'
               write(iolog,'(2a)') 'Formchk command : ',fchkcmd
            endif

            if(lddwrdb) then
               write(iolog,'(a)') 'Database status : Read-write'
               write(iolog,'(a,f8.4)') 'Read distance criterion :',ddmindb
               if (hopipar(2,hl) .eq. 1) &
                  write(iolog,'(a,f8.4)') 'Read energy criterion :',dddvmin
            else
               write(iolog,'(a)') 'Database status : Read only'
            endif

            lqctm1 = .false. 
         elseif ((hoplab(hl)(1:4)=='grow').or.(hoplab(hl)(1:7)=='shepard')) then
           write(iolog,'(a)')'Shepard interpolation (grow) potential'
           write(iolog,'(a)')'(See OUT_SHEPARD)'
         endif
      enddo

      nham=0
!-----------------------------------------------------------------------
! loop over Hamiltonians
!-----------------------------------------------------------------------
   10 continue
      nham = nham+1
      lherma=.false.
      lherm0=.false.
      lhermuncor=.false.
      lrealuncor=.true.
      if (nham .eq. 1) then
         buffer = '********* System Hamiltonian ***********'
      else
         ilbl=index(hamlab(nham),' ')-1
         buffer = '********* '//hamlab(nham)(1:ilbl)// &
              ' Hamiltonian ***********'
      endif
      if (hamdim .gt. 1) write(iolog,'(/,a80)') buffer
      if (kzahl(nham) .gt. 0) then
         write(iolog,*)
         write(iolog,'(a,/,a,/,a)') 'Correlated Terms  ', &
              '    ki,kf: initial and final el. state.',&
              '    Operator numbers are h=hamilton(f,k))'
      endif
      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
      call logkterms(iolog,k1,k2,lhamr,lhamt)
           
      if(iLMR .gt. 0) then
         write(iolog,'(/a)') 'LMR terms present'
         write(iolog,'(5x,a)') 'iLMR      R       M       L'
         do i=1,iLMR
            write(iolog,'(4i8)') i, (hamLMR(i,n),n=1,3)
         enddo
       endif

      if (nmulpot(nham) .gt. 0) then
         write(iolog,'(/a)') 'Multi-D terms present'
         write(iolog,'(a)') '     k  ksi ksf coeff   No.s'
         do n=1,nmulpot(nham)
            k=zham(nham)+kzahl(nham)-1+n
            f=fterm(k)
            write(iolog,'(i6,2i4,i5,3x,i5)')  &
                 k,ksf(k),ksi(k),hamilton(fkoe,k),hamilton(f,k)
         enddo
      endif
            
      if (any(khzahl(:,:,nham) .gt. 0)) then
         luncorr = .true.
         write(iolog,*)
         if (nmode .eq. 1) then
            write(iolog,'(a)') 'Terms'
         else
            write(iolog,'(a)') 'Uncorrelated Terms'
         endif
         if (dentype .gt. 0) then
            if (nstate .gt. 1) then
               write(iolog,'(5x,a)') 'k  kt act  m   s  sym KE/PE   K     Operator'
            else
               write(iolog,'(5x,a)') 'k  kt act  m  sym KE/PE   K     Operator'
            endif
         else
            if (nstate .gt. 1) then
               write(iolog,'(5x,a)') 'k   m   s  sym KE/PE   K     Operator'
            else
               write(iolog,'(5x,a)') 'k   m  sym KE/PE   K     Operator'
            endif
         endif
      endif

      do m=1,nopmode
         do s=1,nstate
            kl=zhun(m,s,nham)
            kh=zhun(m,s,nham)+khzahl(m,s,nham)-1
            do k=kl,kh
               if (kinop(m,k) .gt. 0) then
                  pestr = ' KE '
               else
                  pestr = ' PE '
               endif
               buffer=' '
               ilbl=1
               imag=hsym(mkoe,k)
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  h=hamilton(f,k)
                  if (h.le.0)  cycle   ! LMR
                  if(htmtype(h).eq.3 .or. htmtype(h).eq.6  .or. &
                     htmtype(h).eq.9 .or. htmtype(h).eq.13 .or. &
                     htmtype(h).eq.108) then
                      imag=-imag
                  endif
                  if(htmtype(h).eq.4 .or. htmtype(h).eq.7  .or. &
                     htmtype(h).eq.10.or. htmtype(h).eq.14) then
                      imag=0 
                  endif
                  jlbl=ilbl+4
                  write(buffer(ilbl:jlbl),'(i5)') h
                  ilbl=ilbl+5
               enddo

               if (dentype .gt. 0) then
                  if (nstate .gt. 1) then
                     write(iolog,'(i6,5i4,2x,a4,i5,3x,a,2x,a)') &
                       k,ktwin(k),kact(k),m,s,hsym(m,k)*hsym(mkoe,k),&
                       pestr,hamilton(fkoe,k),bff(imag+2),buffer
                  else
                     write(iolog,'(i6,4i4,2x,a4,i5,3x,a,2x,a)') &
                       k,ktwin(k),kact(k),m,hsym(m,k)*hsym(mkoe,k),&
                       pestr,hamilton(fkoe,k),bff(imag+2),buffer
                  endif
               else
                  if (nstate .gt. 1) then
                     write(iolog,'(i6,3i4,2x,a4,i5,3x,a,2x,a)') &
                       k,m,s,hsym(m,k)*hsym(mkoe,k),pestr,hamilton(fkoe,k),&
                       bff(imag+2),buffer
                  else
                     write(iolog,'(i6,2i4,2x,a4,i5,3x,a,2x,a)') &
                       k,m,hsym(m,k)*hsym(mkoe,k),pestr,hamilton(fkoe,k),&
                       bff(imag+2),buffer
                  endif
               endif
               if(hsym(m,k)*hsym(mkoe,k) .ne. 1) lhermuncor=.true.
               if(imag .ne. 1) lrealuncor=.false.
            enddo
         enddo
      enddo
      if(luncorr) then
         if(lhermuncor .and. .not.lrealuncor)  write(iolog,'(a)') &
      'The sum of uncorrelated operators is not real and not Hermitian!'
         if(lhermuncor .and. lrealuncor)  write(iolog,'(a)') &
      'The sum of uncorrelated operators is real but not Hermitian!'
         if(.not.lhermuncor .and. .not.lrealuncor)  write(iolog,'(a)') &
      'The sum of uncorrelated operators is not real but Hermitian!'
         if(.not.lhermuncor .and. lrealuncor)  write(iolog,'(a)') &
      'The sum of uncorrelated operators is real and Hermitian!'
      endif


      ! terms used by MLOps
      ! Can only write this information after generating the operator,
      ! not if reading an oper file.
      if (lmlop) then
         if (irunoper .eq. 1) then
            call mlop_ham_log(iolog,nham)
         else
            write(iolog,'(/,a,/)') "**** Multi-Layer Operators ****"
            write(iolog,'(a,/,a)') &
             "For details on MLops: See op.log file for calculation in ",&
             "                      which operator was set up."
         endif
      endif

!-----------------------------------------------------------------------
! write out any non-hermitian operators
!-----------------------------------------------------------------------
      lfirst=.true.
      lherm=.false.
      hsymtot=0
      ilbl=1
      jlbl=0

      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
      do k=k1,k2
         do m=1,nmode+1
            lherm=.true.
            hsymtot=1
            if (hsym(m,k) .ne. 1) then
               if (lfirst) then
                  write(iolog,*)
                  write(iolog,'(a)') '**** Non-hermitian operators ****'
                  write(iolog,'(a)') '     Hermitian      =  1 '
                  write(iolog,'(a)') '     Anti-Hermitian = -1 '
                  write(iolog,'(a)') '     No Symmetry    =  0 '
                  write(iolog,*)
                  write(iolog,'(5x,2a)')'k  ki  kf coeff     hsym(m,k)', &
                                     '  (Last column is sym of coeff)'
                  lfirst=.false.
               endif
               lherm=.false.
               lhermm=.false.
            endif
            hsymtot=hsymtot*hsym(m,k)
         enddo
         if(hsymtot.eq.-1) lherma=.true.
         if(hsymtot.eq. 0) lherm0=.true.
         if (.not. lherm) then
            buffer=' '
            ilbl=1
            i=1
            do m=1,nmode+1
               jlbl=ilbl+4
               write(buffer(ilbl:jlbl),'(i5)') hsym(m,k)
               ilbl=ilbl+5
            enddo

            write(iolog,'(i6,2i4,i5,3x,a)') &
              k,kf(k),ki(k),hamilton(fkoe,k),buffer(1:min(jlbl,50))
            if (jlbl.gt. 50) write(iolog,'(22x,a)') &
                      buffer( 51:min(jlbl,100))
            if (jlbl.gt.100) write(iolog,'(22x,a)') &
                      buffer(101:min(jlbl,150))
            if (jlbl.gt.150) write(iolog,'(22x,a)') &
                      buffer(151:min(jlbl,200))
            if (jlbl.gt.200) write(iolog,'(22x,a)') &
                      buffer(201:min(jlbl,250))
         endif
      enddo
      call myflush(iolog)

      write(iolog,*)
      if(nham.eq.1 .and. lcap) write(iolog,'(a)') 'lcap is set'

      if(kzahl(nham).eq.0) then
         write(iolog,'(2a/2a)') 'The correlated operator in zero.'
      else
      if(lhamt) then
         write(iolog,'(2a/2a)') &
               'The correlated operator contains special operator-', &
               'terms (e.g. KLeg, Wigner) which may not be real.', &
               'The correlated operator without these terms is ', &
               'caracterized as follos:'
      endif
      if(lhamr) then
         if(.not.lherma .and. .not.lherm0) then
            write(iolog,'(a)') &
               'The correlated operator is REAL and HERMITIAN'

         else ! lherm)
            if(lherm0) then
               write(iolog,'(2a,/2a)') &
                'The correlated operator is REAL and contains terms ', &
                'of unspecified symmetry','(which may or may not add ', &
                'up to a hermitian sum)'

            elseif(lherma) then
               write(iolog,'(2a,/2a)') &
                'The operator is REAL and contains anti-hermitian ', &
                'terms','(which are unlikely to add up to a ', &
                'hermitian sum)'
            else
               write(iolog,'(a)') &
                'The correlated operator is REAL but NON-HERMITIAN'
            endif  ! lherm0
         endif  ! lherm
      else  ! lhamr
         if(.not.lherma .and. .not.lherm0) then
            write(iolog,'(2a/2a)') &
               'The corelated operator is HERMITIAN but contains ', &
               'complex terms','(e.g. contains "p" or "I"). ', &
               'This operator must not be used for RRDAV.'

         else ! lherm
            if(lherm0) then
               write(iolog,'(2a,/2a)') &
                'The correlated operator is NON-HERMITIAN ', &
                'and contains terms of','unspecified symmetry. ', &
                'Some terms are complex.'

            elseif(lherma) then
               write(iolog,'(2a,/2a)') &
                'The correlated operator contains complex and ', &
                'anti-hermitian terms','(which are unlikely to ', &
                'add up to a hermitian sum)'
            else
               write(iolog,'(a)') &
                'The correlated operator is not REAL and NON-HERMITIAN'
            endif ! lherm0
         endif ! lherm
      endif  !   lhamr
      endif  !  (kzahl(nham).eq.0)

      if(luncorr) then
         if(lhermuncor .and. .not.lrealuncor)  write(iolog,'(a)') &
      'The uncorrelated operator is not real and not Hermitian!'
         if(lhermuncor .and. lrealuncor)  write(iolog,'(a)') &
      'The uncorrelated operator is real but not Hermitian!'
         if(.not.lhermuncor .and. .not.lrealuncor)  write(iolog,'(a)') &
      'The uncorrelated operator is not real but Hermitian!'
         if(.not.lhermuncor .and. lrealuncor)  write(iolog,'(a)') &
      'The uncorrelated operator is real and Hermitian!'
      endif

      if(iLMR.gt.0) then
         write(iolog,'(a,/2a)') 'Warning:','LMR terms are present', &
          ' and the characterization above may not be meaningful.'
      endif

      write(iolog,'(/78a1/)') ('-',i=1,78)

      if (nham .lt. nhamint0-1) go to 10   ! Loop over user defined Hamiltonians
      if (nham .lt. hamdim .and. loplogfull) go to 10   ! Loop over Hamiltonians

!-----------------------------------------------------------------------
!  Write information about auxiliary multi-layer operators.
!  Cannot do this if reading an oper file.
!-----------------------------------------------------------------------
      if (irunoper .eq. 1) then
         call mlop_aux_log(iolog)
      else
         write(iolog,'(/,a,/)') "**** Auxiliary MLOps ****"
         write(iolog,'(a,/,a)') &
          "For details on MLops: See op.log file for calculation in ",&
          "                      which operator was set up."
      endif

      if (khtot(1) .eq. 0) then
         routine = 'Logop (logopmod.f90)'
         message = 'No terms in system Hamiltonian'
         call errormsg
      endif

!-----close op.log file
      if(iolog.ne.6 .and. iolog.ne.ilog) close(iolog)
      call myflush(ilog)

      return
 35   routine='eingutil.F openlog'
      message='Cannot open op.log file!'
      call errormsg

      end subroutine logop

!#######################################################################

      subroutine buffterm(h,string,strlen,length)

      implicit none

      integer(long)                 :: n,h1,ilbl,jlbl,klbl
      integer(long), intent(in)     :: h,strlen
      integer(long), intent(out)    :: length
      character(len=*), intent(out) :: string
      character(len=c4)             :: buffer

      string='  '
      ilbl=1
      jlbl=0
 
      do n=1,htmfac(h)
         buffer=' '
         if (hterm(n,h) .gt. 0) then
            h1=hterm(n,h)
!........Supress printing of npot lines (if lwrnpot=.false.)
            if(.not.lwrnpot .and. (hoplab(h1)(1:4).eq.'npot')) then
               if(hoplab(h1)(1:6).eq.'npot1,') then
                  continue
               elseif(h1 .eq. hopdim .and. hoplab(h1)(1:4).eq.'npot') then
                  write(iolog,'(a,32x,a)') '   ...  ......','.......'
               elseif(hoplab(h1+1)(1:4).ne.'npot') then
                  write(iolog,'(2a)')  '   ...  ......',&
                   '  Set "print-npot" to see all  ......'

               else
                  length=0
                  return
               endif
            endif

!........Supress printing of cpd lines (if lwrnpot=.false.)
            if(.not.lwrnpot .and. (hoplab(h1)(1:3).eq.'cpd')) then
               if(hoplab(h1)(1:6).eq.'cpdtm1,') then
                  continue
               elseif(hoplab(h1+1)(1:5).ne.'cpdtm') then
                  write(iolog,'(2a)')  '   ...  ......', &
                   '  Set "print-npot" to see all  ......'
               else
                  length=0
                  return
               endif
            endif

!........Supress printing of hcpd lines (if lwrnpot=.false.)
            if(.not.lwrnpot .and. (hoplab(h1)(1:4).eq.'hcpd')) then
               if(hoplab(h1)(1:7).eq.'hcpdtm1,') then
                  continue
               elseif(hoplab(h1+1)(1:6).ne.'hcpdtm') then
                  write(iolog,'(2a)')  '   ...  ......', &
                   '  Set "print-npot" to see all  ......'
               else
                  length=0
                  return
               endif
            endif

            buffer=hoplab(h1)
            klbl=index(buffer,' ')-1
            jlbl=ilbl+klbl
            if(jlbl.le.strlen) &
            string(ilbl:jlbl)=buffer(1:klbl)//'*'
            ilbl=jlbl+1
         else if (hterm(n,h) .eq. 0) then
            buffer=' + '
            ilbl=ilbl-1
            jlbl=ilbl+2
            if(jlbl.le.strlen) & 
            string(ilbl:jlbl)=buffer(1:3)
            ilbl=jlbl+1
         endif
      enddo
      if(jlbl.le.strlen) string(jlbl:jlbl)=' '

      length=jlbl-1
      if (length+10 .gt. strlen) then
         routine='Buffterm (genoper/logop.F)'
         write(message,'(a,i7)')'Increase STRLEN. Minimum :',length+20
         call warnmsg1
      endif

      return
      end subroutine buffterm

!#######################################################################

      subroutine buffmdtm(h,string,strlen,length)

      implicit none

      integer(long)                              :: f,n,h1,ilbl,jlbl,nh,&
                                                    m,muld,mode
      integer(long), dimension(maxdim)           :: klbl
      integer(long), intent(in)                  :: h,strlen
      integer(long), intent(out)                 :: length
      character(len=*), intent(out)              :: string
      character(len=c5), dimension(maxdim)       :: buffer
      logical(kind=4), dimension(maxdim)         :: found

      string='  '
      length = 0
      ilbl=1
      jlbl=0

      call zerovxi(klbl,maxdim)
      length = 1
!      if (lanalpes .and. (.not. lgwp .and. .not. ltraj)) then
!         m=1
!      else
         m=htmmode(h)
         if(m.eq.0) then
            write(iolog,'(a)')  &
                 'WARNING in Buffmdtm: m = 0 for mode operator!'
            return
         endif
!      endif

      call zerovxl(found,maxdim)
      do nh=1,htmfac(h)+1
         h1=hterm(nh,h)
         
         if (h1 .gt. 0) then
            muld=htmmuld(h1)
            mode=htmmode(h1)
            if (muld .ne. 0) then
               f=mulddof(1,muld)
            else if (mode .ne. 0) then
               if (htmadof(h1).ne.0) then
                  f=htmadof(h1)
               else
                  f=spfdof(1,mode)
               endif
            else
               f=htmdof(h1)
            endif
            write(buffer(f),'(i5)') h1
            call rmblnk(buffer(f))
            klbl(f)=index(buffer(f),' ')-1
            found(f) = .true.
         else if (h1 .eq. 0) then
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (found(f)) then
                  jlbl=ilbl+klbl(f)+2
                  length=length+klbl(f)+2
                  jlbl=min(jlbl,strlen)
                  if(jlbl .le. strlen) &
                     string(ilbl:jlbl)='('//buffer(f)(1:klbl(f))//')*'
                  ilbl=jlbl+1
                  ilbl=min(ilbl,strlen)
               endif
            enddo
            call zerovxl(found,ndof)
            
            if (klbl(fkoe) .gt. 0 .and. m .ne. mkoe) then
               f=fkoe
               jlbl=ilbl+klbl(f)
               length=length+klbl(f)+2
               jlbl=min(jlbl,strlen)
               if(jlbl .le. strlen)&
                  string(ilbl:jlbl)=buffer(f)(1:klbl(f))//'*'
               ilbl=jlbl+1
               ilbl=min(ilbl,strlen)
            endif
           
            if (nh .lt. htmfac(h)) then
               ilbl=ilbl-1
               jlbl=ilbl+2
               length=length+2
               if(jlbl .le. strlen) string(ilbl:jlbl)=' + '
               ilbl=jlbl+1
               ilbl=min(ilbl,strlen)
               call zerovxi(klbl,maxdim)
            else
               if(jlbl .le. strlen) string(jlbl:jlbl)=' '
            endif
         endif
      enddo
      length=jlbl-1

      if (length+10 .gt. strlen) then
         routine='Buffmdtm  (genoper/logop.F)'
         write(message,'(a,i7)')'Increase STRLEN. Minimum :',length+20
         call warnmsg1
      endif
      length=jlbl-1

      return
      end subroutine buffmdtm

!#######################################################################

      subroutine breakstr(string,buffer,n,length,lenbuf)

      implicit none

      integer(long)                 :: ilbl,jlbl,klbl,i,j
      integer(long), intent(in)     :: length
      integer(long), intent(inout)  :: n
      integer(long), intent(inout)  :: lenbuf
      character(len=*), intent(in)  :: string
      character(len=*), intent(out) :: buffer

      buffer=' '
      ilbl=n
      jlbl=0

      if ((length-ilbl+1) .lt. lenbuf) jlbl=length-ilbl+1

      if (jlbl .eq. 0) then
         do i=lenbuf,1,-1
            j=ilbl+i-1
            if (string(j:j) .eq. '+') then
               if (jlbl .eq. 0) jlbl=i
            endif
         enddo
      endif
      if (jlbl .eq. 0) then
         do i=lenbuf,1,-1
            j=ilbl+i-1
            if (string(j:j) .eq. '*') then
               if (jlbl .eq. 0) jlbl=i
            endif
         enddo
      endif

      if (jlbl .eq. 0) jlbl=lenbuf

      klbl=ilbl+jlbl-1
      buffer(1:jlbl)=string(ilbl:klbl)
      n=klbl+1
      lenbuf=jlbl

      return
      end subroutine breakstr

!#######################################################################

      subroutine outhtm(h,unit)

      implicit none

      integer(long), intent(in)   :: h,unit
      integer(long), parameter    :: strlen=12800

      integer(long)               :: ns,length,lenbuf,j
      character(len=80)           :: buffer
      character(len=strlen)       :: string


      if (htmform(h) .eq. 1) then
         call buffmdtm(h,string,strlen,length)
      else
         call buffterm(h,string,strlen,length)
      endif
      if(length.eq.0) return
      j=0
      ns=1
      do while(ns.le.length)
         j=j+1
         lenbuf=48
   
         call breakstr(string,buffer,ns,length,lenbuf)
  
         if (j .eq. 1) then
            if (htmdof(h) .eq. fkoe) then
               write(unit,'(i6,a4,i8,3i4,4x,a)') h,'   K',htmmode(h), &
                    htmmuld(h),htmtype(h),htmsym(h),buffer(1:lenbuf)
            else if (htmdof(h) .eq. ftime) then
               write(unit,'(i6,a4,i8,3i4,4x,a)') h,'   T',htmmode(h), &
                    htmmuld(h),htmtype(h),htmsym(h),buffer(1:lenbuf)
            else if (htmmode(h) .eq. mkoe) then
               if (any(spfdof(1:nspfdof(mkoe),mkoe) .eq. ftime)) then
                  write(unit,'(i6,2i4,a4,3i4,4x,a)') h,htmdof(h), &
                       htmadof(h),'   T',htmmuld(h),htmtype(h),htmsym(h), &
                       buffer(1:lenbuf)
               else
                  write(unit,'(i6,2i4,a4,3i4,4x,a)') h,htmdof(h), &
                       htmadof(h),'   K',htmmuld(h),htmtype(h),htmsym(h), &
                       buffer(1:lenbuf)
               endif
            else
               write(unit,'(i6,6i4,4x,a)') h,htmdof(h),htmadof(h), &
                    htmmode(h),htmmuld(h),htmtype(h),htmsym(h), &
                    buffer(1:lenbuf)
            endif
         else
            write(unit,'(29x,a)') buffer(1:lenbuf)
         endif
      enddo


      return
      end subroutine outhtm

!#######################################################################

      subroutine logkterms(lun,kl,kh,lhamr,lhamt)

      implicit none

      integer(long), intent(in) :: lun,kl,kh
      integer(long)             :: k,ilbl,jlbl,m,n,f,i,h,nskip,&
                                   i1,i2,imag
      logical(kind=4)           :: lhamr,lhamt
      integer(long)             :: buffl
      character(len=8*maxdim+16)  :: buffer
      character(len = 4)        :: pestr

      lhamr=.true.
      lhamt=.false.
      if (kh .lt. kl) return

      jlbl=0
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            ilbl=jlbl+1 
            jlbl=index(modelabel(f),' ')-1
            i=min(jlbl,6)
            if(i.lt.6) then
               buffer(ilbl:ilbl+5-i)='    '
               ilbl=ilbl+6-i
            endif
            buffer(ilbl:ilbl+i-1)=modelabel(f)(1:i)
            ilbl=ilbl+i
            jlbl=ilbl+1
            buffer(ilbl:jlbl)=', '
            if(n.eq.nspfdof(m)) buffer(ilbl:jlbl)=' |'
         enddo
      enddo
      buffl = jlbl-1     ! remove final |
      if (dentype .gt. 0) then
         write(lun,'(20x,2a)') 'dof order =   K  | ', &
              buffer(1:min(jlbl,128))
         i1=1
         i2=128
         do i=1,int((buffl-8)/128)
            i1=i1+128
            i2=i2+128
            if (jlbl .ge. i1) write(lun,'(32x,a)') buffer(i1:min(jlbl,i2))
         enddo
         write(lun,'(5x,a)') &
               'k  kt act  ki  kf KE/PE   '
      else
         write(lun,'(9x,2a)') 'dof order =   K  | ', &
              buffer(1:min(jlbl,136))
         i1=1
         i2=136
         do i=1,int((buffl-8)/136)
            i1=i1+136
            i2=i2+136
            if (jlbl .ge. i1) write(lun,'(24x,a)') buffer(i1:min(jlbl,i2))
         enddo
         write(lun,'(5x,a)') 'k  ki  kf KE/PE  '
      endif
 
      do k=kl,kh
         buffer=' '
         ilbl=1
         i=1
         if(k-kl.gt.40 .and. .not.lwrnpot) then
            write(iolog,'(2a,/a,i5,a)')  '   ...  ......', &
             '  Set "print-npot" to see all   ...       ...', &
             '  Information on',kh-kl-40,' k-terms is suppressed'
            goto 35
         endif

         do i1=1,buffl
            write(buffer(i1:i1),'(a1)') ' '
         enddo
         ilbl=1
         i=1
         imag=hsym(mkoe,k)

         do m=1,nmode
            nskip=0
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               h=max(1,hamilton(f,k)) ! LMR
               if(htmtype(h).le.2  .or. htmtype(h).eq.5  .or. &
                  htmtype(h).eq.8  .or. htmtype(h).eq.11 .or. &
                  htmtype(h).eq.12 .or. htmtype(h).eq.15 .or. &
                  htmtype(h).eq.17 .or. htmtype(h).eq.31 .or. &
                  htmtype(h).eq.32 .or. htmtype(h).eq.40) then
                  continue ! above operators are real, do nothing.
               elseif(htmtype(h).eq.3  .or. htmtype(h).eq.6  .or. &
                  htmtype(h).eq.9  .or. htmtype(h).eq.13) then
                  imag=-imag
               elseif(htmtype(h).eq.16 .or. htmtype(h).eq.30 .or. &
                  (htmtype(h).ge.18 .and. htmtype(h).le.23) .or. &
                  (htmtype(h).ge.41 .and. htmtype(h).le.80)) then
                  lhamt=.true.
               else
                  lhamr=.false.
               endif

               if (nskip.gt.0) then
                  nskip=nskip-1
               else
                  jlbl=ilbl+7
                  write(buffer(ilbl:jlbl),'(i8)') hamilton(f,k)  ! LMR
                  if (htmmode(h).ne.0) then
                     if (htmtype(h).ge.15 .and. htmtype(h).le.24) then
!                       2D types: skip K dof
!                       FO 12/2007: skip only if operator on K is unity
                        if (hamilton(f+1,k).eq.(f+1)) then
                           nskip=1
                        else
                           nskip=0
                        endif
                     else if (htmtype(h).eq.30) then
!                       4D types: skip K, KLeg, K
                        nskip=3
                     else if (htmtype(h).ge.40 .and. htmtype(h).le.69) then
                        nskip=2 ! Skip k/m for Wigners
                     else
!                       skip remaining DOFs
                        if (htmndof(h).eq.0) then
                           nskip=nspfdof(m)-n
                        else
                           nskip=htmndof(h)-1
                        endif
                     endif
                  endif
               endif  ! (nskip.gt.0)
               ilbl=ilbl+8
            enddo  !  n=1,nspfdof
         enddo     !  m=1,nmode
         if(imag.ne.1) lhamr=.false.  ! There is an (uncanceled) imaginary operator.

         if (any(kinop(1:nmode,k) .gt. 0)) then
            if (kact(k) .ne. 1) then
! dissipative terms
               pestr = ' DE '
            else
               pestr = ' KE '
            endif
         else
            pestr = ' PE '
         endif
         if (dentype .gt. 0) then
            write(lun,'(i6,4i4,1x,a4,i5,a)')  &
              k,ktwin(k),kact(k),ki(k),kf(k),pestr,hamilton(fkoe,k), &
              buffer(1:min(jlbl,128))
            i1=1
            i2=128
            do i=1,int((buffl-8)/128)
               i1=i1+128
               i2=i2+128
               if (jlbl .ge. i1) write(lun,'(30x,a)') buffer(i1:min(jlbl,i2))
            enddo
            if (jlbl .gt. 128) write(lun,'(a)')
         else
            write(lun,'(i6,2i4,1x,a4,i5,a)')  &
              k,ki(k),kf(k),pestr,hamilton(fkoe,k), &
              buffer(1:min(jlbl,136))
            i1=1
            i2=136
            do i=1,int((buffl-8)/136)
               i1=i1+136
               i2=i2+136
               if (jlbl .ge. i1) write(lun,'(24x,a)') buffer(i1:min(jlbl,i2))
            enddo
            if (jlbl .gt. 136) write(lun,'(a)')

         endif
         if (jlbl .ge. buffl) then
            routine='Logop (genoper/logop.F)'
            message='Redimension buffer (parameter buffl) for more dofs'
         endif
         
      enddo

 35   continue

      return
      end subroutine logkterms

!#######################################################################
! Check for Errors and Consistency of muld potentials.
!#######################################################################

      subroutine chkmuld(nham,h,muld)

      implicit none

      integer(long)             ::  hl,n,i2,i
      integer(long), external   ::  slen
      integer(long), intent(in) ::  nham,h,muld

      if(nodiag(nham) .and. vdmuld(muld)*ndmuld(muld).ne.1) then
         write(ilog,'(/2a)') 'ERROR, muld-term: ',hoplab(hterm(1,h))
         write(ilog,'(a,i3,2x,a)') 'nham =',nham,hamlab(nham)
         write(6,'(/2a)') 'ERROR, muld-term: ',hoplab(hterm(1,h))
         write(6,'(a,i3,2x,a)') 'nham =',nham,hamlab(nham)
         routine='chkmuld  (genoper/logop.F, line 1289)'
         message="A muld potential must be of usediag "// &
                 "type, or it must span over all DOFs."
         call errormsg
      endif

! --- Check that the muld potential is not multiplied with a factor
! --- except a constant. The summation of muld potentials is allowed.
      if(htmfac(h).ne.1) then
         i2=0
         do i=1,htmfac(h)
            hl=hterm(i,h)
            if(hl.eq.0) then  ! Terms are summed. Start counting anew.
               i2=0
            else
               if(hopfile(hl).ge.201 .and. hopfile(hl).lt.300) then
                  i2=i2+1     ! A muld-potential
               elseif(hopfile(hl).eq.301 ) then
                  continue    ! This is a constant, OK
               elseif(hopfile(hl).eq.302 .and. hopilab(hl).le.2) then
                  continue    ! This is a "1" or a "-1", OK
               else
                  i2=i2+2     ! Not a constant, not a muld-potential
               endif
            endif
            if(i2.gt.1) then
              write(ilog,'(/2a)')'ERROR, muld-term: ',hoplab(hterm(1,h))
              write(ilog,'(a,i3,a,9(2x,a))') 'htmfac =',  htmfac(h), &
                   ',   hterm :', (hoplab(hterm(n,h)) &
                   (1:slen(hoplab(hterm(n,h)))),n=1,htmfac(h))
              write(6,'(/2a)')'ERROR, muld-term: ',hoplab(hterm(1,h))
              write(6,'(a,i3,a,9(2x,a))') 'htmfac =',  htmfac(h), &
                   ',   hterm :', (hoplab(hterm(n,h)) &
                   (1:slen(hoplab(hterm(n,h)))),n=1,htmfac(h))
              routine='chkmuld'
              message="A muld potential must not be multiplied "// &
                      "with another factor (except a coefficient)."
              call errormsg
            endif
         enddo
      endif

! --- Check that the muld potential is not electronically off-diagonal.
!     if(ki(k).ne.kf(k) .and. vdmuld(muld)*ndmuld(muld).ne.1) then
!        write(ilog,'(/2a)') 'ERROR, muld-term: ',hoplab(hterm(1,h))
!        write(ilog,'(a,i3,a,i4,a,i4,a,2i4)') &
!        'muld =',muld,',  k =',k,',  h =',h,', ki,kf =',ki(k),kf(k)
!        write(6,'(/2a)') 'ERROR, muld-term: ',hoplab(hterm(1,h))
!        write(6,'(a,i3,a,i4,a,i4,a,2i4)') &
!        'muld =',muld,',  k =',k,',  h =',h,', ki,kf =',ki(k),kf(k)
!        routine='chkmuld'
!        message="A muld potential must not be electronically "// &
!                "off-diagonal, except it spans over all DOFs."
!        call errormsg
!     endif

! --- Check that the muld potential is not multiplied with an operator.
!     do f=1,ndof
!        if( (hamilton(f,k).ne.h) .and. (hamilton(f,k).ne.f) &
!             .and. (.not.lmult .or. (f.ne.feb)) ) then
!           write(ilog,'(/2a)') 'ERROR, muld-term: ',hoplab(hterm(1,h))
!           write(ilog,'(a,20i4)') 'hamilton:',(hamilton(m,k),m=1,ndof)
!           if(.not.lmult .and. feb.gt.0)  write(ilog,'(2a,/a)') &
!                'Use "multi-set" if the electronic operator is ', &
!                'the only one','which multiplies the muld potential.'
!           write(6,'(/2a)') 'ERROR, muld-term: ',hoplab(hterm(1,h))
!           write(6,'(a,20i4)') 'hamilton:',(hamilton(m,k),m=1,ndof)
!           if(.not.lmult .and. feb.gt.0)  write(6,'(2a,/a)') &
!                'Use "multi-set" if the electronic operator is ', &
!                'the only one','which multiplies the muld potential.'
!           routine='chkmuld'
!           message="No operators (except '1') must be "// &
!                   "multiplied with a muld potential!"
!           call errormsg
!        endif
!     enddo

      return
      end subroutine chkmuld

!-----------------------------------------------------------------------
subroutine mlop_ham_log(lun,nham)
!-----------------------------------------------------------------------
   integer,intent(in)    :: lun,nham
   type(mlham_t),pointer :: mlham
   integer,pointer       :: immlops(:)
   integer               :: n,immlop
   mlham => mlham_get(nham)
   if (.not.associated(mlham)) return
   write(lun,'(/,a,/)') "**** Multi-Layer Operators ****"
   immlops => llist_to_array(mlham%immlopq)
   do n=1,mlham%nmlop
      immlop = immlops(n)
      call mlop_log(lun,immlop)
   enddo
   deallocate(immlops)
end subroutine mlop_ham_log


!-----------------------------------------------------------------------
subroutine mlop_aux_log(lun)
!-----------------------------------------------------------------------
   integer,intent(in) :: lun
   integer            :: n
   if (nmlop==0) return
   write(lun,'(/,a,/)') "**** Auxiliary MLOps ****"
   do n=1,nmlop
      if (associated(mmlops(n)%ifacq)) cycle
      call mlop_log(lun,n)
   enddo
end subroutine mlop_aux_log


!-----------------------------------------------------------------------
subroutine mlop_log(lun,immlop)
!-----------------------------------------------------------------------
   integer,intent(in)        :: lun
   integer,intent(in)        :: immlop
   type(optree_t),pointer    :: mlop
   integer                   :: l,r,k1,nk
   type(opnode_t),pointer    :: leaf
   integer,pointer           :: ifacs(:)
   type(llist_int_t),pointer :: ifacq
   logical                   :: lhamr,lhamt
   ifacq => mmlops(immlop)%ifacq
   if (associated(ifacq)) then
      write(lun,'(a,i0,a)',advance="no") "MLOp ",immlop," = product of MLOps "
      ifacs => llist_to_array(mmlops(immlop)%ifacq)
      do r=1,size(ifacs)-1
         write(lun,'(i0,a)',advance="no") ifacs(r),", "
      enddo
      write(lun,'(i0)') ifacs(size(ifacs))
      deallocate(ifacs)
   elseif (mmlops(immlop)%mlpfnr /= 0) then
      write(lun,'(a,i0,a,i0)') "MLOp ",immlop," was created from MLPF ",mmlops(immlop)%mlpfnr
   else
      write(lun,'(a,i0,a)') "MLOp ",immlop," was created from a single mode operator"
   endif
   write(lun,*)
   mlop => mmlops(immlop)%mlop
   call optree2txt(lun,mlop,"Tree:   ")
   write(lun,*)
   k1 = 0
   nk = 0
   do l=1,mlop%numleaves
      leaf => mlop%leaves(l)%p
      nk = max(nk,leaf%nops)
      k1 = leaf%ks(1) ! same for all leaves
   enddo
   call logkterms(lun,k1,k1-1+nk,lhamr,lhamt)
   write(lun,*)
   write(lun,*)
   call flush(lun)
end subroutine mlop_log

!=======================================================================

      end module logopmod

