!#######################################################################
!
!              MCTDH  module MATCHLAB
!
! MCTDH module to read and sort the data specifying the Hamiltonian
!
!            Subroutines: matchlab npotlab 
!
! Matches the externally defined terms to those in the Hamiltonian.
! Expands potential terms. The actual reading of the files is done
! in subroutine hintern.
!
! Called from: heingabe
!
! GW
!
!#######################################################################

      module matchlabmod

use decimal, only: long, dop
use logdat
use op1lib,  only: zerovxi, zerovxl, cpvxl, cpvxi, zerovxd, zeromxi, cpvxd

use maxv
use global
use dvrdatmod
use griddatmod
use operdef
use runopermod
use paths
use channels
use combined
use rdnatpot

use iodvrdef, only: rddvrdef
use heintermmod, only: storetm,heinterms
use heinutil, only: muldnum2,rdfnum,hlabnum,htmnum
use rdopermod, only: assocdof
use logopmod, only: logop
use operutil, only: getvgn,mvkend

use opermod_interface
use genmlham, only: mlpf_to_mlop,mlpf_modeinfo

      implicit none

      private
      public :: matchlab,findlabels

      contains

!#######################################################################

      subroutine matchlab(loperr)

      implicit none

      integer(long)       :: n,k,f,h,h1,f1,k1,ktot1,hl,nr,m,muld,&
                             ilbl,jlbl,klbl,np,ierr,nh,&
                             htime,chkdvr
      integer(long)       :: nwarn=0
      integer(long)       :: maxrdf0

      integer(long), external                :: slen
      integer(long), dimension(1)            :: optyp
      integer(long), dimension(maxdim)       :: fdvr,mlpfnr
      character(len=c2)                      :: label,buffer
      character(len=c5)                      :: filename
      logical(kind=4)                        :: lfile,ignore,lmlpf
      logical(kind=4), intent(inout)         :: loperr
      logical(kind=4), dimension(maxdef)     :: found
      logical(kind=4), dimension(maxkoe)     :: delete
      logical(kind=4), dimension(maxrdf)     :: lfast1
      real(dop), dimension(maxrdf)           :: npotcut1

      if (ndef .eq. 0) return
      call zerovxl(delete,maxkoe)
! maxrdf changes in rdfnum, so we need to keep track of array dimensions
      maxrdf0=maxrdf

!-----------------------------------------------------------------------
! match labels and substitute
! retain any power on label
!-----------------------------------------------------------------------
      do n=1,min(ndef,maxdef)
         found(n)=.false.
         do hl=1,min(hopdim,maxhop)
            ilbl=index(hoplab(hl),'^')-1
            jlbl=index(hoplab(hl),' ')-1
            if (ilbl .gt. 0) then
               buffer=hoplab(hl)(ilbl+1:jlbl)
               jlbl=jlbl-ilbl
            else
               buffer=' '
               ilbl=jlbl
               jlbl=1
            endif
            if (hoplab(hl)(1:ilbl) .eq. deflabel(n)) then
               klbl=index(deflabop(n),' ')-1
               hoplab(hl)=deflabop(n)(1:klbl)//buffer(1:jlbl)
               found(n)=.true.
               hopidef(hl)=n
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! if term is to be added, add
!-----------------------------------------------------------------------
      optyp(1)=3
      if (.not.(ldd .and. ((leb .and. nmode .gt. 2) .or. &
               (.not.leb .and. nmode .gt. 1)))) then
         do n=1,min(ndef,maxdef)
            if (.not. found(n)) then
               ilbl=index(deflabel(n),'_')
               if (ilbl .gt. 0) then
                  ilbl=ilbl+1
                  jlbl=index(deflabel(n),' ')-1
                  label=deflabel(n)(ilbl:jlbl)
                  f=0
                  do f1=1,ndof
                     if (label .eq. modelabel(f1)) f=f1
                  enddo
                  if (f .eq. 0) then
                     nwarn=nwarn+1
                     if(nwarn.le.10) then
                        write(6,'(/,2a,/a,/2a)') 'The underscore is a ', &
                             'special character used to specify a dof.', &
                             'E.g.: label_modelabel','The underscore must ', &
                             'not otherwise be part of a label.'
                        write(6,'(2a)') 'operator-label = ',deflabel(n)
                        write(6,'(2a)') 'unknown mode-label = ',label
                        write(ilog,'(/,2a,/a,/2a)') 'The underscore is a ', &
                             'special character used to specify a dof.', &
                             'E.g.: label_modelabel','The underscore must ', &
                             'not otherwise be part of a label.'
                        write(ilog,'(2a)') 'operator-label = ',deflabel(n)
                        write(ilog,'(2a)') 'unknown mode-label = ',label
                        routine = 'matchlab'
                        message = 'Trying to add operator to unknown dof'
                        call warnmsg
                     endif
                  else
                     if (ktot .lt. maxkoe) then
                        minkoe=minkoe+1
                        ktot=ktot+1
                        k=ktot
                     else
                        minkoe=minkoe+1
                        k=maxkoe
                     endif
                     
                     call storetm(deflabop(n),1,optyp,f,h,0,0)
                     hamilton(:,k)=0
                     hamilton(f,k)=h
                     hamnum(k)=1
                     kact(k)  = 1
                     ktwin(k) = k
                     hl=hterm(1,h)
                     if (hl .gt. 0) hopidef(hl)=n
                     
                  endif
               endif
            endif
         enddo
      endif

!-----------------------------------------------------------------------
! set up array of files to be read
!-----------------------------------------------------------------------
      nrdf=0
      l_mlpf=.false.
      call cpvxd(npotcut,npotcut1,maxrdf)
      call zerovxd(npotcut,maxrdf)
      call zeromxi(npmuld,maxrdf,maxrdf)
      call zerovxi(npignore,maxrdf)
      ignore=.false.
inner:do hl=1,min(hopdim,maxhop)
         lfile=.false.
         if (hoplab(hl)(1:6) .eq. 'natpot' .or. &
                  hoplab(hl)(1:6) .eq. 'potfit') then
            ilbl=index(hoplab(hl),'{')+1
            jlbl=index(hoplab(hl),'}')-1
            if (ilbl .eq. 1) then
               message='No arguments with natpot operator'
               call warnmsg
               cycle inner
            endif
            lnpot=.true.

            read(hoplab(hl)(ilbl:jlbl),*) np

            ilbl=zpesopts(1,np)
            if (ilbl .ne. 0) then
               if (pesopts(np)(1:ilbl) .eq. 'name') then
                  filename = name(1:laenge)//'/natpot'
               else
                  filename = pesopts(np)(1:ilbl)//'/natpot'
                  ilbl = ilbl + 7
                  call abspath(filename,ilbl)
               endif
            else
               filename =  name(1:laenge)//'/natpot'
            endif

            call rdfnum(nr,filename)

            ilbl=zpesopts(1,np)+1
            jlbl=zpesopts(2,np)
            ignore=.false.
            npignore(nr)=-9
            nplocal(nr)=0
            if(jlbl.gt.ilbl) then
               if(pesopts(np)(ilbl:jlbl).eq.'ignore') then 
                  ignore=.true.
                  npignore(nr)=0  ! will be set to a positive number later.

               else if (pesopts(np)(ilbl:ilbl+4).eq.'local') then
                     ilocalpr = -1

                  ilbl=zpesopts(2,np)+1
                  jlbl=zpesopts(3,np)
                  if (jlbl .lt. ilbl) then
                     routine = 'matchlab, labels-section'
                     message='No reference point given for local Natpot'
                     call errormsg
                  endif
                  read(pesopts(np)(ilbl:jlbl),*) nplocal(nr)

               else 
                  routine = 'matchlab, labels-section'
                  write(6,'(a,i3,",",2x,2a)') ' nr =',nr, &
                  'Expecting "ignore" or "local" but not: ', &
                        pesopts(np)(ilbl:jlbl)
                  message='Illegal keyword in natpot definition'
                  call errormsg
               endif
            endif

            if (nr .gt. 99999) then
               routine = 'matchlab, labels-section'
               message='Too many potfit files for format. '
               call errormsg
            endif
            write(hoplab(hl),'(a6,i5)') 'natpot',nr
            if (np .le. maxrdf0) then
               npotcut(nr)=npotcut1(np)
            endif

         elseif (hoplab(hl)(1:7) .eq. 'cpdfile') then
!--------------------------------------------------------
!     we use the same readfile setup as for the natpots
!     for the CPD/CANDECOMP files
!--------------------------------------------------------
            lfile=.true.
            ilbl=index(hoplab(hl),'{')+1
            jlbl=index(hoplab(hl),'}')-1
            if (ilbl .eq. 1) then
               message='No arguments with cpdfile operator'
               call warnmsg
               cycle inner
            endif

            read(hoplab(hl)(ilbl:jlbl),*) np

            ilbl=zpesopts(1,np)
            if (ilbl .ne. 0) then
               if (pesopts(np)(1:ilbl) .eq. 'name') then
                  filename = name(1:laenge)//'/cpd'
               else
                  filename = pesopts(np)(1:ilbl)//'/cpd'
                  ilbl = ilbl + 7
                  call abspath(filename,ilbl)
               endif
            else
               filename =  name(1:laenge)//'/cpd'
            endif

            call rdfnum(nr,filename)

            ilbl=zpesopts(1,np)+1
            jlbl=zpesopts(2,np)
            ignore=.false.
            npignore(nr)=-9
            if(jlbl.gt.ilbl) then
               if(pesopts(np)(ilbl:jlbl).eq.'ignore') then
                  ignore=.true.
                  npignore(nr)=0  ! will be set to a positive number later.
               else
                  write(6,'(a,i3,",",2x,2a)') ' nr =',nr,&
                  'Expecting "ignore" but not: ',pesopts(np)(ilbl:jlbl)
                  message='Illegal keyword in natpot definition'
                  call errormsg
               endif
            endif

            write(hoplab(hl),'(a7,i3)') 'cpdfile',nr
            if (lfast1(np)) then
               write(6,'(a,i3,",",2x,2a)') ' nr =',nr,&
                    ', file:', filename(1:min(laenge+4,c5))
               message='"Fast" cannot be used with cpd files'
               call errormsg
            end if
      
            ! TODO: check this for CPD    
            npotcut(nr)=npotcut1(np)

         elseif (hoplab(hl)(1:8) .eq. 'hcpdfile') then
!--------------------------------------------------------
!     we use the same readfile setup as for the natpots
!     for the CPD/CANDECOMP files
!--------------------------------------------------------
            lfile=.true.
            ilbl=index(hoplab(hl),'{')+1
            jlbl=index(hoplab(hl),'}')-1
            if (ilbl .eq. 1) then
               message='No arguments with cpdfile operator'
               call warnmsg
               cycle inner
            endif

            read(hoplab(hl)(ilbl:jlbl),*) np

            ilbl=zpesopts(1,np)
            if (ilbl .ne. 0) then
               if (pesopts(np)(1:ilbl) .eq. 'name') then
                  filename = name(1:laenge)//'/hcpd'
               else
                  filename = pesopts(np)(1:ilbl)//'/hcpd'
                  ilbl = ilbl + 8
                  call abspath(filename,ilbl)
               endif
            else
               filename =  name(1:laenge)//'/hcpd'
            endif

            call rdfnum(nr,filename)
            ilbl=zpesopts(1,np)+1
            jlbl=zpesopts(2,np)
            ignore=.false.
            npignore(nr)=-9
            if(jlbl.gt.ilbl) then
               if(pesopts(np)(ilbl:jlbl).eq.'ignore') then
                  ignore=.true.
                  npignore(nr)=0  ! will be set to a positive number later.
               else
                  write(6,'(a,i3,",",2x,2a)') ' nr =',nr, &
                  'Expecting "ignore" but not: ',pesopts(np)(ilbl:jlbl)
                  message='Illegal keyword in natpot definition'
                  call errormsg
               endif
            endif

            write(hoplab(hl),'(a8,i3)') 'hcpdfile',nr

            if (lfast1(np)) then
               write(6,'(a,i3,",",2x,2a)') ' nr =',nr, &
                    ', file:', filename(1:min(laenge+4,c5))
               message='"Fast" cannot be used with cpd files'
               call errormsg
            end if

            ! TODO: check this for CPD
            npotcut(nr)=npotcut1(np)

         else if (hoplab(hl)(1:8) .eq. 'readmap{' ) then
            ilbl=index(hoplab(hl),'{')+1
            jlbl=index(hoplab(hl),'}')-1
            if (ilbl .eq. 1) then
               message='No arguments with read operator'
               call warnmsg
               cycle inner
            endif
            read(hoplab(hl)(ilbl:jlbl),*) np

            ilbl=zpesopts(1,np)
            jlbl=zpesopts(2,np)
            filename = pesopts(np)(1:ilbl)
            ilbl=ilbl+1
            read(pesopts(np)(ilbl:jlbl),*) n
            ilbl=ilbl-1

            call abspath(filename,ilbl)
            call rdfnum(nr,filename)

            write(hoplab(hl),'(2(a,i7))') 'readmap',n,',',nr
            call rmblnk(hoplab(hl))

         else if (hoplab(hl)(1:5) .eq. 'read{' ) then
            ilbl=index(hoplab(hl),'{')+1
            jlbl=index(hoplab(hl),'}')-1
            if (ilbl .eq. 1) then
               message='No arguments with read operator'
               call warnmsg
               cycle inner
            endif
            read(hoplab(hl)(ilbl:jlbl),*) np

            ilbl=zpesopts(1,np)
            jlbl=zpesopts(2,np)
            filename = pesopts(np)(1:ilbl)
            ilbl=ilbl+1
            read(pesopts(np)(ilbl:jlbl),*) n
            ilbl=ilbl-1

            call abspath(filename,ilbl)
            call rdfnum(nr,filename)

            write(hoplab(hl),'(2(a,i7))') 'read',n,',',nr
            call rmblnk(hoplab(hl))

         else if (hoplab(hl)(1:5) .eq. 'mlpf{') then
            ! Get filename. First get option number.
            ilbl=index(hoplab(hl),'{')+1
            jlbl=index(hoplab(hl),'}')-1
            if (ilbl .eq. 1) then
               message='Missing argument for mlpf operator'
               call warnmsg
               cycle inner
            endif
            l_mlpf=.true.
            read(hoplab(hl)(ilbl:jlbl),*) np
            ! Now get the filename.
            ilbl=zpesopts(1,np)
            filename = pesopts(np)(1:ilbl)//'/mlpf.dat'
            ! Make it absolute.
            ilbl=len_trim(filename)
            call abspath(filename,ilbl)
            ! Add it to the list of readfiles.
            call rdfnum(nr,filename)
            ! Create operator label.
            write(hoplab(hl),'(a4,i3)') 'mlpf',nr

         endif
      enddo inner

!-----------------------------------------------------------------------
! expand any terms defining natural potentials
!-----------------------------------------------------------------------
      maxsubpdim=0
      htime=0
      do h=1,min(htmdim,maxhtm)
         hl=hterm(1,h)
         if (hl .eq. 0) cycle
         if (hoplab(hl)(1:4) .eq. 'read' .and. htmmode(h).gt.0) then
!        Set htmtype for read mode-operators.
            if(hoplab(hl)(1:7).ne.'readsrf' .and. &
               hoplab(hl)(1:6).ne.'read1d')  htmtype(h)=8
            if(hoplab(hl)(1:7).eq.'readmap') htmtype(h)=25
         endif
         if (hoplab(hl)(1:6) .eq. 'natpot' .or. &
             hoplab(hl)(1:6) .eq. 'potfit') then
            if (htmfac(h) .ne. 1) then
               message= &
               'Label for natpot operator must be single factor'
               call warnmsg
            endif

            read(hoplab(hl)(7:11),'(i5)') nr
            if (npignore(nr).ge.0) then
               ignore=.true.
               m=htmmode(h)
               if(m .gt. 0) then   ! get muld information for natpot
                  call muldnum2(muld,spfdof(1,m),nspfdof(m))
                  htmmuld(h)=muld
               endif
               if (npignore(nr).lt.maxrdf) then
                  npignore(nr) = npignore(nr) + 1
                  npmuld(nr,npignore(nr)) = htmmuld(h)
               endif
            endif

            filename = rdfiles(nr)
            open(unit=inpot,iostat=ierr,file=filename,status='old', &
                    form='unformatted')
            if (ierr.ne.0) then
               routine='Matchlab (matchlabmod.f90)'
               message = 'Error opening natpot file : '//trim(filename)
               call errormsg
            endif

! check if natpot is defined as mode or grid operator.
! If ignore is true, ignore the modelabels and use the |i&j&k
! information from the Hamiltonian section.
            call zerovxi(fdvr,maxdim)
            if (htmdof(h) .ne. 0) then
               f=htmdof(h)
               if (ignore) then
                  routine='matchlab'
                  message='Cannot ignore modelabels when no |i&j&k '// &
                  'information is provided in the Hamiltonian-Section'
                  call errormsg
               endif
            elseif (htmmuld(h) .ne. 0) then
               f=mulddof(1,htmmuld(h))
               if(ignore) call cpvxi(mulddof2(:,htmmuld(h)),fdvr,ndof)
            elseif (htmmode(h) .ne. 0) then
!              f=spfdof(1,htmmode(h))
               f=htmadof(h) ! NEWKLEG
               if(ignore) then
                  routine='matchlab'
                  message='Cannot ignore modelabels when no |i&j&k '// &
                  'information is provided in the Hamiltonian-Section'
                  call errormsg
               endif
            endif

            do k=1,ktot
               if (hamilton(f,k) .eq. h) then
                  delete(k)=.true.
                  if(ftime.gt.0) htime = hamilton(ftime,k)
                  chkdvr=0
                  if (ignore) then
                     chkdvr = 4
                  else
                     chkdvr = 3
                  endif
                  call npotlab(inpot,nr,h,k,hamilton(fkoe,k),htime, &
                               fdvr,htmmode(h),htmmuld(h),nr,chkdvr)
               endif
            enddo

            close (inpot)

         endif
      enddo

!-----------------------------------------------------------------------
! expand any terms that use MLPFs
!-----------------------------------------------------------------------
      if(l_mlpf) then
         if(.not.lrunml) then
            routine='matchlab'
            message='A multi-layer potfit (mlpf) can only '// &
           'be used when running ML'
            call errormsg
         endif

         ! Read in all MLPFs.
         do hl=1,min(hopdim,maxhop)
            if (hoplab(hl)(1:4) .eq. 'mlpf') then
               read (hoplab(hl)(5:7),'(i3)') nr
               filename = rdfiles(nr)
               open(unit=inpot,iostat=ierr,file=filename,status='old',&
                    form='unformatted')
               if (ierr.ne.0) then
                  routine='Matchlab (matchlabmod.f90)'
                  message = 'Error opening MLPF file : '//trim(filename)
                  call errormsg
               endif
               call mlpfinfo(inpot,nr)
               call chksyserr(filename,loperr)
               if (loperr) stop
               close(inpot)
            endif
         enddo

         ! Find operator lines that contain MLPFs and turn them into MLOps.
         do k=1,ktot
            mlpfnr(:) = 0
            do f=1,ndof
               h=hamilton(f,k)
               if (h<=0) cycle
               lmlpf=.false.
               ! Check all terms for MLPF.
               do nh=1,htmfac(h)
                  hl=hterm(nh,h)
                  if (hl .eq. 0) cycle
                  if (hoplab(hl)(1:4) .eq. 'mlpf') lmlpf=.true.
               enddo
               if (.not.lmlpf) cycle
               ! MLPF found!
               if (htmfac(h) .ne. 1) then
                  message='Multiplication of MLPF with other operators ' &
                        //'is not implemented yet.'
                  call warnmsg
               endif
               hl=hterm(1,h)
               read (hoplab(hl)(5:7),'(i3)') nr
               mlpfnr(f) = nr
            enddo ! f=1,ndof
            ! Any MLPF on this line?
            if (sum(mlpfnr)==0) cycle

            ! Expand line.
            call mlpflab(k,mlpfnr,loperr)
            if (loperr) then
               routine = "matchlab"
               message = "error in MLPF processing"
               call warnmsg
            endif
            delete(k) = .true.
         enddo  ! k=1,ktot
      endif ! l_mlpf

!-----------------------------------------------------------------------
! expand any terms defining CPD or HCPD terms
!-----------------------------------------------------------------------
      do h=1,min(htmdim,maxhtm)
         hl=hterm(1,h)
         if (hl .eq. 0) cycle

         if (hoplab(hl)(1:7) .eq. 'cpdfile'.or. &
              hoplab(hl)(1:8) .eq. 'hcpdfile') then
            if (htmfac(h) .ne. 1) then
               message= 'Label for CPD/HCPD operator must be single factor'
               call warnmsg
            endif

            if (hoplab(hl)(1:7) .eq. 'cpdfile') then
               read(hoplab(hl)(8:12),'(i3)') nr
            elseif (hoplab(hl)(1:8) .eq. 'hcpdfile') then
               read(hoplab(hl)(9:13),'(i3)') nr
            endif

            if (npignore(nr).ge.0) then
               ignore=.true.
               m=htmmode(h)
               if(m .gt. 0) then   ! get muld information for natpot
                  call muldnum2(muld,spfdof(1,m),nspfdof(m))
                  htmmuld(h)=muld
               endif
               if (npignore(nr).lt.maxrdf) then
                  npignore(nr) = npignore(nr) + 1
                  npmuld(nr,npignore(nr)) = htmmuld(h)
               endif
            endif

            filename = rdfiles(nr)
            open(unit=inpot,iostat=ierr,file=filename,status='old', &
                    form='unformatted')
            if (ierr.ne.0) then
               routine='Matchlab (matchlabmod.f90)'
               message = 'Error opening cpd file : '//trim(filename)
               call errormsg
            endif

! check if cpd is defined as mode or grid operator.
! If ignore is true, ignore the modelabels and use the |i&j&k
! information from the Hamiltonian section.
            call zerovxi(fdvr,maxdim)
            if (htmdof(h) .ne. 0) then
               f=htmdof(h)
               if (ignore) then
                  routine='matchlab'
                  message='Cannot ignore modelabels when no |i&j&k '//&
                  'information is provided in the Hamiltonian-Section'
                  call errormsg
               endif
            elseif (htmmuld(h) .ne. 0) then
               f=mulddof(1,htmmuld(h))
               if(ignore) call cpvxi(mulddof2(:,htmmuld(h)),fdvr,ndof)
            elseif (htmmode(h) .ne. 0) then
!              f=spfdof(1,htmmode(h))
               f=htmadof(h) ! NEWKLEG
               if(ignore) then
                  routine='matchlab'
                  message='Cannot ignore modelabels when no |i&j&k '//&
                  'information is provided in the Hamiltonian-Section'
                  call errormsg
               endif
            endif

            do k=1,ktot

               if (hamilton(f,k) .eq. h) then
                  delete(k)=.true.
                  if(ftime.gt.0) htime = hamilton(ftime,k)
                  chkdvr=0
                  if(ignore) chkdvr=-1
                  if (hoplab(hl)(1:7) .eq. 'cpdfile') then
                     call cpdlab(iop,nr,h,k,hamilton(fkoe,k),htime,&
                          fdvr,htmmode(h),htmmuld(h),nr,chkdvr)
                  elseif (hoplab(hl)(1:8) .eq. 'hcpdfile') then

                     call hcpdlab(iop,nr,h,k,hamilton(fkoe,k),htime,&
                          fdvr,htmmode(h),htmmuld(h),nr,chkdvr)
                  endif

               endif
            enddo

            close (inpot)

         endif
      enddo

!-----------------------------------------------------------------------
! in a G-MCTDH calculation, expand any multi-dimensional potential 
! that goes across GWPs (a GWP muld pot). 
!-----------------------------------------------------------------------
      if (lgwp .and. .not. lvmcg) then
         do h=1,min(htmdim,maxhtm)
            muld = htmmuld(h)
            if (muld .eq. 0) cycle

!find term containing mult pot
            do k=1,min(ktot,maxkoe)
               do m=1,nmode
                  f=spfdof(1,m)
                  h1 = hamilton(f,k)
                  if (h1 .eq. h ) then
! type term as muld LHA
                     htmtype(h) = 104

                     hl = hterm(1,h)
! hl can be 0 if first pass through operator generation
                     if (hl .gt. 0) call lhalabgen(h,mulddof(1,muld),&
                                         nmulddof(muld))

                     delete(k) = .true.
                     exit
                  endif
               enddo
            enddo
            
         enddo
      endif
      minkoe = max(minkoe,ktot)

!-----------------------------------------------------------------------
! delete any terms that have been expanded
!-----------------------------------------------------------------------
      ktot1=ktot
      k1=1
      do k=1,min(ktot1,maxkoe)
         if (delete(k)) then
            call mvkend(k1)
            ktot=ktot-1
          else
            k1=k1+1
         endif
      enddo

!-----------------------------------------------------------------------
! check that all labels have been substituted 
!-----------------------------------------------------------------------
      do n=1,min(ndef,maxdef)
         do hl=1,min(hopdim,maxhop)
            if (hoplab(hl) .eq. deflabel(n)) then
               hoplab(hl)=deflabop(n)
               hopidef(hl)=n
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      loperr=lwarn.or.loperr
      lwarn=.false.

      return
      end subroutine matchlab

!#######################################################################
!
! NPOTLAB
!
! Reads data for natpot operators and expands hamiltonian term
!
!#######################################################################

      subroutine npotlab(ichan,np,hnptm,knptm,hkoe,htime,fdvr, &
                         mm,md,nr,chkdvr)

      implicit none

      integer(long)       :: f,zeig,f1,p1,ktot1,h,m,m1,n,v,p,q,k,&
                             knpot,k1,ndof1,koned,nmode1,&
                             ff,hl,ktot2,ktot3,chkgrd,k2

      integer(long), intent(in)                         :: mm,md,nr,np,chkdvr,ichan,&
                                                           knptm,hnptm,hkoe,htime
      integer(long), dimension(1)                       :: optyp
      integer(long), dimension(maxdim), intent(out)     :: fdvr
      integer(long), dimension(maxdim)                  :: mdvr,mmode,pntbas,&
                                                           nspfdof1,subdim1
      integer(long), dimension(maxdim,maxdim)           :: spfdof1
      character(len=c2), dimension(maxkoe,maxdim)       :: veclab
      character(len=c2), dimension(maxdim,maxkoe)       :: npotop
      real(dop)                                         :: suma
      logical(kind=4)                                   :: lerr,used
      logical(kind=4), dimension(maxdim)                :: loned

!----------------------------------------------------------------------
! read information in the natpot file 
! check the combinations of the degrees of freedom in mctdh
! and in potfit are consistent
!----------------------------------------------------------------------
      k=0
      chkgrd=3
      rewind(ichan)
      pntbas=0
      call npotinfo(ichan,lerr,modc,loned,ndof1,fdvr,chkdvr, &
           chkgrd,0,mdvr,mmode,nspfdof1,spfdof1,nmode1, &
           subdim1,pntbas)

!-----------------------------------------------------------------------
! calculate pointers to multiply hamilton particle operators
! vpdim and npdim ignore the contracted mode
!-----------------------------------------------------------------------
      zeig=1
      do m=1,nmode1
         if (m.ne.modc .and. pdim(m).ne.0) then
            vpdim(m)=zeig
            zeig=zeig*pdim(m)
         else if (pdim(m) .ne. 0) then
            vpdim(m)=1
         else
            vpdim(m)=0
         endif
      enddo
      do m=1,nmode1
         if (m.ne.modc .and. pdim(m).ne.0) then
            zeig=zeig/pdim(m)
            npdim(m)=zeig
         else if (pdim(m) .ne. 0) then
            npdim(m)=1
         else
            npdim(m)=0
         endif
      enddo

! --- Save pdim,vpdim,npdim arrays which defined for Potfit modes, 
! --- to mdim,vmdim,nmdim (for MCTDH modes)
      call savepdim(fdvr,mdvr,nspfdof1,spfdof1,nmode1,np)

!-----------------------------------------------------------------------
! Check if the natpot-DOFs agree with the |i&j&k definition of the
! operator file.
! hnptm and knptm denote the h and k of the (not yet expanded) natpot
!-----------------------------------------------------------------------
      if (md.ne.0) then
         do n=1,ndof1                ! Loop over potfit DOFs
            f=mulddof2(n,md)         ! mctdh DOF   
            f1=fdvr(n)               ! mapped potfit DOF 
            if (f.ne.f1) goto 97
         enddo
      elseif (mm.ne.0)  then
         do n=1,ndof1                ! Loop over potfit DOFs
            f=spfdof(n,mm)           ! mctdh DOF
            f1=fdvr(n)               ! mapped potfit DOF
            if (f.ne.f1) goto 97
         enddo
      endif
! --- Re-define hamilton, remove zeros and set h=hnptm appropriately.
      do f=1,ndof1
         if(hamilton(f,knptm) .eq. hnptm  .or. &
            hamilton(f,knptm) .eq. 0 ) hamilton(f,knptm) = f
      enddo
      do f=1,ndof1
         if(hamilton(f,knptm) .ne. f) then
            h=hamilton(f,knptm)
            hl=hterm(1,h)
            if(hoplab(hl)(1:6) .eq. 'natpot'  .or. &
                 hoplab(hl)(1:6) .eq. 'potfit'  .or. &
                 hoplab(hl)(1:4) .eq. 'mlpf'    .or. &
                 hoplab(hl)(1:7) .eq. 'cpdfile') then
               call logop
               write(6,'(/a,4i4,2x,a)')  &
                     'k, h, hl, f', knptm,h,hl,f,hoplab(hl)
               routine='npotlab  (genoper/matchlab.F  line 567)'
               message='There must not be two natpots/cpds/mlpfs in one '// &
                       'Hamiltonian-Section line.'
               call errormsg
            endif  
         endif 
      enddo
      do f=1,ndof1
         if (fdvr(f).gt.0) hamilton(fdvr(f),knptm) = hnptm 
      enddo

!-----------------------------------------------------------------------
! set up labels
! maxkoe is present size used in arrays, minkoe what it needs to be
!-----------------------------------------------------------------------
      k=0
      do m=1,nmode1
         minkoe = max(pdim(m),minkoe) 
         do n=1,nspfdof1(m)         
            f=spfdof1(n,m) 
            p=0
            if (loned(f)) then
               p=min(pdim(m)+1+p,maxkoe)
               f1=fdvr(f)
               write(veclab(p,m),'(2(a,i7))') 'oned',f1,',',np
               call rmblnk(veclab(p,m))
            endif
         enddo
      enddo

      p1=0
      do m=1,nmode1
         do p=1,min(pdim(m),maxkoe)
            p1=p1+1
            write(veclab(p,m),'(2(a,i7))') 'npot',p1,',',np
            call rmblnk(veclab(p,m))
         enddo
      enddo

!-----------------------------------------------------------------------
! set up natpot operator
!-----------------------------------------------------------------------
      do m=1,nmode1
         do n=1,nspfdof1(m)
            f=spfdof1(n,m)
            p=0
            if (loned(f)) then
               k=min(k+1,maxkoe)
               do f1=1,ndof
                  npotop(f1,k) = '1'
               enddo
               p=min(pdim(m)+1+p,maxkoe)
               f1=fdvr(f)
               npotop(f1,k)=veclab(p,m)
            endif
         enddo
      enddo

      koned=k

      k1=0
      do m=1,nmode1
         minkoe=max(vpdim(m)*pdim(m)*npdim(m),minkoe)
         k1=k
         do q=1,npdim(m)
            do p=1,min(pdim(m),maxkoe)
               do v=1,vpdim(m)
                  k1=min(k1+1,maxkoe)
                  npotop(m,k1)=veclab(p,m)
               enddo
            enddo
         enddo 
      enddo

      knpot=koned+vpdim(1)*pdim(1)*npdim(1)
!-----------------------------------------------------------------------
! add new terms to ktot
! koned is for the substracted one-dimensional potentials
! knpot is for the natpot operators
!-----------------------------------------------------------------------
      ktot1=ktot+1
      ktot2=ktot+koned
      ktot3=ktot2+1
      ktot=ktot+knpot

! calculate required array sizes
      minkoe=max(ktot,minkoe)

! chop bounds to fit present memory
      ktot=min(ktot,maxkoe)
      ktot1=min(ktot1,maxkoe)
      ktot2=min(ktot2,maxkoe)
      ktot3=min(ktot3,maxkoe)

      optyp(1)=3

      k=0
      do k1=ktot1,ktot2
         k=k+1
         do f=1,ndof1
            f1=fdvr(f)
!----------------------------------------------------------------------  
! the 1-D    functions are stored as 1-D operators
! the order of the degrees of freedom in the Primitive-BASIS-SECTION
! can be different in the potfit and mctdh input files :
! this is why f (in potfit) can be different from f1 (in mctdh)
!----------------------------------------------------------------------
            call storetm(npotop(f1,k),1,optyp,f1,h,0,0)
            hamilton(f1,k1)=h
            hamnum(k1)= hamnum(knptm)
            kact(k1)  = kact(knptm)
            ktwin(k1) = ktwin(knptm)
         enddo
      enddo

      k1=ktot3-1
      do k2=ktot3,ktot
         k=k+1
         call setmp(k-koned,np,nmode1,suma) ! pruning of natpot
         if(suma.ge.1.0001d0) goto 789
         k1=k1+1
         do m=1,nmode1
            m1=mmode(m)
            if (nspfdof1(m).eq.1) then
               f=spfdof1(1,m)          
               f1=fdvr(f)
!----------------------------------------------------------------------  
! if the degrees are not combined (nspfdof1(m) .eq. 1) 
! the natpot operators are stored as 1-D operators
! the label associated with the mode can be different
! in potfit and in mctdh :
! this is why m (in potfit) can be different from m1 (in mctdh)
!---------------------------------------------------------------------
               call storetm(npotop(m,k),1,optyp,f1,h,0,0)
            else
               f=spfdof1(1,m)
               f1=fdvr(f)
!----------------------------------------------------------------------  
! if the degrees are combined (nspfdof1(m) .gt. 1) 
! the natpot operators are stored as particle operators
!---------------------------------------------------------------------
               call storetm(npotop(m,k),1,optyp,f1,h,0,m1)
!              for sub-particle operators, store number of DOFs
!              that it acts on
               if (nspfdof1(m).lt.nspfdof(m1)) then
                  htmndof(h)=nspfdof1(m)
                  maxsubpdim=max(maxsubpdim,subdim1(m))
               endif
            endif
!----------------------------------------------------------------------  
! if the degrees are combined (nspfdof1(m) .gt. 1)
! for the k1 natpot operator and 
! for the first degree of freedom f1 in the mode m1 hamilton(f1,k1)=h
! for the other degrees of freedom ff hamilton(ff,k1)=ff
!--------------------------------------------------------------------
            hamilton(f1,k1)=h
            do n=2,nspfdof1(m)
               f=spfdof1(n,m)
               ff=fdvr(f)
               hamilton(ff,k1)=ff
            enddo
            hamnum(k1)= hamnum(knptm)
            kact(k1)  = kact(knptm)
            ktwin(k1) = ktwin(knptm)
         enddo ! do m=1,nmode1
 789  continue
      enddo    ! do k1=ktot3,ktot
      ktot=k1

!-----------------------------------------------------------------------
! copy hamilton term for any dofs not included in potential surface
!-----------------------------------------------------------------------
      do f1=1,ndof
         used=.false.
         do f=1,ndof1
            if (f1.eq.fdvr(f)) used=.true.
         enddo

         if (.not. used) then
            do k=ktot1,ktot
               hamilton(f1,k)=hamilton(f1,knptm)
            enddo
         endif
      enddo
!-----------------------------------------------------------------------
! set coefficients 
!-----------------------------------------------------------------------
      do k=ktot1,ktot
         hamilton(fkoe,k)=hkoe
         if(ftime.gt.0) hamilton(ftime,k)=htime
      enddo

      return

!-----------------------------------------------------------------------
! Error handling
!-----------------------------------------------------------------------
 97   write(ilog,'(/a,/a,i3,a,i3,/a,i3,2x,a/,a,i3,2x,a,/a,i3)') &
       ' ################## ERROR ################################', &
       ' Inconsistent DOFs in natpot no. nr =', nr,', muld =',md, &
       ' original natpot DOF is: f =', n, modelabel(n), &
       ' mapped   natpot DOF is: f =',f1, modelabel(f1), &
       ' mctdh           DOF is: f =', f, modelabel(f)

      write(ilog,'(2x,a,3(/a),/2a,/2a,/a,/a)') &
       ' The natpot modelabels must be consistent with the mctdh ones!', &
       ' Probably a wrong |i&j&k statement in the Hamiltonian-Section.', &
       ' Avoid to specify the DOF-numbers i&j&k of a natpot.', &
       ' Then the assignment of the DOFs is done solely through', &
       ' modelabels.',' Alternatively, set the keyword "ignore" ', &
       ' to ignore the modelabels ', &
       ' and to solely specify the DOF-numbers by i&j&k.',&
       ' --------------------------------------------------------------'
      write(ilog,'(a,i3,a,i3,a,i3)') ' muld =', md,',  mode =',mm,&
                                     ',  natpot no.: nr =', nr
      if(md .gt. 0) then
         write(ilog,'(a,i3,a,i3)') ' ndof1 =', ndof1,&
                                 ',  nmulddof =',nmulddof(md)
         write(ilog,'(a,50i3)') ' natpot    (fdvr) :',&
                                  (fdvr(n),n=1,ndof1)
         write(ilog,'(a,50i3)') ' MCTDH (mulddof2) :',&
                                  (mulddof2(n,md),n=1,ndof1)
      else
         write(ilog,'(a,i3,a,i3)') ' ndof1 =', ndof1,&
                                 ',  nspfdof =',nspfdof(mm)
         write(ilog,'(a,50i3)') ' natpot  (fdvr) :',&
                                  (fdvr(n),n=1,ndof1)
         write(ilog,'(a,50i3)') ' MCTDH (spfdof) :',&
                                  (spfdof(n,mm) ,n=1,ndof1)
      endif
      write(ilog,'(a)')&
       ' #########################################################'

      if(ilog.eq.6) call stopnow
      ilog=6
      goto 97
           
      end subroutine npotlab

!#######################################################################
!
! CPDLAB
!
! Reads data for cpd operators and expands hamiltonian term
!
!#######################################################################

      subroutine cpdlab(ichan,np,hnptm,knptm,hkoe,htime,fdvr,&
                         mm,md,nr,chkdvr)

      implicit none

      integer(long), intent(in)                     :: ichan
      integer(long), intent(in)                     :: knptm,hnptm,hkoe,htime
      integer(long), dimension(maxdim), intent(out) :: fdvr
      integer(long), intent(in)                     :: chkdvr,mm,md,nr,np

      integer(long) :: p,k,knpot,k1,ndof1
      integer(long) :: f,f1,p1,ktot1,chkgrd,h,m,m1,n
      integer(long) :: ff,hl,k2,cpdrank
      integer(long), dimension(1)             :: optyp
      integer(long), dimension(maxdim,maxdim) :: spfdof1
      integer(long), dimension(maxdim)        :: mdvr,mmode,pntbas
      integer(long), dimension(maxdim)        :: nspfdof1,subdim1
      integer(long)                           :: ktot2,ktot3,koned,nmode1
      character(len=c2),dimension(maxkoe,maxdim) :: veclab,npotop
      logical(kind=4),dimension(maxdim)       :: loned
      logical(kind=4)                         :: lerr,used

!----------------------------------------------------------------------
! read information in the natpot file
! check the combinations of the degrees of freedom in mctdh
! and in potfit are consistent
!----------------------------------------------------------------------
      chkgrd=3
      rewind(ichan)
      pntbas=0
      call cpdinfo(ichan,lerr,ndof1,fdvr,chkdvr,&
           chkgrd,0,mdvr,mmode,cpdrank,nspfdof1,spfdof1,nmode1,&
           subdim1,loned,pntbas)

      do m1=1,nmode1
         do n=1,nspfdof1(m1)
            f1=spfdof1(n,m1)
            do f=1,ndof
               if (fdvr(f1).eq.f) mdvr(m1)=dofspf(f)
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! Check if the cpd-DOFs agree with the |i&j&k definition of the
! operator file.
! hnptm and knptm denote the h and k of the (not yet expanded) cpd
!-----------------------------------------------------------------------
      if (md.ne.0) then
         do n=1,ndof1                ! Loop over cpd DOFs
            f=mulddof2(n,md)         ! mctdh DOF
            f1=fdvr(n)               ! mapped cpd DOF
            if (f.ne.f1) goto 197
         enddo
      elseif (mm.ne.0)  then
         do n=1,ndof1                ! Loop over cpd DOFs
            f=spfdof(n,mm)           ! mctdh DOF
            f1=fdvr(n)               ! mapped cpd DOF
            if (f.ne.f1) goto 197
         enddo
      endif
! --- Re-define hamilton, remove zeros and set h=hnptm appropriately.
      do f=1,ndof
         if(hamilton(f,knptm) .eq. hnptm  .or.&
            hamilton(f,knptm) .eq. 0 ) hamilton(f,knptm) = f
      enddo
      do f=1,ndof
         if(hamilton(f,knptm) .ne. f) then
            h=hamilton(f,knptm)
            hl=hterm(1,h)
            if(hoplab(hl)(1:7) .eq. 'cpdfile'  .or.&
                 hoplab(hl)(1:6) .eq. 'natpot' .or.&
                 hoplab(hl)(1:6) .eq. 'potfit' ) then
               call logop
               write(6,'(/a,4i4,2x,a)')&
                     'k, h, hl, f', knptm,h,hl,f,hoplab(hl)
               routine='cpdlab  (genoper/matchlab.F)'
               message='There must not be two natpots or cpds in one'//&
                       ' Hamiltonian-Section line.'
               call errormsg
            endif
         endif
      enddo
      do f=1,ndof
         if (fdvr(f).gt.0) hamilton(fdvr(f),knptm) = hnptm
      enddo

!-----------------------------------------------------------------------
! set up labels
! maxkoe is present size used in arrays, minkoe what it needs to be
!-----------------------------------------------------------------------
      k=0
      do m=1,nmode1
         do n=1,nspfdof1(m)
            f=spfdof1(n,m)
            p = 0
            if (loned(f)) then
               f1=fdvr(f)
               p = min(maxkoe,cpdrank+p+1)
               write(veclab(p,m),'(2(a,i7))') 'oned',f1,',',np
               call rmblnk(veclab(p,m))
            endif
         enddo
      enddo


      p1 = 0
      do p=1,min(maxkoe,cpdrank)
         do m=1,nmode1
            p1 = p1 + 1
            write(veclab(p,m),'(2(a,i7))') 'cpdtm',p1,',',np
            call rmblnk(veclab(p,m))
         enddo
      enddo

!-----------------------------------------------------------------------
! set up cpd operator
!-----------------------------------------------------------------------
      k=0
      do m=1,nmode1
         do n=1,nspfdof1(m)
            f=spfdof1(n,m)
            p = 0
            if (loned(f)) then
               k=k+1
               do f1=1,ndof
                  npotop(f1,k) = '1'
               enddo
               f1=fdvr(f)
               p = min(cpdrank+p+1,maxkoe)
               npotop(f1,k)=veclab(p,m)

            endif
         enddo
      enddo

      koned = k

      k1 = k
      do p=1,min(maxkoe,cpdrank)
         k1=min(k1+1,maxkoe)
         do m=1,nmode1
            npotop(m,k1)=veclab(p,m)
         enddo
      enddo

      knpot=k1                  ! should be .eq. cpdrank

!-----------------------------------------------------------------------
! add new terms to ktot
! knpot is for the natpot operators
!-----------------------------------------------------------------------
      ktot1=ktot+1
      ktot2=ktot+koned
      ktot3=ktot2+1
      ktot=ktot+knpot

! calculate required array sizes
      minkoe=minkoe+knpot

! chop bounds to fit present memory
      ktot=min(ktot,maxkoe)
      ktot1=min(ktot1,maxkoe)
      ktot2=min(ktot2,maxkoe)
      ktot3=min(ktot3,maxkoe)

      optyp(1)=3

      k=0
      do k1=ktot1,ktot2
         k=k+1
         do f=1,ndof1
            f1=fdvr(f)
!----------------------------------------------------------------------
! the 1-D    functions are stored as 1-D operators
! the order of the degrees of freedom in the Primitive-BASIS-SECTION
! can be different in the potfit and mctdh input files :
! this is why f (in potfit) can be different from f1 (in mctdh)
!----------------------------------------------------------------------
            call storetm(npotop(f1,k),1,optyp,f1,h,0,0)
            hamilton(f1,k1)=h
            hamnum(k1)= hamnum(knptm)
         enddo
      enddo


      k1=ktot3-1
      do k2=ktot3,ktot
         k=k+1
         k1=k1+1
         do m=1,nmode1
            m1=mmode(m)
            if (nspfdof1(m).eq.1) then
               f=spfdof1(1,m)
               f1=fdvr(f)
!----------------------------------------------------------------------
! if the degrees are not combined (nspfdof1(m) .eq. 1)
! the natpot operators are stored as 1-D operators
! the label associated with the mode can be different
! in potfit and in mctdh :
! this is why m (in potfit) can be different from m1 (in mctdh)
!---------------------------------------------------------------------
               call storetm(npotop(m,k),1,optyp,f1,h,0,0)
            else
               f=spfdof1(1,m)
               f1=fdvr(f)
!----------------------------------------------------------------------
! if the degrees are combined (nspfdof1(m) .gt. 1)
! the natpot operators are stored as particle operators
!---------------------------------------------------------------------
               call storetm(npotop(m,k),1,optyp,f1,h,0,m1)
!              for sub-particle operators, store number of DOFs
!              that it acts on
               if (nspfdof1(m).lt.nspfdof(m1)) then
                  htmndof(h)=nspfdof1(m)
                  maxsubpdim=max(maxsubpdim,subdim1(m))
               endif
            endif
!----------------------------------------------------------------------
! if the degrees are combined (nspfdof1(m) .gt. 1)
! for the k1 natpot operator and
! for the first degree of freedom f1 in the mode m1 hamilton(f1,k1)=h
! for the other degrees of freedom ff hamilton(ff,k1)=ff
!--------------------------------------------------------------------
            hamilton(f1,k1)=h
            do n=2,nspfdof1(m)
               f=spfdof1(n,m)
               ff=fdvr(f)
               hamilton(ff,k1)=ff
            enddo
            hamnum(k1)= hamnum(knptm)
         enddo ! do m=1,nmode1
      enddo    ! do k1=ktot3,ktot
      ktot=k1

!-----------------------------------------------------------------------
! copy hamilton term for any dofs not included in potential surface
!-----------------------------------------------------------------------
      do f1=1,ndof
         used=.false.
         do f=1,ndof1
            if (f1.eq.fdvr(f)) used=.true.
         enddo

         if (.not. used) then
            do k=ktot1,ktot
               hamilton(f1,k)=hamilton(f1,knptm)
            enddo
         endif
      enddo
!-----------------------------------------------------------------------
! set coefficients
!-----------------------------------------------------------------------
      do k=ktot1,ktot
         hamilton(fkoe,k)=hkoe
      enddo
      if(ftime.gt.0) then
         do k=ktot1,ktot
            hamilton(ftime,k)=htime
         enddo
      endif

      return

!-----------------------------------------------------------------------
! Error handling
!-----------------------------------------------------------------------
 197  write(ilog,&
       '(/a,/a,i3,a,i3,/a,i3,2x,a/,a,i3,2x,a,/a,i3,2x,a,3(/a),/2a,/2a,/a,/a)') &
       ' ################## ERROR ################################', &
       ' Inconsistent DOFs in cpd no. nr =', nr,', muld =',md, &
       ' original cpd DOF is: f =', n, modelabel(n), &
       ' mapped   cpd DOF is: f =',f1, modelabel(f1), &
       ' mctdh        DOF is: f =', f, modelabel(f), &
       ' The cpd modelabels must be consistent with the mctdh ones!', &
       ' Probably a wrong |i&j&k statement in the Hamiltonian-Section.', &
       ' Avoid to specify the DOF-numbers i&j&k of a cpd.', &
       ' Then the assignment of the DOFs is done solely through', &
       ' modelabels.',' Alternatively, set the keyword "ignore" ', &
       ' to ignore the modelabels ', &
       ' and to solely specify the DOF-numbers by i&j&k.', &
       ' --------------------------------------------------------------' 
      write(ilog,'(a,i3,a,i3,a,i3)') ' muld =', md,',  mode =',mm, &
                                     ',  cpd no.: nr =', nr
      if(md .gt. 0) then
         write(ilog,'(a,i3,a,i3)') ' ndof1 =', ndof1, &
                                 ',  nmulddof =',nmulddof(md)
         write(ilog,'(a,50i3)') ' cpd    (fdvr) :', &
                                  (fdvr(n),n=1,ndof1)
         write(ilog,'(a,50i3)') ' M!TDH (mulddof2) :', &
                                  (mulddof2(n,md),n=1,ndof1)
      else
         write(ilog,'(a,i3,a,i3)') ' ndof1 =', ndof1, &
                                 ',  nspfdof =',nspfdof(mm)
         write(ilog,'(a,50i3)') ' cpd  (fdvr) :', &
                                  (fdvr(n),n=1,ndof1)
         write(ilog,'(a,50i3)') ' M!TDH (spfdof) :', &
                                  (spfdof(n,mm) ,n=1,ndof1)
      endif
      write(ilog,'(a)') &
       ' #########################################################'
      if(ilog.eq.6) call stopnow
      ilog=6
      goto 197

      end subroutine cpdlab

!#######################################################################
!
! HCPDLAB
!
! Reads data for hcpd operators and expands hamiltonian term
!
!#######################################################################

      subroutine hcpdlab(ichan,np,hnptm,knptm,hkoe,htime,fdvr, &
                         mm,md,nr,chkdvr)

      implicit none

      integer(long), intent(in)                     :: ichan
      integer(long), intent(in)                     :: knptm,hnptm,hkoe,htime
      integer(long), dimension(maxdim), intent(out) :: fdvr
      integer(long), intent(in)                     :: chkdvr,mm,md,nr,np

      integer(long) :: p,k,knpot,k1,ndof1
      integer(long) :: f,f1,p1,ktot1,chkgrd,h,m,m1,n
      integer(long) :: ff,hl,k2,cpdrank
      integer(long), dimension(1)             :: optyp
      integer(long), dimension(maxdim,maxdim) :: spfdof1
      integer(long), dimension(maxdim)        :: mdvr,mmode,pntbas
      integer(long), dimension(maxdim)        :: nspfdof1,subdim1
      integer(long)                           :: ktot2,ktot3,koned,nmode1
      character(len=c2),dimension(maxkoe,maxdim) :: veclab,npotop
      logical(kind=4),dimension(maxdim)       :: loned
      logical(kind=4)                         :: lerr,used

!----------------------------------------------------------------------
! read information in the natpot file
! check the combinations of the degrees of freedom in mctdh
! and in potfit are consistent
!----------------------------------------------------------------------
      k=0
      chkgrd=3
      rewind(ichan)
      call cpdinfo(ichan,lerr,ndof1,fdvr,chkdvr,&
           chkgrd,0,mdvr,mmode,cpdrank,nspfdof1,spfdof1,nmode1,&
           subdim1,loned,pntbas)


      do m1=1,nmode1
         do n=1,nspfdof1(m1)
            f1=spfdof1(n,m1)
            do f=1,ndof
               if (fdvr(f1).eq.f) mdvr(m1)=dofspf(f)
            enddo
         enddo
      enddo


!-----------------------------------------------------------------------
! Check if the cpd-DOFs agree with the |i&j&k definition of the
! operator file.
! hnptm and knptm denote the h and k of the (not yet expanded) cpd
!-----------------------------------------------------------------------
      if (md.ne.0) then
         do n=1,ndof1                ! Loop over cpd DOFs
            f=mulddof2(n,md)         ! mctdh DOF
            f1=fdvr(n)               ! mapped cpd DOF
            if (f.ne.f1) goto 297
         enddo
      elseif (mm.ne.0)  then
         do n=1,ndof1                ! Loop over cpd DOFs
            f=spfdof(n,mm)           ! mctdh DOF
            f1=fdvr(n)               ! mapped cpd DOF
            if (f.ne.f1) goto 297
         enddo
      endif
! --- Re-define hamilton, remove zeros and set h=hnptm appropriately.
      do f=1,ndof
         if(hamilton(f,knptm) .eq. hnptm  .or. &
            hamilton(f,knptm) .eq. 0 ) hamilton(f,knptm) = f
      enddo
      do f=1,ndof
         if(hamilton(f,knptm) .ne. f) then
            h=hamilton(f,knptm)
            hl=hterm(1,h)
            if(hoplab(hl)(1:7) .eq. 'cpdfile'  .or. &
                 hoplab(hl)(1:8) .eq. 'hcpdfile'  .or. &
                 hoplab(hl)(1:6) .eq. 'natpot' .or. &
                 hoplab(hl)(1:6) .eq. 'potfit' ) then
               call logop
               write(6,'(/a,4i4,2x,a)') &
                     'k, h, hl, f', knptm,h,hl,f,hoplab(hl)
               routine='hcpdlab  (genoper/matchlab.F)'
               message='There must not be two natpots or cpds in one'// &
                       ' Hamiltonian-Section line.'
               call errormsg
            endif
         endif
      enddo
      do f=1,ndof
         if (fdvr(f).gt.0) hamilton(fdvr(f),knptm) = hnptm
      enddo

!-----------------------------------------------------------------------
! set up labels
! maxkoe is present size used in arrays, minkoe what it needs to be
!-----------------------------------------------------------------------

      do m=1,nmode1
         p = cpdrank
         do n=1,nspfdof1(m)
            f=spfdof1(n,m)
            if (loned(f)) then
               f1=fdvr(f)
               p = p + 1
               p = min(maxkoe,p+1)
               write(veclab(p,m),'(2(a,i7))') 'oned',f1,',',np
               call rmblnk(veclab(p,m))
            endif
         enddo
      enddo


      p1 = 0
      do p=1,cpdrank
         do m=1,nmode1
            p1 = p1 + 1
            write(veclab(p,m),'(2(a,i7))') 'hcpdtm',p1,',',np
            call rmblnk(veclab(p,m))
            !print*, "ssss ", trim(veclab(p,m))
         enddo
      enddo

!-----------------------------------------------------------------------
! set up hcpd operator
!-----------------------------------------------------------------------

      k=0
      do m=1,nmode1
         p = cpdrank
         do n=1,nspfdof1(m)
            f=spfdof1(n,m)
            if (loned(f)) then
               k=k+1
               do f1=1,ndof
                  npotop(f1,k) = '1'
               enddo
               f1=fdvr(f)
               p = p + 1
               npotop(f1,k)=veclab(p,m)

            endif
         enddo
      enddo

      koned = k

      if (cpdrank + koned .ge.maxkoe) goto 299

      k1 = k
      do p=1,cpdrank
         k1=k1+1
         do m=1,nmode1
            npotop(m,k1)=veclab(p,m)
         enddo
      enddo

      knpot=k1                  ! should be .eq. cpdrank

!-----------------------------------------------------------------------
! add new terms to ktot
! knpot is for the natpot operators
!-----------------------------------------------------------------------
      ktot1=ktot+1
      ktot2=ktot+koned
      ktot3=ktot2+1
      ktot=ktot+knpot
! calculate required array sizes
      minkoe=minkoe+knpot

      if (ktot .ge. maxkoe) ktot=maxkoe

      optyp(1)=3


      k=0
      do k1=ktot1,ktot2
         k=k+1
         do f=1,ndof1
            f1=fdvr(f)
!----------------------------------------------------------------------
! the 1-D    functions are stored as 1-D operators
! the order of the degrees of freedom in the Primitive-BASIS-SECTION
! can be different in the potfit and mctdh input files :
! this is why f (in potfit) can be different from f1 (in mctdh)
!----------------------------------------------------------------------
            call storetm(npotop(f1,k),1,optyp,f1,h,0,0)
            hamilton(f1,k1)=h
            hamnum(k1)= hamnum(knptm)
         enddo
      enddo


      k1=ktot3-1
      do k2=ktot3,ktot
         k=k+1
         k1=k1+1
         do m=1,nmode1
            m1=mmode(m)
            if (nspfdof1(m).eq.1) then
               f=spfdof1(1,m)
               f1=fdvr(f)
!----------------------------------------------------------------------
! if the degrees are not combined (nspfdof1(m) .eq. 1)
! the natpot operators are stored as 1-D operators
! the label associated with the mode can be different
! in potfit and in mctdh :
! this is why m (in potfit) can be different from m1 (in mctdh)
!---------------------------------------------------------------------
               call storetm(npotop(m,k),1,optyp,f1,h,0,0)
            else
               f=spfdof1(1,m)
               f1=fdvr(f)
!----------------------------------------------------------------------
! if the degrees are combined (nspfdof1(m) .gt. 1)
! the natpot operators are stored as particle operators
!---------------------------------------------------------------------
               call storetm(npotop(m,k),1,optyp,f1,h,0,m1)
!              for sub-particle operators, store number of DOFs
!              that it acts on
               if (nspfdof1(m).lt.nspfdof(m1)) then
                  htmndof(h)=nspfdof1(m)
                  maxsubpdim=max(maxsubpdim,subdim1(m))
               endif
            endif
!----------------------------------------------------------------------
! if the degrees are combined (nspfdof1(m) .gt. 1)
! for the k1 natpot operator and
! for the first degree of freedom f1 in the mode m1 hamilton(f1,k1)=h
! for the other degrees of freedom ff hamilton(ff,k1)=ff
!--------------------------------------------------------------------
            hamilton(f1,k1)=h
            do n=2,nspfdof1(m)
               f=spfdof1(n,m)
               ff=fdvr(f)
               hamilton(ff,k1)=ff
            enddo
            hamnum(k1)= hamnum(knptm)
         enddo ! do m=1,nmode1
      enddo    ! do k1=ktot3,ktot
      ktot=k1

!-----------------------------------------------------------------------
! copy hamilton term for any dofs not included in potential surface
!-----------------------------------------------------------------------
      do f1=1,ndof
         used=.false.
         do f=1,ndof1
            if (f1.eq.fdvr(f)) used=.true.
         enddo

         if (.not. used) then
            do k=ktot1,ktot
               hamilton(f1,k)=hamilton(f1,knptm)
            enddo
         endif
      enddo
!-----------------------------------------------------------------------
! set coefficients
!-----------------------------------------------------------------------
      do k=ktot1,ktot
         hamilton(fkoe,k)=hkoe
      enddo
      if(ftime.gt.0) then
         do k=ktot1,ktot
            hamilton(ftime,k)=htime
         enddo
      endif

      return

!-----------------------------------------------------------------------
! Error handling
!-----------------------------------------------------------------------

 299  continue
      minkoe = k + minkoe
      routine='npotlab (matchlab.F)'
      write(message,'(a,i9)') &
           'maxkoe is too small. Increase at least to:',minkoe
      call warnmsg
      return

 297  write(ilog,'(/a,/a,i3,a,i3,/a,i3,2x,a/,a,i3,2x, &
         & a,/a,i3,2x,a,3(/a),/2a,/2a,/a,/a)') &
       ' ################## ERROR ################################', &
       ' Inconsistent DOFs in cpd no. nr =', nr,', muld =',md, &
       ' original cpd DOF is: f =', n, modelabel(n), &
       ' mapped   cpd DOF is: f =',f1, modelabel(f1), &
       ' mctdh        DOF is: f =', f, modelabel(f), &
       ' The cpd modelabels must be consistent with the mctdh ones!', &
       ' Probably a wrong |i&j&k statement in the Hamiltonian-Section.', &
       ' Avoid to specify the DOF-numbers i&j&k of a cpd.', &
       ' Then the assignment of the DOFs is done solely through', &
       ' modelabels.',' Alternatively, set the keyword "ignore" ', &
       ' to ignore the modelabels ', &
       ' and to solely specify the DOF-numbers by i&j&k.', &
       ' --------------------------------------------------------------'
      write(ilog,'(a,i3,a,i3,a,i3)') ' muld =', md,',  mode =',mm, &
                                     ',  cpd no.: nr =', nr
      if(md .gt. 0) then
         write(ilog,'(a,i3,a,i3)') ' ndof1 =', ndof1, &
                                 ',  nmulddof =',nmulddof(md)
         write(ilog,'(a,50i3)') ' cpd    (fdvr) :', &
                                  (fdvr(n),n=1,ndof1)
         write(ilog,'(a,50i3)') ' MCTDH (mulddof2) :', &
                                  (mulddof2(n,md),n=1,ndof1)
      else
         write(ilog,'(a,i3,a,i3)') ' ndof1 =', ndof1, &
                                 ',  nspfdof =',nspfdof(mm)
         write(ilog,'(a,50i3)') ' cpd  (fdvr) :', &
                                  (fdvr(n),n=1,ndof1)
         write(ilog,'(a,50i3)') ' MCTDH (spfdof) :', &
                                  (spfdof(n,mm) ,n=1,ndof1)
      endif
      write(ilog,'(a)') &
       ' #########################################################'
      if(ilog.eq.6) call stopnow
      ilog=6
      goto 297

      end subroutine hcpdlab



!######################################################################## 
!  Save pdim,vpdim,npdim arrays which defined for Potfit modes,
!  to mdim,vmdim,nmdim (for MCTDH modes)
!######################################################################## 

      subroutine savepdim(fdvr,mdvr,nspfdof1,spfdof1,nmode1,n)

      implicit none

      integer(long)                                       :: m,m1,f,f1,zeig,n1
      integer(long), intent(in)                           :: n, nmode1
      integer(long), dimension(maxdim), intent(in)        :: fdvr,nspfdof1
      integer(long), dimension(maxdim), intent(out)       :: mdvr
      integer(long), dimension(maxdim,maxdim), intent(in) :: spfdof1

!.....calculate mdvr(m): the analogy matrix between Potfit and MCTDH modes
      do m1=1,nmode1
         do n1=1,nspfdof1(m1)
            f1=spfdof1(n1,m1)
            do f=1,ndof
               if (fdvr(f1).eq.f) mdvr(m1)=dofspf(f)
            enddo
         enddo
      enddo

      mmodc(n)=mdvr(modc)

      call zerovxi(mdim(:,n),maxdim)
      call zerovxi(vmdim(:,n),maxdim)
      call zerovxi(nmdim(:,n),maxdim)

!.....calculate mdim(m,n)
      do m=1,nmode
         mdim(m,n)=1
         do m1=1,nmode1
            if (mdvr(m1).eq.m) mdim(m,n)=mdim(m,n)*pdim(m1)
         enddo
      enddo

!........calculate vmdim(m) and nmdim(m)
      zeig=1
      do m=1,nmode
         if (m.ne.mmodc(n)) then
            vmdim(m,n)=zeig
            zeig=zeig*mdim(m,n)
         else
            vmdim(m,n)=1
         endif
      enddo
      do m=1,nmode
         if (m.ne.mmodc(n)) then
            nmdim(m,n)=zeig/(mdim(m,n)*vmdim(m,n))
         else
            nmdim(m,n)=1
         endif
      enddo

      return
      end subroutine savepdim

!#######################################################################

      subroutine setmp(ii,np,nmode1,suma)

! --- The array mii contains the mode indices (minus 1) for
! --- the total potfit D-vector index ii.
! --- This routine is similar to setm on propwf/getdavmat.F

      implicit none

      integer(long), intent(in) :: ii, nmode1, np
      integer(long)             :: m, k, mii
      real(dop), intent(out)    :: suma

      k = ii - 1
      suma=0.d0
      do m = nmode1, 1, -1
         if(m.ne.modc) then
            mii = k/vpdim(m)
            k = k - mii*vpdim(m)
            suma=suma+mii*wpf(m,np)
         endif
      enddo

      return
      end

!=======================================================================
      subroutine mlpfinfo(iop,nr)
!=======================================================================

      implicit none

      integer(long), intent(in) :: iop,nr
      real(dop)                 :: fver
      integer(long)             :: chkdvr,ndof1,f,f1,n
      integer(long),dimension(maxdim) ::  fdvr

      routine = "mlpfinfo"
      read(iop) fver
      if (fver .lt. 8.5001d0) then
         message = "unexpected file version"
         call warnmsg
      endif
      filever(iop) = fver

      call zerovxi(chkerr,nerr)
      chkdvr = 3
      call rddvrdef(iop,chkdvr,ndof1,fdvr)
! remove feb from fdvr list - not used in MPLF definition
      n=ndof1
      do f=1,n
         if (fdvr(f) .eq. feb) then
            do f1 = f,n-1
               fdvr(f1) = fdvr(f1+1)
            enddo
            fdvr(ndof1) = 0
            ndof1 = ndof1-1
     
            exit
         endif
      enddo

      call sv_mlpf_read(iop,nr,ndof1,fdvr,gdim)

      return
      end subroutine mlpfinfo

!=======================================================================
      subroutine mlpflab(k,mlpfnr,lerr)
!=======================================================================

      implicit none

      integer(long), intent(in)                    :: k
      integer(long), dimension(maxdim), intent(in) :: mlpfnr
      logical(long), intent(out)                   ::  lerr
      integer(long), dimension(maxdim) :: nkterm,mmlop
      integer(long), dimension(maxdim) :: imlop
      integer(long)   :: f,nr,md,nd,h,totmlop
      integer(long)   :: addk,k1,kk,m,n,l1,af,nf,xf,xm,xn,d1,d2,d3,i
      integer(long), dimension(1)      :: optyp
      logical(kind=4) :: lmatch
      character(len=c2) :: mlnplab

      lerr = .false.
      nkterm(:) = 1
      imlop(:) = 0
      ! Check that the |i&j&k specification, where given, matches the MLPF DOFs.
      do f=1,ndof
         nr = mlpfnr(f)
         if (nr.ne.0) then
            h = hamilton(f,k)
            ! Does this term have a |i&j&k specification?
            md = htmmuld(h)
            if (md.ne.0) then
               ! Yes, so check that it matches the MLPF DOFs. Order doesn't matter.
               nd = nmulddof(md)
               call sv_mlpf_check_dofs(nr,nd,mulddof(1,md),lmatch)
               if (.not.lmatch) then
                  write(*,'(a)') "MLPF DOFs don't match system. "// &
                     "Check the |i&j&k definition in the operator file."
                  write(*,'(a,i0,a,i0)') "mlpfnr = ",nr,"  muld = ",md
                  lerr = .true.
                  return
               endif
            endif
            ! Convert MLPF to MLOp.
            call mlpf_to_mlop(nr,nspfdof,spfdof,nkterm,imlop)
         endif
      enddo

      ! Check that no mode is affected by more than one MLPF.
      mmlop(:) = 0
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            i=imlop(f)
            if (i.ne.0) then
               if (mmlop(m).eq.0) then
                  mmlop(m)=i
               else
                  if(mmlop(m).ne.i) then
                     write(*,'(a)') "More than one MLOp acting on mode!"
                     write(*,'(a,i0,a,i0,a,i0)') &
                        "m = ",m,"  imlops = ",mmlop(m),",",i
                     lerr = .true.
                     return
                  endif
               endif
            endif
         enddo
      enddo

      ! Rewrite the Hamiltonian line.
      do f=1,ndof
         if (mlpfnr(f).ne.0 .or. hamilton(f,k).eq.0) then
            hamilton(f,k) = f
         endif
      enddo

      ! Create new kterms, for use by the MLOp leaves.
      addk = maxval(nkterm)
      do k1=1,addk
         ! New kterm has number ktot+k1.
         kk = min(maxkoe,ktot+k1)
         do m=1,nmode
            do n=1,nspfdof(m)
               f = spfdof(n,m)
               if (k1.gt.nkterm(f)) cycle
               h = hamilton(f,k)
               if (imlop(f).ne.0) then
                  ! Query MLOp for information about system mode m:
                  ! there must be a leaf which lies in the mode => l1
                  ! first system DOF in leaf => af
                  ! number of DOFs in leaf => nf
                  call mlpf_modeinfo(imlop(f),m,l1,af,nf)
                  if (l1.eq.0) then
                     lerr = .true.
                     return
                  endif
                  ! Create a new Hamiltonian term, but only if this
                  ! DOF is the first DOF in the MLPF leaf.
                  if (af.ne.f) goto 100
                  ! Figure out the "shape" of the Hamiltonian term.
                  if (nf.eq.1) then
                     ! 1D operator
                     xf = f
                     xm = 0
                     xn = 0
                  elseif (nf.eq.nspfdof(m)) then
                     ! full mode operator
                     xf = f
                     xm = m
                     xn = 0
                  else
                     ! partial mode (sub-particle) operator
                     xf = f
                     xm = m
                     xn = nf
                  endif
                  ! Invent a label.
                  write(mlnplab,'(a,i0,a,i0,a,i0)') "mlnp",imlop(f),",",l1,",",k1
                  ! Store the new term.
                  optyp(1)=3
                  call storetm(mlnplab,1,optyp,xf,h,0,xm)
                  htmndof(h) = xn
                  ! Increase maxsubpdim if necessary.
                  call getvgn(h,d1,d2,d3)
                  maxsubpdim = max(maxsubpdim,d2)
               endif
 100           hamilton(f,kk) = h
            enddo
         enddo
         ! Copy Hamiltonian number.
         hamnum(kk) = hamnum(k)
         ! Mark kterm as MLOp.
         hmlop(kk) = -1
      enddo

      ! Store coefficient in the 1st of the new kterms.
      kk = min(maxkoe,ktot+1)
      hamilton(fkoe,kk) = hamilton(fkoe,k)
      if (ftime.gt.0) hamilton(ftime,kk) = hamilton(ftime,k)

      ! The MLOps created so far have the same tree structure as the MLPFs
      ! that they have been created from. The original operator line might
      ! include several MLPFs, as well as modes that no MLPF acts on; but
      ! eventually we need a "total MLOp" that acts on all modes, and that
      ! has the same tree structure as the ML-wavefunction. Solution:
      ! a) create (simple) MLOps for uncovered mode operators
      ! b) multiply out all MLOps that were created for the original op.line
      ! But we can only do this after the final numbering of the k-terms is
      ! known, i.e. (much) later. Here, we only remember which MLOps have
      ! been created, and queue the job for later.
      call sv_mlop_enqueue(totmlop,ndof,imlop,hamnum(kk))
      hmlop(kk) = totmlop

      ! Increase number of kterms.
      minkoe = minkoe + addk
      ktot = min(ktot + addk, maxkoe)

      end subroutine  mlpflab

!***********************************************************************
!
! SUBROUTINE FINDLABELS
!
! Finds the labels of those natpots for which FAST or NATPOTCUT were
! specified, and assigns the parameters given to the corresponding
! variables.
!
! This code used to be part of EINOPER (which reads the OPERATOR-SECTION
! of the input file), but some labels might only be defined after the
! operator file has been read (which happens in HEINGABE), so I put
! the code into a subroutine which then gets called after/during HEINGABE.
!
! SS 2004, FO 02/2005
!
!***********************************************************************

      subroutine findlabels

      implicit none

      integer(long)       :: np,n,length,l1,l2,l,ierr,i1,i

!-----------------------------------------------------------------------
! find the labels defined for FAST and natpotcut in the list of natpots
!-----------------------------------------------------------------------
      np=0
      do n=1,min(ndef,maxdef)
         length=index(deflabop(n),' ')-1
         l1=0
         l2=0
         do l=1,length
            if (deflabop(n)(l:l).eq.'{') l1=l
            if (deflabop(n)(l:l).eq.'}') l2=l
         enddo
         if (l1.ne.0) then
            if (deflabop(n)(1:l1-1).eq.'natpot') then
               read(deflabop(n)(l1+1:l2-1),*,iostat=ierr) np
               if (lncutall) then
                  npotcut(np)=ncutrlabel(ncutn)
               else
                  i1=0
                  do i=1,ncutn
                     if (ncutlabel(i).eq.deflabel(n)) i1=i
                  enddo
                  if (i1.ne.0) then
                     npotcut(np)=ncutrlabel(i1)
                  endif
               endif
            endif
         endif
      enddo

      return
      end subroutine findlabels

!#######################################################################

      subroutine lhalabgen(h,mulddof,nmulddof)

      implicit none

      integer(long), intent(in)                      :: nmulddof,h
      integer(long), dimension(nmulddof), intent(in) :: mulddof
      integer(long)                                  :: k,n,n1,f,f1, &
                                                        hl,hl1,hl2,h1,&
                                                        idx,jdx,kdx,hls,&
                                                        m,m1
      integer(long), dimension(1)                    :: iterm
      character(len=c2)                              :: label,lab0,slabel
      integer(long)                                  :: lablen

! set up required labels
! koeff will get label oplab%0, oplab%1_f or oplab%2_f_f1 for the
! different moments.
      label='q'
      call hlabnum(hl1,label)
      label='q^2'
      call hlabnum(hl2,label)

      hl=hterm(1,h)
      lab0=hoplab(hl)
      lablen=index(lab0,' ')-1

! Work out to which state(s) the QC operator applies.
! This information is then added to the newly generated operators to ensure
! that only the appropriate states are included (via hamilton(feb,k)).
      idx = index(lab0,'&')
      jdx = index(lab0,' ')
      kdx = index(lab0,'m')
      slabel = ' '
      if(idx.eq.0)then
         if(nddstate.eq.1)then
            slabel = 'S1&1'
         else
            slabel = 'S'//lab0(kdx+1:jdx-1)//'&'//lab0(kdx+1:jdx-1)
         endif
      else
         slabel = 'S'//lab0(kdx+1:idx-1)//'&'//lab0(idx+1:jdx-1)
      endif      
      call hlabnum(hls,slabel)    
      iterm(1) = hls
      call htmnum(hls,iterm,1,feb,0,0)

! scalar term
      ktot=ktot+1
      k=min(ktot,maxkoe)
      label=lab0(1:lablen)//'%0'
      call hlabnum(hl,label)
      iterm(1) = hl
      call htmnum(h1,iterm,1,fkoe,0,0)
      htmtype(h1)=104
      hamilton(fkoe,k) = h1
      hamilton(feb,k) = hls
      hamnum(k)= 1

! linear terms
      do n=1,nmulddof
         f = mulddof(n)
         if (f .eq. 0) cycle  ! DOF not in dynamics
         if (basis(f) .eq. 19) cycle
         ktot=ktot+1
         k=min(ktot,maxkoe)

         write(label,'(2a,i5)') lab0(1:lablen),'%1_',f
         call rmblnk(label)
         call hlabnum(hl,label)
         iterm(1) = hl
         call htmnum(h1,iterm,1,fkoe,0,0)
         htmtype(h1)=104
         hamilton(fkoe,k) = h1

         iterm(1) = hl1
         call htmnum(h1,iterm,1,f,0,0)
         hamilton(f,k) = h1
         hamilton(feb,k) = hls
         hamnum(k)= 1
      enddo

! quadratic terms
      do n=1,nmulddof
         f = mulddof(n)
         if (f .eq. 0) cycle  ! DOF not in dynamics
         if (basis(f) .eq. 19) cycle
         m=dofspf(f)

         ktot=ktot+1
         k=min(ktot,maxkoe)

         write(label,'(2a,i5,a1,i5)') lab0(1:lablen),'%2_',f,'_',f
         call rmblnk(label)
         call hlabnum(hl,label)
         iterm(1) = hl
         call htmnum(h1,iterm,1,fkoe,0,0)
         htmtype(h1)=104
         hamilton(fkoe,k) = h1

         iterm(1) = hl2
         call htmnum(h1,iterm,1,f,0,0)
         hamilton(f,k) = h1
         hamilton(feb,k) = hls
         hamnum(k)= 1

! bilinear terms
         do n1=n+1,nmulddof
            f1 = mulddof(n1)
            if (f1 .eq. 0) cycle  ! DOF not in dynamics
            if (basis(f1) .eq. 19) cycle
            m1=dofspf(f1)

            ktot=ktot+1
            k=min(ktot,maxkoe)

            write(label,'(2a,i5,a1,i5)') lab0(1:lablen),'%2_',f,'_',f1
            call rmblnk(label)
            call hlabnum(hl,label)
            iterm(1) = hl
            call htmnum(h1,iterm,1,fkoe,0,0)
            htmtype(h1)=104
            hamilton(fkoe,k) = h1

            iterm(1) = hl1
            call htmnum(h1,iterm,1,f,0,0)
            hamilton(f,k) = h1
            iterm(1) = hl1
            call htmnum(h1,iterm,1,f1,0,0)
            hamilton(f1,k) = h1
            hamilton(feb,k) = hls
            hamnum(k)= 1
         enddo
      enddo

      return
      end subroutine lhalabgen

      end module matchlabmod


