!#######################################################################
!
!                 RUNOPER
!
!#######################################################################

      subroutine runoper

use global
use globmemmod
use maxv
use dvrdatmod
use griddatmod
use operdef
use dirdyn
use runopermod
use channels
use gapmod

use timing

use einsbasmod,  only: eingrd
use einopermod,  only: einoper,inoper,predefined
use einparamod
use eindirdynmod, only: ddoper,ddmctoper
use rddvrmod

use hintern,     only: rdops
use heinsort,    only: sumhtms,sortktms,sortmode,sortmuld,sortanal, &
                       delzero,delke,delkepnt,delkg,sortlha
use zeigopermod, only: zeighop
use htmsetupmod, only: htmsetup,gethtmtype
use findknummod, only: findknum,setkmlop
use logopmod,    only: logop
use wropermod,   only: wroper
use op1lib,      only: zerovxl
use opermod_interface
use realwpmod

use logdat

      implicit none

      logical(kind=4)                            :: loperr
 
      character(len=80)                          :: actual
      character(len=12)                          :: cerr

      integer(long)                              :: pass,f,i,ii,chkdvr,&
                                                    ierr,ilbl,m,icount
      integer(long)                              :: oldhtm,oldhop,olddef,&
                                                    oldrdf,oldfac,nham
      integer(long), dimension(:), allocatable   :: done
      character(len=16)                          :: progpart
      integer(long), dimension(1)                :: idum
      logical(kind=4)                            :: check,lerr
      complex(dop), dimension(1)                 :: zdum,zdum1,zdum2,zdum3

      character(len=c5)                          :: filename
      integer(long), save                        :: tid=0 ! timer ID
      integer(long), save                        :: tid1=0 ! timer ID

      integer(long), external   :: integersize,realsize,complexsize,logicalsize

      ierr = 0

!-----------------------------------------------------------------------
! Initialise and Start timing
!-----------------------------------------------------------------------
      call timinit
      call get_timer(tid, 'GenOPER')
      call start_timer(tid)

      call fzeit(actual)
      ii = index(actual,':')
      actual=actual(ii-9:ii+6)
      write(ilog,'(/,72a1)')  ('#',ii=1,72)
      if (lrunpes) then
         write(ilog,'(/,a)') ' Creating PES file: '//operfile(1:opflaenge)
      else 
         write(ilog,'(/,a)') ' Creating OPER file: '//operfile(1:opflaenge)
      endif
      write(ilog,'(57x,a)')  actual(1:15)
      write(ilog,'(72a1)')  ('#',ii=1,72)

!-----------------------------------------------------------------------
! get basic dimensions (e.g. maxdim, maxsta)
!-----------------------------------------------------------------------
      inquire(idvr,opened=check)
      if (check) close(idvr)
      filename=dname(1:dlaenge)//'/dvr'
      ilbl=index(filename,' ')-1
      open(idvr,file=filename(1:ilbl),form='unformatted',status='old',err=900)
      call rdmemdim(idvr)
      close(idvr)

!-----------------------------------------------------------------------
! Allocate mememory and read system / DVR information
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat

      filename=dname(1:dlaenge)//'/dvr'
      ilbl=index(filename,' ')-1
      open(idvr,file=filename,form='unformatted',status='old',err=900)
      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      close(idvr)

      if(.not.allocated(done))allocate(done(maxhop))
      done(:)=0

!-----------------------------------------------------------------------
! Check input file open for reading
!-----------------------------------------------------------------------
      inquire(iin,opened=check)
      if (check) close(iin)
      filename=inputfile(1:inplaenge)
      open(iin,file=filename,status='old',form='formatted',err=900)

!-----------------------------------------------------------------------
! find where operator is defined
!-----------------------------------------------------------------------
      call inoper

!-----------------------------------------------------------------------
! set up list of predefined operators
!-----------------------------------------------------------------------
      call predefined

!-----------------------------------------------------------------------
! read in parameters
!-----------------------------------------------------------------------
      call einpara

!-----------------------------------------------------------------------
! Read / generate grids 
!-----------------------------------------------------------------------
      call eingrd(1)

!-----------------------------------------------------------------------
! Set apparent no. of states for calculation
!-----------------------------------------------------------------------
      if (lmult) then
         nstate = nddstate
      else
         nstate = 1
      endif

!-----------------------------------------------------------------------
! Set lvmcg flag if only 1 set of multi-dimensional GWPs
!-----------------------------------------------------------------------
      if (lgwp) then
         if (nmode .eq. 1 .or. (nmode .eq. 2 .and. leb)) then
            lvmcg = .true.
         endif
      endif

!-----------------------------------------------------------------------
! Read / generate operator information
!
! first+second pass is to calculate dimensions of arrays
!  (2 passes needed as have to fill arrays to set up all labels and
!   terms so first pass overestimates, second pass gets numbers)
! final pass is to fill arrays
!
! initial guess for maxhtm, maxhop to allow use of hoplab and hterm
! arrays in initial estimate. maxhop >=3 to allow initialisation....
! After calling einoper all dimensions are final except maxhtm and 
! maxhop that may be changed in later sort routines
!-----------------------------------------------------------------------
      pass = 1 
      maxhop=2000
      maxhtm=1000
      maxdef=500
! provide space to build up mode operators in heinmode
      maxfac = 1
      do m=1,nmode
         maxfac=max(maxfac,nspfdof(m))
      enddo
      do 
         oldhtm=maxhtm
         oldhop=maxhop
         olddef=maxdef
         oldrdf=maxrdf
         oldfac=maxfac
         dercpdim = maxsta*(maxsta-1)/2   ! no. of drivative couplings
         call alloc_operdef
         call alloc_runopermod
         if (ldd .or. lgwptrj .or. lddmctdh .or. lddtrans .or. ltraj) then
            call alloc_dirdyn
            call alloc_dddb
            call alloc_dercp
         endif
         if(lddmctdh .or. ldd)then
            call alloc_gapkernel
            if(ldiabdb) call alloc_dirdyn_ddmctdh
         endif
         call alloc_realwpmod
         call einoper(pass)
         if ((maxhtm .eq. oldhtm) .and. (maxhop .eq. oldhop) .and. &
             (maxdef .eq. olddef) .and. (maxrdf .eq. oldrdf) .and. &
             (maxfac .eq. oldfac)) exit
      enddo
      pass = 2 
      call alloc_operdef
      call alloc_runopermod
      if (ldd .or. lgwptrj .or. lddmctdh .or. lddtrans) then
         call alloc_dirdyn
         call alloc_dddb
         call alloc_dercp
      endif
!      if(lddmctdh)call alloc_gapop
      call alloc_realwpmod

      call einoper(pass)

!-----------------------------------------------------------------------
! Close input file
!-----------------------------------------------------------------------
      inquire(iin,opened=check)
      if (check) close(iin)

!-----------------------------------------------------------------------
! Now generate operator
!-----------------------------------------------------------------------
      call memoper
#ifdef DEBUG
      mcdim = mcdim + 2
      mrdim = mrdim + 2
#endif
      if (lBEL) write(1000,*)"lBEL",lBEL,"source/genoper/runoper.F90"
      if(allocated(mc))deallocate(mc)
      if(allocated(mr))deallocate(mr)
      if(allocated(mi))deallocate(mi)
      if(allocated(ml))deallocate(ml)

      cerr = 'pesoper_mc  '
      allocate(mc(mcdim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + mcdim*complexsize()
      cerr = 'pesoper_mr  '
      allocate(mr(mrdim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + mrdim*realsize()
      cerr = 'pesoper_mi  '
      allocate(mi(midim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + midim*integersize()
      cerr = 'pesoper_ml  '
      allocate(ml(mldim),stat=ierr)
      if (ierr > 0) go to 1000
      allocmemory=allocmemory + mldim*logicalsize()

!-----------------------------------------------------------------------
! write memory usage to log
!-----------------------------------------------------------------------
      progpart='runoper'
      call logmem(progpart)

      loperr=.false.

      routine = 'Runoper'

      open(itmpop,file=name(1:laenge)//'/scratch',form='unformatted', &
           status='unknown',err=999)

!-----------------------------------------------------------------------
! Check if mass(f) are set.
!-----------------------------------------------------------------------
      icount=0
      do f = 1, ndof
         if(mass(f) .le. 0.0_dop)  then
            mass(f) = 1.0_dop
            if((basis(f).eq.1 .or.  basis(f).eq.3 .or. basis(f).eq.4 &
                   .or. basis(f).eq.5 .or. basis(f).eq.11 &
                   .or. (basis(f).ge.14 .and. basis(f).le.18) .or. basis(f).ge.20 &
                   .or. basis(f).eq.31 .or. basis(f).eq.32)  &
                   .and. (basis(max(1,f-1)).ne.9 .and.  &
                          basis(max(1,f-1)).ne.21 .and. &
                          basis(max(1,f-1)).ne.33 .and. &
                          basis(max(1,f-2)).ne.33)) then 
               icount=icount+1
               if(icount.le.10) then
                  write(ilog,'(2a,i4,a)') 'WARNING: ', &
                       'Mass for DOF',f,' not set. Set to 1 by default.'
               endif
            endif
        endif
      enddo

!-----------------------------------------------------------------------
! Read DVR data needed to build operator (see rddvr.f for details)
!-----------------------------------------------------------------------
      dvrdata(:) = .false.
      dvrdata(1)=.true.
      dvrdata(2)=.true.
      dvrdata(3)=.true.
      dvrdata(4)=.true.
      dvrdata(8)=.true.
      dvrdata(9)=.true.
      dvrdata(10)=.true.
      chkdvr=2
      call rddvr(mr(mrort),mr(mrtrafo),mr(mrdvrmat),mr(mrfftp),zdum, &
           zdum1,idum,zdum2,zdum3,mi(mijsph),mi(mimsph),mr(mrkinsph), &
           chkdvr)

!-----------------------------------------------------------------------
! read operators from files and store in rdop
!-----------------------------------------------------------------------
      call rdops(mr(mrrdop),mr(mrort))

!-----------------------------------------------------------------------
! MLOps can only be used with ML-MCTDH
!-----------------------------------------------------------------------
      if (lmlop.and..not.lrunml) then
         routine = "runoper"
         message = "Multi-Layer operators used with non-ML wavefunction"
         call errormsg
      endif

!-----------------------------------------------------------------------
! find operator types
!-----------------------------------------------------------------------
      call gethtmtype(mr(mrort),mr(mrdvrmat),mr(mrfftp), &
           mr(mrkinsph),mr(mrrdop),mr(mrtrafo),done)

      call get_timer(tid1, 'CalcOPs')
      call start_timer(tid1)
 
!-----------------------------------------------------------------------
! For PES, delete non-diagonal terms
!-----------------------------------------------------------------------
      if (lrunpes) then
         if(gmatf1.eq.0) then 
            call delke
         else
            call delkg(gmatf1,gmatf2)
         endif
      endif

!-----------------------------------------------------------------------
! If there is a point DVR delete any KE terms involving these DOFs
!-----------------------------------------------------------------------
      if (lpntdvr .or. ltpntdvr) call delkepnt

!-----------------------------------------------------------------------
! sum 1D uncorrelated operator terms where possible
!-----------------------------------------------------------------------
      pass=1
      call sumhtms(pass)

!-----------------------------------------------------------------------
! find operator types for any new (summed) operators
!-----------------------------------------------------------------------
      call gethtmtype(mr(mrort),mr(mrdvrmat),mr(mrfftp), &
           mr(mrkinsph),mr(mrrdop),mr(mrtrafo),done)

!-----------------------------------------------------------------------
! sum 1D correlated operator terms where possible
!-----------------------------------------------------------------------
      pass=2
      call sumhtms(pass)

!-----------------------------------------------------------------------
! pack coefficients into operators if possible
!-----------------------------------------------------------------------
      call sortktms

!-----------------------------------------------------------------------
! find operator types for any new (summed) operators
!-----------------------------------------------------------------------
      call gethtmtype(mr(mrort),mr(mrdvrmat),mr(mrfftp), &
           mr(mrkinsph),mr(mrrdop),mr(mrtrafo),done)

!-----------------------------------------------------------------------
! sort out mode potential terms
!-----------------------------------------------------------------------
      call sortmode

!-----------------------------------------------------------------------
! sort out grid terms
!-----------------------------------------------------------------------
      call sortmuld

!-----------------------------------------------------------------------
! find operator types for any new (product) operators
!-----------------------------------------------------------------------
      call gethtmtype(mr(mrort),mr(mrdvrmat),mr(mrfftp), &
           mr(mrkinsph),mr(mrrdop),mr(mrtrafo),done)

!-----------------------------------------------------------------------
! sum operator terms where possible
!-----------------------------------------------------------------------
      pass=3
      call sumhtms(pass)

!-----------------------------------------------------------------------
! sort analytic potentials
!-----------------------------------------------------------------------
      call sortanal

!-----------------------------------------------------------------------
! Flag any terms to be treated using an LHA
!-----------------------------------------------------------------------
      if (lgwp) call sortlha

!-----------------------------------------------------------------------
! find operator types for any new (summed) operators
!-----------------------------------------------------------------------
      call gethtmtype(mr(mrort),mr(mrdvrmat),mr(mrfftp), &
           mr(mrkinsph),mr(mrrdop),mr(mrtrafo),done)

!-----------------------------------------------------------------------
! set up hterm operators, and write to tmp file 
!-----------------------------------------------------------------------
      call htmsetup(mr(mrort),mr(mrdvrmat),mr(mrfftp), &
           mr(mrkinsph),mr(mrrdop),mi(mijsph),mi(mimsph),mr(mrtrafo),done)

!-----------------------------------------------------------------------
! Close readsrf files
!-----------------------------------------------------------------------
      i = 170
 231  i = i + 1
      inquire(i,opened=check)
      if (check) then
         close(i)
         goto 231
      endif

!-----------------------------------------------------------------------
! delete any terms with zero coefficient or zero operator
!-----------------------------------------------------------------------
      call delzero

!-----------------------------------------------------------------------
! Look to see if this a DD calculation and set up DB
!-----------------------------------------------------------------------
      if(lddmctdh)then
         call ddmctoper
      else
         call ddoper
      endif

!-----------------------------------------------------------------------
! order hamilton
! find kzahl, khzahl, kunzahl, nmulpot, ...
!-----------------------------------------------------------------------
      call findknum

!-----------------------------------------------------------------------
! set final kterm numbers for multi-layer operators, and
! create final "total" MLOps
!
! The tape must now be read in from the input file.
!-----------------------------------------------------------------------
      if (lmlop) then
         call setkmlop

         if (allocated(tape)) deallocate(tape)
         allocate(tape(maxtape))
         call eingrd(3)

         do nham=1,hamdim
            call sv_mlop_finalize(nham,tape)
         enddo
      endif

!-----------------------------------------------------------------------
! stop in error occurred while reading operator input
!-----------------------------------------------------------------------
      if (loperr) then
         routine = 'MCTDH (file.op)'
         message = 'Finished with operator input error. See log file.'
         call errormsg
      endif

!-----------------------------------------------------------------------
! calculate length of hops array and zhop pointers
!-----------------------------------------------------------------------
      call zeighop

!-----------------------------------------------------------------------
! write oper file
! ird file contains 1D operators, ird1 file contains mD operators
!-----------------------------------------------------------------------
      call wroper

!-----------------------------------------------------------------------
! log operator information
!-----------------------------------------------------------------------
      call logop

!-----------------------------------------------------------------------
! close .op file and scratch file
!-----------------------------------------------------------------------
      if (iop .ne. iin) close (iop)
      close(itmpop,status='delete')

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_dirdyn
      call dealloc_dddb
!      if(lddmctdh.and.ldiabdb) call alloc_dirdyn_ddmctdh
      call dealloc_dirdyn_ddmctdh
!      call dealloc_gapop
      call dealloc_realwpmod
      call dealloc_runopermod
      call dealloc_dercp

      if(allocated(done))deallocate(done)

      cerr = 'pesoper_... '
      deallocate(mc,mr,mi,ml,stat=ierr)
      if (ierr > 0) go to 1001

!-----------------------------------------------------------------------
! finish timing
!-----------------------------------------------------------------------
      call stop_timer(tid)
      call stop_timer(tid1)
      if (ltime) then
         call timout
         close(itim)
      endif

      return

  999 continue
      message = 'Cannot open scratch file'
      call errormsg

  900 continue
      routine='Runoper'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop

 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      stop

 1001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
      stop


      end subroutine runoper


