!#######################################################################
! contains: wroper,wr1dop,wr2dop,wropinfo,wroptit,cvmcgtrans,cvmcgtrans1,
! rdddtrinp,rdddtrfile,wrddtrans,rdpoints,chkatord,rddreffile,
! rdcoinfile
!#######################################################################

      module wropermod

use decimal, only: long, dop
use versions, only: progver, getrevno, quanticsver
use keyunitsmod
use logdat
use mmlib,   only: mmtxdd1, mmxxdd1, qqtxdz, qqtxdd
use xvlib,   only: xvxxddo, xvixddo, qvtxdd1
use op1lib,  only: cpvxd, zeromxd, cpmxd, zerovxd, cpvxdz
use dirdyn
use gwptrj

use global
use globmemmod
use maxv
use dvrdatmod
use griddatmod
use operdef
use runopermod
use channels

use rdopermod, only: assocdof
use combined
use realwpmod

use gapmod

use iodvrdef, only: wrdvrdef
use iogrddef, only: wrgrddef
use iopsidef, only: tapelength
use dd_db, only: ddx2q,eckart,eckart_reord,ddf2q,prepsym
use ioqc
use readqc, only: getinam,getmass,getatnam

use directdyn, only: init_qchem_calculations,finalise_qchem_calculations,&
                     distgwp,mksympesa1,addsoc,genhess0
use diabatmod
use gapfuncsmod
use dd_db, only: dddb_wr1, dddb_wrmem, hessian_update1,distdb,hestrans
use mlham, only: mlham_dump
use lalib, only: simtranbd

use module_interface


      implicit none
      private
      public :: wroper,cvmcgtrans,rdddtrinp,rdddtrfile
      contains

!#######################################################################
!
!#######################################################################

      subroutine wroper

      implicit none

      integer(long)                            :: f,h,m,g,typ,muld,&
                                                  f1,m1,sdm,f2,totsdm,&
                                                  nel
      character(len=8)                         :: stat
      real(dop)                                :: rnel
      real(dop), dimension(:), allocatable     :: hopd
      complex(dop), dimension(:), allocatable  :: hopz
      character(len=20)                        :: cerr
      integer(long)                            :: ierr
      character(len=c2)                        :: wrtag
      logical                                  :: lddlvc = .false.

      routine='Wroper'

      if (lovwr) then
         stat='unknown'
      else
         stat='new'
      end if

      open(ioper,file=operfile(1:opflaenge),form='unformatted', &
           status=stat,err=999)

!-----------------------------------------------------------------------
! rewind scratch file containing operators
!-----------------------------------------------------------------------
      rewind(itmpop)

!-----------------------------------------------------------------------
! if transformed direct dynamics, read in and process transformation
! information. (needs to be done before wropinfo)
!-----------------------------------------------------------------------
      if (ddcoo .eq. 0) then
         call cvmcgtrans
      else if (ddtrfile .eq. 'inp ') then
         call rdddtrinp
      else if (lddtrans) then
         lddlvc = .true.
         call rdddtrfile
      endif

      if (ltraj) then
         if (tshtrfile .eq. 'lvc ' .or. lddlvc) then
            call tshtrlvc
         else if (tshtrfile .eq. 'inp ') then
            call rdtshtrinp
         else 
            call genxcooinfo
         endif
      endif

!-----------------------------------------------------------------------
! For DD, we can take the adiabatic surfaces and rediabatise 
! them using the method of choice.
!-----------------------------------------------------------------------
      if (lrediab) call rediabatise

!-----------------------------------------------------------------------
! For DD, convert an existing DB to a GAP DB
!-----------------------------------------------------------------------
      if (ldb2gapot) call dbtogapot

!-----------------------------------------------------------------------
! write operator information to file
!-----------------------------------------------------------------------
      call wropinfo

!-----------------------------------------------------------------------
! get full grid dimension to allow cutting potential terms
!-----------------------------------------------------------------------
      totsdm=1
      do m=1,nmode
         if (m .eq. meb) cycle

         totsdm = totsdm*subdim(m)
 ! check for overflow
         totsdm=min(totsdm,2000000000)
      enddo

!-----------------------------------------------------------------------
! write operator grid representations
!-----------------------------------------------------------------------
   10 continue
      read(itmpop,end=100) h

!-----------------------------------------------------------------------
! get info on associated dof/mode
!-----------------------------------------------------------------------
      call assocdof(h,f,m,muld,f1,m1,sdm)

      read(itmpop,end=100) typ

      write(wrtag,'(a,i10,a)') '<hops',h,'>'
      write(ioper) wrtag
      if (typ .eq. 0) then
         cerr = 'hopz '
         allocate(hopz(sdm*sdm),stat=ierr)
         if (ierr > 0) go to 1000
         read(itmpop) (hopz(g),g=1,sdm*sdm)
         if (zhop(h).ne.0) call wr1dop(hopz,htmtype(h),sdm,ioper)

      else if (typ .eq. 1) then
         if (htmtype(h).eq.11 .and.m1.ne.0) then
            f2=htmadof(h)
            sdm=vgdim(f2+1)*ngdim(f2+1)
         endif
         cerr = 'hopd '
         allocate(hopd(sdm),stat=ierr)
         if (ierr > 0) go to 1000
         read(itmpop) (hopd(g),g=1,sdm)

!-----------------------------------------------------------------------
! cut potential terms if requested.
!-----------------------------------------------------------------------
         if (sdm .eq. totsdm) then
            if (lcutpes) then
               do g=1,sdm
                  hopd(g) = min(pesmax,hopd(g))
                  hopd(g) = max(pesmin,hopd(g))
               enddo
            elseif (lsoftcutpes) then
               do g=1,sdm
                  hopd(g)=min(hopd(g),pesmax*(1.d0+log(abs(hopd(g)/pesmax))))
                  hopd(g)=max(hopd(g),pesmin)
               enddo
            endif
         else
            if (lcutpesall) then
               do g=1,sdm
                  hopd(g) = min(pesmax,hopd(g))
                  hopd(g) = max(pesmin,hopd(g))
               enddo
            elseif (lsoftpesall) then
               do g=1,sdm
                  hopd(g)=min(hopd(g),pesmax*(1.d0+log(abs(hopd(g)/pesmax))))
                  hopd(g)=max(hopd(g),pesmin)
               enddo
            endif
         endif
              
         if (zhop(h).ne.0) write(ioper) (hopd(g),g=1,sdm)

      else if (typ .eq. 2) then ! kleg or pleg
         f1=htmadof(h)
         if (basis(f1).eq.7 .or. basis(f1).eq.20) then
            sdm=gdim(f1)*gdim(f1)*gdim(f1+1)
         elseif (basis(f1).eq.9 .or. basis(f1).eq.21) then
            sdm=gdim(f1)*gdim(f1)*ipbaspar(3,f1+1)
         endif
         cerr = 'hopd '
         allocate(hopd(sdm),stat=ierr)
         if (ierr > 0) go to 1000
         read(itmpop) (hopd(g),g=1,sdm)
         if (zhop(h).ne.0) write(ioper) (hopd(g),g=1,sdm)

      else if (typ .eq. 3) then
         f1=htmadof(h)
         f2=f1+2
         sdm=(gdim(f1)*gdim(f1)+1)*gdim(f1+1) &
            +(gdim(f2)*gdim(f2)+1)*gdim(f2+1)
         cerr = 'hopd '
         allocate(hopd(sdm),stat=ierr)
         if (ierr > 0) go to 1000
         read(itmpop) (hopd(g),g=1,sdm)
         if (zhop(h).ne.0) write(ioper) (hopd(g),g=1,sdm)

      else if (typ .eq. 4) then ! Wigner
         f1=htmadof(h)
         sdm=gdim(f1)*gdim(f1)*ipbaspar(4,f1)*ipbaspar(8,f1)
         cerr = 'hopd '
         allocate(hopd(sdm),stat=ierr)
         if (ierr > 0) go to 1000
         read(itmpop) (hopd(g),g=1,sdm)
         if (zhop(h).ne.0) write(ioper) (hopd(g),g=1,sdm)

      else if (typ .eq. 10) then
         cerr = 'hopz '
         allocate(hopz(sdm),stat=ierr)
         if (ierr > 0) go to 1000
         read(itmpop) (hopz(g),g=1,sdm)
         if (zhop(h).ne.0) call wr2dop(hopz,htmtype(h),sdm,ioper)

     else if (typ .eq. 25) then
         read(itmpop) rnel
         nel=nint(rnel)
         backspace itmpop

         sdm = 3*nel+1
         cerr = 'hopd '
         allocate(hopd(sdm),stat=ierr)
         if (ierr > 0) go to 1000
         read(itmpop) (hopd(g),g=1,sdm)
         if (zhop(h).ne.0) write(ioper) (hopd(g),g=1,sdm)

      else
         routine = 'Wroper (wropermod)'
         message = 'Unknown operator type.'
         call errormsg

      endif

      write(cerr,'(a,i5)') 'wroper_...',h
      if(allocated(hopz))deallocate(hopz,stat=ierr)
      if(allocated(hopd))deallocate(hopd,stat=ierr)
      if (ierr > 0) go to 1001

      go to 10
  100 continue

      write(wrtag,'(a)') '<end hops>'
      write(ioper) wrtag

      close(ioper)

      return

  999 continue
      message = 'Cannot open file: '//operfile(1:opflaenge)
      call errormsg

 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      stop

 1001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
      stop

      end subroutine wroper

!#######################################################################

      subroutine wr1dop(hopz,htmtype,sdm,iwr)

      implicit none

      integer(long)                                 :: g,g1
      integer(long), intent(in)                     :: sdm,htmtype,iwr
      complex(dop), dimension(sdm,sdm), intent(in)  :: hopz

      if (htmtype .eq. 3 .or. htmtype .eq. 6) then
         write(iwr) (dimag(hopz(g,g)),g=1,sdm)
      else if (htmtype .eq. 4 .or. htmtype .eq.7) then
         write(iwr) (dble(hopz(g,g)),g=1,sdm)
         write(iwr) (dimag(hopz(g,g)),g=1,sdm)
      else if (htmtype .eq. 5) then
         write(iwr) (dble(hopz(g,g)),g=1,sdm)
      else if (htmtype .eq. 8) then
         write(iwr) ((dble(hopz(g,g1)),g=1,sdm),g1=1,sdm)
      else if (htmtype .eq. 9) then
         write(iwr) ((dimag(hopz(g,g1)),g=1,sdm),g1=1,sdm)
      else if (htmtype .eq. 10) then
         write(iwr) ((dble(hopz(g,g1)),g=1,sdm),g1=1,sdm)
         write(iwr) ((dimag(hopz(g,g1)),g=1,sdm),g1=1,sdm)
      endif

      return
      end subroutine wr1dop

!#######################################################################

      subroutine wr2dop(hopz,htmtype,sdm,iwr)

      implicit none

      integer(long)                             :: g
      integer(long), intent(in)                 :: sdm,htmtype,iwr
      complex(dop), dimension(sdm), intent(in)  :: hopz

      if (htmtype .eq. 3 .or. htmtype .eq. 6) then
         write(iwr) (dimag(hopz(g)),g=1,sdm)
      else if (htmtype.eq.4 .or. htmtype.eq.7 .or. htmtype.eq.14) then
         write(iwr) (dble(hopz(g)),g=1,sdm)
         write(iwr) (dimag(hopz(g)),g=1,sdm)
      else if (htmtype .eq. 5) then         
         write(iwr) (dble(hopz(g)),g=1,sdm)
      endif

      return
      end subroutine wr2dop

!#######################################################################
! New version (Benjamin, December 2008)
!
! If new information is written to oper file, add read statement 
! to operinfo1 (rdopermod) and add definition at end of operinfo_pre11 
! to keep backward compatability 
!
!#######################################################################

      subroutine wropinfo

      implicit none

      integer(long)                          :: f,k,m,s,h,nh,np,hl,nham,&
                                                muld,gf,n,i
      integer(long), external                :: log2int
      character(len=c2)                      :: wrtag

      write(ioper) progver

!-----------------------------------------------------------------------
! array sizes
!-----------------------------------------------------------------------
      call wrmemdim(ioper)

!-----------------------------------------------------------------------
! write operator title to file
!-----------------------------------------------------------------------
      call wroptit

!-----------------------------------------------------------------------
! write system definition to oper file
!-----------------------------------------------------------------------
      call wrdvrdef(ioper)
      call wrgrddef(ioper)

      wrtag= '<mass>'
      write(ioper) wrtag
      write(ioper) (mass(f),f=1,ndof)
      wrtag= '<npar>'
      write(ioper) wrtag
      write(ioper) npar
      wrtag= '<apar>'
      write(ioper) wrtag
      write(ioper) (apar(np),np=1,npar)
      wrtag= '<rpar>'
      write(ioper) wrtag
      write(ioper) (rpar(np),np=1,npar)

      wrtag= '<nopmode>'
      write(ioper) wrtag
      write(ioper) nopmode
      wrtag= '<nopdof>'
      write(ioper) wrtag
      write(ioper) nopdof
      wrtag= '<nddstate>'
      write(ioper) wrtag
      write(ioper) nddstate

      if (lmlop) then
         wrtag= '<tapelength>'
         write(ioper) wrtag
         write(ioper) tapelength()
         wrtag= '<tape>'
         write(ioper) wrtag
         write(ioper) (tape(m),m=1,tapelength())
      endif

      wrtag= '<opmodelabel>'
      write(ioper) wrtag
      write(ioper) (modelabel(f),f=ndof+1,nopdof)
      wrtag= '<opnspfdof>'
      write(ioper) wrtag
      write(ioper) (nspfdof(m),m=nmode+1,nopmode)
      wrtag= '<opspfdof>'
      write(ioper) wrtag
      write(ioper) ((spfdof(n,m),n=1,nspfdof(m)),m=nmode+1,nopmode)
      wrtag= '<opdofspf>'
      write(ioper) wrtag
      write(ioper) (dofspf(f),f=ndof+1,nopdof)

      wrtag= '<loptcontract>'
      write(ioper) wrtag
      write(ioper) log2int(loptcontract)
      wrtag= '<lanalpes>'
      write(ioper) wrtag
      write(ioper) log2int(lanalpes)

      wrtag= '<hamdim>'
      write(ioper) wrtag
      write(ioper) hamdim
      wrtag= '<ktot>'
      write(ioper) wrtag
      write(ioper) ktot
      wrtag= '<fkoe>'
      write(ioper) wrtag
      write(ioper) fkoe
      wrtag= '<ftime>'
      write(ioper) wrtag
      write(ioper) ftime
      wrtag= '<mkoe>'
      write(ioper) wrtag
      write(ioper) mkoe
      wrtag= '<hamlab>'
      write(ioper) wrtag
      write(ioper) (hamlab(nham),nham=1,hamdim)
      wrtag= '<nodiag>'
      write(ioper) wrtag
      write(ioper) (log2int(nodiag(nham)),nham=1,hamdim)
      wrtag= '<zham>'
      write(ioper) wrtag
      write(ioper) (zham(nham),nham=1,hamdim)
      wrtag= '<khtot>'
      write(ioper) wrtag
      write(ioper) (khtot(nham),nham=1,hamdim)
      wrtag= '<kzahl>'
      write(ioper) wrtag
      write(ioper) (kzahl(nham),nham=1,hamdim)
      wrtag= '<nmulpot>'
      write(ioper) wrtag
      write(ioper) (nmulpot(nham),nham=1,hamdim)
      wrtag= '<ltdh>'
      write(ioper) wrtag
      write(ioper) (log2int(ltdh(nham)),nham=1,hamdim)
      wrtag= '<khzahl>'
      write(ioper) wrtag
      write(ioper) (((khzahl(m,s,nham),m=1,nmode),s=1,nstate),nham=1,hamdim)
      wrtag= '<zhun>'
      write(ioper) wrtag
      write(ioper) (((zhun(m,s,nham),m=1,nmode),s=1,nstate),nham=1,hamdim)

      wrtag= '<iqconst>'
      write(ioper) wrtag
      write(ioper) (iqconst(f),f=1,ndof)
      wrtag= '<rqconst>'
      write(ioper) wrtag
      write(ioper) (rqconst(f),f=1,ndof)

      wrtag= '<hamilton>'
      write(ioper) wrtag
      write(ioper) ((hamilton(f,k),f=1,nopdof),k=1,ktot)
      wrtag= '<diag>'
      write(ioper) wrtag
      write(ioper) ((log2int(diag(m,k)),m=1,nmode),k=1,ktot)
      wrtag= '<kinop>'
      write(ioper) wrtag
      write(ioper) ((kinop(m,k),m=1,nmode),k=1,ktot)
      wrtag= '<hsym>'
      write(ioper) wrtag
      write(ioper) ((hsym(m,k),m=1,nopmode),k=1,ktot)
      wrtag= '<hlsym>'
      write(ioper) wrtag
      write(ioper) ((hlsym(m,k),m=1,nmode),k=1,ktot)
      wrtag= '<kact>'
      write(ioper) wrtag
      write(ioper) (kact(k),k=1,ktot)
      wrtag= '<ktwin>'
      write(ioper) wrtag
      write(ioper) (ktwin(k),k=1,ktot)
      wrtag= '<ki>'
      write(ioper) wrtag
      write(ioper) (ki(k),k=1,ktot)
      wrtag= '<kf>'
      write(ioper) wrtag
      write(ioper) (kf(k),k=1,ktot)
      wrtag= '<ksi>'
      write(ioper) wrtag
      write(ioper) (ksi(k),k=1,ktot)
      wrtag= '<ksf>'
      write(ioper) wrtag
      write(ioper) (ksf(k),k=1,ktot)
      wrtag= '<klha>'
      write(ioper) wrtag
      write(ioper) (klha(k),k=1,ktot)
      wrtag= '<fterm>'
      write(ioper) wrtag
      write(ioper) (fterm(k),k=1,ktot)
      wrtag= '<hmlop>'
      write(ioper) wrtag
      write(ioper) (hmlop(k),k=1,ktot)

      wrtag= '<htmdim>'
      write(ioper) wrtag
      write(ioper) htmdim
      wrtag= '<htmtype>'
      write(ioper) wrtag
      write(ioper) (htmtype(h),h=1,htmdim)
      wrtag= '<htmsym>'
      write(ioper) wrtag
      write(ioper) (htmsym(h),h=1,htmdim)
      wrtag= '<htmdof>'
      write(ioper) wrtag
      write(ioper) (htmdof(h),h=1,htmdim)
      wrtag= '<htmadof>'
      write(ioper) wrtag
      write(ioper) (htmadof(h),h=1,htmdim) ! NEWKLEG
      wrtag= '<htmndof>'
      write(ioper) wrtag
      write(ioper) (htmndof(h),h=1,htmdim) 
      wrtag= '<htmmode>'
      write(ioper) wrtag
      write(ioper) (htmmode(h),h=1,htmdim)
      wrtag= '<htmmuld>'
      write(ioper) wrtag
      write(ioper) (htmmuld(h),h=1,htmdim)
      wrtag= '<htmform>'
      write(ioper) wrtag
      write(ioper) (htmform(h),h=1,htmdim)
      wrtag= '<htmshift>'
      write(ioper) wrtag
      write(ioper) (htmshift(h),h=1,htmdim)
      wrtag= '<htmderiv>'
      write(ioper) wrtag
      write(ioper) (htmderiv(h),h=1,htmdim)
      wrtag= '<ddsi>'
      write(ioper) wrtag
      write(ioper) (ddsi(h),h=1,htmdim)
      wrtag= '<ddsf>'
      write(ioper) wrtag
      write(ioper) (ddsf(h),h=1,htmdim)
      wrtag= '<minfac>'
      write(ioper) wrtag
      write(ioper) minfac
      wrtag= '<htmfac>'
      write(ioper) wrtag
         write(ioper) (htmfac(h),h=1,htmdim)
      wrtag= '<hterm>'
      write(ioper) wrtag
      write(ioper) ((hterm(nh,h),nh=1,htmfac(h)),h=1,htmdim)

      wrtag= '<mulddim>'
      write(ioper) wrtag
      write(ioper) mulddim
      wrtag= '<muldsize>'
      write(ioper) wrtag
      write(ioper) (muldsize(muld),muld=1,mulddim)
      wrtag= '<nmulddof>'
      write(ioper) wrtag
      write(ioper) (nmulddof(muld),muld=1,mulddim)
      wrtag= '<mulddof>'
      write(ioper) wrtag
      write(ioper) ((mulddof(gf,muld),gf=1,nmulddof(muld)),muld=1,mulddim)
      wrtag= '<mulddof2>'
      write(ioper) wrtag
      write(ioper) ((mulddof2(gf,muld),gf=1,nmulddof(muld)),muld=1,mulddim)

      wrtag= '<hopdim>'
      write(ioper) wrtag
      write(ioper) hopdim
      wrtag= '<hoplab>'
      write(ioper) wrtag
      write(ioper) (hoplab(hl),hl=1,hopdim)
      wrtag= '<hopilab>'
      write(ioper) wrtag
      write(ioper) (hopilab(hl),hl=1,hopdim)
      wrtag= '<hopidef>'
      write(ioper) wrtag
      write(ioper) (hopidef(hl),hl=1,hopdim)
      wrtag= '<hopfile>'
      write(ioper) wrtag
      write(ioper) (hopfile(hl),hl=1,hopdim)
      wrtag= '<maxhpar>'
      write(ioper) wrtag
      write(ioper) maxhpar
      wrtag= '<hopipar>'
      write(ioper) wrtag
      write(ioper) ((hopipar(np,hl),np=1,maxhpar),hl=1,hopdim)
      wrtag= '<hoprpar>'
      write(ioper) wrtag
      write(ioper) ((hoprpar(np,hl),np=1,maxhpar),hl=1,hopdim)

      wrtag= '<npesopt>'
      write(ioper) wrtag
      write(ioper) npesopt
      wrtag= '<zpesopts>'
      write(ioper) wrtag
      write(ioper) ((zpesopts(np,n),np=1,maxhpar),n=1,npesopt)
      wrtag= '<pesopts>'
      write(ioper) wrtag
      write(ioper) (pesopts(n),n=1,npesopt)

      wrtag= '<autocap>'
      write(ioper) wrtag
      write(ioper) (autocap(k),k=1,ktot)
      wrtag= '<nstatecap>'
      write(ioper) wrtag
      write(ioper) (nstatecap(k),k=1,ktot)
      if (lrealwp) then
         wrtag= '<autoabs>'
         write(ioper) wrtag
         write(ioper) (autoabs(k),k=1,ktot)
         wrtag= '<autoflux>'
         write(ioper) wrtag
         write(ioper) (autoflux(k),k=1,ktot)
      endif

      wrtag= '<hopsdim>'
      write(ioper) wrtag
      write(ioper) hopsdim
      wrtag= '<zhop>'
      write(ioper) wrtag
      write(ioper) (zhop(h),h=1,htmdim)

      wrtag= '<lcap>'
      write(ioper) wrtag
      write(ioper) log2int(lcap)
      wrtag= '<lnpot>'
      write(ioper) wrtag
      write(ioper) log2int(lnpot)

      wrtag= '<deflabel>'
      write(ioper) wrtag
      write(ioper) (deflabel(n),n=1,maxdef)
      wrtag= '<deflabop>'
      write(ioper) wrtag
      write(ioper) (deflabop(n),n=1,maxdef)

      wrtag= '<splpaths>'
      write(ioper) wrtag
      write(ioper) (splpaths(n),n=1,nsplpaths)
      wrtag= '<splnames>'
      write(ioper) wrtag
      write(ioper) (splnames(n),n=1,nsplnames)
      wrtag= '<splunits>'
      write(ioper) wrtag
      write(ioper) (splunits(1:4,nsplnames),n=1,nsplnames)
     
      wrtag= '<nrdf>'
      write(ioper) wrtag
      write(ioper) nrdf
      if(lnpot) then
         wrtag= '<modc>'
         write(ioper) wrtag
         write(ioper) modc

         wrtag= '<mdim>'
         write(ioper) wrtag
         write(ioper) ((mdim(m,n),m=1,nmode),n=1,nrdf)
         wrtag= '<mmodc>'
         write(ioper) wrtag
         write(ioper) (mmodc(n),n=1,nrdf)

         wrtag= '<ncutted>'
         write(ioper) wrtag
         write(ioper) (ncutted(n),n=1,nrdf)
      endif

      wrtag= '<iLMR>'
      write(ioper) wrtag
      write(ioper) iLMR
      wrtag= '<hamLMR>'
      write(ioper) wrtag
      write(ioper) ((hamLMR(i,n),i=1,iLMR),n=1,3)

      if(l_mlpf) then
         wrtag= '<mlham_optree>'
         write(ioper) wrtag
         do nham=1,hamdim
            call mlham_dump(ioper,nham)
         enddo
      endif

      if (lgwp) then
         wrtag= '<lhaorder>'
         write(ioper) wrtag
         write(ioper) lhaorder
         wrtag= '<lklha>'
         write(ioper) wrtag
         write(ioper) log2int(lklha)
      endif
      wrtag= '<lddtrans>'
      write(ioper) wrtag
      write(ioper) log2int(lddtrans)
      if (ltraj) then
         wrtag= '<ltshtrans>'
         write(ioper) wrtag
         write(ioper) log2int(ltshtrans)
         wrtag= '<natmtsh>'
         write(ioper) wrtag
         write(ioper) natmtsh
         wrtag= '<ndoftsh>'
         write(ioper) wrtag
         write(ioper) ndoftsh
      endif
      wrtag= '<lrefdbrd>'
      write(ioper) wrtag
      write(ioper) log2int(lrefdbrd)
      wrtag= '<dddiab>'
      write(ioper) wrtag
      write(ioper) dddiab
      wrtag= '<ldiabdb>'
      write(ioper) wrtag
      write(ioper) ldiabdb
      wrtag= '<ddcoo>'
      write(ioper) wrtag
      write(ioper) ddcoo
      wrtag= '<ndofdd>'
      write(ioper) wrtag
      write(ioper) ndofdd
      wrtag= '<ndofddpes>'
      write(ioper) wrtag
      write(ioper) ndofddpes
      wrtag= '<ddtransdim>'
      write(ioper) wrtag
      ddtransdim = ndofddpes*ndofdd
      write(ioper) ddtransdim
      wrtag= '<natm>'
      write(ioper) wrtag
      write(ioper) natm
      wrtag= '<ddextcmd>'
      write(ioper) wrtag
      write(ioper) ddextcmd
      wrtag= '<dde0>'
      write(ioper) wrtag
      write(ioper) dde0
      wrtag= '<lfchk>'
      write(ioper) wrtag
      write(ioper) log2int(lfchk)
      wrtag= '<fchkcmd>'
      write(ioper) wrtag
      write(ioper) fchkcmd
      wrtag= '<ddmindb>'
      write(ioper) wrtag
      write(ioper) ddmindb
      wrtag= '<dddbvar>'
      write(ioper) wrtag
      write(ioper) dddbvar
      wrtag= '<dddbhar>'
      write(ioper) wrtag
      write(ioper) dddbhar
      wrtag= '<dbminwgt>'
      write(ioper) wrtag
      write(ioper) dbminwgt
      wrtag= '<dbdisp>'
      write(ioper) wrtag
      write(ioper) dbdisp
      wrtag= '<dbdegen>'
      write(ioper) wrtag
      write(ioper) dbdegen
      wrtag= '<dbmaxe>'
      write(ioper) wrtag
      write(ioper) dbmaxe
      wrtag= '<dbpntgp>'
      write(ioper) wrtag
      write(ioper) dbpntgp
      wrtag= '<lddrddb>'
      write(ioper) wrtag
      write(ioper) log2int(lddrddb)
      wrtag= '<lddwrdb>'
      write(ioper) wrtag
      write(ioper) log2int(lddwrdb)
      wrtag= '<lupdhes>'
      write(ioper) wrtag
      write(ioper) log2int(lupdhes)
      wrtag= '<lqcsimulate>'
      write(ioper) wrtag
      write(ioper) log2int(lqcsimulate)
      wrtag= '<lgausspot>'
      write(ioper) wrtag
      write(ioper) log2int(lgausspot)
      wrtag= '<lpestest>'
      write(ioper) wrtag
      write(ioper) log2int(lpestest)
      wrtag= '<idpestest>'
      write(ioper) wrtag
      write(ioper) idpestest
      wrtag= '<dddvmin>'
      write(ioper) wrtag
      write(ioper) dddvmin
      wrtag= '<lsacas>'
      write(ioper) wrtag
      write(ioper) log2int(lsacas)
      wrtag= '<ddlaenge>'
      write(ioper) wrtag
      write(ioper) ddlaenge0
      wrtag= '<dbdlength>'
      write(ioper) wrtag
      write(ioper) dbdlength
      wrtag= '<ddname>'
      write(ioper) wrtag
      write(ioper) ddname0
      wrtag= '<gwptrjlaenge>'
      write(ioper) wrtag
      write(ioper) gwptrjlaenge0
      wrtag= '<gwptrjdir>'
      write(ioper) wrtag
      write(ioper) gwptrjdir0
      if (ltraj) then
         wrtag= '<ltshtrans>'
         write(ioper) wrtag
         write(ioper) log2int(ltshtrans)
      endif
      if (ldd.or.lddmctdh) then
         wrtag= '<nsmult>'
         write(ioper) wrtag
         write(ioper) nsmult
         wrtag= '<nbasis>'
         write(ioper) wrtag
         write(ioper) nbasis
         wrtag= '<nelectron>'
         write(ioper) wrtag
         write(ioper) nelectron
         wrtag= '<nocc>'
         write(ioper) wrtag
         write(ioper) nocc
         wrtag= '<nclosed>'
         write(ioper) wrtag
         write(ioper) nclosed
         wrtag= '<ncas>'
         write(ioper) wrtag
         write(ioper) ncas
         wrtag= '<nsocstate>'
         write(ioper) wrtag
         write(ioper) nsocstate
         wrtag= '<istatemap>'
         write(ioper) wrtag
         write(ioper) (istatemap(s),s=1,nddstate)
         wrtag= '<imultmap>'
         write(ioper) wrtag
         write(ioper) (imultmap(s),s=1,nddstate)
         wrtag= '<imultqc>'
         write(ioper) wrtag
         write(ioper) (imultqc(s),s=1,nddstate)
         wrtag= '<qcmethod>'
         write(ioper) wrtag
         write(ioper) qcmethod
         wrtag= '<qcprog>'
         write(ioper) wrtag
         write(ioper) qcprog
         wrtag= '<qcsubform>'
         write(ioper) wrtag
         write(ioper) qcsubform
         if (.not. ltraj) then
            wrtag= '<idxmult>'
            write(ioper) wrtag
            write(ioper) (idxmult(s),s=1,nsmult)
            wrtag= '<imultblk>'
            write(ioper) wrtag
            write(ioper) (imultblk(s),s=1,nsmult)
            wrtag= '<imultoff>'
            write(ioper) wrtag
            write(ioper) (imultoff(s),s=1,nsmult)
            wrtag= '<dercpblk>'
            write(ioper) wrtag
            write(ioper) (dercpblk(s),s=1,nsmult)
            wrtag= '<dercpoff>'
            write(ioper) wrtag
            write(ioper) (dercpoff(s),s=1,nsmult)
            wrtag= '<stoff>'
            write(ioper) wrtag
            write(ioper) (stoff(s),s=1,nsmult)
         endif
         wrtag= '<ncasel>'
         write(ioper) wrtag
         write(ioper) ((ncasel(n,i),n=1,nsmult),i=1,2)
         wrtag= '<nconf>'
         write(ioper) wrtag
         write(ioper) ((nconf(n,i),n=1,nsmult),i=1,2)
         wrtag= '<lddsoc>'
         write(ioper) wrtag
         write(ioper) log2int(lddsoc)
         wrtag= '<lsocbas>'
         write(ioper) wrtag
         write(ioper) log2int(lsocbas)
         wrtag= '<lsocdegen>'
         write(ioper) wrtag
         write(ioper) log2int(lsocdegen)
         wrtag= '<lupdhes>'
         write(ioper) wrtag
         write(ioper) log2int(lupdhes)
         wrtag= '<lqcsimulate>'
         write(ioper) wrtag
         write(ioper) log2int(lqcsimulate)
         wrtag= '<lncoupl>'
         write(ioper) wrtag
         write(ioper) log2int(lncoupl)
      endif
      wrtag= '<dercpdim>'
      write(ioper) wrtag
      write(ioper) dercpdim
      wrtag= '<cicocut>'
      write(ioper) wrtag
      write(ioper) cicocut
      wrtag= '<ldecoup>'
      write(ioper) wrtag
      write(ioper) log2int(ldecoup)

      if(lddmctdh.or.lgausspot)then
         wrtag= '<ddmorder>'
         write(ioper) wrtag
         write(ioper) ddmorder
         wrtag= '<iddmtest>'
         write(ioper) wrtag
         write(ioper) iddmtest
         wrtag= '<lgaptest>'
         write(ioper) wrtag
         write(ioper) log2int(lgaptest)
         wrtag= '<lddgbtendec>'
         write(ioper) wrtag
         write(ioper) log2int(lddgbtendec)
         wrtag= '<lddmopt>'
         write(ioper) wrtag
         write(ioper) log2int(lddmopt)
         wrtag= '<ddmoptord>'
         write(ioper) wrtag
         write(ioper) ddmoptord
         wrtag= '<lgapderiv>'
         write(ioper) wrtag
         write(ioper) log2int(lgapderiv)
         wrtag= '<lgapusesym>'
         write(ioper) wrtag
         write(ioper) log2int(lgapusesym)
         wrtag= '<lgapuseinwf>'
         write(ioper) wrtag
         write(ioper) log2int(lgapuseinwf)
         wrtag= '<lgapvarprint>'
         write(ioper) wrtag
         write(ioper) log2int(lgapvarprint)
         wrtag= '<lgapsobol>'
         write(ioper) wrtag
         write(ioper) log2int(lgapsobol)
         wrtag= '<lgapadiabwf>'
         write(ioper) wrtag
         write(ioper) log2int(lgapadiabwf)
         wrtag= '<lgapallgrd>'
         write(ioper) wrtag
         write(ioper) log2int(lgapallgrd)
         wrtag= '<lgapvarmax>'
         write(ioper) wrtag
         write(ioper) log2int(lgapvarmax)
         wrtag= '<lgapparopt>'
         write(ioper) wrtag
         write(ioper) log2int(lgapparopt)
         wrtag= '<gapalpha>'
         write(ioper) wrtag
         write(ioper) (gapalpha(n),n=1,ndofdd)
         wrtag= '<gapker>'
         write(ioper) wrtag
         write(ioper) (gapker(n),n=1,ndofdd)
         wrtag= '<gapkerscl>'
         write(ioper) wrtag
         write(ioper) (gapkerscl(n),n=1,ndofdd)
         wrtag= '<optgapalpha>'
         write(ioper) wrtag
         write(ioper) optgapalpha
         wrtag= '<gapsigma>'
         write(ioper) wrtag
         write(ioper) gapsigma
         wrtag= '<gapkfac>'
         write(ioper) wrtag
         write(ioper) gapkfac
         wrtag= '<gapvartol>'
         write(ioper) wrtag
         write(ioper) gapvartol
         wrtag= '<gapsd>'
         write(ioper) wrtag
         write(ioper) gapsd
         wrtag= '<gapsample>'
         write(ioper) wrtag
         write(ioper) gapsample
         wrtag= '<gapeshift>'
         write(ioper) wrtag
         write(ioper) gapeshift
         wrtag= '<gapethresh>'
         write(ioper) wrtag
         write(ioper) gapethresh
         wrtag= '<lgapethresh>'
         write(ioper) wrtag
         write(ioper) log2int(lgapethresh)
         wrtag= '<lgapcomp>'
         write(ioper) wrtag
         write(ioper) log2int(lgapcomp)
         wrtag= '<ngapcomp>'
         write(ioper) wrtag
         write(ioper) ngapcomp
         wrtag= '<ddgbsing>'
         write(ioper) wrtag
         write(ioper) ddgbsing
         wrtag= '<ddgbsingss1>'
         write(ioper) wrtag
         write(ioper) ddgbsingss1
         wrtag= '<ddgbtens>'
         write(ioper) wrtag
         write(ioper) ddgbtens
         if(ddcoo.eq.0.or.ddcoo.eq.2)then
            wrtag= '<ddxcoo0>'
            write(ioper) wrtag
            write(ioper) ddxcoo0
         endif
         wrtag= '<lmalon>'
         write(ioper) wrtag
         write(ioper) lmalon
      endif

!-----------------------------------------------------------------------
! write DD trafo info
!-----------------------------------------------------------------------
      call wrddtrans

!-----------------------------------------------------------------------
! write TSH trafo info
!-----------------------------------------------------------------------
      if (ltraj) call wrtshtrans

!-----------------------------------------------------------------------
! write DD diabatisation info
!-----------------------------------------------------------------------
      if (ldd .and. (dddiab .eq. 1 .or. dddiab .eq. 2)) then
         if (dreffile .eq. 'inp ' .or. coinfile .eq. 'inp ')  &
             call rdpoints 
         if (dreffile .ne. 'inp' ) call rddreffile
         if (coinfile .ne. 'inp' ) call rdcoinfile

         call wrdiabpnts
      endif

      wrtag= '<end oper info>'
      write(ioper) wrtag

      return
      end subroutine wropinfo

!#######################################################################

      subroutine wroptit

      implicit none

      integer(long)            :: ilbl,ic,iz,ierr
      integer(long), external  :: slen
      logical(kind=4)          :: check,found
      character(len=c5)        :: filop1
      character(len=100)       :: buffer
      character(len=c2)        :: wrtag

      routine='Wroptit'

      call fpstartup(buffer)
      
      found=.false.
      check = .false.
      close(iop) ! This nonsense is required for the intel compiler,
                 ! if iop=iin, i.e. if there is no separate *.op file.
      inquire(iop,opened=check)
      if (check) then
         rewind(iop)
      else if (iop .ne. iin) then
         filop1=oppath(1:opplaenge)//'/'//opname(1:oplaenge)//'.op'
         inquire(file=filop1,exist=check)
         if (.not. check) go to 20
         open(unit=iop,err=999,file=filop1,status='old', &
           form='formatted')
      else if (iop .eq. iin) then
         filop1=inputfile
         open(unit=iop,err=999,file=filop1,status='old', &
           form='formatted')
      endif

      iz=0
   10 call rdinpf(iop,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine = 'wroptit (wroper.F)'
         message = inptit
         call errormsg
      endif
      if( keyword(1) .eq. 'end-input' .or. keyword(1) .eq. 'end-operator') then
         go to 20
      else if( keyword(1) .ne. 'op_define-section' ) then
         goto 10
      endif
   15 call rdinpf(iop,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine = 'wroptit (wroper.F)'
         message = inptit
         call errormsg
      endif

!-----------------------------------------------------------------------
! write title
!-----------------------------------------------------------------------
      wrtag= '<optitle>'
      write(ioper) wrtag

      if( keyword(1) .eq. 'title' .or. &
         keyword(1) .eq. 'titel' ) then
         found=.true.
   30    continue
         call rdinpf(iop,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .eq. 1) then
            message = 'No "end-title" found'
            call warnmsg
         endif

         if (keyword(1) .eq. 'end-title' .or. &
             keyword(1) .eq. 'end-titel' ) then
            goto 20
         else
            write(ioper) inptit
            go to 30
         endif
      else if( keyword(1) .eq. 'end-input' .or. &
              keyword(1) .eq. 'end-operator') then
         message=keyword(1)(1:slen(keyword(1)))//' found!'
         call errormsg 
      else
         go to 15
      endif

   20 continue
      if (.not. found) then
         inptit=runtitle
         if (inptit .eq. '   ') inptit='No title'
         write(ioper) inptit
      endif

      inptit= '<end title>'
      write(ioper) inptit

!-----------------------------------------------------------------------
! write date and time of creation
!-----------------------------------------------------------------------
      call fpstartup(buffer)
      wrtag= '<creation date>'
      write(ioper) wrtag
      write(ioper) buffer

      wrtag= '<end opertitle>'
      write(ioper) wrtag

      if (iop .ne. iin) close(iop)

      return
  999 continue
      ilbl=index(filop1,' ')-1
      message='Cannot open file '//filop1(1:ilbl)
      call errormsg
      end subroutine wroptit

!#######################################################################

      subroutine cvmcgtrans

      implicit none
!
! zero arrays
! 
      call zerovxd(ddxcoo0,ndofddpes)
      call zeromxd(ddtransf,ndofddpes,ndofddpes)
      call zeromxd(ddtransb,ndofddpes,ndofddpes)

      call cvmcgtrans1(ddxcoo0,ddtransf,ddtransb,xingeo,xmass,natm)

      return
      end subroutine cvmcgtrans

!#######################################################################

      subroutine cvmcgtrans1(ddxcoo0,ddtransf,ddtransb,xingeo,xmass,natm)

      implicit none

      integer(long)                                    :: n,i,m,j,n1,m1
      integer(long), intent(in)                        :: natm
      real(dop), dimension(natm), intent(in)           :: xmass
      real(dop), dimension(3,natm), intent(out)        :: ddxcoo0
      real(dop), dimension(3,natm), intent(inout)      :: xingeo
      real(dop), dimension(maxdim,maxdim), intent(out) :: ddtransf,&
                                                          ddtransb
      real(dop), dimension(3,natm,3,natm)              :: workr
      real(dop)                                        :: smass
      real(dop), dimension(3)                          :: com
      real(dop), dimension(natm)                       :: tmass

!-----------------------------------------------------------------------
! tmass(n) is nth partial mass
! smass is total (summed) mass
!-----------------------------------------------------------------------
      smass=0.0_dop
      do n=1,natm
         smass = smass + xmass(n)
         tmass(n) = smass
      enddo

!-----------------------------------------------------------------------
! Initial COM is set to X = 0
!-----------------------------------------------------------------------
      com(1) = 0.0_dop
      com(2) = 0.0_dop
      com(3) = 0.0_dop
      do n=1,natm
         do i=1,3
            com(i)= com(i) + xmass(n)*xingeo(i,n)
         enddo
      enddo
      do i=1,3
         com(i)= com(i)/ smass
      enddo

      do n=1,natm
         do i=1,3
            xingeo(i,n) = xingeo(i,n) - com(i)
            ddxcoo0(i,n) = 0.0_dop
         enddo
      enddo

!-----------------------------------------------------------------------
! x = ddtransf r    
!-----------------------------------------------------------------------
      call zeromxd(workr,natm*3,natm*3)
      do n=2,natm
         do i=1,3
            workr(i,n,i,n-1)=1.0_dop
         enddo
      enddo
      do n=2,natm
         do i=1,3
            do m=n,natm
               workr(i,n,i,m-1)=workr(i,n,i,m-1) - xmass(m)/tmass(m)
            enddo
         enddo
      enddo
      do i=1,3
         do m=2,natm
            workr(i,1,i,m-1)=-xmass(m)/tmass(m)
         enddo
      enddo
      n1=0
      m1=0
      do m=1,natm
         do j=1,3
            m1=m1+1
            n1=0
            do n=1,natm
               do i=1,3
                  n1=n1+1
                  ddtransf(n1,m1)=workr(i,n,j,m)
               enddo
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! r = ddtransb (x - x0)
!-----------------------------------------------------------------------
      call zeromxd(workr,natm*3,natm*3)
      do n=1,natm-1
         do i=1,3
            workr(i,n,i,n+1)=1.0_dop
         enddo
      enddo
      do n=1,natm-1
         do i=1,3
            do m=1,n
               workr(i,n,i,m)=-xmass(m)/tmass(n)
            enddo
         enddo
      enddo
      n1=0
      m1=0
      do m=1,natm
         do j=1,3
            m1=m1+1
            n1=0
            do n=1,natm
               do i=1,3
                  n1=n1+1
                  ddtransb(n1,m1)=workr(i,n,j,m)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine cvmcgtrans1

!#######################################################################
!
! It is assumed that if using Cartesian coordinates, either DD or 
! else an analytic Hamiltonian with mass-scaled general coordinates 
!
!#######################################################################

      subroutine genxcooinfo

      implicit none

      integer(long)                              :: n,n1,f

      if (ldd) then
         f=-2
         n1=0
         do n=1,natm
            f=f+3
! ignore frozen atoms
            if (basis(f) .eq. 19) cycle
            n1=n1+1
            tshmass(n1) = xmass(n)
            tshatnam(n1) = xatnam(n)
         enddo
      else
         do n=1,natmtsh
            tshmass(n) = 1.0_dop
            write(tshatnam(n),'(a,i5)') 'At',n
            call rmblnk(tshatnam(n))
         enddo
      endif

      return
      end subroutine genxcooinfo

!#######################################################################

      subroutine tshtrlvc

      implicit none

      integer(long)                              :: np,ilbl,f,n,f1
      real(dop), dimension(maxdim)               :: freq
      logical(kind=4), dimension(maxdim)         :: lfound
      real(dop), parameter                       :: angst=0.52917725_dop

! get frequencies
      lfound=.false.
      do np=1,npar
         if (apar(np)(1:5) .eq. 'freq0') then
            ilbl=index(apar(np),' ')-1
            read(apar(np)(7:ilbl),*,err=900) f
            if (basis(f) .ne. 27 .and. basis(f) .ne. 19) then
               routine = 'Tshtrlvc (wropermod.f90)'
               write(message,'(2a)') &
                 'LVC transformation frequency specified for non-classical DOF: ',&
                    modelabel(f)
               call warnmsg
            endif
            freq(f) = rpar(np) 
            lfound(f) = .true.
         endif
      enddo

      do f=1,nspfdof(1)
         f1 = spfdof(f,1)
         if (.not. lfound(f1)) then
            routine = 'Tshtrlvc (wropermod.f90)'
            write(message,'(a,i5)') &
                 'Frequency not found in parameters for DOF: ',f1
            call errormsg
         endif
      enddo

      tshtransf=0.0
      tshtransb=0.0
      tshxcoo0 = 0.0_dop
      do f=1,nspfdof(1)
         f1 = spfdof(f,1)
         if (freq(f1) .lt. 0.0) then
            routine='Tshtrlvc'
            write(message,'(a,i5,2x,f10.6)') &
                 'Negative frequency for LVC transformation of Dof: ',f1,freq(f1)
            call errormsg
         endif
            
         tshtransf(f,f) = 1.0/dsqrt(freq(f1))
         tshtransb(f,f) = dsqrt(freq(f1))
      enddo
      do n=1,natmtsh
         tshmass(n) = 1.0_dop
         write(tshatnam(n),'(a,i5)') 'At',n
         call rmblnk(tshatnam(n))
      enddo

      return
  900 routine='Tshtrlvc'
      message = "Frequencies required in format freq0_f "
      message = message(1:39)//"where f is in integer"
      call errormsg
      end subroutine tshtrlvc

!#######################################################################

      subroutine rdtshtrinp

      implicit none

      integer(long)                              :: i,iz,iz0,j,n,ic,&
                                                    ierr,ilbl
      real(dop), dimension(3)                    :: xcoo
      real(dop), dimension(ndoftsh)              :: xmass1
      logical(kind=4)                            :: check,lrdmass,lau
      logical(kind=4), dimension(3)              :: found
      character(len=c1), dimension(ndoftsh)      :: xatnam1
      real(dop), parameter                       :: angst=0.52917725_dop
      character(len=c5)                          :: filename

!-----------------------------------------------------------------------
! if lau = .true.   coords and trafo in au, otherwise Angstrom
! if lrdmass = true. masses are read and not converted from label
!-----------------------------------------------------------------------
      lau = .false.
      lrdmass = .false.

      inquire(iin,opened=check)
      if (.not. check) then
         filename=inputfile(1:inplaenge)
         open(iin,file=filename,status='old',form='formatted',err=900)
      endif

      do i=1,3
         found(i)=.false.
      enddo

      tshxcoo0=0.0
      tshtransf=0.0
      tshtransb=0.0

      rewind(iin)
      iz0=0
!-----------------------------------------------------------------------
! FIND SECTION
!-----------------------------------------------------------------------
      routine = 'read TSHTRANS-SECTION  (rdtshtrinp)'
      iz=0
   10 continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         message = inptit
         call errormsg
      endif

      if (keyword(1).eq.'tshtrans-section') then
         go to 12
      elseif (keyword(1).eq.'end-input') then
         message='No TSHTRANS-SECTION in input file'
         call errormsg
      else
         go to 10
      endif

!-----------------------------------------------------------------------
! READ INFORMATION
!-----------------------------------------------------------------------
   12 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (keyword(1) .eq. 'end-tshtrans-section') then
         go to 60

      else if (keyword(1) .eq. 'coords-au') then
         lau = .true.
         go to 12

      else if (keyword(1) .eq. 'reference') then
         found(1)=.true.
         j=0
         do n=1,natmtsh
            read(iin,*,err=999) xatnam1(n),(xcoo(i),i=1,3)
            do i=1,3
               j=j+1
               if (lau) then
                  tshxcoo0(j)=xcoo(i)
               else
                  tshxcoo0(j)=xcoo(i)/angst
               endif
            enddo
         enddo
         go to 12

      else if (keyword(1) .eq. 'forward') then
         found(2)=.true.
         do i=1,nspfdof(1)
            read(iin,*,err=999) (tshtransf(j,i),j=1,ndoftsh)
            read(iin,*)
         enddo
         go to 12

      else if (keyword(1) .eq. 'backward') then
         found(3)=.true.
         do i=1,ndoftsh
            read(iin,*,err=999) (tshtransb(j,i),j=1,nspfdof(1))
            read(iin,*)
         enddo
         go to 12

      else if (keyword(1) .eq. 'masses') then
         lrdmass = .true.
         do i=1,natmtsh
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            read(keyword(1),*) xmass1(i)
            if (keyword(2) .eq. ',') call keymunits(xmass1(i),keyword(3),ierr)
         enddo
         go to 12

      else
         message = ' unknown keyword : '
         call wrline(i,iz,iz0)
      endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 12
      endif

   60 continue

      if (.not. found(1)) then
         message= ' No reference geometry given'
         call errormsg
      else if (.not. found(2)) then
         message= ' No forward transformation matrix given'
         call errormsg
      else if (.not. found(3)) then
         message= ' No backward transformation matrix given'
         call errormsg
      endif

!-----------------------------------------------------------------------
! convert tranfos from Angstroms to au
!-----------------------------------------------------------------------
      if (.not. lau) then
         do i=1,nspfdof(1)
            call xvixddo(angst,tshtransf(1,i),ndoftsh)
         enddo
         do i=1,ndoftsh
            call xvxxddo(angst,tshtransb(1,i),nspfdof(1))
         enddo
      endif

!-----------------------------------------------------------------------
! get atomic masses if not read
!-----------------------------------------------------------------------
      if (.not. lrdmass) then
         do n=1,natmtsh
            xmass1(n) = 1.0
         enddo
      endif
      do n=1,natmtsh
         write(xatnam1(n),'(a,i5)') 'At',n
         call rmblnk(xatnam1(n))
      enddo
  
!-----------------------------------------------------------------------
! save masses and names for SH calculations (no refdb file)
!-----------------------------------------------------------------------
      tshatnam = xatnam1
      tshmass = xmass1

      close(iin)

      return
  900 continue
      routine='Rdtshtrinp'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop
  999 message='Error reading input file'
      call errormsg
      end subroutine rdtshtrinp

!#######################################################################

      subroutine rdddtrinp

      implicit none

      integer(long)                              :: i,iz,iz0,j,n,ic,&
                                                    ierr,ilbl
      integer(long)                              :: iau,iattmp,isotmp
      real(dop), dimension(3)                    :: xcoo
      real(dop), dimension(ndofddpes)            :: xmass1
      logical(kind=4)                            :: check,lrdmass,lau
      logical(kind=4), dimension(3)              :: found
      character(len=c1), dimension(ndofddpes)    :: xatnam1
      real(dop), parameter                       :: angst=0.52917725_dop
      character(len=c5)                          :: filename

!-----------------------------------------------------------------------
! if lau = .true.   coords and trafo in au, otherwise Angstrom
! if lrdmass = true. masses are read and not converted from label
!-----------------------------------------------------------------------
      lau = .false.
      lrdmass = .false.

      inquire(iin,opened=check)
      if (.not. check) then
         filename=inputfile(1:inplaenge)
         open(iin,file=filename,status='old',form='formatted',err=900)
      endif

      do i=1,3
         found(i)=.false.
      enddo

      call zerovxd(ddxcoo0,ndofddpes)
      call zeromxd(ddtransf,ndofddpes,ndofddpes)
      call zeromxd(ddtransb,ndofddpes,ndofddpes)

      rewind(iin)
      iz0=0
!-----------------------------------------------------------------------
! FIND SECTION
!-----------------------------------------------------------------------
      routine = 'read DDTRANS-SECTION  (rdddtrinp)'
      iz=0
   10 continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         message = inptit
         call errormsg
      endif

      if (keyword(1).eq.'ddtrans-section') then
         go to 12
      elseif (keyword(1).eq.'end-input') then
         message='No DDTRANS-SECTION in input file'
         call errormsg
      else
         go to 10
      endif

!-----------------------------------------------------------------------
! READ INFORMATION
!-----------------------------------------------------------------------
   12 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (keyword(1) .eq. 'end-ddtrans-section') then
         go to 60

      else if (keyword(1) .eq. 'coords-au') then
         lau = .true.
         go to 12

      else if (keyword(1) .eq. 'reference') then
         found(1)=.true.
         j=0
         do n=1,natm
            read(iin,*,err=999) xatnam1(n),(xcoo(i),i=1,3)
            do i=1,3
               j=j+1
               if (lau) then
                  ddxcoo0(j)=xcoo(i)
               else
                  ddxcoo0(j)=xcoo(i)/angst
               endif
            enddo
         enddo
         go to 12

      else if (keyword(1) .eq. 'forward') then
         found(2)=.true.
         do i=1,ndofdd
            read(iin,*,err=999) (ddtransf(j,i),j=1,ndofddpes)
            read(iin,*)
         enddo
         go to 12

      else if (keyword(1) .eq. 'backward') then
         found(3)=.true.
         do i=1,ndofddpes
            read(iin,*,err=999) (ddtransb(j,i),j=1,ndofdd)
            read(iin,*)
         enddo
         go to 12

      else if (keyword(1) .eq. 'masses') then
         lrdmass = .true.
         do i=1,natm
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            read(keyword(1),*) xmass1(i)
            if (keyword(2) .eq. ',') call keymunits(xmass1(i),keyword(3),ierr)
         enddo
         go to 12

      else
         message = ' unknown keyword : '
         call wrline(i,iz,iz0)
      endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 12
      endif

   60 continue

      if (.not. found(1)) then
         message= ' No reference geometry given'
         call errormsg
      else if (.not. found(2)) then
         message= ' No forward transformation matrix given'
         call errormsg
      else if (.not. found(3)) then
         message= ' No backward transformation matrix given'
         call errormsg
      endif

!-----------------------------------------------------------------------
! convert tranfos from Angstroms to au
!-----------------------------------------------------------------------
      if (.not. lau) then
         do i=1,ndofdd
            call xvixddo(angst,ddtransf(1,i),ndofddpes)
         enddo
         do i=1,ndofddpes
            call xvxxddo(angst,ddtransb(1,i),ndofdd)
         enddo
      endif

!-----------------------------------------------------------------------
! get atomic masses if not read
!-----------------------------------------------------------------------
      if (.not. lrdmass) then
         do n=1,natm
            call getatnam(xatnam1(n),ilog)  ! translate numbers to names
            call getinam(xatnam1(n),iattmp,isotmp,ilog)  ! get atomic numbers
            iau=0
            call getmass(xmass1(n),iattmp,iau,ilog)  ! Get mass (in au)
         enddo
      endif
  
!-----------------------------------------------------------------------
! save masses and names for SH calculations (no refdb file)
!-----------------------------------------------------------------------
      if (lrunshzagreb) then
         xatnam = xatnam1
         xmass = xmass1
      endif

!-----------------------------------------------------------------------
! Transform to Eckart frame
! Hack to account for different dimension of eckart and ddtransf, ddtransb
!-----------------------------------------------------------------------
!      if (lddreord) then
!         call eckart_reord(xmass1,ddxcoo0,work,ddeckart)
!      else
!         call eckart(xmass1,ddxcoo0,work,ddeckart)
!      endif
!      do i=1,ndofddpes
!         do j=1,ndofddpes
!            tmpeckart(j,i) = ddeckart(j,i)
!         enddo
!      enddo

!      call cpvxd(work,ddxcoo0,ndofddpes)

!      call zeromxd(work1,maxdim,maxdim)
!      call mmtxdd1(tmpeckart,ddtransf,work1,maxdim,ndofddpes,ndofddpes, &
!           ndofdd)
!      call cpmxd(work1,ddtransf,maxdim,maxdim)
!      call zeromxd(work1,maxdim,maxdim)
!      call mmxxdd1(ddtransb,tmpeckart,work1,maxdim,ndofdd,ndofddpes, &
!           ndofddpes)
!      call cpmxd(work1,ddtransb,maxdim,maxdim)

      close(iin)

      return
  900 continue
      routine='Runoper'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop
  999 message='Error reading input file'
      call errormsg
      end subroutine rdddtrinp

!#######################################################################

      subroutine rdddtrfile

      implicit none

      integer(long)                              :: j,i,ilbl,ifiletyp,ierr,iau
      integer(long), dimension(maxdim)           :: iatiso
      real(dop), dimension(maxdim)               :: freq,fmass,xmass1
      character(len=c5)                          :: filename
      character(len=c1)                          :: fileorient
      character(len=c1), dimension(maxdim)       :: nmlab,fatnam
      real(dop)                                  :: e0,egrd,eorb,eshift
      character(len=10), dimension(maxdim)       :: xatnam1
      logical(kind=4)                            :: lsocdegen

      ilbl = index(ddtrfile,' ')-1
      if (lgwptrj) then
         filename = gwptrjdir(1:gwptrjlaenge)//'/'//ddtrfile(1:ilbl)
      else
         filename = ddname(1:ddlaenge)//'/'//ddtrfile(1:ilbl)
      endif

      open(idat,file=filename,status='old',err=999)
      ilbl = index(filename,' ')-1
      write(ilog,'(/,2a)') 'Opening trafo file: ',filename(1:ilbl)

!-----------------------------------------------------------------------
! get information from file
!-----------------------------------------------------------------------
      call getintyp(idat,ifiletyp,filename,ilog)
      write(ilog,'(15x,a,i5)') 'Ab initio type:',ifiletyp
      iau=0
      fileorient='input'
      call getxcoo(idat,ddxcoo0,ifiletyp,iau,fileorient,iatsym, &
           iatiso,ndofddpes,nddstate,ndofdd,ilog)

!-----------------------------------------------------------------------
! get atom masses (in amu) and labels (fmass and fatnam contain mass and
! labels for the DOF
!-----------------------------------------------------------------------
      j=1
      do i=1,natm
         xmass1(i)=fmass(j)
         xatnam1(i)=fatnam(j)(1:10)
         j=j+3
      enddo

      ierr=0

      if (lrde0) then
         e0=0.0_dop
         egrd=0.0_dop
         eorb=0.0_dop
         eshift=0.0_dop
         iau=0
         lsocdegen=.true.
         call geten(idat,dde0,e0,ifiletyp,iau,1,trfiletyp,filename,&
              ilog,ndofddpes,nddstate,ndofdd,lsocdegen)
      endif

      iau = 0
      call getnm(idat,ifiletyp,nmlab,ddtransf,freq,iatsym,iau,&
           maxdim,ndofddpes,nddstate,ndofdd,ilog,1,.true.)

! log frequencies found
      write(ilog,'(a)') ' Frequencies read (in cm-1)'
      write(ilog,'(8f10.2)') (freq(i)*invcm,i=1,ndofdd)

! make any negative frequencies positive
      freq(1:ndofdd)=abs(freq(1:ndofdd))
      if (ierr .eq. 1) go to 1040

      close(idat)

! create the transformation matrix
      iau=0
      call nm2xmat(ddtransf,ddtransb,iatsym,freq,iau,ndofddpes,maxdim,&
           ndofdd,ilog)

!-----------------------------------------------------------------------
! Transform to Eckart frame
! Hack to account for different dimension of eckart and ddtransf, ddtransb
!-----------------------------------------------------------------------
!      if (lddreord) then
!         call eckart_reord(xmass1,ddxcoo0,work,ddeckart)
!      else
!         call eckart(xmass1,ddxcoo0,work,ddeckart)
!      endif
!      call zeromxd(tmpeckart,maxdim,maxdim)
!      do i=1,ndofddpes
!         do j=1,ndofddpes
!            tmpeckart(j,i) = ddeckart(j,i)
!         enddo
!      enddo

!      call cpvxd(work,ddxcoo0,ndofddpes)

!      call zeromxd(work1,maxdim,maxdim)
!      call mmtxdd1(tmpeckart,ddtransf,work1,maxdim,ndofddpes,ndofddpes,&
!           ndofdd)
!      call cpmxd(work1,ddtransf,maxdim,maxdim)
!      call zeromxd(work1,maxdim,maxdim)
!      call mmxxdd1(ddtransb,tmpeckart,work1,maxdim,ndofdd,ndofddpes,&
!           ndofddpes)
!      call cpmxd(work1,ddtransb,maxdim,maxdim)

      return
  999 routine='Rdddtrfile (wroper.F)'
      ilbl=index(filename,' ')-1
      write(message,'(2a)') 'Error opening file: ',filename(1:ilbl)
      call errormsg
 1040 routine='Rdddtrfile (wroper.F)'
      ilbl=index(filename,' ')-1
      write(message,'(2a)') 'Nmodes not found in: ',filename(1:ilbl)
      call errormsg
      end subroutine rdddtrfile

!#######################################################################

      subroutine wrddtrans

      implicit none

      integer(long)       :: i,j
      character(len=c2)   :: wrtag

      wrtag= '<xatnam>'
      write(ioper) wrtag
      write(ioper) (xatnam(i),i=1,natm)
      wrtag= '<xmass>'
      write(ioper) wrtag
      write(ioper) (xmass(i),i=1,natm)
      if (lddtrans) then
         wrtag= '<ddxcoo0>'
         write(ioper) wrtag
         write(ioper) (ddxcoo0(i),i=1,ndofddpes)
         wrtag= '<ddtransf>'
         write(ioper) wrtag
         write(ioper) ((ddtransf(j,i),j=1,ndofddpes),i=1,ndofdd)
         wrtag= '<ddtransb>'
         write(ioper) wrtag
         write(ioper) ((ddtransb(j,i),j=1,ndofdd),i=1,ndofddpes)
      endif

      return
      end subroutine wrddtrans

!#######################################################################

      subroutine wrtshtrans

      implicit none

      integer(long)       :: i,j
      character(len=c2)   :: wrtag

      wrtag= '<tshatnam>'
      write(ioper) wrtag
      write(ioper) (tshatnam(i),i=1,natmtsh)
      wrtag= '<tshmass>'
      write(ioper) wrtag
      write(ioper) (tshmass(i),i=1,natmtsh)
      if (ltshtrans) then
         wrtag= '<tshxcoo0>'
         write(ioper) wrtag
         write(ioper) (tshxcoo0(i),i=1,ndoftsh)
         wrtag= '<tshtransf>'
         write(ioper) wrtag
         write(ioper) ((tshtransf(j,i),j=1,ndoftsh),i=1,nspfdof(1))
         wrtag= '<tshtransb>'
         write(ioper) wrtag
         write(ioper) ((tshtransb(j,i),j=1,nspfdof(1)),i=1,ndoftsh)
      endif

      return
      end subroutine wrtshtrans

!#######################################################################
!
! read points and vectors for regularized diabatisation vectors from 
! input file
!
!#######################################################################

      subroutine rdpoints 

      implicit none

      integer(long)                              :: i,iz,iz0,j,n,ic,ierr,ilbl
      integer(long)                              :: iau,iattmp,isotmp
      real(dop), dimension(3)                    :: xcoo
      real(dop)                                  :: xunit
      real(dop), dimension(ndofddpes)            :: xmass1
      real(dop), dimension(ndofdd)               :: coin,refpoint,gd0,dcp0
      logical(kind=4), dimension(4)              :: found
      logical(kind=4)                            :: check,cart
      character(len=c1), dimension(ndofddpes)    :: xatnam1
      character(len=c5)                          :: filename
      character(len=20)                          :: cunit

      inquire(iin,opened=check)
      if (.not. check) then
         filename=inputfile(1:inplaenge)
         open(iin,file=filename,status='old',form='formatted',err=900)
      endif

      cart = .true.
      do i=1,4
         found(i) = .false.
      enddo
      ierr=0

      rewind(iin)
      iz0=0
!-----------------------------------------------------------------------
! FIND SECTION
!-----------------------------------------------------------------------
      routine = 'read DDPOINTS-SECTION  (rdpoints)'
      iz=0
   10 continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         message = inptit
         call errormsg
      endif

      if (keyword(1).eq.'ddpoints-section') then
         go to 12
      elseif (keyword(1).eq.'end-input') then
         message='No ddpoints-SECTION in input file'
         call errormsg
      else
         go to 10
      endif

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
   12 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (keyword(1)(1:8).eq.'non-cart') then
         cart=.false.

      else if (keyword(1).eq.'conical_intersection') then
         found(2) = .true.
         xunit = 1.0_dop/0.52917225_dop
         if (keyword(2) .eq. '=') then
            cunit = keyword(3)(1:20)
            call keylunits(xunit,cunit,ierr)
            if (ierr .eq. 1) then
               routine = 'Rdpoints (wropermod.f90) conical_intersection'
               message = 'Length unit given not recognized'
               call wrline(zeroi,iz,iz0)
            endif
         endif

         if (cart) then
            j=0
            do n=1,natm
               read(iin,*,err=999) xatnam1(n),(xcoo(i),i=1,3)
               do i=1,3
                  j=j+1
                  xcoin(j)=xcoo(i)*xunit
               enddo
            enddo
         else
            do n=1,ndofdd
               read(iin,*,err=999) xatnam1(n),coin(n)
            enddo
         endif

      else if (keyword(1).eq.'reference_point') then
         found(1) = .true.
         xunit = 1.0_dop/0.52917225_dop
         if (keyword(2) .eq. '=') then
            cunit = keyword(3)(1:20)
            call keylunits(xunit,cunit,ierr)
            if (ierr .eq. 1) then
               routine = 'Rdpoints (wropermod.f90) reference_point'
               message = 'Length unit given not recognized'
               call wrline(zeroi,iz,iz0)
            endif
         endif

         if (cart) then
            j=0
            do n=1,natm
               read(iin,*,err=999) xatnam1(n),(xcoo(i),i=1,3)
               do i=1,3
                  j=j+1
                  xrefpoint(j)=xcoo(i)*xunit
               enddo
            enddo
         else
            do n=1,ndofdd
               read(iin,*,err=999) xatnam1(n),refpoint(n)
            enddo
         endif

      else if (keyword(1).eq.'grad_diff') then
         found(3) = .true.
         xunit = 1.0_dop/27.2114_dop
         if (keyword(2) .eq. '=') then
            cunit = keyword(3)(1:20)
            call keyeunits(xunit,cunit,ierr)
            if (ierr .eq. 1) then
               routine = 'Rdpoints (wropermod.f90) grad_diff'
               message = 'Length unit given not recognized'
               call wrline(zeroi,iz,iz0)
            endif
         endif

         if (cart) then
            j=0
            do n=1,natm
               read(iin,*,err=999) xatnam1(n),(xcoo(i),i=1,3)
               do i=1,3
                  j=j+1
                  xgd0(j)=0.5_dop*xcoo(i)
               enddo
            enddo
         else
            do n=1,ndofdd
               read(iin,*,err=999) xatnam1(n),gd0(n)
               gd0(n)=gd0(n)*xunit
            enddo
         endif

      else if (keyword(1).eq.'deriv_coup') then
         found(4) = .true.
         xunit = 1.0_dop/27.2114_dop
         if (keyword(2) .eq. '=') then
            cunit = keyword(3)(1:20)
            call keyeunits(xunit,cunit,ierr)
            if (ierr .eq. 1) then
               routine = 'Rdpoints (wropermod.f90) deriv_coup'
               message = 'Length unit given not recognized'
               call wrline(zeroi,iz,iz0)
            endif
         endif

         if (cart) then
            j=0
            do n=1,natm
               read(iin,*,err=999) xatnam1(n),(xcoo(i),i=1,3)
               do i=1,3
                  j=j+1
                  xdcp0(j)=xcoo(i)*xunit
               enddo
            enddo
         else
            do n=1,ndofdd
               read(iin,*,err=999) xatnam1(n),dcp0(n)
               dcp0(n)=dcp0(n)/27.2114_dop
            enddo
         endif

      else if (keyword(i).eq.'end-ddpoints-section') then
         go to 20

      else
         message = ' unknown keyword : '
         call wrline(i,iz,iz0)
      end if

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 12
      endif

 20   continue
      if (dreffile .eq. 'inp' .and. .not. found(1)) then
         message = 'No reference point has been specified'
         call errormsg
      else if (coinfile .eq. 'inp' .and. .not. found(2)) then
         message = 'No conical intersection has been specified'
         call errormsg
      else if (coinfile .eq. 'inp' .and. .not. found(3)) then
         message = 'Gradient Difference vector has not been specified'
         call errormsg
      else if (coinfile .eq. 'inp' .and. .not. found(4)) then
         message = 'Derivative coupling vector has not been specified'
         call errormsg
      endif

!-----------------------------------------------------------------------
! Transform to Eckart frame
!-----------------------------------------------------------------------
       if (cart) then
          do n=1,natm
             call getatnam(xatnam1(n),ilog)  ! translate numbers to names
             call getinam(xatnam1(n),iattmp,isotmp,ilog)  ! get atomic numbers
             iau=0
             call getmass(xmass1(n),iattmp,iau,ilog)  ! Get mass (in au)
          enddo

!         if (lddreord) then
!            call eckart_reord(xmass1,xrefpoint,work,ddeckart)
!         else
!            call eckart(xmass1,xrefpoint,work,ddeckart)
!         endif
!         call cpvxd(work,xrefpoint,ndofddpes)

!         if (lddreord) then
!            call eckart_reord(xmass1,xcoin,work,ddeckart)
!         else
!            call eckart(xmass1,xcoin,work,ddeckart)
!         endif
!         call cpvxd(work,xcoin,ndofddpes)

!         call zerovxd(work,ndofddpes)
!         call qvtxdd1(ddeckart,xdcp0,work,ndofddpes,ndofddpes)
!         call cpvxd(work,xdcp0,ndofddpes)
!         call zerovxd(work,ndofddpes)
!         call qvtxdd1(ddeckart,xgd0,work,ndofddpes,ndofddpes)
!         call cpvxd(work,xgd0,ndofddpes)
       else
          message='RDPOINTS only works correctly with Cartesian input'
          call errormsg
!         call ddq2x(coin,xcoin)
!         call ddq2x(refpoint,xrefpoint)
!         call ddf2x(dcp0,xdcp0)
!         call ddf2x(gd0,xgd0)
       endif

      if (cart .and. lddtrans) then
         call ddx2q(xcoin,coin)
         call ddx2q(xrefpoint,refpoint)
         call ddf2q(xdcp0,dcp0)
         call ddf2q(xgd0,gd0)
      endif

      write(ilog,'(a)') 'Conical intersection'
      do n=1,ndofdd
         write(ilog,'(a10,f12.8)') modelabel(n),coin(n)
      enddo
      write(ilog,'(a)') 'Gradient Difference'
      do n=1,ndofdd
         write(ilog,'(a10,f12.8)') modelabel(n),gd0(n)
      enddo
      write(ilog,'(a)') 'Derivative Coupling'
      do n=1,ndofdd
         write(ilog,'(a10,f12.8)') modelabel(n),dcp0(n)
      enddo
      write(ilog,*)
      call myflush(ilog)
      close(iin)

      return
  900 routine='Rdpoints (wropermod.f90)'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
  999 routine='Rdpoints (wropermod.f90)'
      message='Error reading input file'
      call errormsg
      end subroutine rdpoints

!#######################################################################

      subroutine rddreffile

      implicit none

      real(dop), parameter                       :: angst = 0.52917725_dop
      integer(long)                              :: j,i,ilbl,ifiletyp,iau
      integer(long), dimension(ndofddpes)        :: iatiso
      real(dop), dimension(ndofddpes)            :: fmass,xmass1
      character(len=c5)                          :: filename
      character(len=c1)                          :: fileorient
      character(len=c1), dimension(ndofddpes)    :: fatnam

      ilbl = index(dreffile,' ')-1
      filename = ddname(1:ddlaenge)//'/'//dreffile(1:ilbl)

      open(idat,file=filename,status='old',err=999)
      ilbl = index(filename,' ')-1
      write(ilog,'(/,2a)') 'Opening Diabatisation reference file: ', &
          filename(1:ilbl)

!-----------------------------------------------------------------------
! get information from file
!-----------------------------------------------------------------------
      call getintyp(idat,ifiletyp,filename,ilog)
      write(ilog,'(15x,a,i5)') 'Ab initio type:',ifiletyp

      iau=0
      fileorient='input'
      call getxcoo(idat,xrefpoint,ifiletyp,iau,fileorient,iatsym, &
           iatiso,ndofddpes,nddstate,ndofdd,ilog)
      iau=1
      call assmass(iatsym,iatiso,fatnam,fmass,iau,ndofddpes,ilog)

!-----------------------------------------------------------------------
! get atom masses and labels (fmass and fatnam contain mass and
! labels for the DOF
!-----------------------------------------------------------------------
      j=1
      do i=1,natm
         iau = 0
         call getmass(xmass1(i),iatsym(j),iau,ilog)
         j=j+3
      enddo

!-----------------------------------------------------------------------
! Transform to Eckart frame
!-----------------------------------------------------------------------
!      if (lddreord) then
!         call eckart_reord(xmass1,xrefpoint,work,ddeckart)
!      else
!         call eckart(xmass1,xrefpoint,work,ddeckart)
!      endif
!      call cpvxd(work,xrefpoint,ndofddpes)

!-----------------------------------------------------------------------
! Transform to system coordinates
!-----------------------------------------------------------------------
!      if (lddtrans) then
!         call ddx2q(xrefpoint,refpoint)
!      endif

      return
  999 routine='Rddreffile (wroper.F)'
      ilbl=index(filename,' ')-1
      write(message,'(2a)') 'Error reading file: ',filename(1:ilbl)
      call errormsg
      end subroutine rddreffile

!#######################################################################

      subroutine rdcoinfile

      implicit none

      real(dop), parameter                        :: angst = 0.52917725_dop
      integer(long)                               :: j,i,ilbl,ifiletyp,f,iau
      integer(long), dimension(nddstate)          :: eigen, asta
      integer(long), dimension(ndofddpes)         :: iatiso
      integer(long), dimension(nddstate)          :: asta1
      integer(long)                               :: idum1
      real(dop), dimension(ndofddpes)             :: fmass,xmass1
      real(dop), dimension(maxdim)                :: coin,gd0
      real(dop), dimension(maxdim,maxsta)         :: tmpxdcp0,dcp0
      character(len=c5)                           :: filename
      character(len=c1)                           :: fileorient
      character(len=c1), dimension(ndofddpes)     :: fatnam
      logical(kind=4)                             :: found,ldum

      ilbl = index(coinfile,' ')-1
      filename = ddname(1:ddlaenge)//'/'//coinfile(1:ilbl)

      open(idat,file=filename,status='old',err=999)
      ilbl = index(filename,' ')-1
      write(ilog,'(2a)') 'Opening Conical intersection file: ', &
          filename(1:ilbl)

!-----------------------------------------------------------------------
! get coordinate information from file
!-----------------------------------------------------------------------
      call getintyp(idat,ifiletyp,filename,ilog)
      write(ilog,'(15x,a,i5)') 'Ab initio type:',ifiletyp

      iau=0
      fileorient='input'
      call getxcoo(idat,xcoin,ifiletyp,iau,fileorient,iatsym, &
                 iatiso,ndofddpes,nddstate,ndofdd,ilog)
      iau=1
      call assmass(iatsym,iatiso,fatnam,fmass,iau,ndofddpes,ilog)

!-----------------------------------------------------------------------
! read derivative coupling
! possibly only works correctly for gaussian assuming coupling is between
! top 2 states in CAS space.
!-----------------------------------------------------------------------
      found=.false.
      eigen(1) = nroot-1
      eigen(2) = nroot
      asta(1) = nroot-1
      ldum=.false.
      iau = 0
      call getdcp(idat,ifiletyp,iau,tmpxdcp0,ddtransf,dcp0,idum1,eigen, &
           asta,asta1,found,coinfiletyp,maxdim,ndofddpes, &
           ilog,ndofdd,nddstate,onei,maxsta)
      if (asta1(nroot) .eq. nroot) then
         xdcp0(1:ndofddpes)=tmpxdcp0(1:ndofddpes,asta1(nroot))
      else if (.not. found .or. asta1(nroot) .eq. -1) then
         routine='rdcoinfile (wropermod.f90)'
         write(message,'(2a)') 'DCP data not found'
         call errormsg
      endif

!-----------------------------------------------------------------------
! read gradient difference
!-----------------------------------------------------------------------
      found=.false.
      iau=0
      call getgd(idat,ifiletyp,xgd0,ddtransf,gd0,found, &
           coinfiletyp,ndofddpes,ndofddpes,ilog,ndofdd,iau)
      if (.not. found) then
         routine='rdcoinfile (wropermod.f90)'
         write(message,'(2a)') 'GD data not found'
         call errormsg
      endif

!-----------------------------------------------------------------------
! rotate branching space vectors
!-----------------------------------------------------------------------
      call rotatebs(xdcp0,xgd0,xcoin,xrefpoint,maxdim,ndofddpes)

!-----------------------------------------------------------------------
! get atom masses and labels (fmass and fatnam contain mass and
! labels for the DOF
!-----------------------------------------------------------------------
      j=1
      do i=1,natm
         iau = 0
         call getmass(xmass1(i),iatsym(j),iau,ilog)
         j=j+3
      enddo

!-----------------------------------------------------------------------
! Transform to Eckart frame
!-----------------------------------------------------------------------
!      if (lddreord) then
!         call eckart_reord(xmass1,xcoin,work,ddeckart)
!      else
!         call eckart(xmass1,xcoin,work,ddeckart)
!      endif
!      call cpvxd(work,xcoin,ndofddpes)

!      call zerovxd(work,ndofddpes)
!      call qvtxdd1(ddeckart,xdcp0,work,ndofddpes,ndofddpes)
!      call cpvxd(work,xdcp0,ndofddpes)
!      call zerovxd(work,ndofddpes)
!      call qvtxdd1(ddeckart,xgd0,work,ndofddpes,ndofddpes)
!      call cpvxd(work,xgd0,ndofddpes)
!
! convert to system coordinates to write to log
!
      if (lddtrans) then
         call ddx2q(xcoin,coin)
         call ddf2q(xdcp0,dcp0)
         call ddf2q(xgd0,gd0)
 
         write(ilog,'(15x,a,a,a)') '  Intersection (au) ', &
                                   '    Der Coup (au)   ',&
                                   '    Grad Diff (au)  '
         do f=1,ndofdd
            write(ilog,'(a10,3f20.8)') modelabel(f),coin(f),dcp0(f,1),gd0(f)
         enddo
      endif

      return
  999 routine='Rdcointrans (wroper.F)'
      ilbl=index(filename,' ')-1
      write(message,'(2a)') 'Error reading file: ',filename(1:ilbl)
      call errormsg
      end subroutine rdcoinfile

!#######################################################################
! rotate branching space vectors such that diabatic and adiabatic states
! coincide at xrefpoint

      subroutine rotatebs(xdcp,xgd,xcoin,xrefpoint,ndim,ndofddpes)

      implicit none

      integer                                 :: ndim,ndofddpes
      real(dop),dimension(ndim),intent(inout) :: xdcp,xgd
      real(dop),dimension(ndim),intent(in)    :: xcoin,xrefpoint
      real(dop),dimension(ndofddpes)          :: coin2ref,xdcpint,xgdint
      real(dop)                               :: xdcpproj,xgdproj, &
                                                 cos2a,sin2a

      interface
        function ddot(n,dx,incx,dy,incy)
          use decimal, only: dop
          real(dop) :: ddot
          integer :: n,incx,incy
          real(dop),dimension(n) :: dx,dy
        end function ddot
      end interface

      coin2ref = xrefpoint(:ndofddpes)-xcoin(:ndofddpes)
      xdcpint = xdcp(:ndofddpes)
      xgdint = xgd(:ndofddpes)

      xdcpproj=ddot(ndofddpes,coin2ref,1,xdcpint,1)
      xgdproj=ddot(ndofddpes,coin2ref,1,xgdint,1)

      sin2a=-xdcpproj/(sqrt(xdcpproj**2 + xgdproj**2))
      cos2a=xgdproj/(sqrt(xdcpproj**2 + xgdproj**2))

      xdcp(:ndofddpes) = sin2a*xgdint + cos2a*xdcpint
      xgd(:ndofddpes) = cos2a*xgdint - sin2a*xdcpint
     
      return 
      end subroutine rotatebs

!#######################################################################

      subroutine wrdiabpnts

      implicit none

      integer(long)       :: i
      character(len=c2)   :: wrtag
      wrtag= '<coin>'
      write(ioper) wrtag
      write(ioper) (xcoin(i),i=1,ndofddpes)
      wrtag= '<refpoint>'
      write(ioper) wrtag
      write(ioper) (xrefpoint(i),i=1,ndofddpes)
      wrtag= '<dcp0>'
      write(ioper) wrtag
      write(ioper) (xdcp0(i),i=1,ndofddpes)
      wrtag= '<gd0>'
      write(ioper) wrtag
      write(ioper) (xgd0(i),i=1,ndofddpes)

      return
      end subroutine wrdiabpnts

!#######################################################################
! Routine to take an old DD database consisting of adiabatic energies 
! etc. and to re-diabatise it using the propagation diabatisation scheme
! in routine diabat4.
! GWR 07/15
!
! data has been read from old DB in prepareDB, called from runoper
!#######################################################################

      subroutine rediabatise

      implicit none

      integer(long) :: irec,jrec,i,j,k,mtmp,nrec,dbidx,dbjdx
      integer(long) :: ishell,nnrec,interp1,idbloc,maxnshell,numrec0,&
                       irecshell
      integer(long), dimension(:),allocatable    :: shell,nshell,dbshell
      integer(long), dimension(:,:), allocatable :: interp
      integer(long), dimension(:), allocatable   :: map,mapback
      integer(long), dimension(maxsymop)         :: gwpsymrep

      real(dop), dimension(:), allocatable       :: dist0,dist
      real(dop)                                  :: mindist,mindisp

      real(dop), dimension(ndofddpes)                             :: xgpoint
      real(dop), dimension(nddstate,nddstate)                     :: v,adttrans
      real(dop), dimension(nddstate,nddstate)                     :: cv,cadttrans
      real(dop), dimension(ndofddpes,nddstate,nddstate)           :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate) :: deriv2
      real(dop), dimension(nddstate,nddstate)                     :: av
      real(dop), dimension(ndofddpes,nddstate,nddstate)           :: aderiv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate) :: aderiv2
      real(dop), dimension(ndofddpes,nactdim)                     :: dercp
      real(dop), dimension(3,nddstate,nddstate)                   :: dip,adip
      real(sip), dimension(nbasis,nbasis)                         :: mocoeff
      complex(dop), dimension(nsocstate,nsocstate)                :: asoc
      character(len=30), allocatable,dimension(:,:,:) :: ciconfig
      real(dop), allocatable, dimension(:,:,:,:)      :: cicoeff
      logical(kind=4) :: lskip,lsymdb0

      integer(long), external                    :: slen

! need to keep copy of DB in original order for arrays overwritten in dddb_wrmem

      integer(long)     :: ierr
      character(len=10) :: cerr
      real(dop), dimension(:,:), allocatable, save      :: db0geo
      real(dop), dimension(:,:,:), allocatable, save    :: db0adener
      real(dop), dimension(:,:,:,:), allocatable, save  :: db0adgrad
      real(dop), dimension(:,:,:,:,:), allocatable, save :: db0adhess
      real(dop), dimension(:,:,:), allocatable, save    :: db0dercp
      real(dop), dimension(:,:,:,:), allocatable, save  :: db0addipmom
      real(sip), dimension(:,:,:), allocatable, save    :: db0moco
      complex(dop), dimension(:,:,:), allocatable, save :: db0adsoc
      integer(long), dimension(:), allocatable, save    :: db0interp
      character(len=c5)                                 :: cmd

! arrays for symmetrisation

      integer(long):: dbnrec0,n,n1,dbmemsize,nexclude,iwr
      integer(long):: ddv_size,ddder1_size,ddder2_size,adv_size,&
           dddip_size,addip_size,diabtrans_size,ddnact_size,&
           adsoc_size,interp_size,ddmoco_size,ciconfig_size,cicoeff_size

      real(dop), allocatable, dimension(:,:,:,:)      :: ddv,adv
      real(dop), allocatable, dimension(:,:,:,:)      :: ddnact
      real(dop), allocatable, dimension(:,:,:,:)      :: diabtrans,diabctrans
      real(dop), allocatable, dimension(:,:,:,:)      :: ddcv
      complex(dop), allocatable, dimension(:,:,:,:)   :: adsoc
      real(sip), allocatable, dimension(:,:,:)        :: ddmoco
      real(dop), allocatable, dimension(:,:,:,:,:)    :: ddder1,adder1
      real(dop), allocatable, dimension(:,:,:,:,:,:)  :: ddder2,adder2
      real(dop), allocatable, dimension(:,:,:,:,:)    :: dddip,addip
      real(dop), dimension(:,:,:), allocatable        :: xgpoint0
      real(dop), allocatable, dimension(:,:)          :: gwpdist
      logical(kind=4)                                 :: lrediabsym = .false.

      logical(kind=4), dimension(:), allocatable      :: lexclude
! distance below which near symmetry structures are compared
      real(dop)                                       :: symthresh = 0.05

      write(ilog,'(/,a)') '*** Rediabatising DB ***'
      if (ldbsmall) then
         write(ilog,'(a,i7)') '  Using Local DBs. Size :',numrec
      endif
      write(ilog,'(a)') ddname(1:ddlaenge)
      write(ilog,'(a,i5)') 'No. of records in DB: ', dbnrec

!-----------------------------------------------------------------------
! At the moment only global diabatisation implemented, which requires
! adiabatic gradients and Hessians, along with energy
!-----------------------------------------------------------------------
      if (dddiab .ne. 4) then
         routine = 'rediabatise'
         message = &
     'Rediabatisation only possible at the moment with global diabatisation'
         call errormsg
      endif
      if (.not. lagradb) then
         routine = 'rediabatise'
         message = &
     'Need adiabatic gradients for rediabatisation'
         call errormsg
      endif
      if (.not. lahesdb) then
         routine = 'rediabatise'
         message = &
     'Need adiabatic gradients for rediabatisation'
         call errormsg
      endif

!-----------------------------------------------------------------------
! See if point group specified in input file matches that used in DB.
! If different, then create the new symmetry replicas. If now 
! dbpnt=0, no symmetry is defined.
!-----------------------------------------------------------------------
      if (dbpntgp0 .ne. dbpntgp) then
         write(ilog,'(/,2a)') &
            'Point group changed during rediabatisation.'
         write(ilog,'(2(a,i5))') &
            'Old (from DB) :',dbpntgp,'  New (from input) :',dbpntgp0

         dbpntgp = dbpntgp0
         if (dbpntgp .gt. 0) lrediabsym = .true.
      endif
      if (lrediabsym .and. dbpntgp .eq. 0) then
         lrediabsym = .false.

         routine = 'rediabatise'
         message = 'DB Symmetrisation asked for but no point group &
                    & specified. Ignoring.'
         call warnmsg1
      endif
      if (lrediabsym .and. .not. lrediabmap) then
         routine = 'rediabatise'
         message = 'DB Symmetrisation requires mapping of DB points.'
         call errormsg
      endif

!-----------------------------------------------------------------------
! first delete DB then create empty version
!-----------------------------------------------------------------------
      cmd='rm '//sqldbfile(1:ddlaenge+13)
      call excmd(cmd)

      lsymdb0 = .true.
      if (dbpntgp .gt. 0) then
         if (.not. lsymdb) then ! If no symmetry maps in an old DB, 
                                ! create them.
            lsymdb0=.false.
            lsymdb=.true.
         endif
         call prepsym
      else
         lsymdb0=.false.
      endif
      call makeflagarray
      call makeinfoarray(nddstate)

      call dbopen(sqldbfile,the_db,sqlvers,quanticsver,infoarray,&
           flagarray, 2)

      call dbwrrefdb(the_db, natm, nroot, nddstate, nbasis, dde0, &
                     dimref, nfam, mref, atnam, xref, dercpdim, dbpntgp, &
                     nsymop, nsocstate, dbsymmap, nddmult, qcprog, &
                     qcmethod, qcbasis,ddtrfile,ddname,ddlaenge)
 
!-----------------------------------------------------------------------
! Allocate array for distances between structures and mapping arrays
!-----------------------------------------------------------------------
      if (lrediabsym) then
         dbmemsize = dbnrec*nsymop 
      else
         dbmemsize = dbnrec
      endif

      allocate(dist0(dbmemsize))
      allocate(dist(dbmemsize))
      allocate(map(dbmemsize))
      allocate(mapback(dbmemsize))
      allocate(shell(dbmemsize))
      allocate(nshell(dbmemsize))
      allocate(dbshell(dbmemsize))

!-----------------------------------------------------------------------
! set up arrays to keep original DB order for arrays that get
! overwritten in dddb_wrmem
!-----------------------------------------------------------------------
      cerr = 'db0interp'
      allocate(db0interp(dbmemsize),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'db0geo'
      allocate(db0geo(ndofddpes,dbmemsize),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'db0adener'
      allocate(db0adener(nddstate,nddstate,dbmemsize),stat=ierr)
      if (ierr > 0) go to 1000
      if (lagradb) then
        cerr = 'db0adgrad'
        allocate(db0adgrad(ndofddpes,nddstate,nddstate,dbmemsize),stat=ierr)
        if (ierr > 0) go to 1000
      endif
      if (lahesdb) then
         cerr = 'db0adhess'
         allocate(db0adhess(ndofddpes,ndofddpes,nddstate,nddstate,dbmemsize),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if (lnactdb) then
         cerr = 'db0dercp'
         allocate(db0dercp(ndofddpes,nactdim,dbmemsize),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if (ladipdb) then
         cerr = 'db0addipmo'
         allocate(db0addipmom(3,nddstate,nddstate,dbmemsize),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if (lmocodb) then
         cerr = 'db0moco'
         allocate(db0moco(nbasis,nbasis,dbmemsize),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if (lcasocdb) then
         cerr = 'db0adsoc'
         allocate(db0adsoc(nsocstate,nsocstate,dbmemsize),stat=ierr)
         if (ierr > 0) go to 1000
      endif

!-----------------------------------------------------------------------
! Generate Hessian from GS frequencies
!-----------------------------------------------------------------------
         if (hess0 .eq. 1) then    
            call genhess0( &
              dbadhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,1),&
              dbdercp(1:ndofddpes,1:nactdim,1),&
              dbadener(1:nddstate,1:nddstate,1))
         endif

!-----------------------------------------------------------------------
! save original adiabatic DB entries that get overwritten
!-----------------------------------------------------------------------
      db0interp(1:dbnrec) = dbinterp(1:dbnrec) 
      db0geo(1:ndofddpes,1:dbnrec) = dbgeo(1:ndofddpes,1:dbnrec)
      db0adener(1:nddstate,1:nddstate,1:dbnrec) = &
              dbadener(1:nddstate,1:nddstate,1:dbnrec)
     
      if (lagradb) &
         db0adgrad(1:ndofddpes,1:nddstate,1:nddstate,1:dbnrec) = &
                 dbadgrad(1:ndofddpes,1:nddstate,1:nddstate,1:dbnrec)
      if (lahesdb) &
         db0adhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,1:dbnrec) = &
                 dbadhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,1:dbnrec)
      if (lnactdb) &
         db0dercp(1:ndofddpes,1:nactdim,1:dbnrec) = &
                 dbdercp(1:ndofddpes,1:nactdim,1:dbnrec)
      if (ladipdb) &
         db0addipmom(1:3,1:nddstate,1:nddstate,1:dbnrec) = &
                 dbaddipmom(1:3,1:nddstate,1:nddstate,1:dbnrec)
      if (lmocodb) &
         db0moco(1:nbasis,1:nbasis,1:dbnrec) = &
                 dbmoco(1:nbasis,1:nbasis,1:dbnrec)
      if (lcasocdb) &
         db0adsoc(1:nsocstate,1:nsocstate,1:dbnrec) = &
                 dbadsoc(1:nsocstate,1:nsocstate,1:dbnrec)

!-----------------------------------------------------------------------
! Initialise the map array.
!-----------------------------------------------------------------------
      do irec = 1,dbnrec
         map(irec) = irec
         mapback(irec) = irec
      enddo

!-----------------------------------------------------------------------
! Map the DB entries so we know the order of their proximity to the reference.
!
! Get distances of all DB points from the central reference.
! All done in Cartesians.
! map(irec) = jrec     ! jrec old index
! mapback(jrec) = irec ! irec new index
!-----------------------------------------------------------------------
      if (dbnrec .le. 2) lrediabmap = .false.   ! No need to map 
      if (lrediabmap) then
         dist0(:) = 0.0_dop
         do irec = 2,dbnrec
            do i = 1,ndofddpes
               dist0(irec) = dist0(irec) + (dbgeo(i,1)-dbgeo(i,irec))**2
            enddo
         enddo

         dist(:) = dist0(:)
         call ibubble_up(dist,map,dbnrec)

         do irec = 2,dbnrec
            jrec = map(irec)
            mapback(jrec) = irec
         enddo
      endif

!-----------------------------------------------------------------------
! symmetrise
!-----------------------------------------------------------------------
      if (lrediabsym) then
         write(ilog,'(/,a,i5)') 'Symmetry enforced. Point group:',dbpntgp

! allocate memory
         allocate(xgpoint0(ndofddpes,nsymop,1),lexclude(nsymop))
         allocate(gwpdist(nsymop,nsymop))
         call init_qchem_calculations(1,nsymop,adder1,adder2,addip, &
              adv,ddder1,ddder2,dddip,ddmoco,ddnact,adsoc,ddv, &
              diabtrans,ddcv,diabctrans,ciconfig,cicoeff,interp, &
              addip_size,adv_size,ddder1_size,ddder2_size,dddip_size,&
              ddnact_size,ddv_size, diabtrans_size,adsoc_size,&
              interp_size,ddmoco_size,ciconfig_size,cicoeff_size)

         dbnrec0 = dbnrec   ! save size of original DB
         do irec=1,dbnrec0
            dbidx = map(irec)
            if (dbidx .eq. 0) cycle

            if (ddlogfull) write(ilog,'(a,i0)') 'Symmetrising Record :',irec
            xgpoint0(1:ndofddpes,1,1)=db0geo(1:ndofddpes,dbidx)
            call mksymcent(xgpoint0,dbpntgp,nsymop,1,1)

!-----------------------------------------------------------------------
! get distance between original and all replicas.
!-----------------------------------------------------------------------
            call distgwp(xgpoint0,gwpdist,nsymop,nsymop,1,1,1)

!-----------------------------------------------------------------------
! replicas need to be identical or more than dbmin away from data point
!-----------------------------------------------------------------------
            lexclude(:) = .false.
            nexclude=0
            do n=2,nsymop
               if (gwpdist(n,1) .lt. 1.0d-2) then
                  lexclude(n) = .true.
                  nexclude = nexclude+1
               endif
            enddo

!-----------------------------------------------------------------------
! make DB record symmetric
!-----------------------------------------------------------------------
            if (nexclude .gt. 0) then
               n1 = 1
               do n=2,nsymop
                  if (.not. lexclude(n)) cycle
                  xgpoint0(:,1,1) = xgpoint0(:,1,1) + xgpoint0(:,n,1)
                  n1 = n1+1
               enddo
               xgpoint0(:,1,1) = xgpoint0(:,1,1) / dble(n1)
               db0geo(1:ndofddpes,dbidx) = xgpoint0(1:ndofddpes,1,1) 

               if (ddlogfull) write(ilog,'(a)') 'Geometry Symmetrised '

! Reform symmetry replicas
               call mksymcent(xgpoint0,dbpntgp,nsymop,1,1)
            endif

!-----------------------------------------------------------------------
! check which replicas want to be added
!-----------------------------------------------------------------------
            do n=2,nsymop
               if (lexclude(n)) cycle

               do n1 = n+1,nsymop
                  if (lexclude(n1)) cycle
                  if (gwpdist(n1,n) .lt. symthresh) lexclude(n1) = .true.
               enddo
               if (ddlogfull .and. .not. lexclude(n)) then
                  write(ilog,'(2(a,i5))') &
                         'For DB record ',dbidx,' adding replica ',n
               endif
            enddo

!-----------------------------------------------------------------------
! get distance between new structures and structures in the DB
! 
! check later points in DB and remove if too close to symmetry replicas
!-----------------------------------------------------------------------
            do n=2,nsymop
               if (lexclude(n)) cycle

               call distdb(xgpoint0(:,n,1),idbloc,dist,mindist,mindisp,dbnrec0)

               do jrec = irec+1,dbnrec0
                  dbjdx = map(jrec)
                  if (dbjdx .eq. 0) cycle

                  if (dist(dbjdx) .lt. symthresh) then
                     map(jrec) = 0
                     if (ddlogfull) then
                        write(ilog,'(2(a,i5),a,i2)') ' Removing record ',dbjdx, &
                            ' Too close to replica ',dbidx,' #',n
                     endif
                  endif
               enddo
            enddo

!-----------------------------------------------------------------------
! form new DB points
!-----------------------------------------------------------------------
            adv(1:nddstate,1:nddstate,1,1) = &
                   db0adener(1:nddstate,1:nddstate,dbidx)
            if (lagradb) &
               adder1(1:ndofddpes,1:nddstate,1:nddstate,1,1) = &
                      db0adgrad(1:ndofddpes,1:nddstate,1:nddstate,dbidx)
            if (lahesdb) &
               adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,1,1) = &
                      db0adhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,dbidx)
            if (lnactdb) &
               ddnact(1:ndofddpes,1:nactdim,1,1) = &
                      db0dercp(1:ndofddpes,1:nactdim,dbidx)
            if (ladipdb) &
               addip(1:3,1:nddstate,1:nddstate,1,1) = &
                     db0addipmom(1:3,1:nddstate,1:nddstate,dbidx)
            if (lmocodb) &
               ddmoco(1:nbasis,1:nbasis,1) = &
                     db0moco(1:nbasis,1:nbasis,dbidx) 
            if (lcasocdb) &
                 adsoc(1:nsocstate,1:nsocstate,1,1) = &
                     db0adsoc(1:nsocstate,1:nsocstate,dbidx) 

            iwr = 0
            if (ddlogfull) iwr = 1
            call mksympesa1(adder1(1:ndofddpes,1:nddstate,1:nddstate,1:nsymop,1),&
                  ddnact(1:ndofddpes,1:nactdim,1:nsymop,1),&
                  addip(1:3,1:nddstate,1:nddstate,1:nsymop,1),&
                  adsoc(1:nsocstate,1:nsocstate,1:nsymop,1),&
                  lexclude,nsymop,dbpntgp,iwr)
             
!-----------------------------------------------------------------------
! add symmetry replicas to DB
!
! DO WE NOT NEED TO SYMMETRIZE GRADS etc?? 
!-----------------------------------------------------------------------
            do n=2,nsymop
               if (lexclude(n)) cycle

               dbnrec = dbnrec+1
               db0geo(1:ndofddpes,dbnrec) = &
                      xgpoint0(1:ndofddpes,n,1) 
               db0adener(1:nddstate,1:nddstate,dbnrec) = &
                      adv(1:nddstate,1:nddstate,1,1) 
               if (lagradb) &
                  db0adgrad(1:ndofddpes,1:nddstate,1:nddstate,dbnrec) = &
                         adder1(1:ndofddpes,1:nddstate,1:nddstate,n,1) 
               if (lahesdb) &
                  db0adhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,dbnrec) = &
                         adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,1) 
               if (lnactdb) &
                  db0dercp(1:ndofddpes,1:nactdim,dbnrec) = &
                         ddnact(1:ndofddpes,1:nactdim,n,1) 
               if (ladipdb) &
                  db0addipmom(1:3,1:nddstate,1:nddstate,dbnrec) = &
                         addip(1:3,1:nddstate,1:nddstate,n,1) 
               if (lmocodb) &
                  db0moco(1:nbasis,1:nbasis,dbnrec) = &
                         ddmoco(1:nbasis,1:nbasis,1)
               if (lcasocdb) &
                  db0adsoc(1:nsocstate,1:nsocstate,dbnrec) = &
                         adsoc(1:nsocstate,1:nsocstate,1,1)

               map(dbnrec) = dbnrec
               dbinterp(dbnrec) = dbinterp(dbidx)
            enddo

         enddo

         call finalise_qchem_calculations(adder1,adder2,addip,adv, &
              ddder1,ddder2,dddip,ddmoco,ddnact,adsoc,ddv,diabtrans, &
              ddcv,diabctrans,interp,ciconfig,cicoeff)
         deallocate(xgpoint0,lexclude)

!-----------------------------------------------------------------------
! remap
!-----------------------------------------------------------------------
         if (lrediabmap) then
            do irec = dbnrec0+1,dbnrec
               do i = 1,ndofddpes
                  dist0(irec) = dist0(irec) + (db0geo(i,1)-db0geo(i,irec))**2
               enddo
            enddo

            do irec = 2,dbnrec
               do jrec = irec+1,dbnrec
                  if (map(jrec) .eq. 0) cycle

                  dbidx = map(irec)
                  dbjdx = map(jrec)
                  if (dbidx .eq. 0) then
                     mtmp = map(irec)
                     map(irec) = map(jrec)
                     map(jrec) = mtmp
                  else if(dist0(dbjdx) .lt. dist0(dbidx))then
                     mtmp = map(irec)
                     map(irec) = map(jrec)
                     map(jrec) = mtmp
                  endif
               enddo
            enddo

            do irec = 2,dbnrec
               jrec = map(irec)
               if (jrec .ne. 0) mapback(jrec) = irec
            enddo
         endif

         write(ilog,'(a)') ' '

      endif

!-----------------------------------------------------------------------
! assign shells and diabatise a shell at a time to ensure equivalent
! points treated the same
! 0th shell = record 1 (reference point) and assign record 2 to 1st shell
! nshell(n) is no. of points in nth shell
!-----------------------------------------------------------------------
      shell=0
      nshell=0
      numrec0 = numrec
      if (lsymdb0 .and. lrediabmap) then   ! equivalent points defined by symmetry
                          ! using symmetry maps from DB
         shell(1)=0
         nshell(1) = 1
         ishell = 0
! Set up shells with first symmetry rep
         do irec=2,dbnrec
            dbidx = map(irec)
            if (dbsymrep(1,dbidx) .gt. 0) then   ! new shell
               ishell = ishell+1
               shell(irec)=ishell
               nshell(ishell) = 1
            endif
         enddo
! Assign symmetry group members to shell record with first rep
         do irec=2,dbnrec
            dbidx = map(irec)
            if (dbsymrep(1,dbidx) .lt. 0) then   
               i = abs(dbsymrep(1,dbidx))       
               jrec = mapback(i)
               shell(irec)=shell(jrec)
               nshell(shell(jrec)) = nshell(shell(jrec)) + 1
            endif
         enddo

      else if (lrediabmap) then  ! equivalent points defined by distance
         shell(1)=0
         nshell(1) = 1
         if (dbnrec .gt. 1) then
            shell(2)=1
            ishell = shell(2)
         endif
         do irec=3,dbnrec
            dbidx = map(irec)
            if (dbidx .eq. 0) cycle

            do jrec = 1,irec-1
              dbjdx = map(jrec)
              if (dbjdx .eq. 0) cycle

              if (abs(dist0(dbidx) - dist0(dbjdx)) .lt. 1.0d-6) then
!    add to shell
                  shell(irec) = shell(jrec)
                  nshell(ishell) = nshell(ishell)+1
                  exit
               endif
            enddo
            if (shell(irec) .eq. 0) then
!    if not in shell, start new shell
               ishell = ishell + 1
               shell(irec) = ishell
               nshell(ishell) = 1
            endif
         enddo

         maxnshell = 0
         do irec=1,ishell
            maxnshell = max(maxnshell,nshell(irec))
         enddo
! make sure local DBs big enough to cope for shells
         if (ldbsmall) then
            numrec = max(numrec,2*maxnshell)
            if (numrec0 .ne. numrec) then
               write(ilog,'(a,i7)') '  Local DB Size Changed :',numrec
            endif
         endif

      else     ! if mapping not used, put each record in its own shell
         do irec=1,dbnrec
            shell(irec) = irec-1
            nshell(irec) = 1
         enddo

      endif

!-----------------------------------------------------------------------
! Do Diabatisation
!
! ddaXX are original adiabatic values from DB in order of DB
! ddXX are new diabatic entries in new DB in new order
! aXX are single point adiabatic values
! nrec are no. of points in initial DB. dbnrec is now the no. written so far.
!-----------------------------------------------------------------------
      nrec=dbnrec
      dbnrec=0
      dbignore = 0
      nnrec=0
! shell of final DB records
      dbshell = 0
!
      do irec=1,nrec
         write(ilog,'(a,i0)') '** Diabatising Record : ',irec
         dbidx = map(irec)
         if (dbidx .eq. 0) cycle

! starting new shell. Flag to ignore all entries in DB in same shell
         ishell = shell(irec)
         dbignore = 0
         do i = 1,dbnrec
            if (dbshell(i) .eq. ishell) dbignore(i) = 1
         enddo

         xgpoint = 0.0_dop
         av = 0.0_dop
         aderiv1 = 0.0_dop
         aderiv2 = 0.0_dop
         v = 0.0_dop
         deriv1 = 0.0_dop
         deriv2 = 0.0_dop
         adttrans = 0.0_dop
         mocoeff = 0.0_dop
         asoc = 0.0_dop
         adip = 0.0_dop
         dip = 0.0_dop

         xgpoint(1:ndofddpes) = db0geo(1:ndofddpes,dbidx)
         av=0.0_dop
         do j = 1,nddstate
            do k = 1,nddstate
               av(k,j) = db0adener(k,j,dbidx)
            enddo
            if (lagradb) &
               aderiv1(1:ndofddpes,j,j) = db0adgrad(1:ndofddpes,j,j,dbidx)
            if (lahesdb) &
               aderiv2(1:ndofddpes,1:ndofddpes,j,j) = &
                    db0adhess(1:ndofddpes,1:ndofddpes,j,j,dbidx)
         enddo
         dercp(1:ndofddpes,1:nactdim) = db0dercp(1:ndofddpes,1:nactdim,dbidx)
         if (lmocodb)  mocoeff(1:nbasis,1:nbasis) = &
                                db0moco(1:nbasis,1:nbasis,dbidx)
         if (ladipdb) then
            dip(1:3,1:nddstate,1:nddstate) = &
                                db0addipmom(1:3,1:nddstate,1:nddstate,dbidx)
            adip(1:3,1:nddstate,1:nddstate) = &
                                db0addipmom(1:3,1:nddstate,1:nddstate,dbidx)
         endif
         if (lcasocdb) then
            asoc(1:nsocstate,1:nsocstate) = &
                                db0adsoc(1:nsocstate,1:nsocstate,dbidx)
         endif
         lskip = .false.
         idbloc = 0
         if (irec .gt. 1) then
            if (ldbsmall) then
               num_gp = 1   ! set "GWP number" for dddb_rd_gp
               call dddb_gp(dbnrec,xgpoint,1)  ! create local DB 
               call distdb_gp(xgpoint,idbloc,dist,mindist,mindisp,&
                    dbnrec_gp(num_gp))
            else
               call distdb(xgpoint,idbloc,dist,mindist,mindisp,dbnrec)
            endif
            if (ddlogfull) then
               write(ilog,'(a,i7,2(a,f7.4))') &
                    'Nearest DB ID ',idbloc,'  DelX : ',mindist,&
                             '  Disp : ',mindisp
            endif

         endif

         nnrec = nnrec+1
         if (db0interp(dbidx) .eq. -1) lskip = .true.
         num_gp = 1
         call diabat4(xgpoint,av,aderiv1,aderiv2,dercp,&
              v,deriv1,deriv2,adttrans,lskip,idbloc,nnrec,interp1)
! inherit interp state in original DB if -1 (interpolated adiabatic data)

         if (irec .gt. 1) then
! Form diabatic Hessian by updating
            if (ldbsmall) then
               num_gp = 1   ! set "GWP number" for dddb_rd_gp
               call hessian_update1(xgpoint,deriv1,deriv2,dist,&
                    dbnrec_gp(num_gp))
            else
               call hessian_update1(xgpoint,deriv1,deriv2,dist,dbnrec)
            endif
! Transform diabatic Hessian to adiabatic
       !     call hestrans(adttrans,aderiv2,deriv2,ndofddpes,nddstate)

         endif

! Transform addip -> dddip - each component separately
         if (ladipdb) then
            do i=1,3
               call simtranbd(adip(i,1:nddstate,1:nddstate),&
                    adttrans(1:nddstate,1:nddstate),&
                    dip(i,1:nddstate,1:nddstate),nddstate)
            enddo
         endif

! Transform SOCs to diabatic and add to diabatic potential
         if (lcasocdb) then
            call addsoc(asoc(1:nsocstate,1:nsocstate),&
                 v(1:nddstate,1:nddstate),&
                 cv(1:nddstate,1:nddstate),&
                 adttrans(1:nddstate,1:nddstate))
         endif

! get new symmetry replica maps (based on shells)
         if (lrediabmap .and. dbpntgp .gt. 0) then
            if (irec .eq. 1) then      ! First point
               gwpsymrep(1:nsymop) = irec
            else if (shell(irec) .gt. dbshell(irec-1)) then   ! New shell
               gwpsymrep(1:nsymop) = irec
               do i=2,nshell(ishell)   ! set symmetry group indices
                  gwpsymrep(i) = irec+(i-1)
               enddo
               irecshell = irec   ! save record at start of shell
            else if (shell(irec) .eq. dbshell(irec-1)) then   ! Still in shell
               gwpsymrep(:) = 0
               gwpsymrep(1) = -irecshell
            endif
         endif

! write new record to DB in memory. 
         call dddb_wrmem(xgpoint,v,deriv1,deriv2,av,dip,adttrans, &
              dercp,mocoeff,cv,cadttrans,ciconfig,cicoeff, &
              ndofddpes,dbnrec+1,interp1,gwpsymrep)

! write new record to DB on disk. dbnrec updated here.
         call dddb_wr1(xgpoint,av,aderiv1,aderiv2,dip,v,deriv1,deriv2,&
              adip,mocoeff,adttrans,dercp,cv,cadttrans,ciconfig,cicoeff,&
              ndofddpes,nddstate,interp1,dercpdim,gwpsymrep,asoc,nsocstate)
         dbnrec = dbnrec+1

! store shell of DB point
         dbshell(dbnrec)=shell(irec)

      enddo

! set dbignore to include all records in subsequent usage of DB
! restore size of local DBs
      dbignore = 0
      if (ldbsmall) then
         numrec = numrec0
      endif

      write(ilog,'(/,a)') '*** Rediabatising Done ***'
      write(ilog,'(a,i5)') 'No. Records Written : ',dbnrec
      lrediab = .false.

      call dealloc_dirdyn_rediab
      if (allocated(dist0)) deallocate(dist0)
      if (allocated(dist)) deallocate(dist)
      if (allocated(map)) deallocate(map)
      if (allocated(shell)) deallocate(shell)
      if (allocated(nshell)) deallocate(nshell)
      if (allocated(db0interp)) deallocate(db0interp)
      if (allocated(db0geo)) deallocate(db0geo)
      if (allocated(db0adener)) deallocate(db0adener)
      if (allocated(db0adgrad)) deallocate(db0adgrad)
      if (allocated(db0adhess)) deallocate(db0adhess)
      if (allocated(db0dercp)) deallocate(db0dercp)
      if (allocated(db0dipmom)) deallocate(db0dipmom)
      if (allocated(db0addipmom)) deallocate(db0addipmom)
      if (allocated(db0moco)) deallocate(db0moco)
      if (allocated(db0adsoc)) deallocate(db0adsoc)

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_DIRDYN_REDIAB :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine rediabatise

!#######################################################################
! Routine to take an old DD database consisting of adiabatic energies 
! etc. and to re-diabatise it using the propagation diabatisation scheme
! in routine diabat4.
! GWR 07/15
!
! data has been read from old DB in prepareDB, called from runoper
!#######################################################################

      subroutine dbtogapot

use psidef, only: gwpdim,zxi,rsbaspar,zagmat

      implicit none

      integer(long)     :: i,f,n,nr,n1,f1
      real(dop)         :: time
      integer(long)     :: ierr
      character(len=10) :: cerr
      character(len=c5) :: ddname0
      integer(long)     :: ddlaenge0

      complex(dop), allocatable, dimension(:)       :: psi
      real(dop), dimension(maxdim)                  :: gpoint,maxq,minq

      integer(long)                    :: iddmtest0
      logical(kind=4)                  :: lgaptest0

      write(ilog,'(/,a)') '*** Converting DB to GAP DB ***'
      write(ilog,'(a)') 'New GAP DB '
      write(ilog,'(a)') ddname(1:ddlaenge)

      if ((nmode .eq. 1 .and. lmult) .or.(nmode .eq. 2 .and. leb)) then
         continue
      else
         routine = 'Dbtogapot'
         message = 'Conversion to GAP DB only possible for vMCG calculation '
         call errormsg
      endif

!-----------------------------------------------------------------------
! swap ddname to open old DB
! ddnamedb1 is name of old DB, ddname0 now saved name of new DB
!-----------------------------------------------------------------------
      ddname0=ddname
      ddlaenge0=ddlaenge
      ddname=ddnamedb1
      ddlaenge=ddlaengedb1

!-----------------------------------------------------------------------
! open and read in old DB
!-----------------------------------------------------------------------
      call preparedb(1,0)

!-----------------------------------------------------------------------
! save size of old DB
!-----------------------------------------------------------------------
      db0nrec = dbnrec

!-----------------------------------------------------------------------
! set up arrays to store old DB
!-----------------------------------------------------------------------
      cerr = 'db0geo'
      allocate(db0geo(ndofddpes,db0nrec),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'db0ener'
      allocate(db0ener(nddstate,nddstate,db0nrec),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'db0grad'
      allocate(db0grad(ndofddpes,nddstate,nddstate,db0nrec),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'db0hess'
      allocate(db0hess(ndofddpes,ndofddpes,nddstate,nddstate,db0nrec),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'db0dipmom'
      allocate(db0dipmom(3,nddstate,nddstate,db0nrec),stat=ierr)
      if (ierr > 0) go to 1000

!-----------------------------------------------------------------------
! copy DB values to saved 
!-----------------------------------------------------------------------
      do i=1,db0nrec
         db0geo(1:ndofddpes,i) = dbgeo(1:ndofddpes,i)
         db0ener(1:nddstate,1:nddstate,i) = dbener(1:nddstate,1:nddstate,i)
         db0grad(1:ndofddpes,1:nddstate,1:nddstate,i) = &
                            dbgrad(1:ndofddpes,1:nddstate,1:nddstate,i)
         db0hess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,i) = &
                            dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,i)
      enddo

      dbgeo = 0.0_dop
      dbener = 0.0_dop

!-----------------------------------------------------------------------
! swap ddname to open new DB
!-----------------------------------------------------------------------
      ddname=ddname0
      ddlaenge=ddlaenge0

      call preparedb(1,0)

!-----------------------------------------------------------------------
! store first geometry in rsbaspar (required by gapdbinit)
!-----------------------------------------------------------------------
      cerr = 'psi'
      allocate(rsbaspar(1,maxdim,1),stat=ierr)
      if (ierr > 0) go to 1000
      if (lddtrans) then
         call ddx2q(db0geo(1,1),gpoint)
      else
         gpoint(1:ndofddpes) = db0geo(1:ndofddpes,1)
      endif
      do f = 1,ndofdd
         rsbaspar(1,f,1) = gpoint(f)
      enddo

! set up fake psi and agmat arrays and call GAP machinery
      dgldim = ndofdd
      agmatdim = ndofdd**2

      cerr = 'psi'
      allocate(psi(dgldim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'agmat'
      allocate(agmat(agmatdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'gwpdim'
      allocate(gwpdim(maxdim,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'zxi'
      allocate(zxi(maxdim,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'zagmat'
      allocate(zagmat(maxdim,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      gwpdim = 0
      zxi=0
      zagmat=0

      gwpdim(1,1) = 1
      zxi(1,1) = 1
      zagmat(1,1) = 1

      agmat(:) = 0.0
      call set_width(agmat,nspfdof(1),gwpdim(1,1))

      dbnrec = 0

! store max and min values of coordinates in old DB
      maxq(:) = -999.0_dop
      minq(:) = 999.0_dop

      time=0.0_dop
      lgaptest0 = lgaptest
      iddmtest0 = iddmtest
      lgaptest = .true.
      iddmtest = 6
      do nr=1,db0nrec
         if (lddtrans) then
            call ddx2q(db0geo(1,nr),gpoint)
         else
            gpoint(1:ndofddpes) = db0geo(1:ndofddpes,nr)
         endif
         do n=1,nspfdof(1)
            f = spfdof(n,1)
            psi(n) = gpoint(f)

            maxq(f) = max(maxq(f),gpoint(f))
            minq(f) = min(minq(f),gpoint(f))
         enddo
         call gapdrv(time,psi,mr(mrort),.true.)
         time = time+gapuptim
      enddo

! add extra points at ends of space covered
      if (lddtrans) then
         call ddx2q(db0geo(1,1),gpoint)
      else
         gpoint(1:ndofddpes) = db0geo(1:ndofddpes,1)
      endif
      do n=1,nspfdof(1)
         f = spfdof(n,1)
         psi(n) = gpoint(f)
      enddo
      do n=1,nspfdof(1)
         f = spfdof(n,1)
         psi(n) = maxq(f) + 0.5
         call gapdrv(time,psi,mr(mrort),.true.)
         time = time+gapuptim
         do n1=n+1,nspfdof(1)
            f1 = spfdof(n1,1)
            psi(n1) = maxq(f1) + 0.5
            call gapdrv(time,psi,mr(mrort),.true.)
            time = time+gapuptim
            psi(n1) = minq(f1) - 0.5
            call gapdrv(time,psi,mr(mrort),.true.)
            time = time+gapuptim

            psi(n1) = gpoint(f1)
         enddo

         psi(n) = minq(f) - 0.5
         call gapdrv(time,psi,mr(mrort),.true.)
         time = time+gapuptim
         do n1=n+1,nspfdof(1)
            f1 = spfdof(n1,1)
            psi(n1) = maxq(f1) + 0.5
            call gapdrv(time,psi,mr(mrort),.true.)
            time = time+gapuptim
            psi(n1) = minq(f1) - 0.5
            call gapdrv(time,psi,mr(mrort),.true.)
            time = time+gapuptim

            psi(n1) = gpoint(f1)
         enddo

         psi(n) = gpoint(f) 
      enddo

      lgaptest = lgaptest0
      iddmtest = iddmtest0

! conversion done
      ldb2gapot = .false.

      deallocate(rsbaspar)
      deallocate(psi,gwpdim,zxi)
      deallocate(agmat,zagmat)
      if (allocated(db0geo)) deallocate(db0geo)
      if (allocated(db0ener)) deallocate(db0ener)
      if (allocated(db0grad)) deallocate(db0grad)
      if (allocated(db0hess)) deallocate(db0hess)

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_DIRDYN_REDIAB :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine dbtogapot

!-----------------------------------------------------------------------

      subroutine set_width(agmat,gdof,ngwp)

      implicit none

      integer(long), intent(in) :: gdof,ngwp
      integer(long)             :: f,n
      complex(dop), dimension(gdof,gdof,ngwp), intent(out) :: agmat

      do n = 1,ngwp
         do f = 1,gdof
            agmat(f,f,n) = -10.0
         enddo
      enddo

      return
      end subroutine set_width




      end module wropermod




