!   AGLOBAL.INC  Gobal include file for analyse programs.
!
      module aglobal

      use decimal, only: long, dop
      use global, only : c5

!-----------------------------------------------------------------------
! string containing version no.
!-----------------------------------------------------------------------
      character(len=10), save :: aversion

!-----------------------------------------------------------------------
!  POINTERS for the start positions of
!  the arrays stored in the memory arrays mc, mr, mi and ml.
!
! see zeiger routines for definitions
!
!-----------------------------------------------------------------------
      integer(long), save :: mcpsi,mcpsi1,mcdicht11,mcdicht21,mcdicht41, &
                             mcadgwp1,mcdum
      integer(long), save :: mrpldata,mrpldata1,mrweight,mrdum, &
                             mijindx1,miort2plt,miplt2ort,mspsi,mfgpop

!-----------------------------------------------------------------------
! Memory logicals
!
! logicals to allocate various standard memory arrays
!-----------------------------------------------------------------------
      logical(kind=4), save :: lpsi1,ldmat,ldmat1,lort,lpsigrd,ltrafo,lsphfbr, &
                               lfft,lwrsrf

!-----------------------------------------------------------------------
! VARIOUS INTEGERS DEFINING SIZE OF WAVEFUNCTION
!
! griddim : Dimension of (correlated) primitive product grid.
! spsidim : Dimension of of single precision WF. (Similar to dgldim).
!-----------------------------------------------------------------------
      integer(long), save :: spsidim,griddim

!-----------------------------------------------------------------------
! needed for input options
! nopt is no. of options read in pesinput
! outform : selects desired format of output file
! reim : selects desired part (real, imaginary, abs) to be plotted.
! step : data will be output every n steps
! lenunit is unit of length for plot
! lgnu : if true, gnuplot plot info is writen to file
! lgnu4 : if true, gnuplot plot info is writen to file for versions < 5
! lagnu: if true, gnuplot will be called automatically
! linter: if true, interactive plotting will be enabled
! lallcol: if true, all columns printed at once.
! lprint: if true, gnuplot will be printed
! lnw: if true, no weights will be employed. Show 'naked' DVR values.
! lpop2: if true, second population (e.g. momentum space) will be shown.
! lswap: if true, swapfft will be called.
! lrenorm: if true, the densities (showd1d) are re-normalized.
! lpgrid:   if true, GNUPLOT grid lines will be shown.
! lpop2all: if true, transformation to second population (e.g. momentum
!           space) will be performed for all DOFs. Useful for cuts.
!-----------------------------------------------------------------------
      integer(long), save     :: nopt,step,outform,reim,indx
      integer(long), save     :: laeout, laein,laein1,laeexp,laeoutd,laechk
      real(dop), save         :: lenunit,eunit,xmin,xmax,ymin,ymax,tfin
      logical(kind=4), save   :: lgnu,lagnu,lprint,lnw,lnatorb,lpgrid,&
                                 linter,lpop2,lswap,lfilein,lrenorm,&
                                 lfileexp,lpop2all,lallcol
      logical(kind=4), save   :: lgnu4 = .false.

      character(len=c5), save                :: fileout,filein,filein1,psifile,&
                                                fileexp,dirout,filechk
      character(len=16), dimension(3), save  :: unitlab
      character(len=32), save                :: operator

!-----------------------------------------------------------------------
! needed to control reading of files
!
! lend is true if the end of a file is found
!
!  If set to .true.
!     lrddvr  : A DVR file will be read
!     lrdoper : An OPER file will be read
!     lrdinwf : A RESTART file will be read
!-----------------------------------------------------------------------
      logical(kind=4), save :: lrddvr,lrdoper,lrdinwf
      logical(kind=4), save :: lend

!-----------------------------------------------------------------------
! variables defining system to be analysed
! nanmode = no. of modes in analysis, e.g. nmode + 1 if plots v time 
! nandof = no. of DOF in analysis, e.g. ndof+1 if plots v time
! nactdof = no. of active dof in nmode modes
!-----------------------------------------------------------------------
       integer(long), save :: nandof,nanmode,nactdof
 
!-----------------------------------------------------------------------
! variables needed for GWP analysis
!-----------------------------------------------------------------------
       integer(long), save :: ngwppnts

!-----------------------------------------------------------------------
! variables defining the number of the wavefunctions that will be read
!
!     ncut    : limits the maximal number of Wf to be read by showsys (-n).
!     nskip   : number of Wf to be skipped by showsys (-skip option).
!-----------------------------------------------------------------------
      integer(long), save :: ncut, nskip

      end module aglobal
