!  COMBINED.INC
!
!  05/02 FG
!
!-----------------------------------------------------------------------
! PARAMETERS DEFINING "COMBINED" GRIDS IN POTFIT
!
! lpconm     : calculation contains combined modes.
! npmode     : number of modes (MCTDH particles) in potential expansion.
! psubdim(m) : number of grid points for mode m. 
! psubdim2(m) : number of grid points for mode m. 
! maxpsub    : Max[psubdim(m)]
! maxpsub2   : Max[psubdim(m)**2] 
! zmode(m)   : pointer for ort matrix for mode m.
! potdof(n,m): indicates, that the DOF f is the n-th DOF in the mode m.
! dofpot(f)=m indicates, that the DOF f belongs to the mode m.
! npotdof(m) = number of DOFs in mode m.
!-----------------------------------------------------------------------

      module combined
   
      use decimal, only : long
      use maxv, only : maxdim

      logical(kind=4), save                           :: lpconm
      integer(long), save                             :: npmode,maxpsub,maxpsub2
      integer(long), dimension(:), allocatable, save  :: psubdim,psubdim2,zmode, &
                                                         npotdof,dofpot
      integer(long), dimension(:,:), allocatable, save :: potdof,potdofmap

contains

!#######################################################################
!
! COMBINED_ALLOC
!
! memory required for arrays in combined allocated here.
!
! First check if allocated, then allocate. All arrays must be
! deallocated in
! combined_dealloc
!
!#######################################################################

      subroutine alloc_combined

use maxv
use channels

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_combined

      cerr = 'psubdim'
      allocate(psubdim(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'psubdim2'
      allocate(psubdim2(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'zmode'
      allocate(zmode(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'npotdof'
      allocate(npotdof(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dofpot'
      allocate(dofpot(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'potdof'
      allocate(potdof(maxdim,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'potdofmap'
      allocate(potdofmap(2,maxdim),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_COMBINED :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      write(6,'(a)') 'Error allocating memory in ALLOC_COMBINED :',cerr
      write(6,'(a,i5)') 'Error Code :',ierr

      stop
      end subroutine alloc_combined

!#######################################################################

      subroutine dealloc_combined

      if (allocated(psubdim)) deallocate(psubdim)
      if (allocated(psubdim2)) deallocate(psubdim2)
      if (allocated(zmode)) deallocate(zmode)
      if (allocated(npotdof)) deallocate(npotdof)
      if (allocated(dofpot)) deallocate(dofpot)
      if (allocated(potdof)) deallocate(potdof)
      if (allocated(potdofmap)) deallocate(potdofmap)

      return
      end subroutine dealloc_combined

      end module combined

