!***********************************************************************
!
! MCTDH include file "CROCOFLY.F90"
!
! Contains definitions of variables and common blocks needed in the
! on-the-fly-calculation of the cross-correlation function.
!
! Dec 2001, FO
!
!***********************************************************************


!***********************************************************************
!
! Description of variables:
!
! psirefdim: size of reference wave-function
! ccjindxdim: size of JINDX array of reference WF
! ccmaxblock: max[block(s)] of reference WF
! ovtype:    specifies the type of the overlap calculation
!            0 - normal
!            1 - (not yet implemented)
! ebtype:    specifies the type of calculation used for the electronic
!            basis overlap
!            0 - none
!            1 - (not yet implemented)
!
!***********************************************************************
      module crocoflymod
     
      use decimal, only : long

      implicit none

      integer(long), save :: psirefdim,ccjindxdim,doverdim,ccwcdim,ovtype,&
                             ebtype,ccmaxblock,nst1,nst2

      integer(long), dimension(:,:), allocatable, save    :: dovmat
      integer(long), dimension(:,:,:), allocatable, save :: ddovmat

      contains

!#######################################################################
!
! CROCOFLY_ALLOC
!
! memory required for arrays in crocoflymod allocated here.
!
! First check if allocated, then allocate. All arrays must be
! deallocated in
! crocofly_dealloc
!
!#######################################################################

      subroutine alloc_crocofly

use maxv
use channels

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_crocofly

      cerr = 'dovmat'
      allocate(dovmat(maxdim,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ddovmat'
      allocate(ddovmat(maxdim,maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_CROCOFLY :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_crocofly

!#######################################################################

      subroutine dealloc_crocofly

      if (allocated(dovmat)) deallocate(dovmat)
      if (allocated(ddovmat)) deallocate(ddovmat)

      return
      end subroutine dealloc_crocofly

      end module crocoflymod
