!***********************************************************************
!
!                         MCTDH include file GLOBAL
!
!  MCTDH include file defining various parameters and variables
!  global to the program.
!
!   V6.0 MB
!
!***********************************************************************
      module global 

      use decimal, only : long,dop 

!-----------------------------------------------------------------------
! PARAMETERS FOR CHARACTER STRING LENGTHS
!
! c1, c2, c3, c4, c5: various lengths
! c5: length of line in the input
!-----------------------------------------------------------------------
      integer(long), parameter :: c1 = 16, c2 = 32, & 
                                  c3 = 64, c4 = 80, c5 = 240, c6 =720

      integer(long), parameter :: MAXFNAME = 512

!-----------------------------------------------------------------------
! Tiny is a tolerance for when real or complex numbers are compared.
!-----------------------------------------------------------------------
      real(dop),  parameter         :: tiny = 1.0e-9_dop

!-----------------------------------------------------------------------
! chkerr: array used for checking system definition
!
! chkerr(1)   : Error reading file
! chkerr(2)   : Dofs differ from system
! chkerr(3)   : DVR bases differ from system
! chkerr(4)   : Mode combinations differ from system
! chkerr(5)   : Electronic basis type differs from system
! chkerr(6)   : Spf basis differs from system 
! chkerr(7)   : Dofs not subset of system dofs
! chkerr(8)   : ? not for this system
! chkerr(9)   : Psi type differs from system
! chkerr(10)  : Density matrix type differs from system
! 
!-----------------------------------------------------------------------
      integer(long), parameter              :: nerr = 15
      integer(long), dimension(nerr), save  :: chkerr

!-----------------------------------------------------------------------
! DIRECTORY DEFINITION FOR INPUT/OUTPUT FILES
!
! name        :  name of directory in which the output files are stored
! inputfile   : path of the inputfile (~.inp)
! options     : string containing the list of options
! laenge      : length of string name
! inplaenge   : length of string inputfile
! optlaenge   : length of string options
! rname       : directory from which to take restart file
! rlaenge     : length of string rname
! oname       : directory from which to take operator file
! olaenge     : length of string oname
! dname       : directory from which to take dvr file
! dlaenge     : length of string dname
! vname       : name of a vpot file to read
! vlength     : length of string vname
! operfile    : path and name of oper file
! opflaenge   : length of string operfile
! crossname   : directory from which to take restart file used for
!               calculating the cross-correlation-function
! crosslng    : length of string crossname
! wname       : name of a file with relevant region weights to be read
! wlength     : length of string wname
! ddname      : directory in which to look for and store DD results
! ddlaenge    : length of string ddname
! ddname0     : ddname relative to input file
! ddlaenge0   : length of string ddname0
! dbdlength   : length of name of DB directory w/o full path
! ddpath      : path to DB directory (needed for analysis programs)
! pesfilepath : absolute path to the pes-file
! npotname    : name of natpot file to be read by cutnpot
! npotlaenge  : length of string npotname
! cstartup    : string with time that job is started
!-----------------------------------------------------------------------
      character(len=c5), save :: name,inputfile,rname,dname,oname,vname,operfile,&
                                 options,runtitle,crossname,wname,&
                                 ddname,ddname0,pesfilepath,npotname,cstartup
      character(len=c5), save :: ddpath=' '
      integer(long), save     :: laenge,inplaenge,optlaenge,rlaenge,dlaenge,&
                                 olaenge,vlength,opflaenge,crosslng,&
                                 wlength,ddlaenge,ddlaenge0,dbdlength,&
                                 npotlaenge

!-----------------------------------------------------------------------
! DIRECTORY DEFINITION FOR OPERATOR INPUT/OUTPUT FILES
!
! all directories are relative to the startup directory i.e. where the
!  .inp file is held
! opname:  name of operator module
! oppath    : path for operator files
! parfile   : file with parameters for Hamiltonian
! oplaenge  : length of string opname
! opplaenge : length of string oppath
! parlaenge : length of string parfile
!-----------------------------------------------------------------------
      character(len=c5), save  :: opname,oppath,parfile
      integer(long), save      :: oplaenge,opplaenge,parlaenge

!-----------------------------------------------------------------------
! PARAMETERS FOR CALCULATIONS
!
! pi : pi is defined by 4d0*atan(1d0) in subroutine default.
! fs : time conversion from a.u. to fs (x[a.u.] = fs*x[fs]).
! ev : energy conversion from a.u. to ev (x[a.u.] = x[eV]/ev).
!   Note: If a.u. are output, fs=ev=1
! one: the variable 1
! oner: the variable 1.0d0
! True,False: the logicals .true. and .false., respectively
!-----------------------------------------------------------------------
      integer(long), parameter  :: zeroi=0,onei=1,twoi=2,threei=3, &
                                   fouri=4,fivei=5,sixi=6,seveni=7, &
                                   eighti=8,ninei=9,teni=10
      real(dop), parameter      :: zeror=0.0_dop,oner = 1.0_dop
      complex(dop), parameter   :: ione = (0.0_dop, 1.0_dop), &
                                   cone = (1.0_dop, 0.0_dop), &
                                   czero = (0.0_dop, 0.0_dop)
      logical(kind=4), parameter:: true = .true., false = .false.

      real(dop), save      :: pi, fs, ev

!-----------------------------------------------------------------------
! WORK ARRAY SIZES
!
! workcdim : Length of the complex*16 scratch array
! workrdim : Size of real*8 scratch array
! worksdim : Size of complex*8 scratch array
! workfdim : Size of real*4 scratch array
! workldim : Size of logical scratch array
! workidim : Size of integer scratch array
! 
! minedim  : minimum value of maxedim
!-----------------------------------------------------------------------
      integer(long), save :: workcdim,workrdim,workldim,workidim,worksdim, &
                             workfdim
      integer(long), save :: minedim

!-----------------------------------------------------------------------
! FLAG FOR WAVEFUNCTION / DENSITY MATRIX MODE
!
! dentype: 0 wavefunction
!          1 density matrix type I
!          2 density matrix type II
!-----------------------------------------------------------------------
      integer(long), save :: dentype = 0

!-----------------------------------------------------------------------
! FLAGS FOR GWPs
! lgwp set to .true. if one (or more) dofs is a GWP
! lpergwp set to .true. if one (or more) dofs is periodic !pergauss
! lgwpcx set to true if one or more GWP mode uses CX partitioning
!-----------------------------------------------------------------------
      logical(kind=4), save :: lgwp=.false.
      logical(kind=4), save :: lpergwp=.false.,lgwpcx=.false.
      integer(kind=4) :: iperiod

!-----------------------------------------------------------------------
! LOGICALS TO CONTROL INPUT / OUTPUT OPTIONS
!
! If set to .true.,
!     test     : runs up to propagation to test memory etc.
!     test0    : short test-run (via option -s)
!     lovwr    : files in directory may be overwritten 
!     logisopen: the log file is open
!     lwarn    : is set to 'true' in subroutine 'warnmsg' when a 
!                warning is given.
!     liopt    : is set to 'true' when input is read from name/input. 
!     ltime    : if .true. timing output is written
!     lwrlog   : controls printing rst-file information (rstinfo) to log file.
!     lnpot    : flag if natpot files are used
!     tm_zero  : UNIX time (in sec) of start time of run. (set in mdefaults)
!-----------------------------------------------------------------------
      logical(kind=4), save   :: test = .false., &
                                 test0 = .false.
      logical(kind=4), save   :: lovwr = .false., &
                                 ltime = .true.
      logical(kind=4), save   :: lwarn = .false.
      logical(kind=4), save   :: lwrlog = .false.
      logical(kind=4), save   :: lnpot = .false.
      logical(kind=4), save   :: lbuildrestart,liopt
      logical(kind=4), save   :: logisopen = .false.
      character(len=8), save  :: sstop
      character(len=10), save :: scpu,swall
      integer(long), save     :: tm_zero

!-----------------------------------------------------------------------
! 
! FLAGS TO SPECIFY THE RUN-TYPE AND RW-FILE GENERATION
!
!  frunlevel : sets final runlevel 1 = gendvr
!                                  2 = genoper 
!                                  3 = geninwf
!                                  4 = propagate
!  runlevel : flags level program is at
!
!  The following flags dictate which parts of the program will be
!  run. if set to 0 = do not run
!                 1 = run
!                 2 = read previously generated file
!     irundvr  : A DVR file will be generated / read
!     irunoper : An OPER file will be generated / read
!     iruninwf : A RESTART file will be generated / read
!     irunprop : A propagation or Hamiltonian diagonalisation will be made
!
! The following flags dictate particular options / code for a runlevel
!     lrunpes  : If true a PES file will be generated instead of an OPER
!     proptask = 0 : no propagation
!                1 : MCTDH propagation / relaxation
!                2 : RealWP propagation 
!                3 : trajactory propagation 
!     lrundiag : if true, the Hamiltonian is diagonalised using Lanczos
! 
!     lcont    : Continuation run
!     lncoupl  : no nonadiabatic couplings have to be calculated
!     lrlx     : relaxation calculation
!     lrlxt    : thermal relaxation
!     lrlxun   : unmodified relaxation
!     ldeldvr  : DVR file is deleted after use
!     ldelop   : OPER file is deleted after use
!     lexact   : numerically exact calculation
!     lrealwp  : numerically exact calculation using Anthony J H M Meijer's code
!                with Chebychev iteration of real wavepacket
!     ltraj    : swarm of trajectories calculation
!     lvmcg    : vMCG calculation
!     ldd      : direct dynamics calculation
!     lddmctdh : direct MCTDH calculation
!     ldbsmall : whether to use small DBs for each GWP.
!     lgwptrj  : vMCG where GWPs follow pre-calculated trajectories
!     lrunml   : A multilayer propagation will be made
!     ldynspf  : The number of SPFs will be enlarged on-the-fly
!     lreflex  : reflex calculation (malonaldehyde)
!                4 : Hamiltonian diagonalisation
!     lnatfast : if true fast algorithm used to evaluate natuarl potential
!                operations
!     lfuncnopre : if true func routines return HPsi rather than dtpsi
!     ldtpsi2hpsi : if true subroutine prefactor converts dtpsi to Hpsi
!     lreadidx : index files are read (mccpd, mcpotfit)
!     l_mlpf   : mlpf file is present (Multi-layer potfit)
!-----------------------------------------------------------------------
      integer(long), save   :: frunlevel = 0, &
                               runlevel = 0
      integer(long),save    :: irundvr=0, &
                               irunoper=0, &
                               iruninwf=0, &
                               irunprop=0
      logical(kind=4), save :: lcont=.false., &
                               lncoupl=.false., &
                               lrunpes=.false., &
                               lrundiag=.false., &
                               lrunshzagreb=.false., &
                               lexact=.false., &
                               ldelop=.false., &
                               ldeldvr=.false., &
                               ldd=.false., &
                               ltraj=.false., &
                               lreflex=.false., &
                               lrealwp=.false., &
                               lvmcg=.false., &
                               lrlx=.false., &
                               lrlxt=.false., &
                               lrlxun=.false., &
                               lnatfast=.false., &
                               lrunml=.false.,&
                               ldynspf=.false.,&
                               lddmctdh=.false.,&
                               lreadidx = .false.,&
                               l_mlpf = .false.
      logical(kind=4), save :: lfuncnopre =.false., &
                               ldtpsi2hpsi = .false.
      logical(kind=4), save :: lgwptrj = .false.
      integer(long), save   :: proptask = 0
        
!-----------------------------------------------------------------------
! Global flags for DD runs
! ldbsmall  : whether to use small DBs for each GWP.
! lsocbas   : DD uses SOC basis.
! lcivec    : DD uses the CI vector (Procrustes diabatisation)
! lddinfo   : flag to indicate that dd is being created from info file, 
!             so quantics input file is present
! ddcoo     : Select the type of coordinates for DD-vMCG
!        0  :  Jacobi Cartesians
!        1  :  Normal modes
!        2  :  Cartesians
!-----------------------------------------------------------------------
      logical(kind=4), save :: ldbsmall = .true.,&
                               lsocbas = .false.,&
                               lcivec = .false.

      logical(kind=4), save :: lddinfo=.false.

      integer(long), save   :: ddcoo = -99

!-----------------------------------------------------------------------
! maxpar    : maximum no. of parameters that can be read
! npar      : number of parameters read
! apar(n)   : string with name of nth parameter
! rpar(n)   : (real) value of nth parameter
!-----------------------------------------------------------------------
      integer(long), save                                 :: maxpar=0, &
                                                             npar=0
      real(dop), dimension(:), allocatable, save          :: rpar
      character(len=c1), dimension(:), allocatable, save  :: apar

!-----------------------------------------------------------------------
! parameters given on command line
!-----------------------------------------------------------------------
      integer(long), save                                :: nparcmd=0
      character(len=c1), dimension(:), allocatable, save :: aparcmd
      real(dop), dimension(:), allocatable, save         :: rparcmd

!-----------------------------------------------------------------------
! ACCURACY VARIABLES
!
! macheps: machine precision, i. e. smallest positive number for which
!          1.0+macheps > 1.0 (macheps=2.2d-16)
! if lmacheps = .true. macheps calculated, else macheps = 10^-16
!-----------------------------------------------------------------------
       real(kind=dop), save :: macheps
       logical(kind=4), save:: lmacheps

!-----------------------------------------------------------------------
! VARIABLES USED TO READ INPUT FILE
! (defined here so that same size arrays are used in different routines)
!-----------------------------------------------------------------------
      integer(long), parameter                :: maxkey = 100
      integer(long), parameter                :: maxkeylen = 200
      character(maxkeylen), dimension(maxkey), save  :: keyword, keyorig
      integer(long), dimension(maxkey), save  :: lc

!-----------------------------------------------------------------------
! MULTILAYER VARIABLES
!-----------------------------------------------------------------------
      logical, public, save               :: ml_input_ready  = .false.
      logical, public, save               :: ml_oper_ready  = .false.
      logical, public, save               :: ml_prop_ready  = .false.
      logical, public, save               :: ml_ready       = .false.


!-----------------------------------------------------------------------
! integers for output control of pdensity
! npd1...npd4 -- if set -- specify the modes for which the pdensity is
! written.
!-----------------------------------------------------------------------
      integer(long), save :: npd1,npd2,npd3,npd4

!-----------------------------------------------------------------------
! DOFs used for gengmat 
!-----------------------------------------------------------------------
      integer(long), save :: gmatf1,gmatf2

!-----------------------------------------------------------------------
! gwp reassignment mode
!-----------------------------------------------------------------------
      logical(kind=4), save :: lBEL

!-----------------------------------------------------------------------
! A special flag for Terry to use while hacking!
!-----------------------------------------------------------------------
      logical(kind=4), save :: TerryFlag=.false.

!-----------------------------------------------------------------------
! Process ID for the run i.e. PID. Used for differentiating concurrent
! jobs.
!-----------------------------------------------------------------------
      integer(long), save  :: pid
      character(9), save   :: strpid

      end module global

