!######################################################################r
!
! GWPTRJ
!
! Module to set up trajetcories along which GWPs can be driven.
!
! 3 parts - memory sizes, memory allocation, calculation
!#######################################################################

      module gwptrj

      use decimal, only : long, dop
      use global
      use maxv
      use channels

      implicit none
      public :: alloc_gwptrj,dealloc_gwptrj,memgwptrj

!-----------------------------------------------------------------------
! arrays for trajectories required for gwptrj calculation
!-----------------------------------------------------------------------
      integer(long), save                            :: ngwptrjpnts = 0
      integer(long), save                            :: ngwptrjcoo = 0
      integer(long), save                            :: ngwptrj = 0
      integer(long), save                            :: ngwptrjst = 0
      integer(long), dimension(:), allocatable, save :: gwptrjnp
      real(dop), dimension(:,:),   allocatable, save :: gwptrjtdat
      real(dop), dimension(:,:,:), allocatable, save :: gwptrjxdat
      real(dop), dimension(:,:,:), allocatable, save :: gwptrjpdat
      real(dop), dimension(:,:),   allocatable, save :: gwptrjedat
      real(dop), dimension(:,:,:), allocatable, save :: gwptrjcddat
      real(dop), dimension(:,:,:), allocatable, save :: gwptrjczdat
      real(dop), dimension(:,:,:), allocatable, save :: gwptrjxspl
      real(dop), dimension(:,:,:), allocatable, save :: gwptrjpspl
      real(dop), dimension(:,:),   allocatable, save :: gwptrjespl
      real(dop), dimension(:,:,:), allocatable, save :: gwptrjcdspl
      real(dop), dimension(:,:,:), allocatable, save :: gwptrjczspl

!-----------------------------------------------------------------------
! arrays for:
! gwptrjeovl: electronic function overlaps
! gwptrjetau: RHS time derivative of electronic function overlaps
! gwptrjci: CI coefficients 
!-----------------------------------------------------------------------
      complex(dop), dimension(:,:), allocatable, save :: gwptrjeovl
      complex(dop), dimension(:,:), allocatable, save :: gwptrjetau
      complex(dop), dimension(:,:), allocatable, save :: gwptrjci

!-----------------------------------------------------------------------
! name of directory with trajectories
!-----------------------------------------------------------------------
      character(len=c5), save :: gwptrjdir = ' ', gwptrjdir0 = ' '
      integer(long), save     :: gwptrjlaenge = 0, gwptrjlaenge0 = 0

!-----------------------------------------------------------------------
! the no. of current trajectory, state and time to pass from gh_elements 
! to funcqch1 to calculate potential and used in calculation of time
! deriv of ci vector
!-----------------------------------------------------------------------
      integer(long), save   :: curr_trj,curr_st
      real(dop), save       :: curr_time

      contains

!#######################################################################

      subroutine alloc_gwptrj

      implicit none

      integer(long)     :: ierr
      character(len=12) :: cerr
      integer(long), external   :: integersize,realsize,complexsize

      cerr = 'gwptrjnp'
      allocate(gwptrjnp(ngwptrj),stat=ierr)
      allocmemory=allocmemory + ngwptrj*integersize()
      if (ierr > 0) go to 1000
      cerr = 'gwptrjtdat'
      allocate(gwptrjtdat(ngwptrjpnts,ngwptrj),stat=ierr)
      allocmemory=allocmemory + ngwptrj*ngwptrjpnts*realsize()
      if (ierr > 0) go to 1000
      cerr = 'gwptrjxdat'
      allocate(gwptrjxdat(ngwptrjpnts,ngwptrjcoo,ngwptrj),stat=ierr)
      allocmemory=allocmemory + ngwptrjcoo*ngwptrj*ngwptrjpnts*realsize()
      if (ierr > 0) go to 1000
      cerr = 'gwptrjxspl'
      allocate(gwptrjxspl(ngwptrjpnts,ngwptrjcoo,ngwptrj),stat=ierr)
      allocmemory=allocmemory + ngwptrjcoo*ngwptrj*ngwptrjpnts*realsize()
      if (ierr > 0) go to 1000
      cerr = 'gwptrjpdat'
      allocate(gwptrjpdat(ngwptrjpnts,ngwptrjcoo,ngwptrj),stat=ierr)
      allocmemory=allocmemory + ngwptrjcoo*ngwptrj*ngwptrjpnts*realsize()
      if (ierr > 0) go to 1000
      cerr = 'gwptrjpspl'
      allocate(gwptrjpspl(ngwptrjpnts,ngwptrjcoo,ngwptrj),stat=ierr)
      allocmemory=allocmemory + ngwptrjcoo*ngwptrj*ngwptrjpnts*realsize()
      if (ierr > 0) go to 1000
      cerr = 'gwptrjedat'
      allocate(gwptrjedat(ngwptrjpnts,ngwptrj),stat=ierr)
      allocmemory=allocmemory + ngwptrj*ngwptrjpnts*realsize()
      if (ierr > 0) go to 1000
      cerr = 'gwptrjespl'
      allocate(gwptrjespl(ngwptrjpnts,ngwptrj),stat=ierr)
      allocmemory=allocmemory + ngwptrj*ngwptrjpnts*realsize()
      if (ierr > 0) go to 1000
      cerr = 'gwptrjcddat'
      allocate(gwptrjcddat(ngwptrjpnts,ngwptrjst,ngwptrj),stat=ierr)
      allocmemory=allocmemory + ngwptrj*ngwptrjst*ngwptrjpnts*realsize()
      if (ierr > 0) go to 1000
      cerr = 'gwptrjcdspl'
      allocate(gwptrjcdspl(ngwptrjpnts,ngwptrjst,ngwptrj),stat=ierr)
      allocmemory=allocmemory + ngwptrj*ngwptrjst*ngwptrjpnts*realsize()
      if (ierr > 0) go to 1000
      cerr = 'gwptrjczdat'
      allocate(gwptrjczdat(ngwptrjpnts,ngwptrjst,ngwptrj),stat=ierr)
      allocmemory=allocmemory + ngwptrj*ngwptrjst*ngwptrjpnts*realsize()
      if (ierr > 0) go to 1000
      cerr = 'gwptrjczspl'
      allocate(gwptrjczspl(ngwptrjpnts,ngwptrjst,ngwptrj),stat=ierr)
      allocmemory=allocmemory + ngwptrj*ngwptrjst*ngwptrjpnts*realsize()
      if (ierr > 0) go to 1000
      cerr = 'gwptrjeovl'
      allocate(gwptrjeovl(maxgwp,maxgwp),stat=ierr)
      allocmemory=allocmemory + maxgwp*maxgwp*complexsize()
      if (ierr > 0) go to 1000
      cerr = 'gwptrjetau'
      allocate(gwptrjetau(maxgwp,maxgwp),stat=ierr)
      allocmemory=allocmemory + maxgwp*maxgwp*complexsize()
      cerr = 'gwptrjci'
      allocate(gwptrjci(ngwptrjst,maxgwp),stat=ierr)
      allocmemory=allocmemory + ngwptrjst*maxgwp*complexsize()
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_GWPTRJ :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_gwptrj

!#######################################################################

      subroutine dealloc_gwptrj

      implicit none

      if (allocated(gwptrjnp)) deallocate(gwptrjnp)
      if (allocated(gwptrjtdat)) deallocate(gwptrjtdat)
      if (allocated(gwptrjxdat)) deallocate(gwptrjxdat)
      if (allocated(gwptrjxspl)) deallocate(gwptrjxspl)
      if (allocated(gwptrjpdat)) deallocate(gwptrjpdat)
      if (allocated(gwptrjpspl)) deallocate(gwptrjpspl)
      if (allocated(gwptrjedat)) deallocate(gwptrjedat)
      if (allocated(gwptrjespl)) deallocate(gwptrjespl)
      if (allocated(gwptrjcddat)) deallocate(gwptrjcddat)
      if (allocated(gwptrjcdspl)) deallocate(gwptrjcdspl)
      if (allocated(gwptrjczdat)) deallocate(gwptrjczdat)
      if (allocated(gwptrjczspl)) deallocate(gwptrjczspl)
      if (allocated(gwptrjeovl)) deallocate(gwptrjeovl)
      if (allocated(gwptrjci)) deallocate(gwptrjci)

      end subroutine dealloc_gwptrj

!#######################################################################

      subroutine memgwptrj

      implicit none

      integer(long)     :: ilbl,jlbl,klbl,iend
      integer(long)     :: f,n,s,npnts
      integer(long)     :: slen
      logical(kind=4)   :: lexist
      character(len=c5) :: filename,string,extcmd
      character(len=c2) :: rdtag

!-----------------------------------------------------------------------
! if gwptrj file present in name directory, read sizes. 
! Otherwise find the files to be fitted and count
!-----------------------------------------------------------------------
      filename = gwptrjdir(1:gwptrjlaenge)//'/gwptrj_spl.dat'
      inquire(file=filename,exist=lexist)

      if (lexist) then
! read file

         ilbl=slen(filename)
         open(igwptrj,file=filename,form='unformatted',status='old',err=900)

         do
  10        continue
            read(igwptrj,iostat=iend,err=10) rdtag
            if (iend .lt. 0) exit
            if (rdtag(1:1) .ne. '<') cycle

            ilbl=2
            jlbl=index(rdtag,'>')-1
            if (rdtag(ilbl:jlbl) .eq. 'ngwptrjpnts') then
               read(igwptrj,err=999) ngwptrjpnts
            else if (rdtag(ilbl:jlbl) .eq. 'ngwptrjcoo') then
               read(igwptrj,err=999) ngwptrjcoo
            else if (rdtag(ilbl:jlbl) .eq. 'ngwptrj') then
               read(igwptrj,err=999) ngwptrj
            else if (rdtag(ilbl:jlbl) .eq. 'ngwptrjst') then
               read(igwptrj,err=999) ngwptrjst

            else if (rdtag(ilbl:jlbl) .eq. 'end array dimensions') then
                exit

            endif

         enddo

         close(igwptrj)

      else

! get gwptrj directory name
         filename=oname(1:olaenge)//'/oper'
         ilbl=index(filename,' ')-1
         open(ioper,file=filename,form='unformatted',status='old',err=900)
         do
   20       continue
            read(ioper,err=20,end=200) rdtag
            if (rdtag(1:1) .ne. '<') cycle

            ilbl=2
            jlbl=index(rdtag,'>')-1
            if (rdtag(ilbl:jlbl) .eq. 'gwptrjlaenge') then
               read(ioper,err=999) gwptrjlaenge

            else if (rdtag(ilbl:jlbl) .eq. 'gwptrjdir') then
               read(ioper,err=999) gwptrjdir
            endif
         enddo
  200    continue
         if (gwptrjdir(1:1) .ne. '/') call abspath(gwptrjdir,gwptrjlaenge)
         close(ioper)

! Get sizes from data files
         jlbl=slen(strpid)
         filename=gwptrjdir(1:gwptrjlaenge)//'/gwptrj.ls-'//strpid(1:jlbl)
         ilbl=slen(filename)
         extcmd = "ls "//gwptrjdir(1:gwptrjlaenge)//" > "//filename(1:ilbl)
         call excmd(extcmd)
         open(idump,file=filename,status='old',form='formatted',err=900)

         rewind(idump)
         ngwptrjcoo=0
         ngwptrj=0
         ngwptrjpnts=0
         ngwptrjst=0
         do 
            read(idump,'(a32)',iostat=iend,err=910) string
            if (iend .lt. 0) exit
            klbl=slen(string)

            if (string(klbl-3:klbl) .ne. 'xdat' .and. &
                string(klbl-3:klbl) .ne. 'pdat' .and. &
                string(klbl-3:klbl) .ne. 'edat' .and. &
                string(klbl-4:klbl) .ne. 'cidat') cycle

! file format is f.n.xdat. ilbl,jlbl mark .
! f = dof, n = gwp
! or ds.n.cidat
! s = adiabatic state with 0 as ground state
            if (string(klbl-3:klbl) .eq. 'xdat') then
               jlbl=klbl-4
               ilbl=index(string(1:jlbl-1),'.')

               if (ilbl .gt. 0) read(string(1:ilbl-1),*) f
               read(string(ilbl+1:jlbl-1),*) n
               ngwptrjcoo=max(ngwptrjcoo,f)
               ngwptrj=max(ngwptrj,n)
            else if (string(klbl-4:klbl) .eq. 'cidat') then
               ilbl=index(string(1:klbl),'.')-1

               read(string(2:ilbl),*) s
               ngwptrjst=max(ngwptrjst,s+1)
            endif

            filename=gwptrjdir(1:gwptrjlaenge)//'/'//string(1:klbl)
            ilbl=slen(filename)
            open(itmpin,file=filename,status='old',form='formatted',err=900)
            read(itmpin,*) npnts
            close(itmpin)
            ngwptrjpnts = max(ngwptrjpnts,npnts)
               
         enddo

         close(idump,status='delete')
      endif

      return
  900 continue
      write(ilog,'(a)') 'Error in MEMGWPTRJ opening file: '//filename(1:ilbl)
      stop
  910 continue
      write(ilog,'(a)') 'Error in MEMGWPTRJ reading file: '//filename(1:ilbl)
      stop
  999 continue
      write(ilog,'(a)') 'Error in MEMGWPTRJ reading gwptrj file: '//filename(1:ilbl)
      stop

      end subroutine memgwptrj

!#######################################################################

      end module gwptrj

