!#######################################################################
!
!#######################################################################

      module maxv

use decimal, only: dop, long
use global, only: c2, c5, lgwp, ldd, dentype, lddmctdh, ltraj, &
                  lsocbas, lrunml, ldynspf, ldbsmall, name, laenge,&
                  lcivec

      implicit none

!-----------------------------------------------------------------------
! variable used to sum up amount of memory allocated
!-----------------------------------------------------------------------
      real(dop), save :: allocmemory = 0.0_dop

!-----------------------------------------------------------------------
! PARAMETERS FOR ARRAY DIMENSIONS
!  maxdim:     maximum number of degrees of freedom 
!  maxsta:     maximum number of states (actually maximum no. of
!              sets of wavepackets e.g. for different states).
!  maxpack: no. of packets to be propagated
!  maxgwp: maximum number of GWPS
!-----------------------------------------------------------------------
      integer(long), save :: maxdim = 1
      integer(long), save :: maxsta = 1
      integer(long), save :: maxpack = 1
      integer(long), save :: maxgwp = 1

!-----------------------------------------------------------------------
! maxkoe: Maximum number of hamiltonian terms.
! maxham: Maximum number of Hamiltonians 
! maxhop: Maximum number of operators in hoplab list
! maxhtm: Maximum number of built up operator terms in hterm list
! maxfac: Maximum number of factors in a hterm operator
! maxrdf: Maximum number of files read containing operators 
!         Also used for operator labels info.
! maxdef: Maximum number of labels defined in labels-section
! maxmoco: maximum no. of MO coeffs in a DD calculation
! maxqcsta: maximum no. of QC states in a DD calculation
!-----------------------------------------------------------------------
      integer(long), save :: maxkoe = 1
      integer(long), save :: maxham = 1
      integer(long), save :: maxhop = 1
      integer(long), save :: maxhtm = 1
      integer(long), save :: maxfac = 1
      integer(long), save :: maxrdf = 1

      integer(long), save :: maxhpar = 15
      integer(long), save :: maxmuld = 32
      integer(long), save :: maxLMR = 120
      integer(long), save :: maxdef = 1
      integer(long), save :: maxmoco = 1
      integer(long), save :: maxqcsta = 1

!-----------------------------------------------------------------------
! maxpes: Maximum number of pes operators (used in potfit)
!-----------------------------------------------------------------------
      integer(long), save :: maxpes = 1

!-----------------------------------------------------------------------
! Data for splines
!
! nsplpaths : No. of paths for directories containing spline fits
!             by default one default path is defined for spline files
!             Actual no. of splpaths set in einop
! nsplnames : No. of spline fits
!             Actual no. of splnames set in getsplnames
!-----------------------------------------------------------------------
      integer(long), save :: nsplpaths=1
      integer(long), save :: nsplnames=1

!-----------------------------------------------------------------------
! parameter maxnhtmshift:
! maximum number of different htmshift values for mode m.
! (this must be global as it is needed in runprop.inc and operdef.inc)
!-----------------------------------------------------------------------
      integer(long), save :: maxnhtmshift = 200

!-----------------------------------------------------------------------
! maxset: No. sets of data, e.g. QC data files as used in VCHAM programs.
!-----------------------------------------------------------------------
      integer(long), save      :: maxset    = 2000 !SVN:2000

!-----------------------------------------------------------------------
! length of WF definition in ML. 
!-----------------------------------------------------------------------
      integer(long), save :: maxtape=1

!-----------------------------------------------------------------------
! parameters used if old format files are read 
!-----------------------------------------------------------------------
      integer(long), parameter :: maxdimo = 64  !SVN:32
      integer(long), parameter :: maxstao = 32    !32  !SVN:16
      logical(kind=4)          :: lmemold = .false.

!-----------------------------------------------------------------------
! Maximum no. or records in DB
! (dbmemdim0  allows passing of DB size via keywords)
!-----------------------------------------------------------------------
      integer(long), save :: dbmemdim = 10000
      integer(long), save :: dbmemdim0 = -999

!-----------------------------------------------------------------------
! integers to check largest array sizes
!
! minkoe   : min. no. of Hamiltonian terms
! minfac   : min. no. of factors in Hamiltonian terms
! nhparmax : min. no. of parameters defining operator labels
! nhlabmax : min. no. of operator labels defined in labels-section
!-----------------------------------------------------------------------
      integer(long), save :: minfac,minkoe,nhparmax,nhlabmax

!-----------------------------------------------------------------------
! maxcivec   : max. no. of configurations in CI vector (lcivec = T)
! maxmultblk : max no. of states in a spin block (ignoring degeneracy).
!-----------------------------------------------------------------------
      integer(long), save   :: maxcivec = 0
      integer(long), save   :: maxmultblk = 0

!-----------------------------------------------------------------------
! directory with alloc file if reading results from MCTDH8.6
!-----------------------------------------------------------------------
      character(len=c5) :: alloc86file= ' '

      public

     contains

!#######################################################################
!
! Variables in maxv are written here
!
!#######################################################################

      subroutine wrmemdim(unit)

      implicit none

      integer(long),intent(in)  :: unit
      logical(kind=4), external :: log2int
      character(len=c2)         :: wrtag

      wrtag= '<start array dimensions>'
      write(unit) wrtag

      wrtag= '<maxdim>'
      write(unit) wrtag
      write(unit) maxdim
      wrtag= '<maxsta>'
      write(unit) wrtag
      write(unit) maxsta
      wrtag= '<maxqcsta>'
      write(unit) wrtag
      write(unit) maxqcsta
      wrtag= '<maxgwp>'
      write(unit) wrtag
      write(unit) maxgwp
      wrtag= '<maxkoe>'
      write(unit) wrtag
      write(unit) maxkoe
      wrtag= '<maxham>'
      write(unit) wrtag
      write(unit) maxham
      wrtag= '<maxhop>'
      write(unit) wrtag
      write(unit) maxhop
      wrtag= '<maxhtm>'
      write(unit) wrtag
      write(unit) maxhtm
      wrtag= '<maxfac>'
      write(unit) wrtag
      write(unit) maxfac
      wrtag= '<maxhpar>'
      write(unit) wrtag
      write(unit) maxhpar
      wrtag= '<maxmuld>'
      write(unit) wrtag
      write(unit) maxmuld
      wrtag= '<maxdef>'
      write(unit) wrtag
      write(unit) maxdef
      wrtag= '<maxLMR>'
      write(unit) wrtag
      write(unit) maxLMR
      wrtag= '<maxrdf>'
      write(unit) wrtag
      write(unit) maxrdf
      wrtag= '<maxtape>'
      write(unit) wrtag
      write(unit) maxtape
      wrtag= '<maxpack>'
      write(unit) wrtag
      write(unit) maxpack
      wrtag= '<maxmoco>'
      write(unit) wrtag
      write(unit) maxmoco
      wrtag= '<nsplpaths>'
      write(unit) wrtag
      write(unit) nsplpaths
      wrtag= '<nsplnames>'
      write(unit) wrtag
      write(unit) nsplnames
      wrtag= '<dbmemdim>'
      write(unit) wrtag
      write(unit) dbmemdim

!----------------------------------------------------------------------- 
! add global definitions required for memory allocation
!----------------------------------------------------------------------- 
      wrtag= '<dentype>'
      write(unit) wrtag
      write(unit) dentype
      wrtag= '<ldd>'
      write(unit) wrtag
      write(unit) log2int(ldd)
      wrtag= '<lddmctdh>'
      write(unit) wrtag
      write(unit) log2int(lddmctdh)
      wrtag='<lgwp>'
      write(unit) wrtag
      write(unit) log2int(lgwp)
      wrtag='<ltraj>'
      write(unit) wrtag
      write(unit) log2int(ltraj)
      wrtag='<ldynspf>'
      write(unit) wrtag
      write(unit) log2int(ldynspf)
      wrtag='<lsocbas>'
      write(unit) wrtag
      write(unit) log2int(lsocbas)
      wrtag='<lrunml>'
      write(unit) wrtag
      write(unit) log2int(lrunml)
      wrtag='<ldbsmall>'
      write(unit) wrtag
      write(unit) log2int(ldbsmall)
      wrtag='<lcivec>'
      write(unit) wrtag
      write(unit) log2int(lcivec)
      wrtag='<maxcivec>'
      write(unit) wrtag
      write(unit) maxcivec
      wrtag='<maxmultblk>'
      write(unit) wrtag
      write(unit) maxmultblk

      wrtag= '<end array dimensions>'
      write(unit) wrtag

      return
      end subroutine wrmemdim

!#######################################################################

      subroutine rdmemdim(unit)

      implicit none

      integer(long),intent(in)  :: unit
      integer(long)             :: ilbl,jlbl
      integer(long)             :: idd,igwp,itraj,isocbas,iddmctdh,&
                                   irunml,idbsmall,idynspf,icivec
      real(dop)                 :: filever
      logical(kind=4), external :: int2log
      character(len=c2)         :: rdtag


      rewind(unit)

! read file version
      read(unit,err=999) filever

!-----------------------------------------------------------------------
! 8.6 read alloc file.
! pre 11.0 versions - assign fixed values. 
! post 11.0 read values
!-----------------------------------------------------------------------
      if ((filever .lt. 8.65_dop) .and. (filever .gt. 8.60_dop)) then  
         call rdalloc86

      else if (filever .lt. 11.00_dop) then
         maxdim = maxdimo
         maxsta = maxstao
         maxkoe=2000
         maxham=64
         maxhop=2000
         maxhtm=2000
         maxfac=200
         maxhpar=15
         maxmuld=32
         maxLMR=120
         maxrdf=100
         maxtape = 1000
         maxpack = 1
         maxcivec = 0
         maxmultblk = 0

         dentype = 0
         lgwp = .false.
         ldd=.false.
         lddmctdh=.false.
         lcivec = .false.

         lmemold=.true.

      else

         maxqcsta=-999

! read to start of memory dimensions
         do
   10       continue
            read(unit,err=10,end=100) rdtag
            if (rdtag(1:1) .ne. '<') cycle

            ilbl=index(rdtag,'<')+1
            jlbl=index(rdtag,'>')-1
            if (rdtag(ilbl:jlbl) .eq. 'start array dimensions') exit
! old oper files used this
            if (rdtag(ilbl:jlbl) .eq. 'array dimensions') exit
         enddo

! read memory dimensions
         do
   20       continue
            read(unit,err=20,end=100) rdtag
            if (rdtag(1:1) .ne. '<') cycle

            ilbl=index(rdtag,'<')+1
            jlbl=index(rdtag,'>')-1
            if (rdtag(ilbl:jlbl) .eq. 'maxdim') then
               read(unit,err=900) maxdim
            else if (rdtag(ilbl:jlbl) .eq. 'maxsta') then
               read(unit,err=900) maxsta
            else if (rdtag(ilbl:jlbl) .eq. 'maxqcsta') then
               read(unit,err=900) maxqcsta
            else if (rdtag(ilbl:jlbl) .eq. 'maxgwp') then
               read(unit,err=900) maxgwp
            else if (rdtag(ilbl:jlbl) .eq. 'maxkoe') then
               read(unit,err=900) maxkoe
            else if (rdtag(ilbl:jlbl) .eq. 'maxtape') then
               read(unit,err=900) maxtape
            else if (rdtag(ilbl:jlbl) .eq. 'maxkoe') then
               read(unit,err=900) maxkoe
            else if (rdtag(ilbl:jlbl) .eq. 'maxham') then
               read(unit,err=900) maxham
            else if (rdtag(ilbl:jlbl) .eq. 'maxhop') then
               read(unit,err=900) maxhop
            else if (rdtag(ilbl:jlbl) .eq. 'maxhtm') then
               read(unit,err=900) maxhtm
            else if (rdtag(ilbl:jlbl) .eq. 'maxfac') then
               read(unit,err=900) maxfac
            else if (rdtag(ilbl:jlbl) .eq. 'maxhpar') then
               read(unit,err=900) maxhpar
            else if (rdtag(ilbl:jlbl) .eq. 'maxmuld') then
               read(unit,err=900) maxmuld
            else if (rdtag(ilbl:jlbl) .eq. 'maxdef') then
               read(unit,err=900) maxdef
            else if (rdtag(ilbl:jlbl) .eq. 'maxLMR') then
               read(unit,err=900) maxLMR
            else if (rdtag(ilbl:jlbl) .eq. 'maxrdf') then
               read(unit,err=900) maxrdf
            else if (rdtag(ilbl:jlbl) .eq. 'maxtape') then
               read(unit,err=900) maxtape
            else if (rdtag(ilbl:jlbl) .eq. 'maxpack') then
               read(unit,err=900) maxpack
            else if (rdtag(ilbl:jlbl) .eq. 'maxmoco') then
               read(unit,err=900) maxmoco
            else if (rdtag(ilbl:jlbl) .eq. 'nsplpaths') then
               read(unit,err=900) nsplpaths
            else if (rdtag(ilbl:jlbl) .eq. 'nsplnames') then
               read(unit,err=900) nsplnames
            else if (rdtag(ilbl:jlbl) .eq. 'dbmemdim') then
               read(unit,err=900) dbmemdim

!----------------------------------------------------------------------- 
! read global definitions required for memory allocation
!----------------------------------------------------------------------- 
            else if (rdtag(ilbl:jlbl) .eq. 'dentype') then
               read(unit,err=900) dentype
            else if (rdtag(ilbl:jlbl) .eq. 'ldd') then
               read(unit,err=900) idd
               ldd = int2log(idd)
            else if (rdtag(ilbl:jlbl) .eq. 'lddmctdh') then
               read(unit,err=900) iddmctdh
               lddmctdh = int2log(iddmctdh)
            else if (rdtag(ilbl:jlbl) .eq. 'lgwp') then
               read(unit,err=900) igwp
               lgwp = int2log(igwp)
            else if (rdtag(ilbl:jlbl) .eq. 'ltraj') then
               read(unit,err=900) itraj
               ltraj = int2log(itraj)
            else if (rdtag(ilbl:jlbl) .eq. 'ldynspf') then
               read(unit,err=900) idynspf
               ldynspf = int2log(idynspf)
            else if (rdtag(ilbl:jlbl) .eq. 'lsocbas') then
               read(unit,err=900) isocbas
               lsocbas = int2log(isocbas)
            else if (rdtag(ilbl:jlbl) .eq. 'lrunml') then
               read(unit,err=900) irunml
               lrunml = int2log(irunml)
            else if (rdtag(ilbl:jlbl) .eq. 'ldbsmall') then
               read(unit,err=900) idbsmall
               ldbsmall = int2log(idbsmall)
            else if (rdtag(ilbl:jlbl) .eq. 'lcivec') then
               read(unit,err=900) icivec
               lcivec = int2log(icivec)
            else if (rdtag(ilbl:jlbl) .eq. 'maxcivec') then
               read(unit,err=900) maxcivec
            else if (rdtag(ilbl:jlbl) .eq. 'maxmultblk') then
               read(unit,err=900) maxmultblk

            else if (rdtag(ilbl:jlbl) .eq. 'end array dimensions') then
               exit
            endif
         enddo

         if (maxqcsta .lt. 0) maxqcsta = maxsta ! not in older filesets

      endif

  100 continue
      return
  900 write(6,'(a,i5)') 'Error reading file on unit: ',unit
      stop
  999 write(6,'(a,i5)') 'Error reading file version on unit: ',unit
      stop

      end subroutine rdmemdim

!#######################################################################

      subroutine rdalloc86

      implicit none

      integer(long)     :: itmpin = 24
      integer(long)     :: flen
      character(len=c5) :: filename
      character(len=80) :: string

!-----------------------------------------------------------------------
! It is assumed that the alloc file is in the name directory (usual
! for analysis runs) unless alloc86file is set, e.g. in chnpot where
! alloc is read from old natpot directory.
!-----------------------------------------------------------------------
      flen=index(alloc86file,' ')-1
      if (flen .gt. 0) then
         filename = alloc86file
      else
         filename = name(1:laenge)//'/alloc'
      endif
      open(itmpin,file=filename,status='old',form='formatted',err=900)

      do
         read(itmpin,'(a)') string
         if (index(string,'end-alloc') .ne. 0) then
            exit
         else if (index(string,'maxdim') .ne. 0) then
            read(string,'(14x,i5)') maxdim
         else if (index(string,'maxsta') .ne. 0) then
            read(string,'(14x,i5)') maxsta    
         else if (index(string,'maxtape') .ne. 0) then
            read(string,'(14x,i5)') maxtape   
         else if (index(string,'maxdepth') .ne. 0) then
            continue   ! no longer used
!            read(string,'(14x,i5)') maxdepth  
         else if (index(string,'maxham') .ne. 0) then
            read(string,'(14x,i5)') maxham    
         else if (index(string,'maxpar') .ne. 0) then
            continue   ! not needed in post-processing
!            read(string,'(14x,i5)') maxpar    
         else if (index(string,'maxhpa') .ne. 0) then
            read(string,'(14x,i5)') maxhpar   
         else if (index(string,'maxpes') .ne. 0) then
            read(string,'(14x,i5)') maxpes    
         else if (index(string,'maxrdf') .ne. 0) then
            read(string,'(14x,i5)') maxrdf    
         else if (index(string,'maxkoe') .ne. 0) then
            read(string,'(14x,i5)') maxkoe    
         else if (index(string,'maxhtm') .ne. 0) then
            read(string,'(14x,i5)') maxhtm    
         else if (index(string,'maxhop') .ne. 0) then
            read(string,'(14x,i5)') maxhop    
         else if (index(string,'maxfac') .ne. 0) then
            read(string,'(14x,i5)') maxfac    
         else if (index(string,'maxdef') .ne. 0) then
            read(string,'(14x,i5)') maxdef    
         else if (index(string,'maxmuld') .ne. 0) then
            read(string,'(14x,i5)') maxmuld   
         else if (index(string,'maxsub') .ne. 0) then
            continue   ! not needed in post-processing
!            read(string,'(14x,i5)') maxsub
         else if (index(string,'maxLMR') .ne. 0) then
            read(string,'(14x,i5)') maxLMR    
         else if (index(string,'maxedim') .ne. 0) then
            continue   ! not needed in post-processing
!            read(string,'(14x,i5)') maxedim
         else if (index(string,'maxreadspf') .ne. 0) then
            continue   ! not needed in post-processing
!            read(string,'(14x,i5)') maxreadspf
         else if (index(string,'maxnhtmshift') .ne. 0) then
            read(string,'(14x,i5)') maxnhtmshift 
         endif

      enddo

      close(itmpin)

      return

  900 write(6,'(a)') 'Error in Rdalloc86: File not found '
      write(6,'(a)') filename(1:flen)
      stop

      end subroutine rdalloc86


      end module maxv

