!***********************************************************************
!
!              MCTDH include file  MPI.INC
!
!  MCTDH include file defining array sizes, common blocks etc. common
!  to several parts of the program
!  Declarations independent of modes and states should be placed
!  in include file "global".
!
!  010/08 MRB
!  10/11 Altered to include extra integer definitions.
!        Name changed to mpidata to avoid conflict with "use mpi" 
!        statement which defines the MPI environment. GWR
!
!***********************************************************************
      module mpidata

      use decimal, only : dop, long

      implicit none
      public

!-----------------------------------------------------------------------
! INFORMATION ABOUT NUMBER OF PROCESSES TO BE STARTED
!
! nmpi : number of processes that should be used
!
! mpirank : number of the current process
!
!-----------------------------------------------------------------------
      integer(long), save :: nmpi,mpirank,intordermpi

!-----------------------------------------------------------------------
! Defines the mpi error flag, identity of master and task label.
!-----------------------------------------------------------------------
      integer(long)       :: mpierr
      integer(long), save :: master, mpitask

!-----------------------------------------------------------------------
! Defines the mpi tag for receive and send process
!-----------------------------------------------------------------------
      integer(long),parameter :: tag0=0

!-----------------------------------------------------------------------
! If set, master has stopped on an error so stop slaves.
!-----------------------------------------------------------------------
      logical(kind=4) :: lstopslv

!-----------------------------------------------------------------------
! output from decomp1d (mpi_comm_world)
!-----------------------------------------------------------------------
      integer(long), save :: lstartmpi, lendmpi, psistartmpi, psiendmpi     

!-----------------------------------------------------------------------
! DECIDES WHETHER MPI IS USED OR NOT
!
!  lmpi  : set to true if MPI is used
!-----------------------------------------------------------------------
      logical(kind=4), save :: lmpi,lmpicalcha,lmpifunka2,lmpimfields, &
                               lmpidsyev,lmpidavrun,lmpidav,lmpigetdavmat, &
                               ldavkey,lptimall,lmpistarted,lmpiphihphi, &
                               lmpihlochphi,lmpisummf,lmpigetdiag,&
                               lmpigetddpes,lmpiexpect

!-----------------------------------------------------------------------
! TIMING VARIABLES.
!-----------------------------------------------------------------------
!      integer(long), save :: tmpi
      logical(kind=4), save :: lpartim

!-----------------------------------------------------------------------
! THIS IS FOR OPTIMIZING THE MPI-PARALLELIZATION
!
! Currently only used for calling hlochphi during improved relaxation.
! (mpitask=9).  
! lmpiupdt is sent first and then the indecated entities are sent 
! accordingly.
!
! NOTE: For 'historic reasons' these are all .true. at start-up
!       and should be reset to .true. before exit of any routine 
!       that uses them. I will change this in the near future,  
!       but for now this 
!
!  nmpiupdlen  : length of the lmpiupdt array
!
! Update-Flags:
!
!  lmpiupdt(1) : switch-flag: set  lmpiupdt(i) to lmpiupdt(1) after 
!                sending or recieving entity i. 
!
! Later, these will be context dependent. For now:
!  lmpiupdt(2) : nham, ktype, mode, phidim1, zetf1
!---------------------------------------------------------------------
      integer(long), parameter                :: nmpiupdtlen=2 !PRCS:2
      logical(kind=4), dimension(nmpiupdtlen) :: lmpiupdt
	
!----------------------------------------------------------------------
! THIS IS FOR AVOIDING INTERNAL INTEGER OVERFLOWS IN
! MPI ROUTINES FOR LARGE ARRAYS
!
! mpimaxchunk: send arrays in chunks of at most this number of elements
!----------------------------------------------------------------------

      integer, parameter :: mpimaxchunk=10000000

!---------------------------------------------------------------------
! MPI task numbers.
!---------------------------------------------------------------------
! for potfit
      integer(long), parameter :: mpitask_pf_genpot=1

!---------------------------------------------------------------------
! SCALAPACK
!     MBdefault : default block size of
!     the block-cyclic destribution
!---------------------------------------------------------------------
! for potfit
      integer, parameter :: MBdefault=32

!---------------------------------------------------------------------
! MPI for expectwf
!---------------------------------------------------------------------
      logical(kind=4), save :: lexpectval = .false.

      end module mpidata
