!-----------------------------------------------------------------------
!    source/include/oct.f90
!    For optimal control
!-----------------------------------------------------------------------
      module oct

      use decimal, only : dop, long
      use global,  only : c2, c5

      implicit none

!-----------------------------------------------------------------------
! Pointers to memory used in efield and jfunc:
!
! mrfld      pointer to field in real*8 buffer
! mrpnlty    pointer to penalty function in real*8 buffer
! mrref      pointer to reference function in real*8 buffer
! mrlagr     (parts of) the Lagrange multiplicator (filter)
! mrpref     filtered (1-P) reference function (jfunc.F)
!-----------------------------------------------------------------------

      integer(long), save :: mrfld,mrpnlty,mrref,mrlagr,mrqref,mrqfld
      integer(long), save :: mcdtpsi,mcovr,mcovr1,mcdover

      integer(long), save :: doverdim,adim1,dgldim1,dmatdim1,ovmaxblock

integer(long), save :: ovtype,ebtype
logical(kind=4), save :: lback

integer(long), dimension(:,:), allocatable, save   :: dovmat

!-----------------------------------------------------------------------
! Output channels in efield that remain open:
!
! itarfile    time-dependent target population
! idipfile    time-dependent dipole matrix of target and initial state
! iovlpfile   time-dependent overlap of initial and target state
! irawfldfile time-dependent output of the mkfield routine
!-----------------------------------------------------------------------
      integer(long), save :: itarfile,idipfile,iovlpfile,irawfldfile

!-----------------------------------------------------------------------
! Input dvr file in efield:
!-----------------------------------------------------------------------
      character(len=240),save :: dvrfile

!-----------------------------------------------------------------------
! Array sizes in efield:
!
! nfin       number of points of the field (time steps)
! nfin1      alternate number of points
!-----------------------------------------------------------------------
      integer(long), save :: nfin,nfin1

!-----------------------------------------------------------------------
! Counters and other integers
!
! iiter      counting of iterations
! extorder   order of extrapolation of the field
! ntarop     operator used as target operator
!-----------------------------------------------------------------------
      integer(long), save :: iiter, extorder, ntarop

!-----------------------------------------------------------------------
! real*8 variables used for optimal control
! 
! pnlty      penalty factor for efield
!-----------------------------------------------------------------------
      real(dop), save :: pnlty

!-----------------------------------------------------------------------
! Character variables used for optimal control and efield
! 
! tarop   label of the target operator (if target is an operator)
! filpnlty   File containing penalty function
! filref     File containing reference function
!-----------------------------------------------------------------------
      character(len=c2), save  :: tarop
      character(len=c5), save  :: filpnlty,filref,fillagr

!-----------------------------------------------------------------------
! logical variables used for optimal control
! Boolean flags used in efield:
! 
! ltarop       Target is an operator
! lmultar      Multiple targets are to be optimized
! lwrpop       Write target population as function of time and iteration
! lalgnphs     Align the phases of multiple target states
! lpnlty       A penalty factor is given
! lpnltyfnc    A penalty function is given
! lreffnc      A reference function is given
! ltdovlp      Time-dependent overlap is used
! lwrdipmtx    Write <psi_tar(t)|mu|psi_init(t)>
! lwrrawfld    Write raw field
!              -Im <psi_in|psi_tar><psi_tar(t)|mu|psi_in(t)>
! lwrovlp      Write <psi_tar(t)|psi_in(t)>
! llagrfnc     Load Lagrange multiplier
! lkrotov      Use Krotov algorythm
!-----------------------------------------------------------------------

      logical(kind=4), save :: ltarop,lmultar,lwrpop,lalgnphs, &
                               lpnlty,lpnltyfnc,lreffnc,ltdovlp,lwrdipmtx, &
                               lwrrawfld,lwrovlp,llagrfnc,lkrotov


!#######################################################################
!#########                                                     #########
!#########        Variables used by genfld                     #########
!#########                                                     #########
!#######################################################################
!-----------------------------------------------------------------------
! maxfldop   Maximum number of function definitions
! fldfunclab Labels of functions
! lrdfnc     Read functions from .op file
!-----------------------------------------------------------------------
      integer(long), parameter :: maxfldop = 12 !SVN:12

      character(len=c2), dimension(maxfldop), save :: fldfunclab
      logical(kind=4), save                        :: lrdfnc

!#######################################################################
!#########                                                     #########
!#########        Variables used by fldop                      #########
!#########                                                     #########
!#######################################################################

      integer(long), dimension(maxfldop), save :: fldoptype,fldiparam
      integer(long), save                      :: nfldop
      real(dop), dimension(maxfldop)           :: fldrparam
      character(len=c2), dimension(maxfldop), save :: fldaddprm

      integer(long), save   :: mcfld,mcfld1,mcffthin,mcfftrueck,mcref
      logical(kind=4), save :: lminus

      end module oct

