!***********************************************************************
!                                                                     
!              MCTDH include file  operdef.inc
!                                                                     
!***********************************************************************

      module operdef
 
      use decimal, only : long,dop,vlong
      use global

use maxv
use channels
use combined
use mltypes, only: meta_mlop_t,mlham_tp

      implicit none

!-----------------------------------------------------------------------
! parameters defining operator modes (may be different from system)
!-----------------------------------------------------------------------
      integer(long), save :: nopmode,nopdof

!-----------------------------------------------------------------------
! mass(f)  : (reduced) mass for dof f
!-----------------------------------------------------------------------
      real(dop), dimension(:), allocatable, save :: mass

!-----------------------------------------------------------------------
! POINTERS FOR ARRAYS
!-----------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save :: zhop

!-----------------------------------------------------------------------
! the hterm and associated arrays
!
! htmdof(h)  : dof on which hterm operates. If mode or mD operator htmdof(h)=0.
! htmmode(h) : mode on which hterm operates. If 1D or mD operator htmmode(h)=0.
! htmmuld(h) : mD grid on which hterm operates. If 1D or mode operator 
!              htmmuld(h)=0.
! htmadof(h) : actual DOF on which hterm operates.
! htmfac(h)  : no. of factors in hterm
!-----------------------------------------------------------------------
      integer(long), dimension(:,:), allocatable, save :: hterm
      integer(long), dimension(:), allocatable, save   :: htmdof,htmmode, &
                                                          htmmuld,htmfac, &
                                                          htmform,htmsym, &
                                                          htmadof,htmshift, &
                                                          htmndof,htmderiv

!-----------------------------------------------------------------------
! SOME MORE PARAMETERS NEEDED FOR HAMILTONIAN
!
! nopts(n)  : number of parameters stored in pesopts(n) for nth surface
!-----------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save   :: hopfile,hopilab, &
                                                          hopidef
      integer(long), dimension(:,:), allocatable, save :: hopipar
      integer(long), save                              :: hopdim,npesopt, &
                                                          nhoppar

      integer(long), dimension(:,:), allocatable, save     :: zpesopts
      integer(long), dimension(:), allocatable, save       :: nopts
      character(len=c5), dimension(:), allocatable, save   :: pesopts

      real(dop), dimension(:,:), allocatable, save     :: hoprpar,hoppar
      character(len=c2), save                              :: hoplab0
      character(len=c2), dimension(:), allocatable, save   :: hoplab
      integer(long), save                                :: nrdf

!-----------------------------------------------------------------------
! parameters used to cut high/low values of correlated PES surface,
! and parameters for natpotcut (used for the cut-off of tiny natpot terms, 
! or operator terms)
!-----------------------------------------------------------------------
      logical(kind=4), save                          :: lcutpes = .false.
      logical(kind=4), save                          :: lsoftcutpes = .false.
      logical(kind=4), save                          :: lcutpesall = .false.
      logical(kind=4), save                          :: lsoftpesall = .false.
      logical(kind=4), save                          :: lncutall = .false.

      real(dop), save                                ::  pesmin = -1.0d6
      real(dop), save                                ::  pesmax = 1.0d6
      real(dop), save                                ::  cutoff = tiny
      real(dop), save                                ::  natpotcut = tiny

      integer(long), save                            :: nremoved,ncutn

!-----------------------------------------------------------------------
! flags governing operator generation
!
!  loptcontract : try harder when summing Hamiltonian terms to
!                 optimise contraction.
!-----------------------------------------------------------------------
      logical(kind=4), save :: loptcontract = .false.

!-----------------------------------------------------------------------
! properties of the PES surface
!
! iqconst(f) : if 1, and dof is not in specified system, coordinate
!              is set to value in rqconst(f)
!              if 0, and dof is not in specified system, coordinate
!              is set to 0.0d0
!              if dof is in specified system, this flag is ignored.
!-----------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save  :: iqconst
      real(dop), save                                 :: vmin,vmax
      real(dop), dimension(:), allocatable, save      :: rqconst

!-----------------------------------------------------------------------
! arrays storing information for multi-D operators
!
! mulddim: no. of multi-D sets of coordinates
!
! ldirmuld: if true, direct algorithm used for multi-dimensional 
!           operators
!-----------------------------------------------------------------------
      integer(long), save                              :: mulddim
      integer(long), dimension(:), allocatable, save   :: muldsize,nmulddof
      integer(long), dimension(:,:), allocatable, save :: mulddof,mulddof2
      logical(kind=4), save                            :: ldirmuld
      logical(kind=4), save                            :: lmuld
      integer(long), dimension(:), allocatable, save   :: vdmuld,ndmuld

!-----------------------------------------------------------------------
! PARAMETERS defining (natpot) files containing operators
!-----------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save     :: npignore
      integer(long), dimension(:,:), allocatable, save   :: npmuld
      integer(vlong), dimension(:), allocatable, save    :: nplocal
      character(len=c5), dimension(:), allocatable, save :: rdfiles
      integer(long), dimension(:), allocatable, save     :: ncutted
      real(dop), dimension(:), allocatable, save         :: npotcut
      real(dop), dimension(:), allocatable, save         :: ncutrlabel
      real(dop), dimension(:,:), allocatable, save       :: wpf
      character(len=c2), dimension(:), allocatable, save :: ncutlabel

!-----------------------------------------------------------------------
! PARAMETERS NEEDED FOR HAMILTONIAN
!
! hopsdim : length of array containing hops
!  lreal  : Hamiltonian and WF are real (improved relaxation).
!  lcap   : calculation employing CAPs (or complex potentials)
!  lcdvr  : any multidimensional Hamiltonian terms use the CDVR method
!  ltdvr  : any multidimensional Hamiltonian terms use the TDVR method
!  lanalpes : an analytic PES will be generated (e.g. for CDVR)
!  lanalderiv : analytic derivatives used in GWP LHA if possible
!  ltdh   : time-dependent Hamiltonian
!  lmlop  : Multi-Layer operators are used.
!-----------------------------------------------------------------------
      integer(long), save                      :: hopsdim,ktot
      logical(kind=4), save                    :: lreal=.false., &
                                                  lrreal=.false.,&
                                                  lcap=.false.,&
                                                  lcdvr=.false.,&
                                                  ltdvr=.false.,&
                                                  lanalpes=.false.,&
                                                  lanalderiv=.true.,&
                                                  lmlop=.false.
      logical(kind=4), dimension(:), allocatable, save :: ltdh

!-----------------------------------------------------------------------
! SOME MORE PARAMETERS NEEDED FOR HAMILTONIAN
!
! koeff(k)  : expansion coefficients of correlated Hamiltonian
! diag(m,k)   : array of unit operator spf hamiltonian matrices
! diagg(m,k) : array of unit operator gwp hamiltonian matrices
!              (set in setoperpsi)
! hsym(m,k)   : array of operator symmetries
! hlsym(m,k)  : array of hloch symmetries
! diagh(m)    : auxiliary diag array
! kinop(f,k)  : =1 if a correlated Hamiltonian term is non-diagonal,
!               =0 if diagonal,
!               <0 if side-diagonal.
! laddmode(m,nh)  : logical set to true if the mode m is to be included
!                   into the operator nh. (See keyword 'addmode').   
!
! hamdim       : No. of Hamiltonians
! hamlab(nham) : string with name of nham-th Hamiltonian.
! zham(nham)   : pointer to start of nham-th Hamiltonian
! ndef      : no. of operator labels defined in labels-section
! deflabel  : Label of defined operator. E.g. "V" for a natpot.
! deflabop  : Type of defined operator, e.g. "srffile".
! khtot(nham)    : no. or terms in operator nham
! kzahl(nham)    : no. of correlated terms in operator nham
! khzahl(m,s,nham) : number of uncorrelated term in nham for dimension m
!                    and state s
! hamilton(f,k)  : integer with no. of kth Hamiltonian term for dof f.
! hamLMR(i,n)    : integer with no. of ith LMR term. n=1,2,3 -> R,M,L.
! zhun(m,s,nham) : pointer to start of uncorrelated terms in hamilton array
! ldissop: indicates if dissipative operators are involved
! ldissin: indicates if input wishes dissipative treatment
! ndp:     number of dipole terms
! ldproj:  indicates if the equations of motion of the parameters
!          shall be modified
! nhtmshift(m) : number of different htmshift values for mode m
! ihtmshift(i,m) : i-th htmshift value for mode m
! ki(k),kf(k): defines states for Hamiltonian term <kf|H|ki> in 
!              multi-set calculation.
! ddsi(h),ddsf(h): defines states <sf|H|si> for DD-Hamiltonian term h 
! ksi(k),ksf(k): defines states <sf|H|si> for Hamiltonian term k in
!                single-set calculation.  
! fterm(k): the DOF under which hterm info is stored for term k
! hmlop(k)  : kth Hamiltonian term is used for an MLOp
! h00 : used to prevent summation of a term
! hamnum(k) : Hamiltonian to which term k belongs
!-----------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save       :: kzahl,nmulpot,&
                                                              khtot
      integer(long), dimension(:), allocatable, save       :: ki,kf,ksi,ksf,&
                                                              fterm
      integer(long), dimension(:,:), allocatable, save     :: kinop
      integer(long), dimension(:), allocatable, save       :: ddsi,ddsf
      integer(long), dimension(:), allocatable, save       :: nhtmshift
      integer(long), dimension(:,:), allocatable, save     :: ihtmshift
      integer(long), dimension(:,:), allocatable, save     :: hamLMR
      integer(long), dimension(:), allocatable, save       :: hmlop 
      integer(long), save                                  :: iLMR

      complex(dop), dimension(:), allocatable, save        :: koeff

      logical(kind=4), dimension(:), allocatable, save     :: diagh
      logical(kind=4), dimension(:,:), allocatable, save   :: diag
      logical(kind=4), dimension(:,:), allocatable, save   :: diagg
      logical(kind=4), dimension(:,:), allocatable, save   :: laddmode
      logical(kind=4), save                                :: ldissop=.false.,&
                                                              ldissin=.false., &
                                                              ldproj

      integer(long), dimension(:,:,:), allocatable, save   :: khzahl,zhun
      integer(long), dimension(:,:), allocatable, save     :: hamilton,hsym, &
                                                              hlsym
      integer(long), save                                  :: htmdim,hamdim, &
                                                              nhamint0,h00
      logical(kind=4), save                                :: loplogfull = .false.
      integer(long), dimension(:), allocatable, save       :: htmtype
      integer(long), dimension(:), allocatable, save       :: zham
      integer(long), dimension(:), allocatable, save       :: hamnum
      character(len=c2), dimension(:), allocatable, save   :: hamlab
      integer(long), save                                  :: ndef
      character(len=c2), dimension(:), allocatable, save   :: deflabel,deflabop

!-----------------------------------------------------------------------
! nodiag : flag to turn off use of diag flag and extraction of huncorr terms.
!-----------------------------------------------------------------------
      logical(kind=4), dimension(:), allocatable, save :: nodiag, &
                                                          usepack

!-----------------------------------------------------------------------
!
! Flags for operator action
!
! kfirst, kact, ktwin
!
! kfirst: flags first occurence of operator term. Set up in zeighpsi.
! ktwin points to the twin operator in a dissipative part. 
! kact: 1: for wave-functions
!       2: pure-left
!       3: pure-right
!       4: left-right	
!       5: twin of left-right for dentype=1
! klha(k) = n+1: If > 0 term is a correlated LHA operator in a GWP 
!              calculation. Order of term is n.
!-----------------------------------------------------------------------
      integer(long), dimension(:), allocatable,save    ::   kact,ktwin
      integer(long), dimension(:,:), allocatable, save ::   kfirst
      integer(long), dimension(:), allocatable, save   ::   klha

!-----------------------------------------------------------------------
! Variables needed for autocap option
! 
! autocap(k) : if > 0, k the term includes an autocap on dof f=autocap(k).
!              The cap is then activated when it is populated.
! indicap(f) : number of grid point where the CAP starts
! gpopcap(f) : grid population at the point where the CAP starts
! timecap(f) : time, after which the acap is allowed to be switched on.
!
! autoabs(k), autoflux(k) : flags for cap / flux calculations with a 
!                           realWP
!-----------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save  :: indicap
      integer(long), dimension(:), allocatable, save  :: autocap,nstatecap
      integer(long), dimension(:), allocatable, save  :: autoabs,autoflux

      real(dop), dimension(:), allocatable, save      :: gpopcap,timecap

!-----------------------------------------------------------------------
! lhaorder: order of potential expansion for GWP operators. Default = 2 (LHA)
! lklha : set true if G-MCTDH with correlated LHA
!-----------------------------------------------------------------------
      integer(long), save              :: lhaorder = 2
      logical(kind=4), save            :: lklha = .false.

!-----------------------------------------------------------------------
! Information about natpot terms:
! modc          - the contracted mode of the natpots in Potfit
! vpdim(m)      - Cumulative number of preceeding natural potentials for
!                 mode m in Potfit
! pdim(m)       - Number of natural potentials for mode m in Potfit
! npdim(m)      - Cumulative number of following natural potentials for
!                 mode m in Potfit
! maxsubpdim    - maximum size of a sub-particle grid (for diagonal
!                 terms acting on some but not all of the mode's DOFs)
! onedipot      - integers defining 1d potential mode
!-----------------------------------------------------------------------
      integer(long), save                            :: modc,maxsubpdim
      integer(long), dimension(:), allocatable, save :: vpdim,pdim,npdim
      integer(long), dimension(:), allocatable, save :: onedipot

!-----------------------------------------------------------------------
! Information need by "fast" feature
!
! mmodc(n)      - the contracted mode of the n-th natpot in MCTDH
! vmdim(m,n)    - Cumulative number of preceeding natural potentials for
!                 mode m for the n-th natpot in MCTDH 
! mdim(m,n)     - Number of natural potentials for mode m for the n-th 
!                 natpot in MCTDH 
! nmdim(m,n)    - Cumulative number of following natural potentials for 
!                 mode m for the n-th natpot in MCTDH 
!-----------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save   :: mmodc
      integer(long), dimension(:,:), allocatable, save :: vmdim,mdim, &
                                                          nmdim

!-----------------------------------------------------------------------
! Data for spline operators
!
! splpaths  : Paths to spline files
! splnames  : Names of spline files
! splunits  : Units used for coordinates + energy in spline file
!-----------------------------------------------------------------------
      character(len=c5), dimension(:), allocatable, save   :: splpaths
      character(len=c2), dimension(:), allocatable, save   :: splnames
      character(len=c1), dimension(:,:), allocatable, save :: splunits

!-----------------------------------------------------------------------
! variables for MLops
!-----------------------------------------------------------------------
integer,parameter      :: maxnmlop = 256
integer,save           :: nmlop = 0
type(meta_mlop_t),save :: mmlops(maxnmlop)
type(mlham_tp),allocatable,save :: mlhams(:)

      contains

!#######################################################################
!
! OPERDEF_ALLOC
!
! memory required for arrays in operdef allocated here.
!
! First check if allocated, then allocate. All arrays must be deallocated in 
! operdef_dealloc
!
!#######################################################################

      subroutine alloc_operdef

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr
      integer(long), external :: integersize, realsize, logicalsize

! check all arrays are not allocated
      call dealloc_operdef

      call alloc_hterm
      call alloc_hoplab
      call alloc_combined
! alloc arrays that use maxrdf
      call alloc_rdfiles

      cerr = 'mass'
      allocate(mass(maxdim),stat=ierr)
      allocmemory=allocmemory + maxdim*realsize()
      if (ierr > 0) go to 1000
      cerr = 'iqconst'
      allocate(iqconst(maxdim),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'rqconst'
      allocate(rqconst(maxdim),stat=ierr)
      allocmemory=allocmemory + maxdim*realsize()
      if (ierr > 0) go to 1000
      cerr = 'muldsize'
      allocate(muldsize(maxmuld),stat=ierr)
      allocmemory=allocmemory + maxmuld*integersize()
      if (ierr > 0) go to 1000
      cerr = 'nmulddof'
      allocate(nmulddof(maxmuld),stat=ierr)
      allocmemory=allocmemory + maxmuld*integersize()
      if (ierr > 0) go to 1000
      cerr = 'mulddof'
      allocate(mulddof(maxdim,maxmuld),stat=ierr)
      allocmemory=allocmemory + maxdim*maxmuld*integersize()
      if (ierr > 0) go to 1000
      cerr = 'mulddof2'
      allocate(mulddof2(maxdim,maxmuld),stat=ierr)
      allocmemory=allocmemory + maxdim*maxmuld*integersize()
      if (ierr > 0) go to 1000
      cerr = 'vdmuld'
      allocate(vdmuld(maxmuld),stat=ierr)
      allocmemory=allocmemory + maxmuld*integersize()
      if (ierr > 0) go to 1000
      cerr = 'ndmuld'
      allocate(ndmuld(maxmuld),stat=ierr)
      allocmemory=allocmemory + maxmuld*integersize()
      if (ierr > 0) go to 1000
      cerr = 'ltdh'
      allocate(ltdh(maxham),stat=ierr)
      allocmemory=allocmemory + maxham*logicalsize()
      if (ierr > 0) go to 1000
      cerr = 'kzahl'
      allocate(kzahl(maxham),stat=ierr)
      allocmemory=allocmemory + maxham*integersize()
      if (ierr > 0) go to 1000
      cerr = 'nmulpot'
      allocate(nmulpot(maxham),stat=ierr)
      allocmemory=allocmemory + maxham*integersize()
      if (ierr > 0) go to 1000
      cerr = 'khtot'
      allocate(khtot(maxham),stat=ierr)
      allocmemory=allocmemory + maxham*integersize()
      if (ierr > 0) go to 1000
      cerr = 'ki'
      allocate(ki(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'kf'
      allocate(kf(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'ksi'
      allocate(ksi(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'ksf'
      allocate(ksf(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'fterm'
      allocate(fterm(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'kinop'
      allocate(kinop(maxdim,maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*maxdim*logicalsize()
      if (ierr > 0) go to 1000
      cerr = 'nhtmshift'
      allocate(nhtmshift(maxdim),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'ihtmshift'
      allocate(ihtmshift(maxnhtmshift,maxdim),stat=ierr)
      allocmemory=allocmemory + maxnhtmshift*maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'hamLMR'
      allocate(hamLMR(maxlmr,3),stat=ierr)
      allocmemory=allocmemory + maxlmr*3*integersize()
      cerr = 'hmlop'
      allocate(hmlop(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'koeff'
      allocate(koeff(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'diagh'
      allocate(diagh(maxdim),stat=ierr)
      allocmemory=allocmemory + maxdim*logicalsize()
      if (ierr > 0) go to 1000
      cerr = 'diag'
      allocate(diag(maxdim,maxkoe),stat=ierr)
      allocmemory=allocmemory + maxdim*maxkoe*logicalsize()
      if (ierr > 0) go to 1000
      cerr = 'diagg'
      allocate(diagg(maxdim,maxkoe),stat=ierr)
      allocmemory=allocmemory + maxdim*maxkoe*logicalsize()
      if (ierr > 0) go to 1000
      cerr = 'laddmode'
      allocate(laddmode(maxdim,maxham),stat=ierr)
      allocmemory=allocmemory + maxdim*maxham*logicalsize()
      if (ierr > 0) go to 1000
      cerr = 'khzahl'
      allocate(khzahl(maxdim,maxsta,maxham),stat=ierr)
      khzahl=0
      allocmemory=allocmemory + maxdim*maxsta*maxham*integersize()
      if (ierr > 0) go to 1000
      cerr = 'zhun'
      allocate(zhun(maxdim,maxsta,maxham),stat=ierr)
      zhun=1
      allocmemory=allocmemory + maxdim*maxsta*maxham*integersize()
      if (ierr > 0) go to 1000
      cerr = 'hamilton'
      allocate(hamilton(maxdim,maxkoe),stat=ierr)
      allocmemory=allocmemory + maxdim*maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'hsym'
      allocate(hsym(maxdim,maxkoe),stat=ierr)
      allocmemory=allocmemory + maxdim*maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'hlsym'
      allocate(hlsym(maxdim,maxkoe),stat=ierr)
      allocmemory=allocmemory + maxdim*maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'zham'
      allocate(zham(maxham),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'hamnum'
      allocate(hamnum(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'hamlab'
      allocate(hamlab(maxham),stat=ierr)
      allocmemory=allocmemory + c2*maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'nodiag'
      allocate(nodiag(maxham),stat=ierr)
      allocmemory=allocmemory + maxkoe*logicalsize()
      if (ierr > 0) go to 1000
      cerr = 'usepack'
      allocate(usepack(maxham),stat=ierr)
      allocmemory=allocmemory + maxham*logicalsize()
      if (ierr > 0) go to 1000
      cerr = 'kact'
      allocate(kact(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'ktwin'
      allocate(ktwin(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'kfirst'
      allocate(kfirst(maxdim,maxkoe),stat=ierr)
      allocmemory=allocmemory + maxdim*maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'klha'
      allocate(klha(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'indicap'
      allocate(indicap(maxdim),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'autocap'
      allocate(autocap(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'nstatecap'
      allocate(nstatecap(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxkoe*integersize()
      if (ierr > 0) go to 1000
      cerr = 'gpopcap'
      allocate(gpopcap(maxdim),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'timecap'
      allocate(timecap(maxdim),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'autoabs'
      allocate(autoabs(maxkoe),stat=ierr)     
      if (ierr > 0) go to 1000                
      cerr = 'autoflux'
      allocate(autoflux(maxkoe),stat=ierr)
      if (ierr > 0) go to 1000 
      cerr = 'vpdim'
      allocate(vpdim(maxdim),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'pdim'
      allocate(pdim(maxdim),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'npdim'
      allocate(npdim(maxdim),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'onedipot'
      allocate(onedipot(maxdim),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'deflabel'
      allocate(deflabel(maxdef))
      allocmemory=allocmemory + c2*maxdef*integersize()
      if (ierr > 0) go to 1000
      cerr = 'deflabop'
      allocate(deflabop(maxdef))
      allocmemory=allocmemory + c2*maxdef*integersize()
      if (ierr > 0) go to 1000
      cerr = 'mlhams'
      allocate(mlhams(maxham))
      allocmemory=allocmemory + maxham*integersize()
      if (ierr > 0) go to 1000
      cerr = 'splpaths '
      allocate(splpaths(nsplpaths),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'splnames '
      allocate(splnames(nsplnames),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'splunits '
      allocate(splunits(4,nsplnames),stat=ierr)
      if (ierr > 0) go to 1000

!-----------------------------------------------------------------------
! Set defaults
!-----------------------------------------------------------------------
      kfirst(:,:) = 0
      klha(:) = 0

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_OPERDEF :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_operdef

!#######################################################################

      subroutine dealloc_operdef

      implicit none
      call dealloc_hterm
      call dealloc_hoplab
      call dealloc_combined
      call dealloc_rdfiles
      if (allocated(mass)) deallocate(mass)
      if (allocated(iqconst)) deallocate(iqconst)
      if (allocated(rqconst)) deallocate(rqconst)
      if (allocated(muldsize)) deallocate(muldsize)
      if (allocated(nmulddof)) deallocate(nmulddof)
      if (allocated(mulddof)) deallocate(mulddof)
      if (allocated(mulddof2)) deallocate(mulddof2)
      if (allocated(vdmuld)) deallocate(vdmuld)
      if (allocated(ndmuld)) deallocate(ndmuld)
      if (allocated(ltdh)) deallocate(ltdh)
      if (allocated(kzahl)) deallocate(kzahl)
      if (allocated(nmulpot)) deallocate(nmulpot)
      if (allocated(khtot)) deallocate(khtot)
      if (allocated(ki)) deallocate(ki)
      if (allocated(kf)) deallocate(kf)
      if (allocated(ksi)) deallocate(ksi)
      if (allocated(ksf)) deallocate(ksf)
      if (allocated(fterm)) deallocate(fterm)
      if (allocated(kinop)) deallocate(kinop)
      if (allocated(nhtmshift)) deallocate(nhtmshift)
      if (allocated(ihtmshift)) deallocate(ihtmshift)
      if (allocated(hamLMR)) deallocate(hamLMR)
      if (allocated(hmlop)) deallocate(hmlop)
      if (allocated(koeff)) deallocate(koeff)
      if (allocated(diagh)) deallocate(diagh)
      if (allocated(diag)) deallocate(diag)
      if (allocated(diagg)) deallocate(diagg)
      if (allocated(laddmode)) deallocate(laddmode)
      if (allocated(khzahl)) deallocate(khzahl)
      if (allocated(zhun)) deallocate(zhun)
      if (allocated(hamilton)) deallocate(hamilton)
      if (allocated(hsym)) deallocate(hsym)
      if (allocated(hlsym)) deallocate(hlsym)
      if (allocated(zham)) deallocate(zham)
      if (allocated(hamnum)) deallocate(hamnum)
      if (allocated(hamlab)) deallocate(hamlab)
      if (allocated(nodiag)) deallocate(nodiag)
      if (allocated(usepack)) deallocate(usepack)
      if (allocated(kact)) deallocate(kact)
      if (allocated(ktwin)) deallocate(ktwin)
      if (allocated(kfirst)) deallocate(kfirst)
      if (allocated(klha)) deallocate(klha)
      if (allocated(indicap)) deallocate(indicap)
      if (allocated(autocap)) deallocate(autocap)
      if (allocated(nstatecap)) deallocate(nstatecap)
      if (allocated(gpopcap)) deallocate(gpopcap)
      if (allocated(timecap)) deallocate(timecap)
      if (allocated(autoabs)) deallocate(autoabs)
      if (allocated(autoflux)) deallocate(autoflux)
      if (allocated(vpdim)) deallocate(vpdim)
      if (allocated(pdim)) deallocate(pdim)
      if (allocated(npdim)) deallocate(npdim)
      if (allocated(onedipot)) deallocate(onedipot)
      if (allocated(deflabel)) deallocate(deflabel)
      if (allocated(deflabop)) deallocate(deflabop)
      if (allocated(mlhams)) deallocate(mlhams)
      if (allocated(splpaths)) deallocate(splpaths)
      if (allocated(splnames)) deallocate(splnames)
      if (allocated(splunits)) deallocate(splunits)

      return
      end subroutine dealloc_operdef

!#######################################################################
!
! HTERM_ALLOC
!
! memory required for arrays associated with hterm allocated here.
!
! First check if allocated, then allocate. All arrays must be deallocated in 
! hterm_dealloc
!
!#######################################################################

      subroutine alloc_hterm


      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_hterm

      cerr = 'hterm'
      allocate(hterm(maxfac,maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'htmdof'
      allocate(htmdof(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'htmadof'
      allocate(htmadof(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'htmmode'
      allocate(htmmode(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'htmmuld'
      allocate(htmmuld(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'htmfac'
      allocate(htmfac(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'htmform'
      allocate(htmform(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'htmsym'
      allocate(htmsym(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'htmshift'
      allocate(htmshift(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'htmtype'
      allocate(htmtype(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'htmndof'
      allocate(htmndof(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'htmderiv'
      allocate(htmderiv(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'zhop'
      allocate(zhop(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ddsi'
      allocate(ddsi(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ddsf'
      allocate(ddsf(maxhtm),stat=ierr)
      if (ierr > 0) go to 1000

! initialise variables
      hterm=0
      htmdof=0
      htmadof=0
      htmmode=0
      htmmuld=0
      htmfac=0
      htmform=0
      htmsym=0
      htmshift=0
      htmtype=0
      htmndof=0
      htmderiv=0
      zhop=0
      ddsi=0
      ddsf=0

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_HTERM :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop

      end subroutine alloc_hterm

!#######################################################################

      subroutine dealloc_hterm

      implicit none

      if (allocated(hterm)) deallocate(hterm)
      if (allocated(htmdof)) deallocate(htmdof)
      if (allocated(htmadof)) deallocate(htmadof)
      if (allocated(htmmode)) deallocate(htmmode)
      if (allocated(htmmuld)) deallocate(htmmuld)
      if (allocated(htmfac)) deallocate(htmfac)
      if (allocated(htmform)) deallocate(htmform)
      if (allocated(htmsym)) deallocate(htmsym)
      if (allocated(htmshift)) deallocate(htmshift)
      if (allocated(htmtype)) deallocate(htmtype)
      if (allocated(htmndof)) deallocate(htmndof)
      if (allocated(htmderiv)) deallocate(htmderiv)
      if (allocated(zhop)) deallocate(zhop)
      if (allocated(ddsi)) deallocate(ddsi)
      if (allocated(ddsf)) deallocate(ddsf)

      return
      end subroutine dealloc_hterm

!#######################################################################
!
! HOPLAB_ALLOC
!
! memory required for arrays associated with hoplab allocated here.
!
! First check if allocated, then allocate. All arrays must be deallocated in 
! hoplab_dealloc
!
!#######################################################################

      subroutine alloc_hoplab

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_hoplab

      cerr = 'hoplab'
      allocate(hoplab(maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'hopfile'
      allocate(hopfile(maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'hopilab'
      allocate(hopilab(maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'hopidef'
      allocate(hopidef(maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'hopipar'
      allocate(hopipar(maxhpar,maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'hoprpar'
      allocate(hoprpar(maxhpar,maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'hoppar'
      allocate(hoppar(maxhpar,maxhop),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_HOPLAB :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_hoplab

!#######################################################################

      subroutine dealloc_hoplab

      implicit none

      if (allocated(hoplab)) deallocate(hoplab)
      if (allocated(hopfile)) deallocate(hopfile)
      if (allocated(hopilab)) deallocate(hopilab)
      if (allocated(hopidef)) deallocate(hopidef)
      if (allocated(hopipar)) deallocate(hopipar)
      if (allocated(hoprpar)) deallocate(hoprpar)
      if (allocated(hoppar)) deallocate(hoppar)

      return
      end subroutine dealloc_hoplab

!#######################################################################
!
! Files allocated here have to be included in inc_rdfiles(heinutil.F90)
! which resizes the arrays during operator setup
!
!#######################################################################

      subroutine alloc_rdfiles

use maxv
use channels

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_rdfiles

      cerr = 'rdfiles'
      allocate(rdfiles(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'npotcut'
      allocate(npotcut(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'npignore'
      allocate(npignore(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'npmuld'
      allocate(npmuld(maxrdf,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'nplocal'
      allocate(nplocal(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ncutted'
      allocate(ncutted(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'mmodc'
      allocate(mmodc(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'vmdim'
      allocate(vmdim(maxdim,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'mdim'
      allocate(mdim(maxdim,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'nmdim'
      allocate(nmdim(maxdim,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000

      cerr = 'zpesopts'
      allocate(zpesopts(2*maxhpar,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'nopts'
      allocate(nopts(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'pesopts'
      allocate(pesopts(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000

      cerr = 'ncutrlabel'
      allocate(ncutrlabel(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ncutlabel'
      allocate(ncutlabel(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'wpf'
      allocate(wpf(maxdim,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_RDFILES :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      write(6,'(a)') 'Error allocating memory in ALLOC_RDFILES :',cerr
      write(6,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_rdfiles

!#######################################################################

      subroutine dealloc_rdfiles

      if (allocated(rdfiles)) deallocate(rdfiles)
      if (allocated(npotcut)) deallocate(npotcut)
      if (allocated(npignore)) deallocate(npignore)
      if (allocated(npmuld)) deallocate(npmuld)
      if (allocated(nplocal)) deallocate(nplocal)
      if (allocated(ncutted)) deallocate(ncutted)
      if (allocated(mmodc)) deallocate(mmodc)
      if (allocated(vmdim)) deallocate(vmdim)
      if (allocated(mdim)) deallocate(mdim)
      if (allocated(nmdim)) deallocate(nmdim)

      if (allocated(zpesopts)) deallocate(zpesopts)
      if (allocated(nopts)) deallocate(nopts)
      if (allocated(pesopts)) deallocate(pesopts)

      if (allocated(ncutrlabel)) deallocate(ncutrlabel)
      if (allocated(ncutlabel)) deallocate(ncutlabel)
      if (allocated(wpf)) deallocate(wpf)

      return
      end subroutine dealloc_rdfiles

      end module operdef
