!*****************************************************************
!
!       POTFIT include data:      PGLOBAL.F90
!
!  POTFIT include file defining various parameters and variables       
!  global to the POTFIT program.     
!
!     10/96 AJ
!
!******************************************************************
      module pglobal

      use decimal, only : long, dop
      use global, only : c1, c4,c5

      implicit none

      logical(kind=4) :: lprojection=.false.

!-----------------------------------------------------------------------
! MAXIMUM VALUES FOR STATIC ARRAYS
!
! maxproj   : no more than this number of project_XXX lines are allowed
! maxprojdof: each projector can project onto this number of degrees
!             of freedom at most
! maxprojpar: the functions which are used to project onto the DOFs
!             can have at most this number of parameters (but integer,
!             logical, real... parameters are counted separately)
!-----------------------------------------------------------------------
      integer(long), parameter :: maxproj = 20,maxprojdof=4,maxprojpar = 2
      integer(long), parameter :: maxvec=32

!#######################################################################
! memory pointers
!*********************************************************************
      integer(long), save :: &
              mrvpot,mrcoeff,mrcoeff2,mrevec,mrvvec,mfvvec, &
              mroned,mrwght, mrwghti ,mrevec2,mrvex,mreval, &
              mrwork1,mrwork2,mrwork3,mrwork4,mrvproj,mrvperr, &
              mrbuf,mfbuf,mrcut,mrort1,mrvbuf

      integer(long), save :: miwvec


!---------------------------------------------------------------------
! LOGICALS TO CONTROL INPUT/OUTPUT OPTIONS AND RUN MODI
!
! lausw  : to tell 'pzeiger' whether it is called from 
!          'potfit' (false) or from 'pauswert' (true)
! litf   : whether data is written to file 'iteration'
! lcw    : whether correlated weights are used
! lwarn  : is set to 'true' in subroutine 'warnmsg' 
! lgit   : whether data is written to file 'pgitter'
! lit    : whether 'niters' iteration steps are performed without
!          prompting the user
! litia  : interactive iteration modus
! lprodw : whether data is written to file 'prodwei'
! lsw    : whether separable weights are used
! lcspot : whether coupled states centrifugal potential is included
! ploned : whether one dimensional potentials are to be subtracted
! lsubtract : whether a general potential is to be subtracted
! subtrpes: name of directory for pes to be sutracted
! lensubtr: length of name of directory for pes to be sutracted
!           if lensubtr=0 .and. lsubtract then pes is in name
! lshort : whether a short test run is executed (no calculations)
! lsp    : whether the potential energy vector `vvec' should be
!          of the single precision real (`float') type
! lwo    : whether we want only calculate the potential on the product
!          grid and write it to a file ("write-only")
! lwef   : whether every fit should be written to a separate natpot file
! lreadvpot: whether a name for a vpot file has been specified
! lgenvpot : whether a vpot file should be generated unconditionally
! ldelvpot : whether the vpotfile should be deleted after the run
! ldvronly : only a dvr file should be generated
! loutfil  : output file opend.
! loutl    : long form of output written.
! lproject : true for projection runs
! lprojerr : true if an error measure is calculated in projection runs
! lprojascii : true if projected potentials should also be written as
!              ASCII files
! lprojlowmem: true for low-memory mode of projection
! lchkmc   : true if an error is caused if the mode combinations in the
!            vpot file differ from the system's
! lstop    : true if the stop-file signals to stop the program.
! lwpot    : true if external weights defining the relevant region are given
! lwrcw    : true if correlated weights are to be written
! lwmap    : true if correlated weights which are read are to be mapped
!            form DOF order to mode order (indexmap).
!            POTFIT writes the weights in DOF order.
!
! lpesfile     : true potfit reads a pes-file
! lspenergy    : true potfit is requested to compute and compare a single point
!                energy (potfit and PES subroutine) for a given set of DVR indices
!                located in the array isppoint
! lcutalong    : true potfit computes a cut along the PES (potfit and PES subroutine)
!                for a given DOF. The rest of the DOFs are set up to the values given
!                in the isppoint array.
! ltr          : if the coordiantes of the surface require a transformation 
!
! MGPF:
!
! lmgpf       : MGPF calculation
! lextpot     : ext-natpot file
! lprintrho   : true potfit prints the corresponding rho file(s)
! ldcoeff     : true computes D coeffs
! lfinemode   : true fine mode modus
! lfullcoarse : true compute natpots and natweights on full rep
! lonlydcoeff : true computes D on fine grid and
!               reads cfullN for natpots on full rep
! lfiterr     : true computes fitting error
! lctensor    : true computes evec for ALL modes and computes the ctensor
! lwrallevec  : true writes evec (evecb) files for ALL modes incl. modc
! lcutnpot    : reads existing natpot and reduces size acc to NATPOT-SECTION
! lrdnpot     : reads existing natpot and computes fiterr (if possible)
! lnpotpath   : in cutnpot, .true. means the natpot file to be read is located
!               in the npotpath location
!----------------------------------------------------------------------
!----------------------------------------------------------------------
      logical(kind=4), save     :: &
                               lprodw,loutfil,lausw,litf,loutl,lgit,lit, &
                               litia,lcspot,ploned,lshort,lsp,lwo,lwef, &
                               lreadvpot,lgenvpot,ldelvpot,lproject,lprojerr, &
                               lprojlowmem,lprojascii,lchkmc,lstop,lsubtract, &
                               lwpot,lwrcw,lwmap,lextpot,lprintrho,ldcoeff, &
                               lfinemode,lfullcoarse,lonlydcoeff,lfiterr,lmgpf,&
                               lpesfile,lwrallevec,lcutnpot,lrdnpot,&
                               lnpotpath,lspenergy,lcutalong,lctensor,ltr
      logical(kind=4), save     :: ldvronly = .false.
      logical(kind=4), save     :: lsw = .false.
      logical(kind=4), save     :: lcw = .false.

      character(len=72), save   :: stopcommand
      integer(long), save       :: lensubtr
      character(len=c5), save   :: subtrpes

!-----------------------------------------------------------------------
! INTEGERS RELATED TO MODUS OF PROGRAM
!
! fitcount:
!         if lwef is true every fit is written to a separate file
!         `natpot.iii', where `iii' is the contents of `fitcount' starting
!         with zero for the first fit.
! npest : set via keyword `pestiming'. if >0 the program calculates the
!         exact potential and the fit `npest' times over the complete
!         grid. this is done after the first fit in order to compare the
!         cpu times associated with the calculation of the exact potentials
!         and the fit.
! ifinemode  : mode to be computed on the fine grid
!-----------------------------------------------------------------------
      integer(long), save :: fitcount,npest,ifinemode,vpotfmt

!-----------------------------------------------------------------------
! VARIABLES ASSOCIATED WITH THE POTENTIAL SURFACE
!
! ippes : holds the number of the potential surface
! niobuf: length of i/o buffer used for vpot file
!-----------------------------------------------------------------------
       integer(long), save :: ippes,niobuf

!-----------------------------------------------------------------------
! VARIABLES SPECIFYING 'IWEIGHT' AND 'ITERATION' MODUS
!  niw     : # of grid points with non-zero 'iweight'
!  wsum2   : sum of squares of weights on grid points with 
!                        non-zero 'iweight'
!  swsum2  : sum of squares of sep. weights. (=vpotdim if no sep.weights) 
!  rdcut   : translational coordinate cut off [au]
!  rvcut   : vibrational coordinate cut off [au]
!  iwvmax  : energy cut off in [au]
!  iwvmin  : energy cut off in [au]
!  niters  : number of iteration steps in non-interactive 
!            iteration modus
!-----------------------------------------------------------------------
      integer(long), save :: niw,niters,njump
      real(dop), save      :: iwvmax,iwvmin,wsum2,swsum2

!-----------------------------------------------------------------------
! SOME ERROR MEASURES
! rmserr: weighted and relevant rms-error 
! urms  :              relevant rms-error
! allrms:                       rms-error
! maxerr: modulus of relevant maximum error
! maxall: modulus of          maximum error
!-----------------------------------------------------------------------
      real(dop), save :: rmserr,urms,allrms,maxerr,maxall

!-----------------------------------------------------------------------
! PROJECT-RUN VARIABLES
!
! nproj: number of projectors defined
! nprojdof(p): For the p-th projector, number of degrees of freedom
!    that are projected away.
! projlab(p): Label of the p-th projector. If this is "ABC", the
!    corresponding vpot file will be called "vpot_ABC".
! projdof(p,q): the internal DOF number for the q-th projected
!    DOF in the p-the projector.
! projifunc(p,q): the internal function index associated with the
!    function label for the projector function for the q-th DOF
!    in the p-th projector
! projipar(p,q,n): n-th integer parameter for the function which is
!    used to project onto the q-th DOF in the p-th projector.
! projlpar(p,q,n): n-th logical parameter for the function which is
!    used to project onto the q-th DOF in the p-th projector.
! projerrfile: filename of the projection error vpot file
! projwi(p,q): weight-included-ness of the projector function
!    for the q-th DOF in the p-the projector:
!    0: function values include no DVR weights
!    1: function values include full DVR weights
!    2: function values include square root of DVR weights
!-----------------------------------------------------------------------
      character(len=c5), save                                           :: projerrfile
      character(len=c1), dimension(maxproj), save                       :: projlab
      character(len=c4), dimension(maxproj,maxprojdof,maxprojpar), save :: projcpar

      integer(long), save                                           :: nproj
      integer(long), dimension(maxproj), save                       :: nprojdof
      integer(long), dimension(maxproj,maxprojdof), save            :: projdof,projifunc,projwi
      integer(long), dimension(maxproj,maxprojdof,maxprojpar), save :: projipar


      real(dop), dimension(maxproj), save                       :: projscale
      real(dop), dimension(maxproj,maxprojdof,maxprojpar), save :: projrpar

      logical(kind=4), dimension(maxproj,maxprojdof,maxprojpar), save  :: projlpar

!-----------------------------------------------------------------------
! MCPF / MCCPD
!     lmcpf: flag indicating an mcpotfit run
!     lmcpf: flag indicating an mccpd run
!-----------------------------------------------------------------------
      logical(kind=4), save :: lmcpf, lmccpd

!     enumerate solver methods
      integer(long), parameter :: mc_inv_spd=1
      integer(long), parameter :: mc_inv_conjgrad=2
      integer(long), parameter :: mc_inv_pseudo=3
      integer(long), parameter :: mc_inv_eigen=4

!-----------------------------------------------------------------------
!       Transformation
!       variables for
!       Polyspherical coordinates to Cartesian coordinates
!       K1 added 2013.07.16
!
!       nbvec(maxvec) is the number of vectors read (may not needed)
!       nbdumm(maxvec) is the number of the dummy atom the vector passes
!       at1(maxvec) is the atom or dummy atom sitting on the initial
!               point of the vector.
!       at2(maxvec) is the atom or dummy atom sitting on the terminal
!               point of the vector.
!       vecwei(maxvec) defines the position of the dummy atom on the vector
!       icart defines the vector whose cartesian coordinates is needed
!       tord keeps the new order of coordinates (active one) and is consistant 
!            with Tnum
!       cart_in cartesian coordinates of the molecule on a gpoint
!       cord changes the order of cartesian coordinates of atoms
!-----------------------------------------------------------------------
      integer(long), dimension(maxvec*3) :: tord
      integer(long), dimension(maxvec*3) :: frozen
      real(dop), dimension(maxvec*3)    :: pfroz
      real(dop), dimension(maxvec)      :: vecwei
      integer(long), dimension(maxvec)  ::  nbvec, nbdumm
      integer(long), dimension(maxvec)  ::  cord
      integer(long), dimension(maxvec)  ::  icart
      integer(long)                     ::  nbfr
      character(len=1), dimension(maxvec) :: at1,at2
      logical(kind=4), save             :: lfr = .false.

      end module pglobal
