!***********************************************************************
!
!     plot.inc   include file for the showsys analyse program
!
! ncalcpar must be set to desired value in program before rcalcpar,
!          and acalcpar can be used.
!
!***********************************************************************
      module plot

use decimal, only: long,dop
use global, only: c1, c2, c3, c5

      implicit none

!-----------------------------------------------------------------------
! max no. of data sets. Allows allocation of arrays.
! max no. of grid points for plots
!-----------------------------------------------------------------------
      integer(long), save :: maxdata =  1
      integer(long), save :: maxdata1 = 1
      integer(long), save :: maxdata2 = 1
      integer(long), save :: maxplgrid = 101   ! max no. of points for plotting
      integer(long), save :: maxplgdim = 1     ! max. no of grid points
      integer(long), save :: maxplgdim2 = 1    ! max no. of grid points^2

      logical(kind=4), save       :: lmaport

!-----------------------------------------------------------------------
!     axis(1)   : number of the mode selected for the x axis of the plot
!     axis(2)   : number of the mode selected for the y axis of the plot
!     axis(3)   : number of the mode selected for the min/max search
!     lpltime   : if true, plot is  time v property
!     lplener   : if true, plot is  energy v property
!     lplcoo    : if true, plot is  coordinate v property
!     lplcoo2   : if true, plot is  2D coordinate v property
!     lplcootim : if true, plot is  coordinate v time v property
!     lpltimcoo : if true, plot is  time v coordinate
!     lplcoo2tim: if true, plot is  2D coordinate v time v property
!     lplenetim : if true, plot is  energy v time v property
!     lstate    : if .true. data is for more than one state
!     lplsumst  : if .true. possibility to sum data over states added
!-----------------------------------------------------------------------

      integer(long), dimension(3), save :: axis
      logical(kind=4), save             :: lreplot,lrecalc,lrebound,lredo,lstate,&
                                           lpltime,lplcoo,lplcoo2,lplcootim,&
                                           lplener,lplcoo2tim,lplenetim,lpltimcoo
      logical(kind=4), save             :: lplsumst = .false.

!-----------------------------------------------------------------------
! logicals / variables controlling plot form
!
! ltslice   : if .true. data is for set of time-slices
! lshowpot  : if .true. then showpot is running
! lshowsys  : if .true. then showsys is running
! lshowd1d  : if .true. then showd1d is running
! lshowpsi  : if .true. then showsys plots the wavefunction
! lshowrst  : if .true. then showsys uses the restart rather than psi file
!
! plotform  : 0 = single plot
!             1 = step-through set
!             2 = movie
!-----------------------------------------------------------------------
      integer(long), parameter :: maxnctr = 60

      integer(long), save           :: plotform,nctr,mctr
      real(dop), save               :: ticslevel
      real(dop), dimension(4), save :: rview

      logical(kind=4), save :: lcntr,lsrf,autoz,lhid3d,lkey,ltitle,lgrid,&
                               lpoints,lstick,ltslice,lsmooth,autozo,lsrfo,&
                               lshowpot,lshowsys,lshowd1d,lshowpsi,lshowrst,&
                               lpointsonly

!-----------------------------------------------------------------------
! data for multiplots
!
! multiname : name of .xyz file to be plotted
! loverlay  : controls whether plot or overlay is being plotted
! lmultipl  : if .true. multiplot is made adding 
!-----------------------------------------------------------------------
      character(len=c5), save :: multiname
      integer(long), save     :: nctro
      logical(kind=4), save   :: lmultipl,loverlay,lreploto

!-----------------------------------------------------------------------
! variables for plotting of states / data sets
!
! The data is contained in srf(g,g1,s,t). s is an index for the data
! sets which can be plotted. These are e.g. surfaces and/or
! wavefunctions for different states.
!
! nplstate   : no. of data sets to be plotted
! plstate(n) : nth state to be plotted
! lsumst     : if .true., include sum over states
!-----------------------------------------------------------------------
      integer(long), save                            :: nplstates
      integer(long), dimension(:), allocatable, save :: plstate
      logical, save                                  :: lsumst = .false.

!-----------------------------------------------------------------------
! strings holding default commands
!
! nplottask : no. of possible plottasks 
! cplottask : label describing the possible plots that can be made
! prcmd  : print command (e.g. pprcmd or filename)
! pprcmd : printer command (e.g. lpr)
!-----------------------------------------------------------------------
      integer(long), save                                   :: nplottask
      integer(long), dimension(:), allocatable, save        :: cptlen
      character(len=c3), dimension(:), allocatable, save    :: cplottask
      character(len=c2), dimension(2), save                 :: cctr
      character(len=c5), save                               :: prcmd,gnucmd
      character(len=c2), save                               :: pprcmd

!-----------------------------------------------------------------------
! strings used for labels
!
! alabel(1:alen)  : string containing axis labels
! allmlabels(1:allmlen) : string containing all system mode labels
!-----------------------------------------------------------------------
      integer(long), save                            :: alen,allmlen,alenold
      integer(long), dimension(:), allocatable, save :: modelen
      character(len=1000), save                      :: alabels,allmlabels
      character(len=c5), save                        :: plttitle
      character(len=32), dimension(3), save          :: axislab

!-----------------------------------------------------------------------
! variables defining coordinates and bounds of plot
!
!     rcut(i) : location of the grid point at which mode i is cut.
!     icut(i) : number of the grid point at which mode i is cut. if mode i
!               is not cut, i.e. if it is associated with the x(y)-axis,
!               icut(i)=-1(-2).
!
! cutfile : name of file containing a set of coordinates defining a cut
!-----------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save     :: icut
      integer(long), dimension(:,:), allocatable, save   :: ibound
      real(dop), save                                    :: zmin,zmax,zaxunit,&
                                                            zmino,zmaxo
      real(dop), dimension(2), save                      :: zbound,zboundo
      real(dop), dimension(:), allocatable, save         :: rcut,axunit
      real(dop), dimension(:,:), allocatable, save       :: bound
      character(len=c1), save                            :: zaxunitlab
      character(len=c1), dimension(:), allocatable, save :: axunitlab
      logical(kind=4), save                              :: lcutfile=.false.
      logical(kind=4), save                              :: lqcutfile=.false.
      character(len=c5), save                            :: cutfile

!-----------------------------------------------------------------------
!
! parameters used in analysis controlled over menu
!
! plnham : operator used in showsys for plotting / diabatisation
!-----------------------------------------------------------------------
      integer(long), save                                :: ncalcpar=1
      real(dop), dimension(:), allocatable, save         :: rcalcpar
      character(len=c2), dimension(:), allocatable, save :: acalcpar
      integer(long), save                                :: plnham

      contains

!#######################################################################
!
! PLOT_ALLOC
!
! memory required for arrays in plotmod allocated here.
!
! First check if allocated, then allocate. All arrays must be
! deallocated in
! plot_dealloc
!
!#######################################################################

      subroutine alloc_plot

use maxv
use channels
use griddatmod, only: nddstate

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_plot

      cerr = 'cpltlen'
      allocate(cptlen(nplottask),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'cplottask'
      allocate(cplottask(nplottask),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'plstate'
      allocate(plstate(max(nddstate,2)),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'icut'
      allocate(icut(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ibound'
      allocate(ibound(2,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'rcut'
      allocate(rcut(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'axunit'
      allocate(axunit(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'bound'
      allocate(bound(2,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'axunitlab'
      allocate(axunitlab(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'modelen'
      allocate(modelen(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'rcalcpar'
      allocate(rcalcpar(ncalcpar),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'acalcpar'
      allocate(acalcpar(ncalcpar),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_PLOT :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_plot

!#######################################################################

      subroutine dealloc_plot

      if (allocated(cptlen)) deallocate(cptlen)
      if (allocated(cplottask)) deallocate(cplottask)
      if (allocated(plstate)) deallocate(plstate)
      if (allocated(icut)) deallocate(icut)
      if (allocated(ibound)) deallocate(ibound)
      if (allocated(rcut)) deallocate(rcut)
      if (allocated(axunit)) deallocate(axunit)
      if (allocated(bound)) deallocate(bound)
      if (allocated(axunitlab)) deallocate(axunitlab)
      if (allocated(modelen)) deallocate(modelen)
      if (allocated(rcalcpar)) deallocate(rcalcpar)
      if (allocated(acalcpar)) deallocate(acalcpar)

      return
      end subroutine dealloc_plot


      end module plot
