!***********************************************************************
!                                                                     
!              MCTDH include file   runoper.inc 
!                                                                     
!***********************************************************************

      module runopermod

use decimal, only: long,dop
use global, only: c1,c4,c2,c5

implicit none

!-----------------------------------------------------------------------
! Name including path of the .op file
!-----------------------------------------------------------------------
      character(len=c5), save :: filop

!-----------------------------------------------------------------------
! PARAMETERS defining files containing operators
!-----------------------------------------------------------------------
      integer(long)                                      :: nrdfunc,ilocalpr
      integer(long), dimension(:), allocatable, save     :: ilocal
      character(len=c2), dimension(:), allocatable, save :: rdlab
      integer(long), dimension(:), allocatable, save     :: rdtype

!-----------------------------------------------------------------------
! prelabel  : Pre-defined operator label, e.g "q" or "KE".
!             Allocated in predefine (einopermod)
!-----------------------------------------------------------------------
      character(len=c2), dimension(:), allocatable, save :: prelabel

!-----------------------------------------------------------------------
! SOME MORE PARAMETERS NEEDED FOR HAMILTONIAN
!
! diag(m,k)   : array of unit spf hamiltonian matrices
! diagf(f,k)  : array of unit operators for the degrees of freedom.
! kinop(f,k)  : logical set to true if a correlated Hamiltonian term is
!               a kinetic (i.e. non-diagonal) operator
! hmodelab(m) : string with name of mth mode in Hamiltonian file
! iterm(m,k)  : integer with no. of kth Hamiltonian term for mode m.
! deflabel  : Label for the external operator. E.g. "V" for a natpot.
! deflabop  : Type of external operator, e.g. "srffile".
!
!-----------------------------------------------------------------------
      integer(long)                                      :: htmdimo
      character(len=c1), dimension(:), allocatable, save :: hmodelab

      integer(long), save                                ::  potn,npre

      integer(long), save                            :: mrrdop,rdopdim
      integer(long), dimension(:), allocatable, save :: zrdop

!-----------------------------------------------------------------------
! flags governing operator generation
!
!  lwrnpot : Full information on natpots will be printed to op.log
!-----------------------------------------------------------------------
      logical(kind=4), save :: lwrnpot = .false.


contains

!#######################################################################
!
! RUNOPERMOD_ALLOC
!
! memory required for arrays in griddat allocated here.
!
! First check if allocated, then allocate. All arrays must be deallocated in
! runopermod_dealloc
!
!#######################################################################

      subroutine alloc_runopermod

use maxv
use channels

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_runopermod

      cerr = 'rdtype'
      allocate(rdtype(maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ilocal'
      allocate(ilocal(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'rdlab'
      allocate(rdlab(maxhop),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'hmodelab'
      allocate(hmodelab(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'zrdop'
      allocate(zrdop(maxhop),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_RUNOPERMOD :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      write(6,'(a)') 'Error allocating memory in ALLOC_RUNOPERMOD :',cerr
      write(6,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_runopermod

!#######################################################################

      subroutine dealloc_runopermod

      if (allocated(rdtype)) deallocate(rdtype)
      if (allocated(ilocal)) deallocate(ilocal)
      if (allocated(rdlab)) deallocate(rdlab)
      if (allocated(hmodelab)) deallocate(hmodelab)
      if (allocated(zrdop)) deallocate(zrdop)

      return
      end subroutine dealloc_runopermod

      end module runopermod
