! ------------------------------------------------------------------------
! Include file for Chnpot program
! ------------------------------------------------------------------------
      module wfprojmod

use decimal, only: long,dop
use global, only:c1,c5

      implicit none

      public :: alloc_wfpmat,alloc_wfproj,dealloc_wfpmat,dealloc_wfproj

      integer(long), parameter :: maxprj=8

      logical(kind=4), save :: full,lproj,l2short,ladd,lc3d,lcorr,lcos,lgtt,&
           lid,loper,lread,lsmooth,ltest,ltonly,lwtt
      character(len=c5), save :: dvrfile,capdof,edstrf,unit,unite,unitf
      integer(long), save :: doverdim,prjdim,fulldim,kmax,maxendim,mcdover,&
           mcfullpsi,mcprj,nprj,nst1,nst2,sdmc,khi,klo,nppar,captype,&
           dgldimnew,dgldimold,epoints,fcap,icos,laewmat,mcap,nout2,start,start1,&
           dvrlaenge,nham
      real(dop), save :: emax,emin,ostart,ufac,tovl

      character(len=c5), allocatable :: prjname(:),prjlab(:)
      integer(long), allocatable :: prjmode(:),prjpar(:,:),zetf_f(:,:),zprj(:),&
           dovmat(:,:)

      contains

!#######################################################################
!
! WFPROJMOD_ALLOC
!
! memory required for arrays in wfprojmod allocated here.
!
! First check if allocated, then allocate. All arrays must be deallocated in
! wfprojmod_dealloc
!
!#######################################################################

      subroutine alloc_wfproj

use maxv
use channels
use dvrdatmod, only:mbaspar

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_wfproj

      cerr = 'prjname'
      allocate(prjname(maxprj),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'prjlab'
      allocate(prjlab(maxprj),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'prjmode'
      allocate(prjmode(maxprj),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'prjpar'
      allocate(prjpar(maxprj,maxprj),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'zprj'
      allocate(zprj(maxprj),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_WFPROJ :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_wfproj

!#######################################################################

      subroutine dealloc_wfproj

      if (allocated(prjname)) deallocate(prjname)
      if (allocated(prjlab)) deallocate(prjlab)
      if (allocated(prjmode)) deallocate(prjmode)
      if (allocated(prjpar)) deallocate(prjpar)
      if (allocated(zprj)) deallocate(zprj)

      return
      end subroutine dealloc_wfproj

!#####################################################################
      subroutine alloc_wfpmat

use maxv
use channels

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_wfpmat

      cerr = 'zetf_f'
      allocate(zetf_f(maxdim,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dovmat'
      allocate(dovmat(maxdim,maxsta),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_WFPMAT :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_wfpmat

!#######################################################################

      subroutine dealloc_wfpmat

      if (allocated(zetf_f)) deallocate(zetf_f)
      if (allocated(dovmat)) deallocate(dovmat)

      return
      end subroutine dealloc_wfpmat

      end module wfprojmod
