PRAGMA foreign_keys=ON;
/*
** Master schema for quantics database.
*/

/* Preamble, these must be filled */
/* Version of quantics used to create this, version of DB schema this uses.  One row */
CREATE TABLE versions (quanticsversion INTEGER NOT NULL, dbversion INTEGER NOT NULL);
/* Table with refdb data */
/* Natm, Nroot, Nddstate, Nbasis, Dercpdim, E0, Dimref, Nfam)
      1      2         3       4         5   6       7     8 */
CREATE TABLE refdb (Natm INTEGER NOT NULL, Nroot INTEGER NOT NULL, Nddstate INTEGER NOT NULL, Nbasis INTEGER, E0 DOUBLE, Dimref INTEGER, Nfam INTEGER, Dercpdim INTEGER, DBpntgrp INTEGER, Nsymop INTEGER, Iddsoc INTEGER, Isocbas INTEGER);
/* The refdb.dat reference geometry, with atom masses and names.*/
CREATE TABLE refdbrefgeom(atno INTEGER PRIMARY KEY, mass DOUBLE NOT NULL, name TEXT NOT NULL, x DOUBLE NOT NULL, y DOUBLE NOT NULL, z DOUBLE NOT NULL);
/* The refdb.dat symmetry maps.*/
CREATE TABLE refdbsymmaps(Symopno INTEGER PRIMARY KEY, Natm INTEGER, DBsymmap INTEGER);

/* Data.  Some of these tables may be empty.*/

/*
** Each table corresponds to a file in the old format.  dbentry is the index table, also holds the time
** the entry was inserted and the status value (aka interp).  dbentry.id is the record number.
** The record numbers MUST NOT change during a run.
** Rows in the other tables correspond to the old format files.
** E.g. geo.db now looks like this:
** FIXME
**
*/
CREATE TABLE dbentry (id INTEGER PRIMARY KEY, status INTEGER NOT NULL);
/* Geometries */
CREATE TABLE geo (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,ndof INTEGER NOT NULL,value DOUBLE NOT NULL);
/* Energies */
CREATE TABLE pes (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,state1 INTEGER NOT NULL,state2 INTEGER NOT NULL,compdim INTEGER NOT NULL,energy DOUBLE NOT NULL);
/* Gradients */
CREATE TABLE gra (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,state1 INTEGER NOT NULL,state2 INTEGER NOT NULL,ndof INTEGER NOT NULL,value DOUBLE NOT NULL);
/* Hessians */
CREATE TABLE hes (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,state1 INTEGER NOT NULL,state2 INTEGER NOT NULL,ndof1 INTEGER NOT NULL,ndof2 INTEGER NOT NULL,value DOUBLE NOT NULL);
/* Alternate energies */
CREATE TABLE apes (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,state1 INTEGER NOT NULL,state2 INTEGER NOT NULL,energy DOUBLE NOT NULL);
/* Alternate gradients */
CREATE TABLE agra (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,state1 INTEGER NOT NULL,ndof INTEGER NOT NULL,value DOUBLE NOT NULL);
/* Alternate Hessians */
CREATE TABLE ahes (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,state1 INTEGER NOT NULL,ndof1 INTEGER NOT NULL,ndof2 INTEGER NOT NULL,value DOUBLE NOT NULL);
/* Dipoles */
CREATE TABLE dip (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,state1 INTEGER NOT NULL,state2 INTEGER NOT NULL,dipx DOUBLE NOT NULL,dipy DOUBLE NOT NULL,dipz DOUBLE NOT NULL);
/* Alternate Dipoles */
CREATE TABLE adip (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,state1 INTEGER NOT NULL,state2 INTEGER NOT NULL,dipx DOUBLE NOT NULL,dipy DOUBLE NOT NULL,dipz DOUBLE NOT NULL);
/* Molecular Orbitals */
CREATE TABLE moco (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,mo1 INTEGER NOT NULL,mo2 INTEGER NOT NULL,value DOUBLE NOT NULL);
/* Nact */
CREATE TABLE nact (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,dercp INTEGER NOT NULL,dof INTEGER NOT NULL,value DOUBLE NOT NULL);
/* Transformation data */
CREATE TABLE trans(id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,state1 INTEGER NOT NULL,state2 INTEGER NOT NULL,compdim INTEGER NOT NULL,value DOUBLE NOT NULL);
/* Normal modes */
CREATE TABLE nmode (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,nmdof INTEGER NOT NULL,value DOUBLE NOT NULL);
/* Spin-orbit coupling (?) */
CREATE TABLE soc (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,state1 INTEGER NOT NULL, state2 INTEGER NOT NULL, value DOUBLE NOT NULL);
/* Complex valued Spin-orbit coupling (?) */
CREATE TABLE csoc (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,state1 INTEGER NOT NULL, state2 INTEGER NOT NULL, realvalue DOUBLE NOT NULL, complexvalue DOUBLE NOT NULL);
/* Alternate spin-orbit coupling (?) */
CREATE TABLE asoc (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,socindex INTEGER NOT NULL,value DOUBLE NOT NULL);
/* Complex alternate valued Spin-orbit coupling (?) */
CREATE TABLE casoc (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,socindex INTEGER NOT NULL, realvalue DOUBLE NOT NULL, complexvalue DOUBLE NOT NULL);
CREATE TABLE civecentry (civecid INTEGER PRIMARY KEY, dbid INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE, smid INTEGER NOT NULL, nconf INTEGER NOT NULL);
CREATE TABLE civecsmentry (civecsmid INTEGER PRIMARY KEY, civecid INTEGER NOT NULL REFERENCES civecentry(civecid) ON UPDATE CASCADE ON DELETE CASCADE, confid INTEGER NOT NULL, config TEXT NOT NULL);
CREATE TABLE civec(civecsmid INTEGER NOT NULL REFERENCES civecsmentry(civecsmid) ON UPDATE CASCADE ON DELETE CASCADE, stateid INTEGER NOT NULL, value DOUBLE NOT NULL);
/* Coordinates for GAP potentials */
CREATE TABLE symrep (id INTEGER NOT NULL REFERENCES dbentry(id)ON UPDATE CASCADE ON DELETE CASCADE,nsymop INTEGER NOT NULL,value INTEGER NOT NULL);

