!***********************************************************************
!
!     Additional library-like subroutines for dealing with type 2
!     density matrices.
!
!     
!     rm1_mf(tensor,dens,vdim,dim,ndim):
!         Definition: dens(i,j) = sum tensor(k,j,l,k,i,l)
!         Dimensions: tensor(vdim,dim,ndim,vdim,dim,ndim),dens(dim,dim)
!
!     rm1(tensor,mat,vdim1,dim1,ndim1,vdim2,dim2,ndim2):
!         Definition: mat(i,j) = sum tensor(k,i,l,m,j,n)
!         Dimensions: tensor(vdim1,dim1,ndim1,vdim2,dim2,ndim2),
!                     mat(dim1,dim2)
!
!     rm2_mf(tens,dens,vdim,dim,ndim,bdim)
!         Definition: dens(i,j) = sum conjg(tens(bi,k,j,l))*tens(b,k,i,l)
!         Dimensions: tens(bdim,vdim,dim,ndim),dens(dim,dim)
!
!     rm2_mfa(tens1,tens2,dens,vdim,dim,ndim,bdim)
!         Definition: dens(i,j) = dens(i,j) + 
!                        sum conjg(tens1(b,k,j,l))*tens2(b,k,i,l)
!         Dimensions: tens1(bdim,vdim,dim,ndim),
!                     tens2(bdim,vdim,dim,ndim),dens(dim,dim)
!
!     filldens(dens,dim)
!         Definition: dens(i,j) = conjg(dens(j,i)) , j.ge.i
!         Dimensions: dens(dim,dim)
!
!     ctens(ten,cten,vdim,dim,ndim):
!         Definition: cten(i,j,k,l) = sum ten(i,m,j,k,m,l)
!         Dimensions: ten(vdim,dim,ndim,vdim,dim,ndim), 
!                     cten(vdim,ndim,vdim,ndim)
!
!     ctenst(ten,cten,vdim,dim,ndim):
!         Definition: cten(i,j,k,l) = sum ten(k,m,l,i,m,j)
!         Dimensions: ten(vdim,dim,ndim,vdim,dim,ndim), 
!                     cten(vdim,ndim,vdim,ndim)
!
!     ctens1(ten,cten,vdim1,dim1,ndim1,vdim2,dim2,ndim2):
!         Definition: cten(i,j,k,l) = sum ten(i,m,j,k,n,l)
!         Dimensions: ten(vdim1,dim1,ndim1,vdim2,dim2,ndim2), 
!                     cten(vdim1,ndim1,vdim2,ndim2)

!     ctenst1(ten,cten,vdim1,dim1,ndim1,vdim2,dim2,ndim2):
!         Definition: cten(i,j,k,l) = sum ten(k,m,l,i,n,j)
!         Dimensions: ten(vdim1,dim1,ndim1,vdim2,dim2,ndim2), 
!
!
!     ctens2(ten1,ten2,cten,vdim,dim,ndim,bdim)
!         Definition: cten(i,j,k,l) = sum ten1(i,m,j,b)*ten2(b,k,m,l)
!         Dimensions: ten1(vdim,dim,ndim,bdim),
!                     ten2(bdim,vdim,dim,ndim), 
!                     cten(vdim,ndim,vdim,ndim)
!
!     ctens2_a(ten1,ten2,cten,vdim,dim,ndim,bdim)
!         Definition: cten(i,j,k,l) = cten(i,j,k,l) +
!                                     sum ten1(i,m,j,b)*ten2(b,k,m,l)
!         Dimensions: ten1(vdim,dim,ndim,bdim),
!                     ten2(bdim,vdim,dim,ndim), 
!                     cten(vdim,ndim,vdim,ndim)
!
!     cmtens(mat,ten,cten,vdim1,dim1,ndim1,vdim2,dim2,ndim2):
!         Definition: cten(i,j,k,l) = sum ten(i,n,j,k,m,l)*mat(m,n)
!         Dimensions: mat(dim2,dim1), 
!                     ten(vdim1,dim1,ndim1,vdim2,dim2,ndim2), 
!                     cten(vdim1,ndim1,vdim2,ndim2)     
!
!     cmtenst(mat,ten,cten,vdim1,dim1,ndim1,vdim2,dim2,ndim2):
!         Definition: cten(i,j,k,l) = sum ten(k,m,l,i,n,j)*mat(m,n)
!         Dimensions: mat(dim2,dim1), 
!                     ten(vdim1,dim1,ndim1,vdim2,dim2,ndim2), 
!                     cten(vdim1,ndim1,vdim2,ndim2)     
!
!     cmtens2(mat,ten1,ten2,cten,vdim1,dim1,ndim1,vdim2,dim2,ndim2,bdim)
!         Definition: cten(i,j,k,l) = sum ten1(i,n,j,b)*ten2(b,k,m,l)
!                                        *mat(m,n)
!         Dimensions: mat(dim2,dim1), 
!                     ten1(vdim1,dim1,ndim1,bdim),
!                     ten2(bdim,vdim2,dim2,ndim2), 
!                     cten(vdim1,ndim1,vdim2,ndim2)     
!
!     cmtens2_a(mat,ten1,ten2,cten,vdim1,dim1,ndim1,vdim2,dim2,ndim2,bdim)
!         Definition: cten(i,j,k,l) = cten(i,j,k,l) +
!                                 sum ten1(i,n,j,b)*ten2(b,k,m,l)*mat(m,n)
!         Dimensions: mat(dim2,dim1), 
!                     ten1(vdim1,dim1,ndim1,bdim),
!                     ten2(bdim,vdim2,dim2,ndim2), 
!                     cten(vdim1,ndim1,vdim2,ndim2)     
!
!     mkherm(a,dim)
!         Definition: a(j,i) = (0.5d0,0.0d0)*(a(j,i) + conjg(a(i,j))
!         Dimensions: a(dim,dim)
!
!     mkaherm(a,dim)
!         Definition: a(j,i) = (0.5d0,0.0d0)*(a(j,i) - conjg(a(i,j))
!         Dimensions: a(dim,dim)
!
!     mkherm2(a,b,dim1,dim2)
!         Definition: a(j,i) = (0.5d0,0.0d0)*(a(j,i) + conjg(b(i,j))
!                     b(i,j) = conjg(a(j,i))
!         Dimensions: a(dim1,dim2), b(dim2,dim1)
!
!     chkherm(a,error,dim)
!         Definition: max(error, abs(a(i,j)-conjg(a(j,i)))) = error
!         Dimensions: a(dim,dim)
!
!     chkaherm(a,error,dim)
!         Definition: max(error, abs(a(i,j)+conjg(a(j,i)))) = error
!         Dimensions: a(dim,dim)
!
!     chkherm2(a,b,error,dim1,dim2)
!         Definition: max(error, abs(a(i,j)-conjg(b(j,i)))) = error
!         Dimensions: a(dim1,dim2), b(dim2,dim1)
!
!     chkaherm2(a,b,error,dim1,dim2)
!         Definition: max(error, abs(a(i,j)+conjg(b(j,i)))) = error
!         Dimensions: a(dim1,dim2), b(dim2,dim1)
!
!     mktens_t: makes a tensor from the product of the transpose of
!               a  complex matrix with a complex vector
!               t(i,j,k)=v(i)*m(k,j)
!
!     sorttens: re-sorts a tensor
!               tens2(i,j,k,l) = tens1(i,k,j,l)
!
!     cuttens2(tensor,mat,i,j,vdim,dim,ndim):
!         Definition: mat(k,l,m,n) = tensor(k,i,l,m,j,n)
!         Dimensions: tensor(vdim,dim,ndim,vdim,dim,ndim),
!                     mat(vdim,ndim,vdim,ndim)
!
!     filltens2(mat,tensor,i,j,vdim,dim,ndim):
!         Definition: tensor(k,i,l,m,j,n) = mat(k,l,m,n) 
!         Dimensions: tensor(vdim,dim,ndim,vdim,dim,ndim),
!                     mat(vdim,ndim,vdim,ndim)
!
!     filltens2c(mat,tensor,i,j,vdim,dim,ndim):
!         Definition: tensor(k,i,l,m,j,n) = dconjg(mat(k,l,m,n))
!         Dimensions: tensor(vdim,dim,ndim,vdim,dim,ndim),
!                     mat(vdim,ndim,vdim,ndim)
!
!***********************************************************************
      module d2tens

      use decimal, only: dop, long
      use openmpmod

      implicit none
      private
      public :: rm1_mf, rm1, rm2_mf, rm2_mfa, filldens, &
                ctens1, ctens, ctenst, ctenst1, ctens2, ctens2_a, cmtens, &
                cmtenst, cmtens2, cmtens2_a, mkherm, mkaherm, &
                mkherm2, chkherm, chkaherm, chkherm2, chkaherm2, mktens_t, &
                sorttens, cuttens2, filltens2, filltens2c
      contains

!**********************************************************************      
      
      subroutine rm1_mf(tensor,dens,vdim,dim,ndim)
 
      implicit none

      integer(long)                                                    :: i, j, &
                                                                          k, l
      integer(long), intent(in)                                        :: vdim, &
                                                                          dim, &
                                                                          ndim
      complex(dop), dimension(vdim,dim,ndim,vdim,dim,ndim), intent(in) :: tensor
      complex(dop), dimension(dim,dim), intent(out)                    :: dens


      do j=1,dim
         do i=j,dim
            dens(i,j) = tensor(1,j,1,1,i,1)
         enddo
      enddo

      do l=2,vdim
         do j=1,dim
            do i=j,dim
               dens(i,j) = dens(i,j) + tensor(l,j,1,l,i,1)
            enddo
         enddo
      enddo

      do k=2,ndim
         do l=1,vdim            
            do j=1,dim
               do i=j,dim
                  dens(i,j) = dens(i,j) + tensor(l,j,k,l,i,k)
               enddo
            enddo
         enddo
      enddo

      do j=1,dim
         dens(j,j) = dble( dens(j,j) )
         do i=(j+1),dim
            dens(j,i) = conjg( dens(i,j) )
         enddo
      enddo

      return
      end subroutine rm1_mf


!***************************************************************************
!
!***************************************************************************
      subroutine rm1(tensor,mat,vdim1,dim1,ndim1,vdim2,dim2,ndim2)

      implicit none

      integer(long)                                          :: i, j, k, l, m, n
      integer(long), intent(in)                              :: vdim1, dim1, &
                                                                ndim1, vdim2, &
                                                                dim2, ndim2
      complex(dop), dimension(vdim1,dim1,ndim1,vdim2,dim2,ndim2), &
                    intent(in)                               :: tensor 
      complex(dop), dimension(dim2,dim1), intent(out)        :: mat


      do j=1,dim1
         do i=1,dim2
            mat(i,j) = (0.0d0,0.0d0)
         enddo
      enddo

      do n=1,ndim1
         do i=1,dim2
            do m=1,vdim1
               do l=1,ndim2
                  do j=1,dim1
                     do k=1,vdim2
                        mat(i,j) = mat(i,j) + tensor(k,j,l,m,i,n)
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo
      
      return
      end subroutine rm1
!*************************************************************************
!
!*************************************************************************

      subroutine rm2_mf(tens,dens,vdim,dim,ndim,bdim)
      
      implicit none
      
      integer(long)                                           :: b, i, j, k, l
      integer(long), intent(in)                               :: vdim, dim, &
                                                                 ndim, bdim
      complex(dop), dimension(bdim,vdim,dim,ndim), intent(in) :: tens
      complex(dop), dimension(dim,dim), intent(out)           :: dens
      
      
      do j=1,dim
         do i=j,dim     
            dens(i,j) = (0.0d0,0.0d0)
         enddo
      enddo
      
      do k=1,ndim
         do l=1,vdim            
            do j=1,dim
               do i=j,dim
                  do b=1,bdim
                     dens(i,j) = dens(i,j) + dconjg(tens(b,l,j,k))*tens(b,l,i,k)
                  enddo
               enddo
            enddo
         enddo
      enddo      
           
      return
      end subroutine rm2_mf

!***************************************************************************
!
!***************************************************************************

      subroutine rm2_mfa(tens1,tens2,dens,vdim,dim,ndim,bdim)

      implicit none

      integer(long)                                           :: b, i, j, k, l
      integer(long), intent(in)                               :: vdim, dim, & 
                                                                 ndim, bdim
      complex(dop), dimension(bdim,vdim,dim,ndim), intent(in) :: tens1
      complex(dop), dimension(bdim,vdim,dim,ndim), intent(in) :: tens2 
      complex(dop), dimension(dim,dim), intent(out)           :: dens



      do k=1,ndim
         do l=1,vdim            
            do j=1,dim
               do i=j,dim
                  do b=1,bdim
                     dens(i,j) = dens(i,j) + dconjg(tens1(b,l,j,k))*tens2(b,l,i,k)
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine rm2_mfa

!*************************************************************************
!
!*************************************************************************

      subroutine filldens(dens,dim)

      implicit none

      integer(long)                                   :: i, j
      integer(long), intent(in)                       :: dim
      complex(dop), dimension(dim,dim), intent(inout) :: dens


      do j=1,dim
         dens(j,j) = dble( dens(j,j) )
         do i=(j+1),dim
            dens(j,i) = dconjg( dens(i,j) )
         enddo
      enddo

      return
      end subroutine filldens


! ################################################################
!
!#################################################################
      subroutine ctens1(ten,cten,vdim1,dim1,ndim1,vdim2,dim2,ndim2)

      implicit none

      integer(long)                                                 :: i, j, &
                                                                       k, l, &
                                                                       m, n
      integer(long), intent(in)                                     :: vdim1, &
                                                                       dim1,  &
                                                                       ndim1, &
                                                                       vdim2, &
                                                                       dim2,  &
                                                                       ndim2
      complex(dop), dimension(vdim1,dim1,ndim1,vdim2,dim2,ndim2), &
                    intent(in)                                      :: ten 
      complex(dop), dimension(vdim1,ndim1,vdim2,ndim2), intent(out) :: cten


!!$omp parallel do if(lompthread) num_threads(ompthread) private(i,j,k,l,m,n)
      do l=1,ndim2
         do k=1,vdim2
            do j=1,ndim1
               do i=1,vdim1
                  cten(i,j,k,l) = ten(i,1,j,k,1,l)
                  do m=2,dim1
                     do n=2,dim2
                        cten(i,j,k,l) = cten(i,j,k,l) + ten(i,m,j,k,n,l)
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo
!!$omp end parallel do

      return
      end subroutine ctens1


! ################################################################
!
!#################################################################
      subroutine ctenst1(ten,cten,vdim1,dim1,ndim1,vdim2,dim2,ndim2)

      implicit none

      integer(long)                                                 :: i, j, &
                                                                       k, l, &
                                                                       m, n
      integer(long), intent(in)                                     :: vdim1, &
                                                                       dim1,  &
                                                                       ndim1, &
                                                                       vdim2, &
                                                                       dim2,  &
                                                                       ndim2
      complex(dop), dimension(vdim1,dim1,ndim1,vdim2,dim2,ndim2), &
                    intent(in)                                      :: ten 
      complex(dop), dimension(vdim1,ndim1,vdim2,ndim2), intent(out) :: cten


!!$omp parallel do if(lompthread) num_threads(ompthread) private(i,j,k,l,m,n)
      do l=1,ndim2
         do k=1,vdim2
            do j=1,ndim1
               do i=1,vdim1
                  cten(i,j,k,l) = ten(i,1,j,k,1,l)
                  do m=2,dim1
                     do n=2,dim2
                        cten(i,j,k,l) = cten(i,j,k,l) + ten(k,m,l,i,n,i)
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo
!!$omp end parallel do

      return
      end subroutine ctenst1


! ###################################################################
!
! ###################################################################
      subroutine ctens(ten,cten,vdim,dim,ndim)

      implicit none

      integer(long)                                             :: i, j, k, &
                                                                   l, m
      integer(long), intent(in)                                 :: vdim, dim, &
                                                                   ndim
      complex(dop), dimension(vdim,dim,ndim,vdim,dim,ndim), &
                    intent(in)                                  :: ten
      complex(dop), dimension(vdim,ndim,vdim,ndim), intent(out) :: cten


!!$omp parallel do if(lompthread) num_threads(ompthread) private(i,j,k,l,m)
      do l=1,ndim
         do k=1,vdim
            do j=1,ndim
               do i=1,vdim
                  cten(i,j,k,l) = ten(i,1,j,k,1,l)
                  do m=2,dim
                     cten(i,j,k,l) = cten(i,j,k,l) + ten(i,m,j,k,m,l)
                  enddo
               enddo
            enddo
         enddo
      enddo
!!$omp end parallel do

      return
      end subroutine ctens

! ###################################################################
!
! ###################################################################

      subroutine ctenst(ten,cten,vdim,dim,ndim)

      implicit none

      integer(long)                                             :: i, j, k, &
                                                                   l, m
      integer(long), intent(in)                                 :: vdim, dim, &
                                                                   ndim
      complex(dop), dimension(vdim,dim,ndim,vdim,dim,ndim), &
                    intent(in)                                  :: ten
      complex(dop), dimension(vdim,ndim,vdim,ndim), intent(out) :: cten


!!$omp parallel do if(lompthread) num_threads(ompthread) private(i,j,k,l,m)
      do l=1,ndim
         do k=1,vdim
            do j=1,ndim
               do i=1,vdim
                  cten(i,j,k,l) = ten(k,1,l,i,1,j)
                  do m=2,dim
                     cten(i,j,k,l) = cten(i,j,k,l) + ten(k,m,l,i,m,j)
                  enddo
               enddo
            enddo
         enddo
      enddo
!!$omp end parallel do

      return
      end subroutine ctenst


!####################################################################
!
!####################################################################
      subroutine ctens2(ten1,ten2,cten,vdim,dim,ndim,bdim)

      implicit none

      integer(long)                                             :: b, i, j, &
                                                                   k, l, m
      integer(long), intent(in)                                 :: vdim, dim, &
                                                                   ndim, bdim
      complex(dop), dimension(vdim,dim,ndim,bdim), intent(in)   :: ten1
      complex(dop), dimension(bdim,vdim,dim,ndim), intent(in)   :: ten2
      complex(dop), dimension(vdim,ndim,vdim,ndim), intent(out) :: cten


!!$omp parallel do if(lompthread) num_threads(ompthread) private(i,j,k,l,m,b)
      do l=1,ndim
         do k=1,vdim
            do j=1,ndim
               do i=1,vdim                  
                  cten(i,j,k,l) = ten1(i,1,j,1)*ten2(1,k,1,l)
                  do b=2,bdim
                     cten(i,j,k,l) = cten(i,j,k,l) + &
                          ten1(i,1,j,b)*ten2(b,k,1,l)
                  enddo
                  do m=2,dim
                     do b=1,bdim
                        cten(i,j,k,l) = cten(i,j,k,l) + &
                             ten1(i,m,j,b)*ten2(b,k,m,l)
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo
!!$omp end parallel do

      return
      end subroutine ctens2


!###########################################################################
!
!###########################################################################
      subroutine ctens2_a(ten1,ten2,cten,vdim,dim,ndim,bdim)

      implicit none

      integer(long)                                             :: b, i, j, &
                                                                   k, l, m
      integer(long), intent(in)                                 :: vdim, dim, &
                                                                   ndim, bdim
      complex(dop), dimension(vdim,dim,ndim,bdim), intent(in)   :: ten1 
      complex(dop), dimension(bdim,vdim,dim,ndim), intent(in)   :: ten2
      complex(dop), dimension(vdim,ndim,vdim,ndim), intent(out) :: cten


      do l=1,ndim
         do k=1,vdim
            do j=1,ndim
               do i=1,vdim                  
                  do m=1,dim
                     do b=1,bdim
                        cten(i,j,k,l) = cten(i,j,k,l) + ten1(i,m,j,b)*ten2(b,k,m,l)
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine ctens2_a

!###########################################################################
!
!###########################################################################

      subroutine cmtens(mat,ten,cten,vdim1,dim1,ndim1,vdim2,dim2,ndim2)

      implicit none

      integer(long)                                                 :: i, j, &
                                                                       k, l, &
                                                                       m, n
      integer(long), intent(in)                                     :: vdim1, &
                                                                       dim1,  &
                                                                       ndim1, &
                                                                       vdim2, &
                                                                       dim2,  &
                                                                       ndim2
      complex(dop), dimension(dim2,dim1), intent(in)                :: mat
      complex(dop), dimension(vdim1,dim1,ndim1,vdim2,dim2,ndim2), &
                    intent(in)                                      :: ten 
      complex(dop), dimension(vdim1,ndim1,vdim2,ndim2), intent(out) :: cten

!!$omp parallel do if(lompthread) num_threads(ompthread) private(i,j,k,l,m,n)   
      do l=1,ndim2
         do k=1,vdim2
            do j=1,ndim1
               do i=1,vdim1
                  cten(i,j,k,l) = ten(i,1,j,k,1,l)*mat(1,1)
                  do m=2,dim2
                     cten(i,j,k,l) = cten(i,j,k,l) + ten(i,1,j,k,m,l)*mat(m,1)
                  enddo
                  do n=2,dim1
                     do m=1,dim2
                        cten(i,j,k,l) = cten(i,j,k,l) + &
                             ten(i,n,j,k,m,l)*mat(m,n)
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo
!!$omp end parallel do

      return
      end subroutine cmtens

!###########################################################################
!
!###########################################################################

      subroutine cmtenst(mat,ten,cten,vdim1,dim1,ndim1,vdim2,dim2,ndim2)

      implicit none

      integer(long)                                                 :: i, j, &
                                                                       k, l, &
                                                                       m, n
      integer(long), intent(in)                                     :: vdim1, &
                                                                       dim1,  &
                                                                       ndim1, &
                                                                       vdim2, &
                                                                       dim2,  &
                                                                       ndim2
      complex(dop), dimension(dim2,dim1), intent(in)                :: mat
      complex(dop), dimension(vdim1,dim1,ndim1,vdim2,dim2,ndim2), &
                    intent(in)                                      :: ten 
      complex(dop), dimension(vdim2,ndim2,vdim1,ndim1), intent(out) :: cten

!!$omp parallel do if(lompthread) num_threads(ompthread) private(i,j,k,l,m,n)   
      do l=1,ndim1
         do k=1,vdim1
            do j=1,ndim2
               do i=1,vdim2
                  cten(i,j,k,l) = ten(k,1,l,i,1,j)*mat(1,1)
                  do m=2,dim2
                     cten(i,j,k,l) = cten(i,j,k,l) + ten(k,1,l,i,m,j)*mat(m,1)
                  enddo
                  do n=2,dim1
                     do m=1,dim2
                        cten(i,j,k,l) = cten(i,j,k,l) + &
                             ten(k,n,l,i,m,j)*mat(m,n)
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo
!!$omp end parallel do

      return
      end subroutine cmtenst

!###########################################################################
!
!###########################################################################

      subroutine cmtens2(mat,ten1,ten2,cten,vdim1,dim1,ndim1, vdim2,dim2, &
                         ndim2,bdim)

      implicit none

      integer(long)                                                 :: b, i, &
                                                                       j, k, &
                                                                       l, m, &
                                                                       n
      integer(long), intent(in)                                     :: vdim1, &
                                                                       dim1, &
                                                                       ndim1, &
                                                                       vdim2, &
                                                                       dim2, &
                                                                       ndim2, &
                                                                       bdim
      complex(dop), dimension(dim2,dim1), intent(in)                :: mat 
      complex(dop), dimension(vdim1,dim1,ndim1,bdim), intent(in)    :: ten1
      complex(dop), dimension(bdim,vdim2,dim2,ndim2), intent(in)    :: ten2
      complex(dop), dimension(vdim1,ndim1,vdim2,ndim2), intent(out) :: cten

      
!!$omp parallel if(lompthread) num_threads(ompthread) private(i,j,k,l,m,n,b)
!!$omp do
      do l=1,ndim2
         do k=1,vdim2
            do j=1,ndim1
               do i=1,vdim1
                  cten(i,j,k,l) = (0.0d0,0.0d0)
               enddo
            enddo
         enddo
      enddo
!!$omp end do
!!$omp do
      do l=1,ndim2
         do m=1,dim2
            do k=1,vdim2
               do j=1,ndim1
                  do n=1,dim1
                     do i=1,vdim1
                        do b=1,bdim
                           cten(i,j,k,l) = cten(i,j,k,l) + &
                                ten1(i,n,j,b)*ten2(b,k,m,l)*mat(m,n) 
                        enddo
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo
!!$omp end do
!!$omp end parallel

      return
      end subroutine cmtens2

!###########################################################################
!
!###########################################################################
      subroutine cmtens2_a(mat,ten1,ten2,cten,vdim1,dim1,ndim1, &
                           vdim2,dim2,ndim2,bdim)

      implicit none


      integer(long)                                                 :: b, i, &
                                                                       j, k, &
                                                                       l, m, &
                                                                       n
      integer(long), intent(in)                                     :: vdim1, &
                                                                       dim1, &
                                                                       ndim1, &
                                                                       vdim2, &
                                                                       dim2, &
                                                                       ndim2, &
                                                                       bdim
      complex(dop), dimension(dim2,dim1), intent(in)                :: mat 
      complex(dop), dimension(vdim1,dim1,ndim1,bdim), intent(in)    :: ten1
      complex(dop), dimension(bdim,vdim2,dim2,ndim2), intent(in)    :: ten2
      complex(dop), dimension(vdim1,ndim1,vdim2,ndim2), intent(out) :: cten


      do l=1,ndim2
         do m=1,dim2
            do k=1,vdim2
               do j=1,ndim1
                  do n=1,dim1
                     do i=1,vdim1
                        do b=1,bdim
                           cten(i,j,k,l) = cten(i,j,k,l) + ten1(i,n,j,b)* &
                                           ten2(b,k,m,l)*mat(m,n) 
                        enddo
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine cmtens2_a

!###########################################################################
!
!###########################################################################

      subroutine mkherm(a,dim)

      implicit none

      integer(long)                                   :: i, j
      integer(long), intent(in)                       :: dim
      complex(dop), dimension(dim,dim), intent(inout) :: a

      do i=1,dim
         a(i,i) = dble(a(i,i))
         do j=i+1,dim
            a(j,i) = 0.5d0 * (a(j,i) + dconjg(a(i,j)))
            a(i,j) = dconjg(a(j,i))
         enddo
      enddo

      return
      end subroutine mkherm

!###########################################################################
!
!###########################################################################

      subroutine mkaherm(a,dim)

      implicit none

      integer(long)                                   :: i, j
      integer(long), intent(in)                       :: dim
      real(dop)                                       :: x
      complex(dop), dimension(dim,dim), intent(inout) :: a

      do i=1,dim
         x = dimag(a(i,i))
         a(i,i) = dcmplx(0.0_dop,x)
         do j=i+1,dim
            a(j,i) = 0.5d0 * (a(j,i) - dconjg(a(i,j)))
            a(i,j) = -dconjg(a(j,i))
         enddo
      enddo

      return
      end subroutine mkaherm

!###########################################################################
!
!###########################################################################

      subroutine mkherm2(a,b,dim1,dim2)

      implicit none

      integer(long)                                     :: i, j
      integer(long), intent(in)                         :: dim1, dim2
      complex(dop), dimension(dim1,dim2), intent(inout) :: a 
      complex(dop), dimension(dim2,dim1), intent(inout) :: b

      do i=1,dim2
         do j=1,dim1
            a(j,i) = 0.5d0 * (a(j,i) + dconjg(b(i,j)))
            b(i,j) = dconjg(a(j,i))
         enddo
      enddo

      return
      end subroutine mkherm2


!###########################################################################
!
!###########################################################################

      subroutine chkherm(a,error,dim)

      implicit none

      integer(long)                                :: i, j
      integer(long), intent(in)                    :: dim
      complex(dop), dimension(dim,dim), intent(in) :: a
      real(dop), intent(out)                       :: error 
      real(dop)                                    :: err

      error = 0.0d0
      do i=1,dim
         do j=i,dim
            err = abs(a(i,j)-conjg(a(j,i)))
            error = max(err,error)
         enddo
      enddo


      return
      end subroutine chkherm


!###########################################################################
!
!###########################################################################

      subroutine chkaherm(a,error,dim)

      implicit none

      integer(long)                                :: i, j
      integer(long), intent(in)                    :: dim
      complex(dop), dimension(dim,dim), intent(in) :: a
      real(dop), intent(out)                       :: error 
      real(dop)                                    :: err

      error = 0.0d0
      do i=1,dim
         do j=i,dim
            err = abs(a(i,j)+conjg(a(j,i)))
            error = max(err,error)
         enddo
      enddo


      return
      end subroutine chkaherm


!###########################################################################
!
!###########################################################################

      subroutine chkherm2(a,b,error,dim1,dim2)

      implicit none

      integer(long)                                  :: i, j
      integer(long), intent(in)                      :: dim1, dim2
      complex(dop), dimension(dim1,dim2), intent(in) :: a
      complex(dop), dimension(dim2,dim1), intent(in) :: b
      real(dop), intent(out)                         :: error 
      real(dop)                                      :: err

      error = 0.0d0
      do j=1,dim2
         do i=1,dim1
            err = abs(a(i,j)-conjg(b(j,i)))
            error = max(err,error)
         enddo
      enddo

      return
      end subroutine chkherm2


!###########################################################################
!
!###########################################################################

      subroutine chkaherm2(a,b,error,dim1,dim2)

      implicit none

      integer(long)                                  :: i, j
      integer(long), intent(in)                      :: dim1, dim2
      complex(dop), dimension(dim1,dim2), intent(in) :: a
      complex(dop), dimension(dim2,dim1), intent(in) :: b
      real(dop), intent(out)                         :: error 
      real(dop)                                      :: err

      error = 0.0d0
      do j=1,dim2
         do i=1,dim1
            err = abs(a(i,j)+conjg(b(j,i)))
            error = max(err,error)
         enddo
      enddo

      return
      end subroutine chkaherm2
       
!-----------------------------------------------------------------------
! Library subroutine mktens_t
!
! Makes a tensor of third order by elementwise multiplying a vector with
! the transpose of a matrix:
!           T(i,j,k) = V(i)*M(k,j)   (no summation)
!           i=1,...,a;  j=1,...,b;  k=1,...,c
!
! Input-variables:  V     : complex vector
!                   M     : complex matrix
!                   a,b,c : dimensions of the different arrays
!                   add   : if true, result is added to T
! Output-variables: T     : resulting complex tensor of third order
!-----------------------------------------------------------------------

      subroutine mktens_t (T,V,M,a,b,c,add)

      implicit none

      integer(long)                               :: i, j, k
      integer(long), intent(in)                   :: a, b, c
      complex(dop), dimension(a), intent(in)      :: V
      complex(dop), dimension(c,b), intent(in)    :: M
      complex(dop), dimension(a,b,c), intent(out) :: T
      logical(kind=4)                             :: add

!!$omp parallel if(lompthread) private(i,j,k)
      if (.not. add) then
!!$omp do
         do k=1,c
            do j=1,b
               do i=1,a
                  T(i,j,k)=(0.0d0,0.0d0)
               enddo
            enddo
         enddo
!!$omp end do
      endif
!!$omp do
      do k=1,c
         do j=1,b
            do i=1,a
               T(i,j,k)=T(i,j,k)+V(i)*M(k,j)
            enddo
         enddo
      enddo
!!$omp end do
!!$omp end parallel

      return
      end subroutine mktens_t



!-------------------------------------------------------------------- 
! Library sorttens
!
!    tens2(i,j,k,l) = tens1(i,k,j,l)
!
! This routine re-sorts a tensor
!
!--------------------------------------------------------------------- 
      subroutine sorttens(tens1,tens2,dim1,dim2,dim3,dim4)

      implicit none

      integer(long)                                             :: i, j, k, l
      integer(long), intent(in)                                 :: dim1,dim2, &
                                                                   dim3,dim4
      complex(dop), dimension(dim1,dim2,dim3,dim4), intent(in)  :: tens1
      complex(dop), dimension(dim1,dim3,dim2,dim4), intent(out) :: tens2


      do l=1,dim4
         do j=1,dim3
            do k=1,dim2
               do i=1,dim1
                  tens2(i,j,k,l)  = tens1(i,k,j,l)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine sorttens

!**********************************************************************
! projects out part of the tensor (e.g. i,j define a state pair)
!**********************************************************************

      subroutine cuttens2(tensor,matrix,i,j,vdim,dim,ndim)

      implicit none

      integer(long)                                                    :: k, l, &
                                                                          m, n
      integer(long), intent(in)                                        :: i,j
      integer(long), intent(in)                                        :: vdim, &
                                                                          dim, &
                                                                          ndim
      complex(dop), dimension(vdim,dim,ndim,vdim,dim,ndim), intent(in) :: tensor
      complex(dop), dimension(vdim,ndim,vdim,ndim), intent(out)        :: matrix

      do n=1,ndim
         do m=1,vdim
            do l=1,ndim
               do k=1,vdim
                  matrix(k,l,m,n) = tensor(k,i,l,m,j,n)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine cuttens2

!**********************************************************************
! fills the tensor from a matrix (e.g. i,j define a state pair)
!**********************************************************************

      subroutine filltens2(matrix,tensor,i,j,vdim,dim,ndim)

      implicit none

      integer(long)                                                    :: k, l, &
                                                                          m, n
      integer(long), intent(in)                                        :: i,j
      integer(long), intent(in)                                        :: vdim, &
                                                                          dim, &
                                                                          ndim
      complex(dop), dimension(vdim,dim,ndim,vdim,dim,ndim), intent(out) :: tensor
      complex(dop), dimension(vdim,ndim,vdim,ndim), intent(in)          :: matrix

      do n=1,ndim
         do m=1,vdim
            do l=1,ndim
               do k=1,vdim
                  tensor(k,i,l,m,j,n) = matrix(k,l,m,n)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine filltens2

!**********************************************************************
! fills the tensor from the c.c. of a matrix (e.g. i,j define a state pair)
!**********************************************************************

      subroutine filltens2c(matrix,tensor,i,j,vdim,dim,ndim)

      implicit none

      integer(long)                                                    :: k, l, &
                                                                          m, n
      integer(long), intent(in)                                        :: i,j
      integer(long), intent(in)                                        :: vdim, &
                                                                          dim, &
                                                                          ndim
      complex(dop), dimension(vdim,dim,ndim,vdim,dim,ndim), intent(out) :: tensor
      complex(dop), dimension(vdim,ndim,vdim,ndim), intent(in)          :: matrix

      do n=1,ndim
         do m=1,vdim
            do l=1,ndim
               do k=1,vdim
                  tensor(k,i,l,m,j,n) = dconjg(matrix(k,l,m,n))
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine filltens2c

      end module d2tens


