!-----------------------------------------------------------------------
!                      OP2LIB
!  SUBROUTINES A!TING ON TWO OBJECTS OF THE SAME TYPE (LEVEL 2 ROUTINES)
!
! NOMEN!LATURE:
!    Each of the following routines starts with:
!       add:  the objects are added
!       sub:  the second object is subtracted from the first
!       sum2: the products of the elements are summed
!       swap: the objects are swapped
!       mel:  elements of two objects are multiplied
!    This is followed by three chracters:
!    Character 1 denotes the objects on which the action is performed:
!       s: scalar
!       v: vector
!       q: quadratic matrix
!       m: general (rectangular) matrix
!       t: first object is a tensor
!       h: hermitian matrix
!       x: this is a blank
!    Character 2 specifies information about how first object is used:
!       x: this is a blank
!       a: first entry is taken as complex-conjugate
!    Character 3 specifies information about how second objects is used:
!       x: this is a blank
!       a: second entry is taken as complex-conjugate
!    Character 4 defines the data type of the objects:
!       i: integer
!       s: real single precision (real*4)
!       d: real double precision (real*8)
!       c: complex single precision (complex*8)
!       z: complex double precision (complex*16)
!    Further characters if present give more information:
!       o: the result is stored (overwritten) in second input object
!       o1: the result is stored (overwritten) in first input object
!
! !ontents:
! In the following list of available subroutines, objects on the LHS of
! the definition are input, that on the RHS output. The usual summation
! convention is used i.e. a sum is made over repeated indices on the LHS
!
!    addmxxz (a,b,c,dim1,dim2)
!       Definition: a(i,j) + b(i,j) = c(i,j)
!       Dimensions: a(dim1,dim2),b(dim1,dim2),c(dim1,dim2)
!
!    addmxazo1(a,b,dim1,dim2)
!       Definition: a(i,j) = a(i,j) + dconjg(b(j,i))
!       Dimensions: a(dim2,dim1), b(dim1,dim2)
!
!    submxxz (a,b,c,dim1,dim2)
!       Definition: a(j,i) - b(j,i) = c(j,i)
!       Dimensions: a(dim1,dim2),b(dim1,dim2),c(dim1,dim2)
!
!    subtxxzz (a,b,dim1,dim2,dim3)
!       Definition: b(i,j,k) = b(i,j,k) - a(j)
!       Dimensions: a(dim2),b(dim1,dim2,dim3)
!
!    submxazo1(a,b,dim1,dim2)
!       Definition: a(i,j) = a(i,j) - dconjg(b(j,i))
!       Dimensions: a(dim2,dim1), b(dim1,dim2)
!
!    submxczo1(a,b,dim1,dim2)
!       Definition: a(i,j) = a(i,j) - dconjg(b(i,j))
!       Dimensions: a(dim2,dim1), b(dim1,dim2)
!
!    sum2qxxz (a,b,x,dim)
!       Definition: a(j,i)*b(j,i) = x
!       Dimensions: a(dim,dim),b(dim,dim),x
!
!    sum2mxxz (a,b,x,dim1,dim2)
!       Definition: a(j,i)*b(j,i) = x
!       Dimensions: a(dim1,dim2),b(dim1,dim2),x
!      
!    sum2qxxdz (a,b,x,dim)
!       Definition: a(j,i)*b(j,i) = x
!       Dimensions: a(dim,dim),b(dim,dim),x
!
!    addmxxdo (a,b,dim1,dim2)
!       Definition: a(j,i) + b(j,i) = b(j,i)
!       Dimensions: a(dim1,dim2),b(dim1,dim2)
!
!    addmxxzo (a,b,dim1,dim2)
!       Definition: a(j,i) + b(j,i) = b(j,i)
!       Dimensions: a(dim1,dim2),b(dim1,dim2)
!
!    addvxxdo (a,b,dim)
!       Definition: a(i) + b(i) = b(i)
!       Dimensions: a(dim),b(dim)
!
!    addvxxzo (a,b,dim)
!       Definition: a(i) + b(i) = b(i)
!       Dimensions: a(dim),b(dim)
!
!    addvxxio (a,b,dim)
!       Definition: a(i) + b(i) = b(i)
!       Dimensions: a(dim),b(dim)
!
!    addvcxzo (a,b,dim)
!       Definition: dconjg(a(i)) + b(i) = b(i)
!       Dimensions: a(dim), b(dim)
!
!    submxxzo1 (a,b,dim1,dim2)
!       Definition: a(j,i) - b(j,i) = a(j,i)
!       Dimensions: a(dim1,dim2),b(dim1,dim2)
!
!    subvxxz (a,b,c,dim)
!       Definition: a(i) - b(i) = c(i)
!       Dimensions: a(dim),b(dim),c(dim)
!
!    subvxxzo1 (a,b,dim)
!       Definition: a(i) - b(i) = a(i)
!       Dimensions: a(dim),b(dim)
!
!    subvcxzo (a,b,dim)
!       Definition: b(i) - dconjg(a(i)) = b(i)
!       Dimensions: a(dim), b(dim)
!
!    swapsxxi (i,j)
!       Definition: i <-->j 
!
!    swapsxxd (a,b)
!       Definition: a <--> b
!
!    melmxxz (a,b,c,dim1,dim2)
!       Definition: a(i,j) * b(i,j) = c(i,j)
!       Dimensions: a(dim1,dim2), b(dim1,dim2), c(dim1,dim2)
!
!    melmxxzo (a,b,dim1,dim2)
!       Definition: a(i,j) * b(i,j) = b(i,j)
!       Dimensions: a(dim1,dim2), b(dim1,dim2)
!
!-----------------------------------------------------------------------

      module op2lib

use decimal, only: dop, long
use openmpmod

      implicit none
      private
      public :: addmxxz, addmxazo1, submxxz, submxazo1, submxczo1, sum2qxxz, &
                sum2mxxz, sum2qxxdz, addmxxdo, addvxxdo, addvxxzo, addvxxio, &
                addvcxzo, addmxxzo, submxxzo1, subvxxz, subvxxzo1, subvcxzo, &
                swapsxxi, swapsxxd, melmxxz, melmxxzo, subtxxzz, subvxxdo1 

       contains

!-----------------------------------------------------------------------
! Library subroutine addmxxz
!
! adds two complex rectangular matrices together
!     a(i,j) + b(i,j) = c(i,j)
!-----------------------------------------------------------------------

      subroutine addmxxz (a,b,c,dim1,dim2)

      implicit none

      integer(long), intent(in)                       :: dim1,dim2
      integer(long)                                   :: i,j
      complex(dop), dimension(dim1,dim2), intent(in)  :: a, b
      complex(dop), dimension(dim1,dim2), intent(out) :: c

      do i = 1,dim2
         do j = 1,dim1
            c(j,i) = a(j,i) + b(j,i)
         enddo
      enddo

      return
      end subroutine addmxxz 

!-----------------------------------------------------------------------
! Library subroutine addmxazo1
!
!     Definition: a(i,j) = a(i,j) + dconjg(b(j,i))
!     Dimensions: a(dim2,dim1), b(dim1,dim2)
!-----------------------------------------------------------------------

      subroutine addmxazo1 (a,b,dim1,dim2)

      implicit none

      integer(long), intent(in)                         :: dim1,dim2
      integer(long)                                     :: i,j
      complex(dop), dimension(dim1,dim2), intent(in)    :: b
      complex(dop), dimension(dim1,dim2), intent(inout) :: a

      do i=1,dim1
         do j=1,dim2
            a(j,i) = a(j,i) + dconjg(b(i,j))
         enddo
      enddo

      return
      end subroutine addmxazo1

!-----------------------------------------------------------------------
! Library subroutine submxxz
!
! subtracts one complex rectangular matrix from another
!     a(i,j) - b(i,j) = c(i,j)
!-----------------------------------------------------------------------

      subroutine submxxz (a,b,c,dim1,dim2)

      implicit none

      integer(long), intent(in)                       :: dim1,dim2
      integer(long)                                   :: i,j
      complex(dop), dimension(dim1,dim2), intent(in)  :: a, b
      complex(dop), dimension(dim1,dim2), intent(out) :: c

      do i = 1,dim2
         do j = 1,dim1
            c(j,i) = a(j,i) - b(j,i)
         enddo
      enddo

      return
      end subroutine submxxz

!-----------------------------------------------------------------------
! Library subroutine subtxxdd
!
! Subtraction of a real*8 vector from a real*8 tensor
! of third order:
!           b(i,j,k) - a(j) =  b(i,j,k).
!
! Input-variables:  a      - real vector
!                   b      - real tensor of third order
! Output-variables: b      - resulting real tensor
!-----------------------------------------------------------------------

      subroutine subtxxzz (a,b,dim1,dim2,dim3)

      implicit none

      integer(long), intent(in) :: dim1,dim2,dim3
      integer(long)             :: i, j, k
      real(dop), dimension(dim1,dim2,dim3), intent(inout) :: b
      real(dop), dimension(dim2), intent(in)              :: a

      do k=1,dim3
         do j=1,dim2
            do i=1,dim1
               b(i,j,k)=b(i,j,k)-a(j)
            enddo
         enddo
      enddo

      return
      end

!-----------------------------------------------------------------------
! Library subroutine submxazo1
!
!     Definition: a(j,i) - dconjg(b(i,j)) = a(j,i)
!     Dimensions: a(dim1,dim2),b(dim1,dim2)
!-----------------------------------------------------------------------

      subroutine submxazo1 (a,b,dim1,dim2)

      implicit none

      integer(long), intent(in)                         :: dim1,dim2
      integer(long)                                     :: i,j
      complex(dop), dimension(dim1,dim2), intent(in)    :: b
      complex(dop), dimension(dim2,dim1), intent(inout) :: a

      do i=1,dim1
         do j=1,dim2
            a(j,i) = a(j,i) - dconjg(b(i,j))
         enddo
      enddo

      return
      end subroutine  submxazo1

!-----------------------------------------------------------------------
! Library subroutine submxczo1
!
!     Definition: a(j,i) - dconjg(b(j,i)) = a(j,i)
!     Dimensions: a(dim1,dim2),b(dim1,dim2)
!-----------------------------------------------------------------------

      subroutine submxczo1 (a,b,dim1,dim2)

      implicit none

      integer(long), intent(in)                         :: dim1,dim2
      integer(long)                                     :: i,j
      complex(dop), dimension(dim1,dim2), intent(in)    :: b
      complex(dop), dimension(dim2,dim1), intent(inout) :: a

      do i=1,dim1
         do j=1,dim2
            a(j,i) = a(j,i) - dconjg(b(j,i))
         enddo
      enddo

      return
      end subroutine  submxczo1

!-----------------------------------------------------------------------
! Library subroutine sum2qxxz
!
! sums the product of elements of two complex quadratic matrices
!     a(j,i)*b(j,i) = x
!-----------------------------------------------------------------------

      subroutine sum2qxxz (a,b,x,dim)

      implicit none

      integer(long), intent(in)                    :: dim
      integer(long)                                :: i,j
      complex(dop), dimension(dim,dim), intent(in) :: a,b
      complex(dop), intent(out)                    :: x

      x=(0.0d0,0.0d0)
      do i = 1,dim
         do j = 1,dim
            x = x + a(j,i)*b(j,i)
         enddo
      enddo

      return
      end subroutine sum2qxxz

!-----------------------------------------------------------------------
! Library subroutine sum2mxxz
!
! sums the product of elements of two complex general matrices
!     a(j,i)*b(j,i) = x
!-----------------------------------------------------------------------

      subroutine sum2mxxz (a,b,x,dim1,dim2)

      implicit none

      integer(long), intent(in)                      :: dim1,dim2
      integer(long)                                  :: i,j
      complex(dop), dimension(dim1,dim2), intent(in) :: a,b
      complex(dop), intent(out)                      :: x

      x=(0.0d0,0.0d0)
      do i = 1,dim2
         do j = 1,dim1
            x = x + a(j,i)*b(j,i)
         enddo
      enddo

      return
      end subroutine sum2mxxz

!-----------------------------------------------------------------------
! Library subroutine sum2qxxdz
!
! sums the product of elements of a real and a complex quadratic matrix
!     a(j,i)*b(j,i) = x
!-----------------------------------------------------------------------

      subroutine sum2qxxdz (a,b,x,dim)

      implicit none

      integer(long), intent(in)                    :: dim
      integer(long)                                :: i,j
      real(dop), dimension(dim,dim), intent(in)    :: a
      complex(dop), dimension(dim,dim), intent(in) :: b
      complex(dop), intent(out)                    :: x

      x=(0.0d0,0.0d0)
      do i = 1,dim
         do j = 1,dim
            x = x + a(j,i)*b(j,i)
         enddo
      enddo

      return
      end subroutine sum2qxxdz 

!-----------------------------------------------------------------------
! Library subroutine addmxxdo
!
! adds two real rectangular matrices together
!     a(i,j) + b(i,j) = b(i,j)
!
! NB input matrix b is overwritten and holds output at end of routine
!-----------------------------------------------------------------------

      subroutine addmxxdo (a,b,dim1,dim2)

      implicit none

      integer(long), intent(in)                      :: dim1,dim2
      integer(long)                                  :: i,j
      real(dop), dimension(dim1,dim2), intent(in)    :: a
      real(dop), dimension(dim1,dim2), intent(inout) :: b

      do i = 1,dim2
         do j = 1,dim1
            b(j,i) = a(j,i) + b(j,i)
         enddo
      enddo

      return
      end subroutine addmxxdo

!-----------------------------------------------------------------------
! Library subroutine addvxxdo
!
! adds two real vectors together
!     a(i) + b(i) = b(i)
!
! NB input vector b is overwritten and holds output at end of routine
!-----------------------------------------------------------------------

      subroutine addvxxdo (a,b,dim)

      implicit none

      integer(long), intent(in)                :: dim
      integer(long)                            :: i
      real(dop), dimension(dim), intent(in)    :: a
      real(dop), dimension(dim), intent(inout) :: b

      do i = 1,dim
         b(i) = a(i) + b(i)
      enddo

      return
      end subroutine addvxxdo

!-----------------------------------------------------------------------
! Library subroutine addvxxzo
!
! adds two complex vectors together
!     a(i) + b(i) = b(i)
!
! NB input vector b is overwritten and holds output at end of routine
!-----------------------------------------------------------------------

      subroutine addvxxzo (a,b,dim)

      implicit none

      integer(long), intent(in)                   :: dim
      integer(long)                               :: i
      complex(dop), dimension(dim), intent(in)    :: a
      complex(dop), dimension(dim), intent(inout) :: b

!$omp parallel do num_threads(ompthread)
      do i = 1,dim
         b(i) = a(i) + b(i)
      enddo
!$omp end parallel do 

      return
      end subroutine addvxxzo

!-----------------------------------------------------------------------
! Library subroutine addvxxio
!
! adds two integer vectors together
!     a(i) + b(i) = b(i)
!
! NB input vector b is overwritten and holds output at end of routine
!-----------------------------------------------------------------------

      subroutine addvxxio (a,b,dim)

      implicit none

      integer(long), intent(in)                    :: dim
      integer(long)                                :: i
      integer(long), dimension(dim), intent(in)    :: a
      integer(long), dimension(dim), intent(inout) :: b

      do i = 1,dim
         b(i) = a(i) + b(i)
      enddo

      return
      end subroutine addvxxio

!-----------------------------------------------------------------------
! Library subroutine addvcxzo
!
! adds two complex vectors together
!     dconjg(a(i)) + b(i) = b(i)
!
! NB input vector b is overwritten and holds output at end of routine
!-----------------------------------------------------------------------

      subroutine addvcxzo (a,b,dim)

      implicit none

      integer(long), intent(in)                   :: dim
      integer(long)                               :: i
      complex(dop), dimension(dim), intent(in)    :: a
      complex(dop), dimension(dim), intent(inout) :: b

      do i = 1,dim
         b(i) = dconjg(a(i)) + b(i)
      enddo

      return
      end subroutine addvcxzo

!-----------------------------------------------------------------------
! Library subroutine addmxxzo
!
! adds two complex rectangular matrices together
!     a(i,j) + b(i,j) = b(i,j)
!
! NB input matrix b is overwritten and holds output at end of routine
!-----------------------------------------------------------------------

      subroutine addmxxzo (a,b,dim1,dim2)

      implicit none

      integer(long), intent(in)                         :: dim1,dim2
      integer(long)                                     :: i,j
      complex(dop), dimension(dim1,dim2), intent(in)    :: a
      complex(dop), dimension(dim1,dim2), intent(inout) :: b

      do i = 1,dim2
         do j = 1,dim1
            b(j,i) = a(j,i) + b(j,i)
         enddo
      enddo

      return
      end subroutine addmxxzo

!-----------------------------------------------------------------------
! Library subroutine submxxzo1
!
! subtracts a complex rectangular matrix from another
!     a(i,j) - b(i,j) = a(i,j)
!
! NB input matrix a is overwritten and holds output at end of routine
!-----------------------------------------------------------------------

      subroutine submxxzo1 (a,b,dim1,dim2)

      implicit none

      integer(long), intent(in)                         :: dim1,dim2
      integer(long)                                     :: i,j
      complex(dop), dimension(dim1,dim2), intent(in)    :: b
      complex(dop), dimension(dim1,dim2), intent(inout) :: a

      do i = 1,dim2
         do j = 1,dim1
            a(j,i) = a(j,i) - b(j,i)
         enddo
      enddo

      return
      end subroutine submxxzo1

!-----------------------------------------------------------------------
! Library subroutine subvxxz
! 
! subtracts a complex vector from another
!     a(i) - b(i) = c(i)
! 
!-----------------------------------------------------------------------

      subroutine subvxxz (a,b,c,dim)

      implicit none

      integer(long), intent(in)                 :: dim
      integer(long)                             :: i
      complex(dop), dimension(dim), intent(in)  :: a,b
      complex(dop), dimension(dim), intent(out) :: c

      do i = 1,dim
         c(i) = a(i) - b(i)
      enddo

      return
      end subroutine subvxxz

!-----------------------------------------------------------------------
! Library subroutine subvxxzo1
!
! subtracts a complex vector from another
!     a(i) - b(i) = a(i)
!
! NB input vector a is overwritten and holds output at end of routine
!-----------------------------------------------------------------------

      subroutine subvxxzo1 (a,b,dim)

      implicit none

      integer(long), intent(in)                   :: dim
      integer(long)                               :: i
      complex(dop), dimension(dim), intent(in)    :: b
      complex(dop), dimension(dim), intent(inout) :: a

      do i = 1,dim
         a(i) = a(i) - b(i)
      enddo

      return
      end subroutine subvxxzo1

!-----------------------------------------------------------------------
! Library subroutine subvcxzo
!
! adds two complex vectors together
!     b(i) - dconjg(a(i)) = b(i)
!
! NB input vector b is overwritten and holds output at end of routine
!-----------------------------------------------------------------------

      subroutine subvcxzo (a,b,dim)

      implicit none

      integer(long), intent(in)                   :: dim
      integer(long)                               :: i
      complex(dop), dimension(dim), intent(in)    :: a
      complex(dop), dimension(dim), intent(inout) :: b

      do i = 1,dim
         b(i) = b(i) - dconjg(a(i)) 
      enddo

      return
      end subroutine subvcxzo

!-----------------------------------------------------------------------
! Library subroutine subvxxdo1
!
! subtracts a complex vector from another
!     a(i) - b(i) = a(i)
!
! NB input vector a is overwritten and holds output at end of routine
!-----------------------------------------------------------------------

      subroutine subvxxdo1 (a,b,dim)

      implicit none

      integer(long), intent(in)                :: dim
      integer(long)                            :: i
      real(dop), dimension(dim), intent(in)    :: b
      real(dop), dimension(dim), intent(inout) :: a

      do i = 1,dim
         a(i) = a(i) - b(i)
      enddo

      return
      end subroutine subvxxdo1

!-----------------------------------------------------------------------
! Library subroutine swapsxxi
!
! swappes two integers: a <--> b
!
!-----------------------------------------------------------------------

      subroutine swapsxxi (int1,int2)

      implicit none

      integer(long), intent(inout) :: int1,int2
      integer(long)                :: aux

      aux  = int1
      int1 = int2
      int2 = aux

      return
      end subroutine swapsxxi

!-----------------------------------------------------------------------
! Library subroutine swapsxxd
!
! swappes two real variables: a <--> b
!
!-----------------------------------------------------------------------

      subroutine swapsxxd (x1,x2)

      implicit none

      real(dop), intent(inout) :: x1,x2
      real(dop)                :: aux

      aux  = x1
      x1 = x2
      x2 = aux

      return
      end subroutine swapsxxd

!-----------------------------------------------------------------------
! Library subroutine melmxxz
!
! multiplies elements of two complex rectangular matrices together
!     a(i,j) * b(i,j) = c(i,j)
!
!-----------------------------------------------------------------------

      subroutine melmxxz (a,b,c,dim1,dim2)

      implicit none

      integer(long), intent(in)                       :: dim1,dim2
      integer(long)                                   :: i,j
      complex(dop), dimension(dim1,dim2), intent(in)  :: a,b
      complex(dop), dimension(dim1,dim2), intent(out) :: c

      do i = 1,dim2
         do j = 1,dim1
            c(j,i) = a(j,i) * b(j,i)
         enddo
      enddo

      return
      end subroutine melmxxz

!-----------------------------------------------------------------------
! Library subroutine melmxxzo
!
! multiplies elements of two complex rectangular matrices together
! and stores the result in the second input matrix
!     a(i,j) * b(i,j) = b(i,j)
!
!-----------------------------------------------------------------------

      subroutine melmxxzo (a,b,dim1,dim2)

      implicit none

      integer(long), intent(in)                         :: dim1,dim2
      integer(long)                                     :: i,j
      complex(dop), dimension(dim1,dim2), intent(in)    :: a
      complex(dop), dimension(dim1,dim2), intent(inout) :: b

      do i = 1,dim2
         do j = 1,dim1
            b(j,i) = a(j,i) * b(j,i)
         enddo
      enddo

      return
      end subroutine melmxxzo

      end module op2lib
