!=======================================================================
module gridutil
!=======================================================================

implicit none

contains !==============================================================


!-----------------------------------------------------------------------
function next_point(idx,dims,lmask) result(notdone)
!-----------------------------------------------------------------------
   integer,intent(inout)       :: idx(:)
   integer,intent(in)          :: dims(:)
   logical,intent(in),optional :: lmask(:)
   logical                     :: notdone
   integer                     :: d
   notdone = .true.
   do d=1,size(idx)
      if (present(lmask)) then
         if (.not.lmask(d)) cycle
      endif
      idx(d) = idx(d) + 1
      if (idx(d) <= dims(d)) return
      idx(d) = 1
   enddo
   notdone = .false.
end function next_point


!-----------------------------------------------------------------------
pure function flat_index(idx,dims,lmask) result(flat)
!-----------------------------------------------------------------------
   integer,intent(in)          :: idx(:)
   integer,intent(in)          :: dims(:)
   logical,intent(in),optional :: lmask(:)
   integer                     :: flat
   integer                     :: d
   flat = 0
   do d=size(idx),1,-1
      if (present(lmask)) then
         if (.not.lmask(d)) cycle
      endif
      flat = flat*dims(d) + idx(d) - 1
   enddo
   flat = flat + 1
end function flat_index

!-----------------------------------------------------------------------
subroutine get_indices(idx,dims,lmask,flat)
!-----------------------------------------------------------------------
   integer,intent(inout)       :: idx(:)
   integer,intent(in)          :: dims(:)
   logical,intent(in),optional :: lmask(:)
   integer,intent(in)          :: flat
   integer                     :: d,flat0
   flat0 = flat - 1
   do d=1,size(dims)
      if (present(lmask)) then
         if (.not.lmask(d)) cycle
      endif
      idx(d) = mod(flat0,dims(d)) + 1
      flat0 = flat0/dims(d)
   enddo
end subroutine get_indices


!-----------------------------------------------------------------------
pure function shape3d(dims,mode) result(ijk)
!-----------------------------------------------------------------------
   integer            :: ijk(3)
   integer,intent(in) :: dims(:)
   integer,intent(in) :: mode
   integer            :: ndim
   ndim = size(dims)
   ijk = [1, dims(mode), 1]
   if (mode>1)    ijk(1)=product(dims(1:mode-1))
   if (mode<ndim) ijk(3)=product(dims(mode+1:ndim))
end function shape3d


!=======================================================================
end module gridutil
