!=======================================================================
module llist_int
!=======================================================================

use mltypes, only: llist_int_t,llist_int_tp,llist_node_t

implicit none
private

public :: llist_int_t, llist_int_tp
public :: llist_push, llist_add, llist_to_array

interface llist_push
   module procedure llist_int_push
end interface llist_push

interface llist_add
   module procedure llist_int_add
end interface llist_add

interface llist_to_array
   module procedure llist_int_to_array
end interface llist_to_array


contains !==============================================================


!-----------------------------------------------------------------------
subroutine llist_int_push(list,val)
! Adds val to the end of list.
!-----------------------------------------------------------------------
   type(llist_int_t),intent(inout) :: list
   integer,intent(in)              :: val
   type(llist_node_t),pointer      :: node
   allocate(node)
   node%val = val
   if (associated(list%last)) then
      node%prev => list%last
      list%last%next => node
      list%last => node
      list%sz = list%sz + 1
   else
      list%first => node
      list%last => node
      list%sz = 1
   endif
end subroutine llist_int_push


!-----------------------------------------------------------------------
function llist_int_find(list,val) result(node)
! Returns list node that contains val.
!-----------------------------------------------------------------------
   type(llist_int_t),intent(in) :: list
   integer,intent(in)           :: val
   type(llist_node_t),pointer   :: node
   type(llist_node_t),pointer   :: node1
   nullify(node)
   node1 => list%first
   do while (associated(node1))
      if (node1%val == val) then
         node => node1
         return
      endif
      node1 => node1%next
   enddo
   return
end function llist_int_find


!-----------------------------------------------------------------------
subroutine llist_int_add(list,val)
! Adds val to the end of list, unless val is already in the list.
!-----------------------------------------------------------------------
   type(llist_int_t),intent(inout) :: list
   integer,intent(in)              :: val
   type(llist_node_t),pointer      :: node
   node => llist_int_find(list,val)
   if (associated(node)) return
   call llist_int_push(list,val)
end subroutine llist_int_add


!-----------------------------------------------------------------------
function llist_int_to_array(list) result(array)
! Creates an array with the elements from the list.
!-----------------------------------------------------------------------
   type(llist_int_t),intent(in) :: list
   integer,pointer              :: array(:)
   integer                      :: i
   type(llist_node_t),pointer   :: node
   allocate(array(list%sz))
   node => list%first
   do i=1,list%sz
      array(i) = node%val
      node => node%next
   enddo
end function llist_int_to_array


!=======================================================================
end module llist_int
