module logging
!
! Logging Infrastructure.
!
! Currently this is just a thin wrapper for open/close/write,
! but this can be expanded to handle MPI-parallel runs, where
! all I/O should be done by the master process.


   use mlbase
   use lazyflush_m
   implicit none
   private

   public :: open_log, close_log, write_log

   integer,parameter :: maxunit=999


   contains


   subroutine open_log(lun,filename)
      implicit none
      character(len=*),intent(in) :: filename
      integer,intent(out)         :: lun
      character(len=160)          :: errmsg
      integer                     :: ierr
      lun = getlun()
      if (lun < 0) then
         call stop_code('logging_m', 'open_log', 'no I/O unit available')
      endif
      open (unit=lun,iostat=ierr,file=filename,form='formatted',status='unknown')
      if (ierr /= 0) then
         write (errmsg,'(a,i0,2a)') &
            'cannot open, status=', ierr, ', file=', trim(filename)
         call stop_code('logging_m', 'open_log', errmsg)
      endif
   end subroutine open_log


   subroutine close_log(lun)
      implicit none
      integer,intent(in) :: lun
      close(lun)
   end subroutine close_log


   subroutine write_log(lun,msg)
      implicit none
      integer,intent(in)          :: lun
      character(len=*),intent(in) :: msg
      write (lun,'(a)') trim(msg)
      call lazyflush(lun)
   end subroutine write_log


   function getlun() result (lun)
   ! finds an unused unit number
      implicit none
      integer :: lun
      logical :: isopen,exists
      do lun=maxunit,1,-1
         inquire(unit=lun,exist=exists,opened=isopen)
         if (exists.and..not.isopen) return
      enddo
      lun = -1 ! signal error
   end function getlun


end module logging
