module mlbase

use decimal
! ----------------------------------------------------------------------
! This is the "base" module for other Fortran95 modules. Here we provide
! stuff that is generally useful, but not particularly connected to MCTDH.
! It contains:
! - some mathematical constants.
! - a routine for aborting the program with an error message
!
! Originally part of mlcore/global.f90
! ----------------------------------------------------------------------

    implicit none

    ! Math
!    real(dbl), parameter           ::   PI = 3.14159265358979323846_dbl
!    real(dbl), parameter           ::   EE = 2.71828182845904523536_dbl
!    real(dbl), parameter           ::  ONE = 1.0_dbl
!    real(dbl), parameter           :: ZERO = 0.0_dbl
!    complex(cdbl), parameter       :: IONE = (0.0_dbl, 1.0_dbl)
!    complex(cdbl), parameter       :: CONE = (1.0_dbl, 0.0_dbl)
!    complex(cdbl), parameter       ::CZERO = (0.0_dbl, 0.0_dbl)

    ! I/O channels
    integer :: io_err = 6  ! standard output (usually screen)

    contains

    subroutine stop_code(mymod,myrout,mymsg)
        implicit none
        character(len=*),intent(in)          :: mymod
        character(len=*),intent(in)          :: myrout
        character(len=*),intent(in),optional :: mymsg
        write(io_err,*)
        write(io_err,*) '** ERROR **'
        write(io_err,*) '==========='
        write(io_err,*) ' module:     ',trim(mymod)
        write(io_err,*) ' subroutine: ',trim(myrout)
        if(present(mymsg)) then
           write(io_err,*) ' message:    ',trim(mymsg)
        end if
        write(io_err,*)
        stop 1
    end subroutine stop_code

    subroutine stop_code_2(mymod,myrout,mymsg)
        implicit none
        character(len=*),intent(in)          :: mymod
        character(len=*),intent(in)          :: myrout
        character(len=*),intent(in),optional :: mymsg
        write(2,*) '** ERROR **'
        write(2,*) '==========='
        write(2,*) ' module:     ',trim(mymod)
        write(2,*) ' subroutine: ',trim(myrout)
        if(present(mymsg)) then
           write(2,*)      ' message:    ',trim(mymsg)
        end if
        write(2,*)
        call stop_code(mymod,myrout,mymsg)
    end subroutine stop_code_2

    subroutine warn_msg(mymod,myrout,mymsg)
      implicit none
      character(len=*),intent(in)          :: mymod
      character(len=*),intent(in)          :: myrout
      character(len=*),intent(in),optional :: mymsg

      write(io_err,*)
      write(io_err,*) '** WARNING **'
      write(io_err,*) '============='
      write(io_err,*) ' module:     ',trim(mymod)
      write(io_err,*) ' subroutine: ',trim(myrout)
      if(present(mymsg)) then
         write(io_err,*) ' message:    ',trim(mymsg)
      end if
      write(io_err,*)

    end subroutine warn_msg


    function deql(x,y,eps)
    ! Checks whether x and y are "equal" (within precision eps, default 10^-9)
        implicit none
        real(dbl),intent(in)          :: x,y
        real(dbl),intent(in),optional :: eps
        logical                       :: deql
        real(dbl)                     :: eps1
        if (present(eps)) then
           eps1 = eps
        else
           eps1 = 1.0d-9
        endif
        deql = (abs(x-y) < abs(eps1*y))
    end function deql


    function dgeq(x,y,eps)
    ! Checks whether x is greater than or "equal" to y (within precision eps, default 10^-9)
        implicit none
        real(dbl),intent(in)          :: x,y
        real(dbl),intent(in),optional :: eps
        logical                       :: dgeq
        real(dbl)                     :: eps1
        if (present(eps)) then
           eps1 = eps
        else
           eps1 = 1.0d-9
        endif
        dgeq = (x>y) .or. deql(x,y,eps)
    end function dgeq


    function dleq(x,y,eps)
    ! Checks whether x is less than or "equal" to y (within precision eps, default 10^-9)
        implicit none
        real(dbl),intent(in)          :: x,y
        real(dbl),intent(in),optional :: eps
        logical                       :: dleq
        real(dbl)                     :: eps1
        if (present(eps)) then
           eps1 = eps
        else
           eps1 = 1.0d-9
        endif
        dleq = (x<y) .or. deql(x,y,eps)
    end function dleq


end module mlbase
