!=======================================================================
module partitions
!-----------------------------------------------------------------------
! Generate all ways to split a set into two (non-empty) subsets.
! Only the size of the original set needs to be known. A "split" _pt_
! is given by an array of logicals, such that
!   pt(i) = .true.   if the i-th element is in the 1st subset
!   pt(i) = .false.  if the i-th element is in the 2nd subset.
!=======================================================================

implicit none
private

public :: partition_count, &
          partition_get


contains !==============================================================


!-----------------------------------------------------------------------
function partition_count(sz) result(cnt)
! Returns the number of possible ways to split a set of size _sz_.
!-----------------------------------------------------------------------
   integer,intent(in) :: sz   ! size of set
   integer            :: cnt  ! number of possible splits
   cnt = 2**(sz-1) - 1
end function partition_count


!-----------------------------------------------------------------------
function partition_get(sz,p) result(pt)
! Returns the p-th possible way to split a set of size _st_.
!-----------------------------------------------------------------------
   integer,intent(in) :: sz     ! size of set
   integer,intent(in) :: p      ! which "split" to generate
   logical            :: pt(sz) ! see header
   integer            :: n,b,mask
   n = sz-1
   mask = 1
   pt(1) = .false.
   do b=2,sz
      pt(b) = (IAND(p,mask) /= 0)
      mask = ISHFT(mask,1)
   enddo
end function partition_get


!=======================================================================
end module partitions
