
      module davutilspar

use decimal, only:long, dop

#ifdef MPI
      implicit none
      private
      public :: mpiorthonr,mpiorthonc,mpiovl,mpiovlc,mpikry2finalpsi, &
                mpikryc2finalpsi,mpipsi2kry,mpikry2psi,mpipsi2kryc,mpikryc2psi

      contains

! **********************************************************************
!    Orthogonlisation of real vectors.
!    v is orthogonalized against Krylov(0)...Krylov(n)
! ----------------------------------------------------------------------

      Subroutine mpiorthonr(Krylov,v,n,PsiDim)

      Implicit None

      Integer(long), intent(in)                    :: PsiDim,n
      Integer(long)                                :: i,D
      Real(dop), dimension(PsiDim,0:n), intent(in) :: Krylov
      Real(dop), dimension(PsiDim), intent(out)    :: v
      Real(dop)                                    :: ovl

      Do i = 0, n
         ovl =0.d0
         Do D = 1,PsiDim
            ovl = ovl + Krylov(D,i)*v(D)
         enddo
         Do D = 1,PsiDim
            v(D) = v(D)-ovl*Krylov(D,i)
         enddo
      enddo

      Return
      End Subroutine mpiorthonr

! **********************************************************************
!    Orthogonlisation of complex vectors.
!    v is orthogonalized against Krylov(0)...Krylov(n)
! ----------------------------------------------------------------------

      Subroutine mpiorthonc(Krylov,v,n,PsiDim)

      Implicit None

      Integer(long), intent(in)                       :: PsiDim,n
      Integer(long)                                   :: i,D
      Complex(dop), dimension(PsiDim,0:n), intent(in) :: Krylov
      Complex(dop), dimension(PsiDim), intent(out)    :: v
      Complex(dop)                                    :: ovl

      Do i = 0, n
         ovl =0.d0
         Do D = 1,PsiDim
            ovl = ovl + dconjg(Krylov(D,i))*v(D)
         enddo
         Do D = 1,PsiDim
            v(D) = v(D)-ovl*Krylov(D,i)
         enddo
      enddo

      Return
      End Subroutine mpiorthonc

! **********************************************************************
!    Normalisation of real vector.
!    v is normalized.
! ----------------------------------------------------------------------

      Subroutine mpiovl(v,ovl,PsiDim)

      Implicit None

      Integer(long), intent(in)                    :: PsiDim
      Integer(long)                                :: D
      Real(dop), dimension(PsiDim), intent(out)    :: v
      Real(dop), intent(out)                       :: ovl

      ovl = 0.d0
      Do D = 1,PsiDim
         ovl = ovl + v(D)**2
      enddo
      ovl=sqrt(ovl)
      if(ovl .le. 1.d-40) return
      Do D = 1,PsiDim
         v(D)=v(D)/ovl
      enddo

      Return
      End Subroutine mpiovl

! **********************************************************************
!    Normalisation of complex vector.
!    v is normalized.
! ----------------------------------------------------------------------

      Subroutine mpiovlc(v,ovl,PsiDim)

      Implicit None

      Integer(long), intent(in)                       :: PsiDim
      Integer(long)                                   :: D
      Complex(dop), dimension(PsiDim), intent(out)    :: v
      Real(dop),intent(out)                           :: ovl

      ovl = 0.d0
      Do D = 1,PsiDim
         ovl = ovl + v(D)*dconjg(v(D))
      enddo
      ovl=sqrt(ovl)
      if(dble(ovl) .le. 1.d-40) return
      Do D = 1,PsiDim
         v(D)=v(D)/ovl
      enddo

      Return
      End Subroutine mpiovlc

!***********************************************************************

      subroutine mpikry2finalpsi(Psi,DtPsi,Krylov,HKrylov, &
                              EigenVector,TrueOrder,mpiorder,qmap)

use global
use maxv, only: maxdim, maxsta
use griddatmod
use psidef
use runpropmod
use mpidata

      implicit none

      integer(long)                                      :: s,q,qq,p,d,d1,d2
      integer(long), intent(in)                          :: TrueOrder,mpiorder
      integer(long),dimension(0:8*npackts+9), intent(in) :: qmap
      real(dop), dimension (adim/npackts,0:intordermpi), intent(in) :: Krylov, &
                                                                       HKrylov
      real(dop), dimension (0:intordera,0:intordera), intent(in)    :: EigenVector
      complex(dop), dimension(adim), intent(inout)                  :: psi,dtpsi

      do s = 1, nstate
         Do QQ = 0,npackts-1
            Q = qmap(QQ)
            Do P = 0,mpiOrder-1
               Do D = 1,block(s)/npackts
                  d1=zpsi(s)-1+d
                  d2=(zpsi(s)-1)/npackts+d
                  Psi(d1+QQ*block(s)/npackts) = Psi(d1+QQ*block(s)/npackts) + &
                       Krylov(d2,P)*EigenVector(intordermpi*mpirank+P,Q)
                  DtPsi(d1+QQ*block(s)/npackts) = DtPsi(d1+QQ*block(s)/npackts) + &
                      HKrylov(d2,P)*EigenVector(intordermpi*mpirank+P,Q)
               EndDo
            EndDo
         EndDo
      enddo

      return
      end subroutine mpikry2finalpsi

!***********************************************************************

      subroutine mpikryc2finalpsi(Psi,DtPsi,Krylov,HKrylov, &
                              EigenVector,TrueOrder,mpiorder,qmap)

use global
use maxv, only: maxdim, maxsta
use griddatmod
use psidef
use runpropmod
use mpidata

      implicit none

      integer(long)                                      :: s,q,qq,p,d,d1,d2
      integer(long), intent(in)                          :: TrueOrder,mpiorder
      integer(long),dimension(0:8*npackts+9), intent(in) :: qmap
      complex(dop), dimension (adim/npackts,0:intordermpi), intent(in) :: Krylov, &
                                                                       HKrylov
      complex(dop), dimension (0:intordera,0:intordera), intent(in)  :: EigenVector
      complex(dop), dimension(adim), intent(inout)                   :: psi,dtpsi

      do s = 1, nstate
         Do QQ = 0,npackts-1
            Q = qmap(QQ)
            Do P = 0,mpiOrder-1
               Do D = 1,block(s)/npackts
                  d1=zpsi(s)-1+d
                  d2=(zpsi(s)-1)/npackts+d
                  Psi(d1+QQ*block(s)/npackts) = Psi(d1+QQ*block(s)/npackts) + &
                      Krylov(d2,P)*EigenVector(intordermpi*mpirank+P,Q)
                  DtPsi(d1+QQ*block(s)/npackts) = DtPsi(d1+QQ*block(s)/npackts) + &
                      HKrylov(d2,P)*EigenVector(intordermpi*mpirank+P,Q)
               EndDo
            EndDo
         EndDo
      enddo

      return
      end subroutine mpikryc2finalpsi

!***********************************************************************

      subroutine mpipsi2kry(Psi,Krylov,blk,strtord,stopord)

use global
use maxv, only: maxdim, maxsta
use griddatmod
use psidef
use runpropmod
use mpidata

      implicit none

      integer(long)                                      :: s,q,d
      integer(long), intent(in)                          :: blk,strtord,stopord
      real(dop), dimension (adim/npackts,0:intordermpi), intent(out) :: Krylov
      complex(dop), dimension(adim), intent(in)                      :: psi

      do s = 1, nstate
         Do q = strtord, stopord
            Do d = 1, block(s)/npackts
               Krylov(((zpsi(s)-1)/npackts)+d,q) = &
                  dble(Psi(zpsi(s)-1+d+(q+blk)*(block(s)/npackts)))
            enddo
         enddo
      enddo

      return
      end subroutine mpipsi2kry

!***********************************************************************

      subroutine mpikry2psi(Psi,Krylov,blk,strtord,stopord)

use global
use maxv, only: maxdim, maxsta
use griddatmod
use psidef
use runpropmod
use mpidata

      implicit none

      integer(long)                                      :: s,q,d
      integer(long), intent(in)                          :: blk,strtord,stopord
      real(dop), dimension (adim/npackts,0:intordermpi), intent(in) :: Krylov
      complex(dop), dimension(adim), intent(out)                    :: psi

      do s = 1, nstate
         Do q = strtord, stopord
            Do d = 1, block(s)/npackts
               Psi(zpsi(s)-1+d+(q+blk)*(block(s)/npackts)) = &
                                              Krylov(((zpsi(s)-1)/npackts)+d,q)
            enddo
         enddo
      enddo

      return
      end subroutine mpikry2psi

!***********************************************************************

      subroutine mpipsi2kryc(Psi,Krylov,blk,strtord,stopord)

use global
use maxv, only: maxdim, maxsta
use griddatmod
use psidef
use runpropmod
use mpidata

      implicit none

      integer(long)                                      :: s,q,d
      integer(long), intent(in)                          :: blk,strtord,stopord
      complex(dop), dimension (adim/npackts,0:intordermpi), intent(out) :: Krylov
      complex(dop), dimension(adim), intent(in)                         :: psi

      do s = 1, nstate
         Do q = strtord, stopord
            Do d = 1, block(s)/npackts
               Krylov(((zpsi(s)-1)/npackts)+d,q) = &
                  Psi(zpsi(s)-1+d+(q+blk)*(block(s)/npackts))
            enddo
         enddo
      enddo

      return
      end subroutine mpipsi2kryc

!***********************************************************************

      subroutine mpikryc2psi(Psi,Krylov,blk,strtord,stopord)

use global
use maxv, only: maxdim, maxsta
use griddatmod
use psidef
use runpropmod
use mpidata

      implicit none

      integer(long)                                      :: s,q,d
      integer(long), intent(in)                          :: blk,strtord,stopord
      complex(dop), dimension (adim/npackts,0:intordermpi), intent(in) :: Krylov
      complex(dop), dimension(adim), intent(out)                       :: psi

      do s = 1, nstate
         Do q = strtord, stopord
            Do d = 1, block(s)/npackts
               Psi(zpsi(s)-1+d+(q+blk)*(block(s)/npackts)) = &
                                   Krylov(((zpsi(s)-1)/npackts)+d,q)
            enddo
         enddo
      enddo

      return
      end subroutine mpikryc2psi

!***********************************************************************
#endif
      end module davutilspar
