     module eulerlib

use decimal, only: long, dop

     implicit none
     public :: euler

     contains

!--------------------------------------------------------------------------
! SUBROUTINE EULER
!
! Integrated a system of complex, linear first-order differential equations
! using the one-step Euler method.
! GWR 02/19
! INPUT PARAMETERS:                                                  
!    Psi:       The (complex) initial-value vector.                   
!    DtPsi:     Action of Hamiltonian on the initial-value vector, i.e.
!               H|Psi> or -i H|Psi>, depending on "StdForm".     
!    PsiDim     Length of Psi and DtPsi vectors.                      
!    IntPeriod: Length of time interval to be integrated.             
! OUTPUT PARAMETERS:                                                 
!    Psi:       Propagated Psi.                                       
!--------------------------------------------------------------------------

     subroutine euler(psi,dtpsi,psidim,intperiod)

     implicit none

     integer(long), intent(in) :: psidim
     real(dop), intent(in)     :: intperiod
     complex(dop), dimension(psidim), intent(inout) :: psi
     complex(dop), dimension(psidim), intent(in)    :: dtpsi

     complex(dop), dimension(psidim) :: psi1

     psi1 = psi + dtpsi*intperiod
     psi  = psi1

     return

     end subroutine euler

     end module eulerlib
