      module pardsyevmod

use decimal, only:long, dop

      implicit none
      private
      public :: pardsyev
      contains

      SUBROUTINE PARDSYEV( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, INFO )

#ifdef MPI
use mpi
#endif

use decimal, only:long, dop

use distribmod
use mpidata

      implicit none

!
!     MRB 09/08
!     PARDSYEV is a parallel version of the LAPACK routine DSYEV --
!
!     .. Scalar Arguments ..
      CHARACTER, intent(in)      ::       JOBZ, UPLO
      INTEGER(long), intent(in)  ::   LDA, LWORK, N
      INTEGER(long), intent(out) ::   INFO
!     ..
!     .. Array Arguments ..
      real(dop), dimension(lda,*), intent(out) :: A
      real(dop), dimension(*), intent(out)     :: W, WORK
!     ..
!
!  Purpose
!  =======
!
!  PARDSYEV computes all eigenvalues and, optionally, eigenvectors of a
!  real symmetric matrix A in parallel.
!
!  Arguments
!  =========
!
!  JOBZ    (input) CHARACTER*1
!          = 'N':  Compute eigenvalues only;
!          = 'V':  Compute eigenvalues and eigenvectors.
!
!  UPLO    (input) CHARACTER*1
!          = 'U':  Upper triangle of A is stored;
!          = 'L':  Lower triangle of A is stored.
!
!  N       (input) INTEGER
!          The order of the matrix A.  N >= 0.
!
!  A       (input/output) DOUBLE PRECISION array, dimension (LDA, N)
!          On entry, the symmetric matrix A.  If UPLO = 'U', the
!          leading N-by-N upper triangular part of A contains the
!          upper triangular part of the matrix A.  If UPLO = 'L',
!          the leading N-by-N lower triangular part of A contains
!          the lower triangular part of the matrix A.
!          On exit, if JOBZ = 'V', then if INFO = 0, A contains the
!          orthonormal eigenvectors of the matrix A.
!          If JOBZ = 'N', then on exit the lower triangle (if UPLO='L')
!          or the upper triangle (if UPLO='U') of A, including the
!          diagonal, is destroyed.
!
!  LDA     (input) INTEGER
!          The leading dimension of the array A.  LDA >= max(1,N).
!
!  W       (output) DOUBLE PRECISION array, dimension (N)
!          If INFO = 0, the eigenvalues in ascending order.
!
!  WORK    (workspace/output) DOUBLE PRECISION array, dimension (LWORK)
!          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
!
!  LWORK   (input) INTEGER
!          The length of the array WORK.  LWORK >= max(1,3*N-1).
!          For optimal efficiency, LWORK >= (NB+2)*N,
!          where NB is the blocksize for DSYTRD returned by ILAENV.
!
!          If LWORK = -1, then a workspace query is assumed; the routine
!          only calculates the optimal size of the WORK array, returns
!          this value as the first entry of the WORK array, and no error
!          message related to LWORK is issued by XERBLA.
!
!  INFO    (output) INTEGER
!          = 0:  successful exit
!          < 0:  if INFO = -i, the i-th argument had an illegal value
!          > 0:  if INFO = i, the algorithm failed to converge; i
!                off-diagonal elements of an intermediate tridiagonal
!                form did not converge to zero.
!
!  =====================================================================
!
!     .. Parameters ..
      real(dop), parameter :: RZERO = 0.0_dop
      real(dop), parameter :: RONE = 1.0_dop
!     ..
!     .. Local Scalars ..
      LOGICAL(kind=4) :: LOWER, LQUERY, WANTZ
      INTEGER(long) ::   IINFO, IMAX, INDE, INDTAU, INDWRK, ISCALE, &
                         LLWORK, LOPT, LWKOPT, NB 
      real(dop) ::       ANRM, BIGNUM, EPS, RMAX, RMIN, SAFMIN, SIGMA,SMLNUM
!     ..
!     .. External Functions ..
      LOGICAL(kind=4) :: LSAME
      INTEGER(long) ::   ILAENV
      real(dop) ::       DLAMCH, DLANSY
!     EXTERNAL           LSAME, ILAENV, DLAMCH, DLANSY
!     ..
!     .. External Subroutines ..
!     EXTERNAL           DLASCL, DORGTR, DSCAL, DSTEQR, DSTERF, DSYTRD,XERBLA
!     ..
!     .. Intrinsic Functions ..
!     INTRINSIC          MAX, SQRT
!     ..
!     .. Executable Statements ..
!
!     Start timing
!     Test the input parameters.
!
      WANTZ = LSAME( JOBZ, 'V' )
      LOWER = LSAME( UPLO, 'L' )
      LQUERY = ( LWORK.EQ.-1 )
!
      INFO = 0
      IF( .NOT.( WANTZ .OR. LSAME( JOBZ, 'N' ) ) ) THEN
         INFO = -1
      ELSE IF( .NOT.( LOWER .OR. LSAME( UPLO, 'U' ) ) ) THEN
         INFO = -2
      ELSE IF( N.LT.0 ) THEN
         INFO = -3
      ELSE IF( LDA.LT.MAX( 1, N ) ) THEN
         INFO = -5
      ELSE IF( LWORK.LT.MAX( 1, 3*N-1 ) .AND. .NOT.LQUERY ) THEN
         INFO = -8
      END IF
!
      IF( INFO.EQ.0 ) THEN
         NB = ILAENV( 1, 'DSYTRD', UPLO, N, -1, -1, -1 )
         LWKOPT = MAX( 1, ( NB+2 )*N )
         WORK( 1 ) = LWKOPT
      END IF
!
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DSYEV ', -INFO )
!     Stot timing
         RETURN
      ELSE IF( LQUERY ) THEN
!     Stot timing
         RETURN
      END IF
!
!     Quick return if possible
!
      IF( N.EQ.0 ) THEN
         WORK( 1 ) = 1
!     Stot timing
         RETURN
      END IF
!
      IF( N.EQ.1 ) THEN
         W( 1 ) = A( 1, 1 )
         WORK( 1 ) = 3
         IF( WANTZ ) A( 1, 1 ) = RONE
!     Stot timing
         RETURN
      END IF
!
!     Get machine constants.
!
      SAFMIN = DLAMCH( 'Safe minimum' )
      EPS = DLAMCH( 'Precision' )
      SMLNUM = SAFMIN / EPS
      BIGNUM = RONE / SMLNUM
      RMIN = SQRT( SMLNUM )
      RMAX = SQRT( BIGNUM )
!
!     Scale matrix to allowable range, if necessary.
!
      ANRM = DLANSY( 'M', UPLO, N, A, LDA, WORK )
      ISCALE = 0
      IF( ANRM.GT.RZERO .AND. ANRM.LT.RMIN ) THEN
         ISCALE = 1
         SIGMA = RMIN / ANRM
      ELSE IF( ANRM.GT.RMAX ) THEN
         ISCALE = 1
         SIGMA = RMAX / ANRM
      END IF
      IF( ISCALE.EQ.1 ) CALL DLASCL( UPLO, 0, 0, RONE, SIGMA, N, N, A, LDA, INFO )
!
!     Call DSYTRD to reduce symmetric matrix to tridiagonal form.
!
      INDE = 1
      INDTAU = INDE + N
      INDWRK = INDTAU + N
      LLWORK = LWORK - INDWRK + 1
      CALL DSYTRD( UPLO, N, A, LDA, W, WORK( INDE ), WORK( INDTAU ), &
                   WORK( INDWRK ), LLWORK, IINFO )
      LOPT = 2*N + nint(WORK( INDWRK ))
!
!     For eigenvalues only, call DSTERF.  For eigenvectors, first call
!     DORGTR to generate the orthogonal matrix, then call DSTEQR.
!
      IF( .NOT.WANTZ ) THEN
         CALL DSTERF( N, W, WORK( INDE ), INFO )
      ELSE
         CALL DORGTR( UPLO, N, A, LDA, WORK( INDTAU ), WORK( INDWRK ), &
              LLWORK, IINFO )
#ifdef MPI
         if (lmpi .and.lmpidsyev.and.lmpistarted) then
            CALL PARDSTEQR(JOBZ,N,W,WORK(INDE),A,LDA,WORK(INDTAU),INFO )
         else
            CALL DSTEQR( JOBZ, N, W, WORK( INDE ), A, LDA, WORK(INDTAU),INFO)
         endif
#else
         CALL DSTEQR( JOBZ, N, W, WORK( INDE ), A, LDA, WORK(INDTAU),INFO)
#endif
      END IF
!
!     If matrix was scaled, then rescale eigenvalues appropriately.
!
      IF( ISCALE.EQ.1 ) THEN
         IF( INFO.EQ.0 ) THEN
            IMAX = N
         ELSE
            IMAX = INFO - 1
         END IF
         CALL DSCAL( IMAX, RONE / SIGMA, W, 1 )
      END IF
!
!     Set WORK(1) to optimal workspace size.
!
      WORK( 1 ) = LWKOPT
!
!     Stop timing

      RETURN
!
!     End of DSYEV
!
      END SUBROUTINE PARDSYEV

!#######################################################################

#ifdef MPI
      SUBROUTINE PARDSTEQR( COMPZ, N, D, E, Z, LDZ, WORK, INFO )

use mpi
use global
use distribmod
use mpidata

use mpidsteqmod
use logdat, only: errormsg

      implicit none

!
!     MRB 09/08
!     PARDSTEQR is the parallel version of the QR-algorithm for the
!     parallel PARDSYEV routine.
!
!     .. Scalar Arguments ..
      CHARACTER, intent(in) ::          COMPZ
      INTEGER, intent(in) ::            LDZ, N
      INTEGER, intent(out) ::           INFO
!     ..
!     .. Array Arguments ..
      real(dop), dimension(*), intent(out)     :: D, E, WORK
      real(dop), dimension(LDZ,*), intent(out) :: Z
!     ..
!     .. Parameters ..
      real(dop), parameter :: RZERO = 0.0_dop
      real(dop), parameter :: RONE = 1.0_dop
      real(dop), parameter :: TWO = 2.0_dop
      real(dop), parameter :: THREE = 3.0_dop 
      INTEGER(long), parameter :: MAXIT = 30
!     ..
!     .. Local Scalars ..
      INTEGER(long) ::  I, ICOMPZ, II, ISCALE, J, JTOT, K, L, L1, LEND1, &
                         LENDM1, LENDP1, LENDSV, LM1, LSV, M, MM, MM1, &
                         NM1, NMAXIT
      real(dop) ::       ANORM, B, C, EPS, EPS2, F, G, P, R, RT1, RT2, &
                         S, SAFMAX, SAFMIN, SSFMAX, SSFMIN, TST
!      integer(long)      fsempost,fsemwait,error,fsyncsemwait
      real(dop), dimension(2)     ::  dbdum
      integer(long), dimension(7) :: mpicommi
!     ..
!     .. External Functions ..
      LOGICAL(kind=4) :: LSAME
      real(dop) ::       DLAMCH, DLANST, DLAPY2
!     EXTERNAL           LSAME, DLAMCH, DLANST, DLAPY2
!     ..
!     .. External Subroutines ..
!     EXTERNAL           DLAE2, DLAEV2, DLARTG, DLASCL, DLASET, DLASR, &
!                        DLASRT, DSWAP, XERBLA
!     ..
!     .. Intrinsic Functions ..
!     INTRINSIC          ABS, MAX, SIGN, SQRT
!     ..
!     .. Executable Statements ..
!
!     Test the input parameters.
!
      INFO = 0
!
      IF( LSAME( COMPZ, 'N' ) ) THEN
         ICOMPZ = 0
      ELSE IF( LSAME( COMPZ, 'V' ) ) THEN
         ICOMPZ = 1
      ELSE IF( LSAME( COMPZ, 'I' ) ) THEN
         ICOMPZ = 2
      ELSE
         ICOMPZ = -1
      END IF
      IF( ICOMPZ.LT.0 ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( ( LDZ.LT.1 ) .OR. ( ICOMPZ.GT.0 .AND. LDZ.LT.MAX( 1, N ) ) ) THEN
         INFO = -6
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'DSTEQR', -INFO )
         RETURN
      END IF
!
!     Quick return if possible
!
      IF( N.EQ.0 ) RETURN
!
      IF( N.EQ.1 ) THEN
         IF( ICOMPZ.EQ.2 ) Z( 1, 1 ) = RONE
         RETURN
      END IF
!
!     Determine the unit roundoff and over/underflow thresholds.
!
      EPS = DLAMCH( 'E' )
      EPS2 = EPS**2
      SAFMIN = DLAMCH( 'S' )
      SAFMAX = RONE / SAFMIN
      SSFMAX = SQRT( SAFMAX ) / THREE
      SSFMIN = SQRT( SAFMIN ) / EPS2
!
!     Compute the eigenvalues and eigenvectors of the tridiagonal
!     matrix.
!
      IF( ICOMPZ.EQ.2 ) CALL DLASET( 'Full', N, N, RZERO, RONE, Z, LDZ )
!
      NMAXIT = N*MAXIT
      JTOT = 0
!
!     Determine where the matrix splits and choose QL or QR iteration
!     for each block, according to whether top or bottom diagonal
!     element is smaller.
!
      L1 = 1
      NM1 = N - 1
!
   10 CONTINUE
      IF( L1.GT.N ) GO TO 160
      IF( L1.GT.1 ) E( L1-1 ) = RZERO
      IF( L1.LE.NM1 ) THEN
         DO 20 M = L1, NM1
            TST = ABS( E( M ) )
            IF( TST.EQ.RZERO ) GO TO 30
            IF( TST.LE.( SQRT( ABS( D( M ) ) )*SQRT( ABS( D( M+1 ) ) ) )*EPS ) THEN
               E( M ) = RZERO
               GO TO 30
            END IF
   20    CONTINUE
      END IF
      M = N
!
   30 CONTINUE
      L = L1
      LSV = L
      LEND1 = M
      LENDSV = LEND1
      L1 = M + 1
      IF( LEND1.EQ.L ) GO TO 10
!
!     Scale submatrix in rows and columns L to LEND1
!
      ANORM = DLANST( 'I', LEND1-L+1, D( L ), E( L ) )
      ISCALE = 0
      IF( ANORM.EQ.RZERO ) GO TO 10
      IF( ANORM.GT.SSFMAX ) THEN
         ISCALE = 1
         CALL DLASCL( 'G', 0, 0, ANORM, SSFMAX, LEND1-L+1, 1, D( L ), N, INFO )
         CALL DLASCL( 'G', 0, 0, ANORM, SSFMAX, LEND1-L, 1, E( L ), N, INFO )
      ELSE IF( ANORM.LT.SSFMIN ) THEN
         ISCALE = 2
         CALL DLASCL( 'G', 0, 0, ANORM, SSFMIN, LEND1-L+1, 1, D( L ), N, INFO )
         CALL DLASCL( 'G', 0, 0, ANORM, SSFMIN, LEND1-L, 1, E( L ), N, INFO )
      END IF
!
!     Choose between QL and QR iteration
!
      IF( ABS( D( LEND1 ) ).LT.ABS( D( L ) ) ) THEN
         LEND1 = LSV
         L = LENDSV
      END IF
!
      if(lmpidsyev.and.lmpi) then
         if((lmpidavrun.and.jtot.eq.0).or.(.not.lmpidavrun)) then
            !if(N.gt.max(2*nmpi*nthread,qrlimit)) then
            if(N.gt.2*nmpi) then
               if(.not.lmpidavrun) then
                  mpitask=7
                  call MPI_Bcast(mpitask,1,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
               endif
               mpicommi(1)=ldz
               mpicommi(2)=l
               mpicommi(3)=lend1
               mpicommi(4)=icompz
               mpicommi(5)=n
               mpicommi(6)=nmaxit
               mpicommi(7)=distribint(10)
               call MPI_Bcast(mpicommi,7,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
               call MPI_Bcast(distribprc,1,MPI_LOGICAL,0,MPI_COMM_WORLD,mpierr)
            endif
         endif
      endif
!
      IF( LEND1.GT.L ) THEN
!
!        QL Iteration
!
         if(lmpi.and.lmpidsyev) then
            !if(N.gt.max(2*nmpi*nthread,qrlimit)) then
            if(N.gt.2*nmpi) then
               dbdum(1)=0.0
               dbdum(2)=0.0
               call mpidsteql(e,d,z,ldz,l,lend1,1,work(l),work(n-1+l), &
                                      dbdum(1),dbdum(2),icompz,n,nmaxit)
            endif
         endif
!
         if (l.ge.lend1) goto 140
!        Look for small subdiagonal element.
!
   40    CONTINUE
         IF( L.NE.LEND1 ) THEN
            LENDM1 = LEND1 - 1
            DO 50 M = L, LENDM1
               TST = ABS( E( M ) )**2
               IF( TST.LE.( EPS2*ABS( D( M ) ) )*ABS( D( M+1 ) )+ SAFMIN )GO TO 60
   50       CONTINUE
         END IF
!
         M = LEND1
!
   60    CONTINUE
         IF( M.LT.LEND1 ) E( M ) = RZERO
         P = D( L )
         IF( M.EQ.L ) GO TO 80
!
!        If remaining matrix is 2-by-2, use DLAE2 or SLAEV2
!        to compute its eigensystem.
!
         IF( M.EQ.L+1 ) THEN
            IF( ICOMPZ.GT.0 ) THEN
               CALL DLAEV2( D( L ), E( L ), D( L+1 ), RT1, RT2, C, S )
               WORK( L ) = C
               WORK( N-1+L ) = S
               CALL DLASR( 'R', 'V', 'B', N, 2, WORK( L ), &
                           WORK( N-1+L ), Z( 1, L ), LDZ )
            ELSE
               CALL DLAE2( D( L ), E( L ), D( L+1 ), RT1, RT2 )
            END IF
            D( L ) = RT1
            D( L+1 ) = RT2
            E( L ) = RZERO
            L = L + 2
            IF( L.LE.LEND1 ) GO TO 40
            GO TO 140
         END IF
!
         IF( JTOT.EQ.NMAXIT ) GO TO 140
         JTOT = JTOT + 1
!
!        Form shift.
!
         G = ( D( L+1 )- P ) / ( TWO*E( L ) )
         R = DLAPY2( G, RONE )
         G = D( M ) - P + ( E( L ) / ( G+SIGN( R, G ) ) )
!
         S = RONE
         C = RONE
         P = RZERO
!
!        Inner loop
!
         MM1 = M - 1
         DO 70 I = MM1, L, -1
            F = S*E( I )
            B = C*E( I )
            CALL DLARTG( G, F, C, S, R )
            IF( I.NE.M-1 ) E( I+1 ) = R
            G = D( I+1 ) - P
            R = ( D( I )-G )*S + TWO*C*B
            P = S*R
            D( I+1 ) = G + P
            G = C*R - B
!
!           If eigenvectors are desired, then save rotations.
!
            IF( ICOMPZ.GT.0 ) THEN
               WORK( I ) = C
               WORK( N-1+I ) = -S
            END IF
!
   70    CONTINUE
!
!        If eigenvectors are desired, then apply saved rotations.
!
         IF( ICOMPZ.GT.0 ) THEN
            MM = M - L + 1
            CALL DLASR( 'R', 'V', 'B', N, MM, WORK( L ), WORK( N-1+L ), &
                        Z( 1, L ), LDZ )
         END IF
!
         D( L ) = D( L ) - P
         E( L ) = G
         GO TO 40
!
!        Eigenvalue found.
!
   80    CONTINUE
         D( L ) = P
!
         L = L + 1
         IF( L.LE.LEND1 ) GO TO 40
         GO TO 140
!
      ELSE
!
!        QR Iteration
!
         if(lmpi.and.lmpidsyev) then
            !if(N.gt.max(2*nmpi*nthread,qrlimit)) then
            if(N.gt.2*nmpi) then
               dbdum(1)=0.0
               dbdum(2)=0.0
               call mpidsteqr(e,d,z,ldz,l,lend1,1,work(l),work(n-1+l), &
                                   dbdum(1),dbdum(2),icompz,n,nmaxit)
            endif
         endif
!        Look for small superdiagonal element.
!
         if (l.le.lend1) goto 140

   90    CONTINUE
         IF( L.NE.LEND1 ) THEN
            LENDP1 = LEND1 + 1
            DO 100 M = L, LENDP1, -1
               TST = ABS( E( M-1 ) )**2
               IF( TST.LE.( EPS2*ABS( D( M ) ) )*ABS( D( M-1 ) )+ SAFMIN )GO TO 110
  100       CONTINUE
         END IF
!
         M = LEND1
!
  110    CONTINUE
         IF( M.GT.LEND1 ) E( M-1 ) = RZERO
         P = D( L )
         IF( M.EQ.L ) GO TO 130
!
!        If remaining matrix is 2-by-2, use DLAE2 or SLAEV2
!        to compute its eigensystem.
!
         IF( M.EQ.L-1 ) THEN
            IF( ICOMPZ.GT.0 ) THEN
               CALL DLAEV2( D( L-1 ), E( L-1 ), D( L ), RT1, RT2, C, S )
               WORK( M ) = C
               WORK( N-1+M ) = S
               CALL DLASR( 'R', 'V', 'F', N, 2, WORK( M ), &
                           WORK( N-1+M ), Z( 1, L-1 ), LDZ )
            ELSE
               CALL DLAE2( D( L-1 ), E( L-1 ), D( L ), RT1, RT2 )
            END IF
            D( L-1 ) = RT1
            D( L ) = RT2
            E( L-1 ) = RZERO
            L = L - 2
            IF( L.GE.LEND1 ) GO TO 90
            GO TO 140
         END IF
!
         IF( JTOT.EQ.NMAXIT ) GO TO 140
         JTOT = JTOT + 1
!
!        Form shift.
!
         G = ( D( L-1 )-P ) / ( TWO*E( L-1 ) )
         R = DLAPY2( G, RONE )
         G = D( M ) - P + ( E( L-1 ) / ( G+SIGN( R, G ) ) )
!
         S = RONE
         C = RONE
         P = RZERO
!
!        Inner loop
!
         LM1 = L - 1
         DO 120 I = M, LM1
            F = S*E( I )
            B = C*E( I )
            CALL DLARTG( G, F, C, S, R )
            IF( I.NE.M ) E( I-1 ) = R
            G = D( I ) - P
            R = ( D( I+1 )-G )*S + TWO*C*B
            P = S*R
            D( I ) = G + P
            G = C*R - B

!
!           If eigenvectors are desired, then save rotations.
!
            IF( ICOMPZ.GT.0 ) THEN
               WORK( I ) = C
               WORK( N-1+I ) = S
            END IF
!
  120    CONTINUE
!
!        If eigenvectors are desired, then apply saved rotations.
!
         IF( ICOMPZ.GT.0 ) THEN
            MM = L - M + 1
            CALL DLASR( 'R', 'V', 'F', N, MM, WORK( M ), WORK( N-1+M ), &
                        Z( 1, M ), LDZ )
         END IF
!
         D( L ) = D( L ) - P
         E( LM1 ) = G
         GO TO 90
!
!        Eigenvalue found.
!
  130    CONTINUE
         D( L ) = P
!
         L = L - 1
         IF( L.GE.LEND1 ) GO TO 90
         GO TO 140
!
      END IF
!
!     Undo scaling if necessary
!
  140 CONTINUE
      IF( ISCALE.EQ.1 ) THEN
         CALL DLASCL( 'G', 0, 0, SSFMAX, ANORM, LENDSV-LSV+1, 1, &
                      D( LSV ), N, INFO )
         CALL DLASCL( 'G', 0, 0, SSFMAX, ANORM, LENDSV-LSV, 1, E( LSV ), &
                      N, INFO )
      ELSE IF( ISCALE.EQ.2 ) THEN
         CALL DLASCL( 'G', 0, 0, SSFMIN, ANORM, LENDSV-LSV+1, 1, &
                      D( LSV ), N, INFO )
         CALL DLASCL( 'G', 0, 0, SSFMIN, ANORM, LENDSV-LSV, 1, E( LSV ), &
                      N, INFO )
      END IF
!
!     Check for no convergence to an eigenvalue after a total
!     of N*MAXIT iterations.
!
      IF( JTOT.LT.NMAXIT ) GO TO 10

      DO 150 I = 1, N - 1
         IF( E( I ).NE.RZERO ) INFO = INFO + 1
  150 CONTINUE
      GO TO 190
!
!     Order eigenvalues and eigenvectors.
!
  160 CONTINUE
      IF( ICOMPZ.EQ.0 ) THEN
!
!        Use Quick Sort
!
         CALL DLASRT( 'I', N, D, INFO )
!
      ELSE
!
!        Use Selection Sort to minimize swaps of eigenvectors
!
         DO 180 II = 2, N
            I = II - 1
            K = I
            P = D( I )
            DO 170 J = II, N
               IF( D( J ).LT.P ) THEN
                  K = J
                  P = D( J )
               END IF
  170       CONTINUE
            IF( K.NE.I ) THEN
               D( K ) = D( I )
               D( I ) = P
               CALL DSWAP( N, Z( 1, I ), 1, Z( 1, K ), 1 )
            END IF
  180    CONTINUE
      END IF
!

  190 CONTINUE

      RETURN
!
!     End of DSTEQR
!
      END SUBROUTINE PARDSTEQR
#endif

      end module pardsyevmod
