!#######################################################################
!
!                      MCTDH module WRINTEGRATOR
!
! various small routines used to output step information 
!
! subroutines :
!    writestep: Writes the stepsize and error of the current integration
!               step to a file.
!
! wrtrlx              
! wrtrlxdav        
! wrtrlxbdav       
! sortovlps        
! wrtrlxcdav       
!
!#######################################################################

      module wrintegrat

use decimal, only: long, dop
use keyunitsmod
use logdat

      implicit none
      private
      public :: writestep,writestepdav,writestepcdav,writestepbdav,&
                wrtrlx,wrtrlxdav,wrtrlxbdav,wrtrlxcdav,writestepcheb
      contains

! **********************************************************************
! *                                                                    *
! *                          SUBROUTINE WRITESTEP                      *
! *                                                                    *
! * Writes for all integrators the stepsize and error of the current   *
! * integration step to a file. Two integers having an integrator-     *
! * dependent meaning (typically step and order) are also put out.     *
! *                                                                    *
! * Input parameters:                                                  *
! *   Step:     Typically a counter for the number of steps.           *
! *   Order:    Typically the current integration order.               *
! *   Stepsize: Size of the current integration step (in a. u.).       *
! *   Error:    Error of the current integration step.                 *
! *                                                                    *
! * Output parameters:                                                 *
! *   none                                                             *
! *                                                                    *
! * V6.0 MB                                                            *
! *                                                                    *
! **********************************************************************

      subroutine writestep (step,order,stepsize,error,time,check)

use global
use datenmod
use channels


      Implicit None

      integer(long), intent(in) :: step,order
      real(dop), intent(in)     :: stepsize,error,time
      real(dop), parameter      :: minstep = 1.0d-8
      logical,intent(in),optional::check
      logical::ll

! --- WRITE STEPSIZE AND ERROR ---

      if (lintstep) Then
         write(istp,'(I7,I13,X,2E19.6E2,F18.3)') &
              step, order, stepsize/fs, error, time/fs
         call myflush(istp)
      endIf

! check is a hack to stop this bombing with zvode
ll=.true.
if (present(check)) ll=check

! stepsize is exactly 0.0 in SIL diagonalisation
      if ((abs(stepsize)<minstep).and.(abs(stepsize)>0).and.ll) then
         routine = 'Writestep (wrintegrat.f90)'
         write(message,'(a,f12.8)') &
            'Integration terminated. Integrator step too small :',stepsize
         call errormsg
      endif


      return
      end subroutine writestep

! **********************************************************************

      subroutine writestepdav (step,order,e,beta,error,q,time)

use global
use datenmod
use channels

      Implicit None

      Real(dop), intent(in)     :: e,beta,error,time
      Integer(long), intent(in) :: step,order,q

! --- WRITE STEPSIZE AND ERROR ---

      if (lintstep) then
         if(time.le.9999.999_dop*fs) then
            Write(istp,'(I4,I5,1pE21.12e2,3pF10.2,0pE16.6e2,I6,F12.3)') &
              step, order, e, beta, error, q, time/fs
         else
            Write(istp,'(I4,I5,1pE21.12e2,3pF10.2,0pE16.6e2,I6,F12.1)')  &
              step, order, e, beta, error, q, time/fs
         endif
         Call myflush(istp)
      endIf

      return
      end subroutine writestepdav

! **********************************************************************

      Subroutine WriteStepCDav (Step,Order,E,beta,Error,Q,Time)

use global
use datenmod
use channels

      Implicit None

      Real(dop), intent(in)     :: beta,error,time
      Complex(dop), intent(in)     :: e
      Integer(long), intent(in) :: step,order,q

! --- WRITE STEPSIZE AND ERROR ---

      If (LIntStep) Then
         if(Time.le.9999.999d0*fs) then
         Write(IStp,'(I4,I5,1pE18.9e2,1pE14.6e2,3pF9.2,0pE14.5e2,I5,F11.3)') &
             Step, Order, E, beta, Error, Q, Time/Fs
         else
         Write(IStp,'(I4,I5,1pE18.9e2,1pE14.6e2,3pF9.2,0pE14.5e2,I5,F11.1)') &
             Step, Order, E, beta, Error, Q, Time/Fs
         endif
         Call MyFlush(IStp)
      EndIf

      Return
      End subroutine writestepcdav

! **********************************************************************

      Subroutine WriteStepBDav (Step,Order,E,beta,Error,Q,QQ,Time)

use global
use datenmod
use channels

      Implicit None

      Real(dop), intent(in)     :: e,beta,error,time
      Integer(long), intent(in) :: step,order,q,qq

! --- WRITE STEPSIZE AND ERROR ---

      If (LIntStep) Then
         if(Time.le.9999.999d0*fs) then
         Write(IStp,'(I4,I5,1pE21.12e2,3pF10.2,0pE16.6e2,2I5,F11.3)')&
              Step, Order, E, beta, Error, Q,  QQ, Time/Fs
         else
         Write(IStp,'(I4,I5,1pE21.12e2,3pF10.2,0pE16.6e2,2I5,F11.1)')&
              Step, Order, E, beta, Error, Q,  QQ, Time/Fs
         endif
         Call MyFlush(IStp)
      EndIf

      Return
      End Subroutine WriteStepBDav

! **********************************************************************
! *                                                                    *
! *                        SUBROUTINE WRTRLX                           *
! *                                                                    *
! * Writes some diagonalization-information to the file rlx_info.      *
! * or to the file meigem_info. (Lanczos-Diagonalisation).             *
! * Called from: sildiag
! *                                                                    *
! * Input parameters:                                                  *
! *   q     : No. of eigenvector (counting from zero).                 *
! *   order : Dimension of the matrix diadonalized.                    *
! *   mxvc  : Overlap with previous vector.                            *
! *   beta  : First off diagonal element of the tri-diagonal Lanczos   *
! *           matrix.                                                  *
! *   ew    : Vector containig the eigenvalues.                        *
! *   psi   : A-Vector.                                                *
! *                                                                    *
! * 05/01 HDM                                                          *
! *                                                                    *
! **********************************************************************

      subroutine wrtrlx(q,order,mxvc,beta,time,ew,er,ovl, &
                        firstcall,stdform)

use global
use psidef
use channels

      implicit none

      integer(long)                             :: p,nout
      integer(long), intent(in)                 :: q,order
      real(dop)                                 :: sum
      real(dop), intent(in)                     :: mxvc,beta,time
      real(dop), dimension(*), intent(in)       :: ew,er,ovl
      logical(kind=4), intent(in)               :: firstcall,stdform

      if(firstcall .and. stdform) then
         write(irlx,'(a,/2a,/a,/2a,/2a)') &
        'This file contains information on the SIL improved relaxation.'&
         ,'Q = number of the eigenvector of the Lanczos matrix ',&
          '(counting from zero).' ,&
          'N = Dimension of the Lanczos matrix. ',&
          'ovl = overlap squared of the Q-th Lanczos eigenvector ',&
          'with the starting vector.',&
          'beta = First off-diagonal element of the tri-diagonal ',&
          'Lancos matrix.'  
      endif

      write(irlx,'(a)')
      if( stdform  .and. mxvc .lt. 0.66_dop ) then
        write(irlx,'(2a)') '-*+#-*+#-*+#-*+#-*+#-*+#-*+#-*', &
        ' small overlap *+#-*+#-*+#-*+#-*+#-*+#-*+#-*+#-*+#-'
        write(ilog,'(a,/a,f8.4,a,f8.2,/a,/a,/a,/a)') &
         'WARNING WARNING WARNING WARNING WARNING WARNING WARNING',&
         'The overlap between two time steps is small :', &
          mxvc,' ,  Time =', time/fs,&
         'You are probably converging to another state. ',&
         'Try using a smaller update time, a larger Lanczos space,',&
         'a higher accuracy, more SPF''s, or a better initial state.',& 
         'See rlx_info file for details.'
        If(FirstCall) write(ilog,'(2a,/a)') 'As the small overlap ',&
         'occurs at the initial step, it just tells you that your ',&
         'initial wavefunction is poor. The calculation may be OK.'
        write(ilog,'(a,/a)') ' ', '.'
        call myflush(ilog)
      endif

      write(irlx,'("Q =", i3, ",  N =", i5, ",  ovl =", f12.9,",  beta =",1pe10.3, ",   T =",0pf10.2," fs")') &
         q,order+1, mxvc,beta,time/fs
      write(irlx,'("E =", 1pe20.13," a.u., ",0pf13.4," cm-1, ",0pf13.9," eV")') &
         ew(q+1), ew(q+1)*2.194746d+5,ew(q+1)*ev
 
      nout = min(order+1,max(Q+9,48))
      if(.not.StdForm) nout = min(order+1,360)
      write(irlx,'(a)') 'Eigenvalues[eV] :'
      write(irlx,'(6f13.8)') (ew(p)*ev ,p=1,nout)
      if(FirstCall) then
         sum=0.0_dop
         do p=1,order+1
            sum=sum+er(p)
         enddo
         if(sum.lt.1.0e-12_dop) then
            write(irlx,'(a)') 'Fully converged'
         else
            write(irlx,'(a)') 'Error Estimates[eV] :'
            write(irlx,'(1p6e13.3)') (er(p) ,p=1,nout)
         endif
         write(irlx,'(a)') 'Overlaps * 1000 : '
         write(irlx,'(6f13.7)') (1.0e3_dop*ovl(p),p=1,nout)
      else
          write(irlx,'(a,e12.3)') 'Error Estimate[eV] :',er(q+1)
      endif

      call myflush(irlx)

      return
      end subroutine wrtrlx


! **********************************************************************
! *                                                                    *
! *                        SUBROUTINE WRTRLXDAV                        *
! *                                                                    *
! * Writes some diagonalization-information to the file rlx_info.      *
! * propagation.                                                       *
! *                                                                    *
! * Input parameters:                                                  *
! *   q     : No. of eigenvector (counting from zero).                 *
! *   order : Dimension of the matrix diagonalized.                    *
! *   ovl   : Overlap with start vector.                               *
! *   beta  : Overlap with previous vector                             *
! *   eval  : Eigenvalues.                                             *
! *   eigenval : Vector containig the eigenvalues. (FirstCall).        *
! *   ovlps : Vector containing the overlaps. (only for FirstCall).    *    
! *                                                                    *
! * 06/03 HDM                                                          *
! *                                                                    *
! **********************************************************************


      subroutine wrtrlxdav(time,order,q,q1,eval,eval1,beta,beta1,ovl,&
                           ovl1,eigenval,ovlps,firstcall,maxord)

use global
use psidef
use datenmod
use runpropmod
use channels
  
      implicit none

      integer(long)                                 :: i,n
      integer(long), intent(in)                     :: q,q1,order,maxord
      integer(long), dimension(12)                  :: qlist
      real(dop)                                     :: sum,ermin,ermax
      real(dop), save                               :: eunit
      real(dop), intent(in)                         :: beta,beta1,time,ovl1
      real(dop), intent(out)                        :: ovl
      real(dop), dimension(0:maxord), intent(inout) :: ovlps
      real(dop)                                     :: ew
      real(dop), dimension(0:2), save               :: olde
      real(dop), intent(in)                         :: eval,eval1
      real(dop), dimension(0:maxord), intent(in)    :: eigenval
      logical(kind=4), intent(in)                   :: firstcall
      character(len=1)                              :: s1, s2, s3
      character(len=8)                              :: st
      character(len=12)                             :: sb
      character(len=20)                             :: se
      integer(long), external                       :: slen
      logical(kind=4), save                         :: reset=.true.
      integer(long), save                           :: oldorder = 0

      if(firstcall .and. lcont .and. q.eq.-2) then
         olde(kskip)=eval
         eunit = 1.0_dop
         call keyeunits(eunit,rlxunit,i)
         eunit = 1.0_dop/eunit
         write(irlx,'(a/)') ' ---------   CONTINUATION   ------------'
         return
      endif
      if(FirstCall .and. lcont .and. order.eq.0 .and. q1.eq.0) return

      if(FirstCall) then
         if(order.eq.0) then
            olde(kskip)=eval
            write(irlx,'(a,/2a,/a,/a,/2a,/a)') &
           'This file contains information on the improved relaxation.', &
           'Q = number of the eigenvector of the Davidson matrix ', &
           '(counting from zero).' , &
           'order+1 = Dimension of the Davidson matrix. ', &
           'If order=maxorder, this will be noted by a "*".', &
           'beta = overlap squared of the Q-th Davidson eigenvector ', &
           'with the start vector.', &
           'To view this file type "rdrlx" in name directory.'

            eunit = 1.0_dop
            call keyeunits(eunit,rlxunit,i)
            se = rlxunit(1:slen(rlxunit))//'               '
            if(i.ne.0) then
               routine='wrtrlxdav (propwf/integrat.F)'
               message='Unknown energy unit: '//rlxunit(1:slen(rlxunit))
               call errormsg
            endif

            if(rlxunit .eq. 'au') then
               if(rlxshift.ne.0.d0 ) then
                  write(irlx,'(/a,f11.6,a,1x,g24.10,a16,9x,a)') &
                     ' E-shift =', rlxshift*eunit,' au,  ', &
                     rlxshift,se,'%'
               endif
               eunit = 1.d0/eunit
               if(rlxemin.gt.-1.d99 .and. mod(rlaxnum,1000).eq.930) then
                  ermin=rlxemin*eunit-rlxshift
                  write(irlx,'(1x,a,f11.6,2a,g15.8,a16,9x,a)') &
                        'rlxEmin =',rlxemin,' au,    ', &
                        'shifted:', ermin,se,'%'
               endif
               if(rlxemax.lt. 1.d99 .and. mod(rlaxnum,1000).eq.930) then
                  ermax=rlxemax*eunit-rlxshift
                  write(irlx,'(1x,a,f11.6,2a,g15.8,a16,9x,a)') &
                        'rlxEmax =',rlxemax,' au,    ', &
                        'shifted:', ermax,se,'%'
               endif
            else

               if(rlxshift.ne.0.0_dop ) then
                  write(irlx,'(/a,f11.6,a,1x,g24.10,a16,9x,a)') &
                     ' E-shift =', rlxshift*ev*eunit,' eV,  ', &
                     rlxshift,se,'%'
               endif
               eunit = 1.0_dop/eunit
               if(rlxemin.gt.-1.0e99_dop .and. mod(rlaxnum,1000).eq.930) then
                  ermin=rlxemin*eunit-rlxshift
                  write(irlx,'(1x,a,f11.6,2a,g15.8,a16,9x,a)') &
                        'rlxEmin =',rlxemin*ev,' eV,    ', &
                        'shifted:', ermin,se,'%'
               endif
               if(rlxemax.lt. 1.0e99_dop .and. mod(rlaxnum,1000).eq.930) then
                  ermax=rlxemax*eunit-rlxshift
                  write(irlx,'(1x,a,f11.6,2a,g15.8,a16,9x,a)') &
                        'rlxEmax =',rlxemax*ev,' eV,    ', &
                        'shifted:', ermax,se,'%'
               endif
            endif
            write(irlx,'(/77x,a)') '%'
            se ='Energy['//rlxunit(1:slen(rlxunit))//']        '
            write(irlx,'(/a,5x,a16,a)')'   time  order   q   beta*1000', &
                    se, '   ovl*1000    Delta-E    %'
            if(mod(rlaxnum,1000).ne.930) ovl = 0.0_dop
         elseif(q.eq.q1 .and. .not.lcont) then
            write(irlx,'(/a,/90(8f10.6/))') &
                 ' Eigenvalues [eV]   (first diagonalisation)', &
                 (eigenval(i)*ev,i=0,order)
            write(irlx,'(/a,/90(8f10.3/))') &
                 ' Overlaps * 1000   (first diagonalisation)', &
                 (1.0e3_dop*ovlps(i),i=0,order)
            call sortovlps(ovlps,qlist,order)
            if( rlxunit .ne. 'ev'  .or. abs(rlxshift) .gt. 1.0e-22_dop) then 
               n = min(6,order+1)
               write(irlx,'(/4a)') &
                    "  Q's / Largest overlaps*1000 / Energies[eV] /", &
                    " Shifted energies[",rlxunit(1:slen(rlxunit)),']'
               write(irlx,'(6(i10,3x))') (qlist(i),i=1,n)
               write(irlx,'(6f13.3)') (-ovlps(qlist(i))*1.0e3_dop,i=1,n)
               write(irlx,'(6f13.7)') (eigenval(qlist(i))*ev,i=1,n)
               write(irlx,'(2x,6g13.5e1)') &
                          (eigenval(qlist(i))*eunit-rlxshift,i=1,n)
               if( -ovlps(qlist(7)).gt.0.01_dop .and. order.gt.6)  then
                  n = min(12,order+1)
                  write(irlx,'(/6(i10,3x))') (qlist(i),i=7,n)
                  write(irlx,'(6f13.3)') (-ovlps(qlist(i))*1.0e3_dop,i=7,n)
                  write(irlx,'(6f13.7)') (eigenval(qlist(i))*ev,i=7,n)
                  write(irlx,'(2x,6g13.5e1)') & 
                             (eigenval(qlist(i))*eunit-rlxshift,i=7,n)
               endif
            else
               n = min(6,order+1)
               write(irlx,'(/a)') &
                    "  Q's / Largest overlaps*1000 / Energies[eV] "
               write(irlx,'(6(i10,3x))') (qlist(i),i=1,n)
               write(irlx,'(6f13.3)') (-ovlps(qlist(i))*1.0e3_dop,i=1,n)
               write(irlx,'(6f13.7)') (eigenval(qlist(i))*ev,i=1,n)
               if( -ovlps(qlist(7)).gt.0.01_dop .and. order.gt.6)  then
                  n = min(12,order+1)
                  write(irlx,'(/6(i10,3x))') (qlist(i),i=7,n)
                  write(irlx,'(6f13.3)') (-ovlps(qlist(i))*1.0e3_dop,i=7,n)
                  write(irlx,'(6f13.7)') (eigenval(qlist(i))*ev,i=7,n)
                endif
            endif 
            write(irlx,*)
         elseif(q.eq.-1) then
           write(irlx,'(/a,/90(8f10.6/))') &
                 ' Eigenvalues [eV]   (last diagonalisation)', &
                 (eigenval(i)*ev,i=0,oldorder)
           if(mod(rlaxnum,1000).eq.930) then 
             sum = 0.0_dop
             do i = 0, oldorder
                sum = sum + ovlps(i)
             enddo
             write(irlx,'(/a,/90(8f10.3/))') &
                  ' Overlaps * 1000  (last diagonalisation)', &
                  (1000*ovlps(i),i=0,oldorder)
             write(irlx,'(a,f11.7,2a)') ' Sum over all overlaps :', &
                   sum,' (<1, as orbital sets have changed)'
             call sortovlps(ovlps,qlist,oldorder)
             if( rlxunit.ne.'ev' .or. abs(rlxshift).gt.1.0e-22_dop) then 
               n = min(6,oldorder+1)
               write(irlx,'(/4a)') &
                    "  Q's / Largest overlaps*1000 / Energies[eV] /", &
                    " Shifted energies[",rlxunit(1:slen(rlxunit)),']'
               write(irlx,'(6(i10,3x))') (qlist(i),i=1,n)
               write(irlx,'(6f13.3)') (-ovlps(qlist(i))*1.0e3_dop,i=1,n)
               write(irlx,'(6f13.7)') (eigenval(qlist(i))*ev,i=1,n)
               write(irlx,'(2x,6g13.5e1)') &
                          (eigenval(qlist(i))*eunit-rlxshift,i=1,n)
               if( -ovlps(qlist(7)).gt.0.01_dop .and. oldorder.gt.6) then
                  n = min(12,oldorder+1)
                  write(irlx,'(/6(i10,3x))') (qlist(i),i=7,n)
                  write(irlx,'(6f13.3)') (-ovlps(qlist(i))*1.0e3_dop,i=7,n)
                  write(irlx,'(6f13.7)') (eigenval(qlist(i))*ev,i=7,n)
                  write(irlx,'(2x,6g13.5e1)') &
                             (eigenval(qlist(i))*eunit-rlxshift,i=7,n)
               endif
             else
               n = min(6,oldorder+1)
               write(irlx,'(/a)') &
                    "  Q's / Largest overlaps*1000 / Energies[eV] "
               write(irlx,'(6(i10,3x))') (qlist(i),i=1,n)
               write(irlx,'(6f13.3)') (-ovlps(qlist(i))*1.0e3_dop,i=1,n)
               write(irlx,'(6f13.7)') (eigenval(qlist(i))*ev,i=1,n)
               if( -ovlps(qlist(7)).gt.0.01_dop .and. oldorder.gt.6) then
                  n = min(12,oldorder+1)
                  write(irlx,'(/6(i10,3x))') (qlist(i),i=7,n)
                  write(irlx,'(6f13.3)') (-ovlps(qlist(i))*1.0e3_dop,i=7,n)
                  write(irlx,'(6f13.7)') (eigenval(qlist(i))*ev,i=7,n)
               endif
             endif
           endif
           write(irlx,*)
           return
         endif
      endif

      s1 = ' '
      s2 = ' '
      s3 = ' '
      if(order.ge.maxord)   s1 = '*'
      if(beta.le.0.66667_dop) s2 = '*'
      if(ovl.gt.0.0_dop .and. ovl.le.0.5_dop)     s3 = '*'
      if(abs(time) .lt. tiny .and. order.eq.0) then
         write(st,'(f8.1)') -out2/fs
      elseif(time/fs .lt. 99.99_dop ) then
         write(st,'(f8.3)') time/fs
      elseif(time/fs .lt. 999.9_dop ) then
         write(st,'(f8.2)') time/fs
      elseif(time/fs .lt. 9999.0_dop ) then
         write(st,'(f8.1)') time/fs
      else
         write(st,'(f8.0)') time/fs
      endif

      if(q.ne.q1) then
         if(reset) then
            reset = .false.
            write(irlx,'(/2a)') '  time   order   q  q1    ', &
               'beta   beta1    ovl     ovl1    E           E1     ;'
         endif
         write(irlx,'(a8,i5,a1,2i4,3p4f8.2,0p2f10.6,a)') &
         st,order,s1,q,q1,beta,beta1,ovl,ovl1,eval*ev,eval1*ev,'   ;'
         return
      endif

      if(beta .gt. 0.99999998_dop ) then
         write(sb,'(e12.3)') 1.0_dop-beta
      else
         write(sb,'(f12.5)') beta*1.0e3_dop
      endif
      ew = eval*eunit - rlxshift
      if(abs(ew) .lt. 100.0_dop ) then
         write(se,'(f17.12)') ew
         se(17:20) = ' '//se(15:17)
         se(13:16) = ' '//se(12:14)
         se( 9:12) = ' '//se( 9:11)
      elseif(abs(ew) .lt. 1.0e5_dop ) then
         write(se,'(f18.9)') ew
         se(17:20) = ' '//se(16:18)
         se(13:16) = ' '//se(13:15)
      else
         write(se,'(f18.6)') ew
         se(17:20) = ' '//se(16:18)
         se( 9:16) = ' '//se(9:15)
      endif

      write(irlx,'(a8,i5,a1,i4,a12,a1,a20,3pf11.4,a1,1pe11.3,a)')&
          st,order,s1,q,sb,s2,se,ovl,s3,(eval-olde(kskip))*eunit,'   %'

      call myflush(irlx)
      e1stop    = abs(eval-olde(kskip))
      olde(kskip) = eval
      oldorder  = order

      return
      end subroutine wrtrlxdav

! **********************************************************************
! *                                                                    *
! *                        SUBROUTINE WRTRLXBDAV                       *
! *                                                                    *
! * Writes some diagonalization-information to the file rlx_info.      *
! * Block-Davidson                                                     *
! *                                                                    *
! * Input parameters:                                                  *
! *   q     : No. of eigenvector (counting from zero).                 *
! *   qq    : Running No. of eigenvector (counting from zero).         *
! *   order : Dimension of the matrix diagonalized.                    *
! *   beta  : Overlap with previous vector                             *
! *   eval  : Eigenvalues.                                             *
! *   eigenval : Vector containig the eigenvalues. (FirstCall).        *
! *   ovlps : Vector containing the overlaps. (only for FirstCall).    *
! *                                                                    *
! * 09/07 HDM                                                          *
! *                                                                    *
! **********************************************************************

      subroutine wrtrlxbdav(time,order,qq,q,eval,beta,oldenergy,&
                            conv,EigenVal,ovlps,FirstCall,maxord,bdim)

use global
use psidef
use datenmod
use runpropmod
use channels

      implicit none

      integer(long)                                 :: i,n,conv,bdim,ordr
      integer(long), intent(in)                     :: q,qq,order,maxord
      integer(long), dimension(12)                  :: qlist
      real(dop)                                     :: ermin,ermax
      real(dop), save                               :: eunit
      real(dop), intent(in)                         :: beta,time
      real(dop), dimension(0:maxord), intent(inout) :: ovlps
      real(dop)                                     :: ew
      real(dop), intent(in)                         :: oldenergy
      real(dop), intent(in)                         :: eval
      real(dop), dimension(0:maxord), intent(in)    :: eigenval
      logical(kind=4), intent(in)                   :: firstcall
      character(len=1)                              :: s1, s2
      character(len=8)                              :: st
      character(len=12)                             :: sb
      character(len=20)                             :: se
      integer(long), external                       :: slen
      integer(long), save                           :: oldorder = 0

      if(FirstCall .and. lcont .and. q.eq.-2) then
         eunit = 1.d0
         call keyeunits(eunit,rlxunit,i)
         eunit = 1.d0/eunit
         write(irlx,'(a/)') ' ---------   CONTINUATION   ------------'
         return
      endif
      if(FirstCall.and.lcont.and. order.lt.bdim .and. qq.lt.bdim) return

      if(FirstCall) then
         if(order.eq.0) then
            write(irlx,'(a,/2a,/a,/a,/2a,/a)')  &
           'This file contains information on the improved relaxation.', &
           'Q = number of the eigenvector of the Davidson matrix ',      &
           '(counting from zero).' ,                                     &
           'order+1 = Dimension of the Davidson matrix. ',               &
           'If order=maxorder, this will be noted by a "*".',            &
           'beta = overlap squared of the Q-th Davidson eigenvector ',   &
           'with the start vector.',                                     &
           'To view this file type "rdrlx" in name directory.'

            eunit = 1.d0
            call keyeunits(eunit,rlxunit,i)
            se = rlxunit(1:slen(rlxunit))//'               '
            if(i.ne.0) then
               routine='wrtrlxdav (propwf/integrat.F)'
               message='Unknown energy unit: '//rlxunit(1:slen(rlxunit))
               call errormsg
            endif
            if(rlxunit .eq. 'au') then
               if(rlxshift.ne.0.d0 ) then
                  write(irlx,'(/a,f11.6,a,1x,g24.10,a16,9x,a)') &
                     ' E-shift =', rlxshift*eunit,' au,  ', &
                     rlxshift,se,'%'
               endif
               eunit = 1.d0/eunit
               if(rlxemin.gt.-1.d99) then
                  ermin=rlxemin*eunit-rlxshift
                  write(irlx,'(1x,a,f11.6,2a,g15.8,a16,9x,a)') &
                        'rlxEmin =',rlxemin,' au,    ', &
                        'shifted:', ermin,se,'%'
               endif
               if(rlxemax.lt. 1.d99) then
                  ermax=rlxemax*eunit-rlxshift
                  write(irlx,'(1x,a,f11.6,2a,g15.8,a16,9x,a)') &
                        'rlxEmax =',rlxemax,' au,    ', &
                        'shifted:', ermax,se,'%'
               endif
            else

               if(rlxshift.ne.0.d0 ) then
                  write(irlx,'(/a,f11.6,a,1x,g24.10,a16,9x,a)') &
                     ' E-shift =', rlxshift*ev*eunit,' eV,  ',  &
                     rlxshift,se,'%'
               endif
               eunit = 1.d0/eunit
               if(rlxemin.gt.-1.d99) then
                  ermin=rlxemin*eunit-rlxshift
                  write(irlx,'(1x,a,f11.6,2a,g15.8,a16,9x,a)') &
                        'rlxEmin =',rlxemin*ev,' eV,    ',     &
                        'shifted:', ermin,se,'%'
               endif
               if(rlxemax.lt. 1.d99) then
                  ermax=rlxemax*eunit-rlxshift
                  write(irlx,'(1x,a,f11.6,2a,g15.8,a16,9x,a)') &
                        'rlxEmax =',rlxemax*ev,' eV,    ',     &
                        'shifted:', ermax,se,'%'
               endif
            endif
            write(irlx,'(/77x,a)') '%'
            se ='Energy['//rlxunit(1:slen(rlxunit))//']        '
            write(irlx,'(/a,5x,a16,a)') &
            '   time  order   QQ    Q   beta*1000',se,'    Delta-E    %'
            write(irlx,'(1x,70a1,1x,a1)') ('-',i=1,70),'%' 

         elseif(qq.eq.0 .and. .not.lcont) then
            write(irlx,'(/a,/90(8f10.6/))') &
                 ' Eigenvalues [eV]   (first diagonalisation)', &
                 (eigenval(i)*ev,i=0,order)
            write(irlx,'(/a,/90(8f10.3/))') &
                 ' Overlaps * 1000   (first diagonalisation)', &
                 (1.d3*ovlps(i),i=0,order)
            call sortovlps(ovlps,qlist,order)
            if( rlxunit .ne. 'ev'  .or. abs(rlxshift) .gt. 1.d-22) then
               n = min(6,order+1)
               write(irlx,'(/4a)')  &
                    "  Q's / Largest overlaps*1000 / Energies[eV] /", &
                    " Shifted energies[",rlxunit(1:slen(rlxunit)),']'
               write(irlx,'(6(i10,3x))') (qlist(i),i=1,n)
               write(irlx,'(6f13.3)') (-ovlps(qlist(i))*1.d3,i=1,n)
               write(irlx,'(6f13.7)') (eigenval(qlist(i))*ev,i=1,n)
               write(irlx,'(2x,6g13.5e1)') &
                          (eigenval(qlist(i))*eunit-rlxshift,i=1,n)
               if( -ovlps(qlist(7)).gt.0.01d0 .and. order.gt.6)  then
                  n = min(12,order+1)
                  write(irlx,'(/6(i10,3x))') (qlist(i),i=7,n)
                  write(irlx,'(6f13.3)') (-ovlps(qlist(i))*1.d3,i=7,n)
                  write(irlx,'(6f13.7)') (eigenval(qlist(i))*ev,i=7,n)
                  write(irlx,'(2x,6g13.5e1)') &
                             (eigenval(qlist(i))*eunit-rlxshift,i=7,n)
               endif
            else
               n = min(6,order+1)
               write(irlx,'(/a)') &
                    "  Q's / Largest overlaps*1000 / Energies[eV] "
               write(irlx,'(6(i10,3x))') (qlist(i),i=1,n)
               write(irlx,'(6f13.3)') (-ovlps(qlist(i))*1.d3,i=1,n)
               write(irlx,'(6f13.7)') (eigenval(qlist(i))*ev,i=1,n)
               if( -ovlps(qlist(7)).gt.0.01d0 .and. order.gt.6)  then
                  n = min(12,order+1)
                  write(irlx,'(/6(i10,3x))') (qlist(i),i=7,n)
                  write(irlx,'(6f13.3)') (-ovlps(qlist(i))*1.d3,i=7,n)
                  write(irlx,'(6f13.7)') (eigenval(qlist(i))*ev,i=7,n)
                endif
            endif
            write(irlx,*)

         elseif(q.eq.-1) then
           write(irlx,'(/a,/90(8f10.6/))') &
                 ' Eigenvalues [eV]   (last diagonalisation)', &
                 (eigenval(i)*ev,i=0,oldorder)
           write(irlx,*)
           return
         endif
      endif    ! FirstCall

      s1 = ' '
      s2 = ' '
!      if(order.gt.maxord-bdim)   s1 = '*'
      ordr=order
      if(conv .ne. 0) ordr=abs(conv) 
      if(conv .eq. 0)         s1 = '*'
      if(beta.le.0.666667d0)  s2 = '*'
      if(abs(time) .lt. tiny .and. order.lt.bdim) then
         write(st,'(f8.1)') -out2/fs
      elseif(time/fs .lt. 99.99d0 ) then
         write(st,'(f8.3)') time/fs
      elseif(time/fs .lt. 999.9d0 ) then
         write(st,'(f8.2)') time/fs
      elseif(time/fs .lt. 9999.d0 ) then
         write(st,'(f8.1)') time/fs
      else
         write(st,'(f8.0)') time/fs
      endif


      if(beta .gt. 0.99999998d0 ) then
         write(sb,'(e12.3)') 1.d0-beta
      else
         write(sb,'(f12.5)') beta*1.d3
      endif
      ew = eval*eunit - rlxshift
      if(abs(ew) .lt. 100.d0 ) then
         write(se,'(f17.12)') ew
         se(17:20) = ' '//se(15:17)
         se(13:16) = ' '//se(12:14)
         se( 9:12) = ' '//se( 9:11)
      elseif(abs(ew) .lt. 1.d5 ) then
         write(se,'(f18.9)') ew
         se(17:20) = ' '//se(16:18)
         se(13:16) = ' '//se(13:15)
      else
         write(se,'(f18.6)') ew
         se(17:20) = ' '//se(16:18)
         se( 9:16) = ' '//se(9:15)
      endif

      write(irlx,'(a8,i5,a1,2i5,a12,a1,a20,1pe12.3,a)') &
          st,ordr,s1,qq,q,sb,s2,se,(eval-oldenergy)*eunit,'   %'
      if(QQ.eq.bdim-1) then
         write(irlx,'(2a,i6,a,i4,13x,a1)') '  Total number of ', &
         'iterations:',order,',  Block iterations:',(order+1)/bdim,'%'
         write(irlx,'(1x,70a1,1x,a1)') ('-',i=1,70),'%' 
      endif

      call myflush(irlx)
      if(qq.eq.0) e1stop=0.d0
      e1stop    = max(e1stop,abs(eval-oldenergy))
      oldorder  = order

      return
      end subroutine wrtrlxbdav

! **********************************************************************

      subroutine sortovlps(ovlps,qlist,order)

      implicit none
      integer(long)                                :: i,j,mxj
      integer(long), intent(in)                    :: order
      integer(long), dimension(12), intent(out)    :: qlist
      real(dop)                                    :: mxo
      real(dop), dimension(0:order), intent(inout) :: ovlps

      do i = 1,12 
         mxo = ovlps(0)
         mxj = 0 
         do j = 1, order
            if(ovlps(j) .gt. mxo) then
               mxo = ovlps(j)
               mxj = j 
            endif
         enddo
         qlist(i) = mxj
         ovlps(mxj) = -ovlps(mxj)
         if( i.eq.7  .and.  -ovlps(mxj).lt.0.0095_dop ) return
      enddo
      
      return
      end subroutine sortovlps

! **********************************************************************

      subroutine wrtrlxcdav(time,order,q,q1,eval,eval1,beta,beta1, &
                         ovl,ovl1,eigenval,ovlps,firstcall,maxord)

use global
use psidef
use datenmod
use runpropmod
use channels

      implicit none

      integer(long)                                 :: i
      integer(long), intent(in)                     :: q,q1,order,maxord
      real(dop)                                     :: sum,ermin,ermax
      real(dop), save                               :: eunit
      real(dop), intent(in)                         :: beta,beta1,time,ovl,ovl1
      real(dop), dimension(0:maxord), intent(in)    :: ovlps
      complex(dop)                                  :: ew
      complex(dop), save                            :: oldenergy
      complex(dop), intent(in)                      :: eval,eval1
      complex(dop), dimension(0:maxord)             :: eigenval
      logical(kind=4), intent(in)                   :: firstcall
      character(len=1)                              :: s1, s2, s3
      character(len=8)                              :: st
      character(len=12)                             :: sb
      character(len=20)                             :: se, si
      integer(long), external                       :: slen
      logical(kind=4), save                         :: reset=.true.
      integer(long), save                           :: oldorder = 0

      if(firstcall) then
         if(order.eq.0) then
            oldenergy=eval
            write(irlx,'(a,/2a,/a,/a,/2a,/a)') &
           'This file contains information on the improved relaxation.', &
           'Q = number of the eigenvector of the Davidson matrix ', &
           '(counting from zero).' , &
           'order = Dimension of the Davidson matrix -1. ', &
           'If order=maxorder, this will be noted by a "*".', &
           'beta = overlap squared of the Q-th Davidson eigenvector ', &
           'with the start vector.', &
           'To view this file type "rdrlx" in name directory.'

            eunit = 1.0_dop
            call keyeunits(eunit,rlxunit,i)
            se = rlxunit(1:slen(rlxunit))//'               '
            if(i.ne.0) then
               routine='wrtrlxcdav (propwf/integrat.F)'
               message='Unknown energy unit: '//rlxunit(1:slen(rlxunit))
               call errormsg
            endif

            if(rlxunit .eq. 'au') then
               if(rlxshift.ne.0.d0 ) then
                  se = rlxunit(1:slen(rlxunit))//'               '
                  write(irlx,'(/a,f11.6,a,1x,g24.10,a16,9x,a)') &
                     ' E-shift =', rlxshift*eunit,' au,  ', &
                     rlxshift,se,'%'
               endif
               eunit = 1.d0/eunit
               if(rlxemin.gt.-1.d99 .and. mod(rlaxnum,1000).eq.930) then
                  ermin=rlxemin*eunit-rlxshift
                  write(irlx,'(1x,a,f11.6,2a,g15.8,a16,9x,a)') &
                        'rlxEmin =',rlxemin,' au,    ', &
                        'shifted:', ermin,se,'%'
               endif
            else

               if(rlxshift.ne.0.0_dop ) then
                  se = rlxunit(1:slen(rlxunit))//'               '
                  write(irlx,'(/a,f11.6,a,1x,g24.10,a16,9x,a)') &
                     ' E-shift =', rlxshift*ev*eunit,' eV,  ', &
                     rlxshift,se,'%'
               endif
               eunit = 1.0_dop/eunit
               if(rlxemin.gt.-1.0e99_dop .and. mod(rlaxnum,1000).eq.930) then
                  ermin=rlxemin*eunit-rlxshift
                  write(irlx,'(1x,a,f11.6,2a,g15.8,a16,9x,a)') &
                        'rlxEmin =',rlxemin*ev,' eV,    ', &
                        'shifted:', ermin,se,'%'
               endif
            endif
            if(rlxemax.lt. 1.0e99_dop .and. mod(rlaxnum,1000).eq.930) then
               ermax=rlxemax*eunit-rlxshift
               write(irlx,'(1x,a,f11.6,2a,g15.8,a16,9x,a)') &
                     'rlxEmax =',rlxemax*ev,' eV,    ', &
                     'shifted:', ermax,se,'%'
            endif
            if(rlximin.gt.-1.d99 .and. mod(rlaxnum,1000).eq.930) then
               write(irlx,'(1x,a,f11.6,a,es19.5,a,26x,a)') &
                     'rlxImin =',rlximin*ev,' eV,    ', &
                      rlximin,' au','%'
            endif
            write(irlx,'(/77x,a)') '%'
            se ='Energy['//rlxunit(1:slen(rlxunit))//']        '
            write(irlx,'(/a,5x,a16,a)')'   time  order   q   beta*1000', &
                    se, '   ovl*1000    Delta-E    %'
            write(irlx,'(/77x,a)') '%'
            se ='Energy['//rlxunit(1:slen(rlxunit))//']        '
            write(irlx,'(/a,5x,a16,a)')'   time  order   q   beta*1000', &
                    se, '   ovl*1000    Delta-E    %'
         elseif(q.eq.q1) then
            write(irlx,'(/a,/90(4(2f9.5,2x)/))') &
                 ' Eigenvalues [eV]   (first diagonalisation)', &
                 (eigenval(i)*ev,i=0,order)
            write(irlx,'(/a,/90(8f10.3/))') &
                 ' Overlaps * 1000   (first diagonalisation)', &
                 (1.0e3_dop*ovlps(i),i=0,order)
         elseif(q.eq.-1) then
           write(irlx,'(/a,/90(4(2f9.5,2x)/))') &
                 ' Eigenvalues [eV]   (last diagonalisation)', &
                 (eigenval(i)*ev,i=0,oldorder)
           if(mod(rlaxnum,1000).eq.930) then
              sum = 0.0_dop
              do i = 0, oldorder
                 sum = sum + ovlps(i)
              enddo
              write(irlx,'(/a,/90(8f10.3/))') &
                   ' Overlaps * 1000  (last diagonalisation)', &
                   (1000*ovlps(i),i=0,oldorder)
              write(irlx,'(a,f11.7,2a)') ' Sum over all overlaps :', &
                    sum,' (<1, as orbital sets have changed)'
           endif
           write(irlx,*)
           return
         endif
      endif

      s1 = ' '
      s2 = ' '
      s3 = ' '
      if(order.ge.maxord)   s1 = '*'
      if(beta.le.0.66667_dop) s2 = '*'
      if(ovl.gt.0.0_dop .and. ovl.le.0.5_dop)     s3 = '*'
      if(abs(time) .lt. tiny .and. beta+ovl .gt. 2.0_dop-tiny) then
         write(st,'(f8.3,1x)') -out2/fs
      elseif(time/fs .lt. 99.9_dop ) then
         write(st,'(f8.3,1x)') time/fs
      else
         write(st,'(f8.1,1x)') time/fs
      endif

      if(q.ne.q1) then
         if(reset) then
            reset = .false.
            write(irlx,'(/2a)') '  time   order   q  q1    ', &
               'beta   beta1    ovl     ovl1    E           E1     ;'
         endif
         write(irlx,'(a8,i5,a1,2i4,3p4f8.2,0p2f10.6,a)') &
            st,order,s1,q,q1,beta,beta1,ovl,ovl1,dble(eval*ev), &
            dble(eval1*ev),'   ;'
         return
      endif

      if(beta .gt. 0.99999998_dop ) then
         write(sb,'(e12.3)') 1.0_dop-beta
      else
         write(sb,'(f12.5)') beta*1.0e3_dop
      endif
      ew = eval*eunit - rlxshift
      if(abs(ew) .lt. 100.0_dop ) then
         write(se,'(f17.12)') dble(ew)
         se(17:20) = ' '//se(15:17)
         se(13:16) = ' '//se(12:14)
         se( 9:12) = ' '//se( 9:11)
         write(si,'(f17.12)') dimag(ew)
         si(17:20) = ' '//si(15:17)
         si(13:16) = ' '//si(12:14)
         si( 9:12) = ' '//si( 9:11)
      elseif(abs(ew) .lt. 1.0e5_dop ) then
         write(se,'(f18.9)') dble(ew)
         se(17:20) = ' '//se(16:18)
         se(13:16) = ' '//se(13:15)
         write(si,'(f18.9)') dimag(ew)
         si(17:20) = ' '//si(16:18)
         si(13:16) = ' '//si(13:15)
      else
         write(se,'(f18.6)') dble(ew)
         se(17:20) = ' '//se(16:18)
         se( 9:16) = ' '//se(9:15)
         write(si,'(f18.6)') dimag(ew)
         si(17:20) = ' '//si(16:18)
         si( 9:16) = ' '//si(9:15)
      endif

      write(irlx,'(a8,i5,a1,i4,a12,a1,a20,3pf11.4,a1,1pe11.3,a)') &
        st,order,s1,q,sb,s2,se,ovl,s3,dble(eval-oldenergy)*eunit,'   %'
      write(irlx,'(31x,a20,12x,1pe11.3,a)') &
                                  si,dimag(eval-oldenergy)*eunit,'   %'

      call myflush(irlx)
      e1stop    = abs(eval-oldenergy)
      oldenergy = eval
      oldorder  = order

      return
      end subroutine wrtrlxcdav

! **********************************************************************

      subroutine writestepcheb (niter,error,time)

use global
use datenmod
use channels

      Implicit None

      Real(dop), intent(in)     :: error,time
      Integer(long), intent(in) :: niter

! --- WRITE STEPSIZE AND ERROR ---

      if (lintstep) then
         if(time.le.9999.999_dop*fs) then
            Write(istp,'(I5,0pE16.6e2,F12.3)') &
              niter, error, time/fs
         else
            Write(istp,'(I5,0pE16.6e2,F12.1)') &
              niter, error, time/fs
         endif
         Call myflush(istp)
      endIf

      return
      end subroutine writestepcheb


! **********************************************************************

      end module wrintegrat

