!                                                                                                                                                                           
!===========   L-BFGS-B (version 3.0.  April 25, 2011  ===================
!
!     This is a modified version of L-BFGS-B. Minor changes in the updated 
!     code appear preceded by a line comment as follows 
!     
!      L-BFGS-B is a code for solving large nonlinear optimization
!             problems with simple bounds on the variables.
!
!        The code can also be used for unconstrained problems and is
!        as efficient for these problems as the earlier limited memory
!                          code L-BFGS.
!                                                     
!===========================================================================

    module LBFGSB_optimizer

    
    implicit none                                                    
    
          
    private
    public ::  timer,driver1,driver2,driver3,dpofa,dtrsl,&
               setulb,mainlb,active,bmv,cauchy,cmprlb,errclb,&
               formk,formt,freev,hpsolb,lnsrlb,matupd,prn1lb,&
               prn2lb,prn3lb,projgr,subsm,dcsrch,dcstep

    contains



!===========================================================================    
!                              setulb
!===========================================================================

    subroutine setulb(n, m, x, l, u, nbd, f, g, factr, pgtol, wa, iwa,&
                       task, iprint, csave, lsave, isave, dsave)
    
         character*60     :: task, csave
         logical          :: lsave(4)
         integer          :: n, m, iprint,&
                             nbd(n), iwa(3*n), isave(44)
         double precision :: f, factr, pgtol, x(n), l(n), u(n), g(n),&
                             wa(2*m*n + 5*n + 11*m*m + 8*m), dsave(29)
         integer          :: lws,lr,lz,lt,ld,lxp,lwa ,&
                             lwy,lsy,lss,lwt,lwn,lsnd
   
         if (task .eq. 'START') then
            isave(1)  = m*n
            isave(2)  = m**2
            isave(3)  = 4*m**2
            isave(4)  = 1                      ! ws      m*n
            isave(5)  = isave(4)  + isave(1)   ! wy      m*n
            isave(6)  = isave(5)  + isave(1)   ! wsy     m**2
            isave(7)  = isave(6)  + isave(2)   ! wss     m**2
            isave(8)  = isave(7)  + isave(2)   ! wt      m**2
            isave(9)  = isave(8)  + isave(2)   ! wn      4*m**2
            isave(10) = isave(9)  + isave(3)   ! wsnd    4*m**2
            isave(11) = isave(10) + isave(3)   ! wz      n
            isave(12) = isave(11) + n          ! wr      n
            isave(13) = isave(12) + n          ! wd      n
            isave(14) = isave(13) + n          ! wt      n
            isave(15) = isave(14) + n          ! wxp     n
            isave(16) = isave(15) + n          ! wa      8*m
         endif
         lws  = isave(4)
         lwy  = isave(5)
         lsy  = isave(6)
         lss  = isave(7)
         lwt  = isave(8)
         lwn  = isave(9)
         lsnd = isave(10)
         lz   = isave(11)
         lr   = isave(12)
         ld   = isave(13)
         lt   = isave(14)
         lxp  = isave(15)
         lwa  = isave(16)
   
         call mainlb(n,m,x,l,u,nbd,f,g,factr,pgtol,&
                     wa(lws),wa(lwy),wa(lsy),wa(lss),wa(lwt),&
                     wa(lwn),wa(lsnd),wa(lz),wa(lr),wa(ld),wa(lt),wa(lxp),&
                     wa(lwa),iwa(1),iwa(n+1),iwa(2*n+1),task,iprint,& 
                     csave,lsave,isave(22),dsave)
   
         return
   
        end subroutine setulb

!===========================================================================    
!                              mainlb
!===========================================================================        

                
        subroutine mainlb(n, m, x, l, u, nbd, f, g, factr, pgtol, ws, wy,&
                          sy, ss, wt, wn, snd, z, r, d, t, xp, wa, &
                          index, iwhere, indx2, task,&
                          iprint, csave, lsave, isave, dsave)
         implicit none

         character(len=60)  :: task, csave
         logical            :: lsave(4)
         integer            :: n, m, iprint, nbd(n), index(n),&
                               iwhere(n), indx2(n), isave(23)
         double precision   :: f, factr, pgtol, x(n), l(n),&
                               u(n), g(n), z(n), r(n), d(n), t(n),&
                               xp(n),wa(8*m), ws(n, m), wy(n, m), &
                               sy(m, m), ss(m, m), wt(m, m),&
                               wn(2*m, 2*m), snd(2*m, 2*m), dsave(29)
         logical            :: prjctd,cnstnd,boxed,updatd,wrk
         character(len=3)   ::   word
         integer            :: i,k,nintol,itfile,iback,nskip,&
                               head,col,iter,itail,iupdat,&
                               nseg,nfgv,info,ifun,&
                               iword,nfree,nact,ileave,nenter
         double precision   :: theta,fold,ddot,dr,rr,tol,&
                               xstep,sbgnrm,ddum,dnorm,dtd,epsmch,&
                               cpu1,cpu2,cachyt,sbtime,lnscht,time1,time2,&
                               gd,gdold,stp,stpmx,time
         double precision   :: one=1.0d0,zero=0.0d0
         
         if (task .eq. 'START') then
   
            epsmch = epsilon(one)
   
            call timer(time1)
   
!        Initialize counters and scalars when task='START'.
   
!          for the limited memory BFGS matrices:
            col    = 0
            head   = 1
            theta  = one
            iupdat = 0
            updatd = .false.
            iback  = 0
            itail  = 0
            iword  = 0
            nact   = 0
            ileave = 0
            nenter = 0
            fold   = zero
            dnorm  = zero
            cpu1   = zero
            gd     = zero
            stpmx  = zero
            sbgnrm = zero
            stp    = zero
            gdold  = zero
            dtd    = zero
   
!          for operation counts:
            iter   = 0
            nfgv   = 0
            nseg   = 0
            nintol = 0
            nskip  = 0
            nfree  = n
            ifun   = 0
!           for stopping tolerance:
            tol = factr*epsmch
   
!           for measuring running time:
            cachyt = 0
            sbtime = 0
            lnscht = 0
    
!           'word' records the status of subspace solutions.
            word = '---'
   
!           'info' records the termination information.
            info = 0
   
            itfile = 8
            if (iprint .ge. 1) then
!                                open a summary file 'iterate.dat'
               open (8, file = 'iterate.dat', status = 'unknown')
            endif            
   
!        Check the input arguments for errors.
   
            call errclb(n,m,factr,l,u,nbd,task,info,k)
            if (task(1:5) .eq. 'ERROR') then
               call prn3lb(n,x,f,task,iprint,info,itfile,&
                           iter,nfgv,nintol,nskip,nact,sbgnrm,&
                           zero,nseg,word,iback,stp,xstep,k,&
                           cachyt,sbtime,lnscht)
               return
            endif
   
            call prn1lb(n,m,l,u,x,iprint,itfile,epsmch)
    
!        Initialize iwhere & project x onto the feasible set.
    
            call active(n,l,u,nbd,x,iwhere,iprint,prjctd,cnstnd,boxed) 
   
!       The end of the initialization.
   
         else
!          restore local variables.
   
            prjctd = lsave(1)
            cnstnd = lsave(2)
            boxed  = lsave(3)
            updatd = lsave(4)
   
            nintol = isave(1)
            itfile = isave(3)
            iback  = isave(4)
            nskip  = isave(5)
            head   = isave(6)
            col    = isave(7)
            itail  = isave(8)
            iter   = isave(9)
            iupdat = isave(10)
            nseg   = isave(12)
            nfgv   = isave(13)
            info   = isave(14)
            ifun   = isave(15)
            iword  = isave(16)
            nfree  = isave(17)
            nact   = isave(18)
            ileave = isave(19)
            nenter = isave(20)
   
            theta  = dsave(1)
            fold   = dsave(2)
            tol    = dsave(3)
            dnorm  = dsave(4)
            epsmch = dsave(5)
            cpu1   = dsave(6)
            cachyt = dsave(7)
            sbtime = dsave(8)
            lnscht = dsave(9)
            time1  = dsave(10)
            gd     = dsave(11)
            stpmx  = dsave(12)
            sbgnrm = dsave(13)
            stp    = dsave(14)
            gdold  = dsave(15)
            dtd    = dsave(16)
      
!        After returning from the driver go to the point where execution
!        is to resume.
   
            if (task(1:5) .eq. 'FG_LN') goto 666
            if (task(1:5) .eq. 'NEW_X') goto 777
            if (task(1:5) .eq. 'FG_ST') goto 111
            if (task(1:4) .eq. 'STOP') then
               if (task(7:9) .eq. 'CPU') then
!                                          restore the previous iterate.
                  call dcopy(n,t,1,x,1)
                  call dcopy(n,r,1,g,1)
                  f = fold
               endif
               goto 999
            endif
         endif 
   
!     Compute f0 and g0.
   
         task = 'FG_START' 
!         return to the driver to calculate f and g; reenter at 111.
         goto 1000
    111  continue
         nfgv = 1
    
!     Compute the infinity norm of the (-) projected gradient.
    
         call projgr(n,l,u,nbd,x,g,sbgnrm)
     
         if (iprint .ge. 1) then
            write (6,1002) iter,f,sbgnrm
            write (itfile,1003) iter,nfgv,sbgnrm,f
         endif
         if (sbgnrm .le. pgtol) then
!                                terminate the algorithm.
            task = 'CONVERGENCE: NORM_OF_PROJECTED_GRADIENT_<=_PGTOL'
            goto 999
         endif 
    
! ----------------- the beginning of the loop --------------------------
    
    222  continue
         if (iprint .ge. 99) write (6,1001) iter + 1
         iword = -1

         if (.not. cnstnd .and. col .gt. 0) then 
!                                            skip the search for GCP.
            call dcopy(n,x,1,z,1)
            wrk = updatd
            nseg = 0
            goto 333
         endif
   
!-------------------------------------------------------------------------------
!               Compute the Generalized Cauchy Point (GCP).
!--------------------------------------------------------------------------------
   
         call timer(cpu1) 
         call cauchy(n,x,l,u,nbd,g,indx2,iwhere,t,d,z,&
                     m,wy,ws,sy,wt,theta,col,head,&
                    wa(1),wa(2*m+1),wa(4*m+1),wa(6*m+1),nseg,&
                    iprint, sbgnrm, info, epsmch)
         if (info .ne. 0) then 
!         singular triangular system detected; refresh the lbfgs memory.
            if(iprint .ge. 1) write (6, 1005)
            info   = 0
            col    = 0
            head   = 1
            theta  = one
            iupdat = 0
            updatd = .false.
            call timer(cpu2) 
            cachyt = cachyt + cpu2 - cpu1
            goto 222
         endif
         call timer(cpu2) 
         cachyt = cachyt + cpu2 - cpu1
         nintol = nintol + nseg
   
!     Count the entering and leaving variables for iter > 0; 
!     find the index set of free and active variables at the GCP.
   
         call freev(n,nfree,index,nenter,ileave,indx2,&
                   iwhere,wrk,updatd,cnstnd,iprint,iter)
         nact = n - nfree
   
    333  continue
    
!If there are no free variables or B=theta*I, then
!                                        skip the subspace minimization.
    
         if (nfree .eq. 0 .or. col .eq. 0) goto 555
    
!----------------------------------------------------------------------
!                   Subspace minimization
!-----------------------------------------------------------------------
   
         call timer(cpu1) 
   
!    Form  the LEL^T factorization of the indefinite
!       matrix    K = [-D -Y'ZZ'Y/theta     L_a'-R_z'  ]
!                     [L_a -R_z           theta*S'AA'S ]
!       where     E = [-I  0]
!                     [ 0  I]
   
         if (wrk) call formk(n,nfree,index,nenter,ileave,indx2,iupdat,&
                             updatd,wn,snd,m,ws,wy,sy,theta,col,head,info)
         if (info .ne. 0) then
!        nonpositive definiteness in Cholesky factorization;
!          refresh the lbfgs memory and restart the iteration.
            if(iprint .ge. 1) write (6, 1006)
            info   = 0
            col    = 0
            head   = 1
            theta  = one
            iupdat = 0
            updatd = .false.
            call timer(cpu2) 
            sbtime = sbtime + cpu2 - cpu1 
            goto 222
         endif 
   
!        compute r=-Z'B(xcp-xk)-Z'g (using wa(2m+1)=W'(xcp-x)
!                                                   from 'cauchy')
         call cmprlb(n,m,x,g,ws,wy,sy,wt,z,r,wa,index,&
                    theta,col,head,nfree,cnstnd,info)
         if (info .ne. 0) goto 444
   
!        call the direct method. 
   
         call subsm( n, m, nfree, index, l, u, nbd, z, r, xp, ws, wy,&
                    theta, x, g, col, head, iword, wa, wn, iprint, info)
    444  continue
         if (info .ne. 0) then 
!          singular triangular system detected;
!          refresh the lbfgs memory and restart the iteration.
            if(iprint .ge. 1) write (6, 1005)
            info   = 0
            col    = 0
            head   = 1
            theta  = one
            iupdat = 0
            updatd = .false.
            call timer(cpu2) 
            sbtime = sbtime + cpu2 - cpu1 
            goto 222
         endif
    
         call timer(cpu2) 
         sbtime = sbtime + cpu2 - cpu1 
    555  continue
    
!------------------------------------------------------------------------
!               Line search and optimality tests
!------------------------------------------------------------------------
    
!Generate the search direction d:=z-x.
   
         do 40 i = 1, n
            d(i) = z(i) - x(i)
     40  continue
         call timer(cpu1) 
    666  continue
         call lnsrlb(n,l,u,nbd,x,f,fold,gd,gdold,g,d,r,t,z,stp,dnorm,&
                     dtd,xstep,stpmx,iter,ifun,iback,nfgv,info,task,&
                     boxed,cnstnd,csave,isave(22),dsave(17))
         if (info .ne. 0 .or. iback .ge. 20) then
!          restore the previous iterate.
            call dcopy(n,t,1,x,1)
            call dcopy(n,r,1,g,1)
            f = fold
            if (col .eq. 0) then
!             abnormal termination.
               if (info .eq. 0) then
                  info = -9
!                restore the actual number of f and g evaluations etc.
                  nfgv = nfgv - 1
                  ifun = ifun - 1
                  iback = iback - 1
               endif
               task = 'ABNORMAL_TERMINATION_IN_LNSRCH'
               iter = iter + 1
               goto 999
            else
!             refresh the lbfgs memory and restart the iteration.
               if(iprint .ge. 1) write (6, 1008)
               if (info .eq. 0) nfgv = nfgv - 1
               info   = 0
               col    = 0
               head   = 1
               theta  = one
               iupdat = 0
               updatd = .false.
               task   = 'RESTART_FROM_LNSRCH'
               call timer(cpu2)
               lnscht = lnscht + cpu2 - cpu1
               goto 222
            endif
         else if (task(1:5) .eq. 'FG_LN') then
!          return to the driver for calculating f and g; reenter at 666.
            goto 1000
         else 
!          calculate and print out the quantities related to the new X.
            call timer(cpu2) 
            lnscht = lnscht + cpu2 - cpu1
            iter = iter + 1
    
!       Compute the infinity norm of the projected (-)gradient.
    
            call projgr(n,l,u,nbd,x,g,sbgnrm)
    
!       Print iteration information.
   
            call prn2lb(n,x,f,g,iprint,itfile,iter,nfgv,nact,&
                        sbgnrm,nseg,word,iword,iback,stp,xstep)
            goto 1000
         endif
    777  continue
   
!c     Test for termination.
   
         if (sbgnrm .le. pgtol) then
!c                                terminate the algorithm.
            task = 'CONVERGENCE: NORM_OF_PROJECTED_GRADIENT_<=_PGTOL'
            goto 999
         endif 
   
         ddum = max(abs(fold), abs(f), one)
         if ((fold - f) .le. tol*ddum) then
!                                        terminate the algorithm.
            task = 'CONVERGENCE: REL_REDUCTION_OF_F_<=_FACTR*EPSMCH'
            if (iback .ge. 10) info = -5
!           i.e., to issue a warning if iback>10 in the line search.
            goto 999
         endif 
   
!    Compute d=newx-oldx, r=newg-oldg, rr=y'y and dr=y's.
    
         do 42 i = 1, n
            r(i) = g(i) - r(i)
     42  continue
         rr = ddot(n,r,1,r,1)
         if (stp .eq. one) then  
            dr = gd - gdold
            ddum = -gdold
         else
            dr = (gd - gdold)*stp
            call dscal(n,stp,d,1)
            ddum = -gdold*stp
         endif
    
         if (dr .le. epsmch*ddum) then
!                            skip the L-BFGS update.
            nskip = nskip + 1
            updatd = .false.
            if (iprint .ge. 1) write (6,1004) dr, ddum
            goto 888
         endif 
    
!------------------------------------------------------------------------
!              Update the L-BFGS matrix
!------------------------------------------------------------------------
    
         updatd = .true.
         iupdat = iupdat + 1
   
!Update matrices WS and WY and form the middle matrix in B.
   
         call matupd(n,m,ws,wy,sy,ss,d,r,itail,&
                     iupdat,col,head,theta,rr,dr,stp,dtd)
   
!     Form the upper half of the pds T = theta*SS + L*D^(-1)*L';
!     Store T in the upper triangular of the array wt;
!        Cholesky factorize T to J*J' with
!           J' stored in the upper triangular of wt.
   
         call formt(m,wt,sy,ss,col,theta,info)
    
         if (info .ne. 0) then 
!          nonpositive definiteness in Cholesky factorization;
!          refresh the lbfgs memory and restart the iteration.
            if(iprint .ge. 1) write (6, 1007)
            info = 0
            col = 0
            head = 1
            theta = one
            iupdat = 0
            updatd = .false.
            goto 222
         endif
   
!     Now the inverse of the middle matrix in B is
   
!       [  D^(1/2)      O ] [ -D^(1/2)  D^(-1/2)*L' ]
!       [ -L*D^(-1/2)   J ] [  0        J'          ]
   
    888  continue
    
! -------------------- the end of the loop -----------------------------
    
         goto 222
    999  continue
         call timer(time2)
         time = time2 - time1
         call prn3lb(n,x,f,task,iprint,info,itfile,&
                    iter,nfgv,nintol,nskip,nact,sbgnrm,&
                    time,nseg,word,iback,stp,xstep,k,&
                    cachyt,sbtime,lnscht)
    1000 continue
   
!    Save local variables.
   
         lsave(1)  = prjctd
         lsave(2)  = cnstnd
         lsave(3)  = boxed
         lsave(4)  = updatd
   
         isave(1)  = nintol 
         isave(3)  = itfile 
         isave(4)  = iback 
         isave(5)  = nskip 
         isave(6)  = head 
         isave(7)  = col 
         isave(8)  = itail 
         isave(9)  = iter 
         isave(10) = iupdat 
         isave(12) = nseg
         isave(13) = nfgv 
         isave(14) = info 
         isave(15) = ifun 
         isave(16) = iword 
         isave(17) = nfree 
         isave(18) = nact 
         isave(19) = ileave 
         isave(20) = nenter 
   
         dsave(1)  = theta 
         dsave(2)  = fold 
         dsave(3)  = tol 
         dsave(4)  = dnorm 
         dsave(5)  = epsmch 
         dsave(6)  = cpu1 
         dsave(7)  = cachyt 
         dsave(8)  = sbtime 
         dsave(9)  = lnscht 
         dsave(10) = time1 
         dsave(11) = gd 
         dsave(12) = stpmx 
         dsave(13) = sbgnrm
         dsave(14) = stp
         dsave(15) = gdold
         dsave(16) = dtd  
   
    1001 format (//,'ITERATION ',i5)
    1002 format (/,'At iterate',i5,4x,'f= ',1p,d12.5,4x,'|proj g|= ',1p,d12.5)
    1003 format (2(1x,i4),5x,'-',5x,'-',3x,'-',5x,'-',5x,'-',8x,'-',3x,&
                 1p,2(1x,d10.3))
    1004 format ('  ys=',1p,e10.3,'  -gs=',1p,e10.3,' BFGS update SKIPPED')
    1005 format (/, &
        ' Singular triangular system detected;',/,&
        '   refresh the lbfgs memory and restart the iteration.')
    1006 format (/, &
        ' Nonpositive definiteness in Cholesky factorization in formk;',/,&
        '   refresh the lbfgs memory and restart the iteration.')
    1007 format (/,& 
        ' Nonpositive definiteness in Cholesky factorization in formt;',/,&
        '   refresh the lbfgs memory and restart the iteration.')
    1008 format (/,& 
        ' Bad direction in the line search;',/,&
        '   refresh the lbfgs memory and restart the iteration.')
   
         return   
   
        end subroutine mainlb

!===========================================================================    
!                              setulb
!===========================================================================           
   
        subroutine active(n, l, u, nbd, x, iwhere, iprint,&
                          prjctd, cnstnd, boxed)
   
         implicit none                 
         logical          :: prjctd, cnstnd, boxed
         integer          :: n, iprint, nbd(n), iwhere(n),&
                             nbdd,i
         double precision :: x(n), l(n), u(n),zero=0.0d0
   
!Initialize nbdd, prjctd, cnstnd and boxed.
   
         nbdd = 0
         prjctd = .false.
         cnstnd = .false.
         boxed = .true.
   
!     Project the initial x to the easible set if necessary.
   
         do 10 i = 1, n
            if (nbd(i) .gt. 0) then
               if (nbd(i) .le. 2 .and. x(i) .le. l(i)) then
                  if (x(i) .lt. l(i)) then
                     prjctd = .true.
                     x(i) = l(i)
                  endif
                  nbdd = nbdd + 1
               else if (nbd(i) .ge. 2 .and. x(i) .ge. u(i)) then
                  if (x(i) .gt. u(i)) then
                     prjctd = .true.
                     x(i) = u(i)
                  endif
                  nbdd = nbdd + 1
               endif
            endif
     10  continue
   
!     Initialize iwhere and assign values to cnstnd and boxed.
   
         do 20 i = 1, n
            if (nbd(i) .ne. 2) boxed = .false.
            if (nbd(i) .eq. 0) then
!                                this variable is always free
               iwhere(i) = -1
   
!           otherwise set x(i)=mid(x(i), u(i), l(i)).
            else
               cnstnd = .true.
               if (nbd(i) .eq. 2 .and. u(i) - l(i) .le. zero) then
!                   this variable is always fixed
                  iwhere(i) = 3
               else 
                  iwhere(i) = 0
               endif
            endif
     20  continue
   
         if (iprint .ge. 0) then
            if (prjctd) write (6,*)&
           'The initial X is infeasible.  Restart with its projection.'
            if (.not. cnstnd)&
              write (6,*) 'This problem is unconstrained.'
         endif
   
         if (iprint .gt. 0) write (6,1001) nbdd
   
    1001 format (/,'At X0 ',i9,' variables are exactly at the bounds') 
   
         return
   
        end subroutine active

!===========================================================================    
!                              bmv
!===========================================================================             
    
         subroutine bmv(m, sy, wt, col, v, p, info)

         implicit none

         integer          :: m, col, info,i,k,i2
         double precision :: sy(m, m), wt(m, m), v(2*col), p(2*col),sum
    
         if (col .eq. 0) return
    
!     PART I: solve [  D^(1/2)      O ] [ p1 ] = [ v1 ]
!                   [ -L*D^(-1/2)   J ] [ p2 ]   [ v2 ].
   
!       solve Jp2=v2+LD^(-1)v1.
         p(col + 1) = v(col + 1)
         do 20 i = 2, col
            i2 = col + i
            sum = 0.0d0
            do 10 k = 1, i - 1
               sum = sum + sy(i,k)*v(k)/sy(k,k)
     10     continue
            p(i2) = v(i2) + sum
     20  continue  
!     Solve the triangular system
         call dtrsl(wt,m,col,p(col+1),11,info)
         if (info .ne. 0) return
    
!       solve D^(1/2)p1=v1.
         do 30 i = 1, col
            p(i) = v(i)/sqrt(sy(i,i))
     30  continue 
    
!     PART II: solve [ -D^(1/2)   D^(-1/2)*L'  ] [ p1 ] = [ p1 ]
!                    [  0         J'           ] [ p2 ]   [ p2 ]. 
    
!       solve J^Tp2=p2. 
         call dtrsl(wt,m,col,p(col+1),01,info)
         if (info .ne. 0) return
    
!       compute p1=-D^(-1/2)(p1-D^(-1/2)L'p2)
!                 =-D^(-1/2)p1+D^(-1)L'p2.  
         do 40 i = 1, col
            p(i) = -p(i)/sqrt(sy(i,i))
     40  continue
         do 60 i = 1, col
            sum = 0.d0
            do 50 k = i + 1, col
               sum = sum + sy(k,i)*p(col+k)/sy(i,i)
     50     continue
            p(i) = p(i) + sum
     60  continue
   
         return
   
    end subroutine bmv


!===========================================================================    
!                              cauchy
!===========================================================================    

         subroutine cauchy(n, x, l, u, nbd, g, iorder, iwhere, t, d, xcp,& 
                           m, wy, ws, sy, wt, theta, col, head, p, c, wbp,& 
                           v, nseg, iprint, sbgnrm, info, epsmch)
         implicit none

         integer          :: n, m, head, col, nseg, iprint,& 
                             nbd(n), iorder(n), iwhere(n),&
                             i,j,col2,nfree,nbreak,pointr,&
                             ibp,nleft,ibkmin,iter,info
         double precision :: theta, epsmch,d(n), xcp(n),&
                             x(n), l(n), u(n), g(n), t(n),& 
                             wy(n, col), ws(n, col), sy(m, m),&
                             wt(m, m), p(2*m), c(2*m), wbp(2*m),&
                             v(2*m),f1,f2,dt,dtm,tsum,dibp,zibp,&
                             dibp2,bkmin,tu,tl,wmc,wmp,wmw,ddot,&
                             tj,tj0,neggi,sbgnrm,f2_org,one=1.0d0,zero=0.0d0
         logical          :: xlower,xupper,bnded
    
!     Check the status of the variables, reset iwhere(i) if necessary;
!       compute the Cauchy direction d and the breakpoints t; initialize
!       the derivative f1 and the vector p = W'd (for theta = 1).
    
         if (sbgnrm .le. zero) then
            if (iprint .ge. 0) write (6,*) 'Subgnorm = 0.  GCP = X.'
            call dcopy(n,x,1,xcp,1)
            return
         endif 
         bnded = .true.
         nfree = n + 1
         nbreak = 0
         ibkmin = 0
         bkmin = zero
         col2 = 2*col
         f1 = zero
         if (iprint .ge. 99) write (6,3010)
   
!We set p to zero and build it up as we determine d.
   
         do 20 i = 1, col2
            p(i) = zero
     20  continue 
   
!     In the following loop we determine for each variable its bound
!        status and its breakpoint, and update p accordingly.
!        Smallest breakpoint is identified.
   
         tu = 0.0
         tl = 0.0
         do 50 i = 1, n 
            neggi = -g(i)      
            if (iwhere(i) .ne. 3 .and. iwhere(i) .ne. -1) then
!             if x(i) is not a constant and has bounds,
!             compute the difference between x(i) and its bounds.
               if (nbd(i) .le. 2) tl = x(i) - l(i)
               if (nbd(i) .ge. 2) tu = u(i) - x(i)
   
!           If a variable is close enough to a bound
!             we treat it as at bound.
               xlower = nbd(i) .le. 2 .and. tl .le. zero
               xupper = nbd(i) .ge. 2 .and. tu .le. zero
   
!              reset iwhere(i).
               iwhere(i) = 0
               if (xlower) then
                  if (neggi .le. zero) iwhere(i) = 1
               else if (xupper) then
                  if (neggi .ge. zero) iwhere(i) = 2
               else
                  if (abs(neggi) .le. zero) iwhere(i) = -3
               endif
            endif 
            pointr = head
            if (iwhere(i) .ne. 0 .and. iwhere(i) .ne. -1) then
               d(i) = zero
            else
               d(i) = neggi
               f1 = f1 - neggi*neggi
!             calculate p := p - W'e_i* (g_i).
               do 40 j = 1, col
                  p(j) = p(j) +  wy(i,pointr)* neggi
                  p(col + j) = p(col + j) + ws(i,pointr)*neggi
                  pointr = mod(pointr,m) + 1
     40        continue 
               if (nbd(i) .le. 2 .and. nbd(i) .ne. 0 &
                                 .and. neggi .lt. zero) then
!                                x(i) + d(i) is bounded; compute t(i).
                  nbreak = nbreak + 1
                  iorder(nbreak) = i
                  t(nbreak) = tl/(-neggi)
                  if (nbreak .eq. 1 .or. t(nbreak) .lt. bkmin) then
                     bkmin = t(nbreak)
                     ibkmin = nbreak
                  endif
               else if (nbd(i) .ge. 2 .and. neggi .gt. zero) then
!                                x(i) + d(i) is bounded; compute t(i).
                  nbreak = nbreak + 1
                  iorder(nbreak) = i
                  t(nbreak) = tu/neggi
                  if (nbreak .eq. 1 .or. t(nbreak) .lt. bkmin) then
                     bkmin = t(nbreak)
                     ibkmin = nbreak
                  endif
               else
!               x(i) + d(i) is not bounded.
                  nfree = nfree - 1
                  iorder(nfree) = i
                  if (abs(neggi) .gt. zero) bnded = .false.
               endif
            endif
     50  continue 
    
!   The indices of the nonzero components of d are now stored
!     in iorder(1),...,iorder(nbreak) and iorder(nfree),...,iorder(n).
!       The smallest of the nbreak breakpoints is in t(ibkmin)=bkmin.
    
         if (theta .ne. one) then
!                  complete the initialization of p for theta not= one.
            call dscal(col,theta,p(col+1),1)
         endif
    
!   Initialize GCP xcp = x.
   
         call dcopy(n,x,1,xcp,1)
   
         if (nbreak .eq. 0 .and. nfree .eq. n + 1) then
!                is a zero vector, return with the initial xcp as GCP.
            if (iprint .gt. 100) write (6,1010) (xcp(i), i = 1, n)
            return
         endif    
    
!    Initialize c = W'(xcp - x) = 0.
     
         do 60 j = 1, col2
            c(j) = zero
     60  continue 
    
!     Initialize derivative f2.
    
         f2 =  -theta*f1 
         f2_org  =  f2
         if (col .gt. 0) then
            call bmv(m,sy,wt,col,p,v,info)
            if (info .ne. 0) return
            f2 = f2 - ddot(col2,v,1,p,1)
         endif
         dtm = -f1/f2
         tsum = zero
         nseg = 1
         if (iprint .ge. 99) & 
            write (6,*) 'There are ',nbreak,'  breakpoints '
    
!   If there are no breakpoints, locate the GCP and return. 
    
         if (nbreak .eq. 0) goto 888
                
         nleft = nbreak
         iter = 1
    
    
         tj = zero
    
!------------------- the beginning of the loop -------------------------
    
    777  continue
    
!     Find the next smallest breakpoint;
!       compute dt = t(nleft) - t(nleft + 1).
    
         tj0 = tj
         if (iter .eq. 1) then
!         Since we already have the smallest breakpoint we need not do
!         heapsort yet. Often only one breakpoint is used and the
!        cost of heapsort is avoided.
            tj = bkmin
            ibp = iorder(ibkmin)
         else
            if (iter .eq. 2) then
!            Replace the already used smallest breakpoint with the
!             breakpoint numbered nbreak > nlast, before heapsort call.
               if (ibkmin .ne. nbreak) then
                  t(ibkmin) = t(nbreak)
                  iorder(ibkmin) = iorder(nbreak)
               endif 
!       Update heap structure of breakpoints
!          (if iter=2, initialize heap).
            endif
            call hpsolb(nleft,t,iorder,iter-2)
            tj = t(nleft)
            ibp = iorder(nleft)  
         endif 
            
         dt = tj - tj0
    
         if (dt .ne. zero .and. iprint .ge. 100) then
            write (6,4011) nseg,f1,f2
            write (6,5010) dt
            write (6,6010) dtm
         endif          
    
!    If a minimizer is within this interval, locate the GCP and return. 
    
         if (dtm .lt. dt) goto 888
    
!     Otherwise fix one variable and
!       reset the corresponding component of d to zero.
       
         tsum = tsum + dt
         nleft = nleft - 1
         iter = iter + 1
         dibp = d(ibp)
         d(ibp) = zero
         if (dibp .gt. zero) then
            zibp = u(ibp) - x(ibp)
            xcp(ibp) = u(ibp)
            iwhere(ibp) = 2
         else
            zibp = l(ibp) - x(ibp)
            xcp(ibp) = l(ibp)
            iwhere(ibp) = 1
         endif
         if (iprint .ge. 100) write (6,*) 'Variable  ',ibp,'  is fixed.'
         if (nleft .eq. 0 .and. nbreak .eq. n) then
!                                            all n variables are fixed,
!                                               return with xcp as GCP.
            dtm = dt
            goto 999
         endif
    
!    Update the derivative information.
    
         nseg = nseg + 1
         dibp2 = dibp**2
    
!    Update f1 and f2.
    
!       temporarily set f1 and f2 for col=0.
         f1 = f1 + dt*f2 + dibp2 - theta*dibp*zibp
         f2 = f2 - theta*dibp2
   
         if (col .gt. 0) then
!                            update c = c + dt*p.
            call daxpy(col2,dt,p,1,c,1)
    
!              choose wbp,
!           the row of W corresponding to the breakpoint encountered.

            pointr = head
            do 70 j = 1,col
               wbp(j) = wy(ibp,pointr)
               wbp(col + j) = theta*ws(ibp,pointr)
               pointr = mod(pointr,m) + 1
     70     continue 
    
!          compute (wbp)Mc, (wbp)Mp, and (wbp)M(wbp)'.
            call bmv(m,sy,wt,col,wbp,v,info)
            if (info .ne. 0) return
            wmc = ddot(col2,c,1,v,1)
            wmp = ddot(col2,p,1,v,1) 
            wmw = ddot(col2,wbp,1,v,1)
    
!           update p = p - dibp*wbp. 
            call daxpy(col2,-dibp,wbp,1,p,1)
    
!           complete updating f1 and f2 while col > 0.
            f1 = f1 + dibp*wmc
            f2 = f2 + 2.0d0*dibp*wmp - dibp2*wmw
         endif
   
         f2 = max(epsmch*f2_org,f2)
         if (nleft .gt. 0) then
            dtm = -f1/f2
            goto 777
!                to repeat the loop for unsearched intervals. 
         else if(bnded) then
            f1 = zero
            f2 = zero
            dtm = zero
         else
            dtm = -f1/f2
         endif 
   
 !------------------- the end of the loop -------------------------------
    
    888  continue
         if (iprint .ge. 99) then
            write (6,*)
            write (6,*) 'GCP found in this segment'
            write (6,4010) nseg,f1,f2
            write (6,6010) dtm
         endif 
         if (dtm .le. zero) dtm = zero
         tsum = tsum + dtm
    
!     Move free variables (i.e., the ones w/o breakpoints) and 
!     the variables whose breakpoints haven't been reached.
    
         call daxpy(n,tsum,d,1,xcp,1)
    
    999  continue
    
!    Update c = c + dtm*p = W'(x^c - x) 
!      which will be used in computing r = Z'(B(x^c - x) + g).
    
         if (col .gt. 0) call daxpy(col2,dtm,p,1,c,1)
         if (iprint .gt. 100) write (6,1010) (xcp(i),i = 1,n)
         if (iprint .ge. 99) write (6,2010)
   
    1010 format ('Cauchy X =  ',/,(4x,1p,6(1x,d11.4)))
    2010 format (/,'---------------- exit CAUCHY----------------------',/)
    3010 format (/,'---------------- CAUCHY entered-------------------')
    4010 format ('Piece    ',i3,' --f1, f2 at start point ',1p,2(1x,d11.4))
    4011 format (/,'Piece    ',i3,' --f1, f2 at start point ',&
                1p,2(1x,d11.4))
    5010 format ('Distance to the next break point =  ',1p,d11.4)
    6010 format ('Distance to the stationary point =  ',1p,d11.4) 
    
         return
    
    end subroutine cauchy
   
!===========================================================================    
!                            cmprlb
!===========================================================================

   
        subroutine cmprlb(n, m, x, g, ws, wy, sy, wt, z, r, wa, index,& 
                         theta, col, head, nfree, cnstnd, info)
    
        implicit none 

         logical          :: cnstnd
         integer          :: n, m, col, head, nfree, info,&
                             index(n),i,j,k,pointr
         double precision :: theta, x(n), g(n), z(n), r(n),&
                             wa(4*m), ws(n, m), wy(n, m),&
                             sy(m, m), wt(m, m), a1,a2
   
         if (.not. cnstnd .and. col .gt. 0) then 
            do 26 i = 1, n
               r(i) = -g(i)
     26     continue
         else
            do 30 i = 1, nfree
               k = index(i)
               r(i) = -theta*(z(k) - x(k)) - g(k)
     30     continue
            call bmv(m,sy,wt,col,wa(2*m+1),wa(1),info)
            if (info .ne. 0) then
               info = -8
               return
            endif
            pointr = head 
            do 34 j = 1, col
               a1 = wa(j)
               a2 = theta*wa(col + j)
               do 32 i = 1, nfree
                  k = index(i)
                  r(i) = r(i) + wy(k,pointr)*a1 + ws(k,pointr)*a2
     32        continue
               pointr = mod(pointr,m) + 1
     34     continue
         endif
   
         return
   
        end subroutine cmprlb
   
!===========================================================================    
!                              errclb
!===========================================================================
   
        subroutine errclb(n, m, factr, l, u, nbd, task, info, k)
         
        implicit none

         character(len=60) :: task
         integer           :: n, m, info, k, nbd(n),i
         double precision  :: factr, l(n), u(n),&
                              zero=0.0d0
   
!     Check the input arguments for errors.
   
         if (n .le. 0) task = 'ERROR: N .LE. 0'
         if (m .le. 0) task = 'ERROR: M .LE. 0'
         if (factr .lt. zero) task = 'ERROR: FACTR .LT. 0'
   
!    Check the validity of the arrays nbd(i), u(i), and l(i).
   
         do 10 i = 1, n
            if (nbd(i) .lt. 0 .or. nbd(i) .gt. 3) then
!                                                  return
               task = 'ERROR: INVALID NBD'
               info = -6
               k = i
            endif
            if (nbd(i) .eq. 2) then
               if (l(i) .gt. u(i)) then
!                                   return
                  task = 'ERROR: NO FEASIBLE SOLUTION'
                  info = -7
                  k = i
               endif
            endif
     10  continue
   
         return
   
        end subroutine errclb
   
!===========================================================================    
!                              formk
!===========================================================================

        subroutine formk(n, nsub, ind, nenter, ileave, indx2, iupdat,&
                          updatd, wn, wn1, m, ws, wy, sy, theta, col,&
                          head, info)
   
        implicit none                  

         integer           :: n, nsub, m, col, head, nenter, ileave, iupdat,&
                              info, ind(n), indx2(n), m2,ipntr,jpntr,iy,is,&
                              jy,js,is1,js1,k1,i,k,col2,pbegin,pend,dbegin,&
                              dend,upcl
         double precision  :: theta, wn(2*m, 2*m), wn1(2*m, 2*m),&
                              ws(n, m), wy(n, m), sy(m, m),ddot,&
                              temp1,temp2,temp3,temp4,zero=0.0d0
         logical           :: updatd
   
!    Form the lower triangular part of
!               WN1 = [Y' ZZ'Y   L_a'+R_z'] 
!                     [L_a+R_z   S'AA'S   ]
!        where L_a is the strictly lower triangular part of S'AA'Y
!              R_z is the upper triangular part of S'ZZ'Y.
         
         if (updatd) then
            if (iupdat .gt. m) then 
!                                shift old part of WN1.
               do 10 jy = 1, m - 1
                  js = m + jy
                  call dcopy(m-jy,wn1(jy+1,jy+1),1,wn1(jy,jy),1)
                  call dcopy(m-jy,wn1(js+1,js+1),1,wn1(js,js),1)
                  call dcopy(m-1,wn1(m+2,jy+1),1,wn1(m+1,jy),1)
     10        continue
            endif
    
!        put new rows in blocks (1,1), (2,1) and (2,2).
            pbegin = 1
            pend = nsub
            dbegin = nsub + 1
            dend = n
            iy = col
            is = m + col
            ipntr = head + col - 1
            if (ipntr .gt. m) ipntr = ipntr - m    
            jpntr = head
            do 20 jy = 1, col
               js = m + jy
               temp1 = zero
               temp2 = zero
               temp3 = zero
!            compute element jy of row 'col' of Y'ZZ'Y
               do 15 k = pbegin, pend
                  k1 = ind(k)
                  temp1 = temp1 + wy(k1,ipntr)*wy(k1,jpntr)
     15        continue
!            compute elements jy of row 'col' of L_a and S'AA'S
               do 16 k = dbegin, dend
                  k1 = ind(k)
                  temp2 = temp2 + ws(k1,ipntr)*ws(k1,jpntr)
                  temp3 = temp3 + ws(k1,ipntr)*wy(k1,jpntr)
     16        continue
               wn1(iy,jy) = temp1
               wn1(is,js) = temp2
               wn1(is,jy) = temp3
               jpntr = mod(jpntr,m) + 1
     20     continue
    
!         put new column in block (2,1).
            jy = col       
            jpntr = head + col - 1
            if (jpntr .gt. m) jpntr = jpntr - m
            ipntr = head
            do 30 i = 1, col
               is = m + i
               temp3 = zero
!            compute element i of column 'col' of R_z
               do 25 k = pbegin, pend
                  k1 = ind(k)
                  temp3 = temp3 + ws(k1,ipntr)*wy(k1,jpntr)
     25        continue 
               ipntr = mod(ipntr,m) + 1
               wn1(is,jy) = temp3
     30     continue
            upcl = col - 1
         else
            upcl = col
         endif
    
!      modify the old parts in blocks (1,1) and (2,2) due to changes
!      in the set of free variables.
         ipntr = head      
         do 45 iy = 1, upcl
            is = m + iy
            jpntr = head
            do 40 jy = 1, iy
               js = m + jy
               temp1 = zero
               temp2 = zero
               temp3 = zero
               temp4 = zero
               do 35 k = 1, nenter
                  k1 = indx2(k)
                  temp1 = temp1 + wy(k1,ipntr)*wy(k1,jpntr)
                  temp2 = temp2 + ws(k1,ipntr)*ws(k1,jpntr)
     35        continue
               do 36 k = ileave, n
                  k1 = indx2(k)
                  temp3 = temp3 + wy(k1,ipntr)*wy(k1,jpntr)
                  temp4 = temp4 + ws(k1,ipntr)*ws(k1,jpntr)
     36        continue
               wn1(iy,jy) = wn1(iy,jy) + temp1 - temp3 
               wn1(is,js) = wn1(is,js) - temp2 + temp4 
               jpntr = mod(jpntr,m) + 1
     40     continue
            ipntr = mod(ipntr,m) + 1
     45  continue
    
!      modify the old parts in block (2,1).
         ipntr = head      
         do 60 is = m + 1, m + upcl
            jpntr = head 
            do 55 jy = 1, upcl
               temp1 = zero
               temp3 = zero
               do 50 k = 1, nenter
                  k1 = indx2(k)
                  temp1 = temp1 + ws(k1,ipntr)*wy(k1,jpntr)
     50        continue
               do 51 k = ileave, n
                  k1 = indx2(k)
                  temp3 = temp3 + ws(k1,ipntr)*wy(k1,jpntr)
     51        continue
            if (is .le. jy + m) then
                  wn1(is,jy) = wn1(is,jy) + temp1 - temp3  
               else
                  wn1(is,jy) = wn1(is,jy) - temp1 + temp3  
               endif
               jpntr = mod(jpntr,m) + 1
     55     continue
            ipntr = mod(ipntr,m) + 1
     60  continue
    
!    Form the upper triangle of WN = [D+Y' ZZ'Y/theta   -L_a'+R_z' ] 
!                                    [-L_a +R_z        S'AA'S*theta]
   
         m2 = 2*m
         do 70 iy = 1, col
            is = col + iy
            is1 = m + iy
            do 65 jy = 1, iy
               js = col + jy
               js1 = m + jy
               wn(jy,iy) = wn1(iy,jy)/theta
               wn(js,is) = wn1(is1,js1)*theta
     65     continue
            do 66 jy = 1, iy - 1
               wn(jy,is) = -wn1(is1,jy)
     66     continue
            do 67 jy = iy, col
               wn(jy,is) = wn1(is1,jy)
     67     continue
            wn(iy,iy) = wn(iy,iy) + sy(iy,iy)
     70  continue
   
!    Form the upper triangle of WN= [  LL'            L^-1(-L_a'+R_z')] 
!                                  [(-L_a +R_z)L'^-1   S'AA'S*theta  ]
   
!      first Cholesky factor (1,1) block of wn to get LL'
!                        with L' stored in the upper triangle of wn.
         call dpofa(wn,m2,col,info)
         if (info .ne. 0) then
            info = -1
            return
         endif
!       then form L^-1(-L_a'+R_z') in the (1,2) block.
         col2 = 2*col
         do 71 js = col+1 ,col2
            call dtrsl(wn,m2,col,wn(1,js),11,info)
     71  continue
   
!    Form S'AA'S*theta + (L^-1(-L_a'+R_z'))'L^-1(-L_a'+R_z') in the
!        upper triangle of (2,2) block of wn.
                         
   
         do 72 is = col+1, col2
            do 74 js = is, col2
                  wn(is,js) = wn(is,js) + ddot(col,wn(1,is),1,wn(1,js),1)
     74        continue
     72     continue
   
!    Cholesky factorization of (2,2) block of wn.
   
         call dpofa(wn(col+1,col+1),m2,col,info)
         if (info .ne. 0) then
            info = -2
            return
         endif
   
         return
   
        end subroutine formk
   
!===========================================================================    
!                              formt
!===========================================================================
   
        subroutine formt(m, wt, sy, ss, col, theta, info)
          
        implicit none

         integer          :: m, col, info, i,j,k,k1
         double precision :: theta, wt(m, m), sy(m, m),&
                             ss(m, m),ddum,zero=0.0d0
   
   
!     Form the upper half of  T = theta*SS + L*D^(-1)*L',
!       store T in the upper triangle of the array wt.
    
         do 52 j = 1, col
            wt(1,j) = theta*ss(1,j)
     52  continue
         do 55 i = 2, col
            do 54 j = i, col
               k1 = min(i,j) - 1
               ddum  = zero
               do 53 k = 1, k1
                  ddum  = ddum + sy(i,k)*sy(j,k)/sy(k,k)
     53        continue
               wt(i,j) = ddum + theta*ss(i,j)
     54     continue
     55  continue
    
!     Cholesky factorize T to J*J' with 
!       J' stored in the upper triangle of wt.
    
         call dpofa(wt,m,col,info)
         if (info .ne. 0) then
            info = -3
         endif
   
         return
   
        end subroutine formt

!===========================================================================    
!                              freev
!===========================================================================
    
        subroutine freev(n, nfree, index, nenter, ileave, indx2,& 
                         iwhere, wrk, updatd, cnstnd, iprint, iter)
   
        implicit none                 

         integer  :: n, nfree, nenter, ileave, iprint,&
                     iter, index(n), indx2(n), iwhere(n),&
                     iact,i,k
         logical  :: wrk, updatd, cnstnd

         nenter = 0
         ileave = n + 1
         if (iter .gt. 0 .and. cnstnd) then
!                          count the entering and leaving variables.
            do 20 i = 1, nfree
               k = index(i)
   
 !          write(6,*) ' k  = index(i) ', k
!           write(6,*) ' index = ', i
   
               if (iwhere(k) .gt. 0) then
                  ileave = ileave - 1
                  indx2(ileave) = k
                  if (iprint .ge. 100) write (6,*)&
                    'Variable ',k,' leaves the set of free variables'
               endif
     20     continue
            do 22 i = 1 + nfree, n
               k = index(i)
               if (iwhere(k) .le. 0) then
                  nenter = nenter + 1
                  indx2(nenter) = k
                  if (iprint .ge. 100) write (6,*)&
                     'Variable ',k,' enters the set of free variables'
               endif
     22     continue
            if (iprint .ge. 99) write (6,*)&
              n+1-ileave,' variables leave; ',nenter,' variables enter'
         endif
         wrk = (ileave .lt. n+1) .or. (nenter .gt. 0) .or. updatd
    
!    Find the index set of free and active variables at the GCP.
    
         nfree = 0 
         iact = n + 1
         do 24 i = 1, n
            if (iwhere(i) .le. 0) then
               nfree = nfree + 1
               index(nfree) = i
            else
               iact = iact - 1
               index(iact) = i
            endif
     24  continue
         if (iprint .ge. 99) write (6,*)&
             nfree,' variables are free at GCP ',iter + 1  
   
         return
   
        end subroutine freev
   
!===========================================================================    
!                              hpsolb
!===========================================================================

        subroutine hpsolb(n, t, iorder, iheap)

        implicit none

         integer           :: iheap, n, iorder(n),&
                              j,k,indxin,indxou,i
         double precision  :: t(n), ddum,out
   
         if (iheap .eq. 0) then
   
!        Rearrange the elements t(1) to t(n) to form a heap.
   
            do 20 k = 2, n
               ddum  = t(k)
               indxin = iorder(k)
   
!          Add ddum to the heap.
               i = k
      10       continue
               if (i.gt.1) then
                  j = i/2
                  if (ddum .lt. t(j)) then
                     t(i) = t(j)
                     iorder(i) = iorder(j)
                     i = j
                     goto 10 
                  endif  
               endif  
               t(i) = ddum
               iorder(i) = indxin
      20    continue
         endif
    
!    Assign to 'out' the value of t(1), the least member of the heap,
!       and rearrange the remaining members to form a heap as
!      elements 1 to n-1 of t.
    
         if (n .gt. 1) then
            i = 1
            out = t(1)
            indxou = iorder(1)
            ddum  = t(n)
            indxin  = iorder(n)
   
!      Restore the heap 
      30    continue
            j = i+i
            if (j .le. n-1) then
               if (t(j+1) .lt. t(j)) j = j+1
               if (t(j) .lt. ddum ) then
                  t(i) = t(j)
                  iorder(i) = iorder(j)
                  i = j
                  goto 30
               endif 
            endif 
            t(i) = ddum
            iorder(i) = indxin
    
!    Put the least member in t(n). 
   
            t(n) = out
            iorder(n) = indxou
         endif 
   
         return
   
        end subroutine hpsolb
   
!===========================================================================    
!                              lnsrlb
!===========================================================================
   
        subroutine lnsrlb(n, l, u, nbd, x, f, fold, gd, gdold, g, d, r, t,&
                           z, stp, dnorm, dtd, xstep, stpmx, iter, ifun,&
                           iback, nfgv, info, task, boxed, cnstnd, csave,&
                           isave, dsave)
         implicit none

         character(len=60)    :: task, csave
         logical              :: boxed, cnstnd
         integer              :: n, iter, ifun, iback, nfgv,&
                                 nbd(n), isave(2),i, info
         double precision     :: f, fold, gd, gdold, stp, dnorm, dtd,& 
                                 stpmx, x(n), l(n), u(n), g(n), d(n),&
                                 z(n), dsave(13),ddot,a1,a2,xstep,r(n),&
                                 t(n), one=1.0d0,zero=0.0d0,big=1.0d+10,&
                                 ftol=1.0d-3,gtol=0.9d0,xtol=0.1d0
   
         if (task(1:5) .eq. 'FG_LN') goto 556
   
         dtd = ddot(n,d,1,d,1)
         dnorm = sqrt(dtd)
   
!     Determine the maximum step length.
   
         stpmx = big
         if (cnstnd) then
            if (iter .eq. 0) then
               stpmx = one
            else
               do 43 i = 1, n
                  a1 = d(i)
                  if (nbd(i) .ne. 0) then
                     if (a1 .lt. zero .and. nbd(i) .le. 2) then
                        a2 = l(i) - x(i)
                        if (a2 .ge. zero) then
                           stpmx = zero
                        else if (a1*stpmx .lt. a2) then
                           stpmx = a2/a1
                        endif
                     else if (a1 .gt. zero .and. nbd(i) .ge. 2) then
                        a2 = u(i) - x(i)
                        if (a2 .le. zero) then
                           stpmx = zero
                        else if (a1*stpmx .gt. a2) then
                           stpmx = a2/a1
                        endif
                     endif
                  endif
     43        continue
            endif
         endif
    
         if (iter .eq. 0 .and. .not. boxed) then
            stp = min(one/dnorm, stpmx)
         else
            stp = one
         endif 
   
         call dcopy(n,x,1,t,1)
         call dcopy(n,g,1,r,1)
         fold = f
         ifun = 0
         iback = 0
         csave = 'START'
    556  continue
         gd = ddot(n,g,1,d,1)
         if (ifun .eq. 0) then
            gdold=gd
            if (gd .ge. zero) then
!                               the directional derivative >=0.
!                              Line search is impossible.
               write(6,*)' ascent direction in projection gd = ', gd
               info = -4
               return
            endif
         endif
   
         call dcsrch(f,gd,stp,ftol,gtol,xtol,zero,stpmx,csave,isave,dsave)
   
         xstep = stp*dnorm
         if (csave(1:4) .ne. 'CONV' .and. csave(1:4) .ne. 'WARN') then
            task = 'FG_LNSRCH'
            ifun = ifun + 1
            nfgv = nfgv + 1
            iback = ifun - 1 
            if (stp .eq. one) then
               call dcopy(n,z,1,x,1)
            else
               do 41 i = 1, n
                  x(i) = stp*d(i) + t(i)
     41        continue
            endif
         else
            task = 'NEW_X'
         endif
   
         return
   
        end subroutine lnsrlb
   
!===========================================================================    
!                              matupd
!===========================================================================
   
        subroutine matupd(n, m, ws, wy, sy, ss, d, r, itail,& 
                           iupdat, col, head, theta, rr, dr, stp, dtd)
    
        implicit none

         integer          :: n, m, itail, iupdat, col,&
                             head, j,pointr
         double precision :: theta, rr, dr, stp, dtd, d(n),&
                             r(n), ws(n, m), wy(n, m), sy(m, m),&
                             ss(m, m),ddot,one=1.0d0
   
!     Set pointers for matrices WS and WY.
    
         if (iupdat .le. m) then
            col = iupdat
            itail = mod(head+iupdat-2,m) + 1
         else
            itail = mod(itail,m) + 1
            head = mod(head,m) + 1
         endif
    
!    Update matrices WS and WY.
   
         call dcopy(n,d,1,ws(1,itail),1)
         call dcopy(n,r,1,wy(1,itail),1)
    
!     Set theta=yy/ys.
    
         theta = rr/dr
    
!   Form the middle matrix in B.
    
!       update the upper triangle of SS,
!                                        and the lower triangle of SY:
         if (iupdat .gt. m) then
!                             move old information
            do 50 j = 1, col - 1
               call dcopy(j,ss(2,j+1),1,ss(1,j),1)
               call dcopy(col-j,sy(j+1,j+1),1,sy(j,j),1)
     50     continue
         endif
!       add new information: the last row of SY
!                                             and the last column of SS:
         pointr = head
         do 51 j = 1, col - 1
            sy(col,j) = ddot(n,d,1,wy(1,pointr),1)
            ss(j,col) = ddot(n,ws(1,pointr),1,d,1)
            pointr = mod(pointr,m) + 1
     51  continue
         if (stp .eq. one) then
            ss(col,col) = dtd
         else
            ss(col,col) = stp*stp*dtd
         endif
         sy(col,col) = dr
    
         return
   
        end subroutine matupd
   
!===========================================================================    
!                              prn1lb
!===========================================================================
   
        subroutine prn1lb(n, m, l, u, x, iprint, itfile, epsmch)

        implicit none    
    
         integer          :: n, m, iprint, itfile,i
         double precision :: epsmch, x(n), l(n), u(n)
   
         if (iprint .ge. 0) then
            write (6,7001) epsmch
            write (6,*) 'N = ',n,'    M = ',m
            if (iprint .ge. 1) then
               write (itfile,2001) epsmch
               write (itfile,*)'N = ',n,'    M = ',m
               write (itfile,9001)
               if (iprint .gt. 100) then
                  write (6,1004) 'L =',(l(i),i = 1,n)
                  write (6,1004) 'X0 =',(x(i),i = 1,n)
                  write (6,1004) 'U =',(u(i),i = 1,n)
               endif 
            endif
         endif 
   
    1004 format (/,a4, 1p, 6(1x,d11.4),/,(4x,1p,6(1x,d11.4)))
    2001 format ('RUNNING THE L-BFGS-B CODE',/,/,&
         'it    = iteration number',/,&
         'nf    = number of function evaluations',/,&
         'nseg  = number of segments explored during the Cauchy search',/,&
         'nact  = number of active bounds at the generalized Cauchy point'&
         ,/,&
         'sub   = manner in which the subspace minimization terminated:'&
         ,/,'        con = converged, bnd = a bound was reached',/,&
         'itls  = number of iterations performed in the line search',/,&
         'stepl = step length used',/,&
         'tstep = norm of the displacement (total step)',/,&
         'projg = norm of the projected gradient',/,&
         'f     = function value',/,/,&
         '           * * *',/,/,&
         'Machine precision =',1p,d10.3)
    7001 format ('RUNNING THE L-BFGS-B CODE',/,/,&
         '           * * *',/,/,&
         'Machine precision =',1p,d10.3)
    9001 format (/,3x,'it',3x,'nf',2x,'nseg',2x,'nact',2x,'sub',2x,'itls',&
                2x,'stepl',4x,'tstep',5x,'projg',8x,'f')
   
         return
   
        end subroutine prn1lb
   
!===========================================================================    
!                              prn2lb
!===========================================================================
   
        subroutine prn2lb(n, x, f, g, iprint, itfile, iter, nfgv, nact, &
                          sbgnrm, nseg, word, iword, iback, stp, xstep)
    
        implicit none

         character(len=3)   :: word
         integer            :: n, iprint, itfile, iter, nfgv,&
                               iword, iback,i,imod,nact, nseg
         double precision  :: f, sbgnrm, stp, xstep, x(n), g(n)
   
!          'word' records the status of subspace solutions.
         if (iword .eq. 0) then
!                          the subspace minimization converged.
            word = 'con'
         else if (iword .eq. 1) then
!                         the subspace minimization stopped at a bound.
            word = 'bnd'
         else if (iword .eq. 5) then
!                             the truncated Newton step has been used.
            word = 'TNT'
         else
            word = '---'
         endif
         if (iprint .ge. 99) then
            write (6,*) 'LINE SEARCH',iback,' times; norm of step = ',xstep
            write (6,2001) iter,f,sbgnrm
            if (iprint .gt. 100) then      
               write (6,1004) 'X =',(x(i), i = 1, n)
               write (6,1004) 'G =',(g(i), i = 1, n)
            endif
         else if (iprint .gt. 0) then 
            imod = mod(iter,iprint)
            if (imod .eq. 0) write (6,2001) iter,f,sbgnrm
         endif
         if (iprint .ge. 1) write (itfile,3001)&
                 iter,nfgv,nseg,nact,word,iback,stp,xstep,sbgnrm,f
   
    1004 format (/,a4, 1p, 6(1x,d11.4),/,(4x,1p,6(1x,d11.4)))
    2001 format (/,'At iterate',i5,4x,'f= ',1p,d12.5,4x,'|proj g|= ',1p,d12.5)
    3001 format(2(1x,i4),2(1x,i5),2x,a3,1x,i4,1p,2(2x,d7.1),1p,2(1x,d10.3))
   
         return
   
        end subroutine prn2lb
   
!===========================================================================    
!                              prn3lb
!===========================================================================
   
         subroutine prn3lb(n, x, f, task, iprint, info, itfile,& 
                           iter, nfgv, nintol, nskip, nact, sbgnrm,& 
                           time, nseg, word, iback, stp, xstep, k,&
                           cachyt, sbtime, lnscht)
    
         character(len=60)  :: task
         character(len=3)   :: word
         integer            :: n, iprint, info, itfile, iter,& 
                               nskip, nact, nseg, iback, k,&
                               nfgv, nintol,i
         double precision  ::  f, sbgnrm, time, stp, xstep, cachyt,&
                               sbtime, lnscht, x(n)
   
         if (task(1:5) .eq. 'ERROR') goto 999
   
         if (iprint .ge. 0) then
            write (6,3003)
            write (6,3004)
            write(6,3005) n,iter,nfgv,nintol,nskip,nact,sbgnrm,f
            if (iprint .ge. 100) then
               write (6,1004) 'X =',(x(i),i = 1,n)
            endif  
            if (iprint .ge. 1) write (6,*) ' F =',f
         endif 
    999  continue
         if (iprint .ge. 0) then
            write (6,3009) task
            if (info .ne. 0) then
               if (info .eq. -1) write (6,9011)
               if (info .eq. -2) write (6,9012)
               if (info .eq. -3) write (6,9013)
               if (info .eq. -4) write (6,9014)
               if (info .eq. -5) write (6,9015)
               if (info .eq. -6) write (6,*)' Input nbd(',k,') is invalid.'
               if (info .eq. -7) &
               write (6,*)' l(',k,') > u(',k,').  No feasible solution.'
               if (info .eq. -8) write (6,9018)
               if (info .eq. -9) write (6,9019)
            endif
            if (iprint .ge. 1) write (6,3007) cachyt,sbtime,lnscht
            write (6,3008) time
            if (iprint .ge. 1) then
               if (info .eq. -4 .or. info .eq. -9) then
                  write (itfile,3002)&
                     iter,nfgv,nseg,nact,word,iback,stp,xstep
               endif
               write (itfile,3009) task
               if (info .ne. 0) then
                  if (info .eq. -1) write (itfile,9011)
                  if (info .eq. -2) write (itfile,9012)
                  if (info .eq. -3) write (itfile,9013)
                  if (info .eq. -4) write (itfile,9014)
                  if (info .eq. -5) write (itfile,9015)
                  if (info .eq. -8) write (itfile,9018)
                  if (info .eq. -9) write (itfile,9019)
               endif
               write (itfile,3008) time
            endif
         endif
   
    1004 format (/,a4, 1p, 6(1x,d11.4),/,(4x,1p,6(1x,d11.4)))
    3002 format(2(1x,i4),2(1x,i5),2x,a3,1x,i4,1p,2(2x,d7.1),6x,'-',10x,'-')
    3003 format (/,&
         '           * * *',/,/,&
         'Tit   = total number of iterations',/,&
         'Tnf   = total number of function evaluations',/,&
         'Tnint = total number of segments explored during',&
                   ' Cauchy searches',/,&
         'Skip  = number of BFGS updates skipped',/,&
         'Nact  = number of active bounds at final generalized',&
                  ' Cauchy point',/,&
         'Projg = norm of the final projected gradient',/,&
         'F     = final function value',/,/,&
         '           * * *')
    3004 format (/,3x,'N',4x,'Tit',5x,'Tnf',2x,'Tnint',2x,&
              'Skip',2x,'Nact',5x,'Projg',8x,'F')
    3005 format (i5,2(1x,i6),(1x,i6),(2x,i4),(1x,i5),1p,2(2x,d10.3))
    3007 format (/,' Cauchy                time',1p,e10.3,' seconds.',/ &
               ' Subspace minimization time',1p,e10.3,' seconds.',/ &
                ' Line search           time',1p,e10.3,' seconds.')
    3008 format (/,' Total User time',1p,e10.3,' seconds.',/)
    3009 format (/,a60)
    9011 format (/,&
        ' Matrix in 1st Cholesky factorization in formk is not Pos. Def.')
    9012 format (/,&
        ' Matrix in 2st Cholesky factorization in formk is not Pos. Def.')
    9013 format (/,&
        ' Matrix in the Cholesky factorization in formt is not Pos. Def.')
    9014 format (/,&
        ' Derivative >= 0, backtracking line search impossible.',/,&
        '   Previous x, f and g restored.',/,&
        ' Possible causes: 1 error in function or gradient evaluation;',/,&
        '                  2 rounding errors dominate computation.')
    9015 format (/,&
        ' Warning:  more than 10 function and gradient',/,&
        '   evaluations in the last line search.  Termination',/,&
        '   may possibly be caused by a bad search direction.')
    9018 format (/,' The triangular system is singular.')
    9019 format (/,&
        ' Line search cannot locate an adequate point after 20 function',/&
        ,'  and gradient evaluations.  Previous x, f and g restored.',/,&
        ' Possible causes: 1 error in function or gradient evaluation;',/,&
        '                  2 rounding error dominate computation.')
   
         return
   
        end subroutine prn3lb

!===========================================================================    
!                              projgr
!===========================================================================
   
        subroutine projgr(n, l, u, nbd, x, g, sbgnrm)
         
        implicit  none

         integer          :: n, nbd(n),i
         double precision :: sbgnrm, x(n), l(n), u(n),& 
                             gi,zero=0.0d0, g(n) 
   
         sbgnrm = zero
         do 15 i = 1, n
           gi = g(i)
           if (nbd(i) .ne. 0) then
              if (gi .lt. zero) then
                 if (nbd(i) .ge. 2) gi = max((x(i)-u(i)),gi)
              else
                 if (nbd(i) .le. 2) gi = min((x(i)-l(i)),gi)
              endif
           endif
           sbgnrm = max(sbgnrm,abs(gi))
     15  continue
   
         return
   
        end subroutine projgr
   
!===========================================================================    
!                              subsm
!===========================================================================
   
         subroutine subsm ( n, m, nsub, ind, l, u, nbd, x, d, xp, ws, wy,&
                           theta, xx, gg,col, head, iword, wv, wn, iprint, info )
         implicit none
         integer          :: n, m, nsub, col, head, iword, iprint, info,&
                             ind(nsub), nbd(n), pointr,m2,col2,ibd,jy,js,i,j,k
         double precision :: theta, wv(2*m), wn(2*m, 2*m),&
                             l(n), u(n), x(n), d(n), xp(n),&
                             ws(n, m), wy(n, m), one=1.0d0,zero=0.0d0,dd_p,&
                             xx(n), gg(n), alpha, xk, dk, temp1, temp2 
   
         if (nsub .le. 0) return
         if (iprint .ge. 99) write (6,1001)
   
!     Compute wv = W'Zd.
   
         pointr = head 
         do 20 i = 1, col
            temp1 = zero
            temp2 = zero
            do 10 j = 1, nsub
               k = ind(j)
               temp1 = temp1 + wy(k,pointr)*d(j)
               temp2 = temp2 + ws(k,pointr)*d(j)
     10     continue
            wv(i) = temp1
            wv(col + i) = theta*temp2
            pointr = mod(pointr,m) + 1
     20  continue
    
!    Compute wv:=K^(-1)wv.
   
         m2 = 2*m
         col2 = 2*col
         call dtrsl(wn,m2,col2,wv,11,info)
         if (info .ne. 0) return
         do 25 i = 1, col
            wv(i) = -wv(i)
     25     continue
         call dtrsl(wn,m2,col2,wv,01,info)
         if (info .ne. 0) return
    
!    Compute d = (1/theta)d + (1/theta**2)Z'W wv.
    
         pointr = head
         do 40 jy = 1, col
            js = col + jy
            do 30 i = 1, nsub
               k = ind(i)
               d(i) = d(i) + wy(k,pointr)*wv(jy)/theta &   
                         + ws(k,pointr)*wv(js)
     30     continue
            pointr = mod(pointr,m) + 1
     40  continue
   
         call dscal( nsub, one/theta, d, 1 )

!-----------------------------------------------------------------
!     Let us try the projection, d is the Newton direction
   
         iword = 0
   
         call dcopy ( n, x, 1, xp, 1 )
   
         do 50 i=1, nsub
            k  = ind(i)
            dk = d(i)
            xk = x(k)
            if ( nbd(k) .ne. 0 ) then
   
               if ( nbd(k).eq.1 ) then          ! lower bounds only
                  x(k) = max( l(k), xk + dk )
                  if ( x(k).eq.l(k) ) iword = 1
               else 
        
                  if ( nbd(k).eq.2 ) then       ! upper and lower bounds
                     xk   = max( l(k), xk + dk ) 
                     x(k) = min( u(k), xk )
                     if ( x(k).eq.l(k) .or. x(k).eq.u(k) ) iword = 1
                  else
   
                     if ( nbd(k).eq.3 ) then    ! upper bounds only
                        x(k) = min( u(k), xk + dk )
                        if ( x(k).eq.u(k) ) iword = 1
                     end if 
                  end if
               end if
               
            else                                ! free variables
               x(k) = xk + dk
            end if 
    50   continue
   
         if ( iword.eq.0 ) then
            go to 911
         end if
   
!     check sign of the directional derivative
   
         dd_p = zero
         do 55 i=1, n
            dd_p  = dd_p + (x(i) - xx(i))*gg(i)
    55   continue
         if ( dd_p .gt.zero ) then
            call dcopy( n, xp, 1, x, 1 )
            write(6,*) ' Positive dir derivative in projection '
            write(6,*) ' Using the backtracking step '
         else
            go to 911
         endif
   
!-----------------------------------------------------------------
   
         alpha = one
         temp1 = alpha
         ibd   = 0 
         do 60 i = 1, nsub
            k = ind(i)
            dk = d(i)
            if (nbd(k) .ne. 0) then
               if (dk .lt. zero .and. nbd(k) .le. 2) then
                  temp2 = l(k) - x(k)
                  if (temp2 .ge. zero) then
                     temp1 = zero
                  else if (dk*alpha .lt. temp2) then
                     temp1 = temp2/dk
                  endif
               else if (dk .gt. zero .and. nbd(k) .ge. 2) then
                  temp2 = u(k) - x(k)
                  if (temp2 .le. zero) then
                     temp1 = zero
                  else if (dk*alpha .gt. temp2) then
                     temp1 = temp2/dk
                  endif
               endif
               if (temp1 .lt. alpha) then
                  alpha = temp1
                  ibd = i
               endif
            endif
    60   continue
         
         if (alpha .lt. one) then
            dk = d(ibd)
            k = ind(ibd)
            if (dk .gt. zero) then
               x(k) = u(k)
               d(ibd) = zero
            else if (dk .lt. zero) then
               x(k) = l(k)
               d(ibd) = zero
            endif
         endif
         do 70 i = 1, nsub
            k    = ind(i)
            x(k) = x(k) + alpha*d(i)
    70   continue
    911  continue
   
         if (iprint .ge. 99) write (6,1004)
   
    1001 format (/,'----------------SUBSM entered-----------------',/)
    1004 format (/,'----------------exit SUBSM --------------------',/)
   
         return
   
         end subroutine subsm

!===========================================================================    
!                              dcsrch
!===========================================================================
   
         subroutine dcsrch(f,g,stp,ftol,gtol,xtol,stpmin,stpmax,&
                           task,isave,dsave)
        
        implicit none                   
                           
         character(len=*) :: task
         integer          :: isave(2), stage
         double precision :: f,g,stp,ftol,gtol,xtol,stpmin,&
                             stpmax,dsave(13),finit,ftest,fm,&
                             fx,fxm,fy,fym,ginit,gtest,gm,&
                             gx,gxm,gy,gym,stx,sty,stmin,&
                             stmax,width,width1,&
                             zero=0.0d0,p5=0.5d0,p66=0.66d0,&
                             xtrapl=1.1d0,xtrapu=4.0d0
         logical          :: brackt
         
   
!    Initialization block.
   
         if (task(1:5) .eq. 'START') then
   
!       Check the input arguments for errors.
   
            if (stp .lt. stpmin) task = 'ERROR: STP .LT. STPMIN'
            if (stp .gt. stpmax) task = 'ERROR: STP .GT. STPMAX'
            if (g .ge. zero) task = 'ERROR: INITIAL G .GE. ZERO'
            if (ftol .lt. zero) task = 'ERROR: FTOL .LT. ZERO'
            if (gtol .lt. zero) task = 'ERROR: GTOL .LT. ZERO'
            if (xtol .lt. zero) task = 'ERROR: XTOL .LT. ZERO'
            if (stpmin .lt. zero) task = 'ERROR: STPMIN .LT. ZERO'
            if (stpmax .lt. stpmin) task = 'ERROR: STPMAX .LT. STPMIN'
   
!        Exit if there are errors on input.
   
            if (task(1:5) .eq. 'ERROR') return
   
!      Initialize local variables.
   
            brackt = .false.
            stage = 1
            finit = f
            ginit = g
            gtest = ftol*ginit
            width = stpmax - stpmin
            width1 = width/p5
   
!      The variables stx, fx, gx contain the values of the step, 
!       function, and derivative at the best step. 
!      The variables sty, fy, gy contain the value of the step, 
!      function, and derivative at sty.
!      The variables stp, f, g contain the values of the step, 
!       function, and derivative at stp.
   
            stx = zero
            fx = finit
            gx = ginit
            sty = zero
            fy = finit
            gy = ginit
            stmin = zero
            stmax = stp + xtrapu*stp
            task = 'FG'
   
            goto 1000
   
         else
   
!      Restore local variables.
   
            if (isave(1) .eq. 1) then
               brackt = .true.
            else
               brackt = .false.
            endif
            stage = isave(2) 
            ginit = dsave(1) 
            gtest = dsave(2) 
            gx = dsave(3) 
            gy = dsave(4) 
            finit = dsave(5) 
            fx = dsave(6) 
            fy = dsave(7) 
            stx = dsave(8) 
            sty = dsave(9) 
            stmin = dsave(10) 
            stmax = dsave(11) 
            width = dsave(12) 
            width1 = dsave(13) 
   
         endif
   
!    If psi(stp) <= 0 and f'(stp) >= 0 for some step, then the
!    algorithm enters the second stage.
   
         ftest = finit + stp*gtest
         if (stage .eq. 1 .and. f .le. ftest .and. g .ge. zero) stage = 2
   
!    Test for warnings.
   
         if (brackt .and. (stp .le. stmin .or. stp .ge. stmax))&
           task = 'WARNING: ROUNDING ERRORS PREVENT PROGRESS'
         if (brackt .and. stmax - stmin .le. xtol*stmax) &
           task = 'WARNING: XTOL TEST SATISFIED'
         if (stp .eq. stpmax .and. f .le. ftest .and. g .le. gtest) &
           task = 'WARNING: STP = STPMAX'
         if (stp .eq. stpmin .and. (f .gt. ftest .or. g .ge. gtest)) &
          task = 'WARNING: STP = STPMIN'
   
!   Test for convergence.
   
         if (f .le. ftest .and. abs(g) .le. gtol*(-ginit)) &
           task = 'CONVERGENCE'
   
!   Test for termination.
   
         if (task(1:4) .eq. 'WARN' .or. task(1:4) .eq. 'CONV') goto 1000
   
!    A modified function is used to predict the step during the
!    first stage if a lower function value has been obtained but 
!    the decrease is not sufficient.
   
         if (stage .eq. 1 .and. f .le. fx .and. f .gt. ftest) then
   
!       Define the modified function and derivative values.
   
            fm = f - stp*gtest
            fxm = fx - stx*gtest
            fym = fy - sty*gtest
            gm = g - gtest
            gxm = gx - gtest
            gym = gy - gtest
   
!       Call dcstep to update stx, sty, and to compute the new step.
   
            call dcstep(stx,fxm,gxm,sty,fym,gym,stp,fm,gm,&
                        brackt,stmin,stmax)
   
!      Reset the function and derivative values for f.
   
            fx = fxm + stx*gtest
            fy = fym + sty*gtest
            gx = gxm + gtest
            gy = gym + gtest
   
         else
   
!     Call dcstep to update stx, sty, and to compute the new step.
   
           call dcstep(stx,fx,gx,sty,fy,gy,stp,f,g,&
                       brackt,stmin,stmax)
   
         endif
   
!  Decide if a bisection step is needed.
   
         if (brackt) then
            if (abs(sty-stx) .ge. p66*width1) stp = stx + p5*(sty - stx)
            width1 = width
            width = abs(sty-stx)
         endif
   
!   Set the minimum and maximum steps allowed for stp.
   
         if (brackt) then
            stmin = min(stx,sty)
            stmax = max(stx,sty)
         else
            stmin = stp + xtrapl*(stp - stx)
            stmax = stp + xtrapu*(stp - stx)
         endif
    
!    Force the step to be within the bounds stpmax and stpmin.
    
         stp = max(stp,stpmin)
         stp = min(stp,stpmax)
   
!    If further progress is not possible, let stp be the best
!    point obtained during the search.
   
         if (brackt .and. (stp .le. stmin .or. stp .ge. stmax) &
           .or. (brackt .and. stmax-stmin .le. xtol*stmax)) stp = stx
   
!    Obtain another function and derivative.
   
         task = 'FG'
   
    1000 continue
   
!    Save local variables.
   
         if (brackt) then
            isave(1) = 1
         else
            isave(1) = 0
         endif
         isave(2) = stage
         dsave(1) =  ginit
         dsave(2) =  gtest
         dsave(3) =  gx
         dsave(4) =  gy
         dsave(5) =  finit
         dsave(6) =  fx
         dsave(7) =  fy
         dsave(8) =  stx
         dsave(9) =  sty
         dsave(10) = stmin
         dsave(11) = stmax
         dsave(12) = width
         dsave(13) = width1
   
         return
        end subroutine dcsrch
         
!===========================================================================    
!                              dcstep
!===========================================================================
   
         subroutine dcstep(stx,fx,dx,sty,fy,dy,stp,fp,dp,brackt,&
                           stpmin,stpmax)
         implicit none

         logical          :: brackt
         double precision :: stx,fx,dx,sty,fy,dy,stp,fp,dp,&
                             stpmin,stpmax,&
                             gamma,p,q,r,s,sgnd,stpc,stpf,stpq,theta,&
                             zero=0.0d0,p66=0.66d0,two=2.0d0,three=3.0d0
   
         sgnd = dp*(dx/abs(dx))
   
!    First case: A higher function value. The minimum is bracketed. 
!     If the cubic step is closer to stx than the quadratic step, the 
!    cubic step is taken, otherwise the average of the cubic and 
!     quadratic steps is taken.
   
         if (fp .gt. fx) then
            theta = three*(fx - fp)/(stp - stx) + dx + dp
            s = max(abs(theta),abs(dx),abs(dp))
            gamma = s*sqrt((theta/s)**2 - (dx/s)*(dp/s))
            if (stp .lt. stx) gamma = -gamma
            p = (gamma - dx) + theta
            q = ((gamma - dx) + gamma) + dp
            r = p/q
            stpc = stx + r*(stp - stx)
            stpq = stx + ((dx/((fx - fp)/(stp - stx) + dx))/two)*(stp - stx)
            if (abs(stpc-stx) .lt. abs(stpq-stx)) then
               stpf = stpc
            else
               stpf = stpc + (stpq - stpc)/two
            endif
            brackt = .true.
   
!     Second case: A lower function value and derivatives of opposite 
!    sign. The minimum is bracketed. If the cubic step is farther from 
!     stp than the secant step, the cubic step is taken, otherwise the 
!     secant step is taken.
   
         else if (sgnd .lt. zero) then
            theta = three*(fx - fp)/(stp - stx) + dx + dp
            s = max(abs(theta),abs(dx),abs(dp))
            gamma = s*sqrt((theta/s)**2 - (dx/s)*(dp/s))
            if (stp .gt. stx) gamma = -gamma
            p = (gamma - dp) + theta
            q = ((gamma - dp) + gamma) + dx
            r = p/q
            stpc = stp + r*(stx - stp)
            stpq = stp + (dp/(dp - dx))*(stx - stp)
            if (abs(stpc-stp) .gt. abs(stpq-stp)) then
               stpf = stpc
            else
               stpf = stpq
            endif
            brackt = .true.
   
!     Third case: A lower function value, derivatives of the same sign,
!     and the magnitude of the derivative decreases.
   
         else if (abs(dp) .lt. abs(dx)) then
   
!        The cubic step is computed only if the cubic tends to infinity 
!       in the direction of the step or if the minimum of the cubic
!        is beyond stp. Otherwise the cubic step is defined to be the 
!        secant step.
   
            theta = three*(fx - fp)/(stp - stx) + dx + dp
            s = max(abs(theta),abs(dx),abs(dp))
   
!       The case gamma = 0 only arises if the cubic does not tend
!      to infinity in the direction of the step.
   
            gamma = s*sqrt(max(zero,(theta/s)**2-(dx/s)*(dp/s)))
            if (stp .gt. stx) gamma = -gamma
            p = (gamma - dp) + theta
            q = (gamma + (dx - dp)) + gamma
            r = p/q
            if (r .lt. zero .and. gamma .ne. zero) then
               stpc = stp + r*(stx - stp)
            else if (stp .gt. stx) then
               stpc = stpmax
            else
               stpc = stpmin
            endif
            stpq = stp + (dp/(dp - dx))*(stx - stp)
   
            if (brackt) then
   
!          A minimizer has been bracketed. If the cubic step is 
!         closer to stp than the secant step, the cubic step is 
!          taken, otherwise the secant step is taken.
   
               if (abs(stpc-stp) .lt. abs(stpq-stp)) then
                  stpf = stpc
               else
                  stpf = stpq
               endif
               if (stp .gt. stx) then
                  stpf = min(stp+p66*(sty-stp),stpf)
               else
                  stpf = max(stp+p66*(sty-stp),stpf)
               endif
            else
   
!           A minimizer has not been bracketed. If the cubic step is 
!           farther from stp than the secant step, the cubic step is 
!          taken, otherwise the secant step is taken.
   
               if (abs(stpc-stp) .gt. abs(stpq-stp)) then
                  stpf = stpc
               else
                  stpf = stpq
               endif
               stpf = min(stpmax,stpf)
               stpf = max(stpmin,stpf)
            endif
   
!   Fourth case: A lower function value, derivatives of the same sign, 
!    and the magnitude of the derivative does not decrease. If the 
!    minimum is not bracketed, the step is either stpmin or stpmax, 
!  otherwise the cubic step is taken.
   
         else
            if (brackt) then
               theta = three*(fp - fy)/(sty - stp) + dy + dp
               s = max(abs(theta),abs(dy),abs(dp))
               gamma = s*sqrt((theta/s)**2 - (dy/s)*(dp/s))
               if (stp .gt. sty) gamma = -gamma
               p = (gamma - dp) + theta
               q = ((gamma - dp) + gamma) + dy
               r = p/q
               stpc = stp + r*(sty - stp)
               stpf = stpc
            else if (stp .gt. stx) then
               stpf = stpmax
            else
               stpf = stpmin
            endif
         endif
   
!Update the interval which contains a minimizer.
   
         if (fp .gt. fx) then
            sty = stp
            fy = fp
            dy = dp
         else
            if (sgnd .lt. zero) then
               sty = stx
               fy = fx
               dy = dx
            endif
            stx = stp
            fx = fp
            dx = dp
         endif
   
!Compute the new step.
   
         stp = stpf
   
         return
        end subroutine dcstep

!===========================================================================  
!                           dpofa
!===========================================================================

    subroutine dpofa(a,lda,n,info)
        integer lda,n,info
        double precision a(lda,*)

!     dpofa factors a double precision symmetric positive definite
!     matrix.
!
!     dpofa is usually called by dpoco, but it can be called
!     directly with a saving in time if  rcond  is not needed.
!     (time for dpoco) = (1 + 18/n)*(time for dpofa) .
!
        double precision ddot,t
        double precision s
        integer j,jm1,k

           do 30 j = 1, n
              info = j
              s = 0.0d0
              jm1 = j - 1
              if (jm1 .lt. 1) go to 20
              do 10 k = 1, jm1
                 t = a(k,j) - ddot(k-1,a(1,k),1,a(1,j),1)
                 t = t/a(k,k)
                 a(k,j) = t
                 s = s + t*t
     10       continue
     20       continue
              s = a(j,j) - s

              if (s .le. 0.0d0) go to 40
              a(j,j) = sqrt(s)
     30    continue
           info = 0
     40 continue
        return
    end subroutine dpofa
        
  
!===========================================================================  
!                           dtrsl
!===========================================================================

    subroutine dtrsl(t,ldt,n,b,job,info)
        integer ldt,n,job,info
        double precision t(ldt,*),b(*)
 
!     dtrsl solves systems of the form
!
!                   t * x = b
!    or
!                   trans(t) * x = b
!
!     where t is a triangular matrix of order n. here trans(t)
!     denotes the transpose of the matrix t.

        double precision ddot,temp
        integer case,j,jj

        do 10 info = 1, n
              if (t(info,info) .eq. 0.0d0) go to 150
     10    continue
           info = 0
  
!       determine the task and go to it.

           case = 1
           if (mod(job,10) .ne. 0) case = 2
           if (mod(job,100)/10 .ne. 0) case = case + 2
           go to (20,50,80,110), case
!
!        solve t*x=b for t lower triangular
!
     20    continue
              b(1) = b(1)/t(1,1)
              if (n .lt. 2) go to 40
              do 30 j = 2, n
                 temp = -b(j-1)
                 call daxpy(n-j+1,temp,t(j,j-1),1,b(j),1)
                 b(j) = b(j)/t(j,j)
     30       continue
     40       continue
           go to 140
!
!        solve t*x=b for t upper triangular.
!
     50    continue
              b(n) = b(n)/t(n,n)
              if (n .lt. 2) go to 70
              do 60 jj = 2, n
                 j = n - jj + 1
                 temp = -b(j+1)
                 call daxpy(j,temp,t(1,j+1),1,b(1),1)
                 b(j) = b(j)/t(j,j)
     60       continue
     70       continue
           go to 140
!
!        solve trans(t)*x=b for t lower triangular.
!
     80    continue
              b(n) = b(n)/t(n,n)
              if (n .lt. 2) go to 100
              do 90 jj = 2, n
                 j = n - jj + 1
                 b(j) = b(j) - ddot(jj-1,t(j+1,j),1,b(j+1),1)
                 b(j) = b(j)/t(j,j)
     90       continue
    100       continue
           go to 140
!
!        solve trans(t)*x=b for t upper triangular.
!
    110    continue
              b(1) = b(1)/t(1,1)
              if (n .lt. 2) go to 130
              do 120 j = 2, n
                 b(j) = b(j) - ddot(j-1,t(1,j),1,b(1),1)
                 b(j) = b(j)/t(j,j)
    120       continue
    130       continue
    140    continue
    150 continue
        return
              
    end subroutine dtrsl     

!===========================================================================  
!                           timer
!===========================================================================

    subroutine timer(ttime)
    double precision ttime
    
    real temp
    
!     This routine computes cpu time in double precision; it makes use of 
!     the intrinsic f90 cpu_time therefore a conversion type is
!     needed.

     temp = sngl(ttime)
     call cpu_time(temp)
     ttime = dble(temp) 
    
     return
    
    end subroutine timer  
    

!===========================================================================  
!                           DRIVER1 
!===========================================================================
!
!        This is the simplest driver in the package. It uses all the
!                    default settings of the code.

    subroutine driver1
     
        implicit none
!
!     Declare variables and parameters needed by the code.
!     Note thar we wish to have output at every iteration. 
!     iprint=1
!
!     We also specify the tolerances in the stopping criteria.
!     factr  = 1.0d+7, pgtol  = 1.0d-5 
!
      integer,  parameter    :: n = 25, m = 5, iprint = 1
      integer,  parameter    :: dp = kind(1.0d0)
      real(dp), parameter    :: factr  = 1.0d+7, pgtol  = 1.0d-5
!
      character(len=60)      :: task, csave
      logical                :: lsave(4)
      integer                :: isave(44)
      real(dp)               :: f
      real(dp)               :: dsave(29)
      integer,  allocatable  :: nbd(:), iwa(:)
      real(dp), allocatable  :: x(:), l(:), u(:), g(:), wa(:)

!     Declare a few additional variables for this sample problem

      real(dp)               :: t1, t2
      integer                :: i

!     Allocate dynamic arrays

      allocate ( nbd(n), x(n), l(n), u(n), g(n) )
      allocate ( iwa(3*n) )
      allocate ( wa(2*m*n + 5*n + 11*m*m + 8*m) )
! 
      do 10 i=1, n, 2
         nbd(i) = 2
         l(i)   = 1.0d0
         u(i)   = 1.0d2
  10  continue

!     Next set bounds on the even-numbered variables.

      do 12 i=2, n, 2
         nbd(i) =  2
         l(i)   = -1.0d2
         u(i)   =  1.0d2
  12   continue

!     We now define the starting point.

      do 14 i=1, n
         x(i) = 3.0d0
  14  continue
 
      write (6,16)
  16  format(/,5x, 'Solving sample problem.', &
             /,5x, ' (f = 0.0 at the optimal solution.)',/)               

!     We start the iteration by initializing task.
 
      task = 'START'

!     The beginning of the loop
 
      do while(task(1:2).eq.'FG'.or.task.eq.'NEW_X'.or. &
               task.eq.'START') 
         
!     This is the call to the L-BFGS-B code.
         
         call setulb ( n, m, x, l, u, nbd, f, g, factr, pgtol, &
                       wa, iwa, task, iprint,&
                       csave, lsave, isave, dsave )
         
         if (task(1:2) .eq. 'FG') then

            f=.25d0*( x(1)-1.d0 )**2
            do 20 i=2, n
               f = f + ( x(i)-x(i-1 )**2 )**2
 20         continue
            f = 4.d0*f

!     Compute gradient g for the sample problem.

            t1   = x(2) - x(1)**2
            g(1) = 2.d0*(x(1) - 1.d0) - 1.6d1*x(1)*t1
            do 22 i=2, n-1
               t2   = t1
               t1   = x(i+1) - x(i)**2
               g(i) = 8.d0*t2 - 1.6d1*x(i)*t1
 22         continue
            g(n) = 8.d0*t1

         end if
      end do

!     end of loop do while

      return
    end subroutine driver1

!===========================================================================    
!                              DRIVER 2 
!===========================================================================
!
!    This driver illustrates how to control the termination of the
!            run and how to design customized output.

    subroutine driver2
    
    implicit none
 
!     Declare variables and parameters needed by the code.
!
!     Note that we suppress the default output (iprint = -1)
!     We suppress both code-supplied stopping tests because the
!     user is providing his/her own stopping criteria.
 
      integer,  parameter    :: n = 25, m = 5, iprint = -1
      integer,  parameter    :: dp = kind(1.0d0)
      real(dp), parameter    :: factr  = 0.0d0, pgtol  = 0.0d0

      character(len=60)      :: task, csave
      logical                :: lsave(4)
      integer                :: isave(44)
      real(dp)               :: f
      real(dp)               :: dsave(29)
      integer,  allocatable  :: nbd(:), iwa(:)
      real(dp), allocatable  :: x(:), l(:), u(:), g(:), wa(:)
!
      real(dp)               :: t1, t2
      integer                :: i

      allocate ( nbd(n), x(n), l(n), u(n), g(n) )
      allocate ( iwa(3*n) )
      allocate ( wa(2*m*n + 5*n + 11*m*m + 8*m) )
!
!    First set bounds on the odd numbered variables.

      do 10 i=1, n,2
        nbd(i)=2
        l(i)=1.0d0
        u(i)=1.0d2
 10  continue

!    Next set bounds on the even numbered variables.

     do 12 i=2, n,2
        nbd(i)=2
        l(i)=-1.0d2
        u(i)=1.0d2
 12   continue

!     We now define the starting point.

     do 14 i=1, n
        x(i)=3.0d0
 14  continue

!     We now write the heading of the output.

     write (6,16)
 16  format(/,5x, 'Solving sample problem.', &
            /,5x, ' (f = 0.0 at the optimal solution.)',/)               
              

!     We start the iteration by initializing task.
! 
     task = 'START'

!        ------- the beginning of the loop ----------

     do while( task(1:2).eq.'FG'.or.task.eq.'NEW_X'.or. &
               task.eq.'START') 
     
!     This is the call to the L-BFGS-B code.

        call setulb(n,m,x,l,u,nbd,f,g,factr,pgtol,wa,iwa,task,iprint, &
                    csave,lsave,isave,dsave)

        if (task(1:2) .eq. 'FG') then

!        the minimization routine has returned to request the
!        function f and gradient g values at the current x.

!        Compute function value f for the sample problem.

           f =.25d0*(x(1) - 1.d0)**2
           do 20 i=2,n
              f = f + (x(i) - x(i-1)**2)**2
 20        continue
           f = 4.d0*f

!        Compute gradient g for the sample problem.

           t1 = x(2) - x(1)**2
           g(1) = 2.d0*(x(1) - 1.d0) - 1.6d1*x(1)*t1
           do 22 i= 2,n-1
              t2 = t1
              t1 = x(i+1) - x(i)**2
              g(i) = 8.d0*t2 - 1.6d1*x(i)*t1
 22        continue
           g(n)=8.d0*t1
!
     else
!
        if (task(1:5) .eq. 'NEW_X') then   
!    
!       the minimization routine has returned with a new iterate.
!       At this point have the opportunity of stopping the iteration 
!       or observing the values of certain parameters
!
!       First are two examples of stopping tests.

!       Note: task(1:4) must be assigned the value 'STOP' to terminate  
!         the iteration and ensure that the final results are
!         printed in the default format. The rest of the character
!         string TASK may be used to store other information.

!       1) Terminate if the total number of f and g evaluations
!            exceeds 99.

            if (isave(34) .ge. 99)  &
               task='STOP: TOTAL NO. of f AND g EVALUATIONS EXCEEDS LIMIT'

!       2) Terminate if  |proj g|/(1+|f|) < 1.0d-10, where 
!          "proj g" denoted the projected gradient

            if (dsave(13) .le. 1.d-10*(1.0d0 + abs(f))) &
              task='STOP: THE PROJECTED GRADIENT IS SUFFICIENTLY SMALL'

!       We now wish to print the following information at each
!       iteration:
!       
!         1) the current iteration number, isave(30),
!         2) the total number of f and g evaluations, isave(34),
!         3) the value of the objective function f,
!         4) the norm of the projected gradient,  dsve(13)
!
!       See the comments at the end of driver1 for a description
!       of the variables isave and dsave.
        
           write (6,'(2(a,i5,4x),a,1p,d12.5,4x,a,1p,d12.5)') 'Iterate' &
              , isave(30),'nfg =',isave(34),'f =',f,'|proj g| =',dsave(13)

!        If the run is to be terminated, we print also the information
!        contained in task as well as the final value of x.

           if (task(1:4) .eq. 'STOP') then
              write (6,*) task  
              write (6,*) 'Final X='
              write (6,'((1x,1p, 6(1x,d11.4)))') (x(i),i = 1,n)
           end if

       end if
     end if

     end do
!           ---------- the end of the loop -------------

!     If task is neither FG nor NEW_X we terminate execution.
      
     return
    end subroutine driver2

!===========================================================================    
!                              DRIVER 3
!===========================================================================
   
    subroutine driver3
        
    implicit none

!     We specify a limit on the CPU time (tlimit = 10 seconds)
!
!     We suppress the default output (iprint = -1). The user could 
!     also elect to use the default output by choosing iprint >= 0.)
!     We suppress the code-supplied stopping tests because we will
!     provide our own termination conditions
!     We specify the dimension n of the sample problem and the number
!        m of limited memory corrections stored. 
        
        integer,  parameter    :: n = 1000, m = 10, iprint = -1
        integer,  parameter    :: dp = kind(1.0d0)
        real(dp), parameter    :: factr  = 0.0d0, pgtol  = 0.0d0, &
                                        tlimit = 10.0d0
!
        character(len=60)      :: task, csave
        logical                :: lsave(4)
        integer                :: isave(44)
        real(dp)               :: f
        real(dp)               :: dsave(29)
        integer,  allocatable  :: nbd(:), iwa(:)
        real(dp), allocatable  :: x(:), l(:), u(:), g(:), wa(:)
!
        real(dp)               :: t1, t2, time1, time2
        integer                :: i, j
  
        allocate ( nbd(n), x(n), l(n), u(n), g(n) )
        allocate ( iwa(3*n) )
        allocate ( wa(2*m*n + 5*n + 11*m*m + 8*m) )
        
!     This time-controlled driver shows that it is possible to terminate
!     a run by elapsed CPU time, and yet be able to print all desired
!     information. This driver also illustrates the use of two
!     stopping criteria that may be used in conjunction with a limit
!     on execution time. The sample problem used here is the same as in 
!     driver1 and driver2 (the extended Rosenbrock function with bounds 
!     on the variables).

!     We now specify nbd which defines the bounds on the variables:
!                    l   specifies the lower bounds,
!                    u   specifies the upper bounds. 
         
!     First set bounds on the odd-numbered variables.        
        
        do 10 i=1, n,2
            nbd(i)=2
            l(i)=1.0d0
            u(i)=1.0d2
        10  continue
   
!     Next set bounds on the even-numbered variables.
        
        do 12 i=2, n,2
            nbd(i)=2
            l(i)=-1.0d2
            u(i)=1.0d2
        12   continue
        
!     We now define the starting point.
        
        do 14 i=1, n
            x(i)=3.0d0
        14  continue
         
!     We now write the heading of the output.
        
        write (6,16)
        16  format(/,5x, 'Solving sample problem.',&
                     /,5x, ' (f = 0.0 at the optimal solution.)',/) 
        
!     We start the iteration by initializing task.
         
        task = 'START'
        
!        ------- the beginning of the loop ----------
        
!     We begin counting the CPU time.
        
        call timer(time1)
        
        do while( task(1:2).eq.'FG'.or.task.eq.'NEW_X'.or. &
                        task.eq.'START')
              
!     This is the call to the L-BFGS-B code.
         
        call setulb(n,m,x,l,u,nbd,f,g,factr,pgtol,wa,iwa, &
                             task,iprint, csave,lsave,isave,dsave)
         
        if (task(1:2) .eq. 'FG') then
        
!        the minimization routine has returned to request the
!        function f and gradient g values at the current x.
!        Before evaluating f and g we check the CPU time spent.
        
        call timer(time2)
        if (time2-time1 .gt. tlimit) then
        task='STOP: CPU EXCEEDING THE TIME LIMIT.'
        
!          Note: Assigning task(1:4)='STOP' will terminate the run;
!          setting task(7:9)='CPU' will restore the information at
!          the latest iterate generated by the code so that it can
!          be correctly printed by the driver.
        
!          In this driver we have chosen to disable the
!          printing options of the code (we set iprint=-1);
!          instead we are using customized output: we print the
!          latest value of x, the corresponding function value f and
!          the norm of the projected gradient |proj g|.
        
!          We print out the information contained in task.
        
        write (6,*) task
        
!          We print the latest iterate contained in wa(j+1:j+n), where
         
        j = 3*n+2*m*n+11*m**2
        write (6,*) 'Latest iterate X ='
        write (6,'((1x,1p, 6(1x,d11.4)))') (wa(i),i = j+1,j+n) 
        
!          We print the function value f and the norm of the projected
!          gradient |proj g| at the last iterate; they are stored in
!          dsave(2) and dsave(13) respectively.
        
        write (6,'(a,1p,d12.5,4x,a,1p,d12.5)') &
               'At latest iterate   f =',dsave(2),'|proj g| =',dsave(13)
        else
        
!          The time limit has not been reached and we compute
!          the function value f for the sample problem.
        
            f=.25d0*(x(1)-1.d0)**2
            do 20 i=2, n
            f=f+(x(i)-x(i-1)**2)**2
          20  continue
            f=4.d0*f
        
!          Compute gradient g for the sample problem.
        
            t1 = x(2) - x(1)**2
            g(1) = 2.d0*(x(1)-1.d0)-1.6d1*x(1)*t1
                do 22 i=2,n-1
                    t2=t1
                    t1=x(i+1)-x(i)**2
                    g(i)=8.d0*t2-1.6d1*x(i)*t1
          22   continue
                g(n)=8.d0*t1
            endif
        
!          go back to the minimization routine.
            else
        
        if (task(1:5) .eq. 'NEW_X') then        
        
!        the minimization routine has returned with a new iterate.
!        The time limit has not been reached, and we test whether
!        the following two stopping tests are satisfied:
        
!        1) Terminate if the total number of f and g evaluations
!             exceeds 900.
        
        if (isave(34) .ge. 900) &
        task='STOP: TOTAL NO. of f AND g EVALUATIONS EXCEEDS LIMIT'
        
!        2) Terminate if  |proj g|/(1+|f|) < 1.0d-10.
        
        if (dsave(13) .le. 1.d-10*(1.0d0 + abs(f))) &
            task='STOP: THE PROJECTED GRADIENT IS SUFFICIENTLY SMALL'
        
!        We wish to print the following information at each iteration:
!          1) the current iteration number, isave(30),
!          2) the total number of f and g evaluations, isave(34),
!          3) the value of the objective function f,
!          4) the norm of the projected gradient,  dsve(13)
!
!        See the comments at the end of driver1 for a description
!        of the variables isave and dsave.
                 
            write (6,'(2(a,i5,4x),a,1p,d12.5,4x,a,1p,d12.5)') 'Iterate' &
                    ,isave(30),'nfg =',isave(34),'f =',f,'|proj g| =',dsave(13)
        
!        If the run is to be terminated, we print also the information
!        contained in task as well as the final value of x.
        
        if (task(1:4) .eq. 'STOP') then
            write (6,*) task  
            write (6,*) 'Final X='
            write (6,'((1x,1p, 6(1x,d11.4)))') (x(i),i = 1,n)
        endif
        
        endif
         end if 
            end do
         
        !     If task is neither FG nor NEW_X we terminate execution.
              
        return
    end subroutine driver3

    end module LBFGSB_optimizer
