/***********************************************************************
* library for allowing fortran77 to dynamically allocate and deallocate
* arrays and access them via C functions. it would be ideal if the
* compiler inlined the get* functions...
* by Andreas Markmann, 2002
* Version 2, changed long long to struct{ 2 x int }
* getlli now procedural rather than functional
* changed getchar to getcharacter to avoid name collision
*
*  C-routine dynaccess
* 
***********************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/***********************************************************************
* Cast C names to FORTRAN names. Depends on the machine one uses.
***********************************************************************/

#if (F2C_STYLE == 1)
# define ALLOC alloc
# define RESIZE resize
# define DEALLOC dealloc
# define SETBLOCK setblock
# define GETBLOCK getblock
# define SETCHARACTER setcharacter
# define GETCHARACTER getcharacter
# define SETSHORT setshort
# define GETSHORT getshort
# define SETLONG setlong
# define GETLONG getlong
# define SETLLI setlli
# define GETLLI getlli
# define SETFLOAT setfloat
# define GETFLOAT getfloat
# define SETDOUBLE setdouble
# define GETDOUBLE getdouble
# define SETSCPX setscpx
# define GETSCPX getscpx
# define SETDCPX setdcpx
# define GETDCPX getdcpx
# define NEWSTRING newstring
# define RESIZESTRING resizestring
# define FREESTRING freestring
# define RELEASESTRING releasestring
# define SETSTRING setstring
# define GETSTRING getstring
#elif (F2C_STYLE == 2)
# define ALLOC ALLOC
# define RESIZE RESIZE
# define DEALLOC DEALLOC
# define SETBLOCK SETBLOCK
# define GETBLOCK GETBLOCK
# define SETCHARACTER SETCHARACTER
# define GETCHARACTER GETCHARACTER
# define SETSHORT SETSHORT
# define GETSHORT GETSHORT
# define SETLONG SETLONG
# define GETLONG GETLONG
# define SETLLI SETLLI
# define GETLLI GETLLI
# define SETFLOAT SETFLOAT
# define GETFLOAT GETFLOAT
# define SETDOUBLE SETDOUBLE
# define GETDOUBLE GETDOUBLE
# define SETSCPX SETSCPX
# define GETSCPX GETSCPX
# define SETDCPX SETDCPX
# define GETDCPX GETDCPX
# define NEWSTRING NEWSTRING
# define RESIZESTRING RESIZESTRING
# define FREESTRING FREESTRING
# define RELEASESTRING RELEASESTRING
# define SETSTRING SETSTRING
# define GETSTRING GETSTRING
#else
# define ALLOC alloc_
# define RESIZE resize_
# define DEALLOC dealloc_
# define SETBLOCK setblock_
# define GETBLOCK getblock_
# define SETCHARACTER setcharacter_
# define GETCHARACTER getcharacter_
# define SETSHORT setshort_
# define GETSHORT getshort_
# define SETLONG setlong_
# define GETLONG getlong_
# define SETLLI setlli_
# define GETLLI getlli_
# define SETFLOAT setfloat_
# define GETFLOAT getfloat_
# define SETDOUBLE setdouble_
# define GETDOUBLE getdouble_
# define SETSCPX setscpx_
# define GETSCPX getscpx_
# define SETDCPX setdcpx_
# define GETDCPX getdcpx_
# define NEWSTRING newstring_
# define RESIZESTRING resizestring_
# define FREESTRING freestring_
# define RELEASESTRING releasestring_
# define SETSTRING setstring_
# define GETSTRING getstring_
#endif

/***********************************************************************
* dynamically allocate arrays of length "bytes",
* return pointer casted to integer*8
***********************************************************************/

void ALLOC (void **pointer, int *bytes)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "ALLOC: asked for %i bytes.\n", *bytes);
#endif
  if ((*pointer = malloc(*bytes)) == NULL) {
    printf("ALLOC: failed allocating %i bytes.\n", *bytes);
    exit(1);
  }
#ifdef DYN_DEBUG
  fprintf(stderr, "ALLOC: allocated at %i.\n", *pointer);
#endif
}

/***********************************************************************
* resize a dynamically allocated array to length "bytes"
***********************************************************************/

void RESIZE (void **pointer, int *bytes)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "RESIZE: pointer %i.\n", *pointer);
#endif
  if ((*pointer = realloc(*pointer,*bytes)) == NULL) {
    printf("RESIZE: failed allocating %i bytes.\n", *bytes);
    exit(1);
  }
#ifdef DYN_DEBUG
  fprintf(stderr, "RESIZE: allocated at %i.\n", *pointer);
#endif
}

/***********************************************************************
* the dealloc routine
***********************************************************************/

void DEALLOC (void **pointer)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "DEALLOC: pointer %i.\n", *pointer);
#endif
  if ( *pointer == NULL) {
    printf("DEALLOC: invalid pointer.\n");
    exit(1);
  }
  free(*pointer);
}

/***********************************************************************
* set several bytes starting from "index" of array pointer to byte
* value inpval
***********************************************************************/

void SETBLOCK (char *inpval, char **pointer, int *index, int *buflen)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "SETBLOCK: pointer %i, element %i, inpval %i.\n",
    *pointer, *index, *inpval);
#endif
  memcpy((*pointer)+(*index), inpval, *buflen);
}

/***********************************************************************
* get byte element "index" from array pointer into outval
***********************************************************************/

void GETBLOCK (char *outval, char **pointer, int *index, int *buflen)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "GETBLOCK: pointer %i, element %i.\n",
    *pointer, *index);
#endif
  memcpy(outval, (*pointer)+(*index), *buflen);
}

/***********************************************************************
* set element "index" of array pointer to byte value inpval
***********************************************************************/

void SETCHARACTER (char *inpval, char **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "SETCHAR: pointer %i, element %i, inpval %i.\n",
    *pointer, *index, *inpval);
#endif
  (*pointer)[*index-1] = *inpval;
}

/***********************************************************************
* get byte element "index" from array pointer into outval
***********************************************************************/

char GETCHARACTER (char **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "GETCHAR: pointer %i, element %i.\n",
    *pointer, *index);
#endif
  return (*pointer)[*index-1];
}

/***********************************************************************
* same for word
***********************************************************************/

void SETSHORT (short int *inpval, short int **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "SETSHORT: pointer %i, element %i, inpval %i.\n",
    *pointer, *index, *inpval);
#endif
  (*pointer)[*index-1] = *inpval;
}

short int GETSHORT (short int **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "GETSHORT: pointer %i, element %i.\n",
    *pointer, *index);
#endif
  return (*pointer)[*index-1];
}

/***********************************************************************
* set element "index" of array pointer to integer value inpval
***********************************************************************/

void SETLONG (int *inpval, int **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "SETLONG: pointer %i, element %i, inpval %i.\n",
    *pointer, *index, *inpval);
#endif
  (*pointer)[*index-1] = *inpval;
}

/***********************************************************************
* get integer element "index" from array pointer into outval
***********************************************************************/

int GETLONG (int **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "GETLONG: pointer %i, element %i.\n",
    *pointer, *index);
#endif
  return (*pointer)[*index-1];
}

/***********************************************************************
* same for long long *** this is also used for pointers ***
***********************************************************************/

typedef struct { int one, two; } tint8;

void SETLLI (tint8 *inpval, tint8 **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "SETLLI: pointer %i, element %i, inpval %i.\n",
    *pointer, *index, *inpval);
#endif
  (*pointer)[*index-1] = *inpval;
}

void GETLLI (tint8 *outval, tint8 **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "GETLLI: pointer %i, element %i.\n",
    *pointer, *index);
#endif
  *outval = (*pointer)[*index-1];
}

/***********************************************************************
* set element "index" of array pointer to real*4 value inpval
***********************************************************************/

void SETFLOAT (float *inpval, float **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "SETFLOAT: pointer %i, element %i, inpval %f.\n",
    *pointer, *index, *inpval);
#endif
  (*pointer)[*index-1] = *inpval;
}

/***********************************************************************
* get real*4 element "index" from array pointer into outval
***********************************************************************/

float GETFLOAT (float **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "GETFLOAT: pointer %i, element %i.\n",
    *pointer, *index);
#endif
  return (*pointer)[*index-1];
}

/***********************************************************************
* set element "index" of array pointer to double precision value inpval
***********************************************************************/

void SETDOUBLE (double *inpval, double **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "SETDOUBLE: pointer %i, element %i, inpval %f.\n",
    *pointer, *index, *inpval);
#endif
  (*pointer)[*index-1] = *inpval;
}

/***********************************************************************
* get double precision element "index" from array pointer into outval
***********************************************************************/

double GETDOUBLE (double **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "GETDOUBLE: pointer %i, element %i.\n",
    *pointer, *index);
#endif
  return (*pointer)[*index-1];
}

/***********************************************************************
* set element "index" of array pointer to single complex value inpval
***********************************************************************/

typedef struct { float re, im; } tscpx;

void SETSCPX (tscpx *inpval, tscpx **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "SETSCPX: pointer %i, element %i, inpval (%f,%f).\n",
    *pointer, *index, inpval->re, inpval->im);
#endif
  (*pointer)[*index-1] = *inpval;
}

/***********************************************************************
* get single complex element "index" from array pointer into outval
***********************************************************************/

void GETSCPX (tscpx *outval, tscpx **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "GETSCPX: pointer %i, element %i.\n",
    *pointer, *index);
#endif
  *outval = (*pointer)[*index-1];
}

/***********************************************************************
* set element "index" of array pointer to double complex value inpval
***********************************************************************/

typedef struct { double re, im; } tdcpx;

void SETDCPX (tdcpx *inpval, tdcpx **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "SETDCPX: pointer %i, element %i, inpval (%f,%f), inpval ptr %i.\n",
    *pointer, *index, inpval->re, inpval->im, inpval);
#endif
  (*pointer)[*index-1] = *inpval;
#ifdef DYN_DEBUG
  fprintf(stderr, "SETDCPX: array element is now (%f,%f).\n",
    (*pointer)[*index-1].re, (*pointer)[*index-1].im);
#endif
}

/***********************************************************************
* get double complex element "index" from array pointer into outval
***********************************************************************/

void GETDCPX (tdcpx *outval, tdcpx **pointer, int *index)
{
#ifdef DYN_DEBUG
  fprintf(stderr, "GETDCPX: pointer %i, element %i.\n",
    *pointer, *index);
#endif
  *outval = (*pointer)[*index-1];
}

/***********************************************************************
* dynamically allocate an array holding pointers to num strings
***********************************************************************/

void NEWSTRING (char ***pointer, int *num)
{
  int i;

#ifdef DYN_DEBUG
  fprintf(stderr, "NEWSTRING: asked for %i strings.\n", *num);
#endif
  if ((*pointer = malloc((*num+1)*sizeof(void *))) == NULL) {
    printf("NEWSTRING: failed allocating %i strings.\n", *num);
    exit(1);
  }
#ifdef DYN_DEBUG
  fprintf(stderr, "Now filling with zeroes...\n");
#endif
  for(i = 0; i <= *num; i++)
    (*pointer)[i] = NULL;
#ifdef DYN_DEBUG
  for(i = 0; i <= *num; i++)
    fprintf(stderr, "%c", ((*pointer)[i] == NULL) ? '0' : '1');
  fprintf(stderr, "\nNEWSTRING: allocated at %i.\n", *pointer);
#endif
}

/***********************************************************************
* resize a dynamically allocated array which was capable of holding old 
* pointers strings to now hold num pointers to strings
***********************************************************************/

void RESIZESTRING (char ***pointer, int *num, int *old)
{
  int i;

#ifdef DYN_DEBUG
  fprintf(stderr, "RESIZESTRING: pointer %i.\n", *pointer);
#endif
  if ((*pointer = realloc(*pointer,(*num+1)*sizeof(void *))) == NULL) {
    printf("RESIZESTRING: failed allocating %i strings.\n", *num);
    exit(1);
  }
#ifdef DYN_DEBUG
  fprintf(stderr, "\nRESIZESTRING: allocated at %i.\n", *pointer);
  fprintf(stderr, "Now filling with zeroes...\n");
#endif
  for(i = *old; i <= *num; i++)
    (*pointer)[i] = NULL;
#ifdef DYN_DEBUG
  for(i = 0; i <= *num; i++)
    fprintf(stderr, "%c", ((*pointer)[i] == NULL) ? '0' : '1');
#endif
}

/***********************************************************************
* free an array holding strings and all its string pointers
***********************************************************************/

void FREESTRING (char ***pointer)
{
  int i;

#ifdef DYN_DEBUG
  fprintf(stderr, "NULL is %li.\n", NULL);
  fprintf(stderr, "FREESTRING: pointer %i.\n", *pointer);
#endif
  if ( *pointer == NULL) {
    printf("FREESTRING: invalid pointer.\n");
    exit(1);
  }
  for(i = 0; (*pointer)[i] != NULL; i++)
  {
#ifdef DYN_DEBUG
    fprintf(stderr, "freeing #%i, ptr %i.\n", i, (*pointer)[i]);
#endif
    free((*pointer)[i]);
  }
#ifdef DYN_DEBUG
  fprintf(stderr, "now freeing main ptr %i.\n", *pointer);
#endif
  free(*pointer);
}

/***********************************************************************
* free an array holding strings and the first num string pointers
***********************************************************************/

void RELEASESTRING (char ***pointer, int *num)
{
  int i;

#ifdef DYN_DEBUG
  fprintf(stderr, "RELEASESTRING: pointer %i.\n", *pointer);
#endif
  if ( *pointer == NULL) {
    printf("RELEASESTRING: invalid pointer.\n");
    exit(1);
  }
  for(i = 0; i < *num; i++)
  {
#ifdef DYN_DEBUG
    fprintf(stderr, "freeing #%i, ptr %i.\n", i, (*pointer)[i]);
#endif
    free((*pointer)[i]);
  }
#ifdef DYN_DEBUG
  fprintf(stderr, "now freeing main ptr %i.\n", *pointer);
#endif
  free(*pointer);
}

/***********************************************************************
* function for printing debug info in what follows
***********************************************************************/

#ifdef DYN_DEBUG
void debuginfo(char *name, char *val, char ***pointer, int *index, int buflen)
{
  const int len=256;
  char format[len];
  int actlen;

  for(actlen=buflen-1; val[actlen] == ' '; actlen--)
    ;
  sprintf(format,
    "%%s: ptr %%i, elem %%i, val \"%%%i.%is\", buf %%i.\n",
    actlen, actlen);
#ifdef DYN_META_DEBUG
  fprintf(stderr, "%s", format);
#endif
  fprintf(stderr, format, name, *pointer, *index, val, buflen);
}
#endif

/***********************************************************************
* allocate array element "index" and copy string inpval into it
* buflen is the length of inpval, provided implicitly by fortran77
***********************************************************************/

void SETSTRING (char *inpval, char ***pointer, int *index, int buflen)
{
#ifdef DYN_DEBUG
  debuginfo("SETSTRING", inpval, pointer, index, buflen);
#endif
  if((*pointer)[*index-1] != NULL) 
  {
#ifdef DYN_DEBUG
    fprintf(stderr, "freeing #%i, ptr %i.\n",
      *index-1, (*pointer)[*index-1]);
#endif
    free((*pointer)[*index-1]);
  }
  (*pointer)[*index-1] = NULL;
  if(((*pointer)[*index-1] = malloc(buflen+1)) == NULL)
  {
    printf("SETSTRING: failed allocating %i characters.\n", buflen+1);
    exit(1);
  }
#ifdef DYN_DEBUG
    fprintf(stderr, "allocated #%i, ptr %i.\n",
      *index-1, (*pointer)[*index-1]);
#endif
  strncpy((*pointer)[*index-1], inpval, buflen);
  (*pointer)[*index-1][buflen] = '\0';
}

/***********************************************************************
* copy array element "index" into string outval
* buflen is the length of outval, provided implicitly by fortran77
***********************************************************************/

void GETSTRING (char *outval, char ***pointer, int *index, int buflen)
{
  int i;
  char temp;
  
#ifdef DYN_DEBUG
  debuginfo("GETSTRING", outval, pointer, index, buflen);
#endif
  for(i = 0; i < buflen; i++)
  {
    if((temp = (*pointer)[*index-1][i]) == '\0') break;
    outval[i] = temp;
    /* fprintf(stderr, "%i%c.", i,temp); */
  }
  for( ; i < buflen; i++)
    outval[i] = ' ';
#ifdef DYN_DEBUG
  fprintf(stderr, "have read #%i, ptr %i.\n", 
    *index-1, (*pointer)[*index-1]);
#endif
}

