!#######################################################################
!
!                    lib/utilities/einglib.f
!
!   Contains : rdinpf, abspath, abspath1, getpath, getrevno
!
!#######################################################################


!***********************************************************************
!
!        SUBROUTINE RDINPF
!
!     read input file : reads a line of the input-file and returns
!                       the separate words in character strings
!
! on input:
! iz:  no. of lines previously read from input file
! iin: channel of file to be read
! maxkey: maximum no. of keywords on line. (Constant)
! ierr = 0  : Error flag.
!
! on return:
! ic: no. of words on line 
! lc: length of words
! iz: line no. in input file read
! keyword: separate keywords read from line inptit
!  IF:
!   ierr=0 no error
!   inptit: line read (line no. iz in file)
!  ELSE IF:
!   ierr=1 error in reading iin file. 
!   ierr=2 end of file found
!   inptit: Error message
!
!    03/97  HDM
!
!***********************************************************************

      subroutine rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)

use global, only:c3,c5,maxkeylen

      implicit none

      integer,intent(in)            ::maxkey
      integer,parameter             ::maxbuf=1920
      integer,intent(in)            ::iin
      integer,intent(out)           ::ic
      integer,intent(inout)         ::iz,ierr
      integer,intent(out)           ::lc(maxkey)
      character(len=maxbuf)         ::buffer,ctmp
      character(len=c5),intent(out) ::inptit
      character(len=*),intent(out)  ::keyword(maxkey),keyorig(maxkey)
      character(len=4)              ::rest
      integer,save                  ::maxeof=0 !initialization in first call
      integer                       ::i,j,k,i0,laenge

      ierr=0
      ic = 0
      do  i = 1, maxkey
         lc(i) = 0
         keyword(i) = '  '
         keyorig(i) = '  ' 
      end do
      do i=1,maxbuf
         buffer(i:i)= ' '
      enddo

!-----------------------------------------------------------------------
! Read one line of the input-file. Ignore lines which begin
! with '#' or '-----' or '_____' .
!-----------------------------------------------------------------------
 30   read(iin,'(2a)',end=11,err=12) inptit,rest
      iz = iz + 1
      if( inptit(1:1) .eq. '#' )     goto 30
      if( inptit(1:2) .eq. '/*' )    goto 30
      if( inptit(1:5) .eq. '-----' ) goto 30
      if( inptit(1:5) .eq. '_____' ) goto 30

!-----------------------------------------------------------------------
! Remove tabs and blanks at the beginning. ( ichar(\tab)=9 )
! Skip lines that start with 32 blanks.
!-----------------------------------------------------------------------
      do k=1,32
         if(inptit(k:k).ne.' ' .and. ichar(inptit(k:k)).ne.9) then
            if(inptit(k:k)   .eq. '#')      goto 30
            if(inptit(k:k+1) .eq. '/*')     goto 30
            if(inptit(k:k+4) .eq. '-----' ) goto 30
            if(inptit(k:k+4) .eq. '_____' ) goto 30
            buffer(1:1) = inptit(k:k)
            i0 = k + 1
            goto 15
         endif
      enddo
      goto  30                  ! Ignore lines that start with 16 blanks

!-----------------------------------------------------------------------
! The four last characters must be blank. Otherwise input-line too long.
!-----------------------------------------------------------------------
 15   If( rest .ne. '    ' ) then
         write(6,'(a)') trim(inptit)
         write(6,'(2a)') ' rest : ', rest
         ierr=1
         inptit = '****  Input line too long! *****   line no.:   '
         write(inptit(46:50),'(i5)') iz
      endif

!-----------------------------------------------------------------------
! transfer input line to buffer, with modifications
! (i)   remove semicolons and brackets so they are separators. 
! (ii)  ensure blanks are either side of equals or comma signs so
!       they are keywords. 
! (iii) add end marker '#'
!-----------------------------------------------------------------------
      j=1
      do i = i0,c5
         if ( j+3 .gt. maxbuf ) goto 16
         j=j+1
         if (inptit(i-1:i) .eq. '  ' ) then
            j=j-1               ! Shrink multiple blanks to a single blank. 
         else if (inptit(i:i) .eq. ';') then
            buffer(j:j) = ' '
         else if (ichar(inptit(i:i)) .eq. 9) then  ! Tab->blank
            buffer(j:j) = ' '
         else if (inptit(i:i) .eq. '(') then
            buffer(j:j) = ' '
         else if (inptit(i:i) .eq. ')') then
            buffer(j:j) = ' '
         else if (inptit(i:i) .eq. '=') then
            buffer(j:j+2) = ' = '
            j=j+2
         else if (inptit(i:i) .eq. '<') then
            buffer(j:j+2) = ' < '
            j=j+2
         else if (inptit(i:i) .eq. '>') then
            buffer(j:j+2) = ' > '
            j=j+2
         else if (inptit(i:i) .eq. ',') then
            buffer(j:j+2) = ' , '
            j=j+2
         else if (inptit(i:i) .eq. '[') then
            buffer(j:j+2) = ' [ '
            j=j+2
         else if (inptit(i:i) .eq. ']') then
            buffer(j:j+2) = ' ] '
            j=j+2
         else if (inptit(i:i) .eq. '{') then
            buffer(j:j+2) = ' { '
            j=j+2
         else if (inptit(i:i) .eq. '}') then
            buffer(j:j+2) = ' } '
            j=j+2
         else if (inptit(i:i) .eq. '|') then
            buffer(j:j+1) = ' |'
            j=j+1
         else if (inptit(i:i) .eq. '#') then
            buffer(j:j+1) = ' #'
            goto 33
         else
            buffer(j:j) = inptit(i:i)
         endif
      end do

 16   if (j+3 .gt. maxbuf) then
         ierr=1
         inptit = '*****  Buffer overflow! *****   line no.:'
         write(inptit(44:48),'(i5)') iz
         return
      endif
      buffer(j+1:j+2) = ' #'

!-----------------------------------------------------------------------
! remove leading blank and check if at end of line 
!-----------------------------------------------------------------------
 33   continue
      if (buffer(1:1) .eq. '#') then
         if (ic .eq. 0) then
            goto 30
         else
            return
         endif
      else if (buffer(1:1) .eq. ' ' ) then 
         ctmp = buffer(2:maxbuf)
         buffer=ctmp
         goto 33  
      else if (buffer(1:1) .eq. '!' ) then 
         laenge = index(buffer,' ')
         ctmp = buffer(laenge+1:maxbuf)
         buffer=ctmp
         goto 33  
      endif
      
!-----------------------------------------------------------------------
! store the next word into keyword(ic)
!-----------------------------------------------------------------------
      laenge = index(buffer,' ')-1
      ic = ic + 1
      if (ic .ge. maxkey ) then
         ierr=1
         inptit = '*****  Too many keywords! *****   line no.:'
         write(inptit(44:48),'(i5)') iz
         return
      endif
      if (laenge .gt. maxkeylen ) then
         ierr=1
         inptit = '*****  keyword too long! *****   line no.:'
         write(inptit(43:47),'(i5)') iz
         inptit=inptit(1:47)//' '//buffer
         return
      endif

      if(buffer(1:1) .eq. '!' ) then
         ic = ic -1
      else
         lc(ic) = laenge
         keyword(ic) = buffer(1:laenge)
         keyorig(ic) = buffer(1:laenge)
         call keylowc(keyword(ic))
      end if
      ctmp = buffer(laenge+2:maxbuf)
      buffer=ctmp
      goto 33

 11   ierr=2
      inptit = '*****  End of file found! *****   last line no.:'
      write(inptit(49:53),'(i5)') iz
      maxeof = maxeof +1
      if (maxeof .gt. 9 ) goto 20
      return

 12   ierr=1
      inptit = '*****  Error reading file! *****   last line no.:'
      write(inptit(50:54),'(i5)') iz
      maxeof = maxeof +1
      if (maxeof .gt. 9 ) goto 20
      return

 20   write(6,*) 
      write(6,'(a)') '################################################'
      write(6,'(a,i3)') ' Too many errors while reading channel:', iin
      write(6,'(a)')    ' Program is canceled by "rdinpf" '
      write(6,'(a)') '################################################'
      stop 1

      end subroutine rdinpf

!***********************************************************************
!
!            Subroutine   ABSPATH
!
! Generates the absolute path-name from a given relative one.
! Allows to use the bash symbol ~ for the home-directory.
! fgetcwd : c-routine that returns the name of the current directory.
!
! 06/97 HDM
! 
!***********************************************************************

      subroutine abspath(relp,rl)

use global, only:c5
       
      Implicit None

      integer                 ::i,j,k,cl,al,ig,is
      integer                 ::rl
      integer,save            ::wl=0 !initialization in first call
      integer,parameter       ::d5=c5+40
      logical                 ::lhome
      character(len=*)        ::relp
      character(len=c5)       ::current
      character(len=c5),save  ::workdir
      character(len=d5)       ::absp
      character(len=80)       ::ignore
      character(len=24)       ::substi

      external  fgetcwd
!.....Initialize (=SED=)
      ignore= ' ' 
      substi= ' ' 

!.....Initialize
      ig    = index(ignore,' ')-1
      is    = index(substi,' ')-1
      lhome = .false.
      cl    = 0
      al    = 0
      j     = 1
      do i=1,c5
         absp(i:i)    =' '
         current(i:i) =' '
      end do
      absp(c5:d5)  ='                                         '

      if(wl .eq. 0 ) then
         call fgetcwd(workdir,wl)
!....Remove a trailing part from the path (if any).
         if(ig .gt. 1) then
            if(workdir(1:ig) .eq. ignore(1:ig)) then
               workdir = substi(1:is)//workdir(ig+1:wl)
               wl =  wl - ig + is
            end if
         end if
      end if

!.....Stop when  pathlength is inconsistent.
      if(rl .ge. c5-4) then
         write(6,*) 'abspath : name is too long!'
         write(2,*) 'abspath : name is too long!'
         stop
      elseif(rl .le. 0 ) then
         write(6,*) 'abspath : name is empty!'
         write(2,*) 'abspath : name is empty!'
         stop
      end if

!.....Append a final slash if necessary.
      if( relp(rl:rl) .ne. '/' )  then
!         relp = relp(1:rl)//'/ '
         relp(rl+1:rl+1) = '/'
         rl   = rl + 1
      end if

!....Remove a trailing part from the path (if any).
      if(ig .gt. 1) then
         if(relp(1:ig) .eq. ignore(1:ig)) then
            relp = substi(1:is)//relp(ig+1:rl)
            rl =  rl - ig + is
         end if
      end if

!.....read home or current directory path into the variable current.
!.....return, if the path is already absolute.

      if( relp(1:1) .eq. '/' )  then
          if(rl .eq. 1)  goto 240
          relp(rl:rl) = ' '
          rl = rl - 1
          goto 240
      else if( relp(1:2) .eq. '~/' )  then
!cc         call getenv('HOME',current)
!cc         cl   = index(current,' ')-1
         call fhome(current,cl)
         if(cl .gt. 1 .and. current(cl:cl).eq.'/') cl=cl-1

!....Remove a trailing part from the path (if any).
         if(ig .gt. 1) then
            if(current(1:ig) .eq. ignore(1:ig)) then
               current = substi(1:is)//current(ig+1:cl)
               cl =  cl - ig + is
            end if
         end if

         if(cl .ge. c5-2 ) goto 300
         relp = relp(3:rl)//'  '
         rl   = rl - 2
         lhome= .true.
      else 
         cl = wl
         current = workdir(1:wl)
         if(cl .le. 0  .or.  cl .ge. c5-2 ) goto 300
         if(cl .gt. 1 .and. current(cl:cl).eq.'/') cl=cl-1
      end if

!......check for the various cases.

      if( cl+rl .ge. d5-2 ) goto 300
      if( relp(1:2) .eq. './' ) then
         if (rl.gt.2) then
            relp = relp(3:rl)//'  '
         else
            relp = ' '
         endif
         al   = cl + rl - 1
         rl   = rl - 2
      end if
      
      if( relp(1:3) .eq. '../' ) then
         k = 4
 200     do i = cl-1 , 1, -1
            if( current(i:i) .eq. '/' ) then
               j    = i 
               al   = i + rl + 1 - k
               if( al .ge. d5-2 ) goto 300
               absp = current(1:i)//relp(k:rl)
               goto 100
            endif
         enddo
 100     continue
         if( absp(j+1:j+3) .eq. '../' ) then
            absp(j:j+4) = '     '
            cl = j
            k  = k + 3
            goto 200
         end if
         
      else if(rl.gt.0) then
         al   = cl + rl + 1
         absp = current(1:cl)
         absp(cl+1:cl+1) = '/'
         absp(cl+2:al) = relp(1:rl)
      else
         absp = current(1:cl)//'/'
         al   = cl + 1
      end if
      
      if(al .gt. c5) goto 300
!.....Remove final / and store abspath to relpath.
      absp(al:al+2) = '   '
      relp = absp(1:c5)
      rl   = al - 1
      
 240  if(relp(max(1,rl-3):rl) .eq. '.inp' ) then
!.....Store the path of the directory of the input-file to workdir.
         do i = rl-3, 2, -1
            wl = i-1
            if( relp(i:i) .eq. '/' ) goto 250
         end do
 250     workdir = relp(1:wl)
      end if

      return

 300  write(6,*)'abspath: using relative rather than absolute pathname'
      write(2,*)'abspath: using relative rather than absolute pathname'
      if(lhome) then
         relp = '~/'//relp(1:rl)
         rl = rl + 1
      else
         rl = rl - 1
      end if
!.....Remove final /
      relp(rl+1:rl+1) = ' '
      
      return
      
      end subroutine abspath
      
!***********************************************************************
!
!            Subroutine   ABSPATH1
!
! Generates the absolute path-name from a given relative one.
! In contrast to abspath above, path is relative to a given directory
! rather than the current directory
! Allows to use the bash symbol ~ for the home-directory.
! fgetcwd : c-routine that returns the name of the current directory.
!
! 06/97 HDM
! 
!***********************************************************************

      subroutine abspath1(relp,rl,workdir)
       
use global, only:c5

      Implicit None
 
      integer           ::i,j,k,cl,al,wl,ig,is
      integer           ::rl
      integer,parameter ::d5=c5+40
      logical           ::lhome
      character(len=c5) ::relp,current,workdir
      character(len=d5) ::absp
      character(len=80) ::ignore
      character(len=24) ::substi

      external  fgetcwd


!.....Initialize (=SED=)
      ignore= ' ' 
      substi= ' ' 

      ig    = index(ignore,' ')-1
      is    = index(substi,' ')-1
      lhome = .false.
      cl    = 0
      al    = 0
      j     = 1
      do i=1,c5
         absp(i:i)    =' '
         current(i:i) =' '
      end do
      absp(c5:d5)  ='                                         '

      wl=index(workdir,' ')-1
      if(wl .eq. 0 .or. workdir.eq.'.' .or. workdir.eq.'./') then
         call fgetcwd(workdir,wl)
!....Remove a trailing part from the path (if any).
         if(ig .gt. 1) then
            if(workdir(1:ig) .eq. ignore(1:ig)) then
               workdir = substi(1:is)//workdir(ig+1:wl)
               wl =  wl - ig + is
            end if
         end if
      end if


!......Stop when  pathlength is inconsistent.
      if(rl .ge. c5-4) then
         write(6,*) 'abspath1 : name is too long!'
         write(2,*) 'abspath1 : name is too long!'
         stop
      elseif(rl .le. 0 ) then
         write(6,*) 'abspath1 : name is empty!'
         write(2,*) 'abspath1 : name is empty!'
         stop
      end if

!......Append a final slash if necessary.
      if( relp(rl:rl) .ne. '/' )  then
         relp = relp(1:rl)//'/ '
         rl   = rl + 1
      end if

!....Remove a trailing part from the path (if any).
      if(ig .gt. 1) then
         if(relp(1:ig) .eq. ignore(1:ig)) then
            relp = substi(1:is)//relp(ig+1:rl)
            rl =  rl - ig + is
         end if
      end if

!.....read home or current directory path into the variable current.
!.....return, if the path is already absolute.

      if( relp(1:1) .eq. '/' )  then
         if(rl .eq. 1)  goto 240
         relp(rl:rl) = ' '
         rl = rl - 1
         goto 240
      else if( relp(1:2) .eq. '~/' )  then
!cc         call getenv('HOME',current)
!cc         cl   = index(current,' ')-1
         call fhome(current,cl)
         if(cl .gt. 1 .and. current(cl:cl).eq.'/') cl=cl-1

!....Remove a trailing part from the path (if any).
         if(ig .gt. 1) then
            if(current(1:ig) .eq. ignore(1:ig)) then
               current = substi(1:is)//current(ig+1:cl)
               cl =  cl - ig + is
            end if
         end if

         if(cl .ge. c5-2 ) goto 300
         relp = relp(3:rl)//'  '
         rl   = rl - 2
         lhome= .true.
      else 
         cl = wl
         current = workdir(1:wl)
         if(cl .le. 0  .or.  cl .ge. c5-2 ) goto 300
         if(cl .gt. 1 .and. current(cl:cl).eq.'/') cl=cl-1
      end if

!.....check for the various cases.

      if( cl+rl .ge. d5-2 ) goto 300
      if( relp(1:2) .eq. './' ) then
         if (rl.gt.2) then
            relp = relp(3:rl)//'  '
         else
            relp = ' '
         endif
         al   = cl + rl - 1
         rl   = rl - 2
      end if

      if( relp(1:3) .eq. '../' ) then
         k = 4
  200     do i = cl-1 , 1, -1
            if( current(i:i) .eq. '/' ) then
              j    = i 
              al   = i + rl + 1 - k
              if( al .ge. d5-2 ) goto 300
              absp = current(1:i)//relp(k:rl)
              goto 100
            endif
         enddo
  100    continue
         if( absp(j+1:j+3) .eq. '../' ) then
            absp(j:j+4) = '     '
            cl = j
            k  = k + 3
            goto 200
         end if

      else if(rl.gt.0) then
         absp = current(1:cl)//'/'//relp(1:rl)
         al   = cl + rl + 1
      else
         absp = current(1:cl)//'/'
         al   = cl + 1
      end if

      if(al .gt. c5) goto 300
!.....Remove final / and store abspath to relpath.
      absp(al:al+2) = '   '
      relp = absp(1:c5)
      rl   = al - 1


 240  return

 300  write(6,*)'abspath1: using relative rather than absolute pathname'
      write(2,*)'abspath1: using relative rather than absolute pathname'
      if(lhome) then
         relp = '~/'//relp(1:rl)
         rl = rl + 1
      else
         rl = rl - 1
      end if
!......Remove final /
      relp(rl+1:rl+1) = ' '

      return

      end subroutine abspath1

!######################################################################
!
! GETPATH
!
! extracts the path from a file name
!
!#######################################################################

      subroutine getpath(filename,path,pathlen)

      implicit none

      integer,intent(out)          ::pathlen
      character(len=*),intent(in)  ::filename
      character(len=*),intent(out) ::path

      pathlen=index(filename,'/',back=.true.)-1
      path=filename(1:pathlen)

      return
      end subroutine getpath

!***********************************************************************
!
!            Subroutine   FIND_PARAMAETER
!
! Finds aparameter and returns its value
! 06/23 HDM
!
!***********************************************************************

      subroutine find_parameter(kword,value,lfound)

      use global, only: apar, rpar, npar

      Implicit None

      Integer      n
      real*8       value
      logical      lfound
      character*60 kword

      lfound=.false.
      do n=1,npar
         if (apar(n) .eq. kword) then
            value=rpar(n)
            lfound=.true.
            exit
         endif
      enddo

      return
      end subroutine find_parameter


!######################################################################
