!***********************************************************************
!
! NB It is assumed that quantites read from output files are returned
! in 
! Angstrom for coordinates 
! eV for energies 
! Hartree/Bohr for gradients
!
!
! ept2mcas not used??
!***********************************************************************
      module ioqc

use decimal, only: long, dop
use global, only: zeroi, zeror, onei, oner, c1
use keyunitsmod

use readqc

      implicit none

! Optional data used in reading energy
      integer(long)                :: gndabtype
      real(dop)                    :: eshift
      real(dop)                    :: egrd,eorb
      logical(kind=4)              :: leshift,lovgfall

      public 

      contains

!#######################################################################
!
! GETINTYP
! reads a QC output file and determines the program
! 
! Input: 
! unit     : unit attahced to file to be read
! filename : name of file to be read
! ilog     : unit of log file for errors
!
! Output:
! ifiletyp: 1   Gaussian
!           2   Gamess-uk
!           3   QOMMMA
!           4   Bagel
!           5   Molpro
!           6   DFT/MRCI
!           7   Turbomole
!           8   Gamess-US
!           10  Q-Chem
!           11  Molcas
!           12  Orca
!
! NB These numbers have to match those used in readqc.f90
!#######################################################################

      subroutine getintyp(unit,ifiletyp,filename,ilog)

      implicit none

      integer(long), intent(in)    :: unit,ilog
      integer(long), intent(out)   :: ifiletyp
      character(len=80)            :: string
      character(len=*), intent(in) :: filename

      ifiletyp=0
      
      rewind(unit)
   10 continue
      read(unit,'(a)',end=999) string

      if (string(2:25) .eq. 'Entering Gaussian System') then
         ifiletyp=1
         write(ilog,20) 'Input file format: GAUSSIAN'

      else if (string(57:73) .eq. 'G A M E S S - U K') then
         ifiletyp=2
         write(ilog,20) 'Input file format: Gamess UK'

      else if (string(1:19) .eq. '# ADC spectrum file') then
         ifiletyp=3
         write(ilog,20) 'Input file format: ADC spectrum'

      else if (string(5:47) .eq. 'BAGEL - Freshly leavened quantum chemistry') then
         ifiletyp=4
         write(ilog,20) 'Input file format: BAGEL'

      else if (string(2:28) .eq. 'Primary working directories' .or. &
           string(2:18) .eq. 'Working directory') then
         ifiletyp=5
         write(ilog,20) 'Input file format: Molpro'
         
      else if (string(1:6) .eq. '$coord') then
         ifiletyp=6
         write(ilog,20) 'Input file format: DFT/MRCI'

      else if ((string(2:17) .eq. 'operating system') .or. &
               (string(2:12) .eq. 'hostname is')) then
         ifiletyp=7
         write(ilog,20) 'Input file format: Turbomole'
         
      else if (string(1:29) .eq. '----- GAMESS execution script') then
         ifiletyp=8
         write(ilog,20) 'Gamess US'

      else if (string(19:35) .eq. 'Welcome to Q-Chem') then
         ifiletyp=10
         write(ilog,20) 'Q-Chem'

      else if (string(17:22) .eq. 'MOLCAS') then
         ifiletyp=11
         write(ilog,20) 'Input file format: MOLCAS'

      else if (string(36:48) .eq. 'O   R   C   A') then
         ifiletyp=12
         write(ilog,20) 'Input file format: ORCA'


      else
         go to 10
      endif
   20 format(15x,a)

  999 continue
      if (ifiletyp .eq. 0) then
         write(6,'(a)') 'Unknown file type: ',filename
         stop
      endif
      
      return
      end subroutine getintyp

!#######################################################################
!
! GETABINTYPE
! translates a keyword for an ab initio method to an integer
!
! Input:
! keyword  : string with ab initio type
! ilog     : unit of log file for errors
!
! Output:
! abintyp =
!     1  : RHF
!     2  : MP2
!     3  : MP3
!     4  : MP4
!     9  : ADC
!     10 : OVGF
!     11 : CIS
!     12 : CAS
!     13 : CASPT2
!     14 : MRCI
!     15 : CAS/MRCI
!     16 : 
!     17 : EOM-CCSD
!     18 : DFT/MRCI
!     19 : TDDFT
!     20 : CR-EOM-CCSD(T)
!     21 : Delta-CR-EOM-CCSD(T)
!     22 : CCSD
!     23 : RI-CC2
!     24 : TDDFT with PCM
!     25 : TDDFT with pt-ss-PCM
!     26 : local-ADC(2)
!     27 : MS-CASPT2
!     28 : RMS-CASPT2
!     29 : QOMMMA
!     30 : Ehrenfest
!     31 : External
!     32 : CAS/PT2   (CASPT2 energies, CASSCF gradients + properties)
!     33 : TDDFT
!     34 : MS-CASPT2
!     35:  local-CC2
!     36:  xdw cas
!
!     -1: read a DB
!     -2: read a file with theta,r1,r2 data
!
! NB These numbers have to match those used in readqc.f90
!#######################################################################

      subroutine getabintype(keyword,abintype,ilog)

        implicit none 

        integer(long), intent(out)    :: abintype
        integer(long), intent(in)     :: ilog
        character(len=*), intent(in)  :: keyword

        if (keyword .eq. 'thr1r2') then
           abintype = -2
        else if (keyword .eq. 'db') then
           abintype = -1
        elseif (keyword .eq. 'rhf') then
           abintype = 1
        else if (keyword .eq. 'mp2') then
           abintype = 2
        else if (keyword .eq. 'mp3') then
           abintype = 3
        else if (keyword .eq. 'mp4') then
           abintype = 4
        else if (keyword .eq. 'uhf') then
           abintype = 5
        else if (keyword .eq. 'rohf') then
           abintype = 6
        else if (keyword .eq. 'adc') then
           abintype = 9
        else if (keyword .eq. 'ovgf') then
           abintype = 10
        else if (keyword .eq. 'cis') then
           abintype = 11
        else if (keyword .eq. 'cas') then
           abintype = 12
        else if (keyword .eq. 'caspt2' .or. keyword .eq. 'pt2') then
           abintype = 13
        else if (keyword .eq. 'mrci') then
           abintype = 14
        else if (keyword .eq. 'cas/mrci') then
           abintype = 15
        else if (keyword .eq. 'eom' .or. keyword .eq. 'eomccsd') then
           abintype = 17
        else if (keyword .eq. 'dftmrci') then
           abintype = 18
        else if (keyword .eq. 'dft') then
           abintype = 19
        else if (keyword .eq. 'cr-eom') then
           abintype = 20           
        else if (keyword .eq. 'delta-cr-eom') then
           abintype = 21
        else if (keyword .eq. 'ccsd') then
           abintype = 22
        else if (keyword .eq. 'cc2') then
           abintype = 23
!        else if (keyword .eq. 'ricc2') then
!           abintype = 23
        else if (keyword .eq. 'pcm-tddft') then
           abintype = 24
        else if (keyword .eq. 'ptpcm-tddft') then
           abintype = 25
        else if (keyword .eq. 'ladc') then
           abintype = 26
        else if (keyword .eq. 'ms-cas' .or. keyword .eq. 'ms-caspt2') then
           abintype = 27
        else if (keyword .eq. 'rms-cas') then
           abintype = 28
        else if (keyword .eq. 'qommma') then
           abintype = 29
        else if (keyword .eq. 'ehrenfest') then
           abintype = 30
        else if (keyword .eq. 'external') then
           abintype = 31
        else if (keyword .eq. 'cas/pt2') then
           abintype = 32
        else if (keyword .eq. 'tddft') then
           abintype = 33
        else if (keyword .eq. 'ms-caspt2') then
           abintype = 34
        else if (keyword .eq. 'lcc2') then
           abintype = 35
        else if (keyword .eq. 'xdw') then
           abintype = 36        
        else if (keyword .eq. 'cas/xdw') then
           abintype = 37
       else
           write(ilog,'(/,2a)') 'Ab initio type not known: ',keyword
           STOP
        endif

        return
      end subroutine getabintype

!#######################################################################
!
! GETXCOO
! Reads a set of molecular coordinates from QC ouput
!
! Input:
! unit: unit associated with QC file.
! iau = 0 Bohr
!       1 Angstroem
! orient: string defining geometry to read (orientation in Gaussian)
! ncoo: number of Cartesian coordinates
!
! Output:
! Coordinates from QC in Angstrom or Bohr
! Atomic number for each coordinate
!
!#######################################################################

      subroutine getxcoo(unit,xcoo,ifiletyp,iau,orient,iatsym,iatiso,ncoo,&
                 nsta,nmodes,ilog)

      implicit none

      integer(long), intent(in)                   :: unit,ncoo,nsta,nmodes,&
                                                     ifiletyp,ilog,iau
!atomic number and isotope information
      integer(long), dimension(ncoo), intent(out) :: iatsym,iatiso 
!dummy variables for rdoutqc
      integer(long)                               :: imethod,istate,nroot
      real(dop), dimension(ncoo), intent(out)     :: xcoo
      character(len=*), intent(in)                :: orient
      logical(kind=4)                             :: lfound,lfail

      iatiso(:)=0

! Initialise Interface to QC output. 
      qc_ncoo = ncoo
      qc_nst = nsta
      qc_nmode = nmodes
      qc_nbasis = 0
      call alloc_readqc(ilog)

! read output
      imethod=0
      istate=0
      nroot=0
      qcread(:) = .false.
      qcread(1) = .true.
      lfound = .true.    ! Test coordinates found
      qcorient = orient ! Gaussian input orientation
      call initqcout(lfail)
      call rdqcout(unit,ifiletyp,imethod,ilog,lfound,lfail)
      if (.not. lfail) call chkqcout(ifiletyp,ilog,lfound,lfail)
      if (lfail) then
         write(ilog,'(/,a)') 'ERROR in QC calculation (getxcoo).'
         stop
      endif

! copy data to local arrays
      xcoo(:) = qcxcoo(:)
      iatsym(:) = qciatsym(:)
      if (qcfound(10)) then
         iatiso(:) = qciatiso(:)
      endif

      call dealloc_readqc

! convert units
      if (iau .eq. 1) xcoo(:) = xcoo(:)*angstroem

      return
      end subroutine getxcoo

!#######################################################################
!
! returns energy in eV
! 
! e0  : zero of energy scale in eV
! eigen: number of state to be read. eigen=1 is ground-state.
!
! Energy returned from QC in eV, shifted relative to e0
! 
! iau = 0  energy in au
!       1         in eV
! e0 has to be in same units as iau.
!#######################################################################

      subroutine geten(unit,energy,e0,ifiletyp,iau,eigen,abintype,&
                 filename,ilog,ncoo,nsta,nmodes,lsocdegen)

      implicit none

      integer(long), intent(in) :: unit,eigen,abintype,ifiletyp, &
                                   ilog,ncoo,nsta,nmodes,iau
      integer(long)             :: ilbl
      real(dop), intent(in)                  :: e0
      real(dop), intent(out)                 :: energy
      character(len=*), intent(in) :: filename
      logical(kind=4)              :: found,lfound,lfail,lsocdegen

      found=.false.
      rewind(unit)
      ilbl=index(filename,' ')-1
      if (eigen .gt. nsta) then
         write(ilog,'(/,a,i5)') &
             'Trying to read higher root than highest state specified : ',&
                       eigen
         write(ilog,'(a)') 'Check states keyword for specification '
         print*,'Finished with Error. Check Log'
         stop
      endif

!-----------------------------------------------------------------------
! New interface 
!-----------------------------------------------------------------------
      if (ifiletyp .eq. 1 .and. abintype .eq. 16) then
! Gaussian SOC
         call g09soc(unit,energy)

      else if (ifiletyp .eq. 5 .and. abintype .eq. 17) then
         call energeom(unit,energy,e0,eigen,found)

      else if (ifiletyp .eq. 10 .and. abintype .eq. 24) then
            call energdftqchem(unit,energy,e0,eigen,found)
      else if (ifiletyp .eq. 10 .and. abintype .eq. 25) then
            call energptpcmqchem(unit,energy,eigen,found)

      else
! Initialise Interface to QC output
         qc_ncoo = ncoo
         qc_nst = nsta
         qc_nmode = nmodes
         qc_nbasis = 0
         call alloc_readqc(ilog)
! specify what needs to be read
         qcread(:)=.false.
         qcread(3)=.true.                            ! Energy

! read output
         lfound = .true.
         call initqcout(lfail)
         call rdqcout(unit,ifiletyp,abintype,ilog,lfound,lfail)
         if (.not. lfail) call chkqcout(ifiletyp,ilog,lfound,lfail)
         if (lfail) then
            write(ilog,'(/,a)') 'ERROR in QC calculation (geten).'
            stop
         endif

! copy data to local array
         energy = qcv(eigen)
         found=lqcfound3(eigen)   

         call dealloc_readqc

         if (found) then
! convert units and shift relative to given e0
            if (iau .eq. 1) energy = energy*uev
            energy = energy - e0
         endif

! end of if-endif
      endif

     if (.not. found) then
         write(ilog,'(a)') 'ERROR in Geten (iofiles.F)'
         write(ilog,'(2a)') 'Energy not found :',filename(1:ilbl)
         write(ilog,'(a,i5)') 'Correct calculation type? :',abintype
         write(ilog,'(a,i5)') 'Correct file type? :',ifiletyp

         print*,'Finished with Error. Check log file'
         stop
      else
         lfail = .false.
      endif

      if (lfail) then
         write(ilog,'(/,a)') 'ERROR in QC calculation (geten).'
         stop
      endif

! Apply any energy shifts
      if (leshift) then
         energy = energy + eshift
         write(ilog,*) 'Energy shift of ',eshift,' applied'
      endif

      return
      end subroutine geten

!#######################################################################

      subroutine gettdip(unit,tdip,ifiletyp,ista,abintype,found,&
                 lvelocity,negphase,ilog,ncoo,nmodes,nsta)
 
      implicit none

      integer(long), intent(in)            :: unit,ista,abintype,ifiletyp,negphase
      integer(long), intent(in)            :: ncoo,nsta,nmodes,ilog
      real(dop), dimension(4), intent(out) :: tdip
      logical(kind=4), intent(in)          ::  lvelocity
      logical(kind=4)                      ::  found,lfound,lfail


      rewind(unit)
      if (ifiletyp .eq. 11) then
         if (abintype .eq. 12) then
            call tdipmolcas(unit,tdip,ista,found,negphase)
         endif

      else if (ifiletyp .eq. 5) then
         if (abintype .eq. 13) then
            call tdipmolpt2(unit,tdip,ista,found,negphase)
         else if (abintype .eq. 17) then
            call tdipmoleom(unit,tdip,ista,found,negphase)
         endif

!-----------------------------------------------------------------------
! New interface
!-----------------------------------------------------------------------
      else
! Initialise Interface to QC output
         qc_ncoo = ncoo
         qc_nst = nsta
         qc_nmode = nmodes
         qc_nbasis = 0
         call alloc_readqc(ilog)

! specify what needs to be read
         qcread(:)=.false.
         qcread(9)=.true.     ! Transition Dipoles

         lqctdveloc=lvelocity    ! read velocity gauge?

! read output
         lfound = .false.    ! don't error if not present
         call initqcout(lfail)
         call rdqcout(unit,ifiletyp,abintype,ilog,lfound,lfail)
         if (.not. lfail) call chkqcout(ifiletyp,ilog,lfound,lfail)
         if (lfail) then
            write(ilog,'(/,a)') 'ERROR in QC calculation (gettdip).'
            stop
         endif

         found=lqcfound9(1,ista)
! copy data to local array
         if (found) tdip(1:4) = qctdipole(1:4,1,ista)

         call dealloc_readqc

      endif
         
      return
      end subroutine gettdip

!#######################################################################

      subroutine gettdipall(unit,dipole,ifiletyp,abintype,found,nsta,phaseswitch)
 
      implicit none

      integer(long), intent(in)       :: unit,abintype,ifiletyp,nsta
      integer(long), dimension(nsta,nsta), intent(in) :: phaseswitch
      real(dop), dimension(nsta,nsta,3), intent(out)  :: dipole
      logical(kind=4)                                 :: found      
      
      
      found=.false.
      if (abintype .eq. 13) then
         rewind(unit)
         if (ifiletyp .eq. 5) then
            call dipole_molpt2(unit,dipole,found,nsta,phaseswitch)
         else
            stop
         endif
     
      else if (abintype .eq. 1) then
         rewind(unit)
         if (ifiletyp .eq. 1 .or. ifiletyp .eq. 4 .or. ifiletyp .eq. 9) then
            call dipole_g03hf(unit,dipole,found,nsta)
         else
            stop
         endif
         
      else if (abintype .eq. 17) then
         rewind(unit)
         call dipole_moleom(unit,dipole,found,nsta,phaseswitch)
      else
         print*,''
         write(6,'(3a)') "Reading of all dipole moments is only", &
              " currently implemented for MOLPRO CASPT2 calculations", &
              " and G03 HF calculations."
         print*,''
         STOP
      endif
         
      if (.not.found) then
         write(6,'(/,x,a,/)') "Problem reading dipole moments."
         STOP
      endif
      return
      end subroutine gettdipall

!#######################################################################
!
! Gradients returned from QC in Hartree/Bohr and converted to 
! eV/Angstrom
!
!#######################################################################

      subroutine getder1(unit,ifiletyp,xder1,iau,nmcoo,qder1,info,ista, &
                 abintype,maxdim,ncoo,nmodes,ilog,nsta)

      implicit none

      integer(long), intent(in)  :: unit,ista,abintype,ifiletyp,maxdim,&
                                    ncoo,nmodes,iau,nsta,ilog
      integer(long), intent(out) :: info
      logical(kind=4)            :: found,lfound,lfail
      real(dop), dimension(maxdim), intent(out)       :: xder1,qder1
      real(dop), dimension(maxdim,maxdim), intent(in) :: nmcoo

      rewind(unit)
      found=.false.
      if (ifiletyp .eq. 10 .and. (abintype.eq.24 .or. abintype.eq.25)) then
         call der1pcmqchem(unit,xder1,found,maxdim,ncoo,ista)

!-----------------------------------------------------------------------
! New interface 
!-----------------------------------------------------------------------
      else
! Initialise Interface to QC output
         qc_ncoo = ncoo
         qc_nst = nsta
         qc_nmode = nmodes
         qc_nbasis = 0
         call alloc_readqc(ilog)

! specify what needs to be read
         qcread(:)=.false.
         qcread(4)=.true.                            ! First derivatives

! read output
         lfound = .false.    ! don't error if not present 
         call initqcout(lfail)
         call rdqcout(unit,ifiletyp,abintype,ilog,lfound,lfail)
         if (.not. lfail) call chkqcout(ifiletyp,ilog,lfound,lfail)
         found=lqcfound4(ista)

! copy data to local array
         xder1(1:ncoo) = qcderiv1(1:ncoo,ista)

         call dealloc_readqc

      endif

      if (lfail) then
         write(ilog,'(/,a)') 'ERROR in QC calculation (getder1).'
         stop
      endif
      if (found) then
! convert units to ev/angstroem
         if (iau .eq. 1) xder1=xder1*uev/angstroem

         call dgemv('T',ncoo,nmodes,oner,nmcoo,maxdim,xder1,onei,zeror,qder1,onei)
         info=1
      endif

      return
      end subroutine getder1

!#######################################################################
!
! TO BE FIXED - only Gaussian gives GD directly!
!#######################################################################

      subroutine getgd(unit,ifiletyp,xder1,nmcoo,qder1,found, &
                 abintype,maxdim,ncoo,ilog,nmodes,iau)

      implicit none

      integer(long), intent(in)  :: unit,abintype,ifiletyp,maxdim, &
                                    ncoo,nmodes,ilog,iau
      logical(kind=4)            :: found
      real(dop), dimension(maxdim), intent(out)       :: xder1,qder1
      real(dop), dimension(maxdim,maxdim), intent(in) :: nmcoo

      if (abintype .eq. 13) then
         write(ilog,'(a)') 'grads at CASSCF level'
      elseif (abintype .eq. 17) then
         write(ilog,'(a)') 'grads at EOM-CCSD level'
      elseif (abintype .eq. 24 .or. abintype .eq. 25) then
         write(ilog,'(a)') 'grads at TD-DFT/QCHem level with PCM'
      elseif (abintype .eq. 19 .or. abintype .eq. 33) then
         write(ilog,'(a)') 'grads at TD-DFT/QCHem level'
      elseif (abintype .ne. 12) then
         write(ilog,'(a)') 'ERROR in getgd (ioqc.F)'
         write(ilog,'(a)') 'Unknown ab initio type for GD'
         write(6,'(a)') 'ERROR in getgd (ioqc.F)'
         write(6,'(a)') 'Unknown ab initio type for GD'
         stop
      endif

      rewind(unit)
      if (ifiletyp .eq. 1 .or. ifiletyp .eq. 4 .or. ifiletyp .eq. 9) then
         call gdgaussian(unit,xder1,found,maxdim,ncoo)
      elseif (ifiletyp .eq. 11) then
         call gdmolcas_ci(unit,xder1,found,maxdim,ncoo,ilog)
      elseif (ifiletyp .eq. 10) then
         call gdqchem(unit,xder1,found,maxdim,ncoo)
      else
         return
      endif
      
      if (found) then
! convert to ev/angstroem
         if (iau .eq. 1) xder1=xder1*uev/angstroem
         call dgemv('T',ncoo,nmodes,oner,nmcoo,maxdim,xder1,onei,zeror,qder1,onei)
      endif 

      return
      end subroutine getgd

!#######################################################################
!
! asta is no. of state in database 
! eigen is actual no. of state in file
!
! DCP returned from QC 
! iau = 0 in Hartree/Bohr 
!       1 in eV/Angstrom
!
!#######################################################################

      subroutine getdcp(unit,ifiletyp,iau,xder1,nmcoo,qder1,info,eigen,&
                 asta,asta1,found,abintype,maxdim,ncoo,ilog,nmodes,nsta, &
                 n,maxsta)

      implicit none

      integer(long), intent(in)    :: unit,abintype,ifiletyp,iau,maxdim, &
                                      ncoo,nmodes,ilog,nsta,n,& !eigen,asta,
                                      maxsta
      integer(long), intent(out)   :: info
      integer(long), dimension(maxsta), intent(inout)  :: asta1
      integer(long), dimension(*), intent(in)          ::eigen, asta
      integer(long)                                    ::eigenl,eigenr,astal
      integer(long)                                    :: s
      logical(kind=4)                                  :: found,lfound,lfail
      real(dop), dimension(maxdim,maxsta), intent(out) :: xder1,qder1
      real(dop), dimension(maxdim,maxdim), intent(in)  :: nmcoo

      astal = asta(n)
      asta1 = -1
      rewind(unit)

      if (ifiletyp .eq. 10 .and. (abintype .eq. 24 .or. & 
                  abintype .eq. 25)) then
         call dcppcmqchem(unit,xder1,astal,asta1,found,maxdim,maxsta,ncoo)

!-----------------------------------------------------------------------
! New interface 
!-----------------------------------------------------------------------
      else
! Initialise Interface to QC output
         qc_ncoo = ncoo
         qc_nst = nsta
         qc_nmode = nmodes
         qc_nbasis = 0
         call alloc_readqc(ilog)

! specify what needs to be read
         qcread(:)=.false.
         qcread(7)=.true.                            ! NACT

! read output
         lfound = .false.    ! error if DCP not present
         call initqcout(lfail)
         call rdqcout(unit,ifiletyp,abintype,ilog,lfound,lfail)
         if (.not. lfail) call chkqcout(ifiletyp,ilog,lfound,lfail)
         found=qcfound(7)
         if (lfail) then
            write(ilog,'(/,a)') 'ERROR in QC calculation (getdcp).'
            stop
         endif

! copy data to local array
         eigenl = eigen(n)   ! LHS state for dataset
         do s=1,nsta
            eigenr = eigen(s)  ! RHS state
            if (lqcfound7(eigenl,eigenr)) then
               asta1(s)=s
               xder1(1:ncoo,s) = qcnact(1:ncoo,eigenl,eigenr)
            endif
         enddo

         call dealloc_readqc

      endif

      if (found) then
! convert to ev/angstroem
          if (iau .eq. 1) xder1=xder1*uev/angstroem
          info=1
          do s = 1,nsta
             if (asta1(s) .eq. -1) cycle
             call dgemv('T',ncoo,nmodes,oner,nmcoo,maxdim,xder1(1,s),onei, &
                  zeror,qder1(1,s),onei)
          enddo
      endif
      return
      end subroutine getdcp

!#######################################################################

      subroutine getder2(unit,ifiletyp,der2,iatsym,nmcoo,qder2,iau, &
                 ista,abintype,maxdim,ncoo,nsta,nmodes,info,ilog)

      implicit none

      integer(long), intent(in) :: unit,ifiletyp,maxdim,ncoo,nsta,&
                                   nmodes,ilog,abintype,ista
      integer(long), intent(inout) :: info
      integer(long), intent(in) :: iau
      integer(long), dimension(maxdim), intent(in)     :: iatsym
      real(dop), dimension(maxdim,maxdim), intent(in)  :: nmcoo
      real(dop), dimension(maxdim,maxdim), intent(out) :: der2,qder2
      real(dop), dimension(maxdim,maxdim)              :: tmpnm,tmpfreq
      logical(kind=4)                                  :: lfound,found,lfail
      character(len=16), dimension(maxdim) :: tmplab

      found = .false.
      lfail = .false.

      rewind(unit)
 
!-----------------------------------------------------------------------
! read freq in eV and mass-scaled normal modes for program that doesn't
! provide hessian
! then generate second derivs in eV/A**2
!
! NB no check is made for presence of normal modes so der2 is not
! generated if not found.
!-----------------------------------------------------------------------
! Initialise Interface to QC output
      qc_ncoo = ncoo
      qc_nst = nsta
      qc_nmode = nmodes
      qc_nbasis = 0
      call alloc_readqc(ilog)      
! specify what needs to be read
       qcread(:)=.false.
       qcread(5)=.true.                            ! 2nd derivative

! read output
      lfound = .false.                      ! Don't output error if not present  
      call initqcout(lfail)
      call rdqcout(unit,ifiletyp,abintype,ilog,lfound,lfail)
      if (.not. lfail) call chkqcout(ifiletyp,ilog,lfound,lfail)
      found=lqcfound5(ista)
! copy data to local array
!Use the cartesian Hessian
!der2 correspond to Hessian in cartesian coordinate
!qder2 correspond to Hessian in q coordinate
      der2(1:ncoo,1:ncoo) = qcderiv2(1:ncoo,1:ncoo,ista)
      call dealloc_readqc
! conver units to ev/agnstroem**2
! get Hessian in q coordinates
      if (found) then
         info=2    ! flag that derivs found
         if (iau .eq.1) der2=der2*uev/(angstroem**2)
         call genqder2(der2,nmcoo,qder2,ncoo,maxdim,nmodes,ilog)
      endif

      if (lfail) then
         write(ilog,'(/,a)') 'ERROR in QC calculation (getder2).'
         stop
      endif 

      return
      end subroutine getder2

!#######################################################################
!
! GETNM
! forms mass-scaled normal modes from cartesian
!
! iau = 0 freq in au
!       1 freq in eV
! nmcoo are returned as mass-scaled, ortho-normalised normal modes
!
! lfound = .true. : check will be made for presence of Nmodes
!#######################################################################

      subroutine getnm(unit,ifiletyp,nmlab,nmcoo,freq,iatsym,iau, &
                 maxdim,ncoo,nsta,nmodes,ilog,s,lfound)

      implicit none

      integer(long), intent(in)   :: unit,ifiletyp,maxdim,ncoo,nsta,&
                                     nmodes,iau,ilog,s
      integer(long)               :: imethod,istate,nroot,nm,j,iau1
      integer(long), dimension(maxdim), intent(in)     :: iatsym
      real(dop), dimension(maxdim,maxdim), intent(out) :: nmcoo
      real(dop), dimension(maxdim)                     :: mass
      real(dop), dimension(maxdim), intent(out)        :: freq
      logical(kind=4), intent(in)                      :: lfound
      logical(kind=4)                                  :: found,lfail
      character(len=*), dimension(maxdim), intent(out) :: nmlab

      found = .false.

! Get atomic masses (in amu)
      iau1=1
      do j=1,ncoo
         call getmass(mass(j),iatsym(j),iau1,ilog)
      enddo    

! Initialise Interface to QC output
      qc_ncoo = ncoo
      qc_nst = nsta
      qc_nmode = nmodes
      qc_nbasis = 0
      call alloc_readqc(ilog)
! specify what needs to be read
      qcread(:)=.false.
      qcread(2)=.true.                            ! Normal modes

! read output
      imethod=0
      istate=s
      nroot=0
      call initqcout(lfail)
      call rdqcout(unit,ifiletyp,imethod,ilog,lfound,lfail)
      if (.not. lfail) call chkqcout(ifiletyp,ilog,lfound,lfail)
      if (lfail) then
         write(ilog,'(/,a)') 'ERROR in QC calculation (getnm).'
         stop
      endif


! copy data to local arrays
      nmcoo(1:ncoo,1:nmodes) = qcnmcoo(1:ncoo,1:nmodes)
      nmlab(1:nmodes) = qcnmlab(1:nmodes)
      freq(1:nmodes) = qcfreq(1:nmodes)
      found = qcfound(2)
      
      call dealloc_readqc

      if (found) then
!-----------------------------------------------------------------------
! scale to mass-weighted x to obtain true normal mode vectors
! for nmcoo column=nm, row=cart
! mass is in amu
!-----------------------------------------------------------------------
         do nm=1,nmodes
            do j=1,ncoo
               nmcoo(j,nm)=nmcoo(j,nm)*sqrt(mass(j))
            enddo
         enddo


!-----------------------------------------------------------------------
! ortho-normalise nm coordinates
!-----------------------------------------------------------------------
         write(ilog,'(/,a)') 'Calling nmortho OK'
         call nmortho(nmcoo,maxdim,ncoo,nmodes,ilog)

!-----------------------------------------------------------------------
! convert units (freq to ev)
!-----------------------------------------------------------------------
         if (iau .eq. 1) then
            freq(:) = freq(:)*uev
         endif
      endif

      return
      end subroutine getnm

!#######################################################################
!
! GENQDER2 !maybe replace with dgemv for more efficient matrix multiplication
!
! Generates 2nd derivatives in q from cartesian to normal transformation
! matrix nmcoo and der2 Hessian in eV/angstroem**2
!
! freq1:
! qder2: 2nd derivatives in eV/q**2 (q = normal mode coordinate)
!#######################################################################

      subroutine genqder2(der2,nmcoo,qder2,ncoo,maxdim,nmodes,ilog)

      implicit none

      integer(long), intent(in) :: ncoo,maxdim,nmodes,ilog
      integer(long)             :: i,j,k,l
      real(dop), dimension(maxdim,maxdim), intent(in)  :: nmcoo,der2
      real(dop), dimension(maxdim,maxdim), intent(out) :: qder2


!      der2(:,:) = 0.0_dop
!      do i=1,ncoo
!         do j=1,ncoo
!            der2(i,j)=0.0_dop
!            do k=1,nmodes
!               if (freq1(k) .lt. 0.0_dop) then  ! imaginary freq
!                  der2(i,j)=der2(i,j) - amat(i,k)*freq1(k)*freq1(k)*amat(j,k)
!               else
!                  der2(i,j)=der2(i,j) + amat(i,k)*freq1(k)*freq1(k)*amat(j,k)
!               endif
!            enddo
!            der2(i,j)=der2(i,j)*fact*sqrt(mass(i)*mass(j))
!         enddo
!      enddo
!-----------------------------------------------------------------------
! 2nd derivatives in q
!-----------------------------------------------------------------------
      do i=1,nmodes
         do j=1,nmodes
            qder2(i,j)=0.0_dop
            do k=1,ncoo
               do l=1,ncoo
                  qder2(i,j)=qder2(i,j)+nmcoo(k,i)*der2(k,l)*nmcoo(l,j)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine genqder2

!###############################################################

      integer(long) function getiatsym(atomlab,ilog)

      implicit none

      character(len=*), intent(in) :: atomlab
      integer(long), intent(in)    :: ilog

      if (atomlab(1:2) .eq. 'Cr'.or.atomlab(1:2) .eq. 'cr') then
         getiatsym = 24
      elseif (atomlab(1:2) .eq. 'Cl' .or. atomlab(1:2) .eq. 'cl') then
         getiatsym = 17
      else if (atomlab(1:1) .eq. 'H'.or.atomlab(1:1) .eq. 'h') then
         getiatsym = 1
      elseif (atomlab(1:1) .eq. 'C'.or.atomlab(1:1) .eq. 'c') then
         getiatsym = 6
      elseif (atomlab(1:1) .eq. 'N'.or.atomlab(1:1) .eq. 'n') then
         getiatsym = 7
      elseif (atomlab(1:1) .eq. 'O'.or.atomlab(1:1) .eq. 'o') then
         getiatsym = 8
      elseif (atomlab(1:1) .eq. 'F'.or.atomlab(1:1) .eq. 'f') then
         getiatsym = 9
      elseif (atomlab(1:1) .eq. 'S'.or.atomlab(1:1) .eq. 's') then
         getiatsym = 16
      elseif (atomlab(1:1) .eq. 'I'.or.atomlab(1:1) .eq. 'i') then
         getiatsym = 53
      else
         getiatsym=-999
         write(ilog,'(2a)') 'Unknown label in getiatsym :',atomlab
      endif

      return
      end function getiatsym

!#######################################################################

! ASSMASS
! Find masses (in amu) and names for a set of atoms with atomic numbers
!
! iau = 0 mass in au
!     = 1 mass in amu
! 
!#######################################################################

      subroutine assmass(iatsym,iatiso,atsym,mass,iau,ncoo,ilog)

      implicit none

      integer(long), intent(in)                      :: ncoo,ilog,iau
      integer(long), dimension(ncoo), intent(in)     :: iatsym,iatiso
      integer(long)                                  :: i
      real(dop), dimension(ncoo), intent(out)        :: mass
      character(len=*), dimension(ncoo), intent(out) :: atsym

!-----------------------------------------------------------------------
! assign masses
! and get atom name
!-----------------------------------------------------------------------
      do i=1,ncoo
         call getmass(mass(i),iatsym(i),iau,ilog)
         write(atsym(i),'(i0)') iatsym(i)
         call getatnam(atsym(i),ilog)
      enddo

!-----------------------------------------------------------------------
! check for isotopes
!-----------------------------------------------------------------------
      do i=1,ncoo
         if (iatsym(i) .eq. 1 .and. iatiso(i) .eq. 2) then
            atsym(i) = 'H(Iso=2)'
            mass(i) = 2.0141_dop
         endif
      enddo

      return
      end subroutine assmass

!#######################################################################

      subroutine casmolcas(unit,energy,e0,istate,found,ilog)

      implicit none

      integer(long), intent(in)    :: unit,istate,ilog
      integer(long)                :: s,stat
      real(dop), intent(in)        :: e0
      real(dop), intent(out)       :: energy
      real(dop), dimension(100)    :: ener
      logical(kind=4), intent(out) :: found
      character(len=80)            :: string

      ener(istate)=-99999.0

      s=0


10    continue
         read(unit,'(a)') string
         if (string(7:34) .ne. 'printout of CI-coefficients') then
            go to 10
         endif
         read(string,'(62x,i3)') stat
         if (stat .eq. istate) then
             s=int(stat)
         else
             go to 10
         endif
         read(unit,'(a)') string
         read(string,'(17x,f10.6)') ener(s)
         energy=ener(s)*uev - e0
 !     endif

      found=.true.

      return
      end subroutine casmolcas

!#######################################################################

      subroutine pt2molcas(unit,energy,e0,istate,found,ilog)

         implicit none
   
         integer(long), intent(in)    :: unit,istate,ilog
         integer(long)                :: s,stat
         real(dop), intent(in)        :: e0
         real(dop), intent(out)       :: energy
         real(dop), dimension(100)    :: ener
         logical(kind=4), intent(out) :: found
         character(len=80)            :: string
   
         ener(istate)=-99999.0
   
         s=0
   
   
   10    continue
            read(unit,'(a)') string
            if (string(7:18) .ne. 'CASPT2 Root') then
               go to 10
            endif
            read(string,'(19x,i3)') stat
            if (stat .eq. istate) then
                s=int(stat)
            else
                go to 10
            endif
            read(string,'(41x,f12.8)') ener(s)
            energy=ener(s)*uev - e0
    !     endif

            
         found=.true.
   
         return
         end subroutine pt2molcas

!#######################################################################

      subroutine msmolcas(unit,energy,e0,istate,found,ilog)

         implicit none

         integer(long), intent(in)    :: unit,istate,ilog
         integer(long)                :: s,stat
         real(dop), intent(in)        :: e0
         real(dop), intent(out)       :: energy
         real(dop), dimension(100)    :: ener
         logical(kind=4), intent(out) :: found
         character(len=80)            :: string

         ener(istate)=-99999.0

         s=0

   10    continue
            read(unit,'(a)') string
            if (string(7:21) .ne. 'MS-CASPT2 Root') then
               go to 10
            endif
            read(string,'(22x,i3)') stat
            if (stat .eq. istate) then
                s=int(stat)
            else
                go to 10
            endif
            read(string,'(44x,f12.8)') ener(s)
            energy=ener(s)*uev - e0
    !     endif


         found=.true.

         return
         end subroutine msmolcas

!#######################################################################

      subroutine rmsmolcas(unit,energy,e0,istate,found,ilog)

         implicit none

         integer(long), intent(in)    :: unit,istate,ilog
         integer(long)                :: s,stat
         real(dop), intent(in)        :: e0
         real(dop), intent(out)       :: energy
         real(dop), dimension(100)    :: ener
         logical(kind=4), intent(out) :: found
         character(len=80)            :: string

         ener(istate)=-99999.0

         s=0


   10    continue
            read(unit,'(a)') string
            if (string(7:22) .ne. 'RMS-CASPT2 Root') then
               go to 10
            endif
            read(string,'(23x,i3)') stat
            if (stat .eq. istate) then
                s=int(stat)
            else
                go to 10
            endif
            read(string,'(45x,f12.8)') ener(s)
            energy=ener(s)*uev - e0
    !     endif


         found=.true.

         return
         end subroutine rmsmolcas

!##########################################################################
 
      subroutine ept2mcas(unit,energy,e0,istate,found,maxsta)
      implicit none

      integer(long), intent(in)    :: unit,istate,maxsta
      integer(long)                :: n,s
      real(dop), intent(in)        :: e0
      real(dop), intent(out)       :: energy
      real(dop), dimension(maxsta) :: ener
      real(dop)                    :: stat
      logical(kind=4), intent(out) :: found
      character(len=80)            :: string

      do n=1,maxsta
         write(*,*)"maxsta",maxsta
         ener(n)=-99999.0
      enddo

      s=0
      n=1


   10 continue
      read(unit,'(a)') string
       if (string(1:17) .eq. '::    RASSI State') then
       write(*,*)"found a state"
       else if (string(1:17) .ne. '::    RASSI State') then
         go to 10
       else if ((string(25:38) .ne. 'Total energy:') .and. &
          (string(25:38) .ne. 'Total Energy:')) then
         go to 10
      endif
      write(*,*)"string",string
      read(string,'(19x,f1.0)') stat
      write(*,*)"stat",stat
      if (stat .eq. istate) then
         s=int(stat)
      else
         go to 10
      endif
      read(string,'(41x,f11.4)') ener(s)
      energy=ener(s)*uev - e0

      found=.true.

      return
      end subroutine ept2mcas

!##########################################################################

      subroutine g09soc(unit,energy)

      implicit none

      integer(long), intent(in)    :: unit
      integer(long)                :: state1,state2
      real(dop), intent(out)       :: energy
      real(dop)                    :: spin
      character(len=80)            :: string

      state1=0
      state2=0
      spin=0.0_dop

   10 continue
      read(unit,'(a)') string
      if (string(3:29) .ne. 'Spin-Orbit coupling program') then
         go to 10
      endif
  
   20 continue 
      read(unit,'(a)') string
      if (string(2:13) .ne. '1st state is') then
         go to 20
      else
         backspace(unit)
         read(string,'(24x,i2)') state1 
      endif 

   30 continue
      read(unit,'(a)') string
      if (string(2:13) .ne. '2nd state is') then
         go to 30
      else
         backspace(unit)
         read(string,'(24x,i2)') state2
      endif

   40 continue
      read(unit,'(a)') string
      if (string(2:17) .ne. 'Total magnitude=') then
         go to 40
      else
         backspace(unit)
         read(string,'(22x,f3.1)') spin 
      endif

      energy=spin

      return
      end subroutine g09soc

!#######################################################################

      subroutine energeom(unit,energy,e0,istate,found)

      implicit none

      integer(long), intent(in)    :: unit,istate
      integer(long)                :: ngeom,count,i
      real(dop), intent(in)        :: e0
      real(dop), intent(out)       :: energy
      logical(kind=4), intent(out) :: found
      character(len=80)            :: string

      energy=-999999.0

!-----------------------------------------------------------------------
! Determine no. geometries present
!-----------------------------------------------------------------------
      ngeom=0

 100  continue
      read(unit,'(a)',end=110) string
      if (string(2:26).ne.'Starting EOM-CCSD program') goto 100
      ngeom=ngeom+1
      goto 100

 110  rewind(unit)      

!-----------------------------------------------------------------------
! Read to last calculation
!-----------------------------------------------------------------------
      count=0
 
 120  continue
      read(unit,'(a)',end=999) string
      if (string(2:26).ne.'Starting EOM-CCSD program') goto 120
      count=count+1
      if (count.lt.ngeom) goto 120

!-----------------------------------------------------------------------
! Read energies
!-----------------------------------------------------------------------

 10   continue
      read(unit,'(a)') string
      if (string(3:16).ne.'Final Results:') goto 10

      read(unit,*)
      read(unit,*)
      
      do i=1,istate
         read(unit,*)
      enddo
      
      read(unit,'(a)') string
      read(string(26:38),'(f12.8)') energy

      energy=energy*uev - e0
      
      found=.true.
      
 999  continue

      return
      end subroutine energeom

!#######################################################################

      subroutine energdftqchem(iunit,energy,e0,istate,found)

      integer(long), intent(in)    :: iunit,istate
      integer(long)                :: excst,b_ind,e_ind,stread
      real(dop), intent(in)        :: e0
      real(dop), intent(out)       :: energy
      real(dop)                    :: exen
      logical(kind=4), intent(out) :: found
      character(len=160)           :: string,estring

      excst = 0
      excst=istate - 1

      rewind(iunit)
      energy = 0d0 
      do
      read(iunit,'(a)',end=999,err=1000) string
         if (istate .eq. 1) then
            if (string(2:36) == "Total energy in the final basis set") then
               read(string(40:58),'(f19.10)') energy
               energy=(energy*uev) - e0
               found = .true.
               exit
            endif  
         else
            if (string(2:23) == "Total energy for state") then
               b_ind = index(string, "Total energy for state")
               e_ind = index(string, ":")
               read(string(b_ind+22:e_ind-1),'(I3)',err=1000) stread
               if (stread .eq. excst) then 
                  estring = adjustl(string(e_ind+1:len_trim(string)))
                  b_ind = index(estring, 'au')
                  estring = estring(1:b_ind-2)
                  read(estring,'(f14.8)',err=1000) exen 
                  energy = (exen*uev) - e0
                  found = .true.
                  exit 
               endif
            endif
         endif
      enddo

  999 continue

      return

 1000 print*,'ERROR reading Q-Chem output (Energy)'
      found = .false.
      return

      end subroutine energdftqchem

!#######################################################################

      subroutine energptpcmqchem(iunit,energy,istate,found)

      integer(long), intent(in)    :: iunit,istate
      real(dop), intent(out)       :: energy
      logical(kind=4), intent(out) :: found
      character(len=160)           :: string
      integer(long)                :: i
      interface
        function streq(str1,str2)
          logical :: streq
          character(len=*),intent(in) :: str1,str2
        end function streq
       end interface

      rewind(iunit)
       if (istate.eq.1)then
               energy = 0.0
               goto 100
       else 
         do
            read(iunit,'(a)') string
            if (string(19:46) == "SUMMARY OF LR-PCM AND SS-PCM") then
               do
                  read(iunit,'(a)') string
                    if (string(2:32) == "Unrelaxed singlet excited state") then
                     if (istate .le. 10) then
                        read(string(37:37),*) i
                     else
                        read(string(37:38),*) i
                     endif
                     if (i == istate-1) then
                       read(iunit,'(a)') string
                     if (string(4:47) == "Total  1st-order corrected excitation energy") then
                        read(string(54:61),'(f8.6)') energy
                        goto 100
                     endif
                     endif
                     endif
                 enddo
               exit
            endif
         enddo
      endif
      !
100      found = .true.
      !
      return

      end subroutine energptpcmqchem

!#######################################################################

      subroutine tdipmolcas(unit,tdip,n,found,negphase)

      implicit none

      integer(long), intent(in)            :: unit,n,negphase
      integer(long)                        :: s,i,ngeom,count,s1
      real(dop), dimension(4), intent(out) :: tdip
      real(dop), dimension(3)              :: vec
      real(dop)                            :: mod
      character(len=80)                    :: string
      character(len=1)                     :: cmpt
      logical(kind=4), intent(out)         :: found

!-----------------------------------------------------------------------
! determine no. geometries present
!-----------------------------------------------------------------------
      ngeom=0

 100  continue
      read(unit,'(a)',end=110) string
      if (string(12:48).ne.'MULTI (Direct Multiconfiguration SCF)') goto 100
      ngeom=ngeom+1
      goto 100

 110  rewind(unit)

!-----------------------------------------------------------------------
! read to last calculation
!-----------------------------------------------------------------------
      count=0

 120  continue
      read(unit,'(a)',end=999) string
      if (string(12:48).ne. 'MULTI (Direct Multiconfiguration SCF)') goto 120
      count=count+1
      if (count.lt.ngeom) goto 120
      
      tdip(:)=0.0_dop
      vec(:)=0.0_dop
   10 continue
      read(unit,'(a)',end=999) string
      if (string(2:37).ne. 'State-averaged charge density matrix') goto 10
      found=.true.

      read(unit,*)

   20 continue
      read(unit,'(a)',end=999) string
      if (string(2:17).eq.'NATURAL ORBITALS') goto 900

      if (string(2:13).eq.'!MCSCF trans') then
         read(string(24:24),*) s
         read(string(32:32),*) s1
         read(string(30:30),*) cmpt
      
         if (s.eq.1.or.s1.eq.1) then
            if (s.eq.n.or.s1.eq.n) then
               if (n.gt.1) then
                  
                  if (cmpt.eq.'X') then
                     read(string(40:54),*) vec(1)
                  else if (cmpt.eq.'Y') then
                     read(string(40:54),*) vec(2)
                  else
                     read(string(40:54),*) vec(3)
                  endif
               endif
            endif
         endif

      endif

      goto 20
      
 900   continue

       if (negphase.eq.1) then
          do i=1,3
             vec(i)=-vec(i)
          enddo
       endif

       mod=0.0_dop
       do i=1,3
          mod=mod + vec(i)*vec(i)
       enddo
                  
       if (mod .ge. 1.0e-8_dop) then
          mod=dsqrt(mod)
! tdip(i), i=1-3 is unit vector along dipole
          do i=1,3
             tdip(i) = vec(i) / mod 
          enddo
! as convention take vector direction with positive dot product with (1,1,1)
          mod=0.0_dop
          do i=1,3
             mod = mod + tdip(i)
          enddo
          if (mod .lt. 0.0_dop) then
             do i=1,3
                tdip(i)=-tdip(i)
             enddo
          endif

! tdip(4) is magnitude of dipole
          tdip(4)=0.0_dop
          do i=1,3
             tdip(4) = tdip(4) + vec(i) * tdip(i)
          enddo
          
       else
          do i=1,4
             tdip(i) = 0.0_dop
          enddo
       endif
       
 999  continue
      
      return
      end subroutine tdipmolcas

!#######################################################################

      subroutine tdipmolpt2(unit,tdip,n,found,negphase)

      implicit none

      integer(long), intent(in)            :: unit,n,negphase
      integer(long)                        :: s,i,s1
      real(dop), dimension(4), intent(out) :: tdip
      real(dop), dimension(3)              :: vec
      real(dop)                            :: mod
      character(len=80)                    :: string
      character(len=1)                     :: cmpt
      logical(kind=4), intent(out)         :: found

      tdip(:)=0.0_dop
      vec(:)=0.0_dop
 10   continue
      read(unit,'(a)',end=999) string
      if (string(2:12).ne.'!RSPT trans') goto 10
      found=.true.
      backspace(unit)
      
 20   continue
      read(unit,'(a)',end=999) string
      if (string(2:30).eq.'*****************************') goto 900
      
      if (string(2:12).eq.'!RSPT trans') then
         read(string(24:24),*) s
         read(string(32:32),*) s1
         read(string(30:30),*) cmpt
         
         if (s.eq.1.or.s1.eq.1) then
            if (s.eq.n.or.s1.eq.n) then
               if (n.gt.1) then
         
                  if (cmpt.eq.'X') then
                     read(string(40:54),*) vec(1)
                  else if (cmpt.eq.'Y') then
                     read(string(40:54),*) vec(2)
                  else
                     read(string(40:54),*) vec(3)
                  endif
               
               endif
            endif
         endif

      endif
   
      goto 20
      
 900   continue

       if (negphase.eq.1) then
          do i=1,3
             vec(i)=-vec(i)
          enddo
       endif
       
       mod=0.0_dop
       do i=1,3
          mod=mod + vec(i)*vec(i)
       enddo
       
       if (mod .ge. 1.0e-8_dop) then
          mod=dsqrt(mod)
! tdip(i), i=1-3 is unit vector along dipole
          do i=1,3
             tdip(i) = vec(i) / mod 
          enddo
! as convention take vector direction with positive dot product with (1,1,1)
          mod=0.0_dop
          do i=1,3
             mod = mod + tdip(i)
          enddo
          if (mod .lt. 0.0_dop) then
             do i=1,3
                tdip(i)=-tdip(i)
             enddo
          endif
                     
! tdip(4) is magnitude of dipole
          tdip(4)=0.0_dop
          do i=1,3
             tdip(4) = tdip(4) + vec(i) * tdip(i)
          enddo

       else
          do i=1,4
             tdip(i) = 0.0_dop
          enddo
       endif


 999   continue

      return
      end subroutine tdipmolpt2

!#######################################################################

      subroutine dipole_molpt2(unit,dipole,found,nsta,phaseswitch)

      implicit none

      integer(long), intent(in)            :: unit,nsta
      integer(long)                        :: i,j,k,s,s1
      integer(long), dimension(nsta,nsta), intent(in) :: phaseswitch
      real(dop), dimension(nsta,nsta,3), intent(out)  :: dipole
      character(len=90)                    :: string
      character(len=1)                     :: cmpt
      logical(kind=4), intent(out)         :: found

!-----------------------------------------------------------------------
! Zero dipole array
!-----------------------------------------------------------------------
      do i=1,3
         do j=1,nsta
            do k=1,nsta
               dipole(k,j,i)=0.0_dop
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! Read dipole moments
!-----------------------------------------------------------------------
 30   continue
      read(unit,'(a)',end=999) string
      if (string(2:25).ne.'Coefficient of reference') goto 30
      found=.true.
      
 40   continue
      read(unit,'(a)',end=999) string
      if (string(2:30).eq.'*****************************') goto 999
      
      if (string(2:13).eq.'!RSPT2 STATE'.and.string(19:24).eq.'Dipole') then
         read(string(15:15),*) s
         if (s.le.nsta) then
            read(string(40:50),'(F11.8)') dipole(s,s,1)
            read(string(55:65),'(F11.8)') dipole(s,s,2)
            read(string(70:80),'(F11.8)') dipole(s,s,3)
         endif
      endif

      goto 40

 999  continue

      found=.false.
!-----------------------------------------------------------------------
! Read transition dipole moments
!-----------------------------------------------------------------------
      rewind(unit)
 10   continue
      read(unit,'(a)',end=900) string
      if (string(2:12).ne.'!RSPT trans') goto 10
      found=.true.
      backspace(unit)
      
 20   continue
      read(unit,'(a)',end=900) string
      if (string(2:30).eq.'*****************************') goto 900
      
      if (string(2:12).eq.'!RSPT trans') then
         read(string(24:24),*) s
         read(string(32:32),*) s1

         if (s.le.nsta.and.s1.le.nsta) then
            read(string(30:30),*) cmpt
            if (cmpt.eq.'X') then
               read(string(40:54),*) dipole(s,s1,1)
               if (phaseswitch(s,s1).eq.1) dipole(s,s1,1)=-dipole(s,s1,1)
               dipole(s1,s,1)=dipole(s,s1,1)
            else if (cmpt.eq.'Y') then
               read(string(40:54),*) dipole(s,s1,2)
               if (phaseswitch(s,s1).eq.1) dipole(s,s1,2)=-dipole(s,s1,2)
               dipole(s1,s,2)=dipole(s,s1,2)
            else if (cmpt.eq.'Z') then
               read(string(40:54),*) dipole(s,s1,3)
               if (phaseswitch(s,s1).eq.1) dipole(s,s1,3)=-dipole(s,s1,3)
               dipole(s1,s,3)=dipole(s,s1,3)
            endif
         endif

      endif

      goto 20
      
 900   continue

      return
      end subroutine dipole_molpt2

!#######################################################################

      subroutine dipole_moleom(unit,dipole,found,nsta,phaseswitch)

      implicit none

      integer(long), intent(in)            :: unit,nsta
      integer(long)                        :: i,j,k,s,s1
      integer(long), dimension(nsta,nsta), intent(in) :: phaseswitch
      real(dop), dimension(nsta,nsta,3), intent(out)  :: dipole
      character(len=90)                    :: string
      logical(kind=4), intent(out)         :: found

!-----------------------------------------------------------------------
! Zero dipole array
!-----------------------------------------------------------------------
      do i=1,3
         do j=1,nsta
            do k=1,nsta
               dipole(k,j,i)=0.0_dop
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! Read dipole moments and transition dipole moments
!-----------------------------------------------------------------------
 30   continue
      read(unit,'(a)',end=999) string
      if (string(2:28).ne.'EOM-CCSD properties program') goto 30
      found=.true.

      do i=1,8
         read(unit,*)
      enddo

! First read ground state dipole moment, as this is written using a 
! different format to the remaining elements
      read(unit,'(a)',end=999) string
      read(string(40:50),'(F11.8)') dipole(1,1,1)
      read(string(55:65),'(F11.8)') dipole(1,1,2)
      read(string(70:80),'(F11.8)') dipole(1,1,3)
      
! Now read all other elements
 40   continue
      read(unit,'(a)',end=999) string
      if (string(2:4).ne.'Bra') goto 40
      
      read(string(9:10),*) s
      read(string(28:29),*) s1

! EOM-CCSD dipole matrix is not symmetric so, rather arbitrarily,
! read the lower triangle
      if (s.le.nsta.and.s1.le.nsta.and.s.le.s1) then
         read(unit,*)
         read(unit,'(a)',end=999) string
         read(string(40:50),'(F11.8)') dipole(s,s1,1)
         read(string(55:65),'(F11.8)') dipole(s,s1,2)
         read(string(70:80),'(F11.8)') dipole(s,s1,3)

         if (s.ne.s1) then
            do i=1,3
               if (phaseswitch(s,s1).eq.1) dipole(s,s1,i)=-dipole(s,s1,i)
               dipole(s1,s,i)=dipole(s,s1,i)
            enddo
         endif
      
      endif
      
      if (string(25:50).ne.'Final Results for EOM-CCSD') goto 40

 999  continue

      return
      end subroutine dipole_moleom

!#######################################################################
      
      subroutine dipole_g03hf(unit,dipole,found,nsta)
      
      implicit none

      integer(long), intent(in)            :: unit,nsta
      integer(long)                        :: i,j,k
      real(dop), dimension(nsta,nsta,3), intent(out)  :: dipole
      character(len=90)                    :: string
      logical(kind=4), intent(out)         :: found

!-----------------------------------------------------------------------
! Zero dipole array
!-----------------------------------------------------------------------
      do i=1,3
         do j=1,nsta
            do k=1,nsta
               dipole(k,j,i)=0.0_dop
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! Read dipole moment vector
!-----------------------------------------------------------------------
 30   continue
      read(unit,'(a)',end=999) string
      if (string(2:14).ne.'Dipole moment') goto 30
      found=.true.
      
      read(unit,'(a)') string
      read(string(11:17),'(F7.4)') dipole(1,1,1)
      read(string(28:34),'(F7.4)') dipole(1,1,2)
      read(string(45:51),'(F7.4)') dipole(1,1,3)

!-----------------------------------------------------------------------
! Convert to atomic units
!-----------------------------------------------------------------------
      do i=1,3
         dipole(1,1,i)=dipole(1,1,i)*0.393456d0
      enddo


 999  continue

      return
      end subroutine dipole_g03hf

!#######################################################################

      subroutine tdipmoleom(unit,tdip,n,found,negphase)

      implicit none

      integer(long), intent(in)            :: unit,n,negphase
      integer(long)                        :: s,i,ngeom,count
      real(dop), dimension(4), intent(out) :: tdip
      real(dop), dimension(3)              :: vec
      real(dop)                            :: mod
      character(len=80)                    :: string
      logical(kind=4), intent(out)         :: found

!-----------------------------------------------------------------------
! determine no. geometries present
!-----------------------------------------------------------------------
      ngeom=0

 100  continue
      read(unit,'(a)',end=110) string
      if (string(12:46).ne.'CCSD (Closed-shell coupled cluster)') goto 100
      ngeom=ngeom+1
      goto 100

 110  rewind(unit)

!-----------------------------------------------------------------------
! read to last calculation
!-----------------------------------------------------------------------
      count=0

 120  continue
      read(unit,'(a)',end=999) string
      if (string(12:46).ne. 'CCSD (Closed-shell coupled cluster)') goto 120
      count=count+1
      if (count.lt.ngeom) goto 120
      
      tdip(:)=0.0_dop

 10   continue
      read(unit,'(a)',end=999) string
      if (string(25:50).ne. 'Final Results for EOM-CCSD') goto 10
      found=.true.
     
 20   continue
      read(unit,'(a)',end=999) string
      if (string(2:9).eq.'DATASETS') goto 999

      if (string(3:7).eq.'State') then
         vec(:)=0.0_dop
         read(unit,'(a)',end=999)string
         read(string(4:4),*) s
         if (s.eq.n) then
            read(unit,*)
            read(unit,'(a)') string
            read(string(35:45),*) vec(1)
            read(string(50:60),*) vec(2)
            read(string(65:75),*) vec(3)

            if (negphase.eq.1) then
               do i=1,3
                  vec(i)=-vec(i)
               enddo
            endif

            mod=0.0_dop
            do i=1,3
               mod=mod + vec(i)*vec(i)
            enddo
                  
            if (mod .ge. 1.0e-8_dop) then
               mod=dsqrt(mod)
! tdip(i), i=1-3 is unit vector along dipole
               do i=1,3
                  tdip(i) = vec(i) / mod 
               enddo
! as convention take vector direction with positive dot product with (1,1,1)
               mod=0.0_dop
               do i=1,3
                  mod = mod + tdip(i)
               enddo
               if (mod .lt. 0.0_dop) then
                  do i=1,3
                     tdip(i)=-tdip(i)
                  enddo
               endif

! tdip(4) is magnitude of dipole
               tdip(4)=0.0_dop
               do i=1,3
                  tdip(4) = tdip(4) + vec(i) * tdip(i)
               enddo

            else
               do i=1,4
                  tdip(i) = 0.0_dop
               enddo
            endif
            
         endif
      endif


      goto 20

 999  continue

      return
      end subroutine tdipmoleom

!#######################################################################

      subroutine tdip_dftmrci(unit,tdip,n,found,lvelocity,negphase)

      implicit none

      integer(long), intent(in)            :: unit,n,negphase
      integer(long)                        :: s,i
      real(dop), dimension(4), intent(out) :: tdip
      real(dop), dimension(3)              :: vec
      real(dop)                            :: mod
      character(len=80)                    :: string
      logical(kind=4), intent(out)         :: found
      logical(kind=4), intent(in)          :: lvelocity

      tdip(:)=0.0_dop

 10   continue
      read(unit,'(a)',end=999) string
      if (string(22:52).ne. 'T - D E N S I T Y - O U T P U T') goto 10
      found=.true.
      
 20   continue
      read(unit,'(a)',end=999) string
      if (string(22:48).eq.'D E N S I T Y - O U T P U T') goto 999
      
      if (string(1:10).eq.'transition') then
         read(string(18:18),*) s
         
         if (s.eq.n) then
            vec(:)=0.0_dop
            read(unit,*)
            read(unit,*)
! Velocity and Length gauge results are separated by one line
            if (lvelocity) read(unit,*)

            read(unit,'(a)',end=999) string
            read(string(19:28),*) vec(1)
            read(string(31:40),*) vec(2)
            read(string(43:52),*) vec(3)

            if (negphase.eq.1) then
               do i=1,3
                  vec(i)=-vec(i)
               enddo
            endif

            mod=0.0_dop
            do i=1,3
               mod=mod + vec(i)*vec(i)
            enddo
         
            if (mod .ge. 1.0e-8_dop) then
               mod=dsqrt(mod)
! tdip(i), i=1-3 is unit vector along dipole
               do i=1,3
                  tdip(i) = vec(i) / mod 
               enddo
! as convention take vector direction with positive dot product with (1,1,1)
               mod=0.0_dop
               do i=1,3
                  mod = mod + tdip(i)
               enddo
               if (mod .lt. 0.0_dop) then
                  do i=1,3
                     tdip(i)=-tdip(i)
                  enddo
               endif

! tdip(4) is magnitude of dipole
               tdip(4)=0.0_dop
               do i=1,3
                  tdip(4) = tdip(4) + vec(i) * tdip(i)
               enddo

            else
               do i=1,4
                  tdip(i) = 0.0_dop
               enddo
            endif
            
         endif
      endif
     
      goto 20

 999  continue

      return
      end subroutine tdip_dftmrci

!#######################################################################
! outputs the derivative coupling as der1 in atomic units (Hartrees/Bohr)

      subroutine cpmcscfg94(unit,der1,ista,found,maxdim,ncoo)

      implicit none

      integer(long), intent(in)                 :: unit,ista,maxdim,ncoo
      integer(long)                             :: i,j,n
      real(dop), dimension(maxdim), intent(out) :: der1
      real(dop), dimension(3)                   :: xdum
      character(len=80)                         :: string,match
      logical(kind=4), intent(out)              :: found

      do i=1,ncoo
         der1(i)=0.0_dop
      enddo

!-----------------------------------------------------------------------
! get zero geometry
!-----------------------------------------------------------------------
   10 continue
      read(unit,'(a)',end=999) string

      if (string(2:20) .ne. 'Derivative Coupling') go to 10

!-----------------------------------------------------------------------
! see which states are coupled
!-----------------------------------------------------------------------
      backspace(unit)
      backspace(unit)
      backspace(unit)
      read(unit,'(10x,i5,8x,i5)') i,j
      if (i .ne. ista .and. j .ne. ista) return
      if (i .eq. ista) then
         match='Gradient of iVec State.  '
      else
         match='Gradient of iOther State '
      endif


   30 read(unit,'(a)',end=999) string
      if (string(2:26) .ne. match(1:25)) go to 30

      found=.true.

      n=0
      do i=1,ncoo,3
         read(unit,20) (xdum(j),j=1,3)
         do j=1,3
            n=n+1
            der1(n)=xdum(j)
         enddo
      enddo
   20 format(1x,3f20.10)

      go to 10

  999 continue
      return
      end subroutine cpmcscfg94

!#######################################################################
! outputs the gradient as der1 in atomic units (Hartrees/Bohr)
!
! This routine needs checking 

      subroutine der1qchem(unit,der1,found,maxdim,ncoo,ista)

      implicit none

      integer(long), intent(in)                 :: unit,maxdim,ncoo
      integer(long)                             :: i,j,n,ista
      real                                      :: kk
      real(dop), dimension(maxdim), intent(out) :: der1
      real(dop), dimension(3)                   :: xdum
      character(len=80)                         :: string
      logical(kind=4), intent(out)              :: found
      do i=1,ncoo
         der1(i)=0.0_dop
      enddo
 10   continue
      read(unit,'(a)',end=999) string

      if (string(2:19).eq.'State A: ccsd: 0/A') then
         i=0
         goto 12
      else if (string(2:20).eq.'State A: eomee_ccsd') then
         if(string(40:40).eq.'/') then
         read(string(39:39),*) i
         else
         read(string(39:40),*) i
         end if
      else if (string(2:20).eq.'State A: eomip_ccsd') then
         if(string(26:26).eq.'/') then
         read(string(25:25),*) i
         else
         read(string(25:26),*) i
         end if
      else
         goto 10
      endif

!-----------------------------------------------------------------------
! See which states are coupled
!-----------------------------------------------------------------------
 12   continue
      read(unit,'(a)',end=999) string
      if (string(2:19).eq.'State B: ccsd: 0/A')  goto 10
      if ((string(2:20).ne.'State B: eomee_ccsd').and.   &
      (string(2:20).ne.'State B: eomip_ccsd'))     goto 12

      if (string(2:27).eq.'State B: eomee_ccsd/uhfref') then
         if(string(31:31).eq.'/') then
            read(string(30:30),*) j
         else
            read(string(30:31),*) j
         end if
      else if (string(2:20).eq.'State B: eomee_ccsd') then
         if(string(40:40).eq.'/') then
         read(string(39:39),*) j
         else
         read(string(39:40),*) j
         end if
      else if (string(2:20).eq.'State B: eomip_ccsd') then
         if(string(26:26).eq.'/') then
         read(string(25:25),*) j
         else
         read(string(25:26),*) j
         end if
      end if

!      if (i .ne. ista .or. j .ne. ista) return
      if (i .ne. ista-1 .and. j .ne. ista-1) goto 10
      if (i .eq. ista-1) then
 13   continue
      read(unit,'(a)',end=999) string
      if (string(1:9).ne.'G_I, a.u.') goto 13
      found=.true.
      elseif (j .eq. ista-1) then
 14   continue
      read(unit,'(a)',end=999) string
      if (string(1:9).ne.'G_J, a.u.') goto 14
      found=.true.
      endif
      read(unit,*) string
      read(unit,*) string
      n=0
      do i=1,ncoo,3
         read(unit,20) kk,(xdum(j),j=1,3)
         do j=1,3
            n=n+1
            der1(n)=xdum(j)
         enddo
      enddo


   20 format(3x,f2.0,2x,3f18.8)

  999 continue
      return
      end subroutine der1qchem
      
!#######################################################################
! outputs the gradient as der1 in atomic units (Hartrees/Bohr)

      subroutine der1molpro(unit,der1,ista,found,maxdim,ncoo)
      
      implicit none

      integer(long), intent(in)                 :: unit,maxdim,ncoo,ista
      integer(long)                             :: i,j,n
      real(dop), dimension(maxdim), intent(out) :: der1
      real(dop), dimension(3)                   :: xdum
      character(len=80)                         :: string
      logical(kind=4), intent(out)              :: found

      do i=1,ncoo
         der1(i)=0.0_dop
      enddo

 10   continue
      read(unit,'(a)',end=999) string

      if (string(8:25).ne.'GRADIENT FOR STATE') goto 10

      read(string(26:27),'(i2)') i
      if (i .ne. ista) go to 10
      
      found=.true.
      
      read(unit,*)
      read(unit,*)
      read(unit,*)
      n=0
      do i=1,ncoo,3
         read(unit,20) (xdum(j),j=1,3)
         do j=1,3
            n=n+1
            der1(n)=xdum(j)
         enddo
      enddo

   20 format(4x,3f20.9)

  999 continue
      return
      end subroutine der1molpro

!#######################################################################
! outputs the gradient difference vector as der1 in atomic units (Hartrees/Bohr)
!
! ROUTINE NEEDS ISTA, BUT THIS IS NOT PASSED????
!#######################################################################

      subroutine gdqchem(unit,der1,found,maxdim,ncoo)
!     subroutine gdqchem(unit,der1,found,maxdim,ncoo,ista,jsta)


         implicit none

         integer(long), intent(in)                 :: unit,maxdim,ncoo
         integer(long)                             :: i,j,n,ista
         real                                      :: kk
         real(dop), dimension(maxdim), intent(out) :: der1
         real(dop), dimension(3)                   :: xdum
         character(len=80)                         :: string
         logical(kind=4), intent(out)              :: found

         do i=1,ncoo
            der1(i)=0.0_dop
         enddo

    10   continue
         read(unit,'(a)',end=999) string
         if (string(2:20).ne.'State A: eomee_ccsd') goto 10
   
   !-----------------------------------------------------------------------
   ! See which states are coupled
   !-----------------------------------------------------------------------
         read(string(39:39),'(i1)') i
   
    12   continue
         read(unit,'(a)',end=999) string
         if (string(2:20).ne.'State B: eomee_ccsd') goto 12
   
         read(string(39:39),'(i1)') j
   
         if (i .ne. ista-1 .and. j .ne. ista-1) goto 10
         if (i .eq. ista-1) then
    13   continue
         read(unit,'(a)',end=999) string
         if (string(1:9).ne.'G_I, a.u.') goto 13
         found=.true.
         elseif (j .eq. ista-1) then
    14   continue
         read(unit,'(a)',end=999) string
         if (string(1:9).ne.'G_J, a.u.') goto 14 
         found=.true.
         endif
         read(unit,*) string
         read(unit,*) string
         n=0
         do i=1,ncoo,3
            read(unit,20) kk,(xdum(j),j=1,3)
            do j=1,3
               n=n+1
               der1(n)=xdum(j)
            enddo
         enddo
   
   
      20 format(3x,f2.0,2x,3f18.8)
     999 continue
         return

      end subroutine gdqchem
!#######################################################################

      subroutine der1pcmqchem(unit,der1,found,maxdim,ncoo,ista)

      implicit none

      integer(long), intent(in)                 :: unit,maxdim,ncoo
      integer(long)                             :: i,j,n,ista
      real                                      :: kk
      real(dop), dimension(maxdim), intent(out) :: der1
      real(dop), dimension(3)                   :: xdum
      character(len=80)                         :: string
      logical(kind=4), intent(out)              :: found
      do i=1,ncoo
         der1(i)=0.0_dop
      enddo
   
!-----------------------------------------------------------------------
! See which state is it 
!-----------------------------------------------------------------------

 10   continue
      if (ista-1 .eq. 0) then
         read(unit,'(a)',end=999) string
         if (string(2:34).ne.'PCM graident for the ground-state') then
            goto 10
         endif
         j = 0   
      endif

 12   continue
      if (ista-1 .gt. 0) then
         read(unit,'(a)',end=999) string
         if (string(1:45).ne.'Adjusting gradient with QM/MM Energy...State:') then
            goto 12
         endif
         read(string(47:47),'(i1)') j
         read(unit,'(a)',end=999) string
         read(unit,'(a)',end=999) string
         if (string(1:10).ne.' Computing') then
            goto 12
         endif
      endif 

      if (j .ne. ista-1) then 
         goto 10
      endif

      if (j .eq. ista-1) then
 13   continue
       read(unit,'(a)',end=999) string
       if (string(1:18).ne.' -- total gradient') then 
        goto 13
       endif
       found=.true.
      endif
      read(unit,*) string
      read(unit,*) string
      read(unit,*) string
      n=0
      do i=1,ncoo,3
         read(unit,20) kk,(xdum(j),j=1,3)
         do j=1,3
            n=n+1
            der1(n)=xdum(j)
         enddo
      enddo
   
   
!      20 format(3x,f2.0,2x,3f18.8)
      20 format(2x,f2.0,3f15.6)
   
     999 continue
         return

      end subroutine der1pcmqchem

!#######################################################################

      subroutine gdgaussian(unit,der1,found,maxdim,ncoo)

      implicit none

      integer(long), intent(in)                 :: unit,maxdim,ncoo
      integer(long)                             :: i,j,n
      real(dop), dimension(maxdim), intent(out) :: der1
      real(dop), dimension(3)                   :: xdum
      character(len=80)                         :: string
      logical(kind=4), intent(out)              :: found

      der1=0.0_dop

      do while(.not. found) 
        read(unit,'(a)',end=999) string

        if (index(string,'Unscaled Gradient Difference').ne.0) then
          found=.true.

! GD in GAUSSIAN is der1(2)-der1(1) in au

          n=0
          do i=1,ncoo,3
            read(unit,20) (xdum(j),j=1,3)
            do j=1,3
              n=n+1
              der1(n)=0.5*xdum(j)
            enddo
          enddo

        endif
      enddo

   20 format(1x,3f20.10)

  999 continue
      return
      end subroutine gdgaussian

!#######################################################################
! outputs the derivative coupling as der1 in atomic units (Hartrees/Bohr)

      subroutine dcpqchem(unit,der1,ista,jsta,found,maxdim,maxsta,ncoo,nsta)

      implicit none

      integer(long), intent(in)                 :: unit,maxdim,maxsta,&
                                                   ncoo,ista,nsta
      integer(long), dimension(maxsta), intent(out) :: jsta
      integer(long)                             :: i,j,n,jsta1,nact
      real                                      :: kk
      real(dop), dimension(maxdim,maxsta), intent(out) :: der1
      real(dop), dimension(3)                   :: xdum
      character(len=80)                         :: string
      logical(kind=4), intent(out)              :: found
!for Q-chem state 1 is the first excited state and not the GS
      nact = 0
      der1(:,:)=0.0_dop
      
! state 0, GS, state 1 first excited, etc.

10   continue
      read(unit,'(a)',end=999) string

      if (string(2:19).eq.'State A: ccsd: 0/A') then
         i=0
         goto 12
      else if (string(2:20).eq.'State A: eomee_ccsd') then
         if(string(40:40).eq.'/') then
            read(string(39:39),*) i
         else
            read(string(39:40),*) i
         end if
      else if (string(2:20).eq.'State A: eomip_ccsd') then
         if(string(26:26).eq.'/') then
            read(string(25:25),*) i
         else
            read(string(25:26),*) i
         end if
      else
         goto 10
      endif

!-----------------------------------------------------------------------
! See which states are coupled
!-----------------------------------------------------------------------
      read(string(39:39),'(i1)') i

 12   continue
      read(unit,'(a)',end=999) string
      if (string(2:19).eq.'State B: ccsd: 0/A')  goto 10
      if ((string(2:20).ne.'State B: eomee_ccsd').and.   &
      (string(2:20).ne.'State B: eomip_ccsd'))     goto 12

      if (string(2:27).eq.'State B: eomee_ccsd/uhfref') then
         if(string(31:31).eq.'/') then
            read(string(30:30),*) j
         else
            read(string(30:31),*) j
         end if
      else if (string(2:20).eq.'State B: eomee_ccsd') then
         if(string(40:40).eq.'/') then
         read(string(39:39),*) j
         else
         read(string(39:40),*) j
         end if
      else if (string(2:20).eq.'State B: eomip_ccsd') then
         if(string(26:26).eq.'/') then
         read(string(25:25),*) j
         else
         read(string(25:26),*) j
         end if
      end if


      if (i .ne. ista-1 .and. j .ne. ista-1) goto 10
      if (i .eq. ista-1) then
         jsta1 = j+1
      found=.true.
      elseif (j .eq. ista-1) then
      found=.true.
         jsta1 = i+1
      endif


 13   continue
      read(unit,'(a)',end=999) string
      if (string(1:9).ne.'NAC Force') goto 13




!-----------------------------------------------------------------------
! only include coupling to higher states
!-----------------------------------------------------------------------
      if (jsta1 .lt. ista) go to 10

      found=.true.
      nact = nact + 1
      jsta(nact) = jsta1

!-----------------------------------------------------------------------
! read derivative coupling
!-----------------------------------------------------------------------
      read(unit,*)
      read(unit,*)

      n=0
      do i=1,ncoo,3
         read(unit,20) kk,(xdum(j),j=1,3)
         do j=1,3
            n=n+1
            der1(n,nact)=xdum(j)
         enddo
      enddo

      
!-----------------------------------------------------------------------
! multiply by energy gap between states
!-----------------------------------------------------------------------
!      der1(:,nact) = der1(:,nact)*(ener(jsta1)-ener(ista))

! carry on reading file
      go to 10


      20 format(3x,f2.0,2x,3f18.8)

  999 continue
      return
      end subroutine dcpqchem
!#######################################################################
! outputs the derivative coupling as der1 in atomic units (Hartrees/Bohr)

      subroutine dcppcmqchem(unit,der1,ista,jsta,found,maxdim,maxsta,ncoo)

      implicit none

      integer(long), intent(in)                 :: unit,maxdim,maxsta,ncoo,ista
      integer(long), dimension(maxsta), intent(out) :: jsta
      integer(long)                             :: i,j,n,jsta1,nact
      real                                      :: kk
      real(dop), dimension(maxdim,maxsta), intent(out) :: der1
      real(dop), dimension(3)                   :: xdum
      character(len=80)                         :: string
      logical(kind=4), intent(out)              :: found

!for Q-chem state 1 is the first excited state and not the GS
      nact = 0
      der1(:,:)=0.0_dop
      
! state 0, GS, state 1 first excited, etc.
 10   continue
      read(unit,'(a)',end=999) string

!-----------------------------------------------------------------------
! See which states are coupled
!-----------------------------------------------------------------------
      if (string(15:28).eq.'between states') then
         read(string(30:30),'(i1)') i
         read(string(36:36),'(i1)') j
      else  
         goto 10
      endif

      if (i .ne. ista-1 .and. j .ne. ista-1) then
       goto 10
      endif 
      if (i .eq. ista-1) then
         jsta1 = j+1
      found=.true.
      elseif (j .eq. ista-1) then
      found=.true.
         jsta1 = i+1
      endif

!      if (string(1:27).eq.'Total energy for state ',i) then
!              read(string(46:58),'(f13.8)') ener1
!      endif

      read(unit,'(a)') string
         do while(string(1:24)/="CIS Force Matrix Element")  
         !&
!         do while(string(1:46)/="DC between ground and excited states with ETF:"&
!                    & .and. string(1:32)/="CIS derivative coupling with ETF")
          read(unit,'(a)') string
         enddo

!-----------------------------------------------------------------------
! only include coupling to higher states
!-----------------------------------------------------------------------
      if (jsta1 .lt. ista) then
       go to 10
      endif

      found=.true.
      nact = nact + 1
      jsta(nact) = jsta1

!-----------------------------------------------------------------------
! read derivative coupling
!-----------------------------------------------------------------------
      n=0
      read(unit,'(a)') string
      read(unit,'(a)') string
      do i=1,ncoo,3
         read(unit,20) kk,(xdum(j),j=1,3)
         do j=1,3
            n=n+1
            der1(n,nact)=xdum(j)
         enddo
      enddo

!-----------------------------------------------------------------------
! multiply by energy gap between states - not needed, reading the force
!-----------------------------------------------------------------------
!      der1(:,nact) = der1(:,nact)*delta
! carry on reading file
      go to 10

!      20 format(3x,f2.0,2x,3f18.8)
      20 format(2x,f2.0,3f15.6)

  999 continue
      return
      end subroutine dcppcmqchem

!#######################################################################
! gdmolcas_ci reads the gradient difference at a conical intersection resulting
! from a MECI optimization and reported by the module SLAPAF. It is appropriate
! for regularization diabatization.
! outputs the gradient difference vector as der1 in atomic units (Hartrees/Bohr)

      subroutine gdmolcas_ci(unit,der1,found,maxdim,ncoo,ilog)

      implicit none

      integer(long), intent(in)                 :: unit,maxdim,ncoo,ilog
      integer(long)                             :: i
      real(dop), dimension(maxdim), intent(out) :: der1
      real(dop)                                 :: sdelta_gh,bdelta_gh,&
                                                   scaling
      character(len=80)                         :: string
      character(len=20)                         :: dummy
      logical, intent(out)                      :: found

      found=.false.
      der1=0.0_dop

      do while(.not. found)
        read(unit,'(a)',end=999) string
        call rmlblnk(string) !remove leading blanks

        if(index(string,'Pitch (delta_gh)') .ne. 0) then
           read(string(25:),*,err=1000) sdelta_gh,dummy
        elseif(index(string,'Asymmetry (Delta_gh)') .ne. 0) then
           read(string(25:),*,err=1000) bdelta_gh
        elseif(index(string,'Local x') .ne. 0) then
           read(unit,*) !skip one line
           do i=1,ncoo,3
              read(unit,*,err=1001) der1(i),der1(i+1),der1(i+2)
           enddo
           found=.true.
        endif
      enddo

      scaling=sqrt((sdelta_gh**2)*(1+bdelta_gh))

      der1=scaling*der1

  999 continue
      return

 1000 write(ilog,'(a)') 'ERROR reading MOLCAS output (CI parameters)'
      return
 1001 write(ilog,'(a)') 'ERROR reading MOLCAS output (Gradient difference vector)'
      return

      end subroutine gdmolcas_ci

!#######################################################################
! dcpmolcas_ci reads the derivative coupling at a conical intersection resulting
! from a MECI optimization and reported by the module SLAPAF. It is appropriate
! for regularization diabatization.
! For a generic NAC at any geometry the output of module ALASKA should be used
! instead.
! outputs the derivative as der1 in atomic units (Hartrees/Bohr)

      subroutine dcpmolcas_ci(unit,der1,found,maxdim,ncoo,ilog)

      implicit none

      integer(long), intent(in)                 :: unit,maxdim,ncoo,ilog
      integer(long)                             :: i
      real(dop), dimension(maxdim), intent(out) :: der1
      real(dop)                                 :: sdelta_gh,bdelta_gh,&
                                                   scaling
      character(len=80)                         :: string
      character(len=20)                         :: dummy
      logical, intent(out)                      :: found

      found=.false.
      der1=0.0_dop

      do while(.not. found)
        read(unit,'(a)',end=999) string
        call rmlblnk(string) !remove leading blanks

        if(index(string,'Pitch (delta_gh)') .ne. 0) then
           read(string(25:),*,err=1000) sdelta_gh,dummy
        elseif(index(string,'Asymmetry (Delta_gh)') .ne. 0) then
           read(string(25:),*,err=1000) bdelta_gh
        elseif(index(string,'Local y') .ne. 0) then
           read(unit,*) !skip one line
           do i=1,ncoo,3
              read(unit,*,err=1001) der1(i),der1(i+1),der1(i+2)
           enddo
           found=.true.
        endif
      enddo

      scaling=sqrt((sdelta_gh**2)*(1-bdelta_gh))

      der1=scaling*der1

  999 continue
      return

 1000 write(ilog,'(a)') 'ERROR reading MOLCAS output (CI parameters)'
      return
 1001 write(ilog,'(a)') 'ERROR reading MOLCAS output (Derivative coupling vector)'
      return


      end subroutine dcpmolcas_ci

!######################################################################      

      subroutine nmmolcas(unit,matrix,freq,mass,found, &
                 maxdim,ncoo,nmodes,invcm,ilog,s)

      implicit none

      integer(long), intent(in)   :: unit,maxdim,ncoo,nmodes,ilog,s
      integer(long)               :: i,j,n,nrd,n1,n2,nvib,nrt,ns
      real(dop), dimension(maxdim,maxdim), intent(out) :: matrix
      real(dop), dimension(maxdim), intent(in)         :: mass
      real(dop), dimension(maxdim), intent(out)        :: freq
      real(dop), intent(in)                            :: invcm
      real(dop)                                        :: norm
      real(dop), dimension(10)                         :: xdum
      logical(kind=4), intent(out)                     :: found
      logical(kind=4), dimension(maxdim)               :: vibmode
      character(len=80)                                :: string
      character(len=10), dimension(10)                 :: buffer

      interface
        function streq(str1,str2)
          logical :: streq
          character(len=*),intent(in) :: str1,str2
        end function streq
        function slen(str)
          integer :: slen
          character(len=*),intent(in) :: str
        end function slen
        function dnrm2(n,v,i)
           use decimal, only:dop
           real(dop) :: dnrm2
           integer,intent(in) :: n,i
           real(dop),dimension(n),intent(in) :: v
        end function dnrm2
      end interface

      freq(1:ncoo)=0.0_dop
      matrix(1:ncoo,1:ncoo)=0.0_dop
      found=.False.
      vibmode(1:ncoo)=.True. ! initialise first 3N modes to vibrations
      nvib=ncoo
      ns=0

!-----------------------------------------------------------------------
! Check if second derivatives are calculated numerically or analytically
!-----------------------------------------------------------------------
      do while ( nvib>nmodes )
        read(unit,'(a)',end=900) string
        call rmlblnk(string) !remove leading blanks

!-----------------------------------------------------------------------
! If numerically -> by default 3N-6 modes printed so okay to carry on!
! -> sometimes rotation+translation taken into account: check for this!
! If analytically -> 3N modes printed instead of 3N-6 so need to 
! identify the 6 rotations + vibrations.
!-----------------------------------------------------------------------
        if(streq(string,'* Statistics of the internal coordinates *')) then
          ns=ns+1
          if (ns .eq. s) then
            read(unit,*)
            read(unit,'(30x,i10)',err=900) nrt ! read # of rot+trans considered
            nvib=nmodes+nrt
            vibmode(nvib+1:ncoo)=.False. ! first 3N-6+nrt may be vibrations
          endif
        endif
        if(streq(string,'* Harmonic frequencies in cm-1     *')) ns=ns+1

        if (ns .eq. s .and. string(1:4) .eq. 'Mode') then
          read(string,'(5x,i10)',err=900) n
          read(unit,*)
          read(unit,*)
          read(unit,'(a)',err=900) string
          do while(.not.streq(string,'---------------------------------'))
            call rmlblnk(string) !remove leading blanks
            if (streq(string(1:1),'R') .or. (streq(string(1:1),'T'))) then
              vibmode(n)=.False. ! it is not a vibration
              nvib=nvib-1
            endif
            read(unit,'(a)',err=900) string
          enddo
        endif
      enddo

!-----------------------------------------------------------------------
! Search now for normal modes and frequencies
! "No symmetry" is assumed.
!-----------------------------------------------------------------------
      rewind(unit)
      ns=0
      nrd=0
      do while (.not.found)
        read(unit,'(a)',end=999) string
        call rmlblnk(string) !remove leading blanks

        if((streq(string,'Harmonic frequencies in cm-1')) .or. &
           (streq(string,'* Harmonic frequencies in cm-1     *'))) then
          ns=ns+1
          if (ns .eq. s) then
            found=.true.
            if(streq(string,'Harmonic frequencies in cm-1')) then
              nrd = ((nmodes+nrt-1)/6) + 1 ! 3N-6+nrt normal modes printed 6 by 6
              read(unit,*)
            elseif(streq(string,'* Harmonic frequencies in cm-1     *')) then
              nrd = ((ncoo-1)/6) + 1 ! 3N normal modes printed 6 by 6
              do j=1,9  !move 9 lines forward in the file
                read(unit,*)
              enddo
            endif
            n1=0
            do n=1,nrd
              buffer(:)=''
              do j=1,4  !move 4 lines forward in the file
                read(unit,*)
              enddo
!-----------------------------------------------------------------------
! get frequencies in cm-1
! check for imaginary frequency -> take the negative of the frequency
!-----------------------------------------------------------------------
              read(unit,'(18x,6a)',err=900) (buffer(j),j=1,6)
              n2=0 !number of true vibrations per bunch of 6
              do j=1,6
                if (vibmode((n-1)*6+j)) then !check if vibration
                  n1=n1+1
                  n2=n2+1
                  read(buffer(j),'(a)') string
                  call rmlblnk(string) !remove leading blanks 
                  if (string(1:1)=='i') then !check if imaginary
                    read(string(2:),'(f10.4)') freq(n1)
                    freq(n1)=-freq(n1)
                  else
                    read(string,'(f10.4)') freq(n1)
                  endif
                endif
              enddo
              read(unit,*) !skip blank line
              do while(slen(string).ne.0) !move down to next blank line
                  read(unit,'(a)',end=999) string
              enddo
!-----------------------------------------------------------------------
! get normal modes
! for matrix row=cart, column=nm
!-----------------------------------------------------------------------
              do i=1,ncoo
                read(unit,'(20x,6f10.5)',err=900) (xdum(j),j=1,6)
                n1=n1-n2
                n2=0 !number of true vibrations per bunch of 6
                do j=1,6
                  if (vibmode((n-1)*6+j)) then !check if vibration
                    n1=n1+1
                    n2=n2+1
                    matrix(i,n1)=xdum(j)
                  endif
                enddo
              enddo
            enddo
          endif
        endif
      enddo
!-----------------------------------------------------------------------
! scale to mass-weighted x to obtain true normal mode vectors
!-----------------------------------------------------------------------
      do j=1,ncoo
        matrix(j,1:nmodes)=matrix(j,1:nmodes)*sqrt(mass(j))
      enddo
!-----------------------------------------------------------------------
! normalise nm coordinates
!-----------------------------------------------------------------------
      do n=1,nmodes
         norm=dnrm2(ncoo,matrix(1:ncoo,n),1) ! calulate norm
         matrix(1:ncoo,n)=matrix(1:ncoo,n)/norm ! divide by the norm
      enddo

!-----------------------------------------------------------------------
! convert frequencies from cm-1 to ev
!-----------------------------------------------------------------------
      freq(1:nmodes)=freq(1:nmodes)*invcm

 999  continue
      return
 
 900  continue
      found=.false.
      print*,'Incorrect normal mode format reading Molcas output file.'
      write(ilog,'(15x,a)') 'WARNING: Ignoring second derivative information: '
      write(ilog,'(24x,a)') &
           'Incorrect normal mode format reading Molcas output file.'
      return

      end subroutine nmmolcas

!#######################################################################

      subroutine dbubbletmp (vector, matrix, vecdim, matdim)

      implicit none

      integer(long), intent(in) :: vecdim,matdim
      integer(long)             :: i,j,k
      real(dop), dimension(vecdim), intent(inout)        ::  vector
      real(dop), dimension(vecdim,matdim), intent(inout) ::  matrix
      real(dop)                                          ::  swap

! --- SORT THE DATA ---

      do i = 1,vecdim-1
         do j = i+1,vecdim
            if (vector(i) .gt. vector(j)) then
               swap = vector(i)
               vector(i) = vector(j)
               vector(j) = swap
               do k = 1,matdim
                  swap = matrix(i,k)
                  matrix(i,k) = matrix(j,k)
                  matrix(j,k) = swap
               enddo
            endif
         enddo
      enddo

      return
      end subroutine dbubbletmp 

!#######################################################################
! Routine to Gram-Schmidt orthonormalise normal modes.
!#######################################################################

      subroutine mocomolden(unit,filename,mocoeff,found,nbasis)

use channels, only: idump

      implicit none

      integer(long), intent(in)                        :: nbasis,unit
      real(sip), dimension(nbasis,nbasis), intent(out) :: mocoeff
      logical(kind=4), intent(inout)                   :: found
      integer(long)                                    :: ierr,i,j,iunit
      character(len=*)                                 :: filename
      character(len=80)                                :: string

      mocoeff(:,:)=0.0_sip

      ierr=0
      !Use channel idump if 10 used (already used to open the .out file)
      if (unit.eq.10) then
         iunit=idump
      else
         iunit=unit
      endif

      open(iunit,file=trim(filename),form='formatted',status='old',iostat=ierr)
      
  10  continue
      read(iunit,'(a)',end=999) string
      if (string(1:4).ne.'[MO]') goto 10
      !Loop over the MO
      do i=1,nbasis
         read(iunit,*,end=999)
         read(iunit,*)
         read(iunit,*)
         !Loop over AO coefficients
         do j=1,nbasis
            read(iunit,20) mocoeff(i,j)
         enddo
      enddo
      found=.true.

      close(iunit)
   20 format(4x,1f23.16)

  999 continue
      return
      end subroutine mocomolden

!#######################################################################
!
! NM2XMAT
! nmcoo transforms from nmodes to coo  x = nmcoo*Q
! coonm transforms from coo to nmodes  Q = coonm*x
!
! iau = 0, all factors in atomic units
! iau = 1:
!    coo are Cartesians in Angstrom,
!    freq in ev, 
!    mass in amu
!#######################################################################

      subroutine nm2xmat(nmcoo,coonm,iatsym,freq0,iau,ncoo,maxdim,&
                 nmodes,ilog)

      implicit none

      integer(long), intent(in) :: ncoo,maxdim,nmodes,iau,ilog
      integer(long)             :: i,j
      integer(long), dimension(maxdim), intent(in)       :: iatsym
      real(dop), dimension(maxdim), intent(in)           ::  freq0
      real(dop), dimension(maxdim)                       ::  mass
      real(dop), dimension(maxdim,maxdim), intent(inout) ::  nmcoo
      real(dop), dimension(maxdim,maxdim), intent(out)   ::  coonm
      real(dop)                                          ::  fact
      
!-----------------------------------------------------------------------
! Form transpose of normal modes
!-----------------------------------------------------------------------
      do j=1,ncoo
         do i=1,ncoo
            coonm(j,i)=nmcoo(i,j)
         enddo
      enddo

!-----------------------------------------------------------------------
! Get atomic masses and units
!-----------------------------------------------------------------------
      do j=1,ncoo
         call getmass(mass(j),iatsym(j),iau,ilog)
      enddo
      if (iau .eq. 0) then
         fact = 1.0_dop
      else if (iau .eq. 1) then
         fact = dsqrt(atomicmassunit/uev)
         fact = fact/angstroem
      endif

!-----------------------------------------------------------------------
! multiply matrix by mass and frequency factors
!-----------------------------------------------------------------------
      do j=1,ncoo
         do i=1,nmodes
            nmcoo(j,i)=nmcoo(j,i)/(fact*sqrt(freq0(i)*mass(j)))
            coonm(i,j)=coonm(i,j)*(fact*sqrt(freq0(i)*mass(j)))
         enddo
         do i=nmodes+1,ncoo
            nmcoo(j,i)=0.0_dop
            coonm(i,j)=0.0_dop
         enddo
      enddo

      return
      end subroutine nm2xmat

!#######################################################################
! nm2xmat2: mass-scaling only of the Hessian eigenvectors so that we
!           may work with the full 3Nx3N matrix of eigenvectors
!#######################################################################

      subroutine nm2xmat2(nmcoo,coonm,mass,ncoo,maxdim)

      implicit none

      integer(long), intent(in) :: ncoo,maxdim
      integer(long)             :: i,j
      real(dop), dimension(maxdim), intent(in)           ::  mass
      real(dop), dimension(maxdim,maxdim), intent(inout) ::  nmcoo
      real(dop), dimension(maxdim,maxdim), intent(out)   ::  coonm

      do j=1,ncoo
         do i=1,ncoo
            coonm(j,i)=nmcoo(i,j)
         enddo
      enddo

!-----------------------------------------------------------------------
! multiply matrix by mass factors
!-----------------------------------------------------------------------
      do j=1,ncoo
         do i=1,ncoo
            nmcoo(j,i)=nmcoo(j,i)/(15.4644*sqrt(mass(j)))
            coonm(i,j)=coonm(i,j)*(15.4644*sqrt(mass(j)))
         enddo
      enddo

      return
    end subroutine nm2xmat2

!#######################################################################

      subroutine subspacetrans(unit,ifiletyp,der2,iatsym,nmcoo,qder2,info, &
              abintype,maxdim,ncoo,nsta,nmodes,ilog)

      implicit none
      
      integer(long), intent(in)  :: unit,ifiletyp,maxdim,ncoo,nsta,nmodes,ilog,&
                                    abintype
      integer(long), intent(out) :: info
      integer(long)              :: iau,i,j,info2,ista
      integer(long), dimension(maxdim), intent(in)     :: iatsym
      real(dop), dimension(maxdim,maxdim), intent(in)  :: nmcoo
      real(dop), dimension(maxdim,maxdim), intent(inout)  :: der2,qder2
      real(dop), dimension(maxdim,maxdim)              :: subder2
      real(dop), dimension(maxdim)                     :: subeig
      integer(long), parameter                         :: lwork=89
      real(dop), dimension(lwork)                      :: work

      do i=1,maxdim
         subeig(i)=0.0_dop
      enddo

!     generate Hessian in full space (3N-dimensional)

      iau=1
      ista=1
      call getder2(unit,ifiletyp,der2,iatsym,nmcoo,qder2,iau,&
           ista,abintype,maxdim,ncoo,nsta,nmodes,info2,ilog)

!     remove elements H_ij corresponding to atoms not in fragment

      do i=1,maxdim
         do j=1,maxdim
            subder2(i,j)=der2(i,j)
         enddo
      enddo

      do i=1,ncoo
         do j=1,ncoo
            if ((i.ge.ncoo-2).or.(j.ge.ncoo-2)) then
               subder2(i,j)=0.0_dop
            endif
         enddo
      enddo

!     Note: der2 corresponds to Hessian in mass- frequency-weighted 
!     cartesian coords

!     diagonalise Hessian to obtain subspace A matrix

      call dsyev('V','U',27,subder2,maxdim,subeig,work,lwork,info)

      do j=1,ncoo
         print*,''
         print*,j
         do i=1,ncoo
            print*,subder2(i,j)
         enddo
      enddo
      
      print*,''
      do i=1,ncoo
         print*,i,subeig(i)
      enddo

      return
      end subroutine subspacetrans


      end module ioqc

