! **********************************************************************
! *                                                                    *
! *                       SUBROUTINE LAZYFLUSH                         *
! *                                                                    *
! * This routine uses whatever the user defined as flush.              *
! * It also makes sure that no channel is flushed more than once       *
! * every 10 seconds. Flushing too often can make MCTDH very slow      *
! * if the system is under heavy I/O load.                             *
! *                                                                    *
! * Input parameters:                                                  *
! *    channel: I/O-channel to be flushed                              *
! *                                                                    *
! * Output parameters: none                                            *
! *                                                                    *
! **********************************************************************

      module lazyflush_m

      use decimal

      contains

      subroutine lazyflush (channel)

      implicit none

      integer(long), intent(in)                  :: channel
      integer(long)                              :: tm,i
      logical(kind=4), save                      :: firstcall=.true.
      integer(long), parameter                   :: maxchannel=1000   ! just guessing
      integer(long), dimension(maxchannel), save :: lastflush

      if (firstcall) then
         do i=1,maxchannel
            lastflush(i) = 0
         enddo
         firstcall = .false.
      endif

      call unixzeit(tm)
      if (tm - lastflush(channel) .ge. 10) then
         call myflush(channel)
         lastflush(channel) = tm
      endif

      return
      end subroutine lazyflush 
      
      end module lazyflush_m
