/***********************************************************************
* Module : showtime.c
* Contains the routines :
*  fcputime  :  returns the cpu-time used,
*  fsystime  :  returns the sys-time used,
*  fpstartup :  returns host-name, day and time,
*  fgetcwd   :  returns the name of the current working directory.
*  fzeit     :  returns the date and time.
*  fzeit2    :  returns the date and timei and UNIX-time.
*  unixzeit  :  returns the UNIX time.
*  fhome     :  returns the name of the home directory.
*  excmd     :  excecutes a UNIX command.
*
*
***********************************************************************/



/*JS********************************************************************
*
*    Program : SHOWTIME  [fcputime,fsystime]
*    Language: ANSI-C with POSIX time functions.
*    Author  : Joerg Schoen
*    Purpose : Shows the CPU time the program spend.
*
***********************************************************************/

/*********     INCLUDES                                         *******/
#include <stdio.h>
#include <time.h>

#ifndef NO_POSIX
#include <unistd.h>
#include <sys/types.h>
#include <sys/times.h>
#include <sys/param.h>
#include <sys/resource.h>
#endif

/***********************************************************************
* Cast C names to FORTRAN names. Depends on the machine one uses.
***********************************************************************/

#if (F2C_STYLE == 1)
# define FCPUTIME fcputime
# define FSYSTIME fsystime
# define FPSTARTUP fpstartup
# define FGETCWD fgetcwd
# define FZEIT fzeit
# define FZEIT2 fzeit2
# define UNIXZEIT unixzeit
# define FHOME fhome
# define EXCMD excmd
# define FPCPUTIME fpcputime
# define EXCMD2 excmd2
# define EXCMD3 excmd3
#elif (F2C_STYLE == 2)
# define FCPUTIME FCPUTIME
# define FSYSTIME FSYSTIME
# define FPSTARTUP FPSTARTUP
# define FGETCWD FGETCWD
# define FZEIT FZEIT
# define FZEIT2 FZEIT2
# define UNIXZEIT UNIXZEIT
# define FHOME FHOME
# define EXCMD EXCMD
# define FPCPUTIME FPCPUTIME
# define EXCMD2 EXCMD2
# define EXCMD3 EXCMD3
#else
# define FCPUTIME fcputime_
# define FSYSTIME fsystime_
# define FPSTARTUP fpstartup_
# define FGETCWD fgetcwd_
# define FZEIT fzeit_
# define FZEIT2 fzeit2_
# define UNIXZEIT unixzeit_
# define FHOME fhome_
# define EXCMD excmd_
# define FPCPUTIME fpcputime_
# define EXCMD2 excmd2_
# define EXCMD3 excmd3_
#endif

#define Prototype extern
/*********     PROTOTYPES                                       *******/
Prototype double         showptime(int mode);

#ifdef PARALLEL
double FPCPUTIME (int *pmode)
{
  return(showptime(*pmode));
}

#ifndef NO_POSIX
/*MRB*******************************************************************
*  C-routine showptime
***********************************************************************/

double showptime(int mode)

/**********************************************************************/
{
  struct timespec tms;
  double clk_tck,cpuTime;

  /*  Get time  */
  if(clock_gettime(CLOCK_REALTIME,&tms) == -1) {
    printf("Error in times call!\n");
    return(0.0);
  }

  cpuTime=((double)tms.tv_sec+(((double)tms.tv_nsec)/1000000000.0));
  /*  Print time spent in executing this process  */

  return(cpuTime);
}
#else
/* ***  If POSIX is not available:  *** */
double showstime(int mode)
{
printf("###########################################\n");
printf("###  POSIX not defined!\n");
printf("###  If POSIX is not supported, disable the\n");
printf("###  ptiming keyword in the run-section.\n");
printf("###########################################\n");
exit(10);
return 10;
}
#endif
#endif

#define Prototype extern
/*********     PROTOTYPES                                       *******/
Prototype double         showtime(int mode);

double FCPUTIME (int *pmode)
{
  return(showtime(*pmode));
}

#ifndef NO_POSIX
/*JS********************************************************************
*  C-routine showtime
*   SHOWTIME Shows the total CPU time the program spend and additional
*    information (SYS time and time from child processes). The routine
*    returns the total CPU time of the program including SYS and child
*    time in seconds. If mode is 7, the program prints the time
*    information.
***********************************************************************/

double showtime(int mode)

/**********************************************************************/
{
  struct tms tms;
  double clk_tck,cpuTime;

  /*  Get time  */
  if(times(&tms) == -1) {
    printf("Error in times call!\n");
    return(0.0);
  }

  /*  Get number of clock ticks per second (POSIX)  */
  clk_tck = (double) sysconf(_SC_CLK_TCK);

  /*  Print time spent in executing this process  */
  cpuTime = tms.tms_utime / clk_tck;

  return(cpuTime);
}
#else
/* ***  Primitive function when POSIX is not available  *** */
double showtime(int mode) {return( (double) clock() ); }
#endif


#define Prototype extern
/*********     PROTOTYPES                                       *******/
Prototype double         showstime(int mode);

double FSYSTIME (int *pmode)
{
  return(showstime(*pmode));
}

#ifndef NO_POSIX
/*JS********************************************************************
*   SHOWTIME Shows the total CPU time the program spend and additional
*    information (SYS time and time from child processes). The routine
*    returns the total CPU time of the program including SYS and child
*    time in seconds. If mode is 7, the program prints the time
*    information.
***********************************************************************/

double showstime(int mode)

/**********************************************************************/
{
  struct tms tms;
  double clk_tck,sysTime;

  /*  Get time  */
  if(times(&tms) == -1) {
    printf("Error in times call!\n");
    return(0.0);
  }

  /*  Get number of clock ticks per second (POSIX)  */
  clk_tck = (double) sysconf(_SC_CLK_TCK);

  /*  Print time spent in executing this process  */
  sysTime = tms.tms_stime / clk_tck;

  return(sysTime);
}
#else
/* ***  Primitive function when POSIX is not available  *** */
double showstime(int mode) {return( (double) clock() ); }
#endif

/*JS********************************************************************
*
*  C-routine fpstartup
*
*    Program : PRSTARTUP [fpstartup]
*    Language: ANSI-C
*    Author  : Joerg Schoen
*    Purpose : Returns machine and startup time
*
***********************************************************************/

#include <time.h>
#include <string.h>

#ifndef CONFIG_NO_POSIX
# include <unistd.h>
#endif

/*  Call from FORTRAN:

    character *100 buffer

    CALL FPSTARTUP(buffer)
*/

void FPSTARTUP (char *output,int len)
{
  char buffer[256],*string;
  int i;
  time_t tim = time(NULL);

#ifndef CONFIG_NO_POSIX
  if(gethostname(buffer,256) >= 0) {
    /*  Delete domain name from hostname  */
    if((string = strchr(buffer,'.')))
      *string = '\0';
  } else
#endif
  strcpy(buffer,"UNKNOWN");

  /*  Append time  */
  i = sprintf(output,"------ Host: \"%s\" ----------%s",buffer,ctime(&tim));

  while(i < len) output[i++] = ' ';
}

/*JS********************************************************************
*
*  C-routine getcwd
*
*    Program : GETCWD [fgetcwd]
*    Language: ANSI-C
*    Author  : Joerg Schoen
*    Purpose : Returns the path of the current working directory
*
***********************************************************************/
#include <string.h>
#include <unistd.h>

void FGETCWD (char *buff,int *pLen,int inputLen)
{
  int l;

  if(getcwd(buff,inputLen) == NULL)
    *pLen = -1;
  else
    *pLen = strlen(buff);
}


/*JS********************************************************************
*
*  C-routine fzeit
*
*    Program : ZEIT [fzeit]
*    Language: ANSI-C
*    Author  : Joerg Schoen, HDM
*    Purpose : Returns actual time in hh:mm:ss format
*
***********************************************************************/

#include <time.h>
#include <string.h>

#ifndef CONFIG_NO_POSIX
# include <unistd.h>
#endif

/*  Call from FORTRAN:

    character *80 buffer
    CALL FZEIT(buffer)
*/

void FZEIT (char *output, int len)
{
  int i;
  time_t tim = time(NULL);

  i = sprintf(output,"%s",ctime(&tim));
}

/*JS********************************************************************
*
*  C-routine fzeit2
*
*    Program : ZEIT [fzeit]
*    Language: ANSI-C
*    Author  : Joerg Schoen, HDM
*    Purpose : Returns actual time in UNIX and hh:mm:ss format
*    In addition to fzeit it returns the time in UNIX format, seconds.
*
***********************************************************************/

#include <time.h>
#include <string.h>

#ifndef CONFIG_NO_POSIX
# include <unistd.h>
#endif

/*  Call from FORTRAN:

    integer      tm
    character*80 buffer
    CALL FZEIT2(tm,buffer)
*/

void FZEIT2 ( int *tm, char *output, int len)
{
  int i;
  time_t tim;
  *tm = (int) time(&tim);
  i = sprintf(output,"%s",ctime(&tim));
}

/*HDM*******************************************************************
*
*  C-routine unixzeit
*
*    Program : UNIXZEIT [unixzeit]
*    Language: ANSI-C
*    Author  : HDM
*    Purpose : Returns actual time (wall time) in UNIX format
*
***********************************************************************/

#include <time.h>
#include <string.h>

#ifndef CONFIG_NO_POSIX
# include <unistd.h>
#endif

/*  Call from FORTRAN:

    integer tm
    CALL UNIXZEIT(tm)
*/

void UNIXZEIT ( int *tm )
{
  time_t tim;
  *tm = (int) time(&tim);
}

/*HDM*******************************************************************
*
*  C-routine fhome
*
*    Program : FHOME [fhome]
*    Language: ANSI-C
*    Author  : HDM
*    Purpose : Returns the path of the home directory
*
***********************************************************************/
#include <stdlib.h>

void FHOME(char *buff, int *homelen)
{
   strcpy(buff,getenv("HOME"));
  *homelen = strlen(buff);
}


/*HDM*******************************************************************
*
*  C-routine excmd
*
*    Program : EXCMD [excmd]
*    Language: ANSI-C
*    Author  : HDM
*    Purpose : Excecutes the unix-command "cmd"
*
*    rewritten: FO 04/2006
*
***********************************************************************/
#include <stdlib.h>
#include <string.h>
#include <sys/wait.h>
#include <errno.h>

void EXCMD(const char *cmd, int len)
/* when calling EXCMD(cmd) from Fortran, it actually passes two parameters:
 * - a pointer to the cmd string (which is *not* null-terminated)
 * - the length of this string
 */
{
    char* buf;
    int status;
    /* the cmd string might be read-only, so we need to make a copy in
     * order to null-terminate it */
    buf = (char*) malloc(len+1);
    memcpy(buf, cmd, len);
    buf[len] = '\0';
    /* now we can safely call system */
    status = system(buf); // return status ignored
    free(buf);
}

void EXCMD2(int *ret, const char *cmd, int len)
/* like EXCMD but returns the exit status in the integer ret */
{
    char* buf;
    int status;
    /* the cmd string might be read-only, so we need to make a copy in
     * order to null-terminate it */
    buf = (char*) malloc(len+1);
    memcpy(buf, cmd, len);
    buf[len] = '\0';
    /* now we can safely call system */
    status = system(buf);
    *ret = WEXITSTATUS(status);
    free(buf);
}

void EXCMD3(int *ret, int *ret2, const char *cmd, int len)
/* like EXCMD but returns the exit status in the integer ret and errno in *ierr */
{
    char* buf;
    int status;
    /* the cmd string might be read-only, so we need to make a copy in
     * order to null-terminate it */
    buf = (char*) malloc(len+1);
    memcpy(buf, cmd, len);
    buf[len] = '\0';
    /* now we can safely call system */
    status = system(buf);
    *ret = WEXITSTATUS(status);
    *ret2 = errno;
    free(buf);
}

