
module cpdsamplings_m
  implicit none

  double precision, allocatable :: basissamplings(:,:,:)
end module cpdsamplings_m


subroutine mkcpdsamplings(cpd,modeindex,nsample)
  use cpdsamplings_m
  use cpdtype_m
  use global

  implicit none

  type(cpd_t), intent(IN) :: cpd
  integer, intent(IN)     :: nsample
  integer, intent(IN)     :: modeindex(nsample,cpd%nmode)
  integer                 :: allocstat, asize, m,r,z,n

  if (allocated(basissamplings)) deallocate(basissamplings)

  allocate(basissamplings(nsample, cpd%nmode, cpd%rank), stat=allocstat)
  if (allocstat .ne. 0) then
     asize = cpd%rank*cpd%nmode*nsample*8/1024/1024
     write(message,'(a,i8,a)') "Could not allocate CPD basis sampling array, size = " &
          , asize, " MB."
     call cncntrblnk(message)
     routine = "mkcpdsamplings"
     call errormsg
  end if


  !$omp parallel private(n,r,m,z)
  !$omp do
  do r=1,cpd%rank
     do m=1,cpd%nmode
        do n=1,nsample
           z = modeindex(n,m)
           basissamplings(n,m,r) = cpd%spp(m)%htm(z,r)
        end do
     end do
  end do
  !$omp end do
  !$omp end parallel

#ifdef DEBUG
  if( any(basissamplings(:,:,:) .ne.  basissamplings(:,:,:))) then
     print*, "NaN in basissamplings"
     stop 0
  end if
#endif

end subroutine mkcpdsamplings


subroutine updatecpdsamplings1m(cpd,modeindex,nsample,m)
  use cpdsamplings_m
  use cpdtype_m
  use global

  implicit none

  type(cpd_t), intent(IN) :: cpd
  integer, intent(IN)     :: nsample,m
  integer, intent(IN)     :: modeindex(nsample,cpd%nmode)

  integer                 :: n,r,z


  if (.not. allocated(basissamplings) ) then
     message = "updatecpdsamplings1m called befor initialization."
     routine = "updatecpdsamplings1m"
     call errormsg()
  end if


  !$omp parallel do private(n,z)
  do r=1,cpd%rank
     do n=1,nsample
        z = modeindex(n,m)
        basissamplings(n,m,r) = cpd%spp(m)%htm(z,r)
     end do
  end do
  !$omp end parallel do

#ifdef DEBUG
  if( any(basissamplings(:,m,:) .ne.  basissamplings(:,m,:))) then
     print*, "NaN in basissamplings after update of mode", m
     stop 0
  end if
#endif



end subroutine updatecpdsamplings1m

subroutine disposecpdsamplings()
  use cpdsamplings_m
  implicit none

  if (allocated(basissamplings)) deallocate(basissamplings)

end subroutine disposecpdsamplings

