! -*- f90 -*-

module cpdtype_m
  use global
  use tensortype
  use logdat

  implicit none

  type, extends(htm_t) :: spp_t
     double precision, allocatable :: htm(:,:)
  end type spp_t

  type, extends(tensor_t) :: cpd_t
     integer                        :: rank
     ! actual data
     type(spp_t), allocatable       :: spp(:)
     double precision, allocatable  :: coeff(:)

  end type cpd_t


contains

  subroutine deallocate_cpd_t(cpd)
    implicit none
    type(cpd_t), intent(INOUT)  :: cpd
    integer                     :: m

    if (allocated(cpd%spp)) then
       do m=1, cpd%nmode
          if ( allocated(cpd%spp(m)%htm) ) deallocate(cpd%spp(m)%htm)
       end do
       deallocate(cpd%spp)
    end if
    if (allocated(cpd%coeff))     deallocate(cpd%coeff)


    cpd%rank = 0
    cpd%maxspfdof = 0

    call deallocate_tensor_t(cpd)

    cpd%initialized = .false.

  end subroutine deallocate_cpd_t


  subroutine allocate_cpd_t(cpd,rank,ndof,nmode,gdim,dofspf,potdofmap, &
       modelabel,ort,ortdim, loned)
    implicit none
    type(cpd_t), intent(INOUT)     :: cpd
    integer, intent(IN)            :: rank,ndof,nmode,ortdim
    integer, intent(IN)            :: gdim(ndof),dofspf(ndof)
    integer, intent(IN)            :: potdofmap(2,ndof)
    double precision, intent(IN)   :: ort(ortdim)
    character(len=*), intent(IN)   :: modelabel(ndof)
    logical, intent(IN)            :: loned(ndof)

    integer                        :: m,n,f,slen,lm

    call deallocate_cpd_t(cpd)

!!$   if (ndof .le. 1) then
!!$      write(*,'(a,i4,a)') "ndof= ",ndof,": must be 2 or larger."
!!$      stop 1
!!$   end if
!!$   if (nmode .le. 1) then
!!$      write(*,'(a,i4,a)') "mode= ",nmode,": must be 2 or larger."
!!$      stop 1
!!$   end if
!!$   if (rank .lt. 1) then
!!$      write(*,'(a,i4,a)') "rank= ",rank,": must be 1 or larger."
!!$      stop 1
!!$   end if


    ! simple copy values
    cpd%rank = rank
    cpd%ndof = ndof
    cpd%nmode = nmode

    allocate(cpd%gdim(ndof))
    cpd%gdim = gdim

    allocate(cpd%modelabel(ndof))
    lm = len(cpd%modelabel(ndof))
    do f=1,ndof
       cpd%modelabel(f) = " "
       slen = len_trim(modelabel(f))
       if (slen .le. lm) then
          cpd%modelabel(f)(1:slen) = trim(modelabel(f))
       else
          write(*,*) "Mode label too long: "//modelabel(f)
          stop 1
       end if
    end do

    ! we need to know maxspfdof
    cpd%maxspfdof = 0
    do m=1,cpd%nmode
       n = 0
       do f=1,cpd%ndof
          if (dofspf(f) .eq. m) then
             n = n + 1
          end if
       end do
       cpd%maxspfdof = max(cpd%maxspfdof,n)
    end do


    ! allocate all other pointers
    allocate(cpd%nspfdof(nmode))
    allocate(cpd%dofspf(ndof))
    allocate(cpd%potdofmap(2,ndof))
    allocate(cpd%spfdof(cpd%maxspfdof,nmode))
    allocate(cpd%subdim(nmode))
    allocate(cpd%ort(ortdim))
    allocate(cpd%zort(ndof))
    allocate(cpd%loned(ndof))

    ! copy some direct
    cpd%loned = loned
    if (any(cpd%loned)) allocate(cpd%oned(ortdim), source=0.d0)
    cpd%dofspf = dofspf


    do f=1,ndof
       cpd%potdofmap(1,f) = potdofmap(1,f)
       cpd%potdofmap(2,f) = potdofmap(2,f)
    end do

    cpd%ortdim = ortdim
    cpd%ort = ort

    ! set up all other pointers
    call zeig_cpd_t(cpd)

    ! allocate data and initialize with zero
    allocate(cpd%spp(cpd%nmode))
    do m=1,cpd%nmode
       allocate(cpd%spp(m)%htm(cpd%subdim(m), cpd%rank))
       cpd%spp(m)%htm(:,:) = 0.d0
    end do

    allocate(cpd%coeff(rank))
    cpd%coeff= 0.d0

    cpd%initialized = .true.

  end subroutine allocate_cpd_t




  subroutine zeig_cpd_t(cpd)
    implicit none
    type(cpd_t), intent(INOUT) :: cpd

    integer                    :: f,m,n


    cpd%rvpotdim = 1.d0
    cpd%vpotdim = 1
    cpd%nspfdof = 0
    do f=1,cpd%ndof

       cpd%vpotdim = cpd%vpotdim * cpd%gdim(f)
       cpd%rvpotdim = cpd%rvpotdim * dble(cpd%gdim(f))

       m = cpd%dofspf(f)
       cpd%nspfdof(m) = cpd%nspfdof(m) + 1
    end do


    do f=1,cpd%ndof
       m = cpd%potdofmap(1,f)
       n = cpd%potdofmap(2,f)
       cpd%spfdof(n,m) = f
    end do

    do m=1,cpd%nmode
       cpd%subdim(m) = 1
       do n=1,cpd%nspfdof(m)
          f=cpd%spfdof(n,m)
          cpd%subdim(m) = cpd%subdim(m)*cpd%gdim(f)
       enddo
    end do

    cpd%zort(1) = 1
    do f=2,cpd%ndof
       cpd%zort(f) = cpd%zort(f-1) + cpd%gdim(f-1)
    end do


  end subroutine zeig_cpd_t

end module cpdtype_m

module cpdtype_s

use cpdtype_m

contains

  subroutine randomizecpd(cpd)
    implicit none
    type(cpd_t), intent(INOUT)    :: cpd
    integer                       :: r,m
    double precision              :: norm, ddot

    cpd%coeff = 0.d0

    do m=1,cpd%nmode

       !$omp parallel do private(norm)
       do r=1,cpd%rank

          call RANDOM_NUMBER(cpd%spp(m)%htm(:,r))
          cpd%spp(m)%htm(:,r) = cpd%spp(m)%htm(:,r) - 0.5d0

          norm = ddot(cpd%subdim(m),cpd%spp(m)%htm(:,r),1, &
               cpd%spp(m)%htm(:,r),1)
          norm = sqrt(norm)
          cpd%spp(m)%htm(:,r) = cpd%spp(m)%htm(:,r) / norm

       end do
       !$omp end parallel do
    end do



  end subroutine randomizecpd


  ! normalize SPP.
  ! targetnorm: dot_product(spp,spp) = targetnom
  ! default: targetnorm = 1
  subroutine normspp(cpd,targetnorm)
    implicit none
    type(cpd_t), intent(INOUT)    :: cpd
    double precision, intent(IN), optional :: targetnorm
    double precision              :: norm, ddot
    integer                       :: r,m
    double precision              :: targetnorm1

    if (present(targetnorm)) then
       targetnorm1 = sqrt(targetnorm)
    else
       targetnorm1 = 1.d0
    end if
    if (targetnorm1 .le. 1.d-12) then
       routine = "normspp cpdtype.F90"
       message = "Illegal target-norm."
       call errormsg
    end if

    do m=1,cpd%nmode

       !$omp parallel do private(norm) schedule(static)
       do r=1,cpd%rank
          norm = ddot(cpd%subdim(m), cpd%spp(m)%htm(:,r),1,cpd%spp(m)%htm(:,r),1)
          if (norm.lt.1.d-12) then
             call RANDOM_NUMBER(cpd%spp(m)%htm(:,r))
             cpd%spp(m)%htm(:,r) = cpd%spp(m)%htm(:,r) - 0.5d0

             norm = ddot(cpd%subdim(m),cpd%spp(m)%htm(:,r),1, &
                  cpd%spp(m)%htm(:,r),1)
             norm = sqrt(norm)/targetnorm1
             cpd%spp(m)%htm(:,r) = cpd%spp(m)%htm(:,r) / norm

             cpd%coeff(r) = 0.d0
          else
             norm = sqrt(norm)/targetnorm1
             cpd%spp(m)%htm(:,r) = cpd%spp(m)%htm(:,r)/norm
             cpd%coeff(r) = cpd%coeff(r)*norm
          end if
       end do
       !$omp end parallel do

    end do


  end subroutine normspp


  ! normalize one-mode-terms and re-scale
  ! cefficients to compensate
  subroutine normspp1m(cpd,m, targetnorm)
    implicit none
    type(cpd_t), intent(INOUT)       :: cpd
    integer, intent(IN)              :: m
    double precision, intent(IN), optional :: targetnorm

    integer          :: r
    double precision :: norm, ddot
    double precision :: targetnorm1

    if (present(targetnorm)) then
       targetnorm1 = sqrt(targetnorm)
    else
       targetnorm1 = 1.d0
    end if
    if (targetnorm1 .le. 1.d-12) then
       routine = "normspp cpdtype.F90"
       message = "Illegal target-norm."
       call errormsg
    end if


    !$omp parallel do private(norm) schedule(static)
    do r=1,cpd%rank
       norm =  ddot(cpd%subdim(m), cpd%spp(m)%htm(:,r),1,cpd%spp(m)%htm(:,r),1)
       if (norm.lt.1.d-12) then

          call RANDOM_NUMBER(cpd%spp(m)%htm(:,r))
          cpd%spp(m)%htm(:,r) = cpd%spp(m)%htm(:,r) - 0.5d0
          norm =  ddot(cpd%subdim(m), cpd%spp(m)%htm(:,r),1,cpd%spp(m)%htm(:,r),1)
          norm = sqrt(norm)/targetnorm1
          cpd%spp(m)%htm(:,r) = cpd%spp(m)%htm(:,r) / norm
          cpd%coeff(r) = 0.d0
       else
          norm = sqrt(norm)/targetnorm1
          cpd%spp(m)%htm(:,r) = cpd%spp(m)%htm(:,r) / norm
          cpd%coeff(r) = cpd%coeff(r)*norm
       end if
    end do
    !$omp end parallel do


  end subroutine normspp1m



  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Change the rank of the cpd (add or remove terms)
  !
  ! if fillrandom is present and true and newrank .gt. oldrank
  ! then any additional spp are initialized wit random numbers
  ! and zero coefficients
  !
  ! if fillrandom is false, any additional spp are initialized
  ! with zero and zero coeffitients.
  !
  ! If fillrandom is missing, then fillrandom = .true. is
  ! assumed.
  !
  ! if newrank .le. oldrank then fillrandom is ignored
  ! and terms at the end of the cpd are removed.
  ! (To remove small terms one should sort the cpd
  ! before calling this routine)
  !
  ! Do not use too often - produces overhead.
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  subroutine changerank(cpd, newrank, fillrandom, nchunk)
    implicit none

    type(cpd_t), intent(INOUT)     :: cpd
    integer, intent(IN)            :: newrank
    logical, intent(IN), optional  :: fillrandom
    integer, intent(IN), optional  :: nchunk

    double precision, allocatable  :: oldcoeff(:)
    type(spp_t), allocatable       :: oldspp(:)
    integer                        :: r,r1,r2,m, n
    double precision               :: norm, ddot
    logical                        :: fillrandom1
    integer                        :: nchunk1
    integer                        :: rperchunk, rperchunknew

    if (present(fillrandom)) then
       fillrandom1 = fillrandom
    else
       fillrandom1 = .true.
    end if

    if (present(nchunk)) then
       nchunk1 = nchunk
    else
       nchunk1 = 1
    end if

    if (modulo(newrank,nchunk1) .ne. 0) then
       if (present(nchunk)) then
          write(*,'(a)') "changerank: new rank must be a multiple of nchunks"
          write(*,'(a,i8)') "new rank:", newrank
          write(*,'(a,i8)') "nchunk:  ", nchunk
          stop 1
       end if
    end if

    if (newrank.lt.1) then
       write(*,'(a,i6,a)') "changerank: new rank= ",newrank, &
            ": must be 1 or larger."
       stop 1
    end if

    ! early exit if possible
    if (newrank .eq. cpd%rank) return

    rperchunk = cpd%rank/nchunk1
    rperchunknew = newrank/nchunk1

    allocate(oldcoeff(cpd%rank))
    allocate(oldspp(cpd%nmode))
    !$omp parallel do
    do m=1,cpd%nmode
       allocate(oldspp(m)%htm(cpd%subdim(m),cpd%rank))
       oldspp(m)%htm(:,:) = cpd%spp(m)%htm(:,:)
       deallocate(cpd%spp(m)%htm)
       allocate(cpd%spp(m)%htm(cpd%subdim(m),newrank))
    end do
    !$omp end parallel do


    !$omp parallel do
    do r=1,cpd%rank
       oldcoeff(r) = cpd%coeff(r)
    end do
    !$omp end parallel do

    deallocate(cpd%coeff)
    allocate(cpd%coeff(newrank))


    if (newrank .le. cpd%rank) then

       r1 = 1
       r2 = 1
       do n=1,nchunk1
          do r=1,rperchunknew
             cpd%coeff(r1) = oldcoeff(r2)
             r1 = r1 + 1
             r2 = r2 + 1
          end do
          r2 = n*rperchunk + 1
       end do


       do m=1,cpd%nmode
          r1 = 1
          r2 = 1
          do n=1,nchunk1
             do r=1, rperchunknew
                cpd%spp(m)%htm(:,r1) =  oldspp(m)%htm(:,r2)
                r1 = r1 + 1
                r2 = r2 + 1
             end do
             r2 = n*rperchunk + 1
          end do
       end do

    else
       ! newrank .gt. cpd%rank

       r1 = 1
       r2 = 1
       do n=1,nchunk1
          do r=1,rperchunk
             cpd%coeff(r1) = oldcoeff(r2)
             r1 = r1 + 1
             r2 = r2 + 1
          end do
          do r=rperchunk + 1, rperchunknew
             cpd%coeff(r1) = 0.d0
             r1 = r1 + 1
          end do
       end do


       do m=1,cpd%nmode
          r1 = 1
          r2 = 1
          do n=1,nchunk1
             do r=1,rperchunk
                cpd%spp(m)%htm(:,r1) =  oldspp(m)%htm(:,r2)
                r1 = r1 + 1
                r2 = r2 + 1
             end do ! copy

             do r=rperchunk + 1, rperchunknew

                if (fillrandom1) then

                   call random_number(cpd%spp(m)%htm(:,r1))
                   cpd%spp(m)%htm(:,r1) = cpd%spp(m)%htm(:,r1) -  0.5d0
                   norm = ddot(cpd%subdim(m), cpd%spp(m)%htm(:,r1),1,cpd%spp(m)%htm(:,r1),1)
                   cpd%spp(m)%htm(:,r1) = cpd%spp(m)%htm(:,r1)/sqrt(norm)

                else

                   cpd%spp(m)%htm(:,r1) = 0.d0

                end if

                r1 = r1 + 1

             end do ! fill

          end do ! nchunk

       end do ! m

    end if

    cpd%rank = newrank

    deallocate(oldcoeff)
    do m=1,cpd%nmode
       deallocate(oldspp(m)%htm)
    end do
    deallocate(oldspp)

  end subroutine changerank


  ! sort cpd by descending absolue coefficients
  subroutine sortcpd(cpd)
    implicit none
    type(cpd_t), intent(INOUT)     :: cpd

    integer  :: idx(cpd%rank)
    integer  :: rank, r, r1, k, m

    double precision  :: coeff(cpd%rank)
    type(spp_t), allocatable :: spp(:)
    double precision  :: c

    coeff = cpd%coeff

    allocate(spp(cpd%nmode))
    !$omp parallel do
    do m=1,cpd%nmode
       allocate(spp(m)%htm(cpd%subdim(m), cpd%rank))
       spp(m)%htm = cpd%spp(m)%htm
    end do
    !$omp end parallel do

    rank = 1
    cpd%coeff(2:cpd%rank) = 0.d0
    idx(1) = 1
    idx(2:cpd%rank) = 0

    do k=2,cpd%rank
       c = coeff(k)

       if (abs(cpd%coeff(rank)) .lt. abs(c)) then
          ! insert new coeff in the middle

          if (rank .lt. cpd%rank) then
             rank=rank+1
          end if

          ! where to insert
          r1=1
          do r=rank,1,-1
             if (abs(cpd%coeff(r)) .gt. abs(c)) then
                r1 = r+1
                exit
             end if
          end do
          if (r1 .gt. rank) then
             print*, "This part of the code should never be reached"
          end if

          do r=rank-1,r1,-1
             cpd%coeff(r+1) = cpd%coeff(r)
             idx(r+1) = idx(r)
          end do

          cpd%coeff(r1) = c
          idx(r1) = k

       else if( rank .lt. cpd%rank) then
          ! append at the end if there is space
          rank = rank + 1
          cpd%coeff(rank) = c
          idx(rank) = k
       end if

    end do

    if (rank.ne.cpd%rank) then
       print*, "cpd%rank <>  rank:", cpd%rank, rank
    end if


    do m=1, cpd%nmode
       do r=1,cpd%rank
          r1 = idx(r)
          cpd%spp(m)%htm(:,r) = spp(m)%htm(:,r1)
       end do
    end do

    do m=1,cpd%nmode
       deallocate(spp(m)%htm)
    end do
    deallocate(spp)


  end subroutine sortcpd



  subroutine copy_cpd(cpd_in, cpd_out)
    implicit none

    type(cpd_t), intent(IN)          :: cpd_in
    type(cpd_t), intent(INOUT)       :: cpd_out
    integer                          :: m

    call allocate_cpd_t(cpd_out,cpd_in%rank,cpd_in%ndof,cpd_in%nmode, &
         cpd_in%gdim,cpd_in%dofspf,cpd_in%potdofmap, cpd_in%modelabel, &
         cpd_in%ort, cpd_in%ortdim, cpd_in%loned)

    cpd_out%coeff(:) = cpd_in%coeff(:)

    !$omp parallel do
    do m=1,cpd_in%nmode
       cpd_out%spp(m)%htm = cpd_in%spp(m)%htm
    end do
    !$omp end parallel do

    if (any(cpd_in%loned)) then
       cpd_out%oned = cpd_in%oned
    end if


  end subroutine copy_cpd




  ! write information about the cpd to a channel
  ! default channel is 6 (stdout)
  subroutine wrcpdproperties(cpd, channel)
    implicit none
    type(cpd_t), intent(IN)        :: cpd
    integer, intent(IN), optional  :: channel
    integer                        :: channel1
    logical                        :: isopen
    integer                        :: m,f, ilbl, jlbl
    double precision               :: mem, totmem
    character(len=80)              :: coords

    if (present(channel)) then
       channel1 = channel
    else
       channel1 = 6
    end if

    inquire(channel1, opened=isopen)
    if (.not. isopen) return

    write(channel1,'(a)') ""
    write(channel1,'(a)') "CPD-Info:"
    write(channel1,'(a)') ""
    if (.not. cpd%initialized) then
       write(channel1,'(a)') " >> CPD object is not initialized <<"
       return
    end if
    write(channel1,'(a)') "SPP:"
    write(channel1,'(a)') "  Mode    gdim     MEM(SPP)   coordinates"

    totmem = 0.d0
    do m = 1,cpd%nmode

       coords = ""
       ilbl = 1
       do f=1,cpd%ndof
          if (cpd%dofspf(f) .eq. m) then
             jlbl = ilbl + len_trim(cpd%modelabel(f)) + 1
             coords(ilbl:jlbl) = trim(cpd%modelabel(f))//" "
             ilbl = jlbl +1
          end if
       end do

       mem = dble(cpd%rank*cpd%subdim(m))/131072 !MB
       totmem = totmem + mem
       write(channel1,'(i6,i8,f10.3,a)') m, cpd%subdim(m), &
            mem, " MB   "//trim(coords)
    end do
    if (any(cpd%loned)) then
       mem = dble(cpd%ortdim)/131072 !MB
       totmem = totmem + mem
       write(channel1,'(/a,i8,f10.3,a)') '    1D', cpd%ortdim, mem, " MB   "
    else
       write(channel1,'(a)') '(no 1D terms)'
    end if

    write(channel1,'(a)') "---------------------------------------" &
         //"---------------"
    write(channel1,'(a,f14.3,a)') "Total MEM (SPP):   ", totmem, " MB"
    write(channel1,'(a)') ""
    mem = dble(cpd%rank)/131072 !MB
    write(channel1,'(a,i7,3x,f10.3,a)') "Coefficients:", &
         cpd%rank, mem, " MB"
    totmem = totmem + mem
    write(channel1,'(a)') ""
    write(channel1,'(a,f10.3,a)') "Total memory (cpd):    ", &
         totmem, " MB"
    write(channel1,'(a)') ""

  end subroutine wrcpdproperties




  subroutine mpi_bcast_cpd_t(cpd)

    use mpidata
#ifdef MPI
    use mpi
#endif

    implicit none
    type(cpd_t), intent(INOUT)       :: cpd

#ifdef MPI

    integer                          :: ndof, nmode, rank, ortdim, mpicommi(4)
    integer, allocatable             :: gdim(:),dofspf(:), potdofmap(:,:)
    character(len=c1), allocatable   :: modelabel(:)
    double precision, allocatable    :: ort(:)
    logical, allocatable             :: loned(:)
    integer                          :: ierr, htmsize,m


    if (mpirank.eq.0) then
       mpicommi(1) = cpd%rank
       mpicommi(2) = cpd%ndof
       mpicommi(3) = cpd%nmode
       mpicommi(4) = cpd%ortdim

       call MPI_Bcast(mpicommi,4,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(cpd%gdim,      cpd%ndof,    MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(cpd%dofspf,    cpd%ndof,    MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(cpd%potdofmap, cpd%ndof*2,  MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(cpd%modelabel, cpd%ndof*c1, MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(cpd%ort,       cpd%ortdim,  MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(cpd%loned,     cpd%ndof,    MPI_LOGICAL,0,MPI_COMM_WORLD,ierr)

       call MPI_Bcast(cpd%coeff,     cpd%rank,    MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)


       do m=1,cpd%nmode
          htmsize = cpd%subdim(m)*cpd%rank
          call MPI_Bcast(cpd%spp(m)%htm, htmsize, MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
       end do

       if (any(cpd%loned)) then
          call MPI_Bcast(cpd%oned,   cpd%ortdim,  MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
       end if

    else
       call MPI_Bcast(mpicommi,4,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       rank   = mpicommi(1)
       ndof   = mpicommi(2)
       nmode  = mpicommi(3)
       ortdim = mpicommi(4)

       allocate(gdim(ndof))
       allocate(dofspf(ndof))
       allocate(potdofmap(2,ndof))
       allocate(ort(ortdim))
       allocate(modelabel(ndof))
       allocate(loned(ndof))

       call MPI_Bcast(gdim,      ndof,    MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(dofspf,    ndof,    MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(potdofmap, ndof*2,  MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(modelabel, ndof*c1, MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(ort,       ortdim,  MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(loned,     ndof,    MPI_LOGICAL,0,MPI_COMM_WORLD,ierr)

       call allocate_cpd_t(cpd,rank,ndof,nmode,gdim,dofspf,potdofmap, &
            modelabel,ort, ortdim,loned)


       call MPI_Bcast(cpd%coeff,     cpd%rank,    MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)

       do m=1,cpd%nmode
          htmsize = cpd%subdim(m)*cpd%rank
          call MPI_Bcast(cpd%spp(m)%htm, htmsize, MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
       end do

       if (any(cpd%loned)) then
          call MPI_Bcast(cpd%oned,   cpd%ortdim,  MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
       end if


       deallocate(gdim)
       deallocate(dofspf)
       deallocate(potdofmap)
       deallocate(ort)
       deallocate(modelabel)
       deallocate(loned)

    end if

#endif
    return

  end subroutine mpi_bcast_cpd_t



end module cpdtype_s
