

module hcpdtype_m
  use tensortype
  implicit none


  type, extends(htm_t) :: sph_t
     double precision, allocatable :: htm(:,:,:)
  end type sph_t


  type, extends(tensor_t) :: hcpd_t
     integer                        :: rank

     double precision               :: rvpotdim2
     integer, allocatable           :: subdim2(:)

     type(sph_t), allocatable       :: sph(:)
     double precision, allocatable  :: coeff(:)

  end type hcpd_t

contains

  subroutine deallocate_hcpd_t(hcpd)
    use tensortype
    implicit none
    type(hcpd_t), intent(INOUT)  :: hcpd
    integer                      :: r,m

    if (allocated(hcpd%sph)) then
       do m=1, hcpd%nmode
          if (allocated(hcpd%sph(m)%htm)) deallocate(hcpd%sph(m)%htm)
       end do

       deallocate(hcpd%sph)
    end if

    if (allocated(hcpd%coeff)) deallocate(hcpd%coeff)
    if (allocated(hcpd%subdim2)) deallocate(hcpd%subdim2)



    hcpd%rank = 0
    hcpd%rvpotdim2 = 0.d0
    call deallocate_tensor_t(hcpd)
    hcpd%initialized = .false.

  end subroutine deallocate_hcpd_t



  subroutine allocate_hcpd_t(hcpd,rank,ndof,nmode,gdim,dofspf,potdofmap, &
       modelabel,ort,ortdim, loned)
    implicit none
    type(hcpd_t), intent(INOUT)    :: hcpd
    integer, intent(IN)            :: rank,ndof,nmode,ortdim
    integer, intent(IN)            :: gdim(ndof),dofspf(ndof)
    integer, intent(IN)            :: potdofmap(2,ndof)
    double precision, intent(IN)   :: ort(ortdim)
    character(len=*), intent(IN)   :: modelabel(ndof)
    logical, intent(IN)            :: loned(ndof)

    integer                        :: m,n,f,slen,lm




!!$    if (ndof .le. 1) then
!!$       write(*,'(a,i4,a)') "ndof= ",ndof,": must be 2 or larger."
!!$       stop 1
!!$    end if
!!$    if (nmode .le. 1) then
!!$       write(*,'(a,i4,a)') "mode= ",nmode,": must be 2 or larger."
!!$       stop 1
!!$    end if
!!$    if (rank .lt. 1) then
!!$       write(*,'(a,i4,a)') "rank= ",rank,": must be 1 or larger."
!!$       stop 1
!!$    end if


    ! simple copy values
    hcpd%rank = rank
    hcpd%ndof = ndof
    hcpd%nmode = nmode

    allocate(hcpd%gdim(ndof))
    hcpd%gdim = gdim

    allocate(hcpd%modelabel(ndof))
    lm = len(hcpd%modelabel(ndof))
    do f=1,ndof
       hcpd%modelabel(f) = " "
       slen = len_trim(modelabel(f))
       if (slen .le. lm) then
          hcpd%modelabel(f)(1:slen) = trim(modelabel(f))
       else
          write(*,*) "Mode label too long: "//modelabel(f)
          stop 1
       end if
    end do

    ! we need to know maxspfdof
    hcpd%maxspfdof = 0
    do m=1,hcpd%nmode
       n = 0
       do f=1,hcpd%ndof
          if (dofspf(f) .eq. m) then
             n = n + 1
          end if
       end do
       hcpd%maxspfdof = max(hcpd%maxspfdof,n)
    end do


    ! allocate all other pointers
    allocate(hcpd%nspfdof(nmode))
    allocate(hcpd%dofspf(ndof))
    allocate(hcpd%potdofmap(2,ndof))
    allocate(hcpd%spfdof(hcpd%maxspfdof,nmode))
    allocate(hcpd%subdim(nmode))
    allocate(hcpd%ort(ortdim))
    allocate(hcpd%zort(ndof))
    allocate(hcpd%loned(ndof))

     allocate(hcpd%subdim2(nmode))

    ! copy some direct
    hcpd%loned = loned
    if (any(hcpd%loned)) allocate(hcpd%oned(ortdim))
    hcpd%dofspf = dofspf


    do f=1,ndof
       hcpd%potdofmap(1,f) = potdofmap(1,f)
       hcpd%potdofmap(2,f) = potdofmap(2,f)
    end do

    hcpd%ortdim = ortdim
    hcpd%ort = ort

    hcpd%rvpotdim2 = 1.d0
    hcpd%rvpotdim = 1.d0
    hcpd%nspfdof = 0
    do f=1,hcpd%ndof

       hcpd%rvpotdim = hcpd%rvpotdim * dble(hcpd%gdim(f))
       hcpd%rvpotdim2 = hcpd%rvpotdim2 * dble(hcpd%gdim(f)**2)

       m = hcpd%dofspf(f)
       hcpd%nspfdof(m) = hcpd%nspfdof(m) + 1
    end do

    hcpd%spfdof = 0
    do f=1,hcpd%ndof
       m = hcpd%potdofmap(1,f)
       n = hcpd%potdofmap(2,f)
       hcpd%spfdof(n,m) = f
    end do


    do m=1,hcpd%nmode
       hcpd%subdim(m) = 1
       do n=1,hcpd%nspfdof(m)
          f=hcpd%spfdof(n,m)
          hcpd%subdim(m) = hcpd%subdim(m)*hcpd%gdim(f)
       enddo
       hcpd%subdim2(m) = hcpd%subdim(m)**2
    end do

    hcpd%zort(1) = 1
    do f=2,hcpd%ndof
       hcpd%zort(f) = hcpd%zort(f-1) + hcpd%gdim(f-1)
    end do


    ! allocate data and initialize
    allocate(hcpd%sph(hcpd%nmode))



    do m=1, hcpd%nmode
       allocate(hcpd%sph(m)%htm(hcpd%subdim(m),hcpd%subdim(m), hcpd%rank))
       hcpd%sph(m)%htm(:,:,:) = 0.d0
    end do


    allocate(hcpd%coeff(rank))

    hcpd%coeff= 0.d0


    hcpd%initialized = .true.

  end subroutine allocate_hcpd_t

end module hcpdtype_m

module hcpdtype_s
use hcpdtype_m

contains

  ! normalize htm matrices.
  ! reinit: if zero norm is detected, then:
  !   reinit =  0: random (default)
  !   reinit =  1: random, hermitian
  !   reinit = -1, random, anti-hermitian
  subroutine normsph(hcpd, reinit)
    implicit none
    type(hcpd_t), intent(INOUT)            :: hcpd
    integer, intent(in), optional          :: reinit

    double precision              :: norm
    integer                       :: r,m
    double precision              :: dnrm2
    integer                       :: reinit1
    external ddot


    if (present(reinit)) then
       reinit1 = reinit
    else
       reinit1 = 0
    end if
    if (reinit1 .lt. -1 .or. reinit .gt. 1) then
       write(*,*) "routine normsph,  hcpdtype.f90"
       write(*,*) "Illegal reinit-tyoe:", reinit1
       stop 1
    end if


    do m=1,hcpd%nmode
       call normsph1m(hcpd ,m, reinit1)
    end do


  end subroutine normsph



  ! normalize htm matrices for one mode
  ! reinit: if zero norm is detected, then:
  !   reinit =  0: random (default)
  !   reinit =  1: random, hermitian
  !   reinit = -1, random, anti-hermitian
  subroutine normsph1m(hcpd ,m, reinit)
    implicit none
    type(hcpd_t), intent(INOUT)   :: hcpd
    integer, intent(IN)           :: m
    integer, intent(IN), optional :: reinit

    double precision              :: norm
    integer                       :: r, reinit1
    double precision              :: dnrm2
    external ddot

    if (present(reinit)) then
       reinit1 = reinit
    else
       reinit1 = 0
    end if
    if (reinit1 .lt. -1 .or. reinit .gt. 1) then
       write(*,*) "routine normsph,  hcpdtype.f90"
       write(*,*) "Illegal reinit-tyoe:", reinit1
       stop 1
    end if


    !$omp parallel do private(norm) schedule(static)
    do r=1,hcpd%rank

       norm = dnrm2(hcpd%subdim2(m),hcpd%sph(m)%htm(:,:,r),1)

       if (norm**2 .lt.1.d-12) then

          call RANDOM_NUMBER(hcpd%sph(m)%htm(:,:,r))
          hcpd%sph(m)%htm(:,:,r) = hcpd%sph(m)%htm(:,:,r) - 0.5d0

          if (reinit1 .eq. 1) then
             hcpd%sph(m)%htm(:,:,r) =   hcpd%sph(m)%htm(:,:,r) &
                  + transpose(hcpd%sph(m)%htm(:,:,r))
          else if (reinit1 .eq. -1) then
             hcpd%sph(m)%htm(:,:,r) =   hcpd%sph(m)%htm(:,:,r) &
                  - transpose(hcpd%sph(m)%htm(:,:,r))
          end if

          norm = dnrm2(hcpd%subdim2(m), hcpd%sph(m)%htm(:,:,r), 1)
          hcpd%sph(m)%htm(:,:,r) = hcpd%sph(m)%htm(:,:,r) / norm

          hcpd%coeff(r) = 0.d0

       else

          hcpd%sph(m)%htm(:,:,r) = hcpd%sph(m)%htm(:,:,r) / norm
          hcpd%coeff(r) = hcpd%coeff(r)*norm

       end if
    end do
    !$omp end parallel do


  end subroutine normsph1m



  ! inittype (optional): how to initialize the SPOs
  !   inittype =  0: random (default)
  !   inittype =  1: random, hermitian
  !   inittype = -1, random, anti-hermitian
  subroutine randomizehcpd(hcpd, inittype)
    implicit none
    type(hcpd_t), intent(INOUT)    :: hcpd
    integer, intent(IN), optional  :: inittype

    integer          :: m,r, inittype1
    double precision :: norm, dnrm2

    if (present(inittype)) then
       inittype1 = inittype
    else
       inittype1 = 0
    end if
    if (inittype1 .lt. -1 .or.inittype1  .gt. 1) then
       write(*,*) "routine normsph,  hcpdtype.f90"
       write(*,*) "Illegal nittyoe:", inittype1
       stop 1
    end if

    hcpd%coeff = 0.d0

    do m=1,hcpd%nmode

       !$omp parallel do private(norm)
       do r=1,hcpd%rank

          call RANDOM_NUMBER(hcpd%sph(m)%htm(:,:,r))
          hcpd%sph(m)%htm(:,:,r) = hcpd%sph(m)%htm(:,:,r) - 0.5d0

          if (inittype1 .eq. 1) then
             hcpd%sph(m)%htm(:,:,r) =   hcpd%sph(m)%htm(:,:,r) &
                  + transpose(hcpd%sph(m)%htm(:,:,r))
          else if (inittype1 .eq. -1) then
             hcpd%sph(m)%htm(:,:,r) =   hcpd%sph(m)%htm(:,:,r) &
                  - transpose(hcpd%sph(m)%htm(:,:,r))
          end if

          norm = dnrm2(hcpd%subdim2(m),hcpd%sph(m)%htm(:,:,r),1)
          hcpd%sph(m)%htm(:,:,r) = hcpd%sph(m)%htm(:,:,r) / norm

       end do
       !$omp end parallel do
    end do


  end subroutine randomizehcpd



  subroutine hermitizehcpd(hcpd)
    implicit none
    type(hcpd_t), intent(INOUT)    :: hcpd
    integer :: m,r

    do m=1,hcpd%nmode
       !$omp parallel do
       do r=1, hcpd%rank/2
          hcpd%sph(m)%htm(:,:,r+hcpd%rank/2) = transpose(hcpd%sph(m)%htm(:,:,r))
       end do
       !$omp end parallel do
    end do

    do r=1, hcpd%rank/2
       hcpd%coeff(r+hcpd%rank/2) =  hcpd%coeff(r)
    end do

  end subroutine hermitizehcpd



  ! filltype (optional): how to initialize the SPOs
  !   filltype =  0: random (default)
  !   filltype =  1: random, hermitian
  !   filltype = -1, random, anti-hermitian
  !   not present: fill zeros
  subroutine changehrank(hcpd, newrank, filltype, nchunk)
    implicit none

    type(hcpd_t), intent(INOUT)    :: hcpd
    integer, intent(IN)            :: newrank
    integer, intent(IN), optional  :: filltype
    integer, intent(IN), optional  :: nchunk

    double precision, allocatable  :: oldcoeff(:)
    type(sph_t), allocatable       :: oldsph(:)

    integer                        :: filltype1
    integer                        :: nchunk1

    integer                        :: rperchunk, rperchunknew

    integer                        :: r,r1,r2,m,n,ierr
    double precision               :: norm, dnrm2

    external ddot


    if (present(filltype)) then
       filltype1 = filltype
    else
       filltype1 = 0
    end if
    if (filltype1 .lt. -1 .or.filltype1  .gt. 1) then
       write(*,*) "routine normsph,  hcpdtype.f90"
       write(*,*) "Illegal nittyoe:", filltype1
       stop 1
    end if



    if (present(nchunk)) then
       nchunk1 = nchunk
    else
       nchunk1 = 1
    end if

    if (modulo(newrank,nchunk1) .ne. 0) then
       if (present(nchunk)) then
          write(*,'(a)') "changerank: new rank must be a multiple of nchunks"
          write(*,'(a,i8)') "new rank:", newrank
          write(*,'(a,i8)') "nchunk:  ", nchunk
          stop 1
       end if
    end if

    if (newrank.lt.1) then
       write(*,'(a,i6,a)') "changerank: new rank= ",newrank, &
            ": must be 1 or larger."
       stop 1
    end if

    ! early exit if possible
    if (newrank .eq. hcpd%rank) return

    rperchunk = hcpd%rank/nchunk1
    rperchunknew = newrank/nchunk1

    allocate(oldcoeff(hcpd%rank))
    allocate(oldsph(hcpd%nmode))

    !$omp parallel do
    do m=1,hcpd%nmode
       allocate(oldsph(m)%htm(hcpd%subdim(m),hcpd%subdim(m),hcpd%rank))
       oldsph(m)%htm(:,:,:) = hcpd%sph(m)%htm(:,:,:)
       deallocate(hcpd%sph(m)%htm)
       allocate(hcpd%sph(m)%htm(hcpd%subdim(m),hcpd%subdim(m),newrank))
    end do
    !$omp end parallel do

    oldcoeff(:) = hcpd%coeff(:)


    deallocate(hcpd%coeff)
    allocate(hcpd%coeff(newrank))


    if (newrank .le. hcpd%rank) then

       r1 = 1
       r2 = 1
       do n=1,nchunk1
          do r=1,rperchunknew
             hcpd%coeff(r1) = oldcoeff(r2)
             r1 = r1 + 1
             r2 = r2 + 1
          end do
          r2 = n*rperchunk + 1
       end do


       do m=1,hcpd%nmode
          r1 = 1
          r2 = 1
          do n=1,nchunk1
             do r=1, rperchunknew
                hcpd%sph(m)%htm(:,:,r1) =  oldsph(m)%htm(:,:,r2)
                r1 = r1 + 1
                r2 = r2 + 1
             end do
             r2 = n*rperchunk + 1
          end do
       end do

    else
       ! newrank .gt. hcpd%rank

       r1 = 1
       r2 = 1
       do n=1,nchunk1
          do r=1,rperchunk
             hcpd%coeff(r1) = oldcoeff(r2)
             r1 = r1 + 1
             r2 = r2 + 1
          end do
          do r=rperchunk + 1, rperchunknew
             hcpd%coeff(r1) = 0.d0
             r1 = r1 + 1
          end do
       end do


       do m=1,hcpd%nmode
          r1 = 1
          r2 = 1
          do n=1,nchunk1
             do r=1,rperchunk
                hcpd%sph(m)%htm(:,:,r1) =  oldsph(m)%htm(:,:,r2)
                r1 = r1 + 1
                r2 = r2 + 1
             end do ! copy

             do r=rperchunk + 1, rperchunknew

                if (present(filltype)) then

                   call random_number(hcpd%sph(m)%htm(:,:,r1))
                   hcpd%sph(m)%htm(:,:,r1) = hcpd%sph(m)%htm(:,:,r1) -  0.5d0

                   if (filltype1 .eq. 1) then
                      hcpd%sph(m)%htm(:,:,r1) =   hcpd%sph(m)%htm(:,:,r1) &
                           + transpose(hcpd%sph(m)%htm(:,:,r1))
                   else if (filltype1 .eq. -1) then
                      hcpd%sph(m)%htm(:,:,r1) =   hcpd%sph(m)%htm(:,:,r1) &
                           - transpose(hcpd%sph(m)%htm(:,:,r1))
                   end if

                   norm = dnrm2(hcpd%subdim2(m), hcpd%sph(m)%htm(:,:,r1),1)
                   hcpd%sph(m)%htm(:,:,r1) = hcpd%sph(m)%htm(:,:,r1) / norm

                else

                   hcpd%sph(m)%htm(:,:,r1) = 0.d0

                end if

                r1 = r1 + 1

             end do ! fill

          end do ! nchunk

       end do ! m

    end if

    hcpd%rank = newrank


    deallocate(oldcoeff)
    do m=1,hcpd%nmode
       deallocate(oldsph(m)%htm)
    end do
    deallocate(oldsph)


  end subroutine changehrank


  subroutine copy_hcpd(hcpd_in, hcpd_out)
    implicit none

    type(hcpd_t), intent(IN)          :: hcpd_in
    type(hcpd_t), intent(INOUT)       :: hcpd_out
    integer                           :: m

    call allocate_hcpd_t(hcpd_out,hcpd_in%rank,hcpd_in%ndof,hcpd_in%nmode, &
         hcpd_in%gdim,hcpd_in%dofspf,hcpd_in%potdofmap, hcpd_in%modelabel, &
         hcpd_in%ort, hcpd_in%ortdim, hcpd_in%loned)

    hcpd_out%coeff(:) = hcpd_in%coeff(:)

    !$omp parallel do
    do m=1,hcpd_in%nmode
       hcpd_out%sph(m)%htm(:,:,:) = hcpd_in%sph(m)%htm(:,:,:)
    end do
    !$omp end parallel do

    if (any(hcpd_in%loned)) then
       hcpd_out%oned = hcpd_in%oned
    end if


  end subroutine copy_hcpd


  ! write information about the hcpd to a channel
  ! default channel is 6 (stdout)
  subroutine wrhcpdproperties(hcpd, channel)
    implicit none
    type(hcpd_t), intent(IN)       :: hcpd
    integer, intent(IN), optional  :: channel
    integer                        :: channel1
    logical                        :: isopen
    integer                        :: m,f, ilbl, jlbl
    double precision               :: mem, totmem
    character(len=80)              :: coords

    if (present(channel)) then
       channel1 = channel
    else
       channel1 = 6
    end if

    inquire(channel1, opened=isopen)
    if (.not. isopen) return

    write(channel1,'(a)') ""
    write(channel1,'(a)') "HCPD-Info:"
    write(channel1,'(a)') ""
    if (.not. hcpd%initialized) then
       write(channel1,'(a)') " >> HCPD object is not initialized <<"
       return
    end if
    write(channel1,'(a)') "SPH:"
    write(channel1,'(a)') "  Mode    gdim^2   MEM(SPH)   coordinates"

    totmem = 0.d0
    do m = 1,hcpd%nmode

       coords = ""
       ilbl = 1
       do f=1,hcpd%ndof
          if (hcpd%dofspf(f) .eq. m) then
             jlbl = ilbl + len_trim(hcpd%modelabel(f)) + 1
             coords(ilbl:jlbl) = trim(hcpd%modelabel(f))//" "
             ilbl = jlbl +1
          end if
       end do

       mem = dble(hcpd%rank*hcpd%subdim2(m))/131072 !MB
       totmem = totmem + mem
       write(channel1,'(i6,i8,f10.3,a)') m, hcpd%subdim2(m), &
            mem, " MB   "//trim(coords)
    end do
    if (any(hcpd%loned)) then
       mem = dble(hcpd%ortdim)/131072 !MB
       totmem = totmem + mem
       write(channel1,'(/a,i8,f10.3,a)') '    1D', hcpd%ortdim, mem, " MB   "
    else
       write(channel1,'(a)') '(no 1D terms)'
    end if

    write(channel1,'(a)') "---------------------------------------" &
         //"---------------"
    write(channel1,'(a,f14.3,a)') "Total MEM (SPH):   ", totmem, " MB"
    write(channel1,'(a)') ""
    mem = dble(hcpd%rank)/131072 !MB
    write(channel1,'(a,i7,3x,f10.3,a)') "Coefficients:", &
         hcpd%rank, mem, " MB"
    totmem = totmem + mem
    write(channel1,'(a)') ""
    write(channel1,'(a,f10.3,a)') "Total memory (hcpd):    ", &
         totmem, " MB"
    write(channel1,'(a)') ""
    call myflush(channel1)

  end subroutine wrhcpdproperties



  subroutine mpi_bcast_hcpd_t(hcpd)
#ifdef MPI

    use mpidata
    use global
    use mpi

    implicit none

    integer                          :: ndof, nmode, rank, ortdim, mpicommi(4)
    integer, allocatable             :: gdim(:),dofspf(:), potdofmap(:,:)
    character(len=c1), allocatable   :: modelabel(:)
    double precision, allocatable    :: ort(:)
    logical, allocatable             :: loned(:)
    integer                          :: ierr, htmsize,m

    type(hcpd_t), intent(INOUT)       :: hcpd

    if (mpirank.eq.0) then
       mpicommi(1) = hcpd%rank
       mpicommi(2) = hcpd%ndof
       mpicommi(3) = hcpd%nmode
       mpicommi(4) = hcpd%ortdim

       call MPI_Bcast(mpicommi,4,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(hcpd%gdim,      hcpd%ndof,    MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(hcpd%dofspf,    hcpd%ndof,    MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(hcpd%potdofmap, hcpd%ndof*2,  MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(hcpd%modelabel, hcpd%ndof*c1, MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(hcpd%ort,       hcpd%ortdim,  MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(hcpd%loned,     hcpd%ndof,    MPI_LOGICAL,0,MPI_COMM_WORLD,ierr)

       call MPI_Bcast(hcpd%coeff,     hcpd%rank,    MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)


       do m=1,hcpd%nmode
          htmsize = hcpd%subdim2(m)*hcpd%rank
          call MPI_Bcast(hcpd%sph(m)%htm, htmsize, MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
       end do

       if (any(hcpd%loned)) then
          call MPI_Bcast(hcpd%oned,   hcpd%ortdim,  MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
       end if

    else
       call MPI_Bcast(mpicommi,4,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       rank   = mpicommi(1)
       ndof   = mpicommi(2)
       nmode  = mpicommi(3)
       ortdim = mpicommi(4)

       allocate(gdim(ndof))
       allocate(dofspf(ndof))
       allocate(potdofmap(2,ndof))
       allocate(ort(ortdim))
       allocate(modelabel(ndof))
       allocate(loned(ndof))

       call MPI_Bcast(gdim,      ndof,    MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(dofspf,    ndof,    MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(potdofmap, ndof*2,  MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(modelabel, ndof*c1, MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(ort,       ortdim,  MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
       call MPI_Bcast(loned,     ndof,    MPI_LOGICAL,0,MPI_COMM_WORLD,ierr)

       call allocate_hcpd_t(hcpd,rank,ndof,nmode,gdim,dofspf,potdofmap, &
            modelabel,ort, ortdim,loned)

       call MPI_Bcast(hcpd%coeff,     hcpd%rank,    MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)

       do m=1,hcpd%nmode
          htmsize = hcpd%subdim2(m)*hcpd%rank
          call MPI_Bcast(hcpd%sph(m)%htm, htmsize, MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
       end do

       if (any(hcpd%loned)) then
          call MPI_Bcast(hcpd%oned,   hcpd%ortdim,  MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
       end if


       deallocate(gdim)
       deallocate(dofspf)
       deallocate(potdofmap)
       deallocate(ort)
       deallocate(modelabel)
       deallocate(loned)

    end if

#endif

    return

  end subroutine mpi_bcast_hcpd_t






!!$   ! overlap of two candecomps
!!$  subroutine overlap_hcpd_hcpd(hcpd1, hcpd2, ovlp)
!!$    use tensortype
!!$
!!$    implicit none
!!$    type(hcpd_t), intent(IN), target :: hcpd1, hcpd2
!!$
!!$#ifdef R16
!!$    real(kind=16), intent(OUT)       :: ovlp
!!$    real(kind=16)                    :: s, xi
!!$#else
!!$    real(kind=8), intent(OUT)        :: ovlp
!!$    real(kind=8)                     :: s, xi
!!$#endif
!!$    double precision                 :: DDOT
!!$
!!$    integer                          :: r1, r2, m
!!$
!!$    type(hcpd_t), pointer            :: p1,p2
!!$
!!$    external DDOT
!!$
!!$    if (.not. iscompatible(hcpd1, hcpd2)) then
!!$       write(*,*) "Incompatible tensors!"
!!$       stop 1
!!$    end if
!!$
!!$    p1 => hcpd1
!!$    p2 => hcpd2
!!$
!!$    ovlp = 0.d0
!!$
!!$    if (associated(p1,p2)) then
!!$
!!$       ! the two hcpd's are actually the same objects
!!$
!!$       !$omp parallel do private(r2,s,m) &
!!$       !$omp reduction(+:ovlp) schedule(guided)
!!$       do r1=1,hcpd1%rank
!!$          do r2=r1,hcpd2%rank
!!$             s = hcpd1%coeff(r1) * hcpd2%coeff(r2)
!!$
!!$             do m=1,hcpd1%nmode
!!$
!!$                xi = DDOT(hcpd1%subdim(m)**2, &
!!$                     hcpd1%factors(r2,m)%htm, 1, &
!!$                     hcpd1%factors(r1,m)%htm, 1)
!!$                s = s*xi
!!$
!!$             end do
!!$
!!$             if (r1.eq.r2) then
!!$                ovlp = ovlp + s
!!$             else
!!$                ovlp = ovlp + 2.d0*s
!!$             end if
!!$
!!$          end do
!!$       end do
!!$       !$omp end parallel do
!!$
!!$    else
!!$       ! the two hcpd objects are not the same objects
!!$
!!$       !$omp parallel do private(r2,s,m) &
!!$       !$omp reduction(+:ovlp) schedule(static)
!!$       do r1=1,hcpd1%rank
!!$          do r2=1,hcpd2%rank
!!$             s = hcpd1%coeff(r1) * hcpd2%coeff(r2)
!!$             do m=1,hcpd1%nmode
!!$                s = s * DDOT(hcpd1%subdim(m)**2, &
!!$                     hcpd2%factors(r2,m)%htm, 1, &
!!$                     hcpd1%factors(r1,m)%htm, 1)
!!$             end do
!!$
!!$             ovlp = ovlp + s
!!$          end do
!!$       end do
!!$       !$omp end parallel do
!!$    end if
!!$
!!$
!!$  end subroutine overlap_hcpd_hcpd
!!$
!!$
!!$
!!$
!!$
!!$
!!$
!!$
!!$
!!$  logical function ishermitian2(hcpd)
!!$    implicit none
!!$    type(hcpd_t), intent(IN)                :: hcpd
!!$
!!$    type(hcpd_t)         :: hcpd1
!!$    type(hcpd_t)         :: hcpd1T
!!$
!!$    integer, allocatable :: iflags2(:,:)
!!$    integer, allocatable :: iflags(:)
!!$
!!$    logical :: lmatch
!!$
!!$    integer          :: r,r1,m, nherm, naherm, nunsp
!!$    double precision :: mval
!!$    real(kind=8)    :: ovlp,ovlpt, s
!!$
!!$    allocate(iflags2(hcpd%nmode,hcpd%rank))
!!$    allocate(iflags(hcpd%rank))
!!$
!!$
!!$    nherm = 0
!!$    naherm = 0
!!$    nunsp = 0
!!$
!!$    ! first try line by line, If this is an MCTDH operator
!!$    ! it will likely find all hermitian lines.
!!$
!!$    do r=1,hcpd%rank
!!$       do m=1,hcpd%nmode
!!$          mval = sum((transpose(hcpd%factors(r,m)%htm) - hcpd%factors(r,m)%htm)**2)
!!$          if (mval .le. 1.d-8) then
!!$             iflags2(m,r) = 1
!!$          else
!!$             mval = sum((transpose(hcpd%factors(r,m)%htm) + hcpd%factors(r,m)%htm)**2)
!!$             if (mval .le. 1.d-8) then
!!$                iflags2(m,r) = -1
!!$             else
!!$                iflags2(m,r) = 0
!!$             end if
!!$          end if
!!$       end do
!!$       iflags(r) = product( iflags2(:,r))
!!$
!!$       if (iflags(r) .eq. 1) then
!!$          nherm = nherm+1
!!$       elseif  (iflags(r) .eq. -1) then
!!$          naherm = naherm+1
!!$       elseif (iflags(r) .eq. 0) then
!!$          nunsp = nunsp + 1
!!$       end if
!!$
!!$    end do
!!$
!!$
!!$    print*, "     hermitian: ", nherm
!!$    print*, "anti-hermitian: ", naherm
!!$    print*, "       unknown: ", nunsp
!!$
!!$
!!$    if (nherm .eq. hcpd%rank) then
!!$       ishermitian2 = .true.
!!$       return
!!$    end if
!!$
!!$
!!$
!!$    ! next we loop through all anti-hermitian and unknown
!!$    ! lines and check if another line is the
!!$    ! hermitian conjugated
!!$
!!$
!!$    do r=1,hcpd%rank
!!$       if (iflags(r) .ne. 1) then
!!$          do r1=r+1,hcpd%rank
!!$             if (iflags(r1) .ne. 1) then
!!$
!!$                lmatch = .true.
!!$                do m=1,hcpd%nmode
!!$                   mval = sum((transpose(hcpd%factors(r1,m)%htm) - hcpd%factors(r,m)%htm)**2)
!!$                   lmatch = lmatch .and. (mval .le. 1.d-8)
!!$                end do
!!$
!!$                if (lmatch) then
!!$
!!$                   iflags(r) = 1
!!$                   iflags(r1) = 1
!!$                end if
!!$             end if
!!$
!!$          end do
!!$       end if
!!$    end do
!!$
!!$    nherm = 0
!!$    naherm = 0
!!$    nunsp = 0
!!$    do r=1,hcpd%rank
!!$       if (iflags(r) .eq. 1) then
!!$          nherm = nherm+1
!!$       elseif  (iflags(r) .eq. -1) then
!!$          naherm = naherm+1
!!$       elseif (iflags(r) .eq. 0) then
!!$          nunsp = nunsp + 1
!!$       end if
!!$
!!$    end do
!!$
!!$    print*, "     hermitian: ", nherm
!!$    print*, "anti-hermitian: ", naherm
!!$    print*, "       unknown: ", nunsp
!!$
!!$    if (nherm .eq. hcpd%rank) then
!!$       ishermitian2 = .true.
!!$       return
!!$    end if
!!$
!!$    ! as a last resort we just brute-force
!!$    ! calculate the frobenius norm of
!!$    ! H-H^T of all remaining terms
!!$    !
!!$    ! we set all hermitian terms t0
!!$    ! unit operators with fromebius norm
!!$    ! one. This way only the non-hermitien
!!$    ! parts contribute.
!!$
!!$    ovlpt = 0.d0
!!$
!!$    do r=1,hcpd%rank
!!$       if (iflags(r) .ne. 1) then
!!$          do r1=1 ,hcpd%rank
!!$             if (iflags(r1) .ne. 1) then
!!$
!!$                s = 1.d0
!!$                do m=1,hcpd%nmode
!!$                   if (iflags2(m,r) .ne. 1 .or. iflags2(m,r1) .ne. 1) then
!!$                      s = s * sum( transpose(hcpd%factors(r,m)%htm) *  hcpd%factors(r1,m)%htm)
!!$                   end if
!!$                end do
!!$
!!$                ovlpt = ovlpt + s
!!$
!!$             end if
!!$          end do
!!$       end if
!!$    end do
!!$
!!$
!!$    ovlp = 0.d0
!!$
!!$    do r=1,hcpd%rank
!!$       if (iflags(r) .ne. 1) then
!!$          do r1=1 ,hcpd%rank
!!$             if (iflags(r1) .ne. 1) then
!!$
!!$                s = 1.d0
!!$                do m=1,hcpd%nmode
!!$                   if (iflags2(m,r) .ne. 1 .or. iflags2(m,r1) .ne. 1) then
!!$                      s = s * sum(hcpd%factors(r,m)%htm *  hcpd%factors(r1,m)%htm)
!!$                   end if
!!$                end do
!!$                   ovlp = ovlp + s
!!$
!!$             end if
!!$          end do
!!$       end if
!!$    end do
!!$
!!$
!!$    print*, ovlp - ovlpt
!!$
!!$    ishermitian2 = abs(ovlp - ovlpt) .le. 1.d-8
!!$
!!$  end function ishermitian2
!!$
!!$
!!$
!!$
!!$  logical function ishermitian(hcpd, epsilon, diff)
!!$    implicit none
!!$    type(hcpd_t), intent(IN)                :: hcpd
!!$    double precision, intent(IN), optional  :: epsilon
!!$    double precision, intent(OUT), optional :: diff
!!$    type(hcpd_t)                            :: hcpdT
!!$
!!$    real(kind=16)     :: hh, ht,th, tt
!!$    double precision  :: DLAMCH, epsilon1, diff1
!!$    integer :: r,m
!!$
!!$    if (present(epsilon)) then
!!$       epsilon1 = epsilon
!!$    else
!!$       epsilon1 = DLAMCH("E")
!!$    end if
!!$
!!$    call allocate_hcpd_t(hcpdT,hcpd%rank,hcpd%ndof,hcpd%nmode,hcpd%gdim, &
!!$         hcpd%dofspf,hcpd%potdofmap,hcpd%modelabel,0)
!!$
!!$
!!$    do r=1,hcpd%rank
!!$       do m=1,hcpd%nmode
!!$          hcpdT%factors(r,m)%htm = transpose(hcpd%factors(r,m)%htm)
!!$       end do
!!$       hcpdT%coeff(r) =  hcpd%coeff(r)
!!$    end do
!!$
!!$    call overlap_hcpd_hcpd(hcpd,  hcpd,  hh)
!!$    call overlap_hcpd_hcpd(hcpd,  hcpdT, ht)
!!$
!!$    diff1 = sqrt(abs(2*hh - 2*ht))
!!$
!!$    ishermitian = diff1 .le. epsilon1
!!$
!!$
!!$    if (present(diff)) then
!!$       diff = diff1
!!$    end if
!!$
!!$  end function ishermitian



!!$  subroutine rwhcpd(hcpd,dvr,fname,loverwrite)
!!$    use dvrtype_m
!!$    use eximporttools
!!$    implicit none
!!$    type(hcpd_t), intent(IN)     :: hcpd
!!$    type(dvr_t), intent(IN)      :: dvr
!!$    character(len=*), intent(IN) :: fname
!!$    logical, intent(IN)          :: loverwrite
!!$
!!$    character(len=7)             :: stat
!!$    character(len=80)            :: buffer
!!$
!!$    integer                      :: onedipot(hcpd%ndof)
!!$    integer                      :: m,r,i,ierr, lcount
!!$
!!$
!!$    include "channels.inc"
!!$    include "global.inc"
!!$
!!$
!!$    if (.not. dvr%initialized) then
!!$       write(*,*) "ERROR in rwhcpd: DVR is not initialized."
!!$       write(*,*) "      Aborting. Nothing was written."
!!$       return
!!$    end if
!!$
!!$    if (.not. hcpd%initialized) then
!!$       write(*,*) "ERROR in rwhcpd: tensor is not initialized."
!!$       write(*,*) "      Aborting. Nothing was written."
!!$       return
!!$    end if
!!$
!!$    stat = "new"
!!$    if (loverwrite) stat="unknown"
!!$    open(ioper, file=trim(fname),status=stat,form="unformatted",err=990)
!!$
!!$    write(ioper,iostat=ierr) progver
!!$    call exportDVRdef(dvr,ioper)
!!$
!!$    call fpstartup(buffer)
!!$    write(ioper,err=991) buffer
!!$
!!$
!!$    ! input blabla
!!$    lcount = 2
!!$    write(ioper,iostat=ierr) lcount
!!$    write(buffer,'(a,i6)') ' H-Candecomp rank:',hcpd%rank
!!$    write(ioper,iostat=ierr) buffer
!!$    write(buffer,'(a,11(1x,a))') ' Modelabels:', &
!!$         (trim(hcpd%modelabel(i)),i=1,min(hcpd%ndof,11))
!!$    write(ioper,iostat=ierr) buffer
!!$
!!$    ! no 1D pots
!!$    onedipot = 0
!!$    write(ioper) hcpd%rank, onedipot
!!$
!!$    call exportgrddef(hcpd, ioper,.false.)
!!$
!!$
!!$    do r=1,hcpd%rank
!!$       write (ioper) hcpd%coeff(r)
!!$       do m=1, hcpd%nmode
!!$          call vecout(hcpd%sph(m)%htm(:,:,r), 1, hcpd%subdim2(m),1, ioper)
!!$       end do
!!$    end do
!!$
!!$
!!$
!!$    close(ioper)
!!$
!!$    return
!!$
!!$990 continue
!!$    routine = "rwhcpd"
!!$    message = "ERROR while opening file '"//trim(fname)//"' for writing."
!!$    call errormsg
!!$991 continue
!!$    routine = "rwhcpd"
!!$    message = "ERROR while writing file '"//trim(fname)//"'."
!!$    call errormsg
!!$  end subroutine rwhcpd


end module hcpdtype_s
