! -*- f90 -*-

module mcalsjfunc_m
  use logdat
  implicit none

  double precision, allocatable :: mv2diff(:)
  double precision, allocatable :: mcoeff2(:)
  double precision, allocatable :: modefunc(:)
  character(len=32)             :: jfmtstr
  character(len=32)             :: cfmtstr
  logical, save                 :: jfuncinitialized = .false.


contains

  subroutine init_jfunc(nmode)
    implicit none
    integer, intent(IN) :: nmode

    call free_jfunc()

    allocate(mv2diff(nmode))
    allocate(mcoeff2(nmode))
    allocate(modefunc(nmode))

    write(jfmtstr,'(a,i8,a)') '(i8,', nmode,'ES16.8)'
    call rmblnk(jfmtstr)


    call openjfuncfiles

    jfuncinitialized = .true.

  end subroutine init_jfunc


  subroutine free_jfunc()
    use channels
    implicit none

    if (allocated(mv2diff)) deallocate(mv2diff)
    if (allocated(mcoeff2)) deallocate(mcoeff2)
    if (allocated(modefunc)) deallocate(modefunc)

    close(ijfunc1)
    close(ijfunc2)
    close(ijfunc3)

  end subroutine free_jfunc



  subroutine init_cvecout(rank)
    use global
    use channels
    implicit none

    integer, intent(IN) :: rank


    character(len=256)               :: fname
    integer                          :: ierr
    logical                          :: lexist

    fname = trim(name)//"/coeff"

    inquire(file=fname,exist=lexist)
    if (lexist .and. .not. lovwr) then
       message = "Cannot overwite file, use -w option: "//trim(fname)
       routine = "init_cvecout"
       call errormsg
    end if

    open(unit=ictr,file=fname,iostat=ierr)
    if (ierr.ne.0) then
       message = "Unable to open file: "//trim(fname)
       routine = "init_cvecout"
       call errormsg
    end if

    write(cfmtstr,'(a,i8,a)') '(i8,',rank,'ES16.8)'

  end subroutine init_cvecout



  subroutine cvecout(coeff,rank,iter)
    use channels
    implicit none
    integer, intent(IN)          :: rank, iter
    double precision, intent(IN) :: coeff(rank)
    integer                      :: r

    write(ictr,cfmtstr) iter, (coeff(r), r=1,rank)

  end subroutine cvecout


  subroutine free_cvecout()
    use channels
    implicit none

    cfmtstr = ""
    close(ictr)

  end subroutine free_cvecout



  subroutine openjfuncfiles()

    use global
    use channels
    use logdat
    implicit none

    character(len=256)               :: fname
    logical                          :: lexist
    integer                          :: ierr



    fname = trim(name)//"/modefunc_diff"

    inquire(file=fname,exist=lexist)
    if (lexist .and. .not. lovwr .and. .not. lcont) then
       message = "Cannot overwite file, use -w option: "//trim(fname)
       routine = "openjfuncfiles"
       call errormsg
    end if

    if (lexist .and. lcont) then
       open(unit=ijfunc1,file=fname, iostat=ierr, position='append')
    else
       open(unit=ijfunc1,file=fname, iostat=ierr)
    end if

    if (ierr.ne.0) then
       message = "Error while opening file: "//trim(fname)
       routine = "openjfuncfiles"
       call errormsg
    end if

    if (.not. lcont) then
       write(ijfunc1, '(a)') "#-----------------------------------------------------------#"
       write(ijfunc1, '(a)') "# Contains \sum_s\sum_{i_m} (V_{s,i_m} - Vcpd_{s,i_m})^2    #"
       write(ijfunc1, '(a)') "# for each mode m, i.e., the 1-mode fit error for each mode #"
       write(ijfunc1, '(a)') "# calulated on the complete grid on one mode and the Monte- #"
       write(ijfunc1, '(a)') "# Carlo grid on all other modes.                            #"
       write(ijfunc1, '(a)') "#-----------------------------------------------------------#"
       write(ijfunc1, '(a)') ""
       write(ijfunc1, '(a)') "#   iter          mode 1          mode 2  ..."
       call myflush(ijfunc1)
    end if


    fname = trim(name)//"/modefunc_reg"

    inquire(file=fname,exist=lexist)
    if (lexist .and. .not. lovwr .and. .not. lcont) then
       message = "Cannot overwite file, use -w option: "//trim(fname)
       routine = "openjfuncfiles"
       call errormsg
    end if

    if (lexist .and. lcont) then
       open(unit=ijfunc2,file=fname, iostat=ierr,position='append')
    else
       open(unit=ijfunc2,file=fname, iostat=ierr)
    end if

    if (ierr.ne.0) then
       message = "Error while opening file: "//trim(fname)
       routine = "openjfuncfiles"
       call errormsg
    end if

    if (.not. lcont) then
       write(ijfunc2, '(a)') "#---------------------------------------------------------------#"
       write(ijfunc2, '(a)') "# Contains \sum_r c_r^2 \sum_s\sum_{i_m} \Omega_{s,i_m}^2       #"
       write(ijfunc2, '(a)') "# of the CANDECOP coefficients and the diagonal of the overlap  #"
       write(ijfunc2, '(a)') "# matrix, i.e., the penalty that leads to regularization in the #"
       write(ijfunc2, '(a)') "# working eqautions for mode m. Below the results for each mode #"
       write(ijfunc2, '(a)') "#---------------------------------------------------------------#"
       write(ijfunc2, '(a)') ""
       write(ijfunc2, '(a)') "#   iter          mode 1          mode 2  ..."
       call myflush(ijfunc2)
    end if


    fname = trim(name)//"/modefunc_tot"

    inquire(file=fname,exist=lexist)
    if (lexist .and. .not. lovwr .and. .not. lcont) then
       message = "Cannot overwite file, use -w option: "//trim(fname)
       routine = "openjfuncfiles"
       call errormsg
    end if

    if (lexist .and. lcont) then
       open(unit=ijfunc3,file=fname, iostat=ierr,position='append')
    else
       open(unit=ijfunc3,file=fname, iostat=ierr)
    end if

    if (ierr.ne.0) then
       message = "Error while opening file: "//trim(fname)
       routine = "openjfuncfiles"
       call errormsg
    end if
    if (.not. lcont) then
       write(ijfunc3, '(a)') "#-------------------------------------------------------------#"
       write(ijfunc3, '(a)') "# Contains \sum_s\sum_{i_m} (V_{s,i_m} - Vcpd_{s,i_m})^2      #"
       write(ijfunc3, '(a)') "# + epsilon * \sum_r c_r^2  \sum_s\sum_{i_m} \Omega_{s,i_m}^2 #"
       write(ijfunc3, '(a)') "# I.e., the complete functional for modes m as the sum of the #"
       write(ijfunc3, '(a)') "# difference term and the regularization term                 #"
       write(ijfunc3, '(a)') "# = modefunc_diff + epsilon* modefunc_reg                     #"
       write(ijfunc3, '(a)') "#-------------------------------------------------------------#"
       write(ijfunc3, '(a)') ""
       write(ijfunc3, '(a)') "#   iter          mode 1          mode 2  ..."
       call myflush(ijfunc3)
    end if


  end subroutine openjfuncfiles


end module mcalsjfunc_m



! cpd%subdim might be extended, therefore an extra subdim
! dot_products only use original grid
subroutine calcfuncs1m(m,ovlp,BT,cpd, epsilon, subdim1)
  use cpdtype_m
  use mccuts_m
  use runfit_m, only: curr_ntotsample
  use mcalsjfunc_m
  implicit none

  integer, intent(IN)             :: m, subdim1
  type(cpd_t), intent(IN)         :: cpd
  double precision, intent(IN)    :: ovlp(cpd%rank,cpd%rank)
  double precision, intent(IN)    :: BT(cpd%subdim(m),cpd%rank)
  double precision, intent(IN)    :: epsilon


  double precision                :: Vcpd2, VVcpd, mov, cf2
  integer                         :: r,r1, nsample

  ! BLAS
  double precision                :: ddot


  nsample = curr_ntotsample


  ! we use integral W(V-Vcpd)^2 = integral W(V^2 - 2VVcpd + Vcpd^2)
  ! V^2 comes from the cuts and is only done once while loading them
  ! the numbers are in the mccuts module
  ! The OVLP and BT matrices have the Monte-Carlos integrals
  ! of Vcpd^2 and 2VVcpd already. We just need to do the r-sum here and
  ! the integral over the m'th primitive mode grid
  !
  ! The loop below also calculates the regularization part
  ! as it uses the diagonal terms


  Vcpd2 = 0.d0  ! overlap
  cf2 = 0.d0    ! regularization

  !$omp parallel do private(r1,mov) reduction(+:Vcpd2,cf2) schedule(dynamic,1)
  do r=1,cpd%rank

     mov = ddot(subdim1,cpd%spp(m)%htm(:,r),1,cpd%spp(m)%htm(:,r),1)

     Vcpd2 = Vcpd2 + mov * ovlp(r,r) * cpd%coeff(r)**2

     cf2 = cf2 +  mov * ovlp(r,r) * cpd%coeff(r)**2


     do r1=r+1, cpd%rank

        mov = ddot(subdim1,cpd%spp(m)%htm(:,r),1,cpd%spp(m)%htm(:,r1),1)

        Vcpd2 = Vcpd2 + 2.d0 * mov * ovlp(r1,r) * cpd%coeff(r) *  cpd%coeff(r1)

     end do
  end do
  !$omp end parallel do


  Vcpd2 = Vcpd2/nsample/subdim1
  cf2 = cf2/nsample/subdim1


  VVcpd = 0.d0
  !$omp parallel do private(mov) reduction(+:VVcpd) schedule(static)
  do r=1,cpd%rank

     mov =  ddot(subdim1,BT(:,r),1,cpd%spp(m)%htm(:,r),1)

     VVcpd = VVcpd +  cpd%coeff(r)*mov

  end do
  !$omp end parallel do
  VVcpd = VVcpd/nsample/subdim1


  mv2diff(m) = (intvcuts2(m)/nsample/subdim1 + Vcpd2 - 2.d0*VVcpd)
  mcoeff2(m) = cf2*epsilon
  modefunc(m) =  mv2diff(m) + cf2*epsilon


end subroutine calcfuncs1m





subroutine wralsfuncs(iter)
  use mcalsjfunc_m
  use global
  use channels
  implicit none


  integer, intent(IN) :: iter
  integer             :: m,nmode


  if (.not. jfuncinitialized) then
     routine = "wralsfuncs"
     message = "Functionals are not initialized and files are not open. "// &
          "Please file a bug report."
     call errormsg
  end if

  nmode = size(mv2diff,1)

  write(ijfunc1,jfmtstr) iter, (mv2diff(m),m=1,nmode)
  call myflush(ijfunc1)
  write(ijfunc2,jfmtstr) iter, (mcoeff2(m),m=1,nmode)
  call myflush(ijfunc2)
  write(ijfunc3,jfmtstr) iter, (modefunc(m),m=1,nmode)
  call myflush(ijfunc3)

end subroutine wralsfuncs
