subroutine logmcpfitparams(channel,npmode,potdim)
  use global
  use pglobal

  use rshdmr
  use mcsampling
  use mcprecond_m
  use mcpfitparams
  use conjgrad_m


  implicit none
  integer, intent(IN)  :: channel, npmode
  integer, intent(IN)  :: potdim(npmode)
  logical              :: isopen
  integer              :: m

  inquire(channel, opened=isopen)
  if (.not. isopen) return


  write(channel, '(a)') ""
  write(channel, '(a)') "-----------------------------------"
  write(channel, '(a)') "-->    MC-Potfit: Parameters    <--"
  write(channel, '(a)') "-----------------------------------"
  write(channel, '(a)') ""


  write(channel, '(a,i11)') "Seed for randum number generator is", mcseed
  write(channel, '(a)') ""

  if (lloadevecs) then
     write(channel, '(a)') "SPPs are read from evecs files (if present) or are"
     write(channel, '(a)') "re-calculated from density files in name-directory."
  end if

  call logsampling(channel)


  if (.not. lloadevecs) then
     if (lasciidensity) then
        write(channel, '(a)') "Reduced densities are stored in ASCII format."
     else
        write(channel, '(a)') "Reduced densities are stored in binary format."
     end if
  end if


  if (lsaveevecs) then
     write(channel, '(a)') "Eigenvectors of densities are stored in same format."
  end if

  write(channel, '(a)') ""
  if (lasciisamplings) then
     write(channel, '(a)') "Sampled grid points are stored as ASCII format"
  else
     write(channel, '(a)') "Sampled grid points  are stored in binary format"
  end if
  write(channel, '(a)')

  if (invertmethod .eq. mc_inv_spd) then
     write(channel, '(a)') "!!! The coefficients are calculated by direct"
     write(channel, '(a)') "!!! inversion of the SPP-overlap matrix."
     write(channel, '(a)') "!!! The overlap matrix IS EXPLICITLY built."
  else if (invertmethod .eq. mc_inv_conjgrad) then
     write(channel, '(a)') "!!! Coefficients are calculated using "
     write(channel, '(a)') "!!! Conjugate Gradients."
     write(channel, '(a)') "!!! The overlap matrix IS NOT built"
     write(channel, '(a,i8)') "!!! Maximum CG iterations:", cgmaxiter
     write(channel, '(a,E16.8)') "!!! Maximum CG tolerance:", cgtolerance
     select case( cg_precon_type)
     case(cg_precon_none)
        write(channel, '(a)') "!!!   preconditioner: none"
     case(cg_precon_diag)
        write(channel, '(a)') "!!!   preconditioner: diagonal (jacobi)"
     case(cg_precon_band)
        write(channel, '(a)') "!!!   preconditioner: band-matrix"
        write(channel, '(a,i4)') "!!!    sub-diagonals:", cg_precon_size
     case(cg_precon_sparse)
        write(channel, '(a)') "!!!   preconditioner: sparse-matrix"
        write(channel, '(a)') "!!!     Use mode indices less/equal:"
        do m=1,npmode
           write(channel, '(3(a,i4))') "!!!   mode ", m, ":",cg_precon_luse(m),'/',potdim(m)
        end do
     end select
  end if

  write(channel, '(a)') ""
  if (lregularization_coeff) then
     write(channel, '(a)') "!!! The overlap matrix is regularized before"
     write(channel, '(a)') "!!! invertation:"
     write(channel, '(a,E16.8)') "!!! epsilon-invert = ",regularization_coeff
  end if
  write(channel, '(a)') ""

  if (smethod(mc_task_spp,1) .ne. mc_complete .and. .not. lloadevecs) then
     if (lomegaspp) then
        write(channel, '(a)') "Allocate OmegaSPP matrix: YES"
     else
        write(channel, '(a)') "Allocate OmegaSPP matrix:  NO"
     end if
  end if

  if (smethod(mc_task_coeff,1) .ne. mc_complete) then
     if (lomega) then
        write(channel, '(a)') "Allocate Omega matrix:    YES"
     else
        write(channel, '(a)') "Allocate Omega matrix:     NO"
     end if
     if ( invertmethod .eq. mc_inv_conjgrad) then
        if (lomegat) then
           write(channel, '(a)')  "Allocate Omega^T matrix:  YES"
        else
           write(channel, '(a)')  "Allocate Omega^T matrix:   NO"
        end if
     end if
  end if


  if (lsubtractpes) then
     write(channel, '(a)') "A PES is subsracted from the potential."
     write(channel, '(a)') "PES-file = "//trim(operfile)
  end if

  write(channel, '(a)') ""

  if (lrshdmr) then
     write(channel, '(a)') " Before potfitting, a Random Sampling HDMR is subtracted."
     write(channel, '(a)') ""
  end if

  call myflush(channel)

end subroutine logmcpfitparams




subroutine logmccpdparams(channel)
  use global
  use pglobal
  use mccpdparams
  use mcpfitparams
  implicit none

  integer, intent(IN)  :: channel
  logical              :: isopen

  inquire(channel, opened=isopen)
  if (.not. isopen) return

  write(channel, '(a)') ""
  write(channel, '(a)') "-----------------------------------"
  write(channel, '(a)') "-->      MC-CPD: Parameters     <--"
  write(channel, '(a)') "-----------------------------------"
  write(channel, '(a)') ""

  if (loperfit) then
     write(channel, '(a)') ""
     write(channel, '(a)') "-----------------------------------"
     write(channel, '(a)') "-->  THIS IS AN OPERATOR FIT    <--"
     if (lhermitian) then
        write(channel, '(a)') "-->         HERMITIAN           <--"
     end if
     write(channel, '(a)') "-----------------------------------"
     write(channel, '(a)') ""
  else
     write(channel, '(a)') ""
     write(channel, '(a)') "-----------------------------------"
     write(channel, '(a)') "-->     THIS IS A PES FIT       <--"
     write(channel, '(a)') "-----------------------------------"
     write(channel, '(a)') ""
  end if


  write(channel, '(a,i11)') "Seed for randum number generator is", mcseed
  write(channel, '(a)') ""

  if (linitrandom) then
     write(channel, '(a/)') "CPD is initialized with random-numbers"
  else if (len_trim(restarcpd) .ne. 0 .or. lcont) then
     write(channel, '(a)') "CPD is initialized from file:"
     if (lcont) then
        write(channel, '(a)') trim(name)//"/cpd"
     else
        write(channel, '(a)') trim(restarcpd)
     end if
  else
     write(channel, '(a/)') "CPD is initialized from cuts."
  end if

  if ( lgrow ) then
     write(channel, '(/a)')   "CPD will grow:"
     write(channel, '(a,i8)') " initial rank = ", initrank
     write(channel, '(a,i8)') " final rank   = ", maxrank
     write(channel, '(a,i8)') " delta rank   = ", growrank
     write(channel, '(a,i8)') " iiter grow   = ", growiter
  else
     write(channel, '(/a,i8)')   "CPD rank = ", maxrank
  end if
  write(channel, '(a)') ""


  if (lloadcuts) then
     write(channel, '(a)') "Random cuts are read from files."
  end if

  if (ltryreusecuts) then
     write(channel, '(a)') "trying to re-use existing cut-files"
  end if



  write(channel, '(a)') "!!! The alternating least squares method"
  if (invertmethod .eq. mc_inv_spd) then
     write(channel, '(a)') "!!! uses a Cholesky decomposition to solve"
     write(channel, '(a)') "!!! for coefficients and basis functions."
     write(channel, '(a,ES16.8)') "!!! Regularization: uniform, val=",epsilon_invert
  else if (invertmethod .eq. mc_inv_eigen) then
     write(channel, '(a)') "!!! Uses eigen-decomposition to solve"
     write(channel, '(a)') "!!! for coefficients and basis functions."
     write(channel, '(a,ES16.8)') "!!! Regularization: exponential, minval=",epsilon_invert
  end if

  if (lcoeffoncuts) then
     write(channel, '(/a/)') " Monte-Carlo integration on cuts is used for finalizing coefficients"
  else
     if (lfincoeff) then
        write(channel, '(/a/)') " Standard Monte-Carlo integration is used for finalizing coefficients."
     else
        write(channel, '(/a/)') " Coefficients are not finalized (but used as they come out of the ALS)."
     end if
  end if

  if (lsubtractpes) then
     write(channel, '(a)') "A PES is subsracted from the potential."
     write(channel, '(a)') "PES-file = "//trim(operfile)
  end if

  call myflush(channel)

end subroutine logmccpdparams


subroutine log_parallelization(channel)
  use mpidata
#ifdef _OPENMP
  use omp_lib
#endif
  implicit none
  integer, intent(IN) :: channel
  integer             :: nthreads

  write(channel, '(/a/)') "******* PARALLELIZATION ******"

#ifdef _OPENMP
  !$omp parallel
  nthreads = omp_get_num_threads()
  !$omp end parallel

  write(channel, '(a,i4)') 'Shared memory threads (OpenMP):     ', nthreads
#else
  write(channel, '(a)')    'Shared memory threads (OpenMP):     None'
#endif


#ifdef MPI
  write(channel, '(a,i4,2/)') 'Distributed memory instances (MPI): ', nmpi
#else
  write(channel, '(a,i4,2/)') 'Distributed memory instances (MPI): None'
#endif



end subroutine log_parallelization
