! -*- f90 -*-

!***********************************************************************
!
!            MCPOTFIT  subroutine  MCPEINGABE
!
! Reads the input data from the input file.
!
! Called from: mcpotfit
! Calls to: mcpopenin, mcpopenlog,rdinpf,mcpeinfile,peinbas,
!           mcploginp,mcpopendat,mcpausgab,abspath,warnmsg,errormsg,myflush
!
! 11/97 HDM
! 10/13 MS
!
!***********************************************************************



subroutine mcpeingabe (linperr)
  use mcpfitparams
  use mcsampling
  use rshdmr
  use symmops_m, only: lsymmetries

  use global
  use pglobal
  use maxv
  use griddatmod
  use operdef
  use combined
  use channels
  use mpidata
  use peingutil
  use rddvrmod
  use einpbasmod, only: einpbas
  use peinbasmod, only: peinbas
  use op1lib, only: initvxl

  implicit None

  integer           :: ic,iz,ierr,i,ilbl,chkdvr,m,n,f, task
#ifdef MPI
  integer           :: ndrop, nred
#endif
  integer           :: nfile
  logical           :: modelist(maxdim)
  logical           :: linperr,lerr,lmnd,lghost, lwarn1
  character(len=c5) :: filename

  ! set all modules to default
  call mcpdefault()

!-----------------------------------------------------------------------
! open the inputfile : argument.inp
!-----------------------------------------------------------------------
  call mcpopenin(lmnd)

!-----------------------------------------------------------------------
! read run-section and open log file
!-----------------------------------------------------------------------
  rewind(iin)
  iz=0
10 continue

  call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
  if (ierr .eq. 1) then
     linperr = .true.
     routine = 'mcpeingabe (run-section)'
     message = inptit
     call errormsg
  endif

  if (keyword(1) .eq. 'run-section') then
     call mcpeinrun(linperr,iz)
  else if (keyword(1) .eq. 'end-input') then
     routine = 'mcpeingabe (run-section)'
     message='No RUN-SECTION in input file'
     call warnmsg
  else
     go to 10
  endif


!-----------------------------------------------------------------------
! Check that the name directory exists.
! The inquire command does not work on DECs if applied to directories.
! We thus open a scratch file to find out whether the name-directory
! exists.
!----------------------------------------------------------------------
  open(itmpin,file=name(1:laenge)//'/scratch',status="unknown",err=666)
  close(itmpin,status='delete')
  if(lmnd) write(6,'(2a)') 'Name-directory already exist: ', &
       name(1:laenge)
  goto 667

666 if(lmnd) then
     inptit='mkdir '//name(1:laenge)//char(zeroi)
     call excmd2(ierr, inptit)
     if (ierr.ne.0) then
        write(6,'(2a)') '### ERROR: ', &
             'Invalid path for name-directory (-mnd option). ###'
        call stopnow
     endif
     write(6,'(2a)') 'Created name-directory: ',name(1:laenge)
  else
     write(6,'(a)') &
          '---------------------------------------------------------'
     write(6,'(a)') ' ################   ERROR   ##################'
     write(6,'(a)') ' There is no name-directory! The directory:'
     write(6,'(1x,a)') name(1:laenge)
     write(6,'(a)') ' does not exist! (or has no write permission)'
     write(6,'(2a)')' One may use the option -mnd to create the', &
          ' name-directory.'
     write(6,'(a)') &
          '-------------------------------------------------------'
     if (.not.lshort) call stopnow
  endif

667  if (.not.lshort) then
        filename=name(1:laenge)//'/log'
        call popenlog(filename)
     endif

!-----------------------------------------------------------------------
! read primitive-basis-section
!-----------------------------------------------------------------------
  if (irundvr .eq. 2) then
     filename=dname(1:dlaenge)//'/dvr'
     write(ilog,'(/,1x,75a1)')  ('-',i=1,75)
     write(ilog,'(a)') 'Reading DVR file: '//filename
     write(ilog,'(1x,75a1)')  ('-',i=1,75)
     open(idvr,file=filename,form='unformatted',status='old',err=1000)
     chkdvr=1
     call dvrinfo(lerr,chkdvr)
     close(idvr)

  else if (irundvr .eq. 1) then
     rewind(iin)
     iz=0
210  continue

     call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
     if (ierr .eq. 1) then
        routine = 'mcpeingabe (primitive-basis-section)'
        message = inptit
        call errormsg
     endif

     if (keyword(1) .eq. 'pbasis-section' .or. &
          keyword(1) .eq. 'primitive-basis-section') then
        call einpbas(linperr,iz)
     else if (keyword(1) .eq. 'end-input') then
        routine='mcpeingabe (primitive-basis-section)'
        message='No PRIMITIVE-BASIS-SECTION in input file'
        call warnmsg
     else
        go to 210
     endif
  endif

!-----------------------------------------------------------------------
! read natpot-basis-section
!-----------------------------------------------------------------------
  rewind(iin)
  iz=0
220 continue

  call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
  if (ierr .eq. 1) then
     linperr = .true.
     routine = 'mcpeingabe (natpot-basis-section)'
     message = inptit
     call errormsg
  endif

  if (keyword(1) .eq. 'natpot-basis-section') then
     call peinbas
  else if (keyword(1) .eq. 'end-input') then
     routine = 'mcpeingabe (natpot-basis-section)'
     message='No NATPOT-BASIS-SECTION in input file'
     call warnmsg
  else
     go to 220
  endif

!-----------------------------------------------------------------------
! read operator-section
!-----------------------------------------------------------------------
  rewind(iin)
  iz=0
510 continue

  call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
  if (ierr .eq. 1) then
     routine = 'mcpeingabe (operator-section)'
     message = inptit
     call warnmsg
  endif

  if (keyword(1) .eq. 'operator-section') then
     call mcpeinoper(linperr,iz)
  else if (keyword(1) .eq. 'end-input') then
     routine = 'mcpeingabe (operator-section)'
     message='No OPERATOR-SECTION in input file'
     call warnmsg
  else
     go to 510
  endif


!-----------------------------------------------------------------------
! read sampling-section
!-----------------------------------------------------------------------
  rewind(iin)
  iz=0
810 continue

  call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
  if (ierr .eq. 1) then
     routine = 'mcpeingabe (sampling-section)'
     message = inptit
     call warnmsg
  endif

  if (keyword(1) .eq. 'sampling-section') then
     call mcpeinsampling(linperr,iz)
  else if (keyword(1) .eq. 'end-input' .and. .not. lloadevecs) then
     routine = 'mcpeingabe (sampling-section)'
     message='No SAMMPLING-SECTION in input file'
     call warnmsg
  else
     go to 810
  endif


!-----------------------------------------------------------------------
! read symmetry-section
!-----------------------------------------------------------------------
  rewind(iin)
  iz=0
910 continue

  call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
  if (ierr .eq. 1) then
     routine = 'mcpeingabe (symmetry-section)'
     message = inptit
     call warnmsg
  endif

  if (keyword(1) .eq. 'symmetry-section') then
     lsymmetries = .true.
     call mcpeinsymmeties(linperr,iz)
  else if (keyword(1) .eq. 'end-input') then
     lsymmetries = .false.
     go to 930
  else
     go to 910
  endif
930 continue


!-----------------------------------------------------------------------
! read separable-weight-section
!-----------------------------------------------------------------------
  rewind(iin)
  iz=0


!-----------------------------------------------------------------------
! Checks for consistency
!-----------------------------------------------------------------------
  routine = 'mcpeingabe  (consistency check)'

  call initvxl(modelist,ndof)
  do m=1,npmode
     do n=1,npotdof(m)
        f=potdof(n,m)
        modelist(f)=.false.
     enddo
  enddo
  do f = 1, ndof
     if(basis(f).eq.0) then
        message = 'There must not be an electronic basis in the' &
             //' primitive-basis-section of a potfit run.'
        call errormsg
     endif
     if(basis(f).eq.6) then
         message = 'sphFBR is not allowed for POTFIT (not a grid ' &
              //'representation). See "Remarks on KLeg, PLeg and sphFBR:" ' &
              //'on HTML-docu (Input Documentation/Primitive-Basis-Section).'
         call errormsg
      endif
   enddo
   lghost=.false.
   do f = 1, ndof
      lghost=lghost.or.modelist(f)
   enddo
   if(lghost) then
      i=6
35    write(i,'(4(/a))') ' ## WARNING : There are unused DOFs.', &
           ' One better remove those from the Primitive-Basis-Section,', &
           ' as this may lead to problems, if the natpot is not the only', &
           ' operator in a Hamitonian-Section line.'
      do f = 1, ndof
         if(modelist(f)) write(i,'(2a)') ' Modelabel:  ',modelabel(f)
      enddo
      write(i,*)
      if(i.ne.ilog) then
         i=ilog
         goto 35
      endif
   endif


   do task=1,mc_ntasks
      do nfile = 1,nidxfile(task)
         if (smethod(task,nfile) .eq. mc_complete) then
            ntotsample(task,nfile) = 1
            do f=1,ndof
               ntotsample(task,nfile) = ntotsample(task,nfile) * gdim(f)
            end do

         elseif ( smethod(task,nfile) .eq. mc_readfile) then

            call chksamplingfile(task,nfile)

         end if
      end do


#ifdef MPI
#ifdef SCALAPACK
      ! check if this fits, otherwise we get in trouble
      ! with the block cyclic distribution. (Should be
      ! re-programmed at some point to remove this restriction)
      if (task .eq. mc_task_coeff .and. invertmethod .eq. mc_inv_pseudo) then
         ndrop = modulo(ntotsample(task,0),nmpi)

         if (ndrop .ne.0) then
            nred = 0
            lwarn1 = lwarn
            do nfile=nidxfile(task),0,-1
               ntotsample(task,nfile) = ntotsample(task,nfile) - min(ndrop, ntotsample(task,nfile))
               ndrop = ndrop -  min(ndrop, ntotsample(task,nfile))
               if  (ntotsample(task,nfile) .eq. 0) nred = nred +1
            end do

            write(message,'(a,i4,a)') "Scalapack pseudo inversion requires "// &
                 "no. coeff-sampling-points be divisible "// &
                 "by no. MPI tasks. Dropping ", ndrop, " points of the (last) trajectory."
            call warnmsg
            if (nred .gt. 0) then
               write(message,'(a,i4,a)') "The reduction of the "// &
                    "no. coeff-sampling-points lead to dropping the last " &
                    , nred, " trajectories completely."
               call warnmsg
               nidxfile(task) = nidxfile(task) - nred
            end if

            lwarn = lwarn1
         end if
      end if
#endif
#endif

   end do




   ! set the final numbers of totsamples after checking files
   do task = 1, mc_ntasks
      nfile = nidxfile(task)
      ntotsample(task,0) = sum(ntotsample(task,1:nfile))
   end do

   if (lsamecoeffspp) then
      if (taskdef .eq. mc_task_spp) then
         if (nidxfile(mc_task_coeff) .ne. 0) then
            lwarn1 = lwarn
            message = "Found 'sampling-coeff = sampling-spp' but " &
                 //"found other sampling definitions for the " &
                 //"coefficients. These will be ignored." //trim(idxfile(mc_task_coeff,1))
            call warnmsg
            lwarn = lwarn1
            nidxfile(mc_task_coeff) = 0
         end if
         ntotsample(mc_task_coeff,0) = ntotsample(mc_task_spp,0)
      elseif (taskdef .eq. mc_task_coeff) then
         if (nidxfile(mc_task_spp) .ne. 0) then
            lwarn1 = lwarn
            message = "Found 'sampling-spp = sampling-coeff' but " &
                 //"found other sampling definitions for the " &
                 //"SPP. These will be ignored."//trim(idxfile(mc_task_spp,1))
            call warnmsg
            lwarn = lwarn1
            nidxfile(mc_task_spp) = 0
         end if
         ntotsample(mc_task_spp,0) = ntotsample(mc_task_coeff,0)
      end if

   end if


!-----------------------------------------------------------------------
! open data files and copy input file to data directory
!-----------------------------------------------------------------------
   if (.not.lshort) then
      call popendat
      call pausgab
   end if

!-----------------------------------------------------------------------
! Set 'linperr=true' if a warnmsg occurred
!-----------------------------------------------------------------------
   if (.not.linperr) linperr=lwarn
   lwarn=.false.

   return

1000 continue
   ilbl=index(filename,' ')-1
   routine = 'mcpeingabe'
   message = 'Cannot open file: '//filename(1:ilbl)
   call errormsg

 end subroutine mcpeingabe

