!#######################################################################
!
!              MCPOTFIT/MCCPD  module MCPEINGUTIL
!
! MCPOTFIT module containing utility subroutines for EINGABE to open
! and close the input and  output files and to read the input data.
!
!  includes the subroutines: mcpopenin, mccpdopenini, wrmccpdhelptext
!                            mcwrphelptext, CheckStopFile, mcpzeitstop
!
!  11/97 HDM
!
!
!#######################################################################

!***********************************************************************
!
!                  Subroutine  MCPOPENIN
!
! Opens the input file  file.inp.
!
! Called from: peingabe
! Calls to:    None
!
!***********************************************************************

subroutine mcpopenin(lmnd)

  use global
  use maxv
  use griddatmod
  use operdef
  use runpfitmod
  use channels
  use logdat
  use versions, only: wrversion
  use peingutil, only: wrphelptext

  implicit none

  logical, intent(OUT) :: lmnd

  integer   :: i,length,iup,ierr
  logical   :: linpf, lexist

  character(len=8)  :: flag
  character(len=80) :: buf

!-----------------------------------------------------------------------
! set default values
!-----------------------------------------------------------------------
  routine = 'MCPOPENIN'
  linpf   = .false.
  lmnd    = .false.
  optlaenge=0
  iup     = 0

  ! set a default userpotparam
  call set_up_iparam(iup)
!-----------------------------------------------------------------------
! The input file and the options are read from the UNIX shell.
! There must be at least one argument. If there are no arguments read
! a help text is output and the program is stopped.
! If there is only one argument it is the name
! of the '~.inp' file (the name must not start with an '-').
! If there is more than one argument the last argument is taken as the
! name of the '~.inp' file. The order of the other arguments (i.e. the
! order of the options) is arbitrary.
! If the argument list does not have a correct format
! a help text is output and the program is stopped.
!-----------------------------------------------------------------------

! GETARG is a UNIX specific call:
! mygetarg calls the getarg-routine of the present UNIX system
! GET THE FIRST ARGUMENT

  i = 1
  call mygetarg(1,buf)
  if( buf(1:1) .eq. ' ' ) then
     call mcwrphelptext
  else if( buf(1:1) .eq. '-' ) then
     goto 101
  else
     inputfile = buf
     linpf = .true.
! ----------------------------------------------------------------------
!  Outcomment the following lines, if you want to allow, that the ~.inp
!  file appear as either the first or the last argument.
!  With the following 7 FORTRAN lines you are forced to have the ~.inp
!  file as the last argument. (This is UNIX convention).
! ----------------------------------------------------------------------
     call mygetarg(2,flag)
     if( flag(1:1) .eq. ' ' ) then
        goto 200
     else
        write(6,'(/,a/,a)') '***** ERROR ! *****', &
             ' The ~.inp file must be the last argument.'
        call wrphelptext
     end if
!-----------------------------------------------------------------------

  end if

!-----------------------------------------------------------------------
! LOOP OVER THE ARGUMENTS
123 continue
  i = i + 1
  call mygetarg(i,buf)

101 continue
  flag    = buf(1:8)
  if( flag(1:1) .eq. '-' ) then
     length  = index(flag//' ',' ')-1
     length  = min(length, 79-optlaenge)
     if (optlaenge .eq. 0) then
        options = flag(1:length)//' '
     else
        options = options(1:optlaenge)//flag(1:length)//' '
     endif
     optlaenge = optlaenge+length+1
  end if

  if ( flag .eq. '-h' .or.  flag .eq. '-?' ) then
     call mcwrphelptext
  else if ( flag .eq. '-ver' ) then
     call wrversion(6)
     stop
  else if ( flag(1:1) .eq. ' ' ) then
     goto 200
  else if ( flag .eq. '-w' ) then
     lovwr  = .true.
  else if ( flag .eq. '-rd' ) then
     irundvr  = 2
  else if ( flag .eq. '-mnd' ) then
     lmnd  = .true.
  else if ( flag .eq. '-up' ) then
     i = i + 1
     call mygetarg(i,buf)
     read(buf,*,iostat=ierr) iup
     if(ierr.ne.0) then
        write(6,'(/2a)') '******* ERROR reading option iup ******* ',trim(buf)
        call stopnow
     endif
     call set_up_iparam(iup)
  else if( flag .eq. '-D' ) then
     i = i + 1
     call mygetarg(i,buf)
     if (buf(1:1).eq.'-' .or.  buf(1:1).eq.' ') then
        write(6,'(a)') ' No pathname follows -D'
        call wrphelptext
     else
        name    = buf
        laenge  = index(name,' ')-1
        length  = min(laenge,79-optlaenge)
        options = options(1:optlaenge)//name(1:length)//' '
        optlaenge = optlaenge+length+1
     end if
  else if( flag(1:1) .ne. '-' ) then
     if( linpf ) then
        write(6,'(/,a/,a)') '***** ERROR ! *****', &
             ' There must be only one ~.inp file.'
        call wrphelptext
     end if
     i = i + 1
     call mygetarg(i,flag)
     if( flag(1:1) .ne. ' ' ) then
        write(6,'(/,a/,a)') '***** ERROR ! *****', &
             ' The ~.inp file must be the last argument.'
        call wrphelptext
     else
        linpf = .true.
        inputfile = buf
     end if

  else
     write(6,'(/,a/,2a)') '***** ERROR ! *****', &
          ' Unknown option: ', flag
     call wrphelptext
  end if
  goto 123


200 continue
! FILE NAMES
  laenge = index(name,' ')-1
  if( laenge .gt. 0 ) then
     call abspath(name,laenge)
  endif

  inplaenge = index(inputfile,' ')-1
  if(inputfile(max(1,inplaenge-3):inplaenge) .ne. '.inp') then
     inputfile = inputfile(1:inplaenge)//'.inp'
     inplaenge = inplaenge + 4
  endif
  call abspath(inputfile,inplaenge)

! CHECK EXISTENCE OF INPUT FILE.
  inquire(file=inputfile,exist=lexist)
  if(.not.lexist) then
     write(6,'(a)')'-----------------------------------------------'
     write(6,'(a)') ' ##############   ERROR   ##############'
     write(6,'(a)') ' There is no input-file! The file:'
     write(6,'(a)') inputfile(1:inplaenge)
     write(6,'(a)') ' does not exist!'
     write(6,'(a)')'-----------------------------------------------'
     call stopnow
  end if


! --- OPEN INPUT FILE ---

  open(iin,file=inputfile,status='old',err=10)

  return

10 continue
  write(6,*)
  write(6,'(a)') 'ERROR in subroutine popenin:'
  write(6,'(a)') 'Cannot open input file!'
  write(6,'(a)') 'inputfile: ', inputfile
  write(6,*)
  call stopnow

end subroutine mcpopenin





subroutine mccpdopenin(lmnd)

  use global
  use maxv
  use griddatmod
  use operdef
  use runpfitmod
  use channels
  use logdat
  use mccpdparams, only : luseblas
 
  use versions, only: wrversion
  use peingutil, only: wrphelptext

  implicit none

  logical, intent(OUT) :: lmnd

  integer   :: i,length,iup,ierr
  logical   :: linpf, lexist

  character(len=8)  :: flag
  character(len=80) :: buf

!-----------------------------------------------------------------------
! set default values
!-----------------------------------------------------------------------
  routine = 'MCCPDOPENIN'
  linpf   = .false.
  lmnd    = .false.
  optlaenge=0
  iup     = 0

  ! set a default userpotparam
  call set_up_iparam(iup)

!-----------------------------------------------------------------------
! The input file and the options are read from the UNIX shell.
! There must be at least one argument. If there are no arguments read
! a help text is output and the program is stopped.
! If there is only one argument it is the name
! of the '~.inp' file (the name must not start with an '-').
! If there is more than one argument the last argument is taken as the
! name of the '~.inp' file. The order of the other arguments (i.e. the
! order of the options) is arbitrary.
! If the argument list does not have a correct format
! a help text is output and the program is stopped.
!-----------------------------------------------------------------------

! GETARG is a UNIX specific call:
! mygetarg calls the getarg-routine of the present UNIX system
! GET THE FIRST ARGUMENT

  i = 1
  call mygetarg(1,buf)
  if( buf(1:1) .eq. ' ' ) then
     call wrmccpdhelptext
  else if( buf(1:1) .eq. '-' ) then
     goto 101
  else
     inputfile = buf
     linpf = .true.
! ----------------------------------------------------------------------
!  Outcomment the following lines, if you want to allow, that the ~.inp
!  file appear as either the first or the last argument.
!  With the following 7 FORTRAN lines you are forced to have the ~.inp
!  file as the last argument. (This is UNIX convention).
! ----------------------------------------------------------------------
     call mygetarg(2,flag)
     if( flag(1:1) .eq. ' ' ) then
        goto 200
     else
        write(6,'(/,a/,a)') '***** ERROR ! *****', &
             ' The ~.inp file must be the last argument.'
        call wrphelptext
     end if
!-----------------------------------------------------------------------

  end if

!-----------------------------------------------------------------------
! LOOP OVER THE ARGUMENTS
123 continue
  i = i + 1
  call mygetarg(i,buf)

101 continue
  flag    = buf(1:8)
  if( flag(1:1) .eq. '-' ) then
     length  = index(flag//' ',' ')-1
     length  = min(length, 79-optlaenge)
     if (optlaenge .eq. 0) then
        options = flag(1:length)//' '
     else
        options = options(1:optlaenge)//flag(1:length)//' '
     endif
     optlaenge = optlaenge+length+1
  end if

  if ( flag .eq. '-h' .or.  flag .eq. '-?' ) then
     call wrmccpdhelptext
  else if ( flag .eq. '-ver' ) then
     call wrversion(6)
     stop
  else if ( flag(1:1) .eq. ' ' ) then
     goto 200
  else if ( flag .eq. '-w' ) then
     lovwr  = .true.
  else if ( flag .eq. '-c' ) then
     lcont  = .true.
  else if ( flag .eq. '-rd' ) then
     irundvr  = 2
  else if ( flag .eq. '-mnd' ) then
     lmnd  = .true.
  else if ( flag .eq. '-up' ) then
     i = i + 1
     call mygetarg(i,buf)
     read(buf,*,iostat=ierr) iup
     if(ierr.ne.0) then
        write(6,'(/2a)') '******* ERROR reading option iup ******* ',trim(buf)
        call stopnow
     endif
     call set_up_iparam(iup)
  else if ( flag .eq. '-noblas' ) then
     luseblas  = .false.
  else if( flag .eq. '-D' ) then
     i = i + 1
     call mygetarg(i,buf)
     if (buf(1:1).eq.'-' .or.  buf(1:1).eq.' ') then
        write(6,'(a)') ' No pathname follows -D'
        call wrmccpdhelptext
     else
        name    = buf
        laenge  = index(name,' ')-1
        length  = min(laenge,79-optlaenge)
        options = options(1:optlaenge)//name(1:length)//' '
        optlaenge = optlaenge+length+1
     end if
  else if( flag(1:1) .ne. '-' ) then
     if( linpf ) then
        write(6,'(/,a/,a)') '***** ERROR ! *****', &
             ' There must be only one ~.inp file.'
        call wrmccpdhelptext
     end if
     i = i + 1
     call mygetarg(i,flag)
     if( flag(1:1) .ne. ' ' ) then
        write(6,'(/,a/,a)') '***** ERROR ! *****', &
             ' The ~.inp file must be the last argument.'
        call wrmccpdhelptext
     else
        linpf = .true.
        inputfile = buf
     end if

  else
     write(6,'(/,a/,2a)') '***** ERROR ! *****', &
          ' Unknown option: ', flag
     call wrmccpdhelptext
  end if
  goto 123


200 continue
! FILE NAMES
  laenge = index(name,' ')-1
  if( laenge .gt. 0 ) then
     call abspath(name,laenge)
  endif

  inplaenge = index(inputfile,' ')-1
  if(inputfile(max(1,inplaenge-3):inplaenge) .ne. '.inp') then
     inputfile = inputfile(1:inplaenge)//'.inp'
     inplaenge = inplaenge + 4
  endif
  call abspath(inputfile,inplaenge)

! CHECK EXISTENCE OF INPUT FILE.
  inquire(file=inputfile,exist=lexist)
  if(.not.lexist) then
     write(6,'(a)')'-----------------------------------------------'
     write(6,'(a)') ' ##############   ERROR   ##############'
     write(6,'(a)') ' There is no input-file! The file:'
     write(6,'(a)') inputfile(1:inplaenge)
     write(6,'(a)') ' does not exist!'
     write(6,'(a)')'-----------------------------------------------'
     call stopnow
  end if


! --- OPEN INPUT FILE ---

  open(iin,file=inputfile,status='old',err=10)

  return

10 continue
  write(6,*)
  write(6,'(a)') 'ERROR in subroutine mcpopenin:'
  write(6,'(a)') 'Cannot open input file!'
  write(6,'(a)') 'inputfile: ', inputfile
  write(6,*)
  call stopnow

end subroutine mccpdopenin



!***********************************************************************
!
!              MCCPD  subroutine WRMCCPDHELPTEXT
!
! Writes Unix-like help text to screen if arguments of the MCPOTFIT
! executable are not correct.
!
!  01/98 AJ
!***********************************************************************

subroutine wrmccpdhelptext()
  implicit none

  write(6,*)
  write(6,'(a)')  'Purpose:  Creates a cannonical polyadic fit.'
  write(6,'(2a)') '  Usage:  mccpd<vers><d> [-h|-?] ', &
       ' [-ver -rd -w -c -D name -up iup  -noblas] inpf '
  write(6,'(a)')  '   vers:  Program version (e.g. 86 for version 8.6)'
  write(6,'(a)')  '      d:  Indicates debug version'
  write(6,'(a)')  '   inpf:  Input file (with or without extension ".inp")'
  write(6,'(/a)')'  Options:'
  write(6,'(a)')  '    -h   : Print this help-text.'
  write(6,'(a)')  '    -?   : Print this help-text.'
  write(6,'(a)')  '    -c   : Continue (optimization).'
  write(6,'(a)')  '    -mnd : Make name directory.'
  write(6,'(a)')  '    -ver : Version information.'
  write(6,'(a)')  '    -rd  : Reading of the dvr-file is enforced.'
  write(6,'(a)')  '    -w   : Overwrite enabled.'
  write(6,'(a)')  ' -D name : ''name'' denotes the directory where files are'
  write(6,'(a)')  '           written to, (name in ~.inp file ignored).'
  write(6,'(a/2(11x,a/))') ' -up iup : The integer parameter iup is passed to userpot where',&
       'it may be used to select between alternative behaviours', &
       'such as returning PES or DMS components.'
  write(6,'(/a)') ' -noblas : When compiled with -P option (OpenMP parallelization)'
  write(6,'(a)')  '           this program relies heavily on being linked to an'
  write(6,'(a)')  '           optimized and OpenMP-parallel BLAS/LAPACK installation.'
  write(6,'(a)')  '           Use this flag if you are using OpenMP parallelization'
  write(6,'(a)')  '           but a serial (!) BLAS/LAPACK library is linked.'
  write(6,'(a)')  '           (The default BLAS/LAPACK library is serial.)'



  write(6,'(/a,/a,/)') &
       '   The order of the options does not matter, but "inpf"', &
       '   must be the last argument.'

  STOP 0

end subroutine wrmccpdhelptext


!***********************************************************************
!
!              MCPOTFIT  subroutine MCWRHELPTEXT
!
! Writes Unix-like help text to screen if arguments of the MCPOTFIT
! executable are not correct.
!
!  01/98 AJ
!***********************************************************************

subroutine mcwrphelptext()
  implicit none

  write(6,*)
  write(6,'(a)')  'Purpose:  Creates a natural potential fit.'
  write(6,'(2a)') '  Usage:  mcpotfit<vers><d> [-h|-?] ', &
       ' [-ver -rd -w -D name -up iup ] inpf '
  write(6,'(a)')  '   vers:  Program version (e.g. 84 for version 8.4)'
  write(6,'(a)')  '      d:  Indicates debug version'
  write(6,'(2a)') '   inpf:  Input file (with or without ', &
       'extension ".inp")'
  write(6,'(a)')  '   optn: -h   : Print this help-text.'
  write(6,'(a)')  '         -?   : Print this help-text.'
  write(6,'(a)')  '         -mnd : Make name directory.'
  write(6,'(a)')  '         -ver : Version information.'
  write(6,'(2a)') '         -rd  : Reading of the dvr-file', &
       ' is enforced.'
  write(6,'(a)')  '         -w   : Overwrite enabled.'
  write(6,'(2a,/18x,a)') &
       '         -D name: ''name'' denotes the directory where ', &
       'files are','written to, (name in ~.inp file ignored).'
 write(6,'(a/18x,a)') &
       '         -up iup: The integer parameter iup is passed to userpot where',&
       'it may be used to select between PES and DMS components.'
  write(6,'(/a,/a,/)') &
       '   The order of the options does not matter, but "inpf"', &
       '   must be the last argument.'

  STOP 0

end subroutine mcwrphelptext


subroutine CheckStopFile(lstopnow, stopmessage)

  use global
  use pglobal
  use channels
  use logdat

  implicit none

  logical, intent(OUT)                  :: lstopnow
  character(len=*), intent(IN),optional :: stopmessage
  character(len=C5)                     :: fname
  integer                               :: i, jlbl
  logical                               :: lexist

  ! pzeitstop
  integer :: nextday, oldhour
  common /pstopfile/ nextday, oldhour
  save /pstopfile/
  ! /pzeitstop

  fname = trim(name)//'/stop'
  lstopnow = .false.
  if (present(stopmessage)) then
     jlbl = len_trim(stopmessage)
     jlbl = min(C5,jlbl)
     message = stopmessage(1:jlbl)
  else
     message = " "
  end if


  inquire(file=fname, exist=lexist)
  if (.not. lexist) then
     write(ilog,'(a)') 'STOP: Cannot open stop-file'
     lstopnow = .true.
     return
  end if


  open(istop,file=fname,form='formatted', &
       status='unknown')

  read(istop,'(a)') stopcommand
  ! Go to the end of the stop-file and write the number of
  ! iterations to it in order to 'touch' the stop-file.
  do i = 1, 13
     read(istop,'(a)')
  end do
  write(istop,'(a)') trim(message)
  close(istop)

! --- If the stop-file cannot be opened stop the calculation ---

  if( stopcommand(1:4) .eq. 'cont' ) then
     nextday = 0
     oldhour = 0
  else if( stopcommand(1:4) .eq. 'stop' ) then
     lstopnow = .true.
  else if( stopcommand(3:3).eq.':' .or. &
       stopcommand(10:10).eq.':' ) then
     call mcpzeitstop(lstopnow)
  else
     routine = "CheckStopFile"
     message = "Stop command in stop-file not recognized." &
          //" Will continue. "
     call warnmsg
     lwarn = .false.
  end if


end subroutine CheckStopFile


subroutine mcpzeitstop(lstopnow)

  use global
  use pglobal
  use logdat

  implicit none

  logical, intent(OUT) :: lstopnow

  integer              :: shour, smin, ssec, stime
  integer              :: ahour, amin, atime
  integer              :: i, ierr1, ierr2, ierr3
  integer              :: nextday, oldhour
  double precision     :: fcputime
  character            :: actual*80
  logical              :: lrealtime

  external fzeit,fcputime

  ! common block to ensure that the variables are saved.
  ! This common block also appears in subroutine propagatepsi.
  common /pstopfile/ nextday, oldhour
  save /pstopfile/

  lstopnow = .false.

  if ( stopcommand(1:5) .eq. '     ' ) lrealtime = .false.

  if (lrealtime) then
     ! Realtime stop condition
     read(stopcommand(1:2),*,iostat=ierr1) shour
     read(stopcommand(4:5),*,iostat=ierr2) smin
     if( ierr1 .ne. 0  .or.  ierr2 .ne. 0 )  then
        routine = 'mcpzeitstop (mcpeingutil.f90)'
        message = 'Can''t read stopcommand. Not a proper time-descriptor'
        call warnmsg
        return
     end if
     stime = 60*shour + smin

     call fzeit(actual)

     i = index(actual,':')
     read(actual(i-2:i-1),*) ahour
     read(actual(i+1:i+2),*) amin
     if( ahour .lt. oldhour ) nextday = nextday + 24
     oldhour = ahour
     atime =  60*(ahour+nextday) + amin

     if( atime .ge. stime ) then
        lstopnow = .true.
        return
     end if
  end if

  ! CPU-time stop condition
  if( stopcommand(10:10) .ne. ':' ) return
  read(stopcommand(8:9),*,iostat=ierr1) shour
  read(stopcommand(11:12),*,iostat=ierr2) smin
  read(stopcommand(14:15),*,iostat=ierr3) ssec
  if(ierr1.ne.0 .or. ierr2.ne.0 .or. ierr3.ne.0)  then
     routine = 'pzeitstop (potfit/direct.F)'
     message = 'Can''t read stopcommand. Not a proper CPU time-descriptor'
     call warnmsg
     return
  end if

  stime = 3600*shour + 60*smin + ssec
  atime = int( fcputime(0) + 0.5d0 )
  if( atime .ge. stime )  lstopnow = .true.

  return
end subroutine mcpzeitstop


!***********************************************************************


