!   -*- f90 -*-
!***********************************************************************
!
!            MCPOTFIT subroutine MCPEINRUN
!
!
!***********************************************************************

subroutine mcpeinrun(linperr,iz)
  use mcpfitparams
  use mcsampling
  use rshdmr
  use symmops_m, only: lautoextgrid
  use pesselect_s, only: lomppotential
  use mcprecond_m
  use conjgrad_m

  use global
  use pglobal
  use maxv
  use dvrdatmod
  use griddatmod
  use operdef
  use channels
  use mpidata

  implicit none

  integer           :: i,iz,iz0,ic,ierr,imode,inum
  integer           :: ilbl, jlbl
  logical           :: linperr
  logical           :: lrddvr = .false.
  logical           :: lrundvr = .false.
  character(len=maxkeylen) :: buf
!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
  routine = 'read RUN-SECTION  (mcpeinrun)'
  iz0     = 0
! set lrddvr if already specified on command line
  if (irundvr .eq. 2) lrddvr = .true.

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
10 continue
  call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
  message = inptit
  call cntrl(keyword(1),lc(1),ierr)

  i=1
15 continue

  if (keyword(i).eq.'end-run-section') then
     goto 20
  elseif (keyword(1).eq.'end-input') then
     linperr = .true.
     message = 'No end-run-section keyword at end of RUN-SECTION'
     call warnmsg
     linperr=lwarn
     lwarn=.false.
     RETURN

  else if( keyword(i) .eq. 'title' .or. keyword(i) .eq. 'titel' )  then
     if (keyword(i+1) .ne. '=' ) then
        read(iin,'(a)') runtitle
        iz = iz + 1
     else
        i = i + 2
        ilbl=index(inptit,'title')
        if(ilbl.eq.0) ilbl=index(inptit,'titel')
        runtitle = inptit(ilbl+5:80)
        ilbl=index(runtitle,'=')
        runtitle = runtitle(ilbl+1:80)
        goto 10
     end if

  else if (keyword(i) .eq. 'readdvr') then
     lrddvr=.true.
     if (keyword(i+1) .eq. '=') then
        i=i+2
        dname=keyorig(i)
        dlaenge = lc(i)
     endif
!!$ NOT YET IMPLEMENTED
!!$  elseif (keyword(i).eq.'rdnpot') then
!!$     lrdnpot    = .true.
!!$     if (keyword(i+1) .eq. '=') then
!!$        i=i+2
!!$        lnpotpath = .true.
!!$        npotname    = keyorig(i)
!!$        npotlaenge  = lc(i)
!!$        call abspath(npotname, npotlaenge)
!!$     endif
  else if( keyword(i) .eq. 'gendvr' )  then
     lrundvr=.true.
  else if( keyword(i) .eq. 'dvronly' )  then
     ldvronly=.true.
!!$  else if (keyword(i).eq.'nofiterr') then
!!$     lfiterr=.false.
!!$  else if (keyword(i).eq.'printrho') then
!!$     lprintrho = .true.
!!$  elseif (keyword(i).eq.'rdnpot') then
!!$     lrdnpot    = .true.

  else if( keyword(i) .eq. 'auto-extgrid' )  then
     lautoextgrid = .true.

  else if (keyword(i) .eq. 'cg-precon') then
     if (keyword(i+1) .ne. '=') then
        message = 'No argument specified with cg-precon keyword'
        call warnmsg
     else
        i = i+2
        if (keyword(i) .eq. 'diag') then
           cg_precon_type = cg_precon_diag
        elseif (keyword(i) .eq. 'sparse') then
           cg_precon_type = cg_precon_sparse

           if (keyword(i+1) .ne. ",") then
              message = 'Expecting comma after cg-precon = sparse,' &
                   //"not "//trim(keyword(i+1))
              call warnmsg
           end if

           do while (keyword(i+1) .eq. ",")
              i = i+2

              ilbl = index(keyword(i),":")-1
              jlbl = ilbl+2

              read(keyword(i)(1:ilbl),*,iostat=ierr) imode
              if (ierr.ne.0) then
                 message= "Expecting integer in sparse definition, not" &
                      //trim(keyword(i)(1:ilbl))// trim(keyword(i))
                 call warnmsg
              endif
              if (imode .gt. cg_precon_maxdim ) then
                 write(message,'(a,i4,a)') "Found mode number larger then", &
                      cg_precon_maxdim, ". in sparse definition. "
                 call warnmsg
              end if
              buf = adjustr(keyword(i)(jlbl:))
              read(buf,*,iostat=ierr) inum
              if (ierr.ne.0) then
                 message= "Expecting integer in sparse definition, not" &
                      //trim(buf)//"#"
                 write(*,*)"keyword", buf
                 call warnmsg
              endif
              cg_precon_luse(imode) = inum
           end do

        else if (keyword(i) .eq. 'band') then
           cg_precon_type = cg_precon_band
           if (keyword(i+1) .ne. ",") then
              message = 'Expecing comma afer cg-precon = band.'
              call warnmsg
           else
              i = i+2
              read (keyword(i),*,iostat=ierr) cg_precon_size
              if (ierr .ne. 0) then
                 message = 'Expecing integer afer cg-precon = band, ' // &
                      'not: '//trim(keyword(i))
                 call warnmsg
              end if
           end if
        else if (keyword(i) .eq. 'block') then
           cg_precon_type = cg_precon_block
           if (keyword(i+1) .ne. ",") then
              message = 'Expecing comma afer cg-precon = block.'
              call warnmsg
           else
              i = i+2
              read (keyword(i),*,iostat=ierr) cg_precon_size
              if (ierr .ne. 0) then
                 message = 'Expecing integer afer cg-precon = block, ' // &
                      'not: '//trim(keyword(i))
                 call warnmsg
              end if
           end if

        else
           message = 'Unknown pre-conditioner type: '//trim(keyword(i))
           call warnmsg
        end if
     end if

  else if (keyword(i) .eq. 'deldvr') then
     ldeldvr=.true.
  elseif (keyword(i).eq.'overwrite') then
     lovwr = .true.
  elseif (keyword(i).eq.'name') then
     if (keyword(i+1).ne.'=') then
        message = 'No argument specified with name keyword'
        call warnmsg
     else
        if (laenge .eq. 0) then
           name = keyorig(i+2)
           laenge = lc(i+2)
           if (laenge.eq.0) then
              message = ' No path name specified'
              call wrline(zeroi,iz,iz0)
           endif
           call abspath(name,laenge)
        endif
        i=i+2
     endif
  elseif (keyword(i).eq.'output') then
     loutfil = .true.
     if (keyword(i+1).eq.'=') then
        i=i+2
        if (keyword(i).eq.'long') then
           loutl = .true.
        elseif (keyword(i).eq.'short') then
           loutl = .false.
        else
           message='"long" or "short" must follow "output="'
           call warnmsg
        endif
     endif
  else if(keyword(i) .eq. 'nooutput'  .or. &
       keyword(i) .eq. 'no-output' .or. &
       keyword(i) .eq. 'screen' )  then
     loutfil = .false.


  else if (keyword(i).eq.'timing') then
     ltime = .true.
  else if( keyword(i) .eq. 'notiming' .or. &
       keyword(i) .eq. 'no-timing' ) then
     ltime = .false.

! TODO:
!      elseif (keyword(i).eq.'prodwei') then
!         lprodw  = .true.
!      elseif (keyword(i).eq.'pestiming') then
!         if (keyword(i+1).ne.'=') then
!            message =  'missing "=" after "pestiming" '
!            call warnmsg
!         else
!            read (keyword(i+2),*,iostat=ierr) npest
!            if (ierr.ne.0) then
!               message=
!     +         'An integer number must be argument ' //
!     &              'of "pestiming". Not :'
!               call wrline(i+2,iz,iz0)
!            endif
!         endif
!         i=i+2
!  else if (keyword(i) .eq. 'readvpot') then
!     lreadvpot=.true.
!     if (keyword(i+1) .eq. '=') then
!        i=i+2
!        vname=keyorig(i)
!        vlength = lc(i)
!     endif



! MCPF SPECIFIC:
  else if (keyword(i) .eq. 'sampling-only') then
     lsamplingonly = .true.

  else if (keyword(i) .eq. 'no-omppotential') then
     lomppotential = .false.

  else if (keyword(i) .eq. 'no-omega') then
     lomega = .false.

  else if (keyword(i) .eq. 'no-omega-t') then
     lomegat = .false.
  else if (keyword(i) .eq. 'no-omega-spp') then
     lomegaspp = .false.
!     lwarn1 = lwarn
!     message = "Keyword 'no-omega-spp' is "// &
!          "and not supported any more. The grid-sampling matrix"// &
!          "is not allocated any more"
!     call warnmsg
!     lwarn = lwarn1
  else if (keyword(i) .eq. 'ascii-samplings') then
     lasciisamplings = .true.
  else if (keyword(i) .eq. 'binary-samplings') then
     lasciisamplings = .false.
  else if (keyword(i) .eq. 'cg-maxiter') then
     if (keyword(i+1).ne.'=') then
        message = 'No argument specified with cg-maxiter keyword'
        call warnmsg
     else
        read(keyword(i+2),*,iostat=ierr) cgmaxiter
        if (ierr .ne. 0) then
           message = "Expecting integer after cg-maxiter keyword, not: " &
                //trim(keyword(i+2))
           call warnmsg
        end if
        i=i+2
     endif
  else if (keyword(i) .eq. 'cg-tolerance') then
     if (keyword(i+1).ne.'=') then
        message = 'No argument specified with cg-tolerance keyword'
        call warnmsg
     else
        read(keyword(i+2),*,iostat=ierr) cgtolerance
        if (ierr .ne. 0) then
           message = "Expecting real number after cg-tolerance keyword" &
                //" not: "//trim(keyword(i+2))
           call warnmsg
        end if
        i=i+2
     endif

  else if (keyword(i) .eq. 'invert-method') then
     if (keyword(i+1).ne.'=') then
        message = 'No argument specified with invert-method keyword'
        call warnmsg
     else
        i=i+2
        if (keyword(i) .eq. "direct") then
           invertmethod = mc_inv_spd
        else if (keyword(i) .eq. "conjgrad") then
           invertmethod = mc_inv_conjgrad
        else if (keyword(i) .eq. "pseudoinvert" .or. keyword(i) .eq. "pseudo") then
           invertmethod = mc_inv_pseudo
        else if (keyword(i) .eq. "eigen") then
           invertmethod = mc_inv_eigen
           if (keyword(i+1) .eq. ',') then
              lregularization_coeff = .true.
              i = i + 2
              read(keyword(i),*,iostat=ierr) regularization_coeff
              if (ierr .ne. 0) then
                 message = "Expecting real after invert-method = eigen, ..." &
                      //" not: "//trim(keyword(i))
                 call warnmsg
              end if
           end if
!!$        else if (keyword(i) .eq. "krylov") then
!!$           invertmethod = mc_inv_krylov
!!$           if (keyword(i+1) .eq. ',') then
!!$              i = i + 2
!!$              read(keyword(i),*,iostat=ierr) krdim
!!$              if (ierr .ne. 0) then
!!$                 message = "Expecting integer after invert-method = krylov, ..." &
!!$                      //" not: "//trim(keyword(i))
!!$                 call warnmsg
!!$              end if
!!$           end if
        else
           message = "Argument of 'invert-method' must be one of" &
                //" 'direct', 'pseudo', 'eigen', or 'conjgrad', not: " //trim(keyword(i))
           call warnmsg
        end if
     end if
  else if (keyword(i) .eq. 'cg-inflate') then
     if (keyword(i+1).ne.'=') then
        message = 'No argument specified with cg-inflate keyword'
        call warnmsg
     else
        read(keyword(i+2),*,iostat=ierr) cg_inflate
        if (ierr .ne. 0) then
           message = "Expecting integer after cg-inflate keyword" &
                //" not: "//trim(keyword(i+2))
           call warnmsg
        end if
        i=i+2
     end if
!!$  else if (keyword(i) .eq. 'eps-invert') then
!!$     lregularization_coeff = .true.
!!$     if (keyword(i+1).ne.'=') then
!!$        message = 'No argument specified with eps-invert keyword'
!!$        call warnmsg
!!$     else
!!$        i=i+2
!!$        read(keyword(i),*,iostat=ierr) regularization_coeff
!!$        if (ierr .ne. 0) then
!!$           message = "Expecting read/double after eps-invert keyword" &
!!$                //" not: "//trim(keyword(i))
!!$           call warnmsg
!!$        end if
!!$     end if
  else if (keyword(i) .eq. 'iseed') then
     if (keyword(i+1).ne.'=') then
        message = 'No argument specified with iseed keyword'
        call warnmsg
     else
        read(keyword(i+2),*,iostat=ierr) mcseed
        if (ierr .ne. 0) then
           message = "Expecting integer after mcseed keyword" &
                //" not: "//trim(keyword(i+2))
           call warnmsg
        end if
        i=i+2
     endif

  else if (keyword(i) .eq. 'density') then
     if (keyword(i+1).ne.'=') then
        message = 'No argument specified with density keyword'
        call warnmsg
     else
        if (keyword(i+2) .eq. "ascii") then
           lasciidensity = .true.
        else if (keyword(i+2) .eq. "binary") then
           lasciidensity = .false.
        else
           message = "Expecting 'ASCII' or 'binary' after 'density' keyword" &
                //" not: "//trim(keyword(i+2))
           call warnmsg
        end if
        i=i+2
     endif
  else if (keyword(i) .eq. 'save-evecs') then
     lsaveevecs = .true.
  else if (keyword(i) .eq. 'sampling-energies') then
     lesample = .true.
  else if (keyword(i) .eq. 'use-rshdmr') then
     lrshdmr = .true.
     i=i+1
     if (keyword(i).eq.'=') then
        i=i+1
        if (keyword(i).eq.'rebuild') then
           lrshdmr_rebuild = .true.
        else if (keyword(i).eq.'read') then
           lrshdmr_rebuild = .false.
        else
           message = "Keyword use-rshdmr can only come with" // &
                " arguments rebuild or read. Not:"//trim(keyword(i))
        end if
     end if

  else if (keyword(i) .eq. 'subtract-pes') then
     if (keyword(i+1).ne.'=') then
        message = 'No argument specified with subtract-pes keyword'
        call warnmsg
     endif

     i=i+2
     operfile =  trim(keyword(i))//'/pes'
     lpesfile=.true.
     lsubtractpes = .true.

!!!! if compiled with MPI we use MPI

!!$  else if (keyword(i) .eq. 'usempi') then
!!$#ifdef MPI
!!$         lmpi=.true.
!!$#else
!!$         routine = 'mcpeinrun'
!!$         message = 'Not compiled with MPI support.'
!!$         call errormsg
!!$#endif

  else
     message = ' unknown keyword : '//trim(keyword(i))
     call wrline(i,iz,iz0)
  endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
  if (i.lt.ic) then
     i=i+1
     go to 15
  else
     go to 10
  endif

20 continue



!-----------------------------------------------------------------------
! don't overwrite dvr file pointed to by readdvr keyword
!-----------------------------------------------------------------------
  if (lrddvr) then
     irundvr=2
  else
     irundvr=1
  endif

!-----------------------------------------------------------------------
! force generation of DVR file
!-----------------------------------------------------------------------
  if (lrundvr) irundvr = 1

!-----------------------------------------------------------------------
! check that dname is set.
!-----------------------------------------------------------------------
  if (dlaenge .eq. 0) then
     dname=name
     dlaenge=laenge
  endif

!-----------------------------------------------------------------------
! if we are testing an existing natpot
! and lnpotpath was not given then look in
! name dir.
!-----------------------------------------------------------------------
  if (lrdnpot .and. .not. lnpotpath) then
     npotname = trim(name)//"/natpot"
     npotlaenge = len_trim(npotname)
     call abspath(npotname, npotlaenge)
  end if

!-----------------------------------------------------------------------
! check if MPI is set and Scalapack shall be used
!-----------------------------------------------------------------------

#ifdef MPI
#ifndef SCALAPACK
  if (invertmethod .eq. mc_inv_pseudo .and. nmpi.gt.1) then
     message = "invert-method = pseudoinvert requires linking"// &
          " to ScaLAPACK libraries when using MPI."
     call warnmsg
  end if
#endif
#endif

  if (.not.linperr) linperr=lwarn
  lwarn=.false.

end subroutine mcpeinrun
