!   -*- f90 -*-
!***********************************************************************
!
!            MCPOTFIT subroutine MCPEINRUN
!
!
!***********************************************************************

subroutine mcpeinsampling(linperr,iz)
  use mcpfitparams
  use mccpdparams
  use mcsampling

  use global
  use pglobal
  use maxv
  use dvrdatmod
  use griddatmod
  use operdef
  use channels
  use mpidata
  use logdat
  use keyunitsmod

  implicit none

  integer           :: i,iz,iz0,ic,ierr,k, task
  integer           :: idum(3), samplingmode, nfile
  logical           :: linperr, lsamesampling, lwarn1
  character(len=c5) :: string
  character(len=c2) :: buf
  real*8            :: rdum


!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
  routine = 'read SAMPLING-SECTION  (mcpeinsampling)'
  iz0     = 0
  taskdef = 0
  lsamesampling = .false.
  lreadidx = .false.

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
10 continue
  call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
  message = inptit
  call cntrl(keyword(1),lc(1),ierr)

  i=1
15 continue

  if (keyword(i).eq.'end-sampling-section') then
     goto 20
  elseif (keyword(1).eq.'end-input') then
     linperr = .true.
     message = 'No end-run-section keyword at end of SAMPLING-SECTION'
     call warnmsg
     linperr=lwarn
     lwarn=.false.
     RETURN


  else if (keyword(i)(1:8) .eq. 'sampling') then
     samplingmode = 0
     rdum = 0.d0
     idum= 0
     idum(3) = 1

     select case (keyword(i)(9:))
     case ("-spp")
        task = mc_task_spp
     case ("-coeff")
        task = mc_task_coeff
     case ("-test")
        task = mc_task_test
     case (" ")
        task = 0
        lsamesampling = .true.
     case default
        task = -1
     end select

     if (task.gt.0) then
        nidxfile(task) =  nidxfile(task) + 1
     else if (task.eq.0) then
        nidxfile(:) =  nidxfile(:) + 1
     else if (task .lt. 0) then
        routine = "mcpeinrun"
        message = 'sampling-[...]: unknown: sampling' &
             //trim(keyword(i)(9:))
        call warnmsg
     end if


     if (keyword(i+1) .ne. '=') then
        routine = "mcpeinsampling"
        message = "Keyword requires an argument: "//trim(keyword(i))
        call warnmsg
     else
        i=i+2


        if (keyword(i)(1:9) .eq. "sampling-") then

           if (task .eq. mc_task_spp .and. keyword(i)(9:) .eq. "-coeff") then
              lsamecoeffspp = .true.
              taskdef = mc_task_coeff
           else if (task .eq. mc_task_coeff .and. keyword(i)(9:) .eq. "-spp") then
              lsamecoeffspp = .true.
              taskdef = mc_task_spp
           else
              if (task.eq.mc_task_coeff) then
                 message = "Expecting 'sampling-coeff = sampling-spp', not " &
                      //"sampling-coeff = "//trim(keyword(i))
              else if (task.eq.mc_task_spp) then
                 message = "Expecting 'sampling-spp = sampling-coeff', not " &
                      //"sampling-spp = "//trim(keyword(i))
              else if (task.eq.mc_task_test) then
                 message = "'sampling-test = "//trim(keyword(i))//"' is not supported."
              else if (task.eq.0) then
                 message = "'sampling = sampling-XYZ' is not supported."
              end if
              call warnmsg
           end if

           if (keyword(i+1) .ne. " ") then
              message = "There must not be any further keywords after " &
                   //"'sampling-coeff = sampling-spp' or " &
                   //"'sampling-spp = sampling-coeff'."
              call warnmsg
           end if


        elseif (keyword(i)(1:7) .eq. 'readidx') then
           samplingmode = mc_readfile

           if (keyword(i+1) .ne. '{') then
              message = "Missing '{' after readidx."
              call warnmsg
           end if
           if (keyword(i+3) .ne. '}') then
              message = "Missing '}' after readidx."
              call warnmsg
           end if

           !file name:
           string = keyorig(i+2)

           i=i+3

           ! check optional nsample and nskipfirst, nstep
           do k=1,3
              if (keyword(i+1) .eq. ",") then
                 i=i+2
                 read(keyword(i),*,iostat=ierr) idum(k)
                 if (ierr.ne.0) then
                    message = "Expecting integer after ',' for 'readidx', " &
                         // "not: "//trim(keyword(i))
                    call warnmsg
                 end if
              end if
           end do


        elseif (keyword(i)(1:10) .eq. 'metropolis') then
           samplingmode = mc_metropolis

           ! check nsample, nequilibium and nstep
           do k=1,3
              if (keyword(i+1) .eq. ",") then
                 i=i+2
                 read(keyword(i),*,iostat=ierr) idum(k)

                 if (ierr.ne.0) then
                    message = "Expecting three comma separated integer values " &
                         // "after 'metropolis', not: "//trim(keyword(i))
                    call warnmsg
                 end if
              else
                 message = "Expecting three comma separated integers and one real " &
                      // "after 'metropolis', not: "//trim(keyword(i))
                 call warnmsg
              end if
           end do

           ! check beta
           if (keyword(i+1) .eq. ",") then
              read (keyword(i+2),*,iostat=ierr) rdum
              if (ierr.ne.0) then
                 message = "Expecting three comma separated integers and one real" &
                      // " after 'metropolis', not: "//trim(keyword(i))
              call warnmsg
              end if
           else
              message = "Expecting three comma separated integers and one real" &
                   // "after 'metropolis', not: "//trim(keyword(i))
              call warnmsg
           end if
           i=i+2

           ! check unit of 1/beta
           if (keyword(i+1) .eq. ",") then
              i = i+2
              call keyeunits(rdum,keyword(i),ierr)
              if (ierr .eq. 1) then
                 message = 'No energy unit after comma (metropolis).'
                 call wrline(zeroi,iz,iz0)
              endif
           end if


        elseif (keyword(i)(1:7) .eq. 'uniform') then
           samplingmode = mc_uniform

           ! check nsample
           if (keyword(i+1) .eq. ",") then
              read (keyword(i+2),*,iostat=ierr) idum(1)
              if (ierr.ne.0) then
                 message = "Expecting comma separated integer " &
                      // "after 'uniform', not: "//trim(keyword(i+2))
                 call warnmsg
              end if
           else
              message = "Expecting comma separated integer " &
                   // "after 'uniform'."
              call warnmsg
           end if
           i = i+2

        elseif (keyword(i)(1:8) .eq. 'complete') then
           samplingmode = mc_complete

        else
           message = "sampling[-...]: unknown method. " &
                // "expecting one of 'readidx', 'metropolis' or 'uniform', " &
                // "not: "//trim(keyword(i))
           call warnmsg
        end if



     ! assignment

        if (lsamesampling) then ! all

           nfile = nidxfile(mc_task_spp)
           if (nfile .gt. max_idxfile ) then
              write(message,'(a,i4)') "too many assignmentd for sampling-spp. Max:", &
                   max_idxfile
              call warnmsg
           else
              smethod(mc_task_spp,nfile) = samplingmode
           end if

           nfile = nidxfile(mc_task_coeff)
           if (nfile .gt. max_idxfile ) then
              write(message,'(a,i4)') "too many assignmentd for sampling-coeff. Max:", &
                   max_idxfile
              call warnmsg
           else
              smethod(mc_task_coeff,nfile) = samplingmode
           end if
           nfile = nidxfile(mc_task_test)
           if (nfile .gt. max_idxfile ) then
              write(message,'(a,i4)') "too many assignmentd for sampling-test. Max:", &
                   max_idxfile
              call warnmsg
           else
              smethod(mc_task_test,nfile) = samplingmode
           end if


           if (samplingmode .eq. mc_readfile)then
              do task=1,mc_ntasks
                 nfile = nidxfile(task)
                 idxfile(task,nfile) = string
                 ntotsample(task,nfile) = idum(1)
                 nskipfirst(task,nfile) = idum(2)
                 nstep(task,nfile) = idum(3)
                 beta(task,nfile) = rdum
              end do

           else
              task = mc_task_spp
              nfile = nidxfile(task)
              idxfile(task,nfile)  = trim(name)//'/dvrindex-spp'
              ntotsample(task,nfile) = idum(1)
              nskipfirst(task,nfile) = idum(2)
              nstep(task,nfile) = idum(3)
              beta(task,nfile) = rdum

              task = mc_task_coeff
              nfile = nidxfile(task)
              idxfile(task,nfile)  = trim(name)//'/dvrindex-coeff'
              ntotsample(task,nfile) = idum(1)
              nskipfirst(task,nfile) = idum(2)
              nstep(task,nfile) = idum(3)
              beta(task,nfile) = rdum

              task = mc_task_test
              nfile = nidxfile(task)
              idxfile(task,nfile) = trim(name)//'/dvrindex-test'
              ntotsample(task,nfile) = idum(1)
              nskipfirst(task,nfile) = idum(2)
              nstep(task,nfile) = idum(3)
              beta(task,nfile) = rdum

           end if
           lsamesampling = .false.

        else ! not lsamesampling
           nfile = nidxfile(task)

           if (lsamecoeffspp) then

              if (taskdef .eq. task) then

                 if (nfile .gt. max_idxfile ) then
                    write(message,'(a,i4)') "too many assignmentd for sampling-spp. Max:", &
                         max_idxfile
                    call warnmsg
                 else
                    smethod(task,nfile) = samplingmode
                    if (samplingmode .eq. mc_readfile) then
                       idxfile(task,nfile) = string
                    else
                       if (task .eq. mc_task_spp) then
                          idxfile(task,nfile) = trim(name)//'/dvrindex-spp'
                       else
                          idxfile(task,nfile) = trim(name)//'/dvrindex-coeff'
                       end if
                    end if
                    ntotsample(task,nfile) = idum(1)
                    nskipfirst(task,nfile) = idum(2)
                    nstep(task,nfile) = idum(3)
                    beta(task,nfile) = rdum
                 end if
              else if (task .eq. mc_task_spp .or. task .eq. mc_task_coeff) then
                 nidxfile(task) = nidxfile(task) -1
              else if (task .eq. mc_task_test) then
                 if (nfile .gt. max_idxfile ) then
                    write(message,'(a,i4)') "too many assignmentd for sampling-spp. Max:", &
                         max_idxfile
                    call warnmsg
                 else
                    smethod(task,nfile) = samplingmode
                    if (samplingmode .eq. mc_readfile) then
                       idxfile(task,nfile) = string
                    else
                       idxfile(task,nfile) = trim(name)//'/dvrindex-test'
                    end if
                    ntotsample(task,nfile) = idum(1)
                    nskipfirst(task,nfile) = idum(2)
                    nstep(task,nfile) = idum(3)
                    beta(task,nfile) = rdum
                 end if
              end if
           else

              if (nfile .gt. max_idxfile ) then
                 select case(task)
                 case (mc_task_spp)
                    write(message,'(a,i4)') "too many assignmentd for sampling-spp. Max:", &
                         max_idxfile
                 case (mc_task_coeff)
                    write(message,'(a,i4)') "too many assignmentd for sampling-coeff. Max:", &
                         max_idxfile
                 case (mc_task_test)
                    write(message,'(a,i4)') "too many assignmentd for sampling-test. Max:", &
                         max_idxfile
                 end select
                 call warnmsg
              else
                 smethod(task,nfile) = samplingmode
                 if (samplingmode .eq. mc_readfile)then
                    idxfile(task,nfile) = string
                 else
                    select case (task)
                    case (mc_task_spp)
                       idxfile(task,nfile) = trim(name)//'/dvrindex-spp'
                    case (mc_task_coeff)
                       idxfile(task,nfile) = trim(name)//'/dvrindex-coeff'
                    case (mc_task_test)
                       idxfile(task,nfile) = trim(name)//'/dvrindex-test'
                    end select
                 end if
                 ntotsample(task,nfile) = idum(1)
                 nskipfirst(task,nfile) = idum(2)
                 nstep(task,nfile) = idum(3)
                 beta(task,nfile) = rdum
              end if !nfile

           end if !lsamecoefspp
        end if!lsamesampling
     end if !sampling = ...

  else if (keyword(i) .eq. 'same-sets') then
     message = "The keyword  'same-sets' is not supported any more. " &
          //"You could use 'sampling-coeff = sampling-spp' or " &
          //"'sampling-spp = sampling-coeff'. This will use the same sets " &
          //"for both, the SPP and the Coefficients."
     call warnmsg
  else
     message = ' unknown keyword : '//trim(keyword(i))
     call wrline(i,iz,iz0)
  endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
  if (i.lt.ic) then
     i=i+1
     go to 15
  else
     go to 10
  endif

20 continue


!-----------------------------------------------------------------------
! if we are testing an existing natpot
! and lnpotpath was not given then look in
! name dir. (not implemented yet)
!-----------------------------------------------------------------------
!!$  if (lrdnpot .and. .not. lnpotpath) then
!!$     npotname = trim(name)//"/natpot"
!!$     npotlaenge = len_trim(npotname)
!!$     call abspath(npotname, npotlaenge)
!!$  end if


!-----------------------------------------------------------------------
! check samplings
!-----------------------------------------------------------------------


  if (lmcpf) then
     if (nidxfile(mc_task_spp) .eq.  0 .and. .not. lsamecoeffspp) then
        ! no samplings for SPP ==> we try to load densities / cuts
        lloadevecs = .true.
     end if
  end if

  if (nidxfile(mc_task_test) .eq. 0) then
     ! no samplings for testing ==> we skip the testing
     lskiptest = .true.
  end if

  if (nidxfile(mc_task_coeff)  .eq. 0) then
     ! nidxfile(mc_task_coeff) can be zero if we use the SPP samplings
     if (.not. lsamecoeffspp .and. nidxfile(mc_task_spp) .eq.  0) then
        message = "Coefficients: 'sampling-coeff'" &
             // " not specified."
        call warnmsg
     end if
  end if

  ! define both as complete
  if (lsamecoeffspp) then
     if (smethod(mc_task_coeff,1) .eq. mc_complete) then
        smethod(mc_task_spp,1) = mc_complete
     elseif (smethod(mc_task_spp,1) .eq. mc_complete) then
        smethod(mc_task_coeff,1) = mc_complete
     end if
  end if

  do task = 1, mc_ntasks
     nfile = nidxfile(task)

     ! Check if complete is mixed with other methods
     if (nfile .gt. 1 .and. any(smethod(task,1:nfile) .eq. mc_complete)) then
        message = "Complete sampling cannot be combined with any other " &
             //"sampling methods for the same task (spp, coeff, or test)"
        call warnmsg
     end if

     ! Check if complete is mixed with other methods
     if (lmccpd .and. any(smethod(task,1:nfile) .eq. mc_complete)) then
        message = "Complete sampling is not supported in mccpd runs. " &
             //"Use one of 'Metropolis', 'Uniform' or 'Readidx'. "
        call warnmsg
     end if


  end do




  do task = 1, mc_ntasks
     do nfile = 1,nidxfile(task)
        if (smethod(task,nfile) .ne. mc_readfile .and. &
             smethod(task,nfile) .ne. mc_complete .and. &
             nidxfile(task) .gt. 1) then

           write(buf,'(a,i4)') "-", nfile
           call rmblnk(buf)
           buf = adjustl(buf)

           idxfile(task,nfile) = trim(idxfile(task,nfile))//trim(buf)

        end if
     end do
  end do

  if (.not.linperr) linperr=lwarn
  lwarn=.false.

end subroutine mcpeinsampling
