!   -*- f90 -*-
!***********************************************************************
!
!            MCPOTFIT subroutine MCPEINSYM
!
!
!***********************************************************************



subroutine mcpeinsymmeties(linperr,iz)
  use mcpfitparams
  use mcpeinsym
  use symmops_m, only: lsymmetries

  use global
  use pglobal
  use maxv
  use griddatmod
  use operdef
  use channels
  use mpidata
  use logdat

  implicit none

  integer           :: i,iz,iz0,ic,ierr,j,j1,j2,k,f, slen, nbrac
  logical           :: linperr, lstart1, larithm, lnumber
  character(len=c1) :: buf
  character(len=c3) :: expression
  double precision  :: rshift


!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------

  iz0     = 0

  ! we are using symmetries
  lsymmetries = .true.

  ! we may have periodic coordinates
  allocate(qperiodic(ndof))
  qperiodic = ""


!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
  larithm = .false.
  rshift = 0.d0
  lstart1 = .true.
  k=1


10 continue
  call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)

  message = inptit
  call cntrl(keyword(1),lc(1),ierr)

  slen = len_trim(inptit)
  nbrac = 0
  do i=1,slen
     if (inptit(i:i) .eq. "{") nbrac = nbrac + 1
     if (inptit(i:i) .eq. "}") nbrac = nbrac - 1
  end do
  if (nbrac .ne. 0) then
     routine = 'read SYMMETRY-SECTION  (mcpeinsym)'
     write(message, '(a,i4)') "Unbalanced curly brackets in Line: ", iz
     call warnmsg
  end if

  nbrac = 0
  do i=1,slen
     if (inptit(i:i) .eq. "[") nbrac = nbrac + 1
     if (inptit(i:i) .eq. "]") nbrac = nbrac - 1
  end do
  if (nbrac .ne. 0) then
     routine = 'read SYMMETRY-SECTION  (mcpeinsym)'
     write(message, '(a,i4)') "Unbalanced round brackets in Line: ", iz
     call warnmsg
  end if



  do i=1,ic
     if (keyword(i).eq.'end-symmetry-section') then
        goto 20
     end if
  end do
  do i=1,ic
     if (keyword(1).eq.'end-input') then
        linperr = .true.
        routine = 'read SYMMETRY-SECTION  (mcpeinsym)'
        message = 'No end-symmetry-section keyword at end of SYMMETRY-SECTION'
        call warnmsg
        linperr=lwarn
        lwarn=.false.
        RETURN
     end if
  end do

  if (keyword(1) .eq. "periodic") then
     if (keyword(2) .ne. "=") then
        routine = 'read SYMMETRY-SECTION  (mcpeinsym)'
        message = "Expecting '=' after keyword 'periodic', not: "//trim(keyword(2))
        call warnmsg
     end if
     if (ic.lt.3) then
        routine = 'read SYMMETRY-SECTION  (mcpeinsym)'
        message = "Expecting DOF list after 'periodic = '."
        call warnmsg
     end if
     j=3
     f=0
     lnumber = .false.
     do while(j .le. ic)


        if(keyorig(j) .eq.  "," .and. j.gt.3) then
           j = j+1
           cycle
        elseif(keyorig(j) .eq.  "," .and. j.eq.3) then

           routine = 'read SYMMETRY-SECTION  (mcpeinsym)'
           message = "Expecting DOF label after 'periodic =', not: "//trim(keyorig(j))
           call warnmsg
        else
           f = f+1
           if (f.gt.ndof) then
              routine = 'read SYMMETRY-SECTION  (mcpeinsym)'
              message = "Too many keywords for 'periodic = ...'"
              call warnmsg
              f = ndof
           end if


           qperiodic(f) = trim(keyorig(j))
           j = j+1

        end if

     end do
     nperiodic = f
     goto 10
  end if

  if (lstart1) then
     lstart1 = .false.
     nsymops = ic

     allocate(symtable(0:ndof,nsymops))
     ! label goes into element (0,n)

     allocate(symrshift(ndof,nsymops))
     symrshift = 0.d0

     allocate(lsymarithmetic(ndof,nsymops))
     lsymarithmetic = .false.

     do j=1, nsymops
        symtable(0,j) = trim(keyorig(j))
     end do
     do j=1,nsymops
        do i=1,ndof
           symtable(i,j) = "***"
        end do
     end do
     k=1
     goto 10
  else
     if (k.gt.ndof) then
        routine = 'read SYMMETRY-SECTION  (mcpeinsym)'
        message = "Too many lines in symmetry table"
        call warnmsg()
        goto 20
     end if

     j = 1
     j1 = 1
     do i=1,ic

        if (j1 .gt. nsymops .and. keyorig(j) .ne. " ") then

           routine = 'read SYMMETRY-SECTION  (mcpeinsym)'
           write(message,'(a,2i4)') "Too many keywords in symmetry table. Missing curly brackets? " &
                //"Line: ", iz
           call warnmsg
           k = k+1
           stop
           goto 10

        end if

        if ( keyorig(j)(1:1) .eq. "-") then
           ! grid based
           call Chop1stChar(keyorig(j),'-',j)

           if (j.gt.ic) then
              routine = 'read SYMMETRY-SECTION  (mcpeinsym)'
              write(message,'(a,i4)') "Line must not end with '-'. Line: ", iz
              call warnmsg()
           end if

           buf = "-"//trim(keyorig(j))
        elseif ( keyorig(j)(1:1) .eq. "+") then
           call Chop1stChar(keyorig(j),'+',j)

           if (j.gt.ic) then
              routine = 'read SYMMETRY-SECTION  (mcpeinsym)'
              write(message,'(a,i4)') "Line must not end with '-'. Line: ", iz
              call warnmsg()
           end if

           buf = trim(keyorig(j))
        else if ( keyorig(j)(1:1) .eq. "{") then
           ! coordinate based

           if (j1 .eq. 1) then
              routine = 'read SYMMETRY-SECTION  (mcpeinsym)'
              write(message,'(a,i4)') "There must not be curly prackets within the identity, Line: ", iz
              call warnmsg()
              j=j+1
           else

              expression = ""
              do j2=j+1,ic
                 if (keyorig(j2) .eq. "}") exit ! loop
                 expression = trim(expression)//trim(keyorig(j2))
              end do

              call evaluateSymExpression(expression, buf, rshift, iz)
              larithm = .true.

              ! skip the closing }
              j = j2

           end if
        else
           ! grid based - single label
           buf = trim(keyorig(j))
        end if
        j = j+1

        if (j1.le.nsymops) then
           lsymarithmetic(k,j1) = larithm
           symrshift(k,j1) = rshift
           symtable(k,j1) = buf
           j1 = j1 + 1
        end if

        larithm = .false.
        rshift = 0.d0
     end do

     k = k+1
     goto 10
  endif

20 continue


  ! consistency checks

  if (k.ne.ndof+1) then
     write(message, '(a,i4,a,i4,a)') "Only found ", k-1, " of", &
          ndof," DOF assignments in symmetry table."
     call warnmsg
  end if

  do j=1,nsymops
     do k=0,ndof
        if (symtable(k,j)(1:3) .eq. "***") then
           write(message, '(a,i4,a,i4,a)') &
                "Undefined label in sym-table, elements (",k,',',j,")"
           call warnmsg()
        end if
     end do
  end do

  if (symtable(0,1) .ne. "E") then
     message = "The first column of the symmetry table must " &
          //"be the identity operation labeled with 'E'"
     call warnmsg
  end if

  do k=1,ndof
     if (symtable(k,1)(1:1) .eq. "-" ) then
        message = "The first column of the symmetry table must " &
             //"be the identity operation. Grids cannot be transfomed with '-'."
        call warnmsg
     end if

     if (symtable(k,1) .eq. "usersym" ) then
        message = "The first column of the symmetry table must " &
             //"be the identity operation. 'usersym' is not allowed."
        call warnmsg
     end if

  end do

  if (.not.linperr) linperr=lwarn
  lwarn=.false.

!!$  write(*,*) "BUFFER:"
!!$  do i=0,ndof
!!$     write(*,*) (symtable(i,j), j=1,nsymops)
!!$  end do
!!$  write(*,*)
!!$
!!$  write(*,*) "ARITM:"
!!$  do i=1,ndof
!!$     write(*,*) (lsymarithmetic(i,j), j=1,nsymops)
!!$  end do
!!$  write(*,*)
!!$
!!$
!!$  write(*,*) "SHIFT:"
!!$  do i=1,ndof
!!$     write(*,*) (symrshift(i,j), j=1,nsymops)
!!$  end do
!!$  write(*,*)
!!$



end subroutine mcpeinsymmeties


! deallocate module arrays
subroutine clean_mcpeinsym()
  use mcpeinsym

  if (allocated(symtable)) deallocate(symtable)
  if (allocated(symrshift)) deallocate(symrshift)
  if (allocated(lsymarithmetic)) deallocate(lsymarithmetic)
  if (allocated(qperiodic)) deallocate(qperiodic)
  nsymops = 0

end subroutine clean_mcpeinsym




subroutine Chop1stChar(buf,char,j)
  implicit none
  character(len=16), intent(INOUT) :: buf
  character(len=*),  intent(IN)    :: char
  integer, intent(INOUT)           :: j

  if (buf .eq. char) then
     j=j+1
  else
     buf(1:1) = ""
     buf = adjustl(buf)
  end if

end subroutine Chop1stChar


subroutine evaluateSymExpression(symexpr, coordstr, rshift, iz)

  use global
  use logdat

  implicit none
  character(len=*), intent(IN)   :: symexpr
  character(len=16), intent(OUT) :: coordstr
  double precision, intent(OUT)  :: rshift
  integer, intent(IN)            :: iz
  character(len=LEN(symexpr))    :: str
  character, parameter           :: pm(2)= (/'+','-'/)
  character, parameter           :: pmmd(4)= (/'+','-','*','/'/)
  character(len=16)              :: what
  character(len=1)               :: coordsign
  integer                        :: i,j, idx, nwhat, slen

  rshift = 0.d0
  slen = index(symexpr," ")
  if (slen .eq. 1) then
     routine = 'read SYMMETRY-SECTION  (evaluateSymExpression)'
     write(message,'(a,i4)') "Found empty string or blanks in curly brackets, line: ", iz
     call warnmsg
  end if

  str=symexpr

  do i=1,4
     do j=1,4
        what = pmmd(i)//pmmd(j)
        if (index(str,trim(what)) .gt. 0) then
           routine = 'read SYMMETRY-SECTION  (evaluateSymExpression)'
           write(message,'(a,i4)') "Consecutive "//trim(what)//" found in line ", iz
           call warnmsg
        end if
     end do
  end do

  call FindInList(str,pm,2,what,nwhat,idx)
  i = index(str,'*')
  j = index(str,'/')
  if ( (i .gt. 0 .and. i .lt. idx) .or. (j .gt. 0 .and. j .lt. idx)) then
     routine = 'read SYMMETRY-SECTION  (evaluateSymExpression)'
     write(message,'(a,i4)') "Coordinate scaling is not supported, Line: ", iz
     call warnmsg
  end if

  if (idx .eq. 0) then
     ! there should only be a coordinate label
     ! and we are done
     coordstr = trim(str)
     return

  elseif (idx .eq. 1) then
     ! the coordinate bears a sign
     if (nwhat .eq. 1) then
        ! plus is implicit before coordinate label
        coordsign = ""
     else if (nwhat .eq. 2) then
        coordsign = "-"
     end if
     str(1:1) = " "
     str = adjustl(str)
     call FindInList(str,pm,2,what,nwhat,idx)
     if (idx .eq. 0) then
        coordstr = trim(coordsign)//trim(str)
        return
     else
        coordstr = trim(coordsign)//trim(str(1:idx-1))
        str(1:idx-1) = ""
        str = adjustl(str)
     end if

  else if (idx .gt. 1) then
     ! the coordinate bears no sign
     coordstr = trim(str(1:idx-1))
     str(1:idx-1) = ""
     str = adjustl(str)

  end if

  if (len_trim(str) .ne. 0) then
     call evaluateRealExpression(str, rshift, iz)
  end if


end subroutine evaluateSymExpression


subroutine evaluateRealExpression(string, rval, iz)

  use global
  use logdat

  implicit none
  character(len=*), intent(IN)   :: string
  double precision, intent(OUT)  :: rval
  integer, intent(IN)            :: iz
  character, parameter           :: md(2)= (/'*','/'/)
  character, parameter           :: pm(2)= (/'+','-'/)

  character(len=2)               :: what
  integer                        :: slen, faclen, termlen
  character(len=LEN(string))     :: str1
  integer                        :: nwhat, ierr
  double precision               :: rfac, rfac1
  logical                        :: ltimes

  str1 = trim(adjustl(string))
  slen = len_trim(str1)
  rfac = 1.d0
  rval = 0.d0

222 continue
  if (str1(1:1) .eq. "+") then
     rfac = 1.d0
  elseif (str1(1:1) .eq. "-") then
     rfac = -1.d0
  else if(str1(1:1) .eq. " ") then
     goto 224
  end if
  str1(1:1) = " "
  str1 = trim(adjustl(str1))
  call FindInList(str1,pm,2,what,nwhat,termlen)
  ! no more summands
  if (termlen .eq. 0) then
     termlen = len_trim(str1)
  else
     termlen = termlen -1
  end if

  ltimes = .true.

223 continue


  call FindInList(str1(1:termlen),md,2,what,nwhat,faclen)
  ! only more factor
  if (faclen .eq. 0) then
     faclen = len_trim(str1(1:termlen))
  else
     faclen =  faclen - 1
  end if

  if (str1(1:faclen) .eq. 'pi') then
     rfac1 = atan(1.d0)*4.d0
  elseif (str1(1:faclen) .eq. '2pi') then
     rfac1 = atan(1.d0)*8.d0
  else
     read(str1(1:faclen),*,iostat=ierr) rfac1
     if (ierr .ne. 0) then
        write(message,'(a,i3)') "Error while reading float from expression: " &
             //trim(str1(1:faclen))//", Line: ", iz
        routine = "evaluateRealExpression (reading SYMMETRY-SECTION)"
        call warnmsg
     end if
  end if

  if (ltimes) then
     rfac = rfac * rfac1
  else
     rfac = rfac / rfac1
  end if

  str1(1:faclen) = " "
  str1 = trim(adjustl(str1))
  termlen = termlen - faclen
  if (str1(1:1) .eq. '*') then
     ltimes = .true.
     str1(1:1) = " "
     str1 = trim(adjustl(str1))
     termlen = termlen - 1
     goto 223
  else if (str1(1:1) .eq. '/') then
     ltimes = .false.
     str1(1:1) = " "
     str1 = trim(adjustl(str1))
     termlen = termlen - 1
     goto 223
  else if  (str1(1:1) .eq. '+' .or. str1(1:1) .eq. '-' .or. str1(1:1) .eq. ' ') then
     rval = rval + rfac
     goto 222
  else
     write(message,'(a,i3)') "Error while reading expression, infalid operand: " &
          //trim(str1(1:1))//", Line: ", iz
     routine = "evaluateRealExpression (reading SYMMETRY-SECTION)"
     call warnmsg
  end if

224 continue
  return


end subroutine evaluateRealExpression
