

module mcpfitparams
!  use global
  use pglobal
  use dvrtype_m
!  use rshdmr
!  use mcsampling
!  use mcprecond_m
!  use conjgrad_m

  implicit none

  ! invertmethod: how to invert the overlap matrix
  ! 1: direct invert using LAPACK DPOSV (default)
  ! 2: Conjugate gradients method

  ! cgtolerance: precision at which gonjugate gradients
  !              is considered converged
  ! cgmaxiter:   Maximum number iterations of the
  !              gonjugate gradients routine if
  !              cgtolerance is not achieved

  ! lomega:      allocate omega. if false,
  !              calculate on the fly when needed
  ! lomegaT:     allocate transpose(omega). if false,
  !              calculate on the fly when needed
  !              (presently only with gonjugate gradients)



  integer                    :: taskdef

  character(len=240)         :: pespath

  double precision           :: regularization_coeff
  logical                    :: lregularization_coeff

  logical                    :: lloadevecs, lskiptest
  logical                    :: lsamplingonly

  logical                    :: lrshdmr_rebuild

  logical                    :: lomega, lomegat, lomegaspp
  logical                    :: lsubtractpes

  integer                    :: invertmethod

  integer                    :: mcseed

  logical                    :: lasciidensity
  logical                    :: lsaveevecs
  logical                    :: lsppcheck

contains

  subroutine mcpdefault()

    implicit none

    integer*8 :: time, n8
    integer   :: n


    lomega = .true.
    lomegat = .true.
    lomegaspp = .true.
    lloadevecs = .false.
    lskiptest = .false.
    lasciidensity = .true.

    lsaveevecs = .false.
    lrshdmr_rebuild = .false.


    regularization_coeff = 0.d0
    lregularization_coeff = .false.


    lsamplingonly = .false.
    taskdef = -1
    lsppcheck = .true.

    invertmethod = mc_inv_spd


    lsubtractpes = .false.

    ! random seed from system time
    ! (seconds since Jan 1 1970 00:00:00)
    n8 = time()
    n = int(modulo(n8,2147483647))
    mcseed = n



  end subroutine mcpdefault
end module mcpfitparams




