!   -*- f90 -*-


module mcsamplingio

    use global
    use channels
    use mpidata
    use logdat

contains

!!$
!!$  subroutine saveDVRindex(dvrindex,ndof,nsample,fname,loverwrite)
!!$    use mcsampling, only: lasciisamplings
!!$    implicit none
!!$    integer, intent(IN)            :: ndof,nsample
!!$    character(len=*), intent(IN)   :: fname
!!$    integer, intent(IN)            :: dvrindex(ndof,nsample)
!!$    logical, intent(IN), optional  :: loverwrite
!!$    integer                        :: n,f
!!$    logical                        :: lovwr1, lexist
!!$
!!$    include "channels.inc"
!!$    include "global.inc"
!!$    include "mpi.inc"
!!$
!!$    if (mpirank .ne. 0) then
!!$       return
!!$    end if
!!$
!!$    lovwr1 = .false.
!!$    if (present(loverwrite)) lovwr1 = loverwrite
!!$
!!$    inquire(file=trim(fname), exist=lexist)
!!$
!!$    if (lexist .and. .not. lovwr1) then
!!$       routine = "saveDVRindex"
!!$       message = "Cannot overwrite "//trim(fname)//". Use -w option."
!!$       call errormsg
!!$    end if
!!$
!!$    if (lasciisamplings) then
!!$
!!$       open(idump,file=trim(fname), status='unknown')
!!$       do n=1,nsample
!!$          write(idump,'(1000i4)') (dvrindex(f,n),f=1,ndof)
!!$       end do
!!$       close(idump)
!!$
!!$    else
!!$
!!$       open(idump,file=trim(fname), form='unformatted', status='unknown')
!!$       do n=1,nsample
!!$          write(idump) (dvrindex(f,n),f=1,ndof)
!!$       end do
!!$       close(idump)
!!$
!!$    end if
!!$
!!$
!!$  end subroutine saveDVRindex





  subroutine loadDVRindex(dvrindex,ndof,gdim,nsample,npointfromfile,nskipfirst, &
       nstep,fname,nfiles)
    use mcpmpitools
    use mpidata
#ifdef MPI
    use mpi
#endif
    implicit none
    integer, intent(IN)            :: ndof,nsample,nfiles
    integer, intent(IN)            :: npointfromfile(nfiles),&
                                      nskipfirst(nfiles),nstep(nfiles)
    integer, intent(IN)            :: gdim(ndof)
    character(len=*), intent(IN)   :: fname(nfiles)
    integer, intent(OUT)           :: dvrindex(ndof,nsample)
    integer                        :: n,n1,f, ierr, nline, nchunk, nf,&
                                      n2, nhash,np
    character(len=c3)              :: line, sidx
    logical                        :: lbinary
    integer, allocatable           :: tmpidx(:,:)


    ! this could be made #ifdef DEBUG in the future
    if (any(npointfromfile .lt.0)) then
       routine = "loadDVRindex"
       message = "Internal: points on file is negative. Please use 'mreport' to send a bug report."
       call errormsg
    end if
    if (any(nskipfirst .lt.0)) then
       routine = "loadDVRindex"
       message = "Internal: skipped points on file is negative. Please use 'mreport' to send a bug report."
       call errormsg
    end if
    if (any(nskipfirst .lt.0)) then
       routine = "loadDVRindex"
       message = "Internal: point step on file is negative. Please  use 'mreport' send a bug report."
       call errormsg
    end if



    ierr = 0      ! error
    dvrindex = 0  ! sampling points
    nf = 1        ! counter of files
    np =0         ! counter of points from a file


    if (mpirank .eq. 0) then
       call StartNewFile(idump, fname(nf),nskipfirst(nf),nhash,lbinary)
       nline = nskipfirst(nf)
    end if ! rank 0
    n2 = 0

    chunk: do nchunk=0,nmpi-1

       allocate(tmpidx(ndof,mpi_chunksizes(nchunk)))

       rank0: if (mpirank .eq. 0) then

          chunkloop: do n=1,mpi_chunksizes(nchunk)

             if (np .ge. npointfromfile(nf)) then
                nf = nf + 1
                if (nf .gt. nfiles .and. n2 .lt. sum(mpi_chunksizes)) goto 997
                call StartNewFile(idump,fname(nf),nskipfirst(nf),nhash, lbinary)
                nline = nskipfirst(nf) + nhash
                np = 0 ! reset counter for this file
             end if

             nline = nline + 1
             if (lbinary) then
                read(idump,iostat=ierr) (tmpidx(f,n),f=1,ndof)
             else
                read(idump,*,iostat=ierr) (tmpidx(f,n),f=1,ndof)
             end if
             if (ierr .lt. 0) then !end of file
                goto 997
             else if (ierr .gt. 0) then
                goto 998
             else
                ! we got a line
                n2 = n2 + 1
                ! increase counter for corrent file
                np = np +1
             end if


             ! check the index we just read
             check: do f=1,ndof
                if (tmpidx(f,n) .le. 0) then

                   write(line,'(i8)') nline
                   line = adjustl(line)
                   write(sidx,'(i8)') f
                   sidx = adjustl(sidx)

                   routine = "loadDVRindex"
                   message = "DVR-index in file "//trim(fname(nf)) &
                        // "is smaller then zero " &
                        // "(line "//trim(line)//", index"//trim(sidx)//")."
                   call errormsg()

                else if (tmpidx(f,n) .gt. gdim(f)) then


                   write(line,'(i8)') nline
                   line = adjustl(line)
                   write(sidx,'(i8)') f
                   sidx = adjustl(sidx)

                   routine = "loadDVRindex"
                   message = "DVR-index in file "//trim(fname(nf)) &
                        // "is larger then gdim " &
                        // "(line "//trim(line)//", index "//trim(sidx)//")."
                   call errormsg()

                end if
             end do  check

             ! if nstep .ne. 0 we need to skip a few points
             if (np .lt. npointfromfile(nf)) then

                do n1=1,nstep(nf)-1 ! skip for nstep==1
                   nline = nline + 1

                   if (lbinary) then
                      read(idump,iostat=ierr)
                   else
                      read(idump,*,iostat=ierr)
                   end if

                   if (ierr .lt. 0) then !end of file
                      if (nf .gt. nfiles) goto 997

                   else if (ierr .gt. 0) then
                      goto 998
                   end if
                end do
             end if

          end do chunkloop

       end if rank0


       call mpi_send_chunk(tmpidx,dvrindex,ndof,mpi_chunksizes(nchunk),nchunk)

       if (allocated(tmpidx)) deallocate(tmpidx)

    end do chunk

    close(idump)
    return

997 continue

    write(line,'(i8,a,i8)') nline, " at sampling point ",n2
    call cncntrblnk(line)
    line = adjustl(line)

    routine = "loadDVRindex (mcsamplingio.F90)"

    message = "End of file found in "//trim(fname(nf-1)) &
         //" at line "//trim(line)//". This was the last file to load" &
         //" data from but more points are expected."
    call errormsg()
    return

998 continue


    write(line,'(i8)') nline
    line = adjustl(line)
    routine = "loadDVRindex (mcsamplingio.F90)"

    message = "Error while reading file "//trim(fname(nf)) &
         //" at line "//trim(line)//"."
    call errormsg()
    return


  end subroutine loadDVRindex




  subroutine StartNewFile(iunit,fname,nskip,nhash,lbinary)
    implicit none
    integer, intent(IN)          :: iunit,nskip
    character(len=*), intent(IN) :: fname
    logical, intent(OUT)         :: lbinary
    integer, intent(OUT)         :: nhash
    integer                      :: ierr, nline, n
    logical                      :: lopen, lascii, lhash
    character(len=16)            :: line
    character(len=256)           :: buf


    inquire(iunit,opened=lopen)
    if (lopen) close(iunit)

    call isASCIIFile(fname, lascii, ierr)
    if ( ierr .ne. 0) then
       message = "Error while cheking format of file "//trim(fname)
       routine = "isASCIIFile called from loadDVRindex -> StartNewFile"
       call errormsg
    end if
    lbinary = .not. lascii


    if (lbinary) then
       open(iunit,file=fname, form='unformatted', status='old', iostat=ierr)
    else
       open(iunit,file=fname, form='formatted', status='old', iostat=ierr)
    end if
    if (ierr .ne. 0) then
       routine = "loadDVRindex -> StartNewFile"
       message = "Unable to open sampling file "//trim(fname)
       call errormsg()
    end if



    nhash = 0
    lhash = .true.
    do while (lhash)
       if (lbinary) then
          read(iunit,iostat=ierr) buf
       else
          read(iunit,*,iostat=ierr) buf
       end if
       if (ierr .ne. 0) goto 999

       call rmblnk(buf)
       if (buf(1:1) .ne. "#") then
          lhash = .false.
          backspace(iunit)
       else
          nhash = nhash + 1
       end if
    end do


    do n=1,nskip
       nline = nline + 1
       if (lbinary) then
          read(iunit,iostat=ierr)
       else
          read(iunit,*,iostat=ierr)
       end if
       if (ierr .ne. 0) goto 999
    end do

    return

999 continue

    write(line,'(i8)') nline
    line = adjustl(line)

    routine = "loadDVRindex -> StartNewFile (mcsamplingio.F90)"
    if (ierr .lt.0) then
       message = "Found end of file while reading file "//trim(fname) &
            //" at line "//trim(line)//"."
       write(*,*) nskip
       call errormsg()
    elseif (ierr .gt.0) then
       message = "Error while reading file "//trim(fname) &
            //" at line "//trim(line)//"."
       call errormsg()
    end if


  end subroutine StartNewFile



end module mcsamplingio
