!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! module natpottype
!
! contains the definition of type natpot_t and basic initialization,
! destruction and some helper routines for this type.
!
! 11/13 MS
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

module natpottype
  use tensortype
  use global 
  use channels
  use logdat
  use mtlib, only: mtxxdd,mttxdd
  use schmidtorthomod, only: schmidtorthor

  implicit none

  ! module members are private by default
  private

  ! selected public members
  public :: natpot_t                 ! type
  public :: allocate_natpot_t        ! "constructor"
  public :: deallocate_natpot_t      ! free mem
  public :: save_natpot_t            ! IO
  public :: load_natpot_t            ! IO
  public :: dump_natpot_info         ! IO
  public :: copynatpot_t             ! copy
  public :: wrnpotproperties         ! Logging
  public :: getModeLabelsStr         ! info
  public :: orthonormSPP             ! transform to an
                                     ! orthonormal spp basis
  public :: remove_contracted        ! trafo the contacted
                                     ! mode into an SPP-mode
  public :: contractmode             ! contact one of the modes
                                     ! if all are SPP modes
  public :: zeig_natpot_t            ! setup internal structures


  type, extends(tensor_t) :: natpot_t

     integer                           :: modc
     integer                           :: modcdim
     integer                           :: npotmodes
     integer                           :: ntotspp

     integer                           :: dimbef
     integer                           :: dimmodc
     integer                           :: dimaft

     integer                           :: maxspf
     integer                           :: maxsubdim
     integer, allocatable              :: potdim(:)
     integer, allocatable              :: potvdim(:), potndim(:)

     integer, allocatable              :: vgdim(:), ngdim(:)
     integer, allocatable              :: vgpdim(:), vgmdim(:)


     integer                           :: cdim
     double precision, allocatable     :: coeff(:)
     integer                           :: evecdim
     integer, allocatable              :: zevec(:,:)
     double precision, allocatable     :: evec(:)

     logical                           :: lpconm = .false.

     ! this is not in the tradiional potfit:
     ! syppetry operarions
     !

  end type natpot_t

contains

  ! initialize a natpot type
  subroutine allocate_natpot_t(natpot,ndof,nmode,gdim,potdim,dofspf,potdofmap, &
       modelabel,ort,ortdim,loned)
    implicit none

    type (natpot_t), intent(INOUT)       :: natpot
    integer, intent(IN)                  :: ndof,nmode,ortdim
    integer, intent(IN)                  :: gdim(ndof), dofspf(ndof)
    integer, intent(IN)                  :: potdim(nmode)
    integer, intent(IN)                  :: potdofmap(2,ndof)
    double precision, intent(IN)         :: ort(ortdim)
    logical, intent(IN)                  :: loned(ndof)
    character(len=*), intent(IN)         :: modelabel(ndof)
    integer                              :: m,n,f,len,lblen, slen

    ! destroy first, just to be sure
    call deallocate_natpot_t(natpot)

    if (ndof .le. 1) then
       write(*,'(a,i4,a)') "ndof= ",ndof,": must be 2 or larger."
       stop 1
    end if
    if (nmode .le. 1) then
       write(*,'(a,i4,a)') "mode= ",nmode,": must be 2 or larger."
       stop 1
    end if

    natpot%ndof = ndof
    natpot%nmode = nmode

    natpot%maxspfdof = 0
    natpot%maxspf = 0

    ! detect contacted mode (if any)
    natpot%modc = 0
    do m=1,nmode
       if (potdim(m) .eq. 0) then
          if (natpot%modc .ne. 0) then
             write(*,'(a,i4,a)') "mode ",m,": detected zero SPP as for"
             write(*,'(a)') "the contracted mode (which has been"
             write(*,'(a,i4,a)') "assigned to mode ",natpot%modc," already)."
             stop 1
          end if
          natpot%modc = m
       end if
    end do

    if (natpot%modc .eq. 0) then
       natpot%npotmodes = nmode
    else
       natpot%npotmodes = nmode -1
    end if

    do m=1,nmode
       n = 0
       do f=1,ndof
          if (dofspf(f) .eq. m) then
             n = n + 1
          end if
       end do
       natpot%maxspfdof = max(natpot%maxspfdof,n)
       natpot%maxspf = max(natpot%maxspf,potdim(m))
    end do

    natpot%ortdim = sum(gdim)

    allocate(natpot%ort(natpot%ortdim))
    allocate(natpot%zort(ndof))
    allocate(natpot%gdim(ndof))
    allocate(natpot%vgdim(ndof))
    allocate(natpot%ngdim(ndof))
    allocate(natpot%vgpdim(ndof))
    allocate(natpot%vgmdim(ndof))
    allocate(natpot%potdim(nmode))
    allocate(natpot%potvdim(nmode))
    allocate(natpot%potndim(nmode))
    allocate(natpot%subdim(nmode))
    allocate(natpot%nspfdof(nmode))
    allocate(natpot%dofspf(ndof))
    allocate(natpot%modelabel(ndof))
    allocate(natpot%spfdof(natpot%maxspfdof,nmode))
    allocate(natpot%potdofmap(2,ndof))
    allocate(natpot%zevec(nmode,natpot%maxspf))
    !unused at the moment
    allocate(natpot%loned(ndof))

    natpot%spfdof = 0

    ! things to copy
    natpot%gdim = gdim
    natpot%dofspf = dofspf
    natpot%potdim = potdim
    natpot%loned = loned


    lblen = len(natpot%modelabel(ndof))
    do f=1,ndof
       natpot%modelabel(f) = " "
       slen = len_trim(modelabel(f))
       if (slen .le.lblen) then
          natpot%modelabel(f)(1:slen) = trim(modelabel(f))
       else
          write(*,*) "Mode label too long: "//modelabel(f)
          stop 1
       end if
    end do

    if (natpot%ortdim .ne. ortdim) then
       write(*,*) "The size of 'ort' is not sum(gdim)"
       stop 1
    end if
    natpot%ort = ort

    do f=1,natpot%ndof
       natpot%potdofmap(1,f) = potdofmap(1,f)
       natpot%potdofmap(2,f) = potdofmap(2,f)
    end do

    call zeig_natpot_t(natpot)

    if (any(natpot%loned)) then
       allocate(natpot%oned(natpot%ortdim))
       natpot%oned = 0.d0
    end if

    ! Pointers are done, now allocate
    allocate(natpot%coeff(natpot%cdim))
    natpot%coeff = 0.d0
    allocate(natpot%evec(natpot%evecdim))
    natpot%evec = 0.d0

    natpot%initialized = .true.


  end subroutine allocate_natpot_t



  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! setup pointers for internal natpot structure
  ! needs all arrays allocated and
  ! needs set: ndof, nmode, gdim, dofspf, potdim potdofmap
  ! sets:
  !  rvpotdim
  !  vpotdim
  !  zort
  !  nspfdof
  !  spfdof
  !  lpconm
  !  vgdim
  !  ngdim
  !  subdim
  !  maxsubdim
  !  cdim
  !  modcdim
  !  ntotspp
  !  potvdim
  !  potndim
  !  dimbef
  !  dimmodc
  !  dimaft
  !  evecdim
  !  zevec
  !  vgmdim
  !  vgpdim
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  subroutine zeig_natpot_t(natpot)
    implicit none

    type (natpot_t), intent(INOUT) :: natpot
    integer                        :: f,ff,m,n
    integer                        :: itmp
    integer                        :: reorder(natpot%ndof)


    ! vpotdim
    natpot%rvpotdim = dble(natpot%gdim(1))
    natpot%vpotdim = natpot%gdim(1)
    natpot%zort(1) = 1
    do f=2,natpot%ndof
       natpot%rvpotdim = natpot%rvpotdim * dble(natpot%gdim(f))
       natpot%vpotdim = natpot%vpotdim * natpot%gdim(f)
       natpot%zort(f) = natpot%zort(f-1) + natpot%gdim(f-1)
    end do


    natpot%nspfdof = 0
    do f=1,natpot%ndof
       m = natpot%dofspf(f)
       natpot%nspfdof(m) = natpot%nspfdof(m) + 1
    end do

    do f=1,natpot%ndof
       m = natpot%potdofmap(1,f)
       n = natpot%potdofmap(2,f)
       natpot%spfdof(n,m) = f
    end do

    natpot%lpconm = (maxval(natpot%nspfdof) .gt. 1)

    do m=1,natpot%nmode
       itmp=1
       do n=1,natpot%nspfdof(m)
          f=natpot%spfdof(n,m)
          natpot%vgdim(f) = itmp
          itmp=itmp*natpot%gdim(f)
       enddo
       natpot%subdim(m)=itmp
       do n=1,natpot%nspfdof(m)
          f=natpot%spfdof(n,m)
          natpot%ngdim(f) = itmp/(natpot%gdim(f)*natpot%vgdim(f))
       enddo
    enddo
    natpot%maxsubdim = maxval(natpot%subdim)

    natpot%cdim = 1
    do m=1,natpot%nmode
       if (m.ne.natpot%modc) then
          natpot%cdim = natpot%cdim * natpot%potdim(m)
       else
          natpot%cdim = natpot%cdim * natpot%subdim(m)
       end if
    end do


    natpot%modcdim = 1
    do m=1,natpot%nmode
       if (m.ne.natpot%modc) then
          natpot%modcdim = natpot%modcdim * natpot%potdim(m)
       end if
    end do

    natpot%ntotspp = 0
    do m=1,natpot%nmode
       if (m.ne.natpot%modc) then
          natpot%ntotspp =  natpot%ntotspp + natpot%potdim(m)
       end if
    end do


    natpot%potvdim(1)=1
    do m=2,natpot%nmode
       if (m-1.ne.natpot%modc) then
          natpot%potvdim(m) =natpot%potvdim(m-1)*natpot%potdim(m-1)
       else
          natpot%potvdim(m) =natpot%potvdim(m-1)*natpot%subdim(m-1)
       end if
    end do


    natpot%potndim(natpot%nmode)=1
    do m=natpot%nmode-1,1,-1
       if (m+1.ne.natpot%modc) then
          natpot%potndim(m)= natpot%potndim(m+1)*natpot%potdim(m+1)
       else
          natpot%potndim(m)= natpot%potndim(m+1)*natpot%subdim(m+1)
       end if
    end do

    if (natpot%modc .ne. 0) then
       natpot%dimbef = natpot%potvdim(natpot%modc)
       natpot%dimmodc = natpot%subdim(natpot%modc)
       natpot%dimaft = natpot%potndim(natpot%modc)
    else
       natpot%dimbef = 1
       natpot%dimmodc = 1
       natpot%dimaft = 1
    end if

    ! NOTE: zevec is different from original natpot
    natpot%evecdim = 0
    natpot%zevec = 0
    do m=1,natpot%nmode
       if (m.ne.natpot%modc) then
          do n=1,natpot%potdim(m)
             natpot%zevec(m,n) = natpot%evecdim + 1
             natpot%evecdim = natpot%evecdim +  natpot%subdim(m)
          end do
       end if
    end do

    !-----------------------------------------------------------------------
    ! set arrays vgmdim and vgpdim, which contains the `vor'-grid-dimensions
    ! in mode order and in dof order. Copy from zeigpfit
    !-----------------------------------------------------------------------
    ff=0
    !ff=mode order, f=dof order.
    do m=1,natpot%nmode
       do n=1,natpot%nspfdof(m)
          f=natpot%spfdof(n,m)
          ff=ff+1
          reorder(ff)=f
       enddo
    enddo

    ! set vgmdim, `vor'-grid-dimension in mode order.
    natpot%vgmdim(reorder(1))=1
    do ff=1,natpot%ndof-1
       natpot%vgmdim(reorder(ff+1))= &
            natpot%vgmdim(reorder(ff))*natpot%gdim(reorder(ff))
    enddo

    ! set vgpdim, `vor'-grid-dimension in dof order.
    natpot%vgpdim(1)=1
    do f=1,natpot%ndof-1
       natpot%vgpdim(f+1)=natpot%vgpdim(f)*natpot%gdim(f)
    enddo

  end subroutine zeig_natpot_t


  ! free memory and reset parameters
  subroutine deallocate_natpot_t(natpot)
    type (natpot_t), intent(INOUT)       :: natpot

    if (allocated(natpot%evec)) deallocate(natpot%evec)
    if (allocated(natpot%zevec)) deallocate(natpot%zevec)
    if (allocated(natpot%potdim)) deallocate(natpot%potdim)
    if (allocated(natpot%potvdim)) deallocate(natpot%potvdim)
    if (allocated(natpot%potndim)) deallocate(natpot%potndim)

    if (allocated(natpot%vgdim)) deallocate(natpot%vgdim)
    if (allocated(natpot%ngdim)) deallocate(natpot%ngdim)
    if (allocated(natpot%vgpdim)) deallocate(natpot%vgpdim)
    if (allocated(natpot%vgmdim)) deallocate(natpot%vgmdim)
    if (allocated(natpot%coeff)) deallocate(natpot%coeff)


    natpot%modc = 0
    natpot%modcdim = 0
    natpot%npotmodes = 0
    natpot%ntotspp = 0

    natpot%cdim = 1
    natpot%evecdim = 1

    natpot%vpotdim = 1

    natpot%dimbef = 1
    natpot%dimmodc = 1
    natpot%dimaft = 1

    call deallocate_tensor_t(natpot)

  end subroutine deallocate_natpot_t


  subroutine copynatpot_t(npotin, npotout)
    implicit none
    type(natpot_t), intent(IN)      :: npotin
    type(natpot_t), intent(INOUT)   :: npotout

    ! this calls deallocate and makes all pointers
    call allocate_natpot_t(npotout,npotin%ndof,npotin%nmode,npotin%gdim, &
         npotin%potdim,npotin%dofspf,npotin%potdofmap,npotin%modelabel, &
         npotin%ort,npotin%ortdim, npotin%loned)

    npotout%evec = npotin%evec
    npotout%coeff = npotin%coeff
    if (any(npotin%loned))  npotout%oned = npotin%oned

  end subroutine copynatpot_t



  ! write disk dump
  subroutine save_natpot_t(natpot, fname, lovwr)
    implicit none
    type(natpot_t), intent(IN)     :: natpot
    character(len=*), intent(IN)   :: fname
    integer, parameter             :: idump = 50
    logical, intent(IN), optional  :: lovwr
    logical                        :: lovwr1, lexists

    if (present(lovwr)) then
       lovwr1 = lovwr
    else
       lovwr1 = .false.
    end if

    if (.not. natpot%initialized) then
       write(*,*) "Natpot not initialized. Did not save file "//fname
       return
    end if

    inquire(file=fname, exist=lexists)
    if (lexists .and. .not. lovwr1) then
       write(*,*) "File exists. No permission to overwrite: "//fname
       stop 1
    end if

    open(idump,file=fname,status="unknown",form="unformatted")
    call dump_napot_data(natpot,idump)
    close(idump)

  end subroutine save_natpot_t


  subroutine dump_napot_data(natpot,channel)
    implicit none
    type(natpot_t), intent(IN)     :: natpot
    integer, intent(IN)            :: channel

    call dump_natpot_info(natpot,channel)

    if (any(natpot%loned))  write(channel) natpot%oned
    write(channel) natpot%evec
    write(channel) natpot%coeff

  end subroutine dump_napot_data

  subroutine dump_natpot_info(natpot,channel)
    implicit none
    type(natpot_t), intent(IN)     :: natpot
    integer, intent(IN)            :: channel

    write(channel) natpot%ndof, natpot%nmode
    write(channel) natpot%gdim
    write(channel) natpot%potdim
    write(channel) natpot%dofspf
    write(channel) natpot%potdofmap
    write(channel) natpot%loned
    write(channel) natpot%modelabel
    write(channel) natpot%ort

  end subroutine dump_natpot_info


  ! read disk dump
  subroutine load_natpot_t(natpot, fname)
    implicit none
    type(natpot_t), intent(INOUT)     :: natpot
    character(len=*), intent(IN)      :: fname

    open(idump,file=fname,status="old",form="unformatted")

    call load_natpot_data(natpot,idump)

    close(idump)

  end subroutine load_natpot_t


  subroutine load_natpot_data(natpot,channel)
    implicit none
    type(natpot_t), intent(INOUT)  :: natpot
    integer, intent(IN)            :: channel

    integer                        :: ndof, nmode, ortdim
    integer, allocatable           :: gdim(:), potdim(:), dofspf(:)
    integer, allocatable           :: potdofmap(:,:)
    double precision, allocatable  :: ort(:)
    character(len=C1), allocatable :: modelabel(:)
    logical, allocatable           :: loned(:)

    read(channel) ndof,nmode
    allocate(gdim(ndof))
    allocate(potdim(nmode))
    allocate(dofspf(ndof))
    allocate(modelabel(ndof))
    allocate(potdofmap(2,ndof))
    allocate(loned(ndof))

    read(channel) gdim
    read(channel) potdim
    read(channel) dofspf
    read(channel) modelabel
    read(channel) potdofmap
    read(channel) loned

    ortdim = sum(gdim)
    allocate(ort(ortdim))

    read(channel) ort

    call allocate_natpot_t(natpot,ndof,nmode,gdim,potdim,dofspf,potdofmap, &
         modelabel,ort,ortdim,loned)

    if (any(natpot%loned)) read(channel) natpot%oned
    read(channel) natpot%evec
    read(channel) natpot%coeff

    deallocate(gdim)
    deallocate(potdim)
    deallocate(dofspf)
    deallocate(potdofmap)
    deallocate(modelabel)
    deallocate(ort)
    deallocate(loned)

  end subroutine load_natpot_data



  ! write information about the natpot to a channel
  ! default channel is 6 (stdout)
  subroutine wrnpotproperties(natpot, channel)
    type(natpot_t), intent(IN)     :: natpot
    integer, intent(IN), optional  :: channel
    integer                        :: channel1
    logical                        :: isopen
    integer                        :: m,f, ilbl, jlbl
    double precision               :: mem, totmem
    character(len=80)              :: coords

    if (present(channel)) then
       channel1 = channel
    else
       channel1 = 6
    end if

    inquire(channel1, opened=isopen)
    if (.not. isopen) return

    write(channel1,'(a)') ""
    write(channel1,'(a)') "Natpot-Info:"
    write(channel1,'(a)') ""
    if (.not. natpot%initialized) then
       write(channel1,'(a)') " >> Natpot is not initialized <<"
       return
    end if
    write(channel1,'(a)') "SPP:"
    write(channel1,'(a)') "  Mode  nSPP    gdim     MEM(SPP)   coordinates"

    totmem = 0.d0
    do m = 1,natpot%nmode

       coords = ""
       ilbl = 1
       do f=1,natpot%ndof
          if (natpot%dofspf(f) .eq. m) then
             jlbl = ilbl + len_trim(natpot%modelabel(f)) + 1
             coords(ilbl:jlbl) = trim(natpot%modelabel(f))//" "
             ilbl = jlbl +1
          end if
       end do

       if (m .eq. natpot%modc) then
          write(channel1,'(i6,a,i8,8x,a,6x,a)') m, &
               " contr", natpot%subdim(m),&
               "--",trim(coords)

       else
          mem = dble(natpot%potdim(m)*natpot%subdim(m))/131072 !MB
          totmem = totmem + mem
          write(channel1,'(2i6,i8,f10.3,a)') m, natpot%potdim(m), &
               natpot%subdim(m), &
               mem, " MB   " &
               //trim(coords)
       end if
    end do
    write(channel1,'(a)') "---------------------------------------" &
         //"---------------"
    write(channel1,'(a,f14.3,a)') "Total MEM (SPP):", totmem, " MB"
    write(channel1,'(a)') ""
    mem = dble(natpot%cdim)/131072 !MB
    write(channel1,'(a,i12,3x,f10.3,a)') "Coefficients:", &
         natpot%cdim, mem, " MB"
    totmem = totmem + mem
    write(channel1,'(a)') ""
    write(channel1,'(a,f10.3,a)') "Total Memory (Natpot):      ", &
         totmem, " MB"
    write(channel1,'(a)') ""

  end subroutine wrnpotproperties


  ! contact a mode if no contracted mode exists
  subroutine contractmode(natpotin, natpotout, cmode)
    implicit none
    type(natpot_t), intent(IN),target    :: natpotin
    type(natpot_t), intent(INOUT),target :: natpotout
    integer, intent(IN), optional        :: cmode

    integer                 :: cmode1

    integer                 :: m,zi,zi1, zo,zo1
    double precision        :: ratio, ratio1
    integer                 :: tmp_potdim(natpotin%nmode) !automatic
    integer                 :: dim1,dim2,dim3,dim4

    if (.not. natpotin%initialized) then
       write(*,*) "contractmode (npottype.f90):"
       write(*,*) "natpot not initialized."
       stop 1
    end if

    if (present(cmode)) then
       if (cmode .gt. natpotin%nmode .or. cmode .le. 0) then
          write(*,*) "contractmode (npottype.f90):"
          write(*,*) "contacted mode number is out of bounds."
          stop 1
       end if

       cmode1 = cmode
    else
       ! find mode that is do be contacted:
       ! determine smallest increase of memory
       tmp_potdim = natpotin%potdim
       ratio = dble(natpotin%subdim(1))/natpotin%potdim(1)
       cmode1 = 1

       do m=2,natpotin%nmode
          ratio1 = dble(natpotin%subdim(m))/natpotin%potdim(m)
          if (ratio1 .lt. ratio) then
             ratio = ratio1
             cmode1 = m
          end if
       end do
       tmp_potdim(cmode1) = 0
    end if

    call allocate_natpot_t(natpotout, &
         natpotin%ndof, &
         natpotin%nmode, &
         natpotin%gdim, &
         tmp_potdim, &
         natpotin%dofspf, &
         natpotin%potdofmap, &
         natpotin%modelabel, &
         natpotin%ort, &
         natpotin%ortdim, &
         natpotin%loned)


    ! copy SPP
    do m=1,natpotout%nmode
       if (m.ne. natpotout%modc) then
          zi = natpotin%zevec(m,1)
          zi1 = zi + natpotin%subdim(m)*natpotin%potdim(m) - 1
          zo = natpotout%zevec(m,1)
          zo1 = zo +  natpotout%subdim(m)*natpotout%potdim(m) - 1
          if ( (zi1-zi) .ne. (zo1-zo) ) then
             write(*,*) "contractmode: internal error:"
             write(*,*) "SPP pointers are corrupted."
             stop 1
          end if

          natpotout%evec(zo:zo1) = natpotin%evec(zi:zi1)

       end if
    end do

    ! copy coefficients
    m = natpotout%modc
    zi = natpotin%zevec(m,1)
    dim4 = natpotin%subdim(m)
    dim2 = natpotin%potdim(m)
    dim1 = natpotin%potvdim(m)
    dim3 = natpotin%potndim(m)


    call mtxxdd(natpotin%evec(zi), &  ! a
         natpotin%coeff, &            ! b
         natpotout%coeff, &           ! c
         dim1,dim2,dim3,dim4)


    ! copy oned
    if (any(natpotout%loned)) natpotout%oned = natpotin%oned

  end subroutine contractmode


  ! change the contacted mode into one with SPP.
  ! use as many basis functions so that
  ! the error is smaller than npoperr
  ! npoperr > sum_n eval(n), n: neglected npops
  ! unit of eval(n) is au**2
  ! on exit nspp is the number of SPP used
  ! and npoperr the actual error
  subroutine remove_contracted(natpot, npoperr,nspp)
    implicit none
    type(natpot_t), intent(INOUT)   :: natpot
    double precision, intent(INOUT) :: npoperr
    integer, intent(OUT)            :: nspp

    double precision, allocatable   :: density(:,:)
    double precision, allocatable   :: eval(:)
    double precision, allocatable   :: work(:)
    integer                         :: i,j,m,n,modc,z,z1,zn,zn1
    integer                         :: dim1,dim2,dim3,dim4, maxspp
    integer                         :: potdim(natpot%nmode)

    character, parameter            :: JOBZ = "V"
    character, parameter            :: UPLO = "U"
    integer                         :: LWORK, INFO

    type(natpot_t)                  :: np_new

    modc = natpot%modc
    if (modc.eq.0) then
       nspp = 0
       npoperr = 0.d0
       return ! nothing to do
    end if

    ! create density for contracted mode
    allocate(density(natpot%subdim(modc),natpot%subdim(modc)))
    allocate(eval(natpot%subdim(modc)))

    !$omp parallel do private(i) schedule(dynamic)
    do j=1, natpot%subdim(modc)
       do i=j, natpot%subdim(modc)
          call denij(natpot%coeff,i,j,density(i,j), &
               natpot%potvdim(modc), natpot%subdim(modc),natpot%potndim(modc))
          density(i,j) = -density(i,j)
       end do
    end do
    !$omp end parallel do

    do j=1, natpot%subdim(modc)
       do i=j+1, natpot%subdim(modc)
          density(j,i) = density(i,j)
       end do
    end do

    ! digonalize
    LWORK = (natpot%subdim(modc)+2)*natpot%subdim(modc)
    allocate(work(LWORK))
    call DSYEV( JOBZ, UPLO, natpot%subdim(modc), density,  &
         natpot%subdim(modc), eval, WORK, LWORK, INFO )

    if (info .ne. 0) then
       routine = "remove_cotracted (npottype.F90)"
       write(message,'(a,i6)') "DSYEV exit code: ", info
       call errormsg
    end if

    maxspp = min(natpot%subdim(modc), natpot%potvdim(modc)* natpot%potndim(modc))

    ! calculate error as function of nSPP
    work(maxspp)=0.d0
    do j=maxspp,2,-1
       work(j-1)=work(j)+abs(eval(j))/natpot%rvpotdim
    enddo

    nspp = -1
    do i=1,maxspp
       if (work(i) .lt. npoperr) then
          nspp = i
          exit ! loop
       end if
    end do
    if (nspp .lt. 0) nspp = maxspp
    npoperr = work(nspp)


    ! crate a new natpot object
    ! the only difference to the input one
    ! is that there is no contracted mode
    potdim = natpot%potdim
    potdim(modc) = nspp

    call allocate_natpot_t(np_new,natpot%ndof,natpot%nmode,natpot%gdim, &
         potdim,natpot%dofspf,natpot%potdofmap,natpot%modelabel, &
         natpot%ort,natpot%ortdim,natpot%loned)


    ! copy oned
    if (any(natpot%loned)) np_new%oned = natpot%oned

    !copy the spp
    do m=1,natpot%nmode
       if (m.ne.modc) then
          do n=1,natpot%potdim(m)
             zn = np_new%zevec(m,n)
             zn1 = zn + np_new%subdim(m) - 1
             z = natpot%zevec(m,n)
             z1 = z + natpot%subdim(m) - 1
             np_new%evec(zn:zn1) = natpot%evec(z:z1)
          end do

       else

          do n=1,nspp
             zn = np_new%zevec(m,n)
             zn1 = zn + np_new%subdim(m) - 1
             np_new%evec(zn:zn1) = density(:,n)
          end do

       end if
    end do

    dim1 = natpot%potvdim(modc)
    dim2 = natpot%subdim(modc)
    dim3 = natpot%potndim(modc)
    dim4 = nspp

    call mttxdd(density(:,1:nspp),natpot%coeff, np_new%coeff, &
         dim1,dim2,dim3,dim4)

    call copynatpot_t(np_new, natpot)
    call deallocate_natpot_t(np_new)

    deallocate(density)
    deallocate(eval)
    deallocate(work)

  end subroutine remove_contracted


  subroutine denij(A,i,j,Aij,dim1,dim2,dim3)
    implicit none
    integer, intent(IN)            :: i,j,dim1,dim2,dim3
    double precision, intent(IN)   :: A(dim1,dim2,dim3)
    double precision, intent(OUT)  :: Aij

    integer                        :: k,l

    Aij = 0.d0
    do l=1,dim3
       do k=1,dim1
          Aij =  Aij + A(k,i,l)*A(k,j,l)
       end do
    end do

  end subroutine denij




  subroutine orthonormSPP(natpot)
    implicit none

    type(natpot_t), intent(INOUT) :: natpot
    double precision, allocatable :: Q(:,:),sigma(:,:)
    integer                       :: m,n,n1,z,z1, ierr
    integer                       :: dim1,dim2,dim3,dim4
    double precision              :: newcoeff(natpot%cdim), mnorm

    do m=1,natpot%nmode

       if (m.eq.natpot%modc) cycle

       allocate(Q(natpot%subdim(m), natpot%potdim(m)))
       allocate(sigma(natpot%potdim(m), natpot%potdim(m)))

       !$omp parallel do private(z,z1)
       do n=1,natpot%potdim(m)
          z = natpot%zevec(m,n)
          z1 = z + natpot%subdim(m) - 1
          Q(:,n) = natpot%evec(z:z1)
       end do
       !$omp end parallel do

       ! in-place schmidt ortho
       call schmidtorthor(Q,natpot%subdim(m), &
            natpot%potdim(m),ierr,mnorm)
       if (ierr.ne.0) then
          routine = "orthonormSPP (npottype.f90)"
          message = "Linear dependent SPP detected."
          call errormsg
       end if

       !$omp parallel do private(n,z,z1)
       do n1=1,natpot%potdim(m)
          do n=1,natpot%potdim(m)
             z = natpot%zevec(m,n)
             z1 = z + natpot%subdim(m) - 1
             sigma(n,n1) = dot_product(Q(:,n1),natpot%evec(z:z1))
          end do
       end do
       !$omp end parallel do

       dim1 = natpot%potvdim(m)
       dim2 = natpot%potdim(m)
       dim3 = natpot%potndim(m)
       dim4 = natpot%potdim(m)

       call mttxdd (sigma,natpot%coeff,newcoeff,dim1,dim2,dim3,dim4)

       natpot%coeff = newcoeff

       !$omp parallel do private(z,z1)
       do n=1,natpot%potdim(m)
          z = natpot%zevec(m,n)
          z1 = z + natpot%subdim(m) - 1
          natpot%evec(z:z1) = Q(:,n)
       end do
       !$omp end parallel do


       deallocate(Q,sigma)
    end do


  end subroutine orthonormSPP


end module natpottype

