!   -*- f90 -*-


! Note: the cpd-objects are only modified in
! MPI rank 0 and then distributed if needed.
! All changes anywhere else will not persist... All
! subroutines called from here must adhere to this rule.
!
! MPI ranks > 0 will get a copy of the CPD in rank 0
! after it has been modified and must update their
! re-usable data using this copy.


subroutine runmccpd()
  use tensortype
  use dvrtype_m
  use cpdtype_m
  use cpdtype_s, only: mpi_bcast_cpd_t, wrcpdproperties
  use mccpdparams
  use pesselect_m
  use pesselect_s, only: assignpot
  use runfit_m
  use mcpfitparams
  use mctools
  use mcsampling
  use eximporttools
  use mccuts_m
  use mccuts_s, only: free_cuts, load_mccuts, mk_mccuts, spenergiesfromcuts
  use mcstatistics
  use symmops_m
  use symmops_s
  use mcpeinsym
  use rshdmr
  use cpdtype_s, only: changerank
  use mcals_m, only: mc_optimize_cpd

  use channels
  use maxv, only: maxhop
  use global
  use pglobal
  use griddatmod
  use runpfitmod
  use rdopermod
  use combined
  use operdef
  use callmuldmod, only:defmuld
  use funcsrf, only: surfinfo
  use timing
  use mpidata
#ifdef MPI
  use mpi
#endif

  implicit none

  ! we use different versions of CPDs, depending on
  ! whether we have an extended grid or not
  ! and point the 'cpd' to the one we are using
  type(cpd_t)                    :: cpd_origgrid
  type(cpd_t)                    :: cpd_extgrid
  logical                        :: lextgrid

  type(dvr_t),target             :: systemdvr, operdvr
  type(dvr_t),pointer            :: dvr

  integer                        :: nsample, msize, allocstat
  logical, allocatable           :: loned(:)

  integer                        :: ierr,hl,h,m
  character(len=c5)              :: fname
  logical                        :: lmkinitguess
  logical                        :: lmatch, lerr

  double precision, allocatable, target :: hops(:)
  integer                        :: chkdvr,chkgrd
  integer, dimension(2*maxhop)   :: done

  integer, save                  :: tid=0 ! timer ID
  integer, save                  :: tid1=0 ! timer1 ID

#ifdef MPI
  integer, parameter             :: nmpicomml = 20
  logical                        :: mpicomml(nmpicomml)
  integer, parameter             :: nmpicommi = 9
  integer                        :: mpicommi(nmpicommi)
  integer                        :: isize,isize2

  ! -up command line option
  call bcast_up_iparam()

  if (mpirank.eq.0) then

     mpicommi(1) = invertmethod
     mpicommi(2) = ndof
     mpicommi(3) = npmode
     mpicommi(4) = mcseed
     mpicommi(5) = maxrank
     mpicommi(6) = initrank
     mpicommi(7) = growiter
     mpicommi(8) = alsmaxiter
     mpicommi(9) = growrank
     call MPI_Bcast(mpicommi,nmpicommi,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)

     mpicomml(1) = lomppotential
     mpicomml(2) = lsaveinitguess
     mpicomml(3) = lloadcuts
     mpicomml(4) = lsamplingonly
     mpicomml(5) = lpesfile
     mpicomml(6) = lesample
     mpicomml(7) = lautoextgrid
     mpicomml(8) = lsymmetries
     mpicomml(9) = lsamecoeffspp
     mpicomml(10) = lcont
     mpicomml(11) = lcoeffoncuts
     mpicomml(12) = lfincoeff
     mpicomml(13) = lgrow
     mpicomml(14) = lrshdmr
     mpicomml(15) = ltryreusecuts
     mpicomml(16) = lloadcpd
     mpicomml(17) = lcutsonly
     mpicomml(18) = loperfit
     mpicomml(19) = lhermitian
     mpicomml(20) =  loverride_initrank
     call MPI_Bcast(mpicomml,nmpicomml,MPI_LOGICAL,0,MPI_COMM_WORLD,ierr)

  else

     call MPI_Bcast(mpicommi,nmpicommi,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)

     invertmethod = mpicommi(1)
     ndof         = mpicommi(2)
     npmode       = mpicommi(3)
     mcseed       = mpicommi(4) + mpirank ! different seed in each rank
     maxrank      = mpicommi(5)
     initrank     = mpicommi(6)
     growiter     = mpicommi(7)
     alsmaxiter   = mpicommi(8)
     growrank     = mpicommi(9)

     call MPI_Bcast(mpicomml,nmpicomml,MPI_LOGICAL,0,MPI_COMM_WORLD,ierr)
     lomppotential   = mpicomml(1)
     lsaveinitguess  = mpicomml(2)
     lloadcuts       = mpicomml(3)
     lsamplingonly   = mpicomml(4)
     lpesfile        = mpicomml(5)
     lesample        = mpicomml(6)
     lautoextgrid    = mpicomml(7)
     lsymmetries     = mpicomml(8)
     lsamecoeffspp   = mpicomml(9)
     lcont            = mpicomml(10)
     lcoeffoncuts    = mpicomml(11)
     lfincoeff       = mpicomml(12)
     lgrow           = mpicomml(13)
     lrshdmr         = mpicomml(14)
     ltryreusecuts   = mpicomml(15)
     lloadcpd        = mpicomml(16)
     lcutsonly       = mpicomml(17)
     loperfit        = mpicomml(18)
     lhermitian      = mpicomml(19)
     loverride_initrank = mpicomml(20)
  end if

  ! No. of sampling points etc
  isize = mc_ntasks*max_idxfile
  isize2 =  mc_ntasks*(max_idxfile+1)
  call  MPI_Bcast(ntotsample,isize2,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call  MPI_Bcast(nstep,isize,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call  MPI_Bcast(nskipfirst,isize,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call  MPI_Bcast(smethod,isize,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call  MPI_Bcast(beta,isize,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
  call  MPI_Bcast(nidxfile,mc_ntasks,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call  MPI_Bcast(epsilon_invert,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)


 ! cpd params
  call MPI_Bcast(gdim,      ndof,   MPI_INTEGER,  0, MPI_COMM_WORLD,ierr)
  call MPI_Bcast(dofpot,    ndof,   MPI_INTEGER,  0, MPI_COMM_WORLD,ierr)
  call MPI_Bcast(potdofmap, ndof*2, MPI_INTEGER,  0, MPI_COMM_WORLD,ierr)
  call MPI_Bcast(modelabel, ndof*c1,MPI_CHARACTER,0, MPI_COMM_WORLD,ierr)
  call MPI_Bcast(nmode,     1,      MPI_INTEGER,  0, MPI_COMM_WORLD,ierr)

 ! surface params
  call MPI_Bcast(pesopts,  maxpes*c4,        MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
  call MPI_Bcast(zpesopts, 2*maxhpar*maxpes, MPI_INTEGER,  0,MPI_COMM_WORLD,ierr)
  call MPI_Bcast(pesmin,   1,                MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
  call MPI_Bcast(pesmax,   1,                MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
  call MPI_Bcast(hoppar,   maxhpar*maxhop,   MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
  call MPI_Bcast(lcutpes,  1,                MPI_LOGICAL,0,MPI_COMM_WORLD,ierr)


  ! operator
  call MPI_Bcast(hoplab,maxhop*c2,MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
  call MPI_Bcast(hopdim,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  if (lpesfile) then ! .or. lrshdmr
     call MPI_Bcast(nspfdof,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(spfdof,maxdim**2,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(subdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  end if
  if (lpesfile) then
     call MPI_Bcast(hopilab,maxhop,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(kzahl,maxham,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(khzahl,maxdim*maxsta*maxham,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(zhun,maxdim*maxsta*maxham,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(zhop,maxhtm,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(hamilton,maxdim*maxkoe,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(htmtype,maxhtm,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(hopsdim,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(htmmode,maxhtm,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(htmdof,maxhtm,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(htmndof,maxhtm,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(vgdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(gdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(ngdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(htmadof,maxhtm,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  end if

#endif



  if (mpirank .eq. 0) then
     call logmccpdparams(ilog)
     call logsampling(ilog)
  end if


  if (mpirank.eq.0) then
     call loaddvr(systemdvr,trim(dname)//'/dvr')
     if (irundvr .eq. 2) then   ! dvr file was read
        call logdvrtype(systemdvr, ilog)
     end if
  end if
  call mpi_bcast_dvr_t(systemdvr) ! does nothing if not MPI

  if (loperfit) then
     call mkoperdvr(systemdvr, operdvr)
     call mpi_bcast_dvr_t(operdvr)
     dvr => operdvr
  else
     dvr => systemdvr
  end if


  !***************************************************
  ! set the procedure pointer in module pesselect
  ! to the desired function
  !***************************************************
  hl=hopdim
  call AssignPot(hoplab(hl), dvr, ilog)
  if (hoplab(hopdim) .ne. "usersurf" .and. hoplab(hopdim) .ne. "usersurf-idx"  &
       .and. hoplab(hopdim) .ne. "pes-file" ) then

     call defmuld(hopilab(hl),hoplab(hl),hopfile(hl),pesopts,zpesopts, &
          hopipar(1,hl),hoprpar(1,hl),maxhpar,pesmin,pesmax,&
          lcutpes,ierr,ndof,hoppar)

     if (ierr .ne. 0) then
        routine = "runmcpfit"
        message = "Surface not found in MCTDH library: "//hoplab(hl)
        call errormsg
     else if (hopfile(hl).eq.0) then
        routine = "runmcpfit"
        message = 'PES not found in MCTDH library : '//hoplab(hl)
        call errormsg
     endif
     h=1
     htmmuld(h)=1
     done(:) = 0
     if (mpirank.eq.0) call surfinfo(h,hl,done)
  end if

  !***************************************************
  ! load the PES file if required
  !***************************************************
  if (lpesfile) then
     allocate(hops(hopsdim), stat=allocstat)
     if (allocstat.ne.0) then
        msize = hopsdim
        call mcallocerr('hops', 'real*8',msize,ilog)
     end if

     ! rdoper overwrites hopdim.
     ! we need hopdim+1 if we are
     ! subtracting a pes file
     ! from some other routine
     hopdim1 = hopdim
     if (mpirank.eq.0) then
        chkdvr=2   !(used to be 1, Dieter)
        chkgrd=2   !(used to be 1, Dieter)
        call rdoper(hops,chkdvr,chkgrd)
     end if
     hopdim = hopdim1

#ifdef MPI
     ! the pointers have been send. now we need to send
     ! the actual operator data.
     call MPI_BCast(hops,hopsdim,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
     call MPI_BCast(koeff,maxkoe,MPI_DOUBLE_COMPLEX,0,MPI_COMM_WORLD,ierr)
#endif

     phops => hops

  end if



  ! create the dvr sampling indices if required
  if (mpirank.eq.0) then
     call get_timer(tid, 'ptspepes')
     call start_timer(tid)
  endif
  call setseed(mcseed)
  call mksamplings(dvr)
  if (mpirank.eq.0) call stop_timer(tid)

  !***************************************************
  ! stop here if we do the sampling only
  !***************************************************
  if (lsamplingonly) return


  !***************************************************
  !***************************************************
  ! TASK 1: LOAD SAMPLINGS AND RELATED DATA
  !***************************************************
  !***************************************************
  if (nidxfile(mc_task_spp) .eq. 0 .and. lsamecoeffspp) then
     call loadruntask(mc_task_coeff, dvr, nsample)
  else
     call loadruntask(mc_task_spp, dvr, nsample)
  end if

  ! does nothing if already tested
  ! or if "no-OMPPotenttail"
  ! is set for PES in RUN-SECTION
#ifdef OPENMP
  call test_pes_threadsafe(dvrindex, ndof, min(10000,nsample), ilog)
#endif

  !***************************************************
  !***************************************************
  ! TASK 2: Load or create the initial guess
  !***************************************************
  !***************************************************

  ! initialize the CPD in rank 0 only
  lmkinitguess = .true.
  lextgrid = .false.


  if (lcont) then

     if (mpirank .eq. 0) then
        fname =  trim(name)//"/cpd"
        write(ilog, '(/a)') "Loading file for continuation:"
        write(ilog, '(a/)') trim(fname)
        call myflush(ilog)
        call importMCTDH(cpd_origgrid, dvr, fname)

        if (initrank .ne. cpd_origgrid%rank) then
           write(ilog, '(/a)') "Changing rank of initial cpd:"
           write(ilog, '(a,i8)') " Rank old: ", cpd_origgrid%rank
           write(ilog, '(a,i8)') " Rank new: ", initrank
           call changerank(cpd_origgrid, initrank, .true.)
           write(ilog, '(a)') " OK"
        end if

     end if

     lmkinitguess = .false.
     call mpi_bcast_cpd_t(cpd_origgrid)

  else if (lloadcpd) then

     if (mpirank .eq. 0) then
        fname =  trim(restarcpd)//"/cpd"
        write(ilog, '(/a)') "Loading file as initial guess:"
        write(ilog, '(a/)') trim(fname)
        call myflush(ilog)
        call importMCTDH(cpd_origgrid, dvr, fname)

        if (initrank .ne. cpd_origgrid%rank &
             .and. .not. loverride_initrank) then
           write(ilog, '(/a)') "Changing rank of initial cpd:"
           write(ilog, '(a,i8)') " Rank old: ", cpd_origgrid%rank
           write(ilog, '(a,i8)') " Rank new: ", initrank
           call changerank(cpd_origgrid, initrank, .true.)
           write(ilog, '(a)') " OK"
        else if(initrank .ne. cpd_origgrid%rank &
             .and. loverride_initrank) then

           initrank = min(cpd_origgrid%rank, maxrank )
           if (lsymmetries) then
              if (modulo(initrank, nsymops) .ne. 0) then
                 initrank = initrank - modulo(initrank, nsymops)
                 write(ilog, '(/a)') "Changing rank of initial cpd to fit symmetry constraints:"
                 write(ilog, '(a,i8)') " Rank old: ", cpd_origgrid%rank
                 write(ilog, '(a,i8)') " Rank new: ", initrank
                 call changerank(cpd_origgrid, initrank, .true.)
                 write(ilog, '(a)') " OK"
              end if
           end if
           write(ilog, '(/a,i8)') "Changing initial rank accoring to restart: R=", initrank
        end if
     end if

     lmkinitguess = .false.

     call mpi_bcast_cpd_t(cpd_origgrid)
     ! eiter we have changed initrank accoring to restart above
     ! or we have changed restart to initrank
     ! either way cpd_origgrid should have the correct initial rank
     initrank = cpd_origgrid%rank

  else

     !***************************************************
     ! initialize an empty cpd: allocate memory
     ! and build internal pointers
     ! (in all mpi ranks) - no one-d at the moment
     ! we need this if we want to calculate cuts
     !***************************************************
     allocate(loned(dvr%ndof))
     loned = .false.

     if (loperfit) then
        call allocate_opercpd(cpd_origgrid,dvr,initrank,ndof,npmode,dofpot,potdofmap,loned)
     else

        call allocate_cpd_t(cpd_origgrid,initrank,ndof,npmode,gdim,dofpot,potdofmap, &
             modelabel,dvr%ort,dvr%ortdim, loned)

     end if
     deallocate(loned)

  end if

  ! we have now  cpd_origgrid. We need to build the
  ! symmetry operators on order to know if we need
  ! to use extended grids.


  lsymmetries = lsymmetries .or. (loperfit .and. lhermitian)


  if (lsymmetries) then

     if (loperfit) then
        call symHermitianOperator(cpd_origgrid,dvr)
     else
        call build_symops(cpd_origgrid, dvr)
     end if


     if (mpirank.eq.0) then

        if (.not. any(npermutedmodes .eq. 1)) then
           message = "All modes map into other modes during symmetry " &
                //"operations. The error estimates in the iterations file " &
                //"will not be accurate in this case. RMS values will be accurate " &
                //"in the final cpd-statistics and the log file, though."
           routine = "runmccpd"
           call warnmsg
           lwarn = .false.

           if (.not. lfincoeff) then
              message = "If all modes are swapping one should NOT set " &
                   //"'finalize-coeff = no' in the RUN-SECTION."
              routine = "runmccpd"
              call warnmsg
              lwarn = .false.
           end if

        else if (npermutedmodes(cpd_origgrid%nmode) .ne. 1) then

           if (mpirank.eq.0) then

              write(*,*) "Mode-mappings under symmetry operations:"
              write(*,*) "--------------------------"
              write(*,*) " mode m | maps onto modes:"
              write(*,*) "--------------------------"
              do m=1,cpd_origgrid%nmode
                 write(*,'(i8,a,100i4)') m, " | ", &
                      permutedmodes(m,1:npermutedmodes(m))
              end do
              write(*,*) "---------------------------"
              write(*,*)
              write(ilog,*)
              write(ilog,*) "Mode-mappings under symmetry operations:"
              write(ilog,*) "--------------------------"
              write(ilog,*) " mode m | maps onto modes:"
              write(ilog,*) "--------------------------"
              do m=1,cpd_origgrid%nmode
                 write(ilog,'(i8,a,100i4)') m, " | ", &
                      permutedmodes(m,1:npermutedmodes(m))
              end do
              write(ilog,*) "---------------------------"
              write(ilog,*)

           end if

           message = "Modes must be arranged such that the last mode " &
                //"does not map into another mode. Put one that " &
                //"maps only onto itself in the end of the MODES-SECTION."
           routine = "runmccpd"
           call errormsg
        end if

     end if ! mpirank 0

     if (.not. lcomplete) then
        message = "Symmetry operations must be complete. See sym.log"
        routine = "runmccpd"
        call errormsg
     end if

     if (any(lmextgrid)) then
        ! is extended grids are used we bild
        ! a second cpd with extended grids
        ! using information of the symops module
        call alloc_ext_cpd(cpd_extgrid,npmode, initrank)
        lextgrid = .true.
     end if

  end if ! symmetries


  ! so some logging before we start
  ! calculating or loading the cuts
  if (mpirank.eq.0) then
     call wrcpdproperties(cpd_origgrid, ilog)
     call myflush(ilog)
  end if

  if ( (lcont .or. lloadcpd) .and. lsymmetries) then
     if (lextgrid) then
        ! expand any loaded cpds to extendend grids
        ! and symmetrize the CPD
        write(ilog, '(a)') "Expanding mode-grids of loaded CPD to extended grids."
        call myflush(ilog)
        call origcpd2extcpd(cpd_origgrid,cpd_extgrid)
        call symmetrizecpd(cpd_extgrid)
     else
        ! just symmetrize the CPD
        call symmetrizecpd(cpd_origgrid)
     end if
  end if



  if (.not. lloadcuts) then

     if (mpirank.eq.0) then
        call get_timer(tid1, 'ptspefit')
        call start_timer(tid1)
     endif
     call mk_mccuts(cpd_origgrid,dvr)
     call stop_timer(tid1)

     ! we need to re-load the complete sampling
     if (nidxfile(mc_task_spp) .eq. 0 .and. lsamecoeffspp) then
        call loadruntask(mc_task_coeff, dvr, nsample)
     else
        call loadruntask(mc_task_spp, dvr, nsample)
     end if

  end if

  !***************************************************
  ! stop here if we do the cuts only
  !***************************************************
  if (lcutsonly) return


  ! we need to load in any case:

  if (lextgrid) then
     ! attention: mixed orig and ext
     call load_mccuts(cpd_origgrid%nmode, cpd_extgrid%subdim,dvr,cpd_origgrid)
  else
     ! attention: only orig
     call load_mccuts(cpd_origgrid%nmode, cpd_origgrid%subdim,dvr,cpd_origgrid)
  end if


  if (lrshdmr) then
     if (lextgrid) then
        call init_rshdmr(cpd_extgrid%nmode,cpd_extgrid%subdim)
     else
        call init_rshdmr(cpd_origgrid%nmode,cpd_origgrid%subdim)
     end if
  end if


  ! the sampling points have been loaded in loadruntask
  ! as dvrindex. Now convert to mode-index

  call dvrindex2modeindex(cpd_origgrid, nsample)


  if (lmkinitguess) then
     ! we populate the basis vectors
     ! only with a selection of
     ! the cuts calculated above.
     ! we do not populate the coefficients yet
     ! as we


     if (mpirank.eq.0) then

        if (linitrandom) then
           write(ilog, '(/a)') "Creating initial guess basis as random numbers."
           call myflush(ilog)

           if (lextgrid) then
              do m=1,cpd_extgrid%nmode
                 call random_number(cpd_extgrid%spp(m)%htm(:,:))
              end do
           else
              do m=1,cpd_origgrid%nmode
                 call random_number(cpd_origgrid%spp(m)%htm(:,:))
              end do
           end if
        else
           write(ilog, '(/a)') "Creating initial guess basis from cuts."
           call myflush(ilog)

           if (lextgrid) then
              call mccpd_initguessbasis(cpd_extgrid)
           else
              call mccpd_initguessbasis(cpd_origgrid)
           end if
        end if
        write(ilog, '(a/)') "Initial guess basis done."
        call myflush(ilog)
     end if

     if (lextgrid) then
        call mpi_bcast_cpd_t(cpd_extgrid)
        call extcpd2origcpd(cpd_extgrid,cpd_origgrid)
     end if
     call mpi_bcast_cpd_t(cpd_origgrid)

  end if



  ! We pre-fetch the points on the cpd-basis that are
  ! sampling points so we avoid index
  ! lookups and cache misses
  if (lextgrid) then
     call mkcpdsamplings(cpd_extgrid,modeindex,nsample)
  else
     call mkcpdsamplings(cpd_origgrid,modeindex,nsample)
  end if

  if (allocated(energies)) deallocate(energies)
  allocate(energies(nsample))
  call spenergiesfromcuts(nsample,cpd_origgrid%nmode,modeindex,energies, lerr)
  if (lerr) then
     routine = "runmccpd"
     message = "The spp-sampling points do not match the PES-cuts. Different sampling?"
     call errormsg
  end if

  ! if we have loaded the cuts from file we
  ! need to make sure that the currently loaded
  ! sampling points are actually the ones the
  ! cuts have been created with.
  ! The following checks a subset only (is faster)

  call chk_cuts_vs_routine(lmatch)
  if (.not. lmatch) then
     routine = "runmccpd"
     message = "The loaded cuts have either not been created " &
          //"with the current sampling or with a different PES routine " &
          //"or with a different vcut."
     call errormsg
  end if



  if ( lmkinitguess .and. lsaveinitguess) then
     ! we created an initial guess from cuts
     ! and now need to calculate the coefficients
     ! if the guess should be reasonably good.
     ! we can skip this if we do not want to
     ! write an initial guess to file

     if (mpirank.eq.0) then
        write(ilog, '(a)') "Calculating initial guess coefficients."
        call myflush(ilog)
     end if
     if (lextgrid) then
        call mccpd_coeff(cpd_extgrid, energies,nsample,invertmethod)
        call extcpd2origcpd(cpd_extgrid,cpd_origgrid)

        call mpi_bcast_cpd_t(cpd_origgrid)

     else
        call mccpd_coeff(cpd_origgrid, energies,nsample,invertmethod)
        call mpi_bcast_cpd_t(cpd_origgrid)


     end if

     if (mpirank.eq.0) then
        if (loperfit) then
           fname = trim(name)//"/hcpd_init"
        else
           fname = trim(name)//"/cpd_init"
        end if

        call exportMCTDH(cpd_origgrid, systemdvr, fname, lovwr, loperfit)
        write(ilog, '(a)') "Initial guess complete. Exported to file"
        write(ilog, '(a)') trim(fname)
        call myflush(ilog)
     end if
  else
     if (mpirank.eq.0) then
        write(ilog, '(/a)') "Skipping calculation of" &
             //" coefficients of initial guess (not needed, " &
             //"using zero)."
        call myflush(ilog)
     end if

  end if

  ! note: if no initial guess is requested as file by
  ! the user, the the coefficients have not yet been touched.
  ! (they are zero now). they will in this case be nonzero
  ! only after the first ALS optimization.



  !***************************************************
  !***************************************************
  ! TASK 3: Optimize the fit
  !***************************************************
  !***************************************************
  if (lextgrid) then
     call mc_optimize_cpd(cpd_extgrid, cpd_origgrid, systemdvr, invertmethod, lextgrid)
     call extcpd2origcpd(cpd_extgrid,cpd_origgrid)
  else
     call mc_optimize_cpd(cpd_origgrid, cpd_origgrid, systemdvr, invertmethod, lextgrid)
  end if
  call mpi_bcast_cpd_t(cpd_origgrid)

  ! test the spp-sampling against the cpd
  ! and write statistics to log
  if (mpirank.eq.0) then
     write(ilog, '(/a)') "Statistics of CPD error along the trajectory"
     write(ilog, '( a)') "used for optimization of the SPP"
     write(ilog, '( a)') "before coefficient optimization:"
     call myflush(ilog)
  end if
  call stat_current_traj(cpd_origgrid, ilog, .true.)


  ! finalize: recalculate the coeffs according
  ! to full sampling (not one mode integrated out)
  if (mpirank.eq.0) then
     if (nidxfile(mc_task_coeff) .ge.1 .and. .not. lsamecoeffspp ) then
        write(ilog, '(/a)') "Finalizing: recalculate the coeffs according"
        write(ilog, '(a/)') "to coeff-sampling (not one mode replaced by a cut)"
        write(ilog, '(a)')  "Coeff-sampling results:"
     else
        write(ilog, '(/a)') "Finalizing: recalculate the coeffs according"
        write(ilog, '(a)') "to SPP-sampling (not one mode replaced by a cut)"
        if (lcoeffoncuts) then
           write(ilog, '(a)') "using integration on all cuts."
        end if
        write(ilog, '(/a)')  "Coeff-sampling results:"
     end if
     call myflush(ilog)
  end if

  if (.not. lsamecoeffspp .and. nidxfile(mc_task_coeff) .ge.1 .and. lfincoeff) then
     if (mpirank.eq.0) then
        write(ilog, '(/a/)') "loading coeff-sampling-points ..."
     end if
     call loadruntask(mc_task_coeff, dvr, nsample)
     call update_energies_dvrindex(potential,lomppotential)
     call dvrindex2modeindex(cpd_origgrid, nsample)
     call mkcpdsamplings(cpd_origgrid,modeindex,nsample)
  end if

  ! if required we optimize the coefficients only
  ! in a last step. This we only do on the original dvr grid:
  ! the extended grids are removed already.
  if (lfincoeff) then
     ! we need to refresh the sampling as the CPD
     ! returned may not be the one from the last iteration
     call mkcpdsamplings(cpd_origgrid,modeindex,nsample)
     call mccpd_coeff(cpd_origgrid, energies,nsample,invertmethod)
  end if


  ! write to file
  if (loperfit) then
     fname = trim(name)//"/hcpd"
  else
     fname = trim(name)//"/cpd"
  end if
  call exportMCTDH(cpd_origgrid, systemdvr, fname, lovwr, loperfit)


    ! write the statistics to log
  call stat_current_traj(cpd_origgrid, ilog, .true.)



  if (lsymmetries) then
     if (lextgrid) then
        call testextcpdsymmetry(cpd_origgrid, cpd_extgrid, 50)
     end if

     call testcpdsymmetry(cpd_origgrid, 50)
  end if



  !***************************************************
  !***************************************************
  ! TASK 4: Testing
  !***************************************************
  !***************************************************

  if  (nidxfile(mc_task_test) .le. 0 ) then

     if (mpirank.eq.0) then
        write(ilog,'(/a/)') "NO TESTING TASK!"
        write(ilog,'(/a/)') "finishing..."
     end if

  elseif  (nidxfile(mc_task_test) .gt.1) then

     ! open with printing header
     call Open_stat_file("cpd-statistics", .true.)
     call stat_task_files(mc_task_test, dvr, cpd_origgrid, istat)

     if (mpirank.eq.0) then
        write(istat,'(a)')  "  **************************************"
        write(istat,'(a)')  "       STATISTICS FOR THE COMBINED "
        write(istat,'(a)')  "            TEST TRAJECTORIES"
        write(istat,'(a)')  "  **************************************"
     end if

  elseif  (nidxfile(mc_task_test) .eq.1) then

     ! open without printing header
     call Open_stat_file("cpd-statistics", .false.)

     if (mpirank.eq.0) then
        write(istat,'(a)')  "  **************************************"
        write(istat,'(a)')  "        STATISTICS FOR THE TEST "
        write(istat,'(a)')  "             TRAJECTORIES"
        write(istat,'(a)')  "  **************************************"
     end if

  end if

  if  (nidxfile(mc_task_test) .ge. 1) then
     call loadruntask(mc_task_test, dvr, nsample)
     call stat_current_traj(cpd_origgrid, istat, .true.)
     close(istat)
  end if


  ! release memory
  call free_cuts()
  call dispose_symops()
  call deallocate_cpd_t(cpd_origgrid)
  call deallocate_cpd_t(cpd_extgrid)
  call dispose_current_runtask()
end subroutine runmccpd



! check if the energies calculated
! match the PES routine (stichprobe)
subroutine chk_cuts_vs_routine(lmatch)
  use runfit_m
  use pesselect_m
#ifdef MPI
  use mpidata
  use mpi
#endif
  implicit none

  logical, intent(OUT)        :: lmatch

  double precision, parameter :: eps = 1.d-7
  double precision            :: v
  integer, parameter          :: nstep1 = 13  ! just test every 13'th point
  integer                     :: n, nsample,ndof1
  integer, allocatable        :: idx(:)
  logical                     :: docycle

#ifdef MPI
  logical :: lmatch1
#endif


  ndof1   = size(dvrindex,1)
  nsample = size(dvrindex,2)
  lmatch = .true.



  allocate(idx(ndof1))
  docycle = .false.

  !$omp parallel do private(n,idx,v) firstprivate(docycle) reduction(.and.: lmatch) if(lomppotential)
  do n=1,nsample, nstep1
     if (docycle) cycle
     idx = dvrindex(:,n)
     v = potential(idx,ndof1)
     lmatch = lmatch .and. (abs(v-energies(n)) .le. eps )
     docycle = .not. lmatch
     ! once lmatch is negative we skip the rest of the loops
  end do
  !$omp end parallel do

  deallocate(idx)

#ifdef MPI
  call mpi_allreduce(lmatch, lmatch1, 1, MPI_LOGICAL,  &
       MPI_LAND,MPI_COMM_WORLD,mpierr)
  lmatch = lmatch1
#endif


end subroutine chk_cuts_vs_routine



subroutine alloc_ext_cpd(cpd_extgrid,npmode, initrank)
  use global
  use cpdtype_m
  use symmops_m
  implicit none

  type(cpd_t), intent(INOUT)     :: cpd_extgrid
  integer, intent(IN)            :: npmode, initrank


  integer, allocatable           :: potdofmap2(:,:), dofpot2(:)
  integer                        :: ortdim2
  double precision, allocatable  :: ort2(:)
  character(len=c1), allocatable :: modelabel2(:)
  logical, allocatable           :: loned2(:)
  integer, allocatable           :: ext_subdim(:)
  integer                        :: m


    ! build a fake cpd
  allocate(ext_subdim(npmode))
  allocate(dofpot2(npmode))
  allocate(potdofmap2(2,npmode))
  allocate(loned2(npmode))
  allocate(modelabel2(npmode))
  do m=1,npmode
     ext_subdim(m) = pointmaps(m)%npoint
     dofpot2(m) = m
     potdofmap2(1,m) = m
     potdofmap2(2,m) = 1
     loned2(m) = .false.
     write(modelabel2(m),'(a,i4)')  "mode_", m
     call rmblnk(modelabel2(m))
  end do

  ortdim2 = sum(ext_subdim)
  allocate(ort2(ortdim2))
  ort2 = 0.d0

  call allocate_cpd_t(cpd_extgrid,initrank,npmode,npmode,ext_subdim,dofpot2,potdofmap2, &
       modelabel2,ort2,ortdim2, loned2)

  deallocate(ext_subdim,dofpot2,potdofmap2,loned2,modelabel2,ort2)

end subroutine alloc_ext_cpd



! call this in MPI rank 0 only
! and then call mpi_bcast_cpd_t
subroutine extcpd2origcpd(cpd_extgrid,cpd_origgrid)
  use cpdtype_m
  use cpdtype_s, only: changerank
  use mpidata

  implicit none

  type(cpd_t), intent(INOUT)     :: cpd_origgrid
  type(cpd_t), intent(IN)        :: cpd_extgrid

  integer                        :: m,r

  if (mpirank.ne.0) return


  if (cpd_origgrid%rank .ne. cpd_extgrid%rank) then
     ! this is needed for "grow" option
     call changerank(cpd_origgrid, cpd_extgrid%rank, .true.)
  end if

  do m=1,cpd_extgrid%nmode
     do r=1,cpd_extgrid%rank
        cpd_origgrid%spp(m)%htm(:,r) =  &
             cpd_extgrid%spp(m)%htm(1:cpd_origgrid%subdim(m),r)
     end do
  end do

  cpd_origgrid%coeff(1:cpd_extgrid%rank) = cpd_extgrid%coeff(1:cpd_extgrid%rank)


end subroutine extcpd2origcpd




subroutine origcpd2extcpd(cpd_origgrid,cpd_extgrid)
  use cpdtype_m
  use cpdtype_s, only: mpi_bcast_cpd_t
  use mpidata
  implicit none

  type(cpd_t), intent(INOUT)     :: cpd_extgrid
  type(cpd_t), intent(IN)        :: cpd_origgrid

  integer                        :: m,r
  integer                        :: subdim,subdim1

  if (mpirank.eq.0) then

     do m=1,cpd_origgrid%nmode

        subdim = cpd_origgrid%subdim(m)
        subdim1 = cpd_extgrid%subdim(m)

        do r=1,cpd_origgrid%rank

           cpd_extgrid%spp(m)%htm(1:subdim,r) = cpd_origgrid%spp(m)%htm(1:subdim,r)

           if (subdim .lt. subdim1) then
              cpd_extgrid%spp(m)%htm(subdim+1:subdim1,r)  = 0.d0
           end if
        end do
     end do
  end if

  cpd_extgrid%coeff = cpd_origgrid%coeff

  call mpi_bcast_cpd_t(cpd_extgrid)


end subroutine origcpd2extcpd

