!   -*- f90 -*-
!***********************************************************************
! runmcpfit
!
! allocate the natpot type and call subroutines to calculate
! SPP, coefficents and a test.
!
! 10/13 MS
!
!***********************************************************************


subroutine runmcpfit()

  ! modules
  use mctools
  use pftools
  use dvrtype_m
  use natpottype
  use eximporttools
  use mcpfitparams
  use mcpmpitools
  use mcsampling
  use mcsamplingio
  use mcdensities_m
  use mcdensities_s, only: calcspp
  use symmops_m
  use symmops_s, only: build_symops, build_simple_sym
  use mccomplete
  use runfit_m
  use rshdmr
  use pesselect_m
  use pesselect_s, only: assignpot
  use mcprecond_m
  use conjgrad
  use mcstatistics

  ! old global MCTDH variables
  use global
  use pglobal
  use maxv, only: maxdim,maxsta,maxkoe
  use griddatmod
  use runpfitmod
  use combined
  use channels
  use operdef
  use timing
  use mpidata
  use rdopermod, only: rdoper
  use callmuldmod
  use funcsrf, only: surfinfo
#ifdef MPI
  use mpi
#endif

  implicit none

  ! types
  type(natpot_t)                :: natpot
  type(dvr_t),target            :: dvr

  integer                       :: nsample

  ! locals
  integer                       :: allocstat,size
  integer                       :: ierr,hl,h
  character(len=c5)             :: fname
  logical                       :: lmkvpot
  logical, allocatable          :: loned(:)
  integer(long), dimension(maxhop) :: done


  double precision, allocatable, target :: hops(:)
  integer                       :: chkdvr,chkgrd
  integer                       :: hopdimold

  ! if we have an extended grid on the
  ! contracted mode we need an extra set of
  ! extended coeffifients
  double precision, allocatable :: coeff_ext(:)
  integer                       :: coeff_extdim

  integer, save                        :: tid=0 ! timer ID
  integer, save                        :: tid1=0 ! timer ID
  integer, save                        :: tid2=0 ! timer ID
  integer, save                        :: tid3=0 ! timer ID

#ifdef MPI
  integer, parameter            :: nmpicomml = 14
  logical                       :: mpicomml(nmpicomml)
  integer, parameter            :: nmpicommi = 7
  integer                       :: mpicommi(nmpicommi)
  integer                       :: isize,isize2
#endif

  interface
     subroutine calccoeff(natpot, nindex, dvrindex, coeff_ext, coeff_extdim)
       use pftools
       use conjgrad
       use pesselect_m
       use mcpfitparams
       use mcoverlap
       use symmops_m
       implicit none
       integer, intent(IN)                    :: nindex, coeff_extdim
       type(natpot_t), intent(INOUT), target  :: natpot
       integer, intent(IN), target            :: dvrindex(natpot%ndof,nindex)
       double precision, intent(OUT)          :: coeff_ext(coeff_extdim)
     end subroutine calccoeff
  end interface


#ifdef MPI

  ! -up command line option
  call bcast_up_iparam()

  if (mpirank.eq.0) then

     mpicommi(1) = invertmethod
     mpicommi(2) = ndof
     mpicommi(3) = npmode
     mpicommi(4) = cgmaxiter
     mpicommi(5) = cg_precon_type
     mpicommi(6) = cg_precon_size
     mpicommi(7) = mcseed
!     mpicommi(8) = krdim
     call MPI_Bcast(mpicommi,nmpicommi,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)

     mpicomml(1) = lomppotential
     mpicomml(2) = lomega
     mpicomml(3) = lomegat
     mpicomml(4) = lloadevecs
     mpicomml(5) = lsamplingonly
     mpicomml(6) = lpesfile
     mpicomml(7) = lregularization_coeff
     mpicomml(8) = lesample
     mpicomml(9) = lrshdmr
     mpicomml(10) = lrshdmr_rebuild
     mpicomml(11) = lautoextgrid
     mpicomml(12) = lsymmetries
     mpicomml(13) = lsamecoeffspp
     mpicomml(14) = lomegaspp
     call MPI_Bcast(mpicomml,nmpicomml,MPI_LOGICAL,0,MPI_COMM_WORLD,ierr)


  else
     call MPI_Bcast(mpicommi,nmpicommi,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     invertmethod   = mpicommi(1)
     ndof           = mpicommi(2)
     npmode         = mpicommi(3)
     cgmaxiter      = mpicommi(4)
     cg_precon_type = mpicommi(5)
     cg_precon_size = mpicommi(6)
     mcseed         = mpicommi(7) + mpirank ! different seed in each rank
 !    krdim          = mpicommi(8)
     call MPI_Bcast(mpicomml,nmpicomml,MPI_LOGICAL,0,MPI_COMM_WORLD,ierr)
     lomppotential = mpicomml(1)
     lomega        = mpicomml(2)
     lomegat       = mpicomml(3)
     lloadevecs    = mpicomml(4)
     lsamplingonly = mpicomml(5)
     lpesfile      = mpicomml(6)
     lregularization_coeff =  mpicomml(7)
     lesample      = mpicomml(8)
     lrshdmr       = mpicomml(9)
     lrshdmr_rebuild = mpicomml(10)
     lautoextgrid    = mpicomml(11)
     lsymmetries     = mpicomml(12)
     lsamecoeffspp   = mpicomml(13)
     lomegaspp       = mpicomml(14)
  end if

  ! No. of sampling points etc
  isize = mc_ntasks*max_idxfile
  isize2 =  mc_ntasks*(max_idxfile+1)
  call  MPI_Bcast(ntotsample,isize2,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call  MPI_Bcast(nstep,isize,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call  MPI_Bcast(nskipfirst,isize,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call  MPI_Bcast(smethod,isize,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call  MPI_Bcast(beta,isize,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
  call  MPI_Bcast(nidxfile,mc_ntasks,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call  MPI_Bcast(regularization_coeff,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
  call  MPI_Bcast(cg_precon_luse,cg_precon_maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)

  ! natpot params
  call MPI_Bcast(gdim,      ndof,   MPI_INTEGER,  0, MPI_COMM_WORLD,ierr)
  call MPI_Bcast(potdim,    npmode, MPI_INTEGER,  0, MPI_COMM_WORLD,ierr)
  call MPI_Bcast(dofpot,    ndof,   MPI_INTEGER,  0, MPI_COMM_WORLD,ierr)
  call MPI_Bcast(potdofmap, ndof*2, MPI_INTEGER,  0, MPI_COMM_WORLD,ierr)
  call MPI_Bcast(modelabel, ndof*c1,MPI_CHARACTER,0, MPI_COMM_WORLD,ierr)
  call MPI_Bcast(nmode,     1,      MPI_INTEGER,  0, MPI_COMM_WORLD,ierr)

  ! surface params
  call MPI_Bcast(pesopts,  maxpes*c4,        MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
  call MPI_Bcast(zpesopts, 2*maxhpar*maxpes, MPI_INTEGER,  0,MPI_COMM_WORLD,ierr)
  call MPI_Bcast(pesmin,   1,                MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
  call MPI_Bcast(pesmax,   1,                MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
  call MPI_Bcast(hoppar,   maxhpar*maxhop,   MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
  call MPI_Bcast(lcutpes,  1,                MPI_LOGICAL,0,MPI_COMM_WORLD,ierr)

  ! operator
  call MPI_Bcast(hoplab,maxhop*c2,MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
  call MPI_Bcast(hopdim,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  if (lpesfile .or. lrshdmr) then
     call MPI_Bcast(nspfdof,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(spfdof,maxdim**2,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(subdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  end if
  if (lpesfile) then
     call MPI_Bcast(hopilab,maxhop,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(kzahl,maxham,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(khzahl,maxdim*maxsta*maxham,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(zhun,maxdim*maxsta*maxham,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(zhop,maxhtm,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(hamilton,maxdim*maxkoe,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(htmtype,maxhtm,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(hopsdim,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(htmmode,maxhtm,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(htmdof,maxhtm,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(htmndof,maxhtm,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(vgdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(gdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(ngdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
     call MPI_Bcast(htmadof,maxhtm,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  end if

#endif


  if (mpirank .eq. 0) then
     call logmcpfitparams(ilog,npmode,potdim)

     ! some checks
     if (lsymmetries .and. lrshdmr) then
        message = "Symmetries for rshdmr is not implemented."
        routine = "runmcpfit"
        call errormsg
     end if
     if (lpesfile .and. lsymmetries .and. lautoextgrid) then
        message = "A EES file is used, symmetries are used " &
             //"and auto-extgrid is set. If extended grids " &
             //"are detected later on this run will abort fail."
        routine = "runmcpfit"
        call warnmsg
        lwarn = .false.
     end if
  end if


  !***************************************************
  ! set the procedure pointer in module mcpfitparams
  ! to the desired function
  !***************************************************
  hl=hopdim
  call AssignPot(hoplab(hl), dvr, ilog)
  if (hoplab(hopdim) .ne. "usersurf"  &
       .and. hoplab(hopdim) .ne. "pes-file" ) then

     call defmuld(hopilab(hl),hoplab(hl),hopfile(hl),pesopts,zpesopts, &
          hopipar(1,hl),hoprpar(1,hl),maxhpar,pesmin,pesmax,&
          lcutpes,ierr,ndof,hoppar)

     if (ierr .ne. 0) then
        routine = "runmcpfit"
        message = "Surface not found in MCTDH library: "//hoplab(hl)
        call errormsg
     else if (hopfile(hl).eq.0) then
        routine = "runmcpfit"
        message = 'PES not found in MCTDH library : '//hoplab(hl)
        call errormsg
     endif
     h=1
     htmmuld(h)=1
     done=0
     if (mpirank.eq.0) call surfinfo(h,hl,done)
  end if

  !***************************************************
  ! load the DVR file
  !***************************************************

  if (mpirank.eq.0) then
     call loaddvr(dvr,trim(dname)//'/dvr')
  end if
  call mpi_bcast_dvr_t(dvr) ! does nothing if not MPI


  !***************************************************
  ! load the PES file if required
  !***************************************************
  if (lpesfile) then
     allocate(hops(hopsdim), stat=allocstat)
     if (allocstat.ne.0) then
        size = hopsdim
        call mcallocerr('hops', 'real*8',size,ilog)
     end if

     ! rdoper overwrites hopdim.
     ! we need hopdim+1 if we are
     ! subtracting a pes file
     ! from some other routine
     hopdimold = hopdim
     if (mpirank.eq.0) then
        chkdvr=2   !(used to be 1, Dieter)
        chkgrd=2   !(used to be 1, Dieter)
        call rdoper(hops,chkdvr,chkgrd)
     end if
     hopdim = hopdimold

#ifdef MPI
     ! the pointers have been send. now we need to send
     ! the actual operator data.
     call MPI_BCast(hops,hopsdim,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
     call MPI_BCast(koeff,maxkoe,MPI_DOUBLE_COMPLEX,0,MPI_COMM_WORLD,ierr)
#endif

     phops => hops

  end if



  !***************************************************
  ! initialize an empty natpot: allocate memory
  ! and build internal pointers
  ! (in all mpi ranks) - no oned at the moment
  !***************************************************
  allocate(loned(dvr%ndof))
  loned = .false.
  call allocate_natpot_t(natpot,ndof,npmode,gdim,potdim,dofpot,potdofmap, &
       modelabel,dvr%ort,dvr%ortdim, loned)
  deallocate(loned)


  ! so some logging and make the samples
  ! this is only done in rank 0
  if (mpirank.eq.0) then
     call wrnpotproperties(natpot, ilog)
     call myflush(ilog)
     ! check if enough sampling points are specified
     ! and obtain nsample from file if needed.
     ! this doeas not load any data
     call checksamplingcounts(natpot)

     if (cg_precon_type .eq. cg_precon_band .and. &
          cg_precon_size .gt. natpot%modcdim) then
        routine = "runmcpfit"
        message = "Size of the band matrix in preconditionier "//&
             "cannot be larger that the number of configurations."
        call errormsg
     end if
  end if

  ! create the dvr samlping indices if required
  if (mpirank .eq. 0) then
     call get_timer(tid, 'mksamplings')
     call start_timer(tid)
  endif

  ! mcseed is initialized in mcpdefault
  ! which has been called before reading input
  ! it is either initialized with the system time
  ! of mpirank 0 or with the iseed parameter in
  ! RUN-SECTION. The rank number is added, see above
  call setseed(mcseed)
  call mksamplings(dvr)
  if (mpirank .eq. 0) call stop_timer(tid)

  !***************************************************
  ! stop here if we do the sampling only
  !***************************************************
  if (lsamplingonly) return


  !***************************************************
  ! If some of the methods is "mc_complete" make a vpot
  ! or load it from file if it exits
  !***************************************************

  if (any(smethod .eq. mc_complete)) then
     if (mpirank.eq.0) then

        lmkvpot = .false.
        call loadmcvpot(dvr,natpot,trim(dname)//'/vpot',ierr)
        if (ierr .ne. 0) then
           ! file does not exist or is corrupted
           if (ierr .gt. 1) write(ilog,'(a)') &
                "File mcvpot does not exist. Makeing new vpot"
           if (ierr .eq. 2) write(ilog,'(a)') &
                "File mcvpot and system are not compatible. Makeing new vpot"
           if (ierr .eq. 3) write(ilog,'(a)') &
                "vpot seems corrupted. Makeing new vpot"
           if (allocated(vpot)) deallocate(vpot)
           lmkvpot = .true.

        end if
     end if

#ifdef MPI
     call mpi_bcast(lmkvpot,1,MPI_LOGICAL,0,MPI_COMM_WORLD,ierr)
#endif

     if (lmkvpot) then
        call mcmkvpot(natpot, dvr)
     end if

  end if

  coeff_extdim = natpot%cdim

  if (lsymmetries) then
     ! build the symmetry operations
     ! this just uses the internal pointer structure
     ! of the natpot
     call build_symops(natpot,dvr)

     if (lpesfile .and. any(lmextgrid)) then
        message = "Extended grids cannot be used together " &
             //"with PES files."
        routine = "runmcpfit"
        call errormsg
     end if

     ! we do the symmetrization on a temorary array coeff_ext
     if ( natpot%modc .ne. 0 ) then
        if ( lmextgrid(natpot%modc) ) then
           coeff_extdim = natpot%cdim/natpot%subdim(natpot%modc)
           coeff_extdim = coeff_extdim*pointmaps(natpot%modc)%npoint
        end if
     end if

  else
     ! we use some structures from
     ! the symmetry module elsewhere
     ! so we initialize them here
     call build_simple_sym(natpot)
  end if



  ! we allocate this anyways as we use it in calccoeff
  ! in any case
  allocate(coeff_ext(coeff_extdim), STAT=allocstat)
  if (allocstat.ne.0) then
     size = pointmaps(natpot%modc)%npoint
     call mcallocerr('coeff_ext', 'real*8', size,ilog)
  end if


  !***************************************************
  !***************************************************
  ! TASK 0: CALCULATE RANDOM SAMPLING HDMR (if needed)
  !***************************************************
  !***************************************************
  if (lrshdmr) then
     ierr = 0

     if (lsymmetries) then
        message = "Symmetries for lrshdmr is not implemented. " &
             //"The results may not be predictable."
        routine = "runmcpfit"
        call warnmsg
        lwarn = .false.
     end if

     !try to load the rshdmr terms
     if (.not. lrshdmr_rebuild) then
        if (mpirank.eq.0) then
           write(ilog,'(a)') "Trying to load RSHDMR from file"
        end if
        call laod_rshdmr(natpot%nmode,ierr)
        if (ierr.ne.0 .and. mpirank.eq.0) then
           write(ilog,'(a)') "Reading RSHDMR from file failed. Will re-build."
        end if
     end if

     ! if loeading fails or explicit re-build is wanted
     ! then rebuild
     if (ierr.ne.0 .or.lrshdmr_rebuild ) then
        message = "Sorry, building a RSHDMR is broken at the moment"
        routine = "runmcpfit"
        call errormsg

        call loadruntask(mc_task_coeff, dvr, nsample)

        ! does nothing if already tested
        ! or of no OpenMP or if "no-OMPPotenttial"
        ! is set for PES in RUN-SECTION
        call test_pes_threadsafe(dvrindex, ndof, min(10000,nsample), ilog)

        call init_rshdmr(natpot%nmode,natpot%subdim)
        !call mkrshdmr(potential,dvrindex,natpot%ndof,natpot%nmode, &
        !     nsample,natpot%nspfdof,natpot%spfdof, &
        !     natpot%gdim,natpot%maxspfdof,natpot%subdim, lomppotential)
        call save_rshdmr(natpot%nmode)
     end if

  end if
  nmode = natpot%nmode




  !***************************************************
  !***************************************************
  ! TASK 1: CALCULATE SPP
  !***************************************************
  !***************************************************

  if (.not. lloadevecs) then
     if (nidxfile(mc_task_spp) .eq. 0 .and. lsamecoeffspp) then
        call loadruntask(mc_task_coeff, dvr, nsample)
     else
        call loadruntask(mc_task_spp, dvr, nsample)
     end if


     ! does nothing if already tested
     ! or of no OpenMP or if "no-OMPPotenttial"
     ! is set for PES in RUN-SECTION
     call test_pes_threadsafe(dvrindex, ndof, min(10000,nsample), ilog)
  end if

  swsum2 = natpot%rvpotdim ! compatibility

  ! calculate SPP: generates reduced densities and optains
  ! SPP as eingenfunctions. densities are temporarily allocated
  ! and deallocated on exit. only the relevant SPP are stored

  if (mpirank.eq.0) then
     call get_timer(tid1, 'vecs')
     call start_timer(tid1)
  end if

  call calcspp(natpot,dvrindex,nsample)

  if (mpirank.eq.0) then
     call stop_timer(tid1)
  end if



  !***************************************************
  !***************************************************
  ! TASK 2: CALCULATE COEFFITIENTS
  !***************************************************
  !***************************************************

  ! free vpot if not needed any more
  if (smethod(mc_task_coeff,1) .ne. mc_complete .and. &
       smethod(mc_task_test,1) .ne. mc_complete ) then
     if (allocated(vpot)) deallocate(vpot)
  end if

  ! load data for calculating the coefficients
  if (.not. lsamecoeffspp) then
     call loadruntask(mc_task_coeff, dvr, nsample)
     ! does nothing if already tested
     ! or of no OpenMP or if "no-OMPPotenttial"
     ! is set for PES in RUN-SECTION
     call test_pes_threadsafe(dvrindex, ndof, min(10000,nsample), ilog)
  end if

  ! calculate the coefficients
  if (mpirank .eq. 0) then
     call get_timer(tid2, 'coeffs')
     call start_timer(tid2)
  end if

  if (smethod(mc_task_coeff,1) .eq. mc_complete) then
     if (mpirank .eq. 0) call calccoeffcmplte(natpot)
  else

     call calccoeff(natpot, nsample, dvrindex, coeff_ext, coeff_extdim)
  end if

  ! WE ARE DONE, EXCEPT SYMMETRIZATION

  !***************************************************
  !***************************************************
  ! TASK 3: TEST THE NATPOT
  !***************************************************
  !***************************************************

  if (smethod(mc_task_coeff,1) .ne. mc_complete .and. &
       lsymmetries) then

     if (mpirank.eq.0) then
        write(iout,'(a)') "ERRORS OBTAINED " &
             //"WITH COEFF-SAMPLING " &
             //"BEFORE COEFF-SYMMETRIZATION:"
     end if

     call stat_current_traj(natpot, iout, .true.)

     if (nidxfile(mc_task_test) .gt. 0) then
        call loadruntask(mc_task_test,dvr,nsample)

        if (mpirank.eq.0) then
           write(iout,'(a)') "ERRORS OBTAINED WITH " &
                //"TEST-SAMPLING " &
                //"BEFORE COEFF-SYMMETRIZATION:"
        end if

        call stat_current_traj(natpot, iout, .true.)
     end if

  end if

  if (mpirank .eq. 0 .and. lsymmetries) then
     fname = trim(name) // "/natpot_unsym"
     call exportMCTDH(natpot, dvr, fname, lovwr)
  end if

  !***************************************************
  !***************************************************
  ! SYMMETRIZE THE NAPOT
  !***************************************************
  !***************************************************
  if (lsymmetries) then
     call symmetrizenatpot(natpot, coeff_ext, coeff_extdim)
     deallocate(coeff_ext)

     if (mpirank.eq.0) call testnpotsymmetry(natpot, 50)
     close(isymlog)
  end if


  ! we are done. now write the natopot file
  if (mpirank .eq. 0) then
     call stop_timer(tid2)
     fname = trim(name) // "/natpot"
     call exportMCTDH(natpot, dvr, fname, lovwr)
!     fname = trim(name) // "/natpot_t"
!     call save_natpot_t(natpot, fname, lovwr)
  end if

  if (mpirank .eq. 0) then
     call get_timer(tid3, 'pfiterr')
     call start_timer(tid3)
  endif

  ! first we do a test with the sampling we used for the
  ! coefficients.
  if (smethod(mc_task_coeff,1) .ne. mc_complete) then



     if (mpirank.eq.0) then
        if (lsymmetries) then
           write(iout,'(a)') "ERRORS OBTAINED WITH " &
                //"COEFF-SAMPLING AFTER COEFF-SYMMETRIZATION:"
        else
           write(iout,'(a)') "ERRORS OBTAINED WITH " &
                //"COEFF-SAMPLING:"
        end if
     end if

     if (nidxfile(mc_task_coeff) .eq. 0 .and. lsamecoeffspp) then
        call loadruntask(mc_task_spp,dvr,nsample)
     else
        call loadruntask(mc_task_coeff,dvr,nsample)
     end if
     call stat_current_traj(natpot, iout, .true.)

  else

     ! this is only rank 0
     if (mpirank .eq. 0) then
        write(iout,'(a)') "ERRORS OBTAINED WITH COMPLETE " &
             //"COEFF-SAMPLING:"
     end if

     call stat_complete(natpot, vpot, iout, .true.)

  end if

  if (mpirank .eq. 0) call stop_timer(tid3)


  ! if test is not wanted we are done here.
  if (lskiptest) then
     if (allocated(mpi_chunksizes)) deallocate(mpi_chunksizes)
     if (allocated(vpot)) deallocate(vpot)
     call deallocate_natpot_t(natpot)
     return
  end if


  ! finally we estimate the mean and RMS error of the natpot
  ! using the test trajectory
  ! this is fast usually

  if (mpirank .eq. 0) call start_timer(tid3)

  if (smethod(mc_task_test,1) .eq. mc_complete) then

     if (mpirank .eq. 0) then
        write(iout,'(a)') "ERRORS OBTAINED WITH " &
             //"COMPLETE-SAMPLING:"
     end if

     call stat_complete(natpot, vpot, iout, .true.)


  elseif (.not.lskiptest) then

     if (mpirank .eq. 0) then
        write(iout,'(a)') "ERRORS OBTAINED WITH " &
             //"TEST-SAMPLING:"
     end if

     ! test the complete trajectory
     call loadruntask(mc_task_test,dvr,nsample)
     call stat_current_traj(natpot, iout, .true.)

     ! test all single files
     if (nidxfile(mc_task_test) .gt. 1) then
        call Open_stat_file("natpot-statistics")
        call stat_task_files(mc_task_test, dvr, natpot, istat)
        close(istat)
     end if

  end if

  if (mpirank .eq. 0) call stop_timer(tid3)

  ! final clean up
  if (allocated(mpi_chunksizes)) deallocate(mpi_chunksizes)
  if (allocated(vpot)) deallocate(vpot)

  call deallocate_natpot_t(natpot)

  nullify(phops)
  nullify(pdvr)

end subroutine runmcpfit



! checkmcpf

subroutine checksamplingcounts(natpot)
  use mcpfitparams
  use mcsampling
  use natpottype

  use global
  use pglobal
  use channels
  use logdat

  implicit none
  type(natpot_t), intent(IN)   :: natpot

  integer                      :: m,minsamples
  character(len=c1)            :: strgdim,strsmple


  routine = "checksampling"

  if (.not.natpot%initialized) then
     message = "natpot not initialized."
     call errormsg
  end if


  ! check if files contain enough points
  ! or get sampling number from file if required
  write(strgdim,'(i10)') natpot%modcdim
  strgdim = adjustl(strgdim)

  if (ntotsample(mc_task_coeff,0) .lt. natpot%modcdim) then
     write(strsmple,'(i10)') ntotsample(mc_task_coeff,0)
     strsmple = adjustl(strsmple)
     message = "No. of samples-coeff is smaller than the number of" &
          //" coefficients. Current value: "//trim(strsmple) &
          // ", should be at least: "//trim(strgdim)//"."
     call warnmsg()

     if (invertmethod .ne.  mc_inv_pseudo) then
        message = "If no. of samples-coeff is smaller than the number of" &
             //" coefficients one must use invert-methof=pseudo-invert."
        call errormsg

     end if

  end if


  if (lloadevecs) return

  minsamples = 0
  do m=1,natpot%nmode
     if (m .ne. natpot%modc) then
        if (natpot%potdim(m) .gt. minsamples) then
           minsamples = natpot%potdim(m)
        end if
     end if
  end do


  if (ntotsample(mc_task_spp,0) .lt. 10*minsamples) then
     write(strsmple,'(i10)') ntotsample(mc_task_spp,0)
     strsmple = adjustl(strsmple)
     message = "No. of samples-spp is nut much larger than" &
          //"the No of SPP. Current value: "//trim(strsmple)
     call warnmsg()
  end if

  if (ntotsample(mc_task_spp,0) .lt. minsamples) then
     write(strsmple,'(i10)') ntotsample(mc_task_spp,0)
     strsmple = adjustl(strsmple)
     write(strgdim,'(i10)') minsamples
     strgdim = adjustl(strgdim)
     message = "No. of samples-spp is smaller than the number of" &
          //" SPP. This will lead to singular densities. Current value: "//trim(strsmple) &
          // ", must me at least: "//trim(strgdim)//"."
     call errormsg()
  end if


end subroutine checksamplingcounts

