


module tensortype

  implicit none

  integer, parameter, private       :: lm = 16

  type, abstract :: htm_t
     ! empty
  end type htm_t

  type :: tensor_t
     logical                        :: initialized
     integer                        :: ndof
     integer                        :: nmode
     integer, allocatable           :: gdim(:)
     integer, allocatable           :: subdim(:)
     integer, allocatable           :: nspfdof(:), spfdof(:,:)
     integer                        :: maxspfdof
     integer, allocatable           :: potdofmap(:,:)
     integer, allocatable           :: dofspf(:)
     character(len=lm), allocatable :: modelabel(:)
     integer                        :: vpotdim
     double precision               :: rvpotdim


     integer                        :: ortdim
     integer, allocatable           :: zort(:)
     double precision, allocatable  :: ort(:)

     logical, allocatable           :: loned(:)
     double precision,allocatable   :: oned(:)



  end type tensor_t


contains

  subroutine deallocate_tensor_t(tensor)
    implicit none

    class(tensor_t), intent(INOUT)  :: tensor

    if (allocated(tensor%gdim)) deallocate(tensor%gdim)
    if (allocated(tensor%subdim)) deallocate(tensor%subdim)
    if (allocated(tensor%nspfdof)) deallocate(tensor%nspfdof)
    if (allocated(tensor%spfdof)) deallocate(tensor%spfdof)
    if (allocated(tensor%potdofmap)) deallocate(tensor%potdofmap)
    if (allocated(tensor%dofspf)) deallocate(tensor%dofspf)
    if (allocated(tensor%modelabel)) deallocate(tensor%modelabel)
    if (allocated(tensor%zort)) deallocate(tensor%zort)
    if (allocated(tensor%ort)) deallocate(tensor%ort)
    if (allocated(tensor%loned)) deallocate(tensor%loned)
    if (allocated(tensor%oned)) deallocate(tensor%oned)


    tensor%ndof = 0
    tensor%nmode = 0
    tensor%vpotdim = 0
    tensor%rvpotdim = 0.d0
    tensor%ortdim = 0
    tensor% maxspfdof = 0
    tensor%initialized = .false.

  end subroutine deallocate_tensor_t


  logical function iscompatible(t1,t2)
    implicit none
    class (tensor_t), intent(in) :: t1,t2
    logical   :: lcomp
    integer   :: m,n


    lcomp = .true.
    lcomp = lcomp .and. t1%ndof .eq. t2%ndof
    if (.not.lcomp) goto 1000
    lcomp = lcomp .and. t1%nmode .eq. t2%nmode
    if (.not.lcomp) goto 1000
    lcomp = lcomp .and. all(t1%gdim .eq. t2%gdim)
    if (.not.lcomp) goto 1000
    lcomp = lcomp .and. all(t1%subdim .eq. t2%subdim)
    if (.not.lcomp) goto 1000
    lcomp = lcomp .and. all(t1%nspfdof .eq. t2%nspfdof)
    do m=1,t1%nmode
       do n=1,t1%nspfdof(m)
          lcomp = lcomp .and. t1%spfdof(n,m) .eq. t2%spfdof(n,m)
          if (.not.lcomp) goto 1000
       end do
    end do
    lcomp = lcomp .and. all(t1%potdofmap .eq. t2%potdofmap)
    if (.not.lcomp) goto 1000
    lcomp = lcomp .and. all(t1%dofspf .eq. t2%dofspf)
    if (.not.lcomp) goto 1000
    lcomp = lcomp .and. all(t1%modelabel .eq. t2%modelabel)
    if (.not.lcomp) goto 1000
!!$   lcomp = lcomp .and. all(t1%zort .eq. t2%zort)
!!$   if (.not.lcomp) goto 1000
!!$   lcomp = lcomp .and. all( abs(t1%ort - t2%ort) .le. 1.d-7 )
!!$   if (.not.lcomp) goto 1000

1000 continue

!!$    print*,"ndof    "  ,t1%ndof .eq. t2%ndof
!!$    print*,"nmode   "  ,t1%nmode .eq. t2%nmode
!!$    print*,"gdim    "  ,all(t1%gdim .eq. t2%gdim)
!!$    print*,"subdim  "  ,all(t1%subdim .eq. t2%subdim)
!!$    print*,"nspfdof "  ,all(t1%nspfdof .eq. t2%nspfdof)
!!$    print*,"spfdof  "  ,all(t1%spfdof .eq. t2%spfdof)
!!$    print*, t1%spfdof
!!$    print*, t2%spfdof
!!$    print*,"potdofma"  ,all(t1%potdofmap .eq. t2%potdofmap)
!!$    print*,"dofspf  "  ,all(t1%dofspf .eq. t2%dofspf)
!!$    print*,"modelabe"  ,all(t1%modelabel .eq. t2%modelabel)
!!$    print*,"zort    "  ,all(t1%zort .eq. t2%zort)
!!$    print*,"ort     "  ,all( (t1%ort - t2%ort) .le. 1.d-7 )
!!$
!!$    do i=n,min(t1%ortdim,t2%ortdim)
!!$       print*, "ort n", n, t1%ort(n), t2%ort(n)
!!$    end do

    iscompatible = lcomp

  end function iscompatible



  ! write a string representation of the mode
  ! into 'string'
  subroutine getModeLabelsStr(tensor,mode,string)
    implicit none
    class (tensor_t), intent(IN)   :: tensor
    integer, intent(IN)            :: mode
    character(len=80), intent(OUT) :: string
    integer                        :: n,f, ilbl

    string = " "
    ! return "None" in string for not initialized
    if (.not.tensor%initialized) then
       string = "-None-"
       return
    end if

    if (mode .gt. tensor%nmode .or. mode .le. 0) then
       write(string,'(i8)') mode
       write(*,*) "Non-exosting mode: "//trim(string)
       stop 1
    end if

    ilbl = 1
    do n=1,tensor%nspfdof(mode)
       f = tensor%spfdof(n,mode)
       if (n.ne.tensor%nspfdof(mode)) then
          string(ilbl:) = trim(tensor%modelabel(f))//","
          ilbl = ilbl + len_trim(tensor%modelabel(f))+1
          if (ilbl .gt. 80) exit
       else
          string(ilbl:) = trim(tensor%modelabel(f))
       end if
    end do

  end subroutine getModeLabelsStr



end module tensortype
