! -*- f90 -*-
!
! Template for a custom implementation of
! a potential using a coordinate vector
!
! See also userpotidx.F90 for an index based
! implementation of the PES routine

module UserPotParams
  implicit None

  ! keep the iup variable. See '-up' command line option
  integer       :: iup

  logical, save :: UserPotInitialized = .false.

  ! ADD YOUR PARAMETERS HERE

contains

  subroutine InitUserPotParams()
    implicit None

    if (UserPotInitialized) return

    ! INITIALIZE YOUR PARAMETERS HERE.
    ! FOR FILE IO USE UNIT 50
    ! AND CLOSE IT WHEN DONE
    ! (see 'channels.inc').

    UserPotInitialized = .true.

  end subroutine InitUserPotParams


end module UserPotParams



! This a template function body. it can be used
! to implement a custom PES routine.
! Do not rename or change the interface.

! Input is the coordinate vector 'coord'
! to evaluate the potential.
! The coordinate vector is
! ordered as the DOF are defined in the
! PRIMITIVE-BASIS-SECTION.
!
! Note: The function UserPot is usually called
! in parallel if compiled with the
! OpenMP flag. I.e., the function is called from
! inside an "omp parallel" secion.
! Therefore, the function must be
! thread safe. See also the documentation
! on how to suppress parallel execution
! when compiled with OpenMP flag.
double precision function UserPot(coord,ndof)
  use UserPotParams
  implicit None
  integer, intent(IN)          :: ndof
  double precision, intent(IN) :: coord(ndof)

  ! The routine should initialize
  ! upon the first call
  if (.not. UserPotInitialized) then

     ! MAKE SURE THE INITIALISATION
     ! IS THREAD SAFE!

     !$omp critical
     call InitUserPotParams()
     !$omp end critical
  end if

  ! change this top return a potential
  ! given the coordinate vector 'coord'
  UserPot = 0.d0

end function UserPot

