!    -*- f90 -*-

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Implement your symmetry operations here
!
! Input:
!     qin  : original coordinate values
!     ndof : number of degrees of freedom
!    label : the symmetry operation to
!            perform, as labeled in the
!            SYMMETRY-SECTION
!
! Output
!     qout : equivalent coordinate values
!            according to the symmetry operation
!            given in the label.
!
! Note: The order of 'qin' and 'qout' is the same
!       as in the PRIMITIVE-BASIS-SECTION
!
!       In the SYMMETRY-SECTION a different order
!       can be used to specify the symmetry operations.
!       When this routine is called, the symmetry operations.
!       have already been internally rearranged
!       to the order of the primitive basis section.
!
!       It is not negessary to implement all
!       symmetry operations for all coordinates.
!       It is sufficient to implement them
!       only for those DOF and symmetries where the flag
!       "usersym" is set in the SYMMETRY-SECTION
!       Remember, that if 'usersym' is used in the
!       SYMMETRY-SECTION, it has to be set for all
!       DOF in a mode. Otherwise an error is produced.
!       so the symmetry operations must always be
!       implemented for complete modes.
!
!       If you do not implement the operations for
!       all coordinates, then make sure NOT to assign
!       anything incorrect to unused 'qout' entries.
!       Leave these entries as they are or assign
!       the correct new coordinates. They are used
!       to detect the mappings of the symmetry
!       operations so their output must be correct.
!       or unchanged.
!
! Note also:
!       MC-Potfit is in its essence grid based.
!       If "auto-extgrid" is *not* set in the
!       RUN-SECTION and any symmetry operation
!       implemented below does not stay on
!       the primitive grid (tolerance: 1.d-7) an error
!       will be produced.
!
!       If "auto-extgrid" is set  in the RUN-SECTION
!       the symmetry operations below will be used
!       to automatically create an extended primitive grid.
!       for all modes for which the symmetry operations
!       below leave the original grid. This
!       should be carefully checked. See output files
!       "extgrid_mode_<m>" in the name directory.
!
!
! 08/2017 MS
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

subroutine GetUserSym(qin,qout,ndof,label)

  use global
  use logdat

  implicit none
  integer, intent(IN)             :: ndof
  character(len=*), intent(IN)    :: label
  double precision, intent(IN)    :: qin(ndof)
  double precision, intent(INOUT) :: qout(ndof)

  ! your variables here..

  select case(label)
  case("E")
     ! the first symmetry operation is
     ! always the identity "E".
     qout = qin

!  case("yourlabel")
!    ... Add your code here


  case default
     routine = "GetUserSym"
     message = "Symmetry operation not implemented: "//trim(label)//"."
     call errormsg
  end select


end subroutine GetUserSym
