!#######################################################################
!                                                                     
!              MCTDH  module EINGUTIL
!                                                                     
! MCTDH module containing utility subroutines for EINGABE to open the 
! input and  output files and to read the input data.  
!
! includes the subroutines: openin, openlog, and cpinput.
!
!#######################################################################

      module eingabemod

use decimal, only: long, dop
use global
use dglobal
use versions
use cdate
use logdat
use channels
use keyunitsmod
use datenmod
use dirdyn
use gapmod
use op1lib,  only: zerovxi

! can only fill variables, not arrays in following modules as the array 
! sizes are allocated later
use psidef
use lctmod
use runpropmod
use griddatmod
use operdef
use crocoflymod
use dirdyn
use openmpmod
use runinwfmod
use mpidata
use rdopermod
use expectval, only: lnwexpect

!use random, only: seed,seedsize
#ifdef OMP
use omp_lib
#endif

      implicit none

      logical(kind=4) :: lrddvr = .false., &
                         lrdoper = .false., &
                         lrdinwf = .false.
      character(len=32)             :: opername

      private
      public :: eingabe, cpinput

      contains

!***********************************************************************
!
!            Subroutine      EINGABE
!
! Reads the input data from the input file.
! Called from: mctdh
! Calls to: openin, openlog, rdinpf, einfile, einrun, einbas, eininwf,  
!           einint, loginp, ausgab, warnmsg, errormsg,
!           myflush
!
! 03/97 HDM 
! 
!***********************************************************************

      subroutine eingabe

      implicit none

      integer(long)           :: iz,ic,ierr,taskl
      logical(kind=4)         :: lmnd

      logical(kind=4)                 :: linperr
      logical(kind=4)                 :: lexist,ltimexist
      character(len=c5)               :: filename,line
      integer(long)                   :: ilbl,jlbl
      character(len=c2)               :: rdtag

!-----------------------------------------------------------------------
! open the inputfile : argument.inp
!-----------------------------------------------------------------------
      call openin
      
!-----------------------------------------------------------------------
! read run-section
!-----------------------------------------------------------------------
      rewind(iin)
      iz=0
      linperr=.false.
   10 continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .gt. 0) then
         routine = 'Eingabe, run-section'
         message = inptit
         call errormsg
      endif

      if (keyword(1).eq.'run-section') then
         call einrun
      elseif (keyword(1).eq.'end-input') then
         routine='Eingabe, run-section'
         message='No RUN-SECTION in input file'
         call errormsg
      else
         go to 10
      endif

!-----------------------------------------------------------------------
! Make changes to input based on command-line options
!-----------------------------------------------------------------------
      call readopts(lmnd)
      
!-----------------------------------------------------------------------
! Check that the name directory exists.
! The inquire command does not work if applied to directories.
! We thus open a scratch file to find out whether the name-directory
! exists.
!-----------------------------------------------------------------------
      lexist=.false.
      open(itmpin,file=name(1:laenge)//'/scratch',err=666)
      close(itmpin,status='delete')
      lexist=.true.
  666 continue

      if (lmnd .and. lexist) then
         write(6,'(2a)') 'Name-directory already exists: ', name(1:laenge)
         call stopnow
      else if(lmnd) then
         inptit='mkdir '//name(1:laenge)
         call excmd2(ierr, inptit)
         if (ierr.ne.0) then
            write(6,'(2a)') '### ERROR: ', &
            'Invalid path for name-directory (-mnd option). ###'
            call stopnow
         endif
         write(6,'(2a)') 'Created name-directory: ',name(1:laenge)
      else if (.not. lexist) then
         write(6,'(a)') &
         '---------------------------------------------------------'
         write(6,'(a)') ' ################   ERROR   ##################'
         write(6,'(a)') ' There is no name-directory! The directory:'
         write(6,'(1x,a)') name(1:laenge)
         write(6,'(a)') ' does not exist! (or has no write permission)'
         write(6,'(2a)')' One may use the option -mnd to create the', &
                        ' name-directory.'
        write(6,'(a)') &
           '-------------------------------------------------------'
         inptit='rm -f /tmp/QUANTICS_PID >& /dev/null'
         call excmd(inptit)
         call stopnow
      endif

!-----------------------------------------------------------------------
! open log file
!-----------------------------------------------------------------------
      call openlog

!-----------------------------------------------------------------------
! Stop now if error in run-section 
!-----------------------------------------------------------------------
      if (lwarn) then
         write(ilog,'(a)') '****** Error in RUN-SECTION. STOP ******'
         write(ilog,'(/,a)') &
      ' This error is not written to the log. See screen output for details'
         write(6,'(a)') '****** Error in RUN-SECTION. STOP ******'
         call stopnow
      endif

!-----------------------------------------------------------------------
! Check no. of cores requested does not exceed available
!-----------------------------------------------------------------------
#ifdef OMP
        maxompthread = omp_get_max_threads()
#else
        maxompthread = 1
#endif
       if (lompthread .and. (ompthread .gt. maxompthread)) then
          write(message,'(2(a,i5))') &
             'Too many openmp threads. Asked for:',ompthread, &
                 '   Maximum:', maxompthread
          call errormsg
      endif

!-----------------------------------------------------------------------
! Select tasks to be calculated
!-----------------------------------------------------------------------
      call settask
      
!-----------------------------------------------------------------------
! check necessary sections present
!-----------------------------------------------------------------------
      call chkinp

!-----------------------------------------------------------------------
! set paths to files
!-----------------------------------------------------------------------
      call setpaths

!-----------------------------------------------------------------------
! Print Task to Log file. Check consistency.
!-----------------------------------------------------------------------
      inptit = 'Task: '
      taskl=6
      if(irundvr .eq. 1) then
         inptit = inptit(1:taskl)//'gendvr '
         taskl=taskl+7
      else if(irundvr .eq. 2) then
         inptit = inptit(1:taskl)//'rddvr '
         taskl=taskl+6
      endif
      if(irunoper .eq. 1) then
         if (lrunpes) then
            inptit = inptit(1:taskl)//'genpes '
            taskl=taskl+7
         else 
            inptit = inptit(1:taskl)//'genoper '
            taskl=taskl+8
         endif
      else if(irunoper .eq. 2) then
         inptit = inptit(1:taskl)//'rdoper '
         taskl=taskl+7
      endif
      if(iruninwf .eq. 1) then
         inptit = inptit(1:taskl)//'geninwf '
         taskl=taskl+8
      else if(iruninwf .eq. 2) then
         inptit = inptit(1:taskl)//'rdinwf '
         taskl=taskl+7
      endif
      if(irunprop .eq. 1) then
         if (lrlx) then
            inptit = inptit(1:taskl)//'relaxation '
            taskl=taskl+11
         else if(lrundiag) then
            inptit = inptit(1:taskl)//'diagonalisation '
            taskl=taskl+16
         else if(lrunshzagreb) then
            inptit = inptit(1:taskl)//'surface hopping '
            taskl=taskl+16
         else
            inptit = inptit(1:taskl)//'propagation '
            taskl=taskl+12
         endif
      endif
      if(lcont) then
         inptit = inptit(1:taskl)//'continuation '
         taskl=taskl+13
      endif
      if(test) then
         inptit = inptit(1:taskl)//'test '
         taskl=taskl+5
      endif

      write(ilog,'(/a)') inptit(1:taskl-1)

      If(lcont.and.test) then
         routine = 'Eingabe, consistency check'
         message='One must not give "continuation" and "test"'
         call errormsg
      endif
   
!-----------------------------------------------------------------------
! for a continuation run look for timing file and set ltime
! otherwise if overwrite delete an existing timing file
!-----------------------------------------------------------------------
      if (lcont) then
         filename=name(1:laenge)//'/timing'
         inquire(file=filename,exist=ltimexist)
         if (ltimexist) then
            ltime=.true.
         else if (lovwr .and. ltimexist) then
            ltime=.false.
            line="rm "//filename(1:laenge+7)
            call excmd(line)
         endif
      endif

!-----------------------------------------------------------------------
! check continuation possible (e.g. if files are present and new enough)
!-----------------------------------------------------------------------
      if (lcont) then
         filename=name(1:laenge)//'/restart'
         open(irst,file=filename,form='unformatted',status='old',err=900)

         read(irst) filever(irst)
         if (filever(irst) .lt. 11.0_dop) then
            routine='Eingabe'
            message= &
                'File format too old to form the basis for a continuation run.'
            call errormsg
         endif

! get propagation type for continuation
         do
! read next record - error means it is not a string so ignore and go on to next
  710       continue
            read(irst,err=710,end=750) rdtag
            if (rdtag(1:1) .ne. '<') cycle

            ilbl=2
            jlbl=index(rdtag,'>')-1
            if (rdtag(ilbl:jlbl) .eq. 'proptask') then
               read(irst) proptask
            else if (rdtag(ilbl:jlbl) .eq. 'no integrator info') then
               routine='Eingabe'
               message= 'No integrator information in restart file.'
               call errormsg
            else if (rdtag(ilbl:jlbl) .eq. 'end integrator info') then
               exit
            endif
         enddo

  750    continue
         close(irst)

      end if

!-----------------------------------------------------------------------
! end reading input sections
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! After any call of 'warnmsg' the logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr 
      lwarn  =.false.

!-----------------------------------------------------------------------
! Do not open stop-file if no propagation/relaxation. 
!-----------------------------------------------------------------------
      if (frunlevel .lt. 4) lstopf = .false.

!-----------------------------------------------------------------------
! copy input file to data directory
!-----------------------------------------------------------------------
      call cpinput

      return
  900 routine='Eingabe'
      message='Continuation asked for but restart file not present.'
      call errormsg
      end subroutine eingabe

!#######################################################################
! 
!             MCTDH subroutine EINRUN.F
!
!  01/98 AJ
!
! Einrun reads the RUN-SECTION. This gets the calculation to be run
! and basic options. 
!
!#######################################################################

      subroutine einrun

      implicit none

      integer(long)                 :: iz
      integer(long)                 :: i,j,iz0,ic,ierr,ilbl,&
                                       f1,f2,n,ifreeze,istrt,&
                                       ntout,ntpsi
      real(dop)                     :: arlxshift,rnum
      logical(kind=4)               :: lpsisp,lpsicm,lpsinat,lexist
      logical(kind=4)               :: lmpioff=.false.

!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
      routine = 'read RUN-SECTION  (einrun)'
      iz0     = 0
      opername= 'oper '
      lpsisp  = .false.
      lpsicm  = .false.
      lpsinat = .false.
      lquadphi= .false.
      lbackr  = .false.
      lskip1dav=.false.
      loctpc  = .false.
      rlaxnum = -1
      arlxshift= 0.d0
      lrddvr  = .false.
      lrdoper = .false.
      lrdinwf = .false.

      ntout = -1
      ntpsi = -1
     
      istrt=0

!-----------------------------------------------------------------------
! for natfast algorithm
!-----------------------------------------------------------------------
      fastn = 0
      lfastall=.false.
      ncutn = 0
      lncutall=.false.

!-----------------------------------------------------------------------
! READ KEYWORDS: 
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (keyword(i).eq.'end-run-section') then
         goto 20

      else if( keyword(i) .eq. 'name' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with name keyword'
            call warnmsg
         else
            if (laenge .eq. 0) then
               name = keyorig(i+2)
               laenge = lc(i+2)
               if (laenge .eq. 0) then
                  message = ' No path name specified'
                  call wrline(zeroi,iz,iz0)
               end if
               call abspath(name,laenge)
            endif
            i=i+2
         endif

!
! keywords to select programs to run. This is done by setting final run level.
! Dependencies on lower level programs can then be generated.
!
      else if (keyword(i)(1:4) .eq. 'prop')  then
         frunlevel=max(frunlevel,4)
      else if( keyword(i) .eq. 'test' )  then 
         frunlevel=max(frunlevel,4)
         test = .true.
      else if( keyword(i) .eq. 'relax' .or.  &
            keyword(i) .eq. 'relaxation')  then 
         frunlevel=max(frunlevel,4)
         lrlx = .true.
         if (keyword(i+1).eq.'=') then
            if (keyword(i+2).eq.'unmodified') then
               lrlxun=.true.
               i=i+2
            else if (keyword(i+2).eq.'follow') then
               rlaxnum = 920
               i=i+2
            elseif (keyword(i+2).eq.'lock') then
               rlaxnum = 930
               i=i+2
            else
               read(keyword(i+2),*,iostat=ierr) rlaxnum
               if(ierr.ne.0 .or. rlaxnum.lt.0 .or. rlaxnum.gt.900) then
                  message = '"follow" or integer argument .ge.0'// &
                  ' and .le.900 required for relax. Not: '
                  ilbl    = index(message,':')+2
                  message = message(1:ilbl)//keyword(i+2)
                  call warnmsg
               endif
               i=i+2
            endif
  35        if (keyword(i+1).eq.',') then
               if (keyword(i+2).eq.'full') then
                  rlaxnum = rlaxnum + 10000
               elseif (keyword(i+2).eq.'skip1dav') then
                  lskip1dav = .true.
               elseif (keyword(i+2).eq.'ortho') then
                  lortho = .true.
               elseif (keyword(i+2).eq.'quad' ) then
                  lquad = .true.
               elseif (keyword(i+2).eq.'olsen' ) then
                  lolsen = .true.
               elseif (keyword(i+2).eq.'quadphi' ) then
                  lquadphi=.true.
               elseif (keyword(i+2)(1:4).eq.'back' ) then
                  lbackr=.true.
               elseif (keyword(i+2)(1:1).eq.'c' ) then
                  read(keyword(i+2)(2:3),*,iostat=ierr) phicount
                  if(ierr.ne.0) then
                     message = 'Expecting cX, where X is a integer.'// &
                     ' Not: '
                     ilbl    = index(message,':')+2
                     message = message(1:ilbl)//keyword(i+2)
                     call warnmsg
                  endif
               else
                  message = 'Expecting "full", "ortho",'// &
                  ' "olsen", "quad", "quadphi" or "backrot" but not: '
                  ilbl    = index(message,':')+2
                  message = message(1:ilbl)//keyword(i+2)
                  call warnmsg
               endif
               i=i+2
               goto 35
            endif
         endif

      else if( keyword(i) .eq. 'gendvr' )  then 
         frunlevel=max(frunlevel,1)
      else if( keyword(i) .eq. 'genoper' )  then 
         frunlevel=max(frunlevel,2)
         if (keyword(i+1) .eq. '=') then
            i=i+2
            opername='oper_'//trim(keyorig(i))
         else
            opername='oper'
         endif
      else if( keyword(i) .eq. 'genpes' )  then 
         lrunpes=.true.
         frunlevel=max(frunlevel,2)
         ltime  =.false.
      else if( keyword(i) .eq. 'gengmat' )  then 
         if (keyword(i+1) .ne. '=') then
             message='missing "=" after gengmat'
             call wrline (zeroi,iz,iz0)
         else
            i=i+2
            read(keyword(i),*,iostat=ierr) f1
            if(ierr.ne.0) then
               message = 'Expecting integer as first argument'// &
               ' of gengmat. Not: '
               call wrline (i,iz,iz0)
            endif
            i=i+1
            if(keyword(i) .eq. ',') i=i+1
            read(keyword(i),*,iostat=ierr) f2
            if(ierr.ne.0) then
               message = 'Expecting integer as second argument'// &
               ' of gengmat. Not: '
               call wrline (i,iz,iz0)
            endif
         endif
         if(.not.lrunpes) then  ! Do not overwrite options
            gmatf1=f1
            gmatf2=f2
            lrunpes=.true.
            frunlevel=max(frunlevel,2)
            ltime  =.false.
         endif
      else if( keyword(i) .eq. 'geninwf' )  then 
         frunlevel=max(frunlevel,3)

      else if( keyword(i)(1:4) .eq. 'cont' )  then 
         lcont = .true.
         frunlevel=max(frunlevel,4)
         if (keyword(i+1) .eq. '=') then
            i=i+2
            if( keyword(i)(1:3) .eq. 'int' )  then
               leinint = .true.
            else
               message = 'Expecting "integrator", not :'
               call wrline (i,iz,iz0)
            endif
         endif
!
! Direct Dynamics keywords
!
      elseif (keyword(i) .eq. 'direct' .or. keyword(i) .eq. 'tddirect') then
         ldd = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            if (keyword(i) .eq. 'jacobi') then
               ddcoo = 0
            else if (keyword(i) .eq. 'nmodes') then
               ddcoo = 1
            else if (keyword(i) .eq. 'cartesian') then
               ddcoo = 2
            else
               message = 'Unknown argument specified with keyword : '
               call wrline(i,iz,iz0)
            endif
         endif
         if(keyword(i+1).eq.',')then
            i=i+2
            if(keyword(i).eq.'gausspot')then
               lgausspot = .true.
            else
               message = 'Unknown argument specified with keyword : '
               call wrline(i,iz,iz0)
            endif
         endif

      elseif (keyword(i) .eq. 'dd-gb') then
         lddmctdh = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            if (keyword(i) .eq. 'jacobi') then
               ddcoo = 0
            else if (keyword(i) .eq. 'nmodes') then
               ddcoo = 1
            else if (keyword(i) .eq. 'cartesian') then
               ddcoo = 2
            else
               message = 'Unknown argument specified with keyword : '
               call wrline(i,iz,iz0)
            endif
! Test keywords obsolete - use gausspot in eindirdyn
            if(keyword(i+1).eq.',')then
               i=i+2
               if(keyword(i).eq.'test')then
                  lgaptest = .true.
               else
                  message = 'Unknown argument specified with keyword : '
                  call wrline(i,iz,iz0)
               endif
               if(keyword(i+1).eq.',')then
                  i=i+2
                  if(keyword(i).eq.'salicyl')then
                     iddmtest=1
                  else if(keyword(i).eq.'malon')then
                     iddmtest=2
                  else if(keyword(i).eq.'double')then
                     iddmtest=3
                  else if(keyword(i).eq.'single')then
                     iddmtest=4
                  else if(keyword(i).eq.'but')then
                     iddmtest=5
                  else if(keyword(i).eq.'form')then
                     iddmtest=6
                  else
                     message = 'Unknown argument specified with keyword : '
                     call wrline(i,iz,iz0)
                  endif
               endif
            endif
         endif

!      elseif (keyword(i) .eq. 'ddmorder') then
!         if (keyword(i+1) .eq. '=') then
!            i=i+2
!            read(keyword(i),*,iostat=ierr)ddmorder
!            if (ierr .ne. 0) then
!               message = &
!                 'Argument for ddmorder must be an integer. Not :'
!               ilbl=index(message,':')+2
!               message=message(1:ilbl)//keyword(i)
!               call warnmsg
!            endif
!         else
!            message='ddmorder must be followed by ='
!            call warnmsg
!         endif

      elseif (keyword(i) .eq. 'maxdbsize') then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with maxdbsize keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) dbmemdim
            if (ierr .ne. 0) then
               message = &
                 'Argument for maxdbsize must be integer. Not :'
               ilbl=index(message,':')+2
               message=message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
         endif
         i=i+2

      else if (keyword(i) .eq. 'ngwp') then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with ngwp keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) ngwp
            if (ierr .ne. 0) then
               message = &
                 'Argument for ngwp must be integer. Not :'
               ilbl=index(message,':')+2
               message=message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
            if (keyword(i+1) .eq. ',') then
               if (keyword(i+2).eq.'multi') then
                  lmult = .true.
               else
                  message = 'Unknown argument for ngwp must be integer. Not :'
                  ilbl=index(message,':')+2
                  message=message(1:ilbl)//keyword(i+2)
                  call warnmsg
               endif
               i=i+2
            else 
               lmult = .false.
            endif
         endif

      else if (keyword(i) .eq. 'partition') then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with partition keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) npartgwp
            if (ierr .ne. 0) then
               message = &
                 'Argument for partition must be integer. Not :'
               ilbl=index(message,':')+2
               message=message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
         endif
         i=i+2
!
! End of Direct Dynamics keywords
!
      elseif (keyword(i) .eq. 'gwptrj') then
         lgwptrj  = .true.
         psitype = 4
         gwpphase = -1
         if (keyword(i+1) .eq. '=') then
            i=i+2
            if (keyword(i) .eq. 'nmodes') then
               ddcoo = 1
            else if (keyword(i) .eq. 'cartesian') then
               ddcoo = 2
! following is for non-cartesian operators that do not need a trafo
            else if (keyword(i) .eq. 'notrafo') then
               ddcoo = 2
            else
               message = 'Unknown argument specified with keyword : '
               call wrline(i,iz,iz0)
            endif
         else
            ddcoo = 2
         endif

      elseif (keyword(i) .eq. 'gentrj') then
         lgentrj  = .true.

      else if( keyword(i)(1:7) .eq. 'precon2' )  then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) precon2
            if(ierr.ne.0) then
               message = 'Error reading precon2 :'
               call wrline(i,iz,iz0)
            endif
            if(precon2.le.0) then
               write(6,'(/2a,i9)') 'precon2 must be .gt. 0 .', &
                    ' precon2=', precon2
               message = 'Illegal value of precon2 :'
               call wrline(i,iz,iz0)
            endif
            if(keyword(i+1).eq.',') then
               i=i+2
               if(keyword(i).eq.'olsen') then
                  lolsen = .true.
               else
                  message='Expecting "olsen" after precon, but not: '
                  ilbl   = index(message,':')+2
                  message= message(1:ilbl)//keyword(i)
                  call warnmsg
               endif
            endif
         endif

      else if( keyword(i)(1:7) .eq. 'precon ' )  then
         precon=300
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) precon
            if(ierr.ne.0) then
               message = 'Error reading precon :'
               call wrline(i,iz,iz0)
            endif
            if(precon.lt.0) then
               write(6,'(/2a,i9)') 'precon must be .ge. 0 .', &
                    ' precon=', precon
               message = 'Illegal value of precon :'
               call wrline(i,iz,iz0)
            endif
            if(keyword(i+1).eq.',') then
               i=i+2
               if(keyword(i).eq.'olsen') then
                  lolsen = .true.
               else
                  message='Expecting "olsen" after precon, but not: '
                  ilbl   = index(message,':')+2
                  message= message(1:ilbl)//keyword(i)
                  call warnmsg
               endif
            endif
         endif

      else if( keyword(i) .eq. 'rlxunit' )  then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*,err=66) rlxshift
            if (keyword(i+1).eq.',') then
               i=i+2
               rlxunit=trim(keyword(i))
            endif
            goto 67
   66       rlxunit=trim(keyword(i))
            if (keyword(i+1).eq.',') then
               i=i+2
               read(keyword(i),*,iostat=ierr) rlxshift
               if(ierr.ne.0) then
                  message = 'Error reading rlxshift :'
                  call wrline(i,iz,iz0)
               endif
            endif
   67       arlxshift=rlxshift
            call keyeunits(arlxshift,rlxunit,ierr)
            if(ierr.ne.0) then
                message='Not a proper energy unit for "rlx-unit" : ' &
                         //rlxunit 
               call wrline(zeroi,iz,iz0)
            endif
         else
               message = 'No argument follows "rlx-unit" : '
               call wrline(i,iz,iz0)
         endif

      else if( keyword(i) .eq. 'rlxemin' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with rlxemin keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) rlxemin
            if (ierr .ne. 0) then
              message='Argument for rlxemin must be real number. Not :'
               call wrline(i+2,iz,iz0)
            endif
            i=i+2
         endif
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keyeunits(rlxemin,keyword(i),ierr)
            if (ierr .eq. 1) then
               message = 'no energy unit after comma (rlxemin).'
               call wrline(zeroi,iz,iz0)
            endif
         elseif(abs(rlxemin) .ge. 1.0_dop) then
            write(6,'(/a,/a,e11.4,a,e11.4,a,e11.4,a)') &
            'WARNING: No unit specified for rlxEmin, au assumed.', &
            ' rlxEmin =',rlxemin,' au, =',rlxemin*ev,' eV, =', &
              rlxemin*2.194746e+5_dop,' cm-1' 
         endif

      else if( keyword(i) .eq. 'rlxemax' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with rlxemax keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) rlxemax
            if (ierr .ne. 0) then
              message='Argument for rlxemax must be real number. Not :'
               call wrline(i+2,iz,iz0)
            endif
            i=i+2
         endif
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keyeunits(rlxemax,keyword(i),ierr)
            if (ierr .eq. 1) then
               message = 'no energy unit after comma (rlxemax).'
               call wrline(zeroi,iz,iz0)
            endif
         elseif(abs(rlxemax) .ge. 1.0_dop) then
            write(6,'(/a,/a,e11.4,a,e11.4,a,e11.4,a)') &
            'WARNING: No unit specified for rlxEmax, au assumed.', &
            ' rlxEmax =',rlxemax,' au, =',rlxemax*ev,' eV, =', &
              rlxemax*2.194746e+5_dop,' cm-1'
         endif

      else if( keyword(i) .eq. 'rlximin' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with rlximin keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) rlximin
            if (ierr .ne. 0 .or. rlximin .ge. 0.d0) then
              message='Argument for rlxImin must be a negative '// &
                      'real number. Not :'
               call wrline(i+2,iz,iz0)
            endif
            i=i+2
         endif
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keyeunits(rlximin,keyword(i),ierr)
            if (ierr .eq. 1) then
               message = 'no energy unit after comma (rlximin).'
               call wrline(zeroi,iz,iz0)
            endif
         elseif(abs(rlximin) .ge. 1.d-1) then
            write(6,'(/a,/a,e11.4,a,e11.4,a,e11.4,a)') &
            'WARNING: No unit specified for rlxImin, au assumed.', &
            ' rlxImin =',rlximin,' au, =',rlximin*ev,' eV, =', &
              rlximin*2.194746d+5,' cm-1'
         endif

      else if( keyword(i) .eq. 'freeze' )  then
         message='Freeze keyword moved to integrator-section in 10.2.'
         call warnmsg
         if (keyword(i+1).eq.'=') then
  80        j=j+1
            read(keyword(i+2),*,iostat=ierr) ifreeze
            if (ierr .ne. 0) then
              message='Argument for freeze must be an integer. Not :'
              call wrline(i+2,iz,iz0)
            endif
            i=i+2
            if (keyword(i+1) .eq. ',') goto 80
         endif

      else if( keyword(i) .eq. 'split-rst'  .or. &
               keyword(i) .eq. 'splitrst' )  then
         lsplitrst=.true.

      else if( keyword(i) .eq. 'reflex' )  then
         lreflex=.true.

      else if( keyword(i) .eq. 'realphi' )  then
         lrealphi=.true.

      elseif (keyword(i)(1:8) .eq. 'diagonal') then
         if (keyword(i+1) .ne. '=') then
            message = 'No argument specified with diagonalise keyword'
            call warnmsg
         else if (keyword(i+2) .eq. 'direct') then
            diagham_type = 1
            lexact   = .true.
            i=i+2
            frunlevel = 4
            lrundiag = .true.
         else
            diagham_type = 0
            read(keyword(i+2),*,iostat=ierr) lanczorder
            if (ierr .ne. 0) then
               message = &
                  'Integer argument required for diagonalise. Not:'
               ilbl    = index(message,':')+2
               message = message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
            frunlevel = 4
            lrundiag = .true.
            lexact   = .true.
         endif
! Surface hopping Zagreb keyword
      elseif (keyword(i)(1:13) .eq. 'srfhop_zagreb') then
         lrunshzagreb = .true.
         psitype = 3
         ltraj=.true.
! Local control Keywords
      else if (keyword(i) .eq. 'local-control') then
         lcontrol = .true.
         tog = 1
      else if (keyword(i) .eq. 'nosmooth') then
         lgpulse = .true.
      else if (keyword(i) .eq. 'lct-vib') then
         lctvibpop = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*) nviblct 
         else 
            print*,"You need to define which eigenstate to optimise."
            call stopnow 
         endif
         if (keyword(i+1) .eq. ',') then
            i=i+2
            read(keyword(i),*) nstalct 
         else
            print*,"You need to define the state you are controlling."
            call stopnow
         endif
         if (keyword(i+1) .eq. ',') then
            i=i+2
            read(keyword(i),*) nmlct 
         else
            print*,"You need to define the mode you are controlling."
            call stopnow
         endif
      else if (keyword(i) .eq. 'toggle') then
         ltoggle = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*) fswitch            
            fswitch=nint(fswitch*fs)
            if (keyword(i+1) .eq. ',') then
               i=i+2
               read(keyword(i),*) interswit            
               interswit=nint(interswit*fs)
            endif
            nmb=1
            do n=1,100
               switch(n)=fswitch+(interswit*(n-1))
            enddo
         else
            print*,'ERROR: Integer argument required for toggle' 
            call stopnow
         endif

      else if (keyword(i) .eq. 'lct_target') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*) lctevalue
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keyunits(lctevalue,keyword(i),ierr)
               if (ierr .eq. 1) then
                  message = 'no unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
         else
            message = 'No arguments after keyword lct_target'
            call errormsg
         endif
      else if (keyword(i) .eq. 'lct_ftime') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*) lctftime
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keyunits(lctftime,keyword(i),ierr)
              if (ierr .eq. 1) then
                  message = 'no unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
         else
            message = 'No arguments after keyword lct_ftime'
            call errormsg
         endif
      else if (keyword(i) .eq. 'lct_envelope') then
         if (keyword(i+1) .ne. '=') then
            message = 'No arguments after keyword lct_envelope'
            call errormsg
         endif
         if (keyword(i+3) .ne. ',') then
            message = 'Not enough arguments after keyword lct_envelope'
            call errormsg
         endif
         if (keyword(i+5) .ne. ',') then
            message = 'Not enough arguments after keyword lct_envelope'
            call errormsg
         endif

         i=i+2
         read(keyword(i),*) ilctenv
         i=i+2
         read(keyword(i),*) lctenvstr
         i=i+2
         read(keyword(i),*) lctenvtime
         i=i+2
         read(keyword(i),*) lctenvwidth
         if (keyword(i+1) .eq. ',') then
            i=i+2
            call keyunits(lctenvtime,keyword(i),ierr)
            if (ierr .eq. 1) then
               message = 'no unit after comma'
               call wrline(zeroi,iz,iz0)
            endif
            call keyunits(lctenvwidth,keyword(i),ierr)
         endif

! End of LCT Keywords

      else if( keyword(i) .eq. 'optcntrl' )  then
         loptcntrl = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
 23         continue
            if(keyword(i) .eq. 'pc') then
               loctpc=.true.
               i=i+1
               if(keyword(i) .eq. ',') then
                  i=i+1
                  goto 23
               endif
            elseif (keyword(i) .eq. 'simprop') then
               lsimprop = .true.
               i=i+1
               if(keyword(i) .eq. ',') then
                  i=i+1
                  goto 23
               endif
            elseif (keyword(i) .eq. 'update') then
               loctupdate = .true.
               i=i+1
               if(keyword(i) .eq. ',') then
                  i=i+1
                  goto 23
               endif
            else
               message = "optcntrl = S: unknown keyword: "// &
                    keyword(i)(1:index(keyword(i),' '))
               call errormsg
            endif
            if (loctpc .and. lsimprop) then
               message='"simprop" and "pc" cannot be used'// &
                    ' together as argument for "optcntrl".'
               call errormsg
            endif

            if (.not. loctpc .and. .not. lsimprop .and. .not. loctupdate) then
               message='Expecting "pc", "simprop" or "update" '// &
                    'as argument to "optcntrl". Not:'
               ilbl   = index(message,':')+2
               message= message(1:ilbl)//keyword(i)
               call warnmsg
            endif

            if (.not. lsimprop .and. loctupdate) then
               loctupdate = .false.
               message= 'Using keyword "update" without "parallel"'// &
                    'for optcntrl is useless and will be ignored.'
               call warnmsg
               lwarn = .false.
            endif
         endif

! read in random number generator seed
!*************************************************************************
! If reinstating this, please ensure the array seed is deallocated after 
! it has been used (wherever that is appropriate).
!************************************************************************* 
!      else if( keyword(i) .eq. 'seed' )  then
!         call random_seed(size=seedsize) !check how many seeds are needed. This
                                         !can be compiler dependent
!         allocate(seed(seedsize))
!         if (keyword(i+1).ne.'=') then
!            message = 'No argument specified with seed keyword'
!            call warnmsg
!            deallocate(seed)
!         else
!            do j=1,seedsize
!              read(keyword(i+2),*,iostat=ierr) seed(j)
!              if (ierr .ne. 0) then
!                message='Argument for seed must be an integer. Not :'
!                call wrline(i+2,iz,iz0)
!              endif
!              i=i+2
!              if ((j.lt.seedsize).and.(keyword(i+1).ne.',')) then
!                write(message,'(a,i2,a)') &
!                    'Not enough random generator seed values. Need ', &
!                     seedsize,' seed values. Default seed will be used.'
!                call warnmsg
!                deallocate(seed)
!                exit !exit do loop
!              endif
!            end do
!            if (keyword(i+1) .eq. ',') then
!              write(message,'(a,i2,a,i2,a)') &
!                 'More than ',seedsize,' seed values seem to be given in input. '// &
!                 'Only ',seedsize,' seed values should be given.'
!              call warnmsg
!              endif
!         endif

      else if (keyword(i) .eq. 'readdvr') then
         lrddvr=.true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            dname=keyorig(i)
            dlaenge = lc(i)
            call abspath(dname,dlaenge)
         endif
      else if (keyword(i) .eq. 'deldvr') then
         ldeldvr=.true.

      else if (keyword(i) .eq. 'readoper') then
         lrdoper=.true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            oname=keyorig(i)
            olaenge = lc(i)
            call abspath(oname,olaenge)
         endif
      else if (keyword(i) .eq. 'deloper') then
         ldelop=.true.

      else if (keyword(i) .eq. 'readinwf') then
         lrdinwf=.true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            rname=keyorig(i)
            rlaenge = lc(i)
            call abspath(rname,rlaenge)
         endif

!DWF  Dynamical SPFs spawning thresholds (i.e. spawn)
      else if( keyword(i) .eq. 'spawn' .or. &
                   keyword(i) .eq. 'dynamicwf' )  then
!        Defaults
         ldynspf=.true.
         alphamax = 0.001   ! alphamax = alphathr

!        Optional keywords
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) alphamax
         endif
         if (keyword(i+1).eq.',') then
            i=i+2
            read(keyword(i),*,iostat=ierr) alphathr
            if (alphathr .gt. alphamax) then
               message='Alphathr cannot be .gt. alphamax'
               call errormsg
            endif
         endif
         if (keyword(i+1).eq.',') then
            i=i+2
            read(keyword(i),*,iostat=ierr) falpha
            idynalpha=1
         endif
         if (keyword(i+1).eq.',') then
            i=i+2
            read(keyword(i),*,iostat=ierr) alpha0
         endif
         if (keyword(i+1).eq.',') then
            i=i+2
            read(keyword(i),*,iostat=ierr) alphathr0
         endif
         if (keyword(i+1).eq.',') then
            i=i+2
            read(keyword(i),*,iostat=ierr) idynalpha
         endif

!DWF  Optimal initial orbitals (i.e. initorb)
      else if (keyword(i) .eq. 'optorb') then
!        Defaults
         linitorb=.true.
         loptspf = .true.
         loptspflancz = .true.
         iresize = -1

!        Optional keywords
         if (keyword(i+1) .eq. '=') then
            do 
               i=i+2
               if (keyword(i).eq.'std') then
                  loptspf = .false.
                  linitorb=.false.
                  loptspflancz = .false.
               else if (keyword(i).eq.'opt') then
                  continue
               else if (keyword(i).eq.'optf') then
                  loptspflancz = .false.
               else if (keyword(i) .eq. 'opta') then
                  loptavec = .true.
               else if (keyword(i) .eq. 'opta2') then
                  loptavec = .true.
                  loptavec2nd = .true.
               else if (keyword(i) .eq. 'optanum') then
                  loptavec = .true.
               else if (keyword(i) .eq. 'qopt') then
                  lqoptspf= .true.
               else if (keyword(i).eq.'fixn') then
                  loptnspf = .false.
                  iresize = -1
               else if (keyword(i).eq.'optn') then
                  loptnspf = .true.
                  iresize = 0
               else if (keyword(i).eq.'ibeta') then
                  i=i+1
                  read(keyword(i),*,iostat=ierr) ibeta
               else if (keyword(i)(1:5).eq.'scale') then
                  ioptavec = 6
               else if (keyword(i)(1:5).eq.'fix') then
                  ioptavec = 3
       
               else 
                  write(message,'(2a)') &
                     'Unknown argument for initorb :',keyword(i)
                  call warnmsg
               endif
               if (keyword(i+1).ne.',') exit
            enddo
         endif

!DWF  Optimal initial orbital options
      else if (keyword(i) .eq. 'betathr') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) betathr

         else 
             write(message,'(2a)') &
                     'No argument specified with betathr '
             call warnmsg
         endif
      else if (keyword(i) .eq. 'betavar') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) betathr

         else 
             write(message,'(2a)') &
                     'No argument specified with betathr '
             call warnmsg
         endif

!DWF  Dynamical SPFs (i.e. dynspf)
      else if (keyword(i) .eq. 'ndynspf0') then
!        Optional keywords
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) ndynspf0
         else
            message='No argument specified with "ndynspf0" keyword'
            call errormsg
         endif
!DWF  Dynamical SPFs (i.e. dynspf)
      else if (keyword(i) .eq. 'naddspf') then
!        Optional keywords
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) imaxspf
         else
            message='No argument specified with "naddspf" keyword'
            call errormsg
         endif

      else if( keyword(i) .eq. 'debugspawn') then
         ldebugspawn=.true.

! Number Projection on SPFs (i.e. numberproj)
      else if (keyword(i)(1:10) .eq. 'numberproj') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            numprojfile = keyorig(i)
            npflaenge = lc(i)
            inquire(file=numprojfile,exist=lexist)
            if (lexist) then
               lnumberproj=.true.
               lrdnumproj=.true.
            else
               message='No numberproj file "'// &
                       numprojfile(1:npflaenge)//'" found'// &
                       ' in the current directory'
               call errormsg
            endif
         else
            message='No argument specified with "numberproj" keyword'
            call errormsg
         endif

      else if( keyword(i) .eq. 'exact' ) then
         lexact = .true.

      else if( keyword(i) .eq. 'g-mctdh' ) then
         psitype = 4

      else if( keyword(i) .eq. 'realwp' ) then
         lrealwp = .true.

      else if( keyword(i) .eq. 'no_couplings' ) then
         lncoupl = .true.

      else if( keyword(i) .eq. 'swarm' ) then
         ltraj = .true.
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*) ntraj
            if (keyword(i+1) .eq. ',') then
               i=i+2
               read(keyword(i),*) ntraj0
            endif
            if (ntraj0 .gt. ntraj) then
               write(message,'(a,i5)') &
                  'Initial trajectory number larger than total. Max :',&
                   ntraj
               call warnmsg
            endif
         else
            message =  &
                'No. of trajectories is needed with swarm keyword'
            call wrline (i,iz,iz0)
         endif

      else if( keyword(i) .eq. 'rdsrfhop' ) then
         lrdsrfhop = .true.

      else if( keyword(i) .eq. 'wavefunction' ) then
         dentype=0
      else if( keyword(i) .eq. 'density1' ) then
         dentype=1
      else if( keyword(i) .eq. 'density2' ) then
         dentype=2

      else if( keyword(i) .eq. 'overwrite' )  then
         lovwr = .true.

      else if( keyword(i) .eq. 'nooverwrite'  .or. &
               keyword(i) .eq. 'no-overwrite' )  then
         lovwr = .false.

      else if( keyword(i)(1:7) .eq. 'thermal' )  then
         if (rlxtime .eq. 'K') then
            message= &
              'Do not use both "thermal" and "tfinal" to define a relaxation'
            call warnmsg
         endif
         rlxtime = 'K'
         fs = 1.0_dop
         if (keyword(i+1).eq.'=') then
            read(keyword(i+2),*,iostat=ierr) tfinal
            call keyeunits(tfinal,'kelvin ',ierr)
            if (ierr .ne. 0) then
               message = 'Argument for T must be real number. Not :'
               ilbl=index(message,':')+2
               message=message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            tfinal = 1.0_dop/(2.0_dop*tfinal)
            i=i+2
         endif
         if (keyword(i+1) .eq. ',') then
            i = i+2
            read(keyword(i),*,iostat=ierr) mlseed
            if (ierr .ne. 0) then
               message = 'Incorrect argument (seed) for keyword : thermal'
               call wrline(zeroi,iz,iz0)
            endif
         endif

      else if( keyword(i) .eq. 'tfinal' )  then
         if (rlxtime .eq. 'K') then
            message= &
              'Do not use both "thermal" and "tfinal" to define a relaxation'
            call warnmsg
         endif
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with tfinal keyword'
            call warnmsg
         else
! Read only, if not already set by option -tfinal
            if(tfinal.le.-9.0e99_dop) &
              read(keyword(i+2),*,iostat=ierr) tfinal
            if (ierr .ne. 0) then
               message = 'Argument for tfinal must be real number. Not :'
               ilbl=index(message,':')+2
               message=message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
            if(keyword(i+1) .eq. ',' .and. keyword(i+2) .eq. 'kelvin') then
               call keyeunits(tfinal,keyword(i+2),ierr)
               if (ierr .ne. 0) then
                  message = 'Incorrect unit for relaxation time : '
                  call wrline(onei,iz,iz0)
               endif
               tfinal = 1.0_dop/(2.0_dop*tfinal)
               rlxtime = 'K'
               fs = 1.0_dop
               i=i+2
            else if(keyword(i+1) .eq. ',') then
               message = 'Wrong unit for Relaxation time. Only Kelvin allowed'
               call warnmsg
               i=i+2
            endif
         endif

      else if( keyword(i) .eq. 'tinit' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with tinit keyword'
            call warnmsg
         else
            if(.not. ltinitcmd)&
                 read(keyword(i+2),*,iostat=ierr) tinit
            if (ierr .ne. 0) then
               message = 'Argument for tinit must be real number. Not :'
               ilbl=index(message,':')+2
               message=message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
         endif
         if(keyword(i+1) .eq. ',' .and. keyword(i+2) .eq. 'kelvin') then
            call keyeunits(tinit,keyword(i+2),ierr)
            temperature0 = tinit
            if (ierr .ne. 0) then
               message = 'Incorrect unit for relaxation time : '
               call wrline(onei,iz,iz0)
            endif
            tinit = 1.0_dop/(2.0_dop*tinit)
            i=i+2
         else if(keyword(i+1) .eq. ',') then
            message = 'Wrong unit for Initial time. Only Kelvin allowed'
            call warnmsg
            i=i+2
         endif

      else if( keyword(i) .eq. 'tout' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with tout keyword'
            call warnmsg
         elseif( keyword(i+2) .eq. 'all' ) then
            louteverystep = .true.
            i=i+2
         else
            read(keyword(i+2),*,iostat=ierr) out1
            if (ierr .ne. 0) then
                message = 'Argument for tout must be real number. Not :'
               ilbl=index(message,':')+2
               message=message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
         endif

      else if( keyword(i) .eq. 'tpsi' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with tpsi keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) out2
            if (ierr .ne. 0) then
               message = 'Argument for tpsi must be real number. Not :'
               ilbl=index(message,':')+2
               message=message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
         endif

      else if( keyword(i) .eq. 'ntout' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with ntout keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) ntout
            if (ierr .ne. 0) then
               message = 'Argument for ntout must be an integer. Not :'
               ilbl=index(message,':')+2
               message=message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
         endif

      else if( keyword(i) .eq. 'ntpsi' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with ntpsi keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) ntpsi
            if (ierr .ne. 0) then
               message = 'Argument for ntpsi must be an integer. Not :'
               ilbl=index(message,':')+2
               message=message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
         endif


      else if( keyword(i) .eq. 'tstop' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with tstop keyword'
            call warnmsg
         else
            if (sstop(1:1).eq.' ') sstop = keyword(i+2)(1:8)
            i=i+2
         endif

      else if( keyword(i) .eq. 'tcpu' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with tcpu keyword'
            call warnmsg
         else
            if (scpu(1:1).eq.' ') scpu = keyword(i+2)(1:10)
            i=i+2
         endif

      else if( keyword(i) .eq. 'twall' )  then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with twall keyword'
            call warnmsg
         else
            if (swall(1:1).eq.' ') swall = keyword(i+2)(1:10)
            i=i+2
         endif

      else if (keyword(i) .eq. 'no_normstop') then
         lnormstop = .false.

      else if (keyword(i) .eq. 'normstop') then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with normstop keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) nrmstop
            if (ierr .ne. 0) then
               message = &
                 'Argument for nrmstop must be real number. Not :'
               ilbl=index(message,':')+2
               message=message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2

            if (nrmstop .le. 1.0_dop .and. nrmstop .ge. 0.0_dop) then
               lnormstop = .true.
            else
               message='Argument for normstop must be in [0,1] interval'
               call warnmsg
            endif
         endif

      else if (keyword(i) .eq. 'natpopstop') then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with natpopstop keyword'
            call warnmsg
            goto 71
         endif
         lnatpopstop     = .true.
         natpopstopmode  = 0
         natpopstopstate = 0
         i=i+2
         read(keyword(i),*,iostat=ierr) natpopstop
         if (ierr.ne.0) then
            write (message,'(a,a)') &
               'Argument for natpopstop must be real number, not: ', &
               keyword(i)
            call warnmsg
            goto 71
         endif

         if (keyword(i+1).ne.',') goto 71
         i=i+2
         if (keyword(i).eq.'all') then
            natpopstopmode=0
         elseif (keyword(i).eq.'any') then
            natpopstopmode=-1
         else
            read(keyword(i),*,iostat=ierr) natpopstopmode
            if (ierr.ne.0) then
               write (message,'(a,a,a)') &
                  'Second argument for natpopstop must be "all" or ' &
                  ,'"any" or an integer, not: ',keyword(i)
               call warnmsg
            endif
            goto 71
         endif

         if (keyword(i+1).ne.',') then
            natpopstopstate=0
         else
            i=i+2
            read(keyword(i),*,iostat=ierr) natpopstopstate
            if (ierr.ne.0) then
               write (message,'(a,a)') &
                  'Third argument for natpopstop must be integer, not: ' &
                  ,keyword(i)
               call warnmsg
               goto 71
            endif
         endif
   71    continue

! Change initial norm of WF
      elseif (keyword(i).eq.'init_norm') then
         if (keyword(i+1).eq.'=') then
            read(keyword(i+2),*) initnorm
            i=i+2
         else
            routine='einrun/eingabemod.f90'
            message = 'No argument given after init_norm keyword'
            call errormsg
         endif
   
      else if (keyword(i)(1:6) .eq. 'conver') then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with converged keyword'
            call warnmsg
         else
            read(keyword(i+2),*,iostat=ierr) constop
            if (ierr .ne. 0) then
               message = &
                 'Argument for converged must be real number. Not :'
               ilbl=index(message,':')+2
               message=message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif
            i=i+2
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keyeunits(constop,keyword(i),ierr)
               if (ierr .eq. 1) then
                  message = 'no energy unit after comma (converged).'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
!DWF
!           Variable convergence threshold
!           Used when improved-relaxation is combined with dynspf
!           constop0 is the initial threshold and constop the final threshold
            if (keyword(i+1) .eq. ',') then
               i=i+2
               read(keyword(i),*,iostat=ierr) constop0
               if (ierr .ne. 0) then
                  message = &
                 'Argument for converged must be real number. Not :'
                  ilbl=index(message,':')+2
                  message=message(1:ilbl)//keyword(i+2)
                  call warnmsg
               endif
               i=i+2
               call keyeunits(constop0,keyword(i),ierr)
               if (ierr .eq. 1) then
                  message = 'no energy unit after comma (converged).'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
         endif

      elseif (keyword(i).eq.'expect') then
         if (keyword(i+1).eq.'=') then
            if (keyword(i+2).eq.'real-only') then
               lreonly=.true.
               i=i+2
               if(keyword(i+1).ne.',' ) goto 12 
            endif
            istrt=i+2
 11         continue
            if(i+2.le.ic) then
               nexpect=nexpect+1
               i=i+2
               if(keyword(i+1).eq.',' ) goto 11
            else
               message = 'No argument followed comma! keyword : '
               call wrline(i,iz,iz0)
            endif
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif
! allocate expectlab0 array and load in labels
         if (allocated(expectlab)) then
            routine = 'Einrun (eingabemod.f90)'
            message = 'The expect keyword can only be specified once'
            call errormsg
         else
            allocate(expectlab(nexpect))
         endif
         do n=1,nexpect
            expectlab(n) = trim(keyorig(istrt+(n-1)*2))
         enddo
 12      continue

      elseif (keyword(i).eq.'expect1') then
         if(nexpect.eq.0) then
            message = 'expect1 is allowed only after expect.'
            call wrline(zeroi,iz,iz0)
         endif
         if (keyword(i+1).eq.'=') then
            if (keyword(i+2).eq.'real-only') then
               lreonly1=.true.
               i=i+2
               if(keyword(i+1).ne.',' ) goto 22
            endif
            istrt=i+2
 21         continue
            if(i+2.le.ic) then
               nexpect1=nexpect1+1
               i=i+2
               if(keyword(i+1).eq.',' ) goto 21
            else
               message = 'No argument followed comma! keyword : '
               call wrline(i,iz,iz0)
            endif
         else
            message = 'No argument specified with keyword : '
            call wrline(i,iz,iz0)
         endif
! allocate expectlab1 array and load in labels
         if (allocated(expectlab1)) then
            routine = 'Einrun (eingabemod.f90)'
            message = 'The expect1 keyword can only be specified once'
            call errormsg
         else
            allocate(expectlab1(nexpect1))
         endif
         do n=1,nexpect
            expectlab1(n) = trim(keyorig(istrt+(n-1)*2))
         enddo
 22      continue

      elseif (keyword(i).eq.'expect_nonorm') then
         lnwexpect = .true.
      elseif (keyword(i).eq.'expect_norm') then
         lnwexpect = .false.
         
      else if( keyword(i) .eq. 'title' .or. & 
               keyword(i) .eq. 'titel' )  then
         if (keyword(i+1) .ne. '=' ) then
            read(iin,'(a)') runtitle
            iz = iz + 1
         else
            i = i + 2
            ilbl=index(inptit,'title')
            if(ilbl.eq.0) ilbl=index(inptit,'titel')
            runtitle = inptit(ilbl+5:80)
            ilbl=index(runtitle,'=')
            runtitle = runtitle(ilbl+1:80)
            goto 10
         end if
      else if( keyword(i) .eq. 'energy-not-ev' )  then 
         ev = 1.0_dop
         rlxunit = 'au'
      else if( keyword(i) .eq. 'time-not-fs' )  then
         fs = 1.0_dop
         rlxtime = 'au'

      else if(keyword(i) .eq. 'output' )  then
         loutfil = .true.
      else if(keyword(i) .eq. 'nooutput'  .or. &
              keyword(i) .eq. 'no-output' .or. &
              keyword(i) .eq. 'screen' )  then
         loutfil = .false.
      else if(keyword(i) .eq. 'pdensity' )  then
         lpdens = .true.
         ierr=0
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) npd1
            if (keyword(i+1).eq.',') then
               i=i+2
               read(keyword(i),*,iostat=ierr) npd2
               if (keyword(i+1).eq.',') then
                  i=i+2
                  read(keyword(i),*,iostat=ierr) npd3
                  if (keyword(i+1).eq.',') then
                     i=i+2
                     read(keyword(i),*,iostat=ierr) npd4
                  endif
               endif
            endif
         endif
         if (ierr .gt. 0) then
            message = 'Error when reading the lpens argument(s) '
            call warnmsg
         end if
      else if( keyword(i) .eq. 'gridpop' ) then 
         lgpop = .true.
         if (keyword(i+1).eq.'=') then
            i=i+2
            if (keyword(i) .eq. 'el' ) then
               lgpel=.true.
            else if (keyword(i) .eq. 'no_el' ) then
               lgpel=.false.
            else
               message = &
                  'Unknown argument specified with gridpop keyword :'
               call wrline(i,iz,iz0)
            end if
         end if

      else if (keyword(i)=='sheptraj') then
        lsheptraj=.true.
        if (keyword(i+1).eq.'=') then
          if (keyword(i+2).eq.'nogpoint') lshepgp=.false.
          i=i+2
        endif

      else if (keyword(i)=='activegwp') then
          lactivegp=.true.

      else if( keyword(i) .eq. 'psi'   )  then
!-----------------------------------------------------------------------
!     lpsinat  : psi saved as natural orbitals
!     lpsicm   : psi saved in compact form
!     lpsisp   : psi saved in single precision
! NB dimensions of lpsiopt in global.inc and default.f must be > no.
! options
!-----------------------------------------------------------------------
         lpsi     = .true.
         lpsisp   = .true.
         lpsicm   = .false.
         lpsinat  = .false.

         if (keyword(i+1).eq.'=') then
   70       continue
            i=i+2
            if (keyword(i) .eq. 'single' ) then
               lpsisp=.true.
            else if (keyword(i) .eq. 'natur'  ) then
               lpsinat=.true.
            else if (keyword(i) .eq. 'compact') then
               lpsicm=.true.
               lpsisp=.false.
               if (keyword(i+1).eq.',') then
                  read(keyword(i+2),*,iostat=ierr) rnum
                  if (ierr .eq. 0) then
                     psicut=rnum
                     i=i+2
                  endif
               endif
            else if (keyword(i) .eq. 'double' ) then
               lpsisp=.false.
            else if (keyword(i) .eq. 'no') then
               lpsi = .false.
            else
               message = &
                  'Unknown argument specified with psi keyword :'
               call wrline(i,iz,iz0)
            endif
            if (keyword(i+1).eq.',') go to 70
         endif
         lpsiopt(1)=lpsisp
         lpsiopt(2)=lpsicm
         lpsiopt(3)=lpsinat
! lpsiopt(4) is reserved for selected-ci, allocated in initprop

      else if( keyword(i) .eq. 'auto'   )  then
         lauto = .true.
         if (keyword(i+1).eq.'=') then
   90       continue
            i=i+2
            if (keyword(i) .eq. 'twice') then
               lautodoub=.true.
            else if (keyword(i) .eq. 'once' ) then
               lautodoub=.false.
            else if (keyword(i) .eq. 'no'   ) then
               lauto  = .false.
            else if (keyword(i) .eq. 'error'   ) then
               lautoe = .true.
            elseif (keyword(i) .eq. 'order1') then
               lautoord1 = .true.
            elseif (keyword(i) .eq. 'order2') then
               lautoord1 = .true.
               lautoord2 = .true.
            else
               message = &
                  'Unknown argument specified with auto keyword :'
               call wrline(i+2,iz,iz0)
            endif
            if (keyword(i+1).eq.',') go to 90
         endif
         lautodoub = lautodoub .and. lauto
         lautoe    = lautoe    .and. lauto
         lautoord1 = lautoord1 .and. lauto
         lautoord2 = lautoord2 .and. lauto

! cross-correlation option
      else if (keyword(i) .eq. 'cross') then
         lcross = .true.
         nst1   = -999      ! will be re-set in chkprop
!        check for arguments
         if (keyword(i+1) .eq. '=') then
!           advance keyword counter
            i = i+2
!           check for "cross=name"
            if (keyword(i) .eq. 'name') then
!              default: use the restart-file from this calculation's
!              name-directory for calculating the cross-corr-function
               crossname = name
               crosslng  = laenge
            else
!              use the restart-file from the directory specified by
!              the argument for calculating the cross-corr-function
               crossname = keyorig(i)
               crosslng  = lc(i)
               call abspath(crossname, crosslng)
            endif
            if (keyword(i+1) .eq. ',') then
!              advance keyword counter and read state
               i = i+2
               read(keyword(i),*,iostat=ierr) nst1
               nst2 = nst1
               if (ierr .ne. 0) then
                  message = &
                  'Incorrect argument (state) for keyword : cross'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
         endif

      else if( keyword(i) .eq. 'ctrace' ) then
         lctr = .true.                  
         if (keyword(i+1).eq.'=') then
            if (i+2.le.ic) then
               ctrname = keyorig(i+2)(1:c2)
               i=i+2
            else
               message = 'Missing argument!'
               call wrline(i,iz,iz0) 
            endif
         endif

      else if( keyword(i) .eq. 'timing' ) then
         ltime = .true.                  
      else if( keyword(i) .eq. 'notiming' .or. &
               keyword(i) .eq. 'no-timing' ) then
         ltime = .false.                  
      else if( keyword(i) .eq. 'steps'  ) then
         lintstep = .true.               
      else if( keyword(i) .eq. 'update' ) then
         lupdate = .true.                
      else if( keyword(i) .eq. 'noupdate' .or. &
               keyword(i) .eq. 'no-update' ) then
         lupdate = .false.                
      else if( keyword(i) .eq. 'speed'  ) then
         lspeed  = .true.                
      else if( keyword(i) .eq. 'nospeed' .or. &
               keyword(i) .eq. 'no-speed' ) then
         lspeed  = .false.                
      else if( keyword(i) .eq. 'stop'   ) then
         lstopf  = .true.                
      else if( keyword(i) .eq. 'nostop' .or. &
               keyword(i) .eq. 'no-stop' ) then
         lstopf  = .false.                
      elseif (keyword(i) .eq. 'lanczvec') then
         llanczvec = .true.
      elseif (keyword(i) .eq. 'eigvec'  ) then
         leigvec   = .true.
      elseif (keyword(i) .eq. 'orben'  ) then
         lorben    = .true.
      elseif (keyword(i) .eq. 'veigen'  ) then
         lveigen   = .true.
      elseif (keyword(i) .eq. 'allcomplete'  ) then
         allcomplete   = .true.
      else if (keyword(i) .eq. 'quadpop' ) then
         if (dentype.eq.1) then
            lquadpop = .true.
         else
            message = 'Keyword quadpop only for dentype I allowed!'
            call errormsg
         endif
      elseif (keyword(i) .eq. 'graphviz' .or. keyword(i) .eq. 'graph') then
         lgraphviz = .true.
      else if( keyword(i) .eq. 'all'    ) then
         ltime     = .true.
         lintstep  = .true.
         lupdate   = .true.
         lspeed    = .true.
         lstopf    = .true.
         loutfil   = .true.
         lgpop     = .true.
         lpdens    = .true.
         lpsi      = .true.
         lpsisp    = .true.
         lpsicm    = .false.
         lpsinat   = .false.
         lpsiopt(1)= lpsisp
         lpsiopt(2)= lpsicm
         lpsiopt(3)= lpsinat
         lauto     = .true.
         llanczvec = .true.
         leigvec   = .true.
         lveigen   = .true.

#ifdef MPI
      else if( keyword(i) .eq. 'usempi' )  then
         lmpi=.true.
         if(keyword(i+1) .eq. '=' )  keyword(i+1)=','
  150    if(keyword(i+1) .eq. ',' )  then
            i=i+2
            if(keyword(i) .eq. 'no-mfields' ) then
               lmpimfields=.false.
            elseif(keyword(i) .eq. 'no-funka2' ) then
               lmpifunka2=.false.
            elseif(keyword(i) .eq. 'no-calcha' ) then
               lmpicalcha=.false.
            elseif(keyword(i) .eq. 'no-phihphi' ) then
               lmpiphihphi=.false.
            elseif(keyword(i) .eq. 'no-hlochphi' ) then
               lmpihlochphi=.false.
            elseif(keyword(i) .eq. 'no-dav' ) then
               lmpidav=.false.
            elseif(keyword(i) .eq. 'no-getdavmat' ) then
               lmpigetdavmat=.false.
            elseif(keyword(i) .eq. 'no-getdiag') then
               lmpigetdiag = .false.
            elseif(keyword(i) .eq. 'no-dsyev' ) then
               lmpidsyev=.false.
            elseif(keyword(i) .eq. 'no-summf' ) then
               lmpisummf=.false.
            elseif(keyword(i) .eq. 'no-expect' ) then
               lmpiexpect=.false.
            elseif(keyword(i) .eq. 'dav' )  then
               ldavkey=.true.
               if (keyword(i+1).ne.'=') then
                  message='No argument specified with dav keyword'
                  call warnmsg
               else
                  read(keyword(i+2),*,iostat=ierr) intordermpi
                  if (ierr .ne. 0) then
                     message = 'Argument for dav must be an integer number. Not :'
                     ilbl=index(message,':')+2
                     message=message(1:ilbl)//keyword(i+2)
                     call warnmsg
                  endif
                  i=i+2
               endif

            elseif(keyword(i) .eq. 'off' )  then
! turn off use of mpi
               lmpioff = .true.

            else
               message = 'Unknown argument to "usempi" :'
               call wrline(i,iz,iz0)
             endif
             goto 150
         endif
#else
      else if( keyword(i) .eq. 'usempi' )  then
         message = '"usempi" requires an excecutable which is' &
                //' compiled with MPI support!'
         call errormsg
#endif

      else if( keyword(i) .eq. 'openmp' ) then

         lomptim = .true.
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with openmp keyword'
            call warnmsg

         else
            read(keyword(i+2),*,iostat=ierr) ompthread
            if (ierr .ne. 0) then
               message = 'Argument for openmp must be an integer number. Not :'
               ilbl=index(message,':')+2
               message=message(1:ilbl)//keyword(i+2)
               call warnmsg
            endif

            if (ompthread .gt.0) lompthread=.true.
            i=i+2
         endif

      else if( keyword(i) .eq. 'openmp_mfield' ) then
         lompmfield=.true.
      else if( keyword(i) .eq. 'openmp_qc' ) then
         lompqc=.true.
      else if( keyword(i) .eq. 'openmp_calcha' ) then
         lompcalcha=.true.
      else if( keyword(i) .eq. 'openmp_phihphi' ) then
         lompphihphi=.true.
      else if( keyword(i) .eq. 'openmp_ghelem' ) then
         lompghelem=.true.

      else if( keyword(i) .eq. 'no_openmp_mfield' ) then
         lompmfield=.false.
      else if( keyword(i) .eq. 'no_openmp_qc' ) then
         lompqc=.false.
      else if( keyword(i) .eq. 'no_openmp_calcha' ) then
         lompcalcha=.false.
      else if( keyword(i) .eq. 'no_openmp_phihphi' ) then
         lompphihphi=.false.
      else if( keyword(i) .eq. 'no_openmp_ghelem' ) then
         lompghelem=.false.

! fast algorithm will be used for V(natpot)*A operation
      else if (keyword(i).eq.'natfast') then
         if(lnatfast) then
            routine='einrun'
            message='There must be no more than one FAST keyword!'
            call errormsg
         else
            lnatfast = .true.
         endif

         fastn=1
         j=i+1
! look to see if operators specified, and if so how many.
         if (keyword(j).eq.'=') then
            do
               j=j+2
               if (keyword(j).eq.',') then   ! next operator
                  fastn=fastn+1
               else if (keyword(j).eq.'{') then   ! ignore order
                  j=j+1
               else
                  exit
               endif
            enddo
         endif
         allocate(fastlabel(fastn))
         allocate(fastilabel(fastn))

         i=i+1
         if (keyword(i).eq.'=') then
            lfastall=.false.
            j=0

 160        i=i+1
            j=j+1
            read(keyorig(i),*,iostat=ierr) fastlabel(j)
            if (ierr.ne.0) then
               message='Cannot read label for keyword fast'
               call wrline(i,iz,iz0)
            endif
            if (fastlabel(j).eq.'all') then
               lfastall=.true.
               fastilabel(j)=0
            endif
            i=i+1
            if (keyword(i).eq.'{') then
               i=i+1
               read(keyword(i),*,iostat=ierr) fastilabel(j)
               if (ierr.ne.0) then
                  message='Cannot read the fast order for label '// &
                          fastlabel(j)(1:lc(i))
                  call wrline(i,iz,iz0)
               endif
               if (fastilabel(j).lt.1) then
                  message='The fast order must be a positive integer.'
                  call wrline(i,iz,iz0)
               endif
               i=i+1
               if (keyword(i).ne.'}') then
                  call wrline(i,iz,iz0)
               endif
               i=i+1
               if (keyword(i).eq.',') go to 160
            elseif (keyword(i).eq.',') then
               go to 160
            else
               go to 170
            endif
 170        continue
         else
            lfastall=.true.
            fastilabel(1)=0
            fastlabel(1)='all'
         endif

! error if unknown keyword
      else
         message = ' unknown keyword : '
         call wrline(i,iz,iz0)

         if (keyword(i+1) .eq. '=') then
            i=i+2
            do
               if (keyword(i+1) .eq. ',') then
                  i=i+2
               else
                  exit
               endif
            enddo
         endif

      end if

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

 20   continue
      
!-----------------------------------------------------------------------
!  if initial temeprature has not been set in a relaxation
!  calculation, set it to infinity
!-----------------------------------------------------------------------
      if (lrlx .and. rlxtime .eq. 'K') then
         if (dentype .eq. 0) lrlxt = .true.

         if (abs(tinit - 0.0_dop) .lt. tiny) then
            temperature0 = 99999999.0_dop
         endif
         if (abs(out1 - 0.0d0) .gt. tiny) then
            routine = 'Eingabe (eingabemod.f90) '
            message = &
 'Specify output interval in thermal relaxation using ntout = N rather than tout'
            call errormsg
         endif
         if (abs(out2 - 0.0d0) .gt. tiny) then
            routine = 'Eingabe (eingabemod.f90) '
            message = &
 'Specify psi output interval in thermal relaxation using ntpsi = N rather than tpsi'
            call errormsg
         endif
         if (ntout .gt. 0) then
            out1 = (tfinal - tinit)/dble(ntout)
         else
            out1 = tfinal
         endif
         if (ntpsi .gt. 0) then
            out2 = (tfinal - tinit)/dble(ntpsi)
         endif

      else if (rlxtime .eq. 'K') then
         routine = 'Eingabe (eingabemod.f90) '
         message = &
 'A final temperature has been defined but relaxation not asked for.'
         call errormsg

      else
!-----------------------------------------------------------------------
! convert times to a.u. 
!-----------------------------------------------------------------------
         tinit  = tinit*fs
         tfinal = tfinal*fs
         out1   = out1*fs
         out2   = out2*fs
      endif

!-----------------------------------------------------------------------
!  add rlxshift to rlxemin and rlxemax
!-----------------------------------------------------------------------
      if(rlxemin .gt. -9.0e99_dop)  rlxemin = rlxemin + arlxshift
      if(rlxemax .lt.  9.0e99_dop)  rlxemax = rlxemax + arlxshift
 
!-----------------------------------------------------------------------
! CHECK CONSISTENCY OF INPUT
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!  check flags related to dynamic WF.
!-----------------------------------------------------------------------
      if (loptnspf) then
         message = 'Variable no. of optimal orbitals not working'
         call errormsg
      endif

!-----------------------------------------------------------------------
! Cannot be both test and continuation
!-----------------------------------------------------------------------
      If (lcont.and.test) then
         routine = 'Einrun'
         message='One must not give "continuation" and "test"'
         call errormsg
      endif

!-----------------------------------------------------------------------
! Some files only available for diagonalisation
!-----------------------------------------------------------------------
      leigvec = (leigvec .and. lrundiag)
      llanczvec = (llanczvec .and. lrundiag)

!-----------------------------------------------------------------------
! Some options are not available for wavefunctions
!-----------------------------------------------------------------------
      if (dentype .eq. 0) then
         routine='Einrun'
         if (lctr) then
            message= &
              'The ctrace option is not available for wavefunctions'
            call errormsg
         endif
      endif

!-----------------------------------------------------------------------
! Some options are not available for all density matrix types
!-----------------------------------------------------------------------
      if (dentype .eq. 2) then
         if (lexact) then
            message='Exact propagation not possible for density type II.'
            call errormsg
         endif
      else if (dentype .gt. 0) then
         routine='Einrun'
         if (lrundiag) then
            message='Diagonalization not implemented for densities.'
            call errormsg
         endif
         if (lrealwp) then
            message='Real wavepacket propagation not implemented for densities.'
            call errormsg
         endif
         if (lautoord1) then
            message= &
              'Auto = order1 not implemented for density matrices.'
            call errormsg
         endif
         if (lautoord2) then
            message= &
              'Auto = order2 not implemented for density matrices.'
            call errormsg
         endif
         if (lpsinat) then
            message= &
            'Natural orbital form of psi not implemented for densities.'
                  call errormsg
               endif
         if (lpsicm) then
            message= &
              'Compact form of psi not implemented for densities.'
            call errormsg
         endif
         if (lrlx .and. rlaxnum.ge.0 ) then
            message='Improved Relaxation not implemented for densities.'
            call errormsg
         endif
      endif

!-----------------------------------------------------------------------
! Some options are not available for swarm of trajectory calculations
!-----------------------------------------------------------------------
      if (ltraj) then
         if (lauto) then
            lauto = .false.
            lautodoub = lauto
            lautoord1 = lauto
            lautoord2 = lauto
            message= &
              'Trajectory calc: Cannot calculate autocorrelation. '// &
              'Turning off option'
            call warnmsg1
         endif

         if (ntraj .eq. 0) ntraj = 1
      endif

!-----------------------------------------------------------------------
! check consistency of OMP requests
!-----------------------------------------------------------------------
      lompmfield = lompmfield .and. lompthread
      lompqc= lompqc .and. lompthread
      
!-----------------------------------------------------------------------
! In a diagonalisation run some files are not needed.
! If not a diagonalisation run some files are not needed.
!-----------------------------------------------------------------------
      if (lrundiag) then
         lintstep = .false.
         lupdate  = .false.
         lspeed   = .false.
         lstopf   = .false.
         lgpop    = .false.
         lpsi     = .false.
         lauto    = .false.
         lautoord1= .false.
         lautoord2= .false.
         lcross   = .false.
      else
         leigvec = .false.
         llanczvec = .false.
      endif

!-----------------------------------------------------------------------
! A continuation run requires files to be read
!-----------------------------------------------------------------------
      if (lcont) then
         lrddvr  = .true.
         lrdoper  = .true.
         lrdinwf = .true.
      endif

!-----------------------------------------------------------------------
! A genpes run has to stop at level 2
!-----------------------------------------------------------------------
      if (lrunpes .and. frunlevel .gt. 2) then
         message = 'Asking to generate PES file and continue. Not allowed.'
         call errormsg
      endif

!-----------------------------------------------------------------------
! check for use of MPI
!-----------------------------------------------------------------------
      if (nmpi .gt. 1) lmpi = .true.
      lmpidsyev=.false.

      if (lmpioff) lmpi = .false.

      if (lrealwp.and.nmpi.gt.1) then
         routine = 'Einrun (eingabemod.F90)'
         message = 'Real Wavepackets not implemented with MPI.'
         call warnmsg1

         lmpi = .false.
      endif

      return
      end subroutine einrun

!#######################################################################

      subroutine setpaths

      implicit none

      integer(long)                 :: ierr,ilbl,isec,imin,ihour

!-----------------------------------------------------------------------
! check that dname, oname, rname, crossname and ddname are set.
!-----------------------------------------------------------------------
      if (dlaenge .eq. 0) then
         dname=name
         dlaenge=laenge
      endif
      if (olaenge .eq. 0) then
         oname=name
         olaenge=laenge
      endif
      if (rlaenge .eq. 0) then
         rname=name
         rlaenge=laenge
      endif
      if (crosslng .eq. 0) then
         crossname=name
         crosslng=laenge
      endif
      if (ddlaenge .eq. 0) then
         ddname=name(1:laenge)//'/dd_data'
         ddlaenge=laenge+8
      endif

!-----------------------------------------------------------------------
! set up operfile string
!-----------------------------------------------------------------------
      if (irunoper .eq. 1) then 
         if (lrunpes) then
            operfile=name(1:laenge)//'/pes'
            opflaenge=laenge+4
         else 
            operfile=name(1:laenge)//'/'//opername
            opflaenge=index(operfile,' ')-1
         endif
      else if (irunoper .eq. 2) then
         operfile=oname(1:olaenge)//'/oper'
         opflaenge=olaenge+5
      endif

!-----------------------------------------------------------------------
! Set the string sstop to required hhh:mm form
!-----------------------------------------------------------------------
      if(sstop .ne. ' ') then             ! Do nothing if sstop is empty
         if(sstop(2:2) .eq. ':' ) then    ! Move the first ":" to
            sstop=' 0'//sstop(1:5)        ! its right place
         elseif(sstop(3:3) .eq. ':' ) then
            sstop=' '//sstop(1:6)
         elseif (sstop(4:4) .eq. ':') then
            continue
         else
            message='Incorrect form of tstop argument (hhh:mm):'//sstop
            call warnmsg
         endif
         if(sstop(4:4) .eq. ':' .and. sstop(5:6) .eq. '  ') then
            sstop(5:8) = '00  '
         endif
      endif

!-----------------------------------------------------------------------
! Set the string scpu to required hh:mm:ss form
!-----------------------------------------------------------------------
      if(scpu .ne. ' ') then              ! Do nothing if scpu is empty
         if(scpu(2:2) .eq. ':' ) then     ! Move the first ":" to
            scpu=' 0'//scpu(1:7)          ! its right place
         elseif(scpu(3:3) .eq. ':' ) then
            scpu=' '//scpu(1:8)
         endif
         if(scpu(4:4).eq.':' .and. scpu(7:8) .eq. '  ') then
            scpu = scpu(1:6)//':00'
         endif
         if(scpu(4:4).eq.':' .and. scpu(5:6) .eq. '  ') then
            scpu(5:6) = '00'
         endif
         ilbl = index(scpu,' ') - 1
         if (ilbl .lt. 1) then
            continue
         else if( scpu(ilbl:ilbl) .eq. 's' )  then
            scpu(ilbl:ilbl) = ' '
            read(scpu,*,iostat=ierr) isec
            if (ierr .gt. 0) then
               message = 'Error when reading the tcpu argument: '//scpu
               call warnmsg
            end if
            imin  = isec/60
            isec  = isec - 60*imin
            ihour = imin/60
            imin  = imin - 60*ihour
            ihour = min(99,ihour)
            write(scpu,"(i3.2,':',i2.2,':',i2.2)") ihour, imin, isec
         else if( scpu(ilbl:ilbl) .eq. 'm' )  then
            scpu(ilbl:ilbl) = ' '
            read(scpu,*,iostat=ierr) imin
            if (ierr .gt. 0) then
               message = 'Error when reading the tcpu argument: '//scpu
               call warnmsg
            end if
            isec  = 0
            ihour = imin/60
            imin  = imin - 60*ihour
            ihour = min(99,ihour)
            write(scpu,"(i3.2,':',i2.2,':',i2.2)") ihour, imin, isec
         else if (scpu(4:4).eq.':' .and. scpu(7:7).eq.':' ) then
            continue
         else
            message = 'Incorrect form of tcpu argument (hhh:mm:ss): '//scpu
            call warnmsg
         endif
      endif

!-----------------------------------------------------------------------
! Set the string swall to required hhh:mm:ss form
!----------------------------------------------------------------------
      if(swall .ne. ' ') then              ! Do nothing if swall is empty
         if(swall(2:2) .eq. ':' ) then     ! Move the first ":" to
            swall=' 0'//swall(1:7)         ! its right place
         elseif(swall(3:3) .eq. ':' ) then
            swall=' '//swall(1:8)
         endif
         if(swall(4:4).eq.':' .and. swall(7:8) .eq. '  ') then
            swall = swall(1:6)//':00'
         endif
         if(swall(4:4).eq.':' .and. swall(5:6) .eq. '  ') then
            swall(5:6) = '00'
         endif
         ilbl = index(swall,' ') - 1
         if (ilbl .lt. 1) then
            continue
         else if( swall(ilbl:ilbl) .eq. 's' )  then
            swall(ilbl:ilbl) = ' '
            read(swall,*,iostat=ierr) isec
            if (ierr .gt. 0) then
               message='Error when reading the twall argument:'//swall
               call warnmsg
            end if
            imin  = isec/60
            isec  = isec - 60*imin
            ihour = imin/60
            imin  = imin - 60*ihour
            ihour = min(99,ihour)
            write(swall,"(i3.2,':',i2.2,':',i2.2)") ihour, imin, isec
         else if( swall(ilbl:ilbl) .eq. 'm' )  then
            swall(ilbl:ilbl) = ' '
            read(swall,*,iostat=ierr) imin
            if (ierr .gt. 0) then
              message='Error when reading the twall argument: '//swall
               call warnmsg
            end if
            isec  = 0
            ihour = imin/60
            imin  = imin - 60*ihour
            ihour = min(99,ihour)
            write(swall,"(i3.2,':',i2.2,':',i2.2)") ihour, imin, isec
         else if (swall(4:4).eq.':' .and. swall(7:7).eq.':' ) then
            continue
         else
            message = &
                 'Incorrect form of twall argument (hhh:mm:ss): '//swall
            call warnmsg
         endif
      endif

!-----------------------------------------------------------------------
! Actually seed the random number generator
!***********************************************************************
! If reinstating this, please ensure the array seed is deallocated after 
! it has been used (wherever that is appropriate).
!***********************************************************************
!-----------------------------------------------------------------------
!      if(allocated(seed)) then
!         call random_seed(put=seed)
!      else
!         call random_seed(size=seedsize)
!         allocate(seed(seedsize))
!         call random_seed(get=seed) !get default seed
!      endif
      return
      end subroutine setpaths

!#######################################################################
!
! module CHKINP
!
! checks that input file contains a run-section and does not contain 
! redundant / confusing sections
!
!#######################################################################

      subroutine chkinp

      implicit none

      integer(long), parameter               :: fnddim = 10
      integer(long), dimension(fnddim)       :: ifound = 0
      integer(long)                          :: ilbl,jlbl,klbl,jlbl1,klbl1
      integer(long)                          :: ic,iz,ierr

      rewind(iin)

!-----------------------------------------------------------------------
! look for unknown sections (possible typo in input)
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .gt. 0) then
         routine = 'Eingabe, chkinp'
         message = inptit
         call errormsg
      endif

      klbl=index(keyword(1),' ')-1
      jlbl=klbl-7
      if (jlbl .le. 0) go to 10
! jlbl1,klbl1 needed for e.g. hamiltonian-section_XXX
      klbl1=index(keyword(1),'_')-1
      jlbl1=klbl1-7
      if (jlbl1 .le. 0) then
         jlbl1=1
         klbl1=1
      endif
      if (keyword(1)(jlbl:klbl) .eq. '-section' .or. &
          keyword(1)(jlbl1:klbl1) .eq. '-section') then

! get ends of section name
         if (keyword(1)(1:4) .eq. 'end-') then
            ilbl=5
         else
            ilbl=1
         endif
         if (keyword(1)(jlbl1:klbl1) .eq. '-section') then
            jlbl=jlbl1-1
         else
            jlbl=jlbl-1
         endif

         if (keyword(1)(ilbl:jlbl) .eq. 'primitive-basis' .or. &
                 keyword(1)(ilbl:jlbl) .eq. 'pbasis') then
            ifound(1) = 1
         else if (keyword(1)(ilbl:jlbl) .eq. 'initial-geometry') then
            ifound(2) = 1
         else if (keyword(1)(ilbl:jlbl) .eq. 'operator') then
            ifound(3) = 1
         else if (keyword(1)(ilbl:jlbl) .eq. 'dirdyn') then
            ifound(4) = 1
         else if (keyword(1)(ilbl:jlbl) .eq. 'dd-gb') then
            ifound(5) = 1
         else if (keyword(1)(ilbl:jlbl) .eq. 'spf-basis' .or. &
              keyword(1)(ilbl:jlbl) .eq. 'sbasis') then
            ifound(6) = 1
         else if (keyword(1)(ilbl:jlbl) .eq. 'run') then
            continue
         else if (keyword(1)(ilbl:jlbl) .eq. 'init_wf') then
            continue
         else if (keyword(1)(ilbl:jlbl) .eq. 'parameter') then
            continue
         else if (keyword(1)(ilbl:jlbl) .eq. 'alter-parameters') then
            continue
         else if (keyword(1)(ilbl:jlbl) .eq. 'integrator') then
            continue
         else if (keyword(1)(ilbl:jlbl) .eq. 'sh_zagreb') then
            continue
         else if (keyword(1)(ilbl:jlbl) .eq. 'dd-gb' .or. &
              keyword(1)(ilbl:jlbl) .eq. 'dirdyn-gb') then
            continue
         else if (keyword(1)(ilbl:jlbl) .eq. 'ml-basis' .or. &
              keyword(1)(ilbl:jlbl) .eq. 'mlbasis') then
            continue
         else if (keyword(1)(ilbl:jlbl) .eq. 'spdo-basis') then
            continue
         else if (keyword(1)(ilbl:jlbl) .eq. 'gbf-basis') then
            continue
         else if (keyword(1)(ilbl:jlbl) .eq. 'op_define') then
            continue
         else if (keyword(1)(ilbl:jlbl) .eq. 'hamiltonian') then
            continue
         else if (keyword(1)(ilbl:jlbl) .eq. 'labels') then
            continue
         endif
      
      elseif (keyword(1).eq.'end-input') then
         go to 20

      endif

      go to 10
 
!-----------------------------------------------------------------------
! check that needed sections are present
!-----------------------------------------------------------------------
   20 continue
      if (ifound(2) .eq. 1) then
         if (ifound(1) .eq. 1) then
            routine = 'Chkinp'
            write(message,'(2a)') 'PBASIS-SECTION and ', &
                 'INITIAL-GEOMETRY-SECTION both present'
            call warnmsg
         endif
         if ((irunoper .eq. 1) .and. ifound(4) .eq. 0) then
            if (.not. lgwptrj) then
               routine = 'Chkinp'
               write(message,'(2a)') &
                 'If using an INITIAL-GEOMETRY-SECTION, a DIRDYN-SECTION ', &
                 'is needed'
               call warnmsg
            endif
         endif
      endif

      if(lddmctdh .and. ifound(5).ne.1)then
         message = 'DD-GB requested but no DD-GB-SECTION'
         call warnmsg
      endif

      if(ifound(5) .eq. 1)then
         if(ifound(1) .ne. 1)then
            write(message,'(2a)')'PBASIS-SECTION needed with ',&
                 'DD-GB-SECTION'
            call warnmsg
         endif
         if(ifound(6) .ne. 1)then
            write(message,'(2a)')'SBASIS-SECTION needed with ',&
                 'DD-GB-SECTION'
            call warnmsg
         endif
      endif       

      if (lwarn) then
         routine = 'Chkinp'
         message = 'Error in input file. See log file.'
         call errormsg
      endif

      return
      end subroutine chkinp

!***********************************************************************
!
!                  Subroutine  OPENIN
!
! Opens the input file  file.inp.
!
! Called from: eingabe
! Calls to:    None
!
! V7.0  5/97
!***********************************************************************

      subroutine openin

      implicit none

      integer(long)                 :: i,ierr,iarg,ilbl
      integer(long)                 :: cpppathlen
      logical(kind=4)               :: linpf, ldopt, lexist
      character(len=c5)             :: buf

      integer(long), external       :: myiargc

!-----------------------------------------------------------------------
! set default values
!-----------------------------------------------------------------------
      ierr    = 0
      linpf   = .false.
      ldopt   = .false.
      liopt   = .false.
      lusecpp = .false.

!-----------------------------------------------------------------------
! The input file and the options are read from the UNIX shell.
! There must be at least one argument. If there are no arguments read
! a short help text is output and the program is stopped. 
! If there is only one argument it is the name
! of the '~.inp' file (the name must not start with an '-').
! If there is more than one argument the last argument is taken as the
! name of the '~.inp' file. The other arguments are options (i.e. they 
! start with a '-').
! The '-I' option forces the program to read the input from the 
! 'name/input' file rather than an '~.inp' file.   
! Many options exclude each other. 
! If the argument list does not have a correct format
! a help text is output and the program is stopped.
!-----------------------------------------------------------------------

! GETARG is a UNIX specific call:
! mygetarg calls the getarg-routine of the present UNIX system 
! GET THE ZEROTH ARGUMENT (name of routine as called from bash)

      call mygetarg(0,progname)
      call rempath(progname,lprogname,ierr)
      if(ierr.eq.1) then
         routine='openin'
         ilbl = index(progname,' ')-1
         message=progname(1:ilbl)//' is a directory, not a file name.'
         call errormsg
      endif

! ----------------------------------------------------------------------
! GET No. OF ARGUMENTS
! ----------------------------------------------------------------------
      iarg = myiargc()
      if (iarg .lt. 1) call shorthelp

!-----------------------------------------------------------------------
! LOOP OVER THE ARGUMENTS AND READ THOSE RELATED TO INPUT FILE
!-----------------------------------------------------------------------
      i=1
      do while (i .le. iarg)
         call mygetarg(i,buf)

         if ( buf(1:2) .eq. '-h' .or.  buf(1:2) .eq. '-?' ) then
            call wrhelptext
         else if ( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            call stopnow
         else if( buf(1:2) .eq. '-I' ) then
            i = i + 1
            call mygetarg(i,buf)
            if (buf(1:1).eq.'-' .or.  buf(1:1).eq.' ') then
               write(6,'(a)') ' No pathname follows -I'
               call wrhelptext
            else
               liopt   = .true.
               name    = buf
               laenge  = index(name,' ')-1
            end if
         else if( buf(1:2) .eq. '-D' ) then
! -D option not read here, but checked against -I as both cannot be specified
            ldopt=.true.
         else if( buf(1:4) .eq. '-cpp' ) then
            lusecpp = .true.
         endif
         i=i+1
      enddo

!-----------------------------------------------------------------------
! get name of input file unless -I option set
!-----------------------------------------------------------------------
      if (.not. liopt) then
         i = iarg
         call mygetarg(i,buf)

         if( buf(1:1) .eq. '-' ) then
            write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
            ' The ~.inp file must be the last argument (stem only).'
            call stopnow
         else
            linpf = .true.
            inputfile = buf
         end if 
      endif

! CHECK FOR CONSISTENCY
      if( liopt .and. ldopt ) then
         write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
          ' -D and -I  options are incompatible'
         call stopnow
      end if
      if( liopt .and. linpf ) then
         write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
         ' There must be no ~.inp file when the -I option is given.'
         call stopnow
      end if
      if( .not.liopt .and. .not.linpf ) then
         write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
         ' No inputfile specified. Try option -h.'
         call stopnow
      end if


! FILE NAMES
      if( liopt ) then
         inputfile = name(1:laenge)//'/input'
         inplaenge = laenge + 6 
         call excmd('chmod 644 '//inputfile) ! Make file input read-write.
      else 
         inplaenge = index(inputfile,' ')-1
         if(inputfile(max(1,inplaenge-3):inplaenge) .ne. '.inp') then
            inputfile = inputfile(1:inplaenge)//'.inp'
            inplaenge = inplaenge + 4
         endif
         call abspath(inputfile,inplaenge)
      endif

! CHECK EXISTENCE OF INPUT FILE.
      inquire(file=inputfile,exist=lexist)
      if(.not.lexist) then
         write(6,'(a)')'-----------------------------------------------'
         write(6,'(a)') ' ##############   ERROR   ##############'
         write(6,'(a)') ' There is no input-file! The file:'
         write(6,'(a)') inputfile(1:inplaenge)
         write(6,'(a)') ' does not exist!'
         write(6,'(a)')'-----------------------------------------------'
         call stopnow
      end if

! --- IF CPP REQUESTED, RUN INPUT FILE THROUGH ---
      if( lusecpp ) then

! ------ READ PID  (PID was written by excmd to /tmp/QUANTICS_PID) ---
         pid=0
         open(iaus,file='/tmp/QUANTICS_PID',status='old', &
            form='formatted',err=8)
         read(iaus,*,err=8,end=8) pid
 8       close(iaus)
         
! ------ specify search path for #include, just in case
         cpppathlen = inplaenge
         do while(inputfile(cpppathlen:cpppathlen) .ne. '/')
            cpppathlen = cpppathlen - 1
         end do
          
         write(buf,'(i8.8)') pid

         call excmd('cpp -I'//inputfile(1:cpppathlen)//' ' &
           //inputfile(1:inplaenge)//' ' &
           //inputfile(1:inplaenge)//'.'//buf(1:8) &
           //'.post_cpp')

! ------ OPEN CPP OUTPUT FILE AS INPUT ---
         open(iin,file=inputfile(1:inplaenge)//'.'//buf(1:8)// &
            '.post_cpp',status='old',form='formatted',err=12)

      else

! ------ OPEN UNALTERED INPUT FILE ---
         open(iin,file=inputfile(1:inplaenge) &
            ,status='old',form='formatted',err=10)

      end if   ! End of use-cpp


      return

 10   continue
      write(6,*)
      write(6,'(a)') 'ERROR in subroutine Openin:'
      write(6,'(a)') 'Cannot open input file!'
      write(6,'(2a)') 'inputfile: ', inputfile(1:inplaenge)
      write(6,*)
      call stopnow

 12   continue
      write(6,*)
      write(6,'(a)') 'ERROR in subroutine Openin:'
      write(6,'(a)') 'Cannot open altered input file!'
      write(6,'(2a)') 'inputfile: ', inputfile(1:inplaenge) &
         //'.'//buf(1:8)//'.post_cpp'
      write(6,*)

      end subroutine openin

!***********************************************************************
!
!                  Subroutine  READOPTS
!
! Reads the options from the command line
!
! Called from: eingabe
! Calls to:    None
!
! V7.0  5/97
!***********************************************************************

      subroutine readopts(lmnd)

      implicit none

      integer(long)                 :: i,ierr,iarg,length,lencom,nopt
      real(dop)                     :: temperature
      logical(kind=4)               :: ldopt
      character(len=8)              :: flag
      character(len=c5)             :: buf
      logical(kind=4), intent(out)  :: lmnd
      logical(kind=4)               :: lcomma,ltfinal

      integer(long), external       :: myiargc

!-----------------------------------------------------------------------
! set default values
!-----------------------------------------------------------------------
      ierr    = 0
      nopt    = 0
      ldopt   = .false.
      lmnd    = .false.
      lusecpp = .false.
      ltfinal = .false.

!-----------------------------------------------------------------------
! The options are read from the UNIX shell.
! They start with a '-' possibly followed by a variable. The order of the  
! options is arbitrary. If an option must be specified by a variable
! the variable must follow the option but must be separated from it by
! a blank. (again the variable must not start with an '-'). 
! Some options exclude each other. 
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! get no. of arguments
!-----------------------------------------------------------------------
     iarg = myiargc()

!-----------------------------------------------------------------------
! look for any parameters and allocate arrays
!-----------------------------------------------------------------------
      nparcmd=0
      i=1
      do while (i .le. iarg)
         call mygetarg(i,flag)
         if (flag(1:3) .eq. '-p ') then
            nparcmd=nparcmd+1
         endif
         i = i + 1
      enddo

      if (nparcmd .gt. 0) then
         allocate(aparcmd(nparcmd))
         allocate(rparcmd(nparcmd))
! set counter back to 0 before reading in values
         nparcmd=0
         aparcmd(:)=' '
         rparcmd(:)=0.0
      endif

!-----------------------------------------------------------------------
! LOOP OVER THE ARGUMENTS 
!-----------------------------------------------------------------------
      i = 1
      rdopts: do while (i .le. iarg)

         call mygetarg(i,buf)

         flag    = buf(1:8)
         if( flag(1:1) .eq. '-' ) then
            length  = index(flag//' ',' ')-1
            length  = min(length, c5-1-optlaenge)
            if (optlaenge .eq. 0) then
               options = flag(1:length)//' '
            else
               options = options(1:optlaenge)//flag(1:length)//' '
            endif
            optlaenge = optlaenge+length+1
         end if

         if ( flag .eq. '-w' ) then
            lovwr  = .true.
         else if ( flag .eq. '-mnd' ) then
            lmnd  = .true.
         else if ( flag .eq. '-rd' ) then
            lrddvr = .true.
            lrdoper = .true.
            lrdinwf = .true.
         else if ( flag .eq. '-mpi' ) then
            lmpi  = .true.
         else if ( flag .eq. '-omp' ) then
            i = i + 1
            call mygetarg(i,buf)
            length  = index(buf,' ')-1
            length  = min(length,c5-1-optlaenge)
            options = options(1:optlaenge)//buf(1:length)//' '
            optlaenge = optlaenge+length+1
            read(buf,*,iostat=ierr) ompthread
            if(buf(1:1).eq.' '.or.buf(1:1).eq.'-'.or.ierr.ne.0) then
               write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
                    ' -omp is not followed by an integer.'
               call stopnow
            endif

            if (ompthread .gt. 1) then
               lompthread  = .true.
            endif
         else if ( flag .eq. '-gmat' ) then
            lrunpes  = .true.
            ltime    = .false.
            i = i + 1
            call mygetarg(i,buf)
            length  = index(buf,' ')-1
            length  = min(length,c5-1-optlaenge)
            options = options(1:optlaenge)//buf(1:length)//' '
            optlaenge = optlaenge+length+1
            read(buf,*,iostat=ierr)  gmatf1
            if(buf(1:1).eq.' '.or.buf(1:1).eq.'-'.or.ierr.ne.0) then
               write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
                    ' -gmat is not followed by an integer.'
               call stopnow
            endif
            i = i + 1
            call mygetarg(i,buf)
            length  = index(buf,' ')-1
            length  = min(length,c5-1-optlaenge)
            options = options(1:optlaenge)//buf(1:length)//' '
            optlaenge = optlaenge+length+1
            read(buf,*,iostat=ierr)  gmatf2
            if(buf(1:1).eq.' '.or.buf(1:1).eq.'-'.or.ierr.ne.0) then
               write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
                    ' -gmat is not followed by a second integer.'
               call stopnow
            endif

         else if ( flag .eq. '-s' ) then
            nopt   = nopt + 1
            test0  = .true.
         else if( flag .eq. '-t') then
            nopt  = nopt + 1
            lcont = .false.
            test  = .true.
            frunlevel = 4
         else if( flag .eq. '-c' ) then
            nopt  = nopt + 1
            lcont  = .true.
            lrddvr = .true.
            lrdoper = .true.
            lrdinwf = .true.
         else if( flag .eq. '-ci' ) then
            nopt  = nopt + 1
            lcont  = .true.
            leinint = .true.
         else if( flag .eq. '-D' ) then
            i = i + 1
            call mygetarg(i,buf)
            if (buf(1:1).eq.'-' .or.  buf(1:1).eq.' ') then
               write(6,'(a)') ' No pathname follows -D'
               call stopnow
            else
               ldopt   = .true.
               name    = buf
               laenge  = index(name,' ')-1
               length  = min(laenge,c5-1-optlaenge)
               options = options(1:optlaenge)//name(1:length)//' '
               optlaenge = optlaenge+length+1
            end if
         else if( flag .eq. '-I' ) then
            i = i + 1
            call mygetarg(i,buf)
            if (buf(1:1).eq.'-' .or.  buf(1:1).eq.' ') then
               write(6,'(a)') ' No pathname follows -I'
               call stopnow
            else
               liopt   = .true.
               name    = buf
               laenge  = index(name,' ')-1
               length  = min(laenge,c5-1-optlaenge)
               options = options(1:optlaenge)//name(1:length)//' '
               optlaenge = optlaenge+length+1
            end if
         else if( flag .eq. '-tstop' ) then
            i = i + 1
            call mygetarg(i,buf)
            length = index(buf,' ') - 1
            if(buf(2:2).eq.':') then
               buf =' 0'//buf(1:length)
               length = length + 2
            elseif(buf(3:3).eq.':') then
               buf =' '//buf(1:length)
               length = length + 1
            endif
            if (buf(4:4).ne.':') then
               write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
              ' -tstop is not followed by a valid time-string.'
               call stopnow
            else
               sstop   =  buf(1:8)
               lstopf  = .true.
               length  = min(5,c5-1-optlaenge)
               options = options(1:optlaenge)//sstop(1:length)//' '
               optlaenge = optlaenge+length+1
            end if
         else if( flag .eq. '-tcpu' ) then
            i = i + 1
            call mygetarg(i,buf)
            length = index(buf,' ') - 1
            if(buf(2:2).eq.':') then
               buf =' 0'//buf(1:length)
               length = length + 2
            elseif(buf(3:3).eq.':') then
               buf =' '//buf(1:length)
               length = length + 1
            endif
            if(buf(4:4).eq.':' .and. buf(7:8) .eq. '  ') then
               buf = buf(1:6)//':00'
               length = 9
            endif
            if ((buf(4:4).ne.':'.or. buf(7:7).ne.':') .and. &
                (buf(length:length) .ne. 's') .and.         &
                (buf(length:length) .ne. 'm') ) then
               write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
              ' -tcpu is not followed by a valid time-string.'
               call stopnow
            else
               scpu    =  buf(1:10)
               lstopf  = .true.
               options = options(1:optlaenge)//scpu//' '
               length  = min(8,c5-1-optlaenge)
               options = options(1:optlaenge)//scpu(1:length)//' '
               optlaenge = optlaenge+length+1
            end if
         else if( flag .eq. '-twall' ) then
            i = i + 1
            call mygetarg(i,buf)
            length = index(buf,' ') - 1
            if(buf(2:2).eq.':') then
               buf =' 0'//buf(1:length)
               length = length + 2
            elseif(buf(3:3).eq.':') then
               buf =' '//buf(1:length)
               length = length + 1
            endif
            if(buf(4:4).eq.':' .and. buf(7:8) .eq. '  ') then
               buf = buf(1:6)//':00'
               length = 9
            endif
            if ((buf(4:4).ne.':'.or. buf(7:7).ne.':') .and. &
                (buf(length:length) .ne. 's') .and. &
                (buf(length:length) .ne. 'm') ) then
               write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
              ' -twall is not followed by a valid time-string.'
               call stopnow
            else
               swall    =  buf(1:10)
               lstopf  = .true.
               options = options(1:optlaenge)//swall//' '
               length  = min(8,c5-1-optlaenge)
               options = options(1:optlaenge)//swall(1:length)//' '
               optlaenge = optlaenge+length+1
            end if
         else if( flag .eq. '-tfinal' ) then
            if (ltfinal) then
               write(6,'(3a)') 'Length of propagation specified twice. '
               write(6,'(3a)') 'Don not use both "tfinal" and "thermal"'
               call stopnow
            else
               ltfinal = .true.
            endif

            i = i + 1
            call mygetarg(i,buf)
            length  = index(buf,' ')-1
            length  = min(length,c5-1-optlaenge)
            options = options(1:optlaenge)//buf(1:length)//' '
            optlaenge = optlaenge+length+1
            read(buf,*,iostat=ierr)  tfinal
            if(buf(1:1).eq.' '.or.buf(1:1).eq.'-'.or.ierr.ne.0) then
               write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
                    ' -tfinal is not followed by a real number.'
               call stopnow
            end if
         else if( flag .eq. '-tinit' ) then
            i = i + 1
            call mygetarg(i,buf)
            length  = index(buf,' ')-1
            length  = min(length,c5-1-optlaenge)
            options = options(1:optlaenge)//buf(1:length)//' '
            optlaenge = optlaenge+length+1
            read(buf,*,iostat=ierr)  tinit
            if(buf(1:1).eq.' '.or.buf(1:1).eq.'-'.or.ierr.ne.0) then
               write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
                    ' -tinit is not followed by a real number.'
               call stopnow
            end if
            ltinitcmd = .true.

         else if( flag .eq. '-thermal' ) then
            if (ltfinal) then
               write(6,'(3a)') 'Length of propagation specified twice. '
               write(6,'(3a)') 'Don not use both "tfinal" and "thermal"'
               call stopnow
            else
               ltfinal = .true.
            endif
            lrlxt=.true.
            i = i + 1
            call mygetarg(i,buf)
            length  = index(buf,' ')-1
            length  = min(length,c5-1-optlaenge)
            options = options(1:optlaenge)//buf(1:length)//' '
            optlaenge = optlaenge+length+1
!........If    the comma is not surounded by blanks, lcomma is true.
            lencom  = index(buf,',')-1
            if( lencom.gt.0 .and. lencom.lt.length ) then
               lcomma = .true.
               length = lencom
            else
               lcomma = .false.
            endif
            read(buf(1:length),*,iostat=ierr)  temperature
            if(buf(1:1).eq.' '.or.buf(1:1).eq.'-'.or.ierr.ne.0) then
               write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
                    ' -thermal is not followed by a real number.'
               call stopnow
            end if
            rlxtime = 'K'
            fs = 1.0_dop
            tfinal=1.0_dop/(2.0_dop*temperature)
            if(lcomma) then
               buf = buf(lencom+1:)
               if(buf .eq. ',') optlaenge = optlaenge-1
            else
               i = i + 1
               call mygetarg(i,buf)
            endif
            if (buf .eq. ',') then
               i = i + 1
               call mygetarg(i,buf)
               lcomma=.false.
            elseif(buf(1:1) .eq. ',') then
               buf=buf(2:)
            else
               write(6,'(2a)') 'Missing comma between first ', &
                    'and second argument of option: -thermal'
               call stopnow
            endif
            read(buf,*,iostat=ierr) mlseed
            if (ierr .ne. 0 .or. mlseed .le.0) then
               write(6,'(a)') &
               'Incorrect second argument (seed) for option : -thermal'
               call stopnow
            endif
            if(.not.lcomma) then
               length  = index(buf,' ')-1
               length  = min(length,c5-1-optlaenge)
               options = options(1:optlaenge-1)//','//buf(1:length)//' '
               optlaenge = optlaenge+length+1
            endif

         else if( flag .eq. '-twall' ) then
            i = i + 1
            call mygetarg(i,buf)
            length = index(buf,' ') - 1
            if(buf(2:2).eq.':') then
               buf =' 0'//buf(1:length)
               length = length + 2
            elseif(buf(3:3).eq.':') then
               buf =' '//buf(1:length)
               length = length + 1
            endif
            if(buf(4:4).eq.':' .and. buf(7:8) .eq. '  ') then
               buf = buf(1:6)//':00'
               length = 9
            endif
            if ((buf(4:4).ne.':'.or. buf(7:7).ne.':') .and. &
                 (buf(length:length) .ne. 's') .and.         &
                 (buf(length:length) .ne. 'm') ) then
               write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
                    ' -twall is not followed by a valid time-string.'
               call stopnow
            else
               swall   =  buf(1:10)
               lstopf  = .true.
               options = options(1:optlaenge)//swall//' '
               length  = min(8,c5-1-optlaenge)
               options = options(1:optlaenge)//swall(1:length)//' '
               optlaenge = optlaenge+length+1
            end if
         else if( flag .eq. '-p' ) then
            
            i = i + 1
            call mygetarg(i,buf)

            length  = index(buf,' ')-1
            length  = min(length,c5-1-optlaenge)
            options = options(1:optlaenge)//buf(1:length)//' '
            optlaenge = optlaenge+length+1

            if(buf(1:1).eq.' '.or.buf(1:1).eq.'-') then
               write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
                    ' -p is not followed by a parameter name.'
               call stopnow
            else
               nparcmd=nparcmd+1
               aparcmd(nparcmd)(1:length)=buf(1:length)
            end if

            i = i + 1
            call mygetarg(i,buf)

!........If    the comma is not surounded by blanks, lcomma is true.
            lencom  = index(buf,',')-1
            length  = index(buf,' ')-1
            if( lencom.gt.0 .and. lencom.lt.length ) then
               lcomma = .true.
               length = lencom
            else
               lcomma = .false.
            endif
            length  = min(length,c5-1-optlaenge)
            options = options(1:optlaenge)//buf(1:length)//' '
            optlaenge = optlaenge+length+1
            read(buf,*,iostat=ierr) rparcmd(nparcmd)
!    following line (up to v9.13) did not allow negative numbers!
!           if(buf(1:1).eq.' '.or.buf(1:1).eq.'-'.or.ierr.ne.0) then
            if(buf(1:1).eq.' '.or.ierr.ne.0) then
               length  = index(aparcmd(nparcmd),' ')-1
               write(6,'(/,a/,3a)') '*********** ERROR ! ***********', &
                    ' -p ',aparcmd(nparcmd)(1:length), &
                    ' is not followed by a real number.'  
               call stopnow
            end if
            if(lcomma) then
               buf = buf(lencom+1:)
            else
               i = i + 1
               call mygetarg(i,buf)
            endif
            if( buf(1:1) .eq. ',' ) then
               if( buf .eq. ',' ) then
                  i = i + 1
                  call mygetarg(i,buf)
               else
                  buf = buf(2:)
               endif
               call keylowc(buf)
               call keyunits(rparcmd(nparcmd),buf,ierr)
               if(buf(1:1).eq.' '.or.buf(1:1).eq.'-'.or.ierr.ne.0) then
                  length  = index(aparcmd(nparcmd),' ')-1
                  write(6,'(/,a/,2a,f10.6,a)') &
                       '*********** ERROR ! ***********', &
                       ' -p ',aparcmd(nparcmd)(1:length),rparcmd(nparcmd), &
                       ' not followed by a unit.'
                  call stopnow
               end if
               length  = index(buf,' ')-1
               length  = min(length,77-optlaenge)
               options = options(1:optlaenge-1)//','//buf(1:length)//' '
               optlaenge = optlaenge+length+1
            else
               i = i - 1
            endif

         else if( flag .eq. '-dbm' ) then
            
            i = i + 1
            call mygetarg(i,buf)
            read(buf,*) dbmemdim0

! options used in openin added here for check of options
         else if (buf(1:4) .eq. '-cpp') then
            continue

         else if (buf(1:1) .eq. '-' .and. i .lt. iarg) then
            write(6,'(/,a/,2a/,a)') '*********** ERROR ! ***********', &
                 ' Unknown option: ', flag, &
                 ' Try option -h.'
            call stopnow
         end if   

         i=i+1
      enddo rdopts

! CHECK FOR CONSISTENCY
      if( nopt .gt. 1 ) then
         write(6,'(/,a/,a)') '*********** ERROR ! ***********', &
         ' -s, -t, and -c  options are incompatible'
         call stopnow
      end if

! FILE NAMES
      if( ldopt .or. liopt ) then
         laenge = index(name,' ')-1
         call abspath(name,laenge)
         if (ierr .eq. 1  .or.  laenge .gt. 155 ) then
            write(6,*) &
            'error in abspath, ''name'' too long or empty.'
            write(6,*) 'Laenge = ', Laenge
            write(6,*) 'Output directory : '
            write(6,*)  name
            STOP 1
         endif
      endif

      return
      end subroutine readopts

!***********************************************************************

      subroutine settask

      implicit none

!-----------------------------------------------------------------------
! sort out which sections have to be run
!-----------------------------------------------------------------------
      if (frunlevel .eq. 4) then
         irundvr =1
         irunoper=1
         iruninwf=1
         irunprop=1
      else if (frunlevel .eq. 3) then
         irundvr =1
         irunoper=1
         iruninwf=1
         irunprop=0
      else if (frunlevel .eq. 2) then
         irundvr =1
         irunoper=1
         iruninwf=0
         irunprop=0
      else if (frunlevel .eq. 1) then
         irundvr =1
         irunoper=0
         iruninwf=0
         irunprop=0
      else if (frunlevel .eq. 0) then
         irundvr =0
         irunoper=0
         iruninwf=0
         irunprop=0
      endif

!-----------------------------------------------------------------------
! if file to be read, flag
!-----------------------------------------------------------------------
      if (lrddvr .and. irundvr .eq. 1)  irundvr =2
      if (lrdoper .and. irunoper .eq. 1) irunoper=2
      if (lrdinwf .and. iruninwf .eq. 1) iruninwf=2

!-----------------------------------------------------------------------
! select propagation task
!-----------------------------------------------------------------------
      if (irunprop .eq. 1) then
         if (lrunshzagreb) then
            proptask = 5
         else if (lrundiag) then
            proptask = 4
         else if (ltraj) then
            proptask = 3
         else if (lrealwp) then
            proptask = 2
         else
            proptask = 1
         endif
      endif

      return
      end subroutine settask

!***********************************************************************
!
!                  Subroutine  OPENLOG
!
! Opens the log file in "name" directory.
!
! Called from: eingabe
! Calls to:    None
!
! V7.0  5/97
!***********************************************************************

      subroutine openlog

      implicit none

      character(len=c5)       :: buffer
      integer(long)           :: i

! --- READ PID  (PID was written by excmd to /tmp/QUANTICS_PID) ---
      pid=0
      buffer = '/tmp/QUANTICS_PID'
      open(iaus,file=buffer,status='old',form='formatted',err=8)
      read(iaus,*,err=8,end=8) pid
 8    close(iaus,status='delete',err=9)

! --- Write the PID to a string for use in keeping track of concurrenct processes
      write(strpid,'(i6)')pid
      strpid = adjustl(strpid)

! --- OPEN LOG FILE ---

 9    if(lrunpes) then
         buffer=name(1:laenge)//'/log.pes'
         i=laenge+8
      else
         buffer=name(1:laenge)//'/log'
         i=laenge+4
      endif
      if (lcont) lovwr= .false.
      if (lovwr .or. lcont) then
         open(ilog,file=buffer(1:i),status='unknown', &
              form='formatted',err=20)
      else
         open(ilog,file=buffer(1:i),status='new', &
              form='formatted',err=30)
      endif
      logisopen = .true.

! --- RE-POSITION LOG FILE ---
      if (lcont) then
 10      read(ilog,'(a)',end=12)
         go to 10
 12      backspace(ilog)
         write (ilog,'(/,78a,/)') ('#',i=1,78) 
      endif

! --- WRITE DATE AND MACHINE TO LOG FILE ---

      write(ilog,*)
      write(ilog,'(a80)') trim(cstartup(1:80))
      write(ilog,'(1x,a)') name(1:laenge)

! --- WRITE PID TO LOG FILE  ---
      if(pid.gt.0) write(ilog,'(/a,i6)') ' PID =', pid-1
 
      call wrversion(ilog)
 
      If( test ) then
         write(ilog,'(a,16x,a,16x,a)') '###############', &
                         ' TEST RUN ', '###############'
      else if( lcont ) then
         write(ilog,'(a,13x,a,13x,a)') '###############', &
                   'CONTINUATION RUN', '###############'
         if (leinint) then
            write(ilog,'(23x,a)') 'with new integrator'
         endif
      else 
         write(ilog,'(a,16x,a,16x,a)') '###############', &
                         ' NEW RUN  ', '###############'
      end if
 
      if(runtitle(1:10) .ne. '          ' ) then
         write(ilog,'(/,1x,a)') trim(runtitle(1:78))
         write(ilog,'(/,72a1)')  ('#',i=1,72)
      endif

      if(optlaenge .gt. 1) then
         write(ilog,'(/a)')  '******* OPTIONS ****** '
         write(ilog,'(a/)')  options(1:optlaenge-1)
      end if

      call myflush(ilog)

      return

! --- WRITE ERROR MESSAGE TO SCREEN ---

 20   continue
      write(6,*)
      write(6,'(a)') 'ERROR in subroutine Openlog:'
      write(6,'(a)') 'Cannot open log file! Directory non-existent?'
      write(6,'(a)') 'Logfile: ', name(1:laenge)//'/log'
      write(6,*)
      call stopnow

 30   continue
      write(6,*)
      write(6,'(a)') 'ERROR in subroutine Openlog:'
      write(6,'(a,/a)') &
          'Cannot open log file!' &
         ,'overwrite disabled? (Use -w option to enable overwrite)'
      if(laenge .le. 64) then
         write(6,'(2a)') 'logfile: ', name(1:laenge)//'/log'
      else
         write(6,'(a,/a)') 'logfile: ', name(1:laenge)//'/log'
      end if
      write(6,*)
      call stopnow

      end subroutine openlog

!#######################################################################
! *** V5.0 MB ************> SUBROUTINE CPINPUT <*************************
!                          ~~~~~~~~~~~~~~~~~~~

! Copies the input file to the name directory.
! Writes file information to log-file.

      subroutine cpinput

      implicit None

      integer(long), external  :: slen
      integer(long)            :: i,ilbl,er10,er11
      logical(kind=4)          :: lexist, lexist1, check
      character(len=360)       :: filename,filop1

! --- Do not produce an input file if an inputfile and a logfile already exist. 
      filename=name(1:laenge)//'/log'
      inquire(file=filename,exist=lexist1)
      filename=name(1:laenge)//'/input'
      inquire(file=filename,exist=lexist)
      if(lexist .and. lexist1) return
      
! --- ADD PROGRAM VERSION AND OPTIONS ---

      write(idat,'(a17,f8.3)',err=10) '# HDmctdh ProgramVersion = ',progver
      if(optlaenge .gt. 1) then
         write(idat,'(a)',err=10) '# Command line options: '
         write(idat,'(2a)') '#', options(1:optlaenge)
      end if
      write(idat,'(1x,/,80a1)') '#',('*',i=1,78),'#'

! --- IF liopt OPTION IS SET, JUST APPEND OPTIONS TO INPUTFILE ---
! --- liopt is .true. if input is read from name/input  ---

      if(liopt) then         ! -I option is set

 40      read(iin,'(a)',end=30)
         goto 40
 30      backspace iin
         if(optlaenge .gt. 1) then
            write(iin,'(a/)')  options(1:optlaenge)
         end if

      else

! --- OTHERWISE COPY THE INPUT FILE ---

         er10=1
         rewind(iin,err=10)

         if(lexist) then  ! Make file input read-write.
            call excmd('chmod 644 '//filename)
         endif
         er10=2
         open(idat,file=filename,status='unknown',action='readwrite', &
              form='formatted',err=13)
         if(lcont) then
 50         read(idat,'(a)',end=55)
            goto 50
 55         backspace idat
            write(idat,'(/16x,a)')'### ---- CONTINUATION RUN ---- ###'
         else
            write(idat,'(2a)') '# ',inputfile(1:inplaenge)
         end if
         write(idat,'(16x,a,/)')  '### ------- INPUT FILE ------- ###'
         do
            er10=3
            read(iin,'(a)',err=10,end=20) inptit
            er10=4
            if (inptit(1:1) .ne. '#'     .and. &
                inptit(1:5) .ne. '-----' .and. &  
                inptit(1:5) .ne. '_____' .and. &
                slen(inptit) .gt. 0 ) then
                write(idat,'(a)',err=10) inptit(1:slen(inptit)) 
               call keylowc(inptit(1:10))
               ilbl=index(inptit,'end-input')
               if (ilbl .gt. 0) exit
            endif
         enddo

! --- AND COPY THE OPERATOR FILE ---
         if( iop .ne. iin .and. oplaenge .gt. 0) then
            inquire(iop,opened=check)
            if (.not. check) then
               filename= &
                  oppath(1:opplaenge)//'/'//opname(1:oplaenge)//'.op'
               er11=1
               open(iop,file=filename,status='unknown', &
                    form='formatted',err=12)
            else
               rewind(iop)
            endif
            write(idat,'(16x,a,/)')'### ------ OPERATOR FILE ------ ###'
 300        continue
            er11=2
            read(iop,'(a)',err=11,end=21) inptit
            er11=3
            if (inptit(1:1) .ne. '#' .and. slen(inptit).gt.0)  &
                 write(idat,'(a)',err=11) inptit(1:slen(inptit)) 
            call keylowc(inptit)
            ilbl=index(inptit,'end-operator')
            if (ilbl .gt. 0) goto 400
            Goto 300
 400        write(idat,'(1x,/,80a1)') '#',('*',i=1,78),'#'
         end if

!--- CLOSE FILES ---

         er10=5
         endfile(idat,err=10)
         er10=6
         close(idat,err=10)
         er11=4
         close(iop,err=11)
!...Make file input read only.
         call excmd('chmod 444 '//name(1:laenge)//'/input')
      end if

! --- WRITE INPUT FILE PATH NAMES TO LOG FILE ---

      write(ilog,'(/,72a/)') ('#',i=1,72)
      write(ilog,'(a)') ' Paths and input files '
      write(ilog,'(/,72a/)') ('#',i=1,72)
      if(.not. liopt) write(ilog,'(a)') &
         '~.inp  file copied to input in name directory'
      write(ilog,'(a12,a)') 'name      : ', name(1:laenge)
      write(ilog,'(a12,a)') 'input     : ', inputfile(1:inplaenge)
      if (dlaenge .gt. 0) write(ilog,'(a12,a)') 'DVR file  : ', dname(1:dlaenge)
      if (olaenge .gt. 0) write(ilog,'(a12,a)') 'OPER file : ', oname(1:olaenge)
      if (irunoper .eq. 1) then
         if (iop .ne. iin) then
            filop1=oppath(1:opplaenge)//'/'//opname(1:oplaenge)//'.op'
         else
            filop1=inputfile(1:inplaenge)
         endif
         write(ilog,'(a12,a)') '.op file  : ', filop1(1:slen(filop1))
      endif
      if(rlaenge  .gt. 0 ) write(ilog,'(a12,a)') 'INWF file : ', rname(1:rlaenge)
      if(parlaenge.gt. 0 ) write(ilog,'(a12,a)') 'parfile   : ', parfile(1:parlaenge)
      if(crosslng .gt. 0 ) write(ilog,'(a12,a)') &
                  'cross-ref : ', crossname(1:crosslng)//'/restart'

      if(lovwr) then
         write(ilog,'(/,a/)') 'overwrite enabled'
      else
         write(ilog,'(/,a/)') 'overwrite disabled'
      end if

      call myflush(ilog)
      call myflush(iin)

      return

! --- WRITE ERROR MESSAGE TO LOG FILE ---

 10   continue
      write(6,'(a,i3)') 'cpinput error-number =', er10
      routine = 'cpinput'
      message = 'Cannot copy ~.inp file to output directory'
      call errormsg

 20   continue
      routine = 'cpinput'
      message = &
         'end of ~.inp file found. END-INPUT keyword not present?'
      call errormsg

 11   continue
      write(6,'(a,i3)') 'cpinput error-number =', er11
      routine = 'cpinput'
      message = 'Cannot copy ~.op file to output directory'
      call errormsg

 12   continue
      write(6,'(a,a)') '~.op file: ',filename(1:slen(filename))
      write(ilog,'(a,a)') '~.op file: ',filename(1:slen(filename))
      routine = 'cpinput'
      message = 'Cannot open ~.op file'
      call errormsg

 13   continue
      write(6,'(a,a)') 'file input: ',filename(1:slen(filename))
      write(ilog,'(a,a)') 'file input: ',filename(1:slen(filename))
      routine = 'cpinput'
      message = 'Cannot open file "input"'
      call errormsg

 21   continue
      routine = 'cpinput'
      message = &
         'end of ~.op file found. END-OPER keyword not present?'
      call errormsg

      end subroutine cpinput
!

!***********************************************************************
!
! WRHELPTEXT writes Unix-like help text to screen if arguments of the MCTDH
! of the MCTDH executable are not correct.
!
!***********************************************************************

      subroutine wrhelptext

      integer(long)       :: i,version,release

      call getrevno(progver,version,release)

      write(6,'(/,80a1,/)') ('#',i=1,80)
      write(6,'(a,i3,a,i4,a,i4,a,/)')  &
        ' ----  QUANTICS  Version', version,'   ---    Release ',  &
         release,'   ----' 

      write(6,'(2a)') &
          'Purpose: Solves the Time-dependent Schroedinger Equation for ',&
          'Molecular Quantum Dynamics Simulations'
      write(6,'(/a/,3(18x,a/))') &
#ifdef OMP
        'Usage:   quantics [-h|-?|-rd|-w|-s|-t|-cpp|-mnd|-omp n|-c]', &
#elif MPI
        'Usage:   quantics [-h|-?|-rd|-w|-s|-t|-cpp|-mnd|-mpi|-c]', &
#else
        'Usage:   quantics [-h|-?|-rd|-w|-s|-t|-cpp|-mnd|-c]', &
#endif
        '[-tstop hhh:mm] [-tcpu hhh:mm:ss] [-twall hhh:mm:ss]', &
        '[-tfinal real] [-tinit real] [-gmat f1 f2] [-p apar rpar]',& 
        '[-dbm size] [-D name] inpf'
      write(6,'(a/9x,a)') &
           '  inpf:  input file (with or without extension ''.inp'').',  &
           'The input file must be the last argument in the list.'

      write(6,'(a)') &
        '  or in place of "[-D name] inpf" use "[-I name]" '
      write(6,'(/,a)') 'Options:'
      write(6,'(a)') &
           '      -h : print this help-text.'
      write(6,'(a)') &
           '      -? : print this help-text.'
      write(6,'(2a)') &
           '    -ver : version information.' 
      write(6,'(2a)') &
           '     -rd : reading of the dvr-, oper- and restart- files', &
           ' is enforced.'
      write(6,'(a)') &
           '      -w : overwrite enabled.'
      write(6,'(2a)') &
           '      -s : short test-run: Checks input and stops before ', &
           'running a propagation. '
      write(6,'(a,/,a)') &
           '      -t : test-run: Stops after first energy calculated ', &
           '           (run_type in inpf file ignored).'
      write(6,'(a,3(/11x,a))') &
           '    -cpp : run input file <inpf> through C preprocessor.',  &
           'Allows #include "settings.inc" of consistent settings in '// &
           'several', &
           'input files. Comment input in the form #/* comment */', &
           'For more information, refer to the cpp documentation.'
      write(6,'(2a)') &
           '    -mnd : Make Name Directory.', &
           ' Creates the name directory, if not existent.'
#ifdef OMP
      write(6,'(2a)') &
           '    -omp : "n" is no. of processors for an OMP calculation,', &
           ' (run_type in inpf file ignored).'
#endif
#ifdef MPI
      write(6,'(2a)') &
           '    -mpi : Use multiple cores in an MPI run,', &
           ' Use "mpirun -np N quantics.mpi -mpi input".'
#endif
      write(6,'(2a)') &
           '   -gmat : "f1 f2" denote the DOFs for which a Gmat-file', &
           ' is generated.'
      write(6,'(2a)') &
           '      -c : continuation-run,', &
           ' (run_type in inpf file ignored).'
      write(6,'(2a)') &
           '     -ci : continuation=integrator run,', &
           ' (run_type in inpf file ignored).'
      write(6,'(2a)') &
           '      -D : ''name'' denotes the directory where files ', &
           'are written to,'
      write(6,'(29x,a)') &
           '(name in inpf file ignored).'
      write(6,'(2a)') &
           '      -I : ''name'' denotes the directory where files ', &
           'are written to.'
      write(6,'(11x,a)') &
           'The input is read from name/input and not from inpf file.'
      write(6,*)
      write(6,'(2a/29x,a)') &
           '  -tstop : ''hhh:mm'' denotes the (real) time ', &
           'after which program is stopped,', &
           '(tstop in inpf file ignored).'
      write(6,'(2a/11x,a/11x,a/29x,a)') &
           '   -tcpu : ''hhh:mm:ss'' denotes the cpu time ', &
           'after which program is stopped,', &
           'The short forms ''Is'' or ''Im'' are also possible where', &
           'I=integer, ''s'' stands for seconds and ''m'' for minutes.', &
           '(tcpu  in inpf file ignored).'
      write(6,'(2a/11x,2a/29x,a)') &
           '  -twall : ''hhh:mm:ss'' denotes the elapsed wall-time ', &
           'after which program', &
           'is stopped. Short forms similar as for tcpu ', &
           'are also possible.', &
           '(twall  in inpf file ignored).'
      write(6,'(2a,/29x,a)') &
           ' -tfinal : ''real'' denotes the final integration', &
           ' time in fs,','(tfinal in inpf file ignored).'
      write(6,'(2a,/29x,a)') &
           ' -tinit  : ''real'' denotes the initial integration', &
           ' time in fs,','(tinit in inpf file ignored).'
      write(6,'(2a,/11x,a,/11x,a)') &
           '      -p : ''apar'' is the name of a parameter, with ', &
           'real value ''rpar'',','needed to define the Hamiltonian.', &
           'The value ''rpar'' may carry a unit. E.g.: -p en 5,eV' 
      write(6,'(a,/,a)') &
           '    -dbm : "size" integer specifying the max no. of DB records ', &
           '           for a DD run,'

      write(6,'(//2a/,8x,a)') &
           '  Note: Only one of the options -s, -o, -t, -c or -ci', &
           '  may be used!', &
           'Only one of the options -D or -I  may be used!'
      write(6,'(/2a)')' The order of the options does not matter,', &
                           ' but inpf must be the last argument.'
      write(6,'(/,80a1)') ('-',i=1,80)

      call stopnow
      end subroutine wrhelptext

!***********************************************************************
! SHORTHELP writes a short help text to screen (if there is no input)  
!***********************************************************************

      subroutine shorthelp

      implicit none

      integer(long)       :: version,release,i

      write(6,'(80a1)') ('-',i=1,80)
      write(6,'(2a)') ' QUANTICS. A program for doing',  &
                      ' multi-dimensional quantum dynamics.' 
      call getrevno(progver,version,release)
      write(6,'(a,i3,a,i4,a,i4,a,/)') &
        ' ----  QUANTICS  Version', version,'   ---    Release ', &
         release,'   ----' 


      write(6,'(a)') 'Usage:  quantics [options] "inputfile"'

      write(6,'(/,2a)') ' -h   : ', &
             'Provides a help text explaining the options.'
      write(6,'(a)')    ' -ver : Version Information.'
      
      write(6,'(/,a80,/,80a1)') dateline, ('-',i=1,80)
 
      call stopnow
      end subroutine shorthelp

!***********************************************************************

      end module eingabemod

