!***********************************************************************
!
!            MCTDH  subroutines  WRHELPTEXT and MAXINFO
! 
! WRHELPTEXT writes Unix-like help text to screen if arguments of the MCTDH
! of the MCTDH executable are not correct.
!
! MAXINFO provides information on the setting of the parameters (option -max).
!
! SHORTHELP writes a short help text to screen (if there is no input)  
!
! contains wrhelptext,maxinfo,shorthelp
!                                                                      
!  01/98 AJ, 11/98 HDM
!***********************************************************************
      module wrhtxt 

use decimal, only: long
use logdat
use versions
use cdate
use global


      implicit none

      private
      public :: wrhelptext,shorthelp,maxinfo


      contains
!***********************************************************************

      subroutine wrhelptext

      integer(long)       :: i,version,release

      call getrevno(progver,version,release)

      write(6,'(/,80a1,/)') ('#',i=1,80)
      write(6,'(a,i3,a,i4,a,i4,a,/)')  &
        ' ----  QUANTICS  Version', version,'   ---    Release ',  &
         release,'   ----' 

      write(6,'(2a)') &
          'Purpose: Solves the Time-dependent Schroedinger Equation for ',&
          'Molecular Quantum Dynamics Simulations'
      write(6,'(/2a/,18x,a/,18x,a/,18x,a)') &
        'Usage:   quantics [-h|-?|-max|-rd|-w|-s|-o|-cpp|-mnd|-pes|', &
        '-t|-c]', &
        '[-tstop hhh:mm] [-tcpu hhh:mm:ss] [-twall hhh:mm:ss]', &
        '[-tfinal real] [-tinit real] [-gmat f1 f2] [-p apar rpar]', &
        '[-D name] inpf'
      write(6,'(/2a/,18x,a/,18x,a/,18x,a)') &
        '  or :   quantics [-h|-?|-max|-rd|-w|-s|-o|-mnd|-pes|-t|-c]', &
        '[-tstop hhh:mm] [-tcpu hhh:mm:ss] [-twall hhh:mm:ss]', &
        '[-tfinal real] [-tinit real] [-thermal real,int]', &
        '[-gmat f1 f1] [-p apar rpar] -I name '
      write(6,'(a/9x,a)') &
           '  inpf:  input file (with or without extension ''.inp'').',  &
           'The input file must be the last argument in the list.'
      write(6,'(/,a)') 'Options:'
      write(6,'(a)') &
           '      -h : print this help-text.'
      write(6,'(a)') &
           '      -? : print this help-text.'
      write(6,'(2a)') &
           '    -ver : version information.' 
      write(6,'(2a)') &
           '    -max : print a list of the "max..." parameters.' 
      write(6,'(2a)') &
           '     -rd : reading of the dvr-, oper- and restart- files', &
           ' is enforced.'
      write(6,'(a)') &
           '      -w : overwrite enabled.'
      write(6,'(2a/11x,a)') &
           '      -s : short test-run, checking the syntax ', &
           'of the inpf file, ', &
           'no files created, no dynamical space allocated.'
      write(6,'(2a/11x,2a)') &
           '      -o : similar to -s, but printing the ',  &
           'operator-log to screen.','(This op.log is before ', &
           'summation of Hamiltonian terms).'
      write(6,'(a,3(/11x,a))') &
           '    -cpp : run input file <inpf> through C preprocessor.',  &
           'Allows #include "settings.inc" of consistent settings in '// &
           'several', &
           'input files. Comment input in the form #/* comment */', &
           'For more information, refer to the cpp documentation.'
      write(6,'(2a)') &
           '    -mnd : Make Name Directory.', &
           ' Creates the name directory, if not existent.'
      write(6,'(2a)') &
           '    -pes : generate a pes-oper-file,', &
           ' (run_type in inpf file ignored).'
      write(6,'(2a)') &
           '   -gmat : "f1 f2" denote the DOFs for which a Gmat-file', &
           ' is generated.'
      write(6,'(2a)') &
           '      -t : test-run,        ', &
           ' (run_type in inpf file ignored).'
      write(6,'(2a)') &
           '      -c : continuation-run,', &
           ' (run_type in inpf file ignored).'
      write(6,'(2a)') &
           '     -ci : continuation=integrator run,', &
           ' (run_type in inpf file ignored).'
      write(6,'(2a)') &
           '      -D : ''name'' denotes the directory where files ', &
           'are written to,'
      write(6,'(29x,a)') &
           '(name in inpf file ignored).'
      write(6,'(2a)') &
           '      -I : ''name'' denotes the directory where files ', &
           'are written to.'
      write(6,'(11x,a)') &
           'The input is read from name/input and not from inpf file.'
      write(6,*)
      write(6,'(2a/29x,a)') &
           '  -tstop : ''hhh:mm'' denotes the (real) time ', &
           'after which program is stopped,', &
           '(tstop in inpf file ignored).'
      write(6,'(2a/11x,a/11x,a/29x,a)') &
           '   -tcpu : ''hhh:mm:ss'' denotes the cpu time ', &
           'after which program is stopped,', &
           'The short forms ''Is'' or ''Im'' are also possible where', &
           'I=integer, ''s'' stands for seconds and ''m'' for minutes.', &
           '(tcpu  in inpf file ignored).'
      write(6,'(2a/11x,2a/29x,a)') &
           '  -twall : ''hhh:mm:ss'' denotes the elapsed wall-time ', &
           'after which program', &
           'is stopped. Short forms similar as for tcpu ', &
           'are also possible.', &
           '(twall  in inpf file ignored).'
      write(6,'(2a,/29x,a)') &
           ' -tfinal : ''real'' denotes the final integration', &
           ' time in fs,','(tfinal in inpf file ignored).'
      write(6,'(2a,/29x,a)') &
           ' -tinit  : ''real'' denotes the initial integration', &
           ' time in fs,','(tinit in inpf file ignored).'
      write(6,'(2a,/29x,a)') &
           ' -thermal: ''real,int'' denotes the temperature in K', &
           ' and the seed.','(thermal in inpf file ignored).'
      write(6,'(2a,/11x,a,/11x,a)') &
           '      -p : ''apar'' is the name of a parameter, with ', &
           'real value ''rpar'',','needed to define the Hamiltonian.', &
           'The value ''rpar'' may carry a unit. E.g.: -p en 5,eV' 

      write(6,'(//2a/,8x,a)') &
           '  Note: Only one of the options -s, -o, -t, -c or -ci', &
           '  may be used!', &
           'Only one of the options -D or -I  may be used!'
      write(6,'(/2a)')' The order of the options does not matter,', &
                           ' but inpf must be the last argument.'
      write(6,'(/,80a1)') ('-',i=1,80)
      STOP

      end subroutine wrhelptext

!***********************************************************************

      subroutine maxinfo

use global
use maxv, only:maxdim,maxkoe,maxsta
use dvrdatmod
use griddatmod
use runinwfmod
use operdef
use psidef
use runopermod
use paths

      implicit none

      integer(long)       :: i

      write(6,'(80a1)') ('-',i=1,80)
      write(6,'(/9x,a,/)') '--- Current parameter settings ---'
      write(6,'(2x,a,1pe8.1,a)') 'tiny  =', tiny, &
           '  General cut-off parameter.'
      write(6,'(2x,a,i6,a)') 'maxdim  =',maxdim, &
           '  Maximum number of degrees of freedom.'
      write(6,'(2x,a,i6,a)') 'maxsta  =',maxsta, &
           '  Maximum number of electronic states.'
      write(6,'(2x,a,i6,a)') 'maxkoe  =',maxkoe, &
           '  Maximum number of Hamiltonian terms.'

      write(6,'(a)') 

      write(6,'(2x,a,i6,a)') 'maxhtm  =', maxhtm, &
       '  Maximum number of operators needed to build Hamiltonian.'
      write(6,'(2x,a,i6,a)') 'maxhop  =', maxhop, &
       '  Maximum number of labels used in operators.'
      write(6,'(2x,a,i6,a)') 'maxfac  =', maxfac, &
       '  Maximum number of factors/terms in an operator.'
      write(6,'(2x,a,i6,a)') 'maxdef  =', maxdef, &
       '  Maximum number of operators that can be defined.'
      write(6,'(2x,a,i6,a)') 'maxmuld =', maxmuld, &
       '  Maximum number of multi-dimensional operators.'
      write(6,'(2x,a,i6,a)') 'maxham  =', maxham, &
       '  Maximum number of Hamiltonians.'
      write(6,'(2x,a,i6,a)') 'maxhpar =', maxhpar, &
       '  Maximum number of parameters used to define operators.'
      write(6,'(2x,a,i6,a)') 'maxpar  =', maxpar, &
       '  Maximum number of parameters (see -p option).'
      write(6,'(2x,a,i6,a)') 'maxedim =', maxedim, &
       '  Maximum number of energy eigenvalues (init_wf).'
      write(6,'(2x,a,i6,a)') 'sbaspar =', sbaspar, &
       '  Maximum number of parameters used to define bases (init_wf).'
      write(6,'(2x,a,i6,a)') 'mbaspar =', mbaspar, &
       '  Maximum number of parameters used to define bases (gendvr).'
      write(6,'(2x,a,i6,a)') 'maxkey  =', maxkey, &
       '  Maximum number of keywords in an input line.'
      write(6,'(2x,a,i6,a)') 'maxrdf  =', maxrdf, &
       '  Maximum number of read-files (like natpot).'

  
      i=index(oppathdef,' ')-1
        if(i .le. 55) then
          write(6,'(/2x,a,a55)') 'Default operator-path: ', oppathdef
        else
          write(6,'(/2x,a,/,a)') 'Default operator-path: ', oppathdef
        endif  
      write(6,'(80a1)') ('-',i=1,80)

      call stopnow
      end subroutine maxinfo

!***********************************************************************

      subroutine shorthelp

      implicit none

      integer(long)       :: version,release,i

      write(6,'(80a1)') ('-',i=1,80)
      write(6,'(2a)') ' QUANTICS. A program for doing',  &
                      ' multi-dimensional quantum dynamics.' 
      call getrevno(progver,version,release)
      write(6,'(a,i3,a,i4,a,i4,a,/)') &
        ' ----  QUANTICS  Version', version,'   ---    Release ', &
         release,'   ----' 


      write(6,'(a)') 'Usage:  quantics [options] "inputfile"'

      write(6,'(/,2a)') ' -h   : ', &
             'Provides a help text explaining the options.'
      write(6,'(a)')    ' -ver : Version Information.'
      
      write(6,'(/,a80,/,80a1)') dateline, ('-',i=1,80)
 
      call stopnow
      end subroutine shorthelp
!***********************************************************************
      end module wrhtxt 

