!
! contains :: savecsys,getcsys,savecpsi,getcpsi
!
      module compdat

use decimal, only: long, dop

use global
use dglobal
use maxv, only: maxdim,maxsta
use dvrdatmod
use griddatmod
use psidef
use compdatmod
use operdef
use datenmod
use compdat1mod
 
use op1lib,  only: cpmxi, cpvxi, cpvxd
use if_grid
use wftree

      implicit none

      private
      public :: savecsys,getcsys,savecpsi,getcpsi

      contains
!#######################################################################
!
! COMPARISON SET
!

      subroutine savecsys(nc)

      implicit none

      integer(long), intent(in) :: nc

      cdentype(nc)=dentype
      cndof(nc)=ndof
      cnmode(nc)=nmode
      cnstate(nc)=nstate
      cnpacket(nc)=npacket
      cnpackts(nc)=npackts
      cfeb(nc)=feb
      cmeb(nc)=meb
      cfpb(nc)=fpb
      cmpb(nc)=mpb
      clcap(nc)=lcap
      clreal(nc)=lreal
      clrreal(nc)=lrreal

      call cpvxi(basis,cbasis(:,nc),maxdim)
      call cpmxi(ipbaspar,cipbaspar(:,:,nc),mbaspar,maxdim)
      call cpvxi(subdim,csubdim(:,nc),maxdim)
      call cpvxi(gdim,cgdim(:,nc),maxdim)
      call cpvxi(vgdim,cvgdim(:,nc),maxdim)
      call cpvxi(ngdim,cngdim(:,nc),maxdim)

      call cpvxi(nspfdof,cnspfdof(:,nc),maxdim)
      call cpmxi(spfdof,cspfdof(:,:,nc),maxdim,maxdim)
      call cpvxi(dofspf,cdofspf(:,nc),maxdim)

      call cpvxi(zort,czort(:,nc),maxdim)
      cortdim(nc)=ortdim

      return
      end subroutine savecsys

!#######################################################################

      subroutine getcsys(nc) 

      implicit none

      integer(long), intent(in) :: nc

      ndof=cndof(nc)
      nmode=cnmode(nc)
      nstate=cnstate(nc)
      npacket=cnpacket(nc)
      npackts=cnpackts(nc)
      feb=cfeb(nc)
      meb=cmeb(nc)
      fpb=cfpb(nc)
      mpb=cmpb(nc)
      lcap=clcap(nc)
      lreal=clreal(nc)
      lrreal=clrreal(nc)

      call cpvxi(basis,cbasis(:,nc),maxdim)
      call cpmxi(ipbaspar,cipbaspar(:,:,nc),mbaspar,maxdim)

      call cpvxi(cbasis(:,nc),basis,maxdim)
      call cpmxi(cipbaspar(:,:,nc),ipbaspar,mbaspar,maxdim)
      call cpvxi(csubdim(:,nc),subdim,maxdim)
      call cpvxi(cgdim(:,nc),gdim,maxdim)
      call cpvxi(cvgdim(:,nc),vgdim,maxdim)
      call cpvxi(cngdim(:,nc),ngdim,maxdim)

      call cpvxi(cnspfdof(:,nc),nspfdof,maxdim)
      call cpmxi(cspfdof(:,:,nc),spfdof,maxdim,maxdim)
      call cpvxi(cdofspf(:,nc),dofspf,maxdim)

      call cpvxi(czort(:,nc),zort,maxdim)
      ortdim=cortdim(nc)

      return
      end subroutine getcsys

!#######################################################################

      subroutine savecpsi(nc)

      implicit none

      integer(long)             :: i
      integer(long), intent(in) :: nc

      call cpvxi(zpsi,czpsi(:,nc),maxsta)
      call cpmxi(zetf,czetf(:,:,nc),maxdim,maxsta)
      call cpmxi(dim,cdim(:,:,nc),maxdim,maxsta)
      call cpmxi(vdim,cvdim(:,:,nc),maxdim,maxsta)
      call cpmxi(ndim,cndim(:,:,nc),maxdim,maxsta)
      call cpvxi(block,cblock(:,nc),maxsta)
      call cpmxi(dmat,cdmat(:,:,nc),maxdim,maxsta)
      call cpmxi(d3mat,cd3mat(:,:,nc),maxdim,maxsta)
      if (ldynspf) call cpmxi(maxdimspwn,cmaxdimspwn(:,:,nc),maxdim,maxsta)

! extra pointers for density matrices
      if (cdentype(nc) .eq. 1) then
         call cpmxi(ddim,cddim(:,:,:,nc),maxdim,maxsta*maxsta)
         call cpmxi(vddim,cvddim(:,:,:,nc),maxdim,maxsta*maxsta)
         call cpmxi(nddim,cnddim(:,:,:,nc),maxdim,maxsta*maxsta)
         call cpvxi(dblock,cdblock(:,:,nc),maxsta*maxsta)
         call cpmxi(dzpsi,cdzpsi(:,:,nc),maxsta,maxsta)
         call cpmxi(dzetf,cdzetf(:,:,:,nc),maxdim,maxsta*maxsta)
         call cpmxi(ddmat,cddmat(:,:,:,nc),maxdim,maxsta*maxsta)
         call cpmxi(dd3mat,cdd3mat(:,:,:,nc),maxdim,maxsta*maxsta)
      else if (cdentype(nc) .eq. 2) then
         call cpmxi(dzpsi,cdzpsi(:,:,nc),maxsta,maxsta)
      endif

! extra pointers for GWPs
      if (lgwp) then
         call cpmxi(gwpdim,cgwpdim(:,:,nc),maxdim,maxsta)
         call cpmxi(zpg,czpg(:,:,nc),maxdim,maxsta)
         call cpmxi(zxi,czxi(:,:,nc),maxdim,maxsta)
         call cpmxi(zphi,czphi(:,:,nc),maxdim,maxsta)
         call cpmxi(zdgwp,czdgwp(:,:,nc),maxdim,maxsta)
         call cpmxi(zagmat,czagmat(:,:,nc),maxdim,maxsta)
         call cpmxi(zxivec,czxivec(:,:,nc),maxdim,maxsta)
         call cpmxi(zphase,czphase(:,:,nc),maxdim,maxsta)
         call cpmxi(gsmat,cgsmat(:,:,nc),maxdim,maxsta)
      endif

      cldynspf(nc) = ldynspf
      cpsitype(nc)=psitype
      cdgldim(nc)=dgldim
      cadim(nc)=adim
      cdmatdim(nc)=dmatdim
      cd3matdim(nc)=d3matdim
      if (lgwp) then
         cagmatdim(nc)=agmatdim
         cgwpinfodim(nc)=gwpinfodim
         cgsmatdim(nc)=gsmatdim
         cgwptype(nc)=gwptype
         cgwpphase(nc)=gwpphase
         clgwpphase(nc)=lgwpphase
      endif

      call cpvxi(msymmtr,cmsymmtr(:,nc),2*maxdim)
      call cpvxi(idmode,cidmode(:,nc),maxdim)
      call cpvxi(symcv,csymcv(:,nc),maxdim)

      ccitype(nc)=citype
      ccicut(nc)=cicut
      call cpvxd(ciweight,cciweight(:,nc),maxdim)

      if (psitype .eq. 5) then
         call cpvxi(tape,ctape(:,nc),min(size(tape),size(ctape,1)))
         mladim = cmladim(nc)
         if (ldynspf) &
              call cpvxi(tapeM,ctapeM(:,nc),min(size(tape),size(ctape,1)))
      endif

      do i=1,npsiopt
         clpsiopt(i,nc) = lpsiopt(i)
      end do

      if (cdentype(nc) .gt. 0) then
         cdlstate(nc) = dlstate
         cdrstate(nc) = drstate
      endif

      return
      end subroutine savecpsi

!#######################################################################

      subroutine getcpsi(nc)

      implicit none

      integer(long), intent(in) :: nc
      integer(long)             :: i 

      call cpvxi(czpsi(:,nc),zpsi,maxsta)
      call cpmxi(czetf(:,:,nc),zetf,maxdim,maxsta)
      call cpmxi(cdim(:,:,nc),dim,maxdim,maxsta)
      call cpmxi(cvdim(:,:,nc),vdim,maxdim,maxsta)
      call cpmxi(cndim(:,:,nc),ndim,maxdim,maxsta)
      call cpvxi(cblock(:,nc),block,maxsta)
      call cpmxi(cdmat(:,:,nc),dmat,maxdim,maxsta)
      call cpmxi(cd3mat(:,:,nc),d3mat,maxdim,maxsta)
      if (cldynspf(nc)) call cpmxi(cmaxdimspwn(:,:,nc),maxdimspwn,maxdim,maxsta)

      if (cdentype(nc) .eq. 1) then
         call cpmxi(cddim(:,:,:,nc),ddim,maxdim,maxsta*maxsta)
         call cpmxi(cvddim(:,:,:,nc),vddim,maxdim,maxsta*maxsta)
         call cpmxi(cnddim(:,:,:,nc),nddim,maxdim,maxsta*maxsta)
         call cpvxi(cdblock(:,:,nc),dblock,maxsta*maxsta)
         call cpmxi(cdzpsi(:,:,nc),dzpsi,maxsta,maxsta)
         call cpmxi(cdzetf(:,:,:,nc),dzetf,maxdim,maxsta*maxsta)
         call cpmxi(cddmat(:,:,:,nc),ddmat,maxdim,maxsta*maxsta)
         call cpmxi(cdd3mat(:,:,:,nc),dd3mat,maxdim,maxsta*maxsta)
      else if (cdentype(nc) .eq. 2) then
         call cpmxi(cdzpsi(:,:,nc),dzpsi,maxsta,maxsta)
      endif

      if (lgwp) then
         call cpmxi(cgwpdim(:,:,nc),gwpdim,maxdim,maxsta)
         call cpmxi(czpg(:,:,nc),zpg,maxdim,maxsta)
         call cpmxi(czxi(:,:,nc),zxi,maxdim,maxsta)
         call cpmxi(czphi(:,:,nc),zphi,maxdim,maxsta)
         call cpmxi(czdgwp(:,:,nc),zdgwp,maxdim,maxsta)
         call cpmxi(czagmat(:,:,nc),zagmat,maxdim,maxsta)
         call cpmxi(czxivec(:,:,nc),zxivec,maxdim,maxsta)
         call cpmxi(czphase(:,:,nc),zphase,maxdim,maxsta)
         call cpmxi(cgsmat(:,:,nc),gsmat,maxdim,maxsta)
      endif

      ldynspf=cldynspf(nc)
      psitype=cpsitype(nc)
      dgldim=cdgldim(nc)
      adim=cadim(nc)
      dmatdim=cdmatdim(nc)
      d3matdim=cd3matdim(nc)
      if (lgwp) then
         agmatdim=cagmatdim(nc)
         gwpinfodim=cgwpinfodim(nc)
         gsmatdim=cgsmatdim(nc)
         gwptype=cgwptype(nc)
         gwpphase=cgwpphase(nc)
         lgwpphase=clgwpphase(nc)
      endif

      call cpvxi(cmsymmtr(:,nc),msymmtr,2*maxdim)
      call cpvxi(cidmode(:,nc),idmode,maxdim)
      call cpvxi(csymcv(:,nc),symcv,maxdim)

      citype=ccitype(nc)
      cicut=ccicut(nc)
      call cpvxd(cciweight(:,nc),ciweight,maxdim)

      if (psitype .eq. 5) then
         call cpvxi(ctape(:,nc),tape,min(size(tape),size(ctape,1)))
         cmladim(nc) = mladim
         tree => new_wftree(tape,if_spfdof)

         if (ldynspf) &
             call cpvxi(ctapeM(:,nc),tapeM,min(size(tape),size(ctape,1)))
      endif

      do i=1,npsiopt
         lpsiopt(i) = clpsiopt(i,nc)
      end do

      if (cdentype(nc) .gt. 0) then
         dlstate = cdlstate(nc) 
         drstate = cdrstate(nc) 
      endif

      return
      end subroutine getcpsi

      end module compdat


