! **********************************************************************
! *                                                                    *
! *                         modeule D1CALCHA
! *                                                                    *
! * MCTDH module containing (auxiliary) subroutines to evaluate the    *
! * right hand side of the MCTDH working equations.                    *
! *                                                                    *
! * Contains:                                                          *
! * d1calcdissa:    Calculates the action of the dissipative           *
! *                 terms on the MCTDH-coefficients                    *
! * d1calcha/2:     Calculates the action of the correlated            *
! *                 Hamiltonian on the MCTDH-coefficients              *
! * d1calchuna/2:   Calculates the action of the uncorrelated          *
! *                 Hamiltonian on the MCTDH-coefficients.             *
!
! ************************> SUBROUTINE CALCDISSA <***********************
      module d1calchamod

use decimal, only: long,dop
use mtlib,   only: mtxxzz, qtxxzz, qtxxzza, qttxzzs, mttxzz, qttxzz, qttxzza
use xvlib,   only: xvxxzza,xvxxzzo
use op1lib,  only: cpvxz
use op2lib,  only: addvxxzo, subvcxzo, subvxxzo1, addvcxzo

      implicit none
      private
      public :: d1calcdissa,d1calcha,d1calcha2,d1calchuna,d1calchuna2,d1calchaloch2
      contains

! **********************************************************************
!                          ~~~~~~~~~~~~~~~~~~~~~~
!
! Calculates the action of the dissipative terms on the
! DMCTDH-coefficients, the result is returned as a sum over all 
! terms in dtpsi. 
!
! **********************************************************************

      subroutine d1calcdissa (psi,dtpsi,hteil)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod
use openmpmod
      implicit none

      integer(long)                                    :: k,kt,m,si,sf,t,ti,tf,hzw,h
      integer(long), dimension(2)                      :: zw
      complex(dop), dimension(adim), intent(in)        :: psi
      complex(dop), dimension(adim), intent(inout)     :: dtpsi
      complex(dop), dimension(pmatdim), intent(in)     :: hteil
      complex(dop), dimension(maxblock,2)              :: workc

     
! --- LOOP OVER EACH CORRELATED HAMILTONIAN TERM ---
      do k=1,kzahl(1)

!     pure left acting operators
         if (kact(k).eq.2) then
            sf = kf(k)
            si = ki(k)
            do t=1,nstate
               zw(1) = 1
               zw(2) = 2
               call cpvxz(psi(dzpsi(si,t)),workc(1,zw(1)),dblock(si,t))
               do m = 1,nmode
! hteil -> dim_(sf,t) x dim_(si,t)
                  if (.not. diag(m,k)) then

                     call mtxxzz(hteil(dpmat(m,k,t)),workc(1,zw(1)), &
                          workc(1,zw(2)),vddim(m,sf,t),ddim(m,si,t), &
                          nddim(m,si,t),ddim(m,sf,t))

                     hzw = zw(1)
                     zw(1) = zw(2)
                     zw(2) = hzw
                  endif
               enddo

               h=hamilton(fkoe,k)
               if (htmtype(h) .eq. 1) then
                  call addvxxzo(workc(1,zw(1)),dtpsi(dzpsi(sf,t)), &
                       dblock(sf,t))
               else
                  call xvxxzza(koeff(k),workc(1,zw(1)), &
                       dtpsi(dzpsi(sf,t)),dblock(sf,t))
               endif

            enddo
         endif

!     pure right acting operators
         if (kact(k).eq.3) then
            sf = kf(k)
            si = ki(k)
            do t=1,nstate
               zw(1) = 1
               zw(2) = 2
               call cpvxz(psi(dzpsi(t,sf)),workc(1,zw(1)),dblock(t,sf))
               do m = 1,nmode
! hteil -> dim_(t,si) x dim_(t,sf)
                  if (.not. diag(m,k)) then
                     call mtxxzz(hteil(dpmat(m,k,t)),workc(1,zw(1)), &
                          workc(1,zw(2)),vddim(m,t,si),ddim(m,t,sf), &
                          nddim(m,t,sf),ddim(m,t,si))
                     hzw = zw(1)
                     zw(1) = zw(2)
                     zw(2) = hzw
                  endif
               enddo

               h=hamilton(fkoe,k)
               if (htmtype(h) .eq. 1) then
                  call addvxxzo(workc(1,zw(1)),dtpsi(dzpsi(t,si)), &
                       dblock(t,si))
               else
                  call xvxxzza(koeff(k),workc(1,zw(1)),&
                       dtpsi(dzpsi(t,si)),dblock(t,si))
               endif

            enddo
         endif

!     left-right acting operators
         if (kact(k).eq.4) then
            kt = ktwin(k)            
            sf = kf(k)
            si = ki(k)
            tf = kf(kt)
            ti = ki(kt)
            zw(1) = 1
            zw(2) = 2
            call cpvxz(psi(dzpsi(si,tf)),workc(1,zw(1)),dblock(si,tf))
            do m = 1,nmode
! hteil -> dim_(sf,ti) x dim_(si,tf)
               if (.not. diag(m,k)) then
                  call mtxxzz(hteil(dpmat(m,k,1)),workc(1,zw(1)),&
                       workc(1,zw(2)),vddim(m,si,tf),ddim(m,sf,ti),&
                       nddim(m,sf,ti),ddim(m,si,tf))
                  hzw = zw(1)
                  zw(1) = zw(2)
                  zw(2) = hzw
               endif
            enddo
            h=hamilton(fkoe,k)
            if (htmtype(h) .eq. 1) then
               call addvxxzo(workc(1,zw(1)),dtpsi(dzpsi(sf,ti)),&
                    dblock(sf,ti))
            else
               call xvxxzza(koeff(k),workc(1,zw(1)),&
                    dtpsi(dzpsi(sf,ti)),dblock(sf,ti))
            endif

         endif

      enddo

      return
      end subroutine d1calcdissa


! *** V8.0 AR ************> SUBROUTINE D1 CALC HA  <***********************
!                          ~~~~~~~~~~~~~~~~~~~~~
!
! Calculates the action of the correlated Hamiltonian on the
! MCTDH-coefficients for the inter-state terms.
! Result is returned as a sum over all terms in dtpsi.
! Makes use of hemiticity!

      subroutine d1calcha (psi,dtpsi,hteil,nham)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod
use openmpmod

      implicit none

      integer(long)                                    :: k,m,si,sf,t,hzw,h
      integer(long), dimension(2)                      :: zw
      integer(long), intent(in)                        :: nham
      complex(dop), dimension(adim), intent(in)        :: psi
      complex(dop), dimension(adim), intent(inout)     :: dtpsi
      complex(dop), dimension(pmatdim), intent(in)     :: hteil
      complex(dop), dimension(maxblock,2)              :: workc


! --- LOOP OVER EACH CORRELATED HAMILTONIAN TERM ---
outer:do k=1,kzahl(nham)

!     Take only hamiltonian terms
         if (kact(k) .ne. 1) cycle outer

         sf = kf(k)
         si= ki(k)

         do t=1,nstate

            zw(1) = 1
            zw(2) = 2
            call cpvxz(psi(dzpsi(si,t)),workc(1,zw(1)),dblock(si,t))

            if (si.eq.sf) then
               do m = 1,nmode
                  if (.not. diag(m,k)) then
                     call qtxxzz(hteil(dpmat(m,k,t)),workc(1,zw(1)), &
                          workc(1,zw(2)),vddim(m,sf,t),ddim(m,si,t), &
                          nddim(m,si,t))

                     hzw = zw(1)
                     zw(1) = zw(2)
                     zw(2) = hzw
                  endif
               enddo
            else
!     Note that for si .ne. sf always diag(m,k)=.false.
               do m = 1,nmode

                  call mtxxzz(hteil(dpmat(m,k,t)),workc(1,zw(1)), &
                       workc(1,zw(2)),vddim(m,sf,t),ddim(m,si,t), &
                       nddim(m,si,t),ddim(m,sf,t))

                  hzw = zw(1)
                  zw(1) = zw(2)
                  zw(2) = hzw
               enddo
            endif

            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1) &
                call xvxxzzo(koeff(k),workc(1,zw(1)),dblock(sf,t))

            call addvxxzo(workc(1,zw(1)),dtpsi(dzpsi(sf,t)),&
                 dblock(sf,t))

!     Take care that dblock(s,t) = dblock(t,s)
            if (lrlx) then
               call addvcxzo(workc(1,zw(1)),dtpsi(dzpsi(t,sf)), &
                    dblock(t,sf))                  
            else
               call subvcxzo(workc(1,zw(1)),dtpsi(dzpsi(t,sf)), &
                    dblock(t,sf))
            endif
         enddo

      enddo outer


      return
      end subroutine d1calcha


! *** V8.0 AR ************> SUBROUTINE D1 CALC HA2 <***********************
!                          ~~~~~~~~~~~~~~~~~~~~~
!
! Calculates the action of the correlated Hamiltonian on the
! MCTDH-coefficients for the inter-state terms.
! Result is returned as a sum over all terms in dtpsi.
! Does not make use of hemiticity!

      subroutine d1calcha2 (psi,dtpsi,hteil,nham)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod
use openmpmod

      implicit none

      integer(long)                                    :: k,m,si,sf,t,hzw,h
      integer(long)                                    :: k1,k2
      integer(long), dimension(2)                      :: zw
      integer(long), intent(in)                        :: nham
      complex(dop), dimension(adim), intent(in)        :: psi
      complex(dop), dimension(adim), intent(inout)     :: dtpsi
      complex(dop), dimension(pmatdim), intent(in)     :: hteil
      complex(dop), dimension(maxblock,2)              :: workc


      k1 = zham(nham)
      k2 = zham(nham)+kzahl(nham)-1
! --- LOOP OVER EACH CORRELATED HAMILTONIAN TERM ---
outer:do k=k1,k2

!     Take only hamiltonian terms
!         if (kact(k) .ne. 1) goto 10
         if (kact(k) .ne. 1) cycle outer

         sf = kf(k)
         si= ki(k)

         do t=1,nstate

            zw(1) = 1
            zw(2) = 2
            call cpvxz(psi(dzpsi(si,t)),workc(1,zw(1)),dblock(si,t))

            if (si.eq.sf) then
               do m = 1,nmode
                  if (.not. diag(m,k)) then

                     call qtxxzz(hteil(dpmat(m,k,t)),workc(1,zw(1)), &
                          workc(1,zw(2)),vddim(m,sf,t),ddim(m,si,t), &
                          nddim(m,si,t))

                     hzw = zw(1)
                     zw(1) = zw(2)
                     zw(2) = hzw
                  endif
               enddo
            else
!     Note that for si .ne. sf always diag(m,k)=.false.
               do m = 1,nmode
                  call mtxxzz(hteil(dpmat(m,k,t)),workc(1,zw(1)), &
                       workc(1,zw(2)),vddim(m,sf,t),ddim(m,si,t), &
                       nddim(m,si,t),ddim(m,sf,t))
                  hzw = zw(1)
                  zw(1) = zw(2)
                  zw(2) = hzw
               enddo
            endif

            h=hamilton(fkoe,k)
            if (htmtype(h) .eq. 1) then
               call addvxxzo(workc(1,zw(1)),dtpsi(dzpsi(sf,t)),&
                    dblock(sf,t))
            else
               call xvxxzza(koeff(k),workc(1,zw(1)),&
                    dtpsi(dzpsi(sf,t)),dblock(sf,t))
            endif


!     Second part of the commutator

            zw(1) = 1
            zw(2) = 2
            call cpvxz(psi(dzpsi(t,sf)),workc(1,zw(1)),dblock(t,sf))

            if (si.eq.sf) then
               do m = 1,nmode
                  if (.not. diag(m,k)) then
!     Take care that xddim(m,s,t) = xddim(m,t,s)
                     call qttxzz(hteil(dpmat(m,k,t)),workc(1,zw(1)), &
                          workc(1,zw(2)),vddim(m,t,si),ddim(m,t,sf), &
                          nddim(m,t,sf))
                     hzw = zw(1)
                     zw(1) = zw(2)
                     zw(2) = hzw
                  endif
               enddo
            else
!     Note that for si .ne. sf always diag(m,k)=.false.
               do m = 1,nmode
                  call mttxzz(hteil(dpmat(m,k,t)),workc(1,zw(1)), &
                       workc(1,zw(2)),vddim(m,t,si),ddim(m,t,sf), &
                       nddim(m,t,sf),ddim(m,t,si))
                  hzw = zw(1)
                  zw(1) = zw(2)
                  zw(2) = hzw
               enddo
            endif

            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1) &
                call xvxxzzo(koeff(k),workc(1,zw(1)),dblock(t,si))

            if (lrlx) then
               call addvxxzo(workc(1,zw(1)),dtpsi(dzpsi(t,si)), &
                    dblock(t,si))                  
            else
               call subvxxzo1(dtpsi(dzpsi(t,si)),workc(1,zw(1)), &
                    dblock(t,si))
            endif
         enddo

! 10      continue
      enddo outer


      return
      end subroutine d1calcha2


! *** V8.0 AR ***********> SUBROUTINE D1 CALC HUNA <***********************
!                         ~~~~~~~~~~~~~~~~~~~~~~

! Calculates the action of the uncorrelated Hamiltonian on the
! MCTDH-coefficients.
! Makes use of hemiticity!

      subroutine d1calchuna (psi,dtpsi,hteil)

use global
use griddatmod
use operdef
use psidef
use hpsimod
use openmpmod

      implicit none

      integer(long)                                  :: m,s,t
      complex(dop), dimension(adim), intent(in)      :: psi
      complex(dop), dimension(adim), intent(inout)   :: dtpsi
      complex(dop), dimension(pmatdim), intent(in)   :: hteil
      complex(dop), dimension(maxblock)              :: workc

! --- LOOP OVER EACH UNCORRELATED HAMILTONIAN TERM ---
      do t=1,nstate
         do s=1,nstate

            call qtxxzz(hteil(dpmat(1,kzahl(1)+s,t)),psi(dzpsi(s,t)),&
                 workc,vddim(1,s,t),ddim(1,s,t),&
                 nddim(1,s,t))

            do m=2,nmode
               call qtxxzza(hteil(dpmat(m,kzahl(1)+s,t)),&
                    psi(dzpsi(s,t)),workc,vddim(m,s,t),ddim(m,s,t),&
                    nddim(m,s,t))
            enddo

!     Take care that dblock(s,t) = dblock(t,s)
            call addvxxzo(workc,dtpsi(dzpsi(s,t)),dblock(s,t))
            if (lrlx) then
               call addvcxzo(workc,dtpsi(dzpsi(t,s)),dblock(t,s))         
            else
               call subvcxzo(workc,dtpsi(dzpsi(t,s)),dblock(t,s))
            endif

         enddo
      enddo


      return
      end subroutine d1calchuna

! *** V8.0 AR ***********> SUBROUTINE CALC HUNA2 <***********************
!                         ~~~~~~~~~~~~~~~~~~~~~~

! Calculates the action of the uncorrelated Hamiltonian on the
! MCTDH-coefficients.
! Does not make use of hemiticity!

      subroutine d1calchuna2 (psi,dtpsi,hteil)

use global
use griddatmod
use operdef
use psidef
use hpsimod
use openmpmod

      implicit none

      integer(long)                                :: m,s,t
      complex(dop), dimension(adim), intent(in)    :: psi
      complex(dop), dimension(adim), intent(inout) :: dtpsi
      complex(dop), dimension(pmatdim), intent(in) :: hteil


! --- LOOP OVER EACH UNCORRELATED HAMILTONIAN TERM ---
      do t=1,nstate
         do s=1,nstate
            do m=1,nmode
               call qtxxzza(hteil(dpmat(m,kzahl(1)+s,t)), &
                    psi(dzpsi(s,t)),dtpsi(dzpsi(s,t)),vddim(m,s,t), &
                    ddim(m,s,t),nddim(m,s,t))
            enddo

!     Second part of the commutator

!     Take care that xddim(m,s,t) = xddim(m,t,s)
            if (lrlx) then
               do m=1,nmode
                  call qttxzza(hteil(dpmat(m,kzahl(1)+s,t)), &
                       psi(dzpsi(t,s)),dtpsi(dzpsi(t,s)),vddim(m,t,s), &
                       ddim(m,t,s),nddim(m,t,s))
               enddo
            else
               do m=1,nmode
                  call qttxzzs(hteil(dpmat(m,kzahl(1)+s,t)), &
                       psi(dzpsi(t,s)),dtpsi(dzpsi(t,s)),vddim(m,t,s), &
                       ddim(m,t,s),nddim(m,t,s))
               enddo
            endif

         enddo
      enddo


      return
      end subroutine d1calchuna2


! *******************> SUBROUTINE CALCHALOCH2 <***********************
!                          ~~~~~~~~~~~~~~~~~~~~~
!
! Calculates the action of the correlated Hamiltonian on the
! MCTDH-coefficients for the inter-state terms.
! Result is returned as a sum over all terms in dtpsi.
! Does not make use of hemiticity!
!
!  12/01 CC
!
      subroutine d1calchaloch2 (psi,dtpsi,hteil,nham)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod
use openmpmod

      implicit none

      integer(long)                                    :: k,k1,k2,m,si,sf,t,hzw,h
      integer(long), intent(in)                        :: nham
      integer(long), dimension(2)                      :: zw
      complex(dop), dimension(adim), intent(in)        :: psi
      complex(dop), dimension(adim), intent(inout)     :: dtpsi
      complex(dop), dimension(pmatdim), intent(in)     :: hteil
      complex(dop), dimension(maxblock,2)              :: workc


! --- LOOP OVER EACH CORRELATED HAMILTONIAN TERM ---
      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
outer:do k=k1,k2

!     Take only hamiltonian terms
         if (kact(k) .ne. 1) cycle outer

         sf = kf(k)
         si= ki(k)

         do t=1,nstate

            zw(1) = 1
            zw(2) = 2
            call cpvxz(psi(dzpsi(si,t)),workc(1,zw(1)),dblock(si,t))

            if (si.eq.sf) then
               do m = 1,nmode
                  if (.not. diag(m,k)) then
                     call qtxxzz(hteil(dpmat(m,k,t)),workc(1,zw(1)), &
                          workc(1,zw(2)),vddim(m,sf,t),ddim(m,si,t), &
                          nddim(m,si,t))
                     hzw = zw(1)
                     zw(1) = zw(2)
                     zw(2) = hzw
                  endif
               enddo
            else
!     Note that for si .ne. sf always diag(m,k)=.false.
               do m = 1,nmode
                  call mtxxzz(hteil(dpmat(m,k,t)),workc(1,zw(1)), &
                       workc(1,zw(2)),vddim(m,sf,t),ddim(m,si,t), &
                       nddim(m,si,t),ddim(m,sf,t))
                  hzw = zw(1)
                  zw(1) = zw(2)
                  zw(2) = hzw
               enddo
            endif

            h=hamilton(fkoe,k)
            if (htmtype(h) .eq. 1) then
               call addvxxzo(workc(1,zw(1)),dtpsi(dzpsi(sf,t)), &
                    dblock(sf,t))
            else
               call xvxxzza(koeff(k),workc(1,zw(1)),&
                    dtpsi(dzpsi(sf,t)),dblock(sf,t))
            endif

         enddo       ! of t

      enddo  outer        ! of k

      return
      end subroutine d1calchaloch2

!-----------------------------------------------------------------------
      end module d1calchamod




