! **********************************************************************
! *                                                                    *
! *                         D1LIB (d1lib.f)                            *
! *                                                                    *
! * MCTDH module containing (auxiliary) subroutines related to         *
! * manipulation of type I density matrices
! *                                                                    *
! * Contains:                                                          *
! *   trspdo:       Calculates the traces of the MCTDH SPDOs for one   *
! *                 mode                                               *
! **********************************************************************
      module d1lib

use decimal, only: long, dop
use op1lib,  only: trhxz, trqxz

      implicit none
      private
      public :: trspdo,trspdoc,trspdoel,travec
      contains

! *** V8.0 AR ************> SUBROUTINE TRSPDO    <**********************
!                          ~~~~~~~~~~~~~~~~~~~~~~
! Calculates the traces of the MCTDH SPDOs for one mode
! **********************************************************************

      subroutine trspdo(psi,trace,dim,subdim,supgdim)

      implicit none

      integer(long)                                   :: i
      integer(long), intent(in)                       :: dim,subdim,supgdim
      real(dop), dimension(dim), intent(out)          :: trace
      complex(dop), dimension(subdim,dim), intent(in) :: psi

      do i=1,dim
         call trhxz(psi(1,i),trace(i),supgdim)
      enddo

      return
      end subroutine trspdo


! *** V8.0 AR ************> SUBROUTINE TRSPDOC   <**********************
!                          ~~~~~~~~~~~~~~~~~~~~~~
! Calculates the traces of the MCTDH SPDOs for one mode, non-hermitian
! SPDOs allowed
! **********************************************************************

      subroutine trspdoc(psi,trace,dim,subdim,supgdim)

      implicit none

      integer(long)                                   :: i
      integer(long), intent(in)                       :: dim,subdim,supgdim
      complex(dop), dimension(subdim,dim), intent(in) :: psi
      complex(dop), dimension(dim), intent(out)       :: trace

      do i=1,dim
         call trqxz(psi(1,i),trace(i),supgdim)
      enddo

      return
      end subroutine trspdoc


! *** V8.0 AR ************> SUBROUTINE TRSPDOC   <**********************
!                          ~~~~~~~~~~~~~~~~~~~~~~
! Calculates the traces of the MCTDH SPDOs for one mode, non-hermitian
! SPDOs allowed
! **********************************************************************

      subroutine trspdoel(psi,trace,dim,supgdim,dlstate,drstate)

      implicit none

      integer(long)                                   :: i
      integer(long), intent(in)                       :: dim,supgdim
      integer(long), intent(in)                       :: dlstate,drstate
      complex(dop), dimension(supgdim,supgdim,dim), intent(in) :: psi
      complex(dop), dimension(dim), intent(out)       :: trace

      do i=1,dim
         trace(i) = psi(dlstate,drstate,i)
      enddo

      return
      end subroutine trspdoel

! **********************************************************************

      subroutine travec(psi,trace,vdim1,dim1,ndim1)

      implicit none

      integer(long)                                   :: v,e,n
      integer(long), intent(in)                       :: vdim1,dim1,ndim1
      complex(dop), dimension(vdim1,dim1,ndim1), intent(in) :: psi
      real(dop), dimension(dim1), intent(out)         :: trace

      trace(:) = 0.0_dop
      do n=1,ndim1
         do e=1,dim1
            do v=1,vdim1
               trace(e) = trace(e) + dble(psi(v,e,n))
            enddo
         enddo
      enddo

      return
      end subroutine travec

      end module d1lib
