! **********************************************************************
! *                                                                    *
! *                         D1PHIHPHI
! *                                                                    *
! * MCTDH module containing (auxiliary) subroutines to evaluate the    *
! * right hand side of the MCTDH working equations.                    *
! *                                                                    *
! * Contains:                                                          *
! * d1phihphi:      Computes for all modes the "correlated"            *
! *                 matrix elements <phi|h|phi>.                       *
! * d1phihunphi:    Computes for all modes and states the              *
! *                 "uncorrelated" matrix elements <phi|h|phi>.        *
! *                                                                    *
! **********************************************************************
      module d1phihphimod

use decimal
use global
use griddatmod
use operdef
use psidef
use hpsimod
use openmpmod

use mmlib, only: mmaxzz,mmaxzzh


      implicit none
      private
      public :: d1phihphi,d1phihunphi
      contains

! **********************************************************************
! *** V8.0 AR **************> SUBROUTINE PHIHPHI <**********************
!                            ~~~~~~~~~~~~~~~~~~~~

! Computes for all modes the correlated Hamiltonian matrix elements
! <phi|h|phi>.
! **********************************************************************

      subroutine d1phihphi (psi,hpsi,hteil,nham)

      implicit none

      integer(long)                                 :: k,k1,k2,kt,m,s,t,s1,t1
      integer(long), intent(in)                     :: nham
      complex(dop), dimension(dgldim), intent(in)   :: psi
      complex(dop), dimension(hpsidim), intent(in)  :: hpsi
      complex(dop), dimension(pmatdim), intent(out) :: hteil

! --- LOOP OVER EACH MODE AND STATE ---

      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1

!$omp parallel if(lompthread) num_threads(ompthread) private(m)
      do m = 1,nmode
!$omp do private(k,s,s1,t,t1,kt)

! --- COMPUTE <PHI|H|PHI> MATRICES FOR EACH HAMILTONIAN TERM---
         do k = k1,k2
            if (.not. diag(m,k) .and. k .eq. kfirst(m,k) ) then
!     pure left acting operators
               if ((kact(k).eq.1) .or. (kact(k).eq.2)) then
                  s=kf(k)
                  s1=ki(k)
                  do t=1,nstate
                     call mmaxzz(psi(dzetf(m,s,t)),hpsi(dzhpsi(m,k,t)),&
                           hteil(dpmat(m,k,t)),subdim(m),ddim(m,s,t),&
                           ddim(m,s1,t))
                  enddo
               endif
!     pure right acting operators
               if (kact(k).eq.3) then
                  t=ki(k)
                  t1=kf(k)
                  do s=1,nstate
                     call mmaxzz(psi(dzetf(m,s,t)),hpsi(dzhpsi(m,k,s)),&
                          hteil(dpmat(m,k,s)),subdim(m),ddim(m,s,t),&
                          ddim(m,s,t1))
                  enddo
               endif
!     left-right acting operators
               if (kact(k).eq.4) then
                  kt = ktwin(k)
                  s  = kf(k)
                  s1 = ki(k)
                  t  = kf(kt)
                  t1 = ki(kt)
                  call mmaxzz(psi(dzetf(m,s,t1)),hpsi(dzhpsi(m,k,1)),&
                       hteil(dpmat(m,k,1)),subdim(m),ddim(m,s,t1),&
                       ddim(m,s1,t))
               endif
            endif
         enddo
!$omp end do
      enddo
!$omp end parallel

      return
      end subroutine d1phihphi


! *** V6.0 MB *************> SUBROUTINE PHIHUNPHI <*********************
!                           ~~~~~~~~~~~~~~~~~~~~~~

! Computes for all modes and states the uncorrelated matrix elements
! <phi|h_uncorr|phi>.
! **********************************************************************

      subroutine d1phihunphi (psi,hunpsi,hteil,nham)

      implicit none

      integer(long)                                      :: m,s,t,k
      integer(long), intent(in)                          :: nham
      complex(dop), dimension(dgldim), intent(in)        :: psi
      complex(dop), dimension(adim+1:dgldim), intent(in) :: hunpsi
      complex(dop), dimension(pmatdim), intent(out)      :: hteil

! --- LOOP OVER EACH MODE AND STATE ---
      k=zham(nham)+kzahl(nham)-1

!$omp parallel do if(lompthread) num_threads(ompthread) private(m,t,s) 
      do m = 1,nmode
         do t = 1,nstate
            do s = 1,nstate
               call mmaxzz(psi(dzetf(m,s,t)),hunpsi(dzetf(m,s,t)), &
                    hteil(dpmat(m,k+s,t)),subdim(m), &
                    ddim(m,s,t),ddim(m,s,t))
            enddo
         enddo
      enddo
!$omp end parallel do

      return
      end subroutine d1phihunphi

! **********************************************************************

      end module d1phihphimod

