! **********************************************************************
! *                                                                    *
! *                         D1LIB (d1lib.f)                            *
! *                                                                    *
! * MCTDH module containing (auxiliary) subroutines related to         *
! * manipulation of type I density matrices
! *                                                                    *
! * Contains:                                                          *
! *   trspdo:       Calculates the traces of the MCTDH SPDOs for one   *
! *                 mode                                               *
! **********************************************************************
      module d2lib

use decimal, only: long, dop
use op1lib,  only: trhxz
use xvlib,   only: xvixdzo

use griddatmod
use psidef

      implicit none
      private
      public :: d2retrace
      contains

! *** V8.0 AR ************> SUBROUTINE TRSPDO    <**********************
!                          ~~~~~~~~~~~~~~~~~~~~~~
! Calculates the traces of the MCTDH SPDOs for one mode
! **********************************************************************

      subroutine d2retrace(psi)

      implicit none

      integer(long)                                  :: s,s1
      real(dop)                                      :: trace,dtmp
      complex(dop), dimension(dgldim), intent(inout) :: psi

      trace = 0._dop
      do s=1,nstate
         call trhxz(psi(dzpsi(s,s)),dtmp,block(s))
         trace = trace+dtmp
      enddo
print*,'START',trace
      do s=1,nstate
         do s1=1,nstate
            call xvixdzo(trace,psi(dzpsi(s1,s)),block(s1)*block(s))
         enddo
      enddo

      trace = 0._dop
      do s=1,nstate
         call trhxz(psi(dzpsi(s,s)),dtmp,block(s))
         trace = trace+dtmp
      enddo
print*,'END',trace


      end subroutine d2retrace

      end module d2lib
