!***********************************************************************
!
!           MCTDH subrotuine DEFAULT.F
!
!  In this subroutine all non-local variables and parameters
!  should be initialized which are used by all programs (potfit, mctdh, etc.).
!  This file should only depend on global.inc.
!
!   01/98 AJ
!
!***********************************************************************

      subroutine default

use decimal, only: dop
use global
use keyunitsmod
use globmemmod
use channels
use mpidata
use oct
use openmpmod

use op1lib, only: initvxl

      implicit none

      real(dop), external :: dlamch
      
!-----------------------------------------------------------------------
! Call setunits to set pi, fs, ev (common /constants/ in global.inc)
!-----------------------------------------------------------------------
      call setunits

!-----------------------------------------------------------------------
! Calculate machine precision
!-----------------------------------------------------------------------
      macheps = dlamch('P')

!-----------------------------------------------------------------------
! logicals to control use of oper / dvr files (common /delopt/ in global.inc)
!-----------------------------------------------------------------------
      ldelop   = .false.
      ldeldvr  = .false.

!-----------------------------------------------------------------------
! logicals to control use of MPI
!-----------------------------------------------------------------------
      lmpi           = .false.
      lmpicalcha     = .true.
      lmpifunka2     = .true.
      lmpimfields    = .true.
      lmpidav        = .true.
      lmpidavrun     = .false.
      lmpigetdavmat  = .true.
      lmpidsyev      = .true.
      ldavkey        = .false.
      lmpistarted    = .false.
      lmpiphihphi    = .true.
      lmpihlochphi   = .true.
      lmpisummf      = .true.
      lpartim        = .false.
      lmpigetdiag    = .true.
      lmpiexpect     = .true.
      lrunml         = .false.
      lmpigetddpes   = .true.

      call initvxl(lmpiupdt,nmpiupdtlen)

!      qrlimit     = 1
!      nthread     = 1
!      ctstep      = 1
      intordermpi = 2

!-----------------------------------------------------------------------
! set all run part flags to .false.  (common /runopts/ in global.inc)
!-----------------------------------------------------------------------
      lexact   = .false.
      lrealwp  = .false.

!-----------------------------------------------------------------------
!DWF  Dynamical WF (i.e. spawn, initorb and dynspf) related variables
!-----------------------------------------------------------------------
      call SetDwfDefault

!-----------------------------------------------------------------------
! In subroutine rstinfo: write to log if true 
!(avoid floding of efield-log with optcntrl
!-----------------------------------------------------------------------
      lwrlog   = .true.

!-----------------------------------------------------------------------
! default names and their lengths (commons /names/ and /lengths/ in global.inc)
!-----------------------------------------------------------------------
      name     = '  '
      rname    = '  '
      oname    = '  '
      dname    = '  '
      vname    = '  '
      ddname   = '  '
      inputfile= '  '
      options  = '  '
      runtitle = '  '
      operfile = 'oper'
      crossname= '  '

      laenge   = 0
      rlaenge  = 0
      olaenge  = 0
      dlaenge  = 0
      vlength  = 0
      ddlaenge = 0
      inplaenge= 0
      optlaenge= 0
      crosslng = 0

!-----------------------------------------------------------------------
! no BEL by default
!-----------------------------------------------------------------------
      lBEL = .FALSE.

!-----------------------------------------------------------------------
! initialize some variables related to memory allocation
!-----------------------------------------------------------------------
      mcdim=1
      mrdim=1
      midim=1
      mldim=1
      msdim=1
      mfdim=1

      workcdim=0
      workrdim=0
      workidim=0
      workldim=0
      worksdim=0
      workfdim=0

      
      return
      end subroutine default

!#######################################################################
!  Variables stored in  dglobal.f90
!#######################################################################

      subroutine SetDwfDefault

use global
use dglobal

      implicit none

!     Integer
!     Integer array storing integer parameters obtained from input

!     Real array storing real parameters obtained from input
      tcurrent=0.0
      constop0=0.0
!     Logical
!     Logical array storing definitions obtained from input
      lnspf        = .false.
      lnewspf      = .false.
      lspwnfcall   = .false.

      return
      end subroutine SetDwfDefault

