!#######################################################################

      module directdyn

use decimal, only: long, dop, sip
use global
use globmemmod
use dirdyn
use dvrdatmod
use griddatmod
use logdat
!use op1lib,  only: unitqxd
use dd_db
use operdef
use runpropmod
use channels
use openmpmod
use psidef
use db_sql, only: dbwrcidat

use gwplib, only: gwp2gpoint,gwp2gpoint1,gwp2gpoint2
use funcqchemmod, only: lhaqchem,ddvmcgpestest
use diabatmod, only: diabat1,diabat2,diabat4,diabat6,transform
use mmlib, only: qqtxdd,dmxxdd,mmtxdd
use op1lib, only: normvxd,cpvxd,unitqxd
use lalib, only: simtranbd,simtranbdz
use ioqc, only: getintyp,getnm
use keyunitsmod
#ifdef MPI
use mpi
use mpidata
use mpistructure, only: get_qcalc_mpi_type, QCalcData, init_qcalc_data
#endif

use symoplib
use potevalmod, only: calcpes

      implicit none
      private
      public :: getddpes, init_qchem_calculations,&
                finalise_qchem_calculations,&
                distgwp,mksympesa1,addsoc,genhess0
#ifdef MPI
      public :: mpigetddpes
#endif
      contains

!#######################################################################
! New version (Benjamin, December 2008) for most of the module
!
!  totdim1 : no. of basis functions (sets of coordinates = nsgp(s1)
!  s1 : state for multi-set 
!
!#######################################################################

      subroutine getddpes(time,qcenter,totdim1,s1)

!+---------------------------------------------------------------------+
!     calculates potential energy surface(s) for direct dynamics
!+---------------------------------------------------------------------+
!$ use OMP_LIB

      implicit none

      integer(long)                                   :: m,e,e1,e2,nex2,hl,&
                                                         hl1,h,ngp,n1,n2,&
                                                         order,nrec,i,n,s,f,f1
      integer(long), intent(in)                       :: totdim1
      integer(long), intent(in)                       :: s1
      real(dop), intent(in)                           :: time

      logical(kind=4), allocatable, dimension(:)      :: lskip
      integer(long), dimension(:,:), allocatable      :: idbloc,interp
      integer(long), dimension(:,:), allocatable      :: hopiparp
      real(dop)                                       :: delt,norm
      real(dop)                                       :: mindist,mindisp
      real(dop), dimension(ndofddpes)                 :: xgpoint,xtmp
      real(dop), dimension(:,:,:), allocatable        :: xgp,xgpoint0
      real(dop), dimension(ndofdd)                    :: gpoint
      real(dop), allocatable, dimension(:,:)          :: gwpdist
      real(dop), dimension(qcentdim), intent(in)      :: qcenter
      real(dop), allocatable, dimension(:,:,:)        :: r1
      real(dop), allocatable, dimension(:,:,:,:)      :: ddv,adv
      real(dop), allocatable, dimension(:,:,:,:)      :: ddnact
      real(dop), allocatable, dimension(:,:,:,:)      :: diabtrans,diabctrans
      real(dop), allocatable, dimension(:,:,:,:)      :: ddcv

      complex(dop), allocatable, dimension(:,:,:,:)   :: adsoc 

      real(sip), allocatable, dimension(:,:,:)        :: ddmoco
      real(dop), allocatable, dimension(:,:,:,:,:)    :: ddder1,adder1
      real(dop), allocatable, dimension(:,:,:,:,:,:)  :: ddder2,adder2
      real(dop), allocatable, dimension(:,:,:,:,:)    :: dddip,addip
      character(len=30), allocatable,dimension(:,:,:) :: ciconfig
      real(dop), allocatable, dimension(:,:,:,:)      :: cicoeff
      real(dop), dimension(nddstate,nddstate)         :: pesdia
      complex(dop), dimension(nddstate,nddstate)      :: pesdiaz
      integer(long)                                   :: izflag,iwr
      real(dop)                                       :: emax
      integer(long), dimension(ndofddpes)             :: igpoint

! gwpsymrep(n,1,e) lists DB records relating to symmetry replicas of first point
! gwpsymrep(1,n,e) lists (-ve of ) DB record of point from which symmetry 
! replica n comes from
      integer(long), allocatable,dimension(:,:,:)     :: gwpsymrep
! small DB location for original symmetry rep
      integer(long), dimension(:,:), allocatable      :: locpt0
      integer(long)                                   :: nrec0

      integer(long), dimension(:), allocatable        :: dbnrec0
      logical(kind=4), dimension(:,:), allocatable    :: lrddb
      logical(kind=4)                                 :: lfail
      logical(kind=4)                                 :: lcas
      real(dop), allocatable, dimension(:,:)          :: allcent
      integer(long), dimension(:), allocatable        :: eex2
#ifdef MPI
      integer(long), dimension(nmpi)                  :: mpinex1,mpinex2
!maximum of mpi process possible for getddpes
      integer(long)                                   :: mxmpi 
      real(dop)                                       :: mpitemp
      integer                                         :: qcalc_type, &
                                                         mpi_comm_worker, &
                                                         master_nex1, &
                                                         master_nex2
      type(QCalcData)                                 :: qcalc_data
#endif
      integer(long):: ddv_size,ddder1_size,ddder2_size,adv_size,&
           dddip_size,addip_size,diabtrans_size,ddnact_size,&
           interp_size,adsoc_size,ddmoco_size,ciconfig_size,cicoeff_size

      interface
        function dnrm2(n,x,incx)
          use decimal, only: dop
          real(dop) :: dnrm2
          integer :: n,incx
          real(dop),dimension(n) :: x
        end function dnrm2
      end interface

      routine = 'Getddpes (directdyn.F90)'

!-----------------------------------------------------------------------
! If read only, return
!-----------------------------------------------------------------------
      if (lddrddb .and. .not. lddwrdb) return

!-----------------------------------------------------------------------
! check whether updating wanted
!-----------------------------------------------------------------------
      if (ddupdate .eq. 2) then
         return
      else if (ddupdate .eq. 1) then
         delt = abs(time - dduptim0)
         if ((delt .lt. dduptim) .and. (abs(delt - dduptim) .ge. tiny)) then
            return
         endif
      endif

      if (ddlogfull) then
         write(ilog,'(a,f12.6)') 'DD Entering Update at: ',time/fs
      endif

! if DB is not being used, set no. records to 0
      if (.not. lddrddb) dbnrec = 0

!-----------------------------------------------------------------------
! if using diabatisation that does not generate the ADT, check it is
! not needed
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! get info required by LHAqchem to tell the program to use etc.
! qc program used if hterm(1,h) operator is in file 205
!-----------------------------------------------------------------------
      hl = 0
      do h=1,htmdim
         hl1=hterm(1,h)
         if(hl1.eq.0)cycle
         if (htmform(h) .eq. 1) cycle
         if (hopfile(hl1) .eq. 205) then
            hl = hl1
            exit
         endif
      enddo

!-----------------------------------------------------------------------
! Is this a CAS calculation with MOCOs?
!-----------------------------------------------------------------------
      lcas = .false. 
      lcas = (hopipar(7,hl).eq.12 .or. hopipar(7,hl).eq.13 .or. hopipar(7,hl).eq.27 .or. &
              hopipar(7,hl).eq.15 .or. hopipar(7,hl).eq.32 .or. hopipar(7,hl).eq.37) 

!-----------------------------------------------------------------------
! if no DD potential present, return
!-----------------------------------------------------------------------
      if (hl .eq. 0) return
 
!-----------------------------------------------------------------------
! options passed to control of PES calculation via hopipar array
!-----------------------------------------------------------------------
      hopipar(2,hl) = dddiab

!-----------------------------------------------------------------------
! Calculate overall number of DOF and allocate array for all geometries
!-----------------------------------------------------------------------
      allocate(allcent(ndofdd,totdim1))
      allocate(xgp(ndofddpes,nsymop,totdim1))
      allocate(gwpdist(nsymop,nsymop))

!--------------------------------------------------------------------------
! Get all combinations of the centres of Gaussians covering all the modes.
! Can probably be done easier...
!--------------------------------------------------------------------------
       do m=1,nmode
          if (.not. gwpm(m)) cycle
          call gwpcentres(m,allcent,qcenter(zcent(m,s1)),ndofdd,&
               nspfdof(m),vdimgp(m,s1),dimgp(m,s1),ndimgp(m,s1))
       enddo
 
!--------------------------------------------------------------------------
! add coordinates of point DVRs
!--------------------------------------------------------------------------
      if (lpntdvr) then
         f1=0
         do f=1,ndof
            if (f .eq. feb) cycle
            f1=f1+1
            if (basis(f) .eq. 19) allcent(f1,:) = rpbaspar(1,f)
         enddo
      endif

!-----------------------------------------------------------------------
! Set up transformed coordinates required for quantum chemistry, i.e.
! cartesians. If DB used, orient in correct Eckart frame. 
!-----------------------------------------------------------------------
      if (lddtrans) then
         ngp = totdim1
         do e=1,ngp
            call ddq2x(allcent(1,e),xgpoint)
            xgp(1:ndofddpes,1,e)=xgpoint(1:ndofddpes)
         enddo

      else 
         ngp = totdim1
         do e=1,ngp
!            call eckart(xmass,xgpoint,gp(1,e),ddeckart(1,1,e))
            xgp(1:ndofddpes,1,e)=allcent(1:ndofddpes,e)
!            call unitqxd(ddeckart(1,1,e),ndofddpes)
         enddo

      endif

!---------------------------------------------------------------------------
! If a new DB in a rdwr run, calculate first DB entry.
!
! Need to have a DB entry at the 'FC' point when using the Hessian 
! updating. Seems like a good idea to initialise the DB even if not
! using the updating. This part checks whether one of the GWP's initial 
! positions is there. If not, a DB point is created at the wavepacket centre.
!
! If however the FC point is pathological (i.e. a CoIn), read in a given
! reference point. (lrefdbrd = .true.)
!---------------------------------------------------------------------------
      if ((lddwrdb .and. lddrddb) .and. dbnrec.eq.0) then

         allocate(xgpoint0(ndofddpes,nsymop,1), &
                  lrddb(nsymop,1))
         allocate(gwpsymrep(nsymop,nsymop,1))
         gwpsymrep = 0

         if (lrefdbrd) then

            call rdrefpnt(gpoint,xgpoint)
            if(lddtrans) call ddq2x(gpoint,xgpoint)

            write(ilog,'(a)') '*** Calculating First DB point at :'
            i = 1
            do n=1,natm
               write(ilog,'(a,3f10.6)') xatnam(n),xgpoint(i:i+2)
               i=i+3
            enddo

            hopipar(3,hl)=0

         else 

! Get the position of the centre of the wavepacket.
            if(lddtrans)then
               gpoint(1:ndofdd) = rsbaspar(1,1:ndofdd,1)
               call ddq2x(gpoint,xgpoint)
            else
               xgpoint(1:ndofddpes) = rsbaspar(1,1:ndofddpes,1)
            endif
! If none of the GWPs is at the centre of the wavepacket, then a DB point 
! should be created there. hopipar(3) is number of function passed
! to Qchem routines. Set to 0 if centre of WP not a GWP.
            hopipar(3,hl)=0
            do e = 1, totdim1
               xtmp(:) = xgpoint(:)-xgp(:,1,e)
               norm=dnrm2(ndofddpes,xtmp,1)
               if (norm .lt. 1.0e-8_dop) then
                  if (ltraj) then
                     hopipar(3,hl) = ddtrajnum
                  else
                     hopipar(3,hl) = e
                  endif
               endif
            enddo
            if (hopipar(3,hl) .eq. 0) then
               write(ilog,'(a)') '*** Calculating First DB point at WP centre'
            endif
         endif

!-----------------------------------------------------------------------
! Calculate symmetry replicas 
! Work a bit here as initial point may well be a high symmetry point.
! 1. form symmetry replicas
! 2. check which are close, and symmetrise
! 3. if given reference point, check if symmetrisation has shifted 
!    position due to a poor choice too close to a point of symmetry
! 4. reform symmetry replicas 
! 5. check which are unique
!-----------------------------------------------------------------------
         lrddb = .false.
         if (dbpntgp .gt. 0 .and. lrefdbrd) then
            e = hopipar(3,hl) 
! save initial point in xgpoint
            if (e .gt. 0) xgpoint(1:ndofddpes) = xgp(1:ndofddpes,1,e)

! form symmetry replicas
            xgpoint0(1:ndofddpes,1,1) = xgpoint(1:ndofddpes)
            call mksymcent(xgpoint0,dbpntgp,nsymop,1,1)

! Ignore images if they are identical to the original function (i.e. original
! function is (close to) symmetric
! and symmetrise geometry (average over all "identical" replicas
            call distgwp(xgpoint0,gwpdist,nsymop,1,1,1,1)

            n1=1
            do n=2,nsymop
               if (gwpdist(n,1) .lt. 0.01) then
                  n1 = n1+1
                  xgpoint0(:,1,1) = xgpoint0(:,1,1) + xgpoint0(:,n,1)
               endif
            enddo

! if the struture has been averaged, recalculate symmetry replicas
            if (n1 .gt. 1) then
               xgpoint0(:,1,1) = xgpoint0(:,1,1) / dble(n1)
               call mksymcent(xgpoint0,dbpntgp,nsymop,1,1)
            endif

            call distgwp(xgpoint0,gwpdist,nsymop,1,1,1,1)

! check structure in OK position
            do n=2,nsymop
               if ((gwpdist(n,1) .gt. 0.01) .and. (gwpdist(n,1) .lt. ddmindb)) then
                  routine = 'Directdyn'
                  write(message,'(2a)')  &
                   'Given reference point too close to symmetric point. Move away.'
                  call errormsg
               endif
            enddo

! check all replicas for clashes
! and set up symmetry replica maps
            call distgwp(xgpoint0,gwpdist,nsymop,nsymop,1,1,1)
            lrddb(2:nsymop,1) = .false.
            n2=1
            gwpsymrep(1,1,1)=n2
            do n1=2,nsymop
               n2=n2+1
               gwpsymrep(n1,1,1)=n2
               gwpsymrep(1,n1,1)=-1  ! All relicas based on first DB point
               do n=1,n1-1
                  if (gwpdist(n,n1) .lt. ddmindb) then
                     lrddb(n1,1) = .true.
                     gwpsymrep(n1,1,1)=gwpsymrep(n,1,1)
                     n2=n2-1   ! correct index for missing replica
                     exit
                  endif
               enddo
            enddo

         else if (dbpntgp .gt. 0) then
            e = hopipar(3,hl) 
! save initial point in xgpoint
            if (e .gt. 0) xgpoint(1:ndofddpes) = xgp(1:ndofddpes,1,e)

! form symmetry replicas
            xgpoint0(1:ndofddpes,1,1) = xgpoint(1:ndofddpes)
            call mksymcent(xgpoint0,dbpntgp,nsymop,1,1)

! Ignore images if they are identical to the original function (i.e. original
! function is (close to) symmetric
! and symmetrise geometry (average over all "identical" replicas
            call distgwp(xgpoint0,gwpdist,nsymop,1,1,1,1)

            n1=1
            do n=2,nsymop
               if (gwpdist(n,1) .lt. ddmindb) then
                  n1 = n1+1
                  xgpoint0(:,1,1) = xgpoint0(:,1,1) + xgpoint0(:,n,1)
               endif
            enddo

! if the struture has been averaged, recalculate symmetry replicas
            if (n1 .gt. 1) then
               xgpoint0(:,1,1) = xgpoint0(:,1,1) / dble(n1)
               call mksymcent(xgpoint0,dbpntgp,nsymop,1,1)
            endif

! check all replicas for clashes
            call distgwp(xgpoint0,gwpdist,nsymop,nsymop,1,1,1)
            lrddb(2:nsymop,1) = .false.
            n2=1
            gwpsymrep(1,1,1)=n2
            gwpsymrep(1,n1,1)=-1  ! All relicas based on first DB point
            do n1=2,nsymop
               n2=n2+1
               gwpsymrep(n1,1,1)=n2
               do n=1,n1-1
                  if (gwpdist(n,n1) .lt. ddmindb) then
                     lrddb(n1,1) = .true.
                     gwpsymrep(n1,1,1)=gwpsymrep(n,1,1)
                     n2=n2-1
                     exit
                  endif
               enddo
            enddo

         else
            xgpoint0(1:ndofddpes,1,1) = xgpoint(1:ndofddpes)

         endif

         if (ddlog) write(ilog,'(a)') 'Generating First DB Point'

         hopipar(4,hl)=s1
         if (hess0 .eq. 1) then    ! Generate Hessian from GS frequencies
            hopipar(6,hl)=1
         else
            hopipar(6,hl)=lhaorder
         endif

         nex2=1
         call init_qchem_calculations(nex2,nsymop,adder1,adder2,addip, &
              adv,ddder1,ddder2,dddip,ddmoco,ddnact,adsoc,ddv, &
              diabtrans,ddcv,diabctrans,ciconfig,cicoeff,interp, &
              addip_size,adv_size,ddder1_size, ddder2_size,dddip_size, &
              ddnact_size,ddv_size, diabtrans_size,adsoc_size,interp_size, &
              ddmoco_size,ciconfig_size,cicoeff_size)
         allocate(eex2(1))
         eex2=1
         if(.not.lpestest)then
            call lhaqchem(time,hopilab(hl),xgpoint0,adv,adder1,adder2,&
                 addip,ddmoco,ddnact,adsoc,ndofddpes,hopipar(1,hl),lfail,&
                 ciconfig,cicoeff)
            if(lfail)then
               routine = 'Getddpes (directdyn.F90)'
               message = 'First electronic structure calc. failed'
               call errormsg
            endif
            call unitqxd(diabtrans,nddstate)
            call transform(diabtrans,adv,adder1,adder2,ddv,ddder1,ddder2,&
                 ddnact,ndofddpes,nddstate)
! dddip = addip  
            dddip = addip

         else
            call ddvmcgpestest(idpestest,xgpoint,adv,adder1,&
                 adder2,ndofddpes,nddstate,hopipar(6,hl))
            call cpvxd(adv,ddv,nddstate*nddstate)
            call cpvxd(adder1,ddder1,ndofddpes*nddstate*nddstate)
            call cpvxd(adder2,ddder2, &
                 ndofddpes*ndofddpes*nddstate*nddstate)
         endif

!-----------------------------------------------------------------------
! Generate Hessian from GS frequencies
!-----------------------------------------------------------------------
         if (hess0 .eq. 1) then
            call genhess0(adder2,ddnact,adv)
         endif

!-----------------------------------------------------------------------
! form image data for adiabatic terms
!-----------------------------------------------------------------------
         if (nsymop .gt. 1) then
            iwr = 1
            call mksympesa1(adder1(1:ndofddpes,1:nddstate,1:nddstate,1:nsymop,1),&
                 ddnact(1:ndofddpes,1:nactdim,1:nsymop,1),&
                 addip(1:3,1:nddstate,1:nddstate,1:nsymop,1),&
                 adsoc(1:nsocstate,1:nsocstate,1:nsymop,1),&
                 lrddb(1,1),nsymop,dbpntgp,iwr)
         endif

!-----------------------------------------------------------------------
! diabatise
!-----------------------------------------------------------------------
         if(dddiab .eq. 0) then
           do n=1,nsymop
              ddv(:,:,n,1)=adv(:,:,n,1)
              ddder1(:,:,:,n,1)=adder1(:,:,:,n,1)
              ddder2(:,:,:,:,n,1)=adder2(:,:,:,:,n,1)
              call unitqxd(diabtrans(1,1,n,1),nddstate)
           enddo
         else if(dddiab.eq.1) then
           order=2
           do n=1,nsymop
              if (lrddb(n,1)) cycle
              call diabat1(xgpoint0(1,n,1),adv(1,1,1,1),adder1(1,1,1,n,1),&
                   adder2(1,1,1,1,n,1),ddv(1,1,n,1),ddder1(1,1,1,n,1),&
                   ddder2(1,1,1,1,n,1),order)

              call getadt(ddv(1,1,n,1),diabtrans(1,1,n,1),nddstate) 
           enddo
         else if(dddiab.eq.2) then
           do n=1,nsymop
              if (lrddb(n,1)) cycle
              call diabat2(xgpoint0(1,n,1),adv(1,1,1,1),adder1(1,1,1,n,1),&
                   adder2(1,1,1,1,n,1),ddv(1,1,n,1),ddder1(1,1,1,n,1),&
                   ddder2(1,1,1,1,n,1),order,1,1,time)
              call getadt(ddv(1,1,n,1),diabtrans(1,1,n,1),nddstate) 
           enddo
         else if(dddiab.eq.4) then 
           n=1
           nrec = n
           if (ldbsmall) num_gp = 1
           call diabat4(xgpoint0(1,n,1),adv(1,1,1,1),adder1(1,1,1,n,1),&
                adder2(1,1,1,1,n,1),ddnact(1,1,n,1),&
                ddv(1,1,n,1),ddder1(1,1,1,n,1),&
                ddder2(1,1,1,1,n,1),diabtrans(1,1,n,1),.false., &
                0,nrec,interp(n,1))
         else if(dddiab.eq.6) then 
           do n=1,nsymop
              if (lrddb(n,1)) cycle
              call diabat6(xgpoint0(1,n,1),adv(1,1,1,1),adder1(1,1,1,n,1),&
                   adder2(1,1,1,1,n,1),ddnact(1,1,n,1),&
                   ddv(1,1,n,1),ddder1(1,1,1,n,1),&
                   ddder2(1,1,1,1,n,1),diabtrans(1,1,n,1), &
                   ddmoco(1,1,1), &
                   ciconfig(1,1,1),cicoeff(1,1,1,1), &
                   1,nrec,.false.)   ! idbloc0=1 i.e. diabatise wrt first strtucture
           enddo
         else
            routine = 'Getddpes (directdyn.f90)'
            write(message,'(a,i5)') &
                     'Unknown Diabatisation procedure requested ',dddiab
            call errormsg
         endif

! Transform addip -> dddip - each component separately
         if (ladipdb) then
            do i=1,3
               call simtranbd(addip(i,1:nddstate,1:nddstate,1,1),&
                    diabtrans(1:nddstate,1:nddstate,1,1),&
                    dddip(i,1:nddstate,1:nddstate,1,1),nddstate)
            enddo
         endif

! Transform SOCs to diabatic and add to diabatic potential
         if (lcasocdb) then
            call addsoc(adsoc(1:nsocstate,1:nsocstate,1,1),&
                 ddv(1:nddstate,1:nddstate,1,1),&
                 ddcv(1:nddstate,1:nddstate,1,1),&
                 diabtrans(1:nddstate,1:nddstate,1,1))
         endif

!-----------------------------------------------------------------------
! write first point to memory and then to DB
! This has to be done before symmetry replicas can be diabatised
! using global diabatisation.
!-----------------------------------------------------------------------
         n1=1
         n=1
         if (ldbsave) then  
            nrec=n
            call dddb_wrmem(xgpoint0(1,n,1),ddv(1,1,n,1),ddder1(1,1,1,n,1),&
                 ddder2(1,1,1,1,n,1),adv(1,1,1,1),dddip(1,1,1,n,1),&
                 diabtrans(1,1,n,1),ddnact(1,1,n,1),ddmoco(1,1,1),&
                 ddcv(1,1,n,1), diabctrans(1,1,n,1),&
                 ciconfig(1,1,n),cicoeff(1,1,1,n),&
                 ndofddpes,nrec,interp(n,1),gwpsymrep(1,1,1))
         endif

         call dddb_wr1(xgpoint0(1,n,1),adv(1,1,1,1), &
              adder1(1,1,1,n,1),adder2(1,1,1,1,n,1), &
              addip(1,1,1,n,1),ddv(1,1,n,1),ddder1(1,1,1,n,1),&
              ddder2(1,1,1,1,n,1),dddip(1,1,1,n,1),ddmoco(1,1,1),&
              diabtrans(1,1,n,1),ddnact(1,1,n,1),ddcv(1,1,n,1),&
              diabctrans(1,1,n,1),&
              ciconfig(1,1,n),cicoeff(1,1,1,n),&
              ndofddpes,nddstate,interp(n,1),dercpdim,&
              gwpsymrep(1,1,1),adsoc(1,1,n,1),nsocstate)
! Write CI vector configurations (and data as first call)
         if (lcivecdb) call dbwrcidat(the_db,nsmult,idxmult,imultblk,&
                            nciconf)
         dbnrec=dbnrec+1

! Initialise local DBs
         if (ldbsmall) call gpdbupdate(qcenter)

         if (dddiab.eq.4) then 
            do n=2,nsymop
               if (lrddb(n,1)) cycle
               if (ldbsmall) num_gp = 1

               nrec = nrec+1
               call diabat4(xgpoint0(1,n,1),adv(1,1,1,1),adder1(1,1,1,n,1),&
                    adder2(1,1,1,1,n,1),ddnact(1,1,n,1),&
                    ddv(1,1,n,1),ddder1(1,1,1,n,1),&
                    ddder2(1,1,1,1,n,1),diabtrans(1,1,n,1),.false., &
                    1,nrec,interp(n,1))
            enddo
         endif

! Transform addip -> dddip for symmetry replicas - each component separately
         if (ladipdb) then
            do n=2,nsymop
               do i=1,3
                  call simtranbd(addip(i,1:nddstate,1:nddstate,n,1),&
                       diabtrans(1:nddstate,1:nddstate,n,1),&
                       dddip(i,1:nddstate,1:nddstate,n,1),nddstate)
               enddo
            enddo
         endif

! Transform SOCs to diabatic and add to diabatic potential
         if (lcasocdb) then
            do n=2,nsymop
               call addsoc(adsoc(1:nsocstate,1:nsocstate,n,1),&
                    ddv(1:nddstate,1:nddstate,n,1),&
                    ddcv(1:nddstate,1:nddstate,n,1),&
                    diabtrans(1:nddstate,1:nddstate,n,1))
            enddo
         endif

! write replicas to memory
         if (ldbsave) then  
            do n=2,nsymop
               if (lrddb(n,1)) cycle

               n1 = n1+1
               call dddb_wrmem(xgpoint0(1,n,1),ddv(1,1,n,1),ddder1(1,1,1,n,1),&
                    ddder2(1,1,1,1,n,1),adv(1,1,1,1),dddip(1,1,1,n,1),&
                    diabtrans(1,1,n,1),ddnact(1,1,n,1),ddmoco(1,1,1),&
                    ddcv(1,1,n,1), diabctrans(1,1,n,1),&
                    ciconfig(1,1,1),cicoeff(1,1,1,1),&
                    ndofddpes,n1,interp(n,1),gwpsymrep(1,n,1))
            enddo

         endif
! write replicas to DB
         do n=2,nsymop
            if (lrddb(n,1)) cycle

            call dddb_wr1(xgpoint0(1,n,1),adv(1,1,1,1), &
                 adder1(1,1,1,n,1),adder2(1,1,1,1,n,1), &
                 addip(1,1,1,n,1),ddv(1,1,n,1),ddder1(1,1,1,n,1),&
                 ddder2(1,1,1,1,n,1),dddip(1,1,1,n,1),ddmoco(1,1,1),&
                 diabtrans(1,1,n,1),ddnact(1,1,n,1),ddcv(1,1,n,1),&
                 diabctrans(1,1,n,1),&
                 ciconfig(1,1,1),cicoeff(1,1,1,1),&
                 ndofddpes,nddstate,interp(n,1),dercpdim,&
                 gwpsymrep(1,n,1),adsoc(1,1,n,1),nsocstate)
            dbnrec=dbnrec+1
         enddo

         call finalise_qchem_calculations(adder1,adder2,addip,adv, &
              ddder1,ddder2,dddip,ddmoco,ddnact,adsoc,ddv,diabtrans, &
              ddcv,diabctrans,interp,ciconfig,cicoeff)
         deallocate(eex2)
         deallocate(xgpoint0,lrddb)
         if (allocated(gwpsymrep)) deallocate(gwpsymrep)

      endif   ! finish first DB point

!-----------------------------------------------------------------------
! initialise flags for reading DB and 
! allocate memory for DB distance arrays
!-----------------------------------------------------------------------
      allocate(idbloc(nsymop,ngp))
      allocate(r1((dbnrec+totdim1),nsymop,ngp))
      allocate(lrddb(nsymop,ngp))
      r1=0.0_dop
      idbloc=0
      lrddb=.true.

!-----------------------------------------------------------------------
! allocate arrays for symmetry replica manipulations of Local DBs
!-----------------------------------------------------------------------
      if (dbpntgp .gt. 0) then
         if (ldbsmall) then
            nrec0=maxval(dbnrec_gp(1:ngp))
            allocate(locpt0(nrec0,ngp))
            allocate(dbnrec0(ngp))
            locpt0 = 0
         endif
      endif

      allocate(gwpsymrep(nsymop,nsymop,ngp))
      gwpsymrep = 0

!-----------------------------------------------------------------------
! Look in DB to get closest geometries. Store location.
! If time to update, look whether DB points can be used.
!-----------------------------------------------------------------------
      if (lddrddb .and. dbnrec.gt.0) then

!-----------------------------------------------------------------------
! generate symmetry replicas
!-----------------------------------------------------------------------
         if (nsymop .gt. 1) then
            do e=1,ngp
               call mksymcent(xgp,dbpntgp,nsymop,ngp,e)

! and symmetrise geometry (average over all "identical" replicas
               call distgwp(xgp,gwpdist,nsymop,1,ngp,e,e)

               n1=1
               do n=2,nsymop
                  if (gwpdist(n,1) .lt. ddmindb) then
                     n1 = n1+1
                     xgp(:,1,e) = xgp(:,1,e) + xgp(:,n,e)
                  endif
               enddo
               xgp(:,1,e) = xgp(:,1,e) / dble(n1)

! if the struture has been averaged, recalculate symmetry replicas
               if (n1 .gt. 1) call mksymcent(xgp,dbpntgp,nsymop,ngp,e)
            enddo
         endif

!-----------------------------------------------------------------------
! check DB for nearest point. Flag
!-----------------------------------------------------------------------
         do e=1,ngp
            if (ldbsmall) then
               num_gp=e
               call distdb_gp(xgp(1,1,e),idbloc(1,e),r1(1,1,e),mindist,&
                    mindisp,dbnrec_gp(e))
            else
               call distdb(xgp(1,1,e),idbloc(1,e),r1(1,1,e),mindist,&
                    mindisp,dbnrec)
            endif

            if(ldbvar .or. ldbhar)then
               if(mindist.lt.tiny)cycle
               if(dbnrec.gt.1)then
                  if(ldbvar)then
                     call dbuncertain(xgp(1,1,e),lrddb(1,e),s1)
                     if(.not.lrddb(1,e))write(ilog,'(a,i5)')'DB var. ',e

                  else
                     call dbharmonic(xgp(1,1,e),lrddb(1,e),s1,idbloc(1,e))
                     if(.not.lrddb(1,e))write(ilog,'(a,i5)')'DB har. ',e

                  endif
               endif
               lrddb(1,e) = lrddb(1,e) .and. (mindisp.lt.ddmindb)

               if(mindisp.ge.ddmindb)&
                    write(ilog,'(a,i5)')'DB disp. ',e

            else
               if (ddlogfull) then
                  write(ilog,'(a,i7,2(a,f7.4))') &
                       'Nearest DB ID ',idbloc(1,e),'  DelX : ',mindist,&
                                '  Disp : ',mindisp
               endif

               if (mindisp .ge. ddmindb) lrddb(:,e) = .false.

            endif
         enddo

!-----------------------------------------------------------------------
! estimate energy and ignore if very high (point in unimportant
! region of configuration space).
!-----------------------------------------------------------------------
         if (dbmaxe .lt. 900.0) then
            do e=1,ngp
               if (lrddb(1,e)) cycle
               
               igpoint(:) = 1
               call calcpes(mr(mrhops),pesdia,igpoint,xgp(1,1,e),pesdiaz,izflag,1)
               emax = -999.0
               do s=1,nddstate
                  emax = max(emax,pesdia(s,s))
               enddo
               if (emax .gt. dbmaxe) then
                  lrddb(:,e) = .true.
                  if (ddlogfull) then
                     write(ilog,'(a,i5,a,f10.4)') &
             'GWP config ',e,' QC not calculated as high E :',emax
                  endif
               endif
            enddo
         endif

!-----------------------------------------------------------------------
! compare function to previous functions, including replicas
!-----------------------------------------------------------------------
         do e=2,ngp
            do e1=1,e-1
               if (lrddb(1,e1)) cycle

               call distgwp(xgp,gwpdist,nsymop,1,ngp,e1,e)
               do n = 1,nsymop
                  if (gwpdist(n,1) .lt. ddmindb) lrddb(1,e) = .true.
               enddo
            enddo

! ignore if generated by a previous GWP
            if (lrddb(1,e)) lrddb(:,e) = .true.
         enddo

!-----------------------------------------------------------------------
! check all replicas for clashes and ignore images if they are identical 
! to the original function, i.e. original function is (close to) symmetric
!-----------------------------------------------------------------------
         if (nsymop .gt. 1) then
            if (ldbsmall) call symdb_gp(locpt0,dbnrec0,nrec0,ngp)

            do e=1,ngp
               if (lrddb(1,e)) cycle

               n2=dbnrec+1
               gwpsymrep(1,1,e)=n2
               call distgwp(xgp,gwpdist,nsymop,nsymop,ngp,e,e)
               lrddb(2:nsymop,e) = .false.
               do n1=2,nsymop
                  n2 = n2+1    ! rep will be stored in next DB record
                  gwpsymrep(n1,1,e)=n2
                  gwpsymrep(1,n1,e)=-gwpsymrep(1,1,e)
                  do n=1,n1-1
                     if (gwpdist(n,n1) .lt. ddmindb) then
                        lrddb(n1,e) = .true.
                        gwpsymrep(n1,1,e)=gwpsymrep(n,1,e)   ! rep n1 is same 
                                                             ! struture as rep n
                        n2 = n2-1          ! correct next record for ignored rep
                        exit
                     endif
                  enddo
               enddo

! Get nearest DB points for image geometries to be included in DB
               if (ldbsmall) then
                  do n=1,nsymop    ! need all distances for local DBs
                     if (lrddb(n,e)) cycle
                     num_gp=e
                     call distdb_gp(xgp(1,n,e),idbloc(n,e),r1(1,n,e),mindist,&
                          mindisp,dbnrec_gp(e))
                  enddo
               else
                  do n=2,nsymop
                     if (lrddb(n,e)) cycle
                     call distdb(xgp(1,n,e),idbloc(n,e),r1(1,n,e),mindist,&
                          mindisp,dbnrec)
                  enddo
               endif
            enddo
         endif

      else if (lddrddb .and. dbnrec.eq.0) then
         routine = 'getddpes'
         message = "Trying to read empty DB"
         call errormsg

      else 
         lrddb(:,:)=.false.

      endif   ! end of generating structures and flagging 
              ! points to be calculated

!-----------------------------------------------------------------------
! now calculate new DB point if database cannot be used
!
! if reading from DB, lrddb(e) = .true. 
!-----------------------------------------------------------------------
      allocate(eex2(ngp))
      eex2=0

      nex2=0
      do e=1,ngp
         if(.not.lrddb(1,e)) then
            nex2=nex2+1
            eex2(nex2)=e
         endif
      enddo

!-----------------------------------------------------------------------
! log no. of calculations to be made
!-----------------------------------------------------------------------
      if (ddlogfull) then
         write(ilog,'(a,i5)') 'No. QC calculations : ',nex2
         write(ilog,'(a,i5)') 'No. read from DB    : ',totdim1-nex2
      endif

!-----------------------------------------------------------------------
! Go to end if nothing to calculate
! If symmetrised local DBs used, restore originals
!-----------------------------------------------------------------------
      if (nex2 .eq. 0) then
         if (ldbsmall .and. dbpntgp .gt. 0) &
                 call symdb_gp0(locpt0,dbnrec0,nrec0,ngp)
         go to 900
      endif

      if (ddlog) then
         write(ilog,'(a,f12.6)') 'DD Update requires new point at: ',time/fs
         write(ilog,'(a,i5)') 'No. QC calculations : ',nex2
      endif

!-----------------------------------------------------------------------
! second parallel region start (local order)
!-----------------------------------------------------------------------
            !Add extra layer of MPI parallelization
#ifdef MPI

   if(lmpi.and.lmpigetddpes.and.lmpistarted) then
      mpitask=14
      call MPI_Bcast(mpitask,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
! --- Determine Job array to dispatch to all mpi process
!     Define maximum number of MPI process require to run calculation
      mxmpi=min(nex2,nmpi)
      mpinex1=1
      mpinex2=0
      mpitemp=DBLE(nex2)/DBLE(mxmpi)
      mpinex2(1)=INT(mpitemp)
      do e=2,mxmpi
         mpinex1(e)= mpinex2(e-1)+1
         mpitemp=mpitemp+DBLE(nex2)/DBLE(mxmpi)
         mpinex2(e)= INT(mpitemp)
         if (mpinex2(e).gt.nex2.or.mpinex2(e).eq.nex2) then
            mpinex2(e)=nex2
            exit
         endif
      enddo

      ! Distribute info about required qchem calcs to worker processes
      call MPI_Scatter(mpinex1, 1, MPI_INTEGER, master_nex1, 1, MPI_INTEGER, master, MPI_COMM_WORLD, mpierr)
      call MPI_Scatter(mpinex2, 1, MPI_INTEGER, master_nex2, 1, MPI_INTEGER, master, MPI_COMM_WORLD, mpierr)

      ! Create new communicator for only those processes that will
      ! perform qchem calcs
      call MPI_Comm_split(MPI_COMM_WORLD, 0, mpirank, mpi_comm_worker, mpierr)

      ! Broadcast data needed for qcalcs by workers
      qcalc_data = init_qcalc_data(totdim1,s1,hl,ngp,mxmpi,ndofddpes, &
           nddstate,nsymop,dbnrec,ddlaenge,dbdlength,nroot,maxdim, &
           qcsubform,nbasis,maxhpar,maxhop,lhaorder,dddiab,time,dde0, &
           lpestest,lupdhes,ldbsave,lddrddb,lqcsaveall,ddextcmd,fchkcmd,ddname)
      qcalc_type = get_qcalc_mpi_type(qcalc_data)
      call MPI_Bcast(qcalc_data,1,qcalc_type,master,mpi_comm_worker, &
           mpierr)
      call MPI_Bcast(eex2,ngp,MPI_INTEGER,master,mpi_comm_worker, &
           mpierr)
      call MPI_Bcast(idbloc,nsymop*ngp,MPI_INTEGER,master, &
           mpi_comm_worker,mpierr)
      call MPI_Bcast(r1,(dbnrec*totdim1)*nsymop*ngp, &
           MPI_DOUBLE_PRECISION,master,mpi_comm_worker,mpierr)
      call MPI_Bcast(xgp,ndofddpes*nsymop*totdim1, &
           MPI_DOUBLE_PRECISION,master,mpi_comm_worker,mpierr)
      call MPI_Bcast(atnam,natm*c1,MPI_CHARACTER,master, &
           mpi_comm_worker,mpierr)
      call MPI_Bcast(hopipar,maxhpar*maxhop,MPI_INTEGER,master, &
           mpi_comm_worker,mpierr)

      call MPI_barrier(mpi_comm_worker,mpierr)
      !allocate array for master

      ! Allocate and initialise memory for qchem calculation output
      ! and calculate size factors for later use
      call init_qchem_calculations(nex2,nsymop,adder1,adder2,addip, &
           adv,ddder1,ddder2,dddip,ddmoco,ddnact,adsoc,ddv, &
           diabtrans,ddcv,diabctrans,ciconfig,cicoeff,interp,&
           addip_size,adv_size,ddder1_size,ddder2_size,dddip_size,&
           ddnact_size,ddv_size,diabtrans_size,adsoc_size,interp_size,&
           ddmoco_size,ciconfig_size,cicoeff_size)
      ! Perform required qchem calculations
      call mpigetddpes(time,totdim1,s1,mpinex1(1),mpinex2(1),&
            hl,ngp,nsymop,eex2(mpinex1(1):mpinex2(1)),idbloc(:,:),&
            mxmpi,xgp(:,:,:),r1,ddv,adv,diabtrans,ddnact,ddder1,&
            adder1,ddder2,adder2,dddip,addip,adsoc,ddmoco,interp)

      ! Transfer qchem calculations results from the worker processes
      call qchem_gatherv_master_dop(ddv, ddv_size, mpinex1, mpinex2, &
           mpi_comm_worker)
      call qchem_gatherv_master_dop(adv, adv_size, mpinex1, mpinex2, &
           mpi_comm_worker)
      call qchem_gatherv_master_dop(ddder1, ddder1_size, mpinex1, &
           mpinex2, mpi_comm_worker)
      call qchem_gatherv_master_dop(ddder2, ddder2_size, mpinex1, &
           mpinex2, mpi_comm_worker)
      call qchem_gatherv_master_dop(dddip, dddip_size,  mpinex1, &
           mpinex2, mpi_comm_worker)
      call qchem_gatherv_master_dop(diabtrans, diabtrans_size, mpinex1, &
           mpinex2, mpi_comm_worker)
      call qchem_gatherv_master_dop(ddnact, ddnact_size, mpinex1, &
           mpinex2, mpi_comm_worker)
      call qchem_gatherv_master_cdop(adsoc, adsoc_size, mpinex1, mpinex2, &
           mpi_comm_worker)
      call qchem_gatherv_master_int(interp, interp_size, mpinex1, &
           mpinex2, mpi_comm_worker)

!-----------------------------------------------------------------------
!     If no parallelisation is used then a usual calculation is 
!     performed.
!-----------------------------------------------------------------------
else

#endif
! Allocate and initialise memory for qchem calculation output
      call init_qchem_calculations(nex2,nsymop,adder1,adder2,addip, &
           adv,ddder1,ddder2,dddip,ddmoco,ddnact,adsoc,ddv, &
           diabtrans,ddcv,diabctrans,ciconfig,cicoeff,interp,&
           addip_size,adv_size,ddder1_size, ddder2_size,dddip_size,&
           ddnact_size,ddv_size,diabtrans_size,adsoc_size,interp_size,&
           ddmoco_size,ciconfig_size,cicoeff_size)
      allocate(lskip(nex2))
      lskip = .false.
! array to carry hopipar to funcqchem in parallel loop
      allocate(hopiparp(maxhpar,nex2))
      do e=1,nex2
         e1=eex2(e)
         hopiparp(:,e)=hopipar(:,hl)
         if (ltraj) then
            hopiparp(3,e)=ddtrajnum
         else
            hopiparp(3,e)=e1
         endif
         hopiparp(4,e)=s1
         hopiparp(6,e)=lhaorder
      enddo

!$omp parallel if(lompqc) num_threads(min(ompthread,nex2)) private(e,e1,lfail)
!$omp do schedule(dynamic) 
      do e=1,nex2
         e1=eex2(e)
         if (lupdhes .and. (dbnrec .ne. 0 .or. e.gt.1)) hopiparp(6,e)=1

!-----------------------------------------------------------------------
! Get guess MO coefficients from the database if using CAS.
!-----------------------------------------------------------------------
         if (lcas .and. (dbnrec.gt.0.and.idbloc(1,e1).ne.0)) then
            if (ldbsave) then
               ddmoco(:,:,e) = dbmoco(:,:,idbloc(1,e1))
            else
               call ddmo_rd(ddmoco(1:nbasis,1:nbasis,e),nbasis,idbloc(1,e1))
            endif
         endif

!-----------------------------------------------------------------------
! call interface to external quantum chemistry program
!-----------------------------------------------------------------------
         if (ddlog) write(ilog,'(a,i5)') 'Generating DB Point for GWP : ',e1
         if(.not.lpestest)then
            call lhaqchem(time,hopilab(hl),xgp(1:ndofddpes,1,e1),&
                 adv(1:nddstate,1:nddstate,1,e),&
                 adder1(1:ndofddpes,1:nddstate,1:nddstate,1,e),& 
                 adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,1,e),&
                 dddip(1:3,1:nddstate,1:nddstate,1,e),&
                 ddmoco(1:nbasis,1:nbasis,e),&
                 ddnact(1:ndofddpes,1:nactdim,1,e),&
                 adsoc(1:nsocstate,1:nsocstate,1,e),&
                 ndofddpes,hopiparp(1,e),lfail,&
                 ciconfig(:,:,e),cicoeff(:,:,:,e))

            lskip(e)=.false.
            if (lfail) then
               if (lddwrdb .and. dbnrec.eq.0 .and. e .eq. 1) then
                  routine='Getddpes'
                  message='Initial QC calculation failed: No reference point'
                  call errormsg
               else
                  routine='Getddpes'
                  message='QC calculation failed'
                  call warnmsg
               endif
! if global diabatisation need to interpolate, otherwise ignore point and continue
               if (dddiab .eq. 4) then
                  lskip(e)=.true.
               else
                  cycle
               endif
            endif
         else
            call ddvmcgpestest(idpestest,xgp(1:ndofddpes,1:nsymop,e1),&
                 adv(1:nddstate,1:nddstate,1,e),&
                 adder1(1:ndofddpes,1:nddstate,1:nddstate,1:nsymop,e),&
                 adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,1:nsymop,e),&
                 ndofddpes,nddstate,hopiparp(6,e))
         endif

!-----------------------------------------------------------------------
! form image data for adiabatic terms
!-----------------------------------------------------------------------
         if (nsymop .gt. 1) then
            iwr = 1
            call mksympesa1(adder1(1:ndofddpes,1:nddstate,1:nddstate,1:nsymop,e),&
                 ddnact(1:ndofddpes,1:nactdim,1:nsymop,e),&
                 addip(1:3,1:nddstate,1:nddstate,1:nsymop,e),&
                 adsoc(1:nsocstate,1:nsocstate,1:nsymop,e),&
                 lrddb(:,e1),nsymop,dbpntgp,iwr)
         endif

!-----------------------------------------------------------------------
! Diabatise
!-----------------------------------------------------------------------
         if (dddiab .eq. 0) then
            do n=1,nsymop
               ddv(:,:,n,e)=adv(:,:,1,e)
               ddder1(:,:,:,n,e)=adder1(:,:,:,n,e)
               if (.not. lupdhes) ddder2(:,:,:,:,n,e)=adder2(:,:,:,:,n,e)
               call unitqxd(diabtrans(1,1,n,e),nddstate)
            enddo
         else if (dddiab .eq. 1) then
            order = 2
            do n=1,nsymop
               call diabat1(xgp(1:ndofddpes,n,e1),&
                    adv(1:nddstate,1:nddstate,1,e),&
                    adder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    ddv(1:nddstate,1:nddstate,n,e),&
                    ddder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    ddder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    order)
               call getadt(ddv(1,1,n,e),diabtrans(1,1,n,e),nddstate) 
            enddo
         else if (dddiab .eq. 2) then
            order = 2
            do n=1,nsymop
               call diabat2(xgp(1:ndofddpes,n,e1),&
                    adv(1:nddstate,1:nddstate,1,e),&
                    adder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    ddv(1:nddstate,1:nddstate,n,e),&
                    ddder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    ddder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    order,e,e1,time)
               call getadt(ddv(1,1,n,e),diabtrans(1,1,n,e),nddstate) 
            enddo
         else if (dddiab .eq. 4) then
            order = 2
            if (ldbsmall) num_gp = e1
! set up pointer for new record number
            nrec = dbnrec
            do n=1,nsymop
               if (lrddb(n,e1)) cycle

               nrec = nrec+1
               call diabat4(xgp(1:ndofddpes,n,e1),&
                    adv(1:nddstate,1:nddstate,1,e),&
                    adder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    ddnact(1:ndofddpes,1:nactdim,n,e),&
                    ddv(1:nddstate,1:nddstate,n,e),&
                    ddder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    ddder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    diabtrans(1:nddstate,1:nddstate,n,e),&
                    lskip(e),idbloc(n,e1),nrec,interp(n,e))
            enddo

         else if (dddiab .eq. 6) then
            order = 2
! set up pointer for new record number
            nrec = dbnrec
            do n=1,nsymop
               if (lrddb(n,e1)) cycle

               nrec = nrec+1
               call diabat6(xgp(1:ndofddpes,n,e1),&
                    adv(1:nddstate,1:nddstate,1,e),&
                    adder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    ddnact(1:ndofddpes,1:nactdim,n,e),&
                    ddv(1:nddstate,1:nddstate,n,e),&
                    ddder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    ddder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    diabtrans(1:nddstate,1:nddstate,n,e),&
                    ddmoco(1:nbasis,1:nbasis,e),&
                    ciconfig(1:maxcivec,1:nsmult,e),&
                    cicoeff(1:maxcivec,1:maxmultblk,1:nsmult,e), &
                    1,nrec,lskip(e)) ! idbloc0=1 i.e. diabatise wrt first strtucture
            enddo
         else
            routine = 'Getddpes (directdyn.f90)'
            write(message,'(a,i5)') &
                     'Unknown Diabatisation procedure requested ',dddiab
            call errormsg
         endif

! Transform addip -> dddips - each component separately
         if (ladipdb) then
            do n=1,nsymop
               do i=1,3
                  call simtranbd(addip(i,1:nddstate,1:nddstate,n,e),&
                       diabtrans(1:nddstate,1:nddstate,n,e),&
                       dddip(i,1:nddstate,1:nddstate,n,e),nddstate)
               enddo
            enddo
         endif

! Transform SOCs to diabatic and add to diabatic potential
         if (lcasocdb) then
            do n=1,nsymop
               call addsoc(adsoc(1:nsocstate,1:nsocstate,n,1),&
                    ddv(1:nddstate,1:nddstate,n,1),&
                    ddcv(1:nddstate,1:nddstate,n,1),&
                    diabtrans(1:nddstate,1:nddstate,n,1))
            enddo
         endif

! might need to enforce symmetry of ddder1

         if (lupdhes) then
            if (ldbsmall) num_gp = e1
            do n=1,nsymop
               if (lrddb(n,e1)) cycle

               if (ldbsmall) then
                  call hessian_update1(xgp(1,n,e1),ddder1(1,1,1,n,e),&
                       ddder2(1,1,1,1,n,e),r1(1,n,e1),dbnrec_gp(num_gp))
               else
                  call hessian_update1(xgp(1,n,e1),ddder1(1,1,1,n,e),&
                       ddder2(1,1,1,1,n,e),r1(1,n,e1),dbnrec)
               endif
! Don't form approx adiabatic Hessian - not needed
    !           call hestrans(diabtrans(1,1,n,e),adder2(1,1,1,1,n,e),&
    !                ddder2(1,1,1,1,n,e),ndofddpes,nddstate)

            enddo
         endif

! if CAS fails (interpolated point) save MOCO from nearest point rather than actual
         if (interp(1,e) .eq. -1.and.idbloc(1,e1).ne.0) then
            if (ldbsave) then
               ddmoco(:,:,e) = dbmoco(:,:,idbloc(1,e1))
            else
               call ddmo_rd(ddmoco(1:nbasis,1:nbasis,e),nbasis,idbloc(1,e1))
            endif
         endif

      enddo
!$omp enddo
!$omp end parallel
      if (allocated(hopiparp)) deallocate(hopiparp)
#ifdef MPI
   end if
#endif
!-----------------------------------------------------------------------
! If symmetrised local DBs used, restore originals
!-----------------------------------------------------------------------
      if (ldbsmall .and. dbpntgp .gt. 0) &
                 call symdb_gp0(locpt0,dbnrec0,nrec0,ngp)

!-----------------------------------------------------------------------
! Write new data to memory 
!-----------------------------------------------------------------------
      if (ldbsave) then
         n1=dbnrec
         if ((n1+nsymop) .gt. dbmemdim) then
            routine = 'GETDDPES (directdyn.f90)'
            write(message,'(a,i10)') &
  'DB too small - use maxdbsize in run-section. Present size : ',dbmemdim
            call errormsg
         endif
         do e=1,nex2
            e1=eex2(e)
            do n=1,nsymop
               if (lrddb(n,e1)) cycle

               n1 = n1+1
               call dddb_wrmem(xgp(1:ndofddpes,n,e1),&
                    ddv(1:nddstate,1:nddstate,n,e),&
                    ddder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    ddder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    adv(1:nddstate,1:nddstate,1,e),&
                    dddip(1:3,1:nddstate,1:nddstate,n,e),&
                    diabtrans(1:nddstate,1:nddstate,n,e),&
                    ddnact(1:ndofddpes,1:nactdim,n,e),&
                    ddmoco(1:nbasis,1:nbasis,e),&
                    ddcv(1:nddstate,1:nddstate,n,e),&
                    diabctrans(1,1,n,e),&
                    ciconfig(1:maxcivec,1:nsmult,e),&
                    cicoeff(1:maxcivec,1:maxmultblk,1:nsmult,e),&
                    ndofddpes,n1,interp(n,e),gwpsymrep(1,n,e1))

               if (ldbsmall) then   ! add new point to all DBs
                   do e2=1,ngp      
                      dbnrec_gp(e2)=dbnrec_gp(e2)+1
                      call dddb_wrmem_gp(n1,e2)    
                   enddo
               endif

            enddo
         enddo
      endif

!-----------------------------------------------------------------------
!    write new data in database  
!    if not writing DB set value of DB size held in memory
!-----------------------------------------------------------------------
      if (lddwrdb) then
         do e=1,nex2
            e1=eex2(e)
            do n=1,nsymop
               if (lrddb(n,e1)) cycle

               call dddb_wr1(xgp(1:ndofddpes,n,e1),&
                    adv(1:nddstate,1:nddstate,1,e), &
                    adder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e), &
                    addip(1:3,1:nddstate,1:nddstate,n,e),&
                    ddv(1:nddstate,1:nddstate,n,e),&
                    ddder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    ddder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                    dddip(1:3,1:nddstate,1:nddstate,n,e),&
                    ddmoco(1:nbasis,1:nbasis,e),&
                    diabtrans(1:nddstate,1:nddstate,n,e),&
                    ddnact(1:ndofddpes,1:nactdim,n,e),&
                    ddcv(1:nddstate,1:nddstate,n,e),&
                    diabctrans(1:nddstate,1:nddstate,n,e),&
                    ciconfig(1:maxcivec,1:nsmult,e),&
                    cicoeff(1:maxcivec,1:maxmultblk,1:nsmult,e),&
                    ndofddpes,nddstate,interp(n,e),dercpdim,&
                    gwpsymrep(1,n,e1),&
                    adsoc(1:nsocstate,1:nsocstate,n,e),nsocstate)
               dbnrec=dbnrec+1
            enddo
         enddo
      else
         dbnrec=ngp
      endif

      call finalise_qchem_calculations(adder1,adder2,addip,adv, &
           ddder1,ddder2,dddip,ddmoco,ddnact,adsoc,ddv,diabtrans, &
           ddcv,diabctrans,interp,ciconfig,cicoeff)
      if (allocated(lskip)) deallocate(lskip)

!-----------------------------------------------------------------------
! END of QCHEM
! and
! END of surface update calculation
!-----------------------------------------------------------------------
900 continue
!-----------------------------------------------------------------------
! as DB has been updated save time (only once last state reached)
!-----------------------------------------------------------------------
      if (s1 .eq. nstate) dduptim0 = time

      if (allocated(eex2)) deallocate(eex2)
      if (allocated(xgp)) deallocate(xgp)
      if (allocated(gwpdist)) deallocate(gwpdist)
      if (allocated(allcent)) deallocate(allcent)
      if (allocated(r1)) deallocate(r1)
      if (allocated(idbloc)) deallocate(idbloc)
      if (allocated(lrddb)) deallocate(lrddb)
      if (allocated(locpt0)) deallocate(locpt0)
      if (allocated(dbnrec0)) deallocate(dbnrec0)
      if (allocated(gwpsymrep)) deallocate(gwpsymrep)

      return
      end subroutine getddpes

!-----------------------------------------------------------------------
! Get the ADT matrix for methods that do not generate it
!-----------------------------------------------------------------------

      subroutine getadt(vmat,transmat,nst)

      implicit none

      integer(long), intent(in)                  :: nst
      integer(long)                              :: info
      real(dop), dimension(nst,nst), intent(in)  :: vmat
      real(dop), dimension(nst,nst), intent(out) :: transmat
      real(dop), dimension(nst,nst)              :: transtmp
      real(dop), dimension(nst)                  :: tmpe
      real(dop), dimension(3*nst)                :: work

      transtmp(:,:) = vmat(:,:)
      call dsyev('V','U',nst,transtmp,nst,tmpe,work,3*nst,info)
      transmat(:,:) = transtmp(:,:)

      return
      end subroutine getadt

!#######################################################################
!
! Add SOC to the diabatic PES
!
!#######################################################################

      subroutine addsoc(qcsoc,v,cv,trans)

      implicit none

      real(dop), dimension(nddstate,nddstate)                  :: tmpsoc
      real(dop), dimension(nddstate,nddstate)                  :: diabsoc
      real(dop), dimension(nddstate,nddstate), intent(in)      :: trans
      real(dop), dimension(nddstate,nddstate), intent(inout)   :: v
      real(dop), dimension(nddstate,nddstate), intent(inout)   :: cv
      complex(dop), dimension(nsocstate,nsocstate), intent(in) :: qcsoc
      complex(dop), dimension(nsocstate,nsocstate)             :: cdiabsoc

!-----------------------------------------------------------------------
! If degenerate states, first get average coupling then transform ad -> di
! If not, transform ad -> di (real and imag separately)
!-----------------------------------------------------------------------
      if (lsocdegen) then
         call socdegen(qcsoc,tmpsoc)

         call simtranbd(tmpsoc,trans,diabsoc,nddstate)
         v(:,:) = v(:,:) + diabsoc(:,:)

      else
         call simtranbdz(qcsoc,trans,cdiabsoc,nddstate)

         v(:,:) = v(:,:) + dble(cdiabsoc(:,:))
         cv(:,:) = cv(:,:) + dimag(cdiabsoc(:,:))

      endif

      return
      end subroutine addsoc

!#######################################################################
! SOCDEGEN
!
! Combine SOCs between individual states into magnitudes between
! degenerate states.
! qcsoc(s,s1) : SOCs from QC in complete set of states
! soc(s,s1)   : effective SOCs between degenerate multiplets
! nstdeg: total number of states with degenerate multiplets
! nst: total number of states
! nsmult: number of spin blocks
! imultblk(sp): no. of degenerate multiplets in spin block sp
! imultoff(sp): index offset for start of degenerate spin block sp
! sttoff(sp): index offset for start of spin block sp
! idxmult(sp): multiplicity of spin block sp
!
!#######################################################################

      subroutine socdegen(qcsoc,soc)

      implicit none

      integer(long)     :: i,j,k,sp,sp1,s,s1,t,t1
      real(dop)         :: dsoc
      real(dop), dimension(nddstate,nddstate), intent(out)     :: soc
      complex(dop), dimension(nsocstate,nsocstate), intent(in) :: qcsoc

      soc = 0.0_dop
      do sp = 1,nsmult-1
         s = stoff(sp)     ! spin block offset in full set of states
         do i = 1,imultblk(sp)
            s = s+1
            t = s
            do sp1 = sp+1,nsmult
               s1 = stoff(sp1)    ! spin block offset in full set of states
               t1 = s1
               do j = 1,imultblk(sp1)
                  s1 = s1+1
                  dsoc = 0.0_dop
                  do k = 1,idxmult(sp1)
                     t1 = t1+1
                     dsoc = dsoc + dble(qcsoc(t,t1))*dble(qcsoc(t,t1))
                     dsoc = dsoc + dimag(qcsoc(t,t1))*dimag(qcsoc(t,t1))
                  enddo
                  soc(s,s1) = dsqrt(dsoc)
                  soc(s1,s) = soc(s,s1)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine socdegen

!#######################################################################
! Note: end of what needs to be improved.
!#######################################################################
#ifdef MPI
      subroutine mpigetddpes(time,totdim1,s1,mpinex1,mpinex2,hl,ngp,nsymop, &
         eex2,idbloc,mxmpi,xgp,r1,&
         ddv,adv,diabtrans,ddnact,ddder1,adder1,ddder2,adder2,dddip,addip,&
         adsoc,ddmoco,interp)

      implicit none
      !variable specific to MPI process
      integer(long), intent(in)                       :: mpinex1,mpinex2
      integer(long), intent(in)                       :: mxmpi
      !variable for getddpes transfer to mpigetddpes
      integer(long), intent(in)                       :: totdim1
      integer(long), intent(in)                       :: s1
      real(dop), intent(in)                           :: time
      integer(long), intent(in)                       :: hl,ngp,nsymop
      integer(long), dimension(:), intent(in)         :: eex2
      integer(long), dimension(:,:), intent(in)       :: idbloc !Need checking
      real(dop), dimension(ndofddpes,nsymop,totdim1),intent(in) :: xgp
      real(dop), dimension((dbnrec+totdim1),nsymop,ngp), intent(in) :: r1     
      !local variable for mpigetddpes
      integer(long)                                   :: e,e1,nex1,nex2,&
                                                         order,nrec,n
      integer(long)                                   :: nactdim  
      integer(long)                                   :: iwr
      integer(long), dimension(:,:), allocatable      :: hopiparp
      logical(kind=4)                                 :: lfail
      logical(kind=4), dimension(nsymop,ngp)          :: lrddb
      logical(kind=4), allocatable, dimension(:)      :: lskip 
      !output of mpigetddpes
      real(dop), dimension(:,:,:,:), intent(out)      :: ddv,adv
      real(dop), dimension(:,:,:,:), intent(out)      :: diabtrans,ddnact
      complex(dop), dimension(:,:,:,:), intent(out)   :: adsoc 
      real(sip), dimension(:,:,:), intent(out)        :: ddmoco
      real(dop), dimension(:,:,:,:,:), intent(out)    :: ddder1,adder1
      real(dop), dimension(:,:,:,:,:,:), intent(out)  :: ddder2,adder2
      real(dop), dimension(:,:,:,:,:), intent(out)    :: dddip,addip
      integer(long), dimension(:,:), intent(out)      :: interp

      nactdim=nddstate*(nddstate-1)/2
      nex2=mpinex2
      nex1=mpinex1
      !Safety measure for ensuring that extra mpi process do not run mpigetddpes
      if (mpirank.lt.mxmpi) then
         allocate(lskip(nex2))

         lskip = .false.
         interp = 0
         lrddb = .false.

!$omp parallel if(lompqc) num_threads(min(ompthread,nex2)) private(hopiparp,e,e1,lfail)
! array to carry hopipar to funcqchem in parallel loop
         allocate(hopiparp(maxhpar,nex2))
         do e=nex1,nex2
            hopiparp(:,e)=hopipar(:,hl)
         enddo
         !$omp do schedule(dynamic) private(e,e1,lfail)

         do e=nex1,nex2
            hopiparp(:,e)=hopipar(:,hl)
            e1=eex2(e)
            if (ltraj) then
               hopiparp(3,e)=ddtrajnum
            else
               hopiparp(3,e)=e1
            endif
            hopiparp(4,e)=s1
            hopiparp(6,e)=lhaorder
            if (lupdhes .and. (dbnrec .ne. 0 .or. e.gt.1)) hopiparp(6,e)=1
         
!-----------------------------------------------------------------------
! Get guess MO coefficients from the database if using CAS.
!-----------------------------------------------------------------------
            if (dbnrec.gt.0.and.idbloc(1,e1).ne.0) then
               if (hopipar(7,hl).eq.12 .or. hopipar(7,hl).eq.13 .or. &
                    hopipar(7,hl).eq.32) then
!$omp critical 
                  if (ldbsave) then
                     ddmoco(:,:,e) = dbmoco(:,:,idbloc(1,e1))
                  else
                     call ddmo_rd(ddmoco(1:nbasis,1:nbasis,e),nbasis,idbloc(1,e1))
                  endif
!$omp end critical 
               endif
            endif
!-----------------------------------------------------------------------
! call interface to external quantum chemistry program
!-----------------------------------------------------------------------
            if (ddlog) write(ilog,'(a,i5)') 'Generating DB Point for GWP : ',e1
            if(.not.lpestest)then
               call lhaqchem(time,hopilab(hl),xgp(1:ndofddpes,1,e1),&
                    adv(1:nddstate,1:nddstate,1,e),&
                    adder1(1:ndofddpes,1:nddstate,1:nddstate,1,e),& 
                    adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,1,e),&
                    dddip(1:3,1:nddstate,1:nddstate,1,e),&
                    ddmoco(1:nbasis,1:nbasis,e),&
                    ddnact(1:ndofddpes,1:nactdim,1,e),&
                    adsoc(1:nsocstate,1:nsocstate,1,e),&
                    ndofddpes,hopiparp(1,e),lfail,&
                    ciconfig(:,:,n),cicoeff(:,:,:,n)
               lskip(1)=.false.
               if (lfail) then
                  if (lddwrdb .and. dbnrec.eq.0 .and. e .eq. 1) then
                     routine='Getddpes'
                     message='Initial QC calculation failed: No reference point'
                     call errormsg
                  else
                     routine='Getddpes'
                     message='QC calculation failed'
                     call warnmsg
                  endif
! if global diabatisation need to interpolate, otherwise ignore point and continue
                  if (dddiab .eq. 4) then
                     lskip(e)=.true.
                  else
                     cycle
                  endif
               endif
            else
               call ddvmcgpestest(idpestest,xgp(1:ndofddpes,1:nsymop,e1),&
                  adv(1:nddstate,1:nddstate,1,e),&
                  adder1(1:ndofddpes,1:nddstate,1:nddstate,1:nsymop,e),&
                  adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,1:nsymop,e),&
                  ndofddpes,nddstate,hopiparp(6,e))
            endif
!-----------------------------------------------------------------------
! form image data for adiabatic terms
!-----------------------------------------------------------------------
            if (nsymop .gt. 1) then
               iwr = 1
               call mksympesa1(adder1(1:ndofddpes,1:nddstate,1:nddstate,1:nsymop,e),&
                    ddnact(1:ndofddpes,1:nactdim,1:nsymop,e),&
                    addip(1:3,1:nddstate,1:nddstate,1:nsymop,e),&
                    adsoc(1:nsocstate,1:nsocstate,1:nsymop,e),&
                    lrddb(:,e1),nsymop,dbpntgp,iwr)
            endif

            if (dddiab .eq. 0) then
               do n=1,nsymop
                  ddv(:,:,n,e)=adv(:,:,1,e)
                  ddder1(:,:,:,n,e)=adder1(:,:,:,n,e)
                  if (.not. lupdhes) ddder2(:,:,:,:,n,e)=adder2(:,:,:,:,n,e)
                  call unitqxd(diabtrans(1:nddstate,1:nddstate,n,e),nddstate)
               enddo
            else if (dddiab .eq. 1) then
               order = 2
               do n=1,nsymop
                  call diabat1(xgp(1:ndofddpes,n,e1),&
                       adv(1:nddstate,1:nddstate,1,e),&
                       adder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       ddv(1:nddstate,1:nddstate,n,e),&
                       ddder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       ddder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       order)
                  call getadt(ddv(1:nddstate,1:nddstate,n,e),&
                       diabtrans(1:nddstate,1:nddstate,n,e),nddstate) 
               enddo
            else if (dddiab .eq. 2) then
               order = 2
               do n=1,nsymop
                  call diabat2(xgp(1:ndofddpes,n,e1),&
                       adv(1:nddstate,1:nddstate,1,e),&
                       adder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       ddv(1:nddstate,1:nddstate,n,e),&
                       ddder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       ddder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       order,e,e1,time)
                  call getadt(ddv(1:nddstate,1:nddstate,n,e),&
                       diabtrans(1:nddstate,1:nddstate,n,e),nddstate) 
               enddo
            else if (dddiab .eq. 4) then
               order = 2
! set up pointer for new record number
               nrec = dbnrec
               do n=1,nsymop
                  if (lrddb(n,e1)) cycle

                  nrec = nrec+1
                  call diabat4(xgp(1:ndofddpes,n,e1),&
                       adv(1:nddstate,1:nddstate,1,e),&
                       adder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       ddnact(1:ndofddpes,1:nactdim,n,e),&
                       ddv(1:nddstate,1:nddstate,n,e),&
                       ddder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       ddder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       diabtrans(1:nddstate,1:nddstate,n,e),&
                       lskip(e),idbloc(n,e1),nrec,interp(n,e))
! should transform addip to dip
                  dddip(:,:,:,n,e)=addip(:,:,:,n,e)
               enddo

            else if (dddiab .eq. 6) then
               order = 2
! set up pointer for new record number
               nrec = dbnrec
               do n=1,nsymop
                  if (lrddb(n,e1)) cycle

                  nrec = nrec+1
                  call diabat6(xgp(1:ndofddpes,n,e1),&
                       adv(1:nddstate,1:nddstate,1,e),&
                       adder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       ddnact(1:ndofddpes,1:nactdim,n,e),&
                       ddv(1:nddstate,1:nddstate,n,e),&
                       ddder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       ddder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                       diabtrans(1:nddstate,1:nddstate,n,e),&
                       ddmoco(1:nbasis,1:nbasis,e),&
                       cicondfig(1:maxcivec,1:nsmult,n), &
                       cicoeff(1:maxcivec,1:maxmultblk,1:nsmult,n), &
                       1,nrec,lskip(e)) ! idbloc0=1 i.e. diabatise wrt first strtucture
! should transform addip to dip
                  dddip(:,:,:,n,e)=addip(:,:,:,n,e)
               enddo
            else
               routine = 'Getddpes (directdyn.f90)'
               write(message,'(a,i5)') &
                  'Unknown Diabatisation procedure requested ',dddiab
               call errormsg
            endif

! might need to enforce symmetry of ddder1

            if (lupdhes) then
               do n=1,nsymop
                  if (lrddb(n,e1)) cycle
                  if (ldbsmall) then
                     call hessian_update1(xgp(1:ndofddpes,n,e1),&
                          ddder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                          ddder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                          r1(1:dbnrec,n,e1),dbnrec_gp(num_gp))
                  else
                     call hessian_update1(xgp(1:ndofddpes,n,e1),&
                          ddder1(1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                          ddder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
                          r1(1:dbnrec,n,e1),dbnrec)
                  endif
        !          call hestrans(diabtrans(1:nddstate,1:nddstate,n,e),&
        !               adder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
        !               ddder2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,n,e),&
        !               ndofddpes,nddstate)
               enddo
            endif

! if CAS fails (interpolated point) save MOCO from nearest point rather than actual
            if (interp(1,e) .eq. -1.and.idbloc(1,e1).ne.0) then
               if (ldbsave) then
                  ddmoco(:,:,e) = dbmoco(:,:,idbloc(1,e1))
               else
                  call ddmo_rd(ddmoco(1:nbasis,1:nbasis,e),nbasis,idbloc(1,e1))
               endif
            endif

         enddo
!$omp enddo
         if (allocated(hopiparp)) deallocate(hopiparp)
!$omp end parallel
      endif

      if (allocated(lskip)) deallocate(lskip)

      return
      end subroutine mpigetddpes

      subroutine calc_gatherv_recvcounts_offset_displacements(mpinex1, &
           mpinex2, size_factor, recvcounts, offset, displacements)
        ! Calculate the values of recvcounts, offset and displacements
        ! required for an MPI_gatherv communication
        integer(long), dimension(nmpi), intent(in) :: mpinex1, mpinex2
        integer(long), dimension(nmpi), intent(out) :: recvcounts, &
             displacements
        integer(long), intent(in) :: size_factor
        integer, intent(out):: offset

        ! recvcounts contains the number of array elements expected
        ! from each worker processes
        recvcounts = (mpinex2 - mpinex1 + 1) * size_factor
        ! Offset contains the first memory location in which data
        ! should be written. This is the location after the qchem
        ! calculation produced by master.
        offset = recvcounts(1) + 1
        ! no data is actually transferred from master as this is
        ! already in the correct place
        recvcounts(1) = 0

        ! MPI_Gatherv does not allow send and receive buffers to be
        ! aliased. All displacements all given w.r.t to the array
        ! index given by `offset` above.
        displacements = (mpinex1 * size_factor) - 1
        displacements = displacements - displacements(2)
        displacements(1) = 0

      end subroutine calc_gatherv_recvcounts_offset_displacements

      subroutine qchem_gatherv_master_dop(buffer,size_factor,mpinex1, &
           mpinex2,comm)
        ! Wrapper function for MPI_Gatherv for double precision arrays.
        ! Note use of sequence association for `buffer` dummy argument to
        ! allow arrays of any rank.
        real(dop), dimension(*), intent(inout) :: buffer
        integer(long), intent(in) :: size_factor
        integer(long), dimension(nmpi), intent(in) :: mpinex1, mpinex2
        integer(long), dimension(nmpi) :: recvcounts,displacements
        integer, intent(in) :: comm
        integer :: offset

        call calc_gatherv_recvcounts_offset_displacements(mpinex1, &
             mpinex2, size_factor, recvcounts, offset, &
             displacements)
        call MPI_Gatherv(buffer, 0, MPI_DOUBLE_PRECISION,&
             buffer(offset), recvcounts, displacements,&
             MPI_DOUBLE_PRECISION, master, comm, mpierr)
      end subroutine qchem_gatherv_master_dop

      subroutine qchem_gatherv_master_cdop(buffer,size_factor,mpinex1, &
           mpinex2,comm)
        ! Wrapper function for MPI_Gatherv for double precision complex arrays.
        ! Note use of sequence association for `buffer` dummy argument to
        ! allow arrays of any rank.
        complex(dop), dimension(*), intent(inout) :: buffer
        integer(long), intent(in) :: size_factor
        integer(long), dimension(nmpi), intent(in) :: mpinex1, mpinex2
        integer(long), dimension(nmpi) :: recvcounts,displacements
        integer, intent(in) :: comm
        integer :: offset

        call calc_gatherv_recvcounts_offset_displacements(mpinex1, &
             mpinex2, size_factor, recvcounts, offset, &
             displacements)
        call MPI_Gatherv(buffer, 0, MPI_DOUBLE_COMPLEX,&
             buffer(offset), recvcounts, displacements,&
             MPI_DOUBLE_COMPLEX, master, comm, mpierr)
      end subroutine qchem_gatherv_master_cdop

      subroutine qchem_gatherv_master_int(buffer,size_factor,mpinex1&
           &,mpinex2,comm)
        ! Wrapper function for MPI_Gatherv for integer arrays.
        ! Note use of sequence association for `buffer` dummy argument to
        ! allow arrays of any rank.
        integer(long), dimension(*), intent(inout) :: buffer
        integer(long), intent(in) :: size_factor
        integer(long), dimension(nmpi), intent(in) :: mpinex1, mpinex2
        integer(long), dimension(nmpi) :: recvcounts,displacements
        integer, intent(in) :: comm
        integer :: offset

        call calc_gatherv_recvcounts_offset_displacements(mpinex1, &
             mpinex2, size_factor, recvcounts, offset, &
             displacements)
        call MPI_Gatherv(buffer, 0, MPI_INTEGER, buffer(offset), &
             recvcounts, displacements, MPI_INTEGER, master, comm, &
             mpierr)
      end subroutine qchem_gatherv_master_int

#endif

      subroutine init_qchem_calculations(ncalcs,nsymop,adder1,adder2 &
           &,addip,adv,ddder1,ddder2,dddip,ddmoco,ddnact,adsoc,ddv &
           &,diabtrans,ddcv,diabctrans,ciconfig,cicoeff,interp,addip_size &
           &,adv_size,ddder1_size,ddder2_size,dddip_size,ddnact_size &
           &,ddv_size,diabtrans_size,adsoc_size,interp_size,ddmoco_size &
           &,ciconfig_size,cicoeff_size)
        ! Allocate and initialise memory that will be used to store
        ! the results from qchem calculations. Also calculate the
        ! "size factors" for arrays that will be communicated via MPI
        ! later. The "size factors" denote the number of contiguous
        ! array elements that store data for a single calculation.

        integer(long), intent(in) :: ncalcs,nsymop
        integer(long)             :: nactdim1
        integer(long), intent(out) :: ddv_size,ddder1_size &
             &,ddder2_size,adv_size,dddip_size,addip_size &
             &,diabtrans_size,ddnact_size,adsoc_size,interp_size &
             &,ddmoco_size,ciconfig_size,cicoeff_size
        real(dop), intent(inout), allocatable, dimension(:,:,:,:) ::&
             & ddv,adv,diabtrans,ddnact,ddcv,diabctrans
        real(dop), intent(inout), allocatable, dimension(:,:,:,:,:)&
             & :: ddder1,adder1,dddip,addip
        real(dop), intent(inout), allocatable, dimension(:,:,:,:,:,:)&
             &  :: ddder2,adder2
        character(len=30), intent(inout), allocatable, dimension(:,:,:)&
             & :: ciconfig
        real(dop), intent(inout), allocatable, dimension(:,:,:,:)&
             &  :: cicoeff
        complex(dop), intent(inout), allocatable, dimension(:,:,:,:)&
             &  :: adsoc
        integer(long), intent(inout), dimension(:,:), allocatable ::&
             & interp
        real(sip), intent(inout), allocatable, dimension(:,:,:) :: ddmoco

! If single-state calculation, nactdim = 0. Allocate with size 1.
        nactdim1 = 1
        nactdim1 = max(nactdim,nactdim1)

        allocate(ddv(nddstate,nddstate,nsymop,ncalcs),&
             ddcv(nddstate,nddstate,nsymop,ncalcs),&
             ddder1(ndofddpes,nddstate,nddstate,nsymop,ncalcs),&
             ddder2(ndofddpes,ndofddpes,nddstate,nddstate,nsymop,ncalcs),&
             adv(nddstate,nddstate,nsymop,ncalcs),&
             adder1(ndofddpes,nddstate,nddstate,nsymop,ncalcs),&
             adder2(ndofddpes,ndofddpes,nddstate,nddstate,nsymop,ncalcs),&
             dddip(3,nddstate,nddstate,nsymop,ncalcs),&
             ddmoco(nbasis,nbasis,ncalcs),&
             addip(3,nddstate,nddstate,nsymop,ncalcs),&
             diabtrans(nddstate,nddstate,nsymop,ncalcs),&
             diabctrans(nddstate,nddstate,nsymop,ncalcs),&
             ddnact(ndofddpes,nactdim1,nsymop,ncalcs),&
             adsoc(nsocstate,nsocstate,nsymop,ncalcs),&
             ciconfig(maxcivec,nsmult,ncalcs),&
             cicoeff(maxcivec,maxmultblk,nsmult,ncalcs),&
             interp(nsymop,ncalcs))
        ddv=0
        ddcv=0
        ddder1=0
        ddder2=0
        adv=0
        adder1=0
        adder2=0
        dddip=0
        addip=0
        diabtrans = 0.0_dop
        diabctrans = 0.0_dop
        ddnact = 0.0_dop
        adsoc = dcmplx(0.0_dop,0.0_dop)
        ddmoco = 0.0_sip
        interp = 0

        ddv_size = nddstate * nddstate * nsymop
        ddder1_size = ndofddpes * nddstate * nddstate * nsymop
        ddder2_size = ndofddpes * ndofddpes * nddstate * nddstate * nsymop
        adv_size = nddstate * nddstate * nsymop
        dddip_size = 3 * nddstate * nddstate * nsymop
        addip_size = 3 * nddstate * nddstate * nsymop
        diabtrans_size = nddstate * nddstate * nsymop
        ddnact_size = ndofddpes * nactdim1 * nsymop
        adsoc_size = nsocstate * nsocstate * nsymop
        ddmoco_size = nbasis * nbasis 
        interp_size = nsymop
        ciconfig_size = maxcivec*nsmult
        cicoeff_size = maxcivec*nsmult*maxmultblk

      end subroutine init_qchem_calculations

      subroutine finalise_qchem_calculations(adder1,adder2,addip,adv &
           &,ddder1,ddder2,dddip,ddmoco,ddnact,adsoc,ddv,diabtrans &
           &,ddcv,diabctrans,interp,ciconfig,cicoeff)

        ! Deallocate all arrays used to store qchem calculation
        ! results
        real(dop), intent(inout), allocatable, dimension(:,:,:,:) ::&
             & adv,ddnact,ddv,diabtrans,ddcv,diabctrans
        real(dop), intent(inout), allocatable, dimension(:,:,:,:,:)&
             & :: adder1,addip,ddder1,dddip
        real(dop), intent(inout), allocatable, dimension(:,:,:,:,:,:)&
             &  :: adder2, ddder2
        real(sip), intent(inout), allocatable, dimension(:,:,:) :: ddmoco
        complex(dop), intent(inout), allocatable, dimension(:,:,:,:)     &
             &  :: adsoc 
        integer(long), intent(inout), dimension(:,:), allocatable ::&
             & interp
        character(len=30), intent(inout), allocatable, dimension(:,:,:)&
             & :: ciconfig
        real(dop), intent(inout), allocatable, dimension(:,:,:,:)&
             &  :: cicoeff

        if (allocated(adder1)) deallocate(adder1)
        if (allocated(adder2)) deallocate(adder2)
        if (allocated(addip)) deallocate(addip)
        if (allocated(adv)) deallocate(adv)
        if (allocated(ddder1)) deallocate(ddder1)
        if (allocated(ddder2)) deallocate(ddder2)
        if (allocated(dddip)) deallocate(dddip)
        if (allocated(ddmoco)) deallocate(ddmoco)
        if (allocated(ddnact)) deallocate(ddnact)
        if (allocated(adsoc)) deallocate(adsoc)
        if (allocated(ddv)) deallocate(ddv)
        if (allocated(ddcv)) deallocate(ddcv)
        if (allocated(diabtrans)) deallocate(diabtrans)
        if (allocated(diabctrans)) deallocate(diabctrans)
        if (allocated(interp)) deallocate(interp)
        if (allocated(ciconfig)) deallocate(ciconfig)
        if (allocated(cicoeff)) deallocate(cicoeff)

        ! Deallocate memory used for qchem calculations
      end subroutine finalise_qchem_calculations

!#######################################################################
! Generate initial Hessians
!
! NB Hessian is diabatic and taken to be same as adiabatic. Needs
! correction for NACTs.
!#######################################################################

      subroutine genhess0(deriv2,nact,av)

      implicit none

      integer(long)                               :: ilbl,ifiletyp,info,s,s1,&
                                                     f,f1,idx,iau
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), &
                                                         intent(out) :: deriv2
      real(dop), dimension(ndofddpes,nactdim), intent(in)            :: nact
      real(dop), dimension(ndofddpes,nddstate,nddstate)              :: tmpnact
      real(dop), dimension(nddstate,nddstate), intent(in)            :: av
      real(dop), dimension(maxdim,maxdim)         :: tmp
      real(dop), dimension(maxdim)                :: freq0
      real(dop)                                   :: dtmp
      character(len=c5)                           :: filename
      character(len=16), dimension(maxdim) :: nmlab

      if (.not. lddtrans) then
          routine = 'Genhess0 (directdyn.F90)'
          message = &
             'Can only generate initial Hessian from Freq file if using normal modes '
          call errormsg
      endif

      if (ddtrfile .eq. ' ') then
          routine = 'Genhess0 (directdyn.F90)'
          message = &
             'Cannot generate initial Hessians - GS Freq file not defined '
          call errormsg
      else
         ilbl = index(ddtrfile,' ')-1
         filename = ddname(1:ddlaenge)//'/'//ddtrfile(1:ilbl)
         write(ilog,'(a) ') ' '
         write(ilog,'(a) ') 'Taking initial Hessians for all states from: '
         ilbl = index(filename,' ')-1
         write(ilog,'(a) ') filename(1:ilbl)
      endif

      open(idump,file=filename,status='old',err=999)

      write(ilog,'(a)') 'Initial Hessians in DB generated from :'
      ilbl=index(filename,' ')-1
      write(ilog,'(a)') filename(1:ilbl)

      call getintyp(idump,ifiletyp,filename,ilog)

! read frequencies
      info=0
      freq0 = 0.0_dop
      iau = 0
      call getnm(idump,ifiletyp,nmlab,tmp,freq0,iatsym,iau,&
           maxdim,ndofddpes,nddstate,ndofdd,ilog,1,.true.)

! transform frequencies to form Hessian in Cartesians
      tmp=0.0_dop
      deriv2 = 0.0_dop
      call dmxxdd(freq0,ddtransb(1:ndofdd,1:ndofddpes),tmp(1:ndofdd,1:ndofddpes),&
           ndofdd,ndofddpes)
      call mmtxdd(ddtransb(1:ndofdd,1:ndofddpes),tmp(1:ndofdd,1:ndofddpes),&
           deriv2(1,1,1,1),ndofddpes,ndofdd,ndofddpes)

      do s=2,nddstate
         deriv2(:,:,s,s)=deriv2(:,:,1,1)
      enddo

! correct on-diagonal Hessian for NACT
      do s=1,nddstate
         do s1=s+1,nddstate
            idx = (s1-1)*(s1-2)/2+s
            tmpnact(:,s,s1) = nact(:,idx)
            tmpnact(:,s1,s) = nact(:,idx)
         enddo
      enddo

      do s=1,nddstate
         do s1=1,nddstate
            if (s .eq. s1) cycle
            do f = 1,ndofddpes
               do f1 = 1,ndofddpes
                  dtmp = 2.0*tmpnact(f,s,s1)*tmpnact(f1,s1,s)/ &
                              (av(s1,s1) - av(s,s))
                  deriv2(f,f1,s,s) = &
                          deriv2(f,f1,s,s) - dtmp
               enddo
            enddo
         enddo
      enddo

      return
  999 routine='Genhess0 (directdyn.F90)'
      ilbl=index(filename,' ')-1
      write(message,'(2a)') 'Error opening file: ',filename(1:ilbl)
      call errormsg
      end subroutine genhess0

!#######################################################################
! Routine to calculate the uncertainty in the interpolated PES by 
! comparing the interpolated PES to the individual 2nd-order expansions.
! See Eq. 24 in Collins, TCA, 2002, 108, 313-324.
! GWR 08/15
!#######################################################################
      subroutine dbuncertain(xgp,lrddb,nst)

      implicit none

      integer(long) ,intent(in)                              :: nst
      logical(kind=4), intent(inout)                         :: lrddb
      real(dop), dimension(ndofddpes), intent(in)            :: xgp

      integer(long)                                          :: irec,i,j

      real(dop)                                              :: norm
      real(dop), dimension(dbnrec)                           :: weight
      real(dop), dimension(ndofddpes)                        :: dx
      real(dop), dimension(nddstate,nddstate)                :: vshep,sigsq
      real(dop), dimension(nddstate,nddstate,dbnrec)         :: vi
      real(dop), dimension(ndofddpes,nddstate,nddstate)      :: dv1dum
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate) :: dv2dum
      real(dop), dimension(3,nddstate,nddstate)              :: dipdum,dipdum1

      real(dop)                                        :: dnrm2 !BLAS norm func

      sigsq = 0.0_dop

! Get the Shepard interpolated potential at the point in question.
!      call dddb_rd(vshep,dv1,dv2dum,dipdum,xgp,.false.,.true.,false.)

      vshep = 0.0_dop
      norm = 0.0_dop

      do irec = 1,dbnrec
! Get the Taylor series value of the potential at the point in question,
! expanded around each DB point.
         call shiftdd(vi(1,1,irec),dv1dum,dv2dum,dipdum,xgp,&
              dbener(1:nddstate,1:nddstate,irec),&
              dbgrad(1:ndofddpes,1:nddstate,1:nddstate,irec),&
              dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,irec),&
              dipdum1,dbgeo(1:ndofddpes,irec),2)

! Get the weight of the DB point.
         dx = xgp - dbgeo(1:ndofddpes,irec)
         weight(irec) = dnrm2(ndofddpes,dx,1)
         weight(irec) = 1.0_dop/weight(irec)**(dbinterord) 
         norm = norm + weight(irec)
         ! If you change the exponent, change it in dddb_rd as well.

         vshep(1:nddstate,1:nddstate) = vshep(1:nddstate,1:nddstate) +  &
              vi(1:nddstate,1:nddstate,irec)*weight(irec)

      enddo
      vshep = vshep/norm

! Calculate the energy difference between the full interpolation and the
! single 2nd-order expansion. Then square the individual numbers.
      do irec = 1, dbnrec
         vi(1:nddstate,1:nddstate,irec) = &
              vi(1:nddstate,1:nddstate,irec) - vshep(1:nddstate,1:nddstate)
         do i = 1,nddstate
            do j = 1,nddstate
               vi(i,j,irec) = vi(i,j,irec)*vi(i,j,irec)
            enddo
         enddo
         vi(1:nddstate,1:nddstate,irec) = &
              vi(1:nddstate,1:nddstate,irec) * weight(irec)

         sigsq(1:nddstate,1:nddstate) = &
              sigsq(1:nddstate,1:nddstate) + vi(1:nddstate,1:nddstate,irec)
      enddo

! Normalise the variance array.
      sigsq = sigsq/norm

! Decide whether to accept the interpolation. (Relies on the fact that the 
! PES matrix is symmetric).
! If using multi-set formulation, only checks for state in which these GWPS
! are found along with couplings to other states.
      do i = 1,nddstate
         if(.not.lmult.or.(i.eq.nst))then
            do j = i, nddstate
               if(sigsq(i,j).gt.dddbvar) lrddb = .false.
            enddo
         endif
      enddo

      return
      end subroutine dbuncertain

!#######################################################################
      subroutine dbharmonic(xgp,lrddb,nst,idbloc)

      implicit none

      integer(long) ,intent(in)                              :: nst,idbloc
      logical(kind=4), intent(inout)                         :: lrddb
      real(dop), dimension(ndofddpes), intent(in)            :: xgp

      integer(long)                                          :: i,j

      real(dop), dimension(nddstate,nddstate)                :: vshep,sigsq,vi
      real(dop), dimension(ndofddpes,nddstate,nddstate)      :: dv1dum
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate) :: dv2dum
      real(dop), dimension(3,nddstate,nddstate)              :: dipdum,dipdum1

! Get the Shepard interpolated potential at the point in question.
      call dddb_rd(vshep,dv1dum,dv2dum,dipdum,xgp,.false.,.true.,.false.)

! Get the Taylor series value of the potential at the point in question,
! expanded around each DB point.
      call shiftdd(vi,dv1dum,dv2dum,dipdum,xgp,&
           dbener(1:nddstate,1:nddstate,idbloc),&
           dbgrad(1:ndofddpes,1:nddstate,1:nddstate,idbloc),&
           dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,idbloc),&
           dipdum1,dbgeo(1:ndofddpes,idbloc),2)

! Calculate the energy difference between the full interpolation and the
! single 2nd-order expansion. Then take the absolute values of each.
      sigsq= vi - vshep    
      do i = 1,nddstate
         do j = 1,nddstate
            sigsq(i,j) = abs(sigsq(i,j))
         enddo
      enddo

! Decide whether to accept the interpolation. (Relies on the fact that the 
! PES matrix is symmetric).
! If using multi-set formulation, only checks for state in which these GWPS
! are found along with couplings to other states.
      do i = 1,nddstate
         if(.not.lmult.or.(i.eq.nst))then
            do j = i, nddstate
               if(sigsq(i,j).gt.dddbhar) lrddb = .false.
            enddo
         endif
      enddo

      return
      end subroutine dbharmonic

!########################################################################

      subroutine rdrefpnt(gpoint,xgpoint)

      implicit none

      integer(long)                                :: i,n,iz,ic,ierr
      real(dop), dimension(3,natm), intent(out)    :: xgpoint
      real(dop), dimension(ndofdd), intent(out)    :: gpoint

      logical(kind=4)                              :: lopen=.false.
      character(len=c5)                            :: filename
      character(len=c5)                            :: string
      character(len=c1)                            :: atname

      inquire(iin,opened=lopen)
      if (.not. lopen) then
         filename=inputfile(1:inplaenge)
         open(iin,file=filename,status='old',form='formatted',err=900)
      endif

      rewind(iin)
      iz=0
 430  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .gt. 0) then
         routine = 'Rdrefpnt (directdyn.f90)'
         message = inptit
         call errormsg
      endif

      if (keyword(1) .eq. 'diabref-section') then
         read(iin,'(a)') string
         if (string(1:6) .eq. 'nmodes') then
            i=1
            n=min(i+4,ndofdd)
            do 
               read(iin,*,err=900) gpoint(i:n)
               i=i+5
               if (i .gt. ndofdd) exit
               n=min(i+4,ndofdd)
            enddo
         else if (string(1:9) .eq. 'cartesian') then
            do n=1,natm
               read(iin,*,err=900) atname,(xgpoint(i,n), i=1,3)
            enddo
         endif
         go to 440

      else if (keyword(1) .eq. 'end-input') then
         go to 440
      endif
      go to 430
  440 continue

      close(iin)

      return
  900 continue
      routine = 'Rdrefpnt (directdyn.f90)'
      message='Error Reading Input File: '//inputfile(1:inplaenge)
      call errormsg

      return
      end subroutine rdrefpnt

!#######################################################################
!
! Get distances between two GWPs and (optionally) a set of symmetry
! replicas - returned as lower trianle of matrix
!
! if nsymop2 .eq. nsymop1 = 1 two GWPs compared
! if nsymop2 .eq. nsymop1 > 1 all pairs calculated.
! if nsymop2 .eq. 1 all functions compared to first.
!
! e1,e2 functions to be compared (nsymop1,e1) v (nsymop2,e2)
! output in gwpdist(nsymop1,nsymop2)
! 
!#######################################################################

      subroutine distgwp(xgpoint,gwpdist,nsymop1,nsymop2,ngp1,e1,e2)

      implicit none

      integer(long), intent(in)              :: nsymop1,nsymop2,ngp1,e1,e2
      integer(long)                          :: ig,jg
      real(dop)                              :: rx1
      real(dop), dimension(ndofddpes)        :: distvectmp
      real(dop)                              :: dnrm2 !BLAS norm function
      real(dop), dimension(ndofddpes,nsymop1,ngp1), intent(in) :: xgpoint
      real(dop), dimension(nsymop1,nsymop1), intent(out)  :: gwpdist

      gwpdist=999.0_dop

!-----------------------------------------------------------------------  
! loop over replicas
!----------------------------------------------------------------------- 
      do ig=1,nsymop2
         gwpdist(ig,ig) = 0.0_dop
         do jg=ig+1,nsymop1
            distvectmp(:)=xgpoint(:,jg,e1)-xgpoint(:,ig,e2)

! calculate the distance using BLAS norm function
            rx1 = dnrm2(ndofddpes,distvectmp,1)
            gwpdist(jg,ig) = rx1
         enddo
      enddo
      do ig=1,nsymop2
         do jg=ig+1,nsymop1
            gwpdist(ig,jg) = gwpdist(jg,ig)  
         enddo
      enddo

      end subroutine distgwp

!-----------------------------------------------------------------------

      subroutine mksympesa1(adder1,ddnact,addip,adsoc,lrddb,nsymop,&
                 dbpntgp,iwr)

      implicit none

      integer(long), intent(in) :: dbpntgp,nsymop,iwr
      integer(long)             :: n,s,s1,natm
      real(dop), dimension(ndofddpes,nactdim,nsymop), &
                                                   intent(inout) :: ddnact
      real(dop), dimension(ndofddpes,nddstate,nddstate,nsymop), &
                                                   intent(inout) :: adder1
      real(dop), dimension(3,nddstate,nddstate,nsymop)           :: addip
      complex(dop), dimension(nsocstate,nsocstate,nsymop)        :: adsoc
      real(dop), dimension(ndofddpes)                            :: tmpvec
      logical(kind=4), dimension(nsymop)                         :: lrddb

      natm = ndofddpes/3
      do n=2,nsymop
         if (lrddb(n)) cycle

         if ((iwr .eq. 1) .and. ddlog) then
            write(ilog,'(a,i5)') ' Generating symmetry replica : ',n
         endif

         do s=1,nddstate
            do s1=1,nddstate
               call dosymop(adder1(1,s1,s,1),tmpvec,dbpntgp,n,natm,ilog)
               call dosymperm(tmpvec,adder1(1,s1,s,n),dbsymmap(1,n),natm)
            enddo
         enddo
         if (ladipdb) then
            do s=1,nddstate
               do s1=1,nddstate
                  call dosymop(addip(1,s1,s,1),tmpvec,dbpntgp,n,1,ilog)
               enddo
            enddo
         endif

         if (lnactdb) then
            do s=1,nactdim
               call dosymop(ddnact(1,s,1),tmpvec,dbpntgp,n,natm,ilog)
               call dosymperm(tmpvec,ddnact(1,s,n),dbsymmap(1,n),natm)
            enddo
         endif

! copy SOCs - no trafo needed
         if (lcasocdb) then
            adsoc(1:nsocstate,1:nsocstate,n) = adsoc(1:nsocstate,1:nsocstate,1)
         endif
      enddo

      if (.not. lupdhes) then
         routine = 'Mksympesa1'
         message = 'Symmetry requires Hessian updating '
         call errormsg
  !       do n=2,nsymop
  !          if (lrddb(n)) cycle
!
!            do s=1,nddstate
!               call dosymop2(adder2(1,1,s,s,1),adder2(1,1,s,s,n),&
!                    dbpntgp,n,natm,ilog)
!            enddo
!         enddo
      endif
    
      return
      end subroutine mksympesa1

!-----------------------------------------------------------------------
!
! substitute DB locations to match the symmetry replica being used
!
!-----------------------------------------------------------------------

      subroutine symdb_gp(locpt0,dbnrec0,nrec0,ngp)

      implicit none

      integer(long), intent(in)                        :: ngp,nrec0
      integer(long)                                    :: nrec1,nrec2
      integer(long), dimension(nrec0,ngp), intent(out) :: locpt0
      integer(long), dimension(nrec0*nsymop)           :: locpt1,locpt2
      integer(long), dimension(ngp), intent(out)       :: dbnrec0
      integer(long)          :: irec,e,jrec,nrep,iloc,iloc1
      logical(kind=4)        :: lfound
      type(nextptr), pointer :: current

!-----------------------------------------------------------------------
! Save initial local DBs
!-----------------------------------------------------------------------
      do e=1,ngp
         call list_init(Current)
         current => ngp_loc(e)%locpt
         dbnrec0(e) = dbnrec_gp(e)
         do irec = 1,dbnrec_gp(e)
            locpt0(irec,e) = current%locDB
            current => current%Next
         enddo
      enddo

      do e=1,ngp
         nrec1=dbnrec_gp(e)
         locpt1(1:nrec1) = locpt0(1:nrec1,e)

! first check only primary representations are in list
         do irec = 1,nrec1
            iloc = locpt0(irec,e)

            iloc = abs(dbsymrep(1,iloc))  ! primary structure for replcas

            locpt1(irec) = iloc
         enddo

! next remove repeats
         nrec2=1
         locpt2(1) = locpt1(1)
         do irec = 2,nrec1
            lfound = .false.
            do jrec = 1,irec-1
               if (locpt1(irec) .eq. locpt1(jrec)) then
                  lfound=.true.
                  exit
               endif
            enddo
            if (.not. lfound) then
               nrec2 = nrec2+1
               locpt2(nrec2) = locpt1(irec)
            endif
         enddo

! Now add symmetry replicas
         nrec1=nrec2
         do irec=1,nrec1
            iloc = locpt2(irec)
            do nrep=2,nsymop
               iloc1 = dbsymrep(nrep,iloc)
               if (iloc1 .eq. 0) cycle   ! symrep not defined
               lfound = .false.
               do jrec=1,nrep-1   ! check if a repeat
                  if (iloc1 .eq. dbsymrep(jrec,iloc)) then
                     lfound=.true.
                     exit
                  endif
               enddo
               if (.not. lfound) then
                  nrec2 = nrec2+1
                  locpt2(nrec2) = iloc1
               endif
            enddo
         enddo

! Finally set up local DB
         call list_init(ngp_loc(e)%locpt)
         do irec=1,nrec2
            call list_insert(locpt2(irec),ngp_loc(e)%locpt)
         enddo
         dbnrec_gp(e) = nrec2

      enddo

      return
      end subroutine symdb_gp

!-----------------------------------------------------------------------
! Restore original (unsymmetrised) local DBs
!-----------------------------------------------------------------------

      subroutine symdb_gp0(locpt0,dbnrec0,nrec0,ngp)

      implicit none

      integer(long), intent(in)                       :: ngp,nrec0
      integer(long), dimension(nrec0,ngp), intent(in) :: locpt0
      integer(long), dimension(ngp), intent(in)       :: dbnrec0
      integer(long)          :: irec,e

      do e=1,ngp
         call list_init(ngp_loc(e)%locpt)

         do irec=1,dbnrec0(e)
           call list_insert(locpt0(irec,e),ngp_loc(e)%locpt)
         enddo

         dbnrec_gp(e) = dbnrec0(e)
      enddo

      return
      end subroutine symdb_gp0

      end module directdyn

