!***********************************************************************
! 
!             MCTDH subroutine EININGEOMOD
!
! contains einingeom
!
!  01/98 AJ
!***********************************************************************

      module einingeomod

use decimal, only: long, dop
use logdat

use global
use maxv
use keyunitsmod
use dvrdatmod
use dirdyn
use psidef
use channels
use runinwfmod
use griddatmod
use operdef

use iodvrdef, only:dvrdat
use gwplib, only: gwp_distin
use readqc, only: getmass,getatnam, getinam
use dd_db, only: ddx2q
use gapmod, only: lddmopt,lgapusesym

      implicit none

      private
      public :: einingeo,einddmct
      contains
  
!***********************************************************************
! For direct dynamics an init_geom-section is alternative input. 
! This needs to be read by different parts of the program hence
! the need for a multi-use read module driven by modus argument.
!-----------------------------------------------------------------------
!*****************************************************************************
!
! For direct dynamics, molecule can be specified as a set of
! atoms with cartesian coords for initial geometry
!
! modus: 1 Primitive basis set up (DVR)
!        2 Mass is read (operator)
!        3 Initial WF set up
!
! amass is partial COM
! 
!
! linperr is a flag set to true if error found in input. May be true or false
! on entry.
!
! iz counts the read line in the input file
!*****************************************************************************

      subroutine einingeo(linperr,iz,modus)

      implicit none

      integer(long), intent(inout)         :: iz
      integer(long), intent(in)            :: modus
      logical(kind=4), intent(inout)       :: linperr

! for trajectory calculations ltraj will be set in run-section. If it isn't
! assume GWPs
      if (ltraj) then
         call einingeotr(linperr,iz,modus)
      else
         call einingeogwp(linperr,iz,modus)
      endif

      end subroutine einingeo

!#######################################################################

      subroutine einingeogwp(linperr,iz,modus)

      implicit none

      integer(long), intent(inout)         :: iz
      integer(long), intent(in)            :: modus
      integer(long)                        :: i,ic,iz0,ierr,j,f,n,nf,nat,&
                                              icg,p
      integer(long)                        :: iatnam1,iatiso1,iau
      real(dop)                            :: tmass,xunit
      real(dop), dimension(maxdim/3)       :: amass
      logical(kind=4), intent(inout)       :: linperr
      logical(kind=4), dimension(2)        :: found

      character(len=20)                    :: cunit
!
! local variables
!
      integer(long)  :: ndof1,natm1,nsta1
      logical(kind=4), dimension(maxdim)   :: lclassical1,lfreeze1,lgrid1
      logical(kind=4), dimension(maxdim)   :: lclassical2,lfreeze2,lgrid2
      logical(kind=4)                      :: lcartini 
      character(c1), dimension(maxdim)     :: xatnam1,nmnam1

      real(dop), dimension(maxdim)         :: xmass1

      integer(long)                        :: typ,instate,lenlab,np,nq,gwpp,&
                                              foff,noq0
      integer(long), dimension(maxdim)     :: typ1
      real(dop)                            :: scwidth,gwpwidth_all
      real(dop), dimension(maxdim)         :: gxeq,hofreq,gshift
      real(dop), dimension(maxdim)         :: rwidth1,cwidth1,gmom1,gwpwidth1
      real(dop), dimension(maxdim)         :: rwidth2,cwidth2,gmom2,gwpwidth2
      real(dop), dimension(3,maxdim)       :: xingeo1

      real(dop), dimension(npar)           :: rpar1
      character(len=c1), dimension(npar)   :: apar1
      character(len=30)                    :: string

! check that coordinate type has been specified
      if (ddcoo .lt. 0) then
         routine = 'Einingeo (einingeomod.f90)'
         message = &
           'Specify coordinate type for direct dynamics using "direct" keyword'
         call errormsg
      endif

!-----------------------------------------------------------------------
! set default values
!-----------------------------------------------------------------------
      if (modus .gt. 1) then
         if (nmode .eq. 1 .or. (nmode .eq. 2 .and. leb)) then
            lvmcg = .true.
            psitype = 4
         endif
      endif

      if (modus .eq. 3) then
         typ = 3 !default for basis function distribution
         gwpp = 1 !gwpp toggles coord / mom distribution. Mom is default.

         instate = 1 ! by default on ground-state

! if initial geometry read, WF must be built
         lbuildwf = .true.
      endif
      np=0
      nq=0

! can set the width of GWPs for all modes
      gwpwidth_all = -999.0

!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
      routine = 'read INITIAL-GEOMETRY-SECTION (einingeom)'
      iz0 = 0
      found(:)=.false.
      nsta1=1

!-----------------------------------------------------------------------
! READ INITIAL GEOMETRY
!
! basis(f) : (0,25)->(el,GWP)
!
! ddcoo = 0  Jacobi Cartesians
!         1  Mass-frequency scaled normal modes
!         2  Cartesians
!        -1  User provided
! auxiliary grid for analysis set up in ddgwp_inp. Defined in:
! gdim(f)  : number of grid points for dof f.
! rpbaspar(1,f) xi      start of grid
! rpbaspar(2,f) xf      end of grid
!
! other parameters, specific to basis type, are read and stored in
! NB check that dimensions of these arrays (in main.inc and default.f) 
! are large enough for the number of arguments.
!-----------------------------------------------------------------------
! set default values / zero temporary variables to be read
!-----------------------------------------------------------------------
      f=0
      natm1=0
      ndof1=0
      lfreeze1(:) = .false.
      lclassical1(:) = .false.
      lgrid1(:) = .false.
      lfreeze2(:) = .false.
      lclassical2(:) = .false.
      lgrid2(:) = .false.
      xatnam1(:) = ' '
      nmnam1(:) = ' '
      xmass1(:) = 0.0_dop
      cunit='au'
      xunit=1.0_dop
      typ1(:) = 0
      lcartini = .false.

! defaults for momentum and width for Cartesians
! Note this is 1/rwidth used in genphi
      gmom1(:) = 0.0_dop
      rwidth1(:) = 0.1_dop
      cwidth1(:) =0.0_dop
      gwpwidth1(:)=-999.0_dop

! defaults for momentum and width for Nmodes
      gxeq(:) = 0.0_dop
      hofreq(:)=0.0_dop
      gmom2(:) = 0.0_dop
      rwidth2(:) = 1.0d0/dsqrt(2.0_dop)
      cwidth2(:) =0.0_dop
      gwpwidth2(:)=-999.0_dop

! default shift. 
! For Nmodes use an overlap = 0.7 (matches default in eb_gwp for vMCG)
! For Cartesians use a shift of 0.5
      if (ddcoo .eq. 1) then
         typ1(:) = 0
         gshift(:) = 0.7_dop
      else
         typ1(:) = 1
         gshift(:) = 0.5_dop
      endif

      read_initial_geometry_section: do
        call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
        message = inptit
        call cntrl(keyword(1),lc(1),ierr)

!NOTE: if no 'end-initial-geometry-section' or no 'end-input'
!this will be an infinite loop
        if (keyword(1) .eq. 'end-initial-geometry-section') then
          exit

        else if (keyword(1) .eq. 'end-input') then
          message = 'No end-initial-geometry-section '
          call warnmsg
          linperr=lwarn.or.linperr
          lwarn=.false.
          exit

        endif

        i = 0
        initial_geometry_section : do 
          i=i+1
! exit if all keywords read
          if (i .gt. ic) exit

!-----------------------------------------------------------------------
! Electronic Basis
!-----------------------------------------------------------------------
          if (keyword(i) .eq.'nstates') then
            read(keyword(i+2),*,iostat=ierr) nsta1
            if (ierr .eq. 1) then
              message = &
               'Incorrect init_geom definition for electronic states :'
              call wrline(zeroi,iz,iz0)
            endif
            i=i+2

!-----------------------------------------------------------------------
! Read Cartesian (xyz) input
!
! last keyword controls type of gwp - e.g. frozen 
!-----------------------------------------------------------------------

          else if (keyword(1)(1:9) .eq. 'cartesian') then
            found(1) = .true.
            if (keyword(2) .eq. '=') cunit = keyword(3)(1:20)
! test unit keyword
            xunit = 1.0_dop
            call keylunits(xunit,cunit,ierr)
            if (ierr .eq. 1) then
              message = 'Length unit given not recognized'
              call wrline(zeroi,iz,iz0)
            endif

            read_cartesians : do
              call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
              message = inptit
              call cntrl(keyword(1),lc(1),ierr)

              if (keyword(1)(1:9) .eq. 'end-input') then
                 routine = 'Einingeom (einpbas.F90)'
                 message = 'Missing end-cartesian'
                 call errormsg
              endif

              if (keyword(1)(1:13) .eq. 'end-cartesian') &
                        cycle read_initial_geometry_section

              natm1 = natm1+1

! get atomic names and coords
              xatnam1(natm1) = keyorig(1)(1:c1)
              call getatnam(xatnam1(natm1),ilog)   ! change to symbols if needed
              read(keyword(2),*,iostat=ierr) xingeo1(1,natm1)
              read(keyword(3),*,iostat=ierr) xingeo1(2,natm1)
              read(keyword(4),*,iostat=ierr) xingeo1(3,natm1)
!
! read any further keywords. 
              j = 4
              read_cart_options : do 
                 j=j+1
                 if (j .gt. ic) exit
                 if (keyword(j) .eq. 'freeze') then
! test
                    foff=3*(natm1-1)
                    if(keyword(j+2).eq.'[')then
                       j=j+2
                       do f = 1,3
                          if(keyword(j+1).eq.'t')lfreeze1(foff+f)=.true.
                          if((keyword(j+2).ne.','.and.f.lt.3)&
                               .or. (keyword(j+2).ne.']'.and.f.eq.3))then
                             message='Error reading Cartesian WF freezing'
                             routine='einingeogwp'
                             call errormsg
                          endif
                          j=j+2
                       enddo
                    else
                       lfreeze1(foff+1)=.true.
                       lfreeze1(foff+2)=.true.
                       lfreeze1(foff+3)=.true.
                    endif
! test
!                    lfreeze1(natm1) = .true.
                 else if (keyword(j) .eq. 'classical') then
                    lclassical1(natm1) = .true.
                 else if (keyword(j) .eq. 'grid') then
                    lgrid1(natm1) = .true.
                    icg = j+1
                 else if (keyword(j) .eq. 'mom') then
! test
                    foff=3*(natm1-1)
                    if(keyword(j+2).eq.'[')then
                       j=j+2
                       do f = 1,3
                          read(keyword(j+1),*,iostat=ierr) gmom1(foff+f)
                          if((keyword(j+2).ne.','.and.f.lt.3)&
                               .or. (keyword(j+2).ne.']'.and.f.eq.3))then
                             message='Error reading Cartesian WF momenta'
                             routine='einingeogwp'
                             call errormsg
                          endif
                          j=j+2
                       enddo
                    else
                       read(keyword(j+2),*,iostat=ierr) gmom1(foff+1)
                       gmom1(foff+2)=gmom1(foff+1)
                       gmom1(foff+3)=gmom1(foff+1)
                       j=j+2
                    endif
! test
!                    read(keyword(j+2),*,iostat=ierr) gmom1(natm1)
!                    j=j+2
                 else if (keyword(j) .eq. 'width') then
! test
                    foff=3*(natm1-1)
                    if(keyword(j+2).eq.'[')then
                       j=j+2
                       do f = 1,3
                          read(keyword(j+1),*,iostat=ierr) rwidth1(foff+f)
                          if((keyword(j+2).ne.','.and.f.lt.3)&
                               .or. (keyword(j+2).ne.']'.and.f.eq.3))then
                             message='Error reading Cartesian WF widths'
                             routine='einingeogwp'
                             call errormsg
                          endif
                          j=j+2
                       enddo
                    else
                       read(keyword(j+2),*,iostat=ierr) rwidth1(foff+1)
                       rwidth1(foff+2)=rwidth1(foff+1)
                       rwidth1(foff+3)=rwidth1(foff+1)
                       j=j+2
                    endif
! test
!                    read(keyword(j+2),*,iostat=ierr) rwidth1(natm1)
!                    j=j+2
                 endif
              enddo read_cart_options
              

            end do read_cartesians

!-----------------------------------------------------------------------
! Read Nmode input. Freq comes after incoo so that unit can be added 
! Modelabel   incoo    freq
!-----------------------------------------------------------------------
          else if (keyword(1)(1:5) .eq. 'nmode') then
            found(2) = .true.

            read_nm : do
              call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
              message = inptit
              call cntrl(keyword(1),lc(1),ierr)

              if (keyword(1)(1:9) .eq. 'end-input') then
                 routine = 'Einingeom (einpbas.F90)'
                 message = 'Missing end-nmode'
                 call errormsg
              endif
              if (keyword(1)(1:9) .eq. 'end-nmode') &
                           cycle read_initial_geometry_section 

              ndof1 = ndof1+1
              nmnam1(ndof1) = keyorig(1)(1:c1)

              read(keyword(2),*,iostat=ierr) gxeq(ndof1)
              read(keyword(3),*) hofreq(ndof1)
              j=3
              if (keyword(4) .eq. ',') then
                call keyeunits(hofreq(ndof1),keyword(5),ierr)
                if (ierr .eq. 1) then
                  routine = 'einingeo (einpbas.F)'
                  message = 'no energy unit after comma'
                  call wrline(zeroi,iz,iz0)
                endif
                j=5
              endif
!
! read any keywords. 
!
              read_nm_options : do 
                j=j+1
                if (j .gt. ic) exit

                if (keyword(j) .eq. 'mom') then
                  read(keyword(j+2),*,iostat=ierr) gmom2(ndof1)
                  j=j+2
                else if (keyword(j) .eq. 'width') then
                  read(keyword(j+2),*,iostat=ierr) rwidth2(ndof1)
                  j=j+2
                else if (keyword(j) .eq. 'gwpwidth') then
                  read(keyword(j+2),*,iostat=ierr) gwpwidth2(ndof1)
                  j=j+2
                  if(keyword(j+1) .eq. ',') then
                    call keylunits(gwpwidth2(ndof1),keyword(j+2),ierr)
                    j=j+1
                  endif
                else if (keyword(j) .eq. 'gwpshift') then
                   read(keyword(j+2),*,iostat=ierr) gshift(ndof1)
                   j=j+2
                   if(keyword(j+1) .eq. ',') then
                      call keylunits(gshift(ndof1),keyword(j+2),ierr)
                      j=j+1
                   endif
                   typ1(ndof1) = 1
                else if (keyword(j) .eq. 'gwpovl') then
                   read(keyword(j+2),*,iostat=ierr) gshift(ndof1)
                   typ1(ndof1) = 0
                   j=j+2
                else if (keyword(j) .eq. 'freeze') then
                  lfreeze2(ndof1) = .true.
                else if (keyword(j) .eq. 'classical') then
                  lclassical2(ndof1) = .true.
                else if (keyword(j) .eq. 'grid') then
                  lgrid2(ndof1) = .true.
                  icg = j+1
                  read(keyword(j+1),*,iostat=ierr)gdim(ndof1)
                  j=j+1
                  read(keyword(j+1),*,iostat=ierr)rpbaspar(1,ndof1)
                  j=j+1
                  if(keyword(j+1) .eq. ',') then
                    call keylunits(rpbaspar(1,ndof1),keyword(j+2),ierr)
                    j=j+1
                  endif
                  read(keyword(j+1),*,iostat=ierr)rpbaspar(2,ndof1)
                  j=j+1
                  if(keyword(j+1) .eq. ',') then
                    call keylunits(rpbaspar(2,ndof1),keyword(j+2),ierr)
                    j=j+1
                  endif
                else 
                   write(message,'(2a)')  &
                        'Unknown keyword for Nmode definition: ',keyword(j)
                   call errormsg
                endif
              enddo read_nm_options

            end do read_nm
!-----------------------------------------------------------------------
! other keywords
!-----------------------------------------------------------------------
          else if (keyword(i) .eq. 'gwp_type') then
             if (keyword(i+1) .ne. '=') then
                message = &
                   'No arguments specified with "gwp_type" keyword'
                   call wrline(zeroi,iz,iz0)
             else
                if (keyword(i+2) .eq. 'thawed') then
                   gwptype = 0
                   lgwpsep = .false.
                else if (keyword(i+2)(1:5) .eq. 'separ') then
                   gwptype = 2
                   lgwpsep = .true.
                else if (keyword(i+2) .eq. 'frozen') then
                   gwptype = 3
                   lgwpsep = .true.
                endif
             endif
             i=i+2

          else if (keyword(i) .eq. 'coords') then
            message = &
              'No longer used. Use direct = coordtype in RUN-SECTION '
            call errormsg

          else if (keyword(i) .eq. 'init_state') then
            if (keyword(i+1) .ne. '=') then
                message = 'No arguments specified with "init_state" keyword'
                call wrline(zeroi,iz,iz0)
             else
                read(keyword(i+2),*,iostat=ierr) instate 
                if (ierr .ne. 0) then
                   message = 'Incorrect argument for keyword '//keyword(i)
                   call wrline(onei,iz,iz0)
                endif
              endif
              i=i+2

          else if (keyword(i) .eq. 'mom_distrib') then
            gwpp = 1

          else if (keyword(i) .eq. 'coo_distrib') then
            gwpp = 0

          else if (keyword(i) .eq. 'init_basis_dist') then
            if (keyword(i+1) .ne. '=') then
                message = &
                  'No arguments specified with "init_basis_dist" keyword'
                call wrline(zeroi,iz,iz0)
            endif
            i = i+2
            call gwp_distin(i,typ,np,nq,noq0)

          else if (keyword(i) .eq. 'init_use_cartesian') then
             lcartini=.true.

          else if (keyword(i) .eq. 'gwpwidth') then
            if (keyword(i+1) .ne. '=') then
                message = 'No arguments specified with "gwpwidth" keyword'
                call wrline(zeroi,iz,iz0)
             else
                read(keyword(i+2),*,iostat=ierr) gwpwidth_all
                if (ierr .ne. 0) then
                   message = 'Incorrect argument for keyword '//keyword(i)
                   call wrline(onei,iz,iz0)
                endif
             endif
             i=i+2

! catch all unknown keywords 
          else 
            message = 'Unknown keyword :'//keyword(i)
            call warnmsg

          endif

        end do initial_geometry_section
      end do read_initial_geometry_section

!-----------------------------------------------------------------------
! check all data present. 
! 1. For Jacobi Cartesians need only cartesian
! 2. For normal modes need either
!    (a) cartesians + frequencies
!    (b) nmodes
!-----------------------------------------------------------------------
      if (ddcoo .eq. 0 .or. ddcoo .eq. 2) then
         if (.not. found(1)) then
            message = 'Need cartesian coordinates'
            call warnmsg
         endif
      else if (ddcoo .eq. 1) then
! check all information present. Can have: 
! Cartesian + freq        Cartesian + nmodes   nmodes + atoms
         if (.not. found(2)) then
            message = 'Need Nmodes'
            call warnmsg
         endif
         if (found(2) .and. .not. found(1)) then
            message = 'Need nmodes AND atoms'
            call warnmsg
         endif

      endif

!-----------------------------------------------------------------------
! If widths of GWPs set globally
!-----------------------------------------------------------------------
      if (gwpwidth_all .gt. 0) then
         if (ddcoo .eq. 1) then
            do f=1,ndof
               gwpwidth2(f)=gwpwidth_all
            enddo
         else if (ddcoo .eq. 2) then
            do n=1,natm
!              gwpwidth1(n)=gwpwidth_all
            enddo
         endif
      endif

!-----------------------------------------------------------------------
! Now set up data
!-----------------------------------------------------------------------

      select case (modus)

     case (1) ! set up DVR

      if (ddcoo .eq. 0) then   ! Jacobi Cartesians

         ndof = natm1*3 - 3

         nf = 0
         do n=2,natm1
            nf=nf+1
            write(modelabel(nf),'(i5,a1,a5,a2)') &
                  n,'_',xatnam1(n)(1:5),'_x'
            call rmblnk(modelabel(nf))
            nf=nf+1
            write(modelabel(nf),'(i5,a1,a5,a2)') &
                  n,'_',xatnam1(n)(1:5),'_y'
            call rmblnk(modelabel(nf))
            nf=nf+1
            write(modelabel(nf),'(i5,a1,a5,a2)') &
                  n,'_',xatnam1(n)(1:5),'_z'
            call rmblnk(modelabel(nf))

            f=nf-3
            do i=1,3
               f = f+1
               if (lfreeze1(f)) then
                  basis(f) = 19
                  lpntdvr=.true.
                  gdim(f) = 1
!    store coordinate as "analysis grid"
                  rpbaspar(1,f) = xingeo1(i,n)*xunit
                  rpbaspar(2,f) = xingeo1(i,n)*xunit
               else
                  basis(f) = 25
                  gdim(f) = 101
                  rpbaspar(1,f)=-10.0
                  rpbaspar(2,f)=10.0
                  ldvr(f)  = .false.
               endif
               if (lclassical1(n)) ipbaspar(1,f) = 1
!               if (lgrid1(n)) call ddgwp_inp(iz,iz0,ic,f,icg)

            enddo
         enddo

      else if (ddcoo .eq. 1) then   ! Nmodes

         ndof = ndof1
         do f=1,ndof
            modelabel(f) = nmnam1(f)

            if (lfreeze2(f)) then
               basis(f) = 19
               lpntdvr=.true.
               gdim(f) = 1
! store coordinate as "analysis grid"
! NB Should not multiply gxeq by xunit as NM are unitless.
               rpbaspar(1,f) = gxeq(f)!*xunit
               rpbaspar(2,f) = gxeq(f)!*xunit
            else
               basis(f) = 25
               if(.not.lgrid2(f))then
                  gdim(f) = 101
                  rpbaspar(1,f)=-10.0
                  rpbaspar(2,f)=10.0
               endif
               ldvr(f)  = .false.

               lenlab = index(adjustl(modelabel(f)),' ')-1
            endif

            if (lclassical2(f)) ipbaspar(1,f) = 1
!            if (lgrid2(f)) call ddgwp_inp(iz,iz0,ic,f,icg)
         enddo
      else if (ddcoo .eq. 2) then   ! Cartesians

         ndof = natm1*3

         nf = 0
         do n=1,natm1
            nf=nf+1
            write(modelabel(nf),'(i5,a1,a5,a2)') &
                  n,'_',xatnam1(n)(1:5),'_x'
            call rmblnk(modelabel(nf))
            nf=nf+1
            write(modelabel(nf),'(i5,a1,a5,a2)') &
                  n,'_',xatnam1(n)(1:5),'_y'
            call rmblnk(modelabel(nf))
            nf=nf+1
            write(modelabel(nf),'(i5,a1,a5,a2)') &
                  n,'_',xatnam1(n)(1:5),'_z'
            call rmblnk(modelabel(nf))

            f=nf-3
            do i=1,3
               f = f+1
! test
               if (lfreeze1(f)) then
! test
!               if (lfreeze1(n)) then
                  basis(f) = 19
                  lpntdvr=.true.
                  gdim(f) = 1
!    store coordinate as "analysis grid"
                  rpbaspar(1,f) = xingeo1(i,n)*xunit
                  rpbaspar(2,f) = xingeo1(i,n)*xunit
               else
                  basis(f) = 25
                  gdim(f) = 101
                  rpbaspar(1,f)=-10.0
                  rpbaspar(2,f)=10.0
                  ldvr(f)  = .false.
               endif
               if (lclassical1(n)) ipbaspar(1,f) = 1
!               if (lgrid1(n)) call ddgwp_inp(iz,iz0,ic,f,icg)

            enddo

         enddo
      endif

!-----------------------------------------------------------------------
! add electronic basis
!-----------------------------------------------------------------------
      ndof=ndof+1
      modelabel(ndof) = 'el'
      feb=ndof
      basis(feb) = 0
      gdim(feb) = nsta1

!-----------------------------------------------------------------------
! calculate dvr related data 
!-----------------------------------------------------------------------
      call dvrdat
!
!
     case (2) ! Get atom info and masses for operator

      natm=natm1
      iau=0
      do n=1,natm
         xatnam(n) = xatnam1(n)
         call getinam(xatnam(n),iatnam1,iatiso1,ilog)
         call getmass(xmass(n),iatnam1,iau,ilog)

         xingeo(:,n) = xingeo1(:,n)*xunit
      enddo
      if (ddcoo .eq. 0) then   ! Jacobi Cartesians
! !!!! WILL NOT WORK WITH FEB > 0 !!!!
         tmass = xmass(1)
         do nat=2,natm
!
! calculate mass - tmass = sum mass  up to previous atom in list
!
            amass(nat) = xmass(nat)*tmass
            tmass = tmass + xmass(nat)
            amass(nat) = amass(nat) / tmass
         enddo
         f = 0
         do nat=2,natm
            do n=1,3
               f = f+1
               mass(f) = amass(nat)
            enddo
         enddo

      else if (ddcoo .eq. 1) then   ! Normal modes
         do f=1,ndof
            mass(f)=1.0_dop
         enddo

         ! allocate space in parameter arrays for frequencies
         do n=1,npar
            apar1(n)=apar(n)
            rpar1(n)=rpar(n)
         enddo
         if (allocated(apar)) deallocate(apar)
         if (allocated(rpar)) deallocate(rpar)
         allocate(apar(npar+ndof))
         allocate(rpar(npar+ndof))
         do n=1,npar
            apar(n)=apar1(n)
            rpar(n)=rpar1(n)
         enddo

         do f=1,ndof     ! add HO frequencies to parameter list
            if (f .eq. feb) cycle

            write(string,'(a6,i5)')  'freq0_',f
            call rmblnk(string)
            npar = npar+1
            apar(npar) = string(1:c1)
            rpar(npar) = abs(hofreq(f))
         enddo


      else if (ddcoo .eq. 2) then   ! Cartesians
         f=0
         do n=1,natm
            do i=1,3
               f=f+1
               mass(f)=xmass(n)
            enddo
         enddo

      endif

     case (3) ! get initial WF info

      if (ddcoo .eq. 0) then   ! Jacobi Cartesians
         message = 'Initial Jacobis not yet set up'
         call errormsg

      else if (ddcoo .eq. 1) then   ! Normal modes
! use cartesian initial geometry
         if (lcartini) then
            do n=1,natm
               do i=1,3
                  xingeo(i,n) = xingeo1(i,n)*xunit
               enddo
            enddo
            gxeq = 0.0_dop
            call ddx2q(xingeo,gxeq)
         endif
         do f=1,ndof
            sbasis(f) = 9
            scwidth=0.0d0
            if (gwpwidth2(f) .lt. 0.0) gwpwidth2(f)=rwidth2(f)

!.... Transform to the convention used in genphi1.f for both HO and Gauss.
            rwidth2(f) = 1.0d0/rwidth2(f)
            gwpwidth2(f)=1.0d0/gwpwidth2(f)
            p=1
            rsbaspar(1,f,p)=gxeq(f)
            rsbaspar(2,f,p)=gmom2(f)
            rsbaspar(3,f,p)=rwidth2(f)
            rsbaspar(4,f,p)=cwidth2(f)
            rsbaspar(5,f,p)=gshift(f)
            rsbaspar(6,f,p)=gwpwidth2(f)
            rsbaspar(7,f,p)=scwidth

            isbaspar(1,f,p)=typ
            isbaspar(2,f,p)=typ1(f)
            isbaspar(3,f,p)=gwpp
            isbaspar(4,f,p)=1
            isbaspar(5,f,p)=nq
            isbaspar(6,f,p)=np
! isbaspar(7)=1 if pergauss
            isbaspar(8,f,p)=noq0

            sbfound(f,p)=.true.
         enddo

      else if (ddcoo .eq. 2) then   ! Cartesians
! initial WF Cartesians of atoms
         do n=1,natm
            do i=1,3
               xingeo(i,n) = xingeo1(i,n)*xunit
            enddo
         enddo

         f=0
         do n=1,natm
!            if (gwpwidth1(n) .lt. 0.0) gwpwidth1(n)=rwidth1(n)
!!.... Transform to the convention used in genphi1.f for both HO and Gauss.
!            rwidth1(n) = 1.0d0/rwidth1(n)
!            gwpwidth1(n)=1.0d0/gwpwidth1(n)

            do i=1,3
               f=f+1             
! test
               if (gwpwidth1(f) .lt. 0.0) gwpwidth1(f)=rwidth1(f)

!.... Transform to the convention used in genphi1.f for both HO and Gauss.
               rwidth1(f) = 1.0d0/rwidth1(f)
               gwpwidth1(f)=1.0d0/gwpwidth1(f)
! test

               sbasis(f) = 9
               typ1(f)=0
               scwidth=0.0d0

               p=1

               rsbaspar(1,f,p) = xingeo(i,n)

               rsbaspar(2,f,p)=gmom1(f)
!               rsbaspar(3,f,p)=rwidth1(n)
               rsbaspar(3,f,p)=rwidth1(f)
               rsbaspar(4,f,p)=cwidth1(n)
               rsbaspar(5,f,p)=gshift(f)
!               rsbaspar(6,f,p)=gwpwidth1(n)
               rsbaspar(6,f,p)=gwpwidth1(f)
               rsbaspar(7,f,p)=scwidth

               isbaspar(1,f,p)=typ
               isbaspar(3,f,p)=gwpp
               isbaspar(2,f,p)=typ1(f)
               isbaspar(4,f,p)=1
               sbfound(f,p)=.true.

            enddo
         enddo
! The electronic degree of freedom needs to be found
         sbfound(f+1,1)=.true.
      endif

!-----------------------------------------------------------------------
! classical GWPs no longer defined in init_geom-section
!-----------------------------------------------------------------------
      if (any(lclassical1) .or. any(lclassical2)) then
         routine = 'Einingeogwp (einingeom.f90)'
         message(1:80) = &
           'Basis type Classical no longer used. To set classical GWPs use :'
         message(81:160) = &
           'gwp_model, gwp_model@m, gwpmodel@f keywords in integrator-section.'
         call errormsg
      endif

!-----------------------------------------------------------------------
! check and set up initially populated electronic state
!-----------------------------------------------------------------------
      if (instate .gt. nddstate) then
         message = 'Initial state specified is greater than no. of states '
         call errormsg
      endif
      if (feb.gt.0) then
         sbasis(feb) = 0
         isbaspar(4,feb,1) = instate
      endif

     end select

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr
      lwarn=.false.

      return
      end subroutine einingeogwp

!#######################################################################

      subroutine einingeotr(linperr,iz,modus)

      implicit none

      integer(long), intent(inout)         :: iz
      integer(long), intent(in)            :: modus
      integer(long)                        :: i,ic,iz0,ierr,j,f,n,nf,nat,&
                                              icg,p
      integer(long)                        :: iatnam1,iatiso1,iau
      real(dop)                            :: tmass,xunit
      real(dop), dimension(maxdim/3)       :: amass
      logical(kind=4), intent(inout)       :: linperr
      logical(kind=4), dimension(2)        :: found

      character(len=20)                    :: cunit
!
! local variables
!
      integer(long)  :: ndof1,natm1,nsta1
      logical(kind=4), dimension(maxdim)   :: lfreeze1,lgrid1
      logical(kind=4), dimension(maxdim)   :: lfreeze2,lgrid2
      logical(kind=4)                      :: lcartini 
      character(c1), dimension(maxdim)     :: xatnam1,nmnam1

      real(dop), dimension(maxdim)         :: xmass1

      integer(long)                        :: typ,instate,lenlab,np,nq
      integer(long), dimension(maxdim)     :: typ1
      real(dop)                            :: scwidth
      real(dop), dimension(maxdim)         :: gxeq,hofreq,gshift
      real(dop), dimension(maxdim)         :: rwidth1,cwidth1,gmom1,gwpwidth1
      real(dop), dimension(maxdim)         :: rwidth2,cwidth2,gmom2,gwpwidth2
      real(dop), dimension(3,maxdim)       :: xingeo1

      real(dop), dimension(npar)           :: rpar1
      character(len=c1), dimension(npar)   :: apar1
      character(len=30)                    :: string

! check that coordinate type has been specified
      if (ddcoo .lt. 0) then
         routine = 'Einingeo (einingeomod.f90)'
         message = &
           'Specify coordinate type for direct dynamics using "direct" keyword'
         call errormsg
      endif

!-----------------------------------------------------------------------
! set default values
!-----------------------------------------------------------------------
      if (modus .eq. 3) then
         psitype = 3 !trajectory wavefunction
         pqdistr = 2 !default symmetric Wigner

         instate = 1 ! by default on ground-state

! if initial geometry read, WF must be built
         lbuildwf = .true.
         typ = 0   
      endif
      np=0
      nq=0

!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
      routine = 'read INITIAL-GEOMETRY-SECTION (einingeomtr)'
      iz0 = 0
      found(:)=.false.
      nsta1=1

!-----------------------------------------------------------------------
! READ INITIAL GEOMETRY
!
! basis(f) : (0,27)->(el,classical)
!
! ddcoo = 0  Jacobi Cartesians
!         1  Mass-frequency scaled normal modes
!         2  Cartesians
!        -1  User provided
! auxiliary grid for analysis set up in ddgwp_inp. Defined in:
! gdim(f)  : number of grid points for dof f.
! rpbaspar(1,f) xi      start of grid
! rpbaspar(2,f) xf      end of grid
!
! other parameters, specific to basis type, are read and stored in
! NB check that dimensions of these arrays (in main.inc and default.f) 
! are large enough for the number of arguments.
!-----------------------------------------------------------------------
! set default values / zero temporary variables to be read
!-----------------------------------------------------------------------
      f=0
      natm1=0
      ndof1=0
      lfreeze1(:) = .false.
      lgrid1(:) = .false.
      lfreeze2(:) = .false.
      lgrid2(:) = .false.
      xatnam1(:) = ' '
      nmnam1(:) = ' '
      xmass1(:) = 0.0_dop
      cunit='au'
      xunit=1.0_dop
      typ1(:) = 0
      lcartini = .false.

! defaults for momentum and width for Cartesians
! Note this is 1/rwidth used in genphi
      gmom1(:) = 0.0_dop
      rwidth1(:) = 0.1_dop
      cwidth1(:) =0.0_dop

! defaults for momentum and width for Nmodes
      gxeq(:) = 0.0_dop
      hofreq(:)=0.0_dop
      gmom2(:) = 0.0_dop
      rwidth2(:) = 1.0d0/dsqrt(2.0_dop)
      cwidth2(:) =0.0_dop

      read_initial_geometry_section: do
        call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
        message = inptit
        call cntrl(keyword(1),lc(1),ierr)

!NOTE: if no 'end-initial-geometry-section' or no 'end-input'
!this will be an infinite loop
        if (keyword(1) .eq. 'end-initial-geometry-section') then
          exit

        else if (keyword(1) .eq. 'end-input') then
          message = 'No end-initial-geometry-section '
          call warnmsg
          linperr=lwarn.or.linperr
          lwarn=.false.
          exit

        endif

        i = 0
        initial_geometry_section : do 
          i=i+1
! exit if all keywords read
          if (i .gt. ic) exit

!-----------------------------------------------------------------------
! Electronic Basis
!-----------------------------------------------------------------------
          if (keyword(i) .eq.'nstates') then
            read(keyword(i+2),*,iostat=ierr) nsta1
            if (ierr .eq. 1) then
              message = &
               'Incorrect init_geom definition for electronic states :'
              call wrline(zeroi,iz,iz0)
            endif
            i=i+2

!-----------------------------------------------------------------------
! Read Cartesian (xyz) input
!-----------------------------------------------------------------------

          else if (keyword(1)(1:9) .eq. 'cartesian') then
            found(1) = .true.
            if (keyword(2) .eq. '=') cunit = keyword(3)(1:20)
! test unit keyword
                  xunit = 1.0_dop
                  call keylunits(xunit,cunit,ierr)
                  if (ierr .eq. 1) then
                     message = 'Length unit given not recognized'
                     call wrline(zeroi,iz,iz0)
                  endif

                  read_cartesians : do
                     call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,&
                          ierr,maxkey)
                     message = inptit
                     call cntrl(keyword(1),lc(1),ierr)

              if (keyword(1)(1:9) .eq. 'end-input') then
                 routine = 'Einingeom (einpbas.F90)'
                 message = 'Missing end-cartesian'
                 call errormsg
              endif

              if (keyword(1)(1:13) .eq. 'end-cartesian') &
                        cycle read_initial_geometry_section

              natm1 = natm1+1

! get atomic names and coords
              xatnam1(natm1) = keyorig(1)(1:c1)
              call getatnam(xatnam1(natm1),ilog)   ! change to symbols if needed
              read(keyword(2),*,iostat=ierr) xingeo1(1,natm1)
              read(keyword(3),*,iostat=ierr) xingeo1(2,natm1)
              read(keyword(4),*,iostat=ierr) xingeo1(3,natm1)
!
! read any further keywords. 
              j = 4
              read_cart_options : do 
                j=j+1
                if (j .gt. ic) exit
                if (keyword(j) .eq. 'freeze') then
                  lfreeze1(natm1) = .true.
                else if (keyword(j) .eq. 'grid') then
                  lgrid1(natm1) = .true.
                  icg = j+1
                else if (keyword(j) .eq. 'mom') then
                  read(keyword(j+2),*,iostat=ierr) gmom1(natm1)
                  j=j+2
                else if (keyword(j) .eq. 'width') then
                  read(keyword(j+2),*,iostat=ierr) rwidth1(natm1)
                  j=j+2
                endif
              enddo read_cart_options

            end do read_cartesians

!-----------------------------------------------------------------------
! Read Nmode input. Freq comes after incoo so that unit can be added 
! Modelabel   incoo    freq
!-----------------------------------------------------------------------
          else if (keyword(1)(1:5) .eq. 'nmode') then
            found(2) = .true.

            read_nm : do
              call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
              message = inptit
              call cntrl(keyword(1),lc(1),ierr)

              if (keyword(1)(1:9) .eq. 'end-input') then
                 routine = 'Einingeom (einpbas.F90)'
                 message = 'Missing end-nmode'
                 call errormsg
              endif
              if (keyword(1)(1:9) .eq. 'end-nmode') &
                           cycle read_initial_geometry_section 

              ndof1 = ndof1+1
              nmnam1(ndof1) = keyorig(1)(1:c1)

              read(keyword(2),*,iostat=ierr) gxeq(ndof1)
              read(keyword(3),*) hofreq(ndof1)
              j=3
              if (keyword(4) .eq. ',') then
                call keyeunits(hofreq(ndof1),keyword(5),ierr)
                if (ierr .eq. 1) then
                  routine = 'einingeo (einpbas.F)'
                  message = 'no energy unit after comma'
                  call wrline(zeroi,iz,iz0)
                endif
                j=5
              endif
!
! read any keywords. 
!
              read_nm_options : do 
                j=j+1
                if (j .gt. ic) exit

                if (keyword(j) .eq. 'mom') then
                  read(keyword(j+2),*,iostat=ierr) gmom2(ndof1)
                  j=j+2
                else if (keyword(j) .eq. 'width') then
                  read(keyword(j+2),*,iostat=ierr) rwidth2(ndof1)
                  j=j+2
                else if (keyword(j) .eq. 'freeze') then
                  lfreeze2(ndof1) = .true.
                else if (keyword(j) .eq. 'grid') then
                  lgrid2(ndof1) = .true.
                  icg = j+1
                  read(keyword(j+1),*,iostat=ierr)gdim(ndof1)
                  j=j+1
                  read(keyword(j+1),*,iostat=ierr)rpbaspar(1,ndof1)
                  j=j+1
                  if(keyword(j+1) .eq. ',') then
                    call keylunits(rpbaspar(1,ndof1),keyword(j+2),ierr)
                    j=j+1
                  endif
                  read(keyword(j+1),*,iostat=ierr)rpbaspar(2,ndof1)
                  j=j+1
                  if(keyword(j+1) .eq. ',') then
                    call keylunits(rpbaspar(2,ndof1),keyword(j+2),ierr)
                    j=j+1
                  endif
                else 
                   write(message,'(2a)')  &
                        'Unknown keyword for Nmode definition: ',keyword(j)
                   call errormsg
                endif
              enddo read_nm_options

            end do read_nm
!-----------------------------------------------------------------------
! other keywords
!-----------------------------------------------------------------------
          else if (keyword(i) .eq. 'coords') then
            message = &
              'No longer used. Use direct = coordtype in RUN-SECTION '
            call errormsg

          else if (keyword(i) .eq. 'init_state') then
           if (keyword(i+1) .ne. '=') then
              message = 'No arguments specified with "init_state" keyword'
              call wrline(zeroi,iz,iz0)
            else
              read(keyword(i+2),*,iostat=ierr) instate 
              if (ierr .ne. 0) then
                message = 'Incorrect argument for keyword '//keyword(i)
                call wrline(onei,iz,iz0)
              endif
            endif
            i=i+2

          else if (keyword(i) .eq. 'init_basis_dist') then
            if (keyword(i+1) .ne. '=') then
              message = &
               'No arguments specified with "init_basis_dist" keyword'
              call wrline(zeroi,iz,iz0)
            else if (keyword(i+2).eq.'wigner') then
              pqdistr = 1
            else if (keyword(i+2).eq.'wigner_sym') then
              pqdistr = 2
            else if (keyword(i+2).eq.'wigner_no') then
              pqdistr = 0

            else
              message='Unknown init_basis_dist keyword '//keyword(i+2)
              call warnmsg
            endif
            i=i+2

          else if (keyword(i) .eq. 'init_use_cartesian') then
             lcartini=.true.

! catch all unknown keywords 
          else 
            message = 'Unknown keyword :'//keyword(i)
            call warnmsg

          endif

        end do initial_geometry_section
      end do read_initial_geometry_section

!-----------------------------------------------------------------------
! check all data present. 
! 1. For Jacobi Cartesians need only cartesian
! 2. For normal modes need either
!    (a) cartesians + frequencies
!    (b) nmodes
!-----------------------------------------------------------------------
      if (ddcoo .eq. 0 .or. ddcoo .eq. 2) then
         if (.not. found(1)) then
            message = 'Need cartesian coordinates'
            call warnmsg
         endif
      else if (ddcoo .eq. 1) then
! check all information present. Can have: 
! Cartesian + freq        Cartesian + nmodes   
         if (.not. found(2)) then
            message = 'Need Nmodes'
            call warnmsg
         endif
         if (found(2) .and. .not. found(1)) then
            message = 'Need nmodes AND atoms'
            call warnmsg
         endif
            
      endif

      select case (modus)

     case (1) ! set up DVR

      if (ddcoo .eq. 0) then   ! Jacobi Cartesians

         ndof = natm1*3 - 3

         nf = 0
         do n=2,natm1
            nf=nf+1
            write(modelabel(nf),'(i5,a1,a5,a2)') &
                  n,'_',xatnam1(n)(1:5),'_x'
            call rmblnk(modelabel(nf))
            nf=nf+1
            write(modelabel(nf),'(i5,a1,a5,a2)') &
                  n,'_',xatnam1(n)(1:5),'_y'
            call rmblnk(modelabel(nf))
            nf=nf+1
            write(modelabel(nf),'(i5,a1,a5,a2)') &
                  n,'_',xatnam1(n)(1:5),'_z'
            call rmblnk(modelabel(nf))

            f=nf-3
            do i=1,3
               f = f+1
               if (lfreeze1(n)) then
                  basis(f) = 19
                  lpntdvr=.true.
                  gdim(f) = 1
!    store coordinate as "analysis grid"
                  rpbaspar(1,f) = xingeo1(i,n)*xunit
                  rpbaspar(2,f) = xingeo1(i,n)*xunit
               else
                  basis(f) = 27
                  gdim(f) = 101
                  rpbaspar(1,f)=-10.0
                  rpbaspar(2,f)=10.0
                  ldvr(f)  = .false.
               endif
               ipbaspar(1,f) = 1

            enddo
         enddo

      else if (ddcoo .eq. 1) then   ! Nmodes

         ndof = ndof1
         do f=1,ndof
            modelabel(f) = nmnam1(f)

            if (lfreeze2(f)) then
               basis(f) = 19
               lpntdvr=.true.
               gdim(f) = 1
! store coordinate as "analysis grid"
! NB Should not multiply gxeq by xunit as NM are unitless.
               rpbaspar(1,f) = gxeq(f)!*xunit
               rpbaspar(2,f) = gxeq(f)!*xunit
            else
               basis(f) = 27
               if(.not.lgrid2(f))then
                  gdim(f) = 101
                  rpbaspar(1,f)=-10.0
                  rpbaspar(2,f)=10.0
               endif
               ldvr(f)  = .false.

               lenlab = index(adjustl(modelabel(f)),' ')-1
            endif

!            if (lgrid2(f)) call ddgwp_inp(iz,iz0,ic,f,icg)
         enddo
      else if (ddcoo .eq. 2) then   ! Cartesians

         ndof = natm1*3

         nf = 0
         do n=1,natm1
            nf=nf+1
            write(modelabel(nf),'(i5,a1,a5,a2)') &
                  n,'_',xatnam1(n)(1:5),'_x'
            call rmblnk(modelabel(nf))
            nf=nf+1
            write(modelabel(nf),'(i5,a1,a5,a2)') &
                  n,'_',xatnam1(n)(1:5),'_y'
            call rmblnk(modelabel(nf))
            nf=nf+1
            write(modelabel(nf),'(i5,a1,a5,a2)') &
                  n,'_',xatnam1(n)(1:5),'_z'
            call rmblnk(modelabel(nf))

            f=nf-3
            do i=1,3
               f = f+1
               if (lfreeze1(n)) then
                  basis(f) = 19
                  lpntdvr=.true.
                  gdim(f) = 1
!    store coordinate as "analysis grid"
                  rpbaspar(1,f) = xingeo1(i,n)*xunit
                  rpbaspar(2,f) = xingeo1(i,n)*xunit
               else
                  basis(f) = 27
                  gdim(f) = 101
                  rpbaspar(1,f)=-10.0
                  rpbaspar(2,f)=10.0
                  ldvr(f)  = .false.
               endif
               ipbaspar(1,f) = 1

            enddo

         enddo
      endif

!-----------------------------------------------------------------------
! add electronic basis
!-----------------------------------------------------------------------
      ndof=ndof+1
      modelabel(ndof) = 'el'
      feb=ndof
      basis(feb) = 0
      gdim(feb) = nsta1

!-----------------------------------------------------------------------
! calculate dvr related data 
!-----------------------------------------------------------------------
      call dvrdat
!
!
     case (2) ! Get atom info and masses for operator

      natm=natm1
      iau=0
      do n=1,natm
         xatnam(n) = xatnam1(n)
         call getinam(xatnam(n),iatnam1,iatiso1,ilog)
         call getmass(xmass(n),iatnam1,iau,ilog)

         xingeo(:,n) = xingeo1(:,n)*xunit
      enddo
      if (ddcoo .eq. 0) then   ! Jacobi Cartesians
! !!!! WILL NOT WORK WITH FEB > 0 !!!!
            tmass = xmass(1)
            do nat=2,natm
!
! calculate mass - tmass = sum mass  up to previous atom in list
!
               amass(nat) = xmass(nat)*tmass
               tmass = tmass + xmass(nat)
               amass(nat) = amass(nat) / tmass
            enddo
            f = 0
            do nat=2,natm
               do n=1,3
                  f = f+1
                  mass(f) = amass(nat)
               enddo
            enddo

      else if (ddcoo .eq. 1) then   ! Normal modes
         do f=1,ndof
            mass(f)=1.0_dop
         enddo

         ! allocate space in parameter arrays for frequencies
         do n=1,npar
            apar1(n)=apar(n)
            rpar1(n)=rpar(n)
         enddo
         if (allocated(apar)) deallocate(apar)
         if (allocated(rpar)) deallocate(rpar)
         allocate(apar(npar+ndof))
         allocate(rpar(npar+ndof))
         do n=1,npar
            apar(n)=apar1(n)
            rpar(n)=rpar1(n)
         enddo

         do f=1,ndof     ! add HO frequencies to parameter list
            if (f .eq. feb) cycle

            write(string,'(a6,i5)')  'freq0_',f
            call rmblnk(string)
            npar = npar+1
            apar(npar) = string(1:c1)
            rpar(npar) = abs(hofreq(f))
         enddo

      else if (ddcoo .eq. 2) then   ! Cartesians
         f=0
         do n=1,natm
            do i=1,3
               f=f+1
               mass(f)=xmass(n)
            enddo
         enddo

         f=0
         do n=1,natm
            do i=1,3
               f=f+1
               ddxcoo0(f) = xingeo1(i,n)*xunit
            enddo
         enddo
      endif

     case (3) ! get initial WF info

      if (ddcoo .eq. 0) then   ! Jacobi Cartesians
         message = 'Initial Jacobis not yet set up'
         call errormsg

      else if (ddcoo .eq. 1) then   ! Normal modes
! use cartesian initial geometry
         if (lcartini) then
            do n=1,natm
               do i=1,3
                  xingeo(i,n) = xingeo1(i,n)*xunit
               enddo
            enddo
            gxeq = 0.0_dop
            call ddx2q(xingeo,gxeq)
         endif
         do f=1,ndof
            sbasis(f) = 1
            scwidth=0.0d0

!.... Transform to the convention used in genphi1.f for both HO and Gauss.
            rwidth2(f) = 1.0d0/rwidth2(f)
            gwpwidth2(f)=1.0d0/gwpwidth2(f)
            p=1
            rsbaspar(1,f,p)=gxeq(f)
            rsbaspar(2,f,p)=gmom2(f)
            rsbaspar(3,f,p)=rwidth2(f)
            rsbaspar(4,f,p)=cwidth2(f)

! isbaspar not used in generating trajectory coordinates (clgauss)
            isbaspar(1,f,p)=typ
            isbaspar(2,f,p)=typ1(f)
            isbaspar(3,f,p)=0
            isbaspar(4,f,p)=1
            isbaspar(5,f,p)=nq
            isbaspar(6,f,p)=np

            sbfound(f,p)=.true.
         enddo

      else if (ddcoo .eq. 2) then   ! Cartesians
! initial WF Cartesians of atoms
         do n=1,natm
            do i=1,3
               xingeo(i,n) = xingeo1(i,n)*xunit
            enddo
         enddo

         f=0
         do n=1,natm
            if (gwpwidth1(n) .lt. 0.0) gwpwidth1(n)=rwidth1(n)

!.... Transform to the convention used in genphi1.f for both HO and Gauss.
            rwidth1(n) = 1.0d0/rwidth1(n)
            gwpwidth1(n)=1.0d0/gwpwidth1(n)

            do i=1,3
               f=f+1
               
               sbasis(f) = 1
               typ1(f)=0
               scwidth=0.0d0

               p=1

               rsbaspar(1,f,p) = xingeo(i,n)

               rsbaspar(2,f,p)=gmom1(n)
               rsbaspar(3,f,p)=rwidth1(n)
               rsbaspar(4,f,p)=cwidth1(n)
               rsbaspar(5,f,p)=gshift(f)
               rsbaspar(6,f,p)=gwpwidth1(n)
               rsbaspar(7,f,p)=scwidth

! isbaspar not used in generating trajectory coordinates (clgauss)
               isbaspar(1,f,p)=typ
               isbaspar(2,f,p)=typ1(f)
               isbaspar(4,f,p)=1
               sbfound(f,p)=.true.

            enddo
         enddo
! The electronic degree of freedom needs to be found
         sbfound(f+1,1)=.true.
      endif

!-----------------------------------------------------------------------
! check and set up initially populated electronic state
!-----------------------------------------------------------------------
      if (instate .gt. nddstate) then
         message = 'Initial state specified is greater than no. of states '
         call errormsg
      endif
      if (feb.gt.0) then
         sbasis(feb) = 0
         isbaspar(4,feb,1) = instate
      endif

     end select

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr
      lwarn=.false.

      return
      end subroutine einingeotr

!***********************************************************************
! For direct dynamics an init_geom-section is alternative input. 
! This needs to be read by different parts of the program hence
! the need for a multi-use read module driven by modus argument.
!-----------------------------------------------------------------------
!*****************************************************************************
!
! For direct dynamics, molecule can be specified as a set of
! atoms with cartesian coords for initial geometry
!
! modus: 1 Primitive basis set up (DVR)
!        2 Mass is read (operator)
!        3 Initial WF set up
!
! amass is partial COM
! 
!
! linperr is a flag set to true if error found in input. May be true or false
! on entry.
!
! iz counts the read line in the input file
!*****************************************************************************

      subroutine einddmct(linperr,iz,modus)

      implicit none

      integer(long), intent(inout)         :: iz
      integer(long), intent(in)            :: modus
      integer(long)                        :: i,ic,iz0,ierr,j,f,n,nat
      integer(long)                        :: iatnam1,iatiso1,iau
      real(dop)                            :: tmass,xunit
      real(dop), dimension(maxdim/3)       :: amass
      logical(kind=4), intent(inout)       :: linperr
      logical(kind=4), dimension(2)        :: found

      character(len=20)                    :: cunit
!
! local variables
!
      integer(long)  :: ndof1,natm1,nsta1
      character(c1), dimension(maxdim)     :: xatnam1,nmnam1

      real(dop), dimension(maxdim)         :: xmass1

      real(dop), dimension(maxdim)         :: hofreq
      real(dop), dimension(3,maxdim)       :: xingeo1

      routine = 'Einddmct (einingeomod.f90)'
! check that coordinate type has been specified
      if (ddcoo .lt. 0) then
           message='Specify coordinate type for DD using "dd-gb" keyword'
         call errormsg
      endif

      select case (modus)

      case(1)

         continue

      case(2)
!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
         iz0 = 0
         found(:)=.false.
         nsta1=1
         
!-----------------------------------------------------------------------
! READ INITIAL GEOMETRY
!
! basis(f) : (0,25)->(el,GWP)
!
! ddcoo = 0  Jacobi Cartesians
!         1  Mass-frequency scaled normal modes
!         2  Cartesians
!        -1  User provided
! auxiliary grid for analysis set up in ddgwp_inp. Defined in:
! gdim(f)  : number of grid points for dof f.
! rpbaspar(1,f) xi      start of grid
! rpbaspar(2,f) xf      end of grid
! rpbaspar(3,f) hofreq  Frequency defining normal mode (required
!                       if ddcoo=1)
!
! other parameters, specific to basis type, are read and stored in
! NB check that dimensions of these arrays (in main.inc and default.f) 
! are large enough for the number of arguments.
!-----------------------------------------------------------------------
! set default values / zero temporary variables to be read
!-----------------------------------------------------------------------
         f=0
         natm1=0
         ndof1=0
         xatnam1(:) = ' '
         nmnam1(:) = ' '
         xmass1(:) = 0.0_dop
         cunit='au'
         xunit=1.0_dop

! defaults for momentum and width for Nmodes
         hofreq(:)=0.0_dop

         read_dd_mctdh_section: do
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            message = inptit
            call cntrl(keyword(1),lc(1),ierr)
            
!NOTE: if no 'end-dd-gb-section' or no 'end-input'
!this will be an infinite loop
            if (keyword(1) .eq. 'end-dd-gb-section') then
               exit

            else if (keyword(1) .eq. 'end-input') then
               message = 'No end-dd-gb-section '
               call warnmsg
               linperr=lwarn.or.linperr
               lwarn=.false.
               exit

            endif

            i = 0
            dd_mctdh_section : do 
               i=i+1
! exit if all keywords read
               if (i .gt. ic) exit

!-----------------------------------------------------------------------
! Read Cartesian (xyz) input
!
! last keyword controls type of gwp - e.g. frozen 
!-----------------------------------------------------------------------

               if (keyword(1)(1:9) .eq. 'cartesian') then
                  found(1) = .true.
                  if (keyword(2) .eq. '=') cunit = keyword(3)(1:20)
! test unit keyword
                  xunit = 1.0_dop
                  call keylunits(xunit,cunit,ierr)
                  if (ierr .eq. 1) then
                     message = 'Length unit given not recognized'
                     call wrline(zeroi,iz,iz0)
                  endif

                  read_cartesians : do
                     call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,&
                          ierr,maxkey)
                     message = inptit
                     call cntrl(keyword(1),lc(1),ierr)

                     if (keyword(1)(1:9) .eq. 'end-input') then
                        message = 'Missing end-cartesian'
                        call errormsg
                     endif

                     if (keyword(1)(1:13) .eq. 'end-cartesian') &
                          cycle read_dd_mctdh_section

                     natm1 = natm1+1
! get atomic names and coords
                     xatnam1(natm1) = keyorig(1)(1:c1)
                     call getatnam(xatnam1(natm1),ilog)   ! change to symbols if needed
                     read(keyword(2),*,iostat=ierr) xingeo1(1,natm1)
                     read(keyword(3),*,iostat=ierr) xingeo1(2,natm1)
                     read(keyword(4),*,iostat=ierr) xingeo1(3,natm1)
                     
                  end do read_cartesians

!-----------------------------------------------------------------------
! Read Nmode input. Freq comes after incoo so that unit can be added 
! Modelabel   incoo    freq
!-----------------------------------------------------------------------
               else if (keyword(1)(1:5) .eq. 'nmode') then
                  found(2) = .true.

                  read_nm : do
                     call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,&
                          ierr,maxkey)
                     message = inptit
                     call cntrl(keyword(1),lc(1),ierr)

                     if (keyword(1)(1:9) .eq. 'end-input') then
                        message = 'Missing end-nmode'
                        call errormsg
                     endif
                     if (keyword(1)(1:9) .eq. 'end-nmode') &
                          cycle read_dd_mctdh_section 

                     ndof1 = ndof1+1
                     nmnam1(ndof1) = keyorig(1)(1:c1)

                     read(keyword(2),*) hofreq(ndof1)
                     j=2
                     if (keyword(j+1) .eq. ',') then
                        call keyeunits(hofreq(ndof1),keyword(j+2),ierr)
                        if (ierr .eq. 1) then
                           message = 'no energy unit after comma'
                           call wrline(zeroi,iz,iz0)
                        endif
                        j = j+2
                     endif

                  end do read_nm

! catch all unknown keywords 
               else 
                  message = 'Unknown keyword :'//keyword(i)
                  call warnmsg
                  
               endif
               
            end do dd_mctdh_section
         end do read_dd_mctdh_section

!-----------------------------------------------------------------------
! check all data present. 
! 1. For Jacobi Cartesians need only cartesian
! 2. For normal modes need either
!    (a) cartesians + frequencies
!    (b) nmodes
!-----------------------------------------------------------------------
         if (ddcoo .eq. 0 .or. ddcoo .eq. 2) then
            if (.not. found(1)) then
               message = 'Need cartesian coordinates'
               call warnmsg
            endif
         else if (ddcoo .eq. 1) then
! check all information present. Can have: 
! Cartesian + freq        Cartesian + nmodes   nmodes + atoms
            if (.not. found(2)) then
               message = 'Need Nmodes'
               call warnmsg
            endif
            if (found(2) .and. .not. found(1)) then
               message = 'Need nmodes AND atoms'
               call warnmsg
            endif
            
         endif

! Get atom info and masses for operator

         natm=natm1
         iau=0
         do n=1,natm
            xatnam(n) = xatnam1(n)
            call getinam(xatnam(n),iatnam1,iatiso1,ilog)
            call getmass(xmass(n),iatnam1,iau,ilog)

            xingeo(:,n) = xingeo1(:,n)*xunit
         enddo

         if (ddcoo .eq. 0) then   ! Jacobi Cartesians
! !!!! WILL NOT WORK WITH FEB > 0 !!!!
            tmass = xmass(1)
            do nat=2,natm
!
! calculate mass - tmass = sum mass  up to previous atom in list
!
               amass(nat) = xmass(nat)*tmass
               tmass = tmass + xmass(nat)
               amass(nat) = amass(nat) / tmass
            enddo
            f = 0
            do nat=2,natm
               do n=1,3
                  f = f+1
                  mass(f) = amass(nat)
               enddo
            enddo

         else if (ddcoo .eq. 1) then   ! Normal modes
            do f=1,ndof
               mass(f)=1.0_dop
               hoppar(1,f) = abs(hofreq(f))
            enddo

         else if (ddcoo .eq. 2) then   ! Cartesians
            f=0
            do n=1,natm
               do i=1,3
                  f=f+1
                  mass(f)=xmass(n)
               enddo
            enddo

         endif

         if(ddcoo.eq.0 .or. ddcoo .eq. 2)then
            f=0
            do n=1,natm
               do i=1,3
                  f=f+1
                  ddxcoo0(f) = xingeo1(i,n)*xunit
               enddo
            enddo
         endif

      end select

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr
      lwarn=.false.

      return
      end subroutine einddmct

!-----------------------------------------------------------------------
      end module einingeomod


      

