!***********************************************************************
!
!            Subroutine      EINGABE
!
! Reads the input data from the input file.
! Called from: mctdh
! Calls to: openin, openlog, rdinpf, einfile, einrun, einbas, eininwf,  
!           einint, loginp, ausgab, warnmsg, errormsg,
!           myflush
!
! 03/97 HDM 
! 
!***********************************************************************

      module einparamod

use decimal, only: long, dop
use global
use channels
use logdat
use keyunitsmod

      implicit none

      private
      public :: einpara,heinpara
      contains

!***********************************************************************

      subroutine einpara


      implicit None

      integer(long)                   :: iz,ic,ierr,l,n,n1,pass

      character(len=c5)               :: filename
! guess no. of pars for reading old oper files 
      integer(long)                   :: npar_pre11=10000
      integer(long)                   :: ilbl,jlbl
      integer(long)                   :: slen
      character(len=c2)               :: rdtag
      character(len=c5)               :: filpar
      integer(long), dimension(6)     :: source
      logical(kind=4)                 :: lopen,check
      integer(long)                                 :: npar1
      real(dop), dimension(:), allocatable          :: rpar1
      character(len=c1), dimension(:), allocatable  :: apar1


!-----------------------------------------------------------------------
! Parameters are read in a priority order (those read first take precedence
! if read again later). Different sources are possible depending on the runlevel
! the program is at. Priority order is as follows.
!
! Runlevel 1 (gendvr)
!    1. command line
!    3. input file - parameter-section
! Runlevel 2 (genoper)
!    1. command line
!    2. input file - alter-parameters-section
!    3. input file - parameter-section
!    4. .par file
!    5. .op file - parameter-section
! Runlevel 3 (geninwf)
!    1. command line
!    3. input file - parameter-section
!    6. oper file
! NB  input file and oper file need to be read as oper file may have 
!     been created in a separate run and the parameters in the input
!     file needed to set up a dvr / WF.
!
! for DD run using an internally generated operator, the mode frequencies 
! are added to the parameters in cvmcgop
!-----------------------------------------------------------------------

! zero flags
      source(:)=0     
      npar=0
      maxpar=0
      ilbl=0

      pass=1
!-----------------------------------------------------------------------
! internally defined parameters
!-----------------------------------------------------------------------
      npar=1

!-----------------------------------------------------------------------
! look for parameters given on command line (read in openin)
!-----------------------------------------------------------------------
      if (nparcmd .gt. 0) then
         source(1) = 1
         npar=npar+nparcmd
      endif

!-----------------------------------------------------------------------
! look to see if there is a parameter-sections in the input file
!-----------------------------------------------------------------------
      inquire(iin,opened=lopen)
      if (.not. lopen) then
         filename=inputfile(1:inplaenge)
         open(iin,file=filename,status='old',form='formatted',err=900)
      endif

      rewind(iin)
      iz=0
      do 
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .gt. 0) then
            routine = 'Einpara, search in iin for parameter-section'
            message = inptit
            call errormsg
         endif

         if (keyword(1).eq.'parameter-section') then
            source(3)=1
            call heinpara(iin,pass)       

         elseif (keyword(1).eq.'end-input') then
            exit
         endif
      enddo

!-----------------------------------------------------------------------
! generating oper file (reading from a .op file otherwise all parameters
! are in input file parameter-section
!-----------------------------------------------------------------------
      if ((runlevel .eq. 2) .and. (iop .ne. iin)) then
! read alter-parameters in .inp
         rewind(iin)
         iz=0
         do 
            iz=iz+1
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            if (ierr .gt. 0) then
               routine = 'Eingabe, search for alter-parameters'
               message = inptit
               call errormsg
            endif

            if (keyword(1).eq.'alter-parameters') then
               source(2)=1
               call heinpara(iin,pass)       
            elseif (keyword(1).eq.'end-input') then
               exit
            endif
         enddo

! for parameter file look first in . directory then oppath
         if (parlaenge .gt. 0) then
             check=.false.
             filpar=parfile
             l=parlaenge
             call abspath(filpar,l)
             inquire(file=filpar,exist=check)

             if (.not. check) then
                filpar=oppath(1:opplaenge)//'/'//parfile(1:parlaenge)
                inquire(file=filpar,exist=check)
                if (check) l=slen(filpar)
             endif

             if (.not. check) then
                routine='Heingabe'
                message='Parameter file not found: '//parfile(1:parlaenge) &
                      //' neither in current dir nor in: '&
                      //oppath(1:slen(oppath))
                call errormsg
             endif

!     store file name + path back in parfile variable
             parfile=filpar
             parlaenge=l
             open(unit=ipar,iostat=ierr,file=parfile(1:parlaenge), &
                     form='formatted',status='old')
             if (ierr.ne.0) then
                routine = 'Heingabe'
                message = 'Error opening parameter file : '//parfile(1:parlaenge)
                call errormsg
             endif

! read .par file
            rewind(ipar)
            iz=0
            do 
               call rdinpf(ipar,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
               if (ierr .gt. 0) then
                  routine = 'Eingabe, parameter file'
                  message = inptit
                  call errormsg
               endif

               if (keyword(1).eq.'parameter-section') then
                  source(4)=1
                  call heinpara(ipar,pass)       
                  exit
               elseif (keyword(1).eq.'end-parameter-section') then
                  routine = 'Eingabe, no parameter-section in par file'
                  message = inptit
                  call errormsg
               endif
            enddo
         endif

! parameter-section in .op file
         inquire(iop,opened=lopen)
         if (.not. lopen) then
            filename=opname(1:oplaenge)//'.op'
            open(iop,file=filename,status='old',form='formatted',err=900)
         endif
         rewind(iop)
         iz=0
         do 
            call rdinpf(iop,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            if (ierr .gt. 0) then
               routine = 'Eingabe, search for op file parameter-section'
               message = inptit
               call errormsg
            endif

            if (keyword(1).eq.'parameter-section') then
               source(5)=1
               call heinpara(iop,pass)       
               exit
            elseif (keyword(1).eq.'end-operator') then
               exit
            endif
         enddo
      endif

! look in oper file and read in stored parameters
      if (runlevel .gt. 2) then
         source(6)=1
         inquire(ioper,opened=lopen)
         if (.not. lopen) then
            filename=oname(1:olaenge)//'/oper'
            ilbl=index(filename,' ')-1
            open(ioper,file=filename,form='unformatted',status='old',err=900)
         endif

         rewind(ioper)
         read(ioper) filever(ioper)

         if (filever(ioper) .ge. 11.000_dop) then
            do
   20          continue
               read(ioper,err=20,end=200) rdtag
               if (rdtag(1:1) .ne. '<') cycle

               ilbl=2
               jlbl=index(rdtag,'>')-1
               if (rdtag(ilbl:jlbl) .eq. 'npar') then
                  read(ioper,err=999) npar1
                  allocate(apar1(npar1))
                  allocate(rpar1(npar1))

               else if (rdtag(ilbl:jlbl) .eq. 'apar') then
                  read(ioper,err=999) (apar1(n),n=1,npar1)

               else if (rdtag(ilbl:jlbl) .eq. 'rpar') then
                  read(ioper,err=999) (rpar1(n),n=1,npar1)
                  exit
               endif
            enddo
     200    continue
            npar=npar+npar1

         else

! if reading an old format oper file, this will allocate space and then
! parameters will be read in rdoper.
            npar=npar_pre11
            routine='Einpara'
            message='Reading parameters from old format oper file. '// &
            'Any parameters specified in .inp file will be overwritten.'
            call warnmsg
         endif

      endif

!-----------------------------------------------------------------------
! Allocate memory
!-----------------------------------------------------------------------
      maxpar=npar
      if (allocated(apar)) deallocate(apar)
      if (allocated(rpar)) deallocate(rpar)
      allocate(apar(maxpar))
      allocate(rpar(maxpar))
      apar(:) = ' '
      rpar(:) = 0.0_dop

!-----------------------------------------------------------------------
! Now read in parameters
!-----------------------------------------------------------------------
      pass=2
! internally defined
      npar = 1
      apar(1)='PI'
      rpar(1)=4.0_dop*atan(1.0_dop)

! read from commandf line
      if (source(1) .eq. 1) then
         do n=1,nparcmd
            npar = npar+1
            apar(npar) = aparcmd(n)
            rpar(npar) = rparcmd(n)
         enddo
      endif

! from alter-paramaters section
      if (source(2) .eq. 1) then
         rewind(iin)
         iz=0
         do 
            iz=iz+1
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            if (ierr .gt. 0) then
               routine = 'Eingabe, reading in alter-parameters'
               message = inptit
               call errormsg
            endif

            if (keyword(1).eq.'alter-parameters') then
               call heinpara(iin,pass)       
               exit
            endif
         enddo
      endif

! from paramater-section in iin
      if (source(3) .eq. 1) then
         rewind(iin)
         iz=0
         do 
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            if (ierr .gt. 0) then
               routine = 'Einpara, reading in iin parameter-section'
               message = inptit
               call errormsg
            endif

            if (keyword(1).eq.'parameter-section') then
               call heinpara(iin,pass)       
               exit
            endif
         enddo
      endif

! from parfile
      if (source(4) .eq. 1) then
         rewind(ipar)
         iz=0
         do 
            iz=iz+1
            call rdinpf(ipar,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            if (ierr .gt. 0) then
               routine = 'Eingabe, reading par file'
               message = inptit
               call errormsg
            endif

            if (keyword(1).eq.'parameter-section') then
               call heinpara(ipar,pass)       
               exit
            endif
         enddo
      endif

! from paramater-section in op
      if (source(5) .eq. 1) then
         rewind(iop)
         iz=0
         do 
            iz=iz+1
            call rdinpf(iop,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            if (ierr .gt. 0) then
               routine = 'Eingabe, readin op file'
               message = inptit
               call errormsg
            endif

            if (keyword(1).eq.'parameter-section') then
               call heinpara(iop,pass)       
               exit
            endif
         enddo
      endif

! from oper file
      if (source(6) .eq. 1 .and. filever(ioper) .ge. 11.000_dop) then
         do n=1,npar1
! check against previously loaded parameters
            check=.false.
            do n1=1,npar
               if (apar1(n) .eq. apar(n1)) check=.true.
            enddo
            if (.not. check) then
               npar=npar+1
               apar(npar)=apar1(n)
               rpar(npar)=rpar1(n)
            endif
         enddo

         if (allocated(apar1)) deallocate(apar1)
         if (allocated(rpar1)) deallocate(rpar1)
      endif

      return
  900 routine='Einpara'
      jlbl=slen(filename)
      write(message,'(2a)') 'Error opening file : ',filename(1:jlbl)
      call errormsg
  999 continue
      routine='Einpara'
      message='Error Reading Oper file: '//rdtag(ilbl:jlbl)
      call errormsg
      end subroutine einpara


!#######################################################################
!
!            Subroutine      HEINPARA
!
! Reads the parameters needed to define the Hamiltonian.
! Called from: eingabe, einoper, heingabe
! Calls to: rdinpf,cntrl,wrline,keyunits,rdfactors,mkhpar,warnmsg,errormsg
!
! 03/97 HDM
!
! pass = 1   count parameters
!        2   set up list
!#######################################################################

      subroutine heinpara(ichan,pass)

use decimal, only: long,dop
use global
use dvrdatmod
use griddatmod
use operdef
use runopermod
use channels

use operutil, only: rdfactors,mkhpar

      implicit none

      integer(long)                        :: iz,iz0,len,j,jj,ii,ic,ierr,&
                                              i,n,n1,nfac
      integer(long), intent(in)            :: ichan,pass
      integer(long), dimension(24)         :: optyp
      real(dop)                            :: x,y
      logical(kind=4)                      :: lprint
      character(len=c2),dimension(24)      :: factor

      lprint = .false.

      if (ichan .eq. iin) then
         routine = 'Heinpara (operator-section)'
         write(ilog,'(/a)') 'Parameter values taken from input file'
         write(ilog,'(a)') inptit(1:72)
      else if (ichan .eq. ipar) then
         routine = 'Heinpara (parameter file)'
         write(ilog,'(/2a)') 'Parameter values taken from file :' &
             //parfile(1:parlaenge)
      else if (ichan .eq. iop) then
         routine = 'Heinpara (operator file)'
         write(ilog,'(/2a)') 'Parameter values taken from .op file '
      endif

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
   10 continue
      routine = 'Heinpara'
      call rdinpf(ichan,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      j=index(inptit,'(')
      jj=index(inptit,')')
      if( max(j,jj) .gt. 1 ) then
         if(j  .le. 0 ) j  = jj
         if(jj .le. 0 ) jj = j
         jj=min(j,jj)
         j=index(inptit,'#')
         if( j .le. 0 .or.  j.gt.jj ) then
            message='No brackets allowed in parameter arithmetic!'
            call wrline(zeroi,iz,iz0) 
         endif
      endif
 
      i=1
   15 continue
      if (keyword(i) .eq. 'end-parameter-section') then
         goto 20
      else if (keyword(i) .eq. 'end-alter-parameters') then
         goto 20
      else if (keyword(i) .eq. 'end-operator-section') then
         goto 30
      endif

!-----------------------------------------------------------------------
! Read and convert parameter value.
!-----------------------------------------------------------------------
      ii = i
      if (keyorig(ii).eq.'I') then
         Message = 'The symbol "I" is not a valid parameter name : '
         call wrline(i,iz,iz0)
      endif
      if (keyword(i+1).ne.'=') then
         Routine = 'Heinpara'
         write(Message,'(2a)')  &
              'No argument specified with parameter : ',keyword(i)
         call errormsg
      elseif (keyword(i+3).eq.'[') then   ! Functions
         i=i+2
         if(keyorig(i).eq.'ATAN2' .or. keyorig(i).eq.'MIN' .or.&
            keyorig(i).eq.'MAX' ) then    ! Functions of two variables
            if (keyword(i+4).eq.']') then
               continue
            elseif(keyword(i+3).eq.',' .and. keyword(i+5).eq.']') then
               continue
            else
               Message='There must be exactly two keywords within [...]'
               call wrline(zeroi,iz,iz0)
            endif
            call rdfactors(keyorig(i+2),nfac,factor,optyp,lc(i+2))
            if (pass .eq. 2) call mkhpar(x,nfac,factor,optyp)
            jj=i+3
            if(keyorig(i+3) .eq. ',' ) jj=jj+1 
            call rdfactors(keyorig(jj),nfac,factor,optyp,lc(jj))
            if (pass .eq. 2) call mkhpar(y,nfac,factor,optyp)
            if(keyorig(i).eq.'ATAN2' ) then
               x=atan2(x,y)
            elseif(keyorig(i).eq.'MIN' ) then
               x=min(x,y)
            elseif(keyorig(i).eq.'MAX' ) then
               x=max(x,y)
            else 
               Message = 'Para: Unknown function of two variables :'
               call wrline(i,iz,iz0)
            endif
            i=jj+1
         else                             ! Functions of one variable
            if (keyword(i+3).ne.']') then
               Message='There must be exactly one keyword within [...]'
               call wrline(zeroi,iz,iz0)
            endif
            call rdfactors(keyorig(i+2),nfac,factor,optyp,lc(i+2))
            if (pass .eq. 2) call mkhpar(x,nfac,factor,optyp)
            if(keyorig(i).eq.'EXP' ) then
               x=exp(x)
            elseif(keyorig(i).eq.'LOG' ) then
               x=log(x)
            elseif(keyorig(i).eq.'LOG10') then
               x=log10(x)
            elseif(keyorig(i).eq.'SIN' ) then
               x=sin(x)
            elseif(keyorig(i).eq.'COS' ) then
               x=cos(x)
            elseif(keyorig(i).eq.'TAN' ) then
               x=tan(x)
            elseif(keyorig(i).eq.'ASIN') then
               x=asin(x)
            elseif(keyorig(i).eq.'ACOS') then
               x=acos(x)
            elseif(keyorig(i).eq.'ATAN') then
               x=atan(x)
            elseif(keyorig(i).eq.'SINH') then
               x=sinh(x)
            elseif(keyorig(i).eq.'COSH') then
               x=cosh(x)
            elseif(keyorig(i).eq.'TANH') then
               x=tanh(x)
            elseif(keyorig(i).eq.'ABS' ) then
               x=abs(x)
            elseif(keyorig(i).eq.'INT' ) then
               x=aint(x)
            else
               Message = 'Parameter-Section. Unknown function :'
               call wrline(i,iz,iz0)
            endif
            i=i+3
         endif
      else                    ! Expressions
!....first connect keywords if they start or end with "+-*/".
         i=i+2
         len=lc(i)
         message=keyorig(i)
         jj = i
         do j = i, ic-1
            if(keyorig(j)(lc(j):lc(j)) .eq. '+' .or. &
               keyorig(j)(lc(j):lc(j)) .eq. '-' .or. &
               keyorig(j)(lc(j):lc(j)) .eq. '*' .or. &
               keyorig(j)(lc(j):lc(j)) .eq. '/' .or.&
               keyorig(j+1)(1:1)       .eq. '+' .or. &
               keyorig(j+1)(1:1)       .eq. '-' .or. &
               keyorig(j+1)(1:1)       .eq. '*' .or. &
               keyorig(j+1)(1:1)       .eq. '/' )    then 
               if( jj .ne. j ) goto 50
               jj = j + 1
               message = message(1:len)//keyorig(jj)
               len = len + lc(jj) 
            endif
         enddo 
         i=jj

!....determine the numerical value of the parameter
 50      call rdfactors(message,nfac,factor,optyp,len)
         if (pass .eq. 2) call mkhpar(x,nfac,factor,optyp)
      endif

!....multiply a unit conversion factor
      if (keyword(i+1).eq.',') then
         call keyunits(x,keyword(i+2),ierr)
         if (ierr .ne. 0) then
            message='Unknown unit for parameter : '
            call wrline(i-2,iz,iz0)
         endif
         i=i+2
      endif

!-----------------------------------------------------------------------
! Check if parameter name is not label and if parameter name is allowed.
!-----------------------------------------------------------------------
      do n1=1,npre
         if (keyorig(ii) .eq. prelabel(n1)) then
            write(6,'(/a80)') inptit
            routine='heinpara. PARAMETER-SECTION'
            message= &
             'A parameter-name must not coincide with a pre-defined' &
             //" operator label! See the User's Guide for a list" &
             //" of pre-defined labels. Label : "//trim(keyorig(ii))
            call errormsg
         endif
      enddo
      if(index(keyorig(ii),':') .gt. 0) then
         write(6,'(/a80)') inptit
         routine='heinpara. PARAMETER-SECTION'
         message='A parameter-name must not contain a colon (:).' &
         //' Parameter = '//trim(keyorig(ii))
         call errormsg
      endif

!-----------------------------------------------------------------------
! If parameters are already assigned, do not reassign.
!-----------------------------------------------------------------------
      n=0
      do n1=1,min(npar,maxpar)
         if (keyorig(ii) .eq. apar(n1)) n=n1
      enddo
      if (n .ne. 0 ) then
         if( abs(rpar(n)-x) .gt.  &
                    1.0e-12_dop*(abs(x)+abs(rpar(n)+1.0e-36_dop))) then
            write(ilog,'(2a,1pe13.6e2,4x,"(",1pe13.6e2,")")')  &
          ' Parameter already assigned : ',keyorig(ii)(1:16),rpar(n),x
         end if 
         go to 100
      endif

!-----------------------------------------------------------------------
! Store parameters.
!-----------------------------------------------------------------------
      npar=npar+1
      if (npar .le. maxpar) then
         apar(npar)=trim(keyorig(ii))
         rpar(npar)=x
      endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
  100 continue
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

   20 continue   ! 'end-parameter-section' or 'end-alter-parameters' found

      return
   30 continue
      message='Missing "end-alter-parameters" ? '
      call errormsg
      end subroutine heinpara


      end module einparamod



