!***********************************************************************
! Module containing routines to set up grid and spf information
!***********************************************************************

      module einsbasmod

use decimal, only: long, dop
use logdat
use global
use dglobal
use maxv, only: maxdim,maxsta,maxtape
use channels
use dvrdatmod
use griddatmod
use psidef
use op1lib,  only: zeromxi, zerovxi,cpvxi,cpmxi
use gwptrj, only: ngwptrj
use iogrddef, only: griddat
use rddvrmod, only: dvrinfo
use dirdyn
use runinwfmod, only: lredundant

      implicit none
      private
      public :: eingrd,uncombine,logsbas,einexact,genmlgrd,chkbasis
      contains

!***********************************************************************
!
! Read SBASIS-SECTION. This has 2 sets of information. Which is read is 
! controlled by modus flag
!
! modus: 1  read grid information
!        2  read spf information
!        3  read MLWF tape only (required for MLOps at the end of 
!           runoper)
!
!***********************************************************************

      subroutine eingrd(modus)

      implicit none

      integer(long), intent(in)                  :: modus
      integer(long)                              :: iz,ic,ierr,m,s
      logical(kind=4)                            :: linperr
      logical(kind=4)                            :: check=.false.

!-----------------------------------------------------------------------
! zero information to be set up
!-----------------------------------------------------------------------
      linperr=.false.
      if (modus .eq. 1) then
         meb    = 0
         mpb    = 0
         nstate = 1
         nspfdof(:) = 0
         spfdof(:,:) = 0
         dofspf(:) = 0
         subdim(:) = 0
         gwpm(:) = .false.
         npackts=1
         npacket=1
         lmulpack=.false.
! set default GWP type to frozen, normalised, no phase 
         gwptype = 3
         gwpphase = -1
         lgwpphase = .false.
         lgwpsep = .true.
      else if (modus .eq. 2) then
         if (dentype .eq. 0 .or. dentype .eq. 2) then
            dim(:,:) = 0
         else
            ddim(:,:,:) = 0
         endif
         idmode(:) = 0
         gwpdim(:,:) = 0
      endif

!-----------------------------------------------------------------------
! read spf-basis data 
!-----------------------------------------------------------------------
! CHECK iin open
      inquire(iin,opened=check)
      if (check) close(iin)
      open(iin,file=inputfile(1:inplaenge),status='old',&
           form='formatted',err=900)

      rewind(iin)
      iz=0
      readiin: do

         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .gt. 0) then
            routine = 'Einsbas'
            message = inptit
            call errormsg
         endif
         
         if (dentype .eq. 1 .and. keyword(1) .eq. 'spdo-basis-section') then
            call einsbas(modus,linperr,iz)
            exit

         else if (dentype .ne. 1 .and. &
              (keyword(1) .eq. 'spf-basis-section' .or. &
              keyword(1) .eq. 'sbasis-section')) then
            call einsbas(modus,linperr,iz)
            exit

         else if (keyword(1) .eq. 'multilayer-basis-section' .or. &
              keyword(1) .eq. 'mlbasis-section'  .or. &
              keyword(1) .eq. 'ml-basis-section') then
            lrunml=.true.
            psitype=5
            call einmlbas(modus,linperr,iz)
            exit
            
         else if (keyword(1) == 'end-input' .and. (lexact .or. lrealwp)) then
            call einexact(modus)
            exit
         else if (keyword(1) .eq. 'end-input' .and. ltraj) then
            call eintraj(modus)
            exit
         else if (keyword(1) .eq. 'end-input' .and. ((lgwp .and. ldd) .or. &
                    lgwptrj)) then
            call eincvmcg(modus)
            exit
         else if (keyword(1) .eq. 'end-input' .and. .not. lcont) then
            if (dentype .eq. 1) then
               routine='Eingabe, spdo-basis-section'
               message='No SPDO-BASIS-SECTION in input file'
            else
               routine='Eingabe, spf-basis-section'
               message='No SPF-BASIS-SECTION in input file'
            endif
            call errormsg
         endif

      enddo readiin

!-----------------------------------------------------------------------
! read in GBF basis
! and check consistency
!-----------------------------------------------------------------------
      if (lgwp .and. modus .eq. 2) then
         rewind(iin)
         iz=0
         readgbf: do
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            if (ierr .gt. 0) then
               routine = 'Einsbas'
               message = inptit
               call errormsg
            endif

            if (keyword(1) .eq. 'gbf-basis-section') then
         !      call einsbas(modus,linperr,iz)
               exit

            else if (keyword(1) .eq. 'end-input') then
               do m=1,nmode
                  do s=1,nstate
                     if (gwpm(m) .and. gwpdim(m,s) .eq. 0) gwpdim(m,s) = dim(m,s)
                  enddo
               enddo
               do m=1,nmode
                  do s=1,nstate
                     if (gwpm(m) .and. (gwpdim(m,s) .lt. dim(m,s))) then
                        routine = 'Eingrd'
                        message = 'GWP basis must be at least as big as SPF basis'
                        call errormsg
                     endif
                  enddo
               enddo

               exit
            endif

         enddo readgbf

! Check if specified no. of GWPs fewer than no. of trajectories run
         if (lgwptrj .and. (gwpdim(1,1) .gt. ngwptrj)) then
            routine = 'Eingrd (einsbasmod.F90)'
            write(message,'(a,i5)')  &
                'More GWPs than trajectories. Max ngwp: ',ngwptrj
            call errormsg
         endif

! Check selected type is allowed
         if (.not. lgwpsep .and. gwpphase .eq. -1) then
            routine = 'Eingrd (einsbasmod.F90)'
            write(message,'(a,i5)')  &
                'GWP phase must be propagated for non-separable Gaussian'
            call errormsg
         endif

      endif

!-----------------------------------------------------------------------
! if not set, set "no. of atoms" for Zagreb srfhop code 
! (It may have been set already in einop with shztrans keyword)
!-----------------------------------------------------------------------
      if (ltraj) then
         if (modus .eq. 2) gwpdim(1,1) = dim(1,1)

         if (natmtsh .eq. 0) then
            natmtsh = nspfdof(1)/3
            if (mod(nspfdof(1),3) .ne. 0) natmtsh = natmtsh+1
            ndoftsh = natmtsh*3
         else
            ndoftsh = natmtsh*3
         endif
      endif

      return
  900 continue
      routine='Eingrd (einsbasmod.f90)'
      message = 'Cannot open file: '//inputfile(1:inplaenge)
      call errormsg

      end subroutine eingrd

!***********************************************************************
! 
!             MCTDH subroutine EINSBAS.F
!
!
!***********************************************************************

      subroutine einsbas(modus,linperr,iz)

      implicit none

      integer(long), intent(in)                      :: modus,iz
      integer(long)                                  :: i,iz0,ic,s,ierr,&
                                                        m,f,f1,n,ilbl,p,&
                                                        t,npack1
      logical(kind=4), intent(inout)                 :: linperr
      logical(kind=4)                                :: contline,lcheck
! temp variables for reading
      logical(kind=4)                                :: lmult1,lpacksing1,&
                                                        lfuncs
      integer(long)                                  :: gwptype1,gwpphase1
      logical(kind=4)                                :: lgwpsep1
      integer(long)                                  :: nmode1
      integer(long), dimension(maxdim)               :: nspfdof1,dofspf1,&
                                                        idmode1
      integer(long), dimension(maxdim,maxdim)        :: spfdof1
      integer(long), dimension(maxdim,maxsta*maxsta) :: dimh
      integer(long), dimension(maxdim,maxsta)        :: dim1,gwpdim1
      integer(long), dimension(maxdim,maxsta)        :: maxdimspwn1 

!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
      if (dentype .eq. 1) then
         routine = 'einsbas (read SPDO-BASIS-SECTION )'
      else
         routine = 'einsbas (read SPF-BASIS-SECTION )'
      endif

      iz0 = 0

!-----------------------------------------------------------------------
! set default values for temporary variables
!-----------------------------------------------------------------------
      npack1=0
      lmult1 = .false.
      lpacksing1 = .false.
      lfuncs = .false.
      gwptype1 = -999
      gwpphase1 = -999
      lgwpsep1 = .false.
      gwpdim1(:,:) = 0
      dim1(:,:) = 0
      dimh(:,:) = 0
      nspfdof1(:)=0
      spfdof1(:,:)=0
      dofspf1(:)=0
      idmode1(:)=0
      maxdimspwn1(:,:) = -999

!-----------------------------------------------------------------------
! READ SPF BASIS
!-----------------------------------------------------------------------
      contline=.false.
      m=0
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (dentype .eq. 1 .and. &
         keyword(i) .eq. 'end-spdo-basis-section') then 
         go to 60
      
      else if (dentype .ne. 1 .and. &
          keyword(i) .eq. 'end-sbasis-section') then
         go to 60

      else if (dentype .ne. 1 .and. &
             keyword(i) .eq. 'end-spf-basis-section' ) then
         go to 60

      elseif(keyword(i).eq.'packet' .or. keyword(i).eq.'packets') then
         if (dentype .ne. 0) then
               message='Packets not implemented for densities.'
               call errormsg
         endif         
         if (npack1.gt.0) then
            message = 'There must be no more than one "packets" keyword'
            call errormsg
         endif
         if (keyword(i+1) .ne. '=') then
            message = 'No arguments specified with "packets" keyword'
            call wrline(zeroi,iz,iz0)
         else
            read(keyword(i+2),*,iostat=ierr) npack1
            if (ierr .ne. 0) then
               message = 'Incorrect argument for keyword: '
               call wrline(onei,iz,iz0)
            endif
            if (npack1 .le. 1) then
               message = 'There must be more than one packet.'
               call wrline(zeroi,iz,iz0)
            endif
            if(keyword(i+3).eq.',') then
               i = i+2
               if(keyword(i+2) .eq. 'single-set' ) then
                  lpacksing1=.true.
                  message = 'Multi-packet + single-set not working'// &
                            ' at present. '
                  call errormsg
               elseif(keyword(i+2) .eq. 'multi-set' ) then
                  lpacksing1=.false.
               else
                  message = 'Unknown keyword, expecting'// &
                            ' "single-set" or "multi-set" but not: '
                  call wrline(i+2,iz,iz0)
               endif
            endif

            i = i+2
         endif

      else if (keyword(i) .eq. 'multi-set') then
         lmult1 = .true.
      else if (keyword(i) .eq. 'single-set') then
         lmult1 = .false.

      else if (keyword(i) .eq. 'no-redundancy-check') then
            message =  &
       'no-redundancy-check keyword moved to integrator-section in version 11.0'
            call wrline(zeroi,iz,iz0)

      else if (keyword(i) .eq. 'bel') then
         lBEL = .true.
! insert bel
!-----------------------------------------------------------------------
! set type of GWP basis functions (see griddat.inc)
!-----------------------------------------------------------------------
      else if (keyword(i) .eq. 'gwp_type') then
         if (keyword(i+1) .ne. '=') then
            message = &
               'No arguments specified with "gwp_type" keyword'
               call wrline(zeroi,iz,iz0)
         else
            if (keyword(i+2) .eq. 'thawed') then
               gwptype1 = 0
               lgwpsep1 = .false.
            else if (keyword(i+2)(1:6) .eq. 'correl') then
               gwptype1 = 1
               lgwpsep1 = .false.
            else if (keyword(i+2)(1:5) .eq. 'separ') then
               gwptype1 = 2
               lgwpsep1 = .true.
            else if (keyword(i+2) .eq. 'frozen') then
               gwptype1 = 3
               lgwpsep1 = .true.
            else if (keyword(i+2) .eq. 'frozen_nd') then
               gwptype1 = 4
               lgwpsep1 = .false.
            else if (keyword(i+2)(1:7) .eq. 'nophase') then
               gwpphase1 = -1
            else if (keyword(i+2)(1:6) .eq. 'phase0') then
               gwpphase1 = 0
            else if (keyword(i+2)(1:6) .eq. 'normal') then
               gwpphase1 = 1
            else if (keyword(i+2)(1:6) .eq. 'action') then
               gwpphase1 = 2
            else if (keyword(i+2)(1:2) .eq. 'xp') then
               gwpphase1 = 3
            else
               message = ' unknown keyword : '
               call wrline(i,iz,iz0)
            endif
            i=i+2

            if (keyword(i+1) .eq. ',') then
               i = i+2
               if (keyword(i)(1:7) .eq. 'nophase') then
                  gwpphase1 = -1
               elseif (keyword(i)(1:6) .eq. 'phase0') then
                  gwpphase1 = 0
               else if (keyword(i)(1:6) .eq. 'normal') then
                  gwpphase1 = 1
               elseif (keyword(i)(1:6) .eq. 'action') then
                  gwpphase1 = 2
               elseif (keyword(i)(1:2) .eq. 'xp') then
                  gwpphase1 = 3
               else
                  message = ' unknown keyword : '
                  call wrline(i,iz,iz0)
               endif
            endif
         endif

      else if (keyword(i) .eq. 'nfuncs') then
         lfuncs = .true.
         m=1
         nspfdof1(m)=0
         do f=1,ndof
            if (basis(f) .ne.0) then
               nspfdof1(m)=nspfdof1(m)+1
               n=nspfdof1(m)
               spfdof1(n,m)=f
               dofspf1(f)=m
            endif
         enddo

         if (keyword(i+1).eq.'=') then
            s=0
   30       continue
            s=s+1
            i=i+2
            if (dentype .eq. 1) then
               read(keyword(i),*,iostat=ierr) dimh(m,s)
            else
               read(keyword(i),*,iostat=ierr) dim1(m,s)
            endif
            if (ierr .ne. 0) then
               message ='nspf must be an integer. Not :'
               call wrline(i+2,iz,iz0)
            endif

! look for continuation line or take next number
            if (keyword(i+2).eq.'%') then
               call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                    maxkey)
               message = inptit
               call cntrl(keyword(1),lc(1),ierr)
               i=-1
               go to 30
            else if (keyword(i+1).eq.',') then
               go to 30
            endif

         endif

      else if (keyword(i) .eq. 'gwpbasis') then
         if (m .eq. 0) then
            message = &
         'This keyword is no longer used. To contract GWP basis, use ML-MCTDH'
            call wrline(i+2,iz,iz0)
         endif

! Set up next particle and read SPF numbers 
      else
         m=m+1

   40    continue
         
         if (contline) then
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            message = inptit
            call cntrl(keyword(1),lc(1),ierr)
            i = 1
            contline=.false.
         endif

         f=0
         do f1=1,ndof
            if (keyorig(i)(1:12) .eq. modelabel(f1) ) f=f1
         enddo
! stop if cannot find primitive basis (otherwise number of error messages
! too large)
         if (f.eq. 0) then
            write(message,'(2a)') &
               'Unknown keyword or no primitive basis for spfbasis: '// &
                keyorig(i)(1:12)
            call errormsg
         endif

!-----------------------------------------------------------------------
! if SPFs for electronic dof, flag
! in which case flag electronic mode (meb) 
!-----------------------------------------------------------------------
         if (f .eq. feb .and. lebcont) then
            f = febcont
         else if (f .eq. feb) then
            lebgrid = .true.
            meb=m
         endif

         nspfdof1(m)=nspfdof1(m)+1
         n=nspfdof1(m)
         spfdof1(n,m)=f
         if(dofspf1(f).eq.0) then
            dofspf1(f)=m
         else
            message = 'doubly defined spfbasis: '
            call wrline(i,iz,iz0)
         endif

         if (keyword(i+1).eq.'=' .and. keyword(i+2).eq.'id') then
            i=i+3
            if(keyword(i).eq.',') i=i+1
            read(keyword(i),*,iostat=ierr) idmode1(m)
            if(ierr.ne.0 .or. idmode1(m).le.0 .or. idmode1(m).ge.m) then
               if(idmode(m).ge.m) write(6,'(/a,/a,/a,i4,a,i4)')&
               'The "identical mode" definition must come after the ',&
               'definition of the mode to be identifyed with.',&
               'm =',m,', idmode(m) =',idmode1(m)  
               message ='idmode must be a positive integer. Not :'
               call wrline(i,iz,iz0)
            endif
            do s = 1, maxsta  ! nstate is not defined yet, hence maxsta
               dim1(m,s)=dim1(idmode1(m),s)
            enddo
         elseif (keyword(i+1).eq.'=') then
            s=0
   50       continue
            s=s+1
            if ((dentype .eq. 0 .or. dentype .eq. 2) .and. s .gt. maxsta) then
               routine = "Einsbas"
               message = "More sets of SPFs defined than electronic states"
               call errormsg
            else if (dentype .eq. 1 .and. s .gt. maxsta**2) then
               routine = "Einsbas"
               message = "More sets of SPFs defined than electronic states"
               call errormsg
            endif
            i=i+2
            if (dentype .eq. 1) then
               read(keyword(i),*,iostat=ierr) dimh(m,s)
            else
               read(keyword(i),*,iostat=ierr) dim1(m,s)
            endif
            if (ierr .ne. 0) then
               message ='nspf must be an integer. Not :'
               call wrline(i+2,iz,iz0)
            endif

!DWF        If dynamical WF: read the maximum number of SPFs
!           for multi-set read into s=nstate
            if (ldynspf .and. (keyword(i+1).eq.'<')) then
               i=i+2
               s=1
               do while (i .le. ic)
                  read(keyword(i),*,iostat=ierr) maxdimspwn1(m,s)
                  if (ierr .ne. 0) then
                     message ='maxnspf must be an integer. Not :'
                     call wrline(i+2,iz,iz0)
                  endif
                  i = i+2
                  s=s+1
               enddo
            endif

! look for continuation line or take next number
            if (keyword(i+2).eq.'%') then
               call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                    maxkey)
               message = inptit
               call cntrl(keyword(1),lc(1),ierr)
               i=-1
               go to 50
            else if (keyword(i+1).eq.',') then
               go to 50
            endif

         else if (keyword(i+1).eq.',') then
            if (keyword(i+2).eq.'%') then
               contline=.true.
            else
               i=i+2
            endif
            go to 40
 
         else if (keyword(i+1).eq.'%') then
            contline=.true.
            go to 40

         endif
      endif
     
!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

   60 continue

!-----------------------------------------------------------------------
! m is no. of modes specified
!-----------------------------------------------------------------------
      nmode1=m

!-----------------------------------------------------------------------
! Is calculation only an electronic basis?
! If so, force single-set
!-----------------------------------------------------------------------
      lebonly = .false.
      if (nmode1 .eq. 1) then
         f1 = spfdof1(1,1)
         if (f1 .eq. feb) lebonly = .true.
      endif

!-----------------------------------------------------------------------
! If gdim(feb) = 1 there is only 1 state - force multi-set means it is 
! ignored in dynamics
!-----------------------------------------------------------------------
      if (gdim(feb) .eq. 1) lmult1 = .true.

!-----------------------------------------------------------------------
! load data read into information variables
!-----------------------------------------------------------------------
     select case (modus)

!#######################################################################
! grid input (e.g. combinations, electronic basis representation)
!#######################################################################
     case(1) ! set up grids

      nmode=nmode1
      do m=1,nmode
         nspfdof(m) = nspfdof1(m)
         do n=1,nspfdof(m)
            spfdof(n,m) = spfdof1(n,m)
         enddo
      enddo
      maxnspfdof = maxval(nspfdof(1:nmode))
      do f=1,ndof
         dofspf(f) = dofspf1(f)
      enddo

!-----------------------------------------------------------------------
! set flag for multi-set 
!-----------------------------------------------------------------------
      lmult = lmult1 .or. lmult
      leb = .not. lmult

!-----------------------------------------------------------------------
! force single-set for:
! electronic only calculation 
! swarm of trajectories
! OR
! force multi-set for:
! realwp calculation
! DD-MCTDH
!-----------------------------------------------------------------------
      if (ltraj) then
         lmult = .false.
         leb = .true.

      else if (lebonly .and. .not. lebcont) then
         lmult = .false.
         leb = .true.

      else if (leb .and. (lrealwp .or. lddmctdh)) then
         lmult = .true.
         leb = .false.

      endif

!-----------------------------------------------------------------------
! If single-set add mode to be propagated for electronic basis
!-----------------------------------------------------------------------
      if (leb .and. meb .eq. 0) then
         nmode=nmode+1
         dofspf(feb)=nmode
         meb=dofspf(feb)
         nspfdof(meb)=1
         spfdof(1,meb)=feb
      endif

!-----------------------------------------------------------------------
! check feb is last mode 
!-----------------------------------------------------------------------
      if (meb .gt. 0) then
         if (.not. lmult .and. meb .ne. nmode) then
            message= 'The electronic DOF must be the last mode'
            call errormsg
         endif
      endif

!-----------------------------------------------------------------------
! set gwptype etc. Take defaults unless they have been set here.
!-----------------------------------------------------------------------
      if (gwptype1 .ge. 0) then
         gwptype = gwptype1
         lgwpsep = lgwpsep1
      endif
      if (gwpphase1 .ne. -999) gwpphase = gwpphase1
      if (gwpphase .ge. 0) then
         lgwpphase=.true.
      else
         lgwpphase=.false.
      endif

      if (lxloc .and. lmult) then
         message = 'Multi-set and x-local basis not allowed'
         call warnmsg
      endif
      do m=1,nmode
         f=spfdof(1,m)
         if (basis(f) .eq. 26 .and. nspfdof(m) .gt. 1) then
             message= 'X-local basis functions cannot be in combined modes'
             call warnmsg
         endif
      enddo
      if (lgwptrj .and. gwpphase .ne. 2) then
         message = 'Trajectory driven GWPs must be with action phase'
         call errormsg
      endif

!-----------------------------------------------------------------------
! see if any modes are multi-modes
!-----------------------------------------------------------------------
      do m=1,nmode
         if (nspfdof(m) .gt. 1) lconm = .true.
      enddo

!-----------------------------------------------------------------------
! calculate "grid" data
!-----------------------------------------------------------------------
      call griddat

!#######################################################################
! SPF input (e.g. no of spfs)
!#######################################################################
     case(2) ! set up SPFs

!-----------------------------------------------------------------------
! read in spf-basis
!-----------------------------------------------------------------------
      if (lmult) nstate = nddstate

      do m=1,nmode
         idmode(m) = idmode1(m)
      enddo

      if (dentype .eq. 0 .or. dentype .eq. 2) then
         dim = 0
         do s=1,nstate
            do m=1,nmode
               dim(m,s) = dim1(m,s)
            enddo
         enddo
      else if (dentype .eq. 1) then
         ddim = 0
         do m=1,nmode
            p = 1
            do s=1,nstate
               do t=1,nstate
                  ddim(m,s,t) = dimh(m,p)
                  p = p+1
               enddo
            enddo

         enddo
      endif

!-----------------------------------------------------------------------
! check that spf basis size is correct for electronic basis
!-----------------------------------------------------------------------
      if (dentype .eq. 1 .and. .not. lexact) then
         if (lmult) then
            lcheck=.false.
            do m=1,nmode
               do s=1,nstate
                  do t=1,nstate               
                     if (ddim(m,s,t) .eq. 0) then
                        ddim(m,s,t) = ddim(m,1,1)
                        lcheck = .true.
                     endif
                  enddo
               enddo
            enddo
          
            if (lcheck) then
               write(message,'(2a)') &
           'Not enough basis sets for multi-set calculation. ', &
               'Copying first value for all blanks.'
               call warnmsg
            endif
         endif

      else 
         if (lmult) then
            lcheck=.false.
            do s=2,nstate
               do m=1,nmode
                  if (dim1(m,s) .eq. 0) then
                     dim(m,s) = dim(m,1)
                     lcheck = .true.
                  endif
               enddo
            enddo
            if (lcheck) then
               routine='Einsbas'
               write(message,'(2a)') &
            'Not enough basis sets specified for multi-set calculation. ', &
               'Copying first value for all blanks.'
               call warnmsg1
            endif
         endif
      endif

!-----------------------------------------------------------------------
!DWF  If dynamical WF: read the maximum number of SPFs
!     If not specified, set to values given in file.
!-----------------------------------------------------------------------
      if (ldynspf) then
         if (meb .gt. 0) then
            if (dentype .eq. 1) then
               maxdimspwn1(meb,1) = gdim(feb)*gdim(feb)
            else
               maxdimspwn1(meb,1) = gdim(feb)
            endif
         endif

         do s=1,nstate
            do m=1,nmode
               if (maxdimspwn1(m,s) .le. 0) then
                  maxdimspwn(m,s) = dim(m,s)
                  dim(m,s) = min(ndynspf0,dim(m,s))
               else 
                  maxdimspwn(m,s)=maxdimspwn1(m,s)
               endif
            enddo
         enddo

         do s=1,nstate
            do m=1,nmode
               if (maxdimspwn(m,s).lt.dim(m,s)) then
                  message = 'Maximum number of SPFs must be larger' &
             //' than the initial value in a dynamical nSPF propagation'
                  call errormsg
               endif
            enddo
         enddo

      endif

!-----------------------------------------------------------------------
! Contiuum basis SPFs must have been defined
!-----------------------------------------------------------------------
      if (lebcont) then
         m = dofspf(febcont)
         if (m .le. 0) then
            message = 'No SPF basis defined for continuum electronic states'
            call errormsg
         endif
         if (dentype .eq. 1) then
            if (ddim(m,1,1) .le. 0) then
               message = 'No SPDO basis defined for continuum electronic states'
               call errormsg
            endif
         else
            if (dim(m,1) .le. 0) then
               message = 'No SPF basis defined for continuum electronic states'
               call errormsg
            endif
         endif
      endif

!-----------------------------------------------------------------------
! Unless defined, add complete basis for electronic basis if single-set 
! Force complete basis for electronic basis in exact calculations
!-----------------------------------------------------------------------
      if (meb .gt. 0) then
         if (lexact) then
            dim(meb,1) = gdim(feb)
         endif
! turn of lebgrid flag if electronic basis is complete
         if (dim(meb,1) .eq. gdim(feb)) lebgrid = .false.
      endif

      if (lebgrid .and. lmult) then
         message = &
            'Cannot use SPFs on an electronic basis in a multi-set representation'
         call errormsg
      endif

      if (meb .gt. 0) then
         if (dentype .eq. 1) then
            if (ddim(meb,1,1) .eq. 0) ddim(meb,1,1)=gdim(feb)*gdim(feb)
         else 
            if (dim(meb,1) .eq. 0) dim(meb,1)=gdim(feb)
         endif
      endif

!-----------------------------------------------------------------------
! dim and ddim pointer equivalence for single-set
!-----------------------------------------------------------------------
      if (leb .and. dentype .eq. 1) then
         do m=1,nmode
            dim(m,1) = ddim(m,1,1) 
         enddo
      endif

!-----------------------------------------------------------------------
! If npack1 has been set, read in data for packets
!-----------------------------------------------------------------------
      npacket=1
      npackts=1
      lmulpack=.false.
      if (npack1 .gt. 1) then
         npacktot = npack1
         if (lpacksing1) then
            npackts=npack1
            lmulpack = .false.
         else
            npacket=npack1
            lmulpack = .true.
         endif
      endif

      if(lpacksing1 .and. .not.lreflex) then
         if (ndof .ge. maxdim) then
            routine = 'einsbas'
            message = 'Redimension MAXDIM in maxdim.inc. Minimum: '
            ilbl=index(message,':')+2
            write(message(ilbl:ilbl+4),'(i5)') ndof+2
            call errormsg
         endif
         if(irundvr .eq. 1) ndof = ndof+1 ! If DVR is read, ndof is
         fpb        = ndof             ! already increased.
         nmode      = nmode+1
         mpb        = nmode
         gdim(fpb)  = npackts
         subdim(mpb)= npackts
         basis(fpb) = 0
         ldvr(fpb)  = .false.
         modelabel(fpb)= 'packets'
         nspfdof(mpb)  = 1
         spfdof(1,mpb) = fpb
         dofspf(fpb)   = mpb
         maxgdim=max(maxgdim,npackts)
         maxgdim2=maxgdim**2
!DWF     For dynamical WF: define spawnmaxdim
         if (ldynspf) maxdimspwn(mpb,1) = gdim(fpb)
      elseif(lpacksing1 .and. lreflex) then
         if (ndof .ge. maxdim) then
            routine = 'einsbas'
            message = 'Redimension MAXDIM in maxdim.inc. Minimum: '
            ilbl=index(message,':')+2
            write(message(ilbl:ilbl+4),'(i5)') ndof+2
            call errormsg
         endif
         if( meb .ne. nmode ) then
            routine = 'einsbas'
            message = 'The electronic mode must be the last one '// &
                      'in a reflex calculation.'
            call errormsg
         endif
         if(irundvr .eq. 1) ndof = ndof+1 ! If DVR is read, ndof is
          nmode      = nmode+1          ! already increased.
         fpb        = feb
         mpb        = meb
         feb        = ndof
         meb        = nmode
         gdim(feb)  = gdim(fpb)
         subdim(meb)= gdim(fpb)
         gdim(fpb)  = npackts
         subdim(mpb)= npackts
         basis(fpb) = 0
         basis(feb) = 0
         ldvr(fpb)  = .false.
         modelabel(feb)= modelabel(fpb)
         modelabel(fpb)= 'packets'
         nspfdof(meb)  = 1
         nspfdof(mpb)  = 1
         spfdof(1,mpb) = fpb
         dofspf(fpb)   = mpb
         spfdof(1,meb) = feb
         dofspf(feb)   = meb
         maxgdim=max(maxgdim,npackts)
         maxgdim2=maxgdim**2
      endif

!-----------------------------------------------------------------------
! copy SPF basis to all packets
!-----------------------------------------------------------------------
      if (dentype .ne. 1) then
         do p = 2,npacket
            do s=1,nstate
               t=s+(p-1)*nstate
               do m = 1,nmode
                  dim(m,t) = dim(m,s)
               enddo
            enddo
         enddo


         if(lpacksing1 .and. .not.lreflex) then
            do s=1, nstate
               dim(mpb,s) = gdim(fpb)
            enddo
         elseif(lpacksing1 .and. lreflex) then
            do s=1, nstate
               dim(mpb,s) = gdim(fpb)
               dim(meb,s) = gdim(feb)
            enddo
         endif
      endif

!-----------------------------------------------------------------------
! set nstate to include packets
!-----------------------------------------------------------------------
      nstate = nstate*npacket

!-----------------------------------------------------------------------
! Check that GWPbasis has been set so that ngwp .ge. nspf
! this has to come after griddat called so that gwpm(m) has been set
!-----------------------------------------------------------------------
      if (lgwp) then
         do s=1,nstate
            do m=1,nmode
               if (gwpm(m)) gwpdim(m,s) = max(gwpdim1(m,s),dim(m,s))
            enddo
         enddo
      endif
               
!-----------------------------------------------------------------------
! check that dim .le. subdim. Unless a trajectory or vMCG calculation
!-----------------------------------------------------------------------
      if (ltraj) go to 100
      if (dentype .eq. 1) then
         do s=1,nstate
            do t=1,nstate
               do m=1,nmode
                  if (ddim(m,s,t) .gt. subdim(m) ) then
                     routine = "Einsbas"
                     message = 'Too many spdo s for mode:             '
                     write(message(25:28),'(i4)') m
                     call warnmsg
                  endif
               enddo
            enddo
         enddo
      else
         do s=1,nstate
            do m=1,nmode
               if (gwpm(m)) cycle
               if (dim(m,s) .gt. subdim(m)) then
                     routine = "Einsbas"
                  message = 'Too many spf s for mode:             '
                  write(message(25:28),'(i4)') m
                  call warnmsg
               end if
            enddo
         enddo
      endif
  100 continue

!-----------------------------------------------------------------------
! if any x-local functions, take dim(m,s)=gdim(f)
!-----------------------------------------------------------------------
      do m=1,nmode
         f=spfdof(1,m)
         if (basis(f) .eq. 26) dim(m,1)=gdim(f)
      enddo
   
!-----------------------------------------------------------------------
! Check that an spf basis has not been defined for a "point DVR".
! A point DVR should be part of coefficient "mode".
! Assign any other DOFs not in modes to point DVRs with q=0.
!-----------------------------------------------------------------------
      do f=1,ndof
         m=dofspf(f)
     !    if (m .eq. 0) then
     !       basis(f) = 19
     !       lpntdvr = .true.
     !       rpbaspar(1,f) = 0.0_dop
     !       dofspf(f) = mkoe
     !       nspfdof(mkoe) = npsfdof(mkoe)+1
     !       n = nspfdof(mkoe) 
     !       spfdof(n,mkoe) = f
     !    endif

         if (basis(f) .eq. 19 .or. basis(f) .eq. 22) then
            if (m .ne. mkoe) then 
               message =  &
         'Spf s should not be defined for a point: '//modelabel(m)
               call warnmsg
            endif
         endif
      enddo

!-----------------------------------------------------------------------
! warn if ngwp was specified in ! run-section
!-----------------------------------------------------------------------
      if (ldd .and. ngwp .ne. 0) then
         routine='Einsbas'
         message = 'Ignoring no. GWPs specified in RUN-SECTION (ngwp)' 
         call warnmsg
      endif

!-----------------------------------------------------------------------
! check no. of functions for trajectory GWPs
! and set options
!-----------------------------------------------------------------------
      if (lgwptrj) then
! phase is not propagated
         lgwpphase = .false.
         gwpphase = -1
         if (nmode .gt. 1) then
            routine='Einsbas'
            message='Trajectory GWP calculation requires 1 mode only (vMCG)'
            call errormsg
         endif

         if (gwpdim(1,1) .gt. ngwptrj) then
            routine='Einsbas'
            write(message,'(a,i5)') &
              'Too many GWPs in Trajectory GWP calculation. NTrajectories =',&
            ngwptrj
            call errormsg
         else if (gwpdim(1,1) .lt. ngwptrj) then
            routine='Einsbas'
            write(message,'(2a,i5)') &
              'Fewer GWPs than trajectories in Trajectory GWP calculation. ', &
              'Ignoring last',ngwptrj-gwpdim(1,1)
            call warnmsg1
         endif
      endif

     end select 

!-----------------------------------------------------------------------
! if numerically exact propagation, change various variables
!-----------------------------------------------------------------------
      if (lexact .or. lrealwp) then
         call einexact(modus)
      endif

!-----------------------------------------------------------------------
! if swarm of trajectories, set up single mode etc.
!-----------------------------------------------------------------------
      if (ltraj) then
         call eintraj(modus)
      endif

!-----------------------------------------------------------------------
! if direct dynamics 
! 1. if no spfs specified, set up modes
! ELSE
! 2. check included DOF not frozen
! 3. check frozen DOFs have been set up as point DVRs
!-----------------------------------------------------------------------
      if (ldd .or. lgwptrj) then
         if (.not. leb .and. nmode .eq. 0) then
            call eincvmcg(modus)
         else
            do m=1,nmode
               if (m .eq. meb) cycle
           
               do n=1,nspfdof(m)
                  f = spfdof(n,m)
                  if (basis(f) .eq. 19) then
                     routine='Einsbas'
                     write(message,'(2a,i5)') &
                    'All DOFs included in the dynamics must not be frozen.  ', &
                    'Check the "freeze" options used  in the INITIAL-GEOMETRY-SECTION'
                     call errormsg
                     exit  ! only pront error once
                  endif
               enddo
            enddo
               
            do f=1,ndof
               if (f .eq. feb) cycle

               if (dofspf(f) .eq. 0 .and. basis(f) .ne. 19) then
                  routine='Einsbas'
                  write(message,'(a,i5,a)') 'DOF ',f,&
                      ' not included in the dynamics taken to be at Q=0.'
                  call warnmsg1
               endif
            enddo
         endif
      endif

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr
      lwarn=.false.

      return
      end subroutine einsbas

!#######################################################################

      subroutine einexact(modus)

      implicit none

      integer(long), intent(in)              :: modus
      integer(long)                          :: m,f,n,s,n1,t
      integer(long), dimension(maxdim)       :: spfdof1

      select case (modus)

     case (1) ! set up grid
!-----------------------------------------------------------------------
! if nmode = 0, no sbasis-section was present. 
! Look for electronic basis and set up a single multi-mode spf. 
! 
! if nmode > 0, then take information from sbasis-section, and set
! up single multi-mode function in specified order
! Do not include an electronix basis into "SPF"
!-----------------------------------------------------------------------
      if (nmode .eq. 0) then
         nmode=1
         nspfdof(1)=0
! ebonly calculation
         if (ndof .eq. 1 .and. feb .eq. 1) then
            lebonly = .true.

            nspfdof(1)=1
            spfdof(1,1)=feb
            dofspf(feb)=1
         else
            do f = 1, ndof
               if (basis(f) .eq. 0) then
                  feb=f
                  lmult=.true.
                  nstate = nddstate
               else
                  nspfdof(1)=nspfdof(1)+1
                  n=nspfdof(1)
                  spfdof(n,1)=f
                  dofspf(f)=1
               endif
            end do
         endif

      else
         n1=0
         do m=1,nmode
            if (m .eq. meb) cycle
            do n=1,nspfdof(m)
               n1=n1+1
               f=spfdof(n,m)
               spfdof1(n1)=f
               dofspf(f)=1
            enddo
         enddo

         nmode=1
         nspfdof(1)=n1
         do n=1,nspfdof(1)
            spfdof(n,1)=spfdof1(n)
         enddo
         if (meb .gt. 0) then
            nmode=nmode+1
            nspfdof(nmode)=1
            spfdof(1,nmode)=feb
            dofspf(feb)=meb
         endif

! ebonly calculation
         lebonly = .false.
         if (nmode .eq. 1) then
            f = spfdof(1,1)
            if (f .eq. feb) lebonly = .true.
         endif
      endif
      maxnspfdof = maxval(nspfdof(1:nmode))

! EB calculation requires single-set
      if (lebonly) then
         lmult = .false.
         leb = .true.
         nstate = 1
      endif
 
!-----------------------------------------------------------------------
! calculate "grid" data
!-----------------------------------------------------------------------
      call griddat

     case (2) ! set up SPFs
!-----------------------------------------------------------------------
! 1 "SPF" per state
!-----------------------------------------------------------------------
      if (dentype .eq. 1) then
         do s=1,nstate
            do t=1,nstate
               ddim(1,t,s)=1
            enddo
         enddo
      else
         do s=1,nstate
            dim(1,s)=1
         enddo
      endif

     end select

      return
      end subroutine einexact

!#######################################################################

      subroutine eintraj(modus)

      implicit none

      integer(long), intent(in)              :: modus
      integer(long)                          :: m,f,n,n1
      integer(long), dimension(maxdim)       :: spfdof1

      select case (modus)

     case (1) ! set up grid
!-----------------------------------------------------------------------
! see if no. of states > 0 and electronic basis defined
!-----------------------------------------------------------------------
      if (feb .gt. 0) then
         lmult =.false.
         leb=.true.
      else if (feb .eq. 0) then
         leb =.false.
         lmult = .false.
      endif

!-----------------------------------------------------------------------
! if nmode = 0, no sbasis-section was present. 
! Look for electronic basis and set up a single multi-mode spf. 
! 
! if nmode > 0, then take information from sbasis-section, and set
! up single multi-mode function in specified order
!-----------------------------------------------------------------------
      if (nmode .eq. 0) then
         nmode=1
         nspfdof(1)=0
         do f = 1, ndof
            if (basis(f) .eq. 27) then
               nspfdof(1)=nspfdof(1)+1
               n=nspfdof(1)
               spfdof(n,1)=f
               dofspf(f)=1
            endif
         end do          

      else
         n1=0
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (basis(f) .eq. 27) then
                  n1=n1+1
                  spfdof1(n1)=f
                  dofspf(f)=1
               endif
            enddo
         enddo

         nmode=1
         nspfdof(1)=n1
         do n=1,nspfdof(1)
            spfdof(n,1)=spfdof1(n)
         enddo

      endif
      maxnspfdof = maxval(nspfdof(1:nmode))

      if (leb) then
         nmode=2
         nspfdof(2) = 1
         spfdof(1,2)=feb
         dofspf(feb)=2
         meb=2
      endif

!-----------------------------------------------------------------------
! calculate "grid" data
!-----------------------------------------------------------------------
      call griddat

      return

     case (2) ! set up "SPFs"
!-----------------------------------------------------------------------
! set no. of trajectories per state
!-----------------------------------------------------------------------
      dim(1,1)=ntraj

      if (leb) then
         dim(2,1) = gdim(feb)
      endif

      return

     end select

      end subroutine eintraj

!#######################################################################
!
! if nmode = 0 no sbasis-section has been read so GWP "grid" parameters
! must be set up
!
!#######################################################################

      subroutine eincvmcg(modus)

      implicit none

      integer(long), intent(in) :: modus
      integer(long)             :: f,n,s,ngdof,m,nspfm,nrest
      select case (modus)

     case (1) ! set up grid

      if (ngwp .eq. 0 .and. nmode .eq. 0) then
         routine = 'Eincvmcg (einutil.F)'
         message = &
         'No GWP basis has been specified. Use NGWP keyword in '//&
         'RUN-SECTION. Or add an SBASIS-SECTION'
         call errormsg
      endif

!-----------------------------------------------------------------------
! see if no. of states > 0 and electronic basis defined
!-----------------------------------------------------------------------
      if (gdim(feb) .eq. 1) then
         leb =.false.
         lmult = .true.
      else if (.not. lmult) then
         leb=.true.
      endif

!-----------------------------------------------------------------------
! If gwp "grids" must be set up, set up multi-mode spf. 
!-----------------------------------------------------------------------
      if (nmode .eq. 0) then
! npartgwp is the no. of modes, which is 1 by default (a single-mode 
! vMCG calculation)
         nmode=npartgwp
! no. of GWP modes
         ngdof=0
         do f = 1,ndof
            if (basis(f) .eq. 25) ngdof=ngdof+1
         enddo
! no. of DOFs per mode
         nspfm = ngdof / nmode
         nrest = ngdof - (ngdof/nmode)*nmode

! nspfm DOFs in each mode except the last which takes the rest
         f=0
         do m=1,nmode

            nspfdof(m)=nspfm
            if (m .le. nrest) nspfdof(m) = nspfdof(m) + 1

            do n = 1,nspfdof(m)
  20           continue
               f=f+1
               if (basis(f) .ne. 25) go to 20
               spfdof(n,m)=f
               dofspf(f)=m
            enddo
         end do   

! add electronic DOF as a mode if single-set
         nddstate = gdim(feb)
         if (.not. lmult .and. nddstate .gt. 1) then
            nmode=nmode + 1
            nspfdof(nmode) = 1
            spfdof(1,nmode)=feb
            dofspf(feb)=nmode
            meb=nmode
            subdim(meb) = gdim(feb)
         endif
      endif

!-----------------------------------------------------------------------
! calculate "grid" data
!-----------------------------------------------------------------------
      call griddat

      return

     case (2) ! set up SPFs
!-----------------------------------------------------------------------
! set no. of GWPs per state if ngwp keyword used in place
! of SBASIS-SECTION 
!-----------------------------------------------------------------------
! sbasis specified by sbasis-section
      if (ngwp .eq. 0) return

      if (lmult) then
         do s=1,nstate
            do m=1,nmode
               dim(m,s)=ngwp
               gwpdim(m,s)=ngwp
            enddo
         enddo
      else if (leb) then
         do m=1,nmode
            if (m .eq. meb) cycle
            dim(m,1) = ngwp
            gwpdim(m,1) = ngwp
         enddo
         dim(meb,1) = gdim(feb)
      else 
         do m=1,nmode
            dim(m,1) = ngwp
            gwpdim(m,1) = ngwp
         enddo
      endif

      return

     end select

      end subroutine eincvmcg

!#############################################################################
! MLML
!-----------------------------------------------------------------------------
! Read the mlbasis-section from .inp file
!                     Author: Oriol Vendrell (oriol.vendrell@gmail.com)
!                     Year  : 2008
!-----------------------------------------------------------------------------

      subroutine einmlbas(modus,linperr,iz)
        ! called from eingrd
        ! Process the mlbasis section and generate the tape representation 
        ! of the ML tree

        implicit none

        integer(long), intent(in) ::  modus,iz
        logical(kind=4)           ::  linperr,lprim
        integer(long)             ::  b,f1,iz0,ic,ierr,i,ic0,gdim1
        integer(long)             ::  elnode=0
        integer(long)             ::  itape    ! points to the first 
                                               ! free position on the tape
        integer(long)             ::  f        ! number of coordinate according 
                                               ! to pbas-section order
        integer(long)             ::  m        ! counter for total of primitive 
                                               ! modes
        integer(long)             ::  mt       ! counter for total of modes in 
                                               ! all layers
        integer(long)             ::  n        ! counter for coordinates in 
                                               ! prim. mode
        integer(long)             ::  p        ! accumulates subdim for a given 
                                               ! primitive mode
        integer(long)             ::  lnum        ! counter for layers
        integer(long)             ::  prevlnum    ! stores num of previous layer
        integer(long)             ::  node        ! node number
        integer(long)             ::  nsmodes     ! number of submodes of a 
                                                  ! given mode
        integer(long)             ::  maxbranch   ! max. no. of branches
                                                  ! in any node
        integer(long), allocatable :: nbranch(:) ! stores number of the last 
                                                    ! visited branch of a mode.
        integer(long), allocatable :: nlbranch(:,:) ! stores the number of
                                                  ! branches specified for
                                                  ! each layer and checks
                                                  ! this is correct
        integer(long)              :: proddim     ! stores the product of 
                                                  ! submodes of a given node.
        integer(long), allocatable :: dimbuffer(:,:) ! dimbuffer(:,d) 
                                               ! stores 
                                               ! number of spf's (dim) of the
                                               ! node at depth d when going 
                                               ! down a branch
        integer(long), allocatable :: mdimbuffer(:,:) ! maxdimspwn buffer
               ! note: nbranch and dimbuffer could be done much nicer with 
               ! stacks, but fortran has no stacks...
        integer(long) :: nc
!CDWF   For dynamical WF:
        integer(long)              :: ncount=1
        integer(long), allocatable :: ncountbuffer(:,:)

!-----------------------------------------------------------------------
! ML-MCTDH uses single-set
!-----------------------------------------------------------------------
       leb = .true.
       lmult = .false.

!-----------------------------------------------------------------------
! Check if first read of grid (in genoper) was for an ML-calculation
!-----------------------------------------------------------------------
       if ((modus .eq. 2 .or. modus .eq. 3) .and. maxtape .eq. 1) then
          routine='Einmlbas'
          message='Operator not generated for an ML calculation.'
          call errormsg
       endif
         
!-----------------------------------------------------------------------
! Find max value of nlayers (no. of layers) for ML-MCTDH.
! nlayers = 2 is normal MCTDH (default) 
!-----------------------------------------------------------------------
       if (nlayers .eq. 2) then 
          ! Read in one line
          call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
          message = inptit
          call cntrl(keyword(1),lc(1),ierr)
        
          do while(.not.((keyword(1).eq.'end-multilayer-basis-section') &
              .or.  (keyword(1).eq.'end-mlbasis-section') &
              .or.  (keyword(1).eq.'end-ml-basis-section') ) )

            if(keyword(2) .ne. '>' ) then
               write(6,'(/a,/a72)') '### Wrong line reads:',inptit
               routine='Einmlbas'
               message='Expecting a ">" as second keyword'
               call errormsg
            endif

            read(keyword(1),*,err=888) lnum
! in input layer 1 is labelled 0
            nlayers = max(nlayers,lnum+1)

            ! Read in next line
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            message = inptit
            call cntrl(keyword(1),lc(1),ierr)

          enddo

! rewind and read to start of section again
          rewind(iin)
          do while(.not.((keyword(1).eq.'multilayer-basis-section') &
              .or.  (keyword(1).eq.'mlbasis-section') &
              .or.  (keyword(1).eq.'ml-basis-section') ) )
             call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
          enddo
       endif

!-----------------------------------------------------------------------
! allocate work arrays
!-----------------------------------------------------------------------
      allocate(nbranch(0:nlayers-1))
      allocate(nlbranch(nlayers,2)) 
      allocate(ncountbuffer(50,0:nlayers))
      nbranch = 1
      nlbranch = 0

!-----------------------------------------------------------------------
! Find max value of nbranches (no. of branches) 
!-----------------------------------------------------------------------
       maxbranch = 0
       ! Read in one line
       call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
       message = inptit
       call cntrl(keyword(1),lc(1),ierr)
        
       do while(.not.((keyword(1).eq.'end-multilayer-basis-section') &
           .or.  (keyword(1).eq.'end-mlbasis-section') &
           .or.  (keyword(1).eq.'end-ml-basis-section') ) )

         read(keyword(1),*,err=888) lnum
! first look to see if max spfs specified for DynWF run
         ic0=ic
         do i=3,ic 
            if (keyword(i) .eq. '<') ic0 = i-1
         enddo
! Unless primitive, count no. of branches specified for node 
         if (keyword(3) .ne. '[') then
             nlbranch(lnum+1,1) = nlbranch(lnum+1,1) + ic0 - 2
             maxbranch = max(maxbranch,ic-2)
         endif

         ! Read in next line
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         message = inptit
         call cntrl(keyword(1),lc(1),ierr)

       enddo

! rewind and read to start of section again
       rewind(iin)
       do while(.not.((keyword(1).eq.'multilayer-basis-section') &
           .or.  (keyword(1).eq.'mlbasis-section') &
           .or.  (keyword(1).eq.'ml-basis-section') ) )
          call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
       enddo

!-----------------------------------------------------------------------
! allocate work array for no. of SPFs
!-----------------------------------------------------------------------
       allocate(dimbuffer(maxbranch,0:nlayers-1))
       allocate(mdimbuffer(maxbranch,0:nlayers-1))
       dimbuffer = 0
       mdimbuffer = 0

!-----------------------------------------------------------------------
! If modus=1, set up grids
! If modus=2, read in tree/SPF information
! If modus=3, read in tree information
!-----------------------------------------------------------------------
        itape = 1
        lnum  = 0
        m  = 0
        mt = 0
        node=0
        lprim=.false.

        nspfdof=0
        spfdof=0
        dofspf=0

        ! Read in one line
        call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
        message = inptit
        call cntrl(keyword(1),lc(1),ierr)
        
        do while(.not.((keyword(1).eq.'end-multilayer-basis-section') &
             .or.  (keyword(1).eq.'end-mlbasis-section') &
             .or.  (keyword(1).eq.'end-ml-basis-section') ) )

          prevlnum = lnum

          if(keyword(2) .ne. '>' ) then
             write(6,'(/a,/a72)') '### Wrong line reads:',inptit
             routine='einmlbas'
             message='Expecting a ">" as second keyword'
             call errormsg
          endif

!-----------------------------------------------------------------------
! Write control-info to tape. 
! Go through tree writing info for each node starting at top 
! -1 step down to new layer
! 0 step up to old layer
! nbranch : branch no. of node
! nsmodes : no. of modes in node
! If not primitive node:
!    nsdim : no. of SPFs for nodes of child layer 
! Else
!    gdim(f) : no. grid points for each DOF in node mode
!
! -2 end
!-----------------------------------------------------------------------
          if(itape>1) then
             i=1
! get layer number
             read(keyword(1),*,err=888) lnum
             node=node+1

! check that the no. of branches specifed for each layer (nlbranch(layer,1))
! matches the number actually defined (nlbranch(layer,2)). 
             nlbranch(lnum,2) = nlbranch(lnum,2) + 1

             if((lnum-prevlnum)==1) then
! have moved one layer down
                nbranch(lnum) = 1
                if (modus .eq. 2 .or. modus .eq. 3) then
                   tape(itape)   = -1
                   tape(itape+1) =  nbranch(lnum)
                endif
                itape = itape + 2
             elseif((lnum-prevlnum)==0) then
! have stayed in same layer 
                nbranch(lnum) = nbranch(lnum) + 1
                if (modus .eq. 2 .or. modus .eq. 3) then
                   tape(itape)   =  0
                   tape(itape+1) = -1
                   tape(itape+2) =  nbranch(lnum)
                endif
                itape = itape + 3
             elseif((lnum-prevlnum)<0) then
! have moved back up tree
                nbranch(lnum) = nbranch(lnum) + 1
                if (modus .eq. 2 .or. modus .eq. 3) then
                   do i=1, prevlnum - lnum + 1
                      tape(itape) = 0
                      itape = itape + 1
                   end do
                   tape(itape)   = -1
                   tape(itape+1) = nbranch(lnum)
                else if (modus .eq. 1) then
                   do i=1, prevlnum - lnum + 1
                      itape = itape + 1
                   end do
                endif
                itape = itape + 2

              else
                 routine='Einmlbas(einsbasmod.f90)'
                 message='Error in ML-tree. lnum must be .le. prevlnum+1'
                 call errormsg

             end if
             
          elseif(itape==1) then
! top layer - do not write to tape
             i=1
             read(keyword(1),*,err=888) lnum
             node=node+1
             if(lnum .ne. 0) then
                routine='einmlbas'
                message='The top node must be given the number zero, ' &
                     //'not: '//keyword(1)(1:6)
                call errormsg
             endif
          end if
          
          ! Write mode-info to tape
          nc=0
          if(keyword(3) == '[') then  ! primitive basis found

             if (keyword(ic) .eq. ']') then
                nc = ic 
             else if (keyword(ic-1) .eq. ']') then
                nc = ic - 1
             else
                routine='Einmlbas'
                write(message,'(a,i4)') &
                        'No ] in ML-MCTDH definition, line:',iz
                call errormsg
             end if
! Flag which node is the electronic basis, and check it is in the top
! layer.
             if (keyword(4) .eq. 'el') then
                if (lnum .ne. 1) then
                   routine='Einmlbas'
                   write(message,'(a,i4)') &
               'The electronic basis must be in the top layer. Not:',lnum
                   call errormsg
                endif

                elnode = nbranch(1)
             endif

             m  = m  + 1
             mt = mt + 1
             nsmodes = nc - 4
             nspfdof(m) = nsmodes
             if (modus .eq. 2 .or. modus .eq. 3) tape(itape) = nsmodes
             itape = itape + 1
             p = 1
             n = 0

!---------------------------------------------------------------------------
! get dim(m,s) from ndim of node above primitive layer
!---------------------------------------------------------------------------
             if (modus.eq.2) then
                if (lnum>=1) then
                   b = nbranch(lnum)  ! this mode is the b-th branch of 
                                      ! the one above
                   dim(m,1) = dimbuffer(b,lnum-1)
                   if (ldynspf) maxdimspwn(m,1) = mdimbuffer(b,lnum-1)
                end if
             endif

             do i=4,nc-1  ! process each label
                f=0
                do f1=1,ndof
                   if (keyorig(i)(1:12) .eq. modelabel(f1) ) f=f1
                enddo
                if(f==0) then
                   routine = 'Einmlbas (einsbasmod.f90)'
                   message = 'in multilayer section label not found: '
                   call wrline(i,iz,iz0)
                   cycle
                end if
                n = n + 1
                if (modus .eq. 2) then
! for GWP modes, ndim is no. of GWPs. Else no. of grid points.
                   if (gwpm(m)) then
                      if (keyword(ic-1) == ']') then
                         read(keyword(ic),*,err=888) gwpdim(m,1)
                      else
                         gwpdim(m,1) = dim(m,1)
                      endif
                      tape(itape) = gwpdim(m,1)
                   else if (dentype .eq. 1) then
                      tape(itape) = gdim(f)*gdim(f)
                   else
                      tape(itape) = gdim(f)
                   endif
                endif
                itape = itape + 1
                spfdof(n,m)=f
                if(dofspf(f).eq.0) then
                   dofspf(f)=m
                else
                   message = 'doubly defined spfbasis: '
                   call wrline(i,iz,iz0)
                endif
             end do
             lprim=.true.

!---------------------------------------------------------------------------
! For DynWF, tape wants max SPFs, itape3 start SPFs
!            mdimbuffer max SPFS, dimbuffer start SPFs
!---------------------------------------------------------------------------
             
          else  ! non-primitive basis, just copy numbers into tape
! first look to see if max spfs specified for DynWF
             ic0=ic
             do i=3,ic
                if (keyword(i) .eq. '<') ic0 = i-1
             enddo

             mt = mt + 1
             nsmodes = ic0 - 2
             if (modus .eq. 2 .or. modus .eq. 3) tape(itape) = nsmodes
             itape = itape + 1
             proddim = 1
             lprim=.false.
             do i=3,ic0
                if (modus .eq. 2) then
                   read(keyword(i),*,err=888) tape(itape)
!CDWF              For dynamical WF:
                   if (ldynspf) then
                      itape2(ncount)=itape
                      itape3(ncount)=min(ndynspf0,tape(itape))
                      dimbuffer(i-2,lnum) = itape3(ncount)
                      mdimbuffer(i-2,lnum) = tape(itape)
                   else 
                      dimbuffer(i-2,lnum) = tape(itape)
                   endif
                   ncountbuffer(i-2,lnum)=ncount
                   ncount=ncount+1

                   proddim=proddim*tape(itape)   ! for DynWF, max proddim
                endif
                itape = itape + 1
             end do

! if max spf data present (DynWF calculation), now put that into tape
             if (ldynspf .and. (modus .eq. 2) .and. (ic0 .lt. ic)) then
                if ((ic-(ic0+2)) .ne. (ic0-3)) then
                   routine = 'EinMLbas'
                   write(message,'(a,i5)') &
                       'Not enough maximum SPFs defined for node : ',node
                   call warnmsg
                endif
! take count back to start of node SPF data
                ncount = ncount-nsmodes
                itape = itape-nsmodes
                do i=ic0+2,ic
!  store previously read data as start SPFs in itape3 and dimbuffer
                   itape3(ncount)=tape(itape)
                   dimbuffer(i-1-ic0,lnum) = tape(itape)

!  update tape with max SPFs 
                   read(keyword(i),*,err=888) tape(itape)
                   mdimbuffer(i-1-ic0,lnum) = tape(itape)

                   ncount=ncount+1
                   itape = itape+1
                enddo
             endif

          end if

! Check no. of SPFs for node
          if(modus .eq. 2.) then
             if(lnum.eq.0) then
                do i=1,nsmodes
                   if(dimbuffer(i,lnum)**2 .gt. proddim) then
                      routine = 'einmlbas'
                      message = 'Redundant SPFs in top layer.'
                      call wrline(0,iz,iz0)
                   endif
                enddo
             else
                do i=1,nsmodes
                   if (.not.lprim .and. dimbuffer(i,lnum)**2 .gt. &
                        dimbuffer(nbranch(lnum),lnum-1)*proddim) then
                      write(6,'(/a,i4,a,i4)') 'ERROR in einmlbas. itape =', &
                          itape,',  node =',node
                      write(6,'(a,i4,a,i4,a,i6,a,i8)') &
                          ' i =',i,', lnum =',lnum,', dimbuffer(i,lnum) =', &
                            dimbuffer(i,lnum), &
                          ', dimbuffer**2 =',dimbuffer(i,lnum)**2
                      write(6,'(a,i4,a,i7,/a,i4,a,i8)') &
                         'nbranch(lnum) =',nbranch(lnum), &
                         ',  proddim =',proddim, &
                         'dimbuffer(nbranch,lnum-1) =', &
                          dimbuffer(nbranch(lnum),lnum-1), &
                         ',  dimbuffer*proddim =', &
                         dimbuffer(nbranch(lnum),lnum-1)*proddim
                      routine = 'einmlbas'
                      message = 'Redundant SPFs in layer: '
                      call wrline(1,iz,iz0)
                   endif
                enddo
             endif
             if (lnum .gt. 0 ) then
                if(proddim .lt. dimbuffer(nbranch(lnum),lnum-1)) then
                   write(6,'(/a,i4,/i5,a,i4)') ' ERROR in node =',node, &
                        proddim, ' .lt.',dimbuffer(nbranch(lnum),lnum-1)
                   routine = 'einmlbas'
                   message = 'Product of in-SPFs of a node must be '// &
                             'larger than number of out-SPFs.      '// &
                        '     (in/out : botton to top)'
                   call wrline(0,iz,iz0)
                endif
             endif
          endif
          
          ! Read in next line
          call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
          message = inptit
          call cntrl(keyword(1),lc(1),ierr)
        end do

!-----------------------------------------------------------------------
! check consistency of layer definitions
!-----------------------------------------------------------------------
        do i=1,nlayers
           if (nlbranch(i,1) .ne. nlbranch(i,2)) then
              routine="EinMLbas"
              write(message,'(a,i5)') &
  "No. of branches defined does not match the number speficied. Layer :",i
              call warnmsg
           endif
        enddo
        if (lwarn) stop

        nmode   = m
        nmlmode = mt - 1  ! top node does NOT count as a mode

        if (modus .eq. 2 .or. modus .eq. 3) then
           tape(itape) = -2
!CDWF   For dynamical WF:
           if (ldynspf) itape2(ncount)=-2
        endif
        itape = itape + 1

!-----------------------------------------------------------------------
! set maxtape 
!-----------------------------------------------------------------------
        if (modus .eq. 1) maxtape = itape+2

!-----------------------------------------------------------------------
! check electronic basis
!-----------------------------------------------------------------------
        if (modus .eq. 1 .and. feb.gt.0) then
           if (gdim(feb).gt.1) then
              ! these variables are not used by ML-MCTDH, but they should
              ! be set to be consistent with single-set MCTDH -- FO
              meb = dofspf(feb)
              leb = .true.
           endif

           if (lebcont) then
              if (dofspf(febcont) .eq. 0) then
                 routine = 'EINMLBAS (einsbasmod.f90)'
                 write(message,'(a)') &
         'The continuum electronic basis must be added to top layer of tree '
                 call errormsg
              endif
           endif
        endif

        if (modus .eq. 2 .and. meb.gt.0) then
!-----------------------------------------------------------------------
! Set correct no. of SPFs for feb in a dynamicWF
! as they are in the first layer, Nspf is stored in tape at node no.+1 
! (firat position is no. of nodes in top layer) and itape3 at the node no.
!
! If only 2 nodes in top layer, they must have same no. of SPFs
!-----------------------------------------------------------------------
           if (ldynspf .and. (meb .gt. 0)) then
              if (dentype .eq. 1) then
                 gdim1 = gdim(feb)*gdim(feb)
              else
                 gdim1 = gdim(feb)
              endif
              dim(meb,1) = gdim1
              maxdimspwn(meb,1) = gdim1
              if (nbranch(1) .eq. 2) then
                 tape(2:3) = gdim1
                 itape3(1:2) = gdim1
              else
                 tape(elnode+1) = gdim1
                 itape3(elnode) = gdim1
              endif
           endif

!-----------------------------------------------------------------------
! Check for correct no. of functions
!-----------------------------------------------------------------------
           if (gdim(feb).gt.1) then
              if (dentype .eq. 0 .or. dentype .eq. 2) then
                 if (dim(meb,1) .ne. gdim(feb)) then
                    routine = 'EINMLBAS (einsbasmod.f90)'
                    write(message,'(a,i5)') &
         'The no. of electronic SPF must equal the no. of states :',gdim(feb)
                    call errormsg
                 endif
              else if (dentype .eq. 1) then
                 if (dim(meb,1) .ne. gdim(feb)*gdim(feb)) then
                    routine = 'EINMLBAS (einsbasmod.f90)'
                    write(message,'(a,i5)') &
         'The no. of electronic SPDO must equal the no. of states**2 :',gdim(feb)*gdim(feb)
                    call errormsg
                 endif
              endif

           endif

!-----------------------------------------------------------------------
! define nddstate as the actual number of electronic states
!-----------------------------------------------------------------------
           if (lebcont) then
              nddstate=eltot
           else
              nddstate=gdim(feb)
           endif

        endif

!-----------------------------------------------------------------------
! see if any modes are multi-modes
!-----------------------------------------------------------------------
        do m=1,nmode
           if (nspfdof(m) .gt. 1) lconm = .true.
        enddo

!-----------------------------------------------------------------------
! calculate grid data
!-----------------------------------------------------------------------
      call griddat

!-----------------------------------------------------------------------
! check that dim .le. subdim
!-----------------------------------------------------------------------
      if (modus.eq.2) then
         do m=1,nmode
            if (gwpm(m)) cycle
            if (dim(m,1) .gt. subdim(m) ) then
               routine = "Einmlbas"
               message = 'Too many spf s for mode:             '
               write(message(25:28),'(i4)') m
               call warnmsg
            end if
         enddo

!-----------------------------------------------------------------------
! For dentype = 1 copy no. of SPDOs etc. to relevant arrays
!-----------------------------------------------------------------------
         if (dentype .eq. 1) then
            do m= 1,nmode
               ddim(m,1,1) = dim(m,1)
            enddo
         endif
     
         if (ldynspf) then
            tapeM=tape
            i = 1
            do while (itape2(i) .ne. -2) 
               tape(itape2(i)) = itape3(i)
               i=i+1
            enddo
         endif

      endif

      deallocate(nbranch,dimbuffer,nlbranch)

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr
      lwarn=.false.

      return

888     write(6,'(/a)') '### ERROR in ML-Basis-Section ###'
        write(6,'(a,a6)') 'Expecting integer, not: ',keyword(i)
        write(6,'(a)') 'The wrong line reads:'
        write(6,'(a72)') inptit
        write(ilog,'(/a)') '### ERROR in ML-Basis-Section ###'
        write(ilog,'(a,a6)') 'Expecting integer, not: ',keyword(i)
        write(ilog,'(a)') 'The wrong line reads:'
        write(ilog,'(a72)') inptit
        stop

      end subroutine einmlbas

!########################################################################
! 
! If this is an ML calculation and the operator was generated by a 
! non-ML input, now need to set-up tape
! 
!########################################################################

      subroutine genmlgrd

      implicit none

      integer(long)             :: iz,ic,ierr,ilbl,chkdvr
      logical(kind=4)           :: lerr,check = .false.
      logical(kind=4)           :: lgen = .false.
      character(len=c5)         :: filename
      integer(long), external   :: integersize

      inquire(iin,opened=check)
      if (check) close(iin)
      ilbl=inplaenge
      filename=inputfile(1:ilbl)
      open(iin,file=filename,status='old',form='formatted',err=900)

!-----------------------------------------------------------------------
! read spf-basis to see if ML
!-----------------------------------------------------------------------
      rewind(iin)
      iz=0
      readiin: do

         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .gt. 0) then
            routine = 'Einsbas'
            message = inptit
            call errormsg
         endif
         
         if (keyword(1) .eq. 'multilayer-basis-section' .or. &
              keyword(1) .eq. 'mlbasis-section'  .or. &
              keyword(1) .eq. 'ml-basis-section') then

            if (maxtape .eq. 1) lgen = .true.

            exit
            
         else if (keyword(1) .eq. 'end-input') then 
            exit
         endif

      enddo readiin

      if (lgen) then
!-----------------------------------------------------------------------
! generate tape dimensions
! 1. read modelabels
! 2. read tape
!-----------------------------------------------------------------------
         inquire(idvr,opened=check)
         if (check) close(idvr)
         filename=dname(1:dlaenge)//'/dvr'
         ilbl=index(filename,' ')-1
         open(idvr,file=filename,form='unformatted',status='old',err=900)
         chkdvr=1
         call dvrinfo(lerr,chkdvr)
         close(idvr)

!-----------------------------------------------------------------------
! 2. read tape
! reallocate tape array
!-----------------------------------------------------------------------
         allocmemory=allocmemory - maxtape*integersize()

         rewind(iin)
         call eingrd(1)

         if (allocated(tape)) deallocate(tape)
         allocate(tape(maxtape))
         allocmemory=allocmemory + maxtape*integersize()

      endif

      return
  900 continue
      routine='Runinwf'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg

      end subroutine genmlgrd

!#######################################################################

      subroutine uncombine

      implicit none

      integer(long)       :: m,f,n,s,m1

!-----------------------------------------------------------------------
! un-combine combined modes
!-----------------------------------------------------------------------
      m1=nmode+1
      do m=1,nmode
         if (nspfdof(m) .gt. 1) then
            do n=2,nspfdof(m)
               f=spfdof(n,m)
               spfdof(n,m)=0

               spfdof(1,m1)=f
               dofspf(f)=m1
               m1=m1+1
            enddo
         endif
      enddo
      nmode=m1-1

      do m=1,nmode
         nspfdof(m)=1
      enddo

!-----------------------------------------------------------------------
! set dim to 1 for all modes
!-----------------------------------------------------------------------
      do s=1,nstate
         do m=1,nmode
            dim(m,s)=1
         enddo
      enddo

      return
      end subroutine uncombine

!***********************************************************************
! HDM 06/00

      subroutine logsbas

      implicit none

      integer(long)           :: n,m,s,t,i,nst,aprd,asum,psum,nm
      real(dop)               :: totgrd,asumr,psumr,aprdr,lgtotgrd
      character(len=32)       :: string

      if (dentype .eq. 1) then
         write(ilog,'(//a/)') '********* SPDO Basis ***********'
      else
         write(ilog,'(//a/)') '********* SPF Basis ************'
      endif
      asumr=0
      psumr=0

      totgrd = 1.0_dop
      lgtotgrd = 0.0_dop
      if(dentype .eq. 1 .and. nstate .eq. 1) then
         aprd = 1
         psum = 0
         aprdr= 1.d0
         psumr= 0.d0
         write(ilog,'(a/)') " mode     N    n      dof's"
         do m = 1, nmode
            write(ilog,901) m, subdim(m), ddim(m,1,1),&
                 (spfdof(n,m),n=1,nspfdof(m))
            aprd = aprd*ddim(m,1,1)
            psum = psum + subdim(m)*ddim(m,1,1)
            aprdr = aprdr*ddim(m,1,1)
            psumr = psumr + subdim(m)*ddim(m,1,1)
            totgrd = totgrd*subdim(m)
            lgtotgrd = lgtotgrd + log10(dble(subdim(m)))
         end do
         asum = aprd
         asumr= aprdr
      else if (dentype .eq. 1) then
         nst=min(nstate,3)
         asum = 0
         psum = 0
         asumr= 0.d0
         psumr= 0.d0
         write(string,'(9(a1,i1,1x))') ('n',i, i=1,nst)
         write(ilog,'(3a/)')' mode     N   ',string(1:3*nst),"    dof's"
         do m = 1, nmode
            write(string,'(9i3)') ((ddim(m,s,t),t=1,nst),s=1,nst)
            write(ilog,902) m, subdim(m), string(1:3*nst), &
                 (spfdof(n,m),n=1,nspfdof(m))
            totgrd = totgrd*subdim(m)
            lgtotgrd = lgtotgrd + log10(dble(subdim(m)))
         end do
         do s = 1, nstate
            do t=1,nstate
               aprd = 1
               aprdr=1.d0
               do m = 1, nmode
                  aprd = aprd*ddim(m,s,t)
                  aprdr = aprdr*ddim(m,s,t)
                  psum = psum + subdim(m)*ddim(m,s,t)
                  psumr = psumr + subdim(m)*ddim(m,s,t)
               end do
               asum = asum + aprd
               asumr= asumr+ aprdr
            enddo
         end do
      else if(dentype .ne. 1 .and. nstate .eq. 1) then
         aprd = 1
         psum = 0
         aprdr= 1.d0
         psumr= 0.d0
         write(ilog,'(a/)') " mode     N    n      dof's"
         do m = 1, nmode
            nm=min(nspfdof(m),15)
            write(ilog,901) m, subdim(m), dim(m,1),(spfdof(n,m),n=1,nm)
            if (nspfdof(m) .gt. nm) then
               write(ilog,908) (spfdof(n,m),n=nm+1,nspfdof(m))
            endif
            aprd = aprd*dim(m,1)
            aprdr= aprdr*dim(m,1)
            psum = psum + subdim(m)*dim(m,1)
            psumr= psumr+ subdim(m)*dim(m,1)
            totgrd = totgrd*subdim(m)
            lgtotgrd = lgtotgrd + log10(dble(subdim(m)))
         end do
         asum = aprd
         asumr= aprdr
      else if(dentype .ne. 1) then
         nst=min(nstate,8)
         asum = 0
         asumr= 0.d0
         psum = 0
         psumr= 0.d0
         write(string,'(9(a1,i1,1x))') ('n',i, i=1,nst)
         write(ilog,'(3a/)')' mode     N   ',string(1:4*nst),"    dof's"
         do m = 1, nmode
            write(string,'(9i3)') (dim(m,s),s=1,nst)
            write(ilog,902) m, subdim(m), string(1:4*nst), &
                 (spfdof(n,m),n=1,nspfdof(m))
            totgrd = totgrd*subdim(m)
            lgtotgrd = lgtotgrd + log10(dble(subdim(m)))
         end do
         do s = 1, nstate
            aprd = 1
            aprdr=1.d0
            do m = 1, nmode
               aprd = aprd*dim(m,s)
               aprdr= aprdr*dim(m,s)
               psum = psum + subdim(m)*dim(m,s)
               psumr= psumr+ subdim(m)*dim(m,s)
            end do
            asum = asum + aprd
            asumr= asumr+ aprdr
         end do
      endif

      if (dentype.eq.2) then
         asum = asum*asum
         asumr = asumr*asumr
      endif
      if(totgrd .gt. HUGE(1.d0)*0.99d0 ) then
         write(ilog,'(/a,f9.3)')' Total grid size       : 10^',lgtotgrd
      else if(totgrd .gt. 9.9d7 ) then
         write(ilog,'(/a,1pe13.6)')' Total grid size       :',totgrd
      else
         write(ilog,'(/a,i9,1pe14.3)') &
              ' Total grid size       :',nint(totgrd),totgrd
      endif

      call myflush(ilog)
      if(.not.lrunml .and. abs(dble(asum)-asumr) .gt. 1.d0) then
         routine = 'logsbas (mctdh/einsbas.F)'
         message = ' A-vector size is too large!'
         call errormsg
      endif
      if(abs(dble(psum)-psumr) .gt. 1.d0) then
         routine = 'logsbas (mctdh/einsbas.F)'
         message = ' Phi-vectors size is too large!'
         call errormsg
      endif

 901  format(1x,i4,i9,i5,4x,12i4)
 902  format(1x,i4,i9,2x,a,3x,12i4)
 908  format(20x,15i4)

      return
      end subroutine logsbas

!#######################################################################

      subroutine chkbasis

      implicit none

      integer(long)   :: i,m,m1,s,t,plen
      real(dop)       :: fac
      logical         :: lerr = .false.

      type(wfnode_t),pointer   :: cnode,snode

!-----------------------------------------------------------------------
! Check enough grid points for SPFs. I.e. dim .le. subdim. 
!-----------------------------------------------------------------------
      if (lrunml) then
! ignore top node - no restrictions.
         do i=2,tree%tot_nodes
            cnode => tree%swap_down(i)%p

            plen = product(cnode%ndim)
            if (cnode%isprimitive) then
! Use primitive basis data.
               m=cnode%m
               if (gwpm(m)) cycle
               if (dentype .eq. 1) then
                  if (ddim(m,1,1) .gt. subdim(m)) then
                     routine = 'Chkbasis (Einsbasmod.f90) '
                     write(message,'(3(a,i5))') &
                        'Too many SPDOs for node :', cnode%num, &
                        ' Layer :', cnode%layer, &
                        ' Reduce from ',cnode%navecs
                     call warnmsg
                     lerr = .true.
                  endif
               else
                  if (dim(m,1) .gt. subdim(m)) then
                     routine = 'Chkbasis (Einsbasmod.f90) '
                     write(message,'(3(a,i5))') &
                        'Too many SPFs for node :', cnode%num, &
                        ' Layer :', cnode%layer, &
                        ' Reduce from ',cnode%navecs
                     call warnmsg
                     lerr = .true.
                  endif
               endif
            else if (cnode%navecs .gt. plen) then
               routine = 'Chkbasis (Einsbasmod.f90) '
               write(message,'(3(a,i5))') &
                        'Too many SPFs for node :', cnode%num, &
                        ' Layer :', cnode%layer, &
                        ' Reduce from ',cnode%navecs
               call warnmsg
               lerr = .true.
            endif
         enddo

      else if (dentype .eq. 1) then
         do s=1,nstate
            do t=1,nstate
               do m=1,nmode 
                  if (gwpm(m)) cycle
                  if (ddim(m,s,t) .gt. subdim(m)) then
                     routine = 'Chkbasis (Einsbasmod.f90) '
                     write(message,'(a,2i5)') 'Too many SPDOs for mode, state :', m,s
                     call warnmsg
                     lerr = .true.
                  endif
               enddo
            enddo
         enddo

      else
         do s=1,nstate
            do m=1,nmode
               if (gwpm(m)) cycle
               if (dim(m,s) .gt. subdim(m)) then
                  routine = 'Chkbasis (Einsbasmod.f90) '
                  write(message,'(a,2i5)') 'Too many SPFs for mode, state :', m,s
                  call warnmsg
                  lerr = .true.
               endif
            enddo
         enddo

      endif

      if (.not. lredundant) go to 200
!-----------------------------------------------------------------------
! Check for redundant configurations.
!-----------------------------------------------------------------------
      if (lrunml) then
         cnode => tree%swap_down(1)%p
         plen = product(cnode%ndim)

         fac = 1.0d0 + 1.0d-12  ! 1^-12 added to avoid round-off
         do m = 1, cnode%nmodes
            fac = fac*cnode%ndim(m)
         enddo

         if(fac .gt. 1.0d9) then ! integer overflow
            write(message,'(a,e12.4)') &
              'Too many configurations! A-vector length =',fac
            call warnmsg
            lerr = .true.
         endif
         do m = 1, cnode%nmodes
            snode => tree%submodes(m)%p
            if (snode%isprimitive) then
               m1 = snode%m
! GWPs can be overcomplete so do not check
               if (gwpm(m1)) cycle
            endif

            if( cnode%ndim(m)**2 .gt. fac) then
               routine = 'Chkbasis (Einsbasmod.f90) '
               write(message,'(2a,i3)') &
                   'Redundant configurations in top node. ',&
                   'Reduce SPFs for nmode mode : ',m
                  
               call warnmsg
               lerr = .true.

               write(6,'(2a,i4,/a,/2a)')  &
                  ' Decrease the number of SPFs', &
                  ' of the above mode to ', int(fac)/cnode%ndim(m), &
                  ' or increase the other numbers of SPFs accordingly.',&
                  ' To suppress this check one may set',&
                  ' "no-redundancy-check" in Init_wf-Section.'
               write(ilog,'(2a,i4,/a,/2a)')  &
                  ' Decrease the number of SPFs', &
                  ' of the above mode to ', int(fac)/cnode%ndim(m), &
                  ' or increase the other numbers of SPFs accordingly.',&
                  ' To suppress this check one may set',&
                  ' "no-redundancy-check" in Init_wf-Section.'
            end if
         enddo

      else if (dentype .eq. 0) then
         do s = 1, nstate
            fac = 1.0d0 + 1.0d-12  ! 1^-12 added to avoid round-off
            do m = 1, nmode
               fac = fac*dim(m,s)
            enddo
            if(fac .gt. 1.0d9) then ! integer overflow
               write(message,'(a,e12.4)') &
                 'Too many configurations! A-vector length =',fac
               call warnmsg
            endif
            do m = 1, nmode
               if (gwpm(m)) cycle
               if (m .eq. meb) cycle

               if( dim(m,s)**2 .gt. fac) then
                  routine = 'Chkbasis (Einsbasmod.f90) '
                  write(message,'(a,i3,a,i4)') &
               'Redundant configurations.  State =', s,', Mode =',m
                  call warnmsg
                  lerr = .true.

                  write(6,'(2a,i4,/a,/2a)')  &
                     ' Decrease the number of SPFs', &
                     ' of the above mode to ', int(fac)/dim(m,s), &
                     ' or increase the other numbers of SPFs accordingly.',&
                     ' To suppress this check one may set',&
                     ' "no-redundancy-check" in Init_wf-Section.'
                  write(ilog,'(2a,i4,/a,/2a)')  &
                     ' Decrease the number of SPFs', &
                     ' of the above mode to ', int(fac)/dim(m,s), &
                     ' or increase the other numbers of SPFs accordingly.',&
                     ' To suppress this check one may set',&
                     ' "no-redundancy-check" in Init_wf-Section.'
               end if
            enddo
         enddo

      elseif ( dentype .eq. 1 ) then
         do s=1,nstate
            do t=1,nstate
               fac = 1
               do m=1,nmode
                  fac = fac*ddim(m,s,t)
               enddo
               do m=1,nmode
                  if (gwpm(m)) cycle
                  if (m .eq. meb) cycle

                  if (ddim(m,s,t)**2 .gt. fac) then
                     routine = 'Chkbasis (Einsbasmod.f90) '
                      write(message,'(a,i3,a,i3,a,i4)') &
                          'Redundant configurations. State =', &
                           s,',',t,', Mode =',m
                      call warnmsg
                      lerr =.true.

                      write(6,'(2a,i4,/a)')  &
                        ' Decrease the number of SPFs', &
                        ' of the above mode to ', int(fac)/ddim(m,s,t), &
                        ' or increase the other numbers of SPFs accordingly.'
                      write(ilog,'(2a,i4,/a)')  &
                        ' Decrease the number of SPFs', &
                        ' of the above mode to ', int(fac)/ddim(m,s,t), &
                        ' or increase the other numbers of SPFs accordingly.'
                  endif
               enddo
            enddo
         enddo

      else if (dentype .eq. 2) then
         do s = 1, nstate
            fac = 1.0d0 + 1.0d-12  ! 1^-12 added to avoid round-off
            do m = 1, nmode
               fac = fac*dim(m,s)
            enddo
            if(fac .gt. 1.0d9) then ! integer overflow
               write(message,'(a,e12.4)') &
                 'Too many configurations! A-vector length =',fac
               call warnmsg
            endif
         enddo

      endif
  200 continue

!----------------------------------------------------------------------- 
! If using complete electronic basis, 
! Check enough configurations for no. of states
!----------------------------------------------------------------------- 
      if ((leb .and. psitype .eq. 0) .and. .not. lebonly) then 
         if (dim(meb,1) .eq. gdim(feb)) then
            fac = fac / dble(gdim(feb))

            if (fac .lt. gdim(feb)) then
               routine = 'Chkbasis (Einsbasmod.f90) '
               write(message,'(a,i3,a,i4)') &
               'Not enough configurations: Need at least as many as Nstate: ', &
                nddstate
               call warnmsg
            endif
         endif
      endif

      if (lerr) then
         routine = 'Chkbasis (Einsbasmod.f90) '
         write(message,'(a,2i5)') 'ERROR: Problem in generating wavefunction.'
         call errormsg
      endif

      return
      end subroutine chkbasis

!#######################################################################

      end module einsbasmod

