!######################################################################

      module expectval

use decimal, only: dop,long
use logdat
use xvlib,   only: xvxxzza, vvaxzz, xvxxzzo
use mmlib,   only: mmaxzz, msqzz, mmxxzz
use mtlib,   only: qtxxzz
use d1tens,  only: ptr1,ptrc
use op1lib,  only: zerovxz, zeromxd, sum2vxz, cpvxz, cpmxz, trqxz, &
                   trhxz, unitqxd
use op2lib,  only: swapsxxi, addvxxzo

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod
use dirdyn
use maxv, only: maxdim,maxsta
use lctmod
use potevalmod
use hphimod, only: hphi,hunphi,hunphi1ms,hphi1mk
use phihphimod, only: phihphi,phihunphi
use d1hphimod, only: d1hphi,d1hunphi,d1hunphi1ms,d1hphi1mk
use calchamod, only: calcha,calchuna,calchuna_gwp,calcha_ml
use d2calchamod, only: d2calchaloch2,d2calchunaloch2, &
                 d2calchaloch2_ml,d2calchunaloch2_ml
use gwplib, only: gwp_param,gwp_overlap
use gh_elementsmod, only: gh_elements
use runinwfmod
use d1lib, only: trspdoc, trspdo
use runpropmod
use eqofmotion1, only: compute_hterms_treer
use mlop_eom, only: compute_all_uterms_tree
use wftree1, only: allocate_mtraces_matrices,deallocate_mtraces_matrices,&
                  allocate_zmtraces_matrices
use mlanalysis, only: compute_mtraces_tree
use multilayer, only: traceml,ztraceml

use mpidata

      implicit none

      private
      public :: expectvalue,exexpectvalue,d1expectvalue,d2expectvalue,&
                wfexpectvalue,trexpectvalue,calcenrho,calceuncorr1,&
                calcecorr1,calcspop1
      public :: lnwexpect

      logical(kind=4), save :: lnwexpect = .false.
      contains

!######################################################################
!
!                      MCTDH module EXPECTVAL
!
!     SUBROUTINE TO CALCULATE THE EXPECTATION VALUES
!
!  10/2000 CC        
!***********************************************************************

      subroutine expectvalue (psi,time,operator,result,hops,hpsi,hteil,&
                 jindx,ort,agmat,gfgwp,aigwp,zgwp,qcenter,gh,gs2,hung,&
                 gsval,gsvek,ghderiv1)

      integer(long)                                  :: n,nham
      integer(long), dimension(jindxdim), intent(in) :: jindx
      real(dop), intent(in)                          :: time
      real(dop), dimension(ortdim), intent(in)       :: ort
      real(dop), dimension(hopsdim), intent(in)      :: hops
      real(dop), dimension(qcentdim), intent(out)    :: qcenter
      real(dop), dimension(gwpinfodim), intent(out)  :: gsval
      real(dop)                                      :: resim,resre
      real(dop), dimension(ghder1dim),intent(out)    :: ghderiv1
      complex(dop), dimension(pmatdim), intent(out)  :: hteil
      complex(dop), intent(out)                      :: result
      complex(dop)                                   :: aovl
      complex(dop), dimension(dgldim), intent(inout) :: psi
      complex(dop), dimension(hpsidim), intent(out)  :: hpsi
      complex(dop), dimension(agmatdim),intent(out)  :: agmat
      complex(dop), dimension(gfgwpdim),intent(out)  :: gfgwp
      complex(dop), dimension(aigwpdim),intent(out)  :: aigwp
      complex(dop), dimension(zgwpdim),intent(out)   :: zgwp
      complex(dop), dimension(ghmatdim),intent(out)  :: gh
      complex(dop), dimension(gs2matdim),intent(out) :: gs2
      complex(dop), dimension(gsmatdim),intent(out)  :: gsvek
      complex(dop), dimension(hungdim),intent(out)   :: hung
      logical(kind=4)                                :: lmpicalcha_bak,&
                                                        lmpiphihphi_bak

      character(len=c2), intent(in)                  :: operator

!-----------------------------------------------------------------------
! Set initial paramters
!-----------------------------------------------------------------------
      routine  = 'expectation (expectation values)'
      nham = 0
      lexpectval=.true.

!-----------------------------------------------------------------------
! In case we use MPI make change parallel evaluation accoding to inp
! Note: lmpiexpect is not known in slaves. This works by posting
! mpicalcha or mpiphihphi in master
!-----------------------------------------------------------------------
      lmpicalcha_bak  = lmpicalcha
      lmpiphihphi_bak = lmpiphihphi
      lmpicalcha = lmpicalcha .and. lmpiexpect
      lmpiphihphi = lmpiphihphi .and. lmpiexpect

!-----------------------------------------------------------------------
! Get operator number
!-----------------------------------------------------------------------
      do n=1,hamdim
         if (hamlab(n) .eq. operator) nham=n
      enddo
      if (nham.eq.0) then
         message = 'operator not defined in operator-file: '//operator
         call errormsg
      endif
      
!-----------------------------------------------------------------------
! calculate time-dependent coefficients. No LCT required.
!-----------------------------------------------------------------------
      if (ltdh(nham)) call calctdk_nolct(time,nham)

!-----------------------------------------------------------------------
! workspace divided into 3. 1. dtpsi, 2. hteil, 3. work
! Call special routine according to dentype
!-----------------------------------------------------------------------
      if (dentype.eq.0) then                        ! Wavefunction
         call wfexpectvalue(psi,nham,&      ! input
              result, &                     ! output
              hpsi,hteil,&                  ! hpsi
              hops,jindx,&                  ! various other input 
              ort,&
              agmat,gfgwp,aigwp,zgwp,qcenter,gh,ghderiv1,gs2,&  ! GWP info
              hung,time,gsval,gsvek,aovl)        ! GWP info

      else if (dentype.eq.1) then                 ! Density Type I
         call d1expectvalue(psi,nham,&            ! input
              result,&                            ! output
              hpsi)                               ! dtpsi,hpsi,hunpsi

      else if (dentype.eq.2) then                 ! Density Type II
         call d2expectvalue(psi,nham, &           ! input
              result,&                            ! output
              hpsi,hteil,&                        ! dtpsi,hpsi,hunpsi
              agmat,gfgwp,aigwp,zgwp,qcenter,gh,ghderiv1,gs2,&  ! GWP info
              hung,time,gsval,gsvek)              ! GWP info

      endif

!
! remove underflow
!
      resim = dimag(result)
      resre = dble(result)
      if (dabs(resim) .lt. 10.0*macheps) resim=0.0_dop
      if (dabs(resre) .lt. 10.0*macheps) resre=0.0_dop
      result=dcmplx(resre,resim)

!-----------------------------------------------------------------------
! restore params for propagation
!-----------------------------------------------------------------------
      lexpectval = .false.
      lmpicalcha  = lmpicalcha_bak
      lmpiphihphi = lmpiphihphi_bak

      return
      end subroutine expectvalue

!*************************************************************************
!     calculates the expecatation value for a trajectory based wavepacket
!*************************************************************************

      subroutine trexpectvalue(psi,operator,result,trajst,dim1,gdof)


      implicit none

      integer(long), intent(in)                      :: dim1,gdof
      integer(long)                                  :: nham
      integer(long), dimension(dim1), intent(in)     :: trajst
      complex(dop), dimension(gdof,dim1), intent(in) :: psi 
      complex(dop), intent(out)                      :: result

      complex(dop)                                   :: vz
      real(dop), dimension(maxdim)                   :: q0,p0
      real(dop)                                      :: rfac,v
      real(dop),dimension(maxsta,maxsta)             :: wmat,rotmat,wmat1,vmat1
      real(dop),dimension(maxsta)                    :: vmat
      real(dop),dimension(3*maxsta)                  :: workr

      integer(long) :: s,m,si,sf,nsta,n 
      integer(long) :: pottyp,k,h,e,f,k1,k2,s1,s2,s3,ierr

      character(len=c2), intent(in)                  :: operator
! This flag must be set if operator is in the diabatic picture and has to be 
! transformed
      logical, dimension(maxham) :: ldiabexp

      routine = 'trexpectvalue'

!-----------------------------------------------------------------------
! Set initial paramters
!-----------------------------------------------------------------------
      routine  = 'trexpectation (expectation values)'
      nham = 0
      lexpectval=.true.
      si=0
      sf=0
      ldiabexp(:) = .false.

!-----------------------------------------------------------------------
! Get operator number
!-----------------------------------------------------------------------
      do n=1,hamdim
         if (hamlab(n) .eq. operator) nham=n
      enddo
      if (nham.eq.0) then
         message = 'operator not defined in operator-file: '//operator
         call errormsg
      endif
      
      result = 0.0d0
      if (feb .gt. 0) then
         nsta = gdim(feb)
      else
         nsta = 1
      endif
      k1=zham(nham)
      k2=zham(nham)+khtot(nham)-1
      m=1

!-----------------------------------------------------------------------
! loop over Hamiltonian terms
!-----------------------------------------------------------------------
      do e=1,dim1
         
!-----------------------------------------------------------------------
! get coordinates
!-----------------------------------------------------------------------
         do n=1,gdof
            f=spfdof(n,m)
            q0(f)=dble(psi(n,e))
            p0(f)=dimag(psi(n,e))
         enddo

         if (nsta .gt. 1 .and. ldiabexp(nham)) then
            call zeromxd(wmat,maxsta,maxsta)
! Get diabatic potential energy matrix
            do k=1,khtot(1)
               if (kinop(1,k) .eq. 0) then
! form product of factors in operator
                  rfac = 1.0d0
                  do n=1,nspfdof(m)
                     f=spfdof(n,m)
                     h=hamilton(f,k)
                     si=ksi(k)
                     sf=ksf(k)

                     call poteval (h,v,q0,vz,p0,pottyp)
                     rfac = rfac*v
                  enddo

                  wmat(sf,si) = wmat(sf,si) + dble(koeff(k)*rfac)
                  if (sf .ne. si) wmat(si,sf) = wmat(sf,si)
               endif
            enddo

! diagonalise wmat
            do s=1,nsta
               do s1=1,nsta
                  rotmat(s1,s)=wmat(s1,s)
               enddo
            enddo
            call dsyev('V','U',nsta,rotmat,maxsta,vmat,workr,3*nsta,ierr)
            if (ierr .ne. 0) then
               routine = 'Qdot1m'
               message = 'Cannot diagonalize Wmat matrix'
               call errormsg
            endif
         else
            call unitqxd(rotmat,nsta)
         endif

! get desired operator at q0,p0
         call zeromxd(wmat1,maxsta,maxsta)
         do k=k1,k2
            f=spfdof(1,m)
            h=hamilton(f,k)
            si=ksi(k)
            sf=ksf(k)

! form product of factors in operator
            rfac = 1.0d0
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               h=hamilton(f,k)
               si=ksi(k)
               sf=ksf(k)

               call poteval (h,v,q0,vz,p0,pottyp)
               rfac = rfac*v
            enddo

            wmat1(sf,si) = wmat1(sf,si) + dble(koeff(k)*rfac)
            if (sf .ne. si) wmat1(si,sf) = wmat1(sf,si)

         enddo

! transform operator to adiabatic picture 
         if (nsta .gt. 1) then
            call zeromxd(vmat1,maxsta,maxsta)
            do s=1,nsta
               do s1=1,nsta
                  do s2=1,nsta
                     do s3=1,nsta
                        vmat1(s1,s)=vmat1(s1,s) +         &
                             rotmat(s1,s2)*wmat1(s2,s3)*rotmat(s,s3)
                     enddo
                  enddo
               enddo
            enddo

            s = trajst(e)
         else
            vmat1(1,1) = wmat1(1,1)

            s = 1
         endif


         result = result + vmat1(s,s)
      enddo
      
      result = result / dble(dim1)

!-----------------------------------------------------------------------
! restore params for propagation
!-----------------------------------------------------------------------
      lexpectval = .false.

      return
      end subroutine trexpectvalue

!*************************************************************************
!     calculates the expecatation value of an operator; for wavefunctions
!*************************************************************************

      subroutine wfexpectvalue(psi,nham,result,hpsi,hteil,hops,jindx,&
                 ort,agmat,gfgwp,aigwp,zgwp,qcenter, gh,ghderiv1,gs2,hung,&
                 time,gsval,gsvek,aovl)  

      implicit none

      integer(long)                                  :: ktype,hktype,s,zeig,&
                                                        zeig1,zeig2,m,n
      integer(long), intent(in)                      :: nham
      integer(long), dimension(jindxdim), intent(in) :: jindx
      integer(long)                                   :: z0,zf

      real(dop), intent(in)                          :: time
      real(dop), dimension(ortdim), intent(in)       :: ort
      real(dop), dimension(hopsdim), intent(in)      :: hops
      real(dop), dimension(qcentdim), intent(out)    :: qcenter
      real(dop), dimension(gwpinfodim), intent(out)  :: gsval
      real(dop), dimension(ghder1dim),intent(out)    :: ghderiv1
      complex(dop)                                   :: sum
      complex(dop), intent(out)                      :: aovl
      complex(dop), intent(out)                      :: result
      complex(dop), dimension(maxdim)                :: workovl
      complex(dop), dimension(dgldim)                :: nwpsi
      complex(dop), dimension(dgldim)                :: lctworkc
      complex(dop), dimension(2*maxblock)            :: tmppsi
      complex(dop), dimension(dgldim), intent(inout) :: psi
      complex(dop), dimension(dgldim)                :: dtpsi
      complex(dop), dimension(hpsidim), intent(out)  :: hpsi
      complex(dop), dimension(totphidim)             :: hunpsi
      complex(dop), dimension(pmatdim),intent(out)   :: hteil
      complex(dop), dimension(agmatdim),intent(out)  :: agmat
      complex(dop), dimension(gfgwpdim),intent(out)  :: gfgwp
      complex(dop), dimension(aigwpdim),intent(out)  :: aigwp
      complex(dop), dimension(zgwpdim),intent(out)   :: zgwp
      complex(dop), dimension(ghmatdim),intent(out)  :: gh
      complex(dop), dimension(gs2matdim),intent(out) :: gs2
      complex(dop), dimension(hungdim),intent(out)   :: hung
      complex(dop), dimension(gsmatdim)              :: gsinv
      complex(dop), dimension(gsmatdim)              :: gsvek

      routine = 'wfexpectvalue'

      call zerovxz(dtpsi,dgldim)
!
! Overlap matrices help is gfgwp(gfgwp(m,s,s))
!
      if (lgwp) then
         call gwp_param(psi,agmat,gfgwp,aigwp,zgwp,qcenter,1)
         call gwp_overlap(gs2,gsinv,gsval,gsvek,gfgwp,aigwp,zgwp,0) 
         call gh_elements(gh,hung,ghderiv1,agmat,gfgwp,aigwp,zgwp,&
              qcenter,nham,1,gs2,time)
      endif                                                  

! Local control expectation values
      if (lctvibpop) then
! Zeroing parameters
         call zerovxz(vibpop,maxdim)
         do n=1,adim
            call zerovxz(nwpsi,dgldim)
            call zerovxz(workovl,maxdim)
            call zerovxz(lctworkc,dgldim)
! Setting up the projector
            do s=1,nstate
               do m=1,nmode
                 if (m .eq. nmlct .and. s .eq. nstalct) then
                   call msqzz(lctpsi(zetf(m,s)),lctworkc(zetf(m,s)),&
                   subdim(m),dim(m,s),n)
                 else
                   call cpmxz(psi(zetf(m,s)),lctworkc(zetf(m,s)),&
                   subdim(m),dim(m,s))
                 endif
                 if (leb .and. s .eq. nstalct .and. m .eq. nmode) then
                   call  msqzz(lctpsi(zetf(m,s)),lctworkc(zetf(m,s)),&
                   subdim(m),dim(m,s),nstalct)
                 endif
! Calculates <v|PSI> and then |v><v|PSI>
                 call mmaxzz(lctworkc(zetf(m,s)),psi(zetf(m,s)),&
                  workovl,subdim(m),dim(m,s),dim(m,s))
                 call mmxxzz(lctworkc(zetf(m,s)),workovl,&
                  nwpsi(zetf(m,s)),subdim(m),dim(m,s),dim(m,s)) 
               enddo
            enddo
! Calculates the expectation value       
            ktype=0
            call hphi(nwpsi,hpsi,ktype,nham)
            call phihphi(psi,gh,hpsi,hteil,nham,.false.) 
            call calcha(psi,dtpsi,hteil,hops,psi(zetf(1,1)),nham,&
                 psi(zetf(1,1)),jindx,ort)
            hktype=0           
            call hunphi(nwpsi,hunpsi,hktype,nham)
            call phihunphi(psi,hunpsi,hung,gh,hteil,nham)
            call calchuna(psi,dtpsi,hteil,nham,jindx)
 
            do s=1,nstate
               call vvaxzz(psi(zpsi(s)),dtpsi(zpsi(s)),sum,block(s))
               vibpop(n)=vibpop(n)+sum
            enddo

            aovl=(0.0_dop,0.0_dop)
            if(.not.lnwexpect) then
               do s=1,nstate
                  if (lexact) then
                     call sum2vxz(psi(zetf(1,s)),sum,subdim(1))

                  else if (lgwp .and. psitype .ne. 0) then
                     zeig=1
                     zeig1=block(s)+1
                     call cpvxz(psi(zpsi(s)),tmppsi(zeig),block(s))
                     do m=1,nmode
                        if (gwpm(m))then
                           call qtxxzz(gfgwp(zgfgwp(m,s,s)),&
                                tmppsi(zeig),tmppsi(zeig1),&
                                vdim(m,s),dim(m,s),ndim(m,s))
                           zeig2=zeig
                           zeig=zeig1
                           zeig1=zeig2
                        endif
                     enddo
                     call vvaxzz(psi(zpsi(s)),tmppsi(zeig),sum,block(s))
                  else
                     call vvaxzz(psi(zpsi(s)),psi(zpsi(s)),sum,block(s))
                  endif
                  aovl=aovl+sum
               enddo
               vibpop(n)=vibpop(n)/aovl
            endif
         enddo
         result=dcmplx(0.0_dop)
         result=vibpop(nviblct)

! Normal calculation of expectation value
      else
         ktype=0
         call hphi(psi,hpsi,ktype,nham)
         call phihphi(psi,gh,hpsi,hteil,nham,.false.)

         hktype=0        
         call hunphi(psi,hunpsi,hktype,nham)
         call phihunphi(psi,hunpsi,hung,gh,hteil,nham)

         if (psitype .eq. 5) then
            call compute_hterms_treer(time,tree,psi,nham)
            call compute_all_uterms_tree(time,nham,tree,psi)

            tz => tree%swap_down(1)%p
            z0 = tz%zr(1,1)
            zf = tz%zr(1,2)
            call calcha_ml(tz,psi(z0:zf),dtpsi(z0:zf),nham)
         else
            call calcha(psi,dtpsi,hteil,hops,psi(zetf(1,1)),nham, &
                 psi(zetf(1,1)),jindx,ort)

            if (lgwp) then 
               call calchuna_gwp(psi,dtpsi,hteil,gfgwp,nham)
            else
               call calchuna(psi,dtpsi,hteil,nham,jindx)
            endif
         endif

         result=0.0_dop
         do s=1,nstate
            call vvaxzz(psi(zpsi(s)),dtpsi(zpsi(s)),sum,block(s))
            result=result+sum
         enddo

!-----------------------------------------------------------------------
! normalise expectation value by division through A-norm^2.
!-----------------------------------------------------------------------
         aovl=(0.0_dop,0.0_dop)
         if(.not.lnwexpect) then
            do s=1,nstate
               if (lexact) then
                  call sum2vxz(psi(zetf(1,s)),sum,subdim(1))
               else if (lgwp .and. (psitype .ne. 0 .and. psitype .ne. 5)) then
!                  call gwp_ovl(gs2,gfgwp)
                  zeig=1
                  zeig1=block(s)+1
                  call cpvxz(psi(zpsi(s)),tmppsi(zeig),block(s))
                  do m=1,nmode
                     if (gwpm(m))then
                        call qtxxzz(gfgwp(zgfgwp(m,s,s)),&
                             tmppsi(zeig),tmppsi(zeig1), &
                             vdim(m,s),dim(m,s),ndim(m,s))
                        zeig2=zeig
                        zeig=zeig1
                        zeig1=zeig2
                     endif 
                  enddo
                  call vvaxzz(psi(zpsi(s)),tmppsi(zeig),sum,block(s))
               else 
                  call vvaxzz(psi(zpsi(s)),psi(zpsi(s)),sum,block(s))
               endif
               aovl=aovl+sum
            enddo
            result=result/aovl
         endif
      endif

      return
      end subroutine wfexpectvalue    

!*************************************************************************
!    calculates the expecatation value of an operator; density typ I
!*************************************************************************

      subroutine d1expectvalue(psi,nham,result,hpsi)


      implicit none

      integer(long)                                      :: s
      integer(long)                                      :: hktype,ktype
      integer(long), intent(in)                          :: nham
      real(dop), dimension(maxsta)                       :: edecorr,edeuncorr,&
                                                            spop 
      complex(dop), intent(out)                          :: result
      complex(dop)                                       :: trace
      complex(dop), dimension(dgldim), intent(in)        :: psi
      complex(dop), dimension(hpsidim), intent(out)      :: hpsi
      complex(dop), dimension(adim+1:dgldim)             :: hunpsi

      routine = 'd1expectvalue'

      result = (0.0_dop,0.0_dop)

!-----------------------------------------------------------------------
! Operate on density matrices
!-----------------------------------------------------------------------
      ktype=0           
      call d1hphi(psi,hpsi,ktype,nham)
      hktype=0           
      call d1hunphi(psi,hunpsi,hktype,nham)

!-----------------------------------------------------------------------
! Get uncorrelated and correlated energies
!-----------------------------------------------------------------------
     if (lexact) then
         call calcenrho(psi(dzetf(1,1,1)),edecorr,edeuncorr,nham)
      else
         call calceuncorr1(psi,hunpsi,edeuncorr,nham)
         call calcecorr1(psi,hpsi,edecorr,nham)
      endif

! Sum up expectation values

      do s=1,nstate
         result = result + dcmplx(edeuncorr(s)) + dcmplx(edecorr(s))
      enddo

!        --- calculate tr{rho}   
      if ((.not.lnwexpect).and.(inwfop(1).eq.0)) then
         if (lexact) then
            do s=1,nstate
               call trhxz(psi(dzetf(1,s,s)),spop(s),supgdim(1))
            enddo
         else
            call calcspop1(psi,spop)
         endif

         trace = 0.0_dop
         do s=1,nstate
            trace = trace + dble(spop(s))
         enddo

         result = result/trace
      endif

      return
      end subroutine d1expectvalue 
      
!***********************************************************************
!     CALCEUNCORR1: Calculates uncorrelated energy per state
!     
!     07/99 AR
!***********************************************************************

      subroutine calceuncorr1(psi,hunpsi,euncorr,nham)

      implicit none

      integer(long)                                      :: s,m,m1,m2,z1,z2,hz
      integer(long), intent(in)                          :: nham
      real(dop)                                          :: dtmp
      real(dop), dimension(maxsta), intent(out)          :: euncorr
      real(dop), dimension(d3matdim)                     :: strace
      complex(dop), dimension(dgldim), intent(in)        :: psi
      complex(dop), dimension(adim+1:dgldim), intent(in) :: hunpsi
      complex(dop), dimension(maxblock,2)                :: workc

! calculate SPDO traces for each mode
      do s=1,nstate
         do m=1,nmode
            call trspdo(psi(dzetf(m,s,s)), &
                    strace(dd3mat(m,s,s)),ddim(m,s,s),&
                    subdim(m),supgdim(m))
         enddo
      enddo

      do s=1,nstate

         euncorr(s) = 0.0_dop
         do m1=1,nmode
            if (khzahl(m1,s,nham) .eq. 0) cycle

            call trspdo(hunpsi(dzetf(m1,s,s)),strace(dd3mat(m1,s,s)),&
                 ddim(m1,s,s),subdim(m1),supgdim(m1)) 

            if (lrunml) then
               call allocate_mtraces_matrices(tree,strace)
               call compute_mtraces_tree(psi,tree)

               call traceml(psi,dtmp)

! sum up uncorrelated energy
               euncorr(s) = euncorr(s) + dtmp

            else

               z1 = 1
               z2 = 2
               call cpvxz(psi(dzpsi(s,s)),workc(1,1),dblock(s,s))

               do m2=1,nmode
                  call ptr1(workc(1,z1),strace(dd3mat(m2,s,s)),workc(1,z2),&
                       ddim(m2,s,s),nddim(m2,s,s))
                  hz = z1
                  z1 = z2
                  z2 = hz
               enddo

! sum up uncorrelated energy
               euncorr(s) = euncorr(s) + dble(workc(1,z1))
            endif

! restore SPDO trace
            call trspdo(psi(dzetf(m1,s,s)),strace(dd3mat(m1,s,s)),&
                 ddim(m1,s,s),subdim(m1),supgdim(m1)) 

         enddo
      enddo

      return
      end subroutine calceuncorr1

!***********************************************************************
!     CALCECORR1: Calculates correlated energy per state
!     
!     07/99 AR
!
!***********************************************************************

      subroutine calcecorr1(psi,hpsi,ecorr,nham)

      implicit none

      integer(long)                                     :: si,sf,s,m,k,k1,&
                                                           k2,z1,z2,hz
      integer(long), intent(in)                         :: nham
      real(dop), dimension(maxsta), intent(out)         :: ecorr
      complex(dop), dimension(d3matdim)                 :: ztrace
      complex(dop), dimension(dgldim), intent(in)       :: psi
      complex(dop), dimension(hpsidim), intent(in)      :: hpsi
      complex(dop), dimension(maxblock,2)               :: workc

      do s=1,nstate
         ecorr(s) = 0.0_dop
      enddo

      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1

outer:do k=k1,k2

!     Take only hamiltonian terms
         if (kact(k) .ne. 1) cycle outer

         si=ki(k)
         sf=kf(k)

! calculate SPDO traces for each mode
         do m=1,nmode
            if (.not. diag(m,k)) then
               call trspdoc(hpsi(dzhpsi(m,kfirst(m,k),sf)),&
                    ztrace(dd3mat(m,si,sf)),ddim(m,si,sf),&
                    subdim(m),supgdim(m)) 
            else
               call trspdoc(psi(dzetf(m,si,sf)), &
                    ztrace(dd3mat(m,si,sf)),ddim(m,si,sf),& 
                    subdim(m),supgdim(m))
            endif
         enddo

         z1 = 1
         z2 = 2
! for ML-DMCTDH(I) calculate mode traces for each node
         if (lrunml) then
            call allocate_zmtraces_matrices(tree,ztrace)
            call compute_mtraces_tree(psi,tree,1)

            call ztraceml(psi,workc(1,z1))

         else

            call cpvxz(psi(dzpsi(si,sf)),workc(1,z1),dblock(si,sf))

            do m=1,nmode

               call ptrc(workc(1,z1),ztrace(dd3mat(m,si,sf)),workc(1,z2),&
                    ddim(m,si,sf),nddim(m,si,sf))
               hz = z1
               z1 = z2
               z2 = hz
            enddo
         endif

         if (hamilton(fkoe,k) .eq. 1) then
            ecorr(sf) = ecorr(sf) + dble(workc(1,z1))
         else
            ecorr(sf) = ecorr(sf) + dble(koeff(k))*dble(workc(1,z1))
         endif

      enddo outer

! If no correlated terms, mtraces have not been allocated
      if (lrunml .and. kzahl(nham) .gt. 0) call deallocate_mtraces_matrices(tree)

      return
      end subroutine calcecorr1

!***********************************************************************
!     CALCENRHO: Calculates energy of an exact density matrix
!     
!     07/99 AR
!
!***********************************************************************

      subroutine calcenrho(psi,ecorr,euncorr,nham)

      implicit none

      integer(long)                                     :: s,t,s1,&
                                                           k,k1,k2,&
                                                           hktype,&
                                                           h,pointer
      integer(long), intent(in)                         :: nham
      real(dop), dimension(maxsta), intent(out)         :: ecorr,euncorr
      real(dop)                                         :: dtmp
      complex(dop), dimension(totphidim), intent(in)    :: psi
      complex(dop), dimension(totphidim)                :: tmphpsi,tmphpsi1
      logical(kind=4)                                   :: luncorr=.false.

      do s=1,nstate
         ecorr(s) = 0.0_dop
         euncorr(s) = 0.0_dop
      enddo
      tmphpsi1=0.0_dop

      hktype=0
      do s=1,nstate
         pointer = dzetf(1,s,s)-dzetf(1,1,1)+1
         call d1hunphi1ms(psi(pointer),tmphpsi, &
              1,s,s,ddim(1,s,s),subdim(1),hktype,nham)

         call addvxxzo(tmphpsi,tmphpsi1(pointer),subdim(1))
      enddo


      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1

outer:do k=k1,k2

!     Take only hamiltonian terms
         if (kact(k) .ne. 1) cycle outer
         if (autocap(k) .gt. 0) cycle

!     system operators
         s1=ki(k)
         do t=1,nstate
            pointer = dzetf(1,s1,t)-dzetf(1,1,1)+1
            call d1hphi1mk(psi(pointer),tmphpsi,1,k,&
                 ddim(1,s1,t),subdim(1),supgdim(1),luncorr)

            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1) &
                call xvxxzzo(koeff(k),tmphpsi,subdim(1))
            s=kf(k)
            pointer = dzetf(1,s,t)-dzetf(1,1,1)+1
            call addvxxzo(tmphpsi,tmphpsi1(pointer),subdim(1))
         enddo
      enddo outer

      do s=1,nstate
         pointer = dzetf(1,s,s)-dzetf(1,1,1)+1
         call trhxz(tmphpsi1(pointer),dtmp,supgdim(1))
         euncorr(s) = euncorr(s) + dtmp
      enddo

      return
      end subroutine calcenrho

!***********************************************************************
!     CALCSPOP1: Calculates state populations 
!     
!     06/99 AR
!***********************************************************************

      subroutine calcspop1(psi,spop)

      implicit none

      integer(long)                                    :: s,m,z1,z2,hz
      real(dop), dimension(d3matdim)                   :: strace
      real(dop), dimension(maxsta), intent(out)        :: spop
      complex(dop), dimension(dgldim), intent(in)      :: psi
      complex(dop), dimension(maxblock,2)              :: workc

! calculate SPDO traces for each mode
      do s =1,nstate
         do m=1,nmode
            call trspdo(psi(dzetf(m,s,s)),strace(dd3mat(m,s,s)),&
                 ddim(m,s,s),subdim(m),supgdim(m))
         enddo
      enddo

! for ML-DMCTDH(I) calculate mode traces for each node
      if (lrunml) then
         call allocate_mtraces_matrices(tree,strace)
         call compute_mtraces_tree(psi,tree)

         call traceml(psi,spop(1))

         spop(1) = dble(spop(1)) 

      else
         do s=1,nstate
            spop(s) = 0.0_dop
            call cpvxz(psi(dzpsi(s,s)),workc(1,1),dblock(s,s))
            z1 = 1
            z2 = 2
            
            do m=1,nmode
               call ptr1(workc(1,z1),strace(dd3mat(m,s,s)),workc(1,z2),ddim(m,s,s),&
                    nddim(m,s,s))
               hz = z1
               z1 = z2
               z2 = hz
            enddo
            spop(s) = dble(workc(1,z1)) 
         enddo
      endif

      if (lrunml) call deallocate_mtraces_matrices(tree)

      return
      end subroutine calcspop1


!*************************************************************************
!    calculates the expecatation value of an operator; density typ II
!    NB: dtpsi, hunpsi, and hteil are actually part of workc
!*************************************************************************
      subroutine d2expectvalue(psi,nham,result,hpsi,hteil,agmat,&
                 gfgwp,aigwp,zgwp,qcenter,gh,ghderiv1,gs2,hung,time,gsval,&
                 gsvek)

      implicit none

      integer(long)                                  :: ktype,hktype,s
      integer(long), intent(in)                      :: nham
      integer(long)                                   :: z0,zf

      real(dop), intent(in)                          :: time
      real(dop), dimension(qcentdim), intent(out)    :: qcenter
      real(dop), dimension(gwpinfodim), intent(out)  :: gsval
      real(dop), dimension(ghder1dim),intent(out)    :: ghderiv1
      complex(dop)                                   :: sum,aovl
      complex(dop), intent(out)                      :: result
      complex(dop), dimension(dgldim), intent(inout) :: psi
      complex(dop), dimension(dgldim)                :: dtpsi
      complex(dop), dimension(hpsidim), intent(out)  :: hpsi
      complex(dop), dimension(pmatdim), intent(out)  :: hteil
      complex(dop), dimension(totphidim)             :: hunpsi
      complex(dop), dimension(agmatdim),intent(out)  :: agmat
      complex(dop), dimension(gfgwpdim),intent(out)  :: gfgwp
      complex(dop), dimension(aigwpdim),intent(out)  :: aigwp
      complex(dop), dimension(zgwpdim),intent(out)   :: zgwp
      complex(dop), dimension(ghmatdim),intent(out)  :: gh
      complex(dop), dimension(gs2matdim),intent(out) :: gs2
      complex(dop), dimension(hungdim),intent(out)   :: hung
      complex(dop), dimension(gsmatdim)              :: gsinv
      complex(dop), dimension(gsmatdim)              :: gsvek

      routine = 'd2expectvalue'

      call zerovxz(dtpsi,dgldim)
!
! Overlap matrices help is gfgwp(gfgwp(m,s,s))
!
      if (lgwp) then
         call gwp_param(psi,agmat,gfgwp,aigwp,zgwp,qcenter,1)
         call gwp_overlap(gs2,gsinv,gsval,gsvek,gfgwp,aigwp,zgwp,0)
         call gh_elements(gh,hung,ghderiv1,agmat,gfgwp,aigwp,zgwp,&
              qcenter,nham,1,gs2,time)
      endif

      ktype=0
      call hphi(psi,hpsi,ktype,nham)
      hktype=0           
      call hunphi(psi,hunpsi,hktype,nham)
      
      call phihphi(psi,gh,hpsi,hteil,nham,.false.)
      call phihunphi(psi,hunpsi,hung,gh,hteil,nham)

      call zerovxz(dtpsi,adim)    
      if (psitype .eq. 5) then
         call compute_hterms_treer(time,tree,psi,nham)

         tz => tree%swap_down(1)%p
         z0 = tz%zr(1,1)
         zf = tz%zr(1,2)
         call d2calchaloch2_ml(psi,dtpsi,0,1,nham)
         call d2calchunaloch2_ml(psi,dtpsi,0,nham)
      else
         call d2calchaloch2(psi,dtpsi,hteil,0,1,nham)


         if (lgwp) then
         !   call calchuna_gwp(psi,dtpsi,hteil,gfgwp,nham)
         else
            call d2calchunaloch2(psi,dtpsi,hteil,0,nham)
         endif
      endif

!        - calculate expectation value, i.e. tr{rho H}
      result=0.0_dop
      do s=1,nstate
         call trqxz(dtpsi(dzpsi(s,s)),sum,block(s))
         result = result + sum
      enddo

!        - normalize expectation value, i.e. divide by tr{rho}
      aovl=(0.0_dop,0.0_dop)
      do s=1,nstate
         call trqxz(psi(dzpsi(s,s)),sum,block(s))
         aovl=aovl+sum
      enddo
      
!     if <operate> is used, aovl=0.0 
      if((.not.lnwexpect).and.(inwfop(1).eq.0)) then
         result = result/aovl         
      endif

      return
      end subroutine d2expectvalue                   

!#######################################################################

      subroutine exexpectvalue (psi,time,operator,result)

      implicit none

      integer(long)                                   :: n,nham
      real(dop), intent(in)                           :: time
      complex(dop)                                    :: aovl
      complex(dop), intent(out)                       :: result
      complex(dop), dimension(totphidim), intent(in)  :: psi
      character(len=c2), intent(in)                   :: operator


!-----------------------------------------------------------------------
! Set initial paramters
!-----------------------------------------------------------------------
      routine  = 'exexpectation (expectation values)'
      nham = 0

!-----------------------------------------------------------------------
! Get operator number
!-----------------------------------------------------------------------
      do n=1,hamdim
         if (hamlab(n) .eq. operator) nham=n
      enddo
      if (nham.eq.0) then
         message = 'operator not defined in operator-file: '//operator
         call errormsg
      endif

!-----------------------------------------------------------------------
! calculate time-dependent coefficients. No LCT requried.
!-----------------------------------------------------------------------
      if (ltdh(nham)) call calctdk_nolct(time,nham)

!-----------------------------------------------------------------------
! Calculate expectation value
!-----------------------------------------------------------------------
      call exwfexpectvalue(psi,nham,result,aovl,zetf(1,1))

      return
      end subroutine exexpectvalue 

!*************************************************************************
!     calculates the expectation value of an operator; for wavefunctions
!     NB: dtpsi, hunpsi, and hteil are actually part of workc 
!*************************************************************************

      subroutine exwfexpectvalue(psi,nham,result,aovl,zetf1)

      implicit none

      integer(long)                                      :: h,k,k1,k2,s,s1,m,n,ktype,hktype
      integer(long), intent(in)                          :: zetf1,nham
      complex(dop)                                       :: sum
      complex(dop), dimension(maxdim)                    :: workovl
      complex(dop), dimension(dgldim)                    :: lctworkc
      complex(dop), intent(out)                          :: result,aovl
      complex(dop), dimension(totphidim), intent(in)     :: psi
      complex(dop), dimension(totphidim)                 :: dtpsi
      complex(dop), dimension(totphidim)                 :: tmphpsi
      complex(dop), dimension(zetf1:dgldim)              :: nwpsi


      routine = 'exwfexpectvalue'

      call zerovxz(dtpsi,totphidim)

! Local control expectation values
      if (lctvibpop) then
! Zeroing parameters
         call zerovxz(vibpop,maxdim)
         do n=1,adim
            call zerovxz(nwpsi,dgldim)
            call zerovxz(workovl,maxdim)
            call zerovxz(lctworkc,dgldim)
            call zerovxz(dtpsi,dgldim)
! Setting up the projector
            do s=1,nstate
               do m=1,nmode
                 if (m .eq. nmlct .and. s .eq. nstalct) then
                   call msqzz(lctpsi(zetf(m,s)),lctworkc(zetf(m,s)),&
                   subdim(m),dim(m,s),n)
                 else
                   call cpmxz(psi(zetf(m,s)),lctworkc(zetf(m,s)),&
                   subdim(m),dim(m,s))
                 endif
                 if (leb .and. s .eq. nstalct .and. m .eq. nmode) then
                   call  msqzz(lctpsi(zetf(m,s)),lctworkc(zetf(m,s)),&
                   subdim(m),dim(m,s),nstalct)
                 endif
! Calculates <v|PSI> and then |v><v|PSI>
                 call mmaxzz(lctworkc(zetf(m,s)),psi(zetf(m,s)),&
                  workovl,subdim(m),dim(m,s),dim(m,s))
                 call mmxxzz(lctworkc(zetf(m,s)),workovl,&
                  nwpsi(zetf(m,s)),subdim(m),dim(m,s),dim(m,s))
               enddo
            enddo
! Calculates the expectation value       
            hktype=0
            do s=1,nstate
               call hunphi1ms(nwpsi(zetf(1,s)),dtpsi(zetf(1,s)),&
                 1,s,dim(1,s),subdim(1),hktype,nham)
            enddo

            k1=zham(nham)
            k2=zham(nham)+kzahl(nham)-1
            do k=k1,k2
               s1=ki(k)
               call hphi1mk(nwpsi(zetf(1,s)),tmphpsi,1,k,dim(1,s1),&
                    subdim(1))

               s=kf(k)
               h=hamilton(fkoe,k)
               if (htmtype(h) .eq. 1) then
                  call addvxxzo(tmphpsi,dtpsi(zetf(1,s)),subdim(1))
               else
                  call xvxxzza(koeff(k),tmphpsi,dtpsi(zetf(1,s)),&
                       subdim(1))
               endif
            enddo

            result=0.0_dop
            do s=1,nstate
               call vvaxzz(psi(zetf(1,s)),dtpsi(zetf(1,s)),sum,block(s))
               vibpop(n)=vibpop(n)+sum
            enddo

            aovl=(0.0_dop,0.0_dop)
            if(.not.lnwexpect) then
               do s=1,nstate
                  call sum2vxz(psi(zetf(1,s)),sum,subdim(1))
                  aovl=aovl+sum
               enddo
               vibpop(n)=vibpop(n)/aovl
            endif
         enddo
         result=dcmplx(0.0_dop)
         result=vibpop(nviblct)

! Normal calculation of expectation value

      else
         ktype=0
         hktype=0
         do s=1,nstate
            call hunphi1ms(psi(zetf(1,s)),dtpsi(zetf(1,s)),&
                 1,s,dim(1,s),subdim(1),hktype,nham)
         enddo

         k1=zham(nham)
         k2=zham(nham)+kzahl(nham)-1
         do k=k1,k2
            s1=ki(k)
            call hphi1mk(psi(zetf(1,s1)),tmphpsi,1,k,dim(1,s1),subdim(1))

            s=kf(k)
            h=hamilton(fkoe,k)
            if (htmtype(h) .eq. 1) then
               call addvxxzo(tmphpsi,dtpsi(zetf(1,s)),subdim(1))
            else
               call xvxxzza(koeff(k),tmphpsi,dtpsi(zetf(1,s)),subdim(1))
            endif
         enddo

         result=0.0_dop
         do s=1,nstate
            call vvaxzz(psi(zetf(1,s)),dtpsi(zetf(1,s)),sum,subdim(1))
            result=result+sum
         enddo

         aovl=(0.0_dop,0.0_dop)
         if(.not. lnwexpect) then
            do s=1,nstate
               call sum2vxz(psi(zetf(1,s)),sum,subdim(1))
               aovl=aovl+sum
            enddo
            result=result/aovl
         endif
      endif

      return
      end subroutine exwfexpectvalue

!#######################################################################
!
! Calculate time-dependent coefficients. LCT routines ignored. This
! routine can be called by expectval and other routines where LCT
! not required (in expectval including LCT would result in recursive
! call).
!
!#######################################################################

      subroutine calctdk_nolct(time,nham)

      implicit none

      integer(long)                      :: h,k,kl,kh,idum
      integer(long), intent(in)          :: nham
      real(dop)                          :: c
      real(dop), dimension(maxdim)       :: rdum
      real(dop), intent(in)              :: time
      real(dop), dimension(maxdim)       :: gpoint
      complex(dop)                       :: cz

      kl=zham(nham)
      kh=zham(nham)+khtot(nham)-1

      do k=kl,kh
         h=hamilton(fkoe,k)
         if (htmtype(h) .eq. 101) then
            gpoint(fkoe)=1.0_dop
            gpoint(ftime)=time
            call poteval(h,c,gpoint,cz,rdum,idum)
            koeff(k) = cz
         endif
      enddo

      return
      end subroutine calctdk_nolct


      end module expectval
