!#######################################################################
!
!contains:gh_elements,gh_elements1m,kinetic_elements,dqgwp_elements,
!          dq2gwp_elements,dqgwp_elements_sep,dqgwp_elements_full,
!          dq2gwp_elements_sep,dq2gwp_elements_full,kefac,
!          potential_elements1,potential_elements2,saddle1,
!          sumhung,lhaderivs
!#######################################################################

      module gh_elementsmod

use decimal, only: dop,long, sip
use global
use dvrdatmod
use griddatmod
use psidef
use operdef
use dirdyn
use gwptrj
use logdat
use xvlib,   only: xqxxdzo, xvxxdza, xvxxzza, xvxxzza, xqxxzza, xvxxzzo
use op1lib,  only: cpmxz, zeromxz, zerovxz, cpvxi, zerovxd, zerovxi, &
                   zeromxd
use op2lib,  only: addmxxz, addmxxzo, melmxxzo
use gwptrj
use timing
use runpropmod
use hpsimod
use channels
use openmpmod
use gapmod
use potevalmod, only: poteval
use callanldmod, only: callanld
use errorf, only: erf66,dcerfc
use randommod, only: sobseq
use potevalmod, only: dereval,poteval
use POTmodule, only: keepneigh

use directdyn, only: getddpes
use gwplib, only: moment,par2moment,gwp2gpoint,diagonal_momentsd,&
                  diagonal_momentsz,gwp2gpoint1,getbdot
use lalib,  only: simtran1btz, simtran2btz
use d2tens, only: mkherm
use lalib,  only: simtran1btz, simtran2btz

use logdat, only: errormsg

      implicit none

      private
      public :: gh_elements,gh_elements1m,dq2gwp_elements,dqgwp_elements, &
                lhaderivs,kefac
      contains

!#######################################################################
!
! gwp_param must have called first
!
! modus flag: which elements are to be calculated
!          0: all elements, <g| h |dg/dlam>
!          1: only the zero order elements, <g| h |g>
!          2: do not calculate zero order elements
!          3: only the zero order elements, <g| h |g> and keeping f1, f2
!             fixed at f1point,f2point
!          4: zero order elements for correlated (needed for funk0)
!#######################################################################

      subroutine gh_elements (gh,hung,ghderiv1,agmat,gfgwp,aigwp,zgwp,&
                 qcenter,nham,modus,gs2,time)

!+-----------------------------------------------------------------------+
!     calculates Gaussian matrix elements, of overlap and Hamiltonian
!     type 
!+-----------------------------------------------------------------------+
      implicit none 

      integer(long)                                    :: s,m
      integer(long), intent(in)                        :: modus,nham
      real(dop), intent(in)                            :: time
      real(dop), dimension(qcentdim), intent(in)       :: qcenter
      real(dop), dimension(ghder1dim), intent(out)     :: ghderiv1
      complex(dop), dimension(gfgwpdim), intent(inout) :: gfgwp
      complex(dop), dimension(agmatdim), intent(in)    :: agmat
      complex(dop), dimension(hungdim), intent(inout)  :: hung
      complex(dop), dimension(gs2matdim), intent(in)   :: gs2
      complex(dop), dimension(zgwpdim), intent(in)     :: zgwp
      complex(dop), dimension(aigwpdim), intent(in)    :: aigwp
      complex(dop), dimension(ghmatdim), intent(inout) :: gh
      integer(long), save                              :: tid=0 ! timer ID

      call get_timer(tid, 'Gh_elements')
      call start_timer(tid)

!-----------------------------------------------------------------------
! time needed in trajectory generated potentials (gwptrj calc)
!-----------------------------------------------------------------------
      curr_time = time
!-----------------------------------------------------------------------
! calculate direct dynamics potentials
!-----------------------------------------------------------------------
      if (ldd .and. .not. lgausspot) then
         do s=1,nstate
            call getddpes(time,qcenter,nsgp(s),s)
         enddo
      endif

!+-----------------------------------------------------------------------+
! Hamiltonian elements 
!+-----------------------------------------------------------------------+
      do m = 1,nmode
         if (gwpm(m)) then
            call gh_elements1m (m,gh,hung,ghderiv1,agmat,gfgwp, &
                 aigwp,zgwp,qcenter,nham,modus,gs2,nspfdof(m))
         endif
      enddo
     
!+-----------------------------------------------------------------------+

      call stop_timer(tid)

      return
      end subroutine gh_elements

!#######################################################################
!
!#######################################################################

      subroutine gh_elements1m (m,gh,hung,ghderiv1,agmat,gfgwp,aigwp,&
                 zgwp,qcenter,nham,modus,gs2,gdof)

!+-----------------------------------------------------------------------+
!     calculates Gaussian matrix elements, of overlap and Hamiltonian
!     type 
!+-----------------------------------------------------------------------+

      implicit none

      integer(long)               :: sf,si,k1,k2,k3,h,s,k,f,hl,n,n1
      integer(long), intent(in)   :: m,modus,nham,gdof
      integer(long)               :: npg1,npg2,dim1
      real(dop), dimension(qcentdim), intent(in)           :: qcenter
      complex(dop), dimension(gfgwpdim), intent(inout)     :: gfgwp
      complex(dop), dimension(maxspf)                      :: mom0
      complex(dop), dimension(maxspf,nstate)               :: mom0un
      complex(dop), dimension(gdof,maxspf)                 :: mom1
      complex(dop), dimension(gdof,maxspf,nstate)          :: mom1un
      complex(dop), dimension(gdof,gdof,maxspf)            :: mom2
      complex(dop), dimension(gdof,gdof,maxspf,nstate)     :: mom2un
! mom3, mom4 can be real as only used for potential terms
      real(dop), dimension(:,:,:,:), allocatable           :: mom3
      real(dop), dimension(:,:,:,:,:), allocatable         :: mom4
      real(dop), dimension(:,:,:,:,:), allocatable         :: mom3un
      real(dop), dimension(:,:,:,:,:,:), allocatable       :: mom4un
      real(dop), dimension(gdof,maxspf)                    :: mom1cl
      real(dop), dimension(ghder1dim), intent(out)         :: ghderiv1
      real(dop)                                            :: dkoeff
      complex(dop), dimension(agmatdim), intent(in)        :: agmat
      complex(dop), dimension(hungdim), intent(inout)      :: hung
      complex(dop), dimension(gs2matdim), intent(in)       :: gs2
      complex(dop), dimension(zgwpdim), intent(in)         :: zgwp
      complex(dop), dimension(aigwpdim), intent(in)        :: aigwp
      complex(dop), dimension(ghmatdim), intent(inout)     :: gh
      logical(kind=4)                                      :: luncorr,lcxk
      logical(kind=4), dimension(maxkoe)                   :: ghaddhun
      integer(long)     :: ierr
      character(len=10) :: cerr

!-----------------------------------------------------------------------
! allocate memory for 3rd and 4th order integrals only if required.
!-----------------------------------------------------------------------
      if (gwpintordm(m) .ge. 3) then
         cerr = 'mom3'
         allocate(mom3(gdof,gdof,gdof,maxspf),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'mom3un'
         allocate(mom3un(gdof,gdof,gdof,maxspf,nstate),stat=ierr)
         if (ierr > 0) go to 1000
      else
         cerr = 'mom3'
         allocate(mom3(0,0,0,0),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'mom3un'
         allocate(mom3un(0,0,0,0,0),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      if (gwpintordm(m) .eq. 4) then
         cerr = 'mom4'
         allocate(mom4(gdof,gdof,gdof,gdof,maxspf),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'mom4un'
         allocate(mom4un(gdof,gdof,gdof,gdof,maxspf,nstate),stat=ierr)
         if (ierr > 0) go to 1000
      else
         cerr = 'mom4'
         allocate(mom4(0,0,0,0,0),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'mom4un'
         allocate(mom4un(0,0,0,0,0,0),stat=ierr)
         if (ierr > 0) go to 1000
      endif

      k1=zham(nham)
      k2=zham(nham)+khtot(nham)-1              
      k3=zham(nham)+kzahl(nham)+nmulpot(nham)-1              

! --- select which elements are to be calculated ---
      if (modus.eq.0) then
         npg1=1
         npg2=npg(m)
      else if (modus.eq.1 .or. modus .eq. 4) then
         npg1=1
         npg2=1
      else if (modus.eq.2) then
         npg1=2
         npg2=npg(m)
      else if (modus.eq.3) then
         npg1=1
         npg2=1
      endif

! zero arrays to sum moments of uncorrelated terms
      mom0un=0.0_dop
      mom1un=0.0_dop
      mom2un=0.0_dop
      if (gwpintordm(m) .ge. 3) mom3un = 0.0_dop
      if (gwpintordm(m) .eq. 4) mom4un = 0.0_dop

! uncorrelated gh matrices of  GAP potential terms need to be explicitely
! added
      ghaddhun(:) = .false.

! --- loop over Hamiltonian terms
      do k=k1,k2
! set luncorr if uncorrelated term to be put in xp vector
         luncorr = (k .gt. k3) 
         if (luncorr .and. modus .eq. 4) cycle
! do not calculate unit operators in uncorrelated terms
         if (diag(m,k)) cycle
! if correlated h-matrix has already been calculated cycle
         if ((k .ne. kfirst(m,k)) .and. (.not. luncorr)) cycle

         sf = kf(k)
         si = ki(k)
         dkoeff = dble(koeff(k))

         mom0=0.0_dop
         mom1=0.0_dop
         mom1cl=0.0_dop        ! classical part of mom1 (derivative / momentum)
         mom2=0.0_dop
         if (gwpintordm(m) .ge. 3) mom3 = 0.0_dop
         if (gwpintordm(m) .eq. 4) mom4 = 0.0_dop

! lcxk flag set if the term requires CX derivatives
! i.e. non-KLHA diagonal terms
         lcxk = .false.   
         if (gwpcxm(m) .gt. 0) then
            if ((klha(k) .eq. 0) .and. (ksi(k) .eq. ksf(k)) .and. &
                 .not. lBEL) then
               lcxk = .true.
            endif
         endif

! find DOF with operator information. For KE or flux term this is 
! in n rather than 1. For muld operators this is in fterm(k).
         n=1
         if (fterm(k) .gt. 0) then
            f = fterm(k)
         else
            do n1=1,nspfdof(m)
               f=spfdof(n1,m)
               if (hamilton(f,k) .ne. f) n=n1
            enddo
            f = spfdof(n,m)
         endif
         h = hamilton(f,k)
         hl = hterm(1,h)

! Special operators do not have derivatives
         if (htmtype(h) .gt. 105) lcxk = .false.

! Derivatives only used for system operator
         if (nham .gt. 1) lcxk = .false.

! zero derivs
         if (lcxk) call zeromxd(ghderiv1(ghder1(k,m)),gdof,gwpdim(m,si))

!-----------------------------------------------------------------------
! Analytic potentials
! simple polynomial operators
!-----------------------------------------------------------------------
         if (htmtype(h) .eq. 101 .or. htmtype(h) .eq. 103) then
            call exact_moms(k,m,mom0,mom1,mom2,mom3,mom4,&
                    nspfdof(m),gwpdim(m,si))

! form gh matrices
            if (luncorr) then
               mom0un(:,si) = mom0un(:,si) + dkoeff*mom0(:)
               if (gwpintordm(m) .ge. 1) &
                  mom1un(:,:,si) = mom1un(:,:,si) + dkoeff*mom1(:,:)
               if (gwpintordm(m) .ge. 2) &
                  mom2un(:,:,:,si) = mom2un(:,:,:,si) + dkoeff*mom2(:,:,:)
               if (gwpintordm(m) .ge. 3) mom3un(:,:,:,:,si) = &
                   mom3un(:,:,:,:,si) + dkoeff*mom3(:,:,:,:)
               if (gwpintordm(m) .eq. 4) mom4un(:,:,:,:,:,si) = &
                   mom4un(:,:,:,:,:,si) + dkoeff*mom4(:,:,:,:,:)
            else 
               call gh_elem(gh(ghmat(k,m)),m,mom0,mom1,mom2,mom3,mom4,&
                    gwpdim(m,sf),gwpdim(m,si),npg1,npg2,&
                    gfgwp(zgfgwp(m,sf,si)),&
                    aigwp(zaigwp(m,sf,si)),zgwp(zzgwp(m,sf,si)),nspfdof(m),&
                    agmat(zagmat(m,sf)),agmat(zagmat(m,si)), &
                    agmat(zxivec(m,sf)),agmat(zxivec(m,si)), &
                    agmat(zphase(m,sf)),agmat(zphase(m,si)))
               if (si .eq. sf) &
                  call mkgh_herm(gh(ghmat(k,m)),gwpdim(m,si),npg1,npg2)
            endif

! get derivatives
            if (lcxk) call exact_der1(h,m,mom1cl,qcenter(zcent(m,si)),&
                           gdof,gwpdim(m,si))

!+-----------------------------------------------------------------------+
!     kinetic energy part 
!     notice: Cartesian kinetic-energy terms; diagonal in electronic
!     state si = sf
!+-----------------------------------------------------------------------+
         else if (htmtype(h) .eq. 102)then
            call kinetic_moms(k,m,agmat(zagmat(m,si)),agmat(zxivec(m,si)),&
                 mom0,mom1,mom1cl,mom2,nspfdof(m),gwpdim(m,si))
! form gh matrices.
           if (luncorr) then
              mom0un(:,si) = mom0un(:,si) + dkoeff*mom0(:)
              mom1un(:,:,si) = mom1un(:,:,si) + dkoeff*mom1(:,:)
              mom2un(:,:,:,si) = mom2un(:,:,:,si) + dkoeff*mom2(:,:,:)
           else 
              call gh_elem(gh(ghmat(k,m)),m,mom0,mom1,mom2,mom3,mom4,&
                   gwpdim(m,sf),gwpdim(m,si),npg1,npg2,gfgwp(zgfgwp(m,sf,si)),&
                   aigwp(zaigwp(m,sf,si)),zgwp(zzgwp(m,sf,si)),nspfdof(m),&
                   agmat(zagmat(m,sf)),agmat(zagmat(m,si)), &
                   agmat(zxivec(m,sf)),agmat(zxivec(m,si)), &
                   agmat(zphase(m,sf)),agmat(zphase(m,si)))
           endif

!+-----------------------------------------------------------------------+
! --- LHA  ---
! Multi-D LHA coefficient. Calculated in mfields / calcha
!+-----------------------------------------------------------------------+
         else if (htmtype(h) .eq. 104) then
            continue

!+-----------------------------------------------------------------------+
! --- Normal LHA  ---
!+-----------------------------------------------------------------------+
         else if (htmtype(h) .eq. 105) then
            if (gwpintm(m) .ge. 0 .and. gwpintm(m) .le. 2) then
! --- gwpint = 1. unsymmetric (expanded around central point of RHS GWP) ---
! ---          2. symmetrised after unsymmetric expansion
               call lha_moms(k,m,si,qcenter(zcent(m,si)),mom0,mom1,mom1cl,&
                    mom2,mom3,mom4,nspfdof(m),gwpdim(m,si))

! form gh matrices
               if (luncorr) then
                  mom0un(:,si) = mom0un(:,si) + dkoeff*mom0(:)
                  if (gwpintordm(m) .ge. 1) &
                      mom1un(:,:,si) = mom1un(:,:,si) + dkoeff*mom1(:,:)
                  if (gwpintordm(m) .ge. 2) &
                      mom2un(:,:,:,si) = mom2un(:,:,:,si) + dkoeff*mom2(:,:,:)
                  if (gwpintordm(m) .ge. 3) mom3un(:,:,:,:,si) = &
                      mom3un(:,:,:,:,si) + dkoeff*mom3(:,:,:,:)
                  if (gwpintordm(m) .eq. 4) mom4un(:,:,:,:,:,si) = &
                      mom4un(:,:,:,:,:,si) + dkoeff*mom4(:,:,:,:,:)
               else 
                  call gh_elem(gh(ghmat(k,m)),m,mom0,mom1,mom2,mom3,mom4,&
                       gwpdim(m,sf),gwpdim(m,si),npg1,npg2,&
                       gfgwp(zgfgwp(m,sf,si)),aigwp(zaigwp(m,sf,si)),&
                       zgwp(zzgwp(m,sf,si)),nspfdof(m),&
                       agmat(zagmat(m,sf)),agmat(zagmat(m,si)), &
                       agmat(zxivec(m,sf)),agmat(zxivec(m,si)), &
                       agmat(zphase(m,sf)),agmat(zphase(m,si)))

                 if ((gwpintm(m) .eq. 2) .and. (si .eq. sf)) then
                    call mkgh_herm(gh(ghmat(k,m)),gwpdim(m,si),npg1,npg2)
                 endif

              endif

! --- symmetric LHA (expanded around midpoint point of GWP product) ---
            else if(gwpintm(m) .eq. 3) then
               if (si .eq. sf) then
                  call potential_elements2s(m,nspfdof(m),&
                       dim(m,si),npg1,npg2,&
                       qcenter(zcent(m,si)),&
                       h,k,gh(ghmat(k,m)),&
                       gfgwp(zgfgwp(m,sf,si)),&
                       aigwp(zaigwp(m,sf,si)),&
                       zgwp(zzgwp(m,sf,si)),mom1cl)
               else
                  call potential_elements2(m,nspfdof(m),&
                       dim(m,sf),dim(m,si),npg1,npg2,&
                       qcenter(zcent(m,sf)),&
                       qcenter(zcent(m,si)),h,k,gh(ghmat(k,m)),&
                       gfgwp(zgfgwp(m,sf,si)),&
                       aigwp(zaigwp(m,sf,si)),&
                      zgwp(zzgwp(m,sf,si)))
               endif

            else if (gwpintm(m) .eq. 4) then
! GAP operators.
               if (luncorr) ghaddhun(k) = .true.
               call gh_gap_elem(m,gh(ghmat(k,m)),npg1,npg2,&
                    nspfdof(m),gwpdim(m,sf),gwpdim(m,si),&
                    agmat(zagmat(m,sf)),agmat(zagmat(m,si)),&
                    agmat(zxivec(m,sf)),agmat(zxivec(m,si)),&
                    agmat(zphase(m,sf)),agmat(zphase(m,si)),&
                    ndofdd,dbnrec,dbnm(1:ndofdd,1:dbnrec),&
                    dbgapwgt(ksf(k),ksi(k),1:dbnrec),&
                    gapalpha,gapkfac,ddmorder)

            endif

!-----------------------------------------------------------------------
! flux operator
!-----------------------------------------------------------------------
         else if (htmtype(h) .eq. 106) then
            call flux_gwp(gh(ghmat(k,m)),agmat(zagmat(m,si)),&
                 agmat(zxivec(m,si)),agmat(zphase(m,si)),n,nspfdof(m),&
                 gwpdim(m,si),hoprpar(1,hl),mass(f),gwptype)
            
!-----------------------------------------------------------------------
! step function
!-----------------------------------------------------------------------
         else if (htmtype(h) .eq. 107) then
            call gwpstep(m,f,nspfdof(m),gwpdim(m,sf),gwpdim(m,si),&
                 npg2,k,gh(ghmat(k,m)),gfgwp(zgfgwp(m,sf,si)),&
                 aigwp(zaigwp(m,sf,si)),zgwp(zzgwp(m,sf,si)))

!-----------------------------------------------------------------------
! direct dynamics dipole function, CAP and other operators that use
! saddle-point approx
!-----------------------------------------------------------------------
         else if (htmtype(h) .eq. 108 .or. htmtype(h) .eq. 109) then
            call saddle1(m,sf,si,nspfdof(m),&
                 gwpdim(m,sf),gwpdim(m,si),npg1,npg2,&
                 qcenter(zcent(m,si)),k,gh(ghmat(k,m)),&
                 gfgwp(zgfgwp(m,sf,si)),&
                 aigwp(zaigwp(m,sf,si)),&
                 zgwp(zzgwp(m,sf,si)))

         else
            routine = "gh_elements"
            write(message,'(a,2i5)') "Unknown htmtype: ",h,htmtype(h)
            call errormsg
         endif

         if (lgwptrj) &
            call melmxxzo (gwptrjeovl,gh(ghmat(k,m)),gwpdim(1,1),gwpdim(1,1))
         
!-----------------------------------------------------------------------
! Store gradient / momentum
!-----------------------------------------------------------------------
         if (lcxk) call storeder1(mom1cl,ghderiv1(ghder1(k,m)),&
                        gdof,gwpdim(m,si))

      enddo  ! End of loop over k

!-----------------------------------------------------------------------
! sum up uncorrelated terms
! hung: total uncorrelated <g | h | g>
!
! NB CAP terms not included in hung
!-----------------------------------------------------------------------
      if (modus .ne. 4) then
         do s = 1,nstate
            if (.not. lBEL) call zeromxz(hung(hungmat(m,s,nham)),gwpdim(m,s),&
                                 gwpdim(m,s))
            dim1 = gwpdim(m,s)

            call gh_elem(hung(hungmat(m,s,nham)),m,mom0un(1:dim1,s),&
                 mom1un(1:gdof,1:dim1,s),mom2un(1:gdof,1:gdof,1:dim1,s),&
                 mom3un(:,:,:,:,s),mom4un(:,:,:,:,:,s),&
                 dim1,dim1,npg1,npg2,gfgwp(zgfgwp(m,s,s)),&
                 aigwp(zaigwp(m,s,s)),zgwp(zzgwp(m,s,s)),gdof,&
                 agmat(zagmat(m,s)),agmat(zagmat(m,s)), &
                 agmat(zxivec(m,s)),agmat(zxivec(m,s)), &
                 agmat(zphase(m,s)),agmat(zphase(m,s)))

! Some uncorrelated gh matrices need to be added explicitely
            k1=zhun(m,s,nham)
            k2=zhun(m,s,nham)+khzahl(m,s,nham)-1
            do k=k1,k2
               if (ghaddhun(k)) &
                  call sumhung(koeff(k),gh(ghmat(k,m)),&
                       hung(hungmat(m,s,nham)),gwpdim(m,s),npg(m))
            enddo

         enddo   
      endif

      if (allocated(mom3)) deallocate(mom3,mom3un)
      if (allocated(mom4)) deallocate(mom4,mom4un)

      return
 1000 write(ilog,'(a)') 'Error allocating memory in GH_ELEMENTS1M :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop

      end subroutine gh_elements1m

!#######################################################################
!
! first derivatives of analytical potentials
!
!#######################################################################

      subroutine exact_der1(h,m,mom1cl,qcenter,gdof,gwpdim1)

      implicit none

      integer(long),intent(in) :: h,m,gwpdim1,gdof
      integer(long)            :: ig,n,dertyp,f
      integer(long), dimension(ndof)                  :: order
      real(dop), dimension(gdof,gwpdim1), intent(out) :: mom1cl
      real(dop), dimension(gdof,gwpdim1), intent(in)  :: qcenter
      real(dop), dimension(maxdim)                    :: gpoint,ppoint
      real(dop)                                       :: deriv1

      dertyp = 0
      ppoint = 0.0_dop
      order = 0
      mom1cl = 0.0d0
      do ig = 1,gwpdim1
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            order(f) = 1
            call gwp2gpoint(qcenter(1,ig),gpoint,gdof,maxdim,m)

            call dereval (h,deriv1,gpoint,order,ppoint,dertyp,dermethod)
            mom1cl(n,ig) = deriv1
            order(f) = 0

            if (dertyp .eq. 1) then
               routine='Exact_der1 (gh_elementsmod.F90)'
               message='Kinetic energy term in analytical potentials'
               call errormsg
            endif

         enddo
      enddo

      return
      end subroutine exact_der1

!#######################################################################
!
! copying derivatives to ghderiv1 array needs to be done in a 
! subroutine as ghderiv1 may not be allocated in main routine
!
!#######################################################################

      subroutine storeder1(mom1cl,ghderiv1,gdof,gwpdim1)

      implicit none

      integer(long),intent(in) :: gwpdim1,gdof
      integer(long)            :: ig,n
      real(dop), dimension(gdof,gwpdim1), intent(in)  :: mom1cl
      real(dop), dimension(gdof,gwpdim1), intent(out) :: ghderiv1

      do ig=1,gwpdim1
         do n=1,gdof
            ghderiv1(n,ig) = mom1cl(n,ig)
         enddo
      enddo

      return
      end subroutine storeder1

!#######################################################################

      subroutine mkgh_herm(gh,gwpdim,npg1,npg2)

      implicit none

      integer(long),intent(in) :: npg1,npg2,gwpdim
      integer(long)            :: n
      complex(dop), intent(inout) :: gh(gwpdim,gwpdim,npg2)

      do n=npg1,npg2
         call mkherm(gh(1,1,n),gwpdim)
      enddo

      return
      end subroutine mkgh_herm
     
!#######################################################################

      subroutine kinetic_moms(k,m,agmat,xivec,mom0,mom1,mom1cl,mom2,&
                 gdof,gwpdim1)

!+-----------------------------------------------------------------------+ 
!     potential type terms - exact moments
!+-----------------------------------------------------------------------+

      implicit   none 


      integer(long), intent(in)             :: m,k,gdof,gwpdim1
      integer(long)                         :: n1,f,f1,f2,f3,i,gcount,jg,&
                                               nderiv,icalcv,ng(maxdim)
      real(dop)                                                :: coeff,ptmp
      real(dop), dimension(gdof,gwpdim1), intent(out)            :: mom1cl
      complex(dop), dimension(gwpdim1),intent(inout)             :: mom0
      complex(dop), dimension(gdof,gwpdim1), intent(inout)       :: mom1
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(inout)  :: mom2
      complex(dop), dimension(gdof,gwpdim1), intent(in)        :: xivec
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(in)   :: agmat
      integer(long), save :: tid=0
      
      call get_timer(tid, 'Gh_elems: kinetic')
      call start_timer(tid)

!-----------------------------------------------------------------------
! get factor associated with KE operator (e.g. -1/2M)
! ng(maxdim) is an array whose entries are the derivative orders of the 
! individual dof's as entries
!-----------------------------------------------------------------------
      icalcv=1
      call kefac(k,m,coeff,ng,icalcv)

!-----------------------------------------------------------------------
!     calculate nderiv= sum over all ng elements, for a given (m,k):
!-----------------------------------------------------------------------
      nderiv = 0
      do i=1,gdof
         nderiv = nderiv + ng(i)
      enddo

      mom0 = 0.0_dop
      mom1 = 0.0_dop
      mom2 = 0.0_dop

!-----------------------------------------------------------------------
!     nderiv:
!     1 ---a mixed matrix element, where only one "d1qgwp" derivative
!          occurs in the given particle, and a second derivative appears
!          in a different particle (gwp or spf)
!
!     2 ---for the case where we have either a "d2qgwp" operator
!          or two "d1qgwp" operators within the given particle
!-----------------------------------------------------------------------
      if ( nderiv .eq. 1 ) then
         f = 0
         do n1=1,gdof
            if ( ng(n1) .eq. 1 ) f=n1
         enddo

!-----------------------------------------------------------------------
!        get moments (first derivative)
!-----------------------------------------------------------------------
         do jg=1,gwpdim1
            mom0(jg) = coeff*xivec(f,jg)
            if (lgwpsep) then
               mom1(f,jg) = 2.0_dop*coeff*agmat(f,f,jg)
            else
               do f1=1,gdof
                  mom1(f1,jg) = 2.0_dop*coeff*agmat(f,f1,jg)
               enddo
            endif
         enddo

      elseif ( nderiv .eq. 2 ) then
         gcount = 0
         f      = 0
         f1     = 0
         do n1=1,gdof
            if( ng(n1) .eq. 2) then
               f =n1
               f1=n1
            elseif( ng(n1) .eq. 1) then
               gcount = gcount + 1
               if ( gcount .eq. 1 ) then
                  f = n1
               elseif ( gcount .eq. 2 ) then
                  f1 = n1
               endif
            endif
         enddo

!------------------------------------------------------------------------
!        get moments elements (second gwp derivatives):
!------------------------------------------------------------------------
         do jg=1,gwpdim1
            mom0(jg) = coeff*(2.0_dop*agmat(f,f1,jg) + xivec(f,jg)*xivec(f1,jg))
            if (f .eq. f1) then
               ptmp = dimag(xivec(f,jg))
               mom1cl(f,jg) = 4.0_dop*coeff*dble(agmat(f,f,jg))*ptmp
            endif
            if (lgwpsep) then
               if (f .eq. f1) then
                  mom1(f,jg) = 4.0_dop*coeff*agmat(f,f,jg)*xivec(f,jg)
               else
                  mom1(f,jg) = 2.0_dop*coeff*agmat(f,f,jg)*xivec(f1,jg) 
                  mom1(f1,jg) = 2.0_dop*coeff*agmat(f1,f1,jg)*xivec(f,jg) 
               endif
               mom2(f,f1,jg) = 4.0_dop*coeff*agmat(f,f,jg)*agmat(f1,f1,jg)
            else
               do f2=1,gdof
                  mom1(f2,jg) = 2.0_dop*coeff* &
                     (agmat(f,f2,jg)*xivec(f1,jg) + agmat(f1,f2,jg)*xivec(f,jg))
                  do f3=1,gdof
                     mom2(f2,f3,jg) = &
                        4.0_dop*coeff*agmat(f,f2,jg)*agmat(f1,f3,jg)
                  enddo
               enddo
            endif
         enddo

      endif

      call stop_timer(tid)

      return
      end subroutine kinetic_moms

!#######################################################################

      subroutine lha_moms(k,m,s,qcenter,mom0,mom1,mom1cl,mom2,mom3,mom4, &
                 gdof,gwpdim1)

!$ use OMP_LIB

      implicit   none 

      integer(long), intent(in)                 :: m,s,k,gdof,gwpdim1
      integer(long)                             :: n,n1,n2,n3,f,f1,f2,f3,jg,h
      complex(dop), dimension(gwpdim1),intent(inout)            :: mom0
      complex(dop), dimension(gdof,gwpdim1), intent(inout)      :: mom1
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(inout) :: mom2
      real(dop), dimension(:,:,:,:), intent(inout)              :: mom3
      real(dop), dimension(:,:,:,:,:), intent(inout)            :: mom4
      real(dop), dimension(gdof,gwpdim1), intent(out)           :: mom1cl

      real(dop)                                                :: v0
      real(dop), dimension(maxdim)                             :: gpoint,deriv1
      real(dop), dimension(gdof,gwpdim1), intent(in)           :: qcenter
      real(dop), dimension(maxdim,maxdim)                      :: deriv2
      real(dop), dimension(:,:,:), allocatable                 :: deriv3
      real(dop), dimension(:,:,:,:), allocatable               :: deriv4
      integer(long)     :: ierr
      character(len=10) :: cerr
      integer(long), save :: tid=0

      call get_timer(tid, 'Gh_elems: LHA')
      call start_timer(tid)

      mom0 = 0.0_dop
      mom1 = 0.0_dop
      mom2 = 0.0_dop
      v0=0.0_dop
      deriv1=0.0_dop
      deriv2=0.0_dop
    
!-----------------------------------------------------------------------
! allocate memory for 3rd and 4th order derivatives if required
!-----------------------------------------------------------------------
      cerr = 'deriv3'
      if (gwpintordm(m) .ge. 3) then
         allocate(deriv3(maxdim,maxdim,maxdim),stat=ierr)
         mom3=0.0_dop
         deriv3=0.0_dop
      else
         allocate(deriv3(0,0,0),stat=ierr)
      endif
      if (ierr > 0) go to 1000
      cerr = 'deriv4'
      if (gwpintordm(m) .eq. 4) then
         allocate(deriv4(maxdim,maxdim,maxdim,maxdim),stat=ierr)
         mom4=0.0_dop
         deriv4=0.0_dop
       else
         allocate(deriv4(0,0,0,0),stat=ierr)
      endif
      if (ierr > 0) go to 1000

!-----------------------------------------------------------------------
! get hamiltonian operator for which matrix representation  is wanted
!-----------------------------------------------------------------------
      if (fterm(k) .gt. 0) then
         f = fterm(k)
      else
         f = spfdof(1,m)
      endif
      h = hamilton(f,k)  
!!!$omp parallel do default(none) &
!$omp parallel do ordered schedule(dynamic)&
!$omp firstprivate(gwpdim1,qcenter) &
!$omp firstprivate(gdof,m,h) &
!$omp private(jg,deriv1,deriv2,deriv3,deriv4,gpoint,v0) &
!$omp private(n,n1,n2,n3,f,f1,f2,f3) &
!$omp shared(s,zgp,mom0,mom1,mom2,mom1cl,mom3,mom4,ldbsmall,maxdim) &
!$omp shared(spfdof,gwpintordm) &
!$omp num_threads(ompthread) if(lompthread .and. lompghelem)   
      outerB:do jg=1,gwpdim1
         if (ldbsmall) num_gp=jg+zgp(s)-1
!!map Gaussian dof to actual dof in gpoint array
!!gpoint, deriv1, deriv2 are in space of system degrees of freedom.
         call gwp2gpoint(qcenter(1,jg),gpoint,gdof,maxdim,m)

         call lhaderivs(h,gpoint,v0,deriv1,deriv2,deriv3,deriv4)

!+-----------------------------------------------------------------------+
!     zeroth-order contribution:
!+-----------------------------------------------------------------------+
         mom0(jg) = v0

!+-----------------------------------------------------------------------+
!     store 1st derivative
!+-----------------------------------------------------------------------+
         do n=1,gdof
            f=spfdof(n,m)
            mom1cl(n,jg) = deriv1(f)
         enddo

         if (gwpintordm(m) .lt. 1) cycle outerB
!+-----------------------------------------------------------------------+
!     first-order contribution :
!+-----------------------------------------------------------------------+
         do n=1,gdof
            f=spfdof(n,m)
            mom0(jg) = mom0(jg) - deriv1(f)*gpoint(f)
         enddo

         do n=1,gdof
            f=spfdof(n,m)
            mom1(n,jg) = deriv1(f)
         enddo

         if (gwpintordm(m) .lt. 2) cycle outerB
!+-----------------------------------------------------------------------+
!     second-order contribution :
! only lower triangle of bilinear terms used in gh_elem
!+-----------------------------------------------------------------------+
         do n=1,gdof
            f=spfdof(n,m)
            mom0(jg) = mom0(jg) + 0.5_dop*deriv2(f,f)*gpoint(f)*gpoint(f)
            do n1=1,n-1
               f1=spfdof(n1,m)
               mom0(jg) = mom0(jg) + deriv2(f1,f)*gpoint(f1)*gpoint(f)
            enddo
         enddo


         do n=1,gdof
            f=spfdof(n,m)
            mom1(n,jg) = mom1(n,jg) - deriv2(f,f)*gpoint(f)
            do n1=1,n-1
               f1=spfdof(n1,m)
               mom1(n,jg) = mom1(n,jg) - deriv2(f1,f)*gpoint(f1)
               mom1(n1,jg) = mom1(n1,jg) - deriv2(f1,f)*gpoint(f)
            enddo
         enddo

         do n=1,gdof
            f=spfdof(n,m)
            mom2(n,n,jg)=0.5_dop*deriv2(f,f)

            do n1=1,n-1
               f1=spfdof(n1,m)
               mom2(n1,n,jg)=deriv2(f1,f)
            enddo
         enddo

         if (gwpintordm(m) .lt. 3) cycle outerB
!+-----------------------------------------------------------------------+
!     third-order contribution :
!+-----------------------------------------------------------------------+
         do n=1,gdof
            f=spfdof(n,m)
            mom0(jg) = mom0(jg) - (1/6.0_dop)*deriv3(f,f,f)*&
                              gpoint(f)*gpoint(f)*gpoint(f)
            do n1=1,n-1
               f1=spfdof(n1,m)
               mom0(jg) = mom0(jg) &
                     - 0.5_dop*deriv3(f1,f,f)*&
                              gpoint(f1)*gpoint(f)*gpoint(f)&
                     - 0.5_dop*deriv3(f1,f1,f)*&
                              gpoint(f1)*gpoint(f1)*gpoint(f)
               do n2=1,n1-1
                  f2=spfdof(n2,m)
                  mom0(jg) = mom0(jg) - deriv3(f2,f1,f)*&
                              gpoint(f2)*gpoint(f1)*gpoint(f)
               enddo

            enddo
         enddo

         do n=1,gdof
            f=spfdof(n,m)
            mom1(n,jg) = mom1(n,jg) + 0.5_dop*deriv3(f,f,f)*&
                              gpoint(f)*gpoint(f)
            do n1=1,n-1
               f1=spfdof(n1,m)
               mom1(n,jg) = mom1(n,jg)   &
                         + deriv3(f1,f,f)*gpoint(f1)*gpoint(f)&
                         + 0.5_dop*deriv3(f1,f1,f)*gpoint(f1)*gpoint(f1)
               mom1(n1,jg) = mom1(n1,jg) &
                       + deriv3(f1,f1,f)*gpoint(f1)*gpoint(f)  &
                       + 0.5_dop*deriv3(f1,f,f)*gpoint(f)*gpoint(f)
               do n2=1,n1-1
                  f2=spfdof(n2,m)
                  mom1(n,jg) = mom1(n,jg) &
                         + deriv3(f2,f1,f)*gpoint(f2)*gpoint(f1)
                  mom1(n1,jg) = mom1(n1,jg)&
                         + deriv3(f2,f1,f)*gpoint(f2)*gpoint(f)
                  mom1(n2,jg) = mom1(n2,jg) &
                         + deriv3(f2,f1,f)*gpoint(f1)*gpoint(f)
               enddo
            enddo
         enddo

         do n=1,gdof
            f=spfdof(n,m)
            mom2(n,n,jg) = mom2(n,n,jg) - 0.5_dop*deriv3(f,f,f)*gpoint(f)

            do n1=1,n-1
               f1=spfdof(n1,m)
               mom2(n,n,jg) = mom2(n,n,jg) &
                      - 0.5_dop*deriv3(f1,f,f)*gpoint(f1)
               mom2(n1,n1,jg) = mom2(n1,n1,jg) &
                      - 0.5_dop*deriv3(f1,f1,f)*gpoint(f)
               mom2(n1,n,jg) = mom2(n1,n,jg) &
                      - deriv3(f1,f,f)*gpoint(f)&
                      - deriv3(f1,f1,f)*gpoint(f1)
               do n2=1,n1-1
                  f2=spfdof(n2,m)
                  mom2(n1,n,jg) = mom2(n1,n,jg) &
                        - deriv3(f2,f1,f)*gpoint(f2)
                  mom2(n2,n,jg) = mom2(n2,n,jg) &
                        - deriv3(f2,f1,f)*gpoint(f1)
                  mom2(n2,n1,jg) = mom2(n2,n1,jg)&
                        - deriv3(f2,f1,f)*gpoint(f)
               enddo
            enddo
         enddo

         do n=1,gdof
            f=spfdof(n,m)
            mom3(n,n,n,jg)=(1.0_dop/6.0_dop)*deriv3(f,f,f)

            do n1=1,n-1
               f1=spfdof(n1,m)
               mom3(n1,n,n,jg)= 0.5_dop*deriv3(f1,f,f)
               mom3(n1,n1,n,jg)= 0.5_dop*deriv3(f1,f1,f)

               do n2=1,n1-1
                  f2=spfdof(n2,m)
                  mom3(n2,n1,n,jg) = deriv3(f2,f1,f)
               enddo
            enddo
         enddo

         if (gwpintordm(m) .lt. 4) cycle outerB
!+-----------------------------------------------------------------------+
!     fourth-order contribution :
!+-----------------------------------------------------------------------+
         do n=1,gdof
            f=spfdof(n,m)
            mom0(jg) = mom0(jg) + (1.0_dop/24.0_dop)*deriv4(f,f,f,f)*&
                              gpoint(f)*gpoint(f)*gpoint(f)*gpoint(f)
            do n1=1,n-1
               f1=spfdof(n1,m)
               mom0(jg) = mom0(jg) + (1.0_dop/6.0_dop)*deriv4(f1,f1,f1,f)*&
                              gpoint(f1)*gpoint(f1)*gpoint(f1)*gpoint(f)&
                         + (1.0_dop/6.0_dop)*deriv4(f1,f,f,f)*&
                              gpoint(f1)*gpoint(f)*gpoint(f)*gpoint(f)&
                         + (1.0_dop/4.0_dop)*deriv4(f1,f1,f,f)*&
                              gpoint(f1)*gpoint(f1)*gpoint(f)*gpoint(f)
               do n2=1,n1-1
                  f2=spfdof(n2,m)
                  mom0(jg) = mom0(jg) + 0.5_dop*deriv4(f2,f1,f,f)*&
                              gpoint(f2)*gpoint(f1)*gpoint(f)*gpoint(f)
                  mom0(jg) = mom0(jg) + 0.5_dop*deriv4(f2,f1,f1,f)*&
                              gpoint(f2)*gpoint(f1)*gpoint(f1)*gpoint(f)
                  mom0(jg) = mom0(jg) + 0.5_dop*deriv4(f2,f2,f1,f)*&
                              gpoint(f2)*gpoint(f2)*gpoint(f1)*gpoint(f)
                  do n3=1,n2-1
                     f3=spfdof(n3,m)
                     mom0(jg) = mom0(jg) + deriv4(f3,f2,f1,f)*&
                              gpoint(f3)*gpoint(f2)*gpoint(f1)*gpoint(f)
                  enddo
               enddo
            enddo
         enddo

         do n=1,gdof
            f=spfdof(n,m)
            mom1(n,jg) = mom1(n,jg) - (1.0d0/6.0d0)*deriv4(f,f,f,f)*&
                              gpoint(f)*gpoint(f)*gpoint(f)
            do n1=1,n-1
               f1=spfdof(n1,m)
               mom1(n,jg) = mom1(n,jg) &
                     - (1.0_dop/6.0_dop)*deriv4(f1,f1,f1,f)*&
                              gpoint(f1)*gpoint(f1)*gpoint(f1) &
                     - 0.5_dop*deriv4(f1,f,f,f)*&
                              gpoint(f1)*gpoint(f)*gpoint(f) &
                     - 0.5_dop*deriv4(f1,f1,f,f)*&
                              gpoint(f1)*gpoint(f1)*gpoint(f)
               mom1(n1,jg) = mom1(n1,jg) &
                     - (1.0_dop/6.0_dop)*deriv4(f1,f,f,f)*&
                              gpoint(f)*gpoint(f)*gpoint(f) &
                     - 0.5_dop*deriv4(f1,f1,f1,f)*&
                              gpoint(f1)*gpoint(f1)*gpoint(f) &
                     - 0.5_dop*deriv4(f1,f1,f,f)*&
                              gpoint(f1)*gpoint(f)*gpoint(f)

               do n2=1,n1-1
                  f2=spfdof(n2,m)
                  mom1(n,jg) = mom1(n,jg) - 0.5_dop*deriv4(f2,f2,f1,f)*&
                              gpoint(f2)*gpoint(f2)*gpoint(f1) &
                                  - 0.5_dop*deriv4(f2,f1,f1,f)*&
                              gpoint(f2)*gpoint(f1)*gpoint(f1) &
                                  - deriv4(f2,f1,f,f)*&
                              gpoint(f2)*gpoint(f1)*gpoint(f)
                  mom1(n1,jg) = mom1(n1,jg) - 0.5_dop*deriv4(f2,f2,f1,f)*&
                              gpoint(f2)*gpoint(f2)*gpoint(f) &
                                  - 0.5_dop*deriv4(f2,f1,f,f)*&
                              gpoint(f2)*gpoint(f)*gpoint(f) &
                                  - deriv4(f2,f1,f1,f)*&
                              gpoint(f2)*gpoint(f1)*gpoint(f)
                  mom1(n2,jg) = mom1(n2,jg) - 0.5_dop*deriv4(f2,f1,f1,f)*&
                              gpoint(f1)*gpoint(f1)*gpoint(f) &
                                  - 0.5_dop*deriv4(f2,f1,f,f)*&
                              gpoint(f1)*gpoint(f)*gpoint(f) &
                                  - deriv4(f2,f2,f1,f)*&
                              gpoint(f2)*gpoint(f1)*gpoint(f)
                  do n3=1,n2-1
                     f3=spfdof(n3,m)
                     mom1(n,jg) = mom1(n,jg) - deriv4(f3,f2,f1,f)*&
                              gpoint(f3)*gpoint(f2)*gpoint(f1)
                     mom1(n1,jg) = mom1(n1,jg) - deriv4(f3,f2,f1,f)*&
                              gpoint(f3)*gpoint(f2)*gpoint(f)
                     mom1(n2,jg) = mom1(n2,jg) - deriv4(f3,f2,f1,f)*&
                              gpoint(f3)*gpoint(f1)*gpoint(f)
                     mom1(n3,jg) = mom1(n3,jg) - deriv4(f3,f2,f2,f)*&
                              gpoint(f2)*gpoint(f1)*gpoint(f)
                  enddo
               enddo
            enddo
         enddo

         do n=1,gdof
            f=spfdof(n,m)
            mom2(n,n,jg) = mom2(n,n,jg) &
                + (1.0_dop/4.0_dop)*deriv4(f,f,f,f)*gpoint(f)*gpoint(f)

            do n1=1,n-1
               f1=spfdof(n1,m)
               mom2(n,n,jg) = mom2(n,n,jg) &
                      + 0.5_dop*deriv4(f1,f,f,f)*gpoint(f1)*gpoint(f) &
                      + 0.25_dop*deriv4(f1,f1,f,f)*gpoint(f1)*gpoint(f1)  
               mom2(n1,n1,jg) = mom2(n1,n1,jg) &
                      + 0.5_dop*deriv4(f1,f1,f1,f)*gpoint(f1)*gpoint(f) &
                      + 0.25_dop*deriv4(f1,f1,f,f)*gpoint(f)*gpoint(f)    
               mom2(n1,n,jg) = mom2(n1,n,jg) &
                      + 0.5_dop*deriv4(f1,f,f,f)*gpoint(f)*gpoint(f) &
                      + 0.5_dop*deriv4(f1,f1,f1,f)*gpoint(f1)*gpoint(f1) &
                      + 1.0_dop*deriv4(f1,f1,f,f)*gpoint(f1)*gpoint(f)  
               do n2=1,n1-1
                  f2=spfdof(n2,m)
                  mom2(n,n,jg) = mom2(n,n,jg) &
                      + 0.5_dop*deriv4(f2,f1,f,f)*gpoint(f2)*gpoint(f1)
                  mom2(n1,n1,jg) = mom2(n1,n1,jg) &
                      + 0.5_dop*deriv4(f2,f1,f1,f)*gpoint(f2)*gpoint(f)
                  mom2(n2,n2,jg) = mom2(n2,n2,jg) &
                      + 0.5_dop*deriv4(f2,f2,f1,f)*gpoint(f1)*gpoint(f)

                  mom2(n1,n,jg) = mom2(n1,n,jg) &
                      + deriv4(f2,f1,f,f)*gpoint(f2)*gpoint(f) &
                      + deriv4(f2,f1,f1,f)*gpoint(f2)*gpoint(f1) &
                      + 0.5_dop*deriv4(f2,f2,f1,f)*gpoint(f2)*gpoint(f2)
                  mom2(n2,n,jg) = mom2(n2,n,jg) &
                      + deriv4(f2,f1,f,f)*gpoint(f1)*gpoint(f) &
                      + 0.5_dop*deriv4(f2,f1,f1,f)*gpoint(f1)*gpoint(f1) &
                      + deriv4(f2,f2,f1,f)*gpoint(f2)*gpoint(f1)
                  mom2(n2,n1,jg) = mom2(n2,n1,jg)&
                      + 0.5_dop*deriv4(f2,f1,f,f)*gpoint(f)*gpoint(f) &
                      + deriv4(f2,f1,f1,f)*gpoint(f1)*gpoint(f) &
                      + deriv4(f2,f2,f1,f)*gpoint(f2)*gpoint(f)
                  do n3=1,n2-1
                     f3=spfdof(n3,m)
                     mom2(n1,n,jg) = mom2(n1,n,jg) + deriv4(f3,f2,f1,f)*&
                              gpoint(f3)*gpoint(f2)
                     mom2(n2,n,jg) = mom2(n2,n,jg) + deriv4(f3,f2,f1,f)*&
                              gpoint(f3)*gpoint(f1)
                     mom2(n3,n,jg) = mom2(n3,n,jg) + deriv4(f3,f2,f1,f)*&
                              gpoint(f2)*gpoint(f1)
                     mom2(n2,n1,jg) = mom2(n2,n1,jg) + deriv4(f3,f2,f1,f)*&
                              gpoint(f3)*gpoint(f)
                     mom2(n3,n1,jg) = mom2(n2,n1,jg) + deriv4(f3,f2,f1,f)*&
                              gpoint(f2)*gpoint(f)
                     mom2(n3,n2,jg) = mom2(n3,n2,jg) + deriv4(f3,f2,f1,f)*&
                              gpoint(f1)*gpoint(f)
                  enddo
               enddo
            enddo
         enddo

         do n=1,gdof
            f=spfdof(n,m)
            mom3(n,n,n,jg)=mom3(n,n,n,jg) &
                      -(1.0_dop/6.0_dop)*deriv4(f,f,f,f)*gpoint(f)

            do n1=1,n-1
               f1=spfdof(n1,m)
               mom3(n,n,n,jg)=  mom3(n,n,n,jg) &
                      -(1.0_dop/6.0_dop)*deriv4(f1,f,f,f)*gpoint(f1)
               mom3(n1,n1,n1,jg)=  mom3(n1,n1,n1,jg) &
                      -(1.0_dop/6.0_dop)*deriv4(f1,f1,f1,f)*gpoint(f)
               mom3(n1,n,n,jg)=  mom3(n1,n,n,jg) &
                      -0.5_dop*deriv4(f1,f,f,f)*gpoint(f) &
                      -0.5_dop*deriv4(f1,f1,f,f)*gpoint(f1)   
               mom3(n1,n1,n,jg)= mom3(n1,n1,n,jg) &
                      -0.5_dop*deriv4(f1,f1,f1,f)*gpoint(f1) &
                      -0.5_dop*deriv4(f1,f1,f,f)*gpoint(f)    
               do n2=1,n1-1
                  f2=spfdof(n2,m)
                  mom3(n1,n,n,jg)=  mom3(n1,n,n,jg) &
                         -0.5_dop*deriv4(f2,f1,f,f)*gpoint(f2)
                  mom3(n1,n1,n,jg)=  mom3(n1,n1,n,jg) &
                         -0.5_dop*deriv4(f2,f1,f1,f)*gpoint(f2)
                  mom3(n2,n,n,jg)=  mom3(n2,n,n,jg) &
                         -0.5_dop*deriv4(f2,f1,f,f)*gpoint(f1)
                  mom3(n2,n2,n,jg)=  mom3(n2,n2,n,jg) &
                         -0.5_dop*deriv4(f2,f2,f1,f)*gpoint(f1)
                  mom3(n2,n1,n1,jg)=  mom3(n2,n1,n1,jg) &
                         -0.5_dop*deriv4(f2,f1,f1,f)*gpoint(f)
                  mom3(n2,n2,n1,jg)=  mom3(n2,n2,n1,jg) &
                         -0.5_dop*deriv4(f2,f2,f1,f)*gpoint(f)
                  mom3(n2,n1,n,jg) = mom3(n2,n1,n,jg) &
                         +deriv4(f2,f2,f1,f)*gpoint(f2) &
                         +deriv4(f2,f1,f1,f)*gpoint(f1) &
                         +deriv4(f2,f1,f,f)*gpoint(f)
                  do n3=1,n2-1
                     f3=spfdof(n3,m)
                     mom3(n2,n1,n,jg) = mom3(n2,n1,n,jg) &
                           - deriv4(f3,f2,f1,f)*gpoint(f3)
                     mom3(n3,n1,n,jg) = mom3(n3,n1,n,jg) &
                           - deriv4(f3,f2,f1,f)*gpoint(f2)
                     mom3(n3,n2,n,jg) = mom3(n3,n2,n,jg) &
                           - deriv4(f3,f2,f1,f)*gpoint(f1)
                     mom3(n3,n2,n1,jg) = mom3(n3,n2,n1,jg) &
                           - deriv4(f3,f2,f1,f)*gpoint(f)
                  enddo
               enddo
            enddo
         enddo

         do n=1,gdof
            f=spfdof(n,m)
            mom4(n,n,n,n,jg)=(1.0_dop/24.0_dop)*deriv4(f,f,f,f)

            do n1=1,n-1
               f1=spfdof(n1,m)
               mom4(n1,n,n,n,jg)= (1.0_dop/6.0_dop)*deriv4(f1,f,f,f)
               mom4(n1,n1,n1,n,jg)= (1.0_dop/6.0_dop)*deriv4(f1,f1,f1,f)
               mom4(n1,n1,n,n,jg)= (1.0_dop/4.0_dop)*deriv4(f1,f1,f,f)   

               do n2=1,n1-1
                  f2=spfdof(n2,m)
                  mom4(n2,n2,n1,n,jg)=  0.5_dop*deriv4(f2,f2,f1,f)
                  mom4(n2,n1,n1,n,jg)=  0.5_dop*deriv4(f2,f1,f1,f)
                  mom4(n2,n1,n,n,jg)=  0.5_dop*deriv4(f2,f1,f,f)

                  do n3=1,n2-1
                     f3=spfdof(n3,m)
                     mom4(n3,n2,n1,n,jg) = deriv4(f3,f2,f1,f)
                  enddo
               enddo
            enddo
         enddo

!!end loop over jg
      enddo outerB

!$omp end parallel do
   
      if (allocated(deriv3)) deallocate(deriv3)
      if (allocated(deriv4)) deallocate(deriv4)

      call stop_timer(tid)

      return
 1000 write(ilog,'(a)') 'Error allocating memory in LHA_MOMS :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop
      end subroutine lha_moms

!#######################################################################

      subroutine exact_moms(k,m,mom0,mom1,mom2,mom3,mom4,gdof,gwpdim1)

!+-----------------------------------------------------------------------+ 
!     potential type terms - exact moments
!+-----------------------------------------------------------------------+

      implicit   none 


      integer(long), intent(in)                 :: m,k,gdof,gwpdim1
      integer(long)                             :: h,h1,hl,nh,n,n1,nh1,&
                                                   f,jg,n2,n3
      integer(long)                             :: imom
      integer(long), dimension(maxdim)          :: fmap
      real(dop)                                 :: dtmp
      real(dop), dimension(gdof+1)              :: tmp1d0,tmp1d1,tmp1d2,&
                                                   tmp1d3,tmp1d4
      real(dop)                                 :: tmp0,ntmp0
      real(dop), dimension(gdof)                :: tmp1,ntmp1
      real(dop), dimension(gdof,gdof)           :: tmp2,ntmp2
      complex(dop), dimension(gwpdim1),intent(inout)            :: mom0
      complex(dop), dimension(gdof,gwpdim1), intent(inout)      :: mom1
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(inout) :: mom2
      real(dop), dimension(:,:,:,:), intent(inout)              :: mom3
      real(dop), dimension(:,:,:,:,:), intent(inout)            :: mom4
      real(dop), dimension(:,:,:), allocatable                :: tmp3,ntmp3
      real(dop), dimension(:,:,:,:), allocatable              :: tmp4,ntmp4
      logical, dimension(gdof+1)                :: lfound

      integer(long)     :: ierr
      character(len=10) :: cerr

!-----------------------------------------------------------------------
! allocate memory for 3rd and 4th order integrals only if required.
! note that these do not depend on function as they are only used
! for exact integrals (i.e. for simple polynomials)
!-----------------------------------------------------------------------
      if (gwpintordm(m) .ge. 3) then
         cerr = 'tmp3'
         allocate(tmp3(gdof,gdof,gdof),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'ntmp3'
         allocate(ntmp3(gdof,gdof,gdof),stat=ierr)
         if (ierr > 0) go to 1000
      else
         allocate(tmp3(1,1,1),stat=ierr)
         allocate(ntmp3(1,1,1),stat=ierr)
      endif
      if (gwpintordm(m) .eq. 4) then
         cerr = 'tmp4'
         allocate(tmp4(gdof,gdof,gdof,gdof),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'ntmp4'
         allocate(ntmp4(gdof,gdof,gdof,gdof),stat=ierr)
         if (ierr > 0) go to 1000
      else
         allocate(tmp4(1,1,1,1),stat=ierr)
         allocate(ntmp4(1,1,1,1),stat=ierr)
      endif

!-----------------------------------------------------------------------
! map DOFs 
!-----------------------------------------------------------------------
      do n=1,gdof
         f = spfdof(n,m)
         fmap(f) = n
      enddo

!-----------------------------------------------------------------------
! get hamiltonian operator for which matrix representation  is wanted
!-----------------------------------------------------------------------
      f = spfdof(1,m)
      h = hamilton(f,k)

      mom0 = 0.0_dop
      mom1 = 0.0_dop
      mom2 = 0.0_dop
      if (gwpintordm(m) .ge. 3) mom3 = 0.0_dop
      if (gwpintordm(m) .eq. 4) mom4 = 0.0_dop

!-----------------------------------------------------------------------
! calculate coeffs for each moment
!-----------------------------------------------------------------------
      if (htmform(h) .eq. 0) then
! 1D operator
         dtmp = 1.0_dop
         f = htmdof(h)
         n = fmap(f)
         imom=0
! sum goes to htmfac+1 to allow final summation
         do nh=1,htmfac(h)+1
            hl = 0
            if (nh .le. htmfac(h)) hl = hterm(nh,h)
            if (hl .eq. 0) then
! end of term, sum up
               if (imom .eq. 0) then
                  mom0(:) = mom0(:) + dtmp
               else if (imom .eq. 1) then
                  mom1(n,:) = mom1(n,:) + dtmp
               else if (imom .eq. 2) then
                  mom2(n,n,:) = mom2(n,n,:) + dtmp
               else if (imom .eq. 3) then
                  mom3(n,n,n,:) = mom3(n,n,n,:) + dtmp
               else if (imom .eq. 4) then
                  mom4(n,n,n,n,:) = mom4(n,n,n,n,:) + dtmp
               endif

               dtmp = 1.0_dop
! test
               imom = 0
! test
            else
! multiply out coefficients of term in dtmp OR get order
               if (hopfile(hl) .eq. 301) then  ! number
                  dtmp = dtmp*hoprpar(1,hl)
               else if (hopfile(hl) .eq. 302) then   
                  if (hopilab(hl) .eq. 1) then       ! 1
! test
!                     imom = 0
                     imom = imom + 0
! test
                  else if (hopilab(hl) .eq. 2) then       ! -1
                     dtmp = -1.0_dop*dtmp
                  else if (hopilab(hl) .eq. 3) then  ! q^imom
                     imom = hopipar(1,hl)
                  endif
               else
                  routine = 'Exact_moms (gh_elementsmod.f90)'
                  message = '1D GWP potential not a simple polynomial'
                  call errormsg
               endif
               
            endif
         enddo

      else if (htmform(h) .eq. 1) then
! zero arrays to hold 1D hterms
         tmp1d0 = 0.0_dop
         tmp1d1 = 0.0_dop
         tmp1d2 = 0.0_dop
         tmp1d3 = 0.0_dop
         tmp1d4 = 0.0_dop
         lfound=.false.
         imom = 0

         do nh=1,htmfac(h)+1
! get 1D hterm
            h1=0
            if (nh .le. htmfac(h)) h1 = hterm(nh,h)

            if (h1 .eq. 0) then

! add implicit unit operators (not specified in Hamiltonian)
               do n=1,gdof
                  if (.not. lfound(n)) tmp1d0(n)=1.0_dop
               enddo

! finished 1D terms, multiply together. 
! (tmp1d0 + tmp1d1 x + tmp1d2 x^2 +....)(tmp0 + tmp1 y + tmp2 y^2 + ...)
! Only lower triangle of tensors calculated.
               tmp0 = tmp1d0(1)
               tmp1(1) = tmp1d1(1)
               tmp2(1,1) = tmp1d2(1)
               if (gwpintordm(m) .ge. 3) tmp3(1,1,1) = tmp1d3(1)
               if (gwpintordm(m) .eq. 4) tmp4(1,1,1,1) = tmp1d4(1)

               do n=2,gdof
                  ntmp0 = tmp0*tmp1d0(n)

                  ntmp1(:) = tmp1(:)*tmp1d0(n)
                  ntmp1(n) = tmp0*tmp1d1(n)

                  ntmp2(:,:) = tmp2(:,:)*tmp1d0(n)
                  do n1=1,n-1
                     ntmp2(n1,n) = tmp1(n1)*tmp1d1(n)
                  enddo
                  ntmp2(n,n) = tmp0*tmp1d2(n)

                  if (gwpintordm(m) .ge. 3) then
                     ntmp3(:,:,:) = tmp3(:,:,:)*tmp1d0(n)
                     do n1=1,n-1
                        ntmp3(n1,n,n) = tmp1(n1)*tmp1d2(n)
                        do n2=1,n1
                           ntmp3(n2,n1,n) = tmp2(n2,n1)*tmp1d1(n)
                        enddo
                     enddo
                     ntmp3(n,n,n) = tmp0*tmp1d3(n)
                  endif
                  if (gwpintordm(m) .eq. 4) then
                     ntmp4(:,:,:,:) = tmp4(:,:,:,:)*tmp1d0(n)
                     do n1=1,n-1
                        ntmp4(n1,n,n,n) = tmp1(n1)*tmp1d3(n)
                        do n2=1,n1
                           ntmp4(n2,n1,n,n) = tmp2(n2,n1)*tmp1d2(n)
                           do n3=1,n2
                              ntmp4(n3,n2,n1,n) = tmp3(n3,n2,n1)*tmp1d1(n)
                           enddo
                        enddo
                     enddo
                     ntmp4(n,n,n,n) = tmp0*tmp1d4(n)
                  endif

! copy back before next iteration 
                  tmp0 = ntmp0
                  tmp1 = ntmp1
                  tmp2 = ntmp2
                  if (gwpintordm(m) .ge. 3) tmp3=ntmp3
                  if (gwpintordm(m) .eq. 4) tmp4=ntmp4
               enddo
               if (lfound(gdof+1)) then
                  tmp0 = tmp1d0(gdof+1)*tmp0
                  tmp1 = tmp1d0(gdof+1)*tmp1
                  tmp2 = tmp1d0(gdof+1)*tmp2
                  if (gwpintordm(m) .ge. 3) tmp3=tmp1d0(gdof+1)*tmp3
                  if (gwpintordm(m) .eq. 4) tmp4=tmp1d0(gdof+1)*tmp4
               endif

! sum up and store in momX
               do jg=1,gwpdim1
                  mom0(jg) = mom0(jg) + tmp0
                  mom1(:,jg) = mom1(:,jg) + tmp1(:)
                  mom2(:,:,jg) = mom2(:,:,jg) + tmp2(:,:)
                  if (gwpintordm(m) .ge. 3) mom3(:,:,:,jg) = mom3(:,:,:,jg) + tmp3(:,:,:)
                  if (gwpintordm(m) .eq. 4) &
                           mom4(:,:,:,:,jg) = mom4(:,:,:,:,jg) + tmp4(:,:,:,:)
               enddo

               tmp1d0 = 0.0_dop
               tmp1d1 = 0.0_dop
               tmp1d2 = 0.0_dop
               tmp1d3 = 0.0_dop
               tmp1d4 = 0.0_dop
               lfound=.false.

            else
! get dof on which hterm acts
               f = htmdof(h1)
! If a koeff, treat as a general moment=0 factor and store as f=gdof+1
               if (f .eq. fkoe) then
                  n=gdof+1
                  imom=0
               else
                  n = fmap(f)
               endif
               lfound(n) = .true.

               dtmp = 1.0_dop

               do nh1=1,htmfac(h1)+1
! run over factors in 1D hterm and expand 1D operators. 
                  hl=0
                  if (nh1 .le. htmfac(h1)) hl = hterm(nh1,h1)
                  if (hl .eq. 0) then
! end of term. Store coeffs for order N in tmp1dN.
                     if (imom .eq. 0) then
                        tmp1d0(n) = tmp1d0(n) + dtmp
                     else if (imom .eq. 1) then
                        tmp1d1(n) = tmp1d1(n) + dtmp
                     else if (imom .eq. 2) then
                        tmp1d2(n) = tmp1d2(n) + dtmp
                     else if (imom .eq. 3) then
                        tmp1d3(n) = tmp1d3(n) + dtmp
                     else if (imom .eq. 4) then
                        tmp1d4(n) = tmp1d4(n) + dtmp
                     endif

                     dtmp = 1.0_dop
                  else
! multiply out coefficients of term in dtmp or get order
                     if (hopfile(hl) .eq. 301) then  ! number
                        dtmp = dtmp*hoprpar(1,hl)
                     else if (hopfile(hl) .eq. 302) then   
                        if (hopilab(hl) .eq. 1) then       ! 1
                           imom = 0
                        else if (hopilab(hl) .eq. 2) then       ! -1
                           dtmp = -1.0_dop*dtmp
                        else if (hopilab(hl) .eq. 3) then  ! q^imom
                           imom = hopipar(1,hl)
                        endif
                     else
                        routine = 'GHPOT_EXACT'
                        message = '1D GWP potential not a simple polynomial'
                        call errormsg
                     endif
                        
                  endif
               enddo


            endif

         enddo
      endif 

      if (allocated(tmp3)) deallocate(tmp3)
      if (allocated(tmp4)) deallocate(tmp4)
      if (allocated(ntmp3)) deallocate(ntmp3)
      if (allocated(ntmp4)) deallocate(ntmp4)

      return
 1000 write(ilog,'(a)') 'Error allocating memory in EXACT_MOMS :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop
      end subroutine exact_moms

!#######################################################################
! Calculates matrix elements of a set of GWP on a GAP surface          #
! Only for separable GWP and npg1=1 and npg2=gdof+1 so far             # 
! by I. Polyak 01/2018                                                 #
!#######################################################################

      subroutine gh_gap_elem(m,gh,npg1,npg2,gdof,gwpdim1,gwpdim2,&
                 aleft,aright,xileft,xiright,phileft,phiright,&
                 nmdof,nrec,qgap,w,alpha,kfac,order)

!+-------------------------------------------------------------------------+
! Calculate <g(i)|V|g(j)> and <dg(i)|V|g(j)> GWP ME 
! where V = sum_i(w_i*exp(alpha*(q-q_i)^2)) 
!+-------------------------------------------------------------------------+

      implicit none

      integer(long), intent(in)         :: m,npg1,npg2,gwpdim1,gwpdim2,gdof,&
                                           nmdof,nrec,order
      real(dop), intent(in)             :: kfac
      real(dop), dimension(nmdof), intent(in) :: alpha
      complex(dop), dimension(gwpdim1), intent(in)             :: phileft
      complex(dop), dimension(gwpdim2), intent(in)             :: phiright
      complex(dop), dimension(gdof,gwpdim1), intent(in)        :: xileft
      complex(dop), dimension(gdof,gwpdim2), intent(in)        :: xiright
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(in)   :: aleft
      complex(dop), dimension(gdof,gdof,gwpdim2), intent(in)   :: aright
      complex(dop), dimension(gwpdim1,gwpdim2,npg2), intent(inout)  :: gh
      real(dop), dimension(nmdof,nrec), intent(in)             :: qgap
      real(dop), dimension(nrec), intent(in)                   :: w

      integer(long)                                 :: i,j,ig,jg,p1,n,n1,irec
      integer(long), save                           :: tid=0
      integer(long), dimension(gdof)                :: fmap
      complex(dop)                                  :: norm_tmp,exp_tmp,gh_tmp
      complex(dop), dimension(gdof,gwpdim1,gwpdim2) :: me1d
      complex(dop), allocatable                     :: powersum(:),einterm(:) 
      complex(dop)                                  :: dxi,gh_tmp2,gh_tmp3
      complex(dop), dimension(gwpdim1,gwpdim2)      :: norm,gs
      complex(dop), dimension(gdof,gwpdim1,gwpdim2) :: a,b,gs1d,ag
      complex(dop), dimension(gdof,nrec)            :: c
      complex(dop), dimension(nrec)                 :: d 

      call get_timer(tid, 'GH_GAP_elems')
      call start_timer(tid)

      gh = 0.0_dop

      ! Map n (spf coord) to f (normal mode coord)
      do n=1,gdof
         fmap(n) = spfdof(n,m)
      enddo

      ! Pre-calculate the Norm (i.e. the Phase) terms, contracted over gDOF
      ! Pre-calculate the a and b terms of the Gaussian integral 
      do ig=1,gwpdim1
         do jg=1,gwpdim2
            norm(ig,jg) = 1.0_dop
            do n=1,gdof
               ! Only for the separable GWPs so far
               a(n,ig,jg) = -dconjg(aleft(n,n,ig))-aright(n,n,jg)+alpha(fmap(n))
               norm(ig,jg) = norm(ig,jg)*sqrt(pi/a(n,ig,jg))
               b(n,ig,jg) = dconjg(xileft(n,ig))+xiright(n,jg)
            enddo
            norm(ig,jg) = norm(ig,jg)*exp(dconjg(phileft(ig))+phiright(jg))
         enddo
      enddo
      ! Pre-calculate the two GAP-specific terms c and d
      do irec=1,nrec
         d(irec) = 0.0_dop
         do n=1,gdof
            c(n,irec) = 2.0_dop*alpha(fmap(n))*qgap(fmap(n),irec)
            d(irec) = d(irec) - 0.5_dop*c(n,irec)*qgap(fmap(n),irec) 
         enddo
         d(irec) = exp(d(irec))
      enddo
      ! Combining everything to get the ME (full-D case)
      if (order .eq. 0) then
!$omp parallel do num_threads(ompthread) if(lompthread) default(none),&
!$omp& shared(a,b,c,d,w,nrec,gwpdim1,gwpdim2,gdof,norm,npg1,npg2,kfac,qgap,fmap,phileft,phiright,pi),&
!$omp& private(irec,ig,jg,norm_tmp,exp_tmp,gh_tmp,n,p1,dxi),schedule(static),&
!$omp& reduction(+:gh)
         ! Loop over GAP points
         do irec=1,nrec
            ! Loop over g(i) and g(j)
            do ig=1,gwpdim1
               do jg=1,gwpdim2
                  ! Contract over gDOF (the four-nested looop seems unavoidable)
! Integrals calculated in a less efficient, but more reliable way. More efficient
! to calculate some terms outside these loops, but this can result in intermediate
! terms which exceed the limits of double precision arithmetic, resulting in NaNs. 
                  norm_tmp = 1.0_dop
                  exp_tmp = 0.0_dop
!                  norm_tmp = norm_tmp*norm(ig,jg)*d(irec)
!                  norm_tmp = norm_tmp*norm(ig,jg)
                  do n=1,gdof
!                     exp_tmp = exp_tmp + (b(n,ig,jg)+c(n,irec))**2&
!                              /(4.0_dop*a(n,ig,jg))
! test
                     norm_tmp = norm_tmp*sqrt(pi/a(n,ig,jg))
                     exp_tmp = exp_tmp + (b(n,ig,jg)+c(n,irec))**2&
                              /(4.0_dop*a(n,ig,jg)) &
                              - 0.5_dop*c(n,irec)*qgap(fmap(n),irec)
! test
                  enddo
                  ! Sum over GAP points
! test
                  exp_tmp = exp_tmp+dconjg(phileft(ig))+phiright(jg)
! debug
                  if(exp_tmp.ne.exp_tmp.or.abs(exp_tmp).gt.huge(0.0_dop))then
                     write(*,*)'exponent',irec,ig,jg
                  endif
! debug
! test
                  exp_tmp = exp(exp_tmp)
                  gh_tmp = kfac**2*w(irec)*norm_tmp*exp_tmp
! debug
                  if(exp_tmp.ne.exp_tmp.or.abs(exp_tmp).gt.huge(0.0_dop))then
                     write(*,*)'exponential',irec,ig,jg
                  endif
                  if(norm_tmp.ne.norm_tmp.or.abs(norm_tmp).gt.huge(0.0_dop))then
                     write(*,*)'norm',irec,ig,jg
                  endif
                  if(gh_tmp.ne.gh_tmp.or.abs(gh_tmp).gt.huge(0.0_dop))then
                     write(*,*)'gh',irec,ig,jg
                  endif
! debug

                  if (npg2 .eq. gdof+1) then
                     ! <dg(i)|V|g(j)
                     do p1=2,npg2
                        dxi=(b(p1-1,ig,jg)+c(p1-1,irec))/(2.0_dop*a(p1-1,ig,jg))
                        gh(ig,jg,p1) = gh(ig,jg,p1) + gh_tmp*dxi
                     enddo
                  endif
                  ! <g(i)|V|g(j)
                  if (npg1 .eq. 1) then
                     gh(ig,jg,1) = gh(ig,jg,1) + gh_tmp
                  endif
! debug
                  if(gh(ig,jg,1).ne.gh(ig,jg,1).or.abs(gh(ig,jg,1)).gt.huge(0.0_dop))then
                     write(*,*)'ghig',irec,ig,jg
                  endif
                  do p1=2,npg2
                     if(gh(ig,jg,p1).ne.gh(ig,jg,p1).or.abs(gh(ig,jg,p1)).gt.huge(0.0_dop))&
                          write(*,*)'ghigp1',irec,ig,jg,p1
                  enddo
! debug
               enddo
            enddo
         enddo
      ! Additive kernel case (up to 3d order currently)
      else
         ! Pre-compute 1D and all-D overlaps <g(i)|g(j)>, 
         ! 1D overlaps don't include phase
         do ig=1,gwpdim1
            do jg=1,gwpdim2
               gs(ig,jg) = dconjg(phileft(ig))+phiright(jg)
               norm_tmp = 1.0_dop
               do n=1,gdof
                  ag(n,ig,jg) = -dconjg(aleft(n,n,ig))-aright(n,n,jg)
                  gs1d(n,ig,jg) = b(n,ig,jg)*b(n,ig,jg)/(4.0_dop*ag(n,ig,jg))
                  gs(ig,jg) = gs(ig,jg) + gs1d(n,ig,jg)
                  norm_tmp = norm_tmp*sqrt(pi/ag(n,ig,jg))
!                  gs1d(n,ig,jg) = sqrt(pi/ag(n,ig,jg))*exp(gs1d(n,ig,jg))
               enddo
               gs(ig,jg) = norm_tmp*exp(gs(ig,jg))

            enddo
         enddo
!$omp parallel do num_threads(ompthread) if(lompthread) default(none),&
!$omp& shared(a,b,c,d,w,nrec,gwpdim1,gwpdim2,gdof,npg1,npg2,kfac,order,qgap,fmap,gs,gs1d,pi,ag),&
!$omp& private(irec,ig,jg,gh_tmp,gh_tmp2,gh_tmp3,n,n1,p1,dxi,powersum,einterm,me1d),&
!$omp& schedule(static), reduction(+:gh)
         ! Loop over GAP points
         do irec=1,nrec
            allocate(powersum(order),einterm(0:order))
            do ig=1,gwpdim1
               do jg=1,gwpdim2
                  powersum = 0.0_dop
                  ! Compute 1D <g(i)|Kn|g(j)>/<g(i)|g(j)>
                  do n=1,gdof
                     me1d(n,ig,jg) = (b(n,ig,jg) + c(n,irec))**2
                     me1d(n,ig,jg) = me1d(n,ig,jg)/(4.0_dop*a(n,ig,jg))
                     me1d(n,ig,jg) = me1d(n,ig,jg) - &
                                     0.5_dop*c(n,irec)*qgap(fmap(n),irec)
!                     do n1 = 1,gdof
!                        if(n1.ne.n)&
!                           me1d(n,ig,jg) = me1d(n,ig,jg)-gs1d(n1,ig,jg)
!                     enddo
!                     me1d(n,ig,jg) = exp(me1d(n,ig,jg))
!                     do n1 = 1,gdof
!                        if(n1.ne.n)then
!                           me1d(n,ig,jg) = sqrt(a(n1,ig,jg)/pi)*me1d(n,ig,jg)
!                        else
!!                           me1d(n,ig,jg) = sqrt(pi/a(n1,ig,jg))*me1d(n,ig,jg)
!                        endif
!                     enddo
                     me1d(n,ig,jg) = me1d(n,ig,jg)-gs1d(n,ig,jg)
                     me1d(n,ig,jg) = exp(me1d(n,ig,jg))
                     me1d(n,ig,jg) = sqrt(ag(n,ig,jg)/a(n,ig,jg))*me1d(n,ig,jg)
                  enddo
                  ! Use Newton-Girard identity formula to calculate n-mode ME
                  do i = 1,order
                     do n=1,gdof
                        powersum(i) = powersum(i) + me1d(n,ig,jg)**(i)
                     enddo
                  enddo
                  gh_tmp = 0.0_dop
                  einterm = 0.0_dop
                  einterm(0) = 1.0_dop
                  do i = 1,order
                     do j = 1,i
                        einterm(i) = einterm(i) + &
                             (-1.0_dop)**(j-1)*einterm(i-j)*powersum(j)
                     enddo
                     einterm(i) = einterm(i)/dble(i)
                     gh_tmp = gh_tmp + einterm(i)
                  enddo

                  ! Construct 0th-order ME
                  if (npg1 .eq. 1) then
                     gh_tmp2 = kfac**2*w(irec)*gh_tmp*gs(ig,jg)
                     gh(ig,jg,1) = gh(ig,jg,1) + gh_tmp2
                  endif
                  ! Calculate 1st-order derivative ME
                  if (npg2 .eq. gdof+1) then
                     do p1=2,npg2
                        ! First we overcount terms due to using full n-mode 
                        ! series calculated above using Newton-Girard
                        ! times first Gaussian moment
                        gh_tmp2 = gh_tmp*gs(ig,jg)*0.5_dop*b(p1-1,ig,jg)/&
                                 ag(p1-1,ig,jg)
                        ! First Gaussian moment factor for <dg(i)|Kn|g(j)>
                        dxi=(b(p1-1,ig,jg)+c(p1-1,irec))/(2.0_dop*a(p1-1,ig,jg))
                        ! Calculate corrections
                        ! Have to do it explicitly for each order, should figure
                        ! out a general algorithm such as Newton-Girard identity
                        ! First order
                        gh_tmp3 = me1d(p1-1,ig,jg)
                        ! Second order
                        if (order .ge. 2) then
                           do n=1,gdof
                              if (n .ne. p1-1) then
                                 gh_tmp3 = gh_tmp3 + &
                                           me1d(n,ig,jg)*me1d(p1-1,ig,jg)
                              endif
                           enddo
                        endif
                        ! Third order
                        if (order .ge. 3) then
                           do n=1,gdof
                              do n1=n+1,gdof
                                 if ((n .ne. p1-1) .and. (n1 .ne. p1-1)) then
                                    gh_tmp3 = gh_tmp3 + me1d(n,ig,jg)*&
                                              me1d(n1,ig,jg)*me1d(p1-1,ig,jg)
                                 endif
                              enddo
                           enddo
                        endif
                        ! Add and subtract corrections
                        ! Add correction times first Gaussian moment factor
                        ! for <dg(i)|Kn|g(j)>
                        gh_tmp2 = gh_tmp2 + gh_tmp3*gs(ig,jg)*dxi
                        ! Subtract terms overcounted above times
                        ! first Gaussian moment
                        gh_tmp2 = gh_tmp2 - gh_tmp3*gs(ig,jg)*&
                                 0.5_dop*b(p1-1,ig,jg)/ag(p1-1,ig,jg)
                        ! Finally calculate ME
                        gh_tmp2 = kfac**2*w(irec)*gh_tmp2
                        gh(ig,jg,p1) = gh(ig,jg,p1) + gh_tmp2
                     enddo
                  endif
               enddo
            enddo
            deallocate(powersum,einterm)
         enddo
         if (order .gt. 3) then
            routine='Gh_GAP_elements'
            message = 'GAP ME are only implemented upto 3d order'
            call errormsg
         endif
      endif

      if ((npg2 .gt. npg1) .and. (npg2 .ne. gdof+1)) then
         routine='Gh_GAP_elements'
         message = 'GAP ME can only be currently calculated for separable GWP'
         call errormsg
      endif
 
      !For debugging:
      !do p1=1,npg2
      !   write(*,*) "GH ME, order ",p1
      !   do ig=1,gwpdim1
      !      do jg=1,gwpdim2
      !         write(*,*) gh(ig,jg,p1)
      !      enddo
      !   enddo
      !enddo
      !stop

! debug
!      do p1=1,npg2
!         do ig=1,gwpdim1
!            do jg=1,gwpdim2
!               if(gh(ig,jg,p1).ne.gh(ig,jg,p1))then
!                  write(*,*) 
!                  write(*,*)'gh out'
!                  write(*,*)gh
!                  stop
!               endif
!            enddo
!         enddo
!      enddo
! debug

      call stop_timer(tid)
      !
      end subroutine gh_gap_elem

!#######################################################################
!
! Calculates the matrix elements of a set of GWPs.
! Only lower triangle of momX arrays used
!
!#######################################################################

      subroutine gh_elem(gh,m,mom0,mom1,mom2,mom3,mom4,gwpdim1,gwpdim2,&
                 npg1,npg2,gfgwp,aigwp,zgwp,gdof,&
                 agmat1,agmat2,xivec1,xivec2,phase1,phase2)

!+-----------------------------------------------------------------------+ 
!     potential type terms - exact moments
!+-----------------------------------------------------------------------+

      implicit   none 


      integer(long), intent(in)                 :: gwpdim1,npg1,npg2, &
                                                   gwpdim2,gdof,m
      complex(dop), dimension(gwpdim2),intent(in)            :: mom0
      complex(dop), dimension(gdof,gwpdim2), intent(in)      :: mom1
      complex(dop), dimension(gdof,gdof,gwpdim2), intent(in) :: mom2
      real(dop), dimension(:,:,:,:), intent(in)              :: mom3
      real(dop), dimension(:,:,:,:,:), intent(in)            :: mom4
      complex(dop), dimension(gwpdim1,gwpdim2), intent(inout)           :: gfgwp
      complex(dop), dimension(gdof,gwpdim1,gwpdim2), intent(in)      :: zgwp
      complex(dop), dimension(gdof,gdof,gwpdim1,gwpdim2), intent(in) :: aigwp
      complex(dop), dimension(gwpdim1,gwpdim2,npg2), intent(inout)    :: gh

      complex(dop), dimension(gdof,gdof,gwpdim1), intent(in),optional :: agmat1
      complex(dop), dimension(gdof,gdof,gwpdim2), intent(in),optional :: agmat2
      complex(dop), dimension(gdof,gwpdim1), intent(in),optional      :: xivec1
      complex(dop), dimension(gdof,gwpdim2), intent(in),optional      :: xivec2
      complex(dop), dimension(gwpdim1), intent(in),optional      :: phase1
      complex(dop), dimension(gwpdim2), intent(in),optional      :: phase2

      if (lBEL) then
         call gh_elemBEL(gh,m,mom0,mom1,mom2,mom3,mom4,gwpdim1,gwpdim2,&
                 npg1,npg2,gfgwp,aigwp,zgwp,gdof,&
                 agmat1,agmat2,xivec1,xivec2,phase1,phase2)
      else
         call gh_elemVMCG(gh,m,mom0,mom1,mom2,mom3,mom4,gwpdim1,gwpdim2,&
                 npg1,npg2,gfgwp,aigwp,zgwp,gdof)
      endif

      return
      end

!#######################################################################
!
! Calculates the matrix elements of a set of GWPs.
! Only lower triangle of momX arrays used
!
!#######################################################################

      subroutine gh_elemVMCG(gh,m,mom0,mom1,mom2,mom3,mom4,gwpdim1,gwpdim2,&
                 npg1,npg2,gfgwp,aigwp,zgwp,gdof)

!+-----------------------------------------------------------------------+ 
!     potential type terms - exact moments
!+-----------------------------------------------------------------------+

      implicit   none 


      integer(long), intent(in)                 :: gwpdim1,npg1,npg2, &
                                                   gwpdim2,gdof,m
      integer(long)                             :: ig,jg,p1,pfactor,n,n1,n2,n3
      integer(long), save                       :: tid=0
      integer(long), dimension(maxdim)          :: imoment,pmoment
      complex(dop), dimension(gwpdim2),intent(in)            :: mom0
      complex(dop), dimension(gdof,gwpdim2), intent(in)      :: mom1
      complex(dop), dimension(gdof,gdof,gwpdim2), intent(in) :: mom2
      real(dop), dimension(:,:,:,:), intent(in)              :: mom3
      real(dop), dimension(:,:,:,:,:), intent(in)            :: mom4
      complex(dop), dimension(gwpdim1,gwpdim2), intent(inout)           :: gfgwp
      complex(dop), dimension(gdof,gwpdim1,gwpdim2), intent(in)      :: zgwp
      complex(dop), dimension(gdof,gdof,gwpdim1,gwpdim2), intent(in) :: aigwp
      complex(dop) :: momval

      complex(dop), dimension(gwpdim1,gwpdim2,npg2), intent(inout)    :: gh

      call get_timer(tid, 'Gh_elems: sum')
      call start_timer(tid)

!-----------------------------------------------------------------------
! Calculate matrix elements
!-----------------------------------------------------------------------
      gh = 0.0_dop
      imoment = 0

!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(jg,p1,ig,momval,pfactor,pmoment,imoment,n,n1,n2,n3)
!$omp do schedule(dynamic)
! scalar term
      do jg=1,gwpdim2
         if (abs(mom0(jg)) .lt. tiny) cycle 

         do p1=npg1,npg2
            call par2moment(p1,gdof,pmoment,pfactor,gwptype)
            imoment = pmoment
            if (gwpmodel .eq. 2) then
! independent GWPs diagonal elements only
               momval = &
            moment(imoment,gdof,gfgwp(jg,jg),aigwp(1,1,jg,jg),zgwp(1,jg,jg))
               gh(jg,jg,p1) = gh(jg,jg,p1) + mom0(jg)*pfactor*momval
            else
               do ig=1,gwpdim1
                  momval = &
                     moment(imoment,gdof,gfgwp(ig,jg),aigwp(1,1,ig,jg),zgwp(1,ig,jg))
                  gh(ig,jg,p1) = gh(ig,jg,p1) + mom0(jg)*pfactor*momval
               enddo
            endif
         enddo
      enddo
!$omp end do

! linear terms
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
         do n=1,gdof
            if (abs(mom1(n,jg)) .lt. tiny) cycle
            do p1=npg1,npg2
               call par2moment(p1,gdof,pmoment,pfactor,gwptype)
               imoment = pmoment
               imoment(n) = imoment(n)+1
               if (gwpmodel .eq. 2) then
                  momval = &
                moment(imoment,gdof,gfgwp(jg,jg),aigwp(1,1,jg,jg),zgwp(1,jg,jg))
                  gh(jg,jg,p1) = gh(jg,jg,p1) + mom1(n,jg)*pfactor*momval
               else
                  do ig=1,gwpdim1
                     momval = &
                moment(imoment,gdof,gfgwp(ig,jg),aigwp(1,1,ig,jg),zgwp(1,ig,jg))
                     gh(ig,jg,p1) = gh(ig,jg,p1) + mom1(n,jg)*pfactor*momval
                  enddo
               endif
            enddo
         enddo
      enddo
!$omp end do

! bilinear + quadratic terms
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
         do n=1,gdof
            do n1=1,n
               if (abs(mom2(n1,n,jg)) .lt. tiny) cycle
               do p1=npg1,npg2
                  call par2moment(p1,gdof,pmoment,pfactor,gwptype)

                  imoment = pmoment
                  imoment(n) = imoment(n)+1
                  imoment(n1) = imoment(n1)+1
! independent GWPs diagonal elements only
                  if (gwpmodel .eq. 2) then
                     momval = &
                moment(imoment,gdof,gfgwp(jg,jg),aigwp(1,1,jg,jg),zgwp(1,jg,jg))
                     gh(jg,jg,p1) = gh(jg,jg,p1) + &
                                    mom2(n1,n,jg)*pfactor*momval             
                  else 
                     do ig=1,gwpdim1
                        momval = &
                moment(imoment,gdof,gfgwp(ig,jg),aigwp(1,1,ig,jg),zgwp(1,ig,jg))
                        gh(ig,jg,p1) = gh(ig,jg,p1) + &
                                    mom2(n1,n,jg)*pfactor*momval             
                     enddo
                 endif
               enddo
            enddo
         enddo
      enddo
!$omp end do

! cubic terms. 
      if (gwpintordm(m) .lt. 3) go to 999
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
         do n=1,gdof
            do n1=1,n
               do n2=1,n1
                  if (abs(mom3(n2,n1,n,jg)) .lt. tiny) cycle

                  do p1=npg1,npg2
                     call par2moment(p1,gdof,pmoment,pfactor,gwptype)
                     imoment = pmoment
                     imoment(n) = imoment(n)+1
                     imoment(n1) = imoment(n1)+1
                     imoment(n2) = imoment(n2)+1
! independent GWPs diagonal elements only
                     if (gwpmodel .eq. 2) then
                        gh(jg,jg,p1) = gh(jg,jg,p1) + &
                                       mom3(n2,n1,n,jg)*pfactor* &
             moment(imoment,gdof,gfgwp(jg,jg),aigwp(1,1,jg,jg),zgwp(1,jg,jg))
                     else
                        do ig=1,gwpdim1
                           gh(ig,jg,p1) = gh(ig,jg,p1) + &
                                       mom3(n2,n1,n,jg)*pfactor* &
             moment(imoment,gdof,gfgwp(ig,jg),aigwp(1,1,ig,jg),zgwp(1,ig,jg))
                        enddo
                     endif
                  enddo
               enddo
            enddo
         enddo
      enddo
!$omp end do

! quartic terms 
      if (gwpintordm(m) .lt. 4) go to 999
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
         do n=1,gdof
            do n1=1,n
               do n2=1,n1
                  do n3=1,n2
                     if (abs(mom4(n3,n2,n1,n,jg)) .lt. tiny) cycle

                     do p1=npg1,npg2
                        call par2moment(p1,gdof,pmoment,pfactor,gwptype)
                        imoment = pmoment
                        imoment(n) = imoment(n)+1
                        imoment(n1) = imoment(n1)+1
                        imoment(n2) = imoment(n2)+1
                        imoment(n3) = imoment(n3)+1
! independent GWPs diagonal elements only
                        if (gwpmodel .eq. 2) then
                           gh(jg,jg,p1) = gh(jg,jg,p1) + &
                                          mom4(n3,n2,n1,n,jg)*pfactor* &
             moment(imoment,gdof,gfgwp(jg,jg),aigwp(1,1,jg,jg),zgwp(1,jg,jg))
                        else 
                           do ig=1,gwpdim1
                              gh(ig,jg,p1) = gh(ig,jg,p1) + &
                                          mom4(n3,n2,n1,n,jg)*pfactor* &
             moment(imoment,gdof,gfgwp(ig,jg),aigwp(1,1,ig,jg),zgwp(1,ig,jg))
                           enddo
                        endif
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo
!$omp end do
  999 continue

!$omp end parallel

      call stop_timer(tid)

      return
      end subroutine gh_elemVMCG

!#######################################################################
!
! Calculates the matrix elements of a set of GWPs.
! Only lower triangle of momX arrays used
!
! Extra calls to diagonal_moments due to not allocating zgwp and
!  aigwp matrices.
!#######################################################################

      subroutine gh_elemBEL(gh,m,mom0,mom1,mom2,mom3,mom4,gwpdim1,gwpdim2,&
                 npg1,npg2,gfgwp,aigwp,zgwp,gdof,&
                 agmat1,agmat2,xivec1,xivec2,phase1,phase2)

!+-----------------------------------------------------------------------+ 
!     potential type terms - exact moments
!+-----------------------------------------------------------------------+

      implicit   none 


      integer(long), intent(in)                 :: gwpdim1,npg1,npg2, &
                                                   gwpdim2,gdof,m
      integer(long)                             :: ig,jg,p1,pfactor,n,n1,n2,n3
      integer(long), save                       :: tid=0
      integer(long), dimension(maxdim)          :: imoment,pmoment
      complex(dop), dimension(gwpdim2),intent(in)            :: mom0
      complex(dop), dimension(gdof,gwpdim2), intent(in)      :: mom1
      complex(dop), dimension(gdof,gdof,gwpdim2), intent(in) :: mom2
      real(dop), dimension(:,:,:,:), intent(in)              :: mom3
      real(dop), dimension(:,:,:,:,:), intent(in)            :: mom4
      complex(dop), dimension(gwpdim1,gwpdim2), intent(inout)           :: gfgwp
      complex(dop), dimension(gdof,gwpdim1,gwpdim2), intent(in)      :: zgwp
      complex(dop), dimension(gdof,gdof,gwpdim1,gwpdim2), intent(in) :: aigwp
      complex(dop) :: momval

      complex(dop), dimension(gwpdim1,gwpdim2,npg2), intent(inout)    :: gh

      complex(dop), dimension(gdof,gdof,gwpdim1), intent(in),optional :: agmat1
      complex(dop), dimension(gdof,gdof,gwpdim2), intent(in),optional :: agmat2
      complex(dop), dimension(gdof,gwpdim1), intent(in),optional      :: xivec1
      complex(dop), dimension(gdof,gwpdim2), intent(in),optional      :: xivec2
      complex(dop), dimension(gwpdim1), intent(in),optional      :: phase1
      complex(dop), dimension(gwpdim2), intent(in),optional      :: phase2
      complex(dop), dimension(gdof,gdof)      :: ai2
      complex(dop), dimension(gdof)      :: z2

      call get_timer(tid, 'Gh_elems: sum')
      call start_timer(tid)

!-----------------------------------------------------------------------
! Calculate matrix elements
!-----------------------------------------------------------------------
      gh = 0.0_dop
      imoment = 0
if (present(phase1)) then
!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(jg,p1,ig,momval,pfactor,pmoment,imoment,n,n1,n2,n3)
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
! scalar term
         if (abs(mom0(jg)) .lt. tiny) cycle 
         do p1=npg1,npg2
            call par2moment(p1,gdof,pmoment,pfactor,gwptype)
            imoment = pmoment
            do ig=1,gwpdim1
! independent GWPs diagonal elements only
               if (gwpmodel .eq. 2 .and. ig .ne. jg) cycle

                 call diagonal_momentsz(gdof,agmat1(1,1,ig),agmat2(1,1,jg), &
                      xivec1(1,ig),xivec2(1,jg),phase1(ig),phase2(jg),&
                      gfgwp(ig,jg),ai2,z2,0,1)

               momval = &
                moment(imoment,gdof,gfgwp(ig,jg),ai2,z2)
               gh(ig,jg,p1) = gh(ig,jg,p1) + mom0(jg)*pfactor*momval
            enddo
         enddo
      enddo
!$omp end do

! linear terms
      if (gwpintordm(m) .lt. 1) go to 888
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
         do n=1,gdof
            if (abs(mom1(n,jg)) .lt. tiny) cycle

            do p1=npg1,npg2
               call par2moment(p1,gdof,pmoment,pfactor,gwptype)
               imoment = pmoment
               imoment(n) = imoment(n)+1
               do ig=1,gwpdim1
                  if (gwpmodel .eq. 2 .and. ig .ne. jg) cycle

                 call diagonal_momentsz(gdof,agmat1(1,1,ig),agmat2(1,1,jg), &
                      xivec1(1,ig),xivec2(1,jg),phase1(ig),phase2(jg),&
                      gfgwp(ig,jg),ai2,z2,0,1)

                  momval = &
                moment(imoment,gdof,gfgwp(ig,jg),ai2,z2)
                  gh(ig,jg,p1) = gh(ig,jg,p1) + mom1(n,jg)*pfactor*momval
               enddo
            enddo
         enddo
      enddo
!$omp end do

! bilinear + quadratic terms
      if (gwpintordm(m) .lt. 2) go to 888
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
         do n=1,gdof
            do n1=1,n
               if (abs(mom2(n1,n,jg)) .lt. tiny) cycle

               do p1=npg1,npg2
                  call par2moment(p1,gdof,pmoment,pfactor,gwptype)
                  imoment = pmoment
                  imoment(n) = imoment(n)+1
                  imoment(n1) = imoment(n1)+1
                  do ig=1,gwpdim1
! independent GWPs diagonal elements only
                     if (gwpmodel .eq. 2 .and. ig .ne. jg) cycle

                        call diagonal_momentsz(gdof,agmat1(1,1,ig),agmat2(1,1,jg), &
                             xivec1(1,ig),xivec2(1,jg),phase1(ig),phase2(jg),&
                             gfgwp(ig,jg),ai2,z2,0,1)

                     momval = &
                moment(imoment,gdof,gfgwp(ig,jg),ai2,z2)
                     gh(ig,jg,p1) = gh(ig,jg,p1) + &
                                    mom2(n1,n,jg)*pfactor*momval             
                  enddo
               enddo
            enddo
         enddo
      enddo
!$omp end do

! cubic terms 
      if (gwpintordm(m) .lt. 3) go to 888
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
         do n=1,gdof
            do n1=1,n
               do n2=1,n1
                  if (abs(mom3(n2,n1,n,jg)) .lt. tiny) cycle

                  do p1=npg1,npg2
                     call par2moment(p1,gdof,pmoment,pfactor,gwptype)
                     imoment = pmoment
                     imoment(n) = imoment(n)+1
                     imoment(n1) = imoment(n1)+1
                     imoment(n2) = imoment(n2)+1
                     do ig=1,gwpdim1
! independent GWPs diagonal elements only
                        if (gwpmodel .eq. 2 .and. ig .ne. jg) cycle

                           call diagonal_momentsz(gdof,agmat1(1,1,ig),agmat2(1,1,jg),&
                                xivec1(1,ig),xivec2(1,jg),phase1(ig),phase2(jg),&
                                gfgwp(ig,jg),ai2,z2,0,1)

                        gh(ig,jg,p1) = gh(ig,jg,p1) + &
                                       mom3(n2,n1,n,jg)*pfactor* &
             moment(imoment,gdof,gfgwp(ig,jg),ai2,z2)
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo
!$omp end do

! quartic terms 
      if (gwpintordm(m) .lt. 4) go to 888
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
         do n=1,gdof
            do n1=1,n
               do n2=1,n1
                  do n3=1,n2
                     if (abs(mom4(n3,n2,n1,n,jg)) .lt. tiny) cycle

                     do p1=npg1,npg2
                        call par2moment(p1,gdof,pmoment,pfactor,gwptype)
                        imoment = pmoment
                        imoment(n) = imoment(n)+1
                        imoment(n1) = imoment(n1)+1
                        imoment(n2) = imoment(n2)+1
                        imoment(n3) = imoment(n3)+1
                        do ig=1,gwpdim1
! independent GWPs diagonal elements only
                           if (gwpmodel .eq. 2 .and. ig .ne. jg) cycle

                              call diagonal_momentsz(gdof,agmat1(1,1,ig),agmat2(1,1,jg),&
                                   xivec1(1,ig),xivec2(1,jg),phase1(ig),phase2(jg),&
                                   gfgwp(ig,jg),ai2,z2,0,1)

                           gh(ig,jg,p1) = gh(ig,jg,p1) + &
                                          mom4(n3,n2,n1,n,jg)*pfactor* &
             moment(imoment,gdof,gfgwp(ig,jg),ai2,z2)
                        enddo
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo
!$omp end do

888 continue
!$omp end parallel

else

!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(jg,p1,ig,momval,pfactor,pmoment,imoment,n,n1,n2,n3)
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
! scalar term
         if (abs(mom0(jg)) .lt. tiny) cycle 
         do p1=npg1,npg2
            call par2moment(p1,gdof,pmoment,pfactor,gwptype)
            imoment = pmoment
            do ig=1,gwpdim1
! independent GWPs diagonal elements only
               if (gwpmodel .eq. 2 .and. ig .ne. jg) cycle

               momval = &
                moment(imoment,gdof,gfgwp(ig,jg),aigwp(1,1,ig,jg),zgwp(1,ig,jg))
               gh(ig,jg,p1) = gh(ig,jg,p1) + mom0(jg)*pfactor*momval
            enddo
         enddo
      enddo
!$omp end do

! linear terms
      if (gwpintordm(m) .lt. 1) go to 999
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
         do n=1,gdof
            if (abs(mom1(n,jg)) .lt. tiny) cycle

            do p1=npg1,npg2
               call par2moment(p1,gdof,pmoment,pfactor,gwptype)
               imoment = pmoment
               imoment(n) = imoment(n)+1
               do ig=1,gwpdim1
                  if (gwpmodel .eq. 2 .and. ig .ne. jg) cycle

                  momval = &
                moment(imoment,gdof,gfgwp(ig,jg),aigwp(1,1,ig,jg),zgwp(1,ig,jg))
                  gh(ig,jg,p1) = gh(ig,jg,p1) + mom1(n,jg)*pfactor*momval
               enddo
            enddo
         enddo
      enddo
!$omp end do

! bilinear + quadratic terms
      if (gwpintordm(m) .lt. 2) go to 999
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
         do n=1,gdof
            do n1=1,n
               if (abs(mom2(n1,n,jg)) .lt. tiny) cycle

               do p1=npg1,npg2
                  call par2moment(p1,gdof,pmoment,pfactor,gwptype)
                  imoment = pmoment
                  imoment(n) = imoment(n)+1
                  imoment(n1) = imoment(n1)+1
                  do ig=1,gwpdim1
! independent GWPs diagonal elements only
                     if (gwpmodel .eq. 2 .and. ig .ne. jg) cycle

                     momval = &
                moment(imoment,gdof,gfgwp(ig,jg),aigwp(1,1,ig,jg),zgwp(1,ig,jg))
                     gh(ig,jg,p1) = gh(ig,jg,p1) + &
                                    mom2(n1,n,jg)*pfactor*momval             
                  enddo
               enddo
            enddo
         enddo
      enddo
!$omp end do


! cubic terms 
      if (gwpintordm(m) .lt. 3) go to 999
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
         do n=1,gdof
            do n1=1,n
               do n2=1,n1
                  if (abs(mom3(n2,n1,n,jg)) .lt. tiny) cycle

                  do p1=npg1,npg2
                     call par2moment(p1,gdof,pmoment,pfactor,gwptype)
                     imoment = pmoment
                     imoment(n) = imoment(n)+1
                     imoment(n1) = imoment(n1)+1
                     imoment(n2) = imoment(n2)+1
                     do ig=1,gwpdim1
! independent GWPs diagonal elements only
                        if (gwpmodel .eq. 2 .and. ig .ne. jg) cycle

                        gh(ig,jg,p1) = gh(ig,jg,p1) + &
                                       mom3(n2,n1,n,jg)*pfactor* &
             moment(imoment,gdof,gfgwp(ig,jg),aigwp(1,1,ig,jg),zgwp(1,ig,jg))
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo
!$omp end do

! quartic terms 
      if (gwpintordm(m) .lt. 4) go to 999
!$omp do schedule(dynamic)
      do jg=1,gwpdim2
         do n=1,gdof
            do n1=1,n
               do n2=1,n1
                  do n3=1,n2
                     if (abs(mom4(n3,n2,n1,n,jg)) .lt. tiny) cycle

                     do p1=npg1,npg2
                        call par2moment(p1,gdof,pmoment,pfactor,gwptype)
                        imoment = pmoment
                        imoment(n) = imoment(n)+1
                        imoment(n1) = imoment(n1)+1
                        imoment(n2) = imoment(n2)+1
                        imoment(n3) = imoment(n3)+1
                        do ig=1,gwpdim1
! independent GWPs diagonal elements only
                           if (gwpmodel .eq. 2 .and. ig .ne. jg) cycle

                           gh(ig,jg,p1) = gh(ig,jg,p1) + &
                                          mom4(n3,n2,n1,n,jg)*pfactor* &
             moment(imoment,gdof,gfgwp(ig,jg),aigwp(1,1,ig,jg),zgwp(1,ig,jg))
                        enddo
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo
!$omp end do
  999 continue

!$omp end parallel

!  999 continue
      !For debugging:
!      write(*,*) 'GH_ELEM'
      !do p1=1,npg2
!      do p1=1,1
!         do ig=1,gwpdim1
!            write(*,*) gh(ig,1:gwpdim2,p1)
!         enddo
!      enddo
endif

      call stop_timer(tid)

      return
      end subroutine gh_elemBEL

!#######################################################################

      subroutine saddle1(m,s1,s2,gdof,gwpdim1,gwpdim2,npg1,npg2,qcenter,k,&
                 gh,gfgwp,aigwp,zgwp)

!+-----------------------------------------------------------------------+ 
!    CAP terms evaluated at mid-point only
!+-----------------------------------------------------------------------+

      implicit   none 

      integer(long)                            :: ig,jg,f,pfactor,n,p1,pottyp
      integer(long), intent(in)                :: gwpdim1,m,gdof,k,npg1,&
                                                  npg2,gwpdim2,s1,s2
      integer(long)                            :: h
      integer(long), dimension(maxdim)         :: pmoment
      real(dop)                                                      :: v0
      real(dop), dimension(maxdim)                                   :: gpoint,ppoint
      real(dop), dimension(gdof,gwpdim2), intent(in)                 :: qcenter
      complex(dop)                                                   :: tmp,vz
      complex(dop), dimension(gwpdim1,gwpdim2,npg2), intent(inout)     :: gh
      complex(dop), dimension(gwpdim1,gwpdim2), intent(in)           :: gfgwp
      complex(dop), dimension(gdof,gwpdim1,gwpdim2), intent(in)      :: zgwp
      complex(dop), dimension(gdof,gdof,gwpdim1,gwpdim2), intent(in) :: aigwp

      if (s1 .ne. s2) then
         routine = 'Saddle1 (gh_elementsmod.F90)'
         message = 'Cannot have a CAP connecting states.'
         call errormsg
      endif

      call zerovxd(ppoint,gdof)

      do jg=1,gwpdim2

! map Gaussian dof to actual dof in gpoint array
! gpoint is in the space of system degrees of freedom.
         call gwp2gpoint(qcenter(1,jg),gpoint,gdof,maxdim,m)

! specify current trajectory if gwptrj calculation to pass to trajectory evaluation
         if (lgwptrj) curr_trj = jg

! form product of factors in operator
         tmp = 1.0_dop
         do n=1,nspfdof(m)
            f = spfdof(n,m)
            h = hamilton(f,k)
            call poteval (h,v0,gpoint,vz,ppoint,pottyp)
            tmp = tmp*vz
            if(htmtype(h).eq.108)then
               lcapgwp(n,jg,s2) = lcapgwp(n,jg,s2) .or. abs(tmp).gt.tiny
            endif              
         enddo

         do p1=npg1,npg2
            call par2moment(p1,gdof,pmoment,pfactor,gwptype)
            do ig=1,gwpdim1
! independent GWPs diagonal elements only
               if (gwpmodel .eq. 2 .and. ig .ne. jg) cycle
               gh(ig,jg,p1) = tmp*pfactor* &
                        moment(pmoment,gdof,gfgwp(ig,jg), &
                               aigwp(1,1,ig,jg),zgwp(1,ig,jg))
            enddo
         enddo
      enddo

      return
      end subroutine saddle1
      
!#######################################################################
!
! if ileft = 1 step
!          = -1 1-step
!#######################################################################

      subroutine gwpstep(m,f,gdof,gwpdim1,gwpdim2,npg2,k,&
                 gh,gfgwp,aigwp,zgwp)

!+-----------------------------------------------------------------------+ 
!    CAP terms evaluated at mid-point only
!+-----------------------------------------------------------------------+

      implicit   none 

      integer(long)                            :: ig,jg,pfactor,n,p1
      integer(long), intent(in)                :: gwpdim1,m,f,gdof,k,&
                                                  npg2,gwpdim2
      integer(long)                            :: ileft
      integer(long)                            :: h,h1,hl,nh,n1
      integer(long), dimension(gdof)           :: pmoment
      complex(dop)                             :: v0
      real(dop)                                :: rgpstep,gpstep0,rwidth
      complex(dop)                             :: zwidth,zcenter,zgpstep
      complex(dop)                                                   :: vz
      complex(dop), dimension(gwpdim1,gwpdim2,npg2), intent(inout)     :: gh
      complex(dop), dimension(gwpdim1,gwpdim2), intent(in)           :: gfgwp
      complex(dop), dimension(gdof,gwpdim1,gwpdim2), intent(in)      :: zgwp
      complex(dop), dimension(gdof,gdof,gwpdim1,gwpdim2), intent(in) :: aigwp

      if (.not. lgwpsep) then
         routine='Gwpstep (gh_elementsmod.F90)'
         message='Step operator only implemented for seperable GWPs'
         call errormsg
         return
      endif
!
! Find step function
!
      h=hamilton(f,k)
      if (htmform(h) .eq. 1) then
         h1=hterm(1,h)
         nh=1
         hl=hterm(nh,h1)
      else
         nh=1
         hl=hterm(nh,h)
      endif
!
! get DOF within GWP
!
      n=0
      do n1 = 1,gdof
         if (spfdof(n1,m) .eq. f) n = n1
      enddo

      if (n .eq. 0) return

! step or reverse step? hopilab 24 = reverse step.
      ileft = -99
      if (hopilab(hl) .eq. 23) then
         ileft = 1
      else if (hopilab(hl) .eq. 24) then
         ileft = -1
      else 
         routine = "Gwpstep (gh_elementsmod.f90)"
         message = "This is not a step operator"
         call errormsg
      endif

      gpstep0 = hoprpar(2,hl)

! only calculated for <g|h|g>. 

      p1=1
      call par2moment(p1,gdof,pmoment,pfactor,gwptype)
      do jg=1,gwpdim2
         do ig=1,gwpdim1
! independent GWPs diagonal elements only
            if (gwpmodel .eq. 2 .and. ig .ne. jg) cycle

! map Gaussian dof to actual dof in gpoint array
! gpoint is in the space of system degrees of freedom.
! if gwp = exp(-ax**2 + bx + c)
! aigwp = -1/a  zgwp = -b/a  center = b/2a
! width = a
!
! ileft > 0 step, theta(x) = 1  if x > x0
! ileft < 0 rstep, theta(x) = 1  if x < x0
! NB error function is defined int_0^x = 1 for x-> infty
             zcenter=-0.5_dop*zgwp(n,ig,jg)
             zwidth=-1.0_dop/aigwp(n,n,ig,jg)
             zgpstep = (gpstep0 - zcenter)*sqrt(zwidth)
         
             if (dble(zgpstep) .lt. 0) then
                zgpstep=-zgpstep
                if (ig .eq. jg) then
                   rwidth=dble(zwidth)
                   rgpstep = dble(zgpstep)
                   if (ileft .gt. 0) then 
                      v0=0.5_dop*(1.0+erf(rgpstep))
                   else
                      v0=0.5_dop*(1.0-erf(rgpstep))
                   endif
                   vz=v0
                else
                  if (ileft .gt. 0) then
                     call dcerfc(0,zgpstep,vz)
                     vz=0.5_dop*(1.0+vz)
                  else
                     call dcerfc(1,zgpstep,vz)
                     vz=0.5_dop*(vz)
                  endif
                endif
             else

                if (ig .eq. jg) then
                   rwidth=dble(zwidth)
                   rgpstep = dble(zgpstep)
                   if (ileft .gt. 0) then 
                      v0=0.5_dop*(1.0-erf(rgpstep))
                   else
                      v0=0.5_dop*(1.0+erf(rgpstep))
                   endif
                   vz=v0
                else
                  if (ileft .gt. 0) then
                     call dcerfc(1,zgpstep,vz)
                     vz=0.5_dop*(vz)
                  else
                     call dcerfc(0,zgpstep,vz)
                     vz=0.5_dop*(1.0+vz)
                  endif

                endif
             endif

             gh(ig,jg,1) = vz*gfgwp(ig,jg)
         enddo
      enddo

      return
      end subroutine gwpstep
      
!#######################################################################

      subroutine dqgwp_elements (f,gdof,gh,gwpdim1,agmat,xivec, &
                 gfgwp,aigwp,zgwp,npg1,npg2,phase)
!+-----------------------------------------------------------------------+ 
!     kinetic type terms 
!+-----------------------------------------------------------------------+

      implicit   none 

      integer(long)                                  :: ig,jg,p1,pfactor,f1
      integer(long), intent(in)                      :: gdof,npg1,npg2,gwpdim1,f
      integer(long), dimension(maxdim)               :: imoment,pmoment
      complex(dop)                                                   :: tmp
      complex(dop), dimension(gwpdim1,gwpdim1), intent(inout)           :: gfgwp
      complex(dop), dimension(gwpdim1,gwpdim1)                       :: tmpmom
      complex(dop), dimension(gdof,gwpdim1), intent(in)              :: xivec
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(in)         :: agmat
      complex(dop), dimension(gdof,gwpdim1,gwpdim1), intent(in)      :: zgwp
      complex(dop), dimension(gdof,gdof,gwpdim1,gwpdim1), intent(in) :: aigwp
      complex(dop), dimension(gwpdim1,gwpdim1,npg2), intent(inout)     :: gh
      complex(dop), dimension(gwpdim1), intent(in),optional          :: phase
      complex(dop), dimension(gdof,gdof)                             :: ai2
      complex(dop), dimension(gdof)                                  :: z2
!      complex, external                                        :: moment

if (present(phase)) then
      do p1 = npg1,npg2

!+-----------------------------------------------------------------------+
!     zeroth-order contribution in f : 
!+-----------------------------------------------------------------------+
         call par2moment(p1,gdof,pmoment,pfactor,gwptype)
!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(ig,jg,ai2,z2)
!$omp do schedule(dynamic)
         do ig = 1,gwpdim1
            call diagonal_momentsd(gdof,agmat(1,1,ig), &
                 xivec(1,ig),phase(ig),&
                 gfgwp(ig,ig),ai2,z2,1)
            tmpmom(ig,ig) = pfactor*moment(pmoment,gdof,gfgwp(ig,ig), &
                                   ai2,z2)
            do jg = ig+1,gwpdim1
               call diagonal_momentsz(gdof,agmat(1,1,jg),agmat(1,1,ig), &
                    xivec(1,jg),xivec(1,ig),phase(jg),phase(ig),&
                    gfgwp(jg,ig),ai2,z2,0,1)
               tmpmom(jg,ig) = pfactor*moment(pmoment,gdof,gfgwp(jg,ig), &
                                      ai2,z2)
               tmpmom(ig,jg) = dconjg(tmpmom(jg,ig))
            enddo
         enddo
!$omp end do
!$omp end parallel

!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(ig,jg,tmp,ai2,z2)
!$omp do schedule(dynamic)
         do ig = 1,gwpdim1
            tmp = xivec(f,ig) 
            do jg = 1,gwpdim1
               gh(jg,ig,p1) = tmp*tmpmom(jg,ig)
            enddo
         enddo
!$omp end do
!$omp end parallel

!+-----------------------------------------------------------------------+
!     first-order contribution : 
!+-----------------------------------------------------------------------+
         do f1=1,gdof
            call cpvxi(pmoment,imoment,gdof)
            imoment(f1)=imoment(f1)+1
!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(ig,jg,ai2,z2)
!$omp do schedule(dynamic)
            do ig = 1,gwpdim1
              call diagonal_momentsd(gdof,agmat(1,1,ig), &
                   xivec(1,ig),phase(ig),&
                   gfgwp(ig,ig),ai2,z2,1)
               tmpmom(ig,ig) = pfactor*moment(imoment,gdof,gfgwp(ig,ig), &
                                   ai2,z2)
               do jg = ig+1,gwpdim1
                 call diagonal_momentsz(gdof,agmat(1,1,jg),agmat(1,1,ig), &
                      xivec(1,jg),xivec(1,ig),phase(jg),phase(ig),&
                      gfgwp(jg,ig),ai2,z2,0,1)
                  tmpmom(jg,ig) =  &
                       pfactor*moment(imoment,gdof,gfgwp(jg,ig), &
                                      ai2,z2)
                  tmpmom(ig,jg) = dconjg(tmpmom(jg,ig))
               enddo
            enddo
!$omp end do
!$omp end parallel

!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(ig,jg,tmp)
!$omp do schedule(dynamic)
            do ig = 1,gwpdim1
               tmp=2.0_dop*agmat(f,f1,ig)
               do jg = 1,gwpdim1
                  gh(jg,ig,p1) = gh(jg,ig,p1) + tmp*tmpmom(jg,ig)
               enddo
            enddo
!$omp end do
!$omp end parallel
         enddo

      enddo
else
      do p1 = npg1,npg2

!+-----------------------------------------------------------------------+
!     zeroth-order contribution in f : 
!+-----------------------------------------------------------------------+
         call par2moment(p1,gdof,pmoment,pfactor,gwptype)
!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(ig,jg,ai2,z2)
!$omp do schedule(dynamic)
         do ig = 1,gwpdim1
            tmpmom(ig,ig) = pfactor*moment(pmoment,gdof,gfgwp(ig,ig), &
                                   aigwp(1,1,ig,ig),zgwp(1,ig,ig))
            do jg = ig+1,gwpdim1
               tmpmom(jg,ig) = pfactor*moment(pmoment,gdof,gfgwp(jg,ig), &
                                      aigwp(1,1,jg,ig),zgwp(1,jg,ig))
               tmpmom(ig,jg) = dconjg(tmpmom(jg,ig))
            enddo
         enddo
!$omp end do
!$omp end parallel

!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(ig,jg,tmp,ai2,z2)
!$omp do schedule(dynamic)
         do ig = 1,gwpdim1
            tmp = xivec(f,ig) 
            do jg = 1,gwpdim1
               gh(jg,ig,p1) = tmp*tmpmom(jg,ig)
            enddo
         enddo
!$omp end do
!$omp end parallel

!+-----------------------------------------------------------------------+
!     first-order contribution : 
!+-----------------------------------------------------------------------+
         do f1=1,gdof
            call cpvxi(pmoment,imoment,gdof)
            imoment(f1)=imoment(f1)+1
!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(ig,jg,ai2,z2)
!$omp do schedule(dynamic)
            do ig = 1,gwpdim1
               tmpmom(ig,ig) = pfactor*moment(imoment,gdof,gfgwp(ig,ig), &
                                   aigwp(1,1,ig,ig),zgwp(1,ig,ig))
               do jg = ig+1,gwpdim1
                  tmpmom(jg,ig) =  &
                       pfactor*moment(imoment,gdof,gfgwp(jg,ig), &
                                      aigwp(1,1,jg,ig),zgwp(1,jg,ig))
                  tmpmom(ig,jg) = dconjg(tmpmom(jg,ig))
               enddo
            enddo
!$omp end do
!$omp end parallel

!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(ig,jg,tmp,ai2,z2)
!$omp do schedule(dynamic)
            do ig = 1,gwpdim1
               tmp=2.0_dop*agmat(f,f1,ig)
               do jg = 1,gwpdim1
                  gh(jg,ig,p1) = gh(jg,ig,p1) + tmp*tmpmom(jg,ig)
               enddo
            enddo
!$omp end do
!$omp end parallel
         enddo
      enddo
endif
      return
      end subroutine dqgwp_elements

!#######################################################################

      subroutine dq2gwp_elements (f,ff,gdof,gh,gwpdim1,agmat,xivec, &
                 gfgwp,aigwp,zgwp,npg1,npg2,phase)

!+-----------------------------------------------------------------------+ 
!     kinetic type terms 
!+-----------------------------------------------------------------------+

      implicit   none 

      integer(long)                          :: ig,jg,f1,f2,p1,pfactor
      integer(long), intent(in)              :: gdof,npg1,npg2,gwpdim1,f,ff
      integer(long), dimension(maxdim)       :: imoment,pmoment
      complex(dop)                                             :: tmp
      complex(dop), dimension(gwpdim1,gwpdim1), intent(inout)           :: gfgwp
      complex(dop), dimension(gwpdim1,gwpdim1)                       :: tmpmom
      complex(dop), dimension(gdof,gwpdim1), intent(in)           :: xivec
      complex(dop), dimension(gdof,gwpdim1,gwpdim1), intent(in)      :: zgwp
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(in)      :: agmat
      complex(dop), dimension(gdof,gdof,gwpdim1,gwpdim1), intent(in) :: aigwp
      complex(dop), dimension(gwpdim1,gwpdim1,npg2), intent(inout)     :: gh
      complex(dop), dimension(gwpdim1), intent(in),optional          :: phase
      complex(dop), dimension(gdof,gdof)                             :: ai2
      complex(dop), dimension(gdof)                                  :: z2

if (present(phase)) then
      do p1 = npg1,npg2

!+-----------------------------------------------------------------------+
!     zeroth-order contribution in f : 
!+-----------------------------------------------------------------------+
         call par2moment(p1,gdof,pmoment,pfactor,gwptype)
         do ig = 1,gwpdim1
            call diagonal_momentsd(gdof,agmat(1,1,ig), &
                 xivec(1,ig),phase(ig),&
                 gfgwp(ig,ig),ai2,z2,1)
            tmpmom(ig,ig) = pfactor*moment(pmoment,gdof,gfgwp(ig,ig), &
                                   ai2,z2)
            do jg = ig+1,gwpdim1
               call diagonal_momentsz(gdof,agmat(1,1,jg),agmat(1,1,ig), &
                    xivec(1,jg),xivec(1,ig),phase(jg),phase(ig),&
                    gfgwp(jg,ig),ai2,z2,0,1)
               tmpmom(jg,ig) = pfactor*moment(pmoment,gdof,gfgwp(jg,ig), &
                                      ai2,z2)
               tmpmom(ig,jg) = dconjg(tmpmom(jg,ig))
            enddo
         enddo

         do ig = 1,gwpdim1
            tmp = 2.0_dop*agmat(f,ff,ig) + xivec(f,ig)*xivec(ff,ig)
            do jg = 1,gwpdim1
               gh(jg,ig,p1) = tmp*tmpmom(jg,ig)
            enddo
         enddo

!+-----------------------------------------------------------------------+
!     first-order contribution : 
!+-----------------------------------------------------------------------+
         do f1=1,gdof
            call cpvxi(pmoment,imoment,gdof)
            imoment(f1)=imoment(f1)+1
            do ig = 1,gwpdim1
              call diagonal_momentsd(gdof,agmat(1,1,ig), &
                   xivec(1,ig),phase(ig),&
                   gfgwp(ig,ig),ai2,z2,1)
               tmpmom(ig,ig) = pfactor*moment(imoment,gdof,gfgwp(ig,ig), &
                                   ai2,z2)
               do jg = ig+1,gwpdim1
                 call diagonal_momentsz(gdof,agmat(1,1,jg),agmat(1,1,ig), &
                      xivec(1,jg),xivec(1,ig),phase(jg),phase(ig),&
                      gfgwp(jg,ig),ai2,z2,0,1)
                  tmpmom(jg,ig) =  &
                       pfactor*moment(imoment,gdof,gfgwp(jg,ig), &
                                      ai2,z2)
                  tmpmom(ig,jg) = dconjg(tmpmom(jg,ig))
               enddo
            enddo
            do ig = 1,gwpdim1
               if(ff.eq.f)then
                  tmp=4.0_dop*agmat(f,f1,ig)*xivec(f,ig)
               elseif(ff.ne.f)then
                  tmp=2.0_dop*agmat(f,f1,ig)*xivec(ff,ig)&
                     +2.0_dop*agmat(ff,f1,ig)*xivec(f,ig)
               endif
               do jg = 1,gwpdim1
                  gh(jg,ig,p1) = gh(jg,ig,p1) + tmp*tmpmom(jg,ig)
               enddo
            enddo
         enddo

!+-----------------------------------------------------------------------+
!     second-order contribution : 
!+-----------------------------------------------------------------------+
         do f1=1,gdof
            call cpvxi(pmoment,imoment,gdof)
            imoment(f1)=imoment(f1)+2
!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(ig,jg,ai2,z2)
!$omp do schedule(dynamic)
            do ig = 1,gwpdim1
              call diagonal_momentsd(gdof,agmat(1,1,ig), &
                   xivec(1,ig),phase(ig),&
                   gfgwp(ig,ig),ai2,z2,1)
               tmpmom(ig,ig) = &
                       pfactor*moment(imoment,gdof,gfgwp(ig,ig),&
                                   ai2,z2)
               do jg = ig+1,gwpdim1
                 call diagonal_momentsz(gdof,agmat(1,1,jg),agmat(1,1,ig), &
                      xivec(1,jg),xivec(1,ig),phase(jg),phase(ig),&
                      gfgwp(jg,ig),ai2,z2,0,1)
                  tmpmom(jg,ig) =&
                          pfactor*moment(imoment,gdof,gfgwp(jg,ig),&
                                      ai2,z2)
                  tmpmom(ig,jg) = dconjg(tmpmom(jg,ig))
               enddo
            enddo
!$omp end do
!$omp end parallel

!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(ig,jg,tmp,ai2,z2)
!$omp do schedule(dynamic)
            do ig = 1,gwpdim1
               tmp=4.0_dop*agmat(f,f1,ig)*agmat(ff,f1,ig)
               do jg = 1,gwpdim1
                  gh(jg,ig,p1) = gh(jg,ig,p1) + tmp*tmpmom(jg,ig)
               enddo
            enddo
!$omp end do
!$omp end parallel
            do f2=f1+1,gdof
               call cpvxi(pmoment,imoment,gdof)
               imoment(f1)=imoment(f1)+1
               imoment(f2)=imoment(f2)+1
!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(ig,jg,ai2,z2)
!$omp do schedule(dynamic)
               do ig = 1,gwpdim1
                 call diagonal_momentsd(gdof,agmat(1,1,ig), &
                      xivec(1,ig),phase(ig),&
                      gfgwp(ig,ig),ai2,z2,1)
                  tmpmom(ig,ig) = &
                       pfactor*moment(imoment,gdof,gfgwp(ig,ig),&
                                   ai2,z2)
                  do jg = ig+1,gwpdim1
                    call diagonal_momentsz(gdof,agmat(1,1,jg),agmat(1,1,ig), &
                         xivec(1,jg),xivec(1,ig),phase(jg),phase(ig),&
                         gfgwp(jg,ig),ai2,z2,0,1)
                     tmpmom(jg,ig) =&
                          pfactor*moment(imoment,gdof,gfgwp(jg,ig),&
                                      ai2,z2)
                     tmpmom(ig,jg) = dconjg(tmpmom(jg,ig))
                  enddo
               enddo
!$omp end do
!$omp end parallel

!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(ig,jg,tmp,ai2,z2)
!$omp do schedule(dynamic)
               do ig = 1,gwpdim1
                  tmp=8.0_dop*agmat(f,f1,ig)*agmat(f,f2,ig)
                  do jg = 1,gwpdim1
                     gh(jg,ig,p1) = gh(jg,ig,p1) + tmp*tmpmom(jg,ig)
                  enddo
               enddo
!$omp end do
!$omp end parallel
            enddo
         enddo

      enddo
else
      do p1 = npg1,npg2

!+-----------------------------------------------------------------------+
!     zeroth-order contribution in f : 
!+-----------------------------------------------------------------------+
         call par2moment(p1,gdof,pmoment,pfactor,gwptype)
         do ig = 1,gwpdim1
            tmpmom(ig,ig) = pfactor*moment(pmoment,gdof,gfgwp(ig,ig), &
                                   aigwp(1,1,ig,ig),zgwp(1,ig,ig))
            do jg = ig+1,gwpdim1
               tmpmom(jg,ig) = pfactor*moment(pmoment,gdof,gfgwp(jg,ig), &
                                      aigwp(1,1,jg,ig),zgwp(1,jg,ig))
               tmpmom(ig,jg) = dconjg(tmpmom(jg,ig))
            enddo
         enddo

         do ig = 1,gwpdim1
            tmp = 2.0_dop*agmat(f,ff,ig) + xivec(f,ig)*xivec(ff,ig)
            do jg = 1,gwpdim1
               gh(jg,ig,p1) = tmp*tmpmom(jg,ig)
            enddo
         enddo

!+-----------------------------------------------------------------------+
!     first-order contribution : 
!+-----------------------------------------------------------------------+
         do f1=1,gdof
            call cpvxi(pmoment,imoment,gdof)
            imoment(f1)=imoment(f1)+1
            do ig = 1,gwpdim1
               tmpmom(ig,ig) = pfactor*moment(imoment,gdof,gfgwp(ig,ig), &
                                   aigwp(1,1,ig,ig),zgwp(1,ig,ig))
               do jg = ig+1,gwpdim1
                  tmpmom(jg,ig) =  &
                       pfactor*moment(imoment,gdof,gfgwp(jg,ig), &
                                      aigwp(1,1,jg,ig),zgwp(1,jg,ig))
                  tmpmom(ig,jg) = dconjg(tmpmom(jg,ig))
               enddo
            enddo
            do ig = 1,gwpdim1
               if(ff.eq.f)then
                  tmp=4.0_dop*agmat(f,f1,ig)*xivec(f,ig)
               elseif(ff.ne.f)then
                  tmp=2.0_dop*agmat(f,f1,ig)*xivec(ff,ig)&
                     +2.0_dop*agmat(ff,f1,ig)*xivec(f,ig)
               endif
               do jg = 1,gwpdim1
                  gh(jg,ig,p1) = gh(jg,ig,p1) + tmp*tmpmom(jg,ig)
               enddo
            enddo
         enddo

!+-----------------------------------------------------------------------+
!     second-order contribution : 
!+-----------------------------------------------------------------------+
         do f1=1,gdof
            call cpvxi(pmoment,imoment,gdof)
            imoment(f1)=imoment(f1)+2
            do ig = 1,gwpdim1
               tmpmom(ig,ig) = &
                       pfactor*moment(imoment,gdof,gfgwp(ig,ig),&
                                   aigwp(1,1,ig,ig),zgwp(1,ig,ig))
               do jg = ig+1,gwpdim1
                  tmpmom(jg,ig) =&
                          pfactor*moment(imoment,gdof,gfgwp(jg,ig),&
                                      aigwp(1,1,jg,ig),zgwp(1,jg,ig))
                  tmpmom(ig,jg) = dconjg(tmpmom(jg,ig))
               enddo
            enddo

            do ig = 1,gwpdim1
               tmp=4.0_dop*agmat(f,f1,ig)*agmat(ff,f1,ig)
               do jg = 1,gwpdim1
                  gh(jg,ig,p1) = gh(jg,ig,p1) + tmp*tmpmom(jg,ig)
               enddo
            enddo

            do f2=f1+1,gdof
               call cpvxi(pmoment,imoment,gdof)
               imoment(f1)=imoment(f1)+1
               imoment(f2)=imoment(f2)+1
               do ig = 1,gwpdim1
                  tmpmom(ig,ig) = &
                       pfactor*moment(imoment,gdof,gfgwp(ig,ig),&
                                   aigwp(1,1,ig,ig),zgwp(1,ig,ig))
                  do jg = ig+1,gwpdim1
                     tmpmom(jg,ig) =&
                          pfactor*moment(imoment,gdof,gfgwp(jg,ig),&
                                      aigwp(1,1,jg,ig),zgwp(1,jg,ig))
                     tmpmom(ig,jg) = dconjg(tmpmom(jg,ig))
                  enddo
               enddo

               do ig = 1,gwpdim1
                  tmp=8.0_dop*agmat(f,f1,ig)*agmat(f,f2,ig)
                  do jg = 1,gwpdim1
                     gh(jg,ig,p1) = gh(jg,ig,p1) + tmp*tmpmom(jg,ig)
                  enddo
               enddo
            enddo
         enddo

      enddo
endif
      return
      end subroutine dq2gwp_elements
      
!#######################################################################
!
! coeff : value of "coefficient" e.g. if term is a*dq^2 coeff = a
!
! icalcv = 1: calculate coefficient included in hterm(h). Needed for GWP
!        = 0: Do not calculate coefficient included in hterm(h). 
!
! iflux set to 1 if flux being calculated
!
! ng(maxdim) is an array whose entries are 0, 1, 2, ....  depending on 
!            the order of the derivative appearing for each dof      
!
!#######################################################################

      subroutine kefac(k,m,coeff,ng,icalcv)

      implicit none

      integer(long)                                 :: n,f1,h,nh,hl,hlke,pottyp
      integer(long), intent(in)                     :: k,m
      integer(long), dimension(maxdim), intent(inout) :: ng

      integer(long), intent(inout)                    :: icalcv
      real(dop)                                     :: v,rterm,rfac
      real(dop), intent(inout)                        :: coeff
      real(dop), dimension(1)                       :: rdum

!-----------------------------------------------------------------------      
!     Initialize variables
!-----------------------------------------------------------------------
      do n=1,nspfdof(m)
        ng(n) = 0
      enddo 
      coeff=1.0_dop
      
!-----------------------------------------------------------------------
!     loop over dof's within a given particle, for given m / k :
!     hlke flags the factor containing the KE operator
!-----------------------------------------------------------------------
      hlke=0
      do n=1,nspfdof(m)

         f1 = spfdof(n,m)
         h = hamilton(f1,k)
        
         if(h.eq.f1) cycle

         v=0.0_dop
         rterm=1.0_dop 
         do nh=1,htmfac(h)
           hl=hterm(nh,h)
           if (hl .eq. 0) then
              v=v+rterm
              rterm=1.0_dop
            else
            rfac=0.0_dop
! hopfile 301 are numbers, 302 includes the "functions" -1 etc.
               if (hopfile(hl).eq.301 .or. hopfile(hl) .eq. 302) then
                  f1=1
                  rdum(1)=-9999.0
                  call callanld(rfac,rdum(1),hopilab(hl),hopfile(hl), &
                       hopipar(1,hl),hoprpar(1,hl),rdum(1),pottyp, &
                       hoplab(hl),mass(f1))
                  rterm=rterm*rfac
               else if (hopfile(hl).eq.3 .or.  &
                           hopfile(hl) .eq. 307) then
                  hlke=hl
               else if (hopfile(hl).ne.3 .and.  &
                           hopfile(hl) .ne. 307) then
                  routine='Kinetic_Elements'
                  message='Only coefficients and KE operators allowed: ' &
                  //hoplab(hl) 
                  call errormsg
               endif
            endif
         enddo
 
         v=v+rterm     
         if (abs(v - 0.0_dop) .lt. tiny) icalcv=0

!---------------------------------------------------------------------------
!     Here, "dqgwp^1"/"dqgwp^2" appear as new options, and the "ng" 
!     entries are defined accordingly: 
!---------------------------------------------------------------------------
         if (hlke .gt. 0) then
            if(hoplab(hlke) .eq. "dqgwp^1") then 
               ng(n) = 1
               hlke=0
            elseif(hoplab(hlke) .eq. "dqgwp^2") then 
               ng(n) = 2
               hlke=0
            elseif(hoplab(hlke) .eq. "p2_cl") then 
               ng(n) = 2
               hlke=0
            elseif(hoplab(hlke) .eq. "p2_clm") then 
               ng(n) = 2
               hlke=0
            elseif(hoplab(hlke) .eq. 'KEgwp') then
               ng(n) = 2
               coeff = -0.5_dop*coeff/mass(f1)
               hlke=0
            else
               routine='Kefac'
               message='Unknown KE operator for GWP dof :'// &
                          hoplab(hlke)
               call errormsg
            endif 
         endif

         if (icalcv .gt. 0) coeff=coeff*v

      enddo 


      return
      end subroutine kefac

!#######################################################################
! New version (Benjamin, December 2008)
!#######################################################################

      subroutine potential_elements2s(m,gdof, &
                 dim1,npg1,npg2,qcenter1,h,k,gh,&
                 gfgwp,aigwp,zgwp,mom1cl)
!+-----------------------------------------------------------------------+ 
!     potential type terms 
!+-----------------------------------------------------------------------+

      implicit   none 

      integer(long)                                            :: ig,jg,n,n1,n2,f,&
                                                                  p1,s,s1,f1,f2,&
                                                                  pfactor
      integer(long), intent(in)                                :: dim1,m,gdof,k,&
                                                                  npg1,npg2
      integer(long), intent(inout)                               :: h
      integer(long), dimension(maxdim)                         :: imoment,pmoment
      real(dop)                                                :: v0
      real(dop), dimension(maxdim)                             :: gpoint,deriv1
      real(dop), dimension(gdof,dim1), intent(in)              :: qcenter1
      real(dop), dimension(maxdim,maxdim)                      :: deriv2
      real(dop), dimension(:,:,:), allocatable                 :: deriv3
      real(dop), dimension(:,:,:,:), allocatable               :: deriv4
      real(dop), dimension(gdof,dim1), intent(out)             :: mom1cl

      complex(dop)                                             :: tmp
      complex(dop), dimension(dim1,dim1), intent(in)           :: gfgwp
      complex(dop), dimension(dim1,dim1,npg2), intent(inout)     :: gh
      complex(dop), dimension(gdof,dim1,dim1), intent(in)      :: zgwp
      complex(dop), dimension(gdof,gdof,dim1,dim1), intent(in) :: aigwp
      integer(long)     :: ierr
      character(len=10) :: cerr
      integer(long), save :: tid=0

      if (ldd .and. ldbsmall) then
         routine = 'potential_elements2s'
         message = 'Cannot use Midpoint integrals with local DBs'
         call errormsg
      endif

      call get_timer(tid, 'Gh_elems: symm LHA')
      call start_timer(tid)

      v0=0.0_dop
      deriv1=0.0_dop
      deriv2=0.0_dop

!-----------------------------------------------------------------------
! allocate memory for 3rd and 4th order derivatives if required
!-----------------------------------------------------------------------
      cerr = 'deriv3'
      if (gwpintordm(m) .ge. 3) then
         allocate(deriv3(maxdim,maxdim,maxdim),stat=ierr)
         deriv3=0.0_dop
      else
         allocate(deriv3(0,0,0),stat=ierr)
      endif
      if (ierr > 0) go to 1000
      cerr = 'deriv4'
      if (gwpintordm(m) .eq. 4) then
         routine = 'potential_elements2s'
         message = ' 4th Order Integral not yet implemented'
         call errormsg
!         allocate(deriv4(maxdim,maxdim,maxdim,maxdim),stat=ierr)
!         deriv4=0.0_dop
!       else
         allocate(deriv4(0,0,0,0),stat=ierr)
      endif
      if (ierr > 0) go to 1000

!-----------------------------------------------------------------------
! get hamiltonian operator for which matrix representation  is wanted
!-----------------------------------------------------------------------
      f = spfdof(1,m)
      h = hamilton(f,k)
      s = ki(k)
      s1 = kf(k)

      do jg=1,dim1
         do ig=jg,dim1

! map Gaussian dof to actual dof in gpoint array
! gpoint, deriv1, deriv2 are in space of system degrees of freedom.
            call gwp2gpoint1(qcenter1(1,ig),qcenter1(1,jg),gpoint,gdof,&
                 maxdim,m)

            call lhaderivs(h,gpoint,v0,deriv1,deriv2,deriv3,deriv4)
 
!+-----------------------------------------------------------------------+
!     store 1st derivative
!+-----------------------------------------------------------------------+
            if (ig .eq. jg) then
               do n=1,gdof
                  f=spfdof(n,m)
                  mom1cl(n,jg) = deriv1(f)
               enddo
            endif

            do p1=npg1,npg2
               call par2moment(p1,gdof,pmoment,pfactor,gwptype)

!------------------------------------------------------------------------+
!     zeroth-order contribution :
!+-----------------------------------------------------------------------+
               tmp = v0

               if (gwpintordm(m) .ge. 1) then
                  do n=1,gdof
                     f=spfdof(n,m)
                     tmp = tmp - deriv1(f)*gpoint(f)
                  enddo
               endif
               if (gwpintordm(m) .ge. 2) then
                  do n=1,gdof
                     f=spfdof(n,m)
                     do n1=1,gdof
                        f1=spfdof(n1,m)
                        tmp = tmp + 0.5_dop*deriv2(f,f1)*gpoint(f)*gpoint(f1)
                     enddo
                  enddo
               endif
               if (gwpintordm(m) .ge. 3) then
                  do n=1,gdof
                     f=spfdof(n,m)
                     do n1=1,gdof
                        f1=spfdof(n1,m)
                        do n2=1,gdof
                           f2=spfdof(n2,m)
                           tmp = tmp - (1/6.0_dop)*deriv3(f,f1,f2)*&
                                 gpoint(f)*gpoint(f1)*gpoint(f2)
                        enddo
                     enddo
                  enddo
               endif

               gh(ig,jg,p1) = tmp*pfactor*&
                     moment(pmoment,gdof,gfgwp(ig,jg),&
                            aigwp(1,1,ig,jg),zgwp(1,ig,jg))

               if (gwpintordm(m) .lt. 1) cycle
!+-----------------------------------------------------------------------+
!     first-order contribution :
!+-----------------------------------------------------------------------+
               do n=1,gdof
                  f=spfdof(n,m)
                  call cpvxi(pmoment,imoment,gdof)
                  imoment(n)=imoment(n)+1
                  tmp = deriv1(f)

                  if (gwpintordm(m) .ge. 2) then
                     do n1=1,gdof
                        f1=spfdof(n1,m)
                        tmp = tmp - deriv2(f,f1)*gpoint(f1)
                     enddo
                  endif
                  if (gwpintordm(m) .ge. 3) then
                     do n1=1,gdof
                        f1=spfdof(n1,m)
                        do n2=1,gdof
                           f2=spfdof(n2,m)
                           tmp = tmp + 0.5_dop*deriv3(f,f1,f2)*&
                                 gpoint(f1)*gpoint(f2)
                        enddo
                     enddo
                  endif

                  gh(ig,jg,p1) = gh(ig,jg,p1) +&
                      tmp*pfactor*moment(imoment,gdof,gfgwp(ig,jg),&
                      aigwp(1,1,ig,jg),zgwp(1,ig,jg))
               enddo

               if (gwpintordm(m) .lt. 2) cycle
!+-----------------------------------------------------------------------+
!     second-order contribution :
!+-----------------------------------------------------------------------+
               do n=1,gdof
                  f=spfdof(n,m)
                  do n1=1,gdof
                     f1=spfdof(n1,m)
                     call cpvxi(pmoment,imoment,gdof)
                     imoment(n)=imoment(n)+1
                     imoment(n1)=imoment(n1)+1
                     tmp=0.5_dop*deriv2(f,f1)

                     if (gwpintordm(m) .eq. 3) then
                        do n2=1,gdof
                           f2=spfdof(n2,m)
                           tmp = tmp - 0.5_dop*deriv3(f,f1,f2)*gpoint(f2)
                        enddo
                     endif

                     gh(ig,jg,p1) = gh(ig,jg,p1) +&
                         tmp*pfactor*moment(imoment,gdof,gfgwp(ig,jg),&
                         aigwp(1,1,ig,jg),zgwp(1,ig,jg))               
                  enddo
               enddo

               if (gwpintordm(m) .lt. 3) cycle
!+-----------------------------------------------------------------------+
!     third-order contribution :
!+-----------------------------------------------------------------------+
               do n=1,gdof
                  f=spfdof(n,m)
                  do n1=1,gdof
                     f1=spfdof(n1,m)
                     do n2=1,gdof
                        f2=spfdof(n2,m)
                        call cpvxi(pmoment,imoment,gdof)
                        imoment(n)=imoment(n)+1
                        imoment(n1)=imoment(n1)+1
                        imoment(n2)=imoment(n2)+1
                        tmp=(1/6.0_dop)*deriv3(f,f1,f2)
 
                        gh(ig,jg,p1) = gh(ig,jg,p1) +&
                           tmp*pfactor*moment(imoment,gdof,&
                           gfgwp(ig,jg),&
                           aigwp(1,1,ig,jg),zgwp(1,ig,jg))
                     enddo
                  enddo
               enddo      

            enddo
         enddo

      enddo

!-----------------------------------------------------------------------
! Fill matrix
!-----------------------------------------------------------------------
      do jg=1,dim1
         do ig=jg+1,dim1
            do p1=npg1,npg2
               gh(jg,ig,p1) = dconjg(gh(ig,jg,p1) )
            enddo
         enddo
      enddo

      if (allocated(deriv3)) deallocate(deriv3)
      if (allocated(deriv4)) deallocate(deriv4)

      call stop_timer(tid)

      return
 1000 write(ilog,'(a)') 'Error allocating memory in LHA_MOMS :',cerr
      write(ilog,'(a)') 'Error Code :',ierr
      stop
      end subroutine potential_elements2s
      
!#######################################################################

      subroutine potential_elements2(m,gdof, &
                 dim1,dim2,npg1,npg2,qcenter1,qcenter2,h,k,gh,&
                 gfgwp,aigwp,zgwp)
!+-----------------------------------------------------------------------+ 
!     potential type terms 
!+-----------------------------------------------------------------------+

      implicit   none 

      integer(long)                                            :: ig,jg,n,n1,n2,&
                                                                  f,p1,s,s1,f1,f2,&
                                                                  pfactor
      integer(long), intent(in)                                :: dim1,m,gdof,k,&
                                                                  npg1,npg2,dim2
      integer(long), intent(inout)                               :: h
      integer(long), dimension(maxdim)                         :: imoment,pmoment
      real(dop)                                                :: v0
      real(dop), dimension(maxdim)                             :: gpoint,deriv1
      real(dop), dimension(gdof,dim1), intent(in)              :: qcenter1
      real(dop), dimension(gdof,dim2), intent(in)              :: qcenter2
      real(dop), dimension(maxdim,maxdim)                      :: deriv2
      real(dop), dimension(:,:,:), allocatable                 :: deriv3
      real(dop), dimension(:,:,:,:), allocatable               :: deriv4

      complex(dop)                                             :: tmp
      complex(dop), dimension(dim1,dim2), intent(in)           :: gfgwp
      complex(dop), dimension(dim1,dim2,npg2), intent(inout)     :: gh
      complex(dop), dimension(gdof,dim1,dim2), intent(in)      :: zgwp
      complex(dop), dimension(gdof,gdof,dim1,dim2), intent(in) :: aigwp

      integer(long)     :: ierr
      character(len=10) :: cerr
      integer(long), save :: tid=0

      call get_timer(tid, 'Gh_elems: symm LHA')
      call start_timer(tid)

      v0=0.0_dop
      deriv1=0.0_dop
      deriv2=0.0_dop

!-----------------------------------------------------------------------
! allocate memory for 3rd and 4th order derivatives if required
!-----------------------------------------------------------------------
      cerr = 'deriv3'
      if (gwpintordm(m) .ge. 3) then
         allocate(deriv3(maxdim,maxdim,maxdim),stat=ierr)
         deriv3=0.0_dop
      else
         allocate(deriv3(0,0,0),stat=ierr)
      endif
      if (ierr > 0) go to 1000
      cerr = 'deriv4'
      if (gwpintordm(m) .eq. 4) then
         routine = 'potential_elements2s'
         message = ' 4th Order Integral not yet implemented'
         call errormsg
!         allocate(deriv4(maxdim,maxdim,maxdim,maxdim),stat=ierr)
!         deriv4=0.0_dop
!       else
         allocate(deriv4(0,0,0,0),stat=ierr)
      endif
      if (ierr > 0) go to 1000

! get hamiltonian operator for which matrix representation  is wanted
      f = spfdof(1,m)
      h = hamilton(f,k)
      s = ki(k)
      s1 = kf(k)

      do jg=1,dim2
         do ig=1,dim1

! map Gaussian dof to actual dof in gpoint array
! gpoint, deriv1, deriv2 are in space of system degrees of freedom.
            call gwp2gpoint1(qcenter1(1,ig),qcenter2(1,jg),gpoint,&
                 gdof,maxdim,m)

            call lhaderivs(h,gpoint,v0,deriv1,deriv2,deriv3,deriv4)

            do p1=npg1,npg2
               call par2moment(p1,gdof,pmoment,pfactor,gwptype)

!------------------------------------------------------------------------+
!     zeroth-order contribution :
!+-----------------------------------------------------------------------+
               tmp = v0

               if (gwpintordm(m) .ge. 1) then
                  do n=1,gdof
                     f=spfdof(n,m)
                     tmp = tmp - deriv1(f)*gpoint(f)
                  enddo
               endif
               if (gwpintordm(m) .ge. 2) then
                  do n=1,gdof
                     f=spfdof(n,m)
                     do n1=1,gdof
                        f1=spfdof(n1,m)
                        tmp = tmp + 0.5_dop*deriv2(f,f1)*gpoint(f)*gpoint(f1)
                     enddo
                  enddo
               endif
               if (gwpintordm(m) .ge. 3) then
                  do n=1,gdof
                     f=spfdof(n,m)
                     do n1=1,gdof
                        f1=spfdof(n1,m)
                        do n2=1,gdof
                           f2=spfdof(n2,m)
                           tmp = tmp - (1/6.0_dop)*deriv3(f,f1,f2)*&
                                 gpoint(f)*gpoint(f1)*gpoint(f2)
                        enddo
                     enddo
                  enddo
               endif

               gh(ig,jg,p1) = tmp*pfactor*&
                     moment(pmoment,gdof,gfgwp(ig,jg),&
                            aigwp(1,1,ig,jg),zgwp(1,ig,jg))

               if (gwpintordm(m) .lt. 1) cycle
!+-----------------------------------------------------------------------+
!     first-order contribution :
!+-----------------------------------------------------------------------+
               do n=1,gdof
                  f=spfdof(n,m)
                  call cpvxi(pmoment,imoment,gdof)
                  imoment(n)=imoment(n)+1
                  tmp = deriv1(f)

                  if (gwpintordm(m) .ge. 2) then
                     do n1=1,gdof
                        f1=spfdof(n1,m)
                        tmp = tmp - deriv2(f,f1)*gpoint(f1)
                     enddo
                  endif
                  if (gwpintordm(m) .ge. 3) then
                     do n1=1,gdof
                        f1=spfdof(n1,m)
                        do n2=1,gdof
                           f2=spfdof(n2,m)
                           tmp = tmp + 0.5_dop*deriv3(f,f1,f2)*&
                                 gpoint(f1)*gpoint(f2)
                        enddo
                     enddo
                  endif

                  gh(ig,jg,p1) = gh(ig,jg,p1) +&
                      tmp*pfactor*moment(imoment,gdof,gfgwp(ig,jg),&
                      aigwp(1,1,ig,jg),zgwp(1,ig,jg))
               enddo

               if (gwpintordm(m) .lt. 2) cycle
!+-----------------------------------------------------------------------+
!     second-order contribution :
!+-----------------------------------------------------------------------+
               do n=1,gdof
                  f=spfdof(n,m)
                  do n1=1,gdof
                     f1=spfdof(n1,m)
                     call cpvxi(pmoment,imoment,gdof)
                     imoment(n)=imoment(n)+1
                     imoment(n1)=imoment(n1)+1
                     tmp=0.5_dop*deriv2(f,f1)

                     if (gwpintordm(m) .eq. 3) then
                        do n2=1,gdof
                           f2=spfdof(n2,m)
                           tmp = tmp - 0.5_dop*deriv3(f,f1,f2)*gpoint(f2)
                        enddo
                     endif

                     gh(ig,jg,p1) = gh(ig,jg,p1) +&
                         tmp*pfactor*moment(imoment,gdof,gfgwp(ig,jg),&
                         aigwp(1,1,ig,jg),zgwp(1,ig,jg))               
                  enddo
               enddo

               if (gwpintordm(m) .lt. 3) cycle
!+-----------------------------------------------------------------------+
!     third-order contribution :
!+-----------------------------------------------------------------------+
               do n=1,gdof
                  f=spfdof(n,m)
                  do n1=1,gdof
                     f1=spfdof(n1,m)
                     do n2=1,gdof
                        f2=spfdof(n2,m)
                        call cpvxi(pmoment,imoment,gdof)
                        imoment(n)=imoment(n)+1
                        imoment(n1)=imoment(n1)+1
                        imoment(n2)=imoment(n2)+1
                        tmp=(1/6.0_dop)*deriv3(f,f1,f2)
 
                        gh(ig,jg,p1) = gh(ig,jg,p1) +&
                           tmp*pfactor*moment(imoment,gdof,&
                           gfgwp(ig,jg),&
                           aigwp(1,1,ig,jg),zgwp(1,ig,jg))
                     enddo
                  enddo
               enddo      

! end loop over p1
!              gh(jg,ig,p1) = dconjg(gh(ig,jg,p1) )
            enddo
         enddo

!     if (ig.eq.jg)then 
!        lagrangian(ig) = lagrangian(ig) - gh(ig,ig,1)
!     endif

      enddo

      if (allocated(deriv3)) deallocate(deriv3)
      if (allocated(deriv4)) deallocate(deriv4)

      call stop_timer(tid)

      return
 1000 write(ilog,'(a)') 'Error allocating memory in LHA_MOMS :',cerr
      write(ilog,'(a)') 'Error Code :',ierr

      return
      end subroutine potential_elements2
      
!#######################################################################
!
! Flux operator only ever used for analysis. Hence it is only required
! to calculate the lowest order gh matrix (i.e. <gi | h | gj >)
!
!#######################################################################

      subroutine flux_gwp(gh,agmat,xivec,phigmat,n,gdof,gwpdim1,xc,mass,&
                          gwptype)

      integer(long), intent(in) :: gdof,gwpdim1,n,gwptype
      integer(long)             :: e,e1,n1
      real(dop), intent(in)     :: xc,mass
      complex(dop)              :: ztmp,ztmp1,zgval,ag,xi
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(in)  :: agmat
      complex(dop), dimension(gdof,gwpdim1), intent(in)       :: xivec
      complex(dop), dimension(gwpdim1), intent(in)            :: phigmat
      complex(dop), dimension(gwpdim1,gwpdim1), intent(inout)      :: gh

      if (gwptype .lt. 2 .or. gwptype .gt. 3) then
         routine='Flux_gwp (Gh_elements)'
         message = 'Flux can only be calculated for a separable GWP'
         call errormsg
      endif

      do e=1,gwpdim1
         do e1 = e,gwpdim1
! independent GWPs diagonal elements only
            if (gwpmodel .eq. 2 .and. e1 .ne. e) cycle

! Flux along DoF f
            zgval = dconjg(phigmat(e)) + phigmat(e1)
            zgval = zgval + (dconjg(agmat(n,n,e)) + agmat(n,n,e1))*xc*xc
            zgval = zgval + (dconjg(xivec(n,e)) + xivec(n,e1))*xc

            if (gwptype .eq. 3) then
               ztmp = dconjg(xivec(n,e)) - xivec(n,e1) 
            else 
               ztmp = dconjg(xivec(n,e)) - xivec(n,e1)
               ztmp1 = dconjg(agmat(n,n,e)) - agmat(n,n,e1)
               ztmp = 2.0d0*(ztmp1)*xc + ztmp 
            endif

! Integrate over other DoFs
            do n1=1,gdof
               if (n1 .eq. n) cycle

               ag = dconjg(agmat(n1,n1,e)) + agmat(n1,n1,e1)
               xi = dconjg(xivec(n1,e)) + xivec(n1,e1)
               zgval = zgval - xi**2/(4.0_dop*ag)
               ztmp = ztmp*sqrt(-pi/ag)
            enddo
            gh(e,e1) = ztmp * exp(zgval)
            if (abs(gh(e,e1)) .lt. 1.0d-16) gh(e,e1) = 0.0d0
            gh(e,e1) = dcmplx(0,1.0_dop)*gh(e,e1)
            gh(e,e1) = gh(e,e1) / (2.0d0*mass)
            if (e1 .ne. e) gh(e1,e) = dconjg(gh(e,e1))
         enddo
      enddo

      return
      end subroutine flux_gwp

!#######################################################################
! New version (Benjamin, December 2008)
!#######################################################################

      subroutine lhaderivs(h,q0,v0,deriv1,deriv2,deriv3,deriv4)

!+---------------------------------------------------------------------+
!     calculates potential, first and second derivatives at a given
!     point in coordinate space for the operator h.
!     q0(f) contains the point, in the system dofs
!
! only upper part of derivative tensors calculated
!+---------------------------------------------------------------------+

      implicit none

      integer(long)                        :: m,f,n,n1,f1,dertyp,pottyp,md,&
                                              f2,f3,n2,n3
      integer(long), intent(in)            :: h
      integer(long), dimension(maxdim)     :: order
      real(dop), intent(inout)                                  :: v0
      real(dop), dimension(maxdim)                            :: ppoint
      real(dop), dimension(maxdim), intent(in)                :: q0
      real(dop), dimension(maxdim), intent(inout)               :: deriv1
      real(dop), dimension(:,:), intent(inout)                  :: deriv2
      real(dop), dimension(:,:,:), intent(inout)                :: deriv3
      real(dop), dimension(:,:,:,:), intent(inout)              :: deriv4
      complex(dop)                                            :: vz

! ... find mode
      f=htmdof(h)
      m=htmmode(h)
      md=htmmuld(h)
      if (f .gt. 0) m=dofspf(f)
      if (md .gt. 0) m=1

      v0=0.0
      deriv1=0.0
      deriv2=0.0

! ... energy
      call poteval (h,v0,q0,vz,ppoint,pottyp)

! ... first derivatives (always required):

      keepneigh=.true.
      do n=1,nspfdof(m)
         f=spfdof(n,m)
         call zerovxi(order,maxdim)
         order(f)=1

         call dereval (h,deriv1(f),q0,order,ppoint,dertyp,dermethod)
      enddo

! ... second derivatives :
      if (gwpintordm(m) .ge. 2) then
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            do n1=1,n
               f1=spfdof(n1,m)
               call zerovxi(order,maxdim)
               order(f)=1
               order(f1)=order(f1)+1

               call dereval (h,deriv2(f1,f),q0,order,ppoint,dertyp, &
                    dermethod)
            enddo
         enddo
      endif

! ... third derivatives

      if (gwpintordm(m) .ge. 3) then
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            do n1=1,n
               f1=spfdof(n1,m)
               do n2=1,n1
                  f2=spfdof(n2,m)
                  call zerovxi(order,maxdim)
                  order(f)=1
                  order(f1)=order(f1)+1
                  order(f2)=order(f2)+1

                  call dereval (h,deriv3(f2,f1,f),q0,order,ppoint, &
                       dertyp,dermethod)
               enddo
            enddo
         enddo
      endif

! ... fourth derivatives

      if (gwpintordm(m) .eq. 4) then
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            do n1=1,n
               f1=spfdof(n1,m)
               do n2=1,n1
                  f2=spfdof(n2,m)
                  do n3=1,n2
                     f3=spfdof(n3,m)
                     call zerovxi(order,maxdim)
                     order(f)=1
                     order(f1)=order(f1)+1
                     order(f2)=order(f2)+1
                     order(f3)=order(f3)+1

                     call dereval (h,deriv4(f3,f2,f1,f),q0,order,ppoint, &
                          dertyp,dermethod)
                  enddo
               enddo
            enddo
         enddo
      endif
      keepneigh=.false.

      return
      end subroutine lhaderivs

!#######################################################################

      subroutine sumhung(koeff,gh,hung,gwpdim1,npgm)

      implicit none

      integer(long)                     :: e,e1,p1
      integer(long), intent(in)         :: gwpdim1,npgm
      complex(dop), dimension(gwpdim1,gwpdim1,npgm), intent(inout) :: hung
      complex(dop), dimension(gwpdim1,gwpdim1,npgm), intent(in)    :: gh
      complex(dop), intent(in)                                     :: koeff

      do e=1,gwpdim1
         do e1=1,gwpdim1
            do p1=1,npgm
               hung(e1,e,p1) = hung(e1,e,p1) + koeff*gh(e1,e,p1)
            enddo
         enddo
      enddo

      return
      end subroutine sumhung

      end module gh_elementsmod


