!######################################################################r
!
! GWPTRJMOD
!
! Module to use trajectories along which GWPs can be driven.
!
!#######################################################################

      module gwptrjmod

use decimal, only: long, dop
use global
use versions, only: progver
use griddatmod
use channels
use maxv
use psidef
use logdat
use dirdyn
use gwptrj
use dd_db, only: ddx2q, ddv2p
use gwplib
use splinepotmod

      implicit none
      public :: gwptrjfit,gwptrj_param,gwptrj_pq

      contains

!#######################################################################

      subroutine gwptrjfit

      implicit none

      integer(long)     :: ilbl,jlbl,klbl,llbl,iend
      integer(long)     :: slen
      integer(long)     :: np,nc,ns,nt,npnts
      logical(kind=4)   :: lexist
      character(len=c5) :: filename,extcmd,string
      character(len=c2) :: rdtag,wrtag

!-----------------------------------------------------------------------
! see if gwptrj file present in name directory, i.e. fits already made
!-----------------------------------------------------------------------
      filename = gwptrjdir(1:gwptrjlaenge)//'/gwptrj_spl.dat'
      ilbl=slen(filename)
      inquire(file=filename,exist=lexist)

      if (lexist) then
! read file
         open(igwptrj,file=filename,form='unformatted',status='old',err=900)
         write(ilog,'(a)') 'Trajectory data taken from: '
         write(ilog,'(a)') filename(1:ilbl)

         do
  10        continue
            read(igwptrj,iostat=iend,err=10) rdtag
            if (iend .lt. 0) exit
            if (rdtag(1:1) .ne. '<') cycle

            ilbl=2
            jlbl=index(rdtag,'>')-1
            if (rdtag(ilbl:jlbl) .eq. 'gwptrjnp') then
               read(igwptrj,err=999) (gwptrjnp(nt),nt=1,ngwptrj)

            else if (rdtag(ilbl:jlbl) .eq. 'gwptrjtdat') then
               read(igwptrj,err=999) ((gwptrjtdat(np,nt),&
                        np=1,ngwptrjpnts),nt=1,ngwptrj)

            else if (rdtag(ilbl:jlbl) .eq. 'gwptrjxdat') then
               read(igwptrj,err=999) (((gwptrjxdat(np,nc,nt),&
                        np=1,ngwptrjpnts),nc=1,ngwptrjcoo),nt=1,ngwptrj)

            elseif (rdtag(ilbl:jlbl) .eq. 'gwptrjxspl') then
               read(igwptrj,err=999) (((gwptrjxspl(np,nc,nt),&
                        np=1,ngwptrjpnts),nc=1,ngwptrjcoo),nt=1,ngwptrj)

            else if (rdtag(ilbl:jlbl) .eq. 'gwptrjpdat') then
               read(igwptrj,err=999) (((gwptrjpdat(np,nc,nt),&
                        np=1,ngwptrjpnts),nc=1,ngwptrjcoo),nt=1,ngwptrj)

            elseif (rdtag(ilbl:jlbl) .eq. 'gwptrjpspl') then
               read(igwptrj,err=999) (((gwptrjpspl(np,nc,nt),&
                        np=1,ngwptrjpnts),nc=1,ngwptrjcoo),nt=1,ngwptrj)

            else if (rdtag(ilbl:jlbl) .eq. 'gwptrjedat') then
               read(igwptrj,err=999) ((gwptrjedat(np,nt),&
                        np=1,ngwptrjpnts),nt=1,ngwptrj)

            else if (rdtag(ilbl:jlbl) .eq. 'gwptrjespl') then
               read(igwptrj,err=999) ((gwptrjespl(np,nt),&
                        np=1,ngwptrjpnts),nt=1,ngwptrj)

            elseif (rdtag(ilbl:jlbl) .eq. 'gwptrjcddat') then
               read(igwptrj,err=999) (((gwptrjcddat(np,ns,nt),&
                        np=1,ngwptrjpnts),ns=1,ngwptrjst),nt=1,ngwptrj)

            elseif (rdtag(ilbl:jlbl) .eq. 'gwptrjcdspl') then
               read(igwptrj,err=999) (((gwptrjcdspl(np,ns,nt),&
                        np=1,ngwptrjpnts),ns=1,ngwptrjst),nt=1,ngwptrj)

            else if (rdtag(ilbl:jlbl) .eq. 'gwptrjczdat') then
               read(igwptrj,err=999) (((gwptrjczdat(np,ns,nt),&
                        np=1,ngwptrjpnts),ns=1,ngwptrjst),nt=1,ngwptrj)

            else if (rdtag(ilbl:jlbl) .eq. 'gwptrjczspl') then
               read(igwptrj,err=999) (((gwptrjczspl(np,ns,nt),&
                        np=1,ngwptrjpnts),ns=1,ngwptrjst),nt=1,ngwptrj)

            else if (rdtag(ilbl:jlbl) .eq. 'end gwptrj data') then
                exit

            endif

         enddo

         close(igwptrj)

      else
         write(ilog,'(a)') 'Trajectory files taken from: '
         write(ilog,'(a)') gwptrjdir(1:gwptrjlaenge)
         write(ilog,'(a)') 'Data written to: '
         write(ilog,'(a)') filename(1:ilbl)

         open(igwptrj,file=filename,form='unformatted',status='new',err=900)

         jlbl=slen(strpid)
         filename=gwptrjdir(1:gwptrjlaenge)//'/gwptrj.ls-'//strpid(1:jlbl)
         ilbl=slen(filename)
         extcmd = "ls "//gwptrjdir(1:gwptrjlaenge)//" > "//filename(1:ilbl)
         call excmd(extcmd)
         open(idump,file=filename,status='old',form='formatted',err=900)

         gwptrjtdat=0.0_dop
         gwptrjxdat=0.0_dop
         gwptrjxspl=0.0_dop
         gwptrjpdat=0.0_dop
         gwptrjpspl=0.0_dop
         gwptrjedat=0.0_dop
         gwptrjespl=0.0_dop
         gwptrjcddat=0.0_dop
         gwptrjcdspl=0.0_dop
         gwptrjczdat=0.0_dop
         gwptrjczspl=0.0_dop
         rewind(idump)
         do 
            read(idump,'(a32)',iostat=iend,err=910) string
            if (iend .lt. 0) exit
            klbl=slen(string)

            if (string(klbl-3:klbl) .ne. 'xdat' .and. &
                string(klbl-3:klbl) .ne. 'pdat' .and. &
                string(klbl-3:klbl) .ne. 'edat' .and. &
                string(klbl-4:klbl) .ne. 'cidat') cycle

            filename=gwptrjdir(1:gwptrjlaenge)//'/'//string(1:klbl)
            ilbl=slen(filename)
            open(itmpin,file=filename,status='old',form='formatted',err=900)
! file name is nc.nt.xdat nc.nt.pdat nt.edat or dns.nt.cidat
! llbl:jlbl delimit nt
            jlbl=index(string(1:klbl),'.',.true.)
            llbl=index(string(1:klbl),'.')
            if (llbl .eq. jlbl) llbl=0

            nc=0
            nt=0
            ns=0
! trajectory no.
            read(string(llbl+1:jlbl-1),*) nt
            if (nt .le. 0) then
               routine="Gwptrjfit"
               message="Trajectories must have positive integers starting at 1"
               call errormsg
            endif
            if (nt .gt. ngwp) cycle

! adiabatic state no. (ns). NB Ground State is 0
! OR
! coordinate no. (nc)
            if (llbl .gt. 0) then
               if (string(1:1) .eq. 'd') then
                  read(string(2:llbl-1),*) ns
                  ns=ns+1
               else
                  read(string(1:llbl-1),*) nc
               endif
            endif

! read data and calculate derivatives for spline fits
            read(itmpin,*,err=910) npnts
            gwptrjnp(nt) = npnts
            if (string(klbl-3:klbl) .eq. 'xdat') then
               do np=1,npnts
                  read(itmpin,*) gwptrjtdat(np,nt),gwptrjxdat(np,nc,nt) 
                  gwptrjtdat(np,nt)=gwptrjtdat(np,nt)*fs
               enddo

               call spline_css(gwptrjtdat(1,nt),gwptrjxdat(1,nc,nt), &
                    npnts,1.d30,1.d30,gwptrjxspl(1,nc,nt))

            else if (string(klbl-3:klbl) .eq. 'pdat') then
               do np=1,npnts
                  read(itmpin,*) gwptrjtdat(np,nt),gwptrjpdat(np,nc,nt) 
                  gwptrjtdat(np,nt)=gwptrjtdat(np,nt)*fs
               enddo

               call spline_css(gwptrjtdat(1,nt),gwptrjpdat(1,nc,nt), &
                    npnts,1.d30,1.d30,gwptrjpspl(1,nc,nt))

            else if (string(klbl-3:klbl) .eq. 'edat') then
               do np=1,npnts
                  read(itmpin,*) gwptrjtdat(np,nt),gwptrjedat(np,nt)
                  gwptrjtdat(np,nt)=gwptrjtdat(np,nt)*fs
               enddo

               call spline_css(gwptrjtdat(1,nt),gwptrjedat(1,nt), &
                    npnts,1.d30,1.d30,gwptrjespl(1,nt))

            else if (string(klbl-4:klbl) .eq. 'cidat') then
               do np=1,npnts
                  read(itmpin,*) gwptrjtdat(np,nt),gwptrjcddat(np,ns,nt), &
                               gwptrjczdat(np,ns,nt) 
                  gwptrjtdat(np,nt)=gwptrjtdat(np,nt)*fs
               enddo

               call spline_css(gwptrjtdat(1,nt),gwptrjcddat(1,ns,nt), &
                    npnts,1.d30,1.d30,gwptrjcdspl(1,ns,nt))
               call spline_css(gwptrjtdat(1,nt),gwptrjczdat(1,ns,nt), &
                    npnts,1.d30,1.d30,gwptrjczspl(1,ns,nt))

            endif

            close(itmpin)
         enddo
         close(idump,status='delete')

!-----------------------------------------------------------------------
! write gwptrj file
!-----------------------------------------------------------------------
! write gwptrj file
         write(igwptrj) progver

!-----------------------------------------------------------------------
! array sizes
!-----------------------------------------------------------------------
         wrtag= '<array dimensions>'
         write(igwptrj) wrtag

         wrtag= '<ngwptrjpnts>'
         write(igwptrj) wrtag
         write(igwptrj) ngwptrjpnts
         wrtag= '<ngwptrjcoo>'
         write(igwptrj) wrtag
         write(igwptrj) ngwptrjcoo
         wrtag= '<ngwptrj>'
         write(igwptrj) wrtag
         write(igwptrj) ngwptrj
         wrtag= '<ngwptrjst>'
         write(igwptrj) wrtag
         write(igwptrj) ngwptrjst

         wrtag= '<end array dimensions>'
         write(igwptrj) wrtag

         wrtag= '<gwptrjnp>'
         write(igwptrj) wrtag
         write(igwptrj) (gwptrjnp(nt),nt=1,ngwptrj)
         wrtag= '<gwptrjtdat>'
         write(igwptrj) wrtag
         write(igwptrj) ((gwptrjtdat(np,nt),&
                        np=1,ngwptrjpnts),nt=1,ngwptrj)
         wrtag= '<gwptrjxdat>'
         write(igwptrj) wrtag
         write(igwptrj) (((gwptrjxdat(np,nc,nt),&
                        np=1,ngwptrjpnts),nc=1,ngwptrjcoo),nt=1,ngwptrj)
         wrtag= '<gwptrjxspl>'
         write(igwptrj) wrtag
         write(igwptrj) (((gwptrjxspl(np,nc,nt),&
                        np=1,ngwptrjpnts),nc=1,ngwptrjcoo),nt=1,ngwptrj)
         wrtag= '<gwptrjpdat>'
         write(igwptrj) wrtag
         write(igwptrj) (((gwptrjpdat(np,nc,nt),&
                        np=1,ngwptrjpnts),nc=1,ngwptrjcoo),nt=1,ngwptrj)
         wrtag= '<gwptrjpspl>'
         write(igwptrj) wrtag
         write(igwptrj) (((gwptrjpspl(np,nc,nt),&
                        np=1,ngwptrjpnts),nc=1,ngwptrjcoo),nt=1,ngwptrj)
         wrtag= '<gwptrjedat>'
         write(igwptrj) wrtag
         write(igwptrj) ((gwptrjedat(np,nt),&
                        np=1,ngwptrjpnts),nt=1,ngwptrj)
         wrtag= '<gwptrjespl>'
         write(igwptrj) wrtag
         write(igwptrj) ((gwptrjespl(np,nt),&
                        np=1,ngwptrjpnts),nt=1,ngwptrj)
         wrtag= '<gwptrjcddat>'
         write(igwptrj) wrtag
         write(igwptrj) (((gwptrjcddat(np,ns,nt),&
                        np=1,ngwptrjpnts),ns=1,ngwptrjst),nt=1,ngwptrj)
         wrtag= '<gwptrjcdspl>'
         write(igwptrj) wrtag
         write(igwptrj) (((gwptrjcdspl(np,ns,nt),&
                        np=1,ngwptrjpnts),ns=1,ngwptrjst),nt=1,ngwptrj)
         wrtag= '<gwptrjczdat>'
         write(igwptrj) wrtag
         write(igwptrj) (((gwptrjczdat(np,ns,nt),&
                        np=1,ngwptrjpnts),ns=1,ngwptrjst),nt=1,ngwptrj)
         wrtag= '<gwptrjczspl>'
         write(igwptrj) wrtag
         write(igwptrj) (((gwptrjczspl(np,ns,nt),&
                        np=1,ngwptrjpnts),ns=1,ngwptrjst),nt=1,ngwptrj)

         wrtag= '<end gwptrj data>'
         write(igwptrj) wrtag

         close(igwptrj)

      endif

      return
  900 continue
      routine='Gwptrjfit'
      message='Error opening file: '//filename(1:ilbl)
      call errormsg
      stop
  910 continue
      routine='Gwptrjfit'
      message='Error reading file: '//filename(1:ilbl)
      call errormsg
      stop
  999 continue
      routine='Gwptrjfit'
      message='Error Reading gwptrj file: '//rdtag(ilbl:jlbl)
      call errormsg
      stop

      end subroutine gwptrjfit

!#######################################################################
!
! get GWP parameters from the trajectory at time 
!
!#######################################################################

      subroutine gwptrj_param(m,time,psi,agmat,gfgwp,aigwp,zgwp,qcenter)

      implicit none

      real(dop), intent(in)     :: time
      real(dop), dimension(qcentdim), intent(out)    :: qcenter
      complex(dop), dimension(agmatdim), intent(out) :: agmat
      complex(dop), dimension(dgldim), intent(out)   :: psi
      complex(dop), dimension(gfgwpdim), intent(out) :: gfgwp
      complex(dop), dimension(aigwpdim), intent(out) :: aigwp
      complex(dop), dimension(zgwpdim), intent(out)  :: zgwp
      integer(long)             :: si,sf,m

!-----------------------------------------------------------------------
! get point on each trajectory to be centre of GWPs
!-----------------------------------------------------------------------
      call gwptrj_pq(time,agmat(zagmat(1,1)),agmat(zxivec(1,1)), &
           agmat(zphase(1,1)),&
           psi(zxi(1,1)),psi(zphi(1,1)),qcenter(zcent(1,1)),nspfdof(1),&
           gwpdim(1,1))

!-----------------------------------------------------------------------
! get electronic state overlaps
!-----------------------------------------------------------------------
      call trjeovl(time,gwpdim(1,1))

!+-----------------------------------------------------------------------+
!         determine & store moments:
!+-----------------------------------------------------------------------+
      si=1
      sf=1
      call diagonal_moments(m,si,sf,nspfdof(1),gwpdim(1,sf),gwpdim(1,si), &
              agmat(zagmat(1,sf)),agmat(zagmat(1,si)), &
              agmat(zxivec(1,sf)),agmat(zxivec(1,si)), &
              agmat(zphase(1,sf)),agmat(zphase(1,si)), &
              gfgwp(zgfgwp(1,sf,si)), &
              aigwp(zaigwp(1,sf,si)), &
              zgwp(zzgwp(1,sf,si)),0,1)

      return
      end subroutine gwptrj_param

!#######################################################################

      subroutine gwptrj_pq(time,agmat,xivec,phase,xi,phi,qcenter,gdof,&
                 gdim)

      implicit none

      integer(long), intent(in) :: gdof,gdim
      real(dop), intent(in)     :: time
      real(dop), intent(out)    :: qcenter(gdof,gdim)
      complex(dop), intent(out) :: xivec(gdof,gdim)
      complex(dop), intent(out) :: xi(gdof,gdim)
      complex(dop), intent(in)  :: agmat(gdof,gdof,gdim)
      integer(long)             :: g,n,e,f
      real(dop),dimension(ndofddpes) :: q,p
      real(dop),dimension(ndofdd)    :: q1,p1
      real(dop)                 :: tmp,tmp1,a
      complex(dop), dimension(gdim), intent(inout)  :: phi
      complex(dop), dimension(gdim), intent(inout)  :: phase

      pi=4.0_dop*atan(1.0_dop)

      do e=1,gdim
         do f=1,ndofddpes
             call splint(gwptrjtdat(1,e),gwptrjxdat(1,f,e), &
                         gwptrjxspl(1,f,e),gwptrjnp(e),time,q(f))
             call splint(gwptrjtdat(1,e),gwptrjpdat(1,f,e), &
                         gwptrjpspl(1,f,e),gwptrjnp(e),time,p(f))
         enddo

         if (lddtrans) then
            call ddx2q(q,q1)
            call ddv2p(p,p1)
            do n=1,nspfdof(1)
               f=spfdof(n,1)
               qcenter(n,e)=q1(f)
               xi(n,e) = dcmplx(q1(f),p1(f))

               tmp = -2.0_dop*dble(agmat(n,n,e))*q1(f)
               xivec(n,e) = dcmplx(tmp,p1(f))
            enddo
         else
            do n=1,nspfdof(1)
               f=spfdof(n,1)
               qcenter(n,e)=q(f)
               xi(n,e) = dcmplx(q(f),p(f))

               tmp = -2.0_dop*dble(agmat(n,n,e))*q(f)
               xivec(n,e) = dcmplx(tmp,p(f))
            enddo
         endif

!-----------------------------------------------------------------------
! renormalise GWP using real part of scalar parameter
!-----------------------------------------------------------------------
         tmp = 0.0_dop
         do g=1,gdof
            f=spfdof(g,1)
            a=-dble(agmat(g,g,e))
            if (lddtrans) then
               tmp = tmp - a*q1(f)**2 &
                 - 0.5_dop*dlog(dsqrt(pi/(2.0_dop*dble(a))))
            else
               tmp = tmp - a*q(f)**2 &
                 - 0.5_dop*dlog(dsqrt(pi/(2.0_dop*dble(a))))
             endif
         enddo
         tmp1=dimag(phi(e))
         if (gwpphase .ge. 1)phi(e) = dcmplx(tmp,tmp1)
         phase(e) = dcmplx(tmp,tmp1)
      enddo

      return
      end subroutine gwptrj_pq
      
!#######################################################################

      subroutine trjeovl(time,gdim)

      implicit none

      integer(long), intent(in) :: gdim
      real(dop), intent(in)     :: time
      integer(long)             :: e,e1,s
      real(dop)                 :: dci,zci,norm,ddt,err
      complex(dop), dimension(ngwptrjst) :: cider
      real(dop), external       :: numderr
      logical :: initial = .true.

      do e=1,gdim
         norm = 0.0_dop
         curr_trj = e
         do s=1,ngwptrjst
             curr_st = s
             dci = dcitrj(time)
             zci = zcitrj(time)
             norm = norm + dci*dci + zci*zci
             gwptrjci(s,e) = dcmplx(dci,zci)
          enddo
! ensure normalisation
          norm = dsqrt(norm)
          do s=1,ngwptrjst
             gwptrjci(s,e) = gwptrjci(s,e) / norm
          enddo
      enddo

      gwptrjeovl = 0.0_dop
      do e=1,gdim
         do e1=e,gdim
            do s=1,ngwptrjst
               gwptrjeovl(e1,e) = gwptrjeovl(e1,e) + &
                    dconjg(gwptrjci(s,e1))*gwptrjci(s,e)
            enddo
            if (e .ne. e1) gwptrjeovl(e,e1) = dconjg(gwptrjeovl(e1,e))
         enddo
      enddo
!print*,'TRJEOVL',gdim
!print*,gwptrjeovl

!-----------------------------------------------------------------------
! Calculate i etau = i <psi|psidot>
!-----------------------------------------------------------------------
      gwptrjetau = 0.0_dop
      ddt = 0.00001
      do e=1,gdim
         curr_trj = e
         do s=1,ngwptrjst
            curr_st = s
            dci = numderr(dcitrj,time,ddt,err)
            zci = numderr(zcitrj,time,ddt,err)
            cider(s) = dcmplx(dci,zci)
         enddo
         do e1=1,gdim
            do s=1,ngwptrjst
               gwptrjetau(e1,e) = gwptrjetau(e1,e) + &
                    dconjg(gwptrjci(s,e1))*(0,1)*cider(s)
            enddo
         enddo
         dci=dble(gwptrjetau(e,e))
         if (dci .gt. 0.01 .and. initial) then
            routine = 'Trjeovl'
            write(message(1:80),'(a)') &
                         'Non negligible real part of <psi|psidot> '
            write(message(81:160),'(a)') &
                         'Set up trajectory data on a finer grid? '
            call warnmsg1
            initial = .false.
         endif

         gwptrjetau(e,e) = dble(gwptrjetau(e,e))
      enddo

      return
      end subroutine trjeovl

!#######################################################################

      real(dop) function dcitrj(time)

      implicit none

      integer(long)         :: s,e
      real(dop), intent(in) :: time
      real(dop)             :: dci

      s = curr_st
      e = curr_trj
      call splint(gwptrjtdat(1,e),gwptrjcddat(1,s,e), &
                         gwptrjcdspl(1,s,e),gwptrjnp(e),time,dci)
      dcitrj = dci

      end function dcitrj

!#######################################################################

      real(dop) function zcitrj(time)

      implicit none

      integer(long)         :: s,e
      real(dop), intent(in) :: time
      real(dop)             :: zci

      s = curr_st
      e = curr_trj
      call splint(gwptrjtdat(1,e),gwptrjczdat(1,s,e), &
                         gwptrjczspl(1,s,e),gwptrjnp(e),time,zci)
      zcitrj = zci

      end function zcitrj

!#######################################################################

      end module gwptrjmod

