!#######################################################################
!
! contains: wrdvrdef,rddvrdef,fcoin,chkgdim,dvrdat
!#######################################################################

      module iodvrdef

use decimal, only: long, dop
use op1lib,  only: zeromxi, zeromxd, zerovxi
use versions, only: progver,progname

use global
use maxv, only: maxdim
use dvrdatmod
use griddatmod
use operdef
use channels

use logdat

      implicit none
      private
      public :: wrdvrdef,rddvrdef,dvrdat,rddvrdef11,rddvrdef_pre11
      contains

!#######################################################################
! WRDVRDEF
!
! writes primitive definition of a system (ndof etc) to a file
!
! Any changes to this header must also be made in potfit/wrvpheader
!#######################################################################

      subroutine wrdvrdef(unit)

      implicit none

      integer(long)             :: f,i
      integer(long), external   :: log2int
      integer(long), intent(in) :: unit
      character(len=c2)         :: wrtag

      wrtag= '<start dvr definition>'
      write(unit) wrtag

      wrtag= '<ndof>'
      write(unit) wrtag
      write(unit) ndof
      wrtag= '<modelabel>'
      write(unit) wrtag
      write(unit) (modelabel(f),f=1,ndof)
      wrtag= '<gdim>'
      write(unit) wrtag
      write(unit) (gdim(f),f=1,ndof)

      wrtag= '<mbaspar>'
      write(unit) wrtag
      write(unit) mbaspar
      wrtag= '<basis>'
      write(unit) wrtag
      write(unit) (basis(f),f=1,ndof)
      wrtag= '<ldvr>'
      write(unit) wrtag
      write(unit) (log2int(ldvr(f)),f=1,ndof)
      wrtag= '<ipbaspar>'
      write(unit) wrtag
      write(unit) ((ipbaspar(i,f),i=1,mbaspar),f=1,ndof)
      wrtag= '<rpbaspar>'
      write(unit) wrtag
      write(unit) ((rpbaspar(i,f),i=1,mbaspar),f=1,ndof)
      wrtag= '<xend>'
      write(unit) wrtag
      write(unit) ((xend(i,f),i=1,2),f=1,ndof)

      wrtag= '<feb>'
      write(unit) wrtag
      write(unit) feb
      wrtag= '<febcont>'
      write(unit) wrtag
      write(unit) febcont
      wrtag= '<lebcont>'
      write(unit) wrtag
      write(unit) log2int(lebcont)
      wrtag= '<eltot>'
      write(unit) wrtag
      write(unit) eltot
      wrtag= '<elbnd>'
      write(unit) wrtag
      write(unit) elbnd

      wrtag= '<end dvr definition>'
      write(unit) wrtag

      return
      end subroutine wrdvrdef

!#######################################################################

      subroutine rddvrdef(unit,chkdvr,ndof1,fdvr)

      integer(long), intent(in)                     :: unit,chkdvr
      integer(long), intent(out)                    :: ndof1
      integer(long), dimension(maxdim), intent(out) :: fdvr

      if (filever(unit) .lt. 11.00_dop) then
         call rddvrdef_pre11(unit,chkdvr,ndof1,fdvr)
      else
         call rddvrdef11(unit,chkdvr,ndof1,fdvr)
      endif

      return
      end subroutine rddvrdef

!#######################################################################
! RDDVRDEF
!
! reads primitive definition of a system (ndof etc) from a file, and
! checks that it matches calculation.
!
! check=-1 : dvrinfo read, no action taken
! check=0  : dvrinfo read, no action taken except setting fdvr
! check=1  : dvrinfo read and stored in system data
! check=2  : dvrinfo checked against system data
! check=3  : dvrinfo checked as subset of system data
! check=4  : dvrinfo checked as subset of system data, but fdvr is
!            used and not set (i.e. modelabels are ignored).
! check=5  : dvrinfo checked against system data (as check=2),
!            but sin,FFT, and exp DVRs are unified by fcoin. 
!            and kleg/r and leg/r checked on initial not restricted grids
! check=6  : As check=5, but differences in the numbers of electronic
!            states are ignored. 
! check=7  : As check=2, but a point basis set is allowed in place
!            of a grid
!
! check=20,30,40,50,60: These were removed in 11.0 as spdof is not yet set.
!           They are equivalent to 5-6 followed by chkgrd.
!#######################################################################

      subroutine rddvrdef11(unit,check,ndof1,fdvr)
 

      implicit none
 
      integer(long), intent(in)                :: unit,check
      integer(long), intent(out)               :: ndof1

      integer(long)                            :: ilbl,jlbl
      integer(long)                            :: f,mbaspar1,i,f1,icoin,f2
      integer(long), dimension(maxdim), intent(out) :: fdvr
      integer(long), dimension(maxdim)         :: basis1,gdim1
      integer(long), dimension(mbaspar,maxdim) :: ipbaspar1

      integer(long)                            :: blz1,kmin2,ibleg1, &
                                                  iblegk,eltot1,elbnd1, &
                                                  feb1,mmmin2,febcont1
      integer(long)                            :: ilebcont1
      integer(long), dimension(maxdim)         :: ildvr1
      real(dop), dimension(mbaspar,maxdim)     :: rpbaspar1,xend1
      character(len=c1), dimension(maxdim)     :: modelabel1
      character(len=c5)                        :: filename
      logical(kind=4)                          :: lcoin,lprint,lebcont1
      logical(kind=4)                          :: lerr,lwarn
      logical(kind=4), external                :: int2log
      logical(kind=4), dimension(maxdim)       :: ldvr1
      integer(long), external                  :: slen
      character(len=c2)                        :: rdtag

      if (check .eq. -1) then
         if (ndof .eq. 0) then
            routine='RDDVRDEF11 (rddvrmod.F90)'
            message = 'Chkdvr = -1 but ndof = 0'
            call errormsg
         endif
         ndof1=ndof
         return  ! do not read
      endif

      lprint=.false. 
      routine='rddvrdef'
 
      ipbaspar1(:,:)=0
      rpbaspar1(:,:)=0.0_dop
      lebcont1 = .false.
      febcont1 = 0
      eltot1 = 0
      elbnd1 = 0
      chkerr(:)=0
 
!-----------------------------------------------------------------------
! READ INFO
!-----------------------------------------------------------------------
      do
   10    continue
         read(unit,err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1

         if (rdtag(ilbl:jlbl) .eq. 'ndof') then
            read(unit,iostat=chkerr(1)) ndof1
            if (ndof1 .gt. maxdim) then
              routine = 'rddvrdef'
              write(message,'(a,i4,a,i4)') 'No. of DOFs, ndof1 =', &
                     ndof1,' is .gt. maxdim =',maxdim
              call errormsg
            endif
         else if (rdtag(ilbl:jlbl) .eq. 'modelabel') then
            read(unit,iostat=chkerr(1)) (modelabel1(f),f=1,ndof1)
         else if (rdtag(ilbl:jlbl) .eq. 'gdim') then
            read(unit,iostat=chkerr(1)) (gdim1(f),f=1,ndof1)
         else if (rdtag(ilbl:jlbl) .eq. 'mbaspar') then
            read(unit,iostat=chkerr(1)) mbaspar1
         else if (rdtag(ilbl:jlbl) .eq. 'basis') then
            read(unit,iostat=chkerr(1)) (basis1(f),f=1,ndof1)
         else if (rdtag(ilbl:jlbl) .eq. 'ldvr') then
            read(unit,iostat=chkerr(1)) (ildvr1(f),f=1,ndof1)
            do f=1,ndof1
               ldvr1(f) = int2log(ildvr1(f))
            enddo
         else if (rdtag(ilbl:jlbl) .eq. 'ipbaspar') then
            read(unit,iostat=chkerr(1)) ((ipbaspar1(i,f),i=1,mbaspar1),f=1,ndof1)
         else if (rdtag(ilbl:jlbl) .eq. 'rpbaspar') then
            read(unit,iostat=chkerr(1)) ((rpbaspar1(i,f),i=1,mbaspar1),f=1,ndof1)
         else if (rdtag(ilbl:jlbl) .eq. 'xend') then
            read(unit,iostat=chkerr(1)) ((xend1(i,f),i=1,2),f=1,ndof1)
         else if (rdtag(ilbl:jlbl) .eq. 'feb') then
            read(unit,iostat=chkerr(1)) feb1
         else if (rdtag(ilbl:jlbl) .eq. 'febcont') then
            read(unit,iostat=chkerr(1)) febcont1
         else if (rdtag(ilbl:jlbl) .eq. 'lebcont') then
            read(unit,iostat=chkerr(1)) ilebcont1
            lebcont1=int2log(ilebcont1) 
         else if (rdtag(ilbl:jlbl) .eq. 'eltot') then
            read(unit,iostat=chkerr(1)) eltot1
         else if (rdtag(ilbl:jlbl) .eq. 'elbnd') then
            read(unit,iostat=chkerr(1)) elbnd1

         else if (rdtag(ilbl:jlbl) .eq. 'end dvr definition') then
            exit
         endif

      enddo

  100 continue

!-----------------------------------------------------------------------
! carry out requested action on dvr definition
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! set fdvr - map of DOFs between files
!-----------------------------------------------------------------------
 110  if (check .eq. 0) then     
         i=0                     
         fdvr(:)=0
! f1 are DOFs in file, f those in system
         do f1=1,ndof1
            do f=1,ndof
               if (modelabel(f) .eq. modelabel1(f1)) fdvr(f1)=f
            enddo
            if(fdvr(f1).eq.0) then
               i=i+1
               write(6   ,'(2a)') 'modelabel unknown to system: ', &
                                   modelabel1(f1)
               write(ilog,'(2a)') 'modelabel unknown to system: ', &
                                   modelabel1(f1)
            endif 
         enddo
         if((ndof1-i).ne.ndof) then
            routine='rddvrdef (mctdhlib/iodvrdef.f90)'
            message='Not all of System modes found in DVR file'
            call errormsg
         endif

!-----------------------------------------------------------------------
! read and store system data
!-----------------------------------------------------------------------
      else if (check .eq. 1) then  
         ndof=ndof1
         do f=1,ndof
            modelabel(f)=modelabel1(f)
            gdim(f)=gdim1(f)
            basis(f)=basis1(f)
            ldvr(f)=ldvr1(f)
            do i=1,mbaspar1
               ipbaspar(i,f)=ipbaspar1(i,f)
               rpbaspar(i,f)=rpbaspar1(i,f)
            enddo
            do i=1,2
               xend(i,f)=xend1(i,f)
            enddo
            fdvr(f)=f
         enddo
         feb=feb1
!defn of continuum basis changed in V1.2 from logical to integer
         if (filever(unit) .ge. 12.02d0) then
            febcont=febcont1
         endif
         lebcont=lebcont1
         eltot=eltot1
         elbnd=elbnd1
 
         call dvrdat

!-----------------------------------------------------------------------
!  checked against system data. 2: normal, 5: sin,fft,exp are unified. 
!-----------------------------------------------------------------------
      else if (check .eq. 2 .or. (check .ge. 5 .and. check .le. 7)) then
! continuum basis before 1.2 not specified as a DOF. Adjust Ndof and 
! do not check electronic basis
         if (lebcont .and. (filever(unit) .lt. 12.02d0)) then
            routine = 'Rddvrdef11'
            message =  &
  'Electronic continuum calculations not compatible with version pre 1.2'
            call errormsg
         endif

         if (ndof .gt. ndof1)then
            if(fpb.eq.0 .or. (ndof-ndof1).gt.1) &
                 chkerr(2) = 1
         endif

         lwarn = .false.
         do f=1,ndof
            if (modelabel1(f) .ne. modelabel(f)) lwarn = .true.
         enddo
         if (lwarn) then
            if (check .eq. 2) then
               routine = 'Rddvrdef11'
               message = 'Systems are different: Wrong modelables.'
               call errormsg
            else 
               routine = 'Rddvrdef11'
               message = 'Systems have different modelables.'
               call warnmsg1
            endif
         endif

         if (check .eq. 7) then
            do f=1,ndof
               fdvr(f)=f
               if (basis(f).ne.19 .and. basis1(f).ne.19) then
                  call chkgdim(f,gdim1(f),ipbaspar1(1,f))
               endif
            enddo
            if (feb1 .ne. feb) chkerr(2)=1
            if (febcont1 .ne. febcont) chkerr(2)=1
            if (lebcont1 .neqv. lebcont) chkerr(2)=1
            if (eltot1 .ne. eltot) chkerr(2)=1
            if (elbnd1 .ne. elbnd) chkerr(2)=1

         else if (check .eq. 6) then
            lebcont1=.false.
            eltot1=0
            elbnd1=0
            do f=1,ndof
               fdvr(f)=f
               if (basis(f).ne.0) then
                  call chkgdim(f,gdim1(f),ipbaspar1(1,f))
               endif
            enddo

         else
            do f=1,ndof
               if(f.eq.fpb)cycle
               fdvr(f)=f
               call chkgdim(f,gdim1(f),ipbaspar1(1,f))
            enddo
            if (feb1 .ne. feb) chkerr(2)=1
            if (febcont1 .ne. febcont) chkerr(2)=1
            if (lebcont1 .neqv. lebcont) chkerr(2)=1
            if (eltot1 .ne. eltot) chkerr(2)=1
            if (elbnd1 .ne. elbnd) chkerr(2)=1
         endif
! check that bases are the same
         if(lprint)  write(ilog,'(/a,/a)') &
          " --------- Check primitive basis: ------------ (rddvrdef)", &
          "modelabel  f, gdim, basis, ldvr"
outer1:  do f=1,ndof
            if(check.eq.6 .and. basis(f).eq.0 .or. f.eq.fpb) cycle outer1
            if(lprint) then
               if (basis(f) .eq. 0) then
                  write(ilog,'(a8,i4,i6,i5,i5,l5,2i5)') &
                    modelabel1(f),f,gdim1(f),basis1(f),feb1,febcont1,&
                    eltot1,elbnd1
                  write(ilog,'(a8,i4,i6,i5,i5,l5,2i5)') &
                    modelabel(f), f,gdim(f), basis(f), feb,febcont,&
                    eltot,elbnd
               else 
                  write(ilog,'(a8,i4,i6,i5,l5)')  &
                    modelabel1(f),f,gdim1(f),basis1(f),ldvr1(f)
                  write(ilog,'(a8,i4,i6,i5,l5)')  &
                    modelabel(f), f,gdim(f), basis(f), ldvr(f)
               endif
            endif 
            if(check.eq.7 .and. (basis(f).eq.19 .or. &
               basis1(f).eq. 19)) cycle outer1
            if ((basis1(f).ne.basis(f)) .or.&
                (ldvr1(f).neqv.ldvr(f)) )  then
               if(check .eq. 2) then
                  chkerr(3)=1
               else 
                  call fcoin(basis1(f),basis(f),lcoin,icoin)
                  if(.not.lcoin .or. (icoin.ne.2 .and. icoin.ne.3)) &
                      chkerr(3)=1
               endif
            endif
         enddo outer1
         if(lprint) write(ilog,'(a,2i4)') &
              '   ndof        : ndof , ndof1',ndof,ndof1
         if(lprint) write(ilog,'(a,i3,a,i3)') &
            '   chkerr(2) =',chkerr(2),',    chkerr(3) =',chkerr(3)
outer2:  do f=1,ndof
            if(check.eq.6 .and. basis(f).eq.0 ) cycle outer2
            if(check.eq.7 .and. (basis(f).eq.19 .or. &
               basis1(f).eq. 19)) cycle outer2
            if(check.eq.7 .and. (basis(f).eq.12 .and. &
              basis1(f).eq. 21)) cycle outer2
            if(check.eq.7 .and. (basis(f).eq.5 .and. &
              basis1(f).eq.5)) cycle outer2
            if (basis1(f) .eq. 8) then
               ipbaspar1(5,f) = max(1,ipbaspar1(5,f))
               ipbaspar (5,f) = max(1,ipbaspar (5,f))
            endif
            if(f.eq.fpb)cycle outer2
            if(lprint) then
               write(ilog,'(/a,i3,a)') ' f =', f,&
                   '  (ipbaspar1/ipbaspar/rpbaspar1/rpbaspar)'
               write(ilog,'(9i4)') (ipbaspar1(i,f),i=1,mbaspar1)
               write(ilog,'(9i4)') (ipbaspar (i,f),i=1,mbaspar1)
               write(ilog,'(9f13.6)') (rpbaspar1(i,f),i=1,mbaspar1)
               write(ilog,'(9f13.6)') (rpbaspar (i,f),i=1,mbaspar1)
            endif
            do i=1,mbaspar1
! GWPCL and GWP basis types equivalent
               if (basis(f) .eq. 25 .and. i .eq. 1) then
                  continue
               else
                  if ((ipbaspar1(i,f) .ne. ipbaspar(i,f))) then
                      if (basis(f).ne.15 .and. &
                         (basis(f).ne.1 .and. ipbaspar(1,f).ne.1))&
                          chkerr(3)=1
                  endif
               endif

               if ((abs(rpbaspar1(i,f)-rpbaspar(i,f))/ &
                (1.0_dop + abs(rpbaspar1(i,f))+abs(rpbaspar(i,f))) &
                         .gt. 1.0e-7_dop) .and. &
                (basis(f).ne.1 .and. ipbaspar(1,f).ne.1)) then
                  chkerr(3)=1
                  if(lprint) write(ilog,'(2a,/2i3,2f20.10,e16.3)') &
                  '  i, f,     rpbaspar1(i,f),      rpbaspar(i,f),', &
                  '       diff',i,f,rpbaspar1(i,f),rpbaspar(i,f), &
                  rpbaspar1(i,f)-rpbaspar(i,f)
               endif
            enddo
            lerr=check_rpbaspar(f,f,basis(f),&
               rpbaspar1(:,f),rpbaspar(:,f),&
               ipbaspar1(:,f),ipbaspar(:,f),&
               mbaspar,mbaspar1,lprint,ilog) 
            if (lerr) chkerr(3)=1
         enddo outer2
         if(lprint) write(ilog,'(a,i3)') ' chkerr(3) =',chkerr(3)
 
!-----------------------------------------------------------------------
! checked as subset of system data. 
! 3: use modelabels, 
! 4: use fdvr
!-----------------------------------------------------------------------
      else if (check .eq. 3 .or. check .eq. 4) then
 
         if (check.eq.3) then
            do f1=1,ndof1
               fdvr(f1)=0
               do f=1,ndof
                  if (modelabel(f) .eq. modelabel1(f1)) fdvr(f1)=f
               enddo

               if (fdvr(f1) .eq. 0) then
                  chkerr(7)=1
                  write(ilog,'(2a)') 'modelabel not found: ', &
                                      modelabel(f1)
               endif
            enddo
         endif
         if(lprint) write(ilog,'(a,i3)') ' chkerr(7) =',chkerr(7)
 
! check that bases are the same
         if(lprint) write(ilog,'(/a,/4x,a)')'  Check primitive basis:', &
                              'f        gdim    basis    ldvr    coin'
         do f1=1,ndof1
            f=fdvr(f1)
            call fcoin(basis1(f1),basis(f),lcoin,icoin)
            if(lprint) write(ilog,'(2i3,2x,2i5,2x,2i3,2x,2l3,l5,i3)') &
               f1,f,gdim1(f1), gdim(f), basis1(f1), basis(f), &
               ldvr1(f1),ldvr(f),lcoin,icoin 
            if (basis1(f1) .ne. 19 .and. basis(f) .ne. 19) then
               if (gdim1(f1) .ne. gdim(f)) chkerr(3)=1
               if (basis1(f1).ne.basis(f) .or. &
                  ldvr1(f1).neqv.ldvr(f)) then
                  if (.not.lcoin) chkerr(3)=1
               endif
            endif
         enddo

         if(lprint) write(ilog,'(a,i3)') ' chkerr(3) =',chkerr(3)
 
outer3:  do f1=1,ndof1

            f=fdvr(f1)
            if (basis1(f1) .eq. 19 .or. basis(f) .eq. 19) cycle outer3
            call fcoin(basis1(f1),basis(f),lcoin,icoin)
            if (lcoin .and. icoin.eq.1) then ! KLeg and Leg
!.......... check if  blz=kmin2     and   ibleg=ibleg1
               blz1 = ipbaspar1(1,f1)
               kmin2 = ipbaspar(3,f+1)
               if (blz1.ne.kmin2) chkerr(3)=1
               ibleg1 = ipbaspar1(2,f1)
               iblegk = ipbaspar(1,f)
               if (iblegk.gt.0) iblegk=2
               if (ibleg1.ne.iblegk) chkerr(3)=1
               if(lprint) then
                  if (blz1.ne.kmin2) write(ilog,'(a,2i4)') &
                     'blz1.ne.kmin2 ', blz1, kmin2
                  if (ibleg1.ne.iblegk) write(ilog,'(a,2i4)') &
                     'ibleg1.ne.iblegk ', ibleg1,iblegk   
               endif
            elseif (lcoin .and. icoin.eq.3) then ! PLeg and Leg
               blz1 = ipbaspar1(1,f1)
               kmin2 = 0
               if (blz1.ne.kmin2) chkerr(3)=1
               ibleg1 = ipbaspar1(2,f1)
               iblegk = ipbaspar(1,f)
               if (iblegk.gt.0) iblegk=2
               if (ibleg1.ne.iblegk) chkerr(3)=1
               if(lprint) then
                  if (blz1.ne.kmin2) write(ilog,'(a,2i4)')&
                     'blz1.ne.kmin2 ', blz1, kmin2
                  if (ibleg1.ne.iblegk) write(ilog,'(a,2i4)')&
                     'ibleg1.ne.iblegk ', ibleg1,iblegk 
               endif

            elseif (lcoin .and. icoin.eq.5) then  ! Wigner and Leg
               blz1 = ipbaspar1(1,f1)
               kmin2 = ipbaspar(5,f)
               mmmin2 = ipbaspar(9,f)
               if (.not. ((blz1.eq.kmin2 .and. mmmin2.eq.0) &
                   .or. (blz1.eq.mmmin2 .and. kmin2.eq.0))) chkerr(3)=1
               ibleg1 = ipbaspar1(2,f1)
               iblegk = ipbaspar(1,f)
               if (iblegk.gt.0) iblegk=2
               if (ibleg1.ne.iblegk) chkerr(3)=1
               if (lprint) then
                  if (.not. ((blz1.eq.kmin2 .and. mmmin2.eq.0) &
                      .or. (blz1.eq.mmmin2 .and. kmin2.eq.0))) &
                      write(ilog,'(a,3i4)') 'blz1 inconsistent with'// &
                      ' kmin2 and mmmin2', blz1, kmin2, mmmin2
                  if (ibleg1.ne.iblegk) write(ilog,'(a,2i4)') &
                     'ibleg1.ne.iblegk ', ibleg1,iblegk
               endif

            else
               if(lprint) then
                  write(ilog,'(/a,i3,a,i3,a)') ' f =', f,', f1 =',f1, &
                     '  (ipbaspar1/ipbaspar/rpbaspar1/rpbaspar)'
                  write(ilog,'(9i4)') (ipbaspar1(i,f1),i=1,mbaspar1)
                  write(ilog,'(9i4)') (ipbaspar (i,f),i=1,mbaspar1)
                  write(ilog,'(9f13.6)') (rpbaspar1(i,f1),i=1,mbaspar1)
                  write(ilog,'(9f13.6)') (rpbaspar (i,f),i=1,mbaspar1)
               endif
               do i=1,mbaspar1
! GWPCL and GWP basis types equivalent
                  if (basis(f) .eq. 25 .and. i .eq. 1) then
                     continue
                  else
                     if (ipbaspar1(i,f1) .ne. ipbaspar(i,f)) chkerr(3)=1
                  endif
                  if (abs(rpbaspar1(i,f1)-rpbaspar(i,f)) .gt. tiny) &
                     chkerr(3)=1
               enddo
               lerr = check_rpbaspar(f1,f,basis(f), &
                  rpbaspar1(:,f1),rpbaspar(:,f), &
                  ipbaspar1(:,f1),ipbaspar(:,f), &
                  mbaspar,mbaspar1,lprint,ilog) 
               if (lerr) chkerr(3)=1
            endif
         enddo outer3
         if(lprint) write(ilog,'(a,i3)') ' chkerr(3) =',chkerr(3)

!---  checked while ignoring electronic basis. For "read-inwf".
      else if (check .eq. 6) then
! check that degrees of freedom are the same
         f1=0
         f2=0
         do f=1,ndof
            if(basis(f) .eq.0) f2=f2+1
            if(basis1(f).eq.0) f1=f1+1
            if(basis1(f).ne.0 .and. basis(f).ne.0) then
               f1=f1+1
               f2=f2+1
               if (modelabel1(f1) .ne. modelabel(f2)) chkerr(2)=1
               if (gdim1(f1) .ne. gdim(f2)) chkerr(2)=1
            endif
         enddo
! check that bases are the same
         f1=0
         f2=0
         if(lprint)  write(ilog,'(/a,/a,i3,/a)') &
              " --------- Check primitive basis: ------------ (rddvrdef)", &
              ' chkdvr =',check, "modelabel  f, gdim, basis, ldvr"
         do f=1,ndof
            if(basis(f) .eq.0) f2=f2+1
            if(basis1(f).eq.0) f1=f1+1
            if(basis1(f).ne.0 .and. basis(f).ne.0) then
               f1=f1+1
               f2=f2+1
               if(lprint) then
                  write(ilog,'(a8,i4,i6,i5,l5,2x,a)') &
                       modelabel1(f1),f1,gdim1(f1),basis1(f1),ldvr1(f1),'read'
                  write(ilog,'(a8,i4,i6,i5,l5,2x,a,/)') &
                       modelabel(f2),f2,gdim(f2),basis(f2),ldvr(f2),'system'
               endif
               if ((basis1(f1).ne.basis(f2)) .or. &
                    (ldvr1(f1).neqv.ldvr(f2)) )  then
                  if(check .eq. 2) then
                     chkerr(3)=1
                  else
                     call fcoin(basis1(f),basis(f),lcoin,icoin)
                     if(.not.lcoin .or. icoin.ne.2) chkerr(3)=1
                  endif
               endif
            endif
         enddo
         if(lprint) write(ilog,'(a,2i4)') &
              '   ndof        : ndof , ndof1',ndof,ndof1
         if(lprint) write(ilog,'(a,i3,a,i3)') &
              '   chkerr(2) =',chkerr(2),',    chkerr(3) =',chkerr(3)

! check that base-parameters are the same
         f1=0
         f2=0
         do f=1,ndof
            if(basis(f) .eq.0) f2=f2+1
            if(basis1(f).eq.0) f1=f1+1
            if(basis1(f).ne.0 .and. basis(f).ne.0) then
               f1=f1+1
               f2=f2+1
               if (basis1(f1) .eq. 8) &
                    ipbaspar1(5,f1) = max(1,ipbaspar1(5,f1))
               if (basis(f2) .eq. 8) &
                    ipbaspar (5,f2) = max(1,ipbaspar (5,f2))
               if(lprint) then 
                  write(ilog,'(/a,3i4,a)') ' f,f1,f2 =',f,f1,f2, &
                       '  (ipbaspar1/ipbaspar/rpbaspar1/rpbaspar)'
                  write(ilog,'(9i4)') (ipbaspar1(i,f1),i=1,mbaspar1)
                  write(ilog,'(9i4)') (ipbaspar (i,f2),i=1,mbaspar1)
                  write(ilog,'(9f13.6)') (rpbaspar1(i,f1),i=1,mbaspar1)
                  write(ilog,'(9f13.6)') (rpbaspar (i,f2),i=1,mbaspar1)
               endif
               do i=1,mbaspar1
                  if ((ipbaspar1(i,f1) .ne. ipbaspar(i,f2))) then
                     if (basis(f2).ne.15 .and. &
                        (basis(f2).ne.1 .and. ipbaspar(1,f2).ne.1))&
                        chkerr(3)=1
                  endif
                  if ((abs(rpbaspar1(i,f1)-rpbaspar(i,f2))/ &
                       (1.d0 + abs(rpbaspar1(i,f1))+abs(rpbaspar(i,f2))) &
                       .gt. 1.d-7) .and. &
                       (basis(f2).ne.1 .and. ipbaspar(1,f2).ne.1)) then 
                     chkerr(3)=1
                     if(lprint) write(ilog,'(2a,/2i3,2f20.10,e16.3)') &
                          '  i, f,     rpbaspar1(i,f1),     rpbaspar(i,f2),', &
                          '       diff',i,f2,rpbaspar1(i,f1),rpbaspar(i,f2),&
                          rpbaspar1(i,f1)-rpbaspar(i,f2)
                  endif
               enddo
               lerr=check_rpbaspar(f1,f2,basis(f2), &
                  rpbaspar1(:,f1),rpbaspar(:,f2), &
                  ipbaspar1(:,f1),ipbaspar(:,f2), &
                  mbaspar,mbaspar1,lprint,ilog) 
               if (lerr) chkerr(3)=1
            endif
         enddo
         if(lprint) write(ilog,'(a,i3)') ' chkerr(3) =',chkerr(3)

      else
         write(message,'(a,i5)') ' Illegal value of chkdvr:',check
         routine='rddvrdef (mctdhlib/iodvrdef.f90)'
         call errormsg
      endif

! If error then redo check and print info
      if (lprint) then
         stop
      else if(.not.lprint .and. chkerr(2)+chkerr(3)+chkerr(7).gt.0) then
         lprint=.true.
         write(6,'(/a)') "####  Inconsistent DVR's. See log file! ####"
         chkerr(3)=0
         chkerr(7)=0 
         inquire(unit,name=filename)
         i=slen(filename)
         write(6,   '(2a)') 'File checked: ',filename(1:i)
         write(ilog,'(2a)') 'File checked: ',filename(1:i)
         goto 110
      endif

      return

      end subroutine rddvrdef11

!#######################################################################

      logical(kind=4) function check_rpbaspar(f1,f,b,rp1,rp,ip1,ip, &
                      mbaspar,mbaspar1,lprint,ilog) result(lerr)

      implicit none

      integer(long), intent(in)                      :: ilog
      integer(long), intent(in)                      :: mbaspar,mbaspar1
      integer(long), intent(in)                      :: f1,f,b
      integer(long), dimension(mbaspar), intent(in)  :: ip
      integer(long), dimension(mbaspar1), intent(in) :: ip1
      integer(long)                                  :: i,mbaspar0
      real(dop), dimension(mbaspar), intent(in)      :: rp
      real(dop), dimension(mbaspar1), intent(in)     :: rp1
      real(dop)                                      :: fm1,fm
      logical(kind=4), intent(in)                    :: lprint

      lerr = .false.
      ! HO in freq/mass form
      if ((b==1 .or. b==11) .and. (ip1(1)==0 .and. ip(1)==0)) then
         fm1 = rp1(2)*rp1(3)
         fm = rp(2)*rp(3)
         if (.not.deql(fm1,fm)) then
            lerr=.true.
            if (lprint) write(ilog,'(a,/,2i3,2f16.8,es12.3)') &
               ' f1, f,          hofm1,           hofm,   diff', &
               f1,f,fm1,fm,fm1-fm
         endif
         ! also check xeq
         mbaspar0 = 1
      else
         mbaspar0 = mbaspar1
      endif
      ! everything else
      do i=1,mbaspar0
         if (.not.deql(rp1(i),rp(i))) then
            lerr=.true.
            if (lprint) write(ilog,'(a,/,3i3,2f16.8,es12.3)') &
               '  i,f1, f, rpbaspar1(i,f),  rpbaspar(i,f),   diff', &
               i,f1,f,rp1(i),rp(i),rp1(i)-rp(i)
         endif
      enddo

      end function check_rpbaspar

!#######################################################################
! RDDVRDEF
!
! reads primitive definition of a system (ndof etc) from a file, and
! checks that it matches calculation.
!
! check=0  : dvrinfo read, fdvr set
! check=1  : dvrinfo read and stored in system data
! check=2  : dvrinfo checked against system data
! check=3  : dvrinfo checked as subset of system data
! check=4  : dvrinfo checked as subset of system data, but fdvr is
!            used and not set (i.e. modelabels are ignored).
! check=5  : dvrinfo checked against system data (as check=2),
!            but sin,FFT, and exp DVRs are unified by fcoin. 
!            and kleg/r and leg/r checked on initial not restricted grids
! check=6  : As check=5, but differences in the numbers of electronic
!            states are ignored. 
! check=7  : As check=2, but a point basis set is allowed in place
!            of a grid
!
! check=20,30,40,50,60: These were removed in 11.0 as spdof is not yet set.
!           They are equivalent to 5-6 followed by chkgrd.
!#######################################################################

      subroutine rddvrdef_pre11(unit,check,ndof1,fdvr)
 
      implicit none
 
      integer(long), intent(in)                :: unit,check
      integer(long), intent(out)               :: ndof1
      integer(long), dimension(maxdim), intent(out) :: fdvr

      integer(long)                            :: f,mbaspar1,i,f1,icoin,f2
      integer(long), dimension(maxdim)         :: basis1,gdim1
      integer(long), dimension(mbaspar,maxdim) :: ipbaspar1

      integer(long)                            :: blz1,kmin2,ibleg1, &
                                                  iblegk,eltot1,elbnd1, &
                                                  feb1,mmmin2
      integer(long), dimension(maxdim)         :: ildvr1
      real(dop), dimension(mbaspar,maxdim)     :: rpbaspar1,xend1
      character(len=c1), dimension(maxdim)     :: modelabel1
      character(len=c5)                        :: filename
      logical(kind=4)                          :: lcoin,lprint,&
                                                  lebcont1
      logical(kind=4)                          :: lerr
      logical(kind=4), external                :: int2log
      logical(kind=4), dimension(maxdim)       :: ldvr1
      integer(long), external :: slen

      lprint=.false. 
      routine='rddvrdef'
 
      ipbaspar1(:,:)=0
      rpbaspar1(:,:)=0.0_dop
 
      read(unit,iostat=chkerr(1)) ndof1
      if (ndof1 .gt. maxdim) then
        routine = 'rddvrdef'
        write(message,'(a,i4,a,i4)') 'No. of DOFs, ndof1 =', &
                     ndof1,' is .gt. maxdim =',maxdim
        call errormsg
      endif
      read(unit,iostat=chkerr(1)) (modelabel1(f),f=1,ndof1)
      read(unit,iostat=chkerr(1)) (gdim1(f),f=1,ndof1)
 
      read(unit,iostat=chkerr(1)) mbaspar1
      if (mbaspar .lt. mbaspar1) then
         write(message,'(a,33x,a,45x,a,i5)') &
              'This error is likely to be caused by a corrupted file.', &
              '(hence ignore the following request)', &
              'Increase MBASPAR in griddat.inc. Minimum: ',mbaspar1
         call errormsg
      endif
      read(unit,iostat=chkerr(1)) (basis1(f),f=1,ndof1)
      read(unit,iostat=chkerr(1)) (ildvr1(f),f=1,ndof1)
      do f1 = 1,ndof1
         ldvr1(f1) = int2log(ildvr1(f1))
      enddo
      read(unit,iostat=chkerr(1)) &
           ((ipbaspar1(i,f1),i=1,mbaspar1),f1=1,ndof1)
      read(unit,iostat=chkerr(1)) &
           ((rpbaspar1(i,f1),i=1,mbaspar1),f1=1,ndof1)
      read(unit,iostat=chkerr(1)) &
           ((xend1(i,f1),i=1,2),f1=1,ndof1)

! some versions had feb in dvr data. others had feb in grid data so must
! be added if already defined.
      feb1=0
      lebcont1=.false.
      eltot1=0
      elbnd1=0
      if (filever(unit) .ge. 9.0000_dop) then
         read(unit,iostat=chkerr(1)) feb1
         read(unit,iostat=chkerr(1)) lebcont1
         read(unit,iostat=chkerr(1)) eltot1,elbnd1
      else if ( (filever(unit) .ge. 8.3016_dop) .and. &
           (filever(unit) .lt. 8.40_dop) ) then
         read(unit,iostat=chkerr(1)) feb1
         read(unit,iostat=chkerr(1)) lebcont1
         read(unit,iostat=chkerr(1)) eltot1,elbnd1
      else if (feb .gt. 0) then
         if (feb .gt. ndof1) ndof1=ndof1+1
         feb1=ndof1
         ldvr1(feb1)=ldvr(feb)
         gdim1(feb1)=gdim(feb)
         basis1(feb1)=basis(feb)
         modelabel1(feb1)=modelabel(feb)
      endif

!-----------------------------------------------------------------------
! Make any version dependent changes
!-----------------------------------------------------------------------
! 1. shift Kleg parameters
!-----------------------------------------------------------------------
      if (filever(unit) .le. 9.0002) then
         do f=1,ndof1
            if (basis1(f) .eq. 9) then
               ipbaspar1(1,f+1)=ipbaspar1(2,f)
               ipbaspar1(2,f+1)=ipbaspar1(3,f)
               ipbaspar1(3,f+1)=ipbaspar1(4,f)

               ipbaspar1(2,f)=9999
               ipbaspar1(3,f)=0
               ipbaspar1(4,f)=0
            endif
         enddo
      endif

!-----------------------------------------------------------------------
! 2. add feb to system for checks in a Quantics calculation
!     (e.g. if reading a natpot from an old version).
!-----------------------------------------------------------------------
      if ((check .le. 2 .or. check .gt. 4) .and. &
                              (progname(1:8) .eq. 'quantics')) then
         if (filever(unit) .lt. 11.000_dop .and. feb1 .eq. 0) then
            ndof1=ndof1+1
            feb1=ndof1
            modelabel1(feb1)='el'
            basis1(feb1)=0
            gdim1(feb1)=1
            ldvr1(feb1)=.false.
         endif
      endif

!-----------------------------------------------------------------------
! carry out requested action on dvr definition
!-----------------------------------------------------------------------
      if (check .eq. -1) return  ! read, no action taken.
 100  if (check .eq. 0) then     ! read, no action taken except
         i=0                     ! for setting fdvr.
         fdvr(:)=0
         do f1=1,ndof1
            do f=1,ndof
               if (modelabel(f) .eq. modelabel1(f1)) fdvr(f1)=f
            enddo
            if(fdvr(f1).eq.0) then
               i=i+1
               write(6   ,'(2a)') 'modelabel unknown to system: ', &
                                   modelabel1(f1)
               write(ilog,'(2a)') 'modelabel unknown to system: ', &
                                   modelabel1(f1)
            endif 
         enddo
         if(i.gt.0) then
            routine='rddvrdef (mctdhlib/iodvrdef.f90)'
            message='Inconsistent modelabels'
            call errormsg
         endif

      else if (check .eq. 1) then  ! read and store system data
         ndof=ndof1
         do f=1,ndof
            modelabel(f)=modelabel1(f)
            gdim(f)=gdim1(f)
            basis(f)=basis1(f)
            ldvr(f)=ldvr1(f)
            do i=1,mbaspar1
               ipbaspar(i,f)=ipbaspar1(i,f)
               rpbaspar(i,f)=rpbaspar1(i,f)
               if (i.le.2) xend(i,f)=xend1(i,f)
            enddo
            fdvr(f)=f
         enddo
         feb=feb1
         lebcont=lebcont1
         eltot=eltot1
         elbnd=elbnd1
 
         call dvrdat

!  checked against system data. 2: normal, 5: sin,fft,exp are unified. 
      else if (check .eq. 2 .or.  &
                 (check .ge. 5 .and. check .le. 7)) then

         if (check .eq. 7) then
            do f1=1,ndof1
               fdvr(f1)=f1
               if (modelabel1(f1) .ne. modelabel(f1)) chkerr(2)=1
               if (basis(f1).ne.19 .and. basis1(f1).ne.19) then
                  call chkgdim(f1,gdim1(f1),ipbaspar1(1,f1))
               endif
            enddo
            if (feb1 .ne. feb) chkerr(2)=1
            if (lebcont1 .neqv. lebcont) chkerr(2)=1
            if (eltot1 .ne. eltot) chkerr(2)=1
            if (elbnd1 .ne. elbnd) chkerr(2)=1

         else if (check .eq. 6) then
            lebcont1=.false.
            eltot1=0
            elbnd1=0
            do f1=1,ndof1
               fdvr(f1)=f1
               if (basis(f1).ne.0) then
                  if (modelabel1(f1) .ne. modelabel(f1)) chkerr(2)=1
                  call chkgdim(f1,gdim1(f1),ipbaspar1(1,f1))
               endif
            enddo

         else
            do f1=1,ndof1
               fdvr(f1)=f1
! pre 11.0 there may be a packets DOF that must be ignored
               if (filever(unit) .lt. 11.000_dop .and.  &
                    f1 .eq. feb .and. npacket .gt. 0) cycle
               if (modelabel1(f1) .ne. modelabel(f1)) chkerr(2)=1
               call chkgdim(f1,gdim1(f1),ipbaspar1(1,f1))
            enddo
            if (feb1 .ne. feb) chkerr(2)=1
            if (lebcont1 .neqv. lebcont) chkerr(2)=1
            if (eltot1 .ne. eltot) chkerr(2)=1
            if (elbnd1 .ne. elbnd) chkerr(2)=1
         endif
! check that bases are the same
         if(lprint)  write(ilog,'(/a,/a)') &
          " --------- Check primitive basis: ------------ (rddvrdef)", &
          "modelabel  f, gdim, basis, ldvr"
outer1:  do f=1,ndof1
            if(check.eq.6 .and. basis(f).eq.0 ) cycle outer1
            if(lprint) then
               if (basis(f) .eq. 0) then
                  write(ilog,'(a8,i4,i6,i5,i5,l5,2i5)') &
                    modelabel1(f),f,gdim1(f),basis1(f),feb1,lebcont1,&
                    eltot1,elbnd1
                  write(ilog,'(a8,i4,i6,i5,i5,l5,2i5)') &
                    modelabel(f), f,gdim(f), basis(f), feb,lebcont,&
                    eltot,elbnd
               else 
                  write(ilog,'(a8,i4,i6,i5,l5)')  &
                    modelabel1(f),f,gdim1(f),basis1(f),ldvr1(f)
                  write(ilog,'(a8,i4,i6,i5,l5)')  &
                    modelabel(f), f,gdim(f), basis(f), ldvr(f)
               endif
            endif 
            if(check.eq.7 .and. (basis(f).eq.19 .or. &
               basis1(f).eq. 19)) cycle outer1
            if ((basis1(f).ne.basis(f)) .or.&
                (ldvr1(f).neqv.ldvr(f)) )  then
               if(check .eq. 2) then
                  chkerr(3)=1
               else 
                  call fcoin(basis1(f),basis(f),lcoin,icoin)
                  if(.not.lcoin .or. (icoin.ne.2 .and. icoin.ne.3)) &
                      chkerr(3)=1
               endif
            endif
         enddo outer1
         if(lprint) write(ilog,'(a,2i4)') &
              '   ndof        : ndof , ndof1',ndof,ndof1
         if(lprint) write(ilog,'(a,i3,a,i3)') &
            '   chkerr(2) =',chkerr(2),',    chkerr(3) =',chkerr(3)
outer2:  do f=1,ndof
            if(check.eq.6 .and. basis(f).eq.0 ) cycle outer2
            if(check.eq.7 .and. (basis(f).eq.19 .or. &
               basis1(f).eq. 19)) cycle outer2
            if(check.eq.7 .and. (basis(f).eq.12 .and. &
              basis1(f).eq. 21)) cycle outer2
            if(check.eq.7 .and. (basis(f).eq.5 .and. &
              basis1(f).eq.5)) cycle outer2
            if (basis1(f) .eq. 8) then
               ipbaspar1(5,f) = max(1,ipbaspar1(5,f))
               ipbaspar (5,f) = max(1,ipbaspar (5,f))
            endif
            if(lprint) then
               write(ilog,'(/a,i3,a)') ' f =', f,&
                   '  (ipbaspar1/ipbaspar/rpbaspar1/rpbaspar)'
               write(ilog,'(9i4)') (ipbaspar1(i,f),i=1,mbaspar1)
               write(ilog,'(9i4)') (ipbaspar (i,f),i=1,mbaspar1)
               write(ilog,'(9f13.6)') (rpbaspar1(i,f),i=1,mbaspar1)
               write(ilog,'(9f13.6)') (rpbaspar (i,f),i=1,mbaspar1)
            endif
! GWPCL and GWP basis types equivalent
            if (basis(f) .eq. 25) then
               continue
            else
               lerr=check_pbaspar(f,f,basis(f),basis(f),&
                  rpbaspar1(1,f),rpbaspar(1,f),&
                  ipbaspar1(1,f),ipbaspar(1,f))
               if (lerr) chkerr(3)=1
            endif
         enddo outer2
         if(lprint) write(ilog,'(a,i3)') ' chkerr(3) =',chkerr(3)
 
!---  checked as subset of system data. 3: use modelabels, 4: use fdvr
      else if (check .eq. 3 .or. check .eq. 4) then
! check that dvr is subset of system
         do f1=1,ndof1
            if (check.eq.3) then
               fdvr(f1)=0
               do f=1,ndof
                  if (modelabel(f) .eq. modelabel1(f1)) fdvr(f1)=f
               enddo

               if (fdvr(f1) .eq. 0) then
                  chkerr(7)=1
                  write(ilog,'(a,3i5)') 'f, f1, fdvr(f1)',f,f1,fdvr(f1)
                  write(ilog,'(2a)') 'modelabel not found: ', &
                                      modelabel1(f1)
                  go to 30
               endif
            endif
         enddo
         if(lprint) write(ilog,'(a,i3)') ' chkerr(7) =',chkerr(7)
 
! check that bases are the same
         if(lprint) write(ilog,'(/a,/4x,a)')'  Check primitive basis:', &
                              'f        gdim    basis    ldvr    coin'
         do f1=1,ndof1
            f=fdvr(f1)
            call fcoin(basis1(f1),basis(f),lcoin,icoin)
            if(lprint) write(ilog,'(2i3,2x,2i5,2x,2i3,2x,2l3,l5,i3)') &
               f1,f,gdim1(f1), gdim(f), basis1(f1), basis(f), &
               ldvr1(f1),ldvr(f),lcoin,icoin 
            if (basis1(f1) .ne. 19 .and. basis(f) .ne. 19) then
               if (gdim1(f1) .ne. gdim(f)) chkerr(3)=1
               if (basis1(f1).ne.basis(f) .or. &
                  ldvr1(f1).neqv.ldvr(f)) then
                  if (.not.lcoin) chkerr(3)=1
               endif
            endif
         enddo

         if(lprint) write(ilog,'(a,i3)') ' chkerr(3) =',chkerr(3)
 
outer3:  do f1=1,ndof1

            f=fdvr(f1)
            if (basis1(f1) .eq. 19 .or. basis(f) .eq. 19) cycle outer3
            call fcoin(basis1(f1),basis(f),lcoin,icoin)
            if (lcoin .and. icoin.eq.1) then ! KLeg and Leg
!.......... check if  blz=kmin2     and   ibleg=ibleg1
               blz1 = ipbaspar1(1,f1)
               kmin2 = ipbaspar(3,f+1)
               if (blz1.ne.kmin2) chkerr(3)=1
               ibleg1 = ipbaspar1(2,f1)
               iblegk = ipbaspar(1,f)
               if (iblegk.gt.0) iblegk=2
               if (ibleg1.ne.iblegk) chkerr(3)=1
               if(lprint) then
                  if (blz1.ne.kmin2) write(ilog,'(a,2i4)') &
                     'blz1.ne.kmin2 ', blz1, kmin2
                  if (ibleg1.ne.iblegk) write(ilog,'(a,2i4)') &
                     'ibleg1.ne.iblegk ', ibleg1,iblegk   
               endif
            elseif (lcoin .and. icoin.eq.3) then ! PLeg and Leg
               blz1 = ipbaspar1(1,f1)
               kmin2 = 0
               if (blz1.ne.kmin2) chkerr(3)=1
               ibleg1 = ipbaspar1(2,f1)
               iblegk = ipbaspar(1,f)
               if (iblegk.gt.0) iblegk=2
               if (ibleg1.ne.iblegk) chkerr(3)=1
               if(lprint) then
                  if (blz1.ne.kmin2) write(ilog,'(a,2i4)')&
                     'blz1.ne.kmin2 ', blz1, kmin2
                  if (ibleg1.ne.iblegk) write(ilog,'(a,2i4)')&
                     'ibleg1.ne.iblegk ', ibleg1,iblegk 
               endif

            elseif (lcoin .and. icoin.eq.5) then  ! Wigner and Leg
               blz1 = ipbaspar1(1,f1)
               kmin2 = ipbaspar(5,f)
               mmmin2 = ipbaspar(9,f)
               if (.not. ((blz1.eq.kmin2 .and. mmmin2.eq.0) &
                   .or. (blz1.eq.mmmin2 .and. kmin2.eq.0))) chkerr(3)=1
               ibleg1 = ipbaspar1(2,f1)
               iblegk = ipbaspar(1,f)
               if (iblegk.gt.0) iblegk=2
               if (ibleg1.ne.iblegk) chkerr(3)=1
               if (lprint) then
                  if (.not. ((blz1.eq.kmin2 .and. mmmin2.eq.0) &
                      .or. (blz1.eq.mmmin2 .and. kmin2.eq.0))) &
                      write(ilog,'(a,3i4)') 'blz1 inconsistent with'// &
                      ' kmin2 and mmmin2', blz1, kmin2, mmmin2
                  if (ibleg1.ne.iblegk) write(ilog,'(a,2i4)') &
                     'ibleg1.ne.iblegk ', ibleg1,iblegk
               endif

            else
               if(lprint) then
                  write(ilog,'(/a,i3,a,i3,a)') ' f =', f,', f1 =',f1, &
                     '  (ipbaspar1/ipbaspar/rpbaspar1/rpbaspar)'
                  write(ilog,'(9i4)') (ipbaspar1(i,f1),i=1,mbaspar1)
                  write(ilog,'(9i4)') (ipbaspar (i,f),i=1,mbaspar1)
                  write(ilog,'(9f13.6)') (rpbaspar1(i,f1),i=1,mbaspar1)
                  write(ilog,'(9f13.6)') (rpbaspar (i,f),i=1,mbaspar1)
               endif
! GWPCL and GWP basis types equivalent
               if (basis(f) .eq. 25) then
                  continue
               else
                  lerr = check_pbaspar(f1,f,basis1(f1),basis(f), &
                     rpbaspar1(1,f1),rpbaspar(1,f), &
                     ipbaspar1(1,f1),ipbaspar(1,f))
                  if (lerr) chkerr(3)=1
               endif
            endif
         enddo outer3
         if(lprint) write(ilog,'(a,i3)') ' chkerr(3) =',chkerr(3)

!---  checked while ignoring electronic basis. For "read-inwf".
      else if (check .eq. 6) then
! check that degrees of freedom are the same
         f1=0
         f2=0
         do f=1,ndof
            if(basis(f) .eq.0) f2=f2+1
            if(basis1(f).eq.0) f1=f1+1
            if(basis1(f).ne.0 .and. basis(f).ne.0) then
               f1=f1+1
               f2=f2+1
               if (modelabel1(f1) .ne. modelabel(f2)) chkerr(2)=1
               if (gdim1(f1) .ne. gdim(f2)) chkerr(2)=1
            endif
         enddo
! check that bases are the same
         f1=0
         f2=0
         if(lprint)  write(ilog,'(/a,/a,i3,/a)') &
              " --------- Check primitive basis: ------------ (rddvrdef)", &
              ' chkdvr =',check, "modelabel  f, gdim, basis, ldvr"
         do f=1,ndof1
            if(basis(f) .eq.0) f2=f2+1
            if(basis1(f).eq.0) f1=f1+1
            if(basis1(f).ne.0 .and. basis(f).ne.0) then
               f1=f1+1
               f2=f2+1
               if(lprint) then
                  write(ilog,'(a8,i4,i6,i5,l5,2x,a)') &
                       modelabel1(f1),f1,gdim1(f1),basis1(f1),ldvr1(f1),'read'
                  write(ilog,'(a8,i4,i6,i5,l5,2x,a,/)') &
                       modelabel(f2),f2,gdim(f2),basis(f2),ldvr(f2),'system'
               endif
               if ((basis1(f1).ne.basis(f2)) .or. &
                    (ldvr1(f1).neqv.ldvr(f2)) )  then
                  if(check .eq. 2) then
                     chkerr(3)=1
                  else
                     call fcoin(basis1(f),basis(f),lcoin,icoin)
                     if(.not.lcoin .or. icoin.ne.2) chkerr(3)=1
                  endif
               endif
            endif
         enddo
         if(lprint) write(ilog,'(a,2i4)') &
              '   ndof        : ndof , ndof1',ndof,ndof1
         if(lprint) write(ilog,'(a,i3,a,i3)') &
              '   chkerr(2) =',chkerr(2),',    chkerr(3) =',chkerr(3)

! check that base-parameters are the same
         f1=0
         f2=0
         do f=1,ndof
            if(basis(f) .eq.0) f2=f2+1
            if(basis1(f).eq.0) f1=f1+1
            if(basis1(f).ne.0 .and. basis(f).ne.0) then
               f1=f1+1
               f2=f2+1
               if (basis1(f1) .eq. 8) &
                    ipbaspar1(5,f1) = max(1,ipbaspar1(5,f1))
               if (basis(f2) .eq. 8) &
                    ipbaspar (5,f2) = max(1,ipbaspar (5,f2))
               if(lprint) then 
                  write(ilog,'(/a,3i4,a)') ' f,f1,f2 =',f,f1,f2, &
                       '  (ipbaspar1/ipbaspar/rpbaspar1/rpbaspar)'
                  write(ilog,'(9i4)') (ipbaspar1(i,f1),i=1,mbaspar1)
                  write(ilog,'(9i4)') (ipbaspar (i,f2),i=1,mbaspar1)
                  write(ilog,'(9f13.6)') (rpbaspar1(i,f1),i=1,mbaspar1)
                  write(ilog,'(9f13.6)') (rpbaspar (i,f2),i=1,mbaspar1)
               endif
               lerr=check_pbaspar(f1,f2,basis(f1),basis(f2), &
                  rpbaspar1(1,f1),rpbaspar(1,f2), &
                  ipbaspar1(1,f1),ipbaspar(1,f2))
               if (lerr) chkerr(3)=1
            endif
         enddo
         if(lprint) write(ilog,'(a,i3)') ' chkerr(3) =',chkerr(3)

      else
         write(message,'(a,i5)') ' Illegal value of chkdvr:',check
         routine='rddvrdef (mctdhlib/iodvrdef.f90)'
         call errormsg
      endif

   30 if(.not.lprint .and. chkerr(2)+chkerr(3)+chkerr(7).gt.0) then
         lprint=.true.
         write(6,'(/a)') "####  Inconsistent DVR's. See log file! ####"
         chkerr(3)=0
         chkerr(7)=0 
         inquire(unit,name=filename)
         i=slen(filename)
         write(6,   '(2a)') 'File checked: ',filename(1:i)
         write(ilog,'(2a)') 'File checked: ',filename(1:i)
         goto 100
      endif

!-----------------------------------------------------------------------
! remove unwanted feb before file is read
! change gdim(feb) to be real no. of states if old version
!-----------------------------------------------------------------------
      if (filever(unit) .lt. 11.000_dop .and. feb1 .gt. 0 &
                      .and. (progname(1:8) .eq. 'quantics')) then
         if (npacket .gt. 0) gdim(feb1)=gdim(feb1)/npacket
         if (gdim(feb1) .eq. 1) then
            ndof1=ndof1-1
            feb1=0
         endif
      endif

      return

    contains
!-----------------------------------------------------------------------

      function check_pbaspar(f1,f,b1,b,rp1,rp,ip1,ip) result(lerr)
         implicit none
         integer f1,f,b1,b,ip1(mbaspar1),ip(mbaspar),i,mbaspar0
         real(dop) rp1(mbaspar1),rp(mbaspar),fm1,fm
         logical lerr
         lerr = .false.
         ! Different formats for HO/rHO/Lagu?

         if ( (b==1 .or. b==11 .or. (b.ge.15 .and. b.le.18) ) .and. &
              (ip1(1) /= ip(1)) ) then
            routine = "check_pbaspar"
            message = "skipped checking primitive basis parameters for " &
               //trim(modelabel(f))
            call warnmsg1
            return
         endif
         ! Compare basis type
         if (b1 /= b) lerr=.true.
         ! Compare integer parameters.
         do i=1,mbaspar1
            if (ip1(i) /= ip(i)) lerr=.true.
         enddo
         ! HO in freq/mass form
         if ((b==1 .or. b==11) .and. (ip1(1)==0 .and. ip(1)==0)) then
            fm1 = rp1(2)*rp1(3)
            fm = rp(2)*rp(3)
            if (.not.deql(fm1,fm)) then
               lerr=.true.
               if (lprint) write(ilog,'(a,/,2i3,2f16.8,es12.3)') &
                  ' f1, f,          hofm1,           hofm,   diff', &
                  f1,f,fm1,fm,fm1-fm
            endif
            ! also check xeq
            mbaspar0 = 1
         else
            mbaspar0 = mbaspar1
         endif
         ! everything else
         do i=1,mbaspar0
            if (.not.deql(rp1(i),rp(i))) then
               lerr=.true.
               if (lprint) write(ilog,'(a,/,3i3,2f16.8,es12.3)') &
                  '  i,f1, f, rpbaspar1(i,f),  rpbaspar(i,f),   diff', &
                  i,f1,f,rp1(i),rp(i),rp1(i)-rp(i)
            endif
         enddo
      end function check_pbaspar

      end subroutine rddvrdef_pre11

!-----------------------------------------------------------------------

      logical(kind=4) function deql(x,y)

      implicit none
      real(dop), intent(in) :: x,y

      deql = ( abs(x-y)/(1.d0+abs(x)+abs(y)) .le. 1.d-7 )

      end function deql

!#######################################################################
 
      subroutine fcoin(basis1,basis2,lcoin,icoin)
 
      implicit none

      integer(long), intent(in)    :: basis1,basis2
      integer(long), intent(out)   :: icoin
      logical(kind=4), intent(out) :: lcoin
 
      icoin=-1
      lcoin=.false.
      if (basis1.eq.basis2) then
         lcoin=.true.        ! bases identical 
         icoin=0
      elseif ((basis1.eq.2 .and. basis2.eq.7) .or. &
             (basis1.eq.12 .and. basis2.eq.20)) then
          lcoin=.true.       ! Leg rather than KLeg
          icoin=1            ! Leg/R rather than KLeg/R
      elseif (((basis1.ge.3 .and. basis1.le.5) .or.  & 
               basis1.eq.31 .or. basis1.eq.32) .and. &
             ((basis2.ge.3 .and. basis2.le.5) .or.   &
              basis2.eq.31 .or. basis2.eq.32) ) then
           lcoin=.true.      ! sin, SInc, Wsinc, FFT or exp
           icoin=2
      elseif ((basis1.eq.2 .and. basis2.eq.9) .or.  &
             (basis1.eq.12 .and. basis2.eq.21) .or. &
             (basis1.eq.21 .and. basis2.eq.12)) then
          lcoin=.true.       ! Leg rather than PLeg or PLeg/R
          icoin=3            ! PLeg/R or PLeg rather than Leg/R or Leg
      elseif (basis1.eq.19 .or. basis2 .eq. 19) then
          lcoin=.true.       ! DVR replaces point
          icoin=4
      elseif (basis1.eq.2 .and. basis2.eq.33) then
          lcoin=.true.       ! Leg rather than Wigner
          icoin=5
      endif
 
      return
      end subroutine fcoin
 
!#######################################################################
!
! check that grid lengths are same in file and calculation.
! Must compare original grid length for restricted Legendre bases 
! rather than final
!
!#######################################################################
 
      subroutine chkgdim(f1,gdim1,ipbaspar1)
 
      implicit none

      integer(long), intent(in)                     :: f1,gdim1
      integer(long), dimension(mbaspar), intent(in) :: ipbaspar1
 
      if (basis(f1) .eq. 20 .or. basis(f1) .eq. 21 .or. &
                   basis(f1) .eq. 12) then
         if (ipbaspar1(3) .ne. ipbaspar(3,f1)) chkerr(2)=1
      else
         if (gdim1 .ne. gdim(f1)) chkerr(2)=1
      endif

      return
      end subroutine chkgdim
 
!#######################################################################
 
      subroutine dvrdat
 
      implicit none
 
      integer(long)       :: f,s
 
!-----------------------------------------------------------------------
! if feb and fpb not set see if electronic basis present
!-----------------------------------------------------------------------
      if (feb .eq. 0 .and. fpb .eq. 0) then
         do f=1,ndof
            if (basis(f) .eq. 0) feb=f
         enddo
      endif

!-----------------------------------------------------------------------
! check for continuum electronic basis sets and set sebcont
!-----------------------------------------------------------------------
      sebcont(:)=.false.
      if (lebcont) then
         do s=elbnd+1,eltot
            sebcont(s)=.true.
         enddo
      endif

      do f=1,ndof
         if (basis(f) .le. 0 .or. basis(f) .eq. 4 .or. &
             basis(f) .eq. 6 .or. basis(f) .eq. 25 .or.  &
             basis(f) .eq. 26 .or. basis(f) .eq. 27) then
            ldvr(f)=.false.
         else
            ldvr(f)=.true.
         endif
         if (basis(f) .eq. 25) lgwp=.true.
         if (basis(f) .eq. 19) lpntdvr=.true.
         if (basis(f) .eq. 22) lpntdvr=.true.
      enddo
 
      maxgdim=1
      do f=1,ndof
         maxgdim=max(maxgdim,gdim(f))
         if (basis(f) .eq. 12 .or. basis(f) .eq. 20 .or. &
             basis(f) .eq. 21) then
            maxgdim=max(maxgdim,ipbaspar(3,f))
         end if
      enddo
      maxgdim2=maxgdim**2
 
      return
      end subroutine dvrdat

!-----------------------------------------------------------------------

      end module iodvrdef 

