!#######################################################################
      module iofile
 
use decimal, only: long, dop
use op1lib,  only: zerovxi, zerovxl

use global
use maxv, only: maxdim, maxsta
use datenmod
use dvrdatmod
use psidef
use griddatmod
use channels

use logdat
use iopsidef, only:wrpsidef,rdpsidef
use iodvrdef, only:wrdvrdef, rddvrdef
use iogrddef, only:wrgrddef, rdgrddef

      implicit none
      private
      public :: rdpsiinfo,wrpsiinfo,wrchkinfo,wrrhoinfo,rdchkinfo,&
                rdrhoinfo,rdauteinfo,wrauteinfo
      contains

!#######################################################################
!   Contains the subroutines:
!
!   wrauteinfo, wrpsiinfo  wrchkinfo  wrrhoinfo 
!   rdauteinfo, rdpsiinfo  rdchkinfo  rdrhoinfo 
!
!#######################################################################


      subroutine wrauteinfo(unit)

      implicit none

      integer(long), intent(in) :: unit
      integer(long), external   :: log2int
      character(len=c2)         :: wrtag

      call wrdvrdef(unit)
      call wrgrddef(unit)
      call wrpsidef(unit)
      
      wrtag='<lautodoub>'
      write(unit) wrtag
      write(unit,err=999) log2int(lautodoub)
      wrtag='<tinit>'
      write(unit) wrtag
      write(unit,err=999) tinit
      wrtag='<out1>'
      write(unit) wrtag
      write(unit,err=999) out1

      wrtag='<end autoe info>'
      write(unit) wrtag
      
      return
  999 routine='Wrpsiinfo'  
      message='Error writing info to Autoe file: '//wrtag
      call errormsg
      end

!#######################################################################

      subroutine wrpsiinfo(unit)

      implicit none

      integer(long)             :: n
      integer(long), intent(in) :: unit
      integer(long), external   :: log2int
      character(len=c2)         :: wrtag

!-----------------------------------------------------------------------
! Parameters needed for array dimensioning
! Tag must be written from a buffer to ensure known read length
!-----------------------------------------------------------------------
      wrtag='<start array dimensions>'
      write(unit) wrtag

      wrtag='<maxdim>'
      write(unit) wrtag
      write(unit,err=999) maxdim
      wrtag='<maxsta>'
      write(unit) wrtag
      write(unit,err=999) maxsta
      wrtag='<maxgwp>'
      write(unit) wrtag
      write(unit,err=999) maxgwp
      wrtag='<maxtape>'
      write(unit) wrtag
      write(unit,err=999) maxtape
      wrtag='<lgwp>'
      write(unit) wrtag
      write(unit,err=999) log2int(lgwp)
      wrtag='<ltraj>'
      write(unit) wrtag
      write(unit,err=999) log2int(ltraj)
      wrtag='<ldynspf>'
      write(unit) wrtag
      write(unit,err=999) log2int(ldynspf)
      wrtag='<dentype>'
      write(unit) wrtag
      write(unit,err=999) dentype

      wrtag='<end array dimensions>'
      write(unit) wrtag

      call wrdvrdef(unit)
      call wrgrddef(unit)
      call wrpsidef(unit)

      wrtag='<start psi info>'
      write(unit) wrtag

      wrtag='<timeunit>'
      write(unit) wrtag
      write(unit,err=999) fs
      wrtag='<npsiopt>'
      write(unit) wrtag
      write(unit,err=999) npsiopt
      wrtag='<lpsiopt>'
      write(unit) wrtag
      write(unit,err=999) (log2int(lpsiopt(n)),n=1,npsiopt)
      wrtag='<tinit>'
      write(unit) wrtag
      write(unit,err=999) tinit
      wrtag='<out2>'
      write(unit) wrtag
      write(unit,err=999) out2
      wrtag='<ldd>'
      write(unit) wrtag
      write(unit,err=999) log2int(ldd)
      wrtag='<lddmctdh>'
      write(unit) wrtag
      write(unit,err=999) log2int(lddmctdh)

      wrtag='<end psi info>'
      write(unit) wrtag

      return
  999 routine='Wrpsiinfo'  
      message='Error writing info to Psi file: '//wrtag
      call errormsg
      end subroutine wrpsiinfo

!#######################################################################

      subroutine wrchkinfo(unit)

      implicit none

      integer(long), intent(in) :: unit
      integer(long), external   :: log2int
      character(len=c2)         :: wrtag

!-----------------------------------------------------------------------
! Parameters needed for array dimensioning
! Tag must be written from a buffer to ensure known read length
!-----------------------------------------------------------------------
      wrtag='<start array dimensions>'
      write(unit) wrtag

      wrtag='<maxdim>'
      write(unit) wrtag
      write(unit,err=999) maxdim
      wrtag='<maxsta>'
      write(unit) wrtag
      write(unit,err=999) maxsta
      wrtag='<maxgwp>'
      write(unit) wrtag
      write(unit,err=999) maxgwp
      wrtag='<maxtape>'
      write(unit) wrtag
      write(unit,err=999) maxtape
      wrtag='<lgwp>'
      write(unit) wrtag
      write(unit,err=999) log2int(lgwp)
      wrtag='<ltraj>'
      write(unit) wrtag
      write(unit,err=999) log2int(ltraj)
      wrtag='<ldynspf>'
      write(unit) wrtag
      write(unit,err=999) log2int(ldynspf)
      wrtag='<dentype>'
      write(unit) wrtag
      write(unit,err=999) dentype

      wrtag='<end array dimensions>'
      write(unit) wrtag

      call wrdvrdef(unit)
      call wrgrddef(unit)
      call wrpsidef(unit)

      wrtag='<timeunit>'
      write(unit) wrtag
      write(unit,err=999) fs
      wrtag='<tinit>'
      write(unit) wrtag
      write(unit,err=999) tinit
      wrtag='<out1>'
      write(unit) wrtag
      write(unit,err=999) out1
      wrtag='<tfinal>'
      write(unit) wrtag
      write(unit,err=999) tfinal
      wrtag='<lrlx>'
      write(unit) wrtag
      write(unit,err=999) log2int(lrlx)

      wrtag='<end check info>'
      write(unit) wrtag

      return
  999 routine='Wrchkinfo'  
      message='Error writing info to Check file: '//wrtag
      call errormsg
      end subroutine wrchkinfo

!#######################################################################

      subroutine wrrhoinfo(unit)

      implicit none

      integer(long), intent(in) :: unit
      integer(long)             :: f
      integer(long), external   :: log2int
      character(len=c2)         :: wrtag

!-----------------------------------------------------------------------
! Parameters needed for array dimensioning
! Tag must be written from a buffer to ensure known read length
!-----------------------------------------------------------------------
      wrtag='<start array dimensions>'
      write(unit) wrtag

      wrtag='<maxdim>'
      write(unit) wrtag
      write(unit,err=999) maxdim
      wrtag='<maxsta>'
      write(unit) wrtag
      write(unit,err=999) maxsta
      wrtag='<maxgwp>'
      write(unit) wrtag
      write(unit,err=999) maxgwp
      wrtag='<maxtape>'
      write(unit) wrtag
      write(unit,err=999) maxtape
      wrtag='<lgwp>'
      write(unit) wrtag
      write(unit,err=999) log2int(lgwp)
      wrtag='<ltraj>'
      write(unit) wrtag
      write(unit,err=999) log2int(ltraj)
      wrtag='<dentype>'
      write(unit) wrtag
      write(unit,err=999) dentype

      wrtag='<end array dimensions>'
      write(unit) wrtag

      call wrdvrdef(unit)
      call wrgrddef(unit)

      wrtag='<timeunit>'
      write(unit) wrtag
      write(unit,err=999) fs
      wrtag='<tinit>'
      write(unit) wrtag
      write(unit,err=999) tinit
      wrtag='<out1>'
      write(unit) wrtag
      write(unit,err=999) out1
      wrtag='<ldynspf>'
      write(unit) wrtag
      write(unit,err=999) log2int(ldynspf)
      wrtag='<ortdim>'
      write(unit) wrtag
      write(unit,err=999) ortdim
      wrtag='<zort>'
      write(unit) wrtag
      write(unit,err=999) (zort(f),f=1,ndof)

      wrtag='<end check info>'
      write(unit) wrtag

      return
  999 routine='Wrrhoinfo'
      message='Error writing info to Gridpop file: '//wrtag
      call errormsg
      end subroutine wrrhoinfo

!#######################################################################

      subroutine rdauteinfo(unit,chkdvr,chkgrd,chkpsi,chkdat)

      integer(long), intent(inout)                  :: chkgrd
      integer(long), intent(in)                     :: unit,chkdvr,chkpsi,chkdat

      if (filever(unit) .lt. 11.0_dop) then
         call rdauteinfo_pre11(unit,chkdvr,chkgrd,chkpsi,chkdat)
      else
         call rdauteinfo11(unit,chkdvr,chkgrd,chkpsi,chkdat)
      endif

      return
      end subroutine rdauteinfo

!#######################################################################

      subroutine rdauteinfo11(unit,chkdvr,chkgrd,chkpsi,chkdat)

      implicit none

      integer(long), intent(in) :: unit,chkdvr,chkgrd,chkdat,chkpsi
      integer(long) :: ndof1,fdvr(maxdim),ilautodoub1,mdvr(maxdim),&
                       mmode(maxdim),nmode1,nspfdof1(maxdim),&
                       spfdof1(maxdim,maxdim)
      real(dop)       :: tinit1,out11
      logical(kind=4) :: lautodoub1, int2log
      integer(long)      :: ilbl,jlbl
      character(len=c2)  :: rdtag
      
      call zerovxi(chkerr,nerr)
      call rddvrdef(unit,chkdvr,ndof1,fdvr)
      call rdgrddef(unit,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
      call rdpsidef(unit,chkpsi)
     
      rewind(unit)
      do
! read next record - error means it is not a string so ignore and go on to next
   10    continue
         read(unit,err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'lautodoub') then
            read(unit,err=999) ilautodoub1
            lautodoub1 = int2log(ilautodoub1)
         else if (rdtag(ilbl:jlbl) .eq. 'tinit') then
            read(unit,err=999) tinit1
         else if (rdtag(ilbl:jlbl) .eq. 'tout1') then
            read(unit,err=999) out11

         else if (rdtag(ilbl:jlbl) .eq. 'end autoe info') then
            exit
         endif
      enddo

  100 continue

      if (chkdat .eq. 0) then
         continue

      else if (chkdat .eq. 1) then
         lautodoub=lautodoub1
         tinit=tinit1
         out1=out11

      else if (chkdat .eq. 2) then
         if (tinit .ne. tinit1) chkerr(8)=1
         if (out1 .ne. out11) chkerr(8)=1
         if (lautodoub .neqv. lautodoub1) chkerr(8)=1

      endif

      return
  999 routine='Rdauteinfo11'
      message='Error reading info in Autoe file: '//rdtag
      call errormsg

      end subroutine rdauteinfo11

!#######################################################################

      subroutine rdauteinfo_pre11(unit,chkdvr,chkgrd,chkpsi,chkdat)

      implicit none

      integer(long), intent(in) :: unit,chkdvr,chkgrd,chkdat,chkpsi
      integer(long) :: ndof1,fdvr(maxdim),ilautodoub1,mdvr(maxdim),&
                       mmode(maxdim),nmode1,nspfdof1(maxdim),&
                       spfdof1(maxdim,maxdim)
      real(dop)       :: tinit1,out11
      logical(kind=4) :: lautodoub1, int2log
      
      call zerovxi(chkerr,nerr)
      call rddvrdef(unit,chkdvr,ndof1,fdvr)
      call rdgrddef(unit,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
      call rdpsidef(unit,chkpsi)
     
      read(unit,iostat=chkerr(1)) ilautodoub1
      lautodoub1 = int2log(ilautodoub1)
      read(unit,iostat=chkerr(1)) tinit1,out11
      
      if (chkdat .eq. 0) then
         continue

      else if (chkdat .eq. 1) then
         lautodoub=lautodoub1
         tinit=tinit1
         out1=out11

      else if (chkdat .eq. 2) then
         if (tinit .ne. tinit1) chkerr(8)=1
         if (out1 .ne. out11) chkerr(8)=1
         if (lautodoub .neqv. lautodoub1) chkerr(8)=1

      endif

      return
      end subroutine rdauteinfo_pre11

!#######################################################################

      subroutine rdpsiinfo(unit,chkdvr,chkgrd,chkpsi,chkdat)

      integer(long), intent(inout)                  :: chkgrd
      integer(long), intent(in)                     :: unit,chkdvr,chkpsi,chkdat

      if (filever(unit) .lt. 11.0_dop) then
         call rdpsiinfo_pre11(unit,chkdvr,chkgrd,chkpsi,chkdat)
      else
         call rdpsiinfo11(unit,chkdvr,chkgrd,chkpsi,chkdat)
      endif

      return
      end subroutine rdpsiinfo

!#######################################################################

      subroutine rdpsiinfo11(unit,chkdvr,chkgrd,chkpsi,chkdat)

      implicit none

      integer(long), intent(inout)                  :: chkgrd
      integer(long), intent(in)                     :: unit,chkdvr,chkpsi,chkdat
      integer(long)                                 :: ndof1,npsiopt1,n,nmode1,idd1,&
                                                       iddmctdh1
      integer(long), dimension(maxdim)              :: fdvr,mdvr,mmode,nspfdof1
      integer(long), dimension(:), allocatable      :: ilpsiopt1
      integer(long), dimension(maxdim,maxdim)       :: spfdof1
      real(dop)                                     :: tinit1,out21
      logical(kind=4)                               :: ldd1,lddmctdh1
      logical(kind=4), external                     :: int2log
      logical(kind=4), dimension(:), allocatable    :: lpsiopt1
      integer(long)                                 :: ilbl,jlbl
      character(len=c2)                             :: rdtag

      call rddvrdef(unit,chkdvr,ndof1,fdvr)
      if (chkgrd .eq. -2) return
      call rdgrddef(unit,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
      if (chkpsi .eq. -2) return
      call rdpsidef(unit,chkpsi)

      if (chkdat .eq. -2) return

      npsiopt1=0
      tinit1=-999.0_dop
      out21=-999.0_dop
      ldd1=.false.
      lddmctdh1=.false.

!
! get no. of options and allocate arrays
!
      rewind(unit)
      do
! read next record - error means it is not a string so ignore and go on to next
   10    continue
         read(unit,err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'npsiopt') then
            read(unit,err=999) npsiopt1
         else if (rdtag(ilbl:jlbl) .eq. 'end psi info') then
            exit
         endif
      enddo

  100 continue
      if (npsiopt1.lt.0) then
         routine='rdpsiinfo'
         message='ERROR reading psi file: Npsiopt < 0. '
         call errormsg
      endif
      allocate(ilpsiopt1(npsiopt1))
      allocate(lpsiopt1(npsiopt1))
      ilpsiopt1(:)=0
      lpsiopt1(:)=.false.

! Read data
      rewind(unit)
      do
! read next record - error means it is not a string so ignore and go on to next
   20    continue
         read(unit,err=20,end=200) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'lpsiopt') then
            read(unit,err=999) (ilpsiopt1(n),n=1,npsiopt1)
            do n = 1,npsiopt1
               lpsiopt1(n) = int2log(ilpsiopt1(n))
            enddo
         else if (rdtag(ilbl:jlbl) .eq. 'timeunit') then
            read(unit,err=999) fs
         else if (rdtag(ilbl:jlbl) .eq. 'tinit') then
            read(unit,err=999) tinit1
         else if (rdtag(ilbl:jlbl) .eq. 'out2') then
            read(unit,err=999) out21
         else if (rdtag(ilbl:jlbl) .eq. 'ldd') then
            read(unit,err=999) idd1
            ldd1 = int2log(idd1)
         else if (rdtag(ilbl:jlbl) .eq. 'lddmctdh') then
            read(unit,err=999) iddmctdh1
            lddmctdh1 = int2log(iddmctdh1)
         else if (rdtag(ilbl:jlbl) .eq. 'end psi info') then
            exit
         endif

      enddo

  200 continue

! End reading file

      if (chkdat .eq. 0) then
         continue

      else if (chkdat .eq. 1) then
         do n=1,npsiopt
            lpsiopt(n)=lpsiopt1(n)
         enddo  
         tinit=tinit1
         out2=out21
         ldd=ldd1
         lddmctdh=lddmctdh1

      else if (chkdat .eq. 2) then
         if (tinit .ne. tinit1) chkerr(8)=1
         if (out2 .ne. out21) chkerr(8)=1
         do n=1,npsiopt
            if (lpsiopt(n) .neqv. lpsiopt1(n)) chkerr(8)=1
         enddo  
         if (ldd .neqv. ldd1) chkerr(8)=1
         if (lddmctdh .neqv. lddmctdh1) chkerr(8)=1

      else if (chkdat .eq. 3) then
         tinit=tinit1
         if (out2 .ne. out21) chkerr(8)=1
         do n=1,npsiopt
            if (lpsiopt(n) .neqv. lpsiopt1(n)) chkerr(8)=1
         enddo

      endif

      if (allocated(ilpsiopt1)) deallocate(ilpsiopt1)
      if (allocated(lpsiopt1)) deallocate(lpsiopt1)

      return

  999 routine='Rdpsiinfo11'
      message='Error reading info in Psi file: '//rdtag
      call errormsg

      end subroutine rdpsiinfo11

!#######################################################################

      subroutine rdpsiinfo_pre11(unit,chkdvr,chkgrd,chkpsi,chkdat)

      implicit none

      integer(long), intent(inout)                  :: chkgrd
      integer(long), intent(in)                     :: unit,chkdvr,chkpsi,chkdat
      integer(long)                                 :: ndof1,nopt,n,nmode1,idd1
      integer(long), dimension(maxdim)              :: fdvr,mdvr,mmode,nspfdof1
      integer(long), dimension(1:npsiopt)           :: ilpsiopt1
      integer(long), dimension(maxdim,maxdim)       :: spfdof1
      real(dop)                                     :: tinit1,out21
      logical(kind=4)                               :: ldd1
      logical(kind=4), external                     :: int2log
      logical(kind=4), dimension(npsiopt)           :: lpsiopt1

      nopt=0
      call zerovxi(chkerr,nerr)
      call rddvrdef(unit,chkdvr,ndof1,fdvr)
      if (chkgrd .eq. -2) return
      call rdgrddef(unit,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
      if (chkpsi .eq. -2) return
      call rdpsidef(unit,chkpsi)

      if (chkdat .eq. -2) return
      call zerovxl(lpsiopt1,npsiopt)
      read(unit,iostat=chkerr(1),err=981) &
              nopt,(ilpsiopt1(n),n=1,min(nopt,npsiopt))
      if(nopt.lt.0 .or. nopt.gt.npsiopt) goto 981
      do n = 1,nopt
         lpsiopt1(n) = int2log(ilpsiopt1(n))
      enddo
      read(unit,iostat=chkerr(1),err=981) tinit1,out21
      if (filever(unit) .ge. 9.0000d0) then
         read(unit,iostat=chkerr(1)) idd1
         ldd1 = int2log(idd1)
      else
         ldd1 = .false.
      endif

      if (chkdat .eq. 0) then
         continue

      else if (chkdat .eq. 1) then
         do n=1,npsiopt
            lpsiopt(n)=lpsiopt1(n)
         enddo  
         tinit=tinit1
         out2=out21
         ldd=ldd1

      else if (chkdat .eq. 2) then
         if (tinit .ne. tinit1) chkerr(8)=1
         if (out2 .ne. out21) chkerr(8)=1
         do n=1,npsiopt
            if (lpsiopt(n) .neqv. lpsiopt1(n)) chkerr(8)=1
         enddo  
         if (ldd .neqv. ldd1) chkerr(8)=1

      else if (chkdat .eq. 3) then
         tinit=tinit1
         if (out2 .ne. out21) chkerr(8)=1
         do n=1,npsiopt
            if (lpsiopt(n) .neqv. lpsiopt1(n)) chkerr(8)=1
         enddo

      else if (chkdat .eq. 3) then
         tinit=tinit1
         if (out2 .ne. out21) chkerr(8)=1
         do n=1,npsiopt
            if (lpsiopt(n) .neqv. lpsiopt1(n)) chkerr(8)=1
         enddo

      endif

      return

 981  if(nopt.lt.0 .or. nopt.gt.npsiopt) then
         write(6,'(/a,i3,2x,a)') &
         'Channel:',unit,'Is this a psi-file? Could be a restart file.'
      endif
      routine='rdpsiinfo'
      message='ERROR reading psi file.'
      call errormsg

      end subroutine rdpsiinfo_pre11

!#######################################################################

      subroutine rdchkinfo(unit,chkdvr,chkgrd,chkpsi,chkdat)

      implicit none

      integer(long), intent(inout)                  :: chkgrd
      integer(long), intent(in)                     :: unit,chkdvr,chkpsi,chkdat

      if (filever(unit) .lt. 11.0_dop) then
         call rdchkinfo_pre11(unit,chkdvr,chkgrd,chkpsi,chkdat)
      else
         call rdchkinfo11(unit,chkdvr,chkgrd,chkpsi,chkdat)
      endif

      return
      end subroutine rdchkinfo

!#######################################################################

      subroutine rdchkinfo11(unit,chkdvr,chkgrd,chkpsi,chkdat)

      implicit none

      integer(long), intent(inout)                  :: chkgrd
      integer(long), intent(in)                     :: unit,chkdvr,chkpsi,chkdat
      integer(long)                                 :: ndof1,nmode1,ilrlx
      integer(long), dimension(maxdim)              :: fdvr,mdvr,mmode,nspfdof1
      integer(long), dimension(maxdim,maxdim)       :: spfdof1
      real(dop)                                     :: tinit1,out11,tfinal1
      logical(kind=4)                               :: ldynspf1,lrlx1
      logical(kind=4), external                     :: int2log
      integer(long)                                 :: ilbl,jlbl
      character(len=c2)                             :: rdtag

      call rddvrdef(unit,chkdvr,ndof1,fdvr)
      if (chkgrd .eq. -2) return
      call rdgrddef(unit,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
      if (chkpsi .eq. -2) return
      call rdpsidef(unit,chkpsi)
      if (chkdat .eq. -2) return

      tinit1=-999.0_dop
      out11=-999.0_dop
      ldynspf1=.false.
      lrlx1 = .false.

      rewind(unit)
      do
! read next record - error means it is not a string so ignore and go on to next
   10    continue
         read(unit,err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'timeunit') then
            read(unit,err=999) fs
         else if (rdtag(ilbl:jlbl) .eq. 'tinit') then
            read(unit,err=999) tinit1
         else if (rdtag(ilbl:jlbl) .eq. 'out1') then
            read(unit,err=999) out11
         else if (rdtag(ilbl:jlbl) .eq. 'tfinal') then
            read(unit,err=999) tfinal1
         else if (rdtag(ilbl:jlbl) .eq. 'lrlx') then
            read(unit,err=999) ilrlx
            lrlx1 = int2log(ilrlx)
         endif

      enddo

  100 continue

! End reading file

      if (chkdat .eq. 0) then
         continue

      else if (chkdat .eq. 1) then
         tinit=tinit1
         tfinal=tfinal1
         out1=out11
         ldynspf = ldynspf1
         tfinal = tfinal1
         lrlx = lrlx1

      else if (chkdat .eq. 2) then
         if (tinit .ne. tinit1) chkerr(8)=1
         if (tfinal .ne. tfinal1) chkerr(8)=1
         if (out1 .ne. out11) chkerr(8)=1
         if (ldynspf .neqv. ldynspf1) chkerr(8)=1
         if (lrlx .neqv. lrlx1) chkerr(8)=1
        
      endif

      return
  999 routine='Rdchkinfo11'
      message='Error reading info in Check file: '//rdtag
      call errormsg

      end subroutine rdchkinfo11

!#######################################################################

      subroutine rdchkinfo_pre11(unit,chkdvr,chkgrd,chkpsi,chkdat)

      implicit none

      integer(long), intent(inout)                  :: chkgrd
      integer(long), intent(in)                     :: unit,chkdvr,chkpsi,chkdat
      integer(long)                                 :: ndof1,nmode1,ildynspf
      integer(long), dimension(maxdim)              :: fdvr,mdvr,mmode,nspfdof1
      integer(long), dimension(maxdim,maxdim)       :: spfdof1
      real(dop)                                     :: tinit1,out11
      logical(kind=4)                               :: ldynspf1
      logical(kind=4), external                     :: int2log

      call zerovxi(chkerr,nerr)
      call rddvrdef(unit,chkdvr,ndof1,fdvr)
      if (chkgrd .eq. -2) return
      call rdgrddef(unit,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
      if (chkpsi .eq. -2) return
      call rdpsidef(unit,chkpsi)
      if (chkdat .eq. -2) return

      read(unit,iostat=chkerr(1),err=999) tinit1,out11
      if (filever(unit) .ge. 10.0000_dop) then
         read(unit,iostat=chkerr(1),err=999) ildynspf
         ldynspf1 = int2log(ildynspf)
      else
         ldynspf1 = .false.
      endif

      if (chkdat .eq. 0) then
         continue

      else if (chkdat .eq. 1) then
         tinit=tinit1
         out1=out11
         ldynspf = ldynspf1

      else if (chkdat .eq. 2) then
         if (tinit .ne. tinit1) chkerr(8)=1
         if (out1 .ne. out11) chkerr(8)=1
         if (ldynspf .neqv. ldynspf1) chkerr(8)=1

      endif

      return
  999 continue
      write(6,'(a,i5)') 'Error reading check file on unit :',unit
      end subroutine rdchkinfo_pre11

!#######################################################################

      subroutine rdrhoinfo(unit,chkdvr,chkgrd,chkdat)

      implicit none

      integer(long), intent(inout)                  :: chkgrd
      integer(long), intent(in)                     :: unit,chkdvr,chkdat

      if (filever(unit) .lt. 11.0_dop) then
         call rdrhoinfo_pre11(unit,chkdvr,chkgrd,chkdat)
      else
         call rdrhoinfo11(unit,chkdvr,chkgrd,chkdat)
      endif

      return
      end subroutine rdrhoinfo

!#######################################################################

      subroutine rdrhoinfo11(unit,chkdvr,chkgrd,chkdat)

      implicit none

      integer(long), intent(in)                     :: unit,chkdvr,chkgrd,chkdat
      integer(long)                                 :: ndof1,nmode1,f
      integer(long), dimension(maxdim)              :: fdvr,mdvr,mmode,nspfdof1
      integer(long), dimension(maxdim,maxdim)       :: spfdof1
      real(dop)                                     :: tinit1,out11
      integer(long)                                 :: ilbl,jlbl
      character(len=c2)                             :: rdtag

      call rddvrdef(unit,chkdvr,ndof1,fdvr)
      if (chkgrd .eq. -2) return
      call rdgrddef(unit,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
      if (chkdat .eq. -2) return

      tinit1=-999.0_dop
      out11=-999.0_dop

      rewind(unit)
      do
! read next record - error means it is not a string so ignore and go on to next
   10    continue
         read(unit,err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'timeunit') then
            read(unit,err=999) fs
         else if (rdtag(ilbl:jlbl) .eq. 'tinit') then
            read(unit,err=999) tinit1
         else if (rdtag(ilbl:jlbl) .eq. 'out1') then
            read(unit,err=999) out11
         else if (rdtag(ilbl:jlbl) .eq. 'ortdim') then
            read(unit,err=999) ortdim
         else if (rdtag(ilbl:jlbl) .eq. 'zort') then
            read(unit,err=999) (zort(f),f=1,ndof1)
         endif

      enddo

  100 continue

! End reading file

      if (chkdat .eq. 0) then
         continue

      else if (chkdat .eq. 1) then
         tinit=tinit1
         out1=out11

      else if (chkdat .eq. 2) then
         if (tinit .ne. tinit1) chkerr(8)=1
         if (out1 .ne. out11) chkerr(8)=1

      endif

      return
  999 continue
      routine='Rdrhoinfo11'
      if(unit.eq.irho) then
         write(message,'(a,i5)') 'Error reading gridpop file on unit :',unit
      else if(unit.eq.ipden) then
         write(message,'(a,i5)') 'Error reading pdensity file on unit :',unit
      else
         write(message,'(a,i5)') 'Error reading file on unit :',unit
      endif
      call errormsg

      end subroutine rdrhoinfo11

!#######################################################################

      subroutine rdrhoinfo_pre11(unit,chkdvr,chkgrd,chkdat)

      implicit none

      integer(long), intent(in)                     :: unit,chkdvr,chkgrd,chkdat
      integer(long)                                 :: ndof1,nmode1
      integer(long), dimension(maxdim)              :: fdvr,mdvr,mmode,nspfdof1
      integer(long), dimension(maxdim,maxdim)       :: spfdof1
      real(dop)                                     :: tinit1,out11

      call zerovxi(chkerr,nerr)
      call rddvrdef(unit,chkdvr,ndof1,fdvr)
      if (chkgrd .eq. -2) return
      call rdgrddef(unit,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
      if (chkdat .eq. -2) return
      read(unit,iostat=chkerr(1),err=999) tinit1,out11
      if (chkdat .eq. 0) then
         continue

      else if (chkdat .eq. 1) then
         tinit=tinit1
         out1=out11

      else if (chkdat .eq. 2) then
         if (tinit .ne. tinit1) chkerr(8)=1
         if (out1 .ne. out11) chkerr(8)=1

      endif

      return
  999 continue
      if(unit.eq.irho) then
         write(6,'(a,i5)') 'Error reading gridpop file on unit :',unit
      else if(unit.eq.ipden) then
         write(6,'(a,i5)') 'Error reading pdensity file on unit :',unit
      else
         write(6,'(a,i5)') 'Error reading file on unit :',unit
      endif
      end subroutine rdrhoinfo_pre11

!#######################################################################
      end module iofile

