!
! contains :: wrgrddef,rdgrddef,griddat
!
      module iogrddef

use decimal, only: long, dop
use global
use dvrdatmod
use griddatmod
use logdat
use op1lib,  only: zerovxi, zeromxi, zerovxl

      implicit none

      private
      public :: wrgrddef,rdgrddef,griddat
      contains

!#######################################################################
! WRGRDDEF
!
! writes modes, DOFs etc.  to a file
!
!#######################################################################

      subroutine wrgrddef(unit)

      implicit none

      integer(long)             :: m,n,f
      integer(long), external   :: log2int
      integer(long), intent(in) :: unit
      character(len=c2) :: wrtag

      wrtag= '<start grd definition>'
      write(unit) wrtag

      wrtag= '<dentype>'
      write(unit) wrtag
      write(unit) dentype
      wrtag= '<nmode>'
      write(unit) wrtag
      write(unit) nmode
      wrtag= '<nspfdof>'
      write(unit) wrtag
      write(unit) (nspfdof(m),m=1,nmode)
      wrtag= '<maxnspfdof>'
      write(unit) wrtag
      write(unit) maxnspfdof
      wrtag= '<spfdof>'
      write(unit) wrtag
      write(unit) ((spfdof(n,m),n=1,nspfdof(m)),m=1,nmode)
      wrtag= '<dofspf>'
      write(unit) wrtag
      write(unit) (dofspf(f),f=1,ndof)

      wrtag= '<meb>'
      write(unit) wrtag
      write(unit) meb
      wrtag= '<lmult>'
      write(unit) wrtag
      write(unit) log2int(lmult)
      wrtag= '<leb>'
      write(unit) wrtag
      write(unit) log2int(leb)
      wrtag= '<lmulpack>'
      write(unit) wrtag
      write(unit) log2int(lmulpack)
      wrtag= '<lebonly>'
      write(unit) wrtag
      write(unit) log2int(lebonly)
      wrtag= '<mpb>'
      write(unit) mpb
      wrtag= '<fpb>'
      write(unit) fpb
      wrtag= '<gwpm>'
      write(unit) wrtag
      write(unit) (log2int(gwpm(m)),m=1,nmode)
      wrtag= '<gwptype>'
      write(unit) wrtag
      write(unit) gwptype
      wrtag= '<gwpphase>'
      write(unit) wrtag
      write(unit) gwpphase
      wrtag= '<lsysshell>'
      write(unit) wrtag
      write(unit) log2int(lsysshell)
      wrtag= '<isysshell>'
      write(unit) wrtag
      write(unit) isysshell

      wrtag= '<end grd definition>'
      write(unit) wrtag

      return
      end subroutine wrgrddef

!#######################################################################

      subroutine rdgrddef(unit,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
 
use maxv, only: maxdim
use operdef

      implicit none
 
      integer(long), intent(in)    :: unit, chkgrd
      integer(long), intent(out)   :: nmode1
      integer(long), dimension(maxdim), intent(in)         :: fdvr
      integer(long), dimension(maxdim), intent(out)        :: nspfdof1,mdvr, &
                                                              mmode
      integer(long), dimension(maxdim,maxdim), intent(out) :: spfdof1

      if (filever(unit) .lt. 11.00_dop) then
         call rdgrddef_pre11(unit,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
      else
         call rdgrddef11(unit,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
      endif

      return
      end subroutine rdgrddef

!#######################################################################
! RDGRDDEF
!
! reads refined definition of a system (nmode etc) from a file, and
! checks that it matches calculation.
!
! check=0 : grdinfo read, no action taken
! check=1 : grdinfo read, and stored in system data
! check=2 : grdinfo checked against system data
! check=3 : grdinfo checked as subset of system data 
! check=4 : as check=2, but different electronic bases allowed
!                       and different GWP types allowed
! check=6 : grdinfo checked against system data while ignoring an
!           electronic DOF. This is used for the read-inwf block and
!           the block-spf keyword.
! check=20,40, ..: Similar to check=2,4 but a check of dummy DOFs is avoided
!           if those are at the end of the primitive basis list.
!#######################################################################

      subroutine rdgrddef11(unit,chk,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
 
use maxv, only: maxdim
use operdef

      implicit none
 
      integer(long), intent(in)    :: unit, chk
      integer(long), intent(out)   :: nmode1
      integer(long), dimension(maxdim), intent(in)         :: fdvr
      integer(long), dimension(maxdim), intent(out)        :: nspfdof1,mdvr, &
                                                              mmode
      integer(long), dimension(maxdim,maxdim), intent(out) :: spfdof1

      integer(long) :: check,ilbl,jlbl
      integer(long) :: m,n,f,m1,n1,f1,n0,ndof1,ndof2,ndof3,ilmult1,ileb1, &
                       ilmulpack1,dentype1,gwptype1,gwpphase1, &
                       fpb1,mpb1,maxnspfdof1,itmp,meb1

      integer(long), dimension(maxdim)     :: dofspf1,igwpm1,isysshell1
      integer(long), external              :: slen

      character(len=c1), dimension(maxdim) :: modelabel1
      logical(kind=4)                      :: lmult1,leb1,lmulpack1,&
                                              lred,lebonly1,lsysshell1
      logical(kind=4), external            :: int2log
      logical(kind=4), dimension(maxdim)   :: gwpm1
      character(len=c5)                    :: filename  
      character(len=c2)                    :: rdtag

      if (chk .eq. -1) return  ! read, no action taken.

! --- modify check if chk .ge. 20
      if(chk .lt. 20) then
         check = chk
         lred=.false.
      else
         check = chk/10
         lred=.true.
      endif

      mdvr(:)=0
      mmode(:)=0
      nspfdof1(:)=0
      dofspf1(:)=0
      spfdof1(:,:)=0
      lebonly1=.false.
      n0 = 0

!-----------------------------------------------------------------------
! READ INFO
!-----------------------------------------------------------------------
      rewind(unit)

      do
   10    continue
         read(unit,err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'dentype') then
            read(unit,iostat=chkerr(1)) dentype1
         else if (rdtag(ilbl:jlbl) .eq. 'ndof') then
            read(unit,iostat=chkerr(1)) ndof1
         else if (rdtag(ilbl:jlbl) .eq. 'nmode') then
            read(unit,iostat=chkerr(1)) nmode1
         else if (rdtag(ilbl:jlbl) .eq. 'nspfdof') then
            read(unit,iostat=chkerr(1)) (nspfdof1(m),m=1,nmode1)
         else if (rdtag(ilbl:jlbl) .eq. 'maxnspfdof') then
            read(unit,iostat=chkerr(1)) maxnspfdof1
         else if (rdtag(ilbl:jlbl) .eq. 'spfdof') then
            read(unit,iostat=chkerr(1)) &
                   ((spfdof1(n,m),n=1,nspfdof1(m)),m=1,nmode1)
         else if (rdtag(ilbl:jlbl) .eq. 'dofspf') then
            read(unit,iostat=chkerr(1)) (dofspf1(f),f=1,ndof)

         else if (rdtag(ilbl:jlbl) .eq. 'meb') then
            read(unit,iostat=chkerr(1)) meb1
         else if (rdtag(ilbl:jlbl) .eq. 'lmult') then
            read(unit,iostat=chkerr(1)) ilmult1
            lmult1 = int2log(ilmult1)
         else if (rdtag(ilbl:jlbl) .eq. 'leb') then
            read(unit,iostat=chkerr(1)) ileb1
            leb1 = int2log(ileb1)
         else if (rdtag(ilbl:jlbl) .eq. 'lmulpack') then
            read(unit,iostat=chkerr(1)) ilmulpack1
            lmulpack1 = int2log(ilmulpack1)
         else if (rdtag(ilbl:jlbl) .eq. 'lebonly') then
            read(unit,iostat=chkerr(1)) ileb1
            lebonly1 = int2log(ileb1)
         else if (rdtag(ilbl:jlbl) .eq. 'fpb') then
            read(unit,iostat=chkerr(1)) fpb1
         else if (rdtag(ilbl:jlbl) .eq. 'mpb') then
            read(unit,iostat=chkerr(1)) mpb1
         else if (rdtag(ilbl:jlbl) .eq. 'gwpm') then
            read(unit,iostat=chkerr(1)) (igwpm1(m),m=1,nmode1)
            do m=1,nmode1
               gwpm1(m) = int2log(igwpm1(m))
            enddo
         else if (rdtag(ilbl:jlbl) .eq. 'gwptype') then
            read(unit,iostat=chkerr(1)) gwptype1
         else if (rdtag(ilbl:jlbl) .eq. 'gwpphase') then
            read(unit,iostat=chkerr(1)) gwpphase1
         else if (rdtag(ilbl:jlbl) .eq. 'lsysshell') then
            read(unit,iostat=chkerr(1)) itmp
            lsysshell1 = int2log(itmp)
         else if (rdtag(ilbl:jlbl) .eq. 'isysshell') then
            read(unit,iostat=chkerr(1)) (isysshell1(m),m=1,nmode1)

         else if (rdtag(ilbl:jlbl) .eq. 'end grd definition') then
            exit

         endif

      enddo

  100 continue

!-----------------------------------------------------------------------
! Reduce ndof and ignore dummy DOFs
!-----------------------------------------------------------------------
      ndof2=ndof
      ndof3=ndof1
      if(lred) then
  40     if(dofspf(ndof2).gt.nmode .or. dofspf(ndof2).eq.0) then
            ndof2=ndof2-1
            if(ndof2.gt.0) goto 40
         endif
         if(ndof2.eq.0) then
            routine='rddvrdef (mctdh/iogrddef.f90)'
            message='dofspf seems to be unset'
            call errormsg
         endif
         ndof3=min(ndof2,ndof3) ! DANGER, it is expected that ndof is
      endif      ! checked in rdgrddef. ndof3 cannot be reduced here,
                 ! because dofspf1 not known here.

!-----------------------------------------------------------------------
! carry out requested action on dvr definition
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! calculate grid data
!-----------------------------------------------------------------------
      if (check .eq. 0) then
         do f=1,ndof1
            f1=fdvr(f)
            modelabel1(f)=modelabel(f1)
         enddo

         do m=1,nmode1
            do n=1,nspfdof1(m)
               f=spfdof1(n,m)
               do m1=1,nmode
                  do n1=1,nspfdof(m1)
                     f1=spfdof(n1,m1)
                     if (modelabel(f1) .eq. modelabel1(f)) then
                        mdvr(f)=f1
                        mmode(m)=m1
                     endif
                  enddo
               enddo
            enddo
         enddo

      else if (check .eq. 1) then
         dentype=dentype1
         nmode=nmode1
         do m=1,nmode
            nspfdof(m)=nspfdof1(m)
            do n=1,nspfdof(m)
               spfdof(n,m)=spfdof1(n,m)
            enddo
            gwpm(m)=gwpm1(m)

         enddo
         maxnspfdof=maxnspfdof1
         do f=1,ndof
            mdvr(f)=f
            dofspf(f)=dofspf1(f)
         enddo
         meb = meb1
         lmult = lmult1
         leb = leb1
         lmulpack = lmulpack1
         lebonly = lebonly1
         gwptype = gwptype1
         gwpphase = gwpphase1

         call griddat

      else if (check .eq. 2 .or. check .eq. 4 .or. check .eq. 20) then
! for multi-set calculations meb and dofspf may not be set. Set it now.
         if(lmult .and. feb.gt.0 ) then
            if(dofspf(feb).eq.0 .and. dofspf1(feb).gt.0) then
               dofspf(feb)=dofspf1(feb)
               meb=dofspf1(feb)
            elseif(dofspf1(feb).eq.0 .and. dofspf(feb).gt.0) then
               dofspf1(feb)=dofspf(feb)
            endif
         endif
         ndof2=ndof
         ndof3=ndof1
         if (ndof3 .ne. ndof2)then
            if(fpb.eq.0 .or. (ndof2-ndof3).gt.1) chkerr(2)=1
         endif
         if (nmode1 .ne. nmode) then
            if(fpb.eq.0 .or. (nmode-nmode1).gt.1) chkerr(4)=1
         endif

         if (dentype1 .ne. dentype) chkerr(10)=1

         do m=1,nmode
            if(fpb.ne.0 .and. m.eq.nmode) cycle
            if (nspfdof1(m) .ne. nspfdof(m)) chkerr(4)=1
            do n=1,nspfdof1(m)
               if (spfdof1(n,m) .ne. spfdof(n,m)) chkerr(4)=1
            enddo
         enddo

         do f=1,ndof2
            if(f.eq.fpb)cycle
! The following two if's are for avoiding problems with dummy DOFs.
            if (dofspf(f).eq.0 .and. dofspf1(f).gt.nmode) dofspf1(f)=0
            if (dofspf(f).gt.nmode .and. (dofspf1(f).gt.nmode .or. &
                 dofspf1(f).eq.0) ) dofspf1(f)=dofspf(f)
            if (dofspf1(f) .ne. dofspf(f)) chkerr(4)=1
         enddo

! check that basis types are the same. lebdiff is .true. if this is a
! restart calculation with different electronic basis from old.
         if (check .ne. 4) then
            if (lmult1 .neqv. lmult) chkerr(5)=1
            if (leb1 .neqv. leb) chkerr(5)=1
         endif
! check that basis types are the same. Ignore lmulpack flag after 11.0
!         if (lmulpack1 .neqv. lmulpack) chkerr(5)=1
         if (lgwp .and. check .eq. 2) then
            if (gwptype1 .ne. gwptype) chkerr(11) = 1
            if (gwpphase1 .ne. gwpphase) chkerr(11) = 1
         else if (lgwp .and. check .eq. 4) then
            gwptype=gwptype1
            gwpphase=gwpphase1
            if (gwpphase .ge. 0) then
               lgwpphase=.true.
            else
               lgwpphase=.false.
            endif
            check=-4
         endif
         if(chkerr(2).eq.1) goto 800
         if(chkerr(4).eq.1) goto 810
         if(chkerr(5).eq.1) goto 820

! finally, calculate new data if old and new calculations differ
         if (check .eq. -4) call griddat

      else if (check .eq. 3) then      
! grdinfo checked as subset of system data
! can be useful if the coordinates are combined in potfit
         do f=1,ndof1
            mdvr(f)=fdvr(f)
         enddo
! check that the natpot-particle is a sub-particle
! of the mctdh-particle
         do m1=1,nmode1
!           get the mctdh-particle (m) corresponding to the
!           natpot-particle (m1)
            f1=spfdof1(1,m1)
            f =fdvr(f1)
            m =dofspf(f)
            mmode(m1)=m
! the number of degrees combined in potfit cannot be larger than
! the number of degrees of freedom in the same mode in mctdh
            if (nspfdof1(m1) .gt. nspfdof(m)) chkerr(4)=1
! all the DOFs from the natpot-particle must be inside the
! mctdh-particle, in the same order
!           (f ist the mctdh-dof corresponding to the first
!            natpot-dof inside natpot-particle m1, and the
!            n0-th mctdh-dof inside mctdh-particle m.)
            do n=1,nspfdof(m)
               if (spfdof(n,m).eq.f) n0=n
            enddo
!           (now check the dofs from the natpot-particle)
            do n=1,nspfdof1(m1)
               f1=spfdof1(n,m1)
               f =fdvr(f1)
               if (f.ne.spfdof(n0+n-1,m)) chkerr(4)=1
            enddo
         enddo
 
!.....Check while ignoring electronic basis. Used for "read-inwf".
      else if (check .eq. 6) then
! check that grid definitions are the same.
         if(mpb.gt.0 .and. mpb1.eq.0) then ! packet basis only for system
            if (nmode1 .ne. nmode-1) chkerr(4)=1
         else
            if (nmode1 .ne. nmode) chkerr(4)=1
         endif
         do m=1,min(nmode,nmode1)
            if (nspfdof1(m) .ne. nspfdof(m)) chkerr(4)=1
            do n=1,nspfdof1(m)
!.......The electronic basis have may shift spfdof by one.
               if((spfdof1(n,m) .ne. spfdof(n,m)-1) .and. &
                    (spfdof1(n,m) .ne. spfdof(n,m)+1) .and. &
                    (spfdof1(n,m) .ne. spfdof(n,m))) chkerr(4)=1
            enddo
         enddo

      else
         write(message,'(a,i5)') ' Illegal value of chkgrd:',check
         routine='rdgrddef (mctdhlib/iogrddef.F)'
         call errormsg
      endif

      if(chkerr(4).eq.1) goto 810

      return

 800  write(2,'(/a)') ' Check ERROR 2 in rdgrddef'
      write(2,'(a,2i5)') ' ndof1, ndof ', ndof1, ndof
      if(check.eq.20) write(2,'(a,2i5)') ' ndof3, ndof2', ndof3, ndof2
      if(chkerr(4).eq.1) goto 810
      if(chkerr(5).eq.1) goto 820
      goto 860

 810  write(2,'(/a)') ' Check ERROR 4 in rdgrddef'
      if(check.eq.3) write(2,'(2a)') ' Checking natpot potential.', &
                     ' Equivalence not required! See rdgrddef.'
      write(2,'(a,2i5)') ' nmode1,nmode', nmode1, nmode
      write(2,'(a)') '  m, nspfdof1, nspfdof'
      do m=1,max(nmode,nmode1)
          write(2,'(i4,2i7)') m,nspfdof1(m),nspfdof(m)
      enddo
      write(2,'(/a)') '  f, dofspf1, dofspf'
      do f=1,max(ndof,ndof1)
          write(2,'(i4,2i7)') f,dofspf1(f),dofspf(f)
      enddo
      write(2,'(/a)') '  m, spfdof1'
      write(2,'(a)')  '     spfdof'

      do m=1,max(nmode,nmode1)
          write(2,'(8i4)') m, (spfdof1(n,m),n=1,min(7,nspfdof(m)))
          write(2,'(4x,7i4)') (spfdof(n,m), n=1,min(7,nspfdof(m)))
      enddo

      if(chkerr(5).eq.1) goto 820
      goto 860

 820  write(2,'(/a)') ' Check ERROR 5 in rdgrddef'
      write(2,'(a,2l3)') 'lmult1, lmult ',lmult1, lmult
      write(2,'(a,2l3)') 'leb1, leb ',leb1, leb
      write(2,'(a,2l3)') 'lmulpack1, lmulpack ',lmulpack1,lmulpack

 860  inquire(unit,name=filename)
      n=slen(filename)
      write(6,'(a)') '-------------------------------------------------'
      write(2,'(a)') '-------------------------------------------------'
      write(6,'(a,i3)') 'rdgrddef:  chkgrd =',check
      write(2,'(a,i3)') 'rdgrddef:  chkgrd =',check
      write(6,'(2a)') 'File checked: ',filename(1:n)
      write(2,'(2a)') 'File checked: ',filename(1:n)

      routine='Rdgrddef (iogrddef)'
      message='Grids differ between the system and file read'
      call errormsg

      return

      end subroutine rdgrddef11

!#######################################################################
! RDGRDDEF
!
! reads refined definition of a system (nmode etc) from a file, and
! checks that it matches calculation.
!
! check=0 : grdinfo read, no action taken
! check=1 : grdinfo read, and stored in system data
! check=2 : grdinfo checked against system data
! check=3 : grdinfo checked as subset of system data 
! check=4 : as check=2, but different electronic bases allowed
!                       and different GWP types allowed
! check=6 : grdinfo checked against system data while ignoring an
!           electronic DOF. This is used for the read-inwf block and
!           the block-spf keyword.
! check=20,40, ..: Similar to check=2,4 but a check of dummy DOFs is avoided
!           if those are at the end of the primitive basis list.
!#######################################################################

      subroutine rdgrddef_pre11(unit,chk,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
 
use maxv, only: maxdim
use operdef

      implicit none
 
      integer(long), intent(in)    :: unit, chk
      integer(long), intent(out)   :: nmode1
      integer(long), dimension(maxdim), intent(in)         :: fdvr
      integer(long), dimension(maxdim), intent(out)        :: nspfdof1,mdvr, &
                                                              mmode
      integer(long), dimension(maxdim,maxdim), intent(out) :: spfdof1

      integer(long) :: check
      integer(long) :: m,n,f,m1,n1,f1,n0,ndof1,ndof2,ndof3,ilmult1,ileb1, &
                       ilmulpack1,dentype1,gwptype1,gwpphase1,nstate1, &
                       npacket1,npackts1,feb1,meb1,fpb1,mpb1

      integer(long), dimension(maxdim)     :: dofspf1,igwpm1
      integer(long), external              :: slen

      character(len=c1), dimension(maxdim) :: modelabel1
      logical(kind=4)                      :: lmult1,leb1,lmulpack1,&
                                              tst,lred
      logical(kind=4), external            :: int2log
      logical(kind=4), dimension(maxdim)   :: gwpm1
      character(len=c5)                    :: filename  

! --- modify check if chk .ge. 20
      if(chk .lt. 20) then
         check = chk
         lred=.false.
      else
         check = chk/10
         lred=.true.
      endif

      mdvr(:)=0
      mmode(:)=0
      nspfdof1(:)=0
      dofspf1(:)=0
      spfdof1(:,:)=0
      n0 = 0

      if (filever(unit) .ge. 8.1011_dop) then
         read(unit,iostat=chkerr(1),err=666) dentype1
      else
         dentype1=0
      endif
      read(unit,iostat=chkerr(1)) ndof1,nmode1
      if (filever(unit) .ge. 8.4002_dop .and. filever(unit) .lt. 9.000_dop) then
          read(unit,iostat=chkerr(1)) nstate1,npacket1,npackts1,feb1,meb1,&
               fpb1,mpb1
      endif
      read(unit,iostat=chkerr(1)) (nspfdof1(m),m=1,nmode1)
      do m=1,nmode1
         read(unit,iostat=chkerr(1)) (spfdof1(n,m),n=1,nspfdof1(m))
      enddo
      read(unit,iostat=chkerr(1)) (dofspf1(f),f=1,ndof1)
  
      read(unit,iostat=chkerr(1),err=666) ilmult1,ileb1,ilmulpack1
      lmult1    = int2log(ilmult1)
      leb1      = int2log(ileb1)
      lmulpack1 = int2log(ilmulpack1)
      goto 777
 666  continue
      backspace(unit)
      read(unit,iostat=chkerr(1)) ilmult1,ileb1
      lmult1    = int2log(ilmult1)
      leb1      = int2log(ileb1)
      lmulpack1 = .false.
 777  continue
      if (filever(unit) .ge. 9.0000_dop) then
         read(unit,iostat=chkerr(1)) (igwpm1(m),m=1,nmode1),gwptype1, &
                                      gwpphase1
         do m=1,nmode1
            gwpm1(m)=int2log(igwpm1(m))
         enddo
      else
         gwptype1=gwptype
         gwpphase1=gwpphase
         call zerovxl(gwpm1,nmode1)
      endif

!-----------------------------------------------------------------------
! add feb to system for old versions
!-----------------------------------------------------------------------
      if (filever(unit) .lt. 11.000_dop) then 
         if (feb .eq. ndof1+1) then
            ndof1=ndof1+1
            dofspf1(feb)=nopmode
            modelabel1(feb)=modelabel(feb)
            lmult1=.true.
            leb1=.false.
         endif
      endif

!-----------------------------------------------------------------------
! Reduce ndof and ignore dummy DOFs
!-----------------------------------------------------------------------
      ndof2=ndof
      ndof3=ndof1
      if(lred) then
  40     if(dofspf(ndof2).gt.nmode .or. dofspf(ndof2).eq.0) then
            ndof2=ndof2-1
            if(ndof2.gt.0) goto 40
         endif
         if(ndof2.eq.0) then
            routine='rddvrdef (mctdh/iogrddef.f90)'
            message='dofspf seems to be unset'
            call errormsg
         endif
         ndof3=min(ndof2,ndof3) ! DANGER, it is expected that ndof is
      endif      ! checked in rdgrddef. ndof3 cannot be reduced here,
                 ! because dofspf1 not known here.

!-----------------------------------------------------------------------
! calculate grid data
!-----------------------------------------------------------------------
      if (check .eq. -1) return  ! read, no action taken.

      if (check .eq. 0) then
         do f=1,ndof1
            f1=fdvr(f)
            modelabel1(f)=modelabel(f1)
         enddo

         do m=1,nmode1
            do n=1,nspfdof1(m)
               f=spfdof1(n,m)
               do m1=1,nmode
                  do n1=1,nspfdof(m1)
                     f1=spfdof(n1,m1)
                     if (modelabel(f1) .eq. modelabel1(f)) then
                        mdvr(f)=f1
                        mmode(m)=m1
                     endif
                  enddo
               enddo
            enddo
         enddo

      else if (check .eq. 1) then
         dentype=dentype1
         nmode=nmode1
         do m=1,nmode
            nspfdof(m)=nspfdof1(m)
            do n=1,nspfdof(m)
               spfdof(n,m)=spfdof1(n,m)
            enddo
            gwpm(m)=gwpm1(m)

         enddo
         do f=1,ndof
            mdvr(f)=f
            dofspf(f)=dofspf1(f)
         enddo
         lmult = lmult1
         leb = leb1
         lmulpack = lmulpack1
         gwptype = gwptype1
         gwpphase = gwpphase1

         call griddat

      else if (check .eq. 2 .or. check .eq. 4 .or. check .eq. 20) then
! for multi-set calculations meb and dofspf may not be set. Set it now.
         if(lmult .and. feb.gt.0 ) then
            if(dofspf(feb).eq.0 .and. dofspf1(feb).gt.0) then
               dofspf(feb)=dofspf1(feb)
               meb=dofspf1(feb)
            elseif(dofspf1(feb).eq.0 .and. dofspf(feb).gt.0) then
               dofspf1(feb)=dofspf(feb)
            endif
         endif
         ndof2=ndof
         ndof3=ndof1
         if (ndof3 .ne. ndof2) chkerr(2)=1
         if (nmode1 .ne. nmode) chkerr(4)=1
         if (dentype1 .ne. dentype) chkerr(10)=1

         do m=1,nmode
            if (nspfdof1(m) .ne. nspfdof(m)) chkerr(4)=1
            do n=1,nspfdof1(m)
               if (spfdof1(n,m) .ne. spfdof(n,m)) chkerr(4)=1
            enddo
         enddo

         do f=1,ndof2
! The following two if's are for avoiding problems with dummy DOFs.
            if (dofspf(f).eq.0 .and. dofspf1(f).gt.nmode) dofspf1(f)=0
            if (dofspf(f).gt.nmode .and. (dofspf1(f).gt.nmode .or. &
                 dofspf1(f).eq.0) ) dofspf1(f)=dofspf(f)
            if (dofspf1(f) .ne. dofspf(f)) chkerr(4)=1
         enddo

! check that basis types are the same. lebdiff is .true. if this is a
! restart calculation with different electronic basis from old.
         if (check .ne. 4) then
            if (lmult1 .neqv. lmult) chkerr(5)=1
            if (leb1 .neqv. leb) chkerr(5)=1
         endif
! check that basis types are the same. Ignore lmulpack flag after 11.0
!         if (lmulpack1 .neqv. lmulpack) chkerr(5)=1
         if (lgwp .and. check .eq. 2) then
            if (gwptype1 .ne. gwptype) chkerr(11) = 1
            if (gwpphase1 .ne. gwpphase) chkerr(11) = 1
         else if (lgwp .and. check .eq. 4) then
            gwptype=gwptype1
            gwpphase=gwpphase1
            if (gwpphase .ge. 0) then
               lgwpphase=.true.
            else
               lgwpphase=.false.
            endif
            check=-4
         endif
         if (filever(unit) .ge. 8.4002d0 .and. filever(unit) .lt. 9.000_dop) then
            tst=.false.
            if(nstate.ne.nstate1)   tst=.true.
            if(npacket.ne.npacket1) tst=.true.
            if(npackts.ne.npackts1) tst=.true.
            if(feb.ne.feb1)    tst=.true.
            if(meb.ne.meb1)    tst=.true.
            if(fpb.ne.fpb1)    tst=.true.
            if(mpb.ne.mpb1)    tst=.true.
            if(tst) then
               write(2,'(a)') ' ### WARNING ### grddat variables differ'
               write(2,'(a)')'nstate,npacket,npackts,feb,  meb,  fpb,  mpb'
               write(2,'(1x,7i6)') nstate,npacket,npackts,feb,meb,fpb,mpb
               write(2,'(1x,7i6)') nstate1,npacket1,npackts1,feb1,meb1,fpb1,mpb1
            endif
         endif
         if(chkerr(2).eq.1) goto 800
         if(chkerr(4).eq.1) goto 810
         if(chkerr(5).eq.1) goto 820

! finally, calculate new data if old and new calculations differ
         if (check .eq. -4) call griddat

      else if (check .eq. 3) then      
! grdinfo checked as subset of system data
! can be useful if the coordinates are combined in potfit
         do f=1,ndof1
            mdvr(f)=fdvr(f)
         enddo
! check that the natpot-particle is a sub-particle
! of the mctdh-particle
         do m1=1,nmode1
!           get the mctdh-particle (m) corresponding to the
!           natpot-particle (m1)
            f1=spfdof1(1,m1)
            f =fdvr(f1)
            m =dofspf(f)
            mmode(m1)=m
! the number of degrees combined in potfit cannot be larger than
! the number of degrees of freedom in the same mode in mctdh
            if (nspfdof1(m1) .gt. nspfdof(m)) chkerr(4)=1
! all the DOFs from the natpot-particle must be inside the
! mctdh-particle, in the same order
!           (f ist the mctdh-dof corresponding to the first
!            natpot-dof inside natpot-particle m1, and the
!            n0-th mctdh-dof inside mctdh-particle m.)
            do n=1,nspfdof(m)
               if (spfdof(n,m).eq.f) n0=n
            enddo
!           (now check the dofs from the natpot-particle)
            do n=1,nspfdof1(m1)
               f1=spfdof1(n,m1)
               f =fdvr(f1)
               if (f.ne.spfdof(n0+n-1,m)) chkerr(4)=1
            enddo
         enddo
 
!.....Check while ignoring electronic basis. Used for "read-inwf".
      else if (check .eq. 6) then
! check that grid definitions are the same.
         if(mpb.gt.0 .and. mpb1.eq.0) then ! packet basis only for system
            if (nmode1 .ne. nmode-1) chkerr(4)=1
         else
            if (nmode1 .ne. nmode) chkerr(4)=1
         endif
         do m=1,min(nmode,nmode1)
            if (nspfdof1(m) .ne. nspfdof(m)) chkerr(4)=1
            do n=1,nspfdof1(m)
!.......The electronic basis have may shift spfdof by one.
               if((spfdof1(n,m) .ne. spfdof(n,m)-1) .and. &
                    (spfdof1(n,m) .ne. spfdof(n,m)+1) .and. &
                    (spfdof1(n,m) .ne. spfdof(n,m))) chkerr(4)=1
            enddo
         enddo

      else
         write(message,'(a,i5)') ' Illegal value of chkgrd:',check
         routine='rdgrddef (mctdhlib/iogrddef.F)'
         call errormsg
      endif

      if(chkerr(4).eq.1) goto 810

      return

 800  write(2,'(/a)') ' Check ERROR 2 in rdgrddef'
      write(2,'(a,2i5)') ' ndof1, ndof ', ndof1, ndof
      if(check.eq.20) write(2,'(a,2i5)') ' ndof3, ndof2', ndof3, ndof2
      if(chkerr(4).eq.1) goto 810
      if(chkerr(5).eq.1) goto 820
      goto 860

 810  write(2,'(/a)') ' Check ERROR 4 in rdgrddef'
      if(check.eq.3) write(2,'(2a)') ' Checking natpot potential.', &
                     ' Equivalence not required! See rdgrddef.'
      write(2,'(a,2i5)') ' nmode1,nmode', nmode1, nmode
      write(2,'(a)') '  m, nspfdof1, nspfdof'
      do m=1,max(nmode,nmode1)
          write(2,'(i4,2i7)') m,nspfdof1(m),nspfdof(m)
      enddo
      write(2,'(/a)') '  f, dofspf1, dofspf'
      do f=1,max(ndof,ndof1)
          write(2,'(i4,2i7)') f,dofspf1(f),dofspf(f)
      enddo
      if(chkerr(5).eq.1) goto 820
      goto 860

 820  write(2,'(/a)') ' Check ERROR 5 in rdgrddef'
      write(2,'(a,2l3)') 'lmult1, lmult ',lmult1, lmult
      write(2,'(a,2l3)') 'leb1, leb ',leb1, leb
      write(2,'(a,2l3)') 'lmulpack1, lmulpack ',lmulpack1,lmulpack

 860  inquire(unit,name=filename)
      n=slen(filename)
      write(6,'(a)') '-------------------------------------------------'
      write(2,'(a)') '-------------------------------------------------'
      write(6,'(a,i3)') 'rdgrddef:  chkgrd =',check
      write(2,'(a,i3)') 'rdgrddef:  chkgrd =',check
      write(6,'(2a)') 'File checked: ',filename(1:n)
      write(2,'(2a)') 'File checked: ',filename(1:n)

      routine='Rdgrddef (iogrddef)'
      message='Grids differ between the system and file read'
      call errormsg

      return

      end subroutine rdgrddef_pre11

!#######################################################################
 
      subroutine griddat
 
      implicit none
 
      integer(long)       :: m,f,f1,zeig,n
      real(dop)           :: fac

!-----------------------------------------------------------------------
! check for overflow on grid size
!-----------------------------------------------------------------------
      do m = 1, nmode
! ignore GWP modes as grid as mode grid is never built
         fac = 1.0d0 + 1.0d-12  ! 1^-12 added to avoid round-off
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            if (basis(f) .ge. 25 .and. basis(f) .le. 27) cycle
            fac = fac*gdim(f)
         enddo
         if(fac .gt. 1.0d9) then ! integer overflow
            routine='Griddat (iogrddef.f90)'
            write(message,'(a,i5,a,e12.4)') &
              'Grid size overflow - error in mode combination ',m,&
              ' Asked for:',fac
            call errormsg
         endif
      enddo

!-----------------------------------------------------------------------
! calculate spf grid dimensions (subdim, vgdim, ngdim) for each mode.
!-----------------------------------------------------------------------
!      write(1000,*)"dentype",dentype,"source/mctdhlib/iogrddef.f90"
!      write(1000,*)"gwptype",gwptype,"source/mctdhlib/iogrddef.f90"
!      write(1000,*)"lBEL",lBEL,"source/mctdhlib/iogrddef.f90"
      do m=1,nmode
         zeig=1
         f1 = spfdof(1,m)
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            vgdim(f) = zeig
            zeig=zeig*gdim(f)
         enddo

         if (dentype .eq. 1) then
            supgdim(m)=zeig
            subdim(m)=zeig*zeig

         else

! ...    for Gaussian mode :
!           npg(m)    : no. of parameters defining Gaussian
!           subdim(m) : no. of numbers stored in psi array. 
!
            if (basis(f1) .eq. 25)then
               lgwp=.true.
               gwpm(m)=.true.
               if (gwpphase .eq. -1) then
                  lgwpphase = .false.
               else
                  lgwpphase = .true.
               endif

               if (gwptype .eq. 2 .or. gwptype .eq. 3) then
                  lgwpsep = .true.
               endif

               if (gwptype .eq. 0) then
                  npG(m) = nspfdof(m)*(nspfdof(m)+1)/2 + nspfdof(m) + 1 

                  subdim(m) = nspfdof(m)**2 + nspfdof(m) 
                  if (lgwpphase) subdim(m) = subdim(m) + 1

               else if (gwptype .eq. 1) then
                  npG(m) = nspfdof(m)*(nspfdof(m)+1)/2 - nspfdof(m) + nspfdof(m) + 1 

                  subdim(m) = nspfdof(m)**2 + nspfdof(m) 
                  if (lgwpphase) subdim(m) = subdim(m) + 1

               else if (gwptype .eq. 2) then
                  npG(m) = nspfdof(m) + nspfdof(m) + 1

                  subdim(m) = nspfdof(m) + nspfdof(m)
                  if (lgwpphase) subdim(m) = subdim(m) + 1

               else if (gwptype .ge. 3) then
                  if (lBEL)then 
                     npG(m) = 1
                     write(1000,*)"npg(m)",npg(1),"source/mctdhlib/iogrddef.f90"
                  else
                     npG(m) = nspfdof(m) + 1
                  endif

                  subdim(m) = nspfdof(m)
                  if (lgwpphase) subdim(m) = subdim(m) + 1

               endif

            elseif(basis(f1) .eq. 26)then
               subdim(m) = 3

            elseif(basis(f1) .eq. 27)then
               subdim(m) = nspfdof(m)

            else 
               subdim(m)=zeig
            endif

         endif

         do n=1,nspfdof(m)
            f=spfdof(n,m)
            ngdim(f) = zeig/(gdim(f)*vgdim(f))
         enddo
      enddo

!-----------------------------------------------------------------------
! define nddstate as the actual number of electronic states
! while nstate is the no. of wavepackets (e.g. in a multi-set 
! calculation)
!-----------------------------------------------------------------------
      if (lebcont) then
         nddstate=eltot
      else if (feb .gt. 0) then
         nddstate = gdim(feb)
      endif
      if (leb) meb = dofspf(feb)

      if (lmult) then
         nstate=gdim(feb)
      else
         nstate=1
      endif

!-----------------------------------------------------------------------
! set max of mode grid dimensions
!-----------------------------------------------------------------------
      sdim=1
      do m=1,nmode
         sdim=max(sdim,subdim(m))
         if (dentype .eq. 1) sgdim=max(sgdim,supgdim(m))
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            if(basis(f).eq.12 .or. basis(f).eq.20 .or. basis(f).eq.21) &
               sdim=max(sdim,ipbaspar(3,f)*subdim(m)/gdim(f))
         enddo
      enddo
 
!-----------------------------------------------------------------------
! set max of DOFs in a particle
!-----------------------------------------------------------------------
      maxdof=1
      do m=1,nmode
         maxdof=max(maxdof,nspfdof(m))
      enddo
 
      return
      end subroutine griddat

      end module iogrddef

