!
! contains :: wrpsidef,rdpsidef,psidat,jspacedat,decompj,gwpdat
!
      module iopsidef

use decimal, only: long, dop
use op1lib,  only: zerovxi
use global
use dglobal, only: maxdimspwn,tapeM,itape2
use dvrdatmod
use griddatmod
use psidef
use logdat
use wftree
use if_grid
use maxv, only: maxdim,maxsta,maxtape,maxgwp
use psiutil, only: maxmatwk
use linear, only: ijk_shape
use channels, only: filever

      implicit none

      private
      public :: wrpsidef,rdpsidef,psidat,decompj,gwpdat,setzetf,&
                tapelength,setphidim
      contains

!#######################################################################
! WRSYSDEF2
!
! writes further definition of a system (dim etc) to a file
!
!#######################################################################

      subroutine wrpsidef(unit)

      implicit none

      integer(long)             :: f,m,s,t,p,i
      integer(long), intent(in) :: unit
      integer(long), external   :: log2int
      character(len=c2) :: wrtag

      wrtag= '<start psi definition>'
      write(unit) wrtag

      wrtag= '<dentype>'
      write(unit) wrtag
      write(unit) dentype
      wrtag= '<psitype>'
      write(unit) wrtag
      write(unit) psitype
      wrtag= '<nstate>'
      write(unit) wrtag
      write(unit) nstate
      wrtag= '<npacket>'
      write(unit) wrtag
      write(unit) npacket
      wrtag= '<npackts>'
      write(unit) wrtag
      write(unit) npackts
      wrtag= '<nlayers>'
      write(unit) wrtag
      write(unit) nlayers
      if (dentype .gt. 0) then
         wrtag= '<dlstate>'
         write(unit) wrtag
         write(unit) dlstate
         wrtag= '<drstate>'
         write(unit) wrtag
         write(unit) drstate
      endif
      if (dentype .eq. 1) then
         wrtag= '<ddim>'
         write(unit) wrtag
         write(unit) (((ddim(m,s,t),m=1,nmode),s=1,nstate),t=1,nstate)
      endif
! write maximum dim for dynWF to allow correct memory dimensioning
      wrtag= '<dim>'
      write(unit) wrtag
      if (ldynspf) then
         write (unit) ((maxdimspwn(m,s),m=1,nmode),s=1,nstate)
      else 
         write (unit) ((dim(m,s),m=1,nmode),s=1,nstate)
      endif
      wrtag= '<ntraj>'
      write(unit) wrtag
      write (unit) ntraj
      if (lgwp) then
         wrtag= '<gwpdim>'
         write(unit) wrtag
         write (unit) ((gwpdim(m,s),m=1,nmode),s=1,nstate)
      endif
      if (lBEL) then
         wrtag= '<dimred>'
         write(unit) wrtag
         write (unit) ((dimred(m,s),m=1,nmode),s=1,nstate)
      endif
      wrtag= '<citype>'
      write(unit) wrtag
      write(unit) citype
      if (citype .eq. 1) then
         wrtag= '<ciweight>'
         write(unit) wrtag
         write (unit) (ciweight(m),m=1,nmode)
      endif
      if(psitype.eq.5) then
         wrtag= '<tapelength>'
         write(unit) wrtag
         write(unit) tapelength()
         wrtag= '<tape>'
         write(unit) wrtag
! write maximum dim for dynWF to allow correct memory dimensioning
         if (ldynspf) then
            write(unit) (tapeM(m),m=1,tapelength())
         else
            write(unit) (tape(m),m=1,tapelength())
         endif
         if (ldynspf) then
            wrtag= '<itape2>'
            write(unit) wrtag
            write(unit) (itape2(m),m=1,tapelength())
         endif
         wrtag= '<mladim>'
         write(unit) wrtag
         write(unit) mladim
      end if
      wrtag= '<msymmtr>'
      write(unit) wrtag
      write (unit) (msymmtr(m),m=1,2*nmode)
      wrtag= '<idmode>'
      write(unit) wrtag
      write (unit) (idmode(m),m=1,nmode)
      wrtag= '<symcv>'
      write(unit) wrtag
      write(unit) (symcv(m),m=1,nmode)

      wrtag= '<sbaspar>'
      write(unit) wrtag
      write(unit) sbaspar
      wrtag= '<sbasis>'
      write(unit) wrtag
      write(unit) (sbasis(f),f=1,ndof)
      wrtag= '<isbaspar>'
      write(unit) wrtag
      write(unit) (((isbaspar(i,f,p),i=1,sbaspar),f=1,ndof),p=1,nstate*npacket)
      wrtag= '<rsbaspar>'
      write(unit) wrtag
      write(unit) (((rsbaspar(i,f,p),i=1,sbaspar),f=1,ndof),p=1,nstate*npacket)
      wrtag= '<ltdp>'
      write(unit) wrtag
      write(unit) log2int(ltdp)

      wrtag= '<end psi definition>'
      write(unit) wrtag

      return
      end subroutine wrpsidef

!#######################################################################
! RDPSIDEF
!
! reads further definition of a system (dim etc) from a file, and
! checks that it matches calculation.
!
! check=0 : psiinfo read, no action taken
! check=1 : psiinfo read, and stored in system data
! check=2 : psiinfo checked against system data
! check=3 : psiinfo checked against system data, except nspf's
!#######################################################################

      subroutine rdpsidef(unit,check)

      implicit none

      integer(long), intent(in) :: unit,check

      if (filever(unit) .lt. 11.00_dop) then
         call rdpsidef_pre11(unit,check)
      else
         call rdpsidef11(unit,check)
      endif

      return
      end subroutine rdpsidef

!#######################################################################

      subroutine rdpsidef11(unit,check)


      implicit none

      integer(long), intent(in) :: unit,check
      integer(long)             :: ilbl,jlbl
      integer(long)             :: f,m,s,t,dentype1,ndof1,nmode1,nstate1,&
                                   npacket1,psitype1,citype1,npackts1,i,p,&
                                   ntraj1
      integer(long)             :: iltdp1,sbaspar1,dlstate1,drstate1,&
                                   nlayers1
      integer(long)             :: tapelength1
      integer(long), dimension(maxdim,maxsta)        :: dim1,gwpdim1,dimred1
      integer(long), dimension(maxdim,maxsta,maxsta) :: ddim1
      integer(long), dimension(2*maxdim)             :: msymmtr1
      integer(long), dimension(maxdim)               :: idmode1,symcv1,&
                                                        sbasis1
      integer(long), dimension(maxtape)              :: tape1,itape21
      logical(kind=4)                                :: ltdp1
      real(dop)                                      :: cicut1
      real(dop), dimension(maxdim)                   :: ciweight1
      logical(kind=4), external                      :: int2log
      character(len=c2)                              :: rdtag
      integer(long), dimension(sbaspar,maxdim,maxsta*maxsta)  :: isbaspar1
      real(dop), dimension(sbaspar,maxdim,maxsta*maxsta)      :: rsbaspar1


      if (check .le. 0) return

      cicut1=0
      ltdp1=.false.

      dim1(:,:)=0
      gwpdim1(:,:)=0
      dimred1(:,:)=0

!-----------------------------------------------------------------------
! READ INFO
!-----------------------------------------------------------------------
      rewind(unit)

      do
   10    continue
         read(unit,err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'ndof') then
            read(unit,iostat=chkerr(1)) ndof1
         else if (rdtag(ilbl:jlbl) .eq. 'nmode') then
            read(unit,iostat=chkerr(1)) nmode1
         else if (rdtag(ilbl:jlbl) .eq. 'nstate') then
            read(unit,iostat=chkerr(1)) nstate1
         else if (rdtag(ilbl:jlbl) .eq. 'dentype') then
            read(unit,iostat=chkerr(1)) dentype1
         else if (rdtag(ilbl:jlbl) .eq. 'psitype') then
            read (unit,iostat=chkerr(1)) psitype1
         else if (rdtag(ilbl:jlbl) .eq. 'npacket') then
            read(unit,iostat=chkerr(1)) npacket1
         else if (rdtag(ilbl:jlbl) .eq. 'npackts') then
            read(unit,iostat=chkerr(1)) npackts1
         else if (rdtag(ilbl:jlbl) .eq. 'nlayers') then
            read(unit,iostat=chkerr(1)) nlayers1

         else if (rdtag(ilbl:jlbl) .eq. 'dlstate') then
            read(unit,iostat=chkerr(1)) dlstate1
         else if (rdtag(ilbl:jlbl) .eq. 'drstate') then
            read(unit,iostat=chkerr(1)) drstate1
         else if (rdtag(ilbl:jlbl) .eq. 'ddim') then
            read(unit,iostat=chkerr(1)) &
                  (((ddim1(m,s,t),m=1,nmode1),s=1,nstate1),t=1,nstate1)
         else if (rdtag(ilbl:jlbl) .eq. 'dim') then
            read(unit,iostat=chkerr(1)) ((dim1(m,s),m=1,nmode1),s=1,nstate1)
         else if (rdtag(ilbl:jlbl) .eq. 'ntraj') then
            read(unit,iostat=chkerr(1)) ntraj1
         else if (rdtag(ilbl:jlbl) .eq. 'gwpdim') then
            read(unit,iostat=chkerr(1)) ((gwpdim1(m,s),m=1,nmode1),s=1,nstate1)
         else if (rdtag(ilbl:jlbl) .eq. 'dimred') then
            read(unit,iostat=chkerr(1)) ((dimred1(m,s),m=1,nmode1),s=1,nstate1)
         else if (rdtag(ilbl:jlbl) .eq. 'citype') then
            read (unit,iostat=chkerr(1)) citype1
         else if (rdtag(ilbl:jlbl) .eq. 'cicut') then
            read (unit,iostat=chkerr(1)) (ciweight1(m),m=1,nmode1)
         else if (rdtag(ilbl:jlbl) .eq. 'tapelength') then
            read (unit,iostat=chkerr(1)) tapelength1
            if (tapelength1 .gt. maxtape) then
               routine = 'Rdpsidef11 (Iopsidef.f90)'
               message = 'Tapelength > Maxtape. ML calculations, not set up for one.'
               call errormsg
            endif
         else if (rdtag(ilbl:jlbl) .eq. 'tape') then
            read (unit,iostat=chkerr(1)) (tape1(m),m=1,tapelength1)
         else if (rdtag(ilbl:jlbl) .eq. 'mladim') then
            read (unit,iostat=chkerr(1)) mladim
         else if (rdtag(ilbl:jlbl) .eq. 'itape2') then
            read (unit,iostat=chkerr(1)) (itape21(m),m=1,tapelength1)
         else if (rdtag(ilbl:jlbl) .eq. 'msymmtr') then
            read (unit,iostat=chkerr(1)) (msymmtr1(m),m=1,2*nmode1)
         else if (rdtag(ilbl:jlbl) .eq. 'idmode') then
            read (unit,iostat=chkerr(1)) (idmode1(m),m=1,nmode1)
         else if (rdtag(ilbl:jlbl) .eq. 'symcv') then
            read (unit,iostat=chkerr(1)) (symcv1(m),m=1,nmode1)
         else if (rdtag(ilbl:jlbl) .eq. 'sbaspar') then
            read(unit,iostat=chkerr(1)) sbaspar1
         else if (rdtag(ilbl:jlbl) .eq. 'sbasis') then
            read(unit,iostat=chkerr(1)) (sbasis1(f),f=1,ndof1)
         else if (rdtag(ilbl:jlbl) .eq. 'isbaspar') then
            read(unit,iostat=chkerr(1)) &
                (((isbaspar1(i,f,p),i=1,sbaspar1),f=1,ndof1),p=1,nstate1)
         else if (rdtag(ilbl:jlbl) .eq. 'rsbaspar') then
            read(unit,iostat=chkerr(1)) &
                (((rsbaspar1(i,f,p),i=1,sbaspar1),f=1,ndof1),p=1,nstate1)
         else if (rdtag(ilbl:jlbl) .eq. 'ltdp') then
            read(unit,iostat=chkerr(1)) iltdp1
            ltdp1=int2log(iltdp1)
         endif

      enddo

  100 continue

!-----------------------------------------------------------------------
! carry out requested action on psi definition
!-----------------------------------------------------------------------
      if (check .eq. 1) then
         dentype=dentype1
         npacket=npacket1
         if (npacket .gt. 1) lmulpack = .true.
         npackts=npackts1
         npacktot = max(npacket,npackts)
         nlayers=nlayers1
         ntraj=ntraj1
         if (dentype .eq. 1 .or. dentype .eq. 2) then
            dlstate=dlstate1
            drstate=drstate1
         endif
         if (dentype .eq. 1) then
            do s=1,nstate1
               do t=1,nstate1
                  do m=1,nmode1
                     ddim(m,s,t)=ddim1(m,s,t)
                  enddo
               enddo
            enddo
            do m=1,nmode1
               dim(m,1)=dim1(m,1)
            enddo
         endif
         do s=1,nstate1
            do m=1,nmode1
               dim(m,s)=dim1(m,s)
               if (lgwp) gwpdim(m,s)=gwpdim1(m,s)
               if (lBEL) dimred(m,s)=dimred1(m,s)
               if (ldynspf) maxdimspwn(m,s)=dim1(m,s)
            enddo
         enddo
         psitype=psitype1
         do m=1,nmode1
            idmode(m)=idmode1(m) 
            symcv(m)=symcv1(m)
            msymmtr(m)=msymmtr1(m)
            msymmtr(m+nmode)=msymmtr1(m+nmode)
         enddo
         citype=citype1
         if (citype .eq. 1) then
            cicut=cicut1
            do m=1,nmode1
               ciweight(m)=ciweight1(m)
            enddo
         endif
         ltdp=ltdp1
! ML-MCTDH
         if (psitype .eq. 5) then
            lrunml=.true.
            tape(:) = tape1(:)
            if (ldynspf) then
                itape2(:) = itape21(:)
            endif

! set up tree 
            tree => new_wftree(tape,if_spfdof)
            mladim  = adim_wftree(tree)
            ml_input_ready = .true.
               
         else
            lrunml=.false.
         endif

         call psidat

! parameters for initial WP
         if (sbaspar .ne. sbaspar1) then
            routine='Rdpsidef11'
            message = 'sbaspar differs from sbaspar1. Help.'
            call errormsg
         endif
         sbasis=sbasis1
         isbaspar=isbaspar1
         rsbaspar=rsbaspar1

      else if (check .eq. 2) then
! check that definitions match
         if (npacket1 .ne. npacket) chkerr(4)=1
         if (npackts1 .ne. npackts) chkerr(4)=1
         if (nlayers1 .ne. nlayers) chkerr(4)=1
         if (ntraj1 .ne. ntraj) chkerr(4)=1
         if (psitype1 .ne. psitype) chkerr(9)=1
         if (dentype1 .ne. dentype) chkerr(10)=1
         if (citype1 .ne. citype) chkerr(4)=1
         if (citype1 .eq. 1) then
            do m=1,nmode
               if (abs(ciweight1(m)-ciweight(m)) .gt. tiny) chkerr(4)=1
            enddo
         endif
         do m=1,nmode
            if ( idmode(m) .ne. idmode1(m) ) chkerr(4)=1
            if ( symcv(m)  .ne.  symcv1(m) ) chkerr(4)=1
         enddo
         if (ltdp .neqv. ltdp1) chkerr(4)=1
         if (dentype .eq. 1) then
            if (dlstate .ne. dlstate1) chkerr(6)=1
            if (drstate .ne. drstate1) chkerr(6)=1
            do s=1,nstate
               do t=1,nstate
                  do m=1,nmode
                     if (ddim1(m,s,t) .ne. ddim(m,s,t)) chkerr(6)=1
                  enddo
               enddo
            enddo
         endif
         do s=1,nstate
            do m=1,nmode
               if (dim1(m,s) .ne. dim(m,s)) chkerr(6)=1
               if (lgwp .and. (gwpdim1(m,s) .ne. gwpdim(m,s))) chkerr(6)=1
               if (lBEL .and. (dimred1(m,s) .ne. dimred(m,s))) chkerr(6)=1
            enddo
         enddo
         if(chkerr(4).eq.1) goto 810
         if(chkerr(6).eq.1) goto 820
! should compare old and new tape for ML calcs
!         if (psitype .eq. 5) then
!            do m=1,tapelength
!               if (tape(m) .ne. tape1(m)) chkerr(6) = 1
!            enddo
!         endif

      else if (check.eq.3) then
! check that definitions match for all data except dim(m,s)
         if (npacket1 .ne. npacket) chkerr(4)=1
         if (npackts1 .ne. npackts) chkerr(4)=1
         if (nlayers1 .ne. nlayers) chkerr(4)=1
         if (ntraj1 .ne. ntraj) chkerr(4)=1
         if (psitype1 .ne. psitype) chkerr(9)=1
         if (filever(unit) .ge. 8.4005d0) then
            do m=1,nmode
               if ( idmode(m) .ne. idmode1(m) ) chkerr(4)=1
               if ( symcv(m)  .ne.  symcv1(m) ) chkerr(4)=1
            enddo
         endif
         if (ltdp .neqv. ltdp1) chkerr(4)=1
         if (dentype1 .ne. dentype) chkerr(10)=1
         if (citype1 .ne. citype) chkerr(4)=1
         if (citype1 .gt. 0) then
            do m=1,nmode
               if (abs(ciweight1(m)-ciweight(m)) .gt. tiny) chkerr(4)=1
               if ( idmode(m) .ne. idmode1(m) ) chkerr(4)=1
            enddo
         endif
         if (dentype .eq. 1) then
            if (dlstate .ne. dlstate1) chkerr(6)=1
            if (drstate .ne. drstate1) chkerr(6)=1
            do s=1,nstate
               do t=1,nstate
                  do m=1,nmode
                     ddim(m,s,t)=ddim1(m,s,t)
                  enddo
               enddo
            enddo
         endif
         do s=1,nstate
            do m=1,nmode
               dim(m,s)=dim1(m,s)
               if (lgwp) gwpdim(m,s)=gwpdim1(m,s)
               if (lBEL) dimred(m,s)=dimred1(m,s)
            enddo
         enddo

         call psidat

      endif

      return

 810  write(2,'(/a)') ' Check ERROR 4 in rdpsidef'
      write(2,'(a)') ' nmode,nstate,npacket,npackts,nlayers,ntraj'
      write(2,'(5i7)') nmode,nstate,npacket,npackts,nlayers,ntraj
      write(2,'(5i7)') nmode1,nstate1,npacket1,npackts1,nlayers1,ntraj1
      if(chkerr(6).eq.1) goto 820
      return

 820  write(2,'(/a)') ' Check ERROR 6 in rdpsidef'
      write(2,'(a)') '   s,  m,dim1(m,s),dim(m,s)'
      do s=1,nstate
         do m=1,max(nmode,nmode1)
             write(2,'(2i4,2i7)') s,m,dim1(m,s), dim(m,s)
         enddo
      enddo
      return

      end subroutine rdpsidef11

!#######################################################################

      subroutine rdpsidef_pre11(unit,check)

      implicit none

      integer(long), intent(in) :: unit,check
      integer(long)             :: f,m,s,t,dentype1,nmode1,nstate1,&
                                   npacket1,psitype1,citype1,npackts1,i,p
      integer(long)             :: iltdp,sbaspar1,idum
      integer(long)             :: tlen1
      integer(long), dimension(maxdim,maxsta)        :: dim1,gwpdim1,dimred1
      integer(long), dimension(maxdim,maxsta,maxsta) :: ddim1
      integer(long), dimension(2*maxdim)             :: msymmtr1
      integer(long), dimension(maxdim)               :: idmode1,symcv1
      real(dop)                                      :: cicut1
      real(dop), dimension(maxdim)                   :: ciweight1
      logical(kind=4), external                            :: int2log

      if (filever(unit) .ge. 8.1011_dop) then
          read(unit,iostat=chkerr(1),err=666) dentype1
      else
         dentype1=0
      endif
      if ((filever(unit) .ge. 8.4002_dop .and. filever(unit) .lt. 9.0000_dop) &
           .or. (filever(unit) .ge. 10.0000_dop)) then
         read(unit,iostat=chkerr(1),err=666) nmode1,nstate1,npacket1,npackts1
      else
         read(unit,iostat=chkerr(1),err=666) nmode1,nstate1,npacket1
         npackts1 = 1
      endif

      goto 777
 666  continue
      backspace(unit)
      read(unit,iostat=chkerr(1)) nmode1,nstate1
      npacket1 = 1
      npackts1 = 1
 777  continue
      if (dentype1 .eq. 1) then
         read(unit) dlstate, drstate
         do s=1,nstate1
            do t=1,nstate1
               read (unit,iostat=chkerr(1)) (ddim1(m,s,t),m=1,nmode1)
            enddo
         enddo
      else
         do s=1,nstate1
            read (unit,iostat=chkerr(1)) (dim1(m,s),m=1,nmode1)
            if (filever(unit) .ge. 10.1002_dop) then
               read (unit,iostat=chkerr(1)) (gwpdim1(m,s),m=1,nmode1)
               read (unit,iostat=chkerr(1)) (dimred1(m,s),m=1,nmode1)
            endif
         enddo
      endif
      if (filever(unit) .ge. 8.1004_dop) then
         read (unit,iostat=chkerr(1)) psitype1
! convert for change in GMCTDH psitype at version 10.1003
         if (filever(unit) .lt. 10.1003_dop) then
            if (lgwp .and. psitype1 .eq. 0) then
               psitype1 = 4
            else if (psitype1 .eq. 4) then
               psitype1 = 0
            endif
         endif
      else
         psitype1=0
      endif
! ML wfn was psitype = 3 before v11. It is now psitype=5
      if(psitype1==3 .and. filever(unit) .ge. 8.50) then  ! multilayer wfn
       !  read (unit,iostat=chkerr(1)) mladim,mlatopdim,mltotphidim
         read (unit,iostat=chkerr(1)) idum,idum,idum
         read (unit,iostat=chkerr(1)) tlen1
         read (unit,iostat=chkerr(1)) (tape(m),m=1,tlen1)
! change to value used after 11.0
         psitype1 = 5
         lrunml=.true.          ! Check analyse programs! Dieter Aug.2013
      end if
      if (filever(unit) .ge. 8.2000_dop) then
         read (unit,iostat=chkerr(1)) citype1
         if (citype1 .eq. 1) then
            read (unit,iostat=chkerr(1)) cicut1, &
                (ciweight1(m),m=1,nmode)
         else
            cicut1=2.0
            do m=1,nmode
               ciweight1(m)=1.0
            enddo
         endif
      else
        citype1=0
        cicut1=2.0
        do m=1,nmode
           ciweight1(m)=1.0
        enddo
      endif
      if (filever(unit) .ge. 8.3010_dop) then
         read (unit,iostat=chkerr(1)) (msymmtr1(m),m=1,2*nmode1)
         read (unit,iostat=chkerr(1)) (idmode1(m),m=1,nmode1)
      else
         do m = 1, nmode1
            idmode1(m)=0
            msymmtr1(m)=0
            msymmtr1(m+nmode1)=0
         enddo
      endif
      if ((filever(unit) .ge. 8.4005_dop .and. filever(unit) .lt. 9.0000_dop) &
           .or. (filever(unit) .ge. 10.0000_dop)) then
         read (unit,iostat=chkerr(1)) (symcv1(m),m=1,nmode1)
      else
         do m = 1, nmode1
            symcv1(m)=0
         enddo
      endif
      if (filever(unit) .ge. 10.1006_dop) then
         read(unit,iostat=chkerr(1)) sbaspar1
         read(unit,iostat=chkerr(1)) (sbasis(f),f=1,ndof)
         read(unit,iostat=chkerr(1)) &
                (((isbaspar(i,f,p),i=1,sbaspar1),f=1,ndof),p=1,nstate)
         read(unit,iostat=chkerr(1)) &
                (((rsbaspar(i,f,p),i=1,sbaspar1),f=1,ndof),p=1,nstate)
         read(unit,iostat=chkerr(1)) iltdp
         ltdp=int2log(iltdp)
      endif

!-----------------------------------------------------------------------
! carry out requested action on psi definition
!-----------------------------------------------------------------------
      if (check .eq. 0) then
         continue

      else if (check .eq. 1) then
         dentype=dentype1
         nmode=nmode1
         nstate=nstate1
         npacket=npacket1
         npackts=npackts1
         if (dentype .eq. 1) then
            do s=1,nstate
               do t=1,nstate
                  do m=1,nmode
                     ddim(m,s,t)=ddim1(m,s,t)
                  enddo
               enddo
            enddo
         else
            do s=1,nstate
               do m=1,nmode
                  dim(m,s)=dim1(m,s)
                  if (lgwp) gwpdim(m,s)=gwpdim1(m,s)
                  if (lBEL) dimred(m,s)=dimred1(m,s)
               enddo
            enddo
         endif
         psitype=psitype1
         do m=1,nmode
            idmode(m)=idmode1(m) 
            symcv(m)=symcv1(m)
            msymmtr(m)=msymmtr1(m)
            msymmtr(m+nmode)=msymmtr1(m+nmode)
         enddo
         citype=citype1
         if (citype .eq. 1) then
            cicut=cicut1
            do m=1,nmode
               ciweight(m)=ciweight1(m)
            enddo
         endif

         call psidat

      else if (check .eq. 2) then
! check that definitions match
         if (nmode1 .ne. nmode) chkerr(4)=1
         if (nstate1 .ne. nstate) chkerr(4)=1
         if (npacket1 .ne. npacket) chkerr(4)=1
         if (npackts1 .ne. npackts) chkerr(4)=1
         if (psitype1 .ne. psitype) chkerr(9)=1
         if (dentype1 .ne. dentype) chkerr(10)=1
         if (citype1 .ne. citype) chkerr(4)=1
         if (citype1 .eq. 1) then
            do m=1,nmode
               if (abs(ciweight1(m)-ciweight(m)) .gt. tiny) chkerr(4)=1
            enddo
         endif
         if (filever(unit) .ge. 8.4005d0) then
            do m=1,nmode
               if ( idmode(m) .ne. idmode1(m) ) chkerr(4)=1
               if ( symcv(m)  .ne.  symcv1(m) ) chkerr(4)=1
            enddo
         endif
         if (dentype .eq. 1) then
            do s=1,nstate
               do t=1,nstate
                  do m=1,nmode
                     if (ddim1(m,s,t) .ne. ddim(m,s,t)) chkerr(6)=1
                  enddo
               enddo
            enddo
         else
            do s=1,nstate
               do m=1,nmode
                  if (dim1(m,s) .ne. dim(m,s)) chkerr(6)=1
                  if (gwpdim1(m,s) .ne. gwpdim(m,s)) chkerr(6)=1
                  if (dimred1(m,s) .ne. dimred(m,s)) chkerr(6)=1
               enddo
            enddo
         endif
         if(chkerr(4).eq.1) goto 810
         if(chkerr(6).eq.1) goto 820

      else if (check.eq.3) then
! check that definitions match for all data except dim(m,s)
         if (nmode1 .ne. nmode) chkerr(4)=1
         if (nstate1 .ne. nstate) chkerr(4)=1
         if (npacket1 .ne. npacket) chkerr(4)=1
         if (npackts1 .ne. npackts) chkerr(4)=1
         if (psitype1 .ne. psitype) chkerr(9)=1
         if (filever(unit) .ge. 8.4005d0) then
            do m=1,nmode
               if ( idmode(m) .ne. idmode1(m) ) chkerr(4)=1
               if ( symcv(m)  .ne.  symcv1(m) ) chkerr(4)=1
            enddo
         endif
         if (dentype1 .ne. dentype) chkerr(10)=1
         if (citype1 .ne. citype) chkerr(4)=1
         if (citype1 .gt. 0) then
            do m=1,nmode
               if (abs(ciweight1(m)-ciweight(m)) .gt. tiny) chkerr(4)=1
               if ( idmode(m) .ne. idmode1(m) ) chkerr(4)=1
            enddo
         endif
         if (dentype .eq. 1) then
            do s=1,nstate
               do t=1,nstate
                  do m=1,nmode
                     ddim(m,s,t)=ddim1(m,s,t)
                  enddo
               enddo
            enddo
         else
            do s=1,nstate
               do m=1,nmode
                  dim(m,s)=dim1(m,s)
                  gwpdim(m,s)=gwpdim1(m,s)
                  dimred(m,s)=dimred1(m,s)
               enddo
            enddo
         endif

         call psidat

      endif

      return

 810  write(2,'(/a)') ' Check ERROR 4 in rdpsidef'
      write(2,'(a)') ' nmode,nstate,npacket,npackts'
      write(2,'(4i7)') nmode,nstate,npacket,npackts
      write(2,'(4i7)') nmode1,nstate1,npacket1,npackts1
      if(chkerr(6).eq.1) goto 820
      return

 820  write(2,'(/a)') ' Check ERROR 6 in rdpsidef'
      write(2,'(a)') '   s,  m,dim1(m,s),dim(m,s)'
      do s=1,nstate
         do m=1,max(nmode,nmode1)
             write(2,'(2i4,2i7)') s,m,dim1(m,s), dim(m,s)
         enddo
      enddo
      return

      end subroutine rdpsidef_pre11

!#######################################################################
!
! Sets up Psi data. For ML-WF, data taken from tree
!
!#######################################################################

      subroutine psidat

      implicit none

      integer(long)       :: m,s,t,zeig,e
      integer(long), dimension(3) :: ijk
      type(wfnode_t),pointer      :: node

!-----------------------------------------------------------------------
! set flags for psi type
!-----------------------------------------------------------------------
      if (psitype .eq. 1) lexact = .true.

      if (psitype .eq. 5 .and. .not. ml_input_ready) then
         routine='Psidat (iopsidef.F90)'
         message = 'ML tree has not been set up'
         call errormsg
      endif

!-----------------------------------------------------------------------
! make sure nstate is set to apparent no. of states for wavepacket
!-----------------------------------------------------------------------
      if (lmult .and. nstate .eq. gdim(feb)) then
         nstate = nstate*npacket
      endif

!-----------------------------------------------------------------------
! calculate dim,vdim and ndim for each mode.
! size of block (number of MCTDH coefficients)
!
! For dentype 1, save MCTDH data (dim,vdim,ndim,block,....) as well as the
! full pMCTDH(I) pointers (ddim,dvdim,dndim,dblock,....) as they can be 
! in single-set pMCTDH(I).used
!
! for trajectory calc. there are no coefficients
!-----------------------------------------------------------------------
      if (citype .eq. 1) then
         call jspacedat

      else if (citype .eq. 0) then
         if (psitype.eq.5) then
! ML-MCTDH
            block(1)=atopdim_wftree(tree)

! set up ijk for meb for top layer to allow manipulation of states
            do m=1,tree%nmodes               ! loop over branches of top layer
               node => tree%submodes(m)%p    ! node numder
               if (node%isprimitive .and. node%m .eq. meb) then  ! is node meb?
                  ijk=ijk_shape(m,tree%ndim)
                  vdim(meb,1)=ijk(1)
                  ndim(meb,1)=ijk(3)
               endif
            enddo

! set density 1 pointers for consistence
            if (dentype .eq. 1) then
               dblock(1,1)=block(1)
               if (meb .gt. 0) then
                  vddim(meb,1,1)=vdim(meb,1)
                  nddim(meb,1,1)=ndim(meb,1)
               endif
            endif

         else if (dentype .eq. 1) then
            do s=1,nstate
               do t=1,nstate
                  zeig=1
                  do m=1,nmode
                     vddim(m,s,t) = zeig
                     zeig=zeig*ddim(m,s,t)
                  enddo
                  do m=1,nmode
                     nddim(m,s,t)=zeig/(ddim(m,s,t)*vddim(m,s,t))
                  enddo
                  dblock(s,t)=zeig
               enddo
            enddo
            do m=1,nmode
               vdim(m,1) = vddim(m,1,1)
               ndim(m,1) = nddim(m,1,1)
            enddo
            block(1)=dblock(1,1)

         else
            do s=1,nstate
               zeig=1
               do m=1,nmode
                  vdim(m,s) = zeig
                  zeig=zeig*dim(m,s)
               enddo
               do m=1,nmode
                  ndim(m,s)=zeig/(dim(m,s)*vdim(m,s))
               enddo
               block(s)=zeig
            enddo
         endif
      endif

!-----------------------------------------------------------------------
! get maxblock
!-----------------------------------------------------------------------
      if (dentype .eq. 0 .or. dentype .eq. 2) then
         do s=1,nstate
            maxblock=max(maxblock,block(s))
         enddo
      else if (dentype .eq. 1) then
         do s=1,nstate
            do t=1,nstate
               maxblock=max(maxblock,dblock(s,t))
            enddo
         enddo
      endif
      call maxmatwk(0)

!-----------------------------------------------------------------------
! pointers and size for psi arrays psi,dtpsi,intpsi,hpsi
!  nb this is different from earlier versions !!!!!
!  zpsi(s) points to start of block(s)
!  dzpsi(s,t) points to start of block(s)*block(t)
!
! store zpsi(1) = dzpsi(1,1) for single set (no use of state index)
!-----------------------------------------------------------------------
      zeig=1

      if (dentype .eq. 0) then
         do s=1,nstate
            zpsi(s)=zeig
            zeig=zeig+block(s)
         enddo

      else if (dentype .eq. 1) then
         do s=1,nstate
            do t=1,nstate
               dzpsi(s,t)=zeig
               zeig=zeig+dblock(s,t)
            enddo
         enddo
         zpsi(1) = dzpsi(1,1)

      else if (dentype .eq. 2) then
         do t=1,nstate
            do s=1,nstate
               dzpsi(s,t)=zeig
               zeig=zeig+block(s)*block(t)
            enddo
         enddo
         zpsi(1) = dzpsi(1,1)

      endif
      adim = zeig-1

!-----------------------------------------------------------------------
! zetf(m,s) point to spfs for mode m in state s.
!-----------------------------------------------------------------------
      if (psitype .eq. 5) then
         call wftree_set_pointers(tree,dgldim)
         call setzetf(zetf,maxdim,maxsta)
         if (dentype .eq. 1) then
            do m=1,nmode
               dzetf(m,1,1) = zetf(m,1)
            enddo
         endif

      else if (dentype .eq. 1) then
         zeig=adim+1
         do m=1,nmode
            do s=1,nstate
               do t=1,nstate
                  dzetf(m,s,t)=zeig
                  zeig=zeig+subdim(m)*ddim(m,s,t)
               enddo
            enddo
! set pointer to start of spfs 
            zetf(m,1)=dzetf(m,1,1)
         enddo
         dgldim=zeig-1

      else
         do m=1,nmode
            if (idmode(m) .eq. 0) then
               do s=1,nstate
                  zetf(m,s)=zeig

                  if (gwpm(m)) then
                     if (psitype .eq. 0) zeig=zeig+dim(m,s)*dim(m,s)
                  else
                     zeig=zeig+subdim(m)*dim(m,s)
                  endif
               enddo
! space for GWPs at end of SPFs
               if (gwpm(m)) then
                  do s=1,nstate
                     zeig=zeig+subdim(m)*dim(m,s)
                  enddo
               endif

            else if(idmode(m).ne.0) then
               do s=1,nstate
                  zetf(m,s)=zetf(idmode(m),s)
               enddo  

            endif
         enddo
         dgldim=zeig-1
      endif

!-----------------------------------------------------------------------
! Pointers for GWP parameters
! NB. for ML-GMCTDH SPF parameters (D-matrices) are of size dim*dim 
! stored from zetf. GWP parameters are gwpdim*subdim stored from zphi
!-----------------------------------------------------------------------
      do m=1,nmode
         if (.not. gwpm(m)) cycle

         if (idmode(m) .eq. 0) then
! pointer to SPFs
            do s=1,nstate
               zdgwp(m,s) = zetf(m,s)
            enddo

            s = nstate
            zeig=zetf(m,s)
            if (psitype .eq. 0 .or. psitype .eq. 5) then
! move to after SPFs
               zeig = zeig+dim(m,s)*dim(m,s)
            endif

            do s=1,nstate

! space for phase (if propagated) 
               zphi(m,s) = zeig
               if (lgwpphase) zeig  = zeig + dim(m,s)

! space for linear coefficients
               zxi(m,s)  = zeig
               zeig  = zeig + dim(m,s)*nspfdof(m)

! space for width (if separable or thawed)
               if (gwptype .le. 2) then
                  zpg(m,s)  = zeig
                  zeig  = zeig + dim(m,s)*nspfdof(m)*nspfdof(m)
               else 
                  zpg(m,s)  = zetf(m,s)
               endif
            enddo

         else if(idmode(m).ne.0) then
            do s=1,nstate
               zphi(m,s) = zphi(idmode(m),s)
               zxi(m,s)  = zxi(idmode(m),s)
               zpg(m,s)  = zpg(idmode(m),s)
               zdgwp(m,s)  = zdgwp(idmode(m),s)
            enddo  

         endif

      enddo

!-----------------------------------------------------------------------
! for trajectories add electronic state phase at zphi(1,1) if nstate > 1
!-----------------------------------------------------------------------
      if (ltraj) then
          if (leb) then
             zphi(1,1) = zetf(2,1)
          else
             zphi(1,1) = zetf(1,1)
          endif
      endif

!-----------------------------------------------------------------------
! phidim(m) is length of spf vectors for mode m
! for gwp modes, space includes D-matrix. 
! for ML-MCTDH, space includes all layers of functions (i.e. not top)
!-----------------------------------------------------------------------
      phidim=0
      gbfdim=0
      totphidim=0
      maxphidim=0
      maxgbfdim=0
      if (psitype .eq. 5) then
         call setphidim(phidim,maxdim)
         totphidim=dgldim-adim

      else if (dentype .eq. 1) then
         do m=1,nmode
            phidim(m) = 0
            do s=1,nstate
               do t=1,nstate
                  do e=1,ddim(m,s,t)
                     phidim(m) = phidim(m)+subdim(m)
                  enddo
               enddo
            enddo
            totphidim=totphidim+phidim(m)
         enddo
      else
         do m=1,nmode
            phidim(m) = 0
            do s=1,nstate
               phidim(m) = phidim(m)+subdim(m)*dim(m,s)
               if (gwpm(m) .and. psitype .eq. 0) then
                  phidim(m)=phidim(m)+dim(m,s)*dim(m,s)
               endif
            enddo
            if(idmode(m).eq.0) totphidim=totphidim+phidim(m)
         enddo
      endif
      maxphidim = maxval(phidim)
      maxgbfdim = maxval(gbfdim)

!-----------------------------------------------------------------------
! pointers and size for density type matrices
! dicht1,dicht2,dicht4
!-----------------------------------------------------------------------
      zeig=1
      if (dentype .eq. 1) then
         do s=1,nstate
            do t=1,nstate
               do m=1,nmode
                  ddmat(m,s,t)=zeig
                  zeig=zeig+ddim(m,s,t)*ddim(m,s,t)
               enddo
            enddo
         enddo

         do m=1,nmode
            dmat(m,1)=ddmat(m,1,1)
         enddo
      else
         do s=1,nstate
            do m=1,nmode
               if(idmode(m).eq.0) then
                  dmat(m,s)=zeig
                  zeig=zeig+dim(m,s)*dim(m,s)
               else
                  dmat(m,s)=dmat(idmode(m),s)
               endif
            enddo
         enddo
      endif
      dmatdim=zeig-1

!-----------------------------------------------------------------------
! pointers and size for dicht3
!-----------------------------------------------------------------------
      zeig=1
      if (dentype .eq. 1) then
         do s=1,nstate
            do t=1,nstate
               do m=1,nmode
                  dd3mat(m,s,t)=zeig
                  zeig=zeig+ddim(m,s,t)*2
               enddo
            enddo
         enddo

         do m=1,nmode
            d3mat(m,1)=dd3mat(m,1,1)    ! Allow SS pMCTDH(I) same pointers as MCTDH
         enddo
      else
         do s=1,nstate
            do m=1,nmode
               if(idmode(m).eq.0) then
                  d3mat(m,s)=zeig
                  zeig=zeig+dim(m,s)*2
               else
                  d3mat(m,s)=d3mat(idmode(m),s)
               endif
            enddo
         enddo
      endif
      d3matdim=zeig-1

!-----------------------------------------------------------------------
! compute maximum number of single-particle functions
!-----------------------------------------------------------------------
      maxspf=0
      if (dentype .eq. 1) then
         do t=1,nstate
            do s=1,nstate
               do m=1,nmode
                  maxspf=max(maxspf,ddim(m,s,t))
               enddo
            enddo
         enddo
      else
         do s=1,nstate
            do m=1,nmode
               maxspf=max(maxspf,dim(m,s))
            enddo
         enddo

         do s=1,nstate
            do m=1,nmode
               if (gwpm(m)) then
                  maxspf=max(maxspf,gwpdim(m,s))
                  maxgwp=max(maxgwp,gwpdim(m,s))
               endif
            enddo
         enddo

! Get no. of GWP configurations
         totgp = 0
         if (lgwp) then
            do s=1,nstate
               zeig=1
               do m=1,nmode
                  if (.not. gwpm(m)) cycle
                  vdimgp(m,s) = zeig
                  zeig=zeig*gwpdim(m,s)
               enddo
               do m=1,nmode
                  if (.not. gwpm(m)) cycle
                  dimgp(m,s)=gwpdim(m,s)
               enddo
               do m=1,nmode
                  if (.not. gwpm(m)) cycle
                  ndimgp(m,s)=zeig/(dimgp(m,s)*vdimgp(m,s))
               enddo

               nsgp(s)=zeig
               totgp = totgp + zeig
            enddo
            zgp(1) = 1
            do s=2,nstate
               zgp(s) = zgp(s-1)+nsgp(s-1)
            enddo
         else if (ltraj) then
            s = 1
            m = 1
            vdimgp(m,s) = 1
            dimgp(m,s)=1
            ndimgp(m,s)=1
            nsgp(s)=1
            totgp = nsgp(s)
            zgp(s) = 1
         endif

         maxgwp=max(maxgwp,totgp)

      endif

!-----------------------------------------------------------------------
! size and pointers for GWP width matrix
! agmatdim100 needed to read files from version 100
! agmatdim101 needed to read files from version 101
!-----------------------------------------------------------------------
      zeig=1
      do s = 1,nstate
         do m = 1,nmode
            if (gwpm(m))then
               zagmat(m,s) = zeig
               zeig=zeig+gwpdim(m,s)*(nspfdof(m)**2)
            endif
         enddo
      enddo
      aGmatdim100  = max(zeig-1,1)
      do s = 1,nstate
         do m = 1,nmode
            if (gwpm(m))then
               zxivec(m,s) = zeig
               zeig=zeig+nspfdof(m)*gwpdim(m,s)
            endif
         enddo
      enddo
      aGmatdim101  = max(zeig-1,1)
      do s = 1,nstate
        do m = 1,nmode
          if (gwpm(m))then
            zphase(m,s) = zeig
            zeig=zeig+gwpdim(m,s)
          endif
        enddo
      enddo
      aGmatdim  = max(zeig-1,1)

!-----------------------------------------------------------------------
! size and pointers for GWP overlap matrices
!-----------------------------------------------------------------------
      zeig = 1 
      do s = 1,nstate  
         do m = 1,nmode  
            if (gwpm(m)) then 
                gsmat(m,s) = zeig  
                zeig = zeig+gwpdim(m,s)**2 
             endif 
         enddo 
      enddo  
      gsmatdim = max(zeig-1,1)
      
!-----------------------------------------------------------------------
! size and pointers for GWP center coordinate
!-----------------------------------------------------------------------
      zeig=1
      do s=1,nstate
         do m = 1,nmode
            if (gwpm(m))then
               zcent(m,s)=zeig
               zeig=zeig+gwpdim(m,s)*nspfdof(m)
            endif
         enddo
      enddo
      qcentdim = max(zeig-1,1)

!-----------------------------------------------------------------------
! pointers for GWP arrays
! zgwpinfo(m,s): pointers to arrays with info per function (e.g. trajst) 
!-----------------------------------------------------------------------
      zeig=1
      if (lgwp) then
         do s=1,nstate
            do m=1,nmode
               if (gwpm(m))then
                  zgwpinfo(m,s)=zeig
                  zeig=zeig+gwpdim(m,s)
               else
                  zgwpinfo(m,s)=1
               endif
            enddo
         enddo
      else if (psitype .eq. 3) then
         ltraj=.true.
         zgwpinfo(1,1)=zeig
         zeig=zeig+dim(1,1)
      endif
  
      gwpinfodim=max(zeig-1,1)

      return
      end subroutine psidat

!#######################################################################
!
! JSPACEDAT
!
! for selected-CI calculations, jspace is the total index space 
! required to describe the configurations.
! jdim = max(dim(m,s)) , s=1,nstate
! jvdim, jndim: equivalent to usual vdim and ndim, but using jdim
! jblock = jvdim*jdim*jndim
!
! block = no. selected coefficients
! vdim, ndim assigned in selected coefficient space to give 
!     (vdim,dim,ndim) partitioning.
!#######################################################################

      subroutine jspacedat

      implicit none

      integer(long)                          :: s,m,mindx,jvalue,zeig,v,e,n
      integer(long), dimension(maxdim)       :: j
      real(dop)                              :: weight
      real(dop), dimension(maxsta)           :: jsum

      call zerovxi(jdim,maxdim)
      do s=1,nstate
         block(s)=0
         do m=1,nmode
            jdim(m)=max(jdim(m),dim(m,s))
            vdim(m,s)=0
            ndim(m,s)=0
         enddo
      enddo

      zeig=1
      do m=1,nmode
         jvdim(m) = zeig
         zeig=zeig*jdim(m)
      enddo
      do m=1,nmode
         jndim(m)=zeig/(jdim(m)*jvdim(m))
      enddo
      jblock=zeig
      if (meb .ne. 0) ciweight(meb)=0.0_dop

!-----------------------------------------------------------------------
! loop over all jspace, and select coefficients
!-----------------------------------------------------------------------
      call zerovxi(j,maxdim)
      jvalue=0
   10 continue
      jvalue=jvalue+1
      do s=1,nstate
         jsum(s)=0.0_dop
      enddo
      do s=1,nstate
         do m=1,nmode
            if (j(m) .lt. dim(m,s)) then
               if (dim(m,s) .gt. 1) then
                  weight=dble(ciweight(m)/(dim(m,s)-1))
               else
                  weight=1.0
               endif
               jsum(s)=jsum(s)+weight*j(m)
            else
               jsum(s)=jsum(s)+2.0*cicut
            endif
         enddo
      enddo

      do s=1,nstate
         if (jsum(s) .le. cicut) then
            block(s)=block(s)+1

            do m=1,nmode
               call decompj(jvalue,v,e,n,jvdim(m),jdim(m))
               if (n .eq. 1 .and. e .eq. 1) then
                  vdim(m,s)=vdim(m,s)+1
               endif
               if (v .eq. 1 .and. e .eq. 1) then
                  ndim(m,s)=ndim(m,s)+1
               endif
            enddo
         endif
      enddo

      mindx=1
   20 continue
      if (j(mindx) .lt. (jdim(mindx)-1)) then
         j(mindx)=j(mindx)+1
         go to 10
      else if (mindx .lt. nmode) then
         j(mindx)=0
         mindx=mindx+1
         go to 20
      endif

      return
      end subroutine jspacedat

!#######################################################################
!
! DECOMPJ
!
! decomposes a configuration index, jindx = (v,e,n)
!
!#######################################################################

      subroutine decompj(jindx,v,e,n,vdim,dim)

      implicit none

      integer(long)              :: i
      integer(long), intent(in)  :: vdim,dim,jindx
      integer(long), intent(out) :: n,e,v

      i=jindx-1
      n=i/(vdim*dim)
      n=n+1

      i=i-(n-1)*vdim*dim
      e=i/vdim
      e=e+1

      i=i-(e-1)*vdim
      v=i+1

      return
      end subroutine decompj

!#######################################################################
!
! GWPDAT
!
! calculates information needed for GWP mode propagation
!
!#######################################################################

      subroutine gwpdat

      implicit none

      integer(long)       :: s,si,sf,m,p1,p2,zeig

      if (.not. lgwp) return

!-----------------------------------------------------------------------
! pointers for Gaussian arrays: 
!      gs2mat(m,s)   -     gs2matdim
!-----------------------------------------------------------------------
      zeig = 1 
      do s = 1,nstate  
         do m = 1,nmode 
            if (gwpm(m)) then   
               gs2mat(m,s) = zeig       
               do p1 = 1,npG(m)
                  do p2 = 1,npG(m) 
                     zeig = zeig+gwpdim(m,s)**2 
                  enddo 
               enddo  
            else
               gs2mat(m,s) = 1
            endif 
         enddo 
      enddo 
      gs2matdim = max(zeig-1,1)


      zeig=1
      do sf=1,nstate
        do si=1,nstate
          do m = 1,nmode
            if (gwpm(m)) then
               zgfgwp(m,sf,si) = zeig
               zeig=zeig+gwpdim(m,sf)*gwpdim(m,si)
            endif
          enddo
        enddo
      enddo
      gfgwpdim  = max(zeig-1,1)

      if (lBEL) then
         aigwpdim  = 1
         zaigwp(:,:,:) = 1
      else
         zeig=1
         do sf=1,nstate
           do si=1,nstate
             do m = 1,nmode
               if (gwpm(m)) then
                  zaigwp(m,sf,si) = zeig
                  zeig=zeig+nspfdof(m)*nspfdof(m) &
                    *gwpdim(m,sf)*gwpdim(m,si)
               endif
             enddo
           enddo
         enddo
         aigwpdim  = max(zeig-1,1)
      endif

      if (lBEL) then
         zgwpdim  = 1
         zzgwp(:,:,:) = 1
      else
         zeig=1
         do sf=1,nstate
           do si=1,nstate
             do m = 1,nmode
               if (gwpm(m)) then
                  zzgwp(m,sf,si) = zeig
                  zeig=zeig+nspfdof(m)*gwpdim(m,sf)*gwpdim(m,si)               
               endif
             enddo
           enddo
         enddo
         zgwpdim  = max(zeig-1,1)
      endif

      return
      end subroutine gwpdat

!#######################################################################

      function tapelength() result(l)

        implicit none

        integer(long) :: l,i

        l=0
        do i=1,maxtape
            if(tape(i)==-2) then
                l=i
                exit
            end if
        end do
        if(l==0) then
            write(6,*) 'tapelength: end of tape instruction not found'
            stop
        end if
      end function tapelength

!#######################################################################

      subroutine setzetf(zetf,maxdim,maxsta)
          implicit none
          integer,intent(inout)  :: zetf(maxdim,maxsta)
          integer,intent(in)     :: maxdim,maxsta
          type(wfnode_t),pointer :: tp
          integer                :: m,n
          zetf = 0
          do n = 1,nmode
              tp => tree%swap_phi(n)%p
              m = tp%m
              zetf(m,1) = tp%z(1,1)
          end do
      end subroutine setzetf

!#######################################################################

    subroutine setphidim(phidim,maxdim)
        implicit none
        integer,intent(inout)  :: phidim(maxdim)
        integer,intent(in)     :: maxdim
        type(wfnode_t),pointer :: tp
        integer                :: m,n
        phidim = 0
        do n=1,nmode
            tp => tree%swap_phi(n)%p
            m = tp%m
            phidim(m) = tp%zr(tp%navecs,2)
        end do
    end subroutine setphidim


      end module iopsidef

