!#######################################################################
! contains logmem,errormsg,warnmsg,warnmsg1,wrline,cntrl
!#######################################################################

      module logdat

use maxv, only: maxdim,maxsta
use decimal, only: long, dop
use versions, only: progname
use global
use globmemmod
use channels, only: ilog
use dvrdatmod
use griddatmod
use psidef
use operdef

      implicit none

!-----------------------------------------------------------------------
! CHARACTER STRINGS TO PASS MESSAGES TO ERROR ROUTINE
!-----------------------------------------------------------------------
      character(len=c3), save :: routine=' '
      character(len=c5), save :: message=' ', inptit=' '

      private
      public :: logmem,errormsg,warnmsg,warnmsg1,wrline,cntrl,stopnow
      public :: routine,message,inptit

      contains

!#######################################################################

      subroutine logmem(progpart)

      implicit none

      real(dop), parameter          :: megabyte = 1024_dop**2
      integer(long), external       :: complexsize,realsize,integersize,&
                                       logicalsize
      real(dop)                     :: dynmem,usedmemory

      character(len=*), intent(in)   :: progpart
      character(len=80)              :: string 

!-----------------------------------------------------------------------
! ESTIMATE REQUIRED MEMORY SIZE 
!-----------------------------------------------------------------------
      if (progname(1:8) .ne. 'quantics') ikrylov=0
      if (progname(1:8) .ne. 'quantics') ikryc=0

! dynmem is for dynamically allocated memory during runtime. Not yet used.
      dynmem = 0

      write(ilog,'(/,a)') &
      '------------------------------------------------------------------------'

      if (progpart .eq. 'rundvr') then
         string='***** Memory used in DVR basis generation *****'
      else if (progpart .eq. 'runoper') then
         string='***** Memory used in OPERATOR generation *****'
      else if (progpart .eq. 'runinwf') then
         string='***** Memory used in WAVEFUNCTION generation *****'
      else if (progpart .eq. 'runprop') then
         if (.not. lrundiag) then
            string='***** Memory used in wavefunction PROPAGATION *****'
         else
            string= &
               '***** Memory used in Hamiltonian DIAGONALISATION *****'
         endif
      else if (progpart .eq. 'runpfit') then
         string='***** Memory used in PES FITTING  *****'
      else 
         string='***** Memory used in '//progpart//'  *****'
      endif

      write(ilog,*)
      write(ilog,'(a80)') string

!-----------------------------------------------------------------------
! WRITE MAX DIMENSIONS and total memory allocated
!-----------------------------------------------------------------------
      write(ilog,'(a,i5)') ' Maximum no. of dimension (MAXDIM) :', maxdim
      write(ilog,'(a,i5)') ' Maximum no. of states (MAXSTA)    :', maxsta
      if (maxpack .gt. 1) &
         write(ilog,'(a,i5)') ' Maximum no. of packets (MAXPACK)    :', maxpack
      if (maxkoe .gt. 1) write(ilog,'(a,i5)') &
           ' Maximum no. of Hamiltonian coefficients (MAXKOE)    :', maxkoe
      if (maxham .gt. 1) write(ilog,'(a,i5)') &
           ' Maximum no. of Hamiltonians (MAXHAM)    :', maxham

!-----------------------------------------------------------------------
! WRITE AND CHECK DIMENSIONS OF MEMORY ARRAYS 
!-----------------------------------------------------------------------
      write(ilog,*)
      write(ilog,'(a)') '** Array dimensions **'
      write(ilog,*)


      usedmemory = 0
      if(dgldim.gt.1) write(ilog,'(a,i11,f15.3,a)') &
         'length of psi/dtpsi    array (used): ', dgldim, &
          dgldim*complexsize()/megabyte,' MB'
      if(mcdim.gt.1) write(ilog,'(a,i11,f15.3,a)') &
         'length of complex      array (used): ', mcdim, &
          mcdim*complexsize()/megabyte,' MB'
      if(mrdim.gt.1) write(ilog,'(a,i11,f15.3,a)') &
         'length of real         array (used): ', mrdim, &
          mrdim*realsize()/megabyte,' MB'
      if(midim.gt.1) write(ilog,'(a,i11,f15.3,a)') &
         'length of integer      array (used): ', midim, &
          midim*integersize()/megabyte,' MB'
      if(mldim.gt.1) write(ilog,'(a,i11,f15.3,a)') &
         'length of logical      array (used): ', mldim, &
          mldim*0.5*logicalsize()/megabyte,' MB'
      if(msdim.gt.1) write(ilog,'(a,i11,f15.3,a)') &
         'length of complex (sp) array (used): ', msdim, &
          msdim*0.5*complexsize()/megabyte,' MB'
      if(mfdim.gt.1) write(ilog,'(a,i11,f15.3,a)') &
         'length of real    (sp) array (used): ', mfdim, &
          mfdim*0.5*realsize()/megabyte,' MB'
      if(ikrylov.gt.8) write(ilog,'(a,i11,f15.3,a,/a,i11,f15.3,a)') &
         'length of real  Krylov array (used): ', ikrylov, &
          ikrylov*realsize()/megabyte,' MB', &
         'length of real HKrylov array (used): ', ikrylov, &
          ikrylov*realsize()/megabyte,' MB'
      if(ikryc.gt.8) write(ilog,'(a,i11,f15.3,a,/a,i11,f15.3,a)') &
         'length of cmplx Krylov array (used): ', ikryc, &
          ikryc*complexsize()/megabyte,' MB', &
         'length of cmplx HKrylov arr. (used): ', ikryc, &
          ikryc*complexsize()/megabyte,' MB'

      usedmemory=dble(mcdim)*complexsize()+dble(mrdim)*realsize() &
            +dble(midim)*integersize()+dble(mldim)*logicalsize() &
            +0.5_dop*dble(msdim)*complexsize() &
            +0.5_dop*dble(mfdim)*realsize() & 
            +2.0_dop*dble(ikrylov)*realsize() &
            +2.0_dop*dble(ikryc)*complexsize() &
            +2.0_dop*dble(dgldim)*complexsize()

      write(ilog,'(a)') ' '
      write(ilog,'(a,F10.3,a,F10.3)') &
           'Memory size (core / data) (in Mb): ', &
           Usedmemory/megabyte, ' / ', allocmemory/megabyte
      write(ilog,'(a)') ' '
      write(ilog,'(a,f11.1)') &
         'Dynamically allocated memory in mb    (Max): ', &
           dynmem/megabyte



      write(ilog,'(/,a)') &
      '------------------------------------------------------------------------'

      call myflush(ilog)

      return
      end subroutine logmem

!#######################################################################
! *** V5.0 MB ********> SUBROUTINE ERROR MESSAGE <**********************
!                      ~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! Writes an error message to the log file and to the screen.
! The calculation is then terminated.
!
!#######################################################################

      Subroutine errormsg

      Implicit None

      integer(long) :: lun

! --- WRITE ERROR MESSAGE AND CLOSE FILES---

      if (logisopen) then
         lun=ilog
      else
         lun=0
      endif
      call croak(lun,.true.,"ERROR",routine,message)

      if( .not.test0 ) then
         call stopnow
      endif

      Return
      End subroutine errormsg

!#######################################################################
!
! Terminate the program. Do it properly if MPI is used.
!#######################################################################

      subroutine stopnow

#ifdef MPI
use mpi
use mpidata

      implicit none

      logical(kind=4) :: lmpiinit
      
      call mpi_initialized(lmpiinit,mpierr)

      if (lmpiinit) then
! tell slaves to stop 
         mpitask=-1
         call MPI_Bcast(mpitask,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)

! finalise master
         call MPI_Finalize(mpierr)
      endif

#endif

      stop
      end subroutine stopnow

!#######################################################################
! *** V6.3 GW ********> SUBROUTINE WARNING MESSAGE <********************
!                       ~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! Writes an error message to the log file and to the screen, but does
! not stop the calculation. Cf subroutine errormsg
!
! The logical lwarn is set.
!
!#######################################################################

      subroutine warnmsg

      implicit None

      integer(long) :: lun
      lwarn = .true.

! --- WRITE WARNING MESSAGE TO LOG FILE AND SCREEN ---

      if (logisopen) then
         lun=ilog
      else
         lun=0
      endif
      call croak(lun,.true.,"WARNING",routine,message)

      return
      end subroutine warnmsg

!#######################################################################
! *** V6.3 GW ********> SUBROUTINE WARNING MESSAGE 1 <******************
!                       ~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! Writes a warning message to the log file.
! Used for non-crucial (advisory) warnings.
!
!#######################################################################

      subroutine warnmsg1

      implicit None

! --- WRITE WARNING MESSAGE TO LOG FILE ---

      if (logisopen) then
         call croak(ilog,.false.,"WARNING",routine,message)
         write(ilog,*) ' '
      endif

      return
      end subroutine warnmsg1

!=======================================================================
      subroutine croak(lun,lwr6,what,rout,msg)
!=======================================================================
!
! Common code used by errormsg, warnmsg, warnmsg1.
! -- FO 10/2013
!
!=======================================================================
      implicit none
      integer          :: lun
      character(len=*) :: what,rout,msg
      logical(kind=4)  :: lwr6

      if(lun.gt.0) then
         write(lun,*)
         write(lun,'(a)') trim(what)//' in subroutine '//trim(rout)//':'
      endif
      if(lwr6) then
         write(6,*)
         write(6  ,'(a)') trim(what)//' in subroutine '//trim(rout)//':'
      endif
      call formatmsg(lun,lwr6,msg)
      if(lun.gt.0)write(lun,'(a)')'------------------------------------'
      if(lwr6) write(6,'(a)') '----------------------------------------'

      return
      end subroutine croak

!***********************************************************************
!
!                 MCTDH   subroutine  WRLINE
!
! Writes some additional information to screen and to the 'log' file.
! Helps finding wrong input parameters in the ~.inp file.
! 
! i  : no. of keyword associated with error message (input)
! iz : no. of line in input file containing error   (input)
! iz0: no. of last error containing line in input file (prevents 
!      repeated messages) (input/output)
!
!   11/97 AJ
!***********************************************************************

      subroutine wrline (i,iz,iz0)

      implicit none

      integer(long), intent(in)    :: i,iz
      integer(long), intent(inout) :: iz0
      integer(long)                :: ilbl

      if (iz.ne.iz0) write(6,900) iz, inptit
      iz0     = iz
      if (i .gt. 0) then
         ilbl=index(message,':')+1
         message=message(1:ilbl)//keyorig(i)
      endif
      call warnmsg

 900  format(/,' Input Error in line no.',i4,' The wrong line reads :', &
             /,  a80 )

      return
      end subroutine wrline

!***********************************************************************
!
!            Subroutine   CNTRL
!
! Contolls whether the input read via rdinpf is correct.
! Calls ERRORMSG.
! 06/97 HDM
! 
!***********************************************************************

      subroutine cntrl(keyword,lc,ierr)

      Implicit None

      Character(len=60), intent(in) :: keyword
      Integer(long), intent(in)     :: lc, ierr
      integer(long)                 :: lc1

      lc1 = lc ! cp to prevent unused variable warning

      if (ierr .eq. 1) then
         write(6,'(a)') 'rdinpf returned ierr=1'
         call errormsg
      endif

      if (keyword .eq. 'end-input') then 
          write(6,'(a)') '"END-INPUT" found !'
          write(6,'(a)') 'An "end-..." may be missing (or misspelled)!'
          call errormsg
      endif

      if (keyword .eq. 'end-operator') then
          write(6,'(a)') '"END-OPERATOR" found !'
          write(6,'(a)') &
    'An "end-..." may be missing (or misspelled). Or a compulsory keyword, ', &
    'e.g. modes, missing !'
          call errormsg
          stop
      endif

      return
      end subroutine cntrl

      end module logdat

