!***********************************************************************
! 
!             MCTDH subroutine MEMDIMMOD
!
! contains memdiminp, memdimpack
!
!***********************************************************************

      module memdimmod

use decimal, only: long, dop
use op1lib,  only: zerovxl

use maxv
use global
use dvrdatmod
use channels, only: irst
use dirdyn, only: nbasis

use logdat

      implicit none

      private
      public :: memdiminp, memdimpack
      contains
  
!#######################################################################

      subroutine memdiminp(iin)

      implicit none

      integer(long), intent(in) :: iin
      integer(long)   :: ic,iz,ierr,i,itmp
      logical         :: lddnm,lpbas

!-----------------------------------------------------------------------
! get maxdim and maxsta
! maxdim must be .ge. ndof   
! maxsta must be .ge. nstate
! lddnm = .true. set if direct dynamics and nmodes and pbasis-section.
!         In this case need to add 6 to maxdim to allows space for 
!         cartesians
!-----------------------------------------------------------------------
      rewind(iin)

      iz=0
      lddnm=.false.
      lpbas=.false.
      getmaxdim: do

         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .gt. 0) then
            routine = 'Eingabe, primitive-basis-section'
            message = inptit
            call errormsg
         endif

!-----------------------------------------------------------------------
! Find primitive basis section
!-----------------------------------------------------------------------
         if (keyword(1) .eq. 'primitive-basis-section' .or. &
              keyword(1) .eq. 'pbasis-section') then
            maxdim=0
            maxsta=0

            lpbas=.true.
            readpbas: do
               call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
               message = inptit
               call cntrl(keyword(1),lc(1),ierr)

! finish reading
               if (keyword(1) .eq. 'end-primitive-basis-section' .or. &
                   keyword(1) .eq. 'end-pbasis-section' ) then
                  exit
               else if (keyword(1) .eq. 'end-input') then
                  message = 'No end-pbasis-section '
                  call errormsg

! if electronic basis specified get no. of states - dof added at end
               else if (keyword(2) .eq. 'el' .or. &
               !         keyword(2) .eq. 'elgrid' .or. &
                         keyword(2) .eq. 'elcont') then
                  read(keyword(3),*,iostat=ierr) itmp
                  maxsta = max(maxsta,itmp)
                  if(.not.lddmctdh) maxdim=maxdim+1

! sum up DOFS
               else
                  if(.not.lddmctdh) maxdim=maxdim+1

               endif
            enddo readpbas

            if(.not.lddmctdh)exit

!-----------------------------------------------------------------------
! Or find initial geometry section (vMCG runs)
!-----------------------------------------------------------------------
         else if (keyword(1) .eq. 'initial-geometry-section') then
            maxdim=0
            maxsta=0

            readingeo: do
               call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
               message = inptit
               call cntrl(keyword(1),lc(1),ierr)

               if (keyword(1) .eq. 'end-initial-geometry-section') then
                  exit
               else if (keyword(1) .eq. 'end-input') then
                  message = 'No end-initial-geometry-section '
                  call errormsg
               endif

! read cartesian definition to get ndofs
               if (keyword(1) .eq. 'cartesian') then
                  readcart: do
                     call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
                     message = inptit
                     call cntrl(keyword(1),lc(1),ierr)

                     if (keyword(1) .eq. 'end-cartesian') then
                        exit

                     else if (keyword(1) .eq. 'end-input') then
                        message = 'Missing end-cartesian'
                        call errormsg

                     else
                        maxdim = maxdim+3

                     endif

                  enddo readcart

! look for no. states
               else
                  i=0
                  readkey: do
                     i=i+1
                     if (i .gt. ic) exit

                     if (keyword(i) .eq.'nstates') then
                        read(keyword(i+2),*,iostat=ierr) maxsta
                        if (ierr .eq. 1) then
                          message = &
                           'Incorrect init_geom definition for electronic states :'
                          call errormsg
                       endif
                       i=i+2
                     endif
                  enddo readkey
               endif

            enddo readingeo
            exit

!-----------------------------------------------------------------------
! Or find initial geometry section (DD-GB runs)
!-----------------------------------------------------------------------
         else if (keyword(1) .eq. 'dd-gb-section') then
            maxdim=0

            readddmct: do
               call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
               message = inptit
               call cntrl(keyword(1),lc(1),ierr)

               if (keyword(1) .eq. 'end-dd-gb-section') then
                  exit
               else if (keyword(1) .eq. 'end-input') then
                  message = 'No end-dd-gb-section '
                  call errormsg
               endif

! read cartesian definition to get ndofs
               if (keyword(1) .eq. 'cartesian') then
                  readcartmct: do
                     call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
                     message = inptit
                     call cntrl(keyword(1),lc(1),ierr)

                     if (keyword(1) .eq. 'end-cartesian') then
                        exit

                     else if (keyword(1) .eq. 'end-input') then
                        message = 'Missing end-cartesian'
                        call errormsg

                     else
                        maxdim = maxdim+3

                     endif

                  enddo readcartmct

               endif

            enddo readddmct
            if(lpbas)exit

!-----------------------------------------------------------------------
! Or look for direct keyword. This is needed if DD calculation using
! nmodes is specified via primitive-basis-section to convert maxdim 
! from nmodes to ncoo
!-----------------------------------------------------------------------
         else if (keyword(1) .eq. 'run-section') then

            readrun: do
               call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
               message = inptit
               call cntrl(keyword(1),lc(1),ierr)

               i=0
               readkey1: do
                  i=i+1
                  if (i .gt. ic) exit

                  if (keyword(i) .eq.'direct' .or. keyword(i).eq.'dd-gb') then
                     if (keyword(i+1) .eq. '=' .and. &
                             keyword(i+2) .eq. 'nmodes') then
                        lddnm = .true.
                     endif

                  else if(keyword(i) .eq. 'ngwp')then
                     if (keyword(i+1).ne.'=') then
                        message = 'No argument specified with ngwp keyword'
                        call errormsg
                     else
                        read(keyword(i+2),*,iostat=ierr) maxgwp
                     endif

                  else if (keyword(1) .eq. 'end-input') then
                     message = 'No end-run-section '
                     call errormsg
                  endif
               enddo readkey1

               if (keyword(ic) .eq. 'end-run-section') exit

            enddo readrun

         else if (keyword(1) .eq. 'end-input') then
            exit
         endif

      enddo getmaxdim

! add space for cartesians in nmode DD calculation
      if (lpbas .and. lddnm) maxdim=maxdim+6
! add space for electronic DOF
      maxdim = maxdim+1
! space for koeffs, time, energy, ml-mctdh "nodes"
      maxdim = maxdim+4
! if no. states not specified, single state assumed
      if (maxsta .eq. 0)  maxsta = 1

      return
      end subroutine memdiminp

!#######################################################################

      subroutine memdimpack(iin)

      implicit none

      integer(long), intent(in) :: iin
      integer(long)   :: ic,iz,ierr,i
      integer(long)   :: npacket

!-----------------------------------------------------------------------
!
!-----------------------------------------------------------------------
      npacket=1
      getnpack: do

         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .gt. 0) then
            routine = 'Eingabe, spf-basis-section'
            message = inptit
            call errormsg
         endif

         if (keyword(1) .eq. 'end-input') then
            exit

!-----------------------------------------------------------------------
! Find SPF-BASIS section
!-----------------------------------------------------------------------
         else if (keyword(1) .eq. 'spf-basis-section' .or. &
              keyword(1) .eq. 'sbasis-section') then

            rdlines: do 
               call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
               message = inptit
               call cntrl(keyword(1),lc(1),ierr)

               if (keyword(1) .eq. 'end-sbasis-section' .or. &
                   keyword(1) .eq. 'end-spf-basis-section') exit
              
               do i=1,ic
                  if (keyword(i).eq.'packet' .or. keyword(i).eq.'packets') then

                     if (keyword(i+1) .ne. '=') then
                        routine = "memdimpack"
                        message = 'No arguments specified with "packets" keyword'
                        call errormsg
                     else
                        read(keyword(i+2),*,iostat=ierr) npacket
                     endif
                     if (keyword(i+3).eq.',') then
                        if (keyword(i+4) .eq. 'single-set' ) npacket=1
                     endif

                     exit
                  endif
               enddo
            enddo rdlines

         endif
      enddo getnpack

      maxpack=max(maxpack,npacket)
! add space for packets
      maxkoe = maxkoe*maxpack
      maxsta = maxsta*maxpack

      return
      end subroutine memdimpack

!#######################################################################

      end module memdimmod


      

