
!#######################################################################
! called from analyse routines
!#######################################################################

      module operutil

use decimal, only: long, dop
use mtlib,   only: qtxxzz,dtxxzz
use xvlib,   only: vvaxzz
use op1lib,  only: zerovxd, zeromxz, cpvxz
use global
use logdat
use operdef
use maxv, only:maxdim,maxsta,minkoe
use dvrdatmod
use griddatmod
use psidef
use op1lib, only: zeromxd,cpmxd,sumvxz
use psiutil, only: decoderm

      implicit none
      private
      public :: calcd2d,vshiftk3,wshiftk3,getvgn,vcalc1st,vcalcmlst,&
                findnath,findcpdh,findhcpdh,findread,rdfactors,mkhpar,&
                getmaxeig,mlnp_parse,findreadmap,mvkend

      contains

!#######################################################################
!        
! MVKEND    
!        
! moves the k th term in the Hamiltonian to the end of the list.
! This allows sorting (e.g. removal or expansion) of terms.
!             
!#######################################################################
            
      subroutine mvkend(k)
         
      implicit none
         
      integer(long)             :: k1,k2,f,m
      integer(long), intent(in) :: k
            
      routine='Mvkend'
            
      if (ktot+1 .gt. maxkoe) then
         minkoe=minkoe+1
         return
      endif

      do f=1,nopdof
         hamilton(f,ktot+1)=hamilton(f,k)
      enddo
      kact(ktot+1)  = kact(k)
      if (ktwin(k) .ne. k) then
         k2 = ktwin(k)
         ktwin(k2) = ktot + 1
         ktwin(ktot+1) = k2
      else
         ktwin(ktot+1) = ktot + 1
      endif
      do m=1,nopmode
         diag(m,ktot+1)=diag(m,k)
         kinop(m,ktot+1)=kinop(m,k)
         hsym(m,ktot+1)=hsym(m,k)
      enddo
      ki(ktot+1)=ki(k)
      kf(ktot+1)=kf(k)
      ksi(ktot+1)=ksi(k)
      ksf(ktot+1)=ksf(k)
      hamnum(ktot+1)=hamnum(k)
      fterm(ktot+1)=fterm(k)
      hmlop(ktot+1)=hmlop(k)
      klha(ktot+1)=klha(k)
      autocap(ktot+1)=autocap(k)
      nstatecap(ktot+1)=nstatecap(k)
      autoabs(ktot+1)=autoabs(k)
      autoflux(ktot+1)=autoflux(k)

      do k1=k,ktot
         do f=1,nopdof
            hamilton(f,k1)=hamilton(f,k1+1)
         enddo
         kact(k1)  = kact(k1+1)
         if (((dentype.eq.2).and.(kact(k1+1).ne.1)) &
              .or.((dentype.eq.1).and.(ktwin(k1+1).ne.(k1+1)))) then
            k2 = ktwin(k1+1)
            ktwin(k2) = k1
            ktwin(k1) = k2
         else
            ktwin(k1) = k1
         endif
         do m=1,nopmode
            diag(m,k1)=diag(m,k1+1)
            kinop(m,k1)=kinop(m,k1+1)
            hsym(m,k1)=hsym(m,k1+1)
            hlsym(m,k1)=hlsym(m,k1+1)
         enddo
         ki(k1)=ki(k1+1)
         kf(k1)=kf(k1+1)
         ksi(k1)=ksi(k1+1)
         ksf(k1)=ksf(k1+1)
         hamnum(k1)=hamnum(k1+1)
         fterm(k1)=fterm(k1+1)
         hmlop(k1)=hmlop(k1+1)
         klha(k1)=klha(k1+1)
         autocap(k1)=autocap(k1+1)
         nstatecap(k1)=nstatecap(k1+1)
         autoabs(k1)=autoabs(k1+1)
         autoflux(k1)=autoflux(k1+1)
      enddo


      return
      end subroutine mvkend

!#######################################################################

      subroutine calcd2d(psi,psidia,f1,f2,igpoint,gpoint, &
                 agmat,gs2,trbin,trajst,weight)

      implicit none

      integer(long)                                :: ngdim1,m1,m2,s,f,zeig1,&
                                                      zeig2,zeig3,m,n,n1,n2,s1
      integer(long), intent(in)                    :: f1,f2
      integer(long), dimension(gwpinfodim), intent(in) :: trajst
      integer(long), dimension(maxdim), intent(in) :: igpoint
      real(dop), dimension(2), intent(in)          :: trbin
      real(dop), dimension(maxsta), intent(out)    :: psidia
      real(dop), dimension(maxdim), intent(in)     :: gpoint
      real(dop), dimension(ortdim), intent(in)     :: weight

      complex(dop)                                          :: sum2 
      complex(dop), dimension(dgldim), intent(in)           ::  psi
      complex(dop), dimension(agmatdim), intent(in)         ::  agmat
      complex(dop), dimension(gs2matdim), intent(in)        ::  gs2
      complex(dop), dimension(2*maxblock)                   ::  work
      complex(dop), dimension(maxspf*maxspf,2)              ::  spfpnt

      call zerovxd(psidia,maxsta)
      call zeromxz(spfpnt,maxspf*maxspf,2)
      n1=0
      n2=0
      f = 0
!-----------------------------------------------------------------------
! see 1D cut, or if f1, f2 are in same mode
!-----------------------------------------------------------------------
      m1=dofspf(f1)
      if (f2 .eq. 0) then
         m2=0
! n1 indicates which dof f1 is in particle m1
         do n=1,nspfdof(m1)
            if (spfdof(n,m1) .eq. f1) then
               n1=n
            endif
         enddo

      else
         m2=dofspf(f2)
! f indicates whether f1 or f2 is first in mode
         if ((m1 .eq. m2) .or. (m2 .eq. 0)) then
            f=0
            do n=1,nspfdof(m1)
               if (f .eq. 0) then
                  if (spfdof(n,m1) .eq. f1) then
                     f=f1
                  else if (spfdof(n,m1) .eq. f2) then
                     f=f2
                  endif
               endif
            enddo
! n1,n2 indicate which dofs f1,f2 are in particle m1
            do n=1,nspfdof(m1)
               if (spfdof(n,m1) .eq. f1) then
                  n1=n
               else if (spfdof(n,m1) .eq. f2) then
                  n2=n
               endif
            enddo
         endif
      endif

!-----------------------------------------------------------------------
! get trajectory "density
!-----------------------------------------------------------------------
      if (ltraj) then
         s = 1
         call trden2d(psi(zetf(m1,s)),psidia(s),nspfdof(m1), &
              dim(m1,s),gpoint(f1),gpoint(f2),n1,n2,trbin,trajst, &
              nddstate)
         return
      endif

      if (dentype .eq. 0) then
!-----------------------------------------------------------------------
! get spf "density"
!-----------------------------------------------------------------------
         do s=1,nstate
            if (m2 .ne. m1) then
! both DOF in different modes (or only 1 DOF if m2=0)
               if (gwpm(m1) .and. gwptype .ge. 3) then 
                  if (f2 .eq. 0) then
                     call gwpden2d(m1,agmat(zagmat(m1,s)),agmat(zxivec(m1,s)), &
                          agmat(zphase(m1,s)),spfpnt(1,1),nspfdof(m1),dim(m1,s), & 
                          gpoint(f1),gpoint(f2),n1,n2,f1,f2,weight)
                  else
                     print*,'For GWP plotting dofs must be in same particle'
                     return
                  end if
               else if (gwpm(m1)) then 
                  print*,'Cannot plot density of thawed GWPs'
                  return
               else if (gwpm(m2)) then 
                  print*,'Plotting cannot mix GWP / SPF densities'
                  return
               else
                  call spfden2d(psi(zetf(m1,s)),spfpnt(1,1),vgdim(f1), &
                       gdim(f1),ngdim(f1),1,1,igpoint(f1),1,dim(m1,s))

                  if (m2 .ne. 0) then
                     call spfden2d(psi(zetf(m2,s)),spfpnt(1,2),vgdim(f2), &
                          gdim(f2),ngdim(f2),1,1,igpoint(f2),1,dim(m2,s))
                  endif
               endif
            else if (m2 .eq. m1) then
! both DOF in same mode
               if (gwpm(m1) .and. gwptype .ge. 3) then
                  call gwpden2d(m1,agmat(zagmat(m1,s)),agmat(zxivec(m1,s)), &
                       agmat(zphase(m1,s)),spfpnt(1,1),nspfdof(m1),dim(m1,s), & 
                       gpoint(f1),gpoint(f2),n1,n2,f1,f2,weight)
               else if (gwpm(m1)) then
                  print*,'Cannot plot density of thawed GWPs'
                  return
               else if (f .eq. f1) then
                  ngdim1=ngdim(f1)/(gdim(f2)*ngdim(f2))
                  call spfden2d(psi(zetf(m1,s)),spfpnt(1,1),vgdim(f1), &
                       gdim(f1),ngdim1,gdim(f2),ngdim(f2),igpoint(f1), &
                       igpoint(f2),dim(m2,s))
               else if (f .eq. f2) then
                  ngdim1=ngdim(f2)/(gdim(f1)*ngdim(f1))
                  call spfden2d(psi(zetf(m2,s)),spfpnt(1,1),vgdim(f2), &
                       gdim(f2),ngdim1,gdim(f1),ngdim(f1),igpoint(f2), &
                       igpoint(f1),dim(m2,s))
               endif
            endif

!-----------------------------------------------------------------------
!    multiply A vector by spfs at chosen point
!-----------------------------------------------------------------------
            zeig1 = block(s)+1
            zeig2 = 1
            call cpvxz(psi(zpsi(s)),work(zeig1),block(s))
            if (lgwp) then
               do m=1,nmode
                  if ((m .ne. m1) .and. gwpm(m))then
                     call qtxxzz(gs2(gs2mat(m,s)), &
                             work(zeig1),work(zeig2), &
                             vdim(m,s),dim(m,s),ndim(m,s))
                     zeig3=zeig2
                     zeig2=zeig1
                     zeig1=zeig3
                  endif
               enddo
            endif

            call qtxxzz(spfpnt(1,1),work(zeig1),work(zeig2), &
                 vdim(m1,s),dim(m1,s),ndim(m1,s))
            if (m2 .ne. 0 .and. m1 .ne. m2) then
               zeig3=zeig2
               zeig2=zeig1
               zeig1=zeig3
               call qtxxzz(spfpnt(1,2),work(zeig1),work(zeig2), &
                    vdim(m2,s),dim(m2,s),ndim(m2,s))
            endif

            if (lmult .or. nddstate .eq. 1) then
               call vvaxzz(psi(zpsi(s)),work(zeig2),sum2,block(s))
                  
               if (dble(sum2) .lt. 1.0e-12_dop) sum2=0.0_dop
               psidia(s)=dble(sum2)
            else 
               do s1=1,nddstate
                  call cpvxz(work(zeig2),work(zeig1),block(s))
                  m=meb
                  call ssetproj(s1,nddstate,psi(zetf(m,s)),work(zeig1), &
                       vdim(m,s),dim(m,s),ndim(m,s))

                  call vvaxzz(psi(zpsi(s)),work(zeig1),sum2,block(s))
                  
                  if (dble(sum2) .lt. 1.0e-12_dop) sum2=0.0_dop
                  psidia(s1)=dble(sum2)
               enddo
            endif
         enddo

      else if (dentype .eq. 1) then
!-----------------------------------------------------------------------
! get type I "density"
!-----------------------------------------------------------------------
         do s=1,nstate
            if (m2 .ne. m1) then
! both DOF in different modes (or only 1 DOF if m2=0)
               call rho1den2d(psi(dzetf(m1,s,s)),spfpnt(1,1),supgdim(m1), &
                    m1,f1,0,igpoint(f1),0,ddim(m1,s,s))

               if (m2 .ne. 0) then
                  call rho1den2d(psi(dzetf(m2,s,s)),spfpnt(1,2),supgdim(m2), &
                       m2,f2,0,igpoint(f2),0,ddim(m2,s,s))
               endif
            else if (m2 .eq. m1) then
! both DOF in same mode
               call rho1den2d(psi(dzetf(m1,s,s)),spfpnt(1,1),supgdim(m1), &
                    m1,f1,f2,igpoint(f1),igpoint(f2),ddim(m1,s,s))
            endif

!-----------------------------------------------------------------------
!    multiply A vector by density at chosen point and sum
!-----------------------------------------------------------------------
            zeig1 = block(s)+1
            zeig2 = 1
            call cpvxz(psi(dzpsi(s,s)),work(zeig1),dblock(s,s))

            call dtxxzz(spfpnt(1,1),work(zeig1),work(zeig2), &
                 vddim(m1,s,s),ddim(m1,s,s),nddim(m1,s,s))
            if (m2 .ne. 0 .and. m1 .ne. m2) then
               zeig3=zeig2
               zeig2=zeig1
               zeig1=zeig3
               call dtxxzz(spfpnt(1,2),work(zeig1),work(zeig2), &
                    vddim(m2,s,s),ddim(m2,s,s),nddim(m2,s,s))
            endif

            if (lmult .or. nddstate .eq. 1) then
               call sumvxz(work(zeig2),sum2,dblock(s,s))
                  
               if (dble(sum2) .lt. 1.0e-12_dop) sum2=0.0_dop
               psidia(s)=dble(sum2)
            else 
               print*,'Single Set Density I Not Working'
               stop
               do s1=1,nddstate
                  call cpvxz(work(zeig2),work(zeig1),dblock(s,s))
                  m=meb
                  call ssetproj1(s1,nddstate,psi(dzetf(m,s,s)),work(zeig1), &
                       vddim(m,s,s),ddim(m,s,s),nddim(m,s,s))

                  call sumvxz(work(zeig1),sum2,dblock(s,s))
                  
                  if (dble(sum2) .lt. 1.0e-12_dop) sum2=0.0_dop
                  psidia(s1)=dble(sum2)
               enddo
            endif
         enddo
      endif

      return
      end subroutine calcd2d

!#######################################################################
!
! for a single-set calculation we need to find which coefficients
! relate to which state. The SPFs for the electronic basis must be
! searched to find which is the relevant one
! 
!#######################################################################

      subroutine ssetproj(st,ebdim,ebspfs,psi,vdim,dim,ndim)

      implicit none

      integer(long)             :: e,e1,v,n
      integer(long), intent(in) :: ebdim,vdim,dim,ndim,st

      complex(dop), dimension(ebdim,dim), intent(in)      :: ebspfs
      complex(dop), dimension(dim,dim)                    :: projector
      complex(dop), dimension(dim)                        :: stvec
      complex(dop), dimension(vdim,dim,ndim), intent(out) :: psi
      complex(dop), dimension(vdim,dim,ndim)              :: tmppsi

! set up projector for electronic basis to desired states
      do e=1,dim
         stvec(e) = ebspfs(st,e)
      enddo

      do e=1,dim
         do e1=1,dim
            projector(e1,e) = dconjg(stvec(e1))*stvec(e)
         enddo
      enddo

! and project coefficients
      tmppsi(:,:,:) = psi(:,:,:)
      psi = 0.0_dop
      do v=1,vdim
         do e=1,dim
            do e1=1,dim
               do n=1,ndim
                  psi(v,e1,n) = psi(v,e1,n)+projector(e1,e)*tmppsi(v,e,n)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine ssetproj

!#######################################################################
!
! State projection for density type I single-set
!
!#######################################################################

      subroutine ssetproj1(st,ebdim,ebspfs,psi,vdim,dim,ndim)

      implicit none

      integer(long)             :: e,e1,v,n
      integer(long), intent(in) :: ebdim,vdim,dim,ndim,st

      complex(dop), dimension(dim,dim), intent(in)        :: ebspfs
      complex(dop), dimension(dim,dim)                    :: projector
      complex(dop), dimension(vdim,dim,ndim), intent(out) :: psi
      complex(dop), dimension(vdim,dim,ndim)              :: tmppsi

! set up projector for electronic basis to desired states
      call ss1proj(projector,ebspfs,st,ebdim)

! and project coefficients
      tmppsi(:,:,:) = psi(:,:,:)
      psi = 0.0_dop
      do v=1,vdim
         do e=1,dim
            do e1=1,dim
               do n=1,ndim
                  psi(v,e1,n) = psi(v,e1,n)+projector(e1,e)*tmppsi(v,e,n)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine ssetproj1

      
      subroutine ss1proj(projector,ebspfs,st,ebdim)

      implicit none

      integer(long), intent(in) :: st,ebdim
      integer(long)             :: e,e1
      complex(dop), dimension(ebdim,ebdim,ebdim,ebdim), intent(in)  :: ebspfs
      complex(dop), dimension(ebdim,ebdim,ebdim,ebdim), intent(out) :: projector

      projector=0.0_dop
      do e=1,ebdim
         do e1=1,ebdim
            projector(st,st,e1,e) = ebspfs(st,st,e1,e) 
         enddo
      enddo
            

      return
      end subroutine ss1proj

!#######################################################################

      subroutine spfden2d(psi,den2d,vgdim1,gdim1,ngdim1,gdim2,ngdim2, &
                 g1,g2,dim)

      implicit none

      integer(long)              :: v1,n1,n2,e,e1
      integer(long), intent(in)  :: vgdim1,gdim1,ngdim1,gdim2,ngdim2,g1,g2,dim

      complex(dop), dimension(dim,dim), intent(out)                             :: den2d
      complex(dop), dimension(vgdim1,gdim1,ngdim1,gdim2,ngdim2,dim), intent(in) :: psi

      do e=1,dim
         do e1=1,dim
            den2d(e1,e)=0.0_dop
            do n2=1,ngdim2
               do n1=1,ngdim1
                  do v1=1,vgdim1
                     den2d(e1,e)=den2d(e1,e)+ &
                  dconjg(psi(v1,g1,n1,g2,n2,e1))*psi(v1,g1,n1,g2,n2,e)
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine spfden2d

!#######################################################################

      subroutine rho1den2d(psi,den2d,subdim1,m1,f1,f2,g1,g2,dim)

      implicit none

      integer(long)                    :: e,g
      integer(long), intent(in)        :: subdim1,m1,f1,f2,g1,g2,dim
      integer(long), dimension(maxdim) :: fpoint

      complex(dop), dimension(dim), intent(out)                :: den2d
      complex(dop), dimension(subdim1,subdim1,dim), intent(in) :: psi

      den2d(:)=0.0_dop
      do e=1,dim
         do g=1,subdim1
            call decoderm(g,fpoint,m1)
            if (fpoint(f1) .ne. g1) cycle
            if (f2 .gt. 0) then
               if (fpoint(f2) .ne. g2) cycle
            endif
            den2d(e)=den2d(e) + psi(g,g,e)
         enddo
      enddo

      return
      end subroutine rho1den2d

!#######################################################################

      subroutine gwpden2d(m1,agmat,xi,phi,den2d,gdof,dim1,coo1,coo2,n1,n2,&
                 f1,f2,weight)

      implicit none

      integer(long)                                       :: e,e1,n,f,nim
      integer(long)                                       :: iperiod1,iperiod2,nimage
      integer(long), intent(in)                           :: dim1,gdof,n1,n2,&
                                                             m1,f1,f2
      complex(dop)                                        :: tmp,phig,ag,xig,tmpden2d
      complex(dop), dimension(dim1), intent(in)           :: phi
      complex(dop), dimension(dim1,dim1), intent(out)     :: den2d
      complex(dop), dimension(gdof,dim1), intent(in)      :: xi
      complex(dop), dimension(gdof,gdof,dim1), intent(in) :: agmat
      real(dop), intent(in)                               :: coo1,coo2
      real(dop), dimension(ortdim), intent(in)            :: weight
      real(dop), dimension(8)                             :: imcoo1,imcoo2
      real(dop)                                           :: rtmp,pi,th
      real(dop)                                           :: xrange1,xrange2

      pi = 4.0_dop*datan(1.0_dop)

      nimage = 1
      imcoo1(1) = coo1
      imcoo2(1) = coo2

!-----------------------------------------------------------------------
! Check for periodic GWPs
!-----------------------------------------------------------------------
     xrange1=0.0
     xrange2=0.0
     iperiod1 = ipbaspar(2,f1)
     if (iperiod1 .ne. 0) then
        xrange1 = rpbaspar(2,f1) - rpbaspar(1,f1)

        nimage = nimage+1
        imcoo1(nimage) = coo1+xrange1
        imcoo2(nimage) = coo2
        nimage = nimage+1
        imcoo1(nimage) = coo1-xrange1
        imcoo2(nimage) = coo2
     endif
     if (f2 .gt. 0) then
        iperiod2 = ipbaspar(2,f2)
        if (iperiod2 .ne. 0) then
           xrange2 = rpbaspar(2,f2) - rpbaspar(1,f2)

           nimage = nimage+1
           imcoo1(nimage) = coo1
           imcoo2(nimage) = coo2+xrange2
           nimage = nimage+1
           imcoo1(nimage) = coo1
           imcoo2(nimage) = coo2-xrange1
      
           if (iperiod1 .ne. 0) then
              nimage = nimage+1
              imcoo1(nimage) = coo1+xrange1
              imcoo2(nimage) = coo2+xrange2
              nimage = nimage+1
              imcoo1(nimage) = coo1-xrange1
              imcoo2(nimage) = coo2+xrange2
              nimage = nimage+1
              imcoo1(nimage) = coo1+xrange1
              imcoo2(nimage) = coo2-xrange2
              nimage = nimage+1
              imcoo1(nimage) = coo1-xrange1
              imcoo2(nimage) = coo2-xrange2
           endif
        endif
     endif

     den2d(:,:) = 0.0d0
!-----------------------------------------------------------------------
! Loop over periodic images and add up density at point.
!-----------------------------------------------------------------------
images:  do nim = 1,nimage

!-----------------------------------------------------------------------
! trace out DOFs other than n1,n2 (exponent of integral)
!-----------------------------------------------------------------------
      do e=1,dim1
         do e1=e,dim1
            phig = dconjg(phi(e1)) + phi(e)

            tmpden2d=phig
            do n = 1,gdof
               if ((n .ne. n1) .and. (n .ne. n2)) then
                  ag = dconjg(agmat(n,n,e1)) + agmat(n,n,e)
                  xig = dconjg(xi(n,e1)) + xi(n,e)

                  tmpden2d=tmpden2d - xig**2/(4.0_dop*ag)

                  rtmp=-pi/dble(ag)
                  rtmp=0.5_dop*dlog(rtmp)
                  tmpden2d=tmpden2d + rtmp
               endif
            enddo

! add value of GWP exponent at n1,n2

            if (e1 .eq. e) then
               tmp = agmat(n1,n1,e)*imcoo1(nim)*imcoo1(nim) + xi(n1,e)*imcoo1(nim)
               if (n1 .ne. n2) then
                  tmp = tmp + agmat(n2,n2,e)*imcoo2(nim)*imcoo2(nim) + xi(n2,e)*imcoo2(nim)
               end if
               rtmp = 2.0_dop*dble(tmp) + dble(tmpden2d)

               tmpden2d = dexp(rtmp)

            else
               tmp = agmat(n1,n1,e)*imcoo1(nim)*imcoo1(nim) + xi(n1,e)*imcoo1(nim)
               tmp = tmp + &
                    dconjg(agmat(n1,n1,e1)*imcoo1(nim)*imcoo1(nim) + xi(n1,e1)*imcoo1(nim))
               if (n1 .ne. n2) then
                  tmp = tmp + agmat(n2,n2,e)*imcoo2(nim)*imcoo2(nim) + xi(n2,e)*imcoo2(nim)
                  tmp = tmp + dconjg(agmat(n2,n2,e1)*imcoo2(nim)*imcoo2(nim) + &
                                     xi(n2,e1)*imcoo2(nim))
               end if

               tmpden2d = tmp + tmpden2d

               rtmp = dble(tmpden2d)
               rtmp = dexp(rtmp)
               th=dimag(tmpden2d)
               tmp=dcmplx(dcos(th),dsin(th))
               tmpden2d = rtmp*tmp
            endif

! multiply by DVR weight for grid point. All weights are identical - take first.
            f = spfdof(n1,m1)
            tmpden2d=tmpden2d*weight(zort(f))*weight(zort(f))
            if (n2 .ne. n1) then
               f = spfdof(n2,m1)
               tmpden2d=tmpden2d*weight(zort(f))*weight(zort(f))
            endif

            den2d(e1,e) = den2d(e1,e) + tmpden2d
            if (e1 .ne. e) den2d(e,e1) = dconjg(den2d(e1,e))
         enddo
      enddo

      enddo images

      return
      end subroutine gwpden2d

!#######################################################################

      subroutine trden2d(psi,psidia,nspfdof,dim,gpoint1,gpoint2, &
                 n1,n2,trbin,trajst,nddstate)

      implicit none

      integer(long)                                    :: e,s
      integer(long), intent(in)                        :: dim,nspfdof,n1,n2
      real(dop)                                        :: diff1,diff2,trbin12,trbin22
      real(dop), intent(in)                            :: gpoint1,gpoint2
      integer(long), intent(in)                        :: nddstate
      integer(long), dimension(dim), intent(in)        :: trajst
      real(dop), dimension(nddstate), intent(out)      :: psidia
      real(dop), dimension(2), intent(in)              :: trbin
      complex(dop), dimension(nspfdof,dim), intent(in) :: psi
      logical(kind=4)                                  :: ignore

      psidia = 0.0_dop
      trbin12 = trbin(1)**2
      if (n2 .gt. 0) then
         trbin22 = trbin(2)**2
      else
         trbin22 = 0.0_dop
      endif

!-----------------------------------------------------------------------
! calculate population at grid points (sum trajectories and normalise)
!-----------------------------------------------------------------------
      do e=1,dim
         ignore = .false.
         diff1 = 0.0_dop
         diff2 = 0.0_dop

         diff1 = dble(psi(n1,e)) - gpoint1
         if (n2 .gt. 0) diff2 = dble(psi(n2,e)) - gpoint2
         diff1 = diff1**2
         diff2 = diff2**2

! look to see if outside bin
         if (diff1 .gt. trbin12) ignore=.true.
         if ((n2 .gt. 0) .and. (diff2 .gt. trbin22)) ignore=.true.
         if (.not. ignore) then
            s = trajst(e)
            psidia(s) = psidia(s) + 1.0d0
         endif
      enddo

!-----------------------------------------------------------------------
! normalise and
! multiply by integration interval, i.e. dxi, to match SPF convention
!-----------------------------------------------------------------------
      do s=1,nddstate
         psidia(s) = psidia(s) / dble(dim)
         if (n2 .eq. 0) then
            psidia(s) = psidia(s)*trbin(1)
         else
            psidia(s) = psidia(s)*trbin(1)*trbin(2)
         endif
      enddo

      return
      end  subroutine trden2d 

!#######################################################################
!   SUBROUTINE VSHIFTK3
!
!   Applies multidimensional potential and shifts in K-dof:
!           hpsi(x,...,k,...,y)=V(x,...,y)*psi(x,...,k+m,...,y)
!#######################################################################

      subroutine vshiftk3(hop,m,psi,hpsi,vgdim,kdim,ngdim)

      implicit none

      integer(long)                                          :: k,v,n
      integer(long), intent(in)                              :: m,vgdim,kdim,ngdim
      real(dop), dimension(vgdim,ngdim), intent(in)          :: hop
      complex(dop), dimension(vgdim,kdim,ngdim), intent(in)  :: psi
      complex(dop), dimension(vgdim,kdim,ngdim), intent(out) :: hpsi



! --- for m>0
      if (m .gt. 0) then
         do n=1,ngdim
            do k=1,kdim-m
               do v=1,vgdim
                  hpsi(v,k,n) = hop(v,n)*psi(v,k+m,n)
               enddo
            enddo
         enddo
         do n=1,ngdim
            do k=kdim-m+1,kdim
               do v=1,vgdim
                  hpsi(v,k,n) = (0.0_dop,0.0_dop)
               enddo
            enddo
         enddo
! --- for m<0
      else if (m .lt. 0) then
         do n=1,ngdim
            do k=1,-m
               do v=1,vgdim
                  hpsi(v,k,n) = (0.0_dop,0.0_dop)
               enddo
            enddo
         enddo
         do n=1,ngdim
            do k=-m+1,kdim
               do v=1,vgdim
                  hpsi(v,k,n) = hop(v,n)*psi(v,k+m,n)
               enddo
            enddo
         enddo
! --- for m=0
      else if (m .eq. 0) then
         do n=1,ngdim
            do k=1,kdim
               do v=1,vgdim
                  hpsi(v,k,n) = hop(v,n)*psi(v,k,n)
               enddo
            enddo
         enddo
      endif

      return
      end subroutine vshiftk3

!#######################################################################
!   SUBROUTINE WSHIFTK3
!
!   Applies multidimensional potential and shift in K-dof (for Wigner):
!           hpsi(x,...,k,m,...,y)=V(x,...,y)*psi(x,...,k+kk,m+mm,...,y)
!#######################################################################

      subroutine wshiftk3(hop,kk,mm,psi,hpsi,vgdim,kdim,mmdim,ngdim)

      implicit none

      integer(long), intent(in)   :: mm,kk,vgdim,kdim,mmdim,ngdim
      integer(long)               :: v,k,m,n
      real(dop), dimension(vgdim,ngdim), intent(in)                :: hop
      complex(dop), dimension(vgdim,kdim,mmdim,ngdim), intent(in)  :: psi
      complex(dop), dimension(vgdim,kdim,mmdim,ngdim), intent(out) :: hpsi

      do n=1,ngdim

        if (mm.gt.0) then
          do m=1,mmdim-mm
            if (kk.gt.0) then
              do k=1,kdim-kk
                do v=1,vgdim
                  hpsi(v,k,m,n) = hop(v,n)*psi(v,k+kk,m+mm,n)
                enddo
              enddo
              do k=kdim-kk+1,kdim
                do v=1,vgdim
                  hpsi(v,k,m,n) = (0.d0,0.d0)
                enddo
              enddo
            elseif (kk.lt.0) then
              do k=1,-kk
                do v=1,vgdim
                  hpsi(v,k,m,n) = (0.d0,0.d0)
                enddo
              enddo
              do k=-kk+1,kdim
                do v=1,vgdim
                  hpsi(v,k,m,n) = hop(v,n)*psi(v,k+kk,m+mm,n)
                enddo
              enddo
            else  ! kk=0
              do k=1,kdim
                do v=1,vgdim
                  hpsi(v,k,m,n) = hop(v,n)*psi(v,k,m+mm,n)
                enddo
              enddo
            endif
          enddo
          do m=mmdim-mm+1,mmdim
            do k=1,kdim
              do v=1,vgdim
                hpsi(v,k,m,n) = (0.d0,0.d0)
              enddo
            enddo
          enddo

        elseif (mm.lt.0) then
          do m=1,-mm
            do k=1,kdim
              do v=1,vgdim
                hpsi(v,k,m,n) = (0.d0,0.d0)
              enddo
            enddo
          enddo
          do m=-mm+1,mmdim
            if (kk.gt.0) then
              do k=1,kdim-kk
                do v=1,vgdim
                  hpsi(v,k,m,n) = hop(v,n)*psi(v,k+kk,m+mm,n)
                enddo
              enddo
              do k=kdim-kk+1,kdim
                do v=1,vgdim
                  hpsi(v,k,m,n) = (0.d0,0.d0)
                enddo
              enddo
            elseif (kk.lt.0) then
              do k=1,-kk
                do v=1,vgdim
                  hpsi(v,k,m,n) = (0.d0,0.d0)
                enddo
              enddo
              do k=-kk+1,kdim
                do v=1,vgdim
                  hpsi(v,k,m,n) = hop(v,n)*psi(v,k+kk,m+mm,n)
                enddo
              enddo
            else  ! kk=0
              do k=1,kdim
                do v=1,vgdim
                  hpsi(v,k,m,n) = hop(v,n)*psi(v,k,m+mm,n)
                enddo
              enddo
            endif
          enddo

        else  ! mm=0
          do m=1,mmdim
            if (kk.gt.0) then
              do k=1,kdim-kk
                do v=1,vgdim
                  hpsi(v,k,m,n) = hop(v,n)*psi(v,k+kk,m,n)
                enddo
              enddo
              do k=kdim-kk+1,kdim
                do v=1,vgdim
                  hpsi(v,k,m,n) = (0.d0,0.d0)
                enddo
              enddo
            elseif (kk.lt.0) then
              do k=1,-kk
                do v=1,vgdim
                  hpsi(v,k,m,n) = (0.d0,0.d0)
                enddo
              enddo
              do k=-kk+1,kdim
                do v=1,vgdim
                  hpsi(v,k,m,n) = hop(v,n)*psi(v,k+kk,m,n)
                enddo
              enddo
            else  ! kk=0
              do k=1,kdim
                do v=1,vgdim
                  hpsi(v,k,m,n) = hop(v,n)*psi(v,k,m,n)
                enddo
              enddo
            endif
          enddo
        endif

      enddo  ! ngdim

      return
      end subroutine wshiftk3

!#######################################################################
!
! GETVGN
!
! for Hamiltonian term h, compute vgdim,gdim,ngdim.
! FO 10/2006
!
!#######################################################################

      subroutine getvgn(h,vd,gd,nd)

      implicit none

      integer(long)              :: m,f,d,n,na
      integer(long), intent(in)  :: h
      integer(long), intent(out) :: vd,gd,nd

      na=0
      m=htmmode(h)
      if (m.eq.0) then
!        not a mode operator...
         f=htmdof(h)
         if (f.eq.0) then
            routine='getvgn'
            message='FIXME: m=0 f=0 - muld term?'
            call errormsg
         endif
!        ...but a DOF operator
         vd=vgdim(f)
         gd=gdim(f)
         nd=ngdim(f)
      else
!        mode operator
         d=htmndof(h)
         if (d.eq.0 .or. d.eq.nspfdof(m)) then
!           term acts on full mode
            vd=1
            gd=subdim(m)
            nd=1
         else
!           term acts on part of the mode only
!           compute vgdim
            vd=1
            do n=1,nspfdof(m)
!              the first DOF h acts on is htmadof(h)
!              this is the na-th DOF inside the mode.
               f=spfdof(n,m)
               if (f.eq.htmadof(h)) then
                  na=n
                  goto 10
               endif
               vd=vd*gdim(f)
            enddo
            routine='getvgn'
            write (message,'(a,i5)')&
               'FIXME: htmadof not in mode? h = ',h
            call errormsg
   10       continue
!           compute gdim
            gd=1
            do n=na,na+htmndof(h)-1
               gd=gd*gdim(spfdof(n,m))
            enddo
!           compute ngdim
            nd=1
            do n=na+htmndof(h),nspfdof(m)
               nd=nd*gdim(spfdof(n,m))
            enddo
         endif
      endif

      return
      end subroutine getvgn

!#######################################################################
!
! VCALC1ST
!
! calculates the (1-dim) potential value v at a point 
! specified by the integer array mpoint(m). Used in vinmax.
!
!#######################################################################

      subroutine vcalc1st(hops,v,mpoint,nham)

      implicit none

      integer(long)                                :: m,f,k1,k,h,s,pt,vd,gd,nd,n
      integer(long), intent(in)                    :: nham
      integer(long), dimension(maxdim), intent(in) :: mpoint
      real(dop)                                    :: pescorr
      real(dop), intent(out)                       :: v
      real(dop), dimension(hopsdim), intent(in)    :: hops
      logical(kind=4)                              :: lkin

      v=0.0_dop
      s=1   ! This routine is only for nstate=1
!.....uncorrelated terms.
      do m=1,nmode
!$omp parallel do private(k,n,f,h,vd,gd,nd,pt), reduction(+:v)
         do k1=1,khzahl(m,s,nham)
            k=zhun(m,s,nham)+k1-1
! ignore any KE terms
            if (kinop(m,k) .ne. 0) cycle

            do n=1,nspfdof(m)
               f=spfdof(n,m)
               h=hamilton(f,k)
               if (htmtype(h).ne.1) then
                  call getvgn(h,vd,gd,nd)
                  pt = (mpoint(m)-1)/vd
                  pt = mod(pt,gd) + zhop(h)
                  v=v+dble(koeff(k))*hops(pt)
               end if
            enddo
         enddo
!$omp end parallel do
      enddo

!.....correlated terms.
!$omp parallel do private(pescorr,m,n,f,h,vd,gd,nd,pt), reduction(+:v)
outer:do k=1,kzahl(nham)
! ignore any KE terms
         lkin = .false.
         do m=1,nmode
            lkin = lkin .or. kinop(m,k).ne.0
         enddo
         if(lkin)cycle outer

         pescorr=dble(koeff(k))
         inner: do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               h=hamilton(f,k)
               if (htmtype(h).ne.1) then !unit op has no hops
                  call getvgn(h,vd,gd,nd)
                  pt = (mpoint(m)-1)/vd
                  pt = mod(pt,gd) + zhop(h)
                  pescorr=pescorr*hops(pt)
               end if
            end do
         enddo inner
         v=v+pescorr
      enddo outer
!$omp end parallel do

      return
      end subroutine vcalc1st

!#######################################################################
!
! VCALCMLST
!
! calculates the adiabatic potential values for the electronic
! states at a point specified by the integer array mpoint(m) (fpoint(f)
! for multidim. operator). Used in vminmax.
! Similar to calcdiab on opfuncs/potevalmod.f90
!
!#######################################################################

      subroutine vcalcmlst(hops,pesad,pesdia,rotmat,v,mpoint,fpoint)

      implicit none

      integer(long)                                    :: s,m,f,k1,k,h,s1,n,nm, &
                                                          error,muld,zeig, &
                                                          sadia,sdia1,sdia2,nham, &
                                                          vd,nd,gd,pt
      integer(long), dimension(maxdim)                 :: mdvgdim
      integer(long), dimension(maxdim), intent(in)     :: mpoint,fpoint
      real(dop)                                        :: pescorr
      real(dop), intent(out)                           :: v
      real(dop), dimension(maxsta), intent(out)        :: pesad
      real(dop), dimension(hopsdim), intent(in)        :: hops
      real(dop), dimension(maxsta,maxsta), intent(out) :: pesdia,rotmat
      real(dop), dimension(3*maxsta)                   :: workr
      logical(kind=4)                                  :: lkin

      common /vmmblk/ sadia, sdia1, sdia2, nham

      call zeromxd(pesdia,maxsta,maxsta)

!.....uncorrelated terms.
      do s=1,nstate
         do m=1,nmode
            do k1=1,khzahl(m,s,nham)
               k=zhun(m,s,nham)+k1-1
! ignore any KE terms
               if (kinop(m,k) .ne. 0) cycle

               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  h=hamilton(f,k)
                  if (htmtype(h).ne.1) then !unit op has no hops
                     call getvgn(h,vd,gd,nd)
                     pt = (mpoint(m)-1)/vd
                     pt = mod(pt,gd) + zhop(h)
                     pesdia(s,s)=pesdia(s,s)+dble(koeff(k))*hops(pt)
                  end if
               enddo
            enddo
         enddo
      enddo

!.....correlated terms.
outer:do k=1,kzahl(nham)
! ignore any KE terms
         lkin = .false.
         do m=1,nmode
            lkin = lkin .or. kinop(m,k).ne.0
         enddo
         if(lkin)cycle outer

         s1=ki(k)
         s=kf(k)
         pescorr=dble(koeff(k))
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               h=hamilton(f,k)
               if (htmtype(h).ne.1) then !unit op has no hops
                  call getvgn(h,vd,gd,nd)
                  pt = (mpoint(m)-1)/vd
                  pt = mod(pt,gd) + zhop(h)
                  pescorr=pescorr*hops(pt)
               end if
            end do
         enddo
         pesdia(s,s1)=pesdia(s,s1)+pescorr
      enddo outer

!.....muld terms.
      do nm=1,nmulpot(nham)
         k=kzahl(1)+nm
         s1=ki(k)
         s=kf(k)
         pescorr=dble(koeff(k))

         f=fterm(k)
         h=hamilton(f,k)
         muld=htmmuld(h)
         zeig=1
         do n=1,nmulddof(muld)
            f=mulddof(n,muld)
            mdvgdim(f)=zeig
            zeig=zeig*gdim(f)
         enddo

         pt=zhop(h)
         do n=1,nmulddof(muld)
            f=mulddof(n,muld)
            pt=pt+mdvgdim(f)*(fpoint(f)-1)
         enddo
         pescorr=pescorr*hops(pt)
         pesdia(s,s1)=pesdia(s,s1)+pescorr
      enddo

      if (nstate .eq. 1) then
         pesad(1)=pesdia(1,1)
         v = pesad(1)
      else if (sdia1.gt.0 .and. sdia2.gt.0) then
         v = pesdia(sdia1,sdia2)
      else
         call cpmxd(pesdia,rotmat,maxsta,maxsta)
         call dsyev('N','U',nstate,rotmat,maxsta,pesad,workr,&
                    3*maxsta,error)
         v = pesad(sadia)
         if (error .ne. 0) then
            write(6,'(a)') &
                     'Error diagonalising diabatic PES matrix'
            stop
         endif

      endif

      return
      end subroutine vcalcmlst

!###########################################################################
!  Find the associated natpot number or MLOp number to which belongs
!  the single term h.
!###########################################################################

      subroutine findnath(h,nnat,imlop)

      implicit none

      integer(long)              :: h1,hl,h2,hl2,l,l1,l2,ileaf,ibasis
      integer(long), intent(in)  :: h
      integer(long), intent(out) :: nnat,imlop

      nnat=0
      imlop=0
      if (htmform(h).eq.0) then
         do h1=1,htmfac(h)
            hl=hterm(h1,h)
            if (hl.ne.0) then
               if (hoplab(hl)(1:4) .eq. 'npot') then
                  l=index(hoplab(hl),' ')-1
                  l2=0
                  do l1=l,1,-1
                     if (hoplab(hl)(l1:l1).eq.',') then
                        l2=l1
                     endif
                  enddo
                  if (l2.ne.0) read(hoplab(hl)(l2+1:l),*) nnat
               elseif (hoplab(hl)(1:4) .eq. 'mlnp') then
                  call mlnp_parse(hoplab(hl),imlop,ileaf,ibasis)
               endif
            endif
         enddo
      else
         do h1=1,htmfac(h)
            hl=hterm(h1,h)
            if (hl.ne.0) then
               do h2=1,htmfac(hl)
                  hl2=hterm(h2,hl)
                  if (hl2.ne.0) then
                     if (hoplab(hl2)(1:4) .eq. 'npot') then
                        l=index(hoplab(hl2),' ')-1
                        l2=0
                        do l1=l,1,-1
                           if (hoplab(hl2)(l1:l1).eq.',') then
                              l2=l1
                           endif
                        enddo
                        if (l2.ne.0) read(hoplab(hl2)(l2+1:l),*) nnat
                     elseif (hoplab(hl2)(1:4) .eq. 'mlnp') then
                        call mlnp_parse(hoplab(hl2),imlop,ileaf,ibasis)
                    endif
                  endif
               enddo
            endif
         enddo
      endif

      return
      end subroutine findnath

!###########################################################################
!  Find the associated cpd number to which belongs the single term h.
!###########################################################################

      subroutine findcpdh(h,ncpd)

      implicit none

      integer(long) :: h1,hl,h2,hl2,l,l1,l2
      integer(long), intent(in)  :: h
      integer(long), intent(out) :: ncpd

      ncpd=0
      if (htmform(h).eq.0) then
         do h1=1,htmfac(h)
            hl=hterm(h1,h)
            if (hl.ne.0) then
               if (hoplab(hl)(1:5) .eq. 'cpdtm') then
                  l=index(hoplab(hl),' ')-1
                  l2=0
                  do l1=l,1,-1
                     if (hoplab(hl)(l1:l1).eq.',') then
                        l2=l1
                     endif
                  enddo
                  if (l2.ne.0) read(hoplab(hl)(l2+1:l),*) ncpd
               endif
            endif
         enddo
      else
         do h1=1,htmfac(h)
            hl=hterm(h1,h)
            if (hl.ne.0) then
               do h2=1,htmfac(hl)
                  hl2=hterm(h2,hl)
                  if (hl2.ne.0) then
                     if (hoplab(hl2)(1:5) .eq. 'cpdtm') then
                        l=index(hoplab(hl2),' ')-1
                        l2=0
                        do l1=l,1,-1
                           if (hoplab(hl2)(l1:l1).eq.',') then
                              l2=l1
                           endif
                        enddo
                        if (l2.ne.0) read(hoplab(hl2)(l2+1:l),*) ncpd
                    endif
                  endif
               enddo
            endif
         enddo
      endif

      return
      end subroutine findcpdh

!###########################################################################
!  Find the associated hcpd number to which belongs the single term h.
!###########################################################################

      subroutine findhcpdh(h,nhcpd)

use global
use operdef

      implicit none

      integer h,nhcpd,h1,hl,h2,hl2,l,l1,l2

      nhcpd=0
      if (htmform(h).eq.0) then
         do h1=1,htmfac(h)
            hl=hterm(h1,h)
            if (hl.ne.0) then
               if (hoplab(hl)(1:6) .eq. 'hcpdtm') then
                  l=index(hoplab(hl),' ')-1
                  l2=0
                  do l1=l,1,-1
                     if (hoplab(hl)(l1:l1).eq.',') then
                        l2=l1
                     endif
                  enddo
                  if (l2.ne.0) read(hoplab(hl)(l2+1:l),*) nhcpd
               endif
            endif
         enddo
      else
         do h1=1,htmfac(h)
            hl=hterm(h1,h)
            if (hl.ne.0) then
               do h2=1,htmfac(hl)
                  hl2=hterm(h2,hl)
                  if (hl2.ne.0) then
                     if (hoplab(hl2)(1:6) .eq. 'hcpdtm') then
                        l=index(hoplab(hl2),' ')-1
                        l2=0
                        do l1=l,1,-1
                           if (hoplab(hl2)(l1:l1).eq.',') then
                              l2=l1
                           endif
                        enddo
                        if (l2.ne.0) read(hoplab(hl2)(l2+1:l),*) nhcpd
                    endif
                  endif
               enddo
            endif
         enddo
      endif

      return
      end subroutine findhcpdh


!###########################################################################
!  Find the associated read number to which belongs the single term h.
!###########################################################################

      subroutine findread(h,nrd)

      implicit none

      integer(long) :: h1,hl,l,l1,l2
      integer(long), intent(in)  :: h
      integer(long), intent(out) :: nrd

      nrd=0
      if(htmmode(h).eq.0) return  ! This is for mode operators only.
      if (htmform(h).eq.0) then
         do h1=1,htmfac(h)
            hl=hterm(h1,h)
            if (hl.ne.0) then
               if (hoplab(hl)(1:4) .eq. 'read') then
                  l=index(hoplab(hl),' ')-1
                  l2=0
                  do l1=l,1,-1
                     if (hoplab(hl)(l1:l1).eq.',') then
                        l2=l1
                     endif
                  enddo
                  if (l2.ne.0) read(hoplab(hl)(l2+1:l),*) nrd
               endif
            endif
         enddo
      endif

      return
      end subroutine findread

!###########################################################################
!  Find the associated readmap number to which belongs the single term h.
!###########################################################################

      subroutine findreadmap(h,nmap)

      implicit none

      integer h,nmap,h1,hl,l,l1,l2
      logical lerr


      nmap=0
      if (htmform(h).eq.0) then
         do h1=1,htmfac(h)
            hl=hterm(h1,h)
            if (hl.ne.0) then
               if (hoplab(hl)(1:7) .eq. 'readmap') then
                  l=index(hoplab(hl),' ')-1
                  l2=0
                  do l1=l,1,-1
                     if (hoplab(hl)(l1:l1).eq.',') then
                        l2=l1
                     endif
                  enddo
                  if (l2.ne.0) read(hoplab(hl)(l2+1:l),*) nmap
               endif
            endif
         enddo
      endif

!-----sanity checks
      if(nmap.gt.0) then
         lerr=.false.
         if(htmfac(h).gt.1) then
            routine='findmap on genoper/logop.F'
            message='A readmap operator must appear as a single term,'// &
            ' it cannot be multiplied with or added to other operators.'
             lerr=.true.
         endif
         if(htmdof(h).eq.0) then
            routine='findmap on genoper/logop.F'
            message='A readmap operator must be a DOF operator!'
            lerr=.true.
         endif
         if(lerr) then
            write(6,'(/a,2i5,a)') 'h, hl, hoplab',h,hl,trim(hoplab(hl))
            write(2,'(/a,2i5,a)') 'h, hl, hoplab',h,hl,trim(hoplab(hl))
            call errormsg
         endif
      endif

      return
      end subroutine findreadmap

!#######################################################################
!
! RDFACTORS
!
! breaks up a keyword into its factors.
!
! label: input keyword
! ilen:  length of label
!
! nfac:    no. of factors found in keyword
! factor:  factors found in keyword (e.g. h1, h2, h3)
! optyp:   operation type of factors *=1, /=2, +=3, -=4)
!
! e.g.
! label=' h1*h2+h3'
! factor(1)=h1, factor(2)=h2, factor(3)=h3
! optyp(1)=3,   optyp(2)=1,   optyp(3)=3
!
!#######################################################################

      subroutine rdfactors(label,nfac,factor,optyp,ilen)

      implicit none

      integer(long)                                 :: ilbl,jlbl,i,n
      integer(long), intent(in)                     :: ilen
      integer(long), intent(out)                    :: nfac
      integer(long), dimension(24)                  :: indx
      integer(long), dimension(24), intent(out)      :: optyp
      character(len=c2), dimension(24), intent(out)  :: factor
      character(len=*), intent(in)                  :: label
      logical(kind=4)                               :: ignore

      optyp(:)=0
      do i=1,24
         factor(i)='   '
      enddo
      nfac=1

!.....Check if the arithmetic expression starts with a '+' or '-'.
      if (label(1:1) .eq. '+') then
         indx(1)=1
         optyp(1)=3
      else if (label(1:1) .eq. '-') then
         indx(1)=1
         optyp(1)=4
      else
         indx(1)=0
         optyp(1)=3
      endif

!.....Search for the operators '+-*/'
      do i=2,ilen
         if (label(i:i) .eq. '*') then
            nfac=nfac+1
            indx(nfac)=i
            optyp(nfac)=1
         else if (label(i:i) .eq. '/') then
            nfac=nfac+1
            indx(nfac)=i
            optyp(nfac)=2
         else if (label(i:i) .eq. '+') then
            call chknum(label,indx(nfac),i,ignore)
            if (ignore) then
               nfac=nfac+1
               indx(nfac)=i
               optyp(nfac)=3
            endif
         else if (label(i:i) .eq. '-') then
            call chknum(label,indx(nfac),i,ignore)
            if (ignore) then
               nfac=nfac+1
               indx(nfac)=i
               optyp(nfac)=4
            endif
         endif
      enddo

      if( nfac .ge. 24 ) then
         routine='rdfactors (heinutil.F)'
         message='Too many factors: '//label
         call errormsg
      end if

      indx(nfac+1)=ilen+1

!.....Split the label into factors.
      do n=1,nfac
         ilbl=indx(n)+1
         jlbl=indx(n+1)-1
         factor(n)=label(ilbl:jlbl)
         if (factor(n) .eq. '1.'  )  factor(n)='1'
         if (factor(n) .eq. '1.0' )  factor(n)='1'
         if (factor(n) .eq. '1.00')  factor(n)='1'
         if (factor(n) .eq. '1_dop' )  factor(n)='1'
         if (factor(n) .eq. '1._dop')  factor(n)='1'
         if (factor(n) .eq. '1.0_dop') factor(n)='1'
      enddo

      return
      end subroutine rdfactors

!#######################################################################
!
! MKHPAR
!
! calculates a number from a set of factors, using the list of
! parameters.
!
!#######################################################################

      subroutine mkhpar(r,nfac,factor,optyp)

      implicit none

      integer(long)                                :: nx,n,ilbl,jlbl,ntm,&
                                                      nt,n1,ierr
      integer(long), intent(in)                    :: nfac
      integer(long), dimension(24), intent(in)     :: optyp
      real(dop)                                    :: power, x
      real(dop), dimension(24)                     :: rnum
      real(dop), intent(out)                       :: r
      character(len=*), dimension(24), intent(inout) :: factor
      logical(kind=4)                              :: linperr, hasexponent

!-----------------------------------------------------------------------
! initialise variables
!-----------------------------------------------------------------------
      r=0.0_dop

!-----------------------------------------------------------------------
! find no. or subterms in term
!-----------------------------------------------------------------------
      ntm=0
      do n=1,nfac
         if (optyp(n) .eq. 3 .or. optyp(n) .eq. 4) ntm=ntm+1
      enddo

!-----------------------------------------------------------------------
! read any power operation
! find which par is nth factor
!-----------------------------------------------------------------------
      do n=1,nfac
         ilbl=index(factor(n),'^')+1
         if (ilbl .eq. 1) then
            ilbl=len(factor(n))
            hasexponent = .false.
         else
            jlbl=index(factor(n),' ')-1
            read(factor(n)(ilbl:jlbl),*,iostat=ierr) power
            if (ierr.ne.0 .or. abs(power).gt.1.0e9_dop) then
               write(6,'(/2a,/2a/)') 'An exponent must be a number ', &
               'and cannot be a parameter. (Parameter-Section)', &
                   'Illegal exponent : ',factor(n)(ilbl:15)
               call stopnow
            endif
            ilbl=ilbl-2
            hasexponent = .true.
         endif

         nx=0
         do n1=1,min(npar,maxpar)
            if (factor(n)(1:ilbl) .eq. apar(n1)) nx=n1
         enddo
         if (nx .eq. 0) then
!........It is not a known parameter, hence assume it is a number.
            read(factor(n)(1:ilbl),*,iostat=ierr) rnum(n)
            if (ierr .ne. 0) then
               routine='mkhpar'
               message='Parameter not found in list : '//factor(n)
               linperr=.true.
               call warnmsg
               rnum(n)=0.0_dop
            endif
         else
             rnum(n)=rpar(nx)
         endif
         if (hasexponent) rnum(n)=rnum(n)**power
      enddo

!-----------------------------------------------------------------------
! loop over subterms and multiply factors. (ntm = number of summands).
!-----------------------------------------------------------------------
      n=1
      do nt=1,ntm
         n1=n
         x=rnum(n1)

   10    continue
         n=n+1
         if (optyp(n) .eq. 1) then
            x=x*rnum(n)
            go to 10
         else if (optyp(n) .eq. 2) then
            x=x/rnum(n)
            go to 10

         else if (optyp(n1) .eq. 3) then
            r=r+x
         else if (optyp(n1) .eq. 4) then
            r=r-x
         endif
      enddo

      return
      end subroutine mkhpar

!#########################################################################
!
! CHKMUM
! Checks if a '+' or a '-' is to be ignored, because it belongs to an
! exponent (i.e. a^-3) or to a number in exponential format (i.e. 2.d+3).
! The logical ignore is .true., if '+' or '-' are operators not to be
! ignored.
! HDM  12/00
!#########################################################################

      subroutine chknum(label,indx,i,ignore)

      implicit none

      integer(long)                :: j,ich,ich1,ich2,ich3
      integer(long), intent(in)    :: indx,i
      logical(kind=4), intent(out) :: ignore
      character(len=*), intent(in) :: label

!.....if there is a '^' indicating an exponent, return .false. .
      if( label(i-1:i-1) .eq. '^' ) then
         ignore = .false.
         return
      else
         ignore = .true.
      endif

!.....if there is no 'd' indicating a number in exponential writing, return.
      if( label(i-1:i-1) .ne. 'd' ) return

!.....if the character following  'd+' or 'd-' is not a number, then return.
      ich1 = ichar('0')
      ich2 = ichar('9')
      ich  = ichar(label(i+1:i+1))
      if( ich .lt. ich1  .or.  ich .gt. ich2 ) return

!.....if all characters before  'd+' or 'd-' are not '.' or a number, return.
      if( i-indx .lt. 3 ) return
      ich3 = ichar('.')
      do j = indx+1, i-2
         ich = ichar(label(j:j))
         if( ich .ne. ich3  .and.  &
              ich .lt. ich1  .or.  ich .gt. ich2 ) return
      end do

!.....All checks passed, it must be a number.
      ignore = .false.
      return

      end subroutine chknum

!#######################################################################
! CAsh edit
      subroutine getmaxeig(hops,ort,maxeigval,mineigval)

      implicit none

      integer(long)                             :: h,k,g,f,f1,ierr,&
                                                   si,sf,sdm
      real(dop), intent(out)                    :: maxeigval, mineigval
      real(dop)                                 :: maxk,mink,deltax,hval
      real(dop), dimension(hopsdim), intent(in) :: hops
      real(dop), dimension(ortdim), intent(in)  :: ort
      real(dop), dimension(nddstate,nddstate)   :: ham
      real(dop), dimension(3*nddstate)          :: workr
      real(dop), dimension(nddstate)            :: eigval

      maxeigval=0.0_dop
      mineigval=0.0_dop

      if (dentype .eq. 0) then
         sdm = subdim(1)
      else if (dentype .eq. 1) then
         sdm = supgdim(1)
      endif

! KE terms first
      do k=1,khtot(1)
         maxk = -999.0_dop
         mink = 999.0_dop
         if (kinop(1,k) .eq. 0) cycle
! ignore CAPs
         if (autocap(k) .ne. 0) cycle
! only consider KE once for multi-state
         if (ki(k) .gt. 1) cycle

         f = 0
         do f1=1,ndof
            if (f1 .eq. feb .or. f1 .eq. febcont) cycle
            if (hamilton(f1,k) .ne. f1) f=f1
         enddo
         if (basis(f) .lt. 3 .or. basis(f) .gt. 5) then
            mink=0.0_dop
            
            deltax = ort(zort(f)+gdim(f)-1)-ort(zort(f))
            maxk= pi/deltax
            maxk= maxk*maxk/(2.0_dop*mass(f))
         else
            mink=0.0_dop
            
            deltax = ort(zort(f)+1)-ort(zort(f))
            maxk= pi/deltax
            maxk= maxk*maxk/(2.0_dop*mass(f))
         endif

! sum eigenvalues
         maxeigval = maxeigval+maxk
         mineigval = mineigval+mink

      enddo

! potential term
      if (nddstate .eq. 1) then
         do k=1,khtot(1)
            if (kinop(1,k) .eq. 1) cycle
            if (autocap(k) .ne. 0) cycle
            maxk = -999.0_dop
            mink = 999.0_dop

            h=hamilton(1,k)
            do g=1,sdm
               hval = dble(koeff(k))*hops(zhop(h)+g-1)
               maxk=max(maxk,hval)
               mink=min(mink,hval)
            enddo

! sum eigenvalues
            maxeigval = maxeigval+maxk
            mineigval = mineigval+mink

         enddo
      else
         maxk = -999.0_dop
         mink = 999.0_dop
         do g=1,sdm
            ham = 0.0_dop
            do k=1,khtot(1)
               if (kinop(1,k) .eq. 1) cycle
               if (autocap(k) .ne. 0) cycle

               h=hamilton(1,k)
               si = ki(k)
               sf = kf(k)

               ham(sf,si) = ham(sf,si) + dble(koeff(k))*hops(zhop(h)+g-1)
            enddo
            call dsyev('V','U',nddstate,ham,nddstate,eigval,workr,3*nddstate,ierr)

            maxk=max(maxk,eigval(nddstate))
            mink=min(mink,eigval(1))
         enddo

! sum eigenvalues
         maxeigval = maxeigval+maxk
         mineigval = mineigval+mink
      endif

      write(ilog,'(/,a,2f16.8)') 'Min / Max Eigenvalues (eV) : ',&
                               mineigval*ev,maxeigval*ev
 
      if (dentype .eq. 1) then
         maxeigval = maxeigval - mineigval
         mineigval = -maxeigval 
      endif

      return
      end subroutine getmaxeig

!-----------------------------------------------------------------------
subroutine mlnp_parse(label,imlop,ileaf,ibasis)
!-----------------------------------------------------------------------
   implicit none
   character(len=*) :: label
   integer          :: imlop,ileaf,ibasis
   integer          :: il,ir
   ! Default values, indicating error.
   imlop  = 0
   ileaf  = 0
   ibasis = 0
   ! Read parameters from label: MLOp number, leaf number, basis index
   il = 5
   ir = scan(label,",")
   read(label(il:ir-1),*,err=500) imlop
   il = ir+1
   ir = ir+scan(label(il:),",")
   read(label(il:ir-1),*,err=500) ileaf
   il = ir+1
   read(label(il:)    ,*,err=500) ibasis
   500 continue
   return
end subroutine mlnp_parse

!-----------------------------------------------------------------------

      end module operutil

