!#######################################################################
!               MODUL PSIUTIL
! phipnt,phipnt1,spfpnt,calcpsi,spfpoint,getspfpnt,getweight,showpop2,
! trkleg,qtexp,plegrueck,pleghin,d1regular,maxmatwk,
!
!#######################################################################

      module psiutil

use decimal, only: long, dop
use mmlib,   only: mmxxzz, mmaxzz, mmtxzz
use mtlib,   only: mttxzz, qtxxdz, mtxxdz, dtxxzzo, qtxxzz
use xvlib,   only: vvaxzz, qvaxzz1, xvixdzo
use fftmod,  only: fft, swapfft
use op1lib,  only: zerovxz, zeromxz, zerovxd, cpmxz, cpvxz, initvxz,&
                   normvxz, sum2vxz, cpqxz
use lalib,   only: simtran2bz, simtran2tz
use logdat

use global
use aglobal
use dvrdatmod
use griddatmod
use psidef
use operdef
use maxv, only: maxdim, maxsta
use schmidtorthomod, only: schmidtortho
use d2tens, only: mkherm, mkherm2

use gwplib, only: diagonal_moments
use linear, only: ijk_shape

      implicit none
      private
      public :: adecoder, decoder, decoderm, phipnt, phipnt1, spfpnt, &
                adecoder_nomeb,calcpsi, getweight, showpop2, &
                trkleg, qtexp, plegrueck, pleghin, maxmatwk, d1regular, &
                f2mdecoder, vendecoder, ecalcd2d, espfden2d, renorma, &
                renormphi, trwigner, akcopy,akcopypsi3d,akcopy2, &
                akcopy_wf_to_d2, spfovl, calcpsis, a1decoder, &
                mltop_decoder, akadd, amdecoder_nomeb

      contains

!#######################################################################
! Decodes the A-coeff index
!
! --- The array mii contains the mode indices 
! --- The integer mis contains the multi-set state label 
! --- the total A-vector index ii.
!
!#######################################################################

      subroutine adecoder(ii,mii,mis)

      implicit none

      integer(long)                                 :: m,k,s
      integer(long), intent(in)                     :: ii
      integer(long), dimension(maxdim), intent(out) :: mii
      integer(long), intent(out)                    :: mis

      mii = 0
      mis = 0

      k = ii - 1
      do s = 1, nstate
         mis = s
         if(k.lt.block(s)) exit 
         k = k - block(s)
      enddo 

      s = mis
      do m = nmode, 2, -1
         mii(m) = k/vdim(m,s)
         k = k - mii(m)*vdim(m,s)

         mii(m) = mii(m)+1
      enddo
      mii(1) = k+1

      return
      end subroutine adecoder

!#######################################################################
! Decodes the A-coeff index. Electronic mode ignored.
!
! --- The array mii contains the mode indices 
! --- The integer mis contains the multi-set state label 
! --- the total A-vector index ii.
!
!#######################################################################

      subroutine adecoder_nomeb(ii,mii)

      implicit none

      integer(long)                                 :: m,k,s
      integer(long), intent(in)                     :: ii
      integer(long), dimension(maxdim), intent(out) :: mii

      mii = 0
      s = 1
      k = ii - 1
      do m = nmode, 2, -1
         if (m .eq. meb) cycle

         mii(m) = k/vdim(m,s)
         k = k - mii(m)*vdim(m,s)

         mii(m) = mii(m)+1
      enddo
      mii(1) = k+1

      return
      end subroutine adecoder_nomeb

!#######################################################################
! Decodes the A-coeff index. Electronic mode ignored.
! One mode, m, is in grid rather than SPF representation. As in CDVR.
!
! --- The array mii contains the mode indices 
! --- The integer mis contains the multi-set state label 
! --- the total A-vector index ii.
!
!#######################################################################

      subroutine amdecoder_nomeb(ii,mii,m1)

      implicit none

      integer(long)                                 :: m,k,s
      integer(long), intent(in)                     :: ii,m1
      integer(long), dimension(maxdim), intent(out) :: mii
      integer(long), dimension(maxdim)              :: vdim1

      s = 1
      do m=1,nmode
         if (m .eq. meb) cycle

         if (m .le. m1) then
            vdim1(m) = vdim(m,s)
         else
            vdim1(m) = vdim(m,s)*subdim(m1)/dim(m1,s)
         endif
      enddo

      mii = 0
      k = ii - 1
      do m = nmode, 2, -1
         if (m .eq. meb) cycle

         mii(m) = k/vdim1(m)
         k = k - mii(m)*vdim1(m)

         mii(m) = mii(m)+1
      enddo
      mii(1) = k+1

      return
      end subroutine amdecoder_nomeb

!#######################################################################
! Decodes the A-coeff index for type I density matrices
!
! --- The array mii contains the mode indices 
! --- The integer mis contains the multi-set state label 
! --- the total A-vector index ii.
!
!#######################################################################

      subroutine a1decoder(ii,mii,mis,mit)

      implicit none

      integer(long)                                 :: m,k,s,t
      integer(long), intent(in)                     :: ii
      integer(long), dimension(maxdim), intent(out) :: mii
      integer(long), intent(out)                    :: mis,mit

      mii = 0
      mis = 0

      k = ii - 1
      do s = 1, nstate
         do t = 1, nstate
            mis = s
            mit = s
            if(k.lt.dblock(s,t)) exit 
            k = k - dblock(s,t)
         enddo
      enddo 

      s = mis
      t = mit
      do m = nmode, 2, -1
         mii(m) = k/vddim(m,s,t)
         k = k - mii(m)*vddim(m,s,t)

         mii(m) = mii(m)+1
      enddo
      mii(1) = k+1

      return
      end subroutine a1decoder

!#######################################################################
! Decodes the A-coeff index for the top layer of an ML WF
!
! --- The array mii contains the mode indices 
! --- The integer mis contains the multi-set state label 
! --- the total A-vector index ii.
!
!
!#######################################################################

      subroutine mltop_decoder(tree,ii,mii,mis)

      implicit none

      integer(long)                                 :: m,k,s,nm
      integer(long), dimension(maxdim)              :: nd
      integer(long), intent(in)                     :: ii
      integer(long), dimension(maxdim), intent(out) :: mii
      integer(long), intent(out)                    :: mis
      type(wfnode_t),intent(inout) :: tree
      integer                      :: ijk(3)

      mii = 0
      mis = 0

      k = ii - 1
      mis = 1

      s = mis
      nm = tree%nmodes
      do m = 1,nm
         nd(m) = tree%ndim(m)
      enddo

      do m = nm, 2, -1
         ijk=ijk_shape(m,nd)

         mii(m) = k/ijk(1)
         k = k - mii(m)*ijk(1)

         mii(m) = mii(m)+1
      enddo
      mii(1) = k+1

      return
      end subroutine mltop_decoder

!#######################################################################
! DECODER
!
! decodes the i-th grid point into the integer array mpoint(m) and
! into the integer array fpoint(f). Used in vminmax and adpop.
!
! Electronic basis is ignored - i.e. assumed to be a single state
! so point is in nuclear grid space only
!#######################################################################

      subroutine decoder(i,mpoint,fpoint)

use global
use maxv, only: maxdim
use dvrdatmod
use griddatmod

      implicit none

      integer(long)                                 :: j,jj,f,point,m
      integer(long), intent(in)                     :: i
      integer(long), dimension(maxdim), intent(out) :: fpoint
      integer(long), dimension(nmode), intent(out)  :: mpoint

! assign all DOFs to first point
      fpoint(:) = 1

      j = i - 1
      do m = 1, nmode
         if (m .eq. meb) cycle
         jj = j/subdim(m)
         mpoint(m) = j - jj*subdim(m) + 1
         j = jj
         point=mpoint(m)-1
         do f=1,nspfdof(m)
            jj=point/gdim(spfdof(f,m))
            fpoint(spfdof(f,m))=point-jj*gdim(spfdof(f,m))+1
            point=jj
         enddo
      enddo

      return
      end subroutine decoder

!#######################################################################
! DECODERM
!
! decodes the i-th particle grid point into the integer array fpoint(f).
! Used in adpop.
!
!#######################################################################

      subroutine decoderm(i,fpoint,m)

use global
use maxv, only: maxdim
use dvrdatmod
use griddatmod

      implicit none

      integer(long)                                 :: jj,n,point
      integer(long), intent(in)                     :: i,m
      integer(long), dimension(maxdim), intent(out) :: fpoint

      point=i-1
      do n=1,nspfdof(m)
         jj=point/gdim(spfdof(n,m))
         fpoint(spfdof(n,m))=point-jj*gdim(spfdof(n,m))+1
         point=jj
      enddo

      return
      end subroutine decoderm

!#######################################################################
!
! phix(J) is the vector of configurations at a point, <x | Phi_J>
! igpoint(m) is the grid point for mode m
!
!#######################################################################

      subroutine phipnt(psi,phix,igpoint,s,block1)

      implicit none

      integer(long)                                      :: m 
      integer(long), intent(in)                          :: s,block1
      integer(long), dimension(maxdim)                   :: igpoint
      complex(dop), dimension(block1), intent(out)       :: phix
      complex(dop), dimension(adim+1:dgldim), intent(in) :: psi

      call initvxz(phix,block1)
      do m=1,nmode
         call spfpnt(psi(zetf(m,s)),subdim(m),phix,vdim(m,s),dim(m,s), &
                     ndim(m,s),igpoint(m))
      enddo

      return
      end subroutine phipnt

!#######################################################################
!
! phix(J) is the vector of configurations at a point, <x | Phi_J>,
!         with a "hole" in mode m1
! igpoint(m) is the grid point for mode m
!
!#######################################################################

      subroutine phipnt1(psi,phix,igpoint,m1,s,block1)

      implicit none

      integer(long)                                      :: m
      integer(long), intent(in)                          :: s,block1,m1
      integer(long), dimension(maxdim), intent(in)       :: igpoint
      complex(dop), dimension(block1), intent(out)       :: phix
      complex(dop), dimension(adim+1:dgldim), intent(in) :: psi

      call initvxz(phix,block1)
      do m=1,m1-1
         call spfpnt(psi(zetf(m,s)),subdim(m),phix,vdim(m,s),dim(m,s), &
                     ndim(m,s),igpoint(m))
      enddo
      do m=m1+1,nmode
         call spfpnt(psi(zetf(m,s)),subdim(m),phix,vdim(m,s),dim(m,s), &
                     ndim(m,s),igpoint(m))
      enddo

      return
      end subroutine phipnt1

!#######################################################################

      subroutine spfpnt(psi,subdim,phix,vdim,dim,ndim,g)

      implicit none

      integer(long)                                         :: v,e,n
      integer(long), intent(in)                             :: subdim,vdim,dim,ndim,g
      complex(dop), dimension(subdim,dim), intent(in)       :: psi
      complex(dop), dimension(vdim,dim,ndim), intent(inout) :: phix

      do v=1,vdim
         do e=1,dim
            do n=1,ndim
               phix(v,e,n)=phix(v,e,n)*psi(g,e)
            enddo
         enddo
      enddo

      return
      end subroutine spfpnt

!#######################################################################

      subroutine calcpsi(psi,psiad,psidia,psiarg,rotmat,igpoint,&
                 gpoint,agmat,psipnt,f1,f2,weight)

      implicit none

      integer(long)                                      :: s,m,b,n,f
      integer(long), intent(in)                          :: f1,f2
      integer(long), dimension(maxdim), intent(in)       :: igpoint
      real(dop), dimension(maxdim), intent(in)           :: gpoint
      real(dop), dimension(maxsta), intent(out)          :: psiad,psidia,psiarg
      real(dop), dimension(ortdim), intent(in)           :: weight
      complex(dop), dimension(agmatdim), intent(in)      :: agmat
      complex(dop), dimension(maxsta), intent(out)       :: psipnt
      complex(dop), dimension(dgldim), intent(in)        :: psi
      complex(dop), dimension(dgldim)                    :: workc
      complex(dop), dimension(maxspf)                    :: spfpnt
      complex(dop), dimension(maxsta)                    :: sumdia,sumad 
      complex(dop), dimension(maxsta,maxsta), intent(in) :: rotmat

      call zerovxd(psidia,maxsta)
      call cpvxz(psi,workc,dgldim)

!-----------------------------------------------------------------------
! multiply A vector by spfs at chosen point
!-----------------------------------------------------------------------
      do s=1,nstate
         do m=1,nmode
            if (gwpm(m)) then
               call gwppoint(m,gpoint,spfpnt,agmat(zagmat(m,s)),&
                    agmat(zxivec(m,s)),agmat(zphase(m,s)),nspfdof(m),&
                    dim(m,s),f1,f2,weight)
            else
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  call spfpoint(igpoint(f),workc(zetf(m,s)),vgdim(f), &
                       gdim(f),ngdim(f),dim(m,s))
               enddo
               call getspfpnt(workc(zetf(m,s)),spfpnt,subdim(m),dim(m,s))
            endif

            call dtxxzzo(spfpnt,workc(zpsi(s)),vdim(m,s),dim(m,s), &
                 ndim(m,s))
         enddo
      enddo

!-----------------------------------------------------------------------
! sum wavefunction terms
!-----------------------------------------------------------------------
      do s=1,nstate
         sumdia(s)=0.0_dop
         do b=1,block(s)
            sumdia(s)=sumdia(s)+workc(zpsi(s)+b-1)
         enddo
      enddo

!-----------------------------------------------------------------------
! transform to adiabatic, and get absolute value
!-----------------------------------------------------------------------
      if (nstate .gt. 1) then
         call qvaxzz1(rotmat,sumdia,sumad,maxsta,nstate)
      else
         call cpvxz(sumdia,sumad,nstate)
      endif
      do s=1,nstate
         psipnt(s)=sumdia(s)
         psiad(s)=dble(dconjg(sumad(s))*sumad(s))
         psidia(s)=dble(dconjg(sumdia(s))*sumdia(s))
         psiarg(s)=dble(datan2(dimag(sumdia(s)),dble(sumdia(s))))
         if (psiad(s)  .lt. 1.0e-16_dop) psiad(s)=0.0_dop
         if (psidia(s) .lt. 1.0e-16_dop) psidia(s)=0.0_dop
      enddo

      return
      end subroutine calcpsi

!--------------------------------------------------------------------
! Subroutine to calculate the wavefunction at a gridpoint on a 
! particular state (diabatic only).
! Modified from calcpsi.
! GWR 02/21
!--------------------------------------------------------------------
      subroutine calcpsis(psi,psidia,psiarg,psipnt,igpoint,s)

      implicit none

      integer(long), intent(in)                          :: s
      integer(long)                                      :: m,b,n,f
      integer(long), dimension(maxdim), intent(in)       :: igpoint
      real(dop), intent(out)                             :: psidia,psiarg
      complex(dop), dimension(dgldim), intent(in)        :: psi
      complex(dop), dimension(dgldim)                    :: workc
      complex(dop), dimension(maxspf)                    :: spfpnt
      complex(dop)                                       :: sumdia 
      complex(dop), intent(out)                          :: psipnt

      psidia = 0.0_dop
!      call zerovxd(psidia,maxsta)
      call cpvxz(psi,workc,dgldim)

!-----------------------------------------------------------------------
! multiply A vector by spfs at chosen point
!-----------------------------------------------------------------------
      do m=1,nmode
         if (gwpm(m)) then
!            if (nmode .gt. 1) then
            print*,'ERROR: only works for grid-based'
            stop
!            endif
!            call gwppoint(gpoint,spfpnt,agmat(zagmat(m,s)),&
!                 agmat(zxivec(m,s)),agmat(zphase(m,s)),nspfdof(m),&
!                 dim(m,s))
         else
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               call spfpoint(igpoint(f),workc(zetf(m,s)),vgdim(f), &
                    gdim(f),ngdim(f),dim(m,s))
            enddo
            call getspfpnt(workc(zetf(m,s)),spfpnt,subdim(m),dim(m,s))
         endif
         
         call dtxxzzo(spfpnt,workc(zpsi(s)),vdim(m,s),dim(m,s), &
              ndim(m,s))
      enddo

!-----------------------------------------------------------------------
! sum wavefunction terms
!-----------------------------------------------------------------------
      sumdia=0.0_dop
      do b=1,block(s)
         sumdia=sumdia+workc(zpsi(s)+b-1)
      enddo

!-----------------------------------------------------------------------
! transform to adiabatic, and get absolute value
!-----------------------------------------------------------------------
      psipnt=sumdia
      psidia=dble(dconjg(sumdia)*sumdia)
      psiarg=dble(datan2(dimag(sumdia),dble(sumdia)))
      if (psidia .lt. 1.0e-24_dop) psidia=0.0_dop

      return
      end subroutine calcpsis

!#######################################################################

      subroutine spfpoint(g1,psi,vgdim,gdim,ngdim,dim)

      implicit none

      integer(long)                                              :: g,v,n,e
      integer(long), intent(in)                                  :: vgdim,gdim,ngdim,dim,g1
      complex(dop), dimension(vgdim,gdim,ngdim,dim), intent(out) :: psi

      do e=1,dim
         do n=1,ngdim
            do g=1,gdim
               do v=1,vgdim
                  if (g .ne. g1) psi(v,g,n,e)=0.0_dop
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine spfpoint

!#######################################################################

      subroutine gwppoint(m1,gpoint,gwppnt,agmat,xivec,phase,gdof,dim1,&
                 f1,f2,weight)

      implicit none

      integer(long), intent(in)                           :: dim1,gdof,m1
      integer(long), intent(in)                          :: f1,f2
      integer(long)                                       :: e,f,n
      real(dop)                                           :: dtmp,ztmp
      real(dop), dimension(maxdim), intent(in)            :: gpoint
      real(dop), dimension(ortdim), intent(in)            :: weight
      complex(dop), dimension(dim1), intent(out)          :: gwppnt
      complex(dop), dimension(dim1), intent(in)           :: phase
      complex(dop), dimension(gdof,dim1), intent(in)      :: xivec
      complex(dop), dimension(gdof,gdof,dim1), intent(in) :: agmat

      if (gwptype .lt. 3) then
         print*,'ERROR: Can only plot WF for separable GWPs '
         return
      endif

      do e=1,dim1
         dtmp=dble(phase(e))
         ztmp=dimag(phase(e))
         gwppnt(e)=dexp(dtmp)*(dcos(ztmp) + (0,1)*dsin(ztmp))

         do n = 1,gdof
            f = spfdof(n,m1)
            dtmp=dble(xivec(n,e))*gpoint(f)
            ztmp=dimag(xivec(n,e))*gpoint(f)
            gwppnt(e)=gwppnt(e)*dexp(dtmp)*(dcos(ztmp) + (0,1)*dsin(ztmp))

            dtmp=dble(agmat(n,n,e))*gpoint(f)*gpoint(f)
            ztmp=dimag(agmat(n,n,e))*gpoint(f)*gpoint(f)
            gwppnt(e)=gwppnt(e)*dexp(dtmp)*(dcos(ztmp) + (0,1)*dsin(ztmp))
         enddo

! multiply by DVR weight for grid point. All weights are identical - take first.
         gwppnt(e)=gwppnt(e)*weight(zort(f1))
         if (f2 .ne. 0) gwppnt(e)=gwppnt(e)*weight(zort(f2))

      enddo

      return
      end subroutine gwppoint

!#######################################################################

      subroutine getspfpnt(psi,spfpnt,subdim,dim)

      implicit none

      integer(long)                                    :: e,g,g1
      integer(long), intent(in)                        :: subdim,dim
      real(dop)                                        :: psi2
      complex(dop), dimension(dim), intent(out)        :: spfpnt
      complex(dop), dimension(subdim,dim), intent(in)  :: psi
      complex(dop)                                     :: ztmp

      g1=0
      do g=1,subdim
         psi2=0.d0
         do e=1,dim
            ztmp = dconjg(psi(g,e))*psi(g,e)
            psi2=psi2+dble(ztmp)
         enddo
         if (psi2 .gt. 1.0e-10_dop) g1=g
      enddo

      if (g1 .gt.0) then
         do e=1,dim
            spfpnt(e)=psi(g1,e)
         enddo
      else
         do e=1,dim
            spfpnt(e)=0.0_dop
         enddo
      endif

      return
      end subroutine getspfpnt

!#######################################################################
!
!               SUBROUTINE ECALCD2D
! Computes the two-dimensional (local) pair-correlation from the
! wavefunction. Called from edengen.   (ET+SZ 2009)
!
!#######################################################################

      subroutine ecalcd2d(psi,epsidia,f1,f2,gpoint,spfpnt)

      implicit none

      integer(long), intent(in)                   :: f1,f2
      integer(long), dimension(:), intent(in)     :: gpoint
      integer(long) :: ngdim1,m1,m2,s,f,n,zeig1,zeig2,zeig3
      real(dop), dimension(:), intent(out)        ::  epsidia
      complex(dop), dimension(dgldim), intent(in) :: psi(dgldim)
      complex(dop), dimension(2*maxblock)                   :: workc
      complex(dop), dimension(maxspf*maxspf,2), intent(out) :: spfpnt
      complex(dop) :: sum2

      epsidia(:) = 0.0_dop
      spfpnt(:,:) = 0.0_dop

!-----------------------------------------------------------------------
! see 1D cut, or if f1, f2 are in same mode
!-----------------------------------------------------------------------
      m1=dofspf(f1)
      f=0
      if (f2 .eq. 0) then
         m2=0
      else
         m2=dofspf(f2)
! f indicates whether f1 or f2 is first in mode
         if (m1 .eq. m2) then
            f=0
            do n=1,nspfdof(m1)
               if (f .eq. 0) then
                  if (spfdof(n,m1) .eq. f1) then
                     f=f1
                  else if (spfdof(n,m1) .eq. f2) then
                     f=f2
                  endif
               endif
            enddo
         endif
      endif

!-----------------------------------------------------------------------
! get spf "density"
!-----------------------------------------------------------------------
      do s=1,nstate
         if (m2 .ne. m1) then
            call espfden2d(psi(zetf(m1,s)),spfpnt(1,1),vgdim(f1), &
                 gdim(f1),ngdim(f1),1,1,gpoint(f1),gpoint(f2),dim(m1,s))
            if (m2 .ne. 0) then
               call espfden2d(psi(zetf(m2,s)),spfpnt(1,2),vgdim(f2), &
                    gdim(f2),ngdim(f2),1,1,gpoint(f1),gpoint(f2),dim(m2,s))
            endif
         else if (m2 .eq. m1) then
            if (f .eq. f1) then
               ngdim1=ngdim(f1)/(gdim(f2)*ngdim(f2))
               call espfden2d(psi(zetf(m1,s)),spfpnt(1,1),vgdim(f1), &
                    gdim(f1),ngdim1,gdim(f2),ngdim(f2),gpoint(f1), &
                    gpoint(f2),dim(m2,s))
            else if (f .eq. f2) then
               ngdim1=ngdim(f2)/(gdim(f1)*ngdim(f1))
               call espfden2d(psi(zetf(m2,s)),spfpnt(1,1),vgdim(f2), &
                    gdim(f2),ngdim1,gdim(f1),ngdim(f1),gpoint(f2), &
                    gpoint(f1),dim(m2,s))
            endif
         endif

!-----------------------------------------------------------------------
! multiply A vector by spfs at chosen point
!-----------------------------------------------------------------------
         zeig1 = block(s)+1
         zeig2 = 1
         call cpvxz(psi(zpsi(s)),workc(zeig1),block(s))

         call qtxxzz(spfpnt(1,1),workc(zeig1),workc(zeig2),vdim(m1,s), &
              dim(m1,s),ndim(m1,s))
         if (m2 .ne. 0 .and. m1 .ne. m2) then
            zeig3=zeig2
            zeig2=zeig1
            zeig1=zeig3
            call qtxxzz(spfpnt(1,2),workc(zeig1),workc(zeig2), &
                 vdim(m2,s),dim(m2,s),ndim(m2,s))
         endif

         call vvaxzz(psi(zpsi(s)),workc(zeig2),sum2,block(s))

         if (dble(sum2) .lt. 1.d-12) sum2=0.0d0
         epsidia(s)=dble(sum2)
      enddo

      return
      end subroutine ecalcd2d

!#######################################################################
!
!            SUBROUTINE ESPFDEN2D
! Computes the density on the SPF level. Called from ECALCD2D.
! ET+SZ 2009
!#######################################################################

      subroutine espfden2d(psi,den2d,vgdim1,gdim1,ngdim1,gdim2,ngdim2, &
                 g1,g2,dim)

      implicit none

      integer(long), intent(in) :: vgdim1,gdim1,ngdim1,gdim2,ngdim2,g1,g2,dim
      integer(long) :: v1,n1,n2,e,e1
      complex(dop), dimension (vgdim1,gdim1,ngdim1,gdim2,ngdim2,dim), intent(in) :: psi
      complex(dop), dimension (dim,dim), intent(out) :: den2d

      do e=1,dim
         do e1=1,dim
            den2d(e1,e)=0.0d0
            do n2=1,ngdim2
               do n1=1,ngdim1
                  do v1=1,vgdim1
                     den2d(e1,e)=den2d(e1,e)+ &
                  dconjg(psi(v1,g1,n1,1,n2,e1))*psi(v1,g2,n1,1,n2,e)
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine espfden2d

!#######################################################################
!
!             SUBROUTINE GETWEIGHTS
!
! computes the DVR weights from trafo
! lsq  : if true, square of DVR weights are returned
! lpop2: if true, determine 2nd-pop grids and (for FFT and exp) weights
!
!#######################################################################

      subroutine getweight(trafo,weight,ort,lsq)

      implicit none

      integer(long)       :: gdim1,ggdim,sphtyp,jmax,gj,mmax,mincr,mmin,gm,dj,&
                             j0,m,n,f,p1,kmin,kmax,kmin2,dk,g,ibleg,&
                             mmmin,mmmax,mmmin2,dm,p2

      real(dop)                                 :: dp
      real(dop), dimension(dvrdim), intent(in)  :: trafo
      real(dop), dimension(ortdim), intent(out) :: weight,ort
      logical(kind=4), intent(in)               :: lsq

      ibleg=0
!-----------------------------------------------------------------------
! Special case. lpop2=.true., second population plotted (e.g momenta).
!-----------------------------------------------------------------------
      if(lpop2) then
         do gj=1,ortdim
            weight(gj) = 1.0_dop
         enddo
         do f=1, ndof
! -  HO  -
            if( basis(f) .eq. 1  ) then
               do g = 1, gdim(f)
                  ort(zort(f)-1+g) = g-1
               end do
! -  Leg  Leg/R  Kleg KLeg/R  PLeg Wigner -
            elseif( basis(f) .eq. 2 .or. basis(f) .eq. 7 &
                    .or. basis(f) .eq. 9 .or. basis(f) .eq. 12  &
                    .or. basis(f) .eq. 20 .or. basis(f) .eq. 21 &
                    .or. basis(f) .eq. 33) then
               ggdim = gdim(f)
               if(basis(f) .eq. 12 .or. basis(f) .eq. 20 .or. basis(f)  &
                           .eq. 21) ggdim = ipbaspar(3,f)
               if(basis(f).eq.2 .or. basis(f).eq.12) ibleg=ipbaspar(2,f)
               if(basis(f).eq.7 .or. basis(f).eq.9 .or. &
                  basis(f).eq.20.or. basis(f).eq.21 .or. &
                  basis(f) .eq. 33) ibleg=ipbaspar(1,f)
               if(ibleg .eq. 0 ) then
                  do g = 1, ggdim
                     ort(zort(f)-1+g) = g-1
                  end do
               else
                  do g = 1, ggdim
                     ort(zort(f)-1+g) = g*2 - ibleg
                  end do
               endif
! -  FFT -
            elseif( basis(f) .eq. 4  ) then
               dp = 2.0_dop*pi/( (ort(zort(f)+1)-ort(zort(f)))*gdim(f) )
               gm = gdim(f)/2 + 1
               do g = 1, gdim(f)
                  ort(zort(f)-1+g) = (g-gm)*dp
                  if(lsq) then
                     weight(zort(f)-1+g) = dp
                  else
                     weight(zort(f)-1+g) = sqrt(dp)
                  endif
               end do
! -  exp -
            elseif( basis(f) .eq. 5  ) then
               dp = 2.0_dop*pi/( (ort(zort(f)+1)-ort(zort(f)))*gdim(f) )
               do g = 1, gdim(f)
                  ort(zort(f)-1+g) = dp*(g-gdim(f)/2-1)
                  if(lsq) then
                     weight(zort(f)-1+g) = dp
                  else
                     weight(zort(f)-1+g) = sqrt(dp)
                  endif
               end do
! - rest -
            else
               do g = 1, gdim(f)
                  ort(zort(f)-1+g) = g
               end do
            end if
         enddo
         return                  !  RETURN
      endif

!-----------------------------------------------------------------------
! Determine the DVR weights.
!-----------------------------------------------------------------------
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)

            gdim1=gdim(f)
            ggdim=gdim(f)
            if(basis(f) .eq. 12 .or. basis(f) .eq. 20 .or.  &
               basis(f) .eq. 21)  &
               ggdim = ipbaspar(3,f)

! Special handling for sphFBR
            if( basis(f) .eq. 6 .or. basis(f) .eq. -1 ) then
               if(basis(f) .eq. -1 ) f = f-1
               sphtyp = ipbaspar(1,f)
               jmax   = ipbaspar(2,f)
               gj     = ipbaspar(4,f)
               mmax   = ipbaspar(1,f+1)
               mincr  = ipbaspar(2,f+1)
               mmin   = ipbaspar(3,f+1)
               gm = (mmax-mmin)/mincr + 1
               dj = 1
               if( sphtyp.eq.2  .or. sphtyp.eq.3 ) dj = 2
               j0 = mod(jmax,dj)
               if(basis(f) .eq. -1 ) then
                  gdim1 = gm
                  ggdim = gm
                  do g = 1, gdim1
                     ort(zort(f)-1+g) = mmin + (g-1)*mincr
                  end do
               else
                  gdim1 = gj
                  ggdim = gj
                  do g = 1, gdim1
                     ort(zort(f)-1+g) = j0 + (g-1)*dj
                  end do
               end if
            end if

!...Determine the appropriate 'column' of trafo(gdim,gdim,kdim) for KLeg/PLeg
            p1=0
            if( basis(f) .eq. 7 .or. basis(f) .eq. 20 ) then    ! KLeg
               kmin=ipbaspar(1,f+1)
               kmax=ipbaspar(2,f+1)
               kmin2=ipbaspar(3,f+1)
               p1=(min(kmax,kmin2)-kmin)
               dk=max(ipbaspar(5,f+1),1)
               p1=(p1+dk-1)/dk
!               p1=p1*gdim(f)**2
               p1=p1*ggdim**2
            elseif( basis(f) .eq. 9 .or. basis(f) .eq. 21) then     ! PLeg
!               kmin=ipbaspar(2,f)
!               kmax=ipbaspar(3,f)
!               kmin2=ipbaspar(5,f)

               kmin=ipbaspar(1,f+1)
               kmax=ipbaspar(2,f+1)
               kmin2=ipbaspar(4,f+1)
!               p1=(min(kmax,kmin2)-kmin)*gdim(f)**2
               p1=(min(kmax,kmin2)-kmin)*ggdim**2
            elseif( basis(f) .eq. 33 ) then           ! Wigner
               kmin=ipbaspar(2,f)
               kmax=ipbaspar(3,f)
               kmin2=ipbaspar(5,f)
               p2=min(kmax,kmin2)-kmin
               if (basis(f+1).eq.8) then
                  dk=ipbaspar(5,f+1)
                  p2=(p2+dk-1)/dk
               endif
               p2=p2*gdim(f)**2
               mmmin=ipbaspar(6,f)
               mmmax=ipbaspar(7,f)
               mmmin2=ipbaspar(9,f)
               p1=min(mmmax,mmmin2)-mmmin
               if (basis(f+2).eq.8) then
                  dm=ipbaspar(5,f+2)
                  p1=(p1+dm-1)/dm
               endif
               p1=p1*ipbaspar(4,f)*gdim(f)**2 + p2
            endif

! Determine the DVR weights.
            call dvrweights(trafo(zdvr(f)+p1),ort(zort(f)),gdim1, &
                    ggdim,weight(zort(f)),basis(f),rpbaspar(1,f), &
                    ipbaspar(1,f),ipbaspar(1,f+1),lsq,lnw)

         enddo
      enddo

      return
      end subroutine getweight

!#######################################################################

      subroutine showpop2(psi,trafo,exphin,rueck,fftfak,f1,f2)

!.....This routine transforms to the second population.
!.....HDM 03/04

      implicit none

      integer(long)                                   :: ggdim,s,f,ff,m,kmin,&
                                                         kdim,bf,bff,mm,n
      integer(long), intent(in)                       :: f1,f2
      integer(long), dimension(fftdim), intent(in)    :: fftfak
      real(dop), dimension(dvrdim), intent(in)        :: trafo
      complex(dop), dimension(dgldim), intent(inout)  :: psi
      complex(dop), dimension(sdim*maxspf)            :: workc
      complex(dop), dimension(fftdim), intent(in)     :: rueck
      complex(dop), dimension(expdim), intent(in)     :: exphin
      logical(kind=4)                                 :: ldummy
      
!-----------------------------------------------------------------------
! Transform single-particle functions to second representation
!-----------------------------------------------------------------------
      bf = -99
      bff= -99
      if(f2.gt.0) bff=basis(f2)
      f  = f1
      ff = f2

  99  if( f .gt. 0 ) then
         m=dofspf(f)
         bf=basis(f)

         ldummy=.true.
         do mm=1, nmode
            do n = 1, nspfdof(mm)
               if(spfdof(n,mm).eq.f) ldummy=.false.
            enddo
         enddo
         if(ldummy .or. bf.eq.0 .or. bf.eq.-1 .or. &
                    bf.eq.6 .or.  bf.eq.8 .or. bf.eq.13) then
            continue ! ignore, do not transform: phifbr,sphfbr,k,external
            write(2,'(a,i3,a,i3)') &
                 'POP2 IGNORED! ; f =', f,', basis =', bf

         elseif( bf .eq. 4 ) then           !  FFT
            do s = 1, nstate
               call fft(psi(zetf(m,s)),workc,rueck(zfft(f)), &
                        fftfak(zfft(f)),gdim(f),vgdim(f), &
                        ngdim(f)*dim(m,s),.true.)
               if(lswap) then
                  call swapfft(psi(zetf(m,s)),workc,gdim(f),vgdim(f),&
                               ngdim(f),dim(m,s))
               end if
            enddo 

         elseif( bf .eq. 5 ) then           ! exp
! Do nothing in case of PLeg. The phi->K transformation will be done there.   
            if( (bff.eq.9 .or. bff.eq.21) .and. f.eq.ff+1 .or. &
                (bff.eq.33 .and. f.eq.ff+1) .or. &
                (bff.eq.33 .and. f.eq.ff+2) ) goto 80

            do s = 1, nstate
               call cpmxz(psi(zetf(m,s)),workc,subdim(m),dim(m,s))
               if (f.gt.1 .and. (basis(max(1,f-1)).eq.9 .or. &
                   basis(max(1,f-1)).eq.21)) then
                  kmin=ipbaspar(1,f)
                  kdim=ipbaspar(3,f)
               elseif (f.gt.1 .and. basis(max(1,f-1)).eq.33) then
                  kmin=ipbaspar(2,f-1)
                  kdim=ipbaspar(4,f-1)
               elseif (f.gt.2 .and. basis(max(1,f-2)).eq.33) then
                  kmin=ipbaspar(6,f-2)
                  kdim=ipbaspar(8,f-2)
               else
                  kmin=-(gdim(f)/2)
                  kdim=gdim(f)
               endif
               call qtexp(exphin(zexp(f)),workc,psi(zetf(m,s)), &
                    vgdim(f),gdim(f),ngdim(f)*dim(m,s),kdim,kmin)
            enddo

         elseif( bf .eq. 7 .or. bf .eq. 20) then      ! KLeg KLeg/R
! 1st, copy psi-->workc, then transform workc-->psi
            ggdim = gdim(f)                           
            if(bf .eq. 20) ggdim = ipbaspar(3,f)
            do s = 1, nstate
               call cpmxz(psi(zetf(m,s)),workc,subdim(m),dim(m,s))
               call trkleg(trafo(zdvr(f)),workc,psi(zetf(m,s)),ggdim, &
                    ipbaspar(1,f),ipbaspar(1,f+1),ipbaspar(3,f+1), &
                    vgdim(f),gdim(f),gdim(f+1),ngdim(f+1),dim(m,s))
            enddo 

         elseif( bf .eq. 9 .or. bf .eq. 21) then           ! PLeg PLeg/R
!psi(theta,phi)-->workc(theta,K) then transform workc-->psi
            ggdim = gdim(f)
            if(bf .eq. 21) ggdim = ipbaspar(3,f)
            do s = 1, nstate
               call pleghin(psi(zetf(m,s)),exphin(zexp(f+1)),workc, &
                    vgdim(f+1),gdim(f+1),ngdim(f+1),ipbaspar(3,f+1), &
                    dim(m,s))
               call trkleg(trafo(zdvr(f)),workc,psi(zetf(m,s)),ggdim, &
                  ipbaspar(1,f),ipbaspar(1,f+1),ipbaspar(4,f+1), &
                  vgdim(f),gdim(f),ipbaspar(3,f+1),ngdim(f+1),dim(m,s))
            enddo 

        elseif( bf .eq. 12 ) then           ! Leg/r
            do s = 1, nstate
               call cpmxz(psi(zetf(m,s)),workc,subdim(m),dim(m,s))
               call mtxxdz(trafo(zdvr(f)),workc,psi(zetf(m,s)), &
                    vgdim(f),gdim(f),ngdim(f)*dim(m,s),ipbaspar(3,f))
            enddo 

         elseif(bf.eq.33) then  ! Wigner
            do s = 1, nstate
               if (basis(f+1).eq.5 .and. basis(f+2).eq.5) then
                  call cpmxz(psi(zetf(m,s)),workc,subdim(m),dim(m,s))
                  call pleghin(workc,exphin(zexp(f+1)),psi(zetf(m,s)), &
                       vgdim(f+1),gdim(f+1),ngdim(f+1),ipbaspar(4,f), &
                       dim(m,s))
                  call pleghin(psi(zetf(m,s)),exphin(zexp(f+2)),workc, &
                       vgdim(f+1)*ipbaspar(4,f),gdim(f+2),ngdim(f+2), &
                       ipbaspar(8,f),dim(m,s))
               elseif (basis(f+1).eq.5 .and. basis(f+2).eq.8) then
                  call pleghin(psi(zetf(m,s)),exphin(zexp(f+1)),workc, &
                       vgdim(f+1),gdim(f+1),ngdim(f+1),ipbaspar(4,f), &
                       dim(m,s))
               elseif (basis(f+1).eq.8 .and. basis(f+2).eq.5) then
                  call pleghin(psi(zetf(m,s)),exphin(zexp(f+2)),workc, &
                       vgdim(f+2),gdim(f+2),ngdim(f+2),ipbaspar(8,f), &
                       dim(m,s))
               elseif (basis(f+1).eq.8 .and. basis(f+2).eq.8) then
                  call cpmxz(psi(zetf(m,s)),workc,subdim(m),dim(m,s))
               endif
               call trwigner(trafo(zdvr(f)),workc,psi(zetf(m,s)), &
                    ipbaspar(1,f),ipbaspar(2,f),ipbaspar(5,f), &
                    ipbaspar(6,f),ipbaspar(9,f),vgdim(f),gdim(f), &
                    ipbaspar(4,f),ipbaspar(8,f),ngdim(f+2),dim(m,s))
            enddo


         elseif (ldvr(f)) then              ! general DVR
            do s = 1, nstate
               call cpmxz(psi(zetf(m,s)),workc,subdim(m),dim(m,s))
               call qtxxdz(trafo(zdvr(f)),workc,psi(zetf(m,s)), &
                           vgdim(f),gdim(f),ngdim(f)*dim(m,s))
            enddo
         else
            write(6,'(a,i4)') ' pop2 not implemented for basis:',bf
         endif  
      endif

 80   if( f.eq.f2 ) return                  ! RETURN
      bff = bf
      f   = f2
      ff  = f1

      goto 99

      end subroutine showpop2

!#######################################################################
! TRKLEG
!
! called by dichtout
! For "Kleg-DVR".
!
! input:
!       trafo - transformation matrix
!       psi   - wave function
! output:
!       trpsi - wave function transformed to the j-grid
!
! 11/99 SS
! when KLEG/R, ggdim is the full grid length and gdim the reduced.
! For full KLEG, these are equal
!
! BUG??
! Not sure this routine is correct for KLEG/R as trafo / psi indices
! may not match GAW (7/10)
!#######################################################################

      subroutine trkleg(trafo,psi,trpsi,ggdim,ibleg,kmin,kmin2,vgdim, &
                        gdim,kdim,ngdim,dim)

      implicit none

      integer(long)             :: g,j,m,v,n,e,m1,j1
      integer(long), intent(in) :: vgdim,gdim,ggdim,kdim,ngdim,dim,kmin,kmin2,ibleg

      real(dop), dimension(ggdim,ggdim,kdim), intent(in)               :: trafo
      complex(dop), dimension(vgdim,gdim,kdim,ngdim,dim), intent(in)   :: psi
      complex(dop), dimension(vgdim,ggdim,kdim,ngdim,dim), intent(out) :: trpsi


      call zerovxz(trpsi,vgdim*ggdim*kdim*ngdim*dim)
      do e=1,dim
         do n=1,ngdim
            do j=1,ggdim
               do m=1,kdim
                  m1=m+kmin-1
                  if (ibleg.eq.0) then
                     j1 = j + abs(m1) - kmin2
                  else
                     j1 = j + (abs(m1)+ibleg-1)/2 - (kmin2+ibleg-1)/2
                  endif
                  if (j1 .le. ggdim) then
                     do g=1,gdim
                        do v=1,vgdim
                           trpsi(v,j1,m,n,e) = trpsi(v,j1,m,n,e) + &
                                             trafo(j,g,m)*psi(v,g,m,n,e)
                        enddo
                     enddo
                  endif
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine trkleg

!#######################################################################
! TRWIGNER
!
! same as TRKLEG, but for Wigner functions
!
! input:  trafo - transformation matrix
!         psi   - wavefunction
! output: trpsi - wavefunction transformed to the j-grid
!
! 03/13 PT
!#######################################################################

      subroutine trwigner(trafo,psi,trpsi,ibleg,kmin,kmin2,mmmin,mmmin2, &
                          vgdim,gdim,kdim,mmdim,ngdim,dim)

      implicit none

      integer(long), intent(in)   :: vgdim,gdim,kdim,mmdim,ngdim,dim, &
                                     kmin,kmin2,mmmin,mmmin2,ibleg
      integer(long)               :: g,j,m,j1,m1,mm,v,n,e,j0,mm1,mormm,mormm2
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)               :: trafo
      complex(dop),dimension(vgdim,gdim,kdim,mmdim,ngdim,dim), intent(in)  :: psi
      complex(dop),dimension(vgdim,gdim,kdim,mmdim,ngdim,dim), intent(out) :: trpsi

      mormm2=max(kmin2,mmmin2)
      call zerovxz(trpsi,vgdim*gdim*kdim*mmdim*ngdim*dim)
      do e=1,dim
         do n=1,ngdim
            do j=1,gdim
               do mm=1,mmdim
                  mm1=mm+mmmin-1
                  do m=1,kdim
                     m1=m+kmin-1
                     mormm=max(abs(m1),abs(mm1))
                     if (ibleg.eq.0) then
                        j1=j+mormm-mormm2
                        j0=j1-1 ! Actual j-value (for testing)
                     else
                        j1=j+(mormm+ibleg-1)/2-(mormm2+ibleg-1)/2
                        j0=2*j1-ibleg ! Actual j-value (for testing)
                     endif
                     if (m1.gt.j0 .or. mm1.gt.j0) call errormsg
                     if (j1.le.gdim) then
                        do g=1,gdim
                           do v=1,vgdim
                              trpsi(v,j1,m,mm,n,e) = &
                                    trpsi(v,j1,m,mm,n,e) + &
                                    trafo(j,g,m,mm)*psi(v,g,m,mm,n,e)
                           enddo
                        enddo
                     endif
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine trwigner

!#######################################################################

      subroutine qtexp (a,b,c,dim1,dim2,dim3,kdim,kmin)

!.....Special routine to transform exp-DVR from grid to k-space.
!.....call qtexp(exphin,psi-ort,psi-k,vgdim,gdim,ngdim*dim,kdim,kmin)

      implicit none

      integer(long)                                        :: i,j,g,m,k,dif,kmax
      integer(long), intent(in)                            :: dim1,dim2,dim3,kdim,kmin
      complex(dop), dimension(kdim,dim2), intent(in)       :: a
      complex(dop), dimension(dim1,dim2,dim3), intent(in)  :: b
      complex(dop), dimension(dim1,dim2,dim3), intent(out) :: c

      m=dim2/2
      if (kdim.lt.dim2) then
         kmax=kmin+kdim-1
         do g=1,dim2
            k=g-1-m
            if (k.lt.kmin .or. k.gt.kmax) then
               do j=1,dim3
                  do i=1,dim1
                     c(i,g,j)=(0.0_dop,0.0_dop)
                  enddo
               enddo
            endif
         enddo
      endif

      dif=m+kmin
      do j=1,dim3
         call mmxxzz(b(1,1,j),a,c(1,1+dif,j),dim1,dim2,kdim)
      enddo

      return
      end subroutine qtexp

!***********************************************************************
!
!            MCTDH  subroutine PLEGRUECK
!
!   Transforms KLeg INWF from (Theta,K)-grid to (Theta,Phi)-grid for
!   the combined-mode with (PLeg,Exp)-DVRs.
!
!***********************************************************************

      subroutine plegrueck(psik,exprueck,psi,vgdim,gdim,ngdim,kdim,dim)


      integer(long)                     :: e,ierr
      integer(long), intent(in)         :: vgdim,gdim,kdim,ngdim,dim
                                                       
      complex(dop), dimension(vgdim,gdim,ngdim,dim), intent(out) :: psi
      complex(dop), dimension(vgdim,kdim,ngdim,dim), intent(in)  :: psik
      complex(dop), dimension(kdim,gdim), intent(in)             :: exprueck

      do e=1,dim
         call mttxzz(exprueck,psik(1,1,1,e),psi(1,1,1,e),vgdim,kdim, &
                     ngdim,gdim)
      enddo

      call schmidtortho(psi,vgdim*gdim*ngdim,dim,ierr)

      return
      end subroutine plegrueck

!***********************************************************************
!
!            MCTDH  subroutine PLEGHIN
!
!   Transforms INWF from (Theta,Phi)-grid to (Theta,K)-grid for
!   the combined-mode with (PLeg,Exp)-DVRs.
!
!***********************************************************************

      subroutine pleghin(psi,exphin,psik,vgdim,gdim,ngdim,kdim,dim)


      integer(long), intent(in)                                  :: vgdim,gdim, &
                                                                    kdim,ngdim,dim
      integer(long)                                              :: e,ierr
      complex(dop), dimension(vgdim,gdim,ngdim,dim), intent(in)  :: psi
      complex(dop), dimension(vgdim,kdim,ngdim,dim), intent(out) :: psik
      complex(dop), dimension(gdim,kdim), intent(in)             :: exphin

      do e=1,dim
        call mttxzz(exphin,psi(1,1,1,e),psik(1,1,1,e),vgdim,gdim,ngdim, &
                    kdim)
      enddo

      call schmidtortho(psik,vgdim*kdim*ngdim,dim,ierr)

      return
      end subroutine pleghin

!#######################################################################
!
! forcing Hermiticity of Type I density matrices
!
!#######################################################################

      subroutine d1regular(psi,force)

      implicit none

      complex(dop), dimension(dgldim), intent(inout) :: psi
      logical(kind=4), intent(in)                    :: force

      integer(long), save                            :: ncall = 0
      integer(long)                                  :: s,t,m,d

      ncall = ncall + 1
      if ((ncall.lt.10).and.(.not. force)) return
      ncall = 0

! --- check Eq. 37 in Raab et al., JCP 112, 10718-10729 (2000) 
      do s=1,nstate
         do m=1,nmode
            do d=1,ddim(m,s,s)
!               call chkherm(psi(dzetf(m,s,s)+(d-1)*subdim(m)),error,
!     +              supgdim(m))
!               print*,'herm-error = ',error  
               call mkherm(psi(dzetf(m,s,s)+(d-1)*subdim(m)),supgdim(m))
            enddo
         enddo
         do t=(s+1),nstate
            do m=1,nmode
               do d=1,ddim(m,s,t)
                  call mkherm2(psi(dzetf(m,s,t)+(d-1)*subdim(m)), &
                       psi(dzetf(m,t,s)+(d-1)*subdim(m)), &
                       supgdim(m),supgdim(m))
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine d1regular


!#######################################################################
! MAXMATWK
!
! calculates the size of the work array needed in matrizen2
! i.e. the maximum length of A-vector required for calculating 
! inter-state mean-fields.
!
! Two versions depending on whether:
! maxmatwk0 for MCTDH and pMCTDH(II)
! maxmatwk1 for pMCTDH(I)
!
! V5.0 mb
!#######################################################################

      subroutine maxmatwk(nham)

      implicit none

      integer(long), intent(in) :: nham

      if (dentype .eq. 0 .or. dentype .eq. 2) then
         call maxmatwk0(nham)
      else if (dentype .eq. 1) then
         call maxmatwk1(nham)
      else 
         routine="Maxmatwk (Psiutil.f90)"
         message="Unknown density type for psi"
         call errormsg
      endif

      return
      end subroutine maxmatwk

!-----------------------------------------------------------------------

      subroutine maxmatwk0(nham)

      implicit none

      integer(long)             :: block1,m,m1,vdim1,s,s1,k,k1,k2,k3,h,&
                                   fblk,muld,f
      integer(long), intent(in) :: nham
      logical(kind=4)           :: first

      do s=1,nstate
         maxblock=max(maxblock,block(s))
      enddo

      if(nham.eq.0)  goto 200

      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
      do k=k1,k2
         s=kf(k)
         s1=ki(k)
         if (s.ne.s1) then
            do m=1,nmode
               if (m.ne.1) then
                  block1=vdim(m-1,s)*dim(m-1,s1)*ndim(m-1,s1)
                  maxblock=max(maxblock,block1)
                  block1=vdim(m-1,s)*dim(m-1,s)*ndim(m-1,s1)
                  maxblock=max(maxblock,block1)
               endif

               vdim1=vdim(m,s)*dim(m,s1)
               do m1=m+1,nmode
! before multiplication
                  block1=vdim1*dim(m1,s1)*ndim(m1,s1)
                  maxblock=max(maxblock,block1)
! after multiplication
                  block1=vdim1*dim(m1,s)*ndim(m1,s1)
                  maxblock=max(maxblock,block1)
                  vdim1=vdim1*dim(m1,s)
               enddo
            enddo
            m=nmode
            block1=vdim(m,s)*dim(m,s1)*ndim(m,s1)
            maxblock=max(maxblock,block1)
            block1=vdim(m,s)*dim(m,s)*ndim(m,s1)
            maxblock=max(maxblock,block1)
         endif
      enddo

      k=kzahl(nham)+zham(nham)-1
      do k3=1,nmulpot(nham)
         k=k+1
         s=kf(k)
         s1=ki(k)
         if (lcdvr.or.ltdvr) then
            do m=1,nmode
               block1=block(s1)/dim(m,s1)
               block1=block1*subdim(m)
               maxblock=max(maxblock,block1)
               block1=block(s)/dim(m,s)
               block1=block1*subdim(m)
               maxblock=max(maxblock,block1)
            enddo

         else            ! muld-potential
            f=fterm(k)
            h=hamilton(f,k)
            muld=htmmuld(h)
            if(muld.gt.0) goto 40
            routine='maxmatwk'
            message='No muld-potential found!'
            call errormsg
 40         first=.true.
            vdim1=1
            do m=1,nmode
               if (diag(m,k) .or. m .eq. meb) then    ! Does not work for nodiag
                  vdim1=vdim1*dim(m,s1)
               else
                  if(first) then
                     first=.false.
                     vdmuld(muld)=vdim1
                  endif
                  vdim1=vdim1*subdim(m)
               endif
            enddo
            maxblock=max(maxblock,vdim1)
            ndmuld(muld)=vdim1/(vdmuld(muld)*muldsize(muld))
         endif
      enddo

!-----------------------------------------------------------------------
! for Selected CI calculation, calculate full configuration space size
!-----------------------------------------------------------------------
 200  fmblock=maxblock
      if (citype .gt. 0) then
         do s=1,nstate
            fblk=1
            do m=1,nmode
               fblk=fblk*dim(m,s)
            enddo
            fmblock=max(fmblock,fblk)
         enddo
      endif

      return
      end subroutine maxmatwk0

!-----------------------------------------------------------------------

      subroutine maxmatwk1(nham)

      implicit none

      integer(long)             :: block1,m,m1,vdim1,s,s1,k,k1,k2,k3,h,&
                                   fblk,muld,f,t
      integer(long), intent(in) :: nham
      logical(kind=4)           :: first

      do s=1,nstate
         do t=1,nstate
            maxblock=max(maxblock,dblock(s,t))
         enddo
      enddo

      if(nham.eq.0)  goto 200

      do t=1,nstate
         k1=zham(nham)
         k2=zham(nham)+kzahl(nham)-1
         do k=k1,k2
            s=kf(k)
            s1=ki(k)
            if (s.ne.s1) then
               do m=1,nmode
                  if (m.ne.1) then
                     block1=vddim(m-1,s,t)*ddim(m-1,s1,t)*nddim(m-1,s1,t)
                     maxblock=max(maxblock,block1)
                     block1=vddim(m-1,s,t)*ddim(m-1,s,t)*nddim(m-1,s1,t)
                     maxblock=max(maxblock,block1)
                  endif

                  vdim1=vddim(m,s,t)*ddim(m,s1,t)
                  do m1=m+1,nmode
! before multiplication
                     block1=vdim1*ddim(m1,s1,t)*nddim(m1,s1,t)
                     maxblock=max(maxblock,block1)
! after multiplication
                     block1=vdim1*ddim(m1,s,t)*nddim(m1,s1,t)
                     maxblock=max(maxblock,block1)
                     vdim1=vdim1*ddim(m1,s,t)
                  enddo
               enddo
               m=nmode
               block1=vddim(m,s,t)*ddim(m,s1,t)*nddim(m,s1,t)
               maxblock=max(maxblock,block1)
               block1=vddim(m,s,t)*ddim(m,s,t)*nddim(m,s1,t)
               maxblock=max(maxblock,block1)
            endif
         enddo

         k=kzahl(nham)+zham(nham)-1
         do k3=1,nmulpot(nham)
            k=k+1
            s=kf(k)
            s1=ki(k)
            if (lcdvr.or.ltdvr) then
               do m=1,nmode
                  block1=block(s1)/ddim(m,s1,t)
                  block1=block1*subdim(m)
                  maxblock=max(maxblock,block1)
                  block1=block(s)/ddim(m,s,t)
                  block1=block1*subdim(m)
                  maxblock=max(maxblock,block1)
               enddo

            else            ! muld-potential
               f=fterm(k)
               h=hamilton(f,k)
               muld=htmmuld(h)
               if(muld.gt.0) goto 40
               routine='maxmatwk'
               message='No muld-potential found!'
               call errormsg
 40            first=.true.
               vdim1=1
               do m=1,nmode
                  if (diag(m,k)) then    ! Does not work for nodiag
                     vdim1=vdim1*ddim(m,s1,t)
                  else
                     if(first) then
                        first=.false.
                        vdmuld(muld)=vdim1
                     endif
                     vdim1=vdim1*subdim(m)
                  endif
               enddo
               maxblock=max(maxblock,vdim1)
               ndmuld(muld)=vdim1/(vdmuld(muld)*muldsize(muld))
            endif
         enddo
      enddo

 200  fmblock=maxblock
      if (citype .gt. 0) then
         do t=1,nstate
            do s=1,nstate
               fblk=1
               do m=1,nmode
                  fblk=fblk*ddim(m,s,t)
               enddo
               fmblock=max(fmblock,fblk)
            enddo
         enddo
      endif

      return
      end subroutine maxmatwk1

!#######################################################################
! f2mdecoder
!
! Transform grid-index of DOF into index of mode-grid
!
!#######################################################################

      subroutine f2mdecoder(fpoint,mpoint)

      implicit none

      integer(long), dimension(*), intent(in)      ::   fpoint
      integer(long), dimension(nmode), intent(out) ::   mpoint(nmode)
      integer(long) :: m,vdim1,f,n

      do m=1,nmode
         mpoint(m) = 0
         vdim1 = 1
         do n=1,nspfdof(m)
            f = spfdof(n,m)
            mpoint(m) = mpoint(m) + (fpoint(f)-1)*vdim1
            vdim1 = vdim1 * gdim(f)
         end do
         mpoint(m) = mpoint(m) + 1
      end do

      return
      end subroutine f2mdecoder

!#######################################################################
!
! Decomposes a grid index into the 3 tensor components v1,e1,n1
!
!#######################################################################

      subroutine vendecoder(v1,e1,n1,index,vdim1,dim1)

      implicit none

      integer(long), intent(in)  :: index,vdim1,dim1
      integer(long), intent(out) :: v1,e1,n1
      integer(long)              :: j

      j = index - 1
      n1 = j/(vdim1*dim1)
      j = j - n1*vdim1*dim1
      n1 = n1+1

      e1 = j/(vdim1)
      j = j - e1*vdim1
      e1 = e1+1

      v1 = j+1

      return
      end subroutine vendecoder

! **********************************************************************
! *                                                                    *
! *                           SUBROUTINE RENORMA                       *
! *                                                                    *
! * Renormalizes the MCTDH-coefficients in relaxation calculation.     *
! *                                                                    *
! * Input parameters:                                                  *
! *   psi: The MCTDH-coefficients vector.                              *
! *                                                                    *
! * Output parameters:                                                 *
! *   psi:  The normalised MCTDH-coefficients vector.                  *
! *                                                                    *
! * V6.0 MB                                                            *
! *                                                                    *
! **********************************************************************

      subroutine renorma (psi)

      implicit none

      real(dop)                                    :: norm
      complex(dop), dimension(adim), intent(inout) :: psi

! --- RENORMALISE MCTDH-COEFFICIENTS ---

      call normvxz(psi,norm,adim)
      if(lmulpack) norm=norm/sqrt(dble(npacket))
      if(mpb.gt.0) norm=norm/sqrt(dble(npackts))
      call xvixdzo(norm,psi,adim)
      if(lrlxt) then
         normthermal=normthermal*norm
!!         PRINT *, 'renorma: norm =',norm,' normthermal =',normthermal
      endif

      return
      end subroutine renorma

! **********************************************************************
! *                                                                    *
! *                          SUBROUTINE RENORMPHI                      *
! *                                                                    *
! * Renormalizes the single-particle functions in a relaxation         *
! * calculation (exact calculation).                                   *
! *                                                                    *
! * Input parameters:                                                  *
! *   psi: The single-particle vector.                                 *
! *                                                                    *
! * Output parameters:                                                 *
! *   psi:  The normalised single-particle vector.                     *
! *                                                                    *
! * V8.0 MB                                                            *
! *                                                                    *
! **********************************************************************

      subroutine renormphi (psi)

      implicit none

      integer(long)                                                 :: s
      real(dop)                                                     :: norm
      complex(dop)                                                  :: sum
      complex(dop), dimension(adim+1:adim+totphidim), intent(inout) :: psi

! --- RENORMALISE SINGLE-PARTICLE FUNCTIONS ---
! As this is for an exact calculation, norm of all functions must be 1.

      norm=0.0d0
      do s = 1,nstate
         call sum2vxz(psi(zetf(1,s)),sum,subdim(1))
         norm = norm + dble(sum)
      enddo
      norm = dsqrt(norm)
      call xvixdzo(norm,psi(zetf(1,1)),totphidim)

      return
      end subroutine renormphi            ! of renormphi

!#######################################################################
!
! This routine assigns an A-vector to another A-vector with different
! numbers of spfs. It reduces or expands the second A-vector to fit the
! original A-vector.
!
!#######################################################################

      subroutine akcopy(psiold,psinew,vdim2,dim2,dim1,ndim1)

      implicit none

      integer(long)             :: v,e,n,dimn
      integer(long), intent(in) :: vdim2,dim1,dim2,ndim1

      complex(dop), dimension(vdim2,dim1,ndim1), intent(in)  :: psiold
      complex(dop), dimension(vdim2,dim2,ndim1), intent(out) :: psinew

      dimn=min(dim2,dim1)
      psinew = 0.0_dop
      do n=1,ndim1
         do e=1,dimn
            do v=1,vdim2
               psinew(v,e,n)=psiold(v,e,n)
            enddo
         enddo
      enddo

      return
      end subroutine akcopy

!#######################################################################
!
! This routine assigns an A-vector to another A-vector with different
! number of modes. It reduces or expands the second A-vector to fit the
! original A-vector.
!
!#######################################################################

      subroutine akcopypsi3d(psiold,psinew,vdim1,dim1,ndim1)

      implicit none

      integer(long)             :: v,e,n
      integer(long), intent(in) :: vdim1,dim1,ndim1

      complex(dop), dimension(vdim1,1,ndim1), intent(in)     :: psiold
      complex(dop), dimension(vdim1,dim1,ndim1), intent(out) :: psinew

      do n=1,ndim1
         do v=1,vdim1
            psinew(v,1,n)=psiold(v,1,n)
         enddo
      enddo

      do n=1,ndim1
         do e=2,dim1
            do v=1,vdim1
               psinew(v,e,n)=0.0
            enddo
         enddo
      enddo

      return
      end subroutine akcopypsi3d

!#####################################################################
!
! the same as akcopy, for dentype 2
!
!  12/02 CC
!
!#####################################################################

      subroutine akcopy2(psiold,psinew,vdim2,dim2,dim1,ndim1,&
                 vdim2a,dim2a,dim1a,ndim1a)

      implicit none

      integer(long)             :: v,e,n,w,f,m,dimn,dimna
      integer(long), intent(in) :: vdim2,dim2,dim1,ndim1, &
                                   vdim2a,dim2a,dim1a,ndim1a

      complex(dop), dimension(vdim2,dim1,ndim1,vdim2a,dim1a,ndim1a), &
                         intent(in)    :: psiold
      complex(dop), dimension(vdim2,dim2,ndim1,vdim2a,dim2a,ndim1a), &
                         intent(out) :: psinew

      dimn=min(dim1,dim2)
      dimna=min(dim1a,dim2a)
      psinew = 0.0_dop
      do m=1,ndim1a
         do f=1,dimna
            do w=1,vdim2a
               do n=1,ndim1
                  do e=1,dimn
                     do v=1,vdim2
                        psinew(v,e,n,w,f,m)=psiold(v,e,n,w,f,m)
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine akcopy2                      ! of akcopy2

!#####################################################################
!
! Copies a WF A-coefficient vector into a density type II matrix
!
!#####################################################################

      subroutine akcopy_wf_to_d2(psiold,psiolda,psinew,&
                 vdim1,dim1,ndim1,vdim1a,dim1a,ndim1a)

      implicit none

      integer(long)             :: v,e,n,w,f,m
      integer(long), intent(in) :: vdim1,dim1,ndim1,vdim1a,dim1a,ndim1a

      complex(dop), dimension(vdim1,dim1,ndim1), intent(in)    &
                                :: psiold
      complex(dop), dimension(vdim1a,dim1a,ndim1a), intent(in) &   
                                :: psiolda
      complex(dop), dimension(vdim1,dim1,ndim1,vdim1a,dim1a,ndim1a), &
                    intent(out) :: psinew

      do m=1,ndim1
         do f=1,dim1
            do w=1,vdim1
               do n=1,ndim1a
                  do e=1,dim1a
                     do v=1,vdim1a
                        psinew(v,e,n,w,f,m)= &
                             psiold(v,e,n)*dconjg(psiolda(w,f,m))
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine akcopy_wf_to_d2                      ! of akcopy_wf_to_d2

!#######################################################################
! AKADD
! This routine assigns an A-vector to another A-vector with different
! number of spfs. It reduces or expands the second A-vector to fit the
! original A-vector. When expanding the new values are passed through
! the array psiadd(vdim1,dim1-dim,ndim).
!
! 2019 David Mendive-Tapia
!#######################################################################

      subroutine akadd(psiold,psinew,psiadd,vdim1,dim1,dim,ndim)

      implicit none

      integer(long), intent(in) :: vdim1,dim1,dim,ndim
      integer(long)             :: v,e,n,dim2
      complex(dop), dimension(vdim1,dim,ndim), intent(in)      :: psiold
      complex(dop), dimension(vdim1,dim1,ndim), intent(out)    ::  psinew
      complex(dop), dimension(vdim1,dim1-dim,ndim), intent(in) ::  psiadd

      call zerovxz(psinew,vdim1*dim1*ndim)

      dim2=min(dim,dim1)
      do n=1,ndim
         do e=1,dim2
            do v=1,vdim1
               psinew(v,e,n)=psiold(v,e,n)
            enddo
         enddo
      enddo

      if (dim1 .gt. dim) then
         do n=1,ndim
            do e=dim+1,dim1
               do v=1,vdim1
                  psinew(v,e,n)=psiadd(v,e-dim,n)
               enddo
            enddo
         enddo
      endif

      return
      end

!#######################################################################
!
! Calculates overlap matrices for all SPFs in a wavefunction. Both
! psi1 and psi2 must have the same grids and no. SPFs.
!
! modus = 0   calculate < psi1 | psi2 >
!         1   calculate < psi1* | psi2 >
!#######################################################################

      subroutine spfovl(ovlmat,psi1,psi2,agmat1,agmat2,modus)

      implicit none

      integer(long), intent(in)                     :: modus
      complex(dop), dimension(dgldim), intent(in)   :: psi1
      complex(dop), dimension(dgldim), intent(in)   :: psi2
      complex(dop), dimension(agmatdim), intent(in) :: agmat1
      complex(dop), dimension(agmatdim), intent(in) :: agmat2
      complex(dop), dimension(dmatdim), intent(out) :: ovlmat
      complex(dop), dimension(gfgwpdim)             :: gfgwp
      complex(dop), dimension(aigwpdim)             :: aigwp
      complex(dop), dimension(zgwpdim)              :: zgwp
      integer(long)                                 :: m,s

      if (modus .eq. 0) then
         do s=1,nstate
            do m = 1,nmode
               if (gwpm(m)) then
                  call diagonal_moments(m,s,s,nspfdof(m),gwpdim(m,s),&
                       gwpdim(m,s),agmat1(zaGmat(m,s)),agmat2(zaGmat(m,s)),&
                       agmat1(zxivec(m,s)),agmat2(zxivec(m,s)),&
                       agmat1(zphase(m,s)),agmat2(zphase(m,s)),&
                       gfgwp(zgfgwp(m,s,s)),aigwp(zaigwp(m,s,s)),&
                       zgwp(zzgwp(m,s,s)),0,0)
! gfgwp contains overlap
                  if (psitype .eq. 0 .or. psitype .eq. 5) then
                     call simtran2bz(psi1(zdgwp(m,s)),gfgwp(zgfgwp(m,s,s)),&
                          psi2(zdgwp(m,s)),ovlmat(dmat(m,s)),gwpdim(m,s),&
                          gwpdim(m,s),dim(m,s),dim(m,s))
                  else
                     call cpqxz(gfgwp(zgfgwp(m,s,s)),ovlmat(dmat(m,s)),&
                          gwpdim(m,s))
                  endif
               else
                  call mmaxzz(psi1(zetf(m,s)),psi2(zetf(m,s)),&
                       ovlmat(dmat(m,s)),subdim(m),dim(m,s),dim(m,s))
               endif
            enddo
         enddo

      else if (modus .eq. 1) then
         do s=1,nstate
            do m = 1,nmode
               if (gwpm(m)) then
                  call diagonal_moments(m,s,s,nspfdof(m),gwpdim(m,s),&
                       gwpdim(m,s),agmat1(zaGmat(m,s)),agmat2(zaGmat(m,s)),&
                       agmat1(zxivec(m,s)),agmat2(zxivec(m,s)),&
                       agmat1(zphase(m,s)),agmat2(zphase(m,s)),&
                       gfgwp(zgfgwp(m,s,s)),aigwp(zaigwp(m,s,s)),&
                       zgwp(zzgwp(m,s,s)),1,0)
! gfgwp contains overlap
                  if (psitype .eq. 0 .or. psitype .eq. 5) then
                     call simtran2tz(psi1(zdgwp(m,s)),gfgwp(zgfgwp(m,s,s)),&
                          psi2(zdgwp(m,s)),ovlmat(dmat(m,s)),gwpdim(m,s),&
                          gwpdim(m,s),dim(m,s),dim(m,s))
                  else
                     call cpqxz(gfgwp(zgfgwp(m,s,s)),ovlmat(dmat(m,s)),&
                          gwpdim(m,s))
                  endif
               else
                  call mmtxzz(psi1(zetf(m,s)),psi2(zetf(m,s)),&
                       ovlmat(dmat(m,s)),subdim(m),dim(m,s),dim(m,s))
               endif
            enddo
         enddo

      else
         routine = 'Spfovl (psiutil)'
         write(message,'(a,i3)') 'Unknown overlap modus asked for :',modus
         call errormsg

      endif

      return
      end subroutine spfovl

!############  END of PSIUTIL ##########################################
      end module psiutil


