!#######################################################################

      module rddvrmod

use decimal, only: long, dop
use logdat
use op1lib,  only: zerovxi

use global
use maxv, only: maxdim
use dvrdatmod
use griddatmod
use channels

use iodvrdef, only:rddvrdef11,rddvrdef_pre11

      implicit none

!-----------------------------------------------------------------------
! array to specify which dvr arrays are to be read
!-----------------------------------------------------------------------
      logical(kind=4), dimension(15), save :: dvrdata

      public :: dvrinfo,rddvr
      contains

!#######################################################################
! dvrinfo . Subroutines reading the DVR file.
!#######################################################################

      subroutine dvrinfo(lerr,chkdvr)

      implicit none

      integer(long), intent(in)              :: chkdvr
      logical(kind=4), intent(out)           :: lerr

      rewind(idvr)
      read(idvr) filever(idvr)

      if (filever(idvr) .lt. 11.0_dop) Then
         call dvrinfo_pre11(lerr,chkdvr)
      else
         call dvrinfo11(lerr,chkdvr)
      endif

      end subroutine dvrinfo

!#######################################################################
! dvrinfo . Subroutines reading the DVR file.
!#######################################################################

      subroutine dvrinfo11(lerr,chkdvr)

      implicit none

      integer(long)                          :: f,ndof1,ilbl,jlbl
      integer(long), intent(in)              :: chkdvr
      integer(long), dimension(maxdim)       :: fdvr
      character(len=c4)                      :: file
      logical(kind=4), intent(out)           :: lerr
      character(len=c2)                      :: rdtag

      routine='Dvrinfo'

      file='Dvr'
      chkerr(:)=0
      call rddvrdef11(idvr,chkdvr,ndof1,fdvr)
      call chksyserr(file,lerr)
      if (lerr) return

      do
  10     continue
         read(idvr,iostat=chkerr(1),err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'ortdim') then
            read(idvr,err=999) ortdim
         else if (rdtag(ilbl:jlbl) .eq. 'dvrdim') then
            read(idvr,err=999) dvrdim
         else if (rdtag(ilbl:jlbl) .eq. 'fftdim') then
            read(idvr,err=999) fftdim
         else if (rdtag(ilbl:jlbl) .eq. 'expdim') then
            read(idvr,err=999) expdim
         else if (rdtag(ilbl:jlbl) .eq. 'sphdim') then
            read(idvr,err=999) sphdim
         else if (rdtag(ilbl:jlbl) .eq. 'zort') then
            read(idvr) (zort(f),f=1,ndof1)
         else if (rdtag(ilbl:jlbl) .eq. 'zdvr') then
            read(idvr) (zdvr(f),f=1,ndof1)
         else if (rdtag(ilbl:jlbl) .eq. 'zfft') then
            read(idvr) (zfft(f),f=1,ndof1)
         else if (rdtag(ilbl:jlbl) .eq. 'zexp') then
            read(idvr) (zexp(f),f=1,ndof1)
         else if (rdtag(ilbl:jlbl) .eq. 'zsph') then
            read(idvr) (zsph(f),f=1,ndof1)

         else if (rdtag(ilbl:jlbl) .eq. 'end dvr data') then
            exit

         endif

      enddo

  100 continue

      return
  999 continue
      routine='Dvrinfo11'
      message='Error Reading array info DVR file: '//rdtag(ilbl:jlbl)
      call errormsg
      end subroutine dvrinfo11

!#######################################################################
! dvrinfo . Subroutines reading the DVR file.
!#######################################################################

      subroutine dvrinfo_pre11(lerr,chkdvr)

      implicit none

      integer(long)                          :: f,ndof1
      integer(long), intent(in)              :: chkdvr
      integer(long), dimension(maxdim)       :: fdvr
      character(len=c4)                      :: file
      character(len=100)                     :: buffer
      logical(kind=4), intent(out)           :: lerr


      routine='Dvrinfo'
      file='Dvr'
      call zerovxi(chkerr,nerr)
      call rddvrdef_pre11(idvr,chkdvr,ndof1,fdvr)
      call chksyserr(file,lerr)
      if (lerr) return

      read(idvr) buffer

      if (filever(idvr) .ge. 8.2002d0) then
         read(idvr) ortdim,dvrdim,fftdim,expdim,sphdim
      else 
         read(idvr) ortdim,dvrdim,fftdim,sphdim
      endif

      read(idvr) (zort(f),f=1,ndof1)
      read(idvr) (zdvr(f),f=1,ndof1)
      read(idvr) (zfft(f),f=1,ndof1)
      if (filever(idvr) .ge. 8.2002d0) then
         read(idvr) (zexp(f),f=1,ndof1)
      endif
      read(idvr) (zsph(f),f=1,ndof1)

      if (filever(idvr) .lt. 9.0005) then
         do f=1,ndof1
            if (basis(f) .eq. 25)  lgwp=.true.
         enddo

         if (lgwp) then
            routine='Dvrinfo'
            message= &
         'DVR file does not contain GWP grids. Recreate with > 9.0005'
            call errormsg
         endif
      endif

      return
      end subroutine dvrinfo_pre11

!#######################################################################
! rddvr . Subroutines reading the DVR file.
!#######################################################################

      subroutine rddvr(ort,trafo,dvrmat,fftp,hin,rueck,fftfak,exphin, &
                 exprueck,jsph,msph,kinsph,chkdvr)

      implicit none

      integer(long)                                    :: ilbl
      integer(long), intent(in)                        :: chkdvr
      integer(long), dimension(fftdim), intent(out)    :: fftfak
      integer(long), dimension(sphdim), intent(out)    :: jsph,msph
      real(dop), dimension(ortdim), intent(out)        :: ort
      real(dop), dimension(dvrdim), intent(out)        :: trafo
      real(dop), dimension(fftdim), intent(out)        :: fftp
      real(dop), dimension(3*sphdim), intent(out)      :: kinsph
      real(dop), dimension(dvrdim,dvrord), intent(out) :: dvrmat
      complex(dop), dimension(fftdim), intent(out)     :: hin,rueck
      complex(dop), dimension(expdim), intent(out)     :: exphin,exprueck
      character(len=c5)                                :: filename
      logical(kind=4)                                  :: lexist

      routine='Rddvr'

      lexist=.false.
      filename=dname(1:dlaenge)//'/dvr'
      ilbl=dlaenge+4
      inquire(file=filename(1:ilbl),exist=lexist)
! Long form failed, try not to append 'dvr'
      if (.not. lexist) then
         filename=dname(1:dlaenge)
         ilbl=dlaenge
         inquire(file=filename(1:ilbl),exist=lexist)
      endif
      if (.not. lexist) go to 999

      open(idvr,file=filename(1:ilbl),form='unformatted',status='old',err=999)

      rewind(idvr)
      read(idvr) filever(idvr)

      if (filever(idvr) .lt. 11.0_dop) Then
         call rddvr_pre11(ort,trafo,dvrmat,fftp,hin,rueck,fftfak,exphin, &
                 exprueck,jsph,msph,kinsph,chkdvr)
      else
         call rddvr11(ort,trafo,dvrmat,fftp,hin,rueck,fftfak,exphin, &
                 exprueck,jsph,msph,kinsph,chkdvr)
      endif

      return
  999 continue
      message = 'Cannot open dvr file: '//filename(1:ilbl)
      call errormsg

      end subroutine rddvr

!#######################################################################
! rddvr . Subroutines reading the DVR file.
!#######################################################################

      subroutine rddvr11(ort,trafo,dvrmat,fftp,hin,rueck,fftfak,exphin, &
                 exprueck,jsph,msph,kinsph,chkdvr)

      implicit none

      integer(long)                                    :: g,n,idvrord
      integer(long)                                    :: ilbl,jlbl
      integer(long), intent(in)                        :: chkdvr
      integer(long), dimension(fftdim), intent(out)    :: fftfak
      integer(long), dimension(sphdim), intent(out)    :: jsph,msph
      real(dop), dimension(ortdim), intent(out)        :: ort
      real(dop), dimension(dvrdim), intent(out)        :: trafo
      real(dop), dimension(fftdim), intent(out)        :: fftp
      real(dop), dimension(3*sphdim), intent(out)      :: kinsph
      real(dop), dimension(dvrdim,dvrord), intent(out) :: dvrmat
      complex(dop), dimension(fftdim), intent(out)     :: hin,rueck
      complex(dop), dimension(expdim), intent(out)     :: exphin,exprueck
      logical(kind=4)                                  :: lerr
      character(len=c2)                                :: rdtag

      rewind(idvr)
      call dvrinfo(lerr,chkdvr)
      if (lerr) then
         routine='Rddvr11'
         message='Error reading DVR file'
         call errormsg
      endif

!-----------------------------------------------------------------------
! which data sets are read into memory, and which are read and ignored
!
! if dvrdata(1)=.true. read ort
! if dvrdata(2)=.true. read trafo
! if dvrdata(3)=.true. read dvrmat
! if dvrdata(4)=.true. read fftp
! if dvrdata(5)=.true. read hin
! if dvrdata(6)=.true. read rueck
! if dvrdata(7)=.true. read fftfak
! if dvrdata(8)=.true. read jsph
! if dvrdata(9)=.true. read msph
! if dvrdata(10)=.true. read kinsph
! if dvrdata(11)=.true. read exphin
! if dvrdata(12)=.true. read exprueck
!-----------------------------------------------------------------------
      rewind(idvr)
      do
! read next record - error means it is not a string so ignore and go on to next
   10    continue
         read(idvr,err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'ort' .and. dvrdata(1)) then
            read(idvr,err=999) (ort(g),g=1,ortdim)

         else if (rdtag(ilbl:jlbl) .eq. 'trafo' .and. dvrdata(2)) then
            read(idvr,err=999) (trafo(g),g=1,dvrdim)

         else if (rdtag(ilbl:jlbl) .eq. 'dvrord') then
            read(idvr,err=999) idvrord
      if (idvrord .gt. dvrord) then
         write(message,'(a,i5)') &
            'Increase DVRORD in griddatmod.f90. Minimum : ',idvrord
         call errormsg
      endif

         else if (rdtag(ilbl:jlbl) .eq. 'dvrmat' .and. dvrdata(3)) then
            read(idvr,err=999) ((dvrmat(g,n),g=1,dvrdim),n=1,idvrord)
         else if (rdtag(ilbl:jlbl) .eq. 'fftp' .and. dvrdata(4)) then
            read(idvr,err=999) (fftp(g),g=1,fftdim)
         else if (rdtag(ilbl:jlbl) .eq. 'hin' .and. dvrdata(5)) then
            read(idvr,err=999) (hin(g),g=1,fftdim)
         else if (rdtag(ilbl:jlbl) .eq. 'rueck' .and. dvrdata(6)) then
            read(idvr,err=999) (rueck(g),g=1,fftdim)
         else if (rdtag(ilbl:jlbl) .eq. 'fftfak' .and. dvrdata(7)) then
            read(idvr,err=999) (fftfak(g),g=1,fftdim)
         else if (rdtag(ilbl:jlbl) .eq. 'jsph' .and. dvrdata(8)) then
            read(idvr,err=999) (jsph(g),g=1,sphdim)
         else if (rdtag(ilbl:jlbl) .eq. 'msph' .and. dvrdata(9)) then
            read(idvr,err=999) (msph(g),g=1,sphdim)
         else if (rdtag(ilbl:jlbl) .eq. 'kinsph' .and. dvrdata(10)) then
            read(idvr,err=999) (kinsph(g),g=1,3*sphdim)
         else if (rdtag(ilbl:jlbl) .eq. 'exphin' .and. dvrdata(11)) then
            read(idvr,err=999) (exphin(g),g=1,expdim)
         else if (rdtag(ilbl:jlbl) .eq. 'exprueck' .and. dvrdata(12)) then
            read(idvr,err=999) (exprueck(g),g=1,expdim)

         endif

      enddo

  100 continue

      close(idvr)

      return
  999 continue
      routine='Rddvr11'
      message='Error Reading DVR file: '//rdtag(ilbl:jlbl)
      call errormsg
      end subroutine rddvr11

!#######################################################################
! rddvr . Subroutines reading the DVR file.
!#######################################################################

      subroutine rddvr_pre11(ort,trafo,dvrmat,fftp,hin,rueck,fftfak,exphin, &
                 exprueck,jsph,msph,kinsph,chkdvr)

      implicit none

      integer(long)                                    :: g,n,idvrord
      integer(long), intent(in)                        :: chkdvr
      integer(long), dimension(fftdim), intent(out)    :: fftfak
      integer(long), dimension(sphdim), intent(out)    :: jsph,msph
      real(dop), dimension(ortdim), intent(out)        :: ort
      real(dop), dimension(dvrdim), intent(out)        :: trafo
      real(dop), dimension(fftdim), intent(out)        :: fftp
      real(dop), dimension(3*sphdim), intent(out)      :: kinsph
      real(dop), dimension(dvrdim,dvrord), intent(out) :: dvrmat
      complex(dop), dimension(fftdim), intent(out)     :: hin,rueck
      complex(dop), dimension(expdim), intent(out)     :: exphin,exprueck
      logical(kind=4)                                  :: lerr

      routine='Rddvr'

      call dvrinfo_pre11(lerr,chkdvr)
      if (lerr) then
         message='Error reading DVR file'
         call errormsg
      endif

!-----------------------------------------------------------------------
! which data sets are read into memory, and which are read and ignored
! are governed by dvrdata array in global.inc
!
! if dvrdata(1)=.true. read ort
! if dvrdata(2)=.true. read trafo
! if dvrdata(3)=.true. read dvrmat
! if dvrdata(4)=.true. read fftp
! if dvrdata(5)=.true. read hin
! if dvrdata(6)=.true. read rueck
! if dvrdata(7)=.true. read fftfak
! if dvrdata(8)=.true. read jsph
! if dvrdata(9)=.true. read msph
! if dvrdata(10)=.true. read kinsph
! if dvrdata(11)=.true. read exphin
! if dvrdata(12)=.true. read exprueck
!-----------------------------------------------------------------------
      if (dvrdata(1)) then
         read(idvr,err=990) (ort(g),g=1,ortdim)
      else
         read(idvr,err=990) 
      endif

      if (dvrdata(2)) then
         read(idvr,err=990) (trafo(g),g=1,dvrdim)
      else
         read(idvr,err=990) 
      endif

      read(idvr,err=990) idvrord
      if (idvrord .gt. dvrord) then
         write(message,'(a,i5)') &
            'Increase DVRORD in griddatmod.f90. Minimum : ',idvrord
         call errormsg
      endif
      if (dvrdata(3)) then
         read(idvr,err=990) ((dvrmat(g,n),g=1,dvrdim),n=1,idvrord)
      else
         read(idvr,err=990) 
      endif

      if (dvrdata(4)) then
         read(idvr,err=990) (fftp(g),g=1,fftdim)
      else
         read(idvr,err=990) 
      endif
      if (dvrdata(5)) then
         read(idvr,err=990) (hin(g),g=1,fftdim)
      else
         read(idvr,err=990) 
      endif
      if (dvrdata(6)) then
         read(idvr,err=990) (rueck(g),g=1,fftdim)
      else
         read(idvr,err=990) 
      endif
      if (dvrdata(7)) then
         read(idvr,err=990) (fftfak(g),g=1,fftdim)
      else
         read(idvr,err=990) 
      endif
      if (filever(idvr) .ge. 8.2002d0) then
         if (dvrdata(11)) then
            read(idvr,err=990) (exphin(g),g=1,expdim)
         else
            read(idvr,err=990)
         endif
         if (dvrdata(12)) then
            read(idvr,err=990) (exprueck(g),g=1,expdim)
         else
            read(idvr,err=990)
         endif
      elseif(dvrdata(11) .or. dvrdata(12) ) then
         message='old DVR-file (before 8.2.2) may lead to problems'
         routine='rddvr'
         call warnmsg1
      endif
      if (dvrdata(8) .and. sphdim .gt. 1 ) then
         read(idvr,err=990) (jsph(g),g=1,sphdim)
      else
         read(idvr,err=990) 
      endif
      if (dvrdata(9) .and. sphdim .gt. 1) then
         read(idvr,err=990) (msph(g),g=1,sphdim)
      else
         read(idvr,err=990) 
      endif
      if (dvrdata(10) .and. sphdim .gt. 1) then
         read(idvr,err=990) (kinsph(g),g=1,3*sphdim)
      else
         read(idvr,err=990) 
      endif

      close(idvr)

      return
  990 continue
      message='Error reading dvr file'
      call errormsg
      end subroutine rddvr_pre11

!-----------------------------------------------------------------------

      end module rddvrmod
