
!***********************************************************************
!               MCTDH subroutine npotinfo
!
!  Reads natural potential data from files 'natpot' 
!  and performs some consistency checks.
!  The natpots are actually read by rdops on hintern.F
! 
!  11/97 AJ
!***********************************************************************

      module rdnatpot

use global
use versions, only: progver, progname
use dvrdatmod
use griddatmod
use operdef
use runopermod
use combined
use channels

use maxv,     only: maxdim
use logdat
use iodvrdef, only: rddvrdef
use iogrddef, only: rdgrddef
use op1lib,   only: zerovxi

      implicit none
      private
      public :: npotinfo,cpdinfo

      contains

!#######################################################################
!
! NPOTINFO
!
! reads information about natural potentials
!
!#######################################################################

      subroutine npotinfo(ichan,lerr,modc1,loned,ndof1,fdvr, &
         chkdvr,chkgrd,iwrt,mdvr,mmode,nspfdof1,spfdof1, &
         nmode1,subdim1,pntbas)

      implicit none

      integer(long), intent(out)                            :: modc1,ndof1,nmode1
      integer(long), intent(inout)                          :: chkgrd
      integer(long), intent(in)                             :: ichan,chkdvr,iwrt

      integer(long), dimension(maxdim), intent(out)         :: pntbas,nspfdof1,&
                                                               fdvr,mdvr,mmode,&
                                                               subdim1
      integer(long), dimension(maxdim,maxdim), intent(out)  :: spfdof1
      logical(kind=4), intent(out)                          :: lerr
      logical(kind=4), dimension(maxdim), intent(out)       :: loned

      rewind(ichan)
      read(ichan,err=999) filever(ichan)

      if (filever(ichan) .gt. progver) then
         routine='Npotinfo'
         write(message,'(a,f8.5)') &
           'Natpot file newer than program. Use version :',filever(ichan)
         call errormsg
      endif
!
! read file depending on version
!
      if (filever(ichan) .lt. 11.0_dop) then
         call npotinfo_pre11(ichan,lerr,modc1,loned,ndof1,fdvr, &
         chkdvr,chkgrd,iwrt,mdvr,mmode,nspfdof1,spfdof1, &
         nmode1,subdim1,pntbas)

      else
         call npotinfo11(ichan,lerr,modc1,loned,ndof1,fdvr, &
         chkdvr,chkgrd,iwrt,mdvr,mmode,nspfdof1,spfdof1, &
         nmode1,subdim1,pntbas)

      endif

      return

  999 continue
      routine='Npotinfo'
      message='Error reading file version from natpot file'
      call errormsg

      end subroutine npotinfo

!-----------------------------------------------------------------------

      subroutine npotinfo11(ichan,lerr,modc1,loned,ndof1,fdvr, &
         chkdvr,chkgrd,iwrt,mdvr,mmode,nspfdof1,spfdof1, &
         nmode1,subdim1,pntbas)


      implicit none

      integer(long), intent(out)                            :: modc1,ndof1,nmode1
      integer(long), intent(inout)                          :: chkgrd
      integer(long), intent(in)                             :: ichan,chkdvr,iwrt
      integer(long)                                         :: n,m,f,zeig,f1,&
                                                               lcount,i

      integer(long), dimension(maxdim)                      :: dofspf1,npgdim
      integer(long), dimension(maxdim), intent(out)         :: pntbas,nspfdof1,&
                                                               fdvr,mdvr,mmode,&
                                                               subdim1
      integer(long), dimension(maxdim,maxdim), intent(out)  :: spfdof1
      integer(long)                                 :: ilbl,jlbl
      logical(kind=4), intent(out)                          :: lerr
      logical(kind=4)                                       :: lchnpot
      logical(kind=4), dimension(maxdim), intent(out)       :: loned
      character(len=c1)                                     :: file
      character(len=80)                                     :: buffer
      character(len=c2)                                     :: rdtag


      lchnpot=.false.
      if (progname(1:6) .eq. 'chnpot') lchnpot=.true.

!-----------------------------------------------------------------------
! Initialise variables 
!-----------------------------------------------------------------------
      routine = 'npotinfo'
      lwarn   = .false.

      if (.not. lchnpot) then
         write(ilog,'(/,a)') '**** Reading NATPOT file ****'
         write(ilog,'(a,i3)') 'potfit input file:  nr =',iwrt
      endif

      ilbl=0
      jlbl=0
!-----------------------------------------------------------------------
! read title and write to log file
!-----------------------------------------------------------------------
      rewind(ichan)
      do
  10     continue
         read(ichan,iostat=chkerr(1),err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'creation date') then
            read(ichan,err=999) buffer
            write(ilog,'(/,a)') 'Created on:'
            write(ilog,'(a)') buffer(1:80)

         else if (rdtag(ilbl:jlbl) .eq. 'natpot title') then
            write(ilog,'(a)') 'vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv'
            read(ichan,err=999) lcount
            do i=1,lcount
               read(ichan,err=999) buffer
               write(ilog,'(a)') buffer
            enddo
            write(ilog,'(a)') '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'

         else if (rdtag(ilbl:jlbl) .eq. 'end natpot title') then
            exit
         endif

      enddo

  100 continue

!-----------------------------------------------------------------------
! read dvr data
!-----------------------------------------------------------------------
      file='natpot'
      call zerovxi(chkerr,nerr)
      call zerovxi(dofspf1,maxdim)
      call rddvrdef(ichan,chkdvr,ndof1,fdvr)

      call chksyserr(file,lerr)
      if (lerr) stop
      if(iwrt.eq.-1)  return

!-----------------------------------------------------------------------
! read information about the combination of the degrees of freedom.
!-----------------------------------------------------------------------
      call rdgrddef(ichan,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
      call chksyserr(file,lerr)
      if (lerr) stop

!-----------------------------------------------------------------------
! read potfit parameters
!-----------------------------------------------------------------------
      do
   20    continue
         read(ichan,err=20,end=200) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'modc') then
            read(ichan,err=999) modc1

         else if (rdtag(ilbl:jlbl) .eq. 'onedipot') then
            read(ichan,err=999) (onedipot(f),f=1,ndof1)

         else if (rdtag(ilbl:jlbl) .eq. 'lpconm') then
            read(ichan,err=999) lpconm

         else if (rdtag(ilbl:jlbl) .eq. 'lpntdvr') then
            read(ichan,err=999) lpntdvr

         else if (rdtag(ilbl:jlbl) .eq. 'pntbas') then
            read(ichan,err=999) (pntbas(f),f=1,ndof1)

         else if (rdtag(ilbl:jlbl) .eq. 'pdim') then
            read(ichan,err=999) (pdim(m),m=1,nmode1)

         else if (rdtag(ilbl:jlbl) .eq. 'end natpot info') then
            exit

         endif
      enddo

  200 continue

!-----------------------------------------------------------------------
! Initialise variables containing the information about
! the combined degrees of freedom in potfit
!-----------------------------------------------------------------------
      npmode=nmode1
      do m=1,nmode1
         npotdof(m)=nspfdof1(m)
         do n=1,nspfdof1(m)
            potdof(n,m)=spfdof1(n,m)
            dofspf1(spfdof1(n,m))=m 
         enddo
      enddo
      do f=1,ndof1
         dofpot(f)=dofspf1(f)
      enddo

!-----------------------------------------------------------------------
! allow for any point DVRs used in the POTFIT
!-----------------------------------------------------------------------
      do f=1,ndof1
         f1=fdvr(f)
         if (pntbas(f) .eq. 19) then
            npgdim(f) = 1
         else
            npgdim(f) = gdim(f1)
         endif
      enddo

      maxpsub = 0
!-----------------------------------------------------------------------
! calculate psubdim for each mode.
!----------------------------------------------------------------------
      do m=1,npmode
         zeig=1
         do n=1,npotdof(m)
            f=potdof(n,m)
            zeig=zeig*npgdim(f)
         enddo
         psubdim(m)=zeig
         subdim1(m)=psubdim(m)
         maxpsub  = max(maxpsub,psubdim(m))
      enddo

!-- Edit: Exclude contracted modes (MS 03/2011)
      do m=1,npmode
         if (m.ne.modc) then
            maxpsub2  = max(psubdim(m)**2,maxpsub2)
         end if
      end do

      do f=1,ndof1
         loned(f) = onedipot(f).gt.0
      enddo

      return
  999 continue
      routine = 'npotinfo'
      message = 'Error reading natpot file'
      call errormsg


      end subroutine npotinfo11

!-----------------------------------------------------------------------

      subroutine npotinfo_pre11(ichan,lerr,modc1,loned,ndof1,fdvr, &
         chkdvr,chkgrd,iwrt,mdvr,mmode,nspfdof1,spfdof1, &
         nmode1,subdim1,pntbas)


      implicit none

      integer(long), intent(out)                            :: modc1,ndof1,nmode1
      integer(long), intent(inout)                          :: chkgrd
      integer(long), intent(in)                             :: ichan,chkdvr,iwrt
      integer(long)                                         :: n,m,f,zeig,f1,idum,lcount,i

      integer(long), dimension(maxdim)                      :: dofspf1,npgdim
      integer(long), dimension(maxdim), intent(out)         :: pntbas,nspfdof1,fdvr,mdvr,mmode,subdim1
      integer(long), dimension(maxdim,maxdim), intent(out)  :: spfdof1
      logical(kind=4), intent(out)                          :: lerr
      logical(kind=4)                                       :: lchnpot,ldum
      logical(kind=4), dimension(maxdim), intent(out)       :: loned
      character(len=c1)                                     :: file
      character(len=80)                                     :: buffer


!-----------------------------------------------------------------------
! Initialise variables 
!-----------------------------------------------------------------------
      routine = 'npotinfo'
      lwarn   = .false.

      if (filever(ichan) .ge. 8.3007d0) then
!        skip obsolete FT-natpot variables
         read(ichan,err=999) idum,idum,idum
      endif

      file='natpot'
      call zerovxi(chkerr,nerr)
      call zerovxi(dofspf1,maxdim)
      call rddvrdef(ichan,chkdvr,ndof1,fdvr)

      call chksyserr(file,lerr)
      if (lerr) stop
      if(iwrt.eq.-1)  return

      lchnpot=.false.
      if (progname(1:6) .eq. 'chnpot') lchnpot=.true.

!-----------------------------------------------------------------------
! read potfit input file and write it to the log file
!-----------------------------------------------------------------------
      if (iwrt.eq.0 .or. lchnpot) then
         read(ichan) buffer
         read(ichan,err=999) lcount
         do i=1,lcount
            read(ichan,err=999) buffer
         enddo
      else
!.....read creation date and time.
         write(ilog,'(/,a)') '****** Reading NATPOT file ******'
         write(ilog,'(a,f9.5)') 'File version:',filever(ichan)
         write(ilog,'(a)') 'Created on:'
         read(ichan) buffer
         write(ilog,'(a)') buffer
!.....read information from potfit file.
         write(ilog,'(a,i3)') 'potfit input file:  nr =',iwrt
         write(ilog,'(a)') 'vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv'
         read(ichan,err=999) lcount
         do i=1,lcount
            read(ichan,err=999) buffer
            write(ilog,'(a)') buffer
         enddo
         write(ilog,'(a)') '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
      endif
!-----------------------------------------------------------------------
! read potfit parameters
!-----------------------------------------------------------------------
      read(ichan,err=999) modc1,(onedipot(f),f=1,ndof1)
      if (filever(ichan) .ge. 8.2006d0) then
         read(ichan,err=999) lpconm
         if (filever(ichan).ge.8.3001d0 .and. &
             filever(ichan).le.8.3006d0) then
            read(ichan,err=999) ldum,ldum,idum
         endif 
         if (filever(ichan).ge.9.0002d0) then
            read(ichan,err=999) lpntdvr
            if (lpntdvr) then
               read(ichan,err=999) (pntbas(f),f=1,ndof1)
            endif
         endif
!.....read information about the combination of the degrees of freedom.
         call rdgrddef(ichan,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
         call chksyserr(file,lerr)
         if (lerr) stop

         read(ichan,err=999) (pdim(m),m=1,nmode1)
!-----------------------------------------------------------------------
! Initialise variables containing the information about
! the combined degrees of freedom in potfit
!-----------------------------------------------------------------------
         npmode=nmode1
         do m=1,nmode1
            npotdof(m)=nspfdof1(m)
            do n=1,nspfdof1(m)
               potdof(n,m)=spfdof1(n,m)
               dofspf1(spfdof1(n,m))=m 
            enddo
         enddo
         do f=1,ndof1
            dofpot(f)=dofspf1(f)
         enddo

!-----------------------------------------------------------------------
! allow for any point DVRs used in the POTFIT
!-----------------------------------------------------------------------
         do f=1,ndof1
            f1=fdvr(f)
            if (pntbas(f) .eq. 19) then
               npgdim(f) = 1
            else
               npgdim(f) = gdim(f1)
            endif
         enddo

         maxpsub = 0
!-----------------------------------------------------------------------
! calculate psubdim for each mode.
!----------------------------------------------------------------------
         do m=1,npmode
            zeig=1
            do n=1,npotdof(m)
               f=potdof(n,m)
!              f1=mdvr(f)
               zeig=zeig*npgdim(f)
            enddo
            psubdim(m)=zeig
            subdim1(m)=psubdim(m)
            maxpsub  = max(maxpsub,psubdim(m))
         enddo

!-- Edit: Exclude contracted modes (MS 03/2011)
         do m=1,npmode
            if (m.ne.modc) then
               maxpsub2  = max(psubdim(m)**2,maxpsub2)
            end if
         end do
!-- Old:
!         maxpsub2  = maxpsub**2
!-- End edit.    

      else ! filever .lt. 8.2006
!-----------------------------------------------------------------------
! Initialise variables containing the information about
! the combined modes in potfit (here the degrees of freedom
! are not combined : the modes are the degrees of freedom)
!-----------------------------------------------------------------------
        nmode1=ndof1

        do m=1,nmode1
           mdvr(m)=fdvr(m)
           mmode(m)=fdvr(m)
           f1=fdvr(m)
           subdim1(m)=gdim(f1)
           nspfdof1(m)=1
           spfdof1(1,m)=m
        enddo
         
        maxpsub  = maxgdim
        maxpsub2  = maxgdim2
        read(ichan,err=999) (pdim(f),f=1,ndof1)

      endif

      do f=1,ndof1
         loned(f) = onedipot(f).gt.0
      enddo

      return
  999 continue
      routine = 'npotinfo'
      message = 'Error reading natpot file'
      call errormsg


      end subroutine npotinfo_pre11

!#######################################################################
!
!               MCTDH subroutine cpdinfo
!
!  Reads CPD/CANDECOMP potential data from files 'cpd'
!  and performs some consistency checks.
!  The CPDs are actually read by rdops on hintern.F
!
!  11/18 MS
!***********************************************************************

      subroutine cpdinfo(ichan,lerr,ndof1,fdvr, &
         chkdvr,chkgrd,iwrt,mdvr,mmode,cpdrank,nspfdof1,spfdof1, &
         nmode1,subdim1,loned,pntbas)

      implicit none

      integer(long), intent(out)                            :: ndof1,nmode1
      integer(long), intent(out)                            :: cpdrank
      integer(long), intent(inout)                          :: chkgrd
      integer(long), intent(in)                             :: ichan,chkdvr,iwrt
      integer(long)                                         :: n,m,f,zeig,f1,&
                                                               lcount,i

      integer(long), dimension(maxdim)                      :: dofspf1,npgdim
      integer(long), dimension(maxdim), intent(out)         :: pntbas,nspfdof1,&
                                                               fdvr,mdvr,mmode,&
                                                               subdim1
      integer(long), dimension(maxdim,maxdim), intent(out)  :: spfdof1
      integer(long)                                 :: ilbl,jlbl

      logical(kind=4) ::  lerr,lchnpot,ldum
      logical(kind=4), dimension(maxdim), intent(out)       :: loned
      character(len=c1) :: file
      character(len=80) :: buffer


      rewind(ichan)
      read(ichan,err=999) filever(ichan)

      if (filever(ichan) .gt. progver) then
         routine='Cpdinfo'
         write(message,'(a,f8.5)') &
           'CPD file newer than program. Use version :',filever(ichan)
         call errormsg
      endif
!
! read file depending on version
!
      if (filever(ichan) .lt. 11.0_dop) then
         call  cpdinfo_pre11(ichan,lerr,ndof1,fdvr, &
               chkdvr,chkgrd,iwrt,mdvr,mmode,cpdrank,nspfdof1,spfdof1, &
               nmode1,subdim1,loned)

      else
         call  cpdinfo11(ichan,lerr,ndof1,fdvr, &
               chkdvr,chkgrd,iwrt,mdvr,mmode,cpdrank,nspfdof1,spfdof1, &
               nmode1,subdim1,loned,pntbas)

      endif

      return

  999 continue
      routine='Cpdinfo'
      message='Error reading file version from cpd file'
      call errormsg

      end subroutine cpdinfo

!***********************************************************************

      subroutine cpdinfo11(ichan,lerr,ndof1,fdvr, &
         chkdvr,chkgrd,iwrt,mdvr,mmode,cpdrank,nspfdof1,spfdof1, &
         nmode1,subdim1,loned,pntbas)

      implicit none

      integer(long), intent(out)                            :: ndof1,nmode1
      integer(long), intent(out)                            :: cpdrank
      integer(long), intent(inout)                          :: chkgrd
      integer(long), intent(in)                             :: ichan,chkdvr,iwrt
      integer(long)                                         :: n,m,f,zeig,f1,&
                                                               lcount,i

      integer(long), dimension(maxdim)                      :: dofspf1,npgdim
      integer(long), dimension(maxdim), intent(out)         :: pntbas,nspfdof1,&
                                                               fdvr,mdvr,mmode,&
                                                               subdim1
      integer(long), dimension(maxdim,maxdim), intent(out)  :: spfdof1
      integer(long)                                 :: ilbl,jlbl

      logical(kind=4) ::  lerr,lchnpot,ldum
      logical(kind=4), dimension(maxdim), intent(out)       :: loned
      character(len=c1) :: file
      character(len=80) :: buffer

      routine = 'cpdinfo'
      message = 'CPD INFO for ver > 11 not coded'
      call errormsg

!-----------------------------------------------------------------------
! Initialise variables
!-----------------------------------------------------------------------
      routine = 'cpdinfo'
      lwarn   = .false.

      read(ichan,err=999) filever(ichan)

      file='cpd'
      call zerovxi(chkerr,nerr)
      call zerovxi(dofspf1,maxdim)
      call rddvrdef(ichan,chkdvr,ndof1,fdvr)

      call chksyserr(file,lerr)
      if (lerr) call stopnow
      if(iwrt.eq.-1)  return

      lchnpot=.false.
      if (progname(1:6) .eq. 'chnpot') lchnpot=.true.

!-----------------------------------------------------------------------
! read cpd input file and write it to the log file
!-----------------------------------------------------------------------
      if (iwrt.eq.0 .or. lchnpot) then
         read(ichan) buffer
         read(ichan,err=999) lcount
         do i=1,lcount
            read(ichan,err=999) buffer
         enddo
      else
!.....read creation date and time.
         write(ilog,'(/,a)') '****** Reading CPD file ******'
         write(ilog,'(a,f9.5)') 'File version:',filever(ichan)
         write(ilog,'(a)') 'Created on:'
         read(ichan) buffer
         write(ilog,'(a)') buffer
!.....read information from potfit file.
         write(ilog,'(a,i3)') 'cpd input file:  nr =',iwrt
         write(ilog,'(a)') 'vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv'
         read(ichan,err=999) lcount
         do i=1,lcount
            read(ichan,err=999) buffer
            write(ilog,'(a)') buffer
         enddo
         write(ilog,'(a)') '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
      endif
!-----------------------------------------------------------------------
! read potfit parameters
!-----------------------------------------------------------------------
      read(ichan,err=999) cpdrank, (onedipot(f),f=1,ndof1)
!.....read information about the combination of the degrees of freedom.
      call rdgrddef(ichan,chkgrd,fdvr,mdvr,mmode,nspfdof1,&
           spfdof1,nmode1)
      call chksyserr(file,lerr)
      if (lerr) call stopnow

!-----------------------------------------------------------------------
! Initialise variables containing the information about
! the combined degrees of freedom in potfit
!-----------------------------------------------------------------------
      npmode=nmode1
      do m=1,nmode1
         npotdof(m)=nspfdof1(m)
         do n=1,nspfdof1(m)
            potdof(n,m)=spfdof1(n,m)
            dofspf1(spfdof1(n,m))=m
         enddo
      enddo
      do f=1,ndof1
         dofpot(f)=dofspf1(f)
      enddo

      maxpsub = 0
!-----------------------------------------------------------------------
! calculate psubdim for each mode.
!----------------------------------------------------------------------
      do m=1,npmode
         zeig=1
         do n=1,npotdof(m)
            f=potdof(n,m)
            f1=mdvr(f)
            zeig=zeig*gdim(f1)
         enddo
         psubdim(m)=zeig
         subdim1(m)=psubdim(m)
         maxpsub  = max(maxpsub,psubdim(m))
      enddo

      do f=1,ndof1
         loned(f) = onedipot(f).gt.0
      enddo

      return
  999 continue
      routine = 'cpdinfo'
      message = 'Error reading cpd file'
      call errormsg
      end subroutine cpdinfo11

!***********************************************************************

      subroutine cpdinfo_pre11(ichan,lerr,ndof1,fdvr, &
         chkdvr,chkgrd,iwrt,mdvr,mmode,cpdrank,nspfdof1,spfdof1, &
         nmode1,subdim1,loned)

      implicit none

      integer(long), intent(out)                            :: ndof1,nmode1
      integer(long), intent(out)                            :: cpdrank
      integer(long), intent(inout)                          :: chkgrd
      integer(long), intent(in)                             :: ichan,chkdvr,iwrt
      integer(long)                                         :: n,m,f,zeig,f1,&
                                                               lcount,i

      integer(long), dimension(maxdim)                      :: dofspf1,npgdim
      integer(long), dimension(maxdim), intent(out)         :: nspfdof1,&
                                                               fdvr,mdvr,mmode,&
                                                               subdim1
      integer(long), dimension(maxdim,maxdim), intent(out)  :: spfdof1
      integer(long)                                 :: ilbl,jlbl

      logical(kind=4) ::  lerr,lchnpot,ldum
      logical(kind=4), dimension(maxdim), intent(out)       :: loned
      character(len=c1) :: file
      character(len=80) :: buffer

!-----------------------------------------------------------------------
! Initialise variables
!-----------------------------------------------------------------------
      routine = 'cpdinfo'
      lwarn   = .false.

      read(ichan,err=999) filever(ichan)

      file='cpd'
      call zerovxi(chkerr,nerr)
      call zerovxi(dofspf1,maxdim)
      call rddvrdef(ichan,chkdvr,ndof1,fdvr)

      call chksyserr(file,lerr)
      if (lerr) call stopnow
      if(iwrt.eq.-1)  return

      lchnpot=.false.
      if (progname(1:6) .eq. 'chnpot') lchnpot=.true.

!-----------------------------------------------------------------------
! read cpd input file and write it to the log file
!-----------------------------------------------------------------------
      if (iwrt.eq.0 .or. lchnpot) then
         read(ichan) buffer
         read(ichan,err=999) lcount
         do i=1,lcount
            read(ichan,err=999) buffer
         enddo
      else
!.....read creation date and time.
         write(ilog,'(/,a)') '****** Reading CPD file ******'
         write(ilog,'(a,f9.5)') 'File version:',filever(ichan)
         write(ilog,'(a)') 'Created on:'
         read(ichan) buffer
         write(ilog,'(a)') buffer
!.....read information from potfit file.
         write(ilog,'(a,i3)') 'cpd input file:  nr =',iwrt
         write(ilog,'(a)') 'vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv'
         read(ichan,err=999) lcount
         do i=1,lcount
            read(ichan,err=999) buffer
            write(ilog,'(a)') buffer
         enddo
         write(ilog,'(a)') '^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
      endif
!-----------------------------------------------------------------------
! read potfit parameters
!-----------------------------------------------------------------------
      read(ichan,err=999) cpdrank, (onedipot(f),f=1,ndof1)
!.....read information about the combination of the degrees of freedom.
      call rdgrddef(ichan,chkgrd,fdvr,mdvr,mmode,nspfdof1,&
           spfdof1,nmode1)
      call chksyserr(file,lerr)
      if (lerr) call stopnow

!-----------------------------------------------------------------------
! Initialise variables containing the information about
! the combined degrees of freedom in potfit
!-----------------------------------------------------------------------
      npmode=nmode1
      do m=1,nmode1
         npotdof(m)=nspfdof1(m)
         do n=1,nspfdof1(m)
            potdof(n,m)=spfdof1(n,m)
            dofspf1(spfdof1(n,m))=m
         enddo
      enddo
      do f=1,ndof1
         dofpot(f)=dofspf1(f)
      enddo

      maxpsub = 0
!-----------------------------------------------------------------------
! calculate psubdim for each mode.
!----------------------------------------------------------------------
      do m=1,npmode
         zeig=1
         do n=1,npotdof(m)
            f=potdof(n,m)
            f1=mdvr(f)
            zeig=zeig*gdim(f1)
         enddo
         psubdim(m)=zeig
         subdim1(m)=psubdim(m)
         maxpsub  = max(maxpsub,psubdim(m))
      enddo

      do f=1,ndof1
         loned(f) = onedipot(f).gt.0
      enddo

      return
  999 continue
      routine = 'cpdinfo'
      message = 'Error reading cpd file'
      call errormsg
      end subroutine cpdinfo_pre11

      end module rdnatpot

