!#######################################################################
! New version (Benjamin, December 2008)
!
! contains : operinfo,rdoper
!
!#######################################################################

     module rdopermod

use decimal, only: long, dop
use op1lib,  only: zerovxi, zerovxl
use logdat
use versions

use maxv
use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use dirdyn
use dd_db, only: preparedb
use gwptrj
use channels
use runopermod
use combined
use gapmod
use mlham, only: mlham_load
use operutil, only: findnath,mvkend

use iogrddef, only:rdgrddef
use iodvrdef, only:rddvrdef
use module_interface
use linint, only:li_browse
use hpsimod, only: gwpintordm,gwpcxm
use runinwfmod, only: inwfop
use runpropmod

use file_enquire

      implicit none

!-----------------------------------------------------------------------
! Information need by "fast" feature for the efficient use of Natpots
!
! lfastall      - true, if FAST algorithm for H(npot)*A is used for all
!                 natpots
! fastn         - No. of potfit operators set to use Fast algorithm
!
! lfast(n)      - true, if FAST algorithm for H(npot)*A is used
!
! natpmode(i,n) - mode for which natpot terms are pre-multiplied for the
!                 n-th natpot (i-th order "fast")
! nterm(n)      - the number of the first natpot term of the n-th natpot
! ntermlast(n)  - the number of the last natpot term of the n-th natpot
! mdimf(m,n)    - Number of pre-multiplied terms for mode m and natpot n
! fastorder(n)  - Order of the "fast" algorithm for the n-th natpot
! fastprem(i)   - Array of pre-multiplied terms
! fastnum(i)    - Array of terms to be taken with a given pre-multiplied
!                 term.
! fastdim(i)    - Number of all terms to be taken with a given
!                 pre-multiplied term.
! zfp(n)        - Pointer to fastprem array for given n
! zfn(n)        - Pointer to fastnum array for given n
! zfd(n)        - Pointer to fastdim array for given n
!-----------------------------------------------------------------------

      integer(long), save                                :: fastn
      logical(kind=4), save                              :: lfastall
      integer(long), dimension(:), allocatable, save     :: fastilabel,&
                                                            fastorder
      character(len=c2), dimension(:), allocatable, save :: fastlabel
      logical(kind=4), dimension(:), allocatable, save   :: lfast

      integer(long), dimension(:), allocatable, save   :: zfp,zfn,zfd
      integer(long), dimension(:), allocatable, save   :: nterm,ntermlast
      integer(long), dimension(:,:), allocatable, save :: natpmode,mdimf
      integer(long), dimension(:), allocatable, save   :: fastprem,&
                                                          fastnum, &
                                                          fastdim
! lopsetprop is set if operator changed in setoperpsi
      logical(kind=4), save :: lopsetprop = .false.

      private

      public :: operinfo,rdoper,assocdof,iskleg,get_num_klegs, &
                get_nth_kleg,get_num_wigs,num_kleg_sbas,num_wig_sbas, &
                iswig,setoperpsi,reinitext1d,nth_wig_sbas,&
                dealloc_setoperpsi,setoperprop,fillkoeff

      public :: fastn,lfastall,fastilabel,fastorder,fastlabel,lfast,&
                zfp,zfn,zfd,natpmode,mdimf,fastprem,fastnum,fastdim,&
                nterm,ntermlast
      public :: lopsetprop

! not used
      public :: nth_kleg_sbas

      contains

!#######################################################################
!
! OPERINFO
!
! reads information about operator needed to set up propagation
!  i.e. kzahl, hopsdim etc.
!
!#######################################################################

      subroutine operinfo(lerr,chkdvr,chkgrd)

      implicit none

      integer(long), intent(inout)                  :: chkgrd
      integer(long), intent(in)                     :: chkdvr
      logical(kind=4), intent(out)                  :: lerr

      routine='Operinfo'

      rewind(ioper)
      read(ioper,err=999) filever(ioper)

      if (filever(ioper) .gt. progver) then
         write(message,'(a,f8.5)') &
           'Oper file newer than program. Use version :',filever(ioper)
         call errormsg
      endif
!
! read file depending on version
!
      if (filever(ioper) .le. 8.1_dop) then
         message='Update oper file to at least version 8.4 '
         call errormsg

      else if (filever(ioper) .lt. 11.0_dop) then
         call operinfo_pre11(lerr,chkdvr,chkgrd)

      else
         call operinfo11(lerr,chkdvr,chkgrd)

      endif

      return

  999 continue
      routine='Operinfo'
      message='Error reading file version from oper file'
      call errormsg

      end subroutine operinfo

!#######################################################################
!
! OPERINFO
!
! reads information about operator needed to set up propagation
!  i.e. kzahl, hopsdim etc.
!
! If new information added, make sure to add default to operinfo_pre11
! to keep backwards compatibilty.
!#######################################################################

      subroutine operinfo11(lerr,chkdvr,chkgrd)

      implicit none

      integer(long)       :: f,k,s,h,m,ndof1,np,maxhpar1,nh,hl,nham,&
                             gf,i,n,ilnpot,ilcap,ilfchk,itmp,ierr
      integer(long)       :: ilsacas,muld,nmode1,j,&
                             iddtrans,iddwrdb,iklha,iloptcontract,lenpath,&
                             igausspot,idbsave,iddmctdh,igapderiv,iddmopt,&
                             iddgbtendec,igapusesym,igapcomp,igapuseinwf,&
                             igapsobol,igapvarmax,imalon,iddsoc,isocbas,&
                             igapparopt,ipestest,igapvarprint,igapethresh,&
                             idecoup,igapallgrd,irefdbrd,iqcsimulate,&
                             iddrddb,iupdhes,igapadiabwf,igaptest,isocdegen,&
                             ianalpes

      integer(long), intent(inout)                  :: chkgrd
      integer(long), intent(in)                     :: chkdvr
      integer(long), dimension(maxdim)              :: fdvr,mdvr,mmode,nspfdof1
      integer(long), dimension(maxdim,maxdim)       :: spfdof1
      integer(long)                                 :: ilbl,jlbl,sp
      integer(long), dimension(:), allocatable      :: ihamdum
      integer(long), dimension(:,:), allocatable    :: ihamdum2
      integer(long)                                 :: tapelength=0
      integer(long), dimension(maxtape)             :: tape1

      character(len=c1)                             :: file
      logical(kind=4), external                     :: int2log
      logical(kind=4), intent(out)                  :: lerr
      character(len=100)                            :: buffer
      character(len=c5)                             :: title
      character(len=c2)                             :: rdtag
      real(dop)                                     :: tmpgapalpha

      routine='Operinfo'

      if (logisopen) write(ilog,'(/,a)') '**** Reading OPER file ****'

      ilbl=0
      jlbl=0
      l_mlpf=.false.

!-----------------------------------------------------------------------
! read title and write to log file
!-----------------------------------------------------------------------
      rewind(ioper)
      do
  10     continue
         read(ioper,iostat=chkerr(1),err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'optitle') then
            read(ioper,err=999) title
            do while (title(1:13) .ne. '<end title>')
               if (logisopen) write(ilog,'(a)') trim(title)
               read(ioper,err=999) title
            enddo

         else if (rdtag(ilbl:jlbl) .eq. 'creation date') then
            read(ioper,err=999) buffer
            if (logisopen) write(ilog,'(/,a)') 'Created on:'
            if (logisopen) write(ilog,'(a)') trim(buffer)

         else if (rdtag(ilbl:jlbl) .eq. 'end opertitle') then
            exit

         endif

      enddo

  100 continue

!-----------------------------------------------------------------------
! read system definition
!-----------------------------------------------------------------------
      file='Oper'
      call zerovxi(chkerr,nerr)
      call rddvrdef(ioper,chkdvr,ndof1,fdvr)
      call rdgrddef(ioper,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)

      call chksyserr(file,lerr)
      if (lerr) return

!-----------------------------------------------------------------------
! read operator information and definition
!
! NB parameters (apar / rpar) not read here or else they would overwrite 
! the parameter list set up in einpara
!-----------------------------------------------------------------------
      rewind(ioper)

      do
   20    continue
         read(ioper,err=20,end=200) rdtag
         if (rdtag(1:1) .ne. '<') cycle
 
         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'mass') then
            read(ioper,err=999) (mass(f),f=1,ndof)

         else if (rdtag(ilbl:jlbl) .eq. 'nopmode') then
            read(ioper,err=999) nopmode
            if (data_enquire .gt. 0) call wrenquire(rdtag,nopmode)

         else if (rdtag(ilbl:jlbl) .eq. 'nopdof') then
            read(ioper,err=999) nopdof
            if (data_enquire .gt. 0) call wrenquire(rdtag,nopdof)

         else if (rdtag(ilbl:jlbl) .eq. 'nddstate') then
            read(ioper,err=999) nddstate
            if (data_enquire .gt. 0) call wrenquire(rdtag,nddstate)

         else if (rdtag(ilbl:jlbl) .eq. 'nbasis') then
            read(ioper,err=999) nbasis
            if (data_enquire .gt. 0) call wrenquire(rdtag,nbasis)

         else if (rdtag(ilbl:jlbl) .eq. 'tapelength') then
            read (ioper,err=999) tapelength
            if (data_enquire .gt. 0) call wrenquire(rdtag,tapelength)

         else if (rdtag(ilbl:jlbl) .eq. 'tape') then
            read (ioper,err=999) (tape1(m),m=1,tapelength)

         else if (rdtag(ilbl:jlbl) .eq. 'nelectron') then
            read(ioper,err=999) nelectron

         else if (rdtag(ilbl:jlbl) .eq. 'nocc') then
            read(ioper,err=999) nocc

         else if (rdtag(ilbl:jlbl) .eq. 'nclosed') then
            read(ioper,err=999) nclosed

         else if (rdtag(ilbl:jlbl) .eq. 'ncas') then
            read(ioper,err=999) ncas

         else if (rdtag(ilbl:jlbl) .eq. 'nsmult') then
            read(ioper,err=999) nsmult

         else if (rdtag(ilbl:jlbl) .eq. 'ncasel') then
            read(ioper,err=999) ((ncasel(n,i),n=1,nsmult),i=1,2)

         else if (rdtag(ilbl:jlbl) .eq. 'nconf') then
            read(ioper,err=999) ((nconf(n,i),n=1,nsmult),i=1,2)

         else if (rdtag(ilbl:jlbl) .eq. 'opmodelabel') then
            read(ioper,err=999) (modelabel(f),f=ndof+1,nopdof)

         else if (rdtag(ilbl:jlbl) .eq. 'opnspfdof') then
            read(ioper,err=999) (nspfdof(m),m=nmode+1,nopmode)

         else if (rdtag(ilbl:jlbl) .eq. 'opspfdof') then
            read(ioper,err=999) ((spfdof(n,m),n=1,nspfdof(m)),m=nmode+1,nopmode)

         else if (rdtag(ilbl:jlbl) .eq. 'opdofspf') then
            read(ioper,err=999) (dofspf(f),f=ndof+1,nopdof)

         else if (rdtag(ilbl:jlbl) .eq. 'loptcontract') then
            read(ioper,err=999) iloptcontract
            loptcontract=int2log(iloptcontract)

         else if (rdtag(ilbl:jlbl) .eq. 'lanalpes') then
            read(ioper,err=999) ianalpes
            lanalpes=int2log(ianalpes)

         else if (rdtag(ilbl:jlbl) .eq. 'hamdim') then
            read(ioper,err=999) hamdim

         else if (rdtag(ilbl:jlbl) .eq. 'ktot') then
            read(ioper,err=999) ktot
            if (data_enquire .gt. 0) call wrenquire(rdtag,ktot)

         else if (rdtag(ilbl:jlbl) .eq. 'fkoe') then
            read(ioper,err=999) fkoe
            if (data_enquire .gt. 0) call wrenquire(rdtag,fkoe)

         else if (rdtag(ilbl:jlbl) .eq. 'ftime') then
            read(ioper,err=999) ftime
            if (data_enquire .gt. 0) call wrenquire(rdtag,ftime)

         else if (rdtag(ilbl:jlbl) .eq. 'mkoe') then
            read(ioper,err=999) mkoe
            if (mkoe .gt. 0) subdim(mkoe)=1
            if (data_enquire .gt. 0) call wrenquire(rdtag,mkoe)

         else if (rdtag(ilbl:jlbl) .eq. 'hamlab') then
            read(ioper,err=999) (hamlab(nham),nham=1,hamdim)

         else if (rdtag(ilbl:jlbl) .eq. 'nodiag') then
            allocate(ihamdum(hamdim))
            read(ioper,err=999) (ihamdum(nham),nham=1,hamdim)
            do nham=1,hamdim
               nodiag(nham)=int2log(ihamdum(nham))
            enddo
            deallocate(ihamdum)
            if (data_enquire .gt. 0) call wrenquire(rdtag,nodiag,hamdim)

         else if (rdtag(ilbl:jlbl) .eq. 'zham') then
            read(ioper,err=999) (zham(nham),nham=1,hamdim)

         else if (rdtag(ilbl:jlbl) .eq. 'khtot') then
            read(ioper,err=999) (khtot(nham),nham=1,hamdim)

         else if (rdtag(ilbl:jlbl) .eq. 'kzahl') then
            read(ioper,err=999) (kzahl(nham),nham=1,hamdim)

         else if (rdtag(ilbl:jlbl) .eq. 'nmulpot') then
            read(ioper,err=999) (nmulpot(nham),nham=1,hamdim)

         else if (rdtag(ilbl:jlbl) .eq. 'ltdh') then
            allocate(ihamdum(hamdim))
            read(ioper,err=999) (ihamdum(nham),nham=1,hamdim)
            do nham=1,hamdim
               ltdh(nham)=int2log(ihamdum(nham))
            enddo
            deallocate(ihamdum)

         else if (rdtag(ilbl:jlbl) .eq. 'khzahl') then
            read(ioper,err=999) &
              (((khzahl(m,s,nham),m=1,nmode),s=1,nstate),nham=1,hamdim)

         else if (rdtag(ilbl:jlbl) .eq. 'zhun') then
            read(ioper,err=999) &
               (((zhun(m,s,nham),m=1,nmode),s=1,nstate),nham=1,hamdim)

         else if (rdtag(ilbl:jlbl) .eq. 'iqconst') then
            read(ioper,err=999) (iqconst(f),f=1,ndof)

         else if (rdtag(ilbl:jlbl) .eq. 'rqconst') then
            read(ioper,err=999) (rqconst(f),f=1,ndof)

         else if (rdtag(ilbl:jlbl) .eq. 'hamilton') then
            read(ioper,err=999) ((hamilton(f,k),f=1,nopdof),k=1,ktot)

         else if (rdtag(ilbl:jlbl) .eq. 'diag') then
            allocate(ihamdum2(nmode,ktot))
            read(ioper,err=999) ((ihamdum2(m,k),m=1,nmode),k=1,ktot)
            do k = 1,ktot
               do m = 1,nmode
                  diag(m,k) = int2log(ihamdum2(m,k))
               enddo
            enddo
            deallocate(ihamdum2)

         else if (rdtag(ilbl:jlbl) .eq. 'kinop') then
            read(ioper,err=999) ((kinop(m,k),m=1,nmode),k=1,ktot)
         
         else if (rdtag(ilbl:jlbl) .eq. 'hsym') then
            read(ioper,iostat=ierr) ((hsym(m,k),m=1,nopmode),k=1,ktot)
            if (ierr .ne. 0) then     ! earlier versions stored only nmode
               backspace(ioper)
               read(ioper,err=999) ((hsym(m,k),m=1,nmode),k=1,ktot)
            endif
         
         else if (rdtag(ilbl:jlbl) .eq. 'hlsym') then
            read(ioper,err=999) ((hlsym(m,k),m=1,nmode),k=1,ktot)
         
         else if (rdtag(ilbl:jlbl) .eq. 'kact') then
            read(ioper,err=999) (kact(k),k=1,ktot)
         
         else if (rdtag(ilbl:jlbl) .eq. 'ktwin') then
            read(ioper,err=999) (ktwin(k),k=1,ktot)
      
         else if (rdtag(ilbl:jlbl) .eq. 'ki') then
            read(ioper,err=999) (ki(k),k=1,ktot)
      
         else if (rdtag(ilbl:jlbl) .eq. 'kf') then
            read(ioper,err=999) (kf(k),k=1,ktot)

         else if (rdtag(ilbl:jlbl) .eq. 'ksi') then
            read(ioper,err=999) (ksi(k),k=1,ktot)

         else if (rdtag(ilbl:jlbl) .eq. 'ksf') then
            read(ioper,err=999) (ksf(k),k=1,ktot)

         else if (rdtag(ilbl:jlbl) .eq. 'klha') then
            read(ioper,err=999) (klha(k),k=1,ktot)

         else if (rdtag(ilbl:jlbl) .eq. 'fterm') then
            read(ioper,err=999) (fterm(k),k=1,ktot)

         else if (rdtag(ilbl:jlbl) .eq. 'hmlop') then
            read(ioper,err=999) (hmlop(k),k=1,ktot)
            do k=1,ktot  !! Check if there is an mlpf
               if(hmlop(k).ne.0) then
                  l_mlpf=.true.
                  exit
               endif
            enddo

         else if (rdtag(ilbl:jlbl) .eq. 'htmdim') then
            read(ioper,err=999) htmdim

         else if (rdtag(ilbl:jlbl) .eq. 'htmtype') then
            read(ioper,err=999) (htmtype(h),h=1,htmdim)

         else if (rdtag(ilbl:jlbl) .eq. 'htmsym') then
            read(ioper,err=999) (htmsym(h),h=1,htmdim)

         else if (rdtag(ilbl:jlbl) .eq. 'htmdof') then
            read(ioper,err=999) (htmdof(h),h=1,htmdim)

         else if (rdtag(ilbl:jlbl) .eq. 'htmadof') then
            read(ioper, err=999) (htmadof(h),h=1,htmdim) ! NEWKLEG

         else if (rdtag(ilbl:jlbl) .eq. 'htmndof') then
            read(ioper, err=999) (htmndof(h),h=1,htmdim)
      
         else if (rdtag(ilbl:jlbl) .eq. 'htmmode') then
            read(ioper,err=999) (htmmode(h),h=1,htmdim)
      
         else if (rdtag(ilbl:jlbl) .eq. 'htmmuld') then
            read(ioper,err=999) (htmmuld(h),h=1,htmdim)
         
         else if (rdtag(ilbl:jlbl) .eq. 'htmform') then
            read(ioper,err=999) (htmform(h),h=1,htmdim)
         
         else if (rdtag(ilbl:jlbl) .eq. 'htmshift') then
            read(ioper,err=999) (htmshift(h),h=1,htmdim)
         
         else if (rdtag(ilbl:jlbl) .eq. 'htmderiv') then
            read(ioper,err=999) (htmderiv(h),h=1,htmdim)
         
         else if (rdtag(ilbl:jlbl) .eq. 'ddsi') then
            read(ioper,err=999) (ddsi(h),h=1,htmdim)

         else if (rdtag(ilbl:jlbl) .eq. 'ddsf') then
            read(ioper,err=999) (ddsf(h),h=1,htmdim)

         else if (rdtag(ilbl:jlbl) .eq. 'minfac') then
            read(ioper,err=999) minfac

         else if (rdtag(ilbl:jlbl) .eq. 'htmfac') then
            read(ioper,err=999) (htmfac(h),h=1,htmdim)

         else if (rdtag(ilbl:jlbl) .eq. 'hterm') then
            read(ioper,err=999) ((hterm(nh,h),nh=1,htmfac(h)),h=1,htmdim)

         else if (rdtag(ilbl:jlbl) .eq. 'mulddim') then
            read(ioper,err=999) mulddim

         else if (rdtag(ilbl:jlbl) .eq. 'muldsize') then
            read(ioper,err=999) (muldsize(muld),muld=1,mulddim)

         else if (rdtag(ilbl:jlbl) .eq. 'nmulddof') then
            read(ioper,err=999) (nmulddof(muld),muld=1,mulddim)

         else if (rdtag(ilbl:jlbl) .eq. 'mulddof') then
            read(ioper,err=999) &
                ((mulddof(gf,muld),gf=1,nmulddof(muld)),muld=1,mulddim)

         else if (rdtag(ilbl:jlbl) .eq. 'mulddof2') then
            read(ioper,err=999) &
                ((mulddof2(gf,muld),gf=1,nmulddof(muld)),muld=1,mulddim)

         else if (rdtag(ilbl:jlbl) .eq. 'hopdim') then
            read(ioper,err=999) hopdim

         else if (rdtag(ilbl:jlbl) .eq. 'hoplab') then
            read(ioper,err=999) (hoplab(hl),hl=1,hopdim)

         else if (rdtag(ilbl:jlbl) .eq. 'hopilab') then
            read(ioper,err=999) (hopilab(hl),hl=1,hopdim)

         else if (rdtag(ilbl:jlbl) .eq. 'hopidef') then
            read(ioper,err=999) (hopidef(hl),hl=1,hopdim)

         else if (rdtag(ilbl:jlbl) .eq. 'hopfile') then
            read(ioper,err=999) (hopfile(hl),hl=1,hopdim)

         else if (rdtag(ilbl:jlbl) .eq. 'maxhpar') then
            read(ioper,err=999) maxhpar1

         else if (rdtag(ilbl:jlbl) .eq. 'hopipar') then
            read(ioper,err=999) ((hopipar(np,hl),np=1,maxhpar1),hl=1,hopdim)

         else if (rdtag(ilbl:jlbl) .eq. 'hoprpar') then
            read(ioper,err=999) ((hoprpar(np,hl),np=1,maxhpar1),hl=1,hopdim)

         else if (rdtag(ilbl:jlbl) .eq. 'npesopt') then
            read(ioper,err=999) npesopt

         else if (rdtag(ilbl:jlbl) .eq. 'zpesopts') then
            read(ioper,err=999) ((zpesopts(np,n),np=1,maxhpar1),n=1,npesopt)

         else if (rdtag(ilbl:jlbl) .eq. 'pesopts') then
            read(ioper,err=999) (pesopts(n),n=1,npesopt)

         else if (rdtag(ilbl:jlbl) .eq. 'autocap') then
            read(ioper,err=999) (autocap(k),k=1,ktot)

         else if (rdtag(ilbl:jlbl) .eq. 'nstatecap') then
            read(ioper,err=999) (nstatecap(k),k=1,ktot)

         else if (rdtag(ilbl:jlbl) .eq. 'autoabs') then
            read(ioper,err=999) (autoabs(k),k=1,ktot)

         else if (rdtag(ilbl:jlbl) .eq. 'autoflux') then
            read(ioper,err=999) (autoflux(k),k=1,ktot)

         else if (rdtag(ilbl:jlbl) .eq. 'hopsdim') then
            read(ioper,err=999) hopsdim
            if (data_enquire .gt. 0) call wrenquire(rdtag,hopsdim)

         else if (rdtag(ilbl:jlbl) .eq. 'zhop') then
            read(ioper,err=999) (zhop(h),h=1,htmdim)
            if (data_enquire .gt. 0) call wrenquire(rdtag,zhop,htmdim)

         else if (rdtag(ilbl:jlbl) .eq. 'lcap') then
            read(ioper) ilcap
            lcap  = int2log(ilcap)

         else if (rdtag(ilbl:jlbl) .eq. 'lnpot') then
            read(ioper) ilnpot
            lnpot = int2log(ilnpot)

         else if (rdtag(ilbl:jlbl) .eq. 'modc') then
            read(ioper) modc

         else if (rdtag(ilbl:jlbl) .eq. 'nrdf') then
            read(ioper) nrdf

         else if (rdtag(ilbl:jlbl) .eq. 'mdim') then
            read(ioper) ((mdim(m,n),m=1,nmode),n=1,nrdf)

         else if (rdtag(ilbl:jlbl) .eq. 'mmodc') then
            read(ioper) (mmodc(n),n=1,nrdf)

         else if (rdtag(ilbl:jlbl) .eq. 'ncutted') then
            read(ioper) (ncutted(n),n=1,nrdf)

         else if (rdtag(ilbl:jlbl) .eq. 'deflabel') then
            read(ioper) (deflabel(n),n=1,maxdef)

         else if (rdtag(ilbl:jlbl) .eq. 'deflabop') then
            read(ioper) (deflabop(n),n=1,maxdef)

         else if (rdtag(ilbl:jlbl) .eq. 'splpaths') then
            read(ioper) (splpaths(n),n=1,nsplpaths)

         else if (rdtag(ilbl:jlbl) .eq. 'splnames') then
            read(ioper) (splnames(n),n=1,nsplnames)

         else if (rdtag(ilbl:jlbl) .eq. 'splunits') then
            read(ioper) (splunits(1:4,n),n=1,nsplnames)

         else if (rdtag(ilbl:jlbl) .eq. 'iLMR') then
            read(ioper) iLMR

         else if (rdtag(ilbl:jlbl) .eq. 'hamLMR') then
            read(ioper) ((hamLMR(i,n),i=1,iLMR),n=1,3)

         else if (rdtag(ilbl:jlbl) .eq. 'mlham_optree') then
           do nham=1,hamdim
              call mlham_load(ioper,nham,tape1,lrunml)
           enddo

         else if (rdtag(ilbl:jlbl) .eq. 'lhaorder') then
            read(ioper,err=999) lhaorder

         else if (rdtag(ilbl:jlbl) .eq. 'lklha') then
            read(ioper,err=999) iklha
            lklha = int2log(iklha)

         else if (rdtag(ilbl:jlbl) .eq. 'lddtrans') then
            read(ioper,err=999) iddtrans
            lddtrans = int2log(iddtrans)

         else if (rdtag(ilbl:jlbl) .eq. 'ltshtrans') then
            read(ioper,err=999) iddtrans
            ltshtrans = int2log(iddtrans)

         else if (rdtag(ilbl:jlbl) .eq. 'lrefdbrd') then
            read(ioper,err=999) irefdbrd
            lrefdbrd = int2log(irefdbrd)

         else if (rdtag(ilbl:jlbl) .eq. 'dddiab') then
            read(ioper,err=999) dddiab

         else if (rdtag(ilbl:jlbl) .eq. 'ldiabdb') then
            read(ioper,err=999) itmp
            ldiabdb = int2log(itmp)

         else if (rdtag(ilbl:jlbl) .eq. 'lqcsimulate') then
            read(ioper,err=999) itmp
            lqcsimulate = int2log(itmp)

         else if (rdtag(ilbl:jlbl) .eq. 'lncoupl') then
            read(ioper,err=999) itmp
            lncoupl = int2log(itmp)

         else if (rdtag(ilbl:jlbl) .eq. 'lupdhes') then
            read(ioper,err=999) itmp
            lupdhes = int2log(itmp)

         else if (rdtag(ilbl:jlbl) .eq. 'ddcoo') then
            read(ioper,err=999) ddcoo

         else if (rdtag(ilbl:jlbl) .eq. 'ndofdd') then
            read(ioper,err=999) ndofdd
            if (data_enquire .gt. 0) call wrenquire(rdtag,ndofdd)

         else if (rdtag(ilbl:jlbl) .eq. 'ndofddpes') then
            read(ioper,err=999) ndofddpes
            if (data_enquire .gt. 0) call wrenquire(rdtag,ndofddpes)

         else if (rdtag(ilbl:jlbl) .eq. 'ddtransdim') then
            read(ioper,err=999) ddtransdim

         else if (rdtag(ilbl:jlbl) .eq. 'natm') then
            read(ioper,err=999) natm
            if (data_enquire .gt. 0) call wrenquire(rdtag,natm)
           
         else if (rdtag(ilbl:jlbl) .eq. 'natmtsh') then
            read(ioper,err=999) natmtsh

         else if (rdtag(ilbl:jlbl) .eq. 'ndoftsh') then
            read(ioper,err=999) ndoftsh

         else if (rdtag(ilbl:jlbl) .eq. 'ddextcmd') then
            read(ioper,err=999) ddextcmd

         else if (rdtag(ilbl:jlbl) .eq. 'dde0') then
            read(ioper,err=999) dde0
            if (data_enquire .gt. 0) call wrenquire(rdtag,dde0)

         else if (rdtag(ilbl:jlbl) .eq. 'lfchk') then
            read(ioper,err=999) ilfchk
            lfchk=int2log(ilfchk)

         else if (rdtag(ilbl:jlbl) .eq. 'fchkcmd') then
            read(ioper,err=999) fchkcmd

         else if (rdtag(ilbl:jlbl) .eq. 'ddmindb') then
            read(ioper,err=999) ddmindb
            if (data_enquire .gt. 0) call wrenquire(rdtag,ddmindb)

         else if (rdtag(ilbl:jlbl) .eq. 'dddbvar') then
            read(ioper,err=999) dddbvar

         else if (rdtag(ilbl:jlbl) .eq. 'dddbhar') then
            read(ioper,err=999) dddbhar

         else if (rdtag(ilbl:jlbl) .eq. 'dbminwgt') then
            read(ioper,err=999) dbminwgt

         else if (rdtag(ilbl:jlbl) .eq. 'dbdisp') then
            read(ioper,err=999) dbdisp

         else if (rdtag(ilbl:jlbl) .eq. 'dbdegen') then
            read(ioper,err=999) dbdegen
            if (data_enquire .gt. 0) call wrenquire(rdtag,dbdegen)

         else if (rdtag(ilbl:jlbl) .eq. 'dbmaxe') then
            read(ioper,err=999) dbmaxe
            if (data_enquire .gt. 0) call wrenquire(rdtag,dbmaxe)

         else if (rdtag(ilbl:jlbl) .eq. 'dbpntgp') then
            read(ioper,err=999) dbpntgp
            if (data_enquire .gt. 0) call wrenquire(rdtag,dbpntgp)

         else if (rdtag(ilbl:jlbl) .eq. 'lddrddb') then
            read(ioper,err=999) iddrddb
            lddrddb=int2log(iddrddb)

         else if (rdtag(ilbl:jlbl) .eq. 'lddwrdb') then
            read(ioper,err=999) iddwrdb
            lddwrdb=int2log(iddwrdb)

         else if (rdtag(ilbl:jlbl) .eq. 'lupdhes') then
            read(ioper,err=999) iupdhes
            lupdhes=int2log(iupdhes)

         else if (rdtag(ilbl:jlbl) .eq. 'lqcsimulate') then
            read(ioper,err=999) iqcsimulate
            lqcsimulate=int2log(iqcsimulate)

         else if (rdtag(ilbl:jlbl) .eq. 'ldbsave') then
            read(ioper,err=999) idbsave
            ldbsave=int2log(idbsave)

         else if (rdtag(ilbl:jlbl) .eq. 'lgausspot') then
            read(ioper,err=999) igausspot
            lgausspot=int2log(igausspot)

         else if (rdtag(ilbl:jlbl) .eq. 'lpestest') then
            read(ioper,err=999) ipestest
            lpestest=int2log(ipestest)

         else if (rdtag(ilbl:jlbl) .eq. 'idpestest') then
            read(ioper,err=999) idpestest

         else if (rdtag(ilbl:jlbl) .eq. 'dddvmin') then
            read(ioper,err=999) dddvmin

         else if (rdtag(ilbl:jlbl) .eq. 'lsacas') then
            read(ioper,err=999) ilsacas
            lsacas=int2log(ilsacas)

         else if (rdtag(ilbl:jlbl) .eq. 'ddlaenge') then
            read(ioper,err=999) ddlaenge0

         else if (rdtag(ilbl:jlbl) .eq. 'dbdlength') then
            read(ioper,err=999) dbdlength

         else if (rdtag(ilbl:jlbl) .eq. 'ddname') then
            read(ioper,err=999) ddname0

         else if (rdtag(ilbl:jlbl) .eq. 'gwptrjlaenge') then
            read(ioper,err=999) gwptrjlaenge0

         else if (rdtag(ilbl:jlbl) .eq. 'gwptrjdir') then
            read(ioper,err=999) gwptrjdir0

         else if (rdtag(ilbl:jlbl) .eq. 'nsocstate') then
            read(ioper,err=999) nsocstate

         else if (rdtag(ilbl:jlbl) .eq. 'istatemap') then
            read(ioper,err=999) (istatemap(s),s=1,nddstate)

         else if (rdtag(ilbl:jlbl) .eq. 'imultmap') then
            read(ioper,err=999) (imultmap(s),s=1,nddstate)

         else if (rdtag(ilbl:jlbl) .eq. 'imultqc') then
            read(ioper,err=999) (imultqc(s),s=1,nddstate)

         else if (rdtag(ilbl:jlbl) .eq. 'dercpdim') then
            read(ioper,err=999) dercpdim

         else if (rdtag(ilbl:jlbl) .eq. 'idxmult') then
            read(ioper,err=999) (idxmult(s),s=1,nsmult)

         else if (rdtag(ilbl:jlbl) .eq. 'imultblk') then
            read(ioper,err=999) (imultblk(s),s=1,nsmult)

         else if (rdtag(ilbl:jlbl) .eq. 'imultoff') then
            read(ioper,err=999) (imultoff(s),s=1,nsmult)

         else if (rdtag(ilbl:jlbl) .eq. 'dercpblk') then
            read(ioper,err=999) (dercpblk(s),s=1,nsmult)

         else if (rdtag(ilbl:jlbl) .eq. 'dercpoff') then
            read(ioper,err=999) (dercpoff(s),s=1,nsmult)

         else if (rdtag(ilbl:jlbl) .eq. 'stoff') then
            read(ioper,err=999) (stoff(s),s=1,nsmult)

         else if (rdtag(ilbl:jlbl) .eq. 'lddsoc') then
            read(ioper,err=999) iddsoc
            lddsoc=int2log(iddsoc)

         else if (rdtag(ilbl:jlbl) .eq. 'lsocdegen') then
            read(ioper,err=999) isocdegen
            lsocdegen=int2log(isocdegen)

         else if (rdtag(ilbl:jlbl) .eq. 'lsocbas') then
            read(ioper,err=999) isocbas
            lsocbas=int2log(isocbas)

         else if (rdtag(ilbl:jlbl) .eq. 'qcmethod') then
            read(ioper,err=999) qcmethod
            if (data_enquire .gt. 0) call wrenquire(rdtag,qcmethod)

         else if (rdtag(ilbl:jlbl) .eq. 'qcprog') then
            read(ioper,err=999) qcprog
            if (data_enquire .gt. 0) call wrenquire(rdtag,qcprog)

         else if (rdtag(ilbl:jlbl) .eq. 'qcsubform') then
            read(ioper,err=999) qcsubform

         else if (rdtag(ilbl:jlbl) .eq. 'xatnam') then
            read(ioper) (xatnam(i),i=1,natm)

         else if (rdtag(ilbl:jlbl) .eq. 'xmass') then
            read(ioper) (xmass(i),i=1,natm)

         else if (rdtag(ilbl:jlbl) .eq. 'ddxcoo0') then
            read(ioper) (ddxcoo0(i),i=1,ndofddpes)
            if (data_enquire .gt. 0) & 
                call wrenquire(rdtag,ddxcoo0,ndofddpes,3)

         else if (rdtag(ilbl:jlbl) .eq. 'ddtransf') then
            read(ioper) ((ddtransf(j,i),j=1,ndofddpes),i=1,ndofdd)
            if (data_enquire .gt. 0) & 
                call wrenquire(rdtag,ddtransf,ndofddpes,ndofdd,3)

         else if (rdtag(ilbl:jlbl) .eq. 'ddtransb') then
            read(ioper) ((ddtransb(j,i),j=1,ndofdd),i=1,ndofddpes)
            if (data_enquire .gt. 0) & 
                call wrenquire(rdtag,ddtransb,ndofdd,ndofddpes,3)

         else if (rdtag(ilbl:jlbl) .eq. 'tshatnam') then
            read(ioper) (tshatnam(i),i=1,natmtsh)

         else if (rdtag(ilbl:jlbl) .eq. 'tshmass') then
            read(ioper) (tshmass(i),i=1,natmtsh)

         else if (rdtag(ilbl:jlbl) .eq. 'tshxcoo0') then
            read(ioper) (tshxcoo0(i),i=1,ndoftsh)

         else if (rdtag(ilbl:jlbl) .eq. 'tshtransf') then
            read(ioper) ((tshtransf(j,i),j=1,ndoftsh),i=1,nspfdof(1))

         else if (rdtag(ilbl:jlbl) .eq. 'tshtransb') then
            read(ioper) ((tshtransb(j,i),j=1,nspfdof(1)),i=1,ndoftsh)

         else if (rdtag(ilbl:jlbl) .eq. 'coin') then
            read(ioper) (xcoin(i),i=1,ndofddpes)

         else if (rdtag(ilbl:jlbl) .eq. 'refpoint') then
            read(ioper) (xrefpoint(i),i=1,ndofddpes)

         else if (rdtag(ilbl:jlbl) .eq. 'dcp0') then
            read(ioper) (xdcp0(i),i=1,ndofddpes)

         else if (rdtag(ilbl:jlbl) .eq. 'gd0') then
            read(ioper) (xgd0(i),i=1,ndofddpes)

         else if (rdtag(ilbl:jlbl) .eq. 'cicocut') then
            read(ioper) cicocut

         else if (rdtag(ilbl:jlbl) .eq. 'lddmctdh') then
            read(ioper,err=999) iddmctdh
            lddmctdh=int2log(iddmctdh)

         else if (rdtag(ilbl:jlbl) .eq. 'ldecoup') then
            read(ioper,err=999) idecoup
            ldecoup=int2log(idecoup)

         else if (rdtag(ilbl:jlbl) .eq. 'lddmopt') then
            read(ioper,err=999) iddmopt
            lddmopt=int2log(iddmopt)

         else if (rdtag(ilbl:jlbl) .eq. 'lddgbtendec') then
            read(ioper,err=999) iddgbtendec
            lddgbtendec=int2log(iddgbtendec)

         else if (rdtag(ilbl:jlbl) .eq. 'ddmorder') then
            read(ioper,err=999) ddmorder

         else if (rdtag(ilbl:jlbl) .eq. 'iddmtest') then
            read(ioper,err=999) iddmtest

         else if (rdtag(ilbl:jlbl) .eq. 'lgaptest') then
            read(ioper,err=999) igaptest
            lgaptest=int2log(igaptest)

         else if (rdtag(ilbl:jlbl) .eq. 'ddmoptord') then
            read(ioper,err=999) ddmoptord

         else if (rdtag(ilbl:jlbl) .eq. 'lgapderiv') then
            read(ioper,err=999) igapderiv
            lgapderiv=int2log(igapderiv)

         else if (rdtag(ilbl:jlbl) .eq. 'lgapusesym') then
            read(ioper,err=999) igapusesym
            lgapusesym=int2log(igapusesym)

         else if (rdtag(ilbl:jlbl) .eq. 'lgapuseinwf') then
            read(ioper,err=999) igapuseinwf
            lgapuseinwf=int2log(igapuseinwf)

         else if (rdtag(ilbl:jlbl) .eq. 'lgapvarprint') then
            read(ioper,err=999) igapvarprint
            lgapvarprint=int2log(igapvarprint)

         else if (rdtag(ilbl:jlbl) .eq. 'lgapsobol') then
            read(ioper,err=999) igapsobol
            lgapsobol=int2log(igapsobol)

         else if (rdtag(ilbl:jlbl) .eq. 'lgapadiabwf') then
            read(ioper,err=999) igapadiabwf
            lgapadiabwf=int2log(igapadiabwf)

         else if (rdtag(ilbl:jlbl) .eq. 'lgapallgrd') then
            read(ioper,err=999) igapallgrd
            lgapallgrd=int2log(igapallgrd)

         else if (rdtag(ilbl:jlbl) .eq. 'lgapvarmax') then
            read(ioper,err=999) igapvarmax
            lgapvarmax=int2log(igapvarmax)

         else if (rdtag(ilbl:jlbl) .eq. 'lgapparopt') then
            read(ioper,err=999) igapparopt
            lgapparopt=int2log(igapparopt)

         else if (rdtag(ilbl:jlbl) .eq. 'gapker') then
            read(ioper,err=999) (gapker(n),n=1,ndofdd)

         else if (rdtag(ilbl:jlbl) .eq. 'gapkerscl') then
            read(ioper,err=999) (gapkerscl(n),n=1,ndofdd)

         else if (rdtag(ilbl:jlbl) .eq. 'gapalpha') then
! Slightly clunky for backward compatibility.
            read(ioper,err=998) (gapalpha(n),n=1,ndofdd)
            cycle
998         read(ioper,err=999) tmpgapalpha
            gapalpha(1:ndofdd) = tmpgapalpha

         else if (rdtag(ilbl:jlbl) .eq. 'optgapalpha') then
            read(ioper,err=999) optgapalpha

         else if (rdtag(ilbl:jlbl) .eq. 'gapsigma') then
            read(ioper,err=999) gapsigma

         else if (rdtag(ilbl:jlbl) .eq. 'gapkfac') then
            read(ioper,err=999) gapkfac

         else if (rdtag(ilbl:jlbl) .eq. 'gapvartol') then
            read(ioper,err=999) gapvartol

         else if (rdtag(ilbl:jlbl) .eq. 'gapsd') then
            read(ioper,err=999) gapsd

         else if (rdtag(ilbl:jlbl) .eq. 'gapsample') then
            read(ioper,err=999) gapsample

         else if (rdtag(ilbl:jlbl) .eq. 'gapeshift') then
            read(ioper,err=999) gapeshift

         else if (rdtag(ilbl:jlbl) .eq. 'gapethresh') then
            read(ioper,err=999) gapethresh

         else if (rdtag(ilbl:jlbl) .eq. 'lgapethresh') then
            read(ioper,err=999) igapethresh
            lgapethresh=int2log(igapethresh)

         else if (rdtag(ilbl:jlbl) .eq. 'lgapcomp') then
            read(ioper,err=999) igapcomp
            lgapcomp=int2log(igapcomp)

         else if (rdtag(ilbl:jlbl) .eq. 'ngapcomp') then
            read(ioper,err=999) ngapcomp

         else if (rdtag(ilbl:jlbl) .eq. 'ddgbsing') then
            read(ioper,err=999) ddgbsing

         else if (rdtag(ilbl:jlbl) .eq. 'ddgbsingss1') then
            read(ioper,err=999) ddgbsingss1

         else if (rdtag(ilbl:jlbl) .eq. 'ddgbtens') then
            read(ioper,err=999) ddgbtens

         else if (rdtag(ilbl:jlbl) .eq. 'lmalon') then
            read(ioper,err=999) imalon
            lmalon=int2log(imalon)

         endif
      enddo
  200 continue

!-----------------------------------------------------------------------
! if reading grid info, store tape if present for a ML calculation
!-----------------------------------------------------------------------
      if (chkgrd .eq. 1 .and. tapelength .gt. 0) then
         tape(:) = tape1(:)
      endif

!-----------------------------------------------------------------------
! runs checks
!-----------------------------------------------------------------------
      if (dentype.eq.1) then
         if (lcap) then
            routine='operinfo (rdoper.F)'
            message='CAPs not implemented for densities (type I).'
            call errormsg
         endif
      endif
      if (dentype.ne.0) then  
         if (nmulpot(1).gt.0) then
            routine='operinfo (rdoper.F)'
            message='MULPOT not implemented for densities.'
            call errormsg
         endif
      endif

      ldissop = .false.
      do k=1,kzahl(1)
         if (kact(k).gt.1) ldissop = .true.
      enddo

      if (dentype .eq. 0) then
         if (ldissop) then
            routine='operinfo (rdoper.F)'
            message='Dissop not implemented for wave functions.'
            call errormsg
         endif
      else
         if (ldissin) then
            if (.not. ldissop) then
               routine='operinfo (rdoper.F)'
               message='No dissipative operators found.'
               call warnmsg1
            endif
         else
            do k=1,kzahl(1)
               if (kact(k).gt.1) kact(k)=0
            enddo
         endif
      endif            

! form full path for DD DB
      if (ldd.or.lddmctdh) then
         ddname = ddname0
         ddlaenge = ddlaenge0
         if (ddname(1:1) .ne. '/') then
            lenpath = index(ddpath,' ')-1
            if (lenpath .gt. 0) then
               if (ddpath(lenpath:lenpath) .ne. '/') then
                  ddname = ddpath(1:lenpath)//'/'//ddname0
               else
                  ddname = ddpath(1:lenpath)//ddname0
               endif
               ddlaenge=index(ddname,' ')-1
               if (ddname(1:1) .ne. '/') call abspath(ddname,ddlaenge)
            else
               call abspath(ddname,ddlaenge)
            endif
         endif
      endif

      ldbsave = ldbsave.or.lddmctdh.or.lgausspot

! Finish information for spin blocks in DD
      if (ldd) then
         do sp = 1,nsmult
            stblk(sp) = idxmult(sp)*imultblk(sp)
         enddo
         if (lsocdegen) then
            do sp = 1,nsmult
               ddstblk(sp) = imultblk(sp)
               ddstoff(sp) = imultoff(sp)
            enddo
         else
            do sp = 1,nsmult
               ddstblk(sp) = stblk(sp)
               ddstoff(sp) = stoff(sp)
            enddo
         endif
      endif

! get no. of configurations if using CI vector
      if (ldd .and. lcivec) then
         do sp = 1,nsmult
            nciconf(sp) = nconf(sp,1)*nconf(sp,2)
         enddo
      endif

! form full path for gwptrj trajectories
      if (lgwptrj) then
         gwptrjdir = gwptrjdir0
         gwptrjlaenge = gwptrjlaenge0
         if (gwptrjdir(1:1) .ne. '/') call abspath(gwptrjdir,gwptrjlaenge)
      endif
    
      return

  999 continue
      routine='Operinfo11'
      message='Error Reading Oper file: '//rdtag(ilbl:jlbl)
      call errormsg

      end subroutine operinfo11

!#######################################################################
!
! OPERINFO
!
! reads information about operator needed to set up propagation
!  i.e. kzahl, hopsdim etc.
!
! This is for file versions older than version11.0, i.e. with fixed
! format 
!#######################################################################

      subroutine operinfo_pre11(lerr,chkdvr,chkgrd)

      implicit none

      integer(long)       :: f,k,s,h,m,ndof1,np,maxhpar1,nh,hl,nham,&
                             gf,i,n,ilnpot,ilfast,ilreal,idd,ilcap,&
                             ilcdvr1,iltdvr1,muld,idum,nmode1,j,&
                             idummy,iddtrans,iddwrdb,idddba,mxkoe
      integer(long)       :: ierr

      integer(long), intent(inout)                  :: chkgrd
      integer(long), intent(in)                     :: chkdvr
      integer(long), dimension(maxdim)              :: fdvr,idum1,mdvr,mmode,&
                                                       nspfdof1
      integer(long), dimension(maxrdf)              :: lifast,idum2
      integer(long), dimension(maxkoe)              :: idum3,ddkf,ddki
      integer(long), dimension(maxdim,maxkoe)       :: intaux 
      integer(long), dimension(maxdim,maxdim)       :: spfdof1
      character(len=c1)                             :: file
      logical(kind=4)                               :: rdfast,fastfound,ldum
      logical(kind=4), external                     :: int2log
      logical(kind=4), intent(out)                  :: lerr
      character(len=100)                            :: buffer
      character(len=c4), dimension(maxrdf)          :: pesopts1
      character(len=10)                             :: cerr

      integer(long), dimension(maxrdf)              :: mmodc

      integer(long), external :: integersize, realsize, logicalsize

      routine='Operinfo_pre11'

!-----------------------------------------------------------------------
! Allocate 
! arrays for natfast info. Now set up later and not read form oper file.
!-----------------------------------------------------------------------
      cerr = 'lfast'
      allocate(lfast(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'fastorder'
      allocate(fastorder(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'natpmode'
      allocate(natpmode(maxdim,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'mdimf'
      allocate(mdimf(maxdim,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'nterm'
      allocate(nterm(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ntermlast'
      allocate(ntermlast(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'fastprem'
      allocate(fastprem(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'fastnum'
      allocate(fastnum(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'fastdim'
      allocate(fastdim(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'zfp'
      allocate(zfp(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'zfn'
      allocate(zfn(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'zfd'
      allocate(zfd(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000

      i=0
      if (filever(ioper).ge.8.6006d0 .and. filever(ioper) .lt. 9.0000d0) &
          read(ioper,err=999) mxkoe

      file='Oper'
      call zerovxi(chkerr,nerr)
      call rddvrdef(ioper,chkdvr,ndof1,fdvr)
      call rdgrddef(ioper,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)

      call chksyserr(file,lerr)
      if (lerr) return
 
!-----------------------------------------------------------------------
! read title and write to log file
!-----------------------------------------------------------------------
      if (logisopen) write(ilog,'(/,a)') '**** Reading OPER file ****'
   10 continue
      i=1
      read(ioper,err=999) buffer
      if (buffer(1:10) .eq. 'end-title ' .or. &
          buffer(1:10) .eq. 'end-titel ') then
         continue
      else
         if (logisopen) write(ilog,'(a80)') trim(buffer)
         go to 10
      endif

!-----------------------------------------------------------------------
! read creation date and time
!-----------------------------------------------------------------------
      if (logisopen) write(ilog,'(a)') 'Created on:'
      read(ioper) buffer
      if (logisopen) write(ilog,'(a)') buffer(1:80)

      i=2
      read(ioper,err=999) (mass(f),f=1,ndof1)
      if (filever(ioper) .ge. 8.1008_dop) then
         i=3 
         read(ioper,err=999) npar
         i=4
         read(ioper,err=999) (apar(np),np=1,npar),(rpar(np),np=1,npar)
      endif
      if (filever(ioper) .ge. 8.1009_dop) then
         i=5
         read(ioper,err=999) nstate, npacket
      endif

      if (filever(ioper) .ge. 8.1013_dop) then
         i=6
         read(ioper,err=999) nopmode,nopdof
         i=7
         do m=nmode+1,nopmode
            read(ioper,err=999) nspfdof(m),(spfdof(n,m),n=1,nspfdof(m))
         enddo
         i=8
         read(ioper,err=999) (dofspf(f),f=ndof1+1,nopdof)
      else
         nopdof=ndof1+1
         nopmode=nmode
         if (lmult) nopmode=nopmode+1
      endif

      if (filever(ioper) .ge. 8.1012_dop) then
         i=9
         read(ioper,err=999) hamdim,ktot,fkoe,ftime
      else 
         i=10
         read(ioper,err=999) hamdim,ktot,fkoe
         ftime=0
      endif
      if (filever(ioper) .ge. 8.1013_dop) then
         i=11
         read(ioper,err=999) mkoe
      else
         mkoe=nmode+1
         if (lmult) mkoe=mkoe+1
      endif
      subdim(mkoe)=1

      do nham=1,hamdim
         i=12
         read(ioper,err=999) hamlab(nham)
         if (filever(ioper) .ge. 8.1014_dop) then
            i=13
            read(ioper,err=999) idum
            nodiag(nham)=int2log(idum)
         endif
         i=14
         read(ioper,err=999) zham(nham),khtot(nham),kzahl(nham), &
              nmulpot(nham)
         if (filever(ioper) .ge. 9.0015_dop) then
            read(ioper,err=999) idum
            ltdh(nham)=int2log(idum)
         else
            ltdh(nham) = .false.
            routine = 'rdoper'
            message =&
        'Reading old oper file: Assuming time-independent Hamiltonian'
            call warnmsg1
         endif

         do s=1,nstate
            i=15
            read(ioper,err=999) (khzahl(f,s,nham),f=1,ndof1)
            i=16
            read(ioper,err=999) (zhun(m,s,nham),m=1,nmode)
         enddo
      enddo

      if (filever(ioper) .ge. 8.1012_dop) then
         i=17
         read(ioper,err=999) (iqconst(f),f=1,ndof1), &
            (rqconst(f),f=1,ndof1)
      endif

      if(ktot+max(npacket,nstate) .gt. maxkoe) then
         write(message,'(a,i7)') 'Enlarge maxkoe to at least',&
                                  ktot+max(npacket,nstate)
         routine='operinfo (mctdh/rdoper.F line 146)'
         call errormsg
      endif

      i=18
      read(ioper,err=999) ((hamilton(f,k),f=1,nopdof),k=1,ktot)
      i=19
      read(ioper,err=999) ((intaux(m,k),m=1,nmode),k=1,ktot)
      do k = 1,ktot
         do m = 1,nmode
            diag(m,k) = int2log(intaux(m,k))
         enddo
      enddo
      i=20
      read(ioper,err=999) ((kinop(m,k),m=1,nmode),k=1,ktot)
      if (filever(ioper) .ge. 8.1012_dop) then
         i=21
         read(ioper,err=999) ((hsym(m,k),m=1,nmode),k=1,ktot)
         i=22
         read(ioper,err=999) ((hlsym(m,k),m=1,nmode),k=1,ktot)
      endif
      if (filever(ioper) .ge. 8.1009_dop) then
         i=23
         read(ioper,err=999) (kact(k),k=1,ktot)
         i=24
         read(ioper,err=999) (ktwin(k),k=1,ktot)
      endif
      i=25
      read(ioper,err=999) (ki(k),k=1,ktot)
      i=26
      read(ioper,err=999) (kf(k),k=1,ktot)

      l_mlpf=.false.
      if (filever(ioper) .ge. 8.5007d0) then
         i=27
         read(ioper,err=999) (hmlop(k),k=1,ktot)
         do k=1,ktot  !! Check if there is an mlpf
            if(hmlop(k).ne.0) then
               l_mlpf=.true.
               exit
            endif
         enddo
      endif
!
! ddki/ddkf replaced by ddsi/ddsf below after 9.0009
!
      if (filever(ioper) .ge. 9.0007_dop .and. &
                      filever(ioper) .lt. 9.0009_dop) then
         read(ioper,err=999) (ddki(k),k=1,ktot)
         read(ioper,err=999) (ddkf(k),k=1,ktot)
      endif
      if (filever(ioper) .ge. 9.0010_dop) then
         read(ioper,err=999) (ksi(h),h=1,ktot)
         read(ioper,err=999) (ksf(h),h=1,ktot)
         read(ioper,err=999) (fterm(h),h=1,ktot)
      else if (filever(ioper) .ge. 9.0007_dop .and. &
                      filever(ioper) .lt. 9.0009_dop) then
         do k=1,ktot
            ksi(k) = ddki(k)
            ksf(k) = ddkf(k)
            fterm(k)=1
         enddo
      else
         call zerovxi(ksi,ktot)
         call zerovxi(ksf,ktot)
      endif

      i=28
      read(ioper,err=999) htmdim
      if (htmdim .gt. maxhtm) then
         routine='operinfo (mctdh/rdoper.F line 180)'
         write(message,'(a,i5)') 'Increase MAXHTM. Minimum : ',htmdim
         call errormsg
      endif
      i=29
      read(ioper,err=999) (htmtype(h),h=1,htmdim)
      if (filever(ioper) .ge. 8.10012_dop) then
         i=30 
         read(ioper,err=999) (htmsym(h),h=1,htmdim)
      endif
      i=31
      read(ioper,err=999) (htmdof(h),h=1,htmdim)
      if (filever(ioper) .ge. 8.3013_dop) then
         i=32
         read(ioper, err=999) (htmadof(h),h=1,htmdim) ! NEWKLEG
         if (filever(ioper) .ge. 10.0000d0) then
            read(ioper, err=999) (htmndof(h),h=1,htmdim)
         else if (filever(ioper) .ge. 8.4006d0 .and. &
                filever(ioper) .le. 9.0000d0) then
            read(ioper, err=999) (htmndof(h),h=1,htmdim)
         endif
      endif
      i=33
      read(ioper,err=999) (htmmode(h),h=1,htmdim)
      i=34
      read(ioper,err=999) (htmmuld(h),h=1,htmdim)
      if (filever(ioper) .ge. 8.1008_dop) then
         i=35
         read(ioper,err=999) (htmform(h),h=1,htmdim)
      else
         do h=1,htmdim
            if (htmmode(h) .gt.0 ) htmform(h)=1
         enddo
      endif
      if (filever(ioper) .ge. 8.3014_dop) then
         i=36
         read(ioper,err=999) (htmshift(h),h=1,htmdim)
      else
         do h=1,htmdim
            htmshift(h)=0
         enddo
      endif
      if (filever(ioper) .ge. 9.0014_dop) then
         i=361
         read(ioper,err=999) (htmderiv(h),h=1,htmdim)
      else
         do h=1,htmdim
            htmderiv(h)=0
         enddo
      endif
      if (filever(ioper) .ge. 9.0009_dop) then
         i=37
         read(ioper,err=999) (ddsi(h),h=1,htmdim)
         read(ioper,err=999) (ddsf(h),h=1,htmdim)
      else if (filever(ioper) .ge. 9.0007_dop .and. &
                      filever(ioper) .lt. 9.0009_dop) then
         call zerovxi(ddsi,htmdim)
         call zerovxi(ddsf,htmdim)
         do k=1,ktot
            f = spfdof(1,meb)
            h = hamilton(f,k)
            ddsi(h) = ddki(k)
            ddsf(h) = ddkf(k)
         enddo
      else
         call zerovxi(ddsi,htmdim)
         call zerovxi(ddsf,htmdim)
      endif

      i=38
      read(ioper,err=999) minfac
      if (minfac .gt. maxfac) then
         routine='operinfo (mctdh/rdoper.F line 208)'
         write(message,'(a,i5)') 'Increase MAXFAC. Minimum : ',minfac
         call errormsg
      endif
      i=39
      do h=1,htmdim
         read(ioper,err=999) htmfac(h),(hterm(nh,h),nh=1,htmfac(h))
      enddo

      if (filever(ioper) .ge. 8.1003_dop) then
         i=40
         read(ioper,err=999) mulddim
         if (mulddim .gt. maxmuld) then
            routine='operinfo (mctdh/rdoper.F line 221)'
            write(message,'(a,i5)') 'Increase MAXMULD. Minimum : ', &
                 mulddim
            call errormsg
         endif
         i=41
         do muld=1,mulddim
            read(ioper,err=999) muldsize(muld)
            read(ioper,err=999) nmulddof(muld), &
                (mulddof(gf,muld),gf=1,nmulddof(muld))
            if (filever(ioper) .lt. 8.1012_dop) & 
                read(ioper,err=999) idum,(idum1(i),i=1,idum)
         enddo
      endif

      i=42
      read(ioper,err=999) hopdim
      read(ioper,err=999) (hoplab(hl),hl=1,hopdim)
      read(ioper,err=999) (hopilab(hl),hl=1,hopdim)
      read(ioper,err=999) (hopfile(hl),hl=1,hopdim)
      read(ioper,err=999) maxhpar1
      read(ioper,err=999) ((hopipar(np,hl),np=1,maxhpar1),hl=1,hopdim)
      read(ioper,err=999) ((hoprpar(np,hl),np=1,maxhpar1),hl=1,hopdim)

      if (filever(ioper) .ge. 8.2000_dop) then
         i=43
         read(ioper,err=999) npesopt, &
               ((zpesopts(np,n),np=1,maxhpar1),n=1,npesopt)
         i=44 
         if (filever(ioper) .lt. 9.0007_dop) then
! string length change at version 9.0007
            read(ioper,err=999) (pesopts1(n),n=1,npesopt)
            do n=1,npesopt
               pesopts(n)=pesopts1(n)
            enddo
         else
            read(ioper,err=999) (pesopts(n),n=1,npesopt)
         endif
      endif

      i=45
      read(ioper,err=999) (autocap(k),k=1,ktot)
      i=46
      read(ioper,err=999) hopsdim
      i=47
      read(ioper,err=999) (zhop(h),h=1,htmdim)

      ilnpot=0
      ilfast=0
      if (filever(ioper) .lt. 8.1012_dop) then
         i=48 
         read(ioper,err=999) ilcap,ilcdvr1,iltdvr1
      elseif (filever(ioper) .lt. 8.2005_dop) then
         i=49
         read(ioper,err=999) ilcap
      elseif (filever(ioper) .lt. 9.0000_dop) then
         i=50
         read(ioper,err=999) ilcap,ilnpot,ilfast
      elseif (filever(ioper) .lt. 11.0000_dop) then
         i=51 
         read(ioper,end=33,err=999) ilcap,ilnpot,ilfast,ilreal
         lreal = int2log(ilreal) ! lreal now redundant as
  33     continue                ! also read in rstinfo.
      else
         i=52 
         read(ioper,err=999) ilcap,ilnpot,ilfast
      endif
      lcap  = int2log(ilcap)
      lnpot = int2log(ilnpot)
      lfast(1)=int2log(ilfast)

      fastorder(:)=0
      if(lnpot) then
         if (filever(ioper) .ge. 8.3002_dop) then
             i=53
             if (filever(ioper) .ge. 8.3007_dop) then
                read(ioper,err=999) modc,nrdf
                read(ioper,err=999) (lifast(n),n=1,nrdf),(idum2(n),n=1,nrdf)
                rdfast=.false.
                do n=1,nrdf
                   lfast(n)=int2log(lifast(n))
                   if (lfast(n)) rdfast=.true.
                enddo
                if (rdfast) then
                   i=54
                   if (filever(ioper) .ge. 9.0015_dop) then
                      read(ioper,err=999) ((mdim(m,n),m=1,nmode),n=1,nrdf)
                   else if ((filever(ioper) .ge. 8.4005d0) .and.  &
                             (filever(ioper) .le. 9.0000_dop)) then
                      read(ioper,err=999) ((mdim(m,n),m=1,nmode),n=1,nrdf)
                   else
                      read(ioper,err=999) idum,idum,idum,idum
                   endif
                   read(ioper,err=999) (nterm(n),n=1,nrdf), &
                       (ntermlast(n),n=1,nrdf),(mmodc(n),n=1,nrdf), &
                       (fastorder(n),n=1,nrdf),(idum2(n),n=1,nrdf), &
                       ((mdimf(m,n),m=1,nmode),n=1,nrdf), &
                       ((natpmode(i,n),i=1,nmode-2),n=1,nrdf), &
                       (zfp(n),n=1,nrdf),(zfn(n),n=1,nrdf), &
                       (zfd(n),n=1,nrdf), &
                       (fastprem(j),j=1,maxkoe),(fastnum(j),j=1,maxkoe), &
                       (fastdim(j),j=1,maxkoe),(idum3(i),i=1,maxkoe), &
                       (idum3(j),j=1,maxkoe)
                endif
             else
                if (filever(ioper) .ge. 8.3004_dop) read(ioper,err=999) npmode
                i=55
                read(ioper,err=999) modc,idummy,idummy,idummy,idummy,idummy, &
                           (pdim(m),m=1,npmode),(vpdim(m),m=1,npmode), &
                           (npdim(m),m=1,npmode),(mdim(m,1),m=1,nmode), &
                           (vmdim(m,1),m=1,nmode),(nmdim(m,1),m=1,nmode)
                fastfound=.false.
                do n=1,maxrdf
                   if (lfast(n)) fastfound=.true.
                enddo
                if (filever(ioper) .ge. 8.3004_dop) then
                    i=56
                    read(ioper,err=999) idum,idum
                    if (fastfound) then
                       routine='operinfo (mctdh/rdoper.F line 329)'
                       message='For FAST try to '// &
                               'create a new operator file'
                       call errormsg
                    endif
                else
                   if (fastfound) then
                    routine='operinfo (mctdh/rdoper.F line 336)'
                    message='For FAST try to create a new operator file'
                    call errormsg
                   endif
                endif
             endif
         elseif (filever(ioper) .ge. 8.3001_dop) then
             i=61 
             read(ioper,err=999) modc,idummy,idummy,idummy, &
                        (pdim(f),f=1,ndof1),(vpdim(f),f=1,ndof), &
                        (npdim(f),f=1,ndof1)
         else
             i=62
             read(ioper,err=999) modc,idummy,idummy, &
                        (pdim(f),f=1,ndof1),(vpdim(f),f=1,ndof), &
                        (npdim(f),f=1,ndof1)
         endif
      endif

      if (filever(ioper) .ge. 8.4008d0) then
         read(ioper,err=999) iLMR, ((hamLMR(i,n),i=1,iLMR),n=1,3)
      endif
      if (filever(ioper) .ge. 8.5007d0 .and. &
                 filever(ioper) .lt. 9.0000d0 .and. l_mlpf) then
         do nham=1,hamdim
            call mlham_load(ioper,nham,tape,lrunml)
         enddo
      else if (filever(ioper) .ge. 8.5007d0 .and. &
                 filever(ioper) .lt. 8.6006d0) then
         do nham=1,hamdim
            read(ioper,err=999) lerr
         enddo
      endif

      if (filever(ioper) .gt. 9.0000_dop) then
         i=63
         read(ioper,err=999) idd,iddtrans,dddiab
         ldd = int2log(idd)
         lddtrans = int2log(iddtrans)
         read(ioper,err=999) ndofdd,ndofddpes,ddtransdim
      endif
      if (filever(ioper) .ge. 9.0012_dop) then
         read(ioper,err=999) natm
      else
         natm=ndofddpes/3
      endif

      if (filever(ioper) .ge. 9.0006_dop) then
         i=64
         read(ioper,err=999) nddstate
         read(ioper,err=999) ddextcmd
         read(ioper,err=999) dde0
         read(ioper,err=999) lfchk,fchkcmd
      endif

      if (filever(ioper) .ge. 10.1007_dop) then
         i=64
         read(ioper,err=999) ddmindb,iddwrdb,dddvmin
         lddwrdb=int2log(iddwrdb)
      else if (filever(ioper) .ge. 10.1005_dop) then
         i=64
         read(ioper,err=999) ddmindb,iddwrdb
         lddwrdb=int2log(iddwrdb)
      else if (filever(ioper) .ge. 9.0011_dop .and. &
             filever(ioper) .lt. 10.1005_dop) then
         i=64
         read(ioper,err=999) ddmindb,iddwrdb,idddba
         lddwrdb=int2log(iddwrdb)
      else if (filever(ioper) .ge. 9.0002_dop) then
         i=64
         read(ioper,err=999) ddmindb,ldum,lddwrdb
      endif

      if (filever(ioper) .ge. 9.0004_dop .and. &
          filever(ioper) .lt. 9.0010_dop) then
         i=65
         read(ioper,err=999) lsacas,idum
      else if (filever(ioper) .ge. 9.0010_dop) then
         i=65
         read(ioper,err=999) lsacas
      endif

      if (filever(ioper) .ge. 9.0009_dop) then
         i=66
         read(ioper,err=999) ddlaenge
         read(ioper,err=999) ddname
      else
         ddlaenge = 7
         ddname = "dd_data"
      endif

!-----------------------------------------------------------------------
! if transformed direct dynamics, read in and process transformation
! information
!-----------------------------------------------------------------------
      if (ldd .or. lddmctdh) call rdddtrans(ddtransf,ddtransb,ddxcoo0, &
         ndofdd,ndofddpes,ioper,xatnam,xmass,lddtrans)

!-----------------------------------------------------------------------
! if transformed direct dynamics, read in and process transformation
! information
!-----------------------------------------------------------------------
      if (ldd .and. (dddiab .eq. 1 .or. dddiab .eq. 2)) &
         call rddiab1(ioper,xcoin,xrefpoint,xdcp0,xgd0,ndofddpes)

!-----------------------------------------------------------------------
! runs checks
!-----------------------------------------------------------------------
      if (dentype.eq.1) then
         if (lcap) then
            routine='operinfo (rdoper.F)'
            message='CAPs not implemented for densities (type I).'
            call errormsg
         endif
      endif
      if (dentype.ne.0) then  
         if (nmulpot(1).gt.0) then
            routine='operinfo (rdoper.F)'
            message='MULPOT not implemented for densities.'
            call errormsg
         endif
      endif

      ldissop = .false.
      do k=1,kzahl(1)
         if (kact(k).gt.1) ldissop = .true.
      enddo

      if (dentype .eq. 0) then
         if (ldissop) then
            routine='operinfo (rdoper.F)'
            message='Dissop not implemented for wave functions.'
            call errormsg
         endif
      else
         if (ldissin) then
            if (.not. ldissop) then
               routine='operinfo (rdoper.F)'
               message='No dissipative operators found.'
               call warnmsg1
            endif
         else
            do k=1,kzahl(1)
               if (kact(k).gt.1) kact(k)=0
            enddo
         endif
      endif            

!-----------------------------------------------------------------------
! convert old redundant htmtypes to new usage
!-----------------------------------------------------------------------
      if (filever(ioper) .lt. 8.1012_dop) then
         do h=1,htmdim
            if (htmtype(h) .eq. 12) then
               htmtype(h)=1
            else if (htmtype(h) .eq. 13) then
               htmtype(h)=2
            else if (htmtype(h) .eq. 14) then
               htmtype(h)=3
            else if (htmtype(h) .eq. 15) then
               htmtype(h)=4
            else if (htmtype(h) .eq. 16) then
               htmtype(h)=2
            else if (htmtype(h) .eq. 20) then
               htmtype(h)=101
            else if (htmtype(h) .eq. 21) then
               htmtype(h)=8
            else if (htmtype(h) .eq. 22) then
               htmtype(h)=9
            else if (htmtype(h) .eq. 23) then
               htmtype(h)=10
            else if (htmtype(h) .eq. 24) then
               htmtype(h)=8
            endif
         enddo
      endif

!-----------------------------------------------------------------------
! make any corrections / additions to be compatible with verions >= 11.0
!-----------------------------------------------------------------------
      mulddof2 = mulddof
      nstatecap(:)=0

!-----------------------------------------------------------------------
! Deallocate natpot FAST data
!-----------------------------------------------------------------------
      call dealloc_setoperpsi

      return

  999 continue
      if (logisopen) write(ilog,'(a,i4,f15.9)') ' ERROR:', i, filever(ioper)
      write(6,'(a,i4,f15.9)')    ' ERROR:', i, filever(ioper)
      routine='Operinfo'
      message='Error reading oper file'
      call errormsg

 1000 write(ilog,'(a)') 'Error allocating memory in SETNATFAST :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      write(6,'(a)') 'Error allocating memory in SETNATFAST :',cerr
      write(6,'(a,i5)') 'Error Code :',ierr

      end subroutine operinfo_pre11

!#######################################################################
!
! RDOPER
!
! reads operator (hops)
!
! NB the string "operfile" must be set to the appropriate file name
!    before calling this routine.
!
! ddtransf, ddtransb: Potential may use different coordinates from
! ddxcoo0             dynamics (KE operator). Typically in direct 
!                     dynamics when potential is expressed in 
!                     Cartesians and dynamics may be in internals.
!                     These arrays define transformation
! 
!
!#######################################################################

      subroutine rdoper(hops,chkdvr,chkgrd)

      implicit none

      integer(long)                               :: ilbl,idbrd
      integer(long), intent(in)                   :: chkdvr
      integer(long), intent(inout)                :: chkgrd
      real(dop), dimension(hopsdim), intent(out)  :: hops

      integer(long) :: hl,n

      routine='Rdoper'

      open(ioper,file=operfile,form='unformatted',status='old', &
           err=999)
      read(ioper,err=999) filever(ioper)

      if (filever(ioper) .le. 11.0_dop) then
         call rdoper_pre11(hops,chkdvr,chkgrd)
      else
         call rdoper11(hops,chkdvr,chkgrd)
      endif

!-----------------------------------------------------------------------
! look to see if parameter used in operator has been changed
!-----------------------------------------------------------------------
      do hl=1,hopdim
         if (hopfile(hl) .eq. 301) then
            do n=1,npar
               if (hoplab(hl) .eq. apar(n) .and. & 
                     (abs(rpar(n) - hoprpar(1,hl)) .gt. tiny)) then
                  hoprpar(1,hl) = rpar(n)
               endif
            enddo
         endif
      enddo

!-----------------------------------------------------------------------
! if direct dynamics, open DB files and check for presence of ref.db 
!-----------------------------------------------------------------------
      if (ldb2gapot) then
         idbrd = 4
      else if (lrediab) then
         idbrd = 2
      else if (ldbsave) then
         idbrd = 1
      else
         idbrd = 0
      endif

! In geninwf, no need to read DB unless operate being used.
      if (runlevel .eq. 3 .and. inwfop(1) .eq. 0) idbrd = 0

      if (ldd.or.lddmctdh) call preparedb(idbrd,0)

! Fill koeff array
      call fillkoeff(hops)

      close(ioper)

      return
  999 continue
      ilbl=index(operfile,' ')-1
      message = 'Cannot open file: '//operfile(1:ilbl)
      call errormsg
      end subroutine rdoper

!#######################################################################
! 
! Fill Koeff array for the Hamiltonian
!
!#######################################################################

      subroutine fillkoeff(hops)

      implicit none

      integer(long) :: k,h
      real(dop), dimension(hopsdim), intent(in)  :: hops

!-----------------------------------------------------------------------
! fill koeff array
!-----------------------------------------------------------------------
      do k=1,ktot
         h=hamilton(fkoe,k)
         if (h .ne. fkoe) then
            if (htmtype(h) .eq. 3) then
               koeff(k)=dcmplx(0.0_dop,hops(zhop(h)))
            else if (htmtype(h) .eq. 4) then
               koeff(k)=dcmplx(hops(zhop(h)),hops(zhop(h)+1))
            else if (htmtype(h) .eq. 101) then
               koeff(k)=0.0_dop
            else
               koeff(k)=hops(zhop(h))
            endif
         else
            koeff(k)=1.0_dop
         endif

         if (dentype .ne. 0) then
            if (kact(k).gt.1)  then
               if (k.le.ktwin(k)) then
!      ------- for open systems, this forces the multiplication with koeff(k)
                  if (htmtype(h).eq.1) htmtype(h)=-1
                  koeff(k) = koeff(k)*(0.0_dop,1.0_dop)
               endif
            endif
         endif
      enddo

      return
      end subroutine fillkoeff

!#######################################################################

      subroutine rdoper11(hops,chkdvr,chkgrd)

      implicit none

      integer(long)                               :: h,ilbl,jlbl,f,m,zeig,&
                                                     g,zdim,muld,f1,f2,m1,sdm
      integer(long), intent(in)                   :: chkdvr
      integer(long), intent(inout)                :: chkgrd
      integer(long)                               :: nel
      real(dop)                                   :: rnel
      real(dop), dimension(hopsdim), intent(out)  :: hops
      logical(kind=4)                             :: lerr
      character(len=c2)                           :: rdtag


      routine='Rdoper'

      call operinfo11(lerr,chkdvr,chkgrd)
      if (lerr) then
         message='Error reading OPERATOR file'
         call errormsg
      endif
      
!-----------------------------------------------------------------------
! read hops
!-----------------------------------------------------------------------
      rewind(ioper)
      do
! read next record - error means it is not a string so ignore and go on to next
   10    continue
         read(ioper,err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl-10) .eq. 'hops') then
            read(rdtag(jlbl-9:jlbl),'(i10)') h

            gdim(fkoe)=1     
            if (zhop(h) .eq. 0) cycle
         
!-----------------------------------------------------------------------
! get info on associated dof/mode
!-----------------------------------------------------------------------
            call assocdof(h,f,m,muld,f1,m1,sdm)

            zeig=zhop(h)-1
            if (htmtype(h).eq.1 .or. htmtype(h).eq.2) then
               read(unit=ioper,err=999) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h).eq.3) then
               read(ioper,err=999) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h).eq.4) then
               read(ioper,err=999) (hops(zeig+g),g=1,sdm)
               zeig=zeig+sdm
               read(ioper,err=999) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h) .eq. 5) then
               read(ioper,err=999) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h) .eq. 6) then
               read(ioper,err=999) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h) .eq. 7) then
               read(ioper,err=999) (hops(zeig+g),g=1,sdm)
               zeig=zeig+sdm
               read(ioper,err=999) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h) .eq. 8) then
               read(ioper,err=999) (hops(zeig+g),g=1,sdm*sdm)
            else if (htmtype(h) .eq. 9) then
               read(ioper,err=999) (hops(zeig+g),g=1,sdm*sdm)
            else if (htmtype(h) .eq. 10) then
               read(ioper,err=999) (hops(zeig+g),g=1,sdm*sdm)
               zeig=zeig+sdm*sdm
               read(ioper,err=999) (hops(zeig+g),g=1,sdm*sdm)
            else if (htmtype(h) .eq. 11) then
               if (m1.ne.0) then
                  f1=htmadof(h)
                  sdm=vgdim(f1+1)*ngdim(f1+1)
               endif
               read(ioper,err=999) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h).eq.12 .or. htmtype(h).eq.13) then
               read(ioper,err=999) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h).eq.14) then
               read(ioper,err=999) (hops(zeig+g),g=1,sdm)
               zeig=zeig+sdm
               read(ioper,err=999) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h) .eq. 25) then
               read(ioper) rnel
               nel=nint(rnel)
               backspace ioper
               read(ioper) (hops(zeig+g),g=1,3*nel+1)
            else if (htmtype(h) .ge. 15 .and. htmtype(h) .le. 29) then
               f1=htmadof(h) ! NEWKLEG
               zdim=0
               if (basis(f1).eq.7 .or. basis(f1).eq.20) then   ! Kleg, KLeg/R
                   zdim=gdim(f1)*gdim(f1)*gdim(f1+1)
               elseif (basis(f1).eq.9 .or. basis(f1).eq.21) then
                   zdim=gdim(f1)*gdim(f1)*ipbaspar(3,f1+1)  ! PLeg, Pleg/R
               endif
               read(ioper,err=999) (hops(zeig+g),g=1,zdim)
            else if (htmtype(h) .eq. 30) then
               f1=htmadof(h)
               f2=f1+2
               sdm=(gdim(f1)**2+1)*gdim(f1+1) &
                  +(gdim(f2)**2+1)*gdim(f2+1)
               read(ioper,err=999) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h) .ge. 40 .and. htmtype(h) .le. 69) then
               f1=htmadof(h)
               zdim=gdim(f1)*gdim(f1)*ipbaspar(4,f1)*ipbaspar(8,f1)
               read(ioper,err=999) (hops(zeig+g),g=1,zdim)
            endif

         else if (rdtag(ilbl:jlbl) .eq. 'end hops') then
            exit

         endif

      enddo

  100 continue

      close(ioper)

      return
  999 continue
      routine='Rdoper11'
      message='Error Reading Oper file: '//rdtag(ilbl:jlbl)
      write(message(81:94),'(a,i5)') 'htmtype: ',htmtype(h)
      call errormsg

      end subroutine rdoper11

!#######################################################################

      subroutine rdoper_pre11(hops,chkdvr,chkgrd)

      implicit none

      integer(long)                               :: h,f,m,zeig,g,zdim,&
                                                     muld,f1,f2,m1,sdm
      integer(long), intent(in)                   :: chkdvr
      integer(long), intent(inout)                :: chkgrd
      real(dop), dimension(hopsdim), intent(out)  :: hops
      logical(kind=4)                             :: lerr

      routine='Rdoper_pre11'

      call operinfo_pre11(lerr,chkdvr,chkgrd)
      if (lerr) then
         message='Error reading OPERATOR file'
         call errormsg
      endif
      
!-----------------------------------------------------------------------
! read hops
!-----------------------------------------------------------------------
      gdim(fkoe)=1     
      do h=1,htmdim
         if (zhop(h) .gt. 0) then
         
!-----------------------------------------------------------------------
! get info on associated dof/mode
!-----------------------------------------------------------------------
            call assocdof(h,f,m,muld,f1,m1,sdm)

            zeig=zhop(h)-1
            if (htmtype(h).eq.1 .or. htmtype(h).eq.2) then
               read(unit=ioper) (hops(zeig+g),g=1,sdm)
!               read(unit=ioper,err=1000) (hops(zeig+g),g=1,sdm)
!1000           write(*,*)'error'
            else if (htmtype(h).eq.3) then
               read(ioper) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h).eq.4) then
               read(ioper) (hops(zeig+g),g=1,sdm)
               zeig=zeig+sdm
               read(ioper) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h) .eq. 5) then
               read(ioper) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h) .eq. 6) then
               read(ioper) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h) .eq. 7) then
               read(ioper) (hops(zeig+g),g=1,sdm)
               zeig=zeig+sdm
               read(ioper) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h) .eq. 8) then
               read(ioper) (hops(zeig+g),g=1,sdm*sdm)
            else if (htmtype(h) .eq. 9) then
               read(ioper) (hops(zeig+g),g=1,sdm*sdm)
            else if (htmtype(h) .eq. 10) then
               read(ioper) (hops(zeig+g),g=1,sdm*sdm)
               zeig=zeig+sdm*sdm
               read(ioper) (hops(zeig+g),g=1,sdm*sdm)
            else if (htmtype(h) .eq. 11) then
               if (m1.ne.0) then
                  f1=htmadof(h)
                  sdm=vgdim(f1+1)*ngdim(f1+1)
               endif
               read(ioper) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h).eq.12 .or. htmtype(h).eq.13) then
               read(ioper) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h).eq.14) then
               read(ioper) (hops(zeig+g),g=1,sdm)
               zeig=zeig+sdm
               read(ioper) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h) .ge. 15 .and. htmtype(h) .le. 29) then
               f1=htmadof(h) ! NEWKLEG
               zdim=0
               if (basis(f1).eq.7 .or. basis(f1).eq.20) then   ! Kleg, KLeg/R
                   zdim=gdim(f1)*gdim(f1)*gdim(f1+1)
               elseif (basis(f1).eq.9 .or. basis(f1).eq.21) then
                   zdim=gdim(f1)*gdim(f1)*ipbaspar(3,f1+1)  ! PLeg, Pleg/R
               endif
               read(ioper) (hops(zeig+g),g=1,zdim)
            else if (htmtype(h) .eq. 30) then
               f1=htmadof(h)
               f2=f1+2
               sdm=(gdim(f1)**2+1)*gdim(f1+1) &
                  +(gdim(f2)**2+1)*gdim(f2+1)
               read(ioper) (hops(zeig+g),g=1,sdm)
            else if (htmtype(h) .ge. 40 .and. htmtype(h) .le. 69) then
               f1=htmadof(h)
               zdim=gdim(f1)*gdim(f1)*ipbaspar(4,f1)*ipbaspar(8,f1)
               read(ioper) (hops(zeig+g),g=1,zdim)
            endif
         endif
      enddo

      return
      end subroutine rdoper_pre11

!#######################################################################

      subroutine rdddtrans(ddtransf,ddtransb,ddxcoo0, &
                 ndofdd,ndofddpes,ioper,xatnam,xmass,lddtrans)

      implicit none

      integer(long)                                    :: i,j,natm
      integer(long), intent(in)                        :: ioper,ndofdd,ndofddpes
      real(dop), dimension(maxdim), intent(out)        :: ddxcoo0
      real(dop), dimension(maxdim/3),intent(out)       :: xmass
      real(dop), dimension(maxdim,maxdim), intent(out) :: ddtransf,ddtransb
      character(len=c1), dimension(maxdim/3), intent(out)  :: xatnam
      logical(kind=4), intent(in)                      :: lddtrans

      natm = ndofddpes/3
      if (filever(ioper) .ge. 9.0010_dop) then
         read(ioper) (xatnam(i),i=1,natm)
         read(ioper) (xmass(i),i=1,natm)
      else
         routine='Rdddtrans (rdoper.F)'
         message='Update oper (pes) file to version 9.0010 or above'
         call errormsg
      endif
      if (lddtrans) then
         read(ioper) (ddxcoo0(i),i=1,ndofddpes)
         do i=1,ndofdd
            read(ioper) (ddtransf(j,i),j=1,ndofddpes)
         enddo
         do i=1,ndofddpes
            read(ioper) (ddtransb(j,i),j=1,ndofdd)
         enddo
      endif

      return
      end subroutine rdddtrans
      
!#######################################################################

      subroutine rddiab1(ioper,xcoin,xrefpoint,xdcp0,xgd0,ndofddpes)

      implicit none

      integer(long)                                  :: i
      integer(long), intent(in)                      :: ioper,ndofddpes
      real(dop), dimension(ndofddpes), intent(inout) :: xcoin,xrefpoint,xdcp0,xgd0

      if (filever(ioper) .lt. 9.0010_dop) then
         routine='Rdddtrans (rdoper.F)'
         message='Update oper (pes) file to version 9.0010 or above'
         call errormsg
      endif

      read(ioper) (xcoin(i),i=1,ndofddpes)
      read(ioper) (xrefpoint(i),i=1,ndofddpes)
      read(ioper) (xdcp0(i),i=1,ndofddpes)
      read(ioper) (xgd0(i),i=1,ndofddpes)

      return
      end subroutine rddiab1
      
!#######################################################################
!
! ASSOCDOF
!
! for hterm, find which is the associated dof/mode or muld and
! grid size
!
!#######################################################################

      subroutine assocdof(h,f,m,muld,f1,m1,sdm)

      implicit none

      integer(long), intent(in)  :: h
      integer(long), intent(out) :: f,m,muld,f1,m1,sdm
      integer(long)              :: fa,fn,n,n1

      f=htmdof(h)
      m=htmmode(h)
      fa=htmadof(h)
      fn=htmndof(h)
      muld=htmmuld(h)
      if (f .ne. 0) then
         if (f .eq. fkoe .or. f .eq. ftime) then
            f1=f
            m1=0
            sdm=1
         else
            f1=f
            m1=dofspf(f)
            sdm=gdim(f)
         endif
      else if (m .ne. 0) then
!-- Edit: check for complete mode operator (MS 03/2011)      
         if (fn .eq. 0) then
            f1=spfdof(1,m)
            m1=m
            sdm=subdim(m)
         else
!-- and sub-particle operator
            n1=0
            do n=1,nspfdof(m)
               if (spfdof(n,m) .eq. fa) n1 = n
            end do
            sdm = 1
            do n=n1,n1+fn-1
               f1 = spfdof(n,m)
               sdm = sdm*gdim(f1)
            end do
            f1=fa
            m1=m
         end if
!-- Old
!         f1=spfdof(1,m)
!         m1=m
!         sdm=subdim(m)
!-- End edit
      else if (muld .ne. 0) then
         f1=mulddof(1,muld)
         m1=dofspf(f1)
         sdm=muldsize(muld)
      endif

      return
      end subroutine assocdof

!------------------------------------------------------------------------
! Return true if DOF f is KLeg/PLeg.
!------------------------------------------------------------------------

      logical(kind=4) function iskleg(f,basis)

      implicit none

      integer(long), intent(in)               :: f
      integer(long), dimension(*), intent(in) :: basis

      if (f.gt.0) then
         iskleg = (basis(f).eq.7 .or. basis(f).eq.9 .or. &
                   basis(f).eq.20 .or. basis(f).eq.21)
      else
         iskleg = .false.
      endif

      return
      end function iskleg ! of iskleg
 
!------------------------------------------------------------------------
! Return true if sbasis for DOF f is KLeg/PLeg.
!------------------------------------------------------------------------

      logical(kind=4) function isklegsbas(f,sbasis)

      implicit none

      integer(long), intent(in)               :: f
      integer(long), dimension(*), intent(in) :: sbasis

      if (f.gt.0) then
         isklegsbas = sbasis(f).eq.5
      else
         isklegsbas = .false.
      endif

      return
      end function isklegsbas

!------------------------------------------------------------------------
! Get number of KLegs in mode m. Returned in nk.
!------------------------------------------------------------------------

      subroutine get_num_klegs(nk,m)

      implicit none

      integer(long)              :: f,n
      integer(long), intent(in)  :: m
      integer(long), intent(out) :: nk

      nk=0
      do n=1,nspfdof(m)
         f=spfdof(n,m)
         if (iskleg(f,basis)) nk=nk+1
      enddo

      return
      end subroutine get_num_klegs ! of get_num_klegs


!------------------------------------------------------------------------
! Get nth KLeg DOF in mode m. Returned in f.
!------------------------------------------------------------------------

      subroutine get_nth_kleg(f,n,m)

      implicit none

      integer(long)              :: f1,n1,nk
      integer(long), intent(in)  :: n,m
      integer(long), intent(out) :: f

      f=0
      nk=0
outer:do n1=1,nspfdof(m)
         f1=spfdof(n1,m)
         if (iskleg(f1,basis)) then
            nk=nk+1
            if (nk.eq.n) then
               f=f1
               exit outer
!               goto 10
            endif
         endif
      enddo outer

      return
      end subroutine get_nth_kleg ! of get_nth_kleg

!------------------------------------------------------------------------
! Get number of occurences of KLeg sbasis(=5) in mode m. Returned in nk
!------------------------------------------------------------------------

      subroutine num_kleg_sbas(nk,m)

      implicit none

      integer(long),intent(in)  :: m
      integer(long),intent(out) :: nk
      integer(long)             :: f,n

      nk=0
      do n=1,nspfdof(m)
         f=spfdof(n,m)
         if (isklegsbas(f,sbasis)) nk=nk+1
      enddo

      return
      end subroutine num_kleg_sbas

!------------------------------------------------------------------------
! Get nth KLeg sbasis in mode m. Returned in f.
!------------------------------------------------------------------------

      subroutine nth_kleg_sbas(f,n,m)

      implicit none

      integer(long),intent(in)  :: n,m
      integer(long),intent(out) :: f
      integer(long)             :: f1,n1,nk

      f=0
      nk=0
      do n1=1,nspfdof(m)
         f1=spfdof(n1,m)
         if (isklegsbas(f1,sbasis)) then
            nk=nk+1
            if (nk.eq.n) then
               f=f1
               goto 10
            endif
         endif
      enddo

10    return
      end subroutine nth_kleg_sbas

!------------------------------------------------------------------------
! Return true if DOF f is Wigner.
!------------------------------------------------------------------------

      logical(kind=4) function isWig(f,basis)

      implicit none

      integer(long), intent(in)               :: f
      integer(long), intent(in), dimension(*) :: basis

      if (f.gt.0) then
         isWig = basis(f).eq.19
      else
         isWig = .false.
      endif

      return
      end function isWig

!------------------------------------------------------------------------
! Get number of Wigners in mode m. Returned in nw.
!------------------------------------------------------------------------

      subroutine get_num_Wigs(nw,m)

      implicit none

      integer(long), intent(in)  :: m
      integer(long), intent(out) :: nw
      integer(long)              :: f,n

      nw=0
      do n=1,nspfdof(m)
         f=spfdof(n,m)
         if (isWig(f,basis)) nw=nw+1
      enddo

      return
      end subroutine get_num_Wigs

!------------------------------------------------------------------------
! Return true if sbasis for DOF f is Wigner.
!------------------------------------------------------------------------

      logical(kind=4) function isWigsbas(f,sbasis)

      implicit none

      integer(long), intent(in)               :: f
      integer(long), intent(in), dimension(*) :: sbasis

      if (f.gt.0) then
         isWigsbas = sbasis(f).eq.8
      else
         isWigsbas = .false.
      endif

      return
      end function isWigsbas

!------------------------------------------------------------------------
! Get number of Wigner sbasis(=8) in mode m. Returned in nk.
!------------------------------------------------------------------------

      subroutine num_Wig_sbas(nk,m)

      implicit none

      integer(long), intent(in)  :: m
      integer(long), intent(out) :: nk
      integer(long)              :: f,n

      nk=0
      do n=1,nspfdof(m)
         f=spfdof(n,m)
         if (isWigsbas(f,sbasis)) nk=nk+1
      enddo

      return
      end subroutine num_Wig_sbas


!------------------------------------------------------------------------
! Get nth Wigner sbasis in mode m. Returned in f.
!------------------------------------------------------------------------

      subroutine nth_Wig_sbas(f,n,m)

      implicit none

      integer(long), intent(in)  :: n,m
      integer(long), intent(out) :: f
      integer(long)              :: n1,nk,f1

      f=0
      nk=0
      do n1=1,nspfdof(m)
         f1=spfdof(n1,m)
         if (isWigsbas(f1,sbasis)) then
            nk=nk+1
            if (nk.eq.n) then
               f=f1
               goto 10
            endif
         endif
      enddo

10    return
      end subroutine nth_Wig_sbas

!#######################################################################
! The operator may need to be altered to account for details of the
! specified wavefunction - if a multipacket calculation for example.
!#######################################################################

      subroutine setoperpsi

! old versions of oper need nothing to be done
      if (filever(ioper) .lt. 11.0_dop) return

      call dealloc_setoperpsi

!-----------------------------------------------------------------------
! Duplicate operator for multiple packet runs
!-----------------------------------------------------------------------
      if (npacket .gt. 1) call setoperpack

!-----------------------------------------------------------------------
! Set flags for Natfast algorithm
!-----------------------------------------------------------------------
      if (nrdf .gt. 0) call setnatfast

      return
      end subroutine setoperpsi

!#######################################################################
! The operator may need to be altered to account for details of the
! specified wavefunction and for the propagation options 
! (e.g. GWP integrals)
!#######################################################################

      subroutine setoperprop

      integer(long)      :: kl,kh,m,m1,s,k,nham,h,k1,k2,hl,f,f1,kk
      integer(long)      :: ilbl,jlbl,lablen,iord
      integer(long)      :: ndel
      character(len=c2)  :: label
      logical(kind=4)    :: lgwpsad
      logical(kind=4), dimension(maxkoe) :: kignore

! old versions of oper need nothing to be done
      if (filever(ioper) .lt. 11.0_dop) return

      lopsetprop = .false.  ! flag if operator changed. Can be used to 
                           ! write new op.log file

!-----------------------------------------------------------------------
! Unit operators in correlated terms need to be evaluated for G-MCTDH
! Hence set diag to .false. and use diagg(m,k) to flag unit operators
!
! NB GWP unit operators in uncorrelated terms still flagged with diag
!-----------------------------------------------------------------------
      if (abs(psitype) .eq. 4) then
         diagg = diag
         do nham = 1,hamdim
            kl=zham(nham)
            kh=zham(nham)+kzahl(nham)-1
            do k=kl,kh
               do m=1,nmode
                  if (gwpm(m)) then
                     diag(m,k) = .false.
                     lopsetprop = .true.
                  endif
               enddo
            enddo
         enddo

      else if (psitype .eq. 0 .or. psitype .eq. 5) then
         do nham = 1,hamdim
            kl=zham(nham)
            kh=zham(nham)+kzahl(nham)-1
            do k=kl,kh
               if (klha(k) .gt. 0) then
                  do m=1,nmode
                     if (gwpm(m)) then
                        diagg(m,k) = diag(m,k)
                        diag(m,k) = .false.
                        lopsetprop = .true.
                     endif
                  enddo
               endif
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! LHA required if any GWPs propagated using Saddle point approxiamtion 
!-----------------------------------------------------------------------
      lgwpsad = .false.
      if (lgwp) then
         do m = 1,nmode
            if (gwpmodelm(m) .eq. 5) lgwpsad = .true.
         enddo
      endif

!-----------------------------------------------------------------------
! If LHA requested change any 101 (real analytic) to 105 (real LHA)
!-----------------------------------------------------------------------
      if (lforcelha .or. lgwpsad) then
         if (logisopen) write(ilog,'(a)') &
                   ' *** Setting GWP Integrals for LHA *** '
         if (logisopen) write(ilog,*)
         do h = 1,htmdim
            if (htmtype(h) .eq. 101) then
               htmtype(h) = 105
               lopsetprop = .true.
            endif
         enddo
      endif

!-----------------------------------------------------------------------
! For KLHA, if gwp integrals to lower order than operator setup, adjust
! Flag terms to ignore
! For saddle point GWPs, gwpintordm(m) = 0 and lhaorder = 2 by default.
!-----------------------------------------------------------------------
      if (lklha) then
         kignore(:) = .false.
    
         do m=1,nmode
            if (.not. gwpm(m)) cycle
            if (gwpcxm(m) .eq. 0) cycle
            if (gwpintordm(m) .gt. lhaorder) then
               routine = 'Setoperpsi (rdopermod.F90)'
               write(message,'(a,i0)') &
                     'GWP Intorder cannot be larger than LHA order. Mode :',m
               call errormsg
            endif
         enddo

         nham=1
         k1=zham(nham)
         k2=zham(nham)+kzahl(nham)-1
         do k=k1,k2
            h=hamilton(fkoe,k)
            hl = hterm(1,h)
            label = hoplab(hl)
            lablen = index(label,' ')-1

            ilbl = index(label,'%')
            read(label(ilbl+1:ilbl+1),*) iord

            if (iord .eq. 0) then
               cycle

            else if (iord .eq. 1) then
               ilbl = index(label,'_')+1
               read(label(ilbl:lablen),*) f
               m=dofspf(f)

               if (m .eq. 0) kignore(k) = .true.
               if (gwpintordm(m) .lt. 1) kignore(k) = .true.

            else if (iord .eq. 2) then
               ilbl = index(label,'_')+1
               jlbl = index(label(ilbl:lablen),'_') + ilbl-1
               read(label(ilbl:jlbl-1),*) f
               read(label(jlbl+1:lablen),*) f1
               m=dofspf(f)
               m1=dofspf(f1)

               if (m .eq. 0) kignore(k) = .true.
               if (gwpintordm(m) .lt. 2) kignore(k) = .true.
               if (m1 .eq. 0) kignore(k) = .true.
               if (gwpintordm(m1) .lt. 2) kignore(k) = .true.

            endif
         enddo

         ndel = 0
         kk=0
         do k=k1,k2
            if (kignore(k)) then
               call mvkend(kk+1)
               ktot = ktot-1
               ndel = ndel+1
            else
               kk=kk+1
            endif
         enddo
         if (ndel .gt. 0) then
            khtot(1) = khtot(1)-ndel
            kzahl(1) = kzahl(1)-ndel
            if (kk .ne. kzahl(1)) then
               routine = 'Setoperprop (Rdopermod.f90)'
               message = 'Inconsistent Kzahl(1) removing KLHA terms'
               call errormsg
            endif
            do h=1,hamdim
               do s=1,nstate
                  do m=1,nmode
                     if (zhun(m,s,h) .gt. 0) &
                            zhun(m,s,h) = zhun(m,s,h)-ndel
                  enddo
               enddo
               if (h .gt. 1) zham(h) = zham(h)-ndel
            enddo
            lopsetprop = .true.
         endif
      endif

      return
      end subroutine setoperprop

!#######################################################################
!
! DEALLOC_SETOPERPSI
! 
! Clear arrays set up in SETOPERPSI
!
!#######################################################################

      subroutine dealloc_setoperpsi

      if (allocated(lfast)) deallocate(lfast)
      if (allocated(fastprem)) deallocate(fastprem)
      if (allocated(fastnum)) deallocate(fastnum)
      if (allocated(fastdim)) deallocate(fastdim)
      if (allocated(fastorder)) deallocate(fastorder)
      if (allocated(zfp)) deallocate(zfp)
      if (allocated(zfn)) deallocate(zfn)
      if (allocated(zfd)) deallocate(zfd)
      if (allocated(natpmode)) deallocate(natpmode)
      if (allocated(mdimf)) deallocate(mdimf)
      if (allocated(nterm)) deallocate(nterm)
      if (allocated(ntermlast)) deallocate(ntermlast)

      return
      end subroutine dealloc_setoperpsi

!#######################################################################

      subroutine setoperpack

      integer(long)            :: n,m,p,s,t,zeig,zeig1,k,k1,kk
      integer(long), dimension(maxham)        :: tmpzham
      integer(long), dimension(maxham)        :: tmpkhtot,tmpkzahl,tmpnmulpot
      integer(long), dimension(maxdim,maxsta,maxham)   :: tmpkhzahl,tmpzhun
      integer(long), dimension(maxdim,maxkoe) :: tmphamilton
      integer(long), dimension(maxkoe)        :: tmpki,tmpkf,tmpksi,tmpksf
      integer(long), dimension(maxkoe)        :: tmpktwin,tmpkact,tmpfterm
      integer(long), dimension(maxkoe)        :: tmpnstatecap,tmpautocap
      integer(long), dimension(maxdim,maxkoe) :: tmpkinop
      integer(long), dimension(maxdim,maxkoe) :: tmphsym,tmphlsym
      complex(dop), dimension(maxkoe)         :: tmpkoeff
      logical(kind=4), dimension(maxdim,maxkoe) :: tmpdiag

!
! Calculate pointers for operators after expansion
!
      zeig = 1
      do n=1,hamdim
! total no.
         tmpkhtot(n) = khtot(n)*npacket
! total no. of correlated terms
         tmpkzahl(n) = kzahl(n)*npacket
! total no. of muld terms
         tmpnmulpot(n) = nmulpot(n)*npacket
! no. of uncorrelated terms 
         do p=1,npacket
            do s=1,nddstate
               t=s+(p-1)*nddstate
               do m=1,nmode
                  tmpkhzahl(m,t,n) = khzahl(m,s,n)
               enddo
            enddo
         enddo

! pointer to operator
         tmpzham(n) = zeig
         zeig = zeig+tmpkhtot(n)

! pointer to uncorrelated terms
         zeig1=tmpzham(n)+tmpkzahl(n)+tmpnmulpot(n)
         do m=1,nmode
            do s=1,nddstate
               do p=1,npacket
                  t=s+(p-1)*nddstate
                  tmpzhun(m,t,n) = zeig1
                  zeig1=zeig1+khzahl(m,s,n)
               enddo
            enddo
         enddo
      enddo
!
! copy correlated terms to all packets
! k is term in old operator, k1 in new. kk is a counter for each term in old.
!
      do n=1,hamdim
         kk=0
         do k=zham(n), zham(n)+kzahl(n)-1
            do p=1,npacket
               k1=tmpzham(n) + kk + kzahl(n)*(p-1)

               tmphamilton(:,k1) = hamilton(:,k)
               tmpki(k1) = ki(k)+(p-1)*nddstate
               tmpkf(k1) = kf(k)+(p-1)*nddstate
               tmpksi(k1) = ksi(k)
               tmpksf(k1) = ksf(k)
               tmpktwin(k1) = ktwin(k)
               tmpkact(k1) = kact(k)
               tmpfterm(k1) = fterm(k)
               tmpdiag(:,k1) = diag(:,k)
               tmpnstatecap(k1) = nstatecap(k)
               tmpautocap(k1) = autocap(k)
               tmpkinop(:,k1) = kinop(:,k)
               tmphsym(:,k1) = hsym(:,k)
               tmphlsym(:,k1) = hlsym(:,k)
               tmpkoeff(k1) = koeff(k)
            enddo
            kk = kk+1
         enddo
!
! copy multi-dimensional terms
!
         kk=0
         do k=zham(n)+kzahl(n), zham(n)+kzahl(n)+nmulpot(n)-1
            do p=1,npacket
               k1=tmpzham(n) + tmpkzahl(n) + kk + nmulpot(n)*(p-1)

               tmphamilton(:,k1) = hamilton(:,k)
               tmpki(k1) = ki(k)+(p-1)*nddstate
               tmpkf(k1) = kf(k)+(p-1)*nddstate
               tmpksi(k1) = ksi(k)
               tmpksf(k1) = ksf(k)
               tmpktwin(k1) = ktwin(k)
               tmpkact(k1) = kact(k)
               tmpfterm(k1) = fterm(k)
               tmpdiag(:,k1) = diag(:,k)
               tmpnstatecap(k1) = nstatecap(k)
               tmpautocap(k1) = autocap(k)
               tmpkinop(:,k1) = kinop(:,k)
               tmphsym(:,k1) = hsym(:,k)
               tmphlsym(:,k1) = hlsym(:,k)
               tmpkoeff(k1) = koeff(k)
            enddo
            kk = kk+1
         enddo
!
! copy uncorrelated terms
!
         zeig=tmpzham(n)+tmpkzahl(n)+tmpnmulpot(n)
         do m=1,nmode
            do s=1,nddstate
               kk=0
               do k=zhun(m,s,n), zhun(m,s,n)+khzahl(m,s,n)-1
                  do p=1,npacket
                     t=s+(p-1)*nddstate
                     k1=tmpzhun(m,t,n)+kk
                     tmphamilton(:,k1) = hamilton(:,k)
                     tmpki(k1) = ki(k)+(p-1)*nddstate
                     tmpkf(k1) = kf(k)+(p-1)*nddstate
                     tmpksi(k1) = ksi(k)
                     tmpksf(k1) = ksf(k)
                     tmpktwin(k1) = ktwin(k)
                     tmpkact(k1) = kact(k)
                     tmpfterm(k1) = fterm(k)
                     tmpdiag(:,k1) = diag(:,k)
                     tmpnstatecap(k1) = nstatecap(k)
                     tmpautocap(k1) = autocap(k)
                     tmpkinop(:,k1) = kinop(:,k)
                     tmphsym(:,k1) = hsym(:,k)
                     tmphlsym(:,k1) = hlsym(:,k)
                     tmpkoeff(k1) = koeff(k)
                  enddo
                  kk=kk+1
               enddo
            enddo
         enddo
      enddo

! copy back sizes
      do n=1,hamdim
         khtot(n) = tmpkhtot(n)
         kzahl(n) = tmpkzahl(n)
         nmulpot(n) = tmpnmulpot(n)
         do s=1,nstate
            do m=1,nmode
               khzahl(m,s,n) = tmpkhzahl(m,s,n)
            enddo
         enddo

         zham(n) = tmpzham(n) 

         do m=1,nmode
            do s=1,nstate
               zhun(m,s,n) = tmpzhun(m,s,n) 
            enddo
         enddo
      enddo

!total no. of terms
      n=hamdim
      ktot = zham(n) + khtot(n)-1

! copy back operators
      do k=1,ktot
         hamilton(:,k) = tmphamilton(:,k)
         ki(k) = tmpki(k)
         kf(k) = tmpkf(k)
         ksi(k) = tmpksi(k)
         ksf(k) = tmpksf(k)
         ktwin(k) = tmpktwin(k)
         kact(k) = tmpkact(k)
         fterm(k) = tmpfterm(k)
         diag(:,k) = tmpdiag(:,k)
         nstatecap(k) = tmpnstatecap(k)
         autocap(k) = tmpautocap(k)
         kinop(:,k) = tmpkinop(:,k)
         hsym(:,k) = tmphsym(:,k)
         hlsym(:,k) = tmphlsym(:,k)
         koeff(k) = tmpkoeff(k)
      enddo

      return
      end subroutine setoperpack

!#######################################################################
! SETNATFAST
!
! Search through operators which are linked to files to read. If 
! natpot found, and fast algorithm requested, set order and pointers.
!
!#######################################################################

      subroutine setnatfast

      integer(long)                      :: fddim,fndim,fpdim,i,m,s,n
      integer(long)                      :: i1,ierr,l1,l2,np
      integer(long), dimension(maxdim)   :: mnum
      real(dop), dimension(maxdim)       :: fac
      integer(long), external            :: slen
      integer(long)                      :: ilbl,jlbl
      character(len=c5)                  :: buffer
      character(len=10)                  :: cerr

      integer(long), external :: integersize, realsize, logicalsize

      cerr = 'lfast'
      allocate(lfast(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000

      lfast(:)=.false.

      if (nmode.lt.3) then
         routine='setnatfast'
         message='NATFAST algorithm is only possible for nmode >= 3. Ignoring.'
         call warnmsg1
         return
      endif
      if (nstate .gt. 1) then
         routine='setnatfast'
         message='FAST algorithm is not allowed for multi-set. Ignoring.'
         call warnmsg1
         return
      endif

      cerr = 'fastorder'
      allocate(fastorder(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'natpmode'
      allocate(natpmode(maxdim,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'mdimf'
      allocate(mdimf(maxdim,maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'nterm'
      allocate(nterm(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ntermlast'
      allocate(ntermlast(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'fastprem'
      allocate(fastprem(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'fastnum'
      allocate(fastnum(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'fastdim'
      allocate(fastdim(maxkoe),stat=ierr)
      allocmemory=allocmemory + maxdim*integersize()
      if (ierr > 0) go to 1000
      cerr = 'zfp'
      allocate(zfp(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'zfn'
      allocate(zfn(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'zfd'
      allocate(zfd(maxrdf),stat=ierr)
      if (ierr > 0) go to 1000

      fastorder(:) = 0
      fastprem(:) = 0
      fastnum(:) = 0
      fastdim(:) = 0
      zfp(:) = 0
      zfn(:) = 0
      zfd(:) = 0
      natpmode(:,:) = 0

!-----------------------------------------------------------------------
! Loop over any defined operators.
! If a natpot, see if fast algorithm has been requested
!-----------------------------------------------------------------------
      np=0
      do n=1,ndef
         l1=index(deflabop(n),'{')
         l2=index(deflabop(n),'}')
         if (l1.ne.0) then
            if (deflabop(n)(1:l1-1).eq.'natpot') then
               read(deflabop(n)(l1+1:l2-1),*,iostat=ierr) np
               if (lfastall) then
                  lfast(np)=.true.
                  if (fastilabel(1).ne.0) then
                     fastorder(np)=fastilabel(1)
                  else
                     fastorder(np)=min(4,nmode-2)
                  endif
               else
                  i1=0
                  do i=1,fastn
                     if (fastlabel(i).eq.deflabel(n)) i1=i
                  enddo
                  if (i1.ne.0) then
                     lfast(np)=.true.
                     if (fastilabel(i1).ne.0) then
                        fastorder(np)=fastilabel(i1)
                     else
                        fastorder(np)=min(4,nmode-2)
                     endif
                  endif
               endif
            endif
         endif
      enddo

! If Fast not used, deallocate memory (except lfast) and return
      if (.not. any(lfast)) then
         if (allocated(fastprem)) deallocate(fastprem)
         if (allocated(fastnum)) deallocate(fastnum)
         if (allocated(fastdim)) deallocate(fastdim)
         if (allocated(fastorder)) deallocate(fastorder)
         if (allocated(zfp)) deallocate(zfp)
         if (allocated(zfn)) deallocate(zfn)
         if (allocated(zfd)) deallocate(zfd)
         if (allocated(natpmode)) deallocate(natpmode)
         if (allocated(nterm)) deallocate(nterm)
         if (allocated(ntermlast)) deallocate(ntermlast)

         return
      endif

! check or define fastorder
      do n=1,nrdf
         if (lfast(n)) then
            if (fastorder(n) .eq. 0) fastorder(n)=nmode-2

            if (fastorder(n) .gt. 4) then
               fastorder(n)=min(4,nmode-2)
               write(6,*) 'The requested order for FAST is too big.'
               write(6,*) 'The Program reduced it to ',min(4,nmode-2)
            endif
         endif
      enddo
         
      write(ilog,*)

!----  Parameters for FAST ----------------------
      zfp(:)=1
      zfn(:)=1
      zfd(:)=1
      do n=1,nrdf
         if ((n+1).gt.maxrdf) cycle

         fpdim=0
         fndim=0
         fddim=0

!------ set natpmode(i,n)
         if (lfast(n)) then
             s=1
             do m=1,nmode
                if (m.ne.mmodc(n)) then
                   fac(m)=dim(m,s)**2.0_dop/mdim(m,n)
                else
                   fac(m)=0.0_dop
                endif
                mnum(m)=m
             enddo
             call sortdownr(fac,mnum,nmode)
             do i=1,fastorder(n)
                natpmode(i,n)=mnum(i)
             enddo
         endif

!------- set nterm(n) and ntermlast(n)
         call findnat(n)

!-------- set parameters for FAST and write information to log file
         if (lfast(n)) call fastdefine(fastprem(zfp(n)),fastnum(zfn(n)), &
                                       fastdim(zfd(n)),fpdim,fndim, &
                                       fddim,n)

         zfp(n+1)=zfp(n)+fpdim
         zfn(n+1)=zfn(n)+fndim
         zfd(n+1)=zfd(n)+fddim
         if (max(zfp(n+1),zfn(n+1),zfd(n+1)) .gt. maxkoe) then
            routine='Setnatfast'
            message='Increase dimensions of fastprem,fastnum,'// &
                     'fastdim arrays in operdef.inc'
            call errormsg
         endif
      enddo

!-----------------------------------------------------------------------
!     Write FAST data
!-----------------------------------------------------------------------
       write(ilog,'(/a)') '"FAST" algorithm for H(npot)*A'// &
           ' operation for System Hamiltonian is used:'
       write(ilog,'(a)') ' natpot no.  |  begin   |   end   |  '// &
           ' order   |  pre-multiplied mode(s)'
       do n=1,nrdf
          if (lfast(n)) then
             buffer=' '
             ilbl=1
             jlbl=ilbl+2
             write(buffer(ilbl:jlbl),'(i3)') natpmode(1,n)
             do i=2,fastorder(n)
                ilbl=ilbl+3
                jlbl=ilbl+2
                buffer(ilbl:ilbl)=','
                write(buffer(ilbl+1:jlbl),'(i2)') natpmode(i,n)
             enddo
             write(ilog,'(4x,i2,9x,i4,8x,i4,8x,i2,13x,a)') &
                   n,nterm(n),ntermlast(n),fastorder(n),buffer(1:15)
          endif
       enddo


      return
 1000 write(ilog,'(a)') 'Error allocating memory in SETNATFAST :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      write(6,'(a)') 'Error allocating memory in SETNATFAST :',cerr
      write(6,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine setnatfast

!###########################################################################
!  Find the beginning and end of the n-th natpot hamiltonian.
!###########################################################################

      subroutine findnat(n)

      implicit none

      integer(long)             :: m,n1,f,h,nnat,k,k1,k2,nn,imlop
      integer(long), intent(in) :: n
      logical(kind=4)           :: foundfirst

!.....k1 -> k2 is the system Hamiltonian
      k1=zham(1)
!     k2=zham(1)+kzahl(1)-1
      k2=zham(1)+khtot(1)-1 ! NEWKLEG: search also in uncorrelated terms

      foundfirst=.false.
outer:do k=k1,k2
         nn=0
         do m=1,nmode
            do n1=1,nspfdof(m)
               f=spfdof(n1,m)
               h=hamilton(f,k)
               if(h.gt.0) call findnath(h,nnat,imlop)   ! LMR
               if (nnat.ne.0) nn=nnat
            enddo
         enddo
         if (.not.foundfirst) then
             if (nn .eq. n) then
                 nterm(n)=k
                 foundfirst=.true.
             endif
         elseif (k .ne. k2) then
            if (nn .ne. n) then
                 ntermlast(n)=k-1
                 exit outer
             endif
         else
             ntermlast(n)=k2
             exit outer
         endif
      enddo outer

      return
      end subroutine findnat

!###########################################################################
!  Define the parameters need in FAST algorithm for the n-th natpot operator.
!###########################################################################

      subroutine fastdefine(fastprem1,fastnum1,fastdim1,fpdim,fndim, &
                            fddim,n)

      implicit none

      integer(long)       :: i,j,jj,ji,m,f,k,k1,k2,k3,p,l,l2,natdim,mult,ndim,&
                             h,h3,s1,z1,d2,d1

      integer(long), dimension(4,maxkoe)       :: fo
      integer(long), intent(in)                :: n
      integer(long), intent(out)               :: fpdim,fndim,fddim
      integer(long), dimension(*), intent(out) :: fastprem1,fastnum1,fastdim1
      logical(kind=4)                          :: lsame
      logical(kind=4), dimension(maxkoe)       :: kfind


      k1=nterm(n)
      k2=ntermlast(n)
      call zerovxi(fo,4*maxkoe)

!... check if natpot comes multiple times in hamiltonian
      natdim=mdim(mmodc(n),n)
      ndim=ntermlast(n)-nterm(n)+1 + ncutted(n)

      mult=ndim/natdim
      s1=0
      d1=1
      d2=natdim*mult

! --- Define the parameters for FAST

!.... calculate fastprem(j,n) (j=j1,j2,...)
      do i=1,fastorder(n)
         call zerovxl(kfind,k2)
         m=natpmode(i,n)
         f=spfdof(1,m)
         ji=0
         do k=k1,k2,npacket
            if (.not.kfind(k)) then
               h=hamilton(f,k)
               ji=ji+1
               fastprem1(ji+s1)=k
               fo(i,k)=ji
               do k3=k+npacket,k2,npacket
                  h3=hamilton(f,k3)
                  if (h3.eq.h) then
                     kfind(k3)=.true.
                     fo(i,k3)=ji
                  endif
               enddo
            endif
         enddo
!.........define mdimf(m,n)
         mdimf(m,n)=ji
         d1=d1*mdimf(m,n)
         d2=d2/mdimf(m,n)
         s1=s1+mdimf(m,n)
      enddo
      fpdim=s1
      fndim=d1*d2
      fddim=d1

!...... calculate  fastnum(p,j,n) and fastdim(j,n)
      call zerovxl(kfind,ntermlast(n)-nterm(n)+1)
      do k=k1,k2,npacket
         if (.not.kfind(k)) then
            p=1
            j=fo(1,k)
            do l=2,fastorder(n)
               jj=1
               do l2=1,l-1
                  jj=jj*mdimf(natpmode(l2,n),n)
               enddo
               j=j+jj*(fo(l,k)-1)
            enddo
            z1=p+(j-1)*d2
            fastnum1(z1)=k
            do k3=k+npacket,k2,npacket
               lsame=.true.
               do l=1,fastorder(n)
                  lsame = lsame .and. &
                     fo(l,k3).eq.fo(l,k)
               enddo
               if (lsame) then
                   kfind(k3)=.true.
                   p=p+1
                   z1=p+(j-1)*d2
                   fastnum1(z1)=k3
               endif
            enddo
            fastdim1(j)=p
         endif
      enddo

      return
      end subroutine fastdefine

!#######################################################################
! Sort a real array rfunc in decreasing order
!#######################################################################

      subroutine sortdownr(rfunc,indx,dim)

      implicit none
      integer(long)                                :: i,j,m
      integer(long), intent(in)                    :: dim
      integer(long), dimension(dim), intent(inout) :: indx
      real(dop)                                    :: rfc
      real(dop), dimension(dim), intent(inout)     :: rfunc

      do i=1,dim-1
         do j=i+1,dim
            if(rfunc(i) .lt. rfunc(j)) then
              rfc=rfunc(j)
              rfunc(j)=rfunc(i)
              rfunc(i)=rfc
              m=indx(j)
              indx(j)=indx(i)
              indx(i)=m
            endif
         enddo
      enddo

      return
      end subroutine sortdownr

!----------------------------------------------------------------------
!  REINITEXT1D
! This routine calls integer function li_browse. This is necessary
! in a readoper run, when external1d is used in Labels-Section.
!----------------------------------------------------------------------

      subroutine reinitext1d

      implicit none

      integer(long)           :: hl,k,ilbl,jlbl,np

      routine='reiniext1d'

      do hl=1,hopdim
         if(hoplab(hl)(1:10) .eq. 'external1d') then
            ilbl=index(hoplab(hl),'{')+1
            if (ilbl .gt. 1) then
               jlbl = index(hoplab(hl),'}')-1
            else
               jlbl=ilbl
            endif
            read(hoplab(hl)(ilbl:jlbl),*) np
            ilbl = 1
            jlbl = zpesopts(1,np)
            k = li_browse(pesopts(np)(ilbl:jlbl))
         endif
      enddo

      return
      end subroutine reinitext1d

      end module rdopermod

