!#######################################################################

      module symlib

use decimal,         only: long, dop
use logdat
use xvlib,           only: xvixdzo
use schmidtorthomod, only: schmidtortho2, schmidtortho3
use op1lib,          only: normvxz, cpvxz, initvxi
use timing


      implicit none
      private
      public :: sym1d,asym1d,sym2d,asym2d,sym3d,symcoeff,asymcoeff,nsym2d,&
                nsym2d_fixa,parity
      contains

!-------------------------------------------------------------------------------
!
      subroutine sym1d(vgd,ngd,gdim1,dim1,psi)

      implicit none

      integer(long)                                              :: i,j,k,e,ierr
      integer(long), intent(in)                                  :: vgd,ngd,gdim1,dim1
      complex(dop), dimension(vgd,gdim1,ngd,dim1), intent(inout) :: psi
      real(dop)                                                  :: mnorm

      do e = 1, dim1
         do i = 1, ngd
            do j = 1, (gdim1+1)/2
               do k = 1, vgd
                  psi(k,j,i,e) = 0.5_dop*&
                            (psi(k,j,i,e)+psi(k,gdim1+1-j,i,e))
                  psi(k,gdim1+1-j,i,e) = psi(k,j,i,e)
               enddo
            enddo
         enddo
      enddo

      call schmidtortho2(psi,vgd*gdim1*ngd,dim1,ierr,mnorm)
      write(2,'(1x,a,f17.12)') 'sym1d: minimal norm =', mnorm

      if( ierr .gt. 0 ) then
         write(6,'(a)') ' sym1d: Cannot Schmidt-orthogonalize spf''s.'
         write(2,'(a)') ' sym1d: Cannot Schmidt-orthogonalize spf''s.'
         call stopnow
      end if

      return
      end subroutine sym1d
!-------------------------------------------------------------------------------
!
      subroutine asym1d(vgd,ngd,gdim1,dim1,psi)

      implicit none

      integer(long)                                              :: i,j,k,e,ierr
      integer(long), intent(in)                                  :: vgd,ngd,gdim1,dim1
      complex(dop), dimension(vgd,gdim1,ngd,dim1), intent(inout) :: psi
      real(dop)                                                  :: mnorm


      do e = 1, dim1
         do i = 1, ngd
            do j = 1, (gdim1+1)/2
               do k = 1, vgd
                  psi(k,j,i,e) = 0.5_dop*&
                            (psi(k,j,i,e)-psi(k,gdim1+1-j,i,e))
                  psi(k,gdim1+1-j,i,e) = -psi(k,j,i,e)
               enddo
            enddo
         enddo
      enddo

      call schmidtortho2(psi,vgd*gdim1*ngd,dim1,ierr,mnorm)
      write(2,'(1x,a,f17.12)') 'asym1d: minimal norm =', mnorm

      if( ierr .gt. 0 ) then
         write(6,'(a)') ' asym1d: Cannot Schmidt-orthogonalize spf''s.'
         write(2,'(a)') ' asym1d: Cannot Schmidt-orthogonalize spf''s.'
         call stopnow
      end if

      return
      end subroutine asym1d
!-------------------------------------------------------------------------------
!
      subroutine parity(vgd,ngd,gdim1,dim1,psi)

      implicit none

      integer(long)                                              :: i,j,k,e,ierr
      integer(long), intent(in)                                  :: vgd,ngd,gdim1,dim1
      complex(dop), dimension(vgd,gdim1,ngd,dim1), intent(inout) :: psi
      complex(dop), dimension(gdim1,2)                           :: tmppsi
      real(dop)                                                  :: mnorm,norm1,norm2

      do e = 1, dim1
         do i = 1, ngd
            do j = 1, (gdim1+1)/2
               do k = 1, vgd
                  tmppsi(j,1) = 0.5_dop*(psi(k,j,i,e)+psi(k,gdim1+1-j,i,e))
                  tmppsi(gdim1+1-j,1) = tmppsi(j,1)
                  tmppsi(j,2) = 0.5_dop*(psi(k,j,i,e)-psi(k,gdim1+1-j,i,e))
                  tmppsi(gdim1+1-j,2) = -tmppsi(j,2)
               enddo
            enddo
         enddo
         call normvxz(tmppsi(1,1),norm1,gdim1)
         call normvxz(tmppsi(1,2),norm2,gdim1)
!........The larger norm determines, whether the SPF is to be
!........symmetrized or anti-symmetrized.
         if(norm1.gt.norm2) then
            do i = 1, ngd
               do j = 1, gdim1
                  do k = 1, vgd
                     psi(k,j,i,e) = tmppsi(j,1)
                  enddo
               enddo
            enddo
         else
            do i = 1, ngd
               do j = 1, gdim1
                  do k = 1, vgd
                     psi(k,j,i,e) = tmppsi(j,2)
                  enddo
               enddo
            enddo
         endif
      enddo               ! e=1,dim1

      call schmidtortho2(psi,vgd*gdim1*ngd,dim1,ierr,mnorm)
      write(2,'(1x,a,f17.12)') 'parity: minimal norm =', mnorm

      if( ierr .gt. 0 ) then
         write(6,'(a)') ' parity: Cannot Schmidt-orthogonalize spf''s.'
         write(2,'(a)') ' parity: Cannot Schmidt-orthogonalize spf''s.'
         call stopnow 
      end if

      return
      end subroutine parity
!-------------------------------------------------------------------------------
!
      subroutine sym2d(gdim1,dim1,psi)

      implicit none

      integer(long)                                            :: i,j,e,ierr
      integer(long), intent(in)                                :: gdim1,dim1
      complex(dop), dimension(gdim1,gdim1,dim1), intent(inout) :: psi
      real(dop)                                                :: mnorm

      do e = 1, dim1
         do j = 1, gdim1
            do i = j, gdim1
               psi(i,j,e) = 0.5_dop*(psi(i,j,e)+psi(j,i,e))
               psi(j,i,e) = psi(i,j,e)
            enddo
         enddo
      enddo

      call schmidtortho2(psi,gdim1**2,dim1,ierr,mnorm)
      write(2,'(1x,a,f17.12)') 'sym2d: minimal norm =', mnorm
    
      if( ierr .gt. 0 ) then
         write(6,'(2a,3i5)') ' sym2d: Cannot Schmidt-orthogonalize',&
                             ' spf''s.', gdim1, dim1, ierr
         write(2,'(2a,3i5)') ' sym2d: Cannot Schmidt-orthogonalize',&
                             ' spf''s.', gdim1, dim1, ierr
         call stopnow
      end if

      return
      end subroutine sym2d
!-------------------------------------------------------------------------------
!
      subroutine asym2d(gdim1,dim1,psi)

      implicit none

      integer(long)                                            :: i,j,e,ierr
      integer(long), intent(in)                                :: gdim1,dim1
      complex(dop), dimension(gdim1,gdim1,dim1), intent(inout) :: psi
      real(dop)                                                :: mnorm

      do e = 1, dim1
         do j = 1, gdim1
            do i = j, gdim1
               psi(i,j,e) = 0.5_dop*(psi(i,j,e)-psi(j,i,e))
               psi(j,i,e) = -psi(i,j,e)
            enddo
         enddo
      enddo

      call schmidtortho2(psi,gdim1**2,dim1,ierr,mnorm)
      write(2,'(1x,a,f17.12)') 'asym2d: minimal norm =', mnorm

      if( ierr .gt. 0 ) then
         write(6,'(2a,3i5)') ' asym2d: Cannot Schmidt-orthogonalize',&
                             ' spf''s.', gdim1, dim1, ierr
         write(2,'(2a,3i5)') ' asym2d: Cannot Schmidt-orthogonalize',&
                             ' spf''s.', gdim1, dim1, ierr
         call stopnow
      end if

      return
      end subroutine asym2d
!-------------------------------------------------------------------------------
!
      subroutine nsym2d(gdim1,dim1,psi,nsym2dnorm)

use global
use channels

      implicit none

      integer(long)                                            :: i,j,e,e1
      integer(long), intent(in)                                :: gdim1,dim1
      complex(dop), dimension(gdim1,gdim1,dim1), intent(inout) :: psi
      complex(dop), dimension(gdim1,gdim1,dim1)                :: tmppsi
      real(dop)                                                :: norm
      real(dop), dimension(dim1), intent(out)                  :: nsym2dnorm

      write (ilog,'(a)') ' #symm.spf, #orig.spf'

      e1=1
      e =1

!     try symmetric combination
 10   do j = 1, gdim1
         do i = 1, gdim1
            tmppsi(i,j,e) = psi(i,j,e1)+psi(j,i,e1)
         enddo
      enddo
      call schmidtortho3(tmppsi(1,1,1),tmppsi(1,1,e),gdim1**2,e-1,norm)
      if (norm .gt. 0.1) then
         write(ilog,'(2i9,a,f16.12)')&
            e,e1,'   sym accepted, norm =',norm
         nsym2dnorm(e)=norm
         e=e+1
         if (e.gt.dim1) goto 100
      else
         write(ilog,'(2i9,a,f16.12)')&
            e,e1,'   sym rejected, norm =',norm
      endif

!     try anti-symmetric combination
      do j = 1, gdim1
         do i = 1, gdim1
            tmppsi(i,j,e) = psi(i,j,e1)-psi(j,i,e1)
         enddo
      enddo
      call schmidtortho3(tmppsi(1,1,1),tmppsi(1,1,e),gdim1**2,e-1,norm)
      if (norm .gt. 1.0e-4_dop) then
         write(ilog,'(2i9,a,f16.12)')&
            e,e1,'  asym accepted, norm =',norm
        nsym2dnorm(e)=norm
        e=e+1
        if (e.gt.dim1) goto 100
      else
         write(ilog,'(2i9,a,f16.12)')&
            e,e1,'  asym rejected, norm =',norm
      endif

      e1=e1+1
      if (e1.le.dim1) goto 10
      write(6,   '(a)') ' nsym2d failed!'
      write(ilog,'(a)') ' nsym2d failed!'
      call stopnow

 100  call cpvxz(tmppsi,psi,gdim1*gdim1*dim1)
      return
      end subroutine nsym2d

!***********************************************************************
!
!     NSYM2D_FIXA
!
! Fixes the A vector, in case that nsym2d is used
! (if this isn't used, this routine does nothing).
! This code was part of rdkoeff, but rdkoeff is now called before
! nsym2d, so we now have this routine to be called after nsym2d.
!
! In/Out: psi - the A vector
!       + stuff from common blocks
!
! 12/2007 FO
!
!***********************************************************************

      subroutine nsym2d_fixa(psi,mm,nsym2dnorm)

use global
use maxv, only: maxdim
use griddatmod
use psidef
use runinwfmod
use channels

      implicit none

      integer(long)                          :: s,m,a,b
      integer(long), dimension(maxdim)       :: indx
      integer(long), intent(in)              :: mm

      real(dop)                                    :: norm
      real(dop), dimension(maxspf), intent(in)     :: nsym2dnorm
      complex(dop), dimension(adim), intent(inout) :: psi

!     this was never tested with more than one state!
      do s=1,nstate

!     loop over full A-vector, but also keep a counter (indx) for the
!     SPF of each mode
         do m=1,nmode
            indx(m)=1
         enddo
         do a=1,block(s)
            b = zpsi(s)-1+a
!           skip A-coefficient if it's zero
            if(psi(b) .ne. 0.0_dop) then
               psi(b) = psi(b)*nsym2dnorm(indx(mm))
            endif
!           increase SPF counters
            do m=1,nmode
               indx(m)=indx(m)+1
               if (indx(m).gt.dim(m,s)) then
                  indx(m)=1
               else
                  goto 10
               endif
            enddo
  10        continue
         enddo ! a
      enddo ! s

!     we probably need an overall normalization again
      call normvxz(psi,norm,adim)
      if (lmulpack) norm=norm/sqrt(dble(npacket))
      if (mpb.gt.0) norm=norm/sqrt(dble(npackts))
      if (lreflex)  norm=norm/sqrt(2.0_dop)
      call xvixdzo(norm,psi,adim)

!     log some messages
      write(ilog,*)
      write(ilog,'(a,i3,a)') &
         '**** A-coefficients modified by nsym2d ****  (mode =',mm,')'
      do b = 1,adim
         if (abs(psi(b)) .gt. tiny) &
            write(ilog,'(i6,"  (",f9.6,",",f9.6,")")') b,psi(b)
         enddo
      write(ilog,*)
      call myflush(ilog)

      return
      end subroutine nsym2d_fixa! of nsym2d_fixa

!-------------------------------------------------------------------------------
!
      subroutine sym3d(gdim1,dim1,psi)

      implicit none

      integer(long)                                                  :: i,j,k,e,ierr
      integer(long), intent(in)                                      :: gdim1,dim1
      complex(dop), dimension(gdim1,gdim1,gdim1,dim1), intent(inout) :: psi
      real(dop)                                                      :: mnorm
      real(dop), parameter                                           :: inv6=1.0_dop/6.0_dop

      do e = 1, dim1
         do k = 1, gdim1
            do j = k, gdim1
               do i = j, gdim1
                  psi(i,j,k,e) = inv6* &
                              (psi(i,j,k,e)+psi(j,k,i,e)+psi(k,i,j,e)+ &
                               psi(j,i,k,e)+psi(i,k,j,e)+psi(k,j,i,e))
                  psi(j,k,i,e) = psi(i,j,k,e)
                  psi(k,i,j,e) = psi(i,j,k,e)
                  psi(j,i,k,e) = psi(i,j,k,e)
                  psi(i,k,j,e) = psi(i,j,k,e)
                  psi(k,j,i,e) = psi(i,j,k,e)
               enddo
            enddo
         enddo
      enddo

      call schmidtortho2(psi,gdim1**3,dim1,ierr,mnorm)
      write(2,'(1x,a,f17.12)') 'sym3d: minimal norm =', mnorm

      if( ierr .gt. 0 ) then
         write(6,'(2a,3i5)') ' sym3d: Cannot Schmidt-orthogonalize', &
                             ' spf''s.', gdim1, dim1, ierr
         write(2,'(2a,3i5)') ' sym3d: Cannot Schmidt-orthogonalize', &
                             ' spf''s.', gdim1, dim1, ierr
         call stopnow
      end if

      return
      end subroutine sym3d
!-------------------------------------------------------------------------------

      subroutine symcoeff(psi,qq,lf,symcount)

use global
use maxv, only: maxdim
use dvrdatmod
use griddatmod
use psidef
use runpropmod
use channels

      implicit none

      integer(long)                                   :: nfak,nf,j,k,s,m,mm
      integer(long)                                   :: nmod,nn,nmd,adm
      integer(long), intent(in)                       :: qq,symcount
      integer(long), dimension(maxdim)                :: gpoint,jpart
      integer(long), dimension(:), allocatable        :: prmttn, workprm
      real(dop)                                       :: norm
      complex(dop), dimension(adim), intent(inout)    :: psi
      complex(dop), dimension(maxblock)               :: tmppsi
      logical(kind=4)                                 :: lf

! --  adm=adim, nmod=nmode in normal case
! --  adm=adim/npackts, mmod=nmode-1 for multi-packet single set
      if(qq.gt.0) then
         adm=adim/npackts
         nmod=nmode-1
      else
         adm=adim
         nmod=nmode
      endif

! --- Determine number of modes to be symmetrized.
      nmd=0
      do m=1,nmod
         if(symcv(m) .eq. symcount) then
            nmd=nmd+1
            jpart(nmd)=m  ! jpart maps partial to full set of indices
         endif
      enddo
      if(nmd.le.1) then
         write(6,'(a,i2,a,50i3)')   &
         'nmd=',nmd,' , symcv:',   (symcv(j),j=1,nmod)
         routine='symcoeff'
         message='There must be at least two modes for (a)symcoeff'
         call errormsg
      endif
      if(lf) then
         write(ilog,'(2a,25i3)') 'SYMCOEFF, ',   &
          'asymmetrizing coefficients for modes:',(jpart(j),j=1,nmd)
      else
         write(ilog,'(2a,25i3)') 'SYMCOEFF, ',   &
           'symmetrizing coefficients for modes:',(jpart(j),j=1,nmd)
      endif
      nf=2
      do j = 3, nmd+1
        nf = nf*j
      enddo

      allocate(prmttn(nf))
      allocate(workprm(nf))

      call permutation(prmttn,workprm,nfak,nmd)

      do s = 1, nstate
         call cpvxz(psi(zpsi(s)),tmppsi,min(block(s),adm))
         call initvxi(gpoint,ndof)
         j=0

! ---    Loop over the (comopsite) index, j, of the A-vector.
 10      j=j+1
! ---    Loop over all permutations (except first one).
         do nf = 2, nfak
            k=0
! ----   Compute mapped (comopsite) k index from transformed mode indices
! ----   The first if-block is for symmetrising all modes.
            if(nmd.eq.nmod) then
               do m=nmod, 2, -1
                  mm=prmttn(m+(nmod+1)*(nf-1))
                  k = dim(1,s)*(k+gpoint(mm)-1)
               enddo ! m
               mm=prmttn(1+(nmod+1)*(nf-1)) ! Last step, m=1.
               k = k+gpoint(mm)
! ----   The second if-block is for symmetrising a subset of modes.
            else
               nn=nmd      ! nn points to a mode to be symmetrized.
               do m=nmod, 1, -1
                  if(symcv(m).eq.symcount) then
                     mm=prmttn(nn+(nmd+1)*(nf-1))
                     mm=jpart(mm) 
                     nn=nn-1
                  else
                     mm=m
                  endif
                  if(m.eq.1) then
                     k = k+gpoint(mm)
                  else
                     k = dim(m-1,s)*(k+gpoint(mm)-1)
                  endif
               enddo ! m
            endif ! mnd.ed.nmod
            if(lf) then
               psi(zpsi(s)-1+j)=psi(zpsi(s)-1+j) + tmppsi(k)*prmttn(nf*(nmd+1))
            else
               psi(zpsi(s)-1+j)=psi(zpsi(s)-1+j)+tmppsi(k)
            endif   
         enddo    ! nf
! ----   Set the mode indices for next j
         do m=1,nmod
            if(gpoint(m).lt.dim(m,s)) then
               gpoint(m)=gpoint(m)+1
               goto 10
            else
               gpoint(m)=1
            endif
         enddo    ! m
      enddo       ! s

      call normvxz(psi,norm,adm) ! compute norm
      write(2,'(a,f10.6,a,i5,a,f17.12)') 'symcoeff: norm =',   &
            norm,',  n! =',nfak,',  diff =', dble(nfak)-norm
      if(norm.lt.1.d-9) then
         write(6,*) 'symcoeff: Cannot (a)symmetrize coeffs. norm=',norm
         write(ilog,*) 'symcoeff: Cannot (a)symmetrize coeffs. norm=',norm
         call stopnow
      endif
      call xvixdzo(norm,psi,adm) ! normalise

      deallocate(prmttn)
      deallocate(workprm)

      return
      end subroutine symcoeff
!-------------------------------------------------------------------------------

      subroutine asymcoeff(psi)

use global
use maxv, only: maxdim
use dvrdatmod
use griddatmod
use psidef
use runpropmod

      implicit none

      integer(long)                                   :: nfak,nf,j,k,s,m,mm,vz,dm
      integer(long), dimension(maxdim)                :: gpoint
      integer(long), dimension(:), allocatable        :: prmttn, workprm
      real(dop)                                       :: norm
      complex(dop), dimension(adim), intent(inout)    :: psi
      complex(dop), dimension(maxblock)               :: tmppsi

      integer(long), save                             :: tid=0 ! timer ID

      call get_timer(tid, 'Asymcoeff')
      call start_timer(tid)
      nf=6
      do j = 4, nmode+1
        nf = nf*j
      enddo
      nf=nf+1
      allocate(prmttn(nf))
      allocate(workprm(nf))
      call permutation(prmttn,workprm,nfak,nmode)

!     do nf= 1, nfak
!        write(6,'(i7,i3," |",12i3)')nf,prmttn(nf*(nmode+1)),
!    +                (prmttn(j+(nf-1)*(nmode+1)),j=1,nmode)
!     enddo
!     write(6,*)

      do s = 1, nstate
         call cpvxz(psi(zpsi(s)),tmppsi,block(s))
         call initvxi(gpoint,ndof)
         dm=dim(1,s)
         j=0

 10      j=j+1
! ---    Loop over all permutations (except first one).
         do nf = 2, nfak
            k=0
! ----   Compute mapped (comopsite) k index from transformed mode indices
            do m=nmode, 2, -1
               mm=prmttn(m+(nmode+1)*(nf-1))
               k = dm*(k+gpoint(mm)-1)
            enddo ! m
            mm=prmttn(1+(nmode+1)*(nf-1))
            vz=prmttn(nf*(nmode+1))
            k = k+gpoint(mm)
            psi(zpsi(s)-1+j)=psi(zpsi(s)-1+j)+tmppsi(k)*vz
         enddo    ! nf
! ----   Set the mode indices for next j
         do m=1,nmode
            if(gpoint(m).lt.dm) then
               gpoint(m)=gpoint(m)+1
               goto 10
            else
               gpoint(m)=1
            endif
         enddo    ! m
      enddo       ! s

      call normvxz(psi,norm,adim) ! compute norm
      write(2,'(a,f10.6,a,i5,a,f17.12)') 'asymcoeff: norm =',&
        norm,',  n! =',nfak,',  diff =', dble(nfak)-norm
      if(norm.lt.1.0e-9_dop) then
         write(6,*) 'asymcoeff: Cannot asymmetrize coeffs. norm=',norm
         write(2,*) 'asymcoeff: Cannot asymmetrize coeffs. norm=',norm
         call stopnow
      endif 
      call xvixdzo(norm,psi,adim) ! normalise

      deallocate(prmttn)
      deallocate(workprm)
      call stop_timer(tid)

      return
      end subroutine asymcoeff


! **********************************************************************

      end module symlib
