!######################################################################
!
! MCTDH-module TRAFONAT
!
!     subroutines needed for the transformation of the psi-vector to
!     the natural-orbital picture when CMFs are used
!
!     also contains a subroutine to reorthonormalise psi
!
! contains : trafonat,trafonata,d1trafonat,d2trafonat,reorthopsi
!
! 08/10/97 AR 
!
!#######################################################################

      module trafonatmod

use decimal, only: long, dop
use global
use griddatmod
use psidef
use runpropmod

use psiutil, only: d1regular
use ciselect, only: venpsi
use densitymod
use schmidtorthomod, only: schmidtortho
use wftrafo, only: natorb_tree
use linear, only: gramschmidt
use wftrafo, only: reortho_tree

use logdat
use mmlib,   only: mqxczz
use mtlib,   only: qttxzz, qtaxzz
use op1lib,  only: cpmxz, cpvxz, cpvxz_s 
use op2lib,  only: swapsxxi
use timing

      implicit none
      private
      public :: trafonat, d1trafonat, d2trafonat, reorthopsi
      contains

!#######################################################################
! TRAFONAT
! tfm = transformation matrix, i.e. eigenvector-matrix of rho
!
! V7.0    08/10/97 AR
!#######################################################################
      
      subroutine trafonat(psi,tree,tfm,jindx)

      implicit none

      integer(long), dimension(jindxdim), intent(in)   :: jindx
      complex(dop), dimension(dgldim), intent(inout)   :: psi
      complex(dop), dimension(dmatdim), intent(inout)  :: tfm
      type(wfnode_t),intent(inout)                     :: tree

      integer(long), save                              :: tid=0 ! timer ID

      if (psitype .eq. 5) then
         call natorb_tree(tree,psi)        
      else if (dentype .eq. 0) then
         call trafonat0(psi,tfm,jindx)
      else if (dentype .eq. 1) then
         call d1trafonat(psi,tfm)
      else if (dentype .eq. 2) then
         call d2trafonat(psi,tfm)
      endif

      return
      end subroutine trafonat

!#######################################################################

      subroutine trafonat0(psi,tfm,jindx)

      implicit none

      integer(long)       :: m,s,zeig1,zeig2,zeig3, &
                             izeig1,izeig2,block2,block1,ierr

      integer(long), dimension(jindxdim), intent(in)   :: jindx
      integer(long), dimension(2*fmblock)              :: worki
      complex(dop), dimension(2*fmblock)               :: workci
      complex(dop), dimension(dgldim), intent(inout)   :: psi
      complex(dop), dimension(dmatdim), intent(inout)  :: tfm
      complex(dop), dimension(2*maxblock)              :: tmpa
      complex(dop), dimension(maxphidim)               :: tmpspf
      integer(long), save                             :: tid=0 ! timer ID

      call get_timer(tid, 'Trafonat')
      call start_timer(tid)

!***********************************************************************
! Change phases of eigenvectors of the density matrix.
! Use the phase-convention: evek(e,e).gt.0, i.e. <phi_old|phi_new>.gt.0
!***********************************************************************
      do s=1,nstate
         do m=1,nmode
            call chngphs( tfm(dmat(m,s)), dim(m,s) )
         enddo
      enddo

!***********************************************************************
!     Transform spfs (Do not transform a el or packet mode or a mapped one.)
!***********************************************************************
      do s=1,nstate
         do m=1,nmode
            if (m .eq. mpb .or. m .eq. meb) cycle
            if(idmode(m).eq.0) then  
               call mqxczz(psi(zetf(m,s)),tfm(dmat(m,s)),tmpspf, &
                           subdim(m),dim(m,s))
               call cpmxz(tmpspf,psi(zetf(m,s)),subdim(m),dim(m,s))
!#########################################################
!.....Re-orthogonalise SPF's in case of relaxation.
               if(lrlx) then 
                  call schmidtortho(psi(zetf(m,s)),subdim(m),dim(m,s),ierr)
                  if( ierr .gt. 0 ) then
                     routine='trafonat'
                     write(message,'(a,i3,a,i3,a,i3)') &
                       'Cannot Schmidt-orthogonalize SPFs for s =',s, &
                       ', m =',m,', e =',ierr
                     call errormsg
                  endif
               endif
!#########################################################
            endif
         enddo
      enddo
      
!***********************************************************************
!     Transform A-vectors
!***********************************************************************
      do s=1,nstate
         zeig1=1
         zeig2=fmblock+1     !    maxblock+1

         if (citype .gt. 0) then
            block2=block(s)
            izeig1=1
            izeig2=fmblock+1

            block1=vdim(1,s)*dim(1,s)*ndim(1,s) 
            call venpsi(1,jindx(zpsi(s)),worki(izeig1), &
                 jindx(zpsi(s)),block(s),block1,block2, &
                 nmode,vdim(1,s),dim(1,s),ndim(1,s),jvdim, &
                 workci(zeig1),psi(zpsi(s)))
            block2=block1
            call qttxzz(tfm(dmat(1,s)),workci(zeig1), &
                 workci(zeig2),vdim(1,s),dim(1,s),ndim(1,s))

            do m=2,nmode
               if (m .eq. mpb .or. m .eq. meb) cycle
               block1=vdim(m,s)*dim(m,s)*ndim(m,s)
               zeig3=izeig1
               izeig1=izeig2
               izeig2=zeig3
               call venpsi(m,jindx(zpsi(s)),worki(izeig1), &
                    worki(izeig2),block(s),block1,block2, &
                    nmode,vdim(m,s),dim(m,s),ndim(m,s),jvdim, &
                    workci(zeig1),workci(zeig2))
               block2=block1
               call qttxzz(tfm(dmat(m,s)),workci(zeig1), &
                    workci(zeig2),vdim(m,s),dim(m,s),ndim(m,s))
            enddo

            call cpvxz_s(workci(zeig2),psi(zpsi(s)),worki(izeig1), &
                    jindx(zpsi(s)),block2,block(s))

         else
            call qttxzz(tfm(dmat(1,s)),psi(zpsi(s)), &
                    tmpa(zeig2),vdim(1,s),dim(1,s),ndim(1,s))
            do m=2,nmode
               if (m .eq. mpb .or. m .eq. meb) cycle ! do not transform packet
                                                     ! or electronic mode
               zeig3=zeig1
               zeig1=zeig2
               zeig2=zeig3
               call qttxzz(tfm(dmat(m,s)),tmpa(zeig1), &
                    tmpa(zeig2),vdim(m,s),dim(m,s),ndim(m,s))
            enddo

            call cpvxz(tmpa(zeig2),psi(zpsi(s)),block(s))
         endif
      enddo

      call stop_timer(tid)

      return
      end subroutine trafonat0

!#######################################################################
! TRAFONAT1                                                             
!                                                                       
! - transforms the psi-vector to the natural-orbital picture            
!   for DENSITY I                                                       
!                                                                       
!  06/2002 CC                                                    
!#######################################################################

      subroutine d1trafonat(psi,tfm)

      implicit none

      complex(dop), dimension(dgldim), intent(inout)  :: psi
      complex(dop), dimension(dmatdim), intent(inout) :: tfm
      complex(dop), dimension(maxphidim)              :: tmpspf
      complex(dop), dimension(2*maxblock)             :: tmpa
 
      integer(long)       :: m,s,t,zeig1,zeig2
      integer(long), save                             :: tid=0 ! timer ID

      call get_timer(tid, 'Trafonat')
      call start_timer(tid)

      routine = 'd1trafonat'
      if (nstate.ne.1) then
         message = 'Error in d1trafonat: nstate>=2 not implemented yet!'
         call errormsg
      endif

      call d1regular(psi,.true.)

!***********************************************************************
! Change phases of eigenvectors of the density matrix.
! Use the phase-convention: evek(e,e).gt.0, i.e. <phi_old|phi_new>.gt.0
!***********************************************************************
      do s=1,nstate
         do t=1,nstate
            do m=1,nmode
               call chngphs(tfm(ddmat(m,s,t)), ddim(m,s,t) )
            enddo
         enddo
      enddo
     
!***********************************************************************
!  Transform spdos, SIGMA(I)' =
!***********************************************************************
      do s=1,nstate
         do t=1,nstate
            do m=1,nmode
               call mqxczz(psi(dzetf(m,s,t)),tfm(ddmat(m,s,t)),tmpspf, &
                    subdim(m),ddim(m,s,t))
               call cpmxz(tmpspf,psi(dzetf(m,s,t)),subdim(m),ddim(m,s,t))
            enddo
         enddo
      enddo
 

!***********************************************************************
!     Transform A-vectors,                                              
!     A(KM)' =                 
!***********************************************************************
      do s=1,nstate
         do t=1,nstate
            zeig1 = 1
            zeig2 = maxblock+1
            call qttxzz(tfm(ddmat(1,s,t)),psi(dzpsi(s,t)), &
                 tmpa(zeig2),vddim(1,s,t),ddim(1,s,t),nddim(1,s,t)) 
          
            do m=2,nmode
               call swapsxxi(zeig1,zeig2)
               call qttxzz(tfm(ddmat(m,s,t)),tmpa(zeig1), &
                 tmpa(zeig2),vddim(m,s,t),ddim(m,s,t),nddim(m,s,t))
            enddo
                
            call cpvxz(tmpa(zeig2),psi(dzpsi(s,t)),dblock(s,t))
         enddo
      enddo

      call d1regular(psi,.true.)

      call stop_timer(tid)

      return
      end subroutine d1trafonat  


!#######################################################################
! D2TRAFONAT                                                            
!                                                                       
! - transforms the psi-vector to the natural-orbital picture            
!   for DENSITY II                                                      
!                                                                       
!  09/2000 CC                                                    
!#######################################################################

      subroutine d2trafonat(psi,tfm)

      implicit none

      integer(long)                                   :: m,s,zeig1,zeig2,zeig3
      complex(dop), dimension(dgldim), intent(inout)  :: psi
      complex(dop), dimension(dmatdim), intent(inout) :: tfm
      complex(dop), dimension(maxphidim)              :: tmpspf
      complex(dop), dimension(2*maxblock*maxblock)    :: tmpa

      integer(long), save                             :: tid=0 ! timer ID

      call get_timer(tid, 'Trafonat')
      call start_timer(tid)

      routine = 'd2trafonat'

      if (nstate.ne.1) then
         routine='trafonat2'
         message='Error in trafonat2, nstate>1 not implemented yet!'
         call errormsg
      endif

!***********************************************************************
! Change phases of eigenvectors of the density matrix.
! Use the phase-convention: evek(e,e).gt.0, i.e. <phi_old|phi_new>.gt.0
!***********************************************************************
      do s=1,nstate
         do m=1,nmode
            call chngphs( tfm(dmat(m,s)), dim(m,s) )
         enddo
      enddo

!***********************************************************************
!     Transform spfs, Phi(J)' = SUM(L)[ adjoint(U(JL)) * Phi(L) ]       
!***********************************************************************
      do s=1,nstate
         do m=1,nmode
            call mqxczz(psi(zetf(m,s)),tfm(dmat(m,s)),tmpspf, &
                 subdim(m),dim(m,s))
            call cpmxz(tmpspf,psi(zetf(m,s)),subdim(m),dim(m,s))
         enddo
      enddo

!***********************************************************************
!     Transform A-vectors,                                              
!     A(KM)' = SUM(JL)[ U(JK) * A(JL) * adjoint(U(ML)) ]                
!***********************************************************************
      do s=1,nstate
! ---    left-side transformation
         zeig1=1
         zeig2=block(s)*block(1)+1
         call cpvxz(psi(dzpsi(1,s)),tmpa(zeig2),block(s)*block(1))

         do m=1,nmode
            zeig3=zeig1
            zeig1=zeig2
            zeig2=zeig3
            call qttxzz(tfm(dmat(m,s)),tmpa(zeig1),tmpa(zeig2), &
                 vdim(m,s),dim(m,s),ndim(m,s)*block(s))

!            call qtaxzz(tfm(dmat(m,s)),workc(zeig2),workc(zeig1),
!     +           block(s)*vdim(m,s),dim(m,s),ndim(m,s))

         enddo
         call cpvxz(tmpa(zeig2),psi(dzpsi(s,s)),block(s)*block(1))

! ---    right-side transformation
         zeig1=1
         zeig2=block(s)*block(1)+1
         call cpvxz(psi(dzpsi(1,s)),tmpa(zeig2),block(s)*block(1))
         do m=1,nmode
            zeig3=zeig1
            zeig1=zeig2
            zeig2=zeig3
            call qtaxzz(tfm(dmat(m,s)),tmpa(zeig1),tmpa(zeig2), &
                 block(s)*vdim(m,s),dim(m,s),ndim(m,s))
         enddo
         call cpvxz(tmpa(zeig2),psi(dzpsi(s,s)),block(s)*block(1))

      enddo

      call stop_timer(tid)

      return
      end subroutine d2trafonat

!#######################################################################
!
! Schmidt-orthogonalize initial wave function if desired
!    1. compute density matrices
!    2. transform to natural orbital basis
!    3. Schmidt-orthogonalize single-particle functions
!
!#######################################################################

      subroutine reorthopsi(psi,tree,dicht1,dicht2,dicht3,dicht4,jindx,&
                 gs2)

      implicit none

      integer(long), dimension(jindxdim), intent(in)  :: jindx
      integer(long)                                   :: m,s,t,ierr,i,gdim
      complex(dop), dimension(dgldim), intent(inout)  :: psi
      complex(dop), dimension(dmatdim), intent(inout) :: dicht1,dicht2,&
                                                         dicht4
      complex(dop), dimension(gs2matdim), intent(inout) :: gs2
      real(dop), dimension(d3matdim),intent(inout)      :: dicht3
      type(wfnode_t), intent(inout)                     :: tree
      type(wfnode_t),pointer                            :: cnode

      call density(psi,tree,dicht1,dicht2,dicht3,dicht4,jindx,gs2)
      call trafonat(psi,tree,dicht4,jindx)

      ierr=0
      if (dentype .eq. 0 .or. dentype .eq. 2) then
         do s=1,nstate
            do m=1,nmode
               call schmidtortho(psi(zetf(m,s)),subdim(m),dim(m,s),ierr)
            enddo
         enddo
      else if (dentype .eq. 1) then
         do t=1,nstate
            do s=1,nstate
               do m=1,nmode
                  call schmidtortho(psi(dzetf(m,s,t)),subdim(m),&
                       ddim(m,s,t),ierr)
               enddo
            enddo
         enddo
      endif

! middle layers of ML-MCTDH
      if (psitype .eq. 5) then
        do i=1,tree%tot_nodes

            cnode => tree%swap_up(i)%p
            if(cnode%isprimitive) cycle
            if(cnode%istop) cycle
            gdim = cnode%z(1,2) - cnode%z(1,1) + 1
            ierr = 0
            call gramschmidt(psi(cnode%z(1,1):cnode%z(cnode%navecs,2)),&
                             gdim,cnode%navecs,ierr)
        end do
      endif

      if( ierr .gt. 0 ) then
         routine='Reorthopsi'
         message='Cannot Schmidt-orthogonalize spf''s.'
         call errormsg
      end if

      end subroutine reorthopsi

      end  module trafonatmod

