!#######################################################################
      module zeighpsimod

use decimal, only: long
use maxv, only: maxkoe
use global
use griddatmod
use operdef
use psidef
use hpsimod
use dirdyn

use datenmod
use logdat
use runpropmod
use runinwfmod

      implicit none
      private
      public :: operflag,zeighpsi,d1operflag,d1zeighpsi,d2zeighpsi,d1maxmatwk
      contains

!#######################################################################

      subroutine operflag(nham)

      implicit none

      integer(long), intent(in) :: nham
      integer(long)             :: m,f,k,k1,k2,nmatch,n,k3,k4

!-----------------------------------------------------------------------
! find first occurence of correlated operator term in list
!-----------------------------------------------------------------------
      k3=zham(nham)
      k4=zham(nham)+kzahl(nham)-1
      
      do k=k3,k4
         do m=1,nmode
            k2=0
            do k1=k-1,k3,-1
               if (ki(k) .eq. ki(k1) .and. kf(k) .eq. kf(k1)) then
                  nmatch=0
                  do n=1,nspfdof(m)
                     f=spfdof(n,m)
                     if (hamilton(f,k) .eq. hamilton(f,k1)) &
                        nmatch=nmatch+1
                  enddo
                  if (nmatch .eq. nspfdof(m)) k2=k1
               endif
            enddo
            if (k2.eq.0) then
               kfirst(m,k)=k
            else
               kfirst(m,k)=k2
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! Muld terms oddur only once
!-----------------------------------------------------------------------
      do n=1,nmulpot(nham)
         k=zham(nham)+kzahl(nham)+n-1
         kfirst(1:nmode,k) = k
      enddo

      return
      end subroutine operflag


!#######################################################################

      subroutine zeighpsi(nham)

      implicit none

      integer(long), intent(in) :: nham
      integer(long)             :: m,k,s,s1,zeig,k1,k2,k3,k4,p1
     
!-----------------------------------------------------------------------
! If no operator is defined, return
!-----------------------------------------------------------------------
      if (nham .eq. 0) return

!-----------------------------------------------------------------------
! determine pointers to hpsi array
!-----------------------------------------------------------------------
      zeig = hpsidim+1
      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
      k3=k2+nmulpot(nham)
      do m = 1,nmode
         if (gwpm(m)) cycle

         do k = k1,k2
            if (.not. diag(m,k)) then
               if (k .eq. kfirst(m,k)) then
                  s=ki(k)
                  zhpsi(m,k) = zeig
                  zeig = zeig+subdim(m)*dim(m,s)
               else
                  k4=kfirst(m,k)
                  zhpsi(m,k) = zhpsi(m,k4)
               endif
            else
               zhpsi(m,k) = 0
            endif
         enddo

         do k=k2+1,k3
            zhpsi(:,k) = 0
            if (m .eq. meb) then  ! check muld terms for electronic basis
               zhpsi(meb,k) = zeig
               zeig = zeig+subdim(meb)*dim(meb,1)
            endif
         enddo

      enddo
      hpsidim = zeig-1

!-----------------------------------------------------------------------
! pointers for single particle function matrices hloch
!-----------------------------------------------------------------------
      zeig=hlochdim+1
      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
      do m=1,nmode
         if(idmode(m).eq.0) then
            do k=k1,k2            
               if (.not. diag(m,k)) then
                  s=kf(k)
                  s1=ki(k)
                  phloch(m,k) = zeig
                  zeig=zeig+dim(m,s)*dim(m,s1)
               else
                  phloch(m,k) = 1
               endif
            enddo
         else
            do k=k1,k2            
               phloch(m,k) = phloch(idmode(m),k) 
            enddo
         endif

         k=zham(nham)+kzahl(nham)-1
         do k3=1,nmulpot(nham)
            k=k+1
            if (.not. diag(m,k)) then
               s=kf(k)
               phloch(m,k) = zeig
               zeig=zeig+dim(m,s)*subdim(m)
            else
               phloch(m,k) = 1
            endif
         enddo
      enddo
      hlochdim = zeig-1

      do k=1,khtot(1)
         do m=1,nmode
            phloch(m,k) = max(phloch(m,k),1)
         enddo
      enddo
      hlochdim = max(hlochdim,1)

!-----------------------------------------------------------------------
! pointers for single-particle function matrices
!
! If the "proj-h" input option is used or CAPS are present the matrix
! elements <phi|h|phi> are also needed for the uncorrelated Hamiltonian
! terms. Effectively the number of Hamiltonian terms is then
! kzahl+nstate.
!-----------------------------------------------------------------------
      zeig = pmatdim+1
      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
      k3=k2+nmulpot(nham)
      do m=1,nmode
         do k=k1,k2
            if (.not. diag(m,k)) then
               if (k .eq. kfirst(m,k)) then
                  s=kf(k)
                  s1=ki(k)
                  pmat(m,k) = zeig
                  zeig = zeig+dim(m,s)*dim(m,s1)
               else
                  k4=kfirst(m,k)
                  pmat(m,k)=pmat(m,k4)
               endif
            else
               pmat(m,k)=0
            endif
         enddo

         do k=k2+1,k3
            pmat(:,k) = 0
            if (m .eq. meb) then  ! check muld terms for electronic basis
               pmat(meb,k) = zeig
               zeig = zeig+dim(meb,1)*dim(meb,1)
            endif
         enddo

         k=k3    !  uncorrelated terms
         do s = 1,nstate
            if( khzahl(m,s,nham) .gt. 0 ) then
               pmat(m,k+s) = zeig
               zeig = zeig+dim(m,s)*dim(m,s)
            else
               pmat(m,k+s) = 0
            endif 
         enddo
      enddo
      pmatdim = zeig-1
      pmatdim = max(pmatdim,1)

!-----------------------------------------------------------------------
!      gh        -   ghmat(k,m)     -     ghmatdim
!      hunG      -   hunGmat(s)     -     hunGdim      
!-----------------------------------------------------------------------
      zeig = 1 
      do s = 1,nstate  
         do m = 1,nmode  
            if (gwpm(m))then 
               hungmat(m,s,nham) = zeig  
               do p1 = 1,npg(m)
                  zeig = zeig+gwpdim(m,s)**2 
               enddo 
            endif 
         enddo 
      enddo  
      hungdim = zeig-1  
      
      k1=zham(nham)
      k2=zham(nham)+khtot(nham)-1
      k3=zham(nham)+kzahl(nham)-1       
      zeig = ghmatdim+1 
!      write(*,*)k1,k2,"source/mctdhlib/zeighpsimod.f90"
      do m = 1,nmode 
         if(gwpm(m))then   
            do k=k1,k2
               if (.not. diag(m,k)) then 
                  if (k .eq. kfirst(m,k) .or. k .gt. k3) then 

                     s=kf(k) 
                     s1=ki(k) 
                     ghmat(k,m) = zeig 
                     do p1 = 1,npg(m)
                        zeig = zeig+gwpdim(m,s)*gwpdim(m,s1) 
                     enddo 
                  else 
                     k4=kfirst(m,k)
                     ghmat(k,m) = ghmat(k4,m) 
                  endif 
               else
                  ghmat(k,m) = 1
               endif 
            enddo 
         endif 
      enddo 
      ghmatdim = zeig-1

!-----------------------------------------------------------------------
! Pointers for GWP potential derivatives
!-----------------------------------------------------------------------
      k1=zham(1)
      k2=zham(1)+kzahl(1)-1       
      k3=zham(1)+khtot(1)-1
      zeig=1
      ghder1(:,:)=0
      do m = 1,nmode 
         if (.not. gwpm(m)) cycle
         if (.not. (gwpcxm(m) .gt. 0) .and. .not. lcap) cycle

         do k=k1,k2   ! correlated terms
            if (ksi(k) .ne. ksf(k)) cycle

            if (klha(k) .eq. 0) then    ! usual operator terms
               if (k .eq. kfirst(m,k)) then 
                  s=kf(k) 
                  s1=ki(k) 
                  ghder1(k,m) = zeig 
                  do p1 = 1,npg(m)
                     zeig = zeig+nspfdof(m)*gwpdim(m,s1) 
                  enddo 
               else 
                  k4=kfirst(m,k)
                  ghder1(k,m) = ghder1(k4,m) 
               endif 
            else if (klha(k) .eq. 1) then  ! KLHA terms (store at 0 order)
               s=kf(k) 
               s1=ki(k) 
               ghder1(k,m) = zeig 
               do p1 = 1,npg(m)
                  zeig = zeig+nspfdof(m)*gwpdim(m,s1) 
               enddo 
            endif
         enddo 

         do k=k2+1,k3  ! uncorrelated terms
            if (diag(m,k)) cycle

            s=kf(k) 
            s1=ki(k) 
            ghder1(k,m) = zeig 
            do p1 = 1,npg(m)
               zeig = zeig+nspfdof(m)*gwpdim(m,s1) 
            enddo 
         enddo 
      enddo 
      ghder1dim = max(zeig-1,1)

!-----------------------------------------------------------------------
! Pointers for CDVR points (cdvr3)
!-----------------------------------------------------------------------
      if (lcdvr .or. ltdvr) then
         zeig=1
         do s=1,nstate
            do m=1,nmode
               if(idmode(m).eq.0) then
                  c3mat(m,s)=zeig
                  zeig=zeig+dim(m,s)*nspfdof(m)
               else
                  c3mat(m,s)=c3mat(idmode(m),s)
               endif
            enddo
         enddo
         c3matdim=zeig-1
      else
         c3matdim=1
      endif

      return
      end subroutine zeighpsi

!#######################################################################

      subroutine d1operflag(nham)

      implicit none

      integer(long)             :: m,f,k,k1,k2,k3,k4,kt,kt1,kt2,nmatch,n
      integer(long), intent(in) :: nham

!-----------------------------------------------------------------------
! find first occurence of correlated operator term in list
!-----------------------------------------------------------------------
      k3=zham(nham)
      k4=zham(nham)+kzahl(nham)-1
      do k=k3,k4
!     Hamiltonian operators
         if (kact(k).eq.1) then
            do m=1,nmode
               k2=0
               do k1=k-1,k3,-1
                  if (kact(k1).eq.1) then
                     if (ki(k).eq.ki(k1) .and. kf(k).eq.kf(k1)) then
                        nmatch=0
                        do n=1,nspfdof(m)
                           f=spfdof(n,m)
                           if (hamilton(f,k) .eq. hamilton(f,k1)) then
                              nmatch=nmatch+1
                           endif
                        enddo
                        if (nmatch .eq. nspfdof(m)) k2=k1
                     endif
                  endif
               enddo
               if (k2.eq.0) then
                  kfirst(m,k)=k
               else
                  kfirst(m,k)=k2
               endif
            enddo
         endif
!     pure left acting operators
         if (kact(k).eq.2) then
            do m=1,nmode
               k2=0
               do k1=k-1,k3,-1
                  if (kact(k1).eq.2) then
                     if (ki(k).eq.ki(k1) .and. kf(k).eq.kf(k1)) then
                        nmatch=0
                        do n=1,nspfdof(m)
                           f=spfdof(n,m)
                           if (hamilton(f,k) .eq. hamilton(f,k1)) then
                              nmatch=nmatch+1
                           endif
                        enddo
                        if (nmatch .eq. nspfdof(m)) k2=k1
                     endif
                  endif
               enddo
               if (k2.eq.0) then
                  kfirst(m,k)=k
               else
                  kfirst(m,k)=k2
               endif
            enddo
         endif
!     pure right acting operators
         if (kact(k).eq.3) then
            do m=1,nmode
               k2=0
               do k1=k-1,k3,-1
                  if (kact(k1).eq.3) then
                     if (ki(k).eq.ki(k1) .and. kf(k).eq.kf(k1)) then
                        nmatch=0
                        do n=1,nspfdof(m)
                           f=spfdof(n,m)
                           if (hamilton(f,k) .eq. hamilton(f,k1)) then
                              nmatch=nmatch+1
                           endif
                        enddo
                        if (nmatch .eq. nspfdof(m)) k2=k1
                     endif
                  endif
               enddo
               if (k2.eq.0) then
                  kfirst(m,k)=k
               else
                  kfirst(m,k)=k2
               endif
            enddo
         endif
!     left-right acting operators
         if (kact(k).eq.4) then
            kt = ktwin(k)            
            do m=1,nmode
               k2=0
               kt2=0
               do k1=k-1,k3,-1
                  if (kact(k1).eq.4) then
                     if (ki(k).eq.ki(k1) .and. kf(k).eq.kf(k1)) then
                        kt1 = ktwin(k1)
                        if (ki(kt).eq.ki(kt1) .and. kf(kt).eq.kf(kt1)) &
                             then
                           nmatch=0
                           do n=1,nspfdof(m)
                              f=spfdof(n,m)
                              if (hamilton(f,k) .eq. hamilton(f,k1)) &
                                   then
                                 nmatch=nmatch+1
                              endif
                              if (hamilton(f,kt) .eq. hamilton(f,kt1)) &
                                   then
                                 nmatch=nmatch+1
                              endif
                           enddo
                           if (nmatch .eq. 2*nspfdof(m)) then
                              k2=k1
                              kt2=kt1
                           endif
                        endif
                     endif
                  endif
               enddo
               if (k2.eq.0) then
                  kfirst(m,k)=k
                  kfirst(m,kt)=kt
               else
                  kfirst(m,k)=k2
                  kfirst(m,kt)=kt2
               endif
            enddo
         endif
      enddo

!-----------------------------------------------------------------------
!     Set diag flags for left-right action
!-----------------------------------------------------------------------
      do k=k3,k4
         if (kact(k).eq.4) then
            kt = ktwin(k)
            do m=1,nmode
!     Note that for ki .ne. kf always diag(m,k)=.false.
               diag(m,k) = diag(m,k) .and. diag(m,kt)
            enddo
         endif
      enddo


      return
      end subroutine d1operflag


!*******************************************************************
! 
! subroutine d1zeighpsi
!
! 01/2002 CC
!*******************************************************************
      subroutine d1zeighpsi(nham)

      implicit none

      integer(long)             :: k,k1,k2,k3,k4,kt,zeig,m,s,s1,t,t1,e
      integer(long), intent(in) :: nham

!-----------------------------------------------------------------------
! If no operator is defined, return
!-----------------------------------------------------------------------
      if (nham.eq.0) return

!-----------------------------------------------------------------------
! determine pointers to hpsi array
!-----------------------------------------------------------------------
      zeig = hpsidim+1
      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
      k3=k2+nmulpot(nham)
      do m = 1,nmode
         do k = k1,k2
!           Hamiltonian operators
            if (kact(k).eq.1) then
               if (.not. diag(m,k)) then
                  if (k .eq. kfirst(m,k)) then
                     s=ki(k)
                     do t=1,nstate
                        dzhpsi(m,k,t) = zeig
                        do e = 1,ddim(m,s,t)
                           zeig = zeig+subdim(m)
                        enddo
                     enddo
                  else
                     k4=kfirst(m,k)
                     do t=1,nstate
                        dzhpsi(m,k,t) = dzhpsi(m,k4,t)
                     enddo
                  endif
               else
                  do t=1,nstate
                     dzhpsi(m,k,t) = 0
                  enddo 
               endif

            endif
!     pure left acting operators
            if (kact(k).eq.2) then
               if (.not. diag(m,k)) then
                  if (k .eq. kfirst(m,k)) then
                     s=ki(k)
                     do t=1,nstate
                        dzhpsi(m,k,t) = zeig
                        do e = 1,ddim(m,s,t)
                           zeig = zeig+subdim(m)
                        enddo
                     enddo
                  else
                     k3=kfirst(m,k)
                     do t=1,nstate
                        dzhpsi(m,k,t) = dzhpsi(m,k3,t)
                     enddo
                  endif
               endif
            endif
!     pure right acting operators
            if (kact(k).eq.3) then
               if (.not. diag(m,k)) then
                  if (k .eq. kfirst(m,k)) then
                     t=kf(k)
                     do s=1,nstate
                        dzhpsi(m,k,s) = zeig
                        do e = 1,ddim(m,s,t)
                           zeig = zeig+subdim(m)
                        enddo
                     enddo
                  else
                     k3=kfirst(m,k)
                     do s=1,nstate
                        dzhpsi(m,k,s) = dzhpsi(m,k3,s)
                     enddo
                  endif
               endif
            endif
!     left-right acting operators
            if (kact(k).eq.4) then
               kt = ktwin(k)            
               if (.not. diag(m,k)) then
                  s = ki(k)
                  t = kf(kt)
                  dzhpsi(m,k,1) = zeig
                  do e = 1,ddim(m,s,t)
                     zeig = zeig+subdim(m)
                  enddo
               else
                  k3=kfirst(m,k)
                  dzhpsi(m,k,1) = dzhpsi(m,k3,1)
               endif
            endif                  
         enddo

         do k=k2+1,k3
            t=1  ! this is for single-set
            dzhpsi(:,k,t) = 0
            if (m .eq. meb) then  ! check muld terms for electronic basis
               dzhpsi(meb,k,t) = zeig
               zeig = zeig+subdim(meb)*ddim(meb,1,1)
            endif
         enddo

      enddo
      hpsidim = hpsidim + zeig-1

      if (psitype .eq. 5) zhpsi(:,:) = dzhpsi(:,:,1)

!-----------------------------------------------------------------------
!  pointers and size for density type matrices
!  dicht1,dicht2,dicht4,cdvr2,cdvr4,dicht3  allocated in zeiginwf
!
! Pointers for CDVR points (cdvr3)
!-----------------------------------------------------------------------
      if (lcdvr .or. ltdvr) then
         zeig=1
         do s=1,nstate
            do m=1,nmode
               if (m .eq. meb) cycle

               if(idmode(m).eq.0) then
                  c3mat(m,s)=zeig
                  zeig=zeig+dim(m,s)*nspfdof(m)
               else
                  c3mat(m,s)=c3mat(idmode(m),s)
               endif
            enddo
         enddo
         c3matdim=zeig-1
      else
         c3matdim=1
      endif

!-----------------------------------------------------------------------
! pointers for single particle function matrices hloch
!-----------------------------------------------------------------------
      zeig=hlochdim+1
      do m=1,nmode
         do k=k1,k2
!     Hamiltonian, pure left acting and pure right acting operators
            if ((kact(k).eq.1) .or. (kact(k).eq.2) .or. (kact(k).eq.3))then
               if (.not. diag(m,k)) then
                  s=kf(k)
                  s1=ki(k)
                  do t=1,nstate
                     dphloch(m,k,t) = zeig
!     ddim(m,s,t) = ddim(m,t,s)
                     zeig=zeig+ddim(m,s,t)*ddim(m,s1,t)
                  enddo
               else
                  do t=1,nstate
                     dphloch(m,k,t) = 1
                  enddo
               endif
            endif
!     left-right acting operators
            if (kact(k).eq.4) then
               kt = ktwin(k)            
               if (.not. diag(m,k)) then
                  s = ki(k)
                  t = kf(kt)
                  dphloch(m,k,1) = zeig
                  zeig=zeig+ddim(m,s,t)*ddim(m,s,t)
               else
                  do t=1,nstate
                     dphloch(m,k,t) = 1
                  enddo
               endif
            endif
         enddo

         k=k1
         do k3=1,nmulpot(1)
            message='Multidimensional Hterms not yet implemented.'
            call errormsg
         enddo
      enddo
      hlochdim = hlochdim + zeig-1

      if (psitype .eq. 5) phloch(:,:) = dphloch(:,:,1)

!-----------------------------------------------------------------------
! pointers for single-particle function matrices
!
! If the "proj-h" input option is used or CAPS are present the matrix
! elements <phi|h|phi> are also needed for the uncorrelated Hamiltonian
! terms. Effectively the number of Hamiltonian terms is then
! kzahl(1)+nstate.
!-----------------------------------------------------------------------
      if (lfunkr .or. lcap) then
         if (k1+nstate .gt. maxkoe) then
            routine = 'zeigprop'
            message = 'Redimension maxkoe in global.inc. Minimum: '
            write(message(44:49),'(i5)') kzahl(1)+nstate
            call errormsg
         endif
      endif
         
      zeig = pmatdim+1
      do m = 1,nmode
         do k=k1,k2
!     Hamiltonian operators
            if (kact(k).eq.1) then
               if (.not. diag(m,k)) then
                  if (k .eq. kfirst(m,k)) then
                     s=kf(k)
                     s1=ki(k)
                     do t=1,nstate
                        dpmat(m,k,t) = zeig
                        zeig = zeig+ddim(m,s,t)*ddim(m,s1,t)
                     enddo
                  else
                     k4=kfirst(m,k)
                     do t=1,nstate
                        dpmat(m,k,t)=dpmat(m,k4,t)
                     enddo
                  endif
               else
                  do t=1,nstate
                     dpmat(m,k,t) = 0
                  enddo
               endif

            endif
!     pure left acting operators
            if (kact(k).eq.2) then
               if (.not. diag(m,k)) then
                  if (k .eq. kfirst(m,k)) then
                     s=kf(k)
                     s1=ki(k)
                     do t=1,nstate
                        dpmat(m,k,t) = zeig
                        zeig = zeig+ddim(m,s,t)*ddim(m,s1,t)
                     enddo
                  else
                     k3=kfirst(m,k)
                     do t=1,nstate
                        dpmat(m,k,t)=dpmat(m,k3,t)
                     enddo
                  endif
               else
                  do t=1,nstate
                     dpmat(m,k,t) = 1
                  enddo
               endif
            endif
!     pure right acting operators
            if (kact(k).eq.3) then
               if (.not. diag(m,k)) then
                  if (k .eq. kfirst(m,k)) then
                     t=ki(k)
                     t1=kf(k)
                     do s=1,nstate
                        dpmat(m,k,s) = zeig
                        zeig = zeig + ddim(m,s,t)*ddim(m,s,t1)
                     enddo
                  else
                     k3=kfirst(m,k)
                     do s=1,nstate
                        dpmat(m,k,s)=dpmat(m,k3,s)
                     enddo
                  endif
               else
                  do t=1,nstate
                     dpmat(m,k,t) = 1
                  enddo
               endif
            endif
!     left-right acting operators
            if (kact(k).eq.4) then
               kt = ktwin(k)            
               if (.not. diag(m,k)) then
                  if (k .eq. kfirst(m,k)) then
                     s=kf(k)
                     t=ki(kt)
                     dpmat(m,k,1) = zeig
                     zeig = zeig + ddim(m,s,t)*ddim(m,s,t)
                  else
                     k3=kfirst(m,k)
                     dpmat(m,k,1)=dpmat(m,k3,1)
                  endif
               else
                  do t=1,nstate
                     dpmat(m,k,t) = 1
                  enddo
               endif
            endif
         enddo
         do k=k2+1,k3
            dpmat(:,k,t) = 0
            if (m .eq. meb) then  ! check muld terms for electronic basis
               dpmat(meb,k,t) = zeig
               zeig = zeig+ddim(meb,1,1)*ddim(meb,1,1)
            endif
         enddo
!        if (lfunkr.or.lcap.or.(inwfop(1).gt.0)) then
            k=zham(nham)+kzahl(nham)-1
            do t = 1,nstate
               do s = 1,nstate
                  if (khzahl(m,s,nham) .gt. 0) then
                     dpmat(m,k+s,t) = zeig
                     zeig = zeig+ddim(m,s,t)*ddim(m,s,t)
                  else
                     dpmat(m,k+s,t) = 1
                  endif
               enddo
            enddo
!         endif
      enddo                     ! of m=1,nmode
      pmatdim = pmatdim + zeig-1

      if (psitype .eq. 5) pmat(:,:) = dpmat(:,:,1)

      return 
      end subroutine d1zeighpsi


!#####################################################################
!                                                                     
!                                                                     
! 08/2001 CC                                                          
!                                                                     
!#####################################################################

      subroutine d2zeighpsi(nham)

      implicit none

      integer(long), intent(in) :: nham
      integer(long)             :: zeig,m,k,k1,k2,k3,k4,s,s1,e
      integer(long)             :: n,p1

!--------------------------------------------------------------------
! If no operator is defined, return
!--------------------------------------------------------------------
      if (nham.eq.0) return

!--------------------------------------------------------------------
! determine pointers to hpsi array
!--------------------------------------------------------------------
      zeig = hpsidim+1
      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
      k3=k2+nmulpot(nham)
      do m = 1,nmode
         if (gwpm(m)) cycle

         do k = k1,k2
            if (.not. diag(m,k)) then
               if (k .eq. kfirst(m,k)) then
                  s=ki(k)
                  zhpsi(m,k) = zeig
                  do e = 1,dim(m,s)
                     zeig = zeig+subdim(m)
                  enddo
               else
                  k4=kfirst(m,k)
                  zhpsi(m,k) = zhpsi(m,k4)
               endif
            else
               zhpsi(m,k) = 0
            endif
         enddo

         do k=k2+1,k3
            zhpsi(:,k) = 0
            if (m .eq. meb) then  ! check muld terms for electronic basis
               zhpsi(meb,k) = zeig
               zeig = zeig+subdim(meb)*dim(meb,1)
            endif
         enddo

!     Projection
         if (ldproj) then
            do k=k1,k2
!     k = 2
               if ((kact(k).eq.2) .and. (k.lt.ktwin(k))) then
                  k4 = ktwin(k)
                  if (diag(m,k)) then
                     zhpsi2(m,k) = zhpsi(m,k4)
                  else
                     if (diag(m,k4)) then
                        zhpsi2(m,k) = zhpsi(m,k)
                     else
                        s=ki(k4)
                        zhpsi2(m,k) = zeig
                        do e = 1,dim(m,s)
                           zeig = zeig+subdim(m)
                        enddo
                     endif
                  endif
               else
!     k = 1,3,4
                  zhpsi2(m,k) = zhpsi(m,k)
               endif
            enddo
!     No projection
         else 
            do k = k1,k2
!     k = 2,3
               if (((kact(k).eq.2).or.(kact(k).eq.3)) .and.  &
                    (k.lt.ktwin(k))) then
                  k4 = ktwin(k)
                  if (diag(m,k)) then
                     zhpsi2(m,k) = zhpsi(m,k4)
                  else
                     if (diag(m,k4)) then
                        zhpsi2(m,k) = zhpsi(m,k)
                     else
                        s=ki(k4)
                        zhpsi2(m,k) = zeig
                        do e = 1,dim(m,s)
                           zeig = zeig+subdim(m)
                        enddo
                     endif
                  endif
               else
!     k = 1,4
                  zhpsi2(m,k) = zhpsi(m,k)
               endif
            enddo
         endif

      enddo
      hpsidim = hpsidim+zeig-1

!-----------------------------------------------------------------------
! pointers for single particle function matrices hloch
!-----------------------------------------------------------------------
      zeig=hlochdim+1
      do m=1,nmode
         do k=k1,k2
            if ((kact(k).eq.1) .or. ((kact(k).eq.2).and.(k.lt.ktwin(k))) &
                 .or. ((kact(k).eq.4).and.(k.lt.ktwin(k)))) then
               if (.not. diag(m,k)) then
                  s=kf(k)
                  s1=ki(k)
                  phloch(m,k) = zeig
                  zeig=zeig+dim(m,s)*dim(m,s1)
               else if ((kact(k).eq.2).and.(k.lt.ktwin(k))) then
                  if ( diag(m,k) .and. .not.diag(m,ktwin(k)) ) then
                     s=kf(k)
                     s1=ki(k)
                     phloch(m,k) = zeig
                     zeig=zeig+dim(m,s)*dim(m,s1)
                  endif 
               else
                  phloch(m,k) = 1
               endif
            endif
         enddo
         
         if (nham.eq.1) then
            k=kzahl(1)
            do k4=1,nmulpot(1)
               k=k+1
               if (.not. diag(m,k)) then
                  s=kf(k)
                  phloch(m,k) = zeig
                  zeig=zeig+dim(m,s)*subdim(m)
               else
                  phloch(m,k) = 1
               endif
            enddo
         endif
      enddo
      hlochdim = hlochdim+zeig-1

      do k=1,khtot(1)
         do m=1,nmode
            phloch(m,k) = max(phloch(m,k),1)
         enddo
      enddo
      hlochdim = max(hlochdim,1)

!-----------------------------------------------------------------------
! Needed for gwpcx = 2 in gh_elements.F ??
!-----------------------------------------------------------------------
      do k=1,khtot(1)
         phloch(1,k) = max(phloch(1,k),1)
      enddo
      hlochdim = max(hlochdim,1)

!-----------------------------------------------------------------------
! pointers for single-particle function matrices
!
! If the "proj-h" input option is used or CAPS are present the matrix
! elements <phi|h|phi> are also needed for the uncorrelated Hamiltonian
! terms. Effectively the number of Hamiltonian terms is then
! kzahl(nham)+nstate.
!-----------------------------------------------------------------------
      if (lfunkr .or. lcap) then
         if (kzahl(nham)+nstate .gt. maxkoe) then
            routine = 'zeigprop'
            message = 'Redimension maxkoe in global.inc. Minimum: '
            write(message(44:49),'(i5)') kzahl(nham)+nstate
            call errormsg
         endif
      endif

      zeig = pmatdim+1
      do m = 1,nmode
         do k=k1,k2
            if (.not. diag(m,k)) then
               if (k .eq. kfirst(m,k)) then
                  s=kf(k)
                  s1=ki(k)
                  pmat(m,k) = zeig
                  zeig = zeig+dim(m,s)*dim(m,s1)
               else
                  k4=kfirst(m,k)
                  pmat(m,k)=pmat(m,k4)
               endif
            else
               pmat(m,k)=0
            endif
         enddo

         do k=k2+1,k3
            pmat(:,k) = 0
            if (m .eq. meb) then  ! check muld terms for electronic basis
               pmat(meb,k) = zeig
               zeig = zeig+dim(meb,1)*dim(meb,1)
            endif
         enddo

!     Projection
         if (ldproj) then
            do k=k1,k2
               if ((kact(k).eq.2) .and. (k.lt.ktwin(k))) then
                  k4 = ktwin(k)
                  if (diag(m,k)) then
                     pmat2(m,k) = pmat(m,k4)
                  else
                     if (diag(m,k4)) then
                        pmat2(m,k) = pmat(m,k)
                     else
                        s = kf(k)
                        s1 = ki(k4)
                        pmat2(m,k) = zeig
                        zeig = zeig+dim(m,s)*dim(m,s1)
                     endif
                  endif
               else
                  pmat2(m,k) = pmat(m,k)
               endif
            enddo
!     No projection
         else
            do k=k1,k2
               if (((kact(k).eq.2).or.(kact(k).eq.3)) .and.  &
                    (k.lt.ktwin(k))) then
                  k4 = ktwin(k)
                  if (diag(m,k)) then
                     pmat2(m,k) = pmat(m,k4)
                  else
                     if (diag(m,k4)) then
                        pmat2(m,k) = pmat(m,k)
                     else
                        s = kf(k)
                        s1 = ki(k4)
                        pmat2(m,k) = zeig
                        zeig = zeig+dim(m,s)*dim(m,s1)
                     endif
                  endif
               else
                  pmat2(m,k) = pmat(m,k)
               endif
            enddo   !  k=k1,k2
         endif
            
         k=zham(nham)+kzahl(nham)-1
         do s = 1,nstate
            if( khzahl(m,s,nham) .gt. 0 ) then
               pmat(m,k+s) = zeig
               zeig = zeig+dim(m,s)*dim(m,s)
            else
               pmat(m,k+s) = 1
            endif 
         enddo  !  s = 1,nstate
      enddo     !  m = 1,nmode
      pmatdim = pmatdim+zeig-1

!-----------------------------------------------------------------------
!      gh        -   ghmat(k,m)     -     ghmatdim
!      hunG      -   hunGmat(s)     -     hunGdim      
!-----------------------------------------------------------------------
      zeig = 1
      do n=1,nham
         do s = 1,nstate
            do m = 1,nmode
               if (gwpm(m))then
                  hungmat(m,s,n) = zeig
                  do p1 = 1,npg(m)
                     zeig = zeig+gwpdim(m,s)**2
                  enddo
               endif
            enddo
         enddo
      enddo
      hungdim = zeig-1

      k1=zham(nham)
      k2=zham(nham)+khtot(nham)-1
      k3=zham(nham)+kzahl(nham)-1
      zeig = ghmatdim+1
!      write(*,*)k1,k2,"source/mctdhlib/zeighpsimod.f90"
      do m = 1,nmode
         if(gwpm(m))then

            do k=k1,k2
               if (.not. diag(m,k)) then
                  if (k .eq. kfirst(m,k) .or. k .gt. k3) then

                     s=kf(k)
                     s1=ki(k)
                     ghmat(k,m) = zeig
                     do p1 = 1,npg(m)
                        zeig = zeig+gwpdim(m,s)*gwpdim(m,s1)
                     enddo
                  else

                     k4=kfirst(m,k)
                     ghmat(k,m) = ghmat(k4,m)
                  endif
               else
                  ghmat(k,m) = 1
               endif
            enddo
         endif
      enddo
      ghmatdim = zeig-1

!-----------------------------------------------------------------------
! Pointers for CDVR points (cdvr3)
!-----------------------------------------------------------------------
      if (lcdvr .or. ltdvr) then
         zeig=1
         do s=1,nstate
            do m=1,nmode
               if(idmode(m).eq.0) then
                  c3mat(m,s)=zeig
                  zeig=zeig+dim(m,s)*nspfdof(m)
               else
                  c3mat(m,s)=c3mat(idmode(m),s)
               endif
            enddo
         enddo
         c3matdim=zeig-1
      else
         c3matdim=1
      endif

      return
      end subroutine d2zeighpsi

!#######################################################################
! MAXMATWK
!
! calculates the size of the work array needed in matrizen2
!
! V5.0 mb
!#######################################################################

      subroutine d1maxmatwk(nham)

      implicit none

      integer(long)             :: block1,m,m1,vdim1,s,s1,t,k,k1,k2
      integer(long), intent(in) :: nham

      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
      do s=1,nstate
         do t=1,nstate
            maxblock=max(maxblock,dblock(s,t))
         enddo
      enddo

      do k=k1,k2
         s=kf(k)
         s1=ki(k)
         if (s.ne.s1) then
            do t=1,nstate
               do m=1,nmode
                  if (m.eq.1) then
                     block1=dblock(s1,t)
                     maxblock=max(maxblock,block1)
                     vdim1=ddim(m,s1,t)
                  else
                     block1=vddim(m-1,s,t)*ddim(m-1,s,t)*&
                          nddim(m-1,s1,t)
                     maxblock=max(maxblock,block1)
                     vdim1=vddim(m-1,s,t)*ddim(m,s1,t)
                  endif
                  do m1=m+1,nmode
! before multiplication
                     block1=vdim1*ddim(m1,s1,t)*nddim(m1,s1,t)
                     maxblock=max(maxblock,block1)
! after multiplication
                     block1=vdim1*ddim(m1,s,t)*nddim(m1,s1,t)
                     maxblock=max(maxblock,block1)
                     vdim1=vdim1*ddim(m1,s,t)
                  enddo
               enddo
               m=nmode
               block1=vddim(m,s,t)*ddim(m,s,t)*nddim(m,s1,t)
               maxblock=max(maxblock,block1)
            enddo
         endif
      enddo

      
      do k1=1,nmulpot(1)
         message='Multidimensional Hterms not yet implemented.'
         call errormsg
      enddo
      
      return
      end subroutine d1maxmatwk

!#######################################################################

      end module zeighpsimod

