!!  Copyright (C) 2009 O. Vendrell
!!  MLCORE, Multilayer-MCTDH core modules for quantum dynamics
!!
!!  This file is part of MLCORE.
!!
!!  MLCORE is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  (at your option) any later version.
!!
!!  This program is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU General Public License for more details.
!!
!!  $Id: hterm.f90 293 2010-05-24 15:15:45Z oriol $


module htermmod
  
  use mlbase
  use global
  use mltypes

  private
  public  :: new_hterm,deallocate_hterm,set_hterm,set_htermd,set_htermdv,&
             set_htermv
  
    contains

!------------------------------------------------------------------------------
! Initialize and allocate an hterm
!
! OV, 2008
!------------------------------------------------------------------------------
    function new_hterm(htype,hrange,hrange2) result(h)
        implicit none
        integer,intent(in)       :: htype
        integer,intent(in)       :: hrange
        integer,optional         :: hrange2
        type(hterm_t), pointer   :: h

        allocate(h)

        h%htype   = htype
        h%hrange  = hrange
        h%hrange2 = hrange
        if(present(hrange2)) then
            h%hrange2 = hrange2
        end if

        select case(htype)
            case(1) ! unit operator, nothing to be done
                return
            case(2) ! matrix operator (complex)
                allocate(h%matrix(h%hrange,h%hrange2))
                h%matrix=(0.d0,0.d0)
            case(3) ! local operator (complex)
                allocate(h%vector(hrange))
                h%vector=(0.d0,0.d0)
            case(4) ! matrix (real)
                allocate(h%rmatrix(h%hrange,h%hrange2))
            case(5) ! vector (real)
                allocate(h%rvector(hrange))
            case default
                write(6,*) 'Operator type not defined',htype
                stop
        end select
    end function new_hterm

    function set_hterm(htype,mat,hrange,hrange2) result(h)
        implicit none
        integer,intent(in)       :: htype
        integer,intent(in)       :: hrange
        integer,intent(in)       :: hrange2
        type(hterm_t), pointer   :: h
        complex(cdbl),target     :: mat(hrange,hrange2)

        allocate(h)

        h%htype   = htype
        h%hrange  = hrange
        h%hrange2 = hrange2

        h%matrix => mat

    end function set_hterm

    function set_htermd(htype,mat,hrange,hrange2) result(h)
        implicit none
        integer,intent(in)       :: htype
        integer,intent(in)       :: hrange
        integer,intent(in)       :: hrange2
        type(hterm_t), pointer   :: h
        real(dbl),target         :: mat(hrange,hrange2)

        allocate(h)

        h%htype   = htype
        h%hrange  = hrange
        h%hrange2 = hrange2

        h%rmatrix => mat

    end function set_htermd

    function set_htermv(htype,vec,hrange) result(h)
        implicit none
        integer,intent(in)       :: htype
        integer,intent(in)       :: hrange
        type(hterm_t), pointer   :: h
        complex(cdbl),target     :: vec(hrange)

        allocate(h)

        h%htype   = htype
        h%hrange  = hrange
        h%hrange2 = 1

        h%vector => vec

    end function set_htermv

    function set_htermdv(htype,vec,hrange) result(h)
        implicit none
        integer,intent(in)       :: htype
        integer,intent(in)       :: hrange
        type(hterm_t), pointer   :: h
        real(dbl),target         :: vec(hrange)

        allocate(h)

        h%htype   = htype
        h%hrange  = hrange
        h%hrange2 = 1

        h%rvector => vec

    end function set_htermdv


    subroutine deallocate_hterm(h)
        implicit none
        type(hterm_t), pointer    :: h
        if(associated(h%matrix)) then
            deallocate(h%matrix)
        end if
        if(associated(h%vector)) then
            deallocate(h%vector)
        end if
        if(associated(h%rmatrix)) then
            deallocate(h%rmatrix)
        end if
        if(associated(h%rvector)) then
            deallocate(h%rvector)
        end if
    end subroutine deallocate_hterm

end module htermmod

