module if_grid

   implicit none
   public

contains

   function if_ndof() result(nf)
      ! Return total number of degf.

     use global
     use dvrdatmod

     implicit none

      integer :: nf
      nf=ndof
   end function if_ndof


   function if_nmode() result(nm)
      ! Return total number of modes.

     use global
     use griddatmod

     implicit none

      integer :: nm
      nm=nmode
   end function if_nmode


   function if_spfdof(n,m) result(f)
      ! Return number of degf in n-th position of m-th particle

     use global
     use maxv
     use griddatmod

     implicit none

      integer,    intent(in)   :: n
      integer,    intent(in)   :: m
      integer                  :: f
      f=spfdof(n,m)
    end function if_spfdof


   function if_dofnums(m) result(dofnums)
      ! Return DOF numbers of mode m.

     use global
     use maxv
     use griddatmod
     
     implicit none

      integer,intent(in)  :: m
      integer,allocatable :: dofnums(:)
      integer             :: n
      allocate(dofnums(nspfdof(m)))
      do n=1,nspfdof(m)
         dofnums(n) = spfdof(n,m)
      enddo
   end function if_dofnums


   function if_gdim(f) result(gdm)
      ! Return number of grid points for DOF f.

     use global
     use maxv
     use dvrdatmod

     implicit none

      integer,intent(in) :: f
      integer            :: gdm
      gdm=gdim(f)
   end function if_gdim


   function if_subdim(m) result(sdm)
      ! Return number of grid points for mode m.

     use global
     use maxv
     use griddatmod

     implicit none

      integer,intent(in) :: m
      integer            :: sdm
      sdm=subdim(m)
   end function if_subdim


   subroutine if_modelabel(f,label)
      ! Return modelabel for f-th degree of freedom

     use global
     use maxv
     use dvrdatmod

     implicit none

      integer, intent(in)            :: f
      character(len=c1), intent(out) :: label
      label = modelabel(f)
   end subroutine if_modelabel

   function if_modestring(m) result(label)
       ! Return textual representation for m-th mode

      use global
      use dvrdatmod
      use griddatmod

      implicit none

      integer,intent(in) :: m
      character(len=c4)  :: label
      integer            :: n,f

      label = ""
      do n=1,nspfdof(m)
         f = spfdof(n,m)
         if (n==1) then
            label = trim(label)//trim(modelabel(f))
         else
            label = trim(label)//","//trim(modelabel(f))
         endif
      enddo
   end function if_modestring

end module if_grid

