!#############################################################################
!                 --- Heidelberg MCTDH ---                                   #
!                     File: interface                                        #
!                     Author: Oriol Vendrell (oriol.vendrell@gmail.com)      #
!                     Year  : 2008                                           #
!#############################################################################
!
!-----------------------------------------------------------------------------
! This file is the contact point between the F77 part of MCTDH code and
! the ML-MCTDH code.
!   In particular this means:
!     - All calls FROM the ml-code TO outside are from procedures in this file.
!-----------------------------------------------------------------------------


!       ML-MCTDH --> MCTDH   interface
!-----------------------------------------------------------------------------


      function intfc_getcmf() result(cmftyp)

        use global
        use maxv
        use runpropmod

        implicit none

        integer :: cmftyp
        if (lconst) then
           if (lmlcmfsplit) then
              cmftyp = 2
           else
              cmftyp = 1
           endif
        else
           cmftyp = 0
        endif
      end function intfc_getcmf


      subroutine intfc_getnamedir(namedir)

        use global

        implicit none

         character*(c5) namedir
         namedir = name
      end subroutine intfc_getnamedir


      subroutine intfc_integrator_settings(no,llog,&
           inttyp1,order1,acc1,initstep1,limportho1,lrhonrm1,lcont1)

        use global
        use maxv
        use runpropmod
        use datenmod
        use operdef

        implicit none

         integer :: no ! node number
         logical :: llog,limportho1,lrhonrm1,lcont1
         integer :: inttyp1,order1
         real*8  :: acc1,initstep1
         integer :: sp,i,j,nno,nos(2*maxdim)
         logical,save :: initialized = .false.

!         integer,save :: node2spec(2*maxdim) 
! node2spec moved to runpropmod.f90 as now cannot be saved due to
! maxdim becoming a dummy argument (i.e., is no longer a parameter).
! Is now allocatable and is allocated in after maxdim is determined
! in alloc_runpropmod.


!        common parameters (for now)
         lcont1     = lcont
         llog       = lintstep
         limportho1 = liork
         lrhonrm1   = lrhonrm
!        map the node to the mode-specifications from the INTEGRATOR-SECTION
         if (.not.initialized) then
            do j=1,2*maxdim
               node2spec(j) = 0
            enddo
            do i=1,mnspec
               call mspec_parse(mspec(i),nno,nos)
               do j=1,nno
                  node2spec(nos(j)) = i
               enddo
            enddo
            initialized = .true.
         endif
         sp = node2spec(no)
         if (sp.gt.0) then
!           take parameters from that mode-specification
            inttyp1   = minttyp(sp)
            order1    = mintord(sp)
            acc1      = mintacc(sp)
            initstep1 = mintdt(sp)
         else
!           take parameters from xx/SPF definitions
!           these are also set if xx/ALL or xx/DEF was given in the input
            inttyp1   = inttypephi
            order1    = intorderphi
            acc1      = accphi
            initstep1 = dtphi
         endif
!        RK5/8 are treated as the same type in ML
         if (inttyp1.eq.5) inttyp1=4
!        switch SIL to CSIL if CAP is used
         if (inttyp1.eq.3 .and. lcap) inttyp1=7
      end subroutine intfc_integrator_settings



      subroutine mspec_parse(spec,nno,nos)

        use global
        use logdat

        implicit none

         character(len=*) :: spec
         integer          :: nno,nos(*)
         integer          :: k,k1,k2,ierr1
         character(len=32):: nstr

         nno = 0
         k   = 1
 10      continue
         if (spec(k:k).eq.' ') return
         if (spec(k:k).eq.'@') then
            k1 = k+1
            k2 = scan(spec(k1:),'@ ')
            if (k2.eq.0) then
               nstr = spec(k1:)
               k = len(spec)+1
            elseif (k2.eq.1) then
               goto 99
            else
               nstr = spec(k1:k1+k2-2)
               k = k1+k2-1
            endif
            nno = nno+1
            read(nstr,*,iostat=ierr1) nos(nno)
            if (ierr1.ne.0) goto 99
         else
            goto 99
         endif
         if (k.le.len(spec)) goto 10
         return

 99      routine = 'mspec_parse'
         message = 'error parsing mode specification: '//trim(spec)
         call warnmsg
         return

      end subroutine mspec_parse

