module dmatmod

   use global
   use wftree
   use htermmod
   use linear

   use mltypes

   implicit none

   public :: alloc_dmat_wftree, &
             compute_just_dmat_tree

   private

contains

   subroutine alloc_dmat_wftree(t,full)
   ! allocate memory for just the density matrices of the tree
      implicit none
      type(wfnode_t),intent(inout) :: t
      logical,intent(in),optional  :: full
      type(wfnode_t),pointer       :: no
      integer                      :: i
      logical                      :: lfull
      lfull = .false.
      if (present(full)) lfull=full
      do i=1,t%tot_nodes
         no => t%swap_up(i)%p
         call alloc_dmat_wfnode(no,lfull)
      enddo
   end subroutine alloc_dmat_wftree



   subroutine alloc_dmat_wfnode(no,full)
   ! allocate memory for just the density matrices of a node
      implicit none
      type(wfnode_t),intent(inout) :: no
      logical,intent(in)           :: full
      integer                      :: f,alen
      if (no%isprimitive) return
      allocate(no%dmat(no%nmodes))
      if (full) then
         allocate(no%invdmat(no%nmodes))
         allocate(no%pop(no%nmodes))
         allocate(no%regpop(no%nmodes))
         allocate(no%trfmat(no%nmodes))
      endif
      do f=1,no%nmodes
         alen = no%ndim(f)
         no%dmat(f)%p => new_hterm(2,alen)
         if (full) then
            no%invdmat(f)%p => new_hterm(2,alen)
            no%pop(f)%p     => new_hterm(5,alen)
            no%regpop(f)%p  => new_hterm(5,alen)
            no%trfmat(f)%p  => new_hterm(2,alen)
         endif
      enddo
   end subroutine alloc_dmat_wfnode



   subroutine compute_just_dmat_tree(t,psi)
   ! computes just the density matrices of the tree
   ! (no regularisations or inversions are done)
      implicit none
      type(wfnode_t),intent(inout) :: t
      complex(cdbl),intent(in)     :: psi(:)
      type(wfnode_t),pointer       :: no
      integer                      :: i
      do i=1,t%tot_nodes
         no => t%swap_down(i)%p
         call compute_just_dmat_node(no,psi)
      enddo
   end subroutine compute_just_dmat_tree



   subroutine compute_just_dmat_node(no,psi)
   ! computes just the density matrices of a node
      implicit none
      type(wfnode_t),intent(inout) :: no
      complex(cdbl),intent(in)     :: psi(:)
      integer                      :: f,ns,a,b,z0,zf
      complex(cdbl),pointer        :: dm(:,:),dmpar(:,:)
      complex(cdbl),allocatable    :: work(:,:),phitmp(:,:)

      if (no%isprimitive) return

      if (no%istop) then ! top node
         z0 = no%z(1,1)
         zf = no%z(1,2)
!$OMP parallel default(shared) private(f,dm)
!$OMP do schedule(runtime)
         do f=1,no%nmodes
            dm => no%dmat(f)%p%matrix
            call compute_dmat(dm,f,no%ndim,psi(z0:zf))
         enddo
!$OMP end do
!$OMP end parallel

      else ! inner node
         ns = no%nsubcoord
         dmpar => no%parent%dmat(ns)%p%matrix
         allocate(phitmp(product(no%ndim),no%navecs))
         phitmp = CZERO
         do b=1,no%navecs
            z0 = no%z(b,1)
            zf = no%z(b,2)
            do a=1,no%navecs
               phitmp(:,a) = phitmp(:,a) + dmpar(a,b)*psi(z0:zf)
            enddo
         enddo
!$OMP parallel default(shared) private(f,dm,work,a,z0,zf)
!$OMP do schedule(runtime)
         do f=1,no%nmodes
            dm => no%dmat(f)%p%matrix
            dm = CZERO
            allocate(work(no%ndim(f),no%ndim(f)))
            do a=1,no%navecs
               z0 = no%z(a,1)
               zf = no%z(a,2)
               call compute_dmat(work,f,no%ndim,psi(z0:zf),phitmp(:,a))
               dm = dm + work
            enddo
            deallocate(work)
         enddo
!$OMP end do
!$OMP end parallel
         deallocate(phitmp)
      endif

   end subroutine compute_just_dmat_node

end module dmatmod
