module mltensor

!!  Copyright (C) 2009 O. Vendrell
!!  MLCORE, Multilayer-MCTDH core modules for quantum dynamics
!!
!!  This file is part of MLCORE.
!!
!!  MLCORE is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  (at your option) any later version.
!!
!!  This program is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU General Public License for more details.
!!
!!  $Id: tensor.f90 503 2013-07-10 09:46:05Z frank $

use mtlib, only:qtxxzz,qttxzz,dtxxzzo,qtxxdz,dtxxdzo,qttxdz,qtcxzz,dtcxzzo,&
                qtaxzz
use rmlib, only:rmqxxxzz
use decimal

    private
    public  :: matrix_tensor_compl, &
               matrix_tensor_compl_tp, &
               matrix_tensor_compl_ad, &
               vector_tensor_compl, &
               dmatrix_tensor_compl, &
               dvector_tensor_compl, &
               cmatrix_tensor_compl, &
               cvector_tensor_compl, &
               trmatrix_tensor_compl, &
               dtrmatrix_tensor_compl, &
               recmatrix_tensor_compl, &
               dmat_tensor_tensor_compl, &
               dens_tensor_real

contains

subroutine matrix_tensor_compl(mat,tens,dims)

    implicit none
    integer,intent(in)            :: dims(3)
    complex(8),intent(in)         :: mat(dims(2),dims(2))
    complex(8),intent(inout)      :: tens(dims(1),dims(2),dims(3))
    complex(8)                    :: tmp(dims(1),dims(2),dims(3))

    call qtxxzz(mat,tens,tmp,dims(1),dims(2),dims(3))
    tens=tmp

end subroutine matrix_tensor_compl


subroutine matrix_tensor_compl_tp(mat,tens,dims)
    implicit none
    integer,intent(in)            :: dims(3)
    complex(8),intent(in)         :: mat(dims(2),dims(2))
    complex(8),intent(inout)      :: tens(dims(1),dims(2),dims(3))
    complex(8)                    :: tmp(dims(1),dims(2),dims(3))

    call qttxzz(mat,tens,tmp,dims(1),dims(2),dims(3))
    tens=tmp

end subroutine matrix_tensor_compl_tp

subroutine matrix_tensor_compl_ad(mat,tens,dims)
    implicit none
    integer,intent(in)            :: dims(3)
    complex(8),intent(in)         :: mat(dims(2),dims(2))
    complex(8),intent(inout)      :: tens(dims(1),dims(2),dims(3))
    complex(8)                    :: tmp(dims(1),dims(2),dims(3))

    call qtaxzz(mat,tens,tmp,dims(1),dims(2),dims(3))
    tens=tmp

end subroutine matrix_tensor_compl_ad


subroutine vector_tensor_compl(vec,tens,dims)
    implicit none
    integer,intent(in)            :: dims(3)
    complex(8),intent(in)         :: vec(dims(2))
    complex(8),intent(inout)      :: tens(dims(1),dims(2),dims(3))

    call dtxxzzo(vec,tens,dims(1),dims(2),dims(3))

end subroutine vector_tensor_compl


subroutine dmatrix_tensor_compl(mat,tens,dims)
    implicit none
    integer,intent(in)            :: dims(3)
    real(dbl),intent(in)          :: mat(dims(2),dims(2))
    complex(8),intent(inout)      :: tens(dims(1),dims(2),dims(3))
    complex(8)                    :: tmp(dims(1),dims(2),dims(3))

    call qtxxdz(mat,tens,tmp,dims(1),dims(2),dims(3))
    tens=tmp

end subroutine dmatrix_tensor_compl


subroutine dvector_tensor_compl(vec,tens,dims)
    implicit none
    integer,intent(in)            :: dims(3)
    real(dbl),intent(in)          :: vec(dims(2))
    complex(8),intent(inout)      :: tens(dims(1),dims(2),dims(3))

    call dtxxdzo(vec,tens,dims(1),dims(2),dims(3))

end subroutine dvector_tensor_compl

subroutine trmatrix_tensor_compl(mat,tens,dims)
    implicit none
    integer,intent(in)            :: dims(3)
    complex(dbl),intent(in)       :: mat(dims(2),dims(2))
    complex(8),intent(inout)      :: tens(dims(1),dims(2),dims(3))
    complex(8)                    :: tmp(dims(1),dims(2),dims(3))

    call qttxzz(mat,tens,tmp,dims(1),dims(2),dims(3))
    tens=tmp

end subroutine trmatrix_tensor_compl

subroutine dtrmatrix_tensor_compl(mat,tens,dims)
    implicit none
    integer,intent(in)            :: dims(3)
    real(dbl),intent(in)          :: mat(dims(2),dims(2))
    complex(8),intent(inout)      :: tens(dims(1),dims(2),dims(3))
    complex(8)                    :: tmp(dims(1),dims(2),dims(3))

    call qttxdz(mat,tens,tmp,dims(1),dims(2),dims(3))
    tens=tmp

end subroutine dtrmatrix_tensor_compl

subroutine cmatrix_tensor_compl(mat,tens,dims)
    implicit none
    integer,intent(in)            :: dims(3)
    complex(dbl),intent(in)       :: mat(dims(2),dims(2))
    complex(8),intent(inout)      :: tens(dims(1),dims(2),dims(3))
    complex(8)                    :: tmp(dims(1),dims(2),dims(3))

    call qtcxzz(mat,tens,tmp,dims(1),dims(2),dims(3))
    tens=tmp

end subroutine cmatrix_tensor_compl

subroutine cvector_tensor_compl(vec,tens,dims)
    implicit none
    integer,intent(in)            :: dims(3)
    complex(8),intent(in)         :: vec(dims(2))
    complex(8),intent(inout)      :: tens(dims(1),dims(2),dims(3))

    call dtcxzzo(vec,tens,dims(1),dims(2),dims(3))

end subroutine cvector_tensor_compl


subroutine recmatrix_tensor_compl(mat,tens1,tens2,dim1,rdim)
    ! similar to matrix_tensor_compl, but mat can be rectangular, thus
    ! changing the shape of the output tensor
    implicit none
    integer,intent(in)            :: dim1(3)
    integer,intent(in)            :: rdim
    complex(8),intent(in)         :: mat(rdim,dim1(2))
    complex(8),intent(in)         :: tens1(dim1(1),dim1(2),dim1(3))
    complex(8),intent(out)        :: tens2(dim1(1),rdim,dim1(3))
    integer                       :: i,k
    do k=1,dim1(3)
        do i=1,dim1(1)
            tens2(i,:,k) = matmul(mat,tens1(i,:,k))
        end do
    end do
end subroutine recmatrix_tensor_compl


subroutine dmat_tensor_tensor_compl(dmat,tensor1,tensor2,dims)
    implicit none
    integer                       :: dims(3)
    complex(8),intent(out)        :: dmat(dims(2),dims(2))
    complex(8),intent(in)         :: tensor1(dims(1),dims(2),dims(3))
    complex(8),intent(in)         :: tensor2(dims(1),dims(2),dims(3))
    integer                       :: l,m,i,j

    call rmqxxxzz(tensor1,tensor2,dmat,dims(1),dims(2),dims(3))
    dmat=(0.d0,0.d0)
    do j=1,dims(3)
        do l=1,dims(2)
            do m=1,dims(2)
                do i=1,dims(1)
                    dmat(m,l) = dmat(m,l) + dconjg(tensor1(i,m,j))*tensor2(i,l,j)
                end do
            end do
        end do
    end do
end subroutine dmat_tensor_tensor_compl


subroutine dens_tensor_real(d1d,dens,dims)
    implicit none
    integer               :: dims(3)
    real(8),intent(out)   :: d1d(dims(2))
    real(8),intent(in)    :: dens(dims(1),dims(2),dims(3))
    integer               :: i,j,k

    d1d = 0.d0
    do j=1,dims(3)
        do k=1,dims(2)
            do i=1,dims(1)
                d1d(k) = d1d(k) + dens(i,k,j)
            end do
        end do
    end do
end subroutine dens_tensor_real

end module mltensor
