module wfprop

! ----------------------------------------------------------------------
! WfNode-related code that is only needed/used for propagations.
!
! This is kept in a separate module, because otherwise some programs
! that do need access to wftree_m might develop dependencies on propwf
! and so forth.
!
! FO 11/2012
! ----------------------------------------------------------------------

   use global
   use globmemmod
   use griddatmod, only: gwpm
   use wftree
   use wftree1
   use integrator
   use logging
   use griddatmod
   use mltypes
   use mlbase
   use htermmod
   use runpropmod
   use channels, only: itmpin

   implicit none


contains


   subroutine alloc_prop_wftree(t,lfields)
      implicit none
      type(wfnode_t),intent(inout) :: t
      logical, intent(in)          :: lfields

      type(wfnode_t),pointer :: no
      integer                :: i,a,b,plen,m,zeig
      integer                :: intfc_getcmf

      t%cmftyp = intfc_getcmf()  ! get integrator type (vmf/cmf)
      if (t%cmftyp > 0) then
         do i=1,t%tot_nodes
            no => t%swap_up(i)%p
            call set_wfnode_htmtype(no)
            if (no%isprimitive .and. lfields) then
               m=no%m
               if (gwpm(m)) cycle
            ! space for sum over mean-field matrices times potential operators
               allocate(no%mfsum(no%navecs,no%navecs))
            !   plen = product(no%ndim)
               if (dentype .eq. 1) then
                  plen = supgdim(m)
               else
                  plen = subdim(m)
               endif
               zeig=zmfsum(m,1,1)
               do b=1,no%navecs
                  do a=1,no%navecs
               !      no%mfsum(a,b)%p => new_hterm(3,plen)
                     no%mfsum(a,b)%p => set_htermv(3,mfsum(zeig),plen)
                     zeig = zeig+plen
                  enddo
               enddo
            endif
         enddo
      endif

   end subroutine alloc_prop_wftree



   subroutine init_integrators(t)
      implicit none
      type(wfnode_t),intent(inout) :: t

      integer                    :: inttyp,order,i,lun,ierr,ierrno
      real(dbl)                  :: acc,initstep
      logical                    :: llog,limportho,lpostproc,lrhonrm,lcont
      integer                    :: ipostproc
      type(wfnode_t),pointer     :: no
      complex(cdbl)              :: prefac
      character(len=240)         :: namedir,logname,msg,idesc,extcmd
      logical                    :: lopen,lexist

      ! fetch parameters from F77-land
      call intfc_getnamedir(namedir)
      do i=1,t%tot_nodes
         no => t%swap_down(i)%p
         call intfc_integrator_settings(no%num,llog,inttyp,order,acc,initstep,&
              limportho,lrhonrm,lcont)
         if (llog) then
            lexist=.false.
            write(logname,'(2a)') trim(namedir),'/steps'
            open(itmpin,file=trim(logname)//'/scratch',err=666)
            close(itmpin,status='delete')
            lexist=.true.
  666       continue

            if (.not. lexist) then
               extcmd='mkdir '//trim(logname)
               call excmd3(ierr,ierrno,extcmd)
               if(ierr.ne.0)then
                  write(*,*)'Error making directory: ',trim(logname)
                  write(*,*)'ierrno = ',ierrno
               endif
            endif

            ! open steps file for each node
            write(logname,'(2a,i0,a)') trim(namedir),'/steps/n',no%num,'.steps'
            inquire(file=trim(logname),opened=lopen) ! file may be already open
            if (lopen) then
               inquire(file=trim(logname),number=lun) 
            else
               call open_log(lun,trim(logname))
            endif

            if(lcont) then    ! continuation run
   10          continue
               read(lun,'(a)',end=20)
               go to 10
   20          backspace(lun)
               write(lun,'(a)') &
  '#----------------------------- CONTINUATION ------------------------------'
            endif
         else
            lun = 0
         end if
         ! choose post-processor
         ! - improved orthonormality applies to all nodes except the top node;
         !   using it there would attempt to renormalize the wavefunction
         ! - in case of relaxation, the top node A-vector has to be renormalized
         !   when (C)SIL is used
         if (no%istop) then
            if (no%isrelax .and. (inttyp==inttyp_sil .or. inttyp==inttyp_csil)) then
               lpostproc = .true.
               ipostproc = 2   ! cf int_postprocess
               prefac = -CONE  ! for SIL/CSIL relaxation
            else
               lpostproc = .false.
               ipostproc = 0   ! dummy
               prefac = -IONE  ! for SIL/CSIL propaation
            endif
         else
            lpostproc = limportho
            ipostproc = 1
         endif

         ! make new integrator object and set parameters
         no%integrator => new_integrator(inttyp,llog,lun,lpostproc,ipostproc,&
                                         lrhonrm)
         select case (inttyp)
            case (inttyp_rk)
               call int_rk_init(no%integrator%rk,order,acc,initstep)
            case (inttyp_sil)
               call int_sil_init(no%integrator%sil,order,acc,prefac)
            case (inttyp_csil)
               call int_csil_init(no%integrator%csil,order,acc,prefac)
            case default
               call stop_code('node_m', 'init_integrators', &
                    'unknown/unimplemented integrator type')
         end select
         if (llog) then
            call show_integrator(no%integrator, idesc)
            write (msg,'(a,i0,2a)') '# Node ',no%num,': ',trim(idesc)
            call write_log(lun,msg)
         end if
      end do
   end subroutine init_integrators

end module wfprop
