!=======================================================================
module genmlham
!=======================================================================

use mlbase
use mltypes
use vtree
use optree
use mlham
use wftree
use genmlop
use ilist
use llist_int
use tensors
use if_grid
use op1lib, only: cpvxd
use operdef
implicit none
private

public :: mlpf_read, mlpf_check_dofs, mlpf_modeinfo, mlpf_to_mlop, &
          mlop_get_mlpf, mlop_setk, mlop_enqueue, mlop_finalize

contains !==============================================================


!-----------------------------------------------------------------------
function mlop_get_mlpf(imlop) result(mlpf)
!-----------------------------------------------------------------------
   integer,intent(in)    :: imlop
   type(vtree_t),pointer :: mlpf
   integer               :: nr
   nr = mmlops(imlop)%mlpfnr
   mlpf => mlpfs(nr)%vtree
end function mlop_get_mlpf


!-----------------------------------------------------------------------
subroutine mlpf_read(lun,nr,fmap,gdim)
!-----------------------------------------------------------------------
   integer,intent(in) :: lun     ! channel to read from
   integer,intent(in) :: nr      ! number of the MLPF (assigned in matchlab)
   integer,intent(in) :: fmap(:) ! mapping between MLPF and system DOFs (established by rddvrdef)
   integer,intent(in) :: gdim(:) ! number of gridpoints for MLPF DOFs
   if (nr > maxnmlpf) &
      call stop_code("mlpf_oper_m", "mlpf_read", "maxnmlpf too small")
   if (associated(mlpfs(nr)%vtree)) then
!      write (*,'(a,i0)') "WARNING: attempted multiple reads of MLPF #",nr
      return
   endif
   ! Store DOF mapping.
   mlpfs(nr)%fmap => new_ilist(fmap)
   ! Read definition of tree structure.
   mlpfs(nr)%vtree => load_vtree_def(lun)
   ! Set size of leaves.
   call init_vleaves(mlpfs(nr)%vtree, gdim)
   ! Read MLPF data.
   call load_vtree_data(mlpfs(nr)%vtree, lun)
end subroutine mlpf_read


!-----------------------------------------------------------------------
function mlpf_check_dofs(nr,dofs) result(lmatch)
!-----------------------------------------------------------------------
   integer,intent(in)    :: nr      ! the MLPF number
   integer,intent(in)    :: dofs(:) ! list of system DOFs that should be in the MLPF
   logical               :: lmatch  ! true if the MLPF DOFs match the system DOFs
   type(ilist_t),pointer :: sysdofs
   sysdofs => new_ilist(dofs)
   lmatch = ilist_equals(sysdofs, mlpfs(nr)%fmap)
   call dispose_ilist(sysdofs)
end function mlpf_check_dofs


!-----------------------------------------------------------------------
subroutine mlpf_modeinfo(imlop,m,l1,af,nf)
!-----------------------------------------------------------------------
   integer,intent(in)     :: imlop,m
   integer,intent(out)    :: l1,af,nf
   type(optree_t),pointer :: mlop
   type(opnode_t),pointer :: opleaf
   integer                :: nr
   type(vnode_t),pointer  :: vleaf
   integer,pointer        :: fmap(:)
   integer                :: l
   l1 = 0
   af = 0
   nf = 0
   mlop => mmlops(imlop)%mlop
   do l=1,mlop%numleaves
      opleaf => mlop%leaves(l)%p
      if (opleaf%m == m) then
         l1 = l
         exit
      endif
   enddo
   if (l1==0) return
   nr = mmlops(imlop)%mlpfnr
   fmap => mlpfs(nr)%fmap%i
   vleaf => mlpfs(nr)%vtree%leaves(l1)%p
   nf = vleaf%nmodes
   af = fmap(vleaf%dofnums(1))
end subroutine mlpf_modeinfo


!-----------------------------------------------------------------------
subroutine mlop_setk(k,imlop,ileaf,ibasis)
!-----------------------------------------------------------------------
   integer,intent(in)     :: k,imlop,ileaf,ibasis
   type(optree_t),pointer :: mlop
   type(opnode_t),pointer :: leaf
   mlop => mmlops(imlop)%mlop
   leaf => mlop%leaves(ileaf)%p
   if (ibasis<0 .or. ibasis>leaf%nops) then
      write (*,'(a,i0)') 'k           = ',k
      write (*,'(a,i0)') 'MLOp number = ',imlop
      write (*,'(a,i0)') 'leaf number = ',ileaf
      write (*,'(a,i0)') 'basis index = ',ibasis
      write (*,'(a,i0)') 'leaf nops   = ',leaf%nops
      call stop_code('mlpf_oper_m','mlop_setk','basis index out of range')
   endif
   if (leaf%ks(ibasis) /= 0) then
      write (*,'(a,i0)') 'k           = ',k
      write (*,'(a,i0)') 'MLOp number = ',imlop
      write (*,'(a,i0)') 'leaf number = ',ileaf
      write (*,'(a,i0)') 'basis index = ',ibasis
      call stop_code('mlpf_oper_m','mlop_setk','kterm already assigned')
   endif
   leaf%ks(ibasis) = k
end subroutine mlop_setk


!-----------------------------------------------------------------------
subroutine next_mlop
!-----------------------------------------------------------------------
   ! Check that we have space for another MLOp.
   nmlop = nmlop+1
   if (nmlop > maxnmlop) &
      call stop_code("mlpf_oper_m", "next_mlop", "maxnmlop is too small")
end subroutine next_mlop


!-----------------------------------------------------------------------
subroutine mlpf_to_mlop(nr,modendof,modedofs,nkterm,imlop)
!-----------------------------------------------------------------------
   integer,intent(in)          :: nr            ! the MLPF number
   integer,intent(in)          :: modendof(:)   ! cf. nspfdof
   integer,intent(in)          :: modedofs(:,:) ! cf. spfdof
   integer,intent(inout)       :: nkterm(:)     ! number of kterms needed for each system DOF
   integer,intent(inout)       :: imlop(:)      ! the MLOp number for each system DOF
   type(vtree_t),pointer       :: mlpf
   type(optree_t),pointer      :: mlop
   integer,pointer             :: fmap(:)
   type(vnode_t),pointer       :: vnode
   integer                     :: nmode,l,ff,f0,f1,m0,m,r0,r,r1
   type(opnode_tp),allocatable :: opnodes(:),children(:)
   type(opnode_t),pointer      :: opnode
   integer,allocatable         :: modeleaf(:)

   mlpf => mlpfs(nr)%vtree
   fmap => mlpfs(nr)%fmap%i
   nmode = size(modendof)

   ! Remember which MLPF this MLOp comes from.
   call next_mlop
   mmlops(nmlop)%mlpfnr = nr

   ! Go through the MLPF tree in post-order.
   allocate(opnodes(mlpf%numnodes))
   do l=1,mlpf%numnodes
      vnode => mlpf%postorder(l)%p

      if (vnode%isleaf) then
         ! Check MLPF leaf for consistency with system mode.
         ! First, find the system mode for this MLPF leaf.
         ff = vnode%dofnums(1) ! MLPF DOF
         f0 = fmap(ff)         ! system DOF
         m0 = 0
         r0 = 0
         modeloop: do m=1,nmode
            do r=1,modendof(m)
               if (modedofs(r,m) == f0) then
                  r0 = r
                  m0 = m
                  exit modeloop
               endif
            enddo
         enddo modeloop
         if (m0==0) then
            write (*,'(a,i0)') "MLPF node  = ",vnode%num
            write (*,'(a,i0)') "MLPF DOF   = ",ff
            write (*,'(a,i0)') "System DOF = ",f0
            call stop_code("mlpf_oper_m", "mlpf_to_mlop", "MLPF DOF not found among System DOFs")
         endif
         ! Check that the remaining DOFs in this MLPF leaf appear in
         ! the same order (modulo mapping) as in the system mode.
         do r=2,vnode%nmodes
            ff = vnode%dofnums(r)  ! MLPF DOF
            f0 = fmap(ff)          ! system DOF (mapped)
            r1 = min(r0+r-1,modendof(m0))
            f1 = modedofs(r1,m0)   ! system DOF (in system mode)
            if (f0 /= f1) then
               write (*,'(a,i0)') "MLPF node   = ",vnode%num
               write (*,'(a,i0)') "System mode = ",m0
               write (*,'(a,99i4)') "MLPF DOFs   = ", (vnode%dofnums(r1), r1=1,vnode%nmodes)
               write (*,'(a,99i4)') " -mapped-   = ", (fmap(vnode%dofnums(r1)), r1=1,vnode%nmodes)
               write (*,'(a,99i4)') "mode DOFs   = ", (modedofs(r0+r1-1,m0), r1=1,min(vnode%nmodes,modendof(m0)-r0+1))
               call stop_code("mlpf_oper_m", "mlpf_to_mlop", "MLPF mode doesn't match System mode")
            endif
         enddo
         ! OK.
         ! Create MLOp leaf with this mode.
         opnode => make_opleaf(m0)
         opnodes(l)%p => opnode

         ! Set up preliminary kterms.
         ! The final kterm numbering is only known after genoper is finished.
         opnode%nops = vnode%nbasis
         allocate(opnode%ks(vnode%nbasis))
         opnode%ks = 0
         ! Return number of kterms needed for each system DOF in this mode.
         do r=1,modendof(m0)
            f1 = modedofs(r,m0)
            nkterm(f1) = vnode%nbasis
         enddo
         ! For each system DOF in the MLPF leaf, store the MLOp number,
         do r=1,vnode%nmodes
            ff = vnode%dofnums(r)
            f0 = fmap(ff)
            imlop(f0) = nmlop
         enddo

      else
         ! Collect pointers to children. The children have already been
         ! created because we are walking the tree in post-order.
         allocate(children(vnode%nmodes))
         do r=1,vnode%nmodes
            children(r)%p => opnodes(vnode%modes(r)%p%postnum)%p
         enddo
         ! Create MLOp node with these children.
         opnode => make_opnode(children)
         opnodes(l)%p => opnode
         deallocate(children)

         ! Set up the transfer tensor.
         opnode%nops = vnode%nbasis
         opnode%tt%ndim = opnode%nmodes + 1
         allocate(opnode%tt%dims(opnode%tt%ndim))
         do r=1,opnode%nmodes
            if (opnode%subops(r)%p%nops /= vnode%ndim(r)) then
               write (*,'(a,i0)')      "     MLOp node = ",vnode%num
               write (*,'(a,i0,a,i0)') "subops(",r,")%nops = ",opnode%subops(r)%p%nops
               write (*,'(a,i0,a,i0)') " vnode%ndim(",r,") = ",vnode%ndim(r)
               call stop_code("mlpf_oper_m", "mlpf_to_mlop", "internal inconsistency detected")
            endif
            opnode%tt%dims(r) = vnode%ndim(r)
         enddo
         opnode%tt%dims(opnode%tt%ndim) = opnode%nops
         opnode%tt%vlen = product(opnode%tt%dims)
         allocate(opnode%tt%vr(opnode%tt%vlen))
         call cpvxd(vnode%basis,opnode%tt%vr,opnode%tt%vlen)

      endif
   enddo

   ! All MLOp nodes have been created. Make an MLOp tree.
   mlop => make_optree(opnodes(mlpf%numnodes)%p)
   mmlops(nmlop)%mlop => mlop
   deallocate(opnodes)

   ! Check that each leaf operates on a different mode.
   allocate(modeleaf(nmode))
   modeleaf = 0
   do l=1,mlop%numleaves
      opnode => mlop%leaves(l)%p
      m0 = opnode%m
      if (m0 <= 0 .or. m0 > nmode) then
         write (*,'(a,i0)') "MLOp node = ",opnode%num
         write (*,'(a,i0)') "     mode = ",m0
         call stop_code("mlpf_oper_m", "mlpf_to_mlop", "invalid system mode for MLOp leaf")
      endif
      if (modeleaf(m0) /= 0) then
         write (*,'(a,i0)') " MLOp node = ",opnode%num
         write (*,'(a,i0)') "      mode = ",m0
         write (*,'(a,i0)') "other node = ",modeleaf(m0)
         call stop_code("mlpf_oper_m", "mlpf_to_mlop", "Found MLOp leaves operating on same system mode")
      endif
      modeleaf(m0) = opnode%num
   enddo
   deallocate(modeleaf)

end subroutine mlpf_to_mlop

!-----------------------------------------------------------------------
function mlop_enqueue(imlops,nham) result(totmlop)
!-----------------------------------------------------------------------
   integer,intent(in)        :: imlops(:) ! for each DOF, the number of the MLOp that applies to it
   integer,intent(in)        :: nham      ! the Hamiltonian number (cf. hamnum)
   integer                   :: totmlop   ! the number of the resulting total MLOp
   type(mlham_t),pointer     :: mlham
   type(llist_int_t),pointer :: ifacq
   integer                   :: i
   mlham => mlham_get_or_create(nham)
   ! Get next MLOp number.
   call next_mlop
   ! Turn the passed-in list of MLOp numbers into a queue (ignoring duplicates).
   allocate(ifacq)
   do i=1,size(imlops)
      if (imlops(i)==0) cycle
      call llist_add(ifacq,imlops(i))
   enddo
   ! Store this todo-list in the new MLOp.
   mmlops(nmlop)%ifacq => ifacq
   ! Store a reference to the new MLOp in the Hamiltonian.
   call llist_push(mlham%immlopq, nmlop)
   mlham%nmlop = mlham%nmlop + 1
   ! Return the new MLOp number.
   totmlop = nmlop
end function mlop_enqueue


!-----------------------------------------------------------------------
subroutine mlop_finalize(nham,mlwf)
!-----------------------------------------------------------------------
! Finalize all MLOps for the Hamiltonian _nham_, using the tree
! structure of the ML-wavefunction _mlwf_.
!-----------------------------------------------------------------------
   integer,intent(in)        :: nham
   type(wfnode_t),intent(in) :: mlwf
   type(mlham_t),pointer     :: mlham
   integer,pointer           :: imlops(:)
   type(llist_int_t),pointer :: ifacq
   integer                   :: q,imlop,k
   ! Check if there are any MLOps for this Hamiltonian
   mlham => mlham_get(nham)
   if (.not.associated(mlham)) return
   ! Prepare final form of ML-Hamiltonian.
   allocate(mlham%mlops(mlham%nmlop))
   allocate(mlham%kkoes(mlham%nmlop))
   ! Get the list of total MLOps that have to be created for this Hamiltonian.
   imlops => llist_to_array(mlham%immlopq)
   ! For each MLOp...
   do q=1,size(imlops)
      imlop = imlops(q)
      ! ...check if it has already been created.
      if (associated(mmlops(imlop)%mlop)) cycle
      ! If not, there must be a queue of (existing) MLOps, which will
      ! have to be multiplied out.
      ifacq => mmlops(imlop)%ifacq
      if (.not.associated(ifacq)) &
         call stop_code("mlpf_oper_m", "mlop_process_queues", "cannot create MLOp out of nothing")
      ! Multiply out the queue.
      mmlops(imlop)%mlop => mlop_multiply(ifacq,mlwf,k)
      ! Store the result in final form.
      mlham%mlops(q)%p => mmlops(imlop)%mlop
      mlham%kkoes(q) = k
   enddo
   deallocate(imlops)
end subroutine mlop_finalize


!-----------------------------------------------------------------------
function mlop_multiply(ifacq,mlwf,k) result(mlop)
!-----------------------------------------------------------------------
   type(llist_int_t),intent(inout) :: ifacq  ! list of indices into mmlops(:)
   type(wfnode_t),intent(in)       :: mlwf   ! tree structure that the result should have
   integer,intent(out)             :: k      ! the first k-term of the resulting MLOp
   type(optree_t),pointer          :: mlop   ! create here
   integer,pointer                 :: ifacs(:)
   integer                         :: nfac,k1,l,nmode,i,j,m
   type(optree_t),pointer          :: fac
   type(opnode_t),pointer          :: leaf
   logical,allocatable             :: covered(:)
   type(ilist_t),pointer           :: mlist
   type(opnode_tp),allocatable     :: facnodes(:)
   integer                         :: ierr
   character(len=240)              :: errmsg

   ifacs => llist_to_array(ifacq)
   nfac = size(ifacs)

   ! By construction, all leaves of all MLOps that are being multiplied here
   ! should reference the same first k-term. Check that. Also, that k-term
   ! contains information about the operators that are applied to those modes
   ! that are *not* covered by any of the MLOps, and about the coefficient.
   k = 0
   do i=1,nfac
      fac => mmlops(ifacs(i))%mlop
      if (.not.associated(fac)) &
         call stop_code("mlpf_oper_m", "mlop_multiply", "encountered undefined MLOp")
      do l=1,fac%numleaves
         leaf => fac%leaves(l)%p
         k1= leaf%ks(1)
         if (k==0) then
            k = k1
         else
            if (k /= k1) call stop_code("mlpf_oper_m", "mlop_multiply", &
               "MLOps don't reference the same first k-term")
         endif
      enddo
   enddo

   ! In order to create a "total" MLOp (i.e. it applies to all modes),
   ! we need to check if there are modes that are not covered by any
   ! of the MLOps from _facs_.
   nmode = if_nmode()
   allocate(covered(nmode))
   covered = .false.
   do i=1,nfac
      fac => mmlops(ifacs(i))%mlop
      mlist => mlop_modenums(fac%topnode)
      do j=1,mlist%n
         m = mlist%i(j)
         covered(m) = .true.
      enddo
      call dispose_ilist(mlist)
   enddo
   deallocate(ifacs)

   ! For each uncovered mode, create a (very simple) MLOp that operates
   ! only on that mode.
   do m=1,nmode
      if (covered(m)) cycle
      call next_mlop
      mmlops(nmlop)%mlop => make_optree_from_op1d(m,k)
      call llist_push(ifacq,nmlop)
   enddo

   ! The total MLOp must have the same tree structure as the wavefunction.
   mlop => make_optree_from_mlwf(mlwf)
   ! Collect pointers to the topnodes of all MLOps that take part in the product.
   ifacs => llist_to_array(ifacq)
   nfac = size(ifacs)
   allocate(facnodes(nfac))
   do i=1,nfac
      fac => mmlops(ifacs(i))%mlop
      facnodes(i)%p => fac%topnode
   enddo
   ! Now comes the hard part.
   call mlop_outer_product(mlop%topnode,facnodes,ierr,errmsg)
   ! Check.
   if (ierr /= 0) then
      write(*,'(a,i0)') "ierr   = ",ierr
      write(*,'(2a)') "errmsg = ",trim(errmsg)
      call stop_code("mlpf_oper_m", "mlop_multiply", "mlop_outer_product failed")
   endif
   ! Clean up.
   deallocate(facnodes)
   deallocate(ifacs)

end function mlop_multiply


!=======================================================================
end module genmlham
