!=======================================================================
module mlham
!=======================================================================

use mlbase
use mltypes
use llist_int
use if_grid
use wftree
use wftree1
use optree
use genmlop
use maxv, only: maxham
use operdef, only: mlhams
!use maxv, only:maxham

implicit none
private

public :: mlham_get, mlham_get_or_create, &
          mlham_dump, mlham_load

!integer,parameter   :: maxham = 64
!type(mlham_tp),allocatable,save :: mlhams(:)
!type(mlham_tp),save :: mlhams(maxham)


contains !==============================================================


!-----------------------------------------------------------------------
function mlham_get(nham) result(mlham)
!-----------------------------------------------------------------------
   type(mlham_t),pointer :: mlham
   integer,intent(in)    :: nham
   if (nham > maxham) &
      call stop_code("mlham_m", "mlham_get", "maxham is too small")
   mlham => mlhams(nham)%p
end function mlham_get


!-----------------------------------------------------------------------
function mlham_get_or_create(nham) result(mlham)
!-----------------------------------------------------------------------
   type(mlham_t),pointer :: mlham
   integer,intent(in)    :: nham

   if (nham > maxham) &
      call stop_code("mlham_m", "mlham_get_or_create", "maxham is too small")
   mlham => mlhams(nham)%p
   if (.not.associated(mlham)) then
      allocate(mlham)
      mlhams(nham)%p => mlham
      allocate(mlham%immlopq)
   endif
end function mlham_get_or_create


!-----------------------------------------------------------------------
subroutine mlham_dump(lun,nham)
!-----------------------------------------------------------------------
   integer,intent(in)     :: lun,nham
   type(mlham_t),pointer  :: mlham
   type(optree_t),pointer :: mlop
   integer                :: i

   mlham => mlhams(nham)%p
   if (.not.associated(mlham)) then
      write(lun) .false.
   else
      write(lun) .true.
      write(lun) mlham%nmlop
      do i=1,mlham%nmlop
         mlop => mlham%mlops(i)%p
         call dump_optree_data(mlop,lun)
         write(lun) mlham%kkoes(i)
      enddo
   endif
end subroutine mlham_dump


!-----------------------------------------------------------------------
subroutine mlham_load(lun,nham,tape,lrunml)
!-----------------------------------------------------------------------
   integer,intent(in)      :: lun,nham
   integer,intent(in)      :: tape(:)
   logical,intent(in)      :: lrunml
   type(wfnode_t),pointer  :: mlwf
   type(mlham_t),pointer   :: mlham
   type(optree_t),pointer  :: mlop
   logical                 :: flag
   integer                 :: i

   ! If there are no MLOps for this Hamiltonian, return.
   read(lun) flag
   if (.not.flag) return
   ! Check that we are doing ML.
   if (.not.lrunml) then
      call stop_code("mlham_m", "mlham_load", &
         "Multi-Layer operators can only be used with Multi-Layer wavefunctions.")
   endif
   ! Because the "oper" file is always read, even it was created
   ! in the same run, we need to check if the ML-Hamiltonian is
   ! already defined.
   mlham => mlhams(nham)%p
   if (associated(mlham)) then
      ! Skip all data.
      read(lun) ! nmlop
      do i=1,mlham%nmlop
         mlop => mlham%mlops(i)%p
         call skip_optree_data(mlop,lun)
         read(lun) ! kkoe
      enddo
   else
      ! Get the structure of the WF.
      mlwf => new_wftree(tape,if_spfdof)
      ! Allocate the Hamiltonian.
      allocate(mlham)
      read(lun) mlham%nmlop
      allocate(mlham%mlops(mlham%nmlop))
      allocate(mlham%kkoes(mlham%nmlop))
      ! Read all data.
      do i=1,mlham%nmlop
         mlop => make_optree_from_mlwf(mlwf)
         call load_optree_data(mlop,lun)
         mlham%mlops(i)%p => mlop
         read(lun) mlham%kkoes(i)
      enddo
      mlhams(nham)%p => mlham
      ! Clean up.
      call dispose_wftree(mlwf)
   endif
end subroutine mlham_load


!=======================================================================
end module mlham
