


module chksampling_m
  use dvrtype_m
  use global
  use logdat, only: errormsg, routine, message
  use mlbase, only: stop_code
  use versions

  implicit none

  character(len=c5), parameter :: mdl="chksampling"

  type(dvr_t)             :: dvr

  character(len=MAXFNAME) :: dvrfile
  character(len=MAXFNAME) :: efilein
  character(len=MAXFNAME) :: efileout
  character(len=MAXFNAME) :: idxfilein
  character(len=MAXFNAME) :: idxfileout
  character(len=MAXFNAME) :: logfile

  character(len=MAXFNAME) :: idxfilein_info
  character(len=MAXFNAME) :: efilein_info

  integer                 :: igrid, igrid2, nf, nf2, icont
  integer                 :: covorder, nhist2d
  real*8                  :: emin, emax, factor

  logical                 :: lrem, lrem2, lhist, lhist2
  logical                 :: lcov, lefile, lefileo



  type histogram_t
     character(len=16)    :: modelabel
     integer              :: gdim
     integer, allocatable :: bins(:)
     real*8, allocatable  :: coord(:)
  end type histogram_t

  type histogram2d_t
     character(len=16)    :: modelabel, modelabel1
     integer              :: gdim, gdim1
     integer, allocatable :: bins(:,:)
     real*8, allocatable  :: coord(:), coord1(:)
  end type histogram2d_t



  type(histogram_t), allocatable   :: histogram(:)
  type(histogram2d_t), allocatable :: histogram2d(:)



contains



  subroutine allocate_hiostogram_t(dvr)
    implicit none
    type(dvr_t), intent(IN) :: dvr
    integer                 :: f,z,g,ierr

    call deallocate_hiostogram_t()

    if (dvr%ndof.le.0) then
       routine = "allocate_hiostogram_t"
       message = "DVR contains .le. 0 DOF."
       call stop_code(mdl,routine,message)
    end if

    allocate(histogram(dvr%ndof), stat=ierr)
    if (ierr.ne.0) then
       routine = "allocate_hiostogram_t"
       message = "Error while allocating histograms array."
       call errormsg
    end if

    do f=1,dvr%ndof
       allocate(histogram(f)%bins(dvr%gdim(f)),stat=ierr)
       if (ierr.ne.0) then
          routine = "allocate_hiostogram_t"
          message = "Error while allocating histogram " // &
               "bins for dof "//trim(dvr%modelabel(f))
          call errormsg
       end if
       histogram(f)%bins(:) = 0
       allocate(histogram(f)%coord(dvr%gdim(f)),stat=ierr)
       if (ierr.ne.0) then
          routine = "allocate_hiostogram_t"
          message = "Error while allocating histogram " // &
               "coord for dof "//trim(dvr%modelabel(f))
          call errormsg
       end if

       z = dvr%zort(f)
       do g=1,dvr%gdim(f)
          histogram(f)%coord(g) = dvr%ort(z+g-1)
       end do

       histogram(f)%modelabel = dvr%modelabel(f)
       histogram(f)%gdim = dvr%gdim(f)
    end do

  end subroutine allocate_hiostogram_t


  subroutine deallocate_hiostogram_t()
    implicit none

    integer :: f

    if (allocated(histogram)) then
       do f=1,size(histogram)
          if (allocated(histogram(f)%bins)) then
             deallocate(histogram(f)%bins)
          end if
          if (allocated(histogram(f)%coord)) then
             deallocate(histogram(f)%coord)
          end if
       end do
       deallocate(histogram)
    end if

  end subroutine deallocate_hiostogram_t



  subroutine allocate_hiostogram2d_t(dvr)
    implicit none
    type(dvr_t), intent(IN) :: dvr
    integer                 :: f,f1,z,g,ierr, i

    call deallocate_hiostogram2d_t()

    if (dvr%ndof.le.0) then
       routine = "allocate_hiostogram_t"
       message = "DVR contains .le. 0 DOF."
       call stop_code(mdl,routine,message)
    end if

    nhist2d = (dvr%ndof - 1)*dvr%ndof / 2

    allocate(histogram2d(nhist2d), stat=ierr)
    if (ierr.ne.0) then
       routine = "allocate_hiostogram_t"
       message = "Error while allocating histograms array."
       call stop_code(mdl,routine,message)
    end if

    i = 0
    do f=1,dvr%ndof
       do f1=f+1, dvr%ndof
          i = i + 1

          allocate(histogram2d(i)%bins(dvr%gdim(f),dvr%gdim(f1)),stat=ierr)

          if (ierr.ne.0) then
             routine = "allocate_hiostogram2d_t"
             message = "Error while allocating 2D histogram " // &
                  "bins for dofs "//trim(dvr%modelabel(f1))//", " &
                  //trim(dvr%modelabel(f))
             call stop_code(mdl,routine,message)
          end if

          histogram2d(i)%bins(:,:) = 0

          allocate(histogram2d(i)%coord(dvr%gdim(f)),stat=ierr)
          if (ierr.ne.0) then
             routine = "allocate_hiostogram2d_t"
             message = "Error while allocating 2D histogram " // &
                  "coord for dof "//trim(dvr%modelabel(f))
             call stop_code(mdl,routine,message)
          end if

          allocate(histogram2d(i)%coord1(dvr%gdim(f1)),stat=ierr)
          if (ierr.ne.0) then
             routine = "allocate_hiostogram2d_t"
             message = "Error while allocating 2D histogram " // &
                  "coord for dof "//trim(dvr%modelabel(f))
             call stop_code(mdl,routine,message)
          end if


          z = dvr%zort(f)
          do g=1,dvr%gdim(f)
             histogram2d(i)%coord(g) = dvr%ort(z+g-1)
          end do
          z = dvr%zort(f1)
          do g=1,dvr%gdim(f1)
             histogram2d(i)%coord1(g) = dvr%ort(z+g-1)
          end do

          histogram2d(i)%modelabel = dvr%modelabel(f)
          histogram2d(i)%gdim = dvr%gdim(f)

          histogram2d(i)%modelabel1 = dvr%modelabel(f1)
          histogram2d(i)%gdim1 = dvr%gdim(f1)

       end do !f1

    end do !f

  end subroutine allocate_hiostogram2d_t


  subroutine deallocate_hiostogram2d_t()
    implicit none

    integer :: f

    if (allocated(histogram2d)) then
       do f=1,size(histogram2d)
          if (allocated(histogram2d(f)%bins)) then
             deallocate(histogram2d(f)%bins)
          end if
          if (allocated(histogram2d(f)%coord)) then
             deallocate(histogram2d(f)%coord)
          end if
          if (allocated(histogram2d(f)%coord1)) then
             deallocate(histogram2d(f)%coord1)
          end if
       end do
       deallocate(histogram2d)
    end if

  end subroutine deallocate_hiostogram2d_t




end module chksampling_m






program chksampling
  use chksampling_m

  implicit none

  integer              :: nsample,nsample1,ierr
  integer, allocatable :: dvrindex(:,:)
  logical, allocatable :: lout(:)
  real*8, allocatable  :: energies(:)

  call default()

  call chksampling_input()

  call loaddvr(dvr,dvrfile)

  call linecount(nsample, idxfilein,"#",  ierr)
  if (ierr .ne. 0) then
     routine = "Chksampling, checking files"
     message = "An error occured during read of file "//trim(idxfilein)
     call stop_code(mdl,routine,message)
  end if
  if (lefile) then
     call linecount(nsample1, efilein,"#",  ierr)
     if (ierr .ne. 0) then
        routine = "Chksampling, checking files"
        message = "An error occured during read of file"//trim(efilein)
        call stop_code(mdl,routine,message)
     end if
     if (nsample .ne. nsample1) then
        write(*,*) "WARNING: different number of lines (points) on files:"
        write(*,*) ""
        write(*,*) "Points, filename"
        write(*,*) "--------------------------------"
        write(*,*) nsample,  trim(idxfilein)
        write(*,*) nsample1, trim(efilein)
        write(*,*) "--------------------------------"
        write(*,*) ""
        write(*,*) "Are these files associated?"
        nsample = min(nsample,nsample1)
        write(*,*) "Using nsample = ", nsample
     end if
  end if



  allocate(lout(nsample), stat=ierr)
  if (ierr.ne.0) then
     routine = "Chksampling, checking files"
     message = "Unable to allocate logical workspace"
     call stop_code(mdl,routine,message)
  end if
  lout(:) = .true.
  allocate(dvrindex(dvr%ndof,nsample), stat=ierr)
  if (ierr.ne.0) then
     routine = "Chksampling, checking files"
     message = "Unable to allocate integer workspace"
     call stop_code(mdl,routine,message)
  end if

  call chkloadidxfile(dvrindex,nsample,dvr%ndof)


  if (lhist) then
     call chkmkhistograms(dvrindex,lout,nsample,dvr%ndof,"")
  end if
  if (lhist2) then
     call chkmkhistograms2d(dvrindex,lout,nsample,dvr%ndof,"")
  end if
  if (lcov) then
     call chkmkcovariance(dvrindex,lout,nsample,"")
  end if

  if (lefile) then


     allocate(energies(nsample), stat=ierr)
     if (ierr.ne.0) then
        routine = "Chksampling, checking files"
        message = "Unable to allocate logical workspace"
        call stop_code(mdl,routine,message)
     end if

     call chkloadefile(energies,nsample,dvr%ndof)

     if (lrem.or. lrem2) then
        call chkmarkrubbish(dvrindex,energies,lout,nsample,dvr%ndof,dvr%gdim)
        call chkwrcleaned(dvrindex,energies,lout,nsample,dvr%ndof)
     end if

     if (lrem .and. lhist) then
        call chkmkhistograms(dvrindex,lout,nsample,dvr%ndof,"_chk")
     end if

     if (lrem .and. lhist2) then
        call chkmkhistograms2d(dvrindex,lout,nsample,dvr%ndof,"_chk")
     end if

     if (lrem .and. lcov) then
        call chkmkcovariance(dvrindex,lout,nsample,"_chk")
     end if
  end if


end program chksampling




subroutine chkwrcleaned(dvrindex,energies,lout,nsample,ndof1)
  use chksampling_m
  implicit none
  integer, intent(IN)          :: nsample,ndof1
  integer, intent(IN)          :: dvrindex(ndof1,nsample)
  logical, intent(IN)          :: lout(nsample)
  real*8, intent(IN)           :: energies(nsample)


  character(len=64)            :: formatstr
  integer                      :: nsample1,n,f,ierr, idump

  nsample1 = 0
  do n=1,nsample
     if (lout(n)) nsample1 = nsample1 + 1
  end do

  write(message,'(a,i8,a)') "Removed ", nsample - nsample1, " samples"
  call cncntrblnk(message)
  write(*,'(a)') trim(message)
  write(message,'(a,i8,a)') "Remaining: ", nsample1
  call cncntrblnk(message)
  write(*,'(a)') trim(message)

  open(newunit=idump, file=idxfileout,iostat=ierr, &
       status="unknown",form="formatted")
  if (ierr.ne.0) then
     routine = "chkwrcleaned"
     message = "Unable to open file "//trim(idxfileout)
     call stop_code(mdl,routine,message)
  end if

  write(idump,'(a,i8)',iostat=ierr) trim(idxfilein_info)// &
       " ## chksampling cleaned: remaining samples:", nsample1

  if (ierr.ne.0) then
     routine = "chkwrcleaned"
     message = "Unable to write to file "//trim(idxfileout)
     call stop_code(mdl,routine,message)
  end if

  write(formatstr,'(a,i8,a)') "(",ndof1,"i8)"
  call rmblnk(formatstr)

  do n=1,nsample
     if (lout(n)) then
        write(idump,formatstr,iostat=ierr) (dvrindex(f,n),f=1,ndof1)
        if (ierr.ne.0) then
           routine = "chkwrcleaned"
           message = "Unable to write to file "//trim(idxfileout)
           call stop_code(mdl,routine,message)
        end if
     end if
  end do
  close(idump)



  open(newunit=idump, file=efileout,iostat=ierr, &
       status="unknown",form="formatted")
  if (ierr.ne.0) then
     routine = "chkwrcleaned"
     message = "Unable to open file "//trim(efileout)
     call stop_code(mdl,routine,message)
  end if

  write(idump,'(a,i8)',iostat=ierr) trim(efilein_info)// &
       " cleaned: remaining samples:", nsample1

  if (ierr.ne.0) then
     routine = "chkwrcleaned"
     message = "Unable to write to file "//trim(efileout)
     stop 1
  end if

  do n=1,nsample
     if (lout(n)) then
        write(idump,'(f16.8)',iostat=ierr) energies(n)
        if (ierr.ne.0) then
           routine = "chkwrcleaned"
           message = "Unable to write to file "//trim(efileout)
           call stop_code(mdl,routine,message)
        end if
     end if
  end do
  close(idump)



end subroutine chkwrcleaned




subroutine chkmarkrubbish(dvrindex,energies,lout,nsample,ndof1,gdim1)
  use chksampling_m
  implicit none
  integer, intent(IN)          :: nsample,ndof1
  integer, intent(IN)          :: dvrindex(ndof1,nsample), gdim1(ndof1)
  logical, intent(INOUT)       :: lout(nsample)
  real*8, intent(IN)           :: energies(nsample)

  integer    :: n,n1,f,gcount,gcount2, g, matchcount
  logical    :: lmatch

  matchcount = 0

  do n=1,nsample

     if (.not. lout(n)) cycle

     gcount = 0
     gcount2 = 0

     do f=1,ndof1
        g = dvrindex(f,n)
        if ((g-igrid) .le. 1 .or. (g+igrid) .ge. gdim1(f)) gcount = gcount +1
        if ((g-igrid2) .le. 1 .or. (g+igrid2) .ge. gdim1(f)) gcount2 = gcount2 +1
     end do

     lmatch = .false.
     if (lrem) then
        lmatch = lmatch .or. (gcount .ge. nf .and. energies(n) .lt. emin)
     end if
     if (lrem2) then
        lmatch = lmatch .or. (gcount2 .ge. nf2 .and. energies(n) .gt. emax)
     end if

     if (lmatch) then
        matchcount = matchcount + 1
        lout(n) = .false.
        if (icont .ge. 1) then
           n1 = max(1,n-icont)
           lout(n1:n-1) = .false.
           n1 = min(nsample,n+icont)
           lout(n+1:n1) = .false.
        end if
     end if

  end do

end subroutine chkmarkrubbish




subroutine chkmkhistograms(dvrindex,lout,nsample,ndof1,suffix)
  use chksampling_m
  implicit none
  integer, intent(IN)          :: nsample,ndof1
  integer, intent(IN)          :: dvrindex(ndof1,nsample)
  logical, intent(IN)          :: lout(nsample)
  character(len=*), intent(IN) :: suffix

  integer                      :: n,f,g,ierr, idump
  character(len=MAXFNAME)      :: fname
  logical                      :: lexist

  call allocate_hiostogram_t(dvr)

  do f=1,ndof1
     do n=1,nsample
        if (lout(n)) then
           g = dvrindex(f,n)
           histogram(f)%bins(g) = histogram(f)%bins(g) + 1
        end if
     end do
  end do


  do f=1,ndof1

     fname = trim(idxfilein)//"_hist_"//trim(histogram(f)%modelabel)//trim(suffix)

     inquire(file=fname,exist=lexist)
     if (lexist .and. .not. lovwr .and. lrem) then
        routine = "chkmkhistograms"
        message = "File exists, cannot overwrite. Use -w option: "//trim(fname)
        call stop_code(mdl,routine,message)
     end if

     open(newunit=idump, file=fname,iostat=ierr, &
          status="unknown",form="formatted")
     if (ierr.ne.0) then
        routine = "chkmkhistograms"
        message = "Unable to open file "//trim(fname)
        call stop_code(mdl,routine,message)
     end if

     do g=1,histogram(f)%gdim
        write(idump,'(f16.8,i8)',iostat=ierr) histogram(f)%coord(g), &
             histogram(f)%bins(g)
        if (ierr.ne.0) then
           routine = "chkmkhistograms"
           message = "Unable to write to file "//trim(fname)
           call stop_code(mdl,routine,message)
        end if
     end do
     close(idump)

  end do

  call deallocate_hiostogram_t

end subroutine chkmkhistograms










subroutine chkmkhistograms2d(dvrindex,lout,nsample,ndof1,suffix)
  use chksampling_m
  implicit none
  integer, intent(IN)          :: nsample,ndof1
  integer, intent(IN)          :: dvrindex(ndof1,nsample)
  logical, intent(IN)          :: lout(nsample)
  character(len=*), intent(IN) :: suffix

  integer                      :: n,f,f1,g,g1,ierr,i
  character(len=MAXFNAME)      :: fname
  logical                      :: lexist
  integer                      :: idump

  call allocate_hiostogram2d_t(dvr)

  i = 0
  do f=1,ndof1
     do f1=f+1,ndof1
        i = i+1
        do n=1,nsample
           if (lout(n)) then
              g = dvrindex(f,n)
              g1 = dvrindex(f1,n)
              histogram2d(i)%bins(g,g1) = histogram2d(i)%bins(g,g1) + 1
           end if
        end do
     end do
  end do

  i = 0
  do f=1,ndof1
     do f1=f+1,ndof1
        i = i+1



        fname = trim(idxfilein)//"_hist2d_"//trim(histogram2d(i)%modelabel) &
             //"_"//trim(histogram2d(i)%modelabel1)//trim(suffix)

        inquire(file=fname,exist=lexist)
        if (lexist .and. .not. lovwr .and. lrem) then
           routine = "chkmkhistograms"
           message = "File exists, cannot overwrite. Use -w option: "//trim(fname)
           call stop_code(mdl,routine,message)
        end if

        open(newunit=idump, file=fname,iostat=ierr,status="unknown",form="formatted")
        if (ierr.ne.0) then
           routine = "chkmkhistograms"
           message = "Unable to open file "//trim(fname)
           call stop_code(mdl,routine,message)
        end if

        do g=1,histogram2d(i)%gdim
           do g1=1,histogram2d(i)%gdim1
              write(idump,'(2f16.8,i8)',iostat=ierr) histogram2d(i)%coord(g), &
                   histogram2d(i)%coord1(g1),  histogram2d(i)%bins(g,g1)
              if (ierr.ne.0) then
                 routine = "chkmkhistograms"
                 message = "Unable to write to file "//trim(fname)
                 call stop_code(mdl,routine,message)
              end if
           end do
           write(idump,*) ""
        end do
        close(idump)

     end do
  end do

  call deallocate_hiostogram2d_t

end subroutine chkmkhistograms2d



subroutine chkmkcovariance(dvrindex,lout,nsample,suffix)
  use chksampling_m
  use linkedlist_m
  use dvrtype_m
  implicit none

  integer, intent(IN)          :: nsample
  integer, intent(IN)          :: dvrindex(dvr%ndof,nsample)
  logical, intent(IN)          :: lout(nsample)
  character(len=*), intent(IN) :: suffix


  character(len=c5)   :: fname

  double precision    :: cov(dvr%ndof,dvr%ndof)
  integer             :: f,f1,n, g,g1, z,z1, nout
  double precision    :: mean(dvr%ndof),var(dvr%ndof), q, q1, mn, va
  logical             :: lexist, lmatch
  type(linkedlist_t), pointer :: first, curr
  character(len=8)    :: orderstr
  character(len=16)   :: label, label1
  integer             :: idata(2)
  double precision    :: ddata(1)
  integer             :: idump


  nullify(first)
  nullify(curr)

  nout = 0
  do n=1,nsample
     if (lout(n)) nout = nout+1
  end do

  write(orderstr,'(i8)') covorder
  call rmblnk(orderstr)

  cov(:,:) = 0.d0
  mean(:) = 0.d0
  var(:) = 0.d0

  do f=1,dvr%ndof
     z = dvr%zort(f)
     mn = 0.d0
     va = 0.d0

     !$OMP PARALLEL DO REDUCTION(+:mn,va) PRIVATE(g,q)
     do n=1,nsample
        if (lout(n)) then
           g = dvrindex(f,n)
           q = dvr%ort(z+g-1)**covorder
           mn = mn + q
           va = va + q**2
        end if
     end do
     !$OMP END PARALLEL DO

     mean(f) = mn / nout
     var(f) = va/nout - mean(f)**2

  end do



  do f=1,dvr%ndof
     z = dvr%zort(f)

     do f1 = f, dvr%ndof
        z1 = dvr%zort(f1)

        va = 0.d0

        !$OMP PARALLEL DO REDUCTION(+:va) PRIVATE(g,q)
        do n=1,nsample
           if (lout(n)) then
              g = dvrindex(f,n)
              q = dvr%ort(z+g-1)**covorder
              g1 = dvrindex(f1,n)
              q1 = dvr%ort(z1+g1-1)**covorder
              va = va + q*q1
           end if
        end do
        !$OMP END PARALLEL DO

        va = va/nout - mean(f)*mean(f1)

        cov(f,f1) = va/sqrt(var(f)*var(f1))
        cov(f1,f) = cov(f,f1)

     end do
  end do


  ! build a sorted covariances list
  n = 0
  do f=1,dvr%ndof
     do f1=f+1, dvr%ndof

        ddata(1) = cov(f1,f)
        idata(1) = f1
        idata(2) = f


        ! create first element
        if (.not. associated(first)) then

           first => linkedlist_t(ddata=ddata, idata=idata)
           curr => first
           n = 1
        else

           curr => first
           lmatch = .false.
           do while (.not. lmatch)

              if (abs(curr%ddata(1)) .lt. abs(cov(f1,f)) ) then
                 call  insert_bef(curr,ddata=ddata, idata=idata)
                 lmatch = .true.
                 if (.not. associated(curr%prev%prev)) then
                    first => curr%prev
                 end if
              else if (associated(curr%next)) then
                 curr => curr%next
              else if (abs(curr%ddata(1)) .ge. abs(cov(f1,f)) .and. .not. associated(curr%next) ) then
                 call  insert_after(curr, ddata=ddata, idata=idata)
                 lmatch = .true.
              else
                 print*, "ERROR while sorting covariances."
                 print*, "This should never have happened..."
                 print*, "Please send a bug report."
                 stop 1
              end if
           end do

        end if

     end do
  end do


  fname = trim(idxfilein)//"_covariance"//trim(orderstr)//"_sorted"//trim(suffix)
  inquire(file=fname,exist=lexist)
  if (lexist .and. .not. lovwr) then
     routine = "chkmkcovariance"
     message = "File exists, cannot overwrite. Use -w option: "//trim(fname)
     call stop_code(mdl,routine,message)
  end if

  open(newunit=idump, file=fname, action="write")
  write(idump,'(a)') "#n, f, f1, cov(f,f1), label(f), label(f1)"

  n = 0
  curr => first
  do while (associated(curr))
     n = n+1
     f1 = curr%idata(1)
     f = curr%idata(2)
     if (covorder .gt. 1) then
        label=trim(dvr%modelabel(f))//"^"//trim(orderstr)
        label1=trim(dvr%modelabel(f1))//"^"//trim(orderstr)
     else
        label=dvr%modelabel(f)
        label1=dvr%modelabel(f1)
     end if

     write(idump,'(3i4,3x,f16.12,3x,2a)') n, f,f1, cov(f,f1), label, trim(label1)

     curr => curr%next
  end do
  close(idump)
  call free_list(first)



  fname = trim(idxfilein)//"_covariance"//trim(orderstr)//trim(suffix)

  inquire(file=fname,exist=lexist)
  if (lexist .and. .not. lovwr) then
     routine = "chkmkcovariance"
     message = "File exists, cannot overwrite. Use -w option: "//trim(fname)
     call stop_code(mdl,routine,message)
  end if


  open(newunit=idump, file=fname, action="write")
  write(idump,'(a)') "# f f1 cov(f^n,f1^n), n = "//trim(orderstr)

  do f1=1,dvr%ndof
     do f=1,dvr%ndof
        write(idump,'(2i8,f16.12)') f,f1, cov(f,f1)
     end do
     write(idump,*)
  end do
  close(idump)



  fname = trim(idxfilein)//"_mean_var_"//trim(orderstr)//trim(suffix)

  inquire(file=fname,exist=lexist)
  if (lexist .and. .not. lovwr) then
     routine = "chkmkcovariance"
     message = "File exists, cannot overwrite. Use -w option: "//trim(fname)
     call stop_code(mdl,routine,message)
  end if


  open(newunit=idump, file=fname, action="write")
  write(idump,'(a)') "# f, mean(f^n), variance(f^n), modelabel(f); n = "//trim(orderstr)
  do f=1,dvr%ndof
     if (covorder .gt. 1) then
        write(idump,'(i8,2f16.8,a)') f,mean(f), var(f), "  "//trim(dvr%modelabel(f))
     else
        write(idump,'(i8,2f16.8,a)') f,mean(f), var(f), "  "//trim(dvr%modelabel(f))//"^"//trim(orderstr)
     end if
  end do

  close(idump)





end subroutine chkmkcovariance








subroutine chkloadidxfile(dvrindex,nsample,ndof1)
  use chksampling_m
  implicit none
  integer, intent(IN)  :: nsample,ndof1
  integer, intent(OUT) :: dvrindex(ndof1,nsample)

  integer              :: ierr, idump
  character(len=512)   :: buf
  logical              :: lhash, lstart
  integer              :: n,f


  open(newunit=idump,file=idxfilein, form='formatted', status='old', iostat=ierr)
  if (ierr.ne.0) then
     routine = "chkloadfiles"
     message =  "Unable to open file "//trim(idxfilein)
     call stop_code(mdl,routine,message)
  end if

  lstart = .true.
  do n=1,nsample
     lhash = .true.
     do while (lhash)
        read(idump,'(a)',iostat=ierr) buf
        if (ierr.lt.0) then
           routine = "chkloadfiles"
           message = "Found end of file "//trim(idxfilein)
           call stop_code(mdl,routine,message)
        else if (ierr.gt.0) then
           routine = "chkloadfiles"
           message = "Error while reading file"//trim(idxfilein)
           call stop_code(mdl,routine,message)
        end if

        buf = adjustl(buf)
        if (buf(1:1) .ne. "#") then
           lhash = .false.
           backspace(idump)
        else if (lstart .and. n.eq.1) then
           idxfilein_info = buf
           lstart = .false.
        end if
     end do

     read(idump,*,iostat=ierr) (dvrindex(f,n),f=1,ndof1)

     if (ierr.lt.0) then
        routine = "chkloadfiles"
        message = "Error while reading file"//trim(idxfilein)
        call stop_code(mdl,routine,message)
     end if

  end do

  close(idump)

end subroutine chkloadidxfile





subroutine chkloadefile(energies,nsample,ndof1)
  use chksampling_m
  use global
  implicit none
  integer, intent(IN)  :: nsample,ndof1
  real*8, intent(OUT)  :: energies(nsample)


  integer              :: ierr, idump
  character(len=512)   :: buf
  logical              :: lhash, lstart
  integer              :: n,f


  open(newunit=idump,file=efilein, form='formatted', status='old', iostat=ierr)
  if (ierr.ne.0) then
     routine = "chkloadfiles"
     message = "Unable to open file "//trim(efilein)
     call stop_code(mdl,routine,message)
  end if

  lstart = .true.
  do n=1,nsample
     lhash = .true.
     do while (lhash)
        read(idump,'(a)',iostat=ierr) buf
        if (ierr.lt.0) then
           routine = "chkloadfiles"
           message = "Found end of file "//trim(efilein)
           call stop_code(mdl,routine,message)
        else if (ierr.gt.0) then
           routine = "chkloadfiles"
           message = "Error while reading file"//trim(efilein)
           call stop_code(mdl,routine,message)
        end if

        buf = adjustl(buf)
        if (buf(1:1) .ne. "#") then
           lhash = .false.
           backspace(idump)
        else if (lstart .and. n.eq.1) then
           efilein_info = buf
           lstart = .false.
        end if
     end do

     read(idump,*,iostat=ierr) energies(n)
     if (ierr.lt.0) then
        routine = "chkloadfiles"
        message = "Error while reading file"//trim(efilein)
        call stop_code(mdl,routine,message)
     end if

  end do

  close(idump)

end subroutine chkloadefile





subroutine chksampling_input()
  use chksampling_m
  implicit none

  character(len=MAXFNAME) :: buf, buf1
  integer                 :: i,ierr, iarg, myiargc, slen
  logical                 :: lexist, lstop
  external myiargc


  dvrfile    = "./dvr"
  efilein    = ""
  efileout   = ""
  idxfilein  = ""
  idxfileout = ""
  logfile    = ""

  lrem   = .false.
  lhist  = .false.
  lovwr  = .false.
  lefile   = .false.
  lefileo  = .false.
  igrid  = 0
  nf     = 0
  icont  = 0
  emin   = -1.d99
  factor = 1.d0
  covorder = 0

  iarg = myiargc()

  call mygetarg(1,buf)
  if( buf(1:1) .eq. ' ' ) then
     ! No arguments: print help and exit
     call chksampling_wrhelptext()
     stop 0
  end if

  call mygetarg(0,options)
  do i = 1, iarg
     call mygetarg(i,buf1)
     options = trim(options)//' '//trim(buf1)
     if  ( buf1 .eq. '-h' .or. &
          buf1 .eq. '-?' .or. &
          buf1 .eq. '--help' ) then
        call chksampling_wrhelptext
        stop 0
     else if  ( buf .eq. '-ver' ) then
        call wrversion(6)
        stop 0
     end if
  end do

  if (iarg.lt.2) then
     print*, "Not enough arguments:"
     print*, "Usage: chksampling86 [options] <indexfile> <energiesfile>"
     stop 1
  end if


  i = 0
  do while (i.lt. iarg-1)
     i=i+1

     call mygetarg(i,buf)

     if( buf(1:1) .eq. '-' ) then
        if ( i.gt. iarg -1) then
           print*, "Expecting file names as last two input parameters,"
           print*, "not: "//trim(buf)
           stop 1

        elseif ( buf .eq. '-w') then
           lovwr = .true.
        elseif ( buf .eq. '-hist') then
           lhist = .true.
        elseif ( buf .eq. '-hist2') then
           lhist2 = .true.
        elseif (buf .eq. '-cov') then
           lcov = .true.
           i=i+1
           call mygetarg(i,buf)
           read(buf,*, iostat=ierr) covorder
           if (ierr .ne. 0) then
              print*, "Cannot read value after -cov option. " &
                   // "Expecting integer, not: "//trim(buf)
              stop 1
           end if
           if (covorder .lt. 1) then
              print*, "Integer after -cov option must be positive > 0."
              print*, "Found: "//trim(buf)
           end if


        elseif ( buf .eq. '-C') then
           i=i+1
           call mygetarg(i,buf)
           read(buf,*, iostat=ierr) icont
           if (ierr .ne. 0) then
              print*, "Cannot read value after -C option. " &
                   // "Expecting integer, not: "//trim(buf)
              stop 1
           end if
           if (icont .le. 0) then
              print*, "Integer after -C option must be positive."
              print*, "Found: "//trim(buf)
           end if
        elseif ( buf .eq. '-rem' ) then
           lrem = .true.

           i=i+1
           call mygetarg(i,buf)
           read(buf,*, iostat=ierr) nf
           if (ierr .ne. 0) then
              print*, "Cannot read value after -rem option. " &
                   // "Expecting integer, not: "//trim(buf)
              stop 1
           end if
           if (nf .le. 0) then
              print*, "Integer after -rem option must be positive > 0."
              print*, "Found: "//trim(buf)
           end if


           i=i+1
           call mygetarg(i,buf)
           read(buf,*, iostat=ierr) igrid
           if (ierr .ne. 0) then
              print*, "Cannot read second value after -rem option. " &
                   // "Expecting integer, not: "//trim(buf)
              stop 1
           end if
           if (nf .lt. 0) then
              print*, "Second integer after -rem option must be non-negative."
              print*, "Found: "//trim(buf)
           end if

           i=i+1
           call mygetarg(i,buf)

           read(buf,*, iostat=ierr) emin
           if (ierr .ne. 0) then
              print*, "Cannot read third value after -rem option. " &
                   // "Expecting real, not: "//trim(buf)
              stop 1
           end if

        elseif ( buf .eq. '-rem2' ) then
           lrem2 = .true.

           i=i+1
           call mygetarg(i,buf)
           read(buf,*, iostat=ierr) nf2
           if (ierr .ne. 0) then
              print*, "Cannot read value after -rem2 option. " &
                   // "Expecting integer, not: "//trim(buf)
              stop 1
           end if
           if (nf2 .lt. 0) then
              print*, "Integer after -rem2 option must be non-negative."
              print*, "Found: "//trim(buf)
              stop 1
           end if


           i=i+1
           call mygetarg(i,buf)
           read(buf,*, iostat=ierr) igrid2
           if (ierr .ne. 0) then
              print*, "Cannot read second value after -rem2 option. " &
                   // "Expecting integer, not: "//trim(buf)
              stop 1
           end if
           if (igrid2 .lt. 0) then
              print*, "Second integer after -rem2 option must be non-negative."
              print*, "Found: "//trim(buf)
              stop 1
           end if

           i=i+1
           call mygetarg(i,buf)

           read(buf,*, iostat=ierr) emax
           if (ierr .ne. 0) then
              print*, "Cannot read third value after -rem2 option. " &
                   // "Expecting real, not: "//trim(buf)
              stop 1
           end if





        elseif ( buf .eq. '-d' ) then
           i = i+1
           call mygetarg(i,dvrfile)
        elseif (buf .eq. '-e') then
           i=i+1
           call mygetarg(i,efilein)
           lefile = .true.
        elseif ( buf .eq. '-oi' ) then
           i = i+1
           call mygetarg(i,idxfileout)
        elseif ( buf .eq. '-oe' ) then
           i = i+1
           call mygetarg(i,efileout)
           lefileo = .true.
        else
           write(*,*) "Unknown option: "//trim(buf)
           stop 1
        end if

     else
        write(*,*) "Unknown option: "//trim(buf)
        stop 1
     end if


  end do

  ! the last two arguments must be file names
  i = i+1
  call mygetarg(i,idxfilein)

  if(lrem.or.lrem2)  then
     slen = len_trim(idxfileout)
     if (slen .eq. 0) then
        idxfileout = trim(idxfilein)//".chk"
     end if
  end if

  if (lefile .and. (lrem.or.lrem2) ) then
     slen = len_trim(efileout)
     if (slen .eq. 0) then
        efileout = trim(efilein)//".chk"
     end if
  end if

  ! check if files exist
  ! and if so wether we overwrite
  ! or read

  lstop = .false.
  inquire(file=dvrfile,exist=lexist)
  if (.not. lexist) then
     write(*,*) "File does not exist:"
     write(*,*) trim(dvrfile)
     write(*,*) ""
     lstop = .true.
  end if
  inquire(file=idxfilein,exist=lexist)
  if (.not. lexist) then
     write(*,*) "File does not exist:"
     write(*,*) trim(idxfilein)
     write(*,*) ""
     lstop = .true.
  end if

  if (lefile) then
     inquire(file=efilein,exist=lexist)
     if (.not. lexist) then
        write(*,*) "File does not exist:"
        write(*,*) trim(efilein)
        write(*,*) ""
        lstop = .true.
     end if
  end if

  if (lrem.or.lrem2) then
     inquire(file=idxfileout,exist=lexist)
     if (lexist .and. .not. lovwr) then
        write(*,*) "File exists, cannot overwrite. Use -w option:"
        write(*,*) trim(idxfileout)
        write(*,*) ""
        lstop = .true.
     end if
  end if

  if (lefile .and. (lrem.or.lrem2) ) then
     inquire(file=efileout,exist=lexist)
     if (lexist .and. .not. lovwr) then
        write(*,*) "File exists, cannot overwrite. Use -w option:"
        write(*,*) trim(efileout)
        write(*,*) ""
        lstop = .true.
     end if
  end if

  if (lstop) stop 1


end subroutine chksampling_input



subroutine chksampling_wrhelptext()
  implicit none

  write(6,*)
  write(6,'(a)') 'Purpose: check the sampling points'
  write(6,'(a)') '         of a mccpd/mcpotfit run'
  write(6,*)
  write(6,'(a)') '  Usage:  chksampling<vers><d/D> [-h|-?]  [-ver -d -e -oi -oe'
  write(6,'(32x,a)')                  '-rem -C -hist] <indexfile>'
  write(6,*)
  write(6,'(a)') '   vers:  Program version (e.g. 86 for version 8.6)'
  write(6,'(a)') '    d/D:  Indicates a debug version'
  write(6,*)
  write(6,'(a)') ' optn: -h          : Print this help-text.'
  write(6,'(a)') '       -?          : Print this help-text.'
  write(6,'(a)') '       -ver        : Version information.'
  write(6,*)
  write(6,'(a)') ' -d <dvrfile>      : read dvr from <dvrfile> rather'
  write(6,'(a)') '                     than from ./dvr'
  write(6,'(a)') ' -e <energiesfile> : File containing energies matching the'
  write(6,'(a)') '                     <indexfile>. Required for -rem and -rem2.'
  write(6,'(a)') ' -oi <newindex>    : write cleaned index to <newindex>'
  write(6,'(a)') '                     rather than to <indexfile>_chk'
  write(6,'(a)') ' -oe <newenergies> : write cleaned energies to <newenergies>'
  write(6,'(a)') '                     rather than to <energiesfile>_chk'
  write(6,*)
  write(6,'(a)') ' -rem <n> <i> <e>  : remove points if at least <n>'
  write(6,'(a)') '                     DOF are at most <i> points away from '
  write(6,'(a)') '                     the grid boundary (i=0: first or last'
  write(6,'(a)') '                     point ) AND the energy is less then <e>.'
  write(6,*)
  write(6,'(a)') ' -rem2 <n> <i> <e> : remove points if at least <n>'
  write(6,'(a)') '                     DOF are at most <i> points away from '
  write(6,'(a)') '                     the grid boundary (i=0: first or last'
  write(6,'(a)') '                     point) AND the energy is larger then <e>.'
  write(6,*)
  write(6,'(a)') ' -C <n>            : If -rem or rem2 is set, also remove <n> lines'
  write(6,'(a)') '                     of context before and after a removed point.'
  write(6,*)
  write(6,'(a)') ' -hist             : Write histogram files of the distribution'
  write(6,'(a)') '                     of sampling points. If -rem or -rem2 is set,'
  write(6,'(a)') '                     histograms will be made before and after the'
  write(6,'(a)') '                     trajectory was cleaned. The histgram files'
  write(6,'(a)') '                     are named:'
  write(6,'(a)') '                     <indexfile>_hist_<modelabel>     (before -rem)'
  write(6,'(a)') '                     <indexfile>_hist_<modelabel>_chk (after -rem) '
  write(6,*)
  write(6,'(a)') ' -hist2            : Write 2D histogram files of the distribution'
  write(6,'(a)') '                     of sampling points. If -rem or -rem2 is set,'
  write(6,'(a)') '                     histograms will be made before and after the'
  write(6,'(a)') '                     trajectory was cleaned. The histgram files'
  write(6,'(a)') '                     are named:'
  write(6,'(a)') '                     <indexfile>_hist2d_<modelabel1>_<modelabel2>     (before -rem)'
  write(6,'(a)') '                     <indexfile>_hist2d_<modelabel1>_<modelabel2>_chk (after -rem) '
  write(6,*)
  write(6,'(a)') ' -cov <n>          : Write the normalized covariance matrix of the'
  write(6,'(a)') '                     sampling points to file. <n> is the exponent'
  write(6,'(a)') '                     of the variables, i.e. cov(q_i^n, q_j^n) will'
  write(6,'(a)') '                     be calculated. If -rem or rem2 is is set'
  write(6,'(a)') '                     the matrix is calculated before and after'
  write(6,'(a)') '                     the trajectory was cleaned. The files are named:'
  write(6,'(a)') '                     <indexfile>_covariance<n>     (before -rem)'
  write(6,'(a)') '                     <indexfile>_covariance<n>_chk (after -rem).'
  write(6,'(a)') '                     Also creates a sorted list of covariances.'
  write(6,'(a)') '                     and a file with means and variances'
  write(6,*)



end subroutine chksampling_wrhelptext
