! **********************************************************************
! *                                                                    *
! *                               CHNPOT                               *
! *                                                                    *
! * This analyse program inperpolates natpot terms from one grid to    *
! * another by cubic (bicubic) spline interpolation or angular fit.    *
! *                                                                    *
! * SS 10.02,  Improved version 12.03, cpd added 08.21 HDM             *
! *                                                                    *
! **********************************************************************

      module chnpotmod

use decimal, only: long,dop
use global, only: c1,c5 

      implicit none

!-------------------------------------------------------------------------
! ldiffer : modes have different grids
! intmode : interpolation method
! angmode : type of angular interpolation
! method  : interpolation method in characters
! mmax    : parameter for angular interpolation
!-------------------------------------------------------------------------

      integer(long), save                              :: lnatpot,lndvr,ldvrnew
      integer(long), dimension(:), allocatable, save   :: mmax


      integer(long), save                            :: idvr1 = 7 ! old DVR file
      integer(long), save                            :: idvr2 = 8 ! new DVR file

      integer(long), save                            :: mrcoeff_2,mrort2, &
                                                        dimmodc2,coeffdim_2, &
                                                        ortdim2,evecdim2, &
                                                        evaldim2,maxgdim_2, &
                                                        mroned2,ndof2
      integer(long), dimension(:), allocatable, save :: zort2,zevec2, &
                                                        gdim2,zmode2,basis2, &
                                                        mgdim,intmode,angmode, &
                                                        invfac,enomax,enomin
      integer(long), dimension(:,:), allocatable, save :: ipbaspar2

      real(dop), dimension(:), allocatable, save         :: period,dfac
      real(dop), dimension(:,:), allocatable, save       :: xend2
      real(dop), dimension(:,:), allocatable, save       :: rpbaspar2

      logical(kind=4), save                              :: lrddvr
      logical(kind=4), save                              :: lfitsec
      logical(kind=4), dimension(:), allocatable, save   :: ldiffer,msym,ldvr2

      character(len=20), dimension(:), allocatable, save :: method
      character(len=5), dimension(:), allocatable, save  :: symstring
      character(len=c5), save                            :: dvrfilenew
      character(len=c1), dimension(:), allocatable, save :: modelabel2

      end module chnpotmod


      program chnpot

use global 
use maxv
use pglobal 
use pdefaultmod
use globmemmod
use memdimmod
use dvrdatmod
use griddatmod 
use operdef 
use runpfitmod 
use combined 
use showpotmod 
use timing 
use channels 
use chnpotmod 
use versions
use logdat
use compdatmod
use compdat, only: savecsys,getcsys

use rddvrmod
use logdvrmod
use zeigdvrmod
use rdnatpot
use fitinmod
use wrfitmod
use directutil, only: rdcpd
use einpbasmod, only: einpbas
use vpotpars, only: readvpotpars

use op1lib, only: cpvxi,cpvxd,cpvxl,cpmxi,cpmxd,cptxd1,cpvxi2,&
                  zerovxi,zerovxl,cuttxd1
use op2lib, only: swapsxxi,swapsxxd
use mmlib, only: qqxtdd,qdxxdd,mmxtdd,mmtxdd

      implicit none

      integer   dynbytes(2),realsize
      integer   t1,t2,ilbl,i
      real*8    fcputime,fsystime,tcpu,tsys,ttot,totmem
      external  fcputime,fsystime
      character buffer*100, actual*80, filename*(c5)
      real*8 ,allocatable :: mr2(:)
      logical,allocatable :: loned(:)

      integer    f,g,m,f1,f2,n,n1,idum
      integer    chkdvr
      real*8     grid1,grid2
      logical    ldiff,ldum
      character*(c2) label
      character*(c5) opts
      real*8     hfmndx
      complex(dop), dimension(1) :: zdum,zdum1,zdum2,zdum3
      real(dop), dimension(1) :: ddum,ddum1,ddum2,ddum3
      integer(long), dimension(1) :: idum1,idum2,idum3

! --- READ INPUT DATA ---

      call fpstartup(cstartup)
      call default
      call pdefault
      call chnpotinput0

!-----------------------------------------------------------------------
! Build new dvr file if required
!-----------------------------------------------------------------------
      if (.not.lrddvr) then
         call fzeit(actual)
         i = index(actual,':')
         actual=actual(i-9:i+6)
         filename=dvrfilenew(1:ldvrnew)
         write(ilog,'(/,75a1)')  ('#',i=1,75)
         write(ilog,'(/,a)') ' Creating NEW DVR file: '//filename(1:ldvrnew)
         write(ilog,'(57x,a)')  actual(1:15)
         write(ilog,'(75a1,/)')  ('#',i=1,75)
         dname=name
         dlaenge=laenge
         idvr = idvr2
         call zeigdvr
         call memdvr
         call rundvr
      else
         filename=dvrfilenew(1:ldvrnew)
         write(ilog,'(/,75a1)')  ('#',i=1,75)
         write(ilog,'(/,a)') ' NEW DVR file is read from : '//filename(1:ldvrnew)
         write(ilog,'(/,75a1,/)')  ('#',i=1,75)
      endif

!-----------------------------------------------------------------------
! allocate memory for modules used
!-----------------------------------------------------------------------
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      call alloc_combined
      call alloc_runpfitmod
      call alloc_chnpot
      call alloc_compdat

      call chnpotinput

!-----------------------------------------------------------------------
! set no. of comparison data sets to allow switching between data
!-----------------------------------------------------------------------
      ncomp=2

! --- COMPUTE ARRAY ADDRESSES ---

      call nallocate

!-----------------------------------------------------------------------
      dynbytes(1) = realsize()   *(mcdim/8+1)  ! Here for mr2
      dynbytes(2) = realsize()   *(mrdim/8+1)
      ilbl=0
      totmem=0.d0
      do i=1,2
         totmem=totmem+dynbytes(i)
         if( dynbytes(i).lt.0 ) then
            ilbl=1
            totmem=totmem+4.294967296d+9
         endif
      enddo
      totmem=totmem*8.d0

      call wrversion(ilog)
      write(ilog,'(a)') '  *** Memory allocated dynamically ***'
      write(ilog,'(2(a,i12,/),a,f16.0,a,f11.2,a)')  &
       ' real*8     :', mcdim,                      &
       ' real*8     :', mrdim,                      &
       ' Total   :',totmem,' Bytes ,',totmem/1048576.d0,' MB'
      write(ilog,'(80a1,/)') ('-',i=1,80)
      call myflush(ilog)

      if(ilbl.gt.0) then
         routine='chnpot'
         write(message,'(3a,f9.1,a)')                    &
           'Integer overflow in memory assignment. ',    &
           'You are asking for too much memory.    ',    &
           'Total Memory asked for =',totmem/1048576.d0,' MB'
         call errormsg
      endif

      allocate(mr2(mcdim),mr(mrdim),mi(midim))

      ldiff=.false.

! --- READ OLD AND NEW GRIDS ---

! -- read new grid from new dvr file
      call getcsys(2)
      dvrdata(:) = .false.
      dvrdata(1)=.true.     ! only read ort
      chkdvr=2              ! read info
      idvr = idvr2
      dname = ' '
      dlaenge = ldvrnew-4
      dname(1:dlaenge)=dvrfilenew(1:dlaenge)
      call rddvr(mr2(mrort2),ddum,ddum1,ddum2,zdum, &
           zdum1,idum1,zdum2,zdum3,idum2,idum3,ddum3, &
           chkdvr)
      close(idvr)


! -- read old grid from old dvr file
      idvr = idvr1
      call getcsys(1)
      dvrdata(:) = .false.
      dvrdata(1)=.true.     ! only read ort
      chkdvr=2              ! read info
      idvr = idvr1
      dname = ' '
      dlaenge = lndvr-4
      dname(1:dlaenge)=dvrfile(1:dlaenge)
      call rddvr(mr(mrort),ddum,ddum1,ddum2,zdum, &
           zdum1,idum1,zdum2,zdum3,idum2,idum3,ddum3, &
           chkdvr)
      close(idvr)

! -- inverse the grid order (to have an increasing order), if necessary
      call invgrid(mr(mrort),mr2(mrort2))

! --- READ NATPOT TERMS FROM NATPOT FILE ---
      do f=1,ndof
         loned(f)=onedipot(f) .gt. 0
      enddo

      if (lcpd) then
         call rdcpd(inpot,loned,mr(mroned),mr(mrevec),cpdrank,mi)
      else
         call fitin(inpot,loned,mr(mroned),mr(mrevec),mr(mrcoeff))
      endif
      call readvpotpars(inpot,npotfile(1:lnatpot),progver,label,opts,  &
                        lsp,ldum,idum,pesmin,pesmax,                   &
                        lcspot,jtot,jbf,csmass,.false.)
      hoplab(1)=label
      if (opts(1:1) .ne. ' ') then
         npesopt=1
         nopts(1)=1
         zpesopts(1,1)=index(opts,' ')-1
         pesopts(1)=opts(1:zpesopts(1,1))
      endif
      close(inpot)

! --- FIND DOFS WHICH HAVE DIFFERENT GRID POINTS ---

      do m=1,nmode
         ldiffer(m)=.false.
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            if (gdim(f) .eq. gdim2(f)) then
               do g=1,gdim(f)
                  grid1=mr(mrort+zort(f)-1+g-1)
                  grid2=mr2(mrort2+zort2(f)-1+g-1)
                  if (abs(grid1-grid2) .gt. 1.d-8) then
                     ldiffer(m)=.true.
                     go to 100
                  endif
               enddo
            else
               if (gdim(f) .ge. 4) then
                  ldiffer(m)=.true.
               else
                  routine='Receiver (chnpot)'
                  message='Not enough grid points for '//  &
                          'interpolation for dof '//modelabel(f)
                  call errormsg
               endif
            endif
         enddo
 100     continue
         ldiff = ldiff.or.ldiffer(m)
      enddo

      if(ldiff.and.lfitsec) then
         routine='Receiver (chnpot)'
         message='No FIT-SECTION in input file'
         call errormsg
      endif

! --- DO A CHECK OF PARAMETERS ---
      ! This section probably is messed up. Needs check!
      do m=1,nmode
         if (ldiffer(m)) then
            if (nspfdof(m) .gt. 1) then
               do n=1,nspfdof(m)
                  do n1=n,nspfdof(m)
                     f1=spfdof(n,m)
                     f2=spfdof(n1,m)
                     if (intmode(f1).eq.0 .or. intmode(f2).eq.0) then
                        routine='Receiver (chnpot)'
                        message='Interpolation method for combined '// &
                             modelabel(f1)//'and '//modelabel(f2)//    &
                             ' mode not defined.'
                        call errormsg
                     endif
                  end do
               end do
            else
               f1=spfdof(1,m)
               if (intmode(f1).eq.0) then
                  routine='Receiver (chnpot)'
                  message='Interpolation method for dof '// &
                       modelabel(f1)//'not defined.'
                  call errormsg
               endif
            endif
         else
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               mmax(f)=0
            enddo
         endif
      enddo

! --- WRITE FIT INFORMATION TO LOG FILE ---

      write(ilog,'(/a/)') '############################# INTERPOLATION ' &
                        //'###############################'
      write(ilog,'(a)') ' mode               method           '// &
                        '   sym          M           Period'
      write(ilog,'(77a)') ('-',i=1,77)
      do m=1,nmode
         if (ldiffer(m)) then
            f1=spfdof(1,m)
            if (nspfdof(m) .eq. 1) then
               if (intmode(f1).eq.1) then
                  write(ilog,'(1x,a8,9x,a18)') modelabel(f1),method(f1)
               else
                  write(ilog,'(1x,a8,9x,a18,3x,a5,i10,9x,f8.4)') &
                            modelabel(f1),method(f1),symstring(f1), &
                            mmax(f1),period(f1)
               endif
            else
               do n=1,nspfdof(m)
                  f2=spfdof(n,m)
                  if (intmode(f1).eq.1) then
                     write(ilog,'(1x,a8,a,a8,a18)') modelabel(f1),',', &
                          modelabel(f2),'bi'//method(f1)
                  else
                     write(ilog,&
               '(1x,a8,a,a8,a7,a,a7,4x,a5,a,a5,i5,a,i4,2x,f8.4,a,f8.4)') &
                          modelabel(f1),',',modelabel(f2),               &
                          method(f1),',',method(f2),                     &
                          symstring(f1),',',symstring(f2),               &
                          mmax(f1),',',mmax(f2),                         &
                          period(f1),',',period(f2)
                  endif
               end do
            endif
         else
            do n=1,nspfdof(m)
               write(ilog,'(1x,a8,9x,a)') modelabel(spfdof(1,m)), &
                    'No fit for this DOF. Grids are identical.'
            end do
         endif
      enddo
      write(ilog,'(77a)') ('-',i=1,77)
      write(ilog,'(/77a)') ('*',i=1,77)

! --- DO A CHECK OF GRID BOUNDS FOR NOT-ANGULAR DOFS ---
      do f=1,ndof
         if (basis(f).eq.1 .or. basis(f).eq.3 .or. basis(f).eq.4 .or. &
             basis(f).eq.11 .or. basis(f).eq.12 .or. basis(f).eq.13   &
             .or. (basis(f).ge.15 .and. basis(f).le.18) .or.          &
             basis(f).ge.20) then
             if (intmode(f).eq.1) then
                hfmndx = (max(xend(1,f),xend(2,f)) &
                         -min(xend(1,f),xend(2,f)))/(2*(gdim(f)-1))
                if(min(xend(1,f),xend(2,f))-min(xend2(1,f),xend2(2,f)) &
                   .gt. hfmndx .or.                                    &
                   max(xend2(1,f),xend2(2,f))-max(xend(1,f),xend(2,f)) &
                   .gt. hfmndx) then
                   write(6, &
                      '("min1,-min2:",2f11.5,5x,"max1,-max2:",2f11.5)') &
                   min(xend2(1,f),xend2(2,f)),min(xend(1,f),xend(2,f)), &
                   max(xend2(1,f),xend2(2,f)),max(xend(1,f),xend(2,f))
                   routine='Receiver (chnpot)'
                   message='New grid for dof '//modelabel(f)// &
                        ' should be within old grid.'
                   routine='receiver (chnpot)'
                   call errormsg
                endif
             endif
         endif
      enddo

! --- DO INTERPOLATION TO THE NEW GRID ---

      call intnat(mr(mrcoeff),mr2(mrcoeff_2),mr(mrevec),mr2(mrevec2), &
                  mr(mroned),mr2(mroned2),mr(mrort),mr2(mrort2),      &
                  loned,mr(mrworkr))

! --- REINVERSE THE GRID ORDER, IF IT WAS INVERSED
      call invgrid_back(mr(mrort),mr2(mrort2))

! --- RESET PARAMETERS FOR NEW GRID

      call cpvxi(basis2,basis,maxdim)
      call cpmxd(xend2,xend,2,maxdim)
      call cpvxi(psubdim2,psubdim,maxdim)
      call cpvxi(zmode2,zmode,maxdim)
      call cpvxi(zevec2,zevec,maxdim)
      call cpvxi2(gdim2,gdim,maxdim)
      call cpvxi2(zort2,zort,maxdim)
      call cpmxi(ipbaspar2,ipbaspar,mbaspar,maxdim)
      call cpmxd(rpbaspar2,rpbaspar,mbaspar,maxdim)
      call cpvxl(ldvr2,ldvr,maxdim)
      coeffdim=coeffdim_2
      evecdim=evecdim2
      ortdim=ortdim2
      dimmodc=dimmodc2

! --- WRITE NEW NATPOT FILE
      lsw=.false.
      if(lcpd) then
         write(6,'(2a)') ' Writing : ',name(1:laenge)//'/cpd'
      else
         write(6,'(2a)') ' Writing : ',name(1:laenge)//'/natpot'
      endif

      if(.not.lcpd) potdim(modc)=psubdim(modc)  ! Only for cppinput
      call wrfit(mr2(mrcoeff_2),mr2(mrevec2),mr2(mroned2),mr(mrwght),&
           mr(mrwghti))

! --- GET TIMIMG AND WRITE IT TO THE "LOG" FILE ---

      tcpu = fcputime(0)
      tsys = fsystime(0)
      ttot = tcpu+tsys
      t1   = int(ttot/3600)
      ttot = ttot - 3600*t1
      t2   = int(ttot/60)
      ttot = ttot - 60*t2

      write(ilog,'(/a,/)') '***** Overall Timing *****'
      write(ilog,'(a,1x,f12.2)') 'Total cpu time     [s] : ',tcpu
      write(ilog,'(a,1x,f12.2)') 'Total system time  [s] : ',tsys
      write(ilog,900) 'Total time     [h:m:s] : ',t1,t2,ttot
!      write(6   ,900) 'Total time     [h:m:s] : ',t1,t2,ttot
  900 format( /a,i4,' :',i3,' :',f5.1)

      call fpstartup(buffer)
      write(ilog,'(/a)') ' -- chnpot terminated normally --'
      write(ilog,'(a80)') buffer
!      write(6,'(80a)') ('*',s=1,80)

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_combined
      call dealloc_runpfitmod
      call dealloc_chnpot
      close(ilog)

      stop     ! end of main program

     contains 
! Subroutines

!#######################################################################
!  Subroutine Nallocate
!
!  Compute pointers to the addresses of the integer and real
!  arrays nedded for chnpot program
!#######################################################################

      subroutine nallocate

      implicit none

      integer f,f1,f2,m,zeig,zeig2,mmd,mm,wrkdim,wrkdim1
      integer chkgrd,chkdvr,n,fdvr(maxdim),mdvr(maxdim),mmode(maxdim)
      integer nmode1,ndof1,subdim1(maxdim),pntbas(maxdim),ierr
      logical lerr


! --- READ DVR INFORMATION FROM THE NEW DVR FILE ---
      idvr=idvr2
      open(unit=idvr,file=dvrfilenew(1:ldvrnew),form='unformatted', &
           status='old',iostat=ierr)
      if (ierr.ne.0) then
         routine='nallocate'
         message='Error reading dvr file: '//dvrfilenew(1:ldvrnew)
         call errormsg
      endif

      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      if (lerr) then
         routine='nallocate'
         message='Error reading DVR file '//dvrfilenew(1:ldvrnew)
         call errormsg
      endif
! -- log new basis information
      write(ilog,'(a)') '######################## NEW PRIMITIVE BASIS '  &
                        //'SECTION ########################'
      call logdvr(ilog)
      call myflush(ilog)

! --- copy data needed  (2 denotes the new set)
      call savecsys(2)

! store key parameters 
      zort2 = zort
      ortdim2 = ortdim
      ipbaspar2 = ipbaspar
      rpbaspar2 = rpbaspar
      gdim2 = gdim

! --- READ DVR INFORMATION FROM OLD DVR FILE ---
      idvr=idvr1
      open(unit=idvr,file=dvrfile(1:lndvr),form='unformatted', &
           status='old',iostat=ierr)
      if (ierr.ne.0) then
         routine='nallocate'
         message='Error reading dvr file: '//dvrfile(1:lndvr)
         call errormsg
      endif

      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      if (lerr) then
         routine='nallocate'
         message='Error reading DVR file '//dvrfile(1:lndvr)
         call errormsg
      endif
! -- log old basis information
      write(ilog,'(/a)') '######################## OLD PRIMITIVE BASIS' &
                        //' SECTION ########################'
      call logdvr(ilog)
      call myflush(ilog)
      write(ilog,*)

! --- copy data needed  (1 denotes the new set)
      call savecsys(1)

! --- check number of dofs
      if(ndof2.ne.ndof) then
         write(6,  '(/a,i4,a,i4)') ' old ndof',ndof,',  new ndof',ndof2
         write(ilog,'(a,i4,a,i4)') ' old ndof',ndof,',  new ndof',ndof2
         routine='nallocate'
         message='Inconsistent numbers of degrees of freedom'
         call errormsg
      endif

! --- READ NATPOT INFORMATION FROM OLD NATPOT FILE ---
! --- DO A CHECK AGAINST OLD DVR DATA ---

      open(unit=inpot,file=npotfile(1:lnatpot),form='unformatted', &
            status='old',iostat=ierr)
      if (ierr.ne.0) then
         routine='nallocate'
         message='could not open '//npotfile(1:lnatpot)
         call errormsg
      endif
      chkdvr=2
      chkgrd=1
      if(lcpd) then
         call cpdinfo(inpot,lerr,ndof1,fdvr,      &
              chkdvr,chkgrd,1,mdvr,mmode,cpdrank, &
              nspfdof,spfdof,nmode1,subdim1,lonedflag,pntbas)
      else
         call npotinfo(inpot,lerr,modc,lonedflag,ndof1,fdvr,   &
              chkdvr,chkgrd,1,mdvr,mmode,nspfdof,spfdof,nmode1,&
              subdim1,pntbas)
      endif

! --- define some natpot parameters
      if(lcpd) then
         modc=0
         modcdim=0
         do m=1,nmode
            potdim(m)=cpdrank
            pdim(m)=cpdrank
         enddo
      else
         do m=1,nmode
            potdim(m)=pdim(m)
         enddo
      endif


      if(.not.lcpd) call init_cmode

      do m=1,nmode
         zeig=1
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            zeig=zeig*gdim2(f)
         enddo
         psubdim2(m)=zeig
      enddo

! --- reset the new modelabels
      do f=1,ndof
            modelabel(f)=modelabel2(f)
      enddo

!-----------------------------------------------------------------
! Set pointers to different arrays
!-----------------------------------------------------------------
! -- Set pointers for matrices needed for potfit
      coeffdim  = 1
      coeffdim_2 = 1
      evecdim   = 0
      evecdim2  = 0
      zeig      = 1
      zeig2     = 1
      modcdim   = 1

      zeig  = 1
      zeig2 = 1
      zevec(1) = 1
      zevec2(1)= 1

      do m=1,nmode
         zmode(m) =zeig
         zeig=zeig+psubdim(m)

         zmode2(m)=zeig2
         zeig2=zeig2+psubdim2(m)

         if (m.eq.modc) then
            coeffdim  = coeffdim*psubdim(m)
            coeffdim_2 = coeffdim_2*psubdim2(m)
         else
            coeffdim   = coeffdim*potdim(m)
            coeffdim_2 = coeffdim_2*potdim(m)
            modcdim    = modcdim*potdim(m)
         endif
         if(lcpd) then
            if(m.lt.nmode) then
               zevec(m+1)  = zevec(m)+psubdim(m)
               zevec2(m+1) = zevec2(m)+psubdim2(m)
            endif
            evecdim   = evecdim+psubdim(m)*potdim(m)
            evecdim2  = evecdim2+psubdim2(m)*potdim(m)
         else
            ctdim = modcdim*psubdim(modc)
            zevec(m)  = evecdim+1
            evecdim   = evecdim+psubdim(m)*potdim(m)

            zevec2(m) = evecdim2+1
            evecdim2  = evecdim2+psubdim2(m)*potdim(m)
         endif
      enddo

      evaldim   = zeig-1
      evaldim2  = zeig2-1

      if(lcpd) then
         dimmodc  = 0
         dimmodc2 = 0
         coeffdim = 1
         coeffdim_2 = 1
      else
         dimmodc   = psubdim(modc)
         dimmodc2  = psubdim2(modc)
      endif

! ---
      mcdim=1
      mrdim=1
      midim=cpdrank*npmode+1
      mldim=1
      msdim=1
      mfdim=1

      mrort = mrdim
      mrdim = mrort   + ortdim
      mroned = mrdim
      mrdim = mroned  + ortdim
      mrcoeff = mrdim
      mrdim = mrcoeff + coeffdim
      mrevec = mrdim
      mrdim = mrevec  + evecdim
      mrwght = mrdim
      mrdim = mrwght  + weightdim
      mrwghti = mrdim
      mrdim = mrwghti + weightdim
      oneddim=ortdim

! --- Pointer for mr2. mcdim is (mis)-used to give the length of mr2.
      mrort2 = mcdim
      mcdim = mrort2 + ortdim2
      mroned2 = mcdim
      mcdim = mroned2 + ortdim2
      mrcoeff_2 = mcdim
      mcdim = mrcoeff_2  + coeffdim_2
      mrevec2 = mcdim
      mcdim = mrevec2   + evecdim2

! --- real workspace for spline, eno and angular interpolation routines--

      wrkdim=0
      do m=1,nmode
         f1=spfdof(1,m)
         if (intmode(f1).eq.1) then
            !splines
            if (nspfdof(m).eq.1) then
               wrkdim = max(wrkdim,3*gdim(f1))
            elseif (nspfdof(m).eq.2) then
               f2=spfdof(2,m)
               mmd=max(gdim(f1),gdim(f2))
               wrkdim = max(wrkdim,2*gdim(f1)*gdim(f2)+3*mmd)
            else
               mm=max(psubdim(m),psubdim2(m))
               do n=1,nspfdof(m)
                  f1=spfdof(n,m)
                  wrkdim = max(wrkdim,3*gdim(f1)+2*mm,3*gdim2(f1)+2*mm)
               end do
            endif
         elseif (intmode(f1).eq.2) then
            ! angular interpolation
            if (nspfdof(m).eq.1) then
               wrkdim1 = mmax(f1)*(2*gdim(f1)+gdim2(f1)) +  &
                         3*mmax(f1)**2 + 2*mmax(f1)
               wrkdim=max(wrkdim,wrkdim1)
            else
               f2=spfdof(2,m)
               mm=max(mmax(f1),mmax(f2))
               wrkdim1 = mmax(f1)*(2*gdim(f1)+gdim2(f1)) +  &
                         mmax(f2)*(2*gdim(f2)+gdim2(f2)) +  &
                         mmax(f1)*mmax(f2) + 3*mm**2 + mm
               wrkdim=max(wrkdim,wrkdim1)
            endif
         elseif (intmode(f1).eq.3) then
            ! ENO
            if (nspfdof(m).eq.1) then
               wrkdim= max(wrkdim,gdim(f1)**2+gdim(f1))
            else
               message='2D ENO interpolation not implemented'
               call errormsg
            endif
         endif
      enddo

      workrdim = wrkdim
      mrworkr = mrdim
      mrdim = mrworkr + workrdim
      write(ilog,'(a,i11,es14.3)') 'workrdim =',wrkdim,dble(wrkdim)
      write(ilog,'(a,i11,es14.3)') 'mr2dim   =', mcdim,dble(mcdim)
      write(ilog,'(a,i11,es14.3)') 'mrdim    =', mrdim,dble(mrdim)

      return
      end  subroutine nallocate


!#######################################################################
!  Subroutine Intnat
!
!  Interpolation
!#######################################################################

      subroutine intnat(coeff,coeff2,evec,evec2,oned,oned2,ort,ort2, &
                        loned,workr)

      implicit none

      integer m,mm,z1,z2,z3,z4,z5,z6,z7,f,f1,f2,nt,n,vdim,ndim, &
              tsubdim,i,j,gg1,gg2
      real*8 coeff(coeffdim),coeff2(coeffdim_2),oned(ortdim),          &
             oned2(ortdim2),ort(ortdim),ort2(ortdim2),evec(evecdim),   &
             evec2(evecdim2),workr(workrdim)
      logical loned(ndof)


      routine="intnat"

      do m=1,npmode
        if(lcpd) then
            gg1=zevec(nmode)+psubdim(nmode)-1
            gg2=zevec2(nmode)+psubdim2(nmode)-1
         else
            gg1=psubdim(m)
            gg2=psubdim2(m)
         endif
         if (m.eq.modc) then    ! this is the contracted mode
            if (ldiffer(m)) then
! .......................1D interpolation for coefficients..........
               if (nspfdof(m) .eq. 1) then
                  f=spfdof(1,m)
                  if (intmode(f) .eq. 1) then
                     z1=1+gdim(f)
                     z2=z1+gdim(f)
                     call intspl1d(coeff,coeff2,dimbef,gdim(f),gdim2(f), &
                          dimaft,ort(zort(f)),ort2(zort2(f)),            &
                          workr,workr(z1),workr(z2),gg1,gg2)
                  elseif (intmode(f) .eq. 2) then
                     z1=1+gdim(f)*mmax(f)
                     z2=z1+gdim(f)*mmax(f)
                     z3=z2+gdim2(f)*mmax(f)
                     z4=z3+mmax(f)
                     call intang1d(coeff,coeff2,dimbef,gdim(f),gdim2(f), &
                          dimaft,ort(zort(f)),ort2(zort2(f)),workr,      &
                          workr(z1),workr(z2),workr(z3),workr(z4),       &
                          mmax(f),angmode(f),msym(f),period(f),gg1,gg2)
                  elseif (intmode(f) .eq. 3) then
                     z1=1+gdim(f)
                     call inteno1d(coeff,coeff2,dimbef,gdim(f),gdim2(f), &
                          dimaft,ort(zort(f)),ort2(zort2(f)),workr,workr(z1),&
                          enomax(f),enomin(f),dfac(f),gg1,gg2)
                  endif
! .......................2D interpolation for coefficients .........
               elseif(nspfdof(m) .eq. 2) then
                  f1=spfdof(1,m)
                  f2=spfdof(2,m)
                  if (intmode(f1) .eq. 1) then
                     nt=max(gdim(f1),gdim(f2))
                     z1=1+gdim(f1)*gdim(f2)
                     z2=z1+gdim(f1)*gdim(f2)
                     z3=z2+nt
                     z4=z3+nt
                     call intspl2d(coeff,coeff2,dimbef,                    &
                                gdim(f1),gdim(f2),gdim2(f1),gdim2(f2),     &
                                dimaft,ort(zort(f1)),ort(zort(f2)),        &
                                ort2(zort2(f1)),ort2(zort2(f2)),workr,     &
                                workr(z1),workr(z2),workr(z3),             &
                                workr(z4),nt,gg1,gg2)
                  else
                    mm=max(mmax(f1),mmax(f2))
                    z1=1+gdim(f1)*mmax(f1)
                    z2=z1+gdim(f2)*mmax(f2)
                    z3=z2+gdim2(f1)*mmax(f1)
                    z4=z3+gdim2(f2)*mmax(f2)
                    z5=z4+gdim(f1)*mmax(f1)
                    z6=z5+gdim(f2)*mmax(f2)
                    z7=z6+mmax(f1)*mmax(f2)
                    call intang2d(coeff,coeff2,dimbef,                    &
                                gdim(f1),gdim(f2),gdim2(f1),gdim2(f2),    &
                                dimaft,ort(zort(f1)),ort(zort(f2)),       &
                                ort2(zort2(f1)),ort2(zort2(f2)),workr,    &
                                workr(z1),workr(z2),workr(z3),            &
                                workr(z4),workr(z5),workr(z6),            &
                                workr(z7),mmax(f1),mmax(f2),angmode(f1),  &
                                angmode(f2),mm,msym(f1),msym(f2),         &
                                period(f1),period(f2),gg1,gg2)
                  endif
               else if (nspfdof(m) .ge. 3) then

! .......................ND interpolation for coefficients .........

                  !pointers
                  mm=0
                  do f=1,nspfdof(m)
                     mm=max(mm,gdim(f))
                  end do
                  z1=1+mm
                  z2=z1+mm
                  z5=z2+mm
                  z6=z5+max(psubdim(m),psubdim2(m))

                  do i=1,dimbef !loop over all natorbs
                     do j=1,dimaft

                        ! we have intermediate mixtures of grids
                        ! so we use tsubdim for checks
                        tsubdim=psubdim(m)

                        ! build dims for the primitive grid
                        vdim=1
                        ndim=1
                        do n=1,nspfdof(m)
                           f=spfdof(n,m)
                           ndim=ndim*gdim(f)
                        end do


                        call cuttxd1(coeff,workr(z5),dimbef,dimmodc,dimaft,i,j)

                        do n=1,nspfdof(m) !loop over all DOF
                           f=spfdof(n,m)
                           ndim=ndim/gdim(f)


                           call intspl1d(workr(z5),workr(z6),  &
                                vdim,gdim(f),gdim2(f),         &
                                ndim,ort(zort(f)),ort2(zort2(f)),  &
                                workr,workr(z1),workr(z2),     &
                                gdim(f),gdim2(f))

                           vdim=vdim*gdim2(f)
                           tsubdim=tsubdim/gdim(f)*gdim2(f)

                           call cpvxd(workr(z6),workr(z5),tsubdim)

                        end do


                        if (psubdim2(m).ne. tsubdim) then
                           message='Inconsistent dimensions.'
                           call errormsg
                        end if

                        call cptxd1(workr(z5),coeff2,dimbef,psubdim2(m),dimaft,i,j)

                     end do
                  end do

               endif
            else
               call cpvxd(coeff,coeff2,coeffdim)
            endif
         else   ! these are the other, not contracted, modes
            if (ldiffer(m)) then
! ........................1D interpolation for natpot terms.............
               if (nspfdof(m) .eq. 1) then
                  f=spfdof(1,m)
                  if (intmode(f) .eq. 1) then
                     z1=1+gdim(f)
                     z2=z1+gdim(f)
                     call intspl1d(evec(zevec(m)),evec2(zevec2(m)),1, &
                                gdim(f),gdim2(f),potdim(m),           &
                                ort(zort(f)),ort2(zort2(f)),          &
                                workr,workr(z1),workr(z2),gg1,gg2)
                  elseif (intmode(f) .eq. 2) then
                     z1=1+gdim(f)*mmax(f)
                     z2=z1+gdim(f)*mmax(f)
                     z3=z2+gdim2(f)*mmax(f)
                     z4=z3+mmax(f)
                     call intang1d(evec(zevec(m)),evec2(zevec2(m)),1, &
                          gdim(f),gdim2(f),potdim(m),                 &
                          ort(zort(f)),ort2(zort2(f)),workr,          &
                          workr(z1),workr(z2),workr(z3),workr(z4),    &
                          mmax(f),angmode(f),msym(f),period(f),gg1,gg2)
                  elseif (intmode(f) .eq. 3) then
                     z1=1+gdim(f)
                     call inteno1d(evec(zevec(m)),evec2(zevec2(m)),1, &
                            gdim(f),gdim2(f),potdim(m),               &
                            ort(zort(f)),ort2(zort2(f)),              &
                            workr,workr(z1),enomax(f),enomin(f),      &
                            dfac(f),gg1,gg2)
                  endif
! .........................2D interpolation for natpot terms.............
               else if (nspfdof(m) .eq. 2) then
                  f1=spfdof(1,m)
                  f2=spfdof(2,m)
                  if (intmode(f1) .eq. 1) then
                     nt=max(gdim(f1),gdim(f2))
                     z1=1+gdim(f1)*gdim(f2)
                     z2=z1+gdim(f1)*gdim(f2)
                     z3=z2+nt
                     z4=z3+nt
                     call intspl2d(evec(zevec(m)),evec2(zevec2(m)),1,  &
                                gdim(f1),gdim(f2),gdim2(f1),gdim2(f2), &
                                potdim(m),ort(zort(f1)),ort(zort(f2)), &
                                ort2(zort2(f1)),ort2(zort2(f2)),workr, &
                                workr(z1),workr(z2),workr(z3),         &
                                workr(z4),nt,gg1,gg2)
                  else
                    mm=max(mmax(f1),mmax(f2))
                    z1=1+gdim(f1)*mmax(f1)
                    z2=z1+gdim(f2)*mmax(f2)
                    z3=z2+gdim2(f1)*mmax(f1)
                    z4=z3+gdim2(f2)*mmax(f2)
                    z5=z4+gdim(f1)*mmax(f1)
                    z6=z5+gdim(f2)*mmax(f2)
                    z7=z6+mmax(f1)*mmax(f2)
                    call intang2d(evec(zevec(m)),evec2(zevec2(m)),1,   &
                                gdim(f1),gdim(f2),gdim2(f1),gdim2(f2), &
                                potdim(m),ort(zort(f1)),ort(zort(f2)), &
                                ort2(zort2(f1)),ort2(zort2(f2)),workr, &
                                workr(z1),workr(z2),workr(z3),         &
                                workr(z4),workr(z5),workr(z6),         &
                                workr(z7),mmax(f1),mmax(f2),angmode(f1), &
                                angmode(f2),mm,msym(f1),msym(f2),      &
                                period(f1),period(f2),gg1,gg2)
                  endif
               else if (nspfdof(m) .ge. 3) then
! .........................ND interpolation for natpot terms.............


                  !pointers
                  mm=0
                  do f=1,nspfdof(m)
                     mm=max(mm,gdim(f))
                  end do
                  z1=1+mm
                  z2=z1+mm
                  z5=z2+mm
                  z6=z5+max(psubdim(m),psubdim2(m))
                  z3=zevec(m)
                  z4=zevec2(m)

                  do mm=1,potdim(m) !loop over all npots

                     ! we have intermediate mixtures of grids
                     ! so we use tsubdim for checks
                     tsubdim=psubdim(m)

                     ! build dims for the primitive grid
                     vdim=1
                     ndim=1
                     do n=1,nspfdof(m)
                        f=spfdof(n,m)
                        ndim=ndim*gdim(f)
                     end do

                     call cpvxd(evec(z3),workr(z5),psubdim(m))

                     do n=1,nspfdof(m) !loop over all DOF
                        f=spfdof(n,m)
                        ndim=ndim/gdim(f)

                        call intspl1d(workr(z5),workr(z6),      &
                             vdim,gdim(f),gdim2(f),             &
                             ndim,ort(zort(f)),ort2(zort2(f)),  &
                             workr,workr(z1),workr(z2),gdim(f),gdim2(f))

                        vdim=vdim*gdim2(f)
                        tsubdim=tsubdim/gdim(f)*gdim2(f)

                        call cpvxd(workr(z6),workr(z5),tsubdim)

                     end do

                     if (psubdim2(m).ne. tsubdim) then
                        message='Inconsistent dimensions.'
                        call errormsg
                     end if

                     call cpvxd(workr(z5),evec2(z4),psubdim2(m))
                     z3=z3+gg1
                     z4=z4+gg2

                  end do

               endif
            else  ! just copy the not fitted modes
               if(lcpd) then
                  call cpcpd(evec,zevec(m),evec2,zevec2(m), &
                             psubdim(m),potdim(m),gg1,gg2)
               else
                  call cpvxd(evec(zevec(m)),evec2(zevec2(m)), &
                             psubdim(m)*potdim(m))
               endif
            endif
         endif
      enddo

! .........................Interpolation for oned potentials ..........
      do f=1,ndof
         if (ldiffer(dofspf(f)) ) then
            if (loned(f)) then
               m=dofspf(f)
               if (intmode(f).eq.1) then
                  z1=1+gdim(f)
                  z2=z1+gdim(f)
                  call intspl1d(oned(zort(f)),oned2(zort2(f)),1,gdim(f), &
                                gdim2(f),1,ort(zort(f)),ort2(zort2(f)),  &
                                workr,workr(z1),workr(z2),gg1,gg2)
               elseif(intmode(f) .eq. 2) then
                  z1=1+gdim(f)*mmax(f)
                  z2=z1+gdim(f)*mmax(f)
                  z3=z2+gdim2(f)*mmax(f)
                  z4=z3+mmax(f)
                  call intang1d(oned(zort(f)),oned2(zort2(f)),1,gdim(f), &
                                gdim2(f),1,ort(zort(f)),ort2(zort2(f)),  &
                                workr,workr(z1),workr(z2),workr(z3),     &
                                workr(z4),mmax(f),angmode(f),msym(f),    &
                                period(f),gg1,gg2)
               elseif(intmode(f) .eq. 3) then
                  z1=1+gdim(f)
                  call inteno1d(oned(zort(f)),oned2(zort2(f)),1,gdim(f), &
                        gdim2(f),1,ort(zort(f)),ort2(zort2(f)),          &
                        workr,workr(z1),enomax(f),enomin(f),dfac(f),gg1,gg2)
               endif
            endif
         else
            call cpvxd(oned(zort(f)),oned2(zort2(f)),gdim(f))
         endif
      enddo

      return
      end  subroutine intnat


!#######################################################################
!  Subroutine Intspl1d
!
!  Cubic spline interpolation (1D)
!#######################################################################

      subroutine intspl1d(vec,vec2,v,gdim,gdim2,n,ort,ort2, &
                          y,y2,u,gg1,gg2)

      implicit none

      integer i,j,g,v,gdim,gdim2,n,gg1,gg2
      real*8  vec(v,gg1,n),vec2(v,gg2,n),ort(gdim),ort2(gdim2), &
              y(gdim),y2(gdim),u(gdim),yp1,yp2


      do i=1,n
         do j=1,v
            do g=1,gdim
               y(g)=vec(j,g,i)
            enddo
            yp1=1.d30
            yp2=1.d30
            call splinenm(ort,y,gdim,yp1,yp2,y2,u)
            do g=1,gdim2
               call splintnm(ort,y,y2,gdim,ort2(g),vec2(j,g,i))
            enddo
         enddo
      enddo


      return
      end  subroutine intspl1d




!#######################################################################
!  Subroutine inteno1d
!
!  Essentially non Oscillatory interpolation ENO 1D
!#######################################################################

      subroutine inteno1d(vec,vec2,v,gdim,gdim2,n,ort,ort2, &
                          y,diff,enomax,enomin,dfac,gg1,gg2)

      implicit none

      integer i,j,g,v,gdim,gdim2,n,enomax,enomin,gg1,gg2
      real*8  vec(v,gg1,n),vec2(v,gg2,n),ort(gdim),ort2(gdim2), &
              y(gdim),diff(gdim,gdim),dfac


      do i=1,n
         do j=1,v
            do g=1,gdim
               y(g)=vec(j,g,i)
            enddo
            call fdiff(ort,y,gdim,diff)
            do g=1,gdim2
               call enoi(ort,gdim,diff,ort2(g),enomax,enomin,dfac,&
                    vec2(j,g,i))
            enddo
         enddo
      enddo


      return
      end  subroutine inteno1d


!#######################################################################
!  Subroutine Intspl2d
!
!  Does bicubic spline interpolation (2D)
!#######################################################################

      subroutine intspl2d(vec,vec2,v,gdim11,gdim12,gdim21,gdim22,n, &
                          ort11,ort12,ort21,ort22,y,y2,ytmp,y2tmp,  &
                          u,nt,gg1,gg2)

      implicit none

      integer i,j,g,v,gdim11,gdim12,gdim21,gdim22,n,nt,g1,g2,gg1,gg2
      real*8  vec(v,gg1,n),vec2(v,gg2,n), &
              ort11(gdim11),ort12(gdim12),ort21(gdim21),ort22(gdim22), &
              y(gdim11,gdim12),y2(gdim11,gdim12),ytmp(nt), &
              y2tmp(nt),u(nt)


      do i=1,n
         do j=1,v
            do g1=1,gdim11
               do g2=1,gdim12
                  g=g1+(g2-1)*gdim11
                  y(g1,g2)=vec(j,g,i)
               enddo
            enddo
            call spline2(ort12,y,gdim11,gdim12,y2,ytmp,y2tmp,u,nt)
            do g1=1,gdim21
               do g2=1,gdim22
                  g=g1+(g2-1)*gdim21
                  call splin2nm(ort11,ort12,y,y2,gdim11,gdim12,1.d30,  &
                                1.d30,ort21(g1),ort22(g2),vec2(j,g,i), &
                                ytmp,y2tmp,u,nt)
               enddo
            enddo
         enddo
      enddo

      return
      end  subroutine intspl2d


!#######################################################################
!  Subroutine Intang1d
!
!  1D interpolation for angular dof
!#######################################################################

      subroutine intang1d(vec,vec2,v,gdim,gdim2,n,ort,ort2,a,aa,a2,akg, &
                          work,mmax,angmode,msym,period,gg1,gg2)

      implicit none

      integer i,j,g,v,gdim,gdim2,n,mmax,k,angmode,gg1,gg2
      real*8  vec(v,gg1,n),vec2(v,gg2,n),ort(gdim),ort2(gdim2), &
              a(gdim,mmax),aa(mmax,gdim),a2(gdim2,mmax),        &
              work(mmax+3*mmax**2),akg(mmax),period
      real*8  vvv,eps
      logical msym


      eps = 1.d-8

! --- define A matrix ---
      call amatrix(a,ort,gdim,mmax,angmode,msym,period)

! --- calculate aa = (A^T A)^(-1) A^T
      call ata(a,aa,work,gdim,mmax,eps)


! --- calculate A(x,k) = cos(k*x)/(k^2+1)
      call amatrix(a2,ort2,gdim2,mmax,angmode,msym,period)

! --- calculate V(x)
      do i=1,n
         do j=1,v
! ...........calculate akg(k) = aa(k,g) * V(g)
            do k=1,mmax
               akg(k)=0.d0
               do g=1,gdim
                  akg(k)=akg(k)+aa(k,g)*vec(j,g,i)
               enddo
            enddo
! ...........calculate V(x) = akg(k) * A(x,k)
            do g=1,gdim2
               vvv=0.d0
               do k=1,mmax
                  vvv=vvv+akg(k)*a2(g,k)
               enddo
               vec2(j,g,i)=vvv
            enddo
         enddo
      enddo


      return
      end  subroutine intang1d


!#######################################################################
!  Subroutine Intang2d
!
!  2D interpolation for angular mode
!#######################################################################

      subroutine intang2d(vec,vec2,v,gdim11,gdim12,gdim21,gdim22,n, &
                          ort11,ort12,ort21,ort22,a11,a12,a21,a22,  &
                          aa1,aa2,akg,work,mmax1,mmax2,angmode1,    &
                          angmode2,mm,msym1,msym2,period1,period2,  &
                          gg1,gg2)

      implicit none

      integer i,j,g,v,gdim11,gdim12,gdim21,gdim22,n,mmax1,mmax2,k1,k2, &
              angmode1,angmode2,g1,g2,mm,gg1,gg2
      real*8  vec(v,gg1,n),vec2(v,gg2,n), &
              ort11(gdim11),ort12(gdim12),ort21(gdim21),ort22(gdim22), &
              a11(gdim11,mmax1),a12(gdim12,mmax2),a21(gdim21,mmax1),   &
              a22(gdim22,mmax2),aa1(mmax1,gdim11),aa2(mmax2,gdim12),   &
              akg(mmax1,mmax2),work(mm+3*mm**2),period1,period2
      real*8  vvv,eps
      logical msym1,msym2


      eps = 1.d-8

! --- define A1 matrix ---
      call amatrix(a11,ort11,gdim11,mmax1,angmode1,msym1,period1)

! --- calculate aa1 = (A1^T A1)^(-1) A1^T
      call ata(a11,aa1,work,gdim11,mmax1,eps)


! --- define A2 matrix ---
      call amatrix(a12,ort12,gdim12,mmax2,angmode2,msym2,period2)

! --- calculate aa2 = (A2^T A2)^(-1) A2^T
      call ata(a12,aa2,work,gdim12,mmax2,eps)


! --- calculate A1(x1,k1) = cos(k1*x1)/(k1^2+1)
      call amatrix(a21,ort21,gdim21,mmax1,angmode1,msym1,period1)

! --- calculate A2(x2,k2) = cos(k2*x2)/(k2^2+1)
      call amatrix(a22,ort22,gdim22,mmax2,angmode2,msym2,period2)


! ---  calculate V(x1,x2)
      do i=1,n
         do j=1,v
! .................calculate akg(k1,k2) = aa1(k1,l1) * aa2(k2,l2) * V(l1,l2)
            do k1=1,mmax1
               do k2=1,mmax2
                  akg(k1,k2)=0.d0
                  do g1=1,gdim11
                     do g2=1,gdim12
                        g=g1+(g2-1)*gdim11
                        akg(k1,k2) = akg(k1,k2) + aa1(k1,g1)*aa2(k2,g2)*vec(j,g,i)
                     enddo
                  enddo
               enddo
            enddo
! .................calculate V(x1,x2) = akg(k1,k2) * a1(x1,k1) * a2(x2,k2)
            do g1=1,gdim21
               do g2=1,gdim22
                  vvv=0.d0
                  do k1=1,mmax1
                     do k2=1,mmax2
                        vvv = vvv + akg(k1,k2) * a21(g1,k1) * a22(g2,k2)
                     enddo
                  enddo
                  g=g1+(g2-1)*gdim21
                  vec2(j,g,i)=vvv
               enddo
            enddo
         enddo
      enddo


      return
      end  subroutine intang2d

!#######################################################################
!  Subroutine amatrix
!
!  Define matrix A(l,k) from Eq.(38) in JCP 104, 7974.
!#######################################################################
      subroutine amatrix(a,ort,gdim,mmax,angmode,msym,period)

      implicit none

      integer g,k,gdim,mmax,angmode,j
      real*8  a(gdim,mmax),ort(gdim),period,x,pi
      logical msym

      pi=4.d0*atan(1.d0)

      if (angmode .eq. 1) then
         do g=1,gdim
            x=ort(g)*2.d0*pi/period
            do k=0,mmax-1
               j=k
               if (msym) j=2*k
               a(g,k+1)=cos(j*x)/(j**2.d0+1.d0)
            enddo
         enddo
      elseif (angmode .eq. 2) then
         do g=1,gdim
            x=ort(g)*2.d0*pi/period
            do k=0,mmax-1
               j=k
               if (msym) j=2*k
               a(g,k+1)=sin(j*x)/(j**2.d0+1.d0)
            enddo
         enddo
      else ! angmode=3 i.e. ft
         do g=1,gdim
            x=ort(g)*2.d0*pi/period
            do k=0,mmax-1
               j=k
               if (msym) j=2*k
               if (mod(k,2).eq.0) then
                  a(g,k+1)=cos(j/2.d0*x)/(j**2.d0/4.d0+1.d0)
               else
                  if (msym) j=j+1
                  a(g,k+1)=sin((j+1)/2.d0*x)/((j+1)**2.d0/4.d0+1.d0)
               endif
            enddo
         enddo
      endif

      return
      end  subroutine amatrix


!#######################################################################
!  Subroutine ata
!
!  Calculate aa = (a^T a)^(-1) a^T
!#######################################################################

      subroutine ata(a,aa,work,gdim,mmax,eps)

      implicit none

      integer gdim,mmax,k,info
      real*8  eps
      real*8  a(gdim,mmax),aa(mmax,gdim),work(3*mmax**2+mmax)


      call mmtxdd(a,a,work(mmax+1),mmax,gdim,mmax)
      call dsyev('V','U',mmax,work(mmax+1),mmax,work, &
                 work(mmax**2+mmax+1),3*mmax-1,info)
      if (info.ne.0) then
         write(6,'(a)') 'Error in diagonalising (A^T A) matrix (ata)'
         stop
      endif
      do k=1,mmax
         if (work(k) .gt. eps) then
            work(k)=1.d0/work(k)
         else
            work(k)=0.d0
         endif
      enddo
      call qdxxdd(work(mmax+1),work,work(mmax**2+mmax+1),mmax)
      call qqxtdd(work(mmax+1),work(mmax**2+mmax+1), &
                  work(2*mmax**2+mmax+1),mmax)
      call mmxtdd(work(2*mmax**2+mmax+1),a,aa,mmax,mmax,gdim)


      return
      end  subroutine ata


!#######################################################################
!  Subroutine invgrid
!
!  Inverse the grid if it is in decreasing order, to have always an
!  increasing order.
!#######################################################################

      subroutine invgrid(ort,ort2)

      implicit none

      integer f,g
      real*8  ort(ortdim),ort2(ortdim2)


      do f=1,ndof
         invfac(f)=0
         if (ort(zort(f)).gt.ort(zort(f)+1)) then
            do g=1,gdim(f)
               ort(zort(f)+g-1)=-ort(zort(f)+g-1)
            enddo
            do g=1,gdim2(f)
               ort2(zort2(f)+g-1)=-ort2(zort2(f)+g-1)
            enddo
            invfac(f)=1
            call swapsxxd(xend(1,f),xend(2,f))
         endif
         xend(1,f)=ort(zort(f))
         xend(2,f)=ort(zort(f)+gdim(f)-1)
         if (ort2(zort2(f)).gt.ort2(zort2(f)+1)) then
             xend2(1,f)=ort2(zort2(f)+gdim2(f)-1)
             xend2(2,f)=ort2(zort2(f))
         else
             xend2(1,f)=ort2(zort2(f))
             xend2(2,f)=ort2(zort2(f)+gdim2(f)-1)
         endif
      enddo

      return
      end  subroutine invgrid


!#######################################################################
!  Subroutine invgrid_back
!
!  Reinverse the grid if it was inversed (invfac(f)=1).
!#######################################################################

      subroutine invgrid_back(ort,ort2)

      integer f,g
      real*8  ort(ortdim),ort2(ortdim2)


      do f=1,ndof
         if (invfac(f).eq.1) then
            do g=1,gdim(f)
               ort(zort(f)+g-1)=-ort(zort(f)+g-1)
            enddo
            do g=1,gdim2(f)
               ort2(zort2(f)+g-1)=-ort2(zort2(f)+g-1)
            enddo
            call swapsxxd(xend(1,f),xend(2,f))
         endif
         xend2(1,f)=ort2(zort2(f))
         xend2(2,f)=ort2(zort2(f)+gdim2(f)-1)
      enddo

      return
      end  subroutine invgrid_back


!#######################################################################
!  Subroutine cpcpd
!
!  Copy the cpd file for those modes which are not not fitted
!#######################################################################

      subroutine cpcpd(evec,z1,evec2,z2,subdim,rank,gg1,gg2)

      implicit none

      integer g,r,subdim,rank,z1,z2,zz1,zz2,gg1,gg2
      real*8  evec(*), evec2(*)

      do r=1,rank
         zz1=z1-1+(r-1)*gg1
         zz2=z2-1+(r-1)*gg2
         do g=1,subdim
            evec2(g+zz2) = evec(g+zz1)
         enddo
      enddo
      return
      end  subroutine cpcpd

!#######################################################################
!  Subroutine Chnpotinput
!
!  Read all input parameters needed by program chnpot.
!#######################################################################

      subroutine chnpotinput

      implicit none

      integer  m,n,n1,f,llog,f1,f2,chkdvr,chkgrd, nmode1,ndof1,ierr,i
      integer  slen,fdvr(maxdim),mdvr(maxdim),mmode(maxdim)
      integer  subdim1(maxdim),pntbas(maxdim)
      logical  linperr, lexist, lmnd, lerr, lndspl

      routine='Chnpotinput'
      cpdrank=0
      lcpd=.false.


! --- READ THE COMAND LINE; I.E. OPTIONS AND PATH OF INPUT FILE
      call chncmd(lmnd)

! --- READ RUN-SECTION, PRIMIRIVE-BASIS-SECTION AND FIT-SECTION
! --- FROM THE INPUT FILE

      linperr=.false.
      call einchnpot(linperr)
      if (linperr) then
          message = 'Finished with input error.'
         call errormsg
      endif

! --- Set default for new dvr file name.
      if (ldvrnew.eq.0 .or. .not.lrddvr) then
         dvrfilenew=name(1:laenge)//'/dvr'
         ldvrnew=laenge+4
      endif

! --- Set default for old natpot file name.
      if (lnatpot.eq.0 .and. lndvr.gt.3) then
         if(lcpd) then
            npotfile = dvrfile(1:lndvr-3)//'cpd'
         else
            npotfile = dvrfile(1:lndvr-3)//'natpot'
         endif
         lnatpot=index(npotfile,' ')-1
      endif

! --- Set default for old dvr file name.
      if (lndvr.eq.0 .and. lnatpot.gt.3) then
         dvrfile=npotfile
         call dirpath(dvrfile,lndvr)
         if(dvrfile(lndvr:lndvr).eq.'/') lndvr=lndvr-1
         dvrfile=dvrfile(1:lndvr)//'dvr'
         lndvr=lndvr+3
      endif

! --- CHECK VARIABLES ---

      routine='Chnpotinput'
      if (name(1:1) .eq. ' ') then
         message='No name (output) directory is given.'
         call errormsg
      endif
      if (dvrfile(1:1) .eq. ' ') then
         message='No dvr file for input is given.'
         call errormsg
      endif
      if (npotfile(1:1) .eq. ' ') then
         message='No natpot file for input is given.'
         call errormsg
      endif
      if (inputfile(1:1) .eq. ' ') then
         message='No input file is given'
         call errormsg
      endif

      logfile=name(1:laenge)//'/'//logfile
      llog = index(logfile,' ')-1


! --- CREATE THE NAME DIRECTORY IF NECESSARY ---

      open(itmpin,file=name(1:laenge)//'/scratch',err=113)
      close(itmpin,status='delete')
      if(lmnd) write(6,'(2a)') 'Name-directory already exist: ',name(1:laenge)
      goto 114

 113  if(lmnd) then
         inptit='mkdir '//name(1:laenge)
         call excmd2(ierr, inptit)
         if (ierr.ne.0) then
            write(6,'(2a)') '### ERROR: ', &
            'Invalid path for name-directory (-mnd option). ###'
            stop
         endif
         write(6,'(2a)') 'Created name-directory: ',name(1:laenge)
      else
         write(6,'(a)') &
         '---------------------------------------------------------'
         write(6,'(a)') ' ################   ERROR   ##################'
         write(6,'(a)') ' There is no name-directory! The directory:'
         write(6,'(1x,a)') name(1:laenge)
         write(6,'(a)') ' does not exist! (or has no write permission)'
         write(6,'(2a)')' One may use the option -mnd to create the', &
                        ' name-directory.'
         write(6,'(a)')  &
           '-------------------------------------------------------'
         stop
      endif

 114  continue

! --- OPEN AND WRITE SOME INFORMATION TO THE LOG FILE ---

      open (unit=ilog,file=logfile(1:llog),form='formatted',iostat=ierr)
      if (ierr.ne.0) then
          message='Cannot open file '//logfile(1:llog)
          call errormsg
      endif
      logisopen=.true.

      rewind iin
      write(ilog,'(/a)') trim(cstartup)
      write(ilog,'(/,20x,a,/80a1)') 'INPUT FILE (copy)',('*',i=1,80)
      if(lcpd) write(ilog,'(a)') '          ++++  CPD-file  ++++'
 200  read(iin,'(a)',err=210,end=220) inptit
      if (inptit(1:1) .ne. '#'     .and.   &
           inptit(1:5) .ne. '-----' .and.  &
           inptit(1:5) .ne. '_____' .and.  &
           slen(inptit) .gt. 0 )           &
           write(ilog,'(a)',err=230) inptit(1:slen(inptit))
      call keylowc(inptit)
      if (inptit(1:10) .ne. 'end-input ' ) goto 200
      write(ilog,'(80a1,/)') ('*',i=1,80)
      close(iin)

      write(ilog,'(2a/)')   ' Options : ', options(1:optlaenge)
      write(ilog,'(2a)') ' The old  dvr     file is : ', &
                         dvrfile(1:lndvr)
      write(ilog,'(2a)') ' The new  dvr     file is : ', &
                         dvrfilenew(1:ldvrnew)
      write(6,'(2a)') ' Reading : ',npotfile(1:lnatpot)
      if(lcpd) then
         write(ilog,'(2a)') ' The old  cpd     file is : ', &
                         npotfile(1:lnatpot)
         write(ilog,'(2a)') ' The new  cpd     file is : ', &
                         name(1:laenge)//'/cpd'
      else
         write(ilog,'(2a)') ' The old  natpot  file is : ', &
                         npotfile(1:lnatpot)
         write(ilog,'(2a)') ' The new  natpot  file is : ', &
                         name(1:laenge)//'/natpot'
      endif


! --- CHECK FOR EXISTENCE OF FILES ---

      inquire(file=dvrfilenew(1:ldvrnew),exist=lexist)
      if (.not.lovwr .and. .not.lrddvr .and. lexist) then
         write(6,'(/a)') ' One may use -w or -rd option.'
         message='File already exists: '//dvrfilenew(1:ldvrnew)
         call errormsg
      elseif(lrddvr .and. .not.lexist) then
         message='File does not exists: '//dvrfilenew(1:ldvrnew)
         call errormsg
      endif
      if (.not.lovwr) then
         inquire(file=name(1:laenge)//'/natpot',exist=lexist)
         if (lexist) then
            write(6,'(/a)') ' One may use -w  option.'
            message='File already exists: '//name(1:laenge)//'/natpot'
            call errormsg
         endif
      endif

! --- save new basis parameters
      call cpvxi(basis,basis2,maxdim)
      call cpvxi(gdim,gdim2,maxdim)
      call cpvxi(zort,zort2,maxdim)
      call swapsxxi(ortdim,ortdim2)
      call cpmxi(ipbaspar,ipbaspar2,mbaspar,maxdim)
      call cpmxd(rpbaspar,rpbaspar2,mbaspar,maxdim)
      call cpvxl(ldvr,ldvr2,maxdim)
      ndof2=ndof
! --- save the new modelabels
      do f=1,ndof
            modelabel2(f)=modelabel(f)
      enddo

! --- read information from the natpot file to set nmode and dofspf
      open(unit=inpot,file=npotfile(1:lnatpot),form='unformatted', &
            status='old',iostat=ierr)
      if (ierr.ne.0) then
          message='could not open '//npotfile(1:lnatpot)
          call errormsg
      endif
      chkdvr=1
      chkgrd=1
      if(lcpd) then
         call cpdinfo(inpot,lerr,ndof1,fdvr,      &
              chkdvr,chkgrd,1,mdvr,mmode,cpdrank, &
              nspfdof,spfdof,nmode1,subdim1,lonedflag,pntbas)
      else
         call npotinfo(inpot,lerr,modc,lonedflag,ndof1,fdvr, &
              chkdvr,chkgrd,1,mdvr,mmode,nspfdof,spfdof,nmode1, &
              subdim1,pntbas)
      endif
      nmode=npmode
      close(inpot)

! --- reset the basis parameters to calculate new dvr
      call cpvxi(basis2,basis,maxdim)
      call cpvxi(gdim2,gdim,maxdim)
      call cpvxi(zort2,zort,maxdim)
      call swapsxxi(ortdim2,ortdim)
      call cpmxi(ipbaspar2,ipbaspar,mbaspar,maxdim)
      call cpmxd(rpbaspar2,rpbaspar,mbaspar,maxdim)
      call cpvxl(ldvr2,ldvr,maxdim)
      ndof=ndof2
! --- reset the new modelabels
      do f=1,ndof
            modelabel(f)=modelabel2(f)
      enddo


! --- DO CHECK OF THE PARAMETERS ---


      if (lpconm) then
         do m=1,ndof
            if (nspfdof(m) .gt. 1) then
               do n=1,nspfdof(m)
                  do n1=n+1,nspfdof(m)
                     f1=spfdof(n,m)
                     f2=spfdof(n1,m)
                     if (intmode(f2).eq.0 .and. intmode(f1).ne.0) then
                        intmode(f2)=intmode(f1)
                        method(f2)=method(f1)
                        angmode(f2)=angmode(f1)
                        mmax(f2)=mmax(f1)
                     elseif (intmode(f2).ne.0 .and. intmode(f1).eq.0) then
                        intmode(f1)=intmode(f2)
                        method(f1)=method(f2)
                        angmode(f1)=angmode(f2)
                        mmax(f1)=mmax(f2)
                     elseif (intmode(f2) .ne. intmode(f1)) then
                        message='Interpolation method for combined '// &
                             modelabel(f1)//'and '//modelabel(f2)//    &
                             ' mode must be the same.'
                        call errormsg
                     end if
                  end do
               end do
            endif
         enddo
      endif



      do m=1,nmode
         if (nspfdof(m) .gt. 2) then
            lndspl=.true.
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (intmode(f).ne.1) lndspl=.false.
            end do
            if (.not.lndspl .and. ldiffer(m)) then
               message='Chnpot for modes containing more than 2 '// &
                    'DOFs is only implemented for splines.'
               call errormsg
            end if
         endif
      enddo


      do f=1,ndof
         if (intmode(f).eq.2) then
            if (mmax(f).eq.0) then
               mmax(f)=max(50,2*gdim(f))
            else
               if (mmax(f).lt.gdim(f)) then
                 message='Mmax should be >= gdim for dof '//modelabel(f)
                 call errormsg
               endif
            endif
         endif
      enddo

      return

 210  message='Error while reading input file (rout.: chnpotinput).'
      call errormsg
 220  message= &
        'An end-input is missing in input file (rout.: chnpotinput).'
      call errormsg
 230  message= &
        'Error while writing input to log file (rout.: chnpotinput).'
      call errormsg

      end subroutine chnpotinput


!#######################################################################
!  Subroutine Chncmd
!
!  Read the comand line.
!#######################################################################

      subroutine chncmd(lmnd)

      implicit none

      integer  i,iarg,myiargc,ilbl,length,linput
      logical  lmnd
      character*240  buf

      routine='Chncmd'

! --- SET DEFAULT VALUES ---
      name      =' '
      npotfile  =' '
      inputfile =' '
      dvrfile   =' '
      dvrfilenew=' '
      lnatpot = 0
      laenge  = 0
      linput  = 0
      dlaenge = 0
      ldvrnew = 0
      lndvr   = 0
      logfile = 'chnpot.log'
      lmnd=.false.
      lrddvr=.false.

      iarg  = myiargc()
      if (iarg .le. 0) goto 200
      call mygetarg(0,options)
      optlaenge  = index(options,' ')
      do i = 1, iarg
         call mygetarg(i,buf)
         ilbl  = index(buf,' ')
         ilbl = min(ilbl,100-optlaenge)
         if(ilbl.gt.0) options = options(1:optlaenge)//buf(1:ilbl)
         optlaenge = optlaenge + ilbl
      end do


! --- READ OPTIONS ---
 200  continue
      i=0
 10   continue
      i=i+1
      call mygetarg(i,buf)

      if (buf(1:1) .eq. '-') then
         if (buf(2:3) .eq. 'D ') then
            i=i+1
            call mygetarg(i,name)
            laenge=index(name,' ')-1
            call abspath(name,laenge)
         elseif (buf(2:3) .eq. 'd ') then
            i=i+1
            call mygetarg(i,dname)
            length=index(dname,' ')-1
            if(dname(length:length).eq.'/') length=length-1
            dvrfile = dname(1:length)//'/dvr'
            lndvr=length+4
            call abspath(dvrfile,lndvr)
         elseif (buf(2:3) .eq. 'rd') then
            lrddvr=.true.
            i=i+1
            call mygetarg(i,buf)
            if(buf(1:1) .eq. '-' .or. i.eq. iarg) then
               i=i-1
            else
               length=index(buf,' ')-1
               if(dname(length:length).eq.'/') length=length-1
               dvrfilenew=buf(1:length)//'/dvr'
               ldvrnew=index(dvrfilenew,' ')-1
               call abspath(dvrfilenew,ldvrnew)
            endif
         elseif (buf(2:5) .eq. 'mnd ') then
            lmnd=.true.
            write(6,'(2a)') 'The option -cpd is obsolete. ', &
            'Use options -cd or -cf, or keywords cpddir or cpdfile.'
         elseif( buf(2:5) .eq. 'cpd ') then
            lcpd=.true.
         elseif (buf(2:3) .eq. 'nf') then
            i=i+1
            call mygetarg(i,npotfile)
            lnatpot=index(npotfile,' ')-1
            if(npotfile(lnatpot:lnatpot).eq.'/') lnatpot=lnatpot-1
            call abspath(npotfile,lnatpot)
         elseif (buf(2:3) .eq. 'nd') then
            i=i+1
            call mygetarg(i,buf)
            length=index(buf,' ')-1
            if(buf(length:length).eq.'/') length=length-1
            npotfile=buf(1:length)//'/natpot'
            lnatpot=length+7
            call abspath(npotfile,lnatpot)
         elseif (buf(2:3) .eq. 'cf') then
            i=i+1
            lcpd=.true.
            call mygetarg(i,npotfile)
            lnatpot=index(npotfile,' ')-1
            if(npotfile(lnatpot:lnatpot).eq.'/') lnatpot=lnatpot-1
            call abspath(npotfile,lnatpot)
         elseif (buf(2:3) .eq. 'cd') then
            i=i+1
            lcpd=.true.
            call mygetarg(i,buf)
            length=index(buf,' ')-1
            if(buf(length:length).eq.'/') length=length-1
            npotfile=buf(1:length)//'/cpd'
            lnatpot=length+4
            call abspath(npotfile,lnatpot)
         elseif (buf(2:3) .eq. 'w ') then
            lovwr=.true.
         elseif (buf(2:4) .eq. 'ver') then
            call wrversion(6)
            stop
         elseif (buf(2:3) .eq. 'h ' .or. buf(2:3) .eq. '? ') then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(2x,a)') &
          'Purpose : Interpolation of natpot/cpd terms to another grid.'
            write(6,'(2x,a)') &
                'Usage   : chnpot86 [-D -d -rd -w -mnd -cpd -nf -nd'// &
                ' -cd -cf -ver -h -?] input '
            write(6,'(/,a)') ' Options  : '
            write(6,'(a/,12x,2a)') &
                     ' -D PATH  : The output is directed to PATH. ', &
                     'This option overwrites "name" in ',            &
                     'the run-section of the input file.'
            write(6,'(2a/,12x,a)')  ' -d DIR   : The old dvr is read ', &
                    'from file DIR/dvr. This option overwrites ',       &
                    '"dvrdir" in the run-section of the input file.'
            write(6,'(2a/,12x,a,/12x,a)') &
                    ' -rd DIR  : The new dvr is read ',          &
                    'from directory DIR. If DIR is missing, ',   &
                    'name is assumed. This option sets/overwrites ', &
                    '"readdvr" in the run-section of the input file.'
            write(6,'(2a)')  ' -w       : Overwrite existing', &
                             ' files in name directory.'
            write(6,'(2a)')  ' -mnd     : Make Name Directory. ', &
                    'Creates the name directory, if not existent.'
            write(6,'(2a)')  ' -cpd     : ', &
                    'Use a cpd rather than a natpot file. (Obsolete).'
            write(6,'(2a/,12x,a)')  ' -nf FILE : A natpot is read ', &
                    'from file FILE. This option overwrites ',       &
                    '"natpotfile" in the run-section of the input file.'
            write(6,'(2a/,12x,a)')  ' -nd DIR  : A natpot is read ', &
                    'from file DIR/natpot. This option overwrites ', &
                    '"natpotdir" in the run-section of the input file.'
            write(6,'(2a/,12x,a)')  ' -cf FILE : A cpd is read ', &
                    'from file FILE. This option overwrites ',    &
                    '"cpdfile" in the run-section of the input file.'
            write(6,'(2a/,12x,a)')  ' -cd DIR  : A cpd is read ', &
                    'from file DIR/cpd. This option overwrites ', &
                    '"cpddir" in the run-section of the input file.'
            write(6,'(a)') &
                    ' -ver     : Version information about the program '
            write(6,'(a,/a)') &
                       ' -h       : Print this help text. ', &
                       ' -?       : Print this help text. '
            write(6,'(/2x,2a,/2x,2a/)') 'The argument "input" is ',    &
                      'the file containing the Run-Section, the new ', &
                      'Primitive-Basis-Section, and the Fit-Section. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else
            write(6,'(2a)') ' unknown option : ', buf
            stop
         endif
      endif

      if (i .lt. iarg-1) go to 10

      call mygetarg(iarg,inputfile)
      linput=index(inputfile,' ')-1
      if(linput .gt. 3) then
         if(inputfile(linput-3:linput) .ne. '.inp' ) then
            inputfile=inputfile(1:linput)//'.inp'
            linput=linput+4
         endif
      elseif(linput.gt.0) then
         inputfile=inputfile(1:linput)//'.inp'
         linput=linput+4
      else
         write(6,'(a)') ' There is no input file given'
         stop
      endif
      call abspath(inputfile,linput)

      return
      end subroutine chncmd


!#######################################################################
!  Subroutine Einchnpot
!
!  Read the run-, primitive-basis- and fit- sections from the input file.
!#######################################################################

      subroutine einchnpot(linperr)

      implicit none

      integer iz0,iz,ierr,ic,f,i,iperiod
      logical linperr,lfound

      routine='Einchnpot'

      open(iin,file=inputfile,status='old',iostat=ierr)
      if (ierr.ne.0) then
          message='Could not open input file '//trim(inputfile)
          call errormsg
      endif

! --- read run-section

      rewind(iin)
      iz0=0
      iz=0
  10  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
          message = 'Cannot read input file'
          call errormsg
      endif

      if (keyword(1).eq.'end-input') then
         message='No RUN-SECTION in input file'
         call errormsg
      elseif (keyword(1).eq.'run-section') then
 16      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (keyword(1).eq.'end-run-section') goto 21
         message = inptit
         call cntrl(keyword(1),lc(1),ierr)

         i=1
 15      continue
         if (keyword(i) .eq. 'name') then
            if (keyword(i+1) .ne. '=') then
               message = 'No argument specified with name keyword'
               call warnmsg
            else
               if (laenge.eq.0) then
                  name=keyorig(i+2)
                  laenge = lc(i+2)
                  if (laenge .eq. 0) then
                     message = ' No path name specified'
                     call wrline(zeroi,iz,iz0)
                  endif
                  call abspath(name,laenge)
               endif
               i=i+2
            endif
         elseif (keyword(i).eq.'cpd') then
            lcpd=.true.
         elseif (keyword(i).eq.'readdvr' .and. .not.lrddvr) then
            lrddvr=.true.
            if (keyword(i+1) .eq. '=') then
               if (ldvrnew.eq.0) then
                  dvrfilenew = keyorig(i+2)
                  ldvrnew = lc(i+2)
               if(dvrfilenew(ldvrnew:ldvrnew).eq.'/') ldvrnew=ldvrnew-1
                  dvrfilenew = dvrfilenew(1:ldvrnew)//'/dvr'
                  ldvrnew = ldvrnew + 4
                  call abspath(dvrfilenew,ldvrnew)
               endif
               i=i+2
            endif
         elseif (keyword(i) .eq. 'dvrdir') then
            if (keyword(i+1) .ne. '=') then
               message = 'No argument specified with dvrdir keyword'
               call warnmsg
            else
               if (lndvr.eq.0) then
                  dname = keyorig(i+2)
                  lndvr = lc(i+2)
                  if (lndvr .eq. 0) then
                     message = ' No dvr path specified'
                     call wrline(zeroi,iz,iz0)
                  endif
                  if(dname(lndvr:lndvr).eq.'/') lndvr=lndvr-1
                  dvrfile = dname(1:lndvr)//'/dvr'
                  lndvr=index(dvrfile,' ')-1
                  call abspath(dvrfile,lndvr)
               endif
               i=i+2
            endif
         elseif (keyword(i) .eq. 'natpotdir' .or. keyword(i) .eq. 'cpddir' ) then
            if(keyword(i) .eq. 'cpddir' ) lcpd=.true.
            if (keyword(i+1) .ne. '=') then
               if(lcpd) then
                  message='No argument specified with cpddir keyword'
               else
                  message='No argument specified with natpotdir keyword'
               endif
               call warnmsg
            else
               if (lnatpot.eq.0) then
                  npotfile = keyorig(i+2)
                  lnatpot = lc(i+2)
                  if (lnatpot.eq.0) then
                     message = ' No natpoti/cpd path specified'
                     call wrline(zeroi,iz,iz0)
                  endif
                  if(npotfile(lnatpot:lnatpot).eq.'/') lnatpot=lnatpot-1
                  if(lcpd) then
                     npotfile = npotfile(1:lnatpot)//'/cpd'
                  else
                     npotfile = npotfile(1:lnatpot)//'/natpot'
                  endif
                  lnatpot=index(npotfile,' ')-1
                  call abspath(npotfile,lnatpot)
               endif
               i=i+2
            endif
         elseif (keyword(i) .eq. 'natpotfile' .or. keyword(i) .eq. 'cpdfile' ) then
            if(keyword(i) .eq. 'cpdfile' ) lcpd=.true.
            if (keyword(i+1) .ne. '=') then
               if(lcpd) then
                 message='No argument specified with cpdfile keyword'
               else
                 message='No argument specified with natpotfile keyword'
               endif
               call warnmsg
            else
               if (lnatpot.eq.0) then
                  i=i+2
                  npotfile = keyorig(i)
                  lnatpot = lc(i)
                  if (lnatpot.eq.0) then
                     message = ' No natpot/cpd file specified'
                     call wrline(zeroi,iz,iz0)
                  endif
                  call abspath(npotfile,lnatpot)
               endif
            endif
         elseif (keyword(i).eq.'overwrite') then
            lovwr=.true.
         else
            message='Unknown keyword : '
            call wrline(i,iz,iz0)
         endif
! --  check if more keywords are to be read, else read new line
         if (i.lt.ic) then
            i=i+1
            go to 15
         else
            go to 16
         endif
      else
         go to 10
      endif


! --- read primitive-basis-section

 21   rewind(iin)
      linperr=linperr.or.lwarn
      lwarn=.false.
      iz0=0
      iz=0
 20   continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
          message = 'Cannot read input file'
          call errormsg
      endif

      if (keyword(1) .eq. 'pbasis-section' .or.  &
          keyword(1) .eq. 'primitive-basis-section') then
          call einpbas(linperr,iz)
      elseif (keyword(1) .eq. 'end-input') then
          message='No PRIMITIVE-BASIS-SECTION in input file'
          call errormsg
      else
          go to 20
      endif

! --- read fit-section ---

      routine='Einchnpot'
      rewind(iin)
      linperr=linperr.or.lwarn
      lwarn=.false.
      lfitsec=.false.
      iz0=0
      iz=0
  30  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
          message = 'Cannot read input file'
          call errormsg
      endif

      call zerovxi(intmode,maxdim)
      call zerovxi(angmode,maxdim)
      if (keyword(1).eq.'end-input') then
         lfitsec=.true.
      elseif (keyword(1) .ne. 'fit-section') then
         goto 30

      else
 35      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (keyword(1).eq.'end-fit-section') goto 40
         message = inptit
         call cntrl(keyword(1),lc(1),ierr)

         i=1
         do f=1,ndof
            symstring(f)='     '
            if (keyorig(i) .eq. modelabel(f)) then
               lfound=.true.
               i=i+1
! --- read type of interpolation per degree of freedom
               if (keyword(i) .eq. '=') write(6,'(/a)') &
                    'An equal sign must not follow modelabel!'

               if (keyword(i)(1:3) .eq. 'spl')then
                  intmode(f)=1
                  method(f)='cubic spline'

               elseif (keyword(i)(1:3) .eq. 'eno') then
                  intmode(f)=3
                  method(f)='eno interpolator'

               elseif (keyword(i)(1:1) .eq. ' ' .or. &
                       keyword(i)(1:3) .eq. 'cos') then
                  intmode(f)=2
                  angmode(f)=1
                  method(f)='cosine'

               elseif (keyword(i)(1:3) .eq. 'sin') then
                  intmode(f)=2
                  angmode(f)=2
                  method(f)='sine'

               elseif (keyword(i)(1:7) .eq. 'fourier' .or. &
                       keyword(i)(1:2) .eq. 'ft' ) then
                  intmode(f)=2
                  angmode(f)=3
                  method(f)='fourier'

               else
                  message='Interpolation method should be "spl" '// &
                       '"cos", "sin" or "fourier"("ft"), and not : '
                  call wrline(twoi,iz,iz0)
               endif
               i=i+1
! --- read extra information for angular modes
               if (intmode(f) .eq. 2) then
                  symstring(f)='nosym'
                  period(f)=2.d0*pi
 33               continue
                  if (keyword(i)(1:3) .eq. 'sym') then
                     msym(f)=.true.
                     symstring(f)=' sym '
                     i=i+1
                  elseif (keyword(i)(1:6) .eq. 'period') then
                     i=i+1
                     if (keyword(i)(1:1) .ne. '=') then
                        message='After keyword period should '//'stand "=".'
                        call wrline(zeroi,iz,iz0)
                     else
                        i=i+1
                        if (keyword(i)(1:3) .eq. '2pi') then
                           if (keyword(i)(4:4) .eq. '/') then
                              read(keyword(i)(5:lc(i)),*,iostat=ierr) &
                                   iperiod
                              if (ierr .ne. 0) then
                                 message='Cannot read period for'// &
                                      ' dof '//modelabel(f)
                                 call wrline(zeroi,iz,iz0)
                              endif
                              period(f)=2.d0*pi/dble(iperiod)
                           endif
                        else
                           read(keyword(i),*,iostat=ierr) period(f)
                           if (ierr .ne. 0) then
                              message='Cannot read period for'// &
                                   ' dof '//modelabel(f)
                              call wrline(zeroi,iz,iz0)
                           endif
                        endif
                        i=i+1
                     endif
                  elseif (keyword(i) .ne. ' ') then
                     read(keyword(i),*,iostat=ierr) mmax(f)
                     if (ierr .ne. 0) then
                        message='Cannot read mmax for dof '//modelabel(f)
                        call wrline(zeroi,iz,iz0)
                     endif
                     if (mmax(f) .le. 0) then
                        message='Mmax should be a positive integer.'
                        call wrline(zeroi,iz,iz0)
                     endif
                     i=i+1
                  else
                     goto 34
                  endif
                  go to 33
               endif
! --- read extra information for eno interpolation or set defaults
               if (intmode(f) .eq. 3) then
                   enomax(f)=3
                   enomin(f)=1
                   dfac(f)=50.d0
 37                continue
                   if (keyword(i)(1:1) .eq. ' ') then
                        goto 34
                   elseif (keyword(i)(1:4) .eq. 'rmax') then
                        i=i+1
                        if (keyword(i)(1:1) .ne. '=') then
                                message='= must follow rmax keyword'
                                call errormsg
                        else
                                i=i+1
                                read(keyword(i),*,iostat=ierr) enomax(f)
                                i=i+1
                                goto 37
                        endif
                   elseif (keyword(i)(1:4) .eq. 'rmin') then
                        i=i+1
                        if (keyword(i)(1:1) .ne. '=') then
                                message='= must follow rmin keyword'
                                call errormsg
                        else
                                i=i+1
                                read(keyword(i),*,iostat=ierr) enomin(f)
                                i=i+1
                                goto 37
                        endif
                   elseif (keyword(i)(1:4) .eq. 'dfac') then
                        i=i+1
                        if (keyword(i)(1:1) .ne. '=') then
                                message='= must follow dfac keyword'
                                call errormsg
                        else
                                i=i+1
                                read(keyword(i),*,iostat=ierr) dfac(f)
                                i=i+1
                                goto 37
                        endif
                   endif
               endif
 34            continue
               go to 35
            endif
         enddo
         if (.not. lfound) then
            message='Dof not defined : '
            call wrline(onei,iz,iz0)
         endif
         go to 35
      endif

 40   continue
      linperr=linperr.or.lwarn
      lwarn=.false.

      return
      end subroutine einchnpot

!#######################################################################

      subroutine chnpotinput0

      implicit none

      integer(long) :: ierr
      logical lmnd

! --- READ THE COMAND LINE; I.E. OPTIONS AND PATH OF INPUT FILE
      call chncmd(lmnd)

! --- READ RUN-SECTION, FROM THE INPUT FILE
      call einchnpot0

! --- Set default for old natpot file name.
      if (lnatpot.eq.0 .and. lndvr.gt.3) then
         npotfile = dvrfile(1:lndvr)
         lnatpot=lndvr
      endif

!-----------------------------------------------------------------------
! Get version of natpot file (i.e. code used to create it)
!-----------------------------------------------------------------------
      open(inpot,file=npotfile,status='old',form='unformatted',&
           iostat=ierr)
      if (ierr.ne.0) then
          message='Could not open natpot file '//trim(npotfile)
          call errormsg
      endif
      read(inpot) filever(inpot)

!-----------------------------------------------------------------------
! Store name of alloc file if natpot from ver86 run
!-----------------------------------------------------------------------
      if (filever(inpot) .gt. 8.6d0 .and. filever(inpot) .lt. 8.65d0) then
         call dirpath(npotfile,lnatpot) ! remove filename from path
         alloc86file=npotfile(1:lnatpot)//'alloc'
      endif

!-----------------------------------------------------------------------
! get basic dimensions (maxdim, maxsta, etc)
!-----------------------------------------------------------------------
      call rdmemdim(inpot)
      close(inpot)

      return
      end subroutine chnpotinput0

!#######################################################################

      subroutine einchnpot0

      implicit none

      integer iz0,iz,ierr,ic,i

      routine='Einchnpot0'

      open(iin,file=inputfile,status='old',iostat=ierr)
      if (ierr.ne.0) then
          message='Could not open input file '//trim(inputfile)
          call errormsg
      endif

! --- read run-section

      rewind(iin)
      iz0=0
      iz=0
  10  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
          message = 'Cannot read input file'
          call errormsg
      endif

      if (keyword(1).eq.'end-input') then
         message='No RUN-SECTION in input file'
         call errormsg
      elseif (keyword(1).eq.'run-section') then
 16      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (keyword(1).eq.'end-run-section') goto 21
         message = inptit
         call cntrl(keyword(1),lc(1),ierr)

         i=1
 15      continue
         if (keyword(i) .eq. 'name') then
            if (keyword(i+1) .ne. '=') then
               message = 'No argument specified with name keyword'
               call warnmsg
            else
               if (laenge.eq.0) then
                  name=keyorig(i+2)
                  laenge = lc(i+2)
                  if (laenge .eq. 0) then
                     message = ' No path name specified'
                     call wrline(zeroi,iz,iz0)
                  endif
                  call abspath(name,laenge)
               endif
               i=i+2
            endif
         elseif (keyword(i).eq.'cpd') then
            lcpd=.true.
         elseif (keyword(i).eq.'readdvr' .and. .not.lrddvr) then
            lrddvr=.true.
            if (ldvrnew.eq.0) then
               if (keyword(i+1) .eq. '=') then
                  dvrfilenew = keyorig(i+2)
                  ldvrnew = lc(i+2)
                  dvrfilenew = dvrfilenew(1:ldvrnew)//'/dvr'
                  ldvrnew = ldvrnew + 4
                  call abspath(dvrfilenew,ldvrnew)
               endif
               i=i+2
            endif
         elseif (keyword(i) .eq. 'dvrdir') then
            if (keyword(i+1) .ne. '=') then
               message = 'No argument specified with dvrdir keyword'
               call warnmsg
            else
               if (lndvr.eq.0) then
                  dname = keyorig(i+2)
                  lndvr = lc(i+2)
                  if (lndvr .eq. 0) then
                     message = ' No dvr path specified'
                     call wrline(zeroi,iz,iz0)
                  endif
                  if(dname(lndvr:lndvr).eq.'/') lndvr=lndvr-1
                  dvrfile = dname(1:lndvr)//'/dvr'
                  lndvr=index(dvrfile,' ')-1
                  call abspath(dvrfile,lndvr)
               endif
               i=i+2
            endif
         elseif (keyword(i) .eq. 'natpotdir' .or. &
                 keyword(i) .eq. 'cpddir' ) then
            if(keyword(i) .eq. 'cpddir' ) lcpd=.true.
            if (keyword(i+1) .ne. '=') then
               if(lcpd) then
                  message='No argument specified with cpddir keyword'
               else
                  message='No argument specified with natpotdir keyword'
               endif
               call warnmsg
            else
               if (lnatpot.eq.0) then
                  npotfile = keyorig(i+2)
                  lnatpot = lc(i+2)
                  if (lnatpot.eq.0) then
                     message = ' No natpot/cpd path specified'
                     call wrline(zeroi,iz,iz0)
                  endif
                  if(npotfile(lnatpot:lnatpot).eq.'/') lnatpot=lnatpot-1
                  if(lcpd) then
                     npotfile = npotfile(1:lnatpot)//'/cpd'
                  else
                     npotfile = npotfile(1:lnatpot)//'/natpot'
                  endif
                  lnatpot=index(npotfile,' ')-1
                  call abspath(npotfile,lnatpot)
               endif
               i=i+2
            endif
         elseif (keyword(i) .eq. 'natpotfile' .or. &
                 keyword(i) .eq. 'cpdfile' ) then
            if(keyword(i) .eq. 'cpdfile' ) lcpd=.true.
            if (keyword(i+1) .ne. '=') then
               if(lcpd) then
                 message='No argument specified with cpdfile keyword'
               else
                 message='No argument specified with natpotfile keyword'
               endif
               call warnmsg
            else
               if (lnatpot.eq.0) then
                  npotfile = keyorig(i+2)
                  lnatpot = lc(i+2)
                  if (lnatpot.eq.0) then
                     message = ' No natpoti/cpd file specified'
                     call wrline(zeroi,iz,iz0)
                  endif
                  call abspath(npotfile,lnatpot)
               endif
               i=i+2
            endif
         endif
! --  check if more keywords are to be read, else read new line
         if (i.lt.ic) then
            i=i+1
            go to 15
         else
            go to 16
         endif
      else
         go to 10
      endif

 21   continue

      close(iin)

      return
      end subroutine einchnpot0

!#######################################################################
!
! memory required for arrays in chnpot allocated here.
!
! First check if allocated, then allocate. All arrays must be deallocated in
! chnpotmod_dealloc
!
!#######################################################################

      subroutine alloc_chnpot

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_chnpot

      cerr = 'mmax'
      allocate(mmax(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'zort2'
      allocate(zort2(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'zevec2'
      allocate(zevec2(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'psubdim2'
      allocate(psubdim2(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'gdim2'
      allocate(gdim2(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'zmode2'
      allocate(zmode2(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'basis2'
      allocate(basis2(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'mgdim'
      allocate(mgdim(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'intmode'
      allocate(intmode(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'angmode'
      allocate(angmode(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'invfac'
      allocate(invfac(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'enomax'
      allocate(enomax(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'enomin'
      allocate(enomin(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ipbaspar2'
      allocate(ipbaspar2(mbaspar,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'period'
      allocate(period(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dfac'
      allocate(dfac(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'xend2'
      allocate(xend2(2,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'rpbaspar2'
      allocate(rpbaspar2(mbaspar,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ldiffer'
      allocate(ldiffer(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'msym'
      allocate(msym(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ldvr2'
      allocate(ldvr2(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'method'
      allocate(method(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'symstring'
      allocate(symstring(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'X'
      allocate(modelabel2(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'loned'
      allocate(loned(maxdim))
      if (ierr > 0) go to 1000
      cerr = 'lonedflag'
      allocate(lonedflag(maxdim))
      if (ierr > 0) go to 1000

      mmax(:) = 0
      msym(:) = .false.

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_CHNPOTMOD :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_chnpot

!#######################################################################

      subroutine dealloc_chnpot

      if (allocated(mmax)) deallocate(mmax)
      if (allocated(zort2)) deallocate(zort2)
      if (allocated(zevec2)) deallocate(zevec2)
      if (allocated(psubdim2)) deallocate(psubdim2)
      if (allocated(gdim2)) deallocate(gdim2)
      if (allocated(zmode2)) deallocate(zmode2)
      if (allocated(basis2)) deallocate(basis2)
      if (allocated(mgdim)) deallocate(mgdim)
      if (allocated(intmode)) deallocate(intmode)
      if (allocated(angmode)) deallocate(angmode)
      if (allocated(invfac)) deallocate(invfac)
      if (allocated(enomax)) deallocate(enomax)
      if (allocated(enomin)) deallocate(enomin)
      if (allocated(ipbaspar2)) deallocate(ipbaspar2)
      if (allocated(period)) deallocate(period)
      if (allocated(dfac)) deallocate(dfac)
      if (allocated(xend2)) deallocate(xend2)
      if (allocated(rpbaspar2)) deallocate(rpbaspar2)
      if (allocated(ldiffer)) deallocate(ldiffer)
      if (allocated(msym)) deallocate(msym)
      if (allocated(ldvr2)) deallocate(ldvr2)
      if (allocated(method)) deallocate(method)
      if (allocated(symstring)) deallocate(symstring)
      if (allocated(modelabel2)) deallocate(modelabel2)
      if (allocated(loned)) deallocate(loned)
      if (allocated(lonedflag)) deallocate(lonedflag)

      return
      end subroutine dealloc_chnpot

!#######################################################################
      end program chnpot

