



program compactoper
  use pdefaultmod
  use hcpdtype_m
  use cpdtype_m
  use dvrtype_m
  use eximporttools
  use svdbasis_m
  use svdbasis_s
  use co_opts_m
  use timing
  use hals_m
  use versions
  use operdef, only: alloc_operdef
  use timing
  implicit none

  type(hcpd_t), target          :: oper
  type(hcpd_t), target          :: hcpd
  type(cpd_t), target           :: cpd, zoper, zcpd
  type(cpd_t), target           :: diagoper
  type(dvr_t)                   :: dvr
  class(tensor_t), pointer      :: toper, thcpd

  integer                       :: m
  integer                       :: ccrank,nchunk
  integer                       :: ilog, itim, ndiag, nmat


  logical, allocatable          :: loned(:)
  type(svdbasis_t), pointer     :: svdbasis(:) => Null()
  type(symmap_t), pointer       :: symmap(:) => Null()
  logical                       :: allsvdtermshermitian

  type(coopts_t)                :: opts ! command line options

  character(len=c5)             :: mdl, rout, msg
  integer                       :: maxmltimer

  real                          :: tstart, tfinish
  real(dbl)                     :: percpu, totcpu
  integer, save                 :: tid = 0

  common /mltimer/ maxmltimer

!!$  interface
!!$     subroutine optimize_hcpd(toper,thcpd, opts, ilog, symmap)
!!$       use tensortype_m
!!$       use co_opts_m
!!$       use svdbasis_m, only : symmap_t
!!$       class(tensor_t), intent(IN), target    :: toper ! original tensor
!!$       class(tensor_t), intent(INOUT), target :: thcpd ! fitted tensoer
!!$       type(coopts_t), intent(IN)             :: opts  ! program options
!!$       integer, intent(IN)                    :: ilog
!!$       type(symmap_t), intent(IN), pointer    :: symmap(:)
!!$     end subroutine optimize_hcpd
!!$
!!$  end interface


  maxmltimer=0 ! initialize (from timing_m)
  call cpu_time(tstart)

  call default
  call pdefault

  ! parse command line options
  call compactoper_inp(opts, ilog) !sets ilog

  call alloc_dvrdat
  call alloc_grddat
  call alloc_operdef

  ! read dvr
  call loaddvr(dvr,opts%dvrfile)

  call get_timer(tid, "readoper")
  call start_timer(tid)

  if (opts%lpes) then
     write(ilog,'(a)') ''
     write(*,'(a)') 'Reading operator file:'//trim(opts%operfilename)
     write(ilog,'(a)') 'Reading operator file:'//trim(opts%operfilename)

     call rdpes2cpd(diagoper,dvr,opts%ham,opts%nham,opts%operfilename,ilog)

     call wrcpdproperties(diagoper,ilog)
     write(*,'(a,i8)') ' Operator has rank', diagoper%rank
     toper => diagoper
     nmode = diagoper%nmode

  else

     call rdoper2hcpd(oper,dvr,opts%ham,opts%nham,opts%operfilename,ilog, &
          ndiag, nmat)
     call wrhcpdproperties(oper,ilog)
     call co_hcpd_sanity_check()
     write(*,'(a,i8)') ' Operator has rank', oper%rank

     toper => oper
     nmode = oper%nmode

  end if
  call stop_timer(tid)


  if (opts%lpes) then
     if (diagoper%rank .eq. opts%rank) then
        write(*,'(a)') " Nothing to do: transcribing pes to cpd."
        write(ilog,'(a)') " Nothing to do: transcribing pes to cpd."
        call exportMCTDH(diagoper, dvr, opts%cpdfile, opts%lovwr)
        stop 0
     else if  (diagoper%rank .lt. opts%rank) then
        mdl = "compactoper.f90"
        rout = "(main)"
        msg = "The cpd rank (-r option) must not larger then the rank of the PES."
        call stop_code(mdl,rout,msg)
     end if
  else
     if (oper%rank .eq. opts%rank) then
        write(*,'(a)') " Nothing to do: transcribing oper to hcpd."
        write(ilog,'(a)') " Nothing to do: transcribing oper to hcpd."
        call exportMCTDH(oper, dvr, opts%hcpdfile, opts%lovwr)
        stop 0
     else if  (oper%rank .lt. opts%rank) then
        mdl = "compactoper.f90"
        rout = "(main)"
        msg = "The hcpd rank (-r option) must not larger then the rank of oper."
        call stop_code(mdl,rout,msg)
     end if


  end if


  if (opts%lcompress) then

     if(opts%lhermitian) then
        write(*,'(a)') ""
        write(*,'(a)') "Compacting basis"
        write(*,'(a)') "Mode,    dim  -->    nbasis (herm anti),    err"
        write(*,'(a)') "----------------------------------------------------"
        write(ilog,'(a)') ""
        write(ilog,'(a)') "Compacting basis"
        write(ilog,'(a)') "Mode,    dim  -->    nbasis (herm anti),    err"
        write(ilog,'(a)') "----------------------------------------------------"

     else
        write(*,'(a)') ""
        write(*,'(a)') "Compacting basis"
        write(*,'(a)') "Mode,    dim  -->    nbasis,    err"
        write(*,'(a)') "-----------------------------------------"
        write(ilog,'(a)') ""
        write(ilog,'(a)') "Compacting basis"
        write(ilog,'(a)') "Mode,    dim  -->    nbasis,    err"
        write(ilog,'(a)') "-----------------------------------------"
     end if



     allocate(svdbasis(nmode))

     do m=1,nmode

        if (opts%lpes) then

           call svd_basis_1m_from_sop(diagoper%spp(m), diagoper%rank,&
                svdbasis(m), .false., opts%svderr)
        else

           call svd_basis_1m_from_sop(oper%sph(m), oper%rank, &
                svdbasis(m), opts%lhermitian, opts%svderr)

        end if
        if(opts%lhermitian) then

           write(*,'(i4,i8,a,i4,a,2i4,a,6x,ES14.8)') m, svdbasis(m)%subdim, &
                "  -->      ", &
                svdbasis(m)%nbasis, ' (', svdbasis(m)%nherm, svdbasis(m)%naherm, ')', svdbasis(m)%l2err
           write(ilog,'(i4,i8,a,i4,a,2i4,a,6x,ES14.8)') m, svdbasis(m)%subdim, &
                "  -->      ", &
                svdbasis(m)%nbasis, ' (',  svdbasis(m)%nherm, svdbasis(m)%naherm, ')', svdbasis(m)%l2err

        else
           write(*,'(i4,i8,a,i4,6x,ES14.8)') m, svdbasis(m)%subdim, &
                "  -->      ", &
                svdbasis(m)%nbasis, svdbasis(m)%l2err
           write(ilog,'(i4,i8,a,i4,6x,ES14.8)') m, svdbasis(m)%subdim, &
                "  -->      ", &
                svdbasis(m)%nbasis, svdbasis(m)%l2err
        end if
     end do


     ! build the symmetry maps if necessary
     allsvdtermshermitian = .true.
     if (opts%lhermitian .and. .not. opts%lpes) then
        do m=1,nmode
           allsvdtermshermitian  = allsvdtermshermitian .and. &
                all(svdbasis(m)%htype(:) .eq. HTM_TYPE_HERMITIAN)
        end do
     end if
     if (.not. allsvdtermshermitian) then
        allocate(symmap(nmode))
        do m=1,nmode
           allocate(symmap(m)%symtype(svdbasis(m)%nbasis))
           symmap(m)%gdim = svdbasis(m)%nbasis
           symmap(m)%symtype(:) = svdbasis(m)%htype(:)
        end do
     else if(opts%lhermitian) then
        write(*,*) "All SVD terms are Hermitian. Dublication is not necessary."
        write(ilog,*) "All SVD terms are Hermitian. Dublication is not necessary."
        opts%lhermitian = .false.
     end if

     call trafo_cp_grid_2_svd_basis(toper,zoper,svdbasis,nmode)


     ! need to adjust vpotdim for correct error measures
     if (opts%lpes) then
        zoper%rvpotdim = diagoper%rvpotdim
     else
        zoper%rvpotdim = oper%rvpotdim2
     end if

  end if




  ccrank = 0
  if (opts%lcontinue) then
     write(*,'(a)')   'Reading initial guess file:'//trim(opts%ccfile)
     write(ilog,'(a)')'Reading initial guess file:'//trim(opts%ccfile)
     if (opts%lpes) then
        call importMCTDH(cpd,dvr,opts%ccfile)
        call wrcpdproperties(cpd,ilog)
        ccrank = cpd%rank
        thcpd => cpd
     else
        call importMCTDH(hcpd,dvr,opts%ccfile)
        call wrhcpdproperties(hcpd,ilog)
        ccrank = hcpd%rank
        thcpd => hcpd
     end if
     if (.not. opts%lresetiter) then
        call getstartiter(opts%startiter,opts%iterationsfile)
        if (opts%startiter .ge. opts%maxiter) then
           mdl = "compactoper.f90"
           rout = "compactoper"
           msg = "Number of iterations in 'iterations' file " &
                // "already larger than command line value. " &
                // "Stopping here."
           call stop_code(mdl,rout,msg)
        end if
     else
        opts%startiter = 0
     end if

     ! TODO: checks for mode combinations

  end if



  if (.not. opts%lrank .and. .not. opts%lgrow) then
     if (opts%lcontinue) then
        opts%rank = max(ccrank,oper%rank/10)
     else
        opts%rank = max(1,oper%rank/10)
     end if
     opts%initrank = opts%rank
     opts%maxrank = opts%rank

     write(msg,'(a,i4)') "No rank for fit given in command line. Using rank: ", opts%rank
     call cncntrblnk(msg)
     write(*,'(a)') trim(msg)
     write(ilog,'(a)') trim(msg)
  else if (opts%lgrow) then

     if (opts%lcontinue) then
        opts%rank = max(ccrank,opts%grank)
     else
        opts%rank = opts%grank
     end if

     opts%initrank = opts%rank
     !maxrank is already set from command line

     write(msg,'(a,i4)') "Growing from initial rank of fit: ", opts%initrank
     write(*,'(a)') trim(msg)
     write(ilog,'(a)') trim(msg)
  else if (opts%lrank) then

     if (opts%lcontinue) then
        opts%rank = max(ccrank,opts%rank)
     !else
     !   rank = grank
     end if
     opts%initrank = opts%rank
     write(msg,'(a,i4)') "Initial rank of fit: ", opts%initrank
     write(*,'(a)') trim(msg)
     write(ilog,'(a)') trim(msg)
  end if


  if (.not. opts%lcontinue) then
     write(*,'(a)')    'Setting up random initial guess...'
     write(ilog,'(a)') 'Setting up random initial guess...'
     allocate(loned(dvr%ndof))
     loned(:) = .false.
     opts%startiter = 0

     if (opts%lpes) then

        call allocate_cpd_t(cpd,opts%initrank,diagoper%ndof,diagoper%nmode,diagoper%gdim, &
             diagoper%dofspf,diagoper%potdofmap,diagoper%modelabel,dvr%ort,dvr%ortdim, loned)


        call randomizecpd(cpd)

     else

        call allocate_hcpd_t(hcpd,opts%initrank,oper%ndof,oper%nmode,oper%gdim, &
             oper%dofspf,oper%potdofmap,oper%modelabel,dvr%ort,dvr%ortdim, loned)
        deallocate(loned)

        call randomizehcpd(hcpd, opts%inittype)

        if (opts%lhermitian) then
           call hermitizehcpd(hcpd)
        end if

     end if


  else ! continue

     if (opts%lhermitian) then
        nchunk = 2
     else
        nchunk = 1
     end if


     if (ccrank .gt. opts%maxrank) then
        msg = "Rank of initial guess file exceeds maxrank."
        rout = "compactoper"
        mdl = "compactoper"
        call stop_code(mdl,rout,msg)
     end if

     if (opts%initrank .ne. ccrank) then
        opts%initrank = min(opts%initrank,ccrank)
        call changehrank(hcpd, opts%initrank, 0, nchunk)
        if (opts%lhermitian) then
           call hermitizehcpd(hcpd)
        end if
     end if

  end if

  if (opts%lpes) then
     toper => diagoper
     thcpd => cpd
  else
     toper => oper
     thcpd => hcpd
  end if


  write(*,'(a)')   'Entering optimization step.'
  write(ilog,'(a)')   'Entering optimization step.'

  if (opts%lcompress) then

     call trafo_cp_grid_2_svd_basis(thcpd,zcpd,svdbasis,thcpd%nmode)

     if (opts%lpes) then
        zcpd%rvpotdim = diagoper%rvpotdim
     else
        zcpd%rvpotdim = oper%rvpotdim2
     end if

     call myflush(ilog)
     call optimize_hcpd(zoper, zcpd, opts, ilog, symmap)

     call trafo_cp_svd_2_grid_basis(zcpd,thcpd,svdbasis,zcpd%nmode)

  else

     call myflush(ilog)
     call optimize_hcpd(toper,thcpd, opts, ilog, symmap)

  end if




  if (opts%lpes) then
     call exportMCTDH(cpd, dvr, opts%cpdfile, opts%lovwr)
  else
     call exportMCTDH(hcpd, dvr, opts%hcpdfile, opts%lovwr)
  end if


  call cpu_time(tfinish)
  totcpu = tfinish - tstart
  if (totcpu .gt. 0.d0) then
     percpu = 100.d0/totcpu
  else
     percpu = 0.d0
  end if
  open(newunit=itim, file=trim(opts%timingfile))
  write(itim,'(1x,a10,11x,a7,9x,a5,13x,a3,7x,a4,6x,a6,10x,a4,2x,a8,/)') &
       'Subroutine','Calls N','cpu/N','cpu','%cpu', &
       'real/N','real','cpu/real'

  call print_timers(itim, percpu)
  close(itim)



contains

  subroutine co_hcpd_sanity_check()
    write(6,'(a)') ""
    write(6,'(a)') "Number of single-particle operator types: "
    write(6,'(a,i8)')  "  Matrix:   ", nmat
    write(6,'(a,i8)')  "  Diagonal: ", ndiag
    write(6,'(a)') ""

    write(ilog,'(a)') ""
    write(ilog,'(a)') "Number of single-particle operator types: "
    write(ilog,'(a,i8)')  "  Matrix:   ", nmat
    write(ilog,'(a,i8)')  "  Diagonal: ", ndiag
    write(ilog,'(a)') ""
    call myflush(ilog)

    if ( .not. opts%lforce) then
       if (ndiag .gt. nmat) then
          mdl = "compactoper.f90"
          rout = "co_hcpd_sanity_check"
          msg = "The number of diagonal SPOs is larger then the number of " &
               // "Matrix SPOs. Is this a kinetic energy? Then don't use the program. " &
               // "If you know what you are doing then use the -f option to override this message."
          call stop_code(mdl,rout,msg)
       end if
    else
       if (ndiag .gt. nmat) then
          mdl = "compactoper.f90"
          rout = "co_hcpd_sanity_check"
          msg = "The number of diagonal SPOs is larger then the number of " &
               // "Matrix SPOs. Is this a kinetic energy? Then you may want to reconsider using this program. " &
               // "You have used the -f option therefore I will continue..."
          call warn_msg(mdl,rout,msg)
       end if
    end if

  end subroutine co_hcpd_sanity_check

end program compactoper











