

module cpdcorr_m
  use logdat, only: errormsg, routine, message
  implicit none
  character(len=512) :: dvrfile
  character(len=512) :: cpdfile

  double precision   :: maxerr = 1.d-5
  logical            :: lcompress = .true.

  character(len=*), parameter   :: outfile = "cpd_correlation_sorted"

  double precision,allocatable  :: sigma(:,:,:)

  type cpddensity_t
     integer                       :: subdim
     double precision, allocatable :: density(:,:)
     double precision, allocatable :: evecs(:,:)
     double precision, allocatable :: npop(:)
     double precision              :: entropy
  end type cpddensity_t

  type(cpddensity_t), allocatable  :: densities(:)


end module cpdcorr_m


program cpdcorrelation
  use cpdtype_m
  use dvrtype_m
  use cpdcorr_m
  use eximporttools
  use channels, only: idvr
  use operdef
  use psidef
!  use compdatmod
  implicit none

  type(cpd_t) :: cpd, smallcpd
  type(dvr_t) :: dvr
  integer     :: m,ilbl
  character(len=c5)                     :: filename

  call default()

  call cpdcorr_input()

! get memory dimensions
  filename=dname(1:dlaenge)//'/dvr'
  open(idvr,file=filename,form='unformatted',status='old',err=900)
  call rdmemdim(idvr)
  close(idvr)

  call alloc_dvrdat
  call alloc_grddat
  call alloc_operdef
!  call alloc_compdat
  call alloc_psidef

  call loaddvr(dvr, dvrfile)

  call importMCTDH(cpd,dvr,cpdfile)


  call mksigma(cpd)



  allocate(densities(cpd%nmode))
  do m=1,cpd%nmode
     print*, "Calculating density mode ", m
     allocate(densities(m)%density(cpd%subdim(m),cpd%subdim(m)))
     allocate(densities(m)%evecs(cpd%subdim(m),cpd%subdim(m)))
     allocate(densities(m)%npop(cpd%subdim(m)))
     densities(m)%subdim = cpd%subdim(m)
     call cpddensity1d(cpd,m,densities(m)%density)
  end do

  call reducecpd(cpd,smallcpd)

  if (lcompress) then

     call getdensitycorrelations(cpd, smallcpd)

  else
     call getdensitycorrelations(cpd, cpd)
  end if

 ! call getcorrelations(cpd)

    stop
 900  continue
      routine='showspf'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
end program cpdcorrelation



! mode wise overlap matrix
subroutine mksigma(cpd)
  use cpdtype_m
  use cpdcorr_m
  implicit none

  type(cpd_t), intent(IN)       :: cpd

  integer ::  r,r1,m
  double precision :: ddot
  external ddot


  allocate(sigma(cpd%nmode, cpd%rank, cpd%rank))

  !$omp parallel do private(r,r1,m) schedule(dynamic,1)
  do r1=1,cpd%rank
     do r=r1, cpd%rank
        do m=1,cpd%nmode
           sigma(m,r,r1) = ddot(cpd%subdim(m),cpd%spp(m)%htm(:,r),1,cpd%spp(m)%htm(:,r1),1)
           sigma(m,r1,r) = sigma(m,r,r1)
        end do
     end do
  end do
  !$omp end parallel do



end subroutine mksigma


! mode wise reduced density
subroutine cpddensity1d(cpd,m,density)
  use cpdtype_m
  use cpdcorr_m
  use mcdensities_m
#ifdef _OPENMP
  use omp_lib
#endif
  implicit none

  type(cpd_t), intent(IN)       :: cpd
  integer, intent(IN)           :: m
  double precision, intent(OUT) :: density(cpd%subdim(m), cpd%subdim(m))

  double precision              :: sigmam(cpd%rank,cpd%rank)
  double precision              :: tmp(cpd%subdim(m)),tmp1(cpd%subdim(m))
  integer                       :: m1,r, r1

#ifdef _OPENMP
  integer :: nthreads, nt
  double precision, allocatable  :: tmpdensity(:,:,:)


  !$omp parallel
  nthreads = omp_get_num_threads()
  !$omp end parallel
  allocate(tmpdensity(cpd%subdim(m),cpd%subdim(m),nthreads))
#endif




  !$omp parallel do private(r,r1,m1) schedule(dynamic,1)
  do r1 = 1, cpd%rank
     do r = r1, cpd%rank
        sigmam(r,r1) = 1.d0
        do m1=1,cpd%nmode
           if (m.ne.m1) then
              sigmam(r,r1) = sigmam(r,r1) * sigma(m1,r,r1)
           end if
        end do
        sigmam(r,r1) = sigmam(r,r1)* cpd%coeff(r) * cpd%coeff(r1)
        sigmam(r1,r) = sigmam(r,r1)
     end do
  end do
  !$omp end parallel do



  !$omp parallel private(r,r1,tmp,tmp1,nt)

#ifdef _OPENMP
  nt = omp_get_thread_num() + 1
  tmpdensity(:,:,nt) = 0.d0
#else
  density(:,:) = 0.d0
#endif


  !$omp do schedule(dynamic,1)
  do r1=1,cpd%rank

     tmp1(:) = cpd%spp(m)%htm(:,r1)

     do r=r1,cpd%rank

        tmp(:) = cpd%spp(m)%htm(:,r) * sigmam(r,r1)


#ifdef _OPENMP

        if (r.eq.r1) then
           call dger(cpd%subdim(m),cpd%subdim(m),1.d0,tmp,1,tmp1,1,tmpdensity(:,:,nt),cpd%subdim(m))
        else
           call dger(cpd%subdim(m),cpd%subdim(m),1.d0,tmp,1,tmp1,1,tmpdensity(:,:,nt),cpd%subdim(m))
           call dger(cpd%subdim(m),cpd%subdim(m),1.d0,tmp1,1,tmp,1,tmpdensity(:,:,nt),cpd%subdim(m))
        end if

#else
        if (r.eq.r1) then
           call dger(cpd%subdim(m),cpd%subdim(m),1.d0,tmp,1,tmp1,1,density,cpd%subdim(m))
        else
           call dger(cpd%subdim(m),cpd%subdim(m),1.d0,tmp,1,tmp1,1,density,cpd%subdim(m))
           call dger(cpd%subdim(m),cpd%subdim(m),1.d0,tmp1,1,tmp,1,density,cpd%subdim(m))
        end if

#endif

     end do
  end do
  !$omp end do
  !$omp end parallel



#ifdef _OPENMP
  density = tmpdensity(:,:,1)
  do nt = 2, nthreads
     density = density + tmpdensity(:,:,nt)
  end do
  deallocate(tmpdensity)
#endif


end subroutine cpddensity1d






subroutine cpddensity2d(cpd,m,m1,density)
  use cpdtype_m
  use cpdcorr_m
  use mcdensities_m
#ifdef _OPENMP
  use omp_lib
#endif
  implicit none

  type(cpd_t), intent(IN)       :: cpd
  integer, intent(IN)           :: m,m1
  double precision, intent(OUT) :: density(cpd%subdim(m)*cpd%subdim(m1), &
                                           cpd%subdim(m)*cpd%subdim(m1))

  double precision              :: sigmam(cpd%rank,cpd%rank)
  double precision              :: tmpvec(cpd%subdim(m)*cpd%subdim(m1))
  double precision              :: tmpvec1(cpd%subdim(m)*cpd%subdim(m1))
  integer                       :: mi,r,r1,g,g1,gj,mmdim

#ifdef _OPENMP
  integer                        :: nthreads, nt
  double precision, allocatable  :: tmpdensity(:,:,:)
#endif

  mmdim = cpd%subdim(m)*cpd%subdim(m1)

#ifdef _OPENMP
  !$omp parallel
  nthreads = omp_get_num_threads()
  !$omp end parallel
  allocate(tmpdensity(mmdim,mmdim,nthreads))
#endif


  !$omp parallel do private(r,r1,mi) schedule(dynamic,1)
  do r1 = 1, cpd%rank
     do r = r1, cpd%rank
        sigmam(r,r1) = 1.d0
        do mi=1,cpd%nmode
           if (m.ne.mi .and. m1.ne.mi) then
              sigmam(r,r1) = sigmam(r,r1) * sigma(mi,r,r1)
           end if
        end do
        sigmam(r,r1) = sigmam(r,r1)* cpd%coeff(r) * cpd%coeff(r1)
        sigmam(r1,r) = sigmam(r,r1)
     end do
  end do
  !$omp end parallel do




  !$omp parallel private(r,r1,g,g1,tmpvec,tmpvec1,nt)

#ifdef _OPENMP
  nt = omp_get_thread_num() + 1
  tmpdensity(:,:,nt) = 0.d0
#else
  density(:,:) = 0.d0
#endif



  !$omp do schedule(dynamic,1)
  do r1=1,cpd%rank

     do g1=1,cpd%subdim(m1)
        do g=1,cpd%subdim(m)
           gj = (g1-1)*cpd%subdim(m)+ g
           tmpvec1(gj) = cpd%spp(m)%htm(g,r1) * cpd%spp(m1)%htm(g1,r1)
        end do
     end do

     do r=r1,cpd%rank

        do g1=1,cpd%subdim(m1)
           do g=1,cpd%subdim(m)
              gj = (g1-1)*cpd%subdim(m)+ g
              tmpvec(gj) = cpd%spp(m)%htm(g,r)* cpd%spp(m1)%htm(g1,r)*sigmam(r1,r)
           end do
        end do


#ifdef _OPENMP

        if (r.eq.r1) then
           call dger(mmdim,mmdim,1.d0,tmpvec,1,tmpvec1,1,tmpdensity(:,:,nt),mmdim)
        else
           call dger(mmdim,mmdim,1.d0,tmpvec,1,tmpvec1,1,tmpdensity(:,:,nt),mmdim)
           call dger(mmdim,mmdim,1.d0,tmpvec1,1,tmpvec,1,tmpdensity(:,:,nt),mmdim)
        end if

#else
        if (r.eq.r1) then
           call dger(mmdim,mmdim,1.d0,tmpvec,1,tmpvec1,1,density,mmdim)
        else
           call dger(mmdim,mmdim,1.d0,tmpvec,1,tmpvec1,1,density,mmdim)
           call dger(mmdim,mmdim,1.d0,tmpvec1,1,tmpvec,1,density,mmdim)
        end if
#endif

     end do
  end do
  !$omp end do
  !$omp end parallel



#ifdef _OPENMP
  density = tmpdensity(:,:,1)
  do nt = 2, nthreads
     density = density + tmpdensity(:,:,nt)
  end do
  deallocate(tmpdensity)
#endif



end subroutine cpddensity2d










subroutine reducecpd(cpd,cpd1)
  use cpdtype_m
  use cpdcorr_m
  implicit none


  type(cpd_t), intent(IN)    :: cpd
  type(cpd_t), intent(INOUT) :: cpd1

  integer :: maxgdim, ierr, newsubdim(cpd%nmode)
  integer :: i,j,m,r
  double precision :: relerr(cpd%nmode), trace(cpd%nmode)

  integer :: dofspf(cpd%nmode),potdofmap(2,cpd%nmode)
  integer :: ortdim
  logical :: loned(cpd%nmode)

  character(len=16) :: modelabel(cpd%nmode)
  double precision, allocatable :: ort(:)


  ! BLAS

  double precision ddot
  external ddot

  ! LAPACK
  integer                        :: LWORK
  double precision, allocatable  :: WORK(:)
  character, parameter           :: JOBZ='V'
  character, parameter           :: UPLO='L'


  maxgdim = 0
  do m=1,cpd%nmode
     maxgdim = max(maxgdim, cpd%subdim(m))
  end do
  LWORK = (maxgdim+2)*maxgdim
  allocate(WORK(LWORK))

  if (lcompress) then
     write(*,*) ""
     write(*,*) "Compressing modes"
     write(*,*) "mode, gridsize  --> new gridsize"
     write(*,*) "--------------------------------"
  end if


  do m=1,cpd%nmode

     densities(m)%evecs = densities(m)%density



     call DSYEV( JOBZ, UPLO, densities(m)%subdim, densities(m)%evecs,&
          densities(m)%subdim, densities(m)%npop, WORK, LWORK, ierr )
     if (ierr .ne. 0) then
        write(*,*) "Error in subroutine reducecpd"
        write(*,*) "Lapack dsyev failed with error code: ", ierr
        stop 1
     end if



     do i=1,densities(m)%subdim
        if (densities(m)%npop(i) .le. 0.d0) then
           densities(m)%npop(i) = 0.d0
        end if
     end do

     do i=1,densities(m)%subdim/2
        work(1:densities(m)%subdim) =densities(m)%evecs(:,i)
        densities(m)%evecs(:,i) = densities(m)%evecs(:,densities(m)%subdim-i+1)
        densities(m)%evecs(:,densities(m)%subdim-i+1) = work(1:densities(m)%subdim)
        work(1) = densities(m)%npop(i)
        densities(m)%npop(i) = densities(m)%npop(densities(m)%subdim-i+1)
        densities(m)%npop(densities(m)%subdim-i+1) = work(1)
     end do


     trace(m) = sum(densities(m)%npop)

     densities(m)%entropy = 0.d0
     do i=1,densities(m)%subdim

        if (densities(m)%npop(i) .gt. 0.d0) then
           densities(m)%entropy = densities(m)%entropy - densities(m)%npop(i)/trace(m)*log( densities(m)%npop(i)/trace(m))
        end if
     end do


     if (lcompress) then
        do i=densities(m)%subdim, 1, -1
           relerr(m) = sum(densities(m)%npop(i:densities(m)%subdim))/trace(m)
           if (relerr(m) .gt. maxerr) then
              newsubdim(m) = i
              relerr(m) = sum(densities(m)%npop(i:densities(m)%subdim))/trace(m)
              exit !loop
           end if
        end do


        write(*,'(i4,i8,a,i8)')  m,densities(m)%subdim, "     -->", newsubdim(m)
     end  if


  end do


  ! exit if we do not want to compress
  if (.not. lcompress) return


  ! build fake inputs
  ! for cpd in ntural orbitals
  ortdim=0
  do m=1,cpd%nmode
     dofspf(m) = m
     potdofmap(1,m) = m
     potdofmap(2,m) = 1
     write(modelabel(m), '(a,i4)') "mode_",m
     call rmblnk(modelabel(m))
     ortdim=ortdim + newsubdim(cpd%nmode)
     loned(m) = .false.
  end do
  allocate(ort(ortdim))
  ort=0.d0

  call allocate_cpd_t(cpd1, cpd%rank, cpd%nmode, cpd%nmode,newsubdim, &
       dofspf,potdofmap, modelabel,ort, ortdim, loned)
  deallocate(ort)

  !$omp parallel do private(m,j) schedule(dynamic,1)
  do r=1,cpd%rank
     do m=1,cpd%nmode
        do j=1,cpd1%subdim(m)
           cpd1%spp(m)%htm(j,r) = ddot(cpd%subdim(m),cpd%spp(m)%htm(:,r),1, densities(m)%evecs(:,j),1)
        end do
     end do

     cpd1%coeff(r) = cpd%coeff(r)
  end do
  !$omp end parallel do


end subroutine reducecpd




subroutine getdensitycorrelations(cpd, cpd1)
  use global
  use channels
  use cpdcorr_m
  use linkedlist_m
  use cpdtype_m
  implicit none
  type(cpd_t), intent(IN) :: cpd, cpd1

  integer :: m,m1,mdim, m1dim, g, n, mmdim, ierr
  double precision :: tmp

  double precision, allocatable :: matrix(:,:)
  double precision, allocatable :: sigma2(:)
  double precision, allocatable :: correlation(:,:)

  type(linkedlist_t), pointer :: first, curr
  integer                     :: idata(2), i
  double precision            :: ddata(1)
  logical                     :: lmatch
  character(len=80)           :: mlabel, m1label, label
  character(len=c5)           :: fname
  !LAPACK
  character :: JOBZ, UPLO
  integer   :: LWORK, LSIGMA
  double precision, allocatable :: work(:)
  double precision :: W1(1)! dummy W


  ! we dont want the vectors, just eigenvalues
  JOBZ = "N"
  UPLO = "L"

  allocate(correlation(cpd%nmode,cpd%nmode))



  write(*,*) ""
  write(*,*) "Calculating pair wise entropies"
  write(*,*) "mode1   mode2   correlation"
  write(*,*) "--------------------------------"

  do m=1,cpd1%nmode
     mdim = cpd1%subdim(m)
     do m1=m+1, cpd1%nmode
        m1dim = cpd1%subdim(m1)

        mmdim = mdim*m1dim

        allocate(matrix(mmdim, mmdim), stat=ierr)
        if (ierr .ne. 0) then
           write(*,*) "ERROR in subroutine getdensitycorrelations"
           write(*,*) "Could not allocate 2D-density."
           write(*,*) "Size:", mmdim*mmdim
           stop 1
        end if
        if (mmdim .ge. 10000) then
           write(*,*) "WARNING in subroutine getdensitycorrelations"
           write(*,*) "2D-density is larger then"
           write(*,*) "10000x10000. This may take a while"
        end if

        LSIGMA = mmdim
        allocate(sigma2(LSIGMA), stat=ierr)
        if (ierr .ne. 0) then
           write(*,*) "ERROR in subroutine getdensitycorrelations"
           write(*,*) "Could not allocate 2D-npop array."
           write(*,*) "Size:", mmdim
           stop 1
        end if

        call cpddensity2d(cpd1,m,m1,matrix)

        LWORK = -1

        call DSYEV( JOBZ, UPLO, mmdim, matrix,&
             mmdim, sigma2, W1, LWORK, ierr )

        LWORK = int(W1(1)) + 1
        allocate(WORK(LWORK), stat=ierr)
        if (ierr .ne. 0) then
           write(*,*) "ERROR in subroutine getdensitycorrelations"
           write(*,*) " could not allocate workspace."
           write(*,*) " Size:", LWORK
           stop 1
        end if



        call DSYEV( JOBZ, UPLO, mmdim, matrix,&
             mmdim, sigma2, WORK, LWORK, ierr )

        do i=1,mmdim/2
           work(1) = sigma2(i)
           sigma2(i) =  sigma2(mmdim-i+1)
           sigma2(mmdim-i+1) = work(1)
        end do



        deallocate(WORK)

        sigma2 = sigma2/sum(sigma2)

        tmp = sigma2(1) * log(sigma2(1))
        !$omp parallel do reduction(+:tmp)
        do g=2,LSIGMA
           if (sigma2(g) .gt. 0.d0) then
              tmp = tmp + sigma2(g) * log(sigma2(g))
           end if
        end do
        !$omp end parallel do

        correlation(m1,m) = densities(m)%entropy + densities(m1)%entropy  + tmp

        write(*,'(2i6,f18.12)') m,m1,correlation(m1,m)

        deallocate(sigma2, matrix)

     end do
  end do








  nullify(first)
  nullify(curr)
  ! build a sorted entropy list

  n = 0
  do m=1, cpd%nmode
     do m1=m+1, cpd%nmode

        ddata = correlation(m1,m)
        idata(1) = m1
        idata(2) = m

        ! create first element
        if (.not. associated(first)) then
           first => linkedlist_t(ddata=ddata, idata=idata)
           curr => first
           n = 1
        else

           curr => first
           lmatch = .false.
           do while (.not. lmatch)
              if (curr%ddata(1) .lt. correlation(m1,m) ) then
                 call  insert_bef(curr,ddata=ddata, idata=idata)
                 lmatch = .true.
                 n = n+1
                 if (.not. associated(curr%prev%prev)) then
                    first => curr%prev
                 end if
              else if (associated(curr%next)) then
                 curr => curr%next
              else if (curr%ddata(1) .ge. correlation(m1,m) .and. .not. associated(curr%next) ) then
                 call  insert_after(curr, ddata=ddata, idata=idata)
                 lmatch = .true.
                 n=n+1
              else
                 write(*,*) "ERROR in subroutine getdensitycorrelations"
                 write(*,*) " while sorting entropies."
                 write(*,*) " This should never have happened..."
                 write(*,*) " Please send a bug report."
                 stop 1
              end if
           end do

        end if

     end do
  end do


  fname = outfile

  open(idump, file=fname, action="write")
  write(idump,'(a)') "#n, m, m1, correlation(m,m1), labels(m), labels(m1)"

  n = 0
  curr => first
  do while (associated(curr))
     n = n+1
     m = curr%idata(1)
     m1 = curr%idata(2)

     call getModeLabelsStr(cpd,m,mlabel)
     call getModeLabelsStr(cpd,m1,m1label)
     label = "["//trim(mlabel)//"] ["//trim(m1label)//"]"
     write(idump,'(3i4,3x,f16.12,3x,a)') n, m,m1, curr%ddata(1), trim(label)

     curr => curr%next
  end do
  close(idump)



  call free_list(first)







end subroutine getdensitycorrelations





subroutine cpdcorr_input()
  use  cpdcorr_m
  use global
  use versions
  implicit none
  character(len=512) :: buf, buf1
  logical            :: lexist, lstop
  integer            :: i, iarg, slen, myiargc, ierr
  external myiargc


  dvrfile    = "./dvr"
  cpdfile    = "./cpd"
  lovwr  = .false.
  lcompress = .true.

  iarg = myiargc()
  call mygetarg(0,options)

  do i = 1, iarg
     call mygetarg(i,buf1)
     options = trim(options)//' '//trim(buf1)
     if  ( buf1 .eq. '-h' .or. &
          buf1 .eq. '-?' .or. &
          buf1 .eq. '--help' ) then
        call cpdcorr_wrhelptext
        stop 0
     else if  ( buf1 .eq. '-ver' ) then
        call wrversion(6)
        stop 0
     end if
  end do




  i = 0
  do while (i.lt. iarg)
     i=i+1

     call mygetarg(i,buf)

     if( buf(1:1) .eq. '-' ) then
        if ( buf .eq. '-w') then
           lovwr = .true.
        elseif ( buf .eq. '-no-spp') then
           lcompress = .false.
        elseif ( buf .eq. '-d' ) then
           i = i+1
           call mygetarg(i,dvrfile)

        elseif ( buf .eq. '-f' ) then
           i = i+1
           call mygetarg(i,cpdfile)

        elseif ( buf .eq. '-e' ) then
           i=i+1
           call mygetarg(i,buf)
           read(buf,*, iostat=ierr) maxerr
           if (ierr .ne. 0) then
              print*, "ERROR: Cannot read value after -e option. " &
                   // "Expecting real, not: "//trim(buf)
              stop 1
           end if
           if (maxerr .le. 0.d0) then
              print*, "ERROR_ Compression error, -e option, " &
                   // "must be .gt. 0, found: "//trim(buf)
              stop 1
           end if

        else
           write(*,*) "Unknown option: "//trim(buf)
           stop 1
        end if

     else
        write(*,*) "Unknown option: "//trim(buf)
        stop 1
     end if
  end do

  slen = len_trim(dvrfile)
  call abspath(dvrfile,slen)

  slen = len_trim(cpdfile)
  call abspath(cpdfile,slen)

  lstop = .false.
  inquire(file=dvrfile,exist=lexist)
  if (.not. lexist) then
     write(*,*) "File does not exist:"
     write(*,*) trim(dvrfile)
     write(*,*) ""
     lstop = .true.
  end if
  inquire(file=dvrfile,exist=lexist)
  if (.not. lexist) then
     write(*,*) "File does not exist:"
     write(*,*) trim(dvrfile)
     write(*,*) ""
     lstop = .true.
  end if

  inquire(file=outfile,exist=lexist)
  if (lexist .and. .not. lovwr) then
     routine = "cpdcorr_input"
     message = "File exists, cannot overwrite. Use -w option: "//trim(outfile)
     call errormsg
  end if




  if (lstop) stop 1

end subroutine cpdcorr_input


subroutine cpdcorr_wrhelptext()
  implicit none


  write(6,'(a)') '------------------------------------------------------------'
  write(6,'(a)') ' Purpose: calculate degree of correlation'
  write(6,'(a)') '          between modes of a cpd file'
  write(6,*)
  write(6,'(a)') '  Usage:  cpdcorrelation<d/D><P> [-h|-?|-ver] [-d -f }'
  write(6,*)
  write(6,'(a)') '    d/D:  Indicates a debug version'
  write(6,'(a)') '      P:  compiled with OpenMP support'
  write(6,*)
  write(6,'(a)') ' optn: -h         : Print this help-text.'
  write(6,'(a)') '       -?         : Print this help-text.'
  write(6,'(a)') '       -ver       : Version information.'
  write(6,*)
  write(6,'(a)') ' -e <err>         : Relative compression error per mode,'
  write(6,'(a)') '                    default: err=1.d-5'
  write(6,'(a)') ' -d <dvrfile>     : Read dvr from <dvrfile> rather'
  write(6,'(a)') '                    than from ./dvr'
  write(6,'(a)') ' -f <cpdfile>     : Read cpd from <cpdfile> rather'
  write(6,'(a)') '                    than from ./cpd'
  write(6,'(a)') ' -no-spp          : Do not trafo to natural potentials'
  write(6,'(a)') '                    (i.e. no compression)'
  write(6,'(a)') ' -w               : Allow overwrite'
  write(6,'(a)') ''
  write(6,'(a)') ' The program first calculates the entropies'
  write(6,'(a)') ' '
  write(6,'(a)') '   S = -sum_j p_j ln(p_j)'
  write(6,'(a)') ' '
  write(6,'(a)') ' Where p_j are the normalized natural populations (i.e.,'
  write(6,'(a)') ' sum_j p_j = 1) of one- and two-mode reduced density matrices.'
  write(6,'(a)') ' The correlation is then calculated as'
  write(6,'(a)') ' '
  write(6,'(a)') '   C(m,m1) = S(m) + S(m1) - S(m,m1)'
  write(6,'(a)') ' '
  write(6,'(a)') ' Where m and m1 are two different modes, respectively'
  write(6,'(a)') ' '
  write(6,'(a)') ' In order to reduce the size of the 2D matrices, the single'
  write(6,'(a)') ' mode basis on the primitive grid is first transformed'
  write(6,'(a)') ' to a natural potential basis with a relative error'
  write(6,'(a)') ' (see -e option). The relative error measure is:'
  write(6,'(a)') ' E = sum(relevant p_j)/sum(all p_j)'
  write(6,'(a)') ' '
  write(6,'(a)') ' Output file: cpd_correlation_sorted'
  write(6,'(a)') ' '
  write(6,'(a)') ' The program supports OpenMP (compile with -P option)'
  write(6,'(a)') '------------------------------------------------------------'

end subroutine cpdcorr_wrhelptext
