

module co_opts_m
  use global
  use constants_m
  use versions
  use logdat

  implicit none

  !private
  !public :: coopts_t
  !public :: compactoper_help
  !public :: compactoper_inp

  ! encapsuling all command line options
  ! in a single object
  type coopts_t
     character(len=MAXFNAME)  :: operfilename
     character(len=c4)  :: ham
     integer            :: maxiter, startiter, nham
     integer            :: initrank, rank, maxrank
     real(dbl)          :: alpha, deltastop, errorestimate, svderr
     logical            :: lcontinue, lcompress, lforce
     logical            :: ldeltastop, lmnd, lrank
     logical            :: lresetiter, lhermitian, lpes
     logical            :: lwrovlp,lwrcoeff,lgrow,lcc
     integer            :: deltastopunit
     integer            :: isolver
     integer            :: grank, giter, inittype
     integer            :: ndiis
     logical            :: ldiis
     logical            :: luseblas ! flag to use external blas
     logical            :: lovwr

     character(len=MAXFNAME) :: name
     character(len=MAXFNAME) :: inpdir
     character(len=MAXFNAME) :: dvrdir
     character(len=MAXFNAME) :: dvrfile
     character(len=MAXFNAME) :: hcpdfile
     character(len=MAXFNAME) :: cpdfile
     character(len=MAXFNAME) :: logfile
     character(len=MAXFNAME) :: iterationsfile
     character(len=MAXFNAME) :: stopfile
     character(len=MAXFNAME) :: timingfile
     character(len=MAXFNAME) :: ovlpfile
     character(len=MAXFNAME) :: coefffile
     character(len=MAXFNAME) :: ccfile

  end type coopts_t

contains

  subroutine coopts_default(opts)
    type(coopts_t), intent(INOUT) :: opts
    real(dbl)                     :: DLAMCH
    ! BLAS
    opts%alpha =  DLAMCH("E")
    opts%alpha = sqrt(opts%alpha)


    opts%maxiter = 1000
    opts%startiter = -1
    opts%nham = -1
    opts%initrank = -1
    opts%rank = -1
    opts%maxrank = -1
    opts%deltastop = 0.d0
    opts%errorestimate=-1.d0
    opts%svderr = opts%alpha
    opts%inittype = 0

    opts%lcontinue = .false.
    opts%lcompress = .false.
    opts%ldeltastop = .false.
    opts%lmnd = .false.
    opts%lresetiter = .false.
    opts%lhermitian = .false.
    opts%lpes = .false.
    opts%lwrovlp = .false.
    opts%lwrcoeff = .false.
    opts%lgrow = .false.
    opts%lcc = .false.
    opts%lrank = .false.
    opts%lforce = .false.

    opts%deltastopunit = 1 !au
    opts%isolver = 1
    opts%grank = 0
    opts%giter = 0

    opts%luseblas = .true. ! flag to use external blas
    opts%lovwr = .false.

    opts%ndiis = -1
    opts%ldiis = .false.

    opts%operfilename = " "
    opts%ham = " "
    opts%inpdir = " "
    opts%dvrdir = " "
    opts%dvrfile = " "
    opts%hcpdfile = " "
    opts%cpdfile = " "
    opts%logfile = " "
    opts%iterationsfile = " "
    opts%stopfile = " "
    opts%timingfile = " "
    opts%ovlpfile = " "
    opts%coefffile = " "
    opts%ccfile = " "


  end subroutine coopts_default






  subroutine compactoper_help()
    implicit none

    write(6,*)
    write(6,'(a)')  'Purpose: Read an MCTDH oper file and convert to a'
    write(6,'(a)')  '         Hamiltonian canonical polyadic decomposition (HCPD)'
    write(6,'(a)')  '         which is subsequently optimized using ALS.'
    write(6,*)
    write(6,'(a)')  '  Usage:  compactoper<vers><d/D> [-h|-?|-ver]  [-mnd -o -l -c -C -CC'
    write(6,'(32x,a)')                   ' -r -R -s -n -g -w -f -spd -ev -d -v -D]'
    write(6,'(a)')  '   vers:  Program version (e.g. 85 for version 8.5)'
    write(6,'(a)')  '    d/D:  Indicates a debug version'
    write(6,'(a)')  '      P:  Compiled with OpenMP support.'
    write(6,*)
    write(6,'(a)')  ' optn: -h           : Print this help-text.'
    write(6,'(a)')  '       -?           : Print this help-text.'
    write(6,'(a)')  '       -ver         : Version information.'
    write(6,*)
    write(6,'(2a)') '       -mnd         : Make Name Directory. ', &
         'Creates the name directory, if not existent.'
    write(6,'(a)')  '       -w           : Allow overwrite.'
    write(6,'(a)')  '       -f           : Override error messages in case kinetic energy is'
    write(6,'(a)')  '                      suspected. See note 2 below. Default: not set.'
    write(6,'(a)')  '       -c           : Continue obtimizing ' &
         // 'an existing HCPD file.'
    write(6,'(a)')  '       -C           : Same as -c but reset iteration counter.'
    write(6,'(a)')  '       -CC <dir>    : Same as -C but reads the initial CPD/HCPD-file ' &
         // 'from directory <dir>.'
    write(6,'(2a)') '       -pes         : The operator is a potential and read from'
    write(6,'(2a)') '                      a "pes" file instead of an "oper" file'
    write(6,'(2a)') '                      The resulting file will be a "cpd" file rather'
    write(6,'(2a)') '                      than a "hcpd" file.'
    write(6,'(a)')  '       -r <int>     : Rank of the resulting CPD/HCPD-fit.'
    write(6,'(a)')  '                      Default: half the number of terms'
    write(6,'(a)')  '                      of the original read-in operator.'
    write(6,'(a)')  '       -H           : Enforce HCPD fit to be Hermitian using'
    write(6,'(a)')  '                      Hermitian and Hermitian conjugate single'
    write(6,'(a)')  '                      particle operators.'
    write(6,'(a)')  '                      (requires rank to be an even number).'
    write(6,'(a)')  '       -H1 <int>    : Initial guess (and any on-the fly added)'
    write(6,'(a)')  '                      single particle operators are initialized as'
    write(6,'(a)')  '                       0: random'
    write(6,'(a)')  '                       1: random, hermitian'
    write(6,'(a)')  '                      -1: random, anti-hermitian'
    write(6,'(a)')  '                      default: random. Cannot be used with -H option.'

    write(6,'(aa)') '       -O <str>     : Use Hamiltonian <str> from the oper file',&
         ' (default <str>="system")'
    write(6,'(2a)') '       -a <real>    : Regularization, default: ', &
         ' sqrt(machine_precision).'
    write(6,'(2a)') '       -s <real>[,S]: Stop if |oper-fit| is ', &
         'less than <real>.'
    write(6,'(2a)') '                      Default: not set. <real> may bear a'
    write(6,'(a)')  '                      unit S [au, eV, meV, cm-1]. default a.u.'
    write(6,'(a)')  '       -o           : Write overlap matrix of the fit basis on exit.'
    write(6,'(a)')  '                      Default: not set.'
    write(6,'(a)')  '       -l           : Write coefficients of the fit on exit.'
    write(6,'(a)')  '                      Default: not set.'
    write(6,'(a)')  '       -z <err>     : Compress with error <err> before optimization.'
    write(6,'(a)')  '                      Compression is done using an SVD of the single-'
    write(6,'(a)')  '                      particle operators, <err> is the sum of the'
    write(6,'(a)')  '                      squared singular values that are neglected'
    write(6,'(a)')  '                      devided by the sum of all squared singular values.'
    write(6,'(a)')  '       -spd <real>  : Solve ALS using Cholesky decomposition with'
    write(6,'(a)')  '                      uniform regularization on the diagonal.'
    write(6,'(a)')  '                      If neither -spd nor -ev is set, then -spd with'
    write(6,'(a)')  '                      regularization=sqrt(machine_precision) is assumed.'
    write(6,'(a)')  '       -ev <real>   : Solve ALS using eigen-decomposition with'
    write(6,'(a)')  '                      regularization of smallest eigenvalues.'
    write(6,'(2a)') '       -n <int>     : Maximum number of', &
         ' iterations of ALS optimizer, default 1000.'
    write(6,'(a)')  '       -g <int int1>: Do not start with full rank but with rank <int> '
    write(6,'(a)')  '                      and increase every <int1> iterations by the same amount.'
    write(6,'(a)')  '                      If an initial guess is read from file the starting rank'
    write(6,'(a)')  '                      will be that of the fit from file but the rank increment'
    write(6,'(a)')  '                      will be as given by <int1>.'
    write(6,'(a)')  '       -x <int>     : Use iterative subspace extrapolation (DIIS) with <int>'
    write(6,'(a)')  '                      vectors (default: not set)'
    write(6,*)
    write(6,'(2a)') '       -I <dir>     : Read original operator file ', &
         ' from <dir>. Default: current directory'
    write(6,'(2a)') '       -J <dir>     : Read dvr file ', &
         ' from <dir>. Default: same as oper file.'
    write(6,'(2a)') '       -D <name>    : Write output to <name>.', &
         ' Default: current directory.'
    write(6,*)

    write(6,'(a)')  '       -noblas      : This program relies heavily on being linked to an'
    write(6,'(a)')  '                      optimized and parallel BLAS/LAPACK installation.'
    write(6,'(a)')  '                      Use this flag if you compiled with OpenMP support.'
    write(6,'(a)')  '                      but linked to the default BLAS/LAPACK library.'
    write(6,'(a)')  ''
    write(6,'(a)')  '      IMPORTANT (1):'
    write(6,'(a)')  '         Using a parallel BLAS/LAPACK library is usually sufficient'
    write(6,'(a)')  '         for a good performance. Using OpenMP on top of this for'
    write(6,'(a)')  '         other parts of the program may lead to some speed up'
    write(6,'(a)')  '         especially for large fit and operator ranks (it may'
    write(6,'(a)')  '         however also slow the program down in some cases).'
    write(6,'(a)')  ''
    write(6,'(a)')  '      IMPORTANT (2):'
    write(6,'(a)')  '         Do not use this program on kinetic energy operators!'
    write(6,'(a)')  '         Even if the resulting number of terms is smaller, MCTDH calculations'
    write(6,'(a)')  '         are likely much slower. Use this program on operators where'
    write(6,'(a)')  '         all or most single-particle-operators are matrices (like in SQR)'
    write(6,'(a)')  '         or if all single-particle-operators are diagonal (see -pes option).'
    write(6,'(a)')  ''
    stop



  end subroutine compactoper_help



  subroutine compactoper_inp(opts, ilog)
    implicit none

    type(coopts_t), intent(INOUT) :: opts
    integer, intent(OUT)          :: ilog
    integer                       :: i,j,iarg,myiargc,ierr, stat
    integer                       :: lencc, leninpdir, lendvrdir,lenname
    logical                       :: lcomma, lexists, linpdir, ldvrdir,lname
    logical                       :: leigen, lspd, lsvd, lh1
    character(len=c5)             :: buf, buf1, options
    character(len=c5)             :: rout, mdl, msg, inptit
    integer                       :: istop

    rout = "compactoper_inp"
    mdl  = "compactoper_m"

    call coopts_default(opts)

    iarg = myiargc()

    call mygetarg(0,options)
    do i = 1, iarg
       call mygetarg(i,buf1)
       options = trim(options)//' '//trim(buf1)
    end do

    leigen = .false.
    lspd = .false.
    lsvd = .false.
    lname = .false.
    ldvrdir = .false.
    linpdir = .false.
    lh1 = .false.

    i = 0
    do while (i.lt. iarg)
       i=i+1

       call mygetarg(i,buf)

       if( buf(1:1) .eq. '-' ) then
          if ( buf .eq. '-h' .or. &
               buf .eq. '--help' .or. &
               buf .eq. '-?' ) then
             call compactoper_help()
          elseif  ( buf .eq. '-ver') then
             call wrversion(6)
             stop 0
          elseif  ( buf .eq. '-mnd') then
             opts%lmnd  = .true.
          elseif  ( buf .eq. '-f') then
             opts%lforce  = .true.
          elseif  ( buf .eq. '-w') then
             opts%lovwr = .true.
          elseif ( buf .eq. '-c') then
             opts%lcontinue = .true.
          elseif ( buf .eq. '-C') then
             opts%lcontinue = .true.
             opts%lresetiter = .true.
          elseif ( buf .eq. '-CC') then
             opts%lcontinue = .true.
             opts%lresetiter = .true.
             opts%lcc = .true.
             i=i+1
             call mygetarg(i,buf)
             opts%ccfile = trim(buf)
             if (len_trim(opts%ccfile) .eq. 0) then
                msg  = "Expecting string after -CC option"
                call stop_code(mdl,rout,msg)
             end if
          elseif (buf .eq. '-pes') then
             opts%lpes = .true.
          elseif ( buf .eq. '-o') then
             opts%lwrovlp = .true.
          elseif ( buf .eq. '-z') then
             opts%lcompress = .true.
             i=i+1
             call mygetarg(i,buf)
             read(buf,*, iostat=ierr) opts%svderr
             if (ierr .ne. 0) then
                msg = "Cannot read value after -z option. " &
                     // "Expecting real, not: "//trim(buf)
                call stop_code(mdl,rout,msg)
             end if
          elseif ( buf .eq. '-O') then
             i=i+1
             call mygetarg(i,opts%ham)
          elseif ( buf .eq. '-l') then
             opts%lwrcoeff = .true.
          elseif  ( buf .eq. '-r') then
             opts%lrank = .true.
             i=i+1
             call mygetarg(i,buf)
             read(buf,*, iostat=ierr) opts%rank
             if (ierr .ne. 0) then
                msg = "Cannot read rank after -r option. " &
                     // "Expecting integer, not: "//trim(buf)
                call stop_code(mdl,rout,msg)
             end if
             opts%maxrank = opts%rank
          elseif  ( buf .eq. '-a') then
             i=i+1
             call mygetarg(i,buf)
             read(buf,*, iostat=ierr) opts%alpha
             if (ierr .ne. 0) then
                msg = "Cannot read real after -a option. " &
                     // "Expecting real, not: "//trim(buf)
                call stop_code(mdl,rout,msg)
             end if
          elseif  ( buf .eq. '-d') then
             buf1 = " "
             i=i+1
             call mygetarg(i,buf)
             j = index(buf,",")
             lcomma = j.ne.0

             if (lcomma) then
                call rmblnk(buf(j+1:))
                if (index(buf(j+1:), ' ') .gt.1) then
                   !---
                   buf1=buf(j+1:)
                   call keylowc(buf1)
                else
                   !-- -
                   i=i+1
                   call mygetarg(i,buf1)
                   call keylowc(buf1)
                end if
                buf(j:) = " "
             else if( i.lt.iarg) then
                ! - -- or - - -
                i = i+1
                call mygetarg(i,buf1)
                if ( buf1(1:1) .eq. "-" ) then
                   i=i-1
                   buf1 = " "
                elseif (buf1 .eq. ",") then
                   ! - - -
                   i = i+1
                   call mygetarg(i,buf1)
                elseif (buf1(1:1) .eq. "," ) then
                   ! - --
                   buf1(1:1) = " "
                   call rmblnk(buf1)
                end if
             end if
             read(buf,*, iostat=ierr) opts%errorestimate
             if (ierr .ne. 0  .or.  opts%errorestimate .le. 0.d0 ) then
                msg = "Cannot read error-estimate after -d option. " &
                     // "Expecting positive real, not: "//trim(buf)
                call stop_code(mdl,rout,msg)
             endif
             if (buf1 .eq. "au" .or. buf1 .eq. " ") then
                continue
             elseif (buf1 .eq. "ev") then
                opts%errorestimate=opts%errorestimate/au2ev
             elseif (buf1 .eq. "mev") then
                opts%errorestimate=opts%errorestimate/(1.d3*au2ev)
             elseif (buf1 .eq. "cm-1") then
                opts%errorestimate=opts%errorestimate/au2ic
             else
                msg = "Expecting one of 'au', 'eV', 'meV',  or 'cm-1' " &
                     //"after '-s <real>,', not "//trim(buf1)
                call stop_code(mdl,rout,msg)
             end if

          elseif  ( buf .eq. '-H') then
             opts%lhermitian = .true.
          elseif  ( buf .eq. '-H1') then
             lh1 = .true.
             i=i+1
             call mygetarg(i,buf)
             read(buf,*, iostat=ierr) opts%inittype
             if (ierr .ne. 0) then
                msg = "Cannot read value after -H1 option. " &
                     // "Expecting integer, not: "//trim(buf)
                call stop_code(mdl,rout,msg)
             end if
          elseif  ( buf .eq. '-g') then
             opts%lgrow = .true.
             i=i+1
             call mygetarg(i,buf)
             read(buf,*, iostat=ierr) opts%grank
             if (ierr .ne. 0) then
                msg = "Cannot read first value after -g option. " &
                     // "Expecting integer, not: "//trim(buf)
                call stop_code(mdl,rout,msg)
             end if
             i=i+1
             call mygetarg(i,buf)
             read(buf,*, iostat=ierr) opts%giter
             if (ierr .ne. 0) then
                msg = "Cannot read second value after -g option. " &
                     // "Expecting integer, not: "//trim(buf)
                call stop_code(mdl,rout,msg)
             end if

          elseif  ( buf .eq. '-s') then
             buf1 = " "
             i=i+1
             call mygetarg(i,buf)
             j = index(buf,",")
             lcomma = j.ne.0

             if (lcomma) then
                call rmblnk(buf(j+1:))
                if (index(buf(j+1:), ' ') .gt.1) then
                   !---
                   buf1=buf(j+1:)
                   call keylowc(buf1)
                else
                   !-- -
                   i=i+1
                   call mygetarg(i,buf1)
                   call keylowc(buf1)
                end if
                buf(j:) = " "
             else if( i.lt.iarg) then
                ! - -- or - - -
                i = i+1
                call mygetarg(i,buf1)
                if(buf1(1:1) .eq. "-") then
                   i=i-1
                   buf1 = " "
                elseif (buf1 .eq. ",") then
                   ! - - -
                   i = i+1
                   call mygetarg(i,buf1)
                elseif(buf1(1:1) .eq. ",") then
                   ! - --
                   buf1(1:1) = " "
                   call rmblnk(buf1)
                end if
             end if

             read(buf,*, iostat=ierr) opts%deltastop
             if (ierr.ne.0) then
                msg = "expecting real after -s option, not:"//trim(buf)
                call stop_code(mdl,rout,msg)
             end if

             if (buf1 .eq. "au" .or. buf1 .eq. " ") then
                opts%deltastopunit = 1
             elseif (buf1 .eq. "ev") then
                opts%deltastopunit = 2
             elseif (buf1 .eq. "cm-1") then
                opts%deltastopunit = 3
             elseif (buf1 .eq. "mev") then
                opts%deltastopunit = 4
             else
                msg = "Expecting one of 'au', 'eV', or 'cm-1' " &
                     //"after '-s <real>,', not "//trim(buf1)
                call stop_code(mdl,rout,msg)
             end if

          elseif  ( buf .eq. '-n') then
             i=i+1
             call mygetarg(i,buf)
             read(buf,*, iostat=ierr) opts%maxiter
             if (ierr .ne. 0) then
                msg = "Cannot read iterations after -n option. " &
                     // "Expecting integer, not: "//trim(buf)
                call stop_code(mdl,rout,msg)
             end if

          elseif  ( buf .eq. '-spd') then
             lspd = .true.
             i=i+1
             call mygetarg(i,buf)
             read(buf,*, iostat=ierr) opts%alpha
             if (ierr .ne. 0) then
                msg = "Cannot read regularization parameter after " &
                     // "-spd option. Expecting real, not: "//trim(buf)
                call stop_code(mdl,rout,msg)
             endif

          elseif  ( buf .eq. '-ev') then
             leigen = .true.
             i=i+1
             call mygetarg(i,buf)
             read(buf,*, iostat=ierr) opts%alpha
             if (ierr .ne. 0) then
                msg = "Cannot read regularization parameter after " &
                     // "-ev option. Expecting real, not: "//trim(buf)
                call stop_code(mdl,rout,msg)
             endif

          elseif  ( buf .eq. '-D') then
             i=i+1
             call mygetarg(i,buf)
             opts%name = trim(buf)
             if (len_trim(opts%name) .eq. 0) then
                msg = "Expecting string after -D option"
                call stop_code(mdl,rout,msg)
             end if
             lname = .true.

          elseif  ( buf .eq. '-I') then
             i=i+1
             call mygetarg(i,buf)
             opts%inpdir = trim(buf)
             if (len_trim(opts%inpdir) .eq. 0) then
                msg = "Expecting string after -I option"
                call stop_code(mdl,rout,msg)
             end if
             linpdir = .true.
          elseif  ( buf .eq. '-J') then
             i=i+1
             call mygetarg(i,buf)
             opts%dvrdir = trim(buf)
             if (len_trim(opts%dvrdir) .eq. 0) then
                msg = "Expecting string after -J option"
                call stop_code(mdl,rout,msg)
             end if
             ldvrdir = .true.
          elseif  ( buf .eq. '-noblas') then
             opts%luseblas = .false.


          elseif  ( buf .eq. '-x') then
             opts%ldiis = .true.
             i=i+1
             call mygetarg(i,buf)
             read(buf,*, iostat=ierr) opts%ndiis
             if (ierr .ne. 0) then
                msg = "Cannot read diis space after -x option. " &
                     // "Expecting integer, not: "//trim(buf)
                call stop_code(mdl,rout,msg)
             end if
             if (opts%ndiis .le. 1) then
                msg = "Expecting integer > 1 after -x option, not: "//trim(buf)
                call stop_code(mdl,rout,msg)
             end if

          else
             msg = "Unknown option: "//trim(buf)
             call stop_code(mdl,rout,msg)
          end if
       else
          msg = "Unknown option: "//trim(buf)
          call stop_code(mdl,rout,msg)
       end if
    end do ! arg-loop

    if (lspd .and. leigen) then
       msg = "Only one of '-spd' and '-ev' can be used."
       call stop_code(mdl,rout,msg)
    end if

    opts%isolver = 1
    if (lspd) then
       opts%isolver = 1
    elseif (leigen) then
       opts%isolver = 2
    end if

    if (opts%lgrow .and.  .not. opts%lrank) then
       msg = "-g option requires -r option."
       call stop_code(mdl,rout,msg)
    end  if


    if (opts%lpes .and. opts%lhermitian) then
       msg = "-H option is ignored for -pes."
       opts%lhermitian = .false.
       call warn_msg(mdl,rout,msg)
    end if




    if (len_trim(opts%ham) .eq.0) then
       opts%ham = "system"
    end if


    ! check if name exists and is directory
    if (.not. lname) opts%name = "."
    lenname = len_trim(opts%name)
    call abspath(opts%name,lenname)

    if (.not. linpdir) opts%inpdir = "."
    leninpdir = len_trim(opts%inpdir)
    call abspath(opts%inpdir,leninpdir)

    if (.not. ldvrdir) then
       opts%dvrdir = opts%inpdir
    else
       lendvrdir = len_trim(opts%dvrdir)
       call abspath(opts%dvrdir,lendvrdir)
    end if

    if (opts%lhermitian .and. .not. opts%lrank) then
       msg = "Hermitian constraint (-H option) requires explicit rank (-r option)."
       call stop_code(mdl,rout,msg)
    end if

    if (opts%lhermitian) then
       if (modulo(opts%rank,2) .ne. 0) then
          msg = "Rank (-r option) must be even if -H is set"
          call stop_code(mdl,rout,msg)
       end if
       if (opts%lgrow .and. modulo(opts%grank,2) .ne. 0) then
          msg = "Rank increase (-g option) must be even if -H is set"
          call stop_code(mdl,rout,msg)
       end if
       if (lh1) then
          msg = "Options -H  and -H1 cannot be used together."
          call stop_code(mdl,rout,msg)
       end if
    end if

    if (lh1 .and. opts%lpes) then
       msg = "Option -H1 has no effect with -pes option." &
       // " Unsetting -H1 option."
       opts%inittype = 0
       call warn_msg(mdl,rout,msg)
    end if

    if (lh1 .and. opts%lcompress) then
       msg = "Using -H1 and -z option together is not implemented." &
       // " Feel free to do so."
       call stop_code(mdl,rout,msg)
    end if


    if (opts%inittype .lt. -1 .or. opts%inittype .gt. 1) then
       msg = "Init type (-H1 option) can only be -1, 0, or 1."
       call stop_code(mdl,rout,msg)
    end if


    if(opts%lcc) then
       lencc = len_trim(opts%ccfile)
       call abspath(opts%ccfile,lencc)
       if (opts%lpes) then
          opts%ccfile=trim(opts%ccfile)//"/cpd"
       else
          opts%ccfile=trim(opts%ccfile)//"/hcpd"
       end if
    endif

    opts%dvrfile = trim(opts%inpdir)//"/dvr"

    if (opts%lpes) then
       opts%operfilename = trim(opts%inpdir)//"/pes"
    else
       opts%operfilename = trim(opts%inpdir)//"/oper"
    end if

    if ( opts%lmnd) then
       inquire (file=trim( opts%name), exist=lexists)
       if (lexists) then
          write(6,'(a)') "Name-directory already exists: "//trim(opts%name)
       else
          inptit='mkdir '//trim(opts%name)
          call excmd2(ierr, inptit)
          if (ierr.ne.0) then
             write(6,'(2a)') '### ERROR: ', &
                  'Invalid path for name-directory (-mnd option). ###'
             call stopnow
          endif
          write(6,'(2a)') 'Created name-directory: ',trim(opts%name)
       end if
    end if


    if ( opts%lcontinue .and. .not.  opts%lcc) then
       if (opts%lpes) then
          opts%ccfile=trim(opts%name)//"/cpd"
       else
          opts%ccfile=trim(opts%name)//"/hcpd"
       end if
       lencc = len_trim(opts%ccfile)
    end if

    opts%hcpdfile = trim(opts%name)//      "/hcpd"
    opts%cpdfile = trim(opts%name)//       "/cpd"
    opts%logfile = trim(opts%name)//       "/log"
    opts%stopfile =  trim(opts%name)//     "/stop"
    opts%iterationsfile = trim(opts%name)//"/iterations"
    opts%timingfile = trim(opts%name)//    "/timing"



    inquire (file=opts%dvrfile, exist=lexists)
    if (.not. lexists ) then
       msg = "File "//trim(opts%dvrfile)//" not found."
       call stop_code(mdl,rout,msg)
    end if

    inquire (file=opts%operfilename, exist=lexists)
    if (.not. lexists ) then
       msg = "File "//trim(opts%operfilename)//" not found."
       call stop_code(mdl,rout,msg)
    end if

    inquire (file=opts%timingfile, exist=lexists)
    if (lexists  .and. .not. opts%lovwr) then
       msg = "Cannot overwrite "//trim(opts%timingfile)//". Use -w option."
       call stop_code(mdl,rout,msg)
    end if

    inquire (file=opts%hcpdfile, exist=lexists)
    if (lexists  .and. .not. opts%lovwr) then
       msg = "Cannot overwrite "//trim(opts%hcpdfile)//". Use -w option."
       call stop_code(mdl,rout,msg)
    end if

    if (opts%lpes) then
       inquire (file=opts%cpdfile, exist=lexists)
       if (lexists  .and. .not. opts%lovwr) then
          msg = "Cannot overwrite "//trim(opts%cpdfile)//". Use -w option."
          call stop_code(mdl,rout,msg)
       end if
    end if


    inquire (file=opts%logfile, exist=lexists)
    if (lexists  .and. .not. opts%lovwr) then
       msg = "Cannot overwrite "//trim(opts%logfile)//". Use -w option."
       call stop_code(mdl,rout,msg)
    end if

    inquire (file=opts%iterationsfile, exist=lexists)
    if (lexists  .and. .not. opts%lovwr) then
       msg = "Cannot overwrite "//trim(opts%iterationsfile)//". Use -w option."
       call stop_code(mdl,rout,msg)
    end if

    if (opts%lwrcoeff) then
       opts%coefffile = trim(opts%name)//       "/coefficients"
       inquire (file=opts%coefffile, exist=lexists)
       if (lexists  .and. .not. opts%lovwr) then
          msg = "Cannot overwrite "//trim(opts%coefffile)//". Use -w option."
          call stop_code(mdl,rout,msg)
       end if
    else
       opts%coefffile = " "
    end if

    if (opts%lwrovlp) then
       opts%ovlpfile = trim(opts%name)//       "/overlap"
       inquire (file=opts%ovlpfile, exist=lexists)
       if (lexists  .and. .not. opts%lovwr) then
          msg = "Cannot overwrite "//trim(opts%ovlpfile)//". Use -w option."
          call stop_code(mdl,rout,msg)
       end if
    else
       opts%ovlpfile = " "
    end if

    inquire (file=opts%stopfile, exist=lexists)
    if (lexists  .and. .not. opts%lovwr) then
       msg = "Cannot remove old stopfile "//trim(opts%stopfile)//". Use -w option."
       call stop_code(mdl,rout,msg)
    else
       open(newunit=istop, iostat=stat, file=opts%stopfile, status='unknown')
       if (stat .ne. 0) then
          msg = "Unknown error while opening file "//trim(opts%stopfile)//"."
          call stop_code(mdl,rout,msg)
       end if
       write(istop,'(a)',iostat=stat) "continue"
       if (stat .ne. 0) then
          msg = "Unknown error while writing file "//trim(opts%stopfile)//"."
          call stop_code(mdl,rout,msg)
       end if
       close(istop)
    end if

    open(newunit=ilog, file=trim(opts%logfile), iostat=ierr, form="formatted", &
         status="unknown")
    if (ierr .ne. 0) then
       msg = "Unknown error while opening file "//trim(opts%logfile)//"."
       call stop_code(mdl,rout,msg)
    end if


    call fpstartup(buf(1:100))
    i=index(buf(1:100),'   ')-2
    write(ilog,'(a)') buf(1:i)
    call wrversion(ilog)
    if (len_trim(options) .le. 60) then
       write(ilog,'(2a/)')   'Command line: ',trim(options)
    else
       write(ilog,'(a/,a/)') 'Command line: ',trim(options)
    endif
    call myflush(ilog)


    select case (opts%deltastopunit)
    case (1)
       continue
    case(2)
       opts%deltastop = opts%deltastop/au2ev
    case (3)
      opts% deltastop = opts%deltastop/au2ic
    case (4)
       opts%deltastop = opts%deltastop/(1.d3*au2ev)
    case default
       msg = "Delta-Stop unit unknown."
       call stop_code(mdl,rout,msg)
    end select

  end subroutine compactoper_inp

end module co_opts_m


