

module diistype_m
  implicit none

  type :: spt_t  ! single particle term type
     double precision, allocatable :: ospo(:)  ! old SPO
     double precision, allocatable :: dspo(:)  ! DIIS extrapolated SPO
     double precision, allocatable :: espo(:,:) ! error SPO
     double precision, allocatable :: aspo(:,:) ! Stores SPO of ndiis iteration
  end type spt_t

  type :: diis_t
     logical                        :: initialized
     logical                        :: lhcpdop
     integer                        :: nmode
     integer                        :: nspace
     integer, allocatable           :: subdim(:)
     integer, allocatable           :: subdim2(:)
     type(spt_t), allocatable       :: spt(:)
  end type diis_t

contains

   subroutine deallocate_diis_t(diis)
      implicit none
      type(diis_t), intent(INOUT)    :: diis
      integer                        :: m

      if (allocated(diis%spt)) then
         do m=1,diis%nmode
            if (allocated(diis%spt(m)%ospo)) deallocate(diis%spt(m)%ospo)
            if (allocated(diis%spt(m)%dspo)) deallocate(diis%spt(m)%dspo)
            if (allocated(diis%spt(m)%espo)) deallocate(diis%spt(m)%espo)
            if (allocated(diis%spt(m)%aspo)) deallocate(diis%spt(m)%aspo)
         enddo
         deallocate(diis%spt)
      endif
      if (allocated(diis%subdim)) deallocate(diis%subdim)
      if (allocated(diis%subdim2)) deallocate(diis%subdim2)
      diis%nspace = 0
      diis%nmode = 0
      diis%initialized = .false.

   end subroutine deallocate_diis_t


   subroutine allocate_diis_t(diis,nspace,nrank,nmode,subdim,lhcpdop)
      implicit none
      type(diis_t), intent(INOUT)    :: diis
      integer, intent(IN)            :: nrank,nmode,nspace
      integer, intent(IN)            :: subdim(nmode)
      logical, intent(IN)            :: lhcpdop

      integer                        :: m

      diis%lhcpdop = lhcpdop
      diis%nspace = nspace
      diis%nmode = nmode
      allocate(diis%spt(nmode))
      allocate(diis%subdim(nmode))
      allocate(diis%subdim2(nmode))
      do m=1,diis%nmode
         diis%subdim(m) = subdim(m)
         diis%subdim2(m) = subdim(m)**2
      enddo
      if (lhcpdop) then  ! For hcpd type
         do m=1,diis%nmode
            allocate(diis%spt(m)%ospo(subdim(m)**2*nrank))
            allocate(diis%spt(m)%dspo(subdim(m)**2*nrank))
            allocate(diis%spt(m)%espo(nspace,subdim(m)**2*nrank))
            allocate(diis%spt(m)%aspo(nspace,subdim(m)**2*nrank))
         enddo
      else  ! For cpd type
         do m=1,diis%nmode
            allocate(diis%spt(m)%ospo(subdim(m)*nrank))
            allocate(diis%spt(m)%dspo(subdim(m)*nrank))
            allocate(diis%spt(m)%espo(nspace,subdim(m)*nrank))
            allocate(diis%spt(m)%aspo(nspace,subdim(m)*nrank))
         enddo
      endif
      diis%initialized = .true.

   end subroutine allocate_diis_t


   subroutine diis_pack_hcpd(spo,xgdim,nrank,pspo)
      implicit none
      integer, intent(IN)            :: xgdim,nrank
      double precision, intent(IN)   :: spo(xgdim,xgdim,nrank)
      double precision, intent(OUT)  :: pspo(xgdim**2*nrank)
      pspo = reshape(spo,(/xgdim**2*nrank/))
   end subroutine diis_pack_hcpd
   subroutine diis_pack_cpd(spo,xgdim,nrank,pspo)
      implicit none
      integer, intent(IN)            :: xgdim,nrank
      double precision, intent(IN)   :: spo(xgdim,nrank)
      double precision, intent(OUT)  :: pspo(xgdim*nrank)
      pspo = reshape(spo,(/xgdim*nrank/))
   end subroutine diis_pack_cpd
   subroutine diis_unpack_hcpd(pspo,xgdim,nrank,spo)
      implicit none
      integer, intent(IN)            :: xgdim,nrank
      double precision, intent(IN)  :: pspo(xgdim**2*nrank)
      double precision, intent(OUT)   :: spo(xgdim,xgdim,nrank)
      spo = reshape(pspo,(/xgdim,xgdim,nrank/))
   end subroutine diis_unpack_hcpd
   subroutine diis_unpack_cpd(pspo,xgdim,nrank,spo)
      implicit none
      integer, intent(IN)            :: xgdim,nrank
      double precision, intent(IN)  :: pspo(xgdim*nrank)
      double precision, intent(OUT)   :: spo(xgdim,nrank)
      spo = reshape(pspo,(/xgdim,nrank/))
   end subroutine diis_unpack_cpd

   subroutine diis_saveold1m_hcpd(diis,nrank,m,spo)
      implicit none
      type(diis_t), intent(INOUT)    :: diis
      integer, intent(IN)            :: nrank,m
      double precision, intent(IN)   :: spo(diis%subdim(m),diis%subdim(m),nrank)
      !diis%spt(m)%ospo(:diis%subdim2(m)*nrank) = reshape(spo,(/diis%subdim2(m)*nrank/))
      call diis_pack_hcpd(spo,diis%subdim(m),nrank,diis%spt(m)%ospo(:diis%subdim2(m)*nrank))
   end subroutine diis_saveold1m_hcpd

   subroutine diis_saveold1m_cpd(diis,nrank,m,spo)
      implicit none
      type(diis_t), intent(INOUT)    :: diis
      integer, intent(IN)            :: nrank,m
      double precision, intent(IN)   :: spo(diis%subdim(m),nrank)
      call diis_pack_cpd(spo,diis%subdim(m),nrank,diis%spt(m)%ospo(:diis%subdim(m)*nrank))
      !diis%spt(m)%ospo(:diis%subdim(m)*nrank) = reshape(spo,(/diis%subdim(m)*nrank/))
   end subroutine diis_saveold1m_cpd
   subroutine diis_retnew1m_hcpd(diis,nrank,m,spo)
      implicit none
      type(diis_t), intent(IN)       :: diis
      integer, intent(IN)            :: nrank,m
      double precision, intent(OUT)  :: spo(diis%subdim(m),diis%subdim(m),nrank)
      !spo = reshape(diis%spt(m)%dspo(:diis%subdim2(m)*nrank), &
      !              (/diis%subdim(m),diis%subdim(m),nrank/))
      call diis_unpack_hcpd(diis%spt(m)%dspo(:diis%subdim2(m)*nrank),diis%subdim(m),nrank,spo)
   end subroutine diis_retnew1m_hcpd

   subroutine diis_retnew1m_cpd(diis,nrank,m,spo)
      implicit none
      type(diis_t), intent(IN)       :: diis
      integer, intent(IN)            :: nrank,m
      double precision, intent(OUT)   :: spo(diis%subdim(m),nrank)
      !spo = reshape(diis%spt(m)%dspo(:diis%subdim(m)*nrank),(/diis%subdim(m),nrank/))
      call diis_unpack_cpd(diis%spt(m)%dspo(:diis%subdim(m)*nrank),diis%subdim(m),nrank,spo)
   end subroutine diis_retnew1m_cpd


   subroutine diis_extrapolation_allmode(diis,nrank)
      implicit none
      type(diis_t), intent(INOUT)    :: diis
      integer, intent(IN)            :: nrank
      integer                        :: m,edim
      do m=1,diis%nmode
         if (diis%lhcpdop) then  ! hcpd type
            edim = diis%subdim2(m)*nrank
         else  ! cpd type
            edim = diis%subdim(m)*nrank
         endif
         call diis_extrapolation(diis%nspace,edim,diis%spt(m)%espo(:,:edim), &
                                 diis%spt(m)%aspo(:,:edim),diis%spt(m)%dspo(:edim))
      enddo
   end subroutine diis_extrapolation_allmode

   subroutine diis_extrapolation(ndiis,ndim,emat,hold,hnew)
      implicit none
      integer, intent(in)             :: ndiis,ndim
      double precision, intent(in)    :: emat(ndiis,ndim), hold(ndiis,ndim)
      double precision, intent(out)   :: hnew(ndim)

      integer              :: i,j,info,s
      double precision     :: summ
      integer              :: ipiv(ndiis+1)
      double precision     :: xmat(ndiis,ndiis), ymat(ndiis+1,ndiis+1), zmat(ndiis+1)

      xmat=0.d0
      call dgemm('n','t',ndiis,ndiis,ndim,1.d0,emat,ndiis,emat,ndiis,0.d0,xmat,ndiis)

      do i=1,ndiis
         do j=1,ndiis
            ymat(i,j)=xmat(i,j)
         enddo
      enddo

      do i=1,ndiis
         ymat(i,ndiis+1) = -1.d0
         ymat(ndiis+1,i) = -1.d0
      enddo
      ymat(ndiis+1,ndiis+1) = 0.d0

      zmat=0.d0
      zmat(ndiis+1) = -1.d0

      ipiv=0
      call dgesv(ndiis+1,1,ymat,ndiis+1,ipiv,zmat,ndiis+1,info)

      if(info.ne.0)then
         write(*,*)"Problem in the DIIS extrapolation step.."
         write(*,*)"'dgesv' info",info
         stop 888
      endif

      do i=1,ndim
         summ=0.d0
         do s=1,ndiis
            summ = summ + zmat(s)*hold(s,i)
         enddo
         hnew(i) = summ
      enddo
   end subroutine diis_extrapolation


   subroutine afterdiis_normalization(diis,nrank)
      implicit none
      type(diis_t), intent(INOUT)    :: diis
      integer, intent(IN)            :: nrank
      integer                        :: m,r,g,xgdim,nstart
      double precision               :: norm

      do m=1,diis%nmode
         if (diis%lhcpdop) then  ! hcpd type
            xgdim = diis%subdim2(m)
         else  ! cpd type
            xgdim = diis%subdim(m)
         endif
         do r=1,nrank
            nstart = (r-1)*xgdim
            norm = 0.d0
            do g=1,xgdim
               norm = norm + diis%spt(m)%dspo(nstart+g)**2
            enddo
            norm = dsqrt(norm)
            do g=1,xgdim
               diis%spt(m)%dspo(nstart+g) = diis%spt(m)%dspo(nstart+g)/norm
            enddo
         enddo  ! rank loop
      enddo  ! mode loop

   end subroutine afterdiis_normalization

end module diistype_m


