!#######################################################################
! npmminp.f90
!
! Input routines for npotminmax.f90
!
! 02/12 MS
!#######################################################################

      module npmminpmod

use decimal, only:dop,long
use global
use aglobal
use versions
use channels
use npotminmaxmod
use einpbasmod, only: einpbas
use dvrdatmod
use memdimmod, only: memdiminp

use logdat

use op1lib, only: zerovxi

      implicit none

      private
      public :: npmminput0,npmminput, setuptrajectory, setupcompare
      public :: maxnpotfiles,npotfnames,npotfiles
      contains

!#######################################################################
! Read data needed to set array sizes
!
!#######################################################################

      subroutine npmminput0

      implicit none

      integer(long)           :: iarg, myiargc, i,iz,ierr,ic
      integer(long)           :: ilbl,lnatpot
      integer(long)           :: maxdim0,maxsta0
      character(len=80)       :: buf
      logical(kind=4)         :: lrun,lnp
      character(len=c5)       :: filename,npotfile

      name = '  '
      trajfileinfo = '  '
      lcompare = .false.
      laenge  = 0
      lmnd = .false.
      lrddvr = .false.
      lrdoper = .false.
      lwrsrf = .false.
      lovwr = .false.
      lgnu = .false.
      lrun = .false.
      lnp = .false.

      ndatasets = 10

!-----------------------------------------------------------------------
! Read command input for file names etc.
!-----------------------------------------------------------------------
      iarg = myiargc()

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:3) .eq. '-D ') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
            laenge = index(buf,' ')-1
            call abspath(name,laenge)
         else if( buf(1:5) .eq. '-mnd ') then
            lmnd = .true.
         else if( buf(1:3) .eq. '-d ' ) then
            i=i+1
            call mygetarg(i,buf)
            dname = buf
            lrddvr = .true.
            dlaenge =  index(dname,' ')
            call abspath(dname,dlaenge)
         else if(buf(1:3) .eq. '-n ') then
            i=i+1
            call mygetarg(i,buf)
            read(buf,*, err=201) ndatasets
            if (ndatasets .le. 0) then
               write(*,*) "ERROR while reading option '-n num':"
               write(*,*) "      num must be a positive "//&
                    "Integer, not", ndatasets
               stop 1
            end if

         else if( buf(1:5) .eq. '-srf ' ) then
            lwrsrf = .true.
         else if( buf(1:3) .eq. '-w ' ) then
            lovwr=.true.
         else if( buf .eq. '-ver' ) then
            call wrversion(6)
            stop
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(2a,/10x,2a,/10x,a/)') 'Purpose  : Find lowest ', &
                  'and largest points on a surface generated by one ', &
                  ' or more natpot or CPD files. This is similar to ', &
                  'vminmax but ', &
                 ' skipping the generation of a PES file.'
            write(6,'(2a)') &
       'Usage    : npotminmax [-D -mnd -srf -d -ver -w -h -?] <inp>'
            write(6,'(/,a)') 'Options  :'
            write(6,'(2a,/,11x,a)') &
              ' -D DIR  : The output is directed to DIR.' &
              ,' This option overwrites "name" ' &
              ,'in the run-section of the input file.'  
            write(6,'(2a)') ' -mnd    : Enables creating a name', &
                 ' directory.'
            write(6,'(2a,/,11x,a)') ' -d DIR  : The new dvr is read ', &
                   'from directory DIR. This option overwrites ', &
                   '"readdvr" in the run-section of the input file.'

            write(6,'(2a,/,11x,a)') ' -srf    : Write potential ', &
                 'to ASCII file "srf.asc"', &
                 '(one energy per line like readsrf input).'
            write(6,'(2a)') &
                     ' -n num  : Print num maximal and minimal values.', &
                     ' The default is num=10.'
            write(6,'(a)') &
                     ' -ver    : Version information about the program.'  
            write(6,'(2a)') &
                     ' -w      : Existing output is overwritten.'
            write(6,'(a)') ' -h -?   : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop

         else
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (i .eq. iarg) goto 200
         goto 100
      end if

 200  continue
      call mygetarg(i,filein)
      laein=index(filein,' ')-1
      call abspath(filein,laein)
      buf = filein(laein-3:laein)
      call keylowc(buf)
      if (buf(1:4) .ne. '.inp') then
         write(*,*) "ERROR: input file must have the ending '.inp'."
         write(*,*) "       Wrong file: " //filein(1:laein)
         stop 1
      end if

      if (name .eq. ' ') then
         routine = "npmminput0"
         message = "No name directory was given."
         call errormsg
      end if
      if (dname .eq. ' ') then
         dname = name
         dlaenge = laenge
      endif
      if (npotfiles .eq. 0) then
         routine = "npmminput0"
         message = "No natpot files have been specified."
         call errormsg
      end if

!---------------------------------------------------------------------------------
! read input file for file paths and memory allocation data
!---------------------------------------------------------------------------------
      iz=0

      inputfile=filein
      inplaenge=laein
      open(iin,file=filein(1:laein),status='old',form='formatted',err=140)

      do
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .eq. 1) then
            routine='npmminp0'
            message = inptit
            call errormsg
         endif
         if (keyword(1).eq.'end-input') then
            exit
         elseif (keyword(1).eq.'run-section') then
            lrun = .true.
            call npmmeinrun0

         elseif (keyword(1).eq.'natpot-section' .or. &
                 keyword(1).eq.'cpd-section') then
            lnp = .true.
            call npmmeinnat0(npotfile)

            lnatpot = index(npotfile,' ')-1
            call abspath(npotfile,lnatpot)
            npotfile = npotfile(1:lnatpot)//'natpot'
            lnatpot = index(npotfile,' ')-1
         endif
      enddo

      if (.not. lrun) goto 141
      if (.not. lnp) goto 142

!-----------------------------------------------------------------------
! Get maxdim from dvr or input fie
!-----------------------------------------------------------------------
      if (lrddvr) then
         filename=dname(1:dlaenge)//'/dvr'
         ilbl=index(filename,' ')-1
         open(idvr,file=filename(1:ilbl),form='unformatted',status='old',&
               err=1000)
         call rdmemdim(idvr)
         close(idvr)
      else
         call memdiminp(iin)
      endif
      maxdim0 = maxdim
      maxsta0 = maxsta

!-----------------------------------------------------------------------
! Get version of natpot file (i.e. code used to create it)
!-----------------------------------------------------------------------
      open(inpot,file=npotfile,status='old',form='unformatted',&
           iostat=ierr)
      if (ierr.ne.0) then
          message='Could not open natpot file '//trim(npotfile)
          call errormsg
      endif
      read(inpot) filever(inpot)

!-----------------------------------------------------------------------
! Store name of alloc file if natpot from ver86 run
!-----------------------------------------------------------------------
      if (filever(inpot) .gt. 8.6d0 .and. filever(inpot) .lt. 8.65d0) then
         call dirpath(npotfile,lnatpot) ! remove filename from path
         alloc86file=npotfile(1:lnatpot)//'alloc'
      endif

!-----------------------------------------------------------------------
! get basic dimensions (maxdim, maxsta, etc)
!-----------------------------------------------------------------------
      call rdmemdim(inpot)
      close(inpot)

!-----------------------------------------------------------------------
! Restore complete system sizes
!-----------------------------------------------------------------------
      maxdim = maxdim0
      maxsta = maxsta0

      close(iin)
      return

 140  continue
      write(*,*) "ERROR: input file could not be opened."
      write(*,*) "       File: " //filein(1:laein)
      stop 1

 141  continue
      write(*,*) "ERROR: input does not contain a RUN-SECTION."
      write(*,*) "       File: " //filein(1:laein)
      stop 1

 142  continue
      write(*,*) &
        "ERROR: input does not contain a NATPOT- or CPD-SECTION."
      write(*,*) "       File: " //filein(1:laein)
      stop 1

 201  continue
      write(*,*) "ERROR: Could not read option '-n num'."
      write(*,*) "       num does not seem to be intger:"
      write(*,*) "       mum = "//buf
      stop 1

 1000 continue
      ilbl=index(filename,' ')-1
      routine='npmminp0'
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop

      end subroutine npmminput0

!#######################################################################
! 
! Read Input Data
!
!#######################################################################

      subroutine npmminput

      implicit none

      integer(long)           :: ic,iz,ierr,i
      logical(kind=4)         :: lpbas, linperr
      character(len=80)       :: buf

      lpbas = .false.
      linperr = .false.

      do i=1,maxnpotfiles
         npotfnames(i) = " "
      end do

      routine="npmminp (npmminp.F)"

      inputfile=filein
      inplaenge=laein
      open(iin,file=filein(1:laein),status='old',form='formatted',err=140)

      rewind(iin)
      iz=0
 110  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine='npmminp'
         message = inptit
         call errormsg
      endif

      if (keyword(1)(1:11) .eq. 'run-section') then
         call npmmeinrun()
      else if (keyword(1) .eq. 'end-input') then
         go to 115
      endif
      go to 110
 115  continue
      
      rewind(iin)
      iz=0

 120  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine='npmminp'
         message = inptit
         call errormsg
      endif

      if ((keyword(1)(1:14) .eq. 'natpot-section') .or. &
         (keyword(1)(1:14) .eq. 'cpd-section'))  then
         call npmmeinnat()
      else if (keyword(1) .eq. 'end-input') then
         go to 125
      endif
      go to 120

 125  continue

      rewind(iin)
      iz=0

 130  continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine='npmminp'
         message = inptit
         call errormsg
      endif


      if (keyword(1) .eq. 'pbasis-section' .or. &
           keyword(1) .eq. 'primitive-basis-section') then
         lpbas = .true.
         call einpbas(linperr,iz)
         irundvr = 1
         if (lrddvr .and. (irundvr .eq. 1)) then
            irundvr = 2
            routine = "npmminp"
            message = "Found keyword 'readdvr' in RUN-SECTION. "// &
                 "Ignoring PRIMITIVE-BASIS-SECTION."
            call warnmsg
         end if

      else if (keyword(1) .eq. 'end-input') then
         go to 135
      endif
      go to 130

 135  continue

! we are done
      close (iin)
      if (.not. lpbas .and. .not. lrddvr) goto 143
      return

 140  continue
      write(*,*) "ERROR: input file could not be opened."
      write(*,*) "       File: " //filein(1:laein)
      stop 1


 143  continue
      write(*,*) "ERROR: input does not contain a PRIMITIVE-BASIS"// &
           "-SECTION or a 'readdvr' statement in the RUN-SECTION."
      write(*,*) "       File: " //filein(1:laein)
      stop 1

      end subroutine npmminput


!#######################################################################
! Read input file: RUN-SECTION 
! Get file paths and data needed for memory allocation
!#######################################################################

      subroutine npmmeinrun0

      implicit none

      integer(long) :: ic,iz,ierr,i,iz0
      logical lexist

      i=1
!!    iz=0
      iz0=0
      numcomp=0
      maxtraj=0
 210  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine='npmmeinrun'
         message = inptit
         call errormsg
      endif
      i=0

 211  continue
      i = i+1

      if (keyword(i).eq.'end-run-section') then
         goto 220
      elseif (keyword(i).eq.'name') then
         if (keyword(i+1).ne.'=') then
            routine='npmmeinrun'
            message = 'No argument specified with name keyword'
            call warnmsg
         else
            i=i+2
            if (laenge .eq. 0) then
               name = keyorig(i)
               laenge = lc(i)
               if (laenge .eq. 0) then
                  routine='npmmeinrun'
                  message = ' No path name specified'
                  call wrline(zeroi,iz,iz0)
               end if
               call abspath(name,laenge)
            endif
         endif

      else if (keyword(i) .eq. 'readdvr') then
         if (.not. lrddvr) then
            !otherwise we would overwrite command line options
            lrddvr=.true.
            if (keyword(i+1) .eq. '=') then
               i=i+2
               dname=keyorig(i)
               dlaenge = lc(i)
               call abspath(dname,dlaenge)
               lrddvr = .true.
            else
               routine='npmmeinrun'
               message = 'No argument specified with keyword "readdvr".'
               call errormsg
            end if

         endif

      else if( keyword(i) .eq. 'overwrite' )  then
         lovwr = .true.

      else if( keyword(i) .eq. 'trajectory' )  then
         maxtraj=maxtraj+1

      else if( keyword(i) .eq. 'compare' )  then
         lcompare = .true.
         numcomp=numcomp+1

      else if( keyword(i) .eq. 'wrsrf' )  then
         lwrsrf = .true.
      end if

!     check if more keywords are to be read, else read new line
      if (i.lt.ic) then
         i=i+1
         go to 211
      else
         go to 210
      endif

 220  continue
      if(lcompare .and. numtraj.ne.numcomp) then
         routine='npmmeinrun0'
         message='There must be as many compare files (or none)'//&
                 ' as trajectory files!'
         call errormsg
      endif
      return

      end subroutine npmmeinrun0

!#######################################################################
! Read input file: RUN-SECTION
!#######################################################################

      subroutine npmmeinrun

      implicit none

      integer(long) :: ic,iz,ierr,i,iz0
      logical lexist

      i=1
!!    iz=0
      iz0=0
      numtraj=0
      numcomp=0
 210  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine='npmmeinrun'
         message = inptit
         call errormsg
      endif
      i=0

 211  continue
      i = i+1

      if (keyword(i).eq.'end-run-section') then
         goto 220

      else if( keyword(i) .eq. 'trajectory' )  then
         if (keyword(i+1).ne.'=') then
            routine='npmmeinrun'
            message = 'No argument specified with keyword "trajectory".'
            call errormsg
         else
            ltrajectory = .true.
            numtraj=numtraj+1
            trajfile(numtraj) = trim(adjustl(keyorig(i+2)))
            tlaenge = lc(i+2)
            if (tlaenge .eq. 0) then
               routine='npmmeinrun'
               message = ' No trajectory file specified'
               call wrline(zeroi,iz,iz0)
            end if
            call abspath(trajfile(numtraj),tlaenge)
            i=i+2
         endif

      else if( keyword(i) .eq. 'compare' )  then
         if (keyword(i+1).ne.'=') then
            routine='npmmeinrun'
            message = 'No argument specified with keyword: "compare".'
            call errormsg
         else
            lcompare = .true.
            numcomp=numcomp+1
            compfile(numcomp) = trim(adjustl(keyorig(i+2)))
            claenge = lc(i+2)
            if (claenge .eq. 0) then
               routine='npmmeinrun'
               message = ' No surface file specified'
               call wrline(zeroi,iz,iz0)
            end if
            call abspath(compfile(numcomp),claenge)
            i=i+2
         endif

      end if

!     check if more keywords are to be read, else read new line
      if (i.lt.ic) then
         i=i+1
         go to 211
      else
         go to 210
      endif

 220  continue
      if(lcompare .and. numtraj.ne.numcomp) then
         routine='npmmeinrun'
         message='There must be as many compare files (or none)'//&
                 ' as trajectory files!'
      endif
      return

      end subroutine npmmeinrun

!#######################################################################
! Read input file: NATPOT-SECTION
! read data for memory allocation
! npotfile is a natpot filename to read for memory sizes
!#######################################################################

      subroutine npmmeinnat0(npotfile)

      implicit none

      integer(long) :: ic,iz,ierr,i,klaenge
      character(len=c5), intent(out) :: npotfile
      logical lexist

!!    iz=0
      if(keyword(1).eq.'natpot-section') then
         lcpd=.false.
      elseif(keyword(1).eq.'cpd-section') then
         lcpd=.true.
      else
         write(6,*) 'Cannot happen! subroutine npmmeinnat'
      endif

      maxnpotfiles = 0
      npotfile = ' '
 310  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine='npmmeinnat'
         message = inptit
         call errormsg
      endif

      i=0

 311  continue
      i = i+1

      if (.not.lcpd .and. keyword(i).eq.'end-natpot-section') then
         goto 320
      elseif (lcpd .and. keyword(i).eq.'end-cpd-section') then
         goto 320
      else if (keyword(i) .ne. ' ' )  then
         maxnpotfiles = maxnpotfiles + 1
         npotfile = keyorig(i)
      end if

!     check if more keywords are to be read, else read new line
      if (i.lt.ic) then
         i=i+1
         go to 311
      else
         go to 310
      endif

 320  continue

      return
      end subroutine npmmeinnat0

!#######################################################################
! Read input file: NATPOT-SECTION
!#######################################################################

      subroutine npmmeinnat

      implicit none

      integer(long) :: ic,iz,ierr,i,klaenge
      integer(long) :: natpotfiles
      logical lexist

      natpotfiles = 0
 310  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine='npmmeinnat'
         message = inptit
         call errormsg
      endif

      i=0

 311  continue
      i = i+1

      if (.not.lcpd .and. keyword(i).eq.'end-natpot-section') then
         goto 320
      elseif (lcpd .and. keyword(i).eq.'end-cpd-section') then
         goto 320
      else if (keyword(i) .ne. ' ' )  then
         klaenge = index(keyorig(i),' ')-1
         npotfiles = npotfiles + 1
         npotfnames(npotfiles) = keyorig(i)
         np_lcpd(npotfiles)=lcpd
         call abspath(npotfnames(npotfiles),klaenge)
      end if

!     check if more keywords are to be read, else read new line
      if (i.lt.ic) then
         i=i+1
         go to 311
      else
         go to 310
      endif

 320  continue

      return
      end subroutine npmmeinnat

!######################################################################

      subroutine setuptrajectory

      implicit none

      character(len=c5) :: buf, buf1, buf2
      integer(long)     :: f,ilbl, jlbl, klbl, itraj, icount
      integer(long)     :: idum(maxdim), ntdof, nline
      logical           :: lstart

      routine = "setuptrajectory (npotminmax.F)"

      tottrajpoints = 0
      write(ilog,*)
      write(6, '(a)')

      do icount=1,numtraj
         lstart = .true.
         ntrajpoints(icount) = 0
         itraj = 12
         trajfileinfo  = " -- N/A --"
         open(itraj,file=trajfile(icount),err=222)
 202     continue
         nline = nline + 1
         read(itraj,"(a)",err=223,end=223) buf
         buf2 = adjustl(buf)
         if (buf2(1:1) .eq. "#") then
            if (lstart) then
               lstart = .false.
               buf2(1:1) = " "
               buf2 = adjustl(buf2)
               trajfileinfo = trim(buf2)
            end if
            goto 202
         end if

         call zerovxi(idum,maxdim)

         read(buf,*,end=201) idum
 201     continue
         ntdof = 0
         do f=1,maxdim
            if (idum(f).ne.0) then
               ntdof = ntdof + 1
            end if
         end do
         if (ntdof .ne. ndof_sys) then
            routine = "setuptrajectory (npotminmax.F)"
            message = "Number of indices in file "// &
                      trim(trajfile(icount))// &
                      " does not match system."
            call errormsg
         end if

         ntrajpoints(icount) = 1
 209     continue
         read(itraj,'(a)',err=223,end=211) buf
         nline = nline + 1
         buf = adjustl(buf)
         if (buf(1:1) .eq. "#") goto 209
         read(buf,*,err=223)(idum(f), f=1, ndof_sys)
         ntrajpoints(icount) = ntrajpoints(icount) + 1
         do f=1,ndof_sys
            if (idum(f) .gt. gdim_sys(f) .or. idum(f) .lt. 1) goto 224
         end do
         goto 209


 211     continue
         close(itraj)
         write(buf,*) ntrajpoints(icount)
         call rmblnk(buf)
         ilbl = index(buf," ")

         write(ilog, '(a,i8,a)') trim(trajfile(icount))//" contains " &
                      //buf(1:ilbl)//"index points."
         write(ilog, '(a)') "File info: "//trim(trajfileinfo)
         write(ilog,*)

!         write(6, '(a,i8,a)') trim(trajfile(icount))//" contains " &
!                   //buf(1:ilbl)//"index points."
!         write(6, '(a)') "File info: "//trim(trajfileinfo)
!         write(6, '(a)')

         tottrajpoints = tottrajpoints + ntrajpoints(icount)

      enddo  ! icount=1,numtraj

      return

 222  continue
      message = "Could not open file "//trim(trajfile(icount))
      call errormsg
      stop 1

 223  continue
      close(itraj)
      write(buf,*) nline
      call rmblnk(buf)
      ilbl = index(buf," ")
      message = "File "//trim(trajfile(icount))//" corrupted near" &
           //" line "//buf(1:ilbl)//"."
      call errormsg
      stop 1

 224  continue
      close(itraj)
      write(buf,*) ntrajpoints(icount)
      call rmblnk(buf)
      ilbl = index(buf," ")
      write(buf1,*) f
      call rmblnk(buf1)
      jlbl = index(buf1," ")
      klbl = index(modelabel(f)," ")

      message = "File "//trim(trajfile(icount))//" line "// &
           buf(1:ilbl)//": Index "//buf1(1:jlbl)// &
           "('"//modelabel(f)(1:klbl-1)//"') is out of range."
      call errormsg

      end subroutine setuptrajectory


!#######################################################################

      subroutine setupcompare

      implicit none

      integer(long)     :: ilbl, icount
      character(len=c5) :: buf, fileinfo
      real(dop)         :: rdum
      logical           :: lstart

      routine = "setupcompare (npotminmax.F)"
      do icount=1,numcomp
         ncomppoints(icount) = 0
         fileinfo = " -- N/A --"
         lstart = .true.
         open(idump, file=compfile(icount), err=322)

 309     continue
         read(idump,'(a)',err=323,end=311) buf
         buf = adjustl(buf)
         if (lstart) then
            if (buf(1:1) .eq. "#") fileinfo(1:c5-1) = buf(2:c5)
            lstart = .false.
         end if
         if (buf(1:1) .eq. "#") goto 309
         read(buf,*,err=323,end=311)  rdum
         ncomppoints(icount) = ncomppoints(icount) + 1
         goto 309


 311     continue
         close(idump)
         write(buf,*) ncomppoints(icount)
         call rmblnk(buf)
         ilbl = index(buf," ")
         write(ilog, '(a,i8,a)') trim(compfile(icount))//" contains " &
                        //buf(1:ilbl)//"energy values."
         write(ilog, '(a,i8,a)')"File info: "//trim(fileinfo)
         write(ilog,*)

!         write(6, '(a,i8,a)') trim(compfile(icount))//" contains " &
!           //buf(1:ilbl)//"energy values."
!         write(6, '(a,i8,a)')"File info: "//trim(fileinfo)
!         write(6,*)


         write(buf,*) ntrajpoints(icount)
         call rmblnk(buf)
         ilbl = index(buf," ")

         if (ncomppoints(icount) .lt. ntrajpoints(icount)) then
            if (ltrajectory) then
               message =  "Number of points in file" // &
                          trim(compfile(icount))// &
                          "does not match the trajectory lenth."

            else
               message =  "Number of points in file" // &
                          trim(compfile(icount))// &
                          "does not match the number of grid points."
            end if
            call errormsg
         elseif (ncomppoints(icount) .gt. ntrajpoints(icount)) then
            if (ltrajectory) then
               message =  "Number of points in file" // &
                      trim(compfile(icount))// &
                      "is larger than the trajectory lenth."

            else 
               message =  "Number of points in file" // &
                       trim(compfile(icount))// &
                       "is larger than the number of grid points."
            end if
            call warnmsg
         end if
      enddo

      return

 322  continue
      message = "Could not open file "//trim(compfile(icount))
      call errormsg
      stop 1

 323  continue
      close(idump)
      write(buf,*) ntrajpoints(icount)
      call rmblnk(buf)
      message = "File "//trim(compfile(icount))// &
                " corrupted near line "//trim(buf)//"."
      call errormsg
      stop 1

      end subroutine setupcompare

!#######################################################################
      end module npmminpmod
