! ------------------------------------------------------------------------
! Include file for npotminmax program
! ------------------------------------------------------------------------

      module npotminmaxmod 

use decimal, only: long,dop,vlong
use maxv, only:maxdim
use global

      implicit none

      integer(long), save :: maxnpotfiles=0 
      integer(long), save :: maxtraj=0 
      integer(long), save      :: npotfiles

! file names and pointers of input

      character*(c5), dimension(:), allocatable, save ::  npotfnames
      character*(c5), dimension(:), allocatable, save :: trajfile, compfile
      character(len=c5), save                         :: trajfileinfo
      logical(kind=4), save                 :: ltrajectory, lcompare, lmnd, &
                                               lcpd
      integer(long), save                   :: tlaenge, claenge, cpdrank, &
                                               zpevec, sumrank, numtraj, &
                                               numcomp

 ! pointers and dimensions of current natpot
      integer(long), dimension(:), allocatable, save   :: fdvr, mdvr, mmode
      integer(long), dimension(:), allocatable, save   :: ncomppoints
      logical(kind=4), dimension(:), allocatable, save :: loned

      integer(long), dimension(:), allocatable, save   :: ntrajpoints
      integer(long), save                               :: tottrajpoints

      real(dop), dimension(:), allocatable, save :: mean, rms
      real(dop), save                            :: totmean, totrms, wtim0

      integer(long), dimension(:), allocatable, save   :: pevec

! pointers and dimensions of all natpots

      integer(long), dimension(:), allocatable, save     :: np_zcoeffstart
      integer(long), dimension(:), allocatable, save     :: np_zevecstart
      integer(long), dimension(:), allocatable, save     :: np_evecdim
      integer(long), save                                :: np_totevecdim
      integer(long), dimension(:), allocatable, save     :: np_coeffdim
      integer(long), save                                :: np_totcoeffdim

      integer(long), dimension(:,:), allocatable, save   :: np_zevec
      integer(long), dimension(:,:), allocatable, save   :: np_fdvr
      logical(kind=4), dimension(:,:), allocatable, save :: np_loned

      integer(long), dimension(:), allocatable, save     :: np_oneddim
      integer(long), save                                :: np_totoneddim
      integer(long), dimension(:), allocatable, save     :: np_zonedstart

      integer(long), dimension(:), allocatable, save     :: np_ndof
      integer(long), dimension(:), allocatable, save     :: gdim_sys
      integer(long), save                                :: ndof_sys

      integer(long), dimension(:,:), allocatable, save   :: np_nspfdof
      integer(long), dimension(:,:,:), allocatable, save :: np_spfdof
      integer(long), dimension(:,:), allocatable, save   :: np_psubdim

      integer(long), dimension(:,:,:), allocatable, save :: np_potdof
      integer(long), dimension(:,:), allocatable, save   :: np_npotdof
      integer(long), dimension(:,:), allocatable, save   :: np_potdim

      integer(long), dimension(:), allocatable, save     :: np_modc
      integer(long), dimension(:), allocatable, save     :: np_nmode
      integer(long), dimension(:), allocatable, save     :: np_cpdrank

      integer(long), dimension(:), allocatable, save     :: np_nodbef,np_nodaft
      integer(long), dimension(:), allocatable, save     :: np_dimbef,np_dimaft

      integer(long), dimension(:,:), allocatable, save   :: np_c2v

      logical(kind=4), dimension(:), allocatable, save   :: np_ploned
      logical(kind=4), dimension(:), allocatable, save   :: np_lcpd

! Statistics

      integer(long), save :: ndatasets, currndatasets

      ! smalles values (sum)
      integer(long), dimension(:,:), allocatable, save  :: dsidxsv
      integer(long), dimension(:), allocatable, save    :: dsIDsv
      real(dop),dimension(:,:), allocatable, save       :: dsVnpsv
      real(dop),dimension(:,:), allocatable, save       :: dsV1Dsv
      real(dop),dimension(:), allocatable, save         :: dsVsv
      real(dop),dimension(:), allocatable, save         :: dsVCs

      ! s mallest absolute values (sum)
      integer(long), dimension(:,:), allocatable, save  :: dsidxabssv
      integer(long), dimension(:), allocatable, save    :: dsIDabssv
      real(dop),dimension(:,:), allocatable, save       :: dsVnpabssv
      real(dop),dimension(:,:), allocatable, save       :: dsV1Dabssv
      real(dop),dimension(:), allocatable, save         :: dsVabssv
      real(dop),dimension(:), allocatable, save         :: dsVCa

      ! largest values (sum)
      integer(long), dimension(:,:), allocatable, save  :: dsidxlv
      integer(long), dimension(:), allocatable, save    :: dsIDlv
      real(dop),dimension(:,:), allocatable, save       :: dsVnplv
      real(dop),dimension(:,:), allocatable, save       :: dsV1Dlv
      real(dop),dimension(:), allocatable, save         :: dsVlv
      real(dop),dimension(:), allocatable, save         :: dsVCl

      ! extremes (single natpots)
      integer(long), dimension(:,:), allocatable, save  :: np_largestidx
      integer(long), dimension(:,:), allocatable, save  :: np_lowestidx
      integer(long), dimension(:,:), allocatable, save  :: np_smallestidx
      real(dop),dimension(:,:), allocatable, save       :: np_largestV
      real(dop),dimension(:,:), allocatable, save       :: np_lowestV
      real(dop),dimension(:,:), allocatable, save       :: np_smallestV
      integer(long), dimension(:), allocatable, save    :: np_largestID
      integer(long), dimension(:), allocatable, save    :: np_lowestID
      integer(long), dimension(:), allocatable, save    :: np_smallestID

      ! current natpot
      real(dop),dimension(:,:), allocatable, save       :: npV

      contains

!#######################################################################
!
! NPOTMINMAXMOD_ALLOC
!
! memory required for arrays in npotminmaxmodmod allocated here.
!
! First check if allocated, then allocate. All arrays must be
! deallocated in
! npotminmaxmod_dealloc
!
!#######################################################################

      subroutine alloc_npotminmaxmod

use maxv
use channels

      implicit none

      integer(long)     :: ierr
      character(len=15) :: cerr

! check all arrays are not allocated
      call dealloc_npotminmaxmod

      cerr = 'fdvr'
      allocate(fdvr(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'mdvr'
      allocate(mdvr(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'mmode'
      allocate(mmode(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'loned'
      allocate(loned(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'compfile'
      allocate(compfile(maxtraj),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ntrajpoints'
      allocate(ntrajpoints(maxtraj),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'npotfnames'
      allocate(npotfnames(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ncomppoints'
      allocate(ncomppoints(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'mean'
      allocate(mean(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'rms'
      allocate(rms(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_zcoeffstart'
      allocate(np_zcoeffstart(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_zevecstart'
      allocate(np_zevecstart(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_evecdim'
      allocate(np_evecdim(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_coeffdim'
      allocate(np_coeffdim(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_zevec'
      allocate(np_zevec(maxdim,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_fdvr'
      allocate(np_fdvr(maxdim,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_loned'
      allocate(np_loned(maxdim,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_oneddim'
      allocate(np_oneddim(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_zonedstart'
      allocate(np_zonedstart(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_ndof'
      allocate(np_ndof(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'gdim_sys'
      allocate(gdim_sys(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_nspfdof'
      allocate(np_nspfdof(maxdim,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_spfdof'
      allocate(np_spfdof(maxdim,maxdim,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_psubdim'
      allocate(np_psubdim(maxdim,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_potdof'
      allocate(np_potdof(maxdim,maxdim,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_npotdof'
      allocate(np_npotdof(maxdim,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_potdim'
      allocate(np_potdim(maxdim,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_modc'
      allocate(np_modc(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_nmode'
      allocate(np_nmode(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_cpdrank'
      allocate(np_cpdrank(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_lcpd'
      allocate(np_lcpd(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_ploned'
      allocate(np_ploned(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_nodbef'
      allocate(np_nodbef(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_nodaft'
      allocate(np_nodaft(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_dimbef'
      allocate(np_dimbef(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_dimaft'
      allocate(np_dimaft(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_c2v'
      allocate(np_c2v(maxdim,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsidxsv'
      allocate(dsidxsv(maxdim,ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsIDsv'
      allocate(dsIDsv(ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsVnpsv'
      allocate(dsVnpsv(maxnpotfiles,ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsV1Dsv'
      allocate(dsV1Dsv(maxnpotfiles,ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsVsv'
      allocate(dsVsv(ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsidxabssv'
      allocate(dsidxabssv(maxnpotfiles,ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsIDabssv'
      allocate(dsIDabssv(ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsVnpabssv'
      allocate(dsVnpabssv(maxnpotfiles,ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsV1Dabssv'
      allocate(dsV1Dabssv(maxnpotfiles,ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsVabssv'
      allocate(dsVabssv(ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsidxlv'
      allocate(dsidxlv(maxnpotfiles,ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsIDlv'
      allocate(dsIDlv(ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsVnplv'
      allocate(dsVnplv(maxnpotfiles,ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsV1Dlv'
      allocate(dsV1Dlv(maxnpotfiles,ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dsVlv'
      allocate(dsVlv(ndatasets),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_largestidx'
      allocate(np_largestidx(maxdim,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_lowestidx'
      allocate(np_lowestidx(maxdim,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_smallestidx'
      allocate(np_smallestidx(maxdim,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_largestV'
      allocate(np_largestV(3,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_lowestV'
      allocate(np_lowestV(3,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_smallestV'
      allocate(np_smallestV(3,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_largestID'
      allocate(np_largestID(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_lowestID'
      allocate(np_lowestID(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'np_smallestID'
      allocate(np_smallestID(maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'npV'
      allocate(npV(3,maxnpotfiles),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_NPOTMINMAXMOD :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_npotminmaxmod

!#######################################################################

      subroutine dealloc_npotminmaxmod

      implicit none

      if (allocated(fdvr)) deallocate(fdvr)
      if (allocated(mdvr)) deallocate(mdvr)
      if (allocated(mmode)) deallocate(mmode)
      if (allocated(loned)) deallocate(loned)
      if (allocated(mean)) deallocate(mean)
      if (allocated(rms)) deallocate(rms)
      if (allocated(np_zcoeffstart)) deallocate(np_zcoeffstart)
      if (allocated(np_zevecstart)) deallocate(np_zevecstart)
      if (allocated(np_evecdim)) deallocate(np_evecdim)
      if (allocated(np_coeffdim)) deallocate(np_coeffdim)
      if (allocated(np_zevec)) deallocate(np_zevec)
      if (allocated(np_fdvr)) deallocate(np_fdvr)
      if (allocated(np_loned)) deallocate(np_loned)
      if (allocated(np_oneddim)) deallocate(np_oneddim)
      if (allocated(np_zonedstart)) deallocate(np_zonedstart)
      if (allocated(np_ndof)) deallocate(np_ndof)
      if (allocated(gdim_sys)) deallocate(gdim_sys)
      if (allocated(np_nspfdof)) deallocate(np_nspfdof)
      if (allocated(np_spfdof)) deallocate(np_spfdof)
      if (allocated(np_psubdim)) deallocate(np_psubdim)
      if (allocated(np_potdof)) deallocate(np_potdof)
      if (allocated(np_npotdof)) deallocate(np_npotdof)
      if (allocated(np_potdim)) deallocate(np_potdim)
      if (allocated(np_modc)) deallocate(np_modc)
      if (allocated(np_nmode)) deallocate(np_nmode)
      if (allocated(np_nodbef)) deallocate(np_nodbef)
      if (allocated(np_nodaft)) deallocate(np_nodaft)
      if (allocated(np_dimbef)) deallocate(np_dimbef)
      if (allocated(np_dimaft)) deallocate(np_dimaft)
      if (allocated(np_c2v)) deallocate(np_c2v)
      if (allocated(dsidxsv)) deallocate(dsidxsv)
      if (allocated(dsIDsv)) deallocate(dsIDsv)
      if (allocated(dsVnpsv)) deallocate(dsVnpsv)
      if (allocated(dsV1Dsv)) deallocate(dsV1Dsv)
      if (allocated(dsVsv)) deallocate(dsVsv)
      if (allocated(dsidxabssv)) deallocate(dsidxabssv)
      if (allocated(dsIDabssv)) deallocate(dsIDabssv)
      if (allocated(dsVnpabssv)) deallocate(dsVnpabssv)
      if (allocated(dsV1Dabssv)) deallocate(dsV1Dabssv)
      if (allocated(dsVabssv)) deallocate(dsVabssv)
      if (allocated(dsidxlv)) deallocate(dsidxlv)
      if (allocated(dsIDlv)) deallocate(dsIDlv)
      if (allocated(dsVnplv)) deallocate(dsVnplv)
      if (allocated(dsV1Dlv)) deallocate(dsV1Dlv)
      if (allocated(dsVlv)) deallocate(dsVlv)
      if (allocated(np_largestidx)) deallocate(np_largestidx)
      if (allocated(np_lowestidx)) deallocate(np_lowestidx)
      if (allocated(np_smallestidx)) deallocate(np_smallestidx)
      if (allocated(np_largestV)) deallocate(np_largestV)
      if (allocated(np_lowestV)) deallocate(np_lowestV)
      if (allocated(np_smallestV)) deallocate(np_smallestV)
      if (allocated(np_largestID)) deallocate(np_largestID)
      if (allocated(np_lowestID)) deallocate(np_lowestID)
      if (allocated(np_smallestID)) deallocate(np_smallestID)
      if (allocated(npV)) deallocate(npV)
      if (allocated(npotfnames)) deallocate(npotfnames)
      if (allocated(compfile)) deallocate(compfile)
      if (allocated(np_lcpd)) deallocate(np_lcpd)

      return
      end subroutine dealloc_npotminmaxmod

      end module npotminmaxmod 
