!***********************************************************************
!
!                         npotminmax program
!
! This program !an be used to obtain statistics about a PES
! whi!h is constructed from several natpot files.
!
!
!     MAR!H 2012
!     M.S.
!
!***********************************************************************

      program npotminmax

use decimal, only:dop,long,sip
use global
use globmemmod
use pglobal
use channels
use dvrdatmod
use griddatmod
use combined
use operdef
use npotminmaxmod
use versions

use op1lib, only: zerovxd, zerovxi, zeromxi, zeromxd, cpvxi, cpmxi, cpvxl, &
                  cpvxd, zerovxl, zeromxl
use logdat
use npmminpmod
use pdefaultmod, only: init_cmode
use logdvrmod, only: logdvr
use fitinmod, only: fitin
use pesfit, only: getvfit
use getpesmod, only: getv1d
use rddvrmod
use rdnatpot, only: npotinfo,cpdinfo
use directutil, only: rdcpd,getvcpd,igpoint2impoint
use timing, only: walltime
use eingabemod, only: cpinput

use runpfitmod

      implicit none

      integer(long)     :: ilbl,i,j,chkdvr,ierr
      real(dop)         :: totmem
      character(len=c5) :: filename, cmd
      logical(kind=4)   :: lerr,check

!-----------------------------------------------------------------------
! Initialize variables and set defaults
!-----------------------------------------------------------------------
      routine = "npotminmax"
      call default

!-----------------------------------------------------------------------
! Get file names and memory dimensions 
!-----------------------------------------------------------------------
      call npmminput0

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_npotminmaxmod
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      call alloc_runpfitmod

!-----------------------------------------------------------------------
! read input options
!-----------------------------------------------------------------------
      call npmminput

!-----------------------------------------------------------------------

!     --- CREATE THE NAME DIRECTORY IF NECESSARY ---
      open(itmpin,file=name(1:laenge)//'/scratch',err=113)
      close(itmpin,status='delete')
      if(lmnd) write(6,'(2a)') 'Name-directory already exists: ',name(1:laenge)
      goto 114

 113  if(lmnd) then
         cmd='mkdir '//name(1:laenge)
         call excmd2(ierr, cmd)
         if (ierr.ne.0) then
            write(6,'(2a)') '### ERROR: ', &
            'Invalid path for name-directory (-mnd option). ###'
            stop
         endif
         write(6,'(2a)') 'Created name-directory: ',name(1:laenge)
      else
         write(6,'(a)') &
         '---------------------------------------------------------'
         write(6,'(a)') ' ################   ERROR   ##################'
         write(6,'(a)') ' There is no name-directory! The directory:'
         write(6,'(1x,a)') name(1:laenge)
         write(6,'(a)') ' does not exist! (or has no write permission)'
         write(6,'(2a)')' One may use the option -mnd to create the', &
                        ' name-directory.'
         write(6,'(a)') &
           '-------------------------------------------------------'
         stop
      endif

 114  continue

      if (lovwr) then
         open (unit=ilog,file=name(1:laenge)//'/npotmm.log',status='unknown', &
               form='formatted',err=20)
      else
         open (unit=ilog,file=name(1:laenge)//'/npotmm.log',status='new',&
               form='formatted',err=30)
      endif
      logisopen=.true.

! --- WRITE DATE AND MACHINE TO LOG FILE ---
      write(ilog,*)
      write(ilog,'(a)') trim(cstartup(1:80))
      write(ilog,'(1x,a)') trim(name)
      call wrversion(ilog)

!.....copy *.inp to name/input
      call cpinput
      close (iin)

      if (irundvr .eq. 1) then
         call rundvr
      else

         filename=dname(1:dlaenge)//'/dvr'

         open(idvr, file=filename,err=1000,form='unformatted',status="old")
         chkdvr = 1
         call dvrinfo(lerr,chkdvr)
         if (lerr) then
            routine='npotminmax'
            message = "Unable to obtain DVR data from file "//trim(filename)
            call errormsg
         end if

      endif

      call myflush(ilog)
      call logdvr(ilog)

!     check for dublicates among natpots
      do i=1,npotfiles
         do j=i+1,npotfiles
            if ((npotfnames(i) .eq. npotfnames(j)) .and. &
                (np_lcpd(i) .eqv. np_lcpd(j))) then
               routine='npotminmax'
               message = "Found duplicates among natpot files: " &
                        //trim(npotfnames(i))
               call warnmsg
            end if
         end do
      end do

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call memnpmm

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! assign work arrays
! NB work arrays must be at end of mc, mr, mi, ml arrays.
!-----------------------------------------------------------------------
      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
! call caller routine to assign the memory. Caller calls receiver.
!-----------------------------------------------------------------------
      totmem = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'(a)') '  *** Memory allocated dynamically ***'
      write(ilog,'(6(a,i12,/),a,f16.0,a,f12.2,a)') &
       ' complex*16 :', mcdim, &
       ' real*8     :', mrdim, &
       ' integer    :', midim, &
       ' logical    :', mldim, &
       ' complex*8  :', msdim, &
       ' real*4     :', mfdim, &
       ' Total   :',totmem,' Bytes ,',totmem/1048576.d0,' MB'
      write(ilog,'(80a1,/)') ('-',i=1,80)
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

      call npminmax(mr(mrort),mr(mroned),mr(mrcoeff), mr(mrevec),mr(mrworkr))

deallocate(mc,mr,mi,ml,ms,mf)
      call dealloc_npotminmaxmod
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_runpfitmod

      stop

 20   continue
      write(6,*)
      write(6,'(a)') 'ERROR in program npotminmax:'
      write(6,'(a)') 'Cannot open log file! Directory non-existent?'
      if(laenge .le. 64) then
         write(6,'(2a)') 'logfile: ', name(1:laenge)//'/log'
      else
         write(6,'(a,/a)') 'logfile: ', name(1:laenge)//'/log'
      end if
      write(6,*)
      call stopnow

 30   continue
      write(6,*)
      write(6,'(a)') 'ERROR in program npotminmax:'
      write(6,'(a,/a)') &
          'Cannot open log file!' , &
          'overwrite disabled? (Use -w option to enable overwrite)'
      if(laenge .le. 64) then
         write(6,'(2a)') 'logfile: ', name(1:laenge)//'/log'
      else
         write(6,'(a,/a)') 'logfile: ', name(1:laenge)//'/log'
      end if
      write(6,*)
      call stopnow

 1000 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop

      contains

!***********************************************************************
!
!     HERE IS THE SUBROUTINE TO DO THE ANALYSIS
!
!***********************************************************************

      subroutine npminmax(ort,oned,coeff,evec,workr)

use global
use aglobal
use griddatmod
use runpfitmod
use channels
use combined
use npotminmaxmod

use logdat, only: errormsg
use npmminpmod, only: setuptrajectory, setupcompare

      implicit none

      real(dop), dimension(ortdim), intent(out)         :: ort
      real(dop), dimension(np_totoneddim), intent(out)  :: oned
      real(dop), dimension(np_totevecdim), intent(out)  :: evec
      real(dop), dimension(np_totcoeffdim), intent(out) :: coeff
      real(dop)                   ::  workr(workrdim)
      real(dop)                   ::  v, sumv, vcomp
      real(dop), dimension(1)     ::  rdum, rdum1,rdum2,rdum3
      real(dop)                   ::  rtrajpoints,au2ic
      real(dop)                   :: fcputime,fsystime
      real(dop)                   :: tcpu,ttot,tsys,wtime
      complex(dop), dimension(1)  :: zdum,zdum1,zdum2,zdum3
      integer(long), dimension(1) ::     idum, idum1,idum2
      integer(long) :: i, f, f1,chkdvr, np,m, isrf
      integer(long) :: itraj
      integer(long) :: igpoint(maxdim), igpoint1(maxdim), impoint(maxdim)
      integer(long) :: icountt, icountc, ntrajsum, t1, t2
      integer(long) :: icomp
      character(len=c5) :: buf

      external  fcputime,fsystime

! --- Channels for external data

      isrf = 303
      call zerovxd(mean,maxnpotfiles)
      call zerovxd(rms,maxnpotfiles)
      totmean = 0.d0
      totrms = 0.d0
      rtrajpoints = 0.d0
      routine = "npminmax"

      ! initialize statistics data

      currndatasets = 0
      call zeromxi(dsidxsv,maxdim,ndatasets)
      call zeromxd(dsVnpsv,maxnpotfiles,ndatasets)
      call zeromxd(dsV1dsv,maxnpotfiles,ndatasets)
      do m=1,ndatasets
         dsVsv(m) = +1.d9
      end do

      call zeromxi(dsidxabssv,maxdim,ndatasets)
      call zeromxd(dsVnpabssv,maxnpotfiles,ndatasets)
      call zeromxd(dsV1dabssv,maxnpotfiles,ndatasets)
      do m=1,ndatasets
         dsVabssv(m) = +1.d9
      end do

      call zeromxi(dsidxlv,maxdim,ndatasets)
      call zeromxd(dsVnplv,maxnpotfiles,ndatasets)
      call zeromxd(dsV1dlv,maxnpotfiles,ndatasets)
      do m=1,ndatasets
         dsVlv(m) = -1.d9
      end do

      call zeromxi(np_largestidx,maxdim,maxnpotfiles)
      call zeromxi(np_lowestidx,maxdim,maxnpotfiles)
      call zeromxi(np_smallestidx,maxdim,maxnpotfiles)
      !call zeromxd(np_largestV,3,maxnpotfiles)
      do j=1,maxnpotfiles
         do i=1,3
            np_largestV(i,j) = -1.d99
            np_lowestV(i,j) = 1.d99
            np_smallestV(i,j) = 1.d99
         end do
      end do

      call zerovxi(np_largestID,maxnpotfiles)
      call zerovxi(np_lowestID,maxnpotfiles)
      call zerovxi(np_smallestID,maxnpotfiles)

!-----------------------------------------------------------------------
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! read data from dvr file if desired
! edit which data (set dvrdata(x) to .true. and substitute dummy array)
!-----------------------------------------------------------------------

      if (lrddvr) then
         rewind(idvr)
         dvrdata(:) = .false.
         dvrdata(1) = .true.
         chkdvr=-1
         call rddvr(ort,rdum,rdum1,rdum2,zdum,zdum1, &
                    idum,zdum2,zdum3,idum1,idum2,rdum3,chkdvr)
      endif

      call loadfits(oned,evec,coeff)

      if (ltrajectory) then
         call setuptrajectory()
         rtrajpoints = dble(tottrajpoints)
      else
         rtrajpoints = 1.d0
         do f=1,ndof
            rtrajpoints = rtrajpoints*dble(gdim(f))
         end do
         if(rtrajpoints.le.2.1d+9) then  ! 2^31 = 2.14748d+09
            tottrajpoints = int(rtrajpoints+0.2d0)
            write(ilog,'(a,i10)') "Number of grid points:",tottrajpoints
            write(6,   '(a,i10)') "Number of grid points:",tottrajpoints
         else
            write(ilog,'(a,1pe14.7)') &
                "Number of grid points:",rtrajpoints
            write(6,'(a,1pe14.7)') &
                "Number of grid points:",rtrajpoints
         endif
      end if
      if(abs(rtrajpoints-dble(tottrajpoints)).ge.1.d0) then
         write(6,'(a)') ' WARNING: integer overflow'
         write(6,'(a,1pe16.9)') ' rtrajpoints =', rtrajpoints
         write(6,'(a,i13)')  '  tottrajpoints reduced to 300 000 000'
         write(6,'(a,i13)')  '  This may lead to wrong results!'
         tottrajpoints = 3.d+8
         rtrajpoints   = 3.d+8
      endif


      if (rtrajpoints.gt.3.d+8) then
         routine='npminmax (npotminmax.F)'
         message="Too many points in primitive grid to do a "// &
              "complete scan. Use a trajectory file "// &
              "or remove unused DOF from primitive basis."
         call errormsg()
      end if

      if(.not.ltrajectory) then
         ntrajpoints(1)=tottrajpoints
      endif

      if (lcompare) then
         call setupcompare()
         do icountc = 1,numcomp
            icomp=200+icountc
            open(icomp,file=compfile(icountc))
         enddo
      end if

      if (ltrajectory) then
         do icountt = 1,numtraj
            itraj=100+icountt
            open(itraj,file=trajfile(icountt))
         enddo
      else
         igpoint(1) = 0
         do f=2,ndof
            igpoint(f) = 1
         end do
      end if

      if (lwrsrf) then
         open(unit=isrf, file=name(1:laenge)//"/srf.asc",form='formatted')
      end if


      vcomp = 0.d0
      icountt=0
      itraj=100
      icomp=200
      ntrajsum=0
      do i=1,tottrajpoints
         if(i.gt.ntrajsum) then
            icountt=icountt+1
            ntrajsum=ntrajsum+ntrajpoints(icountt)
            if (ltrajectory) then
               if(itraj.gt.100) close(itraj)
               itraj=itraj+1
 801           continue
               read(itraj,'(a)') buf
               buf = adjustl(buf)
               if (buf(1:1) .eq. "#") goto 801
               backspace(itraj)
            endif
            if (lcompare) then
               if(icomp.gt.200) close(icomp)
               icomp=icomp+1
 802           continue
               read(icomp,'(a)') buf
               buf = adjustl(buf)
               if (buf(1:1) .eq. "#") goto 802
               backspace(icomp)
            endif
         endif
         ! get new gridpoint
         if (ltrajectory) then
            read(itraj,*) (igpoint(f), f=1,ndof_sys)
         else
            call incrementindex(igpoint, gdim_sys, ndof_sys)
         end if

         v = 0.d0
         zpevec=1
         do np=1,npotfiles
            call cpnp2sys(np)
            ! re-sort igpoint
            call zerovxi(igpoint1,maxdim)
            do f=1,ndof
               f1 = fdvr(f)
               igpoint1(f) = igpoint(f1)
            end do

           if (lcpd) then   ! CPD
              call igpoint2impoint(igpoint1,impoint) !or igpoint ??
              call getvcpd(impoint,evec(np_zevecstart(np)), &
                   pevec(zpevec:zpevec+cpdrank*nmode), &
                   np_evecdim(np),cpdrank,nmode,npV(2,np))
              zpevec=zpevec+cpdrank*nmode+1
           else
              call getvfit(igpoint1,coeff(np_zcoeffstart(np)), &
                   evec(np_zevecstart(np)),workr,npV(2,np))
           endif

           if(i.eq.1) then ! only for first grid point
              ploned=.false.
              do f=1,ndof
                 if (loned(f)) then
                    ploned=.true.
                    exit
                 endif
               enddo
               np_ploned(np)=ploned
            endif
            ploned=np_ploned(np)
            call getv1d(igpoint1,oned(np_zonedstart(np)),npV(3,np))
            npV(1,np) = npV(2,np)+npV(3,np)
            v = v +  npV(1,np)
            mean(np) = mean(np) + npV(1,np)
            rms(np) = rms(np) + npV(1,np)**2

         end do  ! np=1,npotfiles


         if (lcompare) then
            read(icomp,*) vcomp
            sumv =  v - vcomp
         else
            sumv =  v
         end if

         if (lwrsrf) then
            write(isrf,'(e16.8)') sumv
         end if

         currndatasets = min(i,ndatasets)

         ! extremes of single natpots
         do np=1,npotfiles
            if (npV(1,np).gt. np_largestV(1,np)) then
               call cpvxi(igpoint,np_largestidx(1,np),ndof_sys)
               call cpvxd(npV,np_largestV(1,np),3)
               np_largestID(np) = i
            end if

            if (abs(npV(1,np)).lt. abs(np_lowestV(1,np))) then
               call cpvxi(igpoint,np_lowestidx(1,np),ndof_sys)
               call cpvxd(npV,np_lowestV(1,np),3)
               np_lowestID(np) = i
            end if

            if (npV(1,np).lt. np_smallestV(1,np)) then
               call cpvxi(igpoint,np_smallestidx(1,np),ndof_sys)
               call cpvxd(npV,np_smallestV(1,np),3)
               np_smallestID(np) = i
            end if

         end do

         ! extremes of complete potential
         if (sumv .lt. dsVsv(1)) then
            dsVsv(1) = sumv
            call cpvxi(igpoint,dsidxsv(1,1),ndof_sys)
            do np=1,npotfiles
               dsVnpsv(np,1) = npV(2,np)
               dsV1Dsv(np,1) = npV(3,np)
            end do
            dsIDsv(1) = i
            call npsortdown(dsVsv,dsVCs,dsidxsv,dsVnpsv,dsV1Dsv, &
                 dsIDsv,.false., ndof_sys, ndatasets, npotfiles, &
                 maxdim,ndatasets,maxnpotfiles)

         end if

         if (abs(sumv) .lt. abs(dsVsv(1))) then
            dsVabssv(1) = sumv
            call cpvxi(igpoint,dsidxabssv(1,1),ndof_sys)
            do np=1,npotfiles
               dsVnpabssv(np,1) = npV(2,np)
               dsV1Dabssv(np,1) = npV(3,np)
            end do
            dsIDabssv(1) = i
            call npsortdown(dsVabssv,dsVCa,dsidxabssv,dsVnpabssv,&
                 dsV1Dabssv, dsIDabssv,.true., ndof_sys, ndatasets,&
                 npotfiles,maxdim,ndatasets,maxnpotfiles)

         end if

         if (sumv .gt. dsVlv(1)) then
            dsVlv(1) = sumv
            call cpvxi(igpoint,dsidxlv(1,1),ndof_sys)
            do np=1,npotfiles
               dsVnplv(np,1) = npV(2,np)
               dsV1Dlv(np,1) = npV(3,np)
            end do
            dsIDlv(1) = i
            call npsortup(dsVlv,dsVCl,dsidxlv,dsVnplv,dsV1Dlv, &
                 dsIDlv, .false., ndof_sys, ndatasets, npotfiles, &
                 maxdim,ndatasets,maxnpotfiles)
         end if


         totmean = totmean + sumv
         totrms = totrms + sumv**2
      end do

      if (lwrsrf) close(isrf)
      if (ltrajectory) close(itraj)
      if (lcompare) close(icomp)

      totmean = totmean/dble(tottrajpoints)
      totrms = dsqrt(totrms/dble(tottrajpoints))
      do np=1,npotfiles
         mean(np) =  mean(np)/dble(tottrajpoints)
         rms(np) = dsqrt(rms(np)/dble(tottrajpoints))
      end do


      au2ic=2.1947463137d+5
      write(6,*) "***** Statistics *****"
      write(6,901) "Mean =",totmean*au2ic," cm-1  (all potentials)"
      write(6,901) "RMS  =",totrms*au2ic ," cm-1  (all potentials)"
      if(npotfiles.gt.1) then
         do np=1,npotfiles
            write(6,902) "Mean =",mean(np)*au2ic," cm-1  (np =",np,')'
            write(6,902) "RMS  =",rms(np)*au2ic ," cm-1  (np =",np,')'
         enddo
      endif
      write(6,*)

      write(ilog,*) "***** Statistics *****"
      write(ilog,901) "Mean =",totmean*au2ic," cm-1  (all potentials)"
      write(ilog,901) "RMS  =",totrms*au2ic ," cm-1  (all potentials)"
      if(npotfiles.gt.1) then
         do np=1,npotfiles
           write(ilog,902)"Mean =",mean(np)*au2ic," cm-1  (np =",np,')'
           write(ilog,902)"RMS  =",rms(np)*au2ic ," cm-1  (np =",np,')'
         enddo
      endif
      write(ilog,*)

      call npoutput(ort) ! writes smalles/largest/statistics files

!-----------------------------------------------------------------------
! Get overall timing and write data to "log" file
!-----------------------------------------------------------------------
      call walltime(wtime)
      wtime=wtime-wtim0
      tcpu = fcputime(0)
      tsys = fsystime(0)
      ttot = tcpu+tsys

      write(ilog,*)
      write(ilog,'(a)') '***** Overall Timing *****'
      write(ilog,*)
      write(ilog,'(a,f13.2)') 'Total cpu time     [s] : ',tcpu
      write(ilog,'(a,f13.2)') 'Total system time  [s] : ',tsys
      write(ilog,'(a,f13.2)') 'Wall time          [s] : ',wtime

      t1   = int(ttot/3600.d0)
      ttot = ttot - dble(3600*t1)
      t2   = int(ttot/60.d0)
      ttot = ttot - dble(60*t2)
      write(ilog, 900) 'Total time     [h:m:s] : ',t1,t2,ttot
      write(iout, 900) 'Total time     [h:m:s] : ',t1,t2,ttot

      t1   = int(wtime/3600.d0)
      ttot = wtime - dble(3600*t1)
      t2   = int(wtime/60.d0)
      wtime = wtime - dble(60*t2)
      write(ilog, 900) 'Wall  time     [h:m:s] : ',t1,t2,wtime
      write(iout, 900) 'Wall  time     [h:m:s] : ',t1,t2,wtime
      write(iout,'(a/)')'----------------------------------------------'
      write(ilog,*)

      return
 900  format( a,i4,' :',i3,' :',f6.2)
 901  format(a,f16.3,a)
 902  format(a,f16.3,a,i3,a)

      end subroutine npminmax

!-----------------------------------------------------------------------
!***********************************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! change name of subroutine (or else mutil gets confused)
!-----------------------------------------------------------------------


!#######################################################################
!***********************************************************************
!
! calculates amount of memory to be allocated and the pointer structure
! Copy from showpot with small modifications
!
!***********************************************************************

      subroutine memnpmm

      implicit none

      integer(long)                      :: f1,f,n,m,mm,wkdim,np,nsub, ilbl
      integer(long)                      :: chkdvr, chkgrd, ierr,lnatpot
      integer(long)                      :: zeigc, zeige, zeigo, zeig
      character(len=c5)                  :: filename, buf
      logical(kind=4), dimension(maxdim) :: loned1, lfoundlabel
      logical(kind=4)                    :: lerr

      integer(long), dimension(maxdim)        :: mmode1,nspfdof1,pntbas
      integer(long), dimension(maxdim)        :: subdim1
      integer(long), dimension(maxdim,maxdim) :: spfdof1
      integer(long)                           :: ndof1, nmode1

      call zerovxl(lfoundlabel,maxdim)

      workrdim  = 1
      maxpsub   = 0
      maxgdim   = 0
      ortdim    = 0
      evecdim   = 0
      modcdim   = 1
      vpotdim   = 1
      np_totcoeffdim = 0
      np_totevecdim = 0
      np_totoneddim = 0

      ! we use the PBAS-SECTION data here
      do f=1,ndof
         ortdim = ortdim + gdim(f)
      end do
      ndof_sys = ndof

      call cpvxi(gdim,gdim_sys,maxdim)

      zeigc = 1
      zeigo = 1
      zeige = 1

      np_totevecdim = 0
      np_totcoeffdim = 0
      np_totoneddim = 0

      call zerovxi(np_zcoeffstart,maxnpotfiles)
      call zerovxi(np_zevecstart,maxnpotfiles)
      call zeromxi(np_zevec,maxdim,maxnpotfiles)
      call zerovxi(np_evecdim,maxnpotfiles)
      call zerovxi(np_coeffdim,maxnpotfiles)

      call zeromxi(np_fdvr,maxdim,maxnpotfiles)
      call zerovxi(np_oneddim,maxnpotfiles)
      call zerovxi(np_zonedstart,maxnpotfiles)
      call zeromxl(np_loned,maxdim,maxnpotfiles)

      call zerovxi(np_ndof,maxnpotfiles)
      call zeromxi(np_nspfdof,maxdim,maxnpotfiles)
      do f=1,maxnpotfiles
         call zeromxi(np_spfdof(:,:,f), maxdim, maxdim)
      end do
      call zeromxi(np_psubdim,maxdim,maxnpotfiles)

      do f=1,maxnpotfiles
         call zeromxi(np_potdof(:,:,f),maxdim,maxdim)
      end do
      call zeromxi(np_npotdof,maxdim,maxnpotfiles)
      call zeromxi(np_potdim,maxdim,maxnpotfiles)

      call zerovxi(np_modc,maxnpotfiles)
      call zerovxi(np_nmode,maxnpotfiles)

      call zerovxi(np_nodbef,maxnpotfiles)
      call zerovxi(np_nodaft,maxnpotfiles)
      call zerovxi(np_dimbef,maxnpotfiles)
      call zerovxi(np_dimaft,maxnpotfiles)

      call zeromxi(np_c2v,maxdim,maxnpotfiles)

!-----------------------------------------------------------------------
! read information from system files
! THIS OVERWRITES THE PBAS-SECTION DATA!
!-----------------------------------------------------------------------
      maxpsub = 0
      sumrank = 1
      do np=1, npotfiles
         lcpd=np_lcpd(np)
         filename = npotfnames(np)
         if(lcpd) then
            filename = trim(filename)//"/cpd"
         else
            filename = trim(filename)//"/natpot"
         endif
         lnatpot = index(filename,' ')-1

         open(inpot,file=filename(1:lnatpot),form='unformatted', &
              status='old',iostat=ierr)
         if (ierr.ne.0) then
            routine='memnpmm (npotminmax.f90)'
            message='Unable to open '//filename(1:lnatpot)//"."
            call errormsg
         endif
         chkdvr=0
         chkgrd=1
         ndof = ndof_sys
         call cpvxi(gdim_sys, gdim, maxdim)

         if(lcpd) then
             call cpdinfo(inpot,lerr,ndof1,fdvr,chkdvr,chkgrd,0,mdvr, &
                   mmode,cpdrank,nspfdof1,spfdof1,nmode1,subdim1,loned1,&
                   pntbas)
             sumrank=sumrank+cpdrank*nmode1+1
             modc=1
         else
            call npotinfo(inpot,lerr,modc,loned1,ndof1,fdvr,chkdvr,chkgrd,&
                 1,mdvr,mmode1,nspfdof1,spfdof1,nmode1,psubdim,pntbas)
         endif
         close(inpot)
         ndof = ndof1

         do m=1,npmode
            zeig=1
            do n=1,npotdof(m)
               f=potdof(n,m)
               f1=fdvr(f)
               zeig=zeig*gdim(f1)
            enddo
            psubdim(m)=zeig
            subdim(m)=psubdim(m)
            maxpsub  = max(maxpsub,psubdim(m))
         enddo

         dimmodc = psubdim(modc)

!        Backwards compatibility needs to be implementd
         if (filever(inpot) .lt. 8.3013d0) then
            routine='memnpmm (npotminmax.F)'
            message = "Backwards compatibility not maintained."// &
                 " Try re-creating the potfit file "// &
                 filename(1:lnatpot)//"/natpot with a Potfit version"// &
                 " newer than 8.3013 (or implement compatibility)."
            call errormsg
         end if

         do m=1,nmode1
            if (m.ne.modc) then
               potdim(m)=pdim(m)
            else
               potdim(m)=-1
            endif

         enddo
         if (.not. lcpd) call init_cmode

         oneddim = 0
         do m=1,nmode
            oneddim = oneddim + subdim(m)
         end do

         evecdim = 0
         modcdim = 1
         call zerovxi(zevec,maxdim)
         do m=1,nmode
            maxpsub  = max(maxpsub,psubdim(m))
            if (lcpd) then
               zevec(m)=evecdim+1
               evecdim=evecdim+psubdim(m)*cpdrank
            else
               if (m.ne.modc) then
                  zevec(m)=evecdim+1
                  evecdim=evecdim+psubdim(m)*potdim(m)
                  modcdim=modcdim*potdim(m)
               else
                  zevec(m)=-1
               endif
            endif
         enddo
         if(lcpd) then
            coeffdim=0
         else
            coeffdim=pdim(modc)*psubdim(modc)
         endif

         call zerovxi(c2v,maxdim)
         c2v(1)=modcdim
         wkdim=c2v(1)+maxpsub
         mm=0
         if (.not. lcpd) then
            do m=1,nmode
               if (m.ne.modc) then
                  mm=mm+1
                  c2v(mm+1)=c2v(mm)/potdim(m)
                  if (mm+1.eq.2) wkdim=wkdim+c2v(mm+1)
               endif
            enddo
         endif

         workrdim=max(workrdim,wkdim)
         workrdim=max(workrdim,niobuf)
         workdim2 = workrdim    ! needed for getvfit

         ! copy
         call cpvxi(fdvr,np_fdvr(1,np),maxdim)
         np_zcoeffstart(np) = zeigc
         zeigc = zeigc + coeffdim
         np_totcoeffdim = np_totcoeffdim + coeffdim
         np_coeffdim(np) = coeffdim

         call cpvxi(zevec,np_zevec(1,np),maxdim)
         np_totevecdim = np_totevecdim + evecdim
         np_evecdim(np) = evecdim
         np_zevecstart(np) = zeige
         zeige = zeige + evecdim

         np_nodbef(np) = nodbef
         np_nodaft(np) = nodaft
         np_dimbef(np) = dimbef
         np_dimaft(np) = dimaft
         np_nmode(np) = nmode ! == npmode
         np_modc(np) = modc
         np_ndof(np) = ndof
         np_cpdrank(np) = cpdrank
         np_lcpd(np) = lcpd

         np_oneddim(np) = oneddim
         np_zonedstart(np) = zeigo
         zeigo = zeigo + oneddim
         np_totoneddim = np_totoneddim + oneddim

         call cpvxi(npotdof,np_npotdof(1,np),maxdim)
         call cpmxi(potdof,np_potdof(1,1,np),maxdim,maxdim)
         call cpvxi(psubdim,np_psubdim(1,np),maxdim)
         call cpvxi(c2v,np_c2v(1,np),maxdim)
         call cpvxi(potdim, np_potdim(1,np),maxdim)
         call cpvxi(nspfdof1,np_nspfdof(1,np),maxdim)
         call cpmxi(spfdof1,np_spfdof(1,1,np),maxdim,maxdim)
         call cpvxl(loned1, np_loned(1,np),maxdim)

         do f=1,ndof
            f1=fdvr(f)
            lfoundlabel(f1) = .true.
         end do

      end do

      nsub = 0
      ilbl = 1
      do f=1,ndof_sys
         if (lfoundlabel(f)) then
            nsub = nsub + 1
         else
            buf(ilbl:ilbl+8) = modelabel(f)(1:8)
            ilbl = ilbl + 8
         end if
      end do
      buf(ilbl:ilbl) = "#"
      call cncntrblnk(buf)
      ilbl = index(buf,"#")-1
      if (nsub .ne. ndof_sys) then
         routine = "memnpmm (npotminmax.f90)"
         message = "Primitive basis contains more DOF than the "// &
              "natpots. Modelabels not contained in natpots: "// &
              buf(1:ilbl)//"- continuing..."
         call warnmsg
      end if

      ! we don't read vpots
      vpotdim = 1

      mrort=mrdim
      mrdim=mrdim+ortdim

      mroned=mrdim
      mrdim=mrdim+np_totoneddim

      mrevec=mrdim
      mrdim=mrdim+np_totevecdim

      mrcoeff = mrdim
      mrdim = mrdim + np_totcoeffdim


      miworki = midim
      midim   = miworki+workidim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

      mldim   = 1

      return
      end subroutine memnpmm

!#######################################################################
      ! copy natpot parameters of natpot file np to system

      subroutine cpnp2sys(np)

use global
use pglobal
use griddatmod
use operdef
use runpfitmod
use combined
use channels
use npotminmaxmod

use op1lib, only: cpvxi, cpmxi, cpvxl
use logdat, only: errormsg

      implicit none

      integer(long)     :: np, ilbl, jlbl,f,f1
      character(len=c1) :: buf, buf1

      if (np .gt. npotfiles) then
         write(buf,'(i4)') np
         write(buf1,'(i4)') npotfiles
         call rmblnk(buf)
         call rmblnk(buf1)
         ilbl = index(buf," ")
         jlbl = index(buf1," ")
         routine = "cpnp2sys (npotminmax.F)"
         message = "Cannot load dataset "//buf(1:ilbl)// &
              ". Only "//buf1(1:jlbl)//" natpot files are defined."
         call errormsg
      end if

!     copy
      call cpvxi(np_fdvr(:,np),fdvr,maxdim)
      coeffdim = np_coeffdim(np)

      call cpvxi(np_zevec(:,np),zevec,maxdim)
      evecdim = np_evecdim(np)

      nodbef = np_nodbef(np)
      nodaft = np_nodaft(np)
      dimbef = np_dimbef(np)
      dimaft = np_dimaft(np)
      nmode = np_nmode(np)
      npmode = nmode !!
      modc = np_modc(np)
      ndof = np_ndof(np)
      oneddim = np_oneddim(np)
      cpdrank = np_cpdrank(np)
      lcpd = np_lcpd(np)

      call cpvxi(np_npotdof(:,np),npotdof,maxdim)
      call cpmxi(np_potdof(:,:,np),potdof,maxdim,maxdim)
      call cpvxi(np_psubdim(:,np),psubdim,maxdim)
      call cpvxi(np_c2v(:,np),c2v,maxdim)
      call cpvxi(np_potdim(:,np),potdim,maxdim)
      call cpvxi(np_nspfdof(:,np),nspfdof,maxdim)
      call cpmxi(np_spfdof(:,:,np),spfdof,maxdim,maxdim)
      call cpvxl(np_loned(:,np),loned,maxdim)

      dimmodc = psubdim(modc)

      do f=1,ndof
         f1 = fdvr(f)
         gdim(f) = gdim_sys(f1)
      end do
      end subroutine cpnp2sys

!#######################################################################


      subroutine loadfits(oned,evec,coeff)

use global
use pglobal
use griddatmod
use operdef
use runpfitmod
use combined
use channels
use npotminmaxmod

      implicit none

      logical(kind=4)                         ::   lerr
      logical(kind=4), dimension(maxdim)      :: loned1
      real(dop), dimension(np_totoneddim)     :: oned
      real(dop), dimension(np_totevecdim)     :: evec
      real(dop), dimension(np_totcoeffdim)    :: coeff
      integer(long), dimension(maxdim)        :: mmode1,nspfdof1,pntbas
      integer(long), dimension(maxdim)        :: subdim1
      integer(long), dimension(maxdim,maxdim) :: spfdof1
      integer(long)                           :: ndof1, nmode1
      character(len=c5)                       :: filename
      integer(long)                           :: np, chkdvr, chkgrd, ierr,  &
                                                 lnatpot, zpevec1
!     Read natpot files

      do np=1,npotfiles
         filename = npotfnames(np)
         lnatpot = index(filename,' ')-1
         if(lcpd) then
            filename = filename(1:lnatpot)//"/cpd"
         else
            filename = filename(1:lnatpot)//"/natpot"
         endif
         lnatpot = index(filename,' ')-1

         open(inpot,file=filename(1:lnatpot),form='unformatted', &
              status='old',iostat=ierr)
         if (ierr.ne.0) then
            routine='loadfits on npotminmax.F'
            message='Unable to open '//filename(1:lnatpot)//"."
            call errormsg
         endif
         chkdvr=0
         chkgrd=1

         ndof = ndof_sys
         call cpvxi(gdim_sys, gdim, maxdim)
         write(ilog,'(/2a)') 'The pot file is: ',trim(filename)
         write(6,'(a,i3,2a)') 'The',np,' potfile is: ',trim(filename)
         if(lcpd) then
             call cpdinfo(inpot,lerr,ndof1,fdvr,chkdvr,chkgrd,np,mdvr, &
                  mmode,cpdrank,nspfdof,spfdof,nmode1,subdim1,loned1, &
                  pntbas)
            np_cpdrank(np)=cpdrank
            call cpnp2sys(np)  ! copy data to system
            zpevec1=zpevec+cpdrank*nmode+1
            call rdcpd(inpot,np_loned(1,np),oned(np_zonedstart(np)), &
                 evec(np_zevecstart(np)),cpdrank,pevec(zpevec:zpevec1))
            zpevec=zpevec1
         else
            call npotinfo(inpot,lerr,modc,loned1,ndof1,fdvr, &
                 chkdvr,chkgrd,np,mdvr,mmode1,nspfdof1,spfdof1, &
                 nmode1,psubdim,pntbas)
            call cpnp2sys(np)  ! copy data to system
            call fitin(inpot,np_loned(1,np),oned(np_zonedstart(np)), &
                 evec(np_zevecstart(np)),coeff(np_zcoeffstart(np)))
         endif
         close(inpot)

      end do

      end subroutine loadfits

!#######################################################################

      subroutine incrementindex(idx,imax,ndof)

use global

      implicit none

      integer(long), intent(in)                   :: ndof
      integer(long), dimension(ndof), intent(in)  :: imax
      integer(long), dimension(ndof), intent(out) :: idx
      integer(long)                               :: f


      idx(1) = idx(1) + 1

      do f=1,ndof-1
         if (idx(f) .gt. imax(f)) then
            idx(f) = 1
            idx(f+1) =  idx(f+1) + 1
         end if
      end do

      if (idx(ndof) .gt. imax(ndof)) then
         routine = "incrementindex (npotminmax.F)"
         message = "Index overflow."
         call errormsg
      end if

      end subroutine incrementindex

!#######################################################################

      subroutine npsortup(V,VC,idx,Vnp1,V1d1, ID, labs, ndof, nset, npf, &
                 maxdim,ndatasets,maxnpotfiles)

      implicit none

      integer(long), intent(in)       :: ndof, nset, npf
      integer(long), intent(in)       ::  maxdim,ndatasets,maxnpotfiles
      integer(long), dimension(maxdim,ndatasets), intent(inout)   :: idx
      integer(long), dimension(nset), intent(inout)                 :: ID
      real(dop), dimension(nset), intent(inout)                     :: V,VC
      real(dop), dimension(maxnpotfiles,ndatasets), intent(inout) :: Vnp1,V1d1
      real(dop)                   :: ri, rip1, rdum
      integer(long)               ::  i,j,idum, N1
      logical(kind=4), intent(in) ::  labs
      logical(kind=4)             ::  lchange

      ! bubblsort

      N1 = nset

 500  continue
      lchange = .false.
      do i=1,N1-1
         if (labs) then
            ri = abs(V(i))
            rip1 = abs(V(i+1))
         else
            ri = V(i)
            rip1 = V(i+1)
         end if
         if (rip1 .lt. ri) then
            lchange = .true.

            rdum = V(i)
            V(i) = V(i+1)
            V(i+1) = rdum

            rdum = VC(i)
            VC(i) = VC(i+1)
            VC(i+1) = rdum

            do j=1,npf
               rdum = Vnp1(j,i)
               Vnp1(j,i) = Vnp1(j,i+1)
               Vnp1(j,i+1) = rdum
            end do

            do j=1,npf
               rdum = V1d1(j,i)
               V1d1(j,i) = V1d1(j,i+1)
               V1d1(j,i+1) = rdum
            end do

            do j=1,ndof
               idum = idx(j,i)
               idx(j,i) = idx(j,i+1)
               idx(j,i+1) = idum
            end do

            idum = ID(i)
            ID(i) = ID(i+1)
            ID(i+1) = idum

         end if
      end do
      if (lchange) goto 500

      end subroutine npsortup

!#######################################################################

      subroutine npsortdown(V,VC,idx,Vnp1,V1d1, ID, labs, ndof, nset, npf,&
                 maxdim,ndatasets,maxnpotfiles)

      implicit none

      integer(long), intent(in)       :: ndof, nset, npf
      integer(long), intent(in)       ::  maxdim,ndatasets,maxnpotfiles
      integer(long), dimension(maxdim,ndatasets), intent(inout)   :: idx
      integer(long), dimension(nset), intent(inout)               :: ID
      real(dop), dimension(nset), intent(inout)                   :: V,VC
      real(dop), dimension(maxnpotfiles,ndatasets), intent(inout) :: Vnp1,V1d1
      real(dop)                   :: ri, rip1, rdum
      integer(long)               ::  i,j,idum, N1
      logical(kind=4), intent(in) ::  labs
      logical(kind=4)             ::  lchange

      ! bubblsort

      N1 = nset

 500  continue
      lchange = .false.
      do i=1,N1-1
         if (labs) then
            ri = abs(V(i))
            rip1 = abs(V(i+1))
         else
            ri = V(i)
            rip1 = V(i+1)
         end if
         if (rip1 .gt. ri) then
            lchange = .true.

            rdum = V(i)
            V(i) = V(i+1)
            V(i+1) = rdum

            rdum = VC(i)
            VC(i) = VC(i+1)
            VC(i+1) = rdum

            do j=1,npf
               rdum = Vnp1(j,i)
               Vnp1(j,i) = Vnp1(j,i+1)
               Vnp1(j,i+1) = rdum
            end do

            do j=1,npf
               rdum = V1d1(j,i)
               V1d1(j,i) = V1d1(j,i+1)
               V1d1(j,i+1) = rdum
            end do

            do j=1,ndof
               idum = idx(j,i)
               idx(j,i) = idx(j,i+1)
               idx(j,i+1) = idum
            end do

            idum = ID(i)
            ID(i) = ID(i+1)
            ID(i+1) = idum

         end if
      end do
      if (lchange) goto 500

      end subroutine npsortdown

!#######################################################################

      subroutine npoutput(ort)

use global
use griddatmod
use npotminmaxmod
use channels

      implicit none

      real(dop), dimension(ortdim), intent(in) :: ort

      call npsortup(dsVsv,dsVCs,dsidxsv,dsVnpsv,dsV1Dsv,dsIDsv,.false., &
           ndof_sys, ndatasets, npotfiles,maxdim,ndatasets,maxnpotfiles)
      open(idump, file=name(1:laenge)//"/smallest",form='formatted')
      write(idump,'(a)') "Smallest values of V (sum over all natpots)"
      call outVdata(ort,dsIDsv,dsVsv,dsVCs,dsVnpsv,dsV1Dsv,dsidxsv, idump)
      close(idump)

      call npsortdown(dsVabssv,dsVCa,dsidxabssv,dsVnpabssv,dsV1Dabssv,&
           dsIDabssv,.True., ndof_sys, ndatasets, npotfiles,maxdim,&
           ndatasets,maxnpotfiles)
      open(idump, file=name(1:laenge)//"/smallest_abs",form='formatted')
      write(idump,'(a)') "Smallest absolute values of V " &
           //"(sum over all natpots)"
      call outVdata(ort,dsIDabssv,dsVabssv,dsVCa,dsVnpabssv,dsV1Dabssv, &
           dsidxabssv, idump)
      close(idump)

      call npsortdown(dsVlv,dsVCl,dsidxlv,dsVnplv,dsV1Dlv,dsIDlv,.false., &
           ndof_sys, ndatasets, npotfiles,maxdim,ndatasets,maxnpotfiles)
      open(idump, file=name(1:laenge)//"/largest",form='formatted')
      write(idump,'(a)') "Largest values of V (sum over all natpots)"
      call outVdata(ort,dsIDlv,dsVlv,dsVCl,dsVnplv,dsV1Dlv,dsidxlv, idump)
      close(idump)

      call outNPdata(ort) ! writes statistics file

      end subroutine npoutput

!#######################################################################

      subroutine outVdata(ort,ID,V,VC,Vnp,V1D,IDX,ichan)

use global
use griddatmod
use npotminmaxmod

      implicit none

      real(dop), dimension(ortdim), intent(in)                 :: ort
      integer(long), dimension(maxdim,ndatasets), intent(in)   :: IDX
      integer(long), dimension(ndatasets), intent(in)          :: ID
      real(dop), dimension(maxnpotfiles,ndatasets), intent(in) :: Vnp, V1D
      real(dop), dimension(ndatasets), intent(in)              :: V,VC
      logical(kind=4)  :: lcoord

      integer(long)    :: ds,np,f,f1,ilbl,noutlab
      integer(long)    :: ichan
      character(len=8) :: buf8
      character(len=1000) :: buf1k

! maximum no. of labels wo be written. increase
! size of buf1k to write more labels

      write(ichan,'(a)') " "
      write(ichan,'(a)') "Summary (details below)"
      if(lcompare) write(ichan,'(a)') "Note: compare data substracted"
      write(ichan,'(a)') " "


      noutlab = min(1000/8,ndof_sys)

      do f=1,noutlab
         buf1k((f-1)*8+1:f*8) = modelabel(f)(1:8)
      end do

      write(ichan,'(a)') &
          "  No          V[au]       "//trim(buf1k(1:8*ndof_sys))

      do ds=1,currndatasets

         buf1k = " "
         write(buf1k,'(i4,x,1pe16.8)') ds,V(ds)

         do f=1,noutlab
            write(buf1k((23+(f-1)*8):(23+(f*8))),'(f8.3)') &
                 ort(zort(f)+IDX(f,ds)-1)
         end do
         write(ichan,'(a)') trim(buf1k(1:(noutlab+4)*8))

      end do
      write(ichan,'(//2a)') "Details for all data sets ", &
                            " (points taken from above):"
      write(ichan,'(2a)') "Coordinates will be printed only if their ", &
                      "number/order differs from those of the dvr-file"
      if(lcompare) write(ichan,'(a)') &
                  "Note: compare data not substracted"
      write(ichan,*)


      do ds=1,currndatasets
         write(buf8,'(i8)') ds
         call rmblnk(buf8)
         ilbl = index(buf8," ")-1
         write(ichan,'(a)') "Set "//buf8(1:ilbl)//":"

         write(buf8,'(i8)') ID(ds)
         call rmblnk(buf8)
         ilbl = index(buf8," ")-1
         if (ltrajectory) then
            buf1k = " the trajectory."
         else
            buf1k = " the total grid."
         end if
         write(ichan,'(a)') "Point no. "//buf8(1:ilbl)// " of"//buf1k(1:16)
         write(ichan,'(a,3(1pe16.8,a))')'Total potential: V = ',V(ds), &
         ' au,',V(ds)*27.21138386d0,' eV,',V(ds)*2.1947463137d+5,' cm-1'
         write(ichan,'(/a)') "Singe natpots contribute:"
         write(ichan,'(72a)') ("-", f=1,72)
         write(ichan,'(a,13x,a,6x,a,3x,a,9x,a)') " No.","value", &
              "one-dim part","correlated part","coordinates"

         write(ichan,'(72a)')("-", f=1,72)

         do np=1,npotfiles
            buf1k = " "
            lcoord=.false.  ! decide if the coordinates are printed or not.
            if(np_ndof(np).eq.ndof_sys) then
               do f=1,np_ndof(np) ! check if theorder is changed
                  if(f.ne.np_fdvr(f,np)) then
                     lcoord=.true.
                     exit
                  endif
               enddo
            else
               lcoord=.true. ! different numbers of coordinates
            endif
            if(lcoord) then
               do f=1,np_ndof(np)
                  f1 = np_fdvr(f,np)
                  buf1k((f-1)*8+1:f*8) = modelabel(f1)(1:8)
               end do
               write(ichan,'(i4,2x,1pe16.8,2x,e16.8,2x,1pe16.8,10x,a)') &
                    np, Vnp(np,ds) + V1D(np,ds) , &
                    V1D(np,ds), Vnp(np,ds), &
                    buf1k(1:8*np_ndof(np))
            else
               write(ichan,'(i4,2x,1pe16.8,2x,e16.8,2x,1pe16.8)') &
                    np, Vnp(np,ds) + V1D(np,ds) , &
                    V1D(np,ds), Vnp(np,ds)
            endif
         end do  ! np=1,npotfiles
         if(lcompare) write(ichan,'(a,1pe16.8)') ' Vcomp', VC(ds)

         write(ichan,'(72a)')("-", f=1,72)
         write(ichan,*)
         write(ichan,*)
      end do  ! ds=1,currndatasets

      end subroutine outVdata

!#######################################################################

      subroutine outNPdata(ort)

use global
use griddatmod
use npotminmaxmod
use channels

      implicit none

      integer(long)                            :: np, f,f1,i,ilbl, jlbl
      real(dop), dimension(ortdim), intent(in) :: ort
      character(len=c5), dimension(3)          :: buf

      open(idump, file=name(1:laenge)//"/statistics",form='formatted')

      write(idump,'(a)')
      write(idump,'(a)') "NATPOT/CPD FILES:"
      if(lcompare) write(idump,'(a)') &
                   "Note: compare data not substracted"
      do np=1, npotfiles
         ilbl = index( npotfnames(np)," ")-1
         write(idump,'(i4,3x,a)')np, npotfnames(np)(1:ilbl)
      end do
      write(idump,'(/a)') "STATISTICS OF NATPOT/CPD FILES:"
      write(idump,'(a,10x,a,11x,a,3x,a,14x,a,11x,a,11x,a,5x,a)') &
           " No.", "Mean", "RMS", "Extremes:", "V", "VND", "V1D", &
           "Coordinates"
      write(idump,"(102a)")("-",f=1,102)
      do np=1,npotfiles

         ilbl = 1
         do f=1,np_ndof(np)
            f1 = np_fdvr(f,np)

            write(buf(1)(ilbl:C5),'(a,f8.3,a)') &
                 modelabel(f1)(1:6)//" = ", &
                 ort(zort(f1)+np_largestidx(f1,np)-1), ",  "

            write(buf(2)(ilbl:C5),'(a,f8.3,a)') &
                 modelabel(f1)(1:6)//" = ", &
                 ort(zort(f1)+np_lowestidx(f1,np)-1), ",  "

            write(buf(3)(ilbl:C5),'(a,f8.3,a)') &
                 modelabel(f1)(1:6)//" = ", &
                 ort(zort(f1)+np_smallestidx(f1,np)-1), ",  "

            ilbl = ilbl + 20
         end do
         ilbl = 0
         do i=1,3
            jlbl = np_ndof(np)*20+1
            buf(i)(jlbl:jlbl) = "#"
            call cncntrblnk(buf(i))
            jlbl = index(buf(i),"#")
            ilbl = max(ilbl,jlbl)
            buf(i)(jlbl:jlbl) = " "
         end do
         jlbl = ilbl

         write(idump,'(i4,1p2e14.5,3x,a,3x,1p3e14.5,a,3x,a)') &
              np, mean(np), rms(np), &
              "largest", (np_largestV(f,np),f=1,3)," |", &
              trim(buf(1)(1:jlbl))

         write(idump,'(35x,a,1p3e14.5,a,3x,a)') &
              "lowest abs", (np_lowestV(f,np),f=1,3)," |", &
              trim(buf(2)(1:jlbl))

         write(idump,'(35x,a,2x,1p3e14.5,a,3x,a)') &
              "smallest", (np_smallestV(f,np),f=1,3)," |", &
              trim(buf(3)(1:jlbl))


         ilbl = index( npotfnames(np)," ")-1
         write(idump,'(a)')"   file = "//npotfnames(np)(1:ilbl)
         write(idump,"(102a)")("-",f=1,102)
      end do

      close(idump)

      end subroutine outNPdata

      end program npotminmax
