!***********************************************************************
!
!                         program rdnatpot
!
! reads a natpot file
!
! HDM 06/2015
!
!
!***********************************************************************

      program rdnatpot

use decimal
use global
use globmemmod
use pglobal
use pdefaultmod
use aglobal
use maxv 
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use showpotmod
use channels
use basismod
use logdat
use versions
use plot
use peingutil
use fitinmod
use rdnatpot
use rddvrmod, only: rddvr, dvrdata
use vpotpars, only: readvpotpars

      implicit none

      integer(long) ::  realsize,integersize,logicalsize,chkdvr,len,ierr,&
                        ndof1,ilbl,i,n,f,m,chkgrd,nmode1
      integer(long), dimension(:), allocatable  :: fdvr,subdim1,mdvr,mmode,&
                                                   pntbas
      integer(long), dimension(6)               :: dynbytes
      real(dop)     ::  totmem
      real(dop), dimension(:), allocatable      :: coeff
      logical(kind=4) :: lerr

!-----------------------------------------------------------------------
! Initialize variables and set defaults
!-----------------------------------------------------------------------
      call default
      call pdefault
      call adefault
      lovwr     = .true.
      natpotcut = 0.0d0    ! default: no cut for natpots
      iout = ilog       ! there is no output file, redirect to log file.

!-----------------------------------------------------------------------
! read input parameters and options
!-----------------------------------------------------------------------
      call rdnatpot_options

!-----------------------------------------------------------------------
! convert filenames to absolute pathnames and open files
!-----------------------------------------------------------------------
      call abspath(name,laenge)
      if(lprint) then
         ilog=6
      else
         if(laeoutd .eq. 0) then
            logfile=name(1:laenge)//'/rdnatpot.log'
         else
            logfile=dirout(1:laeoutd)//'/rdnatpot.log'
         endif
         call popenlog(logfile)
      endif

      len=index(npotfile,' ')-1
      call abspath(npotfile,len)
      open(unit=inpot,file=npotfile(1:len),form='unformatted', &
           status='old',iostat=ierr)
      if (ierr.ne.0) then
         routine='showpot'
         message='could not open '//npotfile(1:len)// &
              '                              Try option "-n no" '
         call errormsg
      endif

      dvrfile = name(1:laenge)//'/dvr'
      len=index(dvrfile,' ')-1
      call abspath(dvrfile,len)
      open(unit=idvr,file=dvrfile(1:len),form='unformatted',&
           status='old',iostat=ierr)
      if (ierr.ne.0) then
         routine='rdnatpot'
         message='could not open '//dvrfile(1:len)
         call errormsg
      endif

!-----------------------------------------------------------------------
! get memory sizes
!-----------------------------------------------------------------------
      call rdmemdim(idvr)

      close(idvr)

!-----------------------------------------------------------------------
! Allocate memory
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      call alloc_runpfitmod

!-----------------------------------------------------------------------
! allocate local memory arrays
!-----------------------------------------------------------------------
      allocate(fdvr(maxdim))
      allocate(subdim1(maxdim))
      allocate(mdvr(maxdim))
      allocate(mmode(maxdim))
      allocate(pntbas(maxdim))
      allocate(lonedflag(maxdim))

!-----------------------------------------------------------------------
! read natpot information
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      call npotinfo(inpot,lerr,modc,lonedflag,ndof1,fdvr, &
           chkdvr,chkgrd,1,mdvr,mmode,nspfdof,spfdof,nmode1, &
           subdim1,pntbas)
      ndof=ndof1

      write(ilog,'(/a)') 'Primitive Basis Info'
      write(ilog,'(a,i6,a,i6)') ' ndof =', ndof,' ,  nmode =',nmode1
      write(ilog,'(a,1x,12a10)') ' mlabel = ', &
              (trim(modelabel(f))//'  ',f=1,ndof)
      write(ilog,'(a,12i10)') ' gdim   =', (gdim(f),f=1,ndof)
      write(ilog,'(a,12i10)') ' basis  =', (basis(f),f=1,ndof)
      write(ilog,'(a,12(2x,a8))') ' basis  =', &
              (trim(cbas(basis(f))),f=1,ndof)
      write(ilog,'(a,12f10.4)') ' xi     =', (xend(1,f),f=1,ndof)
      write(ilog,'(a,12f10.4)') ' xf     =', (xend(2,f),f=1,ndof)
      write(ilog,'(a,12i10)') ' dofspf =', (dofspf(f),f=1,ndof)
      write(ilog,'(/a)') ' spfdof'
      do m=1,nmode
         write(ilog,'(i4,") ",12i10)') m, (spfdof(n,m),n=1,nspfdof(m))
      enddo

      write(ilog,'(/a)') ' ipbaspar'
      do i = 1, mbaspar
         write(ilog,'(i3,")  ",10i6)') i,(ipbaspar(i,f),f=1,ndof)
      enddo
      write(ilog,'(/a)') ' rpbaspar'
      do i = 1, mbaspar
         write(ilog,'(i3,")  ",12f14.6)') i,(rpbaspar(i,f),f=1,ndof)
      enddo

      write(ilog,'(/a/)') '--------------------------------------------'

      coeffdim=pdim(modc)*psubdim(modc)
      do m=1,nmode
         if (m.ne.modc) then
            potdim(m)=pdim(m)
         else
            potdim(m)=-1
         endif
      enddo

      call init_cmode

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call rdnatpot_mem

!-----------------------------------------------------------------------
! call caller routine to assign the memory. Caller calls receiver.
!-----------------------------------------------------------------------
      dynbytes(1) = realsize()*(workrdim/8+1)
      dynbytes(2) = realsize()   *(mrdim/8+1)
      dynbytes(3) = integersize()*(midim/8+1)
      dynbytes(4) = logicalsize()*(mldim/8+1)
      dynbytes(5) = realsize()*(coeffdim/8+1)
      dynbytes(6) = (realsize()/2)   *(mfdim/8+1)
      ilbl=0
      totmem=0.d0
      do i=1,6
         totmem=totmem+dynbytes(i)
         if( dynbytes(i).lt.0 ) then
            ilbl=1
            totmem=totmem+4.294967296d+9
         endif
      enddo
      totmem=totmem*8.d0

      write(ilog,'(80a1,/)') ('-',i=1,80)
      write(ilog,'(a)') '  *** Memory allocated dynamically ***'
      write(ilog,'( (a,i12,f11.2," MB") )') &
       ' work       :', workrdim,dynbytes(1)/131072.d0, &
       ' coeff      :', coeffdim,dynbytes(5)/131072.d0, &
       ' real*8     :', mrdim,   dynbytes(2)/131072.d0, &
       ' integer    :', midim,   dynbytes(3)/131072.d0, &
       ' real*4     :', mfdim,   dynbytes(6)/131072.d0
      write(ilog,'(a,f16.0,a,f11.2,a)') &
       ' Total   :',totmem,' Bytes ,',totmem/1048576.d0,' MB'
      write(ilog,'(80a1,/)') ('-',i=1,80)
      call myflush(ilog)

       if(ilbl.gt.0) then
         routine='showpot'
         write(message,'(3a,f9.1,a)') &
           'Integer overflow in memory assignment. ', &
           'You are asking for too much memory.    ', &
           'Total Memory asked for =',totmem/1048576.d0,' MB'
         call errormsg
      endif

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))
allocate(coeff(coeffdim))

      call sub_rdnatpot(mr(mrort),mr(mroned),coeff,mr(mrevec))

deallocate(mc,mr,mi,ml,ms,mf)
deallocate(coeff)
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_runpfitmod

      deallocate(fdvr)
      deallocate(subdim1)
      deallocate(mdvr)
      deallocate(mmode)
      deallocate(pntbas)
      deallocate(lonedflag)

      stop

      contains

!#######################################################################
!-----------------------------------------------------------------------
! process command line args
!-----------------------------------------------------------------------

      subroutine rdnatpot_options

      implicit none

      integer(long)     :: iarg, myiargc, i, slen, len, ierr
      logical(kind=4)   :: ld
      character(len=80) :: buf

      name     = './'
      npotfile = ' '
      laenge   = 1
      filever(inpot)=0
      zaxunitlab='au'
      zaxunit=1.d0
      ld = .false.
      lcontr=.false.
      lbinary=.false.
      lprint=.false.
      lvpot=.false.
      mevec=0

      iarg = myiargc()
      if (iarg .gt. 8) call rdnp_usage

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf .eq. '-D' ) then
            i = i+1
            call mygetarg(i,buf)
            if(buf .eq. 'no') then
               lprint=.true.
            else
               dirout = buf
               laeoutd=index(dirout,' ')-1
               call abspath(dirout,laeoutd)
            endif
         else if( buf(1:3) .eq. '-n ' ) then
            i = i+1
            call mygetarg(i,buf)
            npotfile = buf
         else if( buf(1:3) .eq. '-b ' ) then
            lbinary = .true.
         else if( buf(1:7) .eq. '-contr ' ) then
            lcontr=.true.
            vpotfile = 'no'
         else if( buf(1:6) .eq. '-evec ' ) then
            vpotfile = 'no'
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) mevec
            if(mevec.le.0 .or. ierr.ne.0) then
               write(6,'(2a)') &
              'Expecting a mode number after "-evec", not: ',buf(1:30)
              stop
            endif
         else if( buf(1:3) .eq. '-h ' .or. buf(1:3) .eq. '-? ' ) then
            call rdnp_usage
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
  200 continue
      if(.not.ld) dname=name
      dlaenge = slen(dname)
      if(dname(dlaenge:dlaenge).eq.'/') dlaenge=dlaenge-1
      if(name(laenge:laenge).eq.'/') laenge=laenge-1


!-----------------------------------------------------------------------
! if natpot has not been set, look in name directory.
! else take name from npotfile
!-----------------------------------------------------------------------
      if (npotfile .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         npotfile = name(1:laenge)//'/natpot'
         lnpot=.true.
         len=index(npotfile,' ')-1
         inquire(file=npotfile(1:len),exist=lnpot)
         if (.not. lnpot) then
            npotfile=' '
            lnpot=.false.
         endif
      else
         len=index(npotfile,' ')-1
         call abspath(npotfile,len)
         lnpot=.true.

         i=len
   10    continue
         if (npotfile(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=npotfile(1:laenge)
      endif

      return
      end subroutine rdnatpot_options


      subroutine rdnp_usage
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     print usage message and stop
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      implicit none

      integer(long) :: i

      write(6,'(78a1)')  ('-',i=1,78)
      write(6,'(a,/11x,a,/,a)') &
       'Purpose  : Prints information on a natpot to file rdnatpot.log', &
       'On demand it prints natural potentials and D-tensor.', &
       'Usage    : rdnatpot [-i -D -n -evec -b -contr -ver -h -?]'
      write(6,'(/a)') 'Options  :'
      write(6,'(a)') &
         ' -i NAME : The files are read/stored in the directory NAME'
      write(6,'(2a,/,11x,a/,11x,a/,11x,a)') &
            ' -D DIR  : The output is written to directory DIR.' &
           ,' This is useful,' &
           ,'if one has no right to write to the name directory.' &
           ,'The directory DIR must exist.' &
           ,'The string DIR may be a relative or a full path-name.' &
           ,'The output is directed to screen when setting "-D no".'
      write(6,'(a,/,11x,a)') &
         ' -n FILE : The potential fit is read from file FILE', &
         'If "-n FILE" is not given, FILE=natpot is assumed.'
      write(6,'(a,/11x,a,/11x,a,/11x,a)') &
         ' -evec M : The eigenvectors (natural potentials)', &
         'of mode M are written to ASCII file evec<M>.', &
         'M must not equal modc (contracted mode).', &
         'If M=99 then natpots of all modes (except modc) are written.'
      write(6,'(a,/11x,a)') &
         ' -b      : Eigenvectors (natural potentials) are additionally' &
         ,'written to binary file evecb<M>, (if -evec is set).'
      write(6,'(a,/11x,a,/11x,a)') &
         ' -contr  : The contracted natural potentials of the', &
         'contracted mode are written to ASCII file contr.', &
         'WARNING: This may be a lot of data.'
      write(6,'(a)') &
         ' -ver    : Version information about the program '
      write(6,'(a,/a)') &
         ' -h      : Print this help text. ', &
         ' -?      : Print this help text. '
      write(6,'(78a1)')  ('-',i=1,78)

      stop
      end subroutine rdnp_usage

!#######################################################################
!***********************************************************************
!
! calculates amount of memory to be allocated and the pointer structure
!
!***********************************************************************

      subroutine rdnatpot_mem

      implicit none

      integer(long) :: ff,f,n,m,mm,wkdim
      integer(long) :: reorder(maxdim)
      real(dop)     :: rvpdim

      maxpsub   = 0
      maxgdim   = 0
      ortdim    = 0
      evecdim   = 0
      modcdim   = 1
      vpotdim   = 1
      rvpdim    = 1.d0


!-----------------------------------------------------------------------
! set arrays vgmdim and vgpdim, which contains the `vor'-grid-dimensions
! in mode order and in dof order. (copy/paste from zeigpfit).
!-----------------------------------------------------------------------
      ff=0
! --- ff=mode order, f=dof order.
      do m=1,npmode
         do n=1,npotdof(m)
            f=potdof(n,m)
            ff=ff+1
            reorder(ff)=f
         enddo
      enddo

! --- set vgmdim, `vor'-grid-dimension in mode order.
      vgmdim(reorder(1))=1
      do ff=1,ndof-1
         vgmdim(reorder(ff+1))=vgmdim(reorder(ff))*gdim(reorder(ff))
      enddo

! --- set vgpdim, `vor'-grid-dimension in dof order.
      vgpdim(1)=1
      do f=1,ndof-1
         vgpdim(f+1)=vgpdim(f)*gdim(f)
      enddo

!-----------------------------------------------------------------------

      do f=1,ndof
         maxgdim=max(maxgdim,gdim(f))
!         if(basis(f).eq.12) maxgdim=max(maxgdim,ipbaspar(3,f))
         zort(f)=ortdim+1
         if(basis(f).eq.12)  then
            ortdim=ortdim+ipbaspar(3,f)
         else
            ortdim=ortdim+gdim(f)
         endif
      enddo
      oneddim=ortdim

      do m=1,nmode
         maxpsub  = max(maxpsub,psubdim(m))
         if (npotfile.ne.' ') then
            if (m.ne.modc) then
               zevec(m)=evecdim+1
               evecdim=evecdim+psubdim(m)*potdim(m)
               modcdim=modcdim*potdim(m)
            else
               zevec(m)=-1
            endif
         endif
         vpotdim=vpotdim*psubdim(m)
         rvpdim=rvpdim*psubdim(m)
      enddo

      if(vpotdim .lt. 0 .or. rvpdim .gt. 2.5d+9) then
         write(ilog,'(a,1pe16.9)') 'vpotdim =', rvpdim
      else
         write(ilog,'(a,i12,1pe12.3)') 'vpotdim =',vpotdim,rvpdim
      endif

      mrort=mrdim
      mrdim=mrdim+ortdim
      if (npotfile.ne.' ') then
         mroned=mrdim
         mrdim=mrdim+oneddim
         mrevec=mrdim
         mrdim=mrdim+evecdim
         coeffdim=modcdim*dimmodc
      else
         mroned=1
         mrevec=1
         coeffdim=1
      endif

      mrpldata=mrdim
      mrdim=mrpldata+maxgdim**2

!-----------------------------------------------------------------------
! assign work arrays
! NB work arrays must be at end of mc, mr, mi, ml arrays.
!-----------------------------------------------------------------------
!     stuff used for the calculation of the potential fit

      c2v(1)=modcdim
      wkdim=c2v(1)+maxpsub
      mm=0
      do m=1,nmode
         if (m.ne.modc) then
            mm=mm+1
            c2v(mm+1)=c2v(mm)/potdim(m)
            if (mm+1.eq.2) wkdim=wkdim+c2v(mm+1)
         endif
      enddo
      workrdim=max(workrdim,wkdim)

      workdim2 = workrdim  ! needed for getvfit

      miworki = midim
      midim   = miworki+workidim

      mldim   = 1
      mfdim   = 1

      return
      end subroutine rdnatpot_mem


!***********************************************************************
!
!     HERE IS THE PLOT SUBROUTINE: sub_plpot
!
!***********************************************************************

      subroutine sub_rdnatpot(ort,oned,coeff,evec)

      implicit none

      real(long), parameter :: huge = 1.0d38

      real(dop) :: ort(ortdim),oned(oneddim),coeff(coeffdim),evec(evecdim)

      integer(long)   :: idum(1)
      real(dop)       :: rdum(1)
      complex(dop)    :: zdum(1)
      logical(kind=4) :: ldum,lwrout

      integer(long)   :: f,f1,n,m,nsets,nplots

!     rcut(i) : location of the grid point at which mode i is cut.
!     icut(i) : number of the grid point at which mode i is cut. if mode i
!               is not cut, i.e. if it is associated with the x(y)-axis,
!               icut(i)=-1(-2).
!               If a minimum (maximum) is searched for icut(i)=-9(-8).

!-----------------------------------------------------------------------
! load data and vpot parameters from natpot file if desired
!-----------------------------------------------------------------------
      call fitin(inpot,lonedflag,oned,evec,coeff)
      lwrout=.true.
      call readvpotpars(inpot,npotfile,progver1,label1,opts1,lsp1, &
           ldum,idum(1),pesmin1,pesmax1,lcspot1,jtot1,jbf1,csmass1,lwrout)
      close(inpot)
      do f=1,ndof
         if (lonedflag(f)) then
            ploned=.true.
            onedipot(f)=1
         endif
      enddo
      lsp=lsp1
      pesmin=pesmin1
      pesmax=pesmax1

!-----------------------------------------------------------------------
! If option -evec or -contr is given, write the eigenvectors (natural
! potentials) of a given mode to file.
!-----------------------------------------------------------------------
      if(lcontr) then
         call wrcontr1(coeff,potdim,psubdim(modc),modc,modcdim,nmode)
      endif

      if(mevec .gt. 0) then
         dvrdata(:) = .false.
         dvrdata(1) = .true.
         chkdvr=2
         inquire(file=dname(1:dlaenge)//'/dvr',exist=ldum)
         if(ldum) then
            call rddvr(ort,rdum,rdum,rdum,zdum, &
               zdum,idum,zdum,zdum,idum,idum,rdum,chkdvr)
         else
            write(6,'(2a)') 'No dvr-file found, ', &
             'hence no ort (coordinate)  information provided.'
         endif
         if(mevec .eq. 99) then
            do m=1,nmode
               if(m.ne.modc) then
                 f=spfdof(1,m)
                 f1=max(1,spfdof(2,m))
                 call wrevec1(evec(zevec(m)),m,potdim(m),psubdim(m), &
                             ort(zort(f)),ort(zort(f1)),gdim(f), &
                             gdim(f1),nspfdof(m),f,f1,lbinary)
               endif
            enddo
         elseif(mevec .eq. modc) then
            write(6,'(a,i3,a,i3)') ' The evec-mode, mevec =', &
            mevec,', must not equal the contracted mode, modc =',modc
         elseif(mevec .gt. nmode) then
            write(6,'(a,i3,a,i3)') ' The evec-mode, mevec =', &
            mevec,', must not be greater than nmode =',nmode
         else
            f=spfdof(1,mevec)
            f1=max(1,spfdof(2,mevec))
            call wrevec1(evec(zevec(mevec)),mevec,potdim(mevec), &
                        psubdim(mevec),ort(zort(f)),ort(zort(f1)), &
                        gdim(f),gdim(f1),nspfdof(mevec),f,f1,lbinary)
         endif
      endif
      if(mevec.gt.0 .or. lcontr)  stop

      return
      end subroutine sub_rdnatpot

!#######################################################################
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     calculates a single datum `x' associated with grid point `igpoint'
!
!     igpoint (i)
!     oned (i)
!     coeff (i)
!     evec (i)
!     workr (w)
!     x (o)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!!      subroutine calcnpot(igpoint,oned,coeff,evec,workr,x)
!!
!!      implicit none
!!
!!      integer igpoint(ndof)
!!      real*8 x,y,v1d,coeff(coeffdim),evec(evecdim),workr(workrdim)
!!      real*8 oned(oneddim)
!!
!!      call getvfit(igpoint,coeff,evec,workr,x)
!!      call getv1d(igpoint,oned,v1d)
!!      x=x+v1d
!!
!!      return
!!      end

!#######################################################################
!    SUBROUTINE WREVEC1
!    This routine writes the eigenvectors (natural potentials) to
!    an ASCII file
!    Same as WREVE! n showpot.F
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine wrevec1(evec,m,pdim,subdim,ort,ort1,gdim,gdim1,nspfdof, &
                        f,f1,lbinary)

      implicit none

      integer(long) :: m,n,pdim,gdim,subdim,g,g1,g2,gdim1,nspfdof,f,f1
      real(dop)     :: evec(subdim,pdim),ort(gdim),ort1(gdim1)
      character(len=240) :: filename,filenameb
      character(len=8)   :: cdof
      logical(kind=4)    :: lbinary

      write(filename,'(2a,i2.2)') name(1:laenge),'/evec',m
      open(ieigvec,file=filename,form='formatted',status='unknown',err=999)

      if(lbinary) then
         write(filenameb,'(2a,i2.2)') name(1:laenge),'/evecb',m
         open(ioutbin,file=filenameb,form='unformatted',status='unknown',err=998)
      endif
      if (nspfdof.eq.1) then
         write(cdof,'(i3)') f
      else
         write(cdof,'(i3,",",i3)') f, f1
      endif

      write(ieigvec,'(a,i3,2a,/2a,/a,/4a,/4a,/a,/a,/3a,/)') &
       '# potfit eigenvectors (natural potentials) of mode =',m, &
       ', DOF =',cdof, &
       '# Columns: grid-point, (coordinate(s)) 1st vector,', &
       ' 2nd vector, ...', &
       '# For uncombined modes plotting may be performed as follows:', &
       '# Use "plgen ',filename(laenge+2:laenge+7),' 1:3 - 1:4 - 1:5"', &
       ' to plot the first three vectors vs grid-points.', &
       '# Use "plgen ',filename(laenge+2:laenge+7),' 2:3 - 2:4 - 2:5"', &
       ' to plot the first three vectors vs coordinate.', &
       '# For 2D combined modes both coordinates are written.', &
       '# For combinations higher than 2D no coordinates are written.', &
       '# Inspect this file with "less -S ', &
        filename(laenge+2:laenge+7),'".'

      if(nspfdof.eq.1) then
         write(ieigvec,'(a,i6,99i10)') '#   g  coordinate',(n,n=1,pdim)
         do g=1,subdim
            write(ieigvec,'(i5,f12.6,99f10.6)') &
                           g, ort(g), (evec(g,n),n=1,pdim)
         enddo
         write(ieigvec,'(a,i6,99i10)') '#   g  coordinate',(n,n=1,pdim)
      elseif(nspfdof.eq.2) then
         write(ieigvec,'(a,i6,99i10)') &
                '#   g  coordinate  coordinate', (n,n=1,pdim)
         do g=1,subdim
            g2=(g-1)/gdim
            g1=g-g2*gdim
            g2=g2+1
            write(ieigvec,'(i5,2f12.6,99f10.6)') &
              g, ort(g1), ort1(g2), (evec(g,n),n=1,pdim)
         enddo
         write(ieigvec,'(a,i6,99i10)') &
                '#   g  coordinate  coordinate', (n,n=1,pdim)

      else
         write(ieigvec,'(a,i6,99i10)') '#   g  ',(n,n=1,pdim)
         do g=1,subdim
            write(ieigvec,'(i5,999f10.6)') g, (evec(g,n),n=1,pdim)
         enddo
         write(ieigvec,'(a,i6,99i10)') '#   g  ',(n,n=1,pdim)
      endif

      write(6,'(2a,i3,a,/a)') 'Eigenvectors (natural potentials)', &
       ' of mode',m,' are written to file ',filename(1:laenge+7)

      close(ieigvec)

! Printing evecb files (binary evec files)

      if(lbinary) then
        do g=1,subdim
          write(ioutbin) (evec(g,n),n=1,pdim)
        enddo
        close(ioutbin)
      end if

      return
 998  write(6,'(/2a)') 'Cannot open evecb-file: ',trim(filenameb)
 999  write(6,'(/2a)') 'Cannot open evec-file: ',trim(filename)
      stop
      end subroutine wrevec1

!#######################################################################
!    SUBROUTINE WRCONTR1
!    This routine writes the contracted mode natpots to an ASCII file
!    Same as WRCONTR on showpot.F
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine wrcontr1(coeff,potdim,csubdim,modc,modcdim,nmode)

      implicit none

      integer(long) :: i,m,g,csubdim,modc,modcdim,nmode
      integer(long) :: indx(16), potdim(nmode)
      real(dop)     :: coeff(csubdim,modcdim),rms,cmax
      character(len=240) :: filename

      write(filename,'(2a)') name(1:laenge),'/contracted'
      open(ieigvec,file=filename,form='formatted',status='unknown',err=999)

      write(ieigvec,'(a,i3)') &
       '# potfit contracted natpots of contracted mode.  modc =',modc

      do m=1,nmode
         indx(m) = 1
      enddo
      indx(modc)=0

      do i=1,modcdim
         rms=0.d0
         cmax=0.d0
         do g=1,csubdim
            rms=rms+coeff(g,i)**2
            cmax=max(cmax,abs(coeff(g,i)))
         enddo
         rms=sqrt(rms/dble(csubdim))
         write(ieigvec,'(/a,i3,a,1pe12.5,a,1pe12.5,a,16i3)') &
             '# I =',i,',  rms =',rms,',  max =',cmax, &
             ', indx:',(indx(m),m=1,nmode)
         write(ieigvec,'(1p5e16.7)') (coeff(g,i),g=1,csubdim)

! --- determine new set of indices
         do m=1,nmode
            if(m.ne.modc) then
               if (indx(m).lt.potdim(m)) then
                  indx(m)=indx(m)+1
                  goto 1
               else
                  indx(m)=1
               endif
            endif
         enddo
 1    continue
      enddo

      write(6,'(2a,i3,a,/a)') 'Contracted natural potentials', &
       ' of mode modc =',modc,' are written to file ', &
        filename(1:laenge+11)

      close(ieigvec)

      return

 999  write(6,'(/2a)') 'Cannot open file: ',filename(1:laenge+11)
      stop
      end subroutine wrcontr1

!#######################################################################

      end program rdnatpot

