!***********************************************************************
!
!                         program showpot
!
! reads a dvr, a vpot, and a natpot file (at least one of the latter two).
! outputs a gnuplot file to plot the potential and/or the potential fit or
! their difference.
!
! stephan wefing, 1999/2000
!
!
!***********************************************************************

      program showpot

use decimal, only: long,dop,sip
use global
use globmemmod
use aglobal
use pglobal
use maxv, only: maxdim
use keyunitsmod
use dvrdatmod
use griddatmod
use operdef
use channels
use combined
use runpfitmod
use showpotmod
use plot

use op1lib, only: cpvxd
use logdat, only: errormsg
use rddvrmod, only: dvrinfo,dvrdata,rddvr
use iogrddef, only: rdgrddef
use peingutil, only: popenlog
use pdefaultmod, only: pdefault,init_cmode
use vpotpars, only: readvpotpars,readvpotinfo
use versions
use logdat
use rdnatpot, only: npotinfo

use pesfit, only: getvfit
use getpesmod, only: getv1d
use fitinmod, only: fitin

use plotutils, only: plotdefs,initplot,inmodebnds,plotmenu,databnds,dotask

use module_interface

      implicit none

      integer(long), external                       :: slen
      integer(long)                                 :: chkdvr,len,ierr,ndof1,&
                                                       ilbl,i,n,f,m,chkgrd,nmode1,&
                                                       idum,dvrcheck,jlbl
      integer(long), dimension(:), allocatable      :: fdvr,subdim1,mdvr,mmode,&
                                                        pntbas,nspfdof1
      integer(long), dimension(:,:), allocatable    :: spfdof1
      logical(kind=4)                               :: ldum,lerr
      character(len=c2)                             :: rdtag

!-----------------------------------------------------------------------
! Initialize variables and set defaults
!-----------------------------------------------------------------------
      call default
      call adefault
      call pdefault
      lovwr     = .true.
      natpotcut = 0.0_dop    ! default: no cut for natpots
      iout = ilog       ! there is no output file, redirect to log file.

!-----------------------------------------------------------------------
! provide default input and output filenames 
!-----------------------------------------------------------------------
      name       = '.'
      laenge     = 1
      dname      = '.'
      dlaenge    = 1
      vpotfile   = ' '
      npotfile   = ' '
      logfile    = 'showpot.log'
      lvpot      = .false.
      filever(ivpot)=0
      filever(inpot)=0

!-----------------------------------------------------------------------
! read input parameters and options 
!-----------------------------------------------------------------------
      call plpot_options

!-----------------------------------------------------------------------
! convert filenames to absolute pathnames and open files
!-----------------------------------------------------------------------
      call abspath(name,laenge)
      if(laeoutd .eq. 0) then
         logfile=name(1:laenge)//'/showpot.log'
      else
         logfile=dirout(1:laeoutd)//'/showpot.log'
      endif
      call popenlog(logfile)

      dvrfile = dname(1:dlaenge)//'/dvr'
      len=index(dvrfile,' ')-1
      call abspath(dvrfile,len)
      dlaenge=len-4
      dname=dvrfile(1:dlaenge)
      open(unit=idvr,file=dvrfile(1:len),form='unformatted',&
         status='old',iostat=ierr)
      if (ierr.ne.0) then
         routine='showpot'
         message='could not open '//dvrfile(1:len)
         call errormsg
      endif

      if (vpotfile.ne.' ') then
         len=index(vpotfile,' ')-1
         call abspath(vpotfile,len)
         open(unit=ivpot,file=vpotfile(1:len),form='unformatted',&
            status='old',iostat=ierr)
         if (ierr.eq.0) then
            vname=vpotfile
            vlength=slen(vpotfile)
         else
            routine='showpot'
            message='could not open '//vpotfile(1:len)//&
            '                              Try option "-v no" '
            call errormsg
         endif
      else
         write(6,   '(a)') '###  No vpot file opened.'
         write(ilog,'(a)') '###  No vpot file opened.'
      endif

      if (npotfile.ne.' ') then
         len=index(npotfile,' ')-1
         call abspath(npotfile,len)
         open(unit=inpot,file=npotfile(1:len),form='unformatted',&
            status='old',iostat=ierr)
         if (ierr.ne.0) then
            routine='showpot'
            message='could not open '//npotfile(1:len)//&
            '                              Try option "-n no" '
            call errormsg
         endif
      else
         write(6,'(a)')     '###  No natpot-file opened. '
         write(ilog,'(a)')  '###  No natpot-file opened. '
      endif

!-----------------------------------------------------------------------
! get memory sizes 
!-----------------------------------------------------------------------
      call rdmemdim(idvr)

!-----------------------------------------------------------------------
! Allocate memory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      call alloc_runpfitmod

!-----------------------------------------------------------------------
! read dvr information from dvr file into common variables
!-----------------------------------------------------------------------
      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      close(idvr)
      call chksyserr(dvrfile,lerr)
      if (lerr) stop

!-----------------------------------------------------------------------
! allocate local memory arrays
!-----------------------------------------------------------------------
      allocate(fdvr(maxdim))
      allocate(subdim1(maxdim))
      allocate(mdvr(maxdim))
      allocate(mmode(maxdim))
      allocate(pntbas(maxdim))
      allocate(nspfdof1(maxdim))
      allocate(spfdof1(maxdim,maxdim))
      allocate(lonedflag(maxdim))

!-----------------------------------------------------------------------
! read vpot information if desired
!-----------------------------------------------------------------------
      if (vpotfile.ne.' ') then

         dvrcheck = 2
         call readvpotinfo(ivpot,progver1,label1,opts1,lsp1,&
                           ldum,idum,pesmin1,pesmax1,dvrcheck,&
                           lcspot1,jtot1,jbf1,csmass1,fdvr)

! hack to read file version for old formats
         if (progver1 .lt. 11.0) then
            read (ivpot,err=35) idum,niobuf
            backspace (unit=ivpot, iostat=ierr)
            if (ierr.ne.0) then
               routine='read(ivpot) after readvpotinfo'
               message='cannot backspace after reading niobuf'
               call errormsg
            endif
         else
            rewind(ivpot)
            do
  20           continue
               read(ivpot,iostat=chkerr(1),err=20) rdtag
               if (rdtag(1:1) .ne. '<') cycle

               ilbl=2
               jlbl=index(rdtag,'>')-1

               if (rdtag(ilbl:jlbl) .eq. 'vpot pes') then
                  exit
               endif
            enddo

            read (ivpot,err=35) idum,niobuf
         endif

         lsp=lsp1
         pesmin=pesmin1
         pesmax=pesmax1
         write(ilog,*) 'niobuf  =',niobuf
         filever(ivpot)=progver1
         if (progver1 .ge. 8.3013_dop) then
            continue
         elseif (progver1 .ge. 8.3011_dop) then
            chkgrd=1
            call rdgrddef(ivpot,chkgrd,fdvr,mdvr,mmode,nspfdof1,spfdof1,nmode1)
         endif

      endif

      goto 32
 35   vpotfile=' '
      plottask1=1
      lvpot=.false.
      write(6,'(a/a)') '## Cannot read vpot file. Probaly old version.',&
                       '## Setting option "-v no"'

!-----------------------------------------------------------------------
! read natpot information if desired
!-----------------------------------------------------------------------
 32   if (npotfile.ne.' ') then
         chkdvr=2
         chkgrd=1 
         call npotinfo(inpot,lerr,modc,lonedflag,ndof1,fdvr,&
            chkdvr,chkgrd,1,mdvr,mmode,nspfdof1,spfdof1,nmode1,&
            subdim1,pntbas)
         ndof=ndof1

         if (filever(inpot).ge.8.20030001_dop) then
!----------------------------------------------------------------------
! The degrees of freedom can be combined in potfit
!---------------------------------------------------------------------- 
           coeffdim=pdim(modc)*psubdim(modc)
           do m=1,nmode
             if (m.ne.modc) then
                potdim(m)=pdim(m)
             else
                potdim(m)=-1
             endif
           enddo
         else
!---------------------------------------------------------------------
! Here the degrees of freedom can not be combined
! the modes are the degrees of freedom 
! initialise the variables  
!---------------------------------------------------------------------
           npmode=ndof1
           nmode=nmode1
           coeffdim=pdim(modc)*gdim(modc)
           do f=1,ndof
              psubdim(f)=gdim(f)
           enddo
           do f=1,ndof
             if (f.ne.modc) then
                potdim(f)=pdim(f)
             else
                potdim(f)=-1
             endif
             npotdof(f)=1
             potdof(1,f)=f
             dofpot(f)=f 
           enddo
         endif
         call init_cmode
      endif

!-----------------------------------------------------------------------
! map ndof to nmode, if natpot-file is not read.
!-----------------------------------------------------------------------
      if (npotfile.eq.' ' ) then
         if (filever(ivpot).ge.8.3011_dop .and.&
             filever(ivpot).lt.8.3013_dop) then
            npmode=nmode
            do m=1,npmode
               npotdof(m)=nspfdof(m)
               psubdim(m)=subdim(m)
               do n=1,npotdof(m)
                  potdof(n,m)=spfdof(n,m)
               enddo
            enddo
            do f=1,ndof
               dofpot(f)=dofspf(f)
            enddo
         else
            nmode=ndof
            npmode=ndof
            do m=1,nmode
               nspfdof(m)=1
               npotdof(m)=1
               spfdof(1,m)=m
               potdof(1,m)=m
               dofspf(m)=m
               dofpot(m)=m 
               psubdim(m)=gdim(m)
            enddo
         endif
      endif

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! assign memory arrays
!-----------------------------------------------------------------------
      call plpot_mem

!-----------------------------------------------------------------------
! call analysis routine
!-----------------------------------------------------------------------
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'(/,80a1)') ('-',i=1,80)
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      write(ilog,'(80a1,/)') ('-',i=1,80)
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

      call sub_plpot(mr(mrort),mf(mfvvec),mr(mroned),mr(mrcoeff),&
                 mr(mrevec),mr(mrworkr),mr(mrpldata))

deallocate(mc,mr,mi,ml,ms,mf)
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_runpfitmod

      deallocate(fdvr)
      deallocate(subdim1)
      deallocate(mdvr)
      deallocate(mmode)
      deallocate(pntbas)
      deallocate(nspfdof1)
      deallocate(spfdof1)
      deallocate(lonedflag)

      stop

      contains

!#######################################################################
!-----------------------------------------------------------------------
! process command line args
!-----------------------------------------------------------------------

      subroutine plpot_options

      implicit none

      integer(long), external :: myiargc,slen
      integer(long)           :: iarg,i,len,ierr
      logical(kind=4)         :: ld
      character(len=80)       :: buf
      
      name   = './'  
      zaxunitlab='au'
      zaxunit=1.0_dop
      plottask1=0
      ld = .false.
      lcontr=.false.
      lbinary=.false.
      mevec=0

      iarg = myiargc() 
      if (iarg .gt. 8) call plpot_usage

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:3) .eq. '-d ') then
            i = i+1
            call mygetarg(i,buf)
            dname   = buf
            dlaenge = slen(dname)
            ld = .true.
         else if(buf .eq. '-i') then
            i=i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf .eq. '-D' ) then
            i = i+1
            call mygetarg(i,buf)
            dirout = buf
            laeoutd=index(dirout,' ')-1
            call abspath(dirout,laeoutd)
         else if( buf(1:3) .eq. '-v ' ) then
            i = i+1
            call mygetarg(i,buf)
            vpotfile = buf
         else if( buf(1:3) .eq. '-b ' ) then
            lbinary = .true.
         else if( buf(1:3) .eq. '-n ' ) then
            i = i+1
            call mygetarg(i,buf)
            npotfile = buf
         else if( buf(1:7) .eq. '-contr ' ) then
            lcontr=.true.
            vpotfile = 'no'
         else if( buf(1:6) .eq. '-evec ' ) then
            vpotfile = 'no'
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) mevec
            if(mevec.le.0 .or. ierr.ne.0) then
               write(6,'(2a)') &
                 'Expecting a mode number after "-evec", not: ',buf(1:30)
              stop
            endif
         else if( buf(1:6) .eq. '-vfit ' ) then
            plottask1=1
         else if( buf(1:6) .eq. '-vpot ' ) then
            plottask1=2
         else if( buf(1:6) .eq. '-diff ' ) then
            plottask1=3
         else if( buf(1:5) .eq. '-abs ' ) then
            plottask1=4
         else if( buf(1:4) .eq. '-ig ' ) then
            lchkmc = .false.
         else if( buf(1:3) .eq. '-u ' ) then
            i = i+1
            call mygetarg(i,buf)
            zaxunitlab = buf(1:c1)
            call keyunits(zaxunit,buf,ierr)
            if (ierr .gt. 0) then
               write(6,'(2a)') 'ERROR : unknown unit: ',buf(1:16) 
               stop
            endif 
         else if( buf .eq. '-gnupl4 ') then
            lgnu4 = .true.                 ! gnuplot ver < 5
         else if( buf(1:3) .eq. '-h ' .or. buf(1:3) .eq. '-? ' ) then
            call plpot_usage
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
  200 continue
      if(.not.ld) dname=name
      dlaenge = slen(dname)
      if(dname(dlaenge:dlaenge).eq.'/') dlaenge=dlaenge-1
      if(name(laenge:laenge).eq.'/') laenge=laenge-1


!-----------------------------------------------------------------------
! if natpot has not been set, look in name directory.
! else take name from npotfile 
!-----------------------------------------------------------------------
      if(npotfile .eq. 'no') then
         npotfile = '  '
         plottask1=2
         lnpot=.false.
      elseif (npotfile .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         npotfile = name(1:laenge)//'/natpot'
         lnpot=.true.
         len=index(npotfile,' ')-1
         inquire(file=npotfile(1:len),exist=lnpot)
         if (.not. lnpot) then
            npotfile=' '
            plottask1=2
            lnpot=.false.
         endif
      else
         len=index(npotfile,' ')-1
         call abspath(npotfile,len)
         lnpot=.true.

         i=len
   10    continue
         if (npotfile(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=npotfile(1:laenge)
      endif

!-----------------------------------------------------------------------
! if vpotfile=no  then set lvpot=.false.
!-----------------------------------------------------------------------
      if (vpotfile .eq. 'no') then
         vpotfile=' '
         plottask1=1
         lvpot=.false.
         return
      endif

!-----------------------------------------------------------------------
! if vpotfile has not been set, try in name directory.
!-----------------------------------------------------------------------
      if (vpotfile .eq. ' ') then
         vpotfile = name(1:laenge)//'/vpot'
      else
         len=index(vpotfile,' ')-1
         call abspath(vpotfile,len)
      endif

      len=index(vpotfile,' ')-1
      inquire(file=vpotfile(1:len),exist=lvpot)
      if (.not. lvpot) vpotfile=' '

      if(.not.lvpot .and. .not.lnpot) then
         message='There is neither a natpot nor a vpot file.'
         routine='plpot_options (showpot.F)'
         call errormsg  
      endif

      return
      end subroutine plpot_options


!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     print usage message and stop
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine plpot_usage

      implicit none

      integer(long)       :: i
      
      write(6,'(78a1)')  ('-',i=1,78)
      write(6,'(a,/,a,/22x,a)') &
        'Purpose  : plots potential and/or potential fit interactively', &
        'Usage    : showpot [-i -d -D -v -n -u -vfit -vpot -diff -abs', &
                             '-evec -b -contr -ig -gnupl4 -ver -h -?]'
      write(6,'(/,a)') 'Options  :'
      write(6,'(a)')&
         ' -i NAME : The files are stored in the directory NAME' 
      write(6,'(a,/,11x,a)')&
         ' -d FILE : The dvr is read from file FILE/dvr',&
         'rather than from ./dvr or NAME/dvr'
      write(6,'(2a,/,11x,a/,11x,a/,11x,a)')&
         ' -D DIR  : The output is written to directory DIR.',&
         ' This is useful,',&
         'if one has no right to write to the name directory.',&
         'The directory DIR must exist.',&
         'The string DIR may be a relative or a full path-name.'
      write(6,'(a,/,11x,a,/,11x,a)')&
         ' -v FILE : The potential is read from file FILE',&
         'If FILE is not given, FILE=vpot is assumed.',&
         'If FILE=no, i.e -v no, vpot file is not read.'
      write(6,'(a,/,11x,a,/,11x,a)')&
         ' -n FILE : The potential fit is read from file FILE',&
         'If FILE is not given, FILE=natpot is assumed.',&
         'If FILE=no, i.e -n no, npot file is not read.'
      write(6,'(a)')&
         ' -u UNIT : The energy unit UNIT is applied.'
      write(6,'(a,/11x,a,/11x,a,/11x,a)') &
         ' -evec M : The eigenvectors (natural potentials)', &
         'of mode M are written to ASCII file evec.', &
         'M must not equal modc (contracted mode).', &
         'If M=99 then natpots of all modes (except modc) are written.'
      write(6,'(a,/11x,a)') &
         ' -b      : Eigenvectors (natural potentials) are additionally' &
         ,'written to binary file evecb, (if -evec is set).'
      write(6,'(a,/11x,a,/11x,a)') &
         ' -contr  : The contracted natural potentials of the', &
         'contracted mode is written to file contr.', &
         'WARNING: This may be a lot of data.'
      write(6,'(a)')&
         ' -vfit   : Set plottask to vfit.'
      write(6,'(a)')&
         ' -vpot   : Set plottask to vpot.'
      write(6,'(a)')&
         ' -diff   : Set plottask to vfit-vpot.'
      write(6,'(a)')&
         ' -abs    : Set plottask to abs(vfit-vpot).'
      write(6,'(a)')&
         ' -ig     : ignore certain errors.'
      write(6,'(a)')&
        ' -gnupl4  : Use gnuplot version < 5 for plotting '
      write(6,'(a)')&
         ' -ver    : Version information about the program '
      write(6,'(a,/a)') &
         ' -h      : Print this help text. ',&
         ' -?      : Print this help text. '
      write(6,'(78a1)')  ('-',i=1,78)

      stop
      end subroutine plpot_usage

!#######################################################################
!***********************************************************************
!
! calculates amount of memory to be allocated and the pointer structure
!
!***********************************************************************

      subroutine plpot_mem

      implicit none

      integer(long)                           :: ff,f,n,m,mm,wkdim
      integer(long), dimension(maxdim)        :: reorder

      maxpsub   = 0
      maxgdim   = 0
      ortdim    = 0
      evecdim   = 0
      modcdim   = 1
      vpotdim   = 1


!-----------------------------------------------------------------------
! set arrays vgmdim and vgpdim, which contains the `vor'-grid-dimensions
! in mode order and in dof order. (copy/paste from zeigpfit).
!-----------------------------------------------------------------------
      ff=0
! --- ff=mode order, f=dof order.
      do m=1,npmode
         do n=1,npotdof(m)
            f=potdof(n,m)
            ff=ff+1
            reorder(ff)=f
         enddo
      enddo

! --- set vgmdim, `vor'-grid-dimension in mode order.
      vgmdim(reorder(1))=1
      do ff=1,ndof-1
         vgmdim(reorder(ff+1))=vgmdim(reorder(ff))*gdim(reorder(ff))
      enddo

! --- set vgpdim, `vor'-grid-dimension in dof order.
      vgpdim(1)=1
      do f=1,ndof-1
         vgpdim(f+1)=vgpdim(f)*gdim(f)
      enddo

! --- Old versions
      if (filever(ivpot) .lt. 8.3013_dop) then
         do f=1,ndof
            vgpdim(f)=vgmdim(f)
         enddo
      endif 

!-----------------------------------------------------------------------
 
      do f=1,ndof
         maxgdim=max(maxgdim,gdim(f))
!         if(basis(f).eq.12) maxgdim=max(maxgdim,ipbaspar(3,f)) 
         zort(f)=ortdim+1
         if(basis(f).eq.12 .or. basis(f).eq.20.or. basis(f).eq.21) then
            ortdim=ortdim+ipbaspar(3,f)
         else
            ortdim=ortdim+gdim(f)
         endif
      enddo 
      oneddim=ortdim
 
      do m=1,nmode
         maxpsub  = max(maxpsub,psubdim(m))
         if (npotfile.ne.' ') then
            if (m.ne.modc) then
               zevec(m)=evecdim+1
               evecdim=evecdim+psubdim(m)*potdim(m)
               modcdim=modcdim*potdim(m)
            else
               zevec(m)=-1
            endif
         endif
         vpotdim=vpotdim*psubdim(m)
      enddo
      maxpsub2  = maxpsub**2
      write(ilog,*) 'vpotdim =',vpotdim

      mrort=mrdim
      mrdim=mrdim+ortdim
      if (npotfile.ne.' ') then
         mroned=mrdim
         mrdim=mrdim+oneddim
         mrevec=mrdim
         mrdim=mrdim+evecdim
         mrcoeff=mrdim
         coeffdim=modcdim*dimmodc
         mrdim=mrdim+coeffdim
      else
         mroned=1
         mrevec=1
         mrcoeff=1
      endif

      mrpldata=mrdim
      mrdim=mrpldata+maxpsub2

      mfvvec=mfdim
      if (vpotfile.ne.' ') mfdim=mfdim+vpotdim

!-----------------------------------------------------------------------
! assign work arrays
! NB work arrays must be at end of mc, mr, mi, ml arrays.
!-----------------------------------------------------------------------
      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      if (npotfile.ne.' ') then
         
!        stuff used for the calculation of the potential fit
         
         c2v(1)=modcdim
         wkdim=c2v(1)+maxpsub
         mm=0
         do m=1,nmode
            if (m.ne.modc) then
               mm=mm+1
               c2v(mm+1)=c2v(mm)/potdim(m)
               if (mm+1.eq.2) wkdim=wkdim+c2v(mm+1)
            endif
         enddo
         workrdim=max(workrdim,wkdim)
      endif
      if (vpotfile.ne.' ' .and. .not.lsp) then
         workrdim=max(workrdim,niobuf)
      elseif (vpotfile.ne.' ' .and. lsp) then
         workfdim=max(workfdim,niobuf)
      endif
      workrdim = workrdim + coeffdim + evecdim
      workdim2 = workrdim
      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

      return
      end subroutine plpot_mem
      

!***********************************************************************
!
!     HERE IS THE PLOT SUBROUTINE: sub_plpot
!
!***********************************************************************

      subroutine sub_plpot(ort,fvvec,oned,coeff,evec,workr,srf)

      implicit none

      real(dop), parameter :: huge = 1.0e+38_dop

      real(dop), dimension(ortdim), intent(out)          :: ort
      real(dop), dimension(oneddim), intent(out)         :: oned
      real(dop), dimension(coeffdim), intent(out)        :: coeff
      real(dop), dimension(evecdim), intent(out)         :: evec
      real(dop), dimension(workrdim), intent(out)        :: workr
      real(dop), dimension(maxgdim,maxgdim), intent(out) :: srf
      real(sip), dimension(vpotdim), intent(out)         :: fvvec
      
      integer(long), external           :: slen
      integer(long)                     :: chkdvr,zbackup,f,task,plotdim,&
                                           plottask,ydim,nsets,nplots,m,n
      integer(long), dimension(maxdim)  :: idum,idum1,idum2
      logical(kind=4)                   :: ldum,lopen
      real(dop), dimension(1)           :: rdum,rdum1,rdum2,rdum3
      complex(dop), dimension(1)        :: zdum,zdum1,zdum2,zdum3
      character(len=c5)                 :: line

!     rcut(i) : location of the grid point at which mode i is cut. 
!     icut(i) : number of the grid point at which mode i is cut. if mode i
!               is not cut, i.e. if it is associated with the x(y)-axis,
!               icut(i)=-1(-2).
!               If a minimum (maximum) is searched for icut(i)=-9(-8).
      
      integer(long)                          :: g1,g2,g3,f1,f2,f3,i,ilb1,iub1,ilb2,&
                                                iub2,ilb3,iub3,cutted,g,p,zcoeff
      integer(long), dimension(maxdim)       :: igpoint
      logical(kind=4)                        :: lsp2,lcspot2,differ,zero,lwrout
      real(dop)                              :: x,pesmin2,pesmax2,jtot2,jbf2,&
                                                csmass2,progver2
      character(len=c2)                      :: label2
      character(len=c5)                      :: opts2
      character(len=c5)                      :: filename

!-----------------------------------------------------------------------
! open file for plotting
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/tmpplot.pl'
      open(iaus,file=filename,status='unknown')

!-----------------------------------------------------------------------
! allocate arrays for plotting
!-----------------------------------------------------------------------
      if (lvpot) then
         nplottask=4
      else
         nplottask=1
      endif
      call alloc_plot

!-----------------------------------------------------------------------
! interactive plot defaults
!-----------------------------------------------------------------------
      call plotdefs
      lshowpot = .true.
      nplots=1
      ilb2=1
      ilb3=1

!-----------------------------------------------------------------------
! read data from dvr file
!-----------------------------------------------------------------------
      dvrdata(1)=.true.
      chkdvr=2
      call rddvr(ort,rdum,rdum1,rdum2,zdum,&
         zdum1,idum,zdum2,zdum3,idum1,idum2,rdum3,chkdvr)

!-----------------------------------------------------------------------
! load data from vpot file if desired
!-----------------------------------------------------------------------
      if (vpotfile.ne.' ') then
         call loadv2(ivpot,fvvec,workr)
      endif

!-----------------------------------------------------------------------
! load data and vpot parameters from natpot file if desired
!-----------------------------------------------------------------------
      if (npotfile.ne.' ') then
         call fitin(inpot,lonedflag,oned,evec,coeff)
         lwrout = .false.
         call readvpotpars(inpot,npotfile,progver2,label2,opts2,lsp2,&
                           ldum,idum(1),pesmin2,pesmax2,&
                           lcspot2,jtot2,jbf2,csmass2,lwrout)
         close(inpot)
         do f=1,ndof
            if (lonedflag(f)) then
               ploned=.true.
               onedipot(f)=1
            endif
         enddo
      endif

!-----------------------------------------------------------------------
! If option -evec or -contr is given, write the eigenvectors (natural
! potentials) of a given mode to file.
!-----------------------------------------------------------------------
      if(lcontr) then
         call wrcontr(coeff,potdim,psubdim(modc),modc,modcdim,nmode)
      endif

      if(mevec .gt. 0) then
         if(mevec .eq. 99) then
            do m=1,nmode
               if(m.ne.modc) then
                 f=spfdof(1,m)
                 f1=max(1,spfdof(2,m))
                 call wrevec(evec(zevec(m)),m,potdim(m),psubdim(m), &
                             ort(zort(f)),ort(zort(f1)),gdim(f), &
                             gdim(f1),nspfdof(m),f,f1,lbinary)
               endif
            enddo
         elseif(mevec .eq. modc) then
            write(6,'(a,i3,a,i3)') ' The evec-mode, mevec =', &
               mevec,', must not equal the contracted mode, modc =',modc
         elseif(mevec .gt. nmode) then
            write(6,'(a,i3,a,i3)') ' The evec-mode, mevec =', &
               mevec,', must not be greater than nmode =',nmode
         else
            f=spfdof(1,mevec)
            f1=max(1,spfdof(2,mevec))
            call wrevec(evec(zevec(mevec)),mevec,potdim(mevec),&
                        psubdim(mevec),ort(zort(f)),ort(zort(f1)),&
                        gdim(f),gdim(f1),nspfdof(mevec),f,f1,lbinary)
         endif
      endif
      if(mevec.gt.0 .or. lcontr)  stop

!-----------------------------------------------------------------------
! compare vpot parameters of vpot and natpot file if both files were
! loaded.
!-----------------------------------------------------------------------
      if (vpotfile.ne.' ' .and. npotfile.ne.' ') then
         differ=.false.
         if (label1.ne.label2 .and. label2.ne.'none') differ=.true.
         if (opts1.ne.opts2)             differ=.true.
         if (lsp1.neqv.lsp2)             differ=.true.
         if (lcspot1.neqv.lcspot2)       differ=.true.
         if (jtot1.ne.jtot2)             differ=.true.
         if (jbf1.ne.jbf2)               differ=.true.
         if (csmass1.ne.csmass2)         differ=.true.
         if (differ) then
            routine='sub_plpot'
            message='potential parameters from '//&
               vpotfile(1:slen(vpotfile))//' and '//&
               npotfile(1:slen(npotfile))//' differ!'
            call warnmsg
            message='compare the associated potfit input files in '//&
               logfile(1:slen(logfile))//'!'
            call warnmsg
            if(label1.ne.label2 .and. label2.ne.'none') &
               write(2,'(3a)') 'label : ', label1, label2 
            if(opts1.ne.opts2) write(2,'(2a,6x,a)')'opts :',opts1,opts2
            if(lsp1.neqv.lsp2) write(2,*) 'lsp : ',lsp1,lsp2
            if(lcspot1.neqv.lcspot2)&
               write(2,*) 'lcspot : ',lcspot1,lcspot2
            if(jtot1.ne.jtot2) write(2,'(a,2i5)') 'jtot :',jtot1,jtot2
            if(jbf1.ne.jbf2)   write(2,'(a,2i5)') 'jbf :',jbf1,jbf2
         endif
      endif

!-----------------------------------------------------------------------
! if only the npotfile has been loaded copy the potential parameters
!-----------------------------------------------------------------------
      if (vpotfile.eq.' ' .and. npotfile.ne.' ') then
         label1=label2
         opts1=opts2
         lsp=lsp2
         lsp1=lsp2
         pesmin=pesmin2
         pesmin1=pesmin2
         pesmax=pesmax2
         pesmax1=pesmax2
         lcspot1=lcspot2
         jtot1=jtot2
         jbf1=jbf2
         csmass1=csmass2
      endif

!-----------------------------------------------------------------------
! menu defaults
!-----------------------------------------------------------------------
      plotdim=2
      lplcoo2=.true.
      filename="tmpplot.pl"
      call initplot(filename,plotdim)

!-----------------------------------------------------------------------
! calculate initial mode bounds
!-----------------------------------------------------------------------
      call inmodebnds(ort,plotdim)

!-----------------------------------------------------------------------
! set plot tasks
!-----------------------------------------------------------------------
      if (lvpot) then
         cplottask(1)='Natural potential fit (vfit)'
         cplottask(2)='True potential (vpot)'
         cplottask(3)='vfit-vpot'
         cplottask(4)='abs(vfit-vpot)'
      else
         cplottask(1)='Natural potential fit (vfit)'
      endif
      do i=1,nplottask
         cptlen(i)=slen(cplottask(i))
      enddo

      plottask=1
      if(plottask1.gt.0) plottask=plottask1

! --- backup the coeffs and evecs in the last part oof the 
!     workr array     
      zbackup = workrdim - (coeffdim+evecdim) + 1
      call cpvxd(coeff(1),workr(zbackup),coeffdim)
      call cpvxd(evec(1),workr(coeffdim+zbackup),evecdim)


!-----------------------------------------------------------------------
! call menu and select task
!-----------------------------------------------------------------------
    1 continue
      call plotmenu(task,plottask,plotdim)

!-----------------------------------------------------------------------
! calculate surface if required
!-----------------------------------------------------------------------
      if (lrecalc) then
         lrecalc=.false.

         f1=axis(1)
         ilb1=ibound(1,f1)
         iub1=ibound(2,f1)
         f2=axis(2)
         if (f2 .gt. 0) then
            ilb2=ibound(1,f2)
            iub2=ibound(2,f2)
         endif
         f3=axis(3)
         if (f3 .gt. 0) then
            ilb3=ibound(1,f3)
            iub3=ibound(2,f3)
         endif
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (f .ne. f1 .and. f .ne. f2) then
                  igpoint(f)=icut(f)
               endif
            enddo
         enddo


! --- remove terms smaller than natpotcut
!        restore the original data
         call cpvxd(workr(zbackup),coeff(1),coeffdim)
         call cpvxd(workr(coeffdim+zbackup),evec(1),evecdim)
         cutted=0
         n=0
         zcoeff=0
         if(natpotcut .gt. 1.0e-14_dop) then
            do m=1,npmode
               do p=1,pdim(m)
                  n=n+1
                  zero=.true.
                  do g=1,psubdim(m)
                     if (m.eq.modc) then
                        if(abs(coeff(zcoeff+g)).gt.natpotcut)&
                             zero=.false.
                     else
                        if(abs(evec(zevec(m)+g)).gt.natpotcut) &
                             zero=.false.
                     endif
                  enddo
            
                  if (zero) then
                     cutted=cutted+1
                     do g=1,psubdim(m)
                        if (m.eq.modc) then
                           coeff(zcoeff+g)=0.0_dop
                        else
                           evec(zevec(m)+g)=0.0_dop
                        endif
                     enddo
                  endif
                  if (m.eq.modc) zcoeff=zcoeff+psubdim(m) 
               enddo               ! of p
            enddo                  ! of m

            
            write(6,*) '--------------------------------------------'
            write(6,'(a,1pe11.4,a,1pe11.4,a)') ' natpot-cut =',&
                          natpotcut,' au  = ',natpotcut*ev,' eV'
            write(6,'(i4,a,i5,a)') &
                          n-cutted,'/',n,' of natpot terms remain'
            write(6,*) '--------------------------------------------'
            write(ilog,*) '--------------------------------------------'
            write(ilog,'(a,1pe11.4,a,1pe11.4,a)') ' natpot-cut =',&
                          natpotcut,' au  = ',natpotcut*ev,' eV'
            write(ilog,'(i4,a,i5,a)') &
                          n-cutted,'/',n,' of natpot terms remain'
            write(ilog,*) '--------------------------------------------'
            call myflush(ilog)
         endif


         if (plotdim.eq.1) then
            do g1=ilb1,iub1
               igpoint(f1)=g1
               if(f3.gt.0) then 
                  if (icut(f3) .eq. -9 ) then ! Search for min
                     srf(g1,1)=huge
                     do g3=ilb3,iub3
                        igpoint(f3)=g3
                        call calcdatum(plottask,igpoint,fvvec,&
                             oned,coeff,evec,workr,x)
                        srf(g1,1)=min(srf(g1,1),x)
                     enddo
                  elseif (icut(f3) .eq. -8 ) then ! Search for max
                     srf(g1,1)=-huge
                     do g3=ilb3,iub3
                        igpoint(f3)=g3
                        call calcdatum(plottask,igpoint,fvvec,&
                             oned,coeff,evec,workr,x)
                        srf(g1,1)=max(srf(g1,1),x)
                     enddo
                  else
                     call calcdatum(plottask,igpoint,fvvec,oned,&
                          coeff,evec,workr,x)
                     srf(g1,1)=x
                  endif
               else
                  call calcdatum(plottask,igpoint,fvvec,oned,&
                       coeff,evec,workr,x)
                  srf(g1,1)=x
               endif
            enddo
         elseif (plotdim.eq.2) then
            do g1=ilb1,iub1
               igpoint(f1)=g1
               do g2=ilb2,iub2
                  igpoint(f2)=g2
                  if(f3.gt.0) then 
                     if (icut(f3) .eq. -9 ) then ! Search for min
                        srf(g1,g2)=huge
                        do g3=ilb3,iub3
                           igpoint(f3)=g3
                           call calcdatum(plottask,igpoint,fvvec,&
                                oned,coeff,evec,workr,x)
                           srf(g1,g2)=min(srf(g1,g2),x)
                        enddo
                     elseif (icut(f3) .eq. -8 ) then ! Search for max
                        srf(g1,g2)=-huge
                        do g3=ilb3,iub3
                           igpoint(f3)=g3
                           call calcdatum(plottask,igpoint,fvvec,&
                                oned,coeff,evec,workr,x)
                           srf(g1,g2)=max(srf(g1,g2),x)
                        enddo
                     else
                        call calcdatum(plottask,igpoint,fvvec,oned,&
                             coeff,evec,workr,x)
                        srf(g1,g2)=x
                     endif
                  else
                     call calcdatum(plottask,igpoint,fvvec,oned,&
                          coeff,evec,workr,x)
                     srf(g1,g2)=x
                  endif
               enddo   !  g2
            enddo      !  g1
         endif
      endif

!-----------------------------------------------------------------------
! get data bounds
!-----------------------------------------------------------------------
      if (lrebound) then
         f1=axis(1)
         f2=axis(2)
         if (f2 .eq. 0) then
            ydim=1
         else
            ydim=gdim(f2)
         endif
         nsets=1
         nplots=1
         call databnds(srf,gdim(f1),ydim,nsets,nplots,maxgdim,&
                 maxgdim,1,1)
      endif

!-----------------------------------------------------------------------
! write title and axis labels
!-----------------------------------------------------------------------
      i=index(label1,'{')-1
      if (i.lt.0) then
         line=label1(1:slen(label1))
         if(slen(label1).eq.0  .or. ichar(label1(1:1)).eq.0 )&
           line='unknown'
      else
         line=label1(1:i)
      endif
      if (slen(opts1).gt.0)&
         line=line(1:slen(line))//' {'//opts1(1:slen(opts1))//'}'
      line=line(1:slen(line))//' / '//&
           cplottask(plottask)(1:cptlen(plottask))//', '//&
           zaxunitlab(1:slen(zaxunitlab))
      write(plttitle,'(a,2a2)') line(1:slen(line)),' \\','n '

      do m=1,nmode
        do n=1,npotdof(m) 
         f=spfdof(n,m)
        if (icut(f).eq.-9) then
            write(line,'(a)')&
                 modelabel(f)(1:slen(modelabel(f)))//'=min'
            call rmblnk(line(1:slen(line)))
            plttitle=&
                 plttitle(1:slen(plttitle))//'  '//line(1:slen(line))

         elseif (icut(f).eq.-8) then
            write(line,'(a)')&
                 modelabel(f)(1:slen(modelabel(f)))//'=max'
            call rmblnk(line(1:slen(line)))
            plttitle=&
                 plttitle(1:slen(plttitle))//'  '//line(1:slen(line))

         elseif (icut(f).gt.0 ) then
            write(line,'(a,e11.4)')&
            modelabel(f)(1:slen(modelabel(f)))//'=',rcut(f)/axunit(f)

            call rmblnk(line(1:slen(line)))
            plttitle=&
                 plttitle(1:slen(plttitle))//'  '//line(1:slen(line))
         endif
        enddo
      enddo

      f1=axis(1)
      f2=axis(2)
      axislab(1)=modelabel(f1)(1:slen(modelabel(f1)))//&
         ' ['//axunitlab(f1)(1:slen(axunitlab(f1)))//'] '
      if (f2 .gt. 0) then
         axislab(2)=modelabel(f2)(1:slen(modelabel(f2)))//&
            ' ['//axunitlab(f2)(1:slen(axunitlab(f2)))//'] '
      else
         axislab(2)=' '
      endif
      axislab(3)='V ['//zaxunitlab(1:slen(zaxunitlab))//'] '

!-----------------------------------------------------------------------
! do task
! TASK=0 STOP  else return to menu
!-----------------------------------------------------------------------
      if (task .gt. 0) then
         f1=axis(1)
         f2=axis(2)
         if (f2 .eq. 0) f2=1
         nsets=1
         nplots=1
         call dotask(task,plottask,plotdim,ort(zort(f1)),&
              ort(zort(f2)),srf,gdim(f1),gdim(f2),nsets,nplots,&
              maxgdim,maxgdim,1,1,ort,ortdim,rdum,1,rdum,1,&
              rdum,1,1,1,1,idum)
         goto 1
      endif

!-----------------------------------------------------------------------
! Finish off
!-----------------------------------------------------------------------
      call dealloc_plot
      inquire(unit=iaus,opened=lopen)
      if (lopen) close(iaus,status='delete')

      return
      end subroutine sub_plpot

!#######################################################################
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     calculates a single datum `x' associated with grid point `igpoint'
!     in accordance with the chosen plottask
!
!     igpoint (i)
!     fvvec (i)
!     oned (i)
!     coeff (i)
!     evec (i)
!     workr (w)
!     x (o)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine calcdatum(plottask,igpoint,fvvec,oned,coeff,evec,&
                 workr,x)

      implicit none

      integer(long), intent(in)                   :: plottask
      integer(long), dimension(ndof), intent(out) :: igpoint
      real(sip), dimension(vpotdim), intent(in)   :: fvvec
      real(dop)                                   :: y,v1d
      real(dop), intent(out)                      :: x
      real(dop), dimension(coeffdim), intent(in)  :: coeff
      real(dop), dimension(evecdim), intent(in)   :: evec
      real(dop), dimension(workrdim), intent(out) :: workr
      real(dop), dimension(oneddim), intent(in)   :: oned

      if (plottask.eq.2) then   ! Vpot
         call getfv_element(igpoint,fvvec,x)
         call vpot_bounds(x,pesmin1,pesmax1)
      else if (plottask.eq.1) then ! Natpot
         call getvfit(igpoint,coeff,evec,workr,x)
         call getv1d(igpoint,oned,v1d)
         x=x+v1d
      else if (plottask.eq.3) then ! Natpot - Vpot
         call getvfit(igpoint,coeff,evec,workr,x)
         call getv1d(igpoint,oned,v1d)
         call getfv_element(igpoint,fvvec,y)
         call vpot_bounds(y,pesmin1,pesmax1)
         x=x+v1d-y
      else if (plottask.eq.4) then ! abs(Natpot-Vpot)
         call getvfit(igpoint,coeff,evec,workr,x)
         call getv1d(igpoint,oned,v1d)
         call getfv_element(igpoint,fvvec,y)
         call vpot_bounds(y,pesmin1,pesmax1)
         x=abs(x+v1d-y)
      else
         routine='calcdatum'
         message='unknown plot task encountered'
         call errormsg
      endif

      return
      end subroutine calcdatum

!#######################################################################
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     cuts down v to the interval [pesmin,pesmax].
!
!     v (i/o)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine vpot_bounds(v,pesmin,pesmax)

      implicit none

      real(dop), intent(in)  :: pesmin,pesmax
      real(dop), intent(out) :: v

      if(v.lt.pesmin) then
         v=pesmin
      else if (v.gt.pesmax) then
         v=pesmax
      endif

      return
      end subroutine vpot_bounds

!#######################################################################
!    SUBROUTINE WREVEC
!    This routine writes the eigenvectors (natural potentials) to
!    an ASCII file
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine wrevec(evec,m,pdim,subdim,ort,ort1,gdim,gdim1,nspfdof, &
                 f,f1,lbinary)

use global
use channels

      implicit none

      integer(long), intent(in)                   :: m,pdim,gdim
      integer(long), intent(in)                   :: subdim,gdim1,nspfdof,f,f1
      integer(long)                               :: g,g1,g2,n
      real(dop), dimension(gdim,pdim), intent(in) :: evec
      real(dop), dimension(gdim), intent(in)      :: ort
      real(dop), dimension(gdim1), intent(in)     :: ort1
      character(len=240)                          :: filename,filenameb
      character(len=8)                            :: cdof
      logical(kind=4)                             :: lbinary

      write(filename,'(2a,i2.2)') name(1:laenge),'/evec',m
      open(ieigvec,file=filename,form='formatted',status='unknown',err=999)

      if(lbinary) then
         write(filenameb,'(2a,i2.2)') name(1:laenge),'/evecb',m
         open(ioutbin,file=filenameb,form='unformatted',status='unknown',err=998)
      endif
      if (nspfdof.eq.1) then
         write(cdof,'(i3)') f
      else
         write(cdof,'(i3,",",i3)') f, f1
      endif

      write(ieigvec,'(a,i3,2a,/2a,/a,/4a,/4a,/a,/a,/3a,/)')             &
       '# potfit eigenvectors (natural potentials) of mode =',m,        &
       ', DOF =',cdof,                                                  &
       '# Columns: grid-point, (coordinate(s)) 1st vector,',            &
       ' 2nd vector, ...',                                              &
       '# For uncombined modes plotting may be performed as follows:',  &
       '# Use "plgen ',filename(laenge+2:laenge+7),' 1:3 - 1:4 - 1:5"', &
       ' to plot the first three vectors vs grid-points.',              &
       '# Use "plgen ',filename(laenge+2:laenge+7),' 2:3 - 2:4 - 2:5"', &
       ' to plot the first three vectors vs coordinate.',               &
       '# For 2D combined modes both coordinates are written.',         &
       '# For combinations higher than 2D no coordinates are written.', &
       '# Inspect this file with "less -S ',                            &
        filename(laenge+2:laenge+7),'".'

      if(nspfdof.eq.1) then
         write(ieigvec,'(a,i6,99i10)') '#   g  coordinate',(n,n=1,pdim)
         do g=1,subdim
            write(ieigvec,'(i5,f12.6,99f10.6)') g, ort(g), (evec(g,n),n=1,pdim)
         enddo
         write(ieigvec,'(a,i6,99i10)') '#   g  coordinate',(n,n=1,pdim)
      elseif(nspfdof.eq.2) then
         write(ieigvec,'(a,i6,99i10)') &
                '#   g  coordinate  coordinate', (n,n=1,pdim)
         do g=1,subdim
            g2=(g-1)/gdim
            g1=g-g2*gdim
            g2=g2+1
            write(ieigvec,'(i5,2f12.6,99f10.6)') &
              g, ort(g1), ort1(g2), (evec(g,n),n=1,pdim)
         enddo
         write(ieigvec,'(a,i6,99i10)') &
                '#   g  coordinate  coordinate', (n,n=1,pdim)

      else
         write(ieigvec,'(a,i6,99i10)') '#   g  ',(n,n=1,pdim)
         do g=1,subdim
            write(ieigvec,'(i5,999f10.6)') g, (evec(g,n),n=1,pdim)
         enddo
         write(ieigvec,'(a,i6,99i10)') '#   g  ',(n,n=1,pdim)
      endif

      write(6,'(2a,i3,a,/a)') 'Eigenvectors (natural potentials)', &
       ' of mode',m,' are written to file ',filename(1:laenge+7)

      close(ieigvec)

! Printing evecb files (binary evec files)

      if(lbinary) then
        do g=1,subdim
          write(ioutbin) (evec(g,n),n=1,pdim)
        enddo
        close(ioutbin)
      end if

      return
 998  write(6,'(/2a)') 'Cannot open evecb-file: ',filenameb(1:laenge+7)
 999  write(6,'(/2a)') 'Cannot open evec-file: ',filename(1:laenge+7)

      stop
      end subroutine wrevec

!#######################################################################
!    SUBROUTINE WRCONTR
!    This routine writes the contracted mode natpots to an ASCII file
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine wrcontr(coeff,potdim,csubdim,modc,modcdim,nmode)

use global
use channels

      implicit none

      integer(long), intent(in) :: csubdim,modc,modcdim,nmode
      integer(long)             :: i,m,g
      integer(long), dimension(16)                      :: indx
      integer(long), dimension(nmode), intent(in)       :: potdim
      real(dop), dimension(csubdim,modcdim), intent(in) :: coeff
      real(dop)                                         :: rms,cmax
      character(len=240)                                :: filename

      write(filename,'(2a)') name(1:laenge),'/contracted'
      open(ieigvec,file=filename,form='formatted',status='unknown',err=999)

      write(ieigvec,'(a,i3)') &
          '# potfit contracted natpots of contraced mode.  modc =',modc

      do m=1,nmode
         indx(m) = 1
      enddo
      indx(modc)=0

      do i=1,modcdim
         rms=0.d0
         cmax=0.d0
         do g=1,csubdim
            rms=rms+coeff(g,i)**2
            cmax=max(cmax,abs(coeff(g,i)))
         enddo
         rms=sqrt(rms/dble(csubdim))
         write(ieigvec,'(/a,i3,a,1pe12.5,a,1pe12.5,a,16i3)') &
             '# I =',i,',  rms =',rms,',  max =',cmax, &
             ', indx:',(indx(m),m=1,nmode)
         write(ieigvec,'(1p5e16.7)') (coeff(g,i),g=1,csubdim)


! --- determine new set of indices
         do m=1,nmode
            if(m.ne.modc) then
               if (indx(m).lt.potdim(m)) then
                  indx(m)=indx(m)+1
                  goto 1
               else
                  indx(m)=1
               endif
            endif
         enddo
 1    continue
      enddo

      write(6,'(2a,i3,a,/a)') 'Contracted natural potentials', &
       ' of mode modc =',modc,' are written to file ', &
        filename(1:laenge+11)

      close(ieigvec)

      return

 999  write(6,'(/2a)') 'Cannot open file: ',filename(1:laenge+11)
      stop
      end subroutine wrcontr

!#######################################################################

      subroutine loadv2(unit,fvvec,buffer)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     Like loadv but loads the real*8 data storred in the vpot file
!     into a real*4 potential vector fvvec. This is for showpot.
!     It is assumed that the file vpot has been opened for reading
!     via channel unit. The data on file is double precision, but
!     the vector fvvec is single precision.
!     This routine is called from showpot.
!
!     unit (i)
!     fvvec (o)
!     buffer (w)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

use global
use pglobal
use griddatmod
use operdef
use runpfitmod
use combined
use channels

use logdat, only: errormsg
use directutil, only: indexmap

      implicit none

      real(sip), dimension(vpotdim), intent(out) :: fvvec
      real(dop), dimension(niobuf), intent(out)  :: buffer
      integer(long), intent(in)                  :: unit
      integer(long)                              :: nitems,niobuf1
      integer(long)                              :: ierr,i,m1,m2,j1,j,jj
      integer(long)                              :: ilbl,jlbl
      character(len=c2)                          :: rdtag

!     load from disk (nitems=vpotdim)

!     call skipvpotpars(ivpot)

! find data. File versions older than 11.0 have been left in correct place
! (with a bit of luck!)
      if (filever(unit) .gt. 11.0) then
         rewind(unit)
         do
  20        continue
            read(unit,iostat=chkerr(1),err=20) rdtag
            if (rdtag(1:1) .ne. '<') cycle

            ilbl=2
            jlbl=index(rdtag,'>')-1

            if (rdtag(ilbl:jlbl) .eq. 'vpot pes') then
               exit
            endif
         enddo
      endif

      read(unit,iostat=ierr) nitems,niobuf1
      if (ierr.ne.0) then
         routine='loadv2'
         message='cannot read nitems from '//vname(1:vlength)
         call errormsg
      endif

! --- check consistency
      if (niobuf1.gt.niobuf) then
         routine='loadv2'
         message='niobuf is too small!'
         call errormsg
      endif

      m1=nitems/niobuf1
      m2=nitems-m1*niobuf1
      j1=0
!.....Read the buffer m1 times (m1 may be zero).
      do i=1,m1
         read(unit,iostat=ierr) (buffer(j),j=1,niobuf1)
         if (ierr.ne.0) then
            write(6,'(a)') ' ERROR while reading vpot-file'
            if( progname(1:6).eq."potfit" ) write(6,'(a)') &
            ' Use option -gv to enforce creating a new vpot-file'
            routine='loadv2'
            message='error on reading '//vname(1:vlength)
            call errormsg
         endif
         do j=1,niobuf1
            call indexmap(j+j1,jj)
            fvvec(jj) = real(buffer(j))
         enddo
         j1=j1+niobuf1
      enddo

!.....Read the rest of the data, which only partly fills the buffer.
      if (m2.ne.0) then
         read(unit,iostat=ierr) (buffer(j),j=1,m2)
         if (ierr.ne.0) then
            write(6,'(a)') ' ERROR while reading vpot-file'
            if( progname(1:6).eq."potfit" ) write(6,'(a)') &
            ' Use option -gv to enforce creating a new vpot-file'
            routine='loadv2'
            message='error on reading '//vname(1:vlength)
            call errormsg
         endif
         do j=1,m2
            call indexmap(j+j1,jj)
            fvvec(jj) = real(buffer(j))
         enddo
      endif

!     inspect fvvec

      call fvprop(fvvec,nitems)

!     handle cut of potential

      call fvcut(fvvec,nitems)

      return
      end subroutine loadv2

!#######################################################################

      subroutine fvcut(fvvec,nitems)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     like vcut but with single precision potential vector
!
!     fvvec (i)
!     nitems (i)
!        logical length of fvvec
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

use global
use pglobal
use griddatmod
use operdef
use runpfitmod

      implicit none

      integer(long), intent(in) :: nitems
      integer(long) :: j
      real(dop)     :: v
      real(sip), dimension(nitems), intent(out) :: fvvec

      if (lcutpes) then
         do j=1,nitems
            v=fvvec(j)
            if (v.gt.pesmax) then
               v=pesmax
            elseif (v.lt.pesmin) then
               v=pesmin
            endif
            fvvec(j)=real(v)
         enddo

!---------------------
! softvcut
!---------------------
      elseif (lsoftcutpes) then
         do j=1,nitems
            v=fvvec(j)
            if (v.gt.pesmax*(1.d0+log(abs(v/pesmax)))) then
               v=pesmax*(1.d0+log(abs(v/pesmax)))
            elseif (v.lt.pesmin) then
               v=pesmin
            endif
            fvvec(j)=real(v)
         enddo

      endif

      return
      end subroutine fvcut

!#######################################################################

      subroutine getfv_element(imgpoint,fvvec,v)
!***********************************************************************
!
!     like getv_element but with single precision potential
!     used in showpot.
!
!     igpoint (i)
!     fvvec (i)
!     v (o)
!
!***********************************************************************

use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined

      implicit none

      integer(long)                              :: f,m,n,offset
      integer(long), dimension(ndof), intent(in) :: imgpoint
      real(dop), intent(out)                     :: v
      real(sip), dimension(vpotdim), intent(in)  :: fvvec

      offset=0
      do m=npmode,1,-1
        do n=npotdof(m),1,-1
          f=potdof(n,m)
          offset=gdim(f)*offset+imgpoint(f)-1
        enddo
      enddo
      offset=offset+1
      v=fvvec(offset)

      return
      end subroutine getfv_element

!#######################################################################

      subroutine fvprop(fvvec,n)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     like vprop but for a real*4 potential vector
!     This routine is called from loadv2 which in turn is called by
!     showpot
!
!     fvvec (i)
!        real*4 potential vector
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

use global
use griddatmod
use operdef
use runpfitmod
use combined
use channels

use lalib,   only: vrminmax

      implicit none

      integer(long), intent(in) :: n
      real(sip), dimension(n)   :: fvvec
      real(sip)                 :: fvmin,fvmax

      logical(kind=4), save :: firstcall=.true.

!-----------------------------------------------------------------------
!     determine minimum and maximum of the potential vector
!-----------------------------------------------------------------------
      if (firstcall) then
         firstcall=.false.
         call vrminmax(fvvec,n,fvmin,fvmax)
         vmin=fvmin
         vmax=fvmax
         write(iout,'(a,e16.8," au, ",e16.8," ev.")') &
               'potential minimum : ',vmin, vmin*ev
         write(iout,'(a,e16.8," au, ",e16.8," ev.")') &
               'potential maximum : ',vmax, vmax*ev
      endif

      return
      end subroutine fvprop

!#######################################################################

      end program showpot

