!***********************************************************************
!
!                         ANALYSE PROGRAM CONVPSI
!
! Reads the "psi" file generated in a multi-state multi-packet run
! and writes the desired packet to a separate file.
!
! V8.0  2/99  MB
!
!
!***********************************************************************

      program convpsi

use decimal, only: long,dop,sip
use maxv, only: maxdim,maxsta
use convpsimod
use psidef

use op1lib, only: cpmxi, cpvxi
use iofile, only: rdpsiinfo

      implicit none

      integer(long), external              :: myiargc
      real(dop)                            :: prgver
      character(len=160)                   :: dum

complex(dop), dimension(:), allocatable    :: mc
real(dop), dimension(:), allocatable       :: mr
integer(long), dimension(:), allocatable   :: mi
logical(kind=4), dimension(:), allocatable :: ml
complex(sip), dimension(:), allocatable    :: ms
real(sip), dimension(:), allocatable       :: mf


      chkgrd = 1
!-----------------------------------------------------------------------
! Get command line arguments
!-----------------------------------------------------------------------
      call mygetarg(1,dir)
      if (dir .eq. '-ver') then
         call wrversion(6)
         stop
      elseif ( dir .eq. '-h'  .or.  myiargc() .ne. 2) then
         write(6,*)
         write(6,'(2a)') &
            'Purpose: Extracts single packets from multi-state ', &
            'multi-packet wavefunctions.'
         write(6,'(a)') 'Usage:   convpsi [-ver|-h] <path> <packet>'
         write(6,'(a)')&
            'path:    path to directory containing "psi" file'
         write(6,'(a)') 'packet:  packet to be extracted (0 = all)'
         write(6,*)
         stop
      else
         lendir=index(dir,' ')-1
         call mygetarg(2,dum)
         read(dum,*) packet
      endif

!-----------------------------------------------------------------------
! Open psi file to get required memory
!-----------------------------------------------------------------------
      open(ipsi,file=dir(1:lendir)//'/psi',form='unformatted',&
           status='old',err=666)
      call rdmemdim(ipsi)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_psidef
      close(ipsi)

!-----------------------------------------------------------------------
! Allocate memory
!-----------------------------------------------------------------------
if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(dgldim))
allocate(mr(1))
allocate(mi(1))
allocate(ml(1))
allocate(ms(1))
allocate(mf(1))

call receiver(mc,mr,mi,ml,ms,mf)

deallocate(mc,mr,mi,ml,ms,mf)

      stop
      
 666  continue
      write(6,'(a)') 'Error in CONVPSI: Cannot open or close file.'
      stop
      contains


! **********************************************************************
! *
! *                        subroutine receiver
! *
! * Is called after memory has been allocated. 
! *
! **********************************************************************

      subroutine receiver (psi,dum1,dum2,dum3,dum4,dum5)

use global
use maxv, only: maxdim,maxsta
use griddatmod
use convpsimod
use psidef

use iofile, only: wrpsiinfo

      implicit none

      complex(sip), dimension(1), intent(in)       :: dum4
      real(dop), dimension(1), intent(in)          :: dum1
      real(sip), dimension(1), intent(in)          :: dum5
      integer(long), dimension(1), intent(in)      :: dum2
      logical(kind=4), dimension(1), intent(in)    :: dum3

      complex(dop), dimension(dgldim), intent(inout) :: psi
      real(dop)                                    :: prgver
      integer(long)                                :: npacket1,nstate1,npes,pos,p,s,m,b,d,f
      integer(long), dimension(maxsta)             :: block1
      integer(long),dimension(maxdim,maxsta)       :: dim1
      character(len=c5)                            :: outname

!-----------------------------------------------------------------------
! Open psi file
!-----------------------------------------------------------------------
      open(psiin,file=dir(1:lendir)//'/psi',form='unformatted',&
           status='old',err=666)
      read(psiin) prgver
      call rdpsiinfo(psiin,chkdvr,chkgrd,chkpsi,chkdat)
      if ((packet .lt. 0) .or. (packet .gt. npacket)) goto 555

!-----------------------------------------------------------------------
! Save some data
!-----------------------------------------------------------------------
      npes = nstate/npacket
      nstate1  = nstate
      npacket1 = npacket
      call cpmxi(dim,dim1,maxdim,maxsta)
      call cpvxi(block,block1,maxsta)
      
!-----------------------------------------------------------------------
! Loop over all packets
!-----------------------------------------------------------------------
      do p = 1,npacket1
         if ((p .eq. packet) .or. (packet .eq. 0)) then

!-----------------------------------------------------------------------
! Rewind input file and open output file
!-----------------------------------------------------------------------
            rewind(psiin)
            read(psiin) prgver
            call rdpsiinfo(psiin,chkdvr,chkgrd,chkpsi,chkdat)
            outname=dir(1:lendir)//'/psi'
            write(outname(lendir+5:lendir+6),'(i2.2)') p
            open(psiout,file=outname,form='unformatted',&
                 status='unknown',err=666)

!-----------------------------------------------------------------------
! Transform some multi-packet quantities to single packet form
!-----------------------------------------------------------------------
            npacket = 1
            nstate = nstate1/npacket1
            do m = 1,nmode
               do s = npes*(p-1)+1,npes*p
                  dim(m,s-npes*(p-1)) = dim1(m,s)
               enddo
            enddo
            adim = 0
            do s = npes*(p-1)+1,npes*p
               block(s-npes*(p-1)) = block1(s)
               adim = adim+block1(s)
            enddo
            do f = 1,ndof
               if (basis(f) .eq. 0) then
                  gdim(f) = nstate
               endif
            enddo
            if (nstate1 .eq. npacket1) then
               ndof  = ndof-1
               lmult = .false.
            endif
            lmulpack = .false.
            write(psiout) prgver
            call wrpsiinfo(psiout)
    
!-----------------------------------------------------------------------
! Read data from psi file
!-----------------------------------------------------------------------
 100        continue
            read(psiin,end=200) (psi(d),d=1,dgldim)

!-----------------------------------------------------------------------
! Move coefficients of desired packet to top of psi vector
!-----------------------------------------------------------------------
            pos = 1
            do s = npes*(p-1)+1,npes*p
               do b = 0,block1(s)-1
                  psi(pos+b) = psi(zpsi(s)+b)
               enddo
               pos = pos+block1(s)
            enddo
         
!-----------------------------------------------------------------------
! Move spfs of desired packet below coefficients
!-----------------------------------------------------------------------
            do m = 1,nmode
               do s = npes*(p-1)+1,npes*p
                  do d = 0,dim1(m,s)*subdim(m)-1
                     psi(pos+d) = psi(zetf(m,s)+d)
                  enddo
                  pos = pos+dim1(m,s)*subdim(m)
               enddo
            enddo
            pos = pos-1

!-----------------------------------------------------------------------
! Write desired packet to file and continue reading wavefunction
!-----------------------------------------------------------------------
            write(psiout) (psi(d), d = 1,pos)
            call myflush(psiout)
            goto 100

!-----------------------------------------------------------------------
! Close output file
!-----------------------------------------------------------------------
 200        continue
            close(psiout,err=666)
         endif
      enddo

      close(psiin,err=666)
      return

!-----------------------------------------------------------------------
! Write error message
!-----------------------------------------------------------------------
 555  continue
      write(6,'(a)') 'Error in CONVPSI: Illegal packet specified.'
      stop

 666  continue
      write(6,'(a)') 'Error in CONVPSI: Cannot open or close file.'
      stop

      end subroutine receiver

      end program convpsi


