!***********************************************************************
!
!                         AUSWERT ROUTINE FMAT
!
!     This routine calculates the overlap matrix <psi(t)|psi(t')> 
!     and the Hamiltonian matrix <psi(t)|H|psi(t')>.
!     This data is used for a filter calculation to obtain 
!     eigenenergies.
!
!     Contents: fmat, receiver, foperin, fmt, fmatinput, rdoption  
!
! HDM Sep 01
!
!
!***********************************************************************

      program fmat

use decimal, only: long,dop,sip
use global
use aglobal
use maxv, only: maxdim,maxsta,maxkoe
use griddatmod
use psidef
use channels
use hpsimod
use datenmod
use operdef
use overlapmod
use compdatmod
use compdat1mod
use fluxmod

use op1lib, only: zerovxz, cpvxz, zerovxl
use xvlib, only: vvaxzz
use logdat, only: errormsg
use compdat, only: savecsys,savecpsi
use auswutil, only: savecdat
use zeigauswmod, only: zeigausw
use memhopsmod, only: memhops
use zeighpsimod, only: operflag,zeighpsi

use module_interface


      implicit none

      integer(long), external           :: slen
      integer(long)                     :: zeig,m,s,t1,t2,n,ilbl,lmtt
      real(dop)                         :: tcpu,tsys,ttot
      real(dop), external               :: fcputime,fsystime
      logical(kind=4)                   :: lexist, lsym
      character(len=100)                :: buffer
      character(len=80)                 :: mttfile
      character(len=2)                  :: pflag

complex(dop), dimension(:), allocatable    :: mc
real(dop), dimension(:), allocatable       :: mr
integer(long), dimension(:), allocatable   :: mi
logical(kind=4), dimension(:), allocatable :: ml
complex(sip), dimension(:), allocatable    :: ms
real(sip), dimension(:), allocatable       :: mf


!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      call fmatinput(lsym)

!-----------------------------------------------------------------------
! to allocate memory for psi and psi1
! 1. set no. of comparison data sets for 2 wavefunctions
! 2. save information in comparison data sets
!-----------------------------------------------------------------------
      ncomp=2
      call savecsys(1)
      call savecpsi(1)
      call savecdat(1)
      call savecsys(2)
      call savecpsi(2)
      call savecdat(2)


!-----------------------------------------------------------------------
! Get operator number
!-----------------------------------------------------------------------
      if (loper) then
 20      nham=0
         do n=1,hamdim
            if (hamlab(n) .eq. operator) nham=n
         enddo

         if (nham .eq. 0) then
            write(6,'(4a)')'Operator "',&
                 operator(1:slen(operator)),'" not found in ',&
                 operfile(1:slen(operfile))
            write(6,'(1x,a)') 'Name of operator ? Choose from:'
            do n=1,hamdim
               write(6,'(2x,a)') hamlab(n)
            enddo
            read(5,'(a)') operator
            if (operator .eq. ' ') stop
            goto 20
         endif
         if(kzahl(nham) .gt. 0  .or. nmulpot(nham) .gt. 0 ) then
            lcorr = .true.
            write(6,'(3a,i3)') ' Using correlated operator : ',&
                  operator(1:slen(operator)),' ,  nham =', nham
            if(.not.nodiag(nham)) then
               ilbl=index(hamlab(nham),' ')-1
               message='The Hamiltonian '//hamlab(nham)(1:ilbl)//&
                    ' must be of "nodiag" type.'  
               call errormsg
            endif
         else
            lcorr = .false.
            write(6,'(3a,i3)') ' Using uncorrelated operator : ',&
                  operator(1:slen(operator)),' ,  nham =', nham
         endif
         write(ilog,'(a,i3,3a,i4,i2,2x,a,9i3)') 'nham =', nham,&
            ',  operator = ',operator(1:slen(operator)),&
              '  kzahl:',kzahl(nham), nmulpot(nham),'khzahl:',&
             (khzahl(n,1,nham),n=1,nmode)
      endif


      if(laeout .gt. 0) then
         if (lsym) then
            pflag='_s'
         else
            pflag='_h'
         endif
         mttfile=trim(filein1)
         lmtt=laein1
         buffer=fileout(1:laeout)//'/'//mttfile(1:lmtt)//pflag
         if(loper) buffer =&
                       fileout(1:laeout)//'/'//mttfile(1:lmtt)//'_'//&
                       operator(1:slen(operator))//pflag

         ilbl=index(buffer,' ')-1
         inquire(file=buffer(1:ilbl),exist=lexist)
         if(lexist .and. .not.lovwr) then
            write(6,'(/,a,/,2a,/a,/a)') &
            '#########################################################',&
            ' The output file: ',buffer(1:ilbl),' already exists!',&
            ' overwrite disabled? (Use -w option to enable overwrite)'
            write(6,'(2a,/a)') ' Cannot open file: ',buffer(1:ilbl),&
            '#########################################################'
            stop
         endif
         open (unit=iwtt,file=buffer(1:ilbl),form='formatted')
      end if


!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
! ncomp  : no. of comparison data sets to be used.
!          See auswutil.F for details of use of comparison sets
! lpsi   : wavefunction
! lpsi1  : second wavefunction
! lpsigrd : wavefunction to be used in grid representation (rather
!           than spf)
! ldmat  : density matrices for psi
! ldmat1 : density matrices for psi1
! lort   : grid coordinates
! lgpop  : grid populations
! ltrafo : FBR/DVR trafo matrices
! lsphfbr : jfbr, mfbr and kinfbr matrices.
!-----------------------------------------------------------------------
      lrddvr  = .true.
      lrdoper = .true.
      ovtype  = 0
      lpsi    = .true.
      lpsi1   = .true.
      lpsigrd = ovtype.eq.1
      lort    = .true.
      ltrafo  = .false.
      lsphfbr = .true.
      ldmat   = .true.
      ldmat1  = .false.
      lgpop   = .false.
      call zeigausw
      call memausw

      if (ovtype .eq. 0) then
         zeig=1
         do s=1,nstate
            do m=1,nmode
               dovmat(m,s)=zeig
               zeig=zeig+dim(m,s)*dim(m,s)
             enddo
          enddo
          doverdim=zeig-1
          griddim=1

      else if (ovtype .eq. 1) then
         doverdim=1
         griddim=nstate
         do m=1,nmode
            griddim=griddim*subdim(m)
         enddo
      endif

!-----------------------------------------------------------------------
! set up various flags and array information needed to use an operator
!-----------------------------------------------------------------------
      if(loper)  then
         call operflag(nham)
         call zeighpsi(nham)
         if(lcorr) then
            mchpsi  = mcdim
            mcdim   = mchpsi+hpsidim
            mcdtpsi = mcdim
            mcdim   = mcdtpsi+dgldim
!            mchteil = mcdim
!            mcdim   = mchteil+pmatdim
            mchunpsi= 1
         else
            mchunpsi= mcdim
            mcdim   = mchunpsi+totphidim
            mcdtpsi = 1
!            mchteil = 1
            mchpsi  = 1
         endif
         workcdim=max(workcdim,dgldim)
      else
         mcdtpsi = 1
         mchpsi  = 1
         mchunpsi= 1
!         mchteil = 1
      end if


!-----------------------------------------------------------------------
! assign work arrays. The variables m?work? must be set for "memhops".
! NB work arrays must be at end of mc, mr, mi, ml, ms, mf arrays.
!-----------------------------------------------------------------------
      if (ovtype .eq. 0) then
         mcdover=mcdim
         mcdim=mcdover+doverdim
         workcdim=max(workcdim,2*maxblock)
      else if (ovtype .eq. 1) then
         mcdover=1
         workcdim=max(workcdim,3*griddim+dgldim)
      endif

      workrdim = max(workrdim,maxgdim*maxgdim)


      mcgt    = mcdim
      mcdim   = mcgt+2*nout2
      mcworkc = mcdim
      mcdim   = mcworkc+workcdim
 
      mrworkr = mrdim
      mrdim   = mrworkr+workrdim
      mrdim   = max(mrdim,4*maxendim)

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
! assign memory for operator
!-----------------------------------------------------------------------
      call memhops

!-----------------------------------------------------------------------
! call aller routine to assign the memory. Caller calls receiver.
!-----------------------------------------------------------------------

      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'(a,i10,/a,i10)') &
       ' workcdim   : ', workcdim,&
       ' workrdim   : ', workrdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)') &
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)
if(allocated(mchteil))deallocate(mchteil)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))
allocate(mchteil(pmatdim))

call receiver(mc,mr,mi,ml,ms,mf,mchteil)
!call receiver(mc,mr,mi,ml,ms,mf)

deallocate(mc,mr,mi,ml,ms,mf,mchteil)
!deallocate(mc,mr,mi,ml,ms,mf)


!-----------------------------------------------------------------------
! Get timing and write data to screen and "log" file
!-----------------------------------------------------------------------
      t1   = int(tovl/3600)
      tovl = tovl - 3600*t1
      t2   = int(tovl/60)
      tovl = tovl - 60*t2
      write(6,*)
      write(6,900) 'Overlaps done  [h:m:s] : ',t1,t2,tovl 

      tcpu = fcputime(0)
      tsys = fsystime(0)
      ttot = tcpu+tsys
      t1   = int(ttot/3600)
      ttot = ttot - 3600*t1
      t2   = int(ttot/60)
      ttot = ttot - 60*t2

      write(ilog,'(/a,/)') '***** Overall Timing *****'
      write(ilog,'(a,1x,f12.2)') 'Total cpu time     [s] : ',tcpu
      write(ilog,'(a,1x,f12.2)') 'Total system time  [s] : ',tsys
      write(ilog,900) 'Total time     [h:m:s] : ',t1,t2,ttot
      write(6   ,900) 'Total time     [h:m:s] : ',t1,t2,ttot
  900 format( a,i4,' :',i3,' :',f5.1)

      call fpstartup(buffer)
      write(ilog,'(/a)') ' -- fmat terminated normally --'
      write(ilog,'(a80)') buffer
      write(6,'(80a)') ('*',s=1,80)

      stop
      contains


!#######################################################################

      subroutine receiver (mc, mr, mi, ml, ms, mf,mchteil)
!      subroutine receiver (mc, mr, mi, ml, ms, mf)

use global
use aglobal
use maxv, only: maxdim,maxsta,maxkoe
use griddatmod
use psidef
use datenmod
use overlapmod
use compdatmod
use compdat1mod
use fluxmod

      implicit none

      complex(dop), dimension(mcdim), intent(inout)  :: mc
      real(dop), dimension(mrdim), intent(inout)     :: mr
      integer(long), dimension(midim), intent(inout) :: mi
      logical(kind=4), dimension(mldim), intent(in)  :: ml
      complex(sip), dimension(msdim), intent(inout)  :: ms
      real(sip), dimension(mfdim), intent(in)        :: mf
      complex(dop), dimension(pmatdim), intent(inout)  :: mchteil

      call foperin(mc,mr,mi)
 
      call fmt(mc(mcpsi),mc(mcpsi1),ms(mspsi),mc(mcdover),&
               mc(mcworkc),mi(mijindx),mc(mchunpsi),mc(mchpsi),&
               mchteil,mr(mrhops),mc(mcdtpsi),mc(mchc),mr(mrhr),&
               mi(mihi),ml(mlhl),mr(mrort),mc(mcagmat),&
               mc(mcadgwp)) 
!      call fmt(mc(mcpsi),mc(mcpsi1),ms(mspsi),mc(mcdover),&
!               mc(mcworkc),mi(mijindx),mc(mchunpsi),mc(mchpsi),&
!               mc(mchteil),mr(mrhops),mc(mcdtpsi),mc(mchc),mr(mrhr),&
!               mi(mihi),ml(mlhl),mr(mrort),mc(mcagmat),&
!               mc(mcadgwp)) 


      return
      end subroutine receiver

!***********************************************************************
!     reads DVR and oper
!***********************************************************************

      subroutine foperin(mc,mr,mi)

use global
use aglobal
use maxv, only: maxdim,maxsta,maxkoe
use griddatmod
use psidef
use datenmod
use operdef
use compdatmod
use compdat1mod


use rddvrmod, only: rddvr
use rdopermod, only: rdoper

      implicit none

      complex(dop), dimension(*), intent(out)  :: mc
      real(dop), dimension(*), intent(out)     :: mr 
      integer(long), dimension(*), intent(out) :: mi

      real(dop), dimension(1)                 :: rdum
      integer(long)                           :: chkdvr,chkgrd

!-----------------------------------------------------------------------
! Read DVR data needed to propagate wavefunction (see rddvr.f for
! details)
!-----------------------------------------------------------------------
      dvrdata(:) = .false.
      dvrdata(1) = .true.    ! read ort
      dvrdata(2) = .false.   ! read trafo
      dvrdata(3) = .false.   ! read dvrmat (dif1mat, dif2mat)
      dvrdata(4) = .false.   ! read fftp   (fft momentum)
      dvrdata(5) = .true.    ! read hin (fft)
      dvrdata(6) = .true.    ! read rueck (fft)
      dvrdata(7) = .true.    ! read fftfak
      dvrdata(8) = .true.    ! read jsph
      dvrdata(9) = .true.    ! read msph
      dvrdata(10)= .false.   ! read kinsph
      dvrdata(11)= .true.    ! read exphin
      dvrdata(12)= .true.    ! read exprueck
      chkdvr=1
      call rddvr(mr(mrort),rdum,rdum,rdum,mc(mchin),mc(mcrueck),&
                 mi(mifftfak),mc(mcexphin),mc(mcexprueck),&
                 mi(mijsph),mi(mimsph),rdum,chkdvr)

!-----------------------------------------------------------------------
! Read data needed by the operator
!-----------------------------------------------------------------------
      chkdvr=2
      chkgrd=1
      call rdoper(mr(mrhops),chkdvr,chkgrd)
     
      return 
      end subroutine foperin


!-----------------------------------------------------------------------     

      subroutine fmt(psi,psi1,spsi,dover,workc,jindx,hunpsi,&
                     hpsi,hteil,hops,dtpsi,hc,hr,hi,hl,ort,&
                     agmat,adgwp,lsym)

use global
use aglobal
use maxv, only: maxdim,maxsta,maxkoe
use griddatmod
use psidef
use channels
use hpsimod
use datenmod
use operdef
use overlapmod
use compdatmod
use compdat1mod
use fluxmod

use calchamod, only: calcha
use hphimod, only: hphi,hunphi
use phihphimod, only: phihphi
use iofile, only: rdpsiinfo
use tdkoeffs, only: calctdk
use rdfilesmod, only: rdpsi
use flxutils, only: tout,ovlp

!***********************************************************************
!     calculates the matrix elements <psi(t)|psi(t')> and
!     <psi(t)|H|psi(t')> where H is similar to the system operator,
!     but generated with "nodiag" and without CAPs.
!
! Calls to: 
!***********************************************************************

      implicit none

      complex(dop), dimension(*), intent(inout) :: hc
      real(dop), dimension(*), intent(inout)    :: hr
      integer(long), dimension(*), intent(in)   :: hi
      logical(kind=4), dimension(*), intent(in) :: hl

      integer(long), external                         :: slen
      integer(long)                                   :: m,s,i,k1,k2,k3,kk,chk,hktype,ktype
      integer(long), dimension(1)                     :: idum
      integer(long), dimension(adim), intent(out)     :: jindx
      complex(dop)                                    :: ovl,sum
      complex(dop), dimension(1)                      :: cdum
      complex(sip), dimension(spsidim), intent(out)   :: spsi
      complex(dop), dimension(dgldim), intent(out)    :: psi1,psi
      complex(dop), dimension(doverdim), intent(out)  :: dover
      complex(dop), dimension(workcdim), intent(out)  :: workc
      complex(dop), dimension(totphidim), intent(out) :: hunpsi
      complex(dop), dimension(hpsidim), intent(out)   :: hpsi
      complex(dop), dimension(pmatdim), intent(out)   :: hteil
      complex(dop), dimension(dgldim), intent(out)    :: dtpsi
      complex(dop), dimension(agmatdim), intent(out)  :: agmat
      complex(dop), dimension(gsmatdim), intent(out)  :: adgwp
      real(dop), dimension(1)                         :: rdum
      real(dop), dimension(ortdim), intent(in)        :: ort
      real(dop), dimension(hopsdim), intent(in)       :: hops
      real(dop)                                       :: tcpu,tsys,time
      logical(kind=4), dimension(1)                   :: ldum
      real(dop), external                             :: fcputime,fsystime
      logical(kind=4)                                 :: lsym

!-----------------------------------------------------------------------
! Get some additional arguments
!-----------------------------------------------------------------------
      tovl = 0.0_dop
      chk  = 0

!-----------------------------------------------------------------------
! Write head lines to fmat-file mtt (channel iwtt).
!-----------------------------------------------------------------------
      if(laeout.gt.0) then
         write(iwtt,'(/a)') '#   klo    khi   kmax   step'
         write(iwtt,'(a2,i5,3i7)') '# ', klo, khi, kmax, step
         if(lsym) then
            write(iwtt,'(a)') '# symmetric scalar product'
         else
            write(iwtt,'(a)') '# hermitian scalar product'
         endif
         if(loper) then 
            write(iwtt,'(2a)') &
                 '# operator = ',operator(1:slen(operator))
         endif
         write(iwtt,'(/a,3x,a,16x,a,17x,a)') &
              '#  Time[fs]', 'Time[fs]', 'Re(ovl)', 'Im(ovl)'
         call myflush(iwtt)
      end if

!-----------------------------------------------------------------------
! Start the loops over time-steps.
!-----------------------------------------------------------------------
      rewind (ipsi)
      read (ipsi)
      time=0.0_dop
      call rdpsiinfo(ipsi,chk,chk,chk,chk)
       
      do k1=1,klo-1
         read (ipsi)
      enddo

      kk = 0
      do k1=klo,khi,step
        if(mod(k1-klo,5).eq.0 ) call tout(k1,klo,time)
        time = (k1-1)*out2

!------------------------------------------------------------------------
! Read psi1 = Psi(t=t_k1)
!------------------------------------------------------------------------
         call rdpsi(ipsi,psi1,spsi,jindx,agmat,idum,adgwp)
         if (lend) goto 77
         do i = 2, min(step,nout2-k1)
            read (ipsi,end=77)
         enddo

!-----------------------------------------------------------------------
! calculate time-dependent coefficients
!-----------------------------------------------------------------------
         if (ltdh(nham)) call calctdk(time,nham,psi,cdum,rdum,idum,ldum)

!------------------------------------------------------------------------
!     Multiply Psi1(t) by the specified operator
!------------------------------------------------------------------------
         if(lcorr) then
            ktype=0
            call hphi(psi1,hpsi,hc,hr,hi,hl,ktype,nham)

         elseif(loper) then
            hktype=0
            call hunphi(psi1,hunpsi,workc,hc,hr,hi,hl,hktype,nham)
            do s = nst1, nst2
               do m = 1, nmode 
                  if(khzahl(m,s,nham) .gt. 0 ) &
                   call cpvxz (hunpsi(zetf(m,s)-adim),&
                               psi1(zetf(m,s)),dim(m,s)*subdim(m))
               enddo
            enddo
         endif

!========================================================================
!------------------------------------------------------------------------
!     Read psi = Psi(t'=t_k2)  for t'<=t
!------------------------------------------------------------------------   
         rewind (ipsi)
         read (ipsi)
         call rdpsiinfo(ipsi,chk,chk,chk,chk)

         do k2=1,klo-1
            read (ipsi)
         enddo
         do k2=klo,k1,step
            kk = kk + 1
            call rdpsi(ipsi,psi,spsi,jindx,agmat,idum,adgwp)
            if (lend) goto 77
            do i = 2, min(step,nout2-k2)
               read (ipsi,end=77)
            enddo

            if(lsym) then       ! lsym is true when -sym is set.
               do i = 1, dgldim
                  psi(i) = dconjg(psi(i))
               enddo
            endif

            if(lcorr) then
               call zerovxz(dtpsi,dgldim)
               ktype=0
               if (ltdh(nham)) call calctdk(time,nham,psi,cdum,rdum,&
                   idum,ldum)
               call phihphi(psi,cdum(1),hpsi,hteil,nham,.false.)
               call calcha(psi1,dtpsi,hteil,workc,hops,psi1(zetf(1,1)),&
                    nham,psi(zetf(1,1)),jindx,time,ort)

               ovl=(0.0_dop,0.0_dop)
               do s=1,nstate
                  call vvaxzz(psi(zpsi(s)),dtpsi(zpsi(s)),sum,block(s))
                  ovl=ovl+sum
               enddo
            else
               call ovlp(ovl,psi,psi1,dover,workc)
            endif
            if(laeout.gt.0) write(iwtt,'(2f11.3,3x,2e24.14)') &
                 (k2-1)*out2/fs,(k1-1)*out2/fs, ovl
         end do  ! k2-loop  (psi)
         k3 = k1+1
      end do  ! k1-loop   (psi1)

      call tout(k3,klo,time)    ! k3 = last k1 +1


!-----------------------------------------------------------------------
! Get timing. Will be written in main program.
!-----------------------------------------------------------------------
      tcpu = fcputime(0)
      tsys = fsystime(0)
      tovl = tcpu+tsys

      close(iwtt)
      if(lproj .and. ipsi.eq.ipsi1) then
         close(ipsi,status='delete') ! Delete the reduced WF file. 
      else
         close(ipsi,status='keep') 
      endif
      return

 77   continue
      write(6,'(a)') ' EOF found when reading the psi-file!'
      stop
      end subroutine fmt

!***********************************************************************

      subroutine fmatinput(lsym)

use global
use aglobal
use maxv, only: maxdim,maxsta,maxkoe
use griddatmod
use psidef
use channels
use datenmod
use overlapmod
use compdatmod
use compdat1mod
use fluxmod
use basismod

use iofile, only: rdpsiinfo
use rddvrmod, only: dvrinfo
use rdopermod, only: operinfo

      implicit none

      integer(long), external :: myiargc,slen
      integer(long)           :: iarg,ilbl,i,m,n,f,chkdvr,&
                                 chkgrd,chkpsi,chkdat,s,ll,dvrlaenge
      logical(kind=4), intent(in) :: lsym
      logical(kind=4)         :: lerr,
      character(len=80)       :: buf,nspf
      character(len=c5)       :: filename,mdlbl

!-----------------------------------------------------------------------

      name    = '.'
      dname   = '.'
      oname   = '.'
      laenge  = 1
      dlaenge = 1
      olaenge = 1
      nppar   = 0
      nprj    = 0
      start   = 0
      nst1    = 1
      nst2    = 0

      filein  = './'
      filein1 = 'mtt'
      laein1  = index(filein1,' ')-1
      fileout = './'
      operfile= 'oper'
      operator= ' '
      dvrfile = 'dvr'
      loper = .false.
      lcorr = .false.
      lsym  = .false.
      lproj = .false.
      ltest = .false.
      lwtt  = .false.
      lgtt  = .false.
      full  = .false.
      ltonly= .false.
      l2short = .false.
      klo   = 1
      khi   = 0
      iarg  = myiargc() 
      if (iarg .le. 0) goto 200 
      call mygetarg(0,options)
      optlaenge  = index(options,' ') 
      do i = 1, iarg
         call mygetarg(i,buf)
         ilbl  = index(buf,' ')
         ilbl = min(ilbl,100-optlaenge)
         if(ilbl.gt.0) options = options(1:optlaenge)//buf(1:ilbl) 
         optlaenge = optlaenge + ilbl
      end do


!........Read the options
 200  call rdoption(iarg,buf,lsym)

      laein=index(filein,' ')-1
      if (filein(laein:laein) .eq. '/') then 
         filein = filein(1:laein)//'psi'
         laein = laein + 3
      end if
      call abspath(filein,laein)

      opflaenge=index(operfile,' ')-1
      if (operfile(opflaenge:opflaenge) .eq. '/') then 
         operfile = operfile(1:opflaenge)//'oper'
         opflaenge = opflaenge + 4
      end if
      call abspath(operfile,opflaenge)

      dvrlaenge=index(dvrfile,' ')-1
      if (dvrfile(dvrlaenge:dvrlaenge) .eq. '/') then 
         dvrfile = dvrfile(1:dvrlaenge)//'dvr'
         dvrlaenge = dvrlaenge + 3
      end if
      call abspath(dvrfile,dvrlaenge)

      if (fileout(1:2) .eq. 'no') then
         laeout = 0
      else
         laeout=index(fileout,' ')-1
         call abspath(fileout,laeout)
         open(99,file=fileout(1:laeout)//'/test-&_%-tset',err=56)  
         goto 57
 56      close(99,status='DELETE')
         write(6,'(a,/a,/a,/a,/a)') &
              '######################################################',&
              ' ERROR: The output directory:', fileout(1:laeout),&
              ' does not exist (or is not writeable)!',&
              '######################################################'
         stop
 57      close(99,status='DELETE')
      end if

      write(6,'(80a)') ('*',i=1,80)
      if(inplaenge .gt. 0 ) &
        write(6,'(2a)') ' The input-file is : ',inputfile(1:inplaenge)

      kmax  = (khi-klo+step)/step

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      filename=filein(1:laein)
      open(ipsi,file=filename,form='unformatted',&
           status='old',err=1000)
      write(6,'(2a)') ' The psi-file   is : ',filein(1:laein)
      filename=operfile(1:opflaenge)
      open(ioper,file=filename,form='unformatted',&
           status='old',err=1000)
      write(6,'(2a)') ' The oper-file  is : ',operfile(1:opflaenge)

      if(laeout .eq. 0) then
         write(6,'(2a)') ' No outputfiles written '
         open (unit=ilog,status='scratch',form='formatted')
      else
         write(6,'(2a)')' The output dir is : ',fileout(1:laeout)
         filename=fileout(1:laeout)//'/fmat.log'
         open (unit=ilog,file=filename,form='formatted',err=1000)
         logisopen=.true.
         call fpstartup(filename)
         write(ilog,'(a80)') filename
         call wrversion(ilog)
      end if

      open(idvr,file=dvrfile,form='unformatted',status='old',err=1000)
      write(6,'(2a)') ' The DVR-file   is : ',dvrfile(1:dvrlaenge)


!-----------------------------------------------------------------------
!  Read DVR, psi-file and oper-file information.
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      call dvrinfo(lerr,chkdvr)
      close(idvr)
      chkdvr=2
      read(ipsi) filever(ipsi)
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      call chksyserr(filein,lerr)
      if(psitype.eq.3) then
         routine='fmatinput'
         message='Sorry, fmat cannot work with ML-wavefunctions.'
         call errormsg
      endif
      if (lerr) goto 999
      chkgrd=2
      call operinfo(lerr,chkdvr,chkgrd)
      close(ioper)
      if (lerr) goto 888
      
      write(6,'(a,f5.1,a,i2,a,4x,f5.1,a,i2,a)') &
           ' program-version =',&
           filever(ipsi), '.',int(filever(ipsi )*1.0e+4_dop) - &
           100*int(filever(ipsi )*1.0e+2_dop), ' (psi-file)',&
           filever(ioper),'.',int(filever(ioper)*1.0e+4_dop) -&
           100*int(filever(ioper)*1.0e+2_dop), ' (oper-file)'
      write(6,'(78a1)')  ('-',i=1,78)
      write(6,'(a,i4)') ' Number of degrees of freedom:', ndof       
      write(6,'(a,i4)') ' Number of combined modes    :', nmode
      write(6,'(a,i4)') ' Number of electronic states :', nstate
      if( nst2 .le. 0 ) then
         nst2 = nstate
      else
         write(6,'(a,2i4)') ' Flux evaluated for states   :',&
              nst1, nst2
      endif
      
      write(6,'(/,a)') ' mode dof  dim   DVR     modelabel'
      do m=1,nmode
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            write(6,'(1x,i3,1x,i4,i5,3x,a8,a10)') &
                 m, f, gdim(f), cbas(basis(f)), modelabel(f) 
         enddo
      enddo

!-----------------------------------------------------------------------
! write mode information.
!-----------------------------------------------------------------------
      write(6,'(/,a)') ' mode: No. of spf''s per state : modelabels'
      do m=1,nmode
         ll = 1
         mdlbl='                                                       '
         nspf ='                                                       '
         do  n=1,nspfdof(m)
            ilbl  = index(modelabel(spfdof(n,m)),' ')
            mdlbl = mdlbl(1:ll)//modelabel(spfdof(n,m))(1:ilbl)
            ll    = ll + ilbl
         end do
         write(nspf,'(15i3)') (dim(m,s),s=1,nstate)
         if(3*nstate+ll .le. 72 ) then
            write(6,'(i4,4a)') m,' :',nspf(1:3*nstate),' : ',&
                 mdlbl(1:ll)
         else
            write(6,'(i4,3a)') m,' :',nspf(1:3*nstate),' :'
            write(6,'(a,/)') mdlbl(1:ll)
         end if
      enddo

!-----------------------------------------------------------------------
! Determine number of data sets.
!-----------------------------------------------------------------------

      nout2 = 0
 101  continue
      read(ipsi,end=102,err=998)
      nout2 = nout2 + 1
      goto 101
 102  tfinal = out2*(nout2-1) + tinit
      write(6,'(/a,i6,a)') ' There are', nout2,&
                          ' data sets on the psi file.'
      write(6,'(a,f7.2,a,f9.2,a,f8.2,a)') &
        ' tinit =', tinit/fs, ' fs,  t-final =', tfinal/fs, &
        ' fs,  tpsi =', out2/fs, ' fs.' 

      if(lsym) then
         write(6,'(/,a)') ' Using the symmetric scalar product.'
      else
         write(6,'(/,a)') ' Using the hermitian scalar product.'
      endif
 
      if(khi .eq. 0) khi = nout2
      if(khi.ne.nout2 .or. klo.ne.1) then
      write(6,'(/a,/a,i5,a,f11.2,a,/a,i5,a,f11.2,a)') &
      ' times for integration are changed by -lo and/or -hi option to:'&
      ,' first data point, klo =', klo,', at:',&
                     ((klo-1)*out2+tinit)/fs,'fs'&
      ,' last  data point, khi =', khi,', at:',&
                     ((khi-1)*out2+tinit)/fs,'fs'
      end if

      if( nst1 .gt. nst2 .or. nst2 .gt. nstate ) then
         write(6,'(/a,/a,2i4,/a,i4,/a)')&
               '######################################################'&
              ,' Illegal value of (option -es) nst1,nst2 =', nst1,nst2&
              ,' 1 .le. nst1 .le. nst2 .le.', nstate    &
              ,'######################################################'
         stop
      end if
      if( khi .gt. nout2 ) then
         write(6,'(/a,/a,i5,/a,i5,/a)')&
               '######################################################'&
              ,' Illegal value of (option -hi) khi =', khi&
              ,' khi must be lower/equal  nout2 =', nout2  &  
              ,'######################################################'
         stop
      end if
      if( klo .ge. khi ) then
         write(6,'(/a,/a,/a,i5,a,i5,/a)')&
               '######################################################'&
              ,' Illegal values of (options -lo, -hi) klo and khi'&
              ,' klo =',klo,' must be lower than  khi =', khi    &
              ,'######################################################'
         stop
      end if

      kmax  = (khi-klo+step)/step

!.....Write to log-file.
      call fpstartup(inptit)
      write(ilog,'(a80)') inptit
      if( optlaenge .le. 60 ) then
         write(ilog,'(2a/)')   ' Options : ', options(1:optlaenge)
      else
         write(ilog,'(a,/a/)') ' Options : ', options(1:optlaenge)
      end if

      if(inplaenge .gt. 0 ) write(ilog,'(2a)')&
                      ' The input-file is : ',inputfile(1:inplaenge)
      write(ilog,'(2a)') ' The psi-file   is : ',filein(1:laein)
      write(ilog,'(2a)') ' The oper-file  is : ',operfile(1:opflaenge)
      write(ilog,'(2a)') ' The output dir is : ',fileout(1:laeout)
      write(ilog,'(2a)') ' The DVR-file   is : ',dvrfile(1:dvrlaenge)

      write(ilog,'(/a,i5,a,i5,a,i3/)') ' klo =', klo,&
         ',  khi =', khi,',  step =', step

      if(loper) write(ilog,'(2a)') ' Operator = ',&
       operator(1:slen(operator))

      return
 

 888  continue
      message= 'Error reading oper-info from:'//operfile(1:opflaenge)
      write(6,'(a)') message
      stop

 998  continue
      message= 'Error reading through:'//filein
      write(6,'(a)') message
      stop

 999  continue
      message= 'Error reading psi-info from:'//filein(1:laein)
      write(6,'(a)') message
      stop
      
 1000 write(6,'(a)') ' Cannot open file : '
      write(6,'(a)')  filename
      stop
      
      end subroutine fmatinput


!***********************************************************************
! Read the options.
!***********************************************************************

      subroutine rdoption(iarg,buf,lsym)

use global
use aglobal
use maxv, only: maxdim,maxsta
use overlapmod
use fluxmod

      implicit none

      integer(long)                 :: i,j,ierr
      integer(long), intent(in)     :: iarg
      character(len=80), intent(in) :: buf
      logical(kind=4), intent(in)   :: lsym

      if(iarg .le. 0) goto 400

!........Read the options
      i = 0
 100  i=i+1
      if (iarg .lt. i) goto 400
      call mygetarg(i,buf)

      if( buf(1:1) .eq. '-' ) then
         j = 1
 99      j = j + 1
         if( buf(j:j) .eq. ' ' ) goto 100
!........Options of more than one character length without arguments.
         if( buf(j:j+2) .eq. 'ver' ) then
            call wrversion(6)
            stop
         else if( buf(j:j+2) .eq. 'sym ' ) then
            lsym = .true.
   
!........Options of more than one character length with arguments.
         else if( buf(j:j+3) .eq. 'dvr ' ) then
            i = i+1
            call mygetarg(i,dvrfile)
         else if( buf(j:j+2) .eq. 'lo ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) klo
            if (ierr .ne. 0  .or.  klo .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -lo',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(j:j+2) .eq. 'hi ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) khi
            if (ierr .ne. 0  .or.  khi .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -hi',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(j:j+2) .eq. 'es ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nst1
            nst2 = nst1
            if (ierr .ne. 0  .or.  nst1 .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -es',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if

!........Options of one character length without arguments
         else if( buf(j:j) .eq. 'w' ) then
            lovwr=.true.
            goto 99
         else if( buf(j:j) .eq. 'h' .or. buf(j:j) .eq. '?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(2x,a,/8x,a)') &
       'Purpose: Computes the overlap- or a Hamiltonian-matrix.', &
       "   Mtt' = <psi(t)|psi(t')> or Mtt'= <psi(t)|H|psi(t')>"

            write(6,'(2x,2a)')&
       'Usage  : fmat<ver> [-f -i -d -o -dvr -n -lo -hi -O ',&
             '-w -sym -es -ver -h -?] .'
            write(6,'(/,a)') ' Options : '
            write(6,'(2a,/,9x,a)') &
                     ' -f FILE : The wavefunction is read '&
                    ,'from file FILE rather than from ./psi'&
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a)') &
                     ' -i FILE : The operator is read '&
                    ,'from file FILE rather than from ./oper'&
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(2a,/,9x,a)') &
                     ' -dvr FL : The DVR information is read '&
                    ,'from file FL rather than from ./dvr'&
            ,'  The string FL may be a relative or a full path-name.'
            write(6,'(2a,/,11x,a/,11x,a/,11x,a)') &
                     ' -d PATH : The output is directed to PATH ',&
                     'directory rather than to the ',&
                     'current directory.',&
            'The string PATH may be a relative or a full path-name.',&
            'If PATH=no, i.e. "-d no", no output-files will be opened.'
            write(6,'(2a)') ' -o NAME : Change the name of the output',&
                            ' file from "mtt_h"("mtt_s") to NAME.'
            write(6,'(2a)') ' -n step : Compute Mtt'' only', &
                            ' every step-th tpsi.'
            write(6,'(2a)') ' -es nst : Compute Mtt'' for the',& 
                            ' electronic state nst only.'
            write(6,'(2a)') ' -lo klo : Compute Mtt'' only', &
                            ' from the klo-th tpsi onwards.'
            write(6,'(2a)') ' -hi khi : Compute Mtt'' only', &
                            ' up to the khi-th tpsi.'
            write(6,'(2a)') ' -O oper : Apply operator ',&
                            '"oper" before evaluating the overlap.' 
            write(6,'(a)')  ' -w      : Overwrite an existing Mtt file.'
            write(6,'(2a)') ' -sym    : Use the symmetric scalar', &
                            ' product rather than the hermitian one.'
            write(6,'(a)')&
                     ' -ver    : Version information about the program '
            write(6,'(a,/a,/)') &
                     ' -h      : Print this help text. ',&
                     ' -?      : Print this help text. '
            write(6,'(a)')  ' Example : fmat<ver> -hi 65 -O ham '
            write(6,'(2a)') ' Important note: The used operator ',&
                            'must be build with the "nodiag" flag!' 
            write(6,'(78a1)')  ('-',i=1,78)
            stop

!........Options of one character length with arguments.
         else if( buf(j:j+1) .eq. 'f ' ) then
            i = i+1
            call mygetarg(i,filein)
         else if( buf(j:j+1) .eq. 'i ' ) then
            i = i+1
            call mygetarg(i,operfile)
         else if( buf(j:j+1) .eq. 'd ' ) then
            i = i+1
            call mygetarg(i,fileout)
         else if( buf(j:j+1) .eq. 'o ' ) then
            i = i+1
            call mygetarg(i,filein1)
            laein1=index(filein1,' ')-1
         else if( buf(j:j+1) .eq. 'n ' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(j:j+1) .eq. 'O ' ) then
            loper=.true.
            i = i+1
            call mygetarg(i,operator)
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
          if (iarg .le. i) goto 400
         goto 100
      end if

      if (iarg .lt. i) goto 400


 400  return
      end subroutine rdoption



      end program fmat
