!***********************************************************************
!
!                         AUSWERT routine HERMA
!
!     This subroutine checks the hermiticity of the A-vector
!
!     06/2003 CC
!
!
!***********************************************************************
      
      program herma

use decimal, only: long,dop,sip
use global
use aglobal
use maxv, only: maxdim,maxsta,maxkoe
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use compdat1mod
use overlapmod

use d2tens, only: chkherm
use iorst, only: rstinfo
use iofile, only: rdpsiinfo
use zeigauswmod, only: zeigausw

use module_interface

      implicit none

      integer(long)                        :: zeig,zeig1,zeig2,m,s,t,chkdvr,&
                                              chkgrd,chkpsi,chkdat,chkprp
      character(len=c5)                    :: filename
      logical(kind=4)                      :: lerr,linwf

complex(dop), dimension(:), allocatable    :: mc
real(dop), dimension(:), allocatable       :: mr
integer(long), dimension(:), allocatable   :: mi
logical(kind=4), dimension(:), allocatable :: ml
complex(sip), dimension(:), allocatable    :: ms
real(sip), dimension(:), allocatable       :: mf


!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      call hermainput 

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
      open (unit=ilog,file='ausw.log',form='formatted')
      logisopen=.true.

      if( lrst ) then
!-----------------------------------------------------------------------
! read data from restart file  (lrst=true --> read restart rathen than psi)
!-----------------------------------------------------------------------
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkprp=1
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         if (linwf) write(6,'(a)') ' Initial wave function' 
         if (lerr) then 
            write(6,'(a)') ' ERROR reading restart file' 
            stop
         end if
      else
!-----------------------------------------------------------------------
! read data from psi file
!-----------------------------------------------------------------------
         read(ipsi) filever(ipsi)
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkdat=1
         call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
         filename='psi'
         call chksyserr(filename,lerr)
         if (lerr) go to 999
      endif


      if (nmode .eq. 1) then
         do s=1,nstate
            if (dentype.eq.1) then
               do t=1,nstate
                  ddim(1,s,t)=1
               enddo
            else
               dim(1,s)=1
            endif
         enddo
      endif
      
!-----------------------------------------------------------------------
! write head-line.
!-----------------------------------------------------------------------
      if (dentype.eq.2) then
         write(6,'(/,a)') &
              '            Hermiticity error of A-vector'
         write(6,'(a,33(a,i2,a),33(a,i2,a),33(a,i2,a))')&
              '  Time[fs]  state  error  ' 
      else if(dentype.eq.0) then
         write(6,'(/,a)') ' Wavefunction not implemented!'
         stop
      else
         write(6,'(/,a)') ' Type I not implemented!'
         stop
      endif



!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
!-----------------------------------------------------------------------
      lpsi=.true.
      lpsigrd=.false.
      call zeigausw

      zeig=1
      if (dentype.eq.1) then
         do s=1,nstate
            do t=1,nstate
               zeig1=dblock(s,t)
               do m=1,nmode
                  zeig=zeig+ddim(m,s,t)*ddim(m,s,t)
                  zeig2=zeig1
                  zeig2=zeig2+zeig1
                  workcdim=max(workcdim,zeig2)
               enddo
            enddo
         enddo
      else
         do s=1,nstate
            zeig1=block(s)
            do m=1,nmode
               zeig=zeig+dim(m,s)*dim(m,s)
               zeig2=zeig1
               zeig2=zeig2+zeig1
               workcdim=max(workcdim,zeig2)
            enddo
         enddo
      endif


!-----------------------------------------------------------------------
! assign memory arrays
!-----------------------------------------------------------------------
      call memausw
      mcovr=mcdim
      mcdim=mcovr+dmatdim

!-----------------------------------------------------------------------
! assign work arrays
!-----------------------------------------------------------------------
      workcdim=max(workcdim,2*maxblock)
      do m=1,nmode
         workcdim = max(workcdim,2*subdim(m)**2)
      enddo

      mcworkc = mcdim
      mcdim = mcworkc+workcdim

      mrworkr = mrdim
      mrdim   = mrworkr+workrdim

      miworki = midim
      midim   = miworki+workidim

      mlworkl = mldim
      mldim   = mlworkl+workldim

      msworks = msdim
      msdim   = msworks+worksdim

      mfworkf = mfdim
      mfdim   = mfworkf+workfdim

!-----------------------------------------------------------------------
! call overlap routine
!-----------------------------------------------------------------------
if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

call receiver(mc,mr,mi,ml,ms,mf)

deallocate(mc,mr,mi,ml,ms,mf)

      stop
  999 continue
      message= 'Error reading :'//filename
      write(6,'(a)') message
      stop
      contains

!#######################################################################
      subroutine receiver (mc, mr, mi, ml, ms, mf)

use global
use aglobal
use maxv, only: maxdim,maxsta,maxkoe
use griddatmod
use psidef
use datenmod
use compdatmod
use compdat1mod
use overlapmod

      implicit none

      complex(dop), dimension(mcdim), intent(inout)  :: mc
      real(dop), dimension(mrdim), intent(in)        :: mr
      integer(long), dimension(midim), intent(inout) :: mi
      logical(kind=4), dimension(mldim), intent(in)  :: ml
      complex(sip), dimension(msdim), intent(inout)  :: ms
      real(sip), dimension(mfdim), intent(in)        :: mf


      call herma1(mc(mcpsi),ms(mspsi),mc(mcovr),&
           mc(mcworkc),mi(miworki),mi(mijindx))

      return
      end subroutine receiver

      
!***********************************************************************
!     calculates the hermiticity error 
!***********************************************************************
      subroutine herma1 (psi,spsi,ovr,workc,worki,jindx)

use global
use aglobal
use maxv, only: maxdim,maxsta,maxkoe
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use compdat1mod
use overlapmod

use rdfilesmod, only: rdpsi


      implicit none

      integer(long)                                 :: i,dgl,k,s
      integer(long), dimension(1)                   :: idum
      integer(long), dimension(adim), intent(out)   :: jindx
      integer(long), dimension(workidim),intent(out):: worki
      complex(dop), dimension(1)                    :: zdum
      complex(sip), dimension(spsidim), intent(out) :: spsi
      complex(dop), dimension(dgldim), intent(out)  :: psi
      complex(dop), dimension(dmatdim), intent(in)  :: ovr
      complex(dop), dimension(workcdim), intent(in) :: workc
      real(dop)                                     :: time,dt,error,hermerr

!-----------------------------------------------------------------------
! calculate time step for data
!-----------------------------------------------------------------------
      dt = out2/fs
      time = tinit/fs
      k    = 0
   10 continue
!-----------------------------------------------------------------------
! choose overlap calculation routine
!-----------------------------------------------------------------------

      if (lrst) then
!-----------------------------------------------------------------------
! read restart wavefunction  (lrst=true --> read restart rather than psi)
!-----------------------------------------------------------------------
         read (irst,err=998) time
         time = time/fs
         read (irst,err=998) (psi(dgl),dgl=1,dgldim)
      else
         call rdpsi(ipsi,psi,spsi,jindx,zdum,idum,zdum)
         if (lend) goto 77
      end if

      if( mod(k,step) .eq. 0 ) then
         error = 0.0_dop
         do s=1,nstate
           call chkherm(psi(dzpsi(s,s)),hermerr,block(s))
           error = error + hermerr
           write(6,'(f12.4, i3, e16.8)') time, s, error  
         enddo 
      endif
 
      k = k + 1
      time=time + dt

!-----------------------------------------------------------------------
! if more data, go to beginning of loop
!-----------------------------------------------------------------------
      if (.not.lrst)  goto 10

 77   continue
      write(6,'(76a)') ('-',i=1,76)
      close(iaus)
      close(ipsi)

      return
         
 998  continue
      write(6,'(a)') ' Error reading restart file'
      stop   
      end subroutine herma1



!#######################################################################
!-----------------------------------------------------------------------
      subroutine hermainput

use global
use aglobal
use maxv, only: maxdim,maxsta,maxkoe
use griddatmod
use psidef
use channels
use datenmod
use compdatmod
use compdat1mod
use overlapmod

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: i,iarg,ierr
      character(len=80)       :: buf
!-----------------------------------------------------------------------

      filein  = '  '
      name    = './'
      lrst    = .false.
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-f') then
            i = i+1
            call mygetarg(i,buf)
            filein = buf
         else if(buf .eq. '-i') then
            i = i+1
            call mygetarg(i,buf)
            name = buf
         else if( buf(1:2) .eq. '-n' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) step
            if (ierr .ne. 0  .or.  step .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
         else if( buf(1:2) .eq. '-r' ) then
            irst = ipsi
            lrst = .true.
            write(6,'(a)') ' *** Reading the RESTART file !  ***'
         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(1x,a)')&
       'Purpose: Checks the hermiticity of the A-vector.'
            write(6,'(1x,a)')&
       '         Only available for density type II.'
            write(6,'(1x,a)')&
       'Usage:   herma<ver> [-f -i -n -r -ver -h -?] .'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/,9x,a/,9x,a)') &
                     ' -f FILE : The density matrix '&
                    ,'is read from file '&
            ,'  FILE rather than from ./psi'&
            ,'  The string FILE may be a relative or a full path-name.'
            write(6,'(a)') ' -i FILE : Same as -f FILE.'
            write(6,'(2a)') ' -n step : Compute the error', &
           ' only every step-th output.'
            write(6,'(a)') &
           ' -r      : Take the restart file rather than the psi file.'
            write(6,'(a)')&
           ' -ver    : Version information about the program '
            write(6,'(a,/a)')& 
           ' -h      : Print this help text. ',&
           ' -?      : Print this help text. '
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if
      
      if (iarg .lt. i) goto 200

 200  continue
      if( step .gt.1  .and. lrst ) then
         write(6,'(a)') ' -n and -r options are incompatible'
         stop
      end if

!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
! Otherwise take name from filein
!-----------------------------------------------------------------------
      if (filein .eq. ' ') then
         laenge=index(name,' ')-1
         call abspath(name,laenge)
         if(lrst)then
            filein=name(1:laenge)//'/restart'
         else 
            filein=name(1:laenge)//'/psi'
         endif
         laein=index(filein,' ')-1
      else
         laein=index(filein,' ')-1
         call abspath(filein,laein)

         i=laein
   10    continue
         if (filein(i:i) .eq. '/') then
            laenge=i-1
         else
            i=i-1
            go to 10
         endif
         name=filein(1:laenge)
      endif

!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
      open(ipsi,file=filein(1:laein),form='unformatted',&
           status='old',err=1000)
      write(6,'(78a1)')  ('*',i=1,78)
      write(6,'(2a)') ' Analyse program "herma<ver>". ', &
                      'Check the hermiticity of the A-vector'
      write(6,'(2a)') ' The inputfile is : ',filein(1:laein)

      return 
      
 1000 write(6,*)' Cannot open file : '
      write(6,*) filein(1:laein)
      stop
      
      end subroutine hermainput

! below routines are to check hermiticity of type I SPDOs
      subroutine hermchk(psi,dtpsi)

use griddatmod
use psidef

      integer(long) :: i,m,s
      real(dop) :: d1,d2
      complex(dop), dimension(dgldim) :: psi,dtpsi

      d1=0.0
      d2=0.0
      do i=1,adim
         d1 = d1 + dimag(psi(i))
         d2 = d2 + dimag(dtpsi(i))
      enddo
      print*,'Imag psi/dtpsi: ',d1,d2

      do m=1,nmode
      print*,'Mode',m
         call subhermchk(psi(dzetf(m,1,1)),dtpsi(dzetf(m,1,1)),supgdim(m),ddim(m,1,1))
      enddo

      return
      end

      subroutine subhermchk(psi,dtpsi,supgdim1,ddim1)

use griddatmod
use psidef

      integer(long), intent(in) :: supgdim1,ddim1
      integer(long) :: i,j,k
      real(dop) :: d1,d2
      complex(dop), dimension(supgdim1,supgdim1,ddim1) :: psi,dtpsi

      d1=0.0
      d2=0.0
      do k=1,ddim1
         do i=1,supgdim1
            d1 = d1 + dimag(psi(i,i,k))
            d2 = d2 + dimag(dtpsi(i,i,k))
         enddo
         print*,k,' Diag Imag psi/dtpsi: ',d1,d2
      enddo
     d1=0.0
      d2=0.0
      do k=1,ddim1
         do i=1,supgdim1
            do j=1,supgdim1
               d1 = d1 + (psi(i,j,k))-dconjg(psi(j,i,k))
               d2 = d2 + (dtpsi(i,j,k))-dconjg(dtpsi(j,i,k))
            enddo
         enddo
         print*,k,' Off Diag Imag psi/dtpsi: ',d1,d2
      enddo

      return
      end

      end program herma
