!***********************************************************************
!
!                         AUSWERT subroutine JOINPSI
!
!     This subroutine joins different psi files with different nspf's
!     together.
!
!     05/04 SS
!***********************************************************************

      program joinpsi

use decimal, only: long,dop,sip
use global
use aglobal
use maxv, only: maxdim,maxsta
use griddatmod
use psidef
use channels
use datenmod
use joinpsimod

use op1lib, only: cpvxz, zerovxz
use logdat, only: errormsg
use iopsidef, only: psidat
use iofile, only: rdpsiinfo

use module_interface

      implicit none

      integer(long)                        :: t1,t2,ilbl,j,s
      real(dop)                            :: tcpu,tsys,ttot
      real(dop), external                  :: fcputime,fsystime
      character(len=100)                   :: buffer

complex(dop), dimension(:), allocatable    :: mc
real(dop), dimension(:), allocatable       :: mr
integer(long), dimension(:), allocatable   :: mi
logical(kind=4), dimension(:), allocatable :: ml
complex(sip), dimension(:), allocatable    :: ms
real(sip), dimension(:), allocatable       :: mf


!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
      call default
      call adefault
      call joininp

! --- CALCULATE MEMORY AND POINTERS
      adim=1
      dmatdim=1
      maxblock=1
      do s=1,nstate
         maxblock=max(maxblock,block(s))
      enddo
      workcdim=2*maxblock

      mrdim=1
      mfdim=1
      midim=1
      mldim=1

      mcdim=1
      mcpsi=mcdim
      mcdim=mcdim+dgldim
      mcpsi1=mcdim
      mcdim=mcdim+dgldim
      mcworkc=mcdim
      mcdim=mcworkc+workcdim

      msdim=1
      mspsi=msdim
      if (lpsiopt(1)) then
         msdim=mspsi+dgldim
      endif

listdim = filenum
maxdim=32
maxsta=10

allocate(lepsi1(listdim))
allocate(jnum(listdim))
allocate(ipsij(listdim))
allocate(nf(listdim))
allocate(dgldim1(listdim))
allocate(t0(listdim))
allocate(cpsi1(listdim))
allocate(zpsi1(maxsta,listdim))
allocate(block1(maxsta,listdim))
allocate(vdim1(maxdim,maxsta,listdim))
allocate(dim1(maxdim,maxsta,listdim))
allocate(vdim1(maxdim,maxsta,listdim))
allocate(zetf1(maxdim,maxsta,listdim))

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

! --- START PROGRAM

call receiver(mi,mr,mc,ml,ms,mf)

deallocate(mc,mr,mi,ml,ms,mf)
deallocate(lepsi1,jnum,ipsij,nf)
deallocate(dgldim1,t0,cpsi1)
deallocate(zpsi1,block1)
deallocate(vdim1,dim1,ndim1,zetf1

 
! --- WRITE MEMORY ALLOCATION INFORMATION TO THE LOG-FILE ---  
      write(ilog,'(/,80a1)') ('*',j=1,80)

      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'(/a,i10,/a,i10)')&
       ' workcdim   : ', workcdim,&
       ' workrdim   : ', workrdim
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)')&
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
 
! --- GET TIMIMG AND WRITE IT TO THE "LOG" FILE ---
 
      tcpu = fcputime(0)
      tsys = fsystime(0)
      ttot = tcpu+tsys
      t1   = int(ttot)/3600
      ttot = ttot - 3600*t1
      t2   = int(ttot)/60
      ttot = ttot - 60*t2
 
      write(ilog,'(/a,/)') '***** Overall Timing *****'
      write(ilog,'(a,1x,f12.2)') 'Total cpu time     [s] : ',tcpu
      write(ilog,'(a,1x,f12.2)') 'Total system time  [s] : ',tsys
      write(ilog,900) 'Total time     [h:m:s] : ',t1,t2,ttot
!      write(6   ,900) 'Total time     [h:m:s] : ',t1,t2,ttot
  900 format( /a,i4,' :',i3,' :',f5.1)
 
      call fpstartup(buffer)
      write(ilog,'(/a)') ' -- joinpsi terminated normally --'
      write(ilog,'(a80)') buffer
!      write(6,'(80a)') ('*',s=1,80)
 
      close(ilog)

      contains

!#######################################################################
!-----------------------------------------------------------------------
!                          SUBROUTINE RECEIVER
!
! This routine is called from "caller" and continues the Joinpsi program
! after having allocated the memory dynamically.
!-----------------------------------------------------------------------


      subroutine receiver (mi, mr, mc, ml, ms, mf)

use global
use aglobal
use maxv, only: maxdim,maxsta
use griddatmod
use psidef
use channels
use datenmod
use joinpsimod

use iofile, only: wrpsiinfo
use psioutmod, only: psiout

      implicit none

      complex(dop), dimension(mcdim), intent(inout)  :: mc
      real(dop), dimension(mrdim), intent(in)        :: mr
      integer(long), dimension(midim), intent(inout) :: mi
      logical(kind=4), dimension(mldim), intent(in)  :: ml
      complex(sip), dimension(msdim), intent(inout)  :: ms
      real(sip), dimension(mfdim), intent(in)        :: mf

      integer(long)                     :: j,i,ierr
      integer(long), dimension(1)       :: idummy
      real(dop)                         :: time,dt,ufs
      complex(dop), dimension(1)        :: zdummy
      logical(kind=4)                   :: lread,lwrite
      character(len=10)                 :: stat


      routine='Receiver (joinpsi)'

! --- OPEN NEW PSI FILE
      if (lovwr) then
         stat='unknown'
      else
         stat='new'
      endif
      open(ipsi,file=psifile(1:lepsi),form='unformatted',&
           status=stat,err=999)

! --- WRITE INFORMATION TO THE NEW PSI FILE
      tinit=t0(1)
      write(ipsi) progver
      call wrpsiinfo(ipsi)

! --- READ PSI FILES, EXTEND THEM TO THE NEW SET OF SPF'S AND WRITE TO THE 
! --- NEW PSI FILE
      do j=1,filenum
         i=jnum(j)
         time=t0(i)
         lread=.true.
         do while(lread)
            if (j.ne.filenum) then
               dt=time-out2-t0(jnum(j+1))
               if (dt.gt.tiny) then
                  ufs=1.0_dop
                  call keyunits(ufs,'fs',ierr)
                  write(6,'(a,f8.3,a,f8.3,a)') 'Error: Psi data from ',&
                          t0(jnum(j+1))/ufs,' fs   to ', &
                          (time-out2)/ufs,' fs overlapes.'
                  write(ilog,'(/a,f8.3,a,f8.3,a/)') &
                          'Error: Psi data from ',&
                          t0(jnum(j+1))/ufs,' fs   to ',&
                          (time-out2)/ufs,' fs overlapes.'
                  stop
               endif
            endif
            call runjpsi(mc(mcpsi),mc(mcpsi1),ms(mspsi),dgldim1(i),&
                         ipsij(i),i,lread,mc(mcworkc))
            if (lread) then
                lwrite=.true.
                if (j.ne.filenum) then
                    dt=time-t0(jnum(j+1))
                    if (abs(dt).lt.tiny) lwrite=.false.
                endif
                if (lwrite) then
                   call psiout(mc(mcpsi),zdummy,time,idummy,zdummy,idummy,zdummy)
                endif
                time=time+out2
            else
                if (j.ne.filenum .and. abs(dt).gt.tiny) then
                   ufs=1.0_dop
                   call keyunits(ufs,'fs',ierr)
                   write(6,'(a,f8.3,a,f8.3,a)') &
                           'Warning: Psi data from ',&
                           (time-out2)/ufs,' fs   to ',&
                           t0(jnum(j+1))/ufs,' fs are missing.'
                   write(ilog,'(/a,f8.3,a,f8.3,a/)') &
                           'Warning: Psi data from ',&
                           (time-out2)/ufs,' fs   to ',&
                           t0(jnum(j+1))/ufs,' fs are missing.'
                endif
            endif
         enddo
         close(ipsij(i))
      enddo

      close(ipsi)
      return

  999 write(6,*) 'Error opening file: ',psifile(1:lepsi)

      return
      end subroutine receiver


!#######################################################################
!  Subroutine runjpsi
!
!  Completes wave function to another set of nspfs with zeros.
!#######################################################################

      subroutine runjpsi(psi,psi1,spsi1,dgldim2,ipsi2,i,lread,workc)

use global
use aglobal
use maxv, only: maxdim,maxsta
use griddatmod
use psidef
use channels
use datenmod
use joinpsimod

use getpsimod, only: akcopy

      implicit none

      integer(long)             :: s,m,spfdim1,spfdim2,g,&
                                   zeig,zeig1,zeig2
      integer(long), intent(in) :: i,dgldim2,ipsi2

      complex(sip), dimension(dgldim2), intent(out) :: spsi1
      complex(dop), dimension(dgldim), intent(out)  :: psi
      complex(dop), dimension(dgldim2), intent(out) :: psi1
      complex(dop), dimension(workcdim), intent(out):: workc
      logical(kind=4)                               :: lpsisp
      logical(kind=4), intent(out)                  :: lread


      lpsisp=lpsiopt(1)
!..........read psi file
      if (lpsisp) then
         read (ipsi2,end=10,err=30) (spsi1(g),g=1,dgldim2)
         do g=1,dgldim2
            psi1(g)=dcmplx(spsi1(g))
         enddo
      else
         read (ipsi2,end=10,err=30) (psi1(g),g=1,dgldim2)
      endif

!........extend it to another set of nspf's
      do s=1,nstate
         zeig=1
         zeig1=zeig+block(s)
         call cpvxz(psi1(zpsi1(s,i)),workc(zeig),block1(s,i))
         do m=1,nmode
!..............extend coefficients
            call akcopy(workc(zeig),workc(zeig1),vdim1(m,s,i),&
                        dim1(m,s,i),dim(m,s),ndim(m,s))
            zeig2=zeig
            zeig=zeig1
            zeig1=zeig2

!...............extend spf's
            spfdim1=subdim(m)*dim1(m,s,i)
            spfdim2=subdim(m)*(dim(m,s)-dim1(m,s,i))
            call cpvxz(psi1(zetf1(m,s,i)),psi(zetf(m,s)),&
                       spfdim1)
            if (spfdim2.ne.0) &
               call zerovxz(psi(zetf(m,s)+spfdim1),spfdim2) 
         enddo
         call cpvxz(workc(zeig),psi(zpsi(s)),block(s))
      enddo
      lread=.true.
      return

 30   write(6,'(/2a)') ' Error reading file: ', cpsi1(i)(1:lepsi1(i))
      stop

 10   lread=.false.
      return

      end subroutine runjpsi



!#######################################################################
!  Subroutine Joininp
!
!  Read psi files to be joined
!#######################################################################
 
      subroutine joininp

use global
use aglobal
use maxv, only: maxdim,maxsta
use griddatmod
use psidef
use channels
use datenmod
use joinpsimod
 
      implicit none

      integer(long)                           :: m,j,s,chkdvr,chkgrd,chkpsi,&
                                                 chkdat,i,ierr
      integer(long), dimension(listdim)       :: d1,n1
      logical(kind=4)                         :: lerr
      character(len=c5)                       :: filename


      routine='Joininp'

! --- READ THE COMAND LINE
      call joincmd

! --- READ DATA FROM ALL PSI FILES

      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1

      do j=1,filenum
         if (j.gt.1) then
            chkdvr=2
            chkgrd=2
            chkpsi=3
            chkdat=3
         endif
         ipsij(j)=10+j
         filename=cpsi1(j)(1:lepsi1(j))
         open(ipsij(j),file=filename,form='unformatted',status='old',&
              err=999)

         read(ipsij(j)) filever(ipsij(j))
         call rdpsiinfo(ipsij(j),chkdvr,chkgrd,chkpsi,chkdat)
         call chksyserr(filename,lerr)
         if (lerr .or. dentype.eq.1) go to 999

         do s=1,nstate
            do m=1,nmode
               dim1(m,s,j)=dim(m,s)
               vdim1(m,s,j)=vdim(m,s)
               ndim1(m,s,j)=ndim(m,s)
               zetf1(m,s,j)=zetf(m,s)
            enddo
            block1(s,j)=block(s)
            zpsi1(s,j)=zpsi(s)
         enddo
         dgldim1(j)=dgldim
         t0(j)=tinit
         jnum(j)=j
      enddo

! --- CALCULATE DIMENSION OF WHOLE PSI VECTOR

      call sortupr(t0,jnum,filenum)
      do s=1,nstate
         do m=1,nmode
            do j=1,filenum
               d1(j)=dim1(m,s,j)
               n1(j)=j
            enddo
            call sortdowni(d1,n1,filenum)
            if (.not. lnf) then
               dim(m,s)=d1(1)
            else
               if (nf(m).lt.d1(1)) then
                  write(6,'(a,i3,a,i4)')&
                       'Error: increase the nspf"s for mode ',m,&
                         ' up to ',d1(1)
                  stop
               endif
               dim(m,s)=nf(m)
            endif
         enddo
      enddo
      call psidat


! --- OPEN AND WRITE SOME INFORMATION TO LOG-FILE

      filename=logfile(1:llog)
      open(ilog,file=filename,form='formatted',iostat=ierr)
      logisopen=.true.
      if (ierr.ne.0) then
          message='Cannot open file '//filename
          call errormsg
      endif
      write(ilog,'(80a1)') ('*',j=1,80)
      write(ilog,'(2a/)')   ' Options : ', options(1:optlaenge)
      write(ilog,'(/a/)') 'The following psi files will be joined :'
      if (lpr) then
         write(6,'(/,80a1)') ('-',j=1,80)
         write(6,'(/a/)') 'The following psi files will be joined :'
      endif
      do j=1,filenum
         i=jnum(j)
         write(ilog,*) '  ',cpsi1(i)(1:lepsi1(i)),' ,  nspf"s =  ',&
                      ((dim1(m,s,i),m=1,nmode),s=1,nstate)
         if (lpr) then
            write(6,*) '  ',cpsi1(i)(1:lepsi1(i)),' ,  nspf"s =  ',&
                          ((dim1(m,s,i),m=1,nmode),s=1,nstate)
         endif
      enddo
      write(ilog,'(/a/)')   ' Output psi file : '
      write(ilog,*) psifile(1:lepsi),' ,  nspf"s =  ',&
                    ((dim(m,s),m=1,nmode),s=1,nstate)
      write(ilog,'(/,80a1,/)') ('-',j=1,80)
      if (lpr) then
          write(6,'(/a/)')   ' Output psi file : '
          write(6,*) psifile(1:lepsi),' ,  nspf"s =  ',&
                        ((dim(m,s),m=1,nmode),s=1,nstate)
          write(6,'(/,80a1)') ('-',j=1,80)
      endif
      return

  999 continue
      write(6,'(a)') 'Error reading file :'//filename
      stop

      end subroutine joininp


!#######################################################################
!  Subroutine joincmd
!
!  Read the comand line.
!#######################################################################
 
      subroutine joincmd

use global
use aglobal
use maxv, only: maxdim,maxsta
use griddatmod
use psidef
use channels
use datenmod
use joinpsimod
 
      implicit none

      integer(long), external  :: myiargc
      integer(long)            :: ilbl,length,i,j,mf,n1,iarg,ifile,ierr
      logical(kind=4)          :: lexist
      character(len=240)       :: buf


      routine='joincmd'

! --- SET DEFAULT VALUES ---
      name='./'
      laenge=2
      filelist=' '
      llist=0
      logfile = 'jpsi.log'
      llog=8
      psifile='psi'
      lepsi=3
      lfile=.false.
      lovwr=.false.
      filenum=0
      lnf=.false.
      lpr=.false.

      iarg  = myiargc()
      if (iarg .le. 0) goto 200
      call mygetarg(0,options)
      optlaenge  = index(options,' ')
      do i = 1, iarg 
         call mygetarg(i,buf)
         ilbl  = index(buf,' ')
         ilbl = min(ilbl,100-optlaenge)
         if(ilbl.gt.0) options = options(1:optlaenge)//buf(1:ilbl)
         optlaenge = optlaenge + ilbl
      end do

! --- READ OPTIONS ---
 200  continue
      i=0
      j=0
 10   continue
      i=i+1
      call mygetarg(i,buf)
 
      if (buf(1:1) .eq. '-') then
         if (buf(2:3) .eq. 'l ') then
            lfile=.true.
            i=i+1
            call mygetarg(i,buf)
            length=index(buf,' ')-1
            filelist=buf(1:length)
            llist=length
            call abspath(filelist,llist)
         elseif (buf(2:3) .eq. 'o ') then
            i=i+1
            call mygetarg(i,buf)
            length=index(buf,' ')-1
            read(buf(1:length),'(a)') psifile
            lepsi = length
         elseif (buf(2:3) .eq. 'D ') then
            i=i+1
            call mygetarg(i,buf)
            laenge=index(buf,' ')-1
            read(buf(1:laenge),'(a)') name
            call abspath(name,laenge)
         elseif (buf(2:3) .eq. 'f ') then
            lnf=.true.
            mf=0
 11         i=i+1
            call mygetarg(i,buf)
            laenge=index(buf,' ')-1
            read(buf(1:laenge),'(i3)',iostat=ierr) n1
            if (ierr.eq.0 .and. buf(1:1).ne.' ') then
               mf=mf+1
               nf(mf)=n1
               go to 11
            else
               i=i-1
            endif
         elseif (buf(2:3) .eq. 'p ') then
            lpr=.true.
         elseif (buf(2:3) .eq. 'w ') then
            lovwr=.true.
         elseif (buf(2:4) .eq. 'ver') then
            call wrversion(6)
            stop
         elseif (buf(2:3) .eq. 'h ' .or. buf(2:3) .eq. '? ') then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(2x,a)')&
              'Purpose : Join psi files with different nspf"s.'
            write(6,'(2x,2a)')&
              'Usage       : joinpsi [-w -D -o -l -f -p -ver -h -?] ',&
               'file1 file2 ...'
            write(6,'(/,a)') ' Options      : '
            write(6,'(2a)')  ' -w           : Overwrite existing',&
                             ' output file.'
            write(6,'(a/,16x,a)')&
                     ' -D PATH      : The output is directed to PATH.',&
            'The string PATH may be a relative or a full path-name.'
            write(6,'(2a)') ' -o NAME      : Change the name of output',&
                     ' file from "psi" to NAME.'
            write(6,'(2a)')  ' -l LIST      : The list if psi files ',&
                   'is read from file LIST, rather from the input line.'
            write(6,'(2a/,16x,2a)')  ' -f n1 n2 ... : Force the number',&
                   ' of spf"s to be equal to n1,n2,... .',&
                   'Otherwise the maximum number from psi files will ',&
                   'be chosen.'
            write(6,'(2a)') ' -p           : Print information to the ',&
                           'screen.'
            write(6,'(2a)')&
                    ' -ver         : Version information about the ',&
                    'program '
            write(6,'(a,/a)')&
                       ' -h           : Print this help text. ',&
                       ' -?           : Print this help text. '
            write(6,'(/1x,2a,/2x,a/)') 'Examples     : joinpsi ',&
                      '../dir1/psi ../dir2/psi ../dir3/psi ',&
                    '              joinpsi -w -f 30 40 20 -l list_psi'
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else
            write(6,'(2a)') ' unknown option : ', buf
            stop
         endif
      else
         call mygetarg(i,buf)
         length=index(buf,' ')-1
         if (length.eq.0 .and. .not.lfile) then
            message='No input files are given.'
            call errormsg
         endif
         j=j+1
         cpsi1(j)=buf(1:length)
         lepsi1(j)=length
         call abspath(cpsi1(j),lepsi1(j))
      endif
      filenum=j

      if (i .lt. iarg) go to 10

      if (lfile) then
         ifile=10
         open(ifile,file=filelist(1:llist),status='old',iostat=ierr)
         if (ierr.ne.0) then
            message='Could not open input file '//filelist(1:llist)
            call errormsg
         endif
         j=1
  30     read(ifile,'(a)',err=999,end=40) cpsi1(j)
         lepsi1(j)=index(cpsi1(j),' ')-1
         call abspath(cpsi1(j),lepsi1(j))
         j=j+1
         goto 30

  40     filenum=j-1
         close(ifile)
      endif

! --- CHECK PARAMETERS ---

      laenge=index(name,' ')-1
      call abspath(name,laenge)
      if (name(laenge:laenge) .ne. '/') name=name(1:laenge)//'/'
      laenge=laenge+1
      psifile=name(1:laenge)//psifile(1:lepsi)
      lepsi=lepsi+laenge
      if (.not.lovwr) then
         inquire(file=psifile(1:lepsi),exist=lexist)
         if (lexist) then
            write(6,'(/a)') ' One may use -w  option.'
            message='File already exists: '//psifile(1:lepsi)
            call errormsg
         endif
      endif

      if (filenum.eq.0) then
         message='No psi files given.'
         call errormsg
      endif

      return
 999  continue
      message='Error reading file '//filelist(1:llist)
      call errormsg

      end subroutine joincmd


      end program joinpsi

