!***********************************************************************
!
!                        PROBRW
!
! This program can be used for calculating reaction probability from 
! flux, which is computed while propagation of the real wavepacket.
! Read flux matrix directly then calculate reaction probability.
!
! (AJHMM 12/12)
!***********************************************************************
      
     program probrw

use decimal, only: long,dop!,sip
use realwpmod

      implicit none

      integer(long)                     :: f,dim1,dim2,dim3
      integer(long)                     :: ifluxf,iprobf,iuprobf

      common / iunit / ifluxf,iprobf,iuprobf


!-----------------------------------------------------------------------
!     Set defaults. Set up the input.
!-----------------------------------------------------------------------
      call default
      call adefault

      call probrwinp(f,dim1,dim2,dim3)

!-----------------------------------------------------------------------
! Call for calculating probability
!-----------------------------------------------------------------------
      call probpsi(f,dim1,dim2,dim3)

      stop
      contains

! **********************************************************************
!
! This subroutine calculates the E-dependent reaction probability
! and the E-dependent non-reaction probability
! Meijer et al CPL 293, 270-276(1998) 
! **********************************************************************
      subroutine probpsi(f,dim1,dim2,dim3)

use global
use griddatmod, only:mass,modelabel
use channels

use realwpmod

      implicit none

 integer(long)                              :: i,j,k,l,m,half,dimr,itel,itel2,itel3,&
                                               ierr,iistp,ilbl,ios
 integer(long), intent(in)                  :: dim1,dim2,dim3,f

 real(dop), parameter                       :: zero=0.0_dop
 real(dop), external                        :: ddot
 real(dop)                                  :: de,energy,erovib,avar,bvar,ener,ener2,&
                                               mscat,test1,tmp,enersca,test3,k0,fac1,&
                                               k1,temp2,dt2
 real(dop), dimension(0:prne-1)             :: pp,pp4,pp6,pp7
 real(dop), dimension(dim1*dim3,250)        :: wavr,dwavr
 real(dop), dimension(dim1*dim3,0:prne-1)   :: rwav,drwav,rwavi,drwavi

 complex(dop), dimension(0:prne-1,2)        :: enersave
 character(len=8)                           :: stat
 integer(long)                              :: ifluxf,iprobf,iuprobf

      common / iunit / ifluxf,iprobf,iuprobf

      dimr=dim1*dim3
      k0=inimom
      mscat=mass(1)
      erovib=energsav-escatsav
      half=ntimfluxo/2
      if (prne > ntimfluxo) then
          prne=ntimfluxo
          write (ilog,*) 'Number of energy points set to maximum of ',prne
      endif
      de=(femax-femin)/(prne-1)
      do j=0,prne-1
         energy=j*de+femin
         energy=-dacos(aunds*energy+bunds)
         enersave(j,1)=cdexp((0.0_dop,1.0_dop)*energy*fluxsave)
         enersave(j,2)=enersave(j,1)
!         do i=1,max0(dimreac(nomega+1),dimnreac(nomega+1)) ! why here max0???
         do i=1,dimr
            rwav(i,j)=zero
            rwavi(i,j)=zero
            drwav(i,j)=zero
            drwavi(i,j)=zero
         enddo
      enddo

     do j=1,ntimfluxo
        if ((((j-1)/250)*250) == (j-1)) then
!           if (myid.eq.0) then
              write (*,*) j-1
              call flush(6)


            itel=1
            itel2=1
            do l=j,min0(ntimfluxo,j+249)
               iistp=2
               read(ifluxf,err=1500,end=1600,iostat=ierr)&
                   test1,(dwavr(m,itel),m=itel2,itel2+dimr-1)

               iistp=3
               read(ifluxf,err=1500,end=1600,iostat=ierr)&
                   test1,(wavr(m,itel),m=itel2,itel2+dimr-1)

               goto 1001

1500           write (*,*) 'Error reading file'
               write (*,*) 'Iostat =',ierr
               write (*,*) 'Stop code =',iistp
!               write (*,*) 'Myid =',myid
               call flush(6)
               exit

1600           write (*,*) 'Unexpected end of file'
               write (*,*) 'Iostat =',ierr
               write (*,*) 'Stop code =',iistp
!               write (*,*) 'Myid =',myid
               call flush(6)
               exit

1001           itel=itel+1
            enddo

            itel=1
         endif

         do i=0,prne-1
            avar=dreal(enersave(i,1))*fluxsave
            bvar=dimag(enersave(i,1))*fluxsave
            call daxpy(dimr,avar,wavr(1,itel),1,rwav(1,i),1)
            call daxpy(dimr,bvar,wavr(1,itel),1,rwavi(1,i),1)
            call daxpy(dimr,avar,dwavr(1,itel),1,drwav(1,i),1)
            call daxpy(dimr,bvar,dwavr(1,itel),1,drwavi(1,i),1)
            enersave(i,1)=enersave(i,1)*enersave(i,2)
         enddo
         itel=itel+1
      enddo


      itel=1
      itel2=0
         do i=0,prne-1
            pp(itel2)=&
              ddot(dimr,rwav(itel,i),1,drwavi(itel,i),1)-&
              ddot(dimr,rwavi(itel,i),1,drwav(itel,i),1)
            itel2=itel2+1
         enddo
      write (ilog,*) ' (Non)Reaction probability finished'

!######################################

      dt2=4.0_dop*aunds**2
!      k0=dabs(inimom)
!
! Test the accuracy of the Fourier transform
!
      itel=0
      test1=zero
      do i=0,prne-1
         test1=test1+pp(itel)*de
         itel=itel+1
      enddo
!      write (ilog,'(a,i2,1x,f12.6)') 'The total energy-dependent (non)reactive flux',&
      write (ilog,*) 'The total energy-dependent (non)reactive flux',&
                  ' integrated over energy',f,(test1/normwav**2)*100.0_dop,' %.'
      write (*,*) ' '

!######################################

      itel=0
      itel2=0
      itel3=0
      do i=0,prne-1
         ener=i*de+femin-erovib
         ener2=ener+erovib
         enersca=1.0_dop/(1.0_dop-(aunds*ener2+bunds)**2)
         if (ener < zero) goto 1000
            k1=dsqrt(2.0_dop*mscat*ener)
            fac1=dsqrt(k1)/((8.0_dop*pi*spread)**(0.25_dop)*dsqrt(mscat))
            temp2=fac1/dexp(-spread*(k1-k0)**2)
            temp2=temp2**2
            tmp=pp(itel3)*temp2*dt2*enersca
            pp4(itel)=tmp
            pp6(itel)=temp2
            pp7(itel)=ener2*ev
            itel=itel+1
1000        itel3=itel3+1
      enddo

      itel=itel-1

      do k=itel2,itel
         write (iprobf,*) pp7(k),pp4(k)
         write (iuprobf,*) pp7(k),pp4(k)/pp6(k)
      enddo

      close(iprobf)
      close(iuprobf)
      close(ifluxf)

      return

10000  write (*,*) ' Iostat = ',ios
      write (*,*) ' Error in probreal'
      write (*,*) 'Stop code =',iistp
      stop

      end subroutine probpsi

!-----------------------------------------------------------------------
      subroutine probrwinp(fflux,vd,gd,nd)

use global, only: name,message,operfile,logisopen,laenge,lovwr
use operdef
use channels, only: irst,ilog,ioper
use iorst, only: rstinforw
use aglobal, only: fileout,filein,laeout,laein
use fluxmod, only: unit,unite,unitf,loper
use griddatmod, only: ndof,nmode,nspfdof,spfdof,modelabel,dofspf
use rdopermod, only: operinfo
use logdat, only: errormsg
use operutil, only: getvgn

use realwpmod

      implicit none

      integer(long)                 :: f,m,n,k,h,iarg,myiargc,i,ierr,chkdvr,chkgrd,&
                                       chkpsi,chkprp,mflux,hflux,ilbl
      integer(long), intent(out)    :: fflux,vd,gd,nd
      character(len=80)             :: buf
      character(len=16)             :: fluxdof
      logical(kind=4)               :: linwf,lerr
      character(len=240)            :: logfile,rstfile,fluxfile,probfile,uprobfile
      integer(long)                 :: ifluxf,iprobf,iuprobf
      character(len=10)             :: stat

      common / iunit / ifluxf,iprobf,iuprobf
!-----------------------------------------------------------------------

      filein  = 'restart'
      fileout = 'probrw.log'
      operfile= 'oper'
      fluxdof = '  '
      unit    = 'au'
      unite   = 'au'
      unitf   = 'au'
      loper   = .false.

      iarg = myiargc()
      if (iarg < 1) goto 111
      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) == '-' ) then
         if(buf(1:2) == '-w') then
            lovwr = .true.
         else if(buf(1:2) == '-d') then
            go to 111
         else if( buf(1:2) == '-h' .or. buf(1:2) == '-?' ) then
            write(6,'(/,a)')&
       'Purpose: Computation of the Reaction Probability.'

            write(6,'(/,2a,/19x,a)')&
       'Usage : probrw<ver> [-w -d -ver -h -?]'
            write(6,'(/,a)') 'Options : '
            write(6,'(2a,/11x,a)') &
            ' -w      : The (un)prob_{dof} file may be overwritten.' 
            write(6,'(2a,/11x,a)') &
            '-d dof   : Probability along the dof.'
            write(6,'(a)')&
            ' -ver    : Version information about the program '
            write(6,'(a,/a)') &
            ' -h      : Print this help text. ',&
            ' -?      : Print this help text. '
            write(6,'(a,/a,/a)') &
                       ' Example: probrw<ver> -d rd'
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) == '-ver' ) then
            call wrversion(6)
            stop
         end if
         if (iarg <= i) goto 111
         goto 100
      end if

111   continue
      i=i+1
      if (iarg < i) goto 300
      call mygetarg(i,fluxdof)

!-----------------------------------------------------------------------
!  Open the Log-file.
!-----------------------------------------------------------------------
         laeout=index(fileout,' ')-1
         logfile=fileout(1:laeout)//'/probrw.log'
         call abspath(logfile,laeout)
         open (ilog,file=logfile(1:laeout),form='formatted',iostat=ierr)
         if (ierr/=0) then
          message='Could not open input file :'//logfile(1:laeout)
          call errormsg
         endif

         if (.not.lovwr) then
 54         read(ilog,'(a1)',end=55,err=55) buf
            goto 54
 55         backspace ilog
            write(ilog,'(//,80a1,/)') ('*',i=1,80)
         else
            endfile (ilog)
            backspace (ilog)
         end if
         logisopen=.true.
         call fpstartup(logfile)
         write(ilog,'(a80)') logfile
         call wrversion(ilog)

!-----------------------------------------------------------------------
! read data from restart file  (lrst=true --> read restart rathen than psi)
!-----------------------------------------------------------------------
         laein=index(filein,' ')-1
         rstfile=filein(1:laein)//'/restart'
         open(irst,file=filein(1:laein),form='unformatted',status='old',iostat=ierr)
         if (ierr/=0) then
            message='Could not open input file :'//filein(1:laein)
            call errormsg
         endif
         chkdvr=1
         chkgrd=1
         chkpsi=1
         chkprp=1
         call rstinforw(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         if (linwf) write(6,'(a)') ' Initial wave function'
         if (lerr) then
            write(6,'(a)') ' ERROR reading restart file'
            stop
         end if
!-----------------------------------------------------------------------
! Determine the dof of the flux
!-----------------------------------------------------------------------
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (fluxdof == modelabel(f) ) then
                  mflux = m
                  fflux = f
               end if
            enddo
         enddo

      ierr = 0
      if (fflux .eq. 0 ) read(fluxdof,*,iostat=ierr) fflux
      if (fluxdof .ne. '0' .and. (fflux .eq. 0 .or. ierr .ne. 0) ) then
         write(6,'(/a)') &
         'Cannot determine the degree of freedom for the Flux.'
         write(6,'(a,a12)') 'Illegal input reads : ', fluxdof
         stop
      end if
      if( fflux .lt. 0 ) then
         write(6,'(/a,a,i6)') &
        'Illegal degree of freedom for the Flux specified.',&
        '   fflux =',fflux
         stop
      elseif( fflux .gt. ndof ) then
         hflux = fflux
         fflux = htmdof(hflux)
         mflux = dofspf(fflux)
         write(6,'(/a,i6)') &
         ' h-term rather then f-flux assumed. h =', hflux
      else
         write(6,*)
      end if
      if( fflux .gt. 0 ) mflux = dofspf(fflux)
      if( fflux.eq.0) then
         write(6,'(/a,/a)') &
        ' No degree of freedom for the Flux specified.',&
        ' The first Flux found on the oper file will be taken.'
      else
         write(6,'(a,i3,a,i3,a,3x,a)') &
        ' flux-mode =', mflux,',  flux-dof =',fflux,',', modelabel(fflux)
      end if

!-----------------------------------------------------------------------
!  Opening the oper file.
!-----------------------------------------------------------------------
      operfile=operfile
      open(ioper,file=operfile,form='unformatted',status='old',iostat=ierr)
      if (ierr/=0) then
         message='Could not open input file :'//operfile
         call errormsg
      endif
      write(ilog,'(a)') ' The operator file is :'//operfile
      chkdvr=1
      chkgrd=1
      call operinfo(lerr,chkdvr,chkgrd)
      if (lerr) then
          message=' Error reading :'//operfile
          call errormsg
      endif
      close(ioper)

!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
!-----------------------------------------------------------------------
      ifluxf  = 10+fflux
      iprobf  = 20+fflux
      iuprobf = 30+fflux
      if (lovwr) then
         stat='unknown'
      else
         stat='new'
      endif
      do k=1,ktot
         if (autoflux(k) > 0) then
            f = autoflux(k)
            if (f == fflux) then
               h = hamilton(f,k)
               call getvgn(h,vd,gd,nd)

               ilbl=index(modelabel(f),' ')-1
               fluxfile=name(1:laenge)//'/uflux_'//modelabel(f)(1:ilbl)
               open(ifluxf,file=fluxfile, &
               form='unformatted',iostat=ierr,status='old')
               if (ierr/=0) then
                  message='Could not open input file :'//fluxfile
                  call errormsg
               endif

               probfile=name(1:laenge)//'/prob_'//modelabel(f)(1:ilbl)
               open(iprobf,file=probfile,form='formatted',status=stat,iostat=ierr)
               if (ierr/=0) then
                  message='Could not open input file :'//probfile
                  call errormsg
               endif
               write(ilog,'(a)') 'File opened :'//probfile

               uprobfile=name(1:laenge)//'/unprob_'//modelabel(f)(1:ilbl)
               open(iuprobf,file=uprobfile,form='formatted',status=stat,iostat=ierr)
               if (ierr/=0) then
                  message='Could not open input file :'//uprobfile
                  call errormsg
               endif
               write(ilog,'(a)') 'File opened :'//uprobfile
            endif
         endif
      enddo

      return

!-----------------------------------------------------------------------
!  Prompting for missing input 
!-----------------------------------------------------------------------

 300  if( fluxdof .ne. '  ' ) goto 400
      write(6,'(2a)') ' flux-dof = ? ',&
                      ' (give a number or a modelabel).'
      write(6,'(2a)') ' 0 -> search for dof of flux.',&
                      ' (useful, if there is only one flux).'
      read(5,'(a)') fluxdof
      if( fluxdof(1:9) .eq. '         ' ) fluxdof = '0'

!-----------------------------------------------------------------------

 400  return


      end subroutine probrwinp

!#######################################################################


      end program probrw
