!***********************************************************************
!
!                      AUSWERT routine RDSPF
!
!     This routine prints the SPF part of the psi-vector.
!     In effect a slightly modified form of RDACOEFF.
!
!     06/08 GWR, 11/19 HDM, 05/21 HDM
!
!
!***********************************************************************

   program rdspf

   implicit none

#include "global.inc"
#include "aglobal.inc"
#include "maxdim.inc"
#include "maxsta.inc"
#include "griddat.inc"
#include "psidef.inc"
#include "daten.inc"
#include "overlap.inc"
#include "channels.inc"


   integer           ::  s,ilbl,chkdvr,chkgrd,chkpsi,chkdat,chkprp
   character(len=c5) :: filename
   logical           :: lerr,linwf

   complex*16, dimension(:), allocatable :: mc
   real*8,     dimension(:), allocatable :: mr
   integer,    dimension(:), allocatable :: mi
   complex*8,  dimension(:), allocatable :: ms

!-----------------------------------------------------------------------
! Initialize variables
!-----------------------------------------------------------------------
   call default
   call adefault
   call rdspfinput

!-----------------------------------------------------------------------
! Open log file channel
!-----------------------------------------------------------------------
   open (unit=ilog,file='rdspf.log',form='formatted')
   logisopen=.true.

   if( ebtype .lt. 0 ) then
!-----------------------------------------------------------------------
! read data from restart file  (ebtype = -1  -> restart rather than psi)
!-----------------------------------------------------------------------
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkprp=1
      call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
      if (lerr) then
         write(6,'(a)') ' ERROR reading restart file'
         stop
      end if
   else
!-----------------------------------------------------------------------
! read data from psi file
!-----------------------------------------------------------------------
      read(ipsi) filever(ipsi)
      chkdvr=1
      chkgrd=1
      chkpsi=1
      chkdat=1
      call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
      filename='psi'
      call chksyserr(filename,lerr)
      if (lerr) go to 99
   endif

   if (nmode .eq. 1) then
      do s=1,nstate
         dim(1,s)=1
      enddo
   endif

!-----------------------------------------------------------------------
! read information from dvr file
!-----------------------------------------------------------------------
   if (lrddvr) then
      chkdvr=2
      filename=name(1:laenge)//'/dvr'
      open(idvr,file=filename,form='unformatted',status='old',err=99)
      call dvrinfo(lerr,chkdvr)
      close(idvr)
      call chksyserr(filename,lerr)
      if (lerr) then
         write(6,'(a)') ' ERROR reading dvr file: ',trim(filename)
         stop
      endif
   endif

!-----------------------------------------------------------------------
! assign pointers and sizes for required arrays
!-----------------------------------------------------------------------
   lpsi=.true.
   lpsigrd=.false.
   lort=lrddvr
   call zeigausw
   call griddat
   call psidat
   call memausw

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
   call wrversion(ilog)
   ilbl = 16*mcdim + 8*mrdim + 4*midim + 8*msdim
   write(ilog,'( 4(a,i10,/),a,i13,a,f10.2,a)') &
    ' complex*16 : ', mcdim,&
    ' complex*8  : ', msdim,&
    ' real*8     : ', mrdim,&
    ' integer    : ', midim,&
    ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0d0,' MB'
   call myflush(ilog)

   if(allocated(mc))deallocate(mc)
   if(allocated(mr))deallocate(mr)
   if(allocated(mi))deallocate(mi)
   if(allocated(ms))deallocate(ms)
   allocate(mc(mcdim))
   allocate(mr(mrdim))
   allocate(mi(midim))
   allocate(ms(msdim))

   call rdspfc(mc(mcpsi),ms(mspsi),mi(mijindx),mr(mrort))

   deallocate(mc,mi,ms)

   stop
99 continue
   write(6,'(a)') 'Error reading :'//trim(filename)
   stop

   end program rdspf


!***********************************************************************
!    Does the work
!***********************************************************************

   subroutine rdspfc(psi,spsi,jindx,ort)

   implicit none

#include "global.inc"
#include "aglobal.inc"
#include "maxdim.inc"
#include "maxsta.inc"
#include "griddat.inc"
#include "psidef.inc"
#include "daten.inc"
#include "overlap.inc"
#include "channels.inc"

   integer, dimension(adim), intent(out)        :: jindx
   integer, dimension(maxdim)                   :: fii
   integer                                      :: i,k,m,n,s,ss,mm,m1,m2,f1,f2,chkdvr
   integer                                      :: l,ldx,s1,s2,ndofm,lenspf,idum
   integer                                      :: absi,cnt,idxspf,idxgrd,dgl,nspf
   complex*8, dimension(spsidim), intent(out)   :: spsi
   complex*16, dimension(dgldim), intent(out)   :: psi
   complex*16                                   :: zdum
   real*8, dimension(ortdim)                    :: ort
   real*8                                       :: time,dt,thr,tm,atm,mx,rdum
   logical                                      :: lcnt,ldblout,lplot,lsurf
   character(len=6), dimension(5)               :: ctype
   character(len=240)                           :: gnucmd
   character(len=80)                            :: lpr

   common /rdspfin/ mx,thr,tm,atm,ss,mm,nspf,absi,lcnt,ldblout,lplot,lsurf,lpr
   data  ctype /' cmplx','  abs ',' abs^2','  Re  ','  Im  '/

!-----------------------------------------------------------------------
! calculate time step for data
!-----------------------------------------------------------------------
   dt=out2/fs
   time=tinit/fs
   tm=tm+dt*1.d-5
   atm=atm-dt*1.d-5
   k=0

   if(lplot) then ! set default values for plotting
      if(mm.eq.0) mm=1
      if(ss.eq.0) ss=1
      if(nspf.eq.0) nspf=1
      if(absi.eq.0) then
         write(6,'(a)') 'Complex not allowed for plotting.',&
            'Use option: -abs, -abs2, -re, or -im'
         stop
      endif
      f1=spfdof(1,mm)
      f2=spfdof(2,mm)
   endif
   if(ss.gt.0) then
      if(ss.gt.nstate) then
         write(6,'(a,i3,a,i3,a)') 'Illegal value for s =', ss, &
               ' , (nstate =',nstate,')'
         stop
      endif
      s1=ss
      s2=ss
   else
      s1 = 1
      s2 = nstate
      if(nstate.eq.1) ss=1
   endif
   if(mm.gt.0) then
      if(mm.gt.nmode) then
         write(6,'(a,i3,a,i3,a)') 'Illegal value for m =', mm, &
               ' , (nmode =',nmode,')'
         stop
      endif
      m1=mm
      m2=mm
   else
      m1=1
      m2=nmode
   endif
   if(lplot) then
      if(nspfdof(mm).ne.2) then
         write(6,'(/a,/a,/a,i4,a,i4,/a/)') &
            ' ERROR: rdspf84 can plot only 2d SPFs.',&
            ' For plotting 1D (uncombined) SPFs use showspf84.',&
            ' m =',mm,',  nspfdof(m) =',nspfdof(mm),' STOP'
         stop
      endif
      write(iaus,'(a)') 'set tics out'
      write(iaus,'(a)') 'set nogrid'
      if(lpr.eq.' ') then
         write(iaus,'(a)') 'set output'
         write(iaus,'(a)') 'set terminal x11'
      else
         write(iaus,'(a)') 'set terminal postscript'
         write(iaus,'(3a)') 'set output "',trim(lpr),'"'
      endif
      if(lsurf) then
         write(iaus,'(a)') 'set surface'
         write(iaus,'(a)') 'set view 60.00, 30.00, 1.00, 1.00'
      else
         write(iaus,'(a)') 'set nosurface'
         write(iaus,'(a)') 'set view 0,0,1,1'
      endif
      write(iaus,'(a)') 'set contour'
      write(iaus,'(a)') 'set clabel "%10.3e"'
      write(iaus,'(a)') 'set size 1.0,1.0'
      write(iaus,'(a)') 'set origin 0.0,0.0'
      write(iaus,'(a)') 'set cntrparam cubicspline'
      write(iaus,'(a)') 'set cntrparam points 5'
      write(iaus,'(a)') 'set dgrid3d'
      write(iaus,'(3a)') 'set xlabel "',trim(modelabel(f1)),'"'
      write(iaus,'(3a)') 'set ylabel "',trim(modelabel(f2)),'"'
      write(iaus,'(a)') 'set autoscale x'
      write(iaus,'(a)') 'set autoscale y'
      write(iaus,'(a)') 'set autoscale z'
      write(iaus,'(a)') 'set cntrparam levels auto 30'
   endif


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! read data from dvr file if desired
!-----------------------------------------------------------------------
   if (lrddvr) then
      dvrdata(:) = .false.
      dvrdata(1) = .true.
      chkdvr=1
      call rddvr(ort,rdum,rdum,rdum,zdum,&
           zdum,idum,zdum,zdum,idum,idum,rdum,chkdvr)
   endif



10 continue  ! Loop
!-----------------------------------------------------------------------
! ebtype = -1   --> read restart rather than psi file.
!-----------------------------------------------------------------------
   if (ebtype .lt. 0) then
!-----------------------------------------------------------------------
! read restart wavefunction
!-----------------------------------------------------------------------
      read (irst,err=98) time
      time = time/fs
      read (irst,err=98) (psi(dgl),dgl=1,dgldim)

   else
      call rdpsi(ipsi,psi,spsi,jindx)
      if (lend) goto 77
      if( mod(k,step) .ne. 0 ) goto 11
      if(time .le. atm) goto 11
   endif
   if(nspf.gt.0 .and. mm.gt.0 .and. ss.gt.0) then
      if(nspf.gt.dim(mm,ss)) then
         write(6,'(a,i3,a,i3,a)') 'Illegal value for nspf =', nspf, &
               ' , (dim(m,s) =',dim(mm,ss),')'
         stop
      endif
   endif

   if(lplot) then
      write(iaus,'(a,f10.4,a,i3,a,i3,a,i3,3a)') 'set title "Time :',time,&
         ', mode =',mm,', state =',ss,', spf =', nspf,',',ctype(absi+1),'"'
      write(iaus,'(a,f9.4,a)') 'splot "-" notitle w l; pause -1 "',time,' fs"'
   else
      write(6,'(/a)') '  mode   DOFs'
      do m=m1,m2
         write(6,'(i4,99(2x,a))') m, (trim(modelabel(spfdof(n,m))),n=1,nspfdof(m))
      enddo
      write(6,'(/a,f10.4,3a/)') '      ------- Time =',time , &
               ' fs -------         (',ctype(absi+1),')'
      if(.not.lcnt) then
         if(absi.eq.0 .and. lrddvr) then
            if(nstate.gt.1) then
               if(ldblout)then
                  write(6,'(6x,a,17x,a,26x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint,  Coordinates'
               else
                  write(6,'(6x,a,17x,a,17x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint,  Coordinates'
               endif
            else
               if(ldblout)then
                  write(6,'(6x,a,17x,a,26x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint,  Coordinates'
               else
                  write(6,'(6x,a,17x,a,17x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint,  Coordinates'
               endif
            endif ! nstate
         elseif(absi.gt.0 .and. lrddvr) then
            if(nstate.gt.1) then
               if(ldblout)then
                  write(6,'(6x,a,7x,a,11x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint,  Coordinates'
               else
                  write(6,'(6x,a,7x,a,7x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint,  Coordinates'
               endif
            else
               if(ldblout)then
                  write(6,'(6x,a,7x,a,11x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint  Coordinates'
               else
                  write(6,'(6x,a,7x,a,7x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint  Coordinates'
               endif
            endif ! nstate

         elseif(absi.eq.0 .and. .not.lrddvr) then
            if(nstate.gt.1) then
               if(ldblout)then
                  write(6,'(6x,a,17x,a,26x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint(mode/DOFs)'
               else
                  write(6,'(6x,a,17x,a,17x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint(mode/DOFs)'
               endif
            else
               if(ldblout)then
                  write(6,'(6x,a,17x,a,26x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint(mode/DOFs)'
               else
                  write(6,'(6x,a,17x,a,17x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint(mode/DOFs)'
               endif
            endif ! nstate
         else ! absi.gt.0 .and. .not.lrddvr
            if(nstate.gt.1) then
               if(ldblout)then
                  write(6,'(6x,a,7x,a,11x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint(mode/DOFs)'
               else
                  write(6,'(6x,a,7x,a,7x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint(mode/DOFs)'
               endif
            else
               if(ldblout)then
                  write(6,'(6x,a,7x,a,11x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint(mode/DOFs)'
               else
                  write(6,'(6x,a,7x,a,7x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint(mode/DOFs)'
               endif
            endif ! nstate
         endif ! absi
      write(6,'(2a)') '  --------------------------------------', &
           '--------------------------------------------------------'
      endif  ! lcnt
   endif  ! lplot

   cnt=0
   ldx=0

   do s = s1, s2

      do m = m1, m2

         if(s.ne.nstate)then
            lenspf = zetf(m,s+1) - zetf(m,s)
         else
            if(m.ne.nmode)then
               lenspf = zetf(m+1,1) - zetf(m,s)
            else
               lenspf = dgldim - zetf(m,s) + 1
            endif
         endif
         idxspf = 0
         ndofm=nspfdof(m)

         do l = zetf(m,s), zetf(m,s) + lenspf - 1

            ldx = ldx+1
            idxgrd = mod(l-zetf(m,s),subdim(m))+1
            if (idxgrd.eq.1) idxspf = idxspf + 1
            if(nspf.ne.0 .and. nspf.ne.idxspf) cycle
            call setf(m,idxgrd,fii)

            if(lplot) then  ! Ploting
               if(lrddvr) then
                  if(absi.eq.1) then
                     write(iaus,'(1p2e14.6,f18.9)') ort(zort(f1)+fii(1)),&
                                     ort(zort(f2)+fii(2)),abs(psi(l))
                  elseif(absi.eq.2) then
                     write(iaus,'(1p2e14.6,f18.9)') ort(zort(f1)+fii(1)),&
                                     ort(zort(f2)+fii(2)),abs(psi(l))**2
                  elseif(absi.eq.3) then
                     write(iaus,'(1p2e14.6,f18.9)') ort(zort(f1)+fii(1)),&
                                     ort(zort(f2)+fii(2)),dble(psi(l))
                  elseif(absi.eq.4) then
                     write(iaus,'(1p2e14.6,f18.9)') ort(zort(f1)+fii(1)),&
                                     ort(zort(f2)+fii(2)),dimag(psi(l))
                  endif
               else
                  if(absi.eq.1) then
                     write(iaus,'(2i6,f18.9)') (fii(n)+1,n=1,ndofm),abs(psi(l))
                  elseif(absi.eq.2) then
                     write(iaus,'(2i6,f18.9)') (fii(n)+1,n=1,ndofm),abs(psi(l))**2
                  elseif(absi.eq.3) then
                     write(iaus,'(2i6,f18.9)') (fii(n)+1,n=1,ndofm),dble(psi(l))
                  elseif(absi.eq.4) then
                     write(iaus,'(2i6,f18.9)') (fii(n)+1,n=1,ndofm),dimag(psi(l))
                  endif
               endif ! lrddvr
            else  ! lplot

               if( abs(psi(l)) .gt. thr .and. abs(psi(l)) .le. mx) then
                  cnt = cnt + 1
                  if(lcnt) goto 15
                  if(lrddvr) then  ! Print the coordinates
                     if(absi.eq.1) then
                        if(.not.ldblout .and. nstate.gt.1)then
                           write(6,'(i9,f21.12,2x,4i7,2x,1p22e14.6)') &
                           ldx,abs(psi(l)),s,m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(ldblout .and. nstate.gt.1)then
                           write(6,'(i9,d25.16,2x,4i7,2x,1p22e14.6)') &
                           ldx,abs(psi(l)),s,m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(.not.ldblout .and. nstate.eq.1)then
                           write(6,'(i9,f21.12,2x,3i7,2x,1p22e14.6)') &
                           ldx,abs(psi(l)),m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(ldblout .and. nstate.eq.1)then
                           write(6,'(i9,d25.16,2x,3i7,2x,1p22e14.6)') &
                           ldx,abs(psi(l)),m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        endif
                     elseif(absi.eq.2) then
                        if(.not.ldblout .and. nstate.gt.1)then
                           write(6,'(i9,f21.12,2x,4i7,2x,1p22e14.6)') &
                           ldx,abs(psi(l))**2,s,m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(ldblout .and. nstate.gt.1)then
                           write(6,'(i9,d25.16,2x,4i7,2x,1p22e14.6)') &
                           ldx,abs(psi(l))**2,s,m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(.not.ldblout .and. nstate.eq.1)then
                           write(6,'(i9,f21.12,2x,3i7,2x,1p22e14.6)') &
                           ldx,abs(psi(l))**2,m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(ldblout .and. nstate.eq.1)then
                           write(6,'(i9,d25.16,2x,3i7,2x,1p22e14.6)') &
                           ldx,abs(psi(l))**2,m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        endif
                     elseif(absi.eq.3) then
                        if(.not.ldblout .and. nstate.gt.1)then
                           write(6,'(i9,f21.12,2x,4i7,2x,1p22e14.6)') &
                           ldx,dble(psi(l)),s,m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(ldblout .and. nstate.gt.1)then
                           write(6,'(i9,d25.16,2x,4i7,2x,1p22e14.6)') &
                           ldx,dble(psi(l)),s,m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(.not.ldblout .and. nstate.eq.1)then
                           write(6,'(i9,f21.12,2x,3i7,2x,1p22e14.6)') &
                           ldx,dble(psi(l)),m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(ldblout .and. nstate.eq.1)then
                           write(6,'(i9,d25.16,2x,3i7,2x,1p22e14.6)') &
                           ldx,dble(psi(l)),m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        endif
                     elseif(absi.eq.4) then
                        if(.not.ldblout .and. nstate.gt.1)then
                           write(6,'(i9,f21.12,2x,4i7,2x,1p22e14.6)') &
                           ldx,dimag(psi(l)),s,m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(ldblout .and. nstate.gt.1)then
                           write(6,'(i9,d25.16,2x,4i7,2x,1p22e14.6)') &
                           ldx,dimag(psi(l)),s,m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(.not.ldblout .and. nstate.eq.1)then
                           write(6,'(i9,f21.12,2x,3i7,2x,1p22e14.6)') &
                           ldx,dimag(psi(l)),m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(ldblout .and. nstate.eq.1)then
                           write(6,'(i9,d25.16,2x,3i7,2x,1p22e14.6)') &
                           ldx,dimag(psi(l)),m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        endif
                     else
                        if(.not.ldblout .and. nstate.gt.1)then
                           write(6,'(i9,2f21.12,2x,4i7,2x,1p22e14.6)') &
                           ldx,psi(l),s,m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(ldblout .and. nstate.gt.1)then
                           write(6,'(i9,2d25.16,2x,4i7,2x,1p22e14.6)') &
                           ldx,psi(l),s,m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(.not.ldblout .and. nstate.eq.1)then
                           write(6,'(i9,2f21.12,2x,3i7,2x,1p22e14.6)') &
                           ldx,psi(l),m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        elseif(ldblout .and. nstate.eq.1)then
                           write(6,'(i9,2d25.16,2x,3i7,2x,1p22e14.6)') &
                           ldx,psi(l),m,idxspf,idxgrd,&
                           (ort(zort(spfdof(n,m))+fii(n)),n=1,ndofm)
                        endif
                     endif
                  else  !  lrddvr , Print the grid points
                    if(nstate.gt.1) then
                     if(absi.eq.1) then
                        if(.not.ldblout)then
                           write(6,'(i9,f21.12,2x,4i7,2x,99i4)') &
                           ldx,abs(psi(l)),s,m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        else
                           write(6,'(i9,d25.16,2x,4i7,2x,99i4)') &
                           ldx,abs(psi(l)),s,m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        endif
                     elseif(absi.eq.2) then
                        if(.not.ldblout)then
                           write(6,'(i9,f21.12,2x,4i7,2x,99i4)') &
                           ldx,abs(psi(l))**2,s,m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        else
                           write(6,'(i9,d25.16,2x,4i7,2x,99i4)') &
                           ldx,abs(psi(l))**2,s,m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        endif
                     elseif(absi.eq.3) then
                        if(.not.ldblout)then
                           write(6,'(i9,f21.12,2x,4i7,2x,99i4)') &
                           ldx,dble(psi(l)),s,m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        else
                           write(6,'(i9,d25.16,2x,4i7,2x,99i4)') &
                           ldx,dble(psi(l)),s,m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        endif
                     elseif(absi.eq.4) then
                        if(.not.ldblout)then
                           write(6,'(i9,f21.12,2x,4i7,2x,99i4)') &
                           ldx,dimag(psi(l)),s,m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        else
                           write(6,'(i9,d25.16,2x,4i7,2x,99i4)') &
                           ldx,dimag(psi(l)),s,m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        endif
                     else
                        if(.not.ldblout)then
                           write(6,'(i9,2f21.12,2x,4i7,2x,99i4)') &
                           ldx,psi(l),s,m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        else
                           write(6,'(i9,2d25.16,2x,4i7,2x,99i4)') &
                           ldx,psi(l),s,m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        endif
                     endif
                    else  ! nstate =1
                     if(absi.eq.1) then
                        if(.not.ldblout)then
                           write(6,'(i9,f21.12,2x,3i7,2x,99i4)') &
                           ldx,abs(psi(l)),m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        else
                           write(6,'(i9,d25.16,2x,3i7,2x,99i4)') &
                           ldx,abs(psi(l)),m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        endif
                     elseif(absi.eq.2) then
                        if(.not.ldblout)then
                           write(6,'(i9,f21.12,2x,3i7,2x,99i4)') &
                           ldx,abs(psi(l))**2,m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        else
                           write(6,'(i9,d25.16,2x,3i7,2x,99i4)') &
                           ldx,abs(psi(l))**2,m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        endif
                     elseif(absi.eq.3) then
                        if(.not.ldblout)then
                           write(6,'(i9,f21.12,2x,3i7,2x,99i4)') &
                           ldx,dble(psi(l)),m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        else
                           write(6,'(i9,d25.16,2x,3i7,2x,99i4)') &
                           ldx,dble(psi(l)),m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        endif
                     elseif(absi.eq.4) then
                        if(.not.ldblout)then
                           write(6,'(i9,f21.12,2x,3i7,2x,99i4)') &
                           ldx,dimag(psi(l)),m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        else
                           write(6,'(i9,d25.16,2x,3i7,2x,99i4)') &
                           ldx,dimag(psi(l)),m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        endif
                     else
                        if(.not.ldblout)then
                           write(6,'(i9,2f21.12,2x,3i7,2x,99i4)') &
                           ldx,psi(l),m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        else
                           write(6,'(i9,2d25.16,2x,3i7,2x,99i4)') &
                           ldx,psi(l),m,idxspf,idxgrd,(fii(n)+1,n=1,ndofm)
                        endif
                     endif
                   endif   ! if(nstate.gt.1)
                  endif  !  lrddvr
15                continue
               endif  ! if( .. thr ..)
            endif  !  lplot
         enddo  ! l
      enddo  ! m=m1,m2
   enddo  ! s=s1,s2

   if(lplot) then
      write(iaus,'(/a1/)') 'e'
   else
      if(.not.lcnt) then
         write(6,'(2a)') '  --------------------------------------', &
             '--------------------------------------------------------'
         if(absi.eq.0 .and. lrddvr) then
            if(nstate.gt.1) then
               if(ldblout)then
                  write(6,'(6x,a,17x,a,26x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint,  Coordinates'
               else
                  write(6,'(6x,a,17x,a,17x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint,  Coordinates'
               endif
            else
               if(ldblout)then
                  write(6,'(6x,a,7x,a,26x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint,  Coordinates'
               else
                  write(6,'(6x,a,17x,a,17x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint,  Coordinates'
               endif
            endif ! nstate
         elseif(absi.gt.0 .and. lrddvr) then
            if(nstate.gt.1) then
               if(ldblout)then
                  write(6,'(6x,a,7x,a,11x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint,  Coordinates'
               else
                  write(6,'(6x,a,7x,a,7x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint,  Coordinates'
               endif
            else
               if(ldblout)then
                  write(6,'(6x,a,7x,a,11x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint,  Coordinates'
               else
                  write(6,'(6x,a,7x,a,7x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint,  Coordinates'
               endif
            endif ! nstate

         elseif(absi.eq.0 .and. .not.lrddvr) then
            if(nstate.gt.1) then
               if(ldblout)then
                  write(6,'(6x,a,17x,a,26x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint(mode/DOFs)'
               else
                  write(6,'(6x,a,17x,a,17x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint(mode/DOFs)'
               endif
            else
               if(ldblout)then
                  write(6,'(6x,a,7x,a,26x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint(mode/DOFs)'
               else
                  write(6,'(6x,a,17x,a,17x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint(mode/DOFs)'
               endif
            endif ! nstate
         else
            if(nstate.gt.1) then
               if(ldblout)then
                  write(6,'(6x,a,7x,a,11x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint(mode/DOFs)'
               else
                  write(6,'(6x,a,7x,a,7x,a)') 'index','Coefficient',&
                     'State,  Mode,  SPF,  Gridpoint(mode/DOFs)'
               endif
            else
               if(ldblout)then
                  write(6,'(6x,a,x,a,11x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint(mode/DOFs)'
               else
                  write(6,'(6x,a,7x,a,7x,a)') 'index','Coefficient',&
                     'Mode,  SPF,  Gridpoint(mode/DOFs)'
               endif
            endif ! nstate
         endif ! absi
      endif  ! lcnt
      write(6,'(/a,1p2e9.2," :",i5)') &
        ' Number of SPF-coeffs with absolute values between',thr,mx,cnt
   endif

11 k = k + 1
   time=time + dt

!-----------------------------------------------------------------------
! if more data, go to begining of loop
!-----------------------------------------------------------------------
   if (ebtype.ge.0 .and. time.le.tm)  goto 10

77 continue
   close(ipsi)

   if(lplot) then
      write(iaus,'(a/)') 'pause -1'
      close(iaus)
 !!     This may cause problems when running on Mac
 !!     gnucmd = 'gnuplot -geometry +0+0 -persist "'//trim(fileout)//'"'
      gnucmd = 'gnuplot -geometry "'//trim(fileout)//'"'
      call excmd(trim(gnucmd))
   else
      write(6,'(76a)') ('-',i=1,76)
   endif

   return

98 continue
   write(6,'(a)') ' Error reading restart file'
   stop
   end subroutine rdspfc

!#######################################################################

   subroutine rdspfinput

   implicit none

#include "global.inc"
#include "aglobal.inc"
#include "maxdim.inc"
#include "maxsta.inc"
#include "overlap.inc"
#include "channels.inc"

   integer        :: i, iarg, myiargc, ierr, ss, mm, absi, nspf
   real*8         :: thr, tm, atm, mx
   logical        :: lcnt,ldblout,lplot,lsurf
   character(len=80) ::  buf,lpr

   common /rdspfin/ mx,thr,tm,atm,ss,mm,nspf,absi,lcnt,ldblout,lplot,lsurf,lpr
!-----------------------------------------------------------------------

   filein = ' '
   name   = './'
   lpr=' '
   ebtype = 0
   ovtype = 0
   ss   = 0
   mm   = 0
   nspf = 0
   absi = 0
   thr  = 1.d-8
   mx   = 1.d+8
   atm  = -1.d99
   tm   =  1.d99
   lcnt = .false.
   lplot=.false.
   lsurf=.false.
   lrddvr=.false.
   ldblout=.false.
   iarg = myiargc()
   if (iarg .lt. 1) goto 13

   i = 0
12 i=i+1
   call mygetarg(i,buf)
   if( buf(1:1) .eq. '-' ) then
      if( buf .eq. '-f') then
         i=i+1
         call mygetarg(i,buf)
         filein = buf
      else if(buf .eq. '-i') then
         i=i+1
         call mygetarg(i,buf)
         name = buf

      else if( buf .eq. '-abs' ) then
         absi=1
      else if( buf .eq. '-abs2' ) then
         absi=2
      else if( buf .eq. '-re' ) then
         absi=3
      else if( buf .eq. '-im' ) then
         absi=4
      else if( buf .eq. '-plot' ) then
         lplot=.true.
      else if( buf .eq. '-surf' ) then
         lsurf=.true.
         lplot=.true.
      else if( buf(1:4) .eq. '-coo' ) then
         lrddvr=.true.
      else if( buf .eq. '-thr' ) then
         i = i+1
         call mygetarg(i,buf)
         read(buf,*,iostat=ierr) thr
         if (ierr .ne. 0) then
            write(6,'(2a/,a)') &
              ' The  argument following the option -thr', &
              ' must be a real number. Not : ', buf
            stop
         end if

      else if( buf .eq. '-max' ) then
         i = i+1
         call mygetarg(i,buf)
         read(buf,*,iostat=ierr) mx
         if (ierr .ne. 0) then
            write(6,'(2a/,a)') &
              ' The  argument following the option -max', &
              ' must be a real number. Not : ', buf
            stop
         end if

      else if( buf .eq. '-t' ) then
         i = i+1
         call mygetarg(i,buf)
         read(buf,*,iostat=ierr) tm
         if (ierr .ne. 0) then
            write(6,'(2a/,a)') &
              ' The  argument following the option -t', &
              ' must be a real number. Not : ', buf
            stop
         end if

      else if( buf .eq. '-at' ) then
         i = i+1
         call mygetarg(i,buf)
         read(buf,*,iostat=ierr) atm
         if (ierr .ne. 0) then
            write(6,'(2a/,a)') &
              ' The  argument following the option -at', &
              ' must be a real number. Not : ', buf
            stop
         end if

      else if( buf .eq. '-n' ) then
         i = i+1
         call mygetarg(i,buf)
         read(buf,*,iostat=ierr) step
         if (ierr .ne. 0  .or.  step .le. 0) then
            write(6,'(2a/,a)') &
              ' The  argument following the option -n', &
              ' must be a positive integer. Not : ', buf
            stop
         end if

      else if( buf .eq. '-s' ) then
         i = i+1
         call mygetarg(i,buf)
         read(buf,*,iostat=ierr) ss
         if (ierr .ne. 0  .or.  ss .le. 0) then
            write(6,'(2a/,a)') &
              ' The  argument following the option -s', &
              ' must be a positive integer. Not : ', buf
            stop
         end if

      else if( buf .eq. '-m' ) then
         i = i+1
         call mygetarg(i,buf)
         read(buf,*,iostat=ierr) mm
         if (ierr .ne. 0  .or.  mm .le. 0) then
            write(6,'(2a/,a)') &
              ' The  argument following the option -m', &
              ' must be a positive integer. Not : ', buf
            stop
         end if

      else if( buf .eq. '-spf' ) then
         i = i+1
         call mygetarg(i,buf)
         read(buf,*,iostat=ierr) nspf
         if (ierr .ne. 0  .or.  nspf .le. 0) then
            write(6,'(2a/,a)') &
              ' The  argument following the option -spf', &
              ' must be a positive integer. Not : ', buf
            stop
         end if

      else if( buf .eq. '-P' ) then
         i = i+1
         call mygetarg(i,lpr)

      else if( buf .eq. '-count') then
         lcnt = .true.

      else if( buf .eq. '-r' .or. buf .eq. '-rst') then
         irst   = ipsi
         ebtype = -1
         write(6,'(a)') ' *** Reading the RESTART file !  ***'

      else if( buf .eq. '-dblout' )then
         ldblout = .true.

      else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
         write(6,'(78a1)')  ('-',i=1,78)
         write(6,'(1x,2a)') &
         'Purpose: Prints  the SPF coefficients of a wavefunction', &
            ' or restart file.'
         write(6,'(1x,2a,/19x,a)') &
         'Usage  : rdspf [-f -i -n -s -max -thr -r -abs -abs2 -re -im',&
            ' -t -at','-count -coord -plot -surf -s -m -spf -P -ver -h -?] .'
         write(6,'(a)') ' Options:'
         write(6,'(2a,/,9x,a/,9x,a)') &
                  ' -f FILE : The wavefunction ' &
                 ,'is read from file ' &
                 ,'  FILE rather than from ./psi', &
        '  The string FILE may be a relative or a full path-name.'
         write(6,'(a)') ' -i DIR  : data is read from directory DIR'
         write(6,'(2a)') ' -n step : Print the SPFs', &
        ' only every step-th output.'
         write(6,'(2a,/10x,a)') ' -thr thr: Print only', &
        ' coefficients whose absolute values', &
        ' are larger than thr (default: 10^-8).'
         write(6,'(2a,/10x,a)') ' -max mx : Print only', &
        ' coefficients whose absolute values', &
        ' are smaller or equal than mx.'
         write(6,'(a)') ' -t   tm : Print only WFs with time .le. tm.'
         write(6,'(a)') ' -at atm : Print only WFs with time .ge. atm.'
         write(6,'(a)') ' -s  s   : Print only coefficients for state s.'
         write(6,'(a)') ' -m  m   : Print only coefficients for mode m.'
         write(6,'(a)') ' -spf n  : Print only coefficients for SPF number n.'
         write(6,'(a)') &
        ' -count  : Count the number of coeffs between thr and max.'
         write(6,'(a)') &
        ' -r      : Take the restart file rather than the psi file.'
         write(6,'(a)')  ' -rst    : Same as -r.'
         write(6,'(a)')  ' -abs    : Print abs(psi).'
         write(6,'(a)')  ' -abs2   : Print abs(psi)**2.'
         write(6,'(a)')  ' -re     : Print Re(psi).'
         write(6,'(a)')  ' -im     : Print Im(psi).'
         write(6,'(a)')  ' -coord  : Print coordinates rather than grid-points.'
         write(6,'(a)')  ' -dblout : Print out coefficients in full.'
         write(6,'(a)')  ' -plot   : Plot a SPF. Only for 2D combined modes.'
         write(6,'(a)')  ' -surf   : Plot a SPF in surface mode (3D picture).'
         write(6,'(a)')  ' -P file : Send the plot to a printer, e.g. -P "| lpr",'
         write(6,'(a)')  '           or write it to a ps-file, e.g. -P plot.ps'
         write(6,'(a)')  ' -ver    : Version information about the program.'
         write(6,'(a,/a)') &
                  ' -h      : Print this help text. ', &
                  ' -?      : Print this help text. '
         write(6,'(/2a)') 'It is in general useful to pipe the', &
         ' output into less: rdspf | less'
         write(6,'(2a)') 'or to re-direct it into a file:', &
                         '  rdspf > spfs '
         write(6,'(78a1)')  ('-',i=1,78)
         stop
      else if( buf(1:4) .eq. '-ver' ) then
         call wrversion(6)
         stop
      else
         write(6,'(2a)') ' unknown option : ', buf
         stop
      end if
      if (iarg .le. i) goto 13
      goto 12
   end if

   if (iarg .lt. i) goto 13

13 continue

   if( step .gt.1  .and. ebtype .lt. 0) then
      write(6,'(a)') ' -n and -r options are incompatible'
      stop
   end if


!-----------------------------------------------------------------------
! If filein has not been set, look in name directory.
! Otherwise take name from filein
!-----------------------------------------------------------------------
   if (filein .eq. ' ') then
      laenge=index(name,' ')-1
      call abspath(name,laenge)
      if (ebtype .ge. 0) then
         filein=name(1:laenge)//'/psi'
      else
         filein=name(1:laenge)//'/restart'
      endif
      laein=index(filein,' ')-1
   else
      laein=index(filein,' ')-1
      call abspath(filein,laein)

      i=laein
10    continue
      if (filein(i:i) .eq. '/') then
         laenge=i-1
      else
         i=i-1
         go to 10
      endif
      name=filein(1:laenge)
   endif


!-----------------------------------------------------------------------
!  Opening the files.
!-----------------------------------------------------------------------
   open(ipsi,file=filein(1:laein),form='unformatted',status='old',err=80)
   write(6,'(2a)') ' The inputfile is : ',filein(1:laein)

   if(lplot) then
      fileout=name(1:laenge)//'/spfplot.pl'
      open(iaus,file=fileout,form='formatted',status='unknown',err=81)
   endif

   return

80 write(6,*)' Cannot open file : '
   write(6,*) filein(1:laein)
   stop

81 write(6,*)' Cannot open file : '
   write(6,*) trim(fileout)
   stop

   end subroutine rdspfinput

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   subroutine setf(m,ii,fii)

! --- The array fii contains the dof indices (minus 1) for the
! --- index ii of the combined mode m. (Similar to setm on getdavmat.F)

   implicit none

#include "global.inc"
#include "maxdim.inc"
#include "griddat.inc"

   integer ii, fii(maxdim), f, m, n, k

   k = ii - 1
   do n = nspfdof(m), 2, -1
      f = spfdof(n,m)
      fii(n) = k/vgdim(f)
      k = k - fii(n)*vgdim(f)
   enddo
   fii(1) = k

   return
   end  subroutine setf

