!#######################################################################
      module callanldmod

use decimal, only: long, dop
use global
use griddatmod
use deranld, only:der1anld,der2anld,der3anld,der4anld
use funcanldmod, only:fdefanld,funcanld
use func1dmod, only:fdef1d,func1d
use funcxlocmod, only:fdefxloc,derxloc,funcxloc
use ch3i, only:fdefch3i, fch3i

use opfuncsmod_interface
use logdat
use channels, only: ilog

      implicit none
      private
      public :: defanld,callanld,callderd
      contains

!#######################################################################
! defanld is called from assignhlab which in turn is called by heingabe
! assignhlab calls, in this order, defgrd,defanld,defanlz,defmuld.
!#######################################################################

      subroutine defanld(ifunc,label,ifile,hopipar,hoprpar,maxhpar,&
                 hoppar,apar,rpar,npar,ierr,zpesopts,pesopts,hamlab,&
                 hamdim)

      implicit none

      integer(long)                                     :: n,ilbl,jlbl,klbl,ipower
      integer(long), intent(in)                         :: maxhpar,npar,hamdim
      integer(long), intent(out)                        :: ierr,ifile, ifunc
      integer(long), dimension(maxhpar), intent(out)    :: hopipar
      integer(long), dimension(2*maxhpar,*), intent(in):: zpesopts
      real(dop)                                         :: power,pw1
      real(dop), dimension(npar), intent(in)            :: rpar
      real(dop), dimension(maxhpar), intent(out)        :: hoprpar
      real(dop), dimension(maxhpar,*), intent(in)       :: hoppar
      character(len=*), intent(in)                      :: label
      character(len=*), dimension(npar), intent(in)     :: apar
      character(len=*), dimension(*), intent(in)        :: pesopts,hamlab
      character(len=64)                                 :: buffer

      ifile=0
      ierr=0

!-----------------------------------------------------------------------
! look for any power
!-----------------------------------------------------------------------
      ilbl=index(label,'^')+1
      jlbl=index(label,' ')-1
      pw1 = 1.0_dop
      if (ilbl .ne. 1) then     ! There is one power
         klbl=index(label(ilbl:jlbl),'^')
         if(klbl .gt. 0) then   ! There is a second power, e.g.: ^-2^3
            klbl = ilbl+klbl-2
            read(label(ilbl:klbl),*,iostat=ierr) pw1
            if (ierr.ne.0 .or. abs(pw1).gt.1.0e9_dop) then
               write(6,'(/2a/2a/)') 'An exponent must be a number ', &
                   'and cannot be a parameter.', &
                   'Illegal exponent : ',label
               call stopnow
            endif

            klbl = klbl + 2
         else
            klbl = ilbl
         endif
         read(label(klbl:jlbl),*,iostat=ierr) power
         if (ierr.ne.0 .or. abs(power).gt.1.0e9_dop) then
            write(6,'(/2a/2a/a)') 'An exponent must be a number ', &
                   'and cannot be a parameter.', &
                   'Illegal exponent : ',label
            call stopnow
         endif
         power = power*pw1
         ipower = nint(power)
         if (abs(dble(ipower)-power) .gt. 1.0e-9_dop) then
            ipower=0
         endif
         ilbl=ilbl-2
      else
         ipower=1
         power=1.0
         ilbl=jlbl
      endif

      buffer=' '
      buffer(1:ilbl)=label(1:ilbl)

!-----------------------------------------------------------------------
! re-assign for zero exponent to unity
!-----------------------------------------------------------------------
      if (ipower .eq. 0 .and. abs(power - 0.0_dop) .lt. 1.0e-8_dop) then
         ifunc=1
         ifile=302
         go to 99
      endif

!-----------------------------------------------------------------------
! search through sets of functions
!-----------------------------------------------------------------------
      hopipar(1)=ipower
      hoprpar(1)=power
! file=302
      call fdefanld(ifunc,buffer,ifile,hopipar,hoprpar,maxhpar, &
                    hoppar,zpesopts,pesopts)
      if (ifile .ne. 0) go to 99

! file=303
      call fdef1d(ifunc,buffer,ifile,hopipar,hoprpar,maxhpar,hoppar)
      if (ifile .ne. 0) go to 99

! file=304
      call fdef1dlib(ifunc,buffer,ifile,hopipar,hoprpar,maxhpar,hoppar)
      if (ifile .ne. 0) go to 99

!-----------------------------------------------------------------------
! search through sets of CH3I functions
!-----------------------------------------------------------------------
! file=305
      call fdefch3i(ifunc,buffer,ifile,hoprpar,maxhpar,hoppar)
      if (ifile .ne. 0) go to 99

! file=306
      call ufdef1d(ifunc,buffer,ifile,hopipar,hoprpar,maxhpar,hoppar)
      if (ifile .ne. 0) go to 99

! file=307
      call fdefxloc(ifunc,buffer,ifile,hoprpar,maxhpar,hoppar)
      if (ifile .ne. 0) go to 99

!-----------------------------------------------------------------------
! local control functions
!-----------------------------------------------------------------------
! file=308
      call fdeflct(ifunc,buffer,ifile,hopipar,hoprpar,maxhpar,hoppar,&
                   zpesopts,pesopts,hamlab,hamdim)
      if (ifile .ne. 0) go to 99

!-----------------------------------------------------------------------
! functions of expectation values
!-----------------------------------------------------------------------
! file=309
      call fdefexp(ifunc,buffer,ifile,hopipar,hoprpar,maxhpar,&
                   zpesopts,pesopts,hamlab,hamdim)
      if (ifile .ne. 0) go to 99

! file=310
      call fdefi2ar(ifunc,buffer,ifile,hopipar,hoprpar,maxhpar,hoppar)
      if (ifile .ne. 0) go to 99

!-----------------------------------------------------------------------
! search parameter list   file = 301
!-----------------------------------------------------------------------
      do n=1,npar
         if (buffer .eq. apar(n)) then
            if (ipower .ne. 0) then
               hoprpar(1)=rpar(n)**ipower
            else
               hoprpar(1)=rpar(n)**power
            endif
            ifunc=n
            ifile=301
         endif
      enddo
      if (ifile .ne. 0) go to 99

!-----------------------------------------------------------------------
! try real number; check first, if the label contains a dot.
!-----------------------------------------------------------------------
      klbl=index(buffer,'.')
      if (klbl .ne. 0) then
         read(buffer,*,err=10) hoprpar(1)
         if (ipower .ne. 0) then
            hoprpar(1)=hoprpar(1)**ipower
         else
            hoprpar(1)=hoprpar(1)**power
         endif
         ifunc=0
         ifile=301
         go to 99
      endif

!-----------------------------------------------------------------------
! error if integer (integer operators must be in libraries)??
!-----------------------------------------------------------------------
   10 continue

!-----------------------------------------------------------------------
! Nothing worked, return to assignhlab to try other types (complex, muld). 
! But first check if it is a number without a dot, i.e. check if content
! of buffer is an integer (by reading buffer into klbl).
!-----------------------------------------------------------------------
      read(buffer(1:ilbl),*,err=99) klbl
      write(6,'(/a/2a)') ' Numbers must contain a dot!', &
            ' Unrecognized label: ', buffer(1:ilbl)

   99 continue
      return
      end subroutine defanld

!#######################################################################
! pottyp = 0 : function of x
!          1 : function of p
!#######################################################################

      subroutine callanld(v,x,ifunc,ifile,hopipar,hoprpar,p,pottyp, &
                 label,mass)

      implicit none

      integer(long)                           :: i
      integer(long), intent(in)               :: ifunc,ifile
      integer(long), intent(out)              :: pottyp
      integer(long), dimension(*), intent(in) :: hopipar
      real(dop), intent(in)                   :: x,p,mass
      real(dop), intent(out)                  :: v
      real(dop), dimension(*), intent(in)     :: hoprpar
      character(len=*), intent(in)            :: label

      pottyp=0

      i=ifile-300

      select case (i)

      case (1)
      v = hoprpar(1)
      return

      case (2)
      call funcanld(v,x,ifunc,hopipar,hoprpar)
      return

      case (3)
      call func1d(v,x,ifunc,hopipar,hoprpar,label)
      return

      case (4)
      call func1dlib(v,x,ifunc,hopipar,hoprpar,ilog)
      return

      case (5)
      call fch3i(v,x,ifunc,hoprpar)
      return

      case (6)
      call ufunc1d(v,x,ifunc,hopipar,hoprpar,ilog)
      return

      case (7) ! file 7 are the x-local (classical) momentum functions.
      pottyp=1
      call funcxloc(v,p,x,ifunc,hoprpar,mass)
      return

      case (8) ! local control operator calculated on-the-fly elsewhere
      return

      case (9) ! operator on expectation values calculated on-the-fly elsewhere
      return

      case (10)
      call fi2ar(v,x,ifunc,hopipar,hoprpar)
      return

      case default
      routine='Callanld'
      write(message,'(a,i5)') 'Function file not found :',ifile
      call errormsg

      end select

      end subroutine callanld

!#######################################################################
!
! dertyp = 0 : derivative wrt x
!          1 : derivative wrt p
!
!#######################################################################

      subroutine callderd(der,x,ifunc,ifile,hopipar,hoprpar,order,p, &
                 dertyp,mass)

      implicit none

      integer(long)                           :: i
      integer(long), intent(in)               :: ifunc,ifile,order
      integer(long), intent(out)              :: dertyp
      integer(long), dimension(*), intent(in) :: hopipar
      real(dop), intent(in)                   :: x,p,mass
      real(dop), intent(out)                  :: der
      real(dop), dimension(*), intent(in)     :: hoprpar

      dertyp = 0

      i=ifile-300

      select case (i)

      case(1) ! constant as factor in term, hence derivative not taken
      der = hoprpar(1)
      return

      case(2)
      if (order .eq. 0) then
         call funcanld(der,x,ifunc,hopipar,hoprpar)
      else if (order .eq. 1) then
         call der1anld(der,x,ifunc,hopipar,hoprpar)
      else if (order .eq. 2) then
         call der2anld(der,x,ifunc,hopipar,hoprpar)
      else if (order .eq. 3) then
         call der3anld(der,x,ifunc,hopipar,hoprpar)
      else if (order .eq. 4) then
         call der4anld(der,x,ifunc,hopipar,hoprpar)
      else 
         routine='Callderd (callanldmod.f90)'
         write(message,'(a,i3)') &
            'Can only calculate integrals up to 4th order. Order: ',order
         call errormsg
      endif
      return

      case(3,4,5,6) ! no derivatives for func1d functions
      return

      case(7)
      if (order .eq. 0) then
         call funcxloc(der,p,x,ifunc,hoprpar,mass)
      else if (order .eq. 1) then
         dertyp=1
         call derxloc(der,p,ifunc,mass)
      endif
      return

      case(8)
      return

      case default
      routine='Callder'
      write(message,'(a,i5)') 'Function derivative file not found :',&
          ifile
      call errormsg

      end select

      end subroutine callderd
                              
      end module callanldmod
