!#######################################################################
!  source/opfuncs/callgrdmod.f90   contains:defgrd,callgrd
!
!  fdefgrd,funcgrd,fch3igrd,fdefch3igrd
!
!  tm4,tm5,tm6,tm7,tm8 
! diag2zop,dop2zop,kedvrterm,kefbrterm,kefftterm,dqfft,pfft,ebas,ebas1,fluxsmat,
! fluxemat,jtotp,jtotm,jtotx,jtoty,kshift,pgauss,qdq,cdq,sdq,udq,udq2,sdq2,
! cdq2,csdq,uqdq,econt,csproj,czproj,d2th1 
!#######################################################################
      module callgrdmod

use decimal, only: long, dop
use funcgrdmod, only: fdefgrd
use ch3igrd, only: fdefch3igrd
use funcklegmod, only: fdefkleg
use func2klegmod, only: fdef2kleg
use funcwigmod, only: fdefwig
use op1lib,  only: zeromxz
use logdat

      implicit none
      private
      public :: defgrd, callgrd, callreadmap
      contains

!#######################################################################

      subroutine defgrd(ifunc,label,ifile,ipar,rpar,maxpar,rdlab, &
                 rdtype,nrdfunc,hoppar)

      implicit none

      integer(long)                                    :: n
      integer(long), intent(in)                        :: maxpar,nrdfunc
      integer(long), intent(out)                       :: ifunc,ifile
      integer(long), dimension(maxpar), intent(out)    :: ipar
      integer(long), dimension(nrdfunc), intent(in)    :: rdtype
      real(dop), dimension(maxpar), intent(out)        :: rpar
      real(dop), dimension(maxpar,*), intent(in)       :: hoppar
      character(len=*), intent(in)                     :: label
      character(len=*), dimension(nrdfunc), intent(in) :: rdlab

!-----------------------------------------------------------------------
! search labels in list from read files (ifunc=hopilab, ifile=hopfile)
!-----------------------------------------------------------------------
      do n=1,nrdfunc
         if (label .eq. rdlab(n)) then
            ifunc=n
            if (rdtype(n) .gt. 100) then
               ifile=12
            else if (rdtype(n) .eq. 2) then
               ifile=1
            else
               ifile=2
            endif
         endif
      enddo
      if (ifile /= 0) return
!-----------------------------------------------------------------------
! search through sets of functions
!-----------------------------------------------------------------------
      call fdefgrd(ifunc,label,ifile,ipar,rpar,hoppar)
      if (ifile .ne. 0) return

      call fdefch3igrd(ifunc,label,ifile)
      if (ifile .ne. 0) return

      call fdefkleg(ifunc,label,ifile)
      if (ifile .ne. 0) return

      call fdef2kleg(ifunc,label,ifile)
      if (ifile .ne. 0) return

      call fdefwig(ifunc,label,ifile)
      if (ifile .ne. 0) return

      call fdefmlop(ifunc,label,ifile,ipar,rpar,maxpar)
      if (ifile .ne. 0) return

      return
      end subroutine defgrd

!#######################################################################

      subroutine callgrd(hopz,ifunc,ifile,hopipar,hoprpar,dif1mat, &
                 dif2mat,kinsph,fftp,ort,rdop,zrdop,f,workc,sdm,trafo,mass)

use global
use dvrdatmod
use griddatmod

use funcgrdmod, only: funcgrd
use ch3igrd, only: fch3igrd

      implicit none

      integer(long)                           :: i
      integer(long), intent(in)               :: ifunc,ifile,sdm,f
      integer(long), dimension(*), intent(in) :: hopipar,zrdop
      real(dop), dimension(*), intent(in)     :: hoprpar,dif1mat,dif2mat, &
                                                 ort,fftp,kinsph,rdop,trafo
      real(dop), intent(in)                   :: mass
      complex(dop), dimension(*), intent(out) :: hopz,workc
      real(dop), dimension(sdm)               :: hopd

      i=ifile

     select case (i)

     case default
      routine="callgrd"
      write(message,'(a,i0)') "illegal hopfile = ",ifile
      call errormsg

     case (1)
      call diag2zop(rdop(zrdop(ifunc)),hopz,gdim(f))
      return

     case (2)
      call dop2zop(rdop(zrdop(ifunc)),hopz,gdim(f))
      return

     case (3)
      call funcgrd(hopz,gdim(f),ifunc,hopipar,hoprpar,mass,dif1mat,&
           dif2mat,kinsph,fftp,ort,sphdim,basis(f),workc,jtot,trafo,&
           rpbaspar(1,f),ipbaspar(1,f))
      return

     case (4)
      call fch3igrd(hopz,gdim(f),ifunc,dif1mat,dif2mat,ort)
      return

     case (5) ! file 105 is the KLeg operators in funckleg.F. This must be called directly.
      continue
      return

     case (6)
      call diag2zop(rdop(zrdop(ifunc)),hopz,sdm)
      return

     case (7) ! operators for 2 combined KLegs must also be called directly.
      continue
      return

     case(8) ! operators for Wigners must also be called directly.
      continue
      return

     case(9)
      ! MLPF leaf potentials (if 1D)
      call callmlop(hopd,gdim(f),ifunc,hopipar(1),hopipar(2))
      call diag2zop(hopd,hopz,gdim(f))
      return

     case(12)
!     ! readmap, mapping operators.
!     call callmap(rdop(zrdop(ifunc)),workr)
      continue
      return

     end select

      end subroutine callgrd


!#######################################################################

      subroutine diag2zop(hops,hopz,dim)

      implicit none

      integer(long)                                 :: i
      integer(long), intent(in)                     :: dim
      real(dop), dimension(dim), intent(in)         :: hops
      complex(dop), dimension(dim,dim), intent(out) :: hopz

      call zeromxz(hopz,dim,dim)
      do i=1,dim
         hopz(i,i)=hops(i)
      enddo

      return
      end subroutine diag2zop

!#######################################################################

      subroutine dop2zop(hops,hopz,gdim)

      implicit none

      integer(long)                                   :: g,g1
      integer(long), intent(in)                       :: gdim
      real(dop), dimension(gdim,gdim,2), intent(in)   :: hops
      complex(dop), dimension(gdim,gdim), intent(out) :: hopz

      call zeromxz(hopz,gdim,gdim)
      do g=1,gdim
         do g1=1,gdim
            hopz(g1,g)=dcmplx(hops(g1,g,1),hops(g1,g,2))
         enddo
      enddo

      return
      end subroutine dop2zop

!#######################################################################

      subroutine callreadmap(rdop,hopd,gdim1,hoplab)

      implicit none

      integer nel, i, gdim1, irow, icol
      real*8  hopd(*), rdop(*)
      character*32 hoplab

      nel=nint(rdop(1))
      do i=1,3*nel+1
         hopd(i) = rdop(i)
      enddo

      do i=1,nel
         irow=nint(hopd(i+1))
         icol=nint(hopd(nel+i+1))
         if(irow.lt.1 .or. irow.gt.gdim1) then
            write(6,'(/a,2i6)') 'callreadmap: irow, gdim1', irow, gdim1
            routine='callreadmap on opfuncs/callgrd.F'
            message='Illegal value of row index of '//trim(hoplab)
            call errormsg
         endif
         if(icol.lt.1 .or. icol.gt.gdim1) then
            write(6,'(/a,2i6)') 'callreadmap: icol, gdim1', icol, gdim1
            routine='callreadmap on opfuncs/callgrd.F'
            message='Illegal value of column index of '//trim(hoplab)
            call errormsg
         endif

      enddo

      return
      end subroutine callreadmap

      end module callgrdmod

