!#######################################################################
! contains :uvpoint,udefsrf,derqchem,fdefthphi,defqchem,defmuld
!
!#######################################################################
      module callmuldmod

use decimal, only: long, dop
use channels, only: ilog
use paths
use mpidata

      implicit none
      private
      public :: defmuld,callmuld,callmulder
      contains

!#######################################################################
! New version (Benjamin, December 2008)
!#######################################################################

      subroutine defmuld(ifunc,label,ifile,pesopts,zpesopts,hopipar,&
                 hoprpar,maxhpar,pesmin,pesmax,lcutpes,ierr,ndof,&
                 hoppar)

use funcsrf, only:defsrf
use functhph, only:fdefthphi
use funcqchemmod, only:defqchem
use funcshep
use funcqch1mod, only:defqchem1

      implicit none

      integer(long), intent(in)                         :: ndof,maxhpar
      integer(long), intent(out)                        :: ifunc,ifile,ierr
      integer(long), dimension(maxhpar,*), intent(out)  :: hopipar
      integer(long), dimension(2*maxhpar,*), intent(in) :: zpesopts
      real(dop), intent(inout)                          :: pesmin,pesmax
      real(dop), dimension(*), intent(out)              :: hoprpar
      real(dop), dimension(maxhpar,*), intent(in)       :: hoppar
      character(len=*), intent(in)                      :: label
      character(len=*), dimension(*), intent(in)        :: pesopts
      logical(kind=4), intent(out)                      :: lcutpes

      ierr=0

!-----------------------------------------------------------------------
! search through sets of functions
!-----------------------------------------------------------------------
      call defsrf(ifunc,label,ifile,pesopts,zpesopts,hopipar,hoprpar, &
                 maxhpar,pesmin,pesmax,lcutpes,ndof,ierr,opdatadef)
      if (ifile .ne. 0) return

      call fdefthphi(ifunc,label,ifile,hopipar,hoprpar,maxhpar, &
                     hoppar)
      if (ifile .ne. 0) return

      call udefsrf(ifunc,label,ifile,pesopts,zpesopts,hopipar,hoprpar, &
                   maxhpar,pesmin,pesmax,lcutpes,ndof,ierr,ilog)
      if (ifile .ne. 0) return

      call defsrflib(ifunc,label,ifile,pesopts,zpesopts,hopipar,hoprpar, &
                   maxhpar,pesmin,pesmax,lcutpes,ndof,ierr,ilog,opdatadef,&
                   mpirank)
      if (ifile .ne. 0) return

      call defqchem(ifunc,label,ifile,pesopts,zpesopts,hopipar,ierr)
      if (ifile .ne. 0) return

      call defshep(ifunc,label,ifile,pesopts,zpesopts,ierr)
      if (ifile .ne. 0) return

      call defqchem1(ifunc,label,ifile,ierr)
      if (ifile .ne. 0) return

      call trasrf(ifunc,label,ifile,ierr)
      if (ifile .ne. 0) return

! V for DD using GPR ??


      end subroutine defmuld

!#######################################################################

      subroutine callmuld(v,gpoint,ndof,ifunc,ifile,ipar,rpar, &
                 ddsi,ddsf,label)

use dirdyn                 
use funcqchemmod, only:vqchem
use funcqch1mod, only:vqchem1
use funcsrf, only:vpoint
use functhph, only:functhphi
use funcshep

      implicit none

      integer(long), intent(in)                :: ndof,ddsi,ddsf
      integer(long), intent(in)                :: ifunc,ifile
      integer(long), dimension(*), intent(in)  :: ipar
      real(dop), intent(out)                   :: v
      real(dop), dimension(*), intent(in)      :: rpar
      real(dop), dimension(ndof), intent(in)   :: gpoint
      character(len=*), intent(in)             :: label
      real(dop)                                :: rms,totsum

     select case (ifile)

     case default
      write(6,'(a,i4)') '### callmuld: Unknown hopfile =', ifile
      write(2,'(a,i4)') '### callmuld: Unknown hopfile =', ifile
      stop

     case (201)
      call vpoint(v,gpoint,ndof,ifunc,ipar,rpar,label)
      return

     case (202)
      call functhphi(v,gpoint(1),gpoint(2),ifunc,ipar,rpar)
      return

     case (203)
      call uvpoint(v,gpoint,ndof,ifunc,ipar,rpar,ilog)
      return

     case (204)
      call vlibpoint(v,gpoint,ndof,ifunc,ipar,rpar,label,ilog,opdatadef)
      return

     case (205)! QChem functions
      call vqchem(v,gpoint,ddsi,ddsf)    
      return

     case (206)  ! modified Shepard interpolation
      call vshepard(v,gpoint,ndof,rms,totsum)
      return

     case (207)! QChem non-potental functions
      call vqchem1(v,gpoint,ifunc,ddsi,ddsf)
      return

     case (208)! QChem potental functions using GPR
!      call vgap(v,gpoint,ddsi,ddsf,label)
      return

!     case (209)! For coordinate transformation
!       call tvpoint(v,gpoint,ndof)
      return

     end select

      end subroutine callmuld


!#######################################################################

      subroutine callmulder(der,gpoint,ndof,ifunc,ifile,ipar,order,&
                 ddsi,ddsf)

use funcqchemmod, only:derqchem
use funcshep
use NH3pot, only: NH3der

      implicit none

      integer(long), intent(in)                   :: ndof,ddsi,ddsf
      integer(long), intent(in)                   :: ifunc,ifile
      integer(long), dimension(*), intent(in)     :: ipar
      integer(long), dimension(ndof), intent(in)  :: order
      real(dop), intent(out)                      :: der
      real(dop), dimension(ndof), intent(in)      :: gpoint

     select case (ifile)

     case default
      write(6,'(a,i4)') '### callmuld: Unknown hopfile =', ifile
      write(2,'(a,i4)') '### callmuld: Unknown hopfile =', ifile
      stop

     case (201:203)
      return

     case (204)
       if (ifunc==25) then
          ! This is the special case of 12D NH3
          ! if default kind is not the same as "long" kind, this will fail!
          call NH3der(der,gpoint,order,ipar(1))
       else
         return
       endif

     case (205)! QChem functions
      call derqchem(der,order,ddsi,ddsf)
      return

     case (206)  ! Shepard
       call dvshepard(der,gpoint,ndof,order)
       return

     case (207:209)
      return

     end select

      end subroutine callmulder

      end module callmuldmod


