!***********************************************************************
!
! CH3I0
!
! 1d-operators needed for fit of CH3I ground state and excited states.
!
! ifile = 305
!
! from Michael Beck / Uwe Manthe
!
!***********************************************************************
      module ch3i

use decimal, only: long, dop

      implicit none
      private
      public :: fdefch3i,fch3i
      contains


!***********************************************************************
      subroutine fdefch3i(ifunc,label,ifile,rpar,maxpar,hoppar)

      implicit none

      integer(long), intent(in)                    :: maxpar
      integer(long)                                :: n,ilbl,jlbl
      integer(long), intent(out)                   :: ifunc,ifile
      real(dop)                                    :: mci,mh,mc
      real(dop), dimension(maxpar), intent(out)    :: rpar
      real(dop), dimension(maxpar,*), intent(in)   :: hoppar
      character(len=*), intent(in)                 :: label

!-----------------------------------------------------------------------
! assign function number
!-----------------------------------------------------------------------
! CH3I S0 surface terms

      if (label .eq. 'MI:vri') then
         ifunc=1
      else if (label .eq. 'MI:cri') then
         ifunc=2
      else if (label .eq. 'MI:wri') then
         ifunc=3
      else if (label(1:5) .eq. 'MI:vt') then
         ilbl=index(label,'[')+1
         jlbl=index(label,']')-1
         read(label(ilbl:jlbl),*) n
         mci=hoppar(1,n)
         mh=hoppar(2,n)

         rpar(1)=mci
         rpar(2)=mh

         ifunc=4
      else if (label .eq. 'MI:rg0') then
         ifunc=5
      else if (label .eq. 'MI:rg1') then
         ifunc=6
      else if (label .eq. 'MI:rg2') then
         ifunc=7
      else if (label .eq. 'MI:g1') then
         ifunc=8
      else if (label .eq. 'MI:g3') then
         ifunc=9
      else if (label .eq. 'MI:ca') then
         ifunc=10
      else if (label .eq. 'MI:va') then
         ifunc=11
      else if (label .eq. 'MI:invsin2') then
         ifunc=12
      else if (label(1:7) .eq. 'MI:tm13') then
         ilbl=index(label,'[')+1
         jlbl=index(label,']')-1
         read(label(ilbl:jlbl),*) n
         mc=hoppar(1,n)
         mh=hoppar(2,n)

         rpar(1)=mc
         rpar(2)=mh

         ifunc=13

! CH3I S1 surface terms
      else if (label .eq. 'MI:ea0') then
         ifunc=14
      else if (label .eq. 'MI:ea1') then
         ifunc=15
      else if (label .eq. 'MI:ea2') then
         ifunc=16
      else if (label .eq. 'MI:eb0') then
         ifunc=17
      else if (label .eq. 'MI:eb1') then
         ifunc=18
      else if (label .eq. 'MI:eb2') then
         ifunc=19
      else if (label .eq. 'MI:ra0') then
         ifunc=20
      else if (label .eq. 'MI:ra1') then
         ifunc=21
      else if (label .eq. 'MI:ra2') then
         ifunc=22
      else if (label .eq. 'MI:rb0') then
         ifunc=23
      else if (label .eq. 'MI:rb1') then
         ifunc=24
      else if (label .eq. 'MI:rb2') then
         ifunc=25
      else if (label .eq. 'MI:ed0') then
         ifunc=26
      else if (label .eq. 'MI:ed1') then
         ifunc=27
      else if (label .eq. 'MI:ed2') then
         ifunc=28
      else if (label .eq. 'MI:a0') then
         ifunc=29
      else if (label .eq. 'MI:a1') then
         ifunc=30
      else if (label .eq. 'MI:a2') then
         ifunc=31
      else if (label .eq. 'MI:a3') then
         ifunc=32
      else if (label .eq. 'MI:b0') then
         ifunc=33
      else if (label .eq. 'MI:b1') then
         ifunc=34
      else if (label .eq. 'MI:b2') then
         ifunc=35
      else if (label .eq. 'MI:d1') then
         ifunc=36
      else if (label .eq. 'MI:a50') then
         ifunc=37
      else if (label .eq. 'MI:a51') then
         ifunc=38
      else if (label .eq. 'MI:a52') then
         ifunc=39
      else if (label .eq. 'MI:g0') then
         ifunc=40

!-----------------------------------------------------------------------
! end of if loop
!-----------------------------------------------------------------------
      endif

!-----------------------------------------------------------------------
! if label is assigned, assign ifile no.
!-----------------------------------------------------------------------
      if (ifunc .ne. 0) ifile=305

      return
      end subroutine fdefch3i

!#######################################################################

      subroutine fch3i(v,xx,ifunc,rpar)

      implicit none

      integer(long), intent(in)               :: ifunc

      real(dop)                           :: pi
      real(dop)                           :: costh,mci,mh,mc,yy
      real(dop)                           :: x,x1,x2
      real(dop), intent(in)               :: xx
      real(dop), intent(out)              :: v
      real(dop), dimension(*), intent(in) :: rpar

      real(dop)       :: rre=4.02889_dop, re=2.0484_dop, de =0.0874_dop, xx0=4.02889_dop, yy0=0.74075_dop,&
                        be01=0.87094_dop, be02=0.4914_dop, akf=0.036225_dop, ak0=0.15581_dop, a00=0.140181_dop,&
                        a01=-0.124719_dop, a02=0.043310_dop, a10=0.088560_dop, a11=-0.048580_dop, a12=-0.163996_dop,&
                        a20=0.141458_dop, a21=-0.384253_dop, a22=0.339277_dop,a30=-0.003932_dop, a31=0.002520_dop,&
                        a32=-0.000993_dop, b00=0.099686_dop, b01=-0.153511_dop, b02=0.064085_dop, b10=-0.068734_dop,&
                        b11=0.068596_dop, b12=-0.059619_dop, b20=0.099668_dop, b21=-0.181578_dop, b22=0.089516_dop,&
                        c0=16.8375_dop, c1=-34.2027_dop, c2=17.3651_dop, d10=0.004618_dop, d11=-0.011405_dop,&
                        d12=0.011226_dop, g2=0.012319_dop, g4=0.026889_dop, g6=-0.000889_dop


      pi=4.0_dop*atan(1.0_dop)

     select case (ifunc)

     case (1)
      v = de*(dexp(-be01*(xx-xx0))-1.0)**2&
                      +0.5*(akf+(ak0-akf)*dexp(-be02*(xx-xx0)))&
                      *(yy0*dexp(-be02*(xx-xx0)))**2
      return

     case (2)
      v = -(akf+(ak0-akf)*dexp(-be02*(xx-xx0))) &
                      *yy0*dexp(-be02*(xx-xx0))
      return

     case (3)
      v=0.5*(akf+(ak0-akf)*dexp(-be02*(xx-xx0)))
      return

     case (4)
      mci=rpar(1)
      mh=rpar(2)
      v=(1.0/(1.0/(mci*rre**2)&
                      +2.0/(3.0*mh*re**2.0*dsin(1.940806128_dop)**2)))&
                      *(0.004019)**2*xx**2/2
      return

     case (5)
      x=dsqrt(3.0_dop)*(xx-2.0278_dop)
      x1=dsqrt(3.0_dop)*(2.0484_dop-2.0278_dop)
      v = 0.3678*x**2/2   -0.6390*x**3/6  +0.9677*x**4/24 &
                     -1.5801*x**5/120 +2.9013*x**6/720 &
                     -(0.3678*x1**2/2-0.6390*x1**3/6+0.9677*x1**4/24 &
                     -1.5801*x1**5/120 +2.9013*x1**6/720)
      return

     case (6)
      x=dsqrt(3.0_dop)*(xx-2.0278_dop)
      x1=dsqrt(3.0_dop)*(2.0484_dop-2.0278_dop)
      v=x-x1
      return

     case (7)
      x=dsqrt(3.0_dop)*(xx-2.0278_dop)
      x1=dsqrt(3.0_dop)*(2.0484_dop-2.0278_dop)
      v=x**2-x1*2
      return

     case (8)
      x=3.0*(dacos(xx)-0.5*pi)**2
      v=-0.0825*x/2+0.607*x**2/24
      return

     case (9)
      x=3.0*(dacos(xx)-0.5*pi)**2
      v=-0.0096*x/4
      return

     case (10)
      yy=-re*xx
      v=yy
      return

     case (11)
      yy=-re*xx
      v=yy**2
      return

     case (12)
      costh=xx
      v=1.0_dop/(1.0_dop-costh**2)
      return

     case (13)
      mc=rpar(1)
      mh=rpar(2)
      v=1.0_dop+3.0_dop*mh/mc*xx**2
      return

     case (14)
      x=xx-rre
      v=dexp(-1.99296*x)
      return

     case (15)
      x=xx-rre
      v=dexp(-1.92832*x)
      return

     case (16)
      x=xx-rre
      v=dexp(-1.7844*x)
      return

     case (17)
      x=xx-rre
      v=dexp(-0.874084*x)
      return

     case (18)
      x=xx-rre
      v=dexp(-0.894141*x)
      return

     case (19)
      x=xx-rre
      v=dexp(-1.06394*x)
      return

     case (20)
      x=xx-rre
      v=dexp(-1.99296*x)*x
      return

     case (21)
      x=xx-rre
      v=dexp(-1.92832*x)*x
      return

     case (22)
      x=xx-rre
      v=dexp(-1.7844*x)*x
      return

     case (23)
      x=xx-rre
      v=dexp(-0.874084*x)*x
      return

     case (24)
      x=xx-rre
      v=dexp(-0.894141*x)*x
      return

     case (25)
      x=xx-rre
      v=dexp(-1.06394*x)*x
      return

     case (26)
      x=xx-rre
      v=1.0_dop/((x-0.19454)**2+0.438362)
      return

     case (27)
      x=xx-rre
      v=1.0_dop/((x-0.27057)**2+0.558465)
      return

     case (28)
      x=xx-rre
      v=1.0_dop/((x-0.34429)**2+0.688788)
      return

     case (29)
      x=dsin(dacos(xx)-pi/2)
      v=a00+a01*x+a02*x**2
      return

     case (30)
      x=dsin(dacos(xx)-pi/2)
      v=a10+a11*x+a12*x**2
      return

     case (31)
      x=dsin(dacos(xx)-pi/2)
      v=a20+a21*x+a22*x**2
      return

     case (32)
      x=dsin(dacos(xx)-pi/2)
      v=a30+a31*x+a32*x**2
      return

     case (33)
      x=dsin(dacos(xx)-pi/2)
      v=b00+b01*x+b02*x**2
      return

     case (34)
      x=dsin(dacos(xx)-pi/2)
      v=b10+b11*x+b12*x**2
      return

     case (35)
      x=dsin(dacos(xx)-pi/2)
      v=b20+b21*x+b22*x**2
      return

     case (36)
      x=dsin(dacos(xx)-pi/2)
      v=d10+d11*x+d12*x**2
      return

     case (37)
      x=dsin(dacos(xx)-pi/2)
      x1=x-0.173648
      x2=x-0.34202
      v=c0*x1*x2
      return

     case (38)
      x=dsin(dacos(xx)-pi/2)
      x2=x-0.34202
      v=c1*x*x2
      return

     case (39)
      x=dsin(dacos(xx)-pi/2)
      x1=x-0.173648
      v=c2*x*x1
      return

     case (40)
      x=dsin(dacos(xx)-pi/2)
      x=(x*re)**2
      v=g2*x+g4*x**2+g6*x**3
      return
     
     end select

      end subroutine fch3i

!#######################################################################

      end module ch3i

