!#######################################################################
      module ch3igrd

use decimal, only: long, dop
use op1lib,  only: zeromxz

      implicit none
      private
      public :: fdefch3igrd,fch3igrd
      contains

!#######################################################################
!
! following functions are for CH3I
! from Michael Beck / Uwe Manthe
! ifile=4
!#######################################################################

      subroutine fdefch3igrd(hopilab,hoplab,hopfile)

      implicit none

      integer(long), intent(out)                    :: hopilab,hopfile
      character(len=*), intent(in)                  :: hoplab

!-----------------------------------------------------------------------
! assign function number
!-----------------------------------------------------------------------
      select case (hoplab)

      case ('MI:tm4 ')
         hopilab=1
      case ('MI:tm5 ')
         hopilab=2
      case ('MI:tm6 ')
         hopilab=3
      case ('MI:tm7 ') 
         hopilab=4
      case ('MI:tm8 ')
         hopilab=5

      end select
!-----------------------------------------------------------------------
! if label is assigned, assign hopfile no.
!-----------------------------------------------------------------------
      if (hopilab .ne. 0) hopfile=4

      return
      end subroutine fdefch3igrd

!#######################################################################

      subroutine fch3igrd(hmat,gdim,ifunc,dif1mat,dif2mat,ort)

      implicit none

      integer(long), intent(in)                       :: gdim,ifunc
      real(dop), dimension(gdim), intent(in)          :: ort
      real(dop), dimension(gdim,gdim), intent(in)     :: dif2mat,dif1mat
      complex(dop), dimension(gdim,gdim), intent(out) :: hmat

     select case (ifunc)
! matrix operators for CH3I
     case (1)
      call tm4(ort,hmat,dif1mat,dif2mat,gdim)
      return
     case (2)
      call tm5(ort,hmat,dif1mat,gdim)
      return
     case (3)
      call tm6(ort,hmat,dif1mat,gdim)
      return
     case (4)
      call tm7(ort,hmat,dif1mat,gdim)
      return
     case (5)
      call tm8(ort,hmat,dif1mat,dif2mat,gdim)
      return
     end select

      end subroutine fch3igrd

!#######################################################################

      subroutine tm4(ort,hmat,dif1mat,dif2mat,gdim)

      implicit none

      integer(long)                                   :: g,gg
      integer(long), intent(in)                       :: gdim
      real(dop), dimension(gdim), intent(in)          :: ort
      real(dop), dimension(gdim,gdim), intent(in)     :: dif1mat,dif2mat
      complex(dop), dimension(gdim,gdim), intent(out) :: hmat

      call zeromxz(hmat,gdim,gdim)

      do g=1,gdim
         hmat(g,g)=-0.25_dop*(3.0_dop*ort(g)**2-1.0_dop)
      enddo
      do gg=1,gdim
         do g=1,gdim
            hmat(g,gg)=hmat(g,gg)+ &
                      2.0_dop*ort(g)*(1.0-ort(g)**2)*dif1mat(g,gg)+ &
                      (1.0-ort(g)**2)**2*(-0.5*dif2mat(g,gg))
         enddo
      enddo

      return
      end subroutine tm4

!#######################################################################

      subroutine tm5(ort,hmat,dif1mat,gdim)

      implicit none

      integer(long)                                   :: g,gg
      integer(long), intent(in)                       :: gdim
      real(dop), dimension(gdim), intent(in)          :: ort
      real(dop), dimension(gdim,gdim), intent(in)     :: dif1mat
      complex(dop), dimension(gdim,gdim), intent(out) :: hmat

      call zeromxz(hmat,gdim,gdim)

      do g=1,gdim
         hmat(g,g)=3.0_dop*ort(g)**2 - 1.0_dop
      enddo
      do gg=1,gdim
         do g=1,gdim
            hmat(g,gg)=hmat(g,gg) - &
               2.0_dop*ort(g)*(1.0_dop-ort(g)**2)*dif1mat(g,gg)
         enddo
      enddo

      return
      end subroutine tm5
!#######################################################################

      subroutine tm6(ort,hmat,dif1mat,gdim)

      implicit none

      integer(long)                                   :: g,gg
      integer(long), intent(in)                       :: gdim
      real(dop), dimension(gdim), intent(in)          :: ort
      real(dop), dimension(gdim,gdim), intent(in)     :: dif1mat
      complex(dop), dimension(gdim,gdim), intent(out) :: hmat

      call zeromxz(hmat,gdim,gdim)

      do gg=1,gdim
         do g=1,gdim
            hmat(g,gg)=((1.0_dop/ort(g))+(1.0_dop/ort(gg))) &
                         *dif1mat(g,gg)
         enddo
      enddo

      return
      end subroutine tm6

!#######################################################################

      subroutine tm7(ort,hmat,dif1mat,gdim)

      implicit none

      integer(long)                                   :: g,gg
      integer(long), intent(in)                       :: gdim
      real(dop), dimension(gdim), intent(in)          :: ort
      real(dop), dimension(gdim,gdim), intent(in)     :: dif1mat
      complex(dop), dimension(gdim,gdim), intent(out) :: hmat

      call zeromxz(hmat,gdim,gdim)

      do g=1,gdim
         hmat(g,g)=ort(g)
      enddo
      do gg=1,gdim
         do g=1,gdim
            hmat(g,gg)=hmat(g,gg)-(1.0_dop-ort(g)**2)*dif1mat(g,gg)
         enddo
      enddo

      return
      end subroutine tm7

!#######################################################################

      subroutine tm8(ort,hmat,dif1mat,dif2mat,gdim)

      implicit none

      integer(long)                                   :: g,gg
      integer(long), intent(in)                       :: gdim
      real(dop), dimension(gdim), intent(in)          :: ort
      real(dop), dimension(gdim,gdim), intent(in)     :: dif1mat,dif2mat
      complex(dop), dimension(gdim,gdim), intent(out) :: hmat

      call zeromxz(hmat,gdim,gdim)

      do gg=1,gdim
         do g=1,gdim
            hmat(g,gg)=ort(g)*dif1mat(g,gg) + &
                  (1.0-ort(g)**2)*(-0.5*dif2mat(g,gg))
         enddo
      enddo

      return
      end subroutine tm8

      end module ch3igrd
