!#######################################################################
      module deranld

use decimal, only: long,dop
use logdat

      implicit none
      private
      public :: fdefderd,der4anld,der3anld,der1anld,der2anld
      contains

!#######################################################################
!
! DERIVATIVES OF ANALYTIC FUNCTIONS. 
! Functions must be in funcanld.F
!
! on return
! found = 1 if function has a derivative in deranld
!       = 0 if it is a constant ( e.g. 1 or -1)
!
!#######################################################################

      subroutine fdefderd(label,found)

      implicit none

      integer(long), intent(out)   :: found
      integer(long)                :: ilbl,jlbl
      character(len=*), intent(in) :: label

!-----------------------------------------------------------------------
! get end of operator label
! look for argument or power
!-----------------------------------------------------------------------
      ilbl=index(label,'[')-1
      if (ilbl .lt. 0) ilbl=index(label,' ')-1
      jlbl=index(label,'^')-1
      if (jlbl .lt. 0) jlbl=index(label,' ')-1
      ilbl=min(ilbl,jlbl)

!-----------------------------------------------------------------------
! function number is same as in function file 
!-----------------------------------------------------------------------
! 1 (unit operator) ifunc = 1
      if (label .eq. '1'    .or.  label .eq. '1.'    .or. &
          label .eq. '1.0'  .or.  label .eq. '1.00'  .or. &
          label .eq. '1._dop'  .or.  label .eq. '1.0_dop') then
         found=0

! -1  ifunc = 2
      else if (label .eq. '-1'    .or.  label .eq. '-1.'    .or. &
               label .eq. '-1.0'  .or.  label .eq. '-1.00'  .or. &
               label .eq. '-1._dop'  .or.  label .eq. '-1.0_dop') then
         found=0

! q^n ifunc = 3
      else if (label(1:ilbl) .eq. 'q') then
         found=1

! sin^n ifunc = 4
! sin(p(x-x0))^n ifunc = 5
      else if (label(1:ilbl) .eq. 'sin') then
         found=1

! cos^n ifunc = 6
! cos(p(x-x0))^n ifunc = 5
      else if (label(1:ilbl) .eq. 'cos') then
         found=1

! gauss^n ifunc = 12
      else if (label(1:ilbl) .eq. 'gauss') then
         found=1

      else
         found=999

!-----------------------------------------------------------------------
! end of if loop
!-----------------------------------------------------------------------
      endif

      return
      end subroutine fdefderd

!#######################################################################

      subroutine der1anld(der,x,ifunc,hopipar,hoprpar)

use global

      implicit none

      integer(long)                           :: ipower,ipower1
      integer(long), intent(in)               :: ifunc
      integer(long), dimension(*), intent(in) :: hopipar
      real(dop)                               :: power,tmp,power1,r
      real(dop), intent(in)                   :: x
      real(dop), intent(out)                  :: der
      real(dop), dimension(*), intent(in)     :: hoprpar

      der1: select case(ifunc)

      case(1)  ! 1.0 and -1.0 are treated as coefficients and the derivative is not
! taken
! unit
      der=1.0_dop
      return

      case(2) ! -unit
      der=-1.0_dop
      return

      case(3) ! q^n
! case 1: real number exponent stored in hoprpar(1)
! case 2: integer power stored in hopipar(1)
! case 3: linear term
      if(hopipar(1).eq.0) then
         power=hoprpar(1)-1.0d0
         der=(x-hoprpar(2))**power
         der=hoprpar(1)*der
      elseif(hopipar(1).eq.1) then
         der=1.0_dop
      elseif(hopipar(1).ne.1) then
         ipower=hopipar(1)-1
         der=(x-hoprpar(2))**ipower
         der=hopipar(1)*der
      endif
      return

      case(4) ! sin^n(x)
      der=cos(x)
      if(hopipar(1).eq.0) then
         tmp=sin(x)
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         tmp=tmp**power1
         der=power*tmp*der
      elseif(hopipar(1).ne.1) then
         tmp=sin(x)
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         tmp=tmp**ipower1
         der=ipower*tmp*der
      endif
      return

      case(5) ! sin^n p(x-x0)
      der=hoprpar(2)*cos(hoprpar(2)*(x-hoprpar(3)))       
      if(hopipar(1).eq.0) then
         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         tmp=tmp**power1
         der=power*tmp*der
      elseif(hopipar(1).ne.1) then
         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         tmp=tmp**ipower1
         der=ipower*tmp*der
      endif
      return

      case(6) ! cos^n(x)
      der=-sin(x)
      if(hopipar(1).eq.0) then
         tmp=cos(x)
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         tmp=tmp**power1
         der=power*tmp*der
      elseif(hopipar(1).ne.1) then
         tmp=cos(x)
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         tmp=tmp**ipower1
         der=ipower*tmp*der
      endif
      return

      case(7) ! cos^n p(x-x0)
      der=-hoprpar(2)*sin(hoprpar(2)*(x-hoprpar(3)))       
      if(hopipar(1).eq.0) then
         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         tmp=tmp**power1
         der=power*tmp*der
      elseif(hopipar(1).ne.1) then
         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         tmp=tmp**ipower1
         der=ipower*tmp*der
      endif
      return

      case(12) ! gauss: exp(-a*(x-x0)^2)
      tmp=hoprpar(2)
! multiply coefficient by a power unless power = 1
      if(hopipar(1).eq.0) then
         tmp=hoprpar(1)*tmp
      elseif(hopipar(1).ne.1) then
         tmp=hopipar(1)*tmp
      endif
      r=x-hoprpar(3)
      der=exp(-tmp*r*r)
! prevent underflow
      if (der .lt. 1.0d-15) der = 0.0d0

      der=-2.0d0*tmp*r*der
      return

      case default
      routine='Der1anld'
      write(message,'(a,i5)') 'Derivative not found :',ifunc
      call errormsg

      end select der1

      end subroutine der1anld

!#######################################################################

      subroutine der2anld(der,x,ifunc,hopipar,hoprpar)

use global

      implicit none

      integer(long)                           :: ipower,ipower1,ipower2
      integer(long), intent(in)               :: ifunc
      integer(long), dimension(*), intent(in) :: hopipar
      real(dop)                               :: power,power1,power2
      real(dop)                               :: der2,tmp,r
      real(dop), intent(in)                   :: x
      real(dop), intent(out)                  :: der
      real(dop), dimension(*), intent(in)     :: hoprpar

      power = 1.0
      ipower = 1

      select case(ifunc)

      case(1)  ! unit
      der=1.0_dop
      return

      case(2) ! -1
      der=-1.0_dop
      return

      case(3) ! q^n
! case 1: real number exponent stored in hoprpar(1)
! case 2: integer power stored in hopipar(1)
! case 3: linear term
      if(hopipar(1).eq.0) then
         power=hoprpar(1)-2.0_dop
         power1=hoprpar(1)-1.0_dop
         der=(x-hoprpar(2))**power
         der=hoprpar(1)*power1*der
      elseif(hopipar(1).eq.2) then
         der=hopipar(1)
      elseif(hopipar(1).eq.1) then
         der=0.0_dop
      elseif(hopipar(1).eq.0) then
         der=0.0_dop
      else
         ipower=hopipar(1)-2
         ipower1=hopipar(1)-1
         der=(x-hoprpar(2))**ipower
         der=hopipar(1)*ipower1*der
      endif
      return

      case(4) ! sin^n(x)
      der=cos(x)
      der2=-sin(x)
      if(hopipar(1).eq.0) then
         tmp=sin(x)
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         power2=hoprpar(1)-2.0d0
         tmp=tmp**power2
         der=power*power1*tmp*der*der

         tmp=sin(x)
         tmp=tmp**power1
         der=power*tmp*der2

      elseif(hopipar(1).ne.1) then
         tmp=sin(x)
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         tmp=tmp**ipower2
         der=ipower*ipower1*tmp*der*der

         tmp=sin(x)
         tmp=tmp**ipower1
         der=der + ipower*tmp*der2
      endif
      return

      case(5) ! sin^n p(x-q)
      der=hoprpar(2)*cos(hoprpar(2)*(x-hoprpar(3)))       
      der2=sin(hoprpar(2)*(x-hoprpar(3)))       
      der2=-(hoprpar(2)**2)*der2
      if(hopipar(1).eq.0) then
         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         power2=hoprpar(1)-2.0d0
         tmp=tmp**power2
         der=power*power1*tmp*der*der

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power1
         der=der + power*tmp*der2

      elseif(hopipar(1).ne.1) then
         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         tmp=tmp**ipower2
         der=ipower*ipower1*tmp*der*der

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower1
         der=der + ipower*tmp*der2
      endif
      return

      case(6) ! cos^n(x)
      der=-cos(x)
      der2=-cos(x)
      if(hopipar(1).eq.0) then
         tmp=cos(x)
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         power2=hoprpar(1)-2.0d0
         tmp=tmp**power2
         der=power*power1*tmp*der*der

         tmp=cos(x)
         tmp=tmp**power1
         der=power*tmp*der2

      elseif(hopipar(1).ne.1) then
         tmp=cos(x)
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         tmp=tmp**ipower2
         der=ipower*ipower1**tmp*der*der

         tmp=cos(x)
         tmp=tmp**ipower1
         der=der + ipower*tmp*der2
      endif
      return

      case(7) ! cos^n p(x-q)
      der=-hoprpar(2)*sin(hoprpar(2)*(x-hoprpar(3)))       
      der2=cos(hoprpar(2)*(x-hoprpar(3)))       
      der2=-(hoprpar(2)**2)*der2
      if(hopipar(1).eq.0) then
         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         power2=hoprpar(1)-2.0d0
         tmp=tmp**power2
         der=power*power1*tmp*der*der

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power1
         der=der + power*tmp*der2

      elseif(hopipar(1).ne.1) then
         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         tmp=tmp**ipower2
         der=ipower*ipower1*tmp*der*der

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower1
         der=der + ipower*tmp*der2
      endif
      return

      case(12) ! gauss: exp(-a*(x-x0)^2)
      tmp=hoprpar(2)
! multiply coefficient by a power unless power = 1
      if(hopipar(1).eq.0) then
         tmp=hoprpar(1)*tmp
      elseif(hopipar(1).ne.1) then
         tmp=hopipar(1)*tmp
      endif
      r=x-hoprpar(3)
      der=exp(-tmp*r*r)
! prevent underflow
      if (der .lt. 1.0d-15) der = 0.0d0

      tmp = 2.0d0*tmp*r
      tmp = tmp*tmp
      tmp = -2.0d0*power*hoprpar(2) + tmp
      der = tmp*der

      return

      case default
      routine='Der2anld'
      write(message,'(a,i5)') 'Derivative not found :',ifunc
      call errormsg

      end select

      end subroutine der2anld


!#######################################################################

      subroutine der3anld(der,x,ifunc,hopipar,hoprpar)

use global

      implicit none

      integer(long)                           :: ipower,ipower1,ipower2,&
                                                 ipower3
      integer(long), intent(in)               :: ifunc
      integer(long), dimension(*), intent(in) :: hopipar
      real(dop)                               :: power,power1,power2,power3
      real(dop)                               :: der2,der3,tmp
      real(dop), intent(in)                   :: x
      real(dop), intent(out)                  :: der
      real(dop), dimension(*), intent(in)     :: hoprpar

      select case(ifunc)

      case(1) ! unit
      der=1.0_dop
      return

      case (2)! -1
      der=-1.0_dop
      return

      case(3) ! q^n
! case 1: real number exponent stored in hoprpar(1)
! case 2: integer power stored in hopipar(1)
! case 3: linear term
      if(hopipar(1).eq.0) then
         power=hoprpar(1)-3
         power1=hoprpar(1)-1
         power2=hoprpar(1)-2
         der=(x-hoprpar(2))**power
         der=hoprpar(1)*power1*power2*der
      elseif(hopipar(1).eq.3) then
         der=6.0_dop
      elseif(hopipar(1).eq.2) then
         der=0.0_dop
      elseif(hopipar(1).eq.1) then
         der=0.0_dop
      else
         ipower=hopipar(1)-3
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         der=(x-hoprpar(2))**ipower
         der=hopipar(1)*ipower1*ipower2*der
      endif
      return

     case(4) ! sin^n(x)
      der=cos(x)       
      der2=-sin(x)       
      der3=-cos(x)       
      if(hopipar(1).eq.0) then
         tmp=sin(x)
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         power2=hoprpar(1)-2.0d0
         power3=hoprpar(1)-3.0d0
         tmp=tmp**power3
         tmp=power*power1*power2*tmp
         der=tmp*(der**3)

         tmp=sin(x)
         tmp=tmp**power2
         tmp=power*power1*tmp
         der=der + 3.0d0*tmp*der2*der

         tmp=sin(x)
         tmp=tmp**power1
         der=der + power*tmp*der3

      elseif(hopipar(1).ne.1) then
         tmp=sin(x)
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         ipower3=hopipar(1)-3
         tmp=tmp**ipower3
         tmp=ipower1*ipower2*ipower3*tmp
         der=tmp*(der**3)

         tmp=sin(x)
         tmp=tmp**ipower2
         tmp=ipower*ipower1*tmp
         der=der + 3.0d0*tmp*der2*der

         tmp=sin(x)
         tmp=tmp**ipower1
         der=der + ipower*tmp*der3

      endif
      return

     case(5) ! sin^n p(x-q)
      der=hoprpar(2)*cos(hoprpar(2)*(x-hoprpar(3)))       
      der2=sin(hoprpar(2)*(x-hoprpar(3)))       
      der2=-(hoprpar(2)**2)*der2
      der3=cos(hoprpar(2)*(x-hoprpar(3)))       
      der3=-(hoprpar(2)**3)*der3
      if(hopipar(1).eq.0) then
         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         power2=hoprpar(1)-2.0d0
         power3=hoprpar(1)-3.0d0
         tmp=tmp**power3
         tmp=power*power1*power2*tmp
         der=tmp*(der**3)

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power2
         tmp=power*power1*tmp
         der=der + 3.0d0*tmp*der2*der

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power1
         der=der + power*tmp*der3

      elseif(hopipar(1).ne.1) then
         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         ipower3=hopipar(1)-3
         tmp=tmp**ipower3
         tmp=ipower*ipower1*ipower2*tmp
         der=tmp*(der**3)

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower2
         tmp=ipower*ipower1*tmp
         der=der + 3.0d0*tmp*der2*der

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower1
         der=der + ipower*tmp*der3

      endif
      return

     case(6) ! cos^n(x)
      der=-sin(x)       
      der2=-cos(x)       
      der3=sin(x)       
      if(hopipar(1).eq.0) then
         tmp=cos(x)
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         power2=hoprpar(1)-2.0d0
         power3=hoprpar(1)-3.0d0
         tmp=tmp**power3
         tmp=power*power1*power2*tmp
         der=tmp*(der**3)

         tmp=cos(x)
         tmp=tmp**power2
         tmp=power*power1*tmp
         der=der + 3.0d0*tmp*der2*der

         tmp=cos(x)
         tmp=tmp**power1
         der=der + power*tmp*der3

      elseif(hopipar(1).ne.1) then
         tmp=cos(x)
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         ipower3=hopipar(1)-3
         tmp=tmp**ipower3
         tmp=ipower*ipower1*ipower2*tmp
         der=tmp*(der**3)

         tmp=cos(x)
         tmp=tmp**ipower2
         tmp=ipower*ipower1*tmp
         der=der + 3.0d0*tmp*der2*der

         tmp=cos(x)
         tmp=tmp**ipower1
         der=der + ipower*tmp*der3
      endif
      return

     case(7) ! cos^n p(x-q)
      der=-hoprpar(2)*sin(hoprpar(2)*(x-hoprpar(3)))       
      der2=cos(hoprpar(2)*(x-hoprpar(3)))       
      der2=-(hoprpar(2)**2)*der2
      der3=sin(hoprpar(2)*(x-hoprpar(3)))       
      der3=(hoprpar(2)**3)*der3
      if(hopipar(1).eq.0) then
         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         power2=hoprpar(1)-2.0d0
         power3=hoprpar(1)-3.0d0
         tmp=tmp**power3
         tmp=power*power1*power2*tmp
         der=tmp*(der**3)

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power2
         tmp=power*power1*tmp
         der=der + 3.0d0*tmp*der2*der

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power1
         der=der + power*tmp*der3


      elseif(hopipar(1).ne.1) then
         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         ipower3=hopipar(1)-3
         tmp=tmp**ipower3
         tmp=ipower1*ipower2*ipower3*tmp
         der=tmp*(der**3)

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower2
         tmp=ipower*ipower1*tmp
         der=der + 3.0d0*tmp*der2*der

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower1
         der=der + ipower*tmp*der3
      endif
      return


      case default
      routine='Der3anld'
      write(message,'(a,i5)') 'Derivative not found :',ifunc
      call errormsg
   
      end select

      end subroutine der3anld

!#######################################################################

      subroutine der4anld(der,x,ifunc,hopipar,hoprpar)

use global

      implicit none

      integer(long)                           :: ipower,ipower1,ipower2,&
                                                 ipower3,ipower4
      integer(long), intent(in)               :: ifunc
      integer(long), dimension(*), intent(in) :: hopipar
      real(dop)                               :: power,power1,power2,power3,&
                                                 power4
      real(dop)                               :: der2,der3,der4,tmp
      real(dop), intent(in)                   :: x
      real(dop), intent(out)                  :: der
      real(dop), dimension(*), intent(in)     :: hoprpar

      select case(ifunc)

      case(1) ! unit
      der=1.0_dop
      return

      case (2)! -1
      der=-1.0_dop
      return

      case(3) ! q^n
! case 1: real number exponent stored in hoprpar(1)
! case 2: integer power stored in hopipar(1)
! case 3: linear term
      if(hopipar(1).eq.0) then
         power=hoprpar(1)-4
         power1=hoprpar(1)-1
         power2=hoprpar(1)-2
         power3=hoprpar(1)-3
         der=(x-hoprpar(2))**power
         der=hoprpar(1)*power1*power2*power3*der
      elseif(hopipar(1).eq.4) then
         der=24.0_dop
      elseif(hopipar(1).eq.3) then
         der=0.0_dop
      elseif(hopipar(1).eq.2) then
         der=0.0_dop
      elseif(hopipar(1).eq.1) then
         der=0.0_dop
      else
         ipower=hopipar(1)-4
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         ipower3=hopipar(1)-3
         der=(x-hoprpar(2))**ipower
         der=hopipar(1)*ipower1*ipower2*ipower3*der
      endif
      return

     case(4) ! sin^n x
      der=cos(x)       
      der2=-sin(x)       
      der3=-cos(x)       
      der4=sin(x)       
      if(hopipar(1).eq.0) then
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         power2=hoprpar(1)-2.0d0
         power3=hoprpar(1)-3.0d0
         power4=hoprpar(1)-4.0d0

         tmp=sin(x)
         tmp=tmp**power4
         tmp=power*power1*power2*power3*tmp
         der=tmp*(der**4)

         tmp=sin(x)
         tmp=tmp**power3
         tmp=power*power1*power2*tmp
         der=der + 6.0d0*tmp*der2*der*der

         tmp=sin(x)
         tmp=tmp**power2
         tmp=power*power1*tmp
         der=der + tmp*(3.0d0*der2*der2 + 4.0d0*der*der3)

         tmp=sin(x)
         tmp=tmp**power1
         tmp=power*tmp
         der=der + tmp*der4

      elseif(hopipar(1).ne.1) then
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         ipower3=hopipar(1)-3
         ipower4=hopipar(1)-4

         tmp=sin(x)
         tmp=tmp**ipower4
         tmp=ipower*ipower1*ipower2*ipower3*tmp
         der=tmp*(der**4)

         tmp=sin(x)
         tmp=tmp**ipower3
         tmp=ipower*ipower1*ipower2*tmp
         der=der + 6.0d0*tmp*der2*der*der

         tmp=sin(x)
         tmp=tmp**ipower2
         tmp=ipower*ipower1*tmp
         der=der + tmp*(3.0d0*der2*der2 + 4.0d0*der*der3)

         tmp=sin(x)
         tmp=tmp**ipower1
         tmp=ipower*tmp
         der=der + tmp*der4

      endif
      return

     case(5) ! sin^n p(x-q)
      der=hoprpar(2)*cos(hoprpar(2)*(x-hoprpar(3)))       
      der2=sin(hoprpar(2)*(x-hoprpar(3)))       
      der2=-(hoprpar(2)**2)*der2
      der3=cos(hoprpar(2)*(x-hoprpar(3)))       
      der3=-(hoprpar(2)**3)*der3
      der4=sin(hoprpar(2)*(x-hoprpar(3)))       
      der4=(hoprpar(2)**4)*der4
      if(hopipar(1).eq.0) then
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         power2=hoprpar(1)-2.0d0
         power3=hoprpar(1)-3.0d0
         power4=hoprpar(1)-4.0d0

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power4
         tmp=power*power1*power2*power3*tmp
         der=tmp*(der**4)

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power3
         tmp=power*power1*power2*tmp
         der=der + 6.0d0*tmp*der2*der*der

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power2
         tmp=power*power1*tmp
         der=der + tmp*(3.0d0*der2*der2 + 4.0d0*der*der3)

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power1
         tmp=power*tmp
         der=der + tmp*der4

      elseif(hopipar(1).ne.1) then
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         ipower3=hopipar(1)-3
         ipower4=hopipar(1)-4

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower4
         tmp=ipower*ipower1*ipower2*ipower3*tmp
         der=tmp*(der**4)

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower3
         tmp=ipower*ipower1*ipower2*tmp
         der=der + 6.0d0*tmp*der2*der*der

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower2
         tmp=ipower*ipower1*tmp
         der=der + tmp*(3.0d0*der2*der2 + 4.0d0*der*der3)

         tmp=sin(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower1
         tmp=ipower*tmp
         der=der + tmp*der4

      endif
      return

     case(6) ! cos^n x
      der=-sin(x)       
      der2=-cos(x)       
      der3=sin(x)       
      der4=cos(x)       
      if(hopipar(1).eq.0) then
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         power2=hoprpar(1)-2.0d0
         power3=hoprpar(1)-3.0d0
         power4=hoprpar(1)-4.0d0

         tmp=cos(x)
         tmp=tmp**power4
         tmp=power*power1*power2*power3*tmp
         der=tmp*(der**4)

         tmp=cos(x)
         tmp=tmp**power3
         tmp=power*power1*power2*tmp
         der=der + 6.0d0*tmp*der2*der*der

         tmp=cos(x)
         tmp=tmp**power2
         tmp=power*power1*tmp
         der=der + tmp*(3.0d0*der2*der2 + 4.0d0*der*der3)

         tmp=cos(x)
         tmp=tmp**power1
         tmp=power*tmp
         der=der + tmp*der4

      elseif(hopipar(1).ne.1) then
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         ipower3=hopipar(1)-3
         ipower4=hopipar(1)-4

         tmp=cos(x)
         tmp=tmp**ipower4
         tmp=ipower*ipower1*ipower2*ipower3*tmp
         der=tmp*(der**4)

         tmp=cos(x)
         tmp=tmp**ipower3
         tmp=ipower*ipower1*ipower2*tmp
         der=der + 6.0d0*tmp*der2*der*der

         tmp=cos(x)
         tmp=tmp**ipower2
         tmp=ipower*ipower1*tmp
         der=der + tmp*(3.0d0*der2*der2 + 4.0d0*der*der3)

         tmp=cos(x)
         tmp=tmp**ipower1
         tmp=ipower*tmp
         der=der + tmp*der4

      endif
      return

     case(7) ! cos^n p(x-q)
      der=-hoprpar(2)*sin(hoprpar(2)*(x-hoprpar(3)))       
      der2=cos(hoprpar(2)*(x-hoprpar(3)))       
      der2=-(hoprpar(2)**2)*der2
      der3=sin(hoprpar(2)*(x-hoprpar(3)))       
      der3=(hoprpar(2)**3)*der3
      der4=cos(hoprpar(2)*(x-hoprpar(3)))       
      der4=(hoprpar(2)**4)*der4
      if(hopipar(1).eq.0) then
         power=hoprpar(1)
         power1=hoprpar(1)-1.0d0
         power2=hoprpar(1)-2.0d0
         power3=hoprpar(1)-3.0d0
         power4=hoprpar(1)-4.0d0

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power4
         tmp=power*power1*power2*power3*tmp
         der=tmp*(der**4)

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power3
         tmp=power*power1*power2*tmp
         der=der + 6.0d0*tmp*der2*der*der

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power2
         tmp=power*power1*tmp
         der=der + tmp*(3.0d0*der2*der2 + 4.0d0*der*der3)

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**power1
         tmp=power*tmp
         der=der + tmp*der4

      elseif(hopipar(1).ne.1) then
         ipower=hopipar(1)
         ipower1=hopipar(1)-1
         ipower2=hopipar(1)-2
         ipower3=hopipar(1)-3
         ipower4=hopipar(1)-4

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower4
         tmp=ipower*ipower1*ipower2*ipower3*tmp
         der=tmp*(der**4)

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower3
         tmp=ipower*ipower1*ipower2*tmp
         der=der + 6.0d0*tmp*der2*der*der

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower2
         tmp=ipower*ipower1*tmp
         der=der + tmp*(3.0d0*der2*der2 + 4.0d0*der*der3)

         tmp=cos(hoprpar(2)*(x-hoprpar(3)))       
         tmp=tmp**ipower1
         tmp=ipower*tmp
         der=der + tmp*der4

      endif
      return

      case default
      routine='Der3anld'
      write(message,'(a,i5)') 'Derivative not found :',ifunc
      call errormsg
   
      end select

      end subroutine der4anld

      end module deranld
