      module diabatmod

use decimal, only: long, sip, dop
use logdat
use global
use griddatmod
use operdef
use dirdyn
use channels

use op1lib, only: unitqxd,tranqxd,cpvxd,cpmxd
use dd_db
use mmlib, only: qqxxdd,qqtxdd,qqxtdd,qqxxzd,qqtxdz
use funcqchemmod, only: lhaqchem, wrmolpro_geom,rdmolpro_ao,rdmolpro_ao_1, &
                        rdmolpro_ci,molpro_moover,&
                        citrans,molpro_wfover

      implicit none                                                    

! diabat3 (global diabatisation with full hessian updating) removed 8/22 as 
! it no longer worked
      public :: diabat1,diabat2,diabat4,diabat6

      contains

!#######################################################################
!
! simple regularization
!#######################################################################

      subroutine diabat1(xgpoint,av,aderiv1,aderiv2,v,deriv1,deriv2,order)

      implicit none

      integer(long)                                          :: s,s1,f,f1
      integer(long), intent(in)                              :: order
      real(dop)                                              :: sigma,delta, &
                                                                del1g,sig1, &
                                                                sig2,del2, &
                                                                 sign,delratio
      real(dop), dimension(ndofddpes)                        :: del1
      real(dop), dimension(ndofddpes), intent(in)            :: xgpoint
      real(dop), dimension(nddstate,nddstate), intent(out) :: v
      real(dop), dimension(ndofddpes,nddstate,nddstate), intent(out) :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), intent(out) :: deriv2
      real(dop), dimension(nddstate,nddstate), intent(in) :: av
      real(dop), dimension(ndofddpes,nddstate,nddstate), intent(in) :: aderiv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), intent(in) :: aderiv2
      real(dop), dimension(nddstate,nddstate)             :: w11
      real(dop)                                              :: tmp,tmp1,tmp2
!      real(dop), parameter                                   :: smooth=0.005 !a suitable general value needs to be found, or make this input

      interface
        function ddot(n,dx,incx,dy,incy)
          use decimal, only: dop
          real(dop) :: ddot
          integer :: n,incx,incy
          real(dop),dimension(n) :: dx,dy
        end function ddot
      end interface

!
! scalar term
!
      sigma = 0.5_dop*(av(2,2)+av(1,1))
      delta = 0.5_dop*(av(2,2)-av(1,1))

! Scalar products of gradient diff. and deriv. coup. vectors with the 
! vector between the current point and the CI respectively.
      tmp=ddot(ndofddpes,xgd0,1,xgpoint(:)-xcoin(:ndofddpes),1)
      tmp1=ddot(ndofddpes,xdcp0,1,xgpoint(:)-xcoin(:ndofddpes),1)

      w11(1,1) = -tmp
      w11(2,2) = tmp
      w11(1,2) = tmp1
      w11(2,1) = tmp1
      del1g = sqrt(tmp**2 + tmp1**2)

!
! ensure that delta/del1g -> 1 as intersection is reached
! this is in general not needed and not the major source of problems
!
      delratio = delta/del1g
!      delratio = delratio*(1.0_dop - exp(-(del1g/smooth)**2)) + exp(-(del1g/smooth)**2)

      do s=1,nddstate
         v(s,s) = sigma + delratio*w11(s,s)
         do s1=s+1,nddstate
            v(s1,s) = delratio*w11(s1,s)
            v(s,s1) = v(s1,s)
         enddo

      enddo
!
! linear term
!
      do f=1,ndofddpes
         sig1 = 0.5_dop*(aderiv1(f,2,2)+aderiv1(f,1,1))
         del1(f) = 0.5_dop*(aderiv1(f,2,2)-aderiv1(f,1,1))
         tmp1 = w11(1,2)*xdcp0(f) + w11(2,2)*xgd0(f)
         do s=1,nddstate
            if (s .eq. 1) then
               sign=-1.0_dop
            else
               sign=1.0_dop
            endif
            deriv1(f,s,s) = sig1 
            deriv1(f,s,s) = deriv1(f,s,s)+del1(f)*w11(s,s)/del1g
            deriv1(f,s,s) = deriv1(f,s,s)+sign*delratio*xgd0(f)
            deriv1(f,s,s) = deriv1(f,s,s)-delratio*w11(s,s)*tmp1/del1g**2

            do s1=s+1,nddstate
               deriv1(f,s1,s) = del1(f)*w11(s1,s)/del1g
               deriv1(f,s1,s) = deriv1(f,s1,s)+delratio*xdcp0(f)
               deriv1(f,s1,s) = deriv1(f,s1,s)-delratio*w11(s1,s)*tmp1/del1g**2

               deriv1(f,s,s1) = deriv1(f,s1,s)
            enddo
         enddo
      enddo

!
! bilinear term
!
      if (order .ge. 2) then
        do f=1,ndofddpes
           tmp1 = w11(1,2)*xdcp0(f) + w11(2,2)*xgd0(f)
 
           do f1=f,ndofddpes
              sig2 = 0.5_dop*(aderiv2(f1,f,2,2)+aderiv2(f1,f,1,1))
              del2 = 0.5_dop*(aderiv2(f1,f,2,2)-aderiv2(f1,f,1,1))
              tmp2 = w11(1,2)*xdcp0(f1) + w11(2,2)*xgd0(f1)
 
              do s=1,nddstate
                 if (s .eq. 1) then
                    sign=-1.0_dop
                 else
                    sign=1.0_dop
                 endif
                 deriv2(f1,f,s,s) = sig2 
                 deriv2(f1,f,s,s) = deriv2(f1,f,s,s)+del2*w11(s,s)/del1g
                 deriv2(f1,f,s,s) = deriv2(f1,f,s,s)+sign*del1(f)*xgd0(f1)/del1g
                 deriv2(f1,f,s,s) = deriv2(f1,f,s,s)+sign*del1(f1)*xgd0(f)/del1g
                 deriv2(f1,f,s,s) = deriv2(f1,f,s,s)- &
                            del1(f)*w11(s,s)*tmp2/del1g**3
                 deriv2(f1,f,s,s) = deriv2(f1,f,s,s)- &
                            del1(f1)*w11(s,s)*tmp1/del1g**3
                 deriv2(f1,f,s,s) = deriv2(f1,f,s,s)+ &
                          3.0_dop*delratio*w11(s,s)*tmp1*tmp2/del1g**4
                 deriv2(f1,f,s,s) = deriv2(f1,f,s,s)- &
                          sign*delratio*tmp2*xgd0(f)/del1g**2
                 deriv2(f1,f,s,s) = deriv2(f1,f,s,s)- &
                          sign*delratio*tmp1*xgd0(f1)/del1g**2
                 deriv2(f1,f,s,s) = deriv2(f1,f,s,s)- &
                   delratio*w11(s,s)*(xgd0(f)*xgd0(f1) + xdcp0(f)*xdcp0(f1))/del1g**2
 
                 deriv2(f,f1,s,s) = deriv2(f1,f,s,s)
 
                 do s1=s+1,nddstate
                    deriv2(f1,f,s1,s) = del2*w11(s1,s)/del1g
                    deriv2(f1,f,s1,s) = deriv2(f1,f,s1,s)+del1(f)*xdcp0(f1)/del1g
                    deriv2(f1,f,s1,s) = deriv2(f1,f,s1,s)- &
                         del1(f)*w11(s1,s)*tmp2/del1g**3
                    deriv2(f1,f,s1,s) = deriv2(f1,f,s1,s)+del1(f1)*xdcp0(f)/del1g
                    deriv2(f1,f,s1,s) = deriv2(f1,f,s1,s)- &
                         del1(f1)*w11(s1,s)*tmp1/del1g**3
                    deriv2(f1,f,s1,s) = deriv2(f1,f,s1,s)+ &
                          3.0_dop*delratio*w11(s1,s)*tmp1*tmp2/del1g**4
                    deriv2(f1,f,s1,s) = deriv2(f1,f,s1,s)- &
                          delratio*tmp2*xdcp0(f)/del1g**2
                    deriv2(f1,f,s1,s) = deriv2(f1,f,s1,s)- &
                          delratio*tmp1*xdcp0(f1)/del1g**2
                    deriv2(f1,f,s1,s) = deriv2(f1,f,s1,s)- delratio* &
                          w11(s1,s)*(xgd0(f)*xgd0(f1) + xdcp0(f)*xdcp0(f1))/del1g**2
 
                    deriv2(f,f1,s1,s) = deriv2(f1,f,s1,s)
                    deriv2(f,f1,s,s1) = deriv2(f1,f,s1,s)
                    deriv2(f1,f,s,s1) = deriv2(f1,f,s1,s)
                 enddo
              enddo
 
           enddo
        enddo
      endif

      return
      end subroutine diabat1

!#######################################################################
!
! Local diabatisation.
!
! Subroutine diabat2 works together with rotcoeffgwp (mctdhlib/gwplib).
! Coordinates and PES derivatives are stored in DB coordinates
! but saved in system coordinates for rotating the A-coefficients.
!
! Here, a crossing is to be understood from a local-diabatic perspective:
! the crossing line is orthogonal to the previous/nearest GD vector and
! a crossing thus corresponds to a sudden change of sign of the GD vector
! (sudden half-loop). A slow crossing will not be detected as such.
! A swapping means that GD and DC have swapped their roles over a step
! (sudden quarter-loop). A slow swapping will not be detected as such.
!
! Possible changes:
! - add an alternative option to accumulate the angle and define a more
!   general diabatic Hamiltonian (together with a local regularisation?);
! - only use DB coordinates (?);
! - when a conical intersection is reached, reorient BS vectors wrt.
!   reference gauge (?);
! - determine conical intersection along some direction (GD or average
!   gradient?) and reformulate expansion as a sort of regularised one (?);
! - at t = 0, unpopulated functions may use central PES rather than PES
!   shifted to displaced point, as this generates extrapolation error.
! 
! To be improved:
! - use cumulative GD and DC angles to measure the extent of looping;
! - make this consistent with possible non-orthogonal BS-like vectors; 
! - add test on average gradient (tilt);
! - add test on vector product (atom per atom) of GD and DC;
! - check if energy difference is increasing or decreasing;
! - restricted to two states but could be generalised;
! - an even number of crossings is treated at the moment as an odd number
!   of crossings, but this should not be the case;
! - situations of non-overlapping BS-like vectors may not be treated
!   adequately (add test to check for intruder states);
! - currently requires no averaging over DB, as this may cause funny
!   zeroing of BS vectors around a conical intersection (however,
!   averaging could be tolerated in regions with large energy gap);
! - DB nearest point should be used rather than previous point to
!   fix the phase of the off-diagonal gradient, as this would make the
!   procedure geometry-dependent rather than time-dependent;
!   similarly, BS initialisation should be made when DB is still empty
!   rather than when t = 0;
! - input keywords should be changed to reflect that phase and side
!   initialisations are made with respect to the BS-like vectors of
!   the starting point rather than a conical intersection,
!   which is misleading (and reference point is redundant at the moment);
! - ambiguity test should be more sophisticated with several subcases;
! - determine which "use only" are required.
! BL 06/16
!#######################################################################

      subroutine diabat2(xgpoint,av,aderiv1,aderiv2,v,deriv1,deriv2,order,e,m,time)

      implicit none

      integer(long), intent(in)                                                :: order,e,m
      real(dop), intent(in)                                                    :: time
      real(dop), dimension(nddstate,nddstate), intent(out)                     :: v
      real(dop), dimension(ndofddpes,nddstate,nddstate), intent(out)           :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), intent(out) :: deriv2
      real(dop), dimension(ndofddpes), intent(in)                              :: xgpoint
      real(dop), dimension(nddstate,nddstate), intent(in)                      :: av
      real(dop), dimension(ndofddpes,nddstate,nddstate), intent(in)            :: aderiv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), intent(in)  :: aderiv2

      integer(long)                                                            :: s,s1,f,f1
      real(dop)                                                                :: r1,r2,a12,rx,ry,axy,x1,y2,x2,y1,side,phase

!
! Save time for consistency check in rotcoeffgwp.
!
      diab2tim(e)=time
!
! Initialise phase and side at t = 0 with respect to reference BS-like vectors
! (only two states assumed here).
! Initialise dia-to-adia rotation angle to zero.
! Initialise BS-like rotation angles to zero.
! Note: xrefpoint could be used to define alternative gauge (H12 = 0).
!
      if (abs(time)<tiny) then
         diab2det(e)=1
         diab2pre(e)=1
         diab2ang(e)=0.0_dop
         diab2bs1(1:ndofddpes,e)=xgd0(1:ndofddpes)
         diab2bs2(1:ndofddpes,e)=xdcp0(1:ndofddpes)
         diab2agd(e)=0.0_dop
         diab2adc(e)=0.0_dop
         if (ddlog) write(ilog,'(a,2i7,f12.6)') &
            'Initialising BS vectors for GWP, mode, time: ',e,m,time/fs
      endif
!
! Check for a conical intersection (within mindv threshold).
! Note: BS vectors may have to be re-oriented according to a reference gauge.
!
      do s=1,nddstate-1
         if (abs(av(s+1,s+1)-av(s,s))<dddvmin) then
            if (ddlog) write(ilog,'(a,2i7,f12.6,2i7)') &
               'Conical intersection approached by GWP, mode, time, states: ', &
               e,m,time/fs,s+1,s
         endif
      enddo

!
! Initialise local quadratic diabatic Hamiltonian to zero.
!
      v(:,:)=0.0_dop
      deriv1(:,:,:)=0.0_dop
      deriv2(:,:,:,:)=0.0_dop
!
! scalar term
!
      do s=1,nddstate
         v(s,s) = av(s,s)
      enddo
!
! linear term
!
      do f=1,ndofddpes
         do s=1,nddstate
            deriv1(f,s,s) = aderiv1(f,s,s)
            do s1=s+1,nddstate
               deriv1(f,s1,s) = aderiv1(f,s1,s)
               deriv1(f,s,s1) = deriv1(f,s1,s)
            enddo
         enddo
      enddo
!
! Determine if a crossing or a swapping is detected.
! Fix random phase of DC against previous/nearest BS-like vectors
! and update the latter (only two states assumed here).
! For a single crossing, BS-like vectors on both sides have negative overlaps.
! The case of a double crossing (curved seam) is not treated correctly by
! assumed to be rare enough.
!
      r1=sqrt(dot_product(diab2bs1(1:ndofddpes,e),diab2bs1(1:ndofddpes,e)))
      r2=sqrt(dot_product(diab2bs2(1:ndofddpes,e),diab2bs2(1:ndofddpes,e)))
      a12=acos(dot_product(diab2bs1(1:ndofddpes,e),diab2bs2(1:ndofddpes,e))/r1/r2)
      rx=0.5_dop*sqrt(dot_product(deriv1(1:ndofddpes,nddstate,nddstate)-deriv1(1:ndofddpes,nddstate-1,nddstate-1), &
                                  deriv1(1:ndofddpes,nddstate,nddstate)-deriv1(1:ndofddpes,nddstate-1,nddstate-1)))
      ry=sqrt(dot_product(deriv1(1:ndofddpes,nddstate,nddstate-1),deriv1(1:ndofddpes,nddstate,nddstate-1)))
      axy=acos(0.5_dop*dot_product(deriv1(1:ndofddpes,nddstate,nddstate)-deriv1(1:ndofddpes,nddstate-1,nddstate-1), &
                                   deriv1(1:ndofddpes,nddstate,nddstate-1))/rx/ry)
      x1=0.5_dop*dot_product(deriv1(1:ndofddpes,nddstate,nddstate)-deriv1(1:ndofddpes,nddstate-1,nddstate-1), &
                             diab2bs1(1:ndofddpes,e))
      y2=dot_product(deriv1(1:ndofddpes,nddstate,nddstate-1),diab2bs2(1:ndofddpes,e))
      x2=0.5_dop*dot_product(deriv1(1:ndofddpes,nddstate,nddstate)-deriv1(1:ndofddpes,nddstate-1,nddstate-1), &
                             diab2bs2(1:ndofddpes,e))
      y1=dot_product(deriv1(1:ndofddpes,nddstate,nddstate-1),diab2bs1(1:ndofddpes,e))
      if (abs(x1) .gt. 1.d-7) then
         side=sign(1.0_dop,x1)
         phase=sign(1.0_dop,y2)
!         diab2agd(e)=diab2agd(e)+... !!! to be continued !!!
         if (side .lt. 0.0_dop) then
            diab2det(e)=-diab2det(e)
            if (ddlog) write(ilog,'(a,2i7,f12.6)') &
               'Crossing detected for GWP, mode, time: ',e,m,time/fs
         endif
      elseif ((abs(x1) .le. 1.d-7) .and. (abs(x2) .gt. 1.d-7)) then
         side=sign(1.0_dop,x2)
         phase=-sign(1.0_dop,y1)
         if (ddlog) write(ilog,'(a,2i7,f12.6)') &
            'Swapping detected for GWP, mode, time: ',e,m,time/fs
      else
         side=1.0_dop
         phase=sign(1.0_dop,y2)
         if (ddlog) write(ilog,'(a,2i7,f12.6)') &
            'Ambiguity detected for GWP, mode, time: ',e,m,time/fs
      endif
      deriv1(1:ndofddpes,nddstate,nddstate-1)=side*phase*deriv1(1:ndofddpes,nddstate,nddstate-1)
      deriv1(1:ndofddpes,nddstate-1,nddstate)=deriv1(1:ndofddpes,nddstate,nddstate-1)
      diab2bs1(1:ndofddpes,e)=0.5_dop*(deriv1(1:ndofddpes,nddstate,nddstate)-deriv1(1:ndofddpes,nddstate-1,nddstate-1))
      diab2bs2(1:ndofddpes,e)=deriv1(1:ndofddpes,nddstate,nddstate-1)
!
! bilinear term
!
      if (order .lt. 2) go to 100
      do f=1,ndofddpes
         do f1=f,ndofddpes
            do s=1,nddstate
               deriv2(f1,f,s,s) = aderiv2(f1,f,s,s)
               do s1=1,nddstate
                  if (s1 .ne. s) then
                     deriv2(f1,f,s,s) = deriv2(f1,f,s,s)- &
                        deriv1(f1,s,s1)*deriv1(f,s1,s)/ &
                        (v(s,s)-v(s1,s1))
                  endif
               enddo
               deriv2(f,f1,s,s) = deriv2(f1,f,s,s)
               do s1=s+1,nddstate
                  deriv2(f1,f,s1,s) = 0.0_dop
                  deriv2(f,f1,s1,s) = deriv2(f1,f,s1,s)
                  deriv2(f,f1,s,s1) = deriv2(f1,f,s1,s)
                  deriv2(f1,f,s,s1) = deriv2(f1,f,s1,s)
               enddo
            enddo
         enddo
      enddo
  100 continue
!
! save current data
!
      if (lddtrans) then
         call ddx2q(xgpoint,diab2geo(1,e))
      else
         diab2geo(:ndofddpes,e)=xgpoint
      endif
      do s=1,nddstate
         do s1=1,nddstate
            diab2pes(s1,s,e)=v(s1,s)
            if (lddtrans) then
               call ddf2q(deriv1(1,s1,s),diab2gra(1,s1,s,e))
               call ddfc2q(deriv2(1,1,s1,s),diab2hes(1,1,s1,s,e))
            else
               diab2gra(:ndofddpes,s1,s,e)=deriv1(:,s1,s)
               diab2hes(:ndofddpes,:ndofddpes,s1,s,e)=deriv2(:,:,s1,s)
            endif
         enddo
      enddo
!
! For initial time, save second set as first set to be used to rotate
! the A-coefficients.
!
      if (abs(time)<tiny) then
         diabogeo(1:ndofddpes,e)=diab2geo(1:ndofddpes,e)
         diabopes(1:nddstate,1:nddstate,e)=diab2pes(1:nddstate,1:nddstate,e)
         diabogra(1:ndofddpes,1:nddstate,1:nddstate,e)=diab2gra(1:ndofddpes,1:nddstate,1:nddstate,e)
         diabohes(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,e)=diab2hes(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,e)
      endif

      return
      end subroutine diabat2

!#######################################################################
!
! Global diabatisation by predictor-corrector method.
! simplified version
!
! adtrans is defined by V = S W S^T
!
! idbloc is location in DB of nearest point
!
! num_gp has to be set for #GWP if local_db used
!#######################################################################

      subroutine diabat4(xgpoint,av,aderiv1,aderiv2,dercp,&
           v,deriv1,deriv2,adttrans,linterp0,idbloc0,nrec,interp)


      implicit none

      integer(long), intent(in)                      :: idbloc0,nrec
      integer(long), intent(out)                     :: interp
      integer(long), dimension(nrec)                 :: loc
      integer(long)                                  :: irec,jrec
      real(dop), dimension(ndofddpes),intent(in)     :: xgpoint

      real(dop), dimension(ndofddpes,nactdim), intent(inout) :: dercp
      real(dop), dimension(nddstate,nddstate), intent(inout) :: av
      real(dop), dimension(ndofddpes,nddstate,nddstate), &
                                               intent(inout) :: aderiv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), &
                                                  intent(in) :: aderiv2

      real(dop), dimension(nddstate,nddstate) :: v,v0,av0
      real(dop), dimension(nddstate,nddstate) :: adttrans,trans0
      real(dop), dimension(ndofddpes,nddstate,nddstate) :: deriv1,aderiv10
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate) :: deriv2

      real(dop), dimension(nddstate,nddstate) :: av1
      real(dop), dimension(ndofddpes,nddstate,nddstate) :: aderiv11
      real(dop), dimension(ndofddpes,nactdim)           :: dercp1

      real(dop), dimension(ndofddpes,nddstate,nddstate) :: tmpderiv1,&
                                                           tmpderiv1a
      real(dop), dimension(ndofddpes,nactdim)           :: deldercp
      real(dop), dimension(nddstate)                    :: dele

      logical(kind=4), intent(in)                :: linterp0
      logical(kind=4)                            :: linterp,linterp1,&
                                                    linterp2,linterp3
      logical(kind=4)                            :: lforce
      logical(kind=4), dimension(nddstate)       :: ldegen
      logical(kind=4)                            :: lcheck

      integer(long)                              :: idx,jdx,kdx,ldx,mdx,&
                                                    info,istep,nstep,&
                                                    ierr,si,sf,idbloc
      integer(long)                              :: diabint0
      integer(long), dimension(nddstate,nddstate) :: dcpover
      real(dop)                                  :: diff0,mindist,mindisp
      real(dop)                                  :: verr
      integer(long), dimension(nddstate)         :: pivots
      real(dop), dimension(ndofddpes)            :: delx
      real(dop), dimension(nddstate,nddstate)    :: iminusp,inttmp,&
                                                    iplusp,ipluspinv,inttmp2,&
                                                    inttmp3
      real(dop), dimension(3*nddstate)           :: work

      real(dop), dimension(nddstate)             :: tmpe,tmpe2
      real(dop), allocatable                     :: r1(:)
      real(dop), dimension(2)                    :: shepdist,weight
      real(dop)                                  :: norm
      real(dop), dimension(:,:), allocatable     :: vstep
                                                    
      integer(long)                              :: dbnrec1,ibest,minscore
      real(dop), dimension(dbnrec)               :: dist
      integer(long), dimension(dbnrec)           :: dbignore0
      real(dop), dimension(3,nddstate,nddstate)  :: tmpdip

! nblock is no. of blocks of spin states
! nstblck(i) is no. of states in block(i)
      integer(long)                              :: nblock
      integer(long), dimension(2)                :: nstblck
      integer(long)                              :: iscore
      real(dop), dimension(:,:), allocatable     :: transtmp,transtmp1

      real(dop), dimension(ndofddpes)            :: xgp
      real(dop), dimension(nactdim)              :: integral,dotp
      real(dop)                                  :: dotp1,dotp3,mindotp1
      real(dop), dimension(nactdim)              :: dotp2
      real(dop), dimension(ndofddpes,nactdim)    :: nact,sumint

      integer(long), dimension(nddstate)  :: diabidx1,diabidx2,diabidx3,&
                                             diabidx4
      logical(kind=4)                     :: lflip,lflip1,lmodfail
      logical(kind=4),dimension(nddstate) :: lintruder

      type(nextptr), pointer :: current

      routine = 'diabat4'

!-----------------------------------------------------------------------
! logicals and flags controlling interpolation to be made
! 
! interp: interpolation flag. If non-zero interpolation has been performed
! If linterp0 = .true., passed to routine if QC has failed and no data.
!                       interpolate DB and store interpolated diab + adiab
!                       interp = -1
! If linterp1 = .true., interpolate using DB but store original adiab
!                       interp = 1 in region of degeneracy where QC not 
!                                  to be trusted
! If linterp2 = .true., interpolate using calculated adiabatic energies 
!                       but add pi shift for states that swap (flip)
!                       interp = 2 
! If linterp3 = .true., intruder state with 0 DCP in present plane
!                       adiabatic energies at new point and unit trafo 
!                       interp = 3 
!-----------------------------------------------------------------------
      interp=0
      linterp =.false.
      linterp1=.false.
      linterp2=.false.
      linterp3=.false.

      lforce = .true.  ! Calculate new point from DB in dddb_rd
!-----------------------------------------------------------------------
! Store dbignore flags that may be overwritten here...
!-----------------------------------------------------------------------
      dbignore0(1:dbnrec) = dbignore(1:dbnrec)

!-----------------------------------------------------------------------
! If this is the first call, we set the transformation matrix to the unit
! matrix and transform the adiabatic quantities to the diabatic.
!-----------------------------------------------------------------------
      if(dbnrec.eq.0)then
! if first QC calc has failed, stop.
         if(linterp0)then
            routine = 'Diabat4 (diabatmod.f90)'
            message = 'First QC calculation failed'
            call errormsg
         endif
            
         linterp = .false.
         do ldx = 1,nddstate-1
            if (imultmap(ldx) .ne. imultmap(ldx+1)) cycle
            if (ldx .ge. mxsinterest) cycle

            diff0 = av(ldx+1,ldx+1) - av(ldx,ldx)

            if (abs(diff0) .lt. dbdegen) then 
                linterp = .true.
                write(ilog,'(a,2i5)')'States near degeneracy : ',ldx,ldx+1
            endif
         enddo
         if (lnodegen0) linterp = .false.
         if (linterp) then
            routine = 'Diabat4 (diabatmod.f90)'
            message = 'First QC calculation at degeneracy. '// &
                'Choose a different reference point using a diabref-section' 
            call errormsg
         endif

         call unitqxd(adttrans,nddstate)
         call transform(adttrans,av(1:nddstate,1:nddstate),&
              aderiv1(1:ndofddpes,1:nddstate,1:nddstate),&
              aderiv2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate),&
              v,deriv1,deriv2,dercp(1:ndofddpes,1:nactdim),&
              ndofddpes,nddstate)

         go to 999     ! Go to End of first call
      endif

!-----------------------------------------------------------------------
! Local DBs: set up array with locations in full DB
!-----------------------------------------------------------------------
      if (ldbsmall) then
         call list_init(Current)
         current => ngp_loc(num_gp)%locpt
         do irec = 1,dbnrec_gp(num_gp)
            loc(irec) = current%locDB
            current => current%Next
         enddo
      else
         do irec = 1,dbnrec
            loc(irec) = irec
         enddo
      endif

!-----------------------------------------------------------------------
! If both singlets and triplets, get no. of each.
! We need to do diabatisation only within a set of spin states. These
! are ordered in blocks in av etc.
!-----------------------------------------------------------------------
      if (lcasocdb) then
         nblock=1
         idx=1
         nstblck(idx)=1
! read imultmap
         do si=2,nddstate
            if (imultmap(si) .ne. imultmap(si-1)) then
               nblock=nblock+1
               idx=idx+1
               nstblck(idx)=1
            else
               nstblck(idx)=nstblck(idx)+1
            endif
         enddo
      else
         nblock=1
         nstblck(1)=nddstate
      endif

!-----------------------------------------------------------------------
! Check adiabatic energies are correctly ordered
!-----------------------------------------------------------------------
      lcheck = .false.
      do idx = 2,nddstate
         if (imultmap(idx) .ne. imultmap(idx-1)) cycle
         if (av(idx,idx) .lt. av(idx-1,idx-1)) lcheck = .true.
      enddo
      if (lcheck) then
         routine = 'Diabat4 (diabatmod.f90)'
         write(message,'(a,i5)') 'Adiabatic energies not ordered:', nrec
         call warnmsg
      endif
         
!-----------------------------------------------------------------------
! get estimated diabatic energies at new point
! v0,deriv1,deriv2  are guess diabatic states at new point
!-----------------------------------------------------------------------
      call dddb_rd(v0,deriv1,deriv2,tmpdip,xgpoint,.false.,.false.,lforce)

!-----------------------------------------------------------------------
! Get estimated adiabatic energies and adt trafo
! remove S/T coupling from Hamiltonian so the adiabatic states are pure 
! spin states
!
! av0, trans0 are guess adiabatic energies + trafo from guess diabatic v0
!-----------------------------------------------------------------------
      trans0 = 0.0_dop
      av0 = 0.0_dop
      jdx = 0
      do idx=1,nblock
         allocate(transtmp(nstblck(idx),nstblck(idx)))

! jdx is offset for spin blocks in Hamiltonian matrix
         if (idx .gt. 1) jdx = jdx+nstblck(idx-1)

         do kdx = 1,nstblck(idx)
            do ldx = 1,nstblck(idx)
               transtmp(ldx,kdx) = v0(jdx+ldx,jdx+kdx) 
            enddo
         enddo

         call dsyev('V','U',nstblck(idx),transtmp,nstblck(idx),tmpe,work,&
              3*nstblck(idx),info)

! Save the adiabatic energy.
         do ldx = 1,nstblck(idx)
            av0(jdx+ldx,jdx+ldx) = tmpe(ldx)
         enddo

! set phases on trafo matrix vectors
         call trmatphase(transtmp,nstblck(idx))

! store transformation matrix S^T in trans0
! NB S^T V S = W
         do kdx = 1,nstblck(idx)
            do ldx = 1,nstblck(idx)
               trans0(jdx+ldx,jdx+kdx) = transtmp(ldx,kdx) 
            enddo
         enddo

         deallocate(transtmp)
      enddo

!-----------------------------------------------------------------------
! add back in any SOC to predicted adiabatic energies
!-----------------------------------------------------------------------
      do si=1,nddstate
         do sf=1,nddstate
            if (imultmap(si) .ne. imultmap(sf)) av0(sf,si) = v0(sf,si)
         enddo
      enddo

!-----------------------------------------------------------------------
! aderiv10 are projected adiabatic forces + NACTs
!-----------------------------------------------------------------------
      do ldx = 1, ndofddpes
         tmpderiv1(ldx,1:nddstate,1:nddstate) = &
              matmul(deriv1(ldx,1:nddstate,1:nddstate),trans0)
         call qqtxdd(trans0,tmpderiv1(ldx,1:nddstate,1:nddstate),&
              aderiv10(ldx,1:nddstate,1:nddstate),nddstate)
      enddo

!-----------------------------------------------------------------------
! if QC failed set flag and store projected data
!-----------------------------------------------------------------------
      if (linterp0) then
         interp = -1
         adttrans=trans0
         call tranqxd(adttrans,nddstate)   ! adttrans = S 
         av = av0
         aderiv1 = 0.0_dop
         jdx = 0
         do ldx = 1,nddstate
            aderiv1(1:ndofddpes,ldx,ldx) = aderiv10(1:ndofddpes,ldx,ldx)
            do kdx = ldx+1,nddstate
               jdx = (kdx-1)*(kdx-2)/2+ldx
               dercp(1:ndofddpes,jdx) = aderiv10(1:ndofddpes,ldx,kdx)
            enddo
         enddo

         write(ilog,'(a,i0)') &
            'Extrapolated adiab and diab written to DB record:  ',nrec

         call transform(adttrans,av(1:nddstate,1:nddstate),&
              aderiv1(1:ndofddpes,1:nddstate,1:nddstate),&
              aderiv2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate),&
              v,deriv1,deriv2,dercp(1:ndofddpes,1:nactdim),&
              ndofddpes,nddstate)

         go to 999  ! Go to End of failed QC
      endif
     
!-----------------------------------------------------------------------
! Output quality of model at new point
!-----------------------------------------------------------------------
      verr = 0.0_dop
      lmodfail = .false.
      do idx = 1,nddstate
         diff0 = av0(idx,idx) - av(idx,idx)
         verr = verr + diff0*diff0
         if (abs(diff0) .gt. dbdegen) then
            write(ilog,'(a,i0,a,f12.8)') &
                  'State ',idx,' (Ecalc - Emodel) / eV:  ',diff0*27.211
            lmodfail = .true.
         endif
      enddo
      verr = dsqrt(verr)/dble(nddstate)
      write(ilog,'(a,f12.8)') 'RMS(Ecalc - Emodel) / eV:  ',verr*27.211
      lmodfail = lmodfail .or. (verr .gt. dbdegen)
      if (lmodfail) then
         write(ilog,'(a)') 'Model does not reproduce potentials well. '
! Now calculate more points until OK
      endif
         
!-----------------------------------------------------------------------
! Check the overlap of the projected and actual vector couplings.
! Change sign of nact vector to match the projected.
!-----------------------------------------------------------------------
      dotp = 0.0_dop
      linterp = .false.
      mindotp1 = 0.1_dop   

      dcpover(:,:)=-999
      do idx = 1,nddstate-1
         do jdx = idx+1,nddstate

            if (imultmap(jdx) .ne. imultmap(idx)) cycle

            kdx = (jdx-1)*(jdx-2)/2+idx
! magnitude of dcp
            dotp2(kdx) = 0.0_dop
            do ldx = 1, ndofddpes
               dotp2(kdx) = dotp2(kdx) + &
                    dercp(ldx,kdx)*dercp(ldx,kdx)
            enddo
            dotp2(kdx)=dsqrt(dotp2(kdx))

! magnitude of aderiv10
            dotp3 = 0.0_dop
            do ldx = 1, ndofddpes
               dotp3 = dotp3 + &
                    aderiv10(ldx,idx,jdx)*aderiv10(ldx,idx,jdx)
            enddo
            dotp3=dsqrt(dotp3)

! projection of interpolated nact (aderiv10) on calculated (dercp)
            dotp1 = 0.0
            do ldx = 1, ndofddpes
               dotp1 = dotp1 + &
                    dercp(ldx,kdx)*aderiv10(ldx,idx,jdx)
            enddo
            dotp1 = dotp1/(dotp2(kdx)*dotp3)

            if (abs(dotp2(kdx)) .lt. 0.01 .and. abs(dotp3) .lt. 0.01) then
               dcpover(idx,jdx)=1    ! zero vector 
            else if (dotp1 .gt. 0.80) then
               dcpover(idx,jdx)=1    ! overlaps predicted
            else if (dotp1 .lt. -0.80) then
               do ldx = 1, ndofddpes
                  dercp(ldx,kdx) = -dercp(ldx,kdx)
               enddo
               dcpover(idx,jdx)=1    ! overlaps predicted - correct sign
            else if (abs(dotp1) .lt. 0.001) then
               dcpover(idx,jdx)=0    ! othogonal to predicted
            else 
               dcpover(idx,jdx)=2    ! poor overlap to predicted
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! Check for intruder states
!-----------------------------------------------------------------------
      lintruder(:) = .true.
      lintruder(1) = .false.
      do idx = 1,nddstate-1
         do jdx = idx+1,nddstate
            if (dcpover(idx,jdx) .gt. 0) lintruder(jdx) = .false.
         enddo
      enddo
      if (ddlog .and. any(lintruder)) then
         verr = 0.0_dop
         do idx = 1,nddstate
            if (lintruder(idx)) cycle
            diff0 = av0(idx,idx) - av(idx,idx)
            verr = verr + diff0*diff0
         enddo
         do idx = 1,nddstate
            if (.not. lintruder(idx)) cycle
            write(ilog,'(a,i0)') 'Intruder State:  ',idx
         enddo
         verr = dsqrt(verr)/dble(nddstate)
         write(ilog,'(a,f12.8)') 'RMS(Ecalc - Emodel) / eV:  ',verr*27.211
      endif
          
!-----------------------------------------------------------------------
! Copy raw adiabatic data for processing
!-----------------------------------------------------------------------
      av1= av
      aderiv11 = aderiv1
      dercp1 = dercp

!-----------------------------------------------------------------------
! flag if near degeneracy in adiabatic surfaces where QC cannot be trusted
! if states have different spin do not check 
! don't bother about high-lying states > mxsinterest
!-----------------------------------------------------------------------
      linterp = .false.
      ldegen  = .false.
      if (.not. lnodegen) then
         do ldx=1,nddstate-1
            if (imultmap(ldx) .ne. imultmap(ldx+1)) cycle
            if (ldx .gt. mxsinterest) cycle

            diff0 = av(ldx+1,ldx+1) - av(ldx,ldx)

            if (abs(diff0) .lt. dbdegen) then 
                linterp = .true.
                write(ilog,'(a,2i5)')'States near degenerate : ',ldx,ldx+1
                ldegen(ldx) = .true.
                ldegen(ldx+1) = .true.
            endif
         enddo
      endif

!-----------------------------------------------------------------------
! Replace derivative data (aderiv1 and dercp) for near degenerate states 
! with interpolated adiab data
! Keep calculated energy.
!-----------------------------------------------------------------------
      if (linterp) then
         linterp1 = .true.
         interp = 1

         do idx = 1,nddstate
            if (ldegen(idx)) then
               aderiv11(1:ndofddpes,idx,idx) = aderiv10(1:ndofddpes,idx,idx)
            endif

            jdx = idx+1
            if (jdx .gt. nddstate) cycle
            if (ldegen(idx) .and. ldegen(jdx)) then
               kdx = (jdx-1)*(jdx-2)/2+idx
               dercp1(1:ndofddpes,kdx) = aderiv10(1:ndofddpes,idx,jdx)

! update length of dercp for analysis
               dotp2(kdx) = 0.0_dop   
               do ldx = 1, ndofddpes
                  dotp2(kdx) = dotp2(kdx) + &
                       dercp1(ldx,kdx)*dercp1(ldx,kdx)
               enddo
               dotp2(kdx)=dsqrt(dotp2(kdx))
            endif
         enddo
      endif

!-----------------------------------------------------------------------
! START of loop to find best nearest neighbour
! if DCP between neighboruing points is 0 then intruder state has 
! appeared in manifold
!
! iscore(irec) : a score of best fit for DB points
!
! by default don't do this - let rediabatisation take care of ordering
!-----------------------------------------------------------------------
      ibest = 0
      if (lchkdcp) then
         if (ldbsmall) then
            dbnrec1 = dbnrec_gp(num_gp)
            call distdb_gp(xgpoint,idbloc,dist,mindist,mindisp,dbnrec_gp(num_gp))
         else
            dbnrec1 = dbnrec
            call distdb(xgpoint,idbloc,dist,mindist,mindisp,dbnrec)
         endif
         minscore = 900
         do irec = 1,dbnrec1  ! loop over confidence set
            jrec = loc(irec)  ! actual location of record in full DB
            iscore=0

            if (dist(irec) .gt. rad_conf) cycle  ! outside confidence radius

            do idx=1,nddstate-1
               if (imultmap(idx) .ne. imultmap(idx+1)) cycle

!-----------------------------------------------------------------------
! look at overlap of calculated DCP and irec DCP.
! Only worry about neighbouring states. I.e. DCP field aroud a possible ]
! CoIn.
!-----------------------------------------------------------------------
               jdx = idx+1
               if (imultmap(jdx) .ne. imultmap(idx)) cycle
               kdx = (jdx-1)*(jdx-2)/2+idx

! check if magnitude of dercp is small (i.e. near zero)
               if (dotp2(kdx) .lt. 0.00001) cycle

               dotp3 = 0.0_dop
               do ldx = 1, ndofddpes
                  dotp3 = dotp3 + &
                       dbdercp(ldx,kdx,jrec)*dbdercp(ldx,kdx,jrec)
               enddo
               dotp3=dsqrt(dotp3)

! projection of DB nact on calculated (dercp)
               dotp1 = 0.0
               do ldx = 1, ndofddpes
                  dotp1 = dotp1 + dercp1(ldx,kdx)*dbdercp(ldx,kdx,jrec)
               enddo
               dotp1 = dotp1/(dotp2(kdx)*dotp3)

! penalise poor overlap
               if (abs(dotp1) .lt. mindotp1) then
                  iscore = iscore+1
               endif
            enddo

! Get best match from DB
            if (iscore .lt. minscore) then
               ibest = irec
               minscore = iscore
            else if (iscore .eq. minscore) then
               if (dist(irec) .lt. dist(ibest)) ibest = irec
            endif

         enddo
      endif

!-----------------------------------------------------------------------
! Store best point found to be nearest neighbour
!-----------------------------------------------------------------------
      if (ibest .gt. 0) then
         ibest = loc(ibest)   ! Actual location of ibest
         idbloc = ibest
      else
         idbloc = idbloc0
      endif

!-----------------------------------------------------------------------
! Propagate transformation (adttrans)
!-----------------------------------------------------------------------
      write(ilog,'(2(a,i0))') &
         'Propagating diabatic states to DB record:          ',nrec,&
         '   from: ',idbloc

      allocate(transtmp(nddstate,nddstate))

      nstep = 20

!-----------------------------------------------------------------------
! Integrate Int(F dx) across interval using trapezium rule
! Form the non-adiabatic couplings (F) from the derivative couplings
!-----------------------------------------------------------------------
! get geometry at nearest DB point and Delx across interval
      xgp = dbgeo(1:ndofddpes,idbloc)
      delx = (xgpoint-xgp)/dble(nstep)

! first add the nearest DB point 
      kdx = 0
      do idx = 1,nddstate-1
         do jdx = idx+1,nddstate
            kdx = (jdx-1)*(jdx-2)/2+idx
            nact(1:ndofddpes,kdx) = &
                 dbdercp(1:ndofddpes,kdx,idbloc)/&
                      (dbadener(jdx,jdx,idbloc)-dbadener(idx,idx,idbloc))
         enddo
      enddo
      sumint = nact(1:ndofddpes,1:nactdim) 

! next add the end point 
      kdx = 0
      do idx = 1,nddstate-1
         do jdx = idx+1,nddstate
            kdx = (jdx-1)*(jdx-2)/2+idx
            nact(1:ndofddpes,kdx) = dercp1(1:ndofddpes,kdx)/ &
                 (av1(jdx,jdx)-av1(idx,idx))
         enddo
      enddo
      sumint = sumint + nact(1:ndofddpes,1:nactdim) 

! If no useable data points in DB, use linear interpolation
      jrec = 0
      do irec = 1,dbnrec
         if (dbignore(irec) .eq. 1) cycle
         if (dbinterp(irec) .lt. 0) cycle
         if (irec .eq. idbloc) cycle
         jrec = jrec+1
      enddo

      diabint0 = diabint
      if (jrec .eq. 0) then
         diabint = 1
      endif

      if (diabint .eq. 0) then
!----------------------------------------------------------------------- 
! Original integration scheme (Pseudo Shepard on nact, Shepard on 
! energies taken from model)
!----------------------------------------------------------------------- 
! Allocate memory
         allocate(vstep(nddstate,nddstate))

! get energies and interpolation norms across interval
         do jdx = 1, nstep-1
            xgp = xgp + delx

            call dddb_rd(vstep,tmpderiv1,deriv2,tmpdip,xgp,&
                 .false.,.false.,.false.)

            call nactdist(shepdist,xgpoint,dbgeo(1,idbloc),xgp,ndofddpes)
            do kdx=1,2
               weight(kdx) = shepdist(kdx)**(dbinterord)
               weight(kdx) = max(weight(kdx),1.0d-6)
               weight(kdx) = 1.0_dop / weight(kdx)
            enddo

! Get the energy at xgp. First by interpolating the DB and diagonalising. Then
! add the adiabatic energies. Bit dodgy but minimises effort.

            nact = 0.0_dop
            tmpe = 0.0_dop

! Diagonalise the interpolated diabatic energy.
            mdx=0
            do idx=1,nblock
               allocate(transtmp1(nstblck(idx),nstblck(idx)))

! mdx is offset for spin blocks in Hamiltonian matrix
               if (idx .gt. 1) mdx = mdx+nstblck(idx-1)
               do kdx = 1,nstblck(idx)
                  do ldx = 1,nstblck(idx)
                     transtmp1(ldx,kdx) = vstep(mdx+ldx,mdx+kdx)
                  enddo
               enddo

               call dsyev('V','U',nstblck(idx),transtmp1,nstblck(idx),&
                    tmpe2,work,3*nstblck(idx),info)

               do ldx = 1,nstblck(idx)
                  tmpe(mdx+ldx) = tmpe2(ldx)
               enddo

               deallocate(transtmp1)
            enddo

! Add on the end-point energy to the diagonalised interpolant.
! Take into account Shepard weights used in interpolated tmpe
            do idx = 1,nddstate
               tmpe(idx) = tmpe(idx)*shep_norm + &
                    av1(idx,idx)*weight(2)
            enddo
            norm = shep_norm + weight(2)
            tmpe = tmpe/norm

! Form the NACT. 
            do idx = 1,ndofddpes
               do kdx = 1,nactdim
                  nact(idx,kdx) = dbdercp(idx,kdx,idbloc)* shepdist(1)&
                       + dercp1(idx,kdx) * shepdist(2)
               enddo
            enddo
            nact = nact/(shepdist(1)+shepdist(2))

            kdx = 0
            do idx = 1,nddstate-1
               do ldx = idx+1,nddstate
                  kdx = (ldx-1)*(ldx-2)/2+idx
                  nact(1:ndofddpes,kdx) = &
                       nact(1:ndofddpes,kdx)/(tmpe(ldx)-tmpe(idx))
               enddo
            enddo
            sumint = sumint + 2.0_dop * nact
         enddo

         deallocate(vstep)

      else if (diabint .eq. 1) then
!-----------------------------------------------------------------------
! Linear interpolation of calculated nact and De.
!-----------------------------------------------------------------------
! get geometry at nearest DB point and Delx across interval
         do kdx = 1,nactdim
            do idx = 1,ndofddpes
               deldercp(idx,kdx) = &
                   (dercp1(idx,kdx) - dbdercp(idx,kdx,idbloc))/dble(nstep)
            enddo
         enddo
         do idx=1,nddstate
            dele(idx) = (av1(idx,idx) - dbadener(idx,idx,idbloc))/dble(nstep)
         enddo

! Now interpolate across interval 
         do istep = 1, nstep-1
            xgp = xgp + delx

            kdx = 0
            do idx = 1,nddstate-1
               do jdx = idx+1,nddstate
                  kdx = (jdx-1)*(jdx-2)/2+idx

                  diff0 = (dbadener(jdx,jdx,idbloc) - dbadener(idx,idx,idbloc)) + &
                          (dele(jdx) - dele(idx))*dble(istep)
                  nact(1:ndofddpes,kdx) = dbdercp(1:ndofddpes,kdx,idbloc) + &
                             deldercp(1:ndofddpes,kdx)*dble(istep)
                  nact(1:ndofddpes,kdx) = nact(1:ndofddpes,kdx) / diff0
               enddo
            enddo

            sumint = sumint + 2.0_dop * nact
         enddo

      else if (diabint .eq. 2) then
!-----------------------------------------------------------------------
! Shepard interpolation of nact and De. Both are predicted by model.
!-----------------------------------------------------------------------
! Allocate memory
         allocate(vstep(nddstate,nddstate))

         dbignore(idbloc) = 1   ! ignore start point in interpolation
         do istep = 1, nstep-1
            xgp = xgp + delx

            call dddb_rd(vstep(1,1),tmpderiv1,deriv2,tmpdip,xgp,&
                 .false.,.false.,.false.)

            nact = 0.0_dop
            tmpe = 0.0_dop

! Get the Shepard distances 
            call nactdist(shepdist,xgpoint,dbgeo(1,idbloc),xgp,ndofddpes)
            do kdx=1,2
               weight(kdx) = shepdist(kdx)**(dbinterord)
               weight(kdx) = max(weight(kdx),1.0d-6)
               weight(kdx) = 1.0_dop / weight(kdx)
            enddo

! Diagonalise the interpolated diabatic energy.
            jdx=0
            do idx=1,nblock
               allocate(transtmp1(nstblck(idx),nstblck(idx)))
               transtmp = 0.0_dop

! mdx is offset for spin blocks in Hamiltonian matrix
               if (idx .gt. 1) jdx = jdx+nstblck(idx-1)
               do kdx = 1,nstblck(idx)
                  do ldx = 1,nstblck(idx)
                     transtmp1(ldx,kdx) = vstep(jdx+ldx,jdx+kdx)
                  enddo
               enddo

               call dsyev('V','U',nstblck(idx),transtmp1,nstblck(idx),&
                    tmpe2,work,3*nstblck(idx),info)

               do ldx = 1,nstblck(idx)
                  tmpe(jdx+ldx) = tmpe2(ldx)
               enddo

! store transformation matrix S^T in trans0
               do kdx = 1,nstblck(idx)
                  do ldx = 1,nstblck(idx)
                     transtmp(jdx+ldx,jdx+kdx) = transtmp1(ldx,kdx)
                  enddo
               enddo

               deallocate(transtmp1)
            enddo

! Add on the end-point energy to the diagonalised interpolant.
! Take into account Shepard weights used in interpolated tmpe
            do idx = 1,nddstate
               tmpe(idx) = tmpe(idx)*shep_norm + &
                    av1(idx,idx)*weight(2) + &
                    dbadener(idx,idx,idbloc)*weight(1)   
            enddo
            norm = shep_norm + weight(1) + weight(2)
            tmpe = tmpe/norm

! set phases on trafo matrix vectors
            call trmatphase(transtmp,nddstate)

!-----------------------------------------------------------------------
! tmpderiv1 are transformed to adiabatic forces + NACTs
!-----------------------------------------------------------------------
            do ldx = 1, ndofddpes
               tmpderiv1a(ldx,1:nddstate,1:nddstate) = &
                    matmul(tmpderiv1(ldx,1:nddstate,1:nddstate),transtmp)
               call qqtxdd(transtmp,tmpderiv1a(ldx,1:nddstate,1:nddstate),&
                    tmpderiv1(ldx,1:nddstate,1:nddstate),nddstate)
            enddo
! extract DCP
            kdx = 0
            do idx = 1,nddstate-1
               do jdx = idx+1,nddstate
                  kdx = (jdx-1)*(jdx-2)/2+idx

                  nact(1:ndofddpes,kdx) = tmpderiv1(1:ndofddpes,idx,jdx) 
               enddo
            enddo


! Add on the weighted  end-point NACT to the Shepard interpolated NACT.
! Take into account Shepard weights used in interpolated NACT.
            do kdx = 1,nactdim
               do idx = 1,ndofddpes
                  nact(idx,kdx) = nact(idx,kdx) + &
                        dercp1(idx,kdx)*weight(2) + &
                        dbdercp(idx,kdx,idbloc)*weight(1)
               enddo
            enddo
            norm = shep_norm + weight(1) + weight(2) 
            nact = nact/norm

            kdx = 0
            do idx = 1,nddstate-1
               do jdx = idx+1,nddstate
                  kdx = (jdx-1)*(jdx-2)/2+idx

                  nact(1:ndofddpes,kdx) = nact(1:ndofddpes,kdx) / &
                         (tmpe(jdx)-tmpe(idx))
               enddo
            enddo

            sumint = sumint + 2.0_dop * nact
         enddo
         dbignore(idbloc) = dbignore0(idbloc)  ! restore dbignore

         deallocate(vstep)

      endif  ! end of forming integrand

      diabint = diabint0  ! restore flag for interpolation

! Evaluate the integral 
      integral = 0.0_dop
      do jdx = 1, nactdim
         do kdx = 1, ndofddpes
            integral(jdx) = integral(jdx) + sumint(kdx,jdx)*delx(kdx)
         enddo
      enddo
      integral = integral * 0.5_dop

! Initialise transformation matrix.
      transtmp(1:nddstate,1:nddstate) = dbtrans(1:nddstate,1:nddstate,idbloc)

! Initialise the Crank-Nicholson arrays. Phys. Rev. A, 2003, 68, 042706
      call unitqxd(iminusp,nddstate)
      call unitqxd(iplusp,nddstate)

! Form the Crank-Nicholson arrays.
      kdx = 0
      inttmp = 0.0_dop
      inttmp2 = 0.0_dop
      do idx = 1,nddstate-1
         do jdx = idx+1,nddstate
            kdx = (jdx-1)*(jdx-2)/2+idx
            iminusp(idx,jdx) = -0.5_dop*integral(kdx)
            iminusp(jdx,idx) = -iminusp(idx,jdx)

            iplusp(idx,jdx) =  0.5_dop*integral(kdx)
            iplusp(jdx,idx) = -iplusp(idx,jdx)

            inttmp(idx,jdx) =  0.5_dop*integral(kdx)
            inttmp(jdx,idx) = -inttmp(idx,jdx)
         enddo
      enddo

! Form Exponential of integral
      call qqxxdd(inttmp,inttmp,inttmp3,nddstate)
      inttmp2 = inttmp3
      iminusp = 0.5_dop*inttmp2+iminusp
      iplusp = 0.5_dop*inttmp2+iplusp
      
      call qqxxdd(inttmp2,inttmp,inttmp3,nddstate)
      inttmp2 = inttmp3
      iminusp = (-1.0_dop/6.0_dop)*inttmp2+iminusp
      iplusp = (1.0_dop/6.0_dop)*inttmp2+iplusp
            
      call qqxxdd(inttmp2,inttmp,inttmp3,nddstate)
      inttmp2 = inttmp3
      iminusp = (1.0_dop/24.0_dop)*inttmp2+iminusp
      iplusp = (1.0_dop/24.0_dop)*inttmp2+iplusp

      call qqxxdd(inttmp2,inttmp,inttmp3,nddstate)
      inttmp2 = inttmp3
      iminusp = (-1.0_dop/120.0_dop)*inttmp2+iminusp
      iplusp = (1.0_dop/120.0_dop)*inttmp2+iplusp

!         call qqxxdd(inttmp2,inttmp,inttmp3,nddstate)
!         inttmp2 = inttmp3
!         iminusp = (1.0_dop/720.0_dop)*inttmp+iminusp
!         iplusp = (1.0_dop/720.0_dop)*inttmp+iplusp

! compute the inverse of iplusp using LAPACK subroutines
      ipluspinv=iplusp
      call dgetrf(nddstate,nddstate,ipluspinv,nddstate,pivots,ierr)
      call dgetri(nddstate,ipluspinv,nddstate,pivots,work,3*nddstate,ierr)
      if(ierr.ne.0)then
         routine = 'diabat4'
         message = 'Singularity in forward transformation prop.'
         call errormsg
      endif

! Form the transformation matrices.
      call qqxxdd(iminusp,transtmp,inttmp3,nddstate)
      call qqxxdd(ipluspinv,inttmp3,transtmp,nddstate)

! store in adttrans = S
      adttrans = transtmp
      call trmatphase(adttrans,nddstate)

!-----------------------------------------------------------------------
! Get new diabatic H and look for state flips due to CoIn
!-----------------------------------------------------------------------
      allocate(transtmp1(nddstate,nddstate))

!-----------------------------------------------------------------------
! See if diabatic states need to change order (flip), which indicates a 
! conical intersection has been crossed
! do not check between blocks of spin states
! diabidx1 : order at idbloc0
! diabidx2 : order at Q, after propgation of QC
! diabidx3 : order at Q, from VC model
!----------------------------------------------------------------------
      jdx=0
      do idx=1,nblock
!!jdx is offset for spin blocks in Hamiltonian matrix
         if (idx .gt. 1) jdx = jdx+nstblck(idx-1)

         do kdx=1,nstblck(idx)
            ldx = jdx+kdx
            tmpe(ldx)=dbener(ldx,ldx,idbloc)
         enddo
         call ibubble_up(tmpe(jdx+1:jdx+nstblck(idx)), &
                 diabidx1(jdx+1:jdx+nstblck(idx)),nstblck(idx))

         transtmp1 = matmul(av1,adttrans)
         call qqtxdd(adttrans,transtmp1,v,nddstate)
         do kdx=1,nstblck(idx)
            ldx = jdx+kdx
            tmpe(ldx)=v(ldx,ldx)
         enddo
         call ibubble_up(tmpe(jdx+1:jdx+nstblck(idx)), &
                 diabidx2(jdx+1:jdx+nstblck(idx)),nstblck(idx))

         do kdx=1,nstblck(idx)
            ldx = jdx+kdx
            tmpe(ldx)=v0(ldx,ldx)
         enddo
         call ibubble_up(tmpe(jdx+1:jdx+nstblck(idx)), &
                 diabidx3(jdx+1:jdx+nstblck(idx)),nstblck(idx))

      enddo
         
!-----------------------------------------------------------------------
! See if diabatic states have swapped order ("flipped")
!-----------------------------------------------------------------------
      lflip = .false.
      lflip1 = .false.
      do idx=1,nddstate-1
         if (diabidx1(idx) .ne. diabidx2(idx)) lflip1 = .true.
         if (diabidx1(idx) .ne. diabidx3(idx)) lflip = .true.
      enddo

!-----------------------------------------------------------------------
! If diabatic states were predicted to flip by VC model but this has
! not been dealt with by propagation, now re-order (rotate) to get
! adttrans
! diabidx1: order at previous DB point
! diabidx2: order after propagating DCP 
! diabidx3: order in guess H.
! NB vector output from bubble is idx(i) is the position of state i
! in matrix, e.g. idx(8) = 4 means state 8 is at H_44
!-----------------------------------------------------------------------
      if (lflip) then
         diabidx4 = diabidx2
         do idx=1,nddstate-1
            if (diabidx3(idx) .eq. diabidx1(idx)) cycle ! not predicted to flip

            if (diabidx4(idx) .ne. diabidx3(idx)) then
!!swap states jdx and kdx.
               jdx = diabidx4(idx) ! present position of state idx
               kdx = diabidx3(idx) ! new position of state idx

               do ldx = 1,nddstate ! find state presently at kdx
                  if (diabidx4(ldx) .eq. kdx) diabidx4(ldx) = jdx
               enddo
               diabidx4(idx) = kdx

               transtmp(:,:) = adttrans(:,:)

!!set up (transpose) of rotation matrix and rotate states in adttrans
               call unitqxd(transtmp1,nddstate)
               transtmp1(jdx,jdx)=0.0_dop
               transtmp1(kdx,kdx)=0.0_dop
               transtmp1(jdx,kdx)=-1.0_dop
               transtmp1(kdx,jdx)=1.0_dop

               call qqxxdd(transtmp,transtmp1,adttrans,nddstate)
                
            endif
         enddo
         call trmatphase(adttrans,nddstate)
      endif

!-----------------------------------------------------------------------
! Perform the transformation:
!-----------------------------------------------------------------------
      call transform(adttrans, &
           av1(1:nddstate,1:nddstate), &
           aderiv11(1:ndofddpes,1:nddstate,1:nddstate),&
           aderiv2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate),&
           v,deriv1,deriv2,dercp1(1:ndofddpes,1:nactdim),&
           ndofddpes,nddstate)

!-----------------------------------------------------------------------
! Log re-ordering of states
!-----------------------------------------------------------------------
      if (ddlog .and. (lflip .or. lflip1)) then
         write(ilog,'(a)') 'States Flip'

         write(ilog,'(a,20i3)') 'DB : ',diabidx1(:)
         if (ddlogfull) then
            write(ilog,'(a,20i3)') 'QC : ',diabidx2(:)
            write(ilog,'(a,20i3)') 'VC : ',diabidx3(:)
         endif

!!jdx is offset for spin blocks in Hamiltonian matrix
         jdx=0
         do idx = 1,nblock
            if (idx .gt. 1) jdx = jdx+nstblck(idx-1)
            do kdx=1,nstblck(idx)
               ldx = jdx+kdx
               tmpe(ldx)=v(ldx,ldx)
            enddo
            call ibubble_up(tmpe(jdx+1:jdx+nstblck(idx)), &
                 diabidx4(jdx+1:jdx+nstblck(idx)),nstblck(idx))
         enddo
         write(ilog,'(a,20i3)') 'End: ',diabidx4(:)
      endif

      if(allocated(transtmp)) deallocate(transtmp)
      if(allocated(transtmp1)) deallocate(transtmp1)

  999 continue
!-----------------------------------------------------------------------
! Clean up and finish
!-----------------------------------------------------------------------
! reset flags for DB use to normal 
      dbignore(1:dbnrec) = dbignore0(1:dbnrec)

! Deallocate
      if(allocated(r1))deallocate(r1)

      call myflush(ilog)

      return
      end subroutine diabat4

!##########################################################################
! Subroutine to diabatise the PES when performing multi-state calculations,
! using the projection operator methods of Robertson et al.
! GWR 04/18
!##########################################################################

      subroutine diabat5(pidx,dipole)

      implicit none

      integer(long), intent(in) :: pidx
      real(dop), dimension(3,nddstate,nddstate), intent(in) :: dipole
!      real(sip), dimension(nbasis,nbasis), intent(in) :: mocoeff

      logical(kind=4) :: fexist
      character(len=c5) :: diabinp,refpun,objpun,ddname1,extcmd,diabout
      integer(long) :: ddlaenge1,reflen,dialen,outlen,idiab,s,s1
      real(dop), dimension(istatemap(nddstate))  :: vtmp
      real(dop), dimension(nddstate,nddstate) :: v,rdum,wdum,wdum1
      complex(dop), dimension(nddstate,nddstate) :: cdum

      routine = 'proj_diab'

! Check that the input file which contains information about the 
! diabatisation exists.
      diabinp = ddname(1:ddlaenge)//'/diabci.inp'
      call rmblnk(diabinp)
      dialen = len(trim(diabinp))
      inquire(file=diabinp,exist=fexist)
      if(.not.fexist)then
         message = 'Diabatisation input file does not exist'
         call errormsg
      endif

! Check that the file which contains the CI vectors at the reference 
! geometry exists.
      refpun = ddname(1:ddlaenge)//'/reference/reference.pun'
      call rmblnk(refpun)
      reflen = len(trim(refpun))
      inquire(file=refpun,exist=fexist)
      if(.not.fexist)then
         message = 'Reference punch file does not exist'
         call errormsg
      endif
      
! Check that the file which contains the CI vectors at the new geometry exists.
      write(ddname1,'(a,2(a,i5))')ddname(1:ddlaenge),'/gwp',pidx,'_V',nddstate
      call rmblnk(ddname1)
      ddlaenge1 = len(trim(ddname1))

      objpun = ddname1(1:ddlaenge1)//'/object.pun'
      inquire(file=objpun,exist=fexist)
      if(.not.fexist)then
         message = 'Object punch file does not exist'
         call errormsg
      endif

! Copy the input and reference CI vector files to the directory containing 
! the results of the calculations at the new geometry.
      extcmd = 'cp '//refpun(1:reflen)//' '//ddname1(1:ddlaenge1)
      call excmd(extcmd)
      extcmd = 'cp '//diabinp(1:dialen)//' '//ddname1(1:ddlaenge1)
      call excmd(extcmd)

! Call the python script which does the actual diabatisation.
      extcmd = 'rm -f '//ddname1(1:ddlaenge1)//'/*endiab'
      call excmd(extcmd)
      extcmd = 'proj_diab diabci.inp '//ddname1(1:ddlaenge1)
      call excmd(extcmd)

! Check that the output file, containing the diabatic energies, exists.
      diabout = ddname1(1:ddlaenge1)//'/object.endiab'
      outlen = len(trim(diabout))
      inquire(file=diabout,exist=fexist)
      if(.not.fexist)then
         message = 'Diabatic energies file does not exist'
         call errormsg
      endif

! Open the output file and read the energies in.      
      idiab = 999
      open(unit=idiab,file=diabout,status='old',err=1000)
      rewind(idiab)
      do s = 1,nddstate
         read(idiab,*) (vtmp(s1),s1=1,nddstate)
         do s1 = 1,nddstate
            v(s,s1) = vtmp(s1)
         enddo
      enddo
      close(idiab,status='keep')

! Shift energy origin.
      do s = 1,nddstate
         v(s,s) = v(s,s)-dde0
      enddo

! Write the diabatic quantities to the DB.
!!      call dddb_wrdiab_gap_mem(v,rdum,dipole,wdum,wdum1,cdum,dbnrec)
!!      call dddb_wr_gap_file(2)

! Delete the directory of the new geometry.
      extcmd = 'rm -rf '//ddname1(1:ddlaenge1)
      call excmd(extcmd)

      return

1000  write(*,*)'Error opening diabatic output file'
      stop

      end subroutine diabat5

!##########################################################################
! Subroutine to diabatise the PES, when performing multi-state 
! calculations, using the solution to the orthogonal Procrustes problem
! using the SVD wavefunction overlaps.
! GWR 08/19
!##########################################################################

      subroutine diabat6(xgpoint0,av,aderiv1,aderiv2,dercp,dv,deriv1,&
                 deriv2,trans,ddmoco,ciconfig,cicoeff,idbloc0,nrec,lqcfail)

      implicit none

      integer(long), intent(in)                      :: idbloc0,nrec
      real(dop), dimension(ndofddpes),intent(in)     :: xgpoint0

      real(dop), dimension(ndofddpes,nactdim), intent(inout) :: dercp
      real(dop), dimension(nddstate,nddstate), intent(inout) :: av
      real(dop), dimension(ndofddpes,nddstate,nddstate), &
                                               intent(inout) :: aderiv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), &
                                                  intent(in) :: aderiv2

      real(dop), dimension(nddstate,nddstate), intent(out)   :: trans
      real(dop), dimension(nddstate,nddstate), intent(out)   :: dv
      real(dop), dimension(ndofddpes,nddstate,nddstate), &
                                                 intent(out) :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate) &
                                                             :: deriv2
      real(sip), dimension(nbasis,nbasis), intent(inout)     :: ddmoco
      character(30), dimension(maxcivec,nsmult), intent(out)        :: ciconfig
      real(dop), dimension(maxcivec,maxmultblk,nsmult), intent(out) :: cicoeff
      character(30), dimension(maxcivec,nsmult)                     :: tmpciconfig
      real(dop), dimension(maxcivec,maxmultblk,nsmult)              :: tmpcicoeff

      logical(kind=4) :: laofail
      logical(kind=4), dimension(:,:), allocatable :: lsign
      integer(long) :: irec,jrec,mtmp,s,s1,s2,sdx,f,idb,iprox,jlbl,msta,ilbl,ncas1,&
           soff,lwork,i,info,ms,send,sact1,sact2,j,k,soff1,irrep,iroffcls,iroffocc,&
           iroffcas,iroffbas,multend,ierr,ierrno,aolaenge
      integer(long), dimension(dbnrec)    :: map,proxidx
      real(dop) :: dtmp,mindist,currdist,distsqdiff
      real(dop), dimension(dbnrec)    :: distsq
      real(dop), dimension(nddstate,nddstate)   :: tmpmat
      real(dop), dimension(nddstate,nddstate)   :: rdum,rdum1
      real(dop), dimension(3,nddstate,nddstate) :: dipdum
      real(dop), dimension(ndofddpes) :: xgpoint,xgpointref
      real(dop), dimension(:), allocatable :: stover,sigma,work,sigmamo
      real(dop), dimension(:,:), allocatable :: sao,smat,dsmat,umat,vmatt,&
           tmptrans,smo,umatmo,vmattmo,moco,assmo,cssmo,motransas,motranscs,&
           tmpdsmat,mocoidb,mocoiprox
      complex(dop), dimension(nddstate,nddstate) :: soc,adsoc,ctmpmat
      character(len=240) :: templdatfileao,templdatfileci,filnamao,inpfileao,&
           outfileao,inpfileci,outfileci,filnamci,orbfileci,dirnameao
      character(len=240) :: extcmd

      logical(kind=4), intent(in) :: lqcfail
      logical(kind=4), save :: lfirst=.true., lexist = .false.

!-----------------------------------------------------------------------
! If this is the first call, we set the transformation matrix to the unit
! matrix and transform the adiabatic quantities to the diabatic.
!-----------------------------------------------------------------------
      if(dbnrec.eq.0)then
! if first QC calc has failed, stop.
         if(lqcfail)then
            routine = 'Diabat6 (diabatmod.f90)'
            message = 'First QC calculation failed'
            call errormsg
         endif

         call unitqxd(trans,nddstate)
         call transform(trans,av(1:nddstate,1:nddstate),&
              aderiv1(1:ndofddpes,1:nddstate,1:nddstate),&
              aderiv2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate),&
              dv,deriv1,deriv2,dercp(1:ndofddpes,1:nactdim),&
              ndofddpes,nddstate)

         go to 999     ! Go to End of first call
      endif

! Allocate arrays to hold state overlaps, and arrays holding AS occupancies etc.
      msta=maxval(imultblk(1:nsmult))

      if(allocated(stover))deallocate(stover)
      allocate(stover(nsmult*msta*msta))

      if(allocated(smat))deallocate(smat)
      allocate(smat(msta,msta))

      if(allocated(dsmat))deallocate(dsmat)
      allocate(dsmat(msta,msta))

      if(allocated(tmpdsmat))deallocate(tmpdsmat)
      allocate(tmpdsmat(msta,msta))

      if(allocated(umat))deallocate(umat)
      allocate(umat(msta,msta))

      if(allocated(vmatt))deallocate(vmatt)
      allocate(vmatt(msta,msta))

      if(allocated(tmptrans))deallocate(tmptrans)
      allocate(tmptrans(msta,msta))

      if(allocated(sigma))deallocate(sigma)
      allocate(sigma(msta))

      lwork = 5*max(msta,nocc)
      if(allocated(work))deallocate(work)
      allocate(work(lwork))

      if(allocated(sao))deallocate(sao)
      allocate(sao(nbasis,nbasis))

      if(allocated(smo))deallocate(smo)
      allocate(smo(nocc,nocc))

      if(allocated(sigmamo))deallocate(sigmamo)
      allocate(sigmamo(nocc))

      if(allocated(umatmo))deallocate(umatmo)
      allocate(umatmo(nocc,nocc))

      if(allocated(vmattmo))deallocate(vmattmo)
      allocate(vmattmo(nocc,nocc))

      if(allocated(motransas))deallocate(motransas)
      allocate(motransas(ncas,ncas))

      if(allocated(motranscs))deallocate(motranscs)
      allocate(motranscs(nclosed,nclosed))

      if(allocated(moco))deallocate(moco)
      allocate(moco(nocc,nbasis))

      if(allocated(mocoidb))deallocate(mocoidb)
      allocate(mocoidb(nbasis,nbasis))

      if(allocated(mocoiprox))deallocate(mocoiprox)
      allocate(mocoiprox(nbasis,nbasis))

      if(allocated(assmo))deallocate(assmo)
      allocate(assmo(ncas,ncas))

      if(allocated(cssmo))deallocate(cssmo)
      allocate(cssmo(nclosed,nclosed))

      if(allocated(lsign))deallocate(lsign)
      allocate(lsign(msta,nsmult))

! the nearest DB entry (iprox).
      iprox = idbloc0
      idb   = nrec

      tmpciconfig = ciconfig
      tmpcicoeff = cicoeff

! check for directory to contain AO overlaps
      open(idump,file=ddname(1:ddlaenge)//'/ao_overlaps/scratch',err=666) 
      close(idump,status='delete')
      lexist=.true.

  666 continue
                                                          
      if (.not. lexist) then
         extcmd='mkdir '//ddname(1:ddlaenge)//'/ao_overlaps'
         call excmd3(ierr,ierrno,extcmd)
         if(ierr.ne.0)then
            write(*,*)'Error making directory: ',ddname(1:ddlaenge)//'/ao_overlaps'
            write(*,*)'ierrno = ',ierrno               
         endif
      endif

! Set up a Molpro input file to get the matrix of AO overlaps at the two
! geometries.
      templdatfileao=ddname(1:ddlaenge)//'/template_ao.dat'
      dirnameao=ddname(1:ddlaenge)//'/ao_overlaps'
      aolaenge=ddlaenge+12
      write(filnamao,'(2(a,i5))') dirnameao(1:aolaenge)//'/ao_over_',iprox,'_',idb
      call rmblnk(filnamao)
      ilbl=index(filnamao,' ')-1
      inpfileao=filnamao(1:ilbl)//'.inp'
      outfileao=filnamao(1:ilbl)//'.out'

      lexist=.false.
      inquire(file=trim(outfileao),exist=lexist)
      if (.not. lexist) then
         xgpoint=xgpoint0(1:ndofddpes)
         xgpointref=dbgeo(1:ndofddpes,iprox)
         call wrmolpro_geom(templdatfileao,inpfileao,xgpoint,xgpointref,xatnam,natm,2)
! Run Molpro.
         jlbl=index(ddextcmd,' ')
         if (qcsubform .eq. 1) then
            extcmd=ddextcmd(1:jlbl)//filnamao(aolaenge+2:ilbl+1)//char(zeroi)
         else if (qcsubform .eq. 2) then
            extcmd=ddextcmd(1:jlbl)//filnamao(aolaenge+2:ilbl+1)// &
                 dirnameao(1:aolaenge+1)//char(zeroi)
         endif
         call excmd(extcmd)
      endif   

! Extract the AO overlap matrix from the output file.
      call rdmolpro_ao(outfileao,sao,nbasis,laofail)

      if(laofail)then
         call unitqxd(sao,nbasis)
         write(ilog,'(a,2i6,a)')'Failed to get AO overlap between DB entries',&
              idb,iprox,' Using unit matrix.'
         flush(ilog)
      endif

! First task is to diabatise the orbitals.
! Get the overlaps of the MOs (same for all multiplicities).
      mocoidb(1:nbasis,1:nbasis) = dble(ddmoco(1:nbasis,1:nbasis))
      mocoiprox(1:nbasis,1:nbasis) = dble(dbmoco(1:nbasis,1:nbasis,iprox))
      call molpro_moover(sao,smo,mocoidb,mocoiprox,nbasis,0)

! Transform the CI vectors using the AS and CS MO transformations.
!! Needed to convert CSFs to Slaters dets??
  !!    call citrans(smo,nciconf,tmpciconfig,tmpcicoeff)

! Get the overlaps of all states of the the adiabatic wavefunctions.
      call molpro_wfover(sao,smo,mocoidb,mocoiprox,nbasis,&
              lsign,msta,stover,2,nciconf,tmpciconfig(1,1),&
              tmpcicoeff(1,1,1),dbciconfig(1,1,iprox),&
              dbcicoeff(1,1,1,iprox))

! Loop over the multiplicities.
      soff = 0
      soff1 = 0
      trans = 0.0_dop

      do ms = 1,nsmult ! start of loop over nsmult
! Copy the adiabatic overlaps to a temporary array. Reference geometry rows, 
! new geometry columns.
         sdx = 1
         do s = 1,imultblk(ms)
            do s1 = 1,imultblk(ms)
               smat(s1,s) = stover(sdx)
               sdx = sdx + 1
            enddo
         enddo

! Transform the reference adiabatic states to the diabatic to get the overlap 
! between the iprox diabatic states and the idb adiabatic.
! First copy adttrans (DD state labels) at iprox to tmpdsmat (QC state labels)
         tmpdsmat = 0.0_dop
         soff = ddstoff(ms)
         do s1 = soff+1,soff+ddstblk(ms)
            sact1 = istatemap(s1)-imultoff(ms)
            do s2 = soff+1,soff+ddstblk(ms)
               sact2 = istatemap(s2)-imultoff(ms)
               tmpdsmat(sact2,sact1) = dbtrans(s2,s1,iprox)
            enddo
         enddo
         do s = 1,imultblk(ms)
            do s1 = 1,imultblk(ms)
               dsmat(s1,s) = tmpdsmat(1,s1)*smat(1,s)
               do s2 = 2,imultblk(ms)
                  dsmat(s1,s) = dsmat(s1,s) + tmpdsmat(s2,s1)*smat(s2,s)
               enddo
            enddo
         enddo

         tmpdsmat = dsmat
         dsmat = transpose(dsmat)
! Carry out the SVD of dsmat.
         call dgesvd('S','S',imultblk(ms),imultblk(ms),dsmat,msta,sigma,umat,msta,&
              vmatt,msta,work,lwork,info)
         if(info.ne.0)then
            routine = 'proc_spin_diab'
            write(message,'(a,i8,a,2i8,a,i8)')'Problem in total WF SVD',info,&
                 ' Records',idb,iprox,' Multiplicity',ms
            call errormsg
         endif
         dsmat = tmpdsmat
! set phases on trafo matrix vectors
!         call trmatphase(umat,imultblk(ms))
! set phases on trafo matrix vectors
!         call trmatphase(vmatt,imultblk(ms))

! The transformation matrix at idb is just the product of the left and right 
! singular vectors transposed (UV^{T}).
         do s = 1,imultblk(ms)
            do s1 = 1,imultblk(ms)
               tmptrans(s1,s) = umat(s1,1)*vmatt(1,s)
               do s2 = 2,imultblk(ms)
                  tmptrans(s1,s) = tmptrans(s1,s) + umat(s1,s2)*vmatt(s2,s)
               enddo
            enddo
         enddo

! Copy the transformation matrix for this multiplicity to the full transformation 
! matrix, including copies for all spin projections.
         soff = ddstoff(ms)
         do s1 = soff+1,soff+ddstblk(ms)
            sact1 = istatemap(s1)-imultoff(ms)
            do s2 = soff+1,soff+ddstblk(ms)
               sact2 = istatemap(s2)-imultoff(ms)
               trans(s1,s2) = tmptrans(sact2,sact1)
            enddo
         enddo

      enddo ! nsmult
! set phases on trafo matrix vectors
      call trmatphase(trans,nddstate)

!-----------------------------------------------------------------------
! Perform the transformation:
!-----------------------------------------------------------------------
      call transform(trans, &
           av(1:nddstate,1:nddstate), &
           aderiv1(1:ndofddpes,1:nddstate,1:nddstate),&
           aderiv2(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate),&
           dv,deriv1,deriv2,dercp(1:ndofddpes,1:nactdim),&
           ndofddpes,nddstate)

! Final deallocation.
      deallocate(stover,sao,smo,lsign,smat,dsmat,sigma,umat,vmatt,tmptrans,&
           sigmamo,umatmo,vmattmo,moco,assmo,cssmo,motransas,motranscs,tmpdsmat,&
           mocoidb,mocoiprox)

  999 continue

      return
      end subroutine diabat6

!########################################################################
! TRMATCHPHASE
!
! Matches the phases of the ADT transformation vectors to a 
! previous set of vectors
!
! Diabatic vectors, i.e. trans = S^T  and trans(i,j) is i th vector
!########################################################################

      subroutine  trmatchphase(transn,transo,nddstate)

      implicit none

      integer(long), intent(in)                    :: nddstate
      integer(long)                                :: i,j,k,lidx
      integer(long), dimension(nddstate)           :: midx
      integer(long), dimension(nddstate,nddstate)  :: itransn, itranso, &
                                                      itmptrans
      real(dop), dimension(nddstate,nddstate), intent(in)    :: transo
      real(dop), dimension(nddstate,nddstate), intent(inout) :: transn
      real(dop)                                    :: tmax
      logical(kind=4), dimension(nddstate)         :: lfound

! get matrices as leading components
      itranso(:,:)=0
      lfound(:)=.false.
      do i=1,nddstate
         tmax=0.0_dop
         do j=1,nddstate
            if (lfound(j)) cycle  ! ensure each vector found once

            if (abs(transo(i,j)) .gt. tmax) then
               tmax = abs(transo(i,j))
               midx = j
            endif
         enddo
         lfound(midx)=.true.
         itranso(i,midx)=1
      enddo

      itransn(:,:)=0
      lfound(:)=.false.
      do i=1,nddstate
         tmax=0.0_dop
         do j=1,nddstate
            if (lfound(j)) cycle  ! ensure each vector found once

            if (abs(transn(i,j)) .gt. tmax) then
               tmax = abs(transn(i,j))
               midx = j
            endif
         enddo
         lfound(midx)=.true.
         itransn(i,midx)=1
      enddo
      
! Get overlap of old and new vectors
      itmptrans(:,:) = 0
      do i=1,nddstate
         do j=1,nddstate
            do k=1,nddstate
               itmptrans(j,i) = itmptrans(j,i) + itranso(j,k)*itransn(i,k)
            enddo
         enddo
      enddo

! Get matching vector 
      do i=1,nddstate
         do j=1,nddstate
            if (itmptrans(j,i) .eq. 1) midx(i)=j   
         enddo
!print*,'MATCH new,old :',i,midx(i)
      enddo

! Get leading component and
! Make phase of new vector so that leading components same as the old
      do i=1,nddstate
         do j=1,nddstate
            if (itransn(i,j) .eq. 1) lidx=j   
         enddo

         j = midx(i)
!print*,'LEAD new :',i,j,lidx
         if (((transn(i,lidx) .lt. 0.0) .and. (transo(j,lidx) .gt. 0.0)) .or. &
             ((transn(i,lidx) .gt. 0.0) .and. (transo(j,lidx) .lt. 0.0))) then
            do k=1,nddstate
               transn(k,lidx) = -transn(k,lidx)
            enddo
         endif
      enddo

      return
      end subroutine  trmatchphase

!########################################################################
! TRMATPHASE
!
! Sets the phase convention for the ADT transformation matrices
!
!########################################################################

      subroutine trmatphase(trmat,trdim)

      implicit none

      integer(long), intent(in)                      :: trdim
      integer(long)                                  :: i,j,info
      real(dop), dimension(trdim,trdim), intent(out) :: trmat
      real(dop), dimension(trdim,trdim)              :: trmat1
      real(dop), dimension(trdim)                    :: tmpe,tmpe2
      real(dop), dimension(1)                        :: rdum,rdum1
      real(dop), dimension(3*nddstate)               :: work
      complex(dop)                                   :: rotdet

! (first vector lies in upper half of rotation plane.)
      do i = 1,trdim
         if(trmat(i,i).lt.0.0_dop) then
            do j = 1,trdim
               trmat(j,i) = -trmat(j,i)
            enddo
         endif
      enddo

! Determinant of the transformation matrix.
      trmat1 = trmat
      call dgeev('N','N',trdim,trmat1,trdim,tmpe,tmpe2,&
           rdum,1,rdum1,1,work,3*trdim,info)
      rotdet = (1.0_dop,0.0_dop)
      do i = 1, trdim
         rotdet = rotdet*dcmplx(tmpe(i),tmpe2(i))
      enddo

! Modify the sign of the transformation matrix if needed.
      if(dble(rotdet).lt.0.0_dop)then
         do i = 1,trdim
            trmat(i,2) = -1.0_dop*trmat(i,2)
         enddo
      endif  

      return
      end subroutine trmatphase

!########################################################################
! TRMATPHASE
!
! Sets the phase convention for the ADT transformation matrices
!
!########################################################################

      subroutine trmatphase1(trmat,trmat0,trdim)

      implicit none

      integer(long), intent(in)                      :: trdim
      integer(long)                                  :: i,j
      real(dop), dimension(trdim,trdim), intent(out) :: trmat
      real(dop), dimension(trdim,trdim), intent(in)  :: trmat0
      real(dop)                                      :: dotp

      do j=1,trdim
! projection of interpolated nact (aderiv10) on calculated (dercp)
         dotp = 0.0
         do i = 1, trdim
            dotp = dotp + &
                 trmat(i,j)*trmat0(i,j)
         enddo

! Modify the sign of the transformation matrix if needed.
         if (dotp .lt. 0.0_dop) then
            do i = 1,trdim
               trmat(i,j) = -1.0_dop*trmat(i,j)
            enddo
         endif  
      enddo

      return
      end subroutine trmatphase1

!########################################################################
! Subroutine to get distances between points for the global 
! diabatisation. 
! GWR 12/14
!########################################################################

      subroutine nactdist(shepdist,xgpoint1,xgpoint0,xgp,ndof)

      implicit none

      integer(long), intent(in)  :: ndof
      real(dop), dimension(2), intent(out)        :: shepdist
      real(dop), dimension(ndof), intent(in)      :: xgpoint0,xgpoint1,xgp

      integer(long)  :: jdx
      real(dop) :: r

      shepdist(:) = 0.0_dop

! Distance to first point
      do jdx = 1,ndof
         r = xgp(jdx) - xgpoint0(jdx)
         shepdist(1) = shepdist(1) + r*r 
      enddo
      shepdist(1) = sqrt(shepdist(1))

! Distance to final point
      do jdx = 1,ndof
         r = xgp(jdx) - xgpoint1(jdx)
         shepdist(2) = shepdist(2) + r*r 
      enddo
      shepdist(2) = sqrt(shepdist(2))

      return  
      end subroutine nactdist

!#######################################################################

      subroutine dbdist_gp(shepdist,xgpoint,xgp,ndof,nrec)

      implicit none

      integer(long), intent(in)  :: ndof,nrec
      integer(long), dimension(nrec)     :: loc
      real(dop), dimension(nrec+1), intent(out)   :: shepdist
      real(dop), dimension(ndof), intent(in)      :: xgpoint,xgp

      integer(long)  :: idx, jdx,irec
      real(dop) :: r
      type(nextptr), pointer :: current

      shepdist(:) = 0.0_dop

      call list_init(Current)
      current => ngp_loc(num_gp)%locpt
      do irec = 1,nrec
         loc(irec) = current%locDB
         current => current%Next
      enddo

! Loop over the database
      do idx = 1, nrec
         if (dbignore(loc(idx)) .eq. 1) cycle
         do jdx = 1,ndof
            r = xgp(jdx) - dbgeo(jdx,loc(idx))
            shepdist(idx) = shepdist(idx) + r*r 
         enddo
         shepdist(idx) = sqrt(shepdist(idx))
      enddo

! Loop over the end points
      do jdx = 1,ndof
         r = xgp(jdx) - xgpoint(jdx)
         shepdist(nrec+1) = shepdist(nrec+1) + r*r 
      enddo
      shepdist(nrec+1) = sqrt(shepdist(nrec+1))

      return  
      end subroutine dbdist_gp

!##########################################################################
! Routine to transform the energy, gradient and Hessian from the adiabatic
! to the diabatic representation.
! GWR 09/14
!##########################################################################

      subroutine transform(transf,av,aderiv1,aderiv2,v,deriv1,deriv2,&
           nactend,ndofddpes,nddstate)

      implicit none

      integer(long), intent(in)                    :: ndofddpes,nddstate
      real(dop), dimension(ndofddpes,nactdim), intent(in)  :: nactend
      real(dop), dimension(nddstate,nddstate), intent(in)  :: av,transf
      real(dop), dimension(ndofddpes,nddstate,nddstate), intent(in) :: aderiv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), intent(in) :: aderiv2

      real(dop), dimension(nddstate,nddstate), intent(out)           :: v
      real(dop), dimension(ndofddpes,nddstate,nddstate), intent(out) :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), intent(out) :: deriv2

      integer(long)                              :: idx,jdx,kdx,ldx
      real(dop)                                  :: dtmp
      real(dop), dimension(nddstate,nddstate)    :: tmpmat,tmpmat2

! We have successfully got the transformation matrix, now use it.
! Transform the potentials, linear terms and quadratic terms to the 
! diabatic representation.
! First the PES:
      tmpmat = matmul(av,transf)
      call qqtxdd(transf,tmpmat,v,nddstate)

! Now the linear terms, DOF by DOF:
      do idx = 1, ndofddpes
         ldx = 0
         do jdx = 1, nddstate
            tmpmat2(jdx,jdx) = aderiv1(idx,jdx,jdx)
            do kdx = jdx+1, nddstate
! nact array stored in upper triangle
               ldx = (kdx-1)*(kdx-2)/2+jdx
               tmpmat2(jdx,kdx) = nactend(idx,ldx)
               tmpmat2(kdx,jdx) = tmpmat2(jdx,kdx)
            enddo
         enddo
               
         tmpmat = matmul(tmpmat2,transf)
         call qqtxdd(transf,tmpmat,tmpmat2,nddstate)
         do jdx = 1, nddstate
            do kdx = 1, nddstate
               deriv1(idx,jdx,kdx) = tmpmat2(jdx,kdx)

            enddo
         enddo
      enddo

! Hessian      
      if(dbnrec.eq.0)then
         deriv2 = 0.0_dop
         do idx=1,nddstate
            deriv2(:,:,idx,idx) = aderiv2(:,:,idx,idx)
            do jdx=1,nddstate
               if (idx .eq. jdx) cycle

! correct on-diagonal Hessian for NACT
               do kdx = 1,ndofddpes              
                  do ldx = 1,ndofddpes              
                     dtmp = 2.0*deriv1(kdx,idx,jdx)*deriv1(ldx,jdx,idx)/ &
                              (av(jdx,jdx) - av(idx,idx))

                     deriv2(kdx,ldx,idx,idx) = &
                          deriv2(kdx,ldx,idx,idx) + dtmp

!set up approximate off-diagonal Hessian
!                    dtmp = deriv1(kdx,idx,jdx)* &
!                         (deriv1(ldx,jdx,jdx) - deriv1(ldx,idx,idx)) 
!                    dtmp = dtmp / (av(jdx,jdx) - av(idx,idx))
!                    deriv2(kdx,ldx,idx,jdx) = deriv2(kdx,ldx,idx,jdx) + dtmp

!                    dtmp = deriv1(ldx,idx,jdx)* &
!                         (deriv1(kdx,jdx,jdx) - deriv1(kdx,idx,idx)) 
!                    dtmp = dtmp / (av(jdx,jdx) - av(idx,idx))
!                    deriv2(kdx,ldx,idx,jdx) = deriv2(kdx,ldx,idx,jdx) + dtmp

!                    do mdx = 1,nddstate
!                       if (mdx .eq. idx) cycle
!                       if (mdx .eq. jdx) cycle

!                       dtmp = deriv1(kdx,idx,mdx)*deriv1(ldx,mdx,jdx)
!                       dtmp = dtmp / (av(mdx,mdx) - av(idx,idx)) 
!                       deriv2(kdx,ldx,idx,jdx) = &
!                             deriv2(kdx,ldx,idx,jdx) + dtmp

!                       dtmp = deriv1(ldx,idx,mdx)*deriv1(kdx,mdx,jdx)
!                       dtmp = dtmp / (av(mdx,mdx) - av(jdx,jdx)) 
!                       deriv2(kdx,ldx,idx,jdx) = &
!                             deriv2(kdx,ldx,idx,jdx) + dtmp
!                    enddo
                  enddo
               enddo
            enddo

         enddo
      endif

      return
      end subroutine transform

!-----------------------------------------------------------------------

      subroutine dcpphase(oldderiv1,newderiv1,ndofddpes,nddstate)

      implicit none

      integer(long)             :: f,s,s1
      integer(long), intent(in) :: nddstate,ndofddpes
      real(dop)                 :: dotp
      real(dop), intent(in), dimension(ndofddpes,nddstate,nddstate)    :: oldderiv1
      real(dop), intent(inout), dimension(ndofddpes,nddstate,nddstate) :: newderiv1

      do s = 1,nddstate-1
         do s1 = s+1,nddstate
            dotp = 0.0_dop
            do f=1,ndofddpes
               dotp = dotp + oldderiv1(f,s1,s)*newderiv1(f,s,s1)
            enddo

            if (dotp .lt. 0.0) then
               if (abs(dotp) .gt. 0.05) then
                  if (ddlog) then
                     write(ilog,'(a,i5,i5)') &
            'Change in direction of DCP predicted -> corrected. Taken. ',s1,s
                  endif
                  do f=1,ndofddpes
                     newderiv1(f,s,s1) = -newderiv1(f,s,s1)
                     newderiv1(f,s1,s) = -newderiv1(f,s1,s)
                  enddo
               else
                  if (ddlog) then
                     write(ilog,'(a,i5,i5)') &
            'Change in direction of DCP predicted -> corrected. Ignored. ',s1,s
                  endif
               endif
            endif
         enddo
      enddo

      return
      end subroutine dcpphase

!##########################################################################
! Routine to Shepard interpolate the derivative couplings.
! GWR 10/14
!##########################################################################

      subroutine shepnact(nrec,ndof,nactdb,nactend,nactcur,xgpoint,xgpcur,&
           linner)

use global
use dirdyn

use dd_db, only: ddx2q

      implicit none
      
      logical(kind=4), intent(in)                 :: linner
      integer(long), intent(in)                   :: nrec,ndof
      real(dop), dimension(nrec,ndof), intent(in) :: nactdb
      real(dop), dimension(ndof), intent(in)      :: nactend,xgpoint,xgpcur
      real(dop), dimension(ndof), intent(out)     :: nactcur

      integer(long)                :: i,j
      real(dop)                    :: wsum,diff
      real(dop), dimension(nrec+1) :: dist
      real(dop), dimension(ndof)   :: dbgpoint,r
      real(dop), dimension(ndofdd) :: gpoint,gpcur

      nactcur(:) = 0.0_dop

      if(lddtrans)then
         call ddx2q(xgpcur,gpcur)
      endif

! Get the distances (loop over the DB, and also for the target point).
      dist(:) = 0.0_dop
      wsum = 0.0_dop
      do i = 1, nrec
         if(lddtrans)then
            call ddx2q(dbgeo(1:ndof,i),dbgpoint)
            do j=1,ndofdd
               r(j) = gpcur(j) - dbgpoint(j)
               dist(i) = dist(i) + r(j)*r(j)
            enddo
         else
            do j=1,ndof
               r(j) = xgpcur(j) - dbgeo(j,i)
               dist(i) = dist(i) + r(j)*r(j)
            enddo
         endif
      enddo
      if(lddtrans)then
         call ddx2q(xgpoint,gpoint)
         do j=1,ndofdd
            r(j) = gpcur(j) - gpoint(j)
            dist(nrec+1) = dist(nrec+1) + r(j)*r(j)
         enddo
      else
         do j=1,ndof
            r(j) = xgpcur(j) - xgpoint(j)
            dist(nrec+1) = dist(nrec+1) + r(j)*r(j)
         enddo
      endif

! Now do the interpolation.
      do i = 1, ndof
         do j = 1, nrec

            diff = dist(j)-dist(1)
            if ((diff .lt. -1.0e-10_dop) .or. (j.eq.1) .or. (.not. linner))then

               nactcur(i) = nactcur(i) + nactdb(j,i)/dist(j)**2
               wsum = wsum + 1.0_dop/dist(j)**2

            endif
         enddo

         diff = dist(nrec+1)-dist(1)
         if ((diff .lt. 1.0e-10_dop)  .or. (j.eq.1) .or. .not. linner) then
            nactcur(i) = nactcur(i) + nactend(i)/dist(nrec+1)**2
            wsum = wsum + 1.0_dop/dist(nrec+1)**2
         endif

! Divide by the sum of the weights.
         nactcur(i) = nactcur(i)/wsum
      enddo

      return
      end subroutine shepnact

!#########################################################################

      subroutine ddshell(s)

use global
use psidef, only: isbaspar,sbasis,gwpdim
use griddatmod, only :nmode,spfdof,nspfdof
use dirdyn

      implicit none

      integer(long), intent(in) :: s
      logical(kind=4)   :: lgwpfill
      integer(long)     :: idx,m,f,shell,subshell,gdof,npnts,mfact,ntmp,&
                           npntsold,i,mshell


      fgwpshell = 0
      gwpshell = 0
      idx = 1
      fgwpshell(1,1) = 1
      lgwpfill = .false.

      do m=1,nmode
         f=spfdof(1,m)
         if (sbasis(f) .eq. 9 .and. isbaspar(1,f,s) .eq. 3) then 
            
            shell=0 
            subshell=0
            gdof=nspfdof(m)
            npnts=1
            npntsold = 0
120         continue
            if (gwpdim(m,s) .eq. npnts) then
               lgwpfill = .true.
            else if (gwpdim(m,s) .lt. npnts) then
               lgwpfill = .false.
            else if (gwpdim(m,s) .gt. npnts) then
               if (shell .eq. 0) then
                  shell=1
                  subshell=1
                  gwpshell(1,1) = shell+1
                  gwpshell(1,2) = subshell
               else if (subshell .lt. gdof) then 
                  subshell=subshell+1
               else if (subshell .eq. gdof) then 
                  shell=shell+1
                  subshell=1
               endif
               
               mshell=2*shell
               mfact=1
               do i=gdof-subshell+1,gdof
                  mfact=mfact*i
               enddo
               do i=1,subshell
                  mfact=mfact/i
               enddo
               
               ntmp=mshell**subshell - (mshell-2)**subshell
               npntsold = npnts
               npnts=npnts + mfact*ntmp
               do i = npntsold+1,min(gwpdim(m,s),npnts)
                  gwpshell(i,1) = shell+1
                  gwpshell(i,2) = subshell
               enddo
               idx = idx + 1
               fgwpshell(shell+1,subshell) = min(gwpdim(m,s),npnts)

               go to 120
            endif
         endif
      enddo

      if(lgwpfill)then
         gwpshell(1,1) = 1
      endif

      return

      end subroutine ddshell

!------------------------------------------------------------------------------    

      end module diabatmod
