!#######################################################################
      module func1dmod

use decimal, only: dop,long
use keyunitsmod
use logdat
use splinepot1dmod
use operdef, only: splpaths, splunits

      implicit none
      private
      public :: fdef1d,func1d
      contains

!#######################################################################
!
! 1D CURVES  (Real functions without parameters).
!
! ifile = 303
!
!#######################################################################


      subroutine fdef1d(ifunc,label,ifile,ipar,rpar,maxpar,hoppar)

      implicit none

      integer(long), intent(in)                    :: maxpar
      integer(long)                                :: ilbl,jlbl,n
      integer(long), intent(out)                   :: ifunc,ifile
      integer(long), dimension(maxpar), intent(out):: ipar
      real(dop)                                    :: depth,freq,x0,xe0,&
                                                      mass,alpha,e0,jtot,&
                                                      jbf,vcsmax,rabs,cabs,&
                                                      lj1,lj2,lj3,lj4
      real(dop), dimension(maxpar), intent(out)    :: rpar
      real(dop), dimension(maxpar,*), intent(in)   :: hoppar
      character(len=*), intent(in)                 :: label

      ipar(:) = 0
      rpar(:) = 0.0
!-----------------------------------------------------------------------
! now assign function number
!-----------------------------------------------------------------------
! NO potential
      if (label .eq. 'v:NO') then
         ifunc=1

! H2 potential
      else if (label .eq. 'v:H2') then
         ifunc=2

! Morse potential
      else if (label(1:7) .eq. 'morse1[') then
         ilbl=index(label,'[')+1
         jlbl=index(label,']')-1
         read(label(ilbl:jlbl),*) n

         depth=hoppar(1,n)
         alpha=hoppar(2,n)
         x0=hoppar(3,n)
         e0=hoppar(4,n)

         rpar(1)=depth
         rpar(2)=alpha
         rpar(3)=x0
         rpar(4)=e0

         ifunc=3

! Morse potential
      else if (label(1:6) .eq. 'morse[') then
         ilbl=index(label,'[')+1
         jlbl=index(label,']')-1
         read(label(ilbl:jlbl),*) n

         depth=hoppar(1,n)
         freq=hoppar(2,n)
         x0=hoppar(3,n)
         xe0=hoppar(4,n)
         mass=hoppar(5,n)
         alpha=(mass*freq*freq)/(2*depth)
         alpha=dsqrt(alpha)
         e0=exp(-alpha*(xe0-x0)) - 1
         e0=-e0*e0*depth

         rpar(1)=depth
         rpar(2)=alpha
         rpar(3)=x0
         rpar(4)=e0

         ifunc=3


! centrifugal potential
      else if (label(1:6) .eq. 'cspot[') then
         ilbl=index(label,'[')+1
         jlbl=index(label,']')-1
         read(label(ilbl:jlbl),*) n

         jtot=hoppar(1,n)
         jbf=hoppar(2,n)
         vcsmax=hoppar(3,n)
         mass=hoppar(4,n)

         rpar(1)=jtot
         rpar(2)=jbf
         rpar(3)=vcsmax
         rpar(4)=mass

         ifunc=4

! Phenol potential for V20 
      else if (label .eq. 'v:phenol_20') then
         ifunc=5

! Phenol potential for V21 
      else if (label .eq. 'v:phenol_21') then
         ifunc=6

! Phenol potential for V22 
      else if (label .eq. 'v:phenol_22') then
         ifunc=7 

! Pyrrolle potential for V22  1B1
      else if (label .eq. 'v:pyrrole_22b') then
         ifunc=8 

! Pyrrolle potential for V22  1A2
      else if (label .eq. 'v:pyrrole_22a') then
         ifunc=9

! Pyrrolle angle-dependent frequency for V22 1B1
      else if (label .eq. 'v:pyrrole_2wb') then
         ifunc=10

! Morse2 potential
         else if (label(1:7) .eq. 'morse2[') then
            ilbl=index(label,'[')+1
            jlbl=index(label,']')-1
            read(label(ilbl:jlbl),*) n

            rpar(1)=hoppar(1,n)
            rpar(2)=hoppar(2,n)
            rpar(3)=hoppar(3,n)

            ifunc=11

! Avoided crossing model potential
         else if (label(1:6) .eq. 'acpot[') then
            ilbl=index(label,'[')+1
            jlbl=index(label,']')-1
            read(label(ilbl:jlbl),*) n

            rpar(1)=hoppar(1,n)
            rpar(2)=hoppar(2,n)
            rpar(3)=hoppar(3,n)
            rpar(4)=hoppar(4,n)
            rpar(5)=hoppar(5,n)
            rpar(6)=hoppar(6,n)
            rpar(7)=hoppar(7,n)
            rpar(8)=hoppar(8,n)

            ifunc=12

! Avoided crossing model potential 2
         else if (label(1:7) .eq. 'acpot2[') then
            ilbl=index(label,'[')+1
            jlbl=index(label,']')-1
            read(label(ilbl:jlbl),*) n

            rpar(1)=hoppar(1,n)
            rpar(2)=hoppar(2,n)
            rpar(3)=hoppar(3,n)
            rpar(4)=hoppar(4,n)
            rpar(5)=hoppar(5,n)
            rpar(6)=hoppar(6,n)
            rpar(7)=hoppar(7,n)
            rpar(8)=hoppar(8,n)
            rpar(9)=hoppar(9,n)

            ifunc=13

! Avoided crossing model potential 3                                                                                                                                    
         else if (label(1:7) .eq. 'acpot3[') then
            ilbl=index(label,'[')+1
            jlbl=index(label,']')-1
            read(label(ilbl:jlbl),*) n

            rpar(1)=hoppar(1,n)
            rpar(2)=hoppar(2,n)
            rpar(3)=hoppar(3,n)
            rpar(4)=hoppar(4,n)
            rpar(5)=hoppar(5,n)
            rpar(6)=hoppar(6,n)
            rpar(7)=hoppar(7,n)
            rpar(8)=hoppar(8,n)
            rpar(9)=hoppar(9,n)

            ifunc=14

! Morse3 potential
         else if (label(1:7) .eq. 'morse3[') then
            ilbl=index(label,'[')+1
            jlbl=index(label,']')-1
            read(label(ilbl:jlbl),*) n

            rpar(1)=hoppar(1,n)
            rpar(2)=hoppar(2,n)
            rpar(3)=hoppar(3,n)

            ifunc=15

! Damping function
      else if (label(1:5) .eq. 'DAMP[') then
         ilbl=index(label,'[')+1
         jlbl=index(label,']')-1
         read(label(ilbl:jlbl),*) n

         rpar(1)=hoppar(1,n)
         rpar(2)=hoppar(2,n)

         rabs=rpar(1)
         cabs=rpar(2)

         ifunc=16

! ifunc = 17 1D spline, assigned in funcsrf1

! Inertial function for CPTHF rotation
      else if (label(1:12) .eq. 'inert:cpthf[') then
         ilbl=index(label,'[')+1
         jlbl=index(label,']')-1
         read(label(ilbl:jlbl),*) n

         rpar(1)=hoppar(1,n)
         rpar(2)=hoppar(2,n)
         rpar(3)=hoppar(2,n)
         rpar(4)=hoppar(2,n)

         ifunc=18

! OH Morse potential. JCP 113, 3150 (2000), from h20.f 
      else if (label .eq. 'v:HO') then
         ifunc=19

! Potential for OF
      else if (label .eq. 'v:OF') then
         ifunc=20

! labels for H+H2 in hypersphaerical coordinates.

      else if (label .eq. 'vrho:H3') then
         ifunc=21

      else if (label .eq. 'vthe:H3') then
         ifunc=22

! Lennard-Jones potential
      else if (label(1:3) .eq. 'lj[') then
         ilbl=index(label,'[')+1
         jlbl=index(label,']')-1
         read(label(ilbl:jlbl),*) n

         depth=hoppar(1,n)
         alpha=hoppar(2,n)
         x0=hoppar(3,n)
         e0=hoppar(4,n)
         lj1=depth*alpha**12
         lj2=-2*depth*alpha**6
         lj3=-depth*alpha**12*x0**(-12) + 2*depth*alpha**6*x0**(-6)
         lj4=x0
         rpar(1)=lj1
         rpar(2)=lj2
         rpar(3)=lj3
         rpar(4)=e0
         rpar(5)=lj4

         ifunc=23
!-----------------------------------------------------------------------
! next free ifunc is 24
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
! end of if loop
!-----------------------------------------------------------------------
      endif

!-----------------------------------------------------------------------
! if label is assigned, assign ifile no.
!-----------------------------------------------------------------------
      if (ifunc .ne. 0) ifile=303

      return
      end subroutine fdef1d

!#######################################################################

      subroutine func1d(v,x,ifunc,hopipar,hoprpar,label)

use global
use potofmod, only:potof

      implicit none


      integer(long), intent(in)               :: ifunc
      integer(long), dimension(*), intent(in) :: hopipar
      integer(long)                           :: imod,icoll

      real(dop)       :: x1,x0,depth,alpha,e0,jtot,jbf,vcsmax,mass,&
                         rho,alf,theta,r1,r2,r3,rabs,cabs,lj1,lj2,lj3,lj4

      real(dop), intent(in)               :: x
      real(dop), dimension(*), intent(in) :: hoprpar
      real(dop), intent(out)              :: v
      real(dop), external                 :: h4dj_v000,h4dj_v022,h4dj_v224 ! external functions
      character(len=*), intent(in)        :: label

! functions
      real(dop), external                  :: licnsrf,voh 
!      real(dop)                           :: vno,morse,cscentri,licnsrf,voh,vh2bmkp

      select case (ifunc) 

      case (1) ! NO potential
      v=vno(x)
      return

      case (2) ! H2 potential
      x1=max(x,0.4_dop)
      call vh2(x1,v)
      return

      case (3) ! morse
      depth=hoprpar(1)
      alpha=hoprpar(2)
      x0=hoprpar(3)
      e0=hoprpar(4)
      v=morse(x,depth,alpha,x0,e0)
      return

      case (4) ! centrifugal
      jtot=hoprpar(1)
      jbf=hoprpar(2)
      vcsmax=hoprpar(3)
      mass=hoprpar(4)
      v=cscentri(x,jtot,jbf,vcsmax,mass)
      return

      case(5) ! phenol V20 (Lan et al JCP (05) 122:224315)
      imod = 1
      call phenol_v1dv20(v,x,imod)
      return

      case(6) ! phenol V21 (Lan et al JCP (05) 122:224315)
      imod = 1
      call phenol_v1dv21(v,x,imod)
      return

      case(7) ! phenol V22 (Lan et al JCP (05) 122:224315)
      imod = 1
      call phenol_v1dv22(v,x,imod)
      return

      case(8) ! pyrrolle 1B1 V22 (Vallet et al JCP (05) 123:144307)
      imod = 2
      call phenol_v1dv20(v,x,imod)
      return

      case(9) ! pyrrolle 1A2 V22 (Vallet et al JCP (05) 123:144307)
      imod = 3
      call phenol_v1dv20(v,x,imod)
      return

      case(10) ! pyrrolle 1B2 V2 frequency (Vallet et al JCP (05) 123:144307)
      imod = 1
      call pyrolle_v1dw2(v,x,imod)
      return

      case(11) ! Morse2
      v=morse2(x,hoprpar(1),hoprpar(2),hoprpar(3))
      return

      case(12) ! Avoided crossing
      v=acpot(x,hoprpar(1),hoprpar(2),hoprpar(3),hoprpar(4),hoprpar(5), &
           hoprpar(6),hoprpar(7),hoprpar(8))
      return

      case(13) ! Avoided crossing 2
      v=acpot2(x,hoprpar(1),hoprpar(2),hoprpar(3),hoprpar(4),hoprpar(5), &
           hoprpar(6),hoprpar(7),hoprpar(8),hoprpar(9))
      return

      case(14) ! Avoided crossing 3
      v=acpot3(x,hoprpar(1),hoprpar(2),hoprpar(3),hoprpar(4),hoprpar(5), &
           hoprpar(6),hoprpar(7),hoprpar(8),hoprpar(9))
      return

      case(15) ! Morse3
      v=morse3(x,hoprpar(1),hoprpar(2),hoprpar(3))
      return

      case (16) ! 1D Damping function 
      rabs=hoprpar(1)
      cabs=hoprpar(2)
      if (x <= rabs) then
         v=1.0_dop
      else
         v=dexp(-cabs*(x-rabs)**2)
      endif
      return
      
      case (17) ! 1D Spline Fits
      call splinepot1d(v,x,label,hopipar,hoprpar)
      return

      case (18) ! CPTHF rotational inertia
      v = sin(x - hoprpar(4))
      v = hoprpar(3)*v**2
      v = hoprpar(2)+v
      v = 1.0d0/v
      return

      case (19) ! HO potential.  JCP 113, 3150 (2000), from h2o.f
      depth=0.1628_dop
      alpha=1.22_dop
      x0=1.85_dop
      e0=-depth
      v=morse(x,depth,alpha,x0,e0)
      return

      case (20) ! Potential for OF
      call potof(x,v)
      return

      case (21) ! vrho: 1D potential in rho, hypersphaerical coordinates, H+H2
      imod=0
      icoll=0
      rho=x
      alf=0.7853981634_dop
      theta=3.1415926536_dop

      r1=rho*sin(alf)
      r2=rho*cos(alf)
      r3=r1**2+r2**2-2.0_dop*r1*r2*cos(theta)
      r3=sqrt(r3)
      r1=max(0.4_dop,r1)
      r2=max(0.4_dop,r2)
      r3=max(0.4_dop,r3)
      call lsth(r1,r2,r3,v,imod,icoll)
      return


      case (22) ! vthe : 1D potential in theta, hypersphaerical coordinates, H+H2
      imod=0
      icoll=0
      rho=2.484773_dop
      alf=0.7853981634_dop
      theta=x

      r1=rho*sin(alf)
      r2=rho*cos(alf)
      r3=r1**2+r2**2-2.0_dop*r1*r2*cos(theta)
      r3=sqrt(r3)
      r1=max(0.4_dop,r1)
      r2=max(0.4_dop,r2)
      r3=max(0.4_dop,r3)
      call lsth(r1,r2,r3,v,imod,icoll)
      return

      case(23) !lennard-jones potential

      lj1=hoprpar(1)
      lj2=hoprpar(2)
      lj3=hoprpar(3)
      e0=hoprpar(4)
      lj4=hoprpar(5)
      v=lj(x,lj1,lj2,lj3,e0,lj4)
      return

      case default
      routine='Func1d'
      write(message,'(a,i5)') 'Function not found :',ifunc
      call errormsg
      return

      end select

      end subroutine func1d

!#######################################################################
!
! NO potential
!
!#######################################################################

      real(dop) function vno(x)

      implicit none

      real(dop), intent(in) :: x
      real(dop)             :: xnew,vnoeq

      vnoeq=2.136_dop
      xnew=x-vnoeq

      vno = 0.6816_dop*xnew**2 - 0.9123_dop*xnew**3 + 0.4115_dop*xnew**4

      return
      end function vno

!#######################################################################
! H2 potential vh2 is in the LSTH routine
! CH potential vch is in the c2h routine (addsurf).
! C2 potential vc2 is in the c2hasec routine (addsurf).
! OH potential voh is in the hoosrf routine (addsurf).
!#######################################################################

!#######################################################################
!
! morse potential
!
! V = depth(Exp(-alpha(x-x0)) - 1)^2 - e0
!
!#######################################################################

      real(dop) function morse(x,depth,alpha,x0,e0)

      implicit none

      real(dop), intent(in) :: x,depth,alpha,x0,e0

      morse=exp(-alpha*(x-x0))-1
      morse=morse*morse*depth
      morse=morse+e0

      return
      end function morse

!#######################################################################
!
! Lennard-jones potential
!
! V = Do*[ (alpha/x+dx)**12 - 2*(alpha/x+dx)**6 -(alpha/dx)**12 +  2*(alpha/dx)**6 ] + e0
!
!#######################################################################

      real(dop) function lj(x,lj1,lj2,lj3,e0,lj4)

      implicit none

      real(dop), intent(in) :: x,lj1,lj2,lj3,lj4,e0

      lj=lj1*(x+lj4)**(-12) + lj2*(x+lj4)**(-6) + lj3 
      lj=lj+e0

      return
      end function lj

!#######################################################################
!
! morse2 potential
!
!#######################################################################

      real(dop) function morse2(x,k1,k2,k3)

      implicit none

      real(dop) :: x,k1,k2,k3

      morse2=k1*(1-exp(-k2*(x-k3)))**2

      return
      end function morse2

!#######################################################################
!
! morse3 potential
!
!#######################################################################

      real(dop) function morse3(x,D0,alpha,q0)

        implicit none

        real(dop) :: x,D0,alpha,q0,E0

        E0=-D0*(1-exp(alpha*q0))**2

        morse3=D0*(1-exp(-alpha*(x-q0)))**2 + E0

        return
      end function morse3

!#######################################################################
!
! avoided crossing model potential
!
!#######################################################################

      real(dop) function acpot(x,D1,alpha1,R1,prex,alpha2,R2,D2,cconst)

      implicit none

      real(dop), intent(in) :: x,D1,D2,alpha1,alpha2,R1,R2,prex,cconst
      real(dop) :: bound,diss

      bound = D1*((1 - exp(-alpha1*(x-R1)))**2)
      diss = prex*(exp(-alpha2*(x-R2))) + D2

      acpot=0.5*(bound + diss - sqrt((bound-diss)**2 + cconst**2))

      return
      end function acpot

!#######################################################################
!
! avoided crossing model potential 2
!
!#######################################################################

      real(dop) function acpot2(x,D1,alpha1,R1,prex,alpha2,R2,D2,cconst1,cconst2)

      implicit none

      real(dop), intent(in) :: x,D1,D2,alpha1,alpha2,R1,R2,prex,cconst1,cconst2
      real(dop) :: bound,diss

      bound = D1*((1 - exp(-alpha1*(x-R1)))**2)
      diss = prex*(exp(-alpha2*(x-R2))) + D2

      acpot2=0.5*(bound + diss - &
           sqrt((bound-diss)**2 + (cconst1*tanh(cconst2*x))**2))

      return
    end function acpot2

!#######################################################################                                                                                                
!                                                                                                                                                                       
! avoided crossing model potential 3                                                                                                                                    
!                                                                                                                                                                       
!#######################################################################                                                                                                

    real(dop) function acpot3(x,D1,alpha1,R1,Einf,alpha2,cconst1,cconst2,E1,E2)

      implicit none

      real(dop), intent(in) :: x,D1,alpha1,R1,Einf,alpha2,cconst1,cconst2,E1,E2
      real(dop) :: bound,diss


      bound = D1*((1-exp(-alpha1*(x-R1)))**2) + E1
      diss = Einf*(exp(-alpha2*x)-1) + E2

      acpot3=0.5*(bound + diss - &
           sqrt((bound-diss)**2 + 4*(cconst1*tanh(cconst2*x))**2))

      return
    end function acpot3

!#######################################################################
!
! centrifugal potential
!
!#######################################################################

      real(dop) function cscentri(x,jtot,jbf,vcsmax,mass)

      implicit none

      real(dop), intent(in) :: x,jtot,jbf,vcsmax,mass

      cscentri=(jtot*(jtot+1_dop)-2_dop*jbf*jbf)/(2_dop*mass)
      cscentri=cscentri/(x*x)
      cscentri=min(vcsmax,cscentri)

      return
      end function cscentri

!##########################################################################

      subroutine splinepot1d(v,gpoint,label,hopipar,hoprpar)

use runopermod

      implicit none

      integer(long)                           :: n,nspl
      integer(long)                           :: ilbl,ll,ierr
      integer(long), dimension(*), intent(in) :: hopipar
      real(dop)                               :: r1,rtmp
      real(dop), intent(out)                  :: v
      real(dop), intent(in)                   :: gpoint
      real(dop)                               :: vmax,vzero
      real(dop), dimension(*), intent(in)     :: hoprpar
      character(len=120)                      :: filename
      character(len=*), intent(in)            :: label

!-----------------------------------------------------------------------
! get splinepath index and options
!-----------------------------------------------------------------------
      nspl = hopipar(1)
      n = hopipar(2)
      vzero = hoprpar(1)
      vmax  = hoprpar(2)

!-----------------------------------------------------------------------
! coordinate unit trafo
!-----------------------------------------------------------------------
      r1 = gpoint

      rtmp=1.0_dop
      call keylunits(rtmp,splunits(1,n),ierr)
      r1=r1/rtmp

!-----------------------------------------------------------------------
! calculate spline fit
!-----------------------------------------------------------------------
      ll=index(label,' ')-1

      ilbl=index(splpaths(nspl),' ')-1
      filename=splpaths(nspl)(1:ilbl)//'/'//label(1:ll)//'.1dspl'
      call splinepot1d_css(r1,v,filename)
      call keyeunits(v,splunits(2,n),ierr)

!-----------------------------------------------------------------------
! add energy zero and cut off high values
!-----------------------------------------------------------------------
      v = v+vzero
      v = min(v,vmax)

      return
      end subroutine splinepot1d

!#######################################################################

      subroutine phenol_v1dv20(v,r,imod)

      implicit none

      integer(long), intent(in) :: imod
      real(dop), intent(out) :: v
      real(dop), intent(in) :: r
      real(dop) :: angst,ev,tmp
      real(dop) :: b201,b202,b203,b204,b205,b206,b207,b208,chi20
      real(dop) :: v201,v202

      ev = 27.2114d0
      angst = 0.52917725d0
      if (imod .eq. 1) then
         b201 = 0.192205/ev
         b202 = 5.67356*angst
         b203 =1.03171/angst
         b204 = 5.50696/ev
         b205 = 4.70601/ev
         b206 = 2.49826*angst
         b207 = 0.988188/angst
         b208 = 3.3257/ev
         chi20 = 0.326432/(ev*ev)
      else if (imod .eq. 2) then
         b201 = 8.070/ev
         b202 = 0.882
         b203 = 1.922
         b204 = 5.584/ev
         b205 = 0.091/ev
         b206 = 1.290
         b207 = 5.203
         b208 = 4.092/ev
         chi20 = 4.0d0*(1.669/ev)**2
      else if (imod .eq. 3) then
         b201 = 4.979/ev
         b202 = 1.293
         b203 = 1.882
         b204 = 4.805/ev
         b205 = 2.644/ev
         b206 = 1.325
         b207 = 2.216
         b208 = 3.956/ev
         chi20 = 4.0d0*(1.248/ev)**2
      else
         b201 = 0.0
         b202 = 0.0
         b203 = 0.0
         b204 = 0.0
         b205 = 0.0
         b206 = 0.0
         b207 = 0.0
         b208 = 0.0
         chi20 = 0.0
      endif

      tmp = -b202*(r - b203)
      v201 = 1.0d0 - exp(tmp)
      v201 = b201*(v201**2) + b204

      tmp = -b206*(r - b207)
      v202 = exp(tmp)
      v202 = b205*v202 + b208

      v = (v201 - v202)**2 + chi20
      v = (v201+v202) - dsqrt(v)
      v = 0.5d0*v

      return
      end subroutine phenol_v1dv20


!#######################################################################

      subroutine phenol_v1dv21(v,r,imod)

      implicit none

      integer(long), intent(in) :: imod
      real(dop), intent(out) :: v
      real(dop), intent(in) :: r
      real(dop) :: angst,ev,tmp
      real(dop) :: b211,b212,b213,b214,b215,b216,b217,chi21
      real(dop) :: v211,v212

      ev = 27.2114d0
      angst = 0.52917725d0
      if (imod .eq. 1) then
         b211 = -0.2902/ev
         b212 = 2.05715/angst
         b213 = 1.01574/angst
         b214 = -73.329/ev
         b215 = 1.48285/angst
         b216 = -0.1111/angst
         b217 = -0.00055/ev
         chi21 = 0.021105/(ev*ev)
      endif

      tmp = (r - b212)/b213
      v211 = 1.0d0 - tanh(tmp)
      v211 = 0.5d0*b211*v211

      tmp = (r - b215)/b216
      v212 = 1.0d0 - tanh(tmp)
      v212 = 0.5d0*b214*v212 + b217

      v = (v211 - v212)**2 + chi21
      v = (v211 + v212) + dsqrt(v)
      v = 0.5d0*v

      return
      end subroutine phenol_v1dv21

!#######################################################################

      subroutine phenol_v1dv22(v,r,imod)

      implicit none

      integer(long), intent(in) :: imod
      real(dop), intent(out) :: v
      real(dop), intent(in) :: r
      real(dop) :: angst,ev,tmp
      real(dop) :: b221,b222,b223,b224,b225,b226
      real(dop) :: v221,v222

      ev = 27.2114d0
      angst = 0.52917725d0
      if (imod .eq. 1) then
         b221 = 27.3756/ev
         b222 = 1.66881/angst
         b223 = 0.20557/angst
         b224 = 0.35567/angst
         b225 = 1.43492/ev
         b226 = 0.56968/angst
      endif

      tmp = (r - b222)/b223
      v221 = 1.0d0 + tanh(tmp)
      v221 = 0.5d0*b221*v221

      tmp = (r - b225)/b226
      v222 = 1.0d0 - tanh(tmp)
      v222 = 0.5d0*b224*v222 

      v = (v221 - v222)**2 
      v = (v221 + v222) - dsqrt(v)
      v = 0.5d0*v

      return
      end subroutine phenol_v1dv22

!#######################################################################

      subroutine pyrolle_v1dw2(v,r,imod)

      implicit none

      integer(long), intent(in) :: imod
      real(dop), intent(out) :: v
      real(dop), intent(in) :: r
      real(dop) :: ev
      real(dop) :: b21,b22,b23,b24
      real(dop) :: v21,v22

      ev = 27.2114d0
      if (imod .eq. 1) then
         b21 = 3.819/ev
         b22 = -1.219/ev
         b23 = 2.335/ev
         b24 = 0.226/ev
      endif

      v21 = b21 + b22*r
      v22 = (b23 + b22*r)**2 + 4*b24**2

      v = 0.5*v21 - 0.5*dsqrt(v22)

      return
      end subroutine pyrolle_v1dw2

      end module func1dmod

