!***********************************************************************
! 
! ifile = 7
! 
!***********************************************************************
! 
! Contains operators for combined KLegs (4D modes):
! cjpm (that's all for now)
!
! 06/2006 FO
!
!***********************************************************************
      module func2klegmod

use decimal, only: long,dop
use logdat

      implicit none
      private
      public :: fdef2kleg,func2kleg
      contains

!***********************************************************************
!
! FDEF2KLEG: assigns function number and file number
!
!***********************************************************************

      subroutine fdef2kleg(hopilab,hoplab,hopfile)

      implicit none

      integer(long), intent(out)                    :: hopfile,hopilab
      character(len=*), intent(in)                  :: hoplab

!     assign function number
      if (hoplab .eq. 'cjpm') then
         hopilab = 1

      endif

!     if label is assigned, assign hopfile no.
      if (hopilab .ne. 0) hopfile=7

      return
      end subroutine fdef2kleg ! of fdef2kleg


!***********************************************************************
!
! FUNC2KLEG: hub for generating the matrix representations of
!            the operators
!
!***********************************************************************

      subroutine func2kleg(hmat,hmatdim,ifunc,jtot, &
                           gdim1,kdim1,kmin1,dif1mat1, &
                           gdim2,kdim2,kmin2,dif1mat2)

use global

      implicit none

      integer(long)                                         :: z1,z2,z3
      integer(long), intent(in)                             :: hmatdim,gdim1,kdim1,kmin1,gdim2,kdim2,kmin2
      integer(long), intent(in)                             :: ifunc
      real(dop), intent(in)                                 :: jtot
      real(dop), dimension(hmatdim), intent(out)            :: hmat
      real(dop), dimension(gdim1,gdim1,2*kdim1), intent(in) :: dif1mat1
      real(dop), dimension(gdim2,gdim2,2*kdim2), intent(in) :: dif1mat2

     select case (ifunc)

     case default
      routine = 'func2kleg'
      write (message,'(a,i3)') 'Unknown ifunc = ',ifunc
      call errormsg

     case (1)!     cjpm
      z1 = 1
      z2 = z1 + kdim1+kdim2
      z3 = z2 + gdim1*gdim1*kdim1
      call cjpm(jtot,dif1mat1,dif1mat2,hmat(z1),hmat(z2),hmat(z3), &
                gdim1,kdim1,kmin1,gdim2,kdim2,kmin2)
      return

     end select

      end subroutine func2kleg ! of func2kleg


!***********************************************************************
!
! CJPM: assigns C+(J,K), and J+ for both KLeg DOFs
!
!***********************************************************************

      subroutine cjpm(jtot,dif1mat1,dif1mat2,cpl,jpl1,jpl2, &
                      gdim1,kdim1,kmin1,gdim2,kdim2,kmin2)

use funcklegmod, only: jpl

      implicit none

      integer(long)                                        :: i,k
      integer(long), intent(in)                            :: gdim1,kdim1,kmin1,gdim2,kdim2,kmin2
      real(dop)                                            :: tmp
      real(dop), intent(in)                                :: jtot
      real(dop), dimension(gdim1,gdim1,kdim1), intent(in)  :: dif1mat1
      real(dop), dimension(gdim2,gdim2,kdim2), intent(in)  :: dif1mat2
      real(dop), dimension(kdim1+kdim2), intent(out)       :: cpl
      real(dop), dimension(gdim1,gdim1,kdim1), intent(out) :: jpl1
      real(dop), dimension(gdim2,gdim2,kdim2), intent(out) :: jpl2

!     calculate C+
      do i=1,kdim1+kdim2
         k = kmin1+kmin2+i-2
         tmp = jtot*(jtot+1) - k*(k+1)
         if (tmp .lt. 0.0_dop) then
            tmp = 0.0_dop
         else
            tmp = sqrt(tmp)
         endif
         cpl(i)=tmp
      enddo

!     calculate j1+ and j2+
      call jpl(dif1mat1,jpl1,gdim1,kdim1)
      call jpl(dif1mat2,jpl2,gdim2,kdim2)

      return
      end subroutine cjpm

      end module func2klegmod
