!#######################################################################
! New version (Benjamin, December 2008)
!#######################################################################
      module funcanldmod

use decimal, only: long,dop
use global
use linintmod

use logdat
use channels, only: ilog
use linint, only:li_browse,linint1d

      implicit none
      private
      public :: fdefanld,funcanld
      contains

!#######################################################################
!
! ANALYTIC FUNCTIONS  
! Real 1D functions with parameters. See also func1d.F.
!
! ifile = 302
!
!#######################################################################

      subroutine fdefanld(ifunc,olabel,ifile,hopipar,hoprpar,maxhpar, &
                          hoppar,zpesopts,pesopts)

      implicit none

      integer(long)                                       :: n,nopt,ilbl,jlbl,np,order
!      integer(long)                                       :: li_browse
      integer(long), dimension(2*maxhpar,*), intent(in) :: zpesopts
      integer(long), intent(in)                           :: maxhpar
      integer(long), intent(out)                          :: ifunc,ifile
      integer(long), dimension(maxhpar), intent(out)      :: hopipar
      real(dop), dimension(maxhpar), intent(out)          :: hoprpar
      real(dop), dimension(maxhpar,*), intent(in)         :: hoppar
      character(len=64)                                   :: label
      character(len=64), intent(in)                       :: olabel
      character(len=c5), dimension(*), intent(in)         :: pesopts

! ___ functions may contain curly {} brackets too, so we need these (nopt, li_browse,zpesopts,pesopts)

!-----------------------------------------------------------------------
! remove option string from beginning of label
!-----------------------------------------------------------------------
      ilbl=index(olabel,'@')+1
      jlbl=index(olabel,' ')-1
      label=olabel(ilbl:jlbl)

!-----------------------------------------------------------------------
! get argument list no.
!-----------------------------------------------------------------------
      np=0
      ilbl=index(label,'[')+1
      if (ilbl .gt. 1) then
         jlbl = index(label,']')-1
         read(label(ilbl:jlbl),*) np
      endif

!-----------------------------------------------------------------------
! get end of operator label
!-----------------------------------------------------------------------
      ilbl=index(label,'[')-1
      if (ilbl .lt. 0) ilbl=index(label,' ')-1

!-----------------------------------------------------------------------
! assign function number
! Note: hopipar(1) and hoprpar(1) are reserved for the exponent.
!-----------------------------------------------------------------------
! 1 (unit operator)
      if (label .eq. '1'    .or.  label .eq. '1.'    .or.&
          label .eq. '1.0'  .or.  label .eq. '1.00'  .or.&
          label .eq. '1d0'  .or.  label .eq. '1.d0') then
         ifunc=1

! -1
      else if (label .eq. '-1'    .or.  label .eq. '-1.'    .or.&
               label .eq. '-1.0'  .or.  label .eq. '-1.00'  .or.&
               label .eq. '-1d0'  .or.  label .eq. '-1.d0') then
         ifunc=2

! q:  (q-a)^n
      else if (label(1:ilbl) .eq. 'q') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
         else
            hoprpar(2)=0.0_dop
         endif
         ifunc=3

! sin:  sin(R(x-a))^n
      else if (label(1:ilbl) .eq. 'sin') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            ifunc=5
         else
            ifunc=4
         endif

! cos:  cos(R(x-a))^n
      else if (label(1:ilbl) .eq. 'cos') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            ifunc=7
         else
            ifunc=6
         endif

! tan:  tan(R(x-a))^n
      else if (label(1:ilbl) .eq. 'tan') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            ifunc=9
         else
            ifunc=8
         endif

! legendre
      else if (label(1:5) .eq. 'legth') then
         order=0
         ilbl=index(label,':')+1
         jlbl=index(label,' ')-1
         if (ilbl .ne. 1) read(label(ilbl:jlbl),*) order
         hopipar(2)=order
         ifunc=10

! exp:  exp(a*(x-x0))
      else if (label(1:ilbl) .eq. 'exp') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=11

! gauss:  exp(-a*(x-x0)^2) 
      else if (label(1:ilbl) .eq. 'gauss') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=12

! exp1:  [ 1-exp(a(x-b)) ]^r
      else if (label(1:ilbl) .eq. 'exp1') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=13

! expcos:  [ exp(a*cos(x)) - exp(a*cos(b)) ]^r
      else if (label(1:ilbl) .eq. 'expcos') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=14

! expcos1: exp(a*(cos(q)-b))^r
      else if(label(1:ilbl) .eq. 'expcos1') then
         if (np .gt.0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=15


! cos1: (cos(a*x) - cos(a*b))^r
      else if (label(1:ilbl) .eq. 'cos1') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=16


! c_p:  sqrt(J(J+1)-K(K+1))
      else if (label(1:ilbl) .eq. 'c_p') then
           do n=1,npar
              if (apar(n) .eq. 'jtot') then
                 hoprpar(2)=rpar(n)
              endif
           enddo
           ifunc=17

! c_m:  sqrt(J(J+1)-K(K-1))
      else if (label(1:ilbl) .eq. 'c_m') then
           do n=1,npar
              if (apar(n) .eq. 'jtot') then
                 hoprpar(2)=rpar(n)
              endif
           enddo
           ifunc=18

! qs:  sqrt(a-x^2)^n
      else if (label(1:ilbl) .eq. 'qs') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
         else
            hoprpar(2)=1.0_dop
         endif
         ifunc=19


! asin:  (asin(a*x-b)-c))^n
      else if (label(1:ilbl) .eq. 'asin') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            hoprpar(4)=hoppar(3,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
            hoprpar(4)=0.0_dop
         endif
         ifunc=20

! acos:  (acos(a*x-b)-c)^n
      else if (label(1:ilbl) .eq. 'acos') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            hoprpar(4)=hoppar(3,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
            hoprpar(4)=0.0_dop
         endif
         ifunc=21

! atan:  (atan(a*x-b)-c)^n
      else if (label(1:ilbl) .eq. 'atan') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            hoprpar(4)=hoppar(3,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
            hoprpar(4)=0.0_dop
         endif
         ifunc=22

! step :  Theta(x-p); Heaviside's step function.
      else if (label(1:ilbl) .eq. 'step') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
         else
            hoprpar(2)=0.0_dop
         endif
         ifunc=23

! rstep : 1-Theta(x-p) = Theta(p-x) ; Heaviside's step function.
      else if (label(1:ilbl) .eq. 'rstep') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
         else
            hoprpar(2)=0.0_dop
         endif
         ifunc=24

! tgauss:  exp(-a*((acos(x)-b)**2))
      else if (label(1:ilbl) .eq. 'tgauss') then
         if (np.gt.0) then
            hoprpar(2) = hoppar(1,np)
            hoprpar(3) = hoppar(2,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=25

! motanh:  tanh(c*(1-exp(-a(x-b))**n)
      else if (label(1:ilbl) .eq. 'motanh') then
         if (np .gt.0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            hoprpar(4)=hoppar(3,np)
            hoprpar(5)=hoppar(4,np)
         endif
         ifunc=26

! qtanh:  tanh(b*(x-a)^c)
      else if (label(1:ilbl) .eq. 'qtanh') then
         if (np .gt.0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            hoprpar(4)=hoppar(3,np)
         endif
         ifunc=27

! cosh(a*(x-b))
      else if (label(1:ilbl) .eq. 'cosh') then
         if (np .gt.0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=28

! sinh(a*(x-b))
      else if (label(1:ilbl) .eq. 'sinh') then
         if (np .gt.0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=29

! tanh(a*(x-b))
      else if (label(1:ilbl) .eq. 'tanh') then
         if (np .gt.0) then
             hoprpar(2)=hoppar(1,np)
             hoprpar(3)=hoppar(2,np)
         else
             hoprpar(2)=1.0_dop
             hoprpar(3)=0.0_dop
         endif
         ifunc=30

! switch1: 0.5*(1-tanh(a*(x-b)))
      else if (label(1:ilbl) .eq. 'switch1') then
         if (np .gt.0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
             hoprpar(2)=1.0_dop
             hoprpar(3)=0.0_dop
         endif
         ifunc=31


! switch2: 0.5*(1+tanh(a*(x-b)))
      else if (label(1:ilbl) .eq. 'switch2') then
         if (np .gt.0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
             hoprpar(2)=1.0_dop
             hoprpar(3)=0.0_dop
         endif
         ifunc=32

!-----------------------------------------------------------------------
! external1d: linear interpolation of 1D function from input file
! calls function linint1d in module linint.F
! This is useful if we want to use an externally produced electric field
! e.g. an optimal control field. --  Andreas Markmann
!-----------------------------------------------------------------------
      else if (label(1:10) .eq. 'external1d') then
         ilbl=index(label,'{')+1
         if (ilbl .gt. 1) then
            jlbl = index(label,'}')-1
         endif
         read(label(ilbl:jlbl),*) np
         jlbl = 0
         nopt = 0
         do while(zpesopts(nopt+1,np).gt.0)
            nopt = nopt + 1
            ilbl = jlbl + 1
            jlbl = zpesopts(nopt,np)
         end do
         if(ilbl.gt.jlbl .or. nopt.ne.1) then
            routine = 'fdefanld'
            message = 'use function external1d with filename, '// &
              'i.e. "external1d{<filename>}"'
            call errormsg
         endif
         hopipar(2) = li_browse(pesopts(np)(ilbl:jlbl))
         ifunc=33

         call cpextfile(pesopts(np)(ilbl:jlbl),jlbl-ilbl+1)

! Box :  Double sided step function.
      else if (label(1:ilbl) .eq. 'box') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(2,np) + hoppar(1,np)
            hoprpar(3)=hoppar(2,np) - hoppar(1,np)
         else
            hoprpar(2)=0.5_dop
            hoprpar(3)=-0.5_dop
         endif
         ifunc=34

! associated Legendre polynomial P_l^m (x)
      else if (label(1:5) .eq. 'asleg') then
         ilbl=index(label,':')+1
         jlbl=index(label,'_')-1
         read(label(ilbl:jlbl),*) hopipar(2)
         ilbl=jlbl+2
         jlbl=index(label,' ')-1
         read(label(ilbl:jlbl),*) hopipar(3)

         ifunc=35
         if (label(1:7) .eq. 'aslegth')  ifunc=36

! chirped cosine function coschir cos(x(w2+(w1-w2)exp(-(x/tau)^2)))
      else if (label(1:ilbl) .eq. 'coschirp') then
         hoprpar(2)=hoppar(1,np)
         hoprpar(3)=hoppar(2,np)
         hoprpar(4)=hoppar(3,np)

         ifunc=37

! absin:  absin(R(x-a))^n
      else if (label(1:ilbl) .eq. 'absin') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            ifunc=39
         else
            ifunc=38
         endif

! sincos: (sin(a*(x-b)) * cos(c*(x-d)))^n
      else if (label(1:ilbl) .eq. 'sincos') then
         if (np .gt.0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            hoprpar(4)=hoppar(3,np)
            hoprpar(5)=hoppar(4,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
            hoprpar(4)=1.0_dop
            hoprpar(5)=0.0_dop
         endif
         ifunc=40

! usincos: (sqrt(1-x^2) * x)^n
      else if (label(1:ilbl) .eq. 'usincos') then
         if (np .gt.0) then
            hoprpar(2)=hoppar(1,np)
         else
            hoprpar(2)=1.0_dop
         endif
         ifunc=41

! ngauss:  exp(-x^2/(2w^2))/SQRT(2 pi w^2)  //normalized Gaussian
      else if (label(1:ilbl) .eq. 'ngauss') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=42

! charfun: characteristic function for the interval [l,r]
      else if (label(1:ilbl) .eq. 'charfun') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
            hoprpar(2)=0.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=43

! regcoul: regularized 1D coulomb potential: V=1/sqrt((x-a)**2 +b) 
      else if (label(1:ilbl) .eq. 'regcoul') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
            hoprpar(2)=0.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=44

! exp0:  E_inf [ exp(-ax)) -1 ]^r
      else if (label(1:ilbl) .eq. 'exp0') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            hoprpar(4)=hoppar(3,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=1.0_dop
            hoprpar(4)=0.0_dop
         endif
         ifunc=45

! exp2:  [ 1+exp(a(x-b)) ]^r
      else if (label(1:ilbl) .eq. 'exp2') then
         if (np .gt. 0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
         else
            hoprpar(2)=1.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=46

! texp:  exp(a*(acos(x)-b))
      else if (label(1:ilbl) .eq. 'texp') then
         if (np.gt.0) then
            hoprpar(2) = hoppar(1,np)
            hoprpar(3) = hoppar(2,np)
         else
            hoprpar(2)=1.0d0
            hoprpar(3)=0.0d0
         endif
         ifunc=47

! vecpot: p1*cos(p2*x-p3*pi)*sin^2(omega*t/(2*Nc))
      else if (label(1:ilbl) .eq. 'vecpot') then
         if (np.gt.0) then
            hoprpar(2) = hoppar(1,np)
            hoprpar(3) = hoppar(2,np)
            hoprpar(4) = hoppar(3,np) 
         else
            hoprpar(2) = 0.0d0
            hoprpar(3) = 0.0d0
            hoprpar(4) = 1.0d0
         endif
         ifunc=48

!-----------------------------------------------------------------------
! end of if loop
!-----------------------------------------------------------------------
      endif

!-----------------------------------------------------------------------
! if label is assigned, assign ifile no.
!-----------------------------------------------------------------------
      if (ifunc .ne. 0) ifile=302

      return
      end subroutine fdefanld


!#######################################################################

      subroutine funcanld(v,x,ifunc,hopipar,hoprpar)

      implicit none

      integer(long)                           :: order
      integer(long), intent(in)               :: ifunc
      integer(long), dimension(*), intent(in) :: hopipar
      real(dop)                               :: r,jtot
!      real(dop), external                     :: dlegendre,plgndr,linint1d
      real(dop), external                     :: dlegendre,plgndr
      real(dop), intent(in)                   :: x
      real(dop), intent(out)                  :: v
      real(dop), dimension(*), intent(in)     :: hoprpar

      ifunc1: select case (ifunc)

      case default
      routine='Funcanld'
      write(message,'(a,i5)') 'Function not found :',ifunc
      call errormsg

      case(1) ! 1:  unit
      v=1.0_dop
      return

      case(2) ! -1:  -unit
      v=-1.0_dop
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(3) ! q:  (x-a)
      v=x-hoprpar(2)
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(4) ! sin:  sin(p(x-x0))
      v=sin(x)
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(5) 
      v=sin(hoprpar(2)*(x-hoprpar(3)))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(6) ! cos:  cos(p(x-x0))
      v=cos(x)
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(7) 
      v=cos(hoprpar(2)*(x-hoprpar(3)))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(8) ! tan:  tan(p(x-xo))
      v=tan(x)
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(9) 
      v=tan(hoprpar(2)*(x-hoprpar(3)))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(10) ! legth: legendre(order,x)
      order=hopipar(2)
      v=dlegendre(order,x)
      return

      case(11) ! exp:  exp(a*(x-x0))
      v=exp(hoprpar(2)*(x-hoprpar(3)))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(12) ! gauss: exp(-a*(x-x0)^2)
      r=x-hoprpar(3)
      call vexponent(r,hoprpar(1),hopipar(1))
      v=exp(-hoprpar(2)*r*r)
! prevent underflow
      if (v .lt. 1.0d-15) v = 0.0d0
      return

      case(13) ! exp1: 1-exp(a(x-b))
      v=1.0_dop-exp(hoprpar(2)*(x-hoprpar(3)))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(14) ! expcos:  exp(a*cos(x)) - exp(a*cos(b))
      v=exp(hoprpar(2)*cos(x)) - exp(hoprpar(2)*cos(hoprpar(3)))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(15) ! expcos1:  exp(a*(cos(x)-b))
      v=exp(hoprpar(2)*(cos(x)-hoprpar(3)))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(16) ! cos1:  (cos(a*x) - cos(a*b))^n
      v=cos(hoprpar(2)*x)-cos(hoprpar(2)*hoprpar(3))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(17) ! c_p:  sqrt(J(J+1)-K(K+1))
      jtot=hoprpar(2)
      if(x.lt.jtot .and. -x.lt.jtot+1.0_dop) then
         v=sqrt(jtot*(jtot+1.0_dop)-x*(x+1.0_dop))
      else
         v=0.0_dop
      endif
      return

      case(18) ! c_m:  sqrt(J(J+1)-K(K-1))
      jtot=hoprpar(2)
      if(x.lt.jtot+1 .and. -x.lt.jtot) then
         v=sqrt(jtot*(jtot+1)-x*(x-1))
      else
         v=0.0_dop
      endif
      return

      case(19) ! qs:  sqrt(a-x^2)^n 
      v=sqrt(hoprpar(2)-x**2)
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(20) ! asin: (asin(a*x-b)-c)
      v=asin(hoprpar(2)*x-hoprpar(3))-hoprpar(4)
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(21) ! acos: (acos(a*x-b)-c) 
      v=acos(hoprpar(2)*x-hoprpar(3))-hoprpar(4)
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(22) ! atan: (atan(a*x-b)-c)
      v=atan(hoprpar(2)*x-hoprpar(3))-hoprpar(4)
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(23) ! step:  Theta(x-p); Heaviside's step function.
      if( x .ge. hoprpar(2) ) then
         v=1.0_dop
      else
         v=0.0_dop
      endif
      return

      case(24) ! rstep:  1-Theta(x-p) = Theta(p-x) ; Heaviside's step function.
      if( x .ge. hoprpar(2) ) then
         v=0.0_dop
      else
         v=1.0_dop
      endif
      return

      case(25) ! tgauss:  exp(-a*((acos(x)-b)**2))
      v=exp(-hoprpar(2)*((acos(x)-hoprpar(3))**2))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(26) ! motanh:  tanh(c*[1-exp(-a(x-b))]**d)^r
      v=tanh( hoprpar(4)*&
              ( 1.0_dop - exp( -hoprpar(2) * ( x - hoprpar(3) ) )&
              )**hoprpar(5) )
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(27) ! qtanh:  tanh(b*(acos(x)-a)**c)^r 
      v=tanh( hoprpar(3) * (acos(x)-hoprpar(2))**hoprpar(4) )
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(28) ! cosh:  cosh(a*(x-b))
      v=cosh( hoprpar(2)*(x-hoprpar(3)) )
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(29) ! sinh:  sinh(a*(x-b))
      v=sinh( hoprpar(2)*(x-hoprpar(3)) )
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(30) ! tanh:  tanh(a*(x-b))
      v=tanh( hoprpar(2)*(x-hoprpar(3)) )
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(31) ! switch1:  0.5*(1-tanh[a*(x-b)])
      v=0.5_dop * ( 1.0_dop - tanh( hoprpar(2)*(x-hoprpar(3)) ) )
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(32) ! switch2:  0.5*(1+tanh[a*(x-b)])
      v=0.5_dop * ( 1.0_dop + tanh( hoprpar(2)*(x-hoprpar(3)) ) )
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(33) ! li1d:  linear interpolation of 1D function from input file (linint1d)
      v=linint1d(x,hopipar(2))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(34) ! Box: 1 inside bounds, 0 outside
      if( x .le. hoprpar(2) .and. x .ge. hoprpar(3)) then
         v=1.0_dop
      else
         v=0.0_dop
      endif
      return

      case(35) ! associated Legendre polynomial P_l^m (x)
      v=plgndr(hopipar(2),hopipar(3),x)
      return

      case(36) ! associated Legendre polynomial P_l^m (cos(x))
      v=plgndr(hopipar(2),hopipar(3),cos(x))
      return

      case(37) ! coschirp: chirped cosine function cos(x[w2+(w1-w2)exp(-(x/tau)^2)])
      v=cos( ( hoprpar(3) + &
              (hoprpar(2)-hoprpar(3))*exp(-(x/hoprpar(4))**2) ) * x )
      return

      case(38) ! absin:  absin(p(x-x0))
      v=dabs(sin(x))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(39) 
      v=dabs(sin(hoprpar(2)*(x-hoprpar(3))))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(40) ! sin(a*(x-b))*cos(c*(x-d))
      v=sin(hoprpar(2)*(x-hoprpar(3)))*cos(hoprpar(4)*(x-hoprpar(5)))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(41) ! sqrt(p-x^2) * x
      v=sqrt(hoprpar(2)-x**2)*x
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(42) ! ngauss: exp(-x^2/(2w^2))/SQRT(2 pi w^2)
      v = exp(-(x-hoprpar(3))**2/(2.0_dop*hoprpar(2)**2))
      v = v / (SQRT(2.0_dop*PI)*hoprpar(2))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(43) ! charfunc: characteristic function for the interval [l,r]
      if ( x .ge. hoprpar(2) ) then
        if (x .le. hoprpar(3) ) then
          v=1.0_dop
        else
          v=0.0_dop
        endif
      else
          v=0.0_dop
      endif
      return

      case(44) ! regcoul: regularized 1D coulomb potential: V=1/sqrt((x-a)**2 +b) 
      v=1.0_dop/sqrt( (x-hoprpar(2))**2 + hoprpar(3) )
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(45) ! exp0: a(exp(-b(x-c)) - 1)
      v=exp(-hoprpar(3)*(x-hoprpar(4)))-1.0_dop
      v = hoprpar(2)*v
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(46) ! exp2:  1+exp(a*(x-x0))
      v=1.0_dop+exp(hoprpar(2)*(x-hoprpar(3)))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case(47) ! texp:  exp(a*(acos(x)-b))
      v=exp(hoprpar(2)*(acos(x)-hoprpar(3)))
      call vexponent(v,hoprpar(1),hopipar(1))
      return

      case (48) ! vecpot: p1*cos(p2*x-p3*pi)*sin^2(omega*t/(2*Nc))
      v = hoprpar(2)*cos(hoprpar(3)*x-hoprpar(4)*pi)* &
          sin(hoprpar(3)*x/(2.0d0*hoprpar(4)))**2
      return

      end select ifunc1

      end subroutine funcanld

!#######################################################################
! Apply exponent.
!#######################################################################

      subroutine vexponent(vx,ehoprpar,ehopipar)

      implicit none

      integer(long), intent(in) :: ehopipar
      real(dop), intent(in)     :: ehoprpar
      real(dop),intent(inout)   :: vx

      if(ehopipar.eq.0) then
         vx=vx**ehoprpar
      elseif(ehopipar.ne.1) then
         vx=vx**ehopipar
      endif
      return
      end subroutine vexponent


!#######################################################################
! save copy of data to interpolate in name directory
!#######################################################################

      subroutine cpextfile(filename,slen)

      implicit none

      integer(long), intent(in)       :: slen
      integer(long)                   :: oldlen,newlen,ilbl
      character(len=slen), intent(in) :: filename
      character(len=240)              :: filenew,fileold
      character(len=480)              :: string
      logical                         :: lcheck=.false.

      write(filenew,'(3a)') name(1:laenge),'/',filename
      newlen=laenge+slen+1
      inquire(file=filenew,exist=lcheck)

      if (lcheck) return
      
      fileold=filename
      oldlen=slen
      call abspath(fileold,oldlen)
      
      string = 'cp '//fileold(1:oldlen)//' '//filenew(1:newlen)
      ilbl = oldlen+newlen+5
      call excmd(string(1:ilbl))

      write(ilog,'(a)') 'Saving External1d data in name directory '

      return
      end subroutine cpextfile
 

      end module funcanldmod

