!#######################################################################
      module funcanlzmod

use decimal, only: long,dop

      implicit none
      private
      public :: fdefanlz,funcanlz
      contains

!#######################################################################
!
! COMPLEX ANALYTIC FUNCTIONS. Only purely imaginary functions are to
! be stored here.
!
! ifile = 101
!
!#######################################################################

      subroutine fdefanlz(ifunc,label,ifile,ipar,rpar,maxpar,hoppar)

      implicit none

      integer(long)                                :: n,ilbl,jlbl,klbl,&
                                                      capend,iord 
      integer(long), intent(in)                    :: maxpar
      integer(long), intent(out)                   :: ifunc,ifile
      integer(long), dimension(maxpar),intent(out) :: ipar
      real(dop)                                    :: eta,zcap,capord
      real(dop), dimension(maxpar), intent(out)    :: rpar
      real(dop), dimension(maxpar,*), intent(in)   :: hoppar
      character(len=*), intent(in)                 :: label

!-----------------------------------------------------------------------
! now assign function number
!-----------------------------------------------------------------------
! I
      if (label .eq. 'I') then
         ifunc=1

      else if (label(1:2) .eq. 'I^') then
         jlbl=index(label,' ')-1
         ilbl=index(label(3:jlbl),'.')+index(label(3:jlbl),'d')
         if(ilbl.eq.0) then
            read(label(3:jlbl),*) ipar(1)
            ifunc=2
         else
            read(label(3:jlbl),*) rpar(1)
            ifunc=3
         endif

! CAP 
      else if (label(1:4) .eq. 'CAP[' .or. label(1:5) .eq. 'ACAP[') then
         ilbl=index(label,'[')+1
         jlbl=index(label,']')-1
         read(label(ilbl:jlbl),*) n
         zcap=hoppar(1,n)
         eta=hoppar(2,n)
         capord=hoppar(3,n)

         iord = nint(capord)
         if (abs(dble(iord)-capord) .lt. 1.0e-8_dop) then
            ipar(1)=iord
         else
            ipar(1)=0
         endif

         rpar(1)=zcap
         rpar(2)=eta
         rpar(3)=capord

         capend=nint(hoppar(4,n))
         if (capend .lt. 0) then
            ifunc=4
         else if (capend .ge. 0) then
            ifunc=5
         endif

         if(label(1:4).eq.'ACAP') call settimecap(hoppar(5,n),n)

! Exp(i*p*(x-x0))
      else if (label(1:4) .eq. 'Exp ' .or. label(1:4) .eq. 'Exp[' .or. &
               label(1:4) .eq. 'Exp^' ) then
         ilbl=index(label,'[')+1
         jlbl=index(label,']')-1
         if(jlbl.ge.ilbl) then
            read(label(ilbl:jlbl),*) n
            rpar(2)=hoppar(1,n)
            rpar(3)=hoppar(2,n)
         else
            rpar(2)=1.0_dop
            rpar(3)=0.0_dop
         endif
         ! look for power
         ipar(1)=0
         rpar(1)=0.0d0
         ilbl=index(label,'^')+1
         if (ilbl .gt. 1) then
            jlbl=index(label,'[')-1
            if (jlbl .lt. 0) jlbl=index(label,' ')-1
            klbl=index(label(ilbl:jlbl),'.')
            if (klbl .gt. 0) then
               read(label(ilbl:jlbl),*) rpar(1)
            else
               read(label(ilbl:jlbl),*) ipar(1)
            endif
         else
            ipar(1)=1
            rpar(1)=0.0d0
         endif

         ifunc=6

! Peierls
      else if (label(1:8) .eq. 'Peierls ' .or. &
               label(1:8) .eq. 'Peierls[' .or. &
               label(1:8) .eq. 'Peierls^' ) then
         ilbl=index(label,'[')+1
         jlbl=index(label,']')-1
         if(jlbl.ge.ilbl) then
            read(label(ilbl:jlbl),*) n
            rpar(2)=hoppar(1,n)
            rpar(3)=hoppar(2,n)
            rpar(4)=hoppar(3,n)
         else
            rpar(2)=0.d0
            rpar(3)=0.d0
            rpar(4)=1.d0
         endif
         ifunc=7

!-----------------------------------------------------------------------
! end of if loop
!-----------------------------------------------------------------------
      endif

!-----------------------------------------------------------------------
! if label is assigned, assign ifile no.
!-----------------------------------------------------------------------
      if (ifunc .ne. 0) ifile=101

      return
      end subroutine fdefanlz

!#######################################################################

      subroutine funcanlz(v,x,ifunc,ipar,rpar)

      implicit none

      integer(long)                           :: iord
      integer(long), intent(in)               :: ifunc
      integer(long), dimension(*), intent(in) :: ipar
      real(dop)                               :: q,eta,zcap,capord,pi
      real(dop), intent(in)                   :: x
      real(dop), dimension(*), intent(in)     :: rpar
      complex(dop), intent(out)               :: v

      select case (ifunc)

      case (1) ! I (imaginary unit operator)
      v=(0.0_dop,1.0_dop)
      return

      case (2) ! I^int (imaginary unit operator)
      v=(0.0_dop,1.0_dop)**ipar(1)
      return

      case (3) ! I^real (imaginary unit operator)
      v=(0.0_dop,1.0_dop)**rpar(1)
      return

      case (4) ! CAP at LHS
      zcap=rpar(1)
      eta=rpar(2)
      capord=rpar(3)
      iord=ipar(1)
      if (x .ge. zcap) then
         v=0.0_dop
      else
         q=zcap-x
         if (iord .ne. 0) then
            q=q**iord
         else
            q=q**capord
         endif
         v = dcmplx(0.0_dop,-eta*q)
      endif
      return

      case (5) ! CAP at RHS
      zcap=rpar(1)
      eta=rpar(2)
      capord=rpar(3)
      iord=ipar(1)
      if (x .le. zcap) then
         v=0.0_dop
      else
         q=x-zcap
         if (iord .ne. 0) then
            q=q**iord
         else
            q=q**capord
         endif
         v = dcmplx(0.0_dop,-eta*q)
      endif
      return

      case (6) ! Exp: exp(i*a*(x-b)) 
      q=rpar(2)*(x-rpar(3))
      v=exp(dcmplx(0.0_dop,q))
      if(ipar(1).eq.0) then
         v=v**rpar(1)
      elseif(ipar(1).ne.1) then
         v=v**ipar(1)
      endif
      return

      case (7) ! Peierls: exp(i*p1*cos(omega*x-Nc*pi)*sin^2(omega*t/(2*Nc)))
      pi = 4.d0*datan(1.d0)
      v=exp(dcmplx(0.d0,rpar(2)*cos(rpar(3)*x-rpar(4)*pi)* &
                        sin(rpar(3)*x/(2.0d0*rpar(4)))**2))
!     if(ipar(1).eq.0) then
!        v=v**rpar(1)
!     elseif(ipar(1).ne.1) then
!        v=v**ipar(1)
!     endif
      return

      end select

      end subroutine funcanlz

!#######################################################################
!     sets the time for ACAP (5th parameter)

      subroutine settimecap(hoppar5,np)

use operdef, only:timecap
use channels, only: ilog

      implicit none

      integer(long), save       :: n = 0 
      integer(long), intent(in) :: np
      real(dop), intent(in)     :: hoppar5

      n = n + 1
      timecap(n) = hoppar5
      write(ilog,*) 'timecap : time=',hoppar5,',  n =',n,', np =',np

      return
      end subroutine settimecap

      end module funcanlzmod
