!#######################################################################
!            Function-DEF for operators on expectation values
!
! The funcexp routines are to be found in mctdhlib/tdkoeffs.F. They
! are called only during propagation from calctdk and rely on 
! routines in mctdhlib
! ifile = 309
! Modified from funclct. GWR 01/14
!#######################################################################

      subroutine fdefexp(ifunc,label,ifile,hopipar,hoprpar,maxhpar,&
                 zpesopts,pesopts,hamlab,hamdim)

use decimal, only:long,dop
use global
use logdat

      implicit none

      integer(long)                                     :: ilbl,jlbl,np,n,nopt,nham,klbl,&
                                                           llbl
      integer(long), intent(in)                         :: maxhpar,hamdim
      integer(long), intent(out)                        :: ifunc,ifile
      integer(long), dimension(*), intent(out)          :: hopipar
      integer(long), dimension(2*maxhpar,*), intent(in) :: zpesopts
      real(dop)                                         :: steppos
      real(dop), dimension(*), intent(out)              :: hoprpar
      character(len=20)                                 :: expopts
      character(len=*), intent(in)                      :: label
      character(len=*), dimension(*), intent(in)        :: pesopts,hamlab

! get argument list no.
! and end of operator label
      np=0
      ilbl=-1
      ilbl=index(label,'[')+1
      if (ilbl .gt. 1) then
         jlbl = index(label,']')-1
         read(label(ilbl:jlbl),*) np
         ilbl=index(label,'[')-1
      else
         ilbl=index(label,'{')+1
         if (ilbl .gt. 1) then
            jlbl = index(label,'}')-1
            read(label(ilbl:jlbl),*) np
            ilbl=index(label,'{')-1
         endif
      endif

      if (ilbl .lt. 0) ilbl=index(label,' ')-1


! now assign function number
! Note: hopipar(1) and hoprpar(1) are reserved for the exponent.
! The exponent was read in defanld (opfuncs/callanld.f).

      if (label(1:ilbl) .eq. 'expectstep') then
         if (np .le.0) then
            routine = 'Funcexp'
            message = 'No operator specified with expectstep operator'
            call errormsg
         endif

! read operator label
         nopt=1
         klbl=1
         llbl=zpesopts(nopt,np)
         expopts = pesopts(np)(klbl:llbl)
         nham = 0
         do n=1,hamdim
            if (expopts .eq. hamlab(n)) nham=n
         enddo
         if (nham .eq. 0) then
            routine = 'Funcexp'
            write(message,'(2a)')&
            'Operator specified with expectstep operator not found',expopts
            call errormsg
         endif
! read position of step i.e. 'c' in \theta(<x>-c)
         nopt=2
         klbl=llbl+1
         llbl=zpesopts(nopt,np)
         if (llbl .eq. 0) then
            routine = 'Funcexp'
            message =&
            'Position of step not specified for expectstep operator'
            call errormsg
         endif
         expopts = pesopts(np)(klbl:llbl)
         steppos = -999.0_dop
         do n=1,npar
            if (expopts .eq. apar(n)) steppos = rpar(n)
         enddo
         if (steppos .lt. -900.0_dop) then
            routine = 'Funcexp'
            write(message,'(2a)')&
            'Position of step in expectstep operator not found :',expopts
            call errormsg
         endif

         hopipar(2)=nham
         hoprpar(2)=steppos
!
! look to see if imaginary or real part
!
!         hopipar(3)=0
!         nopt=3
!         klbl=llbl+1
!         llbl=zpesopts(nopt,np)
!         if (llbl .ne. 0) then
!            if (pesopts(np)(klbl:klbl+1) .eq. 'im') then
!               hopipar(3)=1
!            else if (pesopts(np)(klbl:klbl+1) .eq. 're') then
!               hopipar(3)=0
!            else 
!               routine = 'Funcexp'
!               write(message,'(2a)')&
!                     'Unknown 3rd option for expectstep function'
!               call errormsg
!            endif
!         endif

         ifunc=1
      endif

! if label is assigned, assign ifile no.
      if (ifunc .ne. 0) ifile=309

      return
      end subroutine fdefexp


