      module funcgapmod

use decimal, only: long, dop
use global
use dvrdatmod
use dirdyn
use griddatmod
use gapmod
use operdef

use logdat
use timing
use op1lib, only: sum2vxd
use dd_db, only: ddq2x

      implicit none
      public :: potgap,ddmgapwgt,gapcovar,gapexpeval,ddgbtendec,&
           potgapss1,gapvarddm,acholupd,agapdiag,printvar,dcgapwgt,&
           ddmsocwgt,get_nconfig,gap_diab_wf

      contains

!#######################################################################
! Subroutine to diabatise an intial wavepacket in the exact method.
! For use with read-only calculations where there is an existing DB
! in the diabatic representation.
! Calculates the diabatic energies/couplings at each gridpoint,
! diagonalises the matrix to get the adiabatic-diabatic transformation
! matrix which is then used to transform the wavefunction from the 
! adiabatic representation to the diabatic.
! GWR 03/20
!#######################################################################  
      subroutine gap_diab_wf(psi,ort)

use psidef

      implicit none

      complex(dop), dimension(dgldim), intent(inout) :: psi
      real(dop), dimension(ortdim), intent(in)       :: ort

      integer(long) :: f,n,nf,p,gpnt,mgpnt,modus,ierr,s,s1,maxidx,p1
      integer(long) :: compdim
      integer(long), dimension(maxdim) :: grid
      real(dop) :: maxcom
      real(dop), dimension(maxdim)     :: gpoint,gpoint0,gpoint1
      real(dop), dimension(nddstate)   :: av
      real(dop), dimension(:,:,:), allocatable :: v
      real(dop), dimension(nddstate,nddstate) :: sover
      real(dop), dimension(nddstate,nddstate,subdim(1)) :: rv
      real(dop), dimension(3*nddstate) :: work
      complex(dop), dimension(nddstate,nddstate) :: cv
      complex(dop), dimension(2*nddstate) :: cwork
      complex(dop), dimension(nddstate) :: tmpadpsi,tmppsi

      integer(long) :: mtmp,mp,pp
      integer(long), dimension(subdim(1))    :: map,proxidx
      real(dop) :: dtmp,mindist,currdist,distsqdiff
      real(dop), dimension(subdim(1)) :: distsq

      if (lddsoc .and. .not. lsocdegen) then
         compdim = 2
      else
         compdim = 1
      endif

      allocate(v(compdim,nddstate,nddstate))

! Set up the coordinates.
      grid = 0
      nf = nspfdof(1)
      do n = 1,nf
         grid(n) = spfdof(n,1)
      enddo

! Coordinates of inactive modes.
      do f = 1,ndof
         if(basis(f).eq.19)then
            gpoint(f) = ort(zort(f))
            gpoint1(f) = ort(zort(f))
         else
            gpoint(f) = 0.0_dop
            gpoint1(f) = 0.0_dop
         endif
      enddo

! Loop over all gridpoints to get distance from centre.
      gpoint0 = 0.0_dop
      modus = 0
      distsq = 0.0_dop
      do p = 1,subdim(1)
! Get coordinates of the current gridpoint.
         gpnt=p-1
         do n=1,nf
            f=grid(n)
            mgpnt=gpnt
            gpnt=gpnt/gdim(f)
            mgpnt=mgpnt-gpnt*gdim(f)
            gpoint(f)=ort(zort(f)+mgpnt)  
         enddo
         do f = 1,ndof
            distsq(p) = distsq(p) + (gpoint(f)-gpoint0(f))**2
         enddo
      enddo

! Initialise the map array.
      do p = 1,subdim(1)
         map(p) = p
      enddo

! Map the gridpoints so we know the order of their proximity to the reference,
! i.e. the order in which they will be diabatised.
      do p = 1,subdim(1)
         do p1 = p+1,subdim(1)
            if(distsq(p1) .lt. distsq(p))then
               dtmp = distsq(p)
               distsq(p) = distsq(p1)
               distsq(p1) = dtmp
               mtmp = map(p)
               map(p) = map(p1)
               map(p1) = mtmp
            endif
         enddo
      enddo

! Find the nearest other DB entry to each, which is closer to the reference; 
! this will be DB entry from which the diabatisation will occur.
      proxidx = 0
      do p = 2,subdim(1)
         mindist = 99999999_dop
         gpnt=map(p)-1
         do n=1,nf
            f=grid(n)
            mgpnt=gpnt
            gpnt=gpnt/gdim(f)
            mgpnt=mgpnt-gpnt*gdim(f)
            gpoint(f)=ort(zort(f)+mgpnt)  
         enddo
         do p1 = 1,p-1
            currdist = 0.0_dop
            gpnt=map(p1)-1
            do n=1,nf
               f=grid(n)
               mgpnt=gpnt
               gpnt=gpnt/gdim(f)
               mgpnt=mgpnt-gpnt*gdim(f)
               gpoint1(f)=ort(zort(f)+mgpnt)  
            enddo
            do f = 1,ndof
               currdist = currdist + (gpoint(f)-gpoint1(f))**2
            enddo
            distsqdiff = abs(distsq(p)-distsq(p1))

            if(currdist.lt.mindist .and. distsqdiff.gt.1.0e-10_dop)then
               mindist = currdist
               proxidx(map(p)) = map(p1)
            endif
         enddo
      enddo

! Loop over all gridpoints.
      modus = 0
      do p = 1,subdim(1)
! Get coordinates of the current gridpoint.
         gpnt=p-1
         do n=1,nf
            f=grid(n)
            mgpnt=gpnt
            gpnt=gpnt/gdim(f)
            mgpnt=mgpnt-gpnt*gdim(f)
            gpoint(f)=ort(zort(f)+mgpnt)  
         enddo
! Evaluate the diabatic GAP energy here.
         call potgap(gpoint,v,compdim,modus)
         if(p.eq.1)modus=1

! Diagonalise the diabatic energy matrix to get the adiabatic energies
! and the adiabatic-diabatic transformation matrix (the eigenvectors).
         if(compdim.eq.1)then
            rv(1:nddstate,1:nddstate,p) = v(1,1:nddstate,1:nddstate)
            call dsyev('V','U',nddstate,rv(1,1,p),nddstate,av,work,3*nddstate,ierr)
            if(ierr.ne.0)then
               routine = 'gap_diab_wf'
               message = 'Failure to diagonalise diabatic energy'
               call errormsg
            endif
         else
            cv(1:nddstate,1:nddstate) = cmplx(v(1,1:nddstate,1:nddstate),&
                 v(compdim,1:nddstate,1:nddstate),dop)
            call zheev('V','U',nddstate,cv,nddstate,av,cwork,2*nddstate,&
                 work,ierr)
            if(ierr.ne.0)then
               routine = 'gap_diab_wf'
               message = 'Failure to diagonalise complex diabatic energy'
               call errormsg
            endif
         endif
      enddo

      do p = 1,subdim(1)
! Adiabatic wavefunction.
         mp = map(p)
         do s = 1,nddstate
            tmpadpsi(s) = psi(zetf(1,s)+mp-1)
         enddo

         if(p.eq.1)then
            do s = 1,nddstate
               maxcom = 0.0_dop
               do s1 = 1,nddstate
                  if(abs(rv(s1,s,mp)).gt.maxcom)then
                     maxidx = s1
                     maxcom = abs(rv(s1,s,mp))
                  endif
               enddo
               if(rv(maxidx,s,mp).lt.0.0_dop)then
                  rv(1:nddstate,s,mp) = -1.0_dop*rv(1:nddstate,s,mp)
               endif
            enddo
         else
            pp = proxidx(mp)
            do s = 1,nddstate
               do s1 = 1,nddstate
                  sover(s1,s) = dot_product(rv(1:nddstate,s1,mp),rv(1:nddstate,s,pp))
               enddo
            enddo
            do s = 1,nddstate
               maxcom = 0.0_dop
               do s1 = 1,nddstate
                  if(abs(sover(s,s1)).gt.maxcom)then
                     maxidx = s1
                     maxcom = abs(sover(s,s1))
                  endif
               enddo
               if(sover(s,maxidx).lt.0.0_dop)&
                    rv(1:nddstate,s,mp) = -1.0_dop*rv(1:nddstate,s,mp)
            enddo
         endif

!         tmppsi = matmul(transpose(rv(1:nddstate,1:nddstate,mp)),tmpadpsi)
         tmppsi = matmul(rv(1:nddstate,1:nddstate,mp),tmpadpsi)
         do s = 1,nddstate
            psi(zetf(1,s)+mp-1) = tmppsi(s) 
         enddo
      enddo

! Ensure the eigenvectors have a consistent phase.
! Transform the adiabatic wavefunction at the gridpoint to the diabatic
! representation.
!         do s = 1,nddstate
!            tmpadpsi(s) = psi(zetf(1,s)+p-1)
!         enddo
!         if(compdim.eq.1)then
!            if(p.eq.1)then
!               do s = 1,nddstate
!                  maxcom = 0.0_dop
!                  do s1 = 1,nddstate
!                     if(abs(rv(s1,s)).gt.maxcom)then
!                        maxidx = s1
!                        maxcom = abs(rv(s1,s))
!                     endif
!                  enddo
!                  if(rv(maxidx,s).lt.0.0_dop)then
!                     rv(1:nddstate,s) = -1.0_dop*rv(1:nddstate,s)
!                  endif
!               enddo
!               rvref = rv
!               rvm1 = rv
!            else if (mod(p,gdim(5)).eq.1)then
!               do s = 1,nddstate
!                  do s1 = 1,nddstate
!                     sover(s1,s) = dot_product(rv(1:nddstate,s1),rvref(1:nddstate,s))
!                  enddo
!               enddo
!               do s = 1,nddstate
!                  maxcom = 0.0_dop
!                  do s1 = 1,nddstate
!                     if(abs(sover(s,s1)).gt.maxcom)then
!                        maxidx = s1
!                        maxcom = abs(sover(s,s1))
!                     endif
!                  enddo
!                  if(sover(s,maxidx).lt.0.0_dop)&
!                       rv(1:nddstate,s) = -1.0_dop*rv(1:nddstate,s)
!               enddo
!               rvref = rv
!               rvm1 = rv              
!            else
!               do s = 1,nddstate
!                  do s1 = 1,nddstate
!                     sover(s1,s) = dot_product(rv(1:nddstate,s1),rvm1(1:nddstate,s))
!                  enddo
!               enddo
!               do s = 1,nddstate
!                  maxcom = 0.0_dop
!                  do s1 = 1,nddstate
!                     if(abs(sover(s,s1)).gt.maxcom)then
!                        maxidx = s1
!                        maxcom = abs(sover(s,s1))
!                     endif
!                  enddo
!                  if(sover(s,maxidx).lt.0.0_dop)&
!                       rv(1:nddstate,s) = -1.0_dop*rv(1:nddstate,s)
!               enddo
!               rvm1 = rv              
!            endif
!!            tmppsi = matmul(transpose(rv),tmpadpsi)
!            tmppsi = matmul(rv,tmpadpsi)
!         else
!            tmppsi = matmul(cv,tmpadpsi)
!         endif
!         do s = 1,nddstate
!            psi(zetf(1,s)+p-1) = tmppsi(s) 
!         enddo
!      enddo

!      stop

!      write(*,*)'diab',subdim(1)
!      write(*,*)zetf(1,1:4)
!      do f = 1,subdim(1)
!         write(*,*)f,dble(psi(f+zetf(1,2)-1))
!      enddo

!      do f = 1,ndof
!         write(*,*)f,ort(zort(f))
!      enddo
!      stop

      deallocate(v)

      return
      end subroutine gap_diab_wf

!#######################################################################
! Determines the number of Slater determinants in the CAS expansions
! for each multiplicity.
! GWR 05/19
!#######################################################################
      subroutine get_nconfig

      implicit none

      integer(long) :: sm,nalpha,nbeta,nactive

! How many electrons in the active space, and how many of them are alpha
! or beta. Assume that alpha>beta and these numbers represent the lowest
! multiplicity present.
      nactive = nelectron-2*nclosed
      nbeta = nactive/2
      nalpha = nactive-nbeta

      if(nactive.lt.1 .or. nbeta.lt.0 .or. nalpha.lt.1 .or. ncas.lt.1)then
         routine = 'get_nconfig'
         message = 'Error in number of electrons or orbitals'
         call errormsg
      endif

! Assume multiplicities are a series with same no. electrons 
! e.g. 1,3,5,... or 2,4,...
      do sm = 1,nsmult
         nconf(sm,1) = choose(ncas,nalpha)
         nconf(sm,2) = choose(ncas,nbeta)
         ncasel(sm,1) = nalpha
         ncasel(sm,2) = nbeta
         nalpha = nalpha+1
         nbeta = nbeta-1
      enddo

      return

      end subroutine get_nconfig

!#######################################################################
! Function to evaluate the "choose" value of two integer i.e.
! choose = n!/((n-a)!*a!).
! GWR 05/19
!#######################################################################
      integer*4 function choose(n,a)

      implicit none

      integer(long), intent(in) :: n,a
      integer(long) :: tmpn,tmpa,i

      if(a.gt.n)then
         routine = 'choose'
         write(message,'(a,2i6)')'Invalid entries for choose function',n,a
         call errormsg
      endif

      if(a.eq.0)then
         choose = 1
         return
      endif

      tmpn = n
      tmpa = 1
      do i = 2,a
         tmpa = tmpa*i
         tmpn = tmpn*(n-i+1)
      enddo

      choose = tmpn/tmpa

      return
      end function choose

!#######################################################################
!
! This routine gets the value of a potential at a grid point from a
! DB of energies using Gaussian Process.
!
! The point is contained in gpoint. 
! 
! Mainly used to plot the potential in showsys.
!
!#######################################################################
! vcdim = 2 complex PES

      subroutine potgap(gpoint,v,vcdim,modus)

      implicit none

      integer(long), intent(in) :: modus,vcdim
      real(dop), dimension(vcdim,nddstate,nddstate), intent(out) :: v
      real(dop), dimension(ndofdd), intent(in) :: gpoint

      integer(long) :: irec,s,s1,i

      real(dop) :: covar
      real(dop), dimension(vcdim,nddstate,nddstate) :: vtmp
      real(dop), dimension(ndofddpes) :: xgpoint

      logical(kind=4), save :: lfirst=.true.

!      integer(long), save                     :: tid=0 ! timer ID

!      call get_timer(tid, 'POTGAP')
!      call start_timer(tid)

! First solve the GPR equations to get the weights if required.
      if(lfirst .and. modus.eq.0)then
         call agapdiag
         call acholupd(lfirst,dbnrec,dbnrec)
         if (vcdim .eq. 2) then
            call ddcmgapwgt(dbgapwgt(1:nddstate,1:nddstate,1:dbnrec),&
                 dbener(1:nddstate,1:nddstate,1:dbnrec),&
                 dbcgapwgt(1:nddstate,1:nddstate,1:dbnrec),&
                 dbcener(1:nddstate,1:nddstate,1:dbnrec),&
                 dbnrec,nddstate,ddmorder)
         else
            call ddmgapwgt(dbgapwgt(1:nddstate,1:nddstate,1:dbnrec),&
                 dbener(1:nddstate,1:nddstate,1:dbnrec),&
                 dbnrec,nddstate,ddmorder)
         endif
         if (lddsoc) then
            call ddmsocwgt(dbsocwgt(1:nddstate,1:nddstate,1:dbnrec),&
                 dbsoc(1:nddstate,1:nddstate,1:dbnrec),dbnrec,nddstate,ddmorder)
          endif

         lfirst = .false.
      endif

      vtmp = 0.0_dop

      if(.not.lddtrans)&
           xgpoint = gpoint + ddxcoo0
      if(ddcoo.eq.0)&
           call ddq2x(gpoint,xgpoint)

      do irec = 1,dbnrec
         if(lddtrans.and.ddcoo.eq.1)then
            call gapcovar(gpoint,dbnm(1:ndofdd,irec),covar,ndofdd,&
                 ddmorder,gapalpha,gapkfac)
         else
            call gapcovar(xgpoint,dbgeo(1:ndofddpes,irec),covar,ndofddpes,&
                 ddmorder,gapalpha,gapkfac)
         endif

         do s = 1,nddstate
            vtmp(1,s,s) = vtmp(1,s,s) + dbgapwgt(s,s,irec)*covar
            if(lddsoc.and..not.lsocdegen) vtmp(1,s,s) = vtmp(1,s,s) &
                 + dble(dbsocwgt(s,s,irec))*covar
            do s1 = s+1,nddstate
               vtmp(1,s,s1) = vtmp(1,s,s1) + dbgapwgt(s,s1,irec)*covar
               if(lddsoc)then
                  if(lsocdegen)then
                     if(vcdim.eq.2) vtmp(2,s,s1) = vtmp(2,s,s1)&
                          + dbcgapwgt(s,s1,irec)*covar
                  else
                     vtmp(1,s,s1) = vtmp(1,s,s1) &
                          + dble(dbsocwgt(s,s1,irec))*covar
                     if(vcdim.eq.2) vtmp(2,s,s1) = vtmp(2,s,s1) &
                          + aimag(dbsocwgt(s,s1,irec))*covar
                  endif
               endif
            enddo
         enddo
      enddo

      do s = 1,nddstate
         do s1 = s+1,nddstate
            vtmp(1,s1,s) = vtmp(1,s,s1)
            if(vcdim.eq.2) vtmp(2,s1,s) = -1.0_dop*vtmp(2,s,s1)
         enddo
      enddo

      do s = 1,nddstate
         do s1 = 1,nddstate
            do i = 1,vcdim
               v(i,s1,s) = vtmp(i,s1,s)
            enddo
            if(s.eq.s1) v(1,s1,s) = v(1,s1,s) + gapeshift
         enddo
      enddo

!      call stop_timer(tid)

      return

      end subroutine potgap

!#######################################################################
!
! This routine gets the value of a single potential term at a grid point 
! from a DB of energies using Gaussian Process.
!
! The point is contained in gpoint. 
! 
!
! vcdim = 2 complex PES
!#######################################################################

      subroutine potgapss1(gpoint,v,vcdim,s,s1,modus)

      implicit none

      integer(long), intent(in) :: modus,s,s1,vcdim
      real(dop), dimension(vcdim), intent(out) :: v
      real(dop), dimension(ndofdd), intent(in) :: gpoint

      integer(long) :: irec

      real(dop) :: covar
      real(dop), dimension(vcdim)     :: vtmp
      real(dop), dimension(ndofddpes) :: xgpoint

      logical(kind=4), save :: lfirst=.true.

!      integer(long), save :: tid=0 ! timer ID

!      call get_timer(tid, 'POTGAPSS1')
!      call start_timer(tid)

! First solve the GPR equations to get the weights if required.
      if(lfirst .and. modus.eq.0 .and. &
           dot_product(dbgapwgt(s,s,1:dbnrec),dbgapwgt(s,s,1:dbnrec)).lt.tiny)then
         call agapdiag
         call acholupd(lfirst,dbnrec,dbnrec)
         if (vcdim .eq. 2) then
            call ddcmgapwgt(dbgapwgt(1:nddstate,1:nddstate,1:dbnrec),&
                 dbener(1:nddstate,1:nddstate,1:dbnrec),&
                 dbcgapwgt(1:nddstate,1:nddstate,1:dbnrec),&
                 dbcener(1:nddstate,1:nddstate,1:dbnrec),&
                 dbnrec,nddstate,ddmorder)
         else
            call ddmgapwgt(dbgapwgt(1:nddstate,1:nddstate,1:dbnrec),&
                 dbener(1:nddstate,1:nddstate,1:dbnrec),&
                 dbnrec,nddstate,ddmorder)
         endif
         if(lddsoc) then
            call ddmsocwgt(dbsocwgt(1:nddstate,1:nddstate,1:dbnrec),&
                 dbsoc(1:nddstate,1:nddstate,1:dbnrec),dbnrec,nddstate,ddmorder)
         endif
         lfirst = .false.
      endif

      if(.not.lddtrans)&
           xgpoint = gpoint + ddxcoo0
      if(ddcoo.eq.0)&
           call ddq2x(gpoint,xgpoint)

      vtmp = 0.0_dop
      do irec = 1,dbnrec
         if(lddtrans.and.ddcoo.eq.1)then
            call gapcovar(gpoint,dbnm(1:ndofdd,irec),covar,ndofdd,&
                 ddmorder,gapalpha,gapkfac)
         else
            call gapcovar(xgpoint,dbgeo(1:ndofddpes,irec),covar,ndofddpes,&
                 ddmorder,gapalpha,gapkfac)
         endif
         vtmp(1) = vtmp(1) + dbgapwgt(s,s1,irec)*covar
         if(lddsoc)then
            if(lsocdegen)then
               vtmp(2) = vtmp(2) + dbcgapwgt(s,s1,irec)*covar
            else
               vtmp(1) = vtmp(1) + dble(dbsocwgt(s,s1,irec))*covar
               vtmp(2) = vtmp(2) + aimag(dbsocwgt(s,s1,irec))*covar
            endif
         endif
      enddo

      v = vtmp

      if(s.eq.s1) v(1) = v(1) + gapeshift

!      call stop_timer(tid)

      return

      end subroutine potgapss1

!#########################################################################
! Subroutine to calculate exponential components of the GAP.
! GWR 07/16
!#########################################################################
      subroutine gapexpeval(irec,expon,gpoint,grid,nf)
      
      implicit none

      integer(long), intent(in) :: irec,nf
      integer(long), dimension(nf), intent(in)    :: grid
      real(dop), dimension(nf), intent(out)       :: expon
      real(dop), dimension(ndofddpes), intent(in) :: gpoint

      integer(long) :: n,f
      real(dop) :: distsq
      real(dop) :: xgpoint

      do n = 1,nf
         f = grid(n)
         if(lddtrans.and.ddcoo.eq.1)then
            if(gapker(f).eq.1)then
               distsq = (gpoint(f)-dbnm(f,irec))**2
            else if(gapker(f).eq.2)then
               distsq = abs(gpoint(f)-dbnm(f,irec))
            else if(gapker(f).eq.3)then
               distsq = (exp(-gapkerscl(f)*gpoint(f))&
                    -exp(-gapkerscl(f)*dbnm(f,irec)))**2
            endif
         else
            xgpoint = gpoint(f) + ddxcoo0(f)
            if(gapker(f).eq.1.or.gapker(f).eq.3)then
               distsq = (xgpoint-dbgeo(f,irec))**2
            else if(gapker(f).eq.2)then
               distsq = abs(xgpoint-dbgeo(f,irec))
            endif
         endif
!         expon(n) = exp(-1.0_dop*gapalpha*distsq)
         expon(n) = exp(-1.0_dop*gapalpha(f)*distsq)
      enddo

      return
      end subroutine gapexpeval

!#######################################################################
! Routine to calculate the diagonal term in the A-matrix used by GAP
! GWR 02/18
!#######################################################################
      subroutine agapdiag

      implicit none

      integer(long) :: i,f

! Calculate the self-covariance, which depends on the method.
      if(actdof.eq.0)then
         do f = 1,ndofdd
            if(basis(f).eq.19 .or. dofspf(f).eq.0)cycle
            actdof = actdof + 1
         enddo
      endif
         
      if(ddmorder.eq.0)then
         gapdiag = 1.0_dop
      else
         gapdiag = dble(actdof-ddmorder+1)/dble(ddmorder)
         do i = ddmorder-1,1,-1
            gapdiag = gapdiag + 1.0_dop
            gapdiag = gapdiag * dble(actdof-i+1)/dble(i)
         enddo
      endif

      gapdiag = gapkfac**2*gapdiag + gapsigma
      
      return
      end subroutine agapdiag

!#######################################################################
! Routine to construct the Cholesky update of the A-matrix in GAP by
! use of Cholesky update.
! GWR 02/18
!#######################################################################
      subroutine acholupd(lfirst,nrec,nrecold)

      implicit none

      logical(kind=4), intent(in) :: lfirst
      integer(long), intent(in)   :: nrec,nrecold

      integer(long) :: irec,jrec,krec,info,ndiff
      real(dop)                              :: acovar
      real(dop), dimension(:,:), allocatable :: amat,ablk

      integer(long), save                :: tid = 0 ! timer ID
      call get_timer(tid, 'acholupd')
      call start_timer(tid)

! If the first call of this routine, we must perform the full Cholesky
! decomposition on the covariance matrix, A.
! Nrecold is redundant in this case.
      if(lfirst)then
         if(allocated(achol))deallocate(achol)
         acholdim = nrec
         allocate(achol(acholdim,acholdim))

! Construct full A-matrix i.e. matrix of covariances.
         do irec = 1,nrec
            achol(irec,irec) = gapdiag
            do jrec = irec+1, nrec
               if(lddtrans.and.ddcoo.eq.1)then
                  call gapcovar(dbnm(1:ndofdd,irec),dbnm(1:ndofdd,jrec),&
                       acovar,ndofdd,ddmorder,gapalpha,gapkfac)
               else
                  call gapcovar(dbgeo(1:ndofddpes,irec),dbgeo(1:ndofddpes,jrec),&
                       acovar,ndofddpes,ddmorder,gapalpha,gapkfac)
               endif
               achol(irec,jrec) = acovar
            enddo
         enddo
         do irec = 1,nrec
            do jrec = irec+1,nrec
               achol(jrec,irec) = achol(irec,jrec)
            enddo
         enddo

         call dpotrf('L',dbnrec,achol,dbnrec,info)
         if(info.ne.0)then
            routine = 'acholupd'
            write(message,'(a,i8)') 'A-matrix not positive definite',info
            call errormsg
         endif
      else
! If not the first call to this routine we simply update the Cholesky
! decomposition calculated earlier.
! Check first that the new matrix is bigger than the old.
         if(nrec.le.nrecold)then
            routine = 'acholupd'
            write(message,'(a,2i8)')'New nrec is not greater than old nrec',&
                 nrec,nrecold
            call errormsg
         endif

         ndiff = nrec - nrecold

         if(allocated(amat))deallocate(amat)
         allocate(amat(nrecold,ndiff))
         if(allocated(ablk))deallocate(ablk)
         allocate(ablk(ndiff,ndiff))

! First task is to construct new entries to the A-matrix.
         do irec = nrecold+1,nrec
            do jrec = 1,nrecold
               if(lddtrans.and.ddcoo.eq.1)then
                  call gapcovar(dbnm(1:ndofdd,irec),dbnm(1:ndofdd,jrec),&
                       acovar,ndofdd,ddmorder,gapalpha,gapkfac)
               else
                  call gapcovar(dbgeo(1:ndofddpes,irec),dbgeo(1:ndofddpes,jrec),&
                       acovar,ndofddpes,ddmorder,gapalpha,gapkfac)
               endif
               amat(jrec,irec-nrecold) = acovar
            enddo
         enddo

         do irec = 1,ndiff
            ablk(irec,irec) = gapdiag
            do jrec = 1,irec-1
               if(lddtrans.and.ddcoo.eq.1)then
                  call gapcovar(dbnm(1:ndofdd,irec+nrecold),&
                       dbnm(1:ndofdd,jrec+nrecold),&
                       acovar,ndofdd,ddmorder,gapalpha,gapkfac)
               else
                  call gapcovar(dbgeo(1:ndofddpes,irec+nrecold),&
                       dbgeo(1:ndofddpes,jrec+nrecold),&
                       acovar,ndofddpes,ddmorder,gapalpha,gapkfac)
               endif
               ablk(irec,jrec) = acovar
            enddo
         enddo           

! Second task is to construct the "off-diagonal" part of the update by solution
! of the linear equations Lx=b where L is the original decomposition and b
! is the new part of amat.
         call dtrsm('L','L','N','N',nrecold,ndiff,1.0_dop,acholtmp,&
              acholtmpdim,amat,nrecold)

! Third task is to construct an update to the "diagonal" block of amat based
! on the results of the second task. 
         do irec = 1,ndiff
            do jrec = 1,irec
               do krec = 1,nrecold
                  ablk(irec,jrec) = ablk(irec,jrec) &
                       - amat(krec,irec)*amat(krec,jrec)
               enddo
            enddo
         enddo

! Decompose this update to the diagonal block and put all of the 
! results into the matrix of the decomposition.
         call dpotrf('L',ndiff,ablk,ndiff,info)
         if(info.ne.0)then
            routine = 'acholupd'
            write(message,'(a,i8)')'Failed Cholesky update',info
            call errormsg
         endif

! Finally, copy the new parts into the original matrix.
         do irec = nrecold+1,nrec
            do jrec = 1,nrecold
               acholtmp(irec,jrec) = amat(jrec,irec-nrecold)
            enddo
         enddo

         do irec = 1,ndiff
            do jrec = 1,irec
               acholtmp(nrecold+irec,nrecold+jrec) = ablk(irec,jrec)
            enddo
         enddo

      endif

      call stop_timer(tid)

      return
      end subroutine acholupd

!#######################################################################
! Subroutine to perform a full Cholesky decomposition and output the 
! result to a 1-D array representing the lower triangular form of the 
! update.
! GWR 02/18
!#######################################################################
      subroutine achol1d(amat,chol,amdim)

      implicit none

      integer(long), intent(in) :: amdim
      real(dop), dimension(amdim,amdim), intent(in) :: amat
      real(dop), dimension(amdim,amdim), intent(out) :: chol

      integer(long) :: i,j,k
      real(dop) :: asum

      routine = 'achol'

      chol = 0.0_dop

      do i = 1,amdim
         do j = i,amdim
            asum = amat(i,j)
            do k = i-1,1,-1
               asum = asum - chol(i,k)*chol(j,k)
            enddo
            if(i.eq.j)then
               if(asum .le. 0.0_dop)then
                  message='Matrix not positive definite'
                  call errormsg
               endif
               chol(i,i) = sqrt(asum)
            else
               chol(j,i) = asum/chol(i,i)
            endif
         enddo
         
      enddo

      return
      end subroutine achol1d

!#######################################################################
! Calculate the GAP weight vector after new DB points have been 
! calculated.
! GWR 06/16
! Modified from gapwgt to cope with additive GPR.
! GWR 10/16
! Modified to use updated Cholesky decomposition.
! GWR 02/18
! Modified to allow use of complex potentials (2 real parts).
! GWR 04/19
!#######################################################################

      subroutine ddmgapwgt(weight,ener,nrec,nst,order)

      implicit none

      integer(long), intent(in) :: nrec,nst,order
      real(dop), dimension(nst,nst,nrec), intent(in)  :: ener
      real(dop), dimension(nst,nst,nrec), intent(out) :: weight

      integer(long)                      :: ierr,irec,s,s1
      real(dop), dimension(nrec,nst,nst) :: tmppot

      integer(long), save                :: tid = 0 ! timer ID
      call get_timer(tid, 'ddmgapwgt')
      call start_timer(tid)

! Put the energies into the correct order for the solver.
      do irec = 1,nrec
         do s = 1,nst
            do s1 = 1,nst
               tmppot(irec,s,s1) = ener(s,s1,irec)
               if(s.eq.s1) tmppot(irec,s,s1) &
                    = tmppot(irec,s,s1) - gapeshift

            enddo
         enddo
      enddo

! Solve the weight equations using the pre-calculated Cholesky decomposition.
      call dpotrs('L',nrec,nst*nst,achol,nrec,tmppot,nrec,ierr)

      if(ierr.ne.0)then
         routine = 'ddmgapwgt'
         write(message,'(a,i4)') 'Failure to solve GAP weight eqns.',ierr
         call errormsg
      endif

! Copy the weights back and exit.
      do irec = 1,nrec
         do s = 1,nst
            do s1 = 1,nst
               weight(s,s1,irec) = tmppot(irec,s,s1)
            enddo
         enddo
      enddo

      call stop_timer(tid)
      return
      end subroutine ddmgapwgt

!#######################################################################

      subroutine ddcmgapwgt(weight,ener,cweight,cener,nrec,nst,order)

      implicit none

      integer(long), intent(in) :: nrec,nst,order
      real(dop), dimension(nst,nst,nrec), intent(in)  :: ener
      real(dop), dimension(nst,nst,nrec), intent(out) :: weight
      real(dop), dimension(nst,nst,nrec), intent(in)  :: cener
      real(dop), dimension(nst,nst,nrec), intent(out) :: cweight

      integer(long)                        :: ierr,irec,s,s1
      real(dop), dimension(nrec,2,nst,nst) :: tmppot

      integer(long), save                  :: tid = 0 ! timer ID
      call get_timer(tid, 'ddmgapwgt')
      call start_timer(tid)

! Put the energies into the correct order for the solver.
      do irec = 1,nrec
         do s = 1,nst
            do s1 = 1,nst
               tmppot(irec,1,s,s1) = ener(s,s1,irec)
               tmppot(irec,2,s,s1) = cener(s,s1,irec)
               if(s.eq.s1) tmppot(irec,1,s,s1) &
                    = tmppot(irec,1,s,s1) - gapeshift

            enddo
         enddo
      enddo

! Solve the weight equations using the pre-calculated Cholesky decomposition.
      call dpotrs('L',nrec,nst*nst*2,achol,nrec,tmppot,nrec,ierr)

      if(ierr.ne.0)then
         routine = 'ddmgapwgt'
         write(message,'(a,i4)') 'Failure to solve GAP weight eqns.',ierr
         call errormsg
      endif

! Copy the weights back and exit.
      do irec = 1,nrec
         do s = 1,nst
            do s1 = 1,nst
               weight(s,s1,irec) = tmppot(irec,1,s,s1)
               cweight(s,s1,irec) = tmppot(irec,2,s,s1)
            enddo
         enddo
      enddo

      call stop_timer(tid)
      return
      end subroutine ddcmgapwgt

!#######################################################################
! Calculate the GAP weight vector for SOCs after new DB points have been
! calculated.
! GWR 02/19
!#######################################################################
      subroutine ddmsocwgt(weight,soc,nrec,nst,order)

      implicit none

      integer(long), intent(in) :: nrec,nst,order
      complex(dop), dimension(nst,nst,nrec), intent(in)  :: soc
      complex(dop), dimension(nst,nst,nrec), intent(out) :: weight

      integer(long)                         :: ierr,irec,s,s1,n,n1
      complex(dop), dimension(nrec,nst,nst) :: tmpsoc
      complex(dop), dimension(nrec,nrec)    :: cachol

      integer(long), save                :: tid = 0 ! timer ID
      call get_timer(tid, 'ddmsocwgt')
      call start_timer(tid)

! Put the energies into the correct order for the solver.
      do irec = 1,nrec
         do s = 1,nst
            do s1 = 1,nst
               tmpsoc(irec,s,s1) = soc(s,s1,irec)
               tmpsoc(irec,s1,s) = conjg(tmpsoc(irec,s,s1))
            enddo
         enddo
      enddo

! Copy the real Cholesky decomposition into a temporary complex array.
      cachol = cmplx(0.0_dop,0.0_dop,dop)
      do n = 1,nrec
         do n1 = 1,nrec
            cachol(n,n1) = cmplx(achol(n,n1),0.0_dop,dop)
         enddo
      enddo

! Solve the weight equations using the pre-calculated Cholesky decomposition.
      call zpotrs('L',nrec,nst*nst,cachol,nrec,tmpsoc,nrec,ierr)

      if(ierr.ne.0)then
         routine = 'ddmsocwgt'
         write(message,'(a,i4)') 'Failure to solve GAP weight eqns. for SOCs',ierr
         call errormsg
      endif

! Copy the weights back and exit.
      do irec = 1,nrec
         do s = 1,nst
            do s1 = 1,nst
               weight(s,s1,irec) = tmpsoc(irec,s,s1)
            enddo
         enddo
      enddo

      call stop_timer(tid)
      return
      end subroutine ddmsocwgt

!#######################################################################
! Calculate the GAP weight vector for derivative coupling elements.
! GWR 02/18
!#######################################################################
      subroutine dcgapwgt(dcwgt,dercp,dcdim,nrec,ndof1,order)

      implicit none

      integer(long), intent(in) :: dcdim,nrec,ndof1,order
      real(dop), dimension(ndof1,dcdim,nrec), intent(in)  :: dercp
      real(dop), dimension(ndof1,dcdim,nrec), intent(out) :: dcwgt

      integer(long)                          :: ierr,irec,f,d
      real(dop), dimension(nrec,ndof1,dcdim) :: tmpdc

! Put the couplings into the correct order for the solver.
      do irec = 1,nrec
         do f = 1,ndof1
            do d = 1,dcdim
               tmpdc(irec,f,d) = dercp(f,d,irec)
            enddo
         enddo
      enddo

! Solve the weight equations using the pre-calculated Cholesky decomposition.
      call dpotrs('L',nrec,ndof1*dcdim,achol,nrec,tmpdc,nrec,ierr)

      if(ierr.ne.0)then
         routine = 'dcgapwgt'
         write(message,'(a,i4)') 'Failure to solve GAP weight eqns.',ierr
         call errormsg
      endif

! Copy the weights back and exit.
      do irec = 1,nrec
         do f = 1,ndof1
            do d = 1,dcdim
               dcwgt(f,d,irec) = tmpdc(irec,f,d)
            enddo
         enddo
      enddo

      return
      end subroutine dcgapwgt

!#########################################################################
! Routine to calculate the variance of the GAP at a particular point.
! GWR 08/16
! Upgraded to use updated Cholesky decomposition.
! GWR 02/18
!#########################################################################
      subroutine gapvarddm(gpoint,var,nrec,lvarfst,order)

use op1lib, only: unitqxd
use mmlib, only: qqxtdd
use openmpmod

      implicit none

      logical(kind=4), intent(in) :: lvarfst
      integer(long), intent(in)   :: nrec,order
      real(dop), dimension(ndofdd), intent(in) :: gpoint
      real(dop), intent(out)      :: var

      integer(long)                     :: irec,ierr
      real(dop), dimension(nrec)        :: covar,tmpvec
      real(dop), dimension(ndofddpes)   :: xgpoint

      integer(long), save               :: tid = 0 ! timer ID

! Timing
      call get_timer(tid, 'gapvarddm')
      call start_timer(tid)

      routine = 'gapvarddm'
      
! Jacobis.
      if(ddcoo.eq.0)then
         call ddq2x(gpoint,xgpoint)
      elseif(ddcoo.eq.2)then
         xgpoint = gpoint + ddxcoo0
      endif

! Calculate the covariance vector for the trial geometry.
!$omp parallel do num_threads(ompthread) if(lompthread) default(shared),&
!$omp& schedule(static)
      do irec = 1,nrec
         if(lddtrans.and.ddcoo.eq.1)then
            call gapcovar(gpoint(1:ndofdd),dbnm(1:ndofdd,irec),&
                 covar(irec),ndofdd,order,gapalpha,gapkfac)
         else if(ddcoo.eq.0)then
            call gapcovar(xgpoint(1:ndofddpes),dbgeo(1:ndofddpes,irec),&
                 covar(irec),ndofddpes,order,gapalpha,gapkfac)
         else
            call gapcovar(xgpoint(1:ndofdd),dbgeo(1:ndofdd,irec),&
                 covar(irec),ndofdd,order,gapalpha,gapkfac)
         endif
      enddo
!$omp end parallel do
      tmpvec = covar

! Pass the pre-computed Cholesky factored matrix to the solver.
      if(lvarfst)then
         call dpotrs('L',acholdim,1,achol,acholdim,tmpvec,acholdim,ierr)
      else
         call dpotrs('L',nrec,1,acholtmp,acholtmpdim,tmpvec,nrec,ierr)
      endif
      if(ierr.ne.0)then
         write(message,'(a,i6)') 'Failure to solve linear equations: ',ierr
         call errormsg
      endif

! Calculate the variance.
      var = (gapdiag - dot_product(covar,tmpvec))

      call stop_timer(tid)

      return
      end subroutine gapvarddm

!#########################################################################
! Test routine to print the variance at the grid-locations in an exact 
! calculation.
! GWR 02/18
!#########################################################################
      subroutine printvar(time,ort)
       
      implicit none

      real(dop), intent(in)                       :: time
      real(dop), dimension(ortdim), intent(in)    :: ort

      integer(long) :: f,m,p,gpnt,n,nf,mgpnt
      integer(long), dimension(maxdim) :: grid
      real(dop) :: var
      real(dop), dimension(maxdim) :: gpoint

!      write(*,*)time*fs

      m = 1

      grid = 0
      nf = nspfdof(m)
      do n = 1,nf
         grid(n) = spfdof(n,m)
      enddo

      do p = 1,subdim(m)
         gpnt = p-1
         gpoint = 0.0_dop
         do n=1,nf
            f=grid(n)
            mgpnt=gpnt
            gpnt=gpnt/gdim(f)
            mgpnt=mgpnt-gpnt*gdim(f)
            gpoint(f)=ort(zort(f)+mgpnt)  
         enddo
         
         call gapvarddm(gpoint,var,dbnrec,.true.,ddmorder)

         write(*,*)(gpoint(grid(n)),n=1,nf),var
      enddo

      stop

      return

      end subroutine printvar

!#########################################################################
! Routine to calculate GPR covariance between two vectors, geom1 and geom2
! Can deal with standard GPR as well as additive GPR to any order.
! GWR 10/16
! Modified to add exponential kernels.
! GWR 07/19
!#########################################################################
      subroutine gapcovar(geom1,geom2,covar,ncoord,order,alpha,kfac)

      implicit none

      integer(long), intent(in) :: ncoord,order
      real(dop), intent(out)    :: covar
      real(dop), intent(in)     :: kfac
      real(dop), dimension(ncoord), intent(in) :: geom1,geom2,alpha

      integer(long) :: i,j,f
      real(dop) :: tmpcovar,distsq
      real(dop), dimension(ncoord) :: lincovar
      real(dop), allocatable :: powersum(:),einterm(:)

! Calculate the linear covariances first i.e. Gaussians of distance 
! vector components.
      do f = 1,ncoord
         if(gapker(f).eq.1)then
            distsq = (geom1(f)-geom2(f))**2
         else if(gapker(f).eq.2)then
            distsq = abs(geom1(f)-geom2(f))
         else if(gapker(f).eq.3)then
            distsq = (exp(-gapkerscl(f)*geom1(f))&
                 -exp(-gapkerscl(f)*geom2(f)))**2
         endif
         lincovar(f) = exp(-alpha(f)*distsq)
      enddo

! If order is 0 then just do standard full-D GPR by multiplying all 
! linear terms together.
      if(order.eq.0)then
         tmpcovar = lincovar(1)
         do f = 2,ncoord
            tmpcovar = tmpcovar * lincovar(f)
         enddo
         tmpcovar = kfac**2*tmpcovar
      else
! If order is greater than 0, then combine all terms up to the 
! maximum order using the Newton-Girard recursion.
         allocate(powersum(order),einterm(0:order))

         powersum = 0.0_dop
         do i = 1,order
            do f = 1,ncoord
               if(basis(f).eq.19 .or. dofspf(f).eq.0)cycle
               powersum(i) = powersum(i) + lincovar(f)**(i)
            enddo
         enddo

         einterm = 0.0_dop
         einterm(0) = 1.0_dop
         do i = 1,order
            do j = 1,i
               einterm(i) = einterm(i) + &
                    (-1.0_dop)**(j-1) * einterm(i-j) * powersum(j)
            enddo
            einterm(i) = einterm(i)/dble(i)
         enddo

! Do not include zeroth order term in final covariance.
         tmpcovar = 0.0_dop
         do i = 1,order
            tmpcovar = tmpcovar + kfac**2*einterm(i)
         enddo
         deallocate(powersum,einterm)
      endif

      covar = tmpcovar

      return

      end subroutine gapcovar

!#######################################################################
! Calculate a low-dimensional representation of the PES using linear 
! algebra after new DB points have been calculated and a new full GAP 
! representation of the PES has been calculated.
! GWR 05/17
!#######################################################################
      subroutine ddgbtendec(ort,geo)

      implicit none

      real(dop), dimension(ortdim), intent(in)        :: ort
      real(dop), dimension(ndofdd,dbnrec), intent(in) :: geo

      logical(kind=4), dimension(ktot) :: kinc,kcalc
      logical(kind=4), dimension(nddstate,nddstate) :: lkfirst

      integer(long) :: f,f1,s,s1,m,k,h,h1,mdx,idx,sumgdofs,g,loop1d,&
                       n,nham,modus,vcdim
      integer(long) :: k1c,k2c,k1u,k2u,kcorridx
      integer(long), dimension(ddmoptord) :: modeidx,halias

!      real(dop), dimension(nddstate,nddstate) :: vorig
      real(dop), dimension(:,:,:), allocatable :: vorig
      real(dop), dimension(ndofdd)            :: xidb,xfdb,gpoint

      integer(long), save                     :: tid = 0 ! timer ID
      call get_timer(tid, 'ddgbtendec')
      call start_timer(tid)

!      if(.not.lddtrans)then
!      routine='ddgbtendec'
!         message = 'DD-GB using SVD only implemented for normal modes so far'
!         call errormsg
!      endif

      if(lddsoc .and. .not. lsocdegen)then
         vcdim=2
      else
         vcdim=1
      endif

      if(allocated(vorig))deallocate(vorig)
      allocate(vorig(vcdim,nddstate,nddstate))

! Indices of correlated operators.
      k1c = zham(1)
      k2c = zham(1)+kzahl(1)-1
      kcorridx = 0

! Maximum extent of the database points.
      xidb = 0.0_dop
      xfdb = 0.0_dop
      do f = 1,ndofdd
         xidb(f) = minval(geo(f,:),1)
         xfdb(f) = maxval(geo(f,:),1)
      enddo

! Initialise the linked list in which the DD-GB-SVD parameters will be held.
      if(associated(firstogpar))call ogpardelall(firstogpar,ogpar)
      call ogparinit(firstogpar,ogpar)


! Initialise some variables which determine how many loops to carry out.
      kinc = .false.
      do k = k1c,k2c
         do m = 1,nmode
            kinc(k) = kinop(m,k).eq.0
         enddo
      enddo

      do m = 1,nmode
         do s = 1,nddstate
            k1u = zhun(m,s,1)
            k2u = zhun(m,s,1)+khzahl(m,s,1)-1
            do k = k1u,k2u
               kinc(k) = kinop(m,k).eq.0
            enddo
         enddo
      enddo

      kcalc = .true.

! First task is to calculate all GPR potential terms at the origin (energies
! of all states and values of diabatic couplings, if any).
      gpoint = 0.0_dop
      nham = 2
      modus = 1
      call potgap(gpoint,vorig,vcdim,modus)

! Loop1d variable controls the inital loops, ensuring that all 1D terms are 
! dealt with first, followed by a loop over the uncorrelated nD terms. After 
! this, the loops run over 1D and nD terms, correlated and uncorrelated.
      loop1d = 0
      lkfirst = .true.
      outer: do while(count(kinc).ne.0)
            
! Deal with uncorrelated operators first.
         do m = 1,nmode
            do s = 1,nddstate
               k1u = zhun(m,s,1)
               k2u = zhun(m,s,1)+khzahl(m,s,1)-1
               
               kuloop: do k = k1u,k2u
                  if(kinop(m,k).ne.0 .or. diag(m,k))cycle kuloop
                  if(.not.kcalc(k) .or. .not.kinc(k)) cycle kuloop

                  f = spfdof(1,m)
                  h = hamilton(f,k)
                  if(loop1d.eq.0.and.htmfac(h).gt.1)cycle kuloop
                  if(loop1d.eq.1.and.htmfac(h).eq.1)cycle kuloop
                  halias = 0
                  if(htmmode(h).ne.0)then
                     do h1 = 1,htmfac(h)
                        halias(h1) = hterm(h1,h)
                     enddo
                  else if(htmdof(h).ne.0)then
                     halias(1) = h
                  else
                     cycle kuloop
                  endif

!                  if(hopfile(hterm(1,h)).ne.208)cycle kuloop

! Loop to allocate arrays based upon number of DVR points.
                  ogpar%ndofs = htmfac(h)                  
                  do h1 = 1,htmfac(h)
                     f1 = htmdof(halias(h1))
                     ogpar%dofs(h1) = f1
                     ogpar%gdofs(h1) = gdim(f1)
                  enddo
                  call alloc_ogpararr(ogpar,vcdim)

! Loop to set up operator grid.
                  sumgdofs = 0
                  do h1 = 1,htmfac(h)
                     f1 = htmdof(halias(h1))
                     do g = 1,ogpar%gdofs(h1)
                        ogpar%gaport(g,h1) = ort(zort(f1)+g-1)
                     enddo
                     ogpar%goffs(h1) = sumgdofs
                     sumgdofs = sumgdofs + ogpar%gdofs(h1)
                     ogpar%totgd = ogpar%totgd + ogpar%gdofs(h1)
                     ogpar%xi(h1) = max(xidb(f1),ogpar%gaport(1,h1))
                     ogpar%xf(h1) = min(xfdb(f1),&
                          ogpar%gaport(ogpar%gdofs(h1),h1))
                  enddo
                  ogpar%k  = k
                  ogpar%ki = s
                  ogpar%kf = s
                  if(associated(ogpar%prev))then
                     ogpar%kidx = ogpar%prev%kidx + 1
                  else
                     ogpar%kidx = 1
                  endif

                  ogpar%lcorr = .false.
                  ogpar%lkfirst = lkfirst(s,s)

                  call ddgbtendeccalc(ogpar,vorig(1,s,s),vcdim,kcorridx)
                  lkfirst(s,s) = .false.

                  call ogparadd(ogpar)
                  kinc(k) = .false.
                  kcalc(k) = .false.

               enddo kuloop
            enddo
         enddo
         loop1d = loop1d + 1
         if(loop1d.lt.2)cycle outer
         
         kcloop: do k = k1c,k2c 
            if(.not.kinc(k))cycle kcloop
            modeidx = 0
            mdx = 0
            do m = 1,nmode
               if(kinop(m,k).ne.0) cycle kcloop
               if(ki(k).eq.kf(k))then
                  if(.not.diag(m,k))then
                     mdx = mdx + 1
                     modeidx(mdx) = m
                  endif
               else
                  do n = 1,nspfdof(m)
                     f = spfdof(n,m)
                     if(f.eq.feb)cycle
                     h = hamilton(f,k)
                     if(h.gt.0 .and.htmtype(h).ne.1)then
                        mdx = mdx + 1
                        modeidx(mdx) = m
                     endif
                  enddo
               endif
            enddo

! Loops to allocate arrays based upon number of Gaussians used.
            idx = 0
            do m = 1,mdx
               f = spfdof(1,modeidx(m))
               if(f.eq.feb)cycle
               h = hamilton(f,k)             
               idx = idx + htmfac(h)
            enddo
            ogpar%ndofs = idx
            idx = 0
            sumgdofs = 0
            do m = 1,mdx
               f = spfdof(1,modeidx(m))
               if(f.eq.feb)cycle
               h = hamilton(f,k)    

               halias = 0
               if(htmmode(h).ne.0)then
                  do h1 = 1,htmfac(h)
                     halias(h1) = hterm(h1,h)
                  enddo
               else
                  halias(1) = h
               endif

!               if(hopfile(hterm(1,h)).ne.208)cycle kcloop

               do h1 = 1,htmfac(h)
                  f1 = htmdof(halias(h1))
                  if(f1.eq.feb)cycle
                  idx = idx + 1
                  ogpar%dofs(idx) = f1
                  ogpar%gdofs(idx) = gdim(f1)
               enddo
            enddo
            call alloc_ogpararr(ogpar,vcdim)

! Loops to set up the operator.
            if(lmult)then
               s  = kf(k)
               s1 = ki(k)
            else
               s  = ddsf(hamilton(feb,k))
               s1 = ddsi(hamilton(feb,k))
            endif

            idx = 0
            sumgdofs = 0
            do m = 1,mdx
               f = spfdof(1,modeidx(m))
               if(f.eq.feb)cycle
               h = hamilton(f,k)
               
               halias = 0
               if(htmmode(h).ne.0)then
                  do h1 = 1,htmfac(h)
                     halias(h1) = hterm(h1,h)
                  enddo
               else
                  halias(1) = h
               endif

               do h1 = 1,htmfac(h)
                  f1 = htmdof(halias(h1))
                  if(f1.eq.feb)cycle
                  idx = idx + 1
                  do g = 1,ogpar%gdofs(idx)
                     ogpar%gaport(g,idx) = ort(zort(f1)+g-1)
                  enddo
                  ogpar%goffs(idx) = sumgdofs
                  sumgdofs = sumgdofs + ogpar%gdofs(idx) 
                  ogpar%totgd = ogpar%totgd + ogpar%gdofs(idx)
                  ogpar%xi(idx) = max(xidb(f1),ogpar%gaport(1,idx))
                  ogpar%xf(idx) = min(xfdb(f1),&
                       ogpar%gaport(ogpar%gdofs(idx),idx))
               enddo
            enddo
            ogpar%k  = k
            ogpar%ki = s1
            ogpar%kf = s
            if(associated(ogpar%prev))then
               ogpar%kidx = ogpar%prev%kidx + 1
            else
               ogpar%kidx = 1
            endif
            kcorridx = kcorridx + 1
            ogpar%kcorridx = kcorridx
            ogpar%lcorr = .true.
            ogpar%lkfirst = lkfirst(s,s1)

            call ddgbtendeccalc(ogpar,vorig(1,s,s1),vcdim,kcorridx)
            lkfirst(s,s1) = .false.

            call ogparadd(ogpar)
            kinc(k) = .false.
            kcalc(k) = .false.

         enddo kcloop
      enddo outer

! Total number of operator terms and total number of correlated terms.
      if(associated(ogpar%prev))then
         ktotog = ogpar%prev%kidx
      else
         ktotog = 1
      endif
      ktotcog = kcorridx
      if(ktotog.le.0)then
         routine='ddgbtendec'
         message = '0 or fewer GAP tensor operators!'
         call errormsg
      endif

      write(ilog,*)'Total terms in PES expansion:',ktotog
      flush(ilog)

      if(allocated(vorig))deallocate(vorig)

      call stop_timer(tid)

      return
      end subroutine ddgbtendec

!###########################################################################
! Routine to calculate individual terms of the potential based upon values at
! the DVR gridpoints for 1-D terms, SVD for 2-D terms and tensor 
! decomposition for 3-D terms.
! GWR 10/2017
!###########################################################################
      subroutine ddgbtendeccalc(ogpar1,vorig,vcdim,kcorridx)
      
      implicit none

      type(optgappar), pointer, intent(inout) :: ogpar1
      integer(long), intent(in) :: vcdim
      real(dop), dimension(vcdim), intent(in) :: vorig
      integer(long), intent(inout) :: kcorridx

      logical(kind=4) :: lf1,lf2,lzero
      logical(kind=4), dimension(3) :: lf
      integer(long) :: nft,g1,g2,g3,s,s1,nham,modus,f1,f2,n,nf1,nf2,&
                       gdf1,gdf2,mingdf,maxgdf,lwork,info,ndf,nr,n1,i,j,&
                       iloop,gdx,krpd,nrmax,nrtmp,nrold
      integer(long), dimension(3) :: nf,gdfa,f
      integer(long), dimension(:), allocatable :: ipiv
      real(dop) :: sqrsig,resnrm,resnrmold,svdconv
      real(dop), dimension(:), allocatable :: vg1,vg2,vg3
      real(dop), dimension(:,:), allocatable :: umat,vmat,x1,x2,x3,ata,vinv,&
                                                xtmp,krp,vg
      real(dop), dimension(:,:), pointer :: apt,apt1
      real(dop), dimension(:,:), allocatable, target :: a1,a2,a3

      real(dop), dimension(:,:,:,:), allocatable :: vtmp,vres
      real(dop), dimension(:), allocatable   :: sigma,work,lambda
      real(dop), dimension(ndofdd) :: gpoint
      real(dop), external :: dlange,dnrm2

      complex(dop), dimension(:), allocatable   :: cwork
      complex(dop), dimension(:,:), allocatable :: cumat,cvmat,vcmp

      type(optgappar), pointer :: resogpar

! Allocate arrays.
      allocate(vg1(vcdim),vg2(vcdim),vg3(vcdim),vg(vcdim,3))

! How many DOFs in this term?
      nft = ogpar1%ndofs

! Which state(s) in the operator.      
      s = ogpar1%kf
      s1 = ogpar1%ki

      nham = 2
      modus = 1

! Treatment depends on dimensionality of the operator term.
      gpoint = 0.0_dop

! 1-D terms are simply the value of the potential at the gridpoints
      if(nft.eq.1)then
         allocate(vtmp(vcdim,ogpar1%gdofs(1),1,1))
         f1 = ogpar1%dofs(1)
         do g1 = 1,ogpar1%gdofs(1)
            gpoint(f1) = ogpar1%gaport(g1,1)
            call potgapss1(gpoint,vtmp(1,g1,1,1),vcdim,s,s1,modus)
! The first 1-D term includes any constant term.
            if(ogpar1%lkfirst)then              
               ogpar1%coeff(1:vcdim,g1,1) = vtmp(1:vcdim,g1,1,1)
            else
               ogpar1%coeff(1:vcdim,g1,1) = vtmp(1:vcdim,g1,1,1) - vorig(1:vcdim)
            endif
         enddo          
      else if(nft.eq.2)then
! 2-D terms are constructed by formation of a matrix of the couplings
! at the gridpoints, followed by an SVD to decompose the matrix into a sum of
! outer products of 2 vectors, each of which gives the potential terms along 
! 1 DOF, the other the terms along the other. 
         allocate(vtmp(vcdim,ogpar1%gdofs(1),ogpar1%gdofs(2),1))

! First get the value of the relevant GAP at the gridpoints.
         f1 = ogpar1%dofs(1)
         f2 = ogpar1%dofs(2)
         do g2 = 1,ogpar1%gdofs(2)
            gpoint(f2) = ogpar1%gaport(g2,2)

            do g1 = 1,ogpar1%gdofs(1)
               gpoint(f1) = ogpar1%gaport(g1,1)
                  call potgapss1(gpoint,vtmp(1,g1,g2,1),vcdim,s,s1,modus)
            enddo
         enddo

! Next, subtract the contributions from the previous terms.
         resogpar => ogpar1
         call ogparrew(resogpar)

         do while(associated(resogpar%next))

! If looped term is not of the same state(s) as the term of interest, cycle.
            if(resogpar%kf .ne. s .or. resogpar%ki .ne. s1)then
               resogpar => resogpar%next
               cycle
            endif

            nf1 = 0
            nf2 = 0
            lf1 = .false.
            lf2 = .false.
            do n = 1,resogpar%ndofs
               lf1 = resogpar%dofs(n) .eq. f1
               if(lf1)then
                  nf1 = n
                  exit
               endif
            enddo
            do n = 1,resogpar%ndofs
               lf2 = resogpar%dofs(n) .eq. f2
               if(lf2)then
                  nf2 = n
                  exit
               endif
            enddo

! If another 2-body term, cycle. Should not have the same DOFs as the current term; if
! it does, there's an error.
            if(resogpar%ndofs .eq. 2) then
               if(lf1.and.lf2)then
                  routine = 'ddgbtendeccalc'
                  message = 'Multiple 2rd-order terms of the same states and DOFs.'
                  call errormsg
               endif
               resogpar => resogpar%next
               cycle
            endif

            if (lf1 .and. resogpar%ndofs.eq.1) then
               do g2 = 1,ogpar1%gdofs(2)
                  do g1 = 1,ogpar1%gdofs(1)
                     vg1(1:vcdim) = resogpar%coeff(1:vcdim,g1,nf1)
                     vtmp(1:vcdim,g1,g2,1) = vtmp(1:vcdim,g1,g2,1) - vg1(1:vcdim)
                  enddo
               enddo
            else if(lf2 .and. resogpar%ndofs.eq.1) then
               do g2 = 1,ogpar1%gdofs(2)
                  vg2(1:vcdim) = resogpar%coeff(1:vcdim,g2,nf2)
                  do g1 = 1,ogpar1%gdofs(1)
                     vtmp(1:vcdim,g1,g2,1) = vtmp(1:vcdim,g1,g2,1) - vg2(1:vcdim)
                  enddo
               enddo
            else
               if(resogpar%lkfirst)then
                  do g2 = 1,ogpar1%gdofs(2)
                     do g1 = 1,ogpar1%gdofs(1)
                        vtmp(1:vcdim,g1,g2,1) = vtmp(1:vcdim,g1,g2,1) - vorig(1:vcdim)
                     enddo
                  enddo
               endif
            endif

            resogpar => resogpar%next
         enddo

! Perform SVD on the residue matrix, vtmp.
         gdf1 = ogpar1%gdofs(1)
         gdf2 = ogpar1%gdofs(2)
         mingdf = min(gdf1,gdf2)
         maxgdf = max(gdf1,gdf2)
         lwork = max(3*mingdf+maxgdf,5*mingdf)

         allocate(sigma(mingdf))
         if(vcdim.eq.1)then
            allocate(umat(gdf1,gdf1))
            allocate(vmat(gdf2,gdf2))
            allocate(work(lwork))
            call dgesvd('S','S',gdf1,gdf2,vtmp,gdf1,sigma,umat,gdf1,&
                 vmat,gdf2,work,lwork,info)
         else
            allocate(cumat(gdf1,gdf1))
            allocate(cvmat(gdf2,gdf2))
            allocate(cwork(lwork))
            allocate(work(5*mingdf))
            allocate(vcmp(gdf1,gdf2))
            do g2 = 1,gdf2
               do g1 = 1,gdf1
                  vcmp(g1,g2) = cmplx(vtmp(1,g1,g2,1),vtmp(2,g1,g2,1),dop)
               enddo
            enddo
            call zgesvd('S','S',gdf1,gdf2,vcmp,gdf1,sigma,cumat,gdf1,&
                 cvmat,gdf2,cwork,lwork,work,info)
         endif
         
         if(info .ne. 0)then
            routine = 'ddgbtendeccalc'
            message = 'Failure of SVD'
            call errormsg
         endif

! Transfer the results of the SVD to the operator.
         sqrsig = sqrt(sigma(1))
         do g1 = 1,ogpar1%gdofs(1)
            if(vcdim.eq.1)then
               ogpar1%coeff(1,g1,1) = sqrsig * umat(g1,1)
            else
               ogpar1%coeff(1,g1,1) = sqrsig * dble(cumat(g1,1))
               ogpar1%coeff(2,g1,1) = sqrsig * aimag(cumat(g1,1))
            endif
         enddo
         do g2 = 1,ogpar1%gdofs(2)
            if(vcdim.eq.1)then
               ogpar1%coeff(1,g2,2) = sqrsig * vmat(1,g2)
            else
               ogpar1%coeff(1,g2,2) = sqrsig * dble(cvmat(1,g2))
               ogpar1%coeff(2,g2,2) = sqrsig * aimag(cvmat(1,g2))
            endif
         enddo

         resnrm = sqrt(dot_product(sigma(2:mingdf),sigma(2:mingdf)))

! Convergence of SVD may be different depending on whether we are dealing with
! intra- or inter-state terms.
         if(s.eq.s1)then
            svdconv = ddgbsing
         else
            svdconv = ddgbsingss1
         endif

         if(resnrm.ge.svdconv)then
            do n = 2,mingdf
               call ogparaddcpy(ogpar1,vcdim)
               if(ogpar1%lcorr)kcorridx = kcorridx + 1
               sqrsig = sqrt(sigma(n))
               do g1 = 1,ogpar1%gdofs(1)
                  if(vcdim.eq.1)then
                     ogpar1%coeff(1,g1,1) = sqrsig * umat(g1,n)
                  else
                     ogpar1%coeff(1,g1,1) = sqrsig * dble(cumat(g1,n))
                     ogpar1%coeff(2,g1,1) = sqrsig * aimag(cumat(g1,n))
                  endif
               enddo
               do g2 = 1,ogpar1%gdofs(2)
                  if(vcdim.eq.1)then
                     ogpar1%coeff(1,g2,2) = sqrsig * vmat(n,g2)
                  else
                     ogpar1%coeff(1,g2,2) = sqrsig * dble(cvmat(n,g2))
                     ogpar1%coeff(2,g2,2) = sqrsig * aimag(cvmat(n,g2))
                  endif
               enddo
               
               resnrm = sqrt(dot_product(sigma(n+1:mingdf),sigma(n+1:mingdf)))
               if(resnrm.lt.svdconv)exit
            enddo
         endif

      else if(nft.eq.3)then
! 3-D terms are constructed by formation of a tensor of the couplings
! at the gridpoints, followed by a decomposition into a sum of 
! outer product of 3 vectors, each of which gives the potential terms along 
! 1 DOF, the other the terms along the other. 
         nrmax = min(ogpar1%gdofs(1),ogpar1%gdofs(2),ogpar1%gdofs(3))
         allocate(lambda(nrmax))
         allocate(vtmp(vcdim,ogpar1%gdofs(1),ogpar1%gdofs(2),ogpar1%gdofs(3)))
         allocate(vres(vcdim,ogpar1%gdofs(1),ogpar1%gdofs(2),ogpar1%gdofs(3)))

! First get the value of the relevant GAP at the gridpoints.
         f(1:3) = ogpar1%dofs(1:3)
         gpoint = 0.0_dop
         do g3 = 1,ogpar1%gdofs(3)
            gpoint(f(3)) = ogpar1%gaport(g3,3)

            do g2 = 1,ogpar1%gdofs(2)
               gpoint(f(2)) = ogpar1%gaport(g2,2)

               do g1 = 1,ogpar1%gdofs(1)
                  gpoint(f(1)) = ogpar1%gaport(g1,1)
                  call potgapss1(gpoint,vtmp(1,g1,g2,g3),vcdim,s,s1,modus)
               enddo
            enddo
         enddo

! Next, subtract the contributions from the previous terms.
         resogpar => ogpar1
         call ogparrew(resogpar)

         do while(associated(resogpar%next))

! If looped term is not of the same state(s) as the term of interest, cycle.
            if(resogpar%kf .ne. s .or. resogpar%ki .ne. s1)then
               resogpar => resogpar%next
               cycle
            endif

            nf = 0
            lf = .false.
            do f1 = 1,3
               do n = 1,resogpar%ndofs
                  lf(f1) = resogpar%dofs(n) .eq. f(f1)
                  if(lf(f1)) then
                     nf(f1) = n
                     exit
                  endif
               enddo
            enddo
            ndf = count(lf(1:3))

! If another 3-body term, cycle. Should not have the same DOFs as the current term; if
! it does, there's an error.
            if(resogpar%ndofs .eq. 3) then
               if(ndf.eq.3)then
                  routine = 'ddgbtendeccalc'
                  message = 'Multiple 3rd-order terms of the same states and DOFs.'
                  call errormsg
               endif
               resogpar => resogpar%next
               cycle
            endif

            if(ndf.eq.2  .and. resogpar%ndofs.eq.2)then
               vg = 1.0_dop
               do g3 = 1,ogpar1%gdofs(3)
                  if(lf(3)) vg(1:vcdim,3) = resogpar%coeff(1:vcdim,g3,nf(3))
                  do g2 = 1,ogpar1%gdofs(2)
                     if(lf(2)) vg(1:vcdim,2) = resogpar%coeff(1:vcdim,g2,nf(2))
                     do g1 = 1,ogpar1%gdofs(1)
                        if(lf(1)) vg(1:vcdim,1) = resogpar%coeff(1:vcdim,g1,nf(1))
                        vtmp(1:vcdim,g1,g2,g3) = vtmp(1:vcdim,g1,g2,g3) &
                             - vg(1:vcdim,1)*vg(1:vcdim,2)*vg(1:vcdim,3)
                     enddo
                  enddo
               enddo

            else if(ndf.eq.1 .and. resogpar%ndofs.eq.1)then
               do g3 = 1,ogpar1%gdofs(3)
                  if(lf(3)) vg(1:vcdim,1) = resogpar%coeff(1:vcdim,g3,nf(3))
                  do g2 = 1,ogpar1%gdofs(2)
                     if(lf(2)) vg(1:vcdim,1) = resogpar%coeff(1:vcdim,g2,nf(2))
                     do g1 = 1,ogpar1%gdofs(1)
                        if(lf(1)) vg(1:vcdim,1) = resogpar%coeff(1:vcdim,g1,nf(1))
                        vtmp(1:vcdim,g1,g2,g3) = vtmp(1:vcdim,g1,g2,g3) - vg(1:vcdim,1)
                     enddo
                  enddo
               enddo

            else
               if(resogpar%lkfirst)then
                  do g3 = 1,ogpar1%gdofs(3)
                     do g2 = 1,ogpar1%gdofs(2)
                        do g1 = 1,ogpar1%gdofs(1)
                           vtmp(1:vcdim,g1,g2,g3) = vtmp(1:vcdim,g1,g2,g3) &
                                - vorig(1:vcdim)
                        enddo
                     enddo
                  enddo
               endif

            endif

            resogpar => resogpar%next
         enddo

! See what the initial residue is, then exit if it is sufficiently low already.
         resnrm = dnrm2(ogpar1%gdofs(1)*ogpar1%gdofs(2)*ogpar1%gdofs(3),&
                 vtmp,1)
         if(resnrm.lt.ddgbtens)then
            call ogpardelone(ogpar1)
            goto 1000
         endif

! Perform tensor decomposition on the residue matrix, vtmp.
! Initialise the three 2-D arrays which will make up the decomposition
! (the columns of each of which represent the vectors which are combined
! in the outer products), by performing an SVD on the matricised tensor.
! The final decomposition will be held in arrays a1/a2/a3.
! The procedure follows the pseudo-code in Kolda and Bader, SIAM Rev., 2009, 51, 455.

! Matricisation of the tensor.
         if(allocated(x1))deallocate(x1)
         if(allocated(x2))deallocate(x2)
         if(allocated(x3))deallocate(x3)
         allocate(x1(ogpar1%gdofs(1),ogpar1%gdofs(2)*ogpar1%gdofs(3)))
         allocate(x2(ogpar1%gdofs(2),ogpar1%gdofs(1)*ogpar1%gdofs(3)))
         allocate(x3(ogpar1%gdofs(3),ogpar1%gdofs(1)*ogpar1%gdofs(2)))
         
         gdx = 0
         do g3 = 1,ogpar1%gdofs(3)
            do g2 = 1,ogpar1%gdofs(2)
               gdx = gdx + 1
               do g1 = 1,ogpar1%gdofs(1)
                  x1(g1,gdx) = vtmp(1,g1,g2,g3)
               enddo
            enddo
         enddo

         gdx = 0
         do g3 = 1,ogpar1%gdofs(3)
            do g1 = 1,ogpar1%gdofs(1)
               gdx = gdx + 1
               do g2 = 1,ogpar1%gdofs(2)
                  x2(g2,gdx) = vtmp(1,g1,g2,g3)
               enddo
            enddo
         enddo

         gdx = 0
         do g2 = 1,ogpar1%gdofs(2)
            do g1 = 1,ogpar1%gdofs(1)
               gdx = gdx + 1
               do g3 = 1,ogpar1%gdofs(3)
                  x3(g3,gdx) = vtmp(1,g1,g2,g3)
               enddo
            enddo
         enddo

! Perform SVD on each of the matrix representations of the tensor in order
! to get a starting point for the decomposition.
! The starting guesses are the first few left eigenvectors of the matrices.
         if(allocated(xtmp))deallocate(xtmp)
         gdf1 = ogpar1%gdofs(1)
         gdf2 = ogpar1%gdofs(2)*ogpar1%gdofs(3)
         allocate(xtmp(gdf1,gdf2))
         xtmp = x1
         mingdf = min(gdf1,gdf2)
         maxgdf = max(gdf1,gdf2)
         lwork = max(3*mingdf+maxgdf,5*mingdf)
         if(allocated(sigma))deallocate(sigma)
         if(allocated(umat))deallocate(umat)
         if(allocated(vmat))deallocate(vmat)
         if(allocated(work))deallocate(work)
         allocate(sigma(mingdf))
         allocate(umat(gdf1,gdf1))
         allocate(vmat(gdf2,gdf2))
         allocate(work(lwork))
         call dgesvd('S','N',gdf1,gdf2,xtmp,gdf1,sigma,umat,gdf1,&
              vmat,gdf2,work,lwork,info)


         if(allocated(a1))deallocate(a1)
         gdfa(1) = gdf1
         allocate(a1(gdfa(1),nrmax))
         a1(1:gdfa(1),1:nrmax) = umat(1:gdfa(1),1:nrmax)

         if(allocated(xtmp))deallocate(xtmp)
         gdf1 = ogpar1%gdofs(2)
         gdf2 = ogpar1%gdofs(1)*ogpar1%gdofs(3)
         allocate(xtmp(gdf1,gdf2))
         xtmp = x2
         mingdf = min(gdf1,gdf2)
         maxgdf = max(gdf1,gdf2)
         lwork = max(3*mingdf+maxgdf,5*mingdf)
         if(allocated(sigma))deallocate(sigma)
         if(allocated(umat))deallocate(umat)
         if(allocated(vmat))deallocate(vmat)
         if(allocated(work))deallocate(work)
         allocate(sigma(mingdf))
         allocate(umat(gdf1,gdf1))
         allocate(vmat(gdf2,gdf2))
         allocate(work(lwork))
         call dgesvd('S','N',gdf1,gdf2,xtmp,gdf1,sigma,umat,gdf1,&
              vmat,gdf2,work,lwork,info)


         if(allocated(a2))deallocate(a2)
         gdfa(2) = gdf1
         allocate(a2(gdfa(2),nrmax))
         a2(1:gdfa(2),1:nrmax) = umat(1:gdfa(2),1:nrmax)

         if(allocated(xtmp))deallocate(xtmp)
         gdf1 = ogpar1%gdofs(3)
         gdf2 = ogpar1%gdofs(1)*ogpar1%gdofs(2)
         allocate(xtmp(gdf1,gdf2))
         xtmp = x3
         mingdf = min(gdf1,gdf2)
         maxgdf = max(gdf1,gdf2)
         lwork = max(3*mingdf+maxgdf,5*mingdf)
         if(allocated(sigma))deallocate(sigma)
         if(allocated(umat))deallocate(umat)
         if(allocated(vmat))deallocate(vmat)
         if(allocated(work))deallocate(work)
         allocate(sigma(mingdf))
         allocate(umat(gdf1,gdf1))
         allocate(vmat(gdf2,gdf2))
         allocate(work(lwork))
         call dgesvd('S','N',gdf1,gdf2,xtmp,gdf1,sigma,umat,gdf1,&
              vmat,gdf2,work,lwork,info)


         if(allocated(a3))deallocate(a3)
         gdfa(3) = gdf1
         allocate(a3(gdfa(3),nrmax))
         a3(1:gdfa(3),1:nrmax) = umat(1:gdfa(3),1:nrmax)

         if(allocated(sigma))deallocate(sigma)
         if(allocated(umat))deallocate(umat)
         if(allocated(vmat))deallocate(vmat)
         if(allocated(work))deallocate(work)
         if(allocated(xtmp))deallocate(xtmp)

         if(allocated(ata))deallocate(ata)
         if(allocated(vinv))deallocate(vinv)
         if(allocated(ipiv))deallocate(ipiv)

         nr = nrmax
         nrold = nr

! Outer loop repeated until convergence or too many iterations.
         convloop: do iloop = 1,1000 

            dloop: do n = 1,3
! Allocate some arrays needed in the decomposition algorithm.
               if((iloop.eq.1 .and. n.eq.1) .or. nr.ne.nrold)then
                  if(allocated(vmat))deallocate(vmat)
                  if(allocated(ata))deallocate(ata)
                  if(allocated(vinv))deallocate(vinv)
                  if(allocated(ipiv))deallocate(ipiv)
                  if(allocated(work))deallocate(work)
                  allocate(vmat(nr,nr),ata(nr,nr),vinv(nr,nr),ipiv(nr))
                  lwork = 100*nr
                  allocate(work(lwork))
                  nrold = nr
               endif

! First step is to form the matrix, V, which is the Hadamard product of the 
! matrix products a_{i}a_{i}^{T} where i.ne.n.
               vmat = 1.0_dop
               do n1 = 1,3
                  if(n.eq.n1)cycle
                  if(n1.eq.1)then
                     apt => a1(1:gdfa(1),1:nr)
                  else if(n1.eq.2)then
                     apt => a2(1:gdfa(2),1:nr)
                  else
                     apt => a3(1:gdfa(3),1:nr)
                  endif
                  call dgemm('T','N',nr,nr,gdfa(n1),1.0_dop,apt,gdfa(n1),apt,&
                       gdfa(n1),0.0_dop,ata,nr)

! Hadamard product.
                  do i = 1,nr
                     do j = 1,nr
                        vmat(i,j) = vmat(i,j) * ata(i,j)
                     enddo
                  enddo
               enddo

! Second step is to form the Moore-Penrose pseudo-inverse of V, which is
! just the normal inverse here.
               vinv = vmat
               call dgetrf(nr,nr,vinv,nr,ipiv,info)
               if(info.ne.0)then
                  routine = 'ddgbtendeccalc'
                  write(message,'(a,i5)')'Error in LU decomp of V:',info
                  call errormsg
               endif
               call dgetri(nr,vinv,nr,ipiv,work,lwork,info)
               if(info.ne.0)then
                  routine = 'ddgbtendeccalc'
                  write(message,'(a,i5)')'Error in inversion of V:',info
                  call errormsg
               endif

! Third step is to form the updated a_{n} by multiplying x_{n} by the 
! Khatri-Rao products of all a_{i} where i.ne.n, then multiplying by the 
! pseudo-inverse of V (going left to right).
               if(n.eq.1)then
                  krpd = gdfa(2)*gdfa(3)
               else if(n.eq.2)then
                  krpd = gdfa(1)*gdfa(3)
               else
                  krpd = gdfa(1)*gdfa(2)
               endif
               if(allocated(krp)) deallocate(krp)
               allocate(krp(krpd,nr))
               
               if(n.eq.1)then
                  apt => a3
                  apt1 => a2
               elseif(n.eq.2)then
                  apt => a3
                  apt1 => a1
               else
                  apt => a2
                  apt1 => a1
               endif

               call krprod(krp,krpd,nr,apt,apt1,size(apt,1),size(apt1,1))
               
               if(allocated(xtmp))deallocate(xtmp)
               if(n.eq.1)then
                  allocate(xtmp(ogpar1%gdofs(1),nr))
                  xtmp = matmul(x1,krp)
               else if(n.eq.2)then
                  allocate(xtmp(ogpar1%gdofs(2),nr))
                  xtmp = matmul(x2,krp)
               else
                  allocate(xtmp(ogpar1%gdofs(3),nr))
                  xtmp = matmul(x3,krp)
               endif

               if(n.eq.1)then
                  a1 = matmul(xtmp,vinv)
               else if (n.eq.2)then
                  a2 = matmul(xtmp,vinv)
               else
                  a3 = matmul(xtmp,vinv)
               endif
               
! Finally, normalise the columns of a_{n} and store the norms which will be
! used as coefficients of the decomposed terms.
               lzero = .false.
               do n1 = 1,nr
                  if(n.eq.1)then
                     lambda(n1) = sqrt(dot_product(a1(1:gdfa(n),n1),&
                          a1(1:gdfa(n),n1)))
                     if(lambda(n1).lt.1.0e-6_dop)then
                        lzero = .true.
                        nrtmp = n1-1
                        exit
                     endif
                     a1(1:gdfa(n),n1) = a1(1:gdfa(n),n1)/lambda(n1)
                  else if(n.eq.2)then
                     lambda(n1) = sqrt(dot_product(a2(1:gdfa(n),n1),&
                          a2(1:gdfa(n),n1)))
                     if(lambda(n1).lt.1.0e-6_dop)then
                        lzero = .true.
                        nrtmp = n1-1
                        exit
                     endif
                     a2(1:gdfa(n),n1) = a2(1:gdfa(n),n1)/lambda(n1)
                  else
                     lambda(n1) = sqrt(dot_product(a3(1:gdfa(n),n1),&
                          a3(1:gdfa(n),n1)))
                     if(lambda(n1).lt.1.0e-6_dop)then
                        lzero = .true.
                        nrtmp = n1-1
                        exit
                     endif
                     a3(1:gdfa(n),n1) = a3(1:gdfa(n),n1)/lambda(n1)
                  endif
               enddo

! Resize the arrays if we have encountered a zero column.
               if(lzero) nr=nrtmp

            enddo dloop

! Check for convergence.
            vres = vtmp

            do n1 = 1,nr
!               if(lambda(n1).lt.1.0e-8_dop)cycle
               do g3 = 1,ogpar1%gdofs(3)
                  vg3 = a3(g3,n1)*lambda(n1)
                  do g2 = 1,ogpar1%gdofs(2)
                     vg2 = vg3 * a2(g2,n1)!*lambda(n1)
                     do g1 = 1,ogpar1%gdofs(1)
                        vg1 = vg2 * a1(g1,n1)!*lambda(n1)
                        vres(1:vcdim,g1,g2,g3) = vres(1:vcdim,g1,g2,g3) &
                             - vg1(1:vcdim)
                     enddo
                  enddo
               enddo
            enddo

            resnrmold = resnrm
            resnrm = dnrm2(ogpar1%gdofs(1)*ogpar1%gdofs(2)*ogpar1%gdofs(3),&
                 vres,1)

            if(resnrm .lt. ddgbtens .or. (abs(resnrm-resnrmold) &
                 .lt. 1.0e-6_dop)) exit convloop

         enddo convloop

         do n1 = 1,nr
!            if(lambda(n1).lt.1.0e-8_dop)cycle
            if(n1.gt.1)then
               call ogparaddcpy(ogpar1,vcdim)
               if(ogpar1%lcorr)kcorridx = kcorridx + 1
            endif
            do g1 = 1,ogpar1%gdofs(1)
               ogpar1%coeff(1,g1,1) = lambda(n1) * a1(g1,n1)
            enddo
            do g2 = 1,ogpar1%gdofs(2)
!               ogpar1%coeff(g2,2) = lambda(2,n1) * a2(g2,n1)
               ogpar1%coeff(1,g2,2) = a2(g2,n1)
            enddo
            do g3 = 1,ogpar1%gdofs(3)
!               ogpar1%coeff(g3,3) = lambda(3,n1) * a3(g3,n1)
               ogpar1%coeff(1,g3,3) = a3(g3,n1)
            enddo
         enddo

1000     if(allocated(x1))deallocate(x1)
         if(allocated(x2))deallocate(x2)
         if(allocated(x3))deallocate(x3)
         if(associated(apt))nullify(apt)
         if(allocated(krp)) deallocate(krp)        

      else
         routine = 'ddgbtendeccalc'
         message = 'Wrong number of DOFs in DD-GB tensor decomposition operator'
         call errormsg
      endif

      if(allocated(vtmp))deallocate(vtmp)
      if(allocated(vres))deallocate(vres)
      if(allocated(vcmp))deallocate(vcmp)
      if(allocated(sigma))deallocate(sigma)
      if(allocated(umat))deallocate(umat)
      if(allocated(vmat))deallocate(vmat)
      if(allocated(cumat))deallocate(cumat)
      if(allocated(cvmat))deallocate(cvmat)
      if(allocated(work))deallocate(work)
      if(allocated(cwork))deallocate(cwork)
      if(allocated(xtmp))deallocate(xtmp)
      if(allocated(ata))deallocate(ata)
      if(allocated(vinv))deallocate(vinv)
      if(allocated(ipiv))deallocate(ipiv)
      if(allocated(lambda))deallocate(lambda)
      if(allocated(a1))deallocate(a1)
      if(allocated(a2))deallocate(a2)
      if(allocated(a3))deallocate(a3)
      if(allocated(vg1))deallocate(vg1)
      if(allocated(vg2))deallocate(vg2)
      if(allocated(vg3))deallocate(vg3)

      return
      end subroutine ddgbtendeccalc

!############################################################################
! Subroutine to calculate the Khatri-Rao product of two matrices.
! GWR 04/18
!############################################################################
      subroutine krprod(krp,krpd,nr,apt,apt1,dapt,dapt1)

      implicit none

      integer(long), intent(in) :: krpd,nr,dapt,dapt1
      real(dop), dimension(krpd,nr), intent(out) :: krp
      real(dop), dimension(dapt,nr), intent(in)  :: apt
      real(dop), dimension(dapt1,nr), intent(in) :: apt1

      integer(long) :: i,j,k,kdx

      if(krpd.ne.dapt*dapt1)then
         routine = 'krprod'
         write(message,'(a,2i8)') 'Incompatible dimensions of matrices.',&
              krpd,dapt*dapt1
         call errormsg
      endif

      do i = 1, nr
         kdx = 0
         do j = 1, dapt
            do k = 1, dapt1
               kdx = kdx + 1
               krp(kdx,i) = apt(j,i)*apt1(k,i)
            enddo
         enddo
      enddo

      return
      end subroutine krprod

!############################################################################
      end module funcgapmod
