!#######################################################################
      module funcklegmod

use decimal, only: long,dop
use op1lib,  only: zerovxd

      implicit none
      private
      public :: fdefkleg,funckleg,jpl
      contains

!#######################################################################
! 
! ifile = 5
!--------------------------------------------------------------------------
!  Following subroutines (j2kleg,jpl,jmin,jplmin,jpl2,jmin2,sJp,sJm,sJpk,sJmk)
!  calculate operators:
!
!  j^2, j(+), j(-), c(+)*j(+)+c(-)*j(-), j(+)*j(+), j(-)*j(-), sin*J(+)
!  sin*J(+), (sin*J(+)*k + k*sin*J(+))/2, (sin*J(-)*k + k*sin*J(-))/2 .
!
!  Here: c(+-)=sqrt(jtot*(jtot+1)-k*(k+-1)), (jtot is total angular momentum)
!  j(-) operator is given through j(+) :
!             j(-)(alpha,beta,k)=j(+)(beta,alpha,k-1)
!
!  SS 04/99, HDM 02/09 
!--------------------------------------------------------------------------
!#######################################################################

      subroutine fdefkleg(hopilab,hoplab,hopfile)

      implicit none

      integer(long), intent(out)                    :: hopfile,hopilab
      character(len=*), intent(in)                  :: hoplab

!-----------------------------------------------------------------------
! assign function number. (The ops Jp,Jm,Jx,Jy are defined in funcgrd.f)
! ifile = 5
!-----------------------------------------------------------------------
      if (hoplab .eq. 'j^2kleg') then
         hopilab = 1
      elseif (hoplab .eq. 'j_p') then
         hopilab = 2
      elseif (hoplab .eq. 'j_m') then
         hopilab = 3
      elseif (hoplab .eq. 'jpm') then
         hopilab = 4
      elseif (hoplab .eq. 'jp^2') then
         hopilab = 5
      elseif (hoplab .eq. 'jm^2') then
         hopilab = 6
      elseif (hoplab .eq. 'jpjm') then
         hopilab = 7
      elseif (hoplab .eq. 'jmjp') then
         hopilab = 8
      elseif (hoplab .eq. 'dth1_kleg') then
         hopilab = 9
      elseif (hoplab .eq. 'dth2_kleg') then
         hopilab = 9
      elseif (hoplab .eq. 'sJp') then
         hopilab = 10
      elseif (hoplab .eq. 'sJm') then
         hopilab = 11
      elseif (hoplab .eq. 'sJpk') then
         hopilab = 12
      elseif (hoplab .eq. 'sJmk') then
         hopilab = 13

!-----------------------------------------------------------------------
! end of if loop
!-----------------------------------------------------------------------
      endif

!-----------------------------------------------------------------------
! if label is assigned, assign hopfile no.
!-----------------------------------------------------------------------
      if (hopilab .ne. 0) hopfile=5

      return
      end subroutine fdefkleg

!--------------------------------------------------------------------------
!  Following subroutines (j2kleg,jpl,jmin,jplmin,jpl2,jmin2,sJp,sJm,sJpk,sJmk)
!  calculate operators:
!
!  j^2, j(+), j(-), c(+)*j(+)+c(-)*j(-), j(+)*j(+), j(-)*j(-), sin*J(+)
!  sin*J(+), (sin*J(+)*k + k*sin*J(+))/2, (sin*J(-)*k + k*sin*J(-))/2 .
!
!  Here: c(+-)=sqrt(jtot*(jtot+1)-k*(k+-1)), (jtot is total angular momentum)
!  j(-) operator is given through j(+) :
!             j(-)(alpha,beta,k)=j(+)(beta,alpha,k-1)
!
!  SS 04/99, HDM 02/09 
!--------------------------------------------------------------------------

      subroutine funckleg(hmat,gdim,ifunc,dif1mat,dif2mat, &
                          kdim,jtot,kmin)

      implicit none

      integer(long), intent(in)                          :: gdim,ifunc,kdim,kmin
      real(dop), intent(in)                              :: jtot
      real(dop), dimension(gdim,gdim,2*kdim), intent(in) :: dif1mat
      real(dop), dimension(gdim,gdim,2*kdim), intent(in) :: dif2mat
      real(dop), dimension(gdim,gdim,kdim), intent(out)  :: hmat

     select case (ifunc)

     case default
     write(6,'(/2a,i4)') 'The ifunc value is invalid in subroutine '// &
                        'htmsetupmod/funckleg,ifunc =', ifunc
      stop

     case (1)! j^2kleg (j^2 for Kleg/PLeg)
      call j2kleg(dif2mat,hmat,gdim,kdim)
      return

     case (2)! j_p (j+ for Kleg/PLeg)
      call jpl(dif1mat,hmat,gdim,kdim)
      return

     case (3)! j_m (j- for Kleg/PLeg)
      call jmin(dif1mat,hmat,gdim,kdim)
      return

     case (4)! jpm (jpm for Kleg/PLeg)
      call jplmin(jtot,dif1mat,hmat,gdim,kdim,kmin)
      return

     case (5)! jp^2 (j+^2 for Kleg/PLeg)
      call jpl2(dif1mat,hmat,gdim,kdim)
      return

     case (6)! jm^2 (j-^2 for Kleg/PLeg)
      call jmin2(dif1mat,hmat,gdim,kdim)
      return

     case (7)! jpjm (j+*j- for Kleg/PLeg)
      call jpljmin(dif1mat,hmat,gdim,kdim)
      return

     case(8)! jmjp (j-*j+ for Kleg/PLeg)
      call jminjpl(dif1mat,hmat,gdim,kdim)
      return

     case (9)! dth1 or dth2  (for Kleg). dif1mat(1,1,kdim+1)=dif3mat
      call dthkleg(dif1mat(1,1,kdim+1),hmat,gdim,kdim)
      return

     case (10)!  sJp = sin*J+ , dif4mat(1,1,kdim+1)=dif4mat=trafo
      call sjtotp(hmat,dif2mat(1,1,kdim+1),jtot,gdim,kdim,kmin)
      return

     case (11)!  sJm = sin*J- , dif4mat(1,1,kdim+1)=dif4mat=trafo
      call sjtotm(hmat,dif2mat(1,1,kdim+1),jtot,gdim,kdim,kmin)
      return

     case (12)!  sJpk = (sin*Jp*k + k*sin*Jp)/2
      call sjtotpk(hmat,dif2mat(1,1,kdim+1),jtot,gdim,kdim,kmin)
      return

     case (13)!  sJmk = (sin*Jm*k + k*sin*Jm)/2 
      call sjtotmk(hmat,dif2mat(1,1,kdim+1),jtot,gdim,kdim,kmin)
      return
  
     end select

      end subroutine funckleg

!#######################################################################

      subroutine j2kleg(dif2mat,hops2,gdim,kdim)

      implicit none

      integer(long)                                     :: i,j,k
      integer(long), intent(in)                         :: gdim,kdim
      real(dop), dimension(gdim,gdim,kdim), intent(in)  :: dif2mat
      real(dop), dimension(gdim,gdim,kdim), intent(out) :: hops2

      do k=1,kdim
         do j=1,gdim
            do i=1,gdim
               hops2(i,j,k)=-dif2mat(i,j,k)
            enddo
         enddo
      enddo

      return
      end subroutine j2kleg

!#######################################################################

      subroutine dthkleg(dif3mat,hops2,gdim,kdim)

      implicit none

      integer(long)                                     :: i,j,k
      integer(long), intent(in)                         :: gdim,kdim
      real(dop), dimension(gdim,gdim,kdim), intent(in)  :: dif3mat
      real(dop), dimension(gdim,gdim,kdim), intent(out) :: hops2

      do k=1,kdim
         do j=1,gdim
            do i=1,gdim
               hops2(i,j,k)=dif3mat(i,j,k)
            enddo
         enddo
      enddo

      return
      end subroutine dthkleg

!#######################################################################

      subroutine jpl(dif1mat,hops2,gdim,kdim)

      implicit none

      integer(long)                                     :: i,j,k
      integer(long), intent(in)                         :: gdim,kdim
      real(dop), dimension(gdim,gdim,kdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim), intent(out) :: hops2


      call zerovxd(hops2(1,1,kdim),gdim*gdim) ! zero hops2 for k=kdim

      do k=1,kdim-1
         do j=1,gdim
            do i=1,gdim
               hops2(i,j,k)=-dif1mat(i,j,k)
            enddo
         enddo
      enddo

      return
      end subroutine jpl

!#######################################################################

      subroutine jmin(dif1mat,hops2,gdim,kdim)

      implicit none

      integer(long)                                     :: i,j,k
      integer(long), intent(in)                         :: gdim,kdim
      real(dop), dimension(gdim,gdim,kdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim), intent(out) :: hops2


      call zerovxd(hops2,gdim*gdim) ! zero hops2 for k=1

      do k=2,kdim
         do j=1,gdim
            do i=1,gdim
               hops2(i,j,k)=-dif1mat(j,i,k-1)
            enddo
         enddo
      enddo

      return
      end subroutine jmin

!#######################################################################

      subroutine jplmin(jtot,dif1mat,hops2,gdim,kdim,kmin)

      implicit none

      integer(long)                                     :: i,j,k,k1
      integer(long), intent(in)                         :: gdim,kdim,kmin
      real(dop)                                         :: cpl
      real(dop), intent(in)                             :: jtot
      real(dop), dimension(gdim,gdim,kdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim), intent(out) :: hops2


      call zerovxd(hops2(1,1,kdim),gdim*gdim) ! zero hops2 for k=kdim

      do k=1,kdim-1
         k1=k+kmin-1
         if(k1.lt.jtot .and. -k1.lt.jtot+1.0_dop) then
            cpl=sqrt(jtot*(jtot+1)-k1*(k1+1))
         else
            cpl=0.0_dop
         endif
         do j=1,gdim
            do i=1,gdim
               hops2(i,j,k)=-cpl*dif1mat(i,j,k)
            enddo
         enddo
      enddo

      return
      end subroutine jplmin

!#######################################################################

      subroutine jpl2(dif1mat,hops2,gdim,kdim)

      implicit none

      integer(long)                                     :: i,j,k,l
      integer(long), intent(in)                         :: gdim,kdim
      real(dop), dimension(gdim,gdim,kdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim), intent(out) :: hops2

      call zerovxd(hops2,gdim*gdim*kdim)

      do k=1,kdim-2
         do j=1,gdim
            do i=1,gdim
               do l = 1,gdim
                  hops2(i,j,k) = hops2(i,j,k) + &
                  dif1mat(i,l,k+1)*dif1mat(l,j,k)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine jpl2

!#######################################################################

      subroutine jmin2(dif1mat,hops2,gdim,kdim)

      implicit none

      integer(long)                                     :: i,j,k,l
      integer(long), intent(in)                         :: gdim,kdim
      real(dop), dimension(gdim,gdim,kdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim), intent(out) :: hops2

      call zerovxd(hops2,gdim*gdim*kdim)

      do k=3,kdim
         do j=1,gdim
            do i=1,gdim
               do l = 1,gdim
                  hops2(i,j,k) = hops2(i,j,k) + &
                  dif1mat(l,i,k-2)*dif1mat(j,l,k-1)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine jmin2

!#######################################################################

      subroutine jpljmin(dif1mat,hops2,gdim,kdim)

      implicit none

      integer(long)                                     :: i,j,k,l
      integer(long), intent(in)                         :: gdim,kdim
      real(dop), dimension(gdim,gdim,kdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim), intent(out) :: hops2

      call zerovxd(hops2,gdim*gdim*kdim)

      do k=2,kdim
         do j=1,gdim
            do i=1,gdim
               do l = 1,gdim
                  hops2(i,j,k) = hops2(i,j,k) + dif1mat(i,l,k-1)*dif1mat(j,l,k-1)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine jpljmin

!#######################################################################

      subroutine jminjpl(dif1mat,hops2,gdim,kdim)

      implicit none

      integer(long)                                     :: i,j,k,l
      integer(long), intent(in)                         :: gdim,kdim
      real(dop), dimension(gdim,gdim,kdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim), intent(out) :: hops2

      call zerovxd(hops2,gdim*gdim*kdim)

      do k=1,kdim-1
         do j=1,gdim
            do i=1,gdim
               do l = 1,gdim
                  hops2(i,j,k) = hops2(i,j,k) + dif1mat(l,i,k)*dif1mat(l,j,k)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine jminjpl

!#######################################################################

      subroutine  sjtotp(hops2,trafo,jtot,gdim,kdim,kmin)

!     sin*J_+ 
!     For the B coefficients (bmp, bmm) see appendix 2, PhD-thesis Gatti

      implicit none

      integer(long)                                     :: j,k,m,g,gm,g1,g2
      integer(long), intent(in)                         :: gdim,kdim,kmin
      real(dop)                                         :: cmi,bmp,bmm
      real(dop), intent(in)                             :: jtot
      real(dop), dimension(gdim,gdim,kdim), intent(in)  :: trafo
      real(dop), dimension(gdim,gdim,kdim), intent(out) :: hops2


      call zerovxd(hops2,gdim*gdim*kdim)

      do k=2,kdim
         m=k+kmin-1
         if(m.lt.jtot+0.5_dop .and. -m.lt.jtot-0.5_dop) then
            cmi=sqrt(jtot*(jtot+1)-m*(m-1))
         else
            cmi=0.0_dop
         endif

         do g = 1, gdim
            j=g+abs(m)-1
            gm = g + abs(m) - abs(m-1)
!C This is for ibleg > 0, which is here not implemented.
!C          jmin = abs(m) + mod(abs(m)+ibleg,2)
!C          j = 2*(g-1) + jmin
            bmp = sqrt((j-m+1)*(j-m+2)/dble((2*j+1)*(2*j+3)))
            if(j .ge. 1  .and.  j+m .gt. 1 ) then
               bmm = -sqrt((j+m)*(j+m-1)/dble((2*j-1)*(2*j+1)))
            else
               bmm=0
            endif
            if(gm.lt.gdim) then
               do g2 = 1, gdim
                  do g1 = 1, gdim
                     hops2(g2,g1,k) = hops2(g2,g1,k) + &
                     trafo(gm+1,g2,k-1)*trafo(g,g1,k)*cmi*bmp
                  enddo
               enddo
!c              hops2(j+1,k-1;j,k) = cmi*bmp
            endif
            if(gm.gt.1) then
               do g2 = 1, gdim
                  do g1 = 1, gdim
                     hops2(g2,g1,k) = hops2(g2,g1,k) + &
                     trafo(gm-1,g2,k-1)*trafo(g,g1,k)*cmi*bmm
                  enddo
               enddo
!C             hops2(j-1,k-1;j,k) = cmi*bmm
            endif
         enddo
      enddo

      return
      end subroutine  sjtotp

!#######################################################################

      subroutine  sjtotm(hops2,trafo,jtot,gdim,kdim,kmin)

!     sin*J_- 
!     For the B coefficients (bpp, bpm) see appendix 2, PhD-thesis Gatti

      implicit none

      integer(long)                                     :: j,k,m,g,gp,g1,g2
      integer(long), intent(in)                         :: gdim,kdim,kmin
      real(dop)                                         :: cpl,bpp,bpm
      real(dop), intent(in)                             :: jtot
      real(dop), dimension(gdim,gdim,kdim), intent(in)  :: trafo
      real(dop), dimension(gdim,gdim,kdim), intent(out) :: hops2


      call zerovxd(hops2,gdim*gdim*kdim)

      do k=1,kdim-1
         m=k+kmin-1
         if(m.lt.jtot-0.5_dop .and. -m.lt.jtot+0.5_dop) then
            cpl=sqrt(jtot*(jtot+1.0_dop)-m*(m+1))
         else
            cpl=0.0_dop
         endif

         do g = 1, gdim
            j=g+abs(m)-1
            gp = g + abs(m) - abs(m+1)
!C This is for ibleg > 0, which is here not implemented.
!C          jmin = abs(m) + mod(abs(m)+ibleg,2)
!C          j = 2*(g-1) + jmin
            bpp = -sqrt((j+m+1)*(j+m+2)/dble((2*j+1)*(2*j+3)))
            if(j .ge. 1  .and.  j-m .gt. 1 ) then
               bpm = sqrt((j-m)*(j-m-1)/dble((2*j-1)*(2*j+1)))
            else
               bpm = 0
            endif
            if(gp.lt.gdim) then
               do g2 = 1, gdim
                  do g1 = 1, gdim
                     hops2(g2,g1,k) = hops2(g2,g1,k) + &
                     trafo(gp+1,g2,k+1)*trafo(g,g1,k)*cpl*bpp
                  enddo
               enddo
!c              hops2(j+1,k+1;j,k) = cpl*bpp
            endif
            if(gp.gt.1) then
               do g2 = 1, gdim
                  do g1 = 1, gdim
                     hops2(g2,g1,k) = hops2(g2,g1,k) + &
                     trafo(gp-1,g2,k+1)*trafo(g,g1,k)*cpl*bpm
                  enddo
               enddo
!C             hops2(j-1,k+1;j,k) = cpl*bpm
            endif
         enddo
      enddo

      return
      end subroutine  sjtotm

!#######################################################################

      subroutine  sjtotpk(hops2,trafo,jtot,gdim,kdim,kmin)

!     (sin*J_+ *k + k* sin*J_+)/2
!     For the B coefficients (bmp, bmm) see appendix 2, PhD-thesis Gatti

      implicit none

      integer(long)                                     :: j,k,m,g,gm,g1,g2
      integer(long), intent(in)                         :: gdim,kdim,kmin
      real(dop)                                         :: cmi,bmp,bmm
      real(dop), intent(in)                             :: jtot
      real(dop), dimension(gdim,gdim,kdim), intent(in)  :: trafo
      real(dop), dimension(gdim,gdim,kdim), intent(out) :: hops2


      call zerovxd(hops2,gdim*gdim*kdim)

      do k=2,kdim
         m=k+kmin-1
         if(m.lt.jtot+0.5_dop .and. -m.lt.jtot-0.5_dop) then
            cmi=sqrt(jtot*(jtot+1)-m*(m-1))
         else
            cmi=0.0_dop
         endif

         do g = 1, gdim
            j=g+abs(m)-1
            gm = g + abs(m) - abs(m-1)
!C This is for ibleg > 0, which is here not implemented.
!C          jmin = abs(m) + mod(abs(m)+ibleg,2)
!C          j = 2*(g-1) + jmin
            bmp = sqrt((j-m+1)*(j-m+2)/dble((2*j+1)*(2*j+3)))
            if(j .ge. 1  .and.  j+m .gt. 1 ) then
               bmm = -sqrt((j+m)*(j+m-1)/dble((2*j-1)*(2*j+1)))
            else
               bmm=0
            endif
            if(gm.lt.gdim) then
               do g2 = 1, gdim
                  do g1 = 1, gdim
                     hops2(g2,g1,k) = hops2(g2,g1,k) + &
                     trafo(gm+1,g2,k-1)*trafo(g,g1,k)*cmi*bmp*(m-0.5_dop)
                  enddo
               enddo
!c              hops2(j+1,k-1;j,k) = cmi*bmp*(m-0.5_dop)
            endif
            if(gm.gt.1) then
               do g2 = 1, gdim
                  do g1 = 1, gdim
                     hops2(g2,g1,k) = hops2(g2,g1,k) + &
                     trafo(gm-1,g2,k-1)*trafo(g,g1,k)*cmi*bmm*(m-0.5_dop)
                  enddo
               enddo
!C             hops2(j-1,k-1;j,k) = cmi*bmm*(m-0.5_dop)
            endif
         enddo
      enddo

      return
      end subroutine  sjtotpk

!#######################################################################

      subroutine  sjtotmk(hops2,trafo,jtot,gdim,kdim,kmin)

!     (sin*J_- *k + k* sin*J_-)/2
!     For the B coefficients (bpp, bpm) see appendix 2, PhD-thesis Gatti

      implicit none

      integer(long)                                     :: j,k,m,g,gp,g1,g2
      integer(long), intent(in)                         :: gdim,kdim,kmin
      real(dop)                                         :: cpl,bpp,bpm
      real(dop), intent(in)                             :: jtot
      real(dop), dimension(gdim,gdim,kdim), intent(in)  :: trafo
      real(dop), dimension(gdim,gdim,kdim), intent(out) :: hops2

      call zerovxd(hops2,gdim*gdim*kdim)
      do k=1,kdim-1
         m=k+kmin-1
         if(m.lt.jtot-0.5_dop .and. -m.lt.jtot+0.5_dop) then
            cpl=sqrt(jtot*(jtot+1.0_dop)-m*(m+1))
         else
            cpl=0.0_dop
         endif

         do g = 1, gdim
            j=g+abs(m)-1
            gp = g + abs(m) - abs(m+1)
!C This is for ibleg > 0, which is here not implemented.
!C          jmin = abs(m) + mod(abs(m)+ibleg,2)
!C          j = 2*(g-1) + jmin
            bpp = -sqrt((j+m+1)*(j+m+2)/dble((2*j+1)*(2*j+3)))
            if(j .ge. 1  .and.  j-m .gt. 1 ) then
               bpm = sqrt((j-m)*(j-m-1)/dble((2*j-1)*(2*j+1)))
            else
               bpm = 0
            endif
            if(gp.lt.gdim) then
               do g2 = 1, gdim
                  do g1 = 1, gdim
                     hops2(g2,g1,k) = hops2(g2,g1,k) + &
                     trafo(gp+1,g2,k+1)*trafo(g,g1,k)*cpl*bpp*(m+0.5_dop)
                  enddo
               enddo
!c              hops2(j+1,k+1;j,k) = cpl*bpp*(m+0.5_dop)
            endif
            if(gp.gt.1) then
               do g2 = 1, gdim
                  do g1 = 1, gdim
                     hops2(g2,g1,k) = hops2(g2,g1,k) + &
                     trafo(gp-1,g2,k+1)*trafo(g,g1,k)*cpl*bpm*(m+0.5_dop)
                  enddo
               enddo
!C             hops2(j-1,k+1;j,k) = cpl*bpm*(m+0.5_dop)
            endif
         enddo
      enddo

      return
      end subroutine  sjtotmk


      end module funcklegmod

