!#######################################################################
!            Function-DEF for Local control regime
!
! The funclct routines are to be found in mctdhlib/tdkoeffs.F. They
! are called only during propagation from calctdk and rely on 
! routines in mctdhlib
! ifile = 308
!#######################################################################

      subroutine fdeflct(ifunc,label,ifile,hopipar,hoprpar,maxhpar,&
                 hoppar,zpesopts,pesopts,hamlab,hamdim)

use decimal, only:long,dop
use global
use logdat

      implicit none

      integer(long)                                     :: ilbl,jlbl,np,n,nopt,nham,klbl,llbl,nham1,nham2 
      integer(long), intent(in)                         :: maxhpar,hamdim
      integer(long), intent(out)                        :: ifunc,ifile
      integer(long), dimension(*), intent(out)          :: hopipar
      integer(long), dimension(2*maxhpar,*), intent(in) :: zpesopts
      real(dop)                                         :: lambda
      real(dop), dimension(*), intent(out)              :: hoprpar
      real(dop), dimension(maxhpar,*), intent(in)       :: hoppar
      character(len=20)                                 :: lctopts
      character(len=*), intent(in)                      :: label
      character(len=*), dimension(*), intent(in)        :: pesopts,hamlab

! get argument list no.
! and end of operator label
      np=0
      ilbl=-1
      ilbl=index(label,'[')+1
      if (ilbl .gt. 1) then
         jlbl = index(label,']')-1
         read(label(ilbl:jlbl),*) np
         ilbl=index(label,'[')-1
      else
         ilbl=index(label,'{')+1
         if (ilbl .gt. 1) then
            jlbl = index(label,'}')-1
            read(label(ilbl:jlbl),*) np
            ilbl=index(label,'{')-1
         endif
      endif

      if (ilbl .lt. 0) ilbl=index(label,' ')-1


! now assign function number
! Note: hopipar(1) and hoprpar(1) are reserved for the exponent.
! The exponent was read in defanld (opfuncs/callanld.f).

      if (label(1:ilbl) .eq. 'lct_external') then
         if (np .gt.0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            hoprpar(4)=hoppar(3,np)
         else
            hoprpar(2)=0.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=1
      else if (label(1:ilbl) .eq. 'lct_elec') then
         if (np .gt.0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            hoprpar(4)=hoppar(3,np)
            hoprpar(5)=hoppar(4,np)
         else
            hoprpar(2)=0.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=2
      else if (label(1:ilbl) .eq. 'lct_vibr') then
         if (np .gt.0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            hoprpar(4)=hoppar(3,np)
            hoprpar(5)=hoppar(4,np)
         else
            hoprpar(2)=0.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=3
      else if (label(1:ilbl) .eq. 'lct_nucl') then
         if (np .gt.0) then
            hoprpar(2)=hoppar(1,np)
            hoprpar(3)=hoppar(2,np)
            hoprpar(4)=hoppar(3,np)
         else
            hoprpar(2)=0.0_dop
            hoprpar(3)=0.0_dop
         endif
         ifunc=4

      else if (label(1:ilbl) .eq. 'lct') then
         if (np .le.0) then
            routine = 'Funclct'
            message = 'No operator specified with LCT operator'
            call errormsg
         endif

! read operator label
         nopt=1
         klbl=1
         llbl=zpesopts(nopt,np)
         lctopts = pesopts(np)(klbl:llbl)
         nham = 0
         do n=1,hamdim
            if (lctopts .eq. hamlab(n)) nham=n
         enddo
         if (nham .eq. 0) then
            routine = 'Funclct'
            write(message,'(2a)')&
            'Operator specified with LCT operator not found',lctopts
            call errormsg
         endif
! read value of lambda
         nopt=2
         klbl=llbl+1
         llbl=zpesopts(nopt,np)
         if (llbl .eq. 0) then
            routine = 'Funclct'
            message =&
            'Value for lambda not specified for LCT operator'
            call errormsg
         endif
         lctopts = pesopts(np)(klbl:llbl)
         lambda = -999.0
         do n=1,npar
            if (lctopts .eq. apar(n)) lambda = rpar(n)
         enddo
         if (lambda .lt. -900.0) then
            routine = 'Funclct'
            write(message,'(2a)')&
            'Parameter for lambda in LCT operator not found :',lctopts
            call errormsg
         endif

         hopipar(2)=nham
         hoprpar(2)=lambda
!
! look to see if imaginary or real part
!
         hopipar(3)=0
         nopt=3
         klbl=llbl+1
         llbl=zpesopts(nopt,np)
         if (llbl .ne. 0) then
            if (pesopts(np)(klbl:klbl+1) .eq. 'im') then
               hopipar(3)=1
            else if (pesopts(np)(klbl:klbl+1) .eq. 're') then
               hopipar(3)=0
            else 
               routine = 'Funclct'
               write(message,'(2a)')&
                     'Unknown 3rd option for lct function'
               call errormsg
            endif
         endif

         ifunc=5

      else if (label(1:ilbl) .eq. 'lcttd') then
         if (np .le.0) then
            routine = 'Funclct'
            message = 'No operator specified with LCTTD operator'
            call errormsg
         endif

! read operator label
         nopt=1
         klbl=1
         llbl=zpesopts(nopt,np)
         lctopts = pesopts(np)(klbl:llbl)
         nham = 0
         do n=1,hamdim
            if (lctopts .eq. hamlab(n)) nham=n
         enddo
         if (nham .eq. 0) then
            routine = 'Funclct'
            write(message,'(2a)')&
            'Operator specified with LCT operator not found',lctopts
            call errormsg
         endif

!
! look to see if imaginary or real part
!
         hopipar(3)=0
         nopt=2
         klbl=llbl+1
         llbl=zpesopts(nopt,np)
         if (llbl .ne. 0) then
            if (pesopts(np)(klbl:klbl+1) .eq. 'im') then
               hopipar(3)=1
            else if (pesopts(np)(klbl:klbl+1) .eq. 're') then
               hopipar(3)=0
            else 
               routine = 'Funclct'
               write(message,'(2a)')&
                     'Unknown 2nd option for lct function'
               call errormsg
            endif
         endif

! read second operator label
         nopt=3
         klbl=llbl+1
         llbl=zpesopts(nopt,np)
         lctopts = pesopts(np)(klbl:llbl)
         nham1 = 0
         do n=1,hamdim
            if (lctopts .eq. hamlab(n)) nham1=n
         enddo
         if (nham1 .eq. 0) then
            routine = 'Funclct'
            write(message,'(2a)')&
            'Second operator specified with LCTTD operator not found',lctopts
            call errormsg
         endif

! read third operator label
         nopt=4
         klbl=llbl+1
         llbl=zpesopts(nopt,np)
         lctopts = pesopts(np)(klbl:llbl)
         nham2 = 0
         do n=1,hamdim
            if (lctopts .eq. hamlab(n)) nham2=n
         enddo
         if (nham2 .eq. 0) then
            routine = 'Funclct'
            write(message,'(2a)')&
            'Third operator specified with LCTTD operator not found',lctopts
            call errormsg
         endif

! read final value of expectation
!        nopt=5
!        klbl=llbl+1
!        llbl=zpesopts(nopt,np)
!        if (llbl .eq. 0) then
!           routine = 'Funclct'
!           message =&
!           'Final value for expectation not specified for LCTTD operator'
!           call errormsg
!        endif
!        lctopts = pesopts(np)(klbl:llbl)
!        efinal = -999.0
!        do n=1,npar
!           if (lctopts .eq. apar(n)) efinal = rpar(n)
!        enddo
!        if (lambda .lt. -900.0) then
!           routine = 'Funclct'
!           write(message,'(2a)')&
!           'Parameter for final expectation in LCT operator not found :',lctopts
!           call errormsg
!        endif

!!read time for final value of expectation
!        nopt=6
!        klbl=llbl+1
!        llbl=zpesopts(nopt,np)
!        if (llbl .eq. 0) then
!           routine = 'Funclct'
!           message =&
!           'Final value for expectation not specified for LCTTD operator'
!           call errormsg
!        endif
!        lctopts = pesopts(np)(klbl:llbl)
!        tfinal = -999.0
!        do n=1,npar
!           if (lctopts .eq. apar(n)) tfinal = rpar(n)
!        enddo
!        if (tfinal .lt. -900.0) then
!           routine = 'Funclct'
!           write(message,'(2a)')&
!           'Time for final expectation in LCT operator not found :',lctopts
!           call errormsg
!        endif
         hopipar(2)=nham
         hopipar(4)=nham1
         hopipar(5)=nham2
!        hoprpar(2)=efinal
!        hoprpar(3)=tfinal

         ifunc=6

      else if (label(1:ilbl) .eq. 'lct_2d') then
         if (np .le.0) then
            routine = 'Funclct'
            message = 'No operator specified with LCT_2D operator'
            call errormsg
         endif


! read operator label
         nopt=1
         klbl=1
         llbl=zpesopts(nopt,np)
         lctopts = pesopts(np)(klbl:llbl)
         nham = 0
         do n=1,hamdim
            if (lctopts .eq. hamlab(n)) nham=n
         enddo
         if (nham .eq. 0) then
            routine = 'Funclct'
            write(message,'(2a)')&
            'Operator specified with LCT_2D operator not found',lctopts
            call errormsg
         endif
! read value of lambda
         nopt=2
         klbl=llbl+1
         llbl=zpesopts(nopt,np)
         if (llbl .eq. 0) then
            routine = 'Funclct'
            message =&
            'Value for lambda not specified for LCT_2D operator'
            call errormsg
         endif
         lctopts = pesopts(np)(klbl:llbl)
         lambda = -999.0
         do n=1,npar
            if (lctopts .eq. apar(n)) lambda = rpar(n)
         enddo
         if (lambda .lt. -900.0) then
            routine = 'Funclct'
            write(message,'(2a)')&
            'Parameter for lambda in LCT operator not found :',lctopts
            call errormsg
         endif

!
! look to see if imaginary or real part
!
         hopipar(3)=0
         nopt=3
         klbl=llbl+1
         llbl=zpesopts(nopt,np)
         if (llbl .ne. 0) then
            if (pesopts(np)(klbl:klbl+1) .eq. 'im') then
               hopipar(3)=1
            else if (pesopts(np)(klbl:klbl+1) .eq. 're') then
               hopipar(3)=0
            else 
               routine = 'Funclct'
               write(message,'(2a)')&
                     'Unknown 3rd option for lct function'
               call errormsg
            endif
         endif

! read second expectation operator label
         nopt=4
         klbl=llbl+1
         llbl=zpesopts(nopt,np)
         lctopts = pesopts(np)(klbl:llbl)
         nham1 = 0
         do n=1,hamdim
            if (lctopts .eq. hamlab(n)) nham1=n
         enddo
         if (nham1 .eq. 0) then
            routine = 'Funclct'
            write(message,'(2a)')&
            'Second Expectation operator specified with LCT_2D operator not found',lctopts
            call errormsg
         endif
         hopipar(2)=nham
         hopipar(4)=nham1
         hoprpar(2)=lambda

         ifunc=7
      endif

! if label is assigned, assign ifile no.
      if (ifunc .ne. 0) ifile=308

      return
      end subroutine fdeflct


