!#######################################################################
      module funcqch1mod

use decimal, only: dop, long
use op1lib,  only: cpvxd, unitqxd
use global
use channels
use griddatmod
use dirdyn
use logdat, only: errormsg
use operdef
use gwptrj
use runopermod
use channels
use splinepotmod
 
use dd_db, only:ddq2x,eckart,ddpes2x,ddpes2q,dddb_rd


      implicit none
      private
      public :: defqchem1,vqchem1,qchem1info
      contains

!#######################################################################
! This routine interprets the options of the interface to the
! external quantum chemistry program (V = qchem{...} or V = external{...}
! in .op file) for direct dynamics calculations.
! It is called from defmuld, which is called from assignhlab.
! In the latter, all relevant information passed in arguments comes from
! and goes to common blocks that are defined in operdef.inc.
!
! ifile=207 for non-potentials, e.g. dipole moment, that do not have derivatives
!
!#######################################################################

      subroutine defqchem1(ifunc,label,ifile,ierr)

      implicit none

      integer(long)                                     :: ilbl,jlbl,np
      integer(long), intent(out)                        :: ifunc,ifile,ierr
      character(len=*), intent(in)                      :: label

      ifunc=0
      ifile=0
      ierr=0

!
! if a label starts with "qchem" it must be added to defqchem (funcqchemmod)
! to be ignored
!
      if (label .eq. 'qchemdipx') then
         ifunc=101
      else if (label .eq. 'qchemdipy') then
         ifunc=102
      else if (label .eq. 'qchemdipz') then
         ifunc=103
      else if (label .eq. 'qchemdip') then
         ifunc=104
      else if (label .eq. 'qchemtrj') then
         ifunc=105
      endif
!
! if option passed from .op file as option list qchem{np}, 
! extract option list number and read options
! see funcqchem for how to extract options
!
      ilbl=index(label,'{')+1
      jlbl=index(label,'}')-1
      if (ilbl .gt. 1) then
         read(label(ilbl:jlbl),*) np
      endif
!
! if function assigned, assign ifile no.
!
      if (ifunc .ne. 0) ifile=207

      return
      end subroutine defqchem1


!#######################################################################
!
! This routine gets the value of a potential at a grid point from a
! DB of energies.
!
! The point is contained in gpoint. 
!
!#######################################################################

      subroutine vqchem1 (v,gpoint,ifunc,si,sf)


      implicit none

      integer(long), intent(in)               :: ifunc,si,sf
      integer(long)                           :: n

      real(dop), intent(out)                                  :: v
      real(dop), dimension(ndofddpes)                         :: xgpoint
      real(dop), dimension(ndofddpes), intent(in)             :: gpoint
      real(dop), dimension(nddstate,nddstate)                 :: ddv
      real(dop), dimension(ndofddpes,nddstate,nddstate)       :: ddder1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate) :: ddder2
      real(dop), dimension(3,nddstate,nddstate)               :: dddip
      real(dop)                                               :: time

!-----------------------------------------------------------------------
! get relative atomic masses from refdb.dat file
!
! If trafo is used, dynamics are in internal coordinates so no 
! Eckart trafo needed.
!
! Comparisons are to be done in system coords. Previously the system
! was transformed to Cartesians and all comparsins done in these
! coordinates. The extraplation in Cartesians followed by transformation
! back  
! if Cartesians, xgpoint are original Cartesians, gpoint are
! rotated to Eckart frame
!-----------------------------------------------------------------------
      if (ifunc .ne. 105) then
         if(lddtrans)then
            call ddq2x(gpoint,xgpoint)
         else
            xgpoint = gpoint
         endif
         call dddb_rd(ddv,ddder1,ddder2,dddip,xgpoint,.true.,.false.,.false.)
      endif

!-----------------------------------------------------------------------
! Select value of "potential" to return
!-----------------------------------------------------------------------
      if (ifunc .eq. 101) then
         v = dddip(1,sf,si)
      else if (ifunc .eq. 102) then
         v = dddip(2,sf,si)
      else if (ifunc .eq. 103) then
         v = dddip(3,sf,si)
      else if (ifunc .eq. 104) then
         v = dddip(1,sf,si)*dddip(1,sf,si) + &
             dddip(2,sf,si)*dddip(2,sf,si) + &
             dddip(3,sf,si)*dddip(3,sf,si) 
         v=dsqrt(v)
      else if (ifunc .eq. 105) then
         n = curr_trj
         time = curr_time
         call splint(gwptrjtdat(1,n),gwptrjedat(1,n), &
                     gwptrjespl(1,n),gwptrjnp(n),time,v)

      endif

      return
      end subroutine vqchem1

!##########################################################################
!
! output info to log file
!
!##########################################################################

      subroutine qchem1info (hl)

      implicit none

      integer(long), intent(in)                  :: hl
!
! log information (more is written into op.log)
!
      if (hopilab(hl) .eq. 105) then
         write(ilog,'(a)') 'Potential from previously calculated trajectory'
      else 
         write(ilog,'(a)') 'External QChem program used for "potential"'
      endif

      return
      end subroutine qchem1info


      end module funcqch1mod

