!#######################################################################
! This module interfaces with a surface defined through a modified Shepard
! interpolation, as developed by Mick Collins and coworkers.
! Typically this would be a Grow surface.
!
!  At the moment we only have the 3D-Cartesian coordinates for NOCl
! implemented.  There needs to be a general "to Cartesian" transformation
! framework done.  (Coordinate transform examples in funcsrf.F)
!
! We are also currently not using the nested neighbour list infrastructure
!
! ifile=206
!
!#######################################################################

module funcshep
use decimal, only: long, dop
use shepard
use POTmodule
use channels
use datenmod
use global
implicit none

save
private
public::defshep,shepardinfo,vshepard,dvshepard
public::doforce

integer,parameter::InnerListSize=30000
integer,parameter::OuterListSize=35000
type(POTobject)::potfile
type(POTinstance)::centres
real(dop),dimension(:,:),allocatable::transf,transb,d2vdm
real(dop),dimension(:),allocatable::xcoo,dvdm
integer(long),dimension(:),allocatable::imodes
logical::nmodes
integer(long)::nCart,nnm
real(dop), parameter                       :: invcm=1.23985e-4_dop, angst=0.52917725_dop

contains

subroutine defshep(ifunc,label,ifile,pesopts,zpesopts,ierr)

use griddatmod
use logdat
use InterpTrajData
use xvlib,   only: xvxxddo, xvixddo
use ioqc,  only: getintyp,getxcoo,getnm,nm2xmat
use dvrdatmod, only: ndof

integer,intent(out)::ifunc,ifile,ierr
character(len=*),intent(in)::label
      character(len=*), dimension(*), intent(in)        :: pesopts
      integer(long), dimension(2*maxhpar,*), intent(in) :: zpesopts

real(dop),dimension(:),allocatable::fmass,freq
integer(long),dimension(:),allocatable::iatsym,iatiso
integer::i,i1,i2,np,iau
integer(long)::ifiletyp
character(len=5),dimension(:),allocatable::Astr
character(len=c5)::fname
character(len=20)::str20

ifunc=0
ifile=0
ierr=0
if ((label(1:4)/='grow').and.(label(1:7)/='shepard')) return
if (dop/=8) stop 'dop other than 8 breaks Shepard code'
Shepard2D=.false.
if (len(label)>=9) then
  Shepard2D=(label(1:9)=='shepard2d')
endif

 !  set up interpolated surface
open(unit=igout,file=name(1:laenge)//'/OUT_SHEPARD')
 !  read in the parameters which define the system
call readccm(potfile,'IN_SYSTEM',.false.)
allocate(LastGpoint(potfile%natom*3))

nmodes=.false.
i1=index(label,'{')+1
i2=index(label,'}')-1
if (i1>1) then
  read(label(i1:i2),*)np
   ! nmodes,<g09 freq output>,comma seperated list of modes
  if (pesopts(np)(1:zpesopts(1,np))=='nmodes2D') then
    nmodes=.true.
     ! This is a functional duplicate of a subset of rdddtrfile.
     ! Repeating the functionality here is cleaner than having 
     ! the Shepard code depend on all the DD code for the sake
     ! of one function.
    fname=pesopts(np)(zpesopts(1,np)+1:zpesopts(2,np))
    write(ilog,'(/,2a)') 'Opening nmode file: ',trim(fname)
    open(unit=igin,file=trim(fname))
    call getintyp(igin,ifiletyp,trim(fname),ilog)
    write(ilog,'(15x,a,i5)') 'Ab initio type:',ifiletyp
    iau=1
    str20='input'
    nCart=potfile%natom*3
    nnm=nCart-6
    allocate(iatsym(nCart))
    allocate(iatiso(nCart))
    allocate(Astr(nCart))
    allocate(fmass(nCart))
    allocate(transf(nCart,nCart))
    allocate(transb(nCart,nCart))
    allocate(freq(nCart))
    allocate(xcoo(nCart))
    call getxcoo(igin,xcoo,ifiletyp,iau,str20,iatsym, &
           iatiso,nCart,maxsta,nnm,ilog)
    iau=1
    call getnm(igin,ifiletyp,Astr,transf,freq,iatsym,iau,&
           nCart,nCart,1,nnm,ilog,1,.true.)
    freq(1:nnm)=abs(freq(1:nnm))
    close(igin)
    call nm2xmat(transf,transb,iatsym,freq,iau,nCart,nCart,nnm,ilog)
      do i=1,nnm
         call xvixddo(angst,transf(1,i),nCart)
      enddo
      do i=1,nCart
         call xvxxddo(angst,transb(1,i),nnm)
      enddo
    deallocate(iatsym,iatiso,Astr,fmass,freq)
     ! Now read list of modes
     ! Remember that one of the ndof is the electronic state
    allocate(imodes(ndof-1))
    do i=1,ndof-1
      read(pesopts(np)(zpesopts(i+1,np)+1:zpesopts(i+2,np)),*)imodes(i)
    enddo
    allocate(dvdm(ndof-1))
    allocate(d2vdm(ndof-1,ndof-1))
  endif
endif

LastGpoint=0
 !   set up the bond definitions
call readz(potfile)
!call readz(potfile,'IN_ZMA')
 !  include permutational symmetry
call makebp(potfile,'IN_ATOMPERMS')
 !   read in run control parameters
call cntl(potfile,'IN_CNT',.true.)
 !   read in the initial table of Taylor series potential coefficients 
call readt(potfile,'POT')
if (potfile%ipart.eq.2) call readcon(potfile,'CON')
i=potfile%ipow
if (potfile%ipart==2) i=potfile%lowp
potfile%vtlim=max(1.0e-10_8, &
             exp(-log(huge(1.0_8)/real(potfile%ndata*potfile%ngroup,8)) &
                                        /real(i,8)))*100
write(igout,*)'VTlim:',potfile%vtlim
close(igout)

call AllocateMols(centres,1,InnerListSize,OuterListSize,potfile)

ifile=206
lshepinit=.true.
end subroutine defshep

!#######################################################################
!
! This routine gets the value of a potential at a geometry
!
! The point is contained in gpoint. 
!
!#######################################################################

      subroutine vshepard (vpot,gpoint,ndof1,rms,totsum)

use InterpTrajData
use randommod, only: randuniform

integer,intent(in)::ndof1
real(kind=dop),intent(out)::vpot,rms,totsum
real(kind=dop),dimension(ndof1),intent(in)::gpoint

      real(kind=8),parameter::mO=15.99491_8
      real(kind=8),parameter::mN=14.00307_8
      real(kind=8),parameter::mCl=34.96885_8
      integer::i,j,k
      real(kind=8)::x
      character(len=7)::str
      logical,save::first=.true.

! At the moment the argument to the grow routines is a dummy.  
! An obvious improvement is to have that the Gaussian number, 
! to allow for inner and outer neighbour lists
!write(*,*)'VShepard'
!write(*,*)real(gpoint)
!write(*,*)ndof1

! To allow restarts:
      if (.not.lshepinit) then
        str='shepard'
        stop 'Restarts disabled until I work out how to preserve options'
!        call defshep(if,str,ifl,ie)
      endif
      if (first) then
        if (lsheptraj) then
          write(itout,*)' File contains sampled trajectory configurations'
          write(itout,*)' Powers used in weight fn ; min # points to be added'
          write(itout,*)' plus the average confidence radius**2, and ipart'
          write(itout,1122)potfile%lowp,potfile%ipow, nsel, &
                        potfile%avrads,potfile%ipart,potfile%vmin
1122    format(3i5,4x,e15.6,4x,i5,4x,e17.9)
          write(itout,*)' Beginning trajectory configurations...'
        endif
        first=.false.
      endif

 ! Mick's wrong way round indexing: c(traj,atom,dof)
if (nmodes) then
  centres%c(1,:,1)=xcoo(1:nCart:3)
  centres%c(1,:,2)=xcoo(2:nCart:3)
  centres%c(1,:,3)=xcoo(3:nCart:3)
   ! dx=matmul(tranf,gpoint)
  do i=1,ndof1-1
    centres%c(1,:,1)=centres%c(1,:,1)+transf(1:nCart:3,imodes(i))*gpoint(i)
    centres%c(1,:,2)=centres%c(1,:,2)+transf(2:nCart:3,imodes(i))*gpoint(i)
    centres%c(1,:,3)=centres%c(1,:,3)+transf(3:nCart:3,imodes(i))*gpoint(i)
  enddo
!write(995)transf(1:nCart,1:nnm)
!write(995)transb(1:nnm,1:nCart)
!write(995)xcoo
   ! Convert to bohr
  centres%c(1,:,:)=centres%c(1,:,:)/angst
else
  do i=1,potfile%natom
    centres%c(1,i,:)=gpoint((i-1)*3+1:i*3)
  enddo
endif
call intern(centres,potfile,1)
nloop=0
if(potfile%ipart.eq.1)then
  call neighbour(centres,potfile,1)
  call dvdi(centres,potfile,1)
else
  call neighbour1a(centres,potfile,1)
!  call neighbour1(centres,potfile,1)
!  call neighbour2(centres,potfile,1)
!  call neighbour3(centres,potfile,1)
  call dvdir(centres,potfile,1)
endif

  ! if required, dump the geom to TOUT
  ! here we need a test against dump frequency
if (lsheptraj) then
  x=randuniform()
  if (x<sample) then
    do i=1,potfile%natom
      write(itout,*)centres%c(1,i,:)
    enddo
    if (lshepgp) then
      write(itout,*)centres%en(1),centres%rms(1),centres%totsum(1),gpoint
    else
      write(itout,*)centres%en(1),centres%rms(1),centres%totsum(1)
    endif
  endif
endif

if ((nlowstop/=0).and.(centres%en(1)<potfile%vmin-0.0005_8)) then
  do i=1,3
    write(itout,*)centres%c(1,i,:)
  enddo
  write(itout,*)centres%en(1),centres%rms(1),centres%totsum(1)
  stop 'low energy'
endif

vpot=centres%en(1)
rms=centres%rms(1)
totsum=centres%totsum(1)

if (.not.doforce) return
!   evaluate the cartesian derivatives from the internal 
!   derivatives
call dvdx(centres,potfile,1)
!write(*,*)real(centres%d2v(1,1:48:3,:))
!write(*,*)real(centres%d2v(1,2:48:3,:))
LastGpoint=gpoint

! Convert from derivs w.r.t. Cartesians to derivatives w.r.t. normal modes
if (nmodes) then
  centres%dv(1,:,:)=centres%dv(1,:,:)/angst
  centres%d2v(1,:,:)=centres%d2v(1,:,:)/angst**2
  do i=1,ndof1-1
    dvdm(i)=sum(transf(1:nCart:3,imodes(i))*centres%dv(1,:,1))
    dvdm(i)=dvdm(i)+sum(transf(2:nCart:3,imodes(i))*centres%dv(1,:,2))
    dvdm(i)=dvdm(i)+sum(transf(3:nCart:3,imodes(i))*centres%dv(1,:,3))
  enddo
  d2vdm=0
  do i=1,ndof1-1
    do j=1,ndof1-1
      do k=1,nCart
        d2vdm(i,j)=d2vdm(i,j)+transf(k,imodes(i))*sum(transf(:,imodes(j))*centres%d2v(1,:,k))
      enddo
    enddo
  enddo
endif

      end subroutine vshepard

!#######################################################################
!
! This routine gets the derivatives of a potential at a grid point from 
! an interpolated surface
!
! We assume that this call follows a call to vshepard, and that the
! point hasn't changed.  Thus the derivatives are available
!
!#######################################################################

subroutine dvshepard (der,gpoint,ndof1,order)
real(kind=dop),intent(out)::der
integer,intent(in)::ndof1
real(kind=dop),dimension(ndof1),intent(in)::gpoint
integer,dimension(ndof1),intent(in)::order
integer::i,j,k
!write(*,*)'dVShepard',real(gpoint),order
if (any(abs(gpoint(1:ndof1-1)-LastGpoint(1:ndof1-1))>1.0e-12)) &
        stop 'Assumptions about dvshepard calling are apparently not correct!'
!stop 'in dvshepard'
if (nmodes) then
  do i=1,ndof1-1
    if (order(i)>0) exit
  enddo
  if (sum(order(1:ndof1-1))==1) then    ! first order
    der=dvdm(i)
  else          ! second order
    if (order(i)==2) then
      j=i
    else
      do j=i+1,ndof1-1
        if (order(j)>0) exit
      enddo
    endif
    der=d2vdm(i,j)
  endif
else
  i=-1
  j=-1
  do k=1,ndof1
    if (order(k)>0) then
      if (i<0) then
        i=k
        if (order(k)==2) then
          j=k
          exit
        endif
      else
        j=k
        exit
      endif
    endif
  enddo
  if (j<0) then   ! first deriv
    j=(i-1)/3+1
    k=i-3*(j-1)
    der=centres%dv(1,j,k)
  else            ! second deriv
    der=centres%d2v(1,i,j)
  endif
endif
end subroutine dvshepard

!##########################################################################
!
! output info to log file
!
!##########################################################################

subroutine shepardinfo
write(ilog,'(a)') 'Potential is a modified Shepard interpolation surface'
end subroutine shepardinfo

end module funcshep
