!#######################################################################
      module functhph

use decimal, only:long,dop

      implicit none
      private
      public :: fdefthphi,functhphi
      contains

!#######################################################################
!
! 2D theta/phi  FUNCTIONS
!
! ifile = 202
!
!#######################################################################

      subroutine fdefthphi(ifunc,label,ifile,ipar,rpar,maxhpar,hoppar)

      implicit none

      integer(long)                               :: ilbl,jlbl,np
      integer(long), intent(in)                   :: maxhpar
      integer(long), intent(out)                  :: ifunc,ifile
      integer(long), dimension(*), intent(out)    :: ipar
      real(dop), dimension(*), intent(out)        :: rpar
      real(dop), dimension(maxhpar,*), intent(in) :: hoppar
      character(len=*), intent(in)                :: label

!-----------------------------------------------------------------------
! assign function number
!-----------------------------------------------------------------------
! coshcosth
      if (label(1:9) .eq. 'coshcosth') then
         ilbl=index(label,'[')+1
         if (ilbl .gt. 1) then
            ipar(1)=0
            rpar(1)=0.0_dop
            jlbl = index(label,']')-1
            read(label(ilbl:jlbl),*) np

            rpar(1)=hoppar(1,np)
         endif

         ifunc=1

! sinhcosth
      else if (label(1:9) .eq. 'sinhcosth') then
         ilbl=index(label,'[')+1
         if (ilbl .gt. 1) then
            ipar(1)=0
            rpar(1)=0.0_dop
            jlbl = index(label,']')-1
            read(label(ilbl:jlbl),*) np

            rpar(1)=hoppar(1,np)
         endif

         ifunc=2

! cossinthcosphi
      else if (label(1:14) .eq. 'cossinthcosphi') then
         ilbl=index(label,'[')+1
         if (ilbl .gt. 1) then
            ipar(1)=0
            rpar(1)=0.0_dop
            jlbl = index(label,']')-1
            read(label(ilbl:jlbl),*) np

            rpar(1)=hoppar(1,np)
         endif

         ifunc=3

! cossinthsinphi
      else if (label(1:14) .eq. 'cossinthsinphi') then
         ilbl=index(label,'[')+1
         if (ilbl .gt. 1) then
            ipar(1)=0
            rpar(1)=0.0_dop
            jlbl = index(label,']')-1
            read(label(ilbl:jlbl),*) np

            rpar(1)=hoppar(1,np)
         endif

         ifunc=4

! sinsinthcosphi
      else if (label(1:14) .eq. 'sinsinthcosphi') then
         ilbl=index(label,'[')+1
         if (ilbl .gt. 1) then
            ipar(1)=0
            rpar(1)=0.0_dop
            jlbl = index(label,']')-1
            read(label(ilbl:jlbl),*) np

            rpar(1)=hoppar(1,np)
         endif

         ifunc=5

! sinsinthsinphi
      else if (label(1:14) .eq. 'sinsinthsinphi') then
         ilbl=index(label,'[')+1
         if (ilbl .gt. 1) then
            ipar(1)=0
            rpar(1)=0.0_dop
            jlbl = index(label,']')-1
            read(label(ilbl:jlbl),*) np

            rpar(1)=hoppar(1,np)
         endif

         ifunc=6

! reY = real( Y_l^m(th,phi) ). Spherical harmonics.
      else if (label(1:4) .eq. 'reY[') then
         ilbl=index(label,'[')+1
         jlbl=index(label,']')-1
         read(label(ilbl:jlbl),*) np
         ipar(1)=nint(hoppar(1,np))
         ipar(2)=nint(hoppar(2,np))

         ifunc=7

! imY = im( Y_l^m(th,phi) ). Spherical harmonics.
      else if (label(1:4) .eq. 'imY[') then
         ilbl=index(label,'[')+1
         jlbl=index(label,']')-1
         read(label(ilbl:jlbl),*) np
         ipar(1)=nint(hoppar(1,np))
         ipar(2)=nint(hoppar(2,np))

         ifunc=8


!-----------------------------------------------------------------------
! end of if loop
!-----------------------------------------------------------------------
      endif

!-----------------------------------------------------------------------
! if label is assigned, assign ifile no.
!-----------------------------------------------------------------------
      if (ifunc .ne. 0) ifile=202

      return
      end subroutine fdefthphi

!#######################################################################

      subroutine functhphi(v,th,phi,ifunc,ipar,rpar)

      implicit none

      integer(long), intent(in)               :: ifunc
      integer(long), dimension(*), intent(in) :: ipar
      real(dop), intent(in)                   :: th,phi
      real(dop), intent(out)                  :: v
      real(dop), dimension(*), intent(in)     :: rpar
      complex(dop), external                  :: sphharm

     select case  (ifunc)

     case default
      write(6,'(a,i4)') '### callmuldmod/functhphi: invalid ifunc =', ifunc
      stop

     case (1)! coshcosth
      v=cosh(rpar(1)*cos(th))
      return

     case (2)! sinhcosth
      v=sinh(rpar(1)*cos(th))
      return

     case (3)! cossinthcosphi
      v=cos(rpar(1)*sin(th)*cos(phi))
      return

     case (4)! cossinthsinphi
      v=cos(rpar(1)*sin(th)*sin(phi))
      return

     case (5)! sinsinthcosphi
      v=sin(rpar(1)*sin(th)*cos(phi))
      return

     case (6)! sinsinthsinphi
      v=sin(rpar(1)*sin(th)*sin(phi))
      return

     case (7)! reY = real( Y_l^m(th,phi) ). Spherical harmonics.
      v=dble( sphharm(ipar(1),ipar(2),th,phi) )
      return

     case (8)! imY = im( Y_l^m(th,phi) ). Spherical harmonics.
      v=dimag( sphharm(ipar(1),ipar(2),th,phi) )
      return

     end select

      end subroutine functhphi


      end module functhph
