!#######################################################################
      module funcwigmod

use decimal, only: long,dop
use op1lib,  only: zerovxd

      implicit none
      private
      public :: fdefwig,funcwig

      contains

!#######################################################################
!
! ifile = 8
!-----------------------------------------------------------------------
!  MODULE funcwig
!  Following subroutines (j2wig,jplwig,jmiwig,jpmwig,jp2wig,jm2wig,
!  jplswig,jmiswig,jpmswig,jp2swig,jm2swig,jpjmwig,jmjpwig,jpjmswig,
!  jmjpswig)
!
!  Calculate operators:
!  j^2
!  Molecule-fixed:
!  j(+), j(-), c(+)*j(+)+c(-)*j(-), j(+)*j(+), j(-)*j(-), j(+)*j(-),
!  j(-)*j(+), j(+)*jz, j(-)*jz, jz*j(+), jz*j(-)
!  Space-fixed:
!  js(+), js(-), cs(+)*js(+)+cs(-)*js(-), js(+)*js(+), js(-)*js(-),
!  js(+)*js(-), js(-)*js(+), js(+)*jz, js(-)*jz, jz*js(+), jz*js(-)
!
!  j(-) operator is given through j(+): j(-)(a,b,k)=j(+)(b,a,k-1)
!
!  PST 11/12
!-----------------------------------------------------------------------
!#######################################################################

      subroutine fdefwig(hopilab,hoplab,hopfile)

      implicit none

      integer(long), intent(out)                     :: hopilab,hopfile
      character(len=*), intent(in)                   :: hoplab

!-----------------------------------------------------------------------
! Assign function number
!-----------------------------------------------------------------------
      if (hoplab .eq. 'j^2wig') then
         hopilab = 1
      elseif (hoplab .eq. 'j_pwig') then
         hopilab = 2
      elseif (hoplab .eq. 'j_mwig') then
         hopilab = 3
      elseif (hoplab .eq. 'jpmwig') then
         hopilab = 4
      elseif (hoplab .eq. 'jp2wig') then
         hopilab = 5
      elseif (hoplab .eq. 'jm2wig') then
         hopilab = 6
      elseif (hoplab .eq. 'j_pswig') then
         hopilab = 7
      elseif (hoplab .eq. 'j_mswig') then
         hopilab = 8
      elseif (hoplab .eq. 'jpmswig') then
         hopilab = 9
      elseif (hoplab .eq. 'jp2swig') then
         hopilab = 10
      elseif (hoplab .eq. 'jm2swig') then
         hopilab = 11
      elseif (hoplab .eq. 'jpjmwig') then
         hopilab = 12
      elseif (hoplab .eq. 'jmjpwig') then
         hopilab = 13
      elseif (hoplab .eq. 'jpjmswig') then
         hopilab = 14
      elseif (hoplab .eq. 'jmjpswig') then
         hopilab = 15
      elseif (hoplab .eq. 'jpjzwig') then
         hopilab = 16
      elseif (hoplab .eq. 'jmjzwig') then
         hopilab = 17
      elseif (hoplab .eq. 'jzjpwig') then
         hopilab = 18
      elseif (hoplab .eq. 'jzjmwig') then
         hopilab = 19
      elseif (hoplab .eq. 'jpjzswig') then
         hopilab = 20
      elseif (hoplab .eq. 'jmjzswig') then
         hopilab = 21
      elseif (hoplab .eq. 'jzjpswig') then
         hopilab = 22
      elseif (hoplab .eq. 'jzjmswig') then
         hopilab = 23
      endif

!-----------------------------------------------------------------------
! if label is assigned, assign hopfile no.
!-----------------------------------------------------------------------
      if (hopilab .ne. 0) hopfile=8

      return
      end subroutine fdefwig

!#######################################################################

      subroutine funcwig(hmat,gdim,ifunc,dif1mat,dif2mat, &
                          kdim,jtot,kmin,mmdim,mmmin)

      implicit none

      integer(long), intent(in) :: gdim,ifunc,kdim,kmin,mmdim,mmmin
      real(dop), intent(in)                                    :: jtot
      real(dop), dimension(gdim,gdim,kdim,2*mmdim), intent(in) :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)   :: dif2mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out)  :: hmat

      select case (ifunc)

     case(1)! j^2wig (j^2 for Wigner)
      call j2wig(dif2mat,hmat,gdim,kdim,mmdim)
      return

     case(2)! j_pwig (j+ for Wigner)
      call jplwig(dif1mat,hmat,gdim,kdim,mmdim)
      return

     case(3)! j_mwig (j- for Wigner)
      call jmiwig(dif1mat,hmat,gdim,kdim,mmdim)
      return

     case(4)! jpmwig (jpm for Wigner)
      call jpmwig(jtot,dif1mat,hmat,gdim,kdim,kmin,mmdim)
      return

     case(5)! jp^2wig (j+^2 for Wigner)
      call jp2wig(dif1mat,hmat,gdim,kdim,mmdim)
      return

     case(6)! jm^2wig (j-^2 for Wigner)
      call jm2wig(dif1mat,hmat,gdim,kdim,mmdim)
      return

     case(7)! j_pswig (j+ space-fixed for Wigner)
      call jsplwig(dif1mat(1,1,1,mmdim+1),hmat,gdim,kdim,mmdim)
      return

     case(8)! j_mswig (j- space-fixed for Wigner)
      call jsmiwig(dif1mat(1,1,1,mmdim+1),hmat,gdim,kdim,mmdim)
      return

     case(9)! jpmswig (jpm space-fixed for Wigner)
      call jspmwig(jtot,dif1mat(1,1,1,mmdim+1),hmat,gdim,kdim,mmmin,mmdim)
      return

     case(10)! jp^2swig (j+^2 space-fixed for Wigner)
      call jsp2wig(dif1mat(1,1,1,mmdim+1),hmat,gdim,kdim,mmdim)
      return

     case(11)! jm^2swig (j-^2 space-fixed for Wigner)
      call jsm2wig(dif1mat(1,1,1,mmdim+1),hmat,gdim,kdim,mmdim)
      return

     case(12)! jpjm (j+*j- for Wigner)
      call jpljminwig(dif1mat,hmat,gdim,kdim,mmdim)
      return

     case(13)! jmjp (j-*j+ for Wigner)
      call jminjplwig(dif1mat,hmat,gdim,kdim,mmdim)
      return

     case(14)! jpjms (j+*j- space-fixed for Wigner)
      call jpljminswig(dif1mat(1,1,1,mmdim+1),hmat,gdim,kdim,mmdim)
      return

     case(15)! jmjps (j-*j+ space-fixed for Wigner)
      call jminjplswig(dif1mat(1,1,1,mmdim+1),hmat,gdim,kdim,mmdim)
      return

     case(16)! jpjz (j+*jz for Wigner)
      call jpljzwig(dif1mat,hmat,gdim,kdim,kmin,mmdim)
      return

     case(17)! jmjz (j-*jz for Wigner)
      call jminjzwig(dif1mat,hmat,gdim,kdim,kmin,mmdim)
      return

     case(18)! jzjp (jz*j+ for Wigner)
      call jzjplwig(dif1mat,hmat,gdim,kdim,kmin,mmdim)
      return

     case(19)! jzjm (jz*j- for Wigner)
      call jzjminwig(dif1mat,hmat,gdim,kdim,kmin,mmdim)
      return

     case(20)! jpjzs (j+*jz space-fixed for Wigner)
      call jpljzswig(dif1mat(1,1,1,mmdim+1),hmat,gdim,kdim,mmmin,mmdim)
      return

     case(21)! jmjzs (j-*jz space-fixed for Wigner)
      call jminjzswig(dif1mat(1,1,1,mmdim+1),hmat,gdim,kdim,mmmin,mmdim)
      return

     case(22)! jzjps (jz*j+ space-fixed for Wigner)
      call jzjplswig(dif1mat(1,1,1,mmdim+1),hmat,gdim,kdim,mmmin,mmdim)
      return

     case(23)! jzjms (jz*j- space-fixed for Wigner)
      call jzjminswig(dif1mat(1,1,1,mmdim+1),hmat,gdim,kdim,mmmin,mmdim)
      return

     end select

      end subroutine funcwig

!#######################################################################

      subroutine j2wig(dif2mat,hops2,gdim,kdim,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,mm
      integer(long), intent(in)                         :: gdim,kdim,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif2mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      do mm=1,mmdim
        do k=1,kdim
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-dif2mat(i,j,k,mm)
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine j2wig

!#######################################################################

      subroutine jmiwig(dif1mat,hops2,gdim,kdim,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,mm
      integer(long), intent(in)                         :: gdim,kdim,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      do mm=1,mmdim
        ! zero hops2 for k=kdim
        call zerovxd(hops2(1,1,kdim,mm),gdim*gdim)
        do k=1,kdim-1
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-dif1mat(i,j,k,mm)
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jmiwig

!#######################################################################

      subroutine jplwig(dif1mat,hops2,gdim,kdim,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,mm
      integer(long), intent(in)                         :: gdim,kdim,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      do mm=1,mmdim
        call zerovxd(hops2(1,1,1,mm),gdim*gdim) ! zero hops2 for k=1
        do k=2,kdim
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-dif1mat(j,i,k-1,mm)
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jplwig

!#######################################################################

      subroutine jpmwig(jtot,dif1mat,hops2,gdim,kdim,kmin,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,k1,mm
      integer(long), intent(in)                         :: gdim,kdim,kmin,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2
      real(dop), intent(in)                                   :: jtot
      real(dop)                                               :: cpl

      do mm=1,mmdim
        ! zero hops2 for k=kdim
        call zerovxd(hops2(1,1,kdim,mm),gdim*gdim)
        do k=1,kdim-1
          k1=k+kmin-1
          if(k1.lt.jtot .and. -k1.lt.jtot+1.d0) then
             cpl=sqrt(jtot*(jtot+1)-k1*(k1+1))
          else
             cpl=0.d0
          endif
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-cpl*dif1mat(i,j,k,mm)
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jpmwig

!#######################################################################

      subroutine jm2wig(dif1mat,hops2,gdim,kdim,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,l,mm
      integer(long), intent(in)                         :: gdim,kdim,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      do mm=1,mmdim
        call zerovxd(hops2(1,1,1,mm),gdim*gdim*kdim)
        do k=1,kdim-2
          do j=1,gdim
            do i=1,gdim
              do l = 1,gdim
                hops2(i,j,k,mm) = hops2(i,j,k,mm) + &
                      dif1mat(i,l,k+1,mm)*dif1mat(l,j,k,mm)
              enddo
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jm2wig

!#######################################################################

      subroutine jp2wig(dif1mat,hops2,gdim,kdim,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,l,mm
      integer(long), intent(in)                         :: gdim,kdim,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      do mm=1,mmdim
        call zerovxd(hops2(1,1,1,mm),gdim*gdim*kdim)
        do k=3,kdim
          do j=1,gdim
            do i=1,gdim
              do l = 1,gdim
                hops2(i,j,k,mm) = hops2(i,j,k,mm) + &
                      dif1mat(l,i,k-2,mm)*dif1mat(j,l,k-1,mm)
              enddo
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jp2wig

!#######################################################################

      subroutine jsplwig(dif3mat,hops2,gdim,kdim,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,mm
      integer(long), intent(in)                         :: gdim,kdim,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif3mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      do mm=1,mmdim-1
        do k=1,kdim
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-dif3mat(i,j,k,mm)
            enddo
          enddo
        enddo
      enddo

      ! zero hops2 for mm=mmdim
      do k=1,kdim
        call zerovxd(hops2(1,1,k,mmdim),gdim*gdim)
      enddo

      return
      end subroutine jsplwig

!#######################################################################

      subroutine jsmiwig(dif3mat,hops2,gdim,kdim,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,mm
      integer(long), intent(in)                         :: gdim,kdim,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif3mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      ! zero hops2 for mm=1
      do k=1,kdim
        call zerovxd(hops2(1,1,k,1),gdim*gdim)
      enddo

      do mm=2,mmdim
        do k=1,kdim
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-dif3mat(j,i,k,mm-1)
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jsmiwig

!#######################################################################

      subroutine jspmwig(jtot,dif3mat,hops2,gdim,kdim,mmmin,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,k1,mm
      integer(long), intent(in)                         :: gdim,kdim,mmmin,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif3mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2
      real(dop), intent(in)                                   :: jtot
      real(dop)                                               :: cpl

      ! zero hops2 for mm=mmdim
      do k=1,kdim
        call zerovxd(hops2(1,1,k,mmdim),gdim*gdim)
      enddo

      do mm=1,mmdim-1
        k1=mm+mmmin-1
        if(k1.lt.jtot .and. -k1.lt.jtot+1.d0) then
           cpl=sqrt(jtot*(jtot+1)-k1*(k1+1))
        else
           cpl=0.d0
        endif
        do k=1,kdim
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-cpl*dif3mat(i,j,k,mm)
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jspmwig

!#######################################################################

      subroutine jsp2wig(dif3mat,hops2,gdim,kdim,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,l,mm
      integer(long), intent(in)                         :: gdim,kdim,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif3mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      call zerovxd(hops2,gdim*gdim*kdim*mmdim)

      do mm=1,mmdim-2
        do k=1,kdim
          do j=1,gdim
            do i=1,gdim
              do l = 1,gdim
                hops2(i,j,k,mm) = hops2(i,j,k,mm) + &
                      dif3mat(i,l,k,mm+1)*dif3mat(l,j,k,mm)
              enddo
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jsp2wig

!#######################################################################

      subroutine jsm2wig(dif3mat,hops2,gdim,kdim,mmdim)

      implicit none

      integer i,j,k,l,mm,gdim,kdim,mmdim
      real*8 dif3mat(gdim,gdim,kdim,mmdim),hops2(gdim,gdim,kdim,mmdim)

      call zerovxd(hops2,gdim*gdim*kdim*mmdim)

      do mm=3,mmdim
        do k=1,kdim
          do j=1,gdim
            do i=1,gdim
              do l = 1,gdim
                hops2(i,j,k,mm) = hops2(i,j,k,mm) + &
                      dif3mat(l,i,k,mm-2)*dif3mat(j,l,k,mm-1)
              enddo
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jsm2wig

!#######################################################################

      subroutine jminjplwig(dif1mat,hops2,gdim,kdim,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,l,mm
      integer(long), intent(in)                         :: gdim,kdim,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      call zerovxd(hops2,gdim*gdim*kdim*mmdim)

      do mm=1,mmdim
         do k=2,kdim
            do j=1,gdim
               do i=1,gdim
                  do l = 1,gdim
                     hops2(i,j,k,mm) = hops2(i,j,k,mm) + &
                           dif1mat(i,l,k-1,mm)*dif1mat(j,l,k-1,mm)
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine jminjplwig

!#######################################################################

      subroutine jpljminwig(dif1mat,hops2,gdim,kdim,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,l,mm
      integer(long), intent(in)                         :: gdim,kdim,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      call zerovxd(hops2,gdim*gdim*kdim*mmdim)

      do mm=1,mmdim
         do k=1,kdim-1
            do j=1,gdim
               do i=1,gdim
                  do l = 1,gdim
                     hops2(i,j,k,mm) = hops2(i,j,k,mm) + &
                           dif1mat(l,i,k,mm)*dif1mat(l,j,k,mm)
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine jpljminwig

!#######################################################################

      subroutine jpljminswig(dif1mat,hops2,gdim,kdim,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,l,mm
      integer(long), intent(in)                         :: gdim,kdim,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      call zerovxd(hops2,gdim*gdim*kdim*mmdim)

      do mm=2,mmdim
         do k=1,kdim
            do j=1,gdim
               do i=1,gdim
                  do l = 1,gdim
                     hops2(i,j,k,mm) = hops2(i,j,k,mm) + &
                           dif1mat(i,l,k,mm-1)*dif1mat(j,l,k,mm-1)
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine jpljminswig

!#######################################################################

      subroutine jminjplswig(dif1mat,hops2,gdim,kdim,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,l,mm
      integer(long), intent(in)                         :: gdim,kdim,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      call zerovxd(hops2,gdim*gdim*kdim*mmdim)

      do mm=1,mmdim-1
         do k=1,kdim
            do j=1,gdim
               do i=1,gdim
                  do l = 1,gdim
                     hops2(i,j,k,mm) = hops2(i,j,k,mm) + &
                           dif1mat(l,i,k,mm)*dif1mat(l,j,k,mm)
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine jminjplswig


!#######################################################################

      subroutine jpljzwig(dif1mat,hops2,gdim,kdim,kmin,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,k1,mm
      integer(long), intent(in)                         :: gdim,kdim,kmin,&
                                                           mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      do mm=1,mmdim
        call zerovxd(hops2(1,1,1,mm),gdim*gdim) ! zero hops2 for k=1
        do k=2,kdim
          k1=k+kmin-1 ! multiply by k
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-k1*dif1mat(j,i,k-1,mm)
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jpljzwig

!#######################################################################

      subroutine jminjzwig(dif1mat,hops2,gdim,kdim,kmin,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,k1,mm
      integer(long), intent(in)                         :: gdim,kdim,kmin,&
                                                           mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      do mm=1,mmdim
        ! zero hops2 for k=kdim
        call zerovxd(hops2(1,1,kdim,mm),gdim*gdim)
        do k=1,kdim-1
          k1=k+kmin-1 ! multiply by k
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-k1*dif1mat(i,j,k,mm)
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jminjzwig

!#######################################################################

      subroutine jzjplwig(dif1mat,hops2,gdim,kdim,kmin,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,k1,mm
      integer(long), intent(in)                         :: gdim,kdim,kmin,&
                                                           mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      do mm=1,mmdim
        call zerovxd(hops2(1,1,1,mm),gdim*gdim) ! zero hops2 for k=1
        do k=2,kdim
          k1=k+kmin-2 ! multiply by k-1
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-k1*dif1mat(j,i,k-1,mm)
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jzjplwig

!#######################################################################

      subroutine jzjminwig(dif1mat,hops2,gdim,kdim,kmin,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,k1,mm
      integer(long), intent(in)                         :: gdim,kdim,kmin,&
                                                           mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif1mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      do mm=1,mmdim
        ! zero hops2 for k=kdim
        call zerovxd(hops2(1,1,kdim,mm),gdim*gdim)
        do k=1,kdim-1
          k1=k+kmin ! multiply by k+1
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-k1*dif1mat(i,j,k,mm)
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jzjminwig

!#######################################################################

      subroutine jpljzswig(dif3mat,hops2,gdim,kdim,mmmin,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,mm,mm1
      integer(long), intent(in)                         :: gdim,kdim,mmmin,&
                                                           mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif3mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      do mm=1,mmdim-1
        mm1=mm+mmmin-1 ! multiply by m
        do k=1,kdim
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-mm1*dif3mat(i,j,k,mm)
            enddo
          enddo
        enddo
      enddo

      ! zero hops2 for mm=mmdim
      do k=1,kdim
        call zerovxd(hops2(1,1,k,mmdim),gdim*gdim)
      enddo

      return
      end subroutine jpljzswig

!#######################################################################

      subroutine jminjzswig(dif3mat,hops2,gdim,kdim,mmmin,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,mm,mm1
      integer(long), intent(in)                         :: gdim,kdim,mmmin,mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif3mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      ! zero hops2 for mm=1
      do k=1,kdim
        call zerovxd(hops2(1,1,k,1),gdim*gdim)
      enddo

      do mm=2,mmdim
        mm1=mm+mmmin-1 ! multiply by m
        do k=1,kdim
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-mm1*dif3mat(j,i,k,mm-1)
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jminjzswig

!#######################################################################

      subroutine jzjplswig(dif3mat,hops2,gdim,kdim,mmmin,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,mm,mm1
      integer(long), intent(in)                         :: gdim,kdim,mmmin,&
                                                           mmdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif3mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      do mm=1,mmdim-1
        mm1=mm+mmmin ! multiply by m+1
        do k=1,kdim
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-mm1*dif3mat(i,j,k,mm)
            enddo
          enddo
        enddo
      enddo

      ! zero hops2 for mm=mmdim
      do k=1,kdim
        call zerovxd(hops2(1,1,k,mmdim),gdim*gdim)
      enddo

      return
      end subroutine jzjplswig

!#######################################################################

      subroutine jzjminswig(dif3mat,hops2,gdim,kdim,mmmin,mmdim)

      implicit none

      integer(long)                                     :: i,j,k,mm,mm1
      integer(long), intent(in)                         :: gdim,mmmin,mmdim,&
                                                           kdim
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(in)  :: dif3mat
      real(dop), dimension(gdim,gdim,kdim,mmdim), intent(out) :: hops2

      ! zero hops2 for mm=1
      do k=1,kdim
        call zerovxd(hops2(1,1,k,1),gdim*gdim)
      enddo

      do mm=2,mmdim
        mm1=mm+mmmin-2 ! multiply by m-1
        do k=1,kdim
          do j=1,gdim
            do i=1,gdim
              hops2(i,j,k,mm)=-mm1*dif3mat(j,i,k,mm-1)
            enddo
          enddo
        enddo
      enddo

      return
      end subroutine jzjminswig

!#######################################################################

      end module funcwigmod
