!#######################################################################
      module funcxlocmod

use decimal, only:long,dop
use logdat

      implicit none
      private
      public :: fdefxloc,derxloc,funcxloc, ddefxloc
      contains
 
!#######################################################################
!
! X-LOCAL MOMENTUM TYPE FUNCTIONS
!
! ifile = 307
!
!#######################################################################

      subroutine fdefxloc(hopilab,hoplab,hopfile,hlabrpar, &
                 maxhpar,hoppar)

      implicit none

      real(dop), parameter                          :: tiny=1.0e-8_dop

      integer(long), intent(in)                     :: maxhpar
      integer(long), intent(out)                    :: hopilab,hopfile
      real(dop), dimension(maxhpar), intent(out)    :: hlabrpar
      real(dop), dimension(maxhpar,*), intent(in)   :: hoppar
      character(len=*), intent(in)                  :: hoplab
      integer(long)                                 :: np,ilbl

!-----------------------------------------------------------------------
! assign function number
!-----------------------------------------------------------------------
! classical momentum
      if (hoplab .eq. 'p_cl') then
         hopilab=1

! classical kinetic energy
      else if (hoplab .eq. 'KE_cl') then
         hopilab=2

! classical p^2
      else if (hoplab .eq. 'p2_cl') then
         hopilab=3

! classical -p^2 = d^2/dq^2
      else if (hoplab .eq. 'p2_clm') then
         hopilab=4

! classical "flux operator"
      else if (hoplab(1:7) .eq. 'flux_cl') then
         ilbl = index(hoplab,']')-1
         read(hoplab(9:ilbl),*) np

         hlabrpar(1)=hoppar(1,np) ! x-cut 
         hlabrpar(2)=hoppar(2,np) ! Left or right moving flux
         hopilab=5

!-----------------------------------------------------------------------
! end of if loop
!-----------------------------------------------------------------------
      endif

!-----------------------------------------------------------------------
! if label is assigned, assign ifile no.
!-----------------------------------------------------------------------
      if (hopilab .ne. 0) hopfile=307

      return
      end subroutine fdefxloc

!#######################################################################

    subroutine funcxloc(t,p,x,hopilab,hoprpar,mass)

use global

      implicit none

      integer(long), intent(in)               :: hopilab
      real(dop), intent(in)                   :: mass,p,x
      real(dop), intent(out)                  :: t
      real(dop), dimension(*), intent(in)     :: hoprpar
      real(dop)                               :: xc,xend

     select case (hopilab)

     case default
      routine='Funcxloc'
      write(message,'(a,i5)') 'Function not found :',hopilab
      call errormsg

     case (1)! classical momentum
      t=p
      return

     case (2)! classical Kinetic energy
      t=p*p/(2.0_dop*mass)
      return

     case (3)! classical p^2
      t=p*p
      return

     case (4)! classical -p^2
      t=-p*p
      return

     case(5) ! classical flux
      xc=hoprpar(1)
      xend=hoprpar(2)
      if (xend .gt. 0) then
         if (x .gt. xc) t = 1.0d0
      else if (xend .lt. 0) then
         if (x .lt. xc) t = 1.0d0
      else
         t = 0.0d0
      endif
      return

     end select

    end subroutine funcxloc


!#######################################################################
!
! list of functions for which derivatives are given
!
!#######################################################################

      subroutine ddefxloc(hoplab,found)

      implicit none

      integer(long), intent(out)   :: found
      character(len=*), intent(in) :: hoplab

! classical momentum

      if (hoplab .eq. 'p_cl') then
         found=1

! classical kinetic energy
      else if (hoplab .eq. 'KE_cl') then
         found=1

! classical p^2
      else if (hoplab .eq. 'p2_cl') then
         found=1

! classical -p^2 = d^2/dq^2
      else if (hoplab .eq. 'p2_clm') then
         found=1

      else
         found=999

!-----------------------------------------------------------------------
! end of if loop
!-----------------------------------------------------------------------
      endif

      return
      end subroutine ddefxloc


!#######################################################################


      subroutine derxloc(der,p,hopilab,mass)

use global

      implicit none

      integer(long), intent(in)               :: hopilab
      real(dop), intent(in)                   :: mass,p
      real(dop), intent(out)                  :: der

     select case (hopilab)

     case default
      routine='Derxloc'
      write(message,'(a,i5)') 'Function not found :',hopilab
      call errormsg

     case (1) ! classical momentum
      der=1.0_dop
      return

     case (2) ! classical Kinetic energy
      der=p/mass
      return

     case (3) ! classical p^2
      der=2.0_dop*p
      return

     case (4) ! classical -p^2
      der=-2.0_dop*p
      return

     end select

      end subroutine derxloc

      end module funcxlocmod
