!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!  Modified Shepard interpolated potential implementation
!
!  This is code derived from Mick Collins' GROW codebase
!
!  Specifically, it has come from Terry Frankcombe's 
!  SingleSurfaceGrow/sourcecode/interptraj/AbstractedPotfile
!  implementation, which has some data encapsulation features.
!  That code has been modified for seconds along the lines of
!  SingleSurfaceGrow/sourcecode/EnGradHess.
!   *** seconds not functional for TranslationalSym=true ***
!
!  Three modules:  POTmodule - encapsulation for POTfile data
!                  InterpTrajData - "Instances", including neighbour lists, etc.
!                  shepard - the evaluation routines
!  The MPI_Data is still here, but not used
!  
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

module POTmodule
implicit none

 ! for GJ style gas-surface potentials: TranslationalSym & GasSurface are true
 ! for fragmented gas-surface potentials: GasSurface is true
logical,parameter::RecalculateCon=.false.
logical,parameter::TranslationalSym=.false.
logical,parameter::SurfaceCoordsIncludeHeight=.false.
logical,parameter::UseAnalyticPES=.false.
 ! for gas-surface potentials (GJ style, not fragmented)
 ! we store the bondlengths, then the heights, 
 ! then the x-y coords in rda (in blocks).
logical,parameter::GasSurface=.false.
logical,parameter::SiliconHacks=.false.
logical,save::doforce=.true.,keepneigh=.false.

type POTobject
  real(kind=8),dimension(:,:,:),pointer::ut
  real(kind=8),dimension(:,:),pointer::v1,v2,rda,zda,sumb
  real(kind=8),dimension(:),pointer::v0,rads,amas,maxwt
  real(kind=8),dimension(2)::RecipA,RecipB,LattA,LattB,CellCentre
  integer,dimension(:,:),pointer::ip,nperm
  integer,dimension(:),pointer::mb,nb
  real(kind=8)::wtol,outer,avrads,potmax,vmin,sigma2,Z0depth,CellMax
  real(kind=8)::vtlim
  integer::natom,nbond,nextra,nredundant,nint,ngroup,ndata
  integer::lowp,ipart,ipow,nneigh,mep
  character(len=2),dimension(:),pointer::lab
end type POTobject

type POTinstance
   ! note that we store lengths & heights in r (& dr) in intern, convert later
  real(kind=8),dimension(:,:,:,:),pointer::dr,d2r
  real(kind=8),dimension(:,:,:),pointer::c,v,dv,c0,dv2r,d2v
  real(kind=8),dimension(:,:),pointer::r,dvr
  real(kind=8),dimension(:),pointer::totsum,en,rms,AsympEn
  integer,dimension(:,:),pointer::mda,nda,mdao,ndao   ! molecule is second index
  integer,dimension(:,:),pointer::map
  integer,dimension(:),pointer::nforc,nforco
  integer::maxf,maxfo,nmax              ! nmax ~ nmol
  logical,dimension(:,:),pointer::AsympMask
end type POTinstance

contains

subroutine AllocatePOT1(p)
 ! allocates stuff that depends only on natom, nint, nbond
type(POTobject)::p
allocate(p%amas(p%natom))
allocate(p%mb(p%nbond))
allocate(p%nb(p%nbond) )
allocate(p%lab(p%natom))
end subroutine AllocatePOT1

subroutine AllocatePOT2(p)
 ! allocates stuff that depends on ngroup
type(POTobject)::p
allocate(p%ip(p%nredundant,p%ngroup))
allocate(p%nperm(p%ngroup,p%natom))
end subroutine AllocatePOT2

subroutine AllocatePOT3(p)
 ! Now we have ndata
type(POTobject)::p
allocate(p%v0(p%ndata))
allocate(p%v1(p%ndata,p%nint))
allocate(p%v2(p%ndata,p%nint))
allocate(p%ut(p%ndata,p%nint,p%nredundant))
allocate(p%rda(p%ndata,p%nredundant))
allocate(p%maxwt(p%ndata))
p%maxwt=0
allocate(p%zda(p%ndata,p%nint))
allocate(p%rads(p%ndata))
allocate(p%sumb(p%ndata,p%nredundant))
end subroutine AllocatePOT3

subroutine AllocateMols(p,n,mi,mo,pot)
type(POTinstance)::p
integer,intent(in)::n,mi,mo
type(POTobject)::pot
p%nmax=n
p%maxf=mi
p%maxfo=mo
allocate(p%c(n,pot%natom,3))
allocate(p%v(n,pot%natom,3))
allocate(p%r(n,pot%nredundant))
allocate(p%c0(n,pot%natom,3))
allocate(p%dr(pot%nredundant,pot%natom,3,n))
allocate(p%d2r(pot%nredundant,pot%natom*3,pot%natom*3,n))
allocate(p%dv(n,pot%natom,3))
allocate(p%d2v(pot%natom*3,pot%natom*3,n))
allocate(p%dvr(pot%nredundant,n))
allocate(p%dv2r(pot%nredundant,pot%nredundant,n))
allocate(p%totsum(n))
allocate(p%en(n))
allocate(p%rms(n))
allocate(p%AsympEn(n))
allocate(p%mda(mi,n))
allocate(p%nda(mi,n))
allocate(p%mdao(mo,n))
allocate(p%ndao(mo,n))
allocate(p%nforc(n))
allocate(p%nforco(n))
allocate(p%map(pot%natom,pot%natom))
allocate(p%AsympMask(n,pot%nredundant))
end subroutine AllocateMols

subroutine DeallocatePOT(p)
type(POTobject)::p
deallocate(p%amas)
deallocate(p%mb)
deallocate(p%nb)
deallocate(p%lab)
deallocate(p%ip)
deallocate(p%nperm)
deallocate(p%v0)
deallocate(p%v1)
deallocate(p%v2)
deallocate(p%ut)
deallocate(p%rda)
deallocate(p%maxwt)
deallocate(p%zda)
deallocate(p%rads)
deallocate(p%sumb)
end subroutine DeallocatePOT

subroutine DeallocateMols(p)
type(POTinstance)::p
deallocate(p%c)
deallocate(p%v)
deallocate(p%r)
deallocate(p%c0)
deallocate(p%dr)
deallocate(p%d2r)
deallocate(p%dv)
deallocate(p%d2v)
deallocate(p%dvr)
deallocate(p%dv2r)
deallocate(p%totsum)
deallocate(p%en)
deallocate(p%AsympEn)
deallocate(p%rms)
deallocate(p%mda)
deallocate(p%nda)
deallocate(p%mdao)
deallocate(p%ndao)
deallocate(p%nforc)
deallocate(p%nforco)
deallocate(p%map)
deallocate(p%AsympMask)
end subroutine DeallocateMols

end module POTmodule


module InterpTrajData
use POTmodule
implicit none
save
 ! for single PES interptraj:
real(kind=8)::tups,heutu,delta,pi
real(kind=8)::sample,bipmax,distab,econ,etrans,echeck
real(kind=8),dimension(:,:),pointer::rmin
integer,dimension(:,:),pointer::ichansave
integer,dimension(:),pointer::nfin,ntest,mfrag,ifraga,ifragb,kf
integer::nloop,nstep,nprint,ir,nstop,neighci,neighco,neighc
integer::nsel,nlowstop,nmol,neighci2,neighco2
integer::nfraga,nfragb,nfrag
 ! for fragmented PESs:
!type(POTobject),dimension(:),allocatable::pots
!type(POTinstance),dimension(:),allocatable::inst
integer,dimension(:),allocatable::InstPerPot
integer::npots
logical::Fragmented,AdaptNeigh
logical::Shepard2D
end module InterpTrajData

!module MPI_Data
!implicit none
!logical,parameter::MPI=.false.
!integer::rank,ranks
!end module MPI_Data

module shepard
use InterpTrajData
use channels
implicit none
save
real(kind=8),dimension(:),allocatable::LastGpoint

contains

        subroutine readccm(pot,IN_SYSTEM,rrmin)
        implicit none
        type(POTobject)::pot
        character(len=*),intent(in)::IN_SYSTEM
        logical,intent(in)::rrmin
        character*80 icomm, ielements, ialpha
        integer::length,ifin,i,j
        open (unit=igin,file=IN_SYSTEM)
!  read comment ; a header for the SYSTEM file
        read(igin,80)icomm
80      format(a80)
!  read in the actual number of atoms
        read(igin,80)icomm
        read(igin,*) pot%natom
        if (pot%natom==1) then
          pot%nint=0
        elseif (pot%natom==2) then
          pot%nint=1
        elseif (Shepard2D) then
          pot%nint=2*pot%natom-3
        else
          pot%nint=3*pot%natom-6
        endif
        if (TranslationalSym) then
          pot%nextra=5*pot%natom
          pot%nint=3*pot%natom
        else
          pot%nextra=0
        endif
        pot%nbond=pot%natom*(pot%natom-1)/2
        pot%nredundant=pot%nbond+pot%nextra
        call AllocatePOT1(pot)
!  read in the element labels for the atoms
        read(igin,80)icomm
        read(igin,80) ielements
! extract the elemental labels from ielements
        ialpha=ielements
        do i=1,pot%natom-1
        length=len(ialpha)
        ifin=index(ialpha,',')
          if(ifin.gt.3) stop 'an atomic label in SYSTEM has too many characters'
        pot%lab(i)=ialpha(1:ifin-1)
        ialpha=ialpha(ifin+1:length)
        enddo
        ifin=index(ialpha,' ')
        pot%lab(pot%natom)=ialpha(1:ifin-1)
!  read in the atomic masses
        read(igin,80)icomm
        read(igin,*)(pot%amas(i),i=1,pot%natom)
        if (rrmin) then
!  read in the cutoff lengths rmin(i,j)
          allocate(rmin(pot%natom,pot%natom))
          read(igin,80)icomm
          read(igin,80)icomm
          do i=1,pot%natom-1
            do j=i+1,pot%natom
              read(igin,*) rmin(i,j)
            enddo
          enddo
        endif
        close (unit=igin)
        end subroutine readccm

!      subroutine readz(pot,IN_ZMA)
      subroutine readz(pot)
      type(POTobject)::pot
!      character(len=*),intent(in)::IN_ZMA
!      character(len=80)::comlin
!      integer,dimension(pot%nbond)::ibfraga,ibfragb
!      integer::nbfraga,nbfragb
      integer::i,j,k
      write(igout,*)repeat('-',60)
      write(igout,*) ' *** subroutine readz: creat mb,nb read in fragment data'
      write(igout,*) ' '
!  given that we need all the atom-atom bonds, we just assign the indirect
!  addresses of the bonds, rather than the traditional read from ZMA
        k=1
        do i=1,pot%natom-1
        do j=i+1,pot%natom
           pot%mb(k)=i
           pot%nb(k)=j
           k=k+1
        enddo
        enddo
!   read fragment atom numbers
!      if (IN_ZMA=='') goto 4100
!      open (unit=igin,file=IN_ZMA)
!80    format(a80)
!81    format(1x,a79)
!      read(igin,80)comlin
!      write(igout,81)comlin
!      read(igin,*)nfraga,nfragb
!      write(igout,*)nfraga,nfragb
!      if(nfraga+nfragb.ne.pot%natom)then
!        write(igout,*) 'ABORT: nfraga+nfragb.ne.natom'
!        write(igout,*) nfraga,'+',nfragb,'.ne.',pot%natom
!        stop
!      endif
!      read(igin,80)comlin
!      write(igout,81)comlin
!      if(nfraga.gt.0)then
!      read(igin,*)(ifraga(i),i=1,nfraga)
!      write(igout,*)(ifraga(i),i=1,nfraga)
!      endif
!      read(igin,80)comlin
!      write(igout,81)comlin
!      if(nfragb.gt.0)then
!      read(igin,*)(ifragb(i),i=1,nfragb)
!      write(igout,*)(ifragb(i),i=1,nfragb)
!      endif
!      if (nfraga .le. 1) go to 3100
!      nbfraga=0
!      do 3000 i=1,pot%nbond
!	do 2900 j=1,nfraga
!	  do 2800 k=1,nfraga
!            if ((pot%mb(i).eq.ifraga(j)).and.(pot%nb(i).eq.ifraga(k))) then
!              nbfraga=nbfraga+1
!              ibfraga(nbfraga)=i
!            endif
! 2800     continue 
! 2900   continue
! 3000 continue
!      write (igout,*) 'no. of bonds in fragment a is', nbfraga
!      write (igout,*) 'bonds in fragment a are'
!      write (igout,*) (ibfraga(i),i=1,nbfraga)
! 3100 continue
!      if (nfragb .le. 1) go to 4100
!      nbfragb=0
!      do 4000 i=1,pot%nbond
!	do 3900 j=1,nfragb
!	  do 3800 k=1,nfragb
!            if ((pot%mb(i).eq.ifragb(j)).and.(pot%nb(i).eq.ifragb(k))) then
!              nbfragb=nbfragb+1
!              ibfragb(nbfragb)=i
!              endif
! 3800     continue 
! 3900   continue
! 4000 continue
!      write (igout,*) 'no. of bonds in fragment b is', nbfragb
!      write (igout,*) 'bonds in fragment b are'
!      write (igout,*) (ibfragb(i),i=1,nbfragb)
! 4100 continue
!      close(unit=igin)
      end subroutine readz

      subroutine makebp(pot,IN_ATOMPERMS)
      type(POTobject)::pot
      character(len=*),intent(in)::IN_ATOMPERMS
      character*80 icomm
      integer::idummy,n,i,j,k,ic
!  read in the atomic permutations into the nperm array
       open(unit=igin, file=IN_ATOMPERMS)
!c read in a header
       read(igin,80)icomm
80    format(a80)
!  read in a comment line
       read(igin,80)icomm
!  read in the order of the group
       read(igin,*)pot%ngroup
       call AllocatePOT2(pot)
!  read in the atomic perms
       do i=1,pot%ngroup
!  read in a comment line or separator
       read(igin,80)icomm
       do k=1,pot%natom
       read(igin,*)idummy,pot%nperm(i,k)
       enddo
!  idummy will be equal to k
       enddo
       close(unit=igin)
!  now set up the ip array of bond perms by looking at nperm
      do n=1,pot%ngroup
      ic=0
      do i=1,pot%natom-1
      do j=i+1,pot%natom
      ic=ic+1
      do k=1,pot%nbond
      if(pot%mb(k).eq.pot%nperm(n,i).and.pot%nb(k).eq.pot%nperm(n,j))pot%ip(k,n)=ic
      if(pot%nb(k).eq.pot%nperm(n,i).and.pot%mb(k).eq.pot%nperm(n,j))pot%ip(k,n)=ic
      enddo
      enddo
      enddo
!  Now the surface coords
      if (TranslationalSym) then
        do i=1,pot%natom
          pot%ip(pot%nbond+pot%nperm(n,i),n)=pot%nbond+i
          pot%ip(pot%nbond+pot%natom+4*pot%nperm(n,i)-3  &
                       :pot%nbond+pot%natom+4*pot%nperm(n,i),n)  &
                               =pot%nbond+pot%natom+4*i+(/-3,-2,-1,0/)
        enddo
      endif
      enddo
      end subroutine makebp

      subroutine cntl(pot,IN_CNT,KeepTrajData)
      type(POTobject)::pot
      character(len=*),intent(in)::IN_CNT
      logical,intent(in)::KeepTrajData
!   this subroutine reads in run control parameters from file on unit 5
      character*20 msunit,enunit,lnunit
      character*80 title,comlin
      real(kind=8)::amsc,esc,eukjpm,hkjpms
      real(kind=8)::rsc,scalef,tsc,avanum,hjs
      real(kind=8)::t1
      integer::nunits,i,k,i1
      write(igout,*)repeat('-',60)
      write(igout,*) ' *** subroutine cntl: read in run control parameters ***'
      open (unit=igin,file=IN_CNT)
!   read in title of run
      read(igin,80) title
   80 format(a80)
      nunits=5
!   if (nunits) then (mass  energy   length     scale factor) are:
!         5           g/mol hartrees bohr       9.682886569d14
! on output masses, energies, lengths and times are scaled by
! amsc, esc, rsc and tsc (in seconds), respectively and
! must be multiplied by these scale factors to be converted
! to the stated units.
      amsc=1.0d0
      esc=1.0d0
      rsc=1.0d0
        scalef=9.682886569d14
        msunit=' g/mol'
        enunit=' hartree'
        lnunit=' bohr'
        eukjpm=2625.499964d0
      tsc=dsqrt(amsc/esc)*rsc/scalef
      tups=tsc*1.0d12
!   Avagadro's number
      avanum=6.0221367d23
!   speed of light in cm s-1 and cm tu-1
!      ccmps=2.99792458d10
!      ccmptu=ccmps*tsc
!   Planck's constant in J s, kJ/mol s and eu tu
      hjs=6.6260755d-34
      hkjpms=hjs*avanum*1.0d-03
      heutu=(hkjpms/tsc)/eukjpm
!---------------------------------------------------------------------
!   echo some parameters
      write(igout,*) title
! echo info from readccm
      write(igout,*) 'The number of atoms = ',pot%natom
      if (TranslationalSym) write(igout,*)'This is a 2D-PERIODIC GAS-SURFACE potential'
      if (UseAnalyticPES) write(igout,*)'Using ANALYTIC PES'
      write(igout,*)
      write(igout,*) 'The atomic labels and masses are:'
      do i=1,pot%natom
      write(igout,*) pot%lab(i), pot%amas(i)
      enddo
      write(igout,*)
      write(igout,*) 'The bonded atoms'
      do i=1,pot%nbond
      write(igout,*) pot%mb(i), pot%nb(i)
      enddo
      write(igout,*)
      write(igout,*) 'The number of group elements = ',pot%ngroup
      write(igout,*)
      write(igout,*) 'The permutations of bonds in the group are:'
      write(igout,*)
      do i=1,pot%ngroup
       write(igout,*)
       write(igout,8769)' Reordered atoms ',(pot%nperm(i,k),k=1,pot%natom)
       do k=1,pot%nredundant
       write(igout,*)k, pot%ip(k,i)
       enddo
       write(igout,*)
      enddo
8769  format(a18,20i3)
      write(igout,*) 'The units are defined as atomic units in cntl.f'
      write(igout,*)
      write(igout,40)esc,enunit,rsc,lnunit,amsc,msunit, &
        tsc,tups,scalef
   40 format(/,'  unit scaling parameters', &
      /,'  esc=',g12.5,a10,'  rsc=',g12.5,a10,'  amsc=',g12.5,a10, &
      /,'  tsc=',g17.10,' s =',g17.10,' ps  scalef=',g17.10,' s-1')
      write(igout,*)repeat('-',60)
      write(igout,81) title
   81 format(1x,a79)
 ! Gobble unused lines
do i=1,8
  read(igin,80) comlin
enddo
!   read in time step,no. of traj. step, nprint,no. of molecule etc.
!      read(igin,80) comlin
!      write(igout,81) comlin
!      read(igin,*) t1,i1,i2,i3
!      write(igout,100) t1,i1,i2,i3
!  100 format(1x,g18.10,3i10)
!      if (KeepTrajData) then
!        delta=t1
!        nstep=i1
!        nprint=i2
!        nmol=i3
!      endif
!      read(igin,80) comlin
!      write(igout,81) comlin
!      read(igin,*) t1,t2,t3
!      write(igout,140) t1,t2,t3
!      if (KeepTrajData) then
!        efraga=t1
!        efragb=t2
!        etrans=t3
!      endif
!      read(igin,80) comlin
!      write(igout,81) comlin
!      read(igin,*)t1
!      write(igout,140)t1
!      if (KeepTrajData) econ=t1
!  140 format(1x,3g18.10)
!      read(igin,80) comlin
!      write(igout,81) comlin
!      read(igin,*) t1,t2
!      write(igout,140) t1,t2
!      if (KeepTrajData) then
!        bipmax=t1
!        distab=t2
!        if(bipmax.gt.distab)then
!          write(igout,*) ' WARNING: bipmax =',bipmax,'.gt.distab=',distab
!          write(igout,*) ' RESET distab = bipmax = ',bipmax
!          distab=bipmax
!          write(igout,81) comlin
!          write(igout,140) bipmax,distab
!        endif
!      endif
!  determine one or twp part weight function is used, ipart=1,2
       read(igin,80) comlin
       write(igout,81)comlin
       read(igin,*)pot%ipart
       write(igout,91)pot%ipart
91     format(2x,'we will use the ',i1,' part weight function')
      if(pot%ipart.eq.1.or.pot%ipart.eq.2)go to 92
      write(igout,*)' incorrect value for 1 or 2 part weight function'
      stop
92    continue
!  read in the parameters defining the weight function, the neighbour
!  list and the number of timesteps between calls to neighbour
       read(igin,80) comlin
       write(igout,81)comlin
       read(igin,*)pot%lowp,pot%ipow
       write(igout,*)pot%lowp,pot%ipow
       read(igin,80) comlin
       write(igout,81)comlin
       read(igin,*)pot%wtol,pot%outer
       write(igout,*)pot%wtol,pot%outer
       read(igin,80) comlin
       write(igout,81)comlin
! This will accept either the old two integers (adaptive switched off implied)
! or int,int,logical,int,int
       read(igin,80)comlin
       write(igout,81)comlin
       if (KeepTrajData) then
         if (scan(comlin,'Tt')==0) then
           read(comlin,*)neighco,neighci
           AdaptNeigh=.false.
         else
           read(comlin,*)neighco,neighci,AdaptNeigh,neighco2,neighci2
           if (AdaptNeigh) then
             if (mod(neighco,neighco2)/=0) stop 'Outer updates mismatch'
             if (mod(neighco,nprint)/=0) stop 'print-update mismatch'
             if (.not.Fragmented) stop 'Adapting neighco only for fragmented'
           endif
         endif
         if (.not.AdaptNeigh) then
           neighco2=neighco
           neighci2=neighci
         endif
!  to allow one part neighbour to work
         neighc=neighci
       endif
!  read in the sampling fraction used in outp to determine number
!  of trajectory points output in file TOUT, and number of
!  data points chosen
       read(igin,80) comlin
       write(igout,81)comlin
       read(igin,*) t1
       write(igout,*) t1
       if (KeepTrajData) sample=t1
       read(igin,80) comlin
       write(igout,81)comlin
       read(igin,*) i1
       write(igout,*) i1
       if (KeepTrajData) nsel=i1
!  read in energy maximum and minimum with which to screen data
       read(igin,80)comlin
       write(igout,81)comlin
       read(igin,*) pot%potmax,pot%vmin
       write(igout,*) pot%potmax,pot%vmin
!  read in flag to determine whether low energy trajectories are stopped
       read(igin,80)comlin
       write(igout,81)comlin
       read(igin,*) i1
       write(igout,*) i1
       if (KeepTrajData) nlowstop=i1
! read in the multipicitive factor used in radius.f, confidrad.f
       read(igin,80)comlin
       write(igout,81)comlin
       read(igin,*) pot%nneigh
       write(igout,*) pot%nneigh
!  Enter the energy error tolerance for confidrad.f
       read(igin,80)comlin
       write(igout,81)comlin
       read(igin,*) pot%sigma2
       write(igout,*) pot%sigma2
       pot%sigma2=pot%sigma2**2
! calculate pi, conversion factors
      pi=2.d0*dacos(0.d0)
!  scale atomic masses
      do 150 i=1,pot%natom
        pot%amas(i)=pot%amas(i)/amsc
  150 continue
      close(unit=igin)
      end subroutine cntl

      subroutine readt(pot,IN_POT)
      type(POTobject)::pot
      character(len=*),intent(in)::IN_POT
      character*80 comlin
      real(kind=8),dimension(pot%nredundant*(1+pot%nint))::dummy
      real(kind=8),dimension(pot%nredundant)::lr
      real(kind=8)::vv
      integer::i,j,k,badness,n
      open (unit=igin,file=IN_POT)
      write(igout,*)repeat('-',60)
      write(igout,*) ' '
      write(igout,*) ' *** subroutine readt: read in potential energy surface parameters ***'
      write(igout,*) ' '
! Because we're now using dynamic allocation, we need to 
! know how many datapoints
      k=0
      read(igin,'(a20)')comlin
      do
        read(igin,'(a20)',iostat=badness)comlin
        if (badness/=0) exit
!  Annoying, but index is case sensitive with most (all?) systems:
        if (index(comlin,'data')>0) then
          k=k+1
        elseif (index(comlin,'Data')>0) then
          k=k+1
        elseif (index(comlin,'DATA')>0) then
          k=k+1
        else
          cycle
        endif
!  we've just read a heading line
        read(igin,*)dummy
        read(igin,*)vv
        if (vv>pot%potmax) k=k-1
      enddo
      pot%ndata=k
      rewind(igin)
      call AllocatePOT3(pot)
!   read comment/header
      read(igin,80)comlin
   80 format(a80)
      write(igout,81)comlin
   81 format(1x,a79)
!  read in the geometry (bond lengths, etc.),
!  the matrix defining the normal local coords, 
!  the potential energy, the gradients and diagonal force constants (wrt to 
!  the normal local coords) at all of the Ndata points.
      j=1
10    continue
!write(*,*)j,size(pot%rda,2),pot%nint,pot%nredundant
         read(igin,80,end=20)comlin
         read(igin,*)pot%rda(j,:)
         do i=1,pot%nint
            read(igin,*) (pot%ut(j,i,k),k=1,pot%nredundant)
         enddo
         read(igin,*) pot%v0(j)
         read(igin,*) (pot%v1(j,k),k=1,pot%nint)
         read(igin,*) (pot%v2(j,k),k=1,pot%nint)
         if (pot%v0(j).gt.pot%potmax) goto 10
         if (j==pot%ndata) goto 20
         j=j+1
         go to 10
20    continue
      write(igout,*)' Number of PES data points = ',pot%ndata
! set vmin to be the minimum of the readin value in cntl and the
! minimum found in the POT file
      write(igout,*)' lowest potential energy in the POT file   = ',minval(pot%v0)
      write(igout,*)' lowest potential energy expected from IN_CNT  = ',pot%vmin
      pot%vmin=min(pot%vmin,minval(pot%v0))
      write(igout,*)' lowest potential energy   = ',pot%vmin
      close (unit=igin)
!  construct the local coords for each data point
!  remembering that rda are read in as bond lengths NOT inverses 
!  invert bondlengths if using 2-part
      pot%zda=0
      do n=1,pot%ndata
        lr=pot%rda(n,:)
        lr(1:pot%nbond)=1/lr(1:pot%nbond)
        pot%zda(n,:)=matmul(pot%ut(n,:,:),lr)
        if (pot%ipart==2) pot%rda(n,:)=lr
      enddo
      end subroutine readt

      subroutine readcon(pot,IN_CON)
      type(POTobject)::pot
      character(len=*),intent(in)::IN_CON
      integer::j,nr
      open(unit=igin,file=IN_CON)
      nr=pot%nredundant
      if (TranslationalSym) nr=pot%nredundant-2
      do j=1,pot%ndata
        read(igin,*)pot%rads(j),pot%sumb(j,1:nr)
      enddo
      close(unit=igin)
      if (TranslationalSym) then
        pot%sumb(1:pot%ndata,nr+1)=pot%sumb(1:pot%ndata,nr)
        pot%sumb(1:pot%ndata,nr+2)=pot%sumb(1:pot%ndata,nr)
        pot%sumb(1:pot%ndata,nr)=pot%sumb(1:pot%ndata,nr-1)
      endif
      pot%avrads=sum(pot%rads)/real(pot%ndata,8)
      end subroutine readcon

subroutine intern(mols,pot,j)
real(kind=8),parameter::Pi=3.1415926535897932384626433_8
type(POTinstance)::mols
type(POTobject)::pot
integer,intent(in)::j
real(kind=8),dimension(pot%natom)::y1,y2
integer::i,k,mi,ni,dk,dj
  !  bond lengths
      do i=1,pot%nbond
         mols%r(j,i)=sqrt(sum((mols%c(j,pot%mb(i),:)-mols%c(j,pot%nb(i),:))**2))
!                sqrt((c(j,mb(i),1)-c(j,nb(i),1))**2
!     .              +(c(j,mb(i),2)-c(j,nb(i),2))**2
!     .              +(c(j,mb(i),3)-c(j,nb(i),3))**2)
      enddo
!  bond length derivatives
! dr: dr/dx, d(1/h)/dx, d(su/h**2)/dx
      mols%dr(:,:,:,j)=0
      do i=1,pot%nbond
        mols%dr(i,pot%mb(i),:,j)=(mols%c(j,pot%mb(i),:)-mols%c(j,pot%nb(i),:))/mols%r(j,i)
        mols%dr(i,pot%nb(i),:,j)=-mols%dr(i,pot%mb(i),:,j)
!         dr(j,i,mb(i),1)=(c(j,mb(i),1)-c(j,nb(i),1))/r(j,i)
!         dr(j,i,mb(i),2)=(c(j,mb(i),2)-c(j,nb(i),2))/r(j,i)
!         dr(j,i,mb(i),3)=(c(j,mb(i),3)-c(j,nb(i),3))/r(j,i)
!         dr(j,i,nb(i),1)=-dr(j,i,mb(i),1)
!         dr(j,i,nb(i),2)=-dr(j,i,mb(i),2)
!         dr(j,i,nb(i),3)=-dr(j,i,mb(i),3)
      enddo
mols%d2r(:,:,:,j)=0
do i=1,pot%nbond
  mi=(pot%mb(i)-1)*3
  ni=(pot%nb(i)-1)*3
  do dk=1,3
    do dj=1,3
      mols%d2r(i,mi+dj,mi+dk,j)=-(mols%c(j,pot%mb(i),dj) &
                                                -mols%c(j,pot%nb(i),dj)) &
              *(mols%c(j,pot%mb(i),dk)-mols%c(j,pot%nb(i),dk))/mols%r(j,i)**3
      if (dk==dj) mols%d2r(i,mi+dj,mi+dk,j)= &
                                mols%d2r(i,mi+dj,mi+dk,j)+1/mols%r(j,i)
      mols%d2r(i,ni+dj,ni+dk,j)=mols%d2r(i,mi+dj,mi+dk,j)
      mols%d2r(i,mi+dj,ni+dk,j)=-mols%d2r(i,mi+dj,mi+dk,j)
      mols%d2r(i,ni+dj,mi+dk,j)=-mols%d2r(i,mi+dj,mi+dk,j)
    enddo
  enddo
enddo
      if (TranslationalSym) then
        !  ***  Seconds not implemented here ***
        k=pot%nbond+pot%natom
!  heights
        mols%r(j,pot%nbond+1:k)=mols%c(j,:,3)+pot%Z0depth
!  in plane periodics
        y1=matmul(mols%c(j,:,1:2),pot%RecipA)
        y2=matmul(mols%c(j,:,1:2),pot%RecipB)
        mols%r(j,k+1:pot%nredundant:4)=sin(2*Pi*y1)
        mols%r(j,k+2:pot%nredundant:4)=cos(2*Pi*y1)
        mols%r(j,k+3:pot%nredundant:4)=sin(2*Pi*y2)
        mols%r(j,k+4:pot%nredundant:4)=cos(2*Pi*y2)
        if (SurfaceCoordsIncludeHeight) then
          do i=1,pot%natom
            mols%r(j,k+4*i-3:k+4*i)=mols%r(j,k+4*i-3:k+4*i) &
                       /mols%r(j,pot%nbond+i)**2
          enddo
        endif
!  derivatives
        do i=1,pot%natom
!          even though we've stored h for the moment, the deriv is w.r.t. 1/h
          mols%dr(pot%nbond+i,i,3,j)=-1/mols%r(j,pot%nbond+i)**2
          mols%dr(k+4*i-3,i,1:2,j)=2*Pi*pot%RecipA*cos(2*Pi*y1)
          mols%dr(k+4*i-2,i,1:2,j)=-2*Pi*pot%RecipA*sin(2*Pi*y1)
          mols%dr(k+4*i-1,i,1:2,j)=2*Pi*pot%RecipB*cos(2*Pi*y2)
          mols%dr(k+4*i,i,1:2,j)=-2*Pi*pot%RecipB*sin(2*Pi*y2)
          if (SurfaceCoordsIncludeHeight) then
            mols%dr(k+4*i-3:k+4*i,i,1:2,j)=mols%dr(k+4*i-3:k+4*i,i,1:2,j) &
                       /mols%r(j,pot%nbond+i)**2
            mols%dr(k+4*i-3:k+4*i,i,3,j)=-2*mols%r(j,k+4*i-3:k+4*i) &
                       /mols%r(j,pot%nbond+i)**3
          endif
        enddo
      endif
      end subroutine intern

      subroutine InvCoords(mols,pot,j,rinv)
      use POTmodule
      implicit none
      type(POTinstance),intent(in)::mols
      type(POTobject),intent(in)::pot
      integer,intent(in)::j
      real(kind=8),dimension(pot%nredundant),intent(out)::rinv
      rinv(1:pot%nbond)=1/mols%r(j,1:pot%nbond)
      if (TranslationalSym) then
!  invert heights
        rinv(pot%nbond+1:pot%nbond+pot%natom) &
                       =1/mols%r(j,pot%nbond+1:pot%nbond+pot%natom)
!  but not in-plane coords
        rinv(pot%nbond+pot%natom+1:pot%nredundant) &
                       =mols%r(j,pot%nbond+pot%natom+1:pot%nredundant)
      endif
      end subroutine InvCoords

      subroutine neighbour(mols,pot,j)
      type(POTinstance)::mols
      type(POTobject)::pot
      integer,intent(in)::j
      real(kind=8),dimension(pot%ndata,pot%ngroup)::vt
      real(kind=8)::t,wt1
      integer::i,m,n,nf,ni
      if (keepneigh) return
!    assume a weighting function of 1/r**(2*ipow)
      mols%totsum(j)=0.d0
      vt=0
      if (TranslationalSym) then
        ni=pot%nbond+pot%natom
      else
        ni=pot%nbond
      endif
!  !ocl vi(k)
      do m=1,pot%ndata
      do n=1,pot%ngroup
         do i=1,pot%nbond
            t=1.d0/mols%r(j,i)-1.d0/pot%rda(m,pot%ip(i,n))
            vt(m,n)=t*t+vt(m,n)
         enddo
         if (TranslationalSym) then
           do i=pot%nbond+1,ni
              t=1.d0/mols%r(j,i)-pot%rda(m,pot%ip(i,n))
              vt(m,n)=t*t+vt(m,n)
           enddo
           do i=ni+1,pot%nredundant
              t=(mols%r(j,i)-pot%rda(m,pot%ip(i,n)))/2.0_8
              vt(m,n)=t*t+vt(m,n)
           enddo
         endif
      enddo
      enddo
      if (any(vt<pot%vtlim)) then
        where (vt<pot%vtlim)
          vt=1
        elsewhere
          vt=0
        endwhere
        mols%totsum(j)=1
      else
        vt=1.d0/vt**pot%ipow
        mols%totsum(j)=sum(vt)
      endif
!  we set up the massive neighbour list
!  wtol is passed in common/shepdata
!  nforc is the number of neighbours
!  mda identifies the data point in POT that neighbour k comes from
!  nda identifies which permutation neighbour k is
nf=0
do m=1,pot%ndata
  do n=1,pot%ngroup
    wt1=vt(m,n)/mols%totsum(j)
    if (wt1.gt.pot%wtol) then
      nf=nf+1
      if (nf>mols%maxf) then
        write(*,*)' *** ABORT ***'
        write(*,*)' nforc.gt.maxf'
        write(10,*)' *** ABORT ***'
        write(10,*)' nforc.gt.maxf'
        stop
      endif
      mols%mda(nf,j)=m
      mols%nda(nf,j)=n
    endif
  enddo
enddo
mols%nforc(j)=nf

if (nf==0) then
  write(*,*)j,mols%totsum(j),mols%totsum(j)>huge(1.0_8)
  write(*,*)pot%ipow,(1.0e-13_8)**pot%ipow,1/(1.0e-13_8)**pot%ipow,(1.0e-13_8)**(-pot%ipow)
  write(*,*)pot%ndata,pot%ngroup
  write(*,*)log(huge(1.0_8)),log(huge(1.0_8))/real(pot%ipow,8),exp(-log(huge(1.0_8))/real(pot%ipow,8))
  write(*,*)exp(-log(huge(1.0_8)/real(pot%ndata,8))/real(pot%ipow,8))
  stop 'neighbour'
endif

end subroutine neighbour

      subroutine dvdi(mols,pot,j)
      type(POTinstance)::mols
      type(POTobject)::pot
      integer,intent(in)::j
real(kind=8),dimension(mols%nforc(j),pot%nbond,pot%nbond)::d2w,d2tdr2
real(kind=8),dimension(pot%nbond,pot%nbond)::sumd2v
real(kind=8),dimension(mols%nforc(j),pot%nredundant)::dvt,dtdr,rinvdiff
real(kind=8),dimension(mols%nforc(j),pot%nint)::z,dei
real(kind=8),dimension(pot%nredundant)::sumdvt,rinv
real(kind=8),dimension(mols%nforc(j))::vt1,wt,ei
real(kind=8)::temp
integer,dimension(1)::p
integer::i,ii,k,ip2,l,ninv
 !  evaluate the energy and gradients
 !  we will need to have some quantities for the gradients
 !  of the weights
 !  Note an approx here, due to neglect of small weights
ip2=pot%ipow*2
ninv=min(pot%nredundant,pot%nbond+pot%natom)
call InvCoords(mols,pot,j,rinv)
do i=1,mols%nforc(j)
  rinvdiff(i,1:pot%nbond)=rinv(1:pot%nbond) &
                -1/pot%rda(mols%mda(i,j),pot%ip(1:pot%nbond,mols%nda(i,j)))
  if (TranslationalSym) then
    rinvdiff(i,pot%nbond+1:pot%nredundant)= &
        rinv(pot%nbond+1:pot%nredundant) &
                -pot%rda(mols%mda(i,j), &
                        pot%ip(pot%nbond+1:pot%nredundant,mols%nda(i,j)))
  endif
enddo
 !  calc the raw wts and totsum
do k=1,mols%nforc(j)
  vt1(k)=sum(rinvdiff(k,:)**2)
enddo
if (any(vt1<1.0e-12)) then
  p=minloc(vt1)
  wt=0
  wt(p(1))=1
  mols%totsum(j)=1
  dvt=0
  sumdvt=0
  d2w=0
else
  wt=1.d0/vt1**pot%ipow
  mols%totsum(j)=sum(wt)
  wt=wt/mols%totsum(j)
  if (doforce) then
     !      write(98,*),maxval(wt)
     !  calc the derivatives of the raw weights v
     ! dvt is actually the derivative of the primitive weight 
     !  w.r.t. r, 1/h, su(/h**2), divided by
     ! the sum of the primitive weights
    do k=1,mols%nforc(j)
      temp=ip2*wt(k)/vt1(k)
      dvt(k,1:pot%nbond)=temp*rinvdiff(k,1:pot%nbond)*rinv**2
      if (TranslationalSym) then
        dvt(k,pot%nbond+1:ninv)=temp*rinvdiff(k,pot%nbond+1:ninv)
        dvt(k,ninv+1:pot%nredundant)=-temp*rinvdiff(k,ninv+1:pot%nredundant)
      endif
    enddo
    sumdvt=sum(dvt,dim=1)
     ! for seconds, we first calculate deriv of prim wt divided by totsum
     ! below here is where the TranslationalSym stuff needs fixing
    do i=1,pot%nbond
      do k=i,pot%nbond
        d2w(:,i,k)=2*(pot%ipow+1)/vt1*dvt(:,i)*rinvdiff(:,k)*rinv(k)**2
      enddo
  !    d2w(:,i,i)=d2w(:,i,i)-dvt(:,i)*rinv(i)*(2+rinv(i)/rinvdiff(:,i))
      d2w(:,i,i)=d2w(:,i,i)-2*pot%ipow*wt/vt1*rinv(i)**3  &
                                  *(rinv(i)+2*rinvdiff(:,i))
      d2w(:,i+1:pot%nbond,i)=d2w(:,i,i+1:pot%nbond)
    enddo
    sumd2v=sum(d2w,dim=1)
     ! then put together d2w/dr2
    do k=1,pot%nbond
      do i=k,pot%nbond
        d2w(:,i,k)=d2w(:,i,k)-dvt(:,i)*sumdvt(k)-dvt(:,k)*sumdvt(i) &
                  +wt*(2*sumdvt(i)*sumdvt(k)-sumd2v(i,k))
      enddo
      d2w(:,k,k+1:pot%nbond)=d2w(:,k+1:pot%nbond,k)
    enddo
  endif
   !  calc the derivatives of the relative weights w
  do i=1,pot%nredundant
    do k=1,mols%nforc(j)
      dvt(k,i)= -wt(k)*sumdvt(i) + dvt(k,i)
    enddo
  enddo
endif
 !  construct the normal local coords for each neighbour
 !  recalling that intern calculates bond lengths NOT inverses
do k=1,mols%nforc(j)
  do i=1,pot%nint
    z(k,i)=0.d0
    do l=1,pot%nredundant
      z(k,i)=z(k,i)+pot%ut(mols%mda(k,j),i,pot%ip(l,mols%nda(k,j)))*rinv(l)
    enddo
  enddo
  z(k,:)=z(k,:)-pot%zda(mols%mda(k,j),:)
enddo
 !  calculate the potential energy: scalar term 
ei=pot%v0(mols%mda(1:mols%nforc(j),j))
 !  linear term in energy plus diagonal part of quadratic term
 !  first term in derivative of taylor poly wrt local coords
do i=1,pot%nint
  do k=1,mols%nforc(j)
    ei(k) = z(k,i)*pot%v1(mols%mda(k,j),i) &
                + 0.5d0*z(k,i)**2*pot%v2(mols%mda(k,j),i) + ei(k)
     !  dT/dxi
    if (doforce) &
        dei(k,i) = z(k,i)*pot%v2(mols%mda(k,j),i) + pot%v1(mols%mda(k,j),i)
  enddo
enddo
!if (.true.) then
if (doforce) then
   !  the O(n^2) loop doesn't apply because we diagonalised d2v!!!
   !  transform derivatives of taylor poly wrt normal local coords into 
   !  derivs wrt bondlengths NOT inverses
  do i=1,pot%nredundant
    do k=1,mols%nforc(j)
      dtdr(k,i)=0.d0
      do l=1,pot%nint
         ! dT/d(1/r), dT/d(1/h), dT/d(su/h**2)
        dtdr(k,i)=dei(k,l)*pot%ut(mols%mda(k,j),l,pot%ip(i,mols%nda(k,j)))&
                          + dtdr(k,i)
      enddo
    enddo
     ! dT/dr, dT/d(1/h), dT/d(su/h**2)
    if (i<=pot%nbond) dtdr(1:mols%nforc(j),i)=-dtdr(1:mols%nforc(j),i)*rinv(i)**2
  enddo
   ! seconds
  do k=1,mols%nforc(j)
    do i=1,pot%nbond
      do ii=1,i
        d2tdr2(k,i,ii)=0
        do l=1,pot%nint
          d2tdr2(k,i,ii)=d2tdr2(k,i,ii) &
                  +pot%v2(mols%mda(k,j),l) &
                        *pot%ut(mols%mda(k,j),l,pot%ip(i,mols%nda(k,j))) &
                        *pot%ut(mols%mda(k,j),l,pot%ip(ii,mols%nda(k,j)))
        enddo
        d2tdr2(k,i,ii)=d2tdr2(k,i,ii)*rinv(i)**2*rinv(ii)**2
        if (ii<i) d2tdr2(k,ii,i)=d2tdr2(k,i,ii)
      enddo
      do l=1,pot%nint
        d2tdr2(k,i,i)=d2tdr2(k,i,i) &
                +2*pot%ut(mols%mda(k,j),l,pot%ip(i,mols%nda(k,j)))&
                                *rinv(i)**3*dei(k,l)
      enddo
    enddo
  enddo
endif
 !  scale by relative weights, and their derivatives
mols%en(j)=sum(wt*ei)

if (abs(mols%en(j))<1.0e-20_8) then
  write(*,*)j,mols%nforc(j)
  stop 'dvdi'
endif

mols%rms(j)=sqrt(sum(wt*(mols%en(j)-ei)**2))

if (.not.doforce) return
 ! dvr is dE/dr, dE/d(1/h), dE/d(su/h**2)
do i=1,pot%nredundant
  mols%dvr(i,j)=sum(ei*dvt(:,i)+wt*dtdr(:,i))
enddo
 ! bad energy check moved to funcshep
do i=1,pot%nbond
  do k=i,pot%nbond
    mols%dv2r(i,k,j)=sum(d2w(:,i,k)*ei+dvt(:,i)*dtdr(:,k) &
                        +dvt(:,k)*dtdr(:,i)+wt*d2tdr2(:,i,k))
  enddo
  mols%dv2r(i+1:pot%nbond,i,j)=mols%dv2r(i,i+1:pot%nbond,j)
enddo
end subroutine dvdi

subroutine dvdx(mols,pot,j)
type(POTinstance)::mols
type(POTobject)::pot
integer,intent(in)::j
 !   calculates dv/dx from dv/dr*dr/dx
 ! note that the ordering in d2v is x1,y1,z1,x2,y2,z2,...
integer::i,ia,ib,ii1,ii2,ij1,ij2,ii,ij
mols%dv(j,:,:)=0
do i=1,pot%nbond
  mols%dv(j,pot%mb(i),:)=mols%dv(j,pot%mb(i),:) &
                        +mols%dvr(i,j)*mols%dr(i,pot%mb(i),:,j)
  mols%dv(j,pot%nb(i),:)=mols%dv(j,pot%nb(i),:) &
                        +mols%dvr(i,j)*mols%dr(i,pot%nb(i),:,j)
enddo
mols%d2v(:,:,j)=0
do ia=1,pot%nbond
  mols%d2v(1:3*pot%natom,1:3*pot%natom,j)= &
        mols%d2v(1:3*pot%natom,1:3*pot%natom,j) &
                +mols%dvr(ia,j)*mols%d2r(ia,1:3*pot%natom,1:3*pot%natom,j)
enddo
do ii1=1,pot%natom
  do ii2=1,3
    do ij1=1,pot%natom
      do ij2=1,3
        ii=ii1*3-3+ii2
        ij=ij1*3-3+ij2
        do ia=1,pot%nbond
          do ib=1,pot%nbond
            mols%d2v(ii,ij,j)=mols%d2v(ii,ij,j) &
               +mols%dv2r(ia,ib,j)*mols%dr(ia,ii1,ii2,j)*mols%dr(ib,ij1,ij2,j)
          enddo
        enddo
      enddo
    enddo
  enddo
enddo
end subroutine dvdx

subroutine neighbour1a(mols,pot,j)
use POTmodule
implicit none
type(POTinstance)::mols
type(POTobject)::pot
integer,intent(in)::j
 !  the original neighbour subroutine which calculates totsum from scratch
real(kind=8),dimension(pot%ndata,pot%ngroup)::vt
real(kind=8),dimension(pot%nredundant)::rinv
real(kind=8)::test,wt1,t
integer::i,i1,m,n
if (keepneigh) return
mols%totsum(j)=0.d0
call InvCoords(mols,pot,j,rinv)
do m=1,pot%ndata
  do n=1,pot%ngroup
    vt(m,n)=0.d0
    do i=1,pot%nbond
      i1=pot%ip(i,n)
      t=rinv(i)-pot%rda(m,i1)
      vt(m,n)=t*t/pot%sumb(m,i1)+vt(m,n)
    enddo
!      if (TranslationalSym) then
!        do i=pot%nbond+1,pot%nbond+pot%natom
!              i1=pot%ip(i,n)
!              t=rinv(i)-pot%rda(m,i1)
!              vt(m,n)=t*t/pot%sumb(m,i1)+vt(m,n)
!           enddo
!           do i=1,pot%nbond+pot%natom+1,pot%nredundant
!              i1=pot%ip(i,n)
!              t=(rinv(i)-pot%rda(m,i1))/2.0_8
!              vt(m,n)=t*t/pot%sumb(m,i1)+vt(m,n)
!           enddo
!         endif
!    vt(m,n)=1.d0/(vt(m,n)**pot%lowp+vt(m,n)**pot%ipow)
!    mols%totsum(j)=mols%totsum(j)+vt(m,n)
  enddo
enddo
      if (any(vt<pot%vtlim)) then
        where (vt<pot%vtlim)
          vt=1
        elsewhere
          vt=0
        endwhere
        mols%totsum(j)=1
      else
        vt=1.d0/(vt**pot%lowp+vt**pot%ipow)
        mols%totsum(j)=sum(vt)
      endif
mols%nforc(j)=0
test=pot%wtol
do m=1,pot%ndata
  do n=1,pot%ngroup
    wt1=vt(m,n)/mols%totsum(j)
    if (wt1.gt.test) then
      mols%nforc(j)=mols%nforc(j)+1
!           if (mols%nforco(j).gt.mols%maxfo) then
!             mols%nforco(j)=0
!             test=2.d0*test
!             go to 55
!           endif
      mols%mda(mols%nforc(j),j)=m
      mols%nda(mols%nforc(j),j)=n
    endif
  enddo
enddo
end subroutine neighbour1a

      subroutine neighbour1(mols,pot,j)

      use POTmodule
      implicit none
      type(POTinstance)::mols
      type(POTobject)::pot
      integer,intent(in)::j

!  the original neighbour subroutine which calculates totsum from scratch

      real(kind=8),dimension(pot%ndata,pot%ngroup)::vt
      real(kind=8),dimension(pot%nredundant)::rinv
      real(kind=8)::test,wt1,t
      integer::i,i1,m,n

!    assume a weighting function of 1/r**(2*ipow)

      mols%totsum(j)=0.d0

      call InvCoords(mols,pot,j,rinv)

      do m=1,pot%ndata
      do n=1,pot%ngroup

         vt(m,n)=0.d0

         do i=1,pot%nbond
            i1=pot%ip(i,n)
            t=rinv(i)-pot%rda(m,i1)
            vt(m,n)=t*t/pot%sumb(m,i1)+vt(m,n)
         enddo
         if (TranslationalSym) then
           do i=pot%nbond+1,pot%nbond+pot%natom
              i1=pot%ip(i,n)
              t=rinv(i)-pot%rda(m,i1)
              vt(m,n)=t*t/pot%sumb(m,i1)+vt(m,n)
           enddo
           do i=1,pot%nbond+pot%natom+1,pot%nredundant
              i1=pot%ip(i,n)
              t=(rinv(i)-pot%rda(m,i1))/2.0_8
              vt(m,n)=t*t/pot%sumb(m,i1)+vt(m,n)
           enddo
         endif

! patch to match in with dvdir.f

!         vt(m,n)=vt(m,n)
         vt(m,n)=1.d0/(vt(m,n)**pot%lowp+vt(m,n)**pot%ipow)

!  end of patch

         mols%totsum(j)=mols%totsum(j)+vt(m,n)

      enddo
      enddo

!  we set up the massive neighbour list
!  wtol is passed in common/shepdata
!  nforc is the number of neighbours
!  mda identifies the data point in POT that neighbour k comes from
!  nda identifies which permutation neighbour k is

      mols%nforco(j)=0
      test=pot%wtol*pot%outer
55    continue

      do m=1,pot%ndata
      do n=1,pot%ngroup

         wt1=vt(m,n)/mols%totsum(j)

         if (wt1.gt.test) then
           mols%nforco(j)=mols%nforco(j)+1

           if (mols%nforco(j).gt.mols%maxfo) then
             mols%nforco(j)=0
             test=2.d0*test
             go to 55
           endif

           mols%mdao(mols%nforco(j),j)=m
           mols%ndao(mols%nforco(j),j)=n
         endif

      enddo
      enddo

!      if (mols%nforco(j)<pot%nneigh) then
!        mols%nforco(j)=0
!        test=test/10.0
!        go to 55
!      endif

      end subroutine neighbour1

      subroutine neighbour2(mols,pot,j)

      use POTmodule
      implicit none
      type(POTinstance)::mols
      type(POTobject)::pot
      integer,intent(in)::j

!  modified 19-7-96 by kt to approximate totsum by totsum from previous dvdi
!  and to calculate an outer neighbour list

      real(kind=8),dimension(pot%nredundant)::rinv
      real(kind=8)::test,wt1,t
      integer::i,m,n,npow,i1

      call InvCoords(mols,pot,j,rinv)

!  we set up the outer neighbour list
!  outer is the factor by which the outer wtol is less than the inner wtol
!  wtol is the cutoff for allowing points into the neibour list
!  nforco is the number of outer neighbours
!  mdao identifies the outer data point in POT that neighbour k comes from
!  ndao identifies which permutation outer neighbour k is

      mols%nforco(j)=0
      test=pot%wtol*pot%outer*mols%totsum(j)
      if(test.gt.1.d0)then
        npow=pot%lowp
      else
        npow=pot%ipow
      endif
      test=1.d0/test**(1.d0/npow)
55    continue
      do m=1,pot%ndata
      do n=1,pot%ngroup

         wt1=0.d0

         do i=1,pot%nbond
            i1=pot%ip(i,n)
            t=rinv(i)-pot%rda(m,i1)
            wt1=t*t/pot%sumb(m,i1)+wt1
         enddo
         if (TranslationalSym) then
           do i=pot%nbond+1,pot%nbond+pot%natom
              i1=pot%ip(i,n)
              t=rinv(i)-pot%rda(m,i1)
              wt1=wt1+t*t/pot%sumb(m,i1)
           enddo
           do i=1,pot%nbond+pot%natom+1,pot%nredundant
              i1=pot%ip(i,n)
              t=(rinv(i)-pot%rda(m,i1))/2.0_8
              wt1=wt1+t*t/pot%sumb(m,i1)
           enddo
         endif


         if (wt1.lt.test) then
           mols%nforco(j)=mols%nforco(j)+1
           if (mols%nforco(j).gt.mols%maxfo) then
             mols%nforco(j)=0
             test=0.5d0*test
             go to 55
           endif
           mols%mdao(mols%nforco(j),j)=m
           mols%ndao(mols%nforco(j),j)=n
         endif

      enddo
      enddo

!      if (mols%nforco(j)<pot%nneigh) then
!        mols%nforco(j)=0
!        test=test*4
!        go to 55
!      endif

      return
      end subroutine neighbour2


      subroutine neighbour3(mols,pot,j)

      use POTmodule
      implicit none
      type(POTinstance)::mols
      type(POTobject)::pot
      integer,intent(in)::j

!  modified 19-7-96 by kt to approximate totsum by totsum from previous dvdi
!  and to calculate an inner neighbour list from the outer one

      real(kind=8),dimension(pot%nredundant)::rinv
      real(kind=8)::test,wt1,t
      integer::i,k,i1,i2,npow

      call InvCoords(mols,pot,j,rinv)

!  we set up the inner neighbour list
!  wtol is the cutoff
!  nforc is the number of neighbours
!  mda identifies the data point in POT that neighbour k comes from
!  nda identifies which permutation neighbour k is

      mols%nforc(j)=0
      test=pot%wtol*mols%totsum(j)
      if(test.gt.1.d0)then
        npow=pot%lowp
      else
        npow=pot%ipow
      endif
      test=1.d0/test**(1.d0/npow)

      do k=1,mols%nforco(j)

         wt1=0.d0

         do i=1,pot%nbond
            i1=mols%mdao(k,j)
            i2=pot%ip(i,mols%ndao(k,j))
            t=rinv(i)-pot%rda(i1,i2)
            wt1=t*t/pot%sumb(i1,i2)+wt1
         enddo
         if (TranslationalSym) then
           do i=pot%nbond+1,pot%nbond+pot%natom
              i1=mols%mdao(k,j)
              i2=pot%ip(i,mols%ndao(k,j))
              t=rinv(i)-pot%rda(i1,i2)
              wt1=wt1+t*t/pot%sumb(i1,i2)
           enddo
           do i=1,pot%nbond+pot%natom+1,pot%nredundant
              i1=mols%mdao(k,j)
              i2=pot%ip(i,mols%ndao(k,j))
              t=(rinv(i)-pot%rda(i1,i2))/2.0_8
              wt1=wt1+t*t/pot%sumb(i1,i2)
           enddo
         endif

         if (wt1.lt.test) then
           mols%nforc(j)=mols%nforc(j)+1
           if (mols%nforc(j).gt.mols%maxf) then
             write(*,*)' *** ABORT ***'
             write(*,*)' nforc.gt.maxf'
             write(*,*) ' neigh3: nforc =',mols%nforc(j)
             write(10,*)' *** ABORT ***'
             write(10,*)' nforc.gt.maxf'
             write(10,*) ' neigh3: nforc =',mols%nforc(j)
             stop
           endif
           mols%mda(mols%nforc(j),j)=mols%mdao(k,j)
           mols%nda(mols%nforc(j),j)=mols%ndao(k,j)
         endif

      enddo

      return
      end subroutine neighbour3

subroutine dvdir(mols,pot,j)
type(POTinstance)::mols
type(POTobject)::pot
integer,intent(in)::j
real(kind=8),dimension(mols%nforc(j),pot%nbond,pot%nbond)::d2w,d2tdr2
real(kind=8),dimension(pot%nbond,pot%nbond)::sumd2v
real(kind=8),dimension(mols%nforc(j),pot%nredundant)::dvt,dtdr,rinvdiff
real(kind=8),dimension(mols%nforc(j),pot%nint)::z,dei
real(kind=8),dimension(pot%nredundant)::sumdvt,rinv
real(kind=8),dimension(mols%nforc(j))::vt1,wt,ei,vt,zedp,zed2
integer,dimension(1)::p
integer::i,ii,k,l,ninv,kk
 !  evaluate the energy and gradients
 !  we will need to have some quantities for the gradients
 !  of the weights
 !  Note an approx here, due to neglect of small weights
 ! Recall that rda is already the inverses for ipart=2!
ninv=min(pot%nredundant,pot%nbond+pot%natom)
call InvCoords(mols,pot,j,rinv)
do i=1,mols%nforc(j)
  rinvdiff(i,1:pot%nbond)=rinv(1:pot%nbond) &
                -pot%rda(mols%mda(i,j),pot%ip(1:pot%nbond,mols%nda(i,j)))
  if (TranslationalSym) then
    rinvdiff(i,pot%nbond+1:pot%nredundant)= &
        rinv(pot%nbond+1:pot%nredundant) &
                -pot%rda(mols%mda(i,j), &
                        pot%ip(pot%nbond+1:pot%nredundant,mols%nda(i,j)))
  endif
enddo
 !  calc the raw wts and totsum
do k=1,mols%nforc(j)
  vt1(k)=sum(rinvdiff(k,:)**2/pot%sumb(mols%mda(k,j),pot%ip(:,mols%nda(k,j))))
enddo
if (any(vt1<1.0e-12)) then
  p=minloc(vt1)
  wt=0
  wt(p(1))=1
  mols%totsum(j)=1
  dvt=0
  sumdvt=0
  d2w=0
else
  zedp=vt1**pot%ipow
  zed2=vt1**pot%lowp
  vt=1/(zed2+zedp)
  mols%totsum(j)=sum(vt)
  wt=vt/mols%totsum(j)
  if (doforce) then
     !  calc the derivatives of the raw weights v
     ! dvt is actually the derivative of the primitive weight 
     !  divided by the sum of the primitive weights
     ! below here is where the TranslationalSym stuff needs fixing
    do k=1,mols%nforc(j)
      dvt(k,:)=2*wt(k)*vt(k)*(pot%ipow*zedp(k)+pot%lowp*zed2(k))/vt1(k) &
          *rinvdiff(k,:)*rinv**2/pot%sumb(mols%mda(k,j),pot%ip(:,mols%nda(k,j)))
    enddo
    sumdvt=sum(dvt,dim=1)
     ! for seconds, we first calculate deriv of prim wt divided by totsum
    do i=1,pot%nbond
      do k=i,pot%nbond
        do kk=1,mols%nforc(j)
          d2w(kk,i,k)=2*dvt(kk,i)*dvt(kk,k)/wt(kk) &
                       -4*wt(kk)*vt(kk)*(pot%ipow*(pot%ipow-1)*zedp(kk) &
                               +pot%lowp*(pot%lowp-1)*zed2(kk))/vt1(kk)**2 &
                       *rinvdiff(kk,i)*rinvdiff(kk,k) &
                               *rinv(i)**2*rinv(k)**2 &
                       /pot%sumb(mols%mda(kk,j),pot%ip(i,mols%nda(kk,j))) &
                       /pot%sumb(mols%mda(kk,j),pot%ip(k,mols%nda(kk,j)))
        enddo
      enddo
      d2w(:,i,i)=d2w(:,i,i)-dvt(:,i)*rinv(i)*(2+rinv(i)/rinvdiff(:,i))
      d2w(:,i+1:pot%nbond,i)=d2w(:,i,i+1:pot%nbond)
    enddo
    sumd2v=sum(d2w,dim=1)
     ! then put together d2w/dr2
    do k=1,pot%nbond
      do i=k,pot%nbond
        d2w(:,i,k)=d2w(:,i,k)-dvt(:,i)*sumdvt(k)-dvt(:,k)*sumdvt(i) &
                +wt*(2*sumdvt(i)*sumdvt(k)-sumd2v(i,k))
      enddo
      d2w(:,k,k+1:pot%nbond)=d2w(:,k+1:pot%nbond,k)
    enddo
  endif
endif
if (doforce) then
   !  calc the derivatives of the relative weights w
  do i=1,pot%nredundant
    do k=1,mols%nforc(j)
      dvt(k,i)= -wt(k)*sumdvt(i) + dvt(k,i)
    enddo
  enddo
endif
 !  construct the normal local coords for each neighbour
 !  recalling that intern calculates bond lengths NOT inverses
do k=1,mols%nforc(j)
  do i=1,pot%nint
    z(k,i)=0.d0
    do l=1,pot%nredundant
      z(k,i)=z(k,i)+pot%ut(mols%mda(k,j),i,pot%ip(l,mols%nda(k,j)))*rinv(l)
    enddo
  enddo
  z(k,:)=z(k,:)-pot%zda(mols%mda(k,j),:)
enddo
 !  calculate the potential energy: scalar term 
ei=pot%v0(mols%mda(1:mols%nforc(j),j))
 !  linear term in energy plus diagonal part of quadratic term
 !  first term in derivative of taylor poly wrt local coords
do i=1,pot%nint
  do k=1,mols%nforc(j)
    ei(k) = z(k,i)*pot%v1(mols%mda(k,j),i) &
                + 0.5d0*z(k,i)**2*pot%v2(mols%mda(k,j),i) + ei(k)
  enddo
enddo
if (doforce) then
  do i=1,pot%nint
    do k=1,mols%nforc(j)
     !  dT/dxi
      dei(k,i) = z(k,i)*pot%v2(mols%mda(k,j),i) + pot%v1(mols%mda(k,j),i)
    enddo
  enddo
 !  the O(n^2) loop doesn't apply because we diagonalised d2v!!!
 !  transform derivatives of taylor poly wrt normal local coords into 
 !  derivs wrt bondlengths NOT inverses
do i=1,pot%nredundant
  do k=1,mols%nforc(j)
    dtdr(k,i)=0.d0
    do l=1,pot%nint
       ! dT/d(1/r), dT/d(1/h), dT/d(su/h**2)
      dtdr(k,i)=dei(k,l)*pot%ut(mols%mda(k,j),l,pot%ip(i,mols%nda(k,j)))&
                        + dtdr(k,i)
    enddo
  enddo
   ! dT/dr, dT/d(1/h), dT/d(su/h**2)
  if (i<=pot%nbond) dtdr(1:mols%nforc(j),i)=-dtdr(1:mols%nforc(j),i)*rinv(i)**2
enddo
 ! seconds
do k=1,mols%nforc(j)
  do i=1,pot%nbond
    do ii=1,i
      d2tdr2(k,i,ii)=0
      do l=1,pot%nint
        d2tdr2(k,i,ii)=d2tdr2(k,i,ii) &
                  +pot%v2(mols%mda(k,j),l) &
                        *pot%ut(mols%mda(k,j),l,pot%ip(i,mols%nda(k,j))) &
                        *pot%ut(mols%mda(k,j),l,pot%ip(ii,mols%nda(k,j)))
      enddo
      d2tdr2(k,i,ii)=d2tdr2(k,i,ii)*rinv(i)**2*rinv(ii)**2
      if (ii<i) d2tdr2(k,ii,i)=d2tdr2(k,i,ii)
    enddo
    do l=1,pot%nint
      d2tdr2(k,i,i)=d2tdr2(k,i,i) &
                +2*pot%ut(mols%mda(k,j),l,pot%ip(i,mols%nda(k,j)))&
                                *rinv(i)**3*dei(k,l)
    enddo
  enddo
enddo
endif
 !  scale by relative weights, and their derivatives
mols%en(j)=sum(wt*ei)
mols%rms(j)=sqrt(sum(wt*(mols%en(j)-ei)**2))
 ! bad energy check moved to funcshep
if (doforce) then
   ! dvr is dE/dr, dE/d(1/h), dE/d(su/h**2)
  do i=1,pot%nredundant
    mols%dvr(i,j)=sum(ei*dvt(:,i)+wt*dtdr(:,i))
  enddo
  do i=1,pot%nbond
    do k=i,pot%nbond
      mols%dv2r(i,k,j)=sum(d2w(:,i,k)*ei+dvt(:,i)*dtdr(:,k) &
                        +dvt(:,k)*dtdr(:,i)+wt*d2tdr2(:,i,k))
    enddo
    mols%dv2r(i+1:pot%nbond,i,j)=mols%dv2r(i,i+1:pot%nbond,j)
  enddo
endif
end subroutine dvdir

end module shepard

