!-----------------------------------------------------------------------
!     Returns quadratic interpolation of data from disk.
!     Used in particular to interpolate time-data.
!     See also lib/utilities/dynaccess.c
!-----------------------------------------------------------------------      
      module linint

use decimal, only: long, dop
use logdat

      implicit none
      private
      public :: linint1d,li_update,li_backdate,li_browse,li_load
      contains

!-----------------------------------------------------------------------      

      real(dop) function linint1d(t,currfn)

use global
use channels
use linintmod

      implicit none

      real(dop), intent(in)     :: t
      integer(long), intent(in) :: currfn

! ___ functions
      integer(long), external :: getlong
      real(dop), external     :: getdouble, fqfit

! ___ variables
      integer(long)                     :: index, buflen
      integer(long), dimension(2)       :: pointer
      real(dop)                         :: starttime,timestep,frac,y1,y2,y3,y4

      routine='linint1d'

#ifdef DYN_DEBUG
      write(0,'(a)')'(1) pointers:'
      write(0,'(2(i8,1x),$)')(li_file(index),  index=1,2)
      write(0,'(2(i8,1x),$)')(li_start(index), index=1,2)
      write(0,'(2(i8,1x),$)')(li_step(index),  index=1,2)
      write(0,'(2(i8,1x),$)')(li_buflen(index),index=1,2)
      write(0,'(2(i8,1x),$)')(li_buffer(index),index=1,2)
      write(0,'(2(i8,1x),$)')(li_init(index),  index=1,2)
      write(0,*)
#endif

      if(currfn.gt.li_total)then
        message="number of function called exceeds total number of "//&
          "functions, browsed - internal error?"
        call errormsg
      end if

      if(getlong(li_init, currfn).eq.0) call li_load(currfn)

      buflen = getlong(li_buflen, currfn)
      starttime = getdouble(li_start, currfn)
      timestep = getdouble(li_step, currfn)
      call getlli(pointer, li_buffer, currfn)

      index = int( (t-starttime+timestep)/timestep )
      frac = (t-starttime+timestep)/timestep - index

      routine='linint1d'
      linint1d = 0.0_dop
      if(index.le.0)then
         write(6,*) t, index, starttime, timestep, buflen
         message='data below range'
         call errormsg
      elseif(index.gt.buflen)then
         write(6,*) t, index, starttime, timestep, buflen,  &
                    starttime+timestep*(buflen-1)
         message='data above range'
         call errormsg
      elseif(index.eq.1)then
         y1=getdouble(pointer,index  )
         y2=getdouble(pointer,index+1)
         y3=getdouble(pointer,index+2)
         linint1d = fqfit(frac-1.0_dop,y1,y2,y3)
      elseif(index.eq.buflen-1)then
         y1=getdouble(pointer,index-1)
         y2=getdouble(pointer,index  )
         y3=getdouble(pointer,index+1)
         linint1d = fqfit(frac,y1,y2,y3)
      elseif(index.eq.buflen)then
         linint1d = getdouble(pointer,index)
      else
         y1=getdouble(pointer,index-1)
         y2=getdouble(pointer,index  )
         y3=getdouble(pointer,index+1)
         y4=getdouble(pointer,index+2)
         linint1d = frac*fqfit(frac-1.0_dop,y2,y3,y4) + &
                    (1.0_dop-frac)*fqfit(frac,y1,y2,y3)
      end if

      return
      end function linint1d

!-----------------------------------------------------------------------
!     load data from disk into buffer
!-----------------------------------------------------------------------      
      subroutine li_load(currfn)

use global
use channels
use linintmod

      implicit none

      integer(long), intent(in) :: currfn

! ___ functions
      integer(long), external :: slen, realsize, getlong
      real(dop), external     :: getdouble

! ___ constants
      integer(long), parameter          :: stringlen = 160

! ___ variables
      integer(long)                     :: line, ierr, i
      integer(long)                     :: dataset, buflen, newlength
      integer(long), dimension(2)       :: pointer

      character(len=240) :: string, filename
      real(dop)          :: datapoint, currtime, timestep
      real(dop), save    :: starttime

      common /licb/ pointer

      routine='li_load'

! ___ now allocate buffer and write buffer variable
      buflen = getlong(li_buflen, currfn)
      call alloc(pointer, realsize()*buflen)
      call setlli(pointer, li_buffer, currfn)

      call getstring(filename, li_file, currfn)

      newlength = slen(filename)
      inptit = filename(1:newlength)
      call abspath(inptit,newlength)

      open(idump, file=inptit(1:newlength), status="old", iostat=ierr)
      if(ierr.ne.0)then
         write(string,'(i4)')ierr
         message='Data filename "'//inptit(1:newlength)//&
           '" not found, ierr='//string(1:4)
         call errormsg
      end if
      line=0
      dataset=0
      do while(.true.)
         line=line+1
         read(idump,'(a)',err=100,end=100)string
         if(string(1:1).eq.'#') goto 20
         do i=1,stringlen
            if(string(i:i).ne.' ')goto 10
         end do
 10      continue
         if(i.ge.stringlen) goto 20
         dataset=dataset+1
         if(dataset.gt.buflen) then
           message="number of datasets exceeds number allocated, "// &
             "filename changed?"
           call errormsg
         end if

! ______ compiler "feature": only use '*' as format in reads from string
         read(string,*,err=200) currtime, datapoint
         call setdouble(datapoint, pointer, dataset)

         if(dataset.eq.1)then
            starttime=currtime
         else
            if(dataset.eq.2)then
               timestep=currtime-starttime
               if(timestep.le.0.0_dop)then
                   write(string, '(e16.8)')timestep
                   message='time step should be positive but it is'//&
                      string(1:16)
                   call errormsg
               end if
            else
! _________ allow only linear time (or x) advance, equidistant spacing.            
               if(&
                  currtime.lt.&
                  starttime+(timestep-1.0e-7_dop*timestep)*(dataset-1)&
                  .or.&
                  currtime.gt.&
                  starttime+(timestep+1.0e-7_dop*timestep)*(dataset-1)&
               )then
                  write(string,'(i4.4)')line
                  message='No equidistant spacing in '//&
                    'filename "'//filename(1:slen(filename))//&
                    '" , line '//string(1:4)
                  call errormsg
               end if
            end if
         end if
 20      continue
      end do

100   continue

      close(idump)

#ifdef LI_DEBUG
! ___ write check filename
      write(string, '(i4.4)')currfn
      open(idump, file="chkli1d"//string(1:4)//".dat", status="unknown")
      write(idump,'(a)') &
         '# parsed filename "'//filename(1:slen(filename))//'"'
      write(idump,'(a,i6)')'# lines read: ',line
      write(idump,'(a,i6)')'# data sets allocated:  ',buflen
      write(idump,'(a,i6)')'# data sets read:  ',dataset
      write(idump,'(2(a,e16.8))') &
         '# start: ',starttime,', step: ', timestep
      do i=1,buflen
         write(idump,'(2e16.8)')starttime+(i-1)*timestep, &
           getdouble(pointer, i)
      end do
      close(idump)
#endif


      call setlong(onei, li_init, currfn)

      return

200   continue
      write(string,'(i4.4)')line
      message='Data filename "'//filename(1:slen(filename))// &
         '" contains malformed entry on line'//string(1:4)
      call errormsg

!      ierr=getdouble(pointer,i) ! useless statement is to make fcheck happy

      end subroutine li_load


!-----------------------------------------------------------------------
!     IA: modify parts of the buffer
!     necessary for OCT
!-----------------------------------------------------------------------      
      subroutine li_update(itime,val1,val2,val3)
      implicit none

      integer(long), intent(in) :: itime
      real(dop), intent(out)    :: val1,val2,val3

      integer(long), dimension(2)       :: pointer

      common /licb/ pointer

      call setdouble(val1, pointer, itime+3)
      call setdouble(val2, pointer, itime+4)
      call setdouble(val3, pointer, itime+5)

      return
      end subroutine li_update

!-----------------------------------------------------------------------
!     MS: modify parts of the buffer
!     necessary for OCT for simultaneous propagations
!     (backward)
!----------------------------------------------------------------------- 

      subroutine li_backdate(itime,val1,val2,val3)
      implicit none

      integer(long), intent(in) :: itime
      real(dop), intent(out)    :: val1,val2,val3

      integer(long), dimension(2)       :: pointer

      common /licb/ pointer

      call setdouble(val1, pointer, itime+4)
      call setdouble(val2, pointer, itime+3)
      call setdouble(val3, pointer, itime+2)

      return
      end subroutine li_backdate

!-----------------------------------------------------------------------
!     browse through data filename for interpolation and ask for memory
!-----------------------------------------------------------------------
      integer(long) function li_browse(filename)

use global
use channels
use linintmod

      implicit none

      character(len=*), intent(in) :: filename

! ___ functions
      integer(long), external :: realsize, integersize, slen

! ___ constants
      integer(long), parameter :: stringlen = 160

! ___ variables
      integer(long)       :: line,ierr,ilbl,jlbl,buflen,newlength,i

      character(len=160)       :: string
      real(dop)                :: datapoint, starttime, currtime, timestep

      routine='li_browse'

      li_total = li_total + 1
      li_browse = li_total
      if(li_total .gt. li_allocd)then
! _____ we're using the fact that realloc is the same as malloc if the
! _____ ptr is NULL, which we made sure in mctdh/default.F

        li_allocd = li_allocd + li_blocksize
        call resizestring(li_file,li_allocd,li_allocd-li_blocksize)
        call resize(li_start,realsize()*li_allocd)
        call resize(li_step, realsize()*li_allocd)
        call resize(li_buflen,  integersize()*li_allocd)
        call resize(li_buffer,2*integersize()*li_allocd)
        call resize(li_init,    integersize()*li_allocd)

      end if

#ifdef DYN_DEBUG
      write(0,'(a)')'(2) pointers:'
      write(0,'(2(i8,1x),$)')(li_file(i),i=1,2)
      write(0,'(2(i8,1x),$)')(li_start(i),i=1,2)
      write(0,'(2(i8,1x),$)')(li_step(i),i=1,2)
      write(0,'(2(i8,1x),$)')(li_buflen(i),i=1,2)
      write(0,'(2(i8,1x),$)')(li_buffer(i),i=1,2)
      write(0,'(2(i8,1x),$)')(li_init(i),i=1,2)
      write(0,*)
#endif

      ilbl=1
      jlbl=slen(filename)

      call setstring(filename(ilbl:jlbl), li_file, li_total)

      inptit = filename(ilbl:jlbl)
      newlength = jlbl-ilbl+1
      call abspath(inptit,newlength)
      write(ilog,'(3a,i4)') 'interpolation file ',inptit(1:newlength), &
       ', file number',li_total

      open(idump, file=inptit(1:newlength), status="old", iostat=ierr)
      if(ierr.ne.0)then
         write(string,'(i4)')ierr
         message='Data filename "'//inptit(1:newlength)//'" not found' &
            //', ierr='//string(1:4)
         call errormsg
      end if

      line=0
      buflen=0
 12   continue
         line=line+1
         read(idump,'(a)',err=100,end=100) string
         if(string(1:1).eq.'#')goto 20
         do i=1,stringlen
            if(string(i:i).ne.' ')goto 10
         end do
 10      continue
         if(i.ge.stringlen)goto 20

         buflen=buflen+1

! ______ compiler "feature": only ever use '*' format in reads from str
         read(string,*,err=200) currtime, datapoint
         if(buflen.eq.1)then
            starttime=currtime
         else
            if(buflen.eq.2)then
               timestep=currtime-starttime
               if(timestep.le.0.0_dop)then
                   write(string, '(e16.8)')timestep
                   message='time timestep should be positive but is'// &
                      string(1:16)
                   call errormsg
               end if
            else
! ____________ allow only linear time advance  
               if(&
                  currtime.lt.&
                  starttime+(timestep-1.0e-7_dop*timestep)*(buflen-1)&
                  .or.&
                  currtime.gt.&
                  starttime+(timestep+1.0e-7_dop*timestep)*(buflen-1)&
               )then
                  write(string,'(i4.4)')line
                  message='No equidistant spacing in '//&
                    'filename "'//filename(ilbl:jlbl)//&
                    '" , line '//string(1:4)
                  call errormsg
               end if
            end if
         end if
 20      continue
      goto 12

100   continue
      write(ilog,'(i6,a)') buflen,' data sets read.'

      close(idump)

! ___ now we have the number of data sets, ask for enough memory

! ___ mrdim code could follow here if it were easy to pass mc,mr,mi,ml
! ___ parameters to a evaluate function linint1d(t) - but it isn't
!      li_mrstart = mrdim
!      mrdim = mrdim+buflen+1
! ___ prospective mrdim code ends

! ___ now save collected data in dynamical arrays

      call setlong(buflen, li_buflen, li_total)
      call setlong(zeroi, li_init, li_total)
      call setdouble(starttime, li_start, li_total)
      call setdouble(timestep, li_step, li_total)

! ___ allocation of actual buffer is done in li_load, when we need it

      return

200   continue
      write(string,'(i4.4)')line
      message='Data filename "'//filename(ilbl:jlbl)//&
         '" contains malformed entry on line'//string(1:4)
      call errormsg

      end function li_browse


!-----------------------------------------------------------------------      
      end module linint
      

