! POTENTIALPROGRAMM FUER NOF
!
!  Es berechnet das 3D-Potential in den Jacobi-Koordinaten:
!     y1=Laenge O=O, y2=Abstand von O zum Schwerpunkt S des O=O,
!     y3=Winkel OSO in rad.
!
!     Die Abstaende sind in a.u. und der Winkel in rad einzugeben.
!     Der vom Programm gelieferte Potentialwert v ist in a.u.(Hartree).
!     Potentialbereich: y1 von 1.8 bis 3.0 a.u.
!                       y2     3.5 a.u. bis beliebig (analytisch fort-
!                                                      gesetzt)
!                       y3      0 bis pi (180 Grad).
!      Franck_Condon-Punkt bei y1=2.1550 a.u.
!                              y2=4.3137 a.u.
!                              y3=127.4 Grad
!     Initialisierung des Programms mit  CALL INNOFX ,
!     Aufruf mit CALL NOFX(Y1,Y2,Y3,V).
!     Die Potentialdaten aus dem file DATEN werden auf
!     unit 50 eingelesen.
!#######################################################################

      subroutine defnof_ec(hopilab,label)

      implicit none

      integer hopilab
      character(len=*) label


      if (label .eq. 'nof_ec0') then
         hopilab = 1
!
! -- Read data for the surface --
!
         call innof_ec0

      else if (label .eq. 'nof_ec1') then
         hopilab = 2
!
! -- Read data for the surface --
!
         call innof_ec1

      else if (label .eq. 'nof_ec2') then
         hopilab = 3
!
! -- Read data for the surface --
!
         call innof_ec2


      else if (label .eq. 'nof_ec3') then
         hopilab = 4
!
! -- Read data for the surface --
!
         call innof_ec3

      else if (label .eq. 'nof_ec0_pt2') then
         hopilab = 5
!
! -- Read data for the surface --
!
         call innof_pt2_ec0

      else if (label .eq. 'nof_ec1_pt2') then
         hopilab = 6
!
! -- Read data for the surface --
!
         call innof_pt2_ec1

      else if (label .eq. 'nof_ec2_pt2') then
         hopilab = 7

!
! -- Read data for the surface --
!
         call innof_pt2_ec2

      else if (label .eq. 'nof_ec3_pt2') then
         hopilab = 8

         call innof_pt2_ec3

      else if (label .eq. 'nof_ec_s0') then
 
         hopilab = 9

      else if (label .eq. 'nof_ec_s1') then
         
         hopilab = 10

      else if (label .eq. 'nof_ec_s2') then
         
         hopilab = 11

      else if (label .eq. 'nof_ec_s3') then
         
         hopilab = 12

!
!  Read mrci data for FNO
!

      else if (label .eq. 'nof_ec_mrci_s0') then
        
         hopilab = 13

      else if (label .eq. 'nof_ec_mrci_s1') then
        
         hopilab = 14

      else if (label .eq. 'nof_ec_mrci_s2') then
        
         hopilab = 15

      else if (label .eq. 'nof_ec_mrci_s3') then
        
         hopilab = 16

      else if (label .eq. 'nof_ec_msmrcaspt2_s0') then
        
         hopilab = 17

      else if (label .eq. 'nof_ec_msmrcaspt2_s1') then
        
         hopilab = 18

      else if (label .eq. 'nof_ec_msmrcaspt2_s2') then
        
         hopilab = 19

      else if (label .eq. 'nof_ec_msmrcaspt2_s3') then
        
         hopilab = 20

      else if (label .eq. 'nof_ec_2d_s1') then
        
         hopilab = 21

      endif

      return
      end

!#######################################################################

      subroutine func_nof_ec(v,gpoint,ifunc,icoord)

use paths
use splinepotmod
use splinepot2dmod

      implicit none


      integer icoord,ifunc,ilbl
      real*8 gpoint(3),v,r1,r2,alpha,rv,rd,theta,pi,angfac,b,arg
      real*8 tfac,pesmax,angstroem
      character(len=120) filename,datapath

      parameter (tfac=0.533205d0)
      parameter (angstroem=0.52917725d0)

      pi = 4.0d0*atan(1.0d0)
      angfac = 180.0d0/pi
      datapath="nof_data"
   
      if (icoord .eq. 1) then
         r1 = gpoint(1)
         r2 = gpoint(2)
         alpha = gpoint(3)

      else if (icoord .eq. 2) then
         rv = gpoint(1)
         rd = gpoint(2)
         theta = gpoint(3)
 
! Transformation auf Molekularkoordinaten (r1,r2,alpha), in denen
! die Potentialdaten gegeben sind.
 
         r1 = rv
         b = tfac*rv
         r2=dsqrt(b*b+rd*rd+2.d0*b*rd*dcos(theta))
         arg=(b**2+r2**2-rd**2)/(2.d0*b*r2)
         if(arg.gt.1.d0) then
            alpha=0.d0
         else if(arg.lt.-1.d0) then
            alpha=3.141592654d0
         else
            alpha=dacos(arg)
         end if

         alpha = alpha   

      else if (icoord .eq. 3) then
         rv = gpoint(1)
         rd = gpoint(2)

         r1 = rv
         r2 = rd

      endif      
 
      pesmax=10.0d0     

!     if (alpha .lt. 0.1d0) then
!       v=pesmax 
!       return
!     endif

!     if (alpha .gt. 3.1d0) then
!       v=pesmax     
!       return
!     endif
     
!     if (r1 .lt. 1.5d0) then
!        v = pesmax
!        return
!     endif

!     if (r2 .lt. 1.5d0) then
!        v = pesmax
!        return
!     endif
      
      go to (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21) ifunc
    
      print*,'ERROR: NOF function not found :',ifunc
      stop

    1 call nof_ec0(r1,r2,alpha,v)
      return

    2 call nof_ec1(r1,r2,alpha,v)
      return

    3 call nof_ec2(r1,r2,alpha,v)
      return

    4 call nof_ec3(r1,r2,alpha,v)
      return

    5 call nof_pt2_ec0(r1,r2,alpha,v)
      return

    6 call nof_pt2_ec1(r1,r2,alpha,v)
      return
 
    7 call nof_pt2_ec2(r1,r2,alpha,v)
      return   

    8 call nof_pt2_ec3(r1,r2,alpha,v)
      return   

!
! Good data with C.Sanz splinefit for CASSCF
!

    9 alpha = alpha*angfac
      r1 = r1*angstroem
      r2 = r2*angstroem
      ilbl=index(datapath,' ')-1
      filename=datapath(1:ilbl)//'/nof_ec_s0.inp'
      call splinepot_css(r1,r2,alpha,v,filename)
      v = v+(228.79853122)
      return 

   10 alpha = alpha*angfac
      r1 = r1*angstroem
      r2 = r2*angstroem
      ilbl=index(datapath,' ')-1
      filename=datapath(1:ilbl)//'/nof_ec_s1.inp'
      call splinepot_css(r1,r2,alpha,v,filename)
      v = v+(228.79853122)
      return   

   11 alpha = alpha*angfac
      r1 = r1*angstroem
      r2 = r2*angstroem
      ilbl=index(datapath,' ')-1
      filename=datapath(1:ilbl)//'/nof_ec_s2.inp'
      call splinepot_css(r1,r2,alpha,v,filename)
      v = v+(228.79853122)
      return   
   
   12  alpha = alpha*angfac
      r1 = r1*angstroem
      r2 = r2*angstroem
      ilbl=index(datapath,' ')-1
      filename=datapath(1:ilbl)//'/nof_ec_s3.inp'
      call splinepot_css(r1,r2,alpha,v,filename)
      v = v+(228.79853122)
      return

!
! Good data with C.Sanz splinefit for MRCI
!

   13 alpha = alpha*angfac
      r1 = r1*angstroem
      r2 = r2*angstroem
      ilbl=index(datapath,' ')-1
      filename=datapath(1:ilbl)//'/nof_ec_mrci_s0.inp'
      call splinepot_css(r1,r2,alpha,v,filename)
      v = v+(229.08080701)
      return 

   14 alpha = alpha*angfac
      r1 = r1*angstroem
      r2 = r2*angstroem
      ilbl=index(datapath,' ')-1
      filename=datapath(1:ilbl)//'/nof_ec_mrci_s1.inp'
      call splinepot_css(r1,r2,alpha,v,filename)
      v = v+(229.08080701)
      return   

   15 alpha = alpha*angfac
      r1 = r1*angstroem
      r2 = r2*angstroem
      ilbl=index(datapath,' ')-1
      filename=datapath(1:ilbl)//'/nof_ec_mrci_s2.inp'
      call splinepot_css(r1,r2,alpha,v,filename)
      v = v+(229.08080701)
      return   
   
   16 alpha = alpha*angfac
      r1 = r1*angstroem
      r2 = r2*angstroem
      ilbl=index(datapath,' ')-1
      filename=datapath(1:ilbl)//'/nof_ec_mrci_s3.inp'
      call splinepot_css(r1,r2,alpha,v,filename)
      v = v+(229.08080701)
      return 

!
! Good data with C.Sanz splinefit for MS-MR-CASPT2
!

   17 alpha = alpha*angfac
      r1 = r1*angstroem
      r2 = r2*angstroem
      ilbl=index(datapath,' ')-1
      filename=datapath(1:ilbl)//'/nof_ec_msmrcaspt2_s0.inp'
      call splinepot_css(r1,r2,alpha,v,filename)
      v = v+(229.05914562)
      return 

   18 alpha = alpha*angfac
      r1 = r1*angstroem
      r2 = r2*angstroem
      ilbl=index(datapath,' ')-1
      filename=datapath(1:ilbl)//'/nof_ec_msmrcaspt2_s1.inp'
      call splinepot_css(r1,r2,alpha,v,filename)
      v = v+(229.05914562)
      return 

   19 alpha = alpha*angfac
      r1 = r1*angstroem
      r2 = r2*angstroem
      ilbl=index(datapath,' ')-1
      filename=datapath(1:ilbl)//'/nof_ec_msmrcaspt2_s2.inp'
      call splinepot_css(r1,r2,alpha,v,filename)
      v = v+(229.05914562)
      return 

   20 alpha = alpha*angfac
      r1 = r1*angstroem
      r2 = r2*angstroem
      ilbl=index(datapath,' ')-1
      filename=datapath(1:ilbl)//'/nof_ec_msmrcaspt2_s3.inp'
      call splinepot_css(r1,r2,alpha,v,filename)
      v = v+(229.05914562)
      return 

   21 r1 = r1
      r2 = r2
      ilbl=index(datapath,' ')-1
      filename=datapath(1:ilbl)//'/nof_ec_2d_s1.inp'
      call splinepot2d_css(r1,r2,v,filename)
      v = v / 27.2114
      return 

      end

!#######################################################################

      subroutine nof_ec0(y1,y2,y3,v)

      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      DIMENSION rr(3),fin(4)

      parameter (ndx=160,ndy=60,ndz=110)
      common/gridnof0/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof0/

!     if (y1.le.y2) then
!        rr(1)=y2
!        rr(2)=y1
!     else
!        rr(1)=y1
!        rr(2)=y2
!     endif
      rr(1)=y1
      rr(2)=y2
      rr(3)=y3

!-----------------------------------------------------------------------
! Surface only defined in following region
!-----------------------------------------------------------------------
      rr(1)=max(rr(1),x(1))
      rr(1)=min(rr(1),x(nx))
      rr(2)=max(rr(2),y(1))
      rr(2)=min(rr(2),y(ny))
      rr(3)=max(rr(3),z(1))
      rr(3)=min(rr(3),z(nz))

      call naturl_nof_ec0(rr,0,fin)
      v=fin(1)/27.2114d0
      return

      END

!#######################################################################

      subroutine nof_ec1(y1,y2,y3,v)

      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      DIMENSION rr(3),fin(4)

      parameter (ndx=160,ndy=60,ndz=110)
      common/gridnof1/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof1/

!     if (y1.le.y2) then
!        rr(1)=y2
!        rr(2)=y1
!     else
!        rr(1)=y1
!        rr(2)=y2
!     endif
      rr(3)=y3
      rr(1)=y1
      rr(2)=y2
      rr(3)=y3

!-----------------------------------------------------------------------
! Surface only defined in following region
!-----------------------------------------------------------------------
      rr(1)=max(rr(1),x(1))
      rr(1)=min(rr(1),x(nx))
      rr(2)=max(rr(2),y(1))
      rr(2)=min(rr(2),y(ny))
      rr(3)=max(rr(3),z(1))
      rr(3)=min(rr(3),z(nz))

      call naturl_nof_ec1(rr,0,fin)
      v=fin(1)/27.2114d0
      return

      END

!#######################################################################

      subroutine nof_ec2(y1,y2,y3,v)

      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      DIMENSION rr(3),fin(4)

      parameter (ndx=160,ndy=60,ndz=110)
      common/gridnof2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof2/

!     if (y1.le.y2) then
!        rr(1)=y2
!        rr(2)=y1
!     else
!        rr(1)=y1
!        rr(2)=y2
!     endif
      rr(3)=y3
      rr(1)=y1
      rr(2)=y2
      rr(3)=y3

!-----------------------------------------------------------------------
! Surface only defined in following region
!-----------------------------------------------------------------------
      rr(1)=max(rr(1),x(1))
      rr(1)=min(rr(1),x(nx))
      rr(2)=max(rr(2),y(1))
      rr(2)=min(rr(2),y(ny))
      rr(3)=max(rr(3),z(1))
      rr(3)=min(rr(3),z(nz))

      call naturl_nof_ec2(rr,0,fin)
      v=fin(1)/27.2114d0
      return

      END

!#######################################################################

      subroutine nof_ec3(y1,y2,y3,v)

      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      DIMENSION rr(3),fin(4)

      parameter (ndx=160,ndy=60,ndz=110)
      common/gridnof3/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof3/

!     if (y1.le.y2) then
!        rr(1)=y2
!        rr(2)=y1
!     else
!        rr(1)=y1
!        rr(2)=y2
!     endif
      rr(3)=y3
      rr(1)=y1
      rr(2)=y2
      rr(3)=y3

!-----------------------------------------------------------------------
! Surface only defined in following region
!-----------------------------------------------------------------------
      rr(1)=max(rr(1),x(1))
      rr(1)=min(rr(1),x(nx))
      rr(2)=max(rr(2),y(1))
      rr(2)=min(rr(2),y(ny))
      rr(3)=max(rr(3),z(1))
      rr(3)=min(rr(3),z(nz))

      call naturl_nof_ec3(rr,0,fin)
      v=fin(1)/27.2114d0
      return

      END

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
!
!  INITIALISIERUNG DES SPLINE-PROGRAMMS (EINLESEN DER DATEN):
!
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      subroutine innof_ec0

use paths

      IMPLICIT REAL*8 (A-H,O-Z)
      implicit integer (i-n)

      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      common /surfnof0/ en11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridnof0/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof0/,/gridnof0/
!
      OPEN (50,FILE="nof_data/nof_ec0_data",status="old",iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a,/a)') &
          'Cannot open the surface data file for NOF S0:', &
           "nof_data/nof_ec0_data"
         stop
      endif

      READ (50,*) NX,(X(I),I=1,NX)
      READ (50,*) NY,(Y(I),I=1,NY)
      READ (50,*) NZ,(Z(I),I=1,NZ)
      NXYZ=NX*NY*NZ
      READ (50,*) (EN11(I),I=1,NXYZ)                                 
      READ (50,*) (PX11(I),I=1,NXYZ)                                 
      READ (50,*) (PY11(I),I=1,NXYZ)                                 
      READ (50,*) (PZ11(I),I=1,NXYZ)                                 
      READ (50,*) (PXY11(I),I=1,NXYZ)                                
      READ (50,*) (PXZ11(I),I=1,NXYZ)                                
      READ (50,*) (PYZ11(I),I=1,NXYZ)                                
      READ (50,*) (PXYZ11(I),I=1,NXYZ)                               

      CLOSE (50)
!
      return
 1000 write(6,'(a/,a/,2a)') ' Cannot read the nof_ec0_data file.'
      stop
      end

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
!
!  INITIALISIERUNG DES SPLINE-PROGRAMMS (EINLESEN DER DATEN):
!
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      subroutine innof_ec1

use paths

      IMPLICIT REAL*8 (A-H,O-Z)
      implicit integer (i-n)

      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      common /surfnof1/ en11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridnof1/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof1/,/gridnof1/
!
      OPEN (50,FILE="nof_data/nof_ec1_data",status="old",iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a,/a)') &
          'Cannot open the surface data file for NOF S1:', &
           "nof_data/nof_ec1_data"
         stop
      endif

      READ (50,*) NX,(X(I),I=1,NX)
      READ (50,*) NY,(Y(I),I=1,NY)
      READ (50,*) NZ,(Z(I),I=1,NZ)
      NXYZ=NX*NY*NZ
      READ (50,*) (EN11(I),I=1,NXYZ)                                 
      READ (50,*) (PX11(I),I=1,NXYZ)                                 
      READ (50,*) (PY11(I),I=1,NXYZ)                                 
      READ (50,*) (PZ11(I),I=1,NXYZ)                                 
      READ (50,*) (PXY11(I),I=1,NXYZ)                                
      READ (50,*) (PXZ11(I),I=1,NXYZ)                                
      READ (50,*) (PYZ11(I),I=1,NXYZ)                                
      READ (50,*) (PXYZ11(I),I=1,NXYZ)                               

      CLOSE (50)
!
      return
 1000 write(6,'(a/,a/,2a)') ' Cannot read the nof_ec1_data file.'
      stop
      end

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
!
!  INITIALISIERUNG DES SPLINE-PROGRAMMS (EINLESEN DER DATEN):
!
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      subroutine innof_ec2

use paths

      IMPLICIT REAL*8 (A-H,O-Z)
      implicit integer (i-n)

      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      common /surfnof2/ en11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridnof2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof2/,/gridnof2/
!
      OPEN (50,FILE="nof_data/nof_ec2_data",status="old",iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a,/a)') &
          'Cannot open the surface data file for NOF S2:', &
           "nof_data/nof_ec2_data"
         stop
      endif

      READ (50,*) NX,(X(I),I=1,NX)
      READ (50,*) NY,(Y(I),I=1,NY)
      READ (50,*) NZ,(Z(I),I=1,NZ)
      NXYZ=NX*NY*NZ
      READ (50,*) (EN11(I),I=1,NXYZ)                                 
      READ (50,*) (PX11(I),I=1,NXYZ)                                 
      READ (50,*) (PY11(I),I=1,NXYZ)                                 
      READ (50,*) (PZ11(I),I=1,NXYZ)                                 
      READ (50,*) (PXY11(I),I=1,NXYZ)                                
      READ (50,*) (PXZ11(I),I=1,NXYZ)                                
      READ (50,*) (PYZ11(I),I=1,NXYZ)                                
      READ (50,*) (PXYZ11(I),I=1,NXYZ)                               

      CLOSE (50)
!
      return
 1000 write(6,'(a/,a/,2a)') ' Cannot read the nof_ec2_data file.'
      stop
      end

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
!
!  INITIALISIERUNG DES SPLINE-PROGRAMMS (EINLESEN DER DATEN):
!
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      subroutine innof_ec3

use paths

      IMPLICIT REAL*8 (A-H,O-Z)
      implicit integer (i-n)

      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      common /surfnof3/ en11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridnof3/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof3/,/gridnof3/
!
      OPEN (50,FILE="nof_data/nof_ec3_data",status="old",iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a,/a)') &
          'Cannot open the surface data file for NOF S3:', &
           "nof_data/nof_ec3_data"
         stop
      endif

      READ (50,*) NX,(X(I),I=1,NX)
      READ (50,*) NY,(Y(I),I=1,NY)
      READ (50,*) NZ,(Z(I),I=1,NZ)
      NXYZ=NX*NY*NZ
      READ (50,*) (EN11(I),I=1,NXYZ)                                 
      READ (50,*) (PX11(I),I=1,NXYZ)                                 
      READ (50,*) (PY11(I),I=1,NXYZ)                                 
      READ (50,*) (PZ11(I),I=1,NXYZ)                                 
      READ (50,*) (PXY11(I),I=1,NXYZ)                                
      READ (50,*) (PXZ11(I),I=1,NXYZ)                                
      READ (50,*) (PYZ11(I),I=1,NXYZ)                                
      READ (50,*) (PXYZ11(I),I=1,NXYZ)                               

      CLOSE (50)
!
      return
 1000 write(6,'(a/,a/,2a)') ' Cannot read the nof_ec3_data file.'
      stop
      end

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
                                                                                
      SUBROUTINE NATURL_NOF_EC0(R,INDX,FIN)                                  
!                                                                    
!     INDX = 0  :  INTERPOLATED POTENTIAL ONLY                       
!            3  :        "      X,Y,Z-DERIVATIVES                    
!                                                                    
      IMPLICIT REAL*8(A-H,O-Z)                             
      parameter (ndx=160, ndy=60,ndz=110)
      DIMENSION FIN(4),FIJK(64),HI(3),XR(3),U(3,4),R(3)              
      common/gridnof0/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof0/

      DATA UN/1.D0/,TWO/2.D0/,THREE/3.D0/,SIX/6.D0/,IV,JV,KV/3*0/   
!                                                                   
      LS=1                                                                      
      IF(R(1).LT.X(1).OR.R(1).GT.X(NX)) GOTO 110                   
      LS=2                                                        
      IF(R(2).LT.Y(1).OR.R(2).GT.Y(NY)) GOTO 110                  
      LS=3                                                        
      IF(R(3).LT.Z(1).OR.R(3).GT.Z(NZ)) GOTO 110                  
      DO 10 IS=2,NX                                               
      IF(R(1).LT.X(IS)) GOTO 20                                   
   10 CONTINUE                                                    
   20 DO 30 JS=2,NY                                               
      IF(R(2).LT.Y(JS)) GOTO 40                                   
   30 CONTINUE                                                    
   40 DO 50 KS=2,NZ                                               
      IF(R(3).LT.Z(KS)) GOTO 60                                   
   50 CONTINUE                                                    
   60 IF(IS.NE.IV) GOTO 65                                        
      IF(JS.NE.JV) GOTO 65                                        
      IF(KS.EQ.KV) GOTO 75                                        
   65 CALL FORMV0(IS,JS,KS,FIJK)                                   
      HI(1)=X(IS)-X(IS-1)                                         
      IV=IS                                                       
      HI(2)=Y(JS)-Y(JS-1)                                         
      JV=JS                                                       
      HI(3)=Z(KS)-Z(KS-1)                                         
      KV=KS                                                      
   75 XR(1)=(R(1)-X(IS-1))/HI(1)                                
      XR(2)=(R(2)-Y(JS-1))/HI(2)                                
      XR(3)=(R(3)-Z(KS-1))/HI(3)                                
      IN=INDX                                                   
   90 DO 70 I=1,3                                               
      IF(I.EQ.IN) GOTO 70                                       
      XR2=XR(I)**2                                              
      XR3=XR(I)*XR2                                             
      U(I,1)=-TWO*XR3+THREE*XR2                                 
      U(I,2)=-U(I,1)+UN                                         
      U(I,3)=HI(I)*(XR3-XR2)                                   
      U(I,4)=HI(I)*(XR3-TWO*XR2+XR(I))                         
   70 CONTINUE                                                 
      IF(IN.EQ.0) GOTO 80                                      
      U(IN,1)=SIX*(-XR(IN)+1)*XR(IN)/HI(IN)                    
      U(IN,2)=-U(IN,1)                                         
      U(IN,3)=(THREE*XR(IN)-TWO)*XR(IN)                        
      U(IN,4)=U(IN,3)-TWO*XR(IN)+UN                            
   80 FIN(IN+1)=0.D0                                           
      IJK=1                                                    
      DO 100 K=1,4                                             
      DO 100 J=1,4                                             
      UKJ=U(3,K)*U(2,J)                                        
      DO 100 I=1,4                                            
      FIN(IN+1)=FIN(IN+1)+FIJK(IJK)*UKJ*U(1,I)                
  100 IJK=IJK+1                                               
      IN=IN-1                                                 
      IF(IN.GT.0.AND.INDX.GT.0) GOTO 90                       
      RETURN                                                  
  110 PRINT 120,LS                                            
  120 FORMAT( 1X,15('*'),' R(',I1,') SORT DE LA GRILLE')      
      print*,r(ls)                                                              
      RETURN 
      END                                                       

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
                                                                                
      SUBROUTINE NATURL_NOF_EC1(R,INDX,FIN)                                  
!                                                                    
!     INDX = 0  :  INTERPOLATED POTENTIAL ONLY                       
!            3  :        "      X,Y,Z-DERIVATIVES                    
!                                                                    
      IMPLICIT REAL*8(A-H,O-Z)                             
      parameter (ndx=160, ndy=60,ndz=110)
      DIMENSION FIN(4),FIJK(64),HI(3),XR(3),U(3,4),R(3)              
      common/gridnof1/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof1/

      DATA UN/1.D0/,TWO/2.D0/,THREE/3.D0/,SIX/6.D0/,IV,JV,KV/3*0/   
!                                                                   
      LS=1                                                                      
      IF(R(1).LT.X(1).OR.R(1).GT.X(NX)) GOTO 110                   
      LS=2                                                        
      IF(R(2).LT.Y(1).OR.R(2).GT.Y(NY)) GOTO 110                  
      LS=3                                                        
      IF(R(3).LT.Z(1).OR.R(3).GT.Z(NZ)) GOTO 110                  
      DO 10 IS=2,NX                                               
      IF(R(1).LT.X(IS)) GOTO 20                                   
   10 CONTINUE                                                    
   20 DO 30 JS=2,NY                                               
      IF(R(2).LT.Y(JS)) GOTO 40                                   
   30 CONTINUE                                                    
   40 DO 50 KS=2,NZ                                               
      IF(R(3).LT.Z(KS)) GOTO 60                                   
   50 CONTINUE                                                    
   60 IF(IS.NE.IV) GOTO 65                                        
      IF(JS.NE.JV) GOTO 65                                        
      IF(KS.EQ.KV) GOTO 75                                        
   65 CALL FORMV1(IS,JS,KS,FIJK)                                   
      HI(1)=X(IS)-X(IS-1)                                         
      IV=IS                                                       
      HI(2)=Y(JS)-Y(JS-1)                                         
      JV=JS                                                       
      HI(3)=Z(KS)-Z(KS-1)                                         
      KV=KS                                                      
   75 XR(1)=(R(1)-X(IS-1))/HI(1)                                
      XR(2)=(R(2)-Y(JS-1))/HI(2)                                
      XR(3)=(R(3)-Z(KS-1))/HI(3)                                
      IN=INDX                                                   
   90 DO 70 I=1,3                                               
      IF(I.EQ.IN) GOTO 70                                       
      XR2=XR(I)**2                                              
      XR3=XR(I)*XR2                                             
      U(I,1)=-TWO*XR3+THREE*XR2                                 
      U(I,2)=-U(I,1)+UN                                         
      U(I,3)=HI(I)*(XR3-XR2)                                   
      U(I,4)=HI(I)*(XR3-TWO*XR2+XR(I))                         
   70 CONTINUE                                                 
      IF(IN.EQ.0) GOTO 80                                      
      U(IN,1)=SIX*(-XR(IN)+1)*XR(IN)/HI(IN)                    
      U(IN,2)=-U(IN,1)                                         
      U(IN,3)=(THREE*XR(IN)-TWO)*XR(IN)                        
      U(IN,4)=U(IN,3)-TWO*XR(IN)+UN                            
   80 FIN(IN+1)=0.D0                                           
      IJK=1                                                    
      DO 100 K=1,4                                             
      DO 100 J=1,4                                             
      UKJ=U(3,K)*U(2,J)                                        
      DO 100 I=1,4                                            
      FIN(IN+1)=FIN(IN+1)+FIJK(IJK)*UKJ*U(1,I)                
  100 IJK=IJK+1                                               
      IN=IN-1                                                 
      IF(IN.GT.0.AND.INDX.GT.0) GOTO 90                       
      RETURN                                                  
  110 PRINT 120,LS                                            
  120 FORMAT( 1X,15('*'),' R(',I1,') SORT DE LA GRILLE')      
      print*,r(ls)                                                              
      RETURN 
      END                                                       

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
                                                                                
      SUBROUTINE NATURL_NOF_EC2(R,INDX,FIN)                                  
!                                                                    
!     INDX = 0  :  INTERPOLATED POTENTIAL ONLY                       
!            3  :        "      X,Y,Z-DERIVATIVES                    
!                                                                    
      IMPLICIT REAL*8(A-H,O-Z)                             
      parameter (ndx=160, ndy=60,ndz=110)
      DIMENSION FIN(4),FIJK(64),HI(3),XR(3),U(3,4),R(3)              
      common/gridnof2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof2/

      DATA UN/1.D0/,TWO/2.D0/,THREE/3.D0/,SIX/6.D0/,IV,JV,KV/3*0/   
!                                                                   
      LS=1                                                                      
      IF(R(1).LT.X(1).OR.R(1).GT.X(NX)) GOTO 110                   
      LS=2                                                        
      IF(R(2).LT.Y(1).OR.R(2).GT.Y(NY)) GOTO 110                  
      LS=3                                                        
      IF(R(3).LT.Z(1).OR.R(3).GT.Z(NZ)) GOTO 110                  
      DO 10 IS=2,NX                                               
      IF(R(1).LT.X(IS)) GOTO 20                                   
   10 CONTINUE                                                    
   20 DO 30 JS=2,NY                                               
      IF(R(2).LT.Y(JS)) GOTO 40                                   
   30 CONTINUE                                                    
   40 DO 50 KS=2,NZ                                               
      IF(R(3).LT.Z(KS)) GOTO 60                                   
   50 CONTINUE                                                    
   60 IF(IS.NE.IV) GOTO 65                                        
      IF(JS.NE.JV) GOTO 65                                        
      IF(KS.EQ.KV) GOTO 75                                        
   65 CALL FORMV2(IS,JS,KS,FIJK)                                   
      HI(1)=X(IS)-X(IS-1)                                         
      IV=IS                                                       
      HI(2)=Y(JS)-Y(JS-1)                                         
      JV=JS                                                       
      HI(3)=Z(KS)-Z(KS-1)                                         
      KV=KS                                                      
   75 XR(1)=(R(1)-X(IS-1))/HI(1)                                
      XR(2)=(R(2)-Y(JS-1))/HI(2)                                
      XR(3)=(R(3)-Z(KS-1))/HI(3)                                
      IN=INDX                                                   
   90 DO 70 I=1,3                                               
      IF(I.EQ.IN) GOTO 70                                       
      XR2=XR(I)**2                                              
      XR3=XR(I)*XR2                                             
      U(I,1)=-TWO*XR3+THREE*XR2                                 
      U(I,2)=-U(I,1)+UN                                         
      U(I,3)=HI(I)*(XR3-XR2)                                   
      U(I,4)=HI(I)*(XR3-TWO*XR2+XR(I))                         
   70 CONTINUE                                                 
      IF(IN.EQ.0) GOTO 80                                      
      U(IN,1)=SIX*(-XR(IN)+1)*XR(IN)/HI(IN)                    
      U(IN,2)=-U(IN,1)                                         
      U(IN,3)=(THREE*XR(IN)-TWO)*XR(IN)                        
      U(IN,4)=U(IN,3)-TWO*XR(IN)+UN                            
   80 FIN(IN+1)=0.D0                                           
      IJK=1                                                    
      DO 100 K=1,4                                             
      DO 100 J=1,4                                             
      UKJ=U(3,K)*U(2,J)                                        
      DO 100 I=1,4                                            
      FIN(IN+1)=FIN(IN+1)+FIJK(IJK)*UKJ*U(1,I)                
  100 IJK=IJK+1                                               
      IN=IN-1                                                 
      IF(IN.GT.0.AND.INDX.GT.0) GOTO 90                       
      RETURN                                                  
  110 PRINT 120,LS                                            
  120 FORMAT( 1X,15('*'),' R(',I1,') SORT DE LA GRILLE')      
      print*,r(ls)                                                              
      RETURN 
      END                                                       

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
                                                                                
      SUBROUTINE NATURL_NOF_EC3(R,INDX,FIN)                                  
!                                                                    
!     INDX = 0  :  INTERPOLATED POTENTIAL ONLY                       
!            3  :        "      X,Y,Z-DERIVATIVES                    
!                                                                    
      IMPLICIT REAL*8(A-H,O-Z)                             
      parameter (ndx=160, ndy=60,ndz=110)
      DIMENSION FIN(4),FIJK(64),HI(3),XR(3),U(3,4),R(3)              
      common/gridnof3/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof3/

      DATA UN/1.D0/,TWO/2.D0/,THREE/3.D0/,SIX/6.D0/,IV,JV,KV/3*0/   
!                                                                   
      LS=1                                                                      
      IF(R(1).LT.X(1).OR.R(1).GT.X(NX)) GOTO 110                   
      LS=2                                                        
      IF(R(2).LT.Y(1).OR.R(2).GT.Y(NY)) GOTO 110                  
      LS=3                                                        
      IF(R(3).LT.Z(1).OR.R(3).GT.Z(NZ)) GOTO 110                  
      DO 10 IS=2,NX                                               
      IF(R(1).LT.X(IS)) GOTO 20                                   
   10 CONTINUE                                                    
   20 DO 30 JS=2,NY                                               
      IF(R(2).LT.Y(JS)) GOTO 40                                   
   30 CONTINUE                                                    
   40 DO 50 KS=2,NZ                                               
      IF(R(3).LT.Z(KS)) GOTO 60                                   
   50 CONTINUE                                                    
   60 IF(IS.NE.IV) GOTO 65                                        
      IF(JS.NE.JV) GOTO 65                                        
      IF(KS.EQ.KV) GOTO 75                                        
   65 CALL FORMV3(IS,JS,KS,FIJK)                                   
      HI(1)=X(IS)-X(IS-1)                                         
      IV=IS                                                       
      HI(2)=Y(JS)-Y(JS-1)                                         
      JV=JS                                                       
      HI(3)=Z(KS)-Z(KS-1)                                         
      KV=KS                                                      
   75 XR(1)=(R(1)-X(IS-1))/HI(1)                                
      XR(2)=(R(2)-Y(JS-1))/HI(2)                                
      XR(3)=(R(3)-Z(KS-1))/HI(3)                                
      IN=INDX                                                   
   90 DO 70 I=1,3                                               
      IF(I.EQ.IN) GOTO 70                                       
      XR2=XR(I)**2                                              
      XR3=XR(I)*XR2                                             
      U(I,1)=-TWO*XR3+THREE*XR2                                 
      U(I,2)=-U(I,1)+UN                                         
      U(I,3)=HI(I)*(XR3-XR2)                                   
      U(I,4)=HI(I)*(XR3-TWO*XR2+XR(I))                         
   70 CONTINUE                                                 
      IF(IN.EQ.0) GOTO 80                                      
      U(IN,1)=SIX*(-XR(IN)+1)*XR(IN)/HI(IN)                    
      U(IN,2)=-U(IN,1)                                         
      U(IN,3)=(THREE*XR(IN)-TWO)*XR(IN)                        
      U(IN,4)=U(IN,3)-TWO*XR(IN)+UN                            
   80 FIN(IN+1)=0.D0                                           
      IJK=1                                                    
      DO 100 K=1,4                                             
      DO 100 J=1,4                                             
      UKJ=U(3,K)*U(2,J)                                        
      DO 100 I=1,4                                            
      FIN(IN+1)=FIN(IN+1)+FIJK(IJK)*UKJ*U(1,I)                
  100 IJK=IJK+1                                               
      IN=IN-1                                                 
      IF(IN.GT.0.AND.INDX.GT.0) GOTO 90                       
      RETURN                                                  
  110 PRINT 120,LS                                            
  120 FORMAT( 1X,15('*'),' R(',I1,') SORT DE LA GRILLE')      
      print*,r(ls)                                                              
      RETURN 
      END                                                       

!#######################################################################

      SUBROUTINE FORMV0(I,J,K,G)                                   
      IMPLICIT REAL*8(A-H,O-Z)                         
      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      DIMENSION G(64)                                           

      common /surfnof0/ F11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridnof0/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof0/,/gridnof0/
!      
      IJK=(K-1)*NX*NY+(J-1)*NX+I                               
      IMJK=IJK-1                                               
      IJMK=IJK-NX                                              
      IMJMK=IJMK-1                                             
      IJKM=IJK-NX*NY                                          
      IMJKM=IJKM-1                                            
      IJMKM=IJKM-NX                                           
      IMJMKM=IJMKM-1                                          
!                                                             
      G( 1)=F11(IJK)                                            
      G( 2)=F11(IMJK)                                           
      G( 3)=PX11(IJK)                                          
      G( 4)=PX11(IMJK)                                         
      G( 5)=F11(IJMK)                                          
      G( 6)=F11(IMJMK)                                         
      G( 7)=PX11(IJMK)                                         
      G( 8)=PX11(IMJMK)                                        
      G( 9)=PY11(IJK)                                          
      G(10)=PY11(IMJK)                                        
      G(11)=PXY11(IJK)                                        
      G(12)=PXY11(IMJK)                                       
      G(13)=PY11(IJMK)                                        
      G(14)=PY11(IMJMK)                                       
      G(15)=PXY11(IJMK)                                       
      G(16)=PXY11(IMJMK)                                      
      G(17)=F11(IJKM)                                         
      G(18)=F11(IMJKM)                                        
      G(19)=PX11(IJKM)                                        
      G(20)=PX11(IMJKM)                                       
      G(21)=F11(IJMKM)                                        
      G(22)=F11(IMJMKM)                                       
      G(23)=PX11(IJMKM)                                       
      G(24)=PX11(IMJMKM)                                      
      G(25)=PY11(IJKM)                                        
      G(26)=PY11(IMJKM)                                       
      G(27)=PXY11(IJKM)                                       
      G(28)=PXY11(IMJKM)                                      
      G(29)=PY11(IJMKM)                                       
      G(30)=PY11(IMJMKM)                                      
      G(31)=PXY11(IJMKM)                                      
      G(32)=PXY11(IMJMKM)                                     
      G(33)=PZ11(IJK)                                         
      G(34)=PZ11(IMJK)                                        
      G(35)=PXZ11(IJK)                                        
      G(36)=PXZ11(IMJK)                                       
      G(37)=PZ11(IJMK)                                        
      G(38)=PZ11(IMJMK)                                       
      G(39)=PXZ11(IJMK)                                       
      G(40)=PXZ11(IMJMK)                                      
      G(41)=PYZ11(IJK)                                        
      G(42)=PYZ11(IMJK)                                       
      G(43)=PXYZ11(IJK)                                       
      G(44)=PXYZ11(IMJK)                                      
      G(45)=PYZ11(IJMK)                                       
      G(46)=PYZ11(IMJMK)                                      
      G(47)=PXYZ11(IJMK)                                      
      G(48)=PXYZ11(IMJMK)                                     
      G(49)=PZ11(IJKM)                                        
      G(50)=PZ11(IMJKM)                                       
      G(51)=PXZ11(IJKM)                                       
      G(52)=PXZ11(IMJKM)                                      
      G(53)=PZ11(IJMKM)                                      
      G(54)=PZ11(IMJMKM)                                     
      G(55)=PXZ11(IJMKM)                                     
      G(56)=PXZ11(IMJMKM)                                    
      G(57)=PYZ11(IJKM)                                      
      G(58)=PYZ11(IMJKM)                                    
      G(59)=PXYZ11(IJKM)                                    
      G(60)=PXYZ11(IMJKM)                                   
      G(61)=PYZ11(IJMKM)                                    
      G(62)=PYZ11(IMJMKM)                                   
      G(63)=PXYZ11(IJMKM)                                   
      G(64)=PXYZ11(IMJMKM)                                  
      RETURN                                              
      END                                                 
! END OF 3DSPLINE-PROGRAM!                                
!#######################################################################

      SUBROUTINE FORMV1(I,J,K,G)                                   
      IMPLICIT REAL*8(A-H,O-Z)                         
      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      DIMENSION G(64)                                           

      common /surfnof1/ F22(ndat),px22(ndat),py22(ndat),pz22(ndat), &
                      pxy22(ndat),pxz22(ndat),pyz22(ndat),pxyz22(ndat)
      common/gridnof1/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof1/,/gridnof1/
!      
      IJK=(K-1)*NX*NY+(J-1)*NX+I                               
      IMJK=IJK-1                                               
      IJMK=IJK-NX                                              
      IMJMK=IJMK-1                                             
      IJKM=IJK-NX*NY                                          
      IMJKM=IJKM-1                                            
      IJMKM=IJKM-NX                                           
      IMJMKM=IJMKM-1                                          
!                                                             
      G( 1)=F22(IJK)                                            
      G( 2)=F22(IMJK)                                           
      G( 3)=PX22(IJK)                                          
      G( 4)=PX22(IMJK)                                         
      G( 5)=F22(IJMK)                                          
      G( 6)=F22(IMJMK)                                         
      G( 7)=PX22(IJMK)                                         
      G( 8)=PX22(IMJMK)                                        
      G( 9)=PY22(IJK)                                          
      G(10)=PY22(IMJK)                                        
      G(11)=PXY22(IJK)                                        
      G(12)=PXY22(IMJK)                                       
      G(13)=PY22(IJMK)                                        
      G(14)=PY22(IMJMK)                                       
      G(15)=PXY22(IJMK)                                       
      G(16)=PXY22(IMJMK)                                      
      G(17)=F22(IJKM)                                         
      G(18)=F22(IMJKM)                                        
      G(19)=PX22(IJKM)                                        
      G(20)=PX22(IMJKM)                                       
      G(21)=F22(IJMKM)                                        
      G(22)=F22(IMJMKM)                                       
      G(23)=PX22(IJMKM)                                       
      G(24)=PX22(IMJMKM)                                      
      G(25)=PY22(IJKM)                                        
      G(26)=PY22(IMJKM)                                       
      G(27)=PXY22(IJKM)                                       
      G(28)=PXY22(IMJKM)                                      
      G(29)=PY22(IJMKM)                                       
      G(30)=PY22(IMJMKM)                                      
      G(31)=PXY22(IJMKM)                                      
      G(32)=PXY22(IMJMKM)                                     
      G(33)=PZ22(IJK)                                         
      G(34)=PZ22(IMJK)                                        
      G(35)=PXZ22(IJK)                                        
      G(36)=PXZ22(IMJK)                                       
      G(37)=PZ22(IJMK)                                        
      G(38)=PZ22(IMJMK)                                       
      G(39)=PXZ22(IJMK)                                       
      G(40)=PXZ22(IMJMK)                                      
      G(41)=PYZ22(IJK)                                        
      G(42)=PYZ22(IMJK)                                       
      G(43)=PXYZ22(IJK)                                       
      G(44)=PXYZ22(IMJK)                                      
      G(45)=PYZ22(IJMK)                                       
      G(46)=PYZ22(IMJMK)                                      
      G(47)=PXYZ22(IJMK)                                      
      G(48)=PXYZ22(IMJMK)                                     
      G(49)=PZ22(IJKM)                                        
      G(50)=PZ22(IMJKM)                                       
      G(51)=PXZ22(IJKM)                                       
      G(52)=PXZ22(IMJKM)                                      
      G(53)=PZ22(IJMKM)                                      
      G(54)=PZ22(IMJMKM)                                     
      G(55)=PXZ22(IJMKM)                                     
      G(56)=PXZ22(IMJMKM)                                    
      G(57)=PYZ22(IJKM)                                      
      G(58)=PYZ22(IMJKM)                                    
      G(59)=PXYZ22(IJKM)                                    
      G(60)=PXYZ22(IMJKM)                                   
      G(61)=PYZ22(IJMKM)                                    
      G(62)=PYZ22(IMJMKM)                                   
      G(63)=PXYZ22(IJMKM)                                   
      G(64)=PXYZ22(IMJMKM)                                  
      RETURN                                              
      END                                                 
! END OF 3DSPLINE-PROGRAM!                                
!#######################################################################

      SUBROUTINE FORMV2(I,J,K,G)                                   
      IMPLICIT REAL*8(A-H,O-Z)                         
      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      DIMENSION G(64)                                           

      common /surfnof2/ F22(ndat),px22(ndat),py22(ndat),pz22(ndat), &
                      pxy22(ndat),pxz22(ndat),pyz22(ndat),pxyz22(ndat)
      common/gridnof2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof2/,/gridnof2/
!      
      IJK=(K-1)*NX*NY+(J-1)*NX+I                               
      IMJK=IJK-1                                               
      IJMK=IJK-NX                                              
      IMJMK=IJMK-1                                             
      IJKM=IJK-NX*NY                                          
      IMJKM=IJKM-1                                            
      IJMKM=IJKM-NX                                           
      IMJMKM=IJMKM-1                                          
!                                                             
      G( 1)=F22(IJK)                                            
      G( 2)=F22(IMJK)                                           
      G( 3)=PX22(IJK)                                          
      G( 4)=PX22(IMJK)                                         
      G( 5)=F22(IJMK)                                          
      G( 6)=F22(IMJMK)                                         
      G( 7)=PX22(IJMK)                                         
      G( 8)=PX22(IMJMK)                                        
      G( 9)=PY22(IJK)                                          
      G(10)=PY22(IMJK)                                        
      G(11)=PXY22(IJK)                                        
      G(12)=PXY22(IMJK)                                       
      G(13)=PY22(IJMK)                                        
      G(14)=PY22(IMJMK)                                       
      G(15)=PXY22(IJMK)                                       
      G(16)=PXY22(IMJMK)                                      
      G(17)=F22(IJKM)                                         
      G(18)=F22(IMJKM)                                        
      G(19)=PX22(IJKM)                                        
      G(20)=PX22(IMJKM)                                       
      G(21)=F22(IJMKM)                                        
      G(22)=F22(IMJMKM)                                       
      G(23)=PX22(IJMKM)                                       
      G(24)=PX22(IMJMKM)                                      
      G(25)=PY22(IJKM)                                        
      G(26)=PY22(IMJKM)                                       
      G(27)=PXY22(IJKM)                                       
      G(28)=PXY22(IMJKM)                                      
      G(29)=PY22(IJMKM)                                       
      G(30)=PY22(IMJMKM)                                      
      G(31)=PXY22(IJMKM)                                      
      G(32)=PXY22(IMJMKM)                                     
      G(33)=PZ22(IJK)                                         
      G(34)=PZ22(IMJK)                                        
      G(35)=PXZ22(IJK)                                        
      G(36)=PXZ22(IMJK)                                       
      G(37)=PZ22(IJMK)                                        
      G(38)=PZ22(IMJMK)                                       
      G(39)=PXZ22(IJMK)                                       
      G(40)=PXZ22(IMJMK)                                      
      G(41)=PYZ22(IJK)                                        
      G(42)=PYZ22(IMJK)                                       
      G(43)=PXYZ22(IJK)                                       
      G(44)=PXYZ22(IMJK)                                      
      G(45)=PYZ22(IJMK)                                       
      G(46)=PYZ22(IMJMK)                                      
      G(47)=PXYZ22(IJMK)                                      
      G(48)=PXYZ22(IMJMK)                                     
      G(49)=PZ22(IJKM)                                        
      G(50)=PZ22(IMJKM)                                       
      G(51)=PXZ22(IJKM)                                       
      G(52)=PXZ22(IMJKM)                                      
      G(53)=PZ22(IJMKM)                                      
      G(54)=PZ22(IMJMKM)                                     
      G(55)=PXZ22(IJMKM)                                     
      G(56)=PXZ22(IMJMKM)                                    
      G(57)=PYZ22(IJKM)                                      
      G(58)=PYZ22(IMJKM)                                    
      G(59)=PXYZ22(IJKM)                                    
      G(60)=PXYZ22(IMJKM)                                   
      G(61)=PYZ22(IJMKM)                                    
      G(62)=PYZ22(IMJMKM)                                   
      G(63)=PXYZ22(IJMKM)                                   
      G(64)=PXYZ22(IMJMKM)                                  
      RETURN                                              
      END                                                 
! END OF 3DSPLINE-PROGRAM!                                
!#######################################################################

!#######################################################################

      SUBROUTINE FORMV3(I,J,K,G)                                   
      IMPLICIT REAL*8(A-H,O-Z)                         
      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      DIMENSION G(64)                                           

      common /surfnof3/ F22(ndat),px22(ndat),py22(ndat),pz22(ndat), &
                      pxy22(ndat),pxz22(ndat),pyz22(ndat),pxyz22(ndat)
      common/gridnof3/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof3/,/gridnof3/
!      
      IJK=(K-1)*NX*NY+(J-1)*NX+I                               
      IMJK=IJK-1                                               
      IJMK=IJK-NX                                              
      IMJMK=IJMK-1                                             
      IJKM=IJK-NX*NY                                          
      IMJKM=IJKM-1                                            
      IJMKM=IJKM-NX                                           
      IMJMKM=IJMKM-1                                          
!                                                             
      G( 1)=F22(IJK)                                            
      G( 2)=F22(IMJK)                                           
      G( 3)=PX22(IJK)                                          
      G( 4)=PX22(IMJK)                                         
      G( 5)=F22(IJMK)                                          
      G( 6)=F22(IMJMK)                                         
      G( 7)=PX22(IJMK)                                         
      G( 8)=PX22(IMJMK)                                        
      G( 9)=PY22(IJK)                                          
      G(10)=PY22(IMJK)                                        
      G(11)=PXY22(IJK)                                        
      G(12)=PXY22(IMJK)                                       
      G(13)=PY22(IJMK)                                        
      G(14)=PY22(IMJMK)                                       
      G(15)=PXY22(IJMK)                                       
      G(16)=PXY22(IMJMK)                                      
      G(17)=F22(IJKM)                                         
      G(18)=F22(IMJKM)                                        
      G(19)=PX22(IJKM)                                        
      G(20)=PX22(IMJKM)                                       
      G(21)=F22(IJMKM)                                        
      G(22)=F22(IMJMKM)                                       
      G(23)=PX22(IJMKM)                                       
      G(24)=PX22(IMJMKM)                                      
      G(25)=PY22(IJKM)                                        
      G(26)=PY22(IMJKM)                                       
      G(27)=PXY22(IJKM)                                       
      G(28)=PXY22(IMJKM)                                      
      G(29)=PY22(IJMKM)                                       
      G(30)=PY22(IMJMKM)                                      
      G(31)=PXY22(IJMKM)                                      
      G(32)=PXY22(IMJMKM)                                     
      G(33)=PZ22(IJK)                                         
      G(34)=PZ22(IMJK)                                        
      G(35)=PXZ22(IJK)                                        
      G(36)=PXZ22(IMJK)                                       
      G(37)=PZ22(IJMK)                                        
      G(38)=PZ22(IMJMK)                                       
      G(39)=PXZ22(IJMK)                                       
      G(40)=PXZ22(IMJMK)                                      
      G(41)=PYZ22(IJK)                                        
      G(42)=PYZ22(IMJK)                                       
      G(43)=PXYZ22(IJK)                                       
      G(44)=PXYZ22(IMJK)                                      
      G(45)=PYZ22(IJMK)                                       
      G(46)=PYZ22(IMJMK)                                      
      G(47)=PXYZ22(IJMK)                                      
      G(48)=PXYZ22(IMJMK)                                     
      G(49)=PZ22(IJKM)                                        
      G(50)=PZ22(IMJKM)                                       
      G(51)=PXZ22(IJKM)                                       
      G(52)=PXZ22(IMJKM)                                      
      G(53)=PZ22(IJMKM)                                      
      G(54)=PZ22(IMJMKM)                                     
      G(55)=PXZ22(IJMKM)                                     
      G(56)=PXZ22(IMJMKM)                                    
      G(57)=PYZ22(IJKM)                                      
      G(58)=PYZ22(IMJKM)                                    
      G(59)=PXYZ22(IJKM)                                    
      G(60)=PXYZ22(IMJKM)                                   
      G(61)=PYZ22(IJMKM)                                    
      G(62)=PYZ22(IMJMKM)                                   
      G(63)=PXYZ22(IJMKM)                                   
      G(64)=PXYZ22(IMJMKM)                                  
      RETURN                                              
      END                                                 

!#######################################################################

      subroutine nof_pt2_ec0(y1,y2,y3,v)

      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      DIMENSION rr(3),fin(4)

      parameter (ndx=160,ndy=60,ndz=110)
      common/gridnof0_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof0_pt2/

!     if (y1.le.y2) then
!        rr(1)=y2
!        rr(2)=y1
!     else
!        rr(1)=y1
!        rr(2)=y2
!     endif
      rr(1)=y1
      rr(2)=y2
      rr(3)=y3

!-----------------------------------------------------------------------
! Surface only defined in following region
!-----------------------------------------------------------------------
      rr(1)=max(rr(1),x(1))
      rr(1)=min(rr(1),x(nx))
      rr(2)=max(rr(2),y(1))
      rr(2)=min(rr(2),y(ny))
      rr(3)=max(rr(3),z(1))
      rr(3)=min(rr(3),z(nz))

      call naturl_nof_pt2_ec0(rr,0,fin)
      v=fin(1)/27.2114d0
      return

      END

!#######################################################################

      subroutine nof_pt2_ec1(y1,y2,y3,v)

      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      DIMENSION rr(3),fin(4)

      parameter (ndx=160,ndy=60,ndz=110)
      common/gridnof1_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof1_pt2/

!     if (y1.le.y2) then
!        rr(1)=y2
!        rr(2)=y1
!     else
!        rr(1)=y1
!        rr(2)=y2
!     endif
      rr(3)=y3
      rr(1)=y1
      rr(2)=y2
      rr(3)=y3

!-----------------------------------------------------------------------
! Surface only defined in following region
!-----------------------------------------------------------------------
      rr(1)=max(rr(1),x(1))
      rr(1)=min(rr(1),x(nx))
      rr(2)=max(rr(2),y(1))
      rr(2)=min(rr(2),y(ny))
      rr(3)=max(rr(3),z(1))
      rr(3)=min(rr(3),z(nz))

      call naturl_nof_pt2_ec1(rr,0,fin)
      v=fin(1)/27.2114d0
      return

      END

!#######################################################################

      subroutine nof_pt2_ec2(y1,y2,y3,v)

      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      DIMENSION rr(3),fin(4)

      parameter (ndx=160,ndy=60,ndz=110)
      common/gridnof2_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof2_pt2/

!     if (y1.le.y2) then
!        rr(1)=y2
!        rr(2)=y1
!     else
!        rr(1)=y1
!        rr(2)=y2
!     endif
      rr(3)=y3
      rr(1)=y1
      rr(2)=y2
      rr(3)=y3

!-----------------------------------------------------------------------
! Surface only defined in following region
!-----------------------------------------------------------------------
      rr(1)=max(rr(1),x(1))
      rr(1)=min(rr(1),x(nx))
      rr(2)=max(rr(2),y(1))
      rr(2)=min(rr(2),y(ny))
      rr(3)=max(rr(3),z(1))
      rr(3)=min(rr(3),z(nz))

      call naturl_nof_pt2_ec2(rr,0,fin)
      v=fin(1)/27.2114d0
      return

      END

!#######################################################################

      subroutine nof_pt2_ec3(y1,y2,y3,v)

      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      DIMENSION rr(3),fin(4)

      parameter (ndx=160,ndy=60,ndz=110)
      common/gridnof3_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof3_pt2/

!     if (y1.le.y2) then
!        rr(1)=y2
!        rr(2)=y1
!     else
!        rr(1)=y1
!        rr(2)=y2
!     endif
      rr(3)=y3
      rr(1)=y1
      rr(2)=y2
      rr(3)=y3

!-----------------------------------------------------------------------
! Surface only defined in following region
!-----------------------------------------------------------------------
      rr(1)=max(rr(1),x(1))
      rr(1)=min(rr(1),x(nx))
      rr(2)=max(rr(2),y(1))
      rr(2)=min(rr(2),y(ny))
      rr(3)=max(rr(3),z(1))
      rr(3)=min(rr(3),z(nz))

      call naturl_nof_pt2_ec3(rr,0,fin)
      v=fin(1)/27.2114d0
      return

      END


!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
!
!  INITIALISIERUNG DES SPLINE-PROGRAMMS (EINLESEN DER DATEN):
!
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      subroutine innof_pt2_ec0

use paths

      IMPLICIT REAL*8 (A-H,O-Z)
      implicit integer (i-n)

      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      common /surfnof0_pt2/ en11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridnof0_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof0_pt2/,/gridnof0_pt2/
!
      OPEN (50,FILE="nof_data/nof_pt2_ec0_data",status="old",&
            iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a,/a)') &
          'Cannot open the surface data file for NOF S0:', &
           "nof_data/nof_pt2_ec0_data"
         stop
      endif

      READ (50,*) NX,(X(I),I=1,NX)
      READ (50,*) NY,(Y(I),I=1,NY)
      READ (50,*) NZ,(Z(I),I=1,NZ)
      NXYZ=NX*NY*NZ
      READ (50,*) (EN11(I),I=1,NXYZ)                                 
      READ (50,*) (PX11(I),I=1,NXYZ)                                 
      READ (50,*) (PY11(I),I=1,NXYZ)                                 
      READ (50,*) (PZ11(I),I=1,NXYZ)                                 
      READ (50,*) (PXY11(I),I=1,NXYZ)                                
      READ (50,*) (PXZ11(I),I=1,NXYZ)                                
      READ (50,*) (PYZ11(I),I=1,NXYZ)                                
      READ (50,*) (PXYZ11(I),I=1,NXYZ)                               

      CLOSE (50)
!
      return
 1000 write(6,'(a/,a/,2a)') ' Cannot read the nof_pt2_ec0_data file.'
      stop
      end

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
!
!  INITIALISIERUNG DES SPLINE-PROGRAMMS (EINLESEN DER DATEN):
!
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      subroutine innof_pt2_ec1

use paths

      IMPLICIT REAL*8 (A-H,O-Z)
      implicit integer (i-n)

      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      common /surfnof1_pt2/ en11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridnof1_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof1_pt2/,/gridnof1_pt2/
!
      OPEN (50,FILE="nof_data/nof_pt2_ec1_data",status="old", &
            iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a,/a)') &
          'Cannot open the surface data file for NOF S1:', &
           "nof_data/nof_pt2_ec1_data"
         stop
      endif

      READ (50,*) NX,(X(I),I=1,NX)
      READ (50,*) NY,(Y(I),I=1,NY)
      READ (50,*) NZ,(Z(I),I=1,NZ)
      NXYZ=NX*NY*NZ
      READ (50,*) (EN11(I),I=1,NXYZ)                                 
      READ (50,*) (PX11(I),I=1,NXYZ)                                 
      READ (50,*) (PY11(I),I=1,NXYZ)                                 
      READ (50,*) (PZ11(I),I=1,NXYZ)                                 
      READ (50,*) (PXY11(I),I=1,NXYZ)                                
      READ (50,*) (PXZ11(I),I=1,NXYZ)                                
      READ (50,*) (PYZ11(I),I=1,NXYZ)                                
      READ (50,*) (PXYZ11(I),I=1,NXYZ)                               

      CLOSE (50)
!
      return
 1000 write(6,'(a/,a/,2a)') ' Cannot read the nof_pt2_ec1_data file.'
      stop
      end

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
!
!  INITIALISIERUNG DES SPLINE-PROGRAMMS (EINLESEN DER DATEN):
!
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      subroutine innof_pt2_ec2

use paths

      IMPLICIT REAL*8 (A-H,O-Z)
      implicit integer (i-n)

      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      common /surfnof2_pt2/ en11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridnof2_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof2_pt2/,/gridnof2_pt2/
!
      OPEN (50,FILE="nof_data/nof_pt2_ec2_data",status="old",&
            iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a,/a)') &
          'Cannot open the surface data file for NOF S2:', &
           "nof_data/nof_pt2_ec2_data"
         stop
      endif

      READ (50,*) NX,(X(I),I=1,NX)
      READ (50,*) NY,(Y(I),I=1,NY)
      READ (50,*) NZ,(Z(I),I=1,NZ)
      NXYZ=NX*NY*NZ
      READ (50,*) (EN11(I),I=1,NXYZ)                                 
      READ (50,*) (PX11(I),I=1,NXYZ)                                 
      READ (50,*) (PY11(I),I=1,NXYZ)                                 
      READ (50,*) (PZ11(I),I=1,NXYZ)                                 
      READ (50,*) (PXY11(I),I=1,NXYZ)                                
      READ (50,*) (PXZ11(I),I=1,NXYZ)                                
      READ (50,*) (PYZ11(I),I=1,NXYZ)                                
      READ (50,*) (PXYZ11(I),I=1,NXYZ)                               

      CLOSE (50)
!
      return
 1000 write(6,'(a/,a/,2a)') ' Cannot read the nof_pt2_ec2_data file.'
      stop
      end

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
!
!  INITIALISIERUNG DES SPLINE-PROGRAMMS (EINLESEN DER DATEN):
!
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      subroutine innof_pt2_ec3

use paths

      IMPLICIT REAL*8 (A-H,O-Z)
      implicit integer (i-n)

      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      common /surfnof3_pt2/ en11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridnof3_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof3_pt2/,/gridnof3_pt2/
!
      OPEN (50,FILE="nof_data/nof_pt2_ec3_data",status="old", &
            iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a,/a)') &
          'Cannot open the surface data file for NOF S3:', &
           "nof_data/nof_pt2_ec3_data"
         stop
      endif

      READ (50,*) NX,(X(I),I=1,NX)
      READ (50,*) NY,(Y(I),I=1,NY)
      READ (50,*) NZ,(Z(I),I=1,NZ)
      NXYZ=NX*NY*NZ
      READ (50,*) (EN11(I),I=1,NXYZ)                                 
      READ (50,*) (PX11(I),I=1,NXYZ)                                 
      READ (50,*) (PY11(I),I=1,NXYZ)                                 
      READ (50,*) (PZ11(I),I=1,NXYZ)                                 
      READ (50,*) (PXY11(I),I=1,NXYZ)                                
      READ (50,*) (PXZ11(I),I=1,NXYZ)                                
      READ (50,*) (PYZ11(I),I=1,NXYZ)                                
      READ (50,*) (PXYZ11(I),I=1,NXYZ)                               


      CLOSE (50)
!
      return
 1000 write(6,'(a/,a/,2a)') ' Cannot read the nof_pt2_ec3_data file.'
      stop
      end

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
                                                                                
      SUBROUTINE NATURL_NOF_PT2_EC0(R,INDX,FIN)        
!                                                                    
!     INDX = 0  :  INTERPOLATED POTENTIAL ONLY                       
!            3  :        "      X,Y,Z-DERIVATIVES                    
!                                                                    
      IMPLICIT REAL*8(A-H,O-Z)                             
      parameter (ndx=160, ndy=60,ndz=110)
      DIMENSION FIN(4),FIJK(64),HI(3),XR(3),U(3,4),R(3)              
      common/gridnof0_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof0_pt2/

      DATA UN/1.D0/,TWO/2.D0/,THREE/3.D0/,SIX/6.D0/,IV,JV,KV/3*0/   
!                                                                   
      LS=1                                                                      
      IF(R(1).LT.X(1).OR.R(1).GT.X(NX)) GOTO 110                   
      LS=2                                                        
      IF(R(2).LT.Y(1).OR.R(2).GT.Y(NY)) GOTO 110                  
      LS=3                                                        
      IF(R(3).LT.Z(1).OR.R(3).GT.Z(NZ)) GOTO 110                  
      DO 10 IS=2,NX                                               
      IF(R(1).LT.X(IS)) GOTO 20                                   
   10 CONTINUE                                                    
   20 DO 30 JS=2,NY                                               
      IF(R(2).LT.Y(JS)) GOTO 40                                   
   30 CONTINUE                                                    
   40 DO 50 KS=2,NZ                                               
      IF(R(3).LT.Z(KS)) GOTO 60                                   
   50 CONTINUE                                                    
   60 IF(IS.NE.IV) GOTO 65                                        
      IF(JS.NE.JV) GOTO 65                                        
      IF(KS.EQ.KV) GOTO 75                                        
   65 CALL FORMV0_PT2(IS,JS,KS,FIJK)                                   
      HI(1)=X(IS)-X(IS-1)                                         
      IV=IS                                                       
      HI(2)=Y(JS)-Y(JS-1)                                         
      JV=JS                                                       
      HI(3)=Z(KS)-Z(KS-1)                                         
      KV=KS                                                      
   75 XR(1)=(R(1)-X(IS-1))/HI(1)                                
      XR(2)=(R(2)-Y(JS-1))/HI(2)                                
      XR(3)=(R(3)-Z(KS-1))/HI(3)                                
      IN=INDX                                                   
   90 DO 70 I=1,3                                               
      IF(I.EQ.IN) GOTO 70                                       
      XR2=XR(I)**2                                              
      XR3=XR(I)*XR2                                             
      U(I,1)=-TWO*XR3+THREE*XR2                                 
      U(I,2)=-U(I,1)+UN                                         
      U(I,3)=HI(I)*(XR3-XR2)                                   
      U(I,4)=HI(I)*(XR3-TWO*XR2+XR(I))                         
   70 CONTINUE                                                 
      IF(IN.EQ.0) GOTO 80                                      
      U(IN,1)=SIX*(-XR(IN)+1)*XR(IN)/HI(IN)                    
      U(IN,2)=-U(IN,1)                                         
      U(IN,3)=(THREE*XR(IN)-TWO)*XR(IN)                        
      U(IN,4)=U(IN,3)-TWO*XR(IN)+UN                            
   80 FIN(IN+1)=0.D0                                           
      IJK=1                                                    
      DO 100 K=1,4                                             
      DO 100 J=1,4                                             
      UKJ=U(3,K)*U(2,J)                                        
      DO 100 I=1,4                                            
      FIN(IN+1)=FIN(IN+1)+FIJK(IJK)*UKJ*U(1,I)                
  100 IJK=IJK+1                                               
      IN=IN-1                                                 
      IF(IN.GT.0.AND.INDX.GT.0) GOTO 90                       
      RETURN                                                  
  110 PRINT 120,LS                                            
  120 FORMAT( 1X,15('*'),' R(',I1,') SORT DE LA GRILLE')      
      print*,r(ls)                                                              
      RETURN 
      END                                                       

!#######################################################################

      SUBROUTINE FORMV0_PT2(I,J,K,G)                                   
      IMPLICIT REAL*8(A-H,O-Z)                         
      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      DIMENSION G(64)                                           

      common /surfnof0_pt2/ F11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridnof0_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof0_pt2/,/gridnof0_pt2/
!      
      IJK=(K-1)*NX*NY+(J-1)*NX+I                               
      IMJK=IJK-1                                               
      IJMK=IJK-NX                                              
      IMJMK=IJMK-1                                             
      IJKM=IJK-NX*NY                                          
      IMJKM=IJKM-1                                            
      IJMKM=IJKM-NX                                           
      IMJMKM=IJMKM-1                                          
!                                                             
      G( 1)=F11(IJK)                                            
      G( 2)=F11(IMJK)                                           
      G( 3)=PX11(IJK)                                          
      G( 4)=PX11(IMJK)                                         
      G( 5)=F11(IJMK)                                          
      G( 6)=F11(IMJMK)                                         
      G( 7)=PX11(IJMK)                                         
      G( 8)=PX11(IMJMK)                                        
      G( 9)=PY11(IJK)                                          
      G(10)=PY11(IMJK)                                        
      G(11)=PXY11(IJK)                                        
      G(12)=PXY11(IMJK)                                       
      G(13)=PY11(IJMK)                                        
      G(14)=PY11(IMJMK)                                       
      G(15)=PXY11(IJMK)                                       
      G(16)=PXY11(IMJMK)                                      
      G(17)=F11(IJKM)                                         
      G(18)=F11(IMJKM)                                        
      G(19)=PX11(IJKM)                                        
      G(20)=PX11(IMJKM)                                       
      G(21)=F11(IJMKM)                                        
      G(22)=F11(IMJMKM)                                       
      G(23)=PX11(IJMKM)                                       
      G(24)=PX11(IMJMKM)                                      
      G(25)=PY11(IJKM)                                        
      G(26)=PY11(IMJKM)                                       
      G(27)=PXY11(IJKM)                                       
      G(28)=PXY11(IMJKM)                                      
      G(29)=PY11(IJMKM)                                       
      G(30)=PY11(IMJMKM)                                      
      G(31)=PXY11(IJMKM)                                      
      G(32)=PXY11(IMJMKM)                                     
      G(33)=PZ11(IJK)                                         
      G(34)=PZ11(IMJK)                                        
      G(35)=PXZ11(IJK)                                        
      G(36)=PXZ11(IMJK)                                       
      G(37)=PZ11(IJMK)                                        
      G(38)=PZ11(IMJMK)                                       
      G(39)=PXZ11(IJMK)                                       
      G(40)=PXZ11(IMJMK)                                      
      G(41)=PYZ11(IJK)                                        
      G(42)=PYZ11(IMJK)                                       
      G(43)=PXYZ11(IJK)                                       
      G(44)=PXYZ11(IMJK)                                      
      G(45)=PYZ11(IJMK)                                       
      G(46)=PYZ11(IMJMK)                                      
      G(47)=PXYZ11(IJMK)                                      
      G(48)=PXYZ11(IMJMK)                                     
      G(49)=PZ11(IJKM)                                        
      G(50)=PZ11(IMJKM)                                       
      G(51)=PXZ11(IJKM)                                       
      G(52)=PXZ11(IMJKM)                                      
      G(53)=PZ11(IJMKM)                                      
      G(54)=PZ11(IMJMKM)                                     
      G(55)=PXZ11(IJMKM)                                     
      G(56)=PXZ11(IMJMKM)                                    
      G(57)=PYZ11(IJKM)                                      
      G(58)=PYZ11(IMJKM)                                    
      G(59)=PXYZ11(IJKM)                                    
      G(60)=PXYZ11(IMJKM)                                   
      G(61)=PYZ11(IJMKM)                                    
      G(62)=PYZ11(IMJMKM)                                   
      G(63)=PXYZ11(IJMKM)                                   
      G(64)=PXYZ11(IMJMKM)                                  
      RETURN                                              
      END                                                 
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
                                                                                
      SUBROUTINE NATURL_NOF_PT2_EC1(R,INDX,FIN)  
!                                                                    
!     INDX = 0  :  INTERPOLATED POTENTIAL ONLY                       
!            3  :        "      X,Y,Z-DERIVATIVES                    
!                                                                    
      IMPLICIT REAL*8(A-H,O-Z)                             
      parameter (ndx=160, ndy=60,ndz=110)
      DIMENSION FIN(4),FIJK(64),HI(3),XR(3),U(3,4),R(3)              
      common/gridnof1_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof1_pt2/

      DATA UN/1.D0/,TWO/2.D0/,THREE/3.D0/,SIX/6.D0/,IV,JV,KV/3*0/   
!                                                                   
      LS=1                                                                      
      IF(R(1).LT.X(1).OR.R(1).GT.X(NX)) GOTO 110                   
      LS=2                                                        
      IF(R(2).LT.Y(1).OR.R(2).GT.Y(NY)) GOTO 110                  
      LS=3                                                        
      IF(R(3).LT.Z(1).OR.R(3).GT.Z(NZ)) GOTO 110                  
      DO 10 IS=2,NX                                               
      IF(R(1).LT.X(IS)) GOTO 20                                   
   10 CONTINUE                                                    
   20 DO 30 JS=2,NY                                               
      IF(R(2).LT.Y(JS)) GOTO 40                                   
   30 CONTINUE                                                    
   40 DO 50 KS=2,NZ                                               
      IF(R(3).LT.Z(KS)) GOTO 60                                   
   50 CONTINUE                                                    
   60 IF(IS.NE.IV) GOTO 65                                        
      IF(JS.NE.JV) GOTO 65                                        
      IF(KS.EQ.KV) GOTO 75                                        
   65 CALL FORMV1_PT2(IS,JS,KS,FIJK)                                   
      HI(1)=X(IS)-X(IS-1)                                         
      IV=IS                                                       
      HI(2)=Y(JS)-Y(JS-1)                                         
      JV=JS                                                       
      HI(3)=Z(KS)-Z(KS-1)                                         
      KV=KS                                                      
   75 XR(1)=(R(1)-X(IS-1))/HI(1)                                
      XR(2)=(R(2)-Y(JS-1))/HI(2)                                
      XR(3)=(R(3)-Z(KS-1))/HI(3)                                
      IN=INDX                                                   
   90 DO 70 I=1,3                                               
      IF(I.EQ.IN) GOTO 70                                       
      XR2=XR(I)**2                                              
      XR3=XR(I)*XR2                                             
      U(I,1)=-TWO*XR3+THREE*XR2                                 
      U(I,2)=-U(I,1)+UN                                         
      U(I,3)=HI(I)*(XR3-XR2)                                   
      U(I,4)=HI(I)*(XR3-TWO*XR2+XR(I))                         
   70 CONTINUE                                                 
      IF(IN.EQ.0) GOTO 80                                      
      U(IN,1)=SIX*(-XR(IN)+1)*XR(IN)/HI(IN)                    
      U(IN,2)=-U(IN,1)                                         
      U(IN,3)=(THREE*XR(IN)-TWO)*XR(IN)                        
      U(IN,4)=U(IN,3)-TWO*XR(IN)+UN                            
   80 FIN(IN+1)=0.D0                                           
      IJK=1                                                    
      DO 100 K=1,4                                             
      DO 100 J=1,4                                             
      UKJ=U(3,K)*U(2,J)                                        
      DO 100 I=1,4                                            
      FIN(IN+1)=FIN(IN+1)+FIJK(IJK)*UKJ*U(1,I)                
  100 IJK=IJK+1                                               
      IN=IN-1                                                 
      IF(IN.GT.0.AND.INDX.GT.0) GOTO 90                       
      RETURN                                                  
  110 PRINT 120,LS                                            
  120 FORMAT( 1X,15('*'),' R(',I1,') SORT DE LA GRILLE')      
      print*,r(ls)                                                              
      RETURN 
      END                                                       

!#######################################################################

      SUBROUTINE FORMV1_PT2(I,J,K,G)                                   
      IMPLICIT REAL*8(A-H,O-Z)                         
      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      DIMENSION G(64)                                           

      common /surfnof1_pt2/ F11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridnof1_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof1_pt2/,/gridnof1_pt2/
!      
      IJK=(K-1)*NX*NY+(J-1)*NX+I                               
      IMJK=IJK-1                                               
      IJMK=IJK-NX                                              
      IMJMK=IJMK-1                                             
      IJKM=IJK-NX*NY                                          
      IMJKM=IJKM-1                                            
      IJMKM=IJKM-NX                                           
      IMJMKM=IJMKM-1                                          
!                                                             
      G( 1)=F11(IJK)                                            
      G( 2)=F11(IMJK)                                           
      G( 3)=PX11(IJK)                                          
      G( 4)=PX11(IMJK)                                         
      G( 5)=F11(IJMK)                                          
      G( 6)=F11(IMJMK)                                         
      G( 7)=PX11(IJMK)                                         
      G( 8)=PX11(IMJMK)                                        
      G( 9)=PY11(IJK)                                          
      G(10)=PY11(IMJK)                                        
      G(11)=PXY11(IJK)                                        
      G(12)=PXY11(IMJK)                                       
      G(13)=PY11(IJMK)                                        
      G(14)=PY11(IMJMK)                                       
      G(15)=PXY11(IJMK)                                       
      G(16)=PXY11(IMJMK)                                      
      G(17)=F11(IJKM)                                         
      G(18)=F11(IMJKM)                                        
      G(19)=PX11(IJKM)                                        
      G(20)=PX11(IMJKM)                                       
      G(21)=F11(IJMKM)                                        
      G(22)=F11(IMJMKM)                                       
      G(23)=PX11(IJMKM)                                       
      G(24)=PX11(IMJMKM)                                      
      G(25)=PY11(IJKM)                                        
      G(26)=PY11(IMJKM)                                       
      G(27)=PXY11(IJKM)                                       
      G(28)=PXY11(IMJKM)                                      
      G(29)=PY11(IJMKM)                                       
      G(30)=PY11(IMJMKM)                                      
      G(31)=PXY11(IJMKM)                                      
      G(32)=PXY11(IMJMKM)                                     
      G(33)=PZ11(IJK)                                         
      G(34)=PZ11(IMJK)                                        
      G(35)=PXZ11(IJK)                                        
      G(36)=PXZ11(IMJK)                                       
      G(37)=PZ11(IJMK)                                        
      G(38)=PZ11(IMJMK)                                       
      G(39)=PXZ11(IJMK)                                       
      G(40)=PXZ11(IMJMK)                                      
      G(41)=PYZ11(IJK)                                        
      G(42)=PYZ11(IMJK)                                       
      G(43)=PXYZ11(IJK)                                       
      G(44)=PXYZ11(IMJK)                                      
      G(45)=PYZ11(IJMK)                                       
      G(46)=PYZ11(IMJMK)                                      
      G(47)=PXYZ11(IJMK)                                      
      G(48)=PXYZ11(IMJMK)                                     
      G(49)=PZ11(IJKM)                                        
      G(50)=PZ11(IMJKM)                                       
      G(51)=PXZ11(IJKM)                                       
      G(52)=PXZ11(IMJKM)                                      
      G(53)=PZ11(IJMKM)                                      
      G(54)=PZ11(IMJMKM)                                     
      G(55)=PXZ11(IJMKM)                                     
      G(56)=PXZ11(IMJMKM)                                    
      G(57)=PYZ11(IJKM)                                      
      G(58)=PYZ11(IMJKM)                                    
      G(59)=PXYZ11(IJKM)                                    
      G(60)=PXYZ11(IMJKM)                                   
      G(61)=PYZ11(IJMKM)                                    
      G(62)=PYZ11(IMJMKM)                                   
      G(63)=PXYZ11(IJMKM)                                   
      G(64)=PXYZ11(IMJMKM)                                  
      RETURN                                              
      END   
 
!###########################################################
                                             
      SUBROUTINE NATURL_NOF_PT2_EC2(R,INDX,FIN)        
!                                                                    
!     INDX = 0  :  INTERPOLATED POTENTIAL ONLY                       
!            3  :        "      X,Y,Z-DERIVATIVES                    
!                                                                    
      IMPLICIT REAL*8(A-H,O-Z)                             
      parameter (ndx=160, ndy=60,ndz=110)
      DIMENSION FIN(4),FIJK(64),HI(3),XR(3),U(3,4),R(3)              
      common/gridnof2_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof2_pt2/

      DATA UN/1.D0/,TWO/2.D0/,THREE/3.D0/,SIX/6.D0/,IV,JV,KV/3*0/   
!                                                                   
      LS=1                                                                      
      IF(R(1).LT.X(1).OR.R(1).GT.X(NX)) GOTO 110                   
      LS=2                                                        
      IF(R(2).LT.Y(1).OR.R(2).GT.Y(NY)) GOTO 110                  
      LS=3                                                        
      IF(R(3).LT.Z(1).OR.R(3).GT.Z(NZ)) GOTO 110                  
      DO 10 IS=2,NX                                               
      IF(R(1).LT.X(IS)) GOTO 20                                   
   10 CONTINUE                                                    
   20 DO 30 JS=2,NY                                               
      IF(R(2).LT.Y(JS)) GOTO 40                                   
   30 CONTINUE                                                    
   40 DO 50 KS=2,NZ                                               
      IF(R(3).LT.Z(KS)) GOTO 60                                   
   50 CONTINUE                                                    
   60 IF(IS.NE.IV) GOTO 65                                        
      IF(JS.NE.JV) GOTO 65                                        
      IF(KS.EQ.KV) GOTO 75                                        
   65 CALL FORMV2_PT2(IS,JS,KS,FIJK)                                   
      HI(1)=X(IS)-X(IS-1)                                         
      IV=IS                                                       
      HI(2)=Y(JS)-Y(JS-1)                                         
      JV=JS                                                       
      HI(3)=Z(KS)-Z(KS-1)                                         
      KV=KS                                                      
   75 XR(1)=(R(1)-X(IS-1))/HI(1)                                
      XR(2)=(R(2)-Y(JS-1))/HI(2)                                
      XR(3)=(R(3)-Z(KS-1))/HI(3)                                
      IN=INDX                                                   
   90 DO 70 I=1,3                                               
      IF(I.EQ.IN) GOTO 70                                       
      XR2=XR(I)**2                                              
      XR3=XR(I)*XR2                                             
      U(I,1)=-TWO*XR3+THREE*XR2                                 
      U(I,2)=-U(I,1)+UN                                         
      U(I,3)=HI(I)*(XR3-XR2)                                   
      U(I,4)=HI(I)*(XR3-TWO*XR2+XR(I))                         
   70 CONTINUE                                                 
      IF(IN.EQ.0) GOTO 80                                      
      U(IN,1)=SIX*(-XR(IN)+1)*XR(IN)/HI(IN)                    
      U(IN,2)=-U(IN,1)                                         
      U(IN,3)=(THREE*XR(IN)-TWO)*XR(IN)                        
      U(IN,4)=U(IN,3)-TWO*XR(IN)+UN                            
   80 FIN(IN+1)=0.D0                                           
      IJK=1                                                    
      DO 100 K=1,4                                             
      DO 100 J=1,4                                             
      UKJ=U(3,K)*U(2,J)                                        
      DO 100 I=1,4                                            
      FIN(IN+1)=FIN(IN+1)+FIJK(IJK)*UKJ*U(1,I)                
  100 IJK=IJK+1                                               
      IN=IN-1                                                 
      IF(IN.GT.0.AND.INDX.GT.0) GOTO 90                       
      RETURN                                                  
  110 PRINT 120,LS                                            
  120 FORMAT( 1X,15('*'),' R(',I1,') SORT DE LA GRILLE')      
      print*,r(ls)                                                              
      RETURN 
      END                                                       

!#######################################################################

      SUBROUTINE FORMV2_PT2(I,J,K,G)                                   
      IMPLICIT REAL*8(A-H,O-Z)                         
      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      DIMENSION G(64)                                           

      common /surfnof2_pt2/ F11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridnof2_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof2_pt2/,/gridnof2_pt2/
!      
      IJK=(K-1)*NX*NY+(J-1)*NX+I                               
      IMJK=IJK-1                                               
      IJMK=IJK-NX                                              
      IMJMK=IJMK-1                                             
      IJKM=IJK-NX*NY                                          
      IMJKM=IJKM-1                                            
      IJMKM=IJKM-NX                                           
      IMJMKM=IJMKM-1                                          
!                                                             
      G( 1)=F11(IJK)                                            
      G( 2)=F11(IMJK)                                           
      G( 3)=PX11(IJK)                                          
      G( 4)=PX11(IMJK)                                         
      G( 5)=F11(IJMK)                                          
      G( 6)=F11(IMJMK)                                         
      G( 7)=PX11(IJMK)                                         
      G( 8)=PX11(IMJMK)                                        
      G( 9)=PY11(IJK)                                          
      G(10)=PY11(IMJK)                                        
      G(11)=PXY11(IJK)                                        
      G(12)=PXY11(IMJK)                                       
      G(13)=PY11(IJMK)                                        
      G(14)=PY11(IMJMK)                                       
      G(15)=PXY11(IJMK)                                       
      G(16)=PXY11(IMJMK)                                      
      G(17)=F11(IJKM)                                         
      G(18)=F11(IMJKM)                                        
      G(19)=PX11(IJKM)                                        
      G(20)=PX11(IMJKM)                                       
      G(21)=F11(IJMKM)                                        
      G(22)=F11(IMJMKM)                                       
      G(23)=PX11(IJMKM)                                       
      G(24)=PX11(IMJMKM)                                      
      G(25)=PY11(IJKM)                                        
      G(26)=PY11(IMJKM)                                       
      G(27)=PXY11(IJKM)                                       
      G(28)=PXY11(IMJKM)                                      
      G(29)=PY11(IJMKM)                                       
      G(30)=PY11(IMJMKM)                                      
      G(31)=PXY11(IJMKM)                                      
      G(32)=PXY11(IMJMKM)                                     
      G(33)=PZ11(IJK)                                         
      G(34)=PZ11(IMJK)                                        
      G(35)=PXZ11(IJK)                                        
      G(36)=PXZ11(IMJK)                                       
      G(37)=PZ11(IJMK)                                        
      G(38)=PZ11(IMJMK)                                       
      G(39)=PXZ11(IJMK)                                       
      G(40)=PXZ11(IMJMK)                                      
      G(41)=PYZ11(IJK)                                        
      G(42)=PYZ11(IMJK)                                       
      G(43)=PXYZ11(IJK)                                       
      G(44)=PXYZ11(IMJK)                                      
      G(45)=PYZ11(IJMK)                                       
      G(46)=PYZ11(IMJMK)                                      
      G(47)=PXYZ11(IJMK)                                      
      G(48)=PXYZ11(IMJMK)                                     
      G(49)=PZ11(IJKM)                                        
      G(50)=PZ11(IMJKM)                                       
      G(51)=PXZ11(IJKM)                                       
      G(52)=PXZ11(IMJKM)                                      
      G(53)=PZ11(IJMKM)                                      
      G(54)=PZ11(IMJMKM)                                     
      G(55)=PXZ11(IJMKM)                                     
      G(56)=PXZ11(IMJMKM)                                    
      G(57)=PYZ11(IJKM)                                      
      G(58)=PYZ11(IMJKM)                                    
      G(59)=PXYZ11(IJKM)                                    
      G(60)=PXYZ11(IMJKM)                                   
      G(61)=PYZ11(IJMKM)                                    
      G(62)=PYZ11(IMJMKM)                                   
      G(63)=PXYZ11(IJMKM)                                   
      G(64)=PXYZ11(IMJMKM)                                  
      RETURN                                              
      END  

                                               
      SUBROUTINE NATURL_NOF_PT2_EC3(R,INDX,FIN)     
!                                                                    
!     INDX = 0  :  INTERPOLATED POTENTIAL ONLY                       
!            3  :        "      X,Y,Z-DERIVATIVES                    
!                                                                    
      IMPLICIT REAL*8(A-H,O-Z)                             
      parameter (ndx=160, ndy=60,ndz=110)
      DIMENSION FIN(4),FIJK(64),HI(3),XR(3),U(3,4),R(3)              
      common/gridnof3_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridnof3_pt2/

      DATA UN/1.D0/,TWO/2.D0/,THREE/3.D0/,SIX/6.D0/,IV,JV,KV/3*0/   
!                                                                   
      LS=1                                                                      
      IF(R(1).LT.X(1).OR.R(1).GT.X(NX)) GOTO 110                   
      LS=2                                                        
      IF(R(2).LT.Y(1).OR.R(2).GT.Y(NY)) GOTO 110                  
      LS=3                                                        
      IF(R(3).LT.Z(1).OR.R(3).GT.Z(NZ)) GOTO 110                  
      DO 10 IS=2,NX                                               
      IF(R(1).LT.X(IS)) GOTO 20                                   
   10 CONTINUE                                                    
   20 DO 30 JS=2,NY                                               
      IF(R(2).LT.Y(JS)) GOTO 40                                   
   30 CONTINUE                                                    
   40 DO 50 KS=2,NZ                                               
      IF(R(3).LT.Z(KS)) GOTO 60                                   
   50 CONTINUE                                                    
   60 IF(IS.NE.IV) GOTO 65                                        
      IF(JS.NE.JV) GOTO 65                                        
      IF(KS.EQ.KV) GOTO 75                                        
   65 CALL FORMV3_PT2(IS,JS,KS,FIJK)                                   
      HI(1)=X(IS)-X(IS-1)                                         
      IV=IS                                                       
      HI(2)=Y(JS)-Y(JS-1)                                         
      JV=JS                                                       
      HI(3)=Z(KS)-Z(KS-1)                                         
      KV=KS                                                      
   75 XR(1)=(R(1)-X(IS-1))/HI(1)                                
      XR(2)=(R(2)-Y(JS-1))/HI(2)                                
      XR(3)=(R(3)-Z(KS-1))/HI(3)                                
      IN=INDX                                                   
   90 DO 70 I=1,3                                               
      IF(I.EQ.IN) GOTO 70                                       
      XR2=XR(I)**2                                              
      XR3=XR(I)*XR2                                             
      U(I,1)=-TWO*XR3+THREE*XR2                                 
      U(I,2)=-U(I,1)+UN                                         
      U(I,3)=HI(I)*(XR3-XR2)                                   
      U(I,4)=HI(I)*(XR3-TWO*XR2+XR(I))                         
   70 CONTINUE                                                 
      IF(IN.EQ.0) GOTO 80                                      
      U(IN,1)=SIX*(-XR(IN)+1)*XR(IN)/HI(IN)                    
      U(IN,2)=-U(IN,1)                                         
      U(IN,3)=(THREE*XR(IN)-TWO)*XR(IN)                        
      U(IN,4)=U(IN,3)-TWO*XR(IN)+UN                            
   80 FIN(IN+1)=0.D0                                           
      IJK=1                                                    
      DO 100 K=1,4                                             
      DO 100 J=1,4                                             
      UKJ=U(3,K)*U(2,J)                                        
      DO 100 I=1,4                                            
      FIN(IN+1)=FIN(IN+1)+FIJK(IJK)*UKJ*U(1,I)                
  100 IJK=IJK+1                                               
      IN=IN-1                                                 
      IF(IN.GT.0.AND.INDX.GT.0) GOTO 90                       
      RETURN                                                  
  110 PRINT 120,LS                                            
  120 FORMAT( 1X,15('*'),' R(',I1,') SORT DE LA GRILLE')      
      print*,r(ls)                                                              
      RETURN 
      END                                                       

!#######################################################################

      SUBROUTINE FORMV3_PT2(I,J,K,G)                                   
      IMPLICIT REAL*8(A-H,O-Z)                         
      parameter (ndat=100000,ndx=160,ndy=60,ndz=110)
      DIMENSION G(64)                                           

      common /surfnof3_pt2/ F11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridnof3_pt2/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfnof3_pt2/,/gridnof3_pt2/
!      
      IJK=(K-1)*NX*NY+(J-1)*NX+I                               
      IMJK=IJK-1                                               
      IJMK=IJK-NX                                              
      IMJMK=IJMK-1                                             
      IJKM=IJK-NX*NY                                          
      IMJKM=IJKM-1                                            
      IJMKM=IJKM-NX                                           
      IMJMKM=IJMKM-1                                          
!                                                             
      G( 1)=F11(IJK)                                            
      G( 2)=F11(IMJK)                                           
      G( 3)=PX11(IJK)                                          
      G( 4)=PX11(IMJK)                                         
      G( 5)=F11(IJMK)                                          
      G( 6)=F11(IMJMK)                                         
      G( 7)=PX11(IJMK)                                         
      G( 8)=PX11(IMJMK)                                        
      G( 9)=PY11(IJK)                                          
      G(10)=PY11(IMJK)                                        
      G(11)=PXY11(IJK)                                        
      G(12)=PXY11(IMJK)                                       
      G(13)=PY11(IJMK)                                        
      G(14)=PY11(IMJMK)                                       
      G(15)=PXY11(IJMK)                                       
      G(16)=PXY11(IMJMK)                                      
      G(17)=F11(IJKM)                                         
      G(18)=F11(IMJKM)                                        
      G(19)=PX11(IJKM)                                        
      G(20)=PX11(IMJKM)                                       
      G(21)=F11(IJMKM)                                        
      G(22)=F11(IMJMKM)                                       
      G(23)=PX11(IJMKM)                                       
      G(24)=PX11(IMJMKM)                                      
      G(25)=PY11(IJKM)                                        
      G(26)=PY11(IMJKM)                                       
      G(27)=PXY11(IJKM)                                       
      G(28)=PXY11(IMJKM)                                      
      G(29)=PY11(IJMKM)                                       
      G(30)=PY11(IMJMKM)                                      
      G(31)=PXY11(IJMKM)                                      
      G(32)=PXY11(IMJMKM)                                     
      G(33)=PZ11(IJK)                                         
      G(34)=PZ11(IMJK)                                        
      G(35)=PXZ11(IJK)                                        
      G(36)=PXZ11(IMJK)                                       
      G(37)=PZ11(IJMK)                                        
      G(38)=PZ11(IMJMK)                                       
      G(39)=PXZ11(IJMK)                                       
      G(40)=PXZ11(IMJMK)                                      
      G(41)=PYZ11(IJK)                                        
      G(42)=PYZ11(IMJK)                                       
      G(43)=PXYZ11(IJK)                                       
      G(44)=PXYZ11(IMJK)                                      
      G(45)=PYZ11(IJMK)                                       
      G(46)=PYZ11(IMJMK)                                      
      G(47)=PXYZ11(IJMK)                                      
      G(48)=PXYZ11(IMJMK)                                     
      G(49)=PZ11(IJKM)                                        
      G(50)=PZ11(IMJKM)                                       
      G(51)=PXZ11(IJKM)                                       
      G(52)=PXZ11(IMJKM)                                      
      G(53)=PZ11(IJMKM)                                      
      G(54)=PZ11(IMJMKM)                                     
      G(55)=PXZ11(IJMKM)                                     
      G(56)=PXZ11(IMJMKM)                                    
      G(57)=PYZ11(IJKM)                                      
      G(58)=PYZ11(IMJKM)                                    
      G(59)=PXYZ11(IJKM)                                    
      G(60)=PXYZ11(IMJKM)                                   
      G(61)=PYZ11(IJMKM)                                    
      G(62)=PYZ11(IMJMKM)                                   
      G(63)=PXYZ11(IJMKM)                                   
      G(64)=PXYZ11(IMJMKM)                                  
      RETURN                                              
      END                                                 
