!#######################################################################
      module potofmod

use decimal, only: dop,long

      implicit none
      private
      public :: potof
      contains

!#######################################################################
!  Potential kurve for the OF diatom
!***********************************************************************

      subroutine potof(x,v)

      implicit none
      integer(long)                  :: i
      real(dop), dimension(32)       :: work
      real(dop), dimension(32), save :: xa,ya,y2
      real(dop), intent(in)          :: x
      real(dop), intent(out)         :: v
      logical(kind=4), save          :: firstcall = .true.
      integer(long), parameter       :: npotpt=28

      if(firstcall) then
         firstcall=.false.
         xa( 1)=1.90_dop
         ya( 1)= -0.49784522_dop
         xa( 2)=2.00_dop
         ya( 2)= -0.57195511_dop
         xa( 3)=2.10_dop
         ya( 3)= -0.62202962_dop
         xa( 4)=2.20_dop
         ya( 4)= -0.65482320_dop
         xa( 5)= 2.30_dop
         ya( 5)= -0.67518279_dop
         xa( 6)=2.40_dop
         ya( 6)= -0.68660065_dop
         xa( 7)=2.50_dop
         ya( 7)= -0.69161409_dop
         xa( 8)=2.60_dop
         ya( 8)= -0.69208897_dop
         xa( 9)=2.70_dop
         ya( 9)= -0.68941610_dop
         xa(10)=2.80_dop
         ya(10)= -0.68464497_dop
         xa(11)=2.90_dop
         ya(11)= -0.67857465_dop
         xa(12)=3.00_dop
         ya(12)= -0.67181608_dop
         xa(13)= 3.100
         ya(13)= -0.66483533_dop
         xa(14)=3.20_dop
         ya(14)= -0.65798352_dop
         xa(15)=3.300
         ya(15)= -0.65151721_dop
         xa(16)=3.40_dop
         ya(16)= -0.64561211_dop
         xa(17)= 3.500
         ya(17)= -0.64037302_dop
         xa(18)=3.60_dop
         ya(18)= -0.63584308_dop
         xa(19)=3.70_dop
         ya(19)= -0.63201394_dop
         xa(20)=3.90_dop
         ya(20)= -0.62624513_dop
         xa(21)=4.00_dop
         ya(21)= -0.62415019_dop
         xa(22)=4.10_dop
         ya(22)= -0.62247019_dop
         xa(23)=4.20_dop
         ya(23)= -0.62112769_dop
         xa(24)=4.30_dop
         ya(24)= -0.62005584_dop
         xa(25)=4.40_dop
         ya(25)= -0.61919921_dop
         xa(26)=4.50_dop
         ya(26)= -0.61851316_dop
         xa(27)=4.60_dop
         ya(27)= -0.61796244_dop
         xa(28)=4.70_dop
         ya(28)= -0.61751948_dop

!.......Set the potential minimum to zero.
         do i = 1, npotpt
            ya(i) = ya(i) + 0.6923466_dop
         enddo

! compute second  derivatives
         call OF_spline(xa,ya,npotpt,1.0e40_dop,1.0e40_dop,y2,work)
      endif

! interpolates with cubic spline at grid points
      if (x.lt.xa(1)) then
         v=ya(1)
      elseif (x.gt.xa(npotpt)) then
         v=ya(npotpt)
      else
         call OF_splint (xa,ya,y2,npotpt,x,v)
      endif

      RETURN
      END subroutine potof

!=======================================================================
      SUBROUTINE OF_SPLINT(XA,YA,Y2A,N,X,Y)
!=======================================================================
! computes spline interpolation at point X,for function ya=f(xa)
! given Y2A, second derivative computed in next routine

      implicit none
      integer(long)                       :: KLO,KHI,K
      integer(long), intent(in)           :: N
      real(dop)                           :: A,B,H
      real(dop), intent(in)               :: X
      real(dop), intent(out)              :: Y
      real(dop), dimension(:), intent(in) :: xa,ya,y2a

      KLO=1
      KHI=N
1     IF (KHI-KLO.GT.1) THEN
        K=(KHI+KLO)/2
        IF(XA(K).GT.X)THEN
          KHI=K
        ELSE
          KLO=K
        ENDIF
      GOTO 1
      ENDIF
      H=XA(KHI)-XA(KLO)
      IF (H.EQ.0.D0) THEN
         WRITE(6,'(A)') 'potof.f, splint: Bad XA input'
         STOP
      ENDIF
      A=(XA(KHI)-X)/H
      B=(X-XA(KLO))/H
      Y=A*YA(KLO)+B*YA(KHI)+ &
            ((A**3-A)*Y2A(KLO)+(B**3-B)*Y2A(KHI))*(H**2)/6.D0
      RETURN
      END SUBROUTINE OF_SPLINT

!=======================================================================
      SUBROUTINE OF_SPLINE(X,Y,N,YP1,YPN,Y2,U)
!=======================================================================
! Computes second derivative of function y=f(x)

      implicit none
      integer(long)                       :: I,K
      integer(long), intent(in)           :: N
      real(dop)                           :: P,SIG,QN,UN
      real(dop), intent(in)               :: YP1,YPN
      real(dop), dimension(*), intent(out):: U
      real(dop), dimension(N), intent(in) :: X,Y
      real(dop), dimension(N), intent(out):: Y2

      IF (YP1.GT..99D30) THEN
        Y2(1)=0.D0
        U(1)=0.D0
      ELSE
        Y2(1)=-0.5D0
        U(1)=(3.D0/(X(2)-X(1)))*((Y(2)-Y(1))/(X(2)-X(1))-YP1)
      ENDIF
      DO I=2,N-1
        SIG=(X(I)-X(I-1))/(X(I+1)-X(I-1))
        P=SIG*Y2(I-1)+2.D0
        Y2(I)=(SIG-1.D0)/P
        U(I)=(6.D0*((Y(I+1)-Y(I))/(X(I+1)-X(I))-(Y(I)-Y(I-1)) &
            /(X(I)-X(I-1)))/(X(I+1)-X(I-1))-SIG*U(I-1))/P
      END DO
      IF (YPN.GT..99D30) THEN
        QN=0.D0
        UN=0.D0
      ELSE
        QN=0.5D0
        UN=(3.D0/(X(N)-X(N-1)))*(YPN-(Y(N)-Y(N-1))/(X(N)-X(N-1)))
      ENDIF
      Y2(N)=(UN-QN*U(N-1))/(QN*Y2(N-1)+1.D0)
      DO K=N-1,1,-1
        Y2(K)=Y2(K)*Y2(K+1)+U(K)
      END DO
      RETURN
      END SUBROUTINE OF_SPLINE


!#######################################################################
      end module potofmod

