module splinepot1dmod

use splinepotmod

public
contains

!!!!! Cristina Sanz Sanz: 27 February 2009
! MCTDH does not accept modules yet.
!
! This spline subroutine is modified to read the values from the 
! input file. 
! The file is read once for each different input file, storing the
! coordinates in vectors, that way one does not need to give the
! intervals or initial values. In principle it is general and would
! work with any grid which behaves well.  
! The values to read have to be:
! 1st line of input: Total number of points (ntria)
! 2nd line of input: Number of points in coordinate 1 and in 2
! From 3rd line on: coord1, coord2, energy
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! POT
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine splinepot1d_css(cc1,v,input)

      implicit none

      integer, save :: init=0

!....Arrays to keep the values of the input file
      double precision, allocatable, dimension(:),save :: c1
      double precision, allocatable, dimension(:),save :: vve

!....Variable to be read from input file
      integer, save :: nc1

!....In and out
      double precision, intent(in) :: cc1
      double precision, intent(out) :: v

      character(*), intent(in):: input     
      character(80) :: inp='a'
      character(1) :: string

      integer :: ierr

!....Auxiliary matrices
      double precision, allocatable, dimension(:),save :: yd

!....Loop indexes
      integer :: j

!!.....Fill out variable arrays.

       if(init==0.or.(inp/=input))then
          OPEN (50,FILE=input,status="old",iostat=ierr)
          if (ierr .ne. 0) then
             write(6,*)'Cannot open the surface data file: ',input
             stop
          endif
          
          rewind(50)
   10     continue
          read(50,'(a1)') string
          if (string == '#') go to 10
          backspace(50)
          read(50,*) nc1
 
          if(allocated(c1))deallocate(c1)
          if(allocated(vve))deallocate(vve)

          allocate(c1(nc1))
          allocate(vve(nc1))

          do j=1,nc1
             read(50,*)c1(j),vve(j)
          enddo
          init=1          
          inp=input

!.....Computing derivatives
          if(allocated(yd))deallocate(yd)
          allocate(yd(nc1))
          call spline_css(c1,vve,nc1,1.d30,1.d30,yd) !derivatives
       endif

! 1 dimension splines
      
!.....Interpolation 1d with splines

      call splint (c1,vve,yd,nc1,cc1,v)

! deallocations - should deallocate these arrays when finished. But not
! here...
!      if(allocated(c1))deallocate(c1)
!      if(allocated(vve))deallocate(vve)
!      if(allocated(yd))deallocate(yd)   

      return
      end subroutine splinepot1d_css


end module splinepot1dmod
