module splinepot2dmod

use splinepotmod

public
contains

!!!!! Cristina Sanz Sanz: 27 February 2009
! MCTDH does not accept modules yet.
!
! This spline subroutine is modified to read the values from the 
! input file. 
! The file is read once for each different input file, storing the
! coordinates in vectors, that way one does not need to give the
! intervals or initial values. In principle it is general and would
! work with any grid which behaves well.  
! The values to read have to be:
! 1st line of input: Total number of points (ntria)
! 2nd line of input: Number of points in coordinate 1 and in 2
! From 3rd line on: coord1, coord2, energy
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! POT
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine splinepot2d_css(cc1,cc2,v,input)
      implicit none

      integer, save :: init=0

!....Arrays to keep the values of the input file
      double precision, allocatable, dimension(:),save :: c1,c2
      double precision, allocatable, dimension(:,:),save :: vve

!....Variable to be read from input file
      integer, save :: ntria, nc1, nc2

!....In and out
      double precision, intent(in) :: cc1, cc2
      double precision, intent(out) :: v

      character(*), intent(in):: input     
      character(80),save :: inp='a'
      character(1) :: string

      integer :: ierr

!....Auxiliary matrices
      double precision, allocatable, dimension(:,:),save :: yd

!....Loop indexes
      integer :: j,k

!!.....Fill out variable arrays.

       if(init==0.or.(inp/=input))then
          OPEN (50,FILE=input,status="old",iostat=ierr)
          if (ierr .ne. 0) then
             write(6,*)'Cannot open the surface data file: ',input
             stop
          endif

          rewind(50)
   10     continue
          read(50,'(a1)') string
          if (string == '#') go to 10
          backspace(50)

          read(50,*) ntria
          read(50,*) nc1, nc2
 
          if(allocated(c1))deallocate(c1)
          if(allocated(c2))deallocate(c2)
          if(allocated(vve))deallocate(vve)

          allocate(c1(nc1),c2(nc2))
          allocate(vve(nc1,nc2))

          do j=1,nc1
             do k=1,nc2
                read(50,*)c1(j),c2(k),vve(j,k)
             enddo
             init=1          
             inp=input
          enddo

!.....Computing derivatives
          if(allocated(yd))deallocate(yd)
          allocate(yd(nc1,nc2))
          call splie2(c1,c2,vve,nc1,nc2,yd) !Computing derivatives
       endif

! 2 dimension splines
      
!.....Interpolation 2d with splines

      call splin2(c1,c2,vve,yd,nc1,nc2,cc1,cc2,v)

! deallocations - should deallocate these arrays when finished. But not
! here...
!      if(allocated(c1))deallocate(c1)
!      if(allocated(c2))deallocate(c2)
!      if(allocated(vve))deallocate(vve)
!      if(allocated(yd))deallocate(yd)

      return
      end subroutine splinepot2d_css


end module splinepot2dmod
