module splinepot3dmod

use splinepotmod

public
contains

!!!!! Cristina Sanz Sanz: 19 February 2009
! MCTDH does not accept modules yet.
! 3d splines subroutine, the procedure creates the 2d interpolation 
!  for the 2 distances and then a 1d interpolation for the angle. 
!  It could be in any other way, subroutine is general.
!
! This spline subroutine is modified to read the values from the 
! input file. 
! The file is read once for each different input file, storing the
! coordinates in vectors, that way one does not need to give the
! intervals or initial values. In principle it is general and would
! work with any grid which behaves well.  
! The values to read have to be:
! 1st line of input: Total number of points (ntria)
! 2nd line of input: Number of points in coordinate 1, in 2  and in 3 
! From 3rd line on: coord1, coord2, coord3, energy
! Coordinate given in the first column will be the one use to create
! the final 1 dimensional splines, choose it carefully.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! POT
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine splinepot3d_css(c1,c2,c3,v,input)
      implicit none

      integer, save :: init=0

!....Arrays to keep the values of the input file
      double precision, allocatable, dimension(:),save :: cc1,cc2,cc3 
      double precision, allocatable, dimension(:,:,:),save :: vve

!....Variable to be read from input file
      integer, save :: ntria, n1, n2, n3

!....In and out
      double precision, intent(in) :: c1,c2,c3
      double precision, intent(out) :: v

      character(*), intent(in):: input     
      character(120), save :: inp='a'
      character(1) :: string

      integer :: ierr

!....Subroutine variables
      double precision, allocatable, dimension(:), save :: y1d

!....Loop indexes
      integer :: i,j,k

!!.....Fill out variable arrays.

       if(init==0.or.(inp/=input))then
          OPEN (50,FILE=input,status="old",iostat=ierr)
          if (ierr .ne. 0) then
             write(6,*)'Cannot open the surface data file: ',input
             stop
          endif

          rewind(50)
   10     continue
          read(50,'(a1)') string
          if (string == '#') go to 10
          backspace(50)
          
          read(50,*) ntria
          read(50,*) n1, n2, n3
 
          if(allocated(cc2))deallocate(cc2)
          if(allocated(cc3))deallocate(cc3)
          if(allocated(cc1))deallocate(cc1)
          if(allocated(y1d))deallocate(y1d)
          if(allocated(vve))deallocate(vve)

          allocate(cc2(n2),cc3(n3),cc1(n1),y1d(n1))
          allocate(vve(n1,n2,n3))

          do i=1,n1
             do j=1,n2                
                do k=1,n3
                   read(50,*)cc1(i),cc2(j),cc3(k),vve(i,j,k)
                enddo
             enddo
          enddo

          if((cc1(2)-cc1(1)).lt.0.d0)&
     &    stop 'First coordinate in spline grid not in increasing order'
          if((cc2(2)-cc2(1)).lt.0.d0)&
     &    stop 'Second coordinate in spline grid not in increasing order'
          if((cc3(2)-cc3(1)).lt.0.d0)&
     &    stop 'Third coordinate in spline grid not in increasing order'

          init=1          
          inp=input
       endif

! 2 dimension splines for r1 and r2 distances
      call spline2d(c2,c3,n2,cc2,n3,cc3,n1,cc1,vve,y1d)
! 1 dimension spline for the angle
      call spline1d(c1,n1,y1d,v,cc1)
      
! deallocations - should deallocate these arrays when finished. But not
! here...
!      if(allocated(cc2))deallocate(cc2)
!      if(allocated(cc3))deallocate(cc3)
!      if(allocated(cc1))deallocate(cc1)
!      if(allocated(y1d))deallocate(y1d)
!      if(allocated(vve))deallocate(vve)

      return
      end subroutine splinepot3d_css

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! SPLINE 2D
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine spline2d(rr1,rr2,nr1,r1,nr2,r2,na,thetaj,vve,ve)
      implicit none

      integer, intent(in) :: nr1, nr2, na
      double precision, intent(in) :: rr1,rr2
      double precision, intent(out), dimension(:) :: ve(na)

!....Arrays to keep the values of the table
      double precision, dimension(:), intent(in) :: r1(nr1),r2(nr2)
      double precision, dimension(:), intent(in) :: thetaj(na)
      double precision, dimension(:,:,:), intent(in) :: vve(na,nr1,nr2)

!....Arrays to keep the potential and derivatives for each E
      double precision, allocatable, dimension(:,:) :: y1, yd

!....Auxiliar values and counters
      integer :: i,j,k
      double precision :: ver
      allocate(y1(nr1,nr2),yd(nr1,nr2))
!.....Interpolation 2d with splines for fixed E. Cris make a subroutine with it
      do i=1,na
         do j=1,nr1  
            do k=1,nr2
               y1(j,k)=vve(i,j,k) !storing V for each E
            enddo
         enddo

         call splie2(r1,r2,y1,nr1,nr2,yd) !Computing derivatives

         call splin2(r1,r2,y1,yd,nr1,nr2,rr1,rr2,ver)
!....Keeping values interpolated in r and theta in ve
         ve(i)=ver
      enddo       
      call flush(10)
      return
        
      end subroutine spline2d

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  SPLINE 1D
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine spline1d(angi,na,vinput,vint,thetaj)
      implicit none
      
!....Arrays to keep the values of the table
      integer, intent(in) :: na
      double precision, dimension(:), intent(in) :: thetaj(na)

!.....in/out values
      double precision, intent(in) :: angi   !angle to interpolate
      double precision, intent(in), dimension(:) :: vinput(na)
      double precision, intent(out) :: vint       !interpolated potential

!.....arrays to save the values sorted
      double precision, dimension(:) :: y2(na)
 
      call spline_css(thetaj,vinput,na,1d30,1d30,y2)
!.....subtract atomic energies.
!      etable=etable+1.0d0

      call splint(thetaj,vinput,y2,na,angi,vint)

      return
      end subroutine spline1d

end module splinepot3dmod
