module splinepotmod

public

contains

!!!!! Cristina Sanz Sanz: 25 November 2008
! Module 3d splines, the procedure is creating the 2d interpolation 
!  for the 2 distances and then a 1d interpolation for the angle

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! POT
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine splinepot_css(rno,rnf,ang,v,input)
      implicit none

!....Parameters
      integer, parameter :: nr1=16, nr2=16, na=11
      double precision, parameter :: r1i=0.8d0,r2i=0.8d0,ai=52.1290d0
!      double precision, parameter :: r1i=0.8d0,r2i=0.8d0,ai=4.9032d0
!      double precision, parameter :: dr1=0.2d0, dr2=0.2d0 ,da=5.9033d0
      double precision, parameter :: dr1=0.2d0, dr2=0.2d0 ,da=11.8065d0
      integer, save :: init=0

!....Arrays to keep the values of the table
      double precision, dimension(:) :: r1(nr1),r2(nr2),thetaj(na)
      double precision, dimension(:,:,:) :: vve(na,nr1,nr2)

!....In and out
      double precision, intent(in) :: rno, rnf,ang
      double precision, intent(out) :: v

      character*(*), intent(in):: input

      integer ierr

!....Subroutine variables
      double precision, dimension(:) :: y1d(na)
      double precision, allocatable, dimension(:) :: ytria
      integer :: ntria

!....Variables read from the table 
      double precision :: r1j, r2j, thetajj

!....Loop indexes
      integer :: i,j,k,l,iaux

!!.....Fill out variable arrays.
!       if(init.eq.0) then
          r1(1)=r1i
!          write(6,*)' r1(i) '
          do i=2,nr1
             r1(i)=r1(i-1)+dr1
!             write(6,*)r1(i)
          end do
          r2(1)=r2i
!          write(6,*)' r2(i) '
          do i=2,nr2 
             r2(i)=r2(i-1)+dr2
!             write(6,*)r2(i)
          end do
!          write(6,*)' thetaj(i) '
          thetaj(1)=ai
          do i=2,na
             thetaj(i)=thetaj(i-1)+da
!             write(6,*)thetaj(i)
          end do
!          write(6,*)(i,thetaj(i),i=1,na)
          call flush(6)

          OPEN (50,FILE=input,status="old",iostat=ierr)
          if (ierr .ne. 0) then
              write(6,'(a,/a)')     &
          'Cannot open the surface data file: ',input
              stop
           endif

          open(50,file=input,status='old')
          rewind 50
          read(50,*) ntria

          if(allocated(ytria))deallocate(ytria)
          allocate(ytria(ntria))

          iaux=1
          do i=1,ntria
            read(50,*)thetajj,r1j,r2j,ytria(i)

          enddo
!.....Place potential into VV array
          iaux=1
          do j=1,na
             do k=1,nr1
                do l=1,nr2
                   vve(j,k,l)=ytria(iaux)
                   iaux=iaux+1
                 end do
             end do
          end do
         init=1
!       endif

! 2 dimension splines for r1 and r2 distances
      call spline2d1(rno,rnf,r1,r2,thetaj,vve,y1d)

! 1 dimension spline for the angle
      call spline1d1(ang,y1d,v,thetaj)
      
! Deallocate
      if(allocated(ytria))deallocate(ytria)

      return
      end subroutine splinepot_css

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! SPLINE 2D
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

       subroutine spline2d1(rr1,rr2,r1,r2,thetaj,vve,ve)
       implicit none

       integer, parameter :: nr1=16, nr2=16, na=11
       double precision, intent(in) :: rr1,rr2
       double precision, intent(out), dimension(:) :: ve(na)

!....Arrays to keep the values of the table
      double precision, dimension(:), intent(in) :: r1(nr1),r2(nr2),thetaj(na)
      double precision, dimension(:,:,:), intent(in) :: vve(na,nr1,nr2)

!....Arrays to keep the potential and derivatives for each E
       double precision, dimension(:,:) :: y1(nr1,nr2), yd(nr1,nr2)

!....Auxiliar values and counters
       integer :: i,j,k
       double precision :: ver

!.....Interpolation 2d with splines for fixed E. Cris make a subroutine with it
       do i=1,na
          do j=1,nr1               
             do k=1,nr2
                y1(j,k)=vve(i,j,k) !storing V for each E
             enddo
          enddo
          call splie2(r1,r2,y1,nr1,nr2,yd) !Computing derivatives

          call splin2(r1,r2,y1,yd,nr1,nr2,rr1,rr2,ver)
!....Keeping values interpolated in r and theta in ve
          ve(i)=ver
!          write(*,*)i,rr1,rr2,ve(i)
       enddo       

       return
        
       end subroutine spline2d1

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  SPLINE 1D
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

       subroutine spline1d1(angi,vinput,vint,thetaj)
       implicit none
      
       integer, parameter :: nr1=16, nr2=16, na=11
!....Arrays to keep the values of the table
      double precision, dimension(:), intent(in) :: thetaj(na)

!.....in/out values
       double precision, intent(in) :: angi   !angle to interpolate
       double precision, intent(in), dimension(:) :: vinput(na)
       double precision, intent(out) :: vint       !interpolated potential

!.....arrays to save the values sorted
       double precision, save, dimension(:) :: y2(na) 
       
       call spline_css(thetaj,vinput,na,1d30,1d30,y2)

!.....subtract atomic energies.
!       etable=etable+1.0d0

       call splint(thetaj,vinput,y2,na,angi,vint)

       return
       end subroutine spline1d1

!deck*1, spline
      subroutine spline_css(x,y,n,yp1,ypn,y2)
! the cubic spline interpolation routines copied from Numerical Recipes
! y_i = f(x_i), with x_1 < x_2 < x_3... < x_n, and given values yp1 and 
! ypn for the first derivative of the interpolating function at points
! 1 and n, respectively, this routine returns an array y2(1:n) of length
! n which contains the second derivatives of the interpolatinf function
! at the tabulated points x_i. If yp1 and/or ypn are equal to 1x10^30 or
! larger, the routine is signaled to set the corresponding boundary 
! condition for a natural spline, with zero second derivative on that
! boundary.
      implicit none
!     .. local parameters ..
!     .. Arguments ..
      integer, intent(in) :: n
      double precision, intent(in) :: yp1,ypn,x(n),y(n)
      double precision, intent(out) :: y2(n)
!     .. Local scalars ..
      integer :: i,k
      double precision :: p,qn,sig,un
!     .. Local Arrays ..
      double precision :: u(n)
!     .. Executable Statements ..
      if(yp1.gt.0.99e30) then
!.... the lower boundary condition is set either to be "natural"
         y2(1) = 0.0d0
         u(1) = 0.0d0
      else
!.... or else have a specified first derivative
         y2(1) = -0.5d0
         u(1) = (3.0d0/(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      endif
!.... This is the decomposition loop of the tridiagonal algorithm. 
!.... y2 and u are used for temporary storage of the decomposed factors.
      do i = 2,n-1
         sig = (x(i)-x(i-1))/(x(i+1)-x(i-1))
         p = sig*y2(i-1)+2.0d0
         y2(i) = (sig-1.0d0)/p
         u(i) = (6.0d0*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1))&
              &/(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
      end do
      if(ypn.gt.0.99e30) then
!.... The upper boundary condition is set either to be natural
        qn = 0.0d0
        un = 0.0d0
      else
!.... or else to have a specified first derivative
        qn = 0.5d0
        un = (3.0d0/(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      endif
      y2(n) = (un-qn*u(n-1))/(qn*y2(n-1)+1.0d0)
!.... This is the backsubstitution loop of the tridiagonal algorithm
      do k = n-1,1,-1
         y2(k) = y2(k)*y2(k+1)+u(k)
      end do
      end subroutine spline_css
!****
!deck*2,splint
      subroutine splint(xa,ya,y2a,n,x,y)
! Given the arrays xa(1:n) and ya(1:n), which tabulate a function 
! (with the xa_i's in order), and given the array y2a(1:n), 
! whcih is the output from spline, and given a vale of x, 
! this routine returns a cubic spline interploated value y
      implicit none
!     .. local parameters ..
!     .. Arguments ..
      integer, intent(in):: n
      double precision, intent(in) :: x,xa(n),ya(n),y2a(n)
      double precision, intent(out):: y
!     .. Local Scalars ..
      integer :: k,khi,klo
      double precision :: a,b,h
!.... Test to see if the point lies outside the given range
      if(x.lt.xa(1)) goto 10
      if(x.gt.xa(n)) goto 20
! We find the right place in the table by means of bisection. This is
! optimal if sequential calls to this routine are at random values of x.
! If sequential calls are in order, and closely spaced, 
! one would do better
! to store previous values of klo and khi and test if they remain 
! appropriate on the next call
      klo = 1
      khi = n
 1    if(khi-klo.gt.1) then
        k = (khi+klo)/2
        if(xa(k).gt.x) then
          khi = k
        else 
          klo = k
        end if
        goto 1
      end if
!.... klo and khi now bracket the input value of x
      h = xa(khi) - xa(klo)
!.... The xa's must be distinct
      if(h.eq.0.0d0) then
        write(6,*) 'xa(khi)= ',xa(khi) 
        write(6,*) 'xa(klo)= ',xa(klo) 
        write(6,*) 'Bad input in splint'
        stop
      end if  
!.... Cubic spline polynomial is now evaluated
      a = (xa(khi)-x)/h
      b = (x-xa(klo))/h
      y = a*ya(klo)+b*ya(khi)+& 
        &((a**3.0d0-a)*y2a(klo)+(b**3.0d0-b)*y2a(khi))*(h**2.0d0)/6.0d0
      return
!.... Tentative extrapolation for x less than given data
 10   a = xa(2)-xa(1)
      y = ((ya(2)-ya(1))/a-y2a(1)*a/6.0d0)*(x-xa(1))+ya(1)
      return
!.... Tentative extrapolation for x greater tahn given data
 20   a = xa(n) - xa(n-1)     
      y = ((ya(n)-ya(n-1))/a-y2a(n)*a/6.0d0)*(x-xa(n))+ya(n) 
      return
      end subroutine splint           

! ++++++++++++++++++++++++++++++++++++++++++++++++++
! splie2 subroutine from Numerical Recipes in Fortran 77: The Art of
! Science Computing (ISBN 0-521-43064-X). Cambridge University Press.
! Page. 121
! ++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine splie2(x1a,x2a,ya,m,n,y2a)
      implicit none
      integer, intent(in) :: m,n
      double precision, dimension(:),intent(in) :: x1a(m),x2a(n)
      double precision, dimension(:,:), intent(in) :: ya(m,n)
      double precision, dimension(:,:), intent(out) :: y2a(m,n)

      integer, parameter ::NN=1500

   !Given an m by n tabulated function ya(1:m,1:n), and tabulated
   !independent variables x2a(1:n), this routine constructs one-dimensional
   !natural cubic splines of the rows of ya and returns the second-derivatives
   !in the array y2a(1:m,1:n). (The array x1a is included in the argument
   !list merely for consistency with routine splin2)

      integer :: j,k
      double precision, dimension(NN) :: y2tmp,ytmp

      do j=1,m
         do k=1,n
            ytmp(k)=ya(j,k)
         enddo
         call spline_css(x2a,ytmp,n,1.d30,1.d30,y2tmp)
         do k=1,n
            y2a(j,k)=y2tmp(k)
         enddo
      enddo

      return
      end subroutine splie2



! ++++++++++++++++++++++++++++++++++++++++++++++++++
! splin2 subroutine from Numerical Recipes in Fortran 77: The Art of
! Science Computing (ISBN 0-521-43064-X). Cambridge University Press.
! Page. 121
! ++++++++++++++++++++++++++++++++++++++++++++++++++

      subroutine splin2(x1a,x2a,ya,y2a,m,n,x1,x2,y)
      implicit none
      integer :: m,n
      double precision, intent(in) :: x1,x2
      double precision, dimension(:), intent(in) ::x1a(m),x2a(n)
      double precision, dimension(m,n),intent(in) ::y2a,ya
      double precision, intent(out) :: y

      integer, parameter :: NN=1500

   !Given x1a,x2a,ya,m,n as described in splie2 and y2a as produced by
   !the routine; and given a desired interpolating point x1,x2; this 
   !routine returns an interpolated function value y by bicubic spline
   !interpolation

      integer j,k
      double precision, dimension(NN) :: y2tmp, ytmp, yytmp


      do j=1,m
         do k=1,n
            ytmp(k)=ya(j,k)
            y2tmp(k)=y2a(j,k)
         enddo
         call splint(x2a,ytmp,y2tmp,n,x2,yytmp(j))
      enddo
      call spline_css(x1a,yytmp,m,1.d30,1.d30,y2tmp)
      call splint(x1a,yytmp,y2tmp,m,x1,y)
 
      return
      end subroutine splin2

end module splinepotmod
