!#######################################################################
!     TRASRF    Automatic tranformation-of-SuRFace
!
!     Multi-dimensional surfaces defined here for coordinate
!     transformation
!
!     Do NOT put 1D functions here!
!     Compare with usersrf.F
!
!     ifile=208
!
!#######################################################################

      subroutine trasrf(hopilab,label,ifile,ierr)

use global
use pglobal
use channels
use decimal

      implicit none

      integer(long)                                     :: ilbl,jlbl,ll
      integer(long), intent(out)                        :: hopilab,ifile
      integer(long), intent(out)                        :: ierr
      character(len=*), intent(in)                      :: label

      hopilab=0
      ifile=0
      ierr=0

!-----------------------------------------------------------------------
!     Determine label length
!-----------------------------------------------------------------------
      ilbl=index(label,'{')-1
      jlbl=index(label,' ')-1
      if(ilbl.le.0) ilbl=32
      if(jlbl.le.0) jlbl=32
      ll = min(ilbl,jlbl)

!-----------------------------------------------------------------------
!     FOR EXAMPLES SEE defsrf AND surfinfo ON opfuncs/funcsrf.F
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!     TRASURF surface.
!-----------------------------------------------------------------------
      if (label(1:ll) .eq. 'mysrf') then
         hopilab=1
!
!-----------------------------------------------------------------------
!     end of if loop
!-----------------------------------------------------------------------
      endif

!-----------------------------------------------------------------------
!     if label is assigned, assign ifile number.
!-----------------------------------------------------------------------
      if (hopilab .ne. 0) ifile=208

      return
      end subroutine trasrf


!##########################################################################
!     TVPOINT   Transfo-V-POINT
!
!##########################################################################

      subroutine tvpoint (v,gpoint,ndof1)

use  global
use pglobal
use  channels

      implicit none

!--------------------------------------------------------------------------
!     variables for transformation
!--------------------------------------------------------------------------
      integer(long) ::  i,j,nbref
      integer(long), intent(in)               :: ndof1
      real(dop), dimension(ndof1), intent(in)  :: gpoint
      real(dop), intent(out)                  :: v

!--------------------------------------------------------------------------
!     For the case of frozen coordinates nbref/= ndof
!     Set the nbref here
!     Do not forget to define values for frozen coordinates
!--------------------------------------------------------------------------
      real(dop) ::  inref(nbfr+ndof1)
      real(dop) ::  cart_in((nbfr+ndof1)/3+2,3)
      nbref=nbfr+ndof1
      inref=0.0
      cart_in=0.0
!--------------------------------------------------------------------------
!     Set the frozen coordinates
!     nbfr is the number of frozen coordinates.
!     inref is a vector containing values of all polyspherical coordinates
!     (active and frozen).
!     Here the frozen coordinates are given in inref.
!--------------------------------------------------------------------------
      do i=1, nbfr
         j=frozen(i)
         inref(j)=pfroz(i)
      end do

!--------------------------------------------------------------------------
!     The active coordinates for transformation are assigned to inref here.
!--------------------------------------------------------------------------
      do i=1,ndof1
         j=tord(i)
         inref(j)=gpoint(i)
      end do
!--------------------------------------------------------------------------
!     The transformation is done here and you will have the
!     cartesian coordinates in cart_in.
!--------------------------------------------------------------------------
      call Transfo(inref,cart_in,nbref)

!--------------------------------------------------------------------------
!     copy or call your surface here!
!     Wish you a pleasant potfit

      v = 0.0
!      call pestra(cart_in,v)

      return
      end

!#######################################################################
      subroutine Transfo(inref,cart_in,nbref)

use global
use pglobal
use channels

      implicit none

      Integer(long), intent(in) :: nbref
      Integer(long) :: mid,j,d,p,i
      real(dop) :: cartesian(nbref,3)
      real(dop), intent(out) :: cart_in(nbref/3+2,3)
      real(dop), intent(in)  :: inref(nbref)
      real(dop) :: dpcart(nbref,3)
      real(dop) :: ref(nbref,3)


      mid =nbref/3 +1
!-------------------------------------------------------------------
!     Only for reduced z in H3O2- case
!     inref(9) = inref(9) * (inref(1) - 3.2)
!-------------------------------------------------------------------
      ref(:,:)=0
      ref(1,1)= inref(1)
      ref(1,2)= 1
      ref(2,1)= inref(2)
      ref(2,2)= inref(3)
      do i=1,mid-2
         do j=1,3
            ref(i+2,j)= inref(3*i+j)
         end do
      end do

      d=2
      p=1
      dpcart(1,:)=0
      do i=1,mid
         if ((at1(i).eq.'x').and.(i.ne.icart(1))) then
            dpcart(d,1)=-1*vecwei(i)*ref(i,1)*sqrt(1-ref(i,2)**2) &
             *cos(ref(i,3))+dpcart(nbdumm(i),1)
            dpcart(d,2)=-1*vecwei(i)*ref(i,1)*sqrt(1-ref(i,2)**2) &
            *sin(ref(i,3))+dpcart(nbdumm(i),2)
            dpcart(d,3)=-1*vecwei(i)*ref(i,1)*ref(i,2)+ &
            dpcart(nbdumm(i),3)
            d=d+1
         else if ((at1(i).ne.'x').and.(i.ne.icart(1))) then
            cartesian(p,1)=-1*vecwei(i)*ref(i,1)*sqrt(1-ref(i,2)**2) &
            *cos(ref(i,3))+dpcart(nbdumm(i),1)
            cartesian(p,2)=-1*vecwei(i)*ref(i,1)*sqrt(1-ref(i,2)**2) &
            *sin(ref(i,3))+dpcart(nbdumm(i),2)
            cartesian(p,3)=-1*vecwei(i)*ref(i,1)*ref(i,2)+ &
            dpcart(nbdumm(i),3)
            p=p+1
         else if ((at1(i).eq.'x').and.(i.eq.icart(1))) then
            dpcart(d,1)=-1*vecwei(i)*ref(i,1)+dpcart(nbdumm(i),1)
            dpcart(d,2)=-1*vecwei(i)*ref(i,2)+dpcart(nbdumm(i),2)
            dpcart(d,3)=-1*vecwei(i)*ref(i,3)+dpcart(nbdumm(i),3)
            d=d+1
         else if ((at1(i).ne.'x').and.(i.eq.icart(1))) then
            cartesian(p,1)=-1*vecwei(i)*ref(i,1)+dpcart(nbdumm(i),1)
            cartesian(p,2)=-1*vecwei(i)*ref(i,2)+dpcart(nbdumm(i),2)
            cartesian(p,3)=-1*vecwei(i)*ref(i,3)+dpcart(nbdumm(i),3)
            p=p+1
         end if
         if ((at2(i).eq.'x').and.(i.ne.icart(1))) then
            dpcart(d,1)=(1-vecwei(i))*ref(i,1)*sqrt(1-ref(i,2)**2) &
            *cos(ref(i,3))+dpcart(nbdumm(i),1)
            dpcart(d,2)=(1-vecwei(i))*ref(i,1)*sqrt(1-ref(i,2)**2) &
            *sin(ref(i,3))+dpcart(nbdumm(i),2)
            dpcart(d,3)=(1-vecwei(i))*ref(i,1)*ref(i,2)+ &
            dpcart(nbdumm(i),3)
            d=d+1
         else if ((at2(i).ne.'x').and.(i.ne.icart(1))) then
            cartesian(p,1)=(1-vecwei(i))*ref(i,1)*sqrt(1-ref(i,2)**2) &
            *cos(ref(i,3))+dpcart(nbdumm(i),1)
            cartesian(p,2)=(1-vecwei(i))*ref(i,1)*sqrt(1-ref(i,2)**2) &
            *sin(ref(i,3))+dpcart(nbdumm(i),2)
            cartesian(p,3)=(1-vecwei(i))*ref(i,1)*ref(i,2) &
            +dpcart(nbdumm(i),3)
            p=p+1
         else if ((at2(i).eq.'x').and.(i.eq.icart(1))) then
            dpcart(d,1)=(1-vecwei(i))*ref(i,1)+dpcart(nbdumm(i),1)
            dpcart(d,2)=(1-vecwei(i))*ref(i,2)+dpcart(nbdumm(i),2)
            dpcart(d,3)=(1-vecwei(i))*ref(i,3)+dpcart(nbdumm(i),3)
            d=d+1
         else if ((at2(i).ne.'x').and.(i.eq.icart(1))) then
            cartesian(p,1)=(1-vecwei(i))*ref(i,1)+dpcart(nbdumm(i),1)
            cartesian(p,2)=(1-vecwei(i))*ref(i,2)+dpcart(nbdumm(i),2)
            cartesian(p,3)=(1-vecwei(i))*ref(i,3)+dpcart(nbdumm(i),3)
            p=p+1
         end if
      end do
      do i=1,nbref/3+2
         cart_in(i,:)=cartesian(cord(i),:)
      end do

      end subroutine  Transfo

