!#######################################################################
!
!     POTFIT   module  COMPIWEIGHT
!
!     Contains subroutines for calculating the correlated weights:
!     
!     compiweight
!
!     Calculates 'iweight' array. If iweight(i)=1 the corresponding
!     grid point is assumed to be 'relevant'.
!
!
!#######################################################################

      subroutine compiweight(gpoint,v,weight)
!***********************************************************************
!
!   subroutine compiweight calculates the correlated weights
!     
!        gpoint (i)
!           vector of length ndof containing the coordinates of a
!           grid point
!        v (i/o)
!          the original potential (unweighted, no 1D potentials
!          subtracted)
!        weight (o)
!          weight=0: irrelevant grid point
!          weight=1: relevant gridpoint
!     
!***********************************************************************

use decimal, only:long,dop
use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined

      implicit none

      real(dop)                              :: sine,cosine
      real(dop), intent(in)                  :: v
      real(dop), dimension(ndof), intent(in) :: gpoint
      integer(long)                          :: f
      integer(long), intent(out)             :: weight

      weight=1
      if (.not.lcw) return

      if (v.gt.iwvmax .or. v.lt.iwvmin) then
         weight=0
         return
      endif

      do f=1,ndof
         if(gpoint(f).lt.xxmin(f) .or. gpoint(f).gt.xxmax(f)) then
            weight=0
            return
         endif
         if(sxmin(f).gt.-1.0_dop.or.sxmax(f).lt.1.0_dop) then
            sine=sin(gpoint(f))
            if (sine.lt.sxmin(f) .or. sine.gt.sxmax(f)) then
               weight=0
               return
            endif
         endif
         if(cxmin(f).gt.-1.0_dop.or.cxmax(f).lt.1.0_dop) then
            cosine=cos(gpoint(f))
            if (cosine.lt.cxmin(f) .or. cosine.gt.cxmax(f)) then
               weight=0
               return
            endif
         endif
       enddo

      return
      end subroutine compiweight
