!***********************************************************************
      module direct

use decimal, only: long,dop
use globmemmod
use logdat
use op1lib,  only: initvxi
use timing

      implicit none
      private
      public :: directfit
      contains

!***********************************************************************
!
!                        directfit
!
!
!
!***********************************************************************

      subroutine directfit(work2,coeff,mr,mi,vvec)

use global
use pglobal
use maxv, only:maxdim
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels
use rdnatpot

use wrfitmod, only:wrfit,vecsw,unweight_contr,vecout
use wrpgitter, only:writer,wrprodwei
use vecsmod, only:vecs
use getpesmod, only:vextime,getv1d,subtractv1d
use pdefaultmod, only:init_cmode
use fitinmod, only:fitin
use coeffs, only:vcoeffs,ctensor
use fiterrmod, only:fiterr
use pesfit, only:fittime
use iterationmod, only:iteration
use directutil, only:index2coord
use weightsmod, only:weights,getsepwei
use zeigpfitmod, only: setv2c


      implicit none

      real(dop), dimension(mrdim), intent(inout)     :: mr
      integer(long), dimension(midim), intent(inout) :: mi
      real(dop), dimension(mrvvec), intent(inout)    :: vvec
      real(dop), dimension(workdim2), intent(inout)  :: work2
      real(dop), dimension(coeffdim), intent(inout)  :: coeff

      integer(long) :: yit,nit,nitsteps,nsteps,cw,f,offset,i,n,m,&
                       chkdvr,chkgrd,ndof1,nmode1,old_modc
      integer(long) :: woffset,ccw,k,old_coeffdim
      integer(long), dimension(maxdim) :: igpoint,pntbas,mmode,fdvr,mdvr,subdim1
      real(dop) :: vsum2,vsum2w,iterr1,iterr2,sw,x,y,v,v1d,cpu,sumnw
      real(dop), dimension(maxdim)           :: olddim
      character(len=80)                      :: actual
      logical(kind=4)                        :: lerr
      logical(kind=4), dimension(maxdim)     :: loned
      real(dop), external                    :: fcputime,fsystime
      real(dop)                              :: tcpu,ttot,tsys,tcpu0,tsys0
      integer(long)                          :: t1,t2,icount,count_rate
      character(len=240)                     :: filename
 
      integer(long), save :: nextday=0, oldhour=0
      integer(long), save                    :: tid=0 ! timer ID

      if(lspenergy.or.lcutalong)return

      call get_timer(tid, 'Firstfit')
      call start_timer(tid)

      old_coeffdim=1
      do m=1,nmode
         old_coeffdim=old_coeffdim*potdim(m)
      enddo

!-----------------------------------------------------------------------
!     determine number of grid points niw with non-vanishing correlated
!     weight
!-----------------------------------------------------------------------
      niw=vpotdim

      if (lcw) then
         offset=1
         woffset=miwvec
         k=30
         call initvxi(igpoint,ndof)
 10      v=vvec(offset)
         offset=offset+1
         if(k.eq.30) then
            ccw=mi(woffset)
            woffset=woffset+1
            k=0
         endif
         k=k+1
         cw=ccw             ! de-compress compressed weights
         ccw=ccw/2          ! (ccw=compressed corr. weight)
         cw=cw-2*ccw        ! (cw =correlated weight)
         niw=niw-1+cw
         do m=1,npmode
            do n=1,npotdof(m)
               f=potdof(n,m)
               if (igpoint(f).lt.gdim(f)) then
                  igpoint(f)=igpoint(f)+1
                  goto 10
               else
                  igpoint(f)=1
               endif
            enddo
         enddo
      endif
       
      write(iout,'(2a,/)') '**********',&
       '*************************************************************'

      write(iout,'(a,i9,4x,1pe9.2e2)')&
        'Number of  all     grid points    :', vpotdim, dble(vpotdim)
      if (lcw) then
         write(iout,'(a,i9,4x,1pe9.2e2)')&
           'Number of relevant grid points    :', niw, dble(niw)
         write(iout,'(a,2pf9.3,a)') &
           'Number of Relevant/All grid points:',&
            dble(niw)/dble(vpotdim),' %'
      endif
      call myflush(iout)

      if (niw.eq.0) then
         routine='directfit'
         message='no relevant grid points'
         call errormsg
      endif

      if(lcutnpot) goto 500

!-----------------------------------------------------------------------
!     handle separable weights and subtract 1d potentials
!-----------------------------------------------------------------------
      if (lsw) then
         
!-----------------------------------------------------------------------
!        get separable weights and their inverse 
!-----------------------------------------------------------------------
         call weights(mr(mrwght),mr(mrwghti),mr(mrort))

!-----------------------------------------------------------------------
!        write separable weights to channel 'iout' and to file 'prodwei'
!-----------------------------------------------------------------------
         call wrprodwei(mr(mrort),mr(mrwght))

!-----------------------------------------------------------------------
!        calculate several sums over the complete product grid
!            vsum2: sum over (v-v1d)**2
!           vsum2w: sum over ((v-v1d)*sw)**2
!            wsum2: sum over (sw*cw)**2
!           swsum2: sum over sw**2  (reduction factor)
!        wsum2 swsum2 are needed later on.
!-----------------------------------------------------------------------
         vsum2=0.0_dop
         vsum2w=0.0_dop
         wsum2=0.0_dop
         swsum2=0.0_dop
         offset=1
         woffset=miwvec
         k=30
         call initvxi(igpoint,ndof)
 20      v=vvec(offset)
         call getsepwei(igpoint,mr(mrwght),sw)
         call getv1d(igpoint,mr(mroned),v1d)
         x=(v-v1d)
         y=x*sw
         vsum2=vsum2+x**2
         vsum2w=vsum2w+y**2
         if(lcw) then
            if(k.eq.30) then
               ccw=mi(woffset)
               woffset=woffset+1
               k=0
            endif
            k=k+1
            cw=ccw           ! de-compress compressed weights
            ccw=ccw/2        ! (ccw=compressed corr. weight)
            cw=cw-2*ccw      ! (cw =correlated weight)
            if (cw.gt.0) wsum2=wsum2+sw**2
         endif
         swsum2=swsum2+sw**2
         vvec(offset)=y
         offset=offset+1
         do m=1,npmode
            do n=1,npotdof(m)
               f=potdof(n,m)
            if (igpoint(f).lt.gdim(f)) then
               igpoint(f)=igpoint(f)+1
               goto 20
            else
               igpoint(f)=1
            endif
            enddo
         enddo

         write (iout,*)
         write (iout,'(a,1pe21.12)')&
              'Sum2 of unweighted potential       : ',vsum2
         write (iout,'(a,1pe21.12)')&
              'Sum2 of weighted potential (sw)    : ',vsum2w
         if (lcw) write (iout,'(a,1pe21.12)')&
              'Sum2 of weights (sw*cw)            : ',wsum2
         write (iout,'(a,1pe21.12)')&
              'Sum2 of separ. weights sw          : ',swsum2
         write (iout,*)

!-----------------------------------------------------------------------
!     No separable weights, handle 1d potentials
!-----------------------------------------------------------------------
      else
         if (ploned) call subtractv1d(vvec,mr(mroned))
         vsum2=0.0_dop
         wsum2=niw
         swsum2=vpotdim
         do i=1,vpotdim
            vsum2=vsum2+vvec(i)**2
         enddo
         write (iout,*)
         write (iout,'(a,1pe21.12)')&
              'Sum2 of unweighted potential       : ',vsum2
         write (iout,*)

      endif
      call myflush(iout)
   
!-----------------------------------------------------------------------
! cutnpot
!-----------------------------------------------------------------------

  500 if(lcutnpot)then

         if(lnpotpath)then
            filename=npotname(1:npotlaenge)
            open(inpot,form='unformatted',status='old',err=900, &
            file=filename)
         else
            filename=name(1:laenge)//'/natpot'
            open(inpot,form='unformatted',status='old',err=900, &
            file=filename)
         end if

         chkdvr=2  ! dvrinfo checked against system data
         chkgrd=1  ! grdinfo read, and stored in system data

         call npotinfo(inpot,lerr,modc,loned,ndof1,fdvr,chkdvr, &
              chkgrd,1,mdvr,mmode,nspfdof,spfdof,nmode1,subdim1,pntbas)
         pdim(modc)=psubdim(modc)

         write(ilog,'(/a,i2)') 'Number of degrees of freedom = ', nmode
         write(ilog,'(a,i2,2x,a)') 'Contracted mode = ', modc, &
                                    trim(modelabel(modc))
         write(ilog,'(a,2i10)')  'No. of configurations (old/new):', &
                                  modcdim, old_coeffdim/psubdim(modc)
         write(ilog,'(a,2i10/)') 'Size of coeff-tensor  (old/new):', &
                                  coeffdim, old_coeffdim
         write(ilog,'(a,99i6)')'Old no. of SPPs:',(pdim(m),m=1,nmode)
         write(ilog,'(a,99i6)')'New no. of SPPs:',(potdim(m),m=1,nmode)

         call myflush(ilog)

! NOTE: fitin requires potdim and does not use potdim(modc)
         do i=1,nmode
           olddim(i) = potdim(i)
           potdim(i) = pdim(i)
         enddo
         call fitin(inpot,loned,mr(mroned),mr(mrevec),coeff)
         close(inpot)

! cutnpot: return to old potdim values (requested)
         do i=1,nmode
           potdim(i) = olddim(i)
         enddo

! cutnpot: create array where the final values will be allocated
         call wrfit(coeff,mr(mrevec),mr(mroned),mr(mrwght),mr(mrwghti))

!-----------------------------------------------------------------------
! Get overall timing and write data to "log" file
!-----------------------------------------------------------------------
         call cpu_time(tcpu)
         call system_clock(icount,count_rate)
         tsys = dble(icount)/dble(count_rate)
         tcpu = tcpu-tcpu0
         tsys = tsys-tsys0
         ttot = tsys
         t1   = int(ttot/3600.0_dop)
         ttot = ttot - dble(3600*t1)
         t2   = int(ttot/60.0_dop)
         ttot = ttot - dble(60*t2)

         write(ilog,*)
         write(ilog,'(a)') '***** Overall Timing *****'
         write(ilog,*)
         write(ilog,'(a,1x,f12.2)') 'Total cpu time     [s] : ',tcpu
         write(ilog,'(a,1x,f12.2)') 'Total system time  [s] : ',tsys
         write(ilog, 800) 'Total time      [h:m:s] : ',t1,t2,ttot
  800    format( a,i4,' :',i3,' :',f6.2)
         write(ilog,*)

         write(iout,'(/a)')'REDUCED natpot file was generated'
         write(ilog,'(a/)') 'Program terminated normally.'
         stop

      endif ! if(lcutnpot)

!----------------------------------------------------------
! END cutnpot
!----------------------------------------------------------

!-----------------------------------------------------------------------
! If rdnpot 
!-----------------------------------------------------------------------
      if(lrdnpot) then

         write(iout,'(/a)')'********************************'
         write(iout,'(/a)')'Reading of NATPOT file requested'
         write(iout,'(/a)')'********************************'
         write(iout,*)

         write(ilog,'(/a)')'********************************'
         write(ilog,'(/a)')'Reading of NATPOT file requested'
         write(ilog,'(/a)')'********************************'
         write(ilog,*)

         filename=name(1:laenge)//'/natpot'
         open(inpot,form='unformatted',status='old',err=900,file=filename)

         chkdvr=2  ! dvrinfo checked against system data
         chkgrd=1  ! grdinfo read, and stored in system dat
         call npotinfo(inpot,lerr,modc,loned,ndof1,fdvr,chkdvr, &
              chkgrd,1,mdvr,mmode,nspfdof,spfdof,nmode1,subdim1,pntbas)

         pdim(modc)=psubdim(modc)

!---------------------------------------------------------
! Print information from the natpot file (check purposes)
!    modc,loned,ndof1,fdvr,chkdvr,nmode1,subdim1
!---------------------------------------------------------

         write(iout,'(a,i2)') 'Number of degrees of freedom = ', nmode
         write(iout,'(a,i2,2x,a)') 'Contracted mode = ', modc, &
                trim(modelabel(modc))
         if(lpconm)then
           write(iout,'(2x,a)') 'Combined Modes'
           do i=1, nmode
             write(iout,'(2x,a,i2,a,i2,a,99(1x,i2))') &
                  'Particle= ', i,' consists of= ', nspfdof(i), ' modes'
             write(iout,'(2x,a,99(1x,i2))') 'Mode numbers = ', &
                  (spfdof(m,i), m=1, nspfdof(i))
             write(iout,'(2x,2a,99(2x,a))') 'Mode labels = ', &
                  (trim(modelabel(spfdof(m,i))), m=1, nspfdof(i))
           end do
         else
           write(iout,'(2a,99(2x,a))') 'Modes = ', &
                (trim(modelabel(m)),m=1,nmode)
         end if

         write(iout,'(/a,2i10)') 'No. of configurations (old/new):', &
                                  modcdim, old_coeffdim/psubdim(modc)
         write(iout,'(a,2i10/)') 'Size of coeff-tensor  (old/new):', &
                                  coeffdim, old_coeffdim
         write(iout,'(a,99i6)')'Old no. of SPPs:',(pdim(m),m=1,nmode)
         write(iout,'(a,99i6)')'New no. of SPPs:',(potdim(m),m=1,nmode)
         write(iout,*)

!---------------------------------------------------------
! END of Print information from the natpot file
!---------------------------------------------------------

! NOTE: fitin requires potdim and does not use potdim(modc)
         pdim(modc)=psubdim(modc)
         do m=1,nmode
            olddim(m) = potdim(m)
            potdim(m) = pdim(m)
         enddo

         call fitin(inpot,loned,mr(mroned),mr(mrevec),coeff)
         if (lsw) then
            call vecsw(mr(mrevec),mr(mrwght))
            call unweight_contr(coeff,mr(mrwght),dimbef,dimmodc,dimaft, &
                    npotdof(modc),potdof(1,modc),gdim,maxdim, &
                    zweight,weightdim,ctdim,iwei)
         endif
         close(inpot)

! revert potdim to original values
         do m=1,nmode
            potdim(m) = olddim(m)
         enddo

         goto 50
      endif ! if(lrdnpot)

!-----------------------------------------------------------------------
! If CONTINUATION-RUN read vfit and go to fiterr
!-----------------------------------------------------------------------
      if(lcont) then
         old_modc=modc
         filename=name(1:laenge)//'/natpot'
         open(inpot,form='unformatted',status='old',err=900,file=filename)
         chkdvr=2 ! dvrinfo checked against system data
         chkgrd=1 ! grdinfo read, and stored in system data
         call npotinfo(inpot,lerr,modc,loned,ndof1,fdvr,chkdvr, &
              chkgrd,1,mdvr,mmode,nspfdof,spfdof,nmode1,subdim1,pntbas)

         if(old_modc.ne.modc) then
            write(6,   '(/a)') 'Inconsistent contracted modes!'
            write(ilog,'(/a)') 'Inconsistent contracted modes!'
            write(6,   '(a,i4,a,i4)') ' input-modc =', old_modc, &
                                '      natpot-modc =', modc
            write(ilog,'(a,i4,a,i4)') ' input-modc =', old_modc, &
                                '      natpot-modc =', modc
            call stopnow
         endif

         do m=1,nmode
            if(m.ne.modc .and. pdim(m).ne.potdim(m)) then
               write(ilog,'(/a)') 'input and natpot potdims differ.'
               write(ilog,'(12i6)') (pdim(i),i=1,nmode)
               write(ilog,'(12i6)') (potdim(i),i=1,nmode)
               write(6,   '(/a)') 'input and natpot potdims differ.'
               write(6,'(a,9i6)') 'potdim read from natpot:', &
                                  (pdim(i),i=1,nmode)
               write(6,'(a,9i6)') 'potdim read from  input:', &
                                  (potdim(i),i=1,nmode)
               write(6,   '(a,i4)') &
               'Ignore the numbers for the contracted mode:',modc
               call stopnow
            endif
         enddo
         pdim(modc)=psubdim(modc)
         do m=1,nmode
            potdim(m)=pdim(m)
         enddo
         call fitin(inpot,loned,mr(mroned),mr(mrevec),coeff)
         close(inpot)
         if (lsw) then
            call vecsw(mr(mrevec),mr(mrwght))
            call unweight_contr(coeff,mr(mrwght),dimbef,dimmodc,dimaft, &
                    npotdof(modc),potdof(1,modc),gdim,maxdim, &
                    zweight,weightdim,ctdim,iwei)
         endif
         goto 49
      endif !  if(cont)

!-----------------------------------------------------------------------
!  NORMAL PROCEDURE
!-----------------------------------------------------------------------
! Calculate ALL natural potentials
!     
! if separable weights have (not) been defined the calculation is based
! on the (un)weighted potential giving (un)weighted natural potentials.
! in any case, the calculation refers to the potential with 1d potentials
! (if any) subtracted.
!-----------------------------------------------------------------------

      call fzeit(actual)
      i = index(actual,':')
      actual=actual(i-9:i+6)
      cpu=fcputime(0)+fsystime(0)
      write(ilog,'(a,9x,a16,4x,"CPU =",f10.1)') &
          ' Computing the SPPs.',actual,cpu
      call myflush(ilog)

      call vecs(vvec,mr(mrevec),mr(mrwork1),mr(mreval),work2)

      sumnw=0.d0
      do m=1,npmode
         sumnw = sumnw + relnw(m)
      enddo
      sumnw=sqrt(sumnw)
      write(iout,'(/2a)') ' Global (weighted) L^2 error estimated', &
       ' from neglected natural weights:'
      if( sumnw .gt. 0.999d0 ) then
         write(iout,'(1x,f14.4,a,1pe16.4,a,1pe16.4,a/)') &
         sumnw,' eV', sumnw*8065.54353d0, 'cm^-1', &
         sumnw/ev,' a.u.'
      else
         write(iout,'(1x,f14.4,a,f16.3,a,1pe16.4,a/)') &
         sumnw*1.d3,' meV', sumnw*8065.54353d0, 'cm^-1', &
         sumnw/ev,' a.u.'
      endif

!-----------------------------------------------------------------------
! MGPF: if ldcoeff=false do not compute D coefficients
!-----------------------------------------------------------------------
      if(.not.ldcoeff)return

!-----------------------------------------------------------------------
! Calculate D coefficients D_{j_1...i_modc....j_f} contr. over mode 'modc'
! 
! non sep weights:
!      overlap of unweighted natural potentials and unweighted surface
! product weights:
!      overlap of weighted natural potentials and weighted surface
!-----------------------------------------------------------------------
      call fzeit(actual)
      i = index(actual,':')
      actual=actual(i-9:i+6)
      cpu=fcputime(0)+fsystime(0)
      write(ilog,'(a,5x,a16,4x,"CPU =",f10.1)') &
          ' Computing the D-tensor.',actual,cpu

      call vcoeffs(vvec,mr(mrevec),coeff,work2)

!-----------------------------------------------------------------------
! Compute C tensor as C_J = D_{i_nu J^nu} v_{i_nu j}
!-----------------------------------------------------------------------

      if(lctensor) call ctensor(mr(mrevec),coeff)

!-----------------------------------------------------------------------
! MGPF: if lonlydcoeff write D tensor to the fdcoeff file
!-----------------------------------------------------------------------
      if(lonlydcoeff)then
         write(filename,'(2a)') name(1:laenge),'/fdcoeff'
         open(ioutbin,file=filename,form='unformatted',status='unknown')
         call vecout(coeff,dimbef,dimmodc,dimaft,ioutbin)
         close(ioutbin)
         return
      end if

!-----------------------------------------------------------------------
! Get different fiterrors
!-----------------------------------------------------------------------

  49  if(.not.lfiterr)then
         write(iout,'(/a)') ' Computation of errors disabled (nofiterr).'
         write(ilog,'(/a)') ' Computation of errors disabled (nofiterr).'
         goto 51
      end if

 50   call fzeit(actual)
      i = index(actual,':')
      actual=actual(i-9:i+6)
      cpu=fcputime(0)+fsystime(0)
      write(ilog,'(a,4x,a16,4x,"CPU =",f10.1)') &
          ' Computing the fit-error.',actual,cpu
      write(ilog,'(76a1)')  ('-',i=1,76)
      call myflush(ilog)

      call fiterr(mr(mrort),mr(mrwght),vvec,&
           mr(mroned),mr(mrevec),coeff,mr(mrcut),&
           work2,mr(mrbuf),mi(miwvec))

      if(lrdnpot) return  ! rdnpot computes the fit-error only.

  51  continue

!-----------------------------------------------------------------------
! Write fit if "write_every_fit" is set.
!-----------------------------------------------------------------------
      if (lwef .and. .not.lcont) then
         call wrfit(coeff,mr(mrevec),mr(mroned),&
                    mr(mrwght),mr(mrwghti))
      endif

!-----------------------------------------------------------------------
! Write error data to 'iit' channel
!-----------------------------------------------------------------------
      if (litf) then
         call writer(0,iterr2) 
      endif

!-----------------------------------------------------------------------
! stop timing initialization
!-----------------------------------------------------------------------
      call stop_timer(tid)

!-----------------------------------------------------------------------
! optional timing of potential energy and fit calculation
!-----------------------------------------------------------------------      
      if (npest.gt.0) then
         call vextime(mr(mrort))
         call fittime(coeff,mr(mrevec),work2)
      endif

      if(.not.(lit.or.litia)) goto 999   !  No Iteration

!=======================================================================
!                     ITERATION CYCLE      
!
! Calculate iteratively improved natural potential fit.
!=======================================================================
      nit=0
      iterr1 = 1.0e+33_dop
      iterr2 = iterr1
      call timout
      if( .not.lcw .and. (lit.or.litia) ) then
         lit=.false.
         litia=.false.
         write(6,'(/a/a)')   'There is no correlated-weights-section.',&
                             'Hence skipping iterations.'
         write(ilog,'(/a/a)')'There is no correlated-weights-section.',&
                             'Hence skipping iterations.'
      endif
 
      if (lit) then
         call fzeit(actual)
         i = index(actual,':')
         actual=actual(i-9:i+6)
         cpu=fcputime(0)+fsystime(0)
         write (iout,'(2a,/)') '******************       ', &
          'START OF ITERATION CYCLE      ******************'
         write(ilog,'(/,76a1)')  ('-',i=1,76)
         write(ilog,'(a,3x,a16,4x,"CPU =",f10.1)')&
          ' Starting Iteration Cycle.',actual,cpu
         write(ilog,'(76a1)')  ('-',i=1,76)
      endif
      call myflush(iout)
      call myflush(ilog)


 777  continue   
      if (lit) then
         nitsteps=niters
         if (litia) then
            write (6,*) 'Start/continue iteration [yes=1,no=0] ?'
            read (5,*) yit
            if (yit.eq.1) then
               write (6,*) 'Number of iterations: '
               read(5,*) nitsteps
               write (6,*)
            else
               goto 999
            endif
         endif
         nsteps=0

 888     iterr1=iterr2
         nit = nit+1
         nsteps=nsteps+1
         call iteration(mr(mrort),vvec,mr(mroned),&
              coeff,mr(mreval),mr(mrevec),mr(mrwght),mi(miwvec),&
              mr(mrcut),mr(mrwork1),work2,iterr2,mr(mrbuf),nit)

         call fzeit(actual)
         i = index(actual,':')
         actual=actual(i-9:i+6)
         cpu=fcputime(0)+fsystime(0)
         write(ilog,'(1x,i5,a,3x,a16,4x,"CPU =",f10.1)')&
          nit,'   iteration cycle finished.',actual,cpu
         write (iout,'(a,i5,a,/)') '*************',nit,&
           '   iteration cycle(s) finished  *************'

!-----------------------------------------------------------------------
! Write fit if "write_every_fit" is set.
!-----------------------------------------------------------------------
         if (lwef) then
            call wrfit(coeff,mr(mrevec),mr(mroned),mr(mrwght),mr(mrwghti))
         endif

!-----------------------------------------------------------------------
! Write error data to file 'iteration'
!-----------------------------------------------------------------------
         if (litf) call writer(nit,iterr2)

!-----------------------------------------------------------------------
! 
!-----------------------------------------------------------------------
         if (iterr2.gt.iterr1) then
            write (iout,*) 'Iterated error did NOT decrease !!!'
            write (iout,*)
         endif
         call timout
         call myflush(iout)
         call myflush(ilog)

         open(istop,file=name(1:laenge)//'/stop',&
              form='formatted',status='old',err=666)
         read(istop,'(a)') stopcommand
!........Go to the end of the stop-file and write the number of
!........iterations to it in order to 'touch' the stop-file.
         do i = 1, 13
             read(istop,'(a)')
         end do
         write(istop,'(i5)') nsteps
         close(istop)
         goto 667
! --- If the stop-file cannot be opened stop the calculation ---
 666     stopcommand = 'stop      Cannot open stop-file'
 667     if( stopcommand(1:4) .eq. 'cont' ) then
            nextday = 0
            oldhour = 0
         else if( stopcommand(1:4) .eq. 'stop' ) then
            lstop = .true.
         else if( stopcommand(3:3).eq.':' .or.&
                  stopcommand(10:10).eq.':' ) then
            call pzeitstop
         end if
         if (lstop)  goto 999
            
         if (nsteps.lt.nitsteps) then
            goto 888
         else     
            if (litia) then
               write (iout,*) nsteps,'  iteration steps performed'
               write (iout,*)
               goto 777
            endif
         endif
      endif

!-----------------------------------------------------------------------
! write fitdata to file 'natpot'
!-----------------------------------------------------------------------
 999  lwef=.false.
      call wrfit(coeff,mr(mrevec),mr(mroned),mr(mrwght),mr(mrwghti))

      return

 900  routine='directfit'
      message='Cannot open file '//filename
      call errormsg

      end subroutine directfit

!######################################################################
! *** 06/97  HDM ********> SUBROUTINE PZEITSTOP <**********************
!                         ~~~~~~~~~~~~~~~~~~~~~

! Compares the time read from the stopfile with actual time
! Calls fzeit which is defined in showtime.c (lib).
! This is a copy of zeitstop (propwf/prop.F)

      subroutine pzeitstop

use global
use pglobal

      implicit none

      integer(long)           :: shour,smin,ssec,stime,ahour,amin,atime,&
                                 i,ierr1,ierr2,ierr3,nextday,oldhour
      real(dop), external     :: fcputime
      external                :: fzeit
      character(len=80)       :: actual

!.....common block to ensure that the variables are saved.
!.....This common block also appears in subroutine propagatepsi.
      common /pstopfile/ nextday, oldhour
      save /pstopfile/

      if( stopcommand(1:5) .eq. '     ' ) goto  200

!.....Realtime stop condition
      read(stopcommand(1:2),*,iostat=ierr1) shour
      read(stopcommand(4:5),*,iostat=ierr2) smin
      if( ierr1 .ne. 0  .or.  ierr2 .ne. 0 )  then
         routine = 'pzeitstop (potfit/direct.F)'
         message = 'Can''t read stopcommand. Not a proper time-descriptor'
         call warnmsg
         return
      end if
      stime = 60*shour + smin

      call fzeit(actual)
      i = index(actual,':')
      read(actual(i-2:i-1),*) ahour
      read(actual(i+1:i+2),*) amin
      if( ahour .lt. oldhour ) nextday = nextday + 24
      oldhour = ahour
      atime =  60*(ahour+nextday) + amin

      if( atime .ge. stime ) then
         lstop = .true.
         return
      end if

!.....CPU-time stop condition
 200  continue
      if( stopcommand(10:10) .ne. ':' ) return
      read(stopcommand(8:9),*,iostat=ierr1) shour
      read(stopcommand(11:12),*,iostat=ierr2) smin
      read(stopcommand(14:15),*,iostat=ierr3) ssec
      if(ierr1.ne.0 .or. ierr2.ne.0 .or. ierr3.ne.0)  then
         routine = 'pzeitstop (potfit/direct.F)'
         message =&
       'Can''t read stopcommand. Not a proper CPU time-descriptor'
         call warnmsg
         return
      end if

      stime = 3600*shour + 60*smin + ssec
      atime = int( fcputime(0) + 0.5_dop )
      if( atime .ge. stime )  lstop = .true.

      return
      end subroutine pzeitstop


      end module direct

