!#######################################################################
      module fiterrmod

use decimal, only:long,dop,sip
use op1lib,  only: initvxi, cuttxd2
use timing

      implicit none
      private
      public :: fiterr
      contains

!#######################################################################
!
! calculates several error measures 
!
!     ort (i)
!     weight (i)
!     vvec (i/o)
!     oned (i)
!     evec (i)
!     coeff (i)
!     work2 (w)
!        used for the calculation of the fit
!     
!#######################################################################

      subroutine fiterr(ort,weight,vvec,oned,evec,&
                        coeff,cut,work2,buffer,wvec)

use global
use pglobal
use maxv, only:maxdim
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels

use getpesmod, only:getv1d,loadv
use pesfit, only:getvfit2
use weightsmod, only:getsepwei
use directutil, only:index2coord

      implicit none

      real(dop), dimension(ortdim), intent(in)    :: ort
      real(dop), dimension(weightdim), intent(in) :: weight
      real(dop), dimension(oneddim), intent(in)   :: oned
      real(dop), dimension(evecdim), intent(out)  :: evec
      real(dop), dimension(coeffdim), intent(in)  :: coeff
      real(dop), dimension(workdim2), intent(out) :: work2
      real(dop), dimension(modcdim), intent(out)  :: cut
      real(dop), dimension(vpotdim), intent(out)  :: vvec
      real(dop), dimension(niobuf), intent(out)   :: buffer
      integer(long), dimension(vpotdim/30+1), intent(out)  :: wvec

      integer(long) :: ccw,cw,k,f,voffs,woffs,n,m,imgp_modc
      integer(long), dimension(maxdim)       :: igpoint
      real(dop), dimension(maxdim)           :: gpoint
      real(dop) :: v,vfit,v1d,sw,d1dsw,d1d,d,d2,da,d2sum,d2cwsum,damax,&
                   dacwmax,d2cwswsum,wrms,d2swsum,d2wsum
      integer(long), save                         :: tid=0 ! timer ID

!     start timer
      call get_timer(tid, 'Fiterr')
      call start_timer(tid)

!     load potential without 1d potentials and no separable weights
      call loadv(ivpot,vvec,buffer)

      d2sum=0.0_dop
      d2cwsum=0.0_dop
      d2wsum=0.0_dop
      d2cwswsum=0.0_dop
      damax=0.0_dop
      dacwmax=0.0_dop
      voffs=1
      woffs=1
      cw=0
      call initvxi(igpoint,ndof)

!     calculate cut
 5    imgp_modc=0
      do n=npotdof(modc),1,-1
         f=potdof(n,modc)
         imgp_modc=imgp_modc*gdim(f)+igpoint(f)-1
      enddo
      imgp_modc=imgp_modc+1
      call cuttxd2(coeff,cut,dimbef,dimmodc,dimaft,imgp_modc)

!     calculate offset from index vector
 10   voffs=0
      do m=npmode,1,-1
         do n=npotdof(m),1,-1
            f=potdof(n,m)
            voffs=voffs*gdim(f)+igpoint(f)-1
         enddo
      enddo
      woffs=voffs/30
      k=voffs-30*woffs
      voffs=voffs+1
      woffs=woffs+1

!     calculate errors
      call index2coord(igpoint,ort,gpoint)
      call getv1d(igpoint,oned,v1d)
      v=vvec(voffs)
      call getvfit2(igpoint,evec,cut,work2,vfit)
      call getsepwei(igpoint,weight,sw)
      if(lcw) then
         ccw=wvec(woffs)
         cw=ccw/2**k   ! de-compress compressed weights
         cw=mod(cw,2)
      endif
      d1d=v-v1d
      d1dsw=d1d*sw
      d=d1d-vfit/sw
      d2=d**2
      da=abs(d)
      d2sum=d2sum+d2
      d2swsum=d2swsum+(d1dsw-vfit)**2
      damax=max(damax,da)
      if (cw.gt.0) then
         d2cwsum=d2cwsum+d2
         d2cwswsum=d2cwswsum+(d1dsw-vfit)**2
         dacwmax=max(dacwmax,da)
      endif
      if (cw.gt.0) then
         vvec(voffs)=d1dsw
      else
         vvec(voffs)=vfit
      endif

!     increase index vector components
!     for the dofs that do not belong to the contracted mode
      do m=1,npmode
         if (m.ne.modc) then
            do n=1,npotdof(m)
               f=potdof(n,m)
               if (igpoint(f).lt.gdim(f)) then
                  igpoint(f)=igpoint(f)+1
                  goto 10
               else
                  igpoint(f)=1
               endif
            enddo
         endif
      enddo
!     increase index vector components
!     for the dofs of the contracted mode, and update the cut
      do n=1,npotdof(modc)
         f=potdof(n,modc)
         if (igpoint(f).lt.gdim(f)) then
            igpoint(f)=igpoint(f)+1
            goto 5
         else
            igpoint(f)=1
         endif
      enddo
      
      urms=sqrt(d2cwsum/niw)
      allrms=sqrt(d2sum/vpotdim)
      d2swsum=sqrt(d2swsum/swsum2)

      if (lsw .and. lcw) then
           wrms=sqrt(d2cwswsum/wsum2)
           write (iout,9) &
           ' Weighted   rms-error on rel. grid points [meV]: ',&
             wrms*eV*1000.0_dop, wrms, ' au'
      endif

      if(lsw) write (iout,9) &
        ' Weighted   rms-error on all  grid points [meV]: ', &
             d2swsum*eV*1000.d0, d2swsum, ' au'

      if(lcw) write (iout,9) &
           ' Unweighted rms-error on rel. grid points [meV]: ',&
             urms*eV*1000.0_dop, urms, ' au'
      write (iout,9) &
           ' Unweighted rms-error on all  grid points [meV]: ',&
             allrms*eV*1000.0_dop, allrms,' au'
      if(dacwmax*eV .ge. 1.0_dop ) then 
         if (lcw) write (iout,9)&
           ' Max.  absolute error on rel. grid points [ eV]: ',&
             dacwmax*eV, dacwmax, ' au'
      else
         if (lcw) write (iout,9)&
           ' Max.  absolute error on rel. grid points [meV]: ',&
             dacwmax*eV*1000.0_dop, dacwmax, ' au'
      end if
      if(damax*eV .ge. 1.0_dop ) then 
         write (iout,9)&
           ' Max.  absolute error on all  grid points [ eV]: ',&
             damax*eV, damax, ' au'
      else
         write (iout,9)&
           ' Max.  absolute error on all  grid points [meV]: ',&
             damax*eV*1000.0_dop, damax, ' au'
      end if
      write (iout,*)
      maxerr = dacwmax
      maxall = damax

      if (lsw) then
         rmserr = wrms
      else
         rmserr = urms
      endif

!     stop timer
      call stop_timer(tid)
      call myflush(iout)

 9    format(a,f10.4,1x,e15.4,a)
      return
      end subroutine fiterr

      end module fiterrmod

