!#######################################################################
      module fitinmod

use decimal, only:long,dop
use logdat

      implicit none
      private
      public :: fitin,vecin
      contains

!#######################################################################
!
!              POTFIT  module FITIN
!
!  Used by showpot and chnpot 
!
!  reads the (optional) one dimensional potentials, the natural potentials,
!  and the coefficients from a "natpot" file connected to i/o channel "iounit".
!  we assume that the natpot file was written by subroutine "wrfit" and that
!  the fit parameters have already been read from this file via subroutine
!  "npotinfo".
!
!  actually, we read natural potentials for all modes except the contracted
!  mode "modc". for the latter wie read the contracted coefficients instead.
!  note that the current natpot file does not contain the complete set of
!  natural potentials.
!     
!  iounit  (i)
!  loned   (i)
!  oned    (o)
!  coeff   (o)
!  evec    (o)
!
!  1/00 sw
!
!
!#######################################################################

      subroutine fitin(iounit,loned,oned,evec,coeff)

use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined


      implicit none

      integer(long)                               :: f,m,ierr,g
      integer(long), intent(in)                   :: iounit
      real(dop), dimension(oneddim), intent(out)  :: oned
      real(dop), dimension(evecdim), intent(out)  :: evec
      real(dop), dimension(coeffdim), intent(out) :: coeff
      logical(kind=4), dimension(ndof), intent(in):: loned
      integer(long)                               :: ilbl,jlbl
      character(len=c2)                           :: rdtag

!-----------------------------------------------------------------------
! read one dimensional potentials
!
! and read down to vectors
!-----------------------------------------------------------------------
      if (filever(iounit) .ge. 11.0_dop) then
         rewind(iounit)
         do
  10        continue
            read(iounit,iostat=chkerr(1),err=10,end=100) rdtag
            if (rdtag(1:1) .ne. '<') cycle

            ilbl=2
            jlbl=index(rdtag,'>')-1
            if (rdtag(ilbl:jlbl) .eq. 'onedipot') then
               go to 10

            else if (rdtag(ilbl:ilbl+3) .eq. 'oned') then
               read(rdtag(ilbl+4:jlbl),'(i10)') f

               read(iounit,iostat=ierr) (oned(zort(f)+g-1),g=1,gdim(f))
               if (ierr.ne.0) then
                  routine='fitin'
                  message='error on reading one dimensional potential '&
                     //'from natpot file'
                  call errormsg
               endif

            else if (rdtag(ilbl:jlbl) .eq. 'vectors') then
               exit

            endif
         enddo
  100    continue

      else
         do f=1,ndof
            if (loned(f)) then
               read(iounit,iostat=ierr) (oned(zort(f)+g-1),g=1,gdim(f))
               if (ierr.ne.0) then
                  routine='fitin'
                  message='error on reading one dimensional potential '&
                     //'from natpot file'
                  call errormsg
               endif
            endif
         enddo

      endif

!-----------------------------------------------------------------------
! read down to vectors
!-----------------------------------------------------------------------
!     if (filever(unit) .ge. 11.0_dop) then
!        rewind(unit)
!        do
! 10        continue
!           read(unit,iostat=chkerr(1),err=10,end=100) rdtag
!           if (rdtag(1:1) .ne. '<') cycle

!           ilbl=2
!           jlbl=index(rdtag,'>')-1
!           if (rdtag(ilbl:jlbl) .eq. 'vectors') exit
!        enddo
!     endif

!-----------------------------------------------------------------------
! read relevant natural potentials and contracted coefficients
!-----------------------------------------------------------------------
      do m=1,npmode
         if (m.eq.modc) then
            call vecin(coeff,dimbef,dimmodc,dimaft,iounit)
         else
            call vecin(evec(zevec(m)),onei,psubdim(m),potdim(m),iounit)
         endif
      enddo

      return
      end subroutine fitin

!-----------------------------------------------------------------------
!  Subroutine vecin
!-----------------------------------------------------------------------

      subroutine vecin(vec,v,subdim,n,unit)

use global

      implicit none

      integer(long)                                 :: i,j,g,ierr
      integer(long), intent(in)                     :: v,subdim,n,unit
      real(dop), dimension(v,subdim,n), intent(out) :: vec

      integer(long)                                 :: ilbl,jlbl
      character(len=c2)                             :: rdtag

      do i=1,n
         do j=1,v
            read (unit,iostat=ierr) (vec(j,g,i),g=1,subdim)
            if (ierr.ne.0) then
               write(6,'(/a,2(1x,i4),a,2(1x,i4),a,2i6)') &
                    '  i,potdim',i,n, &
                    ' ; j,v',j,v,' ; g,subdim = ',g,subdim
               routine='vecin'
               message='error on reading natural potential or '&
                  //'contracted coefficients from natpot file'
               call errormsg
            endif
         enddo
      enddo

      return
  100 continue
      routine='vecin'
      message='Cannot find natpot vectors in natpot file '
      call errormsg
      end subroutine vecin


      end module fitinmod


