!#######################################################################

      module getpesmod

use versions, only: progname
use decimal, only:long,dop,sip

use global
use pglobal
use maxv, only:maxdim
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels
use psiutil, only: f2mdecoder
use directutil, only: indexmap
use operutil, only: vcalc1st
use logdat
use op1lib,  only: initvxi
use lalib,   only: vdminmax, vrminmax
use timing

use iodvrdef, only:wrdvrdef
use callmuldmod, only:defmuld, callmuld
use callanldmod, only:defanld, callanld
use funcsrf, only:surfinfo
use vpotpars, only:readvpotinfo, writevpotpars, skipvpotpars
use rdopermod, only:rdoper
use directutil, only:indexmap,index2coord
use pesfit, only:addtxvxdd
use compdat, only:getcsys
use potevalmod, only:calcpes
   
use mpi_pf, only: pf_mpi_genpot, pf_mpi_init2, pf_mpi_genpot2
use mpidata
#ifdef MPI
use mpi
#endif

      implicit none
      private
      public :: getpes,pescut,vextime,getv1d,loadv,subtractv1d,get1dpot,vcut
      contains

!#######################################################################
!
!        POTFIT  potential surface DRIVER:  getpes
!
!   ort (i)
!   vvec (o)
!      holds the potential in double precision.
!   fvvec (o)
!      single precision version of vvec. used only when keyword 
!      "single-precision" is set in RUN-SECTION.
!   oned (o)
!
!#######################################################################
! New version (Benjamin, December 2008)
!#######################################################################

      subroutine getpes(ort,vvec,oned,buffer,hops,done)

      implicit none

      integer(long), external                    :: slen
      integer(long)                              :: f,h,hl,ilbl,ierr,i,sl
      integer(long)                              :: chkdvr, chkgrd
      integer(long), dimension(maxhop), intent(inout) :: done
      real(dop), dimension(ortdim), intent(in)   :: ort
      real(dop), dimension(oneddim), intent(out) :: oned
      real(dop), dimension(niobuf), intent(out)  :: buffer
      real(dop), dimension(hopsdim), intent(out) :: hops

      real(dop), dimension(vpotdim), intent(out) :: vvec
      logical(kind=4)                            :: lbuildvpot,lexist
      character(len=c5)                          :: outname
      character(len=c2)                          :: wrtag
      integer(long), save                        :: tid=0 ! timer ID

      call get_timer(tid, 'Getpes')
      call start_timer(tid)

!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
      routine = 'getpes'

      if(lcont .or. lrdnpot) goto 49  ! Read the vpot file

      if(lpesfile) then
        chkdvr = 2
        chkgrd = 2
        call rdoper(hops,chkdvr,chkgrd)

        if(lextpot.and..not.lgenvpot) then
          lbuildvpot=.false.
          goto 50
        end if

        goto 49

      endif ! if(lpesfile)

      if(lextpot.and..not.lgenvpot) then
        lbuildvpot=.false.
        if(vpotdim .lt. 2.14d+9)then  ! MGPF
           if (lfiterr.eqv..false.) goto 50
           lbuildvpot = .true.
           lfiterr = .true.

           write(ilog,'(/a,/a,/a,/a,/a,/a)') &
                  '-----------------------------------------------' , &
                  ' MGPF calculation: vpotdim is lower than 2.14d+9. ' , &
                  ' Computation of error automatically activated. ', &
                  ' To disable it include the "nofiterr" keyword ', &
                  ' in the RUN-SECTION of your input file.', &
                  '-----------------------------------------------'

           goto 49

        end if

        goto 50

      end if       !   if(lextpot.and..not.lgenvpot)

!-----------------------------------------------------------------------
! search for surface in library
!-----------------------------------------------------------------------
      hl=hopdim1+1

      call defmuld(hopilab(hl),hoplab(hl),hopfile(hl),pesopts,&
           zpesopts,hopipar(:,hl),hoprpar(:,hl),maxhpar,pesmin,&
           pesmax,lcutpes,ierr,ndof,hoppar)

      if (ierr .ne. 0) then
         routine='defmuld'
         call errormsg
      else if (hopfile(hl).eq.0) then
         routine='defmuld'
         message='PES not found in library : '//hoplab(hl)
         call errormsg
      endif

      h=1
      htmmuld(h)=1
      call surfinfo(h,hl,done)

!-----------------------------------------------------------------------
! store number of surface for later use in getvpot
!-----------------------------------------------------------------------
      ippes=hopilab(hl)

      hl=hopdim1+1
      write(ilog,'(/,a)') 'Surface to be fitted : '//trim(hoplab(hl))
      write(ilog,'(a,e20.8)') 'max pes cut (ev) : ',pesmax*ev
      write(ilog,'(a,e20.8)') 'min pes cut (ev) : ',pesmin*ev

49    continue

      call checkvpot(ivpot,lbuildvpot)

50    continue

      if (lprojlowmem) then
!        currently can neither read nor write full vpot
         continue

      elseif (lbuildvpot) then

#ifdef MPI
         if (lmpi) then
            call pf_mpi_init2(ort)
            mpitask = mpitask_pf_genpot
            call mpi_bcast(mpitask,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
         endif
#endif

!-----------------------------------------------------------------------
!        open vpot file
!-----------------------------------------------------------------------
         if (vpotfmt.eq.2) then
            outname=name(1:laenge)//'/vpot2'
         else
            outname=name(1:laenge)//'/vpot'
         endif
         sl=slen(outname)

         if (.not.lovwr) then
            inquire(file=outname,exist=lexist)
            if (lexist) then
               routine='getpes'
               message='File already exists: '//outname(1:sl)
               call errormsg
            endif
         endif   

         write(ilog,'(/,76a1)')  ('-',i=1,76)
         write(ilog,'(a)') ' Creating VPOT file: '//outname(1:sl)
         write(ilog,'(76a1)')  ('-',i=1,76)

         open(unit=ivpot,status='unknown',file=outname(1:sl),&
               form='unformatted',iostat=ierr)
         if (ierr.ne.0) then
            message='could not open '//outname(1:sl)//' for writing'
            routine='getpes'
            call errormsg
         endif
         call myflush(ilog)

!-----------------------------------------------------------------------
!        calculate exact potential and write it to disc
!        if (.not.lwo), the potential is also storred in (f)vvec.
! progver moved to before dvrdef in 83.228 - was previously in wrong 
! place
!-----------------------------------------------------------------------
         call writevpotpars(ivpot,outname,0)
            
!-----------------------------------------------------------------------
! Write pes file to vpot file
!-----------------------------------------------------------------------
         wrtag= '<vpot pes>'
         write(ivpot) wrtag
         if (lpesfile) then
            call write_pes_vpot(ivpot,outname,vvec,buffer,hops)
         else
            if (lmpi) then
               if (vpotfmt.eq.2) then
                  call pf_mpi_genpot2(ort)
               else
                  call pf_mpi_genpot(ort,buffer)
               endif
            else
!--------------compute vpot and writes it to file.
               call write_vpot(ivpot,outname,ort,vvec,buffer,hops)
            endif
         endif
         wrtag= '<end vpot pes>'
         write(ivpot) wrtag

      else if (.not.lwo) then
!-----------------------------------------------------------------------
!        read surface from vpot file into (f)vvec
!        we assume that the vpot file has been opened
!        NOTE: maybe you want to read vpot, use lextpot and fiterr
!-----------------------------------------------------------------------
         if(lextpot .or. lcw)then  ! (See Note)
           call loadv(ivpot,vvec,buffer)
         endif

      endif   !  if (lbuildvpot)

!-----------------------------------------------------------------------
! get oned potentials to be subtracted
!-----------------------------------------------------------------------
      do f=1,ndof
         if(onedipot(f) .gt. 0) then
            hl=onedipot(f)
          
            call defanld(hopilab(hl),hoplab(hl),hopfile(hl),&
                 hopipar(1,hl),hoprpar(1,hl),maxhpar,hoppar,&
                 apar,rpar,npar,ierr,zpesopts,pesopts,hamlab,hamdim)
            ilbl=index(hoplab(hl),' ')-1
            if(hopfile(hl) .eq. 0) then
               routine='getpes'
               message='PES not found in library : '//hoplab(hl)(1:ilbl)
               call errormsg
            endif
            write(ilog,'(a,i3,", ",a,", ",a)') &
                  ' oned potential subtracted. f =',f,&
                  hoplab(hl)(1:ilbl),modelabel(f)(1:slen(modelabel(f)))

            call get1dpot(oned(zort(f)),ort(zort(f)),gdim(f),&
                 hopilab(hl),hopfile(hl),hopipar(1,hl),&
                 hoprpar(1,hl),pesmax,pesmin,lcutpes,hoplab(hl),lsoftcutpes)
         endif
      enddo

!-----------------------------------------------------------------------
! If coupled states option is set, write data to standard output.
!-----------------------------------------------------------------------
      if (lcspot) then
         write (iout,*) '*** subroutine getpesd ***'
         write (iout,*)
         write (iout,*) 'Coupled states approximation parameter'
         write (iout,'(a,1x,f8.3)') '   Jtot     : ',jtot
         write (iout,'(a,1x,f8.3)') '   Jbf      : ',jbf
         write (iout,'(a,1x,f8.3)') '   mass [au]: ',csmass
         write (iout,*)
      endif
      
      call stop_timer(tid)
      return

      end subroutine getpes

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     this routine determines whether a new vpot file must be generated
!     or if we can read an existing one.
!
!     unit (i)
!     lbuildvpot (o)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine checkvpot(unit,lbuildvpot)

      implicit none

      integer(long), intent(in)              :: unit
      integer(long)                          :: ierr,i,hl,pdofft1,chkdvr
      integer(long), dimension(maxdim)       :: fdvr
      logical(kind=4), intent(out)           :: lbuildvpot
      logical(kind=4)                        :: lexist,differ,lsp1,lftp1,lcspot1
      character(len=c2)                      :: label
      character(len=c5)                      :: opts
      real(dop)                              :: pesmin1,pesmax1,progver1,jtot1,jbf1,csmass1


      lbuildvpot=.false.
      
      inquire(file=vname(1:vlength),exist=lexist)
      if (.not.lexist) then
         if (lreadvpot) then
            routine='checkvpot'
            message=vname(1:vlength)//' does not exist'
            call errormsg
         else
            lbuildvpot=.true.
            return
         endif
      endif

      if (.not.lreadvpot .and. lgenvpot) then
         lbuildvpot=.true.
         return
      endif

!     vpot file exists
      
      write(ilog,'(/,76a1)')  ('-',i=1,76)
      write(ilog,'(2a)') ' Reading VPOT file: ',vname(1:vlength)
      write(ilog,'(76a1)')  ('-',i=1,76)

      open(unit=unit,file=vname(1:vlength),form='unformatted',&
         status='old',iostat=ierr)
      if (ierr.ne.0) then
         routine='checkvpot'
         message='cannot open '//vname(1:vlength)//' for reading'
         call errormsg
      endif

      chkdvr=6
      call readvpotinfo(unit,progver1,label,opts,lsp1,&
                        lftp1,pdofft1,pesmin1,pesmax1,chkdvr,&
                        lcspot1,jtot1,jbf1,csmass1,fdvr)

!     compare parameters read from vpot file with parameters
!     read from input file
      
      differ=.false.
      hl = hopdim1+1
      if (label.ne.hoplab(hl)) differ=.true.
      if (npesopt.gt.0) then
         if (nopts(npesopt).gt.0) then
            if (pesopts(npesopt).ne.opts)             differ=.true.
         endif
      endif
      if (lsp1.neqv.lsp) differ=.true.
      if ((lcspot1.neqv.lcspot) .or. (jtot1.ne.jtot)) differ=.true.
      if (jbf1.ne.jbf .or. csmass1.ne.csmass)         differ=.true.
      if (chkerr(2).ne.0 .or. chkerr(3).ne.0)         differ=.true.
      if (progver1 .ge. 8.3013_dop ) then
! --- lftp and pdofft are obsolete, they are kept for concistency only.
! --- lftp=.false., pdofft=0.
         if (lftp1 .neqv. .false.)                    differ=.true.
         if (pdofft1 .ne. 0)                          differ=.true.
      endif 

      if (ippes.eq.8) differ=.false.      !  'none' potential
                                          !  changed from earlier versions
                                          !  due to new surfaces library

      if (differ) then
         write(ilog,'(2a,1x,a)') 'label ',trim(label),trim(hoplab(1))
         write(ilog,*) 'lsp', lsp1,lsp
         if (npesopt.gt.0) then
            if (nopts(npesopt).gt.0) then
               write(ilog,'(2a,1x,a)')'options ', &
                 trim(pesopts(npesopt)),trim(opts)
            endif
         endif
         routine='checkvpot'
         message='Surface parameters of '//vname(1:vlength)//&
            ' differ from those of input file.'
         if (lreadvpot) then
            call errormsg
         else
            call warnmsg
            write(6,'(a)') ' I will try to generate a new vpot file.'
            lbuildvpot=.true.
            close(unit=unit)
         endif
      endif
      call myflush(ilog)

      return
      end subroutine checkvpot
      
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     loads the potential stored in a vpot file into vvec.
!     it is assumed that the file vpot has been opened for reading
!     via channel unit. The data on file and the vector vvec are
!     double precision. (See also loadv2 and floadv).    
!
!     unit (i)
!     vvec (o)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine loadv(unit,vvec,buffer)

      implicit none

      integer(long)                              :: j
      integer(long), save                        :: unit0 = 0, nitems = 0
      real(dop), dimension(vpotdim), intent(out) :: vvec
      real(dop), dimension(niobuf), intent(out)  :: buffer
      integer(long), intent(in)                  :: unit
      integer(long), save                        :: tid=0 ! timer ID

      call get_timer(tid, 'Loadv')
      call start_timer(tid)

! --- don't load from disk  (nitems=vpotdim .eq. niobuf)
      if(unit.eq.unit0 .and. niobuf.eq.nitems)  then
         do j=1,niobuf
            vvec(j) = buffer(j)
         enddo

! read from file
      else if (filever(unit) .ge. 11.000_dop) then
         call loadv11(unit,vvec,buffer,nitems)
      else
         call loadv_pre11(unit,vvec,buffer,nitems)
      endif

! --- in case one does not need to read again
      if(niobuf.eq.nitems)  then
         unit0 = unit
         do j=1,niobuf
            buffer(j) = vvec(j)
         enddo
      endif

      call stop_timer(tid)

      return
      end subroutine loadv

!#######################################################################

      subroutine loadv11(unit,vvec,buffer,nitems)

      implicit none

      integer(long), intent(out)                 :: nitems
      real(dop), dimension(vpotdim), intent(out) :: vvec
      real(dop), dimension(niobuf), intent(out)  :: buffer
      integer(long), intent(in)                  :: unit
      integer(long)                              :: niobuf1,ierr,i,m1,m2,j1,j,jj
      integer(long)                              :: unit0,ilbl,jlbl
      logical(kind=4)                            :: lfound
      character(len=c2)                          :: rdtag


! find potential
      ilbl=0
      jlbl=0
      rewind(unit)
      lfound=.false.
      do
  10     continue
         read(unit,err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'vpot pes') then
            lfound=.true.
            exit
         endif
      enddo
  100 continue
      if (.not. lfound) then
         routine='loadv'
         message='cannot find pes in vpot file'
         call errormsg
      endif

! --- load from disk  (nitems=vpotdim)
      read(unit,iostat=ierr) nitems,niobuf1
      if (ierr.ne.0) then
         routine='loadv'
         message='cannot read nitems from '//vname(1:vlength)
         call errormsg
      endif
      
! --- check consistency
      if (niobuf1.gt.niobuf) then
         routine='loadv'
         message='niobuf is too small! '//&
                 'Set "buffer" in Run-Section appropriately.'
         call errormsg
      endif

      m1=nitems/niobuf1
      m2=nitems-m1*niobuf1
      j1=0

!.....Read the buffer m1 times (m1 may be zero).
      do i=1,m1
         read(unit,iostat=ierr) (buffer(j),j=1,niobuf1)
         if (ierr.ne.0) then
            write(6,'(a)') ' ERROR while reading vpot-file'
            if( progname(1:6).eq."potfit" ) write(6,'(a)')&
            ' Use option -gv to enforce creating a new vpot-file'
            routine='loadv'
            message='error on reading '//vname(1:vlength)
            call errormsg
         endif
         if(lwrcw) then  ! write corr weights, no mapping
            do j=1,niobuf1
               vvec(j+j1) = buffer(j)
            enddo
         else
            do j=1,niobuf1
               call indexmap(j+j1,jj)
               vvec(jj) = buffer(j)
            enddo
         endif
         j1=j1+niobuf1
      enddo

!.....Read the rest of the data, which only partly fills the buffer.
      if (m2.ne.0) then
         read(unit,iostat=ierr) (buffer(j),j=1,m2)
         if (ierr.ne.0) then
            write(6,'(a)') ' ERROR while reading vpot-file'
            if( progname(1:6).eq."potfit" ) write(6,'(a)')&
            ' Use option -gv to enforce creating a new vpot-file'
            routine='loadv'
            message='error on reading '//vname(1:vlength)
            call errormsg
         endif
         do j=1,m2
            call indexmap(j+j1,jj)
            vvec(jj) = buffer(j)
         enddo
      endif

! --- inspect vvec
      call vprop(vvec,nitems)

! --- handle cut of potential
      call vcut(vvec,nitems)

! --- inspect vvec after cut
      call vprop1(vvec,nitems)

! --- in case one does not need to read again
      if(niobuf.eq.nitems)  then
         unit0 = unit
         do j=1,niobuf
            buffer(j) = vvec(j)
         enddo
      endif

      return
      end subroutine loadv11
      
!#######################################################################

      subroutine loadv_pre11(unit,vvec,buffer,nitems)

      implicit none

      integer(long), intent(out)                 :: nitems
      real(dop), dimension(vpotdim), intent(out) :: vvec
      real(dop), dimension(niobuf), intent(out)  :: buffer
      integer(long), intent(in)                  :: unit
      integer(long)                              :: niobuf1,ierr,i,m1,m2,j1,j,jj
      integer(long), save :: unit0 = 0

! --- load from disk  (nitems=vpotdim)
      call skipvpotpars(unit)
      read(unit,iostat=ierr) nitems,niobuf1
      if (ierr.ne.0) then
         routine='loadv'
         message='cannot read nitems from '//vname(1:vlength)
         call errormsg
      endif

! --- check consistency
      if (niobuf1.gt.niobuf) then
         routine='loadv'
         message='niobuf is too small! '//&
                 'Set "buffer" in Run-Section appropriately.'
         call errormsg
      endif

      m1=nitems/niobuf1
      m2=nitems-m1*niobuf1
      j1=0

!.....Read the buffer m1 times (m1 may be zero).
      do i=1,m1
         read(unit,iostat=ierr) (buffer(j),j=1,niobuf1)
         if (ierr.ne.0) then
            write(6,'(a)') ' ERROR while reading vpot-file'
            if( progname(1:6).eq."potfit" ) write(6,'(a)')&
            ' Use option -gv to enforce creating a new vpot-file'
            routine='loadv'
            message='error on reading '//vname(1:vlength)
            call errormsg
         endif
         if(lwrcw) then  ! write corr weights, no mapping
            do j=1,niobuf1
               vvec(j+j1) = buffer(j)
            enddo
         else
            do j=1,niobuf1
               call indexmap(j+j1,jj)
               vvec(jj) = buffer(j)
            enddo
         endif
         j1=j1+niobuf1
      enddo

!.....Read the rest of the data, which only partly fills the buffer.
      if (m2.ne.0) then
         read(unit,iostat=ierr) (buffer(j),j=1,m2)
         if (ierr.ne.0) then
            write(6,'(a)') ' ERROR while reading vpot-file'
            if( progname(1:6).eq."potfit" ) write(6,'(a)')&
            ' Use option -gv to enforce creating a new vpot-file'
            routine='loadv'
            message='error on reading '//vname(1:vlength)
            call errormsg
         endif
         do j=1,m2
            call indexmap(j+j1,jj)
            vvec(jj) = buffer(j)
         enddo
      endif

! --- inspect vvec
      call vprop(vvec,nitems)

! --- handle cut of potential
      call vcut(vvec,nitems)

! --- inspect vvec after cut
      call vprop1(vvec,nitems)

! --- in case one does not need to read again
      if(niobuf.eq.nitems)  then
         unit0 = unit
         do j=1,niobuf
            buffer(j) = vvec(j)
         enddo
      endif

      return
      end subroutine loadv_pre11
      
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     scans the potential vector vvec and replaces all entries outside
!     the interval [pesmin,pesmax] by the boundary values.
!
!     vvec (i)
!     nitems (i)
!        logical length of vvec
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine vcut(vvec,nitems)

      implicit none

      integer(long)                               :: j
      integer(long), intent(in)                   :: nitems
      real(dop)                                   :: v
      real(dop), dimension(nitems), intent(inout) :: vvec

      if (lcutpes) then
         do j=1,nitems
            v=min(vvec(j),pesmax)
            vvec(j)=max(v,pesmin)
         enddo
      elseif(lsoftcutpes .and. v.ge.pesmax) then
         do j=1,nitems
            v = min(vvec(j), pesmax*(1.d0+log(abs(vvec(j)/pesmax))))
            vvec(j) = max(v,pesmin)
         enddo
      endif

      return
      end subroutine vcut
      
!#######################################################################

      subroutine get1dpot(oned,ort,gdim,hopilab,hopfile,hopipar,&
                 hoprpar,pesmax,pesmin,lcutpes,hoplab,lsoftcutpes)

      implicit none

      integer(long)                           :: g,pottyp
      integer(long), intent(in)               :: gdim,hopilab,hopfile
      integer(long), dimension(*), intent(in) :: hopipar
      real(dop)                               :: v
      real(dop), intent(in)                   :: pesmax,pesmin
      real(dop), dimension(*), intent(in)     :: hoprpar
      real(dop), dimension(gdim), intent(in)  :: ort
      real(dop), dimension(gdim), intent(out) :: oned
      logical(kind=4), intent(in)             :: lcutpes,lsoftcutpes
      character(len=*), intent(in)            :: hoplab

      do g=1,gdim
         call callanld(v,ort(g),hopilab,hopfile,hopipar,hoprpar,&
              0.0_dop,pottyp,hoplab,1.0_dop)
         if (lcutpes) then
            v=min(v,pesmax)
            v=max(v,pesmin)
! option: softcut
         elseif (lsoftcutpes) then
            v = min(v, pesmax*(1.d0+log(abs(v/pesmax))))
            v = max(v,pesmin)
         endif
         oned(g)=v
      enddo

      return
      end subroutine get1dpot

!***********************************************************************
!
!     Calculates the complete surface on all grid points and writes it
!     to file `filename' which has been opened via channel `unit'.
!     We output in chunks of niobuf data items. This is done to avoid the
!     allocation of the complete vector vvec in the case of write-only mode.
!     The "raw" potential is written to disk (no cut is applied: this is
!     done during loading).
!
!     unit (i)
!     filename (i)
!     ort (i)
!     vvec (o)
!         only set completely if .not.lwo
!     
!***********************************************************************

      subroutine write_vpot(unit,filename,ort,vvec,buffer,hops)

      implicit none

      integer(long), intent(in)                  :: unit
      character(len=c5), intent(in)              :: filename
      real(dop), dimension(hopsdim), intent(in)  :: hops
      real(dop), dimension(ortdim), intent(in)   :: ort
      real(dop), dimension(maxsta,maxsta)        :: vsub
      real(dop), dimension(vpotdim), intent(out) :: vvec
      real(dop), dimension(niobuf), intent(out)  :: buffer

      integer(long)                          :: f,niobuf1,hl
      integer(long), dimension(maxdim)       :: igpoint,igpoint1
      real(dop)                              :: cpu
      real(dop), dimension(maxdim)           :: gpoint
      integer(long)                          :: j,jj,j1,j2,ierr,idum
      complex(dop), dimension(maxsta,maxsta) :: cdum
      character(len=80)                      :: actual
      real(dop), external                    :: fcputime,fsystime


!     write number of data points and chunksize to disk

      niobuf1=min(vpotdim,niobuf)
      write(unit,iostat=ierr) vpotdim,niobuf1
      if (ierr.ne.0) then
         routine='write_vpot'
         message='error on writing nitems to '//trim(filename)
         call errormsg
      endif
      
      j1=0
      j2=0
      call initvxi(igpoint,ndof)
      
    1 j2=j2+1
      call index2coord(igpoint,ort,gpoint)
      hl=hopdim1+1
      call callmuld(buffer(j2),gpoint,ndof,ippes,hopfile(hl),&
                    hopipar(1,hl),hoprpar(1,hl),idum,idum,hoplab(hl))
!
! subtract a specified function
!
      if (lsubtract) then
         do f=1,ndof
            igpoint1(f)=igpoint(f)+goff(f)
         enddo
         call getcsys(2)
         call calcpes(hops,vsub,igpoint1,gpoint,cdum,idum,1)
         call getcsys(1)
         buffer(j2)=buffer(j2)-vsub(1,1)
      endif

      if (j2.eq.niobuf1) then
         write(unit,iostat=ierr) (buffer(j),j=1,niobuf1)
         if (ierr.ne.0) then
            routine='write_vpot'
            message='error on writing to '//trim(filename)
            call errormsg
         endif

! -----  write timing information to log file.
         call fzeit(actual)
         j = index(actual,':')
         actual=actual(j-9:j+6)
         cpu=fcputime(0)+fsystime(0)
         write(ilog,'(a,i10,i9,3x,a16,"  CPU =",f9.1)')&
          'Buffer written. j1, j2 =',j1,j2,actual,cpu
         call myflush(ilog)

! -----  copy buffer to vvec (no mapping) if write-cw and not write-only
         if (.not.lwo .and. lwrcw) then
            do j=1,j2
               vvec(j+j1) = buffer(j)
            enddo

! -----  map buffer to vvec if not write-only
         else if (.not.lwo) then
            do j=1,j2
               call indexmap(j+j1,jj)
               vvec(jj) = buffer(j)
            enddo
         endif

! -----  reset indices
         j1=j1+niobuf1
         j2=0
      endif
      
! --- determine new set of grid-points
      do f=1,ndof
         if (igpoint(f).lt.gdim(f)) then
            igpoint(f)=igpoint(f)+1
            goto 1
         else
            igpoint(f)=1
         endif
      enddo

! --- write the final partly filled buffer to disk. 
      if (j2.ne.0) then
         write(unit,iostat=ierr) (buffer(j),j=1,j2)
         if (ierr.ne.0) then
            routine='write_vpot'
            message='error on writing to '//trim(filename)
            call errormsg
         endif
! -----  write timing information to log file.
         call fzeit(actual)
         j = index(actual,':')
         actual=actual(j-9:j+6)
         cpu=fcputime(0)+fsystime(0)
         write(ilog,'(a,i10,i9,3x,a16,"  CPU =",f9.1)') &
          'Buffer written. j1, j2 =',j1,j2,actual,cpu
         call myflush(ilog)
      endif

      if (.not.lwo .and. lwrcw) then
         do j=1,j2
            vvec(j+j1) = buffer(j)
         enddo

      else if (.not.lwo) then

! -----  map partly filled buffer to vvec
         do j=1,j2
            call indexmap(j+j1,jj)
            vvec(jj) = buffer(j)
         enddo

! -----  inspect potential
         call vprop(vvec,vpotdim)

! -----  handle potential cut
         call vcut(vvec,vpotdim)

      endif

      return
      end subroutine write_vpot

!***********************************************************************
!
!     returns the element of vvec designated by the coefficients igpoint
!
!     igpoint (i)
!     vvec (i)
!     v (o)
!     
!***********************************************************************
      
      subroutine getv_element(imgpoint,vvec,v)

      implicit none

      real(dop), intent(out)                     :: v
      real(dop), dimension(vpotdim), intent(in)  :: vvec
      integer(long), dimension(ndof), intent(in) :: imgpoint
      integer(long)                              :: f,m,n,offset

      offset=0
      do m=npmode,1,-1
        do n=npotdof(m),1,-1
          f=potdof(n,m)
          offset=gdim(f)*offset+imgpoint(f)-1
        enddo
      enddo
      offset=offset+1
      v=vvec(offset)

      return
      end subroutine getv_element
         
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     calculates the potential energy at grid point `gpoint'
!     (interface to subroutine vpoint)
!
!     gpoint (i)
!        vector of ndof reals representing the coordinates of a grid
!        point.
!     v (o)
!        potential at the grid point described by gpoint.
!        note that one has to select a potential energy surface via
!        defmuld prior to calling this routine.
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine getvpot(gpoint,v)

      implicit none

      integer(long)                          :: hl,idum
      real(dop), intent(out)                 :: v
      real(dop), dimension(ndof), intent(in) :: gpoint

      hl=hopdim1+1
      call callmuld(v,gpoint,ndof,ippes,hopfile(hl),hopipar(1,hl),&
         hoprpar(1,hl),idum,idum,hoplab(hl))

      if (lcutpes) then
         v=min(v,pesmax)
         v=max(v,pesmin)
      endif

      return
      end subroutine getvpot

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     returns via v1d the sum of all 1D potentials which may be defined 
!     for the grid point with indices igpoint
!
!     igpoint (i)
!        vector of ndof integers representing the indeces of a grid
!        point.
!     oned (i)
!        vector containing the 1d potentials at the grid points of
!        every degree of freedom.
!     v1d (o)
!        sum of all 1D potentials defined at the grid point described by
!        igpoint.
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine getv1d(igpoint,oned,v1d)

      implicit none

      integer(long), dimension(ndof), intent(in) :: igpoint
      real(dop), intent(out)                     :: v1d
      real(dop), dimension(oneddim), intent(in)  :: oned
      integer(long)                              :: f

      v1d=0.0_dop
      if (ploned) then
         do f=1,ndof
            if (onedipot(f).gt.0) v1d=v1d+oned(zort(f)+igpoint(f)-1)
         enddo
      endif
      
      return
      end subroutine getv1d

!***********************************************************************
!
!     calculates the complete potential surface on all grid points.
!     this is done 'npest' times for timing purposes. the routine
!     has no output.
!
!     ort (i)
!     
!***********************************************************************

      subroutine vextime(ort)

      implicit none

      real(dop), dimension(ortdim), intent(in) :: ort
      integer(long)                            :: f,i,n,m
      integer(long), dimension(maxdim)         :: igpoint
      real(dop)                                :: v 
      real(dop), dimension(maxdim)             :: gpoint
      integer(long), save                      :: tid=0 ! timer ID

      call get_timer(tid, 'Vex')
      call start_timer(tid)
      
      do i=1,npest
         call initvxi(igpoint,ndof)
 1       call index2coord(igpoint,ort,gpoint)
         call getvpot(gpoint,v)
         do m=1,npmode
           do n=1,npotdof(m)
               f=potdof(n,m)
            if (igpoint(f).lt.gdim(f)) then
               igpoint(f)=igpoint(f)+1
               goto 1
            else
               igpoint(f)=1
            endif
           enddo
         enddo
      enddo

      call stop_timer(tid)
      print*,timers(tid)%ncalls
      
      return
      end subroutine vextime

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     calculates some properties of the potential on the product grid
!     and outputs them via channel iout
!
!     vvec (i)
!        real*8 vector
!     n
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine vprop(vvec,n)

      implicit none
      
      integer(long), intent(in)           :: n
      real(dop), dimension(n), intent(in) :: vvec
      real(dop)                           :: vavr,vrms

      logical(kind=4), save               :: firstcall = .true.
      
!-----------------------------------------------------------------------
!     determine minimum and maximum of the potential vector
!-----------------------------------------------------------------------
      if (firstcall) then
         firstcall=.false.
         call vdminmax(vvec,n,vmin,vmax,vavr,vrms)
         write(iout,'(a,e16.8," au, ",e16.8," ev.")')&
               'potential minimum : ',vmin, vmin*ev
         write(iout,'(a,e16.8," au, ",e16.8," ev.")') &
               'potential maximum : ',vmax, vmax*ev
         write(iout,'(a,e16.8," au, ",e16.8," ev.")') &
               'potential average : ',vavr, vavr*ev 
         write(iout,'(a,e16.8," au, ",e16.8," ev.")') &
               'potential RMS     : ',vrms, vrms*ev
         write(iout,'(a,e16.8," au, ",e16.8," ev.")') &
               'standard deviation: ',sqrt(vrms**2-vavr**2), &
                                      sqrt(vrms**2-vavr**2)*ev
      endif

      return
      end subroutine vprop

!--------------------------------------------------------

      subroutine vprop1(vvec,n)

      implicit none

      integer(long), intent(in)           :: n
      real(dop), dimension(n), intent(in) :: vvec
      real(dop)                           :: vavr,vrms

      logical(kind=4), save               :: firstcall = .true.
      
      if(vmax.lt.pesmax  .and.  vmin.gt.pesmin) return
      if (firstcall) then
         firstcall=.false.
         call vdminmax(vvec,n,vmin,vmax,vavr,vrms)
         write(iout,'(a)') 'After cutting the PES:'
         write(iout,'(a,e16.8," au, ",e16.8," ev.")') &
               'potential maximum : ',vmax, vmax*ev
         write(iout,'(a,e16.8," au, ",e16.8," ev.")') &
               'potential average : ',vavr, vavr*ev
         write(iout,'(a,e16.8," au, ",e16.8," ev.")') &
               'potential average : ',vavr, vavr*ev
         write(iout,'(a,e16.8," au, ",e16.8," ev.")') &
               'potential RMS     : ',vrms, vrms*ev
         write(iout,'(a,e16.8," au, ",e16.8," ev.")') &
               'standard deviation: ',sqrt(vrms**2-vavr**2), &
                                      sqrt(vrms**2-vavr**2)*ev
      endif

      return
      end subroutine vprop1

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     subtracts the oned potentials from the potential vector
!
!     vvec (i)
!     oned (i)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine subtractv1d(vvec,oned)

      implicit none

      real(dop), dimension(ortdim), intent(in)     :: oned
      real(dop), dimension(vpotdim), intent(inout) :: vvec
      integer(long)                                :: f,aft,bef,n,m

      bef = 1
      aft = vpotdim
      do m=1,npmode
         do n=1,npotdof(m)
         f=potdof(n,m)
         aft=aft/gdim(f)
         if (onedipot(f) .gt. 0) then
            call addtxvxdd(vvec,oned(zort(f)),bef,gdim(f),aft,-1)
         endif
         bef=bef*gdim(f)
         enddo
      enddo

      return
      end subroutine subtractv1d

!#######################################################################

      subroutine pescut(hops,ort1,ort)

      implicit none

      integer(long)                               :: f
      real(dop)                                   :: ortold,diff
      real(dop), dimension(hopsdim), intent(in)   :: hops
      real(dop), dimension(ortdim), intent(inout) :: ort
      real(dop), dimension(ortdim1), intent(in)   :: ort1

      write(ilog,'(/,a)') '*** Checking coincidence of PES cut ***'
      do f = 1,ndof

         if (basis(f) .eq. 19) then
            ortold=ort(zort(f))
            call getigp(ort(zort(f)),ort1(zort1(f)),goff(f),gdim1(f))
            diff = abs(ort(zort(f))-ortold)
            if (diff .gt. tiny) then
               write(ilog,'(3a,f10.6,a,f10.6)') &
                   'Grid point for DOF ',modelabel(f),&
                   ' moved from ',ortold, ' to ',ort(zort(f))
            endif
         endif
      enddo

      return
      end subroutine pescut
 
!#######################################################################
 
      subroutine getigp(ort,ort1,goff,gdim1)

      implicit none

      integer(long)                           :: g
      integer(long), intent(in)               :: gdim1
      integer(long), intent(out)              :: goff
      real(dop)                               :: mindiff,diff
      real(dop), dimension(1), intent(inout)  :: ort
      real(dop), dimension(gdim1), intent(in) :: ort1

      mindiff=999.0_dop
      do g=1,gdim1
         diff=abs(ort1(g)-ort(1))
         if (diff .lt. mindiff) then
            mindiff = diff
            goff=g-1
         endif
      enddo
      ort(1) = ort1(goff+1)

      return
      end subroutine getigp

!***********************************************************************
!     Writes vpot file from a previous existing filename (pes) file
!     For an explanation cf. subroutine write_vpot
!***********************************************************************

      subroutine write_pes_vpot(unit,filename,vvec,buffer,hops)

      implicit none

      integer(long), intent(in)                  :: unit
      character*(c5), intent(in)                 :: filename
      real(dop), dimension(vpotdim), intent(out) :: vvec
      real(dop), dimension(niobuf), intent(out)  :: buffer
      real(dop), dimension(hopsdim), intent(in)  :: hops
      integer(long), dimension(maxdim)           :: igpoint
      integer(long)                              :: f,niobuf1,j,j1,j2,ierr,jj
      integer(long), dimension(nmode)            :: mpoint
      real(dop), external                        :: fcputime,fsystime
      real(dop)                                  ::  cpu,v
      character(len=80)                          :: actual

!     write number of data points and chunksize to disk

      niobuf1=min(vpotdim,niobuf)
      write(unit,iostat=ierr) vpotdim,niobuf1
      if (ierr.ne.0) then
         routine='write_pes_vpot'
         message='error on writing nitems to '//trim(filename)
         call errormsg
      endif

      j1=0
      j2=0
      call initvxi(igpoint,ndof)

    1 j2=j2+1
      call f2mdecoder(igpoint,mpoint)

!      WRITE(6,'(a,i7,1x,a,9(2x,i4))')'Aj2= ', j2,
!     +         'ig= ', (igpoint(j), j=1, ndof)
!      WRITE(6,'(a,i7,1x,a,9(2x,i4))')'Bj2= ', j2,
!     +         'm= ', (mpoint(j), j=1, nmode)

      call vcalc1st(hops,v,mpoint,1)
      buffer(j2) = v

      if (j2.eq.niobuf1) then
         write(unit,iostat=ierr) (buffer(j),j=1,niobuf1)
         if (ierr.ne.0) then
            routine='write_pes_vpot'
            message='error on writing to '//trim(filename)
            call errormsg
         endif

! -----  write timing information to log file.
         call fzeit(actual)
         j = index(actual,':')
         actual=actual(j-9:j+6)
         cpu=fcputime(0)+fsystime(0)
         write(ilog,'(a,i10,i9,3x,a16,"  CPU =",f9.1)') &
          'Buffer written. j1, j2 =',j1,j2,actual,cpu
         call myflush(ilog)
! -----  copy buffer to vvec (no mapping) if write-cw and not write-only
         if (.not.lwo .and. lwrcw) then
            do j=1,j2
               vvec(j+j1) = buffer(j)
            enddo

         else if(.not.lwo) then
! -----  map buffer to vvec if not write-only
            do j=1,j2
               call indexmap(j+j1,jj)
               vvec(jj) = buffer(j)
            enddo
         endif

! -----  reset indices
         j1=j1+niobuf1
         j2=0
      endif

! --- determine new set of grid-points
      do f=1,ndof
         if (igpoint(f).lt.gdim(f)) then
            igpoint(f)=igpoint(f)+1
            goto 1
         else
            igpoint(f)=1
         endif
      enddo

! --- write the final partly filled buffer to disk.
      if (j2.ne.0) then
         write(unit,iostat=ierr) (buffer(j),j=1,j2)
         if (ierr.ne.0) then
            routine='write_pes_vpot'
            message='error on writing to '//trim(filename)
            call errormsg
         endif
! -----  write timing information to log file.
         call fzeit(actual)
         j = index(actual,':')
         actual=actual(j-9:j+6)
         cpu=fcputime(0)+fsystime(0)
         write(ilog,'(a,i10,i9,3x,a16,"  CPU =",f9.1)') &
          'Buffer written. j1, j2 =',j1,j2,actual,cpu
         call myflush(ilog)
      endif

      if (.not.lwo .and. lwrcw) then
         do j=1,j2
            vvec(j+j1) = buffer(j)
         enddo

      else if(.not.lwo) then

! -----  map partly filled buffer to vvec
         do j=1,j2
            call indexmap(j+j1,jj)
            vvec(jj) = buffer(j)
         enddo

! -----  inspect potential
         call vprop(vvec,vpotdim)

! -----  handle potential cut
         call vcut(vvec,vpotdim)

! -----  inspect vvec after cut
         call vprop1(vvec,vpotdim)

      endif

      return
      end subroutine write_pes_vpot

      end module getpesmod

