!***********************************************************************
!
!            POTFIT MODULE ITERATION
!
!  11/96 AJ
!
!
!***********************************************************************
      module iterationmod

use decimal, only:long,dop,sip
use global
use pglobal
use maxv, only:maxdim
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels

use getpesmod, only:loadv,getv1d
use directutil, only:index2coord
use pesfit, only:getvfit2
use weightsmod, only:getsepwei

use coeffs, only:vcoeffs
use fiterrmod, only:fiterr

use potmod_interface
use rmlib,   only: rmsxcxdd, rmsxcxrrd
use op1lib,  only: initvxi, cuttxd2
use timing

use diagrsmod

      implicit none
      private
      public :: iteration
      contains

!***********************************************************************
!
!
!  ort (i)
!  vvec (i)
!     modified reference potential
!  oned (i)
!  coeff (i/o)
!     contracted coefficient vector including separable weights if used
!  eval (i/o)
!     natural potential populations including separable weights if used
!  evec (i/o)
!     natural potentials including separable weights if used
!  weight (i)
!  wvec   (i) weights defining the relevant region (if lwpot is set)
!  cut (w)
!     holds the tensor cut
!  rho = work1 (w)
!     holds new potential density matrix and new eigenvectors
!  work2 (w)
!     holds coefficients and new contracted coefficients
!  iterr2 (o)
!     
!***********************************************************************

      subroutine iteration(ort,vvec,oned,coeff,eval,evec,weight,wvec,&
                           cut,rho,work2,iterr2,buffer,nit)

      implicit none

      integer(long)                                :: m,bef,aft
      integer(long), intent(in)                    :: nit
      real(dop), intent(out)                       :: iterr2
      real(dop), dimension(ortdim), intent(in)     :: ort
      real(dop), dimension(oneddim), intent(in)    :: oned
      real(dop), dimension(coeffdim), intent(out)  :: coeff
      real(dop), dimension(evecdim), intent(inout) :: evec
      real(dop), dimension(weightdim), intent(in)  :: weight
      real(dop), dimension(maxpsub2), intent(out)  :: rho
      real(dop), dimension(workdim2), intent(out)  :: work2
      real(dop), dimension(evaldim), intent(inout) :: eval
      real(dop), dimension(vpotdim), intent(inout) :: vvec
      real(dop), dimension(niobuf), intent(inout)  :: buffer
      real(dop), dimension(modcdim), intent(out)   :: cut
      integer(long), dimension(vpotdim/30+1), intent(inout) :: wvec
      integer(long), save                          :: tid=0 ! timer ID

      call get_timer(tid, 'Iterations')
      call start_timer(tid)

!----------------------------------------------------------------------
! calculate new potential density matrices and their eigenvectors
!----------------------------------------------------------------------

      bef=1
      aft=vpotdim
      call vcoeffs(vvec,evec,coeff,work2)
      do m=1,npmode
         aft=aft/psubdim(m)
         call mrpot(ort,coeff,evec,oned,weight,wvec,cut,work2,vvec,buffer)
         if(psubdim(m).ne.potdim(m)) then ! not complete: (m.ne.modc) 
            call rmsxcxdd(vvec,vvec,rho,bef,psubdim(m),aft) ! get density
            call diagrs(eval(zmode(m)),evec(zevec(m)),rho,&
                        psubdim(m),potdim(m),relnw(m),1)
         endif
         call vcoeffs(vvec,evec,coeff,work2)
         bef=bef*psubdim(m)
      enddo 

!-----------------------------------------------------------------------
! calculate new fit errors
!-----------------------------------------------------------------------
      write (iout,*)
      if ( mod(nit,njump).eq.0 .or. nit.eq.niters  .and. lfiterr ) &
         call fiterr(ort,weight,vvec,oned,evec,&
                     coeff,cut,work2,buffer,wvec)
      iterr2=rmserr

      call stop_timer(tid)

      return
      end subroutine iteration


!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     returns via vvec the modified reference potential.
!     it is assumed that lcw=.true.
!
!     ort (i)
!     coeff (i)
!        coefficients contracted over mode 'modc'
!     evec (i)
!     oned (i)
!        vector containing the 1d potentials at the grid points of
!        every degree of freedom.
!     weight (i)
!        separable weights
!     work2 (w)
!        for the calculation of the fit
!     vvec (o)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine mrpot(ort,coeff,evec,oned,weight,wvec,cut,work2,vvec,buffer)

      implicit none

      real(dop), dimension(ortdim), intent(in)    :: ort
      real(dop), dimension(weightdim), intent(in) :: weight
      real(dop), dimension(oneddim), intent(in)   :: oned
      real(dop), dimension(modcdim), intent(out)  :: cut
      real(dop), dimension(evecdim), intent(in)   :: evec
      real(dop), dimension(coeffdim), intent(in)  :: coeff
      real(dop), dimension(workdim2), intent(out) :: work2
      real(dop), dimension(vpotdim), intent(out)  :: vvec
      real(dop), dimension(niobuf), intent(inout) :: buffer
      integer(long), dimension(vpotdim/30+1), intent(out)  :: wvec

      integer(long), dimension(maxdim)       :: igpoint
      integer(long) :: ccw,cw,k,f,voffset,woffset,n,m,imgp_modc
      real(dop), dimension(maxdim)           :: gpoint
      real(dop)                              :: vex,vfit,v,v1d,sw
      integer(long), save                    :: tid=0 ! timer ID

      call get_timer(tid, 'Modrefpot')
      call start_timer(tid)

      call loadv(ivpot,vvec,buffer)

      call initvxi(igpoint,ndof)

!     calculate cut
 5    imgp_modc=0
      do n=npotdof(modc),1,-1
         f=potdof(n,modc)
         imgp_modc=imgp_modc*gdim(f)+igpoint(f)-1
      enddo
      imgp_modc=imgp_modc+1
      call cuttxd2(coeff,cut,dimbef,dimmodc,dimaft,imgp_modc)

!     calculate offset from index vector
 10   voffset=0
      do m=npmode,1,-1
         do n=npotdof(m),1,-1
            f=potdof(n,m)
            voffset=voffset*gdim(f)+igpoint(f)-1
         enddo
      enddo
      woffset=voffset/30
      k=voffset-30*woffset
      voffset=voffset+1
      woffset=woffset+1

      v=vvec(voffset)
      call index2coord(igpoint,ort,gpoint)
      
      ccw=wvec(woffset)
      cw=ccw/2**k
      cw=mod(cw,2)
      if(facit .lt. 1.0001_dop) then ! Old version
         if (cw.eq.0) then
            call getvfit2(igpoint,evec,cut,work2,v)
         else
            call getv1d(igpoint,oned,v1d)
            call getsepwei(igpoint,weight,sw)
            v=(v-v1d)*sw
         endif
         vvec(voffset)=v
      else         ! Using speed-up factor
         call getv1d(igpoint,oned,v1d)
         call getsepwei(igpoint,weight,sw)
         vex=(v-v1d)*sw
         call getvfit2(igpoint,evec,cut,work2,vfit)
         vvec(voffset) = vfit + facit*cw*(vex-vfit) 
      endif
 
!     increase index vector components
!     for the dofs that do not belong to the contracted mode
      do m=1,npmode
         if (m.ne.modc) then
            do n=1,npotdof(m)
               f=potdof(n,m)
               if (igpoint(f).lt.gdim(f)) then
                  igpoint(f)=igpoint(f)+1
                  goto 10
               else
                  igpoint(f)=1
               endif
            enddo
         endif
      enddo
!     increase index vector components
!     for the dofs of the contracted mode, and update the cut
      do n=1,npotdof(modc)
         f=potdof(n,modc)
         if (igpoint(f).lt.gdim(f)) then
            igpoint(f)=igpoint(f)+1
            goto 5
         else
            igpoint(f)=1
         endif
      enddo

      call stop_timer(tid)
      return
      end subroutine mrpot

      end module iterationmod

