      module mpi_pf

use decimal, only:long,dop,vlong
use global
use pglobal
use globmemmod
use dvrdatmod
use griddatmod
use runpfitmod
use channels
use mpidata
use operdef
use combined
use lazyflush_m
use callmuldmod, only: callmuld
use directutil, only:index2coord
use logdat

#ifdef MPI
use mpi
use pdefaultmod, only: init_cmode
#endif

private
public :: pf_mpi_genpot, pf_mpi_init2, pf_mpi_genpot2, pf_mpi_init,&
          pf_mpi_slave

contains
!-----------------------------------------------------------------------
      subroutine pf_mpi_init
!-----------------------------------------------------------------------
! Transmit essential information from master to slaves,
! because the slaves didn't read the input file.
!-----------------------------------------------------------------------

#ifdef MPI

      implicit none

      logical(long), dimension(8) :: mpicomml
      integer(long), dimension(12) :: mpicommi

      if (mpirank.eq.0) then ! master
         if (.not.lmpi) return
         mpicomml(1)=lwrallevec
         mpicomml(2)=lextpot
         mpicomml(3)=lspenergy
         mpicomml(4)=lcutalong
         mpicomml(5)=lcutnpot
         mpicomml(6)=lwo
         mpicomml(7)=lwrcw
         mpicomml(8)=lcw
         call mpi_bcast(mpicomml,8,MPI_LOGICAL,0,MPI_COMM_WORLD,mpierr)
         mpicommi(1)=ndof
         mpicommi(2)=nmode
         mpicommi(3)=nstate
         mpicommi(4)=nproj
         mpicommi(5)=ortdim
         mpicommi(6)=dvrdim
         mpicommi(7)=maxgdim
         mpicommi(8)=npmode
         mpicommi(9)=modc
         mpicommi(10)=vpotfmt
         mpicommi(11)=maxdim
         mpicommi(12)=bufferlength
         call mpi_bcast(mpicommi,12,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)

      else ! slave
         lmpi = .true.
         call mpi_bcast(mpicomml,8,MPI_LOGICAL,0,MPI_COMM_WORLD,mpierr)
         lwrallevec=mpicomml(1)
         lextpot   =mpicomml(2)
         lspenergy =mpicomml(3)
         lcutalong =mpicomml(4)
         lcutnpot  =mpicomml(5)
         lwo       =mpicomml(6)
         lwrcw     =mpicomml(7)
         lcw       =mpicomml(8)
         call mpi_bcast(mpicommi,12,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
         ndof   =mpicommi(1)
         nmode  =mpicommi(2)
         nstate =mpicommi(3)
         nproj  =mpicommi(4)
         ortdim =mpicommi(5)
         dvrdim =mpicommi(6)
         maxgdim=mpicommi(7)
         npmode =mpicommi(8)
         modc   =mpicommi(9)
         vpotfmt=mpicommi(10)
         maxdim =mpicommi(11)
         bufferlength=mpicommi(12)
         allocate(gdim(maxdim))
         allocate(vgdim(maxdim))
         allocate(ngdim(maxdim))
         allocate(npotdof(maxdim))
         allocate(potdof(maxdim,maxdim))
         allocate(potdim(maxdim))
         allocate(psubdim(maxdim))
         allocate(basis(maxdim))
         allocate(ipbaspar(mbaspar,maxdim))
         allocate(vgmdim(maxdim))
         allocate(vgpdim(maxdim))
         allocate(zort(maxdim))
         allocate(zort1(maxdim))
         allocate(zmode(maxdim))
         allocate(zevec(maxdim))
         allocate(zweight(maxdim))
         allocate(wdim(maxdim))
         allocate(pdim(maxdim))
         allocate(c2v(maxdim-1))
         allocate(v2c(maxdim,6))
      endif

      call mpi_bcast(gdim, maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call mpi_bcast(vgdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call mpi_bcast(ngdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call mpi_bcast(npotdof,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call mpi_bcast(potdof,maxdim*maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call mpi_bcast(potdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call mpi_bcast(psubdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call mpi_bcast(basis,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call mpi_bcast(ipbaspar,mbaspar*maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)

      if (mpirank.gt.0) call init_cmode

#endif
      return
      end subroutine pf_mpi_init


!-----------------------------------------------------------------------
      subroutine pf_mpi_init2(ort)
!-----------------------------------------------------------------------
! Transmit more essential information from master to the dumb slaves.
!-----------------------------------------------------------------------
#ifdef MPI

      implicit none

      real(dop), dimension(ortdim), intent(in) :: ort
      integer(long), dimension(6)              :: mpicommi

      if (mpirank.eq.0) then ! master
         mpicommi(1)=fkoe
         mpicommi(2)=ftime
         mpicommi(3)=ippes
         mpicommi(4)=oneddim
         mpicommi(5)=maxhop
         mpicommi(6)=maxhpar
         call mpi_bcast(mpicommi,6,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)

      else ! slave
         call mpi_bcast(mpicommi,6,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
         fkoe   =mpicommi(1)
         ftime  =mpicommi(2)
         ippes  =mpicommi(3)
         oneddim=mpicommi(4)
         maxhop =mpicommi(5)
         maxhpar=mpicommi(6)
         allocate(hopfile(maxhop))
         allocate(hopipar(maxhpar,maxhop))
         allocate(hoprpar(maxhpar,maxhop))
      endif

!     Grid Information
      call mpi_bcast(zort,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call mpi_bcast(ort,ortdim,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,mpierr)

!     Operator Information
      call mpi_bcast(hopfile,maxhop,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call mpi_bcast(hopipar,maxhpar*maxhop,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call mpi_bcast(hoprpar,maxhpar*maxhop,MPI_DOUBLE_PRECISION,0, &
           MPI_COMM_WORLD,mpierr)

#else
      real(dop), dimension(*) :: ort
#endif
      return
      end subroutine pf_mpi_init2


!-----------------------------------------------------------------------
      subroutine pf_mpi_slave(task,mr)
!-----------------------------------------------------------------------

#ifdef MPI

      implicit none

      integer(long),intent(in)                  :: task
      real(dop), dimension(mrdim),intent(inout) :: mr

      if (task.eq.mpitask_pf_genpot) then
         if (vpotfmt.eq.2) then
            call pf_mpi_genpot2(mr(mrort))
         else
            call pf_mpi_genpot(mr(mrort),mr(mrbuf))
         endif

      else
         write (*,'(a,i0,a,i0,a,i0)') &
            'unknown mpitask=',task,' on rank ',mpirank,'/',nmpi
      endif

#else
      implicit none

      integer(long),intent(in)                  :: task
      real(dop), dimension(*),intent(inout)     :: mr
#endif
      return
      end subroutine pf_mpi_slave


!-----------------------------------------------------------------------
      subroutine pf_mpi_genpot(ort,buf)
!-----------------------------------------------------------------------
#ifdef MPI

      implicit none

      real(dop), dimension(ortdim), intent(in) :: ort
      real(dop), dimension(niobuf), intent(in) :: buf
      integer(long)                            :: f,r,i
      integer(long), dimension(nmpi-1)         :: chunk
      integer(long)                            :: chunksize,nchunk,nextchunk,nrcv
      integer(long)                            :: mychunk,nleft,length
      integer(vlong)                           :: i8vdim,i8tmp
      integer(long)                            :: mpitag
      integer(long), dimension(MPI_STATUS_SIZE) :: mpistat
      character(len=c5)                        :: fname

!     compute size of full grid
      i8vdim = 1
      do f=1,ndof
         i8vdim = i8vdim*gdim(f)
      enddo
!     check if grid size is too large
      i8tmp = 2
      if (i8vdim.ge.i8tmp**31) then
         routine = "pf_mpi_genpot"
         message = "full grid too large (integer overflow)"
         call errormsg
      else
         vpotdim = i8vdim
      endif
!     determine how many chunks there are
      chunksize = min(vpotdim,niobuf)
      nchunk = vpotdim/chunksize
      if (mod(vpotdim,chunksize).ne.0) nchunk=nchunk+1
!     if there are less chunks than slaves, make chunks smaller
      if (nchunk.lt.nmpi-1) then
         nchunk = nmpi-1
         chunksize = vpotdim/nchunk
         if (mod(vpotdim,nchunk).ne.0) chunksize=chunksize+1
      endif

      if (mpirank.eq.0) then
!------------------------------------------------------------
! MASTER: distribute tasks and collect data
!------------------------------------------------------------
!        write gridsize and chunksize
         write (ivpot) vpotdim,chunksize
         nextchunk = 1
!        write log
         write(ilog,'(a,i0,a,i0)') &
            "using ",nchunk," chunks of size ",chunksize
!        send initial tasks to all slaves
         do r=1,nmpi-1
            mpitag=0
            chunk(r)=nextchunk
            call mpi_send(chunk(r),1,MPI_INTEGER,r,mpitag,MPI_COMM_WORLD,mpierr)
            nextchunk=nextchunk+1
         enddo
!        receive requests from slaves (in order)
         nrcv = 0
         r = 1
         do while(nrcv.lt.nchunk)
            mpitag=chunk(r)
            call mpi_recv(buf,chunksize,MPI_DOUBLE_PRECISION,r, &
                 mpitag,MPI_COMM_WORLD,mpistat,mpierr)
!           process the data, i.e. write it into the data file
            nleft  = vpotdim - (chunk(r)-1)*chunksize
            length = min(nleft,chunksize)
            write (ivpot) (buf(i), i=1,length)
!           write log
            write(ilog,'(a,i0,a,i0,a,i0)') &
               "received from rank ",r," chunk ",chunk(r),"/",nchunk
            call lazyflush(ilog)
            nrcv=nrcv+1
            if (nextchunk.le.nchunk) then
!              if tasks left, send next task to slave
               chunk(r)=nextchunk
               nextchunk=nextchunk+1
            else
!              no tasks left: tell slave to finish
               chunk(r)=-1
            endif
            mpitag=0
            call mpi_send(chunk(r),1,MPI_INTEGER,r,mpitag,MPI_COMM_WORLD,mpierr)
            r=r+1
            if (r.eq.nmpi) r=1
         enddo

      else
!------------------------------------------------------------
! SLAVE: get task, compute and return data
!------------------------------------------------------------
         do while(.true.)
!           receive next task
            mpitag=0
            call mpi_recv(mychunk,1,MPI_INTEGER,0,mpitag,MPI_COMM_WORLD,&
                 mpistat,mpierr)
!           exit if no task to do
            if (mychunk.lt.0) exit
!           otherwise, compute data for this task
            call calcpot_chunk(mychunk,buf,chunksize,ort)
!           and send data back to master
            mpitag=mychunk
            call mpi_send(buf,chunksize,MPI_DOUBLE_PRECISION,0, &
                 mpitag,MPI_COMM_WORLD,mpierr)
         enddo

      endif

#else
      implicit none

      real(dop), dimension(*), intent(in) :: ort
      real(dop), dimension(*), intent(in) :: buf
#endif
      return
      end subroutine pf_mpi_genpot


!-----------------------------------------------------------------------
      subroutine pf_mpi_genpot2(ort)
!-----------------------------------------------------------------------
#ifdef MPI

      implicit none

      real(dop), dimension(ortdim), intent(in) :: ort
      integer(long)                            :: f,r,fd
      integer(long), dimension(nmpi-1)         :: chunk,request
      integer(long)                            :: chunksize,nchunk,nextchunk,nrcv
      integer(long)                            :: mychunk,myreq
      integer(vlong)                           :: i8vdim,i8tmp,i8offset,i8length
      integer(long)                            :: mpitag
      integer(long), dimension(MPI_STATUS_SIZE) :: mpistat
      character(len=c5)                        :: fname

      real(dop),allocatable :: sndbuf(:)
      real(dop),allocatable :: rcvbuf(:,:)

!     compute size of full grid
      i8vdim = 1
      do f=1,ndof
         i8vdim = i8vdim*gdim(f)
      enddo
!     determine how many chunks there are
      chunksize = bufferlength*1024*128 
      i8tmp  = chunksize
      nchunk = i8vdim/i8tmp
      if (mod(i8vdim,i8tmp).ne.0) nchunk=nchunk+1
!     if there are less chunks than slaves, make chunks smaller
      if (nchunk.lt.nmpi-1) then
         nchunk = nmpi-1
         i8tmp  = nchunk
         chunksize = i8vdim/i8tmp
         if (mod(i8vdim,i8tmp).ne.0) chunksize=chunksize+1
      endif

      if (mpirank.eq.0) then
!------------------------------------------------------------
! MASTER: distribute tasks and collect data
!------------------------------------------------------------
         allocate(rcvbuf(chunksize,nmpi-1))
!        open data file for writing (truncating)
         fname = trim(name)//"/vpot2.raw"
         call c_open(fd, trim(fname), 2)
         nextchunk = 1
!        write log
         write(ilog,'(a,i0,a,i0)') &
            "using ",nchunk," chunks of size ",chunksize
!        send initial tasks to all slaves
         do r=1,nmpi-1
            mpitag=0
            chunk(r)=nextchunk
            call mpi_send(chunk(r),1,MPI_INTEGER,r,mpitag,MPI_COMM_WORLD,mpierr)
            nextchunk=nextchunk+1
         enddo
!        initiate receive requests from all slaves
         do r=1,nmpi-1
            call mpi_irecv(rcvbuf(1,r),chunksize,MPI_DOUBLE_PRECISION,r, &
                 chunk(r),MPI_COMM_WORLD,request(r),mpierr)
         enddo
         nrcv = 0
!        loop until all chunks received
         do while(nrcv.lt.nchunk)
!           wait until any slave returns data
            call mpi_waitany(nmpi-1,request,r,mpistat,mpierr)
!           process the data, i.e. write it into the data file
            i8tmp = chunk(r)-1
            i8offset = i8tmp*chunksize
            i8tmp = chunksize
            i8length = min(i8vdim-i8offset,i8tmp)
            call c_write_at(fd,rcvbuf(1,r),8,i8offset,i8length)
!           write log
            write(ilog,'(a,i0,a,i0,a,i0)') &
               "received from rank ",r," chunk ",chunk(r),"/",nchunk
            call lazyflush(ilog)
            nrcv=nrcv+1
!           if there are tasks left...
            if (nextchunk.le.nchunk) then
!              ... send next task to slave and ...
               chunk(r)=nextchunk
               mpitag=0
               call mpi_send(chunk(r),1,MPI_INTEGER,r,mpitag,MPI_COMM_WORLD,&
                    mpierr)
               nextchunk=nextchunk+1
!              ... initiate receive request from that slave
               call mpi_irecv(rcvbuf(1,r),chunksize,MPI_DOUBLE_PRECISION, &
                    r,chunk(r),MPI_COMM_WORLD,request(r),mpierr)
            else
!              no tasks left: tell slave to finish
               chunk(r)=-1
               mpitag=0
               call mpi_send(chunk(r),1,MPI_INTEGER,r,mpitag,MPI_COMM_WORLD,&
                    mpierr)
            endif
         enddo
!        clean up
         call c_close(fd)
         deallocate(rcvbuf)

      else
!------------------------------------------------------------
! SLAVE: get task, compute and return data
!------------------------------------------------------------
         allocate(sndbuf(chunksize))
         do while(.true.)
!           receive next task
            mpitag=0
            call mpi_recv(mychunk,1,MPI_INTEGER,0,mpitag,MPI_COMM_WORLD,&
                 mpistat,mpierr)
!           exit if no task to do
            if (mychunk.lt.0) exit
!           otherwise, compute data for this task
            call calcpot_chunk(mychunk,sndbuf,chunksize,ort)
!           and send data back to master
            mpitag=mychunk
            call mpi_isend(sndbuf,chunksize,MPI_DOUBLE_PRECISION,0, &
                 mpitag,MPI_COMM_WORLD,myreq,mpierr)
            call mpi_wait(myreq,mpistat,mpierr)
         enddo
!        clean up
         deallocate(sndbuf)

      endif

#else
      implicit none

      real(dop), dimension(*), intent(in) :: ort

#endif
      return
      end subroutine pf_mpi_genpot2


!-----------------------------------------------------------------------
      subroutine calcpot_chunk(chunk,vbuf,chunksize,ort)
!-----------------------------------------------------------------------

      implicit none

      real(dop), dimension(ortdim), intent(in) :: ort
      integer(long)                            :: f,j
      integer(long)                            :: chunk,chunksize
      integer(vlong)                           :: i8pos,i8tmp,i8dim,i8rem
      integer(long), dimension(maxdim)         :: igpoint
      real(dop), dimension(maxdim)             :: gpoint
      real(dop), dimension(chunksize)          :: vbuf
      character(len=c1)                        :: label

!     get starting position of this chunk
      i8tmp = chunk-1
      i8pos = i8tmp*chunksize
!     turn position into index vector
      i8rem = i8pos
      do f=1,ndof
         i8dim = gdim(f)
         igpoint(f) = mod(i8rem,i8dim)+1
         i8rem = i8rem/i8dim
      enddo
      j=0

!     compute potential at grid point
 10   j=j+1
      call index2coord(igpoint,ort,gpoint)
      call callmuld(vbuf(j),gpoint,ndof,ippes,hopfile(1), &
           hopipar(1,1),hoprpar(1,1),1,1,label)
!     if buffer is full, leave
      if (j.eq.chunksize) goto 90

!     goto next grid point
      do f=1,ndof
         if (igpoint(f).lt.gdim(f)) then
            igpoint(f) = igpoint(f)+1
            goto 10
         else
            igpoint(f) = 1
         endif
      enddo

 90   return
      end subroutine calcpot_chunk

      end module mpi_pf
