!***********************************************************************
      module pdefaultmod

use decimal, only:long,dop

      implicit none
      private
      public :: pdefault,init_cmode
      contains

!***********************************************************************
!
!            POTFIT subroutine PDEFAULT
!
!  11/97 AJ
!
!***********************************************************************

      subroutine pdefault

use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use versions
use op1lib, only: zerovxi, zeromxd, zerovxl, zeromxi
use logdat, only: errormsg, routine, message

      implicit none

      integer(long) :: ierror

!-----------------------------------------------------------------------
! Initialize some variables
!-----------------------------------------------------------------------
      logisopen = .false.
      lwarn = .false.
      lausw = .false.
      niw   =  0
      sstop = ' '
      scpu  = ' '
      lwpot = .false.
      lwrcw = .false.
      lwmap = .true.
      vpotfmt = 1

!---------------------------------------------------------------------
! Define program name.
!---------------------------------------------------------------------
! GET THE ZEROTH ARGUMENT (name of routine as called from bash)
      call mygetarg(0,progname)
      call rempath(progname,lprogname,ierror)
      if(ierror.eq.1) then
         routine='pdefault'
         message=progname//' is a directory, not a file name.'
         call errormsg
      endif

!-----------------------------------------------------------------------
! set by program flags
!-----------------------------------------------------------------------
      lshort=.false.
      lchkmc=.true.
      
!-----------------------------------------------------------------------
! read by subroutine peinrun
!-----------------------------------------------------------------------
      lit     = .false.
      litia   = .false.
      lsw     = .false.
      lsp     = .false.
      lwo     = .false.
      lwef    = .false.
      lreadvpot = .false.
      lgenvpot  = .false.
      ldelvpot  = .false.
      lprojerr  = .false.
      lprojascii = .false.
      lprojlowmem= .false.
      niters  = 0
      njump   = 1
      fitcount= 0
      npest   = 0
      nproj   = 0
      facit   = 1.0_dop
      bufferlength = 64
      ifinemode = 0

!-----------------------------------------------------------------------
! read by subroutines popenin and peinfile
!-----------------------------------------------------------------------
      lcont    = .false.
      lovwr    = .false.
      loutfil  = .true.
      loutl    = .false.
      lprodw   = .false.
      litf     = .false.
      ltime    = .true.
      lgit     = .false.

!-----------------------------------------------------------------------
! potfit computes single point energy or cut along the PES
!-----------------------------------------------------------------------
      lspenergy = .false.
      lcutalong = .false.

!-----------------------------------------------------------------------
! potfit reads pes file
!-----------------------------------------------------------------------
      lpesfile      = .false.

!-----------------------------------------------------------------------
!  MGPF options
!-----------------------------------------------------------------------
      lmgpf        = .false.
      lextpot      = .false.
      lprintrho    = .false.
      lfinemode    = .false.
      lonlydcoeff  = .false.
      lfullcoarse  = .false.
      ldcoeff      = .true.
      lfiterr      = .true.
      lwrallevec   = .false.
      lctensor     = .false.
      lcutnpot     = .false.
      lrdnpot      = .false.
      lnpotpath    = .false.

!-----------------------------------------------------------------------
!  MCPF/MCCPD options
!-----------------------------------------------------------------------
      lmcpf        = .false.
      lmccpd       = .false.

!-----------------------------------------------------------------------
! read by subroutines popenin and peinoper
!-----------------------------------------------------------------------
      lcspot = .false.
      jtot   = -2d0
      jbf    = 0d0
      csmass = 1d0
      ploned  = .false.

      npmode=0
      ndof=0
      lpconm=.false.

!-----------------------------------------------------------------------
! needed for cutoff of PES surface
!-----------------------------------------------------------------------
      lsoftcutpes=.false.
      lcutpes=.false.
      pesmin=-1.0d20
      pesmax= 1.0d20

!-----------------------------------------------------------------------
! For include file operdef.inc
!-----------------------------------------------------------------------
      hoplab0=' + '

!--------------------------------------------------------------------------
!   coordinate transformation variables
!--------------------------------------------------------------------------
      nbvec   = 0
      nbdumm  = 0
      tord    = 0
      cord    = 0
      icart   = 0
      frozen  = 0
      nbfr    = 0
      at1     = ' '
      at2     = ' '
      pfroz   = 0.0
      vecwei  = 0.0
      lfr     = .false.
      ltr     = .false.

      return
      end subroutine pdefault

!-----------------------------------------------------------------------
! set variables of common `cmode'
!-----------------------------------------------------------------------

      subroutine init_cmode

use global
use griddatmod
use operdef
use runpfitmod
use combined

      implicit none

      integer(long)       :: m
      
      nodbef=0
      nodaft=0
      dimbef=1
      dimmodc=psubdim(modc)
      dimaft=1
      do m=1,npmode
         if (m.lt.modc) then
            nodbef=nodbef+1
            dimbef=dimbef*potdim(m)
         elseif (m.gt.modc) then
            nodaft=nodaft+1
            dimaft=dimaft*potdim(m)
         endif
      enddo

      return
      end subroutine init_cmode


      end module pdefaultmod

