!#######################################################################
!
!        POTFIT module PEINBAS
!
!  Contains subroutines: peinbas,dimsphfbr
!
!  11/97 AJ
!
!#######################################################################

      module peinbasmod

use decimal, only:long,dop
use global
use pglobal
use maxv
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels

use logdat
use op1lib

      implicit none
      private
      public :: peinbas,peinvec

      contains

!***********************************************************************
!
!   POTFIT subroutine PEINBAS
!
!  11/97 AJ
!***********************************************************************

      subroutine peinbas

      implicit none

      integer(long)                :: i,iz0,ic,ierr,f,f1,ilbl,n,m,zeig
      integer(long)                :: iz
      logical(kind=4)              :: linperr
      integer(long), save          :: ncallpeinbas=0

!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
      iz0 = 0
      routine = 'read NATPOT-BASIS-SECTION (peinbas)'

      call zerovxi(subdim,maxdim)
      call zerovxi(psubdim,maxdim)
      call zeromxi(spfdof,maxdim,maxdim)
      call zerovxi(dofspf,maxdim)
      call zerovxi(nspfdof,maxdim)
      call zeromxi(potdof,maxdim,maxdim)
      call zerovxi(dofpot,maxdim)
      call zerovxi(npotdof,maxdim)
      call zerovxi(potdim,maxdim)

      npmode=0
      lpconm=.false.

!-----------------------------------------------------------------------
!     if write-only: create dummy natpot-basis data
!-----------------------------------------------------------------------
      if (lwo) then
         npmode = ndof
         do f=1,ndof
            npotdof(f) = 1
            potdof(1,f) = f
            potdim(f) = 1
         enddo
         modc = 1
         return
      endif

!-----------------------------------------------------------------------
! read down to natpot-basis-section
!-----------------------------------------------------------------------
      rewind(iin)
      iz=0
      do 

         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .eq. 1) then
            linperr = .true.
            routine = 'peingabe (natpot-basis-section)'
            message = inptit
            call errormsg
         endif

         if (keyword(1) .eq. 'natpot-basis-section') then
            exit
         else if (keyword(1) .eq. 'end-input') then
            routine = 'peingabe (natpot-basis-section)'
            message='No NATPOT-BASIS-SECTION in input file'
            call errormsg
         endif
      enddo

!-----------------------------------------------------------------------
! read natural potential basis - modelabels are assumed to have been 
! already set        
!-----------------------------------------------------------------------
      m=0
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)
      
      i=1
   15 continue
      if (keyword(i) .eq. 'end-natpot-basis-section' ) then
         go to 60

      else
         m=m+1 

   40 continue      
         f=0
         do f1=1,ndof
            if (keyorig(i)(1:12) .eq. modelabel(f1) ) f=f1
         enddo
         if (f.eq. 0) then
            message = 'no primitive basis for mode: '
            call wrline(i,iz,iz0)
            f=ndof+1
         endif

         if (f.gt.maxdim) then
            message = ' MAXDIM in maxdim.inc must be increased to :'
            ilbl=index(message,':')+2
            write(message(ilbl:ilbl+4),'(i5)') f
            call warnmsg
         endif

!--------------------------------------------------------------------- 
! Variables containing the information about the combined
! degrees of freedom in potfit
!----------------------------------------------------------------------

! MGPF cutnpot: I will repeat this only once...
         if(ncallpeinbas.eq.0)then
           npotdof(m)=npotdof(m)+1
           n=npotdof(m)
           potdof(n,m)=f
           dofpot(f)=m
         endif

         if (keyword(i+1).eq.'=') then
            if (keyword(i+2).eq.'contr') then
               if (modc.eq.0) then
                  modc = m
               else
                  message ='only one dof can be contracted'
                  call wrline (zeroi,iz,iz0)
               endif
               i=i+2
               if (keyword(i+1).eq.',') then
                  read(keyword(i+2),*,iostat=ierr) potdim(m)
                  if (ierr .ne. 0) then
                     message ='potdim must be an integer. Not :'
                     call wrline (i+2,iz,iz0)
                  endif
                  i=i+2
               endif
            else
               read(keyword(i+2),*,iostat=ierr) potdim(m)
               if (ierr .ne. 0) then
                  message ='potdim must be an integer. Not :'
                  call wrline (i+2,iz,iz0)
               endif
               i=i+2
               if (keyword(i+1).eq.',') then
                  if (keyword(i+2).eq.'contr') then
                     if (modc.eq.0) then
                        modc = m
                     else
                        message ='only one mode can be contracted'
                        call wrline (zeroi,iz,iz0)
                     endif
                  else
                     message ='unknown argument:'
                     call wrline (i+2,iz,iz0)
                  endif
                  i=i+2
               endif
            endif

         else if (keyword(i+1).eq.',') then
            i=i+2
            go to 40
         endif
   
      endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

!.....'end-natpot-basis-section' has been found.
   60 continue

!.....number of modes
      npmode=m

!-----------------------------------------------------------------------
! see if any degrees of freedom  are combined
!-----------------------------------------------------------------------
      do m=1,npmode
         if (npotdof(m) .gt. 1) lpconm = .true.
      enddo

!-----------------------------------------------------------------------
! calculate psubdim (number of grid points) for each mode.
!-----------------------------------------------------------------------

! MGPF cutnpot: I will repeat this only once...
      if(ncallpeinbas.eq.0)then
        do m=1,npmode
          zeig=1
          do n=1,npotdof(m)
            f=potdof(n,m)
            zeig=zeig*gdim(f)
          enddo
          psubdim(m)=zeig
        enddo
        ncallpeinbas = 1
      endif

      do m = 1, npmode
         if(potdim(m) .gt. psubdim(m) ) then
            linperr=.true.
            write(6,'(/2x,a,i3,a,i4,a,i4)') &
            'm =', m,',  potdim =',potdim(m),',  psubdim =',psubdim(m)
            Message = ' potdim .gt. psubdim '
            call warnmsg
         end if
      enddo

!     alternative mapping
      do m=1,npmode
         do n=1,npotdof(m)
            f = potdof(n,m)
            potdofmap(1,f) = m
            potdofmap(2,f) = n
         end do
      end do

      nmode=npmode
      potdim(modc)=psubdim(modc)
      do m=1,npmode
        pdim(m) = potdim(m)
      enddo

      if (.not.linperr) linperr=lwarn
      lwarn=.false.

      return

      end subroutine peinbas

!#######################################################################
!-----------------------------------------------------------------------
!    reads the vectors in transformation section
!-----------------------------------------------------------------------

      subroutine peinvec

      implicit none

!-----------------------------------------------------------------------
!     vc counts over the vectors whose information (nbvec, nbdumm, 
!     vecwei, at1, at2) is stored
!     itr counts over the number of lines in the transformation section
!-----------------------------------------------------------------------
      integer(long)   :: ierr,i,ic,vc,itr,iz
      logical(kind=4) :: linperr

!-----------------------------------------------------------------------
! read down to transformation-section
!-----------------------------------------------------------------------
      rewind(iin)
      iz=0
      do 

         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .eq. 1) then
            linperr = .true.
            routine = 'peingabe (transformation-section)'
            message = inptit
            call errormsg
         endif

         if (keyword(1) .eq. 'transformation-section') then
            exit
         else if (keyword(1) .eq. 'end-input') then
            routine = 'peingabe (natpot-basis-section)'
            message='No TRANSFORMATION-SECTION in input file'
            call errormsg
         endif
      enddo

      vc=1
      itr= 1
 20   continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      if (keyword(1) .eq. 'end-transformation-section' ) then
         go to 80
      elseif(keyword(1).eq.'order'.and.keyword(2).ne.'=') then
         write(ilog,*) 'No arguments specified with "order" keyword'
      elseif(keyword(1).eq.'order') then
         do i=1, ndof
            read (keyword(i+2),*) tord(i)
         end do
         itr=itr+1
         goto 20
      elseif(keyword(1).eq.'cart'.and.keyword(2).ne.'=') then
         write(ilog,*) 'No arguments specified with "cart" keyword'
      elseif(keyword(1).eq.'cart') then
      read (keyword(3),*) icart(1)
         itr=itr+1
      goto 20

      elseif (keyword(1).eq.'fro'.and.keyword(2).ne.'=') then
         write(ilog,*) 'No arguments specified with "fro" keyword'
      elseif(keyword(1).eq.'fro') then
         lfr= .true.
         read(keyword(3),*) frozen(nbfr+1)
         read(keyword(4),*) pfroz(nbfr+1)
         nbfr=nbfr+1
         itr= itr+1
         goto 20

      elseif (keyword(1).eq.'cord'.and.keyword(2).ne.'=') then
        write(ilog,*) 'No arguments specified with "cord" keyword'
      elseif(keyword(1).eq.'cord') then
         do i=1,ndof/3+3
            read (keyword(i+2),*) cord(i)
         end do
         itr=itr+1
         goto 20
      else
         read(keyword(1),*) nbvec(vc)
         read(keyword(2),*) nbdumm(vc)
         read(keyword(3),*) vecwei(vc)
         at1(vc)    = trim(keyword(4))
         at2(vc)    = trim(keyword(5))
         vc=vc+1
         itr=itr+1
      goto 20
      end if
 80   continue

      return
      end subroutine peinvec

      end module peinbasmod

