!***********************************************************************
!
!            POTFIT subroutine PEINCORWEI
!
!***********************************************************************

      subroutine peincorwei (linperr,iz)

use decimal, only:long,dop
use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels

use logdat
use keyunitsmod

      implicit none

      integer(long)                :: i,iz0,f,f1,ic,ierr
      integer(long), intent(in)    :: iz
      real(dop)                    :: xx
      logical(kind=4), intent(out) :: linperr

!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
      routine = 'read CORRELATED-WEIGHT-SECTION (peincorwei)'
      iz0=0

      iwvmax  = 1.0e+33_dop
      iwvmin  = -1.0e+33_dop
      xxmax(:) =  1.0e+33_dop
      sxmax(:) =  1.0e+33_dop
      cxmax(:) =  1.0e+33_dop
      xxmin(:) = -1.0e+33_dop
      sxmin(:) = -1.0e+33_dop
      cxmin(:) = -1.0e+33_dop

 10   continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1

 15   continue
      if (keyword(i).eq.'end-correlated-weight-section') then
         goto 20
      elseif (keyword(i).eq.'end-input') then 
         message = &
          'No "end-correlated-weight-section" keyword at end of ' //&
          'CORRELATED-WEIGHT-SECTION'
         call warnmsg
         linperr=lwarn
         lwarn=.false.
         return

      elseif (keyword(i).eq.'v') then
         if (keyword(i+1).eq.'='  .or.&
             keyword(i+1).eq.'<') then
            read(keyword(i+2),*,iostat=ierr) iwvmax
            if (ierr .ne. 0) then
               message ='v must be a real number. Not :'
               call wrline (i+2,iz,iz0)
            endif
            i=i+2
            if (keyword(i+1).eq.',' ) then
               i = i+2
               call keyeunits(iwvmax,keyword(i),ierr)
               if (ierr .eq. 1) then
                  message = 'no energy unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
         else if (keyword(i+1).eq.'>') then
            read(keyword(i+2),*,iostat=ierr) iwvmin
            if (ierr .ne. 0) then
               message ='v must be a real number. Not :'
               call wrline (i+2,iz,iz0)
            endif
            i=i+2
            if (keyword(i+1).eq.',' ) then
               i = i+2
               call keyeunits(iwvmin,keyword(i),ierr)
               if (ierr .eq. 1) then
                  message = 'no energy unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            endif
            
         endif

      elseif (keyword(i).eq.'readcw'  .or. keyword(i).eq.'read-cw' .or. &
              keyword(i).eq.'read_cw') then
         lwpot=.true.
         if (keyword(i+1).eq.'=') then
            read(keyorig(i+2),'(a)',iostat=ierr) wname
            if (ierr .ne. 0) then
                message = 'error reading file name: readcw='
                call wrline(zeroi,iz,iz0)
            endif
            i=i+2
            wlength=index(wname,' ')-1
            call abspath(wname,wlength)
            if (keyword(i+1).eq.',') then
               i=i+2
               if(keyorig(i) .eq. 'map') then
                  lwmap=.true.
               elseif(keyorig(i) .eq. 'nomap' .or. &
                         keyorig(i) .eq. 'no-map') then
                  lwmap=.false.
               else
                  message ='Expecting "map" or "no-map" but not:'
                  call wrline(i,iz,iz0)
               endif
            endif
         else
            message ='readcw must be followed by =<path>'
            call wrline(zeroi,iz,iz0)
         endif

      else 

         f=0
         do f1=1,ndof
            if (keyorig(i)(1:12) .eq. modelabel(f1) ) f=f1
         enddo
         if (f .eq. 0) then
            message = ' unknown keyword : '
            call wrline (i,iz,iz0)
         endif

         if( keyword(i+1) .eq. '<' ) then
            i=i+2
            read(keyword(i),*,iostat=ierr) xx
            if (ierr .ne. 0) then
               message ='xxmax must be a real number. Not :'
               call wrline (i+2,iz,iz0)
            endif
            if( keyword(i+1) .eq. ',' ) then
               i=i+2
               if( keyword(i) .eq. 'cos' ) then
                  cxmax(f) = xx
               else if( keyword(i) .eq. 'sin' ) then
                  sxmax(f) = xx
               else
                  call keylunits(xx,keyword(i),ierr)
                  if (ierr .eq. 1) then
                     message = 'no length unit after comma'
                     call wrline(zeroi,i,iz0)
                  endif
                  xxmax(f) = xx
               end if
            else
               xxmax(f) = xx
            end if

         else if( keyword(i+1) .eq. '>' ) then
            i=i+2
            read(keyword(i),*,iostat=ierr) xx
            if (ierr .ne. 0) then
               message ='xxmin must be a real number. Not :'
               call wrline(i+2,iz,iz0)
            endif
            if( keyword(i+1) .eq. ',' ) then
               i=i+2
               if( keyword(i) .eq. 'cos' ) then
                  cxmin(f) = xx
               else if( keyword(i) .eq. 'sin' ) then
                  sxmin(f) = xx
               else
                  call keylunits(xx,keyword(i),ierr)
                  if (ierr .eq. 1) then
                     message = 'no length unit after comma'
                     call wrline(zeroi,i,iz0)
                  endif
                  xxmin(f) = xx
               end if
            else
               xxmin(f) = xx
            end if

            if (ierr .ne. 0) then
               message ='xxmin must be a real number. Not :'
               call wrline(i+2,iz,iz0)  
            endif
         else 
            message ='Expecting "<" or ">". Not :'
            call wrline(i+1,iz,iz0)
         endif
         i=i+2
      endif
      
!-----------------------------------------------------------------------
!  check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

!-----------------------------------------------------------------------
! Set 'linperr=true' if a warnmsg occurred
!-----------------------------------------------------------------------
   20 if (.not.linperr) linperr=lwarn
      lwarn=.false.     
 
      return
      end subroutine peincorwei
