!***********************************************************************
!
!            POTFIT  subroutine  PEINGABE
!
! Reads the input data from the input file.
!
! Called from: potfit
! Calls to: popenin, popenlog,rdinpf,peinfile,peinweight,peinbas,  
!           ploginp,popendat,pausgab,abspath,warnmsg,errormsg,myflush
!
! 11/97 HDM
!
!***********************************************************************

      subroutine peinfit 

use decimal, only:long,dop
use global
use pglobal
use maxv, only:maxdim
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels
use mpidata
#ifdef MPI
   use mpi
#endif

use op1lib, only: initvxl
use peingutil, only:popenin,popenlog,popendat,pausgab
use logdat
use rddvrmod, only:dvrinfo
use einpbasmod, only:einpbas

use potmod_interface

      implicit None

      integer(long)                            :: i,iz,ic,ierr,m,n,f
      logical(kind=4), dimension(maxdim)       :: modelist
      logical(kind=4)                          :: lghost
      logical(kind=4)                          :: linperr

!-----------------------------------------------------------------------
! read separable-weight-section 
!-----------------------------------------------------------------------
      rewind(iin)
      iz=0
 610  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine = 'peingabe (separable-weight-section)'
         message = inptit
         call warnmsg
      endif

      if (keyword(1) .eq. 'separable-weight-section') then
         lsw = .true.
         call peinsepwei(linperr,iz)
      else if (keyword(1) .eq. 'end-input') then
         lprodw = .false.
         go to 620
      else
         go to 610
      endif
 620  continue

!-----------------------------------------------------------------------
! read correlated-weight-section 
! Read correlated-weight-section. Do not read corr-weights if there
! are no iterations and no writing of weights to file.
!-----------------------------------------------------------------------
      if(.not.lit .and. .not.lwrcw .and. .not.lrdnpot) goto 720
      rewind(iin)
      iz=0
 710  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine = 'peingabe (correlated-weight-section)'
         message = inptit
         call warnmsg
      endif

      if (keyword(1) .eq. 'correlated-weight-section') then
         lcw = .true.
         call peincorwei(linperr,iz)
      else if (keyword(1) .eq. 'end-input') then
         goto 720
      else
         go to 710
      endif
  720 continue

!-----------------------------------------------------------------------
! Checks for consistency 
!-----------------------------------------------------------------------
      if( lreadvpot .and. lgenvpot ) then
         routine = 'peingabe  (Check for !onsistency)'
         message='One cannot give "readvpot" (-rv) AND "genvpot" (-gv)'
         call warnmsg
      endif
      if (modc.eq.0) then
         routine = 'peingabe  (Check for consistency)'
         message = 'No mode has been contracted (NATPOT-BASIS-SECTION)'
         call warnmsg
      endif
      call initvxl(modelist,ndof)
      do m=1,npmode
          do n=1,npotdof(m)
             f=potdof(n,m)
             modelist(f)=.false.
         enddo
      enddo
      do f = 1, ndof
         if(basis(f).eq.0) then
            routine = 'peingabe (consistency check)'
            message = 'There must not be an electronic basis in the'&
                    //' primitive-basis-section of a potfit run.'
            call errormsg
         endif
         if(basis(f).eq.6) then
            message = 'sphFBR is not allowed for POTFIT (not a grid ' &
            //'representation). See "Remarks on KLeg, PLeg and sphFBR:" ' &
            //'on HTML-docu (Input Documentation/Primitive-Basis-Section).'
            call errormsg
         endif
      enddo
      lghost=.false.
      do f = 1, ndof
         lghost=lghost.or.modelist(f)
      enddo
      if(lghost) then
         i=6
 35      write(i,'(4(/a))') ' ## WARNING : There are unused DOFs.',&
         ' One better remove those from the Primitive-Basis-Section,',&
         ' as this may lead to problems, if the natpot is not the only',&
         ' operator in a Hamitonian-Section line.'
         do f = 1, ndof
            if(modelist(f)) write(i,'(2a)') ' Modelabel:  ',modelabel(f)
         enddo
         write(i,*)
         if(i.ne.ilog) then
            i=ilog
            goto 35
         endif
      endif
      if (vpotfmt.eq.2) then
         if (.not.lwo) then
            message = "not in write-only mode - vpot-format=2 ignored"
            call warnmsg
            vpotfmt=1
         elseif (.not.lmpi) then
            message = "vpot-format=2 requires usempi"
            call errormsg
         endif
      endif
      if (lmpi) then
         if (.not.lwo) then
            message = "usempi currently only works with write-only"
            call errormsg
         elseif (nmpi.lt.2) then
            message = "write-only/usempi needs at least 2 MPI processes"
            call errormsg
         endif
      endif

      return

      end subroutine peinfit
