!***********************************************************************
!
!            POTFIT  subroutine  PEINGABE
!
! Reads the input data from the input file.
!
! Called from: potfit
! Calls to: popenin, popenlog,rdinpf,peinfile,peinweight,peinbas,  
!           ploginp,popendat,pausgab,abspath,warnmsg,errormsg,myflush
!
! 11/97 HDM
!
!***********************************************************************

      subroutine peingabe (linperr)

use decimal, only:long,dop
use global
use pglobal
use griddatmod
use operdef
use runpfitmod
use combined
use channels
use mpidata
#ifdef MPI
   use mpi
#endif

use op1lib, only: initvxl
use peingutil, only:popenin,popenlog,popendat,pausgab
use logdat
use rddvrmod, only:dvrinfo
use einpbasmod, only:einpbas

use potmod_interface

      implicit None

      integer(long)                            :: iz,ic,ierr,ilbl
      logical(kind=4)                          :: lmnd,lrddvr
      logical(kind=4), intent(out)             :: linperr
      character(len=c5)                        :: filename

!-----------------------------------------------------------------------
! open the inputfile : argument.inp, reads options.
!-----------------------------------------------------------------------
      call popenin(lmnd,lrddvr)

!-----------------------------------------------------------------------
! read run-section and open log file
!-----------------------------------------------------------------------
      rewind(iin)
      iz=0
   10 continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey) 
      if (ierr .eq. 1) then
         linperr = .true.
         routine = 'peingabe (run-section)'
         message = inptit
         call errormsg
      endif

      if (keyword(1) .eq. 'run-section') then
         call peinrun(linperr,iz)
      else if (keyword(1) .eq. 'end-input') then
         routine = 'peingabe (run-section)'
         message='No RUN-SECTION in input file'
         call warnmsg
      else
         go to 10
      endif

!-----------------------------------------------------------------------
! look for PROJECTION-SECTION
!-----------------------------------------------------------------------
      rewind(iin)
      lprojection=.false.
      do
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .eq. 1) then
            linperr = .true.
            routine = 'peingabe (run-section)'
            message = inptit
            call errormsg
         endif

         if (keyword(1) .eq. 'projection-section') then
            lprojection=.true.
         else if (keyword(1) .eq. 'end-input') then
            exit
         endif
      enddo

!     implied options
      if (lprojection .and. lmpi) lprojlowmem=.true.

!-----------------------------------------------------------------------
! Make changes to input based on command-line options
!-----------------------------------------------------------------------
      if (lrddvr) then
          irundvr = 2
      else
          irundvr = 1
      endif

!-----------------------------------------------------------------------
! Check that the name directory exists.
! The inquire command does not work on DECs if applied to directories.
! We thus open a scratch file to find out whether the name-directory
! exists.
!-----------------------------------------------------------------------
      open(itmpin,file=name(1:laenge)//'/scratch',err=666)
      close(itmpin,status='delete')
      if(lmnd) write(6,'(2a)') 'Name-directory already exist: ',&
                                name(1:laenge)
      goto 667

 666  if(lmnd) then
         inptit='mkdir '//name(1:laenge)//char(zeroi)
         call excmd2(ierr, inptit)
         if (ierr.ne.0) then
            write(6,'(2a)') '### ERROR: ',&
            'Invalid path for name-directory (-mnd option). ###'
            call stopnow
         endif
         write(6,'(2a)') 'Created name-directory: ',name(1:laenge)
      else
         write(6,'(a)')&
         '---------------------------------------------------------'
         write(6,'(a)') ' ################   ERROR   ##################'
         write(6,'(a)') ' There is no name-directory! The directory:'
         write(6,'(1x,a)') name(1:laenge)
         write(6,'(a)') ' does not exist! (or has no write permission)'
         write(6,'(2a)')' One may use the option -mnd to create the',&
                        ' name-directory.'
         write(6,'(a)')&
           '-------------------------------------------------------'
         if (.not.lshort) call stopnow
      endif

 667  if (.not.lshort) then
         filename=name(1:laenge)//'/log'
         if(lrdnpot) filename=name(1:laenge)//'/rdnpot_log'
         call popenlog(filename)
      endif

!-----------------------------------------------------------------------
! open data files and copy input file to data directory
!-----------------------------------------------------------------------
      if (.not.lshort) then
         call popendat
         call pausgab
      end if

!-----------------------------------------------------------------------
! Set 'linperr=true' if a warnmsg occurred
!-----------------------------------------------------------------------
      if (.not.linperr) linperr=lwarn
      lwarn=.false.

      return
 1000 continue
      ilbl=index(filename,' ')-1
      routine = 'peingabe'
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg

      end subroutine peingabe
