!#######################################################################
!                                                                     
!              POTFIT  module PEINGUTIL
!                                                                     
! POTFIT module containing utility subroutines for EINGABE to open 
! and close the input and  output files and to read the input data.  
!
!  includes the subroutines: popenin, popenlog, popendat, pclosedat, 
!                            loginp, pausgab, wrphelptext.
!                            wrpjhelptext
!
!  11/97 HDM
!
!
!#######################################################################
      module peingutil

use decimal, only:long
use logdat
use versions
use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use channels

use module_interface

      implicit none
      private
      public :: popenin,popendat,pclosedat,popenlog,pausgab,wrphelptext
      contains

!***********************************************************************
!
!                  Subroutine  POPENIN
!
! Opens the input file  file.inp.
!
! Called from: peingabe
! Calls to:    None
!
! V7.0  11/97
!***********************************************************************

      subroutine popenin(lmnd,lrddvr)

      implicit none

      integer(long)                   :: i,ierr,length
      logical(kind=4)                 :: linpf,lexist
      logical(kind=4), intent(out)    :: lmnd,lrddvr
      character(len=8)                :: flag
      character(len=80)               :: buf

!-----------------------------------------------------------------------
! set default values
!-----------------------------------------------------------------------
      routine = 'POPENIN'
      lshort  = .false.
      linpf   = .false.
      lmnd    = .false.
      lrddvr  = .false.
!-----------------------------------------------------------------------
! The input file and the options are read from the UNIX shell.
! There must be at least one argument. If there are no arguments read
! a help text is output and the program is stopped. 
! If there is only one argument it is the name
! of the '~.inp' file (the name must not start with an '-').
! If there is more than one argument the last argument is taken as the
! name of the '~.inp' file. The order of the other arguments (i.e. the
! order of the options) is arbitrary.
! If the argument list does not have a correct format
! a help text is output and the program is stopped.
!-----------------------------------------------------------------------

! GETARG is a UNIX specific call:
! mygetarg calls the getarg-routine of the present UNIX system 
! GET THE FIRST ARGUMENT 

      i = 1
      call mygetarg(1,buf)
      if( buf(1:1) .eq. ' ' ) then
         call wrphelptext
      else if( buf(1:1) .eq. '-' ) then
         goto 101
      else 
         inputfile = buf
         linpf = .true.
! ----------------------------------------------------------------------
!  Outcomment the following lines, if you want to allow, that the ~.inp
!  file appear as either the first or the last argument.
!  With the following 7 FORTRAN lines you are forced to have the ~.inp 
!  file as the last argument. (This is UNIX convention).
! ----------------------------------------------------------------------
         call mygetarg(2,flag)
         if( flag(1:1) .eq. ' ' ) then
            goto 200
         else
         write(6,'(/,a/,a)') '***** ERROR ! *****', &
         ' The ~.inp file must be the last argument.'
            call wrphelptext
         end if
!-----------------------------------------------------------------------

      end if
 
!-----------------------------------------------------------------------
! LOOP OVER THE ARGUMENTS 
 123  continue
      i = i + 1
      call mygetarg(i,buf)

 101  continue
      flag    = buf(1:8)
      if( flag(1:1) .eq. '-' ) then
         length  = index(flag//' ',' ')-1
         length  = min(length, 79-optlaenge)
         if (optlaenge .eq. 0) then
            options = flag(1:length)//' '
         else
            options = options(1:optlaenge)//flag(1:length)//' '
         endif
         optlaenge = optlaenge+length+1
      end if

      if ( flag .eq. '-h' .or.  flag .eq. '-?' ) then
         call wrphelptext
      else if ( flag .eq. '-ver' ) then
         call wrversion(6)
         stop
      else if ( flag(1:1) .eq. ' ' ) then
         goto 200
      else if ( flag .eq. '-w' ) then
         lovwr  = .true.
      else if ( flag .eq. '-rd' ) then
         lrddvr  = .true.
      else if ( flag .eq. '-rv' ) then
         lreadvpot = .true.
      else if ( flag .eq. '-mnd' ) then
         lmnd  = .true.
      else if ( flag .eq. '-gd' ) then
         lrddvr  = .false.
      else if ( flag .eq. '-gv' ) then
         lgenvpot = .true.
      else if ( flag .eq. '-ww' ) then
         lovwr    = .true.
         lgenvpot = .true.
         lrddvr  = .false.
      else if ( flag .eq. '-s' ) then
         lshort = .true.
         ilog   =  6
      else if ( flag .eq. '-c' ) then
         lcont     = .true.
         lrddvr   = .true.
         lreadvpot= .true.
         lovwr    = .true.
         lrdnpot  = .false.
         lcutnpot = .false.
      else if (flag .eq.'-jtot') then
         i = i + 1
         call mygetarg(i,buf)
         if (buf(1:1).eq.'-' .or.  buf(1:1).eq.' ') then
            write(6,'(a)') ' No number follows -jtot; but:'
            write(6,'(a)')  buf
            call wrphelptext
         else
            read(buf,*,iostat=ierr) jtot
            if (ierr.ne.0) then
               write(6,'(a)') ' No number follows -jtot; but:'
               write(6,'(a)')  buf
               call wrphelptext
            endif
         endif
      else if( flag .eq. '-D' ) then
         i = i + 1
         call mygetarg(i,buf)
         if (buf(1:1).eq.'-' .or.  buf(1:1).eq.' ') then
            write(6,'(a)') ' No pathname follows -D'
            call wrphelptext
         else
            name    = buf
            laenge  = index(name,' ')-1
            length  = min(laenge,79-optlaenge)
            options = options(1:optlaenge)//name(1:length)//' '
            optlaenge = optlaenge+length+1
         end if
      else if( flag(1:1) .ne. '-' ) then
         if( linpf ) then
            write(6,'(/,a/,a)') '***** ERROR ! *****', &
            ' There must be only one ~.inp file.'
            call wrphelptext
         end if 
         i = i + 1
         call mygetarg(i,flag)
         if( flag(1:1) .ne. ' ' ) then
         write(6,'(/,a/,a)') '***** ERROR ! *****', &
         ' The ~.inp file must be the last argument.'
            call wrphelptext
         else
            linpf = .true.
            inputfile = buf
         end if 

      else
         write(6,'(/,a/,2a)') '***** ERROR ! *****', &
              ' Unknown option: ', flag
         call wrphelptext
      end if   
      goto 123

!-----------------------------------------------------------------------
 200  continue
! FILE NAMES
      laenge = index(name,' ')-1
      if( laenge .gt. 0 ) then
         call abspath(name,laenge)
      endif

      inplaenge = index(inputfile,' ')-1
      if(inputfile(max(1,inplaenge-3):inplaenge) .ne. '.inp') then
         inputfile = inputfile(1:inplaenge)//'.inp'
         inplaenge = inplaenge + 4
      endif
      call abspath(inputfile,inplaenge)

! CHECK EXISTENCE OF INPUT FILE.
      inquire(file=inputfile,exist=lexist)
      if(.not.lexist) then
         write(6,'(a)')'-----------------------------------------------'
         write(6,'(a)') ' ##############   ERROR   ##############'
         write(6,'(a)') ' There is no input-file! The file:'
         write(6,'(a)') inputfile(1:inplaenge)
         write(6,'(a)') ' does not exist!'
         write(6,'(a)')'-----------------------------------------------'
         call stopnow
      end if


! --- OPEN INPUT FILE ---

      open(iin,file=inputfile,status='old',err=10)

      return

 10   continue
      write(6,*)
      write(6,'(a)') 'ERROR in subroutine popenin:'
      write(6,'(a)') 'Cannot open input file!'
      write(6,'(a)') 'inputfile: ', inputfile
      write(6,*)
      call stopnow

      end subroutine popenin


!***********************************************************************
!
!                  Subroutine  POPENLOG
!
! Opens the log file on path 'path'.
!
!***********************************************************************

      subroutine popenlog(path)

      implicit none

      character(len=*), intent(in) :: path
      character(len=100)           :: buffer
      integer(long)                :: plen,i
      integer(long), external      :: slen

      plen=slen(path)
      
! --- OPEN LOG FILE ---

      if (lovwr) then
         open(ilog,file=path(1:plen),status='unknown',err=20)
      else
         open(ilog,file=path(1:plen),status='new',err=30)
      endif
      logisopen = .true.

! --- RE-POSITION LOG FILE ---
      if (lcont) then
 10      read(ilog,'(a)',end=12)
         go to 10
 12      backspace(ilog)
         write(ilog,'(/,78a,/)') ('#',i=1,72)
         write(ilog,'(a,13x,a,13x,a)') '###############', &
                   'CONTINUATION RUN', '###############'
         write(ilog,'(78a,/)') ('#',i=1,72)
      end if

! --- WRITE DATE AND MACHINE TO LOG FILE ---

      call fpstartup(buffer)
      write(ilog,*)
      write(ilog,'(a)') trim(buffer)
      write(ilog,'(1x,a)') name(1:laenge)
      if (runtitle(1:10) .ne. '          ' )&
           write(ilog,'(1x,a)') trim(runtitle(1:78))

! --- WRITE PROJECT VERSION NUMBERS TO LOG FILE ---
      call wrversion(ilog)
      if(optlaenge .gt. 1) then
         write(ilog,'(/a)')  '******* OPTIONS ****** '
         write(ilog,'(a/)')  options(1:optlaenge-1)
      end if
      return

! --- WRITE ERROR MESSAGE TO SCREEN ---

 20   continue
      write(6,*)
      write(6,'(a)') 'ERROR in subroutine popenlog:'
      write(6,'(2a,/a)') 'Cannot open ', path(1:plen),&
         '  Directory non-existent?'
      write(6,*)
      call stopnow

 30   continue
      write(6,*)
      write(6,'(a)') 'ERROR in subroutine popenlog:'
      write(6,'(a)')&
          'Cannot open potfit log-file! File already exists?'
      write(6,'(2a)') 'logfile: ', path(1:plen)
      if (lmcpf) then
         write(6,'(a)') &
              'overwrite disabled? (Use -w option to enable overwrite)'
      else
         write(6,'(a,/a,/a,/)') &
              'overwrite disabled? (Use -w option to enable overwrite)', &
              '(Use -gv option to enforce new creation of vpot)', &
              '(Or use the -ww option which sets -w, -gv, and -gd)'
      end if
      call stopnow

      end subroutine popenlog

!***********************************************************************
! 
!                   Subroutine   POPENDAT
!
! called from subroutine peingabe
!
! Opens the output data files: output, prodwei, timing, iteration, and stop.
!
!***********************************************************************

      subroutine popendat

      implicit none

      integer(long)         :: version,release,i
      character(len=100)    :: stat
      logical(kind=4), save :: ncallpeingabe=.true.
      character(len=c5)     :: filename
!
! stat controls whether files are opened as new or unknown.
!
      if (lovwr) then
         stat='unknown'
      else
         stat='new'
      endif

      write(ilog,*)
      write(ilog,'(a)') '***** Output files *****'
      write(ilog,*)

!-----------------------------------------------------------------------
! if output is written to a file, change output channel from 6(screen)
!  to 13 and open file 'output'
! If write-cw is set, write output to log file.
!-----------------------------------------------------------------------
      if (.not.lcutnpot) then
        if (lwrcw) then
           iout=ilog
        else if (loutfil) then
          iout=13
          filename=name(1:laenge)//'/output'
          if(lrdnpot) filename=name(1:laenge)//'/rdnpot_output'
          open (iout,file=filename,status=stat,err=10)
          write(ilog,'(a)')  'output     file opened'
          if(lcont) call preposition(iout,filename)
        else
          write(ilog,'(a)')  'output     directed to the screen'
        endif
      endif

      if (lprodw) then
        filename=name(1:laenge)//'/prodwei'
         open (ipw,file=filename,form='formatted',status=stat,err=10)
         write(ilog,'(a)') 'prodwei    file opened '
      endif

      if (ltime) then
         filename=name(1:laenge)//'/timing'
         if(lrdnpot) filename=name(1:laenge)//'/rdnpot_timing'
         open(itim,file=filename,status=stat,err=10)
         write(ilog,'(a)') 'timing     file opened'
      endif

      if (litf) then
         filename=name(1:laenge)//'/iteration'
         open(iit,file=filename,form='formatted',status=stat,err=10)
         write(ilog,'(a)') 'iteration  file opened'
         if(lcont) call preposition(iit,filename)
      endif

      call myflush(ilog)

      call getrevno(progver,version,release)

!-----------------------------------------------------------
! MGPF: cutnpot calls peingabe twice
!       but should print this only once.
!-----------------------------------------------------------
      if(ncallpeingabe .and. .not.lcutnpot) then
        write (iout,*)
        write (iout,'(2a)') '********************', &
            '***************************************************'
        if (lmcpf) then
           write(iout,'(17x,a,i5)') 'Monte-Carlo POTFIT version ', &
                version
        else if (lmccpd) then
           write(iout,'(17x,a,i5)') 'Monte-Carlo CPD version    ', &
                version
        else
           write(iout,'(31x,a,i3)') 'POTFIT version ',version
        end if
        write(iout,'(36x,a,i5)') 'Release ', release
        write (iout,'(2a,/)') '********************', &
            '***************************************************'
        ncallpeingabe=.false.
      endif

      if (lmccpd .or. lcutnpot .or. lrdnpot) return

      filename=name(1:laenge)//'/stop'
      open(istop,file=filename,form='formatted',status='unknown',err=10)

      if(sstop.eq.'        ' .and. scpu.eq.'        ') then
         write(istop,'(a8)') 'continue'
      else
         write(istop,'(a7,a8)') sstop,scpu
      endif

      write(istop,'(a)' ) 'hh:mm  hh:mm:ss  comment '
      write(istop,'(a)' ) 'real    cpu      optional'
      write(istop,'(a)' )
      write(istop,'(72a)') ('-',i=1,72)
      write(istop,'(a,a)' )&
        ' Replace "continue" by "stop   comment" or',&
        ' "hh:mm  hh:mm:ss  comment"'
      write(istop,'(a,a)' )&
        ' The first input (hh:mm) refers to real-time,',&
        ' the second to cpu-time.'
      write(istop,'(a)' )&
        ' One of the time entries may be left blank.'
      write(istop,'(a)' )&
         ' One has to begin at the first column, e.g : '
      write(istop,'(a)' ) '32:30            Systemarbeiten '
      write(istop,'(a)' )&
         ' will stop the run after 08.30 next day (8+24=32)'
      write(istop,'(a)' ) '       00:56:30  Zeitbegrenzung'
      write(istop,'(a,a)' )&
         ' will stop the run after the first output later',&
         ' than 56.5 cpu minutes'
      write(istop,'(72a)') ('-',i=1,72)
      write(istop,'(a)' )
      close (istop)


      return

   10 continue
      routine = 'POpendat'
      message = 'Cannot open data files'//trim(filename)
      call errormsg

      end subroutine popendat


!***********************************************************************
! PCLOSEDAT
!
! called from main
!
! closes files at end of program and outputs final real time to log file
!
! V5.0 MB
!***********************************************************************

      subroutine pclosedat

      implicit none

      integer(long)            :: i,ilbl,ierr
      character(len=100)       :: buffer
      character(len=c5)        :: filename
      logical(kind=4)          :: check

      call fpstartup(buffer)
      
!-----------------------------------------------------------------------
! Close files
!-----------------------------------------------------------------------
      
      inquire(idvr,opened=check)
      if (check) close(idvr)
      if (ldeldvr) then
         filename=dname(1:dlaenge)//'/dvr'
         open(idvr,file=filename,form='unformatted',status='old',&
              iostat=ierr)
         if (ierr .ne. 0) then
            routine='pclosedat'
            ilbl=index(filename,' ')-1
            message = 'Cannot open file: '//filename(1:ilbl)
            call warnmsg
         endif
         close(idvr,status='delete',iostat=ierr)
         if (ierr.ne.0) then
            routine='pclosedat'
            ilbl=index(filename,' ')-1
            message = 'Cannot delete file: '//filename(1:ilbl)
            call warnmsg
         endif
      endif

      inquire(ivpot,opened=check)
      if (check) close(ivpot)
      if (ldelvpot .and. .not.lreadvpot) then
         open(ivpot,file=vname(1:vlength),form='unformatted',&
            status='old',iostat=ierr)
         if (ierr .ne. 0) then
            routine='pclosedat'
            message = 'Cannot open file: '//vname(1:vlength)
            call warnmsg
         endif
         close(ivpot,status='delete',iostat=ierr)
         if (ierr.ne.0) then
            routine='pclosedat'
            message = 'Cannot delete file: '//vname(1:vlength)
            call warnmsg
         endif
      endif

      close(iin)
      if (lausw) return

      if (lprodw)  close(ipw)
      if (litf)    close(iit)

      write(iout,'(/1x,78a)') ('-',i=1,78)
      write(iout,'(1x,a)') trim(buffer)
      write(iout,'(1x,a)') name(1:laenge)
      if(runtitle(1:10) .ne. '          ' ) &
                write(iout,'(1x,a)') trim(runtitle)
      write(iout,'(1x,78a)') ('-',i=1,78)
      if (loutfil .and. .not. lwrcw) then
         close(iout)
      endif

      if (ltime)  then
         backspace(itim)
         write(itim,*)
         write(itim,'(1x,75a)') ('-',i=1,75)
         write(itim,'(1x,a)') trim(buffer)
         write(itim,'(1x,a)') name(1:laenge)
         if(runtitle(1:10) .ne. '          ' ) &
                   write(itim,'(1x,a)') trim(runtitle)
         close(itim)
      end if

      write(ilog,'(/1x,78a)') ('-',i=1,78)
      write(ilog,'(1x,a)') trim(buffer)
      write(ilog,'(1x,a)') name(1:laenge)
      if(runtitle(1:10) .ne. '          ' )&
           write(ilog,'(1x,a)') trim(runtitle)
      write(ilog,'(1x,78a)') ('-',i=1,78)
      close(ilog)

      inquire(istop,opened=check)
      if (check) then
         close(istop,status='delete')
      else
         inquire(file=name(1:laenge)//'/stop',exist=check)
         if(check) then
            open (istop,file=name(1:laenge)//'/stop',&
                  form='formatted',status='old')
            close(istop,status='delete')
         end if
      end if
 
      return
      end subroutine pclosedat

!***********************************************************************
!         POTFIT  subroutine  PAUSGAB 
!
! Copies the input file to the output directory.
! Writes file information to log-file.
!***********************************************************************

      subroutine pausgab

      implicit None

      integer(long),external  :: slen
      integer(long)           :: ilbl
      character(len=80)       :: string
      character(len=360)      :: filename
      logical(kind=4)         :: lexist

! --- CUT BLANK LINES AT THE END OF INPUT FILE AND REWIND ---

      rewind(iin,err=10)

! --- COPY THE INPUT FILE ---

      filename=name(1:laenge)//'/input'
      if(lrdnpot) filename=name(1:laenge)//'/rdnpot_input'
      ilbl=laenge+6
      if(ilbl.ge.360) then
         routine = 'pausgab, peingutil.F'
         message = 'Filename too long: '//trim(filename)
         call errormsg
      endif
      inquire(file=filename,exist=lexist)
      if(lexist) then  ! Make file input read-write.
         call excmd('chmod 644 '//trim(filename))
      endif
      open(idat,file=filename,status='unknown',err=10)
      if(lcont) call preposition(idat,filename)

 100  continue
      read(iin,'(a80)',err=11,end=20) string
      if (string(1:1).ne.'#' .and. slen(string).gt.0 ) then
         write(idat,'(a)',err=10) string(1:slen(string))
         call keylowc(string(1:11))
         if(string(1:11) .eq. 'end-input') goto 200
      endif
      Goto 100

! --- ADD PROGRAM VERSION AND OPTIONS ---

 200  continue
      write(idat,'(a17,f9.4)',err=10) 'ProgramVersion = ',progver
      if(optlaenge .gt. 1) then
         write(idat,'(2a)') 'Options: ', options(1:optlaenge)
      end if

! --- CLOSE FILES ---

      close(idat,err=12)
!...Make file input read only.
      filename='chmod 444 '//name(1:laenge)//'/input'
      call excmd(filename)

! --- WRITE MESSAGE TO LOG FILE ---

      write(ilog,'(a)') &
           '.inp       file copied to input in name directory'

! --- WRITE INPUT FILE PATH NAMES TO LOG FILE ---

      write(ilog,*)
      write(ilog,'(a)') '***** Paths and input files *****'
      write(ilog,*)
      write(ilog,'(a,a)') 'name   : ', name(1:laenge)
      write(ilog,'(a,a)') 'input  : ', trim(inputfile)
      call myflush(ilog)

      return

! --- WRITE ERROR MESSAGE TO LOG FILE ---

 10   continue
      routine = 'pausgab, peingutil.F'
      message = 'Cannot open ~.inp file:  '//filename(1:ilbl)
      call errormsg

 11   continue
      routine = 'pausgab, peingutil.F'
      message = 'Cannot read ~.inp file:  '//filename(1:ilbl)
      call errormsg

 12   continue
      routine = 'pausgab, peingutil.F'
      message = 'Cannot close ~.inp file:  '//filename(1:ilbl)
      call errormsg

 20   continue
      routine = 'pausgab, peingutil.F'
      message = 'end of ~.inp file found. END-INPUT keyword not present?'
      call errormsg

      end subroutine pausgab


!***********************************************************************
!
!              POTFIT  subroutine WRHELPTEXT
! 
! Writes Unix-like help text to screen if arguments of the POTFIT
! executable are not correct.
!
!  01/98 AJ
!***********************************************************************

      subroutine wrphelptext

      implicit none

      write(6,*)
      write(6,'(a)')&
           'Purpose: creates a natural potential fit.'
      write(6,'(2a,/26x,a)')&
       '  Usage:  potfit [-h|-?] ',&
           ' [-ver -c -rd -rv -s -gd -gv -mnd -w -ww]',&
           '[-jtot R] [-D name] inpf '
      write(6,'(a)')&
           '   inpf:  input file (with or without extension ".inp")'
      write(6,'(a)')&
           '   optn: -h   : print this help-text.'
      write(6,'(a)')&
           '         -?   : print this help-text.'
      write(6,'(a)')&
           '         -ver : Version information.'
      write(6,'(a)')&
           '         -c   : Continuation run.'
      write(6,'(2a)')&
           '         -rd  : Reading of the dvr-file',&
           ' is enforced.'
      write(6,'(2a)')&
           '         -rv  : Reading of the vpot-file',&
           ' is enforced.'
      write(6,'(2a)') &
           '         -gd  : Generation of the dvr-file', &
           ' is enforced.'
      write(6,'(2a)')&
           '         -gv  : Generation of the vpot-file',&
           ' is enforced.'
      write(6,'(2a)')&
           '         -mnd : Make Name Directory. ',&
           'Creates the name directory, if not existent.'
      write(6,'(a)')&
           '         -w   : overwrite enabled.'
      write(6,'(a)')&
           '         -ww  : Sets -w and -gv.'
      write(6,'(2a/16x,a)')&
           '         -s   : short test-run, checking the syntax ',&
           'of the inpf file, ',&
           'no files created, no dynamical space allocated.'
      write(6,'(2a,/18x,a)')&
           '         -jtot R: ''R'' denotes a real number, ',&
           '(jtot in ~.inp file ignored, ', &
           'but cspot keyword must be set in operator-section).'
      write(6,'(2a,/18x,a)')&
           '         -D name: ''name'' denotes the directory where ',&
           'files are','written to, (name in ~.inp file ignored).'
      write(6,'(/a,/a,/)')&
           '   The order of the options does not matter, but "inpf"',&
           '   must be the last argument.'

      STOP

      end subroutine wrphelptext

!#######################################################################
! PREPOSITION
! Identical to reposition on source/propwf/opendat.F
! repositions the formatted file 'filename' on unit 'iunit' to its end
!#######################################################################

      subroutine preposition(iunit,filename)

      implicit none

      integer(long), intent(in)     :: iunit
      integer(long)                 :: ilbl,i
      character(len=c5), intent(in) :: filename

   10 continue
      read(iunit,'(a)',end=20)
      go to 10
   20 backspace(iunit)
      ilbl=index(filename,' ')-1
      write (ilog,'(2a)') filename(1:ilbl)//' file repositioned'
      write(iunit,'(/,78a,/)') ('#',i=1,72)
      write(iunit,'(a,13x,a,13x,a)') '###############', &
                 'CONTINUATION RUN', '###############'
      write(iunit,'(78a,/)') ('#',i=1,72)
      return
      end subroutine preposition

!***********************************************************************
      end module peingutil


