!***********************************************************************
      module peinprojmod

use decimal, only:long,dop
use logdat

      implicit none
      private
      public :: peinproj
      contains

!***********************************************************************
!
! SUBROUTINE PEINPROJ
!
! reads the PROJECTION-SECTION of potfit input files
!
! FO 01/2006
!
!***********************************************************************

      subroutine peinproj(linperr,iz)

use global
use pglobal
use dvrdatmod
use griddatmod
use channels


      implicit none

      integer(long), intent(in)      :: iz
      logical(kind=4), intent(inout) :: linperr

      integer(long)           :: i,iz0,ic,ierr,p,q,f,len,plen
      real(dop)               :: rtmp
      character(len=c1)       :: doflab
      
! ----------------------------------------------------------------------
!     initial values
! ----------------------------------------------------------------------
      routine = 'read PROJECTION-SECTION (peinproj)'
      iz0 = 0
      p = 0

! ----------------------------------------------------------------------
!     read keywords
! ----------------------------------------------------------------------
 10   continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      if (keyword(1).eq.'end-projection-section') goto 1000

      if (keyword(1).eq.'projector') then
! ----------------------------------------------------------------------
!        start of PROJECTOR definition
! ----------------------------------------------------------------------

!        projector label must follow
         if (ic.lt.2) then
            message = ' PROJECTOR must be followed by a label'
            call errormsg
         endif

!        increase counter and store label
         p = p+1
         if (p.gt.maxproj) then
            write(6,*) ' ### Too many projectors! ###'
            write(message,'(a,i2,a)')&
               ' Increase maxproj in "pglobal.inc" to at least ',&
               p, ' and recompile.'
            call errormsg
         endif
         projlab(p) = keyorig(2)
         plen = lc(2)
!        set default scale (none)
         projscale(p) = 1.0_dop

         q=0
!        read next line, check for end of definition
 100     call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         message = inptit
         call cntrl(keyword(1),lc(1),ierr)
         if (keyword(1).eq.'end-projector') goto 200

!        is this an "option = value" line? look for the "="
         if (keyword(2).eq.'=') then

            if (keyword(1).eq.'scale') then
!              scale value is real number
               read(keyword(3),*,iostat=ierr) rtmp
               if (ierr.ne.0) then
                  message = ' scale must be real number, not: '
                  call wrline(3,iz,iz0)
               else
                  projscale(p) = rtmp
               endif

            else
               message = ' unknown projector option: '
               call wrline(1,iz,iz0)
            endif

         else
!           we are expecting a projector function definition,
!           so increase counter
            q=q+1
            if (q.gt.maxprojdof) then
               write (6,*) ' ### Too many projected DOFs! ###'
               write (message,'(a,i2,a)')&
                  ' Increase maxprojdof in "pglobal.inc" to at least ',&
                  q, ' and recompile.'
               call errormsg
            endif
            
!           line starts with modelabel of the DOF to project on
            doflab = keyorig(1)
            len = lc(1)
!           search for a DVR-DOF with matching label
            f=ndof
 110        if (modelabel(f).eq.doflab) then
               goto 120 ! found
            else
               f=f-1
               if (f.gt.0) goto 110
            endif
!           not found
            message = ' Modelabel "'//doflab(1:len)&
               //'", defined in projector "'//projlab(p)(1:plen)&
               //'", not found.'
            call errormsg
!           store found DOF
 120        projdof(p,q)=f

!           delegate parsing of the projector function
            call projfuncparse(p, q, ic-1, keyword(2), keyorig(2),&
                                lc(2), ierr)
            if (ierr.ne.0) then
               message = ' Error while parsing projector function '&
                  //'definition for DOF "'//doflab(1:len)//'" in '&
                  //'projector "'//projlab(p)(1:plen)//'".'
               call errormsg
            endif
         endif

         goto 100

 200     continue
! ----------------------------------------------------------------------
!        projector definition finished, check sanity
! ----------------------------------------------------------------------
         if (q.eq.0) then
            message = ' empty definition for projector: '//projlab(p)
            call warnmsg
         endif
         nprojdof(p)=q

      else
! ----------------------------------------------------------------------
!        read projection options
! ----------------------------------------------------------------------
         i=1
 15      continue

         if (keyword(i).eq.'ascii') then
            lprojascii = .true.

         elseif (keyword(i).eq.'error') then
            lprojerr = .true.
            if (keyword(i+1).eq.'=') then
               i=i+2
               projerrfile = keyorig(i)
            else
               projerrfile = "projerr.vpot"
            endif
            
         else
            message = ' unknown projection option: '
            call wrline(i,iz,iz0)
         endif

!        more keywords? if not, read new line
         if (i.lt.ic) then
            i=i+1
            goto 15
         endif

      endif
      goto 10

! ----------------------------------------------------------------------
!     end of section reached
! ----------------------------------------------------------------------
 1000 continue

!     store number of projectors
      if (p.eq.0) then
         message = ' No projectors in PROJECTION-SECTION !'
         call warnmsg
      endif
      nproj = p

!-----------------------------------------------------------------------
! Set 'linperr=true' if a warnmsg occurred
!-----------------------------------------------------------------------
      if (.not.linperr) linperr=lwarn
      lwarn=.false.

      return
      end subroutine peinproj ! of peinproj





!***********************************************************************
!
! SUBROUTINE PROJFUNCPARSE
!
! In:
!   p - number of projector
!   q - number of projected DOF in projector #p
!   ntoken - number of tokens in function definition
!   token(i) - i-th token of function definition (lowercased)
!   origtoken(i) - original version of i-th token
!   ltoken(i) - length of i-th token
! Out:
!   ierr - zero if nothing went wrong
!   projifunc(p,q), projipar(p,q,j), projrpar(p,q,j), ... -
!     function index and parameters (common block variables)
!
! FO 01/2006
!    
!***********************************************************************

      subroutine projfuncparse(p,q,ntoken,token,origtoken,ltoken,ierr)

use global
use pglobal

      implicit none

      integer(long), intent(in)                        :: p,q,ntoken
      integer(long), intent(out)                       :: ierr
      integer(long), dimension(ntoken), intent(in)     :: ltoken
      character(len=maxkeylen), dimension(ntoken), intent(in) :: token,origtoken

      integer(long)            :: i,itmp,flen
      real(dop)                :: rtmp
      character(len=maxkeylen) :: funclab

!     set defaults:
!     we should multiply with DVR weights
      projwi(p,q) = 0
      ierr = 0

      i = 1
!     first token is the function name
      funclab = token(i)
      flen = ltoken(i)

! ======================================================================

! --> Further parsing depends on the function.
! --> Currently implemented:
! -->
! -->   Label  ifunc  Description
! --> -----------------------------------------------------------
! -->   leg      1    associated Legendre polynomials 
! -->   cos      2    cosine transform
! -->   gauss    3    Gaussian
! -->   read     4    read function values from file
! -->   sin      5    sine transform

! ======================================================================

      if (funclab.eq.'leg') then
!        Function: associated Legendre polynomials, P_lm(cos x)
!        Parameters: Integer l, Integer m
         projifunc(p,q)=1 ! set function index

         i=i+1
         read (token(i),*,iostat=ierr) itmp
         if (ierr.ne.0) then
            message = ' First argument of "leg" projector '&
                 //'must be integer, not: '//origtoken(i)(1:ltoken(i))
            call warnmsg
            return
         endif
         projipar(p,q,1)=itmp ! first parameter

         i=i+1
         read (token(i),*,iostat=ierr) itmp
         if (ierr.ne.0) then
            message = ' Second argument of "leg" projector '&
                 //'must be integer, not: '//origtoken(i)(1:ltoken(i))
            call warnmsg
            return
         endif
         projipar(p,q,2)=itmp ! second parameter

!        consistency check: l>=0, |m|<=l
         if ( (projipar(p,q,1).lt.0) .or.&
              (abs(projipar(p,q,2)).gt.projipar(p,q,1)) ) then
            write(message,'(a,i2,a,i2)')&
                 'Illegal arguments to "leg" projector: l=',&
                 projipar(p,q,1),' m=',projipar(p,q,2)
            call warnmsg
            ierr = 1
            return
         endif

! ----------------------------------------------------------------------

      elseif (funclab.eq.'cos') then
!        Function: cos(mx)
!        Parameters: Integer m
         projifunc(p,q)=2

         i=i+1
         read (token(i),*,iostat=ierr) itmp
         if (ierr.ne.0) then
            message = ' Argument of "cos" projector must be '&
                 //'integer, not: '//origtoken(i)(1:ltoken(i))
            call warnmsg
            return
         endif
         projipar(p,q,1)=itmp

! ----------------------------------------------------------------------

      elseif (funclab.eq.'gauss') then
!        Function: Gaussian
!        Parameters: x0, sigma
         projifunc(p,q)=3

         i=i+1
         read (token(i),*,iostat=ierr) rtmp
         if (ierr.ne.0) then
            message = ' First argument of "gauss" projector must '&
                 //'be real, not: '//origtoken(i)(1:ltoken(i))
            call warnmsg
            return
         endif
         projrpar(p,q,1)=rtmp

         i=i+1
         read (token(i),*,iostat=ierr) rtmp
         if (ierr.ne.0) then
            message = ' Second argument of "gauss" projector must '&
                 //'be real, not: '//origtoken(i)(1:ltoken(i))
            call warnmsg
            return
         endif
         projrpar(p,q,2)=rtmp

!        consistency check: sigma>0
         if (projrpar(p,q,2) .le. 0.0_dop) then
            message = ' Second argument of "gauss" projector '&
                 //'must be > 0 .'
            call warnmsg
            ierr = 1
            return
         endif

! ----------------------------------------------------------------------

      elseif (funclab.eq.'read') then
!        Read function values from file.
!        Parameters: filename, options
         projifunc(p,q)=4

         i=i+1
         projcpar(p,q,1)=origtoken(i)
          
! ----------------------------------------------------------------------

      elseif (funclab.eq.'sin') then
!        Function: sin(mx)
!        Parameters: Integer m
         projifunc(p,q)=5

         i=i+1
         read (token(i),*,iostat=ierr) itmp
         if (ierr.ne.0) then
            message = ' Argument of "sin" projector must be ' &
                 //'integer, not: '//origtoken(i)(1:ltoken(i))
            call warnmsg
            return
         endif
         if (itmp.eq.0) then
            message = ' Argument of "sin" projector must not be ' &
                    //'equal to zero'
            call errormsg
         endif
         projipar(p,q,1)=itmp

!-----------------------------------------------------------------------
! ======================================================================

! If you want to implement other functions, add the corresponding
! elseif-branch before here. Don't forget to use a new function index
! (i.e. ifunc value)! If your functions needs more than two parameters
! of one type, you must adjust maxprojpar in pglobal.inc!
! Also, add the evaluation code for the new function to CALCPRJF.

! ======================================================================
! ----------------------------------------------------------------------

      else
!        unknown function label
         message = ' Unknown function label: '//funclab(1:flen)
         call warnmsg
         ierr = 1
         return

      endif

! ----------------------------------------------------------------------
!     parse common options
! ----------------------------------------------------------------------

 900  i=i+1
      if (i.gt.ntoken) goto 910

      if (token(i).eq.'ft') then
!        obsolete feature
         message = 'The FT feature has been removed. Section 11.3 of '&
            //'the User Guide explains the new method for using '&
            //'Fourier-transformed potentials.'
         call warnmsg
         ierr = 1

      elseif (token(i).eq.'no_weights_included' .or.&
              token(i).eq.'nwi') then
!        function values do _not_ include DVR weights
         projwi(p,q) = 0

      elseif (token(i).eq.'weights_included' .or.&
              token(i).eq.'wi') then
!        function values already include the full DVR weights,
!        so we must not multiply with them later while integrating
         projwi(p,q) = 1

      elseif (token(i).eq.'root_of_weights_included' .or.&
              token(i).eq.'rwi') then
!        function values include square root of DVR weights
         projwi(p,q) = 2

      else
!        unknown option
         message = ' Unknown option: '//origtoken(i)(1:ltoken(i))
         call warnmsg
         ierr = 1
         return
      endif

      goto 900 ! process next token

! ----------------------------------------------------------------------

 910  continue
      return

      end subroutine projfuncparse ! of projfuncparse


      end module peinprojmod


