!***********************************************************************
!
!            PROJECTION  subroutine  PROJEINGABE
!
! Reads the input data from the input file.
!
! Called from: potfit
! Calls to: popenin, popenlog,rdinpf,peinfile,peinweight,peinbas,  
!           abspath,warnmsg,errormsg,myflush
!
! 11/97 HDM
!
!***********************************************************************

      subroutine projein 

use decimal, only:long,dop
use global
use pglobal
use maxv, only:maxdim
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels
use mpidata
#ifdef MPI
   use mpi
#endif

use op1lib, only: initvxl, zerovxz, zerovxd, zerovxi, zerovxs, initvxi
use logdat
use rddvrmod, only: dvrinfo
use einpbasmod, only: einpbas
use projutil, only: projstupid
use peinprojmod, only: peinproj

use potmod_interface

      implicit None

      integer(long)                      :: iz,ic,ierr,i,m,n,f,p,q
      logical(kind=4), dimension(maxdim) :: modelist
      logical(kind=4)                    :: lghost
      logical(kind=4)                    :: linperr

!-----------------------------------------------------------------------
! read projection-section
!-----------------------------------------------------------------------
      rewind(iin)
      iz=0
 215  continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine = 'projeingabe (projection-section)'
         message = inptit
         call errormsg
      endif

      if (keyword(1) .eq. 'projection-section') then
!        read the section
         call peinproj(linperr,iz)
!        then set up some unneeded variables
         call projstupid
      elseif (keyword(1) .eq. 'end-input') then
         routine = 'projeingabe (projection-section)'
         message = 'No PROJECTION-SECTION in input file'
         call warnmsg
      else
         goto 215
      endif

!-----------------------------------------------------------------------
! Checks for consistency 
!-----------------------------------------------------------------------
      if( lsp ) then
         routine = 'projeingabe  (Check for consistency)'
         message='"projection" cannot be done in "single-precision"'
         call warnmsg
      endif
      if( lreadvpot .and. lgenvpot ) then
         routine = 'projeingabe  (Check for consistency)'
         message='One cannot give "readvpot" (-rv) AND "genvpot" (-gv)'
         call warnmsg
      endif
      call initvxl(modelist,ndof)
      do m=1,npmode
          do n=1,npotdof(m)
             f=potdof(n,m)
             modelist(f)=.false.
         enddo
      enddo
      do f = 1, ndof
         if(basis(f).eq.0) then
         routine = 'projeingabe (consistency check)'
         message = 'There must not be an electronic basis in the'&
                 //' primitive-basis-section of a projection run.'
         call errormsg
         endif
      enddo
      lghost=.false.
      do f = 1, ndof
         lghost=lghost.or.modelist(f)
      enddo
      if(lghost) then
         i=6
 35      write(i,'(4(/a))') ' ## WARNING : There are unused DOFs.',&
         ' One better remove those from the Primitive-Basis-Section,',&
         ' as this may lead to problems, if the natpot is not the only',&
         ' operator in a Hamitonian-Section line.'
         do f = 1, ndof
            if(modelist(f)) write(i,'(2a)') ' Modelabel:  ',modelabel(f)
         enddo
         write(i,*)
         if(i.ne.ilog) then
            i=ilog
            goto 35
         endif
      endif
      if (lprojerr.or.lprojlowmem) then
!        check if all projectors project onto the same DOFs
         do p=2,nproj
            if (nprojdof(p).ne.nprojdof(1)) goto 36
            do q=1,nprojdof(p)
               if (projdof(p,q).ne.projdof(1,q)) goto 36
            enddo
         enddo
         goto 37
 36      routine = 'projeingabe (consistency check)'
         message = 'if the projection error is to be calculated, or' &
            //' if the low-memory mode is used, all' &
            //' projectors must project onto the same set of DOFs'
         call errormsg
 37      continue
      endif

!-----------------------------------------------------------------------
! Set 'linperr=true' if a warnmsg occurred
!-----------------------------------------------------------------------
      if (.not.linperr) linperr=lwarn
      lwarn=.false.

      return

      end subroutine projein
