!***********************************************************************
!
!                        RUNPFIT
!
!   11/96 AJ
!
!
!***********************************************************************

      subroutine runpfit 

use decimal, only:long,dop
use global
use globmemmod
use pglobal
use maxv, only:maxdim
use peinbasmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels
use logdat
use rdnatpot
use timing

use op1lib, only: zerovxd, zerovxi, zerovxs, zerovxl, initvxi
use compdat, only:savecsys,getcsys
use iogrddef, only:griddat
use direct, only:directfit
use pdefaultmod, only:init_cmode
use wrpgitter, only: wrgrid,wroned
use rddvrmod, only:rddvr
use rdopermod, only:rdoper
use rdopermod, only:operinfo
use rddvrmod
use getpesmod, only:getpes,pescut
use zeigpfitmod, only:zeigpfit,mempfit,memhops_pfit
use mpidata
#ifdef MPI
   use mpi_pf
   use mpi
#endif
use memhopsmod, only:memhops
use directutil, only: index2coord, indexmap
use logdat, only:errormsg,stopnow
use spenergymod, only: spenergy, cutalong

use potmod_interface

      implicit none

      real(dop), dimension(:), allocatable       :: vvec
      real(dop), dimension(:), allocatable       :: coeff
      real(dop), dimension(:), allocatable       :: work2

      character(len=10)                       :: cerr
      integer(long), parameter                :: tmplabs=20
      integer(long)                           :: aerr, ierr
      integer(long)                           :: chkdvr,i,chkgrd,n,cw,ccw,&
                                                 ashift,f,hl,hopdim2
      integer(long)                           :: j,m,offset,woffset,ilbl
      integer(long)                           :: old_modc
      integer(long), dimension(1)             :: idum
      integer(long), dimension(maxhpar)       :: zpesopts0
      integer(long), dimension(:), allocatable :: igpoint,done
      integer(long), dimension(:), allocatable :: olddim
      real(dop), dimension(:), allocatable    :: gpoint
      real(dop)                               :: cpu,totmem
      real(dop)                               :: v
      real(dop), dimension(1)                 :: rdum
      complex(dop), dimension(1)              :: zdum
      real(dop), external                     :: fcputime,fsystime
      real(dop)                               :: mb
      character(len=16)                       :: progpart
      character(len=80)                       :: actual
      character(len=c4)                       :: pesopts0
      character(len=c2), dimension(tmplabs)   :: peslab
      character(len=c5)                       :: filename
      character(len=400)                      :: string
      logical(kind=4)                         :: lpacked,lerr,check,lexist

!-----------------------------------------------------------------------
! MGPF cutnpot: Declare variables for npotinfo
!-----------------------------------------------------------------------
      integer(long)                              :: ndof1, nmode1
      integer(long), dimension(:), allocatable   :: mmode,fdvr,mdvr,subdim1,&
                                                    pntbas
      logical(kind=4), dimension(:), allocatable :: loned
      integer(long), save                        :: tid=0 ! timer ID

      if (mpirank .gt. 0) go to 999
      aerr = 0
! This may become neccessary if the contracted mode uses Leg/R-DVR.
!-----------------------------------------------------------------------
!      do i = 1, ndof
!         potdim(i) = min(potdim(i),gdim(i))
!      enddo

!-----------------------------------------------------------------------
! get memory sizes from DVR file
!-----------------------------------------------------------------------
      filename=dname(1:dlaenge)//'/dvr'
      ilbl=index(filename,' ')-1
      open(idvr,file=filename,form='unformatted',status='old',err=900)
      call rdmemdim(idvr)
      close(idvr)

!-----------------------------------------------------------------------
! Allocate mememory and read system / DVR information
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_combined
      call alloc_runpfitmod
      allocate(igpoint(maxdim))
      allocate(olddim(maxdim))
      allocate(gpoint(maxdim))
      allocate(mmode(maxdim))
      allocate(fdvr(maxdim))
      allocate(mdvr(maxdim))
      allocate(subdim1(maxdim))
      allocate(loned(maxdim))
      allocate(pntbas(maxdim))

      filename=dname(1:dlaenge)//'/dvr'
      ilbl=index(filename,' ')-1
      open(idvr,file=filename,form='unformatted',status='old',err=900)
      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      close(idvr)

!-----------------------------------------------------------------------
! open input file for reading
!-----------------------------------------------------------------------
      inquire(iin,opened=check)
      if (.not. check) then
         filename=inputfile
         open(iin,file=filename,status='old',err=900)
      endif

!-----------------------------------------------------------------------
! get dimensions for operator
! allocate memory for operator and combined grids
!-----------------------------------------------------------------------
      call pmemdimoper
      call alloc_operdef

!-----------------------------------------------------------------------
! Initialise timing arrays
!-----------------------------------------------------------------------
      call timinit

!-----------------------------------------------------------------------
! Read natpot basis
!-----------------------------------------------------------------------
      call peinbas

!-----------------------------------------------------------------------
! if basic error in input, stop now
!-----------------------------------------------------------------------
      if(ndof.eq.1) then
         routine = 'POTFIT (file.inp)'
         message = 'You are trying to potfit a 1D potential???       '// &
      '                      This is a deep misunderstanding of POTFIT!'
         call errormsg
      elseif(npmode.eq.1.) then
         routine = 'POTFIT (file.inp)'
         message = 'You are trying to potfit a one-mode potential??? '// &
      '                      This is a deep misunderstanding of POTFIT!'
         call errormsg
      endif

!-----------------------------------------------------------------------
! read transformation vectors
!-----------------------------------------------------------------------
      if (ltr) call peinvec

!-----------------------------------------------------------------------
! read pes function
!-----------------------------------------------------------------------
      if (.not. lcutnpot) call peinoper

!-----------------------------------------------------------------------
! read remaining input sections (weights etc)
!-----------------------------------------------------------------------
      call peinfit

!-----------------------------------------------------------------------
! close input file
!-----------------------------------------------------------------------
      close(iin)

!-----------------------------------------------------------------------
! If pes-file is used, call operinfo
!-----------------------------------------------------------------------
      if(lpesfile)then

        operfile = name(1:laenge)//'/pes'
        opflaenge=index(operfile,' ')-1

        call abspath(operfile,opflaenge)

        filename=operfile
        ilbl=index(filename,' ')-1
        write(ilog,'(/,a)') 'Reading pes file', filename(1:ilbl)

        open(ioper, file=filename, form='unformatted', &
             status='old', iostat=ierr)
        if (ierr.ne.0) then
           opflaenge=opflaenge-4
           operfile=operfile(1:opflaenge)
           open(ioper, file=operfile, form='unformatted',&
                status='old', iostat=ierr)
        endif

        if (ierr.ne.0) then
           message = 'could not open pes file: '//operfile(1:opflaenge)
           routine='potfit'
           call errormsg
        else
           write(ilog,'(/2a)')'Reading pes file: ',operfile(1:opflaenge)
        endif

        chkdvr = 2
        chkgrd = 1
        call operinfo(lerr,chkdvr,chkgrd)

        if (lerr) then
           message='error found in pes file'
           routine='potfit (operinfo)'
           call errormsg
        endif

      end if

!-----------------------------------------------------------------------
! rdnpot and cutnpot: get the dimensions from the natpot file
! so that zeigpfit and mempfit allocate enough mem for natpot
! Recall that the input file (read by peingabe) provides a
! smaller potdim (how much we want evec and coeff to be shortened)
!-----------------------------------------------------------------------
      if(lcutnpot .or. lrdnpot)then

         if(lcutnpot) then
            write(ilog,'(/a)')'--------------------------------'
            write(ilog,'(a)') 'NATPOT file shortening requested'
            write(ilog,'(a)') '--------------------------------'
            call myflush(ilog)
            iout=ilog      ! Output directed to log file (cutnpot)
         endif

         if(lnpotpath)then
            filename=npotname(1:npotlaenge)//'/natpot'
            open(inpot,form='unformatted',status='old',err=370, &
                 file=filename)  ! Try long form (with 'natpot' appended)
            npotname=npotname(1:npotlaenge)//'/natpot'
            npotlaenge=npotlaenge+7
            goto 390

!         Long form failed, try not to append 'natpot'
 370        filename=npotname(1:npotlaenge)
            open(inpot,form='unformatted',status='old',err=898,file=filename)
         else
            filename=name(1:laenge)//'/natpot'
            open(inpot,form='unformatted',status='old',err=898,file=filename)
         endif

 390     write(ilog,'(/a,/a)') 'Reading natpot file',trim(filename)

         chkdvr=2  ! dvrinfo checked against system data
         chkgrd=1  ! grdinfo read, and stored in system data
         old_modc=modc

         call npotinfo(inpot,lerr,modc,loned,ndof1,fdvr,chkdvr, &
              chkgrd,1,mdvr,mmode,nspfdof,spfdof,nmode1,subdim1,pntbas)

         close(inpot)

         if(old_modc.ne.modc) then
            write(6,   '(/a)') 'Inconsistent contracted modes!'
            write(ilog,'(/a)') 'Inconsistent contracted modes!'
            write(6,   '(a,i4,a,i4)') ' input-modc =', old_modc, &
                                '      natpot-modc =', modc
            write(ilog,'(a,i4,a,i4)') ' input-modc =', old_modc, &
                                '      natpot-modc =', modc
            call stopnow
         endif

! since init_cmode uses potdim, swap potdim for pdim

         pdim(modc)=psubdim(modc)
         do m=1,npmode
           olddim(m) = potdim(m)
           potdim(m) = pdim(m)
         enddo
      endif ! if cutnpot .or. rdnpot

      call init_cmode

!-----------------------------------------------------------------------
! allocate memory for general potential to be subtracted
!
! The function to be subtracted has been set up as an MCTDH operator
! and is in an oper file. As the grids for this oper system may be
! different from the potfit system, also need a separate ort array
!-----------------------------------------------------------------------
      if (lsubtract) then
!
! set up "operator" mode variables and save potfit system
!
         nmode=npmode
         do m=1,nmode
           nspfdof(m)=npotdof(m)
         enddo
         do m=1,nmode
           do n=1,nspfdof(m)
             spfdof(n,m)=potdof(n,m)
           enddo
         enddo
         do f=1,ndof
           dofspf(f)=dofpot(f)
         enddo
         if (hopdim .gt. tmplabs) then
            routine='Potfit'
            write(message,'(a,i5)')&
                'Redimension tmplabs. Minimum: ',hopdim
            call errormsg
         endif
         do hl=1,hopdim
            peslab(hl)=hoplab(hl)
         enddo
         pesopts0=pesopts(1)
         do n=1,maxhpar
            zpesopts0(n)=zpesopts(n,1)
         enddo
         hopdim2=hopdim
         call griddat
         call savecsys(1)
!
! read oper file system and allocate memory
! if lensubtr=0, pes file to be read is in name directory and
! system must be identical to potfit system. If lensubtr > 0,
! potfit system must be a superset of pes
!
         if (lensubtr .eq. 0) then
            chkdvr=2
            chkgrd=2
            operfile=name(1:laenge)//'/pes'
            opflaenge=laenge+4
         else
            chkdvr=7
            chkgrd=2
            operfile=subtrpes(1:lensubtr)//'/pes'
            opflaenge=lensubtr+4
         endif
         filename=operfile
         open(ioper,file=filename,form='unformatted',status='old',&
              err=900)
         call operinfo(lerr,chkdvr,chkgrd)
         call chksyserr(filename,lerr)
         if (lerr) go to 999
         close(ioper)

         write(ilog,'(/,a)') 'Potential function to be subtracted: '
         write(ilog,'(a)') filename
!
! now read in operator system if required
!
         if (lensubtr .gt. 0) then
            chkdvr=1
            filename=subtrpes(1:lensubtr)//'/dvr'
            open(idvr,file=filename,form='unformatted',status='old',&
              err=900)
            call dvrinfo(lerr,chkdvr)
            close(idvr)

            chkdvr=2
            chkgrd=1
            open(ioper,file=operfile,form='unformatted',status='old',&
                 err=900)
            call operinfo(lerr,chkdvr,chkgrd)
            close(ioper)
         endif
!
! save operator system
!
         call griddat
         call savecsys(2)
!
! copy label of pes to be fit to end of hoplab list
!
         hopdim1 = hopdim
         do f=1,ndof
            if (onedipot(f) .gt. 0) onedipot(f) = onedipot(f)+hopdim1
         enddo
         do hl=1,hopdim2
            hopdim=hopdim+1
            hoplab(hopdim)=peslab(hl)
         enddo
         pesopts(1)=pesopts0
         do n=1,maxhpar
            zpesopts(n,1)=zpesopts0(n)
         enddo
!
! restore potfit system
!
         call getcsys(1)

      endif

!-----------------------------------------------------------------------
! Calculate pointer structure and array dimensions
!-----------------------------------------------------------------------
 999  continue
#ifdef MPI
      call pf_mpi_init
#endif
      call zeigpfit
      call mempfit
      if (mpirank.gt.0) goto 998

!-----------------------------------------------------------------------
! Write memory consumption to log file.
!-----------------------------------------------------------------------
      if (mpirank.gt.0) goto 999
      progpart='runpfit'
      call logmem(progpart)
      mb=131072.0_dop
      write(ilog,'(/a,i11,i13,",",f11.2," Mb")') 'workdim1,2:', &
            workdim1,workdim2,dble(workdim1+workdim2)/mb
      write(ilog,'(a,i13,",",f11.2," Mb",/a,i13,",",f11.2," Mb")') &
           'Buffer size:  niobuf =', niobuf, dble( niobuf)/mb, &
           'Grid size:   vpotdim =',vpotdim, dble(vpotdim)/mb

!-----------------------------------------------------------------------
! RDNPOT, CUTNPOT: Check potdim (input, peinbas) and pdim (natpot) (compare)
!-----------------------------------------------------------------------
      if(lcutnpot .or. lrdnpot)then

! revert potdim to original (requested) values

         do m=1,npmode
           potdim(m) = olddim(m)
         enddo

         write(ilog,'(/,80a1)') ('-',i=1,80)
         do m=1,npmode
          if(m.ne.modc) then
            if(potdim(m).gt.pdim(m)) then
              write(ilog,'(/a,i3,i4,1x,a,i4)') &
               'ERROR: Requested number of SPPs for mode',m, potdim(m), &
               'is larger than available in natpot file ', pdim(m)
              write(6,'(/a,i3," :",i4,/5x,a,i4)') &
               'ERROR: Requested number of SPPs for mode',m,potdim(m), &
               'is larger than available in natpot file:', pdim(m)
              call stopnow
            else
              write(ilog,'(a,i3,a,i4,1x,a,i4)') &
               ' Requested number of SPPs for mode',m,' is',potdim(m), &
               ' whereas in natpot file is',pdim(m)
            endif
          endif
         enddo

      endif ! if cutnpot .or. rdnpot

      if(lcutnpot .and. lnpotpath) then
         vname=name(1:laenge)//'/vpot'
         inquire(file=vname,exist=lexist)
         if(.not.lexist) then
            filename=npotname(1:npotlaenge-6)//'vpot'
            inquire(file=filename,exist=lexist)
            if(lexist) then
               inptit='ln -s '//trim(filename)//' '//vname
               call excmd2(ierr,inptit)
               if(ierr.eq.0) write(ilog,'(2a)') &
                 'Link for vpot is set in name directory.'
            else
               write(ilog,'(2a)') &
                'There is no vpot file on ',trim(filename)
            endif
         endif
      endif

      if(lpesfile) call memhops_pfit

  998 continue
!-----------------------------------------------------------------------
! set up memory for operator arrays
!-----------------------------------------------------------------------
      if (lsubtract) then
         call memhops
      else
         mrhops=1
         hopsdim=1
      endif

      if (mpirank.gt.0) goto 997

!-----------------------------------------------------------------------
! NOTE that cutnpot for LARGE DVRs may lead to integer overflow
!      make the space reserved for the actual potential equal to zero
!-----------------------------------------------------------------------
      if(lcutnpot) mrvvec = 0

      totmem=0.d0
      totmem=totmem+mrdim
      totmem=totmem+midim
      totmem=totmem+workdim2
      totmem=totmem+mrvvec
      totmem=totmem+coeffdim
      totmem=totmem*8.d0
      write(ilog,'(80a1,/)') ('-',i=1,80)
      write(ilog,'(a)') '  *** Memory allocated dynamically ***'
      write(ilog,'( (a,i12," MB") )') &
       ' real*8     :', mrdim, &
       ' integer    :', midim, &
       ' workdim2   :', workdim2, &
       ' coeffdim   :', coeffdim, &
       ' vvec       :', mrvvec
      write(ilog,'(a,f16.0,a,f11.2,a)') &
       ' Total   :',totmem,' Bytes ,',totmem/1048576.d0,' MB'
      write(ilog,'(80a1,/)') ('-',i=1,80)
      call myflush(ilog)

 997  continue
      if(allocated(mr))deallocate(mr)
      if(allocated(mi))deallocate(mi)
      if(allocated(vvec))deallocate(vvec)
      if(allocated(coeff))deallocate(coeff)
      if(allocated(work2))deallocate(work2)

      cerr = 'pfit_mr '
      allocate(mr(mrdim+1),stat=aerr)
      if (aerr > 0) go to 1001
      cerr = 'pfit_mi '
      allocate(mi(midim+1),stat=aerr)
      if (aerr > 0) go to 1001
      cerr = 'pfit_vvec '
      allocate(vvec(mrvvec),stat=aerr)
      if (aerr > 0) go to 1001
      cerr = 'pfit_coeff '
      allocate(coeff(coeffdim),stat=aerr)
      if (aerr > 0) go to 1001
      cerr = 'pfit_work2 '
      allocate(work2(workdim2),stat=aerr)
      if (aerr > 0) go to 1001

!-----------------------------------------------------------------------
! Zero memory arrays
!-----------------------------------------------------------------------
      call zerovxd(mr,mrdim)
      call zerovxi(mi,midim)
      call zerovxd(vvec,mrvvec)
      call zerovxd(coeff,coeffdim)
      call zerovxd(work2,workdim2)
#ifdef MPI
      if (mpirank.gt.0) then
         call pf_mpi_init2(mr(mrort))
         call mpi_bcast(mpitask,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
         call pf_mpi_slave(mpitask,mr)
         return
      endif
#endif

!-----------------------------------------------------------------------
! Read DVR data (see rddvr.f for details)
!-----------------------------------------------------------------------
      dvrdata(:)=.false.
      dvrdata(1)=.true.
      chkdvr=6

      call rddvr(mr(mrort),rdum,rdum,rdum,zdum,zdum,&
                 idum,zdum,zdum,idum,idum,rdum,chkdvr)
      close(idvr)

      if (lsubtract) then
         call getcsys(2)
         chkdvr=0
         chkgrd=0
         if (lensubtr .gt. 0) then
            dname=subtrpes(1:lensubtr)
            dlaenge=lensubtr

            chkdvr=0
            call rddvr(mr(mrort1),rdum,rdum,rdum,zdum,zdum,&
                       idum,zdum,zdum,idum,idum,rdum,chkdvr)
            close(idvr)

         endif
         pesopts0=pesopts(1)
         do n=1,maxhpar
            zpesopts0(n) = zpesopts(n,1)
         enddo
         call rdoper(mr(mrhops),chkdvr,chkgrd)
! remember label for PES to be fit is at end of list
         pesopts(1)=pesopts0
         do n=1,maxhpar
            zpesopts(n,1) = zpesopts0(n)
         enddo
         hopdim=hopdim+1
         close(ioper)

! restore potfit system
         call getcsys(1)

! form cut of PES
         if (lensubtr .gt. 0) then
            call pescut(mr(mrhops),mr(mrort1),mr(mrort))
         endif

      endif

!-----------------------------------------------------------------------
!  Read external relevant region weights, if required
!-----------------------------------------------------------------------
      if(lwpot) then
         open(unit=iwpot,status='old',file=wname, &
              form='formatted',iostat=ierr)
         if (ierr.ne.0) then
            message='could not open '//trim(wname)
            routine='runpfit'
            call errormsg
         endif
         do i=1,vpotdim/200 + 2
            read(iwpot,*,end=33)
         enddo
         lpacked=.false.
         goto 34
  33     lpacked=.true.
  34     rewind (iwpot)
         write(ilog,'(/,76a1)')  ('-',i=1,76)
         write(ilog,'(2a)') ' READING WPOT file: ',wname(1:wlength)
         write(ilog,'(76a1)')  ('-',i=1,76)
         if(lpacked) then
            write(ilog,'(a)') ' PACKED MODUS'
            write(ilog,'(76a1)')  ('-',i=1,76)
            read(iwpot,'(7i11)') (mi(miwvec+j),j=0,vpotdim/30)
         else
            write(ilog,'(a,l2)') ' UNPACKED MODUS. lwmap =',lwmap
            write(ilog,'(76a1)')  ('-',i=1,76)
            woffset=miwvec
            ashift=1
            ccw=0
            if(lwmap) then
               do i=1,vpotdim
                  call indexmap(i,j) ! Because of indexmap one has to
                  read(iwpot,*) cw   ! store the weights before compacting
                  vvec(j) = cw*1.1d0 ! vvec is used as storrage.
               enddo
               do i=1,vpotdim
                  cw = int(vvec(i))
                  ccw = ccw + cw*ashift
                  ashift=ashift*2
                  if(ashift.eq.2**30 .or. i.eq.vpotdim) then
                     mi(woffset)=ccw
                     woffset=woffset+1
                     ashift=1
                     ccw=0
                  endif
               enddo

            else
               do i=1,vpotdim
                  read(iwpot,*) cw
                  ccw = ccw + cw*ashift
                  ashift=ashift*2
                  if(ashift.eq.2**30 .or. i.eq.vpotdim) then
                     mi(woffset)=ccw
                     woffset=woffset+1
                     ashift=1
                     ccw=0
                  endif
               enddo
            endif  ! lwmap
         endif     ! lpacked
      endif        ! lwpot

!-----------------------------------------------------------------------
! Calculate exact potential energy values on grid and write them to vpot.
! Exact potential surface may have an energy cut-off depending on
! logical 'lcutpes'.
!-----------------------------------------------------------------------

! CUTNPOT: If cutnpot then avoid reading or computation of vpot
      if(lcutnpot)then
         write(ilog,'(/a,a)')' Requested cutnpot, ', &
           ' skipping computation of vpot'
         call directfit(work2,coeff,mr,mi,vvec)
         return
      endif

! SPENERGY OR CUTALONG: indicate that vpot computation will be avoided
      if(.not.allocated(done))allocate(done(maxhop))  
      done(:) = 0

      if(lspenergy) then
         write(ilog,'(/a,a)')' Requested single point energy,', &
           ' calculation skipping computation of vpot'
         write(iout,'(/a,a)')' Requested single point energy,', &
           ' calculation skipping computation of vpot'

         call spenergy(mr(mrort),mr(mrevec),coeff,mr(mroned),work2,mr(mrhops),&
              done)
         if(allocated(done))deallocate(done)
         stop
      endif

      if(lcutalong) then
         write(ilog,'(/a,a)')' Requested cut along the PES,', &
           ' calculation skipping computation of vpot'
         write(iout,'(/a,a)')' Requested cut along the PES,', &
           ' calculation skipping computation of vpot'

         call cutalong(mr(mrort),mr(mrevec),coeff,mr(mroned),work2,mr(mrhops),&
         done)
         if(allocated(done))deallocate(done)
         stop
      endif

!-----------------------------------------------------------------------
! Calculate exact potential energy values on grid and write them to vpot. 
! logical 'lcutpes'.
!-----------------------------------------------------------------------
      call getpes(mr(mrort),vvec,mr(mroned),mr(mrbuf),mr(mrhops),done)

!-----------------------------------------------------------------------
!  Compute correlated weights if a correlated weights section exists
!  and weights have not been read.
!-----------------------------------------------------------------------
      if(lcw .and. .not.lwpot .and. .not. lwo) then
         call get_timer(tid, 'c-weights')
         call start_timer(tid)
         offset=1
         woffset=miwvec
         ashift=1
         ccw=0
         call initvxi(igpoint,ndof)
 10      call index2coord(igpoint,mr(mrort),gpoint)
         v=vvec(offset)
         offset=offset+1
         call compiweight(gpoint,v,cw)
         ccw = ccw + cw*ashift
         ashift=ashift*2
         if(ashift.eq.2**30) then
            mi(woffset)=ccw
            woffset=woffset+1
            ashift=1
            ccw=0
         endif
         do m=1,npmode
            do n=1,npotdof(m)
               f=potdof(n,m)
               if (igpoint(f).lt.gdim(f)) then
                  igpoint(f)=igpoint(f)+1
                  goto 10
               else
                  igpoint(f)=1
               endif
            enddo
         enddo
         call stop_timer(tid)
      endif

! --- Write weights, if required. (Packed and in mode order).
      if(lwrcw) then
         wname=name(1:laenge)//'/corrweights'
         wlength=laenge+12
         open(unit=iwpot,status='unknown',file=wname, &
               form='formatted',iostat=ierr)
         if (ierr.ne.0) then
             message='could not open '//wname(1:wlength)
             routine='runpfit'
             call errormsg
         endif
         write(iwpot,'(7i11)') (mi(miwvec+j),j=0,vpotdim/30)

         if(.not.lextpot) return ! Do nothing else but writing correlated weights

      endif

!-----------------------------------------------------------------------
! Write grid points to channel 'iout'.
!-----------------------------------------------------------------------
      if(loutl) call wrgrid(mr(mrort))

!-----------------------------------------------------------------------
! Write one-dimensional potential curves to channel 'iout'.
!-----------------------------------------------------------------------
      if(loutl) call wroned(mr(mroned))

!-----------------------------------------------------------------------
! return here if write-only mode has been chosen
!-----------------------------------------------------------------------
      if (lwo) return

!-----------------------------------------------------------------------
! start timer `ptfirstfit'
!-----------------------------------------------------------------------
      if(.not.lrdnpot .and. .not. lcont) then
         call timout
         call fzeit(actual)
         i = index(actual,':')
         actual=actual(i-9:i+6)
         cpu=fcputime(0)+fsystime(0)
         write(ilog,'(/,76a1)')  ('-',i=1,76)
         write(ilog,'(a,9x,a16,4x,"CPU =",f10.1)') &
             ' Starting First Fit.',actual,cpu
         write(ilog,'(76a1)')  ('-',i=1,76)
         call myflush(ilog)
      endif

!-----------------------------------------------------------------------
! call different routines for single- and double precision. 
!-----------------------------------------------------------------------
      call directfit(work2,coeff,mr,mi,vvec)
      
!-----------------------------------------------------------------------
! deallocate memory
!-----------------------------------------------------------------------
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_runpfitmod
      deallocate(igpoint)
      deallocate(olddim)
      deallocate(gpoint)
      deallocate(mmode)
      deallocate(fdvr)
      deallocate(mdvr)
      deallocate(subdim1)
      deallocate(loned)
      if(allocated(done))deallocate(done)

      cerr = 'pfit_...'
      deallocate(mr,mi,vvec,coeff,work2,stat=aerr)
      if (aerr > 0) go to 1002


      return
  898 routine='Runpfit'
      message='Cannot open file '//name(1:laenge)//'/natpot'
      call errormsg

  900 routine='Runpfit'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg

 1001 write (6,*) 'Error in memory allocation of ',cerr,' Error code is ',aerr
      stop

 1002 write (6,*) 'Error in memory Deallocation of ',cerr,' Error code is ',aerr
      stop

      end subroutine runpfit

