
      module runprojmod

use decimal, only:long,dop
use maxv
use global
use globmemmod
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use pdefaultmod, only:init_cmode
use combined
use channels
use timing
use mpidata
#ifdef MPI
   use mpi
#endif

use op1lib, only: zerovxz, zerovxd, zerovxi, zerovxs, zerovxl
use rddvrmod, only: rddvr
use getpesmod, only: getpes
use projutil, only: projrun,memproj,zeigproj
use rddvrmod

      implicit none
      private
      public :: runproj

      contains

!***********************************************************************
!
!                        RUNPROJ
!
!***********************************************************************

      subroutine runproj


      implicit none

      real(dop), dimension(:), allocatable       :: vvec

      integer(long)                     :: chkdvr,f,aerr,ilbl
      integer(long), dimension(1)       :: idum
      integer(long), dimension(:), allocatable :: done
      real(dop), dimension(1)           :: rdum
      complex(dop), dimension(1)        :: zdum
      real(dop), external               :: fcputime,fsystime
      character(len=10)                 :: cerr
      logical(kind=4)                   :: lerr,check
#ifdef MPI
      logical(kind=4), dimension(2) :: mpicomml
      integer(long), dimension(4)   :: mpicommi
      real(dop), dimension(2)       :: mpicommr
#endif

      character(len=c5)                 :: filename

      if (mpirank .gt. 0) go to 980
      aerr = 0

!-----------------------------------------------------------------------
! get memory sizes from DVR file
!-----------------------------------------------------------------------
      filename=dname(1:dlaenge)//'/dvr'
      ilbl=index(filename,' ')-1
      open(idvr,file=filename,form='unformatted',status='old',err=900)
      call rdmemdim(idvr)
      close(idvr)

!-----------------------------------------------------------------------
! Allocate mememory and read system / DVR information
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_combined
      call alloc_runpfitmod

      filename=dname(1:dlaenge)//'/dvr'
      ilbl=index(filename,' ')-1
      open(idvr,file=filename,form='unformatted',status='old',err=900)
      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      close(idvr)

!-----------------------------------------------------------------------
! open input file for reading
!-----------------------------------------------------------------------
      inquire(iin,opened=check)
      if (.not. check) then
         filename=inputfile
         open(iin,file=filename,status='old',err=900)
      endif

!-----------------------------------------------------------------------
! get dimensions for operator
! allocate memory for operator and combined grids
!-----------------------------------------------------------------------
      call pmemdimoper
      call alloc_operdef

!-----------------------------------------------------------------------
! Initialise timing arrays
!-----------------------------------------------------------------------
      call timinit

!-----------------------------------------------------------------------
! read pes function
!-----------------------------------------------------------------------
      call peinoper

!-----------------------------------------------------------------------
! read remaining input sections (weights etc)
!-----------------------------------------------------------------------
      call projein

!-----------------------------------------------------------------------
! set variables of common-block `cmode' 
!-----------------------------------------------------------------------
      call init_cmode

      close(iin)
!-----------------------------------------------------------------------
! This may become neccessary if the contracted mode uses Leg/R-DVR.
!-----------------------------------------------------------------------
!      do i = 1, ndof
!         potdim(i) = min(potdim(i),gdim(i))
!      enddo


!-----------------------------------------------------------------------
! Calculate pointer structure and array dimensions
!-----------------------------------------------------------------------
   980 continue
#if MPI
      call proj_mpi_init
#endif
      call zeigproj
      call memproj

      if(allocated(mc))deallocate(mc)
      if(allocated(mr))deallocate(mr)
      if(allocated(mi))deallocate(mi)
      if(allocated(ml))deallocate(ml)
      if(allocated(vvec))deallocate(vvec)

      cerr = 'proj_mc '
      allocate(mc(mcdim+1),stat=aerr)
      if (aerr > 0) go to 1001
      cerr = 'proj_mr '
      allocate(mr(mrdim+1),stat=aerr)
      if (aerr > 0) go to 1001
      cerr = 'proj_mi '
      allocate(mi(midim+1),stat=aerr)
      if (aerr > 0) go to 1001
      cerr = 'proj_ml '
      allocate(ml(mldim+1),stat=aerr)
      if (aerr > 0) go to 1001
      cerr = 'proj_vvec '
      allocate(vvec(vpotdim),stat=aerr)
      if (aerr > 0) go to 1001

!-----------------------------------------------------------------------
! Zero memory arrays
!-----------------------------------------------------------------------
      call zerovxz(mc,mcdim)
      call zerovxd(mr,mrdim)
      call zerovxi(mi,midim)
      call zerovxl(ml,mldim)
      call zerovxd(vvec,vpotdim)
      if (mpirank.gt.0) goto 999

!-----------------------------------------------------------------------
! Read DVR data (see rddvr.f for details)
!-----------------------------------------------------------------------
      dvrdata(:)=.false.
      dvrdata(1)=.true.
      dvrdata(2)=.true.
      chkdvr=2

!     For projection runs we also need the DVR weights.
!     Of course these are *not* stored in the DVR file.
!     So we need to read the trafo matrices to calculate them.
      call rddvr(mr(mrort),mr(mrtrafo),rdum,rdum,zdum,zdum,&
                 idum,zdum,zdum,idum,idum,rdum,chkdvr)

!-----------------------------------------------------------------------
! set arrays vgmdim and vgpdim, which contains the `vor'-grid-dimensions
! in mode order and in dof order. (Here dummy setting).
!-----------------------------------------------------------------------
  999 continue
      vgpdim(1)=1
      vgmdim(1)=1
      do f=1,ndof-1  
         vgpdim(f+1)=vgpdim(f)*gdim(f)
         vgmdim(f+1)=vgpdim(f+1)
      enddo
      if (mpirank.gt.0) goto 998

!-----------------------------------------------------------------------
! Calculate exact potential energy values on grid and write them to vpot. 
! Exact potential surface may have an energy cut-off depending on 
! logical 'lcutpes'.
!-----------------------------------------------------------------------
      write(ilog,'(/a,2i10)') 'vpotdim, niobuf',vpotdim,niobuf
      hopsdim=1
      if(.not.allocated(done))allocate(done(maxhop))
      done(:) = 0
      call getpes(mr(mrort),vvec,mr(mroned),mr(mrbuf),rdum,done)
      write(ilog,'(/a,2i10)') 'vpotdim, niobuf',vpotdim,niobuf
      
!-----------------------------------------------------------------------
! Transmit data (about PES) to MPI slaves.
!-----------------------------------------------------------------------
 998  continue
#ifdef MPI
      if (.not.lmpi) goto 899
      if (mpirank.eq.0) then ! master
         mpicomml(1)=lpesfile
         mpicomml(2)=lcutpes
         call MPI_Bcast(mpicomml,2,MPI_LOGICAL,0,MPI_COMM_WORLD,mpierr)
         mpicommi(1)=fkoe
         mpicommi(2)=ftime
         mpicommi(3)=ippes
         mpicommi(4)=oneddim
         call MPI_Bcast(mpicommi,4,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
         mpicommr(1)=pesmax
         mpicommr(2)=pesmin
         call MPI_Bcast(mpicommr,2,MPI_DOUBLE_PRECISION, &
              0,MPI_COMM_WORLD,mpierr)

      else ! slave
         call MPI_Bcast(mpicomml,2,MPI_LOGICAL,0,MPI_COMM_WORLD,mpierr)
         lpesfile   =mpicomml(1)
         lcutpes    =mpicomml(2)
         call MPI_Bcast(mpicommi,4,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
         fkoe       =mpicommi(1)
         ftime      =mpicommi(2)
         ippes      =mpicommi(3)
         oneddim    =mpicommi(4)
         call MPI_Bcast(mpicommr,2,MPI_DOUBLE_PRECISION, &
              0,MPI_COMM_WORLD,mpierr)
         pesmax     =mpicommr(1)
         pesmin     =mpicommr(2)
      endif

!     Grid Information
      call MPI_Bcast(zort,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)

!     Operator Information
      call MPI_Bcast(hopfile,maxhop,MPI_INTEGER, &
           0,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hopipar,maxhpar*maxhop,MPI_INTEGER, &
           0,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hoprpar,maxhpar*maxhop,MPI_DOUBLE_PRECISION, &
           0,MPI_COMM_WORLD,mpierr)
#endif

!-----------------------------------------------------------------------
! return here if write-only mode has been chosen
!-----------------------------------------------------------------------
  899 if (lwo) return

!-----------------------------------------------------------------------
! projection runs starts here
!-----------------------------------------------------------------------
      call projrun(vvec,mr(mrort),mr(mrtrafo),mr(mrvproj),mr(mrvperr))

!-----------------------------------------------------------------------
! deallocate memory
!-----------------------------------------------------------------------
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_runpfitmod
      if(allocated(done))deallocate(done)

      cerr = 'proj_...'
      deallocate(mc,mr,mi,ml,stat=aerr)
      if (aerr > 0) go to 1002

      return
  900 continue
      routine='Runpfit'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop

 1001 write (6,*) 'Error in memory allocation of ',cerr,' Error code is ',aerr
      stop

 1002 write (6,*) 'Error in memory Deallocation of ',cerr,' Error code is ',aerr
      stop

      end subroutine runproj

!#######################################################################


!-----------------------------------------------------------------------
! Transmit essential information from master to slaves,
! because the slaves didn't read the input file.
!-----------------------------------------------------------------------
#ifdef MPI
      subroutine proj_mpi_init

      implicit none

      logical      mpicomml(2)
      integer      mpicommi(7)

      if (mpirank.eq.0) then ! master
         if (.not.lmpi) return
         mpicomml(1)=lprojerr
         mpicomml(2)=lprojlowmem
         call MPI_Bcast(mpicomml,2,MPI_LOGICAL,0,MPI_COMM_WORLD,mpierr)
         mpicommi(1)=ndof
         mpicommi(2)=nmode
         mpicommi(3)=nstate
         mpicommi(4)=nproj
         mpicommi(5)=ortdim
         mpicommi(6)=dvrdim
         mpicommi(7)=maxgdim
         call MPI_Bcast(mpicommi,7,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)

      else ! slave
         lmpi = .true.
         call MPI_Bcast(mpicomml,2,MPI_LOGICAL,0,MPI_COMM_WORLD,mpierr)
         lprojerr   =mpicomml(1)
         lprojlowmem=mpicomml(2)
         call MPI_Bcast(mpicommi,7,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
         ndof   =mpicommi(1)
         nmode  =mpicommi(2)
         nstate =mpicommi(3)
         nproj  =mpicommi(4)
         ortdim =mpicommi(5)
         dvrdim =mpicommi(6)
         maxgdim=mpicommi(7)
      endif

!     Grid Information
      call MPI_Bcast(gdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(vgdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(ngdim,maxdim,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)

!     Projector Information
      call MPI_Bcast(nprojdof,maxproj,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(projdof,maxproj*maxprojdof,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(projscale,maxproj,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,mpierr)

      end ! of proj_mpi_init
#endif


      end module runprojmod
