!***********************************************************************
      module spenergymod

use decimal, only:long,dop
use logdat
use timing

      implicit none
      private
      public :: spenergy,cutalong

      contains


!#######################################################################
!   spenergy computes the vpot and npot energies of a single grid-point.
!   work2, evec, coeff are necessary for the single-point energy
!   calculations
!#######################################################################

      subroutine spenergy(ort,evec,coeff,oned,work2,hops,done)

use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels

use directutil, only: index2coord
use rdopermod, only: rdoper
use operutil, only: vcalc1st
use psiutil, only: f2mdecoder

      implicit none

      integer(long), dimension(maxdim) :: mmode,mdvr,fdvr
      integer(long)                    :: f
      integer(long), dimension(nmode)  :: mpoint
      integer(long), dimension(ndof)   :: igp
      integer(long)                    :: chkdvr, chkgrd
      integer(long), dimension(maxhop), intent(inout) :: done
      real(dop),dimension(ortdim), intent(in)    :: ort
      real(dop),dimension(oneddim), intent(out)  :: oned
      real(dop),dimension(maxdim)                :: sppoint
      real(dop), dimension(coeffdim), intent(out) :: coeff
      real(dop), dimension(evecdim), intent(out) :: evec
      real(dop), dimension(workdim2), intent(out) :: work2
      real(dop), dimension(hopsdim), intent(out) :: hops
      real(dop)                                  :: v
      logical  lexist,loned(maxdim)
      character*(c1) af
      integer(long), save :: tid=0, tid1=0, tid2=0 ! timer IDs

      do f=1, ndof
        if(isppoint(f).gt.gdim(f))then
           write(af,'(i2)')f
           routine = 'getpes (pbasis-section)'
           message = 'In mode= ' //af// &
              '  sp-energy requested index' // &
              ' is larger than the corresponding' // &
              ' number of DVR points '
           call errormsg
        elseif(isppoint(f).le.0)then
           message = 'In mode= ' // af // &
              '  sp-energy index' // &
              ' must be greater than zero'
        end if
      end do

      write(iout,'(/,a,a)') '***** Single point energy ', &
            'calculation *****'
      write(ilog,'(/,a,a)') '***** Single point energy ', &
            'calculation *****'
      write(6,'(/,a,a)') '***** Single point energy ', &
            'calculation *****'

      call index2coord(isppoint,ort,sppoint)

!------------------------------------------------
! In the lpesfile case mpoint(nmode) included
!------------------------------------------------

      if (lpesfile) then
! This is to get hops from pes file
         chkdvr = 2
         chkgrd = 1
         call rdoper(hops,chkdvr,chkgrd)
         
         do f=1, ndof
            igp(f) = isppoint(f)
         end do
         call f2mdecoder(igp,mpoint)
         call get_timer(tid, 'Vcalc1st')
         call start_timer(tid)
         call vcalc1st(hops,v,mpoint,1)
         call stop_timer(tid)
      else
         call get_timer(tid1, 'Spenergypes')
         call start_timer(tid1)
         call spenergypes(sppoint,hops,v,done)
         call stop_timer(tid1)
      end if
       
      write(iout,'(/,a,99(1x,i4))') ' Requested DVR indices : ', &
        (isppoint(f), f=1,ndof)
      write(iout,'(/,a,99(1x,f8.4))') ' Corresponding DVR values : ', &
        (sppoint(f), f=1,ndof)
      write(iout,'(/,a,2(1x,f16.12))') ' PES energy= ', v

      write(6,'(/,a,99(1x,i4))') ' Requested DVR indices : ', &
        (isppoint(f), f=1,ndof)
      write(6,'(/,a,99(1x,f8.4))') ' Corresponding DVR values : ', &
        (sppoint(f), f=1,ndof)
      write(6,'(/,a,2(1x,f16.12))') ' PES energy= ', v

! If natpot provided, then compute also potfit sp-energy

      inquire(file=name(1:laenge)//'/natpot', exist=lexist)

      if (lexist) then
         open(inpot,form='unformatted',status='old',err=899, &
           file=name(1:laenge)//'/natpot')

         write(iout,'(/,a)')' A natpot file was found and the ' // &
                 'fitted energy will be computed'
         write(6,'(/,a)')' A natpot file was found and the ' // &
                 'fitted energy will be computed'

         call get_timer(tid2, 'Spenergyfit')
         call start_timer(tid2)
         call spenergyfit(fdvr,mdvr,mmode,loned,oned,coeff,evec,work2,v)
         call stop_timer(tid2)

         close (inpot)

         write(iout,'(/,a,f16.12,4x,e12.4,a)') ' Fitted energy= ', &
               v, v*27.211396131788d0, ' eV'
         write(6,'(/,a,f16.12)') ' Fitted energy= ', v

      else

         write(iout,'(/,a)')' No natpot file was found and hence ' // &
                 'no fitted energy was computed'
         write(6,'(/,a)')' No natpot file was found and hence ' // &
                 'no fitted energy was computed'

      end if

      write(6,'(a)')
      write(6,'(a)') ' ***************************************'
      write(6,'(a)') ' *  sp-energy successfully completed!  *'
      write(6,'(a)') ' ***************************************'
      write(6,'(a)')

      return

899   routine='getpes (sp-energy)'
      message='Cannot open file '//name(1:laenge)//'/natpot'
      call errormsg

      end subroutine spenergy

!-------------------------------------------------------------------------
!     Subroutine cutalong (DPR)
!       computes a cut along the PES for a specified DOF (selmode)
!       The point given by the DVR indices contained in isppoint
!-------------------------------------------------------------------------

      subroutine cutalong(ort,evec,coeff,oned,work2,hops,done)

use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels

use directutil, only: index2coord
use rdopermod, only: rdoper
use operutil, only: vcalc1st
use psiutil, only: f2mdecoder

      implicit none

      integer(long)                      :: f,i
      integer(long), dimension(maxdim)   ::  mmode,mdvr,fdvr
      integer(long), dimension(ndof)     :: igp
      integer(long), dimension(nmode)    :: mpoint
      integer(long)                      ::  chkdvr, chkgrd
      integer(long), dimension(maxhop), intent(inout) :: done
      logical(kind=4)                    :: lexist
      logical(kind=4), dimension(maxdim) :: loned
      character(len=c1)                  :: af
      character(len=c5)                  :: filename
      real(dop),dimension(ortdim), intent(in)    :: ort
      real(dop),dimension(oneddim), intent(out)  :: oned
      real(dop),dimension(maxdim)                :: sppoint
      real(dop), dimension(coeffdim), intent(out) :: coeff
      real(dop), dimension(evecdim), intent(out) :: evec
      real(dop), dimension(workdim2), intent(out) :: work2
      real(dop), dimension(hopsdim), intent(out) :: hops
      real(dop)                                  :: vpes, vpotfit, vdiff

      do f=1, ndof
        if(f.ne.selmode) then
          if(isppoint(f).gt.gdim(f))then
             write(af,'(i2)')f
             routine = 'getpes (pbasis-section)'
             message = 'In mode= ' //af// &
              '  sp-energy requested index' // &
              ' is larger than the corresponding' // &
              ' number of DVR points '
             call errormsg
          elseif(isppoint(f).lt.0)then
             message = 'In mode= ' // af // &
              '  sp-energy index' // &
              ' must be greater than zero'
          end if
        end if
      end do

      write(iout,'(/,a,i2,a)') '***** PES cut along mode ', selmode, &
            ' requested *****'
      write(ilog,'(/,a,i2,a)') '***** PES cut along mode ', selmode, &
            ' requested *****'
      write(6,'(/,a,i2,a)') '***** PES cut along mode ', selmode, &
            ' requested *****'

      if (lcutpes) then
         write(iout,'(/,a,f5.2,a,f5.2)') &
           'WARNING: vcut was requested pesmax= ', &
           pesmax, ' pesmin = ', pesmin
         write(ilog,'(/,a,f5.2,a,f5.2)') &
           'WARNING: vcut was requested pesmax= ', &
           pesmax, ' pesmin = ', pesmin
         write(6,'(/,a,f5.2,a,f5.2)') &
           'WARNING: vcut was requested pesmax= ', &
           pesmax, ' pesmin = ', pesmin
      elseif (lsoftcutpes) then
         write(iout,'(/,a,f5.2,a,f5.2)') &
           'WARNING: vcut was requested pesmax= ', &
           pesmax, ' pesmin = ', pesmin
         write(ilog,'(/,a,f5.2,a,f5.2)') &
           'WARNING: vcut was requested pesmax= ', &
           pesmax, ' pesmin = ', pesmin
         write(6,'(/,a,f5.2,a,f5.2)') &
           'WARNING: vcut was requested pesmax= ', &
           pesmax, ' pesmin = ', pesmin
      endif

! The output file

      write(filename,'(2a,i2.2)') name(1:laenge),'/cutalong', selmode

      open(unit=600,file=filename,form='formatted',status='unknown')

! If natpot provided, then compute also potfit cutalong

      inquire(file=name(1:laenge)//'/natpot', exist=lexist)

      if (lexist) then
         write(iout,'(/,a)')' A natpot file was found and the ' // &
                 'fitted energy will be computed'
         write(6,'(/,a)')' A natpot file was found and the ' // &
                 'fitted energy will be computed'

      else
          write(iout,'(/,a)')' No natpot file was found and hence ' // &
                 'no fitted energy was computed'
          write(6,'(/,a)')' No natpot file was found and hence ' // &
                 'no fitted energy was computed'

      end if


      write(600,'(a,6x,a,11x,a)') '#  DVR index    DVR value', &
         'E_pes           E_fit','DIFF (au)        DIFF (eV)'

      do i=1, gdim(selmode)

         isppoint(selmode) = i
         call index2coord(isppoint,ort,sppoint)

         write(iout,'(/,a,99(1x,i4))') ' Requested DVR indices : ', &
           (isppoint(f), f=1,ndof)
         write(iout,'(/,a,99(1x,f8.4))') ' Corresponding DVR values : ', &
           (sppoint(f), f=1,ndof)

         write(6,'(/,a,99(1x,i4))') ' Requested DVR indices : ', &
           (isppoint(f), f=1,ndof)
         write(6,'(/,a,99(1x,f8.4))') ' Corresponding DVR values : ', &
           (sppoint(f), f=1,ndof)

         write(iout,'(/,a,i2,a,i2,a,f8.4,/)') &
           ' Requested mode= ', selmode, &
           ' Current DVR index= ', i, &
           ' Current DVR value= ', sppoint(selmode)

!-----------------------------------------------
! In the lpesfile case mpoint(nmode) included
!-----------------------------------------------

         if (lpesfile) then
!-----------------------------------------------
! This is to get hops from pes file
!-----------------------------------------------
           chkdvr = 2
           chkgrd = 1
           call rdoper(hops,chkdvr,chkgrd)

           do f=1, ndof
             igp(f) = isppoint(f)
           end do

           call f2mdecoder(igp,mpoint)
           call vcalc1st(hops,vpes,mpoint,1)

         else

           call spenergypes(sppoint,hops,vpes,done)

         end if ! lpesfile

         if (lexist) then

           open(inpot,form='unformatted',status='old',err=899, &
                file=name(1:laenge)//'/natpot')

            call spenergyfit(fdvr,mdvr,mmode,loned,oned,coeff,evec,work2,vpotfit)

            vdiff = vpes - vpotfit

            close(inpot)

         else

            vpotfit = 0.d0
            vdiff   = 0.d0

         end if

         write(iout,'(5x,i2,9x,f8.4,4(1x,f16.12))') &
          i, sppoint(selmode), vpes, vpotfit,vdiff, vdiff * 27.21138386d0

         write(600,'(5x,i2,9x,f8.4,4(1x,f16.12))') &
          i, sppoint(selmode), vpes, vpotfit, &
          vpes - vpotfit, (vpes - vpotfit) * 27.21138386d0

      end do

      close(600)

!      if (lexist) close(inpot)

      write(6,'(a)')
      write(6,'(a)') ' ***************************************'
      write(6,'(a)') ' *  cutalong successfully completed!  *'
      write(6,'(a)') ' ***************************************'
      write(6,'(a)')

      return

899   routine='getpes (sp-energy)'
      message='Cannot open file '//name(1:laenge)//'/natpot'
      call errormsg

      end subroutine cutalong

!-------------------------------------------------------------------------
!     Subroutine spenergyfit (DPR)
!       computes the single point energy (potfit) for
!       the point given by the DVR indices contained in isppoint
!-------------------------------------------------------------------------

      subroutine spenergyfit(fdvr,mdvr,mmode,loned,oned,coeff,evec,work2,v)

use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels
use rdnatpot

use pesfit, only: getvfit
use getpesmod, only: getv1d
use fitinmod, only: fitin
use pdefaultmod, only: init_cmode

      implicit none

      integer(long) ::  chkdvr, chkgrd
      integer(long), dimension(maxdim), intent(out) :: mmode,mdvr,fdvr
      integer(long), dimension(maxdim)             :: subdim1,pntbas
      real(dop)                                    :: v, v1d
      real(dop),dimension(oneddim), intent(out)  :: oned
      real(dop), dimension(coeffdim), intent(out) :: coeff
      real(dop), dimension(evecdim), intent(out) :: evec
      real(dop), dimension(workdim2), intent(out) :: work2
      logical(kind=4), dimension(maxdim)          :: loned
      logical(kind=4)                             :: lerr

      chkdvr=2
      chkgrd=1

      call npotinfo(inpot,lerr,modc,loned,ndof,fdvr,chkdvr, &
           chkgrd,1,mdvr,mmode,nspfdof,spfdof,nmode,subdim1,pntbas)
      call init_cmode
      call fitin(inpot,loned,oned,evec,coeff)
      call getvfit(isppoint,coeff,evec,work2,v)
      call getv1d(isppoint,oned,v1d)
      v=v+v1d

      return
      end subroutine spenergyfit

!-------------------------------------------------------------------------
!     Subroutine spenergypes (DPR)
!       computes the single point energy (PES routine) for
!       the point given by the DVR values contained in sppoint
!-------------------------------------------------------------------------

      subroutine spenergypes(sppoint,hops,v,done)

use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels

use callmuldmod, only: defmuld, callmuld
use funcsrf, only: surfinfo

      implicit none

      integer(long) ::  h, hl, ierr, ddst
      integer(long), dimension(maxhop), intent(inout) :: done
      real(dop), dimension(hopsdim), intent(in) :: hops
      real(dop), dimension(maxdim), intent(out) :: sppoint
      real(dop)                                 :: v

!-----------------------------------------------------------------------
! search for surface in library
!-----------------------------------------------------------------------

      hl=1
      call defmuld(hopilab(hl),hoplab(hl),hopfile(hl),pesopts,zpesopts, &
              hopipar(1,hl),hoprpar(1,hl),maxhpar,pesmin,pesmax, &
              lcutpes,ierr,ndof,hoppar)

      if (ierr .ne. 0) then
         routine='defmuld'
         call errormsg
      else if (hopfile(hl).eq.0) then
         routine='getpes line 57. (defmuld)'
         message='PES not found in library : '//hoplab(hl)
         call errormsg
      endif

      h=1
      htmmuld(h)=1
      call surfinfo(h,hl,done)

      write(ilog,'(/,a)') 'Surface to be fitted : '//hoplab(1)
      write(ilog,'(a,e20.8)') 'max pes cut (ev) : ',pesmax*ev
      write(ilog,'(a,e20.8)') 'min pes cut (ev) : ',pesmin*ev

!-----------------------------------------------------------------------
! sp-energy calculation
!-----------------------------------------------------------------------

      ddst = 1
      call callmuld(v,sppoint,ndof,hopilab(1),hopfile(1), &
                 hopipar(1,1),hoprpar(1,1),ddst,ddst,hoplab(hl))

! vcut and softvcut

      if (lcutpes) then
         v=min(v,pesmax)
         v=max(v,pesmin)
      elseif (lsoftcutpes .and. v.ge.pesmax) then
         v=min(v,pesmax*(1.d0+log(v/pesmax)))
         v=max(v,pesmin)
      endif

      return
      end subroutine spenergypes


      end module spenergymod
