!************************************************************************
      module vecsmod

use decimal, only:long,dop,sip
use logdat
use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined
use channels

use wrfitmod, only: vecout

use potmod_interface

use rmlib,   only: rmsxcxdd, rmsxcxrrd
use timing
use diagrsmod

      implicit none
      private
      public :: vecs
      contains

!************************************************************************
!
!              POTFIT   subroutine  VECS
!
!   11/96 AJ 
!
!
!************************************************************************

      subroutine vecs(vpot,evec,rho,eval,work)

      implicit none
  
      integer(long)                                          :: f,bef,aft, &
                                                                ioformat,m
      integer(long)                                          :: i, j, k, ii, jj
      real(dop), dimension(evecdim), intent(out)             :: evec
      real(dop), dimension(maxpsub2), intent(out)            :: rho
      real(dop), dimension(evaldim), intent(out)             :: eval
      real(dop), dimension((maxpsub+2)*maxpsub), intent(out) :: work
      real(dop), dimension(vpotdim), intent(in)              :: vpot
      character(len=240)                                     :: filename
      integer(long), save                             :: tid=0 ! timer ID

      call get_timer(tid, 'Vecs')
      call start_timer(tid)

      bef=1
      aft=vpotdim

!------------------------------------------------------------------------
! Normal POTFIT
!------------------------------------------------------------------------
      if(.not.lmgpf)then

        relnw(modc)=0.d0
        do m=1,npmode
           aft=aft/psubdim(m)

! Calculate density matrix for mode m.

           if(m.ne.modc) call rmsxcxdd(vpot,vpot,rho,bef,psubdim(m),aft)

! MGPF: if wrallevec calculate density matrix for modc.

           if(lwrallevec.and.m.eq.modc) &
              call rmsxcxdd(vpot,vpot,rho,bef,psubdim(m),aft)

! Diagonalize density matrix for mode m.

           ioformat = 0
           f=potdof(1,m)
           write (iout,'(a,i4,3x,2a/)') &
            '*************      Mode:',m,modelabel(f), &
            '*****************************'

           if(m.eq.modc) then
              if(lctensor) then
                  write(iout,'(a,i7,a,i7,/)') &
                ' Contracted mode. Dimension =', mctens,'  /',potdim(m)
                  write(iout,'(a)') &
                ' NOTE: Computation of C tensor will be performed.' 
                  write(iout,'(a,i7,/)') &
                ' Requested number of SPPs for contracted mode =',mctens
              else
                  write(iout,'(a,i7,a,i7,/)') &
                ' Contracted mode. Dimension =',psubdim(m),'  /',potdim(m)
              end if
           end if

           if(m.ne.modc) call diagrs(eval(zmode(m)),evec(zevec(m)),rho, &
                  psubdim(m),potdim(m),relnw(m),ioformat)

! --------------------------------------------------------------
! OLD
! MGPF: if wrallevec diagonalize density matrix for modc.
!           if(lwrallevec.and.m.eq.modc)
!     +       call diagrs(eval(zmode(m)),evec(zevec(m)),rho,
!     +            psubdim(m),potdim(m),relnw(m),ioformat)
!
!           if(lwrallevec)
!     +  call wrevecall(evec(zevec(m)),m,potdim(m),psubdim(m))
! --------------------------------------------------------------

! --------------------------------------------------------------
! If lwrallevec diagonalize density matrix for modc.
! Same if lctensor, but relevant eigenvectors are mctens
! subroutine diagrs(eval,evec,matrix,dim,neval,srnw,
!     &     ioformat)
!   input: matrix  -- real symmetric matrix
!          dim     -- order of real symm. matrix
!          neval   -- number of "relevant" eigenvectors
!          work    -- work array
! --------------------------------------------------------------

           if(lwrallevec.and.m.eq.modc) then
              if(lctensor) then 
                 call diagrs(eval(zmode(m)),evec(zevec(m)),rho, &
                      psubdim(m),mctens,relnw(m),ioformat)
              else ! just lwrallevec
                 call diagrs(eval(zmode(m)),evec(zevec(m)),rho, &
                      psubdim(m),potdim(m),relnw(m),ioformat)
              end if
           end if ! m.eq.modc

           if(lwrallevec) then
              if(lctensor) then
                 call wrevecall(evec(zevec(m)),m,mctens,psubdim(m))
              else
                 call wrevecall(evec(zevec(m)),m,potdim(m),psubdim(m))
              end if
           end if

           bef=bef*psubdim(m)
        enddo

        call stop_timer(tid)
        return

!------------------------------------------------------------------------
! MGPF calculation
!------------------------------------------------------------------------
      else
!------------------------------------------------------------------------
! MGPF calculation: fullcoarse
!------------------------------------------------------------------------
        if(lfullcoarse)then
          do m=1,npmode

            aft=aft/psubdim(m)

            if(m.eq.modc)then

              write(iout,'(a,i7,a,i7,/)') &
              ' Contracted mode. Dimension =',psubdim(m),'  /',potdim(m)

            else

! Compute density matrix

              call rmsxcxdd(vpot,vpot,rho,bef,psubdim(m),aft)

! Print the corresponding rho for each DOF

               if(lprintrho)then

                  write(filename,'(2a,i2.2)') name(1:laenge),'/rhofull',m

                  open(ioutbin,file=filename,form='unformatted', &
                       status='unknown')

                  write(ioutbin) m, psubdim(m)

                  i = 0
                  do j=1, psubdim(m)
                    write(ioutbin) (rho(i + k), k=1, psubdim(m))
                    i = i + psubdim(m)
                  end do

                  close(ioutbin)

               end if

! Diagonalize density matrix for mode m.

              ioformat=0
              f=potdof(1,m)
              write (iout,'(a,i4,3x,2a/)') &
               '*************      Mode:',m,modelabel(f), &
               '*****************************'

              call diagrs(eval(zmode(m)),evec(zevec(m)),rho, &
                     psubdim(m),potdim(m),relnw(m),ioformat)


              write(filename,'(2a,i2.2)') name(1:laenge),'/cfull',m

              open(ioutbin,file=filename,form='unformatted',status='unknown')

              write (ioutbin) m, psubdim(m)
              write (ioutbin) (eval(zmode(m)-1+i),i=1,psubdim(m))
              call vecout(evec(zevec(m)),onei,psubdim(m),psubdim(m),ioutbin)
              close(ioutbin)

              call writeevec(evec(zevec(m)),m,psubdim(m),psubdim(m))

              write(iout,'(2a,i3,a,/a)') &
              'Eigenvectors (natural potentials)' &
              ,' of mode',m,' on fine grid are written to file ', &
              filename(1:laenge+7)

              write(iout,'(/2a,i3,a,/a)') &
              'Eigenvalues (natural weights)', &
              ' of mode',m,' on fine grid are written to file ', &
              filename(1:laenge+7)

            end if

            bef=bef*psubdim(m)

          end do

!------------------------------------------------------------------------
! MGPF calculation: fdcoeff
!------------------------------------------------------------------------
        else if(lonlydcoeff)then

          do m=1,npmode
            aft=aft/psubdim(m)

            if(m.eq.modc)then

              write(iout,'(a,i7,a,i7,/)') &
              ' Contracted mode. Dimension =',psubdim(m),'  /',potdim(m)

            else

              write(filename,'(2a,i2.2)') name(1:laenge),'/cfull',m

              open(ioutbin,file=filename,form='unformatted', &
                   status='unknown', err=999)

              read (ioutbin) ii, jj
              read (ioutbin) (eval(zmode(m)-1+i),i=1,jj)
              call readvec(evec(zevec(m)),onei,jj,jj,ioutbin)

              write (iout,'(a,i4,1x,a, 2a,i2.2)') &
                    ' Mode=',m,'read from file ', filename

              close(ioutbin)

            end if

            bef=bef*psubdim(m)

          end do

!------------------------------------------------------------------------
! MGPF calculation: finemode
!------------------------------------------------------------------------
        else if(lfinemode)then

          do m=1,npmode
            aft=aft/psubdim(m)

            if(m.eq.modc)then

              write(iout,'(a,i7,a,i7,/)') &
              ' Contracted mode. Dimension =',psubdim(m),'  /',potdim(m)

            else

! NOTE: MGPF needs only the natpots for the mode on the fine grid

              if(m.eq.ifinemode)then

                call rmsxcxdd(vpot,vpot,rho,bef,psubdim(m),aft)

! Print rho for the finemode

                if(lprintrho)then

                   write(filename,'(2a,i2.2)') name(1:laenge),'/rho',m

                   open(ioutbin,file=filename,form='unformatted', &
                       status='unknown')

                   write(ioutbin) m, psubdim(m)

                   i = 0
                   do j=1, psubdim(m)
                     write(ioutbin) (rho(i + k), k=1, psubdim(m))
                     i = i + psubdim(m)
                   end do

                   close(ioutbin)

                end if

                ioformat=0
                f=potdof(1,m)
                write (iout,'(a,i4,3x,2a/)') &
                 '*************      Mode:',m,modelabel(f), &
                 '*****************************'

                call diagrs(eval(zmode(m)),evec(zevec(m)),rho, &
                       psubdim(m),potdim(m),relnw(m),ioformat)

                write(filename,'(2a,i2.2)') name(1:laenge),'/fmod',m

                open(ioutbin,file=filename,form='unformatted', &
                     status='unknown')

                write (ioutbin) m, psubdim(m)
                write (ioutbin) (eval(zmode(m)-1+i),i=1,psubdim(m))
                call vecout(evec(zevec(m)),onei,psubdim(m),psubdim(m),ioutbin)

                close(ioutbin)

                call writeevec(evec(zevec(m)),m,psubdim(m),psubdim(m))

                write(iout,'(2a,i3,a,/a)') &
                'Eigenvectors (natural potentials)' &
                ,' of mode',m,' on fine grid are written to file ', &
                filename(1:laenge+7)

                write(iout,'(/2a,i3,a,/a)') &
                'Eigenvalues (natural weights)', &
                ' of mode',m,' on fine grid are written to file ', &
                filename(1:laenge+7)

              else

                continue

              end if

            end if

            bef=bef*psubdim(m)

          end do

!------------------------------------------------------------------------
! MGPF calculation: ext-natpot
!      read fine grid natpot files (ext-natpot)
!      subroutine readvec (cf. subroutine vecout)
!------------------------------------------------------------------------
        else if(lextpot)then

          write(filename,'(2a)') name(1:laenge),'/ext-natpot'
          open(iextpot,file=filename,form='unformatted', &
               status='unknown', err=999)

          aft = 1
          do m=1, npmode
            aft = aft*psubdim(m)
          end do

          do m=1,npmode

            if(m.eq.modc)then

              write(iout,'(a,i7,a,i7,/)') &
              ' Contracted mode. Dimension =',psubdim(m),'  /',potdim(m)

            else

              call readvec(evec(zevec(m)),onei,psubdim(m),potdim(m),iextpot)

              ioformat=0
              f=potdof(1,m)
              write (iout,'(a,i4,3x,2a/)') &
              '*************      Mode:',m,modelabel(f), &
              '*****************************'

              write (iout,'(a,/)') 'Read from external file'

            end if  ! modc

          end do  ! nmode

          close(iextpot)

        end if ! MGPF: type of calculation

      end if ! lmgpf

      call stop_timer(tid)
      return

999   write(6,'(/2a)') 'Cannot open file: ',filename(1:laenge+10)
      call stopnow
      end subroutine vecs

!***********************************************************************
!   Subroutine readvec  (MGPF)
!   Read vec from ext-natpot file
!***********************************************************************

      subroutine readvec(vec,v,gdim,n,unit)

      implicit none

      integer(long)                               :: i,j,g
      integer(long), intent(in)                   :: v,gdim,n,unit
      real(dop), dimension(v,gdim,n), intent(out) :: vec

       do i=1,n
         do j=1,v
           read (unit) (vec(j,g,i),g=1,gdim)
         enddo
       enddo

      return
      end subroutine readvec

!***********************************************************************
!   Subroutine writeevec  (MGPF)
!   Write evec at every call of fullcoarse or finemode
!***********************************************************************

      subroutine writeevec(evec,m,pdim,gdim)

      implicit none

      integer(long), intent(in) :: m,pdim,gdim
      integer(long)             :: g,n
      real(dop), dimension(gdim,pdim), intent(out) :: evec
      character(len=240)                           :: filename

      write(filename,'(2a,i2.2)') name(1:laenge),'/evec',m
      open(ieigvec,file=filename,form='formatted',status='unknown')

      write(ieigvec,'(a,i3,/a,/3a,/3a,/)') &
       '# potfit eigenvectors (natural potentials) of mode =',m, &
       '# Columns: grid-point, 1st vector, 2nd vector, ...', &
       '# Use "plgen ',filename(laenge+2:laenge+7), &
       ' 1:2 - 1:3 - 1:4" to plot the first three vectors.', &
       '# Inspect this file with "less -S ', &
        filename(laenge+2:laenge+7),'".'

      write(ieigvec,'(a,i6,9999i10)') '#   g',(n,n=1,pdim)
      do g=1,gdim
         write(ieigvec,'(i5,9999f10.6)') g, (evec(g,n),n=1,pdim)
      enddo
      write(ieigvec,'(a,i6,9999i10)') '#   g',(n,n=1,pdim)
      write(6,'(2a,i3,a,/a)') 'Eigenvectors (natural potentials)', &
       ' of mode',m,' are written to file ',filename(1:laenge+7)

      close(ieigvec)

      write(filename,'(2a,i2.2)') name(1:laenge),'/evecb',m

      open(ioutbin,file=filename,form='unformatted',status='unknown')

      do g=1,gdim
         write(ioutbin) g, (evec(g,n),n=1,pdim)
      enddo

      close(ioutbin)

      return

      end subroutine writeevec

!***********************************************************************
!    Subroutine wrevecall
!    This routine writes all eigenvectors (natural potentials) to
!    an ASCII file and a binary one
!***********************************************************************

      subroutine wrevecall(evec,m,pdim,gdim)

      implicit none

      integer(long), intent(in) :: m,pdim,gdim
      integer(long)             :: g,n
      real(dop), dimension(gdim,pdim), intent(out) :: evec
      character(len=240)                           :: filename, filenameb

      write(filename,'(2a,i2.2)') name(1:laenge),'/evec',m
      open(ieigvec,file=filename,form='formatted',status='unknown',err=999)

      write(filenameb,'(2a,i2.2)') name(1:laenge),'/evecb',m
      open(ioutbin,file=filenameb,form='unformatted',status='unknown',err=998)

      write(ieigvec,'(a,i3,/a,/3a,/3a,/)') &
       '# potfit eigenvectors (natural potentials) of mode =',m, &
       '# Columns: grid-point, 1st vector, 2nd vector, ...', &
       '# Use "plgen ',filename(laenge+2:laenge+7), &
       ' 1:2 - 1:3 - 1:4" to plot the first three vectors.', &
       '# Inspect this file with "less -S ', &
        filename(laenge+2:laenge+7),'".'
      write(ieigvec,'(a,i6,99i10)') '#   g',(n,n=1,pdim)
      do g=1,gdim
         write(ieigvec,'(i5,99f10.6)') g, (evec(g,n),n=1,pdim)
         write(ioutbin) (evec(g,n),n=1,pdim)
      enddo
      write(ieigvec,'(a,i6,99i10)') '#   g',(n,n=1,pdim)
      write(6,'(2a,i3,a,/a)') 'Eigenvectors (natural potentials)', &
       ' of mode',m,' are written to file ',filename(1:laenge+7)

      close(ieigvec)
      close(ioutbin)

      return
 998  write(6,'(/2a)') 'Cannot open evecb-file: ',filenameb(1:laenge+7)
 999  write(6,'(/2a)') 'Cannot open evec-file: ',filename(1:laenge+7)
      call stopnow
      end subroutine wrevecall


      end module vecsmod
