!***********************************************************************
      module wrfitmod

use decimal, only:long,dop
use global
use pglobal
use pdefaultmod, only: init_cmode
use maxv, only:maxdim
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use showpotmod
use combined
use channels

use iodvrdef, only:wrdvrdef
use iogrddef, only:wrgrddef
use vpotpars, only:writevpotpars,cppinput

use logdat
use mtlib,   only: dtxxdd
use versions, only: progver

      implicit none
      private
      public :: wrfit,vecsw,unweight_contr,vecout
      contains

!***********************************************************************
!
!         POTFIT  subroutines: WRFIT, CPPINPUT, VECSW
!                      FITOUT, unweight_contr,VECOUT
!
!    11/96 AJ
!
!***********************************************************************

      subroutine wrfit(coeff,evec,oned,weight,weighti)

      implicit none
      
      integer(long)                                :: n,m,f,g,zeig,sl,ierr
      real(dop), dimension(ctdim), intent(inout)   :: coeff
      real(dop), dimension(evecdim), intent(inout) :: evec
      real(dop), dimension(weightdim), intent(in)  :: weight,weighti
      real(dop), dimension(oneddim), intent(in)    :: oned
      character(len=80)                            :: buffer
      character(len=c5)                            :: outname
      integer(long), external                      :: slen
      logical(kind=4)                              :: lexist
      character(len=c2)                            :: wrtag

      if (lcpd) then  ! cpd
         outname=name(1:laenge)//'/cpd'
      elseif (lwef) then
         write(buffer,'(i3.3)') fitcount
         fitcount=fitcount+1
         outname=name(1:laenge)//'/natpot.'//buffer(1:3)
      else
         outname=name(1:laenge)//'/natpot'
      endif
      sl=slen(outname)
      
      if (.not.lovwr) then
         inquire(file=outname(1:sl),exist=lexist)
         if (lexist) then
            routine='wrfit'
            message='File already exists: '//outname(1:sl)
            call errormsg
         endif
      endif

      open(inpot,form='unformatted',status='unknown',err=999,&
              file=outname(1:sl))
      
      write(inpot,iostat=ierr) progver
      if (ierr.ne.0) then
         routine='wrfit'
         message='cannot write program version to '//outname(1:sl)
         call errormsg
      endif

! --- write parameters for FT-natpots (obsolete, for consistency only).
      if(.not.lcpd) write(inpot) 0,0,ipbaspar(1,1)  !! .not.lcpd

!-----------------------------------------------------------------------
! array sizes
!-----------------------------------------------------------------------
      call wrmemdim(inpot)

!-----------------------------------------------------------------------
! write date and time of creation
!-----------------------------------------------------------------------
      call fpstartup(buffer)
      wrtag= '<creation date>'
      write(inpot) wrtag
      write(inpot) buffer
      
!-----------------------------------------------------------------------
! write some input information
!-----------------------------------------------------------------------
      wrtag= '<natpot title>'
      write(inpot) wrtag
      nmode=npmode
      call cppinput(inpot,outname)
      wrtag= '<end natpot title>'
      write(inpot) wrtag

!-----------------------------------------------------------------------
! write system definition
!-----------------------------------------------------------------------
      call wrdvrdef(inpot)

      do m=1,nmode 
        nspfdof(m)=npotdof(m)
      enddo
      do m=1,nmode
        do n=1,nspfdof(m)
          spfdof(n,m)=potdof(n,m)
        enddo
      enddo
      do f=1,ndof
        dofspf(f)=dofpot(f)
      enddo

      call wrgrddef(inpot)

!-----------------------------------------------------------------------
! write fit parameters
!-----------------------------------------------------------------------
      if (lcpd) then
         wrtag= '<cpdrank>'
         write(inpot) wrtag
         write(inpot) cpdrank
      else
         wrtag= '<modc>'
         write(inpot) wrtag
         write(inpot) modc
         wrtag= '<lpconm>'
         write(inpot) wrtag
         write(inpot) lpconm
      endif
      wrtag= '<onedipot>'
      write(inpot) wrtag
      write(inpot) (onedipot(f),f=1,ndof)
      wrtag= '<lpntdvr>'
      write(inpot) wrtag
      write(inpot) lpntdvr
      if (lpntdvr) then
         wrtag= '<pntbas>'
         write(inpot) wrtag
         write(inpot) (basis(f),f=1,ndof)
      endif

!-----------------------------------------------------------------------
! MGPF cutnpot: NOTE pdim and potdim carry different values
! NOTE: pdim (as read from natpot) potdim (as requested)
! RECALL: modcdim = Prod_(m.ne.modc)[potdim(m)]
!-----------------------------------------------------------------------

      if(lcutnpot) then
         potdim(modc) = 1
         do m=1, nmode
            if(m.ne.modc) potdim(modc)=potdim(modc)*potdim(m)
         end do
         wrtag= '<pdim>'
         write(inpot) wrtag
         write(inpot) (potdim(m),m=1,npmode)
      elseif(.not.lcpd) then
         do m=1,nmode
            if (m .eq. modc) then
               pdim(m)=modcdim
            else
               pdim(m)=potdim(m)
            endif
         enddo
         wrtag= '<pdim>'
         write(inpot) wrtag
         write(inpot) (pdim(m),m=1,npmode)
      endif

      wrtag= '<end natpot info>'
      write(inpot) wrtag

      write(wrtag,'(a)') '<natpots>'
      write(inpot) wrtag
!-----------------------------------------------------------------------
! write out 1D pots
!-----------------------------------------------------------------------
      do f=1,ndof
         if (onedipot(f) .gt. 0) then
            write(wrtag,'(a5,i10,a1)') '<oned',f,'>'
            write(inpot) wrtag
            zeig=zort(f)-1
            write(inpot) (oned(zeig+g),g=1,gdim(f))
         endif
      enddo

!-----------------------------------------------------------------------
! Get unweighted eigenfunctions
! Calculate C coeffs which are contracted over mode 'modc'
! and write fitdata to output channel 'inpot'.
!-----------------------------------------------------------------------
      if (lsw) then
         call vecsw(evec,weighti)
         call unweight_contr(coeff,weighti,dimbef,dimmodc,dimaft,&
              npotdof(modc),potdof(1,modc),gdim,maxdim,zweight,&
              weightdim,ctdim,iwei)
      endif

!-----------------------------------------------------------------------
! write coefficients and eigenvectors
!-----------------------------------------------------------------------
      write(wrtag,'(a)') '<vectors>'
      write(inpot) wrtag
      if(lcpd) then  !! cpd
         call cpdout(evec)
      else
         call fitout(coeff,evec)
      endif

!-----------------------------------------------------------------------
! write vpot parameters (for showpot), and close file.
!-----------------------------------------------------------------------
      write(wrtag,'(a)') '<vpotpars>'
      write(inpot) wrtag
      call writevpotpars(inpot,outname,0)

      write(wrtag,'(a)') '<end natpots>'
      write(inpot) wrtag
      close(inpot)

!-----------------------------------------------------------------------
! Restore weighted eigenfunctions
!-----------------------------------------------------------------------
      if (lsw .and. lwef) then
         call vecsw(evec,weight)
!         call unweight_contr(coeff,weight,dimbef,dimmodc,dimaft,
!     +        npotdof(modc),potdof(1,modc),gdim,maxdim,zweight,
!     +        weightdim,ctdim,iwei)
      endif

      return
  999 continue
      message = 'Cannot open potw file'
      call errormsg
      end subroutine wrfit

!***********************************************************************
!
!          POTFIT  subroutine VECSW
!
!  + multiplies eigenvectors with weights
!    or the inverse of the weights depending on input
!
!    input:  evec -- array with eigenvectors
!            weight -- array with weights or their inverse
!
!    output:  evec -- (un)weighted eigenvectors
!
!   11/96 AJ
!
!***********************************************************************

      subroutine vecsw(evec,weight)

      implicit none

      integer(long)                                :: m,n,f,bef,aft
      real(dop), dimension(evecdim), intent(inout) :: evec
      real(dop), dimension(weightdim), intent(in)  :: weight
      
      do m=1,npmode
         if(m.ne.modc) then
            bef=1
            aft=psubdim(m)*potdim(m) 
            do n=1,npotdof(m)
               f=potdof(n,m)
               aft=aft/gdim(f)
               if(iwei(f).gt.0) call dtxxdd(weight(zweight(f)),&
                  evec(zevec(m)),evec(zevec(m)),bef,gdim(f),aft)
               bef=bef*gdim(f)
            enddo
         endif
      enddo

      return
      end subroutine vecsw

!#######################################################################
!
!              subroutine FITOUT
!
!  + writes natural potentials to output channel "inpot"
!  + format of data is as following:
!
!           -- potdim(1) natural potentials
!               ......
!           -- coeffdim contracted natural potential of mode modc
!               ....
!           -- potdim(ndof) natural potentials
!
!  Note: evec contains already the unweighted natural potentials.
!
!   11/96 AJ
!
!
!#######################################################################

      subroutine fitout(coeff,evec)

      implicit none

      integer(long)                               :: m
      real(dop), dimension(coeffdim), intent(out) :: coeff
      real(dop), dimension(evecdim), intent(in)   :: evec

!-----------------------------------------------------------------------
! Write relevant natural potentials (contracted and 
! uncontracted) to channel "inpot"
!   NOTE: orig vecout(vec,v,gdim,n,psb,ptd,unit)
!   CUTNPOT: pdim (as read from natpot) potdim (as requested)
!-----------------------------------------------------------------------
      do m=1,npmode
         if (m.eq.modc) then
! -------------- MGPF cutnpot: if lcutnpot use subroutine cutnpotcoeffout
            if(lcutnpot)then
              call cutnpotcoeffout(coeff,dimbef,dimmodc,dimaft,inpot)
            else
              call vecout(coeff,dimbef,dimmodc,dimaft,inpot)
            endif
         else ! any mode except modc
            call vecout(evec(zevec(m)),onei,psubdim(m),potdim(m),inpot)
         endif
      enddo

      return
      end subroutine fitout

!-----------------------------------------------------------------------

      subroutine cpdout(evec)

      implicit none

      integer(long) :: m,g,r,p
      real(dop)     ::  evec(evecdim)
      real(dop)     ::  coeff

      coeff=1.d0
      p=0
      do r=1,cpdrank
         write(inpot) coeff
         do m=1, nmode
            write(inpot) (evec(p+g),g=1,psubdim2(m))
            p=p+psubdim2(m)
         end do
      end do

      return
      end subroutine cpdout

!***********************************************************************
!
!          Subroutine  unweight_contr
!
!  un-weights the contracted mode.
!
!  11.01.01
!
!***********************************************************************

      subroutine unweight_contr(coeff,weighti,v,gdim1,n,npotdof1,&
          potdof1,gdim,maxdim,zweight,weightdim,ctdim,iwei)

      implicit none

      integer(long)                                  :: n1,f,aft,bef
      integer(long), intent(in)                      :: v,n,gdim1,npotdof1, &
                                                        maxdim,weightdim,ctdim
      integer(long), dimension(npotdof1), intent(in) :: potdof1
      integer(long), dimension(maxdim), intent(in)   :: gdim,zweight,iwei
      real(dop), dimension(ctdim), intent(inout)     :: coeff
      real(dop), dimension(weightdim), intent(in)    :: weighti

      bef=1*v
      aft=gdim1*n
      do n1=1,npotdof1
         f=potdof1(n1)
         aft=aft/gdim(f)
         if(iwei(f).gt.0)&
           call dtxxdd(weighti(zweight(f)),coeff,coeff,bef,gdim(f),aft)
         bef=bef*gdim(f)
      enddo

      return
      end subroutine unweight_contr


!***********************************************************************
!
!          Subroutine VECOUT
!
!  !!!!! vec(b,c,a)
!
!  13.02.95
!
!***********************************************************************

      subroutine vecout(vec,v,gdim,n,unit)

      implicit none

      integer(long)                              :: i,j,g
      integer(long), intent(in)                  :: v,gdim,n,unit
      real(dop), dimension(v,gdim,n), intent(in) :: vec

      do i=1,n
         do j=1,v
            write (unit) (vec(j,g,i),g=1,gdim)
        enddo
      enddo

      return
      end subroutine vecout

!***********************************************************************
!          Subroutine CUTNPOTCOEFFOUT
!
!  !!!!! vec(b,c,a)
!
!  Given a reduction in the number of natural weights to use (cutnpot),
!  this subroutine eliminates the coefficientes corresponding to the
!  removed SPPs.
!
!  DPR 2011
!
!***********************************************************************

      subroutine cutnpotcoeffout(vec,v,gdim,n,unit)

      implicit none

      integer(long), intent(in)                   :: v,gdim,n,unit
      integer(long)                               :: i,j,k,m,g 
      integer(long)                               :: contbef, contaft
      integer(long)                               :: incrbef, incraft, &
                                                     totcont, nbef, naft
      integer(long)                               :: innerindx, outerindx,tt,pp
      integer(long)                               :: ttbef, ttaft, ppbef, ppaft
      integer(long), dimension(npmode)            :: icounter,diffdim,olddim
      real(dop), dimension(v,gdim,n), intent(out) :: vec

!-------------------------------------------------------------
!  PREVIOUS
!-------------------------------------------------------------

! MGPF cutnpot: generate new values dimbef, dimaft
! NOTE: pdim (as read from natpot) potdim (as requested)

         do i=1,npmode
           olddim(i) = potdim(i)
           potdim(i) = pdim(i)
         enddo

         call init_cmode

         do i=1,npmode
           potdim(i) = olddim(i)
         enddo

         if(potdim(modc).le.0)potdim(modc) = psubdim(modc)

!-------------------------------------------------------------
!  COUNTERS
!-------------------------------------------------------------

! set up which mode is which (innermost runs the fastest)

      if(modc.eq.1)then
         innerindx = 2
         outerindx = npmode
      elseif(modc.eq.npmode)then
         innerindx = 1
         outerindx = npmode-1
      else
         innerindx = 1
         outerindx = npmode
      end if

      nbef = 0
      naft = 0
      do m=1, npmode
        if(m.lt.modc)then
          nbef = nbef + 1
        elseif(m.gt.modc)then
          naft = naft + 1
        endif
      enddo

! create counters for each m if counter does not exceed potdim(m) write

      do m=1, npmode
        icounter(m) = 1
        if(m.ne.modc)then
          diffdim(m) = pdim(m)-potdim(m)
        endif
      enddo

      icounter(modc) = 0
      diffdim(modc)  = 0

      contbef  = 1
      contaft  = 1
      totcont  = 0

      do k=1, dimaft*dimbef

          write (unit) (vec(contbef,g,contaft),g=1,gdim)

          totcont  = totcont + 1
          icounter(innerindx) = icounter(innerindx) + 1

! NOTE: incr means that a mode (bef or aft modc) has exceeded the 
! requested value of potdim(i) the indices have to be advanced until pdim(i)

          incrbef = 0
          incraft = 0

          do i = 1, npmode
            if(i.eq.modc)then
              continue
            elseif(i.eq.outerindx)then
              continue
            else
              if(icounter(i).gt.potdim(i))then
                 icounter(i) = 1
                 if(i.eq.modc-1)then
                   icounter(i+2) = icounter(i+2) + 1  ! advance icounter next index
                 else
                   icounter(i+1) = icounter(i+1) + 1  ! advance icounter next index
                 endif
                 if(i.le.modc-1)then
                   incrbef = incrbef + 1
                 elseif(i.ge.modc+1)then
                   incraft = incraft + 1
                 endif
              endif
            endif
          enddo

!---------------------------
! if modc=1 or modc=npmode (OK)
!---------------------------

          if(modc.eq.1.or.modc.eq.npmode)then
            tt = 0
            if(incrbef.gt.0)then
              do i=innerindx, nbef
                if(i.gt.innerindx+incrbef-1)then
                  continue
                else
                  pp = 1
                  do j=innerindx,i-1
                    pp = pp * pdim(j)
                  enddo
                  tt = tt + pp * diffdim(i)
                endif
              enddo
              contbef = contbef + tt + 1
            elseif(incraft.gt.0)then
              do i=innerindx, naft
                if(i.gt.innerindx+incraft-1)then
                  continue
                else
                  pp = 1
                  do j=innerindx,i-1
                    pp = pp * pdim(j)
                  enddo
                  tt = tt + pp * diffdim(i)
                endif
              enddo
              contaft = contaft + tt + 1
            elseif(incrbef.eq.0.and.modc.eq.npmode)then
              contbef = contbef + 1
            elseif(incraft.eq.0.and.modc.eq.1)then
              contaft = contaft + 1
            endif
          endif

!---------------------------
! if modc.gt.1 and modc.lt.npmode
!---------------------------

          if(modc.gt.1.and.modc.lt.npmode)then
            if(incrbef.gt.0.and.incraft.eq.0)then
              ttbef = 0
              do i=innerindx, nbef
                if(i.gt.innerindx+incrbef-1)then
                  continue
                else
                  ppbef = 1
                  do j=innerindx,i-1
                    ppbef = ppbef * pdim(j)
                  enddo
                  ttbef = ttbef + ppbef * diffdim(i)
                endif
              enddo
              contbef = contbef + ttbef + 1

              if(contbef.gt.dimbef)then
                contbef = abs(dimbef-contbef)
                contaft = contaft+1
              endif
            elseif(incraft.gt.0)then
              ttaft = 0
              do i=innerindx, npmode
                if(i.eq.modc)then
                  continue
                elseif(i.gt.modc+1+incraft-1)then
                  continue
                else
                  ppaft = 1
                  do j=innerindx,i-1
                    if(j.ne.modc) ppaft = ppaft * pdim(j)
                  enddo
                  ttaft = ttaft + ppaft * diffdim(i)
                endif
              enddo

! NOTE: for incraft, do-loop over ALL modes ttaft MUST be distributed 
! among contbef and contaft
! NOTE: ttaft sets of indices to advance, this include both contbef and contaft

               do i=1, ttaft+1
                 contbef = contbef + 1
                 if(contbef.gt.dimbef)then
                   contbef = 1
                   contaft = contaft + 1
                 endif
               enddo

            elseif(incrbef.eq.0.and.incraft.eq.0)then
              contbef = contbef + 1

              if(contbef.gt.dimbef)then
                contbef = abs(dimbef-contbef)
                contaft = contaft+1
              endif
            endif
          endif

          if(icounter(outerindx).gt.potdim(outerindx))goto 114

      enddo       ! dimaft

114   continue

! --------------------------------------------------------------------------------
!  END COUNTERS
! --------------------------------------------------------------------------------

      return
      end subroutine cutnpotcoeffout


      end module wrfitmod

