!#######################################################################
!
!          POTFIT module WRPGITTER
!
!  12/97 AJ
!
!#######################################################################
      module wrpgitter

use decimal, only:long,dop
use logdat

      implicit none
      private
      public :: writer,wrgrid,wroned,wrprodwei
      contains

!***********************************************************************
!         POTFIT  subroutine WRITER
! 
!    11/97 AJ
!***********************************************************************

      subroutine writer(nit,iterr2)

use global
use pglobal
use dvrdatmod
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use channels

      implicit none

      integer(long)             :: f
      integer(long), intent(in) :: nit
      real(dop)                 :: www
      real(dop), intent(out)    :: iterr2

!-----------------------------------------------------------------------
! Write error data to file 'iteration' 
!-----------------------------------------------------------------------
      if (nit.eq.0) then
         write (iit,'(a,i4)') '# ndof =',ndof
         write (iit,'(3a)') '# it      Wei-rel       Unw-rel       ',&
                 'Unw-all       Max-rel       Max-all           ',&
                 'sqrt((tr-sum)/sum-weights) 1...f'
         iterr2=rmserr
      endif
      if (lsw) then
         www = wsum2
      else
         www = niw
      endif

!-----------------------------------------------------------------------
! Write error data to file 'iteration'
!-----------------------------------------------------------------------
      write (iit,'(i4,5f14.9,2x,19f15.10)',err=10) &
         nit,iterr2*eV,urms*eV,allrms*eV,maxerr*eV,maxall*eV,&
         (sqrt(relnw(f)/www)*eV,f=1,ndof)
      call myflush(iit)
      RETURN

 10   routine = 'WRITER'
      message = 'Error in writing to file "iteration"'
      call errormsg

      return
      end subroutine writer


!***********************************************************************
!            POTFIT   subroutine WRGRID
!
! Write grid points to channel 'iout' [au] or [rad]
!
!  sw
!
!***********************************************************************

      subroutine wrgrid(ort)

use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use channels

      implicit none

      integer(long)                            :: g,f
      real(dop), dimension(ortdim), intent(in) :: ort

      do f=1,ndof
            write (iout,*)
            write (iout,*) &
                 'grid points of mode: '//&
                 modelabel(f)
            write (iout,'(8(f8.4,1x))') &
                 (ort(zort(f)+g),g=0,gdim(f)-1)
      enddo
       
      return
      end subroutine wrgrid


!***********************************************************************
!            POTFIT   subroutine WRONED
!
! Write one-dimensional potential curves to channel 'iout' in [eV]
!
!   12/97 AJ
!
!***********************************************************************

      subroutine wroned(oned)

use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use channels

      implicit none

      integer(long)                            :: g,f
      real(dop), dimension(ortdim), intent(in) :: oned

      do f=1,ndof
         if (onedipot(f) .gt. 0) then
            write (iout,*)
            write (iout,*) &
                 'One-dimensional potential curve in [eV] of mode: '//&
                 modelabel(f)
            write (iout,'(8(f8.4,1x))') &
                 (oned(zort(f)+g)*ev,g=0,gdim(f)-1)
         endif
      enddo
       
      return
      end subroutine wroned


!***********************************************************************
!         POTFIT  subroutine WRPRODWEI
!
! Write weights to channel 'iout' and to file 'prodwei'.
!
!   12/97 AJ
!***********************************************************************

      subroutine wrprodwei(ort,weight)

use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use channels

      implicit none

      integer(long)                               :: g,f
      real(dop), dimension(ortdim), intent(in)    :: ort
      real(dop), dimension(weightdim), intent(in) :: weight

      do f=1,ndof
         if (iwei(f).ne.0 .and. loutl) then
            write (iout,*)
            write (iout,*) 'Weight function for mode: '//modelabel(f)
            write (iout,'(10(f6.4,1x))') &
                 (weight(zweight(f)+g),g=0,gdim(f)-1)
         endif
      enddo
      if (lprodw) then
         write (ipw,'(a,i4)',err=10) "#  number of dof's =",ndof
         do f=1,ndof
            write (ipw,'(/a,i3,2x,a12,a,i4)',err=10)& 
                 '# dof =', f, modelabel(f),'  gdim =',  gdim(f)
            do g=0,gdim(f)-1
               write(ipw,'(f11.5,1pe16.6)',err=10) &
                    ort(zort(f)+g),weight(zweight(f)+g)
            enddo
            write (ipw,*)
         enddo
         call myflush(ipw)
      endif

      RETURN

 10   routine = 'WRPRODWEI'
      message = 'Error in writing to prodwei file'
      call errormsg

      return
      end subroutine wrprodwei


      end module wrpgitter

