!***********************************************************************
!              POTFIT  subroutine  zeigpfit
!
! Calculates pointer structure and allocates the memory arrays to the
! standard variable arrays for the original potfit version.
!
!
!***********************************************************************
      module zeigpfitmod

use decimal, only:long,dop
use globmemmod
use logdat

      implicit none
      private
      public :: zeigpfit,mempfit,setv2c,memhops_pfit
      contains

!***********************************************************************
!
! Calculates pointer structure and allocates the memory arrays to the
! standard variable arrays for potfit.
!***********************************************************************

      subroutine zeigpfit

use global
use pglobal
use maxv, only:maxdim
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use channels
use combined

use compdat, only:getcsys
use vpotpars, only:skipvpotpars

use mpidata
#ifdef MPI
   use mpi
#endif

      implicit none

      integer(long) :: n,m,f,mm,ff,zeig,work2a,work2b,ierr,bfl,nitems,offs
      integer(long) :: ilbl,jlbl
      integer(long), dimension(maxdim) :: reorder
      real(dop) :: avpotdim
      character(len=c2)                          :: rdtag
      
!-----------------------------------------------------------------------
! calculate psubdim(m) for each mode. Set potdim(m).
!-----------------------------------------------------------------------
      do m=1,npmode
         zeig=1
         do n=1,npotdof(m)
            f=potdof(n,m)
            zeig=zeig*gdim(f)
         enddo
         psubdim(m)=zeig
      enddo
      potdim(modc)=psubdim(modc)
      
      do m = 1, npmode
         if(potdim(m) .le. 0 ) then
            potdim(m) = psubdim(m)
         else if(potdim(m) .gt. psubdim(m) ) then
           write(6,'(a,i3,a,2i5)') ' m =',m, &
                   ', potdim/psubdim:',potdim(m),psubdim(m)
           routine= 'zeigpfit'
           message = ' potdim .gt. psubdim '
            call errormsg
         end if
      enddo

!-----------------------------------------------------------------------
! Calculate cvecdim, dimension of the (uncontracted) C tensor
!-----------------------------------------------------------------------
      cvecdim = 1
      if (lctensor) then
        if(mctens.gt.psubdim(modc)) then
           message = ' mctens greater than primitive basis for contr '
           call errormsg
        end if
        do m=1,npmode
          if(m.eq.modc) then
            cvecdim = cvecdim * mctens
          else
            cvecdim = cvecdim * potdim(m)
          end if
        enddo
      end if

!-----------------------------------------------------------------------
! set arrays vgmdim and vgpdim, which contains the 'vor'-grid-dimensions
! in mode order and in dof order.
!-----------------------------------------------------------------------
      ff=0
! --- ff=mode order, f=dof order.
      do m=1,npmode
         do n=1,npotdof(m)
            f=potdof(n,m)
            ff=ff+1
            reorder(ff)=f
         enddo
      enddo

! --- set vgmdim, `vor'-grid-dimension in mode order.
      vgmdim(reorder(1))=1
      do ff=1,ndof-1
         vgmdim(reorder(ff+1))=vgmdim(reorder(ff))*gdim(reorder(ff))
      enddo

! --- set vgpdim, `vor'-grid-dimension in dof order.
      vgpdim(1)=1
      do f=1,ndof-1
         vgpdim(f+1)=vgpdim(f)*gdim(f)
      enddo

!----------------------------------------------------------------------
! Pointers to and dimensions of arrays
!----------------------------------------------------------------------
      avpotdim = 1.0
      vpotdim  = 1
      maxgdim  = 0
      evecdim  = 0
      ctdim     = 1
      zeig     = 1

      do f=1,ndof
         zort(f)  = zeig
         if(basis(f) .eq. 12 .or. basis(f) .eq. 20 .or. &
            basis(f) .eq. 21) then
            zeig  = zeig+ipbaspar(3,f)
         else
            zeig  = zeig+gdim(f)
         end if
         maxgdim  = max(maxgdim,gdim(f)) 
         if(basis(f).eq.12 .or. basis(f).eq.20 .or. basis(f) .eq. 21) &
             maxgdim=max(maxgdim,ipbaspar(3,f))
      enddo

      ortdim    = zeig-1
      maxgdim2  = maxgdim**2
 
      zeig     = 1
      maxpsub  = 0

      do m=1,npmode
         zmode(m) = zeig
         zeig     = zeig+psubdim(m) 
         maxpsub  = max(maxpsub,psubdim(m))
         ctdim     = ctdim*potdim(m)
         vpotdim  = vpotdim*psubdim(m)
         avpotdim  = avpotdim*psubdim(m)
         zevec(m) = evecdim+1
         if(m.ne.modc) evecdim  = evecdim+psubdim(m)*potdim(m)
         if(m.eq.modc.and.lwrallevec) then
            if(lctensor) then
              evecdim  = evecdim+psubdim(modc)*mctens
            else ! lwrallevec only
              evecdim  = evecdim+psubdim(modc)*psubdim(modc)
            end if
         end if
      enddo

      if(lextpot.and.avpotdim.gt.2.14d+9) vpotdim  = 1
      if(lspenergy.or.lcutalong) vpotdim  = 1

      evaldim   = zeig-1
      oneddim   = evaldim 
      maxpsub2  = 1 ! maxpsub2 = max of psubdim**2 without contr. mode
      do m=1,npmode
         if (m.ne.modc) then
            maxpsub2  = max(psubdim(m)**2,maxpsub2)
         end if
      end do

      if (lwrallevec) maxpsub2  = max(psubdim(modc)**2,maxpsub2)

!-----------------------------------------------------------------
! MGPF mode lextpot (read both ext-natpot and fdcoeff)
!-----------------------------------------------------------------
      if(mpirank.gt.0) goto 999
      if(lextpot)then

         if(avpotdim .gt. 2.14d+9) then
            write(6,'(/a,/a,1pe10.3,a,/2a,/a)') &
            '------------------------------------------------------------', &
              'Number of grid-points, vpotdim =',avpotdim, &
              ', is too large (>2.14*10^9).', 'Integer overflow. ', &
              'Better stay below 10^9 grid-points. STOP.', &
            '############################################################'
            write(ilog,'(/a,1pe10.3,a,/2a)') &
              'Number of grid-points, vpotdim =',avpotdim, &
              ', is too large (>2.14*10^9).', 'Integer overflow. ', &
              'Better stay below 10^9 grid-points. STOP.'
            stop

         else
            write(6,'(/a,/a,/a,1pe10.3,/a,/a)') &
         '------------------------------------------------------------', &
         ' ATTENTION: ext-natpot mode.', &
         ' The number of grid-points is vpotdim =', avpotdim, &
         ' External files ext-natpot and fdcoeff will be read.', &
         '------------------------------------------------------------'
            write(ilog,'(/a,/a,/a,1pe10.3,/a,/a)') &
         '------------------------------------------------------------', &
         ' ATTENTION: ext-natpot mode.', &
         ' The number of grid-points is vpotdim =', avpotdim, &
         ' External files ext-natpot and fdcoeff will be read.', &
         '------------------------------------------------------------'
         endif

      elseif(lcutnpot)then
         if(avpotdim .gt. 2.14d+9) then
            write(6,'(/a,/a,1pe10.3,a,/a,/a)') &
         '------------------------------------------------------------', &
              'Number of grid-points, vpotdim =',avpotdim, &
              ', is too large (>2.14*10^9).', &
              'Mode cutnpot.', &
         '------------------------------------------------------------'

            write(ilog,'(/a,/a,1pe10.3,a,/a,/a)') &
         '------------------------------------------------------------', &
              'Number of grid-points, vpotdim =',avpotdim, &
              ', is too large (>2.14*10^9).', &
              'Mode cutnpot.', &
         '------------------------------------------------------------'
         end if

      elseif(lcutalong)then
            write(6,'(/a,/a,1pe10.3,/a,/a)') &
         '------------------------------------------------------------', &
         ' Number of grid-points, vpotdim =',avpotdim, &
         ' not taken into account in cutalong calculation ', &
         '------------------------------------------------------------'

            write(ilog,'(/a,/a,1pe10.3,/a,/a)') &
         '------------------------------------------------------------', &
         ' Number of grid-points, vpotdim =',avpotdim, &
         ' not taken into account in cutalong calculation ', &
         '------------------------------------------------------------'

      elseif(lspenergy)then
            write(6,'(/a,/a,1pe10.3,/a,/a)') &
         '------------------------------------------------------------', &
         ' Number of grid-points, vpotdim =',avpotdim, &
         ' not taken into account in single point energy calculation ', &
         '------------------------------------------------------------'

            write(ilog,'(/a,/a,1pe10.3,/a,/a)') &
         '------------------------------------------------------------', &
         ' Number of grid-points, vpotdim =',avpotdim, &
         ' not taken into account in single point energy calculation ', &
         '------------------------------------------------------------'

      else if (.not.(lwo.and.vpotfmt.eq.2)) then

         if(avpotdim .gt. 2.14d+9) then
            write(6,'(/a,/a,1pe10.3,a,/2a,/a)') &
         '------------------------------------------------------------', &
              'Number of grid-points, vpotdim =',avpotdim, &
              ', is too large (>2.14*10^9).', 'Integer overflow. ', &
              'Better stay below 10^9 grid-points. STOP.', &
          '############################################################'
            write(ilog,'(/a,1pe10.3,a,/2a)') &
              'Number of grid-points, vpotdim =',avpotdim, &
              ', is too large (>2.14*10^9).', 'Integer overflow. ', &
              'Better stay below 10^9 grid-points. STOP.'
            call stopnow

         elseif(avpotdim .gt. 1.d+9) then
            write(6,'(/a,/a,1pe10.3,a,/a,/a)') &
         '------------------------------------------------------------', &
             'Number of grid-points, vpotdim =', avpotdim, &
             ', is very large (>10^9).', &
             'WARNING, you may run into problems.', &
         '------------------------------------------------------------'
            write(ilog,'(/a,/a,1pe10.3,a,/a,/a)') &
         '------------------------------------------------------------', &
             'Number of grid-points, vpotdim =', avpotdim, &
             ', is very large (>10^9).', &
             'WARNING, you may run into problems.', &
         '------------------------------------------------------------'
          endif
      endif

!-----------------------------------------------------------------------
! Size of buffer
! really want to be able to read filever and then access niobuf directly
!-----------------------------------------------------------------------
      if(lreadvpot) then
         open(ivpot,file=vname(1:vlength),status='old',&
              form='unformatted',err=912)
         call skipvpotpars(ivpot)
         if (filever(ivpot) .lt. 11.000_dop) then
            read(ivpot,iostat=ierr) nitems,niobuf
            if (ierr.ne.0) then
               routine='zeigpfit'
               message='cannot read nitems and niobuf from '
               call errormsg
            endif
         else
            rewind(ivpot)
            do
  10           continue
               read(ivpot,err=10,end=100) rdtag
               if (rdtag(1:1) .ne. '<') cycle

               ilbl=2
               jlbl=index(rdtag,'>')-1
               if (rdtag(ilbl:jlbl) .eq. 'vpot pes') exit
            enddo
  100       continue
            read(ivpot,iostat=ierr) nitems,niobuf

         endif
         close(ivpot)
      else
         bfl=bufferlength*1024*1024/8
         if(vpotdim .le. bfl) then
            niobuf=vpotdim
         else
            niobuf=bfl
            if(vpotdim .le. 2*bfl) niobuf=bfl/2
            do f=1,ndof
               if(vgpdim(f).le.niobuf) ff=f
            enddo
            mm=niobuf/vgpdim(ff)
            niobuf=mm*vgpdim(ff)
         endif
      endif
 999  if (lwo.and.lmpi) then
         if (vpotfmt.eq.2) then
!           buffer will be dynamically allocated
            niobuf = 1
         else
            niobuf = bufferlength*1024/8*1024
         endif
      endif

!-----------------------------------------------------------------------
! Additional pointers needed for weights
!-----------------------------------------------------------------------
      weightdim = ortdim 
      do f=1,ndof
         zweight(f) = zort(f)
         wdim(f)    = gdim(f)
      enddo
!
! set up pointers for operator ort array
! and operator arrays
!
      if (lsubtract .and. lensubtr .gt. 0) then
         call getcsys(2)
         do f=1,ndof
            gdim1(f)=gdim(f)
         enddo

         zeig=1
         do f=1,ndof
            zort1(f)  = zeig
            if(basis(f) .eq. 12 .or. basis(f) .eq. 20 .or. &
               basis(f) .eq. 21) then
               zeig  = zeig+ipbaspar(3,f)
            else
               zeig  = zeig+gdim(f)
            end if
            maxgdim  = max(maxgdim,gdim(f)) 
            if(basis(f).eq.12 .or. basis(f).eq.20 .or. basis(f) .eq. 21) &
                maxgdim=max(maxgdim,ipbaspar(3,f))
         enddo
         ortdim1=zeig-1
         maxgdim2  = maxgdim**2
         call getcsys(1)

      else
         do f=1,ndof
            zort1(f)=1
         enddo
         ortdim1=1
      endif

!-----------------------------------------------------------------------
! Additional pointers and dimensions 
!-----------------------------------------------------------------------
      modcdim=ctdim/potdim(modc)
      coeffdim=modcdim*psubdim(modc)
      c2v(1)=modcdim
      work2a=c2v(1)+maxpsub
      mm=0
      do m=1,npmode
         if (m.ne.modc) then
            mm=mm+1
            c2v(mm+1)=c2v(mm)/potdim(m)
            if (mm+1.eq.2) work2a=work2a+c2v(mm+1)
         endif
      enddo

      work2b=0

!     calculate the maximal amount of memory which is needed for the
!     transformation potential -> coefficients.

      call setv2c
      if (npmode.gt.3) then
         work2b=v2c(1,2)+v2c(2,2)
      elseif(npmode.eq.3) then
         work2b=v2c(1,2)
      else
        work2b=0
      endif
      
!-----------------------------------------------------------------------
! set minimum array sizes to check available workspace
!
!  complex workspace: none
!  real workspace   : see below
!  integer workspace: none
!  logical workspace: none
!-----------------------------------------------------------------------

     workdim1 = max(maxpsub2,maxpsub*npmode)
     workdim2 = max(maxpsub2+2*maxpsub,coeffdim,work2a,work2b)

      if (lspenergy.or.lcutalong) then
        if (dimbef.eq.1) then
          workdim2 = dimaft
        else if (dimaft.eq.1) then
          workdim2 = dimbef
        else
          workdim2 = dimbef * dimaft
        endif
! offs are necessary for the potfit transformation
        offs=1
        offs=offs+c2v(1)
        offs=offs+c2v(2)
        workdim2 = workdim2 + offs
      end if
      
      return
 912  routine='zeigpfit'
      message='continuation and rdnpot require an existing vpot file!'// &
              '               Cannot open vpot file: '//vname(1:vlength)
      call errormsg

      end subroutine zeigpfit


!***********************************************************************
!
!     calculate the order of the degrees of freedom for 
!     transforming the potential into the contracted coefficients.
!     the transformation leading to the largest decrease of
!     memory has the highest priority.
!
!***********************************************************************

      subroutine setv2c

use global
use pglobal
use maxv, only:maxdim
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use combined

      implicit none

      integer(long)                      :: m,mm,size,ub,i,bef
      real(dop)                          :: x
      real(dop), dimension(maxdim)       :: key
      logical(kind=4), dimension(maxdim) :: transformed

      do m=1,npmode
! MGPF: if cutnpot
!         if(.not.lcutnpot) pdim(m)=potdim(m)
         transformed(m)=.false.
      enddo
      
      ub=npmode-1
      mm=0
      do m=1,npmode
         if (m.ne.modc) then
            mm=mm+1
            v2c(mm,1)=m
            key(mm)=dble(psubdim(m))/pdim(m)
         endif
      enddo

!     sort v2c(1:ub,1) according to decreasing keys
      
      do m=1,ub-1
         do mm=m+1,ub
            if (key(mm).gt.key(m)) then
               x=key(m)
               key(m)=key(mm)
               key(mm)=x
               i=v2c(m,1)
               v2c(m,1)=v2c(mm,1)
               v2c(mm,1)=i
            endif
         enddo
      enddo

!     now the order of the degrees of freedoms is fixed: fill up v2c

      size=vpotdim
      
      do m=1,ub
         mm=v2c(m,1)
         transformed(mm)=.true.

         bef=1
         do i=1,mm-1
            if (transformed(i)) then
               bef=bef*pdim(i)
            else
               bef=bef*psubdim(i)
            endif
         enddo

         v2c(m,3)=bef
         v2c(m,5)=size/(bef*psubdim(mm))
            
         size=(size/psubdim(mm))*pdim(mm)
         v2c(m,2)=size
         v2c(m,4)=psubdim(mm)
         v2c(m,6)=pdim(mm)

      enddo

      return
      end subroutine setv2c


!***********************************************************************
!
!  determines the offsets for the different arrays used by potfit
!***********************************************************************

      subroutine mempfit

use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod
use channels
use mpidata
#ifdef MPI
use mpi
#endif

      implicit none

      character(len=16) :: progpart
      real(dop)         :: mb

!--------------------------------------------------------------------
! assign standard arrays to memory arrays
! mcdim, mrdim, midim and mldim are final addresses assigned in mc, 
! mr, mi, ml and ms. Everything after this is available as workspace.
!--------------------------------------------------------------------

!-----------------------------------------------------------------
! Set pointers to complex arrays in memory array mc
! Set pointer mrhops=1, this may be overwritten by memhops_pfit
!-----------------------------------------------------------------
      mcdim=1

!-----------------------------------------------------------------
! Set pointers to real arrays in memory array mr
! NB mrvvec is a dimension not a pointer
!-----------------------------------------------------------------
      mrvvec = 1
      mrdim = 1
      mrbuf = 1
      mrort = mrdim
      mrdim = mrort+ortdim
      if (lwo) then
         mrvvec = 1
      else
         mrvvec = vpotdim
      endif
      mrbuf=mrdim
      mrdim=mrdim+niobuf
      if (.not.lwo .and. .not. lwrcw) then
         mrevec = mrdim
         mrdim = mrevec  + evecdim
!----------------------------------------------------------------
!  NOTE: ext-natpot + readvpot + writecw
!    This is option is very unsual (test purposes)
!    since it implies a MGPF with a DVR grid size
!    which could be computed with a potfit and the
!    further analysis by fiterr
!----------------------------------------------------------------
      else if (lextpot .and. lcw) then
         mrevec = mrdim
         mrdim = mrevec  + evecdim
!----------------------------------------------------------------
!  NOTE: ext-natpot + readvpot + writecw
!----------------------------------------------------------------
      else
         coeffdim = 1
      endif
      mroned = mrdim
      mrdim = mroned + ortdim
      if (.not.lwo) then
         mrwght = mrdim
         mrdim = mrwght  + weightdim 
         mrwghti = mrdim
         mrdim = mrwghti + weightdim 
         mreval = mrdim
         mrdim = mreval    + evaldim 
         mrcut = mrdim
         mrdim = mrcut + modcdim
         mrwork1  = mrdim
         mrdim = mrwork1 + workdim1 
      else
         workdim2 = 1
      endif
      if (lsubtract .and. lensubtr .gt. 0) then
         mrort1=mrdim
         mrdim=mrdim+ortdim1
      else
         mrort1=1
      endif
      mrworkr = mrdim
      mrdim = mrworkr+workrdim

!-----------------------------------------------------------------
! Set pointers to integer arrays in memory array mi 
!-----------------------------------------------------------------
      midim=1
      miwvec= 1
      if(lcw) then
         miwvec = midim
         midim  = miwvec + vpotdim/30 + 1
      endif

!-----------------------------------------------------------
! Set pointers to logical arrays in memory array ml 
!-----------------------------------------------------------
      mldim=1

!-----------------------------------------------------------------------
! Set pointers to single precision real. They are no longer used in
! POTFIT
!-----------------------------------------------------------------------
      mfdim=2*(mrvvec+coeffdim) ! This is for logdat only!
      mfvvec=1

!-----------------------------------------------------------------------
! Write memory consumption to log file.
!-----------------------------------------------------------------------
      if (mpirank.gt.0) goto 999
      progpart='runpfit'
      call logmem(progpart)
      mb=131072.d0
      write(2,'(/a,i11,i13,",",f11.2," Mb")') 'workdim1,2:', &
            workdim1,workdim2,dble(workdim1+workdim2)/mb
      write(2,'((a,i13,",",f11.2," Mb"))')                     &
           'Buffer size:  niobuf =', niobuf, dble( niobuf)/mb, &
           'Grid size:   vpotdim =',vpotdim, dble(vpotdim)/mb, &
           'C-tensor:      ctdim =',ctdim,   dble(ctdim)/mb   
      write(2,'(a,i8,", ",a,i8)') &
            '   modcdim =',modcdim,'evecdim =',evecdim

 999  return

      return
      end subroutine mempfit

!#######################################################################

      subroutine memhops_pfit

use global
use dvrdatmod
use griddatmod
use psidef
use operdef
use memhopsmodmod

      implicit none

!-----------------------------------------------------------------------
! calculate work array sizes
!-----------------------------------------------------------------------
      workhcdim=1
! for hcall
      workhcdim=max(workhcdim,2*maxgdim+3*sdim)

      workhrdim=1
      workhidim=1
      workhldim=1

!----------------------------------------------------------------------
! set pointers to real arrays in memory array hr
!-----------------------------------------------------------------------
      hrdim=1
      hrhops=hrdim
      hrdim=hrhops+hopsdim + 1
      hrworkr=hrdim
      hrdim=hrdim+workhrdim

!-----------------------------------------------------------------------
! now alter memory assignments for mr to include hr.
!-----------------------------------------------------------------------
      mrhr=mrdim
      mrhops=mrdim
      mrdim=mrhops+hopsdim + 1
      mrdim=mrdim+workhrdim

      return
      end subroutine memhops_pfit


      end module zeigpfitmod

