! **********************************************************************
! *                                                                    *
! *                         D1ADDHUNPHI
! *                                                                    *
! * MCTDH module containing (auxiliary) subroutines to evaluate the    *
! * right hand side of the MCTDH working equations.                    *
! *                                                                    *
! * Contains:                                                          *
! * d1addhunphi:    Adds for all modes and states the action of the    *
! *                 uncorrelated Hamiltonian on the single-particle    *
! *                 functions to the time derivative vector.           *
! * d1addhunphi1ms: Adds for one mode and state the action of the      *
! *                 uncorrelated Hamiltonian on the single-particle    *
! *                 functions to the time derivative vector.           *
! *                                                                    *
! **********************************************************************
      module d1addhunphimod

use decimal, only: dop, long
use op2lib,  only: addmxxzo, submxxzo1

      implicit none
      private
      public :: d1addhunphi,d1addhunphi1ms
      contains

! *** V8.0 AR *************> SUBROUTINE ADDHUNPHI <*********************
!                           ~~~~~~~~~~~~~~~~~~~~~~

! Adds for all modes and states the action of the uncorrelated
! Hamiltonian on the single-particle functions to the time derivative.

      subroutine d1addhunphi (dtpsi,hunpsi)

use global
use griddatmod
use psidef
use maxv, only: maxdim,maxsta

      implicit none

      integer(long)                                      :: m,s,t
      complex(dop), dimension(dgldim), intent(inout)     :: dtpsi
      complex(dop), dimension(adim+1,dgldim), intent(in) :: hunpsi

! --- LOOP OVER EACH MODE AND STATE ---

      do m = 1,nmode
         do s = 1,nstate
            do t = 1,nstate
               call d1addhunphi1ms(dtpsi(dzetf(m,1,1)),hunpsi,m,s,t, &
                    ddim(m,s,t),subdim(m),dzetf(m,1,1),phidim(m))
            enddo
         enddo
      enddo

      return
      end subroutine d1addhunphi


! *** V8.0 AR ************> SUBROUTINE ADDHUNPHI 1MS <******************
!                          ~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! Adds for one mode and state the action of the uncorrelated
! Hamiltonian on the single-particle functions to the time derivative.
!#######################################################################

      subroutine d1addhunphi1ms (dtpsi,hunpsi,m,s,t,&
                                 dim1,subdim1,dzetf1,phidim1)

use global
use griddatmod
use psidef
use runpropmod
use maxv,   only: maxdim,maxsta
use d1tens, only: makeadj

      implicit none

      integer(long), intent(in)                                       :: m,s,t,dim1,subdim1,dzetf1,phidim1
      complex(dop), dimension(dzetf1:dzetf1+phidim1-1), intent(inout) :: dtpsi
      complex(dop), dimension(adim+1:dgldim), intent(in)              :: hunpsi
      complex(dop), dimension(dim1*subdim1)                           :: workc

! --- ADD ACTION OF UNCORRELATED HAMILTONIAN TO TIME DERIVATIVE ---

      call addmxxzo(hunpsi(dzetf(m,s,t)),dtpsi(dzetf(m,s,t)),subdim(m),&
           ddim(m,s,t))

!     ddim(m,s,t) = ddim(m,t,s)
      call makeadj(hunpsi(dzetf(m,s,t)),workc,supgdim(m),ddim(m,s,t))
      if (lrlx) then
         call addmxxzo(workc,dtpsi(dzetf(m,t,s)),subdim(m),ddim(m,t,s))
      else
         call submxxzo1(dtpsi(dzetf(m,t,s)),workc,subdim(m),ddim(m,t,s))
      endif

      return
      end subroutine d1addhunphi1ms


      end module d1addhunphimod



