!#######################################################################
      module d1mfieldsmod

use decimal,    only: long, dop
use globmemmod
use logdat
use mtlib,      only: mtxxzz, qtxxzz
use xvlib,      only: xmxxzzo, xvxxzza, xvxxzzo, xqxxzzo
use rmlib,      only: rmmxxxzz, rmhxxxzz,rmqxxxzz
use op1lib,     only: cpvxz, zerovxz
use op2lib,     only: swapsxxi, addvxxzo, subvcxzo, addvcxzo
use timing

      implicit none
      private
      public :: d1mfields
      contains

!#######################################################################
! MCTDH-module Matrizen         
!
!#######################################################################

      subroutine d1mfields(psi,dtpsi)

use global
use griddatmod
use operdef
use psidef
use runpropmod

use d1evaluate, only: chkinib

      implicit none

      integer(long)                                    :: s,s1,s2,t,t1,t2,&
                                                          k,kt,n
      complex(dop), dimension(dgldim), intent(in)      :: psi
      complex(dop), dimension(dgldim), intent(inout)   :: dtpsi
      integer(long)                                    :: tid=0 ! timer ID

      call get_timer(tid, 'Mfields')
      call start_timer(tid)

      if (lcdvr.or.ltdvr) then
         message='CDVR not yet implemented.'
         call errormsg
      endif

      do k=1,kzahl(1)
!     Hamiltonian terms
         if (kact(k).eq.1) then
            s=kf(k)
            s1=ki(k)
            if (s1 .ne. s) then
               do t=1,nstate
                  call d1matrizen2(k,s,s1,t,psi,dtpsi,hteil, &
                       mc(mchloch))
               enddo
            else 
               do t=1,nstate
                  call d1matrizen1(k,s,t,psi,dtpsi,hteil,mc(mchloch))
               enddo
            endif
         endif
!     Purely left-acting dissipative terms
         if (kact(k).eq.2) then
            s1 = kf(k)
            t1 = ki(k)
            do t=1,nstate
               call d1matrizen2d(k,s1,t,t1,t,t,psi,dtpsi,hteil, &
                    mc(mchloch))
            enddo
         endif
!     Purely right-acting dissipative terms
         if (kact(k).eq.3) then
            s2 = ki(k)
            t2 = kf(k)
            do t=1,nstate
               call d1matrizen2d(k,t,s2,t,t2,t,psi,dtpsi,hteil, &
                    mc(mchloch))
            enddo
         endif
!     left-right acting dissipative terms
         if (kact(k).eq.4) then
            kt = ktwin(k)
            s1 = kf(k)
            s2 = ki(kt)
            t1 = ki(k)
            t2 = kf(kt)
            call d1matrizen2d(k,s1,s2,t1,t2,1,psi,dtpsi,hteil, &
                 mc(mchloch))
         endif
      enddo

      if (ldissin) call chkinib(dtpsi,.false.)

      do n=1,nmulpot(1)
         message='Multidimensional Hterms not yet implemented.'
         call errormsg
      enddo

      call stop_timer(tid)

      return
      end subroutine d1mfields

!#######################################################################
! D1MATRIZEN1
!
! called by funk,funkr
! calls to matvec,hrhomat,matnum
!
! For Hamiltonian terms involving only a single state,
! calculates the mean field matrices. It also calculates the 
! time derivative of the MCTDH coefficients.
!
! This involves multiplying the coefficients by the
! corresponding single particle operator matrix elements.
!
! input:
! hteil, workc are standard arrays, see
!   documentation
! block,dim,vdim,ndim are standard arrays for one state
! psi  : coefficients for relevant state
! k    : number of correlated Hamiltonian term
! output:
! hloch: mean fields for kth Hamiltonian term
! dtpsi: derivative wrt time of wavefunction coefficients
!
! V5.0 MB
!
!#######################################################################

      subroutine d1matrizen1(k,s,t,psi,dtpsi,hteil,hloch)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod
use runpropmod
use openmpmod

      implicit none

      integer(long)                                  :: zeig1,zeig2,zeig3,m,m1,h
      integer(long), intent(in)                      :: k,s,t
      complex(dop), dimension(adim), intent(in)      :: psi
      complex(dop), dimension(adim), intent(inout)   :: dtpsi
      complex(dop), dimension(pmatdim), intent(in)   :: hteil
      complex(dop), dimension(hlochdim), intent(out) :: hloch
      complex(dop), dimension(3*MaxBlock)            :: workc
      logical(kind=4)                                :: work2eq3

! --- INITIALISE VARIABLES ---------------------------------------------
! workc is divided into three parts of length block.
! The result of a multiplication is stored after zeig2.
! Before multiplication, the vector to be multiplied is moved to after
! zeig1. After zeig3 is stored the vector multiplied up to mode m-1.
!-----------------------------------------------------------------------
      zeig1 = dblock(s,t)+1
      zeig2 = 1
      zeig3 = 2*dblock(s,t)+1
      call cpvxz(psi(dzpsi(s,t)),workc(zeig2),dblock(s,t))
      work2eq3 = .false.

!-----------------------------------------------------------------------
!  Loop over modes. 
!  First set initial pointers, then multiply wavefunction 
!  with single-particle hamiltonian matrices corresponding to 
!  dimensions numbers lower than mode 
!-----------------------------------------------------------------------
      do m=1,nmode
         if (m.ne.1) then
            if (.not. work2eq3) then
               call cpvxz(workc(zeig3),workc(zeig2),dblock(s,t))
               work2eq3 = .true.
            endif
            if (.not.diag(m-1,k)) then
               call swapsxxi(zeig1,zeig2)
               work2eq3 = .false.
! Parallel?
               call qtxxzz(hteil(dpmat(m-1,k,t)),workc(zeig1), &
                    workc(zeig2),vddim(m-1,s,t),ddim(m-1,s,t), &
                    nddim(m-1,s,t))
            endif
         endif
!-----------------------------------------------------------------------
!  save wavefunction multiplied up to mode m-1, if it has changed.
!-----------------------------------------------------------------------
         if (.not. work2eq3) then
            call cpvxz(workc(zeig2),workc(zeig3),dblock(s,t))
            work2eq3 = .true.
         endif

!-----------------------------------------------------------------------
!  Multiply array with single-particle hamiltonian matrices for
!   modes higher than m.
!  If H is unitary for mode m, or basis is complete, do not bother as 
!   hloch is never needed.
!-----------------------------------------------------------------------
         if (.not. diag(m,k) .and. .not. complete(m)) then
            do m1=m+1,nmode
               if (.not.diag(m1,k)) then
                  call swapsxxi(zeig1,zeig2)
                  work2eq3 = .false.
! Parallel?
                  call qtxxzz(hteil(dpmat(m1,k,t)),workc(zeig1), &
                       workc(zeig2),vddim(m1,s,t),ddim(m1,s,t), &
                       nddim(m1,s,t))
               endif
            enddo

!-----------------------------------------------------------------------
!  calculate single-hole matrix
!-----------------------------------------------------------------------
! Parallel?
            call rmhxxxzz(psi(dzpsi(s,t)),workc(zeig2), &
                 hloch(dphloch(m,k,t)),vddim(m,s,t),ddim(m,s,t), &
                 nddim(m,s,t))

            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1) &
               call xqxxzzo(koeff(k),hloch(dphloch(m,k,t)),ddim(m,s,t))
         endif
      enddo

!-----------------------------------------------------------------------
! Now calculate the time derivative of the coefficients.
! calculation of hloch for final mode has left workc(zeig2) with the 
! product of psi(block) and hteil matrices for all, excluding the final 
! mode. 
!-----------------------------------------------------------------------
      m=nmode
      if (.not.diag(m,k)) then
         call swapsxxi(zeig1,zeig2)
         work2eq3 = .false.
! Parallel?
         call qtxxzz(hteil(dpmat(m,k,t)),workc(zeig1),workc(zeig2), &
              vddim(m,s,t),ddim(m,s,t),nddim(m,s,t))
      endif

!-----------------------------------------------------------------------
!  multiply by Hamiltonian coefficient and add to dtpsi
!-----------------------------------------------------------------------
      h=hamilton(fkoe,k)
      if (htmtype(h) .ne. 1) call xvxxzzo(koeff(k),workc(zeig2),dblock(s,t))
!     Take care that dblock(s,t) = dblock(t,s)
      call addvxxzo(workc(zeig2),dtpsi(dzpsi(s,t)),dblock(s,t))

      if (lrlx) then
         call addvcxzo(workc(zeig2),dtpsi(dzpsi(t,s)),dblock(t,s))    
      else
         call subvcxzo(workc(zeig2),dtpsi(dzpsi(t,s)),dblock(t,s))
      endif

      return
      end subroutine d1matrizen1

!#######################################################################
! MATRIZEN2
!
! called by funk,funkr
! calls to usmatvec,usrhomat,matnum
!
! calculates the mean field matrices involving hamiltonian coefficients
!  that connect two states. It also calculates the time derivative of 
!  the MCTDH coefficients.
!
! This involves multiplying the coefficients of the state s1 by the
! corresponding single particle operator matrix elements.
!
! input:
! s: state on LHS of mean field matrix
! s1: state on RHS of mean field matrix
! psi: coefficients for all states 
! hteil, workc are standard arrays, see documentation
! k: which correlated Hamiltonian coefficient passed from calling
! routine
! output:
! dtpsi: derivative wrt time of A coefficients for state on LHS of
!        mean field matrix
! hloch: mean field for kth Hamiltonian term.
!
! V6.0 GW
!#######################################################################

      subroutine d1matrizen2(k,s,s1,t,psi,dtpsi,hteil,hloch)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod
use runpropmod

      implicit none
      
      integer(long)                                      :: zeig1,zeig2,zeig3,&
                                                            swapzeig,m,m1, &
                                                            vdim1,block1,h
      integer(long), intent(in)                          :: k,s,s1,t
      complex(dop), dimension(adim), intent(in)          :: psi
      complex(dop), dimension(adim), intent(inout)       :: dtpsi
      complex(dop), dimension(pmatdim), intent(in)       :: hteil
      complex(dop), dimension(hlochdim), intent(out)     :: hloch
      complex(dop), dimension(1:3*MaxBlock)              :: workc


! --- INITIALISE VARIABLES ---------------------------------------------
! workc is divided into three parts of length maxblock, the longest
! possible
! length of the array during the multiplication.
! The result of a multiplication is stored after zeig2.
! Before multiplication, the vector to be multiplied is moved to after
! zeig1.
! After zeig3 is stored the vector multiplied up to mode m-1
!-----------------------------------------------------------------------
      call zerovxz(workc,3*MaxBlock)
      zeig1=maxblock+1
      zeig2=1
      zeig3=2*maxblock+1
      call cpvxz(psi(dzpsi(s1,t)),workc(zeig2),dblock(s1,t))
      block1=dblock(s1,t)

!-----------------------------------------------------------------------
!  First set initial pointers, then multiply wavefunction of state s1
!  with single-particle hamiltonian matrices corresponding to 
!  dimensions numbers lower than mode 
!  as the matrix elements are between different states with different
!   number of coefficients, the following pointers are needed:
!  block1: number of elements in multiplied array.
!  vdim1: equivalent of vdim during multiplication over the modes lower
!  than m.
!  vdim2: equivalent of vdim during multiplication over the modes higher
!  then m.
!  before a multiplication, block=vdim1*ddim(m,s1,t)*nddim(m,s1,t)
!  after a multiplication, block=vdim1*ddim(m,s,t)*nddim(m,s1,t)
!-----------------------------------------------------------------------
      do m=1,nmode
         if (m.ne.1) then
            call cpvxz(workc(zeig3),workc(zeig2),block1)

            swapzeig=zeig1
            zeig1=zeig2
            zeig2=swapzeig
! Parallel?
            call mtxxzz(hteil(dpmat(m-1,k,t)),workc(zeig1), &
                 workc(zeig2),vddim(m-1,s,t),ddim(m-1,s1,t), &
                 nddim(m-1,s1,t),ddim(m-1,s,t))

            block1=vddim(m-1,s,t)*ddim(m-1,s,t)*nddim(m-1,s1,t)
         endif

!-----------------------------------------------------------------------
!  save wavefunction multiplied up to m-1, if it has changed.
!-----------------------------------------------------------------------
         call cpvxz(workc(zeig2),workc(zeig3),block1)

!-----------------------------------------------------------------------
!  Multiply array with single-particle hamiltonian matrices for
!   modes higher than m.
!  If H is unitary for mode m, or basis is complete, do not bother as 
!   hloch is never needed.
!-----------------------------------------------------------------------
         if (.not. complete(m)) then
            vdim1=vddim(m,s,t)*ddim(m,s1,t)
            do m1=m+1,nmode
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig

! Parallel?
               call mtxxzz(hteil(dpmat(m1,k,t)),workc(zeig1), &
                    workc(zeig2),vdim1,ddim(m1,s1,t),nddim(m1,s1,t), &
                    ddim(m1,s,t))

               vdim1=vdim1*ddim(m1,s,t)
            enddo
!-----------------------------------------------------------------------
!  calculate single-hole matrix
!-----------------------------------------------------------------------
! Parallel?
            call rmmxxxzz(psi(dzpsi(s,t)),workc(zeig2), &
                 hloch(dphloch(m,k,t)),vddim(m,s,t),ddim(m,s,t), &
                 nddim(m,s,t),ddim(m,s1,t))
            
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1) &
                 call xmxxzzo(koeff(k),hloch(dphloch(m,k,t)),ddim(m,s,t) &
                 ,ddim(m,s1,t))
         endif
      enddo

!-----------------------------------------------------------------------
! now calculate the time derivative of the coefficients.
! calculation of hloch for final mode has left workc(zeig2) with the 
! product of psi(block) and hteil matrices for all, excluding the final 
! mode. 
!-----------------------------------------------------------------------
      m=nmode
      swapzeig=zeig1
      zeig1=zeig2
      zeig2=swapzeig
! Parallel?
      call mtxxzz(hteil(dpmat(m,k,t)),workc(zeig1),workc(zeig2), &
           vddim(m,s,t),ddim(m,s1,t),nddim(m,s,t),ddim(m,s,t))

!-----------------------------------------------------------------------
!  multiply by Hamiltonian coefficient
!-----------------------------------------------------------------------
      h=hamilton(fkoe,k)
      if (htmtype(h) .ne. 1)&
           call xvxxzzo(koeff(k),workc(zeig2),dblock(s,t)) 
!     Take care that dblock(s,t) = dblock(t,s)
      call addvxxzo(workc(zeig2),dtpsi(dzpsi(s,t)),dblock(s,t))
      if (lrlx) then
         call addvcxzo(workc(zeig2),dtpsi(dzpsi(t,s)),dblock(t,s)) 
      else
         call subvcxzo(workc(zeig2),dtpsi(dzpsi(t,s)),dblock(t,s))
      endif

      return
      end subroutine d1matrizen2


!#######################################################################
! MATRIZEN2D
!
! Calculates the hloch matrices corresponding to the dissipative terms,
! and, as a by-product, parts of the time-dervative of the coefficients.
!
! (s1,s2) -> (alpha,beta), (t1,t2) -> (alpha',beta'), t -> hteil
! hteil, hloch -> ddim(m,s1,s2) x ddim(m,t1,t2)
!
! 10/99 AR
!#######################################################################

      subroutine d1matrizen2d(k,s1,s2,t1,t2,t,psi,dtpsi,hteil,hloch)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod
use runpropmod

      implicit none
      
      integer(long)                                      :: zeig1,zeig2,zeig3,swapzeig,m,m1,vdim1,block1,h
      integer(long), intent(in)                          :: k,s1,s2,t1,t2,t
      complex(dop), dimension(adim), intent(in)          :: psi
      complex(dop), dimension(adim), intent(inout)       :: dtpsi
      complex(dop), dimension(pmatdim), intent(in)       :: hteil
      complex(dop), dimension(hlochdim), intent(out)     :: hloch
      complex(dop), dimension(1:3*MaxBlock)              :: workc


! --- INITIALISE VARIABLES ---------------------------------------------
! workc is divided into three parts of length maxblock, the longest
! possible
! length of the array during the multiplication.
! The result of a multiplication is stored after zeig2.
! Before multiplication, the vector to be multiplied is moved to after
! zeig1.
! After zeig3 is stored the vector multiplied up to mode m-1
!-----------------------------------------------------------------------
      call zerovxz(workc,3*MaxBlock)
      zeig1=maxblock+1
      zeig2=1
      zeig3=2*maxblock+1
      call cpvxz(psi(dzpsi(t1,t2)),workc(zeig2),dblock(t1,t2))
      block1=dblock(t1,t2)

!-----------------------------------------------------------------------
!  First set initial pointers, then multiply wavefunction of state s1
!  with single-particle hamiltonian matrices corresponding to 
!  dimensions numbers lower than mode 
!  as the matrix elements are between different states with different
!   number of coefficients, the following pointers are needed:
!  block1: number of elements in multiplied array.
!  vdim1: equivalent of vdim during multiplication over the modes lower
!  than m.
!  vdim2: equivalent of vdim during multiplication over the modes higher
!  then m.
!  before a multiplication, block=vdim1*ddim(m,t1,t2)*nddim(m,t1,t2)
!  after a multiplication, block=vdim1*ddim(m,s1,s2)*nddim(m,t1,t2)
!-----------------------------------------------------------------------
      do m=1,nmode
         if (m.ne.1) then
            call cpvxz(workc(zeig3),workc(zeig2),block1)
!     Note that for ki .ne. kf always diag(m,k)=.false.
            if (.not. diag(m-1,k)) then
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig
! Parallel?
               call mtxxzz(hteil(dpmat(m-1,k,t)),workc(zeig1), &
                    workc(zeig2),vddim(m-1,s1,s2),ddim(m-1,t1,t2), &
                    nddim(m-1,t1,t2),ddim(m-1,s1,s2))
               block1=vddim(m-1,s1,s2)*ddim(m-1,s1,s2)*nddim(m-1,t1,t2)
            endif
         endif

!-----------------------------------------------------------------------
!  save wavefunction multiplied up to m-1, if it has changed.
!-----------------------------------------------------------------------
         call cpvxz(workc(zeig2),workc(zeig3),block1)

!-----------------------------------------------------------------------
!  Multiply array with single-particle hamiltonian matrices for
!   modes higher than m.
!  If H is unitary for mode m, or basis is complete, do not bother as 
!   hloch is never needed.
!-----------------------------------------------------------------------
         if (.not. diag(m,k) .and. .not. complete(m)) then
            vdim1=vddim(m,s1,s2)*ddim(m,t1,t2)
            do m1=m+1,nmode
!     Note that for ki .ne. kf always diag(m,k)=.false.
               if (.not. diag(m1,k)) then
                  swapzeig=zeig1
                  zeig1=zeig2
                  zeig2=swapzeig
! Parallel?
                  call mtxxzz(hteil(dpmat(m1,k,t)),workc(zeig1), &
                       workc(zeig2),vdim1,ddim(m1,t1,t2), &
                       nddim(m1,t1,t2),ddim(m1,s1,s2))
                  vdim1=vdim1*ddim(m1,s1,s2)
               endif
            enddo
!-----------------------------------------------------------------------
!  calculate single-hole matrix
!-----------------------------------------------------------------------
! Parallel?
            call rmmxxxzz(psi(dzpsi(s1,s2)),workc(zeig2), &
                 hloch(dphloch(m,k,t)),vddim(m,s1,s2),ddim(m,s1,s2), &
                 nddim(m,s1,s2),ddim(m,t1,t2))

            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1) then
               call xmxxzzo(koeff(k),hloch(dphloch(m,k,t)), &
                    ddim(m,s1,s2),ddim(m,t1,t2))
            endif
               
         endif
      enddo

!-----------------------------------------------------------------------
! now calculate the time derivative of the coefficients.
! calculation of hloch for final mode has left workc(zeig2) with the 
! product of psi(block) and hteil matrices for all, excluding the final 
! mode. 
!-----------------------------------------------------------------------
!     Note that for ki .ne. kf always diag(m,k)=.false.
      if (.not. diag(nmode,k)) then
         m=nmode
         swapzeig=zeig1
         zeig1=zeig2
         zeig2=swapzeig
! Parallel?
         call mtxxzz(hteil(dpmat(m,k,t)),workc(zeig1),workc(zeig2), &
              vddim(m,s1,s2),ddim(m,t1,t2),nddim(m,t1,t2),ddim(m,s1,s2))
      endif

      h=hamilton(fkoe,k)
      if (htmtype(h) .ne. 1) then
         call xvxxzza(koeff(k),workc(zeig2), &
              dtpsi(dzpsi(s1,s2)),dblock(s1,s2))
      else
         call addvxxzo(workc(zeig2),dtpsi(dzpsi(s1,s2)), &
              dblock(s1,s2))
      endif


      return
      end subroutine d1matrizen2d


      end module d1mfieldsmod

