!***********************************************************************
!
!                       module D1TRACEOUT
!
! V81 AR
!
!***********************************************************************
      module d1traceoutmod

use decimal, only: long,dop
use globmemmod
use logdat
use op1lib,  only: cpvxz, zerovxz

use global
use maxv, only: maxsta
use griddatmod
use psidef
use runpropmod
use channels
use operdef
use hpsimod

use d1lib,        only: trspdoc,trspdoel
use d1hphimod,    only: d1hphi,d1hunphi
use expectval,    only: calceuncorr1,calcecorr1,calcenrho
use tdkoeffs,     only: calctdk
use d1tens,       only: ptrc
use dichtoutmod,  only: stproj
use dvrdatmod,    only: ortdim
use multilayer,   only: ztraceml
use wftree1,      only: allocate_zmtraces_matrices
use mlanalysis,   only: compute_mtraces_tree
use linear,       only: ijk_shape
use mtlib,        only: qtxxzz,qtaxzz

 
      implicit none
      private
      public :: d1traceout
      contains

!***********************************************************************
!                               TRACEOUT1
!
!    writes out trace of rho(dlstate,drstate), needed for spectrum
!
! V81 AR
!***********************************************************************

      subroutine d1traceout(psi,atime,firstcall)

      implicit none

      integer(long)                               :: m,n,s,t,z1,z2,hz
      real(dop)                                   :: time,old_intrace
      real(dop), dimension(maxsta)                :: old_decorr,old_deuncorr,&
                                                     old_detot
      real(dop), intent(in)                       :: atime
      complex(dop)                                :: trc
      complex(dop), dimension(dgldim), intent(inout) :: psi
      complex(dop), dimension(dgldim)                :: psi1
      complex(dop), dimension(dgldim)                :: tmppsi
      complex(dop), dimension(d3matdim)           :: trace
      complex(dop), dimension(2*maxblock)         :: workc
      logical(kind=4), intent(in)                 :: firstcall

      time = atime*fs

!-----------------------------------------------------------------------
!     get operator number
!-----------------------------------------------------------------------
      if (firstcall) then
         if (firstcall.and.(ctrname.ne.' ')) then
            do n=1,hamdim
               if (hamlab(n).eq.ctrname) ctrnum=n
            enddo
            if (ctrnum.eq.0) then
               message = 'Operator used with <ctrace> not defined!'
               call warnmsg
            endif
         endif
      else
         ctrnum=0
      endif

      z1 = 1
      z2 = z1 + maxblock

      if (lmult) then
         s = dlstate
         t = drstate
         call cpvxz(psi(dzpsi(s,t)),workc(z1),dblock(s,t))
      else
         if (lrunml) call getprojdim(tree,vddim(meb,1,1),nddim(meb,1,1))
         call qtxxzz(psi(zetf(meb,1)),psi(1),psi1,vddim(meb,1,1),ddim(meb,1,1),&
                     nddim(meb,1,1))  ! transform to state basis
         call astprojlr(psi1,workc(z2),vddim(meb,1,1),&
                    nddstate,nddim(meb,1,1))
         call qtaxzz(psi(zetf(meb,1)),workc(z2),workc(z1),vddim(meb,1,1),&
              ddim(meb,1,1),nddim(meb,1,1))  ! transform back to spf basis
         s = 1
         t = 1
      endif

      if (ctrnum.eq.0) then
! simple overlap
         do m=1,nmode
            if (leb .and. (m .eq. meb) .and. (dlstate .ne. drstate)) then
               call trspdoel(psi(dzetf(m,s,t)),trace(dd3mat(m,s,t)),ddim(m,s,t),&
                    supgdim(m),dlstate,drstate)
            else
               call trspdoc(psi(dzetf(m,s,t)),trace(dd3mat(m,s,t)),ddim(m,s,t),&
                    subdim(m),supgdim(m))
            endif
         enddo

         if (lrunml) then
            tmppsi(1:adim) = workc(1:adim)
            tmppsi(adim+1:dgldim) = psi(adim+1:dgldim)
            
            call allocate_zmtraces_matrices(tree,trace)
            call compute_mtraces_tree(tmppsi,tree,1)

            call ztraceml(tmppsi,workc(z1))

         else

            do m=1,nmode
               call ptrc(workc(z1),trace(dd3mat(m,s,t)),workc(z2),ddim(m,s,t),&
                    nddim(m,s,t))
               hz = z1
               z1 = z2
               z2 = hz
            enddo
         endif

      else
! off-diagonal operator elements
!        --- backup old variable
         do s=1,nstate
            old_decorr(s) = decorr(s)
            old_deuncorr(s) = deuncorr(s)
            old_detot(s) = detot(s)
         enddo

         old_intrace = intrace
         intrace = 1.0_dop

!        --- calculate tr{H rho}         
         trc = (0.0_dop,0.0_dop)

! calculate time-dependent coefficients. 
         if (ltdh(ctrnum)) call calctdk(time,ctrnum,psi)

         if (lexact) then
            call calcenrho(psi(dzetf(1,1,1)),decorr,deuncorr,ctrnum)

         else
            call d1hphi(psi,mc(mchpsi),0,ctrnum)
            call d1hunphi(psi,mc(mchunpsi),0,ctrnum)

            call calceuncorr1(psi,mc(mchunpsi),deuncorr,ctrnum)
            call calcecorr1(psi,mc(mchpsi),decorr,ctrnum)
         endif

         do s=1,nstate
            trc = trc + dcmplx(deuncorr(s)) + dcmplx(decorr(s))
         enddo

!      - restore old value of intrace      
         do s=1,nstate
            decorr(s) = old_decorr(s)
            deuncorr(s) = old_deuncorr(s)
            detot(s)    = old_detot(s)
         enddo

         intrace = old_intrace
      endif

      trc = 0.0_dop
      if (firstcall .and. .not. lcont) then
         if (ctrnum.eq.0) then
            inctr = workc(z1)
            if (abs(inctr) .lt. tiny) inctr = 999.00
            trc  = (1.0_dop,0.0_dop)
         else
            trc=(0.0_dop,0.0_dop)
         endif
      else
         if (ctrnum.eq.0) then
            trc = workc(z1) / inctr
         endif
      endif

      write(ictr,'(1x,g18.9,3(1x,f18.14))')&
           atime,dble(trc),dimag(trc),abs(trc)
      call myflush(ictr)

      return
      end subroutine d1traceout

!#######################################################################
!
! ASTPROJLR
!
! For single-set type I density matrices.
! Project out coefficients for the initial (potentially mixed) 
! electronic state. Need to select the real and imaginary parts
! which are in separate hermitian basis functions.
!
! NB ddim(meb,1,1) = nddstate*nddstate hence coefficent vector
! psi(vddim,ddim,nddim) = psi(vddim,nst,nst,nddim)
!
!#######################################################################

      subroutine astprojlr(psi,tmppsi,vddim1,nst,nddim1)

      implicit none

      integer(long), intent(in) :: vddim1,nst,nddim1
      complex(dop), dimension(vddim1,nst,nst,nddim1), intent(out) :: tmppsi
      complex(dop), dimension(vddim1,nst,nst,nddim1), intent(in)  :: psi


      if (dlstate .eq. drstate) then
         tmppsi(:,:,:,:) = 0.0_dop
         tmppsi(:,1,1,:) = psi(:,1,1,:) 
      else
         tmppsi(:,:,:,:) = 0.0_dop
         tmppsi(:,dlstate,drstate,:) = psi(:,dlstate,drstate,:) 
         tmppsi(:,drstate,dlstate,:) = psi(:,drstate,dlstate,:) 
      endif

      return
      end subroutine astprojlr

!#######################################################################

      subroutine getprojdim(tree,vdim1,ndim1)

      type(wfnode_t),intent(in)        :: tree
      type(wfnode_t),pointer           :: childf 
      integer, intent(out)             :: vdim1,ndim1
      integer                          :: ijk(3)
      integer                          :: f,m1

      if (.not. tree%istop) then
         routine = 'Getprojdim (d1traceout.f90)'
         message = 'Not at top of tree '
         call errormsg
      endif

      do f=1,tree%nmodes
         childf => tree%submodes(f)%p
         if (childf%isprimitive .and. childf%m .eq. meb) then
            m1=f
         endif
      enddo

      ijk = ijk_shape(m1,tree%ndim)
      vdim1 = ijk(1)
      ndim1 = ijk(3)

      end subroutine getprojdim
      

      end module d1traceoutmod

