!#######################################################################
!contains: d1zeigprop,d1wkprop,d1maxmatwk
!#######################################################################
      module d1zeigpropmod

use decimal, only: dop,long

      implicit none
      private
      public :: d1zeigprop
      contains

!#######################################################################

      subroutine d1zeigprop

use global
use griddatmod
use operdef
use psidef
use hpsimod
use runpropmod
use datenmod

use zeigpropmod, only: intdimens
use iopsidef, only: psidat
use d1crocofly, only: d1zeigcroco
use zeighpsimod, only: d1operflag,d1zeighpsi,d1maxmatwk,zeighpsi

      implicit none

      integer(long)       :: i,m,s,s1,t,zeig

!-----------------------------------------------------------------------
! set various operator related flags (kfirst etc.)
!-----------------------------------------------------------------------
      do i=1,hamdim
         call d1operflag(i)
      enddo

!-----------------------------------------------------------------------
! get information about wavefunction 
!-----------------------------------------------------------------------
      call psidat

!-----------------------------------------------------------------------
! flag any complete spf bases
!-----------------------------------------------------------------------
      do m=1,nmode
         if (subdim(m) .eq. ddim(m,1,1)) then
            complete(m)=.true.
            do s=2,nstate
               do s1=1,nstate
                  ddim(m,s,s1)=subdim(m)
               enddo
            enddo
         endif
      enddo

!-----------------------------------------------------------------------
! set up pointers for hpsi operation (and for hteil, hloch, etc.)
! using all hamiltonians 
!-----------------------------------------------------------------------
      hpsidim = 0
      hlochdim = 0
      pmatdim = 0
      do i=1,hamdim
         call d1zeighpsi(i)
      enddo


!-----------------------------------------------------------------------
! pointers for overlap matrices of single-particle functions
!-----------------------------------------------------------------------
      spfovrdim = 1

!-----------------------------------------------------------------------
! pointers for tensor containing sum over mean-field matrices
!-----------------------------------------------------------------------
      if (lconst) then
         zeig = 1
         do m = 1,nmode
            do t=1,nstate
               do s = 1,nstate
                  do s1 = 1,nstate
                     dzmfsum(m,s1,s,t) = zeig
                     zeig = zeig+supgdim(m)*ddim(m,s,t)*ddim(m,s1,t)
                  enddo
               enddo
            enddo
            zmfsum(m,1,1) = dzmfsum(m,1,1,1)
         enddo
         mfsumdim = zeig-1
      else
         mfsumdim = 1
      endif

!-----------------------------------------------------------------------
! calculate maximum "block" size needed in matrizen routines
!-----------------------------------------------------------------------
      do i=1,hamdim 
         call d1maxmatwk(i)
      enddo

!-----------------------------------------------------------------------
! calculate size of intpsi array
!-----------------------------------------------------------------------
      call intdimens

!-----------------------------------------------------------------------
! obtain size needed for calculation of cross-correlation-function
!-----------------------------------------------------------------------
      if (lcross) call d1zeigcroco

      call d1wkprop

      return
      end subroutine d1zeigprop

!#######################################################################

      subroutine d1wkprop

use global
use griddatmod
use operdef
use psidef
use hpsimod
use runpropmod
use datenmod

use zeigpropmod, only: wksil

      implicit none

!-----------------------------------------------------------------------
! set minimum array sizes to check available workspace
! complex workspace:
!  dgldim, or 3*block, or 3*maxblock (workspace needed in matrizen), or
!  2*max[dim(m,s)*subdim(m)] (needed in "natorbs1,2")
!  if (restart) then if old wavefunction is larger than new
!  workspace must be large enough for this
! real workspace:
!  max[subdim(m)]**2
! integer workspace:
!  none
! logical workspace:
!  none, unless (restart) when it must be number of coefficients in old
!  or new wavefunction, whichever is larger.
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! SIZE OF COMPLEX WORK ARRAY
!
! when the mean-fields are held constant the routine "funk3" uses
! the "intpsi" array as complex work array. For that reason "intpsi"
! must be large enough to keep three times the largest "block" (which is
! normally already the case), whereas for "workc" a size of twice the
! largest "block" suffices.
!-----------------------------------------------------------------------
      workcdim=1
      workrdim = 1
      workidim=1
      workldim = 1
      call wksil

      return
      end subroutine d1wkprop

      end module d1zeigpropmod

