!***********************************************************************
!
!                              D2AUTOCORR.F
!
! Contains all subroutines related to the output of the autocorrelation
! function.
!
! V81 AR
!
!***********************************************************************
      module d2autocorr

use decimal, only: long, dop
use mmlib,   only: mmtxzz,mmaxzz
use mtlib,   only: qtcxzz, qtxxzz,mtxxzz,qtaxzz,mttxzz
use op1lib,  only: trmmcxzz,trqxz
use psiutil, only: spfovl

use global
use griddatmod
use psidef
use channels
use multilayer, only: automl

      implicit none
      private
      public :: d2autoout
      contains

!***********************************************************************
!                               D2AUTOOUT
!
!     Computes tr( conjg(rho_1) rho_2 ) (needed in auto routines)
!
!***********************************************************************

      subroutine d2autoout (psi,agmat,atime)

      implicit none

      real(dop), intent(in)                       :: atime
      complex(dop)                                :: auto
      complex(dop), dimension(dgldim), intent(in) :: psi
      complex(dop), dimension(agmatdim), intent(in) :: agmat

      call d2calcauto(psi,agmat,auto)

      write(iaut,'(1x,g18.9,3(1x,f18.14))')&
              atime,dble(auto),dimag(auto),abs(auto)

      call myflush(iaut)

      return
      end subroutine d2autoout

!***********************************************************************
!                               D2CALCTRACE
!         
!     Computes tr( conjg(rho_1) rho_2 ) (needed in auto routines)
!         
!***********************************************************************

      subroutine d2calcauto(psi,agmat,auto)

      implicit none

      integer(long)                                 :: hzw,m,s,t,i
      integer(long), dimension(2)                   :: zw
      complex(dop)                                  :: trace
      complex(dop), intent(out)                     :: auto
      complex(dop), dimension(dgldim), intent(in)   :: psi
      complex(dop), dimension(agmatdim), intent(in) :: agmat
      complex(dop), dimension(maxblock*maxblock,2)  :: workc
      complex(dop), dimension(dmatdim)              :: overlap

      auto  = (0.0_dop, 0.0_dop)
      trace = (0.0_dop, 0.0_dop)

!-----------------------------------------------------------------------
! calculate SPF overlaps
!-----------------------------------------------------------------------
      call spfovl(overlap,psi,psi,agmat,agmat,1)

      if (lrunml) then
         call automl(auto,psi,psi,overlap)

      else
         do s = 1,nstate
            do t = 1, nstate
               zw(1) = 1
               zw(2) = 2
               do i=1,block(s)*block(t)
                  workc(i,zw(1)) = psi(dzpsi(t,s)+i-1)
               enddo
               do m=1,nmode
!                 (1) tensor of rho times spf overlaps of state s
                  call qtcxzz(overlap(dmat(m,s)),workc(1,zw(1)), &
                       workc(1,zw(2)),block(t)*vdim(m,s),dim(m,s), &
                       ndim(m,s))
                  hzw = zw(1)
                  zw(1) = zw(2)
                  zw(2) = hzw
               enddo

               do m=1,nmode
!                 (2) multiplies the scalarproduct to the spf overlaps of state t
                  call qtxxzz(overlap(dmat(m,t)),workc(1,zw(1)), &
                       workc(1,zw(2)),vdim(m,t),dim(m,t), &
                       ndim(m,t)*block(s))
                  hzw = zw(1)
                  zw(1) = zw(2)
                  zw(2) = hzw
               enddo

!              (3) derive the trace of:
!                            (tensor of rho times the result of (2)) ,
!                  rho is assumed to be hermitian

               call trmmcxzz(psi(dzpsi(s,t)),workc(1,zw(1)), &
                    trace,block(s),block(t))

               auto = auto + trace
            enddo
         enddo
      endif

      return
      end subroutine d2calcauto

!***********************************************************************
!                               D2CALCAUTO2
!         
!***********************************************************************

      subroutine d2calcauto2(rho,auto)

      implicit none

      integer(long)                                 :: hzw,m,s,t,i
      integer(long), dimension(4)                   :: zw
      complex(dop)                                  :: trace
      complex(dop), intent(out)                     :: auto
      complex(dop), dimension(dgldim), intent(in)   :: rho
      complex(dop), dimension(maxblock*maxblock,4)  :: workc
      complex(dop), dimension(maxspf*maxspf,nmode,2) :: ovl

      auto  = (0.0_dop, 0.0_dop)
      trace = (0.0_dop, 0.0_dop)

! s is final (upper) state
! t is initial (ground) state
      t = drstate
      s = dlstate

!     (1) calculates a matrix, which contains all scalarproducts
!         between the spfs of states s and t
      do m=1,nmode
         call mmaxzz(rho(zetf(m,s)),rho(zetf(m,s)), &
              ovl(1,m,1),subdim(m),dim(m,s),dim(m,s))
         call mmaxzz(rho(zetf(m,s)),rho(zetf(m,t)), &
              ovl(1,m,2),subdim(m),dim(m,t),dim(m,s))
      enddo

      zw(1) = 1
      zw(2) = 2
      do i=1,block(t)*block(s)
         workc(i,zw(1)) = rho(dzpsi(t,s)+i-1)
      enddo

!     (2) tensor of rho times scalarproducts of (1) (initial state - drstate)
!      do m=1,nmode
!         call qtaxzz(ovl(1,m,1),workc(1,zw(1)), &
!              workc(1,zw(2)),block(t)*vdim(m,s),dim(m,s),ndim(m,s))
!         hzw = zw(1)
!         zw(1) = zw(2)
!         zw(2) = hzw
!      enddo

!     (3) multiplies the scalarproduct to the product of (2) 
      do m=1,nmode
         call mttxzz(ovl(1,m,2),workc(1,zw(1)), &
              workc(1,zw(2)),block(s)*vdim(m,s),dim(m,t), &
              ndim(m,t),dim(m,s))
         hzw = zw(1)
         zw(1) = zw(2)
         zw(2) = hzw
      enddo

!     (4) get the trace 

      call trqxz(workc(1,zw(1)),auto,block(s))

      auto = 2.0*auto

      return
      end subroutine d2calcauto2

      end module d2autocorr

