!#######################################################################
!
!                      MCTDH module D2CALCDAT
!
!
! subroutines :
!    CALCDAT2 : Calculates energies and state populations  
!    OUTGPOP2 : Calculates grid populations in coordinate and 
!               momentum representations and outputs results 
!    
!
!  V81 AR
!
!***********************************************************************

      module d2calcdatmod

use decimal, only: long,dop
use op1lib,  only: trhxz
use op2lib,  only: sum2qxxz

      implicit none
      private
      public :: d2calcdat
      contains

!***********************************************************************
!
!                  MCTDH subroutine D2CALCDAT
!
! calculates data to be written to various files.
!
! passed back to calling routine:
!   spop(s):    population for each state.
!   detot(s):    Total energy for each state
!   decorr(s):   Correlated energy for each state
!   deuncorr(s): Uncorrelated energy for each state
!
! V80 AR
!
!***********************************************************************

      subroutine d2calcdat(psi,hunpsi,spop,firstcall,ecorr,etot,trace)

use global
use griddatmod
use psidef
use runpropmod
use d2tens, only: rm1_mf
use maxv,   only: maxdim,maxsta
use mmlib,  only: mmaxzzh

      implicit none

      integer(long)                                         :: m,s
      real(dop), intent(out)                                :: trace
      real(dop), dimension(nstate), intent(out)             :: spop,ecorr,etot
      complex(dop)                                          :: sum
      complex(dop), dimension(dgldim), intent(in)           :: psi
      complex(dop), dimension(adim+1:dgldim), intent(in)    :: hunpsi
      complex(dop), dimension(maxspf*maxspf,2)              :: workc
      logical(kind=4), intent(in)                           :: firstcall

!-----------------------------------------------------------------------
! calculate state populations and trace fro rho.
!-----------------------------------------------------------------------
      trace = 0.0_dop
      do s=1,nstate
         call trhxz(psi(dzpsi(s,s)),spop(s),block(s))
         trace = trace + dble(spop(s))
      enddo

!-----------------------------------------------------------------------
! intrace is the initial trace
!-----------------------------------------------------------------------
      if (firstcall.and.(.not.lcont)) then
         if (lrlx) then
            intrace = 1.0_dop
         else
            intrace = trace
            if (intrace.le.tiny) intrace=1.0e99_dop
         endif
      endif

!-----------------------------------------------------------------------
! Calculate expectation values
!-----------------------------------------------------------------------
      do s=1,nstate
         spop(s) = spop(s)/trace
         ecorr(s) = decorr(s) /trace
      enddo

!-----------------------------------------------------------------------
!     Calculate total energy
! NB for funkr, deuncorr is calculated in d2calchamod
!-----------------------------------------------------------------------
      if (lfunkl) then
         do s=1,nstate
            deuncorr(s)=0.0_dop
            do m=1,nmode
               call rm1_mf(psi(dzpsi(s,s)),workc(1,1),vdim(m,s), &
                    dim(m,s),ndim(m,s))
               call mmaxzzh(psi(zetf(m,s)),hunpsi(zetf(m,s)), &
                    workc(1,2),subdim(m),dim(m,s))
               call sum2qxxz(workc(1,1),workc(1,2),sum,dim(m,s))
               deuncorr(s)=deuncorr(s)+dble(sum)
            enddo
         enddo
      endif

      do s=1,nstate
         etot(s)=ecorr(s)+deuncorr(s)/trace
      enddo

      return
      end subroutine d2calcdat

      end module d2calcdatmod

