!***********************************************************************
!
! D2CROCOFLY.F
!
! Module to calculate the cross-correlation function on the fly
! for density matrices of type 2.
!
! Contains the following routines:
!   D2WKCROCO
!   D2CCOVERLAP
!   D2CCOUT
!
! Mar 2002, FO
!
!***********************************************************************
      module d2crocofly

use decimal, only: long, dop
use globmemmod
use mmlib,   only: mmaxzz
use mtlib,   only: mtxxzz, mttxzz
use xvlib,   only: vvaxzz
use op1lib,  only: cpvxz
use timing

      implicit none
      private
      public :: d2ccoverlap,d2ccout
      contains

!***********************************************************************
!
! subroutine D2CCOVERLAP
!
! Calculates the overlap between two wavefucntions in MCTDH form.
! [mainly cut'n'pasted from ANALYSE/CROSSCORR.F]
!
! In:
!   psiref : reference WF in MCTDH form
!   psi    : system WF
!   dover  : memory needed for overlap matrices [=mc(mcccdover)]
! Out:
!   ovl = tr( psiref^(dagger) psi )
!
! Mar 2002, FO
!
!***********************************************************************

      subroutine d2ccoverlap(ovl,psiref,psi,dover)

use global
use griddatmod
use psidef
use crocoflymod
use compdatmod

      implicit none

      integer(long)                                        :: hzw,m,s,t
      integer(long), dimension(2)                          :: zw
      integer(long)                                        :: tmpdim
      complex(dop)                                         :: x
      complex(dop), intent(out)                            :: ovl
      complex(dop), dimension(*), intent(in)               :: psi, psiref
      complex(dop), dimension(doverdim,2), intent(out)     :: dover
      complex(dop), dimension(:,:), allocatable            :: tempc
      integer(long), save                            :: tid=0 ! timer ID

      call get_timer(tid, 'Cross')
      call start_timer(tid)

!-----------------------------------------------------------------------
! work array
!-----------------------------------------------------------------------
      tmpdim=max(ccwcdim*ccwcdim,ccmaxblock*ccmaxblock)
      allocate(tempc(tmpdim,2))

      do s = nst1,nst2
         do m=1,nmode
            call mmaxzz(psi(zetf(m,s)), psiref(czetf(m,s,2)), &
                 dover(dovmat(m,s),1),subdim(m),dim(m,s),cdim(m,s,2))
            call mmaxzz(psiref(czetf(m,s,2)), psi(zetf(m,s)), &
                 dover(dovmat(m,s),2),subdim(m),cdim(m,s,2),dim(m,s))
         enddo
      enddo

      ovl  = (0.0_dop, 0.0_dop)
      do s = nst1,nst2
         do t = nst1,nst2

            zw(1) = 1
            zw(2) = 2
            call cpvxz(psiref(cdzpsi(s,t,2)), tempc(1,zw(1)), &
                 cblock(s,2)*cblock(t,2))

            do m=1,nmode
               call mtxxzz(dover(dovmat(m,s),1),tempc(1,zw(1)), &
                    tempc(1,zw(2)),vdim(m,s),cdim(m,s,2), &
                    cndim(m,s,2)*cblock(t,2),dim(m,s))
               hzw = zw(1)
               zw(1) = zw(2)
               zw(2) = hzw
            enddo

            do m=1,nmode
               call mttxzz(dover(dovmat(m,t),2),tempc(1,zw(1)), &
                    tempc(1,zw(2)),block(s)*vdim(m,t),cdim(m,t,2), &
                    cndim(m,t,2),dim(m,t))
               hzw = zw(1)
               zw(1) = zw(2)
               zw(2) = hzw
            enddo

            call vvaxzz(psi(dzpsi(s,t)),tempc(1,zw(1)),x, &
                 block(s)*block(t))
            ovl = ovl + x
         enddo
      enddo

      deallocate(tempc)

      call stop_timer(tid)

      return
      end subroutine d2ccoverlap


!***********************************************************************
!
! subroutine D2CCOUT
!
! Writes the cross-correlation function to the ICROCO file.
!
! Mar 2002, FO
!
!***********************************************************************

      subroutine d2ccout(time,psi)

use global
use griddatmod
use psidef
use crocoflymod
use channels

      implicit none

      real(dop), intent(in)                  :: time
      complex(dop)                           :: ovl
      complex(dop), dimension(*), intent(in) :: psi

!     calculate cross-correlation function
      call d2ccoverlap(ovl,mc(mcpsiref),psi,mc(mcccdover))

!     write result to ICROCO file
      write(icroco,'(1x,g18.9,3(1x,f18.14))')&
            time, dble(ovl), dimag(ovl), abs(ovl)
      call myflush(icroco)

      end subroutine d2ccout

      end module d2crocofly

