! **********************************************************************
! *                                                                    *
! *                       D2EVALUATION                                 *
! *                                                                    *
! * MCTDH module containing (auxiliary) subroutines to evaluate the    *
! * right hand side of the MCTDH working equations.                    *
! *                                                                    *
! * Contains:                                                          *
! * d2subenergy:    Subtracts the energy from the Hamiltonian, i. e.   *
! *                 H|psi> is transformed to (H-E)|psi> where E =      *
! *                 <psi|H|psi>/<psi|psi> (needed for relaxation).     *
! *                                                                    *
!
! *** V6.0 MB ************> D2SUBROUTINE D2SUBENERGY <******************
!                          ~~~~~~~~~~~~~~~~~~~~~~~~~
!
! Subtracts the energy from the Hamiltonian, i. e. H|psi> is transformed
! to (H-E)|psi> where E = <psi|H|psi>/<psi|psi> (needed for relaxation).
! **********************************************************************

      subroutine d2subenergy (psi,dtpsi)

use decimal, only: long,dop
use global
use griddatmod
use runpropmod
use psidef
use maxv,    only: maxdim,maxsta
use xvlib,   only: xvxxdzr
use op1lib,  only: trhxz

      implicit none

      integer(long)                                :: s
      real(dop)                                    :: trace,energy,htr
      complex(dop), dimension(adim), intent(in)    :: psi
      complex(dop), dimension(adim), intent(inout) :: dtpsi

!     if (temperature0 .gt.tiny .or. rlxtime .eq. 'K') then
!         rlxenergy = 0.0_dop
!         return
!      endif

! --- COMPUTE ENERGY ---

      energy = 0.0_dop
      do s=1,nstate
         detot(s) = decorr(s) + deuncorr(s)
         energy = energy + detot(s)
      enddo

! --- COMPUTE TR(PSI) (SUMMED OVER ALL STATES) IF NECESSARY ---

      if (lconst) then
         rlxenergy = 2.0_dop * energy
      else
         trace = 0.0_dop
         do s = 1,nstate
            call trhxz(psi(dzpsi(s,s)),htr,block(s))
            trace = trace + htr
         enddo
         rlxenergy = 2.0_dop * energy/trace
      endif

! --- SUBTRACT ENERGY TIMES PSI FROM TIME DERIVATIVE ---

      call xvxxdzr(rlxenergy,psi,dtpsi,adim)

      return
      end subroutine d2subenergy

      
