!#######################################################################
      module d2mfieldsmod

use decimal,  only: long, dop
use globmemmod
use logdat
use d2tens,   only: cmtens, ctens, rm1
use mmlib,    only: mmxxzz, mmxxzza
use mtlib,    only: mtxxzz, mttxzz
use xvlib,    only: xvxxzzo, xvxxzza, xmxxzzo, xqxxzzo
use op1lib,   only: cpvxz, zerovxz, tranmxz, tranqxz
use timing
use linear,   only: ijk_shape


      implicit none
      private
      public :: d2mfields,d2mfieldsml
      contains

!#######################################################################
! MCTDH-module D2MFIELDS         
!
!#######################################################################

      subroutine d2mfields(psi)

use global
use griddatmod
use operdef
use psidef
use runpropmod
use maxv,   only: maxdim,maxsta,maxkoe


      implicit none

      integer(long)                               :: si,sf,k,kt
      complex(dop), dimension(dgldim), intent(in) :: psi
      integer(long)                               :: tid=0 ! timer ID
      complex(dop), dimension(:),allocatable      :: tmppsi,tmppsi1

      call get_timer(tid, 'Mfields')
      call start_timer(tid)
      
      if (lcdvr.or.ltdvr) then
         message='CDVR and TDVR not yet implemented.'
         call errormsg
      endif


!     LINEAR MEAN-FIELD APPROACH
      if (lmf) then
         allocate(tmppsi(adim))

         do k=1,kzahl(1)
!     Hamiltonian action
            if (kact(k).eq.1) then
               sf=kf(k)
               si=ki(k)
               if (sf .ne. si) then
                  call d2matrizen2(k,sf,si,psi,hteil,mc(mchloch))
               else 
                  call d2matrizen1(k,si,psi,hteil,mc(mchloch))
               endif
            endif
!     pure left action
            if ((kact(k).eq.2) .and. (k.lt.ktwin(k))) then
               kt = ktwin(k)
               sf=kf(k)
               si=ki(kt)
               if (sf .ne. si) then
                  call d2matrizen2l(k,sf,si,psi,hteil,mc(mchloch))
               else 
                  call d2matrizen1l(k,kt,si,psi,hteil,mc(mchloch))
               endif
            endif
!     pure right action: not relevant because of the projector,
!                        hloch matrices would however not differ
!                        from those of a pure left action
!     left-right action
            if ((kact(k).eq.4) .and. (k.lt.ktwin(k))) then
               kt = ktwin(k)
               if (ki(kt) .eq. kf(k)) then
                  si = ki(k)
                  sf = kf(k)
!              right action
                  call ract1(psi,tmppsi,hteil,kt)
!              calculate hloch together with left action
                  if (sf .ne. si) then
                     call d2matrizen2(k,sf,si,tmppsi,hteil,&
                          mc(mchloch))
                  else 
                     call d2matrizen1(k,si,tmppsi,hteil,&
                          mc(mchloch))
                  endif
               endif
            endif
         enddo


!     DIRAC-FRENKEL VARIATIONAL PRINCIPLE
      else
         allocate(tmppsi(adim))
         allocate(tmppsi1(adim))

         call rho2(psi,tmppsi)
         do k=1,kzahl(1)
!     Hamiltonian action
            if (kact(k).eq.1) then
               sf=kf(k)
               si=ki(k)
               if (sf .ne. si) then
                  call d2matrizen2(k,sf,si,tmppsi,hteil,mc(mchloch))
               else 
                  call d2matrizen1(k,si,tmppsi,hteil,mc(mchloch))
               endif

!     pure left action
            else if ((kact(k).eq.2) .and. (k.lt.ktwin(k))) then
               kt = ktwin(k)
               sf=kf(k)
               si=ki(kt)
               if (sf .ne. si) then
                  call d2matrizen2l(k,sf,si,tmppsi,hteil,mc(mchloch))
               else 
                  call d2matrizen1l(k,kt,si,tmppsi,hteil,mc(mchloch))
               endif

!     pure right action: not relevant because of the projector,
!                        hloch matrices would however not differ
!                        from those of a pure left action
!     left-right action
            else if ((kact(k).eq.4) .and. (k.lt.ktwin(k))) then
               kt = ktwin(k)
               si = ki(k)
               sf = kf(k)
!              right action
               call ract2(psi,tmppsi1,hteil,kt)
!              calculate hloch together with left action
               if (sf .ne. si) then
                  call d2matrizen2(k,sf,si,tmppsi1,hteil, &
                       mc(mchloch))
               else 
                  call d2matrizen1(k,si,tmppsi1,hteil, &
                       mc(mchloch))
               endif
            endif
         enddo

      endif

    
      if (nmulpot(1).gt.0) then
         message = 'Mulpot is not yet implemented.'
         call errormsg
      endif

      if (allocated(tmppsi)) deallocate(tmppsi)
      if (allocated(tmppsi1)) deallocate(tmppsi1)

      call stop_timer(tid)

      return
      end subroutine d2mfields

!#######################################################################

      subroutine d2mfieldsml(psi,t,k,nmodes1)

use global
use griddatmod
use operdef
use psidef
use runpropmod
use maxv,   only: maxdim,maxsta,maxkoe


      implicit none

      integer(long),intent(in)                    :: nmodes1,k
      integer(long)                               :: kt
      complex(dop), dimension(dgldim), intent(in) :: psi
      complex(dop), dimension(:),allocatable      :: tmppsi,tmppsi1
      type(wfnode_t),intent(inout)                :: t

      if (lcdvr.or.ltdvr) then
         message='CDVR and TDVR not yet implemented.'
         call errormsg
      endif

!     LINEAR MEAN-FIELD APPROACH
      if (lmf) then
         allocate(tmppsi(adim))

!     Hamiltonian action
         if (kact(k).eq.1) then
            call d2matrizenml(k,psi,t,nmodes1)
         endif
!     pure left action
     !    if ((kact(k).eq.2) .and. (k.lt.ktwin(k))) then
     !       kt = ktwin(k)
     !       call d2matrizen1l(k,kt,si,psi,t,nmodes1)
     !    endif
!     pure right action: not relevant because of the projector,
!                        hloch matrices would however not differ
!                        from those of a pure left action
!     left-right action
         if ((kact(k).eq.4) .and. (k.lt.ktwin(k))) then
            kt = ktwin(k)
!           right action
!           call ract1(psi,tmppsi,hteil,kt)
!           calculate hloch together with left action
!           call d2matrizenml(k,tmppsi,t,nmodes1)
         endif

!     DIRAC-FRENKEL VARIATIONAL PRINCIPLE
      else
         allocate(tmppsi(adim))
         allocate(tmppsi1(adim))

         call rho2(psi,tmppsi)
!     Hamiltonian action
         if (kact(k).eq.1) then
            call d2matrizenml(k,tmppsi,t,nmodes1)

!     pure left action
         else if ((kact(k).eq.2) .and. (k.lt.ktwin(k))) then
            kt = ktwin(k)
    !       call d2matrizen1l(k,kt,si,tmppsi,t,nmodes1)

!     pure right action: not relevant because of the projector,
!                        hloch matrices would however not differ
!                        from those of a pure left action
!     left-right action
         else if ((kact(k).eq.4) .and. (k.lt.ktwin(k))) then
            kt = ktwin(k)
!              right action
!            call ract2(psi,tmppsi1,hteil,kt)
!              calculate hloch together with left action
!            call d2matrizenml(k,tmppsi1,t,nmodes1)
         endif

      endif

    
      if (nmulpot(1).gt.0) then
         message = 'Mulpot is not yet implemented.'
         call errormsg
      endif

      if (allocated(tmppsi)) deallocate(tmppsi)
      if (allocated(tmppsi1)) deallocate(tmppsi1)

      return
      end subroutine d2mfieldsml

!#######################################################################
! D2MATRIZEN1
!
!
! 10/98 AR
!
!
! called by mfields.f
! calls to matvec,hrhomat,matnum
!
! For Hamiltonian terms involving only a single state,
! calculates the mean field matrices. 
!
! This involves multiplying the coefficients by the
! corresponding single particle operator matrix elements.
!
! input:
! hteil, workc are standard arrays, see
!   documentation
! block,dim,vdim,ndim are standard arrays for one state
! psi  : coefficients for relevant state
! k    : number of correlated Hamiltonian term
! output:
! hloch: mean fields for kth Hamiltonian term,
!        note that hloch(dim(m,s),dim(m,s)) is just the 
!        transpose of the corresponding MCDTH array,
!        see matrizen2 for additional information
!
!#######################################################################

      subroutine d2matrizen1(k,s,psi,hteil,hloch)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod
use runpropmod

use op1lib, only: tranqxz

      implicit none
      
      integer(long)                                   :: m,e,hzw,h
      integer(long), intent(in)                       :: k,s
      integer(long), dimension(3)                     :: zw
      complex(dop), dimension(adim), intent(in)       :: psi
      complex(dop), dimension(pmatdim), intent(in)    :: hteil
      complex(dop), dimension(hlochdim), intent(out)  :: hloch
      complex(dop), dimension(maxblock*maxblock,3)    :: workc

!***********************************************************************
!     Contract A-matrix with single-particle hamiltonian matrices:
!     Note that diag(m,k)=.true. means hteil(pmat(m,k))=1 .
!     Note also that only those hloch-matrices are needed for which
!     diag(m,k)= .false. .
!***********************************************************************

      zw(1) = 1
      zw(2) = 2
      zw(3) = 3

      if (nmode .gt. 2) then
!     Treat m=1
         if (diag(1,k)) then
            call ctens(psi(dzpsi(s,s)),workc(1,zw(1)),&
                 vdim(1,s),dim(1,s),ndim(1,s))
         else if (complete(1)) then
!     Prepare next step, i.e. contract mode 1 but do not calculate hloch
            call cmtens(hteil(pmat(1,k)),psi(dzpsi(s,s)),&
                 workc(1,zw(1)),vdim(1,s),dim(1,s),ndim(1,s),&
                 vdim(1,s),dim(1,s),ndim(1,s))

         else 
!     Prepare next step, i.e. contract mode 1
            call cmtens(hteil(pmat(1,k)),psi(dzpsi(s,s)),&
                 workc(1,zw(1)),vdim(1,s),dim(1,s),ndim(1,s),&
                 vdim(1,s),dim(1,s),ndim(1,s))

!     This is not needed if diag(1,k)=.true.
!     Contract mode 2
            if (diag(2,k)) then
               call ctens(psi(dzpsi(s,s)),workc(1,zw(2)),&
                    vdim(2,s),dim(2,s),ndim(2,s))
            else
               call cmtens(hteil(pmat(2,k)),psi(dzpsi(s,s)),&
                    workc(1,zw(2)),vdim(2,s),dim(2,s),ndim(2,s),&
                    vdim(2,s),dim(2,s),ndim(2,s))
            endif
!     Contract modes 3..(nmode-1)
            do m=3,(nmode-1)
               if (diag(m,k)) then
                  call ctens(workc(1,zw(2)),workc(1,zw(3)),&
                       vdim(2,s),dim(m,s),ndim(m,s))
               else
                  call cmtens(hteil(pmat(m,k)),workc(1,zw(2)),&
                       workc(1,zw(3)),vdim(2,s),dim(m,s),ndim(m,s),&
                       vdim(2,s),dim(m,s),ndim(m,s))
               endif
               hzw = zw(2)
               zw(2) = zw(3)
               zw(3) = hzw
            enddo
!     Contract mode nmode, i.e. calculate hloch(phloch(1,k))
!     Take transpose of contracted tensor to leave hloch in same form 
!     as usual MCTDH: <psi^k_f | H | psi^k_i>
            if (diag(nmode,k)) then
               call ctens(workc(1,zw(2)),hloch(phloch(1,k)),&
                    dim(1,s),dim(nmode,s),ndim(nmode,s))
            else
               call cmtens(hteil(pmat(nmode,k)),workc(1,zw(2)),&
                    hloch(phloch(1,k)),dim(1,s),dim(nmode,s),&
                    ndim(nmode,s),dim(1,s),dim(nmode,s),&
                    ndim(nmode,s))
            endif
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
                 call xqxxzzo(koeff(k),hloch(phloch(1,k)),&
                 dim(1,s))

            call tranqxz(hloch(phloch(1,k)),dim(1,s))
         endif

!     Treat modes 2..(nmode-2)
         do m=2,(nmode-2)
            if (diag(m,k)) then
               call ctens(workc(1,zw(1)),workc(1,zw(3)),&
                    vdim(1,s),dim(m,s),ndim(m,s))
               hzw = zw(1)
               zw(1) = zw(3)
               zw(3) = hzw

            else if (.not. complete(m)) then
!     Contract mode m+1, mode m is 'hloch'-mode - no hloch if complete basis
               if (diag(m+1,k)) then
                  call ctens(workc(1,zw(1)),workc(1,zw(2)),&
                       dim(m,s),dim(m+1,s),ndim(m+1,s))
               else
                  call cmtens(hteil(pmat(m+1,k)),workc(1,zw(1)),&
                       workc(1,zw(2)),dim(m,s),dim(m+1,s),&
                       ndim(m+1,s),dim(m,s),dim(m+1,s),ndim(m+1,s))
               endif
!     Contract modes (m+2)..(nmode-1) 
               do e=(m+2),(nmode-1)
                  if (diag(e,k)) then
                     call ctens(workc(1,zw(2)),workc(1,zw(3)),&
                          dim(m,s),dim(e,s),ndim(e,s))
                  else
                     call cmtens(hteil(pmat(e,k)),workc(1,zw(2)),&
                          workc(1,zw(3)),dim(m,s),dim(e,s),&
                          ndim(e,s),dim(m,s),dim(e,s),ndim(e,s))
                  endif
                  hzw = zw(2)
                  zw(2) = zw(3)
                  zw(3) = hzw
               enddo
!     Contract mode nmode, i.e. calculate hloch(phloch(m,k))
               if (diag(nmode,k)) then
                  call ctens(workc(1,zw(2)),hloch(phloch(m,k)),&
                       dim(m,s),dim(nmode,s),ndim(nmode,s))
               else
                  call cmtens(hteil(pmat(nmode,k)),workc(1,zw(2)),&
                       hloch(phloch(m,k)),dim(m,s),dim(nmode,s),&
                       ndim(nmode,s),dim(m,s),dim(nmode,s),&
                       ndim(nmode,s))
               endif
               h=hamilton(fkoe,k)
               if (htmtype(h) .ne. 1)&
                    call xqxxzzo(koeff(k),hloch(phloch(m,k)),&
                         dim(m,s))
               call tranqxz(hloch(phloch(m,k)),dim(m,s))
!     Prepare next step, i.e. contract mode m
               call cmtens(hteil(pmat(m,k)),workc(1,zw(1)),&
                    workc(1,zw(3)),vdim(1,s),dim(m,s),&
                    ndim(m,s),vdim(1,s),dim(m,s),ndim(m,s))
               hzw = zw(1)
               zw(1) = zw(3)
               zw(3) = hzw
            endif
         enddo
!     Calculate (nmode-1)-th hloch matrix
         if (.not. diag(nmode-1,k) .and. .not. complete(nmode-1)) then
            if (diag(nmode,k)) then
               call ctens(workc(1,zw(1)),hloch(phloch(nmode-1,k)),&
                    dim(nmode-1,s),dim(nmode,s),ndim(nmode,s))
            else
               call cmtens(hteil(pmat(nmode,k)),workc(1,zw(1)),&
                    hloch(phloch(nmode-1,k)),dim(nmode-1,s),&
                    dim(nmode,s),ndim(nmode,s),dim(nmode-1,s),&
                    dim(nmode,s),ndim(nmode,s))
            endif         
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
                 call xqxxzzo(koeff(k),hloch(phloch(nmode-1,k)),&
                 dim(nmode-1,s))
            call tranqxz(hloch(phloch(nmode-1,k)),dim(nmode-1,s))
         endif
!     Calculate nmode-th hloch matrix
         if (.not. diag(nmode,k) .and. .not. complete(nmode)) then
            if (diag(nmode-1,k)) then
               call ctens(workc(1,zw(1)),hloch(phloch(nmode,k)),&
                    vdim(1,s),dim(nmode-1,s),ndim(nmode-1,s))
            else
               call cmtens(hteil(pmat(nmode-1,k)),workc(1,zw(1)),&
                    hloch(phloch(nmode,k)),vdim(1,s),dim(nmode-1,s),&
                    ndim(nmode-1,s),vdim(1,s),dim(nmode-1,s),&
                    ndim(nmode-1,s))
            endif
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
                 call xqxxzzo(koeff(k),hloch(phloch(nmode,k)),&
                 dim(nmode,s))
            call tranqxz(hloch(phloch(nmode,k)),dim(nmode,s))
         endif

      endif


!     Nmode = 2
      if (nmode .eq. 2) then
!     Calculate first hloch matrix
         if (.not. diag(1,k) .and. .not. complete(1)) then
            if (diag(2,k)) then
               call ctens(psi(dzpsi(s,s)),hloch(phloch(1,k)),&
                    vdim(2,s),dim(2,s),ndim(2,s))
            else
               call cmtens(hteil(pmat(2,k)),psi(dzpsi(s,s)),&
                    hloch(phloch(1,k)),vdim(2,s),dim(2,s),&
                    ndim(2,s),vdim(2,s),dim(2,s),ndim(2,s))               
            endif
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
               call xqxxzzo(koeff(k),hloch(phloch(1,k)),dim(1,s))
            call tranqxz(hloch(phloch(1,k)),dim(1,s))
         endif
!     Calculate second hloch matrix
         if (.not. diag(2,k) .and. .not. complete(2)) then
            if (diag(1,k)) then
               call ctens(psi(dzpsi(s,s)),hloch(phloch(2,k)),&
                    vdim(1,s),dim(1,s),ndim(1,s))
            else
               call cmtens(hteil(pmat(1,k)),psi(dzpsi(s,s)),&
                    hloch(phloch(2,k)),vdim(1,s),dim(1,s),&
                    ndim(1,s),vdim(1,s),dim(1,s),ndim(1,s))               
            endif
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
               call xqxxzzo(koeff(k),hloch(phloch(2,k)),dim(2,s))
            call tranqxz(hloch(phloch(2,k)),dim(2,s))
         endif
      endif

!     Nmode = 1
      if (nmode .eq. 1 .and. .not. complete(1)) then
         if (.not. diag(1,k)) then
            call ctens(psi(dzpsi(s,s)),hloch(phloch(1,k)),&
                 vdim(1,s),dim(1,s),ndim(1,s))
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
               call xqxxzzo(koeff(k),hloch(phloch(1,k)),dim(1,s))
            call tranqxz(hloch(phloch(1,k)),dim(1,s))
         endif
      endif

      return 
      end subroutine d2matrizen1


!#######################################################################
! D2MATRIZEN2
!
!
!
! 10/98 AR
!
!
! called by funk,funkr
! calls to usmatvec,usrhomat,matnum
!
! calculates the mean field matrices involving hamiltonian coefficients
!  that connect two states. 
!
! This involves multiplying the coefficients of the state s1 by the
! corresponding single particle operator matrix elements.
!
! input:
! sf: state on LHS of mean field matrix, final state
! si: state on RHS of mean field matrix, initial state
! psi: coefficients for all states 
! hteil, workc are standard arrays, see documentation
! k: which correlated Hamiltonian coefficient passed from calling
! routine
! output:
! hloch: mean field for kth Hamiltonian term.
!        note that hloch(dim(m,si),dim(m,sf)) is just the 
!        transpose of the corresponding MCDTH array.
!
! V6.0 GW
!#######################################################################

      subroutine d2matrizen2(k,sf,si,psi,hteil,hloch)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod

      implicit none
      
      integer(long)                                   :: m,e,hzw,h
      integer(long), intent(in)                       :: k,si,sf
      integer(long), dimension(3)                     :: zw
      complex(dop), dimension(adim), intent(in)       :: psi
      complex(dop), dimension(pmatdim), intent(in)    :: hteil
      complex(dop), dimension(hlochdim), intent(out)  :: hloch
      complex(dop), dimension(maxblock*maxblock,3)    :: workc
      complex(dop), dimension(maxspf*maxspf)          :: tmphloch


!***********************************************************************
!     Contract A-matrix with single-particle hamiltonian matrices:
!     Note that for si .ne. sf always diag(m,k)=.false.
!***********************************************************************

      zw(1) = 1
      zw(2) = 2
      zw(3) = 3
      if (nmode .gt. 2) then
!     Treat m=1
!     Prepare next step, i.e. contract mode 1
         call cmtens(hteil(pmat(1,k)),psi(dzpsi(si,sf)),&
              workc(1,zw(1)),vdim(1,si),dim(1,si),ndim(1,si),&
              vdim(1,sf),dim(1,sf),ndim(1,sf))
!     This is not needed if diag(1,k)=.true.
!     Contract mode 2
         call cmtens(hteil(pmat(2,k)),psi(dzpsi(si,sf)),&
              workc(1,zw(2)),vdim(2,si),dim(2,si),ndim(2,si),&
              vdim(2,sf),dim(2,sf),ndim(2,sf))
!     Contract modes 3..(nmode-1)
         do m=3,(nmode-1)
            call cmtens(hteil(pmat(m,k)),workc(1,zw(2)),&
                 workc(1,zw(3)),vdim(2,si),dim(m,si),ndim(m,si),&
                 vdim(2,sf),dim(m,sf),ndim(m,sf))
            hzw = zw(2)
            zw(2) = zw(3)
            zw(3) = hzw
         enddo
!     Contract mode nmode, i.e. calculate hloch(phloch(1,k))
         call cmtens(hteil(pmat(nmode,k)),workc(1,zw(2)),&
              tmphloch,dim(1,si),dim(nmode,si),&
              ndim(nmode,si),dim(1,sf),dim(nmode,sf),&
              ndim(nmode,sf))
         h=hamilton(fkoe,k)
         if (htmtype(h) .ne. 1)&
              call xmxxzzo(koeff(k),tmphloch,dim(1,sf),&
              dim(1,si))
         call tranmxz(tmphloch,hloch(phloch(1,k)),dim(1,si),dim(1,sf))

!     Treat modes 2..(nmode-2)
         do m=2,(nmode-2)
!     Contract mode m+1, mode m is 'hloch'-mode
            call cmtens(hteil(pmat(m+1,k)),workc(1,zw(1)),&
                 workc(1,zw(2)),dim(m,si),dim(m+1,si),&
                 ndim(m+1,si),dim(m,sf),dim(m+1,sf),ndim(m+1,sf))
!     Contract modes (m+2)..(nmode-1) 
            do e=(m+2),(nmode-1)
               call cmtens(hteil(pmat(e,k)),workc(1,zw(2)),&
                    workc(1,zw(3)),dim(m,si),dim(e,si),&
                    ndim(e,si),dim(m,sf),dim(e,sf),ndim(e,sf))
               hzw = zw(2)
               zw(2) = zw(3)
               zw(3) = hzw
            enddo
!     Contract mode nmode, i.e. calculate hloch(phloch(m,k))
            call cmtens(hteil(pmat(nmode,k)),workc(1,zw(2)),&
                 tmphloch,dim(m,si),dim(nmode,si),&
                 ndim(nmode,si),dim(m,sf),dim(nmode,sf),&
                 ndim(nmode,sf))
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
                 call xmxxzzo(koeff(k),tmphloch,dim(m,sf),&
                      dim(m,si))
            call tranmxz(tmphloch,hloch(phloch(m,k)),dim(m,si),dim(m,sf))
!     Prepare next step, i.e. contract mode m
            call cmtens(hteil(pmat(m,k)),workc(1,zw(1)),&
                 workc(1,zw(3)),vdim(1,si),dim(m,si),&
                 ndim(m,si),vdim(1,sf),dim(m,sf),ndim(m,sf))
            hzw = zw(1)
            zw(1) = zw(3)
            zw(3) = hzw
         enddo
!     Calculate (nmode-1)-th hloch matrix
         call cmtens(hteil(pmat(nmode,k)),workc(1,zw(1)),&
              tmphloch,dim(nmode-1,si),&
              dim(nmode,si),ndim(nmode,si),dim(nmode-1,sf),&
              dim(nmode,sf),ndim(nmode,sf))
         h=hamilton(fkoe,k)
         if (htmtype(h) .ne. 1)&
              call xmxxzzo(koeff(k),tmphloch,dim(nmode-1,sf),dim(nmode-1,si))
         call tranmxz(tmphloch,hloch(phloch(nmode-1,k)),dim(nmode-1,si),dim(nmode-1,sf))
!     Calculate nmode-th hloch matrix
         call cmtens(hteil(pmat(nmode-1,k)),workc(1,zw(1)),&
              tmphloch,vdim(1,si),dim(nmode-1,si),&
              ndim(nmode-1,si),vdim(1,sf),dim(nmode-1,sf),&
              ndim(nmode-1,sf))
         h=hamilton(fkoe,k)
         if (htmtype(h) .ne. 1)&
              call xmxxzzo(koeff(k),tmphloch,dim(nmode,sf),dim(nmode,si))
         call tranmxz(tmphloch,hloch(phloch(nmode,k)),dim(nmode,si),dim(nmode,sf))

      endif

!     Nmode = 2
      if (nmode .eq. 2) then
!     Calculate first hloch matrix
         call cmtens(hteil(pmat(2,k)),psi(dzpsi(si,sf)),&
              tmphloch,vdim(2,si),dim(2,si),&
              ndim(2,si),vdim(2,sf),dim(2,sf),ndim(2,sf))               
         h=hamilton(fkoe,k)
         if (htmtype(h) .ne. 1)&
              call xmxxzzo(koeff(k),tmphloch,dim(1,sf),dim(1,si))
         call tranmxz(tmphloch,hloch(phloch(1,k)),dim(1,si),dim(1,sf))
!     Calculate second hloch matrix
         call cmtens(hteil(pmat(1,k)),psi(dzpsi(si,sf)),&
              tmphloch,vdim(1,si),dim(1,si),&
              ndim(1,si),vdim(1,sf),dim(1,sf),ndim(1,sf))               
         h=hamilton(fkoe,k)
         if (htmtype(h) .ne. 1)&
              call xmxxzzo(koeff(k),tmphloch,dim(2,sf),dim(2,si))
         call tranmxz(tmphloch,hloch(phloch(2,k)),dim(2,si),dim(2,sf))
      endif
         

!     Nmode = 1
      if (nmode .eq. 1) then
         !message = 'Nmode=1 not yet implemented.'
         !call errormsg
         call rm1(psi(dzpsi(si,sf)),&
              tmphloch,vdim(1,si),dim(1,si),&
              ndim(1,si),vdim(1,sf),dim(1,sf),ndim(1,sf))               
         h=hamilton(fkoe,k)
         if (htmtype(h) .ne. 1)&
            call xmxxzzo(koeff(k),tmphloch,dim(1,sf),dim(1,si))
         call tranmxz(tmphloch,hloch(phloch(1,k)),dim(1,si),dim(1,sf))
      endif


      return 
      end subroutine d2matrizen2


!#######################################################################
! D2MATRIZEN1L
!
!
! 04/00 AR
!
! similar as D2MATRIZEN1, but hloch is calculated for the pure-left acting
! dissipative operators
!#######################################################################

      subroutine d2matrizen1l(k,kt,s,psi,hteil,hloch)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod

      implicit none
      
      integer(long)                                  :: m,e,hzw,h
      integer(long), intent(in)                      :: k,kt,s
      integer(long), dimension(3)                    :: zw
      complex(dop), dimension(adim), intent(in)      :: psi
      complex(dop), dimension(pmatdim), intent(in)   :: hteil
      complex(dop), dimension(hlochdim), intent(out) :: hloch
      complex(dop), dimension(maxblock*maxblock,3)   :: workc


!***********************************************************************
!     Contract A-matrix with single-particle hamiltonian matrices:
!     Note that diag(m,k)=.true. means hteil(pmat(m,k))=1 .
!     Note also that only those hloch-matrices are needed for which
!     diag(m,k)= .false. .
!***********************************************************************

      zw(1) = 1
      zw(2) = 2
      zw(3) = 3

!     initialize auxiliary diagh array
      do m=1,nmode
         diagh(m) = diag(m,k) .and. diag(m,kt)
      enddo

      if (nmode .gt. 2) then
!     Treat m=1
         if (diagh(1)) then
            call ctens(psi(dzpsi(s,s)),workc(1,zw(1)),&
                 vdim(1,s),dim(1,s),ndim(1,s))
         else
!     Prepare next step, i.e. contract mode 1
            call cmtens(hteil(pmat2(1,k)),psi(dzpsi(s,s)),&
                 workc(1,zw(1)),vdim(1,s),dim(1,s),ndim(1,s),&
                 vdim(1,s),dim(1,s),ndim(1,s))
!     This is not needed if diag(1,k)=.true.
!     Contract mode 2
            if (diagh(2)) then
               call ctens(psi(dzpsi(s,s)),workc(1,zw(2)),&
                    vdim(2,s),dim(2,s),ndim(2,s))
            else
               call cmtens(hteil(pmat2(2,k)),psi(dzpsi(s,s)),&
                    workc(1,zw(2)),vdim(2,s),dim(2,s),ndim(2,s),&
                    vdim(2,s),dim(2,s),ndim(2,s))
            endif
!     Contract modes 3..(nmode-1)
            do m=3,(nmode-1)
               if (diagh(m)) then
                  call ctens(workc(1,zw(2)),workc(1,zw(3)),&
                       vdim(2,s),dim(m,s),ndim(m,s))
               else
                  call cmtens(hteil(pmat2(m,k)),workc(1,zw(2)),&
                       workc(1,zw(3)),vdim(2,s),dim(m,s),ndim(m,s),&
                       vdim(2,s),dim(m,s),ndim(m,s))
               endif
               hzw = zw(2)
               zw(2) = zw(3)
               zw(3) = hzw
            enddo
!     Contract mode nmode, i.e. calculate hloch(phloch(1,k))
            if (diagh(nmode)) then
               call ctens(workc(1,zw(2)),hloch(phloch(1,k)),&
                    dim(1,s),dim(nmode,s),ndim(nmode,s))
            else
               call cmtens(hteil(pmat2(nmode,k)),workc(1,zw(2)),&
                    hloch(phloch(1,k)),dim(1,s),dim(nmode,s),&
                    ndim(nmode,s),dim(1,s),dim(nmode,s),&
                    ndim(nmode,s))
            endif
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
                 call xqxxzzo(koeff(k),hloch(phloch(1,k)),dim(1,s))
            call tranqxz(hloch(phloch(1,k)),dim(1,s))
         endif

!     Treat modes 2..(nmode-2)
         do m=2,(nmode-2)
            if (diagh(m)) then
               call ctens(workc(1,zw(1)),workc(1,zw(3)),&
                    vdim(1,s),dim(m,s),ndim(m,s))
               hzw = zw(1)
               zw(1) = zw(3)
               zw(3) = hzw
            else
!     Contract mode m+1, mode m is 'hloch'-mode
               if (diagh(m+1)) then
                  call ctens(workc(1,zw(1)),workc(1,zw(2)),&
                       dim(m,s),dim(m+1,s),ndim(m+1,s))
               else
                  call cmtens(hteil(pmat2(m+1,k)),workc(1,zw(1)),&
                       workc(1,zw(2)),dim(m,s),dim(m+1,s),&
                       ndim(m+1,s),dim(m,s),dim(m+1,s),ndim(m+1,s))
               endif
!     Contract modes (m+2)..(nmode-1) 
               do e=(m+2),(nmode-1)
                  if (diagh(e)) then
                     call ctens(workc(1,zw(2)),workc(1,zw(3)),&
                          dim(m,s),dim(e,s),ndim(e,s))
                  else
                     call cmtens(hteil(pmat2(e,k)),workc(1,zw(2)),&
                          workc(1,zw(3)),dim(m,s),dim(e,s),&
                          ndim(e,s),dim(m,s),dim(e,s),ndim(e,s))
                  endif
                  hzw = zw(2)
                  zw(2) = zw(3)
                  zw(3) = hzw
               enddo
!     Contract mode nmode, i.e. calculate hloch(phloch(m,k))
               if (diagh(nmode)) then
                  call ctens(workc(1,zw(2)),hloch(phloch(m,k)),&
                       dim(m,s),dim(nmode,s),ndim(nmode,s))
               else
                  call cmtens(hteil(pmat2(nmode,k)),workc(1,zw(2)),&
                       hloch(phloch(m,k)),dim(m,s),dim(nmode,s),&
                       ndim(nmode,s),dim(m,s),dim(nmode,s),&
                       ndim(nmode,s))
               endif
               h=hamilton(fkoe,k)
               if (htmtype(h) .ne. 1)&
                    call xqxxzzo(koeff(k),hloch(phloch(m,k)),dim(m,s))
               call tranqxz(hloch(phloch(m,k)),dim(m,s))
!     Prepare next step, i.e. contract mode m
               call cmtens(hteil(pmat2(m,k)),workc(1,zw(1)),&
                    workc(1,zw(3)),vdim(1,s),dim(m,s),&
                    ndim(m,s),vdim(1,s),dim(m,s),ndim(m,s))
               hzw = zw(1)
               zw(1) = zw(3)
               zw(3) = hzw
            endif
         enddo
!     Calculate (nmode-1)-th hloch matrix
         if (.not. diagh(nmode-1)) then
            if (diagh(nmode)) then
               call ctens(workc(1,zw(1)),hloch(phloch(nmode-1,k)),&
                    dim(nmode-1,s),dim(nmode,s),ndim(nmode,s))
            else
               call cmtens(hteil(pmat2(nmode,k)),workc(1,zw(1)),&
                    hloch(phloch(nmode-1,k)),dim(nmode-1,s),&
                    dim(nmode,s),ndim(nmode,s),dim(nmode-1,s),&
                    dim(nmode,s),ndim(nmode,s))
            endif         
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
                 call xqxxzzo(koeff(k),hloch(phloch(nmode-1,k)),&
                 dim(nmode-1,s))
            call tranqxz(hloch(phloch(nmode-1,k)),dim(nmode-1,s))
         endif
!     Calculate nmode-th hloch matrix
         if (.not. diagh(nmode)) then
            if (diagh(nmode-1)) then
               call ctens(workc(1,zw(1)),hloch(phloch(nmode,k)),&
                    vdim(1,s),dim(nmode-1,s),ndim(nmode-1,s))
            else
               call cmtens(hteil(pmat2(nmode-1,k)),workc(1,zw(1)),&
                    hloch(phloch(nmode,k)),vdim(1,s),dim(nmode-1,s),&
                    ndim(nmode-1,s),vdim(1,s),dim(nmode-1,s),&
                    ndim(nmode-1,s))
            endif
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
                 call xqxxzzo(koeff(k),hloch(phloch(nmode,k)),&
                 dim(nmode,s))
            call tranqxz(hloch(phloch(nmode,k)),dim(nmode,s))
         endif

      endif


!     Nmode = 2
      if (nmode .eq. 2) then
!     Calculate first hloch matrix
         if (.not. diagh(1)) then
            if (diagh(2)) then
               call ctens(psi(dzpsi(s,s)),hloch(phloch(1,k)),&
                    vdim(2,s),dim(2,s),ndim(2,s))
            else
               call cmtens(hteil(pmat2(2,k)),psi(dzpsi(s,s)),&
                    hloch(phloch(1,k)),vdim(2,s),dim(2,s),&
                    ndim(2,s),vdim(2,s),dim(2,s),ndim(2,s))
            endif
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
               call xqxxzzo(koeff(k),hloch(phloch(1,k)),dim(1,s))
            call tranqxz(hloch(phloch(1,k)),dim(1,s))
         endif
!     Calculate second hloch matrix
         if (.not. diagh(2)) then
            if (diagh(1)) then
               call ctens(psi(dzpsi(s,s)),hloch(phloch(2,k)),&
                    vdim(1,s),dim(1,s),ndim(1,s))
            else
               call cmtens(hteil(pmat2(1,k)),psi(dzpsi(s,s)),&
                    hloch(phloch(2,k)),vdim(1,s),dim(1,s),&
                    ndim(1,s),vdim(1,s),dim(1,s),ndim(1,s))              
            endif
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
               call xqxxzzo(koeff(k),hloch(phloch(2,k)),dim(2,s))
            call tranqxz(hloch(phloch(2,k)),dim(2,s))
         endif
      endif


!     Nmode = 1
      if (nmode .eq. 1) then
       !  message = 'Nmode=1 not yet implemented.'
       !  call errormsg
         if (.not. diagh(1)) then
            call ctens(psi(dzpsi(s,s)),hloch(phloch(1,k)),&
                 vdim(1,s),dim(1,s),ndim(1,s))
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
               call xqxxzzo(koeff(k),hloch(phloch(1,k)),dim(1,s))
            call tranqxz(hloch(phloch(1,k)),dim(1,s))
         endif
      endif



      return 
      end subroutine d2matrizen1l


!#######################################################################
! D2MATRIZEN1
!
!
! 10/98 AR
!
!
! called by mfields.f
! calls to matvec,hrhomat,matnum
!
! For Hamiltonian terms involving only a single state,
! calculates the mean field matrices. 
!
! This involves multiplying the coefficients by the
! corresponding single particle operator matrix elements.
!
! input:
! hteil, workc are standard arrays, see
!   documentation
! block,dim,vdim,ndim are standard arrays for one state
! psi  : coefficients for relevant state
! k    : number of correlated Hamiltonian term
! output:
! hloch: mean fields for kth Hamiltonian term,
!        note that hloch(dim(m,s),dim(m,s)) is just the 
!        transpose of the corresponding MCDTH array,
!        see matrizen2 for additional information
!
!
! translation of original Rabb routine for multi-layer structure
! ijk(1),ijk(2),ijk(3) take place of vdim,dim,ndim as tensor 
! dimensions for top layer coefficients. 
! vdim1(m) and dim1(m) store mode values.
!#######################################################################

      subroutine d2matrizenml(k,psi,t,nmode1)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod
use runpropmod

use op1lib, only: tranqxz

      implicit none
      
      integer(long)                                   :: m,e,hzw,h
      integer(long), intent(in)                       :: k,nmode1
      integer(long), dimension(3)                     :: zw
      complex(dop), dimension(adim), intent(in)       :: psi
      complex(dop), dimension(maxblock*maxblock,3)    :: workc

      integer(long), dimension(nmode1) :: vdim1,dim1
      integer(long), dimension(3) :: ijk
      type(wfnode_t),intent(in)  :: t
      type(wfnode_t),pointer     :: childf
      type(hterm_tp),pointer     :: hk(:)
      complex(dop),pointer       :: mfcurrent(:,:)

      logical(kind=4), dimension(nmode1) :: completeml

      completeml(:) = .false.
      do m=1,nmode1
         childf => t%submodes(m)%p
         if (childf%isprimitive)then
            if (complete(childf%m)) completeml(m) = .true.
         endif
      enddo

!***********************************************************************
!     Contract A-matrix with single-particle hamiltonian matrices:
!     Note that diag(m,k)=.true. means hteil(pmat(m,k))=1 .
!     Note also that only those hloch-matrices are needed for which
!     diag(m,k)= .false. .
!***********************************************************************

      zw(1) = 1
      zw(2) = 2
      zw(3) = 3
      hk => t%hterms(k,:)

      if (nmode1 .gt. 2) then
!     Treat m=1
         m=1
         ijk = ijk_shape(m,t%ndim)
         vdim1(1)=ijk(1)
         dim1(1)=ijk(2)

         if (t%isunit(k,1)) then
            call ctens(psi,workc(1,zw(1)),ijk(1),ijk(2),ijk(3))

         else if (completeml(1)) then
!     Prepare next step, i.e. contract mode 1 but do not calculate hloch
            call cmtens(hk(m)%p%matrix,psi,workc(1,zw(1)), &
                 ijk(1),ijk(2),ijk(3),ijk(1),ijk(2),ijk(3))

         else 
!     Prepare next step, i.e. contract mode 1
            call cmtens(hk(m)%p%matrix,psi,workc(1,zw(1)), &
                 ijk(1),ijk(2),ijk(3),ijk(1),ijk(2),ijk(3))

!     This is not needed if diag(1,k)=.true.
!     Contract mode 2
            m=2
            ijk = ijk_shape(m,t%ndim)
            vdim1(2) = ijk(1)
            dim1(2) = ijk(2)
            if (t%isunit(k,2)) then
               call ctens(psi,workc(1,zw(2)),ijk(1),ijk(2),ijk(3))
            else
               call cmtens(hk(m)%p%matrix,psi,workc(1,zw(2)), &
                    ijk(1),ijk(2),ijk(3),ijk(1),ijk(2),ijk(3))
            endif
!     Contract modes 3..(nmode-1)
            do m=3,(nmode1-1)
               ijk = ijk_shape(m,t%ndim)
               if (t%isunit(k,m)) then
                  call ctens(workc(1,zw(2)),workc(1,zw(3)),vdim1(2),ijk(2),ijk(3))
               else
                  call cmtens(hk(m)%p%matrix,workc(1,zw(2)),workc(1,zw(3)), &
                       vdim1(2),ijk(2),ijk(3),vdim1(2),ijk(2),ijk(3))
               endif
               hzw = zw(2)
               zw(2) = zw(3)
               zw(3) = hzw
            enddo
!     Contract mode nmode, i.e. calculate hloch(phloch(1,k))
!     Take transpose of contracted tensor to leave hloch in same form 
!     as usual MCTDH: <psi^k_f | H | psi^k_i>
            m=nmode1
            ijk = ijk_shape(m,t%ndim)
            mfcurrent => t%mfields(k,1)%p%matrix

            if (t%isunit(k,nmode1)) then
               call ctens(workc(1,zw(2)),mfcurrent,dim1(1),ijk(2),ijk(3))
            else
               call cmtens(hk(m)%p%matrix,workc(1,zw(2)),mfcurrent, &
                    dim1(1),ijk(2),ijk(3),dim1(1),ijk(2),ijk(3))
            endif
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
                 call xqxxzzo(koeff(k),mfcurrent,dim1(1))

            call tranqxz(mfcurrent,dim1(1))
         endif

!     Treat modes 2..(nmode-2)
         do m=2,(nmode1-2)
            ijk = ijk_shape(m,t%ndim)
            vdim1(m) = ijk(1)
            dim1(m)=ijk(2)
            if (t%isunit(k,m)) then
               call ctens(workc(1,zw(1)),workc(1,zw(3)),vdim1(1),ijk(2),ijk(3))
               hzw = zw(1)
               zw(1) = zw(3)
               zw(3) = hzw

            else if (.not. completeml(m)) then
!     Contract mode m+1, mode m is 'hloch'-mode - no hloch if complete basis
               ijk = ijk_shape(m+1,t%ndim)
               if (t%isunit(k,m+1)) then
                  call ctens(workc(1,zw(1)),workc(1,zw(2)),dim1(m),ijk(2),ijk(3))
               else
                  call cmtens(hk(m+1)%p%matrix,workc(1,zw(1)),workc(1,zw(2)), &
                       dim1(m),ijk(2),ijk(3),dim1(m),ijk(2),ijk(3))
               endif
!     Contract modes (m+2)..(nmode-1) 
               do e=(m+2),(nmode1-1)
                  ijk = ijk_shape(e,t%ndim)
                  if (t%isunit(k,e)) then
                     call ctens(workc(1,zw(2)),workc(1,zw(3)),dim1(m),ijk(2),ijk(3))
                  else
                     call cmtens(hk(e)%p%matrix,workc(1,zw(2)),workc(1,zw(3)), &
                          dim1(m),ijk(2),ijk(3),dim1(m),ijk(2),ijk(3))
                  endif
                  hzw = zw(2)
                  zw(2) = zw(3)
                  zw(3) = hzw
               enddo
!     Contract mode nmode, i.e. calculate hloch(phloch(m,k))
               mfcurrent => t%mfields(k,m)%p%matrix
               ijk = ijk_shape(nmode1,t%ndim)
               if (t%isunit(k,nmode1)) then
                  call ctens(workc(1,zw(2)),mfcurrent,dim1(m),ijk(2),ijk(3))
               else
                  call cmtens(hk(nmode1)%p%matrix,workc(1,zw(2)),mfcurrent, &
                       dim1(m),ijk(2),ijk(3),dim1(m),ijk(2),ijk(3))
               endif
               h=hamilton(fkoe,k)
               if (htmtype(h) .ne. 1) call xqxxzzo(koeff(k),mfcurrent,dim1(m))
               call tranqxz(mfcurrent,dim1(m))
!     Prepare next step, i.e. contract mode m
               ijk = ijk_shape(m,t%ndim)
               call cmtens(hk(m)%p%matrix,workc(1,zw(1)),workc(1,zw(3)), &
                    vdim1(1),ijk(2),ijk(3),vdim1(1),ijk(2),ijk(3))
               hzw = zw(1)
               zw(1) = zw(3)
               zw(3) = hzw
            endif
         enddo
!     Calculate (nmode-1)-th hloch matrix
         m=nmode1-1
         ijk = ijk_shape(m,t%ndim)
         vdim1(m) = ijk(1)
         dim1(m) = ijk(2)
         mfcurrent => t%mfields(k,m)%p%matrix

         m=nmode1
         ijk = ijk_shape(m,t%ndim)
         if (.not. t%isunit(k,nmode1-1) .and. .not. completeml(nmode1-1)) then
            if (t%isunit(k,nmode1)) then
               call ctens(workc(1,zw(1)),mfcurrent,dim1(nmode1-1),ijk(2),ijk(3))
            else
               call cmtens(hk(m)%p%matrix,workc(1,zw(1)),mfcurrent, &
                    dim1(nmode1-1),ijk(2),ijk(3),dim1(nmode1-1),ijk(2),ijk(3))
            endif         
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
                 call xqxxzzo(koeff(k),mfcurrent,dim1(nmode1-1))
            call tranqxz(mfcurrent,dim1(nmode1-1))
         endif
!     Calculate nmode-th hloch matrix
         m=nmode1
         mfcurrent => t%mfields(k,m)%p%matrix
         m=nmode1-1
         ijk = ijk_shape(m,t%ndim)
         vdim1(m) = ijk(1)
         dim1(m) = ijk(2)
         if (.not. t%isunit(k,nmode1) .and. .not. completeml(nmode1)) then
            if (t%isunit(k,nmode1-1)) then
               call ctens(workc(1,zw(1)),mfcurrent,vdim1(1),ijk(2),ijk(3))
            else
               call cmtens(hk(m)%p%matrix,workc(1,zw(1)),mfcurrent, &
                    vdim1(1),ijk(2),ijk(3),vdim1(1),ijk(2),ijk(3))
            endif
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1) call xqxxzzo(koeff(k),mfcurrent,dim1(nmode1))
            call tranqxz(mfcurrent,dim1(nmode1))
         endif

      endif

!     Nmode = 2
      if (nmode1 .eq. 2) then
         m=1
         mfcurrent => t%mfields(k,m)%p%matrix
         ijk = ijk_shape(m,t%ndim)
         dim1(m) = ijk(2)
         m=2
         ijk = ijk_shape(m,t%ndim)
!     Calculate first hloch matrix
         if (.not. t%isunit(k,1) .and. .not. completeml(1)) then
            if (t%isunit(k,2)) then
               call ctens(psi,mfcurrent,ijk(1),ijk(2),ijk(3))
            else
               call cmtens(hk(m)%p%matrix,psi,mfcurrent, &
                    ijk(1),ijk(2),ijk(3),ijk(1),ijk(2),ijk(3))
            endif
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1) call xqxxzzo(koeff(k),mfcurrent,dim1(1))
            call tranqxz(mfcurrent,dim1(1))
         endif
!     Calculate second hloch matrix
         m=2
         mfcurrent => t%mfields(k,m)%p%matrix
         ijk = ijk_shape(m,t%ndim)
         dim1(m) = ijk(2)
         m=1
         ijk = ijk_shape(m,t%ndim)
         if (.not. t%isunit(k,2) .and. .not. completeml(2)) then
            if (t%isunit(k,1)) then
               call ctens(psi,mfcurrent,ijk(1),ijk(2),ijk(3))
            else
               call cmtens(hk(m)%p%matrix,psi,mfcurrent, &
                    ijk(1),ijk(2),ijk(3),ijk(1),ijk(2),ijk(3))
            endif
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1) call xqxxzzo(koeff(k),mfcurrent,dim1(2))
            call tranqxz(mfcurrent,dim1(2))
         endif
      endif

!     Nmode = 1
      if (nmode1 .eq. 1 .and. .not. completeml(1)) then
         m=1
         mfcurrent => t%mfields(k,m)%p%matrix
         ijk = ijk_shape(m,t%ndim)
         if (.not. t%isunit(k,1)) then
            call ctens(psi,mfcurrent,ijk(1),ijk(2),ijk(3))
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1) call xqxxzzo(koeff(k),mfcurrent,ijk(1))
            call tranqxz(mfcurrent,ijk(2))
         endif
      endif

      return 
      end subroutine d2matrizenml


!#######################################################################
! D2MATRIZEN2L
!
!
! 04/00 AR
!
! similar as D2MATRIZEN2, but hloch is calculated for the pure-left acting
! dissipative operators
!#######################################################################

      subroutine d2matrizen2l(k,sf,si,psi,hteil,hloch)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod

      implicit none
      
      integer(long)                                   :: m,e,hzw,h
      integer(long), intent(in)                       :: k,si,sf
      integer(long), dimension(3)                     :: zw
      complex(dop), dimension(adim), intent(in)       :: psi
      complex(dop), dimension(pmatdim), intent(in)    :: hteil
      complex(dop), dimension(hlochdim), intent(out)  :: hloch
      complex(dop), dimension(maxblock*maxblock,3)    :: workc
      complex(dop), dimension(maxspf*maxspf)          :: tmphloch

!***********************************************************************
!     Contract A-matrix with single-particle hamiltonian matrices:
!     Note that for si .ne. sf always diag(m,k)=.false.
!***********************************************************************

      zw(1) = 1
      zw(2) = 2
      zw(3) = 3
      if (nmode .gt. 2) then
!     Treat m=1
!     Prepare next step, i.e. contract mode 1
         call cmtens(hteil(pmat2(1,k)),psi(dzpsi(si,sf)),&
              workc(1,zw(1)),vdim(1,si),dim(1,si),ndim(1,si),&
              vdim(1,sf),dim(1,sf),ndim(1,sf))
!     This is not needed if diag(1,k)=.true.
!     Contract mode 2
         call cmtens(hteil(pmat2(2,k)),psi(dzpsi(si,sf)),&
              workc(1,zw(2)),vdim(2,si),dim(2,si),ndim(2,si),&
              vdim(2,sf),dim(2,sf),ndim(2,sf))
!     Contract modes 3..(nmode-1)
         do m=3,(nmode-1)
            call cmtens(hteil(pmat2(m,k)),workc(1,zw(2)),&
                 workc(1,zw(3)),vdim(2,si),dim(m,si),ndim(m,si),&
                 vdim(2,sf),dim(m,sf),ndim(m,sf))
            hzw = zw(2)
            zw(2) = zw(3)
            zw(3) = hzw
         enddo
!     Contract mode nmode, i.e. calculate hloch(phloch(1,k))
         call cmtens(hteil(pmat2(nmode,k)),workc(1,zw(2)),&
              tmphloch,dim(1,si),dim(nmode,si),&
              ndim(nmode,si),dim(1,sf),dim(nmode,sf),&
              ndim(nmode,sf))
         h=hamilton(fkoe,k)
         if (htmtype(h) .ne. 1)&
              call xmxxzzo(koeff(k),tmphloch,dim(1,sf),dim(1,si))
         call tranmxz(tmphloch,hloch(phloch(1,k)),dim(1,si),dim(1,sf))

!     Treat modes 2..(nmode-2)
         do m=2,(nmode-2)
!     Contract mode m+1, mode m is 'hloch'-mode
            call cmtens(hteil(pmat2(m+1,k)),workc(1,zw(1)),&
                 workc(1,zw(2)),dim(m,si),dim(m+1,si),&
                 ndim(m+1,si),dim(m,sf),dim(m+1,sf),ndim(m+1,sf))
!     Contract modes (m+2)..(nmode-1) 
            do e=(m+2),(nmode-1)
               call cmtens(hteil(pmat2(e,k)),workc(1,zw(2)),&
                    workc(1,zw(3)),dim(m,si),dim(e,si),&
                    ndim(e,si),dim(m,sf),dim(e,sf),ndim(e,sf))
               hzw = zw(2)
               zw(2) = zw(3)
               zw(3) = hzw
            enddo
!     Contract mode nmode, i.e. calculate hloch(phloch(m,k))
            call cmtens(hteil(pmat2(nmode,k)),workc(1,zw(2)),&
                 tmphloch,dim(m,si),dim(nmode,si),&
                 ndim(nmode,si),dim(m,sf),dim(nmode,sf),&
                 ndim(nmode,sf))
            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1)&
                 call xmxxzzo(koeff(k),tmphloch,dim(m,sf),dim(m,si))
            call tranmxz(tmphloch,hloch(phloch(m,k)),dim(m,si),dim(m,sf))
!     Prepare next step, i.e. contract mode m
            call cmtens(hteil(pmat2(m,k)),workc(1,zw(1)),&
                 workc(1,zw(3)),vdim(1,si),dim(m,si),&
                 ndim(m,si),vdim(1,sf),dim(m,sf),ndim(m,sf))
            hzw = zw(1)
            zw(1) = zw(3)
            zw(3) = hzw
         enddo
!     Calculate (nmode-1)-th hloch matrix
         call cmtens(hteil(pmat2(nmode,k)),workc(1,zw(1)),&
              tmphloch,dim(nmode-1,si),&
              dim(nmode,si),ndim(nmode,si),dim(nmode-1,sf),&
              dim(nmode,sf),ndim(nmode,sf))
         h=hamilton(fkoe,k)
         if (htmtype(h) .ne. 1)&
              call xmxxzzo(koeff(k),tmphloch,dim(nmode-1,sf),dim(nmode-1,si))
         call tranmxz(tmphloch,hloch(phloch(nmode-1,k)),dim(nmode-1,si),dim(nmode-1,sf))
!     Calculate nmode-th hloch matrix
         call cmtens(hteil(pmat2(nmode-1,k)),workc(1,zw(1)),&
              tmphloch,vdim(1,si),dim(nmode-1,si),&
              ndim(nmode-1,si),vdim(1,sf),dim(nmode-1,sf),&
              ndim(nmode-1,sf))
         h=hamilton(fkoe,k)
         if (htmtype(h) .ne. 1)&
              call xmxxzzo(koeff(k),tmphloch,dim(nmode,sf),dim(nmode,si))
         call tranmxz(tmphloch,hloch(phloch(nmode,k)),dim(nmode,si),dim(nmode,sf))

      endif


!     Nmode = 2
      if (nmode .eq. 2) then
!     Calculate first hloch matrix
         call cmtens(hteil(pmat2(2,k)),psi(dzpsi(si,sf)),&
              tmphloch,vdim(2,si),dim(2,si),&
              ndim(2,si),vdim(2,sf),dim(2,sf),ndim(2,sf))               
         h=hamilton(fkoe,k)
         if (htmtype(h) .ne. 1)&
              call xmxxzzo(koeff(k),tmphloch,dim(1,sf),dim(1,si))
         call tranmxz(tmphloch,hloch(phloch(1,k)),dim(1,si),dim(1,sf))
!     Calculate second hloch matrix
         call cmtens(hteil(pmat2(1,k)),psi(dzpsi(si,sf)),&
              tmphloch,vdim(1,si),dim(1,si),&
              ndim(1,si),vdim(1,sf),dim(1,sf),ndim(1,sf))               
         h=hamilton(fkoe,k)
         if (htmtype(h) .ne. 1)&
              call xmxxzzo(koeff(k),tmphloch,dim(2,sf),dim(2,si))
         call tranmxz(tmphloch,hloch(phloch(2,k)),dim(2,si),dim(2,sf))
      endif
         

!     Nmode = 1
      if (nmode .eq. 1) then
       !  message = 'Nmode=1 not yet implemented.'
       !  call errormsg
         call rm1(psi(dzpsi(si,sf)),&
              tmphloch,vdim(1,si),dim(1,si),&
              ndim(1,si),vdim(1,sf),dim(1,sf),ndim(1,sf))               
         h=hamilton(fkoe,k)
         if (htmtype(h) .ne. 1)&
            call xmxxzzo(koeff(k),tmphloch,dim(1,sf),dim(1,si))
         call tranmxz(tmphloch,hloch(phloch(1,k)),dim(1,si),dim(1,sf))
      endif


      return 
      end subroutine d2matrizen2l


!***********************************************************************
!
!     RACT1: Calculates for right term V(k) the product rho*V(k)
!
!     10/99 AR
!***********************************************************************

      subroutine ract1(psi1,psi2,hteil,k)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod

      implicit none
      
      integer(long)                                  :: m,sf,si,hzw,s,h
      integer(long), intent(in)                      :: k
      integer(long), dimension(2)                    :: zw
      complex(dop), dimension(adim), intent(in)      :: psi1
      complex(dop), dimension(adim), intent(out)     :: psi2
      complex(dop), dimension(pmatdim), intent(in)   :: hteil
      complex(dop), dimension(2*maxblock*maxblock,2) :: workc


      call zerovxz(psi2,adim)
      si = kf(k)
      sf = ki(k)

      do s=1,nstate
         zw(1) = 1
         zw(2) = 2
         call cpvxz(psi1(dzpsi(s,si)),workc(1,zw(1)), &
              block(s)*block(si))
         do m=1,nmode
!     Note that for si .ne. sf always diag(m,k)=.false.
            if (.not. diag(m,k)) then
!     Note that si=kf(k), sf=ki(k)
               call mttxzz(hteil(pmat(m,k)),workc(1,zw(1)), &
                    workc(1,zw(2)),block(s)*vdim(m,sf), &
                    dim(m,si),ndim(m,si),dim(m,sf))
               hzw = zw(1)
               zw(1) = zw(2)
               zw(2) = hzw
            endif
         enddo
         h=hamilton(fkoe,k)
         if (htmtype(h) .eq. 1) then
            call cpvxz(workc(1,zw(1)),psi2(dzpsi(s,sf)), &
                 block(s)*block(sf))
         else
            call xvxxzza(koeff(k),workc(1,zw(1)), &
                 psi2(dzpsi(s,sf)),block(s)*block(sf))
         endif
      enddo

      return
      end subroutine ract1



!***********************************************************************
!
!     RACT2: Calculates for right term V(k) the product rho*V(k)*rho
!
!     10/99 AR
!***********************************************************************

      subroutine ract2(psi1,psi2,hteil,k)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use hpsimod

      implicit none
      
      integer(long)                                 :: m,sf,si,hzw,s,t,h
      integer(long), intent(in)                     :: k
      integer(long), dimension(2)                   :: zw
      complex(dop), dimension(adim), intent(in)     :: psi1
      complex(dop), dimension(adim), intent(out)    :: psi2
      complex(dop), dimension(pmatdim), intent(in)  :: hteil
      complex(dop), dimension(maxblock*maxblock,2)  :: workc


      si = ki(k)
      sf = kf(k)

      do s=1,nstate
         do t=1,nstate
            zw(1) = 1
            zw(2) = 2
!     Calculate V(k)*rho
            call cpvxz(psi1(dzpsi(si,t)),workc(1,zw(1)),&
                 block(si)*block(t))
            do m=1,nmode
!     Note that for si .ne. sf always diag(m,k)=.false.
               if (.not. diag(m,k)) then
!     Note that si=ki(k), sf=kf(k)
                  call mtxxzz(hteil(pmat(m,k)),workc(1,zw(1)),&
                       workc(1,zw(2)),vdim(m,sf),dim(m,si),&
                       ndim(m,si)*block(t),dim(m,sf))
                  hzw = zw(1)
                  zw(1) = zw(2)
                  zw(2) = hzw
               endif
            enddo
            call mmxxzz(psi1(dzpsi(s,sf)),workc(1,zw(1)),&
                 psi2(dzpsi(s,t)),block(s),block(sf),block(t))

            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1) then
               call xvxxzzo(koeff(k),psi2(dzpsi(s,t)),&
                    block(s)*block(t))
            endif
         enddo
      enddo

      return
      end subroutine ract2



!***********************************************************************
!
!     RHO2: Calculates rho^2
!
!     10/99 AR
!***********************************************************************

      subroutine rho2(psi1,psi2)

use global
use griddatmod
use operdef
use psidef
use hpsimod

      implicit none
      
      integer(long)                              :: s,t,u
      complex(dop), dimension(adim), intent(in)  :: psi1
      complex(dop), dimension(adim), intent(out) :: psi2

      do s=1,nstate
         do t=1,nstate            

            call mmxxzz(psi1(dzpsi(s,1)),psi1(dzpsi(1,t)), &
                 psi2(dzpsi(s,t)),block(s),block(1),block(t))

            do u=2,nstate

               call mmxxzza(psi1(dzpsi(s,u)),psi1(dzpsi(u,t)), &
                    psi2(dzpsi(s,t)),block(s),block(u),block(t))
            enddo
         enddo
      enddo

      return
      end subroutine rho2

      end module d2mfieldsmod
