! **********************************************************************
!                                                                       
!                   module D2SUMMF
!                                                                       
!   d2summf:        Multiplies for all modes and all states the sum     
!                   over the mean-field matrices times the              
!                   single-particle potential operator matrices with    
!                   |phi>.                                              
!                                                                       
! **********************************************************************
      module d2summfmod

use decimal, only: long, dop
use op1lib,  only: zerovxz, initvxz

      implicit none
      private
      public :: d2summf
      contains

! *** V6.0 MB ***************> SUBROUTINE D2SUMMF <*********************
!                             ~~~~~~~~~~~~~~~~~~~
!
! Sums up for all modes and all states the product of the mean-field
! matrices times the single-particle potential operators.
! **********************************************************************

      subroutine d2summf (mfsum,hloch)

use global
use griddatmod
use operdef
use runpropmod
use psidef
use hpsimod
use maxv,    only: maxdim,maxsta,maxkoe
use hphimod, only: hcall
use lalib,  only: mktens

      implicit none

      integer(long)                                  :: m,k,kt,s,s1,zeig,zeig1,hzeig
      complex(dop), dimension(mfsumdim), intent(out) :: mfsum
      complex(dop), dimension(hlochdim), intent(in)  :: hloch
      complex(dop), dimension(3*sdim)                :: workc
      logical(kind=4)                                :: lcalc

! --- INITIALISE VARIABLES ---

      call zerovxz(mfsum,mfsumdim)

! --- LOOP OVER EACH MODE AND EACH CORRELATED HAMILTONIAN TERM ---
! --- SUM UP MEAN-FIELD TIMES SINGLE-PARTICLE OPERATOR ---
      do m = 1,nmode
         do k = 1,kzahl(1)
!     kact = 1
            if ((kact(k).eq.1) .and. (.not. diag(m,k))) then
               if (kinop(m,k).eq.0) then
                  zeig=1
                  zeig1=zeig+subdim(m)
                  s1=ki(k)
                  s=kf(k)
                  call initvxz(workc(zeig1),subdim(m))
                  call hcall(m,k,workc(zeig),workc(zeig1),subdim(m))
                  call mktens(mfsum(zmfsum(m,s1,s)),workc(zeig), &
                       hloch(phloch(m,k)),subdim(m),dim(m,s), &
                       dim(m,s1),true)
               endif
            endif
!     kact = 2
            if ((kact(k).eq.2) .and. (k.lt.ktwin(k))) then
               kt = ktwin(k)
               if ((.not.diag(m,k)) .or. (.not.diag(m,kt))) then
                  if ((kinop(m,k).eq.0) .and. (kinop(m,kt).eq.0)) &
                       then
                  zeig=1
                  zeig1=zeig+subdim(m)
                  s1=ki(kt)
                  s=kf(k)
                  call initvxz(workc(zeig1),subdim(m))
                  if (.not. diag(m,kt)) then
                     call hcall(m,kt,workc(zeig),workc(zeig1),subdim(m))
                     hzeig = zeig1
                     zeig1 = zeig
                     zeig  = hzeig
                  endif
                  if (.not. diag(m,k)) then
                     call hcall(m,k,workc(zeig),workc(zeig1),subdim(m))
                     hzeig = zeig1
                     zeig1 = zeig
                     zeig  = hzeig
                  endif
                  call mktens(mfsum(zmfsum(m,s1,s)),workc(zeig1), &
                       hloch(phloch(m,k)),subdim(m),dim(m,s), &
                       dim(m,s1),true)
                   endif
               endif
            endif
!     kact = 4
            if ((kact(k).eq.4) .and. (k.lt.ktwin(k))) then
               if ((kinop(m,k).eq.0) .and. (.not. diag(m,k))) then
                  kt = ktwin(k)
                  lcalc = .false.
                  if (lmf) then
                     if (ki(kt) .eq. kf(k)) lcalc = .true.
                  else
                     lcalc = .true.
                  endif
                  if (lcalc) then
                     zeig=1
                     zeig1=zeig+subdim(m)
                     s1=ki(k)
                     s=kf(k)
                     call initvxz(workc(zeig1),subdim(m))
                     call hcall(m,k,workc(zeig),workc(zeig1),subdim(m))
                     call mktens(mfsum(zmfsum(m,s1,s)),workc(zeig), &
                          hloch(phloch(m,k)),subdim(m),dim(m,s), &
                          dim(m,s1),true)
                  endif
               endif
            endif
         enddo
      enddo

      return
      end  subroutine d2summf


      end module d2summfmod

