!***********************************************************************
!
!                              D2TRACEOUT.F
!
! Contains all subroutines related to the output of the trace
!
! Subroutines:
!    traceout:  writes out trace of rho(dlstate,drstate), needed for 
!               spectrum
!    calctrace: calculates trace of rho(s,t)
!
! V81 AR
!
!***********************************************************************
      module d2traceoutmod

use decimal, only: long,dop
use maxv, only: maxdim,maxsta
use globmemmod
use logdat
use global
use griddatmod 
use psidef
use operdef
use runpropmod
use hpsimod
use datenmod
use channels

use hphimod, only: hphi,hunphi
use phihphimod, only: phihphi,phihunphi
use d2calchamod, only: d2calchunaloch2,d2calchaloch2
use d2tens, only: cmtens,cuttens2
use psiutil, only: spfovl
use gwplib, only: diagonal_moments
use lalib,  only: simtran2bz

use mmlib,   only: mmaxzz
use xvlib,   only: vvaxzz
use op1lib,  only: cpvxz, zerovxz, trqxz
use linear,  only: ijk_shape
use wftree1, only: allocate_overlap_matrices,deallocate_overlap_matrices
use mlanalysis, only: compute_overlap_tree

      implicit none
      private
      public :: d2traceout
      contains

!***********************************************************************
!                               TRACEOUT2
!
!    writes out trace of rho(dlstate,drstate), needed for spectrum
!
! V81 AR
!***********************************************************************

      subroutine d2traceout(psi,agmat,atime,firstcall)

      implicit none

      integer(long)                               :: s,k1,k2,n,ktype,hktype
      real(dop)                                   :: time
      real(dop), intent(in)                       :: atime
      complex(dop)                                :: trc,trace
      complex(dop), dimension(dgldim), intent(in) :: psi
      complex(dop), dimension(agmatdim), intent(in) :: agmat
      logical(kind=4), intent(in)                 :: firstcall

      complex(dop), save                          :: inctr2  = (1.0_dop,0.0_dop)
      complex(dop), dimension(:), allocatable     :: tmphteil,tmppsi


      time = atime*fs


!        get operator number      
      if (firstcall.and.(ctrname.ne.' ')) then
         do n=1,hamdim
            if (hamlab(n) .eq. ctrname) ctrnum=n
         enddo
         if (ctrnum.eq.0) then
            message = 'Operator used with <ctrace> not defined!'
            call warnmsg
         endif
      endif

      if (ctrnum.eq.0) then
!        system Hamiltonian         
         call calctrace(psi,agmat,trc,dlstate,drstate)
      else
         if (lgwp) then
            routine="Calctrace (d2traceout)"
            message="Ctrace with an operator not implemented for GWP basis"
            call errormsg
         endif
            
         allocate(tmphteil(pmatdim))
         allocate(tmppsi(adim))
!       operate on spfs of initial WF
         ktype=0
         call hphi(psi,mc(mchpsi),ktype,ctrnum)
         hktype=0
         call hunphi(psi,mc(mchunpsi),hktype,ctrnum)

         call zerovxz(tmphteil,pmatdim)
         call phihphi(psi,gh,mc(mchpsi),tmphteil,ctrnum,.false.)
         call phihunphi(psi,mc(mchunpsi),hung,gh,tmphteil,ctrnum)

         call zerovxz(tmppsi,adim)
         
         call d2calchunaloch2(psi,tmppsi,tmphteil,0,ctrnum)
         k1=zham(ctrnum)
         k2=zham(ctrnum)+kzahl(ctrnum)-1

         call d2calchaloch2(psi,tmppsi,tmphteil,0,1,k1,k2)
         
         trc=(0.0_dop,0.0_dop)
         do s=1,nstate
            call trqxz(tmppsi(dzpsi(s,s)-1),trace,block(s))
            trc = trc + trace
         enddo

         call vvaxzz(psi,tmppsi,trc,adim)
         call vvaxzz(tmppsi,psi,trc,adim)
         
         call calctrace(psi,agmat,trc,dlstate,drstate)        

      endif


      if (ctrnum.eq.0) then
         if (firstcall) then
            if (abs(trc).le.tiny) then
               write(ictr,'(a,2f18.14)')&
                    '# abs(trc).le.tiny !! trace = ',trc
               write(ictr,'(a)')&        
                    '# trc renormalized to (1.0_dop,0.0_dop)'
               trc = (1.0_dop,0.0_dop)
            endif
            !if (.not. (lrlx .and. (temp.gt.tiny))) then
            if (.not. (lrlx)) then
               inctr2 = trc
               trc  = (1.0_dop,0.0_dop)
            endif
         else
            trc = trc / inctr2
         endif
      else
         if (firstcall) then
            trc=(0.0_dop,0.0_dop)
         endif
      endif

      write(ictr,'(1x,g18.9,3(1x,f18.14))')&
           atime,dble(trc),dimag(trc),abs(trc)
     
      call myflush(ictr)


      return
      end subroutine d2traceout


!***********************************************************************
!                               CALCTRACE
!
!    calculates trace of rho(s,t)
!
! V81 AR
!***********************************************************************

      subroutine calctrace (psi,agmat,trc,s,t)

      implicit none

      integer(long)                                :: m,z1,z2,hzw
      integer(long)                                :: f,meb1,ndim1
      integer(long), dimension(3)                  :: ijk
      integer(long), intent(in)                    :: s,t
      complex(dop), intent(out)                    :: trc
      complex(dop), dimension(dgldim), intent(in)  :: psi
      complex(dop), dimension(agmatdim), intent(in) :: agmat
      complex(dop), dimension(gfgwpdim)            :: gfgwp
      complex(dop), dimension(aigwpdim)            :: aigwp
      complex(dop), dimension(zgwpdim)             :: zgwp
      complex(dop), dimension(maxblock*maxblock,2) :: workc
      complex(dop), dimension(dmatdim)             :: overlap
      type(wfnode_t),pointer                       :: childf
 
!-----------------------------------------------------------------------
! Compute autocorrelation
!-----------------------------------------------------------------------
      trc = (0.0_dop,0.0_dop)
      z1 = 1
      z2 = 2

      if (lmult) then
         call cpvxz(psi(dzpsi(s,t)),workc(1,z1),block(s)*block(t))

         do m = 1,nmode
            if (gwpm(m)) then
               call diagonal_moments(m,t,s,nspfdof(m),gwpdim(m,t),&
                    gwpdim(m,s),agmat(zaGmat(m,t)),agmat(zaGmat(m,s)),&
                    agmat(zxivec(m,t)),agmat(zxivec(m,s)),&
                    agmat(zphase(m,t)),agmat(zphase(m,s)),&
                    gfgwp(zgfgwp(m,t,s)),aigwp(zaigwp(m,t,s)),&
                    zgwp(zzgwp(m,t,s)),0,0)
            
! gfgwp contains GWP overlap. Transform to spf overlap
               call simtran2bz(psi(zdgwp(m,t)),gfgwp(zgfgwp(m,t,s)),&
                    psi(zdgwp(m,s)),overlap,gwpdim(m,s),&
                    gwpdim(m,t),dim(m,s),dim(m,t))
            else

               call mmaxzz(psi(zetf(m,t)),psi(zetf(m,s)),overlap, &
                    subdim(m),dim(m,t),dim(m,s))
            endif

            call cmtens(overlap,workc(1,z1),workc(1,z2), &
                 1,dim(m,s),ndim(m,s),1,dim(m,t),ndim(m,t))
            hzw = z1
            z1 = z2
            z2 = hzw
         enddo

      else
         z1 = 1
         z2 = 2
! calculate SPF overlaps
         if (lrunml) then
            call spfovl(overlap,psi,psi,agmat,agmat,0)

            call allocate_overlap_matrices(tree,tree,overlap)

            call compute_overlap_tree(psi,psi,tree)

            tz => tree%swap_down(1)%p
            do f=1,tree%nmodes
               childf => tz%submodes(f)%p
               if (childf%m .eq. meb) meb1=f
            enddo

         else
            call spfovl(overlap,psi,psi,agmat,agmat,0)
         endif

         if (lrunml) then
            ijk = ijk_shape(meb1,tree%ndim)
            if (s .eq. t) then
               call cuttens2(psi(dzpsi(1,1)),workc(1,z1),1,1,ijk(1),ijk(2),ijk(3))
            else
               call cuttens2(psi(dzpsi(1,1)),workc(1,z1),s,t,ijk(1),ijk(2),ijk(3))
            endif

         else if (s .eq. t) then
            call cuttens2(psi(dzpsi(1,1)),workc(1,z1),1,1,vdim(meb,1),dim(meb,1),&
                 ndim(meb,1))
         else
            call cuttens2(psi(dzpsi(1,1)),workc(1,z1),s,t,vdim(meb,1),dim(meb,1),&
                 ndim(meb,1))
         endif

         if (lrunml) then
            tz => tree%swap_down(1)%p
            do f=1,tree%nmodes
               childf => tz%submodes(f)%p
               if (childf%m .eq. meb1) cycle

               ijk = ijk_shape(f,tree%ndim)
! contraction over meb has already taken place - correct ndim if meb > m
               if (meb1 .gt. f) ijk(3) = ijk(3)/nddstate
               call cmtens(tree%overlap(f)%p%matrix,workc(1,z1),workc(1,z2), &
                    1,ijk(2),ijk(3),1,ijk(2),ijk(3))
               hzw = z1
               z1 = z2
               z2 = hzw
            enddo
            call deallocate_overlap_matrices(tree)
         else
            do m = 1,nmode
               if (m .eq. meb) cycle

               ndim1=ndim(m,1)
! contraction over meb has already taken place - correct ndim if meb > m
               if (meb .gt. m) ndim1 = ndim1/dim(meb,1)
               call cmtens(overlap(dmat(m,1)),workc(1,z1),workc(1,z2), &
                    1,dim(m,1),ndim1,1,dim(m,1),ndim1)
               hzw = z1
               z1 = z2
               z2 = hzw
            enddo
         endif
      endif

      trc = workc(1,z1) 

      return
      end subroutine calctrace

      end module d2traceoutmod

