!#######################################################################
      module absorbrwmod

use decimal, only: long, dop
use timing

      implicit none
      private
      public :: absorbrwp
      contains

!#######################################################################
!  This subroutine absorbs the wave packet and calculates the reaction
!  probability at this time tim. This is done in the flux formalism and
!  using the method of Pernot and Lester.
!#######################################################################

      subroutine absorbrwp(lprob,f,h,hops,psi1,psi2,ppl,pplt)

use dvrdatmod
use griddatmod
use operdef, only: hopsdim, zhop
use psidef, only: totphidim
!use runinwfmod
!use runpropmod

use operutil, only: getvgn

      implicit none

      integer(long), intent(in)                     :: f,h
      real(dop), dimension(hopsdim), intent(in)     :: hops

      real(dop), dimension(totphidim), intent(inout):: psi1,psi2
      real(dop), intent(out)                        :: ppl
      real(dop), dimension(ndof), intent(inout)     :: pplt
      integer(long)                                 :: vd,gd,nd
      logical(kind=4), intent(in)                   :: lprob
      integer(long), save                           :: tid=0 ! timer ID

      call get_timer(tid, 'Absorbrw')
      call start_timer(tid)

      call getvgn(h,vd,gd,nd)
      call absorbpsi(lprob,f,hops(zhop(h)),psi1,psi2,ppl,pplt,vd,gd,nd) 

      call stop_timer(tid)

    return
    end subroutine absorbrwp

! **********************************************************************
!
!  This subroutine absorb psi and calculate reactin probability using
!  the method of Pernot and Lester
!
! **********************************************************************
      subroutine absorbpsi(lprob,f,a,b1,b2,pr,prt,dim1,dim2,dim3)

use dvrdatmod
use griddatmod

use realwpmod

      implicit none

      integer(long)                                       :: i,j,k,isabs
      integer(long), intent(in)                           :: dim1,dim2,dim3,f
      real(dop), dimension(dim2), intent(in)              :: a
      real(dop), dimension(dim1,dim2,dim3), intent(inout) :: b1,b2
      real(dop), intent(out)                              :: pr 
      real(dop), dimension(ndof), intent(inout)           :: prt
      real(dop)                                           :: prob
      logical(kind=4), intent(in)                         :: lprob

      prob = 0.0_dop
      pr   = 0.0_dop
      isabs=indiabs(f)

      if (dim1.ne.1 .and. dim3.ne.1) then
         do k=1,dim3
            do j=isabs,dim2
               do i=1,dim1
                  prob=prob+b1(i,j,k)*b1(i,j,k)*(1.0_dop-a(j)*a(j))
                  b1(i,j,k)=a(j)*b1(i,j,k)
                  b2(i,j,k)=a(j)*b2(i,j,k)
               enddo
            enddo
         enddo
      else if (dim1.eq.1 .and. dim3.ne.1) then
         do k=1,dim3
            do j=isabs,dim2
               prob=prob+b1(1,j,k)*b1(1,j,k)*(1.0_dop-a(j)*a(j))
               b1(1,j,k)=a(j)*b1(1,j,k)
               b2(1,j,k)=a(j)*b2(1,j,k)
            enddo
         enddo
      else if (dim1.ne.1 .and. dim3.eq.1) then
         do j=isabs,dim2
            do i=1,dim1
               prob=prob+b1(i,j,1)*b1(i,j,1)*(1.0_dop-a(j)*a(j))
               b1(i,j,1)=a(j)*b1(i,j,1)
               b2(i,j,1)=a(j)*b2(i,j,1)
            enddo
         enddo
      else if (dim1.eq.1 .and. dim3.eq.1) then
         do j=isabs,dim2
            prob=prob+b1(1,j,1)*b1(1,j,1)*(1.0_dop-a(j)*a(j))
            b1(1,j,1)=a(j)*b1(1,j,1)
            b2(1,j,1)=a(j)*b2(1,j,1)
         enddo
      endif

      if (lprob) then
         pr     = prob
         prt(f) = prob+prt(f)
      endif

      return
      end subroutine absorbpsi


!#######################################################################
      end module absorbrwmod
