!######################################################################
      module datoutrwmod

use versions
use decimal, only: long, dop

      implicit none
      private
      public :: datoutrw
      contains

!######################################################################
!
!                      MCTDH module DATOUT
!
! writes data to output file
!
!    datout   : writes output file
!    outexp0, outexp1, outexp2, outexp3 : outputs expectation values for
!               the different basis set types.
!    wrdicht3o: writes the density matrix eigenvalues i.e. natural
!               orbital populations to output file.
!
! 10/96 AJ
!***********************************************************************

      subroutine datoutrw(t,spop,ecorr,etot,dicht3,gpop1,&
                 gpop2,ort,jsph,msph,qdq)

use global
use dvrdatmod
use griddatmod
use psidef
use runpropmod
use channels

use datoutmod, only: wrdicht3o,outexp0,outexp1,outexp3
use realwpmod, only: aunds,bunds

      implicit none

      real(dop), external                             :: fcputime,fsystime
      external                                        :: fpstartup

      integer(long)                                   :: s,m,i,n,f,&
                                                         version,release
      integer(long), dimension(sphdim), intent(in)    :: jsph,msph
      real(dop)                                       :: norm2,normt,ecorrt,&
                                                         etott,deltae,tcpu
      real(dop), intent(in)                           :: t
      real(dop), dimension(nstate), intent(in)        :: spop,ecorr,etot
      real(dop), dimension(d3matdim), intent(in)      :: dicht3
      real(dop), dimension(ortdim), intent(in)        :: ort
      real(dop), dimension(ortdim,nstate), intent(in) :: gpop1,gpop2
      real(dop), dimension(4,ndof,nstate), intent(out) :: qdq

      character(len=100)                              :: buffer
      logical(kind=4), save                           :: firstcall = .true.


!----------------------------------------------------------------------
! create headline and initialize variables
!----------------------------------------------------------------------
      if (firstcall) then
         if(lcont) then
            write(iout,*)
            write(iout,'(1x,a)') 'Continuation Run'
         else
            call getrevno(progver,version,release)
            write(iout,*)
            write(iout,'(79a)') ('*', i=1,79)
            write(iout,'(31x,a,i3)') 'MCTDH version',version
            write(iout,'(34x,a,i5)') 'Release ', release
            write(iout,'(79a)') ('*', i=1,79)
            write(iout,*)
         endif
         call fpstartup(buffer)
         write(iout,'(1x,a)') buffer(1:72)
         write(iout,'(1x,a)') name(1:laenge)
         if(runtitle(1:10) .ne. '          ' ) &
         write(iout,'(1x,a)') runtitle(1:78)
         write(iout,'(78a)') ('-',i=1,78)
      endif
      write(iout,*)

!----------------------------------------------------------------------
! 1. write state populations, norm and energy expectation values
!----------------------------------------------------------------------
      norm2    = 0.0_dop
      ecorrt   = 0.0_dop
      etott    = 0.0_dop

      do s=1,nstate
         norm2 = norm2+spop(s)
         ecorrt= ecorrt+ecorr(s)
         etott = etott+etot(s)
      enddo

      normt  = sqrt(norm2/dble(max(npackts,npacket)))
      if(lreflex) normt=normt/sqrt(2.d0)
      etott  = etott/norm2
      ecorrt = ecorrt/norm2
      etott  = (etott-bunds)*ev/aunds 
      ecorrt = (ecorrt-bunds)*ev/aunds
      if (firstcall) then
         firstcall = .false.
         if (abs(estart-0.0_dop) .lt. tiny) estart = etott
      end if
      deltae = etott - estart

!-----------------------------------------------------------------------
! Renormalize energy expectation values
!-----------------------------------------------------------------------
      tcpu = fcputime(0)+fsystime(0)
      write(iout,800)  t/fs,   tcpu,  normt
      if (ltraj .or. (lgwp .and. nmode .eq. 1)) then
         write(iout,804) etott, ecorrt, deltae
      else
         write(iout,802) etott, ecorrt, deltae
      endif
      if (nstate.gt.1) then
         write(iout,*)
         do s=1,nstate
            write (iout,803)s,spop(s),ecorr(s)*ev,etot(s)*ev
         enddo
      endif

 800  format(' Time  =',f11.2,' fs,       CPU =',f11.2,' s,',3x, &
             ' Norm    =',f11.8)
 802  format(' E-tot =',f11.6,' eV,    E-corr =',f11.6,'eV,',3x, &
             ' Delta-E =',3pf11.4,' meV')
 803  format(' state =',i2,'  pop.:',f11.8,3x,'E-corr:', &
             f11.6,'eV,    E-tot =',f11.6,'eV')
 804  format(' E-tot =',f11.6,' eV,     E-pot =',f11.6,'eV,',3x, &
             ' Delta-E =',3pf11.4,' meV')


!-----------------------------------------------------------------------
! loop over states and output:
! 2. natural orbital populations
! 3. if NO propagation check dicht2 is diagonal i.e. largest off
!    diagonal element is smaller than 1d-6
! 4. Output various expectation values, depending on basis type
!-----------------------------------------------------------------------
      do s=1,nstate

         if(nstate .eq. 1 ) then
            write(iout,'(/" Natural weights *1000 :")')
         else
            write(iout,'(/" Natural weights *1000,  state =", i2)') s
         end if

!        if (nmode .eq. 1 .and. lgwp) then
! for vMCG, output populations of GWPs
!           m=1
!           f=spfdof(1,m)
!           call wrdicht31(dicht2(dmat(m,s)),dim(m,s),
!    +              modelabel(f),nspfdof(m))
!        else
            do m=1,nmode
               f=spfdof(1,m)
               call wrdicht3o(dicht3(d3mat(m,s)),dim(m,s), &
                    modelabel(f),nspfdof(m))
            enddo
!        endif

         if(nstate .eq. 1 ) then
         write(iout,'(/" Mode expectation values and variances :")')
         else
         write(iout,&
         '(/" Mode expectation values and variances,  state =", i2)') s 
         end if

         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (basis(f).eq.0) then
                  call outexp0(gpop1(zort(f),s),gdim(f))
               elseif (basis(f).eq.-1) then
                  continue
               elseif (ldvr(f)) then
                  call outexp1(gpop1(zort(f),s),gpop2(zort(f),s), &
                       spop(s),gdim(f),ort(zort(f)),basis(f), &
                       iout,modelabel(f),ipbaspar(1,f),ipbaspar(1,f+1),qdq(1,f,s))
               elseif (basis(f).eq.6) then
                  call outexp3(gpop1(zort(f),s),spop(s),gdim(f), &
                        jsph(zsph(f)),msph(zsph(f)),iout, &
                        modelabel(f),modelabel(f+1),qdq(1,f,s),qdq(1,f+1,s))
               elseif (basis(f)==4 .or. basis(f)==25 .or. basis(f)==26 &
                      .or. basis(f)==27) then
                  write (6,*) 'Error in datoutrw for basis=', basis(f)
                  stop
               endif
            enddo
         enddo

      enddo
      write(iout,'(78a)') ('-',m=1,78)
      call myflush(iout)

      return
      end subroutine datoutrw

!#######################################################################

      end module datoutrwmod

