!***********************************************************************
!
!                  MCTDH module dichtout
!
! calculates grid populations in both "coordinate" and "momentum /
!   occupation" representations, and outputs results to gridpop file.
!
!   test for the automatic activation of the CAP / calls to activcap
!
! calls to firstpop, secpop0, secpop2.
!
! passed back to calling routine:
!   gpop1(m,s): first set of grid populations 
!   gpop2(m,s): second set of grid populations 
!
! gpop1 contains:
!   if time independent basis, basis function population
!   if DVR of FFT, coordinate representation populations
! gpop2 contains:
!   if time independent basis, basis function population (identical to 
!      gpop1)
!   if DVR, FBR basis function occupation numbers
!   if FFT, momentum representation populations
!
! if lpop2=.false. only gpop1 calculated. 
! V6.3  GW
!
!***********************************************************************
      module dichtoutrwmod

use decimal, only: long, dop
use op1lib,  only: zerovxd, zeromxd, cpmxd
use mtlib, only: qtxxdd, mtxxdd

      implicit none
      private
      public :: dichtoutrw
      contains

!***********************************************************************

      subroutine dichtoutrw(psit1,gpop1,gpop2,trafo,gp,jsph,msph,lpop2)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use channels

      implicit none

      integer(long)       :: f1,s,m,f,n,zeig1,zeig2,k,kk,ggdim,subdm

      integer(long), dimension(sphdim), intent(in)        :: jsph,msph
      real(dop), dimension(dvrdim), intent(in)            :: trafo
      real(dop), dimension(2*maxgdim), intent(out)        :: gp
      real(dop), dimension(ortdim,nstate), intent(out)    :: gpop1,gpop2
      real(dop), dimension(totphidim), intent(in)         :: psit1
      real(dop), dimension(4*sdim*maxspf)                 :: workr
      logical(kind=4), intent(in)                         :: lpop2
      character(len=c2)                                   :: wrtag

!-----------------------------------------------------------------------
! calculate first set of grid populations
!  1. transform to natural orbitals
!  2. sum up populations on grid points for each degree of freedom
!-----------------------------------------------------------------------
      m=1
      s=1
      f1=spfdof(1,m)
      if (basis(f1) .ne. 25 .and. &
         basis(f1) .ne. 26 .and. basis(f1) .ne. 27) then
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            call calcgpoprw(psit1,vgdim(f),gdim(f),&
                 ngdim(f),gpop1(zort(f),s))
         enddo
      endif

      if (lpdens) call myflush(ipden)

!-----------------------------------------------------------------------
! return if only gpop1 required
!-----------------------------------------------------------------------
      if (.not. lpop2) return

      call zeromxd(gpop2,ortdim,nstate)
!-----------------------------------------------------------------------
! Get basis identification and calculate second set of grid populations
! zeig is a pointer to divide workspace
!-----------------------------------------------------------------------
      do s=1,nstate
         do m=1,nmode
            zeig1=1
            zeig2=subdim(m)*dim(m,s)+1
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if(basis(f).eq.12 .or. basis(f).eq.20 .or. &
                  basis(f).eq.21) &
                    zeig2=(ipbaspar(3,f)*subdim(m)/gdim(f))*dim(m,s)+1 
               if (basis(f).eq.0) then
                  call secpop0 (gpop1(zort(f),s),&
                      gpop2(zort(f),s),gdim(f))
               elseif (basis(f).eq.4) then
                   write(6,*)'Error in dichtoutrw for calculation of gpop2',&
                              ' not work for fft basis yet'
                   stop
               elseif (basis(f).eq.6) then
                  call sphpop(gpop1(zort(f),s),gpop2(zort(f),s),gdim(f),&
                       jsph(zsph(f)),msph(zsph(f)),f)
               elseif (basis(f).eq.-1) then
                  continue
               elseif (basis(f).eq.25) then
                  continue
               else if (basis(f) .eq. 26 .or. basis(f) .eq. 27) then
                  continue
               elseif (ldvr(f)) then
                  call cpmxd(psit1,workr(zeig1),&
                       subdim(m),dim(m,s))
                  if(basis(f) .eq. 5)  then   ! exp
                  write(6,*)'Error in dichtoutrw for calculation of gpop2',&
                            ' not work for EXP basis yet'
                   stop
                  elseif( basis(f) .eq. 7 .or. basis(f) .eq. 20) then  ! KLeg KLeg/R
                     ggdim = gdim(f)
                     if(basis(f) .eq. 20) ggdim = ipbaspar(3,f+1)
                     call trklegrw(trafo(zdvr(f)),workr(zeig1),&
                          workr(zeig2),ggdim,ipbaspar(1,f),&
                          ipbaspar(1,f+1),ipbaspar(3,f+1),vgdim(f),&
                          gdim(f),gdim(f+1),ngdim(f+1),dim(m,s))

                  elseif (basis(f).eq.9 .or. basis(f).eq.21) then    ! PLeg  PLeg/R
                  write(6,*)'Error in dichtoutrw for calculation of gpop2',&
                            ' not work for PLeg/PLeg/R basis yet'
                   stop
                  elseif (basis(f).eq.12) then   ! Leg/r
                     call mtxxdd(trafo(zdvr(f)),workr(zeig1),&
                          workr(zeig2),vgdim(f),gdim(f),&
                          ngdim(f)*dim(m,s),ipbaspar(3,f))
                  elseif (basis(f).eq.25) then
                     continue
                  else
                     call qtxxdd(trafo(zdvr(f)),workr(zeig1),&
                                 workr(zeig2),vgdim(f),gdim(f),&
                                 ngdim(f)*dim(m,s))
                  endif
                  if(basis(f).eq.12 .or. basis(f).eq.20 .or. &
                     basis(f).eq.21) then        ! Leg/r, Kleg/r or Pleg/r 
                     ggdim = ipbaspar(3,f) 
                     subdm = ggdim*subdim(m)/gdim(f)
                     call calcgpoprw(workr(zeig2), &
                          vgdim(f),ggdim,ngdim(f), &
                          gpop2(zort(f),s))
                  elseif (basis(f).eq.25) then
                     continue
                  else
                     call calcgpoprw(workr(zeig2), &
                          vgdim(f),gdim(f),ngdim(f), &
                          gpop2(zort(f),s))
                  endif
               endif
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! output grid populations in single precision to gridpop file
!-----------------------------------------------------------------------
      if (lgpop) then
         countrho=countrho+1

!-----------------------------------------------------------------------
! Write Data Set
!-----------------------------------------------------------------------
         write(wrtag,'(a9,i10,a1)') '<data set',countrho,'>'
         write(irho) wrtag

         wrtag='<gpop>'
         write(irho) wrtag
         if (lgpel  .or.  nstate .eq. 1) then
            do s=1,nstate
               do m=1,nmode
                  do n=1,nspfdof(m)
                     f=spfdof(n,m)
                     ggdim = gdim(f)
                     if(basis(f).eq.12 .or.basis(f).eq.20 .or.  &
                        basis(f).eq.21)  &
                         ggdim = ipbaspar(3,f) 
                     write(irho)  &
                      (real(gpop1(zort(f)+k,s)),k=0,gdim(f)-1)  &
                     ,(real(gpop2(zort(f)+k,s)),k=0,ggdim-1)
                  enddo
               enddo
            enddo

         else
            do m=1,nmode
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  ggdim = gdim(f)
                  if(basis(f).eq.12 .or. basis(f).eq.20 .or.  &
                     basis(f).eq.21)  &
                     ggdim = ipbaspar(3,f) 
                  do k = 1, gdim(f)+ggdim
                     gp(k) = 0.0_dop
                  enddo
                  do s=1,nstate
                     do k = 1, gdim(f)
                        gp(k) = gp(k) + gpop1(zort(f)+k-1,s)
                     enddo
                     do k = 1, ggdim
                        kk = k + gdim(f)
                        gp(kk) = gp(kk) + gpop2(zort(f)+k-1,s)
                     enddo
                  enddo
                  write(irho) (real(gp(k)),k=1,gdim(f)), &
                              (real(gp(k)),k=gdim(f)+1,gdim(f)+ggdim)
               enddo
            enddo
         endif
         
         write(wrtag,'(a13,i10,a1)') '<end data set',countrho,'>'
         write(irho) wrtag
         call myflush(irho)
      endif

!-----------------------------------------------------------------------

      return
      end subroutine dichtoutrw

!#######################################################################
! CALCGPOP
!
! For time independent bases, calculates the populations of the
!   natural orbitals
! For DVR or FFT bases, calculates the populations of the grid points 
!   in the coordinate representation
!
! input for a single mode and state:
!  dicht3: eigenvalues of density matrix
!  dim   : number of single particle functions
!  gdim  : number of grid points
!  natur : natural orbitals
! output:
!  gpop  : population of grid points
!
! V6.3 GW
!#######################################################################

      subroutine calcgpoprw(natur,vgdim,gdim,ngdim,gpop)

      implicit none

      integer(long)                                      :: g,n,v
      integer(long), intent(in)                          :: vgdim,ngdim,gdim
      real(dop), dimension(gdim), intent(out)            :: gpop
      real(dop), dimension(vgdim,gdim,ngdim), intent(in) :: natur


!-----------------------------------------------------------------------
! Calculate populations
!-----------------------------------------------------------------------
         call zerovxd(gpop,gdim)

         do n=1,ngdim
            do g=1,gdim
               do v=1,vgdim
                  gpop(g)=gpop(g)+natur(v,g,n)*natur(v,g,n)
               enddo
            enddo
         enddo

      return
      end subroutine calcgpoprw

!#######################################################################
! SECPOP0
!
! For time independent bases, no further populations required and so
!  gpop2 is merely set equal to gpop1.
!
! input for a single mode and state:
!  gpop1  : population of natural orbitals points
!  gdim   : number of grid points
! output:
!  gpop2  : population of natural orbitals points
!
! V6.3 GW
!#######################################################################

      subroutine secpop0(gpop1,gpop2,gdim)

      implicit none

      integer(long)                           :: g
      integer(long), intent(in)               :: gdim
      real(dop), dimension(gdim), intent(in)  :: gpop1
      real(dop), dimension(gdim), intent(out) :: gpop2

      do g=1,gdim
         gpop2(g)=gpop1(g)
      enddo

      return
      end subroutine secpop0


!#######################################################################
! SPHPOP
!
! called by dichtout. Computes the 1D populations of the shpFBR.
!
! 05/00 HDM
!#######################################################################

      subroutine  sphpop(gpop1,gpop2,gdim1,jsph,msph,f)

use global
use dvrdatmod
use griddatmod

      implicit none

      integer(long)                               :: gj,mmin,mmax,mincr,&
                                                     sphtyp,dj,j0,im,ij,jmax,g
      integer(long), intent(in)                   :: gdim1,f
      integer(long), dimension(gdim1), intent(in) :: jsph,msph
      real(dop), dimension(gdim1), intent(in)     :: gpop1
      real(dop), dimension(gdim1), intent(out)    :: gpop2

      call zerovxd(gpop2,gdim1)

      sphtyp = ipbaspar(1,f)
      jmax   = ipbaspar(2,f)
      gj     = ipbaspar(4,f)
      mmax   = ipbaspar(1,f+1)
      mincr  = ipbaspar(2,f+1)
      mmin   = ipbaspar(3,f+1)
      dj = 1
      if(sphtyp .eq.2  .or. sphtyp .eq.3 ) dj = 2
      j0 = mod(jmax,dj)
      
      do g = 1, gdim1
         ij = (jsph(g)-j0)/dj + 1
         im = (msph(g)-mmin)/mincr + 1 + gj
         gpop2(ij) = gpop2(ij) + gpop1(g)
         gpop2(im) = gpop2(im) + gpop1(g)
      enddo
       
      return
      end subroutine  sphpop

!#######################################################################
! TRKLEG
!
! called by dichtout
! For "Kleg-DVR".
!
! input:
!       trafo - transformation matrix
!       psi   - wave function
! output:
!       trpsi - wave function transformed to the j-grid
!
! 11/99 SS
! when KLEG/R, ggdim is the full grid length and gdim the reduced.
! For full KLEG, these are equal
!
! BUG??
! Not sure this routine is correct for KLEG/R as trafo / psi indices
! may not match GAW (7/10)
!#######################################################################

      subroutine trklegrw(trafo,psi,trpsi,ggdim,ibleg,kmin,kmin2,vgdim, &
                        gdim,kdim,ngdim,dim)

      implicit none

      integer(long)             :: g,j,m,v,n,e,m1,j1
      integer(long), intent(in) :: vgdim,gdim,ggdim,kdim,ngdim,dim,kmin,kmin2,ibleg

      real(dop), dimension(ggdim,ggdim,kdim), intent(in)            :: trafo
      real(dop), dimension(vgdim,gdim,kdim,ngdim,dim), intent(in)   :: psi
      real(dop), dimension(vgdim,ggdim,kdim,ngdim,dim), intent(out) :: trpsi

      call zerovxd(trpsi,vgdim*ggdim*kdim*ngdim*dim)
      do e=1,dim
         do n=1,ngdim
            do j=1,ggdim
               do m=1,kdim
                  m1=m+kmin-1
                  if (ibleg.eq.0) then
                     j1 = j + abs(m1) - kmin2
                  else
                     j1 = j + (abs(m1)+ibleg-1)/2 - (kmin2+ibleg-1)/2
                  endif
                  if (j1 .le. ggdim) then
                     do g=1,gdim
                        do v=1,vgdim
                           trpsi(v,j1,m,n,e) = trpsi(v,j1,m,n,e) + &
                                             trafo(j,g,m)*psi(v,g,m,n,e)
                        enddo
                     enddo
                  endif
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine trklegrw


      end module dichtoutrwmod

