!#######################################################################
      module fluxrwmod

use decimal, only: long, dop
use timing

      implicit none
      private
      public :: fluxrwp
      contains

!#######################################################################
!  This subroutine absorbs the wave packet and calculates the reaction
!  probability at this time tim. This is done in the flux formalism and
!  using the method of Pernot and Lester.
!#######################################################################

      subroutine fluxrwp(time,unitrf,f,h,hops,psi1,prflux,prfluxt)

use operdef
use psidef, only:totphidim
use realwpmod

use operutil, only: getvgn

      implicit none

      integer(long), intent(in)                  :: f,h
      integer(long)                              :: vd,gd,nd

      integer(long), intent(in)                  :: unitrf
      real(dop), intent(in)                      :: time
      real(dop), intent(out)                     :: prflux
      real(dop), dimension(nfluxf), intent(inout):: prfluxt
      real(dop), dimension(hopsdim), intent(in)  :: hops
      real(dop), dimension(totphidim), intent(in):: psi1
      integer(long), save                        :: tid=0 ! timer ID

      call get_timer(tid, 'Fluxrw')
      call start_timer(tid)

      call getvgn(h,vd,gd,nd)
      call fluxpsi(time,unitrf,f,hops(zhop(h)),psi1,prflux,prfluxt,vd,gd,nd) 

      call stop_timer(tid)

    return
    end subroutine fluxrwp

! **********************************************************************
!
!  This subroutine absorb psi and calculate reactin probability using
!  the method of Pernot and Lester
!
! **********************************************************************
      subroutine fluxpsi(time,unitrf,f,a,b1,pr,prt,dim1,dim2,dim3)

use realwpmod

      implicit none

      integer(long)                                       :: i,j,k,nflux,itel
      integer(long), intent(in)                           :: dim1,dim2,dim3,f
      integer(long), intent(in)                           :: unitrf

      real(dop)                                           :: flux,temp,fltest,timepoint
      real(dop), intent(in)                               :: time
      real(dop), dimension(dim2), intent(in)              :: a
      real(dop), intent(out)                              :: pr
      real(dop), dimension(nfluxf), intent(inout)         :: prt
      real(dop), dimension(dim1,dim2,dim3), intent(in)    :: b1
      real(dop), dimension(dim1*dim3)                     :: coeftmp


      flux = 0.0_dop
      pr   = 0.0_dop
      timepoint=time*timestep
      nflux=indiflux(f)

      itel=1
      do k=1,dim3
         do i=1,dim1
            temp=0.0_dop
            do j=1,dim2
               temp=temp+b1(i,j,k)*a(j)
            enddo
            fltest=temp*b1(i,nflux,k)
            flux=flux+fltest
            coeftmp(itel)=temp 
            itel=itel+1
         enddo
      enddo

      pr =flux*timestep
      prt(f)=prt(f)+(flux*timestep)
      if (lsave) then
         write (unitrf)timepoint,(coeftmp(i),i=1,itel-1)

         itel=1
         do k=1,dim3
            do i=1,dim1
               coeftmp(itel)=b1(i,nflux,k)
            itel=itel+1
            enddo
         enddo
         write (unitrf)timepoint,(coeftmp(i),i=1,itel-1)

         call flush(unitrf)
      endif

      return
      end subroutine fluxpsi


!#######################################################################
      end module fluxrwmod
