!#######################################################################
      module initproprwmod

use decimal, only: long, dop
use globmemmod
use logdat

      implicit none
      private
      public :: initproprw,scalh,fluxrd,wkinitproprw
      contains

!#######################################################################
!                                                                      
!                  module initprop
!                                                                      
!  MCTDH module initialising the propagation
!                                                                      
!#######################################################################
! INITPROP
!
!#######################################################################

      subroutine initproprw(hops,trafo,ort)

use global
use maxv, only: maxkoe
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
!use runpropmod

use realwpmod
use operutil, only: getvgn
use op1lib, only: cpvxd, zerovxd

      implicit none

      integer(long)                                :: nh,hl,hl1,nh1
      integer(long), dimension(maxkoe)             :: fcoup
      integer(long)                                :: f,h,k,n,ggdim,vd,gd,nd
      real(dop), dimension(hopsdim), intent(inout) :: hops
      real(dop), dimension(dvrdim), intent(in)     :: trafo
      real(dop), dimension(ortdim), intent(in)     :: ort
      real(dop), dimension(totphidim)              :: workr

call zerovxd(workr,totphidim)

!--------------------------------------
! Set lrotcoup(k) and lrotvec(k) here
! lrotvec is set for setting the rotational vector(subdim1) for ALL rotational terms!!
!--------------------------------------
outer2:do k=1,khtot(1)
          lrotcoup(k)=.false.
          lrotvec(k)=.false.
          nrotcoup(k)=0
          do n=1,nspfdof(1)
             f=spfdof(n,1)
             h=max(1,hamilton(f,k))
             do nh=1,htmfac(h)
                hl=hterm(nh,h)
                if (hl /= 0) then
                   do nh1=1,htmfac(hl)
                      hl1=hterm(nh1,hl)
                      if (hoplab(hl1)(1:3) == 'j^2' .or. hoplab(hl1)(1:7) == 'j^2kleg') then
                         lrotvec(k)=.true.
                         cycle outer2
                      else if (basis(f)==7 .or. basis(f)==20) then
                          if (hoplab(hl1)(1:3) == 'jpm') then
                             fcoup(k)=f     
                             lrotcoup(k)=.true.
                             nrotcoup(k)=1  
                             cycle outer2
                          else if (hoplab(hl1)(1:3) == 'j_p') then
                             fcoup(k)=f     
                             lrotcoup(k)=.true.
                             nrotcoup(k)=2  
                             cycle outer2
                          else if (hoplab(hl1)(1:3) == 'j_m') then
                             fcoup(k)=f     
                             lrotcoup(k)=.true.
                             nrotcoup(k)=3  
                             cycle outer2
                          else if (hoplab(hl1)(1:4) == 'jp^2') then
                             fcoup(k)=f     
                             lrotcoup(k)=.true.
                             nrotcoup(k)=4  
                             cycle outer2
                          else if (hoplab(hl1)(1:4) == 'jm^2') then
                             fcoup(k)=f     
                             lrotcoup(k)=.true.
                             nrotcoup(k)=5  
                             cycle outer2
                          endif
                      endif
                   enddo
                endif
             enddo
          enddo
       enddo outer2
!--------------------------------------------------------------------------------------------

       nfluxf=0
       nabsf =0
       do k=1,khtot(1)
          do n=1,nspfdof(1)
             f=spfdof(n,1)
             h=max(1,hamilton(f,k))
             if (htmtype(h) == 2 .and. .not. lrotvec(k) .and. .not. lrotcoup(k)) then
                call getvgn(h,vd,gd,nd)
                if (gd == totphidim) call cpvxd(hops(zhop(h)),workr,totphidim)
             endif
          enddo
       enddo

outer:do k=1,khtot(1)
         if (lrotvec(k)) then
             h=hamilton(1,k)
             do n=1,nspfdof(1)
                f=spfdof(n,1)
                ggdim=gdim(f) 
                if(basis(f)==12 .or. basis(f)==20 .or. basis(f)==21) ggdim = ipbaspar(3,f)
                if (basis(f) == 2) then
                   call cutrot(f,gdim(f),ggdim,trafo(zdvr(f)),ort(zort(f)),hops(zhop(h)),workr)
                else if (basis(f) == 7 .or. basis(f) == 20) then
                   call cutkrot(f,gdim(f),gdim(f+1),ggdim,trafo(zdvr(f)),ort(zort(f)),hops(zhop(h)),workr)
                endif
             enddo
             cycle outer
         endif

         if (lrotcoup(k)) then
             h=hamilton(fcoup(k),k)
             call cutcoup(hops(zhop(h)))
             cycle outer
         endif

!-----------------------------------------------------

         if (autoabs(k) > 0) then
            nabsf=nabsf+1
            f=autoabs(k)
            h=hamilton(f,k)
            call absdata(hops(zhop(h)),gdim(f),indiabs(f))
            cycle outer
         endif

!-----------------------------------------------------
         if (autoflux(k) > 0) then
            nfluxf=nfluxf+1
            f=autoflux(k)
            h=hamilton(f,k)
            call fluxdata(hops(zhop(h)),gdim(f),indiflux(f))
            cycle outer
         endif
      enddo outer

      return
      end subroutine initproprw

!-----------------------------------------------------
! Scale and shift the Hamiltonian
!-----------------------------------------------------

      subroutine scalh(mr)

use global
use dvrdatmod
use griddatmod
use operdef

use realwpmod


      implicit none

      real(dop), dimension(*), intent(inout)      :: mr
      integer(long)                               :: k,f,h


inner:do k=1,khtot(1)
         if ((autoabs(k) .gt. 0) .or. (autoflux(k) .gt. 0)) cycle inner
            do f=1,ndof
               h=hamilton(f,k)
               if (htmtype(h) /= 1) call scalehmat(f,h,k,mr(mrhops))
            enddo
      enddo inner

      return
      end subroutine scalh

! **********************************************************************
!
!  This subroutine scales the matrices that form the Hamiltonian
!  to get its eigenvalues between -1 and 1.
!
! **********************************************************************
      subroutine scalehmat(f,h,k,hops)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use runpropmod

use realwpmod

use operutil, only: getvgn


       implicit none

       integer(long)                                 :: vd,gd,nd,f,h!,g,n
       integer(long), intent(in)                     :: k
       real(dop), dimension(hopsdim), intent(inout)  :: hops

      select case (htmtype(h))

      case default
       routine='scalehmat'
       write(message,'(a,i3,a,3i5)') 'Unknown htmtype :',htmtype(h), &
                                    ',  h, k, f', h,k,f
      call errormsg

      case (1)!     unit matrix
      return

      case (2)!     real vector (diagonal matrix)
      if (lrotvec(k) .or. lrotcoup(k)) then
         call dscal(totphidim,aunds,hops(zhop(h)),onei)
      else 
         call getvgn(h,vd,gd,nd)
         if (gd == totphidim) call dscal(gd,aunds,hops(zhop(h)),onei) ! for scaling potential energy
      endif

      return

      case (8)!     real matrix 
      call getvgn(h,vd,gd,nd)
      call dscal1(hops(zhop(h)),vd,gd) ! for scaling all the real matrix

      return

      end select

      end subroutine scalehmat

!#######################################################################
      subroutine dscal1(a,dim1,dim2)

use realwpmod

      implicit none

      integer(long)                                 :: i
      integer(long), intent(in)                     :: dim1,dim2
      real(dop), dimension(dim2,dim2),intent(inout) :: a

     do i=1,dim2
        call dscal(dim2,aunds,a(1,i),1)
        if (dim1 == 1) a(i,i) = a(i,i) + bunds
     enddo

      return
      end subroutine dscal1

!#######################################################################

      subroutine cutcoup(hops)

use operdef
use psidef

      implicit none

      integer(long)                                  :: g
      real(dop)                                      :: npesmax
      real(dop), dimension(totphidim), intent(inout) :: hops

         npesmax=-pesmax
         do g=1,totphidim
            hops(g)=min(hops(g),pesmax)
            hops(g)=max(hops(g),npesmax)
         enddo

         return

        end subroutine cutcoup

!#######################################################################

      subroutine cutrot(f,gdim1,ggdim,trafo,ort,hops,pot)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use channels

      implicit none

      integer(long)                                  :: n,g,rdim,itel2,itel3
      integer(long), intent(in)                      :: f,gdim1,ggdim
      real(dop), dimension(totphidim), intent(in)    :: pot
      real(dop), dimension(totphidim), intent(inout) :: hops
      real(dop), dimension(ggdim,gdim1), intent(in)   :: trafo
      real(dop), dimension(gdim1), intent(in)         :: ort
      real(dop), dimension(gdim1)                     :: weight
      real(dop)                                      :: temp1,temp2


      itel3=0
      itel2=1

!-----------------------------------------------------------------------
! see adpop routine in analysis
! get weight
!-----------------------------------------------------------------------
      call dvrweights(trafo,ort,gdim1,&
           ggdim,weight,basis(f),rpbaspar(1,f),&
           ipbaspar(1,f),ipbaspar(1,f+1),.false.,.false.)

      rdim=totphidim/gdim1 ! try to do it in vgdim(f),gdim(f),ngdim(f) way, here rdim is vgdim(f)
outer:do n=1,rdim
         do g=1,gdim1
            temp1=pot(itel2)*weight(g)
            temp2=hops(itel2)
            if ((temp1+temp2) > pesmax) then
               temp1=temp1+temp2-pesmax
               hops(itel2)=temp2-temp1 
               itel3=itel3+1
            endif 
            itel2=itel2+1
         enddo
          
      enddo outer

      write (ilog,*) ' '
      write (ilog,*) ' The number of matrix elements that',&
                  ' were cut-off equals ',itel3
      write (ilog,*) ' '


      return
      end subroutine cutrot

!#######################################################################

      subroutine cutkrot(f,gdim1,kdim,ggdim,trafo,ort,hops,pot)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use channels

      implicit none

      integer(long)                                     :: i,n,g,k,rdim,itel2,&
                                                           itel3,zeig,zeig1
      integer(long), intent(in)                         :: f,kdim,gdim1,ggdim
      real(dop), dimension(totphidim), intent(in)       :: pot
      real(dop), dimension(totphidim), intent(inout)    :: hops
      real(dop), dimension(gdim1*ggdim*kdim), intent(in):: trafo
      real(dop), dimension(gdim1), intent(in)           :: ort
      real(dop), dimension(gdim1,kdim)                  :: weight
      real(dop)                                         :: temp1,temp2


!-----------------------------------------------------------------------
! see adpop routine in analysis
! get weight
!-----------------------------------------------------------------------
   zeig=1
   zeig1=1
   rdim=totphidim/(gdim1*kdim)
   do k=1,kdim
      call dvrweights(trafo(zeig),ort,gdim1,&
           ggdim,weight(1,k),basis(f),rpbaspar(1,f),&
           ipbaspar(1,f),ipbaspar(1,f+1),.false.,.false.)
      zeig=zeig+ggdim*gdim1
   enddo

   itel3=0
   itel2=1
       do k=1,kdim
         do g=1,gdim1
            do n=1,rdim ! vgdim(f)
               temp1=pot(itel2)*weight(g,k)
               temp2=hops(itel2)
               if ((temp1+temp2) > pesmax) then

                  temp1=temp1+temp2-pesmax
                  hops(itel2)=temp2-temp1
                  itel3=itel3+1

               endif
               itel2=itel2+1
            enddo 
         enddo

       enddo 

      write(ilog,'(/,72a1)')  ('#',i=1,72)
      write (ilog,*) ' The number of matrix elements that',&
                  ' were cut-off equals ',itel3
      write(ilog,'(/,72a1)')  ('#',i=1,72)

      return
      end subroutine cutkrot


!#######################################################################

      subroutine absdata(hops,gdim,indiabs)

use global, only:tiny

      implicit none

      integer(long)                          :: g
      integer(long), intent(in)              :: gdim
      integer(long), intent(out)             :: indiabs
      real(dop), dimension(gdim), intent(in) :: hops

!-----------------------------------------------------------------------
! see starting point of ABS at RHS < 1.0
!-----------------------------------------------------------------------
outer:do g=1,gdim
         if ((abs(1.0_dop - hops(g))) > tiny) then
            indiabs = g
            exit outer
         endif
      enddo outer

      return
      end subroutine absdata  


!#######################################################################

      subroutine fluxdata(hops,gdim,indiflux)

use global, only:tiny

      implicit none

      integer(long)                          :: g
      integer(long), intent(in)              :: gdim
      integer(long), intent(out)             :: indiflux
      real(dop), dimension(gdim), intent(in) :: hops

!-----------------------------------------------------------------------
! see the point where flux is 0.0
!-----------------------------------------------------------------------
outer:do g=1,gdim
         if ((abs(hops(g)) - 0.0_dop) < tiny) then
            indiflux = g
            exit outer
         endif
      enddo outer

      return
      end subroutine fluxdata 

!#######################################################################
!
! read the unformatted flux file 'filename' on unit 'iunit' to its nread
!
!#######################################################################

      subroutine fluxrd(iunit,filename,nread)

use global
use channels

      implicit none

      integer(long)                 :: ilbl,i,ios
      integer(long), intent(in)     :: iunit,nread
      character(len=c5), intent(in) :: filename

      do i=1,nread
         read(iunit,err=10,end=20,iostat=ios)
      enddo
      ilbl=index(filename,' ')-1
      write (ilog,'(2a)') filename(1:ilbl)//' file repositioned'

      return

10       write (*,*) ' Error reading file ',filename
         write (*,*) ' Error code is ',ios
         write (*,*) ' Point of error is entry ',i
         write (*,*) ' Number of points to be read is ',nread
         write (*,*) ' Error in fluxrd'
         return

20       write (*,*) ' Too few fluxpoints on file ',filename
         write (*,*) ' Number of points should be ',nread
         write (*,*) ' Number is ',i
         write (*,*) ' The iostat for this error is ',ios
         write (*,*) ' Error in fluxrd'
         return

      end subroutine fluxrd

!#######################################################################
!
!               WKINITPROP
!
! Here the workspace required for the module is assigned
!
! workcdim, workrdim, workidim and workldim are passed via a common
! block in global.inc, and are checked that they are large enough for
! the module.
!
!#######################################################################

      subroutine wkinitproprw

use global
use griddatmod
use runpropmod

      implicit none

!-----------------------------------------------------------------------
! assign workspace used in module
!-----------------------------------------------------------------------
      workcdim=workcdim
      workrdim=workrdim
      workidim=workidim
      workldim=workldim

      return
      end subroutine wkinitproprw

      end module initproprwmod

