!***********************************************************************
      module memproprwmod

use decimal!, only: dop, long
use globmemmod
use logdat

      implicit none
      private
      public :: memproprw,checkmemproprw
      contains

!***********************************************************************
!
!          MCTDH   subroutine MEMPROP
!
! called from main and auswert
! calls maxmatwk
!
! calculate pointers and dimensions for standard arrays.
!
!
! 10/96 aJ
!
!***********************************************************************

      subroutine memproprw

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use hpsimod
use dirdyn
use crocoflymod

use memhopsmod, only: memhops

      implicit none

!-----------------------------------------------------------------------
! with the parameter "pdum" the pointers to arrays that are not
! used are initialized with the value one in order to avoid an abortion
! of the program when array bounds are checked (compiler option "-c").
!-----------------------------------------------------------------------
      integer(long), parameter :: pdum = 1

!-----------------------------------------------------------------------
! assign standard arrays to memory arrays
! mcdim, mrdim, midim and mldim are final addresses assigned in mc, 
!  mr, mi and ml. 
!-----------------------------------------------------------------------
! set pointers to complex arrays in memory array mc 
!-----------------------------------------------------------------------
      mcdim=1
      mchpsi=mcdim
      mcdim=mchpsi+hpsidim + 1
      mchunpsi=mcdim
      mcdim=mchunpsi+totphidim + 1  
      mcintpsi=mcdim
      mcdim=mcintpsi+intpsidim + 1
!
      mcworkc = mcdim
      mcdim = mcworkc+workcdim + 1
!
!-----------------------------------------------------------------------
! set pointers to real arrays in memory array mr 
!-----------------------------------------------------------------------
      mrdim=1
      mrort=mrdim
      mrdim=mrort+ortdim + 1
      mrtrafo=mrdim
      mrdim=mrtrafo+dvrdim + 1
      mrfftp=mrdim
      mrdim=mrfftp+fftdim + 1
      mrgpop1=mrdim
      mrdim=mrgpop1+ortdim*nstate + 1
      mrgpop2=mrdim
      mrdim=mrgpop2+ortdim*nstate + 1
      if (lcdvr.or.ltdvr) then
         mrcdvr3=mrdim
         mrdim=mrcdvr3+d3matdim + 1
      else 
         mrcdvr3=mrdim
         mrdim = mrcdvr3 + 1
      endif
      if (ltraj .or. lgwp) then
         mrqcent = mrdim
         mrdim   = mrqcent + qcentdim + 1
      else
         mrqcent = mrdim
         mrdim   = mrqcent + 1
      endif
      if (lgwp) then
         mrqcent = mrdim
         mrdim   = mrqcent + qcentdim + 1
      else
         mrqcent = mrdim
         mrdim   = mrqcent + 1
      endif
      if (ltraj) then
         mrpcent = mrdim
         mrdim   = mrpcent + qcentdim + 1
      else
         mrpcent = mrdim
         mrdim   = mrpcent + 1
      endif
      mrintr = mrdim
      mrdim  = mrdim+intrdim + 1 
!     unused variables, but also checked in checkmemprop
      mrdvrmat = mrdim
      mrdim = mrdvrmat + 1
      mrkinsph = mrdim
      mrdim = mrkinsph + 1
!
!      mrppl=mrdim
!      mrdim=mrppl+ndof
!      mrpplt=mrdim
!      mrdim=mrpplt+mrdim

      mrworkr = mrdim
      mrdim = mrworkr+workrdim + 1

!-----------------------------------------------------------------------
! set pointers to integer arrays in memory array mi 
!-----------------------------------------------------------------------
      midim=1
      mijsph = midim
      midim  = mijsph+sphdim
      mimsph = midim
      midim  = mimsph+sphdim
      if (citype .gt. 0) then
         mijindx = midim
         midim = mijindx+adim
      else
         mijindx = pdum
      endif
      if (lcross) then
         miccjindx = midim
         midim = midim+ccjindxdim
      else
         miccjindx = pdum
      endif
      if (ltraj) then
         mitrajst = midim
         midim   = mitrajst + gwpinfodim + 1
      else
         mitrajst = midim
         midim   = mitrajst + 1
      endif
      miworki = midim
      midim   = miworki+workidim

!----------------------------------------------------------------------
! set pointers to logical arrays in memory array ml 
!-----------------------------------------------------------------------
! ml(1) is used in ABM to flag whether in predicotor or corrector step
      mldim=2
      mlworkl = mldim
      mldim   = mlworkl+workldim


!-----------------------------------------------------------------------
! now add space for operator data
!-----------------------------------------------------------------------
      call memhops

      return
      end subroutine memproprw                    ! of memprop

!***********************************************************************
!
!          subroutine checkmemproprw
!
! This routine checks the consistency of the mc,mr,mi,ml arrays for real wavepacket.
!
! Input:
!   check : true,  check the consitency  
!           false, set the control variables
!
!  03/2002 CC
!
!***********************************************************************
      subroutine checkmemproprw(check,mr,indx)

use global
use psidef
use datenmod
use runpropmod
use channels
use hpsimod
use crocoflymod
use mempropmod, only:wrmcchckmem,wrmrchckmem

      implicit none

      integer(long), parameter                       :: mrcount=12,micount=8,&
                                                        mlcount=3,mscount=2,mfcount=2
      real(dop), parameter                           :: realcheck = 3.45990253977358381_dop

      real(dop), dimension(mrdim), intent(inout)     :: mr

      integer(long)                                  :: pointer
      integer(long), intent(in)                      :: indx
      logical(kind=4), save                          :: called
      logical(kind=4), intent(in)                    :: check
      integer(long)                                  :: n
      integer(long), dimension(mrcount)              :: mrcheck

      mrcheck(1)=mrdim
      mrcheck(2)=mrort
      mrcheck(3)=mrdvrmat
      mrcheck(4)=mrtrafo
      mrcheck(5)=mrfftp
      mrcheck(6)=mrgpop1
      mrcheck(7)=mrgpop2
      mrcheck(8)=mrcdvr3
      mrcheck(9)=mrkinsph
      mrcheck(10)=mrhops
      mrcheck(11)=mrintr
      mrcheck(12)=mrworkr

      if (check) then
!    --- check the mr-array
         if (mr(mrcheck(1)).ne.realcheck) then
            call wrmrchckmem(mr(mrcheck(1)),realcheck,mrcheck(1), &
                             1,ilog,called)
            call wrmrchckmem(mr(mrcheck(1)),realcheck,mrcheck(1), &
                             1,6,called)
         endif
         do n=3,mrcount
            pointer = mrcheck(n)-1
            if (mr(pointer).ne.realcheck) then
               call wrmrchckmem(mr(pointer),realcheck,pointer, &
                             n,ilog,called)
               call wrmrchckmem(mr(pointer),realcheck,pointer, &
                             n,6,called)
            endif
         enddo
         if(called .and. indx.gt.0 ) then
            write(6,*)    'Stoped at:', indx
            write(ilog,*) 'Stoped at:', indx
            write (6,99) 'hpsi',mchpsi,hpsidim,mchpsi+hpsidim
            write (6,99) 'hunpsi',mchunpsi,totphidim,mchunpsi+totphidim
            write (6,99) 'intpsi',mcintpsi,intpsidim,mcintpsi+intpsidim
            write (6,99) 'oldpsi', mcoldpsi,dgldim,mcoldpsi+dgldim
            write (6,99) 'predpsi', mcpredpsi,totphidim, &
                                    mcpredpsi+totphidim
            write (6,99) 'autopsi',mcautodtpsi,dgldim,mcautodtpsi+dgldim
            write (6,99) 'hloch',mchloch,hlochdim,mchloch+hlochdim
            if (lcross) then
               write (6,99) 'psiref',mcpsiref,psirefdim, &
                                     mcpsiref+psirefdim
               write (6,99) 'ccdover',mcccdover,doverdim, &
                                   mcccdover+doverdim
            endif
            write (6,99) 'workc',mcworkc,workcdim,mcworkc+workcdim
            write (6,99) 'mcdim', mcdim
            write(6,*) lcross,lconstvar,lcmfadaa,lconst
  99        format(a10,3i10)
            stop
         endif


! --- Set the check variables
      else
         called=.false.
         mr(mrcheck(1)) = realcheck
         do n=3,mrcount
            mr(mrcheck(n)-1) = realcheck
         enddo

      endif

      return
      end subroutine checkmemproprw                      ! of checkmemprop

!***********************************************************************




      end module memproprwmod

