!######################################################################
!                                                                      
!                      MCTDH module OUTPUT                             
!                                                                      
! Controls the output of data.                                         
!                                                                      
!######################################################################
      module outputrwmod

use decimal, only: dop, long
use globmemmod
use logdat

      implicit none
      private
      public :: outputrw
      contains

!**********************************************************************
!                                                                      
!            MCTDH subroutine OUTPUT                                   
!                                                                      
! time  : actual time of output                                        
! psit1 : wavefunction at time                                         
! hpsit1: derivative of wavefunction at time i.e. H|psi>            
!                                                                      
!  V6.6 GW                                                             
!                                                                      
!**********************************************************************

      subroutine outputrw(time,laststep,psit1,hpsit1,mr,mi)

use global
use maxv, only: maxsta
use dvrdatmod
use griddatmod
use psidef
use runpropmod
use channels
use operdef
use datenmod

use realwpmod

use timing
use psioutrwmod, only:psioutrw
use calcdatrwmod, only: calcdatrw
use dichtoutrwmod, only: dichtoutrw
use daten, only: wrchck,speedout
use datoutrwmod, only: datoutrw

use op1lib, only: initvxd

use module_interface

      implicit none

      real(dop), dimension(*), intent(inout)       :: mr
      integer(long), dimension(*), intent(inout)   :: mi

      logical(kind=4), save                          :: firstcall = .true.
      integer(long)                                  :: n,s,itime
      integer(long), save                            :: it
      real(dop)                                      :: norm,rdum1
      real(dop), dimension(4,ndof,nstate)            :: qdq
      real(dop), dimension(1)                        :: rdum
      real(dop), intent(in)                          :: time
      real(dop), dimension(maxsta)                   :: spop,ecorr,etot,euncorr
      real(dop), dimension(nddstate)                 :: sspop
      real(dop), dimension(totphidim), intent(in)    :: psit1,hpsit1
      logical(Kind=4)                                :: laststep
      integer(long)                                  :: tid=0 ! timer ID

      if(test0) return

      call get_timer(tid, 'Output')
      call start_timer(tid)
     
!-----------------------------------------------------------------------
! write out psi file, if required
!-----------------------------------------------------------------------
      if (lpsi) then 
         if (abs(time-out2*nint(time/out2)) < (out2*1.0e-4_dop) .or. laststep) then
            call psioutrw(psit1)
         endif
      endif
!-----------------------------------------------------------------------
! Initiating dicht3
!-----------------------------------------------------------------------
      call initvxd(dicht3,d3matdim)

!-----------------------------------------------------------------------
! calculate energy, norm etc for later output
!-----------------------------------------------------------------------
      call calcdatrw(psit1,hpsit1,spop,ecorr,etot,euncorr)

!-----------------------------------------------------------------------
! calculate grid population data, and write gridpop file if required
!-----------------------------------------------------------------------
      call dichtoutrw(psit1,mr(mrgpop1),mr(mrgpop2),mr(mrtrafo),&
           mr(mrworkr),mi(mijsph),mi(mimsph),.true.)

!-----------------------------------------------------------------------
! write data to output file 
!-----------------------------------------------------------------------
      call datoutrw(time,spop,ecorr,etot,dicht3,mr(mrgpop1),&
           mr(mrgpop2),mr(mrort),mi(mijsph),mi(mimsph),qdq)
      
!-----------------------------------------------------------------------
! write data to check file 
!-----------------------------------------------------------------------
      call wrchck(spop,sspop,ecorr,etot,dicht3,qdq,norm,rdum(1),&
           rdum1)

!-----------------------------------------------------------------------
! write out expectation values
!-----------------------------------------------------------------------
      if (nexpect.gt.0) then
         if(firstcall) then
            itime=nint(10000.0_dop*out1/fs)
            if(mod(itime,10000).eq.0) then
               it=0
            elseif(mod(itime,1000).eq.0) then
               it=1
            elseif(mod(itime,100).eq.0) then
               it=2
            elseif(mod(itime,10).eq.0) then
               it=3
            else
               it=4
            endif 
         endif
         norm= 0.0
         do s=1,nstate
            norm = norm + spop(s)
         enddo
         norm = sqrt(norm/dble(npacket))
         if(lreonly) then
            if(it.eq.0) then
               write(iexpect,'(i9,1x,f11.8,1x,90(1pe16.8,1x))') &
                  nint(time/fs),norm,(dble(expvalue(n)),n=1,nexpect)
            elseif(it.eq.1) then
               write(iexpect,'(f9.1,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue(n)),n=1,nexpect)
            elseif(it.eq.2) then
               write(iexpect,'(f9.2,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue(n)),n=1,nexpect)
            elseif(it.eq.3) then
               write(iexpect,'(f9.3,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue(n)),n=1,nexpect)
            else
               write(iexpect,'(f9.4,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue(n)),n=1,nexpect)
            endif
         else
           if(it.eq.0) then
               write(iexpect,'(i9,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  nint(time/fs),norm,(expvalue(n),n=1,nexpect)
            elseif(it.eq.1) then
               write(iexpect,'(f9.1,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue(n),n=1,nexpect)
            elseif(it.eq.2) then
               write(iexpect,'(f9.2,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue(n),n=1,nexpect)
            elseif(it.eq.3) then
               write(iexpect,'(f9.3,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue(n),n=1,nexpect)
            else
               write(iexpect,'(f9.4,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue(n),n=1,nexpect)
            endif
         endif
         call myflush(iexpect)
      endif

      if (nexpect1.gt.0) then
         if(lreonly1) then
            if(it.eq.0) then
               write(iexpect1,'(i9,1x,f11.8,1x,90(1pe16.8,1x))') &
                  nint(time/fs),norm, &
                  (dble(expvalue1(n)),n=1,nexpect1)
            elseif(it.eq.1) then
               write(iexpect1,'(f9.1,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue1(n)),n=1,nexpect1)
            elseif(it.eq.2) then
               write(iexpect1,'(f9.2,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue1(n)),n=1,nexpect1)
            elseif(it.eq.3) then
               write(iexpect1,'(f9.3,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue1(n)),n=1,nexpect1)
            else
               write(iexpect1,'(f9.4,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue1(n)),n=1,nexpect1)
            endif
         else
           if(it.eq.0) then
               write(iexpect1,'(i9,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  nint(time/fs),norm,(expvalue1(n),n=1,nexpect1)
            elseif(it.eq.1) then
               write(iexpect1,'(f9.1,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue1(n),n=1,nexpect1)
            elseif(it.eq.2) then
               write(iexpect1,'(f9.2,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue1(n),n=1,nexpect1)
            elseif(it.eq.3) then
               write(iexpect1,'(f9.3,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue1(n),n=1,nexpect1)
            else
               write(iexpect1,'(f9.4,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue1(n),n=1,nexpect1)
            endif
         endif
         call myflush(iexpect1)
      endif

!-----------------------------------------------------------------------
! write out time spent since last output
!-----------------------------------------------------------------------
      if(lspeed) call speedout(time)

!-----------------------------------------------------------------------
! write restart file
!-----------------------------------------------------------------------

      firstcall = .false.

!-----------------------------------------------------------------------
! write out timing statistics
!-----------------------------------------------------------------------
      call stop_timer(tid)
      if (ltime) call timout

      return
      end subroutine outputrw

!#######################################################################

      end module outputrwmod

