! **********************************************************************
!                                                                      
!                   module PROP
!                                                                      
!  MCTDH module controlling the propagation of the wavefunction,       
!  i. e. calling the required evaluation and integration routines.     
!                                                                      
! **********************************************************************
      module proprw

use decimal, only: long, dop
use globmemmod
use logdat

      implicit none
      private
      public :: propagaterw
      contains

! **********************************************************************
! *** V6.2 MB *********> SUBROUTINE PROPAGATE PSI <*********************
!                       ~~~~~~~~~~~~~~~~~~~~~~~~~~
! Controls the propagation of the wavefunction.
! **********************************************************************

      subroutine propagaterw (psit1,hpsit1,psit2,prplt,prfluxt)

use maxv
use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use dirdyn
use channels

use realwpmod

use expectval, only: trexpectvalue
use functionrw, only: chebsqrt
use outputrwmod, only: outputrw
use memproprwmod, only: checkmemproprw
use proprw1, only: propexrw
use prop, only: zeitstop
use fluxrwmod, only: fluxrwp
!use probrwmod, only: probreal
use initproprwmod, only: fluxrd
use iorst, only:wrrstrw

      implicit none

      real(dop), dimension(totphidim), intent(inout) :: psit1,psit2
      real(dop), dimension(totphidim), intent(out)   :: hpsit1
      real(dop), dimension(nabsf), intent(inout)     :: prplt
      real(dop), dimension(nfluxf), intent(inout)    :: prfluxt

      real(dop)                                      :: time,timesinceoutput

      integer(long)                                  :: f,i,k,h,nextday,oldhour
      character(len=72)                              :: stopcommand

! For flux and probability calculation
integer(long)                            :: aerr,nout1,ntimestep,isave,itime,ios,nt,&
                                            ilbl,timsav,nfluxrd
real(dop), external                      :: dnrm2
real(dop), dimension(ndof)               :: prflux
real(dop)                                :: timeout
logical(kind=4)                          :: lold,laststep

character(len=19)                        :: prerr
integer(long), dimension(:), allocatable :: unitrf
character(len=c5)                       :: filename
character(len=10)                        :: stat


!.....Common block to ensure that the variables are saved.
!.....This common block also appears in subroutine zeitstop. 
      common /stopfile/ nextday, oldhour
      save /stopfile/

      if(allocated(unitrf))deallocate(unitrf)

      prerr = 'propagaterw_unitrf '
      allocate(unitrf(nfluxf+1),stat=aerr)
      if (aerr > 0) go to 1000

! --- CHECK ---

      if( tfinal .le. tinit + tiny ) then
         write(ilog,'(a)') '*******************************'
         write(iout,'(/a)') 'Nothing left to do!'
         write(ilog,'(/a)') 'Nothing left to do!'
         write(ilog,'(a,f15.6,a/,a,f15.6,a/)') &
         't-init =', tinit/fs,' fs','tfinal =',tfinal/fs,' fs'
         lstop = .false.
         goto 201
      end if

! --- INITIALIZE VARIABLES ---
      timesinceoutput = 0.0_dop
      ntimestep=0
      ntimfluxo=0
      isave    = 0
      itime     = 0
      laststep = .false.

! --- EVALUATE RIGHT HAND SIDE OF ODE WITH INITIAL WAVEFUNCTION ---
  
       call chebsqrt(tinit,psit1,hpsit1,psit2,mr)
!      timestep=aunds/dsqrt(1.0_dop-energ**2)

! --- WRITE INITIAL DATA AND WAVEFUNCTION ---
      lsave=.true. ! make it lflux
      if (.not. lcont) call outputrw(tinit,laststep,psit1,hpsit1,mr,mi)

! --- CHECK WHETHER RUN IS TEST RUN ---

      if (test) then
         write(iout,'(/,a,/)') 'Test run was successful.'
         write(ilog,'(/,a,/)') '***** Test run *****'
         write(ilog,'(a,/)')   'Test run completed.'
         goto 200
      endif

!###############################################################################################
! Check the number of variables for energy dependent probability calculation

      ntimestep=nint(tfinal/timestep)
      ntimfluxo=ntimestep/fluxsave
      if (prne > ntimestep/fluxsave) then
         prne=ntimestep/fluxsave
         write (ilog,*) 'Number of energy points set to maximum of ',prne
      endif

      nout1=nint(out1/timestep)
      timsav=nint(out2/timestep)
      out2=timestep*nint(out2/timestep)
      if (.not. lcont) tinit = tinit + 2.0_dop 
      timesinceoutput=timestep

!---Initializing files for flux and probability
      do f=1,nfluxf
         unitrf(f)=200+f-1
      enddo
! Initializing files for flux calculations! better to set all these in opendat, think!!
      do k=1,khtot(1)
         if (autoflux(k) > 0) then
            f = autoflux(k)
            ilbl=index(modelabel(f),' ')-1
            filename=name(1:laenge)//'/uflux_'//modelabel(f)(1:ilbl)
            if (lcont) then
               inquire(file=filename,exist=lold)
               stat = 'new'
               if(lold) stat = 'old'
               open(unitrf(f),file=filename, &
               form='unformatted',iostat=ios,status=stat,err=2000)
               write(ilog,'(2a)')'Uflux    file opened.  Status: ',stat
               if (lold) then
                  nfluxrd=((int(tinit)-1)/fluxsave)*2
                  if (fluxsave == 1) nfluxrd=nfluxrd-2
                  call fluxrd(unitrf(f),filename,nfluxrd)
               else
                  write(ilog,*) 'No uflux-file for continuation run.'
               end if
            else
               if (lovwr) then
                  stat='unknown'
               else
                  stat='new'
               endif
               open(unit=unitrf(f),file=filename,iostat=ios,err=2000,status=stat,form='unformatted')
               write(ilog,'(a)') 'Uflux       file opened'
            endif
         endif
      enddo


! --- Initialization ---

      if (timsav > 1) itime=mod((int(tinit)-1),timsav)
      if (fluxsave > 1) isave=mod((int(tinit)-1),fluxsave)
      lsave=.false. ! make it lflux

      if (int(tinit) == 2) then
         itime=1
         if (fluxsave > 1) isave=1
! for calculating the Flux
         do k=1,khtot(1)
            if (autoflux(k) > 0) then
               f = autoflux(k)
               h = hamilton(f,k)
               call fluxrwp(tinit,unitrf(f),f,h,mr(mrhops),psit1,prflux(f),prfluxt)
            endif
         enddo
      endif


! --- PROPAGATION LOOP ---
  do nt=int(tinit),ntimestep  
     if (nt .eq. ntimestep) laststep = .true.
     time=real(nt)

!     --- check consistency of mr,mi arrays, 
#ifdef DEBUG
      call checkmemproprw(.true.,mr,0)
#endif

! --- WRITE TIME TO INTSTEP FILE ---
! --- The stopcommand String is used when lstop is set true in integrat.F --
      stopcommand = 'stop  by internal check'
      if (lintstep) write(istp,'(a,f12.5)') 'Time in fs: ', time*timestep/fs

!-----------------------------------------------------
         lsave=.false.
         isave=isave+1
         if (isave == fluxsave) then
            isave=0
            lsave=.true.
         endif
!-----------------------------------------------------

      call propexrw(time,unitrf,psit1,hpsit1,psit2,prplt,prfluxt,mr)

! --- CALL OUTPUT ROUTINES AND LOOK FOR STOP COMMAND ---
      if ((abs(nout1*timestep-timesinceoutput) .lt. tiny*out1) .or. laststep) then
         timesinceoutput = 0.0_dop
! --- WRITE DATA (including RESTART file)
         timeout = (time-1.0)*timestep
 
         call outputrw(timeout,laststep,psit1,hpsit1,mr,mi) ! here psit1 is actually psit2 at time=time-1

!........dummy write to ilog in order to touch the log-file.
!        (do not write if psi=compact)
         if (.not. lpsiopt(2)) then
            write(ilog,'(a,f12.3)') ' time[fs] ', ((time-1.0)*timestep)/fs
         endif

!........flush data
         if(lspeed)  call myflush(ispeed)

         if(lstopf.and.(time-1.0).lt.(tfinal-0.1_dop*out1).and. .not.lstop) then
            open(istop,file=name(1:laenge)//'/stop', &
            form='formatted',status='old',err=666)
            read(istop,'(a)') stopcommand
!...........Go to the end of the stop-file and write the propagation
!...........time to it in order to 'touch' the stop-file.
            do i = 1, 21
                read(istop,'(a)')
            end do
            write(istop,'(f12.3)') ((time-1.0)*timestep)/fs
            close(istop)
            goto 667
! --- If the stop-file cannot be opened stop the calculation ---
 666        stopcommand = 'stop      Cannot open stop-file'
 667        if( stopcommand(1:4) .eq. 'cont' ) then
               nextday = 0
               oldhour = 0
            else if( stopcommand(1:4) .eq. 'stop' ) then
               lstop = .true.
            else if( stopcommand(3:3).eq.':' .or.  &
                     stopcommand(10:10).eq.':' ) then
               call zeitstop(stopcommand)
            end if
         end if
      endif

! --- UPDATE TIME VARIABLES ---
      timesinceoutput = timesinceoutput+timestep

! --- CONTINUE PROPAGATION ---
      if (lstop) go to 201
     enddo ! nt


! --- WRITE TIME TO INTSTEP FILE ---
      if (lintstep) write(istp,'(a,f12.5)') 'Time in fs: ', (time*timestep)/fs


!--------------------------------
      do f=1,nfluxf
         close(unitrf(f))
      enddo
!-------------------------------------------------------------------------------
! --- WRITE FINAL MESSAGES TO FILES ---
  201 continue
      write(iout,*)
      if (.not. lstop) then
         write(iout,'(a)') 'Propagation was successful.'
      else
         write(iout,'(a)') 'Propagation was stopped.'
         write(iout,'(a)') 'The stop-command reads:'
         write(iout,'(a)')  stopcommand
         write(iout,*)
      endif
      write(iout,*)
      call myflush(iout)

 200  continue

      prerr = 'propagaterw_prob...'
      deallocate(unitrf,stat=aerr)
      if (aerr > 0) go to 1001

      return

2000  write (*,*) ' Iostat = ',ios
      write (*,*) ' Error in proprw (Stop code 2000)'
      stop

 1000 write (6,*) 'Error in Memory Allocation of ',prerr,' Error code is ',aerr
      stop

 1001 write (6,*) 'Error in Memory Deallocation of ',prerr,' Error code is ',aerr
      stop

      end subroutine propagaterw

      end module proprw

