!#######################################################################
      module proprw1

use decimal, only: long, dop
use globmemmod
use logdat

      implicit none
      private
      public :: propexrw,energycrw
      contains

!#######################################################################
!
!  This subroutine performs the propagation of a wave packet in time.
!  For this it uses a Chebychev integration
!
!#######################################################################

      subroutine propexrw (time,unitrf,psit1,hpsit1,psit2,&
                           prplt,prfluxt,mr)

use global
use griddatmod
use psidef
use operdef
use runpropmod
use channels

use realwpmod

use functionrw, only: funkrwp
use absorbrwmod, only: absorbrwp
use fluxrwmod, only: fluxrwp

use iorst, only: wrrstrw


      implicit none

      real(dop), dimension(*), intent(inout)       :: mr

      integer(long), dimension(nfluxf), intent(in)      :: unitrf
      real(dop), dimension(nfluxf)                      :: prflux
      real(dop), dimension(nabsf)                       :: prpl
      real(dop), dimension(nfluxf), intent(inout)       :: prfluxt
      real(dop), dimension(nabsf), intent(inout)        :: prplt

      real(dop), external                              :: ddot
      real(dop)                                        :: norm,norm_all,pernot
      real(dop), intent(in)                            :: time
      real(dop), dimension(totphidim), intent(inout)   :: psit1
      real(dop), dimension(totphidim), intent(inout)   :: psit2
! keep in mind that here hpsit1 is actually hpsit2!!
      real(dop), dimension(totphidim), intent(out)     :: hpsit1

      integer(long), parameter                         :: m = 1, s = 1, nham = 1
      integer(long)                                    :: k,k1,k2,h,f
      logical(kind=4)                                  :: lprob,linwf


        norm=0.0_dop
        pernot=0.0_dop

! --- PROPAGATE MCTDH-COEFFICIENTS (FIRST HALF-STEP) ---

       call funkrwp(psit2,hpsit1,mr) ! psi(t2)---> hpsi
 
       call dscal(totphidim,-oner,psit1,onei) ! dscaling psi(t1)
       call daxpy(totphidim,2.0_dop,hpsit1,onei,psit1,onei) ! psi(t1) + H*psi(t2) ---> psi(t1) ! see Eq. 13, JCP 108,950(1998)

!---------------------------------------------------------------------------------------------
      if(khzahl(m,s,nham).eq.0) return
      k1=zhun(m,s,nham)
      k2=zhun(m,s,nham)+khzahl(m,s,nham)-1

! for calculating the Flux
      do k=k1,k2
         if (autoflux(k) > 0) then
            f = autoflux(k)
            h = hamilton(f,k)
            call fluxrwp(time,unitrf(f),f,h,mr(mrhops),psit1,prflux(f),prfluxt)
         endif
      enddo

! for absorbing the wavepacket
      do k=k1,k2
         if (autoabs(k) > 0) then
            f = autoabs(k)
            h = hamilton(f,k)
            lprob=.true.
            call absorbrwp(lprob,f,h,mr(mrhops),psit1,psit2,prpl(f),prplt) 
            lprob=.false.
            write (ilog,*) ' '
            write (ilog,'(a,i2,1x,f9.5)')' Total normalized (non)reaction probability (PL)',f,&
                                          (prplt(f)*100.0_dop)/(normwav**2),' %.'
            pernot=pernot+prplt(f)
         endif
      enddo


      call dswap(totphidim,psit1,onei,psit2,onei) ! psi(t1) <---> psi(t2)

!---------------------------------------------------
      if (lsave) then
         linwf=.false.
         call wrrstrw(time,psit1,psit2,prplt,prfluxt,linwf)
      endif 
      norm=norm+ddot(totphidim,psit2,onei,psit2,onei)
      norm_all=dsqrt(norm+pernot)
      write (ilog,'(a,f12.2,2x,f12.6,a,f12.6)') ' After step and time(fs) ',time,time*timestep/fs,&
                                               ', the norm of the wave function is ',norm_all
      write (ilog,'(a,f12.6)')' The difference with the initial norm is',&
                              dsqrt(dabs(norm_all**2-normwav**2))
      write (ilog,*) ' '


      return
      end  subroutine propexrw


! **********************************************************************
!
!  This subroutine calculates the energy of a wave packet
!
! **********************************************************************
      subroutine energycrw(rpsi,impsi,mr)

use global
use griddatmod
use operdef
use psidef
use runinwfmod
use runpropmod

use realwpmod

use functionrw, only: funkrwp

      implicit none

      real(dop), dimension(mrdim), intent(in)        :: mr

      integer(long), parameter                        :: m = 1, s = 1, nham = 1

      integer(long)                                   :: k,k1,k2,h
  
      complex(dop), external                          :: zdotc

      real(dop)                                       :: norm
      real(dop), dimension(totphidim)                 :: workrr,workrim
      real(dop), dimension(totphidim), intent(in)     :: rpsi,impsi
      complex(dop), dimension(totphidim)              :: psi1
      complex(dop), dimension(totphidim)              :: psi


      escatsav = 0.0_dop
      energsav = 0.0_dop

! keep in mind all these hamiltonian components are not scaled!!!! give a flag for this!!
      if (sbasis(ftr) /= 1) then
          message = 'Initial WF for translational mode must be gauss'
          call errormsg
      endif

      if(khzahl(m,s,nham).eq.0) return
      k1=zhun(m,s,nham)
      k2=zhun(m,s,nham)+khzahl(m,s,nham)-1
      do k=k1,k2
         h=hamilton(1,k)
         if (htmtype(h) == 8) then
            call energyscat(h,k,rpsi,escatsav,mr(mrhops))
            call energyscat(h,k,impsi,escatsav,mr(mrhops))
         endif
      enddo   

      call funkrwp(rpsi,workrr,mr)
      call funkrwp(impsi,workrim,mr)

      psi(:) = dcmplx(rpsi(:),impsi(:)) 
      psi1(:) = dcmplx(workrr(:),workrim(:)) 

      norm = dreal(zdotc(totphidim,psi,onei,psi,onei)) 

      if (norm > tiny) then
         energsav = dreal(zdotc(totphidim,psi,onei,psi1,onei))/norm
         escatsav = escatsav/norm
      else
         energsav = 0.0_dop
         escatsav = 0.0_dop
      endif

      return
      end subroutine energycrw

! **********************************************************************
!
!  This subroutine calculates the scattering energy of a wave packet
!
! **********************************************************************
      subroutine energyscat(h,k,psit1,escat,hops)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use runpropmod

use functionrw, only: hoprw

      implicit none

      real(dop), external                          :: ddot

      integer(long), intent(in)                    :: h,k
      real(dop), intent(inout)                     :: escat
      real(dop), dimension(hopsdim), intent(in)    :: hops
      real(dop), dimension(dvrdim)                 :: trafo
      real(dop), dimension(totphidim), intent(in)  :: psit1
      real(dop), dimension(totphidim)              :: workr

       call hoprw(onei,h,k,workr,psit1,hops,trafo,totphidim)
       escat = escat + ddot(totphidim,psit1,onei,workr,onei)

      return
      end subroutine energyscat

      
! **********************************************************************
      end module proprw1
