!-----------------------------------------------------------------------
!                        SUBROUTINE RUNPROP
!
!-----------------------------------------------------------------------

      subroutine runproprw

use global
use decimal, only: dop, long
use globmemmod
use logdat
use op1lib,  only: zerovxz, zerovxd, zerovxi, zerovxl
use maxv
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use channels 
use rddvrmod
use logintmod

use iorst, only: rstinforw,rdrstrw
use rdopermod, only: rdoper,operinfo
use zeigproprwmod, only:zeigproprw
use memproprwmod, only: memproprw,checkmemproprw
use initproprwmod, only:initproprw,scalh
use opendatmod, only: opendat
use proprw, only: propagaterw
use einintmod, only: einintreal
use realwpmod

use proprw1, only: energycrw

      implicit none

      integer(long)        :: ilbl,chkdvr,chkgrd,chkpsi,chkprp
!
! psi(dgldim) holds wavefunction
!
      integer(long)                              :: i,aerr
      character(len=80)                          :: actual
      character(len=16)                          :: cerr
      character(len=16)                          :: progpart

!      complex(dop), dimension(:), allocatable    :: psi
      real(dop), dimension(:), allocatable       :: psit1,psit2,hpsit1
      real(dop), dimension(:), allocatable       :: prplt,prfluxt

      character(len=c5)                          :: filename
      real(dop)                                  :: time
      real(dop), dimension(1)                    :: rdum,rdum1,rdum2
      complex(dop), dimension(1)                 :: zdum,zdum1,zdum2,zdum3
      integer(long), dimension(1)                :: idum,idum1,idum2
      logical(kind=4)                            :: linwf,lerr,check

      routine='Runproprealwp'

      aerr = 0

      call fzeit(actual)
      i = index(actual,':')
      actual=actual(i-9:i+6)
      write(ilog,'(/,72a1)')  ('#',i=1,72)
      write(ilog,'(/,a)') ' Propagating Real Wavefunction'
      write(ilog,'(57x,a)')  actual(1:15)
      write(ilog,'(72a1)')  ('#',i=1,72)

!-----------------------------------------------------------------------
! Check input file open for reading
!-----------------------------------------------------------------------
      inquire(iin,opened=check)
      if (check) close(iin)
      filename=inputfile(1:inplaenge)
      open(iin,file=filename,status='old',form='formatted',err=900)

!-----------------------------------------------------------------------
! get array dimensions 
!-----------------------------------------------------------------------
      inquire(irst,opened=check)
      if (check) close(irst)
      filename=name(1:laenge)//'/restart'
      ilbl=index(filename,' ')-1
      open(irst,file=filename(1:ilbl),form='unformatted',status='old',err=900)
      call rdmemdim(irst)
      close(irst)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      call alloc_psidef
      call alloc_realwpmod

!-----------------------------------------------------------------------
! Read system / DVR information
!-----------------------------------------------------------------------
      filename=dname(1:dlaenge)//'/dvr'
      ilbl=index(filename,' ')-1
      open(idvr,file=filename,form='unformatted',status='old',err=900)
      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      close(idvr)

!-----------------------------------------------------------------------
! Read data from oper file
!-----------------------------------------------------------------------
      filename=oname(1:olaenge)//'/oper'
      ilbl=index(filename,' ')-1
      open(ioper,file=filename,form='unformatted',status='old',err=900)
      chkdvr=2
      chkgrd=1
      call operinfo(lerr,chkdvr,chkgrd)
      close(ioper)

!-----------------------------------------------------------------------
! Read data from restart file
!-----------------------------------------------------------------------
      filename=rname(1:rlaenge)//'/restart'
      ilbl=index(filename,' ')-1
      open(irst,file=filename,form='unformatted',status='old', err=900)
      chkdvr=2
      chkgrd=2
      chkpsi=1
      chkprp=1
      call rstinforw(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
      close(irst)

!-----------------------------------------------------------------------
! Read integrator information
!-----------------------------------------------------------------------
      if (.not. lcont) then
         call einintreal
      endif

!-----------------------------------------------------------------------
! Close input file
!-----------------------------------------------------------------------
      close(iin)

!-----------------------------------------------------------------------
! check propagation variables
!-----------------------------------------------------------------------
      call chkproprw

      call zeigproprw
      call memproprw

      if(allocated(psit1))deallocate(psit1)
      if(allocated(hpsit1))deallocate(hpsit1)
      if(allocated(psit2))deallocate(psit2)
      if(allocated(mc))deallocate(mc)
      if(allocated(mr))deallocate(mr)
      if(allocated(mi))deallocate(mi)
      if(allocated(ml))deallocate(ml)
      if(allocated(prplt))deallocate(prplt)
      if(allocated(prfluxt))deallocate(prfluxt)
      if(allocated(dicht3))deallocate(dicht3)

!      cerr = 'runproprw_psi   '
!      allocate(psi(dgldim+1),stat=aerr)
!      if (aerr > 0) go to 1000
      cerr = 'runproprw_mc    '
      allocate(mc(mcdim+1),stat=aerr)
      if (aerr > 0) go to 1000
      cerr = 'runproprw_mr    '
      allocate(mr(mrdim+1),stat=aerr)
      if (aerr > 0) go to 1000
      cerr = 'runproprw_mi    '
      allocate(mi(midim+1),stat=aerr)
      if (aerr > 0) go to 1000
      cerr = 'runproprw_ml    '
      allocate(ml(mldim+1),stat=aerr)
      if (aerr > 0) go to 1000

      cerr = 'runproprw_psit1 '
      allocate(psit1(totphidim+1),stat=aerr)
      if (aerr > 0) go to 1000
      cerr = 'runproprw_hpsit1'
      allocate(hpsit1(totphidim+1),stat=aerr)
      if (aerr > 0) go to 1000
      cerr = 'runproprw_psit2 '
      allocate(psit2(totphidim+1),stat=aerr)
      if (aerr > 0) go to 1000
      cerr = 'runprop_dicht3 '
      allocate(dicht3(d3matdim),stat=aerr)
      if (aerr > 0) go to 1000

!-----------------------------------------------------------------------
! write memory usage to log
!-----------------------------------------------------------------------
      progpart='runprop'
      call logmem(progpart)

!-----------------------------------------------------------------------
! Zero memory arrays
!-----------------------------------------------------------------------
!      call zerovxz(psi,dgldim)
      call zerovxz(mc,mcdim)
      call zerovxd(mr,mrdim)
      call zerovxi(mi,midim)
      call zerovxl(ml,mldim)

      call zerovxd(psit1,totphidim)
      call zerovxd(hpsit1,totphidim)
      call zerovxd(psit2,totphidim)

!     initialise check variables in the arrays
      call checkmemproprw(.false.,mr,0)

!-----------------------------------------------------------------------
! Zero propagation variables
!-----------------------------------------------------------------------
      estart=0.0_dop
!-----------------------------------------------------------------------
! Read DVR data needed to propagate wavefunction (see rddvr.f for
! details)
!-----------------------------------------------------------------------
      dvrdata(:)=.false.
      dvrdata(1)=.true.
      dvrdata(2)=.true.
      dvrdata(4)=.false.
      dvrdata(5)=.false.
      dvrdata(6)=.false.
      dvrdata(7)=.false.
      dvrdata(8)=.false.
      dvrdata(9)=.false.
      dvrdata(11)=.false.
      dvrdata(12)=.false.
      chkdvr=1
      call rddvr(mr(mrort),mr(mrtrafo),rdum,rdum1, &
           zdum,zdum1,idum,zdum2,zdum3,idum1,idum2,rdum2,chkdvr)

!-----------------------------------------------------------------------
! Read data needed by the operator
!-----------------------------------------------------------------------
      chkdvr=2
      chkgrd=1
      call rdoper(mr(mrhops),chkdvr,chkgrd)

!-----------------------------------------------------------------------
! initialise any further information
!-----------------------------------------------------------------------
      complete(1) = .false.
      call initproprw(mr(mrhops),mr(mrtrafo),mr(mrort))

!-----------------------------------------------------------------------
      cerr = 'runproprw_prplt '
      allocate(prplt(nabsf+1),stat=aerr)
      if (aerr > 0) go to 1000
      cerr = 'runproprw_prflxt'
      allocate(prfluxt(nfluxf+1),stat=aerr)
      if (aerr > 0) go to 1000

      call zerovxd(prplt,nabsf)
      call zerovxd(prfluxt,nfluxf)

!-----------------------------------------------------------------------
! Get initial wavefunction
! for continuation, take lrlx and tinit from restart file.
! reset rname so that restart file is written to name directory
!-----------------------------------------------------------------------
      filename=rname(1:rlaenge)//'/restart'
      open(irst,file=filename,form='unformatted',status='old',err=900)
      chkdvr=2
      chkgrd=2
      chkpsi=1
      if (lcont) then
         chkprp=1
      else
         chkprp=0
      endif

      call rdrstrw(time,psit1,psit2,prplt,prfluxt,linwf,chkdvr, &
                   chkgrd,chkpsi,chkprp)
      close(irst)
      rname=name
      rlaenge=laenge

      if (lcont) then
         tinit=time+1.0_dop
      else
         inimom=dabs(rsbaspar(2,1,1))
         spread=1.0_dop/rsbaspar(3,1,1)**2
      endif

!-----------------------------------------------------------------------
!     CALCULATE ENERGY OF THE WAVE PACKET
!-----------------------------------------------------------------------

      if (.not. lcont) call energycrw(psit1,psit2,mr)
! need to change the energyc routine and after this call outputrw for 
! output at time=0
! for scaling the hamiltonian matrix
      call scalh(mr)

!-----------------------------------------------------------------------
! log propagation information   and  open output data files.
!-----------------------------------------------------------------------
      call logint
      call opendat

!-----------------------------------------------------------------------
! Propagate wavefunction
!-----------------------------------------------------------------------
!      if (.not. lcont) then
!         do j=zetf(1,1), dgldim
!            psit1(j) = dble(psi(j))
!            psit2(j) = dimag(psi(j))
!         enddo
!      endif

!      cerr = 'runproprw_...  '
!      deallocate(psi,mc,stat=aerr)
!      if (aerr > 0) go to 1001

      call propagaterw(psit1,hpsit1,psit2,prplt,prfluxt)

!----------------------------------------------------------------------
!     check the variables in the mc array
!----------------------------------------------------------------------

!       need to include later
!      call checkmemprop(.true.,0)

      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_realwpmod

      cerr = 'runproprw_...   '
      deallocate(psit1,hpsit1,psit2,prplt,prfluxt,dicht3,mr,mi,ml,stat=aerr)
      if (aerr > 0) go to 1001

      return
  900 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg

 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',aerr
      stop

 1001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',aerr
      stop

      end subroutine runproprw

!#######################################################################
!
! CHKPROPRW
!
! checks the settings for the variables controlling the propagation.
!
!#######################################################################

      subroutine chkproprw

use global
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use crocoflymod
use channels
use logdat

      implicit none

      real(dop)                      :: ratio

      routine = 'Chkprop  (Check for consistency)'

!-----------------------------------------------------------------------
! Check that nodiag is not set for system hamiltonian.
!-----------------------------------------------------------------------
      if(nodiag(1)) then
         message='The System Hamiltonian must not be '// &
              'generated with the "nodiag" keyword'
         call warnmsg
      endif

!-----------------------------------------------------------------------
! if tfinal missing, set to 0 to check rest of input
!-----------------------------------------------------------------------
      if (tfinal .le. -9.0e99_dop) then
         tfinal=0.0
         message='Missing keyword or invalid arguments: tfinal'
         call errormsg
      endif

!-----------------------------------------------------------------------
! set defaults for out1 and out2 if not input
!-----------------------------------------------------------------------
      if (abs(out1) .lt. tiny) out1=tfinal-tinit
      if (abs(out2) .lt. tiny) out2=out1
      if (louteverystep .and. abs(out2).lt.tiny)  out2=10.0_dop*fs
      if (louteverystep)  out1=out2
      if(rlaxnum.lt.0 .and. louteverystep) then
         write(6,'(/,2a,/)') '## WARNING: ',  &
          ' "tout=all" is meaningful for improved relaxation ONLY.'
         write(ilog,'(/,2a,/)') '## WARNING: ', & 
          ' "tout=all" is meaningful for improved relaxation ONLY.'
      endif

!-----------------------------------------------------------------------
! Special treatment for improved relaxation.
!-----------------------------------------------------------------------
      if(rlaxnum.ge.0)  then
         message='Improved Relaxation not allowed with Real WP.'
         call warnmsg
      endif

!-----------------------------------------------------------------------
!     Check for consistency
!-----------------------------------------------------------------------
      if (tfinal .le. tinit ) Then
         message = 'tfinal must be larger than tinit'
         call errormsg
      endif
      if (out1 .le. tiny ) Then
         message = 'tout must be positive'
         call warnmsg
      endif
      if (out2 .le. tiny ) Then
         message = 'tpsi must be positive'
         call warnmsg
      endif

      ratio = (tfinal-tinit)/out1
      if (abs(ratio-nint(ratio)) .gt. tiny*ratio) then
         write(6,'(a)') ' *** WARNING : inconsistent times ***'
         write(6,'(a,f13.6,/a,f13.6,/a,f13.6)') 'tinit  =', &
            tinit/fs, 'tfinal =', tfinal/fs, 'tpsi   =', out2/fs
         tfinal = tinit + NInt(Ratio+0.3_dop)*out1
         write(6,'(a,/a,f13.6,a)') &
        ' Ratio (tfinal-tinit)/tout must be an integer number', &
        ' tfinal changed to :', tfinal/fs,' fs'
         if(.not.test0) write(ilog,'(a,/a,f13.6,a)') &
        ' Ratio (tfinal-tinit)/tout must be an integer number', &
        ' tfinal changed to :', tfinal/fs,' fs'
      endif

      ratio = (tfinal-tinit)/out2
      if (abs(ratio-nint(ratio)) .gt. tiny*ratio .and. .not.lrealwp) Then
      message='Ratio (tfinal-tinit)/tpsi must be an integer number'
         call warnmsg
         write(6,'(a,f13.6,/a,f13.6,/a,f13.6)') 'tinit  =', &
            tinit/fs, 'tfinal =', tfinal/fs, 'tpsi   =', out2/fs
      endif
      ratio = out2/out1
      if (abs(ratio-nint(ratio)) .gt. tiny*ratio .and. .not.lrealwp) then
         message = 'Ratio tpsi/tout must be an integer number'
         call warnmsg
         write(6,'(a,f13.6,/a,f13.6)')'tout =',out1/fs,'tpsi =',out2/fs
      endif

      if( .not.lstopf .and. &
           (sstop.ne.' ' .or. scpu.ne.' ' .or. swall.ne.' ') ) then
         message = 'You must open the stop-file when specifying ' &
                   //'tstop or tcpu or twall.'
         call warnmsg
      end if

!-----------------------------------------------------------------------
! check consistency of input
!-----------------------------------------------------------------------
      if(loptcntrl) then
         if((irunprop .ne. 1) .or. lrlx .or. dentype.ne.0) then
            message = 'optcntrl requires full WF propagation'
            call warnmsg
         endif
      endif
 
!-----------------------------------------------------------------------
! check for DD 
!-----------------------------------------------------------------------
      if (ldd) then
         routine='Chkprop'
         message= 'Direct Dynamics Not allowed in Real WP Propagation'
         call warnmsg
      endif

!-----------------------------------------------------------------------
! After any call of 'warnmsg' the logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      if (lwarn) then
         message = 'Error in propagation input. STOP.'
         call errormsg
      endif

      return
      end subroutine chkproprw 


