!#######################################################################
      module zeigproprwmod

use decimal!, only: long
use globmemmod
use logdat

      implicit none
      private
      public :: zeigproprw
      contains

!#######################################################################
! Zeiger (i.e. pointer) routines for propagation step.
!#######################################################################

      subroutine zeigproprw

use global
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use channels
use hpsimod

use iopsidef, only: psidat,gwpdat
use crocofly, only: zeigcroco
use zeighpsimod, only: operflag!,zeighpsi,d2zeighpsi

      implicit none

      integer(long)       :: i!,m,s,s1,zeig,f,n

!-----------------------------------------------------------------------
! set various operator related flags (kfirst etc.)
!-----------------------------------------------------------------------
      do i=1,hamdim
         call operflag(i)
      enddo

!-----------------------------------------------------------------------
! get pointers for wavefunction. vdim, ndim, zpsi, etc. 
!-----------------------------------------------------------------------
      call psidat

!-----------------------------------------------------------------------
! set up pointers for hpsi operation (and for hteil, hloch etc)
!-----------------------------------------------------------------------
      hpsidim  = 0
      hlochdim = 0
      pmatdim  = 0      
      ghmatdim = 0
!      if (dentype.eq.0) then
!         do i=1,hamdim
!            call zeighpsi(i)
!         enddo
!      else if (dentype.eq.2) then
!         do i=1,hamdim
!            call d2zeighpsi(i)
!         enddo
!      endif
      
!-----------------------------------------------------------------------
! calculate maximum "block" size needed in matrizen routines
!-----------------------------------------------------------------------
!      maxblock = 0
!      do i=1,hamdim
!         call maxmatwk(i)
!      enddo

!-----------------------------------------------------------------------
! calculate size of intpsi array
!-----------------------------------------------------------------------
         if (inttypephi .eq. 1) then                   ! ABM
            intpsidim = max(intpsidim,totphidim*(intorderphi+1))
         elseif (inttypephi .eq. 2) then               ! BS
            intpsidim = max(intpsidim,totphidim*(intorderphi+2))
         elseif (inttypephi .eq. 3) then               ! SIL / CSIL
            intpsidim = max(intpsidim,totphidim*(intorderphi-1))
            if(lcap.or.ldissin) then                   ! CSIL: EigVec,Hess
               intpsidim = intpsidim + 2*(intorderphi+1)**2
            else                                       ! SIL: EigVal,Diag..
               intrdim = max(intrdim,4*(intorderphi+1))
            endif
         elseif (inttypephi .eq. 4) then
            intpsidim = max(intpsidim,totphidim*7)
         elseif (inttypephi .eq. 5) then
            intpsidim = max(intpsidim,totphidim*12)
         endif

!-----------------------------------------------------------------------
! obtain size needed for calculation of cross-correlation-function
!-----------------------------------------------------------------------
      if (lcross) call zeigcroco

!-----------------------------------------------------------------------
! calculate workspace
!-----------------------------------------------------------------------
      workcdim=1
      workrdim=1
      workidim=1
      workldim=1

      write(ilog,'(a,i11,f13.3,a)') &
        ' hopsdim  =',hopsdim,  hopsdim* 7.6294e-6_dop,' MB'
      write(ilog,'(a,i11,f13.3,a)') &
        ' dgldim   =', dgldim,   dgldim*7.6294e-6_dop,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' pmatdim  =', pmatdim, pmatdim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' hpsidim  =', hpsidim, hpsidim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' hlochdim =',hlochdim,hlochdim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f13.3,a)') &
        ' mfsumdim =',mfsumdim,mfsumdim*15.2588d-6,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' intpsidim=',intpsidim,intpsidim*15.2588e-6_dop,' MB'
      if(intrdim.ge.40)  write(ilog,'(a,i11,f13.3,a)') &
        ' intrdim  =',intrdim,intrdim*7.6294d-6,' MB'
      if(ikrylov.ge.40)  write(ilog,'(a,i11,f13.3,a)') &
        ' ikrylov  =',ikrylov,ikrylov*7.6294d-6,' MB'
      if(ikryc  .ge.40)  write(ilog,'(a,i11,f13.3,a)') &
        ' ikryc    =',ikryc,ikryc*15.2588d-6,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' workcdim =',workcdim,workcdim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' workrdim =',workrdim, workrdim*7.6294e-6_dop,' MB'
      if(workidim .gt. 100) &
       write(ilog,'(a,i11,f11.3,a)') &
        ' workidim =',workidim, workidim*3.8147e-6_dop,' MB'

      return
      end subroutine zeigproprw


!#######################################################################

      end module zeigproprwmod

