!***********************************************************************
      module calcdattrmod

use decimal
use logdat
use op1lib, only: unitqxz, zeromxd
    
      implicit none

! energies read from SHZ output files 
      real(dop) :: epot_shz,ekin_shz

      private
      public :: calcdattr
      public :: epot_shz,ekin_shz
      contains

!***********************************************************************
!
! GW Need to add pntdvr !!

      subroutine calcdattr(psi,gdof1,dim1,spop,ecorr,etot, &
                 euncorr,trajst,dicht3,nsta,dicht4)

use global
use maxv, only: maxdim,maxsta
use griddatmod
use psidef
use runpropmod
use operdef
use hpsimod

use potevalmod, only: poteval

      implicit none

      integer(long)                                     :: m,s,h,n,f,k,nham,e,nst, &
                                                           pottyp,si,sf,ierr
      integer(long), intent(in)                         :: gdof1,dim1,nsta
      integer(long), dimension(gwpinfodim), intent(in)  :: trajst
      real(dop)                                         :: v,rfac,ke
      real(dop), dimension(maxsta), intent(out)         :: spop,ecorr,euncorr,etot
      real(dop), dimension(maxdim)                      :: gpoint,ppoint
      real(dop), dimension(d3matdim), intent(out)       :: dicht3
      real(dop), dimension(maxsta,maxsta)               :: wmat
      real(dop), dimension(maxsta)                      :: vmat
      real(dop), dimension(1000)                        :: workr
      real(dop)                                         :: rsum
      complex(dop)                                      :: vz
      complex(dop), dimension(gdof1,dim1), intent(in) :: psi
      complex(dop), dimension(dmatdim), intent(out) :: dicht4

!-----------------------------------------------------------------------
! get no. of states
!-----------------------------------------------------------------------
      if (leb) then
         nst = nsta
      else
         nst = 1
      endif

!-----------------------------------------------------------------------
! calculate state populations
!-----------------------------------------------------------------------
      if (nst .gt. 1) then
         do s=1,nst
            rsum=0.0d0
            do e=1,dim1
               if (trajst(e) .eq. s) rsum=rsum+1.0d0
            enddo
            rsum = rsum/dble(dim1)

            dicht3(d3mat(meb,1)+s-1) = rsum
            call unitqxz(dicht4(dmat(meb,1)),nst)
         enddo
      endif

!-----------------------------------------------------------------------
! calculate energy expectation value 
! A trajectory calculation is treated as a single-set MCTDH run.
! Therefore nstate = 1 and spop(1) = 1.0 for all runs
!-----------------------------------------------------------------------
      spop(1)=1.0d0
      ecorr(1) = 0.0d0
      euncorr(1) = 0.0d0

      if (lrunshzagreb) then
! energies read from files and summed
         ecorr(1) = epot_shz
         euncorr(1) = ekin_shz

      else
         nham=1
         m=1
         do e=1,dim1
            ke = 0.0d0
            call zeromxd(wmat,maxsta,maxsta)

            do n=1,nspfdof(m)
               f=spfdof(n,m)
               gpoint(f)=dble(psi(n,e))
               ppoint(f)=dimag(psi(n,e))
            enddo

            do k=1,khtot(nham)
               if (kinop(m,k) .eq. 1) then
                  v=1.0d0
                  do n=1,nspfdof(m)
                     f=spfdof(n,m)
                     h=hamilton(f,k)
                     call poteval(h,rfac,gpoint,vz,ppoint,pottyp)
                     v=v*rfac
                  enddo
                  v=dble(koeff(k))*v

                  ke = ke + v
               else
                  f=spfdof(1,m)
                  h=hamilton(f,k)
                  si=ksi(k)
                  sf=ksf(k)

                  call poteval(h,v,gpoint,vz,ppoint,pottyp)
                  wmat(sf,si) = wmat(sf,si) + dble(koeff(k))*v
               endif
            enddo

            if (nst .gt. 1) then
               call dsyev('V','U',nsta,wmat,maxsta,vmat,workr,3*nsta,ierr)
               if (ierr .ne. 0) then
                  routine = 'Calcdattr'
                  message = 'Cannot diagonalize Wmat matrix'
                  call errormsg
               endif
               s = trajst(e)
            else
               vmat(1)=wmat(1,1)
               s = 1
            endif

            euncorr(1)=euncorr(1)+ke
            ecorr(1)=ecorr(1)+vmat(s)
         enddo
      endif

      euncorr(1) = euncorr(1)/dble(dim1)
      ecorr(1) = ecorr(1)/dble(dim1)
      etot(1) = ecorr(1) + euncorr(1)

      return
      end subroutine calcdattr

!#######################################################################
 
      end module calcdattrmod
