!######################################################################
      module datouttrmod

use versions
use decimal, only: long, dop

      implicit none
      private
      public :: datouttr
      contains

!######################################################################
!
!                      MCTDH module DATOUT
!
! writes data to output file
!
!    datout   : writes output file
!    outexp0, outexp1, outexp2, outexp3 : outputs expectation values for
!               the different basis set types.
!    wrdicht3o: writes the density matrix eigenvalues i.e. natural
!               orbital populations to output file.
!
! 10/96 AJ
!***********************************************************************

      subroutine datouttr(t,spop,ecorr,etot,gpop1,psi,qdq)

use global
use dvrdatmod
use griddatmod
use psidef
use runpropmod
use channels

use datoutmod, only: outexp0,outexp4

      implicit none

      real(dop), external                             :: fcputime,fsystime
      external                                        :: fpstartup

      integer(long)                                   :: s,m,i,n,f,version,&
                                                         release
      real(dop)                                       :: norm2,normt,ecorrt, &
                                                         etott,deltae,tcpu
      real(dop), intent(in)                           :: t
      real(dop), dimension(nstate), intent(in)        :: spop,ecorr,etot
      real(dop), dimension(ortdim,nstate), intent(in) :: gpop1
      real(dop), dimension(4,ndof,nstate), intent(out):: qdq

      complex(dop), dimension(dgldim), intent(in)     :: psi

      character(len=100)                              :: buffer
      logical(kind=4), save                           :: firstcall = .true.

!----------------------------------------------------------------------
! create headline and initialize variables
!----------------------------------------------------------------------
      if (firstcall) then
         if(lcont) then
            write(iout,*)
            write(iout,'(1x,a)') 'Continuation Run'
         else
            call getrevno(progver,version,release)
            write(iout,*)
            write(iout,'(79a)') ('*', i=1,79)
            write(iout,'(31x,a,i3)') 'MCTDH version',version
            write(iout,'(34x,a,i5)') 'Release ', release
            write(iout,'(79a)') ('*', i=1,79)
            write(iout,*)
         endif
         call fpstartup(buffer)
         write(iout,'(1x,a)') buffer(1:72)
         write(iout,'(1x,a)') name(1:laenge)
         if(runtitle(1:10) .ne. '          ' ) &
         write(iout,'(1x,a)') runtitle(1:78)

         write(iout,'(1x,a,i7)') 'Swarm of N_traj = ',dim(1,1)
         write(iout,'(78a)') ('-',i=1,78)
      endif
      write(iout,*)

!----------------------------------------------------------------------
! 1. write state populations, norm and energy expectation values
!----------------------------------------------------------------------
      norm2    = 0.0_dop
      ecorrt   = 0.0_dop
      etott    = 0.0_dop

      do s=1,nstate
         norm2 = norm2+spop(s)
         ecorrt= ecorrt+ecorr(s)
         etott = etott+etot(s)
      enddo

      normt  = sqrt(norm2/dble(max(npackts,npacket)))
      etott  = etott*ev/norm2
      ecorrt = ecorrt*ev/norm2
      if (firstcall) then
         firstcall = .false.
         if (abs(estart-0.0_dop) .lt. tiny) estart = etott
      end if
      deltae = etott - estart

!-----------------------------------------------------------------------
! Renormalize energy expectation values
!-----------------------------------------------------------------------
      tcpu = fcputime(0)+fsystime(0)
      write(iout,800)  t/fs,   tcpu,  normt
      write(iout,804) etott, ecorrt, deltae
      if (nstate.gt.1) then
         write(iout,*)
         do s=1,nstate
            write (iout,803)s,spop(s),ecorr(s)*ev,etot(s)*ev
         enddo
      endif

 800  format(' Time  =',f11.2,' fs,       CPU =',f11.2,' s,',3x, &
             ' Norm    =',f11.8)
 803  format(' state =',i2,'  pop.:',f11.8,3x,'E-corr:', &
             f11.6,'eV,    E-tot =',f11.6,'eV')
 804  format(' E-tot =',f11.6,' eV,     E-pot =',f11.6,'eV,',3x, &
             ' Delta-E =',3pf11.4,' meV')


!-----------------------------------------------------------------------
! loop over states and output:
! 2. natural orbital populations
! 3. if NO propagation check dicht2 is diagonal i.e. largest off
!    diagonal element is smaller than 1d-6
! 4. Output various expectation values, depending on basis type
!-----------------------------------------------------------------------
      do s=1,nstate

         if(nstate .eq. 1 ) then
         write(iout,'(/" Mode expectation values and variances :")')
         else
         write(iout,&
         '(/" Mode expectation values and variances,  state =", i2)') s 
         end if

         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (basis(f).eq.0) then
                  call outexp0(gpop1(zort(f),s),gdim(f))
               else
                  call outexp4(psi(zetf(m,s)),n,dim(m,s),nspfdof(m), &
                       iout,modelabel(f),spop(s),qdq(1,f,s))
               endif
            enddo
         enddo

      enddo
      write(iout,'(78a)') ('-',m=1,78)
      call myflush(iout)

      return
      end subroutine datouttr

      end module datouttrmod
