!######################################################################
!                                                                      
!                      MCTDH module OUTPUT                             
!                                                                      
! Controls the output of data.                                         
!                                                                      
!######################################################################
      module outputtrmod

use decimal, only: dop, long
use globmemmod
use logdat

      implicit none
      private
      public :: outputtr
      contains

!**********************************************************************
!                                                                      
!            MCTDH subroutine OUTPUT                                   
!                                                                      
! time  : actual time of output                                        
! psi   : wavefunction at time                                         
!                                                                      
!  V6.6 GW                                                             
!                                                                      
!**********************************************************************

      subroutine outputtr(time,psi)

use global
use maxv, only: maxsta
use dvrdatmod
use griddatmod
use psidef
use runpropmod
use channels
use operdef
use datenmod

use timing
use psioutmod, only: psiout
use calcdattrmod, only: calcdattr
use dichtoutmod, only: dichtout
use daten, only: wrchck,speedout
use datouttrmod, only: datouttr
use iorst, only: wrrst

use op1lib, only: cpvxd

use module_interface

      implicit none

      logical(kind=4), save                          :: firstcall = .true.
      integer(long)                                  :: idum,n,s,itime,nsta
      integer(long), save                            :: it
      real(dop)                                      :: norm,rdum1
      real(dop), intent(in)                          :: time
      real(dop), dimension(1)                        :: rdum
      real(dop), dimension(maxsta)                   :: spop,ecorr,etot,euncorr
      real(dop), dimension(nddstate)                 :: sspop
      real(dop), dimension(4,ndof,nstate)            :: qdq
      complex(dop), dimension(1)                     :: cdum
      complex(dop), dimension(dgldim), intent(inout) :: psi
      logical(kind=4)                                :: linwf
      integer(long)                                  :: tid=0 ! timer ID

      if(test0) return

      call get_timer(tid, 'Output')
      call start_timer(tid)
     
!-----------------------------------------------------------------------
! write out psi file, if required
!-----------------------------------------------------------------------
      if (lpsi) then 
         if (abs(time-out2*nint(time/out2)).lt.(out2*1.0e-4_dop)) then
            call psiout(psi,dicht4,time,mi(mijindx), &
                 cdum,mi(mitrajst),mi(migwpdep))
         endif
      endif

!-----------------------------------------------------------------------
! calculate energy, norm etc for later output
!-----------------------------------------------------------------------
      if (feb .gt. 0) then
         nsta=gdim(feb)
      else
         nsta=1
      endif
      call calcdattr(psi(zetf(1,1)),nspfdof(1),dim(1,1), &
           spop,ecorr,etot,euncorr,mi(mitrajst), &
           dicht3,nsta,dicht4)

!-----------------------------------------------------------------------
! calculate grid population data, and write gridpop file if required
!-----------------------------------------------------------------------
      call dichtout(psi,dicht3,mr(mrgpop1), &
           mr(mrgpop2),mr(mrtrafo),mi(mifftfak),mc(mcrueck), &
           mc(mcexphin),mc(mcexprueck),mi(mijsph),mi(mimsph),time, &
           mr(mrort),.true.,cdum,mi(mijindx),mi(mitrajst))

!-----------------------------------------------------------------------
! write data to output file 
!-----------------------------------------------------------------------
      qdq(:,:,:) = 0.0_dop
      call datouttr(time,spop,ecorr,etot,mr(mrgpop1),psi,qdq)
      
!-----------------------------------------------------------------------
! for single-set calculations extract state populations from gpop
!-----------------------------------------------------------------------
      if (leb) call cpvxd(mr(mrgpop1+zort(feb)-1),sspop,nddstate)

!-----------------------------------------------------------------------
! write data to check file 
!-----------------------------------------------------------------------
      norm=1.0_dop
      call wrchck(spop,sspop,ecorr,etot,dicht3,qdq,norm,rdum(1),rdum1)

!-----------------------------------------------------------------------
! write out expectation values
!-----------------------------------------------------------------------
      if (nexpect.gt.0) then
         if(firstcall) then
            itime=nint(10000.0_dop*out1/fs)
            if(mod(itime,10000).eq.0) then
               it=0
            elseif(mod(itime,1000).eq.0) then
               it=1
            elseif(mod(itime,100).eq.0) then
               it=2
            elseif(mod(itime,10).eq.0) then
               it=3
            else
               it=4
            endif 
         endif
         norm= 0.0
         do s=1,nstate
            norm = norm + spop(s)
         enddo
         norm = sqrt(norm/dble(npacket))
         if(lreonly) then
            if(it.eq.0) then
               write(iexpect,'(i9,1x,f11.8,1x,90(1pe16.8,1x))') &
                  nint(time/fs),norm,(dble(expvalue(n)),n=1,nexpect)
            elseif(it.eq.1) then
               write(iexpect,'(f9.1,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue(n)),n=1,nexpect)
            elseif(it.eq.2) then
               write(iexpect,'(f9.2,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue(n)),n=1,nexpect)
            elseif(it.eq.3) then
               write(iexpect,'(f9.3,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue(n)),n=1,nexpect)
            else
               write(iexpect,'(f9.4,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue(n)),n=1,nexpect)
            endif
         else
           if(it.eq.0) then
               write(iexpect,'(i9,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  nint(time/fs),norm,(expvalue(n),n=1,nexpect)
            elseif(it.eq.1) then
               write(iexpect,'(f9.1,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue(n),n=1,nexpect)
            elseif(it.eq.2) then
               write(iexpect,'(f9.2,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue(n),n=1,nexpect)
            elseif(it.eq.3) then
               write(iexpect,'(f9.3,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue(n),n=1,nexpect)
            else
               write(iexpect,'(f9.4,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue(n),n=1,nexpect)
            endif
         endif
         call myflush(iexpect)
      endif

      if (nexpect1.gt.0) then
         if(lreonly1) then
            if(it.eq.0) then
               write(iexpect1,'(i9,1x,f11.8,1x,90(1pe16.8,1x))') &
                  nint(time/fs),norm, &
                  (dble(expvalue1(n)),n=1,nexpect1)
            elseif(it.eq.1) then
               write(iexpect1,'(f9.1,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue1(n)),n=1,nexpect1)
            elseif(it.eq.2) then
               write(iexpect1,'(f9.2,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue1(n)),n=1,nexpect1)
            elseif(it.eq.3) then
               write(iexpect1,'(f9.3,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue1(n)),n=1,nexpect1)
            else
               write(iexpect1,'(f9.4,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm,(dble(expvalue1(n)),n=1,nexpect1)
            endif
         else
           if(it.eq.0) then
               write(iexpect1,'(i9,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  nint(time/fs),norm,(expvalue1(n),n=1,nexpect1)
            elseif(it.eq.1) then
               write(iexpect1,'(f9.1,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue1(n),n=1,nexpect1)
            elseif(it.eq.2) then
               write(iexpect1,'(f9.2,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue1(n),n=1,nexpect1)
            elseif(it.eq.3) then
               write(iexpect1,'(f9.3,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue1(n),n=1,nexpect1)
            else
               write(iexpect1,'(f9.4,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm,(expvalue1(n),n=1,nexpect1)
            endif
         endif
         call myflush(iexpect1)
      endif

!-----------------------------------------------------------------------
! write out time spent since last output
!-----------------------------------------------------------------------
      if(lspeed) call speedout(time)

!-----------------------------------------------------------------------
! write restart file
!-----------------------------------------------------------------------
      linwf=.false.             ! 
      idum=0                    ! just dummies
      rdum(1)=0.0_dop             !      "
      cdum(1)=(0.0_dop,0.0_dop)     !      "
      call wrrst(time,psi,linwf,cdum,idum,rdum,rdum,mi(mijindx), &
           cdum,mi(mitrajst))

      firstcall = .false.

!-----------------------------------------------------------------------
! write out timing statistics
!-----------------------------------------------------------------------
      call stop_timer(tid)
      if (ltime) call timout

      return
      end subroutine outputtr

!#######################################################################

      end module outputtrmod

