!#######################################################################
      module proptr1

use decimal, only: long, dop
use global
use logdat
use griddatmod
use operdef
use psidef
use runpropmod
use channels
use rklib
use verletlib

use integrattr, only: runrk5tr,runrk8tr,runbstr,runabmtr,runverlettr

      implicit none
      private
      public :: proptraj,writeintmethodtr
      contains

!#######################################################################
! *** V6.2 MB ***********> SUBROUTINE PROP VMF <************************
! Drives the propagation of the wavefunction in variable mean-field
! (VMF) mode.
!#######################################################################

      subroutine proptraj (psi,dtpsi,time,trystep,madestep,restartabm, &
                           restartbs,steps,repabmsteps,goodbssteps, &
                           badbssteps,smallbssteps)
     
      implicit none

      real(dop)                                      :: intperiod
      real(dop), intent(out)                         :: madestep
      real(dop), intent(in)                          :: trystep,time
      integer(long), intent(inout)                   :: steps,repabmsteps, &
                                                        goodbssteps,badbssteps,&
                                                        smallbssteps
      complex(dop), dimension(dgldim), intent(inout) :: psi,dtpsi
      logical(kind=4), intent(inout)                 :: restartabm,restartbs


! --- CALL INTEGRATION ROUTINE ---
! evaltype is not used
      intperiod = trystep
      if (inttyp .eq. 1) then
         call runabmtr(psi,dtpsi,intperiod,time,restartabm,steps, &
              repabmsteps)
         restartabm = .false.
      elseif (inttyp .eq. 2) then
         call runbstr(psi,dtpsi,intperiod,time,restartbs,goodbssteps, &
              badbssteps,smallbssteps)
         restartbs = .false.
      elseif (inttyp .eq. 4) then
         call runrk5tr(psi,dtpsi,intperiod,time)
      elseif (inttyp .eq. 5) then
         call runrk8tr(psi,dtpsi,intperiod,time)
      elseif (inttyp .eq. 10) then
         call runverlettr(psi,dtpsi,intperiod,time)
      else
         routine = 'proptraj'
         message = 'Illegal integration type'
         call errormsg
      endif
      madestep = trystep

      return
      end subroutine proptraj

! *** V6.4 MB ********> SUBROUTINE WRITE INT METHOD <*******************
!                      ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! Writes the integration method to log file.
!-------------------------------------------------------------------------------

      subroutine writeintmethodtr (abmsteps,repabmsteps,smallbssteps, &
                                 goodbssteps,badbssteps)

      implicit none
      
      integer(long), intent(in) :: abmsteps,repabmsteps,smallbssteps,&
                                   goodbssteps,badbssteps
                                   
! --- WRITE INTEGRATION METHOD TO LOG FILE ---

      write(ilog,*)
      write(ilog,'(a)') '***** Integration method *****'
      write(ilog,*)
      write(ilog,'(a)')&
         'Integration method used for the trajectories:'
      if (inttyp .eq. 1) then
         write(ilog,'(a)')&
           'Adams-Bashforth-Moulton (ABM) predictor-corrector.'
         write(ilog,'(a,I2)') 'Integration order: ',intord
         write(ilog,*)
         write(ilog,'(a,I7)')&
           'Total number of ABM steps:    ',abmsteps
         write(ilog,'(a,I7)')&
           'Number of repeated ABM steps: ',repabmsteps
         write(ilog,*)
      elseif (inttyp .eq. 2) then
         write(ilog,'(a)')&
           'Bulirsch-Stoer (BS) with polynomial extrapolation.'
         write(ilog,'(a,I2)') 'Optimal column: ',intord
         write(ilog,*)
         write(ilog,'(a,I7)')&
           'Number of successful large BS steps:       ',goodbssteps
         write(ilog,'(a,I7)')&
           'Number of failed but fixed large BS steps: ',badbssteps
         write(ilog,'(a,I7)')&
           'Total number of small BS steps:            ',smallbssteps
         write(ilog,*)
      elseif (inttyp .eq. 4) then
         write(ilog,'(a)')&
           'Runge-Kutta order 5 (RK5) from ODESIM'
         if(limportho) write(ilog,'(a)')'with improved orthogonality'
         write(ilog,*)
         write(ilog,'(a,I7)')&
           'Total number of RK5 steps:     ',rk5steps
         write(ilog,'(a,I7)')&
           'Number of accepted RK5 steps:  ',rk5accpt
         write(ilog,'(a,I7)')&
           'Number of rejected RK5 steps:  ',rk5rejct
         write(ilog,*)
      elseif (inttyp .eq. 5) then
         write(ilog,'(a)')&
           'Runge-Kutta order 8 (RK8) from ODESIM'
         if(limportho) write(ilog,'(a)')'with improved orthogonality'
         write(ilog,*)
         write(ilog,'(a,I7)')&
           'Total number of RK8 steps:     ',rk8steps
         write(ilog,'(a,I7)')&
           'Number of accepted RK8 steps:  ',rk8accpt
         write(ilog,'(a,I7)')&
           'Number of rejected RK8 steps:  ',rk8rejct
         write(ilog,*)
      elseif (inttyp .eq. 10) then
         write(ilog,'(a)')&
           'Velocity-Verlet Algorithm'
         write(ilog,*)
         write(ilog,'(a,I7)')&
           'Total number of steps taken:     ',versteps
         write(ilog,*)
      else
         routine = 'writeintmethod'
         message = 'Illegal integration type'
         call errormsg
      endif
      call myflush(ilog)

      return
      end subroutine writeintmethodtr

! **********************************************************************

      end module proptr1
