!-----------------------------------------------------------------------
!                        SUBROUTINE RUNPROP
!
!-----------------------------------------------------------------------

      subroutine runproptr

use global
use decimal, only: dop, long
use globmemmod
use logdat
use maxv
use dvrdatmod
use griddatmod
use psidef
use datenmod
use runpropmod
use channels 
use hpsimod
use compdatmod

use iorst, only: rdrst,rstinfo
use rdopermod, only: rdoper,operinfo
use rddvrmod, only: rddvr,dvrdata,dvrinfo
use operdef
use zeigproptrmod, only:zeigproptr
use mempropmod, only: checkmemprop
use memproptrmod, only: memproptr
use initpropmod, only: initprop
use opendatmod, only: opendat
use proptr, only: propagatetr
use einintmod, only: eininttraj
use op1lib,  only: zerovxz, zerovxd, zerovxi, zerovxl

use logintmod

use mpidata

      implicit none

      integer(long)        :: ilbl,chkdvr,chkgrd,chkpsi,chkprp,lanczord,iflag
!
! psi(dgldim) holds wavefunction
!
      integer(long), dimension(6)                :: dynbytes
      integer(long)                              :: i,aerr
      character(len=80)                          :: actual
      character(len=20)                          :: cerr
      character(len=16)        :: progpart

      complex(dop), dimension(:), allocatable    :: psi,dtpsi

      character(len=c5)                          :: filename
      real(dop)                                  :: time
      real(dop), dimension(1)                    :: rdum,rdum1,rdum2,rdum3
      complex(dop), dimension(1)                 :: zdum,zdum1,zdum2,zdum3
      integer(long), dimension(1)                :: idum,idum1,idum2
      logical(kind=4)                            :: linwf,lerr,check

      integer(long), external   :: integersize,realsize,complexsize,logicalsize

      routine='Runprop'

!-----------------------------------------------------------------------
!  MPI not yet implemented
!-----------------------------------------------------------------------
#ifdef MPI
      if(mpirank.ne.master)then
        return
      endif
#endif

      aerr = 0

      call fzeit(actual)
      i = index(actual,':')
      actual=actual(i-9:i+6)
      write(ilog,'(/,72a1)')  ('#',i=1,72)
      write(ilog,'(/,a)') ' Propagating Classical Wavefunction '
      write(ilog,'(57x,a)')  actual(1:15)
      write(ilog,'(72a1)')  ('#',i=1,72)

!-----------------------------------------------------------------------
! Check input file open for reading
!-----------------------------------------------------------------------
      inquire(iin,opened=check)
      if (check) close(iin)
      filename=inputfile(1:inplaenge)
      open(iin,file=filename,status='old',form='formatted',err=900)

!-----------------------------------------------------------------------
! get array dimensions 
!-----------------------------------------------------------------------
      inquire(irst,opened=check)
      if (check) close(irst)
      filename=name(1:laenge)//'/restart'
      ilbl=index(filename,' ')-1
      open(irst,file=filename(1:ilbl),form='unformatted',status='old',err=900)
      call rdmemdim(irst)
      close(irst)

!-----------------------------------------------------------------------
! Allocate mememory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      call alloc_psidef
      call alloc_dirdyn
      call alloc_runpropmod
      call alloc_compdat
      call alloc_hpsimod

!-----------------------------------------------------------------------
! Read system / DVR information
!-----------------------------------------------------------------------
      filename=dname(1:dlaenge)//'/dvr'
      ilbl=index(filename,' ')-1
      open(idvr,file=filename,form='unformatted',status='old',err=900)
      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      close(idvr)

!-----------------------------------------------------------------------
! Allocate memory for grid data
! Read data from oper file
!-----------------------------------------------------------------------
      inquire(ioper)
      filename=oname(1:olaenge)//'/oper'
      ilbl=index(filename,' ')-1
      open(ioper,file=filename,form='unformatted',status='old',err=900)
      chkdvr=2
      chkgrd=1
      call operinfo(lerr,chkdvr,chkgrd)
      close(ioper)

!-----------------------------------------------------------------------
! Read data from restart file
!-----------------------------------------------------------------------
      filename=rname(1:rlaenge)//'/restart'
      ilbl=index(filename,' ')-1
      open(irst,file=filename,form='unformatted',status='old',err=900)
      chkdvr=2
      chkgrd=2
      chkpsi=1
      if (lcont .and. .not. leinint) then
         chkprp=2
      else
         chkprp=0
      endif
      call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
      close(irst)

!-----------------------------------------------------------------------
! Read integrator information
!-----------------------------------------------------------------------
      if (.not. lcont .or. leinint) then
         call eininttraj
      endif

!-----------------------------------------------------------------------
! Close input file
!-----------------------------------------------------------------------
      close(iin)

!-----------------------------------------------------------------------
! check propagation variables
!-----------------------------------------------------------------------
      call chkproptr

      call zeigproptr
      call memproptr
      dynbytes(1) = dgldim + 1
      dynbytes(2) = dgldim + 1
      dynbytes(3) = mcdim + 1
      dynbytes(4) = mrdim + 1
      dynbytes(5) = midim + 1
      dynbytes(6) = mldim + 1

      if(allocated(psi))deallocate(psi)
      if(allocated(dtpsi))deallocate(dtpsi)
      if(allocated(mc))deallocate(mc)
      if(allocated(mr))deallocate(mr)
      if(allocated(mi))deallocate(mi)
      if(allocated(ml))deallocate(ml)
      if(allocated(dicht1))deallocate(dicht1)
      if(allocated(dicht2))deallocate(dicht2)
      if(allocated(dicht3))deallocate(dicht3)
      if(allocated(dicht4))deallocate(dicht4)

      cerr = 'runproptr_psi'
      allocate(psi(dgldim+1),stat=aerr)
      allocmemory=allocmemory + dgldim*complexsize()
      if (aerr > 0) go to 1000
      cerr = 'runproptr_dtpsi'
      allocate(dtpsi(dgldim+1),stat=aerr)
      allocmemory=allocmemory + dgldim*complexsize()
      if (aerr > 0) go to 1000
      cerr = 'runproptr_mc '
      allocate(mc(mcdim+1),stat=aerr)
      allocmemory=allocmemory + mcdim*complexsize()
      if (aerr > 0) go to 1000
      cerr = 'runproptr_mr '
      allocate(mr(mrdim+1),stat=aerr)
      allocmemory=allocmemory + mrdim*realsize()
      if (aerr > 0) go to 1000
      cerr = 'runproptr_mi '
      allocate(mi(midim+1),stat=aerr)
      allocmemory=allocmemory + midim*integersize()
      if (aerr > 0) go to 1000
      cerr = 'runproptr_ml '
      allocate(ml(mldim+1),stat=aerr)
      allocmemory=allocmemory + mldim*logicalsize()
      if (aerr > 0) go to 1000
      cerr = 'runproptr_dicht1  '
      allocate(dicht1(dmatdim+1),stat=aerr)
      allocmemory=allocmemory + dmatdim*complexsize()
      if (aerr > 0) go to 1000
      cerr = 'runproptr_dicht2  '
      allocate(dicht2(dmatdim+1),stat=aerr)
      allocmemory=allocmemory + dmatdim*complexsize()
      cerr = 'runproptr_dicht3  '
      allocate(dicht3(d3matdim+1),stat=aerr)
      allocmemory=allocmemory + d3matdim*realsize()
      if (aerr > 0) go to 1000
      cerr = 'runproptr_dicht4  '
      allocate(dicht4(dmatdim+1),stat=aerr)
      allocmemory=allocmemory + dmatdim*complexsize()
      if (aerr > 0) go to 1000

!-----------------------------------------------------------------------
! write memory usage to log
!-----------------------------------------------------------------------
      progpart='runprop'
      call logmem(progpart)

!-----------------------------------------------------------------------
! Zero memory arrays
!-----------------------------------------------------------------------
      call zerovxz(psi,dgldim)
      call zerovxz(dtpsi,dgldim)
      call zerovxz(mc,mcdim)
      call zerovxd(mr,mrdim)
      call zerovxi(mi,midim)
      call zerovxl(ml,mldim)
      dicht1(:) = 0.0_dop
      dicht2(:) = 0.0_dop
      dicht3(:) = 0.0_dop
      dicht4(:) = 0.0_dop

!     initialise check variables in the mc array
      call checkmemprop(.false.,0)

!-----------------------------------------------------------------------
! Zero propagation variables
!-----------------------------------------------------------------------
      estart=0.0_dop
!-----------------------------------------------------------------------
! Read DVR data needed to propagate wavefunction (see rddvr.f for
! details)
!-----------------------------------------------------------------------
      dvrdata(:)=.false.
      dvrdata(1)=.true.
      dvrdata(2)=.false.
      dvrdata(4)=.false.
      dvrdata(5)=.false.
      dvrdata(6)=.false.
      dvrdata(7)=.false.
      dvrdata(8)=.false.
      dvrdata(9)=.false.
      dvrdata(11)=.false.
      dvrdata(12)=.false.
      chkdvr=1
      call rddvr(mr(mrort),rdum,rdum1,rdum2,zdum,zdum1,idum,zdum2,&
           zdum3,idum1,idum2,rdum3,chkdvr)

!-----------------------------------------------------------------------
! Read data needed by the operator
!-----------------------------------------------------------------------
      chkdvr=2
      chkgrd=1
      call rdoper(mr(mrhops),chkdvr,chkgrd)

!-----------------------------------------------------------------------
! Get initial wavefunction
! for continuation, take lrlx and tinit from restart file.
! reset rname so that restart file is written to name directory
!-----------------------------------------------------------------------
      filename=name(1:laenge)//'/restart'
      open(irst,file=filename,form='unformatted',status='old',err=900)
      chkdvr=2
      chkgrd=2
      chkpsi=1
      iflag=1
      if (lcont) then
         chkprp=1
      else
         chkprp=0
      endif
      call rdrst(time,psi,linwf,chkdvr,chkgrd,chkpsi,chkprp, &
                 mc(mcintpsi),lanczord,mr(mrworkr),iflag,idum, &
                 zdum,mi(mitrajst))
      close(irst)
      if (lcont) tinit=time
      rname=name
      rlaenge=laenge

!-----------------------------------------------------------------------
! initialise any further information
!-----------------------------------------------------------------------
      complete(1) = .false.

!-----------------------------------------------------------------------
! log propagation information   and  open output data files.
!-----------------------------------------------------------------------
      call logint
      call opendat

!-----------------------------------------------------------------------
! End short test
!-----------------------------------------------------------------------
      if (test0) then
         write(iout,'(/,a,/)') 'Short Test run was successful.'
         write(ilog,'(/,a,/)') '***** Test run *****'
         write(ilog,'(a,/)')   'Short test run completed.'
         goto 600
      endif

!-----------------------------------------------------------------------
! Propagate wavefunction
!-----------------------------------------------------------------------
      call propagatetr(psi,dtpsi)

!----------------------------------------------------------------------
!     check the variables in the mc array
!----------------------------------------------------------------------
      call checkmemprop(.true.,0)

  600 continue ! End calculation
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_psidef
      call dealloc_dirdyn
      call dealloc_runpropmod
      call dealloc_compdat
      call dealloc_hpsimod

!-----------------------------------------------------------------------

      cerr = 'runproptr_...'
      deallocate(psi,dtpsi,dicht1,dicht2,dicht3,dicht4,mc,mr,mi,ml,stat=aerr)
      if (aerr > 0) go to 1001

      return
  900 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg

 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',aerr
      stop

 1001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',aerr
      stop

      end subroutine runproptr

!#######################################################################
!
! CHKPROPTR
!
! checks the settings for the variables controlling the propagation.
!
!#######################################################################

      subroutine chkproptr

use global
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use channels
use logdat
use hpsimod, only: gwpintm

      implicit none

      real(dop)                      :: ratio

      routine = 'Chkproptr (Check for consistency)'

!-----------------------------------------------------------------------
! Check that nodiag is not set for system hamiltonian.
!-----------------------------------------------------------------------
      if(nodiag(1)) then
         message='The System Hamiltonian must not be '// &
              'generated with the "nodiag" keyword'
         call warnmsg
      endif

!-----------------------------------------------------------------------
! if tfinal missing, set to 0 to check rest of input
!-----------------------------------------------------------------------
      if (tfinal .le. -9.0e99_dop) then
         tfinal=0.0
         message='Missing keyword or invalid arguments: tfinal'
         call errormsg
      endif

!-----------------------------------------------------------------------
! set defaults for out1 and out2 if not input
!-----------------------------------------------------------------------
      if (abs(out1) .lt. tiny) out1=tfinal-tinit
      if (abs(out2) .lt. tiny) out2=out1
      if (louteverystep .and. abs(out2).lt.tiny)  out2=10.0_dop*fs
      if (louteverystep)  out1=out2

!-----------------------------------------------------------------------
!     Check for consistency
!-----------------------------------------------------------------------
      if (tfinal .le. tinit ) Then
         message = 'tfinal must be larger than tinit'
         call errormsg
      endif
      if (out1 .le. tiny ) Then
         message = 'tout must be positive'
         call warnmsg
      endif
      if (out2 .le. tiny ) Then
         message = 'tpsi must be positive'
         call warnmsg
      endif

      ratio = (tfinal-tinit)/out1
      if (abs(ratio-nint(ratio)) .gt. tiny*ratio) then
         write(6,'(a)') ' *** WARNING : inconsistent times ***'
         write(6,'(a,f13.6,/a,f13.6,/a,f13.6)') 'tinit  =', &
            tinit/fs, 'tfinal =', tfinal/fs, 'tpsi   =', out2/fs
         tfinal = tinit + NInt(Ratio+0.3_dop)*out1
         write(6,'(a,/a,f13.6,a)') &
        ' Ratio (tfinal-tinit)/tout must be an integer number', &
        ' tfinal changed to :', tfinal/fs,' fs'
         if(.not.test0) write(ilog,'(a,/a,f13.6,a)') &
        ' Ratio (tfinal-tinit)/tout must be an integer number', &
        ' tfinal changed to :', tfinal/fs,' fs'
      endif

      ratio = (tfinal-tinit)/out2
      if (abs(ratio-nint(ratio)) .gt. tiny*ratio .and. .not.lrealwp) Then
      message='Ratio (tfinal-tinit)/tpsi must be an integer number'
         call warnmsg
         write(6,'(a,f13.6,/a,f13.6,/a,f13.6)') 'tinit  =', &
            tinit/fs, 'tfinal =', tfinal/fs, 'tpsi   =', out2/fs
      endif
      ratio = out2/out1
      if (abs(ratio-nint(ratio)) .gt. tiny*ratio .and. .not.lrealwp) then
         message = 'Ratio tpsi/tout must be an integer number'
         call warnmsg
         write(6,'(a,f13.6,/a,f13.6)')'tout =',out1/fs,'tpsi =',out2/fs
      endif

      if( .not.lstopf .and. &
           (sstop.ne.' ' .or. scpu.ne.' ' .or. swall.ne.' ') ) then
         message = 'You must open the stop-file when specifying ' &
                   //'tstop or tcpu or twall.'
         call warnmsg
      end if

!-----------------------------------------------------------------------
! check consistency of input
!-----------------------------------------------------------------------
      if(loptcntrl) then
         message = 'optcntrl requires WF propagation'
         call warnmsg
      endif
 
!-----------------------------------------------------------------------
! check that DD uses LHA integration for PE elements
!-----------------------------------------------------------------------
      if (ldd) then
         if (gwpintm(1) .ne. 1 .and. gwpintm(1) .ne. 2) then
            routine='Chkprop'
            message= 'Must use LHA scheme for DD matrix elements'
            call warnmsg
         endif
      endif

!-----------------------------------------------------------------------
! After any call of 'warnmsg' the logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      if (lwarn) then
         message = 'Error in propagation input. STOP.'
         call errormsg
      endif

      return
      end subroutine chkproptr 


!***********************************************************************

