!#######################################################################
      module zeigproptrmod

use decimal, only: long
use global
use globmemmod
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use channels
use hpsimod
use logdat

use iopsidef, only: psidat,gwpdat
use psiutil, only: maxmatwk
use crocofly, only: zeigcroco
use zeighpsimod, only: operflag,zeighpsi,d2zeighpsi
use function, only: wkfunction

      implicit none
      private
      public :: zeigproptr
      contains

!#######################################################################
! Zeiger (i.e. pointer) routines for propagation step.
!#######################################################################

      subroutine zeigproptr

      implicit none

      integer(long)       :: i

!-----------------------------------------------------------------------
! set various operator related flags (kfirst etc.)
!-----------------------------------------------------------------------
      do i=1,hamdim
         call operflag(i)
      enddo

!-----------------------------------------------------------------------
! get pointers for wavefunction. vdim, ndim, zpsi, etc. 
!-----------------------------------------------------------------------
      call psidat

!-----------------------------------------------------------------------
! see if direct dynamics calculation
! New version (Benjamin, December 2008)
!-----------------------------------------------------------------------
      if (ldd .and. lmult) then
         if (nmode .ne. 1) then
            routine="Zeigprop"
            message="Multi-set direct dynamics only possible with a "// &
                    "single particle"
            call errormsg
         endif
      else if (ldd .and. leb) then
         if (nmode .ne. 2) then
            routine="Zeigprop"
            message="Single-set direct dynamics only possible with a "// &
                    "double particle"
            call errormsg
         endif
      endif

!-----------------------------------------------------------------------
! set up pointers for hpsi operation (and for hteil, hloch etc)
!-----------------------------------------------------------------------
!      hpsidim  = 0
!      hlochdim = 0
!      pmatdim  = 0      
!      ghmatdim = 0
!      do i=1,hamdim
!         call zeighpsi(i)
!      enddo
      
!-----------------------------------------------------------------------
! set up pointers for direct dynamics
!-----------------------------------------------------------------------
!      call zeigdd

!-----------------------------------------------------------------------
! pointers for x-local arrays (needed for analysis)
!-----------------------------------------------------------------------
!      zeig = 1 
!      do s = 1,nstate 
!         do m = 1,nmode 
!            f = spfdof(1,m)  
!            if (basis(f) .eq. 26) then
!               zpcons(f)=zeig
!               zeig=zeig+gdim(f)*gdim(f)
!            else if (basis(f) .eq. 27) then
!               zpcons(f)=1
!            endif
!         enddo
!      enddo
!      pconsdim=zeig
           
!-----------------------------------------------------------------------
! calculate maximum "block" size needed in matrizen routines
!-----------------------------------------------------------------------
!      maxblock = 0
!      do i=1,hamdim
!         call maxmatwk(i)
!      enddo

!-----------------------------------------------------------------------
! calculate size of intpsi array
!-----------------------------------------------------------------------
      if (inttyp .eq. 1) then                  ! ABM
         intpsidim = max(intpsidim,dgldim*(intord+1))
      elseif (inttyp .eq. 2) then              ! BS
         intpsidim = max(intpsidim,dgldim*(intord+2))
      elseif (inttyp .eq. 4) then              ! RK5
         intpsidim = max(intpsidim,dgldim*7)
      elseif (inttyp .eq. 5) then              ! RK8
         intpsidim = max(intpsidim,dgldim*12)
      endif

!-----------------------------------------------------------------------
! obtain size needed for calculation of cross-correlation-function
!-----------------------------------------------------------------------
      if (lcross) call zeigcroco

!-----------------------------------------------------------------------
! calculate workspace
!-----------------------------------------------------------------------
      call wkproptr

      write(ilog,'(a,i11,f13.3,a)') &
        ' hopsdim  =',hopsdim,  hopsdim* 7.6294e-6_dop,' MB'
      write(ilog,'(a,i11,f13.3,a)') &
        ' dgldim   =', dgldim,   dgldim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' pmatdim  =', pmatdim, pmatdim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' hpsidim  =', hpsidim, hpsidim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' hlochdim =',hlochdim,hlochdim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f13.3,a)') &
        ' mfsumdim =',mfsumdim,mfsumdim*15.2588d-6,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' intpsidim=',intpsidim,intpsidim*15.2588e-6_dop,' MB'
      if(intrdim.ge.40)  write(ilog,'(a,i11,f13.3,a)') &
        ' intrdim  =',intrdim,intrdim*7.6294d-6,' MB'
      if(ikrylov.ge.40)  write(ilog,'(a,i11,f13.3,a)') &
        ' ikrylov  =',ikrylov,ikrylov*7.6294d-6,' MB'
      if(ikryc  .ge.40)  write(ilog,'(a,i11,f13.3,a)') &
        ' ikryc    =',ikryc,ikryc*15.2588d-6,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' workcdim =',workcdim,workcdim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' workrdim =',workrdim, workrdim*7.6294e-6_dop,' MB'
      if(workidim .gt. 100) &
       write(ilog,'(a,i11,f11.3,a)') &
        ' workidim =',workidim, workidim*3.8147e-6_dop,' MB'

      return
      end subroutine zeigproptr

!#######################################################################
!
! set minimum array sizes to check available workspace
!
!#######################################################################

      subroutine wkproptr

      implicit none

      workcdim=1
      workrdim=1
      workidim=1
      workldim=1

      call wkfunction

      return
      end subroutine wkproptr

!#######################################################################

      end module zeigproptrmod

