! **********************************************************************
!
!              module ADDHUNPHI
!                                                                       
!   Contains:                                                           
!     addhunphi:    Adds for all modes and states the action of the     
!                   uncorrelated Hamiltonian on the single-particle     
!                   functions to the time derivative vector.            
!     addhunphi1ms: Adds for one mode and state the action of the       
!                   uncorrelated Hamiltonian on the single-particle     
!                   functions to the time derivative vector.            
!                                                                       
! **********************************************************************
      module addhunphimod

use decimal
use op2lib, only: addmxxzo

      implicit none
      private
      public :: addhunphi,addhunphi1ms
      contains

! *** V6.0 MB *************> SUBROUTINE ADDHUNPHI <*********************
!                           ~~~~~~~~~~~~~~~~~~~~~~
! Adds for all modes and states the action of the uncorrelated
! Hamiltonian on the single-particle functions to the time derivative.
!
! **********************************************************************

      subroutine addhunphi (dtpsi,hunpsi)

!$ use omp_lib

use global
use griddatmod
use psidef
use openmpmod

use timing

      implicit none

      integer(long)                                      :: m,s
      integer(long), save                                :: tid=0
      complex(dop), dimension(dgldim), intent(inout)     :: dtpsi
      complex(dop), dimension(adim+1:dgldim), intent(in) :: hunpsi

      call get_timer(tid,'AddHunPhi')
      call start_timer(tid)

! --- LOOP OVER EACH MODE AND STATE ---

! GWR This can be improved by using collapse when we have a newer version of 
! openmp. 

!! omp parallel do num_threads(ompthread) private(m,s) &
!! omp if(lompthread .and. nmode.ge.4) 
      do m = 1,nmode
         if (idmode(m).eq.0 .and. m .ne. mpb) then
            do s = 1,nstate
               call addhunphi1ms(dtpsi(zetf(m,s)),hunpsi,m,s,dim(m,s),subdim(m))
            enddo
         endif
      enddo
!! omp end parallel do

      call stop_timer(tid)

      return
      end subroutine addhunphi

! *** V6.0 MB ************> SUBROUTINE ADDHUNPHI 1MS <******************
!                          ~~~~~~~~~~~~~~~~~~~~~~~~~~
! Adds for one mode and state the action of the uncorrelated
! Hamiltonian on the single-particle functions to the time derivative.
!
! **********************************************************************

      subroutine addhunphi1ms (dtpsi,hunpsi,m,s,dim1,subdim1)

use global
use griddatmod
use runpropmod
use psidef

      implicit none

      integer(long), intent(in)                            :: m,s,dim1,subdim1
      complex(dop), dimension(subdim1,dim1), intent(inout) :: dtpsi
      complex(dop), dimension(adim+1:dgldim), intent(in)   :: hunpsi

      if (gwpm(m)) return

! --- ADD ACTION OF UNCORRELATED HAMILTONIAN TO TIME DERIVATIVE ---

      call addmxxzo(hunpsi(zetf(m,s)),dtpsi,subdim(m),dim(m,s))

      return
      end subroutine addhunphi1ms

      end module addhunphimod
