!######################################################################
      module datoutmod

use versions
use decimal, only: long, dop
use xvlib,   only: xmxxdzo, xmxxzzo
use op1lib,  only: cpvxz, cpqxz, zerovxi, normvxz, trhxz
use op2lib,  only: sum2qxxz
use mtlib,  only: mtxxzz

use global
use dglobal
use keyunitsmod
use dvrdatmod
use griddatmod
use psidef
use operdef, only: ldissin
use runpropmod
use channels
use multilayer
use gapmod, only: wfposn,wfwidth
use logdat

use maxv,           only: maxdim
use lalib,          only: simtran1btz
use gwplib,         only: moment, gwp_param,diagonal_momentsd,diagonal_momentsz
use gh_elementsmod, only: dq2gwp_elements,dqgwp_elements
use gwptrj
use psiutil, only: spfovl
use openmpmod

      implicit none
      private
      public :: datout,wrdicht3o,outexp0,outexp1,outexp2,outexp3,outexp4,&
                outexp5,spawnprint,spawnmctdh
      contains

!######################################################################
!
!                      MCTDH module DATOUT
!
! writes data to output file
!
!    datout   : writes output file
!    outexp0, outexp1, outexp2, outexp3 : outputs expectation values for
!               the different basis set types.
!    wrdicht3o: writes the density matrix eigenvalues i.e. natural
!               orbital populations to output file.
!
! 10/96 AJ
!***********************************************************************

      subroutine datout(t,spop,ecorr,etot,dicht3,dicht2,gpop1,&
                 gpop2,ort,fftp,jsph,msph,psi,qdq,gfgwp,aigwp,zgwp,&
                 agmat,gs2,spoptr,ggp,normt,trace,etott)


      implicit none

      real(dop), external                             :: fcputime,fsystime
      external                                        :: fpstartup

      integer(long)                                   :: s,m,i,n,f,s1,&
                                                         version,release
      integer(long), dimension(sphdim), intent(in)    :: jsph,msph
      real(dop), intent(out)                          :: normt
      real(dop), intent(in)                           :: trace
      real(dop), intent(out)                          :: etott
      real(dop)                                       :: ecorrt,&
                                                         deltae,tcpu,&
                                                         etottcm,ecorrtcm,&
                                                         deltaecm,&
                                                         norm,norm2
      real(dop), intent(in)                           :: t
      real(dop), dimension(nstate), intent(inout)     :: spop,ecorr,etot
      real(dop), dimension(nstate)                    :: spopout
      real(dop), dimension(ngwptrjst), intent(in)     :: spoptr
      real(dop), dimension(d3matdim), intent(in)      :: dicht3
      real(dop), dimension(fftdim), intent(in)        :: fftp
      real(dop), dimension(ortdim), intent(in)        :: ort
      real(dop), dimension(ortdim,nstate), intent(in) :: gpop1,gpop2
      real(dop), dimension(4,ndof,nstate), intent(out):: qdq
      real(dop), dimension(gwpinfodim), intent(out)   :: ggp

      complex(dop), dimension(dmatdim), intent(in)    :: dicht2
      complex(dop), dimension(dgldim), intent(in)     :: psi
      complex(dop), dimension(gfgwpdim), intent(inout)  :: gfgwp
      complex(dop), dimension(aigwpdim), intent(in)  :: aigwp
      complex(dop), dimension(zgwpdim), intent(in)   :: zgwp
      complex(dop), dimension(agmatdim), intent(inout):: agmat
      complex(dop), dimension(gs2matdim), intent(in)  :: gs2
      complex(dop), dimension(dmatdim)                :: overlap
      complex(dop)                                    :: norm2z

      character(len=8)                                :: deleunit,outunit

      logical(kind=4), save                           :: firstcall = .true.

!----------------------------------------------------------------------
! create headline and initialize variables
!----------------------------------------------------------------------
      if (firstcall) then
         if(lcont) then
            write(iout,*)
            write(iout,'(1x,a)') 'Continuation Run'
         else
            call getrevno(progver,version,release)
            write(iout,*)
            write(iout,'(79a)') ('*', i=1,79)
            write(iout,'(31x,a,i3)') 'QUANTICS version',version
            write(iout,'(34x,a,i5)') 'Release ', release
            write(iout,'(79a)') ('*', i=1,79)
            write(iout,*)
         endif
         write(iout,'(1x,a)') trim(cstartup)
         write(iout,'(1x,a)') name(1:laenge)
         if(runtitle(1:10) .ne. '          ' ) &
         write(iout,'(1x,a)') trim(runtitle)
         write(iout,'(78a)') ('-',i=1,78)

         if (dentype .eq. 1) then
            if (lexact) then
               write(iout,*)
               write(iout,'(a)') ' Exact Density Matrix '
               write(iout,*)
            else
               write(iout,*)
               write(iout,'(a)') ' Type I Density Matrix '
               write(iout,*)
            endif
         else if (dentype .eq. 2) then
            write(iout,*)
            write(iout,'(a)') ' Type II Density Matrix '
            write(iout,*)
         endif

         if (dentype .gt. 0) then
            if (.not. lcont) then
                if (.not. lrlx) then
                   temperature0 = max(temperature0,0.0d0)
                   write(iout,'(a,f15.2,a)') ' Initial T = ',&
                       temperature0*kelvin,' Kelvin'
                else
                   write(iout,'(a)') ' Relaxation '
                endif
                if (ldissin) then
                   write(iout,'(a,f11.8,a)') ' Initial Trace = ',&
                       intrace,'   Open System'
                else
                   write(iout,'(a,f11.8,a)') ' Initial Trace = ',&
                       intrace,'   Closed System'
                endif
            endif
            write(iout,'(78a)') ('-',i=1,78)
         endif

        if(rlxtime .eq. 'au') write(iout,'(1x,a)') 'time-not-fs is set!'
        if(rlxunit .eq. 'au') write(iout,'(1x,a)') 'energy-not-ev is set!'

      endif
      write(iout,*)

!----------------------------------------------------------------------
! Set up energy and norm
!----------------------------------------------------------------------
      norm2    = 0.0_dop
      if (dentype .eq. 0) then
         do s=1,nstate
            norm2 = norm2+spop(s)
         enddo

      else if (dentype .eq. 1) then
         if (lexact) then
            do s = 1,nstate
               call normvxz(psi(dzetf(1,s,s)),norm,subdim(1))
               norm2=norm2+norm
            enddo
         else
            call normvxz(psi,norm2,adim)
         endif

      else if (dentype .eq. 2) then
         if (lrunml) then
            call spfovl(overlap,psi,psi,agmat,agmat,0)
            call normml(psi,norm2z,overlap)
            norm2=dble(norm2z)
         else
            call normvxz(psi,norm2,adim)
         endif
      endif
      normt = sqrt(norm2/dble(max(npackts,npacket)))
      if(lreflex) normt=normt/sqrt(2.0_dop)

      ecorrt   = 0.0_dop
      etott    = 0.0_dop
      do s=1,nstate
         ecorrt= ecorrt+ecorr(s)
         etott = etott+etot(s)
      enddo
      if (firstcall) then
         firstcall = .false.
         if (abs(estart-0.0_dop) .lt. tiny) estart = etott
      end if

      ecorr(:) = ecorr(:)/norm2
      etot(:) = etot(:)/norm2
      etott  = etott/norm2
      ecorrt = ecorrt/norm2
      deltae = etott - estart

      etott  = etott*ev/norm2
      ecorrt = ecorrt*ev/norm2
      deltae = deltae*ev/norm2
      etottcm  = etott*8065.54d0
      ecorrtcm = ecorrt*8065.54d0
      deltaecm = deltae*8065.54d0

!-----------------------------------------------------------------------
! Correct total norm /spops if initnorm not unity (spops only for output)
!-----------------------------------------------------------------------
      if (initnorm .gt. 0.0) then
         normt = normt / initnorm
         spopout(:) = spop(:) / (initnorm*initnorm)
      else
         spopout(:) = spop(:) 
      endif

!-----------------------------------------------------------------------
! Output expectation values
!-----------------------------------------------------------------------
!----------------------------------------------------------------------
! 1. write state populations, norm and energy expectation values
!
! output can only be in eV or au, whereas rlxunit can be anything.
!----------------------------------------------------------------------
      tcpu = fcputime(0)+fsystime(0)
      if (rlxunit .eq. 'au') then
         outunit = 'au'
         deleunit = 'mau'
      else
         outunit = 'eV'
         deleunit = 'meV'
      endif
      if (rlxtime .eq. 'K') then
         if (abs(t-0.0_dop) .lt. tiny) then
            temperature_psi = temperature0
            write(iout,801)  '  Infty    ',rlxtime,tcpu,normt
         else
            temperature_psi = kelvin/(2.0_dop*t)
            write(iout,800)  temperature_psi,rlxtime,tcpu,normt
         endif
      else
         write(iout,800)  t/fs,rlxtime,tcpu,normt
      endif
      write(iout,802) etott,outunit,ecorrt,outunit,deltae,deleunit
 ! Is energy in cm-1 needed? Make as an option?
 !     write(iout,804) etottcm,ecorrtcm,deltaecm
      if (dentype .gt. 0) write(iout,'(a,f11.8)') ' Tr(p) / Tr(p0) =',&
              trace/intrace
      if(lrlxt) write(iout,'(45x,a,1pe16.9)') 'normthermal =',normthermal
      if (nstate.gt.1) then
         write(iout,*)
         do s=1,nstate
            write (iout,803) s,spopout(s),ecorr(s)*ev,outunit,&
                             etot(s)*ev,rlxunit
         enddo
      endif

 800  format(' Time  =',f11.2,1x,a2,',       CPU =',f11.2,' s,',3x, &
             ' Norm    =',f11.8)
 801  format(' Time  =',a11,1x,a2,',       CPU =',f11.2,' s,',3x, &
             ' Norm    =',f11.8)
 802  format(' E-tot =',f11.6,1x,a2,',    E-corr =',f11.6,1x,a2,',',2x, &
             ' Delta-E =',3pf11.4,1x,a3)
 803  format(' state =',i2,'  pop.:',f11.8,3x,'E-corr:', &
             f11.6,1x,a2',   E-tot =',f11.6,1x,a2)
 804  format(' WMtot =',f9.2,1x,'cm-1,    WMcorr =',f9.2,1x,'cm-1,',2x, &
             ' DeltaWM =',f9.2,1x,'cm-1')

!-----------------------------------------------------------------------
! loop over states and output:
! 2. natural orbital populations
! 3. if NO propagation check dicht2 is diagonal i.e. largest off
!    diagonal element is smaller than 1d-6
! 4. Output various expectation values, depending on basis type
!-----------------------------------------------------------------------0
      do s=1,nstate
         
         if (lexact) then
            continue

         else if (.not. lrunml) then

            if (psitype .eq. 0) then
               if(nstate .eq. 1 ) then
                  write(iout,'(/" Natural weights *1000 :")')
               else
                  write(iout,'(/" Natural weights *1000,  state =", i2)') s
               end if
            else if (psitype .eq. 4) then
! In G-MCTDH, diagonal elements of density matrix are stored in d3mat as 
! natural weights not known
               if(nstate .eq. 1 ) then
                  write(iout,'(/" Diagonal Densities *1000 :")')
               else
                  write(iout,'(/" Diagonal Densities *1000,  state =", i2)') s
               end if
            endif
               
            if (dentype .eq. 1) then
               do m=1,nmode
                  f=spfdof(1,m)
                  call wrdicht3o(dicht3(dd3mat(m,s,s)),ddim(m,s,s), &
                       modelabel(f),nspfdof(m))
               enddo

            elseif (lBEL) then
              if (BELflags(14)==0) then
                do m=1,nmode
                   f=spfdof(1,m)
                   call wrdicht3o(dicht3(d3mat(m,s)),dim(m,s), &
                        modelabel(f),nspfdof(m))
                enddo
              else
                write(iout,'(/" skip wrdicht3o")')
              endif
            else
               do m=1,nmode
                  f=spfdof(1,m)
!    Do not output bound state values if a multi-set continuum calc
                  if (lmult .and. f .eq. febcont .and. s .le. elbnd) cycle

                  call wrdicht3o(dicht3(d3mat(m,s)),dim(m,s), &
                       modelabel(f),nspfdof(m))
               enddo
            endif
           
         else  ! multilayer
            
            call outputml(iout)

         endif

! output GGP populations of GWPs
         if (lgwp) then
            if (lweightggp) then
               if(nstate .eq. 1 ) then
                  write(iout,'(/" Gross Gaussian Populations *10 (weighted) :")')
               else
                  write(iout, &
            '(/" Gross Gaussian Populations *10 (weighted),  state =", i2)') s
               end if
            else
               if(nstate .eq. 1 ) then
                  write(iout,'(/" Gross Gaussian Populations *10 :")')
               else
                  write(iout, &
            '(/" Gross Gaussian Populations *10 ,  state =", i2)') s
               end if
            end if

            if (lBEL) then
              if (BELflags(14)==0) then
                do m=1,nmode
                   if (.not. gwpm(m)) cycle
                   f=spfdof(1,m)
                   call wrdicht31(dicht2(dmat(m,s)),gs2(gs2mat(m,s)),&
                        dim(m,s),modelabel(f),nspfdof(m),gwpdim(m,s),&
                        psi(zdgwp(m,s)),psitype,ggp(zgwpinfo(m,s)))
                enddo
              else
                write(iout,'(/" skip wrdicht31")')
              endif
            else
              do m=1,nmode
                 if (.not. gwpm(m)) cycle
                 f=spfdof(1,m)
                 if (lmult .and. f .eq. febcont .and. s .le. elbnd) cycle

                 call wrdicht31(dicht2(dmat(m,s)),gs2(gs2mat(m,s)),&
                    dim(m,s),modelabel(f),nspfdof(m),gwpdim(m,s),&
                    psi(zdgwp(m,s)),psitype,ggp(zgwpinfo(m,s)))
              enddo
            endif
         endif

! output DOF expectation values
         if(nstate .eq. 1 ) then
            write(iout,'(/" Mode expectation values and variances :")')
         else
            write(iout,&
                 '(/" Mode expectation values and variances,  state =", i2)') s 
         end if

         if (BELflags(14)/=0) write(iout,'(/" skip outexp5")')
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               if (lmult .and. f .eq. febcont .and. s .le. elbnd) cycle

               if (basis(f).eq.0 .or. basis(f) .eq. 10) then
                  if (lebgrid) then
                     call outexp1(gpop1(zort(f),s),gpop2(zort(f),s), &
                          spop(s),gdim(f),ort(zort(f)),basis(f),iout, &
                          modelabel(f),ipbaspar(1:,f),ipbaspar(1:,f+1),&
                          qdq(1,f,s))
                  else
                     call outexp0(gpop1(zort(f),s),gdim(f))
                  endif
               elseif (basis(f).eq.-1) then
                  continue
               elseif (ldvr(f)) then
                  call outexp1(gpop1(zort(f),s),gpop2(zort(f),s), &
                       spop(s),gdim(f),ort(zort(f)),basis(f),iout, &
                       modelabel(f),ipbaspar(1,f),ipbaspar(1,f+1),qdq(1,f,s))
               elseif (basis(f).eq.4) then
                  call outexp2(gpop1(zort(f),s),gpop2(zort(f),s), &
                        spop(s),gdim(f),ort(zort(f)), &
                        fftp(zfft(f)),iout,modelabel(f),qdq(1,f,s))
               elseif (basis(f).eq.6) then
                  call outexp3(gpop1(zort(f),s),spop(s),gdim(f), &
                        jsph(zsph(f)),msph(zsph(f)),iout, &
                        modelabel(f),modelabel(f+1),qdq(1,f,s),qdq(1,f+1,s))
               elseif (basis(f).eq.25) then
                 if (lBEL) then
                   if (BELflags(14)==0) then
                     call outexp5(gfgwp(zgfgwp(m,s,s)),aigwp(zaigwp(m,s,s)), &
                       zgwp(zzgwp(m,s,s)),agmat(zagmat(m,s)), & 
                       dicht2(dmat(m,s)),nspfdof(m),gwpdim(m,s),&
                       dim(m,s),agmat(zxivec(m,s)),modelabel(f),&
                       iout,n,psi(zdgwp(m,s)),psitype,qdq(1,f,s),&
                       agmat(zphase(m,s)))
                   endif
                 else
                  call outexp5(gfgwp(zgfgwp(m,s,s)),aigwp(zaigwp(m,s,s)), &
                       zgwp(zzgwp(m,s,s)),agmat(zagmat(m,s)), & 
                       dicht2(dmat(m,s)),nspfdof(m),gwpdim(m,s),&
                       dim(m,s),agmat(zxivec(m,s)),modelabel(f),&
                       iout,n,psi(zdgwp(m,s)),psitype,qdq(1,f,s))
                 endif
               elseif (basis(f) .eq. 27) then
                  call outexp4(psi(zetf(m,s)),n,dim(m,s),nspfdof(m), &
                       iout,modelabel(f),spop(s),qdq(1,f,s))
               endif
            enddo
         enddo

      enddo

!-----------------------------------------------------------------------
! output natural weights of off-diagonal DOs
!-----------------------------------------------------------------------
      if (dentype .eq. 1 .and. .not. lexact) then
         do s=1,nstate
            do s1=1,nstate

               if (s .eq. s1) cycle

               write(iout,'(/" Natural weights *1000,  states =", 2i2)') s1,s

               do m=1,nmode
                  f=spfdof(1,m)
                  call wrdicht3o(dicht3(dd3mat(m,s1,s)),ddim(m,s1,s), &
                       modelabel(f),nspfdof(m))
               enddo
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! gwptrj state populations
!-----------------------------------------------------------------------
      if (lgwptrj) then
         write(iout,'(/" State Populations :",8f8.5)') (spoptr(s),s=1,ngwptrjst)
      endif

      write(iout,'(78a)') ('-',m=1,78)
      call myflush(iout)

!-----------------------------------------------------------------------
! Wavefunction position and widths saved if using DD-MCTDH.
!-----------------------------------------------------------------------
      if(lddmctdh)then
         wfposn  = 0.0_dop
         wfwidth = 0.0_dop
         do s = 1,nstate
            do f = 1,ndof
               if(f.eq.feb)cycle
               wfposn(f,s) = qdq(1,f,s)
               wfwidth(f,s) = qdq(2,f,s)
            enddo
         enddo
      endif

      return
      end subroutine datout

!#######################################################################
! OUTEXP0
!
! for time independent basis, output population of natural orbitals
!
! V6.0 GW
!#######################################################################

      subroutine outexp0(gpop1,gdim)

      implicit none

      integer(long)                          :: g
      integer(long), intent(in)              :: gdim
      real(dop), dimension(gdim), intent(in) :: gpop1

      if (gdim.le.8) then
         write(iout,'(a,8f8.5)')&
               'population : ',(gpop1(g),g=1,gdim)
      else
         write(iout,'(a,8f8.5)')&
               'population : ',(gpop1(g),g=1,8)
         write(iout,'(13x,8f8.5)') (gpop1(g),g=9,gdim)
      endif

      return
      end subroutine outexp0

!***********************************************************************
! OUTEXP1
!
! for DVR basis output: 
!   q(1): expectation value of the coordinate operator, <x>
!   q(2): <x**2>
!   occ(1): mean value of the FBR basis occupation number, <n>
!   occ(2): <n**2>
!
!  In case of Legendre-DVR (basis=2) occ(1) and occ(2) are the
!  mean value and the mean value of the square of the rotational
!  quantum number n which only for ibleg=0 and blz=0 is identical
!  to the FBR basis numbers !!!
!  For no symmetry (ibleg=0) the Legender-FBR basis functions correspond
!  to n=blz,blz+1,...,blz+gdim-1, for symmetry (ibleg=1,2 i.e n=odd/even) 
!  to n=nmin,nmin+2,...,nmin+2*(gdim-1), where nmin is the initial 
!  value of the n (depending from blz and ibleg): nmin=blz+mod(blz+ibleg,2) 
!
!  If the grid-length is 2*PI and the DVR is exp, then the expectation
!  values and variances are evaluated via the expectations <sin> and 
!  <cos>. (HDM 02/02)
!
! 01/98 AJ    
!***********************************************************************

      subroutine outexp1(gpop1,gpop2,spop,gdim,ort,basis,iout, &
                 modelabel,ipbaspar,ipbaspar1,qdq)

      implicit none

      integer(long)                           :: g,ibleg,blz,xmin
      integer(long), intent(in)               :: gdim,basis,iout
      real(dop), dimension(4), intent(out)    :: qdq
      integer(long), dimension(*), intent(in) :: ipbaspar,ipbaspar1
      real(dop)                               :: norm,dp,pi2,x,xvar,nvar,&
                                                 xstd,nstd
      real(dop), intent(in)                   :: spop
      real(dop), dimension(2)                 :: q,occ
      real(dop), dimension(*), intent(in)     :: gpop2
      real(dop), dimension(gdim), intent(in)  :: gpop1,ort
      character(len=*), intent(in)            :: modelabel
      logical(kind=4)                         :: l2pi

      l2pi = .false.
      pi2 = 8.0_dop*atan(1.0_dop)
      blz=0
      ibleg=0
      dp=0.0
      if (basis .eq. 2 .or. basis .eq. 12) then !  Leg, Leg/R
         blz=ipbaspar(1)
         ibleg=ipbaspar(2)
      else if (basis .eq. 7 .or. basis .eq.20) then  !  KLeg, KLeg/R
         blz=ipbaspar1(3)
         ibleg=ipbaspar(1)
      else if (basis .eq. 9 .or. basis .eq.21) then  !  PLeg, PLeg/R
         blz=0
         ibleg=ipbaspar(1)
      else if (basis .eq. 33) then              !  Wigner
         blz=max(ipbaspar(5),ipbaspar(9))       !  ???
         ibleg=ipbaspar(1)
      else if (basis .eq. 5) then               !  exp-DVR
         dp = pi2/(gdim*(ort(2)-ort(1)))
         if(abs(ort(gdim)+ort(2)-ort(1)-pi2) .lt. 1.0e-8_dop) l2pi=.true.
      endif

      q(1)=0.0_dop
      q(2)=0.0_dop
      occ(1)=0.0_dop
      occ(2)=0.0_dop

      if (l2pi) then             ! FFT or exp-DVR, 2pi-periodic
         do g=1,gdim
            q(1)=q(1)+sin(ort(g))*gpop1(g)
            q(2)=q(2)+cos(ort(g))*gpop1(g)
            x=dble(g-1)
            occ(1)=occ(1)+x*gpop2(g)
            occ(2)=occ(2)+x*x*gpop2(g)
         enddo
      else if (basis .eq. 12 .or. basis .eq.20 .or.  &
               basis .eq.21) then !  Leg/R, KLeg/R, PLeg/R 
         do g=1,gdim
            x=ort(g)
            q(1)=q(1)+x*gpop1(g)
            q(2)=q(2)+x*x*gpop1(g)
         enddo
         do g=1,ipbaspar(3)
            if (ibleg.eq.0) then
               x=dble(g-1+blz)
            else
               xmin = blz + mod(blz+ibleg,2)
               x=dble(2*(g-1)+xmin)
            endif
            occ(1)=occ(1)+x*gpop2(g)
            occ(2)=occ(2)+x*x*gpop2(g)
         enddo
      else if (basis.eq.2.or.basis.eq.7.or.basis.eq.9 .or. &
              basis .eq. 33) then  ! Leg,KLeg,PLeg,Wigner
         do g=1,gdim
            x=ort(g)
            q(1)=q(1)+x*gpop1(g)
            q(2)=q(2)+x*x*gpop1(g)
            if (ibleg.eq.0) then
               x=dble(g-1+blz)
            else
               xmin = blz + mod(blz+ibleg,2)
               x=dble(2*(g-1)+xmin)
            endif
            occ(1)=occ(1)+x*gpop2(g)
            occ(2)=occ(2)+x*x*gpop2(g)
         enddo
      else                      !  HO, rHO, sin, Sinc, Wsinc, exp, ...
         do g=1,gdim
            x=ort(g)
            q(1)=q(1)+x*gpop1(g)
            q(2)=q(2)+x*x*gpop1(g)
            x=dble(g-1)
            occ(1)=occ(1)+x*gpop2(g)
            occ(2)=occ(2)+x*x*gpop2(g)
         enddo
      end if

!-----------------------------------------------------------------------
! Renormalize expectation values with the population of their state 
!-----------------------------------------------------------------------
      norm=spop+1.0e-16_dop
      q(1)   = q(1)/norm
      q(2)   = q(2)/norm
      occ(1) = occ(1)/norm
      occ(2) = occ(2)/norm

!-----------------------------------------------------------------------
! Evaluate mean value and variance for periodic grid.
!-----------------------------------------------------------------------
      if(l2pi)  then
         x = sqrt(q(1)*q(1)+q(2)*q(2))
         if (x .gt. 1.0e-5_dop) then
            q(1) = atan2(q(1),q(2))
            if( q(1) .lt. 1.5_dop*ort(1)-0.5_dop*ort(2) )&
                   q(1)=q(1)+pi2
         else
            q(1) = 0.25_dop*pi2*nint(2.0_dop*(ort(1)+ort(gdim))/pi2)
         endif
         xstd = 2.0_dop*atan2(sqrt(abs(1.0_dop-x)),sqrt(1.0_dop+x))

!-----------------------------------------------------------------------
! check that variance is positive before taking sqrt (can become negative
!  when variance is close to 0)
!-----------------------------------------------------------------------
      else
         xvar=q(2)-q(1)**2
         if (xvar.lt.0.0_dop) then
            xvar=abs(xvar)
            xstd=-sqrt(xvar)
         else
            xstd=sqrt(xvar)
         endif
      endif
      nvar=occ(2)-occ(1)**2
      if (nvar.lt.0.0_dop) then
         nvar=abs(nvar)
         nstd=-sqrt(nvar)
      else
         nstd=sqrt(nvar)
      endif

      if(basis.eq.2 .or. basis.eq.7.or. basis.eq.9.or. basis.eq.12 .or. &
         basis.eq.20 .or. basis.eq.21 .or. basis .eq. 33) then
         write (iout,'(a12,a,f9.4,a,f9.4,a,f9.4,a,f9.4)') &
              modelabel ,': <q>=',q(1),'  <dq>=',xstd, &
              '  <j>=',occ(1),'  <dj>=',nstd
      else if (basis.eq.5) then             ! exp-DVR
         occ(1) = occ(1) - (gdim-1)/2
         write (iout,'(a12,a,f9.4,a,f9.4,a,f9.4,a,f9.4)') &
              modelabel ,': <q>=',q(1),'  <dq>=',xstd, &
              '  <p>=',occ(1)*dp,'  <dp>=',nstd*dp

      else
         write (iout,'(a12,a,f9.4,a,f9.4,a,f9.4,a,f9.4)') &
              modelabel ,': <q>=',q(1),'  <dq>=',xstd, &
              '  <n>=',occ(1),'  <dn>=',nstd
      endif

!-----------------------------------------------------------------------
! store expectation values in qdq array for output in check file
!-----------------------------------------------------------------------
      qdq(1) = q(1)
      qdq(2) = xstd
      qdq(3) = occ(1)
      qdq(4) = nstd

      return
      end subroutine outexp1

!***********************************************************************
! OUTEXP2
!
! for FFT or X-local basis output: 
!   q(1): expectation value of the coordinate operator, <x>
!   q(2): <x**2> 
!   imp(1): expectation value of the momentum operator, <p>
!   imp(2): <p**2> 
!
!
!  If the grid-length is 2*PI, then the expectation values and variances 
!  are evaluated via the expectations <sin> and <cos>. (HDM 02/02) 
!
!
! 10/96 AJ
!***********************************************************************

      subroutine outexp2(gpop1,gpop2,spop,gdim,ort,fftp,iout, &
                 modelabel,qdq)

      implicit none

      integer(long)                           :: g
      integer(long), intent(in)               :: gdim,iout
      real(dop), dimension(4), intent(out)    :: qdq
      real(dop)                               :: x,dort,norm,xvar,pvar,xstd,&
                                                 pstd,pi2
      real(dop), intent(in)                   :: spop
      real(dop), dimension(2)                 :: q,imp
      real(dop), dimension(gdim), intent(in)  :: gpop1,gpop2,ort,fftp
      character(len=*), intent(in)            :: modelabel
      logical(kind=4)                         :: l2pi

      l2pi = .false.
      pi2  = 8.0_dop*atan(1.0_dop)
      if( abs(ort(gdim)+ort(2)-ort(1)-pi2) .lt. 1.0e-8_dop ) l2pi = .true.

      q(1)=0.0
      q(2)=0.0
      imp(1)=0.0
      imp(2)=0.0
      dort=ort(2)-ort(1)

      do g=1,gdim
         if(l2pi) then
            q(1)=q(1)+sin(ort(g))*gpop1(g)
            q(2)=q(2)+cos(ort(g))*gpop1(g)
         else
            x=ort(g)
            q(1)=q(1)+x*gpop1(g)
            q(2)=q(2)+x*x*gpop1(g)
         endif
         x=fftp(g)
         imp(1)=imp(1)+x*gpop2(g)
         imp(2)=imp(2)+x*x*gpop2(g)
      enddo

!-----------------------------------------------------------------------
! Renormalize expectation values 
!-----------------------------------------------------------------------
      norm=spop+1.0e-16_dop
      q(1)   = q(1)/norm
      q(2)   = q(2)/norm
      imp(1) = imp(1)/norm
      imp(2) = imp(2)/norm

!-----------------------------------------------------------------------
! Evaluate mean value and variance for periodic grid.
!-----------------------------------------------------------------------
      if(l2pi)  then
         x = sqrt(q(1)*q(1)+q(2)*q(2))
         if (x .gt. 1.0e-5_dop) then
            q(1) = atan2(q(1),q(2))
            if( q(1) .lt. 1.5_dop*ort(1)-0.5_dop*ort(2) ) &
                   q(1)=q(1)+pi2
         else
            q(1) = 0.25_dop*pi2*nint(2.0_dop*(ort(1)+ort(gdim))/pi2)
         endif
         xstd = 2.0_dop*atan2(sqrt(abs(1.0_dop-x)),sqrt(1.0_dop+x))

!-----------------------------------------------------------------------
! check that variance is positive before taking sqrt (can become negative
!  when variance is close to 0)
!-----------------------------------------------------------------------
      else
         xvar=q(2)-q(1)**2
         if (xvar.lt.0.0) then
            xvar=abs(xvar)
            xstd=-sqrt(xvar)
         else
            xstd=sqrt(xvar)
         endif
      endif
      pvar=imp(2)-imp(1)**2
      if (pvar.lt.0.0) then
         pvar=abs(pvar)
         pstd=-sqrt(pvar)
      else
         pstd=sqrt(pvar)
      endif

!-----------------------------------------------------------------------
! Renormalize expectation values in case of CAP calculations
!-----------------------------------------------------------------------
      write (iout,'(a12,a,f9.4,a,f9.4,a,f9.4,a,f9.4)')&
              modelabel,': <q>=',q(1),'  <dq>=',xstd,&
              '  <p>=',imp(1),'  <dp>=',pstd

!-----------------------------------------------------------------------
! store expectation values in qdq array for output in check file
!-----------------------------------------------------------------------
      qdq(1) = q(1)
      qdq(2) = xstd
      qdq(3) = imp(1)
      qdq(4) = pstd

      return
      end subroutine outexp2

!***********************************************************************
! OUTEXP3
!
! for spherical-FBR basis output:
!   j(1): expectation value of the j rotational quantum number <j>
!   j(2): <j**2>
!   m(1): expectation value of the m rotational quantum number, <m>
!   m(2): <m**2>
!
! 01/98 MCH
!***********************************************************************

      subroutine outexp3(gpop1,spop,gdim,jsph,msph,iout,modelabel,mode2,&
                 jqdq,mqdq)

      implicit none

      integer(long)                              :: g
      integer(long), intent(in)                  :: gdim,iout
      real(dop), dimension(4), intent(out)       :: jqdq,mqdq
      integer(long), dimension(gdim), intent(in) :: jsph,msph
      real(dop)                                  :: x,norm,jvar,mvar,jstd,mstd
      real(dop), intent(in)                      :: spop
      real(dop), dimension(2)                    :: m,j
      real(dop), dimension(gdim), intent(in)     :: gpop1

      character(len=*), intent(in)               :: modelabel,mode2


      j(1)=0.0
      j(2)=0.0
      m(1)=0.0
      m(2)=0.0
      do g=1,gdim
         x=jsph(g)
         j(1)=j(1)+x*gpop1(g)
         j(2)=j(2)+x*x*gpop1(g)

         x=msph(g)
         m(1)=m(1)+x*gpop1(g)
         m(2)=m(2)+x*x*gpop1(g)
      enddo

!-----------------------------------------------------------------------
! Renormalize expectation values
!-----------------------------------------------------------------------
      norm=spop+1.0e-16_dop
      j(1)   = j(1)/norm
      j(2)   = j(2)/norm
      m(1) = m(1)/norm
      m(2) = m(2)/norm

!-----------------------------------------------------------------------
! check that variance is positive before taking sqrt (can become negative
!  when variance is close to 0)
!-----------------------------------------------------------------------
      jvar=j(2)-j(1)**2
      if (jvar.lt.0.0) then
         jvar=abs(jvar)
         jstd=-sqrt(jvar)
      else
         jstd=sqrt(jvar)
      endif
      mvar=m(2)-m(1)**2
      if (mvar.lt.0.0) then
         mvar=abs(mvar)
         mstd=-sqrt(mvar)
      else
         mstd=sqrt(mvar)
      endif

!-----------------------------------------------------------------------
! Renormalize expectation values in case of CAP calculations
!-----------------------------------------------------------------------
      write (iout,'(a12,a,f9.4,a,f9.4)')&
              modelabel,': <j>=',j(1),'  <dj>=',jstd
      write (iout,'(a12,a,f9.4,a,f9.4)')&
              mode2,    ': <m>=',m(1),'  <dm>=',mstd

!-----------------------------------------------------------------------
! store expectation values in qdq array for output in check file
!-----------------------------------------------------------------------
      jqdq(1) = j(1)
      jqdq(2) = jstd
      jqdq(3) = 0.0_dop
      jqdq(4) = 0.0_dop

      mqdq(1) = m(1)
      mqdq(2) = mstd
      mqdq(3) = 0.0_dop
      mqdq(4) = 0.0_dop

      return
      end subroutine outexp3

!***********************************************************************
! OUTEXP4
!
! for X-local basis output: 
!   q(1): expectation value of the coordinate operator, <x>
!   q(2): <x**2>
!   imp(1): expectation value of the momentum operator, <p>
!   imp(2): <p**2> 
!
!***********************************************************************

      subroutine outexp4(psi,n,dim1,gdof,iout,modelabel,spop,qdq)

      implicit none

      integer(long)                                  :: e
      integer(long), intent(in)                      :: gdof,iout,dim1,n
      real(dop), dimension(4), intent(out)           :: qdq
      real(dop)                                      :: x,p,xvar,nvar,xstd,nstd
      real(dop), intent(in)                          :: spop
      real(dop), dimension(2)                        :: q,imp
      complex(dop), dimension(gdof,dim1), intent(in) :: psi

      character(len=*), intent(in)                    :: modelabel

!-----------------------------------------------------------------------
! x-expectation values taken from coordinates
! p-expectation values taken from momenta
!-----------------------------------------------------------------------
      q(1)=0.0_dop
      q(2)=0.0_dop
      imp(1)=0.0_dop
      imp(2)=0.0_dop
      do e=1,dim1
         x=dble(psi(n,e))
         p=dimag(psi(n,e))
         q(1)=q(1)+x
         q(2)=q(2)+x*x
         imp(1)=imp(1)+p
         imp(2)=imp(2)+p*p
      enddo
      q(1)=q(1)/dim1
      q(2)=q(2)/dim1
      imp(1)=imp(1)/dim1
      imp(2)=imp(2)/dim1

!-----------------------------------------------------------------------
! Renormalize expectation values 
!-----------------------------------------------------------------------
      q(1)   = q(1)*spop
      q(2)   = q(2)*spop
      imp(1) = imp(1)*spop
      imp(2) = imp(2)*spop

!-----------------------------------------------------------------------
! check that variance is positive before taking sqrt (can become negative
!  when variance is close to 0)
!-----------------------------------------------------------------------
      xvar=q(2)-q(1)**2
      if (xvar.lt.0.0) then
         xvar=abs(xvar)
         xstd=-sqrt(xvar)
      else
         xstd=sqrt(xvar)
      endif
      nvar=imp(2)-imp(1)**2
      if (nvar.lt.0.0) then
         nvar=abs(nvar)
         nstd=-sqrt(nvar)
      else
         nstd=sqrt(nvar)
      endif

      write (iout,'(a12,a,f9.4,a,f9.4,a,f9.4,a,f9.4)') &
              modelabel ,': <q>=',q(1),'  <dq>=',xstd, &
              '  <p>=',imp(1),'  <dp>=',nstd

!-----------------------------------------------------------------------
! store expectation values in qdq array for output in check file
!-----------------------------------------------------------------------
      qdq(1) = q(1)
      qdq(2) = xstd
      qdq(3) = imp(1)
      qdq(4) = nstd

      return
      end subroutine outexp4

!#######################################################################
!+------------------------------------------------------------------------+
!     exp values for gwp modes
!+------------------------------------------------------------------------+

      subroutine outexp5(gfgwp,aigwp,zgwp,agmat,dicht2,gdof,&
                 gwpdim1,dim1,xivec,modelabel,iout,f,dgwp,psitype1,qdq,&
                 phase)


      implicit none
      
      integer(long)                           :: ig,jg
      integer(long), intent(in)               :: f,gwpdim1,dim1,gdof,iout,&
                                                 psitype1
      real(dop), dimension(4), intent(out)    :: qdq
      integer(long), dimension(maxdim)        :: imoment
      real(dop)                               :: xvar,xstd,pvar,pstd,x,p,fac,&
                                                 norma
      real(dop), dimension(2)                 :: q,imp

      complex(dop)                            :: coeffa,qtmp1,qtmp2,ptmp1,&
                                                 ptmp2,ntmp,zfac
      complex(dop), dimension(gwpdim1,gwpdim1)                 :: tmpgh
      complex(dop), dimension(gdof,gwpdim1), intent(in)        :: xivec
      complex(dop), dimension(dim1,dim1), intent(in)           :: dicht2
      complex(dop), dimension(gwpdim1,gwpdim1), intent(inout)     :: gfgwp
      complex(dop), dimension(gwpdim1,dim1), intent(in)        :: dgwp
      complex(dop), dimension(gwpdim1,gwpdim1)                 :: gdicht2
      complex(dop), dimension(gdof,gwpdim1,gwpdim1), intent(in)      :: zgwp
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(in)         :: agmat
      complex(dop), dimension(gdof,gdof,gwpdim1,gwpdim1), intent(in) :: aigwp
      complex(dop), dimension(gwpdim1), intent(in),optional     :: phase
      complex(dop), dimension(gdof,gdof)                        :: ai2
      complex(dop), dimension(gdof)                             :: z2

      character(len=*), intent(in)                             :: modelabel
!      complex(dop), external                                   :: moment

      if (psitype1 .eq. 0 .or. psitype .eq. 5) then
         call simtran1btz(dicht2,dgwp,gdicht2,dim1,gwpdim1)
      else
         call cpqxz(dicht2,gdicht2,gwpdim1)
      endif

      qtmp1=0.0_dop
      qtmp2=0.0_dop
      ptmp1=0.0_dop
      ptmp2=0.0_dop
      ntmp = 0.0_dop
if ((present(phase)) ) then
!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(ig,jg,coeffa,imoment,ai2,z2)
!$omp do schedule(dynamic)
      do ig=1,gwpdim1
         do jg = 1,gwpdim1
            call zerovxi(imoment,gdof)
            coeffa = gdicht2(ig,jg)

               call diagonal_momentsz(gdof,agmat(1,1,ig),agmat(1,1,jg), &
                    xivec(1,ig),xivec(1,jg),phase(ig),phase(jg),&
                    gfgwp(ig,jg),ai2,z2,0,1)

            ntmp = ntmp + coeffa*moment(imoment,gdof,gfgwp(ig,jg), &
                                 ai2,z2)

!+---------------------------------------------------------------------+
!             x expectation value
!+---------------------------------------------------------------------+
            call zerovxi(imoment,gdof)
            imoment(f)   = 1

            qtmp1 = qtmp1 + coeffa*moment(imoment,gdof,gfgwp(ig,jg), &
                        ai2,z2)

!+---------------------------------------------------------------------+
!               x^2 expectation value
!+---------------------------------------------------------------------+
            imoment(f)   = imoment(f)  + 1
            qtmp2 = qtmp2 + coeffa*moment(imoment,gdof,gfgwp(ig,jg), &
                            ai2,z2)
         enddo
      enddo
!$omp end do
!$omp end parallel
else
      do ig=1,gwpdim1
         do jg = 1,gwpdim1
            call zerovxi(imoment,gdof)
            coeffa = gdicht2(ig,jg)

            ntmp = ntmp + coeffa*moment(imoment,gdof,gfgwp(ig,jg), &
                                 aigwp(1,1,ig,jg),zgwp(1,ig,jg))

!+---------------------------------------------------------------------+
!             x expectation value
!+---------------------------------------------------------------------+
            call zerovxi(imoment,gdof)
            imoment(f)   = 1

            qtmp1 = qtmp1 + coeffa*moment(imoment,gdof,gfgwp(ig,jg), &
                        aigwp(1,1,ig,jg),zgwp(1,ig,jg))

!+---------------------------------------------------------------------+
!               x^2 expectation value
!+---------------------------------------------------------------------+
            imoment(f)   = imoment(f)  + 1
            qtmp2 = qtmp2 + coeffa*moment(imoment,gdof,gfgwp(ig,jg), &
                            aigwp(1,1,ig,jg),zgwp(1,ig,jg))
         enddo    
      enddo    
endif
!+---------------------------------------------------------------------+
!               p expectation value
!+---------------------------------------------------------------------+
if ((present(phase)) ) then
      call dqgwp_elements (f,gdof,tmpgh,gwpdim1,agmat,xivec,gfgwp, &
           aigwp,zgwp,1,1,phase)
else
      call dqgwp_elements (f,gdof,tmpgh,gwpdim1,agmat,xivec,gfgwp, &
           aigwp,zgwp,1,1)
endif
      zfac=(0.0_dop,-1.0_dop)
      call xmxxzzo(zfac,tmpgh,gwpdim1,gwpdim1)
      call sum2qxxz(gdicht2,tmpgh,ptmp1,gwpdim1)

!+---------------------------------------------------------------------+
!               p^2 expectation value
!               build matrix elements for p1=1
!+---------------------------------------------------------------------+
if ((present(phase)) ) then
      call dq2gwp_elements (f,f,gdof,tmpgh,gwpdim1,agmat,xivec,gfgwp, &
           aigwp,zgwp,1,1,phase)
else
      call dq2gwp_elements (f,f,gdof,tmpgh,gwpdim1,agmat,xivec,gfgwp, &
           aigwp,zgwp,1,1)
endif

      fac=-1.0_dop
      call xmxxdzo(fac,tmpgh,gwpdim1,gwpdim1)
      call sum2qxxz(gdicht2,tmpgh,ptmp2,gwpdim1)

      normA  = dble(ntmp)+1.0e-16_dop
      q(1)   = dble(qtmp1) /normA
      q(2)   = dble(qtmp2) /normA
      imp(1) = dble(ptmp1) /normA
      imp(2) = dble(ptmp2) /normA

!-----------------------------------------------------------------------
! check that variance is positive before taking sqrt (can become negative
!  when variance is close to 0)
!-----------------------------------------------------------------------
      x=dble(q(1))
      p=dble(imp(1))
      xvar=q(2)-q(1)**2
      if (xvar.lt.0.0_dop) then
         xvar=abs(xvar)
         xstd=-sqrt(xvar)
      else
         xstd=sqrt(xvar)
      endif
      pvar=imp(2)-imp(1)**2
      if (pvar.lt.0.0_dop) then
         pvar=abs(pvar)
         pstd=-sqrt(pvar)
      else
         pstd=sqrt(pvar)
      endif

! ... write out q and imp !
      if (ddcoo .eq. 2) then
         write (iout,'(a12,a,f9.4,a,f9.4,a,f9.4,a,f9.4)') &
                 modelabel,': <xyz>=',q(1),'  <dq>=',xstd , &
              '  <p>=',imp(1),'  <dp>=',pstd
      else
         write (iout,'(a12,a,f9.4,a,f9.4,a,f9.4,a,f9.4)') &
                 modelabel,': <q>=',q(1),'  <dq>=',xstd , &
              '  <p>=',imp(1),'  <dp>=',pstd
      endif

!-----------------------------------------------------------------------
! store expectation values in qdq array for output in check file
!-----------------------------------------------------------------------
      qdq(1) = q(1)
      qdq(2) = xstd
      qdq(3) = imp(1)
      qdq(4) = pstd

      return
      end subroutine outexp5

!#######################################################################
!
!              Subroutine WRDICHT3O
!
! called by datout
!
! writes the density matrix eigenvalues i.e. natural orbital 
!   populations to output file
!
! for a single mode:
!  dicht3: eigenvalues of density matrix, or grid populations
!  dim   : number of single particle functions, or grid points
!
!  V6.0 GW
!
!***********************************************************************

      subroutine wrdicht3o(dicht3,dim,mode,nspfdof1)

      implicit none

      integer(long)                                :: i,j,k
      integer(long), intent(in)                    :: dim,nspfdof1
      real(dop), dimension(dim), intent(in)        :: dicht3
      real(dop)                                    :: c
      character(len=*), intent(in)                 :: mode

      c=1.d0
      if (initnorm .gt. 0.0) c=1.0_dop/(initnorm*initnorm)

      if(mpb.gt.0) c=1.d0/dble(npackts) ! multi-packets single set
      if(lreflex)  c=c/dble(2.d0)       ! Reflex calculation

      j = min(dim,7)
      if (nspfdof1 .eq. 1) then
         write(iout,810) mode, (dicht3(i)*c,i=1,j)
      else
         write(iout,811) mode,nspfdof1,(dicht3(i)*c,i=1,j)
      end if

 100  if( j .eq. dim ) return
      k = j + 1
      j = min(dim,k+6)
      write(iout,820) k, j, (dicht3(i)*c,i=k,j)
      goto 100

 810  format(a12,':', 3p7f9.4)
 811  format(a10,'C',i3,':', 3p7f9.4)
 820  format(3x,i3,' -',i3,'> ', 3p7f9.4)
 !820  format(3x,i5,' -',i5,'> ', 3p7f9.4)

      end subroutine wrdicht3o

!#######################################################################
!
!              Subroutine WRDICHT31
!
! called by datout
!
! writes the diagonal elements of the density matrix to output file
!
! for a single mode:
!  dicht2: density matrix
!  dim   : number of single particle functions, or grid points
!
!***********************************************************************

      subroutine wrdicht31(dicht2,gs2,dim,mode,nspfdof,gwpdim,dgwp,&
                 psitype,reggp)

      implicit none

      integer(long)                                :: i,j,k
      integer(long), intent(in)                    :: dim,gwpdim,&
                                                      nspfdof,psitype
      complex(dop), dimension(dim,dim), intent(in)       :: dicht2
      complex(dop), dimension(gwpdim,gwpdim)             :: gdicht2
      complex(dop), dimension(gwpdim,gwpdim), intent(in) :: gs2
      real(dop)                                          :: c
      real(dop), dimension(gwpdim,gwpdim)                :: weight
      complex(dop), dimension(gwpdim,dim), intent(in)    :: dgwp
      real(dop), dimension(gwpdim)                 :: reggp
      complex(dop)                                 :: ctemp
      character(len=*), intent(in)                 :: mode

      c=1.d0
      if (initnorm .gt. 0.0) c=1.0_dop/(initnorm*initnorm)

      if (psitype .eq. 0 .or. psitype .eq. 5) then
         call simtran1btz(dicht2,dgwp,gdicht2,dim,gwpdim)
      else
         gdicht2 = dicht2
      endif

      if (lweightggp) then
         do i=1,gwpdim
            do j=1,gwpdim
               if (i .eq. j) then
                  weight(i,j) = 1.0_dop
               else
                  weight(i,j) = &
                      2.0d0*dble(gdicht2(i,i)) / &
                      dble(gdicht2(i,i) + gdicht2(j,j) + 1.0d-8)
               endif
            enddo
         enddo
      else
         weight = 1.0_dop
      endif

      do i=1,gwpdim
         ctemp = 0.0_dop
         do j=1,gwpdim
            ctemp = ctemp + weight(i,j)*gdicht2(i,j)*gs2(i,j)
         enddo
         reggp(i) = dble(ctemp)*c
      enddo

      j = min(gwpdim,7)
      if (nspfdof .eq. 1) then
         write(iout,810) mode, (reggp(i),i=1,j)
      else
         write(iout,811) mode,nspfdof,(reggp(i),i=1,j)
      end if

      do while (j .lt. gwpdim)
         k = j + 1
         j = min(gwpdim,k+6)
         write(iout,820) k, j, (reggp(i),i=k,j)
      enddo

 810  format(a12,'Re:', 1p7f9.4)
 811  format(a8,'ReC',i2,':', 1p7f9.4)
 820  format(3x,i5,' -',i5,'> ', 1p7f9.4)
! 820  format(3x,i3,' -',i3,'> ', 1p7f9.4)

      end subroutine wrdicht31

!******************************************************************
!
!         Subroutine spawnmctdh
!
!         Monitor the natutal orbital populations durning dynamic
!         SPFs (i.e. spawning)
!
!         Called from datout
!
!         2019 David Mendive-Tapia
!******************************************************************

      subroutine spawnmctdh(t,dicht3,lcheck)

      implicit none

      integer(long)         :: m,s
      real(dop), intent(in) ::  t
      real(dop), dimension(d3matdim), intent(in) :: dicht3
      character(len=16)     :: string
      logical(kind=4), intent(inout) :: lcheck

!     Read/write file
      integer(long), parameter :: fileversion=1

!     Rewind read-write file ispwn
      if (ldebugspawn) then
         rewind(ispwn)
!        To sort out later: time should be read from psi file
         write(ispwn,'(a,f22.16,a)') 'time:',t/fs,' fs'
      endif
!     Process population values
      do m=1,nmode
         if ((m.eq.meb).or.(m.eq.mpb)) then
! complete basis does not change
            dimtape(m,1)=maxdimspwn(m,1)
         else
            do s=1,nstate
!              Stopping criteria
               if (dicht3(d3mat(m,s)+dim(m,s)-1)*1000.gt.alphamax) then
                  lcheck=.true.
                  lnewspf=.true.
               endif
!              Write SPFs into ispwn read-write file
               if (dicht3(d3mat(m,s)+dim(m,s)-1)*1000.gt.alphathr) then
                  write(string,'(i4,a)') dim(m,s)+imaxspf
                  dimtape(m,s)=min(dim(m,s)+imaxspf,maxdimspwn(m,s))
               else
                  write(string,'(i4)') dim(m,s)
                  dimtape(m,s)=dim(m,s)
               endif
            enddo
         endif
      enddo
      call myflush(ilog)
      call myflush(iopteigen)

!     Write data to read/write file inspf
      write(inspf,'(f12.5)',advance='no') t/fs
      do s=1,nstate
         do m=1,nmode
            write(string,'(i4)') dim(m,s)
            write(inspf,'(x,a)',advance='no') trim(adjustl(string))
         enddo
      enddo
      write(inspf,'(a)')
      call myflush(inspf)
      return
      end subroutine spawnmctdh

!******************************************************************
!
!         Subroutine spawnprint
!
!         Output SPF basis section durning dynamic SPFs
!         (i.e. spawning)
!
!         Called from output
!
!         2019 David Mendive-Tapia
!******************************************************************

      subroutine spawnprint(t)

      implicit none

      integer(long) :: m,cm,s,f
      real(dop), intent(in) :: t
      character(len=16) :: string

!     Rewind read-write file ispwn and write info
      rewind(ispwn)
      write(ispwn,'(a,f22.16,a)') 'time:',t/fs,' fs'
      write(ispwn,'(a)') 'SPF-BASIS-SECTION'
      if (lmult) then
         write(ispwn,'(a)') 'multi-set'
      else
         write(ispwn,'(a)') 'single-set'
      endif
      do m=1,nmode
         if ((m.ne.meb).and.(m.ne.mpb)) then
!           Mode labels
            do cm=1,nspfdof(m)
               f=spfdof(cm,m)
               write(string,'(a12)') modelabel(f)
               write(ispwn,'(x,a)',advance='no') trim(adjustl(string))
               if (cm.ne.nspfdof(m)) then
                  write(ispwn,'(x,a)',advance='no') ','
               else
                  write(ispwn,'(x,a)',advance='no') '='
               endif
            enddo
            do s=1,nstate
!              Write SPFs into ispwn read-write file
               write(string,'(i4)') dim(m,s)
               write(ispwn,'(x,a)',advance='no') trim(adjustl(string))
               if (s.ne.nstate) then
                  write(ispwn,'(a)',advance='no') ','
               else
                  write(ispwn,'(x,a)',advance='no') '<'
                  write(string,'(i4)') maxdimspwn(m,s)
                  write(ispwn,'(x,a)') trim(adjustl(string))
               endif
            enddo
         endif
      enddo
      write(ispwn,'(a)') 'end-spf-basis-section'

      return
      end subroutine spawnprint


      end module datoutmod

