! **********************************************************************
!***  subroutine lancz_writer is now in lib/eigen/lanczosmod.f90

      module diaghammod

use versions
use decimal, only: long, dop
use globmemmod
use logdat
use global
use globmemmod
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use channels

use iorst, only: wrrst
use lanczosmod, only: lanczos,lancz_errmsg
use function, only: funkex


use xvlib,   only: xvxxzzo
use timing

      implicit none
      private
      public :: diagham,wkdiagham,lanczout,eigvecout,lancz_restartout,&
                hampsi
      contains

! **********************************************************************
!                                                                     
!                      module DIAGHAM                                 
!                                                                     
!   MCTDH module controlling the diagonalisation of the Hamiltonian.  
!                                                                     
! lanczord: if > 0 on input, actual order reached by a previous run
!           on output, order reached by present run
!
! lanczorder: no. of lanczos iterations asked for
! **********************************************************************

      subroutine diagham(lanczord,psi)

      implicit none

      complex(dop), dimension(1:dgldim), intent(inout):: psi
      complex(dop), dimension(:), allocatable         :: tmppsi
      integer(long), intent(inout)                    :: lanczord
      integer(long)                                   :: nbasis

      logical(kind=4)                                 :: linwf
      integer(long)                                   :: rworkdim,neigs,errflag,i,version,release
      real(dop)                                       :: time
      real(dop), dimension(:), allocatable            :: eigval,eigint,eigerr
      character(len=100)                              :: buffer
      integer(long), save                             :: tid=0 ! timer ID
      integer(long)     :: ierr
      character(len=10) :: cerr

! --- INITIALISE VARIABLES ---

      routine        = 'diagham'
      linwf          = .false.
      rworkdim       = workrdim
      time           = 0.0_dop
      if (.not. lcont) lanczord = 0
      if (lebonly .and. lmult) then
         if (lebcont) then
            nbasis = elbnd + gdim(febcont)*(eltot-elbnd) 
         else
            nbasis = gdim(feb)
         endif
      else
         nbasis = totphidim
      endif

! --- CREATE HEADLINE IN OUTPUT FILE ---      

      call getrevno(progver,version,release)
      write(iout,*)
      write(iout,'(79a)') ('*', i=1,79)
      write(iout,'(31x,a,i3)') 'MCTDH version',version
      write(iout,'(34x,a,i5)') 'Release ', release
      write(iout,'(79a)') ('*', i=1,79)
      write(iout,*)
      call fpstartup(buffer)
      write(iout,'(1x,a)') buffer(1:72)
      write(iout,'(1x,a)') name(1:laenge)
      if (runtitle(1:10) .ne. '          ') &
         write(iout,'(1x,a)') runtitle(1:78)
      write(iout,'(78a)') ('-',i=1,78)
      write(iout,*)

! --- CALL LANCZOS ROUTINE ---

      if (diagham_type .eq. 0) then

         if(test) goto 200

         call get_timer(tid, 'Lanczos')
         call start_timer(tid)

         cerr = 'eigval'
         allocate(eigval(lanczorder),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'eigint'
         allocate(eigint(lanczorder),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'eigerr'
         allocate(eigerr(lanczorder),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'tmppsi'
         allocate(tmppsi(nbasis),stat=ierr)
         if (ierr > 0) go to 1000

         if (lebonly) then
            tmppsi(1:nbasis) = 1.0_dop / dsqrt(dble(nbasis))
         else
            tmppsi(1:nbasis) = psi(zetf(1,1):zetf(1,1)+totphidim-1)
         endif

         call lanczos(lanczord,tmppsi,nbasis,lanczorder,macheps, &
                      leigvec,workidim,rworkdim,intpsidim,neigs,eigval, &
                      eigint,eigerr,errflag,lancz_sclrprod,lanczout, &
                      eigvecout,lancz_restartout,hampsi, &
                      mi(miworki),mr(mrworkr),mc(mcintpsi))
         call stop_timer(tid)
         if (errflag .ne. 0) then
            call lancz_errmsg(errflag,message)
            call errormsg
         endif

! --- SAVE DATA NEEDED TO RESTART LANCZOS ROUTINE ---

!     Here we use the fact that the Lanczos routine stores the first
!     on-diagonal element, alpha(1), at mr(mrworkr), and the first
!     off-diagonal element, beta(2), at mr(mrworkr+lanczorder).

         call wrrst(time,psi,linwf,mc(mcintpsi),lanczord,mr(mrworkr), &
              mr(mrworkr+lanczorder),mi(mijindx),agmat,mi(mitrajst))

! --- WRITE EIGENVALUES TO FILE ---

         if (rlxunit .eq. 'au') then
            write(ieigval,'(a1,2x,a,7x,a,9x,a,13x,a,4x,a,4x,a)') &
                 '#','Number','Eigenvalue [au]','Intensity', &
                 'Eigenerror [au]','Eigenvalue [au]','Excitation [au]'
            do i = 1,neigs
               write(ieigval,'(3x,i6,2x,f23.16,2(4x,1pe18.9e3),0p2f20.3)') &
                    i,eigval(i),eigint(i),eigerr(i),eigval(i),  &
                    (eigval(i)-eigval(1))
            enddo

         else
            write(ieigval,'(a1,2x,a,7x,a,9x,a,13x,a,4x,a,4x,a)') &
                 '#','Number','Eigenvalue [eV]','Intensity', &
                 'Eigenerror [eV]','Eigenvalue [cm-1]','Excitation [cm-1]'
            do i = 1,neigs
               write(ieigval,'(3x,i6,2x,f23.16,2(4x,1pe18.9e3),0p2f20.3)') &
                    i,eigval(i)*ev,eigint(i),eigerr(i)*ev,eigval(i)*  &
                    2.1947463137d+5,(eigval(i)-eigval(1))*2.1947463137d+5
            enddo
         endif

         deallocate(eigval,eigint,eigerr)

!    --- WRITE SOME INFO TO LOG AND OUTPUT FILE ---

 200     write(iout,'(a,i7)') 'Number of Lanczos iterations: ',lanczorder
         write(iout,'(a,i7)') 'Number of eigenvalues found:  ',neigs
         write(iout,'(a,i7)') 'Length of Lanczos vectors:    ',nbasis
         write(ilog,'(a,i7)') 'Number of Lanczos iterations: ',lanczorder
         write(ilog,'(a,i7)') 'Number of eigenvalues found:  ',neigs
         write(ilog,'(a,i7)') 'Length of Lanczos vectors:    ',nbasis
         write(ilog,*)

      else if (diagham_type .eq. 1) then
         call get_timer(tid, 'Direct Diag')
         call start_timer(tid)

         call directdiag(mr(mrhops))

         call stop_timer(tid)

      else 
         routine = 'Diagham (diaghammod.f90)'
         message = 'Unknown diagionalisaton type '
         call errormsg

      endif

! --- WRITE TIMINGS TO FILE ---

      if (ltime) call timout
      
! --- WRITE FINAL MESSAGES TO FILES ---

      write(iout,*)
      if(test) then
         write(iout,'(a)') 'Test run.'
      else
         write(iout,'(a)') 'Diagonalisation was successful.'
      endif
      write(iout,*)
      call myflush(iout)

      write(ilog,'(a)') '***** God''s final message *****'
      write(ilog,*)
      write(ilog,'(a)') 'Program terminated normally.'
      call myflush(ilog)

      return
 1000 write(ilog,'(a)') 'Error allocating memory in DIAGHAM :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      write(6,'(a)') 'Error allocating memory in DIAGHAM :',cerr
      write(6,'(a,i5)') 'Error Code :',ierr
 
      end subroutine diagham

! **********************************************************************
! *                                                                    *
! *                         SUBROUTINE LANCZOUT                        *
! *                                                                    *
! * Writes a single Lanczos vector to file.                            *
! *                                                                    *
! **********************************************************************

      subroutine lanczout (lanczvec, lanczdim)

      implicit none

      integer(long)                                   :: i
      integer(long), intent(in)                       :: lanczdim
      complex(dop), dimension(1:lanczdim), intent(in) :: lanczvec

! --- WRITE LANCZOS VECTOR TO FILE ---

      if (llanczvec) then
         write(ilancz) (lanczvec(i), i=1,lanczdim)
      endif

      return
      end subroutine lanczout

! **********************************************************************
! *                                                                    *
! *                         SUBROUTINE EIGVECOUT                       *
! *                                                                    *
! * Writes a single eigenvector to file.                               *
! *                                                                    *
! **********************************************************************

      subroutine eigvecout (eigvec, eigdim)

      implicit none

      integer(long)                              :: i
      integer(long), intent(in)                  :: eigdim
      real(dop), dimension(1:eigdim), intent(in) :: eigvec

! --- WRITE EIGENVECTOR TO FILE ---

      if (leigvec) then
         write(ieigvec,'(6f12.8)') (eigvec(i), i=1,eigdim)
         write(ieigvec,*) 
      endif

      return
      end subroutine eigvecout

! **********************************************************************
! *                                                                    *
! *                           SUBROUTINE HAMPSI                        *
! *                                                                    *
! * This is an interface to the "funkex" routine for computing the     *
! * action of the Hamiltonian on a wavefunction.                       *
! *                                                                    *
! **********************************************************************

      subroutine hampsi (psiin, psiout, psidim)

      implicit none

      complex(dop), parameter                        :: imag = (0.0_dop,1.0_dop)

      integer(long), intent(in)                      :: psidim
      real(dop)                                      :: time
      complex(dop), dimension(1:psidim), intent(inout) :: psiin
      complex(dop), dimension(1:psidim), intent(out) :: psiout

! --- COMPUTE ACTION OF HAMILTONIAN ON WAVEFUNCTION ---

      time = 0.0_dop
      call funkex(time,psiin,psiout)
      call xvxxzzo(imag,psiout,psidim)

      return
      end subroutine hampsi

! **********************************************************************
! *                                                                    *
! *                         FUNCTION LANCZ_SCLRPROD                    *
! *                                                                    *
! * Computes the Euklidian scalar product of two vectors.              *
! *                                                                    *
! * Input parameters:                                                  *
! *   psi1: "Bra" vector.                                              *
! *   psi2: "Ket" vector.                                              *
! *   dim:  Length of vectors.                                         *
! *                                                                    *
! * Output parameters:                                                 *
! *   none                                                             *
! *                                                                    *
! **********************************************************************

      complex(dop) function lancz_sclrprod (psi1, psi2, dim)

      implicit none

      integer(long), intent(in)                  :: dim
      complex(dop), dimension(1:dim), intent(in) :: psi1,psi2

      integer(long)       :: i
      complex(dop)        :: s

! --- COMPUTE SCALAR PRODUCT ---

      s = dconjg(psi1(1))*psi2(1)
      do i = 2,dim
         s = s+dconjg(psi1(i))*psi2(i)
      enddo
      lancz_sclrprod = s

      return
      end function lancz_sclrprod


! **********************************************************************
! *                                                                    *
! *                      SUBROUTINE LANCZ_RESTARTOUT                   *
! *                                                                    *
! * Writes the two current Lanczos vectors as well as the on- and      *
! * off-diagonal elements of the tridiagonal Lanczos matrix to file.   *
! * This data is required to restart the Lanczos routine. (In its      *
! * current form this is a dummy routine doing absolutely nothing; it  *
! * is included here for formal reasons but can (if desired) be        *
! * extended such that it indeed outputs the data.                     *
! *                                                                    *
! * Input parameters:                                                  *
! *   lanczvec1: first Lanczos vector.                                 *
! *   lanczvec2: second Lanczos vector.                                *
! *   lanczdim:  length of Lanczos vectors.                            *
! *   alpha:     on-diagonal elements of tridiagonal Lanczos matrix.   *
! *   beta:      off-diagonal elements of tridiagonal Lanczos matrix.  *
! *   order:     length of "alpha" and "beta" vectors.                 *
! *                                                                    *
! * Output parameters:                                                 *
! *   none                                                             *
! *                                                                    *
! * Other parameters:                                                  *
! *    lhpsi, ihpsi, rhpsi, chpsi: see above                           *
! *                                                                    *
! **********************************************************************

      subroutine lancz_restartout (lanczvec1, lanczvec2, lanczdim,&
                                   alpha, beta, order, lhpsi, ihpsi,&
                                   rhpsi, chpsi)

      implicit none

      logical(kind=4), dimension(*), intent(in)       :: lhpsi
      integer(long), intent(in)                       :: lanczdim,order
      integer(long), dimension(*), intent(in)         :: ihpsi
      real(dop), dimension(*), intent(in)             :: rhpsi
      real(dop), dimension(1:order), intent(in)       :: alpha,beta
      complex(dop), dimension(*), intent(in)          :: chpsi
      complex(dop), dimension(1:lanczdim), intent(in) :: lanczvec1,lanczvec2
      integer(long)                                   :: iunit
      logical(kind=4)                                 :: lprint

! --- WRITE RESTART DATA TO FILE ---

! Fill in your personal output commands here if desired.
! this is a dummy loop to allow clean compilation (otherwise all variable
! unused)
      lprint=.false.
      if (lprint) then
         write(iunit) lhpsi(1)
         write(iunit) lanczdim
         write(iunit) order
         write(iunit) ihpsi(1)
         write(iunit) rhpsi(1)
         write(iunit) alpha
         write(iunit) beta
         write(iunit) chpsi(1)
         write(iunit) lanczvec1
         write(iunit) lanczvec2
      endif


      return
      end subroutine lancz_restartout


! **********************************************************************
! *                                                                    *
! *                 SUBROUTINE DIRECTDIAG                              *
! *                                                                    *
! * Simple routine for direct diagonalisation of a Hamiltonian matrix  *
! *                                                                    *
! **********************************************************************
 
      subroutine directdiag(hops)

      implicit none
 
      integer(long)                              :: griddim,lwork,n
      integer(long)                              :: k,h,f,f1,m1,i,j,si,sf
      real(dop), dimension(hopsdim), intent(in)  :: hops
      real(dop), dimension(:,:,:,:), allocatable :: hamiltmat
      real(dop), dimension(:,:), allocatable     :: hamiltmat1
      real(dop), dimension(:), allocatable       :: workr
      real(dop), dimension(:), allocatable       :: eigval,eigint

      integer(long)     :: ierr
      character(len=10) :: cerr

      if (leb) then
         griddim = subdim(1)
      else
         griddim = subdim(1)*nddstate
      endif
      lwork = 3*griddim

      if (leb) then
         cerr = 'hamiltmat'
         allocate(hamiltmat(subdim(1),subdim(1),1,1),stat=ierr)
         if (ierr > 0) go to 1000
      else
         cerr = 'hamiltmat'
         allocate(hamiltmat(subdim(1),subdim(1),nddstate,nddstate),stat=ierr)
         if (ierr > 0) go to 1000
      endif
      cerr = 'hamiltmat1'
      allocate(hamiltmat1(griddim,griddim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'eigval'
      allocate(eigval(griddim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'eigint'
      allocate(eigint(griddim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'workr'
      allocate(workr(lwork),stat=ierr)
      if (ierr > 0) go to 1000

!-----------------------------------------------------------------------
! Build Hamiltonian Matrix
!-----------------------------------------------------------------------
      hamiltmat = 0.0_dop
      do k=1,khtot(1)
         si = ki(k)
         sf = kf(k)
! find first listed non-unit operator
         h=0
         do n=1,nspfdof(1)
            f=spfdof(n,1)
            h=hamilton(f,k)
            if (htmtype(h) .ne. 1) exit
         enddo
         if (htmdof(h) .gt. 0) then
            f1 = htmdof(h)    
            if (htmtype(h).eq. 1 .or. htmtype(h).eq. 2) then
               call addhvec1(hamiltmat(1,1,sf,si),koeff(k),hops(zhop(h)),&
                    vgdim(f1),gdim(f1),ngdim(f1))

            else if(htmtype(h) .eq. 8) then
               call addhmat1(hamiltmat(1,1,sf,si),koeff(k),hops(zhop(h)),&
                    vgdim(f1),gdim(f1),ngdim(f1))
            else
               routine = 'Directdiag (diagham.f90)'
               message = 'Can only use real operators'
               call errormsg
            endif
    
         else if (htmmode(h) .gt. 0) then
            m1 = htmmode(h)
            if (htmtype(h).eq. 1 .or. htmtype(h).eq. 2) then
               call addhvec(hamiltmat(1,1,sf,si),koeff(k),hops(zhop(h)),&
                    subdim(m1))
            else
               routine = 'Directdiag (diagham.f90)'
               message = 'Can only use real operators'
               call errormsg
            endif

         endif

      enddo

!-----------------------------------------------------------------------
! The Hamiltonian matrix needs to be re-ordered as the multi-set
! setup means the indices need to be swapped before diagonalisation
!-----------------------------------------------------------------------
      if (leb) then
         hamiltmat1(:,:)=hamiltmat(:,:,1,1)
      else
         call reorder_ham(hamiltmat,hamiltmat1,subdim(1),nddstate)
      endif

!-----------------------------------------------------------------------
! Diagonalise operator
! get all eigenvalues "eigval" and the corresponding normalized
! eigenvectors "evec" 
! of the real symmetric matrix "hamiltmat" 
!-----------------------------------------------------------------------
      call dsyev('V','U',griddim,hamiltmat1,griddim,eigval,workr,lwork,ierr)

      if (ierr.ne.0) then
         write (ilog,*) 'Error in subroutine DIRECTDIAG'
         write (ilog,*) 'Error diagonalising the Hamiltonian matrix'
         write (ilog,*) 'Error code : ',ierr
         stop
      endif

      eigint = 0.0
! calculate intensities from overlap of pis with eigenvectors
      eigint = 1.0

! --- WRITE EIGENVALUES TO FILE ---

      if (rlxunit .eq. 'au') then
         write(ieigval,'(a1,2x,a,7x,a,9x,a,13x,a)') &
            '#', 'Number', 'Eigenvalue [au]', 'Intensity'
      else
         write(ieigval,'(a1,2x,a,7x,a,9x,a,13x,a)') &
            '#', 'Number', 'Eigenvalue [eV]', 'Intensity'
      endif
      do i = 1,griddim
         write(ieigval,'(3x,i6,2x,f23.16,4x,1pe18.9e3)') &
            i, eigval(i)*ev, eigint(i)
      enddo

      if (leigvec) then
         do j = 1,griddim
            write(ieigvec,'(6f12.8)') (hamiltmat1(i,j), i=1,griddim)
            write(ieigvec,'(a)') '   '
         enddo 
      endif

      deallocate(hamiltmat,hamiltmat1,workr,eigval,eigint)
 
      return
 1000 write(ilog,'(a)') 'Error allocating memory in DIRECTDIAG :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      write(6,'(a)') 'Error allocating memory in DIRECTDIAG :',cerr
      write(6,'(a,i5)') 'Error Code :',ierr
      end subroutine directdiag

!#######################################################################

      subroutine addhvec1(hamiltmat,koeff,hops,vgdim1,gdim1,ngdim1)

      implicit none

      integer(long), intent(in) :: vgdim1,gdim1,ngdim1
      real(dop), intent(in), dimension(gdim1) :: hops
      real(dop), intent(inout), &
                dimension(vgdim1,gdim1,ngdim1,vgdim1,gdim1,ngdim1) :: hamiltmat
      complex(dop), intent(in)                         :: koeff
      integer(long) :: v,g,n

      do n=1,ngdim1
         do g=1,gdim1
            do v=1,vgdim1
               hamiltmat(v,g,n,v,g,n) = hamiltmat(v,g,n,v,g,n) +  &
                       dble(koeff)*hops(g)
            enddo
         enddo
      enddo

      return
      end subroutine addhvec1

!#######################################################################

      subroutine addhmat1(hamiltmat,koeff,hops,vgdim1,gdim1,ngdim1)

      implicit none

      integer(long), intent(in) :: vgdim1,gdim1,ngdim1
      real(dop), intent(in), dimension(gdim1,gdim1) :: hops
      real(dop), intent(inout), &
                dimension(vgdim1,gdim1,ngdim1,vgdim1,gdim1,ngdim1) :: hamiltmat
      complex(dop), intent(in)                         :: koeff
      integer(long) :: v,g,g1,n

      do n=1,ngdim1
         do g=1,gdim1
            do g1=1,gdim1
               do v=1,vgdim1
                  hamiltmat(v,g1,n,v,g,n) = hamiltmat(v,g1,n,v,g,n) +  &
                       dble(koeff)*hops(g1,g)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine addhmat1

!#######################################################################

      subroutine addhvec(hamiltmat,koeff,hops,subdim1)

      implicit none

      integer(long), intent(in) :: subdim1
      real(dop), intent(in), dimension(subdim1) :: hops
      real(dop), intent(inout), &
                dimension(subdim1,subdim1) :: hamiltmat
      complex(dop), intent(in)                         :: koeff
      integer(long) :: g

      do g=1,subdim1
         hamiltmat(g,g) = hamiltmat(g,g) + dble(koeff)*hops(g)
      enddo

      return
      end subroutine addhvec

!#######################################################################

      subroutine reorder_ham(hamiltmat,hamiltmat1,subdim1,nst1)

      implicit none

      integer(long), intent(in) :: subdim1,nst1
      integer(long)             :: g,g1,s,s1
      real(dop), intent(in), &
                dimension(subdim1,subdim1,nst1,nst1) :: hamiltmat
      real(dop), intent(out), &
                dimension(subdim1,nst1,subdim1,nst1) :: hamiltmat1

      do s=1,nst1
         do g=1,subdim1
            do s1=1,nst1
               do g1=1,subdim1
                  hamiltmat1(g1,s1,g,s) = hamiltmat(g1,g,s1,s) 
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine reorder_ham

!#######################################################################
!
!               WKDIAGHAM
!
! Here the workspace required for the module is assigned
!
! workcdim, workrdim, workidim and workldim are passed via a common
! block in global.inc, and are checked that they are large enough for
! the module.
!
!#######################################################################

      subroutine wkdiagham

      implicit none

!-----------------------------------------------------------------------
! assign workspace used in module
! Needed for Lanczos routine
!-----------------------------------------------------------------------
      workcdim=workcdim
      workrdim=workrdim
      if (leigvec) then
         workrdim = max(workrdim,lanczorder**2+4*lanczorder-1)
      else
         workrdim = max(workrdim,6*lanczorder-1)
      endif
      workidim=max(workidim,lanczorder)
      workldim=workldim

      return
      end subroutine wkdiagham

!***********************************************************************

      end module diaghammod

