! **********************************************************************
      module dicht1phimod

use decimal
use timing
use mmlib,  only: mqxtzz
use op1lib, only: cpmxz

      implicit none
      private
      public :: dicht1phi,dicht1phi1ms
      contains

! **********************************************************************
!                                                                       
!             module DICHT1PHI
!                                                                       
!   Contains:                                                           
!     dicht1phi:    Multiplies for all modes and states the             
!                   single-particle functions with the inverse of the   
!                   density matrix.                                     
!     dicht1phi1ms: Multiplies for one mode and state the               
!                   single-particle functions with the inverse of the   
!                   density matrix.                                     
!                                                                       
! *** V6.0 MB *************> SUBROUTINE DICHT1PHI <*********************
!                           ~~~~~~~~~~~~~~~~~~~~~~
! Multiplies for all modes and states the single-particle functions
! with the inverse of the density matrix.
! **********************************************************************

      subroutine dicht1phi (dtpsi,dicht1)

!$ use omp_lib
use global
use griddatmod
use operdef
use psidef
use runpropmod
use openmpmod

use maxv, only: maxdim,maxsta,maxkoe

      implicit none

      integer(long)                                  :: m,s
      integer(long), save                            :: tid=0
      complex(dop), dimension(dgldim), intent(inout) :: dtpsi
      complex(dop), dimension(dmatdim), intent(in)   :: dicht1

      call get_timer(tid,'Dicht1Phi')
      call start_timer(tid)

! --- LOOP OVER EACH MODE AND STATE---

! omp parallel num_threads(ompthread) 
! omp do private(m,s) 
         do m = 1,nmode
            do s = 1,nstate
               call dicht1phi1ms(dtpsi(zetf(m,s)),dicht1,m,s,dim(m,s),subdim(m))
            enddo
         enddo
! omp end do
! omp end parallel 

      call stop_timer(tid)

      return
      end subroutine dicht1phi

! *** V6.0 MB ************> SUBROUTINE DICHT1PHI 1MS <******************
!                          ~~~~~~~~~~~~~~~~~~~~~~~~~~
! Multiplies for one mode and one state the single-particle functions
! with the inverse of the density matrix.
! **********************************************************************

      subroutine dicht1phi1ms (dtpsi,dicht1,m,s,dim1,subdim1)

use global
use dvrdatmod
use griddatmod
use psidef

      implicit none

      integer(long)                                        :: f
      integer(long), intent(in)                            :: m,s,dim1,subdim1
      complex(dop), dimension(dmatdim), intent(in)         :: dicht1
      complex(dop), dimension(subdim1,dim1)                :: workc
      complex(dop), dimension(subdim1,dim1), intent(inout) :: dtpsi

! --- IF BASIS IS COMPLETE, DO NOT CALCULATE ---
      if(complete(m)) return

! --- IF GWP BASIS RETURN ---
      if (gwpm(m)) return

! --- IF X-local mode, do not calculate

      f = spfdof(1,m)
      if(basis(f) .eq. 26 .or. basis(f) .eq. 27)return

! --- MULTIPLY SINGLE-PARTICLE FUNCTION WITH INVERSE DENSITY MATRIX ---

      call cpmxz(dtpsi,workc,subdim(m),dim(m,s))
      call mqxtzz(workc,dicht1(dmat(m,s)),dtpsi,subdim(m),dim(m,s))

      return
      end subroutine dicht1phi1ms

!#######################################################################

      end module dicht1phimod


