!***********************************************************************
!
!                  MCTDH module dichtout
!
! calculates grid populations in both "coordinate" and "momentum /
!   occupation" representations, and outputs results to gridpop file.
!
!   test for the automatic activation of the CAP / calls to activcap
!
! calls to firstpop, secpop0, secpop2.
!
! passed back to calling routine:
!   gpop1(m,s): first set of grid populations 
!   gpop2(m,s): second set of grid populations 
!
! gpop1 contains:
!   if time independent basis, basis function population
!   if DVR of FFT, coordinate representation populations
! gpop2 contains:
!   if time independent basis, basis function population (identical to 
!      gpop1)
!   if DVR, FBR basis function occupation numbers
!   if FFT, momentum representation populations
!
! if lpop2=.false. only gpop1 calculated. 
! V6.3  GW
!
!***********************************************************************

      module dichtoutmod

use decimal, only: long, dop
use mmlib,   only: mqxczz, qqxxzz, qqaxzz, dqxxzz, mmxxzz, mmaxzz
use mtlib,   only: qtxxdz, mtxxdz, qtxxzz, qtaxzz
use lalib, only: tranmxtzz
use fftmod,  only: fft
use op1lib,  only: zerovxd, zeromxd, zeromxz, cpmxz


use global
use globmemmod
use maxv,  only: maxdim
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use channels
use logdat

use daten, only: activcap
use psiutil, only: trkleg,qtexp,pleghin,plegrueck,trwigner
use densitymod, only: density
use eqofmotion1, only: compute_dmat_tree,compute_hterms_treer
use hpsimod, only: hpsidim,pmatdim,ghmatdim
use calchamod, only:calcha,calcha_ml
use d1calchamod
use d2calchamod
use phihphimod, only: phihphi
use hphimod, only: hphi
use d1hphimod, only: d1hphi
use d1phihphimod, only: d1phihphi
use gh_elementsmod

use d1lib, only: trspdo
use mtlib, only: dtxxdzo
use hpsimod, only:dzhpsi,zhpsi,pmat
use linear, only: ijk_shape
 
      implicit none
      private
      public :: dichtout,calcgpop,stproj,gwp2grid,gwpgpopwei
      contains

!***********************************************************************

      subroutine dichtout(psi,dicht3,gpop1,gpop2,trafo,fftfak, &
                 rueck,exphin,exprueck,jsph,msph,time,ort,lpop2,agmat,&
                 jindx,trajst)

      implicit none

      integer(long)       :: kmin,kdim,f1,vggdim,s,m,f,n,&
                             k,kk,ggdim,subdm,s1,nst

      integer(long), dimension(fftdim), intent(in)     :: fftfak
      integer(long), dimension(sphdim), intent(in)     :: jsph,msph
      integer(long), dimension(jindxdim), intent(in)   :: jindx
      integer(long), dimension(gwpinfodim), intent(in) :: trajst
      real(dop)                                        :: tgpop
      real(dop), intent(in)                            :: time
      real(dop), dimension(d3matdim), intent(inout)    :: dicht3
      real(dop), dimension(dvrdim), intent(in)         :: trafo
      real(dop), dimension(ortdim), intent(in)         :: ort
      real(dop), dimension(2*maxgdim)                  :: gp
      real(dop), dimension(ortdim,nstate), intent(out) :: gpop1,gpop2
      real(dop), dimension(ortdim,nddstate*npacktot)   :: tmpgpop1,tmpgpop2
      complex(dop), dimension(dgldim), intent(in)      :: psi
      complex(dop), dimension(dgldim)                  :: tmppsi
      complex(dop), dimension(fftdim), intent(in)      :: rueck
      complex(dop), dimension(expdim), intent(in)      :: exphin,exprueck
      complex(dop), dimension(agmatdim), intent(in)    :: agmat
      complex(dop), dimension(sdim*maxspf)             :: tmpspf1,tmpspf2,tmpspf3
      logical(kind=4), intent(in)                      :: lpop2
      character(len=c2)                                :: wrtag

!-----------------------------------------------------------------------
! copy densities to the temp arrays used below 
! Cases:
! lmult .and. lgpel: nstate sets of densites
! leb .and. lgpel  : nddstate*npacktot sets of densites
! .not. lgpel      : 1 summed density 
!
! remember that it can be a multi-packet as well as multi-state 
! calculation. Multi-packet + SS to be checked!
!-----------------------------------------------------------------------
      if (leb .and. (npackts .gt. 1)) then
         routine = 'Dichtout (dichtoutmod.f90)'
         message = 'Single-set + multi-packet density output needs to be checked'
         call errormsg
      endif
      if (leb .and. meb .gt. 0) then
         if (nspfdof(meb) .gt. 1) then
            routine='Stprojpsi (dichtoutmod.f90)'
            message=&
        'State projection does not work for combined mode electronic basis'
            call errormsg
         endif
      endif

      if (lmult .or. .not. lgpel) then
         nst = nstate

      else
         nst = nddstate*npacktot

      endif

!-----------------------------------------------------------------------
! Start new particle density data set
!-----------------------------------------------------------------------
      if(lpdens .and. lpop2) then
         countpden=countpden+1
      
         write(wrtag,'(a9,i10,a1)') '<data set',countpden,'>'
         write(ipden) wrtag
      endif

      gpop1=0.0_dop
      tmpgpop1=0.0_dop
!-----------------------------------------------------------------------
! calculate first set of grid populations
!  1. transform to natural orbitals
!  2. sum up populations on grid points for each degree of freedom
!
! loop runs over electronic states (unless lgpel = .false.)
! and s is set to 1 for single-set or else to state no. if multi-set.
! trajectory calculations are single-set
!-----------------------------------------------------------------------
      do s1=1,nst
! get state dependent densities for single-set calculations
         if (ltraj) then
            s = 1
            call stprojtr(s1,psi,tmppsi,dim(1,1),nddstate)
            call density(tmppsi,tree,dicht1,dicht2,dicht3,dicht4,jindx,gs2)

         else if (leb .and. lgpel .and. nst .gt. 1) then
            s=1
! project wavefunction onto state s1
            call stproj(s1,s1,psi,tmppsi)

            if (lrunml) then
               call compute_dmat_tree(tree,tmppsi)
            else
               call density(tmppsi,tree,dicht1,dicht2,dicht3,dicht4,jindx,gs2)
            endif
         else
            s = s1
         endif

         do m=1,nmode
            f1=spfdof(1,m)
            if (basis(f1) .eq. 25) then
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  call gwp2grid(n,f,agmat(zagmat(m,s)),nspfdof(m), &
                       agmat(zxivec(m,s)),agmat(zphase(m,s)),dim(m,s), & 
                       gdim(f),ort(zort(f)),dicht2(dmat(m,s)), &
                       tmpgpop1(zort(f),s1),psi(zdgwp(m,s)))
               enddo

            else if (basis(f1) .eq. 27) then
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  call traj2grid(n,nspfdof(m),psi(zetf(m,s)),dim(m,s), &
                       gdim(f),ort(zort(f)),tmpgpop1(zort(f),s1),&
                       ltraj,trajst,s1)
               enddo

            else
               call zeromxz(tmpspf1,subdim(m),dim(m,s))
               call mqxczz(psi(zetf(m,s)),dicht4(dmat(m,s)),tmpspf1, &
                    subdim(m),dim(m,s))
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  call calcgpop(tmpspf1,dicht3(d3mat(m,s)),dim(m,s), &
                       vgdim(f),gdim(f),ngdim(f),tmpgpop1(zort(f),s1))
               enddo

               if(lpdens .and. lpop2) then
                  if(npd1.eq.0 .or. npd1.eq.m .or. npd2.eq.m .or.  &
                                 npd3.eq.m .or. npd4.eq.m) then 
                     wrtag='<mode state>'
                     write(ipden) wrtag
                     write(ipden) m,s
                     wrtag='<dicht3>'
                     write(ipden) wrtag
                     write(ipden) (dicht3(d3mat(m,s)+k), &
                               k=0,dim(m,s)-1)
                     wrtag='<spfs>'
                     write(ipden) wrtag
                     write(ipden) (tmpspf1(k),k=1,subdim(m)*dim(m,s))
                  endif
               endif

            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! end particle density data set
!-----------------------------------------------------------------------
      if(lpdens .and. lpop2) then
         write(wrtag,'(a13,i10,a1)') '<end data set',countpden,'>'
         write(ipden) wrtag
      endif

!-----------------------------------------------------------------------
! return if only gpop1 required
!
! - restore full density matrices if projected ones have been used
! - copy (or sum up for ss) gpop1 to use later
!-----------------------------------------------------------------------
      if (.not. lpop2) then
       !  if (ltraj) then
       !     continue
       !  else if (leb .and. lgpel .and. nst .gt. 1) then
         if (leb .and. lgpel .and. nst .gt. 1) then
            if (lrunml) then
               call compute_dmat_tree(tree,psi)
            else
               call density(psi,tree,dicht1,dicht2,dicht3,dicht4,jindx,gs2)
            endif
         endif
         if (leb) then
            do s=1,nst
               do k=1,ortdim
                  gpop1(k,1) = gpop1(k,1) + tmpgpop1(k,s)
               enddo
            enddo

         else
            gpop1(:,:) = tmpgpop1(:,:)

         endif
         go to 300
      endif

!-----------------------------------------------------------------------
! carry on for second populations
!-----------------------------------------------------------------------
      if (lpdens) call myflush(ipden)

      gpop2=0.0_dop
      tmpgpop2=0.0_dop

!-----------------------------------------------------------------------
! Get basis identification and calculate second set of grid populations
!-----------------------------------------------------------------------
      do s1=1,nst

         if (ltraj) then
            s = 1
            call stprojtr(s1,psi,tmppsi,dim(1,1),nddstate)
            call density(tmppsi,tree,dicht1,dicht2,dicht3,dicht4,jindx,gs2)

         else if (leb .and. lgpel .and. nst .gt. 1) then
! get state dependent densities for single-set calculations
            s = 1

            call stproj(s1,s1,psi,tmppsi)

            if (lrunml) then
               call compute_dmat_tree(tree,tmppsi,psi)
            else
               call density(tmppsi,tree,dicht1,dicht2,dicht3,dicht4,jindx,gs2,psi)
            endif

         else
            s = s1
         endif

         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)

               if (basis(f).eq.0 .or. basis(f) .eq. 10) then
                  call secpop0 (tmpgpop1(zort(f),s1),tmpgpop2(zort(f),s1),gdim(f))

               elseif (basis(f).eq.4) then
                  call secpop2 (psi(zetf(m,s)),dicht3(d3mat(m,s)),&
                       dicht4(dmat(m,s)),rueck(zfft(f)),fftfak(zfft(f)),&
                       dim(m,s),vgdim(f),gdim(f),ngdim(f),tmpgpop2(zort(f),s1))

               elseif (basis(f).eq.6) then
                  call sphpop(gpop1(zort(f),s),tmpgpop2(zort(f),s1),gdim(f),&
                       jsph(zsph(f)),msph(zsph(f)),f)

               elseif (basis(f).eq.-1) then
                  continue
               elseif (basis(f).ge.25 .and. basis(f) .le. 27) then
                  continue

               elseif (ldvr(f)) then
                  if(basis(f) .eq. 5)  then   ! exp
                     if (basis(max(1,f-1)).eq.9 .or. &
                         basis(max(1,f-1)).eq.21 .or. &
                         basis(max(1,f-1)).eq.33) then
                        kmin=ipbaspar(1,f)
                        kdim=ipbaspar(3,f)
                     elseif (basis(max(1,f-2)).eq.33) then
                        kmin=ipbaspar(6,f-2)
                        kdim=ipbaspar(8,f-2)
                     else
                        kmin=-gdim(f)/2
                        kdim=gdim(f)
                     endif
                     call qtexp(exphin(zexp(f)),psi(zetf(m,s)),&
                                tmpspf2,vgdim(f),gdim(f),&
                                ngdim(f)*dim(m,s),kdim,kmin)

                  elseif( basis(f) .eq. 7 .or. basis(f) .eq. 20) then  ! KLeg KLeg/R
                     ggdim = gdim(f)
                     if(basis(f) .eq. 20) ggdim = ipbaspar(3,f+1)
                     call trkleg(trafo(zdvr(f)),psi(zetf(m,s)),&
                          tmpspf2,ggdim,ipbaspar(1,f),&
                          ipbaspar(1,f+1),ipbaspar(3,f+1),vgdim(f),&
                          gdim(f),gdim(f+1),ngdim(f+1),dim(m,s))

                  elseif (basis(f).eq.9 .or. basis(f).eq.21) then ! PLeg  PLeg/R
                     call pleghin(psi(zetf(m,s)),exphin(zexp(f+1)),&
                                 tmpspf1,vgdim(f+1),gdim(f+1),&
                                 ngdim(f+1),ipbaspar(3,f+1),dim(m,s))
                          ggdim = gdim(f)
                          vggdim = vgdim(f+1)
                          if (basis(f) .eq. 21) then
                             ggdim = ipbaspar(3,f)
                             vggdim = (vgdim(f+1)/gdim(f)) * ggdim
                          endif

                     call trkleg(trafo(zdvr(f)),tmpspf1,&
                              tmpspf3,ggdim,ipbaspar(1,f),&
                              ipbaspar(1,f+1),ipbaspar(4,f+1),vgdim(f),&
                              gdim(f),ipbaspar(3,f+1),ngdim(f+1),dim(m,s))
                     call plegrueck(tmpspf3,exprueck(zexp(f+1)),&
                                 tmpspf2,vggdim,gdim(f+1),&
                                 ngdim(f+1),ipbaspar(3,f+1),dim(m,s))

                  elseif (basis(f).eq.33) then   ! Wigner
                     if (basis(f+1).eq.5 .and. basis(f+2).eq.5) then
                       call pleghin(psi(zetf(m,s)),exphin(zexp(f+1)), &
                            tmpspf2,vgdim(f+1),gdim(f+1), &
                            ngdim(f+1),ipbaspar(4,f),dim(m,s))
                       call pleghin(tmpspf2,exphin(zexp(f+2)), &
                            tmpspf1,vgdim(f+1)*ipbaspar(4,f), &
                            gdim(f+2),ngdim(f+2),ipbaspar(8,f),dim(m,s))
                       call trwigner(trafo(zdvr(f)),tmpspf1, &
                            tmpspf2,ipbaspar(1,f),ipbaspar(2,f), &
                            ipbaspar(5,f),ipbaspar(6,f),ipbaspar(9,f), &
                            vgdim(f),gdim(f),ipbaspar(4,f),ipbaspar(8,f), &
                            ngdim(f+2),dim(m,s))
                       call plegrueck(tmpspf2,exprueck(zexp(f+2)), &
                            tmpspf1,vgdim(f+1)*ipbaspar(4,f), &
                            gdim(f+2),ngdim(f+2),ipbaspar(8,f),dim(m,s))
                       call plegrueck(tmpspf1,exprueck(zexp(f+1)), &
                            tmpspf2,vgdim(f+1),gdim(f+1), &
                            ngdim(f+1),ipbaspar(4,f),dim(m,s))
                     elseif (basis(f+1).eq.5 .and. basis(f+2).eq.8) then
                       call pleghin(tmpspf1,exphin(zexp(f+1)), &
                            tmpspf2,vgdim(f+1),gdim(f+1), &
                            ngdim(f+1),ipbaspar(4,f),dim(m,s))
                       call trwigner(trafo(zdvr(f)),tmpspf2, &
                            tmpspf1,ipbaspar(1,f),ipbaspar(2,f), &
                            ipbaspar(5,f),ipbaspar(6,f),ipbaspar(9,f), &
                            vgdim(f),gdim(f),ipbaspar(4,f),ipbaspar(8,f), &
                            ngdim(f+2),dim(m,s))
                       call plegrueck(tmpspf1,exprueck(zexp(f+1)), &
                            tmpspf2,vgdim(f+1),gdim(f+1), &
                            ngdim(f+1),ipbaspar(4,f),dim(m,s))
                     elseif (basis(f+1).eq.8 .and. basis(f+2).eq.5) then
                       call pleghin(tmpspf1,exphin(zexp(f+2)), &
                            tmpspf2,vgdim(f+1)*ipbaspar(4,f), &
                            gdim(f+2),ngdim(f+2),ipbaspar(8,f),dim(m,s))
                       call trwigner(trafo(zdvr(f)),tmpspf2, &
                            tmpspf1,ipbaspar(1,f),ipbaspar(2,f), &
                            ipbaspar(5,f),ipbaspar(6,f),ipbaspar(9,f), &
                            vgdim(f),gdim(f),ipbaspar(4,f),ipbaspar(8,f), &
                            ngdim(f+2),dim(m,s))
                       call plegrueck(tmpspf1,exprueck(zexp(f+2)), &
                            tmpspf2,vgdim(f+1)*ipbaspar(4,f), &
                            gdim(f+2),ngdim(f+2),ipbaspar(8,f),dim(m,s))
                     elseif (basis(f+1).eq.8 .and. basis(f+2).eq.8) then
                       call trwigner(trafo(zdvr(f)),tmpspf1, &
                            tmpspf2,ipbaspar(1,f),ipbaspar(2,f), &
                            ipbaspar(5,f),ipbaspar(6,f),ipbaspar(9,f), &
                            vgdim(f),gdim(f),ipbaspar(4,f),ipbaspar(8,f), &
                            ngdim(f+2),dim(m,s))
                     endif

                  elseif (basis(f).eq.12) then   ! Leg/r
                     call mtxxdz(trafo(zdvr(f)),psi(zetf(m,s)),&
                          tmpspf2,vgdim(f),gdim(f),&
                          ngdim(f)*dim(m,s),ipbaspar(3,f))
                  else
                     call qtxxdz(trafo(zdvr(f)),psi(zetf(m,s)),&
                                 tmpspf2,vgdim(f),gdim(f),&
                                 ngdim(f)*dim(m,s))
                  endif

                  if(basis(f).eq.12 .or. basis(f).eq.20 .or. &
                     basis(f).eq.21) then        ! Leg/r, Kleg/r or Pleg/r 
                     ggdim = ipbaspar(3,f) 
                     subdm = ggdim*subdim(m)/gdim(f)
                     call mqxczz(tmpspf2,dicht4(dmat(m,s)),&
                          tmpspf1,subdm,dim(m,s))
                     call calcgpop(tmpspf1,dicht3(d3mat(m,s)), &
                          dim(m,s),vgdim(f),ggdim,ngdim(f), &
                          tmpgpop2(zort(f),s1))

                  else
                     call mqxczz(tmpspf2,dicht4(dmat(m,s)), &
                          tmpspf1,subdim(m),dim(m,s))
                     call calcgpop(tmpspf1,dicht3(d3mat(m,s)), &
                          dim(m,s),vgdim(f),gdim(f),ngdim(f), &
                          tmpgpop2(zort(f),s1))

                  endif
               endif
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! output grid populations in single precision to gridpop file
!-----------------------------------------------------------------------
      if (lgpop) then
         countrho=countrho+1

!-----------------------------------------------------------------------
! Write Data Set
!-----------------------------------------------------------------------
         write(wrtag,'(a9,i10,a1)') '<data set',countrho,'>'
         write(irho) wrtag

         wrtag='<gpop>'
         write(irho) wrtag
         if (lgpel) then
            do s=1,nst
               do m=1,nmode
                  do n=1,nspfdof(m)
                     f=spfdof(n,m)
                     ggdim = gdim(f)
                     if(basis(f).eq.12 .or.basis(f).eq.20 .or.  &
                        basis(f).eq.21) ggdim = ipbaspar(3,f) 
                     write(irho) (real(tmpgpop1(zort(f)+k,s)),k=0,gdim(f)-1), &
                                 (real(tmpgpop2(zort(f)+k,s)),k=0,ggdim-1)
                  enddo
               enddo
            enddo

         else
            do m=1,nmode
               do n=1,nspfdof(m)
                  f=spfdof(n,m)
                  ggdim = gdim(f)
                  if(basis(f).eq.12 .or. basis(f).eq.20 .or.  &
                     basis(f).eq.21)  &
                     ggdim = ipbaspar(3,f) 
                  do k = 1, gdim(f)+ggdim
                     gp(k) = 0.0_dop
                  enddo
                  do s=1,nst
                     do k = 1, gdim(f)
                        gp(k) = gp(k) + tmpgpop1(zort(f)+k-1,s)
                     enddo
                     do k = 1, ggdim
                        kk = k + gdim(f)
                        gp(kk) = gp(kk) + tmpgpop2(zort(f)+k-1,s)
                     enddo
                  enddo
                  write(irho) (real(gp(k)),k=1,gdim(f)), &
                              (real(gp(k)),k=gdim(f)+1,gdim(f)+ggdim) 
               enddo
            enddo
         endif
         
         write(wrtag,'(a13,i10,a1)') '<end data set',countrho,'>'
         write(irho) wrtag
         call myflush(irho)
      endif

!-----------------------------------------------------------------------
! sum up grid populations into gpop arrays for single set
! or simply copy for muti-set
!-----------------------------------------------------------------------
      if (leb) then
         do s=1,nst
            do k=1,ortdim
               gpop1(k,1) = gpop1(k,1) + tmpgpop1(k,s)
               gpop2(k,1) = gpop2(k,1) + tmpgpop2(k,s)
            enddo
         enddo

      else
         gpop1(:,:) = tmpgpop1(:,:)
         gpop2(:,:) = tmpgpop2(:,:)

      endif

!-----------------------------------------------------------------------
! restore full density matrices for ML-MCTDH
!-----------------------------------------------------------------------
      if (ltraj) then
         continue
      else if (leb .and. lgpel .and. nst .gt. 1) then
         if (lrunml) then
            call compute_dmat_tree(tree,psi)
         else
            call density(psi,tree,dicht1,dicht2,dicht3,dicht4,jindx,gs2)
         endif
      endif

!-----------------------------------------------------------------------
! turn on autocap if grid population has entered it
!-----------------------------------------------------------------------
  300 n=0
      do k=1,khtot(1)
         if (autocap(k) .gt. 0) then
            f=autocap(k)
            n=n+1
            tgpop=0.0_dop
            do s=1,nstate
               tgpop=tgpop+gpop1((zort(f)+indicap(f)-1),s)
            enddo
            call activcap(tgpop,autocap(k),gpopcap(f),timecap(n), &
                          time/fs,ilog,modelabel(f),countpsi,f,n)
         endif
      enddo

      return
      end subroutine dichtout

!#######################################################################
!
! project out coefficients for a single state in a trajectory
! calculation, i.e. a single-set WF when the states are definitely 
! ordered 1,2,3....
!
!#######################################################################

      subroutine stprojtr(st,psi,tmppsi,ntraj,nst)

      implicit none

      integer(long), intent(in) :: st,ntraj,nst
      complex(dop), dimension(ntraj,nst), intent(in)   :: psi
      complex(dop), dimension(ntraj,nst), intent(out)  :: tmppsi

      tmppsi(:,:) = 0.0_dop
      tmppsi(:,st) = psi(:,st)

      end subroutine stprojtr

!#######################################################################
!
! STPROJ
! 
! project out the wavepacket for a single-state, s1, from a 
! single-set wavefunction
!
!#######################################################################

      subroutine stproj(s1,s2,psi,projpsi)

      implicit none

      integer(long), intent(in) :: s1,s2
      integer(long)             :: meb1,f,s,m,vdim1,dim1,ndim1,ierr
      integer(long), dimension(3)                   :: ijk
      complex(dop), dimension(:,:), allocatable     :: proj,ztmp,proj1,ztmp1
      complex(dop), dimension(dgldim), intent(in)   :: psi
      complex(dop), dimension(dgldim), intent(out)  :: projpsi
      type(wfnode_t),pointer                        :: childf

      allocate(proj(nddstate,nddstate))
      if (dentype .eq. 0 .or. dentype .eq. 2) then
         allocate(ztmp(nddstate,dim(meb,1)),stat=ierr)
         if (ierr > 0) go to 1000
      else if (dentype .eq. 1) then
         allocate(proj1(nddstate*nddstate,nddstate*nddstate),stat=ierr)
         if (ierr > 0) go to 1000
         allocate(ztmp1(nddstate*nddstate,nddstate*nddstate),stat=ierr)
         if (ierr > 0) go to 1000
      endif

!-----------------------------------------------------------------------
! Get array dimensions
!-----------------------------------------------------------------------
      m=meb
      s=1
      if (psitype .eq. 5) then
! find node containing meb and then get vdim,ndim
         tz => tree%swap_down(1)%p
         do f=1,tree%nmodes
            childf => tz%submodes(f)%p
            if (childf%m .eq. meb) meb1=f
         enddo

         ijk = ijk_shape(meb1,tree%ndim)
         vdim1 = ijk(1)
         dim1  = ijk(2)
         ndim1 = ijk(3)
      else if (dentype .eq. 1) then
         vdim1 = vddim(m,s,s)
         dim1  = ddim(m,s,s)
         ndim1 = nddim(m,s,s)

      else
         vdim1 = vdim(m,s)
         dim1  = dim(m,s)
         ndim1 = ndim(m,s)
      endif

!-----------------------------------------------------------------------
! set up projector
!
! ztmp is SPFs for electronic DOF projected onto |s2><s1|
!-----------------------------------------------------------------------
      proj = 0.0_dop
      proj(s2,s1) = 1.0_dop
      if (dentype .eq. 0 .or. dentype .eq. 2) then
         call mmxxzz(proj,psi(zetf(m,s)),ztmp,nddstate,nddstate,dim(m,1))
         call mmaxzz(psi(zetf(m,s)),ztmp,proj,nddstate,dim(m,1),dim(m,1))
      else if (dentype .eq. 1) then
         call dqxxzz(proj,psi(dzetf(m,1,1)),ztmp1,nddstate*nddstate)
         call qqaxzz(psi(dzetf(m,1,1)),ztmp1,proj1,nddstate*nddstate)
      else
         routine = 'Stproj (dichtoutmod.f90)'
         write(message,'(a,i5)') 'Unknown density type : ',dentype
         call errormsg
      endif

!-----------------------------------------------------------------------
! ML-MCTDH has eb in layer 1 so can be dealt with the same as MCTDH
!-----------------------------------------------------------------------
      if (dentype .eq. 0) then
         call qtxxzz(proj,psi,projpsi,vdim1,dim1,ndim1)

      else if (dentype .eq. 1) then
         call qtxxzz(proj1,psi,projpsi,vdim1,dim1,ndim1)

      else if (dentype .eq. 2) then
         call d2stproj(psi,projpsi,proj,vdim1,dim1,ndim1)

      endif

!-----------------------------------------------------------------------
! Copy SPFs to projected wavefunction
!-----------------------------------------------------------------------
      projpsi(adim+1:dgldim) = psi(adim+1:dgldim)

      deallocate(proj)
      if (dentype .eq. 0 .or. dentype .eq. 2) then
         deallocate(ztmp)
      else if (dentype .eq. 1) then
         deallocate(proj1,ztmp1)
      endif

      return
 1000 routine = 'Stproj (dichtoutmod.F90) '
      message = 'Cannot allocate memory for state projection. Try gridpop=no_el'
      call errormsg

      end subroutine stproj

!#######################################################################
!
! STPROJ
! 
! project out the wavepacket for a single-state, s1, from a 
! single-set wavefunction
!
!#######################################################################

      subroutine d2stproj(psi,tmppsi,proj,vdim1,dim1,ndim1)

      implicit none

      integer(long), intent(in) :: vdim1,dim1,ndim1
      integer(long)             :: v,e,n,v1,e1,n1
      complex(dop), dimension(dim1,dim1), intent(in) :: proj
      complex(dop), dimension(vdim1,dim1,ndim1,vdim1,dim1,ndim1), &
                                                 intent(in)   :: psi
      complex(dop), dimension(vdim1,dim1,ndim1,vdim1,dim1,ndim1), &
                                                 intent(out)  :: tmppsi

      do n1=1,ndim1
         do e1=1,dim1
            do n=1,ndim1
               do v1=1,vdim1
                  do e=1,dim1
                     do v=1,vdim1
                        tmppsi(v,e,n,v1,e1,n1) = &
                               psi(v,e,n,v1,e1,n1)*proj(e1,e)
                     enddo
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine d2stproj

!#######################################################################
! CALCGPOP
!
! For time independent bases, calculates the populations of the
!   natural orbitals
! For DVR or FFT bases, calculates the populations of the grid points 
!   in the coordinate representation
!
! input for a single mode and state:
!  dicht3: eigenvalues of density matrix
!  dim   : number of single particle functions
!  gdim  : number of grid points
!  natur : natural orbitals
! output:
!  gpop  : population of grid points
!
! V6.3 GW
!#######################################################################

      subroutine calcgpop(natur,dicht3,dim,vgdim,gdim,ngdim,gpop)

      implicit none

      integer(long)                                      :: e,g,n,v
      integer(long), intent(in)                          :: dim,vgdim,&
                                                            ngdim,gdim
      real(dop), dimension(dim), intent(in)              :: dicht3
      real(dop), dimension(gdim), intent(inout)          :: gpop
      complex(dop), dimension(vgdim,gdim,ngdim,dim), intent(in) :: natur

!-----------------------------------------------------------------------
! Calculate populations
!-----------------------------------------------------------------------
      do e=1,dim
         do n=1,ngdim
            do g=1,gdim
               do v=1,vgdim
                  gpop(g)=gpop(g)+ &
                    dicht3(e)*dble(dconjg(natur(v,g,n,e))*natur(v,g,n,e))
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine calcgpop

!#######################################################################
! SECPOP0
!
! For time independent bases, no further populations required and so
!  gpop2 is merely set equal to gpop1.
!
! input for a single mode and state:
!  gpop1  : population of natural orbitals points
!  gdim   : number of grid points
! output:
!  gpop2  : population of natural orbitals points
!
! V6.3 GW
!#######################################################################

      subroutine secpop0(gpop1,gpop2,gdim)

      implicit none

      integer(long)                           :: g
      integer(long), intent(in)               :: gdim
      real(dop), dimension(gdim), intent(in)  :: gpop1
      real(dop), dimension(gdim), intent(out) :: gpop2

      do g=1,gdim
         gpop2(g)=gpop1(g)
      enddo

      return
      end subroutine secpop0


!***********************************************************************
!
!               Subroutine SECPOP2 
!
! for an FFT basis, calculates the population of the grid points in 
!   the momentum representation. 
!
! input for a single state (FFT basis):
!  psi   : single particle functions in position representation
!  dicht3: eigenvalues of density matrix
!  dicht4: eigenvectors of density matrix
!  rueck : array for position --> momentum FFT transformation
!  fftfak: FFT decomposition factors 
!  dim   : number of single particle functions
!  gdim  : number of grid points
! calculated in routine:
!  ppsi  : single particle functions in momentum representation
!  natur : natural orbitals in momentum representation
! output:
!  gpop2 : populations in momentum representation
!
! V6.3 GW
!
!***********************************************************************

      subroutine secpop2(psi,dicht3,dicht4,rueck,fftfak,dim, &
                 vgdim,gdim,ngdim,gpop2)

      implicit none


      integer(long)                                              :: subdim,g,e
      integer(long), intent(in)                                  :: dim,vgdim,&
                                                                    gdim,ngdim
      integer(long), dimension(gdim), intent(in)                 :: fftfak
      complex(dop), dimension(gdim), intent(in)                  :: rueck
      complex(dop), dimension(dim,dim), intent(in)               :: dicht4
      complex(dop), dimension(vgdim,gdim,ngdim,dim)              :: natur
      complex(dop), dimension(vgdim,gdim,ngdim,dim)              :: ppsi
      complex(dop), dimension(vgdim,gdim,ngdim,dim), intent(in)  :: psi
      real(dop), dimension(dim), intent(in)                      :: dicht3
      real(dop), dimension(gdim), intent(inout)                  :: gpop2

      subdim=vgdim*gdim*ngdim
!-----------------------------------------------------------------------
! Transform single-particle functions to momentum representation
! natur used as work space
!-----------------------------------------------------------------------
      call cpmxz(psi,ppsi,subdim,dim)
      natur=0.0_dop
      do e=1,dim
         call fft (ppsi(1,1,1,e),natur(1,1,1,e),rueck,fftfak,&
              gdim,vgdim,ngdim,.true.)
      enddo

!-----------------------------------------------------------------------
! Calculate natural single-particle functions in momentum representation
!-----------------------------------------------------------------------
      call zeromxz(natur,subdim,dim)

      call mqxczz(ppsi,dicht4,natur,subdim,dim)

!-----------------------------------------------------------------------
! Calculate populations in momentum representation
! and momentum expectation values
!-----------------------------------------------------------------------
      call calcgpop(natur,dicht3,dim,vgdim,gdim,ngdim,gpop2)

      do g=1,gdim
         gpop2(g)=gpop2(g)/real(gdim)
      enddo

      return
      end subroutine secpop2

!#######################################################################
! SPHPOP
!
! called by dichtout. Computes the 1D populations of the shpFBR.
!
! 05/00 HDM
!#######################################################################

      subroutine  sphpop(gpop1,gpop2,gdim1,jsph,msph,f)

      implicit none

      integer(long)                               :: gj,mmin,mmax,mincr,&
                                                     sphtyp,dj,j0,im,ij,jmax,g
      integer(long), intent(in)                   :: gdim1,f
      integer(long), dimension(gdim1), intent(in) :: jsph,msph
      real(dop), dimension(gdim1), intent(in)     :: gpop1
      real(dop), dimension(gdim1), intent(out)    :: gpop2

      call zerovxd(gpop2,gdim1)

      sphtyp = ipbaspar(1,f)
      jmax   = ipbaspar(2,f)
      gj     = ipbaspar(4,f)
      mmax   = ipbaspar(1,f+1)
      mincr  = ipbaspar(2,f+1)
      mmin   = ipbaspar(3,f+1)
      dj = 1
      if(sphtyp .eq.2  .or. sphtyp .eq.3 ) dj = 2
      j0 = mod(jmax,dj)
      
      do g = 1, gdim1
         ij = (jsph(g)-j0)/dj + 1
         im = (msph(g)-mmin)/mincr + 1 + gj
         gpop2(ij) = gpop2(ij) + gpop1(g)
         gpop2(im) = gpop2(im) + gpop1(g)
      enddo
       
      return
      end subroutine  sphpop

!#######################################################################

      subroutine gwp2grid(n1,f1,agmat,gdof,xi,phi,dim1,gdim1,ort,dicht2,&
                 gpop1,dgwp)

      implicit none

      integer(long)                                       :: n,e,e1,g
      integer(long)                                       :: iperiod
      integer(long), intent(in)                           :: n1,f1,gdof,dim1,gdim1
      complex(dop)                                        :: phig,tmp,ctmp
      complex(dop), dimension(dim1), intent(in)           :: phi
      complex(dop), dimension(maxdim)                     :: ag,xig
      complex(dop), dimension(gdof,dim1), intent(in)      :: xi
      complex(dop), dimension(dim1,dim1), intent(in)      :: dicht2,dgwp
      complex(dop), dimension(dim1,dim1)                  :: tmpdicht,trover
      complex(dop), dimension(gdof,gdof,dim1), intent(in) :: agmat
      real(dop)                                           :: rtmp,th,x0,xn,xrange
      real(dop), dimension(gdim1), intent(in)             :: ort
      real(dop), dimension(gdim1), intent(out)            :: gpop1
 
      call zerovxd (gpop1,gdim1)
!
! only valid for separable GWPs
!
      if (gwptype .ne. 2 .and. gwptype .ne. 3) return

!-----------------------------------------------------------------------
! check for periodicity
!-----------------------------------------------------------------------
      iperiod = ipbaspar(2,f1)
      xrange = 0.0
      if (iperiod .ne. 0) then
         x0 = rpbaspar(1,f1)
         xn = rpbaspar(2,f1)
         xrange = xn - x0
      endif
         
!-----------------------------------------------------------------------
! get dicht2 in GWP basis if psitype = 0
!-----------------------------------------------------------------------
      if (psitype .eq. 0 .or. psitype .eq. 5) then
         call tranmxtzz(dgwp,dicht2,dgwp,tmpdicht,dim1,dim1)
      else
         call cpmxz(dicht2,tmpdicht,dim1,dim1)
      endif

!-----------------------------------------------------------------------
! rho(x) = sum_ij A_i^* A_j dz g_i^* (x,z) g_j (x,z)
! 
! first calculate ln(rho) to avoid problems with exponentials
! assume real width 
!-----------------------------------------------------------------------
      do e=1,dim1
         do e1=e,dim1
            phig = dconjg(phi(e1)) + phi(e)

            trover(e1,e)=phig
            do n = 1,gdof
               if (n .ne. n1) then
                  ag(n) = dconjg(agmat(n,n,e1)) + agmat(n,n,e)
                  xig(n) = dconjg(xi(n,e1)) + xi(n,e)

                  trover(e1,e)=trover(e1,e) - xig(n)**2/(4.0_dop*ag(n))

                  rtmp=-pi/dble(ag(n))
                  rtmp=0.5_dop*dlog(rtmp)
                  trover(e1,e)=trover(e1,e) + rtmp
               endif
            enddo
            if (e1 .ne. e) trover(e,e1)=dconjg(trover(e1,e))
         enddo
      enddo
!-----------------------------------------------------------------------
! calculate population at grid points. 
!-----------------------------------------------------------------------
      do g=1,gdim1
         do e=1,dim1
            ctmp = agmat(n1,n1,e)*ort(g)*ort(g) + xi(n1,e)*ort(g)
            rtmp = 2.0_dop*dble(ctmp)
            rtmp = rtmp + dble(trover(e,e))

            rtmp = dexp(rtmp)
            gpop1(g) = gpop1(g) + dble(tmpdicht(e,e))*rtmp

            do e1=e+1,dim1
               tmp = agmat(n1,n1,e1)*ort(g)*ort(g) + xi(n1,e1)*ort(g)
               tmp = ctmp + dconjg(tmp)
               tmp = tmp + trover(e1,e)

               rtmp = dble(tmp) 
               rtmp = dexp(rtmp)
               th=dimag(tmp)
               tmp=dcmplx(dcos(th),dsin(th))
               tmp = rtmp*tmp
               gpop1(g) = gpop1(g) + 2.0_dop*dble(tmpdicht(e1,e)*tmp)
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! if periodic coordinate need to add density from images either
! side of range.
!-----------------------------------------------------------------------
      if (iperiod .ne. 0) then
         do g=1,gdim1
            x0 = ort(g) + xrange
            do e=1,dim1
               ctmp = agmat(n1,n1,e)*x0*x0 + xi(n1,e)*x0
               rtmp = 2.0_dop*dble(ctmp)
               rtmp = rtmp + dble(trover(e,e))

               rtmp = dexp(rtmp)
               gpop1(g) = gpop1(g) + dble(tmpdicht(e,e))*rtmp

               do e1=e+1,dim1
                  tmp = agmat(n1,n1,e1)*x0*x0 + xi(n1,e1)*x0
                  tmp = ctmp + dconjg(tmp)
                  tmp = tmp + dble(trover(e1,e))

                  rtmp = dble(tmp) 
                  rtmp = dexp(rtmp)
                  th=dimag(tmp)
                  tmp=dcmplx(dcos(th),dsin(th))
                  tmp = rtmp*tmp
                  gpop1(g) = gpop1(g) + 2.0_dop*dble(tmpdicht(e1,e)*tmp)
               enddo
            enddo

            x0 = ort(g) - xrange
            do e=1,dim1
               ctmp = agmat(n1,n1,e)*x0*x0 + xi(n1,e)*x0
               rtmp = 2.0_dop*dble(ctmp)
               rtmp = rtmp + dble(trover(e,e))

               rtmp = dexp(rtmp)
               gpop1(g) = gpop1(g) + dble(tmpdicht(e,e))*rtmp

               do e1=e+1,dim1
                  tmp = agmat(n1,n1,e1)*x0*x0 + xi(n1,e1)*x0
                  tmp = ctmp + dconjg(tmp)
                  tmp = tmp + dble(trover(e1,e))

                  rtmp = dble(tmp) 
                  rtmp = dexp(rtmp)
                  th=dimag(tmp)
                  tmp=dcmplx(dcos(th),dsin(th))
                  tmp = rtmp*tmp
                  gpop1(g) = gpop1(g) + 2.0_dop*dble(tmpdicht(e1,e)*tmp)
               enddo
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! multiply by integration interval, i.e. dxi, to match SPF convention
! This is needed for analysis programs.
!-----------------------------------------------------------------------
      do g=1,gdim1
         gpop1(g) = gpop1(g)*(ort(2) - ort(1))
      enddo

      return
      end subroutine gwp2grid

!#######################################################################

      subroutine traj2grid(n,nspfdof1,psi,dim1,gdim1,ort,gpop1,ltraj,&
                 trajst,st)


      implicit none

      integer(long)                                      :: e,g,gpoint
      integer(long), intent(in)                          :: n,nspfdof1,&
                                                            dim1,gdim1,st
      integer(long), intent(in),dimension(dim1)          :: trajst
      complex(dop), dimension(nspfdof1,dim1), intent(in) :: psi
      real(dop)                                          :: coo,diff,gdiff,&
                                                            gap,gap2
      real(dop), dimension(gdim1), intent(in)            :: ort
      real(dop), dimension(gdim1), intent(out)           :: gpop1
      logical(kind=4), intent(in)                        :: ltraj
      logical(kind=4)                                    :: ignore
 
      call zerovxd (gpop1,gdim1)
      gap = ort(2) - ort(1)
      gap2 = gap**2

!-----------------------------------------------------------------------
! calculate population at grid points (sum trajectories and normalise)
! for a trajectory calculation, store in correct state population
!-----------------------------------------------------------------------
      diff=999.0_dop
      gpoint = -999
      do e=1,dim1
         if (ltraj .and. trajst(e) .ne. st) cycle

         gdiff = 9999999.000
         coo = dble(psi(n,e))
         do g=1,gdim1
            diff = coo - ort(g)
            diff = diff**2
            if (diff .lt. gdiff) then
               gpoint=g
               gdiff=diff
            endif
         enddo

         if (gpoint .lt. 0) then
            routine='Traj2grid'
            message = 'Error in coordinates - nearest grid point not found'
            call errormsg
         endif

! look to see if off grid
         ignore = .false.
         if ((gpoint .eq. 1) .and. (diff .gt. gap2)) ignore=.true.
         if ((gpoint .eq. gdim1) .and. (diff .gt. gap2)) ignore=.true.
         if (.not. ignore) then
            gpop1(gpoint) = gpop1(gpoint) + 1
         endif
      enddo

!-----------------------------------------------------------------------
! normalise and
! multiply by integration interval, i.e. dxi, to match SPF convention
!-----------------------------------------------------------------------
      do g=1,gdim1
         gpop1(g) = gpop1(g) / dble(dim1)
         gpop1(g) = gpop1(g)*(ort(2) - ort(1))
      enddo

      return
      end subroutine traj2grid

!#######################################################################
!
! Add quadrature weight to GWP grid populations
!
!#######################################################################

      subroutine gwpgpopwei(gpop,ort)

      implicit none

      integer(long)                                      :: s,m,f,n
      real(dop), dimension(ortdim,nstate), intent(inout) :: gpop 
      real(dop), dimension(ortdim), intent(in)           :: ort
      real(dop)                                          :: weight

      do m=1,nmode
         do n=1,nspfdof(m)
            f = spfdof(n,m)
            if (basis(f) .eq. 25) then
               weight = ort(zort(f)+1) - ort(zort(f))
               do s=1,nstate
                  gpop(zort(f):zort(f)+gdim(f)-1,s) =  &
                      gpop(zort(f):zort(f)+gdim(f)-1,s)*weight
               enddo
            endif
         enddo
      enddo

      return
      end subroutine gwpgpopwei

      end module dichtoutmod

