!***********************************************************************
! 
!             MCTDH module EININT
!
!  01/98 AJ
!
!
! integrator set in 3 stages. 
! 1. intdef  sets basic defaults
! 2. einint  read in user given options
! 3. setint
!***********************************************************************

      module einintmod

use decimal, only: long,dop
use logdat

use global
use maxv
use channels
use keyunitsmod
use datenmod
use hpsimod, only: gwpcxm,gwpintordm,gwpintm,dermethod,fddx
use runpropmod
use mpidata, only: lmpi
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use channels
use dirdyn
use realwpmod


      implicit none
      private
      public :: einintprop,eininttraj,einintreal,einintdiag
      contains

!***********************************************************************

      subroutine einintprop

      implicit none

      integer(long)                              :: ic,iz,ierr,ilbl
      logical(kind=4)                            :: linperr
      logical(kind=4)                            :: check=.false.
      character(len=c5)                          :: filename

! CHECK iin open
      inquire(iin,opened=check)
      if (check) close(iin)
      filename=inputfile(1:inplaenge)
      open(iin,file=filename,status='old',form='formatted',err=900)

!-----------------------------------------------------------------------
! set defaults
!-----------------------------------------------------------------------
      call intdef
      rewind(iin)
      iz=0
      linperr=.false.

      readiin: do
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .gt. 0) then
            routine = 'Eingabe, integrator-section'
            message = inptit
            call errormsg
         endif

         if (keyword(1) .eq. 'integrator-section' .or. &
             keyword(1) .eq. 'integrator_section' ) then
            call einint(linperr,iz)
            call setint
            exit
         else if (keyword(1) .eq. 'end-input') then
            call setint
            exit
         endif

      enddo readiin
!
! check settings
!
!.....Do not open update file, if not CMF
      lupdate = lupdate.and.lconst

      return
  900 continue
      routine='Einintprop (einintmod.f90)'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop

      end subroutine einintprop

!***********************************************************************

      subroutine einintdiag

      implicit none

      integer(long)                              :: iz

      call intdef
      rewind(iin)
      iz=0

      call setint

      end subroutine einintdiag

!***********************************************************************

      subroutine eininttraj

      implicit none

      integer(long)                              :: ic,iz,ierr
      logical(kind=4)                            :: linperr

      call intdef
      rewind(iin)
      iz=0
      linperr=.false.

      readiin: do
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .gt. 0) then
            routine = 'Eingabe, integrator-section'
            message = inptit
            call errormsg
         endif

         if (keyword(1) .eq. 'integrator-section' .or. &
             keyword(1) .eq. 'integrator_section' ) then
            call eininttr(linperr,iz)
            call setinttr
            exit
         else if (keyword(1) .eq. 'end-input') then
            call setinttr
            exit
         endif

      enddo readiin

      end subroutine eininttraj

!***********************************************************************

      subroutine einintreal

      implicit none

      integer(long)                              :: ic,iz,ierr
      logical(kind=4)                            :: linperr

      call intdef
      rewind(iin)
      iz=0
      linperr=.false.

      readiin: do
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ierr .gt. 0) then
            routine = 'Eingabe, integrator-section'
            message = inptit
            call errormsg
         endif

         if (keyword(1) .eq. 'integrator-section' .or. &
             keyword(1) .eq. 'integrator_section' ) then
            call einintrw(linperr,iz)
            call setint
            exit
         else if (keyword(1) .eq. 'end-input') then
            call setint
            exit
         endif

      enddo readiin

      end subroutine einintreal

!***********************************************************************

      subroutine einint(linperr,iz)

      implicit none


      integer(long)                    :: i,i0,iz0,ic,ord,ierr,ierr1, &
                                          iprojh,f,m,j,ifreeze, &
                                          irenormgwpa,ityp,slpos
      integer(long), intent(in)        :: iz
      integer(long)                    :: gwpcx1,gwpintord1,gwpint1
      integer(long), dimension(maxdim) :: gwpcxm1,gwpintordm1,gwpintm1
      real(dop)                        :: acc,dt0,testinput
      logical(kind=4)                  :: lstdorb,ltdgwp
      logical(kind=4), intent(inout)   :: linperr

!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
      routine = 'einint (read INTEGRATOR-SECTION)'
      iz0 = 0
      ierr1 = 0
      iprojh = 0
      lstdorb = .false.
      mnspec = 0

! uncoupled GWPs are time-dependent 
      ltdgwp = .true.
! flag if gwp_renorma keyword is given.
      irenormgwpa = 0

      gwpcx1 = -999
      gwpintord1 = -999
      gwpint1 = -999
      gwpcxm1 = -999
      gwpintordm1 = -999
      gwpintm1 = -999

! ML CMF Split doesn't work dor pMCTDH(I)
      if (dentype .eq. 1 .and. lmlcmfsplit) lmlcmfsplit = .false.

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (keyword(i) .eq. 'end-integrator-section') then
         goto 20
      else if (keyword(1) .eq. 'end-input') then
         message = &
        'No end-integrator-section keyword at end of INTEGRATOR-SECTION'
         call warnmsg
         linperr=lwarn.or.linperr
         lwarn=.false.
         RETURN

      else if( keyword(i) .eq. 'lmf' ) then
         lmf = .true.
      else if( keyword(i) .eq. 'df' ) then
         lmf = .false.

      else if( keyword(i) .eq. 'vmf' ) then
         lconst = .false.
! use h-proj for VMF unless GWP calc
         if (.not. lgwp) then
            lfunkl = .false.
            lfunkr = .true.
         endif
         if(lrlx) then
            lfunkl = .false.
            lfunkr = .true.
         endif
         if (lexact) then
            message=&
          'VMF integrator selected for numerically exact calculation'
            call errormsg
         endif
      else if( keyword(i)(1:3) .eq. 'cmf' ) then
         if (lexact) then
            message=&
          'CMF integrator selected for numerically exact calculation'
            call errormsg
         endif

         lconst = .true.
         lfunkl = .false.
         lfunkr = .true.

         lconstvar  = .false.
         lconstfix  = .false.
         lcmfadaa   = .false.
         lcmfadaphi = .false.
         if( keyword(i)(5:8) .eq. 'fix ' ) then
            lconstfix  = .true.
         else if( keyword(i)(5:8) .eq. 'var ' ) then
            lconstvar  = .true.
         else if( keyword(i)(5:11) .eq. 'varphi ' ) then
            lcmfadaphi = .true.
         else if( keyword(i)(5:9) .eq. 'vara ' ) then
            lcmfadaa   = .true.
         else if(keyword(i)(4:4) .ne. '/' .and. rlaxnum .lt. 0) then
            lconstvar = .true.
         else if(keyword(i)(4:4) .ne. '/' .and. rlaxnum .ge. 0) then
            lcmfadaphi = .true.
         end if

         i0   = i
         ierr1= 0
         if(keyword(i+1) .eq. '=' )  then
            i = i + 1
            if(keyword(i+1) .ne. ',' )  then
               i = i + 1
               read(keyword(i),*,iostat=ierr)  UpDateTime
               if (ierr .ne. 0) ierr1=ierr
               UpDateTime = abs(UpDateTime)*Fs
            end if
            if(keyword(i+1) .eq. ',' .and. .not.lconstfix )  then
               i = i + 2
               read(keyword(i),*,iostat=ierr) epsupdate
               if (ierr .ne. 0) ierr1=ierr
            end if
         end if
         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif

      else if( keyword(i) .eq. 'update' ) then
         i0   = i
         ierr1= 0
         if(keyword(i+1) .eq. '=' )  then
            i = i + 2
            read(keyword(i),*,iostat=ierr)  UpDateTime
            if (ierr .ne. 0) ierr1=ierr
            UpDateTime = abs(UpDateTime)*Fs
         end if
         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif

! === Adams-Bashforth-Moulton predictor-corrector ===

      else if( keyword(i)(1:3) .eq. 'abm' ) then
         i0   = i
         ierr1= 0
         ord  = 0
         acc  = 0.0_dop
         Dt0  = 0.0_dop
         
         if(keyword(i+1) .eq. '=' )  then
            i = i + 1
            if(keyword(i+1) .ne. ',' )  then
               i = i + 1
               read(keyword(i),*,iostat=ierr) ord
               if (ierr .ne. 0) ierr1=ierr
            end if
            if(keyword(i+1) .eq. ',' )  then
               i = i + 1
               if(keyword(i+1) .ne. ',' )  then
                  i = i + 1
                  read(keyword(i),*,iostat=ierr) acc
                  if (ierr .ne. 0) ierr1=ierr
                  lexpacc = .true.
               end if
               if(keyword(i+1) .eq. ',' )  then
                  i = i + 2
                  read(keyword(i),*,iostat=ierr) Dt0
                  Dt0 = Dt0*Fs
                  if (ierr .ne. 0) ierr1=ierr 
               end if
            end if
         end if
         
         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif
         
         if( keyword(i0) .eq. 'abm/spf' ) then
            inttypePhi  = 1
            intorderPhi = ord
            accphi      = acc
            dtphi       = dt0
            if( .not.lconst .and. .not.lexact ) then
               message = ' /spf not allowed for VMF  '
               call wrline(zeroi,iz,iz0)
            endif
         else If( keyword(i0) .eq. 'abm/a' ) then
            inttypeA  = 1
            intorderA = ord
            accavec   = acc
            dtA       = dt0
            if( .not.lconst ) then
               message = ' /A not allowed for VMF or exact  '
               call wrline(zeroi,iz,iz0)
            endif
         else if( keyword(i0) .eq. 'abm/all' .or.&
                  keyword(i0)(4:4) .ne. '/') then
            inttyp    = 1
            intord    = ord
            accuracy  = acc
            dt        = dt0
         else if (keyword(i0)(4:5) .eq. '/@') then
            call store_mspec(keyorig(i0)(5:),m)
            minttyp(m) = 1
            mintord(m) = ord
            mintacc(m) = acc
            mintdt(m)  = dt0
         else if (keyword(i0)(4:5) .eq. '/@') then
            call store_mspec(keyorig(i0)(5:),m)
            minttyp(m) = 1
            mintord(m) = ord
            mintacc(m) = acc
            mintdt(m)  = dt0
         else
            write(6,906)
            message = ' unknown keyword : '
            call wrline(i0,iz,iz0)
         end if

      else if( keyword(i)(1:5) .eq. 'lsoda' ) then
         i0   = i
         ierr1= 0
         acc  = 0.0_dop
         if (lconst) then
           message = 'LSODA not allowed for CMF'
           call wrline(zeroi,iz,iz0)
         endif
         if (keyword(i+1) .eq. '=' )  then
           i = i + 2
           read(keyword(i),*,iostat=ierr) acc
           if (ierr .ne. 0) ierr1=ierr
           lexpacc = .true.
         end if
         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif
         inttyp    = 7
         accuracy  = acc

! === Bulirsch-Stoer with polynomial extrapolation ===

      else if( keyword(i)(1:2) .eq. 'bs' ) then
         i0   = i
         ierr1= 0
         ord  = 0
         acc  = 0.0_dop
         Dt0  = 0.0_dop

         if(keyword(i+1) .eq. '=' )  then
            i = i + 1
            if(keyword(i+1) .ne. ',' )  then
               i = i + 1
               read(keyword(i),*,iostat=ierr) ord
               if (ierr .ne. 0) ierr1=ierr
            end if
            if(keyword(i+1) .eq. ',' )  then
               i = i + 1
               if(keyword(i+1) .ne. ',' )  then
                  i = i + 1
                  read(keyword(i),*,iostat=ierr) acc
                  if (ierr .ne. 0) ierr1=ierr
                  lexpacc = .true.
               end if
               if(keyword(i+1) .eq. ',' )  then
                  i = i + 2
                  read(keyword(i),*,iostat=ierr) Dt0
                  Dt0 = Dt0*Fs
                  if (ierr .ne. 0) ierr1=ierr 
               end if
            end if
         end if

         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif

         If( keyword(i0) .eq. 'bs/spf' ) then
            inttypePhi  = 2
            intorderPhi = ord
            accphi      = acc
            DtPhi       = Dt0
            if( .not.lconst .and. .not.lexact ) then
               message = ' /spf not allowed for VMF  '
               call wrline(zeroi,iz,iz0)
            endif
         else If( keyword(i0) .eq. 'bs/a' ) then
            inttypeA  = 2
            intorderA = ord
            accavec   = acc
            DtA       = Dt0
            if( .not.lconst ) then
               message = ' /A not allowed for VMF or exact  '
               call wrline(zeroi,iz,iz0)
            endif
         else if( keyword(i0) .eq. 'bs/all' .or.&
                  keyword(i0) .eq. 'bs/def' .or.&
                  keyword(i0)(3:3) .ne. '/') then
            inttyp    = 2
            intord    = ord
            accuracy  = acc
            Dt        = Dt0
         else if (keyword(i0)(3:4) .eq. '/@') then
            call store_mspec(keyorig(i0)(4:),m)
            minttyp(m) = 2
            mintord(m) = ord
            mintacc(m) = acc
            mintdt(m)  = dt0
         else
            write(6,906)
            message = ' unknown keyword : '
            call wrline(i0,iz,iz0)
         end if

! === Short iterative Lanczos or Lanczos-Arnoldi with variable order ===

      else if( keyword(i)(1:3) .eq.  'sil'  .or. &
               keyword(i)(1:4) .eq. 'csil') then
         i0   = i
         ierr1= 0
         ord  = 0
         acc  = 0.0_dop
         if(keyword(i)(1:4) .eq. 'csil') then
            lcap  = .true.
            slpos = 5 ! position of slash (if any)
         else
            slpos = 4
         endif

         if(keyword(i+1) .eq. '=' )  then
            ierr1=0
            i = i + 1
            if(keyword(i+1) .ne. ',' )  then
               i = i + 1
               read(keyword(i),*,iostat=ierr) ord
               if (ierr .ne. 0) ierr1=ierr
            end if
            if(keyword(i+1) .eq. ',' )  then
               i = i + 1
               if(keyword(i+1) .ne. ',' )  then
                  i = i + 1
                  read(keyword(i),*,iostat=ierr) acc
                  if (ierr .ne. 0) ierr1=ierr
                  lexpacc = .true.
               end if
               if (keyword(i+1) .eq. ',') then
                  i = i+2
                  if (keyword(i) .eq. 'standard') then
                     loldsilerr = .true.
                  elseif (keyword(i) .eq. 'novel') then
                     loldsilerr = .false.
                  else
                     message = 'Expecting "standard" or "novel", not '
                     call wrline(i,i,iz0)
                  endif
               else
                  loldsilerr = .true.
               endif
            endif
         endif

         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif

         If( keyword(i0)(slpos:) .eq. '/spf' ) then
            inttypePhi  = 3
            intorderPhi = ord
            accphi      = acc
            if( .not.lconst .and. .not.lexact ) then
               message = ' /spf not allowed for VMF  '
               call wrline(zeroi,iz,iz0)
            endif
         else If( keyword(i0)(slpos:) .eq. '/a') then
            inttypeA  = 3
            intorderA = ord
            accavec   = acc
            if( .not.lconst ) then
               message = ' /A not allowed for VMF or exact  '
               call wrline(zeroi,iz,iz0)
            endif
         else if( keyword(i0)(slpos:) .eq. '/all' .or. &
                  keyword(i0)(slpos:) .eq. '/def' .or. &
                  keyword(i0)(slpos:slpos) .ne. '/') then
            inttyp    = 3
            intord    = ord
            accuracy  = acc
         else if (keyword(i0)(slpos:slpos+1) .eq. '/@') then
            call store_mspec(keyorig(i0)(slpos+1:),m)
            minttyp(m) = 3
            mintord(m) = ord
            mintacc(m) = acc
            mintdt(m)  = 0.d0
         else
            write(6,906)
            message = ' unknown keyword : '
            call wrline(i0,iz,iz0)
         end if

!************************************************************         
! SOD integrator CALL
!************************************************************
      else if( keyword(i)(1:3) .eq.  'sod') then
         ord=30
         acc=1.0d-10
         dt0=0.8

         if(keyword(i+1) .eq. '=' )  then
            i = i + 1
            if(keyword(i+1) .ne. ',' )  then
               i = i + 1
               read(keyword(i),*,iostat=ierr) Dt0
               if (ierr .ne. 0) ierr1=ierr
            else
               i=i+1
            endif
            if(keyword(i+1) .eq. ',' )  then
               i = i + 1
               if(keyword(i+1) .ne. ',' )  then
                  read(keyword(i),*,iostat=ierr) acc
                  if (ierr .ne. 0) ierr1=ierr
                  lexpacc = .true.
               endif
             else
               i=i+1
             endif
             if(keyword(i+1) .eq. ',' )  then
               i = i + 1
               if(keyword(i+1) .ne. ',' )  then
                  read(keyword(i),*,iostat=ierr) ord
                  if (ierr .ne. 0) ierr1=ierr
               endif
             else
               i=i+1
             endif
         end if

         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif

         inttypephi = 69
         intorderphi = ord
         accphi = acc
         dtphi  = dt0

!end sod call

!************************************************************         
! SPO integrator CALL
!************************************************************
      else if( keyword(i)(1:3) .eq.  'spo') then
         dt0=0.01

         if(keyword(i+1) .eq. '=' )  then
            i = i + 1
            if(keyword(i+1) .ne. ',' )  then
               i = i + 1
               read(keyword(i),*,iostat=ierr) Dt0
               if (ierr .ne. 0) ierr1=ierr
            else
               i=i+1
            endif
         end if

         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif

         inttypephi = 70
         intorderphi = 0
         accphi = 0.0
         dtphi  = dt0

!end spo call

!************************************************************         
! CHEB integrator CALL
!************************************************************
      else if( keyword(i)(1:4) .eq.  'cheb') then
         if(keyword(i+1) .eq. '=' )  then
            i = i + 2
            read(keyword(i),*,iostat=ierr) accphi
            if (ierr .ne. 0) ierr1=ierr
            lexpacc = .true.
         end if

         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif

         inttypephi = 8

!end cheb call
!************************************************************
!    Davidson "integrator" for improved relaxation
!************************************************************
      else if( keyword(i) .eq. 'dav' .or. keyword(i) .eq. 'dav/a' .or. &
           keyword(i) .eq. 'rdav' .or. keyword(i) .eq. 'rdav/a'   .or. &
           keyword(i) .eq. 'rrdav' .or. keyword(i) .eq. 'rrdav/a' .or. &
           keyword(i) .eq. 'cdav' .or. keyword(i) .eq. 'cdav/a') then
         if( .not.lconst ) then
            message = ' Dav not allowed for VMF or exact  '
            call wrline(zeroi,iz,iz0)
         endif
         if( keyword(i)(1:1) .eq. 'r' ) lreal = .true.
         if( keyword(i)(2:2) .eq. 'r' ) lrreal= .true.
         if( keyword(i)(1:1) .eq. 'c' ) lcap  = .true.
         if(keyword(i+1) .eq. '=' )  then
            ierr1=0
            i = i + 1
            if(keyword(i+1) .ne. ',' )  then
               i = i + 1
               read(keyword(i),*,iostat=ierr) ord
               if (ierr .ne. 0) ierr1=ierr
            end if
            if(keyword(i+1) .eq. ',' )  then
               i = i + 1
               if(keyword(i+1) .ne. ',' )  then
                  i = i + 1
                  read(keyword(i),*,iostat=ierr) acc
                  if (ierr .ne. 0) ierr1=ierr
                  lexpacc = .true.
               end if
            endif
            if (ierr1 .ne. 0) then
               message = 'Incorrect arguments for keyword :'
               call wrline(i0,iz,iz0)
            endif
         endif
         inttypeA  = 6
         intorderA = ord
         accavec   = acc
         lenorb    = .true.    ! Energy orbitals set for DAV
         if(mpb.gt.0) then     ! Reset intorderA for Block-DAV
            intorderA = ( ord + 1 + ((npackts+1)/5) )/npackts
            intorderA = npackts*intorderA - 1
         endif

! === Runge-Kutta integrator, fixed order ===

      else if (keyword(i)(1:2) .eq. 'rk') then
         i0   = i
         ierr1= 0
         acc  = 0.0_dop
         ityp=0
         if (keyword(i)(1:4) .eq. 'rk5f')  then
            dt0  = 0.05*fs
            acc  = 1.0d-6
            slpos=5
         else
            dt0  = 0.0_dop
            slpos=4
         endif

         read (keyword(i)(3:3),*,iostat=ierr) ord
         if (ierr.ne.0) then
            message = 'Cannot determine order for RK integrator'
            call wrline(i0,iz,iz0)
            ord = 0
         elseif (ord.eq.5) then
            if (keyword(i)(4:4) .eq. 'f') then
               ityp = 41
            else
               ityp = 4
            endif
         elseif (ord.eq.8) then
            ityp = 5
         else
            message = 'Incorrect order for RK integrator'
            call warnmsg
         endif

         if (keyword(i)(1:4) .eq. 'rk5f' .and. keyword(i+1) .eq. '=' )  then
            i = i + 2
            read(keyword(i),*,iostat=ierr) dt0
            dt0 = dt0*fs
            if (ierr .ne. 0) ierr1=ierr
 28         continue
            if (keyword(i+1) .eq. ',') then
               i = i+2
               if((keyword(i)(1:6) .eq. 'imp-or') .or. &
                  (keyword(i)(1:10).eq. 'improved-o')) then
                  liork = .true.
               else if((keyword(i)(1:9) .eq. 'no_imp-or') .or. &
                  (keyword(i)(1:13).eq. 'no_improved-o')) then
                  liork = .false.
               else if(keyword(i)(1:6) .eq. 'rktest') then
                  lrktest = .true.
               else if(keyword(i)(1:9) .eq. 'no_rktest') then
                  lrktest = .false.
               else
                  message = 'Unknown keyword :'
                  call wrline(i,iz,iz0)
               endif
               goto 28 
            endif

         else if (keyword(i+1) .eq. '=' )  then
            i = i + 1
!           read accuracy
            if (keyword(i+1) .ne. ',') then
               i = i+1
               read(keyword(i),*,iostat=ierr) acc
               if (ierr .ne. 0) ierr1=ierr
               lexpacc = .true.
            endif
!           read initial stepsize and/or imp-ortho
 30         continue
            if (keyword(i+1) .eq. ',') then
               i = i+2
               if((keyword(i)(1:6) .eq. 'imp-or') .or. &
                  (keyword(i)(1:10).eq. 'improved-o')) then
                  liork = .true.
               else if((keyword(i)(1:9) .eq. 'no_imp-or') .or. &
                  (keyword(i)(1:13).eq. 'no_improved-o')) then
                  liork = .false.
               else if(keyword(i)(1:6) .eq. 'rktest') then
                  lrktest = .true.
               else if(keyword(i)(1:9) .eq. 'no_rktest') then
                  lrktest = .false.
               elseif(keyword(i)(1:8) .eq. 'rho-norm') then
                  lrhonrm = .true.
               elseif (keyword(i)(1:7) .eq. 'minstep') then
                  i=i+1
                  read(keyword(i),*,iostat=ierr) minrk5
                  minrk5 = minrk5*fs
                  if (ierr .ne. 0) ierr1=ierr
               elseif (dt0 .eq. 0) then
                  read(keyword(i),*,iostat=ierr) dt0
                  dt0 = dt0*fs
                  if (ierr .ne. 0) ierr1=ierr
               else
                  message = 'Unknown keyword :'
                  call wrline(i,iz,iz0)
               endif
               goto 30 
            endif
         endif
         
         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif
         
         if (keyword(i0)(slpos:) .eq. '/spf' ) then
            inttypePhi  = ityp
            accphi      = acc
            dtphi       = dt0
            intorderphi = ord
            if( .not.lconst .and. .not.lexact ) then
               message = ' /spf not allowed for VMF  '
               call wrline(zeroi,iz,iz0)
            endif
         else if (keyword(i0)(slpos:) .eq. '/a' ) then
            inttypeA  = ityp
            accavec   = acc
            dtA       = dt0
            intorderA = ord
            if( .not.lconst ) then
               message = ' /A not allowed for VMF or exact  '
               call wrline(zeroi,iz,iz0)
            endif
            if( lrhonrm ) then
               message='rho-norm not allowed for A-vector propagation'
               call wrline(zeroi,iz,iz0)
            endif
         else if (keyword(i0)(slpos:) .eq. '/all' .or.&
                  keyword(i0)(slpos:) .eq. '/def' .or.&
                  keyword(i0)(slpos:slpos) .eq. ' ')    then
            inttyp    = ityp
            accuracy  = acc
            dt        = dt0
            intord    = ord
         else if (keyword(i0)(slpos:slpos+1) .eq. '/@') then
            call store_mspec(keyorig(i0)(slpos+1:),m)
            minttyp(m) = ityp
            mintord(m) = ord
            mintacc(m) = acc
            mintdt(m)  = dt0
         else
            write(6,906)
            message = ' unknown keyword : '
            call wrline(i0,iz,iz0)
         endif

!     end ODESIM stuff

! This is the Euler integrator. For testing exact calculations only.
      else if (keyword(i)(1:5) .eq. 'euler')then
         if(.not.lexact)then
            routine='einint'
            message='Euler integrator only valid for exact calculations'
            call errormsg
         endif
         lconst=.false.
         inttyp = 6
         inttypephi = 6
         dt = out1

    ! this is ZVODE integrator
      else if (keyword(i)(1:5) .eq. 'zvode') then
         i0   = i
         ierr1= 0
         acc  = 0.0_dop
         dt0  = 0.0_dop

         if (keyword(i+1) .eq. '=' )  then
            i = i + 1
!           read accuracy
            if (keyword(i+1) .ne. ',') then
               i = i+1
               read(keyword(i),*,iostat=ierr) acc
               if (ierr .ne. 0) ierr1=ierr
               lexpacc = .true.
            endif
!           read initial stepsize and/or imp-ortho
 40         continue
            if (keyword(i+1) .eq. ',') then
               i = i+2
               if((keyword(i)(1:6) .eq. 'imp-or') .or. &
                  (keyword(i)(1:10).eq. 'improved-o')) then
                  limportho = .true.
               elseif (keyword(i)=='stiff') then
                  iflagzvode=22
               elseif (keyword(i)=='no_overshoot') then
                  lzvodecrit=.true.
               elseif (keyword(i)=='maxit') then
                  if (keyword(i+1)=='=') then
                     i=i+2
                     read(keyword(i),*,iostat=ierr) zvodemaxit
                     if (ierr/=0) ierr1=ierr
                  else
                     message='ZVODE maxit specified, but no iteration count given'
                     call wrline(i,iz,iz0)
                  end if
               elseif (keyword(i)=='minstp') then
                  if (keyword(i+1)=='=') then
                     i=i+2
                     read(keyword(i),*,iostat=ierr) zvodeminstp
                     if (ierr/=0) ierr1=ierr
                  else
                     message='ZVODE minstp specified, but no step size given'
                     call wrline(i,iz,iz0)
                  end if
               elseif (keyword(i)=='maxstp') then
                  if (keyword(i+1)=='=') then
                     i=i+2
                     read(keyword(i),*,iostat=ierr) zvodemaxstp
                     if (ierr/=0) ierr1=ierr
                  else
                     message='ZVODE maxstp specified, but no step size given'
                     call wrline(i,iz,iz0)
                  end if
               elseif (keyword(i)=='first') then
                  if (keyword(i+1)=='=') then
                     i=i+2
                     read(keyword(i),*,iostat=ierr) zvode1ststp
                     if (ierr/=0) ierr1=ierr
                  else
                     message='ZVODE first specified, but no step size given'
                     call wrline(i,iz,iz0)
                  end if
               elseif (dt0 .eq. 0) then
                  read(keyword(i),*,iostat=ierr) dt0
                  dt0 = dt0*fs
                  if (ierr .ne. 0) ierr1=ierr
               else
                  message = 'Unknown keyword :'
                  call wrline(i,iz,iz0)
               endif
               goto 40 
            endif
         endif
         
         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif
         
         if (keyword(i0) .eq. 'zvode/spf' ) then
            inttypePhi  = 9
            accphi      = acc
            dtphi       = dt0
            intord      = 5   ! unused
            if( .not.lconst .and. .not.lexact ) then
               message = ' /spf not allowed for VMF  '
               call wrline(zeroi,iz,iz0)
            endif
         else if (keyword(i0) .eq. 'zvode/a' ) then
            inttypeA  = 9
            accavec   = acc
            dtA       = dt0
            intord    = 5   ! unused
            if( .not.lconst ) then
               message = ' /A not allowed for VMF or exact  '
               call wrline(zeroi,iz,iz0)
            endif
         else if (keyword(i0) .eq. 'zvode/all' .or.&
                  keyword(i0) .eq. 'zvode')    then
            inttyp    = 9
            accuracy  = acc
            dt        = dt0
            intord    = 5   ! unused
         else
            write(6,906)
            message = ' unknown keyword : '
            call wrline(i0,iz,iz0)
         end if
!     end ZVODE stuff

! verlet algorithm call

!     else if( keyword(i)(1:6) .eq. 'verlet' ) then

!        if (keyword(i)(8:10) .ne. 'gwp' .and. .not. lconst) then
!           message = 'Verlet integrator can only be used for GWPs in CMF scheme'
!           call wrline(i0,iz,iz0)
!        endif
!        
!        i0   = i
!        ierr1= 0
!        Dt0  = 0.1*fs

!        if(keyword(i+1) .eq. '=' )  then
!           i = i + 2
!           read(keyword(i),*,iostat=ierr) Dt0
!           if (ierr .ne. 0) ierr1=ierr
!           if (keyword(i+1) .eq. ',') then
!              i=i+2
!              call keyunits(dt0,keyword(i),ierr)
!             if (ierr .eq. 1) then
!                 message = 'no unit after comma'
!                 call wrline(zeroi,iz,iz0)
!              endif
!           else
!              Dt0 = Dt0*Fs
!           endif

!        end if

!        if (ierr1 .ne. 0) then
!           message = 'Incorrect arguments for keyword :'
!           call wrline(i0,iz,iz0)
!        endif

!        inttypegwp  = 10
!        dtgwp    = dt0

! end verlet

! === other keywords ===

      else if( keyword(i) .eq. 'eps_inv' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with eps_inv keyword'
            call wrline(zeroi,iz,iz0)
         else
            read(keyword(i+2),*,iostat=ierr) epsmat
            if (ierr .ne. 0) then
               message = 'Incorrect arguments for keyword :'
               call wrline(i,iz,iz0)
            endif
            i=i+2
         endif

      else if( keyword(i) .eq. 'eps_svd' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with eps_svd keyword'
            call wrline(zeroi,iz,iz0)
         else
            read(keyword(i+2),*,iostat=ierr) epssvd
            if (ierr .ne. 0 .or. epssvd.le.0.d0) then
               message = 'Incorrect arguments for keyword :'
               call wrline(i,iz,iz0)
            endif
            i=i+2
         endif

      else if( keyword(i) .eq. 'eps_gsinv' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with eps_gsinv keyword'
            call wrline(zeroi,iz,iz0)
         else
            read(keyword(i+2),*,iostat=ierr) epsgsmat
            if (ierr .ne. 0) then
               message = 'Incorrect arguments for keyword :'
               call wrline(i,iz,iz0)
            endif
            i=i+2
         endif

      else if( keyword(i) .eq. 'eps_gwp' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with eps_gwp keyword'
            call wrline(zeroi,iz,iz0)
         else
            read(keyword(i+2),*,iostat=ierr) epsgwp
            if (ierr .ne. 0) then
               message = 'Incorrect arguments for keyword :'
               call wrline(i,iz,iz0)
            endif
            i=i+2
         endif

      else if( keyword(i) .eq. 'eps_dyn' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with eps_dyn keyword'
            call wrline(zeroi,iz,iz0)
         else
            read(keyword(i+2),*,iostat=ierr) epsdyn
            igwpdynm(:) = 1
            if (ierr .ne. 0) then
               message = 'Incorrect arguments for keyword :'
               call wrline(i,iz,iz0)
            endif
            i=i+2
         endif

      else if( keyword(i) .eq. 'eps_ld' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with eps_ld keyword'
            call wrline(zeroi,iz,iz0)
         else
            read(keyword(i+2),*,iostat=ierr) epsld
            if (ierr .ne. 0) then
               message = 'Incorrect arguments for keyword :'
               call wrline(i,iz,iz0)
            endif
            i=i+2
         endif

      else if( keyword(i) .eq. 'ml-cmf' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with ml-cmf keyword'
            call wrline(zeroi,iz,iz0)
         else
            if (keyword(i+2).eq.'split') then
               lmlcmfsplit = .true.
            else if (keyword(i+2).eq.'unified') then
               lmlcmfsplit = .false.
            else
               message = 'Only "split" or "unified" allowed for ml-cmf ' &
                    //'keyword'
               call wrline(zeroi,iz,iz0)
            endif
            i=i+2
         endif

      else if( keyword(i) .eq. 'rm_noise' ) then
         lrmnoise = .true.

      else if( keyword(i) .eq. 'no_rm_noise' ) then
         lrmnoise = .false.

      else if( keyword(i) .eq. 'ovl_thresh' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with ovl_thresh keyword'
            call wrline(zeroi,iz,iz0)
         else
            read(keyword(i+2),*,iostat=ierr) ovlthr
            if (ierr .ne. 0) then
               message = 'Incorrect arguments for keyword :'
               call wrline(i,iz,iz0)
            endif
            i=i+2
         endif

      else if( keyword(i) .eq. 'default' ) then
         acclevel = 0

      else if( keyword(i) .eq. 'tight' ) then
         acclevel = 1

      else if( keyword(i) .eq. 'medium' ) then
         acclevel = 2

      else if( keyword(i) .eq. 'loose' ) then
         acclevel = 3

      else if( keyword(i) .eq. 'smat_ld' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with smat_ld keyword'
            call wrline(zeroi,iz,iz0)
         endif
         if (keyword(i+2) .eq. 'overlaps') then
            lsmatovl = .true.
            i = i+2
         elseif (keyword(i+2) .eq. 'eigen') then
            lsmatovl = .false.
            i = i+2
         else
            message = 'Incorrect arguments for keyword :'
            call wrline(i,iz,iz0)
            i=i+2
         endif

      else if( keyword(i) .eq. 'gwp_model' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with gwp_model keyword'
            call wrline(zeroi,iz,iz0)
         endif

         if (keyword(i+2) .eq. 'vmcg') then
            gwpmodel = 0
         else if (keyword(i+2) .eq. 'classical') then
            gwpmodel = 1
         else if (keyword(i+2) .eq. 'independent') then
            gwpmodel = 2
         else if (keyword(i+2) .eq. 'time_independent' .or. &
                  keyword(i+2) .eq. 'static') then
            gwpmodel = 3
         else if (keyword(i+2) .eq. 'bel') then
            gwpmodel = 4
         else if (keyword(i+2) .eq. 'saddle') then
            gwpmodel = 5
         endif
         i=i+2

      else if( keyword(i)(1:11) .eq. 'gwp_model@m' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with gwp_model@m keyword'
            call wrline(zeroi,iz,iz0)
         endif

         gwpmodel = -1
         read(keyword(i)(12:lc(i)),*) m
         if (keyword(i+2) .eq. 'vmcg') then
            gwpmodelm(m) = 0
         else if (keyword(i+2) .eq. 'classical') then
            gwpmodelm(m) = 1
         else if (keyword(i+2) .eq. 'independent') then
            gwpmodelm(m) = 2
         else if (keyword(i+2) .eq. 'time_independent' .or. &
                  keyword(i+2) .eq. 'static') then
            gwpmodelm(m) = 3
         else if (keyword(i+2) .eq. 'saddle') then
            gwpmodelm(m) = 5
         endif
         i=i+2

      else if( keyword(i) .eq. 'gwp_eoms' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with gwp_eoms keyword'
            call wrline(zeroi,iz,iz0)
         endif

   50    continue
         if (keyword(i+2) .eq. 'inverse') then
            gwpeoms = 0
            i=i+2
         else if (keyword(i+2).eq.'lu_decomp') then
            gwpeoms = 1
            i=i+2
         else if (keyword(i+2)(1:5).eq.'gauss') then
            gwpeoms = 2
            i=i+2
         else if (keyword(i+2) .eq. 'conj_grad') then
            gwpeoms = 3
            i=i+2
         else if (keyword(i+2) .eq. 'direct') then
            gwpeoms = 4
            i=i+2
         else if (keyword(i+2) .eq. 'tikhonov') then
            gwpeoms = 5
            i=i+2
         !   if (keyword(i+3).eq.',') then
         !      read(keyword(i+4),*,iostat=ierr) tikh_fac
         !      if (ierr .ne. 0) then
         !         message = 'Incorrect arguments for keyword :'
         !         call wrline(i,iz,iz0)
         !      endif
         !   endif
         !   i=i+4
         else if (keyword(i+2) .eq. 'svd') then
            gwpeoms = 6
            i=i+2
         !   if (keyword(i+1).eq.',') then
         !      read(keyword(i+2),*,iostat=ierr) svd_fac
         !      if (ierr .ne. 0) then
         !         message = 'Incorrect arguments for keyword :'
         !         call wrline(i,iz,iz0)
         !      endif
         !      i=i+2
         !   endif
         else if (keyword(i+2) .eq. 'cholesky') then
            gwpeoms = 7
            i=i+2

         else if (keyword(i+2) .eq. 'clsq') then
            gwpeoms = 8
            i=i+2

         else if (keyword(i+2) .eq. 'project') then
            gwpeoms = 9
            i=i+2

         else if (keyword(i+2) .eq. 'precond') then
            message = 'Solution of equation of motion with a preconditioner &
                       &not yet implemented. Falling back to default method. &
                       & Keyword :'
            call wrline(i,iz,iz0)
            gwpeoms = 0
            i=i+2

         else if (keyword(i+2) .eq. 'ovr_eigen') then
            ovrinv = 0
            i=i+2
         else if (keyword(i+2) .eq. 'ovr_direct') then
            ovrinv = 1
            i=i+2
         else if (keyword(i+2) .eq. 'ovr_svd') then
            ovrinv = 2
            i=i+2

! Old keywords, left for backword compatability - moved to gwp_model = 

         else if (keyword(i+2) .eq. 'independent') then
            gwpmodel = 2
            i=i+2

         else if (keyword(i+2) .eq. 'classical') then
            gwpmodel = 1
            i=i+2

         else if (keyword(i+2) .eq. 'bel') then
            gwpmodel = 4
            lBEL = .TRUE.
            i=i+2

         else
            message = 'Incorrect arguments for keyword :'
            call wrline(i,iz,iz0)
            i=i+2

         endif

         if (keyword(i+1).eq.',') go to 50

      else if( keyword(i) .eq. 'cx' ) then
         if (keyword(i+1).eq.'=') then
            if (keyword(i+2).eq.'dynamic') then
               gwpcx1 = 1
            else if (keyword(i+2).eq.'static') then
               gwpcx1 = 2
            else if (keyword(i+2).eq.'none') then
               gwpcx1 = 0
            else
               message = 'Incorrect arguments for keyword :'
               call wrline(i,iz,iz0)
            endif
            i = i+2

         else
            gwpcx1 = 1
         endif

      else if( keyword(i) .eq. 'no_cx' ) then
         gwpcx1 = 0

      else if( keyword(i)(1:4) .eq. 'cx@m' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No arguments given for keyword :'
            call wrline(i,iz,iz0)
         endif

         read(keyword(i)(5:lc(i)),*) m
         if (keyword(i+2).eq.'dynamic') then
            gwpcxm1(m) = 1
         else if (keyword(i+2).eq.'static') then
            gwpcxm1(m) = 2
         else if (keyword(i+2).eq.'none') then
            gwpcxm1(m) = 0
         else
            message = 'Incorrect arguments for keyword :'
            call wrline(i,iz,iz0)
         endif
         i = i+2

      else if( keyword(i) .eq. 'gwpintord' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument for keyword :'
            call wrline(i,iz,iz0)
         endif

         read (keyword(i+2),*) gwpintord1
         i = i+2

      else if( keyword(i)(1:11) .eq. 'gwpintord@m' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No arguments given for keyword :'
            call wrline(i,iz,iz0)
         endif

         read(keyword(i)(12:lc(i)),*) m
         read (keyword(i+2),*) gwpintordm1(m)
         i = i+2

      else if( keyword(i) .eq. 'sym_trj' ) then
         ltrjsym = .true.

      else if( keyword(i) .eq. 'no_sym_trj' ) then
         ltrjsym = .false.

      else if (keyword(i) .eq. 'chk_lindep') then
         linchk = .true.
         if (keyword(i+1).eq.'=') then
            read(keyword(i+2),*,iostat=ierr) ovlthr
            if (ierr .ne. 0) then
               message = 'Incorrect arguments for keyword :'
               call wrline(i,iz,iz0)
            endif
            i=i+2
         endif

      else if (keyword(i) .eq. 'nochk_lindep') then
         linchk = .false.

      else if (keyword(i) .eq. 'gwp_renorma') then
         lrenormgwpa = .true.
         irenormgwpa = 1

      else if (keyword(i) .eq. 'nogwp_renorma') then
         lrenormgwpa = .false.

      else if (keyword(i) .eq. 'gwp_optyp') then
         lgwpoptyp = .true.

      else if (keyword(i) .eq. 'nogwp_optyp') then
         lgwpoptyp = .false.

      else if (keyword(i) .eq. 'gwp_optdyn') then
         lgwpoptdyn = .true.

      else if (keyword(i) .eq. 'nogwp_optdyn') then
         lgwpoptdyn = .false.

      else if(keyword(i) .eq. 'gwp_coupling' .or. &
              keyword(i) .eq. 'dynamic_gwps' ) then
        if (keyword(i+1).ne.'=') then
          message = 'No argument specified with eps_dyn keyword'
          call wrline(zeroi,iz,iz0)
        else
          j=i+2
          do while (j/=i)
            if (keyword(j)(1:3) .eq. 'min') then
               igwpdynm(:) = 1
               epsdyn = 1.0E-4_dop
            else if (keyword(j) .eq. 'normal') then
               igwpdynm(:) = 1
               epsdyn = 1.0E-6_dop
            else if (keyword(j) .eq. 'max') then
               igwpdynm(:) = 1
               epsdyn = 1.0E-8_dop
            else if (keyword(j) .eq. 'none') then
               igwpdynm(:) = 2
               epsdyn = 999.0_dop
            else if (keyword(j) .eq. 'all') then
               igwpdynm(:) = 0
               epsdyn = 1.0E-16_dop
            elseif (keyword(j)=='whole_g') then
              lwholeg=.true.
            elseif (keyword(j)=='restart_int') then
              lresint=.true.
            elseif (keyword(j)=='td') then
              ltdgwp=.true.
            elseif (keyword(j)=='tind') then
              ltdgwp=.false.
            else 
               message = 'Incorrect arguments for keyword :'
               call wrline(i,iz,iz0)
            endif
            if (keyword(j+1)==',') then
              j=j+2
            else
              i=j
            endif
          enddo
        endif

      else if(keyword(i) .eq. 'cysinv' ) then
         lcysinv=.true.

      else if(keyword(i) .eq. 'nocysinv' ) then
         lcysinv=.false.

      else if( keyword(i) .eq. 'bel' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with BEL keyword'
            call wrline(zeroi,iz,iz0)
         else
            read(keyword(i+2),*,iostat=ierr) BELint
            if (ierr .ne. 0) then
               message = 'Incorrect arguments for keyword :'
               call wrline(i,iz,iz0)
            endif
            i=i+2
            if (keyword(i+1) .eq. '*' ) then
               read(keyword(i+2),*,iostat=ierr) BELoff
               if (ierr .ne. 0) then
                  message = 'Incorrect arguments for keyword :'
                  call wrline(i,iz,iz0)
               endif
               i=i+2
            endif
            if (keyword(i+1) .eq. ',' ) then
               i = i+2
               call keytunits(BELint,keyword(i),ierr)
               if (ierr .ne. 0) then
                  message = 'Incorrect arguments for keyword :'
                  call wrline(i,iz,iz0)
               else
                  call keytunits(BELoff,keyword(i),ierr)
               endif
            end if
         endif

      else if( keyword(i) .eq. 'belprint' ) then
         if (keyword(i+1) == '=') then
            read(keyword(i+2),*,iostat=ierr) BELprint
            if (ierr .ne. 0) then
               message = 'Incorrect arguments for keyword :'
               call wrline(i,iz,iz0)
            endif
            i=i+2
         else
            BELprint=1000
         end if
         if (keyword(i+1) == ':') then
            m=1
            f=1
            ierr=0
            j=i
            do while (ierr== 0)
               read(keyword(i+2),*,iostat=ierr) testinput
               if (ierr == 0) then
                  i=i+2
                  if (keyword(i+1) == ',') then
                     m=m+1
                  else
                     ierr=1
                  end if
               end if
            end do
            allocate(BELplotlims(m,3))
            i=j
            ierr=0
            m=1
            do while (ierr== 0)
               read(keyword(i+2),*,iostat=ierr) BELplotlims(m,f)
               if (ierr == 0) then
                  i=i+2
                  if (keyword(i+1) == ',') then
                     m=m+1
                     if (m>size(BELplotlims)/3) then
                        message = 'Maximum number of dimensions exceeded in BELprint'
                        call wrline(i,iz,iz0)
                        m=1
                        f=f+1
                     end if
                  else if (keyword(i+1) == ':') then
                     m=1
                     f=f+1
                     if (f>3) then
                        message = 'Only need three sets of limits in BELprint allowed (min;max;delta_x)'
                        call wrline(i,iz,iz0)
                        m=1
                        f=f+1
                     end if
                  else
                     ierr=1
                  end if
               end if
            end do
            ierr=0
         end if

      else if( keyword(i) .eq. 'belflags' ) then
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with BELflags keyword'
            call wrline(zeroi,iz,iz0)
         else
            do m=1,size(BELflags)
               read(keyword(i+2),*,iostat=ierr) BELflags(m)
               if (ierr .ne. 0) then
                  message = 'Incorrect arguments for keyword :'
                  call wrline(i,iz,iz0)
                  exit
               endif
               i=i+2
               if (keyword(i+1) .ne. ',' ) then
                  exit
               endif
            end do
         endif

      else if( keyword(i) .eq. 'wf_error' ) then
         lwferror = .true.

      else if( keyword(i) .eq. 'natorb' ) then
         if (dentype .gt. 0) then
            message='Natural orbitals not implemented for densities.'
            call errormsg
         endif
         lnaturcmf = .true.

      else if( keyword(i) .eq. 'energyorb' .or. &
               keyword(i) .eq. 'enorb' ) then
         if (dentype .gt. 0) then
            message='Energy orbitals not implemented for densities.'
            call errormsg
         endif
         lenorb = .true.

      else if( keyword(i) .eq. 'tindorb' ) then
         ltindspf = .true.

      else if( keyword(i) .eq. 'stdorb' ) then
         lstdorb= .true.

      else if( keyword(i) .eq. 'svd_prop' ) then
         lsvd = .true.

      else if( keyword(i) .eq. 'proj-h' ) then
         iprojh = 1
      else if( keyword(i) .eq. 'h-proj' ) then
         iprojh = 2

      else if( keyword(i)(1:11) .eq. 'simple-proj'  .or.  &
               keyword(i)(1:11) .eq. 'simpel-proj'  ) then
         lopro = .false.

      else if ( (keyword(i)(1:9)  .eq. 'imp-ortho') .or.&
                (keyword(i)(1:11) .eq. 'improved-or') ) then
         limportho = .true.
         liork = .true.

      else if ( (keyword(i)(1:9)  .eq. 'no_imp-ortho') .or.&
                (keyword(i)(1:11) .eq. 'no_improved-or') ) then
         limportho = .false.
         liork = .false.

      else if( keyword(i) .eq. 'tddvr' ) then
         if (dentype .gt. 0) then
            message='Tddvr not implemented for densities.'
            call errormsg
         endif
         ltdvr = .true.
      else if( keyword(i) .eq. 'cdvr' ) then
         if (dentype .gt. 0) then
            message='Cdvr not implemented for densities.'
            call errormsg
         endif
         lcdvr = .true.

      else if (keyword(i) .eq. 'direct-multi-d') then
         ldirmuld=.true.

      else if ( keyword(i) .eq. 'nohsym' ) then
         usehsym=.false.

      else if ( keyword(i) .eq. 'gwp_integrals' ) then
         if( keyword(i+1).eq. '=' )then
            i = i + 1
            if (keyword(i+1) .eq. 'unsymm') then
               gwpint1=1
            else if (keyword(i+1) .eq. 'symm') then
               gwpint1=2
            else if (keyword(i+1) .eq. 'midpoint') then
               gwpint1=3
            else if (keyword(i+1) .eq. 'gausspot') then
               gwpint1=4
            else
               message(1:40+maxkeylen) = &
                       'Unknown argument with keyword gwp_interals: '//keyword(i+1)
               call wrline(i,iz,iz0)
            endif
         else
            message = 'No argument for keyword :'
            call wrline(i,iz,iz0)
         endif
         i = i + 1 

      else if( keyword(i)(1:15) .eq. 'gwp_integrals@m' ) then
         read(keyword(i)(16:lc(i)),*) m
         if( keyword(i+1).eq. '=' )then
            i = i + 1
            if (keyword(i+1) .eq. 'unsymm') then
               gwpintm1(m)=1
            else if (keyword(i+1) .eq. 'symm') then
               gwpintm1(m)=2
            else if (keyword(i+1) .eq. 'midpoint') then
               gwpintm1(m)=3
            else if (keyword(i+1) .eq. 'gausspot') then
               gwpintm1(m)=4
            else
               message(1:40+maxkeylen) = &
                       'Unknown argument with keyword gwp_interals: '//keyword(i+1)
               call wrline(i,iz,iz0)
            endif
         else
            message = 'No argument for keyword :'
            call wrline(i,iz,iz0)
         endif
         i = i + 1 

      else if ( keyword(i) .eq. 'lha_all' ) then
         lforcelha = .true.

      else if ( keyword(i) .eq. 'lha_step' ) then

         if( keyword(i+1).eq. '=' )then
            i = i + 1
            read(keyword(i+1),*,iostat=ierr) fddx
         else
            message = 'No argument for keyword :'
            call wrline(i,iz,iz0)
         endif
         i = i + 1 

      else if ( keyword(i) .eq. 'improved_num_derivs' ) then
         dermethod = 1

      else if ( keyword(i) .eq. 'simple_num_derivs' ) then
         dermethod = 0

!-----------------------------------------------------------------------
! update options for direct dynamics potential
!-----------------------------------------------------------------------
      else if( keyword(i) .eq. 'dd_update' )  then
         message = 'dd_update  keyword no longer used. Use update '//&
                   'in dirdyn-section instead.'
         call errormsg

      else if( keyword(i) .eq. 'freeze' )  then
         j=0
         if (keyword(i+1).ne.'=') then
            message = 'No argument specified with freeze keyword'
            call warnmsg
         else
  80        j=j+1
            read(keyword(i+2),*,iostat=ierr) ifreeze
            complete(ifreeze)=.true.
            if (ierr .ne. 0) then
              message='Argument for freeze must be an integer. Not :'
              call wrline(i+2,iz,iz0)
            endif
            i=i+2
            if (keyword(i+1) .eq. ',') goto 80

         endif

      else
         message = ' unknown keyword : '
         call wrline(i,iz,iz0)
      end if
      
!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

   20 continue   ! end of reading keywords

!-----------------------------------------------------------------------
! CDVR has to be single to make one set of points for all states and 
! couplings
!-----------------------------------------------------------------------
      if (lcdvr) then
         if ((nddstate .gt. 1) .and. lmult) then
            routine ='Eintint (einintmod.f90)'
            message = 'Single-set must be used for CDVR'
            call errormsg
         endif
      endif

!-----------------------------------------------------------------------
! select model for propagating GWPs and set parameters required
!
!   gwpmodel = 0 : full coupling as for vMCG
!              1 : GWPs follow classical trajectories
!              2 : GWPs are independent (overlaps ignored in propagation)
!              3 : GWPs are time-independent 
!              4 : BEL algorithm used
!              5 : Saddle point approx used (GWPs follow classical 
!                  trajectories and gwpintord = 0)
!
! control is through setting igwpdynm (C-matrix coupling), gwpcx (use
! of CX decomposition) and method flags.
!
! finer control given afterwards by application of explicit input of these
! parameters
!
!-----------------------------------------------------------------------
      gwpintordm(:) = lhaorder   ! order of potential (2 if LHA)

      if (gwpmodel .ge. 0) gwpmodelm(:) = gwpmodel
      if (gwpmodel .eq. 0) then   
         continue    ! all parameters previously defined.

      else if (gwpmodel .eq. 1) then
         gwpcxm(:) = 1
         gwpintordm(:) = lhaorder
         igwpdynm(:)=2

      else if (gwpmodel .eq. 2) then
         if (lmult .and. nstate .gt. 1) then
            routine = 'Einint'
            message = 'Independent GWPs cannot be used in a multi-set calculation'
            call errormsg
         endif

         gwpcxm(:) = 1
         gwpintordm(:) = lhaorder
         igwpdynm(:) = 2
         gwpintm(:) = 1

      else if (gwpmodel .eq. 3) then
         gwpcxm(:) = 0
         gwpintordm(:) = lhaorder
         igwpdynm(:) = 2
         gwpintm(:) = 1

      else if (gwpmodel .eq. 4) then
         lBEL = .true.

      else if (gwpmodel .eq. 5) then
         gwpcxm(:) = 1
         gwpintordm(:) = 0
         igwpdynm(:) = 2
         gwpintm(:) = 2 

      else if (gwpmodel .eq. -1) then
         igwpdynm(:)=1
         do m=1,nmode
            if (.not. gwpm(m)) cycle
            if (gwpmodelm(m) .eq. 0) then
               continue    ! all parameters previously defined.
            else if (gwpmodelm(m) .eq. 1) then
               gwpcxm(m)=1
               igwpdynm(m)=2
               gwpintordm(m)=lhaorder
               gwpintm(m) = 2
            else if (gwpmodelm(m) .eq. 2) then
               gwpcxm(m)=1
               igwpdynm(m)=2
               gwpintordm(m)=lhaorder
               gwpintm(m) = 2
            else if (gwpmodelm(m) .eq. 3) then
               gwpcxm(m)=0
               igwpdynm(m)=2
               gwpintordm(m)=lhaorder
               gwpintm(m) = 2
            else if (gwpmodelm(m) .eq. 5) then
               gwpcxm(m)=1
               igwpdynm(m)=2
               gwpintordm(m)=0
               gwpintm(m) = 2 

            else
               routine = 'Einint'
               message = 'GWP Model not yet implemented for mixed propagations.'
               call errormsg
            endif              
         enddo
      endif

!-----------------------------------------------------------------------
! Use any explicitely set parameters for GWP model
! (overide defaults)
!-----------------------------------------------------------------------
      if (gwpcx1 .ge. 0) gwpcxm(:) = gwpcx1
      if (gwpintord1 .ge. 0) gwpintordm(:) = gwpintord1
      if (gwpint1 .ge. 0) gwpintm(:) = gwpint1
      do m=1,nmode
         if (gwpcxm1(m) .ge. 0) gwpcxm(m) = gwpcxm1(m)
         if (gwpintordm1(m) .ge. 0) gwpintordm(m) = gwpintordm1(m)
         if (gwpintm1(m) .ge. 0) gwpintm(m) = gwpintm1(m)
      enddo

!-----------------------------------------------------------------------
! Midpoint cannot be used with vMCG + CX
!-----------------------------------------------------------------------
      do m=1,nmode
         if (gwpmodelm(m) .eq. 0) then
            if (gwpcxm(m) .gt. 0 .and. gwpintm(m) .eq. 3) then
               routine = 'Einint (einintmod.f90)'
               message = &
          'Cannot use midpoint integrals and CX decomposition for vMCG'
               call errormsg
            endif
         endif
      enddo

!-----------------------------------------------------------------------
! Set flag if CX decomposition used
!-----------------------------------------------------------------------
      lgwpcx = .false.
      do m=1,nmode
         if (gwpcxm(m) .gt. 0) lgwpcx = .true.
      enddo
         
!-----------------------------------------------------------------------
! trajectory data can only be generated with independent GWPs
!-----------------------------------------------------------------------
      if (lgentrj .and. gwpmodel .ne. 2) then
         routine = 'Einint'
         message = 'Trajectory data can only be generated with independent GWPs'
    !     call errormsg
      endif

!-----------------------------------------------------------------------
! if static uncoupled GWPs wanted check gwpcx is set correctly.
! gwpcx = 2 CX uncoupled GWPs static
!-----------------------------------------------------------------------
      if (.not. ltdgwp) then
         do m=1,nmode
            gwpcxm(m) = 2
         enddo
      endif

!-----------------------------------------------------------------------
! Check gwpintord set correctly for model
!-----------------------------------------------------------------------
     do m=1,nmode
        if (gwpmodelm(m) .eq. 0 .and. gwpintordm(m) .lt. 2) then
            routine = 'Einint'
            message = 'GWP Integrals must be at least 2nd order for vMCG'
            call warnmsg
        endif
     enddo

!-----------------------------------------------------------------------
! turn off CX for GAP potentials
!-----------------------------------------------------------------------
      if (lgwp .and. lgausspot) then
         if (gwpcxm(1) .gt. 0) then
            routine = 'Einint'
            message = 'CX decomposition cannot be used with GAP potential'
            call warnmsg1

            gwpcxm(1) = 0
         endif
      endif

!-----------------------------------------------------------------------
! set orbital type
!-----------------------------------------------------------------------
      if (lstdorb) then
         lenorb = .false.
      endif

!-----------------------------------------------------------------------
! check use of rhonrm
!-----------------------------------------------------------------------
      if( lrhonrm .and. lexact ) then
         message='rho-norm not allowed for VMF or exact'
         call wrline(zeroi,iz,iz0)
      endif
      if( lrhonrm .and. lrunml .and. .not.lmlcmfsplit ) then
         message='rho-norm requires ml-cmf=split for ML runs'
         call wrline(zeroi,iz,iz0)
      endif

!-----------------------------------------------------------------------
! for direct multi-D potential algorithm, an analytic PES is needed
!-----------------------------------------------------------------------
      if (ldirmuld .and. .not. lanalpes) then
         message = 'Analytic PES required for direct evaluation '// &
                   ' of multi-D functions'
         call warnmsg
      endif
     
!-----------------------------------------------------------------------
! force lfunkl and lfunkr in case the proj-h or h-proj keyword was given.
!-----------------------------------------------------------------------
      if( iprojh .eq. 1 ) then
         lfunkl = .false.
         lfunkr = .true.
      else if( iprojh .eq. 2 ) then
         routine = 'Einint (einintmod.F90)'
         if (lconst) then
            message = 'Cannot use h-proj scheme with CMF integrator'
            call errormsg
         else if (lrunml) then
            message = 'Cannot use h-proj scheme with ML-MCTDH'
            call errormsg
         endif
         lfunkl = .true.
         lfunkr = .false.
      endif

!-----------------------------------------------------------------------
! force lfunkr for ML-MCTDH
!-----------------------------------------------------------------------
      if (lrunml) then
         lfunkl = .false.
         lfunkr = .true.
      endif

!-----------------------------------------------------------------------
! force lfunkr for time independent SPFs
!-----------------------------------------------------------------------
      if (ltindspf) then
         lfunkl = .false.
         lfunkr = .true.
      endif

!-----------------------------------------------------------------------
! BEL doesn't work with MPI
!-----------------------------------------------------------------------
      if (lbel .and. lmpi) then
         message = 'BEL does not work with MPI. Continuing on 1 core.'
         call warnmsg1
         lmpi = .false.
      endif

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr
      lwarn=.false.

 906  format(' Integrator keywords (i.e. ABM, BS or SIL) must have', &
             ' one of the extensions : /all, /spf or /A ')
 
      return
      end subroutine einint


!#######################################################################
!
! SETINT
!
! integrator is set by a combination of choices in einint and here.
!
! Default type of CMF is CMF/var for propagation
!                        CMF/varphi for improved relaxation
! Default integrator for CMF is BS/spf and SIL/A (MCTDH)
!                               RK5/spf and SIL/A (G-MCTDH)              
! Default integrator for VMF is ABM, unless vMCG which uses rk5
!
!#######################################################################

      subroutine setint

      implicit none

      integer(long)     :: m
      character(len=c3) :: string
      logical           :: lfound

!-----------------------------------------------------------------------
! check correct wavefunction type for CMF integration with GWPs
!-----------------------------------------------------------------------
      if (psitype .eq. 4 .and. lconst) then
         lconst = .false.
         message = &
       'G-MCTDH cannot use CMF integrator. Taking default VMF.'
         call warnmsg1
      endif

!-----------------------------------------------------------------------
! check single-set formulation and non-symmetrical LHA are used for GWP
! propagation when local diabatisation scheme is chosen
!-----------------------------------------------------------------------
      if (ldd) then
         if (dddiab .eq. 2 .and. gwpintm(1) .gt. 1) then
            message = 'Local diabatisation requires non-symmetrical LHA'
            call errormsg

         else if (dddiab .eq. 2 .and. nstate .ne. 1) then
            message = 'Local diabatisation requires single-set formulation'
            call errormsg
         endif
      endif

!-----------------------------------------------------------------------
! Make sure linear dependency checking is off for non-GWP runs
!-----------------------------------------------------------------------
      if (.not. lgwp) linchk = .false.

!-----------------------------------------------------------------------
! If dynamic GWP coupling or lin dep testing, make sure test is turned on!
!-----------------------------------------------------------------------
      if (lgwp) then
         if (any(igwpdynm .gt. 0) .or. linchk) lrktest = .true.
      endif

!-----------------------------------------------------------------------
! if time-dependent Hamiltonian select vmf
!-----------------------------------------------------------------------
      if (ltdh(1) .or. ltdp) then
         if (lconst) then
            routine='Setint'
            message = &
      'Time-dependent Hamiltonian - cannot Use CMF. Using VMF/ABM.'
            call warnmsg1
         endif
         lconst = .false.
      endif

!-----------------------------------------------------------------------
! If using DD-MCTDH use VMF
!-----------------------------------------------------------------------
      if(lddmctdh .and.lconst)then
         routine='Setint'
         message = &
              'DD-MCTDH only implemented for VMF so far. Setting to default.'
         call warnmsg1
         lconst = .false.
         lfunkl = .false.
         lfunkr = .true.
      endif

!-----------------------------------------------------------------------
! if exact calculation make sure CMF turned off
!-----------------------------------------------------------------------
      if (lexact) then
         lconst = .false.
         lfunkl=.false.
         lfunkr=.true.
      endif

!-----------------------------------------------------------------------
! force lfunkr for relaxation
!-----------------------------------------------------------------------
      if(lrlx) then
         lfunkl = .false.
         lfunkr = .true.
      endif

!-----------------------------------------------------------------------
! if trajectory driven GWPs cannot use CMF
!-----------------------------------------------------------------------
      if (lgwptrj) then
         lconst = .false.
      endif

!-----------------------------------------------------------------------
! use proj-h for BEL propagation
!-----------------------------------------------------------------------
      if (lbel) then
         lfunkl = .false.
         lfunkr = .true.
      endif

!-----------------------------------------------------------------------
! set default minstep for RK5 if not set explicitly
!-----------------------------------------------------------------------
      if (minrk5 .lt. -990.0) then
         if (lgwp) then
            minrk5 = 1.d-4*fs
         else
            minrk5 = 1.0d-8
         endif
      endif

!-----------------------------------------------------------------------
! If CMF has been chosen without specifying scheme, set defaults
!-----------------------------------------------------------------------
      if(lconst) then
         if (.not. lconstvar .and. .not. lconstfix &
             .and. .not. lcmfadaphi .and. .not. lcmfadaa) then
            if (rlaxnum .ge. 0) then
               lcmfadaphi=.true.
            else 
               lconstvar=.true.
            endif
         endif
      else
         lconstfix = .false.
         lconstvar = .false.
         lcmfadaa = .false.
         lcmfadaphi = .false.
      endif

!-----------------------------------------------------------------------
! first see if "part integrators" needed for CMF or exact have been
! defined using /all.
!-----------------------------------------------------------------------
      if(lconst) then
         if (inttyp .ne. 0) then
            inttypephi = inttyp
            intorderphi = intord
            accphi = accuracy
            dtphi = dt

            inttypea = inttyp
            intordera = intord
            accavec = accuracy
            dta = dt
         endif

      else if (lexact) then
         if (inttyp .ne. 0) then
            inttypephi = inttyp
            intorderPhi = intord
            accphi  = accuracy
            dtphi = dt
         endif

      endif
    
!-----------------------------------------------------------------------
! if integrators still undefined, set defaults.
!
! For CMF calculations set SIL/A BS/SPF RK5/GWP
! For exact set abm
! For VMF set abm
! For DD use RK5
! For vMCG use RK5
!-----------------------------------------------------------------------
      if(lconst) then
         if (inttypea .eq. 0) inttypea = 3
         if (inttypephi .eq. 0) then
            if (lgwp .or. lrunml) then
               inttypephi = 4
            else
               inttypephi = 2
            endif
         endif
      else if (lexact .and. dentype .eq. 0) then
         if (inttypephi .eq. 0) inttypephi = 3
      else if (lexact .and. dentype .gt. 0) then
         if (inttypephi .eq. 0) inttypephi = 1
      else
         if (inttyp .eq. 0) then
            if (ldd .or. lgwp) then
               inttyp = 4
            else
               inttyp = 1
            endif
         endif
      endif

!-----------------------------------------------------------------------
! now set parameter defaults for the various integrators
!
! default for RK is dt = 0.0. Then initial step chosen in routine OD5CR.
!-----------------------------------------------------------------------
      if(lconst) then
!        this is for CMF
         if ( updateTime .eq. 0.0_dop ) UpDateTime = out1
         if ( epsupdate  .eq. 0.0_dop ) epsupdate  = 1.0e-6_dop
         if(intorderPhi .eq. 0) then
            if( inttypePhi .eq. 1 ) intorderPhi = 6
            if( inttypePhi .eq. 2 ) intorderPhi = 8
            if( inttypePhi .eq. 4 ) intorderPhi = 5
         end if
         if(intorderA .eq. 0) then
            if( inttypeA .eq. 1 ) intorderA = 6
            if( inttypeA .eq. 2 ) intorderA = 8
            if( inttypeA .eq. 3 ) intorderA = 12
            if( inttypeA .eq. 6 ) intorderA = 100
         end if
         if(accPhi .eq. 0.0_dop) then
            if( inttypePhi .eq. 1 ) accPhi = 1.0e-5_dop
            if( inttypePhi .eq. 2 ) accPhi = 1.0e-6_dop
            if( inttypePhi .eq. 4 ) accPhi = 1.0e-6_dop
            if( inttypePhi .eq. 5 ) accPhi = 1.0e-6_dop
            if( inttypePhi .eq. 9 ) accPhi = 1.0e-6_dop
         end if
         if(accavec .eq. 0.0_dop) then
            if( inttypeA .eq. 1 ) accavec = 1.0e-5_dop
            if( inttypeA .eq. 2 ) accavec = 1.0e-6_dop
            if( inttypeA .eq. 3 ) accavec = 1.0e-6_dop
            if( inttypeA .eq. 4 ) accavec = 1.0e-6_dop
            if( inttypeA .eq. 5 ) accavec = 1.0e-6_dop
            if( inttypeA .eq. 6 ) accavec = 1.0e-9_dop
            if( inttypeA .eq. 9 ) accavec = 1.0e-9_dop
         end if
         if(dtphi .eq. 0.0_dop ) then
            if(inttypePhi .eq. 1 ) DtPhi = 1.0e-4_dop*Fs
            if(inttypePhi .eq. 2 ) DtPhi = UpDateTime*0.5_dop
            if(inttypePhi .eq. 4 ) DtPhi = 0.0_dop
            if(inttypePhi .eq. 5 ) DtPhi = 0.0_dop
         end if
         if(dta .eq. 0.0_dop) then
            if(inttypeA .eq. 1 ) DtA = 1.0e-4_dop*Fs
            if(inttypeA .eq. 2 ) DtA = UpDateTime*0.5_dop
            if(inttypeA .eq. 3 ) DtA = UpDateTime*0.5_dop
            if(inttypeA .eq. 4 ) DtA = 0.0_dop
            if(inttypeA .eq. 5 ) DtA = 0.0_dop
         end if

      elseif (lexact) then
!        this is for exact propagation
         if ( updateTime .eq. 0.0_dop ) UpDateTime = out1
         if(intorderPhi .eq. 0) then
            if( inttypePhi .eq. 1 ) intorderPhi = 6
            if( inttypePhi .eq. 2 ) intorderPhi = 8
            if( inttypePhi .eq. 3 ) intorderPhi = 15
         end if
         if(accPhi .eq. 0.0_dop) then
            if( inttypePhi .eq. 1 ) accPhi = 1.0e-5_dop
            if( inttypePhi .eq. 2 ) accPhi = 1.0e-6_dop
            if( inttypePhi .eq. 3 ) accPhi = 1.0e-6_dop
            if( inttypePhi .eq. 4 ) accPhi = 1.0e-6_dop
            if( inttypePhi .eq. 5 ) accPhi = 1.0e-6_dop
            if( inttypePhi .eq. 8 ) accPhi = 1.0e-6_dop
            if( inttypePhi .eq. 9 ) accPhi = 1.0e-6_dop
         end if
         if(dtphi .eq. 0.0_dop ) then
            if(inttypePhi .eq. 1 ) DtPhi = 1.0e-4_dop*fs
            if(inttypePhi .eq. 2 ) DtPhi = 0.2_dop*Fs
            if(inttypePhi .eq. 3 ) DtPhi = UpDateTime
            if(inttypePhi .eq. 4 ) DtPhi = 0.0_dop
            if(inttypePhi .eq. 5 ) DtPhi = 0.0_dop
         endif

      else
!        this is for VMF
         if(inttyp.eq.1 .and. dt.eq.0.0_dop) dt = 1.0e-4_dop*Fs
         if(inttyp.eq.2 .and. dt.eq.0.0_dop) dt = 0.2_dop*Fs
         if(inttyp.eq.4 .and. dt.eq.0.0_dop) dt = 0.0_dop
         if(inttyp.eq.5 .and. dt.eq.0.0_dop) dt = 0.0_dop

         if(inttyp.eq.1 .and. accuracy.eq.0.0_dop) accuracy = 1.0e-5_dop
         if(inttyp.eq.2 .and. accuracy.eq.0.0_dop) accuracy = 1.0e-6_dop
         if(inttyp.eq.4 .and. accuracy.eq.0.0_dop) accuracy = 1.0e-6_dop
         if(inttyp.eq.5 .and. accuracy.eq.0.0_dop) accuracy = 1.0e-6_dop
         if(inttyp.eq.7 .and. accuracy.eq.0.0_dop) accuracy = 1.0e-6_dop
         if(inttyp.eq.9 .and. accuracy.eq.0.0_dop) accuracy = 1.0e-6_dop

         if(inttyp.eq.1 .and. intord.eq.0) intord = 6
         if(inttyp.eq.2 .and. intord.eq.0) intord = 8
      end if

! Set SIL for top node of ML-MCTDH CMF split
! Check first if it has been specified - either by a /def or /@1 definition
      if (lrunml .and. lmlcmfsplit) then
         lfound = .false.
         string = "@1"
         do m=1,mnspec
            if (mspec(m) .eq. string) lfound=.true.
         enddo
         if (.not. lfound .and. inttyp .eq. 0) then
            call store_mspec(string,m)
            minttyp(m) = inttypea
            mintord(m) = intordera
            mintacc(m) = accavec
            mintdt(m)  = dta
         endif
      endif

! Change accuracy level if asked for
      if (acclevel .eq. 1) then
         accuracy = accuracy*0.1_dop
         epsupdate = epsupdate*0.1_dop
         accphi = accphi*0.1_dop
         accavec = accavec*0.1_dop
      else if (acclevel .eq. 2) then
         accuracy = accuracy*10_dop
         epsupdate = epsupdate*10_dop
         accphi = accphi*10_dop
         accavec = accavec*10_dop
      else if (acclevel .eq. 3) then
         accuracy = accuracy*100_dop
         epsupdate = epsupdate*100_dop
         accphi = accphi*100_dop
         accavec = accavec*100_dop
      else if (acclevel .ne. 0) then
         routine = 'Setint (einintmod.F90) '
         message = 'Integrator Accuraccy level has not been set'
         call errormsg
      endif

      return
      end subroutine setint

!***********************************************************************

      subroutine store_mspec(spec,m)

      implicit none

      character(len=c3), intent(in) :: spec
      integer(long), intent(out)    :: m

      mnspec = mnspec+1
      if (mnspec.gt.2*maxdim) then
         routine = 'store_mspec'
         message = 'too many mode specifications'
         call warnmsg
         return
      endif
      mspec(mnspec) = spec
      m = mnspec
      return
      end subroutine store_mspec

!***********************************************************************
! 
!             MCTDH module EININTTR
!
!  01/98 AJ
!
!***********************************************************************

      subroutine eininttr(linperr,iz)

      implicit none


      integer(long)                    :: i,i0,iz0,ic,ord,ierr,ierr1,iprojh
      integer(long), intent(in)        :: iz
      real(dop)                        :: acc,dt0
      logical(kind=4)                  :: lstdorb
      logical(kind=4), intent(inout)   :: linperr

!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
      routine = 'einint (read INTEGRATOR-SECTION)'
      iz0 = 0
      ierr1 = 0
      iprojh = 0
      lstdorb = .false.
!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------

   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (keyword(i) .eq. 'end-integrator-section') then
         goto 20

      else if (keyword(1) .eq. 'end-input') then
         message = &
        'No end-integrator-section keyword at end of INTEGRATOR-SECTION'
         call warnmsg
         linperr=lwarn.or.linperr
         lwarn=.false.
         RETURN


      else if( keyword(i)(1:3) .eq. 'abm' ) then
         i0   = i
         ierr1= 0
         ord  = 0
         acc  = 0.0_dop
         Dt0  = 0.0_dop
         
         if(keyword(i+1) .eq. '=' )  then
            i = i + 1
            if(keyword(i+1) .ne. ',' )  then
               i = i + 1
               read(keyword(i),*,iostat=ierr) ord
               if (ierr .ne. 0) ierr1=ierr
            end if
            if(keyword(i+1) .eq. ',' )  then
               i = i + 1
               if(keyword(i+1) .ne. ',' )  then
                  i = i + 1
                  read(keyword(i),*,iostat=ierr) acc
                  if (ierr .ne. 0) ierr1=ierr
               end if
               if(keyword(i+1) .eq. ',' )  then
                  i = i + 2
                  read(keyword(i),*,iostat=ierr) Dt0
                  Dt0 = Dt0*Fs
                  if (ierr .ne. 0) ierr1=ierr 
               end if
            end if
         end if
         
         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif
         
         inttyp    = 1
         intord    = ord
         accuracy  = acc
         dt        = dt0

! verlet algorithm call

      else if( keyword(i)(1:6) .eq. 'verlet' ) then
         i0   = i
         ierr1= 0
         Dt0  = 0.1*fs
         
         if(keyword(i+1) .eq. '=' )  then
            i = i + 2
            read(keyword(i),*,iostat=ierr) Dt0
            if (ierr .ne. 0) ierr1=ierr 
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keyunits(dt0,keyword(i),ierr)
              if (ierr .eq. 1) then
                  message = 'no unit after comma'
                  call wrline(zeroi,iz,iz0)
               endif
            else
               Dt0 = Dt0*Fs
            endif

         end if
         
         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif

         inttyp   = 10
         dt       = dt0

! end verlet

      else if( keyword(i)(1:2) .eq. 'bs' ) then
         i0   = i
         ierr1= 0
         ord  = 0
         acc  = 0.0_dop
         Dt0  = 0.0_dop

         if(keyword(i+1) .eq. '=' )  then
            i = i + 1
            if(keyword(i+1) .ne. ',' )  then
               i = i + 1
               read(keyword(i),*,iostat=ierr) ord
               if (ierr .ne. 0) ierr1=ierr
            end if
            if(keyword(i+1) .eq. ',' )  then
               i = i + 1
               if(keyword(i+1) .ne. ',' )  then
                  i = i + 1
                  read(keyword(i),*,iostat=ierr) acc
                  if (ierr .ne. 0) ierr1=ierr
               end if
               if(keyword(i+1) .eq. ',' )  then
                  i = i + 2
                  read(keyword(i),*,iostat=ierr) Dt0
                  Dt0 = Dt0*Fs
                  if (ierr .ne. 0) ierr1=ierr 
               end if
            end if
         end if

         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif

         inttyp    = 2
         intord    = ord
         accuracy  = acc
         Dt        = Dt0

!     stuff for the new integrators from ODESIM
!     here: Runge-Kutta 
      else if (keyword(i)(1:2) .eq. 'rk') then
         i0   = i
         ierr1= 0
         acc  = 0.0_dop
         dt0  = 0.0_dop

         if (keyword(i+1) .eq. '=' )  then
            i = i + 1
!           read accuracy
            if (keyword(i+1) .ne. ',') then
               i = i+1
               read(keyword(i),*,iostat=ierr) acc
               if (ierr .ne. 0) ierr1=ierr
            endif
!           read initial stepsize and/or imp-ortho
 30         continue
            if (keyword(i+1) .eq. ',') then
               i = i+2
               if((keyword(i)(1:6) .eq. 'imp-or') .or. &
                  (keyword(i)(1:10).eq. 'improved-o')) then
                  liork = .true.
               else if((keyword(i)(1:9) .eq. 'no_imp-or') .or. &
                  (keyword(i)(1:13).eq. 'no_improved-o')) then
                  liork = .false.
               else if(keyword(i)(1:6) .eq. 'rktest') then
                  lrktest = .true.
               else if(keyword(i)(1:9) .eq. 'no_rktest') then
                  lrktest = .false.
               elseif (dt0 .eq. 0) then
                  read(keyword(i),*,iostat=ierr) dt0
                  dt0 = dt0*fs
                  if (ierr .ne. 0) ierr1=ierr
               else
                  message = 'Unknown keyword :'
                  call wrline(i,iz,iz0)
               endif
               goto 30 
            endif
         endif
         
         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif
         
         if (keyword(i0) .eq. 'rk5')    then
            inttyp    = 4
            accuracy  = acc
            dt        = dt0
            intord    = 5   ! unused
         else if (keyword(i0) .eq. 'rk8')    then
            inttyp    = 5
            accuracy  = acc
            dt        = dt0
            intord    = 8   ! unused

         else
            message = ' unknown keyword : '
            call wrline(i0,iz,iz0)
         end if
!     end ODESIM stuff

!     stuff for the ZVODE integrator
      else if (keyword(i)(1:5) .eq. 'zvode') then
         i0   = i
         ierr1= 0
         acc  = 0.0_dop
         dt0  = 0.0_dop

         if (keyword(i+1) .eq. '=' )  then
            i = i + 1
!           read accuracy
            if (keyword(i+1) .ne. ',') then
               i = i+1
               read(keyword(i),*,iostat=ierr) acc
               if (ierr .ne. 0) ierr1=ierr
            endif
!           read initial stepsize and/or imp-ortho
 40         continue
            if (keyword(i+1) .eq. ',') then
               i = i+2
               if((keyword(i)(1:6) .eq. 'imp-or') .or. &
                  (keyword(i)(1:10).eq. 'improved-o')) then
                  limportho = .true.
               elseif (dt0 .eq. 0) then
                  read(keyword(i),*,iostat=ierr) dt0
                  dt0 = dt0*fs
                  if (ierr .ne. 0) ierr1=ierr
               else
                  message = 'Unknown keyword :'
                  call wrline(i,iz,iz0)
               endif
               goto 40 
            endif
         endif
         
         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif
         
         if (keyword(i0) .eq. 'zvode')    then
            inttyp    = 9
            accuracy  = acc
            dt        = dt0
            intord    = 5   ! unused
         else
            message = ' unknown keyword : '
            call wrline(i0,iz,iz0)
         end if
!     end ZVODE stuff

      else if ( keyword(i) .eq. 'improved_num_derivs' ) then
         dermethod = 1

      else if ( keyword(i) .eq. 'simple_num_derivs' ) then
         dermethod = 0


      else
         message = ' unknown keyword : '
         call wrline(i,iz,iz0)
      end if
      
!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

   20 continue

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr
      lwarn=.false.

      return
      end subroutine eininttr


!#######################################################################
!
! SETINTTR
!
! integrator is set by a combination of choices in 
! mdefault(mdefault.F), einint and here.
!
! This routine sets the defaults for a trajectory calculation
!
! inttyp = 1 ABM
!        = 2 BS
!        = 4 RK5
!        = 5 RK8
!#######################################################################

      subroutine setinttr

      implicit none

! set default as RK5
      if (inttyp .eq. 0) inttyp = 4

      if(inttyp.eq.1 .and. dt.eq.0.0_dop) dt = 1.0e-4_dop*Fs
      if(inttyp.eq.2 .and. dt.eq.0.0_dop) dt = 0.2_dop*Fs
      if(inttyp.eq.4 .and. dt.eq.0.0_dop) dt = 0.0_dop
      if(inttyp.eq.5 .and. dt.eq.0.0_dop) dt = 0.0_dop

      if(inttyp.eq.1 .and. accuracy.eq.0.0_dop) accuracy = 1.0e-5_dop
      if(inttyp.eq.2 .and. accuracy.eq.0.0_dop) accuracy = 1.0e-6_dop
      if(inttyp.eq.4 .and. accuracy.eq.0.0_dop) accuracy = 1.0e-6_dop
      if(inttyp.eq.5 .and. accuracy.eq.0.0_dop) accuracy = 1.0e-6_dop
      if(inttyp.eq.9 .and. accuracy.eq.0.0_dop) accuracy = 1.0e-6_dop

      if(inttyp.eq.1 .and. intord.eq.0) intord = 6
      if(inttyp.eq.2 .and. intord.eq.0) intord = 8

      return
      end subroutine setinttr

!***********************************************************************
! 
!             MCTDH module EININTRW
!
!
!***********************************************************************

      subroutine einintrw(linperr,iz)

      implicit none

      integer(long)                    :: i,i0,iz0,ic,ord,ierr,ierr1,iprojh,k1,k2
      integer(long), intent(in)        :: iz
      real(dop)                        :: emin,emax,test1,test2
      logical(kind=4)                  :: ltest
      logical(kind=4), intent(inout)   :: linperr

!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
      routine = 'einintrw (read INTEGRATOR-SECTION)'
      iz0 = 0
      ierr1 = 0
      iprojh = 0
!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (keyword(i) .eq. 'end-integrator-section') then
         goto 20

      else if (keyword(1) .eq. 'end-input') then
         message = &
        'No end-integrator-section keyword at end of INTEGRATOR-SECTION'
         call warnmsg
         linperr=lwarn.or.linperr
         lwarn=.false.
         RETURN

      else if(keyword(i)(1:5) .eq. 'ncheb' ) then
         i0   = i
         ierr1= 0
         ord  = 0

         if(keyword(i+1) .eq. '=' )  then
            i = i + 1
            if(keyword(i+1) .ne. ',' )  then
               i = i + 1
               read(keyword(i),*,iostat=ierr) ord
               if (ierr .ne. 0) ierr1=ierr
            end if
         end if

         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif

         inttyp    = 1
         norder    = ord

      else if(keyword(i)(1:6) .eq. 'hscale' ) then
         i0   = i
         ierr1= 0
         emin  = 0.0_dop
         emax  = 0.0_dop

         if(keyword(i+1) .eq. '=' )  then
            i = i + 1
            if(keyword(i+1) .ne. ',' )  then
               i = i + 1
               read(keyword(i),*,iostat=ierr) emin
               if (ierr .ne. 0) ierr1=ierr
            end if
            if(keyword(i+1) .eq. ',' )  then
               i = i + 1
               if(keyword(i+1) .ne. ',' )  then
                  i = i + 1
                  read(keyword(i),*,iostat=ierr) emax
                  if (ierr .ne. 0) ierr1=ierr
                  if (keyword(i+1).eq.',' ) then
                     i=i+2
                     call keyeunits(emax,keyword(i),ierr)
                     if (ierr .eq. 1) then
                         message = '(hscale)no energy unit after comma'
                         call wrline(zeroi,iz,iz0)
                     endif
                     if (ierr /= 0) ierr1=ierr
                     call keyeunits(emin,keyword(i),ierr)
                     if (ierr /= 0) ierr1=ierr
                  endif
               end if

            end if
         end if
         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif

         inttyp    = 2
         aunds     = 2.0_dop/(emax-emin)
         bunds     = -1.0_dop-emin*aunds

      else if(keyword(i)(1:5) == 'eprob' ) then
         i0      = i
         ierr1   = 0
         fluxsave= 0
         femin   = 0.0_dop
         femax   = 0.0_dop
         prne    = 0

         if(keyword(i+1) .eq. '=' )  then
            i = i + 1
            if(keyword(i+1) .ne. ',' )  then
               i = i + 1
               read(keyword(i),*,iostat=ierr) fluxsave
               if (ierr .ne. 0) ierr1=ierr
            end if
            if(keyword(i+1) .eq. ',' )  then
               i = i + 1
               if(keyword(i+1) .ne. ',' )  then
                  i = i + 1
                  read(keyword(i),*,iostat=ierr) prne
                  if (ierr .ne. 0) ierr1=ierr
               end if
            end if
            if(keyword(i+1) .eq. ',' )  then
               i = i + 1
               if(keyword(i+1) .ne. ',' )  then
                  i = i + 1
                  read(keyword(i),*,iostat=ierr) femin
                  if (ierr .ne. 0) ierr1=ierr
               end if
            end if
            if(keyword(i+1) .eq. ',' )  then
               i = i + 1
               if(keyword(i+1) .ne. ',' )  then
                  i = i + 1
                  read(keyword(i),*,iostat=ierr) femax
                  if (ierr .ne. 0) ierr1=ierr
                  if (keyword(i+1).eq.',' ) then
                     i=i+2
                     call keyeunits(femax,keyword(i),ierr)
                     if (ierr .eq. 1) then
                         message = '(Probability)no energy unit after comma'
                         call wrline(zeroi,iz,iz0)
                     endif
                     if (ierr /= 0) ierr1=ierr
                     call keyeunits(femin,keyword(i),ierr)
                     if (ierr /= 0) ierr1=ierr
                  endif
               end if
            end if
         end if

         if (ierr1 .ne. 0) then
            message = 'Incorrect arguments for keyword :'
            call wrline(i0,iz,iz0)
         endif

!------------------------------------------------------------------------
!  Check whether fluxsave gets us into trouble with aliasing
!  for these choices of pemin and pemax
!------------------------------------------------------------------------
         test1=-DACOS(aunds*femin+bunds)*fluxsave/pi
         test2=-DACOS(aunds*femax+bunds)*fluxsave/pi
         k1=int(test1-1.0_dop)
         k2=int(test2)
!
         if (((test1-0.1_dop) <= k1).or.((test2+0.1_dop) >= k2).or.&
                                       ((abs(k1-k2)) /= 1)) then
            write(ilog,'(/,72a1)')  ('#',i=1,72)
            write (ilog,'(a)')'This value for fluxsave is not acceptable,',&
                              ' because of expected aliasing.'
            ltest=.true.
            fluxsave=fluxsave-1
            do while (ltest)
               test1=-DACOS(aunds*femin+bunds)*fluxsave/pi
               test2=-DACOS(aunds*femax+bunds)*fluxsave/pi
               k1=int(test1-1.0_dop)
               k2=int(test2)
!
               if (((test1-0.1_dop) <= k1).or.((test2+0.1_dop) >= k2).or.&
                                              ((abs(k1-k2)) /= 1)) then
                  fluxsave=fluxsave-1
               else
                  ltest=.false.
                  write (ilog,*)'Fluxsave set to ',fluxsave
               endif
               if (fluxsave < 1) then
                  message = 'fluxsave becomes too low. Pick higher starting point.'
                  call errormsg
               endif
            enddo
         endif

      else
         message = ' unknown keyword : '
         call wrline(i,iz,iz0)
      end if
!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

   20 continue

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      linperr=lwarn.or.linperr
      lwarn=.false.

      return
      end subroutine einintrw


!#######################################################################
!
! Some defaults need to be set for different run types
!
!#######################################################################

      subroutine intdef

      implicit none

      integer(long) :: f,h,k,m,n
      logical(kind=4) :: llha=.false., lcheck=.true.

!-----------------------------------------------------------------------
! Defaults for integrator
!
! default integration scheme is CMF and (1-P)(h + ...) phi
! lconst, lfunkr = .true.  lfunkl = .false.
! (changed from prcs 90.30 - previously VMF and h + (1-P)(...) phi
! lconst, lfunkr = .false. lfunkl = .true. )
!
! relaxation default is VMF
!
! choice of CMF type (var, fix etc.) and defaults for integrator 
! (ABM, BS etc.) set in setint(einint.F)
!-----------------------------------------------------------------------
      lopro       = .true.
      limportho   = .false.
      if ((psitype .eq. 0 .or. psitype .eq. 5) .and. .not. lrlx) then
         lconst      = .true.
      else
         lconst      = .false.
      endif
      if(lddmctdh)then
         lconst      = .false.
         lfunkl      = .true.
         lfunkr      = .false.
      else
         lfunkl      = .false.
         lfunkr      = .true.
      endif
      lconstvar   = .true.
      lconstfix   = .false.
      lcmfadaphi  = .false.
      lcmfadaa    = .false.
      loldsilerr  = .true.
      inttyp      = 0
      intord      = 0
      Accuracy    = 0.0_dop
      Dt          = 0.0_dop
      UpDateTime  = 0.0_dop
      epsupdate   = 0.0_dop
      inttypePhi  = 0
      intorderPhi = 0
      accphi      = 0.0_dop
      dtphi       = 0.0_dop
      inttypeA    = 0
      intorderA   = 0
      accavec     = 0.0_dop
      DtA         = 0.0_dop
      maxsilorder = 0
      iflagzvode  = 10
      lzvodecrit  = .true.
      zvodemaxit  = 10000
      zvodeminstp = 0.0_dop
      minrk5      = -999.0_dop

      lnaturcmf   = .false.
      lenorb      = .false.
      ltdvr       = .false.
      lcdvr       = .false.

      lmf         = .false.

      usehsym   = .true.
      lwferror = .false.

      if (lgwp) then
         gwpintordm(:) = lhaorder    ! set GWP integrals same as potential
         do m=1,nmode
            if (gwpm(m)) cycle
            gwpmodelm(m) = -999    ! flag non-GWP Modes
         enddo           

         maxngdep(:,:) = 0
         maxngdyn(:,:) = 0
         maxtgdyn(:,:) = 0.0_dop
         mincgdyn(:,:) = 0.0_dop
         lcapgwp(:,:,:)  = .false.

         ngwpovr(:,:)=0

         lrmnoise = .true. ! remove noise from dtpsi
      endif

      BELint = 0.0_dop
      BELoff = 0.0_dop
      BELprint = 0
      BELflags(:) = 0
      BELflags(1) = 1e-2
      BELflags(2) = 1e-2
      BELflags(3) = 1e-2
      BELflags(4) = 40
      BELflags(5) = 1e-3
      BELflags(6) = 3e-0
      BELflags(7) = 1
      BELflags(8) = 2
      BELflags(9) = 4000
      BELflags(10) = 1
      BELflags(11) = 1.0e-5_dop
      BELflags(12) = 1
      BELflags(13) = 10
      BELflags(14) = 0
      reCompDtPsi = 1

!-----------------------------------------------------------------------
! GWP runs
!-----------------------------------------------------------------------
      if (lgwp) then

! Look whether potential operator is an LHA or not
         llha = .false.
         do k=1,khtot(1)
            do f = 1,ndof
               h = hamilton(f,k)
               if (htmtype(h) .eq. 105) llha = .true.
            enddo
         enddo
! GAP potentials are not LHA
         if (lgausspot) llha = .false.

!-----------------------------------------------------------------------
! CX not working for dentype = 2
!-----------------------------------------------------------------------
         if (dentype .eq. 2) gwpcxm(:)=0

!-----------------------------------------------------------------------
! For classical modes set CX
!-----------------------------------------------------------------------
         do m=1,nmode
            if (.not. gwpm(m)) cycle

            f=spfdof(1,m)
            if (ipbaspar(1,f) .eq. 1) then
               lcheck = .true.
               gwpcxm(m) = 1
               gwpmodelm(m) = 1
               gwpmodel = 1
               do n=2,nspfdof(m)
                  f=spfdof(n,m)
                  if (ipbaspar(1,f) .ne. 1) lcheck = .false.
               enddo
            else
               lcheck = .true.
               do n=2,nspfdof(m)
                  f=spfdof(n,m)
                  if (ipbaspar(1,f) .eq. 1) lcheck = .false.
               enddo
            endif

            if (.not. lcheck) then
               routine = 'Intdef (Einintmod.F90)'
               message = 'Cannot mix classical and quantum DOF in a mode'
               call errormsg
            endif
         enddo

!-----------------------------------------------------------------------
! Check if only some GWP Modes are classical (mixed mode)
!-----------------------------------------------------------------------
         lcheck = .true.
         do m=1,nmode
            if (.not. gwpm(m)) cycle
            if (gwpmodelm(m) .ne. gwpmodel) lcheck = .false.
         enddo
         if (.not. lcheck) gwpmodel = -1

!-----------------------------------------------------------------------
! Set dynamical coupling.
! for relaxations, do not use CX and use rough criteria for 
! selecting dynamical parameters
!-----------------------------------------------------------------------
         igwpdynm(:) = 1
         if (lrlx) then
            gwpintm(:) = 0
            epsdyn = 1.0e-4_dop
         else
            if (llha) then
               gwpintm(:) = 1
            else
               gwpintm(:) = 0
            endif
         endif

      endif

      return
      end subroutine intdef

      end module einintmod

