!########################################################################

      module gaussian

use decimal, only: long, dop
use global
use logdat
use mmlib,   only: mmaxzzh, qqxazz, qqxxzz, mmxtzz, mmxxzz, qqaxzz,&
                   dqxxdz, ddxxdz, mmxazz, mmaxzz
use mtlib,   only: qtxxzz, qtxxzzr, qtxxzza, dtxxzzr
use xvlib,   only: qvxxzz, vvaxzz, xvxxdzr, xvixdzo, xvxxdzo, mvaxzz
use d2tens,  only: mkherm
use op1lib,  only: cpvxz, cpmxz, zerovxz, zeromxz, sum2qxz, &
                   cpqxz, zeromxd, unitqxz
use op2lib,  only: submxxzo1, submxxz, subvxxzo1, sum2qxxz, addmxxzo
use lalib,   only: simtran1btz, simtran2btz,simtran1bz,simtran2bz,&
                   simtranbz

use dvrdatmod

use griddatmod
use psidef
use gwptrjmod
use gwptrj
use globmemmod
use potevalmod, only: calcpes
use d2tens, only: mkherm
use gwplib, only: gwporthonorm, getbdot,renormdgwp1, testgs2

use maxv, only:maxdim
use timing
use datenmod, only: lgentrj
use operdef
use hpsimod
use splinepotmod
use runpropmod
use xlocphihphimod, only: xlocphihunphi
use evaluate, only: calcenergy_g 

use channels

use openmpmod

      implicit none
      private
      public :: gdot,adot,subengwp,wkgaussian,gdot1ms, &
                gwpcons,gwpcmat,gwpymat,&
                gdottrj,dgwpdot,dgwpdot1ms,chkdmato,&
! following not used but may be needed
                cont_pt,cont_diag,diag_cont,xivectopq,adot1bel
      contains

!#######################################################################

      subroutine gwpymat(yp,dicht2,hloch,gh,hung,xp,npg1,gwpdim1,dim1,m,s,&
                 gs2,gsinv,psi,zetf1,phidim1)

!+-----------------------------------------------------------------------+
!     calculates time evolution of Gaussian parameters.
!+-----------------------------------------------------------------------+

      implicit   none

      integer(long)                        :: p1,ig,k,si,sf
      integer(long), intent(in)            :: s,m,npg1,gwpdim1,dim1,zetf1,phidim1
      complex(dop)                                                 :: coeff
      complex(dop), dimension(npg1,gwpdim1)                        :: xp
      complex(dop), dimension(gwpdim1,gwpdim1,npg1)                :: hungx
      complex(dop), dimension(ghmatdim), intent(in)                :: gh
      complex(dop), dimension(hlochdim), intent(in)                :: hloch
      complex(dop), dimension(zetf1:zetf1+phidim1-1), intent(in)   :: psi
      complex(dop), dimension(gwpdim1,gwpdim1,npg1), intent(in)    :: hung
      complex(dop), dimension(gwpdim1,gwpdim1), intent(in)         :: gsinv
      complex(dop), dimension(dim1,dim1), intent(in)               :: dicht2
      complex(dop), dimension(npg1-1,gwpdim1), intent(inout)         :: yp
      complex(dop), dimension(gwpdim1,gwpdim1,npg1,npg1), intent(in) :: gs2

      complex(dop), dimension(gwpdim1,gwpdim1)        :: gdicht2,temp1,temp2,&
                                                         projector

      integer(long)                                    :: ghlochdim
      integer(long), dimension(maxkoe)                 :: pghloch
      complex(dop), dimension(:), allocatable          :: ghloch
      integer(long), save                              :: tid=0 ! timer ID

      call get_timer(tid, 'Gwp_Ymat')
      call start_timer(tid)

      ghlochdim=0
      do k=1,kzahl(1)
         si=ki(k)
         sf=kf(k)
         pghloch(k) = ghlochdim + 1
         ghlochdim = ghlochdim+gwpdim(m,si)*gwpdim(m,sf)
      enddo
      ghlochdim = max(1,ghlochdim)
      allocate(ghloch(ghlochdim))

      if (psitype .eq. 0 .or. psitype .eq. 5) then
         call simtran1btz(dicht2,psi(zdgwp(m,s)),gdicht2,dim1,gwpdim1)

         do k=1,kzahl(1)
            si=ki(k)
            sf=kf(k)

            if (klha(k) .gt. 0) then
! for klha diag not used and transformation has been done in mfields.
               call cpmxz(hloch(phloch(m,k)),ghloch(pghloch(k)),&
                    gwpdim(m,sf),gwpdim(m,si))
            else if (.not. diag(m,k) .and. sf .eq. s) then
               call simtran2btz(psi(zdgwp(m,sf)),hloch(phloch(m,k)),&
                    psi(zdgwp(m,si)),ghloch(pghloch(k)),dim(m,si),&
                    dim(m,sf),gwpdim(m,si),gwpdim(m,sf))
            endif
         enddo

      else
         gdicht2=dicht2(1:gwpdim1,1:gwpdim1)

         do k=1,kzahl(1)
            si=ki(k)
            sf=kf(k)
            if (.not. diag(m,k) .and. sf .eq. s) then
               call cpmxz(hloch(phloch(m,k)),ghloch(pghloch(k)),&
                    gwpdim(m,sf),gwpdim(m,si))
            endif
         enddo
      endif

      hungx = 0.0_dop
      if (gwpcxm(m) .gt. 0) then
         call sumhungx(hungx,xp,gs2,gwpdim(m,s),npg(m))

         hungx = hung - hungx
      else
         hungx = hung
      endif

!+-----------------------------------------------------------------------+  
!     To calculate vector YP: 
!+-----------------------------------------------------------------------+  

! ... initialize vector yp :
      yp=(0.0_dop,0.0_dop)

! first term Y_alpha,j = sum_l <H^(alpha,0)_jl>_jl

! omp parallel if(lompthread) num_threads(ompthread) &
! omp private(p1,k,temp1)
! omp do schedule(dynamic)     
      do p1 = 2,npg1
         temp1=(0.0_dop,0.0_dop)
         do k=1,kzahl(1)
            si=ki(k)
            sf=kf(k)
            coeff=(1.0_dop,0.0_dop)
            if (.not. diag(m,k) .and. sf .eq. s) then
               call ypmatrix(ghloch(pghloch(k)),gh(ghmat(k,m)),&
                    temp1,gwpdim(m,sf),gwpdim(m,si),coeff,p1,npg1)
            endif
         enddo

          coeff=(1.0_dop,0.0_dop)
          call ypmatrix(gdicht2,hungx,temp1,gwpdim(m,s),&
               gwpdim(m,s),coeff,p1,npg1)

         yp(p1-1,1:gwpdim(m,s))=temp1(1:gwpdim(m,s),1)
      enddo
! omp end do

! second term Y_alpha,j = sum_l [S^(alpha,0) S^-1 <H_jl>]_jl
      if(.not.lcysinv) call gwpproject(gs2,projector,dim1)

      temp1=(0.0_dop,0.0_dop)   
! omp do schedule(dynamic)     
      do k=1,kzahl(1)
         si=ki(k)
         sf=kf(k)
         if (.not. diag(m,k) .and. sf .eq. s) then
            call ymatrix(ghloch(pghloch(k)),gh(ghmat(k,m)),&
                                  temp1,gwpdim(m,sf),gwpdim(m,si))
         endif
      enddo
! omp enddo
      call ymatrix(gdicht2,hungx,temp1,gwpdim(m,s),gwpdim(m,s))

      if(lcysinv)then
!         call mmxxzz(gsinv,temp1,temp2,gwpdim(m,s),gwpdim(m,s),gwpdim(m,s))
! S^-1 is hermitian
          call zhemm('L','U',gwpdim(m,s),gwpdim(m,s),(1.0_dop,0.0_dop),&
                     gsinv,gwpdim1,temp1,gwpdim1,(0.0_dop,0.0_dop),&
                     temp2,gwpdim1)
      else
!         call qqxxzz(projector,temp1,temp2,gwpdim1)
! the projector is hermitian
         call zhemm('L','U',gwpdim1,gwpdim1,(1.0_dop,0.0_dop),&
                    projector,gwpdim1,temp1,gwpdim1,(0.0_dop,0.0_dop),&
                    temp2,gwpdim1)
      endif

! omp do schedule(dynamic)     
      do p1=2,npg1
!         call qqxxzz(gs2(1,1,p1,1),temp2,temp1,dim1)
         call zgemm('N','N',gwpdim1,gwpdim1,gwpdim1,(1.0_dop,0.0_dop),&
                     gs2(:,:,p1,1),gwpdim1,temp2,gwpdim1,(0.0_dop,0.0_dop),&
                     temp1,gwpdim1)
! only the diagonal of temp1 sees to be used
! do we need to calculate all the matrix?
         do ig=1,gwpdim1
            yp(p1-1,ig)=yp(p1-1,ig) - temp1(ig,ig)
         enddo

      enddo
! omp enddo
! omp end parallel

      if (allocated(ghloch)) deallocate(ghloch)

      call stop_timer(tid)

      return
      end subroutine gwpymat

!#######################################################################

      subroutine sumhungx(hungx,xp,gs2,gwpdim1,npgm)

      implicit none

      integer(long)                     :: e,e1,p1,p2
      integer(long), intent(in)         :: gwpdim1,npgm
      complex(dop), dimension(gwpdim1,gwpdim1,npgm), intent(inout)   :: hungx
      complex(dop), dimension(npgm,gwpdim1), intent(in)              :: xp
      complex(dop), dimension(gwpdim1,gwpdim1,npgm,npgm), intent(in) :: gs2

      do e=1,gwpdim1
         do e1=1,gwpdim1
            do p1=1,npgm
               do p2=2,npgm
                  hungx(e1,e,p1) = hungx(e1,e,p1) + gs2(e1,e,p1,p2)*xp(p2,e)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine sumhungx

!#######################################################################

      subroutine ypmatrix (hloch,gh,yp,gwpdim,gwpdim1,koeff,p,npg1)

      implicit none

      integer(long)                                      :: e,e2
      integer(long), intent(in)                          :: gwpdim,gwpdim1,p,npg1
      complex(dop), intent(in)                           :: koeff
      complex(dop), dimension(gwpdim), intent(inout)           :: yp
      complex(dop), dimension(gwpdim,gwpdim1), intent(in)      :: hloch
      complex(dop), dimension(gwpdim,gwpdim1,npg1), intent(in) :: gh

! seems it could be implemented as a vector where each line is the dot product
! of the lines of the matrices
      do e=1,gwpdim1
         do e2=1,gwpdim
            yp(e2)=yp(e2)+hloch(e2,e)*gh(e2,e,p)*koeff
         enddo
      enddo

      return
      end subroutine ypmatrix

!***********************************************************************

      subroutine ymatrix (hloch,gh,ymat,gwpdim,gwpdim1)

      implicit none

      integer(long)                                   :: e,e1,e2
      integer(long), intent(in)                       :: gwpdim,gwpdim1
      complex(dop), dimension(gwpdim,gwpdim1), intent(in)   :: hloch,gh
      complex(dop), dimension(gwpdim,gwpdim), intent(inout) :: ymat

! seems matrix multiplication by the transpose
! (note that ymat may not be zero at input)
      do e=1,gwpdim1
         do e1=1,gwpdim
            do e2=1,gwpdim
               ymat(e2,e1)=ymat(e2,e1)+hloch(e1,e)*gh(e2,e)
            enddo
         enddo
      enddo

      return
      end subroutine ymatrix

!#######################################################################

      subroutine gwpcmat(cp,dicht2,gs2,gsinv,npg1,gwpdim1,dim1,dgwp,&
                 psitype1)

!+-----------------------------------------------------------------------+  
!     calculates time evolution of Gaussian parameters. 
!+-----------------------------------------------------------------------+  

      implicit   none

      integer(long)             :: p1,p2,cpdim
      integer(long), intent(in) :: npg1,gwpdim1,dim1,psitype1

      complex(dop), dimension(gwpdim1,gwpdim1,npg1,npg1), intent(in)     :: gs2
      complex(dop), dimension(npg1-1,gwpdim1,npg1-1,gwpdim1),intent(inout) :: cp
      complex(dop), dimension(gwpdim1,gwpdim1), intent(in) :: gsinv
      complex(dop), dimension(dim1,dim1), intent(in)       :: dicht2
      complex(dop), dimension(gwpdim1,dim1), intent(in)    :: dgwp
      complex(dop), dimension(gwpdim1,gwpdim1)             :: gdicht2,projector
      complex(dop), dimension(:,:), allocatable            :: temp1,temp2

      if (psitype1 .eq. 0 .or. psitype1 .eq. 5) then
         call simtran1btz(dicht2,dgwp,gdicht2,dim1,gwpdim1)
      else
         gdicht2=dicht2(1:gwpdim1,1:gwpdim1)
      endif

      if(.not.lcysinv) call gwpproject(gs2(:,:,1,1),projector,gwpdim1)

      cp=(0.e0_dop,0.e0_dop)

!+-----------------------------------------------------------------------+  
!     To calculate matrix CP : 
!     (i)   calculate array temp1 = gs1 * gsinv (for given parameter p1)
!     (ii)                  temp2 = gs2(p1,p2) - temp1 * gs1(p2)
!     (iii) multiply temp2 by the density for corresponding indices.
!+-----------------------------------------------------------------------+
!$omp parallel if(lompthread) num_threads(ompthread) &
!$omp private(p1,p2,temp1,temp2)
      allocate(temp1(gwpdim1,gwpdim1),temp2(gwpdim1,gwpdim1))
!$omp do schedule(dynamic)     
      do p1 = 2,npg1
         if(lcysinv)then
! temp1 = gsinv*gs2(p1,1)  S^-1 is hermitian
            call zhemm('R','U',gwpdim1,gwpdim1,(1.e0_dop,0.e0_dop),&
                       gsinv,gwpdim1,gs2(:,:,p1,1),gwpdim1,&
                       (0.e0_dop,0.e0_dop),temp1,gwpdim1)
         else
! temp1 = projector*gs2(p1,1) projector is hermitian
            call zhemm('R','U',gwpdim1,gwpdim1,(1.e0_dop,0.e0_dop),&
                       projector,gwpdim1,gs2(:,:,p1,1),gwpdim1,&
                       (0.e0_dop,0.e0_dop),temp1,gwpdim1)
         endif

         do p2 = 2,npg1
            temp2=gs2(:,:,p1,p2)
! temp2 = -1*temp1*gs2(1,p2) + gs2(p1,p2)
            call zgemm('N','N',gwpdim1,gwpdim1,gwpdim1,(-1.e0_dop,0.e0_dop),&
                       temp1,gwpdim1,gs2(:,:,1,p2),gwpdim1,&
                       (1.e0_dop,0.e0_dop),temp2,gwpdim1)

            !element-wise multiplication
            cp(p1-1,:,p2-1,:)=gdicht2*temp2

         enddo
      enddo
!$omp end do
      deallocate(temp1,temp2)
!$omp end parallel

      cpdim = (npg1-1)*gwpdim1
      call mkherm(cp,cpdim)

      return
      end subroutine gwpcmat

!###########################################################################
! Routine to construct a projector of orthogonal functions from a set of 
! (generally) non-orthogonal GWPs i.e
! \sum_i |\phi_{i}><\phi_{i}| rather than \sum_rs |\g_{r}>S^{-1}_{rs}<g_{s}|
! This is used to form the C-matrix and Y-vector, avoiding the use of the 
! inverse of the overlap matrix.
!
! GWR 04/16
!###########################################################################

      subroutine gwpproject(gs2,projector,gwpdim1)

      implicit none

      integer(long), intent(in)                             :: gwpdim1
      integer(long)                                         :: nzero
      complex(dop), dimension(gwpdim1,gwpdim1), intent(in)  :: gs2
      complex(dop), dimension(gwpdim1,gwpdim1), intent(inout) :: projector

      complex(dop), dimension(gwpdim1,gwpdim1) :: transmat

      integer(long), save                         :: tid=0 ! timer ID

      call get_timer(tid, 'Gwp_projector')
      call start_timer(tid)

! Call to the Gram-Schmidt-like orthonormalisation routine. 
      call unitqxz(transmat,gwpdim1)
      call gwporthonorm(transmat,gs2,gwpdim1,nzero,epsgwp)

! The product of the transformation matrix and its adjoint gives the
! projector.
!      call qqxazz(transmat,transmat,projector,gwpdim1)
      call zgemm('N','C',gwpdim1,gwpdim1,gwpdim1,(1.e0_dop,0.e0_dop),&
                 transmat,gwpdim1,transmat,gwpdim1,(0.e0_dop,0.e0_dop),&
                 projector,gwpdim1)

     call stop_timer(tid)

      return
      end subroutine gwpproject

!#######################################################################
!
!      subroutine gwpcmat_norho(cp,gs2,gsinv,npg1,gwpdim1)
!
!+-----------------------------------------------------------------------+  
!     calculates time evolution of Gaussian parameters. 
!+-----------------------------------------------------------------------+  
!
!      implicit   none
!
!      integer(long)             :: p1,p2,ig,jg,cpdim
!      integer(long), intent(in) :: npg1,gwpdim1
!
!      complex(dop), dimension(gwpdim1,gwpdim1,npg1,npg1), intent(in)     :: gs2
!      complex(dop), dimension(npg1-1,gwpdim1,npg1-1,gwpdim1),intent(inout) :: cp
!      complex(dop), dimension(gwpdim1,gwpdim1), intent(in)   :: gsinv
!      complex(dop), dimension(gwpdim1,gwpdim1)               :: temp1,temp2
!
!      cpdim = (npg1-1)*gwpdim1
!      call zeromxz(cp,cpdim,cpdim)
!
!+-----------------------------------------------------------------------+  
!     To calculate matrix CP : 
!     (i)   calculate arrays temp1 = gs1 * gsinv (for given parameter p1)
!     (ii)                   temp2 = temp1 * gs1 (for given parameter p2)
!     (iii) add array gs2 (for given p1 and p2) and multiply by the
!           density for corresponding indices.
!+-----------------------------------------------------------------------+
!      do p1 = 2,npg1
!         call qqxxzz(gs2(1,1,p1,1),gsinv,temp1,gwpdim1)
!
!         do p2 = 2,npg1
!            call qqxxzz(temp1,gs2(1,1,1,p2),temp2,gwpdim1)
!
!            do ig = 1,gwpdim1
!               do jg = 1,gwpdim1
!                  cp(p1-1,ig,p2-1,jg) = gs2(ig,jg,p1,p2) - temp2(ig,jg)
!               enddo
!            enddo
!         enddo
!      enddo
!
!      call mkherm(cp,cpdim)
!
!      return
!      end subroutine gwpcmat_norho
!
!#######################################################################

      subroutine gwpcmatinv(cp,yp,dtlambda,ngwpdyn1)

!+-----------------------------------------------------------------------+  
!     calculates time evolution of Gaussian parameters. 
!+-----------------------------------------------------------------------+  

      implicit   none

      integer(long)                               :: i,j,k,nzero
      integer(long), intent(in)                   :: ngwpdyn1
      complex(dop), dimension(ngwpdyn1,ngwpdyn1), intent(inout) :: cp
      complex(dop), dimension(ngwpdyn1), intent(in)             :: yp 
      complex(dop), dimension(ngwpdyn1), intent(inout)            :: dtlambda
      complex(dop), dimension(ngwpdyn1,ngwpdyn1)  :: evec
      real(dop), dimension(ngwpdyn1)              :: eval

      integer(long)                               :: l,e2,lda,info
      complex(dop), dimension(ngwpdyn1,ngwpdyn1)  :: workc
      real(dop), dimension(3*ngwpdyn1)            :: workr
      integer(long), save                         :: tid=0 ! timer ID

      call get_timer(tid, 'Gwp_Cmat')
      call start_timer(tid)

!+-----------------------------------------------------------------------+
!     To calculate inverse of CP: use diagonalization + division by
!     eigenvalues.
!+-----------------------------------------------------------------------+
      l=ngwpdyn1
      e2 = l*l
      info  = 0
      lda=l
      evec(:,:)=cp(:,:)

      call zheev('V','U',lda,evec,lda,eval,workc,e2,workr,info)
      if (info .ne. 0) then
         routine = 'gwpcmatinv'
         message = 'Cannot diagonalize C matrix'
         call errormsg
      endif

      if (gwpeoms .eq. 9) then
!-----------------------------------------------------------------------
! Project out zero eigenvectors
!-----------------------------------------------------------------------
         nzero = 0
         do i=1,l
            if( abs(eval(i)) .lt. epsgwp) then
               nzero = nzero+1
               eval(i) = 0.0_dop
            endif
         enddo
         do i=nzero+1,l
            eval(i) = 1.0_dop/eval(i)
         enddo
         cp(:,:)=(0.0_dop,0.0_dop)
         do i = 1,l
            do j = 1,l
               do k = nzero+1,l
                   cp(i,j) = cp(i,j) + evec(i,k)*eval(k)*dconjg(evec(j,k))
               enddo
            enddo
         enddo

      else
!+-----------------------------------------------------------------------+
!     regularize :
!+-----------------------------------------------------------------------+
         do i=1,l
            if( abs(eval(i)) .lt. 64.0_dop*epsgwp) then
!          write(6,*) ' r e g u l a r i z a t i o n (gdot)'
               eval(i)=eval(i)+epsgwp*exp(-eval(i)/epsgwp)
               eval(i) = 1.0_dop/eval(i)
            else
               eval(i) = 1.0_dop/eval(i)
            endif
         enddo
         cp(:,:)=(0.0_dop,0.0_dop)
         do i = 1,l
            do j = 1,l
               do k = 1,l
                   cp(i,j) = cp(i,j) + evec(i,k)*eval(k)*dconjg(evec(j,k))
               enddo
            enddo
         enddo
      endif
      call mkherm(cp,l)

!+-----------------------------------------------------------------------+
!     multiply to get the solution: dtlambda = cp * yp
!+-----------------------------------------------------------------------+
      call zhemv('U',l,(1.0_dop,0.0_dop),cp,l,yp,1,(0.d0,0.d0),dtlambda,1)

      call stop_timer(tid)

      return
      end subroutine gwpcmatinv

!#######################################################################

      subroutine tikhonov(cp,yp,dtlambda,ngwpdyn1)

!+----------------------------------------------------------------------+
!  Tikhonov regularization, with the use of SVD for inversion
!
! This solves the equation
! dtlambda = Inv[C^{dag}*C + G^{dag}*G] * C^{dag} * Y
! where G is a regularization matrix. Here a simple diagonal G is used
! with a small factor, and since C is hermitian the algorithm is
! dtlambda = Inv[C^2 + diag(tikh_fac)] * C * Y
!+----------------------------------------------------------------------+

      implicit   none

      integer(long)                                             :: i,info
      integer(long), intent(in)                                 :: ngwpdyn1
      complex(dop), dimension(ngwpdyn1,ngwpdyn1), intent(in)    :: cp
      complex(dop), dimension(ngwpdyn1), intent(in)             :: yp 
      complex(dop), dimension(ngwpdyn1), intent(inout)            :: dtlambda
      
      complex(dop), dimension(ngwpdyn1,ngwpdyn1)                :: leftmat,rightmat


      ! build diagonal regularization matrix which will be overwritten
      ! tikh_fac is an input parameter with default = 1.0e-12
      leftmat(:,:)=(0.0_dop,0.0_dop)
      do i=1,ngwpdyn1
         !leftmat(i,i)=tikh_fac
         leftmat(i,i)=epsgwp
      enddo

      ! calculate leftmat = C^2 + diag(tikh_fac)

      call zherk('U','C',ngwpdyn1,ngwpdyn1,1.0_dop,cp,ngwpdyn1,1.0_dop,leftmat,ngwpdyn1)


      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      ! Pseudoinversion using SVD (by yapol)              !
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      !
      ! Do SVD
      !
!      allocate(sx(ngwpdyn1),stat=info)
!      if (info /= 0) then
!         routine = 'tikhonov'
!         message = 'Error allocating S array'
!         call errormsg
!      endif
!      !
!      allocate(umat(ngwpdyn1,ngwpdyn1),vmat(ngwpdyn1,ngwpdyn1),wmat_inv(ngwpdyn1,ngwpdyn1),tmp_mat(ngwpdyn1,ngwpdyn1),stat=info)
!      if (info /= 0) then
!         routine = 'tikhonov'
!         message = 'Error allocating arrays for SVD'
!         call errormsg
!      endif
!      !
!      !write(*,*) "C-matrix before SVD:"
!      !do i=1,ngwpdyn1
!      !   write(*,*) (cp1(i,j),j=1,ngwpdyn1)
!      !enddo
!      !write(*,*)
!      !
!      lworkx = 10*ngwpdyn1
!      allocate(work(lworkx),rwork(5*ngwpdyn1),stat=info)
!      if (info /= 0) then
!         routine = 'tikhonov'
!         message = 'Error allocating working arrays for SVD'
!         call errormsg
!      endif
!      !
!      call zgesvd('A','A',ngwpdyn1,ngwpdyn1,leftmat,ngwpdyn1,sx,umat,ngwpdyn1,vmat,ngwpdyn1,work,lworkx,rwork,info) ! leftmat -> (cp*cp+tikh_fac)
!      !
!      do i=1,ngwpdyn1
!         do j=1,ngwpdyn1
!            if (i == j) then
!      !         if (sx(i) .lt. 64.0_dop*epsgwp) then             
!      !            sx(i) = sx(i) + epsgwp*exp(-sx(i)/epsgwp)
!      !         endif
!               wmat_inv(i,i) = cmplx(1.0d0/sx(i),0.0d0)
!            else
!               wmat_inv(i,j) = (0.0d0,0.0d0)
!            endif
!         enddo
!      enddo
!      leftmat(:,:) = 0.0d0
!      !
!      !write(*,*) "Singular values:"
!      !write(*,*) sx
!      !write(*,*)
!      !
!      alpha = (1.0_dop,0.0_dop)
!      beta = (0.0_dop,0.0_dop)
!      call zgemm('N','C',ngwpdyn1,ngwpdyn1,ngwpdyn1,alpha,wmat_inv,ngwpdyn1,umat,ngwpdyn1,beta,tmp_mat,ngwpdyn1)
!      !
!      call zgemm('C','N',ngwpdyn1,ngwpdyn1,ngwpdyn1,alpha,vmat,ngwpdyn1,tmp_mat,ngwpdyn1,beta,leftmat,ngwpdyn1) ! leftmat -> (cp*cp+tikh_fac)-1
!      !
!      deallocate(sx,umat,vmat,wmat_inv,tmp_mat,work,rwork)
!      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!      !
!      call zgemm('N','C',ngwpdyn1,ngwpdyn1,ngwpdyn1,alpha,leftmat,ngwpdyn1,cp,ngwpdyn1,beta,rightmat,ngwpdyn1)


 
      !--------------------------------------------------------------------
      ! Use the Cholesky solver with multiple right-hand-sides to calculate
      ! rightmat = Inv[C^2 + diag(tikh_fac)] * C 
      !--------------------------------------------------------------------
      rightmat=cp
      call zposv('U',ngwpdyn1,ngwpdyn1,leftmat,ngwpdyn1,rightmat,ngwpdyn1,info)


      call mkherm(rightmat,ngwpdyn1)

      ! multiply to get the solution: dtlambda = rightmat * yp
      call zhemv('U',ngwpdyn1,(1.0_dop,0.0_dop),rightmat,ngwpdyn1,yp,1,&
              (0.0_dop,0.0_dop),dtlambda,1)
      
      return
      end subroutine tikhonov

!#######################################################################

      subroutine svdcmat(cp,yp,dtlambda,ngwpdyn1)

!+----------------------------------------------------------------------+
! Routine to solve for the iCL = Y equations by using a SVD of C. 
! GWR 02/15
!+----------------------------------------------------------------------+

      implicit   none

      integer(long), intent(in)                                  :: ngwpdyn1
      complex(dop), dimension(ngwpdyn1,ngwpdyn1), intent(in)     :: cp
      complex(dop), dimension(ngwpdyn1), intent(in)              :: yp
      complex(dop), dimension(ngwpdyn1), intent(inout)             :: dtlambda

      integer(long)                                              :: i,j,lworkx,&
                                                                    info,idsin

      real(dop), dimension(ngwpdyn1)                             :: sx
      real(dop), dimension(5*ngwpdyn1)                           :: rwork
      complex(dop), dimension(10*ngwpdyn1)                       :: work
      complex(dop), dimension(ngwpdyn1,ngwpdyn1)                 :: umat,vmat,&
                                                                    leftmat
      complex(dop) :: proj
      complex(dop), dimension(ngwpdyn1)  :: ytmp,lamtmp

      routine = 'svdcmat'

! Perform SVD on C      
      leftmat = cp
      lworkx = 10*ngwpdyn1
      call zgesvd('A','A',ngwpdyn1,ngwpdyn1,leftmat,ngwpdyn1,sx,umat,&
           ngwpdyn1,vmat,ngwpdyn1,work,lworkx,rwork,info) ! leftmat = cp

      if(info.ne.0) write(*,*)'C-matrix SVD problem',info

! idsin is the index of the first "zero". This assumes the use of an SVD routine
! which orders sx(i) in decreasing order, the last few being the singular 
! values.
      idsin = ngwpdyn1+1
      do i = 1,ngwpdyn1
         !if(abs(sx(i)).le.svd_fac*sx(1).or.abs(sx(i)).le.1.0e-6_dop)then
         if(abs(sx(i)).le.epsgwp*sx(1).or.abs(sx(i)).le.1.0e-6_dop)then
            idsin = i
            exit
         endif
      enddo

      ytmp = dcmplx(0.0_dop,0.0_dop)
      do i = 1,ngwpdyn1
         do j = 1,ngwpdyn1
            ytmp(i) = ytmp(i) + dconjg(umat(j,i))*yp(j)
         enddo
      enddo

      lamtmp = dcmplx(0.0_dop,0.0_dop)
      do i = 1,idsin-1
         lamtmp(i) = ytmp(i)/sx(i)
      enddo

      dtlambda = dcmplx(0.0_dop,0.0_dop)
      do i = 1,ngwpdyn1
         do j = 1,ngwpdyn1
            dtlambda(i) = dtlambda(i) + dconjg(vmat(j,i))*lamtmp(j)
         enddo
      enddo

! Project out the kernel of C from lambda. Refinement which improves speed.
      if(idsin.le.ngwpdyn1)then
         do i = idsin,ngwpdyn1
            proj = dcmplx(0.0_dop,0.0_dop)
            do j = 1, ngwpdyn1
               proj = proj  + dconjg(vmat(i,j))*dtlambda(j)
            enddo
            do j = 1, ngwpdyn1
               dtlambda(j) = dtlambda(j) - proj*dconjg(vmat(i,j))
            enddo
         enddo
      endif

      return
      
      end subroutine svdcmat

!########################################################################
      subroutine clsq(cp,yp,dtlambda,ngwpdyn1)

      implicit none

      integer(long), intent(in)                              :: ngwpdyn1
      complex(dop), dimension(ngwpdyn1,ngwpdyn1), intent(in) :: cp
      complex(dop), dimension(ngwpdyn1), intent(in)          :: yp
      complex(dop), dimension(ngwpdyn1), intent(inout)         :: dtlambda

      integer(long)                                          :: lwork,rworkdim,&
                                                                info,i,nzero,&
                                                                nredim
      real(dop)                                              :: epseig
      real(dop), dimension(ngwpdyn1)                         :: eigen
      real(dop), allocatable                                 :: rwork(:)
      complex(dop), dimension(ngwpdyn1,ngwpdyn1)             :: eigenvec
      complex(dop), allocatable, dimension(:)                :: cwork,yptrans,&
                                                                dtlamtmp
!      complex(dop), allocatable, dimension(:,:)              :: eigenvred
      complex(dop), dimension(ngwpdyn1)                      :: testvec
      complex(dop)                                           :: func

      if(ngwpdyn1.eq.1)then
         dtlambda(1) = yp(1)/cp(1,1)
         return
      endif

      epseig=1.0e-8_dop
      lwork = max(1,2*ngwpdyn1-1)
      rworkdim = max(1,3*ngwpdyn1-2)
      allocate(cwork(lwork),rwork(rworkdim))

! Diagonalise the C-matrix. Hermitian so eigenvalues are real.
! The eigenvectors would also diagonalise the Hessian, C^{dag}*C.
      eigenvec = cp
      call zheev('V','U',ngwpdyn1,eigenvec,ngwpdyn1,eigen,cwork,lwork,&
           rwork,info)

! Number of zero eigenvalues.
      nzero  = 0
      do 
!         nzero  = 0
!         do i = 1, ngwpdyn1
!            if(abs(eigen(i)).lt.epseig)nzero=nzero+1
!         enddo

! Number of non-zero eigenvectors.
         nredim = ngwpdyn1-nzero

         if(nredim.eq.0)then
            write(ilog,'(a)')'All eigenvalues of C are zero, cannot invert'
            dtlambda = dcmplx(0.0_dop,0.0_dop)
            exit
!            write(ilog,*)'eigen',eigen
!            write(ilog,*)'cp',cp
!            routine = 'clsq'
!            message = 'All eigenvalues of C are zero, cannot invert'           
!            call errormsg
         endif

! Allocate the necessaryre-dimensioned arrays.
!         allocate(eigenvred(ngwpdyn1,nredim),yptrans(nredim),dtlamtmp(nredim))
         allocate(yptrans(ngwpdyn1),dtlamtmp(ngwpdyn1))

! Copy the eigenvectors with non-zero eigenvalues.
!         do i = 1,ngwpdyn1
!            do j = 1,nredim
!               eigenvred(i,j) = eigenvec(i,j+nzero)
!            enddo
!         enddo

! Transform the y-vector.
!         call mvaxzz(eigenvred,yp,yptrans,ngwpdyn1,nredim)
         call mvaxzz(eigenvec,yp,yptrans,ngwpdyn1,ngwpdyn1)

! Find the minimum of the transformed quadratic.
         dtlamtmp = dcmplx(0.0_dop,0.0_dop)
         do i = 1,nredim
!            dtlamtmp(i) = yptrans(i)/eigen(i+nzero)
            dtlamtmp(i+nzero) = yptrans(i+nzero)/eigen(i+nzero)
         enddo

!         do i = 1,nzero
!            dtlamtmp(i) = yp(i)
!         enddo

! Back transform to get the required answer.
!         dtlambda = matmul(eigenvred,dtlamtmp)
         dtlambda = matmul(eigenvec,dtlamtmp)

! Tidy up.
!         deallocate(yptrans,eigenvred,dtlamtmp)
         deallocate(yptrans,dtlamtmp)

         testvec = matmul(cp,dtlambda)-yp
         call vvaxzz(testvec,testvec,func,ngwpdyn1)
         func = 0.5_dop*func
         if(dble(func).le.1.0e-8_dop)exit

         nzero = nzero+1
!         epseig = epseig*10.0_dop
      enddo

      deallocate(cwork,rwork)

      return
      end subroutine clsq

!#######################################################################

      subroutine gwpcmatinvd(cp,yp,dtlambda,ngwpdyn1)

!+-----------------------------------------------------------------------+  
!     calculates time evolution of Gaussian parameters. 
!+-----------------------------------------------------------------------+  

      implicit   none

      integer(long), intent(in)                   :: ngwpdyn1
      integer(long)                               :: ierr,i
      complex(dop), dimension(ngwpdyn1,ngwpdyn1), intent(inout) :: cp
      complex(dop), dimension(ngwpdyn1), intent(in)             :: yp 
      complex(dop), dimension(ngwpdyn1), intent(inout)            :: dtlambda
      integer(long), save                         :: tid=0 ! timer ID

      call get_timer(tid, 'Gwp_Cmatd')
      call start_timer(tid)

      do i=1,ngwpdyn1
         if (abs(cp(i,i)) .lt. tiny) cp(i,i) = tiny
      enddo

! do Cholesky factorization of the matrix
      call zpotrf('L',ngwpdyn1,cp,ngwpdyn1,ierr)
! invert factorized matrix
      call zpotri('L',ngwpdyn1,cp,ngwpdyn1,ierr)
      if (ierr .ne. 0) then
         routine='gwpcmatinvd'
         message='Error inverting matrix'
         call errormsg
      endif

! multiply to get the solution: dtlambda = rightmat * yp
      call zhemv('L',ngwpdyn1,(1.0_dop,0.0_dop),cp,ngwpdyn1,yp,1,&
              (0.d0,0.d0),dtlambda,1)

      call stop_timer(tid)

      return
      end subroutine gwpcmatinvd

!#######################################################################

      subroutine gwpredim(cp,yp,ncpdim,cp1,yp1,gwpdyn,ngwpdyn)

      implicit none

      integer(long) :: i,j,m,n
      integer(long), intent(in)                                 :: ncpdim,&
                                                                   ngwpdyn
      integer(long), dimension(ncpdim), intent(in)              :: gwpdyn
      complex(dop), dimension(ncpdim,ncpdim), intent(in)        :: cp
      complex(dop), dimension(ncpdim), intent(in)               :: yp
      complex(dop), dimension(ngwpdyn,ngwpdyn), intent(inout)     :: cp1
      complex(dop), dimension(ngwpdyn), intent(inout)             :: yp1
      
      if (ncpdim .eq. ngwpdyn) then
         call cpmxz(cp,cp1,ncpdim,ncpdim)
         call cpvxz(yp,yp1,ncpdim)
      else
         m = 0
         do i=1,ncpdim
            if (gwpdyn(i) .ne. 0) cycle
            m = m+1
            n = 0
            yp1(m) = yp(i)
            do j=1,ncpdim
               if (gwpdyn(j) .ne. 0) cycle
               n = n + 1
               cp1(n,m) = cp(j,i)
            enddo
         enddo
      endif

      return
      end subroutine gwpredim

!#######################################################################

      subroutine gwpredim1(dtlambda,ncpdim,dtlambda1,gwpdyn,ngwpdyn)

      implicit none

      integer(long), intent(in)                      :: ncpdim,ngwpdyn
      integer(long), dimension(ncpdim), intent(in)   :: gwpdyn
      integer(long)                                  :: i,m
      complex(dop), dimension(ncpdim), intent(inout)   :: dtlambda
      complex(dop), dimension(ngwpdyn), intent(in)   :: dtlambda1
      
      if (ncpdim .eq. ngwpdyn) then
         call cpvxz(dtlambda1,dtlambda,ncpdim)
      else
         dtlambda = 0.0_dop
         m = 0
         do i=1,ncpdim
            if (gwpdyn(i) .ne. 0) cycle
            m = m+1
            dtlambda(i) = dtlambda1(m)
         enddo
      endif

      return
      end subroutine gwpredim1

!#######################################################################
!
      subroutine gdot(gs2,gsinv,hung,gsdot,psi,dtpsi,dicht2,gwpdyn,&
                 gwpdep,hloch,gh,agmat,ghderiv1)

      implicit   none

      integer(long)                                  :: wzeig,s,m,kz,&
                                                        phidim1,nham
      integer(long), dimension(gwpdyndim), intent(inout) :: gwpdyn
      integer(long), dimension(gwpinfodim), intent(inout) :: gwpdep
      complex(dop), dimension(dgldim), intent(inout) :: psi
      complex(dop), dimension(dgldim),intent(inout)    :: dtpsi
      complex(dop), dimension(hungdim), intent(in)   :: hung
      complex(dop), dimension(gsmatdim), intent(inout) :: gsdot
      complex(dop), dimension(gs2matdim), intent(in) :: gs2
      complex(dop), dimension(agmatdim), intent(in)  :: agmat
      complex(dop), dimension(gsmatdim), intent(in)  :: gsinv
      complex(dop), dimension(dmatdim), intent(in)   :: dicht2
      complex(dop), dimension(ghmatdim), intent(in)  :: gh
      complex(dop), dimension(hlochdim), intent(in)  :: hloch
      real(dop), dimension(ghder1dim), intent(in)    :: ghderiv1
      integer(long), save                            :: tid=0 ! timer ID

      call get_timer(tid, 'Gwp_Gdot')
      call start_timer(tid)


      kz=kzahl(1)
      nham=1
      do s=1,nstate
         do m = 1,nmode
            if(gwpm(m))then
               wzeig=(npg(m)-1)*gwpdim(m,s)+1
               phidim1 = phidim(m)+gbfdim(m)
               call subgdot1(dtpsi(zphi(m,s)),dtpsi(zxi(m,s)),&
                    dtpsi(zpg(m,s)),dtpsi(zpg(m,s)), &
                    gs2(gs2mat(m,s)),gsinv(gsmat(m,s)),&
                    npg(m),nspfdof(m),gwpdim(m,s),dim(m,s),m,s,&
                    hung(hungmat(m,s,1)),dicht2(dmat(m,s)),&
                    gwpdyn(zgwpdyn(m,s)),ngwpdyn(m,s),&
                    psi(zetf(m,1)),zetf(m,1),phidim1,hloch,gh,&
                    agmat(zagmat(m,s)),ghderiv1,&
                    gsdot(gsmat(m,s)),psi(zdgwp(m,s)),&
                    gwpdep(zgwpinfo(m,s)),ngwpdep(m,s))
            endif
         enddo
      enddo

      call stop_timer(tid)

! ... note: we need to multiply by (-i): this is done by routine 
!    'prefactor' at the end of 'func' routine !

      return
      end subroutine gdot

!#######################################################################
!
! passing routine for gdot1 for a single mode (required by funkphi)
!
!#######################################################################

      subroutine gdot1ms(m,s,psi,dtpsi,zetf1,phidim1,gs2,gsinv,hung,&
                 gsdot,dicht2,gwpdyn,gwpdep,hloch,gh,agmat,ghderiv1)

      implicit   none

      integer(long)                                              :: kz
      integer(long), intent(in)                                  :: s,m,zetf1,&
                                                                    phidim1
      integer(long), dimension(gwpdyndim), intent(inout)         :: gwpdyn
      integer(long), dimension(gwpinfodim), intent(inout)        :: gwpdep
      complex(dop), dimension(zetf1:zetf1+phidim1-1), intent(inout) :: psi
      complex(dop), dimension(zetf1:zetf1+phidim1-1),intent(inout) :: dtpsi
      complex(dop), dimension(hungdim), intent(in)               :: hung
      complex(dop), dimension(gsmatdim), intent(inout)           :: gsdot
      complex(dop), dimension(gs2matdim), intent(in)             :: gs2
      complex(dop), dimension(gsmatdim), intent(in)              :: gsinv
      complex(dop), dimension(agmatdim), intent(in)              :: agmat
      complex(dop), dimension(dmatdim), intent(in)               :: dicht2
      complex(dop), dimension(ghmatdim), intent(in)              :: gh
      complex(dop), dimension(hlochdim), intent(in)              :: hloch
      real(dop), dimension(ghder1dim), intent(in)                :: ghderiv1

      kz=kzahl(1)
      call subgdot1(dtpsi(zphi(m,s)),dtpsi(zxi(m,s)),&
                    dtpsi(zpg(m,s)),dtpsi(zpg(m,s)), &
                    gs2(gs2mat(m,s)),gsinv(gsmat(m,s)),&
                    npg(m),nspfdof(m),gwpdim(m,s),dim(m,s),m,s,&
                    hung(hungmat(m,s,1)),dicht2(dmat(m,s)),&
                    gwpdyn(zgwpdyn(m,s)),ngwpdyn(m,s),&
                    psi(zetf1),zetf1,phidim1,hloch,gh,&
                    agmat(zagmat(m,s)),ghderiv1,&
                    gsdot(gsmat(m,s)),psi(zdgwp(m,s)),&
                    gwpdep(zgwpinfo(m,s)),ngwpdep(m,s))

! ... note: we need to multiply by (-i): this is done by routine 
!    'prefactor' at the end of 'func' routine !

      return
      end subroutine gdot1ms


!#######################################################################

      subroutine xivectopq(gdof,gwpdim1,agmat,xivec,xi)

implicit none

      integer(long)                                        :: ig,i
      integer(long), intent(in)                            :: gdof,gwpdim1
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(in)  :: agmat
      complex(dop), dimension(gdof,gwpdim1), intent(in)    :: xivec
      complex(dop), dimension(gdof,gwpdim1)                :: xivectmp
      complex(dop), dimension(gdof,gwpdim1), intent(inout)   :: xi
      real(dop), dimension(gdof)                           :: p,q

      if(gwptype .le. 1 .or. gwptype .eq. 4)then
         routine = 'xivectopq'
         message = 'Not implemented for this type of GWP'
         call errormsg
      endif

      xivectmp = dcmplx(0.0_dop,1.0_dop)*xivec
      do ig = 1,gwpdim1
         do i = 1,gdof
            p(i) = dimag(xivectmp(i,ig))
            q(i) = dble(xivectmp(i,ig))
            q(i) = -0.5_dop*q(i)/dble(agmat(i,i,ig))
            xi(i,ig) = dcmplx(q(i),p(i))
         enddo
      enddo
     
      return
      end subroutine xivectopq

!#######################################################################

      subroutine subgdot1(phidot,xidot,agdot,addot,gs2,gsinv,npg1,gdof,&
                 gwpdim1,dim1,m,s,hung,dicht2,gwpdyn1,ngwpdyn1,psi,&
                 zetf1,phidim1,hloch,gh,agmat,ghderiv1,gsdot,dgwp,gwpdep1,&
                 ngwpdep1)

!+-----------------------------------------------------------------------+  
!     calculates time evolution of Gaussian parameters. 
!+-----------------------------------------------------------------------+  

      implicit   none

      character(len=1)                     ::equed
      character(len=2)                     ::cdigits

      integer(long)                        :: p1,j,lwork,n,ierr,ncpdim,ig,jg
      integer(long), dimension(:), allocatable       :: ipiv
      integer(long), intent(in)            :: s,m,npg1,gwpdim1,gdof,dim1
      integer(long), dimension(npg1-1,gwpdim1), intent(inout) :: gwpdyn1
      integer(long), dimension(gwpdim1), intent(in)  :: gwpdep1
      integer(long), intent(inout)                   :: ngwpdyn1
      integer(long), intent(in)                      :: ngwpdep1
      integer(long), intent(in)                      :: zetf1,phidim1

      real(dop)                                      :: rcond,ferr,berr
      real(dop), dimension(:), allocatable           :: workr,scal
      real(dop), dimension(ghder1dim), intent(in)    :: ghderiv1

      complex(dop), dimension(gwpdim1), intent(inout)           :: phidot
      complex(dop), dimension(gwpdim1)                        :: tmpphidot
      complex(dop), dimension(gdof,gwpdim1), intent(inout)      :: addot,xidot
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(inout) :: agdot
      complex(dop), dimension(:), allocatable                 :: workc

      complex(dop), dimension(gwpdim1,gwpdim1), intent(in)    :: gsinv
      complex(dop), dimension(dim1,dim1), intent(in)          :: dicht2
      complex(dop), dimension(npg1,gwpdim1)                   :: xp
      complex(dop), dimension(gwpdim1,gwpdim1), intent(in)    :: hung
      complex(dop), dimension(gwpdim1,gwpdim1,npg1,npg1), intent(in) :: gs2
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(in)  :: agmat
      complex(dop), dimension(ghmatdim), intent(in)           :: gh
      complex(dop), dimension(hlochdim), intent(in)           :: hloch
      complex(dop), dimension(npg1-1,gwpdim1,npg1-1,gwpdim1)  :: cp
      complex(dop), dimension(npg1-1,gwpdim1)                 :: yp
      complex(dop), dimension(npg1-1,gwpdim1)                 :: dtlambda

      complex(dop), dimension(zetf1:zetf1+phidim1-1), intent(inout) :: psi
      complex(dop), dimension(gwpdim1,gwpdim1), intent(inout)      :: gsdot
      complex(dop), dimension(gwpdim1,dim1), intent(in)          :: dgwp
      complex(dop), dimension(:), allocatable               :: yp1,dtlambda1
      complex(dop), dimension(:,:), allocatable             :: cp1,cp1fac
      integer(long)          :: tid=0

      integer(long) :: i,isign(2,npg1-1,gwpdim1)
      real(dop) :: pt2,qt2,qteps,dtmp,vec1(2,npg1-1,gwpdim1),vec2(2,npg1-1)
      logical :: found(gwpdim1)

! Timing
      call get_timer(tid,'subgdot1')
      call start_timer(tid)

!-----------------------------------------------------------------------
! calculate dimension of C matrix. 
!-----------------------------------------------------------------------
      ncpdim = (npg1-1)*gwpdim1
      dtlambda=(0.0_dop,0.0_dop)
      xp = 0.0_dop

!-----------------------------------------------------------------------
! set up derivatives for GWP EOMs using CX
!-----------------------------------------------------------------------
      if (gwpcxm(m) .gt. 0) then
         call gwpderiv1(m,s,ghderiv1,hloch,dicht2,xp,nspfdof(m),&
              gwpdim1,npg1,dgwp,dim(m,s))
      endif

!-----------------------------------------------------------------------
! If classical GWPs 
! or 
! no time-dependent GWPs, do not calculate C^-1 Y
!-----------------------------------------------------------------------
      if (ngwpdyn1 .ne. 0) then

!-----------------------------------------------------------------------
! Calculate C matrix
!-----------------------------------------------------------------------
         call gwpcmat(cp,dicht2,gs2,gsinv,npg1,gwpdim1,dim1,dgwp,psitype)

!-----------------------------------------------------------------------
! Calculate Y matrix
!-----------------------------------------------------------------------
         call gwpymat(yp,dicht2,hloch,gh,hung,xp,npg1,gwpdim1,dim1, &
                      m,s,gs2,gsinv,psi(zetf1),zetf1,phidim(m))

!-----------------------------------------------------------------------
! Remove time-independent functions from matrices
!-----------------------------------------------------------------------
         allocate(yp1(ngwpdyn1))
         allocate(cp1(ngwpdyn1,ngwpdyn1))
         allocate(dtlambda1(ngwpdyn1))
         call gwpredim(cp,yp,ncpdim,cp1,yp1,gwpdyn1,ngwpdyn1)

!-----------------------------------------------------------------------
! solving dtlambda = Inv[cp] * yp. 
!-----------------------------------------------------------------------
         if (gwpeoms .eq. 0 .or. gwpeoms .eq. 9) then
! calculate dtlambda = Inv[cp]*yp using regularized Inv[c1] from eigenvalues
            call gwpcmatinv(cp1,yp1,dtlambda1,ngwpdyn1)

         else if (gwpeoms.eq.1 .or. gwpeoms.eq.2) then
! solve cp*dtlambda = yp with respect to dtlambda directly by LU decomposition
            dtlambda1=yp1
            allocate(ipiv(ngwpdyn1))
            call zgesv(ngwpdyn1,1,cp1,ngwpdyn1,ipiv,dtlambda1,ngwpdyn1,ierr)
            if (ierr .ne. 0) then
               routine = 'SUBGDOT1'
               message = 'C-matrix singular: LU'
               call errormsg
            endif
            deallocate(ipiv)

         else if (gwpeoms .eq. 3) then
! solve cp*dtlambda = yp with respect to dtlambda directly with the
! Conjugate-Gradient algorithm
            lwork=8*ngwpdyn1+2
            allocate(workc(lwork))
            call lineq_cgz(ngwpdyn1,ngwpdyn1,cp1,dtlambda1,yp1, &
                 workc,lwork,ilog)
            deallocate(workc)

         else if (gwpeoms .eq. 4) then
! calculate dtlambda = Inv[cp]*yp using direct inversion of cp1
            call gwpcmatinvd(cp1,yp1,dtlambda1,ngwpdyn1)

         else if (gwpeoms .eq. 5) then
! calculate dtlambda = Inv[cp]*yp using Tikhonov regularization of Inv[cp]
            call tikhonov(cp1,yp1,dtlambda1,ngwpdyn1)

         else if (gwpeoms .eq. 6) then
! SVD decomposition of cp matrix
            call svdcmat(cp1,yp1,dtlambda1,ngwpdyn1)

         else if (gwpeoms .eq. 7) then
! solve cp*dtlambda = yp with respect to dtlambda directly by 
! Cholesky decomposition. The system is equilibrated if necessary 
!prior to solving the equations
            allocate(cp1fac(ngwpdyn1,ngwpdyn1),workc(2*ngwpdyn1),&
                     workr(ngwpdyn1),scal(ngwpdyn1))
            call zposvx('E','U',ngwpdyn1,1,cp1,ngwpdyn1,cp1fac,ngwpdyn1,&
                 equed,scal,yp1,ngwpdyn1,dtlambda1,ngwpdyn1,rcond,ferr,&
                 berr,workc,workr,ierr)
            if (ierr .lt. 0) then
               routine = 'SUBGDOT1'
               message = 'Input problem in Cholesky decomposition of C-matrix'
               call errormsg
            else if (ierr.gt.0 .and. ierr.le.ngwpdyn1) then
               routine = 'SUBGDOT1'
               message = &
                   'C-matrix not positive definite in Cholesky decomposition'
               call errormsg
            else if (ierr.gt.ngwpdyn1) then
               routine = 'SUBGDOT1'
               message = &
                    'C-matrix is ill conditioned and solution is dominated &
                          &by numerical errors'
               call errormsg
            else if (rcond .lt. 1.d-6) then
               write(cdigits,'(I2)') nint(-log10(rcond))
               routine = 'SUBGDOT1'
               message = 'C-matrix is ill conditioned, &
                   &expect numerical errors of about '//cdigits//' digits.'
               call warnmsg
            endif
            deallocate(cp1fac,workc,workr,scal)

         else if (gwpeoms .eq. 8) then
! Transformation of |cp*dtlambda-yp|**2 and direct solution       
            call clsq(cp1,yp1,dtlambda1,ngwpdyn1)
         endif

!-----------------------------------------------------------------------+
! Add time-independent functions back in
!-----------------------------------------------------------------------+
         call gwpredim1(dtlambda,ncpdim,dtlambda1,gwpdyn1,ngwpdyn1)
         if (allocated(yp1)) deallocate(yp1)
         if (allocated(cp1)) deallocate(cp1)
         if (allocated(dtlambda1)) deallocate(dtlambda1)

      endif !end if on coupled GWPs

!-----------------------------------------------------------------------+
! now add zero order part
! gwpdyn(n,j) = 0 flags a correlated DOF
!
! gwpcx = 0 no CX decomposition has been used
!       > 0 CX decomposition has been used and 
!           1 classical part added to all functions
!             For single-set the "Ehrenfest force" must be added to 
!             classical functions 
!           2 classical part added to active functions only
!-----------------------------------------------------------------------+
      if (gwpcxm(m) .eq. 1) then
         do j=1,gwpdim1
            do n=1,npg1-1
               dtlambda(n,j) = dtlambda(n,j) + xp(n+1,j)
            enddo
         enddo
      else if (gwpcxm(m) .eq. 2) then
         do j=1,gwpdim1
            do n=1,npg1-1
               if (gwpdyn1(n,j) .eq. 0) then
                  dtlambda(n,j) = dtlambda(n,j) + xp(n+1,j)
               endif
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! make functions in CAPs static
!-----------------------------------------------------------------------
      if (lcap) then
         do j=1,gwpdim1
            if (any(gwpdyn1(:,j) .eq. -2)) then
               dtlambda(:,j) = 0.0_dop
               call setpzero(psi(zxi(m,s)),gwpdim1,nspfdof(m),j) 
            endif
         enddo
      endif

!-----------------------------------------------------------------------
! make linearly dependent functions static
!-----------------------------------------------------------------------
      if (ngwpdep1 .gt. 0) then
         do j=1,gwpdim1
            if (gwpdep1(j) .gt. 0) dtlambda(:,j) = 0.0_dop
         enddo
      endif

!-----------------------------------------------------------------------
!symmetry?
!-----------------------------------------------------------------------
      if (ltrjsym) then
         qteps = 1.0d-10
         found(:) = .false.
         isign(:,:,:) = 1
         vec1(:,:,:) = 0.0_dop

         do j=1,gwpdim1
! divide dtlambda in real and imaginary
            found(j) = .true.
            do n=1,npg1-1
               dtmp = dble(dtlambda(n,j))
               if (dtmp .lt. 0.0_dop) isign(1,n,j)=-1
               vec1(1,n,j) = dabs(dtmp)
               if (vec1(1,n,j) .gt. qteps) found(j) = .false.
               dtmp = dimag(dtlambda(n,j))
               if (dtmp .lt. 0.0_dop) isign(2,n,j)=-1
               vec1(2,n,j) = dabs(dtmp)
               if (vec1(2,n,j) .gt. qteps) found(j) = .false.
            enddo
         enddo

         do j=1,gwpdim1-1
! do not check zero vectors
            if (found(j)) cycle

            do i=j+1,gwpdim1
               if (found(i)) cycle

! look for symmetrical match
               do n=1,npg1-1
                  vec2(1,n) = dabs(vec1(1,n,j) - vec1(1,n,i))
                  vec2(2,n) = dabs(vec1(2,n,j) - vec1(2,n,i))
               enddo
               found(i) = .true.
               do n=1,npg1-1
                  if (vec2(1,n) .gt. qteps) found(i) = .false.
                  if (vec2(2,n) .gt. qteps) found(i) = .false.
               enddo
               if (.not. found(i)) cycle
           
               do n=1,npg1-1
                  qt2=vec1(1,n,j)
                  if (isign(1,n,i) .lt. 0) qt2 = -qt2
                  pt2=vec1(2,n,j)
                  if (isign(2,n,i) .lt. 0) pt2 = -pt2

                  dtlambda(n,i)=dcmplx(qt2,pt2)
              enddo
           enddo
        enddo
     endif

!+-----------------------------------------------------------------------+
! assign dtlambda to xidot and adot 
! and transform linear parameter derivative from B to q+ip
!+-----------------------------------------------------------------------+
      call bdot2pqdot(xidot,agdot,addot,nspfdof(m),gwpdim1,npg1,dtlambda, &
           agmat,psi(zxi(m,s)))

!-----------------------------------------------------------------------+
! fix phase
! gwpphase=1 Normalised
! gwpphase=0 Fixed
! gwpphase=2 "Action"
!
! gwpphase = -1 : phase not propagated but is needed for gsdot. Hence
!                 use local array tmpphidot and save as phidot if required.
!-----------------------------------------------------------------------+
      if (gwpphase .eq. 0) then
         call zerovxz(tmpphidot,gwpdim1)

      else if (abs(gwpphase) .eq. 1) then
         call calcphidot(tmpphidot,gs2,dtlambda,gwpdim1,npg1,gdof,&
              psi(zxi(m,s)),xidot)

      else if (gwpphase .eq. 2) then
         call calcphidot(tmpphidot,gs2,dtlambda,gwpdim1,npg1,gdof,&
              psi(zxi(m,s)),xidot)
         do j=1,gwpdim1
            tmpphidot(j)=tmpphidot(j) + hung(j,j)
         enddo

      else if (gwpphase .eq. 3) then
         do j=1,gwpdim1
            tmpphidot(j)=xp(1,j)
         enddo

      endif

      if (gwpphase .ge. 0) phidot=tmpphidot

!-----------------------------------------------------------------------
! calculate matrix gsdot =  i<G | Gdot>
!     if lrlx      gsdot = -<G|Gdot>
!-----------------------------------------------------------------------
      gsdot=0.0_dop
      do ig=1,gwpdim1
         do jg=1,gwpdim1
            gsdot(jg,ig)=gs2(jg,ig,1,1)*tmpphidot(ig)
            do p1=2,npg1
               gsdot(jg,ig) = gsdot(jg,ig) + gs2(jg,ig,1,p1) * dtlambda(p1-1,ig)
            enddo
         enddo
      enddo

      call stop_timer(tid)

      return
      end subroutine subgdot1

!#######################################################################
!
! xp(npg,gwpdim) : only npg = 2,gdof+1 filled with derivatives, 
!                  irrespective of GWP type
!
!#######################################################################

      subroutine gwpderiv1(m,s,ghderiv1,hloch,dicht2,xp,gdof,gwpdim1,&
                 npg1,dgwp,dim1)

      implicit none

      integer(long), intent(in)                            :: m,s,gwpdim1,&
                                                              npg1,gdof,dim1
      integer(long)                                        :: k,n,n1,k3
      integer(long)                                        :: f,h
      real(dop),dimension(ghder1dim), intent(in)           :: ghderiv1
      complex(dop), dimension(npg1,gwpdim1), intent(out)   :: xp
      complex(dop), dimension(hlochdim), intent(in)        :: hloch
      complex(dop), dimension(gwpdim1,gwpdim1), intent(in) :: dicht2
      complex(dop), dimension(gwpdim1,dim1), intent(in)    :: dgwp
      complex(dop), dimension(gwpdim1,gwpdim1)             :: gdicht2,&
                                                              ghloch
      logical(kind=4), dimension(maxkoe)                   :: kignore 
      logical(kind=4) :: luncorr

      xp = 0.0_dop
      k3=kzahl(1)+nmulpot(1)   ! flag for uncorrelated terms

!-----------------------------------------------------------------------
! Work out terms to ignore
!-----------------------------------------------------------------------
      kignore(:) = .false.
      do k=1,khtot(1)
! only include uncorrelated and zero-order correlated terms
! for on-diagonal states.
         kignore(k) = (ki(k) .ne. s)  ! wrong state packet
         if (kignore(k)) cycle
         kignore(k) = (ksi(k) .ne. ksf(k)) ! not diagonal state operator
         if (kignore(k)) cycle
         kignore(k) = (klha(k) .gt. 1)    ! high order correlated LHA, 
         if (kignore(k)) cycle
         kignore(k) = (klha(k) .lt. 0)    ! ignored correlated LHA, 
      enddo

      do k=1,khtot(1)

         if (diag(m,k)) cycle
         if (kignore(k)) cycle
         luncorr = (k .gt. k3) ! uncorrelated term

!-----------------------------------------------------------------------
! transform from SPF to GWP basis
!-----------------------------------------------------------------------
         if (.not. luncorr) then
            if (psitype .eq. 0 .or. psitype .eq. 5) then
               call simtran1btz(dicht2,dgwp,gdicht2,dim1,gwpdim1)
               call simtran1btz(hloch(phloch(m,k)),dgwp,ghloch,dim1,gwpdim1)
            else
               gdicht2(:,:)=dicht2(:,:)
               call cpqxz(hloch(phloch(m,k)),ghloch,gwpdim1)
            endif
         endif

! find DOF with operator information. For KE or flux term this is
! in n rather than 1. For muld operators this is in fterm(k).
         if (fterm(k) .gt. 0) then
            f = fterm(k)
         else
            n=1
            do n1=1,nspfdof(m)
               f=spfdof(n1,m)
               if (hamilton(f,k) .ne. f) n=n1
            enddo
            f = spfdof(n,m)
         endif
         h = hamilton(f,k)

         if (ghder1(k,m) .gt. 0) then
            call gwpderiv1k(h,luncorr,klha(k),koeff(k),ghderiv1(ghder1(k,m)),&
                 ghloch,gdicht2,xp,gdof,gwpdim1,npg1)
         endif
        
      enddo

      return
      end subroutine gwpderiv1

!#######################################################################

      subroutine gwpderiv1k(h,luncorr,klha1,koeff1,ghderiv1,hloch,&
                 dicht2,xp,gdof,gwpdim1,npg1)

      implicit none

      integer(long), intent(in)                            :: h,gwpdim1,&
                                                              npg1,gdof,&
                                                              klha1
      integer(long)                                        :: ig,n
      real(dop), dimension(gwpdim1)                        :: wght
      real(dop),dimension(gdof,gwpdim1), intent(in)        :: ghderiv1
      complex(dop), intent(in)                             :: koeff1
      complex(dop), dimension(npg1,gwpdim1), intent(out)   :: xp
      complex(dop), dimension(gwpdim1,gwpdim1), intent(in) :: dicht2,hloch
      logical(kind=4), intent(in)                          :: luncorr

!-----------------------------------------------------------------------
! Multiply derivative by hloch/dicht2
! If klha1=1 derivative is from a klha term and already includes 
!    hloch
! Uncorrelated terms, hloch = dicht2
!-----------------------------------------------------------------------
      if (klha1 .eq. 1) then ! klha derivative
         do ig=1,gwpdim1
            wght(ig) = dble(dicht2(ig,ig))
            if (wght(ig) .lt. tiny) &
                  wght(ig)=wght(ig)+epsmat*exp(-wght(ig)/epsmat)
            wght(ig) = 1.0_dop/wght(ig)
         enddo
      else if (luncorr) then ! uncorrelated term
         wght(:)=dble(koeff1)
      else
         do ig=1,gwpdim1
            wght(ig) = dble(dicht2(ig,ig))
            if (wght(ig) .lt. tiny) &
                  wght(ig)=wght(ig)+epsmat*exp(-wght(ig)/epsmat)
            wght(ig) = dble(hloch(ig,ig))/wght(ig)
         enddo
      endif

      if (htmtype(h) .eq. 102) then
         do ig=1,gwpdim1
            do n=1,gdof
               xp(n+1,ig) = xp(n+1,ig) + (0,1)*wght(ig)*ghderiv1(n,ig)
            enddo
         enddo
      else
         do ig=1,gwpdim1
            do n=1,gdof
               xp(n+1,ig) = xp(n+1,ig) + wght(ig)*ghderiv1(n,ig)
            enddo
         enddo
      endif

      return
      end subroutine gwpderiv1k

!#######################################################################

      subroutine setpzero(xi,gwpdim1,gdof,j) 

      implicit none

      integer(long), intent(in) :: gwpdim1,gdof,j
      complex(dop), intent(inout) :: xi(gdof,gwpdim1)

      xi(:,j) = dble(xi(:,j))

      return
      end subroutine setpzero

!#######################################################################

      subroutine gdottrj(time,phidot,gs2,gsdot,agmat,hung,xi,gdof,&
                 gwpdim1,npg1,gh)

!+-----------------------------------------------------------------------+  
!     calculates time evolution of Gaussian parameters. 
!+-----------------------------------------------------------------------+  

      implicit   none

      integer(long), intent(in)            :: gdof,gwpdim1,npg1
      integer(long)                        :: g,n,k
      real(dop), intent(in)                :: time
      real(dop)                            :: tmp,tmp1,dt,tp,tm,qp,qm,pp,pm
      real(dop), dimension(gdof,gwpdim1)   :: dq,dp

      complex(dop), dimension(gwpdim1), intent(inout)           :: phidot
      complex(dop), dimension(gwpdim1)                        :: tmpphidot
      complex(dop), dimension(gdof,gwpdim1), intent(in)       :: xi
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(in)  :: agmat
      complex(dop), dimension(gdof,gwpdim1)                   :: dtlambda
      complex(dop), dimension(gdof,gwpdim1)                   :: xidot
      complex(dop), dimension(gwpdim1,gwpdim1), intent(in)    :: hung

      integer(long)                        :: ig,jg,p1
      complex(dop), dimension(gwpdim1,gwpdim1,npg1,npg1), intent(in) :: gs2
      complex(dop), dimension(*), intent(in) :: gh
      complex(dop), dimension(gwpdim1,gwpdim1), intent(inout)      :: gsdot
      complex(dop), dimension(gwpdim1,gwpdim1)                   :: tmpetau

!-----------------------------------------------------------------------
! numerical derivatives of x and p
!-----------------------------------------------------------------------
      do n=1,gwpdim1
         do g=1,gdof
! get step value
            tmp = 1/3.0d0
            tmp1 = macheps**tmp
            tmp = tmp1*time
            dt = max(tmp,tmp1)

! force step value to be accurately represented
            tmp = time + dt
            dt = tmp - time

            tp  = time + dt
            tm  = time - dt
            call splint(gwptrjtdat(1,n),gwptrjxdat(1,g,n), &
                        gwptrjxspl(1,g,n),gwptrjnp(n),tp,qp)
            call splint(gwptrjtdat(1,n),gwptrjxdat(1,g,n), &
                        gwptrjxspl(1,g,n),gwptrjnp(n),tm,qm)
            call splint(gwptrjtdat(1,n),gwptrjpdat(1,g,n), &
                        gwptrjpspl(1,g,n),gwptrjnp(n),tp,pp)
            call splint(gwptrjtdat(1,n),gwptrjpdat(1,g,n), &
                        gwptrjpspl(1,g,n),gwptrjnp(n),tm,pm)
            dq(g,n) = (qp - qm)/(2.0d0*dt)
            dp(g,n) = (pp - pm)/(2.0d0*dt)
         enddo
      enddo

!-----------------------------------------------------------------------
! Restore or calculate time-derivative of xi to be used in calculation
! of phase and gsdot
!
! if CX, Get Bdot from qdot + i pdot (dtlambda overwritten)
! else restore
!-----------------------------------------------------------------------
      do n=1,gwpdim1
         do g=1,gdof
            dtlambda(g,n) = -dp(g,n)-2.0_dop*(0,1)*agmat(g,g,n)*dq(g,n) 
            xidot(g,n) = -dp(g,n) + (0,1)*dq(g,n)
         enddo
      enddo

!-----------------------------------------------------------------------+
! gwpphase = -1 : phase not propagated but is needed for gsdot. Hence
!                 use local array tmpphidot and save as phidot if required.
!-----------------------------------------------------------------------+
      tmpphidot=0
      call calcphidot(tmpphidot,gs2,dtlambda,gwpdim1,npg1,gdof,xi,xidot)
      if (gwpphase .eq. 2) then
         do n=1,gwpdim1
            tmp = dimag(tmpphidot(n))
            tmpphidot(n) = (0,1)*tmp
            phidot(n)=tmpphidot(n) + hung(n,n)
         enddo
         phidot = tmpphidot
      endif

!-----------------------------------------------------------------------
! calculate matrix gsdot =  i<G | Gdot>
!-----------------------------------------------------------------------
      gsdot=0.0_dop
      do ig=1,gwpdim1
         do jg=1,gwpdim1
            gsdot(jg,ig)=gs2(jg,ig,1,1)*tmpphidot(ig)
            do p1=2,npg1
               gsdot(jg,ig) = gsdot(jg,ig) + gs2(jg,ig,1,p1) * dtlambda(p1-1,ig)
            enddo
         enddo
      enddo

      tmpetau = 0.0
      do k=1,khtot(1)
         if (kinop(1,k) .eq. 0) call addmxxzo(gh(ghmat(k,1)),tmpetau,gwpdim1,gwpdim1)
      enddo

!-----------------------------------------------------------------------
! multiply gsdot = tau*Sel + S*tau_el
!-----------------------------------------------------------------------
      do ig=1,gwpdim1
         do jg=1,gwpdim1
            gsdot(jg,ig) = gsdot(jg,ig) * gwptrjeovl(jg,ig) 
            gsdot(jg,ig) = gsdot(jg,ig) + gs2(jg,ig,1,1) * gwptrjetau(jg,ig) 
            !gsdot(jg,ig) = gsdot(jg,ig) + tmpetau(jg,ig) 
         enddo
      enddo

      return
      end subroutine gdottrj

!#######################################################################
! calculate derivative of DGWP
!#######################################################################

      subroutine dgwpdot(dtpsi,psi,gsinv,gsdot,hteil,gs2)

      implicit   none

      integer(long)                                  :: s,m,kz
      complex(dop), dimension(dgldim), intent(in)    :: psi
      complex(dop), dimension(dgldim),intent(inout)  :: dtpsi
      complex(dop), dimension(gsmatdim), intent(in)  :: gsdot
      complex(dop), dimension(gsmatdim), intent(in)  :: gsinv
      complex(dop), dimension(gs2matdim), intent(in) :: gs2
      complex(dop), dimension(pmatdim), intent(in)   :: hteil
      integer(long), save                            :: tid=0 ! timer ID

      call get_timer(tid, 'Gwp_Gdot')
      call start_timer(tid)

      kz=kzahl(1)
      do s=1,nstate
         do m = 1,nmode
            if (.not. gwpm(m)) cycle
            call subdgwpdot(dtpsi(zdgwp(m,s)),psi(zdgwp(m,s)),&
                 gsinv(gsmat(m,s)),gsdot(gsmat(m,s)),hteil(pmat(m,kz+s)),&
                 gs2(gs2mat(m,s)),gwpdim(m,s),dim(m,s),m,s)
         enddo
      enddo

      call stop_timer(tid)

      return
      end subroutine dgwpdot

!#######################################################################

      subroutine dgwpdot1ms(m,s,dtpsi,psi,gsinv,gsdot,hteil,gs2,zetf1,phidim1)

      implicit   none

      integer(long)                                  :: kz
      integer(long), intent(in)                      :: m,s,zetf1,phidim1
      complex(dop), dimension(zetf1:zetf1+phidim1-1), intent(in)   :: psi
      complex(dop), dimension(zetf1:zetf1+phidim1-1),intent(inout) :: dtpsi
      complex(dop), dimension(gsmatdim), intent(in)  :: gsdot
      complex(dop), dimension(gsmatdim), intent(in)  :: gsinv
      complex(dop), dimension(gs2matdim), intent(in) :: gs2
      complex(dop), dimension(pmatdim), intent(in)   :: hteil
      integer(long), save                            :: tid=0 ! timer ID

      call get_timer(tid, 'Gwp_Gdot1')
      call start_timer(tid)

      kz=kzahl(1)
      call subdgwpdot(dtpsi(zdgwp(m,s)),psi(zdgwp(m,s)),&
           gsinv(gsmat(m,s)),gsdot(gsmat(m,s)),hteil(pmat(m,kz+s)),&
           gs2(gs2mat(m,s)),gwpdim(m,s),dim(m,s),m,s)

      call stop_timer(tid)

      return
      end subroutine dgwpdot1ms

!#######################################################################
!
! hunteil is uncorrelated energy in SPF basis
!
!#######################################################################

      subroutine subdgwpdot(dtdgwp,dgwp,gsinv,gsdot,hunteil,gs2,&
                 gwpdim1,dim1,m,s)

      implicit none

      integer(long), intent(in)                            :: gwpdim1,dim1,m,s
      complex(dop), dimension(gwpdim1,dim1), intent(inout) :: dtdgwp
      complex(dop), dimension(gwpdim1,dim1), intent(in)    :: dgwp
      complex(dop), dimension(gwpdim1,gwpdim1), intent(in) :: gs2
      complex(dop), dimension(dim1,dim1), intent(in)       :: hunteil
      complex(dop), dimension(gwpdim1,gwpdim1), intent(in) :: gsinv,gsdot
      complex(dop), dimension(gwpdim1,gwpdim1)             :: dgsinv, tmpdgwp

      integer(long)         :: i,j,k,l,nzero

      dtdgwp=0.0_dop
! Get orthonormal vectors that transform S to unit matrix. Do not use propagated D
! matrices due to numerical erros
      if (lcysinv) then
         dgsinv = gsinv
      else
         call unitqxz(tmpdgwp,gwpdim1)
         call gwporthonorm(tmpdgwp,gs2,gwpdim1,nzero,tiny)
         call qqxazz(tmpdgwp,tmpdgwp,dgsinv,gwpdim1)
      endif

! SPF basis is complete for GWPs i.e. <g| (1-P) |g> = 0
      do i=1,dim1
         do j=1,gwpdim1
            do k=1,gwpdim1
               do l=1,gwpdim1
                    dtdgwp(j,i) = dtdgwp(j,i) - &
                         dgsinv(j,k)*gsdot(k,l)*dgwp(l,i)
               enddo
            enddo
         enddo
      enddo

      if (lfunkl .and. khzahl(m,s,1) .gt. 0) then
         do i=1,dim1
            do j=1,gwpdim1
               do k=1,dim1
                  dtdgwp(j,i) = dtdgwp(j,i) + dgwp(j,k)*hunteil(k,i)
               enddo
            enddo
         enddo
      endif

      return
      end subroutine subdgwpdot

!#######################################################################

      subroutine bdot2pqdot(xidot,agdot,addot,gdof,gwpdim1,npg1,dtlambda,&
                 agmat,xi)

!+-----------------------------------------------------------------------+
!     calculates center coordinate q0(ndof) for Gaussian labeled ig
!     (appearing on the r.h.s. of a given matrix element), using the
!     relation xi = - 2 a.q + ip , i.e., q = -1/2 (Re a)^-1 Re xi
!+-----------------------------------------------------------------------+

      implicit none

      integer(long)                          :: info,i,j,ig,lwork,&
                                                g,g1,g2,counter
      integer(long), intent(in)              :: gdof,gwpdim1,npg1

      real(dop), dimension(3*gdof)                        :: work
      real(dop), dimension(gdof)                          :: tmp
      real(dop), dimension(gdof)                          :: qdot,pdot
      real(dop), dimension(gdof,gdof)                     :: ai,ad
      real(dop), dimension(gdof,gdof)                     :: ainv
      real(dop), dimension(gdof)                          :: aval

      complex(dop), dimension(npg1-1,gwpdim1), intent(in)    :: dtlambda
      complex(dop), dimension(gdof,gdof,gwpdim1), intent(in) :: agmat
      complex(dop), dimension(gdof,gdof,gwpdim1),intent(inout) :: agdot
      complex(dop), dimension(gdof,gwpdim1), intent(inout)     :: addot
      complex(dop), dimension(gdof,gwpdim1), intent(inout)     :: xidot
      complex(dop), dimension(gdof,gwpdim1)                  :: xi
      
!+-----------------------------------------------------------------------+
! For gwptype = 3 Frozen GWP, assume real diagonal agr matrix.
! For thawed GWP obtain the inverse of Re agr, via routines
!   dgetrf.f + dgetri.f  :
!+-----------------------------------------------------------------------+
      if (gwptype .eq. 3) then
         do ig=1,gwpdim1
            do i=1,gdof
               ainv(i,i) = 1.0_dop/(-2.0_dop*dble(agmat(i,i,ig)))
               pdot(i) = -1.0_dop*dble(dtlambda(i,ig))
               qdot(i) = ainv(i,i)*dimag(dtlambda(i,ig))

               xidot(i,ig) = dcmplx(-pdot(i),qdot(i))

!               dtlambda(i,ig) = cmplx(0.0_dop,1.0_dop)*ainv(i,i)*&
!                    dimag(dtlambda(i,ig)) + dble(dtlambda(i,ig))
            enddo
         enddo

      else if (gwptype .eq. 2) then
         do ig=1,gwpdim1

            counter = gdof
            do i  = 1,gdof
               counter = counter + 1
               addot(i,ig) = dtlambda(counter,ig)
               ad(i,i) = dble(agmat(i,i,ig))
               ai(i,i) = dimag(agmat(i,i,ig))

               ainv(i,i) = 1.0_dop/(ad(i,i))
               xidot(i,ig) = dtlambda(i,ig) + 2.0_dop*addot(i,ig)*dble(xi(i,ig))

               qdot(i) = -0.5_dop*dimag(xidot(i,ig))*ainv(i,i)
               pdot(i) = -dble(xidot(i,ig)) + 2.0d0*ai(i,i)*qdot(i)

               xidot(i,ig) = dcmplx(-pdot(i),qdot(i))
            enddo
         enddo

      else if (gwptype .eq. 0) then
         do ig=1,gwpdim1
            do i=1,gdof
               do j=1,gdof
                  ad(i,j) = dble(agmat(i,j,ig))
                  ai(i,j) = dimag(agmat(i,j,ig))
               enddo
            enddo

            lwork = 3*gdof
            info=0
            call dsyev('V','U',gdof,ad,gdof,aval,work,lwork,info)

            call zeromxd(ainv,gdof,gdof)
            do g=1,gdof
               do g1=1,gdof
                  do g2=1,gdof
                     ainv(g1,g)=ainv(g1,g)+ad(g,g2)*ad(g1,g2)/aval(g2)
                  enddo
               enddo
            enddo

            counter = gdof
            do i  = 1,gdof
               do j = i,gdof
                  counter = counter + 1
                  agdot(i,j,ig) = dtlambda(counter,ig)

                  if(i.ne.j)then
                     agdot(j,i,ig) = agdot(i,j,ig)
                  endif
               enddo
            enddo

            do i=1,gdof
               xidot(i,ig) = dtlambda(i,ig)
               do j=1,gdof
                  xidot(i,ig) = xidot(i,ig) + 2.0_dop*agdot(i,j,ig)*dble(xi(j,ig))
               enddo
               tmp(i) = -0.5_dop*dimag(xidot(i,ig))
            enddo
            do i=1,gdof
               qdot(i) = 0.0d0
               do j=1,gdof
                  qdot(i) = qdot(i) + ainv(i,j)*tmp(j)
               enddo
            enddo
            

            do i=1,gdof
               pdot(i) = -dble(xidot(i,ig))
               do j=1,gdof
                  pdot(i) = pdot(i) + 2.0d0*ai(i,j)*qdot(j)
               enddo
            enddo

            do i=1,gdof
               xidot(i,ig) = dcmplx(-pdot(i),qdot(i))
            enddo
         enddo

      endif

      return
      end subroutine bdot2pqdot

!#######################################################################

      subroutine calcphidot(phidot,gs2,dtlambda,gwpdim1,npg1,gdof,xi,xidot)

      implicit none

      integer(long)                                    :: e,p1,g
      integer(long), intent(in)                        :: gwpdim1,npg1,gdof

      complex(dop), dimension(gwpdim1), intent(inout)     :: phidot
      complex(dop), dimension(gwpdim1,gwpdim1,npg1,npg1), intent(in)   :: gs2
      complex(dop), dimension(npg1-1,gwpdim1), intent(in) :: dtlambda
      real(dop)                                        :: s2
      !
      complex(dop), dimension(gdof,gwpdim1), intent(in)       :: xi
      complex(dop), dimension(gdof,gwpdim1), intent(in)       :: xidot

      call zerovxz(phidot,gwpdim1)

      do e=1,gwpdim1
         do p1=2,npg1
            phidot(e)=phidot(e) - gs2(e,e,1,p1)*dtlambda(p1-1,e)
         enddo
         !
         ! Make i*gammadot =0 and i<g|gdot> = p*xdot ! added by yapol !
         !
         do g=1,gdof
            phidot(e) = phidot(e) + dimag(xi(g,e))*dimag(xidot(g,e))
         enddo
      enddo

      do e=1,gwpdim1
         s2=dble(gs2(e,e,1,1))
         if (s2 - 1.0_dop .gt. 0.5_dop) then
            routine='Calcphidot'
            message='Norm growing too large. Integration Error?'
            call errormsg
         endif
            
         if (dabs(s2) .gt. 1.0e-8_dop) then
             phidot(e)=phidot(e)/s2
         else
            phidot(e)=0.0_dop
         endif
      enddo
      
      return
      end subroutine calcphidot

!#########################################################################
!
! ADOT
!
!     calculates the modified time derivative of the A coefficients,
!     from the intermediate form iA = HA, and the time derivatives of
!     the parameters lambda, as obtained in gdot.
!
!#########################################################################

      subroutine adot(psi,dtpsi,gsinv,gsdot,hteil)

      implicit   none

      integer(long), parameter :: pd=5
      integer(long)            :: s,m,zeig1,zeig2,zeig3

      complex(dop), dimension(dgldim), intent(in)    :: psi
      complex(dop), dimension(dgldim), intent(inout) :: dtpsi
      complex(dop), dimension(gsmatdim), intent(in)  :: gsinv,gsdot
      complex(dop), dimension(pmatdim), intent(in)   :: hteil
      complex(dop), dimension(2*maxblock)            :: workc

      if (lgentrj) then
         call adottrjdat(psi,dtpsi,psi(zxi(1,1)),nddstate,dim(1,1),nspfdof(1))

      else if (gwpmodel .eq. 2) then
!-----------------------------------------------------------------------
! A-vector only has diagonal uncorrelated terms if independent GWPs
!-----------------------------------------------------------------------

         do s=1,nstate
            do m = 1,nmode
               if (.not. gwpm(m)) cycle

               call adotind(m,s,psi(zpsi(s)),dtpsi(zpsi(s)),&
                       gsdot(gsmat(m,s)),hteil(pmat(m,kzahl(1)+s)),block(s),&
                       dim(m,s))
            enddo
         enddo

      else
!-----------------------------------------------------------------------
! multiply S^{-1}*H_R
!------------ -----------------------------------------------------------
         if (kzahl(1) .gt. 0) then
            do s=1,nstate
               zeig1=1
               zeig2=block(s)+1
               call cpvxz(dtpsi(zpsi(s)),workc(zeig2),block(s))
               do m = 1,nmode
                  if (.not. gwpm(m)) cycle
                  zeig3=zeig1
                  zeig1=zeig2
                  zeig2=zeig3
                  call qtxxzz(gsinv(gsmat(m,s)),workc(zeig1),workc(zeig2),&
                    vdim(m,s),dim(m,s),ndim(m,s))
               enddo
               call cpvxz(workc(zeig2),dtpsi(zpsi(s)),block(s))
            enddo
         endif

!-----------------------------------------------------------------------
! add on uncorrelated terms
!-----------------------------------------------------------------------
         zeig3=1
         do s=1,nstate
            do m = 1,nmode
               if (.not. gwpm(m)) cycle

               if (lBEL) then
                  call adot1BEL1(m,s,hteil(pmat(m,kzahl(1)+s)),&
                       gsinv(gsmat(m,s)),psi(zpsi(s)),dtpsi(zpsi(s)),&
                       saveworkc(zeig3),gwpdim(m,s),block(s))
                  zeig3=zeig3+gwpdim(m,s)*gwpdim(m,s)
               else
                  call adot1(m,s,psi(zpsi(s)),dtpsi(zpsi(s)),gsinv(gsmat(m,s)),&
                       gsdot(gsmat(m,s)),hteil(pmat(m,kzahl(1)+s)),block(s),&
                       dim(m,s))
               endif
            enddo
         enddo
           if (BELflags(7) /= 0.0_dop) then
              reCompDtPsi=0
           end if
      endif
!-----------------------------------------------------------------------

      return
      end subroutine adot

!#######################################################################

      subroutine adot1BEL (h,gsinv,psi1,dtpsi,saveworkc,gwpdim1)

        implicit none

        integer(long),intent(in) ::     gwpdim1
        complex(dop),intent(in) ::  h(gwpdim1,gwpdim1)
        complex(dop),intent(inout) ::  saveworkc(gwpdim1,gwpdim1)
        complex(dop)             ::  workc(gwpdim1,gwpdim1)
        complex(dop),intent(in) ::  gsinv(gwpdim1,gwpdim1)
        complex(dop),intent(in) :: psi1(gwpdim1,1)
        complex(dop),intent(inout) :: dtpsi(gwpdim1,1)

        if  ((BELflags(7)==0) .or.(reCompDtPsi==1)) then
           workc=h
!           write(1000,*)"hteil",h(1:gwpdim1,1:gwpdim1)

           workc=matmul(gsinv,workc)
!           write(1000,*)"gsinv",gsinv(1:gwpdim1,1:gwpdim1)

           dtpsi=dtpsi+matmul(workc,psi1)
!           write(1000,*)"dtpsi",dtpsi(1:gwpdim1,1)

           if (BELflags(7) /= 0.0_dop) then
              saveworkc=workc
           end if
        else
           dtpsi=dtpsi+matmul(saveworkc,psi1)
        end if

      end subroutine adot1BEL

!#######################################################################

      subroutine adot1BEL1 (m,s,h,gsinv,psi1,dtpsi,saveworkc,gwpdim1,block1)

        implicit none

        integer(long),intent(in) ::     gwpdim1,block1,m,s
        complex(dop),intent(in) ::  h(gwpdim1,gwpdim1)
        complex(dop),intent(inout) ::  saveworkc(gwpdim1,gwpdim1)
        complex(dop)             ::  workc(gwpdim1,gwpdim1)
        complex(dop),intent(in) ::  gsinv(gwpdim1,gwpdim1)
        complex(dop),intent(in) :: psi1(block1)
        complex(dop),intent(inout) :: dtpsi(block1)

        if  ((BELflags(7)==0) .or.(reCompDtPsi==1)) then
           workc=h
!           write(1000,*)"hteil",h(1:gwpdim1,1:gwpdim1)

           workc=matmul(gsinv,workc)
!           write(1000,*)"gsinv",gsinv(1:gwpdim1,1:gwpdim1)

      call qtxxzza(workc,psi1,dtpsi,vdim(m,s),gwpdim1,ndim(m,s))
!           write(1000,*)"dtpsi",dtpsi(1:gwpdim1,1)

           if (BELflags(7) /= 0.0_dop) then
              saveworkc=workc
           end if
        else
           call qtxxzza(saveworkc,psi1,dtpsi,vdim(m,s),gwpdim1,ndim(m,s))
!write(1000,*)"m,s,vdim(m,s),dim1,ndim(m,s)",m,s,vdim(m,s),gwpdim1,ndim(m,s)
        end if

      end subroutine adot1BEL1

!#######################################################################

      subroutine adot1(m,s,psi,dtpsi,gsinv,gsdot,hteil,block1,dim1)

!+---------------------------------------------------------------------+
!     calculates the modified time derivative of the A coefficients,
!     from the intermediate form iA = HA, and the time derivatives of
!     the parameters lambda, as obtained in gdot.
!+---------------------------------------------------------------------+

      implicit   none

      integer(long),intent(in) :: s,m,block1,dim1

      complex(dop), dimension(block1), intent(in)    :: psi
      complex(dop), dimension(block1), intent(inout) :: dtpsi
      complex(dop), dimension(dim1,dim1), intent(in) :: gsinv,gsdot
      complex(dop), dimension(dim1,dim1), intent(in) :: hteil
      complex(dop), dimension(dim1,dim1)             :: hcons,ztmp

      if (khzahl(m,s,1) .gt. 0) then
         call submxxz(gsdot,hteil,ztmp,dim1,dim1)
         call qqxxzz(gsinv,ztmp,hcons,dim1)
      else
         call qqxxzz(gsinv,gsdot,hcons,dim1)
      endif
      if (gwpphase .eq. 4) call rezero(hcons,dim1)
      call qtxxzzr(hcons,psi,dtpsi,vdim(m,s),dim1,ndim(m,s))

      return
      end subroutine adot1

!#######################################################################

      subroutine adotind(m,s,psi,dtpsi,gsdot,hteil,block1,dim1)

!+---------------------------------------------------------------------+
!     calculates the modified time derivative of the A coefficients,
!     from the intermediate form iA = HA, and the time derivatives of
!     the parameters lambda, as obtained in gdot.
!+---------------------------------------------------------------------+

      implicit   none

      integer(long),intent(in) :: s,m,block1,dim1
      integer(long)            :: e

      complex(dop), dimension(block1), intent(in)    :: psi
      complex(dop), dimension(block1), intent(inout) :: dtpsi
      complex(dop), dimension(dim1,dim1), intent(in) :: gsdot
      complex(dop), dimension(dim1,dim1), intent(in) :: hteil
      complex(dop), dimension(dim1)                  :: hcons
      real(dop) :: imhcons

      if (khzahl(m,s,1) .gt. 0) then
         do e=1,dim1
            hcons(e) = gsdot(e,e) - hteil(e,e) 
         enddo
         
         if (gwpphase .eq. 4) then
            do e=1,dim1
               imhcons = dimag(hcons(e))
               hcons(e) = dcmplx(0.0_dop,imhcons)
            enddo
         endif

         call dtxxzzr(hcons,psi,dtpsi,vdim(m,s),dim(m,s),ndim(m,s))
      endif

      return
      end subroutine adotind

!#######################################################################

      subroutine adottrjdat(psi,dtpsi,xi,nsta,dim1,gdof)

!+---------------------------------------------------------------------+
!     calculates the modified time derivative of the A coefficients,
!     from the intermediate form iA = HA, and the time derivatives of
!     the parameters lambda, as obtained in gdot.
!+---------------------------------------------------------------------+

      implicit   none

      integer(long),intent(in) :: nsta,dim1,gdof
      integer(long)            :: e,s,s1,f,g,izflag

      complex(dop), dimension(dim1,nsta), intent(in)  :: psi
      complex(dop), dimension(gdof,dim1), intent(in)  :: xi
      complex(dop), dimension(dim1,nsta), intent(inout) :: dtpsi

      real(dop), dimension(dim1,nsta,nsta)   :: hamelpot
      real(dop), dimension(maxsta,maxsta)    :: pesdia
      complex(dop), dimension(maxsta,maxsta) :: pesdiaz
      real(dop), dimension(maxdim)       :: gpoint,ppoint
      integer(long), dimension(maxdim)   :: igpoint


      do e=1,dim1
! igpoint is the "grid point" - not used for GBFs
         igpoint = 1
         do g=1,gdof
            f=spfdof(g,1)
            gpoint(f) = dble(xi(g,e))
            ppoint(f) = dimag(xi(g,e))
         enddo
! add coordinates for frozen DOFs
         do f=1,ndof
            if (dofspf(f) .ne. 0) cycle
            gpoint(f) = rsbaspar(1,f,1)
            ppoint(f) = 0.0_dop
         enddo

         call calcpes(mr(mrhops),pesdia,igpoint,gpoint,pesdiaz,izflag,1)

         do s=1,nsta
            do s1=1,nsta
               hamelpot(e,s1,s) = pesdia(s1,s)
            enddo
         enddo
      enddo

      dtpsi = 0.0_dop
      do e=1,dim1
         do s=1,nsta
            do s1=1,nsta
               dtpsi(e,s1) = dtpsi(e,s1) + hamelpot(e,s1,s) * psi(e,s)
            enddo
         enddo
      enddo

      return
      end subroutine adottrjdat

!#######################################################################

      subroutine rezero(hcons,gwpdim1)

      implicit none

      integer(long) :: i,j
      integer(long), intent(in) :: gwpdim1
      complex(dop), dimension(gwpdim1,gwpdim1), intent(inout) :: hcons
      real(dop) :: imhcons

      do i=1,gwpdim1
         do j=1,gwpdim1
            imhcons = dimag(hcons(j,i))
            hcons(j,i) = dcmplx(0.0_dop,imhcons)
         enddo
      enddo

      return
      end subroutine rezero

      subroutine gwpcons(dtpsi,psi,hteil)

      implicit   none

      integer(long)            :: s,m,kz
      complex(dop), dimension(dgldim), intent(in)    :: psi
      complex(dop), dimension(dgldim), intent(inout) :: dtpsi
      complex(dop), dimension(pmatdim), intent(in)   :: hteil

      kz = kzahl(1)
      do s=1,nstate
         do m=1,nmode
            call gwpcons1ms(dtpsi(zpsi(s)),dtpsi(zdgwp(m,s)), &
                            psi(zpsi(s)),psi(zdgwp(m,s)), &
                            hteil(pmat(m,kz+s)),vdim(m,s),dim(m,s),ndim(m,s))
         enddo
      enddo

      return
      end subroutine gwpcons

      subroutine gwpcons1ms(dtpsi,dtdgwp,psi,dgwp,hteil,vdim1,dim1,ndim1)

      implicit   none

      integer(long)            :: vdim1,dim1,ndim1
      complex(dop), dimension(vdim1,dim1,ndim1), intent(in)  :: psi
      complex(dop), dimension(vdim1,dim1,ndim1), intent(inout) :: dtpsi
      complex(dop), dimension(dim1,dim1), intent(in)         :: dgwp
      complex(dop), dimension(dim1,dim1), intent(inout)        :: dtdgwp
      complex(dop), dimension(dim1,dim1), intent(in)         :: hteil
      complex(dop), dimension(dim1,dim1)                     :: hcons
      complex(dop), dimension(dim1,dim1)                     :: tmp

      call cpqxz(hteil,hcons,dim1)
      call zerodiag(hcons,dim1)
      call qtxxzzr(hcons,psi,dtpsi,vdim1,dim1,ndim1)

      call qqxxzz(dgwp,hcons,tmp,dim1)
      call addmxxzo(tmp,dtdgwp,dim1,dim1)

      end subroutine gwpcons1ms

      subroutine zerodiag(hcons,dim1)

      integer(long)                                     :: e
      integer(long),intent(in)                          :: dim1
      complex(dop), dimension(dim1,dim1), intent(inout) :: hcons

       do e = 1,dim1
          hcons(e,e) = 0.0d0
       enddo

      return
      end subroutine zerodiag

!#######################################################################
!+---------------------------------------------------------------------+
!     for a given combination of Gaussians (m,n), OF SEPARABLE FORM,
!     and a given integer array imoment, computes the value of the
!     corresponding moment.
!+---------------------------------------------------------------------+

      complex(dop) function smoment(imoment,gdof,ai,z)


      implicit   none

      integer(long)                                :: i
      integer(long), intent(in)                    :: gdof
      integer(long), dimension(maxdim), intent(in) :: imoment

      complex(dop)                                   :: p, q
!      complex, external                              :: integral_1d
      complex(dop), dimension(gdof), intent(in)      :: z
      complex(dop), dimension(gdof,gdof), intent(in) :: ai

      smoment = dcmplx(1.0_dop,0.0_dop)

      do i = 1,gdof
        p = 1.0_dop/ai(i,i)
        q = z(i) / ai(i,i)
!       q = xid(i)
        smoment = smoment*integral_1D(imoment(i),p,q)
      enddo

!     smoment = smoment*exp(phiG)

      return
      end  function smoment


!+---------------------------------------------------------------------+
!     returns value of 1D moment integral, for integration limits
!     \pm infty
!     integrate over    x^n exp(p x^2 + 2 q x)
!+---------------------------------------------------------------------+

      complex(dop) function integral_1d (n,p,q)

      implicit       none

      integer(long)             :: k
      integer(long), intent(in) :: n
      complex(dop), intent(in)  :: p,q
      complex(dop)              :: sum

!      integer, external         :: factorial

      if(n.eq.0)then

        integral_1d =  &
             sqrt(-pi/p) * exp(-q**2/(4.0_dop*p))

      else

! ...   cf. 3.462 GR

        sum = dcmplx(0.00_dop,0.0_dop)
        do k = 0, int(n/2.0_dop)
           sum = sum +  &
             (-p/(q**2))**k/ &
             (factorial(n-2*k)*factorial(k))
        enddo

        integral_1d = factorial(n) &
         * exp(-q**2/(4.0_dop*p)) * sqrt(-pi/p) &
         * (-q/(2.0_dop*p))**n * sum

      endif

      return
      end function integral_1d

!+---------------------------------------------------------------------+
!     returns factorial n!
!+---------------------------------------------------------------------+
      integer(long) function factorial(n)

      implicit  none
      integer(long)             :: i
      integer(long), intent(in) :: n

      if(n.eq.0)then
         factorial = 1
      else
         factorial = 1
         do i=1,n
           factorial = factorial*i
         enddo
      endif

      return
      end function factorial


!#######################################################################

      subroutine diag_cont(gdof,ag,eval,evr,ig,jg)
!+---------------------------------------------------------------------+
!     Robin's continued diagonalization routine 
!+---------------------------------------------------------------------+       

      implicit     none

      integer(long)                             :: i,j,lda,lwork,info
      integer(long), intent(in)                 :: ig,jg,gdof

      real(dop), dimension(maxdim)              :: eval_r
      real(dop), dimension(maxdim,maxdim)       :: evec_r
      real(dop), dimension(60)                  :: work_r

      complex(dop), dimension(maxdim), intent(inout)        :: eval
      complex(dop), dimension(maxdim,maxdim), intent(in)    :: ag
      complex(dop), dimension(maxdim,maxdim), intent(inout) :: evr
      complex(dop), dimension(maxdim,maxdim,maxdim)         :: eval_tmp
      complex(dop), dimension(maxdim,maxdim,maxdim,maxdim)  :: evec_tmp

      logical(kind=4), dimension(2,2)                       :: firstg

      common/first_index/firstg

!     common/temp_DC/evec_tmp,
!    &               eval_tmp

      if(firstg(ig,jg))then

        do i=1,gdof
         do j=1,gdof
          evec_r(i,j)=dble(ag(i,j))
         enddo
        enddo

        lda   = maxdim
        lwork = 60
        call dsyev('V','U',gdof,evec_r,lda, &
              eval_r,work_r,lwork,info)

! ...   store:

        do i=1,gdof
          do j=1,gdof
           evec_tmp(i,j,ig,jg) = dcmplx(evec_r(i,j),0.0_dop)
           evr     (i,j)       = evec_tmp(i,j,ig,jg)
          enddo
           eval_tmp(i,ig,jg  ) = dcmplx(eval_r(i)  ,0.0_dop)
           eval    (i)         = eval_tmp(i,ig,jg  )
        enddo

        firstg(ig,jg) = .false.

      else

        do i=1,gdof
           do j=1,gdof
             evr(i,j)     = evec_tmp(i,j,ig,jg)
           enddo
          eval(i) = eval_tmp(i,ig,jg)
        enddo

        lda = maxdim
!       call cont_diag(ag,gdof,lda,evr,eval)

        call cont_PT(ag,gdof,lda,evr,eval)

        do i=1,gdof
           do j=1,gdof
              evec_tmp(i,j,ig,jg) = evr (i,j)
              eval_tmp(i,ig,jg  ) = eval(i)
           enddo
        enddo

      endif

      return
      end  subroutine diag_cont

!#######################################################################

      subroutine cont_diag(amat,dim,lda,evec,eval)

      implicit none

      integer(long), parameter :: maxtemp = 10
      real(dop), parameter     :: eps=1.0e-8_dop

      integer(long)              :: i,j,k
      integer(long), intent(in)  :: dim
      integer(long), intent(inout) :: lda

      real(dop), external        :: dznrm2
      real(dop)                  :: error

      complex(dop), external                                :: zdotu
      complex(dop), dimension(maxtemp)                      :: auxvec
      complex(dop), dimension(maxtemp), intent(inout)         :: eval
      complex(dop), dimension(maxtemp,maxtemp), intent(in)  :: amat,evec
      complex(dop), dimension(maxtemp,maxtemp)              :: auxmat

!     real*8 norm_temp
!     parameter(eps=1.0d-13)

      lda = maxtemp

      do k = 1, dim

         error = 1.0_dop
         do while (error .gt. eps)

            do j = 1, dim
               do i = 1, dim
                  auxmat(i,j) = amat(i,j)
               end do
               auxmat(j,j) = auxmat(j,j) - eval(k)
            end do

!           call zgesv(dim,1,auxmat,lda,ipiv,evec(1,k),lda,info)

!           norm = zdotu(dim,evec(1,k),1,evec(1,k),1)
!           norm = 1.0_dop/zsqrt(norm)
!           call zscal(dim,norm,evec(1,k),1)

!+---------------------------------------------------------------------+
!           add Hermitian normalization:
!+---------------------------------------------------------------------+

!           do i = 1,dim
!              norm_temp = 0._dop
!           do j = 1,dim
!              norm_temp = norm_temp +
!    &         dble(dconjg(evec(i,j))
!    &                     *evec(j,i))
!           enddo
!           do j = 1,dim
!              evec(j,i) = 1._dop/dsqrt(norm_temp)*
!    &                     evec(j,i)
!           enddo
!
!           call zgemv('N',dim,dim,(1.0_dop,0.0_dop),Amat,
!    &           lda,evec(1,k),1,(0.0_dop,0.0_dop),auxvec,1)
            eval(k) = zdotu(dim,evec(1,k),1,auxvec,1)
!           call zaxpy(dim,-eval(k),evec(1,k),1,auxvec,1)
            error = dznrm2(dim,auxvec,1)

         end do

      end do

      return
      end subroutine cont_diag


!+---------------------------------------------------------------------+
!     This routine is written by Robin and uses perturbation theory
!     to calculate new eigenvalues/eigenvectors, starting from the ones
!     of the preceding integration step
!+---------------------------------------------------------------------+
      subroutine cont_pt(amat,dim,lda,evec,eval)

      implicit none

      integer(long), parameter :: maxtemp = 10, maxit = 10
      real(dop), parameter     :: eps=1.0e-13_dop

      integer(long)              :: i,j,k
      integer(long), intent(in)  :: dim
      integer(long), intent(inout) :: lda

      real(dop)                  :: error,auxscal,s2

      complex(dop), external                                :: zdotu
      complex(dop)                                          :: auxeval,norm
      complex(dop), dimension(maxtemp), intent(inout)         :: eval
      complex(dop), dimension(maxtemp,maxtemp), intent(in)  :: amat,evec
      complex(dop), dimension(maxtemp,maxtemp)              :: auxmat

      lda = maxtemp
      auxscal = 2.0_dop/dble(dim*(dim-1))

! dummy line to prevent unused variable warning
      auxmat(1,1)=amat(1,1)

      print*,'CONT_PT NEEDS CHECKING'
      stop

      do k = 1, maxit

         call zeromxz(auxmat,maxtemp,maxtemp)
!        call zsymm('l','u',dim,dim,(1.0_dop,0.0_dop),Amat,lda,evec,
!    &        lda,(0.0_dop,0.0_dop),auxvec,lda)

!        call zgemm('t','n',dim,dim,dim,(1.0_dop,0.0_dop),evec,lda,
!    &        auxvec,lda,(0.0_dop,0.0_dop),auxmat,lda)

!         write(*,*) 'transformed matrix'
!         do j = 1, dim
!            do i = 1, dim
!               write(*,*) i, j, auxmat(i,j)
!            end do
!         end do

         error = 0.0_dop
         do j = 1, dim
            do i = 1, j-1
               s2=dble(auxmat(i,j))
               error = error + dabs(s2)
            end do
            eval(j) = auxmat(j,j)
         end do
         error = error*auxscal

!         write(*,*) 'error', error

         if (error .le. eps) return

         do j = 1, dim
            auxeval=eval(j)
            do i = 1, j-1
               auxmat(i,j) = auxmat(i,j)/(auxeval-eval(i))
            end do
            auxmat(j,j) = (1.0_dop,0.0_dop)
            do i = j+1, dim
               auxmat(i,j) = auxmat(i,j)/(auxeval-eval(i))
            end do
         end do

!        call zgemm('n','n',dim,dim,dim,(1.0_dop,0.0_dop),evec,
!    &        lda,auxmat,lda,(0.0_dop,0.0_dop),auxvec,lda)

         do j = 1, dim
!           call zswap(dim,auxvec(1,j),1,evec(1,j),1)
            norm = zdotu(dim,evec(1,j),1,evec(1,j),1)
            norm = (1.0_dop,0.0_dop)/sqrt(norm)
!           call zscal(dim,norm,evec(1,j),1)
         end do

!         write(*,*) 'new eigenvalues (cont_diag2)'
!         do i = 1, dim
!            write(*,*) i, eval(i)
!         end do
!         write(*,*) 'new eigenvectors (cont_diag2)'
!         do j = 1, dim
!            do i = 1, dim
!               write(*,*) i, j, evec(i,j)
!            end do
!         end do

      end do

      write(*,*) 'No convergence in ', maxit, ' iterations!'

      return
      end subroutine cont_pt

!****************************************************************************

      subroutine chkdmato(psi,adgwp,gs2,gsinv)

      implicit none

      integer(long)                                  :: m,s,e,e1,e2
      complex(dop)                                   :: sum2
      complex(dop), dimension(dgldim), intent(in)    :: psi
      complex(dop), dimension(gsmatdim), intent(in)  :: adgwp,gsinv
      complex(dop), dimension(gs2matdim), intent(in) :: gs2
      complex(dop), dimension(maxspf*maxspf)         :: workc
      real*8 err,err1,err2

      do s=1,nstate
         do m=1,nmode
            if (gwpm(m)) then
               call qqxxzz(adgwp(gsmat(m,s)),psi(zdgwp(m,s)),workc,dim(m,s))
               call chkdmat1(workc,dim(m,s),err)
               call qqxxzz(psi(zdgwp(m,s)),adgwp(gsmat(m,s)),workc,dim(m,s))
               call chkdmat1(workc,dim(m,s),err1)
               call qqaxzz(adgwp(gsmat(m,s)),adgwp(gsmat(m,s)),workc,dim(m,s))
               call chkdmat2(workc,gs2(gs2mat(m,s)),dim(m,s),err2)
               print*,m,s,'ERROR  D+D~  D~D+   DD+  '
               print*,err,err1,err2

               call simtranbz(gs2(gs2mat(m,s)),psi(zdgwp(m,s)),workc,dim(m,s))
               print*,'OVL',m,s,gsmatdim,dim(m,s)
               e2=0
               do e=1,dim(m,s)
                  do e1=1,dim(m,s)
                     e2=e2+1
                     print*,e1,e,workc(e2)
                  enddo
               enddo
               call qqxazz(psi(zdgwp(m,s)),psi(zdgwp(m,s)),workc,dim(m,s))
               call submxxzo1(workc,gsinv(gsmat(m,s)),dim(m,s),dim(m,s))
               call sum2qxz(workc,sum2,dim(m,s))
               print*,'D~ - S-1 : ',sum2
            endif
         enddo
      enddo

      return
      end subroutine chkdmato

!#######################################################################

      subroutine chkdmat1(mat,dim1,err)

      implicit none

      integer(long)                                  :: e,e1
      integer(long), intent(in)                      :: dim1
      complex(dop), dimension(dim1,dim1), intent(in) :: mat
      real(dop), intent(inout)                         :: err

      err=0.0_dop
      do e=1,dim1
         do e1=e+1,dim1
            err=err+dble(mat(e,e1))**2+dimag(mat(e,e1))**2
         enddo
         do e1=1,e-1
            err=err+dble(mat(e,e1))**2+dimag(mat(e,e1))**2
         enddo
         err=err+dble(1.0_dop-mat(e,e))**2+dimag(1.0_dop-mat(e,e))**2
      enddo

      return
      end subroutine chkdmat1


!#######################################################################

      subroutine chkdmat2(mat,gs2,dim1,err)

      implicit none

      integer(long)                                  :: e,e1
      integer(long), intent(in)                      :: dim1
      complex(dop)                                   :: tmp
      complex(dop), dimension(dim1,dim1), intent(in) :: mat,gs2
      real(dop), intent(inout)                         :: err

      err=0.0_dop
      do e=1,dim1
         do e1=1,dim1
            tmp=gs2(e,e1)-mat(e,e1)
            err=err+dble(tmp)**2 + dimag(tmp)**2
         enddo
      enddo

      return
      end subroutine chkdmat2


!***********************************************************************

      subroutine subengwp(dicht2,psi,hunpsi,gs2,hloch,hteil,pcons,dtpsi)

      implicit none

      integer(long)                                      :: m,s,f,nham,zeig,zeig1,zeig2
      real(dop)                                          :: norm,factor
      complex(dop)                                       :: sum
      complex(dop), dimension(dgldim), intent(in)        :: psi
      complex(dop), dimension(dmatdim), intent(in)       :: dicht2
      complex(dop), dimension(gs2matdim), intent(in)     :: gs2
      complex(dop), dimension(hlochdim), intent(in)      :: hloch
      complex(dop), dimension(pmatdim), intent(in)       :: hteil
      complex(dop), dimension(pconsdim), intent(in)      :: pcons
      complex(dop), dimension(adim+1:dgldim), intent(in) :: hunpsi
      complex(dop), dimension(dgldim), intent(inout)       :: dtpsi
      complex(dop), dimension(2*maxblock)                :: tmppsi
      complex(dop), dimension(maxspf*maxspf)             :: tmphteil

!-----------------------------------------------------------------------
! if independent GWPs
!-----------------------------------------------------------------------
       if (gwpmodel .eq. 2) return

!-----------------------------------------------------------------------
! get norm
!-----------------------------------------------------------------------
      norm=0.0_dop
      do s=1,nstate
         zeig=1
         zeig1=block(s)+1
         call cpvxz(psi(zpsi(s)),tmppsi(zeig),block(s))
         do m=1,nmode
            f = spfdof(1,m)
            if(basis(f).eq.25)then
              call qtxxzz(gs2(gs2mat(m,s)), &
                          tmppsi(zeig),tmppsi(zeig1), &
                          vdim(m,s),dim(m,s),ndim(m,s))
              zeig2=zeig
              zeig=zeig1
              zeig1=zeig2
            endif
         enddo
         call vvaxzz(psi(zpsi(s)),tmppsi(zeig),sum,block(s))

         norm=norm+dble(sum)
      enddo

      rlxenergy=0.0_dop
!-----------------------------------------------------------------------
! calculate uncorrelated energy expectation value
!-----------------------------------------------------------------------
      do s=1,nstate
         do m=1,nmode
            f=spfdof(1,m)
            if (basis(f) .ne. 25 .and. &
                 basis(f) .ne. 26 .and. basis(f) .ne. 27) then
               call mmaxzzh(psi(zetf(m,s)),hunpsi(zetf(m,s)),tmphteil,&
                    subdim(m),dim(m,s))
            else if (basis(f) .ne. 25) then
               nham=1
               call xlocphihunphi(psi(zetf(m,1)),tmphteil,m,f,3,dim(m,1), &
                    nham)
               if (basis(f) .eq. 26) &
                  call addmxxzo(pcons(zpcons(f)),tmphteil,dim(m,s),dim(m,s))
            elseif(basis(f).eq.25)then
               call cpqxz(hteil(pmat(m,kzahl(1)+s)),tmphteil,dim(m,s))

            endif
            call sum2qxxz(dicht2(dmat(m,s)),tmphteil,sum,dim(m,s))
            rlxenergy=rlxenergy+dble(sum)
         enddo
      enddo

!-----------------------------------------------------------------------
! calculate correlated energy 
!-----------------------------------------------------------------------
      call calcenergy_g(psi,dtpsi,rlxenergy,0)

      factor=rlxenergy/norm
      call xvxxdzr(factor,psi,dtpsi,adim)

      return
      end subroutine subengwp

!#######################################################################
!
!               WKGAUSSIAN
!
! Here the workspace required for the module is assigned
!
! workcdim, workrdim, workidim and workldim are passed via a common
! block in global.inc, and are checked that they are large enough for
! the module.
!
!#######################################################################

      subroutine wkgaussian

      implicit none

      integer(long)       :: m,s,gdof,maxcp,maxyp,mcp,myp,itemp

!-----------------------------------------------------------------------
! calculate size of cp and yp matrices
!-----------------------------------------------------------------------
      maxcp=1
      maxyp=1
      do s=1,nstate
         do m=1,nmode
            if (gwpm(m)) then
               gdof=nspfdof(m)

               myp=(npg(m)-1)*dim(m,s)
               mcp=myp*myp
               maxyp=max(maxyp,myp)
               maxcp=max(maxcp,mcp)
            endif
          enddo
      enddo

!
      itemp=0
! for param_from_psi
      itemp=max(itemp,2*maxdim*maxdim)
! for diagonal_moments and overlap_elements
      itemp=max(itemp,maxspf*maxspf + maxdim*maxspf*maxspf + &
                            maxdim*maxdim*maxspf*maxspf)
      workcdim=max(workcdim,3*dmatdim+agmatdim+itemp)

!-----------------------------------------------------------------------
! assign workspace used in module
!-----------------------------------------------------------------------
! for renormgwp
      if (lrlx) then
         workcdim=max(workcdim,2*maxblock)
      endif
! for gdot1
      workcdim = max(workcdim,maxspf*maxspf)
      workcdim = max(workcdim,3*maxspf*maxspf+2*maxcp+maxyp)
! for adot
      workcdim = max(workcdim,2*maxblock+2*maxspf*maxspf)
      workrdim=max(workrdim,4*maxyp)
! for invert_ovr
      workcdim=max(workcdim,maxspf*maxspf)
      workrdim=max(workrdim,6*maxspf)
! for dgwp_inv
      workcdim=max(workcdim,2*maxspf*maxspf)
! for gwplindep
      workcdim=max(workcdim,dgldim)

if (lBEL) workcdim = 1

      workidim=max(workidim,maxspf)

      workldim=workldim

      return
      end subroutine wkgaussian


      end module gaussian


