!***********************************************************************
      module gwpdepdynmod

use decimal, only: long,dop
use global
use globmemmod
use dvrdatmod
use griddatmod
use op1lib, only: overmxz,cpvxz, cpmxz,cpmxi,cpqxz,zerovxz, zeromxz,&
                  zerovxi,cpvxi
use lalib, only: simtran1btz

use channels
use datenmod

use logdat, only: errormsg,message,routine
use gwplib, only: invert_ovr
use mmlib
use mtlib, only: mtxxzz

use psidef
use operdef, only: lcap
use runpropmod
use gaussian, only: gwpcmat
use hpsimod, only: gwpcxm
use timing
use logdat

      implicit none
      private
      public :: setgwpdyn,wrgwpdyn_final,gwpodetest,gwpodetest1m

      contains

!***********************************************************************
!
! SUBROUTINE GWPODETESTVMF
!
! Post-processing function called from the RK integrators (maybe others
! in the future, too) during VMF propagations.
!
!***********************************************************************
!
! INPUT:
!     psi: complete MCTDH wavefunction
! OUTPUT:
!     psi1: MCTDH wavefunction, after sorting of dependent and
!          dynamical GWPs
!
! psi1 may be different from psi so that linear dependencies can be 
! calculated on one wavefunction and projected out of a different. This 
! is required, e.g. when an integration step is rejected so that the
! offending function can be removed from psi at the start of the step.
!
!***********************************************************************

      subroutine gwpodetest(psi,lrecalc,time)

      implicit none

      complex(dop), dimension(*), intent(in)    :: psi
      logical(kind=4), intent(out)              :: lrecalc
      real(dop),intent(in)                      :: time
      real(dop) :: rdum

! copy to use "unused variables" 
      rdum=time
      rdum=dble(psi(1))

! Could use test to ask for a recalculation with smaller step size...
      lrecalc = .false.

! no tests set for non-GWPs
      if (.not. lgwp) return
   
      call setgwpdyn(psi,gs2,mi(migwpdyn),mi(migwpdep),lrecalc,&
              gsinv,dicht2,time)

      return
      end subroutine gwpodetest

!#######################################################################

      subroutine gwpodetest1m(psi,lrecalc,time)

      implicit none

      complex(dop), dimension(*), intent(in)    :: psi
      logical(kind=4), intent(out)              :: lrecalc
      real(dop),intent(in)                      :: time

      integer(long) :: m

      real(dop) :: rdum

! copy to use "unused variables" 
      rdum=time
      rdum=dble(psi(1))

! Could use test to ask for a recalculation with smaller step size...
      lrecalc = .false.

      m=currentmode
      if (.not. gwpm(m)) return

      call setgwpdyn1m(m,psi,gs2,mi(migwpdyn),mi(migwpdep),lrecalc,&
              gsinv,dicht2,time)

      return
      end subroutine gwpodetest1m

!#######################################################################
!
! Sets dynamical status of parameters
! gwpdyn(n,e) = 0   coupled (Ldot = X + C^-1 Y)
!              -1   clasical and uncoupled (Ldot = X)
!              -2   static (Ldot = 0)
!               1   inactive (C^-1 Y = 0; Ldot = X or Ldot = 0)
!               2   static as linearly dependent
!#######################################################################

      subroutine setgwpdyn(psi,gs2,gwpdyn,gwpdep,lrecalc,gsinv,dicht2,time)

use global
use runpropmod
use psidef
use griddatmod

use rmlib, only: rm1hxxxzz,rmhxxxzz
use mtlib, only: qtxxzz
use xvlib, only: vvaxzz, xmxxdza
use gwplib, only: renormdgwp,renormgwpa
use psiutil, only: renorma

      implicit none

      integer(long) :: m
      complex(dop), dimension(dgldim), intent(in)       :: psi
      complex(dop),dimension(gs2matdim),intent(inout)   :: gs2
      complex(dop), dimension(dmatdim), intent(in)      :: dicht2
      complex(dop), dimension(gsmatdim), intent(in)     :: gsinv
      real(dop),intent(in)                              :: time
      integer(long),dimension(gwpdyndim),intent(inout)  :: gwpdyn
      integer(long), dimension(gwpinfodim), intent(inout) :: gwpdep

      logical(kind=4), intent(out) :: lrecalc
      logical(kind=4)              :: init=.true.
      integer(long), save          :: tid=0

      call get_timer(tid,'Setgwpdyn')
      call start_timer(tid)

      if (init) then
         gwpdyn = 1
         gwpdep = 0
         maxngdep = 0
         maxogdep = 0
         maxtgdep = 0
         maxngdyn = 0
         mincgdyn = 0
         maxtgdyn = 0
      endif

      do m=1,nmode
         if (.not. gwpm(m)) cycle
         call setgwpdyn1m(m,psi,gs2,mi(migwpdyn),mi(migwpdep),lrecalc,&
              gsinv,dicht2,time)
      enddo
      init=.false.

      call stop_timer(tid)

      return
      end subroutine setgwpdyn

!#######################################################################

      subroutine setgwpdyn1m(m,psi,gs2,gwpdyn,gwpdep,lrecalc,gsinv,dicht2,time)

use global
use runpropmod
use psidef
use griddatmod

use rmlib, only: rm1hxxxzz,rmhxxxzz
use mtlib, only: qtxxzz
use xvlib, only: vvaxzz, xmxxdza
use gwplib, only: renormdgwp,renormgwpa
use psiutil, only: renorma

      implicit none

      integer(long),intent(in) :: m
      integer(long)            :: s,z1,z2
      complex(dop), dimension(dgldim), intent(in)       :: psi
      complex(dop),dimension(gs2matdim),intent(inout)   :: gs2
      complex(dop), dimension(dmatdim), intent(in)      :: dicht2
      complex(dop), dimension(gsmatdim), intent(in)     :: gsinv
      real(dop),intent(in)                              :: time
      integer(long),dimension(gwpdyndim),intent(inout)  :: gwpdyn
      integer(long), dimension(gwpinfodim), intent(inout) :: gwpdep
      integer(long), dimension(gwpdyndim)               :: gwpdynold
      integer(long), dimension(gwpinfodim)              :: gwpdepold
      integer(long)                                     :: cpdim

      logical(kind=4), intent(out) :: lrecalc
      logical(kind=4)              :: init=.true.

      lrecalc = .false.
      do s=1,nstate
         call cpvxi(gwpdyn(zgwpdyn(m,s)),gwpdynold(zgwpdyn(m,s)), &
              (npg(m)-1)*gwpdim(m,s))
         call cpvxi(gwpdep(zgwpinfo(m,s)),gwpdepold(zgwpinfo(m,s)),&
              gwpdim(m,s))
      enddo

!-----------------------------------------------------------------------
! initialise gwpdyn arrays so GWPs are flagged as dynamic / inactive
!-----------------------------------------------------------------------
      do s=1,nstate
         call initgwpdyn(m,gwpdyn(zgwpdyn(m,s)),npg(m),gwpdim(m,s),&
              igwpdynm(m),ngwpdyn(m,s),gwpdep(zgwpinfo(m,s)),ngwpdep(m,s),&
              gwpdynold(zgwpdyn(m,s)),init)
      enddo

!-------------------------------------------------------------------------
! first flag any linear dependent functions 
! Choice between using individual overlaps of GWPs or eigenvalues/vectors.
!-------------------------------------------------------------------------
      if (linchk) then
         do s=1,nstate
            if(lsmatovl)then
               call gwplindep1msovl(gs2(gs2mat(m,s)),gwpdep(zgwpinfo(m,s)),&
                    gwpdepold(zgwpinfo(m,s)),gwpdim(m,s),m,s)
            else
               call gwplindep1ms(gs2(gs2mat(m,s)),gwpdep(zgwpinfo(m,s)),&
                    gwpdepold(zgwpinfo(m,s)),gwpdim(m,s),m,s)
            endif
         enddo
      endif

!-----------------------------------------------------------------------
! Deactivate GWPs in CAPS
!-----------------------------------------------------------------------
      if (lcap) then
         do s=1,nstate
            call gwpcapinact(m,s,gwpdyn(zgwpdyn(m,s)),&
                 dicht2(dmat(m,s)),npg(m),gwpdim(m,s),&
                 psi(zdgwp(m,s)),dim(m,s))
         enddo
      endif

!-----------------------------------------------------------------------
! igwpdynm(m) = 1 dynamic selection 
!-----------------------------------------------------------------------
      if (igwpdynm(m) .eq. 1) then
         do s=1,nstate
            cpdim = (npg(m)-1)*gwpdim(m,s)
            call setgwpdyn1ms(gs2(gs2mat(m,s)),gwpdyn(zgwpdyn(m,s)), &
                 gwpdim(m,s),dim(m,s),npg(m),dicht2(dmat(m,s)),&
                 gsinv(gsmat(m,s)),psi(zdgwp(m,s)), &
                 m,s,time,cpdim,gwpdynold(zgwpdyn(m,s)))
         enddo
      endif

!-----------------------------------------------------------------------
! flag any active linearly dependent functions as inactive
!-----------------------------------------------------------------------
      do s=1,nstate
         if (ngwpdep(m,s) .gt. 0) then
            call lindepinact(gwpdep(zgwpinfo(m,s)),gwpdyn(zgwpdyn(m,s)),&
                 npg(m),gwpdim(m,s),ngwpdyn(m,s))
         endif
      enddo

!-----------------------------------------------------------------------
! write information to log file
!-----------------------------------------------------------------------
      if (lactivegp) then     
         do s=1,nstate
            cpdim = (npg(m)-1)*gwpdim(m,s)
            call wrgwpdyn(gwpdyn(zgwpdyn(m,s)),gwpdynold(zgwpdyn(m,s)),&
                 cpdim,ngwpdyn(m,s),time,m,s)
         enddo
      endif

!-----------------------------------------------------------------------
! if any changes made to allocations, then lrecalc is set to true.
! This is passed to integrator to know that dtpsi must be recalculated.
!-----------------------------------------------------------------------
      do s=1,nstate
         z1=zgwpdyn(m,s)
         z2=zgwpdyn(m,s)+(npg(m)-1)*gwpdim(m,s)-1
         if (any(gwpdynold(z1:z2) /= gwpdyn(z1:z2))) then
            lrecalc = .true.
            if (lresint) dores = .true.
         endif
      enddo

      return
      end subroutine setgwpdyn1m

!#######################################################################

      subroutine lindepinact(gwpdep,gwpdyn,npg1,gwpdim1,ngwpdyn1)

      implicit none

      integer(long), intent(in)  :: npg1,gwpdim1
      integer(long), intent(out) :: ngwpdyn1
      integer(long), dimension(npg1-1,gwpdim1), intent(out) :: gwpdyn
      integer(long), dimension(gwpdim1), intent(in)         :: gwpdep
      integer(long)              :: e,n,e1

      do e = 1,gwpdim1
         e1 = gwpdep(e)
         if (e1 .gt. 0) then
            do n=1,npg1-1
               if (gwpdyn(n,e) .eq. 0) then
                  gwpdyn(n,e) = 2
                  ngwpdyn1 = ngwpdyn1-1
               endif
            enddo
         endif
      enddo

      return
      end subroutine lindepinact

!#######################################################################
!
! GWPs interacting with caps are controlled as followed:
! 1. in saddle1 (gh_elementsmod) the flag lgwpcap(f,e,s) set true. if the 
!    centre coordinate is in a region with significant cap strength
! 2. in gwpcapinact (gwpdepdynmod) if lcapgwp = true then either:
!      a. GWP is classical if carrying signifcant density (dicht2 >0)
!      b. GWP is kept fixed in phase space if density has been absorbed
!         by cap (dicht2 = 0)
!    This is done by changing gwpdyn(f,e)
! 3. gwpdyn is used in subgdot1 (gaussian) to define dtpsi
!
!#######################################################################

      subroutine gwpcapinact(m,s,gwpdyn,dicht2,npg1,gwpdim1,&
                 dgwp,dim1)

      implicit none

      integer(long), intent(in)  :: npg1,gwpdim1,m,s,dim1
      integer(long)              :: e,n
      integer(long), dimension(npg1-1,gwpdim1), intent(out) :: gwpdyn
      complex(dop), dimension(dim1,dim1), intent(in)        :: dicht2
      complex(dop), dimension(gwpdim1,dim1), intent(in)     :: dgwp
      complex(dop), dimension(gwpdim1,gwpdim1)              :: gdicht2

      if (psitype .eq. 0 .or. psitype .eq. 5) then
         call simtran1btz(dicht2,dgwp,gdicht2,dim1,gwpdim1)
      else
         call cpqxz(dicht2,gdicht2,gwpdim1)
      endif

      do e = 1,gwpdim1
! Set static if coefficient small
! Set GWP to be classical if it has passed a CAP.
          do n=1,npg1-1
             if (gwpdyn(n,e) .eq. -2) cycle
             if (lcapgwp(n,e,s))then
                if (abs(gdicht2(e,e)) .lt. tiny) then
                   gwpdyn(:,e) = -2
                else 
                   gwpdyn(:,e) = -1
                   gwpcxm(m) = 1
                endif
             endif
          enddo
      enddo

      return
      end subroutine gwpcapinact

!#######################################################################

      subroutine wrgwpdyn(gwpdyn,gwpdynold,cpdim,ngwpdyn1,time,m1,s1)

      implicit none

      integer(long), intent(in)                   :: cpdim,m1,s1,ngwpdyn1
      integer(long), dimension(cpdim), intent(in) :: gwpdyn,gwpdynold
      integer(long)                               :: p1,ig,jg,m,s
      real(dop),intent(in)                        :: time
      logical(kind=4),save                        :: init=.true.
      character(len=19)                           :: ActiveFormat
      character(len=120)                          :: filename

      
!-----------------------------------------------------------------------
! initialize activegwp files and write headers
!-----------------------------------------------------------------------
      if (init) then
         do s=1,nstate
            do m=1,nmode
               if (.not. gwpm(m)) cycle

               filename = '  '
               if (nmode .eq. 1 .or. (nmode .eq. 2 .and. leb)) then
                  write(filename,'(2a,i5)') name(1:laenge),'/activegwp'

               else if (nstate .gt. 1) then
                  write(filename,'(2a,i5,a,i5)') name(1:laenge),'/activegwp_m',m,'_s',s
                  call rmblnk(filename)

               else
                  write(filename,'(2a,i5)') name(1:laenge),'/activegwp_m',m
                  call rmblnk(filename)
               endif
               open(iactive,file=filename,status='old',form='formatted')

               ig=npg(m)
               jg=gwpdim(m,s)

               write(iactive,'(a)',advance='no')' Time[fs]   '
               do p1=1,jg
                  write(ActiveFormat,'(i1,''  '')')p1
                  if (p1>9) write(ActiveFormat,'(i2,'' '')')p1
                  if (p1>99) write(ActiveFormat,'(i3)')p1
                  write(iactive,'(a)',advance='no')  &
                              ActiveFormat(1:3)//repeat(' ',ig-nspfdof(m))
               enddo
               write(iactive,*)

               close(iactive)
            enddo
         enddo
         init=.false.
      endif

!-----------------------------------------------------------------------
! open file and write active GWP data
!-----------------------------------------------------------------------
      filename = '  '
      if (nmode .eq. 1 .or. (nmode .eq. 2 .and. leb)) then
         write(filename,'(2a,i5)') name(1:laenge),'/activegwp'

      else if (nstate .gt. 1) then
         write(filename,'(2a,i5,a,i5)') name(1:laenge),'/activegwp_m',m1,'_s',s1
         call rmblnk(filename)

      else
         write(filename,'(2a,i5)') name(1:laenge),'/activegwp_m',m1
         call rmblnk(filename)
      endif
      open(iactive,file=filename,status='old',position='append',form='formatted')

      ig=npg(m1)
      jg=gwpdim(m1,s1)
      ActiveFormat='(f9.3,''   '',    i1)'
      write(ActiveFormat(13:16),'(i4)')(ig-1)*jg

      write(iactive,ActiveFormat) time/fs,abs(gwpdyn)
      call myflush(iactive)

      close(iactive)

!-----------------------------------------------------------------------
! write changes in active GWPs to log file
!-----------------------------------------------------------------------
      if (any(gwpdyn/=gwpdynold) .and. lintstep) then
         if (nstate .gt. 1) then
            write(ilog,'(a,f8.3,a,2i4,4x,2(i4,a))') &
                   'Time ',time/fs,' Mode, State =',m1,s1, &
                   ngwpdyn1,' of ',cpdim,' Gaussian DoFs active'
         else if (nmode .gt. 1) then
            write(ilog,'(a,f8.3,a,i4,4x,2(i4,a))') &
                   'Time ',time/fs,'  Mode =',m1, &
                    ngwpdyn1,' of ',cpdim,' Gaussian DoFs active'
         else
             write(ilog,'(a,f8.3,2(i4,a))') &
                    'Time ',time/fs,ngwpdyn1,' of ',&
                    cpdim,' Gaussian DoFs active'
         endif
      endif

      flush(ilog)

      return
      end subroutine wrgwpdyn

!#######################################################################

      subroutine setgwpdyn1ms(gs2,gwpdyn,gwpdim1,dim1,npg1,dicht2,&
                 gsinv,dgwp,m,s,time,cpdim,gwpdynold)


      implicit none

      integer(long),intent(in)  :: gwpdim1,dim1,npg1,m,s,cpdim
      integer(long),save        :: tid=0
      integer(long)             :: e,ndyn,ndep,gdof
      complex(dop),dimension(gwpdim1,gwpdim1),intent(in)    :: gs2
      integer(long),dimension(cpdim),intent(out)            :: gwpdyn
      integer(long),dimension(cpdim),intent(in)             :: gwpdynold
      complex(dop),dimension(cpdim,cpdim)                   :: cp
      complex(dop), dimension(gwpdim1,gwpdim1), intent(in)  :: gsinv
      complex(dop), dimension(dim1,dim1), intent(in)        :: dicht2
      complex(dop), dimension(gwpdim1,dim1), intent(in)     :: dgwp
      real(dop),intent(in)                                  :: time

      integer(long) :: ndel,nclass,nstatic
      logical(kind=4), dimension(cpdim)           :: delgwp

      integer(long) :: oldmax
      real(dop)     :: minc

      call get_timer(tid,'Setgwpdyn1ms')
      call start_timer(tid)

!+-----------------------------------------------------------------------+
!  Calculate C matrix 
!+-----------------------------------------------------------------------+
      call gwpcmat(cp,dicht2,gs2,gsinv,npg1,gwpdim1,dim1,dgwp,psitype)

      nclass = 0
      ndel = 0
      delgwp = .false.
      minc = 999.0_dop

      gdof = nspfdof(m)
      do e = 1,cpdim
! Checks based on the diagonal values of the C-matrix
         minc = min(minc,abs(cp(e,e)))
         if(abs(cp(e,e)).lt. 10.0_dop*epsdyn)then
            if(abs(cp(e,e)) .lt. epsdyn .or. gwpdynold(e) .ge. 1)then
               delgwp(e) = .true.
               ndel = ndel + 1
            endif
         endif
      enddo
      nclass = count(gwpdynold == -1)
      ndyn = count(gwpdynold == 0)
      ndep = count(gwpdynold == 2)
      nstatic = count(gwpdyn == -2)

! easy cases - all or none
      if (ndel .eq. 0) then
         ngwpdyn(m,s)=cpdim-nclass-nstatic
         do e=1,cpdim
            if (gwpdyn(e) .ge. 0) gwpdyn(e)=0
         enddo
         
      else if (ndel .eq. cpdim) then
         ngwpdyn(m,s)=0
         do e=1,cpdim
            if (gwpdyn(e) .eq. 0) gwpdyn(e)=1
         enddo
         
      else
!-----------------------------------------------------------------------
! Look at Cpmat elements and choose active GWPs
! if |CP| > epsdyn make active
! If |CP| < 0.01*epsdyn make inactive. This means that once active GWPs
! stay active until |CP| drops below the lower threshold
! gwpdyn(p1-1,ig) = 0 active
!                 = 1 inactive
!                 = 2 inactive due to linear dependence (set in lindepinact)
!                 = -1 always inactive
!                 = -2 always static (set in capgwpinact)
! Any parameters flagged as always inactive remain so.
! If CX formulation used, inactive GWPs follow classical trajectories
!-----------------------------------------------------------------------
         ngwpdyn(m,s)=0
         do e = 1,cpdim
            if (gwpdyn(e) .lt. 0) then
               continue
            else if (.not. delgwp(e)) then
               gwpdyn(e) = 0
            else
               gwpdyn(e) = 1
            endif
         enddo
         
         if (lwholeg) call chkwholeg(gwpdyn,npg(m),gwpdim(m,s))
         
      endif

      ngwpdyn(m,s) = count(gwpdyn==0)
!
! collect statistics
!
      oldmax = maxngdyn(m,s)
      maxngdyn(m,s) = max(maxngdyn(m,s),ngwpdyn(m,s))
      if (maxngdyn(m,s) .gt. oldmax) maxtgdyn(m,s) = time
      if(ndel .ne. 0 .and. ndel .ne. cpdim) &
           mincgdyn(m,s) = max(mincgdyn(m,s),minc)
       
      call stop_timer(tid)

      return
      end subroutine setgwpdyn1ms

!#######################################################################

      subroutine chkwholeg(gwpdyn,npg,gwpdim)

      implicit none 

      integer(long),intent(in)                          :: npg,gwpdim
      integer(long),dimension(npg-1,gwpdim),intent(out) :: gwpdyn
      integer(long)                                     :: ig
      
      do ig=1,gwpdim
         if (any(gwpdyn(:,ig)==0)) gwpdyn(:,ig)=0
      enddo

      return
      end subroutine chkwholeg

!#######################################################################

      subroutine gwplindep1ms(gs2,gwpdep,gwpdepold,gwpdim1,m,s)

      implicit none

      integer(long)                                         :: e,e1,lwork,info,&
                                                               ndep
      integer(long), save                                   :: tid=0
      integer(long), intent(in)                             :: gwpdim1,m,s
      integer(long), dimension(gwpdim1), intent(out)        :: gwpdep
      integer(long), dimension(gwpdim1), intent(in)         :: gwpdepold
      complex(dop), dimension(gwpdim1,gwpdim1), intent(in)  :: gs2

      real(dop), dimension(gwpdim1)               :: eval,project
      integer(long), dimension(gwpdim1)           :: iorder
      complex(dop), dimension(gwpdim1,gwpdim1)    :: evek
      real(dop), dimension(3*gwpdim1)             :: work
      complex(dop), dimension(3*gwpdim1)          :: workc

      call get_timer(tid,'GWPlindep1ms')
      call start_timer(tid)

      gwpdep = 0 

      evek = gs2
      lwork=3*gwpdim1
      call zheev('V','U',gwpdim1,evek,gwpdim1,eval,workc,lwork,work,info)
      if (info .ne. 0) then
         routine = 'gwplindep1ms'
         message = 'Cannot diagonalize overlap matrix'
         call errormsg
      endif

      ndep = count(eval .lt. epsld)

      if(ndep .eq. 0)then
         call stop_timer(tid)
         return
      endif

! project GWPS onto lowest (linearly dependent) eigenvectors
      do e=1,gwpdim1
         iorder(e) = e
         do e1=1,ndep
            project(e)=project(e) + dble(evek(e,e1)*dconjg(evek(e,e1)))
         enddo
      enddo
       
! order
      call ibubble_down(project,iorder,gwpdim1)

      do e = gwpdim1,gwpdim1-ndep+1
         e1 = iorder(e)
         gwpdep(e1) = 1
      enddo

      ngwpdep(m,s) = count(gwpdep/=0)

      if (any(gwpdep /= gwpdepold)) then
         if (lresint) dores = .true.
         if (lintstep) then
            if (lmult) then
               write(ilog,'(2(a,i3),a,i5)')  &
                  'Linear Dependence mode: ',m,' state: ',s,': ',ngwpdep(m,s)
            else
               write(ilog,'(a,i3,a,i5)')  &
                  'Linear Dependence mode: ',m,': ',ngwpdep(m,s)
            endif
         endif
      endif

! collect statistics
      maxngdep(m,s) = max(maxngdep(m,s),ngwpdep(m,s))

      call stop_timer(tid)

      return
      end subroutine gwplindep1ms

!#######################################################################

      subroutine gwplindep1msovl(gs2,gwpdep,gwpdepold,gwpdim1,m,s)

      implicit none

      integer(long)                                         :: e,e1
      integer(long), save                                   :: tid=0
      integer(long), intent(in)                             :: gwpdim1,m,s
      integer(long), dimension(gwpdim1), intent(out)        :: gwpdep
      integer(long), dimension(gwpdim1)                     :: gwpdepold
      real(dop)                                             :: ovl
      complex(dop), dimension(gwpdim1,gwpdim1), intent(in)  :: gs2

      call get_timer(tid,'GWPlindep1msovl')
      call start_timer(tid)

! make linear dependence static
      do e=1,gwpdim1
         do e1=e+1,gwpdim1
            ovl = dble(gs2(e1,e))
            maxogdep(m,s) = max(maxogdep(m,s),ovl)
! use lower threshold if already fixed
            if ((gwpdepold(e1) .eq. e) .and. (ovl .gt. (ovlthr-0.05_dop))) then
               gwpdep(e1)=e
! fix
            else if (ovl .gt. ovlthr) then
               gwpdep(e1)=e
            endif
         enddo
      enddo
      ngwpdep(m,s) = count(gwpdep/=0)

      if (any(gwpdep /= gwpdepold)) then
         if (lresint) dores = .true.
         if (lintstep) then
            if (lmult) then
               write(ilog,'(2(a,i3),a,i5)')  &
                  'Linear Dependence mode: ',m,' state: ',s,': ',ngwpdep(m,s)
            else
               write(ilog,'(a,i3,a,i5)')  &
                  'Linear Dependence mode: ',m,': ',ngwpdep(m,s)
            endif
         endif
      endif

! collect statistics
      maxngdep(m,s) = max(maxngdep(m,s),ngwpdep(m,s))

      call stop_timer(tid)

      return
      end subroutine gwplindep1msovl

!#######################################################################

      subroutine wrgwpdyn_final

!+-----------------------------------------------------------------------+
!     Writes information on dynamic GWPs to log file
!+-----------------------------------------------------------------------+

use psidef
use runpropmod
use channels


      implicit none

      integer(long) :: m,s,ncpdim

      if (any(maxngdep .gt. 0)) then
         write(ilog,'(a)') '**** GWP linear dependence ****'
         do m=1,nmode
            if (.not. gwpm(m)) cycle
            
            if (lmult) then
               do s=1,nstate
                  write(ilog,'(i5,a,2i5)') maxngdep(m,s), &
                   ' Linear Dependencies Avoided in mode / state',m,s
                  write(ilog,'(a,f8.5)') 'Maximum overlap :',maxogdep(m,s)
               enddo
            else
               write(ilog,'(i5,a,i5)') maxngdep(m,1), &
                   ' Linear Dependencies Avoided in mode ',m
               write(ilog,'(a,f8.5)') 'Maximum overlap :',maxogdep(m,1)
            endif
         enddo
      endif

      write(ilog,*)

      if (any(igwpdynm .eq. 1)) then
         write(ilog,'(a)') '**** GWP basis dynamic coupling ****'
         if (lmult) then
            write(ilog,'(a)') ' mode  state      Ndyn /   N      Tmax Cmin'
         else
            write(ilog,'(a)') ' mode      Ndyn /   N      Tmax Cmin'
         endif
      endif
      do m=1,nmode
         if (.not. gwpm(m)) cycle
         if (igwpdynm(m) .ne. 1) cycle

         if (lmult) then
            do s=1,nstate
               ncpdim = (npg(m)-1)*gwpdim(m,s)
               write(ilog,'(2i5,6x,i5,a,i5,3x,f8.3,e14.6)') m,s,&
                     maxngdyn(m,s),' /',ncpdim,&
                     maxtgdyn(m,s)/fs,mincgdyn(m,s)
            enddo
         else
            ncpdim = (npg(m)-1)*gwpdim(m,1)
            write(ilog,'(i5,6x,i5,a,i5,3x,f8.3,e14.6)') m,&
                  maxngdyn(m,1),' /',ncpdim,&
                  maxtgdyn(m,1)/fs,mincgdyn(m,1)
         endif
      enddo

      write(ilog,*)

      return
      end subroutine wrgwpdyn_final

!#######################################################################
! --- initialise gwpdyn arrays so all GWPs are flagged as dynamic
!
! if gwpdyn(:,e) =  0 : coupled
!                   1 : uncoupled
!
!#######################################################################

      subroutine initgwpdyn(m,gwpdyn,npg1,gwpdim1,igwpdyn,ngwpdyn,&
                 gwpdep,ngwpdep,gwpdynold,linit)

      implicit none

      integer(long), intent(in)                             :: m,npg1,gwpdim1,&
                                                               igwpdyn
      integer(long), dimension(npg1-1,gwpdim1), intent(in)  :: gwpdynold
      integer(long), dimension(npg1-1,gwpdim1), intent(out) :: gwpdyn
      integer(long), dimension(gwpdim1), intent(out)        :: gwpdep
      integer(long), intent(out)                            :: ngwpdyn, ngwpdep
      integer(long)                                         :: e,n
      logical(kind=4), intent(in)                           :: linit

!-----------------------------------------------------------------------
! flag all GWP parameters to be initially active or inactive 
! igwpdyn = 0 all active
!           1 dynamic selection
!           2 all inactive
!-----------------------------------------------------------------------
      if (igwpdyn .eq. 0) then
         gwpdyn = 0
         ngwpdyn = (npg1-1)*gwpdim1
      else if (igwpdyn .eq. 1) then
         gwpdyn = 1
         ngwpdyn = 0
      else if (igwpdyn .eq. 2) then
         gwpdyn =  1
         ngwpdyn = 0
      endif
      ngwpdep=0
      gwpdep=0

!-----------------------------------------------------------------------
! if initial call, any GWP DOFs flagged as "classical" set parameter status to -1
! else copy permament classical / static from old array
!-----------------------------------------------------------------------
      if (linit) then
         do n=1,nspfdof(m)
            do e=1,gwpdim1
               if (gwpmodelm(m) .eq. 1) gwpdyn(n,e)=-1
            enddo
         enddo
      else
         do n=1,nspfdof(m)
            do e=1,gwpdim1
               if (gwpdynold(n,e) .lt. 0) gwpdyn(n,e)=gwpdynold(n,e)
            enddo
         enddo
      endif

      return
      end subroutine initgwpdyn

      end module gwpdepdynmod

