      module fcmattrmod

use decimal, only: long, dop

use global
use griddatmod
use psidef

      integer(long) :: m,s
      complex(dop), dimension(:), allocatable :: psi1
      save psi1,m,s

      end module fcmattrmod

!#######################################################################

      module f1com

use decimal, only: long, dop

      INTEGER(long) :: ncom
      INTEGER(long), parameter  :: NMAX = 50000
      REAL(dop), dimension(NMAX) :: pcom,xicom

      save ncom,pcom,xicom

      end module f1com

!#######################################################################

      module gwpoptimmod

use decimal, only: long, dop
use global
use globmemmod
use mmlib, only: qqxxzz, qqaxzz
use op2lib, only: addmxxzo, sum2qxxz
use op1lib, only: normvxz
use xvlib, only: xvixdzo, xvixddo
use integrat, only: runrk5,runrk5f
use griddatmod
use psidef
use runpropmod
use channels
use datenmod
use logdat

use densitymod, only: density
use gh_elementsmod, only: gh_elements
use phihphimod, only: phihphi,phihunphi
use mfieldsmod, only: mfields
use hphimod, only: hphi
use hpsimod, only: gwpcxm

use genpsimod, only: genpsi
use function, only: funkr,ddmfunkr
use gwplib, only: renormdgwp, renormgwpa

      implicit none
      private
      public :: gwpoptdyn,gwpoptyp

! following routines are not used anywhere and are declared public
! to prevent compilation warnings
      public :: dprojsum,dymatsum,init_simplex,simplexopt

      contains

!#######################################################################

      subroutine gwpoptdyn(psi,dtpsi,gs2,out1,gwpdyn,gwpdep)

      complex(dop), dimension(dgldim), intent(inout) :: psi
      complex(dop), dimension(dgldim), intent(out)   :: dtpsi
      complex(dop), dimension(gs2matdim), intent(in) :: gs2
      real(dop), intent(in)                          :: out1
      logical(kind=4)             :: restartrk5 = .true.
      logical(kind=4)             :: liork0
      real(dop)                   :: intperiod,time0,dtold,accold,&
                                     norm,minrk5old
      real(dop), parameter        :: small =1.0d-4
      integer(long)               :: evaltype,mode,i,m,s,nfunc
      integer(long), dimension(gwpinfodim), intent(out) :: gwpdep
      integer(long), dimension(gwpdyndim), intent(out) :: gwpdyn

!-----------------------------------------------------------------------
! find GWPs that are initially populated and freeze them
! this is done by pretending they are linearly dependent functions
! and setting gwpdep = 1
!-----------------------------------------------------------------------
      ngwpdep = 0
      nfunc = 0
      do s=1,nstate
         do m=1,nmode
            if (.not. gwpm(m)) cycle

            if (dentype .eq. 0) then
               call setgwpoptdyn(gwpdep(zgwpinfo(m,s)),psi(1),psi(zdgwp(m,s)),&
                    npg(m),vdim(m,s),dim(m,s),ndim(m,s),small,&
                    ngwpdep(m,s),gwpdyn(zgwpdyn(m,s)),ngwpdyn(m,s))
               nfunc = nfunc + ngwpdep(m,s)
            else if (dentype .eq. 2) then
               call d2setgwpoptdyn(gwpdep(zgwpinfo(m,s)),psi(1),&
                    npg(m),vdim(m,s),dim(m,s),ndim(m,s),small,&
                    ngwpdep(m,s),gwpdyn(zgwpdyn(m,s)),ngwpdyn(m,s))
               nfunc = nfunc + ngwpdep(m,s)
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! if nfunc = 0, no functions to optimise
!-----------------------------------------------------------------------
      if (nfunc .eq. 0) then
         routine = 'Gwpoptim (gwpoptimmod)'
         message = 'No GWP functions to optimise'
         call warnmsg1
         return
      endif

!-----------------------------------------------------------------------
! Give a small value to zero coefficients.
!-----------------------------------------------------------------------
      if (dentype .eq. 0) then
         do i=1,adim
            norm = dble(dconjg(psi(i))*psi(i))
            if (norm .lt. 1.0d-4) psi(i) = 1.0d-04
         enddo

!-----------------------------------------------------------------------
! Normalise wavefunction
!-----------------------------------------------------------------------
         if (psitype .eq. 4) then
            call renormgwpa(psi,gs2)
         else
            call normvxz(psi,norm,adim)
            call xvixdzo(norm,psi,adim)
         endif
      else if (dentype .eq. 2) then
         call d2setoptcoeffs(psi,block(1))
      endif

!-----------------------------------------------------------------------
! do integration
!-----------------------------------------------------------------------
      write(ilog,'(/,a)') &
          '*** Optimising initially unpopulated GWPs ***'
      write(ilog,'(a,i5)') ' No. functions not optimised : ',nfunc

      intperiod = min(out1,0.5*fs)
      time0 = 0.0_dop
      evaltype = -1
      mode = 0
! save integrator values for real propagation
      dtold = dt
      accold = accuracy
      liork0 = liork
      minrk5old = minrk5

      if(lddmctdh)then
         call ddmfunkr(time0,psi,dtpsi)
      else
         call funkr(time0,psi,dtpsi)
      endif

      if (lintstep) write(istp,'(a)') 'GWP optimisation '

      dt = 0.001*fs
      accuracy = 0.0001
      gwpcxm(:) = 0
      minrk5 = 0.001*fs
      if (psitype .ne. 4) liork = .true.

      call runrk5(psi,dtpsi,intperiod,time0,evaltype,mode,restartrk5)

      if (lintstep) write(istp,'(a)') 'End GWP optimisation '

! restore integrator values
      dt = dtold
      accuracy = accold
      liork = liork0
      minrk5 = minrk5old

!-----------------------------------------------------------------------
! regenerate coefficients and SPFs from new GWP positions
!-----------------------------------------------------------------------
      call genpsi(psi,mc(mcspf1d),mr(mrort),mi(mijindx),agmat,1)


      end subroutine gwpoptdyn

!#######################################################################

      subroutine setgwpoptdyn(gwpdep,psi,dgwp,npg1,&
                 vdim1,dim1,ndim1,small,ngwpdep1,gwpdyn,ngwpdyn1)

      integer(long), intent(in)    :: npg1,vdim1,dim1,ndim1
      integer(long), intent(out)   :: ngwpdep1,ngwpdyn1
      real(dop), intent(in)        :: small
      integer(long), dimension(dim1), intent(out)        :: gwpdep
      integer(long), dimension(npg1-1,dim1), intent(out)   :: gwpdyn
      complex(dop), dimension(vdim1,dim1,ndim1), intent(in) :: psi
      complex(dop), dimension(dim1,dim1), intent(inout)  :: dgwp
      real(dop)       :: norm
      integer(long)   :: g,v,e,n
      logical(kind=4) :: freeze

!-----------------------------------------------------------------------
! turn all couplings on
!-----------------------------------------------------------------------
      gwpdyn(:,:) = 0
      ngwpdyn1 = (npg1-1)*dim1

!-----------------------------------------------------------------------
! initially freeze all functions and then look at coefficients.
! If no population found for a function, unfreeze it.
!
! as complete SPF basis used (dim1 = dim1) all GWPs are defined in 
! intial SPFs and can be populated by changing small coefficients.
!-----------------------------------------------------------------------
      gwpdep(:) = 1
      ngwpdep1 = dim1

      do g=1,dim1
         freeze = .false.

         if (psitype .eq. 0) then
            do e=1,dim1
               norm = dble(dconjg(dgwp(g,e))*dgwp(g,e))
               if (norm .lt. small) then
                  dgwp(g,e) = small
               else
                  freeze = .true.
               endif
            enddo

         else
! otherwise look at coefficients. The small coefficients are given values 
! after all modes checked.
            do n=1,ndim1
               do v=1,vdim1
                  norm = dble(dconjg(psi(v,g,n))*psi(v,g,n))
                  if (norm .gt. small) freeze = .true.
               enddo
            enddo

         endif

         if (.not. freeze) then
            ngwpdep1 = ngwpdep1-1
            gwpdep(g) = 0
         endif
      enddo

      return
      end subroutine setgwpoptdyn

!#######################################################################
!
! Dentype = 2 only available with an rho-MCTDH/G structure
!
!#######################################################################

      subroutine d2setgwpoptdyn(gwpdep,psi,npg1,&
                 vdim1,dim1,ndim1,small,ngwpdep1,gwpdyn,ngwpdyn1)

      integer(long), intent(in)    :: npg1,vdim1,dim1,ndim1
      integer(long), intent(out)   :: ngwpdep1,ngwpdyn1
      real(dop), intent(in)        :: small
      integer(long), dimension(dim1), intent(out)        :: gwpdep
      integer(long), dimension(npg1-1,dim1), intent(out)   :: gwpdyn
      complex(dop), dimension(vdim1,dim1,ndim1,vdim1,dim1,ndim1), &
                                                        intent(in) :: psi
      real(dop)       :: norm
      integer(long)   :: g,v,n
      logical(kind=4) :: freeze

!-----------------------------------------------------------------------
! turn all couplings on
!-----------------------------------------------------------------------
      gwpdyn(:,:) = 0
      ngwpdyn1 = (npg1-1)*dim1

!-----------------------------------------------------------------------
! initially freeze all functions and then look at coefficients.
! If no population found for a function, unfreeze it.
!-----------------------------------------------------------------------
      gwpdep(:) = 1
      ngwpdep1 = dim1

      do g=1,dim1
         freeze = .false.

! Look at density matrix coefficients. The small coefficients are given values 
! after all modes checked.
         do n=1,ndim1
            do v=1,vdim1
               norm = dble(dconjg(psi(v,g,n,v,g,n))*psi(v,g,n,v,g,n))
               if (norm .gt. small) freeze = .true.
            enddo
         enddo

         if (.not. freeze) then
            ngwpdep1 = ngwpdep1-1
            gwpdep(g) = 0
         endif
      enddo

      return
      end subroutine d2setgwpoptdyn

!#######################################################################

      subroutine d2setoptcoeffs(psi,block1)

      implicit none

      integer(long), intent(in)    :: block1
      complex(dop), dimension(block1,block1), intent(inout) :: psi

      real(dop), dimension(block1) :: diagpsi
      real(dop) :: norm

      integer(long) :: e,e1

      do e=1,block1
         diagpsi(e) = dble(psi(e,e))
         diagpsi(e) = dsqrt(diagpsi(e))
         if (diagpsi(e) .lt. 1.0d-4) diagpsi(e) = 1.0d-04
      enddo

      call normvxd(diagpsi,norm,block1)
      call xvixddo(norm,diagpsi,block1)

      do e=1,block1
         do e1=1,block1
            psi(e1,e) = diagpsi(e1)*diagpsi(e)
         enddo
      enddo

      return
      end subroutine d2setoptcoeffs

!#######################################################################

      subroutine gwpoptyp(psi)

      implicit none

      integer(long) :: m,s
      complex(dop), dimension(dgldim),intent(inout)     :: psi
      logical(kind=4), dimension(nmode) :: loptim

!-----------------------------------------------------------------------
! calculate density and check only 1 spf populated
!-----------------------------------------------------------------------
      call density(psi,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      do m=1,nmode
         loptim(m) = .false.
         do s=1,nstate
            if (dim(m,s) .eq. 1) cycle
            if (abs(dicht2(dmat(m,s)) - 1.0_dop) .lt. tiny) loptim(m) = .true.
         enddo
      enddo

!-----------------------------------------------------------------------
! optimise (minimise) Y-matrix
!-----------------------------------------------------------------------
      do s=1,nstate
         do m=1,nmode
            if (loptim(m)) call gwpoptim1(psi(zetf(m,s)),m,s,npg(m), &
                                dim(m,s),phidim(m),nspfdof(m))
         enddo
      enddo

      end subroutine gwpoptyp

!#######################################################################

      subroutine gwpoptim1(psi,m1,s1,npg1,dim1,phidim1,gdof)

use global
use channels

use fcmattrmod

      implicit none

      integer(long) :: n,e,nvar,iter,zeig,gdof
      integer(long), intent(in) :: m1,s1,dim1,npg1,phidim1
      complex(dop), dimension(phidim1),intent(inout)     :: psi 
      real(dop), dimension(npg1-1,dim1-1)     :: gwppar
!      real(dop), dimension((npg1-1)*dim1,npg1*dim1)   :: p
!      real(dop), dimension(npg1*dim1)   :: y
      real(dop)   :: fret,tol

      allocate(psi1(phidim1))

      m=m1
      s=s1
!
! Copy wavefunction to optimise
!
      do e=1,phidim1
         psi1(e) = psi(e)
      enddo
!
! do not optimise first GWP or scalar parameters or widths
!
      zeig=dim1+gdof
      do e=1,dim1-1
         do n=1,npg1-1
            zeig=zeig+1
            gwppar(n,e) = dble(psi1(zeig))
         enddo
      enddo

!      fret=fymatsum(gwppar)
      fret=fcmattr(gwppar)
      print*,'Initial Value',fret

      nvar = (npg1-1)*(dim1-1)
      iter=100
      tol = 1.0d-10
!      call cgmin(gwppar,nvar,tol,iter,fret,fymatsum,dymatsum,&
!              ilog,-9999.0_dop,1,mc,mr)
      call cgmin(gwppar,nvar,tol,iter,fret,fcmattr,dcmattr,&
              ilog,-9999.0_dop,1,mc,mr)

!      iter = 100
!      call init_simplex(p,y,gwppar,nvar,fcmattr,psi,m,s)
!      call simplexopt(p,y,nvar,0.001_dop,fcmattr,iter,psi,mc,m,s)
!      write(6,'(a,i5)') 'converged in Iterations: ',iter

!      fret=fymatsum(gwppar)
      fret=fcmattr(gwppar)
      print*,'Final Value',fret
!
! Copy optimised wavefunction to psi
!
      do e=1,phidim1
         psi(e) = psi1(e)
      enddo

      if (allocated(psi1)) deallocate(psi1)

      return
      end subroutine gwpoptim1

!#######################################################################

      real(dop) function fprojsum(gwppar)

use global
use griddatmod
use psidef

use gwplib, only: gwp_param1m_a
use gh_elementsmod, only: gh_elements1m

use fcmattrmod

      implicit none

      integer(long) :: phidim1
      real(dop), dimension(maxphidim), intent(inout) :: gwppar

      call restore_gwp(gwppar,psi1,npg(m),dim(m,s),phidim(m),nspfdof(m))

      phidim1 = phidim(m)+gbfdim(m)
      call gwp_param1m_a(m,psi1,agmat,gfgwp, &
           aigwp,zgwp,mr(mrqcent),zetf(m,1), phidim1,1)

! this will not work for MC integration - f1point etc. set to 0
      call gh_elements1m(m,gh,hung,ghderiv1, agmat,gfgwp,aigwp,zgwp,&
           mr(mrqcent),1,0,gs2,nspfdof(m))

      call fprojsum1(fprojsum,gsinv,dim(m,s),m,s, &
           dicht2,hung)
      print*,'CMATtr',fprojsum

      return
      end function fprojsum

!#######################################################################

      subroutine restore_gwp(gwppar,psi,npg1,dim1,phidim1,gdof)

      implicit none

      integer(long), intent(in) :: npg1,dim1,phidim1,gdof
      integer(long) :: n,e,zeig
      complex(dop), dimension(phidim1) :: psi
      real(dop), dimension(npg1-1,dim1-1) :: gwppar
      real(dop) :: repsi,impsi

      zeig=dim1+gdof
      do e=1,dim1-1
         do n=1,gdof
            zeig=zeig+1
            impsi=dimag(psi(zeig))
            repsi=gwppar(n,e)
            psi(zeig) = dcmplx(repsi,impsi)
         enddo
      enddo

      return
      end subroutine restore_gwp

      
!#######################################################################

      subroutine fprojsum1(fprojsum,gsinv,dim1,m,s,dicht2,hung)

use global
use griddatmod
use psidef
use operdef
use hpsimod

use gaussian, only: gwpymat

      implicit none

      integer(long), intent(in) :: dim1,m,s
      complex(dop), dimension(gsmatdim), intent(in)  :: gsinv
      complex(dop), dimension(dmatdim), intent(in)   :: dicht2
      complex(dop), dimension(hungdim), intent(in)   :: hung
      complex(dop), dimension(dim1,dim1) :: temp1,temp2,sumtmp
      complex(dop) :: cprojsum
      real(dop) :: fprojsum

      sumtmp = 0.0_dop
!
! should multiply correlated terms by hloch**2
!
!      do k=1,kzahl(1)
!         temp1 = 0.0_dop
!         temp2 = 0.0_dop
!         call qqxxzz(gsinv(gsmat(m,s)),gh(ghmat(k,m)),temp1,dim1)
!         call qqaxzz(gh(ghmat(k,m)),temp1,temp2,dim1)
!         koeff2 = koeff(k)*koeff(k)
!         call xmxxzza(koeff2,temp2,sumtmp,dim1,dim1)
!      enddo
      if (khzahl(m,s,1) .gt. 0) then
         temp1 = 0.0_dop
         temp2 = 0.0_dop
         call qqxxzz(gsinv(gsmat(m,s)),hung(hungmat(m,s,1)),temp1,dim1)
         call qqaxzz(hung(hungmat(m,s,1)),temp1,temp2,dim1)
         call addmxxzo(temp2,sumtmp,dim1,dim1)
      endif

!      call unitqxz(unit,dim1)
!      call sum2qxxz(unit,sumtmp,cprojsum,dim1)
      call sum2qxxz(dicht2(dmat(m,s)),sumtmp,cprojsum,dim1)
      fprojsum=-dble(cprojsum)
!      fprojsum= -dsqrt(fprojsum)

      return
      end subroutine fprojsum1

!#######################################################################

      subroutine dprojsum(gwppar,dgwppar,nvar)

use global
use maxv, only: maxdim

      implicit none

      integer(long)                                 :: n
      integer(long), intent(in)                     :: nvar
      real(dop), dimension(nvar), intent(in)        :: gwppar
      real(dop), dimension(nvar), intent(out)       :: dgwppar
      real(dop)                                     :: err,h
      real(dop), dimension(4*maxdim*maxdim*2)       :: work

      do n=1,nvar
         h=0.001
         dgwppar(n)=dfnummc(fprojsum,gwppar,nvar,n,h,err,work)
      enddo

      return
      end subroutine dprojsum

!#######################################################################

      real(dop) function fymatsum(gwppar)

use global
use griddatmod
use psidef

use gwplib, only: gwp_param1m_a
use gh_elementsmod, only: gh_elements1m

use fcmattrmod

      implicit none

      integer(long) :: phidim1
      real(dop), dimension(maxphidim), intent(inout) :: gwppar

      call restore_gwp(gwppar,psi1,npg(m),dim(m,s),phidim(m),nspfdof(m))

      phidim1 = phidim(m)+gbfdim(m)
      call gwp_param1m_a(m,psi1,agmat,gfgwp, &
           aigwp,zgwp,mr(mrqcent),zetf(m,1),phidim1,1)
! this will not work for MC integration - f1point etc. set to 0
      call gh_elements1m(m,gh,hung,ghderiv1,agmat,gfgwp,aigwp,zgwp,&
           mr(mrqcent),1,0,gs2,nspfdof(m))

      call fymatsum1(fymatsum,gs2,gsinv,npg(m),gwpdim(m,s),m,s, &
           psi1,zetf(m,s),phidim(m),dicht2,mc(mchloch), &
           gh,hung)

      return
      end function fymatsum

!#######################################################################

      subroutine fymatsum1(ymatsum,gs2,gsinv,npg1,dim1,m,s,psi, &
                 zetf1,phidim1,dicht2,hloch,gh,hung)

use global
use griddatmod
use psidef
use hpsimod

use gaussian, only: gwpymat

      implicit none

      integer(long), intent(in) :: npg1,dim1,m,s,zetf1,phidim1
      integer(long) :: ypdim,i
      complex(dop), dimension(npg1,dim1)              :: xp
      complex(dop), dimension(gs2matdim), intent(in)  :: gs2
      complex(dop), dimension(gsmatdim), intent(in)  :: gsinv
      complex(dop), dimension((npg1-1)*dim1)      :: yp
      complex(dop), dimension(dmatdim), intent(in)   :: dicht2
      complex(dop), dimension(ghmatdim), intent(in)  :: gh
      complex(dop), dimension(hlochdim), intent(in)  :: hloch
      complex(dop), dimension(hungdim), intent(in)   :: hung
      complex(dop), dimension(zetf1:phidim1), intent(in) :: psi
      real(dop) :: ymatsum

! calculate Y-matrix
      call gwpymat(yp,dicht2,hloch,gh,hung,xp,npg1,gwpdim(m,s),dim(m,s), &
                    m,s,gs2,gsinv,psi(zetf1),zetf1,phidim(m))

! get trace of C-matrix
      ypdim = (npg1-1)*dim1
      ymatsum = 0.0_dop
      do i = 1,ypdim
         ymatsum = ymatsum + dble(yp(i))*dble(yp(i))
      enddo
      ymatsum=dsqrt(ymatsum)

      return
      end subroutine fymatsum1

!#######################################################################

      subroutine dymatsum(gwppar,dgwppar,nvar)

use global
use maxv, only: maxdim

      implicit none

      integer(long)                                 :: n
      integer(long), intent(in)                     :: nvar
      real(dop), dimension(nvar), intent(in)        :: gwppar
      real(dop), dimension(nvar), intent(out)       :: dgwppar
      real(dop)                                     :: err,h
      real(dop), dimension(4*maxdim*maxdim*2)       :: work

      do n=1,nvar
         h=0.001
         dgwppar(n)=dfnummc(fymatsum,gwppar,nvar,n,h,err,work)
      enddo

      return
      end subroutine dymatsum

!#######################################################################

      real(dop) function fcmattr(gwppar)

use global
use griddatmod
use psidef

use gwplib, only: gwp_param1m_a

use fcmattrmod

      implicit none

      integer(long) :: phidim1
      real(dop), dimension(maxphidim), intent(inout) :: gwppar

      phidim1 = phidim(m)+gbfdim(m)
      call restore_gwp(gwppar,psi1,npg(m),dim(m,s),phidim1,nspfdof(m))

      call gwp_param1m_a(m,psi1,agmat,gfgwp, &
           aigwp,zgwp,mr(mrqcent),zetf(m,1),phidim1,1)

      call fcmattr1(fcmattr,gs2,gsinv,npg(m),gwpdim(m,s),m,s, &
           psi1,psitype,zetf(m,s),phidim(m),dicht2)

      return
      end function fcmattr

!#######################################################################

      subroutine fcmattr1(cmattr,gs2,gsinv,npg1,dim1,m,s,psi,psitype1, &
                 zetf1,phidim1,dicht2)

use global
use griddatmod
use psidef

use gaussian, only: gwpcmat

      implicit none

      integer(long), intent(in) :: npg1,dim1,m,s,zetf1,phidim1
      integer(long) :: psitype1,cpdim,i
      complex(dop), dimension(gs2matdim), intent(in)  :: gs2
      complex(dop), dimension(gsmatdim), intent(in)  :: gsinv
      complex(dop), dimension((npg1-1)*dim1,(npg1-1)*dim1) :: cp
      complex(dop), dimension(dmatdim)         :: dicht2
      complex(dop), dimension(zetf1:phidim1), intent(in) :: psi
      real(dop) :: cmattr,tmp

! calculate C-matrix
      call gwpcmat(cp,dicht2(dmat(m,s)),gs2(gs2mat(m,s)),gsinv(gsmat(m,s)), &
           npg1,dim1,dim(m,s),psi(zdgwp(m,s)),psitype1)

! get trace of C-matrix
      cpdim = (npg1-1)*dim1
      cmattr = 0.0_dop
      do i = 1,cpdim
         tmp = dble(cp(i,i))
         cmattr = cmattr + tmp*tmp
      enddo
!      if (cmattr .lt. 0.0_dop) cmattr = 0.0_dop
      cmattr = sqrt(cmattr)

      return
      end subroutine fcmattr1

!#######################################################################

      subroutine dcmattr(gwppar,dgwppar,nvar)

use global
use maxv, only: maxdim

      implicit none

      integer(long)                                 :: n
      integer(long), intent(in)                     :: nvar
      real(dop), dimension(nvar), intent(in)        :: gwppar
      real(dop), dimension(nvar), intent(out)       :: dgwppar
      real(dop)                                     :: err,h
      real(dop), dimension(4*maxdim*maxdim*2)       :: work

      do n=1,nvar
         h=0.001
         dgwppar(n)=dfnummc(fcmattr,gwppar,nvar,n,h,err,work)
      enddo

      return
      end subroutine dcmattr

!#######################################################################

      SUBROUTINE init_simplex(p,y,x,ndim,func,psi,m,s)

      implicit none

      integer(long)                                  :: i,j,ndim
      integer(long), intent(in)                      :: m,s
      real(dop), external                            :: func
      real(dop), dimension(ndim,ndim+1), intent(out) :: p
      real(dop), dimension(ndim+1), intent(out) :: y
      real(dop), dimension(ndim), intent(in) :: x
      real(dop) :: h
      complex(dop), dimension(*), intent(inout) :: psi

      print*,"Initialising Amoeba..."

      do i=1,ndim
         p(i,1)=x(i)
      enddo
      y(1)=func(p(1,1),psi,mc,m,s)
      do j=2,ndim+1
         do i=1,ndim
            p(i,j)=x(i)
         enddo
         h=0.1
   10    continue
         p(j-1,j)=x(j-1)+h
         y(j)=func(p(1,j),psi,mc,m,s)
         if (y(j) .gt. y(1)*1000.0) then
            h=h*0.9
            go to 10
         endif
      enddo

      return
      end SUBROUTINE init_simplex

!#######################################################################

!
! p(ndim+1,ndim) = input points
! y(ndim+1) = input values
! mp=ndim+1
! psum(ndim),ptry(ndim) = work arrays
!
      SUBROUTINE simplexopt(p,y,ndim,ftol,func,iter,psi,m1,s)

      INTEGER(long), intent(in) :: ndim,m1,s
      INTEGER(long), intent(inout) :: iter
      REAL(dop), intent(in) :: ftol
      REAL(dop), dimension(ndim,ndim+1), intent(inout) :: p
      REAL(dop), dimension(ndim+1), intent(inout) :: y
      REAL(dop), external :: func
      integer(long), PARAMETER :: ITMAX=100000
!U    USES amotry,func
      INTEGER(long) :: i,ihi,ilo,inhi,j,m,n
      REAL(dop) :: rtol,sum,swap,ysave,ytry
      REAL(dop),dimension(ndim) :: psum,ptry
      complex(dop), dimension(*), intent(inout) :: psi

      print*,"Fitting Parameters..."

      iter=0
1     do 12 n=1,ndim
        sum=0.
        do 11 m=1,ndim+1
          sum=sum+p(n,m)
11      continue
        psum(n)=sum
12    continue
2     ilo=1
      if (y(1).gt.y(2)) then
        ihi=1
        inhi=2
      else
        ihi=2
        inhi=1
      endif
      do 13 i=1,ndim+1
        if(y(i).le.y(ilo)) ilo=i
        if(y(i).gt.y(ihi)) then
          inhi=ihi
          ihi=i
        else if(y(i).gt.y(inhi)) then
          if(i.ne.ihi) inhi=i
        endif
13    continue
      rtol=2.*abs(y(ihi)-y(ilo))/(abs(y(ihi))+abs(y(ilo)))
      if (rtol.lt.ftol) then
        swap=y(1)
        y(1)=y(ilo)
        y(ilo)=swap
        do 14 n=1,ndim
          swap=p(n,1)
          p(n,1)=p(n,ilo)
          p(n,ilo)=swap
14      continue
        return
      endif
      if (iter.ge.ITMAX) stop 'ITMAX exceeded in amoeba'
      iter=iter+2
      print*,'Hi'
      ytry=simplex_try(p,y,psum,ndim,func,ihi,-1.0d0,ptry,psi,m1,s)
      print*,'Hi',ytry
      if (ytry.le.y(ilo)) then
      print*,'Hi1'
        ytry=simplex_try(p,y,psum,ndim,func,ihi,2.0d0,ptry,psi,m1,s)
      print*,'Hi1',ytry
      else if (ytry.ge.y(inhi)) then
        ysave=y(ihi)
      print*,'Hi2'
        ytry=simplex_try(p,y,psum,ndim,func,ihi,0.5d0,ptry,psi,m1,s)
      print*,'Hi2',ytry
        if (ytry.ge.ysave) then
          do 16 i=1,ndim+1
            if(i.ne.ilo)then
              do 15 j=1,ndim
                psum(j)=0.5*(p(j,i)+p(j,ilo))
                p(j,i)=psum(j)
15            continue
              y(i)=func(psum,psi,mc,m,s)
            endif
16        continue
          iter=iter+ndim
          goto 1
        endif
      else
        iter=iter-1
      endif
      print*,'Done'
      goto 2
      END SUBROUTINE simplexopt

!#######################################################################

      real(dop) FUNCTION simplex_try(p,y,psum,ndim,func,ihi,fac,ptry,psi,m,s)

      INTEGER(long), intent(in) :: ihi,ndim,m,s
      REAL(dop) :: fac
      REAL(dop), dimension(ndim,ndim+1), intent(inout) :: p
      REAL(dop), dimension(ndim), intent(inout) :: psum
      REAL(dop), dimension(ndim+1), intent(inout) :: y
!U    USES func
      REAL(dop), external :: func
      INTEGER :: j
      REAL(dop) :: fac1,fac2,ytry
      REAL(dop), dimension(ndim) :: ptry
      complex(dop), dimension(*), intent(inout) :: psi

      fac1=(1.-fac)/ndim
      fac2=fac1-fac
      do 11 j=1,ndim
        ptry(j)=psum(j)*fac1-p(j,ihi)*fac2
11    continue
      ytry=func(ptry,psi,mc,m,s)
      if (ytry.lt.y(ihi)) then
        y(ihi)=ytry
        do 12 j=1,ndim
          psum(j)=psum(j)-p(j,ihi)+ptry(j)
          p(j,ihi)=ptry(j)
12      continue
      endif
      simplex_try=ytry
      return
      END FUNCTION simplex_try

!#######################################################################
!
! iprn is a print flag
!      0 work silently   1 print 
!
      SUBROUTINE cgmin(p,n,ftol,ITMAX,fret,func,dfunc,ilog,lb,iprn, &
                 cdata,rdata)

use f1com

      INTEGER(long), intent(in) :: n,ilog,iprn
      INTEGER(long) :: iter
      INTEGER(long), intent(in) :: ITMAX
      REAL(dop), intent(in) :: ftol,lb
      REAL(dop), intent(inout) :: fret
      REAL(dop), external :: func
      external :: dfunc
      REAL(dop), dimension(n), intent(inout) :: p
      REAL(dop), parameter :: EPS=1.e-10
      REAL(dop)  :: maxxi
      INTEGER(long) :: its,j
      REAL(dop) :: dgg,fp,gam,gg
      REAL(dop), dimension(NMAX) :: g,h,xi
      complex(dop), dimension(*), intent(inout) :: cdata
      real(dop), dimension(*), intent(inout)    :: rdata

      if (n .gt. nmax) then
         write(6,'(a,i5)') 'Redimension NMAX in cgmin. Minimum :',n
         stop
      endif
      if (iprn .eq. 1) then
         write(ilog,'(a,i5,5x,a,e16.8)') 'Niter = ',iter,'Tol = ',ftol
         write(6,'(a,i5,5x,a,e16.8)') 'Niter = ',iter,'Tol = ',ftol
      endif
      fp=func(p,cdata,rdata)
      call dfunc(p,xi,n,cdata,rdata)
      maxxi=0.0
      do 6 j=1,n
         maxxi=max(maxxi,abs(xi(j)))
 6    continue
      if(iprn .eq. 1 .and. maxxi.le.ftol) then
         write(ilog,'(a)') 'Zero gradients at start of Optimisation '
         write(6,'(a)') 'Zero gradients at start of Optimisation '

         return
      endif

      do 11 j=1,n
        g(j)=-xi(j)
        h(j)=g(j)
        xi(j)=h(j)
11    continue
      do 14 its=1,ITMAX
        iter=its
        if (iprn .eq. 1) then
           write(ilog,'(a,i7)') '+++++++++++ calculating iteration: ',iter
           write(6,'(a,i7)') '+++++++++++ calculating iteration: ',iter
        endif
        call linmincg(p,xi,n,fret,func,cdata,rdata)
        if (fret .lt. lb) then
           if (iprn .eq. 1) then
              write(ilog,'(a)') 'Under lower bounds for function '
              write(6,'(a,f12.8)') 'Under lower bounds for function ',fret
           endif

           return
        endif

        call dfunc(p,xi,n,cdata,rdata)
        maxxi=0.0
        do 21 j=1,n
           maxxi=max(maxxi,abs(xi(j)))
21      continue
        if(maxxi.le.ftol) then
           if (iprn .eq. 1) then
              write(ilog,'(a)') 'Optimisation converged '
              write(6,'(a)') 'Optimisation converged '
           endif

           return
        endif

        if(2.*abs(fret-fp).le.ftol*(abs(fret)+abs(fp)+EPS)) then
           if (iprn .eq. 1) then
              write(ilog,'(/,2a)') &
          'Optimisation: gradients above tolerance at convergence of', &
          ' function'
              write(ilog,'(a,2e16.8)') 'Max. Grad/ tolerance : ', &
                 maxxi,ftol
              write(ilog,'(a)') &
          'Try different optimisation method? e.g. optimiser = simplex'

              write(6,'(/,2a)') &
          'Optimisation: gradients above tolerance at convergence of', &
          ' function'
              write(6,'(a,2e16.8)') 'Max. Grad/ tolerance : ',maxxi,ftol
              write(6,'(a)') &
          'Try different optimisation method? e.g. optimiser = simplex'

           endif

           return
        endif

        fp=func(p,cdata,rdata)

        gg=0.
        dgg=0.
        do 12 j=1,n
          gg=gg+g(j)**2
          dgg=dgg+(xi(j)+g(j))*xi(j)
12      continue

        if(gg.eq.0.)return
        gam=dgg/gg
        do 13 j=1,n
          g(j)=-xi(j)
          h(j)=g(j)+gam*h(j)
          xi(j)=h(j)
13      continue
        if (iprn .eq. 1) then
           write(ilog,'(a,e16.8)') '  function value = ',fret
           write(6,'(a,e16.8)') '  function value = ',fret
        endif
14    continue
      if (iprn .eq. 1) then
         write(ilog,'(a)') 'Maximum optimisation iterations exceeded'
         write(6,'(a)') 'Maximum optimisation iterations exceeded'
      endif

      return
      END subroutine cgmin

!#######################################################################
!
! LINMIN
! line minimiser
!
! p : point (coordinates)
! xi : gradient at p
! n : no. of coordinates
! fret: returned function
! func: routine for calculating function to be minimised
!
!#######################################################################

      SUBROUTINE linmincg(p,xi,n,fret,func,cdata,rdata)

use f1com

      INTEGER(long), intent(in) :: n
      REAL(dop), intent(inout) :: fret
      REAL(dop), dimension(n), intent(inout) :: p,xi
      REAL(dop), parameter :: TOL=1.e-4
      REAL(dop), external :: func
      REAL(dop) :: ratio
      INTEGER(long) :: j
      REAL(dop) :: ax,bx,fa,fb,fx,xmin,xx
      complex(dop), dimension(*), intent(inout) :: cdata
      real(dop), dimension(*), intent(inout)    :: rdata

      if (n .gt. nmax) then
         write(6,'(a,i5)') 'Redimension NMAX in linmin. Minimum :',n
         write(6,'(a)') 'Check dimensions in f1dim'
         stop
      endif

      ax=0.
      fa = func(p,cdata,rdata)
!
! xx is step size. New func is calculated at pcom + xx*xicom
! Get scale of coordinates. Choose xx so that xx*xicom = 0.1*pcom
! 
      xx=1.
      ncom=n
      do 11 j=1,n
        pcom(j)=p(j)
        xicom(j)=xi(j)
        if(pcom(j) .eq. 0.0) then
            ratio=abs(0.1/xicom(j))
        else
            ratio=abs(0.1*pcom(j)/xicom(j))
        endif
        xx = min(xx,ratio)

11    continue

! check that fx is not too large
 8    fx = f1dim(xx,func,cdata,rdata)
      if (abs(fx) .gt. abs(10.0*fa)) then
         xx = 0.5*xx
         go to 8
      endif

      call mnbrakcg(ax,xx,bx,fa,fx,fb,f1dim,func,cdata,rdata)
      fret=brentcg(ax,xx,bx,f1dim,TOL,xmin,func,cdata,rdata)
      do 12 j=1,n
        xi(j)=xmin*xi(j)
        p(j)=p(j)+xi(j)
12    continue

      return
      END subroutine linmincg

!#######################################################################

      real(dop) FUNCTION brentcg(ax,bx,cx,f1dim,tol,xmin,func,cdata,rdata)

      REAL(dop), intent(in) :: ax,bx,cx,tol
      REAL(dop), intent(out) :: xmin
      REAL(dop), external :: f1dim,func
      real(dop), parameter :: CGOLD=.3819660,ZEPS=1.0e-10
      integer(long), parameter :: ITMAX=100
      INTEGER(long) :: iter
      REAL(dop) :: a,b,d,e,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm 
      complex(dop), dimension(*), intent(inout) :: cdata
      real(dop), dimension(*), intent(inout)    :: rdata

      a=min(ax,cx)
      b=max(ax,cx)
      v=bx
      w=v
      x=v
      d=0.
      e=0.
      fx=f1dim(x,func,cdata,rdata)
      fv=fx
      fw=fx
      do 11 iter=1,ITMAX
        xm=0.5*(a+b)
        tol1=tol*abs(x)+ZEPS
        tol2=2.*tol1
        if(abs(x-xm).le.(tol2-.5*(b-a))) goto 3
        if(abs(e).gt.tol1) then
          r=(x-w)*(fx-fv)
          q=(x-v)*(fx-fw)
          p=(x-v)*q-(x-w)*r
          q=2.*(q-r)
          if(q.gt.0.) p=-p
          q=abs(q)
          etemp=e
          e=d
          if(abs(p).ge.abs(.5*q*etemp).or.p.le.q*(a-x).or.p.ge.q*(b-x)) goto 1
          d=p/q
          u=x+d
          if(u-a.lt.tol2 .or. b-u.lt.tol2) d=sign(tol1,xm-x)
          goto 2
        endif
1       if(x.ge.xm) then
          e=a-x
        else
          e=b-x
        endif
        d=CGOLD*e
2       if(abs(d).ge.tol1) then
          u=x+d
        else
          u=x+sign(tol1,d)
        endif
        fu=f1dim(u,func,cdata,rdata)
        if(fu.le.fx) then
          if(u.ge.x) then
            a=x
          else
            b=x
          endif
          v=w
          fv=fw
          w=x
          fw=fx
          x=u
          fx=fu
        else
          if(u.lt.x) then
            a=u
          else
            b=u
          endif
          if(fu.le.fw .or. w.eq.x) then
            v=w
            fv=fw
            w=u
            fw=fu
          else if(fu.le.fv .or. v.eq.x .or. v.eq.w) then
            v=u
            fv=fu
          endif
        endif
11    continue
      stop 'brent exceed maximum iterations'
3     xmin=x
      brentcg=fx

      return
      END function brentcg

!#######################################################################

      SUBROUTINE mnbrakcg(ax,bx,cx,fa,fb,fc,f1dim,func,cdata,rdata)

      REAL(dop), intent(inout) :: ax,bx,cx,fa,fb,fc
      REAL(dop), external :: func,f1dim
      real(dop), PARAMETER :: GOLD=1.618034, GLIMIT=100., TINY=1.e-20
      REAL(dop) :: dum,fu,q,r,u,ulim
      complex(dop), dimension(*), intent(inout) :: cdata
      real(dop), dimension(*), intent(inout)    :: rdata

      if(fb.gt.fa)then
        dum=ax
        ax=bx
        bx=dum
        dum=fb
        fb=fa
        fa=dum
      endif
      cx=bx+GOLD*(bx-ax)
      fc=f1dim(cx,func,cdata,rdata)
1     if(fb.ge.fc)then
        r=(bx-ax)*(fb-fc)
        q=(bx-cx)*(fb-fa)
        u=bx-((bx-cx)*q-(bx-ax)*r)/(2.*sign(max(abs(q-r),TINY),q-r))
        ulim=bx+GLIMIT*(cx-bx)
        if((bx-u)*(u-cx).gt.0.)then
          fu=f1dim(u,func,cdata,rdata)
          if(fu.lt.fc)then
            ax=bx
            fa=fb
            bx=u
            fb=fu
            return
          else if(fu.gt.fb)then
            cx=u
            fc=fu
            return
          endif
          u=cx+GOLD*(cx-bx)
          fu=f1dim(u,func,cdata,rdata)
        else if((cx-u)*(u-ulim).gt.0.)then
          fu=f1dim(u,func,cdata,rdata)
          if(fu.lt.fc)then
            bx=cx
            cx=u
            u=cx+GOLD*(cx-bx)
            fb=fc
            fc=fu
            fu=f1dim(u,func,cdata,rdata)
          endif
        else if((u-ulim)*(ulim-cx).ge.0.)then
          u=ulim
          fu=f1dim(u,func,cdata,rdata)
        else
          u=cx+GOLD*(cx-bx)
          fu=f1dim(u,func,cdata,rdata)
        endif
        ax=bx
        bx=cx
        cx=u
        fa=fb
        fb=fc
        fc=fu
        goto 1
      endif

      return
      END subroutine mnbrakcg

!#######################################################################

      real(dop) FUNCTION f1dim(x,func,cdata,rdata)

use f1com

      REAL(dop), external :: func
      REAL(dop), intent(in) :: x
!U    USES func
      INTEGER(long) :: j
      REAL(dop), dimension(NMAX) :: xt
      complex(dop), dimension(*), intent(inout) :: cdata
      real(dop), dimension(*), intent(inout)    :: rdata

      do 11 j=1,ncom
        xt(j)=pcom(j)+x*xicom(j)
11    continue

      f1dim=func(xt,cdata,rdata)

      return
      END function f1dim

!#######################################################################
!
! numerical derivatives

      real(dop) function dfnummc(func,x,nvar,n,h,err,work)

       integer(long), parameter :: ntab=10
       real(dop), parameter     :: con=1.4_dop,con2=con*con, &
                                   big=1.0e30_dop,safe=2.0_dop

       integer(long)                             :: i,j
       integer(long), intent(in)                 :: n,nvar
       real(dop), dimension(nvar,2), intent(out) :: work
       real(dop), dimension(nvar), intent(in)    :: x
       real(dop), dimension(ntab,ntab)           :: a
       real(dop)                                 :: errt,fac,hh
       real(dop), intent(in)                     :: h
       real(dop), intent(out)                    :: err
       real(dop), external                       :: func

      if (h.eq.0) stop 'h must be nonzero in dfridr'
      do i=1,nvar
         work(i,1)=x(i)
         work(i,2)=x(i)
      enddo

      dfnummc=0.0_dop
      hh=h
      work(n,1)=x(n)+hh
      work(n,2)=x(n)-hh
      a(1,1)=(func(work(1,1))-func(work(1,2)))
      a(1,1)=a(1,1)/(2.0*hh)
      err=big
      do i=2,ntab
         hh=hh/con
         work(n,1)=x(n)+hh
         work(n,2)=x(n)-hh
         a(1,i)=(func(work(1,1)) - func(work(1,2)))
         a(1,i)=a(1,i)/(2.0*hh)
         fac=con2
         do j=2,i
            a(j,i)=(a(j-1,i)*fac-a(j-1,i-1))/(fac-1.0)
            fac=con2*fac
            errt=max(abs(a(j,i)-a(j-1,i)),abs(a(j,i)-a(j-1,i-1)))
            if (errt .le. err) then
               err=errt
               dfnummc=a(j,i)
            endif
         enddo
         if(abs(a(i,i)-a(i-1,i-1)) .ge. safe*err) return
      enddo

      return
      end function dfnummc


      end module gwpoptimmod
