!#######################################################################
      module initpropmod

use decimal, only: long, dop
use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use logdat
use op1lib,  only: zerovxi, initvxi

      implicit none
      private
      public :: initprop
      contains

!#######################################################################
!                                                                      
!                  module initprop
!                                                                      
!  MCTDH module initialising the propagation
!                                                                      
!#######################################################################
! INITPROP
!
!#######################################################################

      subroutine initprop(hops,gwpdyn)

      implicit none

      integer(long)                                    :: f,h,k,s,m,&
                                                          t
      integer(long), dimension(gwpdyndim), intent(out) :: gwpdyn
      real(dop), dimension(hopsdim), intent(in)        :: hops

!-----------------------------------------------------------------------
! set lpsiopt(4) if selected-ci
!-----------------------------------------------------------------------
      if (citype .gt. 0) lpsiopt(4)=.true.

!-----------------------------------------------------------------------
! flag any complete spf bases
! GWP basis can be over-complete so ignore
!-----------------------------------------------------------------------
      if (dentype .eq. 1) then
         do t=1,nstate
            do s=1,nstate
               do m=1,nmode
                  if (ddim(m,s,t) .ge. subdim(m)) complete(m)=.true.
               enddo
            enddo
         enddo
      else
         do s=1,nstate
            do m=1,nmode
               if (dim(m,s) .ge. subdim(m)) complete(m)=.true.
            enddo
         enddo
      endif

      do m=1,nmode
         if (gwpm(m)) complete(m)=.false.
      enddo

!-----------------------------------------------------------------------
! set complete true, if mode is flagged as identical (keyword "id")
!-----------------------------------------------------------------------
      do m=1,nmode
         if (idmode(m).gt.0) complete(m)=.true.
      enddo

!-----------------------------------------------------------------------
! set complete true if SPFs are kept fixed
!-----------------------------------------------------------------------
      if (ltindspf) complete(:) = .true.

!-----------------------------------------------------------------------
! find indicap for automatic cap
!-----------------------------------------------------------------------
      do k=1,khtot(1)
         if (autocap(k) .gt. 0) then
            f=autocap(k)
            h=hamilton(f,k)
            call acapdata(hops(zhop(h)),gdim(f),indicap(f),tiny)
         endif
      enddo

!-----------------------------------------------------------------------
! Set all GWPs to dynamic or static (igwpdyn = 0, 2 respectively)
!-----------------------------------------------------------------------
      do m=1,nmode
         if (.not. gwpm(m)) cycle

         do s=1,nstate
            call setingwpdyn(gwpdyn(zgwpdyn(m,s)),ngwpdyn(m,s),&
                 igwpdynm(m),nspfdof(m),gwpdim(m,s))
         enddo
      enddo

      return
      end subroutine initprop

!#######################################################################

      subroutine setingwpdyn(gwpdyn1,ngwpdyn1,igwpdyn1,gdof,gwpdim1)

      implicit none

      integer(long), intent(in)                           :: igwpdyn1,&
                                                            gdof,gwpdim1
      integer(long), intent(out)                          :: ngwpdyn1
      integer(long), dimension(gdof,gwpdim1), intent(out) :: gwpdyn1
      integer(long)                                       :: e,n

      if (igwpdyn1 .eq. 0) then
         do e=1,gwpdim1
            do n=1,gdof
               gwpdyn1(n,e) = 0
               ngwpdyn1 = ngwpdyn1 + 1
            enddo
         enddo

      else if (igwpdyn1 .eq. 1 .or. igwpdyn1 .eq. 2) then
         ngwpdyn1 = 0
         do e=1,gwpdim1
            do n=1,gdof
               gwpdyn1(n,e) = 1
            enddo
         enddo

      else 
         routine = 'Setingwpdyn (initprop.f90)'
         message = 'Unknown value for igwpdyn (control of GWP coupling)'
         call errormsg

      endif

      return 
      end subroutine setingwpdyn

!#######################################################################

      subroutine acapdata(hops,gdim,indicap,tiny)

      implicit none

      integer(long)                          :: g
      integer(long), intent(in)              :: gdim
      integer(long), intent(out)             :: indicap
      real(dop), intent(in)                  :: tiny
      real(dop), dimension(gdim), intent(in) :: hops

!-----------------------------------------------------------------------
! see if CAP at RHS (LHS). Then search for first (last) non-zero point
!-----------------------------------------------------------------------
      if (abs(hops(1) - 0.0_dop) .lt. tiny) then
         indicap=2
         do g=2,gdim
            if (abs(hops(g) - 0.0_dop) .lt. tiny) indicap=indicap+1
         enddo
      else
         indicap=1
         do g=2,gdim
            if (abs(hops(g) - 0.0_dop) .gt. tiny) indicap=indicap+1
         enddo
      endif
      
      return
      end subroutine acapdata  

!#######################################################################

      end module initpropmod

