!#######################################################################
! New version (Benjamin, December 2008)
!#######################################################################
!         MODULE LOGDAT
!  contains : wrhinfo, loginp, logop, logmem, errormsg, warnmsg
!
!#######################################################################

     module logintmod

use decimal, only: long
use global
use logdat
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use dirdyn
use runpropmod
use channels
use hpsimod
use crocoflymod
use openmpmod
use rdopermod
use keyunitsmod, only: kelvin
#ifdef MPI
use mpidata
#endif

     implicit none

     public :: logint, logdd

contains

      subroutine logint

      implicit none

      integer(long)           :: k,rlx,l,m,f,n,i
      integer(long)           :: ilbl,jlbl
      character(len=80)       :: string 
      character(len=80)       :: buffer
      character(len=3)        :: strfs
      logical                 :: lfreeze ,usedfast

! prepare string with time unit for output
      strfs=' '//rlxtime(1:2)

      write(ilog,'(/,a,/)') '***** Calculation type *****'

      if (lrlx) then
         if (dentype .eq. 0) then
            if (rlxtime .eq. 'K') then
               write(ilog,'(a,f8.3,a)') &
                   'Wavepacket relaxation. Final Temperature: ',&
                   kelvin/(2.0_dop*tfinal),' K'
            else
               write(ilog,'(a)') 'Wavefunction relaxation.'
               if(rlaxnum .ge. 0 ) then
                  string='relaxation='
                  l=12
                  rlx=rlaxnum
                  if(rlx .ge. 9999) then
                     string(l:l+4) = 'full '
                     l=l+5
                     rlx = rlx - 10000
                  endif 
                  if(rlx.eq.920) then
                     string(l:l+6) = 'follow '
                     l=l+7
                  elseif(rlx.eq.930) then
                     string(l:l+4) = 'lock '
                     l=l+5
                  elseif(rlx.ge.0 .and. rlx.le.9) then
                     write(string(l:l+1),'(i1,1x)') rlx
                     l=l+2
                  else
                     write(string(l:l+4),'(i4,1x)') rlx
                     l=l+5
                  endif
                  if(lolsen) then
                     string(l:l+5) = 'olsen '
                     l=l+6
                  endif 
                  if(lquad) then
                     string(l:l+4) = 'quad '
                     l=l+5
                  endif 
                  if(lortho) then
                     string(l:l+5) = 'ortho '
                     l=l+6
                  endif
                  if(lbackr) then
                     string(l:l+10) = 'backrotate '
                     l=l+11
                  endif
                  if(lquadphi) then
                     string(l:l+7) = 'quadphi '
                     l=l+8
                  endif
                  if(phicount.gt.0) then
                     write(string(l:l+13),'(a,i3,1x)')'phicount =',phicount
                     l=l+14
                  endif 
                  if(inttypea.eq.6 .and. precon.gt.0) then
                     write(string(l:l+11),'(a,i4,1x)') 'precon=',precon
                     call rmblnk(string(l:l+11))
                     k=index(string(l:l+11),' ')
                     l=l+k
                  endif
                  write(ilog,'(a)') string(1:l-2)
               endif
            endif
         else if (dentype .eq. 2) then
            if (rlxtime .eq. 'K') then
               write(ilog,'(a,f8.3,a)') &
                   'Density of type II relaxation. Final Temperature: ',&
                   kelvin/(2.0_dop*tfinal),' K'
            else
               write(ilog,'(a)') 'Density of type II relaxation.'
            endif
         endif

      elseif (lrundiag) then
         write(ilog,'(a)') 'Hamiltonian diagonalisation.'

      else 
         if (dentype .eq. 0) then
            if (psitype .eq. 11) then
               write(ilog,'(a)') 'Real Wavefunction propagation.'
            else
               write(ilog,'(a)') 'Wavefunction propagation.'
            endif
         else if (dentype .eq. 1) then
            write(ilog,'(a)') 'Density of type I propagation.'
            if (ldissin) then
               write(ilog,'(a)') 'Propagation: Open system.'
            else
               write(ilog,'(a)') 'Propagation: Closed system.'
            endif
         else if (dentype .eq. 2) then
            write(ilog,'(a)') 'Density of type II propagation.'
            if (ldissin) then
               write(ilog,'(a)') 'Propagation: Open system.'
            else
               write(ilog,'(a)') 'Propagation: Closed system.'
            endif
         endif
      endif

      if (lexact) then
         write(ilog,'(a)') 'Numerically exact calculation'
      endif

      if (psitype .eq. 4) then
         write(ilog,'(a)') 'G-MCTDH calculation'
      endif
      if (lgwptrj) then
         write(ilog,'(a)') &
                 'vMCG calculation using precalculated GWP trajectories'
      endif
      if (ldd) then
         write(ilog,'(a)') 'Direct dynamics calculation'
      else if (lddmctdh) then
         write(ilog,'(a)') 'Direct dynamics MCTDH calculation'
      endif

! --- Write Information about the distributed memory parallelisation
#ifdef MPI
      if(lmpi) Then
         l=1
         if(.not. lmpimfields) then
            string(l:l+10)='no-mfields '
            l=l+11
         endif
         if(.not. lmpifunka2) then
            string(l:l+9)='no-funka2 '
            l=l+10
         endif
         if(.not. lmpicalcha) then
            string(l:l+9)='no-calcha '
            l=l+10
         endif
         if(.not. lmpiphihphi) then
            string(l:l+10)='no-phihphi '
            l=l+11
         endif
         if(.not. lmpidav) then
            string(l:l+7)='no-dav '
            l=l+7
         endif
         if(.not.lmpihlochphi) then
            string(l:l+12)='no-hlochphi '
            l=l+12
         endif
         if(.not.lmpigetdavmat) then
            string(l:l+13)='no-getdavmat '
            l=l+13
         endif
         if(.not.lmpidsyev) then
            string(l:l+9)='no-dsyev '
            l=l+9
         endif

         write(ilog,'(a,i4)') &
              'Using MPI parallelisation. Number of processes =',nmpi
         if(l.gt.1.or.ldavkey) then 
            if (ldavkey) then
               write(ilog,'(a,a,a,i4)') &
                    'Options: ',string(1:l-1),'dav = ',intordermpi
            else
               write(ilog,'(a,a)')'Options: ',string(1:l-1)
            endif
         endif
      endif
#endif

! --- Write Information about the OpenMP parallelisation
      if(lompthread) Then
         l=1
!         if(.not. lsummf) then
!            string='no-summf '
!            l=l+9
!         endif
!         if(.not. lfunkphi) then
!            string(l:l+10)='no-funkphi '
!            l=l+11
!         endif
!         if(.not. lmfields) then
!            string(l:l+10)='no-mfields '
!            l=l+11
!         endif
!         if(.not. lphihphi) then
!            string(l:l+10)='no-phihphi '
!            l=l+11
!         endif
!         if(.not. lhlp1m) then
!            string(l:l+13)='no-hlochphi1m '
!            l=l+14
!         endif
!         if(.not. lfunka) then
!            string(l:l+9)='no-funka '
!            l=l+10
!         endif
!         if(.not. ldsyev) then
!            string(l:l+9)='no-dsyev '
!            l=l+10
!         endif
!         if(lmemmfields) then
!            string(l:l+11)='mem-mfields '
!            l=l+12
!         endif
!         if(lsummf2) then
!            string(l:l+6)='summf2 '
!            l=l+7
!         elseif(lsummf) then
!            string(l:l+6)='summf1 '
!            l=l+7
!         endif

         write(ilog,'(a,i4)') &
         'Using OpenMP parallelisation. Number of threads =',ompthread
      endif

      if (lmulpack) then
         write(ilog,'(a,i3,a)') 'Propagating',npacket, &
                    ' wavepackets, using multi-set formalism.'
      endif
      if(mpb.gt.0) then
         write(ilog,'(a,i3,a)') 'Propagating',npackts, &
                    ' wavepackets, using single-set formalism.'
      endif

      if (lmult .and. nstate.gt.npacket) then
         write(ilog,'(a)') 'Electronic states in multi-set formalism.'
      else if (leb) then
         write(ilog,'(a)') 'Electronic states in single-set formalism.'
      endif
      if (lreflex) then
         write(ilog,'(a)') 'Using Reflex algorithm.'
      endif

!-----------------------------------------------------------------------
! log how DD is run
!-----------------------------------------------------------------------
      if (ldd .or. lddmctdh) call logdd

!-----------------------------------------------------------------------
!     Loop over Hamiltonians finished. Write FAST data is fast
!     algorithm used for natpots
!-----------------------------------------------------------------------
      usedfast=.false.
      do n=1,nrdf
         if (lfast(n)) usedfast=.true.
      enddo

      if (usedfast) then
          write(iolog,'(/a)') '"FAST" algorithm for H(npot)*A'// &
              ' operation for System Hamiltonian is used:'
          write(iolog,'(a)') ' natpot no.  |  begin   |   end   |  '// &
              ' order   |  pre-multiplied mode(s)'
          do n=1,nrdf
             if (lfast(n)) then
                buffer=' '
                ilbl=1
                jlbl=ilbl+2
                write(buffer(ilbl:jlbl),'(i3)') natpmode(1,n)
                do i=2,fastorder(n)
                   ilbl=ilbl+3
                   jlbl=ilbl+2
                   buffer(ilbl:ilbl)=','
                   write(buffer(ilbl+1:jlbl),'(i2)') natpmode(i,n)
                enddo
                write(iolog,'(4x,i2,9x,i4,8x,i4,8x,i2,13x,a)') &
                      n,nterm(n),ntermlast(n),fastorder(n),buffer(1:jlbl)
             endif
          enddo
      endif

! --- OUTPUT INTEGRATOR ALGORITHM ---

      if (lrundiag) goto 100

      write(ilog,*)
      write(ilog,'(a)') '***** Integrator algorithm *****'
      write(ilog,*)
      if (lconst) then
         write(ilog,'(a)') 'Mean-fields etc. are kept constant.'
         if (lconstfix) then
            write(ilog,'(a,f7.3,a)')&
            'CMF stepsize is fixed to:',updatetime/fs,strfs
         else
            write(ilog,'(a)',advance='no') &
            'CMF stepsize is adaptive (error control for '
            if (lrunml) then
               write(ilog,'(a)') 'all modes).'
            else if (lconstvar) then
               write(ilog,'(a)') 'A and phi).'
            else if (lcmfadaphi) then
               write(ilog,'(a)') 'phi only).'
            else if (lcmfadaa) then
               write(ilog,'(a)') 'A only).'
            endif
            write(ilog,'(a,1pe9.2,a,6x,a,0pf7.3,a)') 'CMF accuracy:', &
               epsupdate,',','Initial stepsize:',updatetime/fs,strfs
         endif
         if (lrunml) then
            write(ilog,'(a)',advance='no') 'MULTI-LAYER run, '
            if (lmlcmfsplit) then ! ml-cmf=split
               write(ilog,'(a)') 'propagating modes separately ' &
                   // '(split propagation)'
!              ML is not initialized, so delay logging more information
               write(ilog,'(a)') 'For integrator details, see below.'
               goto 95
            else ! ml-cmf=unified
               write(ilog,'(a)') 'propagating all modes together ' &
                    //'(unified propagation)'
            endif
         endif
         if (lrunml) then
            write(ilog,'(a)',advance='no') 'For complete equation, '
         else
            write(ilog,'(a)',advance='no') 'For SPFs, '
         endif
         if (inttypephi.eq.1) then
            write(ilog,'(a)') 'For spfs, ABM integrator'
            write(ilog,'(a,i3,a,1pe9.2,a,1pe9.2,a)')&
              'order:',intorderphi,',  accuracy:',accphi,&
              ',  initial step:', dtphi/fs,strfs
         else if (inttypephi.eq.2) then
            write(ilog,'(a)') 'For spfs, BS integrator'
            write(ilog,'(a,i3,a,1pe9.2,a,0pf7.3,a)')&
              'order:',intorderphi,',  accuracy:',accphi,&
              ',  initial step:', dtphi/fs,strfs
         elseif (inttypephi .eq. 4) then
            if(liork) then 
               write(ilog,'(2a)') 'For spfs, RK5 integrator',&
                              ' with improved orthogonality'
            else
               write(ilog,'(a)')  'For spfs, RK5 integrator'
            endif
            write(ilog,'(a,1pe9.2,a,0pf7.3,a)')&
                 'accuracy:', accphi, ', initial step', dtphi/fs,strfs
         elseif (inttypephi .eq. 5) then
            if(liork) then
               write(ilog,'(2a)') 'For spfs, RK8 integrator',&
                              ' with improved orthogonality'
            else
               write(ilog,'(a)')  'For spfs, RK8 integrator'
            endif
            write(ilog,'(a,1pe9.2,a,0pf7.3,a)')&
                 'accuracy:', accphi, ', initial step', dtphi/fs,strfs
         endif

         if (lrunml) goto 94 ! no special treatment for A in ML

         if (inttypea.eq.1) then
            write(ilog,'(a)') 'For A-coefficients, ABM integrator'
         else if (inttypea.eq.2) then
            write(ilog,'(a)') 'For A-coefficients, BS integrator'
         else if (inttypea.eq.3) then
            write(ilog,'(a)') 'For A-coefficients, SIL integrator'
            write(ilog,'(a,i4,a,1pe9.2,a,0pf7.3,a)')&
                       'order:',intordera,',  accuracy:',accavec,&
                       ',  initial step:', dta/fs,strfs
            if (loldsilerr) then
               write(ilog,'(a)') 'error criterion: standard'
            else
               write(ilog,'(a)') 'error criterion: novel'
            endif
            if(lcap .or. ldissin) then  
               write(ilog,'(a)') 'Arnoldi-SIL (complex, CSIL) used.'
            else
               write(ilog,'(a)') 'Standard-SIL (real) used.'
            endif
         else if (inttypea.eq.4) then
            write(ilog,'(a)') 'For A-coefficients, RK5 integrator'
         else if (inttypea.eq.5) then
            write(ilog,'(a)') 'For A-coefficients, RK8 integrator'
         else if (inttypea.eq.6) then
            if(lrreal) then
               write(ilog,'(a)') &
         'For A-coefficients, DAV "integrator", restriced-real version'
            elseif(lreal) then
               write(ilog,'(a)') &
               'For A-coefficients, DAV "integrator", real version'
            elseif(lcap) then
               write(ilog,'(a)')&
               'For A-coefficients, DAV "integrator", complex version'
            else
               write(ilog,'(a)')&
               'For A-coefficients, DAV "integrator", hermitian version'
            endif
            if(mpb.gt.0) then
               write(ilog,'(a,i4,a,1pe9.2,a,i4,a)') &
                       'order:',intordera,',  accuracy:',accavec, &
                       ',  Block-DAV with', npackts,' packets.'
            else
               write(ilog,'(a,i4,a,1pe9.2)') &
                       'order:',intordera,',  accuracy:',accavec
            end if

         end if
         
94       continue

      else if (lexact) then
         write(ilog,'(a)')&
              'Numerically exact calculation'
         if (inttypephi.eq.1) then
            write(ilog,'(a,i3,a,1pe9.2,a,1pe9.2,a)') &
             'ABM integrator, order:',intorderphi,'  accuracy:', &
             accphi,'  initial step:', dtphi/fs,strfs
         else if (inttypephi.eq.2) then
            write(ilog,'(a,i3,a,1pe9.2,a,0pf7.3,a)') &
             'BS integrator, order:',intorderphi,'  accuracy:', &
             accphi,'  initial step:', dtphi/fs,strfs
         else if (inttypephi.eq.3) then
            write(ilog,'(a,i3,a,1pe9.2,a,0pf7.3,a)') &
             'SIL integrator, order:',intorderphi,'  accuracy:', &
             accphi,'  initial step:', dtphi/fs,strfs
            if(lcap .or. ldissin) then  
               write(ilog,'(a)') 'Arnoldi-SIL (complex, CSIL) used.'
            else
               write(ilog,'(a)') 'Standard-SIL (real) used.'
            endif
         else if (inttypephi.eq.4) then
            write(ilog,'(a,1pe9.2,a,0pf7.3,a)') &
                 'RK5 integrator, accuracy:', accphi, &
                 '  initial step:', dtphi/fs,strfs
         else if (inttypephi.eq.5) then
            write(ilog,'(a,1pe9.2,a,0pf7.3,a)') &
                 'RK8 integrator, accuracy:', accphi, &
                 '  initial step:', dtphi/fs,strfs
         else if (inttypephi.eq.6) then
            write(ilog,'(a,0pf7.3,a)') &
                 'Euler integrator; step:', dtphi/fs,strfs
         else if (inttypephi.eq.8) then
            write(ilog,'(a,1pe9.2)') &
                  'Chebyshev integrator, accuracy ',accphi
         else if (inttypephi.eq.69) then
            write(ilog,'(a)') 'SOD integrator '
         else if (inttypephi.eq.70) then
            write(ilog,'(a)') 'Split Operator integrator '
         endif

      else
         write(ilog,'(a)') &
              'Mean-fields etc. are computed at each integration step '
         if (inttyp.eq.1) then
            write(ilog,'(a,i3,a,1pe9.2,a,1pe9.2,a)') &
             'ABM integrator, order:',intord,'  accuracy:', &
             accuracy,'  initial step:', dt/fs,strfs
         else if (inttyp.eq.2) then
            write(ilog,'(a,i3,a,1pe9.2,a,1pe9.2,a)') &
             'BS integrator, order:',intord,'  accuracy:', &
             accuracy,'  initial step:', dt/fs,strfs
         else if (inttyp.eq.4) then
            if(liork) then 
               write(ilog,'(a)') 'RK integrator with improved orthogonality'
            endif
            write(ilog,'(a,1pe9.2,a,1pe9.2,a)') &
                 'RK5 integrator, accuracy:', accuracy, &
                 '  initial step:', dt/fs,strfs
         else if (inttyp.eq.5) then
            if(liork) then 
               write(ilog,'(a)') 'RK integrator with improved orthogonality'
            endif
            write(ilog,'(a,1pe9.2,a,1pe9.2,a)') &
                 'RK8 integrator, accuracy:', accuracy, &
                 '  initial step:', dt/fs,strfs
         else if (inttyp.eq.10) then
            write(ilog,'(a,0pf7.3,a)') &
                 'Verlet integrator, step size:', dt/fs,' fs'
         endif
      endif

95    continue ! lmlcmfsplit jumps here
      if (limportho) then
         write(ilog,'(a)') &
              'The wavefunction is reorthonormalized after each step.'
      endif

      if (.not. lexact) then
         write(ilog,'(a)') 'The working equation for the spfs reads:'
         if (lfunkl) then
            write(ilog,'(a)') 'i dphi/dt = (h + (1-P) ...) phi'
         else
            write(ilog,'(a)') 'i dphi/dt = (1-P)(h + ...) phi'
         endif

         if (lnaturcmf) then
           write(ilog,'(a)') 'Using approximate natural orbitals.(CMF)'
         else if (lenorb) Then
           write(ilog,'(a)') 'Using energy orbitals.(CMF)'
         else
           write(ilog,'(a)') 'Using standard orbitals.'
         endif

         if(.not.lopro) Write(ILog,'(/A)') &
               'The simple projector is used '
      endif
      if (lrhonrm) then
         write(ilog,'(a)') 'Using density weighted L2-error (rho-norm)'
      endif

      if (usehsym) then
         write(ilog,'(a)') 'Using operator symmetries'
      else
         write(ilog,'(a)') 'Not using operator symmetries'
      endif

      if (ldirmuld) write(ilog,'(/,a)')&
              'Direct algorithm used for multi-dimensional potentials'

      if (lgwp) then
         if (gwptype .eq. 1) then
            write(ilog,'(a)') &
              'GWPs propagated using width matrix with frozen diagonal'
         else if (gwptype .eq. 2) then
            write(ilog,'(a)') &
              'GWPs propagated using separable width matrix'
         else if (gwptype .eq. 3) then
            write(ilog,'(a)') 'GWPs propagated using diagonal FG'
         else if (gwptype .eq. 4) then
            write(ilog,'(a)') 'GWPs propagated using non-diagonal FG'
         else
            write(ilog,'(a)') 'GWPs propagated using full width matrix'
         endif

         if (gwpphase .eq. -1) then
            write(ilog,'(a)') &
              'Phaseless GWPs with non-propagated scalar parameters'
         else if (gwpphase .eq. 0) then
            write(ilog,'(a)') &
              'Non-normalised GWPs with stationary scalar parameters'
         else if (gwpphase .eq. 1) then
            write(ilog,'(a)') &
              'Phaseless GWPs with propagated scalar parameters'
         endif
         
!-----------------------------------------------------------------------
! Log model used for GWPs
!-----------------------------------------------------------------------
         if (gwpmodel .eq. 0) then
            if (lvmcg) then
               write(ilog,'(a)') 'GWPs follow vMCG trajectories'
            else
               write(ilog,'(a)') 'GWPs follow full G-MCTDH trajectories'
            endif
         else if (gwpmodel .eq. 1 .or. gwpmodel .eq. 5) then
            write(ilog,'(a)') 'GWPs follow classical trajectories'
         else if (gwpmodel .eq. 2) then
            write(ilog,'(a)') 'Independent GWP approximation used'
         else if (gwpmodel .eq. 3) then
            write(ilog,'(a)') 'GWP basis time-independent'
         else if (gwpmodel .eq. 4) then
            write(ilog,'(a)') 'GWPs propagated using BEL algorithm'
         else if (gwpmodel .eq. -1) then
            write(ilog,'(a)') 'GWPs use mode depedent EOMs'
         endif

!-----------------------------------------------------------------------
! Log parameters.
! gwpmodel  >= 0, All modes use the same model. 
! gwpmodel  = -1, Modes depedent modesl
!-----------------------------------------------------------------------
         if (gwpmodel  .eq. 0) then
            if (gwpcxm(1) .eq. 0) then
               write(ilog,'(a)') 'GWP CX formulation not used'
            else if (gwpcxm(1) .eq. 1) then
               write(ilog,'(a)') &
                   'CX formulation uses dynamic uncorrelated GWPs'
            else if (gwpcxm(1) .eq. 2) then
               write(ilog,'(a)') &
                   'CX formulation uses static uncorrelated GWPs'
            endif

            if (igwpdynm(1) .eq. 0) then
               write(ilog,'(a)') 'All GWP parameters coupled'
            else if (igwpdynm(1) .eq. 1) then
               write(ilog,'(a,e12.4)') &
         'GWP parameter coupling allocated dynamically. Eps =', epsdyn
            else if (igwpdynm(1) .eq. 2) then
               write(ilog,'(a)') 'All GWPs uncoupled (classical)'
            endif

            if (gwpintordm(1) .eq. 0) then
               write(ilog,'(a)') &
                  'Saddle Point Approximation used for GWP integrals'
            else if (gwpintordm(1) .eq. 1) then
               write(ilog,'(a)') 'Using local linear expansion of PES'
            else if (gwpintordm(1) .eq. 2) then
               write(ilog,'(a)') 'Using local harmonic approximation of PES'
            else if (gwpintordm(1) .eq. 3) then
               write(ilog,'(a)') 'Using local cubic expansion of PES'
            else if (gwpintordm(1) .eq. 4) then
               write(ilog,'(a)') 'Using local quartic expansion of PES'
            endif

            if (gwpintm(1) .eq. 1) then
               write(ilog,'(a)') 'Expansion around centre of GWP on RHS'
            else if (gwpintm(1) .eq. 2) then
               write(ilog,'(a)') 'Symmetrised expansion around centre of GWP on RHS'
            else if (gwpintm(1) .eq. 3) then
               write(ilog,'(a)') 'Expansion around midpoint of GWPs in integral'
            endif

         else if (gwpmodel  .gt. 0) then

            if (gwpintordm(1) .eq. 0) then
               write(ilog,'(a)') &
                  'Saddle Point Approximation used for GWP integrals'
            else if (gwpintordm(1) .eq. 1) then
               write(ilog,'(a)') 'Using local linear expansion of PES'
            else if (gwpintordm(1) .eq. 2) then
               write(ilog,'(a)') 'Using local harmonic approximation of PES'
            else if (gwpintordm(1) .eq. 3) then
               write(ilog,'(a)') 'Using local cubic expansion of PES'
            else if (gwpintordm(1) .eq. 4) then
               write(ilog,'(a)') 'Using local quartic expansion of PES'
            endif

            if (gwpintm(1) .eq. 1) then
               write(ilog,'(a)') 'Expansion around centre of GWP on RHS'
            else if (gwpintm(1) .eq. 2) then
               write(ilog,'(a)') 'Symmetrised expansion around centre of GWP on RHS'
            else if (gwpintm(1) .eq. 3) then
               write(ilog,'(a)') 'Expansion around midpoint of GWPs in integral'
            endif

         else
            write(ilog,'(a)') &
                 ' Mode     type         gwpcx    gwpdyn   gwpintord gwpint'
            do m=1,nmode
               if (.not. gwpm(m)) cycle

               f = spfdof(1,m)
               if (gwpmodelm(m) .eq. 0) then
                  write(ilog,'(a7,a13,4(i7,2x))') &
                        modelabel(f),' vMCG        ',&
                        gwpcxm(m),igwpdynm(m),gwpintordm(m),gwpintm(m)
               else if (gwpmodelm(m) .eq. 1) then
                  write(ilog,'(a7,a13,4(i7,2x))') &
                        modelabel(f),' Classical   ',&
                        gwpcxm(m),igwpdynm(m),gwpintordm(m),gwpintm(m)
               else if (gwpmodelm(m) .eq. 2) then
                  write(ilog,'(a7,a13,4(i7,2x))') &
                        modelabel(f),' Independent ',&
                        gwpcxm(m),igwpdynm(m),gwpintordm(m),gwpintm(m)
               else if (gwpmodelm(m) .eq. 3) then
                  write(ilog,'(a7,a13,4(i7,2x))') &
                        modelabel(f),' Saddle      ',&
                        gwpcxm(m),igwpdynm(m),gwpintordm(m),gwpintm(m)
               endif
            enddo
  
           
         endif

         if (gwpeoms .eq. 0) then
            write(ilog,'(a)') &
                'GWP EOMs solved using regularisation of C matrix'
         else if (gwpeoms .eq. 2) then
            write(ilog,'(a)') &
                'GWP EOMs solved using LU decomposition'
         else if (gwpeoms .eq. 3) then
            write(ilog,'(a)') &
                'GWP EOMs solved using Conjugate Gradient algorithm'
         else if (gwpeoms .eq. 4) then
            write(ilog,'(a)') &
                'GWP EOMs solved using Direct inversion of C-matrix'
         else if (gwpeoms .eq. 5) then
            write(ilog,'(a)') &
                'GWP EOMs solved using Tikhonov regularisation'
         else if (gwpeoms .eq. 6) then
            write(ilog,'(a)') &
                'GWP EOMs solved using SVD'
         else if (gwpeoms .eq. 7) then
            write(ilog,'(a)') &
                'GWP EOMs solved using Cholesky decomposition'
         else if (gwpeoms .eq. 8) then
            write(ilog,'(a)') &
                'GWP EOMs solved using transformation and direct solution'
         endif
         if (ovrinv .eq. 0) then
            write(ilog,'(a)') 'GWP overlap matrix regularised on inversion'
         else if (ovrinv .eq. 1) then
            write(ilog,'(a)') 'GWP overlap matrix directly inverted'
         else if (ovrinv .eq. 2) then
            write(ilog,'(a)') 'GWP overlap matrix inverted using SVD.'
         endif

         if (.not. gwptype .eq. 2 .and. .not. gwptype .eq. 3) then
            write(ilog,'(a)') &
              'WARNING: No 1D densities for non-separable GWPs'
         endif

         if (linchk) then
            write(ilog,'(a,e12.4)') & 
              'Checking for linear dependencies. Threshold: ',ovlthr
         endif
         if (lcysinv) then
            write(ilog,'(a)') &
               'Projector uses inverse of overlap matrix'
         else
            write(ilog,'(a)') &
               'Projector uses Gram-Schmidt orthonormal functions'
         endif

         if (lforcelha) then
            write(ilog,'(a)') 'LHA used for all potential terms.'
         endif
            
      endif
!
 100  continue
      write(ilog,*)
      write(ilog,'(A)') '***** Numerical parameters *****'
      write(ilog,*)

      write(ilog,'(A,34x,1PE10.3)') 'Machine precision: ', macheps
      write(ilog,'(A,1PE10.3)') &
            'Parameter used to regularise inverse density matrix: ', epsmat
      write(ilog,'(A,1PE10.3)') &
            'Parameter used to regularise SVD singular values   : ', EpsSVD

      if (lgwp) then
         write(ilog,'(A,1PE10.3)') &
            'Parameter used to regularise inverse C-matrix      : ', epsgwp
         if (lcysinv) then
            write(ilog,'(A,1PE10.3)') &
            'Parameter used to regularise inverse overlap matrix: ', epsgsmat
         endif
      endif

      Write(ILog,*)
      Write(ILog,'(A)') '******** Flags  ********'
      Write(ILog,*)

      if(lTDVR) Write(ILog,'(A)') &
         'TDVR is used to evaluate the interaction potential '
      if(lcDVR) Write(ILog,'(A)') &
         'CDVR is used to evaluate the interaction potential '
      if(lnpot) Write(ILog,'(A)') &
         'natural potentials are used '
      if(lautodoub) Write(ILog,'(A)') &
         'Autocorrelation function is written twice per TOut '

!-----------------------------------------------------------------------
! log any complete or frozen modes
!-----------------------------------------------------------------------
      lfreeze = .false.
      do m=1,nmode
         if (m .eq. meb) cycle

         if (complete(m)) lfreeze=.true.
      enddo

      if (lfreeze) then
         Write(ILog,*)
         Write(ILog,'(A)') '******** Time-independent DOFs  ********'
         Write(ILog,*)

         do m=1,nmode
            if (m .eq. meb) cycle
            if (.not. complete(m)) cycle

            if (any(dim(m,1:nstate) .ne. subdim(m))) then
               f=spfdof(1,m)
               write(ilog,'(2a)') &
                       ' Time-independent (frozen) mode: ',modelabel(f)
            else
               f=spfdof(1,m)
               write(ilog,'(2a)') &
                       ' Time-independent (complete) mode:',modelabel(f)
            endif
         enddo
      endif

!-----------------------------------------------------------------------
! make final checks
!-----------------------------------------------------------------------
      if (ltdh(1) .and. lconst) then
         routine='Logint'
!         message=
!     +     'Cannot use CMF integrator with time-dependent operators'
!         call errormsg
         message= 'The use of CMF integrator with time-dependent '// &
           'operators is only safe when the td-op is uncorrelated. '
         call warnmsg1
      endif

      call myflush(ilog)

      return
      end subroutine logint

!#######################################################################

      subroutine logdd

      implicit none

      write(ilog,*)
      write(ilog,'(a)') '***** DD algorithm *****'
      write(ilog,*)
      if (dddiab .eq. 0) then
         write(ilog,'(a)') 'Using uncoupled adiabatic picture'
      else if (dddiab .eq. 1) then
         write(ilog,'(a)') 'Using regularized diabatic picture'
      else if (dddiab .eq. 2) then
         write(ilog,'(a)') 'Using local diabatic picture'
      else if (dddiab .eq. 3 .or. dddiab .eq. 4) then
         write(ilog,'(a)') 'Using global diabatic picture'
      else if (dddiab .eq. 5) then
         write(ilog,'(a)') 'Using projected diabatic picture'
      else 
         write(ilog,'(a)') 'Using unknown picture'
      endif

      write(ilog,'(a,f10.6)') 'DBmin parameter : ',ddmindb
      if (ldbsave) write(ilog,'(a)') 'Storing DB in memory'
      if (ldbsmall) write(ilog,'(a,i5)') &
         'Using local DBs for each GWP. Size: ',numrec
      if (lupdhes) write(ilog,'(a)') 'Using Hessian updating'
      write(ilog,'(a)') 'Shepard Interpolation parameters :'
      write(ilog,'(a,i5,a)') &
         '     Confidence radius defined by nearest ',dbnconf,' points'
      write(ilog,'(a,i5,i5)') &
         '     Power laws for inner and outer weights: ',dbinterord,dbinterord1
      if (dbmaxe .lt. 900.0) then
         write(ilog,'(a,f10.4,a)') &
            '     Maximum energy threshold for : ',dbmaxe*ev,' eV'
      endif
      if (dddiab .eq. 4) then
         if (diabint .eq. 0) then
            write(ilog,'(a,f10.4,a)') &
            '     Global diabatisation uses Pseudo-Shepard Interpolants '
         else if (diabint .eq. 1) then
            write(ilog,'(a,f10.4,a)') &
            '     Global diabatisation uses Linear interpolants'
         endif
      endif

      call myflush(ilog)

      return
      end subroutine logdd

      end module logintmod
