!#######################################################################

      module mfieldsmod

use decimal, only: long, dop
use maxv
use global
use globmemmod
use logdat
use dvrdatmod
use griddatmod
use operdef
use psidef
use lctmod
use hpsimod
use runpropmod
use eqofmotion1, only: compute_mfields_tree
use openmpmod
use ciselect, only: venpsi,venpsi1,venindex1
use psiutil, only: phipnt,phipnt1,adecoder_nomeb,amdecoder_nomeb,&
                   decoderm

#ifdef OMP
use omp_lib
#endif

use distribmod
use mpidata
use mpiutilsmod, only: mpibcastchunks_cmplx,mpe_decomp1d
#ifdef MPI
use mpi
use distribmod
use mpidata
#endif
use potevalmod
use d2tens, only: mkherm


use xvlib,   only: vvtxzz, xvxczza, xvxxzza, xmxxzzo, xqxxzzo, dvxxdzo
use mtlib,   only: mtxxzz, mtaxzz, qtxxzz, qtaxzz, dtxxdzo, dtxxdz
use mmlib,   only: mmaxzzh, mqxazz,ddxxdz
use rmlib,   only: rmmxxxzz, rmhxxxzz, rmmtxxzz, rmaxxxzz
use op1lib,  only: cpvxz, zerovxz, cpvxi, cpvxz_s, zeromxz, cpmxz, cpmxz
use op2lib,  only: addvxxzo, swapsxxi, addmxxzo

use timing
use datenmod
use channels

      implicit none
      private
      public :: mfields,kmfields,wkmfields,getcdvr
#ifdef MPI
      public :: mpimfields
#endif

      integer(long)                                :: nklha
      integer(long), dimension(:), allocatable     :: hklha,npklha,kklha
      logical(kind=4), dimension(:,:), allocatable :: hlignore 

      contains

!#######################################################################
!
! MCTDH-module MFIELDS
!
!
! modus = 0 : calculate mean-fields and dtpsi(adim)
! modus = 1 : calculate mean-fields only
!
! calctdk must be called before mfields is used to calculate 
! time-dependent coefficients 
!
! In an ML-MCTDH calc, only the mean-fields are calculated (i.e.
! modus=1 always)
! 
! size of psi1 varies with usage.
!#######################################################################

      subroutine mfields(psi,dtpsi,nham,psi1,modus,lsym)

      implicit none

      integer(long)                                  :: s,s1,k,n,k1,k2,&
                                                        h,m
      integer(long), intent(in)                      :: nham,modus
      complex(dop), dimension(dgldim), intent(in)    :: psi
      complex(dop), dimension(*), intent(in)         :: psi1   ! LHS coeffs
      complex(dop), dimension(dgldim), intent(inout) :: dtpsi
      complex(dop), dimension(:,:), allocatable      :: tmpdtpsi
      logical(kind=4), intent(in)                    :: lsym
      integer(long), save                            :: tid=0 ! timer ID

      integer(long)                                 :: hl,ilbl,iord
      character(len=c2)                             :: label,nplabel
      integer(long)            :: ithr
       
      call get_timer(tid, 'Mfields')
      call start_timer(tid)
      
!-----------------------------------------------------------------------
! allocate memory for omp threads
!-----------------------------------------------------------------------
      if (lompthread) then
         allocate(tmpdtpsi(adim,0:ompthread-1))
      else
         allocate(tmpdtpsi(1,1))
      endif
      tmpdtpsi=0.0_dop

      if (lcdvr.or.ltdvr) then
         call getcdvr(psi,mc(mccdvr2),mc(mccdvr4),mr(mrcdvr3), &
                      mr(mrort),0)
      endif

      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1

!-----------------------------------------------------------------------
! Initialize and calculate Hamiltonian moments for correlated LHA
! 1. Get how many potentials to be treated (nklha)
! 2. Allocate arrays
! 3. Load term no. defining LHA potential into hklha and potential
!    no. for each Hamiltonian term into npklha
! 4. Calculate potentials for each configuration
!-----------------------------------------------------------------------
      allocate(hlignore(maxdim,maxkoe))
      hlignore = .false.  ! if set, mfields are not calculated
      if (lklha) then
         nklha=0
         do k=k1,k2
            if (klha(k) .eq. 1) nklha = nklha+1
         enddo

         allocate(hklha(nklha),npklha(ktot),kklha(nklha))
         call alloc_klha(nklha)

         n=0
         npklha(:)=0
         nplabel = ' '
         hklha(:) = 0
         kklha(:) = 0
         do k=k1,k2
            h=hamilton(fkoe,k)
            hl = hterm(1,h)
            label = hoplab(hl)

            ilbl = index(label,'%')
            read(label(ilbl+1:ilbl+1),*) iord
            if (iord .eq. 0) then
               n=n+1
               hklha(n) = h
               kklha(n) = k
               nplabel = label(1:ilbl-1)
               npklha(k) = n
               if (ksi(k) .ne. ksf(k)) then
                  do m=1,nmode
                     if (gwpmodelm(m) .gt. 0) hlignore(m,k) = .true.
                  enddo
               endif
            else if (label(1:ilbl-1) .eq. nplabel(1:ilbl-1)) then
               npklha(k) = n

               do m=1,nmode
                  if (gwpmodelm(m) .gt. 0) hlignore(m,k) = .true.
               enddo
            endif
                  
         enddo

         do n=1,nklha
            h=hklha(n)
            call klhaconfig(h,n,psi(adim+1),adim+1)
         enddo

! Calculate mean-field forces if mode uses CX
         do m=1,nmode
            if (m .eq. meb) cycle
            if (gwpcxm(m) .eq. 0) cycle  
            do n=1,nklha
               k=kklha(n)   ! Get term no. for start of LHA 
               if (ksi(k) .ne. ksf(k)) cycle  ! Off-diagonal term
               s = ki(k)
               call mfder1(k,m,s,n,ghderiv1(ghder1(k,m)),psi,hteil,&
                    nspfdof(m),gwpdim(m,s))
            enddo
         enddo

      endif

! Selected CI
      if (citype .gt. 0) then
         do k=k1,k2
            s=kf(k)
            s1=ki(k)
            if (s1 .ne. s) then
               call matrizen6(k,s,s1,psi,dtpsi,hteil, &
                    mc(mchloch),mi(mijindx),mi(miworki))
            else
               call matrizen5(k,s,psi,dtpsi,hteil,mc(mchloch), &
                    mi(mijindx),mi(miworki))
            endif
         enddo

      else
#ifdef MPI
         if(lmpi.and.lmpimfields.and.lmpistarted) then
            mpitask=3
! --- Determine Job
            call MPI_Bcast(mpitask,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
            call mpimfields(psi,dtpsi,psi1,mc(mchloch),hteil,nham,lsym,modus)

!-----------------------------------------------------------------------
!     If no parallelisation is used then a usual calculation is 
!     performed.
!-----------------------------------------------------------------------
         else
#endif

!$omp parallel num_threads(ompthread) private(k,s,s1,ithr) &
!$omp shared(klha)  if(lompmfield)
!$omp do schedule(dynamic)
         do k=k1,k2
! GWPTRj calcs do not need potential terms
            if (lgwptrj .and. (kinop(1,k) .le. 0)) cycle
! Ignored terms in KLHA
            if (klha(k) .lt. 0) cycle

#ifdef OMP
            ithr = omp_get_thread_num()
#else
            ithr = 0
#endif
            s=kf(k)
            s1=ki(k)
            h=hamilton(fkoe,k)
            if (s .eq. s1 .and. ithr .eq. 0) then
               call matrizen1(k,s,psi,dtpsi,hteil,mc(mchloch), &
                    psi1,modus,lsym)
            else if (s .eq. s1) then
               call matrizen1(k,s,psi,tmpdtpsi(1,ithr),hteil, &
                    mc(mchloch), psi1,modus,lsym)
            else if (s1 .ne. s .and. ithr .eq. 0) then
               call matrizen2(k,s,s1,psi,dtpsi,hteil, &
                    mc(mchloch),psi1,modus)
            else
               call matrizen2(k,s,s1,psi,tmpdtpsi(1,ithr),hteil, &
                    mc(mchloch),psi1,modus)
            endif
         enddo

!$omp enddo 
!$omp end parallel 
         if (lompthread) then
            do ithr = 1,ompthread-1
               call addvxxzo(tmpdtpsi(1,ithr),dtpsi,adim)
            enddo
         endif

#ifdef MPI
         endif
#endif
      endif

!$omp parallel num_threads(ompthread) private(n,s,s1,ithr) if(lompmfield)
!$omp do schedule(dynamic)
      do n=1,nmulpot(nham)
#ifdef OMP
         ithr = omp_get_thread_num()
#else
         ithr = 0
#endif
         k=zham(nham)+kzahl(nham)+n-1
         s=kf(k)
         s1=ki(k)
         if (lcdvr.or.ltdvr) then
            call matrizen4(k,psi,dtpsi,mc(mchloch),mr(mrort),mr(mrcdvr3),&
                 mc(mccdvr4),hteil,modus)
         else if (ldirmuld) then
            call matrizen7(k,s,s1,psi,dtpsi,mc(mchloch),mr(mrort),psi1,modus)
         else
            call matrizen3(k,s,s1,psi,dtpsi,mc(mchloch),mr(mrhops),psi1,&
                 hteil,modus)
         endif
      enddo
!$omp enddo 
!$omp end parallel 

#ifdef MPI
      if(lmpi.and.lmpistarted .and. lmpimfields) then
         mpitask=21
! --- Just send HLoch to slaves
         call MPI_Bcast(mpitask,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      !   call MPI_Bcast(mc(mchloch),hlochdim,MPI_DOUBLE_COMPLEX,master, &
      !       MPI_COMM_WORLD,mpierr)
         call mpibcastchunks_cmplx(mc(mchloch),hlochdim)
      end if
#endif

      if (allocated(tmpdtpsi)) deallocate(tmpdtpsi)
      if (allocated(hlignore)) deallocate(hlignore)
      if (lklha) then
         call dealloc_klha
         deallocate(hklha,npklha,kklha)
      endif

      call stop_timer(tid)

      return
      end subroutine mfields

!#######################################################################
!
!   MPIMFIELDS
!
!#######################################################################

#ifdef MPI
      subroutine mpimfields(psi,dtpsi,psi1,hloch,hteil,nham,&
                 lsym,modus)

      implicit none

      complex(dop), dimension(adim), intent(in)      :: psi
      complex(dop), dimension(*), intent(in)         :: psi1
      complex(dop), dimension(pmatdim), intent(in)   :: hteil
      complex(dop), dimension(adim), intent(out)     :: dtpsi 
      complex(dop), dimension(:), allocatable        :: tmpdtpsi
      complex(dop), dimension(:), allocatable        :: tmphloch
      complex(dop), dimension(hlochdim), intent(out) :: hloch 
      integer(long), intent(in)      :: nham,modus
      logical(kind=4), intent(in)    :: lsym

      integer(long) :: s,s1,i1,i2,k,k1,k2

      call MPI_Bcast(lklha,1,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(ktot,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      if (lklha) then
         call MPI_Bcast(nklha,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(maxgwp,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      endif

! allocate arrays on master and slaves
      if (mpirank .eq. master) then
         allocate(tmpdtpsi(adim))
         allocate(tmphloch(hlochdim))
      else if (mpirank .ne. master) then
         if (lklha) then
            allocate(npklha(ktot))
            call alloc_klha(nklha)
         endif
         allocate(hlignore(maxdim,maxkoe))
      endif


! --- Get Data
      call MPI_Bcast(nham,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(modus,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(lsym,1,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
! ... A-Vector
      call MPI_Bcast(psi,adim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(psi1,adim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)
! ... Hteil matrices
      call MPI_Bcast(hteil,pmatdim,MPI_DOUBLE_COMPLEX,master,&
                     MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hlignore,maxdim*maxkoe,MPI_LOGICAL,master,&
                     MPI_COMM_WORLD,mpierr)

! If using time-dependent operators, broadcast the coefficients again.
      if(ftime.gt.0) call MPI_Bcast(koeff,maxkoe,MPI_DOUBLE_COMPLEX,master,&
                          MPI_COMM_WORLD,mpierr)

! Initialize Hamiltonian moments for correlated LHA
      call MPI_Bcast(klha,ktot,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      if (lklha) then
         call MPI_Bcast(npklha,ktot,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(mom0,maxblock*maxsta*nklha,MPI_DOUBLE,master,&
                        MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(mom1,ndof*maxblock*maxsta*nklha,MPI_DOUBLE,master,&
                        MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(mom2,ndof*ndof*maxblock*maxsta*nklha,MPI_DOUBLE,master,&
                        MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(gconfig,ndof*maxgwp*nstate,MPI_DOUBLE,master,&
                        MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(maxhop,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(hoplab,c2*maxhop,MPI_CHARACTER,master,&
                        MPI_COMM_WORLD,mpierr)
      endif

      call zerovxz(hloch,hlochdim)
      if (mpirank.gt.master) call zerovxz(dtpsi,adim)

      i1=zham(nham)
      i2=zham(nham)+kzahl(nham)-1

! Divide up the work.
      call mpe_decomp1d(i1,i2,k1,k2)

      do k=k1,k2
         s=kf(k)
         s1=ki(k)

         if (s1 .ne. s) then
            call matrizen2(k,s,s1,psi,dtpsi,hteil,hloch,psi1,modus)
         else
            call matrizen1(k,s,psi,dtpsi,hteil,hloch,psi1,modus,lsym)
         endif
      enddo

! --- Reduce Data
      call MPI_Reduce(dtpsi,tmpdtpsi,adim,MPI_DOUBLE_COMPLEX,MPI_SUM,master, &
                      MPI_COMM_WORLD,mpierr)
      if(mpirank.eq.master)  call cpvxz(tmpdtpsi,dtpsi,adim)

      call MPI_Reduce(hloch,tmphloch,hlochdim,MPI_DOUBLE_COMPLEX,MPI_SUM,&
                      master,MPI_COMM_WORLD,mpierr)
      if(mpirank.eq.master) call cpvxz(tmphloch,hloch,hlochdim)

      if (mpirank .eq. master) then
         deallocate(tmpdtpsi)
         deallocate(tmphloch)
      else if (mpirank .ne. master) then
         if (lklha) then
            call dealloc_klha
            deallocate(npklha)
         endif
         deallocate(hlignore)
      endif

      return
      end subroutine mpimfields

#endif

!#######################################################################
! MATRIZEN1
!
! For Hamiltonian terms involving only a single state,
! calculates the mean field matrices. It also calculates the 
! time derivative of the MCTDH coefficients.
!
! This involves multiplying the coefficients by the
! corresponding single particle operator matrix elements.
!
! input:
! hteil, workc are standard arrays, see
!   documentation
! block,dim,vdim,ndim are standard arrays for one state
! psi  : coefficients for relevant state
! k    : number of correlated Hamiltonian term
! output:
! hloch: mean fields for kth Hamiltonian term
! dtpsi: derivative wrt time of wavefunction coefficients
!
! V5.0 MB
!
!#######################################################################

      subroutine matrizen1(k,s,psi,dtpsi,hteil,hloch,psi1,modus,lsym)

      implicit none
      
      integer(long)                                      :: zeig1,zeig2,zeig3,&
                                                            m,m1,h
      integer(long), intent(in)                          :: k,s,modus
      complex(dop), dimension(dgldim), intent(in)        :: psi
      complex(dop), dimension(*), intent(in)             :: psi1
      complex(dop), dimension(adim), intent(inout)       :: dtpsi
      complex(dop), dimension(pmatdim), intent(in)       :: hteil
      complex(dop), dimension(hlochdim), intent(out)     :: hloch
      complex(dop), dimension(3*maxblock)                :: workc
      complex(dop), dimension(:), allocatable            :: psig
      complex(dop), dimension(maxblock)                  :: psilhs
      complex(dop)                                       :: ztmp
      logical(kind=4)                                    :: work2eq3
      logical(kind=4), intent(in)                        :: lsym
      real(dop), dimension(:), allocatable               :: koelha

      if (klha(k) .gt. 0) then
         allocate(psig(maxblock))
         allocate(koelha(maxblock))
      endif

! --- INITIALISE VARIABLES ---------------------------------------------
! workc is divided into three parts of length block.
! The result of a multiplication is stored after zeig2.
! Before multiplication, the vector to be multiplied is moved to after
! zeig1. After zeig3 is stored the vector multiplied up to mode m-1.
!-----------------------------------------------------------------------
      zeig1 = block(s)+1
      zeig2 = 1
      zeig3 = 2*block(s)+1
      call cpvxz(psi(zpsi(s)),workc(zeig2),block(s))
      work2eq3 = .false.

!-----------------------------------------------------------------------
!  If correlated LHA calculate coefficients and transform to
!  GWP basis
!-----------------------------------------------------------------------
      if (klha(k) .gt. 0) then
         h = hamilton(fkoe,k)
         call klhacalc(koelha,s,h,npklha(k))

! transform to GWP basis
         if (psitype .eq. 0 .or. psitype .eq. 5) then
            do m = 1,nmode
               if (.not. gwpm(m)) cycle

               call swapsxxi(zeig1,zeig2)
               call qtxxzz(psi(zdgwp(m,s)),workc(zeig1), &
                    workc(zeig2),vdim(m,s),dim(m,s),ndim(m,s))
            enddo
! save transformed psi
            call cpvxz(workc(zeig2),psig,block(s))
         else
            call cpvxz(psi(zpsi(s)),psig,block(s))
         endif

! multiply by LHA coefficients
         call swapsxxi(zeig1,zeig2)
         call ddxxdz(koelha,workc(zeig1),workc(zeig2),block(s))

! NB - it is assumed that psi1 = psi for klha. This is true during propagation
! but not if used in opwf, which would then require a transformation of
! psi1
         call cpvxz(psig,psilhs,block(s))
      else if (.not. lsvd) then
         call cpvxz(psi1(zpsi(s)),psilhs,block(s))
      endif

!-----------------------------------------------------------------------
!  Loop over modes. 
!  First set initial pointers, then multiply wavefunction 
!  with single-particle hamiltonian matrices corresponding to 
!  dimensions numbers lower than mode 
!-----------------------------------------------------------------------
      do m=1,nmode
         if (lsvd) then
            call cpvxz(psi1(zpsi(s)+(m-1)*adim),psilhs,block(s))
         endif

         if (m.ne.1) then
            if (.not. work2eq3) then
               call cpvxz(workc(zeig3),workc(zeig2),block(s))
               work2eq3 = .true.
            endif
            if (.not.diag(m-1,k)) then
               call swapsxxi(zeig1,zeig2)
               work2eq3 = .false.
               call qtxxzz(hteil(pmat(m-1,k)),workc(zeig1), &
                    workc(zeig2),vdim(m-1,s),dim(m-1,s),ndim(m-1,s))
            endif
         endif
!-----------------------------------------------------------------------
!  save wavefunction multiplied up to mode m-1, if it has changed.
!-----------------------------------------------------------------------
         if (.not. work2eq3) then
            call cpvxz(workc(zeig2),workc(zeig3),block(s))
            work2eq3 = .true.
         endif

         if (hlignore(m,k)) cycle
!-----------------------------------------------------------------------
!  Multiply array with single-particle hamiltonian matrices for
!  modes higher than m.
!  If H is unitoperator for mode m, or basis is complete, do not bother 
!  as hloch is never needed.
!-----------------------------------------------------------------------
         if (.not. diag(m,k) .and. .not. complete(m)) then
            do m1=m+1,nmode
               if (.not.diag(m1,k)) then
                  call swapsxxi(zeig1,zeig2)
                  work2eq3 = .false.
                  call qtxxzz(hteil(pmat(m1,k)),workc(zeig1), &
                       workc(zeig2),vdim(m1,s),dim(m1,s),ndim(m1,s))
               endif
            enddo

!-----------------------------------------------------------------------
!  calculate single-hole matrix
!-----------------------------------------------------------------------
            if (hlsym(m,k) .eq. 0 .or. .not. lsym .or. &
                       (klha(k) .ge. 0)) then
               call rmmxxxzz(psilhs,workc(zeig2), &
                    hloch(phloch(m,k)),vdim(m,s),dim(m,s),ndim(m,s), &
                    dim(m,s))
            else if (hlsym(m,k) .eq. 1) then
               call rmhxxxzz(psilhs,workc(zeig2), &
                    hloch(phloch(m,k)),vdim(m,s),dim(m,s),ndim(m,s))
            else if (hlsym(m,k) .eq. -1) then
               call rmaxxxzz(psilhs,workc(zeig2), &
                    hloch(phloch(m,k)),vdim(m,s),dim(m,s),ndim(m,s))
            endif

            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1 .and. htmtype(h) .ne. 104)  &
               call xqxxzzo(koeff(k),hloch(phloch(m,k)),dim(m,s))
         endif
      enddo  ! m=1,nmode

      if (modus .eq. 1) go to 999

!-----------------------------------------------------------------------
! Now calculate the time derivative of the coefficients.
! calculation of hloch for final mode has left workc(zeig2) with the 
! product of psi(block) and hteil matrices for all, excluding the final 
! mode. 
!-----------------------------------------------------------------------
      m=nmode
      if (.not.diag(m,k)) then
         call swapsxxi(zeig1,zeig2)
         work2eq3 = .false.
         call qtxxzz(hteil(pmat(m,k)),workc(zeig1),workc(zeig2), &
              vdim(m,s),dim(m,s),ndim(m,s))
      endif

!-----------------------------------------------------------------------
!  If correlated LHA calculate coefficients transform back to SPF basis
!-----------------------------------------------------------------------
      if (klha(k) .gt. 0) then
         if (psitype .eq. 0 .or. psitype .eq. 5) then
            do m = 1,nmode
               if (.not. gwpm(m)) cycle

               call swapsxxi(zeig1,zeig2)
               call qtaxzz(psi(zdgwp(m,s)),workc(zeig1), &
                    workc(zeig2),vdim(m,s),dim(m,s),ndim(m,s))
            enddo
         endif
      endif

!-----------------------------------------------------------------------
!  multiply by Hamiltonian coefficient and add to dtpsi
!-----------------------------------------------------------------------
      h=hamilton(fkoe,k)

      if (htmtype(h) .eq. 1) then
         call addvxxzo(workc(zeig2),dtpsi(zpsi(s)),block(s))
      else if (htmtype(h) .eq. 104) then
         ztmp = 0.5_dop
         call xvxxzza(ztmp,workc(zeig2),dtpsi(zpsi(s)),block(s))
      else
         call xvxxzza(koeff(k),workc(zeig2),dtpsi(zpsi(s)),block(s))
      endif

!----------------------------------------------------------------------- 
! If LHA, add symmetric part to HA
!----------------------------------------------------------------------- 
      if (klha(k) .gt. 0) then
         zeig1 = block(s)+1
         zeig2 = 1
         call cpvxz(psig,workc(zeig2),block(s))

         do m = 1,nmode

            if (.not. diag(m,k)) then

!       --- SWAP POINTERS OR SELECT COEFFICIENTS ---

               zeig3 = zeig1
               zeig1 = zeig2
               zeig2 = zeig3

!       --- MULTIPLY WITH MATRIX ELEMENTS ---

               call qtaxzz(hteil(pmat(m,k)),workc(zeig1), &
                    workc(zeig2),vdim(m,s),dim(m,s),ndim(m,s))
           endif
         enddo

!-----------------------------------------------------------------------
!  If correlated LHA calculate coefficients transform back to SPF basis
!-----------------------------------------------------------------------
         call swapsxxi(zeig1,zeig2)
         call ddxxdz(koelha,workc(zeig1),workc(zeig2),block(s))

         if (psitype .eq. 0 .or. psitype .eq. 5) then
            do m = 1,nmode
               if (.not. gwpm(m)) cycle

               call swapsxxi(zeig1,zeig2)
               call qtaxzz(psi(zdgwp(m,s)),workc(zeig1), &
                    workc(zeig2),vdim(m,s),dim(m,s),ndim(m,s))
            enddo
         endif

!    --- STORE RIGHT HAND SIDE IN DTPSI ---

         ztmp = 0.5_dop
         call xvxxzza(ztmp,workc(zeig2),dtpsi(zpsi(s)),block(s))
      endif

  999 continue

      if (klha(k) .gt. 0) then
         deallocate(psig,koelha)
      endif

      return
      end subroutine matrizen1

!#######################################################################
! MATRIZEN2
!
! called by funk,funkr
! calls to usmatvec,usrhomat,matnum
!
! calculates the mean field matrices involving hamiltonian coefficients
!  that connect two states. It also calculates the time derivative of 
!  the MCTDH coefficients.
!
! This involves multiplying the coefficients of the state s1 by the
! corresponding single particle operator matrix elements.
!
! input:
! s: state on LHS of mean field matrix
! s1: state on RHS of mean field matrix
! psi: coefficients for all states 
! hteil, workc are standard arrays, see documentation
! k: which correlated Hamiltonian coefficient passed from calling
! routine
! output:
! dtpsi: derivative wrt time of A coefficients for state on LHS of
!        mean field matrix
! hloch: mean field for kth Hamiltonian term.
!
! V6.0 GW
!#######################################################################

      subroutine matrizen2(k,s,s1,psi,dtpsi,hteil,hloch,psi1,modus)

      implicit none

      integer(long)                                  :: zeig1,zeig2,&
                                                        zeig3,swapzeig,&
                                                        m,m1,block1,&
                                                        vdim1,h
      integer(long), intent(in)                      :: k,s,s1,modus
      complex(dop), dimension(adim), intent(in)      :: psi
      complex(dop), dimension(*), intent(in)         :: psi1
      complex(dop), dimension(adim), intent(inout)   :: dtpsi
      complex(dop), dimension(pmatdim), intent(in)   :: hteil
      complex(dop), dimension(hlochdim), intent(out) :: hloch
      complex(dop), dimension(1:3*maxblock)          :: workc
      complex(dop), dimension(maxblock)              :: psilhs
      real(dop), dimension(:), allocatable           :: koelha,koelha1

      if (klha(k) .gt. 0) then
         allocate(koelha(maxblock))
         allocate(koelha1(maxblock))
      endif

! --- INITIALISE VARIABLES ---------------------------------------------
! workc is divided into three parts of length maxblock, the longest
! possible
! length of the array during the multiplication.
! The result of a multiplication is stored after zeig2.
! Before multiplication, the vector to be multiplied is moved to after
! zeig1.
! After zeig3 is stored the vector multiplied up to mode m-1
!-----------------------------------------------------------------------
      call zerovxz(workc,3*MaxBlock)
      zeig1=maxblock+1
      zeig2=1
      zeig3=2*maxblock+1
      if (klha(k) .gt. 0) then
! for klha transform LHS WF to GWP basis 
         if (psitype .eq. 0 .or. psitype .eq. 5) then
            call cpvxz(psi(zpsi(s)),workc(zeig2),block(s))
            do m = 1,nmode
               if (.not. gwpm(m)) cycle

               call swapsxxi(zeig1,zeig2)
               call qtxxzz(psi(zdgwp(m,s)),workc(zeig1), &
                    workc(zeig2),vdim(m,s),dim(m,s),ndim(m,s))
            enddo
! save transformed psi at zeig3
            call cpvxz(workc(zeig2),workc(zeig3),block(s))
         else
            call cpvxz(psi(zpsi(s)),workc(zeig3),block(s))
         endif
      endif
! save RHS WF at zeig2
      call cpvxz(psi(zpsi(s1)),workc(zeig2),block(s1))
      block1=block(s1)

!-----------------------------------------------------------------------
!  If correlated LHA calculate coefficients and transform to
!  GWP basis
!-----------------------------------------------------------------------
      if (klha(k) .gt. 0) then
         h = hamilton(fkoe,k)
         call klhacalc(koelha,s,h,npklha(k))
         call klhacalc(koelha1,s1,h,npklha(k))

! transform to GWP basis
         if (psitype .eq. 0 .or. psitype .eq. 5) then
            do m = 1,nmode
               if (.not. gwpm(m)) cycle

               call swapsxxi(zeig1,zeig2)
               call qtxxzz(psi(zdgwp(m,s1)),workc(zeig1), &
                    workc(zeig2),vdim(m,s1),dim(m,s1),ndim(m,s1))
            enddo
         endif

! multiply by LHA coefficients
         call swapsxxi(zeig1,zeig2)
         call ddxxdz(koelha1,workc(zeig1),workc(zeig2),block(s1))

! Save wavefunction, operated on by LHA coefficients, for LHS of
! mean-fields
         call ddxxdz(koelha,workc(zeig3),psilhs,block(s))

      else if (.not. lsvd) then
         call cpvxz(psi1(zpsi(s)),psilhs,block(s))

      endif

!-----------------------------------------------------------------------
!  First set initial pointers, then multiply wavefunction of state s1
!  with single-particle hamiltonian matrices corresponding to 
!  dimensions numbers lower than mode 
!  as the matrix elements are between different states with different
!   number of coefficients, the following pointers are needed:
!  block1: number of elements in multiplied array.
!  vdim1: equivalent of vdim during multiplication over the modes lower
!  than m.
!  vdim2: equivalent of vdim during multiplication over the modes higher
!  then m.
!  before a multiplication, block=vdim1*dim(m,s1)*ndim(m,s1)
!  after a multiplication, block=vdim1*dim(m,s)*ndim(m,s1)
!-----------------------------------------------------------------------
      do m=1,nmode
         if (lsvd) then
            call cpvxz(psi1(zpsi(s)+(m-1)*adim),psilhs,block(s))
         endif
         if (m.ne.1) then
            call cpvxz(workc(zeig3),workc(zeig2),block1)

            swapzeig=zeig1
            zeig1=zeig2
            zeig2=swapzeig
            call mtxxzz(hteil(pmat(m-1,k)),workc(zeig1), &
                 workc(zeig2),vdim(m-1,s),dim(m-1,s1),ndim(m-1,s1), &
                 dim(m-1,s))
            block1=vdim(m-1,s)*dim(m-1,s)*ndim(m-1,s1)
         endif

!-----------------------------------------------------------------------
!  save wavefunction multiplied up to m-1, if it has changed.
!-----------------------------------------------------------------------
         call cpvxz(workc(zeig2),workc(zeig3),block1)

!-----------------------------------------------------------------------
!  Multiply array with single-particle hamiltonian matrices for
!  modes higher than m.
!  If H is unit-operator for mode m, or basis is complete, do not bother 
!  as hloch is never needed.
!-----------------------------------------------------------------------
         if (.not. complete(m)) then
            vdim1=vdim(m,s)*dim(m,s1)
            do m1=m+1,nmode
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig

               call mtxxzz(hteil(pmat(m1,k)),workc(zeig1), &
                    workc(zeig2),vdim1,dim(m1,s1),ndim(m1,s1),dim(m1,s))
               vdim1=vdim1*dim(m1,s)
            enddo
!-----------------------------------------------------------------------
!  calculate single-hole matrix
!-----------------------------------------------------------------------
            call rmmxxxzz(psilhs,workc(zeig2),&
                 hloch(phloch(m,k)),vdim(m,s),dim(m,s),ndim(m,s),&
                 dim(m,s1))

            h=hamilton(fkoe,k)
            if (htmtype(h) .ne. 1 .and. htmtype(h) .ne. 104) &
               call xmxxzzo(koeff(k),hloch(phloch(m,k)),dim(m,s),&
                    dim(m,s1))
         endif
      enddo

      if (modus .eq. 1) go to 999

!-----------------------------------------------------------------------
! now calculate the time derivative of the coefficients.
! calculation of hloch for final mode has left workc(zeig2) with the 
! product of psi(block) and hteil matrices for all, excluding the final 
! mode. 
!-----------------------------------------------------------------------
      m=nmode
      swapzeig=zeig1
      zeig1=zeig2
      zeig2=swapzeig
      call mtxxzz(hteil(pmat(m,k)),workc(zeig1),workc(zeig2),&
           vdim(m,s),dim(m,s1),ndim(m,s),dim(m,s))

!-----------------------------------------------------------------------
!  If correlated LHA calculate coefficients transform back to SPF basis
!-----------------------------------------------------------------------
      if (klha(k) .gt. 0) then
         if (psitype .eq. 0 .or. psitype .eq. 5) then
            do m = 1,nmode
               if (.not. gwpm(m)) cycle

               call swapsxxi(zeig1,zeig2)
               call qtaxzz(psi(zdgwp(m,s)),workc(zeig1), &
                    workc(zeig2),vdim(m,s),dim(m,s),ndim(m,s))
            enddo
         endif
      endif

!-----------------------------------------------------------------------
!  multiply by Hamiltonian coefficient
!-----------------------------------------------------------------------
      h=hamilton(fkoe,k)

      if (htmtype(h) .eq. 1 .or. htmtype(h) .eq. 104) then
         call addvxxzo(workc(zeig2),dtpsi(zpsi(s)),block(s))
      else
         call xvxxzza(koeff(k),workc(zeig2),dtpsi(zpsi(s)),block(s))
      endif

  999 continue

      if (klha(k) .gt. 0) then
         deallocate(koelha,koelha1)
      endif

      return
      end subroutine matrizen2

!#######################################################################
! MATRIZEN3
!
! called by funk,funkr
! calls to usmatvec,usrhomat,matnum
!
! calculates the mean field matrices involving hamiltonian terms
!  that are multidimensional. It also calculates the time derivative of 
!  the MCTDH coefficients.
!
! This involves multiplying the coefficients of the state s1 by the
! corresponding psi elements to transform from the spf basis to the
! grid point basis. After this, the operator can be applied, and the
! resulting vector transformed back to the spf basis.
!
! input:
! s: state on LHS of mean field matrix
! s1: state on RHS of mean field matrix
! psi, hteil, workc are standard arrays, see documentation
! k: which correlated Hamiltonian term
! output:
! hloch: mean field matrices for kth Hamiltonian term
! dtpsi: derivative wrt time of A coefficients for state on LHS of
!        mean field matrix
!
! V6.0 GW
!#######################################################################

      subroutine matrizen3(k,s,s1,psi,dtpsi,hloch,hops,psi1,hteil,modus)

      implicit none
      
      integer(long)                                      :: zeig1,zeig2,zeig3, &
                                                            swapzeig,m,m1,b, &
                                                            block1,block2,&
                                                            vdim1,vdim2,ndim1, &
                                                            ndim2,h,muld,hkoe,f
      integer(long), intent(in)                          :: k,s,s1,modus
      real(dop), dimension(hopsdim), intent(in)          :: hops
      complex(dop), dimension(dgldim), intent(in)        :: psi,psi1
      complex(dop), dimension(pmatdim), intent(in)       :: hteil
      complex(dop), dimension(adim), intent(inout)       :: dtpsi
      complex(dop), dimension(hlochdim), intent(out)     :: hloch
      complex(dop), dimension(3*maxblock)                :: workc
      logical(kind=4)                                    :: work2eq3

! --- INITIALISE VARIABLES ---------------------------------------------
! workc is divided into three parts of length maxblock, the longest
! possible
! length of the array during the multiplication.
! The result of a multiplication is stored after zeig2.
! Before multiplication, the vector to be multiplied is moved to after
! zeig1.
! After zeig3 is stored the vector multiplied up to mode m-1
!-----------------------------------------------------------------------
      call zerovxz(workc,3*MaxBlock)
      zeig1=maxblock+1
      zeig2=1
      zeig3=2*maxblock+1

!-----------------------------------------------------------------------
! copy coeffs to workc(zeig2). At this point there are "block(s1)"
! coeffs
!-----------------------------------------------------------------------
      call cpvxz(psi(zpsi(s1)),workc(zeig2),block(s1))

!-----------------------------------------------------------------------
! Operate on coeffs by electronic operator (single set)
!-----------------------------------------------------------------------
      if (leb .and. meb .gt. 0) then
         swapzeig=zeig1
         zeig1=zeig2
         zeig2=swapzeig
         call qtxxzz(hteil(pmat(meb,k)),workc(zeig1), &
              workc(zeig2),vdim(meb,s1),dim(meb,s1),ndim(meb,s1))
      endif

!-----------------------------------------------------------------------
!  now transform from spf basis to grid point basis
!  block1 is the number of elements in the transformed array.
!  vdim1 is equivalent to vdim, but takes into account growth of block1.
!  before multiplication, block1=vdim1*dim(m,s1)*ndim(m,s1)
!  after multiplication,  block1 has increased to 
!  vdim1*subdim(m)*ndim(m,s1)
!  and vdim1 changes to vdim1*subdim(m) for the next mode.
!-----------------------------------------------------------------------
      f=fterm(k)
      h=hamilton(f,k)
      muld=htmmuld(h)
      vdim1=1
      block1=0
      do m=1,nmode
         if (diag(m,k) .or. m .eq. meb) then   ! Does not work for nodiag
            block1=vdim1*dim(m,s1)*ndim(m,s1)
            vdim1=vdim1*dim(m,s1)
         else
            swapzeig=zeig1
            zeig1=zeig2
            zeig2=swapzeig
            call mtxxzz (psi(zetf(m,s1)),workc(zeig1), &
                 workc(zeig2),vdim1,dim(m,s1),ndim(m,s1),subdim(m))
            block1=vdim1*subdim(m)*ndim(m,s1)
            vdim1=vdim1*subdim(m)
         endif
      enddo

!-----------------------------------------------------------------------
! now operate on coeffs by V(q1,....qf)
!-----------------------------------------------------------------------
      call dtxxdzo(hops(zhop(h)),workc(zeig2),vdmuld(muld), &
           muldsize(muld),ndmuld(muld)) 

!-----------------------------------------------------------------------
!  calculate single-hole matrix
!  This involves  the reverse transformation from grid points to
!  spfs, ignoring the mode m. block1 now contracts back to block(s).
!  ndim1 is equivalent to ndim, but takes into account contraction of
!  block1.
!  before multiplication, block1=vdim(m,s)*subdim(m)*ndim1
!  after multiplication,  block1 has decreased to
!  vdim(m,s)*dim(m,s)*ndim1
!-----------------------------------------------------------------------
      work2eq3=.false.
      do m=1,nmode
         if (m .eq. meb) cycle

         if (m.ne.1) then
            if (.not. work2eq3) then
               do b=1,block1
                  workc(zeig2+b-1)=workc(zeig3+b-1)
               enddo                                    
            endif
            if (.not. diag(m-1,k)) then  ! Does not work for nodiag
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig
               work2eq3=.false.
               ndim1=block1/(vdim(m-1,s)*subdim(m-1))
               call mtaxzz(psi1(zetf(m-1,s)),workc(zeig1), &
                    workc(zeig2),vdim(m-1,s),subdim(m-1),ndim1,dim(m-1,s))
               block1=vdim(m-1,s)*dim(m-1,s)*ndim1
            endif
         endif
!-----------------------------------------------------------------------
!  save wavefunction multiplied up to mode m-1, if it has changed.
!-----------------------------------------------------------------------
         if (.not. work2eq3) then
            do b=1,block1
               workc(zeig3+b-1)=workc(zeig2+b-1)
            enddo
            work2eq3=.true.
         endif
!-----------------------------------------------------------------------
! Multiply array with c.c. of psi vectors for modes higher than m.
! If H is unit-operator for mode m, or basis is complete, do not bother 
! as hloch is never needed.
! vdim2, ndim2 and block2 calculate the relevant dimensions of the
! array as it is contracted to spf basis in all modes except m.
!-----------------------------------------------------------------------
         if (.not. diag(m,k) .and. .not. complete(m)) then
            vdim2=vdim(m,s)*subdim(m)
            block2=block1
            do m1=m+1,nmode
               if (m1 .eq. meb) cycle 

               if (.not. diag(m1,k)) then   ! Does not work for nodiag
                  swapzeig=zeig1
                  zeig1=zeig2
                  zeig2=swapzeig
                  work2eq3=.false.
                  ndim2=block2/(vdim2*subdim(m1))
                  call mtaxzz(psi1(zetf(m1,s)),workc(zeig1), &
                       workc(zeig2),vdim2,subdim(m1),ndim2,dim(m1,s))
                  block2=vdim2*dim(m1,s)*ndim2
                  vdim2=vdim2*dim(m1,s)
               endif
            enddo

!-----------------------------------------------------------------------
!  calculate single-hole matrix
!-----------------------------------------------------------------------
            call rmmtxxzz(psi1(zpsi(s)),workc(zeig2), &
                 hloch(phloch(m,k)),vdim(m,s),dim(m,s),ndim(m,s), &
                 subdim(m))
            hkoe=hamilton(fkoe,k)
            if (htmtype(hkoe) .ne. 1) &
               call xmxxzzo(koeff(k),hloch(phloch(m,k)),subdim(m), &
                 dim(m,s))   
         endif
      enddo

      if (modus .eq. 1) go to 999

!-----------------------------------------------------------------------
! now calculate the time derivative of the coefficients.
! calculation of hloch for final mode has left workc(zeig2) with the 
! array contracted back to spf basis in all except the final mode. 
!-----------------------------------------------------------------------
      m=nmode
      if (m .eq. meb) m=m-1  ! if last mode is meb, step back  to last grid
      if (.not.diag(m,k)) then
         swapzeig=zeig1
         zeig1=zeig2
         zeig2=swapzeig
         work2eq3=.false.
         ndim1=block1/(vdim(m,s)*subdim(m))
         call mtaxzz (psi(zetf(m,s)),workc(zeig1),workc(zeig2), &
              vdim(m,s),subdim(m),ndim1,dim(m,s))
      endif

!-----------------------------------------------------------------------
!  multiply by Hamiltonian coefficient
!-----------------------------------------------------------------------
      hkoe=hamilton(fkoe,k)
      if (htmtype(hkoe) .eq. 1) then
         call addvxxzo(workc(zeig2),dtpsi(zpsi(s)),block(s))
      else
         call xvxxzza(koeff(k),workc(zeig2),dtpsi(zpsi(s)),block(s))
      endif

  999 continue

      return
      end subroutine matrizen3

!######################################################################
! MATRIZEN4
!
! called by funk,funkr
! calls to usmatvec,usrhomat,matnum
!
! calculates the mean field matrices for TDVR or CDVR.
!
! input:
! psi, hteil, workc are standard arrays, see documentation
! k: which correlated Hamiltonian term
! output:
! hloch: mean field matrices for kth Hamiltonian term
! dtpsi: derivative wrt time of A coefficients for state on LHS of
!        mean field matrix
! hteil: operator mtrices. Needed for electronic basis.
!
! V6.0 GW
!#######################################################################

      subroutine matrizen4(k,psi,dtpsi,hloch,ort,cdvr3,cdvr4,hteil,modus)

      implicit none
      
      integer(long)                                    :: zeig1,zeig2,zeig3,&
                                                          swapzeig,m,m1,b,block1,&
                                                          zeig4,mpotdim1,h,p,&
                                                          idum,f,vdim1,ndim1,&
                                                          g,n,zeig
      integer(long), intent(in)                        :: k,modus
      integer(long)                                    :: s
      integer(long), dimension(maxdim)                 :: mpoint,fpoint
      real(dop), dimension(maxdim)                     :: rdum
      real(dop), dimension(ortdim), intent(in)         :: ort
      real(dop), dimension(maxdim)                     :: gpoint
      real(dop), dimension(c3matdim), intent(in)       :: cdvr3
      complex(dop)                                     :: vz
      complex(dop), dimension(dgldim), intent(in)      :: psi
      complex(dop), dimension(pmatdim), intent(in)     :: hteil
      complex(dop), dimension(adim), intent(inout)     :: dtpsi
      complex(dop), dimension(hlochdim), intent(out)   :: hloch
      complex(dop), dimension(4*maxblock)              :: workc
      complex(dop), dimension(dmatdim), intent(in)     :: cdvr4
      logical(kind=4)                                  :: work2eq3
      real(dop), dimension(:), allocatable             :: tmpv

! --- INITIALISE VARIABLES ---------------------------------------------
! workc is divided into three parts of length maxblock, the longest
! possible length of the array during the multiplication.
! The result of a multiplication is stored after zeig2.
! Before multiplication, the vector to be multiplied is moved to after
! zeig1.
! After zeig3 is stored the vector multiplied up to mode m-1
! After zeig4 is stored the CDVR correction term summed over the modes.
!-----------------------------------------------------------------------
      call zerovxz(workc,4*maxblock)
      zeig1=maxblock+1
      zeig2=1
      zeig3=2*maxblock+1
      zeig4=3*maxblock+1
      work2eq3=.false.

!-----------------------------------------------------------------------
! Single-set
!-----------------------------------------------------------------------
      s=1

!-----------------------------------------------------------------------
! transform A coefficients from spf to TDVR basis
!   and save in zeig3
!-----------------------------------------------------------------------
      call cpvxz(psi(zpsi(s)),workc(zeig3),block(s))
      do m=1,nmode
         if (m .eq. meb) cycle

         if (.not. diag(m,k)) then
            swapzeig=zeig1
            zeig1=zeig3
            zeig3=swapzeig
            call qtaxzz(cdvr4(dmat(m,s)),workc(zeig1), &
                 workc(zeig3),vdim(m,s),dim(m,s),ndim(m,s))
         endif
      enddo

!-----------------------------------------------------------------------
! calculate mean field matrices and CDVR correction terms, if required.
! loop over modes
! If H is unit-operator for mode m, or basis is complete, do not bother 
! as hloch is never needed.
!-----------------------------------------------------------------------
      do m=1,nmode
         if (diag(m,k) .or. complete(m) .or. (m .eq. meb)) cycle

         do b=1,block(s)
            workc(zeig2+b-1)=workc(zeig3+b-1)
         enddo

!-----------------------------------------------------------------------
! Apply electronic basis operator
!-----------------------------------------------------------------------
         if (meb .gt. 0) then
            swapzeig=zeig1
            zeig1=zeig2
            zeig2=swapzeig
            call qtaxzz(hteil(pmat(meb,k)),workc(zeig1), &
                 workc(zeig2),vdim(meb,s),dim(meb,s),ndim(meb,s))
         endif

!-----------------------------------------------------------------------
! transform from TDVR basis to DVR points for mode m
! block1 is the length of the transformed array
!-----------------------------------------------------------------------
         swapzeig=zeig1
         zeig1=zeig2
         zeig2=swapzeig
         call qtxxzz(cdvr4(dmat(m,s)),workc(zeig1),workc(zeig2), &
              vdim(m,s),dim(m,s),ndim(m,s))
         swapzeig=zeig1
         zeig1=zeig2
         zeig2=swapzeig
         call mtxxzz(psi(zetf(m,s)),workc(zeig1),workc(zeig2), &
              vdim(m,s),dim(m,s),ndim(m,s),subdim(m))
         block1=vdim(m,s)*subdim(m)*ndim(m,s)

!-----------------------------------------------------------------------
! now operate on vector by V(q1,.xm...qf)
! by recursively dividing out the number of grid points for each
! mode, the gridpoint gpnt is transformed to the mode grid point mgpnt
!-----------------------------------------------------------------------
         mpotdim1=block1
         if (meb .gt. 0) mpotdim1 = mpotdim1/dim(meb,s)
         allocate(tmpv(mpotdim1))

         do p=1,mpotdim1
            call amdecoder_nomeb(p,mpoint,m)

            do m1=1,nmode
               if (m1 .eq. meb) then
                  cycle
               else if (m1 .eq. m) then
                  g=mpoint(m1)
                  call decoderm(g,fpoint,m1)
                  do n=1,nspfdof(m1)
                     f=spfdof(n,m1)
                     g=fpoint(f)
                     gpoint(f)=ort(zort(f)+g-1)
                  enddo
               else
                  do n=1,nspfdof(m1)
                     f=spfdof(n,m1)
                     g=mpoint(m1)
                     zeig = c3mat(m1,1)-1+dim(m1,1)*(n-1)
                     gpoint(f) = cdvr3(zeig+g)
                  enddo
               endif
            enddo
            if (lpntdvr) then
               do f=1,ndof
                  if (basis(f) .eq. 19) gpoint(f) = rpbaspar(1,f)
               enddo
            endif
            f=fterm(k)
            h=hamilton(f,k)
            call poteval (h,tmpv(p),gpoint,vz,rdum,idum)
         enddo

         if (meb .gt. 0) then
            if (meb .lt. m) then
               ndim1 = ndim(meb,s)*subdim(m)/(dim(m,s))
               call vcdvr_op(tmpv,workc(zeig2),vdim(meb,s),dim(meb,s),ndim1)
            else
               vdim1 = vdim(meb,s)*subdim(m)/(dim(m,s))
               call vcdvr_op(tmpv,workc(zeig2),vdim1,dim(meb,s),ndim(meb,s))
            endif

         else
            call dvxxdzo(tmpv,workc(zeig2),mpotdim1)
         endif
         deallocate(tmpv)

!-----------------------------------------------------------------------
!  calculate single-hole matrix
!  first H(x,q) is calculated. This is then transformed to H(x,phi)
!-----------------------------------------------------------------------
         call rmmtxxzz(workc(zeig3),workc(zeig2),workc(zeig1), &
              vdim(m,s),dim(m,s),ndim(m,s),subdim(m))
         call mqxazz(workc(zeig1),cdvr4(dmat(m,s)), &
              hloch(phloch(m,k)),subdim(m),dim(m,s))
         h=hamilton(fkoe,k)
         if (htmtype(h) .ne. 1)  &
            call xmxxzzo(koeff(k),hloch(PHLoch(m,k)),subdim(m),dim(m,s))   

!-----------------------------------------------------------------------
! now calculate the CDVR correction term
!-----------------------------------------------------------------------
         if (lcdvr) then
            swapzeig=zeig1
            zeig1=zeig2
            zeig2=swapzeig
            call mtaxzz (psi(zetf(m,s)),workc(zeig1), &
                 workc(zeig2),vdim(m,s),subdim(m),ndim(m,s),dim(m,s))
            swapzeig=zeig1
            zeig1=zeig2
            zeig2=swapzeig
            call qtaxzz (cdvr4(dmat(m,s)),workc(zeig1), &
                 workc(zeig2),vdim(m,s),dim(m,s),ndim(m,s))
            do b=1,block(s)
               workc(zeig4+b-1)=workc(zeig4+b-1)+workc(zeig2+b-1)
            enddo
         endif

!-----------------------------------------------------------------------
! end of loop to calculate mean fields
!-----------------------------------------------------------------------
      enddo

      if (modus .eq. 1) go to 999

!-----------------------------------------------------------------------
! calculate the time derivative of the coefficients.
! First apply electronic basis operator
! then act on transformed coefficients with potential terms
!-----------------------------------------------------------------------
      if (meb .gt. 0) then
         swapzeig=zeig1
         zeig1=zeig3
         zeig3=swapzeig
         call qtaxzz(hteil(pmat(meb,k)),workc(zeig1), &
              workc(zeig3),vdim(meb,s),dim(meb,s),ndim(meb,s))
      endif

      mpotdim1=block(s)
      if (meb .gt. 0) mpotdim1=mpotdim1/dim(meb,s)
      allocate(tmpv(mpotdim1))

      do p=1,mpotdim1
         call adecoder_nomeb(p,mpoint)

         do m=1,nmode
            if (m .eq. meb) cycle

            do n=1,nspfdof(m)
               f=spfdof(n,m)
               g=mpoint(m)
               zeig = c3mat(m,1)-1+dim(m,1)*(n-1)
               gpoint(f) = cdvr3(zeig+g)
            enddo
         enddo
         if (lpntdvr) then
            do f=1,ndof
               if (basis(f) .eq. 19) gpoint(f) = rpbaspar(1,f)
            enddo
         endif
         f=fterm(k)
         h=hamilton(f,k)
         call poteval (h,tmpv(p),gpoint,vz,rdum,idum)
      enddo
      if (meb .gt. 0) then
         call vcdvr_op(tmpv,workc(zeig3),vdim(meb,s),dim(meb,s),ndim(meb,s))
      else
         call dvxxdzo(tmpv,workc(zeig3),mpotdim1)
      endif
      deallocate(tmpv)

!-----------------------------------------------------------------------
!  add CDVR correction if required
!-----------------------------------------------------------------------
      if (lcdvr) then
         m1=nmode
         do m=1,nmode
            if (diag(m,k) .or. (m .eq. meb)) m1=m1-1
         enddo
         do b=1,block(s)
            workc(zeig3+b-1)=workc(zeig4+b-1)-(m1-1)*workc(zeig3+b-1)
         enddo
      endif

!-----------------------------------------------------------------------
! transform back to spfs
!-----------------------------------------------------------------------
      do m=1,nmode
         if (m .eq. meb) cycle
         if (.not. diag(m,k)) then
            swapzeig=zeig1
            zeig1=zeig3
            zeig3=swapzeig
            call qtxxzz (cdvr4(dmat(m,s)),workc(zeig1), &
                 workc(zeig3),vdim(m,s),dim(m,s),ndim(m,s))
         endif
      enddo

      h=hamilton(fkoe,k)
      if (htmtype(h) .eq. 1) then
         call addvxxzo(workc(zeig3),dtpsi(zpsi(s)),block(s))
      else
         call xvxxzza(koeff(k),workc(zeig3),dtpsi(zpsi(s)),block(s))
      endif

  999 continue

      return
      end subroutine matrizen4

      subroutine vcdvr_op(tmpv,tmppsi,vdim1,dim1,ndim1)

      implicit none

      integer(long), intent(in) :: vdim1,dim1,ndim1
      integer(long)             :: v,e,n
      real(dop), intent(in), dimension(vdim1,ndim1) :: tmpv
      complex(dop), intent(inout), dimension(vdim1,dim1,ndim1) :: tmppsi

      do n=1,ndim1
         do e=1,dim1
            do v=1,vdim1
               tmppsi(v,e,n) = tmpv(v,n)*tmppsi(v,e,n)
            enddo
         enddo
      enddo

      return
      end subroutine vcdvr_op

!#######################################################################
! GETCDVR
!
! called from mfields
!
! calculates the CDVR grid points and CDVR/spf transformation matrices 
! for each degree of freedom
!
! psi  : wavefunction
! cdvr2: <phi|x|phi> matrix
! cdvr3: CDVR points (eigenvalues of cdvr2)
! cdvr4: transformation matrix <phi|q> (eigenvectors of cdvr2)
! ort  : x values
! modus = 1 : Do not calculate for GWP modes
!
!#######################################################################

      subroutine getcdvr(psi,cdvr2,cdvr4,cdvr3,ort,modus)

      implicit none

      integer(long), intent(in)                         :: modus
      integer(long)                                     :: s,m,zeig
      real(dop), dimension(c3matdim), intent(out)       :: cdvr3
      real(dop), dimension(ortdim), intent(in)          :: ort
      complex(dop), dimension(dgldim), intent(in)       :: psi
      complex(dop), dimension(dmatdim), intent(out)     :: cdvr2,cdvr4
      complex(dop), dimension(1)  :: ztmp,ztmp1

      do s=1,nstate
         do m=1,nmode
            if (m .eq. meb) cycle

            zeig=dim(m,s)*subdim(m)+1
            if (gwpm(m) .and. (modus .ne. 1)) then
               call cdvrpnts(m,psi(zetf(m,s)),cdvr2(dmat(m,s)),&
                    cdvr3(c3mat(m,s)),cdvr4(dmat(m,s)),ort,&
                    dim(m,s),subdim(m),gs2(gs2mat(m,s)),npg(m),&
                    psi(zdgwp(m,s)),nspfdof(m))
            else if (.not. gwpm(m)) then
               call cdvrpnts(m,psi(zetf(m,s)),cdvr2(dmat(m,s)),&
                    cdvr3(c3mat(m,s)),cdvr4(dmat(m,s)),ort,&
                    dim(m,s),subdim(m),ztmp,1,ztmp1,nspfdof(m))
            endif
         enddo
      enddo

      return
      end subroutine getcdvr

!#######################################################################
! CDVRPNTS
!
! called from getcdvr
!
! calculates the CDVR grid points and CDVR/spf transformation matrices 
! for a single degree of freedom
!
! psi  : spfs for mode
! cdvr2: <phi|x|phi> matrix (destroyed on exit)
! cdvr3: CDVR points (eigenvalues of cdvr2)
! cdvr4: transformation matrix <phi|q> (eigenvectors of cdvr2)
! ort  : x values
! xpsi : work array to store x|phi>
!
!#######################################################################

      subroutine cdvrpnts(m1,psi,cdvr2,cdvr3,cdvr4,ort,dim,subdim,gs2,&
                          npgm,dgwp,nspfdof1)

      implicit none

      integer(long)                                   :: e,e1,g,g1,error,f,n
      integer(long), intent(in)                       :: m1,npgm,dim,subdim,&
                                                         nspfdof1
      real(dop), dimension(dim,nspfdof1), intent(out) :: cdvr3
      real(dop), dimension(ortdim), intent(in)        :: ort
      real(dop), dimension(3*dim)                     :: workr
      complex(dop), dimension(subdim,dim), intent(in) :: psi
      complex(dop), dimension(dim,dim), intent(in)    :: dgwp
      complex(dop), dimension(subdim,dim)             :: xpsi
      complex(dop), dimension(dim,dim), intent(out)   :: cdvr2,cdvr4
      complex(dop), dimension(dim,dim,nspfdof1)       :: tmpc4
      complex(dop), dimension(dim,dim)                :: ctmp
      complex(dop), dimension(dim,dim,npgm,npgm), intent(in) :: gs2

!-----------------------------------------------------------------------
! form <phi|x|phi> matrix
!-----------------------------------------------------------------------
      if (gwpm(m1)) then
         ctmp=0.0_dop
         cdvr4 = 0.0_dop
         do e=1,dim
            do g1=1,dim
               do g=1,dim
                  ctmp(g1,e)=ctmp(g1,e) + gs2(g1,g,1,2)*dgwp(g,e)
               enddo
            enddo
         enddo
         do e=1,dim
            do e1=1,dim
               do g=1,dim
                  cdvr4(e1,e)=cdvr4(e1,e) + dconjg(dgwp(g,e1))*ctmp(g,e)
               enddo
            enddo
         enddo
         
      else
         do n=1,nspfdof1
            f=spfdof(n,m1)
            do e=1,dim
               call dtxxdz(ort(zort(f)),psi(1,e),xpsi(1,e),vgdim(f),&
                    gdim(f),ngdim(f))
            enddo
            call mmaxzzh(psi,xpsi,tmpc4(1,1,n),subdim,dim)
         enddo
         cdvr4(:,:)=tmpc4(:,:,1)

      endif

!-----------------------------------------------------------------------
! diagonalise <phi|x|phi> matrix to get:
! CDVR2 is used as work-space for LAPACK-routine zheev.
! CDVR3 holds eigenvalues (CDVR grid points)
! CDVR4 holds on input the matrix to be diagonalised.
! CDVR4 holds on output the eigenvectors 
!       i.e.: (CDVR/spf transformation matrix) <phi|q>
!-----------------------------------------------------------------------
      g = dim*dim
      call zheev('V','U',dim,cdvr4,dim,cdvr3,cdvr2,g,workr,error)
      if (error .ne. 0) then
         routine = 'cdvrpnts'
         message = 'Cannot diagonalize CDVR matrix'
         call errormsg
      endif

! If multi-D mode, now try nd diagonalise the higher DOFs, using CDVR4 
! from DOF 1 as a guess
      if (nspfdof1 .gt. 1) then
         call simul_diag(tmpc4,dim,nspfdof1,cdvr4)
         call getcdvr3(cdvr3,tmpc4,dim,nspfdof1)
      endif

!-----------------------------------------------------------------------
! Use same phase convention for eigenvectors as DVR 
!-----------------------------------------------------------------------
      do g=1,dim
         if (dble(cdvr4(1,g)) .lt. 0.0_dop) then
            do g1=1,dim
               cdvr4(g1,g) = -cdvr4(g1,g)
            enddo
         endif
      enddo

      return
      end subroutine cdvrpnts

!#######################################################################

      subroutine getcdvr3(cdvr3,tmpc4,dim,nspfdof1)

      implicit none

      integer(long), intent(in)                             :: dim,nspfdof1
      integer(long)                                         :: n,e
      real(dop), dimension(dim,nspfdof1), intent(out)       :: cdvr3
      complex(dop), dimension(dim,dim,nspfdof1), intent(in) :: tmpc4

      do n=1,nspfdof1
         do e=1,dim
            cdvr3(e,n) = dble(tmpc4(e,e,n))
         enddo
      enddo

      return
      end subroutine getcdvr3

!#######################################################################

      subroutine matrizen5(k,s,psi,dtpsi,hteil,hloch,jindx,jworki)

      implicit none

      integer(long)                                    :: zeig1,zeig2,zeig3, &
                                                          swapzeig,m,m1, &
                                                          block1,block2,block3
      integer(long), intent(in)                        :: k,s
      integer(long), dimension(jindxdim), intent(inout):: jindx
      integer(long), dimension(3*fmblock), intent(out) :: jworki
      complex(dop), dimension(adim), intent(in)        :: psi
      complex(dop), dimension(adim), intent(inout)     :: dtpsi
      complex(dop), dimension(pmatdim), intent(in)     :: hteil
      complex(dop), dimension(hlochdim), intent(out)   :: hloch
      complex(dop), dimension(3*fmblock)               :: workc

! --- INITIALISE VARIABLES ---------------------------------------------
! workc is divided into three parts of length block.
! The result of a multiplication is stored after zeig2.
! Before multiplication, the vector to be multiplied is moved to after
! zeig1. After zeig3 is stored the vector multiplied up to mode m-1.
!-----------------------------------------------------------------------
      zeig1 = fmblock+1
      zeig2 = 1
      zeig3 = 2*fmblock+1

      block2=block(s)
      call cpvxz(psi(zpsi(s)),workc(zeig2),block2)
      call cpvxi(jindx(zpsi(s)),jworki(zeig2),block2)

!-----------------------------------------------------------------------
!  Loop over modes.
!  First set initial pointers, then multiply wavefunction
!  with single-particle hamiltonian matrices corresponding to
!  dimensions numbers lower than mode
!-----------------------------------------------------------------------
      do m=1,nmode
         if (m.ne.1) then
            if (.not.diag(m-1,k)) then
               block1=vdim(m-1,s)*dim(m-1,s)*ndim(m-1,s)
               call venpsi(m-1,jindx(zpsi(s)),jworki(zeig1), &
                    jworki(zeig2),block(s),block1,block2, &
                    nmode,vdim(m-1,s),dim(m-1,s),ndim(m-1,s),jvdim, &
                    workc(zeig1),workc(zeig2))
               block2=block1
               call cpvxi(jworki(zeig1),jworki(zeig2),block1)
               call qtxxzz(hteil(pmat(m-1,k)),workc(zeig1), &
                    workc(zeig2),vdim(m-1,s),dim(m-1,s),ndim(m-1,s))

            endif
         endif

!-----------------------------------------------------------------------
!  Multiply array with single-particle hamiltonian matrices for
!  modes higher than m.
!  If H is unit-operator for mode m, or basis is complete, do not bother
!  as hloch is never needed.
!-----------------------------------------------------------------------
         if (.not. diag(m,k) .and. .not. complete(m)) then
            block1=vdim(m,s)*dim(m,s)*ndim(m,s)
            call venpsi(m,jindx(zpsi(s)),jworki(zeig1), &
                    jworki(zeig2),block(s),block1,block2, &
                    nmode,vdim(m,s),dim(m,s),ndim(m,s),jvdim, &
                    workc(zeig1),workc(zeig2))
            swapzeig=zeig1
            zeig1=zeig2
            zeig2=swapzeig
            block2=block1

            block3=block(s)
            call cpvxz(psi(zpsi(s)),workc(zeig3),block3)
            call cpvxi(jindx(zpsi(s)),jworki(zeig3),block3)
            do m1=nmode,m+1,-1
               if (.not.diag(m1,k)) then
                  block1=vdim(m1,s)*dim(m1,s)*ndim(m1,s)
                  call venpsi(m1,jindx(zpsi(s)),jworki(zeig1), &
                       jworki(zeig3),block(s),block1,block3, &
                       nmode,vdim(m1,s),dim(m1,s),ndim(m1,s),jvdim, &
                       workc(zeig1),workc(zeig3)) 
                  block3=block1
                  call cpvxi(jworki(zeig1),jworki(zeig3),block1)
                  call qtaxzz(hteil(pmat(m1,k)),workc(zeig1), &
                       workc(zeig3),vdim(m1,s),dim(m1,s),ndim(m1,s))
               endif
            enddo

!-----------------------------------------------------------------------
!  calculate single-hole matrix
!-----------------------------------------------------------------------
            block1=vdim(m,s)*dim(m,s)*ndim(m,s)
            call venpsi(m,jindx(zpsi(s)),jworki(zeig1), &
                       jworki(zeig3),block(s),block1,block3, &
                       nmode,vdim(m,s),dim(m,s),ndim(m,s),jvdim, &
                       workc(zeig1),workc(zeig3))
            swapzeig=zeig1
            zeig1=zeig3
            zeig3=swapzeig
            block3=block1

            call rmhxxxzz(workc(zeig3),workc(zeig2), &
                 hloch(phloch(m,k)),vdim(m,s),dim(m,s),ndim(m,s))

            if (hamilton(fkoe,k) .ne. 1) &
               call xqxxzzo(koeff(k),hloch(phloch(m,k)),dim(m,s))
         endif
      enddo

!-----------------------------------------------------------------------
! Now calculate the time derivative of the coefficients.
! calculation of hloch for final mode has left workc(zeig2) with the
! product of psi(block) and hteil matrices for all, excluding the final
! mode.
!-----------------------------------------------------------------------
      m=nmode
      if (.not.diag(m,k)) then
         block1=vdim(m,s)*dim(m,s)*ndim(m,s)
         call venpsi(m,jindx(zpsi(s)),jworki(zeig1), &
                       jworki(zeig2),block(s),block1,block2, &
                       nmode,vdim(m,s),dim(m,s),ndim(m,s),jvdim, &
                       workc(zeig1),workc(zeig2))
         block2=block1
         call cpvxi(jworki(zeig1),jworki(zeig2),block2)
         call qtxxzz(hteil(pmat(m,k)),workc(zeig1),&
                       workc(zeig2),vdim(m,s),dim(m,s),ndim(m,s)) 
      endif
      call cpvxz_s(workc(zeig2),workc(zeig1),jworki(zeig2), &
              jindx(zpsi(s)),block2,block(s))

!-----------------------------------------------------------------------
!  multiply by Hamiltonian coefficient and add to dtpsi
!-----------------------------------------------------------------------
      if (hamilton(fkoe,k) .eq. 1) then
         call addvxxzo(workc(zeig1),dtpsi(zpsi(s)),block(s))
      else
         call xvxxzza(koeff(k),workc(zeig1),dtpsi(zpsi(s)),block(s))
      endif

      return
      end subroutine matrizen5

!#######################################################################

      subroutine matrizen6(k,s,s1,psi,dtpsi,hteil,hloch,jindx,jworki)

      implicit none

      integer(long)                                    :: zeig1,zeig2,zeig3, &
                                                          swapzeig,m,m1, &
                                                          block1,block2,block3
      integer(long), intent(in)                        :: k,s,s1
      integer(long), dimension(jindxdim), intent(inout):: jindx
      integer(long), dimension(3*fmblock), intent(out) :: jworki
      complex(dop), dimension(adim), intent(in)        :: psi
      complex(dop), dimension(adim), intent(inout)     :: dtpsi
      complex(dop), dimension(pmatdim), intent(in)     :: hteil
      complex(dop), dimension(hlochdim), intent(out)   :: hloch
      complex(dop), dimension(3*fmblock)               :: workc

! --- INITIALISE VARIABLES ---------------------------------------------
! workc is divided into three parts of length maxblock, the longest
! possible
! length of the array during the multiplication.
! The result of a multiplication is stored after zeig2.
! Before multiplication, the vector to be multiplied is moved to after
! zeig1.
! After zeig3 is stored the vector multiplied up to mode m-1
!-----------------------------------------------------------------------
      call zerovxz(workc,3*fmblock)
      zeig1=fmblock+1
      zeig2=1
      zeig3=2*fmblock+1

      block2=block(s1)
      call cpvxz(psi(zpsi(s1)),workc(zeig2),block2)
      call cpvxi(jindx(zpsi(s1)),jworki(zeig2),block2)

!-----------------------------------------------------------------------
!  First set initial pointers, then multiply wavefunction of state s1
!  with single-particle hamiltonian matrices corresponding to
!  dimensions numbers lower than mode
!  as the matrix elements are between different states with different
!   number of coefficients, the following pointers are needed:
!  block1: number of elements in multiplied array.
!  vdim1: equivalent of vdim during multiplication over the modes lower
!  than m.
!  vdim2: equivalent of vdim during multiplication over the modes higher
!  then m.
!  before a multiplication, block=vdim1*dim(m,s1)*ndim(m,s1)
!  after a multiplication, block=vdim1*dim(m,s)*ndim(m,s1)
!-----------------------------------------------------------------------
      do m=1,nmode
         if (m.ne.1) then
            block1=vdim(m-1,s)*dim(m-1,s1)*ndim(m-1,s1)
            call venpsi1(m-1,jindx(zpsi(s)),jindx(zpsi(s1)), &
                    jworki(zeig1), &
                    jworki(zeig2),block(s),block(s1),block1,block2, &
                    nmode,vdim(m-1,s),dim(m-1,s1),ndim(m-1,s1),jvdim, &
                    workc(zeig1),workc(zeig2))
            block2=vdim(m-1,s)*dim(m-1,s)*ndim(m-1,s1)
            call venindex1(m-1,jindx(zpsi(s)),jindx(zpsi(s1)), &
                    jworki(zeig2),block(s),block(s1),block2, &
                    nmode,vdim(m-1,s),dim(m-1,s),ndim(m-1,s1),jvdim)

            call mtxxzz(hteil(pmat(m-1,k)),workc(zeig1), &
                 workc(zeig2),vdim(m-1,s),dim(m-1,s1),ndim(m-1,s1), &
                 dim(m-1,s))
         endif

         block1=vdim(m,s)*dim(m,s1)*ndim(m,s1)
         call venpsi1(m,jindx(zpsi(s)),jindx(zpsi(s1)), &
                 jworki(zeig1), &
                 jworki(zeig2),block(s),block(s1),block1,block2, &
                 nmode,vdim(m,s),dim(m,s1),ndim(m,s1),jvdim, &
                 workc(zeig1),workc(zeig2))
         swapzeig=zeig1
         zeig1=zeig2
         zeig2=swapzeig
         block2=block1

!-----------------------------------------------------------------------
!  wavefunction now multiplied up to m-1
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!  Multiply array with single-particle hamiltonian matrices for
!  modes higher than m.
!  If H is unit-operator for mode m, or basis is complete, do not bother
!  as hloch is never needed.
!-----------------------------------------------------------------------
         if (.not. complete(m)) then
            block3=block(s)
            call cpvxz(psi(zpsi(s)),workc(zeig3),block3)
            call cpvxi(jindx(zpsi(s)),jworki(zeig3),block3)
            do m1=nmode,m+1,-1
               block1=vdim(m1,s)*dim(m1,s)*ndim(m1,s1)
               call venpsi1(m1,jindx(zpsi(s)),jindx(zpsi(s1)),&
                    jworki(zeig1),&
                    jworki(zeig3),block(s),block(s1),block1,block3,&
                    nmode,vdim(m1,s),dim(m1,s),ndim(m1,s1),jvdim,&
                    workc(zeig1),workc(zeig3))
               block3=vdim(m1,s)*dim(m1,s1)*ndim(m1,s1)
               call venindex1(m1,jindx(zpsi(s)),jindx(zpsi(s1)),&
                    jworki(zeig3),block(s),block(s1),block3,&
                    nmode,vdim(m1,s),dim(m1,s1),ndim(m1,s1),jvdim)

               call mtaxzz(hteil(pmat(m1,k)),workc(zeig1),&
                    workc(zeig3),vdim(m1,s),dim(m1,s),ndim(m1,s1),&
                    dim(m1,s1))
            enddo
!-----------------------------------------------------------------------
!  calculate single-hole matrix
!-----------------------------------------------------------------------
            block1=vdim(m,s)*dim(m,s)*ndim(m,s1)
            call venpsi1(m,jindx(zpsi(s)),jindx(zpsi(s1)),&
                    jworki(zeig1),&
                    jworki(zeig3),block(s),block(s1),block1,block3,&
                    nmode,vdim(m,s),dim(m,s),ndim(m,s1),jvdim,&
                    workc(zeig1),workc(zeig3))
            swapzeig=zeig1
            zeig1=zeig3
            zeig3=swapzeig
            block3=block1

            call rmmxxxzz(workc(zeig3),workc(zeig2),&
                 hloch(phloch(m,k)),vdim(m,s),dim(m,s),ndim(m,s1),&
                 dim(m,s1))

            if (hamilton(fkoe,k) .ne. 1)&
               call xmxxzzo(koeff(k),hloch(phloch(m,k)),dim(m,s),&
                    dim(m,s1))
         endif
      enddo

!-----------------------------------------------------------------------
! now calculate the time derivative of the coefficients.
! calculation of hloch for final mode has left workc(zeig2) with the
! product of psi(block) and hteil matrices for all, excluding the final
! mode.
!-----------------------------------------------------------------------
      m=nmode
      block1=vdim(m,s)*dim(m,s1)*ndim(m,s1)
      call venpsi1(m,jindx(zpsi(s)),jindx(zpsi(s1)),&
                    jworki(zeig1),&
                    jworki(zeig2),block(s),block(s1),block1,block2,&
                    nmode,vdim(m,s),dim(m,s1),ndim(m,s1),jvdim,&
                    workc(zeig1),workc(zeig2))
      block2=vdim(m,s)*dim(m,s)*ndim(m,s1)
      call venindex1(m,jindx(zpsi(s)),jindx(zpsi(s1)),&
                    jworki(zeig2),block(s),block(s1),block2,&
                    nmode,vdim(m,s),dim(m,s),ndim(m,s1),jvdim)

      call mtxxzz(hteil(pmat(m,k)),workc(zeig1),&
                 workc(zeig2),vdim(m,s),dim(m,s1),ndim(m,s1),&
                 dim(m,s))

      call cpvxz_s(workc(zeig2),workc(zeig1),jworki(zeig2),&
              jindx(zpsi(s)),block2,block(s))

!-----------------------------------------------------------------------
!  multiply by Hamiltonian coefficient
!-----------------------------------------------------------------------
      if (hamilton(fkoe,k) .eq. 1) then
         call addvxxzo(workc(zeig1),dtpsi(zpsi(s)),block(s))
      else
         call xvxxzza(koeff(k),workc(zeig1),dtpsi(zpsi(s)),block(s))
      endif

      return
      end subroutine matrizen6

!#######################################################################
! MATRIZEN7
!
! called by funk,funkr
! calls to usmatvec,usrhomat,matnum
!
! calculates the mean field matrices involving hamiltonian terms
!  that are multidimensional. It also calculates the time derivative of 
!  the MCTDH coefficients.
!
! This involves multiplying the coefficients of the state s1 by the
! corresponding psi elements to transform from the spf basis to the
! grid point basis. After this, the operator can be applied, and the
! resulting vector transformed back to the spf basis. A direct algorithm
! is used which means that the transformation is made pointwise, and
! the whole primitive grid does not need to be held in memory.
!
! input:
! s: state on LHS of mean field matrix
! s1: state on RHS of mean field matrix
! psi, hteil, workc are standard arrays, see documentation
! k: which correlated Hamiltonian term
! output:
! hloch: mean field matrices for kth Hamiltonian term
! dtpsi: derivative wrt time of A coefficients for state on LHS of
!        mean field matrix
!
! V6.0 GW
!#######################################################################

      subroutine matrizen7(k,s,s1,psi,dtpsi,hloch,ort,psi1,modus)

      implicit none

      integer(long)                                 :: zeig,m,h,muld,potdim,gpnt,mgpnt,p,n,f,idum
      integer(long), dimension(maxdim)              :: igpoint
      integer(long), intent(in)                     :: k,s,s1,modus
      real(dop)                                     :: v
      real(dop), dimension(maxdim)                  :: rdum
      real(dop), dimension(maxdim)                  :: gpoint
      real(dop), dimension(ortdim), intent(in)      :: ort
      complex(dop)                                  :: psix,vz
      complex(dop), dimension(dgldim), intent(in)   :: psi,psi1
      complex(dop), dimension(adim), intent(inout)  :: dtpsi
      complex(dop), dimension(hlochdim), intent(out):: hloch
      complex(dop), dimension(2*maxblock)           :: workc


! --- INITIALISE VARIABLES ---------------------------------------------
! workc is divided into three parts of length maxblock, the longest
! possible
! length of the array during the multiplication.
! The result of a multiplication is stored after zeig2.
! Before multiplication, the vector to be multiplied is moved to after
! zeig1.
! After zeig3 is stored the vector multiplied up to mode m-1
!-----------------------------------------------------------------------
      call zerovxz(workc,2*MaxBlock)
      zeig=maxblock+1

      do m=1,nmode
         call zeromxz(hloch(phloch(m,k)),subdim(m),dim(m,s))
      enddo

      f=fterm(k)
      h=hamilton(f,k)
      muld=htmmuld(h)
      potdim=muldsize(muld)

      do p=1,potdim
! get coordinates of grid point
         gpnt=p-1
         do m=1,nmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               mgpnt=gpnt
               gpnt=gpnt/gdim(f)
               mgpnt=mgpnt-gpnt*gdim(f)
               gpoint(f)=ort(zort(f)+mgpnt)
            enddo
         enddo
! get grid point for each mode
         gpnt=p-1
         do m=1,nmode
            mgpnt=gpnt
            gpnt=gpnt/subdim(m)
            mgpnt=mgpnt-gpnt*subdim(m)
            igpoint(m)=mgpnt+1
         enddo

!-----------------------------------------------------------------------
! calculate V(x) 
!-----------------------------------------------------------------------
         call poteval (h,v,gpoint,vz,rdum,idum)
         if (lcutpes) then
            v=min(v,pesmax)
            v=max(v,pesmin)
         endif

!-----------------------------------------------------------------------
! calculate |x> V(x) <x | Psi>
!-----------------------------------------------------------------------
         call phipnt(psi(adim+1),workc(zeig),igpoint,s1,block(s1))
         call vvtxzz(workc(zeig),psi(zpsi(s1)),psix,block(s1))

         psix=psix*v*koeff(k)

!-----------------------------------------------------------------------
!  calculate contribution to single-hole matrices
!-----------------------------------------------------------------------
         do m=1,nmode
            call phipnt1(psi1(adim+1),workc(zeig),igpoint,m,s,block(s))
            call mfpnt(workc(zeig),psi(zpsi(s)),hloch(phloch(m,k)), &
                 psix,igpoint(m),subdim(m),vdim(m,s), &
                 dim(m,s),ndim(m,s))
         enddo

         if (modus .eq. 1) go to 999

!-----------------------------------------------------------------------
! now calculate the time derivative of the coefficients.
!-----------------------------------------------------------------------
         call phipnt(psi1(adim+1),workc(zeig),igpoint,s,block(s))
         call xvxczza(psix,workc(zeig),dtpsi(s),block(s))

  999    continue
      enddo

      return
      end subroutine matrizen7

!#######################################################################

      subroutine mfpnt(phix,psi,hloch,psix,ig,subdim,vdim,dim,ndim)

      implicit none

      integer(long)                                      :: n,e,v
      integer(long), intent(in)                          :: ig,subdim, &
                                                            vdim,dim,ndim
      complex(dop)                                       :: ctmp
      complex(dop), intent(in)                           :: psix
      complex(dop), dimension(vdim,dim,ndim), intent(in) :: phix,psi
      complex(dop), dimension(subdim,dim), intent(inout) :: hloch

      do n=1,ndim
         do e=1,dim
            do v=1,vdim
               ctmp = phix(v,e,n)*psi(v,e,n)
               hloch(ig,e) = hloch(ig,e) + dconjg(ctmp)*psix
            enddo
         enddo
      enddo

      return
      end subroutine mfpnt

!#######################################################################
!
! The mfields produced by the multi-layer code do not include the
! Hamiltonian coefficient. Here, all the mfields of the lowest
! layer are multiplied by the coefficient to be equivalent to the
! mfields produced by the MCTDH rotuines
!
!#######################################################################

      subroutine kmfields(hloch)

      implicit none

      integer(long) :: k,m,s,s1,h,plen
      complex(dop), dimension(hlochdim), intent(inout) :: hloch

      do k=1,kzahl(1)
         s=kf(k)
         s1=ki(k)
         do m=1,nmode
            if (diag(m,k) .or. complete(m)) cycle

            h=hamilton(fkoe,k)
            if (htmtype(h) .eq. 1 .or. htmtype(h) .eq. 104) cycle

            plen = dim(m,s1)*dim(m,s)

            hloch(phloch(m,k):phloch(m,k)-1+plen) = &
               koeff(k)*hloch(phloch(m,k):phloch(m,k)-1+plen)

         enddo
      enddo

      return
      end subroutine kmfields

!#######################################################################

      subroutine mfder1(k,m,s,np,ghderiv1,psi,hteil,gdof,gwpdim1)

      implicit none
      
      integer(long)                                      :: zeig1,zeig2,zeig3,&
                                                            m1,f,n,i
      integer(long), intent(in)                          :: k,s,m,np,&
                                                            gdof,gwpdim1
      complex(dop), dimension(dgldim), intent(in)        :: psi
      complex(dop), dimension(pmatdim), intent(in)       :: hteil
      complex(dop), dimension(3*maxblock)                :: workc
      complex(dop), dimension(:), allocatable            :: psig
      real(dop), dimension(:), allocatable               :: koelha
      real(dop),dimension(gdof,gwpdim1), intent(out)     :: ghderiv1
      complex(dop),dimension(gwpdim1,gwpdim1)            :: mfmat

      allocate(psig(maxblock))
      allocate(koelha(maxblock))

! --- INITIALISE VARIABLES ---------------------------------------------
! workc is divided into three parts of length block.
! The result of a multiplication is stored after zeig2.
! Before multiplication, the vector to be multiplied is moved to after
! zeig1. After zeig3 is stored the vector multiplied up to mode m-1.
!-----------------------------------------------------------------------
      zeig1 = block(s)+1
      zeig2 = 1
      zeig3 = 2*block(s)+1

!-----------------------------------------------------------------------
!  Transform to GWP basis
!-----------------------------------------------------------------------
! transform to GWP basis
      if (psitype .eq. 0 .or. psitype .eq. 5) then
         call cpvxz(psi(zpsi(s)),workc(zeig2),block(s))

         do m1 = 1,nmode
            if (.not. gwpm(m1)) cycle

            call swapsxxi(zeig1,zeig2)
            call qtxxzz(psi(zdgwp(m1,s)),workc(zeig1), &
                 workc(zeig2),vdim(m1,s),dim(m1,s),ndim(m1,s))
         enddo
! save transformed psi
         call cpvxz(workc(zeig2),psig,block(s))
      else
         call cpvxz(psi(zpsi(s)),psig,block(s))
      endif

      do n=1,nspfdof(m)
         f = spfdof(n,m)
         call cpvxz(psig(zpsi(s)),workc(zeig1),block(s))

! extract derivs for DOF f and store as koelha
         do i=1,block(s)
            koelha(i) = gconfder1(f,i,s,np)
         enddo
! multiply by "LHA coefficients" (gradients)
         call ddxxdz(koelha,workc(zeig1),workc(zeig2),block(s))

!-----------------------------------------------------------------------
!  Loop over modes. 
!  First set initial pointers, then multiply wavefunction 
!  with single-particle hamiltonian matrices corresponding to 
!  dimensions numbers lower than mode 
!-----------------------------------------------------------------------
         do m1=1,nmode
            if (m .eq. m1)    cycle

            call swapsxxi(zeig1,zeig2)
            call qtxxzz(hteil(pmat(m1,k)),workc(zeig1), &
                    workc(zeig2),vdim(m1,s),dim(m1,s),ndim(m1,s))
         enddo  

!-----------------------------------------------------------------------
!  calculate single-hole matrix
!-----------------------------------------------------------------------
         call rmmxxxzz(psig,workc(zeig2), &
                    mfmat,vdim(m,s),dim(m,s),ndim(m,s), &
                    dim(m,s))

         do i=1,gwpdim1
            ghderiv1(n,i) = dble(mfmat(i,i))
         enddo

      enddo

      deallocate(psig,koelha)

      return
      end subroutine mfder1

!#######################################################################
!
!               WKMFIELDS
!
! Here the workspace required for the module is assigned
!
! workcdim, workrdim, workidim and workldim are passed via a common
! block in global.inc, and are checked that they are large enough for
! the module.
!
! when the mean-fields are held constant the routine "funk3" uses
! the "intpsi" array as complex work array. For that reason "intpsi"
! must be large enough to keep three times the largest "block" (which is
! normally already the case), whereas for "workc" a size of twice the
! largest "block" suffices.
!
!#######################################################################

      subroutine wkmfields

      implicit none

!-----------------------------------------------------------------------
! assign workspace used in module
!-----------------------------------------------------------------------
! for matrizen1,2,3,5,6
      if (.not. lconst) then
         workcdim=max(workcdim,3*maxblock)
      endif
! for matrizen4
      if (.not. lconst) then
         if (lcdvr.or.ltdvr) workcdim=max(workcdim,4*maxblock)
      endif
! for matrizen5+6
      if (citype .gt. 0) then
         workidim=max(workidim,3*fmblock)
         workcdim=max(workcdim,3*fmblock)
      else
         workidim=workidim
      endif
      workldim=workldim

      return
      end subroutine wkmfields

      end module mfieldsmod

