      module mpidsteqmod

use decimal, only: dop, long
use logdat
use globmemmod

      implicit none
      private

#ifdef MPI
      public :: mpidsteqr,mpidsteql,strtdsteqr
      contains
!#######################################################################

!-----------------------------------------------------------------------
!  STRTDSTEQR
!-----------------------------------------------------------------------

      SUBROUTINE STRTDSTEQR(ldz,l,lend1,icompz,n,nmaxit,mr)

use mpi

use global
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use timing
use channels
use mpidata
use hpsimod

      implicit none

      integer(long), intent(inout) :: ldz,l,lend1,icompz,n,nmaxit
      real(dop), dimension(mrdim), intent(inout) :: mr
      integer(long) :: error,p2,p3
      real(dop), dimension(2)                    :: dbdum

      integer(long),dimension(6) :: mpicommi
!      integer(long)              :: mpierr

      call MPI_Bcast(mpicommi,6,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      ldz           =mpicommi(1)
      l             =mpicommi(2)
      lend1         =mpicommi(3)
      icompz        =mpicommi(4)
      n             =mpicommi(5)
      nmaxit        =mpicommi(6)

      IF( LEND1.GT.L ) THEN
         dbdum(1)=0.0
         dbdum(2)=0.0
         p2=adim
         if (mpb.gt.0) p2=p2/npackts
         p3=(intordera+1)*(intordera+3)
         p2=(intordera+1)*(intordera+1)
         call mpidsteql(mr(mrworkr),mr(mrintr+p2),mr(mrintr),ldz,l, &
              lend1,1,mr(mrworkr+ldz),mr(mrworkr+ldz+n),dbdum(1),dbdum(2), &
              icompz,n,nmaxit)
      else
         dbdum(1)=0.0
         dbdum(2)=0.0
         p2=adim
         if (mpb.gt.0) p2=p2/npackts
         p3=(intordera+1)*(intordera+3)
         p2=(intordera+1)*(intordera+1)
         call mpidsteqr(mr(mrworkr),mr(mrintr+p2),mr(mrintr),ldz,l, &
              lend1,1,mr(mrworkr+ldz),mr(mrworkr+ldz+n),dbdum(1),dbdum(2), &
              icompz,n,nmaxit)
      endif
 
      return
      end SUBROUTINE STRTDSTEQR

!-----------------------------------------------------------------------
!  MPIDSTEQL
!-----------------------------------------------------------------------

      SUBROUTINE MPIDSTEQL(e,d,z,ldz,l,lend1,thread1,carr,sarr,marr,larr, &
                                                        icompz,n,nmaxit)

use mpi

use global
use griddatmod
use mpidata

      implicit none


      integer(long), intent(inout) :: l
      integer(long), intent(in) :: lend1,thread1,ldz,icompz,N,NMAXIT
      real(dop), dimension(N-1,1), intent(out)   :: carr,sarr
      real(dop), dimension(1), intent(out)       :: larr,marr
      real(dop), dimension(*), intent(out)       ::  e,d
      real(dop), dimension(ldz,*), intent(inout) ::  z
      integer(long) :: m,dist,jtots,lpst,lpfn,sct,I,K, &
                       lastl,lanf,lpstold,lpfnold
      integer(long) :: error,dist1,rdist1,rdist
      integer(long) :: diagnmpi,diagnthread
      real(dop)     :: ztmp1,ztmp2
      real(dop)     :: tst
      real(dop)     :: B,F,G,S,C,P,R,EPS,EPS2,SAFMIN
      real(dop)     ::  DLAMCH,DLAPY2
      logical(kind=4) :: lwait
 
      integer(long),dimension(mpi_status_size) :: status
 
      diagnmpi=nmpi

      if (.not.lmpidsyev) diagnmpi=1

      diagnthread=1

      lwait  = .true.
      SAFMIN = DLAMCH( 'Safe minimum' )
      EPS = DLAMCH( 'E' )
      EPS2 = EPS**2
      jtots=0
 
  1   CONTINUE
 
      JTOTs = JTOTs + 1
 
      sct=mod(jtots-1,diagnthread)+1

  10  CONTINUE
 
      if (thread1.eq.1) then
         if(mpirank.eq.0) then
!
!        Look for small subdiagonal element.
!
            IF( L.NE.LEND1) THEN
               do M=L,LEND1-1
                  TST = ABS( E( M ) )**2
                  IF(TST.LE.(EPS2*ABS(D(M)))*ABS(D(M+1))+ SAFMIN) GOTO 3
               enddo
            END IF
 
            M = LEND1
 
   3        CONTINUE
            IF( M.LT.LEND1) E( M ) = 0.0d0
            P = D( L )
            IF( M.EQ.L )  GOTO 2
!
!        Form shift.
!
            G = ( D( L+1 )-P ) / ( 2.0d0*E( L ) )
            R = DLAPY2( G, 1.0d0)
            G = D(M) + ( E( L ) / ( G+SIGN( R, G ) ) ) - P
            S = 1.0d0
            C = 1.0d0
            P = 0.0d0
 
            if (icompz.gt.0)then
               do i = lend1-1, m, -1
                  sarr(i,sct)=0.0d0
                  carr(i,sct)=1.0d0
               enddo
            endif
            DO I = M-1, L, -1
               F = S*E( I )
               B = C*E( I )
               CALL DLARTG( G, F, C, S, R )
               IF( I.NE.M-1 )  E( I+1 ) = R
               G = D( I+1 ) - P
               R = ( D( I )-G )*S + 2.0d0*C*B
               P = S*R
               D( I+1 ) = G + P
               G = C*R - B
!
!           If eigenvectors are desired, then save rotations.
!
               IF( ICOMPZ.GT.0 ) THEN
                  sarr(i,sct)=S
                  carr(i,sct)=C
               END IF
 
            ENDDO
 
            D( L ) = D( L ) - P
            E( L ) = G
 
            dist1=(LEND1-L)/diagnmpi
            rdist1=LEND1-L-dist1*diagnmpi
            lastl=L
         endif
!
! --- Distribute Matrix
!
         if (jtots.eq.1) then

            if (lmpidsyev) then
               call MPI_Bcast(lastl,1,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
            endif

            lanf=lastl
            dist=(LEND1-lastl)/diagnmpi
            dist1=dist
            rdist1=LEND1-lastl-diagnmpi*dist
            if (mpirank.gt.diagnmpi-rdist1-1) dist1=dist+1
            lpst =LEND1-1
            lpfn =lpst-dist1+1

! --- First Pthread of MPI-Prozess 0 sends the Z-Matrix to the first
! --- Phtread of all the other MPI-Prozesses
!
            if (lmpidsyev) then
               if (mpirank.eq.0.and.diagnmpi.gt.1) then
                  do i=1,diagnmpi-2
                     lpst =lpfn-1
                     lpfn =lpst-dist+1
                     if (i.gt.diagnmpi-rdist1-1) lpfn=lpfn-1
                     call MPI_Send(Z(1,lpfn),(lpst-lpfn+1)*ldz, &
                          MPI_DOUBLE_PRECISION,i,500+i,MPI_COMM_WORLD,mpierr)
                  enddo
                  call MPI_Send(Z(1,lastl),(lpfn-lastl)*ldz, &
                       MPI_DOUBLE_PRECISION,diagnmpi-1,499+nmpi, &
                                                  MPI_COMM_WORLD,mpierr)
                  lpst=LEND1-1
                  lpfn=lpst-dist1+1
               elseif(diagnmpi.gt.1) then
                  lpst=LEND1-1-mpirank*dist
                  if (mpirank.gt.diagnmpi-rdist1) lpst=lpst+diagnmpi-rdist1-mpirank
                  lpfn=lpst-dist1+1
                  if (mpirank.eq.diagnmpi-1) lpfn=lastl
! debug
      write(*,*)'lmp',mpirank
! debug
                  call MPI_Recv(Z(1,lpfn),(lpst-lpfn+1)*ldz, &
                       MPI_DOUBLE_PRECISION,0,500+mpirank,MPI_COMM_WORLD, &
                       status,mpierr)
! debug
      write(*,*)'lmp2',mpirank
! debug
               endif
            endif

         endif
!
! --- save parameters
!
         lpstold=lpst
         lpfnold=lpfn
!

! --- First Pthread of an MPI-Prozess receives the parameters and
! --- rotation matrices from the last Pthread of the previous
! --- MPI-Prozess
!
         if (lmpidsyev) then
            if(mpirank.gt.0) then
               call MPI_Recv(lpst,1,MPI_INTEGER,mpirank-1,1999+mpirank, &
                                           MPI_COMM_WORLD,status,mpierr)
               call MPI_Recv(lastl,1,MPI_INTEGER,mpirank-1,2999+mpirank, &
                                           MPI_COMM_WORLD,status,mpierr)
               call MPI_Recv(carr(1,sct),N-1,MPI_DOUBLE_PRECISION, &
                    mpirank-1,4999+mpirank,MPI_COMM_WORLD,status,mpierr)
               call MPI_Recv(sarr(1,sct),N-1,MPI_DOUBLE_PRECISION, &
                    mpirank-1,5499+mpirank,MPI_COMM_WORLD,status,mpierr)
            endif
         endif

 
! --- Determine loop boundaries for the first Pthread
         dist1=(LEND1-lastl)/diagnmpi
         rdist1=LEND1-lastl-diagnmpi*dist1
         if (mpirank.gt.diagnmpi-rdist1-1) dist1=dist1+1
         dist=dist1/diagnthread
         rdist=dist1-diagnthread*dist
         if (thread1.gt.diagnthread-rdist) dist=dist+1
         lpfn=lpst-dist+1
         if(thread1.eq.diagnthread.and.mpirank.eq.diagnmpi-1) lpfn=lastl
 

! --- First Pthread of an MPI-Prozess receives the first eigenvector
! --- from the last Pthread of the previous MPI-Prozess for the next
! --- transformation, or a larger part of the Z-Matrix to, additionally,
! --- get a better load balancing.
 
         if (lmpidsyev) then
            if(mpirank.gt.0) then
               if(lpst.gt.lpstold) then
                  call MPI_Recv(Z(1,lpstold+1),(lpst-lpstold+1)*ldz, &
                            MPI_DOUBLE_PRECISION,mpirank-1,3999+mpirank, &
                                           MPI_COMM_WORLD,status,mpierr)
               else
                  call MPI_Recv(Z(1,lpst+1),ldz,MPI_DOUBLE_PRECISION, &
                       mpirank-1,1499+mpirank,MPI_COMM_WORLD,status,mpierr)
               endif
            endif
         endif

      else
! --- Get the parameters from the previous Pthread
         lpstold=lpst
         lpfnold=lpfn
         lpst =int(marr(thread1-1))
         lastl=int(larr(thread1-1))
! --- Determine loop boundaries for the other Pthreads
         if(jtots.eq.1)lanf=lastl
         dist1=(LEND1-lastl)/diagnmpi
         rdist1=LEND1-lastl-diagnmpi*dist1
         if (mpirank.gt.diagnmpi-rdist1-1) dist1=dist1+1
         dist=dist1/diagnthread
         rdist=dist1-diagnthread*dist
         if (thread1.gt.diagnthread-rdist) dist=dist+1
         lpfn=lpst-dist+1
         if(thread1.eq.diagnthread.and.mpirank.eq.diagnmpi-1) lpfn=lastl
         if (jtots.eq.1) lpfnold=lpfn
      endif
!
!     Now apply saved rotations to eigenvector matrix
!
      IF( ICOMPZ.GT.0 ) THEN
         DO I = lpst, lpfn, -1
!
            C=carr(i,sct)
            S=sarr(i,sct)
            do K=1,N
               ztmp1=Z(K,I)
               ztmp2=Z(K,I+1)
               Z(K,I)=C*ztmp1-S*ztmp2
               Z(K,I+1)=S*ztmp1+C*ztmp2
            enddo
 

! --- Send first eigenvector to the previous MPI-Prozess to allow for
! --- the last transformation of the following run
            if (lmpidsyev) then
               if (I.eq.lpst.and.mpirank.gt.0.and.thread1.eq.1) then
                  call MPI_Send(Z(1,lpst+1),ldz,MPI_DOUBLE_PRECISION, &
                            mpirank-1,999+mpirank,MPI_COMM_WORLD,mpierr)
               endif
            endif

         ENDDO
      ENDIF
 
 66   CONTINUE
 
! --- Send Parameter
 

! --- The last Pthread of an MPI-Prozess sends the parameters and
! --- rotation matrices to the first Pthread of the following
! --- MPI-Prozess
      if (lmpidsyev) then
         if(mpirank.lt.diagnmpi-1.and.thread1.eq.diagnthread) then
            call MPI_Send(lpfn-1,1,MPI_INTEGER,mpirank+1,2000+mpirank, &
                                                  MPI_COMM_WORLD,mpierr)
            call MPI_Send(lastl,1,MPI_INTEGER,mpirank+1,3000+mpirank, &
                                                  MPI_COMM_WORLD,mpierr)
            call MPI_Send(carr(1,sct),N-1,MPI_DOUBLE_PRECISION,mpirank+1, &
                                     5000+mpirank,MPI_COMM_WORLD,mpierr)
            call MPI_Send(sarr(1,sct),N-1,MPI_DOUBLE_PRECISION,mpirank+1, &
                                     5500+mpirank,MPI_COMM_WORLD,mpierr)
 
! --- Last Pthread of an MPI-Prozess sends the last eigenvector to the
! --- first Pthread of the following MPI-Prozess for the next
! --- transformation, or a larger part of the Z-Matrix to, additionally,
! --- get a better load balancing.
 
            if(lpfn.gt.lpfnold) then
               call MPI_Send(Z(1,lpfnold),(lpfn-lpfnold+1)*ldz, &
                    MPI_DOUBLE_PRECISION,mpirank+1,4000+mpirank,MPI_COMM_WORLD, &
                                                                 mpierr)
            else
               call MPI_Send(Z(1,lpfn),ldz,MPI_DOUBLE_PRECISION, &
                           mpirank+1,1500+mpirank,MPI_COMM_WORLD,mpierr)
            endif
            if (dist.gt.0) then
!
! --- Receive last eigenvector from the following MPI-Prozess to do the
! --- last transformation.
!
              call MPI_Recv(Z(1,lpfn),ldz,MPI_DOUBLE_PRECISION,mpirank+1, &
                              1000+mpirank,MPI_COMM_WORLD,status,mpierr)
            endif
         endif
      endif

 
! --- When the matrix becomes too small for the prallelization the
! --- semaphores are set to the values they had before the calculation
 
      if ((LEND1-lastl).le.(2*diagnmpi*diagnthread).or.jtots.ge.nmaxit) then

! --- Last Pthread of MPI-Prozess 0 receives the eigenvectors from last
! --- Pthread of all the ohter MPI-Prozesses
 
         if (lmpidsyev) then
            if(mpirank.eq.0.and.diagnmpi.gt.1 .and. thread1.eq.diagnthread) then
               do i=1,diagnmpi-2
                  lpst=lpfn-1
                  dist1=(LEND1-lastl)/diagnmpi
                  rdist1=LEND1-lastl-diagnmpi*dist1
                  if (i.gt.diagnmpi-rdist1-1) dist1=dist1+1
                  lpfn=lpst-dist1+1
                  call MPI_Recv(Z(1,lpfn),(lpst-lpfn+1)*ldz, &
                       MPI_DOUBLE_PRECISION,i,9500+i,MPI_COMM_WORLD,status, &
                                                                 mpierr)
               enddo
               call MPI_Recv(Z(1,lanf),(lpfn-lanf)*ldz, &
                    MPI_DOUBLE_PRECISION,i,9500+i,MPI_COMM_WORLD,status, &
                                                                 mpierr)
            elseif(diagnmpi.gt.1.and.thread1.eq.diagnthread) then
               if (mpirank.ne.diagnmpi-1) then
                  call MPI_Send(Z(1,lpfn),dist1*ldz, &
                     MPI_DOUBLE_PRECISION,0,9500+mpirank,MPI_COMM_WORLD, &
                                                                 mpierr)
               else
                 call MPI_Send(Z(1,lanf),(dist1+lpfn-lanf)*ldz, &
                     MPI_DOUBLE_PRECISION,0,9500+mpirank,MPI_COMM_WORLD, &
                                                                 mpierr)
               endif
            endif
         endif

 
         return
      else
         goto 1
      endif
!
!        Eigenvalue found.
!
   2  CONTINUE
      D( L ) = P
 
      L = L + 1
      IF( L.LE.LEND1 ) GOTO 10
 
      if(diagnmpi.gt.1.or.diagnthread.gt.1) then
         dist=0
         lastl=l
         lpstold=lpst
         lpst=lend1-1
         lpfnold=lpfn
         lpfn=lpst-dist+1
         goto 66
      endif
 
      RETURN
 
      END SUBROUTINE MPIDSTEQL

!-----------------------------------------------------------------------
!  MPIDSTEQR (same as above but for qr-iteration)
!-----------------------------------------------------------------------

      SUBROUTINE MPIDSTEQR(e,d,z,ldz,l,lend1,thread1,carr,sarr,marr,larr, &
                                                        icompz,n,nmaxit)

use mpi

use global
use griddatmod
use mpidata

      implicit none

      integer(long), intent(inout) :: l
      integer(long), intent(in) :: lend1,thread1,ldz,icompz,N,NMAXIT
      integer(long) :: m,dist,jtots,lpst,lpfn,sct,I,K, &
                       lastl,lanf,lpstold,lpfnold
      integer(long) :: dist1,rdist1,rdist
      integer(long) :: diagnmpi,diagnthread
      real(dop), dimension(N-1,1), intent(out) :: carr,sarr
      real(dop), dimension(1), intent(out) ::  larr,marr
      real(dop) :: ztmp1,ztmp2
      real(dop) :: tst
      real(dop) ::  B,F,G,S,C,P,R,EPS,EPS2,SAFMIN
      real(dop) ::  DLAMCH,DLAPY2
      real(dop), dimension(*), intent(out) ::  e,d
      real(dop), dimension(ldz,*), intent(inout) ::  z
      logical(kind=4) :: lwait

      integer(long) :: error
      integer(long),dimension(mpi_status_size) :: status
 
      diagnmpi=nmpi
      if (.not.lmpidsyev) diagnmpi=1
      diagnthread=1
 
      lwait  = .true.
      SAFMIN = DLAMCH( 'Safe minimum' )
      EPS = DLAMCH( 'E' )
      EPS2 = EPS**2
      jtots=0
 
  1   CONTINUE
 
      JTOTs = JTOTs + 1
 
      sct=mod(jtots-1,diagnthread)+1
 
  10  CONTINUE
 
      if (thread1.eq.1) then
         if(mpirank.eq.0) then
!
!        Look for small superdiagonal element.
!
            IF( L.NE.LEND1) THEN
               do M=L,LEND1+1,-1
                  TST = ABS( E( M - 1 ) )**2
                  IF(TST.LE.(EPS2*ABS(D(M)))*ABS(D(M-1))+ SAFMIN) GOTO 3
               enddo
            END IF
 
            M = LEND1
 
   3        CONTINUE
            IF( M.GT.LEND1) E( M - 1 ) = 0.0d0
            P = D( L )
            IF( M.EQ.L )  GOTO 2
!
!        Form shift.
!
            G = ( D( L-1 )-P ) / ( 2.0d0*E( L - 1 ) )
            R = DLAPY2( G, 1.0d0)
            G = D(M) + ( E( L - 1) / ( G+SIGN( R, G ) ) ) - P
            S = 1.0d0
            C = 1.0d0
            P = 0.0d0
 
            if (icompz.gt.0)then
               do i = lend1,m-1
                  sarr(i,sct)=0.0d0
                  carr(i,sct)=1.0d0
               enddo
            endif
            DO I = M, L-1
               F = S*E( I )
               B = C*E( I )
               CALL DLARTG( G, F, C, S, R )
               IF( I.NE.M ) E( I-1 ) = R
               G = D( I ) - P
               R = ( D( I + 1 )-G )*S + 2.0d0*C*B
               P = S*R
               D( I ) = G + P
               G = C*R - B
!
!           If eigenvectors are desired, then save rotations.
!
               IF( ICOMPZ.GT.0 ) THEN
                  sarr(i,sct)=-1.0*S
                  carr(i,sct)=C
               END IF
 
            ENDDO
 
            D( L ) = D( L ) - P
            E( L - 1 ) = G
 
            dist1=(L-LEND1)/diagnmpi
            rdist1=L-LEND1-dist1*diagnmpi
            lastl=L
         endif
 
! --- Distribute Matrix
 
         if (jtots.eq.1) then
            if (lmpidsyev) then
               call MPI_Bcast(lastl,1,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
            endif

            lanf=lastl
            dist=(lastl-LEND1)/diagnmpi
            dist1=dist
            rdist1=lastl-LEND1-diagnmpi*dist
            if (mpirank.gt.diagnmpi-rdist1-1) dist1=dist+1
            lpst =LEND1
            lpfn =lpst+dist1-1

! --- First Pthread of MPI-Prozess 0 sends the Z-Matrix to the first
! --- Phtread of all the other MPI-Prozesses
 
            if (lmpidsyev) then
               if (mpirank.eq.0.and.diagnmpi.gt.1) then
                  do i=1,diagnmpi-2
                     lpst =lpfn+1
                     lpfn =lpst+dist-1
                     if (i.gt.diagnmpi-rdist1-1) lpfn=lpfn+1
                     call MPI_Send(Z(1,lpst),(lpfn-lpst+2)*ldz, &
                          MPI_DOUBLE_PRECISION,i,500+i,MPI_COMM_WORLD,mpierr)
                  enddo
                  call MPI_Send(Z(1,lpfn+1),(lastl-lpfn)*ldz, &
                       MPI_DOUBLE_PRECISION,diagnmpi-1,499+nmpi, &
                                                  MPI_COMM_WORLD,mpierr)
                  lpst=LEND1
                  lpfn=lpst+dist1-1
               elseif(diagnmpi.gt.1) then
                  lpst=LEND1+mpirank*dist
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                  if (mpirank.gt.diagnmpi-rdist1) lpst=lpst-diagnmpi+rdist1+mpirank
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                  lpfn=lpst+dist1-1
                  if (mpirank.eq.diagnmpi-1) lpfn=lastl-1
                  call MPI_Recv(Z(1,lpst),(lpfn-lpst+2)*ldz, &
                      MPI_DOUBLE_PRECISION,0,500+mpirank,MPI_COMM_WORLD, &
                                                          status,mpierr)
               endif
            endif

         endif
 
! --- save parameters
 
         lpstold=lpst
         lpfnold=lpfn
 
! --- First Pthread of an MPI-Prozess receives the parameters and
! --- rotation matrices from the last Pthread of the previous
! --- MPI-Prozess
 
         if (lmpidsyev) then
            if(mpirank.gt.0) then
               call MPI_Recv(lpst,1,MPI_INTEGER,mpirank-1,1999+mpirank, &
                    MPI_COMM_WORLD,status,mpierr)
               call MPI_Recv(lastl,1,MPI_INTEGER,mpirank-1,2999+mpirank, &
                                           MPI_COMM_WORLD,status,mpierr)
               call MPI_Recv(carr(1,sct),N-1,MPI_DOUBLE_PRECISION, &
                    mpirank-1,4999+mpirank,MPI_COMM_WORLD,status,mpierr)
               call MPI_Recv(sarr(1,sct),N-1,MPI_DOUBLE_PRECISION, &
                    mpirank-1,5499+mpirank,MPI_COMM_WORLD,status,mpierr)
            endif
         endif
 
! --- Determine loop boundaries for the first Pthread
         dist1=(lastl-LEND1)/diagnmpi
         rdist1=lastl-LEND1-diagnmpi*dist1
         if (mpirank.gt.diagnmpi-rdist1-1) dist1=dist1+1
         dist=dist1/diagnthread
         rdist=dist1-diagnthread*dist
         if (thread1.gt.diagnthread-rdist) dist=dist+1
         lpfn=lpst+dist-1
         if(thread1.eq.diagnthread.and.mpirank.eq.diagnmpi-1) lpfn=lastl-1
 
! --- First Pthread of an MPI-Prozess receives the first eigenvector
! --- from the last Pthread of the previous MPI-Prozess for the next
! --- transformation, or a larger part of the Z-Matrix to, additionally,
! --- get a better load balancing.
 
         if (lmpidsyev) then
            if(mpirank.gt.0) then
               if(lpst.lt.lpstold) then
                  call MPI_Recv(Z(1,lpst),(lpstold-lpst+1)*ldz, &
                            MPI_DOUBLE_PRECISION,mpirank-1,3999+mpirank, &
                                           MPI_COMM_WORLD,status,mpierr)
               else
                  call MPI_Recv(Z(1,lpst),ldz,MPI_DOUBLE_PRECISION, &
                    mpirank-1,1499+mpirank,MPI_COMM_WORLD,status,mpierr)
               endif
            endif
         endif
      else
! --- Get the parameters from the previous Pthread
         lpstold=lpst
         lpfnold=lpfn
         lpst =int(marr(thread1-1))
         lastl=int(larr(thread1-1))
! --- Determine loop boundaries for the other Pthreads
         if(jtots.eq.1)lanf=lastl
         dist1=(lastl-LEND1)/diagnmpi
         rdist1=lastl-LEND1-diagnmpi*dist1
         if (mpirank.gt.diagnmpi-rdist1-1) dist1=dist1+1
         dist=dist1/diagnthread
         rdist=dist1-diagnthread*dist
         if (thread1.gt.diagnthread-rdist) dist=dist+1
         lpfn=lpst+dist-1
         if(thread1.eq.diagnthread.and.mpirank.eq.diagnmpi-1) lpfn=lastl-1
         if (jtots.eq.1) lpfnold=lpfn
      endif
!
!     Now apply saved rotations to eigenvector matrix
!
      IF( ICOMPZ.GT.0 ) THEN
         DO I = lpst, lpfn
 
            C=carr(i,sct)
            S=sarr(i,sct)
            do K=1,N
               ztmp1=Z(K,I)
               ztmp2=Z(K,I+1)
               Z(K,I)=C*ztmp1-S*ztmp2
               Z(K,I+1)=S*ztmp1+C*ztmp2
            enddo
 
! --- Send first eigenvector to the previous MPI-Prozess to allow for
! --- the last transformation of the following run
            if (lmpidsyev) then
               if (I.eq.lpst.and.mpirank.gt.0.and.thread1.eq.1) then
                  call MPI_Send(Z(1,lpst),ldz,MPI_DOUBLE_PRECISION, &
                            mpirank-1,999+mpirank,MPI_COMM_WORLD,mpierr)
               endif
            endif
         ENDDO
      ENDIF
 
 66   CONTINUE
 
! --- Send Parameter
 
! --- The last Pthread of an MPI-Prozess sends the parameters and
! --- rotation matrices to the first Pthread of the following
! --- MPI-Prozess
      if (lmpidsyev) then
         if(mpirank.lt.diagnmpi-1.and.thread1.eq.diagnthread) then
            call MPI_Send(lpfn+1,1,MPI_INTEGER,mpirank+1,2000+mpirank, &
                                                  MPI_COMM_WORLD,mpierr)
            call MPI_Send(lastl,1,MPI_INTEGER,mpirank+1,3000+mpirank, &
                                                  MPI_COMM_WORLD,mpierr)
            call MPI_Send(carr(1,sct),N-1,MPI_DOUBLE_PRECISION,mpirank+1, &
                                    5000+mpirank,MPI_COMM_WORLD,mpierr)
            call MPI_Send(sarr(1,sct),N-1,MPI_DOUBLE_PRECISION,mpirank+1, &
                                    5500+mpirank,MPI_COMM_WORLD,mpierr)
 
! --- Last Pthread of an MPI-Prozess sends the last eigenvector to the
! --- first Pthread of the following MPI-Prozess for the next
! --- transformation, or a larger part of the Z-Matrix to, additionally,
! --- get a better load balancing.
 
            if(lpfn.lt.lpfnold) then
               call MPI_Send(Z(1,lpfn+1),(lpfnold-lpfn+1)*ldz, &
                            MPI_DOUBLE_PRECISION,mpirank+1,4000+mpirank, &
                                                  MPI_COMM_WORLD,mpierr)
            else
               call MPI_Send(Z(1,lpfn+1),ldz,MPI_DOUBLE_PRECISION, &
                           mpirank+1,1500+mpirank,MPI_COMM_WORLD,mpierr)
            endif
 
! --- Receive last eigenvector from the following MPI-Prozess to do the
! --- last transformation.
 
            if (dist.gt.0) then
               call MPI_Recv(Z(1,lpfn+1),ldz,MPI_DOUBLE_PRECISION, &
                    mpirank+1,1000+mpirank,MPI_COMM_WORLD,status,mpierr)
            endif
         endif
      endif
 
! --- When the matrix becomes too small for the prallelization the
! --- semaphores are set to the values they had before the calculation
 
      if ((lastl-LEND1).le.(2*diagnmpi*diagnthread).or.jtots.ge.nmaxit) then

! --- Last Pthread of MPI-Prozess 0 receives the eigenvectors from last
! --- Pthread of all the ohter MPI-Prozesses
 
         if (lmpidsyev) then
            if(mpirank.eq.0.and.diagnmpi.gt.1 .and. thread1.eq.diagnthread) then
               do i=1,diagnmpi-2
                  lpst=lpfn+1
                  dist1=(lastl-LEND1)/diagnmpi
                  rdist1=lastl-LEND1-diagnmpi*dist1
                  if (i.gt.diagnmpi-rdist1-1) dist1=dist1+1
                  lpfn=lpst+dist1-1
                  call MPI_Recv(Z(1,lpst),(lpfn-lpst+1)*ldz, &
                    MPI_DOUBLE_PRECISION,i,9500+i,MPI_COMM_WORLD,status, &
                                                                 mpierr)
               enddo
               call MPI_Recv(Z(1,lpfn+1),(lanf-lpfn)*ldz, &
                    MPI_DOUBLE_PRECISION,i,9500+i,MPI_COMM_WORLD,status, &
                                                                 mpierr)
            elseif(diagnmpi.gt.1.and.thread1.eq.diagnthread) then
               dist1=(lastl-LEND1)/diagnmpi
               rdist1=lastl-LEND1-diagnmpi*dist1
               if (0.gt.diagnmpi-rdist1-1) dist1=dist1+1
               lpst=LEND1
               lpfn=lpst+dist1-1
               do i=1,mpirank
                  lpst=lpfn+1
                  dist1=(lastl-LEND1)/diagnmpi
                  rdist1=lastl-LEND1-diagnmpi*dist1
                  if (i.gt.diagnmpi-rdist1-1) dist1=dist1+1
                  lpfn=lpst+dist1-1
               enddo
               if (mpirank.ne.diagnmpi-1) then
                  call MPI_Send(Z(1,lpst),dist1*ldz, &
                     MPI_DOUBLE_PRECISION,0,9500+mpirank,MPI_COMM_WORLD, &
                                                                 mpierr)
               else
                 call MPI_Send(Z(1,lpst),(lanf-lpst+1)*ldz, &
                     MPI_DOUBLE_PRECISION,0,9500+mpirank,MPI_COMM_WORLD, &
                                                                 mpierr)
               endif
            endif
         endif
 
         return
      else
         goto 1
      endif
!
!        Eigenvalue found.
!
   2  CONTINUE
      D( L ) = P
 
      L = L - 1
      IF( L.GE.LEND1 ) GOTO 10
 
      if(diagnmpi.gt.1.or.diagnthread.gt.1) then
         dist=0
         lastl=l
         lpstold=lpst
         lpst=lend1-1
         lpfnold=lpfn
         lpfn=lpst-dist+1
         goto 66
      endif
 
      RETURN
 
      END SUBROUTINE MPIDSTEQR

#endif
      end module mpidsteqmod
