      module mpipropmod

#ifdef MPI

use decimal, only: dop, long
use globmemmod
use logdat
use dirdyn
use operdef, only: hopipar,lhaorder
use maxv, only:maxhpar,maxhop,maxdim
use gapmod

use mpidata
use mpi
use mpiutilsmod, only: mpibcastchunks_cmplx
use mfieldsmod, only: mpimfields
use timing
      implicit none
      private
      public :: mpiprop,mpiehlochphi,mpiecalcha

      contains
!#######################################################################
!  MPITASKS
!
!  Task  Serial analogon    Description
!-----------------------------------------------------------------------
!  -1    --                 Stop slave
!   0    --                 Return
!   1    funka2             Calculate dA/dt 
!   2    calcha             Calculate H*A for all correlated terms 
!   3    mfields            Calculate the mean fields (hloch)
!   4    brdavstep          
!   5    rdavstep
!   6    getdavmat
!   7    DSTEQR             Lapack: diaginalize tridiagonal matrix
!   8    phihphi            Calculate <phi_ik|h_k^r|phi'_jk> (hteil) 
!   9    hlochphi           Calculate <H>_k |phi'_jk>
!  10    ecalcha
!  11    ehlochphi
!  12    cdavstep
!  13    summf              Sum mean-fields 
!  14    getddpes           Do the Qchem 
!  21    --                 Broadcast hloch tensor
!  22    --                 Broadcast hteil tensor
!  23    --                 Broadcast mfsum tensor
!  41    bdavstep      
!  42    bgetdiagonal
! 999    timing             MPI Timing    
!-----------------------------------------------------------------------

      subroutine mpiprop(psi,dtpsi,Krylov,HKrylov,Kryc,HKryc)

use global
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use channels
use distribmod
use hpsimod

use function, only: funka,funka2
use mpidsteqmod
use mpifunctionmod, only: mpifunka2
use davlibpar, only: mpibrdavstep,mpibdavstep,mpirdavstep,mpicdavstep
use getdiagonalmod, only: bgetdiagonal
use calchamod, only: mpicalcha
use summfmod, only: mpisummf
use hlochphimod, only: mpihlochphi
use getdavmatmod, only: mpigetdavmat
use phihphimod, only: mpiphihphi
use gapfuncsmod, only: mpihlochphigap,mpiphihphigap
use directdyn, only: mpigetddpes, init_qchem_calculations,&
     finalise_qchem_calculations
use mpistructure, only: QCalcData, get_qcalc_mpi_type,&
     init_qcalc_data, unpack_qcalc_data

      implicit none

      complex(dop),dimension(*), intent(inout)    :: psi
      complex(dop),dimension(*), intent(inout)    :: dtpsi
      complex(dop),dimension(*), intent(inout)    :: Kryc
      complex(dop),dimension(*), intent(inout)    :: HKryc
      real(dop),dimension(*), intent(inout)       :: Krylov
      real(dop),dimension(*), intent(inout)       :: HKrylov
!      external   funka,funka2

      integer(long) :: koffset,k0,m,nham,modus, &
                 maxorder,error,steps,nmod, &
                 nn,psidim,ldz,l,icompz,n,nmaxit, &
                 ktype,zetf1,phidim1,mode
      integer(long) :: p1,p2,p3,p4,p5
      integer(long), dimension(20) :: mpicommi
!      variable for mpigetddpes
      integer(long) :: nex1,nex2
      integer(long), allocatable, dimension(:)       :: eex2
      integer(long), allocatable, dimension(:,:)     :: idbloc
      real(dop), allocatable, dimension(:,:,:)       :: xgp,r1
      real(dop), allocatable, dimension(:,:,:,:)     :: ddv,adv,ddcv
      real(dop), allocatable, dimension(:,:,:,:)     :: diabtrans,diabctrans
      real(dop), allocatable, dimension(:,:,:,:)     :: ddnact
      complex(dop), allocatable, dimension(:,:,:,:)  :: ddsoc, adsoc 
      real(sip), allocatable, dimension(:,:,:)       :: ddmoco
      real(dop), allocatable, dimension(:,:,:,:,:)   :: ddder1,adder1
      real(dop), allocatable, dimension(:,:,:,:,:,:) :: ddder2,adder2
      real(dop), allocatable, dimension(:,:,:,:,:)   :: dddip,addip
      complex(dop), allocatable, dimension(:)        :: tmpdtpsi,psi1
      integer(long), allocatable, dimension(:,:)     :: interp
      real(dop)               :: silaccuracy,time,stepsize
      real(dop), dimension(6) :: mpicommr
      real(dop), dimension(2,nmpi) :: mpicpu
      logical(kind=4) :: lf
      type(QCalcData) :: qcalc_data
      integer :: qcalc_type, mpi_comm_worker, colour
      integer(long) :: ddv_size,ddder1_size,ddder2_size,adv_size, &
           dddip_size,addip_size,diabtrans_size,ddnact_size, &
           adsoc_size,interp_size

! --- Get Job
 100  call MPI_Bcast(mpitask,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)

      if (mpitask.eq.-1) then
         call MPI_Finalize(mpierr)
         stop

      else if (mpitask.eq.0) then
         return

      elseif (mpitask.eq.1) then
         call mpifunka2(Krylov,HKrylov)

      elseif (mpitask.eq.2) then
         call MPI_Bcast(lexpectval,1,MPI_LOGICAL,0,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(nham,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)

         if (lexpectval) then
! In expectval "dtpsi" is a temp array to contain H psi.
            allocate(tmpdtpsi(adim))
            call mpicalcha(psi,psi(zetf(1,1)),tmpdtpsi,hteil,nham,koffset,&
                 k0,mi(mijindx))
            deallocate(tmpdtpsi)
         else
            call mpicalcha(psi,psi(zetf(1,1)),dtpsi,hteil,nham,koffset,&
                 k0,mi(mijindx))
         endif

      elseif (mpitask.eq.3) then
         allocate(psi1(adim))
         call mpimfields(psi,dtpsi,psi1,mc(mchloch),hteil,nham,usehsym,modus)
         deallocate(psi1)

      elseif (mpitask.eq.4) then
         call MPI_Bcast(mpicommi,5,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
         steps=mpicommi(1)
         p2   =mpicommi(2)
         p3   =mpicommi(3)
         p4   =mpicommi(4)
         nmod =mpicommi(5)
         call MPI_Bcast(mpicommr,4,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,mpierr)
         silaccuracy=mpicommr(1)
         time       =mpicommr(2)
         rlxemin    =mpicommr(3)
         rlxemax    =mpicommr(4)
         call MPI_Bcast(lf,1,MPI_LOGICAL,0,MPI_COMM_WORLD,mpierr)
         lmpidavrun=.true.
         call mpibrdavstep(psi(zpsi(1)),dtpsi(zpsi(1)),adim/npackts, &
              intordera,silaccuracy,steps,rlaxnum, &
              Krylov,HKrylov, &                     ! Krylov, HKrylov
              mc(mcintpsi),mr(mrintr+p4), &         ! HDiagonal, Tmat
              mr(mrintr),mr(mrintr+p2),   &         ! EigenVector, EigenVal
              mr(mrintr+p2+intordera+1),  &         ! ovlps
              mr(mrworkr),mr(mrintr+p3),  &         ! workr, Rdiagonal
              mr(mrintr+p3+adim/npackts), &         ! davmat
              maxorder,error,time,        &
              precon,lolsen,funka,funka2,lrreal, &
              rlxemin,rlxemax,npackts,nmod,lf, &
              mi(miworki),mi(miworki+precon),  &     ! indxdav,converged
              mi(miworki+precon+8*npackts+10))       ! qmap
         lmpidavrun=.false.

      elseif (mpitask.eq.5) then
         call MPI_Bcast(mpicommi,6,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
         steps=mpicommi(1)
         p2   =mpicommi(2)
         p3   =mpicommi(3)
         p4   =mpicommi(4)
         nmod =mpicommi(5)
         p5   =mpicommi(6)
         call MPI_Bcast(mpicommr,4,MPI_DOUBLE_PRECISION,0, &
                                                  MPI_COMM_WORLD,mpierr)
         silaccuracy=mpicommr(1)
         time       =mpicommr(2)
         rlxemin    =mpicommr(3)
         rlxemax    =mpicommr(4)
         call MPI_Bcast(lf,1,MPI_LOGICAL,0,MPI_COMM_WORLD,mpierr)
         lmpidavrun=.true.
         call mpirdavstep(psi,dtpsi,adim, &
              intordera,silaccuracy,steps,rlaxnum, &
              Krylov,HKrylov,                  &    ! Krylov, HKrylov
              mc(mcintpsi),mr(mrintr+p4),      &    ! HDiagonal, Tmat
              mr(mrintr+p5),                   &    ! Qmat
              mr(mrintr),mr(mrintr+p2),        &    ! EigenVector, EigenVal
              mr(mrintr+p2+intordera+1),       &    ! ovlps
              mr(mrworkr),mr(mrintr+p3),       &    ! workr, Rdiagonal
              mr(mrintr+p3+adim),mi(miworki),  &    ! davmat,indxdav 
              maxorder,error,time, &
              rlxemin,rlxemax,precon,lquad,lolsen, &
              funka,funka2,lrreal,nmod,lf, &
              mi(miworki),mc(mcworkc))
         lmpidavrun=.false.

      elseif (mpitask.eq.6) then
         call MPI_Bcast(nn,1,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(psidim,1,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
         p2=adim
         if (mpb.gt.0) p2=p2/npackts
         p3=(intordera+1)*(intordera+3)
         call mpigetdavmat(mr(mrintr+p3),mr(mrintr+p3+p2),mi(miworki), &
                                               PsiDim,nn,mr(mrworkr))

      elseif (mpitask.eq.7) then
         call STRTDSTEQR(ldz,l,lendmpi,icompz,n,nmaxit,mr)

      elseif (mpitask.eq.8) then
         call mpiphihphi(psi,mc(mchpsi),hteil,nham,usehsym,gh)

      elseif (mpitask.eq.9) then
         call MPI_Bcast(lmpiupdt,nmpiupdtlen,MPI_LOGICAL,master, &
             MPI_COMM_WORLD,mpierr)
         if (lmpiupdt(2)) then
            call MPI_Bcast(phidim1,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
            call MPI_Bcast(zetf1,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
            call MPI_Bcast(m,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
            call MPI_Bcast(nham,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
            call MPI_Bcast(ktype,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         end if
         call mpihlochphi(psi(zetf1),dtpsi(zetf1),mc(mchpsi),mc(mchloch),m,&
              nham,ktype,zetf1,phidim1)

      elseif (mpitask.eq.10) then
!         call MPI_Bcast(fmblock,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call mpiecalcha(psi,mc(mcworkc),hteil,nham,mode,koffset,k0,mi(mijindx))

      elseif (mpitask.eq.11) then
         call MPI_Bcast(phidim1,1,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
         call mpiehlochphi(psi,mc(mcworkc),mc(mchpsi),mc(mchloch),m, &
              nham,ktype,phidim1)

      elseif (mpitask.eq.12) then
!         call MPI_Bcast(hteil,pmatdim,MPI_DOUBLE_COMPLEX,0, &
!                                                  MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(mpicommi,6,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
         steps=mpicommi(1)
         p1   =mpicommi(2)
         p2   =mpicommi(3)
         p3   =mpicommi(4)
         p4   =mpicommi(5)
         p5   =mpicommi(6)
         call MPI_Bcast(mpicommr,6,MPI_DOUBLE_PRECISION,0, &
                                                  MPI_COMM_WORLD,mpierr)
         silaccuracy=mpicommr(1)
         time       =mpicommr(2)
         rlxemin    =mpicommr(3)
         rlxemax    =mpicommr(4)
         stepsize   =mpicommr(5)
         rlximin    =mpicommr(6)
         lmpidavrun=.true.
         call mpicdavstep(psi(zpsi(1)),dtpsi(zpsi(1)),adim, &
              intordera,silaccuracy,steps,rlaxnum, &
              Kryc,HKryc,                         &  ! Krylov, HKrylov
              mc(mcintpsi),mc(mcintpsi+p2),       &  ! HDiagonal, Tmat
              mc(mcworkc+p4),                     &  ! TTmat
              mc(mcintpsi+p3),mc(mcworkc+p5),     &  ! EigenVector, EigenVal
              mc(mcworkc),mr(mrworkr),mr(mrintr), &  ! workc, workr, ovlps
              stepsize,maxorder,error,time,rlxemin,rlxemax,rlximin, &
              precon,lquad,lolsen,funka)
         lmpidavrun=.false.

      elseif (mpitask.eq.13) then
         ! MPI parallel summf
         call mpisummf(mfsum,mc(mchloch),p1,p2)
      
      elseif (mpitask.eq.14) then

         ! receive info about required qchem calcs for this worker process
         call MPI_Scatter(nex1, 0, MPI_DATATYPE_NULL, nex1, 1, MPI_INTEGER, master, MPI_COMM_WORLD, mpierr)
         call MPI_Scatter(nex2, 0, MPI_DATATYPE_NULL, nex2, 1, MPI_INTEGER, master, MPI_COMM_WORLD, mpierr)

         ! Create new communicator for only those processes that will
         ! perform qchem calcs
         if (nex2.ne.0) then
            colour = 0
         else
            colour = MPI_UNDEFINED
         end if
         call MPI_Comm_split(MPI_COMM_WORLD, colour, mpirank, mpi_comm_worker, mpierr)

         if (nex2.ne.0) then
            ! Receive data from master and set global variables
            qcalc_type = get_qcalc_mpi_type(qcalc_data)
            call MPI_Bcast(qcalc_data,1,qcalc_type,master,mpi_comm_worker,mpierr)
            call unpack_qcalc_data(qcalc_data,ndofddpes,nddstate, &
                 ddlaenge,dbdlength,nroot,maxdim,qcsubform,nbasis, &
                 maxhpar,maxhop,lhaorder,dddiab,time,dde0,lpestest, &
                 lupdhes,ldbsave,lddrddb,lqcsaveall,ddextcmd,fchkcmd,ddname)
            natm=int(qcalc_data%ndofddpes/3)
            nactdim=nddstate*(qcalc_data%nddstate-1)/2

            ! Clean the atnam allocated in dirdyn.F90
            if (allocated(atnam)) deallocate(atnam)
            if (allocated(hopipar)) deallocate(hopipar)
            allocate(eex2(qcalc_data%ngp),&
                 idbloc(qcalc_data%nsymop,qcalc_data%ngp),&
                 r1((qcalc_data%dbnrec*qcalc_data%totdim1),qcalc_data%nsymop,qcalc_data%ngp), &
                 xgp(ndofddpes,qcalc_data%nsymop,qcalc_data%totdim1),&
                 atnam(natm),&
                 hopipar(maxhpar,maxhop))

            ! Receive data from master for allocatable memory
            call MPI_Bcast(eex2,qcalc_data%ngp,MPI_INTEGER,master,mpi_comm_worker,mpierr)
            call MPI_Bcast(idbloc,qcalc_data%nsymop*qcalc_data%ngp,MPI_INTEGER,master,mpi_comm_worker,mpierr)
            call MPI_Bcast(r1,(qcalc_data%dbnrec*qcalc_data%totdim1)*qcalc_data%nsymop*qcalc_data%ngp,&
                 MPI_DOUBLE_PRECISION,master,mpi_comm_worker,mpierr)
            call MPI_Bcast(xgp,ndofddpes*qcalc_data%nsymop*qcalc_data%totdim1,MPI_DOUBLE_PRECISION,master,mpi_comm_worker,mpierr)
            call MPI_Bcast(atnam,natm*c1,MPI_CHARACTER,master,mpi_comm_worker,mpierr)
            call MPI_Bcast(hopipar,maxhpar*maxhop,MPI_INTEGER,master,mpi_comm_worker,mpierr)
            call MPI_barrier(mpi_comm_worker,mpierr)

            ! Allocate and initialise memory for qchem calculation output and
            ! calculate size factors for later use
            call init_qchem_calculations(nex2,qcalc_data%nsymop, &
                 adder1,adder2,addip,adv,ddder1,ddder2,dddip, &
                 ddmoco,ddnact,adsoc,ddv,diabtrans,ddcv,diabctrans,&
                 interp,addip_size,adv_size,ddder1_size,ddder2_size, &
                 dddip_size,ddnact_size,ddv_size,diabtrans_size, &
                 adsoc_size,interp_size)
            ! Perform qchem calculations
            call mpigetddpes(qcalc_data%time,qcalc_data%totdim1,qcalc_data%s1,&
                 nex1,nex2,qcalc_data%hl,qcalc_data%ngp,&
                 qcalc_data%nsymop,eex2(:),idbloc(: ,:),qcalc_data%mxmpi,&
                 xgp(:,:,:),r1(: ,:,:),ddv,adv,diabtrans,ddnact,ddder1,&
                 adder1,ddder2,adder2,dddip,addip,adsoc,ddmoco,interp)

            ! Send qchem calculation results back to master process
            call qchem_gatherv_worker_dop(ddv(1,1,1,nex1), ddv_size, &
                 nex1, nex2, mpi_comm_worker)
            call qchem_gatherv_worker_dop(adv(1,1,1,nex1), adv_size, &
                 nex1, nex2, mpi_comm_worker)
            call qchem_gatherv_worker_dop(ddder1(1,1,1,1,nex1), &
                 ddder1_size, nex1, nex2, mpi_comm_worker)
            call qchem_gatherv_worker_dop(ddder2(1,1,1,1,1,nex1), &
                 ddder2_size, nex1, nex2, mpi_comm_worker)
            call qchem_gatherv_worker_dop(dddip(1,1,1,1,nex1), &
                 dddip_size, nex1, nex2, mpi_comm_worker)
            call qchem_gatherv_worker_dop(diabtrans(1,1,1,nex1), &
                 diabtrans_size, nex1, nex2, mpi_comm_worker)
            call qchem_gatherv_worker_dop(ddnact(1,1:nactdim,1,nex1), &
                 ddnact_size, nex1, nex2, mpi_comm_worker)
            call qchem_gatherv_worker_cdop(adsoc(1,1,1,nex1), adsoc_size, &
                 nex1, nex2,mpi_comm_worker)
            call qchem_gatherv_worker_int(interp(1,nex1), interp_size, &
                 nex1, nex2, mpi_comm_worker)

            ! Deallocate memory for qchem calculation results
            call finalise_qchem_calculations(adder1,adder2,addip,adv, &
                 ddder1,ddder2,dddip,ddmoco,ddnact,adsoc,ddv,&
                 diabtrans,ddcv,diabctrans,interp)
         endif
            !deallocate global variable
            if (allocated(eex2)) deallocate(eex2)
            if (allocated(idbloc)) deallocate(idbloc)
            if (allocated(r1)) deallocate(r1)
            if (allocated(xgp)) deallocate(xgp)
   
      elseif (mpitask.eq.21) then 
         ! just recieve hloch
         !call MPI_Bcast(mc(mchloch),hlochdim,MPI_DOUBLE_COMPLEX,master, &
         !     MPI_COMM_WORLD,mpierr)
         call mpibcastchunks_cmplx(mc(mchloch),hlochdim)

      elseif (mpitask.eq.22) then
         ! just recieve hteil
         call MPI_Bcast(hteil,pmatdim,MPI_DOUBLE_COMPLEX,master, &
              MPI_COMM_WORLD,mpierr)

      elseif (mpitask.eq.23) then
!     just recieve mfsum
         call MPI_Bcast(mfsum,mfsumdim,MPI_DOUBLE_COMPLEX,master, &
              MPI_COMM_WORLD,mpierr)
       
      elseif (mpitask.eq.24) then
!     just recieve hpsi
         call MPI_Bcast(mc(mchpsi),hpsidim,MPI_DOUBLE_COMPLEX,master, &
              MPI_COMM_WORLD,mpierr)

      elseif (mpitask.eq.41) then
 !        call MPI_Bcast(hteil,pmatdim,MPI_DOUBLE_COMPLEX,0, &
 !                       MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(mpicommi,6,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
         steps=mpicommi(1)
         p2   =mpicommi(2)
         p3   =mpicommi(3)
         p4   =mpicommi(4)
         p5   =mpicommi(5)
         nmod =mpicommi(6)
         call MPI_Bcast(mpicommr,4,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,mpierr)
         silaccuracy=mpicommr(1)
         time       =mpicommr(2)
         rlxemin    =mpicommr(3)
         rlxemax    =mpicommr(4)
         call MPI_Bcast(lf,1,MPI_LOGICAL,0,MPI_COMM_WORLD,mpierr)
         lmpidavrun=.true.
         call mpibdavstep(psi(zpsi(1)),dtpsi(zpsi(1)), &
                 adim/npackts,intordera,silaccuracy,steps,rlaxnum, &
                 Kryc,HKryc,                    &  ! Krylov, HKrylov
                 mc(mcintpsi),mc(mcintpsi+p4),  &  ! HDiagonal, Tmat
                 mc(mcintpsi+p5),mr(mrintr+p2), &  ! EigenVector, EigenVal
                 mr(mrintr+p2+intordera+1),     &  ! ovlps
                 mr(mrworkr),mr(mrintr+p3),     &  ! workr, Rdiagonal
                 mr(mrintr+p3+adim/npackts),    &  ! davmat
                 maxorder,error,time, &
                 precon,lolsen,funka, &
                 rlxemin,rlxemax,npackts,nmod,lf, &
                 mi(miworki),mi(miworki+precon),  &! indxdav,converged 
                 mi(miworki+precon+8*npackts+10)) ! qmap
         lmpidavrun=.false.

      elseif (mpitask.eq.42) then ! bgetdiagonal
         call bgetdiagonal(mc(mcintpsi))

      elseif (mpitask.eq.50) then
!     receive updated GAP operator and update array sizes
         call MPI_Bcast(dbnrec,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)        
         call alloc_gappntmod
         if (.not. lexact) then
            call MPI_Bcast(pmatgapdim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
            call MPI_Bcast(hpsigapdim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
            call MPI_Bcast(hlochgapdim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
            call MPI_Bcast(hopsgapdim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
            call alloc_gaparrmod

            call MPI_Bcast(zhpsigap(1,zham(1),1),nmode*kzahl(1)*dbnrec,MPI_INTEGER,&
                 master,MPI_COMM_WORLD,mpierr)
            call MPI_Bcast(pmatgap(1,zham(1),1),nmode*(kzahl(1)+maxsta)*dbnrec,&
                 MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
            call MPI_Bcast(phlochgap(1,zham(1),1),nmode*kzahl(1)*dbnrec,&
                 MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
            call MPI_Bcast(zhopgap,nmode*khtot(1)*dbnrec,MPI_INTEGER,master,&
                 MPI_COMM_WORLD,mpierr)
            call MPI_Bcast(hopsgap,hopsgapdim,MPI_DOUBLE,master,MPI_COMM_WORLD,&
                 mpierr)
         else
            call MPI_Bcast(hopsgapdim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
            call alloc_gaparrmod
            call MPI_Bcast(zhopgap,nmode*khtot(1),MPI_INTEGER,master,&
                 MPI_COMM_WORLD,mpierr)
            call MPI_Bcast(hopsgap,hopsgapdim,MPI_DOUBLE,master,MPI_COMM_WORLD,&
                 mpierr)
         endif

      elseif (mpitask.eq.51)then
!     Phihphi when using GAP for DD-MCTDH
         call mpiphihphigap(psi(zetf(1,1)),hteilgap)
       
      elseif (mpitask.eq.52)then
!     Hlochphi when using GAP for DD-MCTDH
         call mpihlochphigap(psi(zetf(1,1)),dtpsi(zetf(1,1)),hlochgap,ktype)
       
      elseif (mpitask.eq.999) then
         call mpitiming(mpicpu)

      endif

! Back to start to get next instruction
      goto 100

      return
      end subroutine mpiprop

!#######################################################################

      subroutine qchem_gatherv_worker_dop(buffer, size_factor,nex1, nex2, comm)
        use mpidata

        real(dop), dimension(*), intent(inout) :: buffer
        integer(long), intent(in) :: size_factor, nex1, nex2
        integer, intent(in) :: comm
	integer, dimension(1) :: sizein

	sizein(1)=nex2

        call MPI_Gatherv( &
             buffer, &
             size_factor * (nex2 - nex1 + 1), &
             MPI_DOUBLE_PRECISION, &
             buffer, sizein, sizein, MPI_DATATYPE_NULL, &  ! these arguments are ignored
             master, comm, mpierr &
        )

      end subroutine qchem_gatherv_worker_dop

      subroutine qchem_gatherv_worker_cdop(buffer, size_factor,nex1, nex2, comm)
        use mpidata

        complex(dop), dimension(*), intent(inout) :: buffer
        integer(long), intent(in) :: size_factor, nex1, nex2
        integer, intent(in) :: comm
	integer, dimension(1) :: sizein

	sizein(1)=nex2

        call MPI_Gatherv( &
             buffer, &
             size_factor * (nex2 - nex1 + 1), &
             MPI_DOUBLE_COMPLEX, &
             buffer, sizein, sizein, MPI_DATATYPE_NULL, &  ! these arguments are ignored
             master, comm, mpierr &
        )

      end subroutine qchem_gatherv_worker_cdop

      subroutine qchem_gatherv_worker_int(buffer, size_factor,nex1, nex2, comm)
        use mpidata

        integer(long), dimension(*), intent(inout) :: buffer
        integer(long), intent(in) :: size_factor, nex1, nex2
        integer, intent(in) :: comm
        integer, dimension(1) :: sizein

        sizein(1)=nex2

        call MPI_Gatherv( &
             buffer, &
             size_factor * (nex2 - nex1 + 1), &
             MPI_INTEGER, &
             buffer, sizein, sizein, MPI_DATATYPE_NULL, &  ! these arguments are ignored
             master, comm, mpierr &
        )

      end subroutine qchem_gatherv_worker_int
#endif

!-----------------------------------------------------------------------
!   mpiecalcha
!-----------------------------------------------------------------------

#ifdef MPI
      subroutine mpiecalcha(psi,dtpsi,hteil,nham,mode,koffset,k0,jindx)

use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use hpsimod
use timing
use channels
use distribmod
use mpidata

use op1lib, only: zerovxz,cpvxz
use op2lib, only: addvxxzo
use calchamod, only: calcha1k
use xvlib, only: xvxxzza

      implicit none

      integer(long), dimension(jindxdim), intent(in)   :: jindx
      complex(dop), dimension(dgldim), intent(in)      :: psi
      complex(dop), dimension(pmatdim), intent(in)     :: hteil
      complex(dop), dimension(adim), intent(out)       :: dtpsi

      integer(long), intent(in) :: koffset,k0,nham,mode

      complex(dop), dimension(maxblock)                :: workc
      integer(long)                                    :: s,h,k

! --- Get Data
      call MPI_Bcast(nham,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
! ... A-Vector
      call MPI_Bcast(psi,adim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)

      call MPI_Bcast(mode,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(koffset,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(k0,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)

      if(mpirank.gt.master) call zerovxz(dtpsi,adim)

      do k=k0+mpirank,koffset,nmpi
         s = kf(k)
         if (s.eq.ki(k) .and. diag(mode,k)) then
            call calcha1k(psi,psi(adim+1),hteil,workc,k,s,jindx)
            
!    --- STORE RIGHT HAND SIDE IN DTPSI ---

            h=hamilton(fkoe,k)
            if (htmtype(h) .eq. 1) then
               call addvxxzo(workc,dtpsi(zpsi(s)),block(s))
            else
               call xvxxzza(koeff(k),workc,dtpsi(zpsi(s)),block(s))
            endif
         endif
      enddo

! --- Reduce Data
      call MPI_Reduce(dtpsi,workc,adim,MPI_DOUBLE_COMPLEX,MPI_SUM,master, &
                                                  MPI_COMM_WORLD,mpierr)
      call cpvxz(workc,dtpsi,adim)

      return
      end subroutine mpiecalcha
#endif

!-----------------------------------------------------------------------
!   mpiehlochphi
!-----------------------------------------------------------------------

#ifdef MPI
      subroutine mpiehlochphi(psi,dtpsi,hpsi,hloch,m,nham,ktype,phidim1)
use global
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use hpsimod
use timing
use channels
use distribmod
use mpidata
use memhopsmodmod

use op1lib, only: zerovxz,trqxz,cpvxz
use xvlib, only: xmxxzza
use hphimod, only: hphi1mk

      implicit none

      integer(long), intent(in)                     :: m,nham,ktype,phidim1
      complex(dop), dimension(dgldim), intent(in)   :: psi
      complex(dop), dimension(phidim1), intent(out) :: dtpsi
      complex(dop), dimension(hpsidim), intent(out)  :: hpsi
      complex(dop), dimension(hlochdim), intent(in) :: hloch
      complex(dop), dimension(phidim1)              :: workc

      integer(long) :: k,k1,k2,pointer,s
      complex(dop)  :: trace

! --- Get Data
      call MPI_Bcast(nham,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(ktype,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(m,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(psi,dgldim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)
!      call MPI_Bcast(hloch,hlochdim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)

      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1
      if (mpirank.gt.master) call zerovxz(dtpsi,phidim1)

      do k = k1+mpirank,k2,nmpi
         s =kf(k)
         if (ktype .eq. 0) then
            if (.not. diag(m,k)) then
               if (autocap(k) .le. 0) then
                  if (ki(k).eq.kf(k)) then
                     pointer = zetf(m,s)-zetf(m,1)+1
                     call trqxz(hloch(phloch(m,k)),trace,dim(m,s))
                     call hphi1mk(psi(zetf(m,s)),hpsi,m,k, &
                          dim(m,s),subdim(m))
                     call xmxxzza(trace,hpsi,dtpsi(pointer), &
                          subdim(m),dim(m,s))
                  endif
               endif
            endif
         else if (ktype .eq. 1) then
            if (kinop(m,k) .eq. 1) then
               if (autocap(k) .le. 0) then
                  if (ki(k).eq.kf(k)) then
                     pointer = zetf(m,s)-zetf(m,1)+1
                     call trqxz(hloch(phloch(m,k)),trace,dim(m,s))
                     call hphi1mk(psi(zetf(m,s)),hpsi,m,k, &
                          dim(m,s),subdim(m))
                     call xmxxzza(trace,hpsi,dtpsi(pointer), &
                          subdim(m),dim(m,s))
                  endif
               endif
            endif
         endif
      enddo
      
      call MPI_Reduce(dtpsi,workc,phidim1, &
                   MPI_DOUBLE_COMPLEX,MPI_SUM,0,MPI_COMM_WORLD,mpierr)
      call cpvxz(workc,dtpsi,phidim1)

      return
      end subroutine mpiehlochphi
#endif

!-----------------------------------------------------------------------
!   mpigetdavmatz   (Complex dummy routine)
!-----------------------------------------------------------------------
#ifdef MPI
      subroutine mpigetdavmatz(HDiagonal,davmat,indxdav,PsiDim,nn,mc,workc)

      implicit none

      integer(long), intent(in) :: PsiDim,nn
      integer(long), dimension(nn), intent(in) :: indxdav
      real(dop), dimension(nn,nn), intent(in)  :: davmat
      complex(dop), dimension(*), intent(in) ::  mc,workc
      complex(dop), dimension(psidim), intent(in) ::  HDiagonal
      integer(long) :: i,j,fsempost,fsemwait,error,mpierr
      integer(long), dimension(maxdim) :: mii(maxdim),mjj(maxdim)

      write(6,'(a)') "MPI CDAV does not support olsen and"//&
                     " preconditioner yet. STOP"
      call stopnow
      return
      end subroutine mpigetdavmatz
#endif

      end module mpipropmod
