!***********************************************************************
      module opendatmod

use versions, only: progver
use decimal, only: long, dop
use global
use dglobal
use dvrdatmod
use griddatmod
use iopsidef
use operdef
use psidef
use hpsimod
use datenmod
use runpropmod
use dirdyn
use crocoflymod
use lctmod

use logdat
use iofile
use daten, only: speedout

use module_interface

use channels

      implicit none
      private
      public :: opendat,writestop
      contains

!***********************************************************************
! 
!                   Subroutine   OPENDAT
!
! called from subroutine runprop
!
! Opens the output data files and writes "continue" to the stop file.
! if "continue" is altered to "stop" by some editor the program
! stops in a contolled manner at the next output interval.
! The file stop is read in the subroutine "propagatepsi".
!
! variables passed from main:
!  name: name of directory, relative to startup directory containing the
!        .inp file, in which the output data files are stored.
!  laenge: length of string 'name'
!
! V6.0 AJ
!
!***********************************************************************

      subroutine opendat

      implicit none

      integer(long)           :: i,m,s,chkdvr,chkgrd,chkpsi,chkdat,nout,&
                                 tlbl
      integer(long), dimension(maxdim,maxsta) :: tmpdim
      integer(long), external :: log2int
      character(len=10)       :: stat
      character(len=c5)       :: filename,extcmd
      character(len=c4)       :: string
      character(len=c2)       :: timestr
      logical(kind=4)         :: lpsisp,lpsicm,lpsinat,lerr,lold,lexist


      write(ilog,*)
      write(ilog,'(a)') '***** Output files *****'
      write(ilog,*)
      
      lupdate = lupdate.and.(lconstvar.or.lcmfadaphi.or.lcmfadaa)

!-----------------------------------------------------------------------
! Open stop file
!-----------------------------------------------------------------------
      if (lstopf .and. (.not.test)) then
         filename=name(1:laenge)//'/stop'
         open(istop,file=filename,form='formatted',status='unknown', &
              err=10)

         call writestop
     endif

!-----------------------------------------------------------------------
! timestr writes time unit to output string used in file headers
!-----------------------------------------------------------------------
      tlbl=index(rlxtime,' ')-1
      write(timestr,'(3a)') '#    time[',rlxtime(1:tlbl),']'
      tlbl=index(timestr,']')

!-----------------------------------------------------------------------
! OPEN FILES
!
! stat controls whether files are opened as old or unknown.
!-----------------------------------------------------------------------
      if (.not. lcont) then
         if (lovwr) then
            stat='unknown'
         else
            stat='new'
         endif

!-----------------------------------------------------------------------
! if output is written to a file, change output channel from 6 (screen)
! to 13 and open file 'output'
!-----------------------------------------------------------------------
         if (loutfil) then
            iout=13
            filename=name(1:laenge)//'/output'
            open(iout,file=filename,&
                 status=stat,form='formatted',err=10)
            if(lovwr) then
               endfile   iout
               backspace iout
            end if
            write(ilog,'(a)')  'Output      file opened'
         else
            iout=6
            write(ilog,'(a)') 'Output directed to the screen'
         endif

         if (lsheptraj) then
           open(itout,file=name(1:laenge)//'/sheptraj', &
                status=stat,form='formatted')
           write(ilog,'(a)')  'Grow TOUT file opened'
         endif

         if (lactivegp) then
            if (nmode .eq. 1 .or. (nmode .eq. 2 .and. leb)) then
               open(iactive,file=name(1:laenge)//'/activegwp', &
                    status=stat,form='formatted')
               write(ilog,'(a)')  'Active GWP DoF identity file opened'
               close(iactive)

            else if (nstate .gt. 1) then
               do s=1,nstate
                  do m=1,nmode
                     if (.not. gwpm(m)) cycle

                     write(filename,'(2a,i5,a,i5)') &
                           name(1:laenge),'/activegwp_m',m,'_s',s
                     call rmblnk(filename)
                     open(iactive,file=filename,status=stat,form='formatted')
                     write(ilog,'(2(a,i5))')  &
                           'Active GWP DoF identity file opened. Mode: ',&
                            m,' State: ',s
                     close(iactive)
                  enddo
               enddo

            else 
               do m=1,nmode
                  if (.not. gwpm(m)) cycle

                  write(filename,'(2a,i5)') name(1:laenge),'/activegwp_m',m
                  call rmblnk(filename)
                  open(iactive,file=filename,status=stat,form='formatted')
                  write(ilog,'(a,i5)') &
                      'Active GWP DoF identity file opened. Mode: ',m
                  close(iactive)
               enddo
            endif

         endif

         if (lpsi) then
            countpsi=0
            filename=name(1:laenge)//'/psi'
            open(ipsi,file=filename,&
                 form='unformatted',status=stat,err=10)
            write(ipsi) progver
            call wrpsiinfo(ipsi)
            string='Psi         file opened   '
            lpsisp=lpsiopt(1)
            lpsicm=lpsiopt(2)
            lpsinat=lpsiopt(3)
            if( lpsicm ) then
               write(string(26:52),'(a,1pe8.1,a)') &
                     '(compact,',psicut,')'
            else if( lpsinat .and. lpsisp) then
               string(26:50)='(natur,single)'
            else if( lpsinat .and. .not. lpsisp) then
               string(26:50)='(natur,double)'
            else if( lpsisp) then
               string(26:50)='(single)'
            else 
               string(26:50)='(double)'
            endif
            write(ilog,'(a)') trim(string)
         end if 

         countchk=0
         filename=name(1:laenge)//'/check'
         open(ichk,file=filename,form='unformatted',status=trim(stat),err=10)
         write(ichk) progver
         call wrchkinfo(ichk)
         write(ilog,'(a)') 'Check       file opened'

         if (lgpop) then
            countrho=0
            filename=name(1:laenge)//'/gridpop'
            open(irho,file=filename,&
                 form='unformatted',status=stat,err=10)
            write(irho) progver, log2int(lgpel)
            call wrrhoinfo(irho)
            call myflush(irho)
            write(ilog,'(a)') 'Gridpop     file opened'
         end if

         if (lpdens) then
            countpden=0
            filename=name(1:laenge)//'/pdensity'
            open(ipden,file=filename,&
                 form='unformatted',status=stat,err=10)
            write(ipden) progver, log2int(lgpel)
            call wrrhoinfo(ipden)
            call wrpsidef(ipden)
            if(npd1.eq.0) then
               write(ilog,'(a)') 'Pdensity    file opened'
            else
               write(ilog,'(a,i2,3i3,a)') 'Pdensity    file opened  (',&
                       npd1,npd2,npd3,npd4,')'
            end if 
            call myflush(ipden)
         end if 

         if (lspeed) then
            filename=name(1:laenge)//'/speed'
            open(ispeed,file=filename,&
                 form='formatted',status=stat,err=10)
             if(lovwr) then
               endfile   iout
               backspace iout
            end if
            write(ilog,'(a)') 'Speed       file opened'
         endif

         if (lctr) then
            filename=name(1:laenge)//'/ctrace'
            open(ictr,file=filename,&
                 form='formatted',status=stat,err=10)
            write(ictr,'(a,9x,a,5x,a)') timestr(1:tlbl),'Re(corr_trace)'&
                      ,'Im(corr_trace)     Abs(corr_trace)'
            write(ilog,'(a,2i5)') &
                  'Ctrace      file opened  left / right = ',dlstate,drstate
         endif 

         if (lcross) then
            filename=name(1:laenge)//'/cross'
            open(icroco,file=filename,&
                 form='formatted',status=stat,err=10)
            write(icroco,'(a,a)')&
                 '# Cross-correlation with ',crossname(1:crosslng)
            if(nst1.ne.1 .or. nst2.ne.nstate) &
            write(icroco,'(a,i3)') '# State =', nst1
            write(icroco,'(a,9x,a,5x,a)') timestr(1:tlbl),&
                 'Re(cross-corr)','Im(cross-corr)     Abs(cross-corr)' 
            if(lovwr) then
               endfile   icroco
               backspace icroco
            endif
            write(ilog,'(a)') 'Cross       file opened'
         endif

         if (lauto) then
            filename=name(1:laenge)//'/auto'
            open(iaut,file=filename,&
                 form='formatted',status=stat,err=10)
            write(iaut,'(a,9x,a,5x,a)') timestr(1:tlbl),'Re(autocorrel)'&
                      ,'Im(autocorrel)     Abs(autocorrel)' 
            if(lovwr) then
               endfile   iaut
               backspace iaut
            end if
            if(lautodoub) then
               write(ilog,'(a)') 'Auto        file opened  (twice)'
            else
               write(ilog,'(a)') 'Auto        file opened  (once)'
            endif
         endif

         if (lautoe) then
            filename=name(1:laenge)//'/autoe'
            open(iaute,file=filename, &
                 form='unformatted',status=stat,err=10)
            write(iaute) progver
            call wrauteinfo(iaute)
            if( lautodoub ) then
               write(ilog,'(a)') 'Autoe       file opened  (twice)'
            else
               write(ilog,'(a)') 'Autoe       file opened  (once)'
            endif
         end if

         if (lautoord1) then
            filename=name(1:laenge)//'/auto1'
            open(iaut1,file=filename, &
                 form='formatted',status=stat,err=10)
            write(iaut1,'(a,9x,a,5x,a)') timestr(1:tlbl), &
                'Re(autocorrel)','Im(autocorrel)     Abs(autocorrel)' 
            if (lautodoub) then
               write(ilog,'(a)') 'Auto1       file opened  (twice)'
            else
               write(ilog,'(a)') 'Auto1       file opened  (once)'
            endif
         endif

         if (lautoord2) then
            filename=name(1:laenge)//'/auto2'
            open(iaut2,file=filename, &
                 form='formatted',status=stat,err=10)
            write(iaut2,'(a,9x,a,5x,a)') timestr(1:tlbl), &
                'Re(autocorrel)','Im(autocorrel)     Abs(autocorrel)' 
            if (lautodoub) then
               write(ilog,'(a)') 'Auto2       file opened  (twice)'
            else
               write(ilog,'(a)') 'Auto2       file opened  (once)'
            endif
         endif

!-----------------------------------------------------------------------
! If expectation values required open files and load operator names 
! into expectlab array
!-----------------------------------------------------------------------
         if (nexpect.ne.0) then
            filename=name(1:laenge)//'/expectation'
            open(iexpect,file=filename, &
                 form='formatted',status=stat,err=10)
            if(lreonly) then
               write(iexpect,'(a,5x,a/)') '# real-only', &
               'expect = Re(<psi|op|psi>/<psi|psi>)'
               write(iexpect,'(a,4x,a,6x,90(a16,1x))') &
               timestr(1:tlbl),'norm',(expectlab(i)(1:16),i=1,nexpect)
            else
               write(iexpect,'(a,5x,a/)') '# complex', &
               'expect = <psi|op|psi>/<psi|psi>'
               write(iexpect,'(a,4x,a,12x,90(a20,12x))') & 
               timestr(1:tlbl),'norm',(expectlab(i)(1:20),i=1,nexpect)
            endif
            write(ilog,'(a)') 'Expectation file opened'
         endif

         if (nexpect1.ne.0) then
            filename=name(1:laenge)//'/expect1'
            open(iexpect1,file=filename, &
                 form='formatted',status=stat,err=10)
            if(lreonly1) then
               write(iexpect1,'(a,5x,a/)') '# real-only', &
               'expect = Re(<psi|op|psi>/<psi|psi>)'
               write(iexpect1,'(a,4x,a,6x,90(a16,1x))') timestr(1:tlbl), &
                  'norm',(expectlab1(i)(1:16),i=1,nexpect1)
            else
               write(iexpect1,'(a,5x,a/)') '# complex', &
               'expect = <psi|op|psi>/<psi|psi>'
               write(iexpect1,'(a,4x,a,12x,90(a20,12x))') timestr(1:tlbl),  &
                  'norm',(expectlab1(i)(1:20),i=1,nexpect1)
            endif
            write(ilog,'(a)') 'Expect1 file opened'
         endif


         if (lorben) then
            filename=name(1:laenge)//'/orben'
            open(iorb,file=filename, &
                 form='formatted',status=stat,err=10)
            write(iorb,'(a)')'******** Orbital energies in eV ********'
            write(ilog,'(a)') 'Orben       file opened'
         endif

         if (lintstep.and..not.lmlcmfsplit) then
            filename=name(1:laenge)//'/steps'
            open(istp,file=filename, &
            form='formatted',status=stat,err=10)
                 write(ilog,'(a)') 'Steps       file opened'
            if(lovwr) then
               endfile   istp
               backspace istp
            end if
         endif
         
         if (lupdate) then
            filename=name(1:laenge)//'/update'
            open(iupd,file=filename,form='formatted',status=stat,err=10)
            write(ilog,'(a)') 'Update      file opened'
            if (lrunml) then
               write(iupd,'(a12,a12,a18,a16)') '#Update-step',&
                    'Updatetime','Total Error','Time in fs'
            else
               write(iupd,'(a12,a12,a15,a20,a16)') '#Update-step',&
                    'Updatetime','Error of A','Error of phi','Time in fs'
            endif
            if(lovwr) then
               endfile   iupd
               backspace iupd
            end if
         endif

         if (lrundiag) then
            filename=name(1:laenge)//'/eigval'
            open(ieigval,file=filename, &
               form='formatted',status=stat,err=10)
            write(ilog,'(a)') 'Eigenvalue  file opened'
         endif

         if (llanczvec) then
            filename=name(1:laenge)//'/lanczvec'
            open(ilancz,file=filename, &
                 form='unformatted',status=stat,err=10)
            write(ilog,'(a)') 'Lanczos     file opened'
         endif

         if (leigvec) then
            filename=name(1:laenge)//'/eigvec'
            open(ieigvec,file=filename, &
                 form='formatted',status=stat,err=10)
            write(ilog,'(a)') 'Eigenvector file opened'
         endif

         if (rlaxnum.ge.0) then
            filename=name(1:laenge)//'/rlx_info'
            open(irlx,file=filename, &
                 form='formatted',status=stat,err=10)
            write(ilog,'(a)') 'rlx_info    file opened'
            if(lovwr) then
               endfile   irlx
               backspace irlx
            end if
         endif

         if (lrunml) then
            filename=name(1:laenge)//'/lownatpop'
            open(ilnp,file=filename,form='formatted',&
                 status=stat,err=10)
            write(ilog,'(a)') 'lownatpop   file opened'
            if(lovwr) then
               endfile   ilnp
               backspace ilnp
            end if
         endif

!        If dynamical WF: open read-write files
         if (ldynspf.or.linitorb) then
            filename=name(1:laenge)//'/spwneval'
            open(iopteigen,file=filename,form='formatted',status=stat,err=10)
            write(ilog,'(a)') 'spwneval    file opened'
            if (ldynspf) then
               filename=name(1:laenge)//'/nspf'
               open(inspf,file=filename,form='formatted',status=stat,err=10)
               write(inspf,'(7x,a)') 'Time   SPFs'
               write(ilog,'(a)') 'nspf        file opened'
            endif
            if (ldebugspawn) then
               if (ldynspf) then
                  filename=name(1:laenge)//'/spawn'
                  open(ispwn,file=filename,form='formatted',status=stat,err=10)
                  write(ilog,'(a)') 'spawn       file opened'
               endif
               if (linitorb) then
                  do m=1,nmode
                     do s=1,nstate
                        if (m.ne.meb) then
                           write(filename,'(a,i3,a,i2)') '/spwnevec_',m,'_',s
                           call rmblnk(filename)
                           inquire(file=name(1:laenge)//filename,exist=lold)
                           if(lold) stat = 'replace'
                           open(unit   = ioptveig, &
                              file   = name(1:laenge)//filename, &
                              form   = 'formatted', &
                              status = stat)
                        endif
                     enddo
                  enddo
               endif
            endif
         endif

         if (lcontrol) then
            filename=name(1:laenge)//'/field.lct'
            open(icontrol,file=filename, &
                 form='formatted',status=stat,err=10)
            write(ilog,'(a)') 'LCT field file opened'
         endif
      
         if (lBEL) then
            filename=name(1:laenge)//'/BELout'
            open(iBEL,file=filename,&
                 form='formatted',status=stat,err=10)
         endif

         if (lgentrj) then
! see if files exist and delete them
            filename=name(1:laenge)//'/trjdat/1.edat'
            inquire(file=filename,exist=lold)
            if (lold) then
               extcmd='rm -rf '//name(1:laenge)//'/trjdat'
               call excmd(extcmd)
            endif

! set up directory "mkdir -p" gives no error if it already exists.
            filename=name(1:laenge)//'/trjdat'
            extcmd='mkdir -p '//name(1:laenge)//'/trjdat'
            call excmd(extcmd)

            nout = nint((tfinal-tinit)/out1) + 1
            stat='new'
            call opentrjdat(ndof,nddstate,gwpdim(1,1),nout,stat)
            
         endif

! -----------------------------------------------------------------

      else if (lcont) then

         if (loutfil) then
            iout=13
            filename=name(1:laenge)//'/output'
            inquire(file=filename,exist=lold)
            if(lold) then 
               open(iout,file=filename, &
                    status='old',form='formatted',position='append',err=10)
               write(iout,'(2a)') '# -------------------------------', &
                      ' continuation -------------------------------'
               write(ilog,'(2a)')'Output      file opened.  Status: old'
            else
               open(iout,file=filename, &
                    status='new',form='formatted',err=10)
               write(ilog,'(2a)')'Output      file opened.  Status: new'
            endif
         else
            iout=6
            write(ilog,'(a)') 'Output directed to the screen'
         endif

         if (ldynspf) then  ! in data files, recorded dim is maxdimspwn
            tmpdim = dim
            dim = maxdimspwn
         endif

         filename=name(1:laenge)//'/check'
         inquire(file=filename,exist=lold)
         if(.not.lold) then
            routine='Opendat'
            message='No check-file for continuation run.'
            call errormsg
         end if
         open(ichk,file=filename,form='unformatted',status='old',err=10)
         write(ilog,'(a)') 'Check       file opened'
         read(ichk) filever(ichk)
         chkdvr=2
         chkgrd=2
         chkpsi=2
         chkdat=0
         call rdchkinfo(ichk,chkdvr,chkgrd,chkpsi,chkdat)
         call chksyserr(filename,lerr)
         if (lerr) go to 999

         call reposition2(ichk,filename,countchk)

         if (lpsi) then
            filename=name(1:laenge)//'/psi'
            inquire(file=filename,exist=lold)
            if(.not.lold) then
               routine='opendat'
               message='No psi-file for continuation run.'
               call errormsg
            end if
            open(ipsi,file=filename,form='unformatted',status='old',err=10)
            string='Psi         file opened   '
            lpsisp=lpsiopt(1)
            lpsicm=lpsiopt(2)
            lpsinat=lpsiopt(3)
            if( lpsicm ) then
               write(string(26:52),'(a,1pe8.1,a)') &
                    '(compact,',psicut,')'
            else if( lpsinat .and. lpsisp) then
               string(26:50)='(natur,single)'
            else if( lpsinat .and. .not. lpsisp) then
               string(26:50)='(natur,double)'
            else if( lpsisp) then
               string(26:50)='(single)'
            else 
               string(26:50)='(double)'
            endif
            write(ilog,'(a)') trim(string)

            read(ipsi) filever(ipsi)
            chkdvr=2
            chkgrd=2
            chkpsi=2
            chkdat=0
            call rdpsiinfo(ipsi,chkdvr,chkgrd,chkpsi,chkdat)
            call chksyserr(filename,lerr)
            if (lerr) go to 999
            
            call reposition2(ipsi,filename,countpsi)
         endif
         
         if (ldynspf) then  ! restore actual dim
            dim = tmpdim
         endif
         
         if (lgpop) then
            filename=name(1:laenge)//'/gridpop'
            inquire(file=filename,exist=lold)
            stat = 'new'
            if(lold) stat = 'old'
            open (irho,file=filename, &
                  form='unformatted',status=stat,err=10)
            write(ilog,'(2a)')'Gridpop     file opened.  Status: ',stat
            if (.not.lold) then
               write(irho) progver, log2int(lgpel)
               call wrrhoinfo(irho)
            else 
               read(irho) filever(irho)
               chkdvr=2
               chkgrd=2
               chkdat=0
               call rdrhoinfo(irho,chkdvr,chkgrd,chkdat)
               call chksyserr(filename,lerr)
               if (lerr) go to 999
               call reposition2(irho,filename,countrho)
            end if
         end if 

         if (lpdens) then
            filename=name(1:laenge)//'/pdensity'
            inquire(file=filename,exist=lold)
            stat = 'new'
            if(lold) stat = 'old'
            open (ipden,file=filename, &
                  form='unformatted',status=stat,err=10)
            write(ilog,'(2a)')'pdensity    file opened.  Status: ',stat
            if (.not.lold) then
               write(ipden) progver, log2int(lgpel)
               call wrrhoinfo(ipden)
               call wrpsidef(ipden)
            else 
               read(ipden) filever(ipden)
               chkdvr=2
               chkgrd=2
               chkdat=0
               call rdrhoinfo(ipden,chkdvr,chkgrd,chkdat)
               call rdpsidef(ipden,chkdat)
               call chksyserr(filename,lerr)
               if (lerr) go to 999
               call reposition2(ipden,filename,countpden)
            end if
         end if 

         if (lspeed) then
            filename=name(1:laenge)//'/speed'
            inquire(file=filename,exist=lold)
            stat = 'new'
            if(lold) stat = 'old'
            open(ispeed,file=filename, &
                 form='formatted',status=stat,err=10)
            write(ilog,'(2a)')'Speed       file opened.  Status: ',stat
            if (lold) then
               call reposition(ispeed,filename)
               write(ispeed,'(2a)') '# -------------------------------', &
                      ' continuation -------------------------------'
               call speedout(-2.d40)
            endif
         endif


         if (lctr) then
            filename=name(1:laenge)//'/ctrace'
            inquire(file=filename,exist=lold)
            if(.not.lold) then
               routine='opendat'
               message='No ctrace-file for continuation run.'
               call errormsg
            end if
            open (ictr,file=filename, &
                  form='formatted',status='old',err=10)
            write(ilog,'(2a)')'Ctrace      file opened.  Status: old'
            call reposition(ictr,filename)
         endif

         if (lcross) then
            filename=name(1:laenge)//'/cross'
            inquire(file=filename,exist=lold)
            stat = 'new'
            if(lold) stat = 'old'
            open (icroco,file=filename, &
                  form='formatted',status=stat,err=10)
            write(ilog,'(2a)')'Cross       file opened.  Status: ',stat
            if(lold) then
               call reposition(icroco,filename)
            else
               write(icroco,'(a,a)') &
                  '# Cross-correlation with ',crossname(1:crosslng)
               if(nst1.ne.1 .or. nst2.ne.nstate)  &
                  write(icroco,'(a,i3)') '# State =', nst1
               write(icroco,'(a,9x,a,5x,a)') timestr(1:tlbl), &
                  'Re(cross-corr)','Im(cross-corr)     Abs(cross-corr)' 
            endif
         endif

         if (lauto) then
            filename=name(1:laenge)//'/auto'
            inquire(file=filename,exist=lold)
            if(.not.lold) then
               routine='opendat'
               message='No auto-file for continuation run.'
               call errormsg
            end if
            open (iaut,file=filename,&
                  form='formatted',status='old',err=10)
            if( lautodoub ) then
               write(ilog,'(a)') 'Auto        file opened  (twice)'
            else
               write(ilog,'(a)') 'Auto        file opened  (once)'
            endif
            call reposition(iaut,filename)
         endif

         if (lautoe) then
            filename=name(1:laenge)//'/autoe'
            inquire(file=filename,exist=lold)
            if(.not.lold) then
               routine='opendat'
               message='No autoe-file for continuation run.'
               call errormsg
            end if
            open (iaute,file=filename, &
                  form='unformatted',status='old',err=10)
            if( lautodoub ) then
               write(ilog,'(a)') 'Autoe       file opened  (twice)'
            else
               write(ilog,'(a)') 'Autoe       file opened  (once)'
            endif
            read(iaute) filever(iaute)
            chkdvr=2
            chkgrd=2
            chkpsi=2
            chkdat=0
            call rdauteinfo(iaute,chkdvr,chkgrd,chkpsi,chkdat)
            call chksyserr(filename,lerr)
            if (lerr) go to 999
            call reposition1(iaute,filename,i)
         endif

         if (lautoord1) then
            filename=name(1:laenge)//'/auto1'
            inquire(file=filename,exist=lold)
            if(.not.lold) then
               routine='opendat'
               message='No auto1-file for continuation run.'
               call errormsg
            end if
            open(iaut1,file=filename,&
                 form='formatted',status='old',err=10)
            if (lautodoub) then
               write(ilog,'(a)') 'Auto1       file opened  (twice)'
            else
               write(ilog,'(a)') 'Auto1       file opened  (once)'
            endif
            call reposition(iaut1,filename)
         endif
         
         if (lautoord2) then
            filename=name(1:laenge)//'/auto2'
            inquire(file=filename,exist=lold)
            if(.not.lold) then
               routine='opendat'
               message='No auto2-file for continuation run.'
               call errormsg
            end if
            open(iaut2,file=filename,&
                 form='formatted',status='old',err=10)
            if (lautodoub) then
               write(ilog,'(a)') 'Auto2       file opened  (twice)'
            else
               write(ilog,'(a)') 'Auto2       file opened  (once)'
            endif
            call reposition(iaut2,filename)
         endif
      
         if (nexpect.ne.0) then
            filename=name(1:laenge)//'/expectation'
            inquire(file=filename,exist=lold)
            stat = 'new'
            if(lold) stat = 'old'
            open (iexpect,file=filename, &
                  form='formatted',status=stat,err=10)
            write(ilog,'(2a)')'Expectation file opened.  Status: ',stat
            if (lold) then
               call reposition(iexpect,filename)
            else
               if(lreonly) then
                  write(iexpect,'(a,5x,a/)') '# real-only', &
                     'expect = Re(<psi|op|psi>/<psi|psi>)'
                  write(iexpect,'(a,4x,a,6x,90(a16,1x))') &
                  timestr(1:tlbl),'norm',(expectlab(i)(1:16),i=1,nexpect)
               else
                  write(iexpect,'(a,5x,a/)') '# complex', &
                     'expect = <psi|op|psi>/<psi|psi>'
                  write(iexpect,'(a,4x,a,12x,90(a20,12x))') &
                  timestr(1:tlbl),'norm',(expectlab(i)(1:20),i=1,nexpect)
               endif
            endif
         endif

         if (nexpect1.ne.0) then
            filename=name(1:laenge)//'/expect1'
            inquire(file=filename,exist=lold)
            stat = 'new'
            if(lold) stat = 'old'
            open (iexpect1,file=filename, &
                  form='formatted',status=stat,err=10)
            write(ilog,'(2a)')'Expect1 file opened.  Status: ',stat
            if (lold) then
               call reposition(iexpect1,filename)
            else
               if(lreonly1) then
                  write(iexpect1,'(a,5x,a/)') '# real-only', &
                     'expect = Re(<psi|op|psi>/<psi|psi>)'
                  write(iexpect1,'(a,4x,a,6x,90(a16,1x))') timestr(1:tlbl), &
                     'norm',(expectlab(i+nexpect)(1:16),i=1,nexpect1)
               else
                 write(iexpect1,'(a,5x,a/)') '# complex', &
                     'expect = <psi|op|psi>/<psi|psi>'
                 write(iexpect1,'(a,4x,a,12x,90(a20,12x))') timestr(1:tlbl), &
                     'norm',(expectlab(i+nexpect)(1:20),i=1,nexpect1)
               endif
            endif
         endif

 
         if (lorben) then
            filename=name(1:laenge)//'/orben'
            inquire(file=filename,exist=lold)
            stat = 'new'
            if(lold) stat = 'old'
            open (iorb,file=filename, &
                  form='formatted',status=stat,err=10)
            write(ilog,'(2a)')'Orben       file opened.  Status: ',stat
            if (lold) then
               call reposition(iorb,filename)
            else
               write(iorb,'(a)') &
                    '******** Orbital energies in eV ********'
            endif
         endif

         if (lintstep.and..not.lmlcmfsplit) then
            filename=name(1:laenge)//'/steps'
            inquire(file=filename,exist=lold)
            stat = 'new'
            if(lold) stat = 'old'
            open (istp,file=filename, &
                 form='formatted',status=stat,err=10)
            write(ilog,'(2a)')'Steps       file opened.  Status: ',stat
            if (lold) then
               call reposition(istp,filename)
               write(istp,'(2a)') '# -------------------------------', &
                      ' continuation -------------------------------'
            endif
         endif

         if (lupdate) then
            filename=name(1:laenge)//'/update'
            inquire(file=filename,exist=lold)
            stat = 'new'
            if(lold) stat = 'old'
            open(iupd,file=filename, &
                 form='formatted',status=stat,err=10)
            write(ilog,'(2a)')'Update      file opened.  Status: ',stat
            if (lold) then
               call reposition(iupd,filename)
               write(iupd,'(2a)') '# -------------------------------', &
                      ' continuation -------------------------------'
            else
               if (rlxtime .eq. 'au') then
                  string='Time in au'
               else if (rlxtime .eq. 'fs') then
                  string='Time in fs'
               endif
               if (lrunml) then
                  write(iupd,'(a12,a12,a18,a16)') '#Update-step',&
                       'Updatetime','Total Error',trim(string)
               else
                  write(iupd,'(a12,a12,a15,a20,a16)') '#Update-step',&
                       'Updatetime','Error of A','Error of phi',trim(string)
               end if
            end if
         endif
         
         if (lrundiag) then
            filename=name(1:laenge)//'/eigval'
            inquire(file=filename,exist=lold)
            if(.not.lold) then
               routine='opendat'
               message='No eigval-file for continuation run.'
               call errormsg
            end if
            open(ieigval,file=filename, &
                 form='formatted',status='old',err=10)
            write(ilog,'(a)') 'Eigenvalue  file opened'
         endif
         
         if (llanczvec) then
            filename=name(1:laenge)//'/lanczvec'
            inquire(file=filename,exist=lold)
            if(.not.lold) then
               routine='opendat'
               message='No lanczvec-file for continuation run.'
               call errormsg
            end if
            open(ilancz,file=filename, &
                 form='unformatted',status='old',err=10)
            write(ilog,'(a)') 'Lanczos     file opened'
            call reposition1(ilancz,filename,i)
         endif
         
         if (leigvec) then
            filename=name(1:laenge)//'/eigvec'
            inquire(file=filename,exist=lold)
            if(.not.lold) then
               routine='opendat'
               message='No eigvec-file for continuation run.'
               call errormsg
            end if
            open(ieigvec,file=filename, &
                 form='unformatted',status='old',err=10)
            write(ilog,'(a)') 'Eigenvector file opened'
            call reposition1(ieigvec,filename,i)
         endif

         if (rlaxnum.ge.0) then
            filename=name(1:laenge)//'/rlx_info'
            inquire(file=filename,exist=lold)
            if(.not.lold) then
               routine='opendat'
               message='No rlx_info file for continuation run.'
               call errormsg
            end if
            open(irlx,file=filename, &
                 form='formatted',status='old',err=10)
            write(ilog,'(a)') 'rlx_info    file opened'
            call reposition(irlx,filename)
         endif

         if (lrunml) then
            filename=name(1:laenge)//'/lownatpop'
            inquire(file=filename,exist=lold)
            stat = 'new'
            if(lold) stat = 'old'
            open (ilnp,file=filename,&
                 form='formatted',status=stat,err=10)
            write(ilog,'(2a)')'lownatpop   file opened.  Status: ',stat
            if (lold) call reposition(ilnp,filename)
         endif

!        If dynamical WF: open read-write files
         if (ldynspf.or.linitorb) then
            filename=name(1:laenge)//'/spwneval'
            inquire(file=filename,exist=lold)
            stat = 'new'
            if(lold) stat = 'old'
            open (iopteigen,file=filename,form='formatted',status=stat,err=10)
            write(ilog,'(2a)')'spwneval    file opened.  Status: ',stat
            if (lold) call reposition(ilnp,filename)
            if (ldynspf) then
               filename=name(1:laenge)//'/nspf'
               inquire(file=filename,exist=lold)
               stat = 'new'
               if(lold) stat = 'old'
               open (inspf,file=filename,form='formatted',status=stat,err=10)
               write(ilog,'(2a)')'inspf       file opened.  Status: ',stat
               if (lold) call reposition(ilnp,filename)
            endif
            if (linitorb) then
               do m=1,nmode
                  do s=1,nstate
                     if (m.ne.meb) then
                     write(filename,'(a,i3,a,i2)') '/spwnevec_',m,'_',s
                     call rmblnk(filename)
                     inquire(file=name(1:laenge)//filename,exist=lold)
                     stat = 'new'
                     if(lold) stat = 'old'
                     open(unit   = ioptveig, &
                          file   = name(1:laenge)//filename, &
                          form   = 'formatted', &
                          status = stat)
                     if (lold) call reposition(ilnp,filename)
                     endif
                  enddo
               enddo
            endif

         endif

         if (lcontrol) then
            filename=name(1:laenge)//'/field.lct'
            inquire(file=filename,exist=lold)
            if(.not.lold) then
               routine='opendat'
               message='No field.lct file for continuation run.'
               call errormsg
            endif
            open(icontrol,file=filename, &
                 form='formatted',status='old',err=10)
            write(ilog,'(a)') 'LCT field file opened'
            call reposition(icontrol,filename)
         endif

         if (lBEL) then
            filename=name(1:laenge)//'/BELout'
            inquire(file=filename,exist=lold)
            if(.not.lold) then
               routine='opendat'
               message='No BELout-file for continuation run.'
               call errormsg
            end if
            open (iBEL,file=filename,&
                  form='formatted',status='old',err=10)
            call reposition(iBEL,filename)
         endif

         if (lgentrj) then
! check directory exists
            filename=name(1:laenge)//'/trjdat'
            inquire(file=filename,exist=lexist)

            if (.not. lexist) then
               routine='opendat'
               message='No trjdat data directroy for continuation run.'
               call errormsg
            endif

         endif
         
      endif

      call myflush(ilog)
      
      return
      
   10 continue
      write(6,'(2a)') 'opendat.F; ERROR open. Status: ',stat
      routine = 'OpenDat'
      message = 'Cannot open data file: '//filename(laenge+2:)
      call errormsg
  999 continue
      routine = 'OpenDat (cont)'
      message= 'Error reading :'//filename(laenge+2:)
      call errormsg

      end subroutine opendat

!#######################################################################
! WRITESTOP
!
! writes text to the stop file for editing during a run
!#######################################################################

      subroutine writestop

      implicit none

      integer(long) :: i

      if(sstop.eq.' ' .and. scpu.eq.' ' .and. swall.eq.' ') then
         write(istop,'(a8)') 'continue'
      else
         write(istop,'(a6,2x,a9,2x,a9)') sstop,scpu,swall
      endif

      write(istop,'(a)' ) 'hhh:mm  hhh:mm:ss  hhh:mm:ss  comment '
      write(istop,'(a)' ) 'real    cpu        wall-time  optional'
      write(istop,'(a)' )
      write(istop,'(75a)') ('-',i=1,75)
      write(istop,'(a,a)' )                                &
        ' Replace "continue" by "stop   comment" or',      &
        ' "hhh:mm  hhh:mm:ss  comment"'
      write(istop,'(a,a)' )                                &
        ' The first input (hhh:mm) refers to real-time,',  &
        ' the second to cpu-time.'
      write(istop,'(a)' )                                  &
      ' One or two of the time entries may be left blank.'
      write(istop,'(a)' )                                  &
      ' The colon ":" must appear at the correct position, e.g : '
      write(istop,'(a)' ) '032:30             system update '
      write(istop,'(a,/a,/a,/a)' )                         &
      ' will stop the run after 08:30 next day (8+24=32)', &
      ' NB: The day-count starts after the first read of "hhh:mm",',  &
      ' e.g. after "continue" has been replaced by "hhh:mm".',        &
      ' A first line which reads:'
      write(istop,'(a)' ) '        000:56:30             time limit'
      write(istop,'(2a,/a,/a,/2a)' )                       &
      ' will stop the run after the first output later',   &
      ' than 56.5 cpu minutes',                            &
      ' and similarly a first line which reads:',          &
      '                   001:30:00  time limit',          &
      ' will stop the run after the first output later',   &
      ' than 90 wall-time minutes'
      write(istop,'(/2a,/2a,/2a)' )                        &
      ' When this file (the stop file) is removed, the run will',   &
      ' terminate',' after next output. One may use the keywords',  &
      ' "tstop", "tcpu" and "twall"', &
      ' (Run-Section) to set the times',      &
      ' "hhh:mm", "hhh:mm:ss" and "hhh:mm:ss".'
      write(istop,'(/2a)') ' When the run terminates normally,',    &
           ' the stop file will be removed.'
      write(istop,'(75a)') ('-',i=1,75)
      write(istop,'(a)' )
      write(istop,'(a)' ) ' <tout>'
      endfile(istop)
      close (istop)
      write(ilog,'(a)') 'Stop        file created'

      return
      end subroutine writestop

!#######################################################################
! REPOSITION
!
! repositions the formatted file 'filename' on unit 'iunit' to its end
!#######################################################################

      subroutine reposition(iunit,filename)

      implicit none

      integer(long)                 :: ilbl
      integer(long), intent(in)     :: iunit
      character(len=c5), intent(in) :: filename


   10 continue
      read(iunit,'(a)',end=20)
      go to 10
   20 backspace(iunit)
      ilbl=index(filename,' ')-1
      write (ilog,'(2a)') filename(1:ilbl)//' file repositioned'

      return
      end subroutine reposition

!#######################################################################
! REPOSITION1
!
! repositions the unformatted file 'filename' on unit 'iunit' to its end
!#######################################################################

      subroutine reposition1(iunit,filename,count)

      implicit none

      integer(long)                 :: ilbl
      integer(long), intent(in)     :: iunit
      integer(long), intent(out)    :: count
      character(len=c5), intent(in) :: filename

   10 continue
      read(iunit,end=20)
      count = count + 1
      go to 10
   20 backspace(iunit)
      ilbl=index(filename,' ')-1
      write (ilog,'(2a)') filename(1:ilbl)//' file repositioned'

      return
      end subroutine reposition1

!#######################################################################
! REPOSITION2
!
! repositions an unformatted file 'filename' on unit 'iunit' to its end
!#######################################################################

      subroutine reposition2(unit,filename,count)

      implicit none

      integer(long)                 :: ilbl,jlbl,klbl
      integer(long), intent(in)     :: unit
      integer(long), intent(out)    :: count
      character(len=c5), intent(in) :: filename
      logical(kind=4)               :: check
      character(len=c2)             :: rdtag

      inquire(unit,opened=check)
      if (check) close(unit)

      klbl=index(filename,' ')-1
      open(unit,file=filename(1:klbl),form='unformatted',status='old', &
                  position='append',err=10)
 
      backspace(unit)

      read(unit,err=10) rdtag
      if (rdtag(1:1) .ne. '<') go to 10

      ilbl=2
      jlbl=index(rdtag,'>')-1
      read(rdtag(jlbl-9:jlbl),'(i10)') count

      write (ilog,'(2a)') filename(1:klbl)//' file repositioned'
      write (ilog,'(a,i7)') '          No. records found: ',count

      return
   10 routine='Reposition2'
      write(message,'(a,a,i5)') 'Error Opening file :', &
                filename(1:klbl),unit
      call errormsg
      end subroutine reposition2

!#######################################################################
! 
! opens files to generate trajectory data files
!
!#######################################################################

      subroutine opentrjdat(ndof1,nsta,ntrj,nout,stat)

      implicit none

      integer(long), intent(in)     :: ndof1,nsta,ntrj,nout
      integer(long)                 :: f,s,e
      character(len=c5)             :: string
      character(len=10)             :: stat

      do e=1,ntrj
         write(string,'(2a,i5,a5)') name(1:laenge),'/trjdat/',e,'.edat'
         call rmblnk(string)
         open(unit=idump, file=string,status=stat,form='formatted')
         write(idump,*) nout
         close(idump)

         do f=1,ndof1
            if (f .eq. feb) cycle
            write(string,'(2a,i5,a1,i5,a5)') name(1:laenge),'/trjdat/', &
                 f,'.',e,'.xdat'
            call rmblnk(string)
            open(unit=idump, file=string,status=stat,form='formatted')
            write(idump,*) nout
            close(idump)

            write(string,'(2a,i5,a1,i5,a5)') name(1:laenge),'/trjdat/', &
                 f,'.',e,'.pdat'
            call rmblnk(string)
            open(unit=idump, file=string,status=stat,form='formatted')
            write(idump,*) nout
            close(idump)
         enddo

         do s=1,nsta
            write(string,'(2a,i5,a1,i5,a6)') name(1:laenge),'/trjdat/d', &
                 s-1,'.',e,'.cidat'
            call rmblnk(string)
            open(unit=idump, file=string,status=stat,form='formatted')
            write(idump,*) nout
            close(idump)
         enddo
      enddo
            
      return
      end subroutine opentrjdat
      
      end module opendatmod

