!######################################################################
!                                                                      
!                      MCTDH module OUTPUT                             
!                                                                      
! Controls the output of data.                                         
!                                                                      
!######################################################################
      module outputmod

use decimal, only: dop, long
use globmemmod
use logdat
use xvlib, only: qvxxzz
use global
use dglobal
use dvrdatmod
use griddatmod
use psidef
use runpropmod
use channels
use operdef
use datenmod
use lctmod
use hpsimod, only: ghmatdim,ghmat,pmatdim,pmat

use autocorr, only: autoout,autoouthigh
use timing
use psioutmod, only: psiout
use iorst, only: wrrst
use function, only: funk2,funk3
use datoutmod, only: datout
use dichtoutmod, only: dichtout
use daten, only: wrchck,speedout
use orbenmod, only: e_funkphi
use calcdatmod, only: calcdat
use crocofly, only: ccout
use d1outgpop, only: outgpop1
use d2outgpop, only: outgpop2
use d1calcdatmod, only: d1calcdat
use gwplib, only: gwp_param,transahelp
use d2calcdatmod, only: d2calcdat
use d1crocofly, only: d1ccout
use d1autocorr, only: d1autoout
use d1traceoutmod, only: d1traceout
use d2crocofly, only: d2ccout
use d2autocorr, only: d2autoout
use d2traceoutmod, only: d2traceout
use op1lib, only: cpvxd, cpqvzd
use op2lib, only: sum2qxxdz
use potevalmod, only: calcpes,calcdiab
use expectval, only: lnwexpect

use propmod_interface
use multilayer
use gwptrj
use gwptrjmod

      implicit none
      private
      public :: output
      contains

!**********************************************************************
!                                                                      
!            MCTDH subroutine OUTPUT                                   
!                                                                      
! time  : actual time of output                                        
! psi   : wavefunction at time                                         
! dtpsi : derivative of wavefunction at time i.e. -i H|psi>            
!         (or -H|psi> in relaxation run)                               
!                                                                      
!  V6.6 GW                                                             
!                                                                      
!**********************************************************************

      subroutine output(time,psi,dtpsi,normt,trace,etott)

      implicit none

      logical(kind=4), save                          :: firstcall = .true.
      logical(kind=4), save                          :: lhead

      integer(long)                                  :: m,n,s
      real(dop)                                      :: rftime, &
                                                        newtime,atime,runtime
      real(dop), intent(in)                          :: time
      real(dop), intent(out)                         :: normt,trace,etott
      real(dop), dimension(maxsta)                   :: spop,ecorr,etot,euncorr
      real(dop)                                      :: detot2, norm1
      real(dop), dimension(ngwptrjst)                :: spoptr
      real(dop), dimension(nddstate)                 :: sspop
      real(dop), dimension(4,ndof,nstate)            :: qdq
      real(dop), dimension(gwpinfodim)               :: ggp
      complex(dop), dimension(dgldim), intent(inout) :: psi
      complex(dop), dimension(dgldim), intent(inout) :: dtpsi
      character(len=c5)                              :: filename
      integer(long), save                            :: tid=0 ! timer ID

      if(test0) return

      call get_timer(tid, 'Output')
      call start_timer(tid)

      qdq(:,:,:)=0.0_dop
     
!-----------------------------------------------------------------------
! sort current GWP parameters
!-----------------------------------------------------------------------
      if (lgwptrj) then
         call gwptrj_param(m,time,psi,agmat,gfgwp,aigwp,zgwp,mr(mrqcent))
      else if (lgwp) then
         call gwp_param(psi,agmat,gfgwp,aigwp,zgwp,mr(mrqcent),1)
      endif

!-----------------------------------------------------------------------
! write out psi file, if required
!-----------------------------------------------------------------------
      runtime = time - tinit
      if ((lpsi) .and. (.not.lBEL)) then 
         if (abs(runtime-out2*nint(runtime/out2)).lt.(out2*1.0e-4_dop)) then
            call psiout(psi,dicht4,time,mi(mijindx), &
                    agmat,mi(mitrajst),mi(migwpdep))
         endif
      endif

!-----------------------------------------------------------------------
! calculate energy etc for later output
!-----------------------------------------------------------------------
      if (dentype.eq.0) then
         call calcdat(psi,dtpsi,dicht2,mc(mchunpsi),spop, &
              ecorr,etot,euncorr,spoptr,time)
      else if (dentype.eq.1) then
         call d1calcdat(psi,dtpsi,mc(mchpsi),mc(mchunpsi),dicht3,spop,&
              firstcall,dicht2,detot2,ecorr,etot,trace)
      else if (dentype.eq.2) then
         call d2calcdat(psi,mc(mchunpsi),spop,firstcall,ecorr,etot,trace)
      endif

!-----------------------------------------------------------------------
! calculate grid population data, and write gridpop file if required
!-----------------------------------------------------------------------
      if (dentype.eq.0) then
         call dichtout(psi,dicht3,mr(mrgpop1), &
              mr(mrgpop2),mr(mrtrafo),mi(mifftfak),mc(mcrueck), &
              mc(mcexphin),mc(mcexprueck),mi(mijsph),mi(mimsph),time, &
              mr(mrort),.true.,agmat,mi(mijindx),mi(mitrajst))
      else if (dentype.eq.1) then
         call outgpop1(psi,mr(mrgpop1),mr(mrgpop2),mr(mrtrafo), &
              mi(mifftfak),mc(mchin),mc(mcrueck),mr(mrworkr), &
              time,trace)
      else if (dentype.eq.2) then
         call outgpop2(psi,mr(mrgpop1),mr(mrgpop2),mr(mrtrafo), &
              mi(mifftfak),mc(mchin),mc(mcrueck),time,mr(mrort), &
              dicht3,agmat,trace)
      endif

!-----------------------------------------------------------------------
! write out trace of rho(dlstate,drstate)
!-----------------------------------------------------------------------
      if (lctr) then 
         atime = time/fs
         if (dentype.eq.0) then
            message='ctrace not implemented for wavefunctions!'
            call errormsg
         else if (dentype.eq.1) then
            call d1traceout(psi,atime,firstcall)
         else if (dentype.eq.2) then
            call d2traceout(psi,agmat,atime,firstcall)
         endif
      endif

!-----------------------------------------------------------------------
! write data to output file 
! and get the norm (required in prop.F90 for normstop)
!-----------------------------------------------------------------------
! for density operators, calculate Hilbert-Schmidt norm (norm)
! and the total norm (normt).
!-----------------------------------------------------------------------
       call datout(time,spop,ecorr,etot,dicht3,dicht2, &
            mr(mrgpop1),mr(mrgpop2),mr(mrort),mr(mrfftp),mi(mijsph), &
            mi(mimsph),psi,qdq,gfgwp,aigwp,zgwp, &
            agmat,gs2,spoptr,ggp,normt,trace,etott)

!-----------------------------------------------------------------------
! for single-set calculations extract state populations from gpop
!-----------------------------------------------------------------------
      if (leb) then
         call cpvxd(mr(mrgpop1+zort(feb)-1),sspop,nddstate)
         sspop(:) = max(sspop(:),macheps)
      endif

!-----------------------------------------------------------------------
! write data to check file 
!-----------------------------------------------------------------------
      call wrchck(spop,sspop,ecorr,etot,dicht3,qdq,normt,ggp,trace)

!-----------------------------------------------------------------------
! ML-MCTDH specific output:
! - lnp (lowest natural populations)
!-----------------------------------------------------------------------
      if (lrunml) then
         lhead = firstcall.and.(.not.lcont)
         call outlnp(ilnp,time/fs,lhead)
      endif

!-----------------------------------------------------------------------
! write out autocorrelation function
!-----------------------------------------------------------------------
      if (lauto) then 
         atime = 2.0_dop*time/fs
         if (dentype.eq.0) then
            call autoout(psi,atime,mi(mijindx),agmat)
         else if (dentype.eq.1) then
            call d1autoout(psi,atime)
         else if (dentype.eq.2) then
            call d2autoout(psi,agmat,atime)
         endif
      endif
      if (lautoord1 .or. lautoord2) then
         atime = 2.0_dop*time/fs
         if (dentype.eq.0) then
            call autoouthigh(psi,dtpsi,atime)
         else
            message='order1/2 not implemented for density operators!'
            call errormsg 
         endif
      endif

!-----------------------------------------------------------------------
! write cross-correlation function
!-----------------------------------------------------------------------
      if (lcross) then
         if (dentype.eq.0) then
            call ccout(time/fs,psi)
         else if (dentype .eq. 1) then
            call d1ccout(time/fs,psi)
         else if (dentype .eq. 2) then
            call d2ccout(time/fs,psi)
         endif
      endif

!-----------------------------------------------------------------------
! write out expectation values
! If WP has norm > 1, report actual norm used for expectation
!-----------------------------------------------------------------------
      if (nexpect.gt.0) then
         norm1 = normt
         if (lnwexpect .and. initnorm .gt. 0.0) norm1 = norm1*initnorm
         if (lreonly) then
            write(iexpect,'(f10.3,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,norm1,(dble(expvalue(n)),n=1,nexpect)
         else
            write(iexpect,'(f10.3,1xx,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,norm1,(expvalue(n),n=1,nexpect)
         endif
         call myflush(iexpect)
      endif

      if (nexpect1.gt.0) then
         if (lreonly1) then
            write(iexpect1,'(f10.3,1x,f11.8,1x,90(1pe16.8,1x))') &
                  time/fs,normt,(dble(expvalue1(n)),n=1,nexpect1)
         else
            write(iexpect1,'(f10.3,1x,f11.8,1x,90(1p2e16.8,1x))') &
                  time/fs,normt,(expvalue1(n),n=1,nexpect1)
         endif
         call myflush(iexpect1)
      endif

!----------------------------------------------------------------------
! Write Local Control Stuff  to file
!----------------------------------------------------------------------
      if (lctvibpop) then
         filename=rname(1:rlaenge)//'/vibpop.lct'
         open(ilctvib,file=filename,form='formatted',status='unknown')
         write(ilctvib,99) time/fs,(vibpop(n),n=1,adim)
   99    format(f12.4,1x,90(1p2e16.8,1x))
      endif

      if (lcontrol) then
         write(icontrol,98) time/fs,vfield
         call myflush(icontrol)
   98    format(f12.4,6x,f17.14,15x,f16.6)

! New field if (lct_external)
         newtime=(time/fs)+1.0 !sort out time steps
         if (iofunc .eq. 1) then
            filename=rname(1:rlaenge)//'/rdfield.lct'
            open(ilct,file=filename,form='formatted',status='unknown')
   95       continue
            read(ilct,20) vfield,rftime
            if (abs((rftime-newtime)) .gt. 0.000001) then
              go to 95
            endif
            close(ilct)
         endif
      endif
   20 format(18x,f17.4,14x,f16.6)

!-----------------------------------------------------------------------
! write out time spent since last output
!-----------------------------------------------------------------------
      if(lspeed) call speedout(time)

!-----------------------------------------------------------------------
! write SVD data to log file for vMCG propagation
!-----------------------------------------------------------------------
      if (lgwp .and. ovrinv .eq. 2) then
         do s=1,nstate
            do m=1,nmode
               if (ngwpovr(m,s) .gt. 0) then
                  write(ilog,'(a,2i5,a,i5)') &
                     'Linear dependence mode, state ',m,s, &
                     '  Found : ',ngwpovr(m,s)
               endif
            enddo
         enddo
         ngwpovr(:,:) = 0
      endif

!-----------------------------------------------------------------------
! write trajectory data
!-----------------------------------------------------------------------
      if (lgentrj) then
         call wrtrjdat(time,psi(zpsi(1)),psi(zxi(1,1)),nspfdof(1), &
              nddstate,gwpdim(1,1),psi(zetf(meb,1)))
      endif 

!-----------------------------------------------------------------------
! write restart file
! Now written in propagatepsi
!-----------------------------------------------------------------------
!      linwf=.false.             ! 
!      idum=0                    ! just dummies
!      rdum(1)=0.0_dop             !      "
!      cdum(1)=(0.0_dop,0.0_dop)     !      "
!      call wrrst(time,psi,linwf,cdum,idum,rdum,rdum,mi(mijindx), &
!           agmat,mi(mitrajst))

      firstcall = .false.

!-----------------------------------------------------------------------
! write out timing statistics. 
! Stop and restart overall timing for propagation
!-----------------------------------------------------------------------
      call stop_timer(tid)
      call stop_timer(tpropwf)
      timers(tpropwf)%ncalls=1
      if (ltime) call timout
      call start_timer(tpropwf)

      return
      end subroutine output

!#######################################################################
!
! time written in fs, other quantities in au
!
!#######################################################################

      subroutine wrtrjdat(time,psi,xi,gdof,nsta,ntrj,elspf)

      implicit none

      integer(long), intent(in)     :: gdof,nsta,ntrj
      real(dop), intent(in)                          :: time
      complex(dop), dimension(ntrj,nsta), intent(in) :: psi
      complex(dop), dimension(gdof,ntrj), intent(in) :: xi
      complex(dop), dimension(nsta,nsta), intent(in) :: elspf

      integer(long)                 :: f,g,s,e,s1
      real(dop)                     :: norm
      real(dop)                     :: pot
      real(dop), dimension(maxsta,maxsta)    :: pesdia
      real(dop), dimension(maxsta)           :: pesad
      complex(dop), dimension(maxsta,maxsta) :: rotmatz
      real(dop), dimension(maxdim)       :: gpoint,ppoint
      integer(long), dimension(maxdim)   :: igpoint
      complex(dop)                       :: ztmp
      complex(dop), dimension(nsta,nsta) :: dens
      complex(dop), dimension(maxsta)    :: tmppsi,tmppsi1
      character(len=c5)             :: string

      do e=1,ntrj
         write(string,'(2a,i5,a5)') name(1:laenge),'/trjdat/',e,'.edat'
         call rmblnk(string)
         open(unit=idump, file=string,status='old',form='formatted', &
              position='append')

! get density matrix for trajectory e
         do s=1,nsta
            dens(s,s) = dconjg(psi(e,s))*psi(e,s)
            do s1=s+1,nsta
               dens(s1,s) = dconjg(psi(e,s1))*psi(e,s)
               dens(s,s1) = dconjg(dens(s1,s))
            enddo
         enddo

! sum potential terms
         pot = 0.0

! igpoint is the "grid point" - not used for GBFs
         igpoint = 1
         do g=1,gdof
            f=spfdof(g,1)
            gpoint(f) = dble(xi(g,e))
            ppoint(f) = dimag(xi(g,e))
         enddo
! add coordinates for frozen DOFs
         do f=1,ndof
            if (dofspf(f) .ne. 0) cycle
            gpoint(f) = rsbaspar(1,f,1)
            ppoint(f) = 0.0_dop
         enddo

     !    call calcpes(mr(mrhops),pesdia,igpoint,gpoint,pesdiaz,izflag,1)
         call calcdiab(mr(mrhops),pesad,pesdia,rotmatz,igpoint,gpoint,1)
         call sum2qxxdz(pesdia,dens,ztmp,nsta)
! need to multiply by ntrj due to normalisation of WF.
         pot=ntrj*dble(ztmp)

         write(idump,*) time/fs,pot
         close(idump)

         do f=1,ndof
            if (f .eq. feb) cycle
            write(string,'(2a,i5,a1,i5,a5)') name(1:laenge),'/trjdat/', &
                 f,'.',e,'.xdat'
            call rmblnk(string)
            open(unit=idump, file=string,status='old',form='formatted', &
                 position='append')
            !write(idump,*) time/fs,dble(xi(g,e))
            write(idump,*) time/fs,gpoint(f)
            close(idump)

            write(string,'(2a,i5,a1,i5,a5)') name(1:laenge),'/trjdat/', &
                 f,'.',e,'.pdat'
            call rmblnk(string)
            open(unit=idump, file=string,status='old',form='formatted', &
                 position='append')
            !write(idump,*) time/fs,dimag(xi(g,e))
            write(idump,*) time/fs,ppoint(f)
            close(idump)
         enddo

         norm=0.0_dop
         do s=1,nsta
            norm=norm+dble(dens(s,s))
            tmppsi(s) = psi(e,s)
         enddo
         norm = dsqrt(norm)
         tmppsi = tmppsi/norm
         call qvxxzz(elspf,tmppsi,tmppsi1,nsta)
     !    call qvxxzz(rotmatz,tmppsi1,tmppsi,maxsta)
     !    do s=1,nsta
     !       dens(s,s) = dconjg(tmppsi(s))*tmppsi(s)
     !       tmppsi1(s) = dsqrt(dble(dens(s,s)))*(cos(pesad(s)*time) - &
     !                              (0,1)*sin(pesad(s)*time))
     !  !     tmppsi1(s) = (cos(pesad(s)*time) - &
     !  !                            (0,1)*sin(pesad(s)*time))
     !    enddo

         do s=1,nsta
            write(string,'(2a,i5,a1,i5,a6)') name(1:laenge),'/trjdat/d', &
                 s-1,'.',e,'.cidat'
            call rmblnk(string)
            open(unit=idump, file=string,status='old',form='formatted', &
                 position='append')
            write(idump,*) time/fs,dble(tmppsi1(s)),dimag(tmppsi1(s))
            close(idump)
         enddo
      enddo

      return
      end subroutine wrtrjdat

!#######################################################################

      end module outputmod

