! **********************************************************************
!                                                                      
!                   module PROP
!                                                                      
!  MCTDH module controlling the propagation of the wavefunction,       
!  i. e. calling the required evaluation and integration routines.     
!                                                                      
! **********************************************************************
      module prop

use versions
use decimal, only: long, dop
use global
use dglobal
use globmemmod
use logdat

use d2lib, only: d2retrace
use dvrdatmod
use griddatmod
use psidef
use operdef
use datenmod
use runpropmod
use dirdyn
use channels
use openmpmod
use gapmod
use keyunitsmod

use iorst, only: wrrst,splitrst
use autocorr, only: autoout2,autoout2high
use d1autocorr, only: d1autoout2
use expectval, only: expectvalue
use function, only: funk,funkex,funk3,funk2,funkr,ddmfunk,ddmfunk2,ddmfunk3,&
                    ddmfunkex,ddmfunkr,funka
use densitymod
use eqofmotion1, only: compute_dmat_tree
use gwplib, only: gwp_param,transavec,transavec1,gwptophi,renormgwp,&
                  transahelp,gwp_overlap,renormdgwp,gwp_param_a,&
                  renormgwpa,rotcoeffgwp,renormgwptrja,renormgwpBEL
use gwptrjmod, only: gwptrj_param
use gwpdepdynmod, only: wrgwpdyn_final,setgwpdyn
use linint, only: li_update,li_backdate
use mempropmod, only: checkmemprop
use outputmod, only: output
use prop1, only: writeintmethod,propex,propvmfBEL,propvmf,propcmffix
use propcmfadamod, only: propcmfadarlx,propcmfada,propcmfadaphi,&
                         propcmfadaa,propcmfada2
use integrat, only: runsil, rundav
use wrintegrat, only: wrtrlxcdav,wrtrlxdav,wrtrlxbdav
use evaluate, only: calcenergy
use trafonatmod
use davutils, only: autoblock,sautoblock,csautoblock
use orbenmod, only: orben
use initorbmod, only: runinitorb
use getdiagonalmod, only: bgetdiagonal, psi2kry, kry2psi, postmpibgetdiag
use gwpoptimmod, only: gwpoptyp,gwpoptdyn
use multilayer, only: setpropml,logintml,normml
use gapfuncsmod, only: gapdrv
use psioutmod, only: psiout
use dd_db, only: gpdbupdate
use psiutil, only: renorma,spfovl,d1regular
use datoutmod, only: spawnmctdh,spawnprint
use wftree1, only: mlspwn_wftree
     
    use iopsidef, only : psidat
    use BEL, only : restoreMaxDim,restoreRedDim,renormalize

use xvlib,   only: xvixdzo,xvxxdzo
use op1lib,  only: cpvxzd, cpvxz, sum2vxz, normvxz, cpvxzd,cpmxi,zerovxi
use schmidtorthomod, only: schmidtortho, schmidtorthor
use timing
use operutil, only: getmaxeig

      implicit none
      private
      public :: propagatepsi,zeitstop
      public :: nextday,oldhour
      save
      integer(long)                                   :: nextday,oldhour
      contains

! **********************************************************************
! *** V6.2 MB *********> SUBROUTINE PROPAGATE PSI <*********************
!                       ~~~~~~~~~~~~~~~~~~~~~~~~~~
! Controls the propagation of the wavefunction.
! **********************************************************************

      subroutine propagatepsi (psi,dtpsi,Krylov,HKrylov,Kryc,HKryc)


      implicit none

      complex(dop), dimension(dgldim), intent(inout)  :: psi
      complex(dop), dimension(ikryc), intent(inout)   :: Kryc,HKryc
      real(dop), dimension(ikrylov), intent(inout)    :: Krylov,HKrylov
      complex(dop), dimension(dgldim), intent(inout)  :: dtpsi
      complex(dop)                                    :: cdum,cdum1,cnorm2
      complex(dop),dimension(1)                       :: cdm
      complex(dop), dimension(:), allocatable         :: overlap

      real(dop)                                  :: time,timesinceoutput, &
                                                    newupdatetime,timestep,&
                                                    remaintime,sumupdate,&
                                                    sum2update,atime,normi,&
                                                    normt,norm2,oldenergy,&
                                                    energy,trace
      real(dop)                                  :: dum,dum1,dum2,dum3,dum4,dum5
      real(dop), dimension(1)                    :: rdm
      real(dop)                                  :: val1,val2,val3,norm,minpop
      integer(long)                              :: abmsteps,repabmsteps,&
                                                    smallbssteps,goodbssteps,&
                                                    badbssteps,goodsilsteps,&
                                                    badsilsteps,updatesteps,&
                                                    badupdatesteps,badbackw,&
                                                    i,s,evaltype,pt,nfin,&
                                                    idm,m,imsym
      integer(long), dimension(maxdim)           :: idx
      logical(kind=4)                            :: restartabm,restartrk5,&
                                                    restartbs,&
                                                    restartbs2,restartsil,&
                                                    restartzvode, &
                                                    relaxev,linwf,lcheck,&
                                                    lcheck1s,lcheck1m,lpc,&
                                                    lf,lrecalc
      character(len=72)                          :: stopcommand

      integer                 :: spawnopt       ! spawn option
      real(dbl)               :: spawnf         ! spawn thresholds
      logical(kind=4), save   :: lhead


      integer(long)                              :: tid=0 ! timer ID

! --- CHECK ---
      if( tfinal .le. tinit + tiny ) then
         write(ilog,'(a)') '*******************************'
         write(iout,'(/a)') 'Nothing left to do!'
         write(ilog,'(/a)') 'Nothing left to do!'
         write(ilog,'(a,f15.6,a/,a,f15.6,a/)') &
         't-init =', tinit/fs,' fs','tfinal =',tfinal/fs,' fs'
         lstop = .false.
         goto 201
      end if

! --- Check that SPFs are real for RDAV and RRDAV.
      if(inttypea.eq.6 .and. lreal) then
         dum=0
         do i=zetf(1,1),dgldim
            dum = dum + abs(dimag(psi(i)))
         enddo
         if(dum.gt.1.d-14) then
           routine='propagatepsi'
           message='Initial SPFs are not real. RDAV requires real SPFs.'
           write(6,'(/a,1pe9.2)') ' Imaginary part of SPFs =',dum
           call errormsg
         endif
      endif
 
!-----------------------------------------------------------------------
! SPF Overlap needed for ML normalisation
!-----------------------------------------------------------------------
      if (lrunml .and. lrlx) allocate(overlap(dmatdim))

!-----------------------------------------------------------------------
!     Update tinit during dynamical SPFs (i.e. spawning)
!     In a standard propagation tinit=0.0, but during dynspf tinit must
!     be the time value when the number of SPFs was increased, which was
!     saved in tcurrent.
!     if idyncyc=0 this is the first step of the propagation, so nothing 
!     should be done.
!-----------------------------------------------------------------------
      if ((ldynspf).and.(idyncyc.gt.0)) tinit=tcurrent

! --- Check proj-h is used if GWPs and a CAP
      if (lgwp .and. lcap) then
         lfunkr=.true.
         lfunkl=.false.
      endif

! --- INITIALIZE VARIABLES ---
      time = tinit
      timesinceoutput = 0.0_dop
      abmsteps = 0
      repabmsteps = 0
      smallbssteps = 0
      goodbssteps = 0
      badbssteps = 0
      goodsilsteps = 0
      badsilsteps = 0
      restartabm = .true.
      restartrk5 = .true.
      restartbs = .true.
      restartbs2 = .true.
      restartsil = .true.
      restartzvode = .true.
      relaxev = .false.
      sumupdate = 0.0_dop
      sum2update = 0.0_dop
      updatesteps = 0
      badupdatesteps = 0
      badbackw = 0
      nfin = 0
      if(rlaxnum.ge.0 .and. lconst) relaxev = .true.
      dores = .false.

! Initialise local DBs
      if (ldd .and. ldbsmall) then
         call gwp_param(psi,agmat,gfgwp,aigwp,zgwp,mr(mrqcent),0)
         call gpdbupdate(mr(mrqcent))
      endif

      if (lBEL) then
        call restoreMaxDim(psi,agmat)
        call restoreRedDim(psi,agmat)
      endif

! --- Pass memory arrays to ML part
      if(lrunml) then
          call setpropml(1)  ! 1 = system Hamiltonian
! ---     for split CMF propagation, log integrator details now
          if (lconst.and.lmlcmfsplit) then
             call logintml(ilog)
             call myflush(ilog)
          endif
      end if

! --- IF REQUESTED, REORTHONORMALIZE ---
      if (limportho) then    
         call reorthopsi(psi,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)

         if(lrunml .and. lrlx) then
            call spfovl(overlap,psi,psi,agmat,agmat,0)
            call normml(psi,cnorm2,overlap)
            normi=1.d0/sqrt(dble(cnorm2))
            call xvxxdzo (normi,psi,adim)
         endif

         if (dentype .eq. 1) call d1regular(psi,.false.)
      endif

! --- For multipacket improved relaxation orthonormalize the A-vectors.
      if(mpb.gt.0 .and. relaxev) then
         lcheck=.false.
         imsym=0
         do i = 1, 2*maxdim
            if(mod(abs(msymmtr(i)),65536).eq.51200) then
               lcheck=.true.
               lf=.false.
               if(msymmtr(i).lt.0) lf=.true. ! (fermions)
               imsym=i
            endif
         enddo
         if(lautoblk.and.lreflex) then   ! start vectors for reflex
            nmode    = nmode-1      ! ignore packet mode
            adim     = adim/2       ! adjust adim
            maxblock = maxblock/2   ! adjust maxblock
            block(1) = block(1)/2   ! adjust block
            do m = 1, nmode         ! adjust ndim
               ndim(m,1) = ndim(m,1)/2
            enddo
            call funk2(time,psi)
            ! initialize mpi parallel bgetdiagonal
            ! postmpibgetdiag just posts an MPI-task for the
            ! slaves if MPI is defined. In the DAV drivers
            ! this is not done as the slaves are not in the
            ! waiting loop and bgetdiagonal is called directly
            call postmpibgetdiag
            call bgetdiagonal(mc(mcintpsi))
            call cpvxzd(mc(mcintpsi),mr(mrintr),adim/npackts)
            call autoblock(Psi,mr(mrintr),adim/npackts,adim,npackts,   &
                           rlxemin,mi(miworki),mr(mrworkr),nstate,     &
                           block,nmode,ilog,idx)
            call cpvxz(Psi,Psi(adim+1),adim)
            do m = 1, nmode        ! restore ndim
               ndim(m,1) = ndim(m,1)*2
            enddo
            adim     = adim*2      ! restore adim
            maxblock = maxblock*2  ! restore maxblock
            block(1) = block(1)*2  ! restore block
            nmode    = nmode+1     ! restore nmode

        elseif(lautoblk) then   ! automatic generation of start vectors
            call funk2(time,psi)
            ! initialize mpi parallel bgetdiagonal
            ! postmpibgetdiag just posts an MPI-task for the
            ! slaves if MPI is defined. In the DAV drivers
            ! this is not done as the slaves are not in the
            ! waiting loop and bgetdiagonal is called directly
            call postmpibgetdiag
            call bgetdiagonal(mc(mcintpsi))
            call cpvxzd(mc(mcintpsi),mr(mrintr),adim/npackts)
            if(lcheck) then
               if(lmult) then
                  routine='propagatepsi'
                  message='sautoblock (autoblock for (a)symmetrized'//  &
                  ' A-vectors) does not work with multi-set states.'
                  call errormsg
               endif
               if(lcap) then
                  routine='propagatepsi'
                  message='sautoblock (autoblock for (a)symmetrized'//  &
                  ' A-vectors) does not work with cDAV.'
                  call errormsg
               endif
               if(abs(msymmtr(imsym)).eq.51200) then
                  do i=imsym,2*maxdim-1  ! remove the imsym-th entrance
                     msymmtr(i)=msymmtr(i+1)
                  enddo
               else
                  msymmtr(imsym)=msymmtr(imsym) - sign(65536,msymmtr(imsym))
               endif

               if(lreal) then
                  call sautoblock(Psi,mr(mrintr),Krylov,adim/npackts,  &
                                  adim,npackts,nmode-1,rlxemin,        &
                                  mi(miworki),mr(mrworkr),lf,ilog,idx)
               else
                  call csautoblock(Psi,mr(mrintr),Kryc,adim/npackts,   &
                                  adim,npackts,nmode-1,rlxemin,        &
                                  mi(miworki),mr(mrworkr),lf,ilog,idx)
               endif

            else
               call autoblock(Psi,mr(mrintr),adim/npackts,adim,npackts,  &
                              rlxemin,mi(miworki),mr(mrworkr),nstate,    &
                              block,nmode,ilog,idx)
            endif
            call myflush(ilog)

         elseif(.not.lreflex) then
            if(lmult) then
               call psi2kry(Psi,Krylov,0)
               call schmidtorthor(Krylov,adim/npackts,npackts,i,dum)
               if(i.gt.0) then
                  routine='propagatepsi (prop.F:159) '
                  write(message,'(a,i5,9x)') 'schmidtorthor: ierr =', i
                  message=message(1:35)//'    -------------------------'  &
                      //'                          Initial A-vectors'//   &
                         ' of multi-packet run are linearly dependent!'
                  call errormsg
               endif
               call kry2psi(Psi,Krylov,0)
            else
               call schmidtortho (psi,adim/npackts,npackts,i)
               if(i.gt.0) then
                  routine='propagatepsi (prop.F:170)'
                  write(message,'(a,i5,9x)') 'schmidtortho: ierr =', i
                  message=message(1:34)//'    -------------------------'  &
                      //'                          Initial A-vectors'//   &
                         ' of multi-packet run are linearly dependent!'
                  call errormsg
               endif
            endif
         endif
      endif

!-----------------------------------------------------------------------
! If exact, set up "density matrices" (needed in output)
!-----------------------------------------------------------------------
      if (lexact) then
         call density(psi,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
      endif

! --- TRANSFORM TO NATURAL ORBITALS FOR CMF-MODE ---
      if (lnaturcmf) then
         call density(psi,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
         call trafonat(psi,tree,dicht4,mi(mijindx))
      endif

!-----------------------------------------------------------------------
! Optimise GWP starting positions to minismise Y-matrix
!-----------------------------------------------------------------------
      if (lgwp .and. lgwpoptyp) call gwpoptyp(psi)

!-----------------------------------------------------------------------
! Optimise GWP starting positions using a short propagation
!-----------------------------------------------------------------------
      if (lgwp .and. lgwpoptdyn .and. psitype .ne. -4) then
         call gwp_param(psi,agmat,gfgwp,aigwp,zgwp,mr(mrqcent),0)
         call gwp_overlap(gs2,gsinv,mr(mrgsval),gsvek,gfgwp,aigwp,zgwp,0)
         call density(psi,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)

         call gwpoptdyn(psi,dtpsi,gs2,out1,mi(migwpdyn),&
              mi(migwpdep))
      endif

!-----------------------------------------------------------------------
! ---- GWP modes: Check for linearly dependent functions
! and
! set up and find active "dynamically coupled" functions
!-----------------------------------------------------------------------
      if (lgwp) then
         if (psitype /= -4) then
            if (lgwptrj) then
               call gwptrj_param(m,time,psi,agmat,gfgwp,aigwp,zgwp,mr(mrqcent))
            else
               call gwp_param(psi,agmat,gfgwp,aigwp,zgwp,mr(mrqcent),0)
            endif
              if (lBEL) then
                call gwp_overlap(gs2,gsinv,mr(mrgsval), &
                     gsvek,gfgwp,aigwp,zgwp,1)
              else
                call gwp_overlap(gs2,gsinv,mr(mrgsval), &
                     gsvek,gfgwp,aigwp,zgwp,0)
              endif
! need to renormalise iMCG at start
            if ((lgentrj .or. gwpmodel .eq. 2) .and. .not. lcont) then
               call renormgwpa(psi,gs2)
            endif
            call density(psi,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)

            call setgwpdyn(psi,gs2,mi(migwpdyn),mi(migwpdep),lrecalc,&
                 gsinv,dicht2,time)
         endif
      endif

! --- CALCULATE EXPECTATION VALUE FOR t=t0 IF REQUIRED ---
      if (.not. lBEL) then
         do i=1,nexpect
            call expectvalue(psi,time,expectlab(i),expvalue(i), &
                 mr(mrhops),mc(mchpsi),hteil,&
                 mi(mijindx),mr(mrort),agmat,gfgwp, &
                 aigwp,zgwp,mr(mrqcent),gh,gs2, &
                 hung,mr(mrgsval),gsvek,ghderiv1)
         enddo
         do i=1,nexpect1
            call expectvalue(psi,time,expectlab(i),expvalue(i), &
                 mr(mrhops),mc(mchpsi),hteil,&
                 mi(mijindx),mr(mrort),agmat,gfgwp, &
                 aigwp,zgwp,mr(mrqcent),gh,gs2, &
                 hung,mr(mrgsval),gsvek,ghderiv1)
         enddo
      endif
!write(1000,*)"hteil after first expect",hteil(1),"source/propwf/prop.F90"

!-----------------------------------------------------------------------
! Deal with the machinery of calculating the GAP weights.
!-----------------------------------------------------------------------
      if (lddmctdh.or.lgausspot) &
            call gapdrv(time,psi,mr(mrort),(.not.lcont.or.lgapsobol))

!-----------------------------------------------------------------------
! Form optimal initial orbitals
!-----------------------------------------------------------------------
      if (linitorb) then
         call runinitorb(psi,dtpsi)
         linitorb = .false.
      endif

! turn on QC output concatenation
      if (lqcsaveout) lqcsaveall=.true.

! --- EVALUATE RIGHT HAND SIDE OF ODE WITH INITIAL WAVEFUNCTION ---
      
! Note that "funk2" followed by "funk3" is equivalent to "funkr" except

      if (lexact) then
! SOD and CHEB ARE EXACT METHODS SO THIS IS THE PART WHERE THE ROUTINE CALCULATES
! HPSI ETC. ALSO CALCULATES ESTIMATE OF MAX + MIN EIGENVALUE
         if (inttypephi .eq. 69 .or. inttypephi .eq. 70 .or. inttypephi .eq. 8) &
               call getmaxeig(mr(mrhops),mr(mrort),maxeigval,mineigval)
         if(lddmctdh)then
            call ddmfunkex(time,psi(zetf(1,1)),dtpsi(zetf(1,1)))
         else
            call funkex(time,psi(zetf(1,1)),dtpsi(zetf(1,1)))
         endif

      else if (relaxev) then      !    impoved relaxation
         evaltype = 5
         call funk2(time,psi)
         call funk3(time,psi,dtpsi)
         call calcenergy (psi,dtpsi,oldenergy,0)
         if (.not. lcont .and. .not.test) then
            if(lorben) then
               call orben(time,psi)
               call funk3(time,psi,dtpsi)
             endif
            call output(time,psi,dtpsi,normt,trace,energy)
            time = time + out2
            if(lskip1dav) goto 16
            if(inttypea .eq. 6) then
               call rundav(psi,dtpsi,updatetime,time,goodsilsteps,  &
                    badsilsteps,evaltype,Krylov,HKrylov,Kryc,HKryc)            
            elseif(inttypea .eq. 3) then
               call runsil(psi,dtpsi,updatetime,time,restartsil,    &
                    goodsilsteps,badsilsteps,evaltype)
! De-normalise A-coefficients in multi-pack run
               if (lmulpack .or. mpb.gt.0) then
                  norm = sqrt(dble(max(npackts,npacket)))
                  norm = 1.0_dop/norm
                  call xvixdzo(norm,psi,adim)
                  call xvixdzo(norm,dtpsi,adim)
               endif

            else
               routine='propagatepsi'
               message='Unknown inttypea (Line 228)'
               call errormsg
            endif
!......Transform to natural orbitals after first diagonalisation (tfinal=0).
            if (lnaturcmf .or. abs(tfinal).lt.tiny) then
               call density(psi,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
               call trafonat(psi,tree,dicht4,mi(mijindx))
               call funk2(time,psi)
            endif
            call funk3(time,psi,dtpsi)
   16       continue  ! if(lskip1dav) goto 16
         endif

      else if (.not. lconst .and. dentype .eq. 1) then
         call funkr(time,psi,dtpsi)
      else if (.not. lconst) then !    VMF integration
         if (lfunkl) then
            if(lddmctdh)then
               call ddmfunk(time,psi,dtpsi)
            else
               call funk(time,psi,dtpsi)
            endif
         else
            if(lddmctdh)then
               call ddmfunkr(time,psi,dtpsi)
            else
               call funkr(time,psi,dtpsi)
            endif
         endif
      else                       !     CMF integration
         if(lddmctdh)then
            call ddmfunk2(time,psi)
            call ddmfunk3(time,psi,dtpsi)
         else
            call funk2(time,psi)
            call funk3(time,psi,dtpsi)
         endif
      endif

! turn off QC output concatenation
      if (lqcsaveout) lqcsaveall=.false.

! --- Compute orbital energies and transform to energy-orbitals ---
      if(.not. lcont .and. lorben) then
         call orben(time,psi)
         call funk3(time,psi,dtpsi)
      endif

! --- WRITE INITIAL DATA AND WAVEFUNCTION ---
! 
      if (.not. lcont) then
         call output(time,psi,dtpsi,normt,trace,oldenergy)
      else
! for continuation runs normt needs to be set here for normstop
! to work correctly.
         normt = 1.0d0
      endif

! --- IA: in the OCT case, send the total number of timesteps  ----
! ---     via fifo to the efield routine signalling also       ----
! ---     that the restart file is available.                  ----
! ---     But first do a handshake.                            ----
      if (loptcntrl) then
       
         if (loctupdate .or. .not. lsimprop) then
            open(iefifo,file='efield.fifo',status='old')
            open(ipfifo,file='prop.fifo',status='old')
            nfin = nint((tfinal-tinit)/out2+0.1_dop)
            write(ipfifo,*) -nfin
            close(ipfifo)
            read(iefifo,*) idm  ! dummy read for synchronization
            open(ipfifo,file='prop.fifo',status='old')
            write(ipfifo,*) nfin, out1, out2
            
            close(ipfifo)
            if(idm.lt.0) read(iefifo,*) idm ! dummy read for sync.
            open(ipfifo,file='prop.fifo',status='old')
            i=nint(time/out2+0.1_dop)
!     ------ send the current timestep to the efield routine ------
            write(ipfifo,*) i
            close(ipfifo)

         else if( lsimprop .and. .not. loctupdate ) then

            open(ie1fifo,file='efield1.fifo',status='old')
            open(ip1fifo,file='prop1.fifo',status='old')
            nfin = nint((tfinal-tinit)/out2+0.1_dop)
            write(ip1fifo,*) -nfin
            close(ip1fifo)
            read(ie1fifo,*) idm  ! dummy read for synchronization
            open(ip1fifo,file='prop1.fifo',status='old')
            write(ip1fifo,*) nfin
            close(ip1fifo)
            if(idm.lt.0) read(ie1fifo,*) idm ! dummy read for sync.
            open(ip1fifo,file='prop1.fifo',status='old')
            i=nint(time/out2+0.1_dop)
!     ------ send the current timestep to the efield routine ------
            write(ip1fifo,*) i
            close(ip1fifo)

         end if


         if (loctupdate .or. .not. lsimprop) then
            read(iefifo,*) val1,val2,val3        
            call li_update(i,val1,val2,val3)
         else if( lsimprop .and. .not. loctupdate ) then
            read(ie1fifo,*) val1,val2,val3
!            call li_backdate(i,val1,val2,val3)
         endif 

      endif

!.....Stop if tfinal=0. One diagonalisation only.
      if(relaxev .and. abs(tfinal).lt.tiny) then
! ------ WRITE RESTART FILE
         linwf=.false.            !
         idm=0                    ! just dummies
         rdm(1)=0.0d0             !      "
         cdm(1)=(0.0d0,0.0d0)     !      "
         call wrrst(time,psi,linwf,cdm,idm,rdm,rdm,mi(mijindx), &
              agmat,mi(mitrajst))
         goto 200
      endif


! --- CHECK WHETHER RUN IS TEST RUN ---

      if (lqcsimulate .and. test) then
         write(iout,'(/,a,/)') 'Test run with simulation of QC was successful.'
         write(ilog,'(/,a,/)') '***** Test run *****'
         write(ilog,'(a,/)')   'Test run with simulation of QC completed.'
         goto 200
      else if (test) then
         write(iout,'(/,a,/)') 'Test run was successful.'
         write(ilog,'(/,a,/)') '***** Test run *****'
         write(ilog,'(a,/)')   'Test run completed.'
         goto 200
      endif


! --- SAVE PSI-VECTOR FOR COMPUTATION OF AUTOCORRELATION FUNCTION ---

      if (lautodoub) then
         if (lauto) call cpvxz(psi,mc(mcautopsi),dgldim)
         if (lautoord2) call cpvxz(dtpsi,mc(mcautodtpsi),dgldim)
         if (lautoe) then
            dicht4o(:) = dicht4(:)
            dicht3o(:) = dicht3(:)
         endif
      endif
      
      write(ilog,*)
      if(lrealphi) write(ilog,*)

! --- DETERMINE INITIAL E FOR IMPROVED RELAXATION AND CONTINUATION RUN
      if(relaxev .and. inttypea.eq.6 .and. lcont) then
         call calcenergy (psi,dtpsi,energy,0)
         i=-2
         dum=0.d0
         if(mpb.gt.0) then  ! single-set multi-packets run
            call wrtrlxbdav(time,intordera,i,i,energy,dum,dum1,   &
                            i,mr,mr,.true.,intordera,npackts)
         else
            call wrtrlxdav(time,intordera,i,i,energy,dum,dum1,    &
                           dum2,dum3,dum4,mr,mr,.true.,intordera)
         endif
      endif

!----------- OCT PC ----
      lpc=.false.
      if (loctpc) then
         call cpvxz(psi,mc(mcoctpsi),dgldim)
      endif

! --- PROPAGATION LOOP ---

! THIS IS WHERE THE PROPAGATION STARTS

 100  continue

!     --- check consistency of mc,mr,mi,ml arrays, 
#ifdef DEBUG
      call checkmemprop(.true.,0)
#endif

! --- Restart integrator if asked for

      if (dores) then
        restartabm=.true.
        restartrk5=.true.
        restartbs=.true.
        restartbs2=.true.
        restartsil=.true.
        restartzvode=.true.
        dores=.false.
      endif

! --- WRITE TIME TO INTSTEP FILE ---
! --- The stopcommand String is used when lstop is set true in integrat.F --
      stopcommand = 'stop  by internal check'
      if (lintstep.and..not.lmlcmfsplit) then
         if (rlxtime .eq. 'fs') then
            write(istp,'(a,f12.5)') 'Time in fs: ', time/fs
         else if (rlxtime .eq. 'au') then
            write(istp,'(a,f12.5)') 'Time in au: ', time
         endif
      endif

! --- PROPAGATE WAVEPACKET ---

      if (lexact) then
         call propex(psi,dtpsi,time,updatetime,timestep,restartabm,restartrk5,&
                     restartbs,restartsil,restartzvode,abmsteps,&
                     repabmsteps,goodbssteps,badbssteps,&
                     smallbssteps,goodsilsteps,badsilsteps)
      else if (relaxev .and. lcmfadaphi) then
         call propcmfadarlx(psi,dtpsi,time,timesinceoutput,&
                            newupdatetime,timestep,restartabm,restartrk5,&
                            restartbs,restartbs2,restartsil,restartzvode, &
                            abmsteps,repabmsteps,&
                            goodbssteps,badbssteps,smallbssteps,&
                            goodsilsteps,badsilsteps,sumupdate,&
                            sum2update,updatesteps,badupdatesteps,&
                            oldenergy,Krylov,HKrylov,Kryc,HKryc)

      else if (.not. lconst) then
         if ((lgwp) .and. (lBEL)) then
            call propvmfBEL(psi,dtpsi,time,out1,timestep,restartabm,&
                 restartrk5,restartbs,restartzvode, &
                 abmsteps,repabmsteps,goodbssteps,badbssteps,&
                 smallbssteps)
         else
            call propvmf(psi,dtpsi,time,out1,timestep,restartabm,restartrk5,&
                 restartbs,restartzvode, &
                 abmsteps,repabmsteps,goodbssteps,badbssteps,&
                 smallbssteps)
         end if
      elseif (lconstfix) then
         call propcmffix(psi,dtpsi,time,updatetime,timestep,restartabm,&
                         restartrk5,restartbs,restartbs2,restartsil,&
                         restartzvode,abmsteps,&
                         repabmsteps,goodbssteps,badbssteps,&
                         smallbssteps,goodsilsteps,badsilsteps,&
                         oldenergy,relaxev,Krylov,HKrylov, &
                         Kryc,HKryc)
! MLML
      elseif (lconst .and. lrunml) then
         call propcmfada2(psi,dtpsi,time,timesinceoutput,&
                          newupdatetime,timestep,restartabm,restartbs,&
                          restartsil,abmsteps,repabmsteps,&
                          goodbssteps,badbssteps,smallbssteps,&
                          goodsilsteps,badsilsteps,sumupdate,sum2update,&
                          updatesteps,badupdatesteps,restartrk5)
      elseif (lconstvar) then
         call propcmfada(psi,dtpsi,time,timesinceoutput, &
                         newupdatetime,timestep,restartabm,restartrk5,&
                         restartbs,restartbs2,restartsil,restartzvode, &
                         abmsteps,repabmsteps, &
                         goodbssteps,badbssteps,smallbssteps, &
                         goodsilsteps,badsilsteps,sumupdate,sum2update, &
                         updatesteps,badupdatesteps,badbackw, &
                         Krylov,HKrylov,Kryc,HKryc)
      elseif (lcmfadaphi) then
         call propcmfadaphi(psi,dtpsi,time,timesinceoutput, &
                            newupdatetime,timestep,restartabm,restartrk5,&
                            restartbs,restartbs2,restartsil,restartzvode, &
                            abmsteps,repabmsteps, &
                            goodbssteps,badbssteps,smallbssteps, &
                            goodsilsteps,badsilsteps,sumupdate, &
                            sum2update,updatesteps,badupdatesteps, &
                            Krylov,HKrylov,Kryc,HKryc)
      elseif (lcmfadaa) then
         call propcmfadaa(psi,dtpsi,time,timesinceoutput, &
                         newupdatetime,timestep,restartabm,restartrk5,&
                         restartbs,restartbs2,restartsil,restartzvode, &
                         abmsteps,repabmsteps, &
                         goodbssteps,badbssteps,smallbssteps, &
                         goodsilsteps,badsilsteps,sumupdate,sum2update, &
                         updatesteps,badupdatesteps,badbackw, &
                         Krylov,HKrylov,Kryc,HKryc)
      endif

!update local DBs
      if (ldd .and. ldbsmall) then
         call gwp_param(psi,agmat,gfgwp,aigwp,zgwp,mr(mrqcent),0)
         call gpdbupdate(mr(mrqcent))
      endif

! --- TAKE REAL PART OF WF FOR RDAV RUN ---
      if(lrealphi) then
         dum=0
         do i=zetf(1,1),dgldim
            dum = dum + abs(dimag(psi(i)))
            psi(i)=dble(psi(i))
         enddo
         backspace ilog
         write(ilog,'(a,e11.3,a,f12.3,/)') &
         'realphi: removed imaginary part =',dum,',  Time =',time/fs
         endfile ilog
         backspace ilog
      endif

! --- UPDATE TIME VARIABLES ---
! SOD - THIS IS WHERE THE TIME STEP IS TAKEN
      time = time+timestep
      timesinceoutput = timesinceoutput+timestep
      if(louteverystep) timesinceoutput = out1 

! --- TRANSFORM TO NATURAL ORBITALS FOR CMF-MODE ---
      if (lnaturcmf) then
         call density(psi,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)
         call trafonat(psi,tree,dicht4,mi(mijindx))
         call funk2(time,psi)
      endif

!-----------------------------------------------------------------------
! ---- GWP modes: Check for linearly dependent functions
! -Renormalise (only needed if scalar param is propagated)
! -Check for linearly dependent functions
! -flag active "dynamically coupled" functions
! -Rotate A-coefficients if local diabatisation
!-----------------------------------------------------------------------
      if (lBEL) then
           write(1000,*)"time",time/fs
            call renormgwpBEL(psi,gs2)
            call renormgwpBEL(psi,gs2)
!            call renormalize(psi,agmat)
!           call renormalize(psi,agmat)
      endif
      if (lgwp .and. psitype /= -4) then

         if (dddiab .eq. 2) then
            call rotcoeffgwp(time,psi,dgldim)
            call renormgwpa(psi,gs2)

         else if (lrenormgwpa .and. psitype .eq. 4) then
            if (gwpphase .ge. 0) call renormgwp(psi,agmat)
            if(lrlx)then
               call gwp_param_a(psi,agmat,gfgwp,aigwp,zgwp,mr(mrqcent),2)
            else
              call gwp_param(psi,agmat,gfgwp,aigwp,zgwp,mr(mrqcent),2)
            endif
            call gwp_overlap(gs2,gsinv,mr(mrgsval),gsvek,gfgwp,aigwp,zgwp,0)

            if (lgwptrj) then
               call renormgwptrja(psi,gs2,dim(1,1))

            else 
               call renormgwpa(psi,gs2)
            endif
            dores = .true.

         else if (lrenormgwpa .and. psitype .ne. 4) then
            call renormgwp(psi,agmat)
            call renormdgwp(psi,gs2,dicht4)
            if (.not. lcap) then
               call normvxz(psi,norm,adim)
               call xvixdzo(norm,psi,adim)
            endif
         endif

      else if(lrlx .and. .not.lrunml) then
         call renorma(psi)

      endif

! --- COMPUTE ORBITAL ENERGIES AND TRANSFORM TO ENERGY ORBITALS  ---
      if(lorben) then
         call orben(time,psi)
      endif

! --- IF REQUESTED, REORTHONORMALIZE SPFS ---
! Should recompute dicht matrices if A-vector has been renromaloased?
      if (limportho) then
         call reorthopsi(psi,tree,dicht1,dicht2,dicht3,dicht4,mi(mijindx),gs2)

         if(lrunml .and. lrlx) then
            call spfovl(overlap,psi,psi,agmat,agmat,0)
            call normml(psi,cnorm2,overlap)
            normi=1.d0/sqrt(dble(cnorm2))
            call xvxxdzo (normi,psi,adim)
         endif

         if (dentype .eq. 1) call d1regular(psi,.false.)
      endif

! --- WRITE RESTART FILE
      linwf=.false.            !
      idm=0                    ! just dummies
      rdm(1)=0.0d0             !      "
      cdm(1)=(0.0d0,0.0d0)     !      "
      if (lBEL) then
        call restoreMaxDim(psi,agmat)
!         dim(:,:)=dimmax(:,:)
!         call psidat
      endif
! for thermal relaxation need to record present temp in rst file
      if (lrlx .and. rlxtime .eq. 'K') then
         if (abs(time-0.0_dop) .lt. tiny) then
            temperature_psi = temperature0
         else if (dentype .ge. 1) then
            temperature_psi = 1.0_dop/(2.0_dop*time)
         else
            temperature_psi = 1.0_dop/time
         endif

         if (dentype .lt. 2) call renorma(psi)

      endif

! if at tout concatenate QC output if requested (set lqcsaveall flag)
      if ((abs(out1-timesinceoutput) .lt. tiny*out1) .and. lqcsaveout) lqcsaveall=.true.
! GAP DB/weights first if required. Moved before wrrst to avoid problem, on HPC, of running out of time
! during the gapdrv call. With the call to gapdrv after the call to wrrst, this can lead to an incomplete 
! output file as the continuation begins from the restart time, but the output data was not written at 
! that time during the prior run. Can happen with any propagation, but especially likely with GAP as 
! gapdrv can take a long time. GWR 01/23 
      if(lddmctdh.or.lgausspot) call gapdrv(time,psi,mr(mrort),.true.) 

      call wrrst(time,psi,linwf,cdm,idm,rdm,rdm,mi(mijindx), &
           agmat,mi(mitrajst))
      if (lBEL) then
        if (lpsi) then
        call psiout(psi,dicht4,time,mi(mijindx),agmat,mi(mitrajst),&
             mi(migwpdep))
        endif
        call restoreRedDim(psi,agmat)
!         dim(:,:)=dimred(:,:)
!         call psidat
      endif

! --- Switch off SVD regularization, if no longer required
      if(lsvd) then
         minpop=1.d0
         do s=1,nstate
            do m=1,nmode
               minpop=min(minpop,dicht3(d3mat(m,s)+dim(m,s)-1))
            enddo
         enddo
         if(minpop.gt.epsmat) then
            lsvd=.false.
            write(ilog,'(/a,f11.4)') &
            'SVD regularization switched off at time =',time/fs
            write(ilog,'(a,1p2e11.3//)') 'minpop, epsmat',minpop,epsmat
            call myflush(ilog)
         endif
       endif

! --- EVALUATE RIGHT HAND SIDE OF ODE ---

      if (lexact) then
         if(lddmctdh)then
            call ddmfunkex(time,psi(zetf(1,1)),dtpsi(zetf(1,1)))
         else
            call funkex(time,psi(zetf(1,1)),dtpsi(zetf(1,1)))
         endif
      else if (.not. lconst) then
         if (lfunkl) then
            if(lddmctdh) then
!               call gapdrv(time,psi,mr(mrort),.true.) ! GAP DB/weights first.
               call ddmfunk(time,psi,dtpsi)
            else
               call funk(time,psi,dtpsi)
            endif
         else
            if(lddmctdh) then
!               call gapdrv(time,psi,mr(mrort),.true.) ! GAP DB/weights first.
               call ddmfunkr(time,psi,dtpsi)
            else
               call funkr(time,psi,dtpsi)
            endif
         endif
      elseif(allcomplete) then
         call funka(time,psi,dtpsi)
      else
         if(.not.lrunml) then
            if(lddmctdh)then
               call ddmfunk3(time,psi,dtpsi) ! GAP DB/weights in propcmfada
            else
               if (lgwp) call funk2(time,psi)
               call funk3(time,psi,dtpsi)
            endif
         endif
      endif

! turn off QC output concatenation
      if (lqcsaveout) lqcsaveall=.false.

! --- CALL OUTPUT ROUTINES AND LOOK FOR STOP COMMAND ---
      if (abs(out1-timesinceoutput) .lt. tiny*out1) then
         timesinceoutput = 0.0_dop
         if (lautodoub) then
            call get_timer(tid, 'Autoout2')
            call start_timer(tid)
            if (lauto) then
               atime = (2.0_dop*time-out1)/fs
               if (dentype .eq. 0) then
                  call autoout2(mc(mcautopsi),psi,atime,mi(mijindx),agmat)
               else if (dentype .eq. 1) then
                  call d1autoout2(mc(mcautopsi),psi,atime)
               else if (dentype .eq. 2) then
                  routine = 'Prop.F90'
                  message = 'Cannot use auto=twice with density (II)'
                  call errormsg
               endif
            endif
            if (dentype .eq. 0 .and. (lautoord1 .or. lautoord2)) then
               atime = (2.0_dop*time-out1)/fs
               call autoout2high(mc(mcautopsi),mc(mcautodtpsi),psi,dtpsi,atime)
            endif
            if (lauto) call cpvxz(psi,mc(mcautopsi),dgldim)
            if (lautoord2) call cpvxz(dtpsi,mc(mcautodtpsi),dgldim)
            if (lautoe) then
               dicht4o(:) = dicht4(:)
               dicht3o(:) = dicht3(:)
            endif
            call stop_timer(tid)
         endif

! --- CALCULATE EXPECTATION VALUE FOR ACTUAL TIME, IF REQUIRED ---
         if (.not. lBEL) then
            do i=1,nexpect+nexpect1
            call expectvalue(psi,time,expectlab(i),expvalue(i), &
                 mr(mrhops),mc(mchpsi),hteil,&
                 mi(mijindx),mr(mrort),agmat,gfgwp, &
                 aigwp,zgwp,mr(mrqcent),gh,gs2, &
                 hung,mr(mrgsval),gsvek,ghderiv1)
            enddo
         endif

! --- WRITE DATA (call output)
         if(loctpc) then
            if(lpc) then
               call cpvxz(psi,mc(mcoctpsi),dgldim)
               call output(time,psi,dtpsi,normt,trace,energy)
               lpc=.false.
            else
               call cpvxz(mc(mcoctpsi),psi,dgldim)
               lpc=.true.
            endif                   ! (lpc)
         else
            call output(time,psi,dtpsi,normt,trace,energy)
         endif                      ! (loctpc)

!-----------------------------------------------------------------------
!DWF  If dynamical SPFs: monitor the natutal orbital populations
!DWF  Skip for Improved relaxation
!-----------------------------------------------------------------------
         if (ldynspf.and.(rlaxnum.eq.-1)) then
            if (tfinal-time.lt.tiny) then
! Switch-off dynamical WF if at end of propagation
               ldynspf=.false.

               if (ldebugspawn) call spawnprint(time)

            else
               lcheck=.false.
               if (lrunml) then
                  spawnopt=0
                  spawnf=2.0
                  call mlspwn_wftree(inspf,ispwn,time/fs,lhead,lnspf,&
                       lcheck,spawnopt,imaxspf,alphathr,&
                       maxtape,itape3,itape2,tape,tapeM)
               else
                  call spawnmctdh(time,dicht3,lcheck)
               endif
               if (lcheck) then
                  lstop=.true.
                  stopcommand='spawnstop'
               endif
            endif

         endif

! --- STOP IF NATURAL POPULATIONS EXCEED THRESHOLD ---
         if (lnatpopstop) then
            lcheck=.true.
            do s=1,nstate
               if (natpopstopstate.eq.0 .or. natpopstopstate.eq.s) then
                  lcheck1s=.false.
                  if (natpopstopmode.gt.0) then
                     m=natpopstopmode
                     call natpopstopcheck(lcheck1m,natpopstop,m,s,dicht3)
                     lcheck1s = lcheck1m
                  elseif (natpopstopmode.eq.0) then
                     lcheck1s=.true.
                     do m=1,nmode
                        call natpopstopcheck(lcheck1m,natpopstop,m,s,dicht3)
                        lcheck1s = lcheck1s.and.lcheck1m
                     enddo
                  elseif (natpopstopmode.eq.-1) then
                     lcheck1s=.false.
                     do m=1,nmode
                        call natpopstopcheck(lcheck1m,natpopstop,m,s,dicht3)
                        lcheck1s = lcheck1s.or.lcheck1m
                     enddo
                  endif
                  lcheck = lcheck.and.lcheck1s
               endif
            enddo
            if (lcheck) then
               lstop=.true.
               stopcommand='natpopstop'
            endif
         endif

! --- check for purity of density matrix during relaxation 
! In a relaxation calculation the density matrix should start and remain 
! in a pure state. However, the trace can drop below 1.0 on convergence
! so we need to check for this and stop if required.
! Also check the energy is not rising (convergence error)
         if (lrlx .and. dentype .gt. 0) then
            if (energy .gt. oldenergy) then
               lstop=.true.
               stopcommand='convergestop'
            endif
            oldenergy=energy
            if (dentype .gt. 0) then
               if (trace .lt. 0.9995d0) then
                  lstop=.true.
                  stopcommand='puritystop'
               endif
            endif
         endif

!------- IA : OCT with FIFO -----------------------------------
         if (loptcntrl) then

!     --- if we are in parallel mode only recieve field
!     --- if we are the instance running with the
!     --- new field. Otherwise also send psi to efield routine 
            if (loctupdate .or. .not. lsimprop ) then
               
               open(ipfifo,file='prop.fifo',status='old')
               i=nint(time/out2+0.1_dop)
! ------ send the current timestep to the efield routine ------
               write(ipfifo,*) i
               close(ipfifo)
            else if( lsimprop .and. .not. loctupdate ) then
               open(ip1fifo,file='prop1.fifo',status='old')
               i=nint(time/out2+0.1_dop)
! ------ send the current timestep to the efield routine ------
               write(ip1fifo,*) i
               close(ip1fifo)
            end if

! ------ and wait for the efield to finish --------------------
            if (loctupdate .or. .not. lsimprop) then
               read(iefifo,*) val1,val2,val3
               call li_update(i,val1,val2,val3)

            else if (lsimprop .and. .not. loctupdate) then
!              This in the data from the last propagation
!              is transmitted again to have the same propagation               
               read(ie1fifo,*) val1,val2,val3
               call li_backdate(i,val1,val2,val3)    
               
            endif
           
            if(lpc) time=time-out2
            if (lfunkl) then
               call funk(time,psi,dtpsi)
            else
               call funkr(time,psi,dtpsi)
            endif
            if(lpc) goto 100    ! Do the oct-pc corrector step 

         endif
            
!........dummy write to ilog in order to touch the log-file.
!        (do not write if psi=compact)
         if (.not. lpsiopt(2)) then
!           backspace ilog
            write(ilog,'(a,f12.3)') ' time[fs] ', time/fs
!           endfile ilog
!           backspace ilog
         endif

!........flush data
         if(lupdate) call myflush(iupd)
         if(lspeed)  call myflush(ispeed)
         if(lcross)  call myflush(icroco)
         if(lpdens)  call myflush(ipden)
         if(lorben)  call myflush(iorb)
         if(nexpect.gt.0) call myflush(iexpect)
         if(nexpect1.gt.0)call myflush(iexpect1)
         if(rlaxnum.ge.0) call myflush(irlx)

! --- STOP IF REQUIRED BY STOP FILE  ---
         if(lstopf.and.time.lt.(tfinal-0.1_dop*out1).and. .not.lstop) then
            open(istop,file=name(1:laenge)//'/stop', &
            form='formatted',status='old',err=666)
            read(istop,'(a)') stopcommand
!...........Go to the end of the stop-file and write the propagation
!...........time to it in order to 'touch' the stop-file.
            do i = 1, 26
                read(istop,'(a)') 
            end do
            write(istop,'(f12.3)') time/fs
            close(istop)
            goto 667
! --- If the stop-file cannot be opened stop the calculation ---
 666        stopcommand = 'stop      Cannot open stop-file'  
 667        if( stopcommand(1:4) .eq. 'cont' ) then
               nextday = 0
               oldhour = 0
            else if( stopcommand(1:4) .eq. 'stop' ) then
               lstop = .true.
            else if( stopcommand(4:4).eq.':' .or.  &
                     stopcommand(12:12).eq.':' .or. &
                     stopcommand(23:23).eq.':' ) then
               call zeitstop(stopcommand)
            end if
            if(lstop) ldynspf=.false.
         end if
      endif

! --- STOP IF NORM IS SMALLER THAN nrmstop  ---
! normt was calculated in output
      if (lnormstop) then
         norm2 = normt*normt
         if (norm2 .lt. nrmstop) then
            lstop=.true.
            ldynspf=.false.
            stopcommand='normstop'
         endif
      endif

! --- STOP IF ENERGY DIFFERENCE IS SMALLER THAN constop (impr.relax.)  ---
      if(e1stop+e2stop .lt. constop .and. e1stop .le. e2stop) then
         if (ldynspf.and.(.not.lrunml)) then
!           If dynamical SPFs: monitor the natutal orbital populations
            lcheck = .false.
            call spawnmctdh(tcurrent,dicht3,lcheck)
            if (.not.lcheck) then
!              Stop relaxation
               ldynspf=.false.
               lstop=.true.
               stopcommand='converged'
            endif
         else
            lstop=.true.
            stopcommand='converged'
         endif
      endif
      e2stop=e1stop
 
! --- DEFINE NEW UPDATE-TIME ---
      if (lconstvar .or. lcmfadaphi .or. lcmfadaa) then
         if(louteverystep) timesinceoutput =  &
                       time - out1*int(time/out1+tiny)
         remaintime=out1-timesinceoutput
         if(relaxev) then
            newupdatetime=min(newupdatetime,3.0_dop*updatetime,remaintime)
! --- The following two lines may help to avoid failed update steps
! --- Enlargements are restricted to a factor of 1.6, but if
! --- updatetime=0.5*out1 an enlargement by a factor of 2 must be allowed.
         elseif (updatetime .lt. 0.49d0*out1) then
            newupdatetime=min(newupdatetime,1.6d0*updatetime,remaintime)
         else
            newupdatetime=min(newupdatetime,2.1_dop*updatetime,remaintime)
         endif
         updatetime=remaintime/int(remaintime/newupdatetime+0.98_dop)

         call stop_timer(tpropwf)
         timers(tpropwf)%ncalls=1
         if(ltime) call timout
         call start_timer(tpropwf)
      endif

! --- CONTINUE PROPAGATION ---
      if (((tfinal-time).gt.tiny*abs(tfinal)) .and. .not.lstop) goto 100


!-----------------------------------------------------------------------
! If Dyn WF save time as current time.
!-----------------------------------------------------------------------
      if (ldynspf) then
!        First spawning call has taken place
         lspwnfcall=.true.
!        Save tinit during dynamical SPFs (i.e. spawning)
         if (lrlx .and. rlaxnum.ge.0) then
!           Improved relaxation
            tcurrent=time
         else
!           Propagation or imaginary-time relaxation
            if (.not. lconst) then
!              VMF
               tcurrent=time
            else
!              CMF
               if (timesinceoutput.eq.0.0) then
!                 time is t=tlast+tout
                  tcurrent=time
               else
!                 time is t=tlast+updatetime
                  tcurrent=time-timestep+out1
               endif
            endif
         endif

         go to 200  ! go to end

      endif

!-----------------------------------------------------------------------
! Finish propagation
!-----------------------------------------------------------------------
! --- send the final (negative) timestep to stop the efield routine ----
      if (loptcntrl) then
         if (loctupdate .or. .not. lsimprop ) then
            open(ipfifo,file='prop.fifo',status='old')
            i=-nint(time/out2+0.1_dop)
            write(ipfifo,*) i
            close(ipfifo)
         else if( lsimprop .and. .not. loctupdate ) then
            open(ip1fifo,file='prop1.fifo',status='old')
            i=-nint(time/out2+0.1_dop)
            write(ip1fifo,*) i
            close(ip1fifo)
         endif
      endif
    
! --- WRITE FINAL A-ENERGY TO RLX_INFO FILE ---
      if(relaxev) then
         call calcenergy (psi,dtpsi,energy,0)
         if(inttypea.eq.6) then ! DAV
            write(irlx,'(a,f10.3,f18.10," eV,",f16.10," meV")') &
                 'After Davidson   T =',time/fs,energy*ev, &
                  (energy-oldenergy)*ev*1.0e+3_dop
            s=0
            i=-1 
            dum=0.0_dop
            if(lreal .and. mpb.gt.0) then  ! BDAV
               pt = mrintr + (intordera+1)*(intordera+1)
               call wrtrlxbdav(time,intordera,i,i,dum,dum1,dum2,i,  &
                    mr(pt),mr(pt+intordera+1),.true.,intordera,npackts)
            elseif(lreal) then             ! RDAV
               pt = mrintr + (intordera+1)*(intordera+1)
               call wrtrlxdav(time,intordera,i,s,dum,dum1,dum2,dum3,dum4, &
                      dum5,mr(pt),mr(pt+intordera+1),.true.,intordera)
            elseif(.not.lcap) then      ! DAV
               call wrtrlxdav(time,intordera,i,s,dum,dum1,dum2,dum3,dum4, &
               dum5,mr(mrintr),mr(mrintr+intordera+1),.true.,intordera)
            elseif(lcap) then           ! CDAV 
               cdum=0.0_dop
               pt = mcworkc+16*intordera+18+(intordera+1)*(intordera+1)
               call wrtrlxcdav(time,intordera,i,s,cdum,cdum1,dum,dum1, &
                         dum2,dum3,mc(pt),mr(mrintr),.true.,intordera)
            endif
         else
            write(irlx,'(a,f10.3,f18.10," eV,",f16.10," meV")') &
                 'After  Lanczos   T =',time/fs,-energy*ev, &
                  (oldenergy-energy)*ev*1.0e+3_dop
         endif
         call myflush(irlx)
      endif

! --- WRITE TIME TO INTSTEP FILE ---
      if (lintstep.and..not.lmlcmfsplit) then
         if (rlxtime .eq. 'fs') then
            write(istp,'(a,f12.5)') 'Time in fs: ', time/fs
         else if (rlxtime .eq. 'au') then
            write(istp,'(a,f12.5)') 'Time in au: ', time
         endif
      endif

! --- WRITE INTEGRATION METHOD TO LOG FILE ---
      call writeintmethod(abmsteps,repabmsteps,smallbssteps,goodbssteps, &
                          badbssteps,goodsilsteps,badsilsteps, &
                          sumupdate,sum2update,updatesteps, &
                          badupdatesteps,badbackw)

! --- WRITE STATISTICS ON DYNAMIC GWP BASIS TO LOG FILE ---
      if (lgwp) call wrgwpdyn_final

!! --- Optimisation of GAP parameters if required.
!     call gapdrv(time,psi,mr(mrort),.false.)

! --- WRITE FINAL MESSAGES TO FILES ---
  201 continue
      if (.not. ldynspf) then
         write(iout,*)
         if (.not. lstop) then
            write(iout,'(/,a)') 'Propagation was successful.'
         else
            write(iout,'(a)') 'Propagation was stopped.'
            write(iout,'(a)') 'The stop-command reads:'
            write(iout,'(a)')  trim(stopcommand)
            write(iout,*)

            write(ilog,'(a)') 'Propagation was stopped.'
            write(ilog,'(a)') 'The stop-command reads:'
            write(ilog,'(a)')  trim(stopcommand)
            write(ilog,*)
         endif
      endif
      write(iout,*)
      call myflush(iout)

 200  continue

! --- For block-Davidson split the RESTART file ---
      if(lsplitrst .and. relaxev .and. mpb.gt.0) then
         i=dgldim
         call splitrst(psi,dtpsi,i,time,mi(mijindx),agmat,mi(mitrajst))
      endif

      if (loptcntrl) then
! --- wait for efield to finish (dummy read for synchronization).
         if (loctupdate .or. .not. lsimprop) then
            read(iefifo,*) irst 
            close(iefifo)   
         else if ( lsimprop .and. .not. loctupdate ) then
            read(ie1fifo,*) irst 
            close(ie1fifo)   
         endif
      endif

! deallocate arrays needed for symmetrisation of GWP basis
!      if(ldd.and.ddcoo.eq.1)then
!         if(allocated(agmattmp))deallocate(agmattmp)
!         if(allocated(gfgwptmp))deallocate(gfgwptmp)
!         if(allocated(aigwptmp))deallocate(aigwptmp)
!         if(allocated(zgwptmp))deallocate(zgwptmp)
!         if(allocated(qcenttmp))deallocate(qcenttmp)
!      endif

      if(lddmctdh .or. lgausspot) call dealloc_gapmod

      return
      end subroutine propagatepsi

!######################################################################
! *** 06/97  HDM ********> SUBROUTINE ZEITSTOP <***********************
!                         ~~~~~~~~~~~~~~~~~~~~~

! Compares the time read from the stopfile with actual time
! Calls fzeit which is defined in showtime.c (lib).
!######################################################################

      subroutine zeitstop(stopcommand)

      implicit none

      external                      :: fzeit
      real(dop), external           :: fcputime

      integer(long)                 :: shour,smin,ssec,stime,ahour,amin,atime,&
                                       i,ierr1,ierr2,ierr3
      character(len=72), intent(in) :: stopcommand
      character(len=80)             :: actual

      if( stopcommand(1:17) .eq. '                 ' ) goto 300
      if( stopcommand(1:6) .eq. '     ' ) goto  200

!.....Realtime stop condition
      read(stopcommand(1:3),*,iostat=ierr1) shour
      read(stopcommand(5:6),*,iostat=ierr2) smin
      if( ierr1 .ne. 0  .or.  ierr2 .ne. 0 )  then
         routine = 'zeitstop (propagation.f)'
         call warnmsg
         return
      end if
      stime = 60*shour + smin

      call fzeit(actual)
      i = index(actual,':')
      read(actual(i-2:i-1),*) ahour
      read(actual(i+1:i+2),*) amin
      if( ahour .lt. oldhour ) nextday = nextday + 24
      oldhour = ahour
      atime =  60*(ahour+nextday) + amin

      if( atime .ge. stime ) then
         lstop = .true.
         return
      end if

!.....CPU-time stop condition
 200  continue
      if( stopcommand(12:12) .ne. ':' ) return
      read(stopcommand(9:11),*,iostat=ierr1) shour
      read(stopcommand(13:14),*,iostat=ierr2) smin
      read(stopcommand(16:17),*,iostat=ierr3) ssec
      if(ierr1.ne.0 .or. ierr2.ne.0 .or. ierr3.ne.0)  then
         routine = 'zeitstop (propwf/prop.F)'
         message = &
       'Can''t read stopcommand. Not a proper CPU time-descriptor'
         call warnmsg
         return
      end if

      stime = 3600*shour + 60*smin + ssec
      atime = int( fcputime(0) + 0.5_dop )
      if( atime .ge. stime )  lstop = .true.

!.....Wall-time stop condition
 300  continue
      if( stopcommand(23:23) .ne. ':' ) return
      read(stopcommand(20:22),*,iostat=ierr1) shour
      read(stopcommand(24:25),*,iostat=ierr2) smin
      read(stopcommand(27:28),*,iostat=ierr3) ssec
      if(ierr1.ne.0 .or. ierr2.ne.0 .or. ierr3.ne.0)  then
         routine = 'zeitstop (propwf/prop.F)'
         message = &
              'Can''t read stopcommand. Not a proper wall-time descriptor'
         call warnmsg
         return
      end if

      call unixzeit(atime)
      stime = 3600*shour + 60*smin + ssec
      atime = atime - tm_zero
      if( atime .ge. stime )  lstop = .true.

      return
      end subroutine zeitstop

!***********************************************************************

      subroutine natpopstopcheck(lchk,thrshld,m,s,dicht3)

      implicit none

      logical(kind=4), intent(out)               :: lchk
      integer(long), intent(in)                  :: m,s
      real(dop), intent(in)                      :: thrshld
      real(dop), dimension(d3matdim), intent(in) :: dicht3

      lchk = ( dicht3(d3mat(m,s)+dim(m,s)-1) .gt. thrshld )

      return
      end subroutine natpopstopcheck ! of natpopstopcheck


!######################################################################

      end module prop

