!#######################################################################

            module prop1

      use decimal, only: long, dop
      use global
      use maxv, only: maxdim,maxsta
      use globmemmod
      use runpropmod
      use logdat
      use mmlib,   only: trhhxtzz
      use xvlib,   only: xvixdzo
      use op1lib,  only: cpvxz, normvxz, trhxz, overmxz, initvxl
      use op2lib,  only: submxxz, addmxxzo, subvxxzo1
      use rklib

      use channels
      use griddatmod
      use operdef
      use psidef
      use integrator
      use integrat
      use multilayer
      use function
      use wftrafo, only: renorm_tree
      use psiutil, only: d1regular
      use evaluate, only: calcenergy
      use BEL, only : resetGwp,restoreMaxDim,renormalize,checknorm
      use iopsidef, only : psidat
      use BELglobals, only : ifunkr,agmatdimmax,dgldimmax,errorBELprop
      use gwplib, only: transahelp

      use mpidata
     
      implicit none
      private
      public :: propvmfBEL,propvmf,propex,propa,propphi,propcmffix,&
                errora,errorphi,errorphi1,writeintmethod,propphiml,&
                errortot
      contains

!#######################################################################
! *** V6.2 MB ***********> SUBROUTINE PROP VMF <************************
! Drives the propagation of the wavefunction in variable mean-field
! (VMF) mode.
!#######################################################################

      subroutine propvmfBEL (psi,dtpsi,time,trystep,madestep,restartabm, &
                 restartrk5,restartbs,restartzvode,steps,repabmsteps,&
                 goodbssteps,badbssteps,smallbssteps)

      implicit none

      complex(dop), dimension(dgldimmax), intent(inout) :: psi,dtpsi
      real(dop)                                      :: intperiod
      real(dop), intent(out)                         :: madestep
      real(dop), intent(in)                          :: trystep,time
      integer(long), intent(inout)                   :: steps,repabmsteps,&
                                                        goodbssteps,badbssteps,&
                                                        smallbssteps
      logical(kind=4), intent(inout)                 :: restartabm,restartbs,&
                                                        restartrk5,restartzvode

      real(dop),save                                 :: itime,atime,etime,&
                                                        rtime,quality
      real(dop),save                                 :: lastrtime,lastrRtime,&
                                                        BELstep=0.0_dop,&
                                                        cstep=0.0_dop

      complex(dop), dimension(:),allocatable,save    :: psiold,aold,psiRold,&
                                                        aRold,psinew,anew
      integer(long), dimension(:,:), allocatable,save :: dimold,dimRold,dimnew,&
                                                         dimredold,dimrednew
      integer(long), dimension(:,:), allocatable,save :: gwpdimold,gwpdimRold,&
                                                         gwpdimnew
      integer(long),save :: iflag=0 !TM
      integer(long),save :: ierr=0 !TM
      real(dop),save                                 :: BELorgtime=0.0_dop
      logical :: lrepeat

      if (BELstep==0.0_dop) then
         BELstep=abs(BELint)
         BELorgtime=abs(BELint)
         itime=time
         etime=time+trystep
         atime=time
      else
         etime=etime+trystep
         itime=atime
      end if

      if (allocated(psinew)) then
        if (size(psinew) < dgldimmax) deallocate(psinew)
      end if
      if (.not. (allocated(psinew))) then
         allocate(psinew(dgldimmax))
      end if
      if (allocated(anew)) then
        if (size(anew) < agmatdimmax) deallocate(anew)
      end if
      if (.not. (allocated(anew))) then
         allocate(anew(agmatdimmax))
      end if
      if (allocated(psiold)) then
        if (size(psiold) < dgldimmax) deallocate(psiold)
      end if
      if (.not. (allocated(psiold))) then
         allocate(psiold(dgldimmax))
         lastrtime=-1.0_dop
      end if
      if (allocated(aold)) then
        if (size(aold) < agmatdimmax) deallocate(aold)
      end if
      if (.not. (allocated(aold))) then
         allocate(aold(agmatdimmax))
         lastrtime=-1.0_dop
      end if
      if (allocated(psiRold)) then
        if (size(psiRold) < dgldimmax) deallocate(psiRold)
      end if
      if (.not. (allocated(psiRold))) then
         allocate(psiRold(dgldimmax))
         lastRrtime=-1.0_dop
      end if
      if (allocated(aRold)) then
        if (size(aRold) < agmatdimmax) deallocate(aRold)
      end if
      if (.not. (allocated(aRold))) then
         allocate(aRold(agmatdimmax))
         lastRrtime=-1.0_dop
      end if
      if (.not. allocated(dimold)) allocate(dimold(maxdim,maxsta))
      if (.not. allocated(dimrold)) allocate(dimrold(maxdim,maxsta))
      if (.not. allocated(dimnew)) allocate(dimnew(maxdim,maxsta))
      if (.not. allocated(gwpdimold)) allocate(gwpdimold(maxdim,maxsta))
      if (.not. allocated(gwpdimrold)) allocate(gwpdimrold(maxdim,maxsta))
      if (.not. allocated(gwpdimnew)) allocate(gwpdimnew(maxdim,maxsta))
      if (.not. allocated(dimredold)) allocate(dimredold(maxdim,maxsta))
      if (.not. allocated(dimrednew)) allocate(dimrednew(maxdim,maxsta))

!      if (atime/fs == 2.0d-3) then
!         BELint=0.5d0*abs(BELint)
!         BELorgtime=abs(BELint)
!      endif
      do while (atime < etime-tiny)
         intperiod = min(etime-atime,abs(BELint))
!         intperiod = (etime-atime)
!write(*,*)'etime-atime',etime/fs-atime/fs
!write(*,*)'BELint',BELint/fs
!write(*,*)'intperiod',intperiod/fs

         if (lastrtime == -1.0_dop) then
            ! find the largest reset time smaller than the end of this step
            rtime=ceiling((atime-BELoff)/abs(BELint)-tiny)*abs(BELint)+BELoff
            do while ((rtime+tiny<BELoff))
               rtime=rtime+abs(BELint)
            end do
            ! if BELint<0, the zero reset doesn't count -> shift back one step
            if ((abs(rtime)<tiny) .and. (BELint<0.0_dop)) then
               rtime=rtime+abs(BELint)
            end if
         else
            rtime=lastrtime+abs(BELint)
         end if
         ! ! if there is time left before the reset, do a step
         ! if ((rtime-atime>tiny) .and. (rtime+tiny<etime)) then
         !    intperiod=rtime-atime
         !    call propvmf (psi,dtpsi,atime,rtime-atime,madestep,restartabm, &
         !         restartrk5,restartbs,restartzvode,steps,repabmsteps,&
         !         goodbssteps,badbssteps,smallbssteps,mc,mr,mi,ml)
         !    atime=atime+madestep
         !    intperiod = min(etime-atime,abs(BELint))
         ! end if

         ! if there is a reset "now", do it
!write(*,*)'rtime',rtime/fs
!write(*,*)'tiny',tiny/fs
!write(*,*)'atime',atime/fs
!write(*,*)'lastrtime',lastrtime/fs
         lrepeat=.false.
! 888 continue allows repeat of step from below
888 continue
         if ((rtime-tiny<atime) .and. (rtime-tiny>lastrtime)) then
!         if ((rtime-tiny<atime) .and. (rtime-tiny>lastrtime)  &
!            .or. iflag==1 ) then !TM
            if (.not. lrepeat) then
               if (cstep/=0.0_dop) then
                  BELint=sign(cstep,BELint)
               end if
               psinew(1:dgldim)=psi(1:dgldim)
               anew(1:agmatdim)=agmat(1:agmatdim)
               dimnew(:,:)=dim(:,:)
               gwpdimnew(:,:)=gwpdim(:,:)
               dimrednew(:,:)=dimred(:,:)
            endif 
!write(*,*)'1',atime/fs
! 888 continue was here
            errorBELprop=0
            call resetGwp(atime,psi,agmat,quality,iflag,ierr)
            iflag=0
            ierr=0
            if (quality>abs(BELflags(1))*1.0_dop) stop "abs(BELflags(1))*1.0_dop"
!write(*,*)'1',atime
!write(*,*)' '
        !if (atime>1.0e-6) stop 'R'

            ! tolerate a little transgression past maximum deviation (factor 2)
            ! but beyond that go a step back and reduce the step size
            if (BELflags(1)>0) then

               if ((quality>abs(BELflags(1))*1.0_dop) .and. (lastrtime /= -1.0_dop) .and. &
!               if ((quality>abs(BELflags(1))*2.0_dop) .and. (lastrtime /= -1.0_dop) .and. &
!               if ((quality>abs(BELflags(1))*1.5_dop) .and. (lastrtime /= -1.0_dop) .and. &
                    (abs(BELint)>0.01*BELstep) .and. (cstep== 0.0_dop)) then

!               if ((quality>abs(BELflags(1))*1.0_dop) .and. (lastrtime /= -1.0_dop) ) then
                  if ((lastRrtime /= -1.0_dop) .and. (abs(BELint)<0.1*BELstep) .and. &
!                  if ((lastRrtime /= -1.0_dop)  .and. &
                       (lastRrtime /= lastrtime)) then
                     lastrtime=lastRrtime
                     dimold(:,:)=dimRold(:,:)
                     gwpdimold(:,:)=gwpdimRold(:,:)
                     psiold(:)=psiRold(:)
                     aold(:)=aRold(:)
                  end if
                  atime=lastrtime
!                   BELint=BELint/1.001_dop !TM
!                  BELint=BELint/(1.1_dop**4 )
!                  BELint=BELint/(1.4_dop )
!                  BELint=BELint/(1.1_dop )
!                  BELint=BELint/(2.2_dop**2 )
                  BELint=BELint/(3_dop )
        !          write(*,*)'BELint',BELint,quality
                  dim(:,:)=dimold(:,:)
                  gwpdim(:,:)=gwpdimold(:,:)
!                  write(*,*)"call psidat"
                  call psidat
                  psi(1:dgldim)=psiold(1:dgldim)
                  agmat(1:agmatdim)=aold(1:agmatdim)
               else
                  if (quality>abs(BELflags(1))*1.0_dop) then
!                  if (quality>abs(BELflags(1))*2.0_dop) then
!                  if (quality>abs(BELflags(1))*1.5_dop) then
                     dim(:,:)=dimnew(:,:)
                     gwpdim(:,:)=gwpdimnew(:,:)
!                     call psidat
!                  write(*,*)"call psidat"
                     call psidat
                     psi(1:dgldim)=psinew(1:dgldim)
                     agmat(1:agmatdim)=anew(1:agmatdim)
                     BELint=BELint/1.001_dop !TM
                  end if
                  lastRrtime=lastrtime
                  psiRold(:)=psiold(:)
                  aRold(:)=aold(:)
                  dimRold(:,:)=dimold(:,:)
                  gwpdimRold(:,:)=gwpdimold(:,:)

                  lastrtime=atime
                  psiold(1:dgldim)=psi(1:dgldim)
                  aold(1:agmatdim)=agmat(1:agmatdim)
                  dimold(:,:)=dim(:,:)
                  gwpdimold(:,:)=gwpdim(:,:)

            dtpsi=(0.0,0.0)
!write(*,*)"dtpsi before funkr in propvmfbel",dtpsi
!            call renormalize(psi,agmat)
!            call funkr(atime,psi,dtpsi)
!write(*,*)"dtpsi after funkr in propvmfbel",dtpsi

                  if (quality<abs(BELflags(1)) .and. (BELint>=BELorgtime) ) then
!                     BELint=BELint*1.02_dop
!                     BELint=BELint*1.001_dop
!                     BELint=BELint*1.0001_dop
                     BELint=BELorgtime
                  elseif (quality<abs(BELflags(1)) .and. (BELint<BELorgtime) ) then
!                     BELint=BELint*1.25_dop
                     BELint=BELint*1.125_dop
                     if (BELint >=  BELorgtime) BELint=BELorgtime
!                     BELint=BELint*1.001_dop !TM
!                     BELint=BELorgtime
                  end if
        !          write(*,*)'BELint',BELint,quality
               end if
            else
               lastrtime=atime
            !    quality=0.0_dop
            end if

            cstep=0.0_dop

            restartzvode=.true.
            ! have funk(r) recompute S^-1 H on next call
            reCompDtPsi=1
         else
            quality=0.0_dop
         end if

         if (quality==0.0_dop) then
            ifunkr=0
!123 continue
            call propvmf (psi,dtpsi,atime,intperiod,madestep,restartabm, &
                 restartrk5,restartbs,restartzvode,steps,repabmsteps,&
                 goodbssteps,badbssteps,smallbssteps)
             if (errorBELprop == 1) then
               write(1000,*)"atime",atime/fs,"Prop failed. re-reset"
               dim(:,:)=dimnew(:,:)
               gwpdim(:,:)=gwpdimnew(:,:)
               dimred(:,:)=dimrednew(:,:)
               call psidat
               psi(1:dgldim)=psinew(1:dgldim)
               agmat(1:agmatdim)=anew(1:agmatdim)
               ierr=1
               lrepeat=.true.
               go to 888
             endif

!            call checknorm(psi,agmat,norm0)
!
!            if ((abs(norm0)<0.99999d0) .or. (abs(norm0)>1.00001d0)) then
!!              madestep=0.0d0
!!              dtpsi=(0.0,0.0)
!              BELint=BELint/2.0d0
!              psi(1:dgldim)=psiold(1:dgldim)
!              agmat(1:agmatdim)=aold(1:agmatdim)
!!              dim(:,:)=dimold(:,:)
!!              gwpdim(:,:)=gwpdimold(:,:)
!              write(*,*)"checknorm",atime/fs,abs(BELint)/fs,intperiod/fs,abs(norm0)
!              intperiod = abs(BELint)
!              go to 123
!            else
!              if (BELint >=  BELorgtime) then
!                BELint=BELorgtime
!              else
!                BELint=BELint*1.5d0
!              endif
!            endif 
!stop"AAA"
            atime=atime+madestep
!write(*,*)"madestep",madestep/fs
!write(*,*)"atime",atime/fs
!write(*,*)"etime-tiny",etime/fs-tiny/fs
!            if (madestep<intperiod-tiny) then ! this means the integrator bailed out. so force a reset now.
!               if (madestep<0.01_dop*BELstep) then
!                  cstep=abs(BELint)
!               end if
!               BELint=sign(atime-lastrtime,BELint)
!               ! lastrtime=atime-BELstep
!            end if
         end if
!write(*,*)"quality",quality
!write(*,*)"atime",atime/fs
!write(*,*)"etime-tiny",etime/fs-tiny/fs
!write(*,*)"lastrtime",lastrtime/fs
!write(*,*)" "
!stop "AAA"
      end do


      ! resize the wave function to maximum number of GWPs
      ! otherwise output will be garbled due to inconsistent array sizes
!      call renormalize(psi,agmat)
      restartzvode=.true.
      reCompDtPsi=1
!      call restoreMaxDim(psi,agmat)
      iflag=1 !TM

      madestep= trystep
!write(*,*)"madestep",madestep
      return
      end subroutine propvmfbel


!#######################################################################

      subroutine propvmf (psi,dtpsi,time,trystep,madestep,restartabm, &
                          restartrk5,restartbs,restartzvode,steps,&
                          repabmsteps,goodbssteps, &
                          badbssteps,smallbssteps)

      implicit none

      complex(dop), dimension(dgldim), intent(inout) :: psi,dtpsi
      real(dop)                                      :: intperiod
      real(dop), intent(out)                         :: madestep
      real(dop), intent(in)                          :: trystep,time
      integer(long)                                  :: evaltype,mode
      integer(long), intent(inout)                   :: steps,repabmsteps,&
                                                        goodbssteps,badbssteps,&
                                                        smallbssteps
      logical(kind=4), intent(inout)                 :: restartabm,restartrk5,&
                                                        restartbs,restartzvode

! --- CALL INTEGRATION ROUTINE ---
      intperiod = trystep
      mode = 0
      if (lfunkl) evaltype = 0
      if (lfunkr) evaltype = 1
      if (inttyp .eq. 1) then
         call runabm(psi,dtpsi,intperiod,time,restartabm,steps, &
              repabmsteps,evaltype,mode)
         restartabm = .false.
      elseif (inttyp .eq. 2) then
         call runbs(psi,dtpsi,intperiod,time,restartbs,goodbssteps, &
              badbssteps,smallbssteps,evaltype,mode)
         restartbs = .false.
      elseif (inttyp .eq. 4) then
         call runrk5(psi,dtpsi,intperiod,time,evaltype,mode,restartrk5)
      elseif (inttyp .eq. 41) then
         call runrk5F(psi,dtpsi,intperiod,time,evaltype,mode,restartrk5)
      elseif (inttyp .eq. 5) then
         call runrk8(psi,dtpsi,intperiod,time,evaltype,mode,restartrk5)
      elseif (inttyp .eq. 7) then
         call runlsoda(psi,intperiod,time)
      elseif (inttyp .eq. 9) then
         call runzvode(psi,intperiod,time,evaltype,mode,restartzvode)
      else
         routine = 'propvmf'
         message = 'Illegal integration type'
         call errormsg
      endif
      madestep = intperiod

      return
      end subroutine propvmf

! *** V6.2 MB **********> SUBROUTINE PROP CMF FIX <*********************
!                        ~~~~~~~~~~~~~~~~~~~~~~~~~

! Drives the propagation of the wavefunction in constant mean-field
! (CMF) mode with fixed (FIX) update time.
!#######################################################################

      subroutine propcmffix (psi,dtpsi,time,trystep,madestep,restartabm, &
                             restartrk5,restartbs,restartbs2,restartsil, &
                             restartzvode,abmsteps, &
                             repabmsteps,goodbssteps,badbssteps, &
                             smallbssteps,goodsilsteps,badsilsteps, &
                             oldenergy,relaxev,Krylov,HKrylov,Kryc,HKryc)

      implicit none

      complex(dop), dimension(dgldim), intent(inout) :: psi,dtpsi
      complex(dop), dimension(ikryc), intent(inout)  :: Kryc,HKryc
      real(dop), dimension(ikrylov), intent(inout)   :: Krylov,HKrylov
      real(dop) :: intperiod,localtime,energy
      real(dop), intent(out)                         :: madestep
      real(dop), intent(in)                          :: trystep,time
      real(dop), intent(inout)                       :: oldenergy
      integer(long), intent(inout)                   :: abmsteps,repabmsteps,&
                                                        goodbssteps,badbssteps,&
                                                        smallbssteps,&
                                                        goodsilsteps,badsilsteps
      integer(long)                                  :: nmodesave = 0
      logical(kind=4), intent(inout)                 :: restartabm,restartrk5,&
                                                        restartsil,relaxev
      logical(kind=4), intent(inout)                 :: restartbs,restartbs2
      logical(kind=4), intent(inout)                 :: restartzvode

      if(lrunml) then
          nmodesave = nmode
          nmode     = nmlmode
      end if

! --- SAVE SPFS IN PHI ARRAY (for multi-D H terms)
      if (nmulpot(1) .gt. 0 .or. lklha) then
         call cpvxz(psi(adim+1),mc(mcphi),totphidim)
      endif

! --- WRITE INFORMATION TO rlx_info FILE (IMPROVED RELAXATION ONLY) ---
      if(relaxev) then
         call calcenergy (psi,dtpsi,energy,0)
         if(inttypea.eq.6) then     ! DAV
            write(irlx,'(a,f10.3,f18.10," eV,",f16.10," meV")') &
                 'After Davidson   T =',time/fs,energy*ev, &
                  (energy-oldenergy)*ev*1.0e+3_dop
         else
            write(irlx,'(a,f10.3,f18.10," eV,",f16.10," meV")') &
                  'After  Lanczos   T =',time/fs,-energy*ev, &
                   (oldenergy-energy)*ev*1.0e+3_dop
         endif
         oldenergy = energy
         call myflush(irlx)

      else ! normal propagation
!    --- SAVE OLD SPFs ---
         call cpvxz(psi(adim+1),mc(mcoldpsi),totphidim)

!    --- PROPAGATE MCTDH-COEFFICIENTS (FIRST HALF-STEP) ---
         intperiod = 0.5_dop*trystep
         call propa(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
              restartbs2,restartsil,restartzvode, &
              abmsteps,repabmsteps,goodbssteps,badbssteps, &
              smallbssteps,goodsilsteps,badsilsteps,Krylov,HKrylov,Kryc,HKryc)

         if(allcomplete) then
            localtime = time+trystep
            call funka(localtime,psi,dtpsi)
            goto 700
         endif

!    --- PROPAGATE PHI (PREDICTOR HALF-STEP) ---
         intperiod = 0.5_dop*trystep
         call propphi(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
              restartbs,restartzvode, &
              abmsteps,repabmsteps,goodbssteps,badbssteps, &
              smallbssteps,goodsilsteps,badsilsteps,restartsil)

!    --- COMPUTE PREDICTED MEAN-FIELDS ---
         localtime = time+0.5_dop*trystep
         call funk0(localtime,psi,dtpsi)

!    --- RE-ESTABLISH OLD SPFS AND COMPUTE THEIR TIME DERIVATIVE ---
         call cpvxz(mc(mcoldpsi),psi(adim+1),totphidim)
         call funk1(time,psi,dtpsi)
      endif

! --- PROPAGATE PHI ---
      intperiod = trystep
      call propphi(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
                   restartbs,restartzvode, &
                   abmsteps,repabmsteps,goodbssteps,badbssteps, &
                   smallbssteps,goodsilsteps,badsilsteps,restartsil)
      
! --- SAVE NEW SPFS IN PHI ARRAY (for multi-D H terms)
      if (nmulpot(1) .gt. 0 .or. lklha) then
         call cpvxz(psi(adim+1),mc(mcphi),totphidim)
      endif

! --- DETERMINE NEW MATRIX ELEMENTS AND DERIVATIVE OF A-VECTOR ---

! Note that "funk2" uses the "intpsi" array as complex work array!

      localtime = time+trystep
      call funk2(localtime,psi)
      call funka(localtime,psi,dtpsi)

      if(relaxev) then
         call calcenergy (psi,dtpsi,energy,0)
         if(inttypea.eq.6) then    ! DAV
            write(irlx,'(a,f10.3,f18.10," eV,",f16.10," meV")') &
                 'After Relaxation T =',localtime/fs, &
                  energy*ev,(energy-oldenergy)*ev*1.0e+3_dop
         else                      ! SIL
            write(irlx,'(a,f10.3,f18.10," eV,",f16.10," meV")') &
                  'After Relaxation T =',localtime/fs, &
                  -energy*ev,(oldenergy-energy)*ev*1.0e+3_dop
         endif
         oldenergy = energy
         call myflush(irlx)
      endif

! --- PROPAGATE MCTDH-COEFFICIENTS (SECOND HALF-STEP) ---

  700 continue
      intperiod = 0.5_dop*trystep
      localtime = time+0.5_dop*updatetime
      call propa(psi,dtpsi,localtime,intperiod,restartabm,restartrk5,&
                 restartbs2,restartsil,restartzvode, &
                 abmsteps,repabmsteps,goodbssteps,badbssteps, &
                 smallbssteps,goodsilsteps,badsilsteps,Krylov,HKrylov,Kryc,HKryc)
      madestep = trystep

      if(lrunml) then
          nmode     = nmodesave
      end if

      return
      end subroutine propcmffix

! *** V6.2 MB **************> SUBROUTINE PROPA <************************
!                            ~~~~~~~~~~~~~~~~~~
! Drives the propagation of the MCTDH-coefficients.
!#######################################################################

      subroutine propa (psi,dtpsi,time,trystep,restartabm,restartrk5,&
                        restartbs,restartsil,restartzvode, &
                        abmsteps,repabmsteps,goodbssteps, &
                        badbssteps,smallbssteps,goodsilsteps, &
                        badsilsteps,Krylov,HKrylov,Kryc,HKryc)
      
      implicit none

      complex(dop), dimension(dgldim), intent(inout) :: psi,dtpsi
      complex(dop), dimension(ikryc), intent(inout)  :: Kryc,HKryc
      real(dop), dimension(ikrylov), intent(inout)   :: Krylov,HKrylov
      real(dop)                                      :: norm
      real(dop), intent(in)                          :: time,trystep
      real(dop)                                      :: intperiod
      integer(long)                                  :: evaltype,mode
      integer(long), intent(inout)                   :: abmsteps,repabmsteps, &
                                                        goodbssteps, &
                                                        badbssteps,&
                                                        smallbssteps, &
                                                        goodsilsteps, &
                                                        badsilsteps
      logical(kind=4), intent(inout)                 :: restartabm,restartrk5,&
                                                        restartsil
      logical(kind=4), intent(inout)                 :: restartbs,restartzvode

      intperiod=trystep
      
! --- NORMALIZE COEFFICIENTS IN A MULTI-PACKET RUN USING SIL ---

      if ((lmulpack .or. mpb.gt.0) .and. (inttypea.eq.3)) then
         norm = sqrt(dble(max(npackts,npacket)))
         call xvixdzo(norm,psi,adim)
         call xvixdzo(norm,dtpsi,adim)
      endif

! --- PROPAGATE MCTDH-COEFFICIENTS ---

      mode = 0
      evaltype = 5
      if (inttypea .eq. 1) then
         call runabm(psi,dtpsi,intperiod,time,restartabm,abmsteps,&
              repabmsteps,evaltype,mode)

      elseif (inttypea .eq. 2) then
         call runbs(psi,dtpsi,intperiod,time,restartbs,goodbssteps, &
              badbssteps,smallbssteps,evaltype,mode)
         restartbs = .false.
      elseif (inttypea .eq. 3) then
         call runsil(psi,dtpsi,intperiod,time,restartsil, &
              goodsilsteps,badsilsteps,evaltype)
      elseif (inttypea .eq. 4) then
         call runrk5(psi,dtpsi,intperiod,time,evaltype,mode,restartrk5)
      elseif (inttypea .eq. 5) then
         call runrk8(psi,dtpsi,intperiod,time,evaltype,mode,restartrk5)
      elseif (inttypea .eq. 6) then
         if (dentype .eq. 0) then
            call rundav(psi,dtpsi,intperiod,time,goodsilsteps, &
                 badsilsteps,evaltype,Krylov,HKrylov,Kryc,HKryc)
         else
            routine = 'propa'
            message = 'Davidson only for wavefunctions'
            call errormsg
         endif
      elseif (inttypea .eq. 9) then
         call runzvode(psi,intperiod,time,evaltype,mode,restartzvode)
      else
         routine = 'propa'
         message = 'Illegal integration type'
         call errormsg
      endif

! --- DE-NORMALIZE COEFFICIENTS IN A MULTI-PACKET RUN USING SIL ---

      if ((lmulpack .or. mpb.gt.0) .and. (inttypea .eq. 3)) then
         norm = 1.0_dop/norm
         call xvixdzo(norm,psi,adim)
         call xvixdzo(norm,dtpsi,adim)
      endif

      return
      end subroutine propa

! *** V6.2 MB *************> SUBROUTINE PROPPHI <***********************
!                   ~~~~~~~~~~~~~~~~~~~~
! Drives the propagation of the single-particle functions.
!#######################################################################

      subroutine propphi (psi,dtpsi,time,trystep,restartabm,restartrk5,&
                          restartbs,restartzvode, &
                          abmsteps,repabmsteps,goodbssteps,badbssteps, &
                          smallbssteps,goodsilsteps,badsilsteps, &
                          restartsil)

      implicit none

      complex(dop), dimension(dgldim), intent(inout):: psi,dtpsi
      real(dop), intent(in)                         :: time,trystep
      real(dop)                                     :: intperiod
      integer(long)                                 :: evaltype,m,m1,nnode
      integer(long), intent(inout)                  :: abmsteps,repabmsteps,&
                                                       goodbssteps,badbssteps,&
                                                       smallbssteps,&
                                                       goodsilsteps,&
                                                       badsilsteps
      logical(kind=4), intent(inout)                :: restartabm,restartrk5,&
                                                       restartsil
      logical(kind=4), intent(inout)                :: restartbs,restartzvode

      type(wfnode_t), pointer :: cnode
      integer       :: z0,zf

      intperiod=trystep

      if (lrunml) then
         nnode = tree%tot_nodes
      else
         nnode = nmode
      endif

! --- PROPAGATE SINGLE-PARTICLE FUNCTIONS ---
! THE STARTING DO LOOP ENABLES THE MULTI-LAYER WHEN RUNNING ML-MCTDH. 
! run over all nodes
      do m1 = 1,nnode

! get m for primitive basis or flag middle layer SPFs
         if (lrunml) then
            cnode => tree%swap_down(m1)%p
            if (cnode%istop) cycle
            z0 = cnode%z(1,1)
            zf = cnode%z(cnode%navecs,2)

            if (cnode%isprimitive) then
               m = cnode%m
               evaltype = 4
            else
               m = 0
               currentnode => cnode ! for funkphinode
               evaltype = 7
            endif
         else
! THIS IS WHERE WE SET THE evaltype = 4 
! SO HERE "m" STARTS AT THE VALUE OF "m1" ie 1
            m = m1
            evaltype = 4
         endif
         currentmode = m

         if (m .ne. 0) then
            if (complete(m)) cycle   ! complete primitive basis 
         endif

         ! lmpiupdt is used to avoid redundant MPI communitation
         ! lmpiupdt(1) is a reset flag. if lmpiupdt(1) is .false.
         ! lmpiupdt(i) is set to .false. after a certain entity
         ! has been sent via MPI. what lmpiupdt(i) actually means
         ! is context dependent. Here only lmpiupdt(2) is used and
         ! it means that the pointers for this mode need to be sent
         ! within routine mpihlochphi. 
         ! as lmpiupdt(1) is false, they will not be re-sent until 
         ! lmpiupdt(2) is set to .true. again. 
         ! This is usefull, as the integrators will repeatedly 
         ! call this routine numerous times. So we avoid unnecessary 
         ! communication. 
         call initvxl(lmpiupdt,nmpiupdtlen)
         lmpiupdt(1) = .false.
         if (inttypephi .eq. 1) then
            call runabm(psi,dtpsi,intperiod,time,restartabm, &
                 abmsteps,repabmsteps,evaltype,m)

         elseif (inttypephi .eq. 2) then
            call runbs(psi,dtpsi,intperiod,time,restartbs,&
                 goodbssteps,badbssteps,smallbssteps,evaltype,m)
         elseif (inttypephi .eq. 3) then
            call runsil(psi,dtpsi,intperiod,time,restartsil,&
                 goodsilsteps,badsilsteps,evaltype)
         elseif (inttypephi .eq. 4) then
!      RK5 integrator
            call runrk5(psi,dtpsi,intperiod,time,evaltype,m,restartrk5)
         elseif (inttypephi .eq. 5) then
!      RK8 integrator
            call runrk8(psi,dtpsi,intperiod,time,evaltype,m,restartrk5)
         elseif (inttypephi .eq. 6)then
            if(dentype.ne.0 .and. .not.lexact)then
               routine = 'propphi'
               message = 'Euler integrator only for exact wavefunctions.'
               call errormsg
            else
               call runeuler(psi,dtpsi,intperiod)
            endif
         elseif (inttypephi .eq. 8) then
!CHEB - CALLING CHEBYSHEV INTEGRATOR 
            if (.not. lexact) then
               routine = 'propphi'
               message = 'Cheb only for exact wavefunctions'
               call errormsg
            else
               call runcheb(psi,dtpsi,intperiod) 
            endif
         elseif (inttypephi .eq. 9) then
!      ZVODE integrator
            if (dentype .ne. 0) then
               routine = 'propphi'
               message = 'ZVODE only for wavefunctions'
               call errormsg
            else
               call runzvode(psi,intperiod,time,evaltype,m,restartzvode)
            endif
         elseif (inttypephi .eq. 41) then
! Fixed step RK5 integrator
            call runrk5f(psi,dtpsi,intperiod,time,evaltype,m,restartrk5)
         elseif (inttypephi .eq. 69) then
! SOD - CALLING THE SODDING INTEGRATOR!!
            if (dentype .ne. 0 .and. .not. lexact) then
               routine = 'propphi'
               message = 'SOD only for exact wavefunctions'
               call errormsg
            else
               call runsod(psi,dtpsi,mc(mcintpsi),time,intperiod)
            endif
         elseif (inttypephi .eq. 70) then
! SPO - CALLING THE SPO     INTEGRATOR!!
            if (dentype .ne. 0 .and. .not. lexact) then
               routine = 'propphi'
               message = 'SPO only for exact wavefunctions'
               call errormsg
            else
               call runspo(psi,mr(mrhops),mr(mrfftp),intperiod,&
                    mc(mchin),mc(mcrueck),mi(mifftfak))
            endif
         else
            routine = 'propphi'
            message = 'Illegal integration type'
            call errormsg
         endif
      enddo

      call initvxl(lmpiupdt,nmpiupdtlen)
      restartbs = .false.

      return
      end subroutine propphi

! *** V8.5 OV/HDM *********> SUBROUTINE PROPPHIML <*********************
!                           ~~~~~~~~~~~~~~~~~~~~

! Drives the propagation of the all coefficients in a CMF Multilayer run.

      subroutine propphiml(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
                          restartbs,&
                          abmsteps,repabmsteps,goodbssteps,badbssteps,&
                          smallbssteps,goodsilsteps,badsilsteps,&
                          restartsil)

      implicit none

      complex(dop), dimension(dgldim) :: psi,dtpsi
      real(dop)                       :: time,intperiod
      integer(long)                   :: abmsteps,repabmsteps,goodbssteps,&
                                         badbssteps,smallbssteps,evaltype,&
                                         m,goodsilsteps,badsilsteps
      logical(long)                   :: restartabm,restartbs,restartsil,&
                                         restartrk5

      m = 0 ! mode number, dummy in an ML run.

! --- PROPAGATE SINGLE-PARTICLE FUNCTIONS ---
      evaltype = 6

      if (.not.lmlcmfsplit) then
! --- Propagate SPFs of all modes together

         if (inttypephi .eq. 1) then
            call runabm(psi,dtpsi,intperiod,time,restartabm,&
                 abmsteps,repabmsteps,evaltype,m)

         elseif (inttypephi .eq. 2) then
            call runbs(psi,dtpsi,intperiod,time,restartbs,&
                 goodbssteps,badbssteps,smallbssteps,evaltype,m)
         elseif (inttypephi .eq. 3) then
            call runsil(psi,dtpsi,intperiod,time,restartsil,&
                 goodsilsteps,badsilsteps,evaltype)
         elseif (inttypephi .eq. 4) then
!        RK5 integrator
            call runrk5(psi,dtpsi,intperiod,time,evaltype,m,restartrk5)
         elseif (inttypephi .eq. 5) then
!        RK8 integrator
            call runrk8(psi,dtpsi,intperiod,time,evaltype,m,restartrk5)
         else
            routine = 'propphiml'
            message = 'Illegal integration type'
            call errormsg
         endif
         restartbs = false

      else
! --- Propagate SPFs of all modes separately
!     (each node stores its own integrator settings)

         call mlsplitprop(time,psi,dtpsi,intperiod,restartabm,restartbs,&
              restartsil)

      endif
      return
    end subroutine propphiml

!#######################################################################

    subroutine mlsplitprop(time,psi,dtpsi,intperiod,&
               restartabm,restartbs,restartsil)
        implicit none
        real(dbl)     :: time
        complex(cdbl) :: psi(dgldim)
        complex(cdbl) :: dtpsi(dgldim)
        real(dbl)     :: intperiod
        logical       :: restartabm,restartbs,restartsil
        type(wfnode_t),pointer :: cnode, tp       ! Dieter apr2021 tp added
        complex(cdbl),pointer  :: dmcurr(:,:)  ! Dieter apr2021
        integer       :: i,z0,zf  ,  f            ! Dieter apr2021  f added
        logical       :: restart
        do i=1,tree%tot_nodes
            cnode => tree%swap_down(i)%p
            z0 = cnode%z(1,1)
            zf = cnode%z(cnode%navecs,2)
            if(cnode%istop) then
               dmcurr=>null()
               f=0
            else
               f=cnode%nsubcoord
               tp=>cnode%parent
               dmcurr=>tp%dmat(f)%p%matrix
            endif
            select case (cnode%integrator%inttyp)
               case (inttyp_abm)
                  restart = restartabm
               case (inttyp_bs)
                  restart = restartbs
               case (inttyp_sil,inttyp_csil)
                  restart = restartsil
               case default
                  restart = .false.
            end select
            currentnode => cnode ! for funkphinode (ugly but what can you do without procedure pointers?)
            if (cnode%isprimitive) then
               currentmode = cnode%m
               call drive_integrator(psi(z0:zf), dtpsi(z0:zf), intperiod, &
                    time, restart, cnode%integrator, funkphiml, dmcurr, &
                    cnode%navecs)
            else
               call drive_integrator(psi(z0:zf), dtpsi(z0:zf), intperiod, &
                    time, restart, cnode%integrator, funkphinode, dmcurr, &
                    cnode%navecs)
            endif
           ! If relaxation, renormalize.
           if (tree%isrelax) then
              select case (tree%integrator%inttyp)
                 case (inttyp_sil,inttyp_csil)
                    continue ! silstep already did it
                 case default
                    call renorm_tree(tree,psi)
              end select
           endif

        end do
    end subroutine mlsplitprop

! *** V6.4 MB *************> SUBROUTINE ERROR A <***********************
!                           ~~~~~~~~~~~~~~~~~~~~
! Computes the propagation error of the MCTDH-coefficients caused by
! holding the mean-fields constant.
!#######################################################################

      subroutine errora (erra,psi1,psi2)

      implicit none
      
      real(dop), parameter                      :: errcorr = 0.25_dop

      complex(dop), dimension(adim), intent(in)    :: psi2
      complex(dop), dimension(adim), intent(inout) :: psi1
      real(dop)                                 :: norm1,norm2
      real(dop), intent(out)                    :: erra

! --- COMPUTE DIFFERENCE OF THE TWO A-VECTORS ---

      call subvxxzo1(psi1,psi2,adim)  ! psi1-psi2 -> psi1

! --- DETERMINE NORM OF A-VECTORS AND ESTIMATE ERROR ---

      call normvxz(psi1,norm1,adim)
      call normvxz(psi2,norm2,adim)
      if (lmulpack) then  ! multi-packets multi-set
         norm1 = norm1/sqrt(dble(npacket))
         norm2 = norm2/sqrt(dble(npacket))
      elseif (mpb.gt.0) then ! multi-packets single-set
         norm1 = norm1/sqrt(dble(npackts))
         norm2 = norm2/sqrt(dble(npackts))
      endif
!      erra = errcorr*5.0_dop*norm1**2/(4.0_dop*norm2**2+norm2)
      erra = errcorr*norm1**2/norm2**2

      return
      end subroutine errora

! *** V6.4 MB ************> SUBROUTINE ERROR PHI <**********************
!                          ~~~~~~~~~~~~~~~~~~~~~~
! Computes the propagation error of the single-particle functions caused
! by holding the mean-fields constant.
!#######################################################################

      subroutine errorphi (deltasum,phi1,phi2,dicht2,agmat1,agmat2)

use gwplib, only: diagonal_moments
use lalib,  only: simtran1bz, tran1mxazz
use op1lib, only: cpqxz,sumqxz
use op2lib, only: sum2qxxz

      implicit none
      
      complex(dop), dimension(adim+1:dgldim), intent(in) :: phi1,phi2
      complex(dop), dimension(dmatdim), intent(in)       :: dicht2
      complex(dop), dimension(maxphidim)                 :: deltaphi
      complex(dop), dimension(maxspf*maxspf)             :: ovr1,ovr2,workc
      complex(dop), dimension(1)                         :: zdum,zdum1
      complex(dop)                                       :: sumovr
      complex(dop), dimension(agmatdim), intent(in)      :: agmat1,agmat2
      real(dop)                                          :: rhosum,dltsum,&
                                                            trrho,trovr
      real(dop), intent(out)                             :: deltasum
      integer(long)                                      :: m,s

! --- INITIALISE VARIABLES ---

      deltasum=0.0

! --- LOOP OVER ALL MODES AND STATES ---

      do m=1,nmode
         rhosum=0.0
         dltsum=0.0
         do s=1,nstate

!       --- GENERATE OVERLAP MATRIX FROM DIFFERENCE OF SINGLE-PARTICLE 
!           FUNCTIONS ---

            if (gwpm(m)) then
               call diagonal_moments(m,s,s,nspfdof(m),gwpdim(m,s), &
                    gwpdim(m,s),agmat1(zagmat(m,s)),agmat1(zagmat(m,s)), &
                    phi1(zxi(m,s)),phi1(zxi(m,s)),agmat1(zphase(m,s)), & ! phase mod yapol
                    agmat1(zphase(m,s)),workc,zdum,zdum1,0,0)             ! phase mod yapol
               if (psitype .eq. 0 .or. psitype .eq. 5) then
                  call simtran1bz(workc,phi1(zdgwp(m,s)),ovr1,gwpdim(m,s),dim(m,s))
               else
                  call cpqxz(workc,ovr1,gwpdim(m,s))
               endif

               call diagonal_moments(m,s,s,nspfdof(m),gwpdim(m,s),&
                    gwpdim(m,s),agmat2(zagmat(m,s)),agmat2(zagmat(m,s)),&
                    phi2(zxi(m,s)),phi2(zxi(m,s)),agmat2(zphase(m,s)),&  ! phase mod yapol
                    agmat2(zphase(m,s)),workc,zdum,zdum1,0,0)             ! phase mod yapol
               if (psitype .eq. 0 .or. psitype .eq. 5) then
                  call simtran1bz(workc,phi2(zdgwp(m,s)),ovr2,gwpdim(m,s),dim(m,s))
               else
                  call cpqxz(workc,ovr2,gwpdim(m,s))
               endif
               call addmxxzo(ovr2,ovr1,dim(m,s),dim(m,s))    

               call diagonal_moments(m,s,s,nspfdof(m),gwpdim(m,s),&
                    gwpdim(m,s),agmat1(zagmat(m,s)),agmat2(zagmat(m,s)),&
                    phi1(zxi(m,s)),phi2(zxi(m,s)),agmat1(zphase(m,s)),&  ! phase mod yapol
                    agmat2(zphase(m,s)),workc,zdum,zdum1,2,0)             ! phase mod yapol
               if (psitype .eq. 0 .or. psitype .eq. 5) then
                  call tran1mxazz(phi1(zdgwp(m,s)),workc,phi2(zdgwp(m,s)),&
                       ovr2,gwpdim(m,s),dim(m,s))
               else
                  call cpqxz(workc,ovr2,gwpdim(m,s))
               endif
               call subgwpovl(ovr1,ovr2,dim(m,s))   

            else

               call submxxz(phi1(zetf(m,s)),phi2(zetf(m,s)),&
                            deltaphi,subdim(m),dim(m,s))

               call overmxz(deltaphi,ovr1,subdim(m),dim(m,s))

            endif

            if (abs(psitype) .eq. 4) then
! new error measure for G-MCTDH  GW 23/6/16 ??
               call sum2qxxz(ovr1,dicht2(dmat(m,s)),sumovr,gwpdim(m,s))
               dltsum=dltsum+abs(sumovr)   ! Sum differences over states
               call sumqxz(dicht2(dmat(m,s)),sumovr,gwpdim(m,s))
               rhosum=rhosum+abs(sumovr)   ! sum tr(rho) over states

            else

!       --- GET TRACE OF WEIGHTED OVERLAP WITH DENSITY MATRIX ---

               call trhhxtzz(ovr1,dicht2(dmat(m,s)),trovr,dim(m,s))

!       --- DETERMINE TRACE OF DENSITY AND WEIGHTED OVERLAP MATRIX ---

               call trhxz(dicht2(dmat(m,s)),trrho,dim(m,s))
               dltsum=dltsum+trovr   ! Sum differences over states
               rhosum=rhosum+trrho   ! sum tr(rho) over states
            endif
         enddo

!       --- ESTIMATE ERROR CAUSED BY CONSTANT MEAN-FIELDS ---

         deltasum=deltasum + dltsum/(rhosum+tiny) ! Sum over modes

! deltasum may be negative due to numerics if minimal change has occurred
         if (deltasum .lt. 0.0_dop)  deltasum = 0.0_dop
      enddo
      if (lmulpack) deltasum = deltasum/dble(npacket)

      return
      end subroutine errorphi

!#######################################################################

      subroutine subgwpovl(ovr1,ovr2,dim1)   

      implicit none

      integer(long)                                     :: e,e1
      integer(long), intent(in)                         :: dim1
      complex(dop), dimension(dim1,dim1), intent(in)    :: ovr2
      complex(dop), dimension(dim1,dim1), intent(inout) :: ovr1

! overlap <g_i pred | g_j corr > not Hermitian ??
      do e=1,dim1
         do e1=1,dim1
            ovr1(e1,e) = ovr1(e1,e) - ovr2(e1,e) - dconjg(ovr2(e,e1))
         enddo
      enddo

      return
      end subroutine subgwpovl

!***********************************************************************
!     ERRORPHI1 for density operators of type I
!***********************************************************************

      subroutine errorphi1(deltasum,phi1,phi2,dicht2)

      implicit none
      
      integer(long)                                       :: m,s,t
      real(dop)                                           :: dltsum,rhosum,trrho,trovr
      real(dop), intent(out)                              :: deltasum
      complex(dop), dimension(totphidim), intent(in)      :: phi1,phi2
      complex(dop), dimension(dmatdim), intent(in)        :: dicht2
      complex(dop), dimension(maxphidim)                  :: deltaphi
      complex(dop), dimension(maxspf*maxspf)              :: ovr1

! --- INITIALISE VARIABLES ---

      deltasum=0.0

! --- LOOP OVER ALL MODES AND STATES ---

      do m=1,nmode
         dltsum=0   
         rhosum=0   
         do s=1,nstate
            do t=1,nstate

!       --- COMPUTE DIFFERENCE OF SINGLE-PARTICLE FUNCTIONS ---

               call submxxz(phi1(dzetf(m,s,t)-adim), &
                    phi2(dzetf(m,s,t)-adim),deltaphi,subdim(m), &
                    ddim(m,s,t))

!       --- GENERATE OVERLAP MATRIX ---

               call overmxz(deltaphi,ovr1,subdim(m),ddim(m,s,t))

!       --- TRACE OF WEIGHTED OVERLAP WITH DENSITY MATRIX ---

               call trhhxtzz(ovr1,dicht2(ddmat(m,s,t)),trovr,ddim(m,s,t))

!       --- DETERMINE TRACE OF DENSITY AND WEIGHTED OVERLAP MATRIX ---

               call trhxz(dicht2(ddmat(m,s,t)),trrho,ddim(m,s,t))
               dltsum=dltsum+trovr   ! Sum differences over states
               rhosum=rhosum+trrho   ! sum tr(rho) over states
            enddo
         enddo

!       --- ESTIMATE ERROR CAUSED BY CONSTANT MEAN-FIELDS ---
  
         deltasum=deltasum+dltsum/(rhosum+tiny)
      enddo
      if (lmulpack) deltasum = deltasum/dble(npacket)
      if (mpb.gt.0) deltasum = deltasum/dble(npackts)

      return
      end  subroutine errorphi1

! *** V8.5 OV ************> SUBROUTINE ERROR TOT <**********************
!                          ~~~~~~~~~~~~~~~~~~~~~~

! Computes the propagation error of the total wavefunction caused
! by holding the mean-fields constant.
! This routine is by now used only when performing a ML calculation
! in CMF mode, and it is just a wrapper around the routine that handles
! ML wavefunctions.

      subroutine errortot(deltasum,psi1,psi2,agmat1,agmat2)
        implicit none

        real(dop), intent(out)                :: deltasum
        complex(dop),dimension(*), intent(in) ::  psi1,psi2
        complex(dop),dimension(*), intent(in) ::  agmat1,agmat2

        if(lrunml) then
            call errortotml(deltasum,psi1,psi2,agmat1,agmat2)
        else
!  --> implement here the normal MCTDH code if necessary
            write(6,*) 'errortot only implemented for ML wavefunctions'
            stop
        end if
      end subroutine errortot

! *** V6.4 MB ********> SUBROUTINE WRITE INT METHOD <*******************
!                      ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! Writes the integration method to log file.
!-------------------------------------------------------------------------------

      subroutine writeintmethod (abmsteps,repabmsteps,smallbssteps, &
                                 goodbssteps,badbssteps,goodsilsteps, &
                                 badsilsteps,sumupdate,sum2update, &
                                 updatesteps,badupdatesteps,badbackw)

use channels
use odesim
use multilayer, only: wim_ml

      implicit none
      
      integer(long), intent(in) :: abmsteps,repabmsteps,smallbssteps,goodbssteps,badbssteps,&
                                   goodsilSteps,badsilSteps,updatesteps,badupdatesteps,&
                                   badbackw
      real(dop), intent(in)     :: sumupdate,sum2update

! --- WRITE INTEGRATION METHOD TO LOG FILE ---

      write(ilog,*)
      write(ilog,'(a)') '***** Integration method *****'
      write(ilog,*)

      if (lrunml) then ! multi-layer
         if (lconst) then ! ML-CMF
            if (lmlcmfsplit) then ! ML-CMF=split
               call wim_ml(ilog)
            else ! ML-CMF=unified
               call wim_one(ilog,inttypephi,intorderphi,liork,&
                    abmsteps,repabmsteps,&
                    smallbssteps,goodbssteps,badbssteps,&
                    rk5steps,rk5accpt,rk5rejct,&
                    rk8steps,rk8accpt,rk8rejct,&
                    zvodesteps,zvodeaccpt,zvoderejct)
            endif
         else ! ML-VMF
            call wim_one(ilog,inttyp,intord,liork,&
                 abmsteps,repabmsteps,&
                 smallbssteps,goodbssteps,badbssteps,&
                 rk5steps,rk5accpt,rk5rejct,&
                 rk8steps,rk8accpt,rk8rejct,&
                 zvodesteps,zvodeaccpt,zvoderejct)
         endif

      else if (lconst) then
         write(ilog,'(a)')&
              'Integration method used for the MCTDH-coefficients:'
         if (inttypea .eq. 1) then
            write(ilog,'(a)')&
                 'Adams-Bashforth-Moulton (ABM) predictor-corrector.'
            write(ilog,'(a,I2)') 'Integration order: ',intordera
         elseif (inttypea .eq. 2) then
            write(ilog,'(a)')&
                 'Bulirsch-Stoer (BS) with polynomial extrapolation.'
            write(ilog,'(a,I2)') 'Optimal column: ',intordera
         elseif (inttypea .eq. 3) then
            if(lcap .or. ldissin) then
               write(ilog,'(a)')&
                    'Short iterative Lanczos-Arnoldi with variable order.'
            else
               write(ilog,'(a)')&
                    'Short iterative Lanczos (real) with variable order.'
            endif
            write(ilog,'(a,I3)') 'Maximum order employed: ',maxsilorder
            write(ilog,'(a,I3)') 'Maximum order allowed:  ',intordera
         elseif (inttypea .eq. 4) then
            write(ilog,'(a)')&
                 'Runge-Kutta order 5 (RK5) from ODESIM'
         elseif (inttypea .eq. 41) then
            write(ilog,'(a)')&
                 'Runge-Kutta order 5 (RK5) from ODESIM. Fixed Time Step.'
         elseif (inttypea .eq. 5) then
            write(ilog,'(a)')&
                 'Runge-Kutta order 8 (RK8) from ODESIM'
         elseif (inttypea .eq. 6) then
            if(mpb.gt.0) then
               if(lrreal) then
                  write(ilog,'(2a)') 'Block-Davidson "integrator"', &
                       ' (restricted-real)'
               elseif(lreal) then
                  write(ilog,'(a)') 'Block-Davidson "integrator" (real)'
               endif
            elseif(lrreal) then
               write(ilog,'(a)')'Davidson "integrator" (restricted-real)'
            elseif(lreal) then
               write(ilog,'(a)') 'Davidson "integrator" (real)'
            elseif(lcap) then
               write(ilog,'(a)') 'Davidson "integrator" (complex)'
            else
               write(ilog,'(a)') 'Davidson "integrator" (hermitian)'
            endif
            write(ilog,'(a,I5)') 'Maximum order employed: ',maxsilorder
            write(ilog,'(a,I5)') 'Maximum order allowed:  ',intordera
            if(inttypea.eq.6 .and. precon.gt.0) write(ilog,'(a,I5)') &
                 'Dimension of PreConditioner:  ',precon
         elseif (inttypea .eq. 9) then
            write(ilog,'(a)')&
                 'ZVODE from ODEPACK'
         endif
         write(ilog,*)
         write(ilog,'(a)')&
              'Integration method used for the single-particle functions:'
         if (inttypephi .eq. 1) then
            write(ilog,'(a)')&
                 'Adams-Bashforth-Moulton (ABM) predictor-corrector.'
            write(ilog,'(a,I2)') 'Integration order: ', intorderphi
         elseif (inttypephi .eq. 2) then
            write(ilog,'(a)')&
                 'Bulirsch-Stoer (BS) with polynomial extrapolation.'
            write(ilog,'(a,I2)') 'Optimal column: ',intorderphi
         elseif (inttypephi .eq. 4) then
            write(ilog,'(a)')&
                 'Runge-Kutta order 5 (RK5) from ODESIM'
            if(liork) write(ilog,'(a)')'with improved orthogonality'
         elseif (inttypephi .eq. 41) then
            write(ilog,'(a)')&
                 'Runge-Kutta order 5 (RK5) from ODESIM. Fixed Time Step.'
         elseif (inttypephi .eq. 5) then
            write(ilog,'(a)')&
                 'Runge-Kutta order 8 (RK8) from ODESIM'
            if(liork) write(ilog,'(a)')'with improved orthogonality'
         elseif (inttypephi .eq. 9) then
            write(ilog,'(a)')&
                 'ZVODE from ODEPACK'
            if (limportho) write(ilog,'(a)') &
               'with improved orthogonality (not implemented)'
         endif
         write(ilog,*)
         if ((inttypea .eq. 1) .or. (inttypephi .eq. 1)) then
            write(ilog,'(a,I9)')&
                 'Total number of ABM steps:    ',abmsteps
            write(ilog,'(a,I9)')&
                 'Number of repeated ABM steps: ',repabmsteps
            write(ilog,*)
         endif
         if ((inttypea .eq. 2) .or. (inttypephi .eq. 2)) then
            write(ilog,'(a,I9)')&
                 'Number of succesful large BS steps:        ',goodbssteps
            write(ilog,'(a,I9)')&
                 'Number of failed but fixed large BS steps: ',badbssteps
            write(ilog,'(a,I9)')&
                 'Total number of small BS steps:            ',smallbssteps
            write(ilog,*)
         endif
         if (inttypea .eq. 3) then
            write(ilog,'(a,I9)')&
                 'Number of successful Lanczos steps:       ',goodsilsteps
            write(ilog,'(a,I9)')&
                 'Number of failed but fixed Lanczos steps: ',badsilsteps
            write(ilog,*)
         endif
         if ((inttypea .eq. 4) .or. (inttypephi .eq. 4)) then
            write(ilog,'(a,I9)')&
                 'Total number of RK5 steps:     ',rk5steps
            write(ilog,'(a,I9)')&
                 'Number of accepted RK5 steps:  ',rk5accpt
            write(ilog,'(a,I9)')&
                 'Number of rejected RK5 steps:  ',rk5rejct
            write(ilog,*)
         endif
         if ((inttypea .eq. 41) .or. (inttypephi .eq. 41)) then
            write(ilog,'(a,I9)')&
                 'Total number of RK5 steps:     ',rk5steps
            write(ilog,*)
         endif
         if ((inttypea .eq. 5) .or. (inttypephi .eq. 5)) then
            write(ilog,'(a,I9)')&
                 'Total number of RK8 steps:     ',rk8steps
            write(ilog,'(a,I9)')&
                 'Number of accepted RK8 steps:  ',rk8accpt
            write(ilog,'(a,I9)')&
                 'Number of rejected RK8 steps:  ',rk8rejct
            write(ilog,*)
         endif
         if (inttypea .eq. 6) then
            write(ilog,'(a,I6)') &
                 'Number of successful Davidson steps:',goodsilsteps
         endif
         if ((inttypea .eq. 9) .or. (inttypephi .eq. 9)) then
            write(ilog,'(a,I9)')&
                 'Total number of ZVODE steps:     ',zvodesteps
            write(ilog,'(a,I9)')&
                 'Number of accepted ZVODE steps:  ',zvodeaccpt
            write(ilog,'(a,I9)')&
                 'Number of rejected ZVODE steps:  ',zvoderejct
            write(ilog,*)
         endif
      elseif (lexact) then
         write(ilog,'(a)')&
              'Integration method used for the exact wavefunction :'
         if (inttypephi .eq. 1) then
            write(ilog,'(a)')&
                 'Adams-Bashforth-Moulton (ABM) predictor-corrector.'
            write(ilog,'(a,I2)') 'Integration order: ', intorderphi
         elseif (inttypephi .eq. 2) then
            write(ilog,'(a)')&
                 'Bulirsch-Stoer (BS) with polynomial extrapolation.'
            write(ilog,'(a,I2)') 'Optimal column: ',intorderphi
         elseif (inttypephi .eq. 3) then
            if(lcap .or. ldissin) then
               write(ilog,'(a)')&
                    'Short iterative Lanczos-Arnoldi with variable order.'
            else
               write(ilog,'(a)')&
                    'Short iterative Lanczos (real) with variable order.'
            endif
            write(ilog,'(a,I3)') 'Maximum order employed: ',maxsilorder
            write(ilog,'(a,I3)') 'Maximum order allowed:  ',intorderphi
         elseif (inttypephi .eq. 4) then
            write(ilog,'(a)')&
                 'Runge-Kutta order 5 (RK5) from ODESIM'
         elseif (inttypephi .eq. 41) then
            write(ilog,'(a)')&
                 'Runge-Kutta order 5 (RK5) from ODESIM. Fixed Time Step.'
         elseif (inttypephi .eq. 5) then
            write(ilog,'(a)')&
                 'Runge-Kutta order 8 (RK8) from ODESIM'
         elseif (inttypephi .eq. 9) then
            write(ilog,'(a)')&
                 'ZVODE'
         endif
         write(ilog,*)
         if (inttypephi .eq. 1) then
            write(ilog,'(a,I7)')&
                 'Total number of ABM steps:    ',abmsteps
            write(ilog,'(a,I7)')&
                 'Number of repeated ABM steps: ',repabmsteps
            write(ilog,*)
         endif
         if (inttypephi .eq. 2) then
            write(ilog,'(a,I7)')&
                 'Number of succesful large BS steps:        ',goodbssteps
            write(ilog,'(a,I7)')&
                 'Number of failed but fixed large BS steps: ',badbssteps
            write(ilog,'(a,I7)')&
                 'Total number of small BS steps:            ',smallbssteps
            write(ilog,*)
         endif
         if (inttypephi .eq. 3) then
            write(ilog,'(a,I7)')&
                 'Number of successful Lanczos steps:       ',goodsilsteps
            write(ilog,'(a,I7)')&
                 'Number of failed but fixed Lanczos steps: ',badsilsteps
            write(ilog,*)
         endif
         if (inttypephi .eq. 4) then
            write(ilog,'(a,I7)')&
                 'Total number of RK5 steps:     ',rk5steps
            write(ilog,'(a,I7)')&
                 'Number of accepted RK5 steps:  ',rk5accpt
            write(ilog,'(a,I7)')&
                 'Number of rejected RK5 steps:  ',rk5rejct
            write(ilog,*)
         endif
         if (inttypephi .eq. 41) then
            write(ilog,'(a,I7)')&
                 'Total number of RK5 steps:     ',rk5steps
            write(ilog,*)
         endif
         if (inttypephi .eq. 5) then
            write(ilog,'(a,I7)')&
                 'Total number of RK8 steps:     ',rk8steps
            write(ilog,'(a,I7)')&
                 'Number of accepted RK8 steps:  ',rk8accpt
            write(ilog,'(a,I7)')&
                 'Number of rejected RK8 steps:  ',rk8rejct
            write(ilog,*)
         endif
         if (inttypephi .eq. 9) then
            write(ilog,'(a,I7)')&
                 'Total number of ZVODE steps:     ',zvodesteps
            write(ilog,'(a,I7)')&
                 'Number of accepted ZVODE steps:  ',zvodeaccpt
            write(ilog,'(a,I7)')&
                 'Number of rejected ZVODE steps:  ',zvoderejct
            write(ilog,*)
         endif
         
      else
         call wim_one(ilog,inttyp,intord,liork, &
                      abmsteps,repabmsteps, &
                      smallbssteps,goodbssteps,badbssteps, &
                      rk5steps,rk5accpt,rk5rejct, &
                      rk8steps,rk8accpt,rk8rejct, &
                      zvodesteps,zvodeaccpt,zvoderejct)

      endif

      if (lconstvar .or. lcmfadaphi .or. lcmfadaa) then
         write(ilog,'(a)') '***** Update times *****'
         write(ilog,*)
         write(ilog,'(a,i5)') 'Number of successful update steps: ',&
            updatesteps
         write(ilog,'(a,i5)') 'Number of failed update steps:     ',&
            badupdatesteps
         if (rlxtime .eq. 'fs') then
            write(ilog,'(a,f12.5)') 'Mean update time (in fs):   ',&
               sumupdate/(updatesteps*fs)
            write(ilog,'(a,f12.5)') 'Standard deviation (in fs): ',&
               sqrt(abs(sum2update/updatesteps -&
                  (sumupdate/updatesteps)**2))/fs
         else if (rlxtime .eq. 'au') then
            write(ilog,'(a,f12.5)') 'Mean update time (in au):   ',&
               sumupdate/(updatesteps*fs)
            write(ilog,'(a,f12.5)') 'Standard deviation (in au): ',&
               sqrt(abs(sum2update/updatesteps -&
                  (sumupdate/updatesteps)**2))/fs
         endif
         write(ilog,*)
         if (badbackw .gt. 0) then
            write(ilog,'(a)') '***** Warning on Lanczos order *****'
            write(ilog,*)
            write(ilog,'(a,i4,a)') 'During the calculation, ',badbackw,&
                                   ' SIL backward propagations failed '
            write(ilog,'(a,a)')    '(but were fixed). You''d better ',&
                                   'increase the Lanczos order!'
            write(ilog,*)
         endif
      endif
      call myflush(ilog)

      return
    end subroutine writeintmethod

! ----------------------------------------------------------------------
! Write Integration Method for runs where one integrator is used for
! the full equation, i.e. VMF of ML-CMF=unified
! ----------------------------------------------------------------------
    subroutine wim_one(ilog,inttyp,intord,liork,&
                       abmsteps,repabmsteps,&
                       smallbssteps,goodbssteps,badbssteps,&
                       rk5steps,rk5accpt,rk5rejct,&
                       rk8steps,rk8accpt,rk8rejct,&
                       zvodesteps,zvodeaccpt,zvoderejct)

      implicit none

      integer(long)   :: ilog,inttyp,intord
      logical(kind=4) :: liork
      integer(long)   :: abmsteps,repabmsteps,&
                         smallbssteps,goodbssteps,badbssteps,&
                         rk5steps,rk5accpt,rk5rejct,&
                         rk8steps,rk8accpt,rk8rejct,&
                         zvodesteps,zvodeaccpt,zvoderejct

      write(ilog,'(a)') &
           'Integration method used for the complete ODE:'
      if (inttyp .eq. 1) then
         write(ilog,'(a)') &
              'Adams-Bashforth-Moulton (ABM) predictor-corrector.'
         write(ilog,'(a,I2)') 'Integration order: ',intord
         write(ilog,*)
         write(ilog,'(a,I7)') &
              'Total number of ABM steps:    ',abmsteps
         write(ilog,'(a,I7)') &
              'Number of repeated ABM steps: ',repabmsteps
         write(ilog,*)
      elseif (inttyp .eq. 2) then
         write(ilog,'(a)') &
              'Bulirsch-Stoer (BS) with polynomial extrapolation.'
         write(ilog,'(a,I2)') 'Optimal column: ',intord
         write(ilog,*)
         write(ilog,'(a,I7)') &
              'Number of successful large BS steps:       ',goodbssteps
         write(ilog,'(a,I7)') &
              'Number of failed but fixed large BS steps: ',badbssteps
         write(ilog,'(a,I7)') &
              'Total number of small BS steps:            ',smallbssteps
         write(ilog,*)
      elseif (inttyp .eq. 4) then
         write(ilog,'(a)') &
              'Runge-Kutta order 5 (RK5) from ODESIM'
         if(liork) write(ilog,'(a)')'with improved orthogonality'
         write(ilog,*)
         write(ilog,'(a,I7)') &
              'Total number of RK5 steps:     ',rk5steps
         write(ilog,'(a,I7)') &
              'Number of accepted RK5 steps:  ',rk5accpt
         write(ilog,'(a,I7)') &
              'Number of rejected RK5 steps:  ',rk5rejct
         write(ilog,*)
      elseif (inttyp .eq. 41) then
         write(ilog,'(a)')&
              'Runge-Kutta order 5 (RK5) from ODESIM. Fixed Time Step.'
         write(ilog,*)
         write(ilog,'(a,I7)') &
              'Total number of RK5 steps:     ',rk5steps
      elseif (inttyp .eq. 5) then
         write(ilog,'(a)') &
              'Runge-Kutta order 8 (RK8) from ODESIM'
         if(liork) write(ilog,'(a)')'with improved orthogonality'
         write(ilog,*)
         write(ilog,'(a,I7)') &
              'Total number of RK8 steps:     ',rk8steps
         write(ilog,'(a,I7)') &
              'Number of accepted RK8 steps:  ',rk8accpt
         write(ilog,'(a,I7)') &
              'Number of rejected RK8 steps:  ',rk8rejct
         write(ilog,*)
      elseif (inttyp .eq. 9) then
         write(ilog,'(a)')&
              'ZVODE'
         if(limportho) write(ilog,'(a)') &
             'with improved orthogonality (not implemented)'
         write(ilog,*)
         write(ilog,'(a,I7)')&
              'Total number of ZVODE steps:     ',zvodesteps
         write(ilog,'(a,I7)')&
              'Number of accepted ZVODE steps:  ',zvodeaccpt
         write(ilog,'(a,I7)')&
              'Number of rejected ZVODE steps:  ',zvoderejct
         write(ilog,*)

      else
         routine = 'writeintmethod'
         message = 'Illegal integration type'
         call errormsg
      endif
      return
    end subroutine wim_one


! *** V7.1 GW *************> SUBROUTINE PROP EX <***********************
!                           ~~~~~~~~~~~~~~~~~~~~
! Drives the propagation of the wavefunction in exact propagations.
!-------------------------------------------------------------------------------

      subroutine propex (psi,dtpsi,time,trystep,madestep,restartabm,&
                         restartrk5,restartbs,restartsil,restartzvode,abmsteps,&
                         repabmsteps,goodbssteps,badbssteps,&
                         smallbssteps,goodsilsteps,badsilsteps)

      implicit none

      complex(dop), dimension(dgldim), intent(inout) :: psi,dtpsi
      real(dop)                                      :: intperiod
      real(dop), intent(in)                          :: trystep,time
      real(dop), intent(out)                         :: madestep
      integer(long), intent(out)                     :: abmsteps,repabmsteps
      integer(long), intent(inout)                   :: goodbssteps,badbssteps,smallbssteps,goodsilsteps,badsilsteps
      logical(kind=4), intent(inout)                 :: restartabm,restartrk5,&
                                                        restartsil
      logical(kind=4), intent(inout)                 :: restartbs,restartzvode


      inttypea=1

! --- PROPAGATE MCTDH-COEFFICIENTS (FIRST HALF-STEP) ---

      intperiod=trystep
      call propphi(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
                   restartbs,restartzvode, &
                   abmsteps,repabmsteps,goodbssteps,badbssteps, &
                   smallbssteps,goodsilsteps,badsilsteps,restartsil)

      madestep = trystep

      return
      end  subroutine propex

! **********************************************************************

      end module prop1
