!----------------------------------------------------------------------
!              MODULE PROPBCASTMOD
!
! These routines broadcast, from the Master to the Slaves, the variables 
! needed during a propagation run. Takes commands that were previously 
! in runprop and puts them in one place.
! 
! GWR 11/12
!----------------------------------------------------------------------

      module propbcastmod

#ifdef MPI

use decimal, only: dop, long
use logdat

use mpi
use mpidata
use global
use globmemmod
use maxv, only: maxdim,maxsta
use dvrdatmod
use psidef
use datenmod
use runpropmod
use channels 
use griddatmod
use memhopsmodmod
use hpsimod
use operdef
use mpidata
use openmpmod

      implicit none

      integer(long), parameter                :: mpicommllen=22
      logical(kind=4), dimension(mpicommllen) :: mpicomml
      integer(long), parameter                :: mpicommilen=70
      integer(long), dimension(mpicommilen)   :: mpicommi
      integer(long) ,parameter                :: mpidimlen=16
      integer                                 :: mpidim(mpidimlen)

      private

      public :: slvbcast1,slvbcast2,mastbcast1,mastbcast2,dimbcast
      contains

!-----------------------------------------------------------------------
! Initial reception of data from master including that necessary to 
! allocate arrays.
!-----------------------------------------------------------------------
      subroutine slvbcast1(rank)

      integer(long), intent(in)      :: rank
! Check.
      if(rank.eq.master)then
         routine = 'slvbcast1'
         message = 'Master not supposed to be here'
         call errormsg
      endif

! ... scalar Parameters
      lmpistarted=.true.
      call MPI_Bcast(mpicomml,mpicommllen,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
      lolsen       = mpicomml(1)
      lrreal       = mpicomml(2)
      lmpi         = mpicomml(3)
      lmult        = mpicomml(4)
      lmpicalcha   = mpicomml(5)
      lmpifunka2   = mpicomml(6)
      lfunkr       = mpicomml(7)
      lcap         = mpicomml(8)
      lmpihlochphi = mpicomml(9)
      lquad        = mpicomml(10)
      lmpimfields  = mpicomml(11)
      lmpigetdavmat= mpicomml(12)
      lmpidsyev    = mpicomml(13)
      ldavkey      = mpicomml(14)
      lpartim      = mpicomml(15)
      lptimall     = mpicomml(16)
      lmpiphihphi  = mpicomml(17)
      lgwp         = mpicomml(18)
      lgwpsep      = mpicomml(19)
      lompthread   = mpicomml(20)
      lexact       = mpicomml(21)
      lconst       = mpicomml(22)
      call MPI_Bcast(mpicommi,mpicommilen,MPI_INTEGER,master,MPI_COMM_WORLD,&
           mpierr)
      adim         = mpicommi(1)
      maxblock     = mpicommi(2)
      mrworkr      = mpicommi(3)
      pmatdim      = mpicommi(5)
      fkoe         = mpicommi(6)
      nmode        = mpicommi(7)
      nstate       = mpicommi(8)
      ftime        = mpicommi(9)
      hlochdim     = mpicommi(10)
      mcworkc      = mpicommi(11)
      mchloch      = mpicommi(12)
      workcdim     = mpicommi(13)
      npackts      = mpicommi(14)
      intordera    = mpicommi(15)
      rlaxnum      = mpicommi(16)
      mcintpsi     = mpicommi(17)
      mrintr       = mpicommi(18)
      miworki      = mpicommi(19)
      precon       = mpicommi(20)
      intordermpi  = mpicommi(21)
      workrdim     = mpicommi(22)
      mpb          = mpicommi(23)
      laenge       = mpicommi(24)
      mchpsi       = mpicommi(25)
      hpsidim      = mpicommi(26)
      mchc         = mpicommi(27)
      hcdim        = mpicommi(28)
      mrhr         = mpicommi(29)
      hrdim        = mpicommi(30)
      mihi         = mpicommi(31)
      hidim        = mpicommi(32)
      mlhl         = mpicommi(33)
      hldim        = mpicommi(34)
      dgldim       = mpicommi(35)
      hrhops       = mpicommi(36)
      hifftfak     = mpicommi(37)
      hchin        = mpicommi(38)
      hcrueck      = mpicommi(39)
      hcexphin     = mpicommi(40)
      hcexprueck   = mpicommi(41)
      hcworkc      = mpicommi(42)
      maxgdim      = mpicommi(43)
      maxphidim    = mpicommi(44)
      sdim         = mpicommi(45)
      hopsdim      = mpicommi(46)
      fftdim       = mpicommi(47)
      expdim       = mpicommi(48)
      intpsidim    = mpicommi(49)
      mcdim        = mpicommi(50)
      mrdim        = mpicommi(51)
      midim        = mpicommi(52)
      mldim        = mpicommi(53)
      mfsumdim     = mpicommi(54)
      ikryc        = mpicommi(56)
      ikrylov      = mpicommi(57)
      citype       = mpicommi(58)
      jindxdim     = mpicommi(59)
      ghmatdim     = mpicommi(60)
      fmblock      = mpicommi(62)
      workidim     = mpicommi(63)
      psitype      = mpicommi(64)
      ilmr         = mpicommi(65)
      mijindx      = mpicommi(66)
      totphidim    = mpicommi(67)
      ompthread    = mpicommi(68)
      mcphi        = mpicommi(69)
      mrort        = mpicommi(70)

      return
      end subroutine slvbcast1

!-----------------------------------------------------------------------
! Reception of remaining data from the master, once arrays have been
! allocated.
!-----------------------------------------------------------------------
      subroutine slvbcast2(rank)

      integer(long), intent(in)                      :: rank

! Check.
      if(rank.eq.master)then
         routine = 'slvbcast2'
         message = 'Master not supposed to be here'
         call errormsg
      endif

! Carry on with the broadcast.

      call MPI_Bcast(name,laenge,MPI_CHARACTER,master,MPI_COMM_WORLD,mpierr)
! ... dyanmic SPF flag
      call MPI_Bcast(ldynspf,1,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
! ... Vectors for funka2
      if (dentype .eq. 0 .or. dentype .eq. 2) then
         call MPI_Bcast(ndim,maxdim*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(dim,maxdim*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(vdim,maxdim*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      else if (dentype .eq. 1) then
         call MPI_Bcast(nddim,maxdim*maxsta*maxsta,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(ddim,maxdim*maxsta*maxsta,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(vddim,maxdim*maxsta*maxsta,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
      endif
      call MPI_Bcast(pmat,maxdim*maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(koeff,maxkoe,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmtype,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hamilton,maxdim*maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(kzahl,maxham,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(zham,maxham,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(diag,maxdim*maxkoe,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(lklha,1,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(khtot,maxham,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)

      call MPI_Bcast(ndof,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(gwpm,maxdim,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
      if (lgwp) then
         call MPI_Bcast(zdgwp,maxdim*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(ghmat,maxkoe*maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(gwpdim,maxsta*maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      endif

! ... Additional Vectors for calcha
      call MPI_Bcast(ki,maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(kf,maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      if (dentype .eq. 0) then
         call MPI_Bcast(zpsi,maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(block,maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      else if (dentype .eq. 1) then
         call MPI_Bcast(dzpsi,maxsta*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,&
              mpierr)
         call MPI_Bcast(dblock,maxsta*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,&
              mpierr)
      else if (dentype .eq. 2) then
         call MPI_Bcast(dzpsi,maxsta*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,&
              mpierr)
         call MPI_Bcast(block,maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,&
              mpierr)
      endif
! ... Additional Vectors for mfields
      call MPI_Bcast(phloch,maxkoe*maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hlsym,maxkoe*maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(complete,maxdim,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmfac,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hterm,maxfac*maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmdof,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmmode,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmmuld,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(nmulddof,maxmuld,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mulddof,maxdim*maxmuld,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mulddof2,maxdim*maxmuld,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(nspfdof,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(spfdof,maxdim*maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hopfile,maxhop,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hopilab,maxhop,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hopipar,maxhpar*maxhop,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hoprpar,maxhpar*maxhop,MPI_DOUBLE_PRECISION,master,&
           MPI_COMM_WORLD,mpierr)
! ... Additional Vectors for getdavmat
      call MPI_Bcast(khzahl,maxdim*maxsta*maxham,MPI_INTEGER,master,&
           MPI_COMM_WORLD,mpierr)
! ... Additional Vectors for phihphi
      call MPI_Bcast(kfirst,maxdim*maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      if (dentype .eq. 0 .or. dentype .eq. 2) then
         call MPI_Bcast(zetf,maxdim*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      else if (dentype .eq. 1) then
         call MPI_Bcast(dzetf,maxdim*maxsta*maxsta,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
      endif
      call MPI_Bcast(subdim,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hsym,maxdim*maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(dofspf,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(zhop,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(vgdim,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(gdim,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(ngdim,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(zfft,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(zexp,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hamLMR,maxLMR*3,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(zhpsi,maxdim*maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(basis,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(ipbaspar,mbaspar*maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmshift,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmndof,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmadof,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
! ... Additional Vectors for hlochphi
      call MPI_Bcast(autocap,maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(kinop,maxdim*maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(nmulpot,maxham,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
! ... Additional Vectors for summf
      call MPI_Bcast(ihtmshift,maxnhtmshift*maxdim,MPI_INTEGER,master,&
           MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(nhtmshift,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
   
      if (dentype .eq. 0 .or. dentype .eq. 2) then
         call MPI_Bcast(zmfsum,maxdim*maxsta*maxsta,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
      else if (dentype .eq. 1) then
         call MPI_Bcast(dzmfsum,maxdim*maxsta*maxsta*maxsta,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
      endif
      call MPI_Bcast(zmfsum1,maxdim*maxsta*maxsta*maxnhtmshift,&
                     MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(idmode,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)

! ... Hamiltonian operators and DVR arrays
      call MPI_Bcast(usehsym,1,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mrhops,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mchin,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mcrueck,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mifftfak,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mcexphin,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mcexprueck,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mr(mrhops),hopsdim,MPI_DOUBLE_PRECISION,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mc(mchin),fftdim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mc(mcrueck),fftdim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mi(mifftfak),fftdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mc(mcexphin),expdim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mc(mcexprueck),expdim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)

      return

      end subroutine slvbcast2

!-----------------------------------------------------------------------
! Broadcast from Master to slaves.
!-----------------------------------------------------------------------

      subroutine mastbcast1(rank)

      integer(long), intent(in)             :: rank

! Check.
      if(rank.ne.master)then
         routine = 'propbcast'
         message = 'Slave not supposed to be here'
         call errormsg
      endif

      lmpistarted=.true.
! ... Scalar Parameters
      mpicomml(1) = lolsen        
      mpicomml(2) = lrreal       
      mpicomml(3) = lmpi         
      mpicomml(4) = lmult        
      mpicomml(5) = lmpicalcha   
      mpicomml(6) = lmpifunka2   
      mpicomml(7) = lfunkr       
      mpicomml(8) = lcap         
      mpicomml(9) = lmpihlochphi 
      mpicomml(10)= lquad        
      mpicomml(11)= lmpimfields  
      mpicomml(12)= lmpigetdavmat
      mpicomml(13)= lmpidsyev    
      mpicomml(14)= ldavkey      
      mpicomml(15)= lpartim      
      mpicomml(16)= lptimall     
      mpicomml(17)= lmpiphihphi  
      mpicomml(18)= lgwp         
      mpicomml(19)= lgwpsep      
      mpicomml(20)= lompthread
      mpicomml(21)= lexact
      mpicomml(22)= lconst

      call MPI_Bcast(mpicomml,mpicommllen,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)

      mpicommi(1) = adim         
      mpicommi(2) = maxblock     
      mpicommi(3) = mrworkr      
      mpicommi(4) = 0
      mpicommi(5) = pmatdim      
      mpicommi(6) = fkoe         
      mpicommi(7) = nmode        
      mpicommi(8) = nstate       
      mpicommi(9) = ftime        
      mpicommi(10)= hlochdim     
      mpicommi(11)= mcworkc      
      mpicommi(12)= mchloch      
      mpicommi(13)= workcdim     
      mpicommi(14)= npackts      
      mpicommi(15)= intordera    
      mpicommi(16)= rlaxnum      
      mpicommi(17)= mcintpsi     
      mpicommi(18)= mrintr       
      mpicommi(19)= miworki      
      mpicommi(20)= precon       
      mpicommi(21)= intordermpi  
      mpicommi(22)= workrdim     
      mpicommi(23)= mpb          
      mpicommi(24)= laenge       
      mpicommi(25)= mchpsi       
      mpicommi(26)= hpsidim      
      mpicommi(27)= mchc         
      mpicommi(28)= hcdim        
      mpicommi(29)= mrhr         
      mpicommi(30)= hrdim        
      mpicommi(31)= mihi         
      mpicommi(32)= hidim        
      mpicommi(33)= mlhl         
      mpicommi(34)= hldim        
      mpicommi(35)= dgldim       
      mpicommi(36)= hrhops       
      mpicommi(37)= hifftfak     
      mpicommi(38)= hchin        
      mpicommi(39)= hcrueck      
      mpicommi(40)= hcexphin     
      mpicommi(41)= hcexprueck   
      mpicommi(42)= hcworkc      
      mpicommi(43)= maxgdim      
      mpicommi(44)= maxphidim    
      mpicommi(45)= sdim         
      mpicommi(46)= hopsdim      
      mpicommi(47)= fftdim       
      mpicommi(48)= expdim       
      mpicommi(49)= intpsidim    
      mpicommi(50)= mcdim        
      mpicommi(51)= mrdim        
      mpicommi(52)= midim        
      mpicommi(53)= mldim        
      mpicommi(54)= mfsumdim     
      mpicommi(56)= ikryc        
      mpicommi(57)= ikrylov      
      mpicommi(58)= citype       
      mpicommi(59)= jindxdim     
      mpicommi(60)= ghmatdim     
      mpicommi(61)= 0
      mpicommi(62)= fmblock      
      mpicommi(63)= workidim     
      mpicommi(64)= psitype      
      mpicommi(65)= ilmr
      mpicommi(66)= mijindx
      mpicommi(67)= totphidim
      mpicommi(68)= ompthread
      mpicommi(69)= mcphi
      mpicommi(70)= mrort

      call MPI_Bcast(mpicommi,mpicommilen,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)

      return

      end subroutine mastbcast1

!-----------------------------------------------------------------------
! Broadcast from Master to slaves.
!-----------------------------------------------------------------------

      subroutine mastbcast2(rank)

      integer(long), intent(in)             :: rank

! Check.
      if(rank.ne.master)then
         routine = 'propbcast'
         message = 'Slave not supposed to be here'
         call errormsg
      endif

      call MPI_Bcast(name,laenge,MPI_CHARACTER,master,MPI_COMM_WORLD,mpierr)
! ... dyanmic SPF flag
      call MPI_Bcast(ldynspf,1,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)

! ... Vectors for funka2
      if (dentype .eq. 0 .or. dentype .eq. 2) then
         call MPI_Bcast(ndim,maxdim*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(dim,maxdim*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(vdim,maxdim*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      else if (dentype .eq. 1) then
         call MPI_Bcast(nddim,maxdim*maxsta*maxsta,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(ddim,maxdim*maxsta*maxsta,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(vddim,maxdim*maxsta*maxsta,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
      endif
      call MPI_Bcast(pmat,maxdim*maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(koeff,maxkoe,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmtype,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hamilton,maxdim*maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(kzahl,maxham,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(zham,maxham,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(diag,maxdim*maxkoe,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(lklha,1,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(khtot,maxham,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)

      call MPI_Bcast(ndof,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(gwpm,maxdim,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
      if (lgwp) then
         call MPI_Bcast(zdgwp,maxdim*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(ghmat,maxkoe*maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(gwpdim,maxsta*maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      endif

! ... Additional Vectors for calcha
      call MPI_Bcast(ki,maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(kf,maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      if (dentype .eq. 0) then
         call MPI_Bcast(zpsi,maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(block,maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      else if (dentype .eq. 1) then
         call MPI_Bcast(dzpsi,maxsta*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,&
              mpierr)
         call MPI_Bcast(dblock,maxsta*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,&
              mpierr)
      else if (dentype .eq. 2) then
         call MPI_Bcast(dzpsi,maxsta*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,&
              mpierr)
         call MPI_Bcast(block,maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,&
              mpierr)
      endif
! ... Additional Vectors for mfields
      call MPI_Bcast(phloch,maxkoe*maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hlsym,maxkoe*maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(complete,maxdim,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmfac,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hterm,maxfac*maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmdof,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmmode,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmmuld,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(nmulddof,maxmuld,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mulddof,maxdim*maxmuld,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mulddof2,maxdim*maxmuld,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(nspfdof,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(spfdof,maxdim*maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hopfile,maxhop,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hopilab,maxhop,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hopipar,maxhpar*maxhop,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hoprpar,maxhpar*maxhop,MPI_DOUBLE_PRECISION,master, &
           MPI_COMM_WORLD,mpierr)
! ... Additional Vectors for getdavmat
      call MPI_Bcast(khzahl,maxdim*maxsta*maxham,MPI_INTEGER,master, &
           MPI_COMM_WORLD,mpierr)
! ... Additional Vectors for phihphi
      call MPI_Bcast(kfirst,maxdim*maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      if (dentype .eq. 0 .or. dentype .eq. 2) then
         call MPI_Bcast(zetf,maxdim*maxsta,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      else if (dentype .eq. 1) then
         call MPI_Bcast(dzetf,maxdim*maxsta*maxsta,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
      endif
      call MPI_Bcast(subdim,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hsym,maxdim*maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(dofspf,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(zhop,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(vgdim,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(gdim,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(ngdim,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(zfft,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(zexp,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hamLMR,maxLMR*3,MPI_INTEGER,0,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(zhpsi,maxdim*maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(basis,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(ipbaspar,mbaspar*maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmshift,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmndof,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(htmadof,maxhtm,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
! ... Additional Vectors for hlochphi
      call MPI_Bcast(autocap,maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(kinop,maxdim*maxkoe,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(nmulpot,maxham,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)    
! ... Additional Vectors for summf
      call MPI_Bcast(ihtmshift,maxnhtmshift*maxdim,MPI_INTEGER,master,&
           MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(nhtmshift,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      if (dentype .eq. 0 .or. dentype .eq. 2) then
         call MPI_Bcast(zmfsum,maxdim*maxsta*maxsta,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
      else if (dentype .eq. 1) then
         call MPI_Bcast(dzmfsum,maxdim*maxsta*maxsta*maxsta,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
      endif
      call MPI_Bcast(zmfsum1,maxdim*maxsta*maxsta*maxnhtmshift,MPI_INTEGER,&
           master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(idmode,maxdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)

! ... Hamiltonian operators and DVR arrays
      call MPI_Bcast(usehsym,1,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mrhops,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mchin,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mcrueck,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mifftfak,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mcexphin,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mcexprueck,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mr(mrhops),hopsdim,MPI_DOUBLE_PRECISION,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mc(mchin),fftdim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mc(mcrueck),fftdim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mi(mifftfak),fftdim,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mc(mcexphin),expdim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(mc(mcexprueck),expdim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)

      return

      end subroutine mastbcast2

!-----------------------------------------------------------------------

      subroutine dimbcast(rank)

      integer(long), intent(in)             :: rank

      if(rank.eq.0)then
         mpidim(1) = maxdim
         mpidim(2) = maxsta
         mpidim(3) = maxkoe
         mpidim(4) = maxhtm
         mpidim(5) = maxhop
         mpidim(6) = maxfac
         mpidim(7) = maxdef
         mpidim(8) = maxmuld
         mpidim(9) = maxham
         mpidim(10)= maxhpar
         mpidim(11)= maxpes
         mpidim(12)= maxrdf
         mpidim(13)= maxLMR
         mpidim(14)= maxpar
         mpidim(15)= maxtape
         mpidim(16)= maxnhtmshift
         call MPI_Bcast(mpidim,mpidimlen,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      else
         call MPI_Bcast(mpidim,mpidimlen,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         maxdim      = mpidim(1)
         maxsta      = mpidim(2)
         maxkoe      = mpidim(3)
         maxhtm      = mpidim(4)
         maxhop      = mpidim(5)
         maxfac      = mpidim(6)
         maxdef      = mpidim(7)
         maxmuld     = mpidim(8)
         maxham      = mpidim(9)
         maxhpar     = mpidim(10)
         maxpes      = mpidim(11)
         maxrdf      = mpidim(12)
         maxLMR      = mpidim(13)
         maxpar      = mpidim(14)
         maxtape     = mpidim(15)
         maxnhtmshift= mpidim(16)
      endif
      if(mpierr.ne.0) write(6,'(a,2i5)') &
                'MPI error in dimbcast: mpirank, mpierr',mpirank,mpierr

      end subroutine dimbcast

#endif

      end module propbcastmod
