! **********************************************************************
!                                                                       
!                 module PROPCMFADA
!                                                                       
!   MCTDH module containing the routines for adaptive stepsize CMF
!   integration
!                                                                       
!   Contains routines: propcmfada propcmfadarlx propcmfadaphi propcmfadaa
!                                                                       
! **********************************************************************
      module propcmfadamod

use decimal, only: long, dop
use globmemmod
use dvrdatmod
use mmlib,   only: mmaxzz, mqxazz, dqxxdz2, mqxazz, qqaxzz, mqxxzz
use op1lib,  only: cpvxz, normvxz, cpmxz
use schmidtorthomod, only: schmidtortho2

use global
use maxv, only: maxdim
use griddatmod
use operdef
use psidef
use datenmod
use runpropmod
use channels

use function
use prop1
use symlib
use evaluate, only: calcenergy
use multilayer, only: errortotml
use trafonatmod, only: reorthopsi
use timing
! test
!use gapfuncsmod, only: gapdrv
! test
use gwplib, only: renormdgwp

      implicit none
      private
      public :: propcmfada,propcmfadaa,propcmfadarlx,propcmfadaphi,&
                propcmfada2
      contains

! *** V6.2 MB **********> SUBROUTINE PROP CMF ADA <*********************
!                        ~~~~~~~~~~~~~~~~~~~~~~~~~
! Drives the propagation of the wavefunction in constant mean-field
! (CMF) mode with adaptive (ADA) update time.
! **********************************************************************

      subroutine propcmfada (psi,dtpsi,time,timesinceout, &
                             newupdatetime,madestep,restartabm, &
                             restartrk5,restartbs,restartbs2,restartsil, &
                             restartzvode,abmsteps, &
                             repabmsteps,goodbssteps,badbssteps, &
                             smallbssteps,goodsilsteps,badsilsteps, &
                             sumupdate,sum2update,updatesteps, &
                             badupdatesteps,badbackw,Krylov,HKrylov,Kryc,HKryc)
     
      implicit none

      real(dop), parameter                      :: epscorr = 1.0_dop

      complex(dop), dimension(dgldim), intent(inout) :: psi,dtpsi
      complex(dop), dimension(ikryc), intent(inout)  :: Kryc,HKryc
      real(dop), dimension(ikrylov), intent(inout)   :: Krylov,HKrylov
      real(dop)                                      :: intperiod,localtime, &
                                                        deltasum,remaintime, &
                                                        erra,epsa,epsphi, &
                                                        newupdtimephi, &
                                                        newupdtimea
      real(dop), intent(in)                          :: timesinceout,time
      real(dop), intent(out)                         :: newupdatetime,madestep
      real(dop), intent(inout)                       :: sumupdate,sum2update
      integer(long)                                  :: oldbadsilsteps
      integer(long), intent(inout)                   :: abmsteps,repabmsteps, &
                                                        goodbssteps, &
                                                        badbssteps, &
                                                        smallbssteps,&
                                                        badbackw,goodsilsteps,&
                                                        badsilsteps,updatesteps
      integer(long), intent(inout)                   :: badupdatesteps
      logical(kind=4), intent(inout)                 :: restartabm,restartrk5
      logical(kind=4), intent(inout)                 :: restartsil,restartbs, &
                                                        restartbs2,restartzvode
      complex(dop), dimension(:), allocatable        :: agpred

      real(dop), save :: maxupdate = 1.0e99_dop
!
! Allocate local memory
!
      if (lgwp) then
         allocate(agpred(agmatdim))
      endif

!      if(lddmctdh .and. &
!           abs(timesinceout).lt.tiny .and. &
!           abs(time-tinit*fs).gt.tiny) call gapdrv(time,psi,mr(mrort),.true.)

! --- SET ACCURACIES ---
      epsa = epscorr*epsupdate
      epsphi = epsupdate

! --- SAVE OLD VALUES ---
      call cpvxz(psi,mc(mcoldpsi),dgldim)

! --- SAVE SPFS IN PHI ARRAY (for multi-D H terms)
      if (nmulpot(1) .gt. 0 .or. lklha) then
         call cpvxz(psi(adim+1),mc(mcphi),totphidim)
      endif

 100  continue

! --- PROPAGATE MCTDH-COEFFICIENTS (FIRST HALF-STEP) ---
      intperiod = 0.5_dop*updatetime

      call propa(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
                 restartbs2,restartsil,restartzvode, &
                 abmsteps,repabmsteps,goodbssteps,badbssteps, &
                 smallbssteps,goodsilsteps,badsilsteps,Krylov,HKrylov,Kryc,HKryc)

! --- PROPAGATE PHI (PREDICTOR STEP) ---
      intperiod = 0.5_dop*updatetime
      call propphi(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
                   restartbs,restartzvode, &
                   abmsteps,repabmsteps,goodbssteps,badbssteps, &
                   smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- SAVE PREDICTED SPF'S AND PROPAGATED A-VECTOR ---
      call cpvxz(psi,mc(mcpredpsi),dgldim)
      if (lgwp) call cpvxz(agmat,agpred,agmatdim)

! --- COMPUTE PREDICTED MEAN-FIELDS ---
      localtime = time+0.5_dop*updatetime
!     if(lddmctdh) then
!         call ddmfunk0(localtime,psi,dtpsi,0)
!      else
      call funk0(localtime,psi,dtpsi)

! --- RE-ESTABLISH OLD SPF'S AND COMPUTE THEIR TIME DERIVATIVE ---
      call cpvxz(mc(mcoldpsi+adim),psi(adim+1),totphidim)
!     if(lddmctdh) then
!         call ddmfunk1(time,psi,dtpsi)
!      else
      call funk1(time,psi,dtpsi)

! --- PROPAGATE PHI (CORRECTOR STEP) ---
      intperiod = 0.5_dop*updatetime
      call propphi(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
                   restartbs,restartzvode, &
                   abmsteps,repabmsteps,goodbssteps,badbssteps, &
                   smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- ESTIMATE ERROR OF CONSTANT MEAN-FIELD PROPAGATION ---
      if (dentype .eq. 1) then
         call errorphi1(deltasum,psi(adim+1),mc(mcpredpsi+adim), &
              dicht2)
      else
          call errorphi(deltasum,psi(adim+1),mc(mcpredpsi+adim), &
              dicht2,agmat,agpred)
      endif

! --- CALCULATE NEW UPDATE-TIME AND REPEAT STEP IF NECCESSARY ---
      if (deltasum .gt. 2.0_dop*epsphi) then

!    --- OUTPUT FAILED UPDATE TIME AND DETERMINE NEW ONE ---
         if (lupdate) then
            write(iupd,'(i6,36x,e18.6e3)') updatesteps+1,deltasum
            call myflush(iupd)
         endif
         badupdatesteps = badupdatesteps+1
         remaintime = out1-timesinceout
         updatetime = 0.7_dop*updatetime*(epsphi/deltasum)**0.25_dop
         updatetime = remaintime &
                      /nint(remaintime/min(updatetime,remaintime))

!    --- RE-ESTABLISH OLD PSI-VECTOR AND RECOMPUTE TIME DERIVATIVE ---

         call cpvxz(mc(mcoldpsi),psi,dgldim)
!        if(lddmctdh) then
!            call ddmfunk2(time,psi)
!            call ddmfunk3(time,psi,dtpsi)
!        else
         call funk2(time,psi)
         call funk3(time,psi,dtpsi)

!    --- REPEAT LAST STEP ---
         goto 100
      endif

! --- COMPUTE TIME DERIVATIVE OF SINGLE-PARTICLE FUNCTIONS ---

      localtime = time+0.5_dop*updatetime
!     if(lddmctdh) then
!         call ddmfunk1(localtime,psi,dtpsi)
!     else
      call funk1(localtime,psi,dtpsi)

! --- PROPAGATE PHI (SECOND HALF-STEP) ---

      intperiod = 0.5_dop*updatetime
      localtime = time+0.5_dop*updatetime
      call propphi(psi,dtpsi,localtime,intperiod,restartabm,restartrk5,&
                   restartbs,restartzvode, &
                   abmsteps,repabmsteps,goodbssteps,badbssteps, &
                   smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- SAVE NEW SPFS IN PHI ARRAY (for multi-D H terms)
      if (nmulpot(1) .gt. 0 .or. lklha) then
         call cpvxz(psi(adim+1),mc(mcphi),totphidim)
      endif

! --- COMPUTE TIME DERIVATIVE OF SINGLE-PARTICLE FUNCTIONS ---
! --- DETERMINE MATRIX ELEMENTS AND DERIVATIVE OF A-VECTOR ---

!      if (lddmctdh) then
!         localtime = time+updatetime
!         call ddmfunk2(localtime,psi)
!         localtime = time+0.5_dop*updatetime
!         call ddmfunka(localtime,psi,dtpsi)
!     else
      localtime = time+updatetime
      call funk2(localtime,psi)
      localtime = time+0.5_dop*updatetime
      call funka(localtime,psi,dtpsi)


! --- PROPAGATE MCTDH-COEFFICIENTS (SECOND HALF-STEP) ---

      oldbadsilsteps = badsilsteps
      intperiod = 0.5_dop*updatetime
      call propa(psi,dtpsi,localtime,intperiod,restartabm,restartrk5,&
                 restartbs2,restartsil,restartzvode, &
                 abmsteps,repabmsteps,goodbssteps,badbssteps, &
                 smallbssteps,goodsilsteps,badsilsteps,Krylov,HKrylov,Kryc,HKryc)

! --- REPEAT STEP IF LANCZOS STEP FAILED BUT HAS BEEN FIXED ---

! If the Lanczos integrator didn't converge within the maximum order but
! needed more than one step, the Krylov space doesn't belong to
! psi(localtime) anymore. In that case, the following back-propagation
! which uses the "old" Krylov vectors would lead to wrong results, so
! decrease the update-interval and repeat the step.

      if (badsilsteps .gt. oldbadsilsteps) then

!    --- REDUCE UPDATE-INTERVAL AND WRITE TO UPDATE FILE ---

         if (lupdate) then
            write(iupd,'(a1,i5,3x,a1,f14.8,a)') '#', updatesteps+1, &
            '(',updatetime/fs,')    SIL failed! Step will be repeated.'
            call myflush(iupd)
         endif
         maxupdate=0.75_dop*updatetime
         updatetime=0.5_dop*updatetime
         badbackw=badbackw+1

!    --- RE-ESTABLISH OLD PSI-VECTOR AND RECOMPUTE TIME DERIVATIVE ---

         call cpvxz(mc(mcoldpsi),psi,dgldim)
!        if(lddmctdh) then
!            call ddmfunk2(time,psi)
!            call ddmfunk3(time,psi,dtpsi)
!        else
         call funk2(time,psi)
         call funk3(time,psi,dtpsi)

!    --- REPEAT LAST STEP ---

         goto 100
      endif

! --- BACK-PROPAGATE MCTDH-COEFFICIENTS (FOR ERROR ESTIMATE) ---

! Be careful: the "intpsi" array mc(mcintpsi) containing the Krylov vectors 
! must not have been overwritten since the last call of the Lanczos integrator!

      intperiod = -0.5_dop*updatetime
      restartsil = .false.
      call propa(mc(mcpredpsi),dtpsi,localtime,intperiod,restartabm, &
                 restartrk5,restartbs2,restartsil,restartzvode, &
                 abmsteps,repabmsteps,goodbssteps, &
                 badbssteps,smallbssteps,goodsilsteps,badsilsteps, &
                 Krylov,HKrylov,Kryc,HKryc)
      restartsil = .true.

! --- ESTIMATE ERROR OF A-VECTOR DUE TO CONSTANT MEAN-FIELDS ---

      call errora(erra,mc(mcpredpsi),mc(mcoldpsi))

! --- CALCULATE NEW UPDATE-TIME AND REPEAT STEP IF NECCESSARY ---

      if (erra .gt. 2.0_dop*epsa) then

!    --- OUTPUT FAILED UPDATE TIME AND DETERMINE NEW ONE ---

         if (lupdate) then
            write(iupd,'(i6,18x,e18.6e3)') updatesteps+1,erra
            call myflush(iupd)
         endif
         badupdatesteps = badupdatesteps+1
         remaintime = out1-timesinceout
         updatetime = 0.7_dop*updatetime*(epsa/erra)**0.25_dop
         updatetime = remaintime&
                      /nint(remaintime/min(updatetime,remaintime))

!    --- RE-ESTABLISH OLD PSI-VECTOR AND RECOMPUTE TIME DERIVATIVE ---

         call cpvxz(mc(mcoldpsi),psi,dgldim)
!        if(lddmctdh) then
!            call ddmfunk2(time,psi)
!            call ddmfunk3(time,psi,dtpsi)
!        else
         call funk2(time,psi)
         call funk3(time,psi,dtpsi)

!    --- REPEAT LAST STEP ---

         goto 100
      else

!    --- OUTPUT ACCEPTED UPDATE TIME AND DETERMINE NEW ONE ---

         sumupdate = sumupdate+updatetime
         sum2update = sum2update+updatetime**2
         updatesteps = updatesteps+1
         if (lupdate) then
            write(iupd,'(i6,f18.8,e18.6e3,e18.6e3,f14.4)')&
            updatesteps,updatetime/fs,erra,deltasum,time/fs
            call myflush(iupd)
         end if
         erra = erra+0.05_dop*epsa
         deltasum = deltasum+0.05_dop*epsphi
         newupdtimephi = updatetime*(epsphi/deltasum)**0.25_dop
         newupdtimea = updatetime*(epsa/erra)**0.25_dop
         newupdatetime = min(newupdtimea,newupdtimephi,maxupdate)
      endif

! --- RETURN THE INTERVAL THAT HAS ACTUALLY BEEN INTEGRATED ---

      madestep = updatetime

      call myflush(iupd)

      if (lgwp) then
         deallocate(agpred)
      endif

      return
      end subroutine propcmfada                      ! of propcmfada


! *** V8.2 HDM *********> SUBROUTINE PROP CMF ADA RLX <******************
!                       ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
! Drives the 'propagation' of the wavefunction in constant mean-field
! (CMF) mode with adaptive (ADA) update time controlled by the spf's
! error only. Special routine for "relaxation to excited states".
! ------------------------------------------------------------------

      subroutine propcmfadarlx (psi,dtpsi,time,timesinceout, &
                                newupdatetime,madestep,restartabm,restartrk5, &
                                restartbs,restartbs2,restartsil,restartzvode, &
                                abmsteps,repabmsteps,goodbssteps, &
                                badbssteps,smallbssteps,goodsilsteps, &
                                badsilsteps,sumupdate,sum2update, &
                                updatesteps,badupdatesteps,oldenergy, &
                                Krylov,HKrylov,Kryc,HKryc)

      implicit none

      complex(dop), dimension(dgldim), intent(inout) :: psi,dtpsi
      complex(dop), dimension(ikryc), intent(inout)  :: Kryc,HKryc
      real(dop), dimension(ikrylov), intent(inout)   :: Krylov,HKrylov

      real(dop)                                      :: intperiod,localtime,&
                                                        deltasum,remaintime,&
                                                        epsphi,energy,mnorm
      real(dop), dimension(maxdim)                   :: norm1,norm2
      real(dop), intent(in)                          :: timesinceout
      real(dop), intent(out)                         :: newupdatetime,madestep
      real(dop), intent(inout)                       :: sumupdate,sum2update
      real(dop), intent(in)                          :: time
      real(dop), intent(inout)                       :: oldenergy
      real(dop), dimension(maxspf)                   :: nsym2dnorm
      integer(long)                                  :: f,m,s,j,ierr
      integer(long)                                  :: icount,symcount
      integer(long), intent(inout)                   :: abmsteps,repabmsteps,&
                                                        goodbssteps,badbssteps,&
                                                        smallbssteps,updatesteps,&
                                                        goodsilsteps,badsilsteps
      integer(long), intent(inout)                   :: badupdatesteps
      logical(kind=4)                                :: zwischen
      logical(kind=4), intent(inout)                 :: restartabm,restartrk5,&
                                                        restartsil
      logical(kind=4), intent(inout)                 :: restartbs,restartbs2,&
                                                        restartzvode
      character(len=c1) :: str,str1


! --- SET ACCURACIES ---

      epsphi = epsupdate
      icount=0

! --- Write A-energy to rlx_info file  ---

      call calcenergy (psi,dtpsi,energy,0)
      if (rlxunit .eq. 'ev') then
         str=' eV'
         str1=' meV'
      else if (rlxunit .eq. 'au') then
         str=' au'
         str1=' mau'
      endif
      if(inttypea.eq.6) then     ! DAV
         if(abs(energy-oldenergy)*ev .gt. 2.d0) then
            write(irlx,'(a,f10.3,f18.10,a3,",",f16.10,a4)') &
              'After Davidson   T =',time/fs,energy*ev,str, &
              (energy-oldenergy)*ev,str1
         else
            write(irlx,'(a,f10.3,f18.10,a3,",",f16.10,a4)') &
              'After Davidson   T =',time/fs,energy*ev, str,&
              (energy-oldenergy)*ev*1.d+3,str1
         endif
      else
         write(irlx,'(a,f10.3,f18.10,a3,",",f16.10,a4)') &
              'After  Lanczos   T =',time/fs,-energy*ev, &
               (oldenergy-energy)*ev*1.0e+3_dop
      endif
      oldenergy = energy
      call myflush(irlx)

! --- SAVE OLD VALUES ---

      call cpvxz(psi,mc(mcoldpsi),dgldim)
      call cpvxz(psi(adim+1),mc(mcpredpsi),totphidim)

! --- SAVE SPFS IN PHI ARRAY (for multi-D H terms)
      if (nmulpot(1) .gt. 0 .or. lklha) then
         call cpvxz(psi(adim+1),mc(mcphi),totphidim)
      endif

 100  continue
      icount=icount+1

! ---- THIS IS FOR CHECKING PHI-DOT
      if(lquadphi) then
         do m=1,nmode
            currentmode=m 
            call normvxz(dtpsi(zetf(m,1)),norm1(m),dim(m,1)*subdim(m))
            call funkphi2(time,psi(zetf(m,1)),dtpsi(zetf(m,1)))
            call normvxz(dtpsi(zetf(m,1)),norm2(m),dim(m,1)*subdim(m))
         enddo
         write(irlx,'(a,f11.3,16e9.2)') ':: ',time/fs,(norm1(m),m=1,nmode)
         write(irlx,'(a,f11.3,16e9.2)') '.. ',time/fs,(norm2(m),m=1,nmode)
      else
         do m=1,nmode
            currentmode=m
            call normvxz(dtpsi(zetf(m,1)),norm1(m),dim(m,1)*subdim(m))
         enddo
         write(irlx,'(a,f11.3,16e9.2)') ':: ',time/fs,(norm1(m),m=1,nmode)
      endif

! --- PROAGATE PHI (FIRST HALF STEP) ---

      zwischen = .false.
      intperiod = 0.5_dop*updatetime
      call propphi(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
                   restartbs,restartzvode, &
                   abmsteps,repabmsteps,goodbssteps,badbssteps, &
                   smallbssteps,goodsilsteps,badsilsteps, &
                   restartsil)

! --- BACK-ROTATE THE ORBITALS ---

      if(lbackr) then
         call backrotate(psi,mc(mcpredpsi))
      endif


! --- ESTIMATE ERROR OF CONSTANT MEAN-FIELD PROPAGATION ---

      if (dentype .eq. 1) then
         call errorphi1(deltasum,psi(adim+1),mc(mcpredpsi),dicht2)
      else
         call errorphi(deltasum,psi(adim+1),mc(mcpredpsi), &
              dicht2,agmat,agmat)
      endif

! --- CALCULATE NEW UPDATE-TIME AND REPEAT STEP IF NECCESSARY ---

      if (deltasum .gt. 3.0_dop*epsphi) then  ! failed

!    --- OUTPUT FAILED UPDATE TIME AND DETERMINE NEW ONE ---

         if (lupdate) then
            write(iupd,'(i6,36x,e18.6e3)') updatesteps+1,deltasum
            call myflush(iupd)
         endif
         badupdatesteps = badupdatesteps+1
         remaintime = out1-timesinceout
         updatetime = 0.1_dop*updatetime*(epsphi/deltasum)**0.25_dop
         updatetime = remaintime &
                      /nint(remaintime/min(updatetime,remaintime))

!    --- RE-ESTABLISH OLD PSI-VECTOR AND RECOMPUTE TIME DERIVATIVE ---

         call cpvxz(mc(mcoldpsi),psi,dgldim)
         call funk2(time,psi)
         call funk3(time,psi,dtpsi)

!    --- REPEAT LAST STEP ---

         goto 100

      else if (deltasum .gt. 0.25_dop*epsphi) then  ! additional diag

         zwischen = .true.
         if (lupdate) then
            write(iupd,'(i6," * ",f15.8,e18.6e3,e18.6e3,f14.4)') &
            updatesteps+1,0.5*updatetime/fs,0.0_dop,deltasum,time/fs
            call myflush(iupd)
         end if


!--- SYMMETRIZE DOFs and 2D SPFs.
!  m=200.and.f=0 -> symcoeff,  f=200 -> sym3d, f=0 -> sym2d,
!  f=201 -> sym2kleg, f=202 -> nsym2d, m>100.and.f>0 -> parity.
         do j = 1, 2*maxdim
            if( msymmtr(j) .eq. 0 ) then
               goto 200

            elseif( msymmtr(j) .gt. 0 ) then
               m=mod(msymmtr(j),65536)/256
               f=mod(msymmtr(j),256)
               if(m.eq.200 .and. f.eq.0) then
                  continue   ! symcoeff
               elseif (m.gt.100 .and. f.gt.0) then
                  m=m-100
                  do s = 1, nstate
                     call parity(vgdim(f),ngdim(f),gdim(f),dim(m,s), &
                                 psi(zetf(m,s)))
                  enddo
               elseif(f.eq.0) then
                  do s = 1, nstate
                     call sym2d(gdim(spfdof(1,m)),dim(m,s),psi(zetf(m,s)))
                  enddo
               elseif(f.eq.200) then
                  do s = 1, nstate
                     call sym3d(gdim(spfdof(1,m)),dim(m,s),psi(zetf(m,s)))
                  enddo
               elseif (f.eq.201) then
                  do s = 1,nstate
                     call sym2d(gdim(spfdof(1,m))*gdim(spfdof(2,m)), &
                                dim(m,s),psi(zetf(m,s)))
                  enddo
               elseif (f.eq.202) then
                  do s = 1, nstate
                     call nsym2d(gdim(spfdof(1,m)),dim(m,s), &
                          psi(zetf(m,s)),nsym2dnorm)
                  enddo
!                 if nsym2d is used, the A-vetcor must be fixed
                  call nsym2d_fixa(psi,m,nsym2dnorm)
               else
                  do s = 1, nstate
                     call sym1d(vgdim(f),ngdim(f),gdim(f),dim(m,s), &
                                psi(zetf(m,s)))
                  enddo
               endif   ! f,m check

            elseif( msymmtr(j) .lt. 0 ) then
               m=mod(-msymmtr(j),65536)/256
               f=mod(-msymmtr(j),256)
               if(m.eq.200 .and. f.eq.0) then
                  continue   ! asymcoeff
               elseif(f.eq.0) then
                  do s = 1, nstate
                     call asym2d(gdim(spfdof(1,m)),dim(m,s), &
                                 psi(zetf(m,s)))
                  enddo
               elseif (f.eq.201) then
                  do s = 1,nstate
                     call asym2d(gdim(spfdof(1,m))*gdim(spfdof(2,m)), &
                                 dim(m,s),psi(zetf(m,s)))
                  enddo
               else
                  do s = 1, nstate
                     call asym1d(vgdim(f),ngdim(f),gdim(f),dim(m,s), &
                                 psi(zetf(m,s)))
                  enddo
               endif  ! f,m check
            endif     ! msmmtr gt or lt 0
         enddo        ! j
 200     if(msymmtr(1) .gt. 0 ) call myflush(ilog)

!--- SAVE NEW SPFS IN PHI ARRAY (for multi-D H terms)
         if (nmulpot(1) .gt. 0 .or. lklha) then
            call cpvxz(psi(adim+1),mc(mcphi),totphidim)
         endif

! --- DETERMINE MATRIX ELEMENTS AND DERIVATIVE OF A-VECTOR ---
! Note that "funk2" uses the "intpsi" array as complex work array!

         localtime = time+0.5_dop*updatetime
         call funk2(localtime,psi)
         call funka(localtime,psi,dtpsi)
         
!--- RESET TIME AND WRITE PHI-ENERGY TO RLX_INFO FILE
         
         intperiod = 0.5_dop*updatetime
         localtime = time+intperiod
         call calcenergy (psi,dtpsi,energy,0)
         if(inttypea.eq.6) then    ! DAV
            write(irlx,'(a,f10.3,f18.10,a3,",",f16.10,a4)') &
                 'After Relaxation T =',localtime/fs, &
                 energy*ev,str,(energy-oldenergy)*ev*1.0e+3_dop,str1
         else                      ! SIL
            write(irlx,'(a,f10.3,f18.10,a3,",",f16.10,a4)')  &
                 'After Relaxation T =',localtime/fs, &
                 -energy*ev,str,(oldenergy-energy)*ev*1.0e+3_dop,str1
         endif
         oldenergy = energy
         call myflush(irlx)

! ---- THIS IS FOR CHECKING PHI-DOT
         if(lquadphi) then
            do m=1,nmode
               currentmode=m
               call funkphi(time,psi(zetf(m,1)),dtpsi(zetf(m,1)))
               call normvxz(dtpsi(zetf(m,1)),norm1(m),dim(m,1)*subdim(m))
               call funkphi2(time,psi(zetf(m,1)),dtpsi(zetf(m,1)))
               call normvxz(dtpsi(zetf(m,1)),norm2(m),dim(m,1)*subdim(m))
            enddo
            write(irlx,'(a,f11.3,16e9.2)') ':: ',localtime/fs,  &
                 (norm1(m),m=1,nmode)
            write(irlx,'(a,f11.3,16e9.2)') '.. ',localtime/fs,  &
                 (norm2(m),m=1,nmode)
         else
            do m=1,nmode
               currentmode=m
            call normvxz(dtpsi(zetf(m,1)),norm1(m),dim(m,1)*subdim(m))
            enddo
            write(irlx,'(a,f11.3,16e9.2)') ':: ',localtime/fs, &
                                       (norm1(m),m=1,nmode)
         endif

!--- RELAX MCTDH-COEFFICIENTS (HALF-STEP) ---
         call propa(psi,dtpsi,localtime,intperiod,restartabm,restartrk5,&
              restartbs2,restartsil,restartzvode, &
              abmsteps,repabmsteps,goodbssteps,badbssteps, &
              smallbssteps,goodsilsteps,badsilsteps,Krylov,HKrylov,Kryc,HKryc)
     
!--- SYMMETRIZE COEFFICIENTS
         symcount=0
         do j = 1, 2*maxdim
            if( msymmtr(j) .eq. 0 ) then
               goto 22
!              182272=51200+2*65536
            elseif(msymmtr(j).eq.51200 .or. msymmtr(j).eq.182272) then
               symcount=symcount+1
               if(mpb.gt.0) then  ! single-set multi-packets run
                  do s=1,npackts
                     call symcoeff(psi(1+(s-1)*adim/npackts), &
                          s,.false.,symcount)
                  enddo
                  call schmidtortho2(psi,adim/npackts,npackts,ierr,mnorm)
                  write(ilog,'(a,i3,a,1pe11.4,a,0pf12.4)') '#1: ierr =', &
                        ierr,',  min-norm =',mnorm,',  time =',time/fs

               else
                  call symcoeff(psi,0,.false.,symcount)
               endif
               write(ilog,'(a)') '------' ! (will be overwritten)
            elseif(msymmtr(j).eq.-51200.or.msymmtr(j).eq.-182272) then
               symcount=symcount+1
               if(mpb.gt.0) then  ! single-set multi-packets run
                  do s=1,npackts
                     call symcoeff(psi(1+(s-1)*adim/npackts),  &
                          s,.true.,symcount)
                  enddo
                  call schmidtortho2(psi,adim/npackts,npackts,ierr,mnorm)
                  write(ilog,'(a,i3,a,1pe11.4,a,0pf12.4)') '#1: ierr =', &
                        ierr,',  min-norm =',mnorm,',  time =',time/fs
               else
                  call symcoeff(psi,0,.true.,symcount)
               endif
               write(ilog,'(a)') '------' ! (will be overwritten)
            endif
         enddo
 22      continue
         
!     --- COMPUTE TIME DERIVATIVE ---
         
         call funk3(localtime,psi,dtpsi)

! --- SAVE HALF-STEP VALUES ---
         call cpvxz(psi(adim+1),mc(mcpredpsi),totphidim)

! --- Write A-energy to rlx_info file ---
         call calcenergy (psi,dtpsi,energy,0)
         if(inttypea.eq.6) then     ! DAV
            write(irlx,'(a,f10.3,f18.10,a3,",",f16.10,a4)')  &
                 'After Davidson   T =',localtime/fs,energy*ev,str, &
                  (energy-oldenergy)*ev*1.0e+3_dop,str1
         else
            write(irlx,'(a,f10.3,f18.10,a3,",",f16.10,a4)') &
                 'After  Lanczos   T =',localtime/fs,-energy*ev,str, &
                  (oldenergy-energy)*ev*1.0e+3_dop,str1
         endif
         oldenergy = energy
         call myflush(irlx)

      else     ! Do second half step normally, no additional diagon.
         intperiod = 0.5_dop*updatetime
         localtime = time+intperiod
         call funk1(localtime,psi,dtpsi)
      endif

    
! --- PROPAGATE PHI (SECOND HALF STEP) ---
! ---- THIS IS FOR CHECKING PHI-DOT
      if(lquadphi) then
      do m=1,nmode
         currentmode=m
         call funkphi(time,psi(zetf(m,1)),dtpsi(zetf(m,1)))
         call normvxz(dtpsi(zetf(m,1)),norm1(m),dim(m,1)*subdim(m))
         call funkphi2(time,psi(zetf(m,1)),dtpsi(zetf(m,1)))
         call normvxz(dtpsi(zetf(m,1)),norm2(m),dim(m,1)*subdim(m))
      enddo
      write(irlx,'(a,f11.3,16e9.2)') ':: ',time/fs,(norm1(m),m=1,nmode)
      write(irlx,'(a,f11.3,16e9.2)') '.. ',time/fs,(norm2(m),m=1,nmode)
      else
      do m=1,nmode
         currentmode=m
!         call funkphi(time,psi(zetf(m,1)),dtpsi(zetf(m,1)),mc,mr,mi,ml)
         call normvxz(dtpsi(zetf(m,1)),norm1(m),dim(m,1)*subdim(m))
      enddo
      write(irlx,'(a,f11.3,16e9.2)') ':: ',time/fs,(norm1(m),m=1,nmode)
      endif

      call propphi(psi,dtpsi,localtime,intperiod,restartabm,restartrk5,&
                   restartbs,restartzvode, &
                   abmsteps,repabmsteps,goodbssteps,badbssteps, &
                   smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- BACK-ROTATE THE ORBITALS ---

      if(lbackr) then
         call backrotate(psi,mc(mcpredpsi))
      endif

! ---- THIS IS FOR CHECKING PHI-DOT
      if(lquadphi) then
      do m=1,nmode
         currentmode=m 
         call funkphi(time,psi(zetf(m,1)),dtpsi(zetf(m,1)))
         call normvxz(dtpsi(zetf(m,1)),norm1(m),dim(m,1)*subdim(m))
         call funkphi2(time,psi(zetf(m,1)),dtpsi(zetf(m,1)))
         call normvxz(dtpsi(zetf(m,1)),norm2(m),dim(m,1)*subdim(m))
      enddo
      write(irlx,'(a,f11.3,16e9.2)') ':: ',time/fs,(norm1(m),m=1,nmode)
      write(irlx,'(a,f11.3,16e9.2)') '.. ',time/fs,(norm2(m),m=1,nmode)
      else
      do m=1,nmode
         currentmode=m
         if(lbackr) &
         call funkphi(time,psi(zetf(m,1)),dtpsi(zetf(m,1)))
         call normvxz(dtpsi(zetf(m,1)),norm1(m),dim(m,1)*subdim(m))
      enddo
      write(irlx,'(a,f11.3,16e9.2)') ':: ',time/fs,(norm1(m),m=1,nmode)
      endif


! --- ESTIMATE PHI-ERROR OF CONSTANT MEAN-FIELD PROPAGATION ---

      if (dentype .eq. 1) then
         call errorphi1(deltasum,psi(adim+1),mc(mcpredpsi),dicht2)
      else
         call errorphi(deltasum,psi(adim+1),mc(mcpredpsi), &
              dicht2,agmat,agmat)
      endif

! --- CALCULATE NEW UPDATE-TIME AND REPEAT STEP IF NECCESSARY ---

      if( (deltasum .gt. 3.0_dop*epsphi) .and. .not.zwischen ) then


!    --- OUTPUT FAILED UPDATE TIME AND DETERMINE NEW ONE ---

         if (lupdate) then
            write(iupd,'(i6,36x,e18.6e3)') updatesteps+1,deltasum
            call myflush(iupd)
         endif
         badupdatesteps = badupdatesteps+1
         remaintime = out1-timesinceout
         updatetime = 0.5_dop*updatetime*(epsphi/deltasum)**0.25_dop
         updatetime = remaintime &
                      /nint(remaintime/min(updatetime,remaintime))

!    --- RE-ESTABLISH OLD PSI-VECTOR AND RECOMPUTE TIME DERIVATIVE ---

         call cpvxz(mc(mcoldpsi),psi,dgldim)
         call funk2(time,psi)
         call funk3(time,psi,dtpsi)

!    --- REPEAT LAST STEP ---

         goto 100
      endif

!--- SYMMETRIZE DOFs and 2D SPFs.
!  m=200.and.f=0 -> symcoeff,  f=200 -> sym3d, f=0 -> sym2d,
!  f=201 -> sym2kleg, f=202 -> nsym2d, m>100.and.f>0 -> parity.
      do j = 1, 2*maxdim
         if( msymmtr(j) .eq. 0 ) then
            goto 201

         elseif( msymmtr(j) .gt. 0 ) then
            m=mod(msymmtr(j),65536)/256
            f=mod(msymmtr(j),256)
            if(m.eq.200 .and. f.eq.0) then
               continue   ! symcoeff
            elseif (m.gt.100 .and. f.gt.0) then
               m=m-100
               do s = 1, nstate
                  call parity(vgdim(f),ngdim(f),gdim(f),dim(m,s), &
                              psi(zetf(m,s)))
               enddo
            elseif(f.eq.0) then
               do s = 1, nstate
                  call sym2d(gdim(spfdof(1,m)),dim(m,s), &
                             psi(zetf(m,s)))
               enddo
            elseif(f.eq.200) then
               do s = 1, nstate
                  call sym3d(gdim(spfdof(1,m)),dim(m,s), &
                             psi(zetf(m,s)))
               enddo
            elseif (f.eq.201) then
               do s = 1,nstate
                  call sym2d(gdim(spfdof(1,m))*gdim(spfdof(2,m)), &
                             dim(m,s),psi(zetf(m,s)))
               enddo
            elseif (f.eq.202) then
               do s = 1, nstate
                  call nsym2d(gdim(spfdof(1,m)),dim(m,s),psi(zetf(m,s)),&
                       nsym2dnorm)
               enddo
!              if nsym2d is used, the A-vetcor must be fixed
               call nsym2d_fixa(psi,m,nsym2dnorm)
            else
               do s = 1, nstate
                  call sym1d(vgdim(f),ngdim(f),gdim(f),dim(m,s), &
                             psi(zetf(m,s)))
               enddo
            endif  ! f,m check

         elseif( msymmtr(j) .lt. 0 ) then
            m=mod(-msymmtr(j),65536)/256
            f=mod(-msymmtr(j),256)
            if(m.eq.200 .and. f.eq.0) then
               continue   ! asymcoeff
            elseif(f.eq.0) then
               do s = 1, nstate
                  call asym2d(gdim(spfdof(1,m)),dim(m,s),psi(zetf(m,s)))
               enddo
            elseif (f.eq.201) then
               do s = 1,nstate
                  call asym2d(gdim(spfdof(1,m))*gdim(spfdof(2,m)),dim(m,s), &
                       psi(zetf(m,s)))
               enddo
            else
               do s = 1, nstate
                 call asym1d(vgdim(f),ngdim(f),gdim(f),dim(m,s),psi(zetf(m,s)))
               enddo
            endif  ! f,m check
         endif     ! msmmtr gt or lt 0
      enddo        ! j
 201  if(msymmtr(1) .gt. 0 ) call myflush(ilog)


!--- SAVE NEW SPFS IN PHI ARRAY (for multi-D H terms)
      if (nmulpot(1) .gt. 0 .or. lklha) then
         call cpvxz(psi(adim+1),mc(mcphi),totphidim)
      endif

! --- DETERMINE MATRIX ELEMENTS AND DERIVATIVE OF A-VECTOR ---

! Note that "funk2" uses the "intpsi" array as complex work array!

      localtime = time+updatetime
      call funk2(localtime,psi)
      call funka(localtime,psi,dtpsi)

!--- RESET TIME AND WRITE PHI-ENERGY TO RLX_INFO FILE

      intperiod = 0.5_dop*updatetime
      localtime = time+updatetime
      call calcenergy (psi,dtpsi,energy,0)
      if(inttypea.eq.6) then    ! DAV
         write(irlx,'(a,f10.3,f18.10,a3,",",f16.10,a4)') &
              'After Relaxation T =',localtime/fs, &
              energy*ev,str,(energy-oldenergy)*ev*1.0e+3_dop,str1
      else                     ! SIL
         write(irlx,'(a,f10.3,f18.10,a3,",",f16.10,a4)') &
              'After Relaxation T =',localtime/fs, &
              -energy*ev,str,(oldenergy-energy)*ev*1.0e+3_dop,str1
      endif
      oldenergy = energy
      call myflush(irlx)

! --- (c<n> option of relaxation) --- 
      if(icount .lt. phicount) then
          call funk3(time,psi,dtpsi)
          goto 100
      endif
         

! --- PROPAGATE MCTDH-COEFFICIENTS (SECOND HALF-STEP) ---
      call propa(psi,dtpsi,localtime,intperiod,restartabm,restartrk5,&
                 restartbs2,restartsil,restartzvode, &
                 abmsteps,repabmsteps,goodbssteps,badbssteps, &
                 smallbssteps,goodsilsteps,badsilsteps, Krylov,HKrylov,Kryc,HKryc)

!--- SYMMETRIZE COEFFICIENTS
      symcount=0
      do j = 1, 2*maxdim
         if( msymmtr(j) .eq. 0 ) then
            goto 21
         elseif(msymmtr(j).eq.51200 .or. msymmtr(j).eq.182272) then
            symcount=symcount+1
            if(mpb.gt.0) then  ! single-set multi-packets run
               do s=1,npackts
                  call symcoeff(psi(1+(s-1)*adim/npackts),  &
                       s,.false.,symcount)
               enddo
               call schmidtortho2(psi,adim/npackts,npackts,ierr,mnorm)
               write(ilog,'(a,i3,a,1pe11.4,a,0pf12.4)') '#2: ierr =', &
                    ierr,',  min-norm =',mnorm,',  time =',time/fs
            else
               call symcoeff(psi,0,.false.,symcount)
            endif
            write(ilog,'(a)') '------' ! (will be overwritten)
         elseif(msymmtr(j).eq.-51200.or.msymmtr(j).eq.-182272) then
            symcount=symcount+1
            if(mpb.gt.0) then  ! single-set multi-packets run
               do s=1,npackts
                  call symcoeff(psi(1+(s-1)*adim/npackts),  &
                       s,.true.,symcount)
               enddo
               call schmidtortho2(psi,adim/npackts,npackts,ierr,mnorm)
               write(ilog,'(a,i3,a,1pe11.4,a,0pf12.4)') '#2: ierr =', &
                    ierr,',  min-norm =',mnorm,',  time =',time/fs
            else
               call symcoeff(psi,0,.true.,symcount)
            endif
            write(ilog,'(a)') '------' ! (will be overwritten)
         endif
      enddo
 21   continue

! --- OUTPUT ACCEPTED UPDATE TIME AND DETERMINE NEW ONE ---

      sumupdate = sumupdate+updatetime
      sum2update = sum2update+updatetime**2
      updatesteps = updatesteps+1
      if (lupdate) then
         intperiod = updatetime/fs
         localtime = time/fs
         if(zwischen) then
            intperiod = 0.5_dop*intperiod
            localtime = localtime + intperiod   
         endif
         write(iupd,'(i6,f18.8,e18.6e3,e18.6e3,f14.4)')&
            updatesteps,intperiod,0.0_dop,deltasum,localtime
         call myflush(iupd)
      end if
      if(zwischen) deltasum = 5.0_dop*deltasum  
!.....should be factor 4 because of quadratic behavior, factor 5 for safety.
      deltasum = deltasum+0.05_dop*epsphi
!      newupdatetime = updatetime*(epsphi/deltasum)**0.25_dop
!.....Exponent should be 0.5, but 0.4 avoids failed steps.
      newupdatetime = updatetime*(epsphi/deltasum)**0.4_dop

! --- RETURN THE INTERVAL THAT HAS ACTUALLY BEEN INTEGRATED ---

      madestep = updatetime

      return
      end subroutine propcmfadarlx

! ------------------------------------------------------------------

      subroutine backrotate(psi,predpsi)

      implicit none

      complex(dop), dimension(dgldim), intent(inout)      :: psi
      complex(dop), dimension(totphidim), intent(in)      :: predpsi
      complex(dop), dimension(maxspf*maxspf)              :: ww,vv,ovlmat,workc
      real(dop), dimension(8)                             :: rsum
      real(dop), dimension(maxspf)                        :: eig
      real(dop), dimension(3*maxspf)                      :: workr
      integer(long)                                       :: i,m,s

      do s=1,nstate
         do m=1,nmode
            call mmaxzz(predpsi(zetf(m,s)-adim),psi(zetf(m,s)),&
                        ovlmat,subdim(m),dim(m,s),dim(m,s))
            call mqxazz(ovlmat,ovlmat,vv,dim(m,s),dim(m,s))
            call zheev('V','U',dim(m,s),vv,dim(m,s),eig,workc,&
                       maxspf*maxgdim,workr,i)
            do i=1,dim(m,s)
               eig(i)=1.0_dop/sqrt(eig(i))
            enddo
            call dqxxdz2(eig,vv,ww,dim(m,s))
            call mqxazz(vv,ww,workc,dim(m,s),dim(m,s))
            call qqaxzz(ovlmat,workc,vv,dim(m,s))
            call mqxxzz(psi(zetf(m,s)),vv,workc,subdim(m),dim(m,s))
            call cpmxz(workc,psi(zetf(m,s)),subdim(m),dim(m,s))
            rsum(m)=0.0_dop
            do i=1,dim(m,s)
               rsum(m) = rsum(m) + abs(vv(i+(i-1)*dim(m,s))-1.0_dop)
            enddo 
         enddo
         write(irlx,'(a,8f12.6)') 'backr',(rsum(i),i=1,nmode)
      enddo
      return
      end subroutine backrotate

! *** V6.4 MB *********> SUBROUTINE PROP CMF ADA PHI <******************
!                       ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

! Drives the propagation of the wavefunction in constant mean-field
! (CMF) mode with adaptive (ADA) update time controlled by the spf's
! error only.
! **********************************************************************

      subroutine propcmfadaphi (psi,dtpsi,time,timesinceout, &
                                newupdatetime,madestep,restartabm,restartrk5, &
                                restartbs,restartbs2,restartsil,restartzvode, &
                                abmsteps,repabmsteps,goodbssteps, &
                                badbssteps,smallbssteps,goodsilsteps, &
                                badsilsteps,sumupdate,sum2update, &
                                updatesteps,badupdatesteps,Krylov,HKrylov,Kryc,HKryc)

      implicit none

      complex(dop), dimension(dgldim), intent(inout) :: psi,dtpsi
      complex(dop), dimension(ikryc), intent(inout)  :: Kryc,HKryc
      real(dop), dimension(ikrylov), intent(inout)     :: Krylov,HKrylov
      real(dop)                                      :: epsphi,deltasum,&
                                                        intperiod,localtime,&
                                                        remaintime
      real(dop), intent(out)                         :: newupdatetime,madestep
      real(dop), intent(inout)                       :: sumupdate,sum2update
      real(dop), intent(in)                          :: timesinceout,time
      integer(long), intent(inout)                   :: updatesteps,&
                                                        badupdatesteps
      integer(long), intent(inout)                   :: abmsteps,repabmsteps,&
                                                        goodbssteps,&
                                                        badbssteps,&
                                                        smallbssteps,&
                                                        goodsilsteps,&
                                                        badsilsteps
      logical(kind=4), intent(inout)                 :: restartabm,restartrk5,&
                                                        restartsil
      logical(kind=4), intent(inout)                 :: restartbs,restartbs2,&
                                                        restartzvode
      complex(dop), dimension(:), allocatable        :: agpred


!
! Allocate local memory
!
      if (lgwp) then
         allocate(agpred(agmatdim))
      endif

! --- SET ACCURACIES ---

      epsphi = epsupdate

! --- SAVE OLD VALUES ---

      call cpvxz(psi,mc(mcoldpsi),dgldim)

! --- SAVE SPFS IN PHI ARRAY (for multi-D H terms)
      if (nmulpot(1) .gt. 0 .or. lklha) then
         call cpvxz(psi(adim+1),mc(mcphi),totphidim)
      endif

 100  continue

! --- PROPAGATE MCTDH-COEFFICIENTS (FIRST HALF-STEP) ---

      intperiod = 0.5_dop*updatetime
      call propa(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
                 restartbs2,restartsil,restartzvode, &
                 abmsteps,repabmsteps,goodbssteps,badbssteps,&
                 smallbssteps,goodsilsteps,badsilsteps,Krylov,HKrylov,Kryc,HKryc)

! --- PROPAGATE PHI (PREDICTOR STEP) ---

      intperiod = 0.5_dop*updatetime
      call propphi(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
                   restartbs,restartzvode,&
                   abmsteps,repabmsteps,goodbssteps,badbssteps,&
                   smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- SAVE PREDICTED SPF'S ---

      call cpvxz(psi(adim+1),mc(mcpredpsi),totphidim)
      if (lgwp) call cpvxz(agmat,agpred,agmatdim)

! --- COMPUTE PREDICTED MEAN-FIELDS ---

      localtime = time+0.5_dop*updatetime
      call funk0(localtime,psi,dtpsi)

! --- RE-ESTABLISH OLD SPF'S AND COMPUTE THEIR TIME DERIVATIVE ---

      call cpvxz(mc(mcoldpsi+adim),psi(adim+1),totphidim)
      call funk1(time,psi,dtpsi)

! --- PROPAGATE PHI (CORRECTOR STEP) ---

      intperiod = 0.5_dop*updatetime
      call propphi(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
                   restartbs,restartzvode, &
                   abmsteps,repabmsteps,goodbssteps,badbssteps, &
                   smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- ESTIMATE ERROR OF CONSTANT MEAN-FIELD PROPAGATION ---

      if (dentype .eq. 1) then
         call errorphi1(deltasum,psi(adim+1),mc(mcpredpsi),dicht2)
      else
         call errorphi(deltasum,psi(adim+1),mc(mcpredpsi), &
              dicht2,agmat,agpred)
      endif

! --- CALCULATE NEW UPDATE-TIME AND REPEAT STEP IF NECCESSARY ---

      if (deltasum .gt. 2.0_dop*epsphi) then

!    --- OUTPUT FAILED UPDATE TIME AND DETERMINE NEW ONE ---

         if (lupdate) then
            write(iupd,'(i6,36x,e18.6e3)') updatesteps+1,deltasum
            call myflush(iupd)
         endif
         badupdatesteps = badupdatesteps+1
         remaintime = out1-timesinceout
         updatetime = 0.7_dop*updatetime*(epsphi/deltasum)**0.25_dop
         updatetime = remaintime &
                      /nint(remaintime/min(updatetime,remaintime))

!    --- RE-ESTABLISH OLD PSI-VECTOR AND RECOMPUTE TIME DERIVATIVE ---

         call cpvxz(mc(mcoldpsi),psi,dgldim)
         call funk2(time,psi)
         call funk3(time,psi,dtpsi)

!    --- REPEAT LAST STEP ---

         goto 100
      endif

! --- COMPUTE TIME DERIVATIVE OF SINGLE-PARTICLE FUNCTIONS ---

      localtime = time+0.5_dop*updatetime
      call funk1(localtime,psi,dtpsi)

! --- PROPAGATE PHI (SECOND HALF-STEP) ---

      intperiod = 0.5_dop*updatetime
      call propphi(psi,dtpsi,localtime,intperiod,restartabm,restartrk5,&
                   restartbs,restartzvode, &
                   abmsteps,repabmsteps,goodbssteps,badbssteps, &
                   smallbssteps,goodsilsteps,badsilsteps,restartsil)

!--- SAVE NEW SPFS IN PHI ARRAY (for multi-D H terms)
      if (nmulpot(1) .gt. 0 .or. lklha) then
         call cpvxz(psi(adim+1),mc(mcphi),totphidim)
      endif

! --- DETERMINE MATRIX ELEMENTS AND DERIVATIVE OF A-VECTOR ---

! Note that "funk2" uses the "intpsi" array as complex work array!

      localtime = time+updatetime
      call funk2(localtime,psi)
      localtime = time+0.5_dop*updatetime
      call funka(localtime,psi,dtpsi)

! --- PROPAGATE MCTDH-COEFFICIENTS (SECOND HALF-STEP) ---

      intperiod = 0.5_dop*updatetime
      call propa(psi,dtpsi,localtime,intperiod,restartabm,restartrk5,&
                 restartbs2,restartsil,restartzvode, &
                 abmsteps,repabmsteps,goodbssteps,badbssteps, &
                 smallbssteps,goodsilsteps,badsilsteps,Krylov,HKrylov,Kryc,HKryc)

! --- OUTPUT ACCEPTED UPDATE TIME AND DETERMINE NEW ONE ---

      sumupdate = sumupdate+updatetime
      sum2update = sum2update+updatetime**2
      updatesteps = updatesteps+1
      if (lupdate) then
         write(iupd,'(i6,f18.8,e18.6e3,e18.6e3,f14.4)') &
            updatesteps,updatetime/fs,0.0_dop,deltasum,time/fs
         call myflush(iupd)
      end if
      deltasum = deltasum+0.05_dop*epsphi
      newupdatetime = updatetime*(epsphi/deltasum)**0.25_dop

! --- RETURN THE INTERVAL THAT HAS ACTUALLY BEEN INTEGRATED ---

      madestep = updatetime

      if (lgwp) then
         deallocate(agpred)
      endif

      return
      end subroutine propcmfadaphi

! *** V6.2 MB **********> SUBROUTINE PROP CMF ADA A <********************
!                        ~~~~~~~~~~~~~~~~~~~~~~~~~~
! Drives the propagation of the wavefunction in constant mean-field
! (CMF) mode with adaptive (ADA) update time.
!#######################################################################

      subroutine propcmfadaa (psi,dtpsi,time,timesinceout, &
                             newupdatetime,madestep,restartabm,restartrk5, &
                             restartbs,restartbs2,restartsil, &
                             restartzvode,abmsteps, &
                             repabmsteps,goodbssteps,badbssteps, &
                             smallbssteps,goodsilsteps,badsilsteps, &
                             sumupdate,sum2update,updatesteps, &
                             badupdatesteps,badbackw,Krylov,HKrylov,Kryc,HKryc)

      implicit none

      real(dop), parameter                       :: epscorr = 1.0_dop

      complex(dop), dimension(dgldim), intent(inout) :: psi,dtpsi
      complex(dop), dimension(ikryc), intent(inout)  :: Kryc,HKryc
      real(dop), dimension(ikrylov), intent(inout)     :: Krylov,HKrylov
      real(dop)                                      :: erra,intperiod,&
                                                        localtime,epsa,&
                                                        remaintime,&
                                                        newupdtimea
      real(dop), intent(out)                         :: newupdatetime,madestep
      real(dop), intent(inout)                       :: sumupdate,sum2update
      real(dop), intent(in)                          :: time,timesinceout
      integer(long)                                  :: oldbadsilsteps
      integer(long), intent(inout)                   :: updatesteps,badbackw,&
                                                        badupdatesteps,goodbssteps,&
                                                        badbssteps,smallbssteps,&
                                                        goodsilsteps,badsilsteps
      integer(long), intent(out)                     :: abmsteps,repabmsteps
      logical(kind=4), intent(inout)                 :: restartabm,restartrk5
      logical(kind=4), intent(inout)                 :: restartbs,restartbs2,&
                                                        restartsil,restartzvode


      real(dop), save :: maxupdate = 1.0e99_dop

! --- SET ACCURACIES ---

      epsa = epscorr*epsupdate

! --- SAVE OLD VALUES ---

      call cpvxz(psi,mc(mcoldpsi),dgldim)

! --- SAVE SPFS IN PHI ARRAY (for multi-D H terms)
      if (nmulpot(1) .gt. 0 .or. lklha) then
         call cpvxz(psi(adim+1),mc(mcphi),totphidim)
      endif

 100  continue

! --- PROPAGATE MCTDH-COEFFICIENTS (FIRST HALF-STEP) ---

      intperiod = 0.5_dop*updatetime
      call propa(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
                 restartbs2,restartsil,restartzvode, &
                 abmsteps,repabmsteps,goodbssteps,badbssteps, &
                 smallbssteps,goodsilsteps,badsilsteps,Krylov,HKrylov,Kryc,HKryc)

! --- PROPAGATE PHI (PREDICTOR STEP) ---

      intperiod = 0.5_dop*updatetime
      call propphi(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
                   restartbs,restartzvode, &
                   abmsteps,repabmsteps,goodbssteps,badbssteps, &
                   smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- SAVE PREDICTED SPF'S AND PROPAGATED A-VECTOR ---

      call cpvxz(psi,mc(mcpredpsi),dgldim)


! --- COMPUTE PREDICTED MEAN-FIELDS ---

      localtime = time+0.5_dop*updatetime
      call funk0(localtime,psi,dtpsi)

! --- RE-ESTABLISH OLD SPF'S AND COMPUTE THEIR TIME DERIVATIVE ---

      call cpvxz(mc(mcoldpsi+adim),psi(adim+1),totphidim)
      call funk1(time,psi,dtpsi)

! --- PROPAGATE PHI (CORRECTOR STEP) ---

      intperiod = 0.5_dop*updatetime
      call propphi(psi,dtpsi,time,intperiod,restartabm,restartrk5,&
                   restartbs,restartzvode,&
                   abmsteps,repabmsteps,goodbssteps,badbssteps,&
                   smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- COMPUTE TIME DERIVATIVE OF SINGLE-PARTICLE FUNCTIONS ---

      localtime = time+0.5d0*updatetime
      call funk1(localtime,psi,dtpsi)

! --- PROPAGATE PHI (SECOND HALF-STEP) ---

      intperiod = 0.5d0*updatetime
      localtime = time+0.5d0*updatetime
      call propphi(psi,dtpsi,localtime,intperiod,restartabm,restartrk5,&
                   restartbs,restartzvode, &
                   abmsteps,repabmsteps,goodbssteps,badbssteps, &
                   smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- SAVE NEW SPFS IN PHI ARRAY (for multi-D H terms)
      if (nmulpot(1) .gt. 0 .or. lklha) then
         call cpvxz(psi(adim+1),mc(mcphi),totphidim)
      endif

! --- DETERMINE MATRIX ELEMENTS AND DERIVATIVE OF A-VECTOR ---

! Note that "funk2" uses the "intpsi" array as complex work array!

      localtime = time+updatetime
      call funk2(localtime,psi)
      localtime = time+0.5_dop*updatetime
      call funka(localtime,psi,dtpsi)

! --- PROPAGATE MCTDH-COEFFICIENTS (SECOND HALF-STEP) ---

      oldbadsilsteps = badsilsteps
      intperiod = 0.5_dop*updatetime
      call propa(psi,dtpsi,localtime,intperiod,restartabm,restartrk5,&
                 restartbs2,restartsil,restartzvode, &
                 abmsteps,repabmsteps,goodbssteps,badbssteps,&
                 smallbssteps,goodsilsteps,badsilsteps,Krylov,HKrylov,Kryc,HKryc)

! --- REPEAT STEP IF LANCZOS STEP FAILED BUT HAS BEEN FIXED ---

! If the Lanczos integrator didn't converge within the maximum order but
! needed more than one step, the Krylov space doesn't belong to
! psi(localtime) anymore. In that case, the following back-propagation
! which uses the "old" Krylov vectors would lead to wrong results, so
! decrease the update-interval and repeat the step.

      if (badsilsteps .gt. oldbadsilsteps) then

!    --- REDUCE UPDATE-INTERVAL AND WRITE TO UPDATE FILE ---

         if (lupdate) then
            write(iupd,'(i6)') updatesteps+1
            call myflush(iupd)
         endif
         maxupdate=0.75_dop*updatetime
         updatetime=0.5_dop*updatetime
         badbackw=badbackw+1

!    --- RE-ESTABLISH OLD PSI-VECTOR AND RECOMPUTE TIME DERIVATIVE ---

         call cpvxz(mc(mcoldpsi),psi,dgldim)
         call funk2(time,psi)
         call funk3(time,psi,dtpsi)

!    --- REPEAT LAST STEP ---

         goto 100
      endif

! --- BACK-PROPAGATE MCTDH-COEFFICIENTS (FOR ERROR ESTIMATE) ---

! Be careful: the "intpsi" array containing the Krylov vectors must not
! have been overwritten since the last call of the Lanczos integrator!

      intperiod = -0.5_dop*updatetime
      restartsil = .false.
      call propa(mc(mcpredpsi),dtpsi,localtime,intperiod,restartabm, &
                 restartrk5,restartbs2,restartsil,restartzvode, &
                 abmsteps,repabmsteps,goodbssteps, &
                 badbssteps,smallbssteps,goodsilsteps,badsilsteps, &
                 Krylov,HKrylov,Kryc,HKryc)
      restartsil = .true.

! --- ESTIMATE ERROR OF A-VECTOR DUE TO CONSTANT MEAN-FIELDS ---

      call errora(erra,mc(mcpredpsi),mc(mcoldpsi))

! --- CALCULATE NEW UPDATE-TIME AND REPEAT STEP IF NECCESSARY ---

      if (erra .gt. 2.0_dop*epsa) then

!    --- OUTPUT FAILED UPDATE TIME AND DETERMINE NEW ONE ---

         if (lupdate) then
            write(iupd,'(i6,18x,e18.6e3)') updatesteps+1,erra
            call myflush(iupd)
         endif
         badupdatesteps = badupdatesteps+1
         remaintime = out1-timesinceout
         updatetime = 0.7_dop*updatetime*(epsa/erra)**0.25_dop
         updatetime = remaintime &
                      /nint(remaintime/min(updatetime,remaintime))

!    --- RE-ESTABLISH OLD PSI-VECTOR AND RECOMPUTE TIME DERIVATIVE ---

         call cpvxz(mc(mcoldpsi),psi,dgldim)
         call funk2(time,psi)
         call funk3(time,psi,dtpsi)

!    --- REPEAT LAST STEP ---

         goto 100
      else

!    --- OUTPUT ACCEPTED UPDATE TIME AND DETERMINE NEW ONE ---

         sumupdate = sumupdate+updatetime
         sum2update = sum2update+updatetime**2
         updatesteps = updatesteps+1
         if (lupdate) then
            write(iupd,'(i6,f18.8,e18.6e3,e18.6e3,f14.4)') &
            updatesteps,updatetime/fs,erra,0.0,time/fs
            call myflush(iupd)
         end if
         erra = erra+0.05_dop*epsa
         newupdtimea = updatetime*(epsa/erra)**0.25_dop
         newupdatetime = min(newupdtimea,maxupdate)
      endif

! --- RETURN THE INTERVAL THAT HAS ACTUALLY BEEN INTEGRATED ---

      madestep = updatetime

      return
      end subroutine propcmfadaa

! *** V8.5 OV **********> SUBROUTINE PROP CMF ADA 2 <*******************
!                        ~~~~~~~~~~~~~~~~~~~~~~~~~

! Drives the propagation of the wavefunction in constant mean-field
! (CMF) mode with adaptive (ADA) update time for ML-MCTDH.
!
      subroutine propcmfada2 (psi,dtpsi,time,timesinceout,&
                              newupdatetime,madestep,restartabm,&
                              restartbs,restartsil,abmsteps,&
                              repabmsteps,goodbssteps,badbssteps,&
                              smallbssteps,goodsilsteps,badsilsteps,&
                              sumupdate,sum2update,updatesteps,&
                              badupdatesteps,restartrk5)

        implicit none

      real*8 epscorr
      parameter (epscorr = 1.0d0)

      complex(dop), dimension(dgldim) :: psi,dtpsi



      real(dop)       :: time,timesinceout,newupdatetime,madestep,&
                         intperiod,localtime,sumupdate,sum2update,&
                         deltasum,remaintime,epstot,newupdtime,savetime,&
                         maxupdate
      integer(long)   :: abmsteps,repabmsteps,goodbssteps,badbssteps,&
                         smallbssteps,goodsilsteps,badsilsteps,updatesteps,&
                         badupdatesteps
      logical(kind=4) :: restartabm,restartbs,restartsil,restartrk5

      complex(dop), dimension(dgldim) :: psi14
      complex(dop), dimension(dgldim) :: psi12
      complex(dop), dimension(dgldim) :: psi34
      complex(dop), dimension(dgldim) :: psipred
      complex(dop), dimension(dgldim) :: psiold
      complex(dop), dimension(:), allocatable :: agpred

      data maxupdate /1.0d99/
      save maxupdate
      data savetime /-42.42d42/  ! some unlikely value
      save savetime

!
! Allocate local memory
!
      if (lgwp) then
         allocate(agpred(agmatdim))
      endif

! --- SET ACCURACIES ---
      epstot = epsupdate

! --- compute initial mean fields and dtpsi unless they were
!     done in the previous step
      if (time.ne.savetime) then
         call funk2(time,psi)
         call funk3(time,psi,dtpsi)
      endif
! --- SAVE t=0 PSI ---
      call cpvxz(psi,psiold,dgldim)

 100  continue

! --- PROPAGATE |PSI>(0 to Dt/4) with H(0)---
      intperiod = 0.25d0*updatetime
      call propphiml(psi,dtpsi,time,intperiod,restartabm,restartrk5,restartbs,&
                     abmsteps,repabmsteps,goodbssteps,badbssteps,&
                     smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- SAVE |PSI>(Dt/4) ---
      call cpvxz(psi,psi14,dgldim)

! --- RESTORE |PSI>(0) AND COMPUTE TIME DERIVATIVES---
      call cpvxz(psiold,psi,dgldim)
      call funk1(time,psi,dtpsi)
      call funka(time,psi,dtpsi)

! --- PROPAGATE |PSI>(0 to Dt/2) with H(0)---
      intperiod = 0.5d0*updatetime
      call propphiml(psi,dtpsi,time,intperiod,restartabm,restartrk5,restartbs,&
                     abmsteps,repabmsteps,goodbssteps,badbssteps,&
                     smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- SAVE |PSI>(Dt/2) ---
      call cpvxz(psi,psi12,dgldim)

! --- COMPUTE H(Dt/2) ---
      localtime = time+0.5d0*updatetime
      call funk0(time,psi,dtpsi)

! --- PROPAGATE |PSI>(Dt/4 to 3Dt/4) with H(Dt/2) ---
      call cpvxz(psi14,psi,dgldim)
      localtime = time+0.25d0*updatetime
      intperiod = 0.5d0*updatetime
      call funk1(time,psi,dtpsi)
      call funka(time,psi,dtpsi)

      call propphiml(psi,dtpsi,localtime,intperiod,restartabm,restartrk5,&
                     restartbs,abmsteps,repabmsteps,goodbssteps,badbssteps,&
                     smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- SAVE |PSI>(3Dt/4) ---
      call cpvxz(psi,psi34,dgldim)

! --- PROPAGATE |PSI>(0 to Dt) with H(Dt/2) [predictor step]---
      call cpvxz(psiold,psi,dgldim)
      localtime = time
      intperiod = updatetime
      call funk1(time,psi,dtpsi)
      call funka(time,psi,dtpsi)

      call propphiml(psi,dtpsi,localtime,intperiod,restartabm,restartrk5,&
                     restartbs,abmsteps,repabmsteps,goodbssteps,badbssteps,&
                     smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- SAVE PREDICTED Dt PSI ---
      call cpvxz(psi,psipred,dgldim)
      if (lgwp) call cpvxz(agmat,agpred,agmatdim)

! --- COMPUTE H(Dt) ---
      localtime = time+updatetime
      call funk0(time,psi,dtpsi)

! --- PROPAGATE |PSI>(3Dt/4 to Dt) with H(Dt) ---
      call cpvxz(psi34,psi,dgldim)
      localtime = time+0.75d0*updatetime
      intperiod = 0.25d0*updatetime
      call funk1(time,psi,dtpsi)
      call funka(time,psi,dtpsi)

      call propphiml(psi,dtpsi,localtime,intperiod,restartabm,restartrk5,&
                     restartbs,abmsteps,repabmsteps,goodbssteps,badbssteps,&
                     smallbssteps,goodsilsteps,badsilsteps,restartsil)

! --- ESTIMATE ERROR OF CONSTANT MEAN-FIELD PROPAGATION ---
      call errortotml(deltasum,psi,psipred,agmat,agpred)

! --- CALCULATE NEW UPDATE-TIME AND REPEAT STEP IF NECCESSARY ---
      if (deltasum .gt. 1.5d0*epstot) then

!    --- OUTPUT FAILED, UPDATE TIME AND DETERMINE NEW ONE ---
         if (lupdate) then
            write(iupd,'(i6,18x,e18.6e3)') updatesteps+1,deltasum
            call myflush(iupd)
         endif
         badupdatesteps = badupdatesteps+1
         remaintime = out1-timesinceout
         updatetime = 0.7d0*updatetime*(epstot/deltasum)**0.1666666667d0
         updatetime = remaintime &
                      /(1+int(remaintime/min(updatetime,remaintime)))
!        updatetime = min(updatetime,remaintime)

!    --- RE-ESTABLISH OLD PSI-VECTOR AND RESTART STEP ---
         call cpvxz(psiold,psi,dgldim)
         lipictrafo = false
         call funk2(time,psi)
         call funk3(time,psi,dtpsi)

         lipictrafo = true

!    --- REPEAT LAST STEP ---

         goto 100
      endif

!    --- STEP ACCEPTED ---

!    --- IF REQUESTED, REORTHONORMALIZE ---
      if (limportho) call reorthopsi(psi,tree,dicht1,dicht2,dicht3,dicht4,&
                          mi(mijindx),gs2)

!    --- DO DERIVATIVE FOR THE OUTPUT ROUTINES OUTSIDE
      localtime = time+updatetime

      call funk2(time,psi)
      call funk3(time,psi,dtpsi)

      savetime = localtime

!    --- UPDATE TIME AND DETERMINE NEW ONE ---
         sumupdate = sumupdate+updatetime
         sum2update = sum2update+updatetime**2
         updatesteps = updatesteps+1
         if (lupdate) then
            write(iupd,'(i6,f18.8,e18.6e3,f14.4)') &
            updatesteps,updatetime/fs,deltasum,time/fs
            call myflush(iupd)
         end if
         deltasum = deltasum+0.001d0*epstot
         newupdtime = updatetime*(epstot/deltasum)**0.1666666667d0
         newupdatetime = min(newupdtime,maxupdate)

! --- RETURN THE INTERVAL THAT HAS ACTUALLY BEEN INTEGRATED ---

      madestep = updatetime
   !   nmode = nmodesave

      call myflush(iupd)

      if (lgwp) then
         deallocate(agpred)
      endif

      return
      end subroutine propcmfada2


      end module propcmfadamod

