!######################################################################
      module psioutmod

use decimal, only: long, dop
use mmlib,   only: mmxczz
use mtlib,   only: qttxzz
use op1lib,  only: cpvxz, sum2vxz

use global
use dglobal
use maxv, only: maxdim,maxsta,maxkoe
use dvrdatmod
use griddatmod
use psidef
use iopsidef, only: tapelength
use datenmod
use runpropmod
use channels

      implicit none
      private
      public :: psiout
      contains

!######################################################################
!
!                      MCTDH module PSIOUT
!
! Controls the input and output of the wavefunction.
!
! subroutines :
!    psiout   : write psi to psi file
!
!  V6.4 GW    
!
!######################################################################

      subroutine psiout (psi,dicht4,time,jindx,agmat,trajst,gwpdep)

      implicit none

      integer(long)                                       :: d,adimcm,dgldimcm,&
                                                             m,s,t,i,b,zeig
      integer(long), dimension(maxsta)                    :: blockcm
      integer(long), dimension(maxsta,maxsta)             :: dblockcm
      integer(long), dimension(jindxdim), intent(in)      :: jindx
      integer(long), dimension(:), allocatable            :: worki
      integer(long), dimension(gwpinfodim), intent(in)    :: trajst
      integer(long), dimension(gwpinfodim), intent(in)    :: gwpdep
      real(dop)                                           :: normcm,absnat
      real(dop), intent(in)                               :: time
      complex(dop)                                        :: sum
      complex(dop), dimension(dgldim), intent(in)         :: psi
      complex(dop), dimension(dgldim)                     :: natur
      complex(dop), dimension(dmatdim), intent(in)        :: dicht4
      complex(dop), dimension(agmatdim), intent(in)       :: agmat
      complex(dop), dimension(:), allocatable             :: workc
      logical(kind=4)                                     :: lpsisp,lpsicm,&
                                                             lpsinat,lselect
      character(len=c2)                                   :: wrtag

!-----------------------------------------------------------------------
! unpack various options
!-----------------------------------------------------------------------
      lpsisp=lpsiopt(1)
      lpsicm=lpsiopt(2)
      lpsinat=lpsiopt(3)
      lselect=lpsiopt(4)

      if (lpsiopt(3) .or. lpsiopt(2)) then
         allocate(workc(2*dgldim))
      else
         allocate(workc(1))
      endif
      if (lpsiopt(2)) then
         allocate(worki(adim))
      else
         allocate(worki(1))
      endif

!-----------------------------------------------------------------------
! calculate spf transformation to natural orbitals if psi to be 
! saved in that format, or if compact storage to be used.
!-----------------------------------------------------------------------
      if (lpsinat .or. lpsicm) then
         if (dentype .eq. 1) then
            do s=1,nstate
               do t=1,nstate
                  do m=1,nmode
                     call mmxczz(psi(dzetf(m,s,t)),dicht4(ddmat(m,s,t)), &
                          natur(dzetf(m,s,t)),subdim(m),ddim(m,s,t), &
                          ddim(m,s,t))
                  enddo
                  call cpvxz(psi(dzpsi(s,t)),natur(dzpsi(s,t)), &
                       dblock(s,t))
                  do m=1,nmode
                     call qttxzz(dicht4(ddmat(m,s,t)),natur(dzpsi(s,t)), &
                          workc,vddim(m,s,t),ddim(m,s,t),nddim(m,s,t))
                     call cpvxz(workc,natur(dzpsi(s,t)),dblock(s,t))
                  enddo
               enddo
            enddo
         else 
            do s=1,nstate
               do m=1,nmode
                  call mmxczz(psi(zetf(m,s)),dicht4(dmat(m,s)), &
                       natur(zetf(m,s)),subdim(m),dim(m,s),dim(m,s))
               enddo
               call cpvxz(psi(zpsi(s)),natur(zpsi(s)),block(s))
               do m=1,nmode
                  call qttxzz(dicht4(dmat(m,s)),natur(zpsi(s)), &
                       workc,vdim(m,s),dim(m,s),ndim(m,s))
                  call cpvxz(workc,natur(zpsi(s)),block(s))
               enddo
            enddo
         endif
      endif

!-----------------------------------------------------------------------
! calculate indices saved for compact output
!-----------------------------------------------------------------------
      if (lpsicm .and. dentype .eq. 1) then
         d=0
         i=0
         do s=1,nstate
            do t=1,nstate
               dblockcm(s,t)=0
               do b=1,dblock(s,t)
                  d=d+1
                  absnat=dble(natur(d))*dble(natur(d)) + &
                       dimag(natur(d))*dimag(natur(d))
                  if (absnat .gt. psicut) then
                     dblockcm(s,t)=dblockcm(s,t)+1
                     i=i+1
                     worki(i)=b
                     workc(i)=natur(d)
                  endif
               enddo
            enddo
         enddo
         adimcm=0
         do s=1,nstate
            do t=1,nstate
               adimcm=adimcm+dblockcm(s,t)
            enddo
         enddo
         i=adimcm+1
         do d=adim+1,dgldim
            workc(i)=natur(d)
            i=i+1
         enddo
         call sum2vxz(workc,sum,adimcm)
         normcm=dble(sum)
         normcm=sqrt(normcm)
      else if (lpsicm) then
         d=0
         i=0
         do s=1,nstate
            blockcm(s)=0
            do b=1,block(s)
               d=d+1
               absnat=dble(natur(d))*dble(natur(d)) + &
                   dimag(natur(d))*dimag(natur(d))
               if (absnat .gt. psicut) then
                  blockcm(s)=blockcm(s)+1
                  i=i+1
                  worki(i)=b
                  workc(i)=natur(d)
               endif
            enddo
         enddo
         adimcm=0
         do s=1,nstate
            adimcm=adimcm+blockcm(s)
         enddo
         i=adimcm+1
         do d=adim+1,dgldim
            workc(i)=natur(d)
            i=i+1
         enddo
         call sum2vxz(workc,sum,adimcm)
         normcm=dble(sum)
         normcm=sqrt(normcm)
      endif

!-----------------------------------------------------------------------
! Write Data Set
!-----------------------------------------------------------------------
      countpsi=countpsi+1

      write(wrtag,'(a9,i10,a1)') '<data set',countpsi,'>'
      write(ipsi) wrtag
      
!-----------------------------------------------------------------------
! write indices for compact or S-MCTDH wavefunction
!-----------------------------------------------------------------------
      if (lpsicm) then
         wrtag='<compact indices>'
         write(ipsi) wrtag
         zeig=0
         dgldimcm=dgldim
         do s=1,nstate
            write(ipsi) blockcm(s)
            write(ipsi) (worki(zeig+b), b=1,blockcm(s))
            zeig=zeig+blockcm(s)
            dgldimcm=dgldimcm - (block(s)-blockcm(s))
         enddo
         write(ilog,*)
         write(ilog,'(a,f8.3,a)') 'Psi written at ',time/fs,'fs'
         write(ilog,'(a,i7,a,f8.5)') 'No. coeffs / Norm = ',adimcm,'/', &
              normcm
      else if (lselect) then
         wrtag='<sci indices>'
         write(ipsi) wrtag
         write(ipsi) ((jindx(zpsi(s)+b-1), b=1,block(s)),s=1,nstate)
      endif

!-----------------------------------------------------------------------
! write width matrix for FGA
!-----------------------------------------------------------------------
      if (lgwp) then
         if (gwptype .ge. 3) then
            wrtag='<agmat>'
            write(ipsi) wrtag
            write(ipsi) (agmat(d),d=1,agmatdim)
         endif
         wrtag='<ngwpdep>'
         write(ipsi) wrtag
         write(ipsi) ((ngwpdep(m,s),m=1,nmode),s=1,nstate)
         wrtag='<gwpdep>'
         write(ipsi) wrtag
         write(ipsi) (gwpdep(d),d=1,gwpinfodim)
      endif

!-----------------------------------------------------------------------
! write state information for trajectory points
!-----------------------------------------------------------------------
      if (ltraj) then
         wrtag='<trajst>'
         write(ipsi) wrtag
         write(ipsi) (trajst(d),d=1,gwpinfodim)
      endif                                                                     

!-----------------------------------------------------------------------
! If dynamical WF: write extra info about the wavefunction
!-----------------------------------------------------------------------
      if (ldynspf) then
         wrtag= '<dimtape>'
         write(ipsi) wrtag
         write (ipsi) ((dim(m,s),m=1,nmode),s=1,nstate)
         if (lrunml) then
            write(ipsi) tapelength()
            write(ipsi) (tape(m),m=1,tapelength())
         endif
      endif

!-----------------------------------------------------------------------
! write wavefunction
!-----------------------------------------------------------------------
      wrtag='<psi>'
      write(ipsi) wrtag
      if (lpsisp .and. lpsinat) then
         write(ipsi) (cmplx(natur(d)), d = 1,dgldim)
      else if (lpsisp .and. .not. lpsinat) then
         write(ipsi) (cmplx(psi(d)), d = 1,dgldim)
      else if (lpsinat) then
         write(ipsi) (natur(d), d = 1,dgldim)
      else if (lpsicm) then
         zeig=0
         dgldimcm=dgldim
         if (dentype .eq. 1) then
            do s=1,nstate
               do t=1,nstate
                  dgldimcm=dgldimcm - (dblock(s,t)-dblockcm(s,t))
               enddo
            enddo
         else
            do s=1,nstate
               dgldimcm=dgldimcm - (block(s)-blockcm(s))
            enddo
         endif
         write(ipsi) (cmplx(workc(d)), d = 1,dgldimcm)
      else
         write(ipsi) (psi(d), d = 1,dgldim)
      endif

      write(wrtag,'(a13,i10,a1)') '<end data set',countpsi,'>'
      write(ipsi) wrtag
      call myflush(ipsi)

      if (allocated(workc)) deallocate(workc)
      if (allocated(worki)) deallocate(worki)

      return
      end subroutine psiout


      end module psioutmod

