!-----------------------------------------------------------------------
!                        SUBROUTINE RUNPROP
!
!-----------------------------------------------------------------------

module runpropmodules

use decimal, only: dop, long
use global
use dglobal
use globmemmod
use logdat
use op1lib,  only: zerovxz, zerovxd, zerovxi, zerovxl
#ifdef MPI
use mpi
#endif
use maxv
use dvrdatmod
use griddatmod
use psidef
use datenmod
use compdatmod
use runpropmod
use dirdyn
use channels 
use hpsimod
use operdef
use gwptrj
use gwptrjmod
use gapmod
use logintmod
#ifdef MPI
use mpidata
use mpipropmod, only: mpiprop
use propbcastmod
#endif
use iorst, only: rdrst,rstinfo,wrrst
use rdopermod
use mempropmod, only: memprop,checkmemprop
use rddvrmod, only: rddvr,dvrdata,dvrinfo
use initpropmod, only: initprop
use opendatmod, only: opendat
use prop, only: propagatepsi
use diaghammod, only: diagham
use einintmod, only: einintprop,einintdiag
use crocoflymod
use crocofly, only: rdccref2
use zeigpropmod, only:zeigprop
use d1zeigpropmod, only:d1zeigprop
use multilayer, only: setoperml
use wftree1, only: allocate_wftree
use timing
use iopsidef, only: psidat
use inputmlmod, only: inputml
use dwfn

use linintmod
use linint, only: li_load

use op1lib, only: cpmxi
use logopmod

    use BEL, only:alloc_beldat

end module runpropmodules

!#######################################################################

      subroutine runprop

use runpropmodules

      implicit none

      integer(long)     :: ilbl,chkdvr,chkgrd,chkpsi,chkprp,lanczord,iflag
      character(len=16)        :: progpart
!
! psi(dgldim) holds wavefunction
!
      integer(long)                              :: i,aerr
      character(len=80)                          :: actual
      character(len=15)                          :: cerr

      complex(dop), dimension(:), allocatable    :: Kryc,HKryc
      real(dop), dimension(:), allocatable       :: Krylov,HKrylov

      complex(dop), dimension(:), allocatable    :: psi,dtpsi,psi1

      character(len=c5)                          :: filename
      real(dop)                                  :: time
      real(dop), dimension(1)                    :: rdum,rdum1
      logical(kind=4)                            :: linwf,lerr,check
      logical(kind=4), external                  :: int2log

      integer(long), external   :: integersize,realsize,complexsize,logicalsize

      routine='Runprop'

      aerr = 0
!-----------------------------------------------------------------------
! MPI slaves do not read files
!-----------------------------------------------------------------------
#ifdef MPI
   if (mpirank.gt.0) go to 111
#endif

!-----------------------------------------------------------------------
! Initialise timing arrays
!-----------------------------------------------------------------------
      call timinit

      tpropwf=0
      call get_timer(tpropwf, 'PropWF')
      call start_timer(tpropwf)

      call fzeit(actual)
      i = index(actual,':')
      actual=actual(i-9:i+6)
      if (lrundiag) then
         write(ilog,'(/,72a1)')  ('#',i=1,72)
         write(ilog,'(/,a)') ' Diagonalising Hamiltonian '
         write(ilog,'(57x,a)')  actual(1:15)
         write(ilog,'(72a1)')  ('#',i=1,72)
      else if (lrlx) then
         write(ilog,'(/,72a1)')  ('#',i=1,72)
         write(ilog,'(/,a)') ' Wavefunction Relaxation '
         write(ilog,'(57x,a)')  actual(1:15)
         write(ilog,'(72a1)')  ('#',i=1,72)
      else
         write(ilog,'(/,72a1)')  ('#',i=1,72)
         write(ilog,'(/,a)') ' Propagating Wavefunction '
         write(ilog,'(57x,a)')  actual(1:15)
         write(ilog,'(72a1)')  ('#',i=1,72)
      endif

!-----------------------------------------------------------------------
! Check input file open for reading
!-----------------------------------------------------------------------
      inquire(iin,opened=check)
      if (check) close(iin)
      filename=inputfile(1:inplaenge)
      open(iin,file=filename,status='old',form='formatted',err=900)

!-----------------------------------------------------------------------
! get array dimensions 
!-----------------------------------------------------------------------
      inquire(irst,opened=check)
      if (check) close(irst)
      filename=name(1:laenge)//'/restart'
      ilbl=index(filename,' ')-1
      open(irst,file=filename(1:ilbl),form='unformatted',status='old',err=900)
      call rdmemdim(irst)
      close(irst)

!-----------------------------------------------------------------------
! get memory sizes for gwp trajectory calculations
!-----------------------------------------------------------------------
      if (lgwptrj) call memgwptrj

!-----------------------------------------------------------------------
! broacast memory dimensions
! and flags required for memory allocation
!-----------------------------------------------------------------------
#ifdef MPI
  111 call dimbcast(mpirank)
      !call MPI_Bcast(maxpack,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(dentype,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(lgwp,1,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(lmemold,1,MPI_LOGICAL,master,MPI_COMM_WORLD,mpierr)
#endif

!-----------------------------------------------------------------------
! Allocate memory 
!-----------------------------------------------------------------------
      allocmemory=0
      call alloc_dvrdat
      call alloc_grddat
      call alloc_operdef
      if (lgwptrj) call alloc_gwptrj
      call alloc_psidef
      if (linitorb .or. ldynspf) call alloc_dynwf
      call alloc_runpropmod
      call alloc_compdat
      call alloc_hpsimod
      if (ldd.or.lddmctdh) then
         call alloc_dirdyn
         call alloc_dddb
         call alloc_dercp
      endif
      if (lddmctdh.or.lgausspot)then
         call alloc_gapkernel
         if(ldiabdb) call alloc_dirdyn_ddmctdh
      endif
      if (lcross) call alloc_crocofly

!-----------------------------------------------------------------------
! MPI slaves do not read files
!-----------------------------------------------------------------------
#ifdef MPI
   if (mpirank.gt.0) go to 222
#endif

!-----------------------------------------------------------------------
! Read system / DVR information
!-----------------------------------------------------------------------
      filename=dname(1:dlaenge)//'/dvr'
      ilbl=index(filename,' ')-1
      open(idvr,file=filename,form='unformatted',status='old',err=900)
      chkdvr=1
      call dvrinfo(lerr,chkdvr)
      close(idvr)

!-----------------------------------------------------------------------
! Read data from oper file
!-----------------------------------------------------------------------
      filename=oname(1:olaenge)//'/oper'
      ilbl=index(filename,' ')-1
      open(ioper,file=filename,form='unformatted',status='old',err=900)
      chkdvr=2
      chkgrd=1
      call operinfo(lerr,chkdvr,chkgrd)
      close(ioper)

!-----------------------------------------------------------------------
! Read data from restart file
! If geninwf has been run restart file is in name directory, if not
! get it from rname directory
!-----------------------------------------------------------------------
      if (iruninwf .eq. 1) then
         filename=name(1:laenge)//'/restart'
      else
         filename=rname(1:rlaenge)//'/restart'
      endif
      ilbl=index(filename,' ')-1
      open(irst,file=filename,form='unformatted',status='old',err=900)
      chkdvr=2
      chkgrd=2
      chkpsi=1
      if (lcont .and. .not. leinint) then
         chkprp=1
      else
         chkprp=0
      endif
      call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
      close(irst)

      if (lBEL) call alloc_beldat !allocate elements for BEL

!-----------------------------------------------------------------------
! set default values for propagation variables
!-----------------------------------------------------------------------
      call propdefaults

!-----------------------------------------------------------------------
! Read integrator information
!-----------------------------------------------------------------------
if (lBEL) write(1000,*)"BELflags",BELflags(:),"source/propwf/runprop.F90"
      if (.not. lcont .or. leinint) then
         if (lrundiag) then
            call einintdiag
         else
            call einintprop
         endif
      endif
if (lBEL) write(1000,*)"BELflags",BELflags(:),"source/propwf/runprop.F90"

!-----------------------------------------------------------------------
! Close input file
!-----------------------------------------------------------------------
      close(iin)

!-----------------------------------------------------------------------
! check propagation variables
!-----------------------------------------------------------------------
      if (.not. lrundiag) call chkprop

!-----------------------------------------------------------------------
! Set up operator for calculation dependant on WF + prop settings
!-----------------------------------------------------------------------
      call setoperpsi
      call setoperprop

!-----------------------------------------------------------------------
! calculate dimensions of multi-D operators
!-----------------------------------------------------------------------
      call opermddim

!-----------------------------------------------------------------------
! Set up memory arrays
!-----------------------------------------------------------------------
      if ((dentype.eq.0).or.(dentype.eq.2)) then
         call zeigprop
      else if (dentype .eq. 1) then
         call d1zeigprop
      endif
      call memprop

#ifdef MPI
! Broadcast variables to slave nodes.
  222 continue
 !     if (mpirank.eq.master) then
 !        call mastbcast1(mpirank)
 !     else
 !        call slvbcast1(mpirank)
 !     endif
      if (mpirank.ne.master) call slvbcast1(mpirank)
#endif

if (lBEL) write(1000,*)"lBEL",lBEL,"source/propwf/runprop.F90"
if (lBEL) write(1000,*)"mcdim",mcdim,"source/propwf/runprop.F90"
if (lBEL) write(1000,*)"dgldim",dgldim,"source/propwf/runprop.F90"
if (lBEL) write(1000,*)"dim",dim(1:nmode,1:nstate),"source/propwf/runprop.F90"
      if(allocated(psi))deallocate(psi)
      if(allocated(psi1))deallocate(psi1)
      if(allocated(dtpsi))deallocate(dtpsi)
      if(allocated(mc))deallocate(mc)
      if(allocated(mr))deallocate(mr)
      if(allocated(mi))deallocate(mi)
      if(allocated(ml))deallocate(ml)
      if(allocated(gfgwp))deallocate(gfgwp)
      if(allocated(aigwp))deallocate(aigwp)
      if(allocated(zgwp))deallocate(zgwp)
      if(allocated(hteil))deallocate(hteil)
      if(allocated(mfsum))deallocate(mfsum)
      if(allocated(gh))deallocate(gh)
      if(allocated(ghderiv1))deallocate(ghderiv1)
      if(allocated(gs2))deallocate(gs2)
      if(allocated(gsinv))deallocate(gsinv)
      if(allocated(gsvek))deallocate(gsvek)
      if(allocated(gsdot))deallocate(gsdot)
      if(allocated(hung))deallocate(hung)
      if(allocated(dicht1))deallocate(dicht1)
      if(allocated(dicht2))deallocate(dicht2)
      if(allocated(dicht3))deallocate(dicht3)
      if(allocated(dicht4))deallocate(dicht4)
      if(allocated(dicht3o))deallocate(dicht3o)
      if(allocated(dicht4o))deallocate(dicht4o)

      if(allocated(Kryc))deallocate(Kryc)
      if(allocated(HKryc))deallocate(HKryc)
      if(allocated(Krylov))deallocate(Krylov)
      if(allocated(HKrylov))deallocate(HKrylov)

      cerr = 'runprop_psi  '
      allocate(psi(dgldim),stat=aerr)
      allocmemory=allocmemory + (dgldim)*complexsize()
      if (aerr > 0) go to 1000
      cerr = 'runprop_psi1 '
      allocate(psi1(dgldim),stat=aerr)
      allocmemory=allocmemory + (dgldim)*complexsize()
      if (aerr > 0) go to 1000
      cerr = 'runprop_dtpsi'
      allocate(dtpsi(dgldim),stat=aerr)
      allocmemory=allocmemory + (dgldim)*complexsize()
      if (aerr > 0) go to 1000
      cerr = 'runprop_mc  '
      allocate(mc(mcdim+1),stat=aerr)
      allocmemory=allocmemory + mcdim*complexsize()
      if (aerr > 0) go to 1000
!     cerr = 'runprop_mc1 '
!     allocate(mc1(mcdim+1),stat=aerr)
!     allocmemory=allocmemory + mcdim*complexsize()
!     if (aerr > 0) go to 1000
      cerr = 'runprop_mr  '
      allocate(mr(mrdim+1),stat=aerr)
      allocmemory=allocmemory + mrdim*realsize()
      if (aerr > 0) go to 1000
!     cerr = 'runprop_mr1 '
!     allocate(mr1(mrdim+1),stat=aerr)
!     allocmemory=allocmemory + mrdim*realsize()
!     if (aerr > 0) go to 1000
      cerr = 'runprop_mi  '
      allocate(mi(midim+1),stat=aerr)
      allocmemory=allocmemory + midim*integersize()
      if (aerr > 0) go to 1000
!     cerr = 'runprop_mi1 '
!     allocate(mi1(midim+1),stat=aerr)
!     allocmemory=allocmemory + midim*integersize()
!     if (aerr > 0) go to 1000
      cerr = 'runprop_ml  '
      allocate(ml(mldim+1),stat=aerr)
      allocmemory=allocmemory + mldim*logicalsize()
      if (aerr > 0) go to 1000
      cerr = 'runprop_hteil  '
      allocate(hteil(pmatdim),stat=aerr)
      allocmemory=allocmemory + pmatdim*complexsize()
      if (aerr > 0) go to 1000
      if (lconst) then
         if (dentype .eq. 1) then
            cerr = 'runprop_mfsum  '
            allocate(mfsum(3*mfsumdim),stat=aerr)
            allocmemory=allocmemory + mfsumdim*complexsize()
            if (aerr > 0) go to 1000
         else
            cerr = 'runprop_mfsum  '
            allocate(mfsum(mfsumdim),stat=aerr)
            allocmemory=allocmemory + mfsumdim*complexsize()
            if (aerr > 0) go to 1000
         endif
      endif
      if (lgwp) then
         cerr = 'runprop_gfgwp  '
         allocate(gfgwp(gfgwpdim+1),stat=aerr)
         allocmemory=allocmemory + gfgwpdim*complexsize()
         if (aerr > 0) go to 1000
         if (.not. lBEL) then
            cerr = 'runprop_aigwp  '
            allocate(aigwp(aigwpdim+1),stat=aerr)
            allocmemory=allocmemory + aigwpdim*complexsize()
            if (aerr > 0) go to 1000
            cerr = 'runprop_zgwp  '
            allocate(zgwp(zgwpdim+1),stat=aerr)
            allocmemory=allocmemory + zgwpdim*complexsize()
            if (aerr > 0) go to 1000
            cerr = 'runprop_gsdot  '
            allocate(gsdot(gsmatdim+1),stat=aerr)
            allocmemory=allocmemory + 2*gsmatdim*complexsize()
            if (aerr > 0) go to 1000
            cerr = 'runprop_gsvek '
            allocate(gsvek(gsmatdim+1),stat=aerr)
            allocmemory=allocmemory + gsmatdim*complexsize()
            if (aerr > 0) go to 1000
            cerr = 'runprop_gsvek '
            allocate(ghderiv1(ghder1dim),stat=aerr)
            allocmemory=allocmemory + ghder1dim*complexsize()
            if (aerr > 0) go to 1000
         else
            allocate(aigwp(1),stat=aerr)
            allocate(zgwp(1),stat=aerr)
            allocate(gsdot(1),stat=aerr)
            allocate(gsvek(1),stat=aerr)
            allocate(ghderiv1(1),stat=aerr)
         endif
         cerr = 'runprop_gh  '
         allocate(gh(ghmatdim),stat=aerr)
         allocmemory=allocmemory + ghmatdim*complexsize()
         if (aerr > 0) go to 1000
         cerr = 'runprop_hung  '
         allocate(hung(hungdim+1),stat=aerr)
         allocmemory=allocmemory + hungdim*complexsize()
         if (aerr > 0) go to 1000
         cerr = 'runprop_gs2 '
         allocate(gs2(gs2matdim),stat=aerr)
         allocmemory=allocmemory + gs2matdim*complexsize()
         if (aerr > 0) go to 1000
         cerr = 'runprop_gsinv  '
         allocate(gsinv(gsmatdim),stat=aerr)
         allocmemory=allocmemory + gsmatdim*complexsize()
         if (aerr > 0) go to 1000
         cerr = 'runprop_agmat '
         allocate(agmat(agmatdim),stat=aerr)
         allocmemory=allocmemory + agmatdim*complexsize()
         if (aerr > 0) go to 1000
      endif
      cerr = 'runprop_dicht1 '
      allocate(dicht1(dmatdim),stat=aerr)
      allocmemory=allocmemory + dmatdim*complexsize()
      if (aerr > 0) go to 1000
      cerr = 'runprop_dicht2 '
      allocate(dicht2(dmatdim),stat=aerr)
      allocmemory=allocmemory + dmatdim*complexsize()
      if (aerr > 0) go to 1000
      cerr = 'runprop_dicht3 '
      allocate(dicht3(d3matdim),stat=aerr)
      allocmemory=allocmemory + d3matdim*realsize()
      if (aerr > 0) go to 1000
      cerr = 'runprop_dicht4 '
      allocate(dicht4(dmatdim),stat=aerr)
      allocmemory=allocmemory + dmatdim*complexsize()
      if (aerr > 0) go to 1000
      if (lautoe .and. lautodoub) then
         cerr = 'runprop_dicht3o'
         allocate(dicht3o(d3matdim),stat=aerr)
         allocmemory=allocmemory + d3matdim*realsize()
         if (aerr > 0) go to 1000
         cerr = 'runprop_dicht4o'
         allocate(dicht4o(dmatdim),stat=aerr)
         allocmemory=allocmemory + dmatdim*complexsize()
         if (aerr > 0) go to 1000
      endif

      if (lBEL) write(1000,*)"gsmatdim",gsmatdim
      if (lBEL) write(1000,*)"gs2matdim",gs2matdim
      if (lBEL) write(1000,*)"ghmatdim",ghmatdim
      if (lBEL) write(1000,*)"pmatdim",pmatdim
      if (lBEL) write(1000,*)"hungdim",hungdim
      if (lBEL) write(1000,*)"dmatdim",dmatdim

      allocate(Kryc(ikryc))
      allocate(HKryc(ikryc))
      allocate(Krylov(ikrylov))
      allocate(HKrylov(ikrylov))
 
!-----------------------------------------------------------------------
! allocate space for operator-related matrices in the multilayer modules
! 
! 1 = number of system Hamiltonian
! .true. = allocate space for mfields, dmats, etc.
!-----------------------------------------------------------------------
      if(lrunml) then
        call allocate_wftree(tree,.true.,dicht1,dicht2,dicht3,dicht4)
        call setoperml(1,.true.,.true.)
      endif

!-----------------------------------------------------------------------
! MPI-slaves skip to propagation
!-----------------------------------------------------------------------
#ifdef MPI
      if (mpirank.ne.master) go to 500
#endif

!-----------------------------------------------------------------------
! write memory usage to log
!-----------------------------------------------------------------------
      progpart='runprop'
      call logmem(progpart)

!-----------------------------------------------------------------------
! Zero memory arrays
!-----------------------------------------------------------------------
      psi(:) = 0.0_dop
      dtpsi(:) = 0.0_dop
      mc(:) = 0.0_dop
      if (lgwp) then
         gfgwp(:) = 0.0_dop
         gh(:) = 0.0_dop
         gs2(:) = 0.0_dop
         gsinv(:) = 0.0_dop
         if (.not. lBEL) then
            aigwp(:) = 0.0_dop
            zgwp(:) = 0.0_dop
            gsdot(:) = 0.0_dop
            dicht1(:) = 0.0_dop
            dicht4(:) = 0.0_dop
            hung(:) = 0.0_dop
            gsvek(:) = 0.0_dop
            ghderiv1(:) = 0.0_dop
         endif
      endif
      hteil(:) = 0.0_dop
      if (lconst) mfsum(:) = 0.0_dop
      dicht1(:) = 0.0_dop
      dicht2(:) = 0.0_dop
      dicht3(:) = 0.0_dop
      dicht4(:) = 0.0_dop
      mr(:) = 0.0_dop
      mi(:) = 0
      ml(:) = .false.

!     initialise check variables in the mc array
      call checkmemprop(.false.,0)

!-----------------------------------------------------------------------
! Zero propagation variables
!-----------------------------------------------------------------------
      estart=0.0_dop

!-----------------------------------------------------------------------
! Read DVR data needed to propagate wavefunction (see rddvr.f for
! details)
!-----------------------------------------------------------------------
      dvrdata(:) = .false.
      dvrdata(1)=.true.
      dvrdata(2)=.true.
      dvrdata(4)=.true.
      dvrdata(5)=.true.
      dvrdata(6)=.true.
      dvrdata(7)=.true.
      dvrdata(8)=.true.
      dvrdata(9)=.true.
      dvrdata(11)=.true.
      dvrdata(12)=.true.
      chkdvr=1
      call rddvr(mr(mrort),mr(mrtrafo),rdum,mr(mrfftp), &
           mc(mchin),mc(mcrueck),mi(mifftfak),mc(mcexphin), &
           mc(mcexprueck),mi(mijsph),mi(mimsph),rdum1,chkdvr)

!-----------------------------------------------------------------------
! Read data needed by the operator
! if ML-MCTDH do not re-initialise grid. Otherwise this must be 
! doen for multipack
!-----------------------------------------------------------------------
      if (psitype .eq. 5) then
         chkdvr=2
         chkgrd=2
      else
         chkdvr=2
         chkgrd=1
      endif
      call rdoper(mr(mrhops),chkdvr,chkgrd)

!-----------------------------------------------------------------------
! Set up operator again for calculation dependant on WF + prop settings
!-----------------------------------------------------------------------
      call setoperpsi
      if (npacket .gt. 1) nstate = nstate*npacket
      call setoperprop

!-----------------------------------------------------------------------
! lopsetprop = true. if operator changed in setoperprop.
!-----------------------------------------------------------------------
      if (lopsetprop) then
         call fillkoeff(mr(mrhops))   ! refill koeff array
         write(ilog,'(/,a)') &
             ' *** Operator changed due to form of psi ***'
         write(ilog,'(a)') '     See op.log.psi for details '
         call logop
      endif

!-----------------------------------------------------------------------
! get data for gwp trajectory calculations
!-----------------------------------------------------------------------
      if (lgwptrj) call gwptrjfit

!-----------------------------------------------------------------------
! For Initord or DynSPF,
! Save sizes of maximum size WF
!-----------------------------------------------------------------------
      if (linitorb .or. ldynspf) then
         maxdimspwn(:,:) = dim(:,:)
         tapeM(:) = tape(:)
         dglmax=dgldim
      endif

!-----------------------------------------------------------------------
! Get initial wavefunction
! for continuation, take lrlx and tinit from restart file.
!
! Check WF fits system data
!-----------------------------------------------------------------------
      if (iruninwf .eq. 1) then
         filename=name(1:laenge)//'/restart'
      else
         filename=rname(1:rlaenge)//'/restart'
      endif
      open(irst,file=filename,form='unformatted',status='old',err=900)
      chkdvr=2
      if (lexact) then
         chkgrd=1 ! because getex may have changed the DOF-order
         chkpsi=1
      else if (psitype .eq. 5) then
         chkgrd=2
         chkpsi=2
      else
         chkgrd=2
         chkpsi=2
      endif
      if (ldynspf) chkpsi = 1
      iflag=1
      chkprp=0
      call rdrst(time,psi,linwf,chkdvr,chkgrd,chkpsi,chkprp, &
                 mc(mcintpsi),lanczord,mr(mrworkr),iflag,mi(mijindx), &
                 agmat,mi(mitrajst))
      close(irst)

! -- added to allow restart runs with external1d for time.
      if(irunoper .eq. 2)  call reinitext1d
! -- preload the field buffers (otherwise threads will mess up data) 
      call preloadli

      if (lcont) tinit=time
      if (.not. lrlx .and. dentype .gt. 0) temperature0 = temperature_psi

!-----------------------------------------------------------------------
! initialise any further information
!-----------------------------------------------------------------------
      call initprop(mr(mrhops),mi(migwpdyn))

!-----------------------------------------------------------------------
! Change norm if requested
!-----------------------------------------------------------------------
      if (initnorm .gt. 0.0) then
         call xvxxdzo(initnorm,psi,adim)
         write(ilog,'(/,a,f12.4)') &
             'Initial Norm Set at : ',initnorm
      endif

!-----------------------------------------------------------------------
! get reference wavefunction (for cross-correlation function)
!-----------------------------------------------------------------------
      if (lcross) call rdccref2
      
!-----------------------------------------------------------------------
! log propagation information and open output data files.
!-----------------------------------------------------------------------
      call logint
      if (ldynspf) call logdynwf
      call opendat

!-----------------------------------------------------------------------
! Set gpopcap for continuation.
!-----------------------------------------------------------------------
      if (lcont) gpopcap(:)=1.d-10

!-----------------------------------------------------------------------
! Start dynamical SPF calculation
!-----------------------------------------------------------------------
      if (ldynspf) then
         call dynwfsizes
         idyncyc=0
      endif

!-----------------------------------------------------------------------
! End short test
!-----------------------------------------------------------------------
      if (test0) then
         write(iout,'(/,a,/)') 'Short Test run was successful.'
         write(ilog,'(/,a,/)') '***** Test run *****'
         write(ilog,'(a,/)')   'Short test run completed.'
         goto 600
      endif

#ifdef MPI
      call mastbcast1(mpirank)
#endif
!-----------------------------------------------------------------------
! Propagate wavefunction
!-----------------------------------------------------------------------
  500 continue

!-----------------------------------------------------------------------
! Broadcast information in MPI-run
!-----------------------------------------------------------------------
#ifdef MPI
      if (mpirank.eq.master) then
         call MPI_Bcast(dgldim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(adim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call mastbcast2(mpirank)
      else if(mpirank.ne.master)then
         call MPI_Bcast(dgldim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(adim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call slvbcast2(mpirank)
         call mpiprop(psi,dtpsi,Krylov,HKrylov,Kryc,HKryc)
         if (ldynspf) then  ! if spawning, run again
            goto 500
         else
            goto 888
         endif
      endif
#endif

      if (lrundiag) then
         call diagham(lanczord,psi)
      else
         call propagatepsi(psi,dtpsi,Krylov,HKrylov,Kryc,HKryc)
      endif

!-----------------------------------------------------------------------
! Increase size of WF in a dynWF calculation
!-----------------------------------------------------------------------
      if (ldynspf .and. .not. test) then
         call callspwn(psi,dtpsi)
         lcont = .true.   ! flag as continuation run
! get slaves to return from mpiprop
#ifdef MPI    
         lmpiupdt(2) = .true. ! update mpi data for hlochphi
         mpitask=0
         call MPI_Bcast(mpitask,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
#endif
         go to 500
      endif

  600 continue ! End calculation
      call dealloc_dvrdat
      call dealloc_grddat
      call dealloc_operdef
      call dealloc_gwptrj
      call dealloc_psidef
      call dealloc_dynwf
      call dealloc_runpropmod
      call dealloc_compdat
      call dealloc_hpsimod
      call dealloc_dirdyn
      call dealloc_dddb
!      call dealloc_gapop
      call dealloc_gapkernel
      call dealloc_crocofly
      call dealloc_dercp
      call dealloc_setoperpsi

      cerr = 'runprop_kry'
      deallocate(Kryc,HKryc,Krylov,HKrylov,stat=aerr)
      if (aerr > 0) go to 1001

!----------------------------------------------------------------------
!     check the variables in the mc array
!----------------------------------------------------------------------
      call checkmemprop(.true.,0)

!-----------------------------------------------------------------------
! flag end of run and finish timing
!-----------------------------------------------------------------------
      if (ltime) then
         call stop_timer(tpropwf)
         timers(tpropwf)%ncalls=1
         call timout
      endif

      cerr = 'runprop_...'
      deallocate(psi,dtpsi,mc,mr,mi,ml,hteil,&
                 dicht1,dicht2,dicht3,dicht4,stat=aerr)
      if (lgwp) deallocate(gfgwp,aigwp,zgwp,gh,gs2,gsinv,gsvek,gsdot, &
                hung,agmat,stat=aerr)
      if (allocated(dicht3o)) deallocate(dicht3o,stat=aerr)
      if (allocated(dicht4o)) deallocate(dicht4o,stat=aerr)
      if (allocated(mfsum)) deallocate(mfsum)
      if (aerr > 0) go to 1001

#ifdef MPI
      if (mpirank.eq.master.and..not.lrealwp) then
         mpitask=0
         call MPI_Bcast(mpitask,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      endif
#endif

      return

!-----------------------------------------------------------------------
! End for MPI-slaves
!-----------------------------------------------------------------------
#ifdef MPI
  888 continue
! Deallocations
      if(allocated(psi))deallocate(psi)
      if(allocated(psi1))deallocate(psi1)
      if(allocated(dtpsi))deallocate(dtpsi)
      if(allocated(mc))deallocate(mc)
      if(allocated(mr))deallocate(mr)
      if(allocated(mi))deallocate(mi)
      if(allocated(ml))deallocate(ml)
      if(allocated(gfgwp))deallocate(gfgwp)
      if(allocated(aigwp))deallocate(aigwp)
      if(allocated(zgwp))deallocate(zgwp)
      if(allocated(hteil))deallocate(hteil)
      if(allocated(mfsum))deallocate(mfsum)
      if(allocated(gh))deallocate(gh)
      if(allocated(ghderiv1))deallocate(ghderiv1)
      if(allocated(gs2))deallocate(gs2)
      if(allocated(gsinv))deallocate(gsinv)
      if(allocated(gsvek))deallocate(gsvek)
      if(allocated(gsdot))deallocate(gsdot)
      if(allocated(hung))deallocate(hung)
      if(allocated(dicht1))deallocate(dicht1)
      if(allocated(dicht2))deallocate(dicht2)
      if(allocated(dicht3))deallocate(dicht3)
      if(allocated(dicht4))deallocate(dicht4)

      if(allocated(Kryc))deallocate(Kryc)
      if(allocated(HKryc))deallocate(HKryc)
      if(allocated(Krylov))deallocate(Krylov)
      if(allocated(HKrylov))deallocate(HKrylov)


      return
#endif

  900 continue
      routine='Runprop'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
      stop

 1000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',aerr
      write (6,*) 'Workcdim',workcdim
      write (6,*) 'mcdim',mcdim
      stop

 1001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',aerr
      stop

      end subroutine runprop

!----------------------------------------------------------------------
! Early loading of external1d data. Otherwise different threads may
! start reading the data file(s) simultaneously and mess things up.
!----------------------------------------------------------------------

      subroutine preloadli

use runpropmodules

      implicit none

      integer(long) :: i,getlong

      do i=1,li_total
         !li_init(i): is buffer i initialized?
         if(getlong(li_init, i).eq.0) call li_load(i)
      end do

      return
      end subroutine preloadli

!***********************************************************************

      subroutine opermddim

use runpropmodules

      implicit none

      integer(long) :: h,k,f,muld,gf,hl,i,m,vdim1,s,nham
      character(len=c1), dimension(maxdim)       :: label
      integer(long), external :: slen
      logical(kind=4)         :: lfirst

!-----------------------------------------------------------------------
! look for muld terms
!-----------------------------------------------------------------------
      lmuld=.false.
      do nham=1,hamdim
         do k = zham(nham)+kzahl(nham),  &
                zham(nham)+kzahl(nham)-1+nmulpot(nham)
            f = fterm(k)
            h=hamilton(f,k)
            muld=htmmuld(h)
            if(muld.gt.0  .and.  htmmode(h).eq.0) lmuld=.true.
          enddo
      enddo

      if (lmuld) then
         call psidat
         write(ilog,'(/a)')  &
             'Coordinate sets for multi-dimensional muld operators'
         write(ilog,'(2a)') 'Label      nham    k    h muld  vd  ', &
                             '    Size    nd    Dofs'
         do nham=1,hamdim
            do k = zham(nham)+kzahl(nham), zham(nham)+kzahl(nham)+nmulpot(nham)-1
               f = fterm(k)
               h=hamilton(f,k)
               muld=htmmuld(h)
               if(muld.gt.0  .and.  htmmode(h).eq.0) then
                  lfirst=.true.
                  s=ki(k)
                  vdim1=1
                  do m=1,nmode
                     if (diag(m,k)) then    ! Does not work for nodiag
                        vdim1=vdim1*dim(m,s)
                     else
                        if(lfirst) then
                           lfirst=.false.
                           vdmuld(muld)=vdim1
                        endif
                        vdim1=vdim1*subdim(m)
                     endif
                  enddo
                  ndmuld(muld)=vdim1/(vdmuld(muld)*muldsize(muld))
                  do gf=1,nmulddof(muld)
                     f=mulddof(gf,muld)
                     label(gf)=modelabel(f)
                  enddo
                  i=1
 44               hl=hterm(i,h)
                  if(hopfile(hl).lt.200 .or. hopfile(hl).gt.300) then
                     i=i+1
                     goto 44
                  endif
                  write(ilog,'(a12,i3,2i5,i3,i6,i10,i6,12(2x,a))')  &
                     hoplab(hl),nham,k,h,muld, &
                     vdmuld(muld),muldsize(muld),ndmuld(muld), &
                     (label(gf)(1:slen(label(gf))),gf=1,nmulddof(muld))
                  call chkmuld(nham,h,muld)
               endif         ! muld
            enddo            ! k-loop
         enddo               ! nham-loop
      endif

      end subroutine opermddim

!***********************************************************************
!
! set or change any default values that may be manually set (overridden)
! in the input.
!
!***********************************************************************

      subroutine propdefaults

use runpropmodules

      complete(:) = .false.
!      epssvd = macheps

      return
      end subroutine propdefaults

!#######################################################################
!
! CHKPROP
!
! checks the settings for the variables controlling the propagation.
!
!#######################################################################

      subroutine chkprop

use runpropmodules

      implicit none

      integer(long)                  :: s,m,rlxn,f,i
      real(dop)                      :: ratio
      logical(kind=4)                :: lflag,lflag2

      routine = 'Chkprop  (Check for consistency)'

      if (.not.(lexact .or. lrealwp .or. lgwp) .and. nmode .lt. 2)  then
         message='There should be at least two MCTDH-modes'
         call warnmsg1
      endif
   
!-----------------------------------------------------------------------
! Check that nodiag is not set for system hamiltonian.
!-----------------------------------------------------------------------
      if(nodiag(1)) then
         message='The System Hamiltonian must not be '// &
              'generated with the "nodiag" keyword'
         call warnmsg
      endif

!-----------------------------------------------------------------------
! set natural orbital flag for S-MCTDH calculations
!-----------------------------------------------------------------------
      if (citype.gt.0 .and. .not. lnaturcmf) then
         lnaturcmf = .true.
         lenorb=.false.
         write(ilog,'(a)') ' ###  S-MCTDH : setting "natorb". '
      endif
      if(citype.gt.0 .and. (lorben.or.lcross) ) then
         message='S-MCTDH is incompatible with "orben" and "cross".'
         call warnmsg
      endif

!-----------------------------------------------------------------------
! check lnaturcmf flag only set if CMF integrator
!-----------------------------------------------------------------------
      if (lnaturcmf .and. .not. lconst) then
         message='Natural orbitals only possible with CMF integrator'
         call errormsg
      endif

!-----------------------------------------------------------------------
! cannot use dynamic SPFs for a 1 mode problem
!-----------------------------------------------------------------------
      if (ldynspf .and. (nmode .eq. 1)) then
         message='Cannot use dynamical orbitals for a 1 mode problem.'
         call errormsg
      endif

!-----------------------------------------------------------------------
! Check that lenorb (energy-orbitals) flag is set for CMF integrator only
!-----------------------------------------------------------------------
      if (lenorb) then
         if (.not.lconst) Then
            message='Energy-orbitals allowed for CMF only'
            call errormsg
         endif
         if(.not.lorben) then
            message='Energy-orbitals (DAV) require orben-file'
            call errormsg
         endif
      endif

!-----------------------------------------------------------------------
! if tfinal missing, set to 0 to check rest of input
!-----------------------------------------------------------------------
      if (tfinal .le. -9.0e99_dop) then
         tfinal=0.0
         message='Missing keyword or invalid arguments: tfinal'
         call errormsg
      endif

!-----------------------------------------------------------------------
! set defaults for out1 and out2 if not input
!-----------------------------------------------------------------------
      if (abs(out1) .lt. tiny) out1=tfinal-tinit
      if (abs(out2) .lt. tiny) out2=out1
      if (louteverystep .and. abs(out2).lt.tiny)  out2=10.0_dop*fs
      if (louteverystep)  out1=out2
      if(rlaxnum.lt.0 .and. louteverystep) then
         write(6,'(/,2a,/)') '## WARNING: ',  &
          ' "tout=all" is meaningful for improved relaxation ONLY.'
         write(ilog,'(/,2a,/)') '## WARNING: ', & 
          ' "tout=all" is meaningful for improved relaxation ONLY.'
      endif

!--------------------------------------------------------------------------
! reflex (used for malonaldehyde) is only possible for 2 packets with rrDAV
!--------------------------------------------------------------------------
      if(lreflex) then
         if(rlaxnum.lt.0 .or. inttypea.ne.6 .or. .not.lrreal) then
            message='reflex is only possible for improved '// &
                    'relaxation with rrDAV'
            call warnmsg
         endif
         if(.not.leb .or. gdim(feb).ne.2) then
            message='For reflex there must be a single-set '// &
                    'electronic basis with 2 states.'
            call warnmsg
         endif
      endif

!-----------------------------------------------------------------------
! Special treatment for improved relaxation.
!-----------------------------------------------------------------------
      if(rlaxnum.ge.0)  then
         if(nmode.eq.1) then
!            message='Improved Relaxation is not possible for 1D '//&
!                    '(or 1-mode) problems.                      '//&
!                    'For 1D diagonalisations use eigenf or meigenf'//&
!                    ' in a geninwf run. (See HTML docu).'
!            call warnmsg   ! Dieter 24.01.14
            message='Improved Relaxation is not meaningful (though '//&
            'possible when "no-redundancy-check" is set) for 1D or '//&
            '1-mode problems. Use eigenf or meigenf instead.'
            call warnmsg1   ! Dieter 24.01.14
         endif
         if(.not. lcont .and. .not.test)  tinit=tinit-out2
         if(lexact)  then
            message='Improved Relaxation requires an MCTDH '// &
                    'wavefunction, not one in "exact" format.'
            call warnmsg
         endif
         if(.not.lconst)  then
            message='Improved Relaxation requires CMF.'
            call warnmsg
         endif
         if(inttypea.eq.3 .and. lcap)  then
            message='Improved Relaxation with SIL requires '// &
                    'Lanczos-integrator (not Arnoldi). Use CDAV'
            call warnmsg
         endif
         if(inttypea.ne.3 .and. inttypea.ne.6)  then
            message='Improved Relaxation requires '// &
                    'SIL, DAV, RDAV, or CDAV for A-integration.' 
            call warnmsg
         endif
         if(lcap .and. mpb.gt.0) then
            message='Block Improved Relaxation is not possible '// &
                    'for non-hermitian Hamiltonians, i.e. for CDAV.'
            call warnmsg
         endif

         adim=0
         do s=1,nstate
            block(s)=1
            do m=1,nmode
               block(s)=block(s)*dim(m,s)
            enddo
            adim=adim+block(s)
         enddo

         if(adim == 1) then
            message='Improved relaxation is not possible for Hartree'// &
                    ' calculations. Use normal relaxation.'
            call warnmsg
         endif

         if(intordera .ge. adim  .and. .not. lcont) then
            write(6,'(/,a)') '## WARNING: '
            write(6,'(2a)') 'Davidson iteration order ', &
                            'must be samller than A-vector length!'
            write(6,'(a,i5,a,i6)')'IntOrder =',intordera,', adim =',adim
            write(6,'(a)') 'IntOrder is reduced to adim-1'
            write(ilog,'(2a)') 'WARNING: Davidson iteration order ', &
                              'must be smaller than A-vector length!'
            write(ilog,'(a,i5,a,i6)') 'IntOrder =',intordera, &
                                      ', adim =',adim
            write(ilog,'(a)') 'IntOrder is reduced to adim-1'
            intordera = adim - 1
         endif

         if(precon .gt. adim/npackts) then
            write(6,*) 'precon=',precon,' , PsiDim=',adim/npackts
            message='precon must not be larger than PsiDim, (adim).'
            call warnmsg
         endif

         if(lautoblk) then
            do m=2,nmode
               if(symcv(m) .ne. symcv(1) ) then
                  message='Partial (a)symmetrisation is not'// &
                          ' compatible with autoblock.'
                  call warnmsg
               endif
            enddo
         endif
      endif   ! (rlaxnum.gt.0)

!-----------------------------------------------------------------------
!     Set complete if allcomplete is .true.  Check for CMF/fix
!-----------------------------------------------------------------------
      if(allcomplete) then
         do m=1,nmode
            complete(m)=.true.
         enddo
         if(.not.lconstfix) then
           message='allcomplete requites CMF/fix as integration scheme!'
            call warnmsg
         endif
      endif

!-----------------------------------------------------------------------
!     Check for consistency
!-----------------------------------------------------------------------
      if(lsvd .and. nmulpot(1).gt.0) then
         message = 'SVD-regularization is not implemented for'// &
                   ' non-SOP multi-dimensional (muld) potentials.'
         call warnmsg
      endif
      if (lrlx .and. rlaxnum .ge. 0 .and.  &
                          .not.(lconstfix.or.lcmfadaphi) ) then
         message = 'improved relaxation (relaxation="number")'//  &
                   ' requires CMF/fix or CMF/varphi integration.'
         call errormsg
      endif

      if (tfinal .le. tinit ) Then
         message = 'tfinal must be larger than tinit'
         call errormsg
      endif
      if (out1 .le. tiny ) Then
         message = 'tout must be positive'
         call warnmsg
      endif
      if (out2 .le. tiny ) Then
         message = 'tpsi must be positive'
         call warnmsg
      endif


      ratio = (tfinal-tinit)/out1
      if (abs(ratio-nint(ratio)) .gt. tiny*ratio) then
         write(6,'(a)') ' *** WARNING : inconsistent times ***'
         write(6,'(a,f13.6,/a,f13.6,/a,f13.6)') 'tinit  =', &
            tinit/fs, 'tfinal =', tfinal/fs, 'tpsi   =', out2/fs
         tfinal = tinit + NInt(Ratio+0.3_dop)*out1
         write(6,'(a,/a,f13.6,a)') &
        ' Ratio (tfinal-tinit)/tout must be an integer number', &
        ' tfinal changed to :', tfinal/fs,' fs'
         if(.not.test0) write(ilog,'(a,/a,f13.6,a)') &
        ' Ratio (tfinal-tinit)/tout must be an integer number', &
        ' tfinal changed to :', tfinal/fs,' fs'
      endif

      ratio = (tfinal-tinit)/out2
      if (abs(ratio-nint(ratio)) .gt. tiny*ratio .and. .not.lrealwp) Then
      message='Ratio (tfinal-tinit)/tpsi must be an integer number'
         call warnmsg
         write(6,'(a,f13.6,/a,f13.6,/a,f13.6)') 'tinit  =', &
            tinit/fs, 'tfinal =', tfinal/fs, 'tpsi   =', out2/fs
      endif
      ratio = out2/out1
      if (abs(ratio-nint(ratio)) .gt. tiny*ratio .and. .not.lrealwp) then
         message = 'Ratio tpsi/tout must be an integer number'
         call warnmsg
         write(6,'(a,f13.6,/a,f13.6)')'tout =',out1/fs,'tpsi =',out2/fs
      endif

      if( .not.lstopf .and. &
           (sstop.ne.' ' .or. scpu.ne.' ' .or. swall.ne.' ') ) then
         message = 'You must open the stop-file when specifying ' &
                   //'tstop or tcpu or twall.'
         call warnmsg
      end if

      if (lconstfix) then
         ratio = out1/updatetime
         if (abs(ratio-nint(ratio)) .gt. tiny*ratio) Then
            message = &
         'Ratio of tout/updatetime must be an integer number'
            call warnmsg
            write(6,'(a,f13.6,/a,f13.6)') 'tout       =',out1/fs, &
                                    'updatetime =',updatetime/fs
         endif
      endif
      if (lconst) then
         if(updatetime .gt. out1+1.0e-7_dop*out1 ) then
            write(6,'(a,/a,/a,f13.6,/a,f13.6,/a)') '### WARNING ###', &
                 'updatetime must be smaller than tout.', &
                 'tout       =',out1/fs,'updatetime =',updatetime/fs, &
                 'Setting  updatetime equal to tout.'
            write(ilog,'(a,/a,/a,f13.6,/a,f13.6,/a)')'### WARNING ###', &
                 'updatetime must be smaller than tout.', &
                 'tout       =',out1/fs,'updatetime =',updatetime/fs, &
                 'Setting  updatetime equal to tout.'
            updatetime = out1
         elseif(updatetime .gt. out1) then
            updatetime = out1
         endif

         if (lconstfix) then
            ratio = out1/updatetime
            if (dabs(ratio-nint(ratio)) .gt. tiny*ratio) then
               write(6,'(a,/a,/a,f13.6,/a,f13.6)') &
                    '### WARNING ###', &
                    'tout must be a multiple of updatetime.', &
                    'tout       =',out1/fs, &
                    'updatetime =',updatetime/fs
               updatetime = out1/(nint(ratio)+1)
               write(6,'(a,f13.6)') &
                    'Decreasing updatetime to',updatetime/fs
            endif
         endif
      endif

!-----------------------------------------------------------------------
! check consistency of input
!-----------------------------------------------------------------------
      if(loptcntrl) then
         if((irunprop .ne. 1) .or. lrlx .or. dentype.ne.0) then
            message = 'optcntrl requires WF propagation'
            call warnmsg
         endif
      endif
      if(lexact .and. lorben) then
         message='"orben" not allowed in "exact" calculations!'
         call warnmsg
      endif
      rlxn = (rlaxnum+1)/10000
      if(lolsen .and. inttypea.eq.3) then
         message = &
           'olsen not allowed for relaxation with SIL. Use RDAV.'
         call warnmsg
      endif
      if(lquad .and. inttypea.eq.3) then
         message = &
           'quad not allowed for relaxation with SIL. Use RDAV.'
         call warnmsg
      endif
      if(lortho .and. inttypea.eq.6) then
         message='ortho not allowed for relaxation with DAV/RDAV/CDAV.'
         call warnmsg
      endif
      if(rlaxnum .gt. 10930) then
         message = 'rlaxnum too large. Set full only once.'
         call warnmsg
      endif
      rlxn = mod(rlaxnum+2,10000) - 2
      if ( inttypea .eq. 6 ) then
         if( lreal .and. lcap ) then
            message = 'Real Davidson (RDAV) not for complex'&
                 //' Hamiltonians (CAP)'
            call warnmsg
         endif
         if( lreal .and. (.not.lrealphi .or. inttypephi.lt.4) ) then
            lflag=.false.
            lflag2=.false.
            do  f=1,ndof
               if(basis(f).eq.4) lflag=.true.
               if(basis(f).eq.9) lflag2=.true.
            enddo
            if(lflag) then
               message='FFT can be used in combination with RDAV only' &
                //' if realphi is set in Run-Section and RK8/spf with' &
                //' imp-ortho is used.'
               call warnmsg
            endif
            if(lflag2) then
               message='If PLeg is used in combination with RDAV' &
                //' or RRDAV, then it is recommend to set "realphi"' &
                //' in Run-Section and to use' &
                //' "RK8/spf=1.d-9,<initial-step>,imp-ortho".'
               call warnmsg1
            endif

         endif
         if(.not.lrlx .or. .not.lconst .or. rlaxnum.lt.0) then
            message = 'Davidson only for improved relaxation'&
                 //' with CMF/varphi or CMF/fix'
            call warnmsg
         endif
         if(rlxn.lt.0 .or. (rlxn.gt.920 .and. rlxn.ne.930)) then
            message = 'Davidson requires: relaxation=n (n=0...900)'&
                 //'  or  relaxation=follow  or  relaxation=lock'
            call warnmsg
         endif
         if(rlxn .eq. 930  .and.  .not.lcross ) then
            message = 'relaxation=lock requires that cross '&
                 //'is given.'
            call warnmsg
         endif
      endif
      if(rlxn .eq. 930  .and.  inttypea.eq.3) then
         message='lock not implemented for SIL, use DAV/RDAV/CDAV.'
         call warnmsg
      endif
      if (lrlx) Then
         if (lfunkl .and. .not. lgwp) then
            message = 'Relaxation cannot use "h-proj" variant'
            call warnmsg
         endif
      endif
      if (lconst) Then
         if (lrunml) then
            if(lrlx .and. rlaxnum.ge.0) then
               message = 'Improved Relaxation not implemented for ML'
               call warnmsg
            endif
            if (lconstfix) then
               message = 'CMF/fix not implemented for ML'
               call warnmsg
            endif
            if (lcmfadaphi.or.lcmfadaa) then
               message = 'ML does not distinguish between Phi and A, '&
                    //'hence only CMF/var makes sense'
               call warnmsg
            endif
            if (lfunkl) then
               message = 'h-proj not implemented for ML'
               call warnmsg
            endif
            if (lenorb) then
               message = 'energyorb unsupported for ML'
               call warnmsg
            endif
         else if (lconstvar .and. inttypea.ne.3 .and. inttypea.ne.6) then
            message = &
                  'Adaptive constant mean-fields only possible with Lanczos'
             call warnmsg
         endif
      endif
      if (lmlcmfsplit) then
         if (.not.lconst .or. .not. lrunml) then
            lmlcmfsplit = .false.
         endif
      endif

      if (lautodoub .and. .not. lconst) then
         message = 'auto = twice only allowed with CMF integrator'
         call warnmsg
      endif

      lflag=.false.
      if (lcmfadaphi) Then
         if(accphi .gt. 1.51_dop*epsupdate) lflag=.true.
      elseif(lcmfadaa) Then
         if(accavec .gt. 1.51_dop*epsupdate) lflag=.true.
      elseif(lconstvar) Then
         if(max(accphi,accavec) .gt. 1.51_dop*epsupdate) lflag=.true.
      endif  
      if(lflag) then
         message = 'CMF accuracy parameter should not be better than' &
              //' integrator accuracies' 
         call warnmsg1
         write(6,'(2a)') 'WARNING: ', message
         lflag=.false.
      endif  

      if(lrealphi) then
         if(lrlx .and. rlaxnum.ge.0 .and. inttypea.eq.6 .and.lreal) then
            continue
         else
            message = 'realphi only allowed for impr.relax. with RDAV or RRDAV'
            call warnmsg
         endif
      endif
 
!-----------------------------------------------------------------------
! Warning for use of unsymmetric Hamiltonians and autocorrelation
! functions
!-----------------------------------------------------------------------
      if (lauto) then
         routine='Chkprop'
         message= 'Symmetric Hamiltonian and real initial state '// &
           'assumed when calculating autocorrelation function. '// &
           'Use "cross" if these conditions are not fulfilled.'
         call warnmsg1
      endif

!-----------------------------------------------------------------------
! Warning for lack of autocorrelation error with ML
!-----------------------------------------------------------------------
      if (lautoe .and. lrunml) then
         routine='Chkprop'
         message = 'auto=error (autoe) unsupported for ML'
         call warnmsg
      endif

!-----------------------------------------------------------------------
! Set states limits for cross-correlation calculation.
!-----------------------------------------------------------------------
      if (lcross) then
         if (nst1.eq.-999) then
            nst1 = 1
            nst2 = nstate
         elseif(nst1.le.0 .or. nst2.gt.nstate) then
            routine='Chkprop'
            message= 'Invalid value for cross-correlation state'
            write(6,*) 'nst1 =', nst1,'  nst2 =', nst2
            call warnmsg
         endif
      endif

!-----------------------------------------------------------------------
! Check for correct setting of the hamLMR array
!-----------------------------------------------------------------------
       do i=1,iLMR
          if(hamLMR(i,1).eq.0) then
             write(6,'(a,i3,a,3i4)') 'chkprop: iLMR =',i, &
              ', hamLMR:',hamLMR(i,1),hamLMR(i,2),hamLMR(i,3)
             message='There must be an R term for the LMR operator set.'
             call warnmsg
          endif
          if(hamLMR(i,1).gt.0 .and. hamLMR(i,3) .eq.0) then
             write(6,'(a,i3,a,3i4)') 'chkprop: iLMR =',i, &
              ', hamLMR:',hamLMR(i,1),hamLMR(i,2),hamLMR(i,3)
             message='There must be an L term for the LMR operator set.'
             call warnmsg
          endif
       enddo

!-----------------------------------------------------------------------
! some propagation options not allowed for ML wavefunctions
!-----------------------------------------------------------------------
      if (lrunml) then
         if (lpsiopt(3)) then
            routine = 'ChkProp (Runprop.F90)'
            message = &
         'Cannot store an ML wavefunction with natural orbitals (option psi=natur)'
            call warnmsg
         endif
         if (lpsiopt(2)) then
            routine = 'ChkProp (Runprop.F90)'
            message = &
         'Cannot store an ML wavefunction in compact format (option psi=compact)'
            call warnmsg
         endif
      endif

!-----------------------------------------------------------------------
! check if unimplemented features were requested
!-----------------------------------------------------------------------

!     General imp-ortho currently only for ML.
      if (limportho) then
         if (.not.(lrunml .or. lgwp)) then
            routine='Chkprop'
            write(message,'(2a)') &
               'General imp-ortho not supported for standard MCTDH.', &
               'Switching to RK-specific imp-ortho.'
            call warnmsg1
            limportho = .false.
            liork = .true.
         endif
      endif

!     RK-specific imp-ortho needs RK5 or RK8
      if (liork) then
         if (lconst) then
            if (inttypephi.ne.4 .and. inttypephi.ne.5) then
               routine='Chkprop'
               write(message,'(3a)') &
                    'Integrator chosen for SPFs doesn''t '// &
                    'support imp-ortho.', &
                    'Switching imp-ortho off.'
               call warnmsg1
               liork = .false.
            endif
         else
            if (inttyp.ne.4 .and. inttyp.ne.5) then
               routine='Chkprop'
               write(message,'(2a)') &
                    'Integrator doesn''t support imp-ortho.', &
                    'Switching imp-ortho off.'
               call warnmsg1
               liork = .false.
            endif
         endif
      endif

      if (lcross) then
         if (lgwp) then
            routine='Runprop'
            message='Cannot calculate cross-correlation in a GWP calculation'
            call errormsg
         endif
      endif

!-----------------------------------------------------------------------
! After any call of 'warnmsg' the logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      if (lwarn) then
         message = 'Error in propagation input. STOP.'
         call errormsg
      endif

      return
      end subroutine chkprop 

