!#######################################################################
      module zeigpropmod

use maxv, only:maxnhtmshift
use decimal, only: long
use global
use dglobal
use globmemmod
use logdat
use dvrdatmod
use griddatmod
use operdef
use psidef
use datenmod
use channels
use hpsimod
use iopsidef, only: psidat,gwpdat
use psiutil, only: maxmatwk
use crocofly, only: zeigcroco
use zeighpsimod, only: operflag,zeighpsi,d2zeighpsi

use runinwfmod
use runpropmod
use hpsimod
use dirdyn

use mfieldsmod, only: wkmfields
use function, only: wkfunction
use getdiagonalmod, only: wkgetdiag
use gaussian, only: wkgaussian
use diaghammod, only: wkdiagham
use orbenmod, only: wkorben
use mpidata

      implicit none
      private
      public :: zeigprop,wksil,intdimens
      contains

!#######################################################################
! Zeiger (i.e. pointer) routines for propagation step.
!#######################################################################

      subroutine zeigprop

      implicit none

      integer(long)       :: i,m,s,s1,zeig,f,n

!-----------------------------------------------------------------------
! set various operator related flags (kfirst etc.)
!-----------------------------------------------------------------------
      do i=1,hamdim
         call operflag(i)
      enddo

!-----------------------------------------------------------------------
! get pointers for wavefunction. vdim, ndim, zpsi, etc. 
!-----------------------------------------------------------------------
      call psidat

!-----------------------------------------------------------------------
! see if direct dynamics calculation
!-----------------------------------------------------------------------
      if (ldd .and. .not. lgwp) then
         routine="Zeigprop"
         message= "Direct dynamics only possible using GWPs"
         call errormsg
      endif

!-----------------------------------------------------------------------
! flag any complete spf bases
!-----------------------------------------------------------------------
      do m=1,nmode
         if (.not. gwpm(m) .and. subdim(m) .eq. dim(m,1)) then
            complete(m)=.true.
            do s=2,nstate
               dim(m,s)=subdim(m)
            enddo
         endif
      enddo

!-----------------------------------------------------------------------
! set up pointers for hpsi operation (and for hteil, hloch etc)
!-----------------------------------------------------------------------
      if (.not. lexact) then
         hpsidim  = 0
         hlochdim = 0
         pmatdim  = 0
         ghmatdim = 0
         ghder1dim = 0
         if (dentype.eq.0) then
            do i=1,hamdim
               call zeighpsi(i)
            enddo
         else if (dentype.eq.2) then
            do i=1,hamdim
               call d2zeighpsi(i)
            enddo
         endif
      else
! Exact calculations where an expectation value of an operator(e.g. flux) 
! is requested.
         if(hamdim.gt.1)then
            hpsidim  = 0
            hlochdim = 0
            pmatdim  = 0
            ghmatdim = 0
            ghder1dim = 0
            if (dentype.eq.0) then
               do i=2,hamdim
                  call zeighpsi(i)
               enddo
            else if (dentype.eq.2) then
               do i=2,hamdim
                  call d2zeighpsi(i)
               enddo
            endif
         else
            hpsidim  = 1
            hlochdim = 1
            pmatdim  = 1      
            ghmatdim = 1
            ghder1dim = 1
         endif
      endif
         
!-----------------------------------------------------------------------
! zgwpdyn(m,s): pointers to arrays flagging dynamic GWPs
!-----------------------------------------------------------------------
      zeig=1
      do s=1,nstate
         do m=1,nmode
            if (gwpm(m))then
               zgwpdyn(m,s)=zeig
               zeig=zeig+dim(m,s)*(npg(m)-1)
            endif
         enddo
      enddo
      gwpdyndim=max(zeig-1,1)

!-----------------------------------------------------------------------
! pointers for overlap matrices of single-particle functions
!-----------------------------------------------------------------------
      if (lautoord1 .or. lautoord2) then
         zeig = 1
         do s = 1,nstate
            do m = 1,nmode
               zspfovr(m,s) = zeig
               zeig = zeig+dim(m,s)**2
            enddo
         enddo
         spfovrdim = zeig-1
      else
         spfovrdim = 1
      endif

!-----------------------------------------------------------------------
! scan operator for the occuring htmshift values
!-----------------------------------------------------------------------
      call findhtmshift

!-----------------------------------------------------------------------
! pointers for tensor containing sum over mean-field matrices
!-----------------------------------------------------------------------
      if (lconst) then
         zeig = 1
         do m = 1,nmode
            do s = 1,nstate
               do s1 = 1,nstate
                  if(idmode(m).eq.0) then
                     zmfsum(m,s1,s) = zeig
                     do n=1,nhtmshift(m)
                        zmfsum1(m,s1,s,n) = zeig
                        zeig = zeig + subdim(m)*dim(m,s)*dim(m,s1)
                     enddo
                  else
                     zmfsum(m,s1,s) = zmfsum(idmode(m),s1,s) 
                     do n=1,nhtmshift(idmode(m))
                        zmfsum1(m,s1,s,n) = zmfsum1(idmode(m),s1,s,n)
                     enddo
                  endif 
               enddo
            enddo
         enddo
         mfsumdim = zeig-1
      else
         mfsumdim = 1
      endif

!-----------------------------------------------------------------------
! pointers for GWP arrays
!-----------------------------------------------------------------------
      call gwpdat

!-----------------------------------------------------------------------
! pointers for x-local arrays (needed for analysis)
!-----------------------------------------------------------------------
      zeig = 1 
      do s = 1,nstate 
         do m = 1,nmode 
            f = spfdof(1,m)  
            if (basis(f) .eq. 26) then
               zpcons(f)=zeig
               zeig=zeig+gdim(f)*gdim(f)
            else if (basis(f) .eq. 27) then
               zpcons(f)=1
            endif
         enddo
      enddo
      pconsdim=zeig
           
!-----------------------------------------------------------------------
! calculate maximum "block" size needed in matrizen routines
!-----------------------------------------------------------------------
      maxblock = 0
      do i=1,hamdim
         call maxmatwk(i)
      enddo

!-----------------------------------------------------------------------
! calculate size of intpsi array
!-----------------------------------------------------------------------
      call intdimens

!-----------------------------------------------------------------------
! obtain size needed for calculation of cross-correlation-function
!-----------------------------------------------------------------------
      if (lcross) call zeigcroco

!-----------------------------------------------------------------------
! calculate workspace
!-----------------------------------------------------------------------
      if (dentype.eq.0) then
         call wkprop
      else if (dentype.eq.2) then
         call d2wkprop
      endif

      write(ilog,'(a,i11,f13.3,a)') &
        ' hopsdim  =',hopsdim,  hopsdim* 7.6294e-6_dop,' MB'
      write(ilog,'(a,i11,f13.3,a)') &
        ' dgldim   =', dgldim,   dgldim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' pmatdim  =', pmatdim, pmatdim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' hpsidim  =', hpsidim, hpsidim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' hlochdim =',hlochdim,hlochdim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f13.3,a)') &
        ' mfsumdim =',mfsumdim,mfsumdim*15.2588d-6,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' intpsidim=',intpsidim,intpsidim*15.2588e-6_dop,' MB'
      if(intrdim.ge.40)  write(ilog,'(a,i11,f13.3,a)') &
        ' intrdim  =',intrdim,intrdim*7.6294d-6,' MB'
      if(ikrylov.ge.40)  write(ilog,'(a,i11,f13.3,a)') &
        ' ikrylov  =',ikrylov,ikrylov*7.6294d-6,' MB'
      if(ikryc  .ge.40)  write(ilog,'(a,i11,f13.3,a)') &
        ' ikryc    =',ikryc,ikryc*15.2588d-6,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' workcdim =',workcdim,workcdim*15.2588e-6_dop,' MB'
      write(ilog,'(a,i11,f11.3,a)') &
        ' workrdim =',workrdim, workrdim*7.6294e-6_dop,' MB'
      if(workidim .gt. 100) &
       write(ilog,'(a,i11,f11.3,a)') &
        ' workidim =',workidim, workidim*3.8147e-6_dop,' MB'

      return
      end subroutine zeigprop


!#######################################################################
!
! set minimum array sizes to check available workspace
!
!#######################################################################

      subroutine wkprop

      implicit none

      workcdim=1
      workrdim=1
      workidim=1
      workldim=1

      if (lrundiag) then
         call wkdiagham
      else
         call wkmfields
         call wkfunction
         if (lgwp) call wkgaussian
         if (lorben) call wkorben
         call wksil
         if (inttypea .eq. 6) call wkgetdiag
         if (lmpi) workrdim=max(workrdim,(intordera+1)**2)
      endif

! Ensure workidim is as large as needed in calcha.
      workidim = max(workidim,2*maxblock)

      return
      end subroutine wkprop

!#######################################################################

      subroutine d2wkprop

      implicit none

      workcdim=1
      workrdim = 1
      workidim = 1
      workldim = 1
      call wksil

      return
      end subroutine d2wkprop


!#######################################################################
      subroutine wksil

      implicit none

!.....Davidson integrator.

      if (inttypea.eq.6 .and. .not.lreal .and. mpb.gt.0) then ! BDav
!!!! -- THIS NEEDS TO BE CHECKED ! (23.03.2010) -------------------
         workrdim=max(workrdim,24*max(intordera,precon)+48)
         workrdim=max(workrdim,2*precon+adim/npackts)
         workcdim=max(workcdim,24*max(intordera,precon)+48)
         workcdim=max(workcdim,2*precon+adim/npackts)
         workidim=max(workidim,2*precon+16*npackts+20)
      elseif (inttypea.eq.6 .and. lreal .and. mpb.gt.0) then  ! BrDav
         workrdim=max(workrdim,24*max(intordera,precon)+48)
         workrdim=max(workrdim,2*precon+adim/npackts)
         if(lrreal) workrdim=max(workrdim,2*adim)     ! BrDav/rrDAV
         workidim=max(workidim,2*precon+16*npackts+20)
      elseif (inttypea.eq.6 .and. lcap .and. mpb.gt.0) then  ! BcDav
         write(6,'(a)') ' Block-C-DAV not implemented'
         write(2,'(a)') ' Block-C-DAV not implemented'
         stop
      elseif (inttypea.eq.6 .and. lreal ) then        ! rDavison
         workrdim=max(workrdim,24*max(intordera,precon)+48)
         workrdim=max(workrdim,2*precon+adim)
!          workrdim=max(workrdim,2*adim)  ! for precon2 (Dieter) ###
         workidim=max(workidim,precon)
         if(lrreal) workrdim=max(workrdim,2*adim)     ! rrDAV
      elseif (inttypea.eq.6 .and. lcap ) then         ! cDavison
         workrdim=max(workrdim,3*intordera+4)
!........workc + TTmat + EigenVal
         workcdim=max(workcdim,(intordera+1)**2 + 17*intordera+19)
      elseif (inttypea.eq.6 .and. .not.lreal ) then   ! Davison
         workrdim=max(workrdim,3*intordera+4)
         workcdim=max(workcdim,16*intordera+18)
      endif


      if (lcap.or.ldissin) then
!.....CSIL : AuxMat
         if (inttypea.eq.3) &
             workcdim=max(workcdim,(intordera+1)**2)
         if (inttypephi.eq.3) &
             workcdim=max(workcdim,(intorderphi+1)**2)
      else
!.....SIL : Eigenvector-matrix
         if (inttypea.eq.3)    &
             workrdim=max(workrdim,(intordera+1)*(intordera+3))
         if (inttypephi.eq.3)  &
             workrdim=max(workrdim,(intorderphi+1)*(intorderphi+3))
      endif
   
      return
      end subroutine wksil


!#######################################################################

      subroutine intdimens

      implicit none

      integer(long)       :: icr, adim2
      real(dop)           :: rkrylov,rkryc

      intpsidim = 1
      intrdim   = 1
      icr       = 1
      ikrylov   = 1
      ikryc     = 1
      rkrylov   = 1.d0
      rkryc     = 1.d0
      if(mod(rlaxnum,10000).eq.930) icr = 2               ! lock
      if (lconst) then
         if (inttypea .eq. 1) then                        ! ABM
            intpsidim = max(intpsidim,adim*(intordera+1))
         elseif (inttypea .eq. 2) then                    ! BS
            intpsidim = max(intpsidim,adim*(intordera+2))
         elseif (inttypea .eq. 3) then                    ! SIL / CSIL
            intpsidim = max(intpsidim,adim*(intordera-1))
            if(lcap) then
               intpsidim = intpsidim + 2*(intordera+1)**2 ! CSIL: EigVec,Hess
            else
               intrdim = max(intrdim,4*(intordera+1))     ! SIL: EigVal,Diag..
            endif
         elseif (inttypea .eq. 4 .or. inttypea .eq. 41) then                    ! RK5
            intpsidim = max(intpsidim,adim*7)
         elseif (inttypea .eq. 5) then                    ! RK8
            intpsidim = max(intpsidim,adim*12)
!
! ------------ BDAV (block Davidson hermitian)    -------------------
         elseif (inttypea.eq.6 .and. .not.lreal .and. mpb.gt.0) then ! BDAV
            intpsidim = max(intpsidim,(adim/npackts) &    ! HDiagonal
                      +  ((intordera+1)*(intordera+2))/2 &! Tmat
                      +  (intordera+1)**2 )              ! EigenVec   
            ikryc = (adim/npackts)*(intordera+1)         ! Krylov,HKrylov
            rkryc = dble(adim/npackts)*dble(intordera+1)
            if (lmpi.and.lmpidav) then
               intordermpi=max(intordermpi,intordera/nmpi+1)
               ikryc=(adim/npackts)*(intordermpi+1)
               rkryc=dble(adim/npackts)*dble(intordermpi+1)
            endif
!...........RDiagonal,Tmat,EigenVector,EigenVal,ovlps,davmat
            intrdim = max(intrdim,adim/npackts + &
                      ((intordera+1)*(3*intordera+8))/2 + &
                      precon*(precon+1) )
            if(lquad) intrdim=intrdim+((intordera+1)*(intordera+2))/2
            if(lorben) intpsidim = max(intpsidim,dmatdim)
!
! ------------ BRDAV (block Davidson real)    -----------------------
         elseif (inttypea.eq.6 .and. lreal .and. mpb.gt.0) then ! BRDAV
            if(lreflex) then
               adim2=adim/(2*npackts)
            else
               adim2=adim/npackts
            endif
            intpsidim = max(intpsidim,adim2)     ! HDiagonal
            ikrylov   = adim2*(intordera+1)      ! Krylov,HKrylov
            rkrylov   = dble(adim2)*dble(intordera+1)
            if (lmpi.and.lmpidav) then
               intordermpi=max(intordermpi,intordera/nmpi+1)
               ikrylov=adim2*(intordermpi+1)
               rkrylov=dble(adim2)*dble(intordermpi+1)
            endif
!...........RDiagonal,Tmat,EigenVector,EigenVal,ovlps,davmat
            intrdim = max(intrdim,adim2 + &
                      ((intordera+1)*(3*intordera+8))/2 + &
                      precon*(precon+1) )
            if(lquad) intrdim=intrdim+((intordera+1)*(intordera+2))/2
            if(lorben) intpsidim = max(intpsidim,dmatdim)
!
! ------------   DAV, cDAV (hermitian and complex Davidson) ---------
         elseif (inttypea .eq. 6 .and. .not.lreal) then   ! DAV, cDAV
!...........HDiagonal,Tmat,EigenVector
            intpsidim = max(intpsidim,icr*adim + 2*((intordera+1)**2))
!...........Krylov,HKrylov
            if (lmpi.and.lmpidav.and.lcap) then
               intordermpi=max(intordermpi,intordera/nmpi+2)
               ikryc = adim*intordermpi
               rkryc = dble(adim)*dble(intordermpi)
            else
               ikryc = adim*intordera
               rkryc = dble(adim)*dble(intordera)
            endif
!...........Eigenval (DAV only), ovlps (DAV and cDAV) 
            intrdim = max(intrdim,intordera+1)
            if(precon > 0 .and. lcap) then                 ! for cdav
               intpsidim = intpsidim+precon*(precon+1)     ! davmat
               intpsidim = intpsidim+2*precon*precon       ! leigf,reigf
            elseif (precon > 0) then                       ! for dav
               intpsidim = intpsidim+precon*(precon+1)     ! davmat
            end if                                         ! Letitia
            if(.not.lcap) intrdim = max(intrdim,2*intordera+2)
            if(lorben)  intpsidim = max(intpsidim,dmatdim)
!
! ------------ RDAV (Davidson real)    ------------------------------
         elseif (inttypea .eq. 6 .and. lreal) then        ! rDAV
            if(lreflex) then
               adim2=adim/2
            else
               adim2=adim
            endif
            intpsidim = max(intpsidim,icr*adim2) ! HDiagonal
            ikrylov = adim2*(intordera+1)        ! Krylov,HKrylov
            rkrylov = dble(adim2)*dble(intordera+1)
!...........RDiagonal,Tmat,EigenVector,EigenVal,ovlps,davmat
            intrdim = max(intrdim,adim2 + &
                      ((intordera+1)*(3*intordera+8))/2 + &
                      precon*(precon+1) )
            if(lquad) intrdim=intrdim+((intordera+1)*(intordera+2))/2
            if(lorben) intpsidim = max(intpsidim,dmatdim)
            if (lmpi.and.lmpidav) then
               intordermpi=max(intordermpi,intordera/nmpi+1)
               ikrylov=adim2*(intordermpi+1)
               rkrylov=dble(adim2)*dble(intordermpi+1)
            endif
            intpsidim = intpsidim
         endif     ! End of loop over integrators

         if (inttypephi .eq. 1) then
            intpsidim = max(intpsidim,maxphidim*(intorderphi+1))
            intpsidim = max(intpsidim,maxgbfdim*(intorderphi+1))
         elseif (inttypephi .eq. 2) then
            if (lrunml) then
               intpsidim = max(intpsidim,dgldim*(intorderphi+2))
            else
               intpsidim = max(intpsidim,maxphidim*(intorderphi+2))
               intpsidim = max(intpsidim,maxgbfdim*(intorderphi+2))
            endif
         elseif (inttypephi .eq. 4 .or. inttypephi .eq. 41) then
            if (lrunml) then
               intpsidim = max(intpsidim,dgldim*7)
            else
               intpsidim = max(intpsidim,maxphidim*7)
               intpsidim = max(intpsidim,maxgbfdim*7)
            endif
         elseif (inttypephi .eq. 5) then
            if (lrunml) then
               intpsidim = max(intpsidim,dgldim*12)
            else
               intpsidim = max(intpsidim,maxphidim*12)
            endif
         endif
      else if (lexact .and. .not. lrundiag) then
         if (inttypephi .eq. 1) then                   ! ABM
            intpsidim = max(intpsidim,totphidim*(intorderphi+1))
         elseif (inttypephi .eq. 2) then               ! BS
            intpsidim = max(intpsidim,totphidim*(intorderphi+2))
         elseif (inttypephi .eq. 3) then               ! SIL / CSIL
            intpsidim = max(intpsidim,totphidim*(intorderphi-1))
            if(lcap.or.ldissin) then                   ! CSIL: EigVec,Hess
               intpsidim = intpsidim + 2*(intorderphi+1)**2
            else                                       ! SIL: EigVal,Diag..
               intrdim = max(intrdim,4*(intorderphi+1))
            endif
         elseif (inttypephi .eq. 4 .or. inttypephi .eq. 41) then
            intpsidim = max(intpsidim,totphidim*7)
         elseif (inttypephi .eq. 5) then
            intpsidim = max(intpsidim,totphidim*12)
         elseif (inttypephi .eq. 69) then              ! SOD
            intpsidim = max(intpsidim,dgldim)
         endif
      elseif (lrundiag) then                      ! Lanczos diagonalisation.  
         intpsidim = 2*totphidim
      else                                        ! VMF
         if (inttyp .eq. 1) then                  ! ABM
            intpsidim = max(intpsidim,dgldim*(intord+1))
         elseif (inttyp .eq. 2) then              ! BS
            intpsidim = max(intpsidim,dgldim*(intord+2))
         elseif (inttyp .eq. 4 .or. inttyp .eq. 41) then              ! RK5
            intpsidim = max(intpsidim,dgldim*7)
         elseif (inttyp .eq. 5) then              ! RK8
            intpsidim = max(intpsidim,dgldim*12)
         endif
      endif

!-----------------------------------------------------------------------
!  Check for overflow of ikrylov or ikryc
!-----------------------------------------------------------------------
      if( abs(dble(ikrylov)-rkrylov) .gt. 1.d0) then
         write(6,'(a,i12,1pe15.5)') 'ikrylov :', ikrylov,rkrylov
         write(2,'(a,i12,1pe15.5)') 'ikrylov :', ikrylov,rkrylov
         routine='intdimens, zeigprop.F'
         message='Integer overflow of ikrylov!    '//  &
          '                                       '//  &
          'Reduce numbers of SPFs and/or number of Davidson iterations.' &
          //'    Product of those numbers must be smaller than 2.14d+9.'
         call errormsg
       endif
      if( abs(dble(ikryc)-rkryc) .gt. 1.d0) then
         write(6,'(a,i12,1pe15.5)') 'ikryc :', ikryc,rkryc
         write(2,'(a,i12,1pe15.5)') 'ikryc :', ikryc,rkryc
         routine='intdimens, zeigprop.F'
         message='Integer overflow of ikryc       '//  &
          '                                       '//  &
          'Reduce numbers of SPFs and/or number of Davidson iterations.'
         call errormsg
       endif

      return
      end subroutine intdimens

! ----------------------------------------------------------------------
!
! FINDHTMSHIFT
!
! Scans the operator terms for occurances of side-diagonal matrices
! and stores the occuring htmshift values for each mode:
!   ihtmshift(i,m) = i-th occuring htmshift value for mode m
!   nhtmshift(m)   = number of occuring htmshift values
! This data is needed to later sum the products of mean-field matrices
! times side-diagonal operators.
!
! 08/2006 FO
!
! ----------------------------------------------------------------------
      subroutine findhtmshift

      implicit none

      integer(long)       :: k,m,n,f,h,i
      logical(kind=4)     :: lerr

      lerr=.false.

!     initialize arrays: first htmshift value is zero (diagonal terms)
      do m=1,nopmode
         nhtmshift(m)=1
         ihtmshift(1,m)=0
      enddo

!     scan operator
      do k=1,ktot
         do m=1,nopmode
            do n=1,nspfdof(m)
               f=spfdof(n,m)
               h=max(1,hamilton(f,k))   ! LMR
!              only consider terms operating on the full mode,
!              others won't be summed anyway
               if ((nspfdof(m).eq.1 .or. htmmode(h).eq.m).and. &
                   (htmtype(h).eq.12)) then
!                  check if we already have this htmshift
                   do i=1,nhtmshift(m)
                      if (ihtmshift(i,m).eq.htmshift(h)) goto 50
                   enddo
!                  increase counter and store the htmshift
                   if (nhtmshift(m).lt.maxnhtmshift) then
                      nhtmshift(m)=nhtmshift(m)+1
                      ihtmshift(nhtmshift(m),m)=htmshift(h)
                   else
                      lerr=.true.
                   endif
  50               continue
               endif
            enddo
         enddo
      enddo

!     bail if error
      if (lerr) then
         routine='findhtmshift'
         write (message,'(a)') 'Increase maxnhtmshift in maxv.f90'
         call errormsg
      endif

      return
      end subroutine findhtmshift  ! of findhtmshift


!#######################################################################

      end module zeigpropmod

