!----------------------------------------------------------------------
! KES ArAr potential 1
!----------------------------------------------------------------------
!      From the paper:
!      Accurate ab initio ptoential for argon dimer including highly
!      repulsive region
!      K. Patkowski, G. Murdachaew, C-M. Fou and K. Szalewicz
!      Mol. Phys. 103 15-16 2005 2031-2045
!      Using Equations 5 and 6, and parameters from Table 2
!
!      Equation 5:
!              (------------------term1----------------------)
!              (----term1apar-----)(-------term1exp----------)
!       V(R) = (A + {A}'R + {A}''R)e^{-\alpha R + \beta R^{2}}
!                 - \sum_{n=3}^{8} f_{2n}(bR) \frac{C_{2n}}{R^{2n}}
!                  (--loop on n--)(-dampfunc-)(-----assymconst----)
!                   (--------------- term2------------------------)
!       where (Equation 6):
!       f_{2n}(x) = 1 - e^{-x} \sum_{k=0}^{2n} \frac{x^k}{k!}
!
!      Variables:
!      term1      : (A+A'R+A''/R)exp(-aR+bR^2) or
!                   (A+AprR+Aprpr/R)exp(-alphaR+betaR^2)
!      term1apar  : Term 1, A parameters - (A+A'R+A''/R)
!      term1exp   : Term 1, exponential - exp(-aR+bR^2)
!      term2      : sum_{n=3}^8 f_{2n}(bR)C_{2n}/R^{2n}
!      dampfunc   : Tang-Toennies damping function f_{2n}(x)
!                   1- e^{-x} \sum_{n=3}^{2n} (x^k)/k!
!      kfact      : k!
!      assymconst : Assymptotic constant (C_{2n})  C_{2n}/R^{2n}
!
!----------------------------------------------------------------------

       subroutine arpotv(V,R2)

       implicit none

       integer, parameter      :: sp = kind(1.0),  &
                      dp = selected_real_kind(2*precision(1.0_sp)),  &
                      qp = selected_real_kind(2*precision(1.0_dp))
       integer                 :: n,k,d
       real(dp), intent(out)   :: V
!      real(dp), intent(in)    :: R1,R2,xcos
       real(dp), intent(in)    :: R2
       real(dp)                :: angst, waveno
       real(dp)                :: A,Apr,Aprpr,alpha,beta,b
       real(dp)                :: C6,C8,C10,C12,C14,C16
       real(dp)                :: x,term1,term1apar,term1exp,term2, &
                                  dampfunc,kfact,assymconst
       real(dp), dimension(16) :: Cpar

       waveno = 219474.63d0
       angst = 0.529177249d0
       Cpar = 0.0

       A = -134.2099075
       Apr = 62.9325922
       Aprpr = 206.8758202
       alpha = 1.623806026
       beta = -0.0467301127
       b = 1.500497187
       C6 = 64.691
       C8  = 1644
       C10 = 50240
       C12 = 1898195
       C14 = 86445426
       C16 = 4619452502.0
       Cpar(6) = C6
       Cpar(8) = C8
       Cpar(10) = C10
       Cpar(12) = C12
       Cpar(14) = C14
       Cpar(16) = C16

       x = b*R2

       dampfunc = 0
       term2 = 0

       term1apar = (A + Apr*R2 + Aprpr/R2)
       term1exp = exp(-alpha*R2 + beta*(R2**2))
       term1 = term1apar*term1exp

       do n=3,8
          dampfunc = 0
          do k=0,(2*n)
             kfact = 1.0
             do d = 1,k
                kfact = kfact*d
             end do
             dampfunc = dampfunc + (x**k)/kfact
          end do
          assymconst = Cpar(2*n)/R2**(2*n)
          dampfunc = 1 - exp(-x)*dampfunc
          term2 = term2+dampfunc*assymconst
       end do

       V = term1 - term2

       end subroutine arpotv

