!
! file contains c2ha1 and ch2asec
!
!#######################################################################
!
! C2H 1A' adiabatic surface from Martial
!
! ider=0 no derivatives
!     =1 analytic derivatives
! r(3) input (binding) coordinates in au
! hemat output PE in au
! drv(3) derivatives
!
! needs c2ha1_fit
!#######################################################################

      subroutine c2ha1(ider,r,hemat,drv,opdatadef)
!
!     Modele DMBE pour la surface d'energie potentielle du
!     premier etat doublet A' de !2H.
!     On utilise les unites atomiques.
!     r(1)=Rcc, r(2)=Rch1, r(3)=Rch2
!     Attention, si r(i)=0, la subroutine c2vehf plante!
!
!     Points caracteristiques de la surface:
!       CH :  De = 3.50963931 eV, Re = 2.12056949 Bohrs.
!       C2 :  De = 5.4328 eV, Re = 2.4984 Bohrs.
!
! Important: Essayer de mettre toute la subroutine c2hpot dans le cache!
!
      implicit double precision(a-h,o-z)
      character(len=*), intent(in) :: opdatadef
      dimension r(3),drv(3),dv3ehf(3),dv3dc(3)
      dimension ac2x(5),ac2b(5),ac2a(7),achx(5),cnc2(6:10),cnch(6:10), &
                anc2(6:10),bnc2(6:10),anch(6:10),bnch(6:10),gamt(3), &
                ant(6:10,2),bnt(6:10,2),cnt(6:10,2),rlrt(2),rot(2)
      logical first
      data first/.true./,vzero/0.1289768189d0/
      data avc/9.2d-3/,bvc/4.6103d0/,r0vc/3.082972d0/
      data avc2/4.d-4/,bvc2/5.0d0/,r0vc2/2.470203d0/
!
!           Donnees pour les energies EHF diatomiques
!           ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!             Molecules diatomiques :  C2(X)   C2(B')   C2(a)   CH(X)
!        Coefficients des polynomes :  ac2x    ac2b     ac2a    achx
!     Energies MRCI de dissociation :  dc2x    dc2b     dc2a    dchx
!        distances d'equilibre MRCI :  rmc2x   rmc2b    rmc2a   rmchx
!      Exposants de l'exponentielle :  gamc2x  gamc2b   gamc2a  gamchx
!
      data ac2x/2.77809812d0,1.93807491d0,0.03734836d0,-0.65559660d0, &
                0.d0/,nc2x/4/, &
           ac2b/3.69371035d0,5.40027712d0,4.85626128d0,3.05071249d0, &
                0.99449799d0/,nc2b/5/, &
           ac2a/5.10698068d0,11.56392279d0,16.00668809d0,15.38040556d0, &
                12.08870934d0,8.98101993d0,4.04117753d0/,nc2a/7/, &
           achx/3.25455574d0,4.07632920d0,3.01484681d0,1.49861077d0, &
                0.41536235d0/,nchx/5/, &
           dc2x/0.47907831d0/,rmc2x/2.38572933d0/,gamc2x/2.32933114d0/, &
           dc2b/0.35535923d0/,rmc2b/2.65433956d0/,gamc2b/3.29428518d0/, &
           dc2a/0.50387902d0/,rmc2a/2.52175126d0/,gamc2a/4.70170718d0/, &
           dchx/0.250033256d0/,rmchx/2.15068639d0/,gamchx/2.79330556d0/
!
!           Donnees pour les energies DC diatomiques
!           ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!     rlrc2, rlrch : rayons de Le Roy pour C2 et CH
!     cnc2(i), cnch(i): coefficients Ci pour C2 et CH (i=6,8,10)
!     alpha0, alpha1, beta0, beta1: constantes universelles
!
      data cnc2(6)/40.0d0/,rlrc2/7.8890808d0/, &
           cnch(6)/16.1d0/,rlrch/7.4095846d0/, &
           alpha0/16.36606d0/,alpha1/0.70172d0/, &
           beta0/17.19338d0/,beta1/0.09574d0/
!
!           Donnees pour l'energie DC triatomique
!           ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!     cnt(n,k) : coefficients Cn (n=6,8,10) pour les interactions:
!                   CC -- H    pour k=1
!                   CH -- C    pour k=2
!     rlrt(k) : rayon de Le Roy pour l'arrangement k.
!
      data cnt(6,1)/66.50d0/,cnt(6,2)/31.05d0/, &
           rlrt/10.5018345d0,7.23256522d0/,gamt/2.32d0,0.64d0,0.64d0/
!
!     Initialisation des constantes
!
      save anc2,bnc2,cnc2,anch,bnch,cnch,ant,bnt,cnt
!
      if (first) then
         first = .false.
         cnc2(8) = cnc2(6) * rlrc2**(1.57d0)
         cnc2(10) = cnc2(6) * 1.13d0*rlrc2**(3.14d0)
         cnch(8) = cnch(6) * rlrch**(1.57d0)
         cnch(10) = cnch(6) * 1.13d0*rlrch**(3.14d0)
         roc2 = (11.0d0+2.5d0*rlrc2)/2.0d0
         roch = (11.0d0+2.5d0*rlrch)/2.0d0
         do  k=1,2
            rot(k) = (11.0d0+2.5d0*rlrt(k))/2.0d0
         enddo
         do n=6,10,2
            aa = -alpha0*dble(n)**(-alpha1)
            bb = -beta0*dexp(-beta1*n)
            anc2(n) = aa / roc2
            bnc2(n) = bb / (roc2*roc2)
            anch(n) = aa / roch
            bnch(n) = bb / (roch*roch)
            do k=1,2
               ant(n,k) = aa / rot(k)
               bnt(n,k) = bb / (rot(k)*rot(k))
            enddo
         enddo
        do k=1,2
           cnt(8,k) = cnt(6,k) * rlrt(k)**(1.57d0)
           cnt(10,k) = cnt(6,k) * 1.13d0*rlrt(k)**(3.14d0)
        enddo
      end if
!
!     Termes diatomiques
!
      call cv2ehf(ac2x,nc2x,dc2x,rmc2x,gamc2x,ider,r(1),ehfc2x,dhfc2x)
      call cv2ehf(ac2b,nc2b,dc2b,rmc2b,gamc2b,ider,r(1),ehfc2b,dhfc2b)
      call cv2ehf(ac2a,nc2a,dc2a,rmc2a,gamc2a,ider,r(1),ehfc2a,dhfc2a)
      call cv2ehf(achx,nchx,dchx,rmchx,gamchx,ider,r(2),ehfch1,dhfch1)
      call cv2ehf(achx,nchx,dchx,rmchx,gamchx,ider,r(3),ehfch2,dhfch2)
!
      call cv2dc(anc2,bnc2,cnc2,ider,r(1),dcc2,ddcc2)
      call cv2dc(anch,bnch,cnch,ider,r(2),dcch1,ddcch1)
      call cv2dc(anch,bnch,cnch,ider,r(3),dcch2,ddcch2)
!
!     Couplage des etats X et B' de C2
!
      vc2x = ehfc2x + dcc2
      vc2b = ehfc2b + dcc2
      vc2a = ehfc2a + dcc2
      vcoup = avc*dexp(-bvc*(r(1)-r0vc)**2)
      vcoup2 = avc2*dexp(-bvc2*(r(1)-r0vc2)**2)
      rac = sqrt((vc2x-vc2b)**2 + 4.d0*vcoup**2)
      vc2xad = 0.5d0*(vc2x+vc2b-rac)
      rac2 = sqrt((vc2xad-vc2a)**2 + 4.d0*vcoup2**2)
      vc2 = 0.5d0*(vc2xad+vc2a-rac2)
!
!     Terme triatomique
!
      call cv3ehfa1(ider,r,v3ehf,dv3ehf,opdatadef)
      call cv3dc(gamt,ant,bnt,cnt,rmc2x,rmchx,ider,r,v3dc,dv3dc)
!
!     Energie totale. "vzero" permet de placer le zero d'energie
!     pour la geometrie C loin de CH.
!
      hemat = vzero + vc2 + ehfch1+dcch1 + ehfch2+dcch2 + v3ehf + v3dc
!     hemat = vc2 + ehfch1+dcch1 + ehfch2+dcch2 + v3ehf + v3dc
!
!     Derivees
!
      if (ider.eq.1) then
         dx = dhfc2x + ddcc2
         db = dhfc2b + ddcc2
         if (rac.eq.0.d0) then
            drvx = 0.5d0*(dx+db)
         else
             term = (vc2x-vc2b)*(dx-db) - 8.d0*bvc*(r(1)-r0vc)*vcoup**2
             drvx = 0.5d0*(dx+db-term/rac)
         end if

         da = dhfc2a + ddcc2
         if (rac2.eq.0.d0) then
            drv(1) = 0.5d0*(drvx+da)
         else
             term2 = (vc2xad-vc2a)*(drvx-da) - &
                      8.d0*bvc2*(r(1)-r0vc2)*vcoup2**2
             drv(1) = 0.5d0*(drvx+da-term2/rac2)
         end if

         drv(2) = ddcch1 + dhfch1
         drv(3) = ddcch2 + dhfch2
         do i=1,3
            drv(i) = drv(i) + dv3ehf(i) + dv3dc(i)
         enddo
      end if
!
      return
      end
!-----------------------------------------------------------------------
      subroutine cv3ehfa1(ider,r,v3ehf,dv3ehf,opdatadef)
!-----------------------------------------------------------------------
!     Calcul de la partie EHF du terme triatomique et de ses
!     derivees.
!
      implicit double precision(a-h,o-z)


      character(len=160) opdata
      character(len=*), intent(in) :: opdatadef

      parameter(ncpmax=50,ncpmx3=ncpmax*3)
      dimension r(3),dv3ehf(3),rc2v(3),gam1(3,3),ro(3),coes(ncpmax), &
                cpol(ncpmax),gamehf(3),dcoes(3,ncpmax),dcr(3,ncpmax), &
                dpoly(3),fth(3),q(3)
      logical first
!
!     rc2v : point de reference pour les coordonnees de symetrie C2v
!     gam1 : matrice de rotation pour les coordonnees de symetrie C2v
!     gamehf : coefficients de la fonction de coupure
!
      data dcoes/ncpmx3*0.d0/,rc2v/3.d0,3.8d0,3.8d0/
      data first/.true./,ncp11/22/
      save gam1,v0,gamehf,cpol
!
      if (first) then
        first = .false.
!       write(6,'(''ordre du polynome? '')')
!       read(5,*) ncp11
        gam1(1,1) = 1.d0/sqrt(3.d0)
        gam1(1,2) = 1.d0/sqrt(3.d0)
        gam1(1,3) = 1.d0/sqrt(3.d0)
        gam1(2,1) = 0.d0
        gam1(2,2) = 1.d0/sqrt(2.d0)
        gam1(2,3) = -1.d0/sqrt(2.d0)
        gam1(3,1) = sqrt(2.d0/3.d0)
        gam1(3,2) = -1.d0/sqrt(6.d0)
        gam1(3,3) = -1.d0/sqrt(6.d0)
!
        opdata=opdatadef
        ilbl=index(opdata,' ')-1
        open(40,file=opdata(1:ilbl)//'/c2ha1_fit',status='old')
        read(40,*) v0,gamehf(1),gamehf(2)
        do i=1,22
           read(40,*) cpol(i)
        enddo
        close(40,status='keep')
        gamehf(3) = gamehf(2)
      end if
!
!     Calcul des coordonnees de symetrie C2v
!
      do i=1,3
        ro(i) = r(i) - rc2v(i)
      end do
      do i=1,3
        q(i) = 0.0d0
        do j=1,3
        q(i) = q(i) + gam1(i,j)*ro(j)
        end do
      end do
!
!     Calcul du polynome
!
      s2a2 = q(2)*q(2) + q(3)*q(3)
      s2b2 = q(2)*q(2) - q(3)*q(3)
      s33  = q(3) * (q(3)*q(3) - 3.d0*q(2)*q(2))
!
!     Termes lineaires
!
      coes(1) = 1.0d0
      coes(2) = q(1)
      coes(3) = q(3)
!
!     Termes quadratiques
!
      coes(4) = q(1)*q(1)
      coes(5) = s2a2
      coes(6) = q(1)*q(3)
      coes(7) = s2b2
!
!     Termes cubiques
!
      coes(8) = q(1)*coes(4)
      coes(9) = q(1)*s2a2
      coes(10) = s33
      coes(11) = q(1)*coes(6)
      coes(12) = q(1)*s2b2
      coes(13) = q(3)*s2a2
!
!     Termes quartiques
!
      coes(14) = q(1)*coes(8)
      coes(15) = q(1)*coes(9)
      coes(16) = s2a2*s2a2
      coes(17) = q(1)*s33
      coes(18) = q(1)*coes(11)
      coes(19) = q(1)*coes(12)
      coes(20) = q(1)*coes(13)
      coes(21) = q(3)*s33
      coes(22) = s2a2*s2b2
!
!     Termes d'ordre 5
!
!      coes(23) = q(1)*coes(14)
!      coes(24) = q(1)*coes(15)
!      coes(25) = q(1)*coes(16)
!      coes(26) = q(1)*coes(17)
!      coes(27) = s2a2*s33
!      coes(28) = q(1)*coes(18)
!      coes(29) = q(1)*coes(19)
!      coes(30) = q(1)*coes(20)
!      coes(31) = q(1)*coes(21)
!      coes(32) = q(1)*coes(22)
!      coes(33) = s2a2*coes(13)
!      coes(34) = s2b2*s33
!
!     Termes d'ordre 6
!
!      coes(35) = q(1)*coes(23)
!      coes(36) = q(1)*coes(24)
!      coes(37) = q(1)*coes(25)
!      coes(38) = q(1)*coes(26)
!      coes(39) = q(1)*coes(27)
!      coes(40) = s2a2*coes(16)
!      coes(41) = s33*s33
!      coes(42) = q(1)*coes(28)
!      coes(43) = q(1)*coes(29)
!      coes(44) = q(1)*coes(30)
!      coes(45) = q(1)*coes(31)
!      coes(46) = q(1)*coes(32)
!      coes(47) = q(1)*coes(33)
!      coes(48) = q(1)*coes(34)
!      coes(49) = q(3)*coes(27)
!      coes(50) = s2a2*coes(22)
!
      polyno = 0.0d0
      do i=1,ncp11
        polyno = polyno + coes(i)*cpol(i)
      end do
!
!     Fonction de coupure
!
      do i=1,3
         fth(i) = tanh(0.5d0*gamehf(i)*ro(i))
      enddo
      tcoup = (1-fth(1))*(1-fth(2))*(1-fth(3))
!
!     write(6,100) q,tcoup,polyno
      v3ehf = v0*polyno*tcoup
!
!     Calcul des derivees
!
      if (ider.eq.1) then
!
!       Derivees des termes du polynome par rapport a "q(i)"
!
        dcoes(1,2) = 1.d0
        dcoes(1,4) = 2.d0*q(1)
        dcoes(1,6) = q(3)
        dcoes(1,8) = 3.d0*coes(4)
        dcoes(1,9) = s2a2
        dcoes(1,11) = 2.d0*coes(6)
        dcoes(1,12) = s2b2
        dcoes(1,14) = 4.d0*coes(8)
        dcoes(1,15) = 2.d0*coes(9)
        dcoes(1,17) = s33
        dcoes(1,18) = 3.d0*coes(11)
        dcoes(1,19) = 2.d0*coes(12)
        dcoes(1,20) = coes(13)
        dcoes(1,23) = 5.d0*coes(14)
        dcoes(1,24) = 3.d0*coes(15)
        dcoes(1,25) = coes(16)
        dcoes(1,26) = 2.d0*coes(17)
        dcoes(1,28) = 4.d0*coes(18)
        dcoes(1,29) = 3.d0*coes(19)
        dcoes(1,30) = 2.d0*coes(20)
        dcoes(1,31) = coes(21)
        dcoes(1,32) = coes(22)
!	dcoes(1,35) = 6.d0*coes(23)
!	dcoes(1,36) = 4.d0*coes(24)
!	dcoes(1,37) = 2.d0*coes(25)
!	dcoes(1,38) = 3.d0*coes(26)
!	dcoes(1,39) = coes(27)
!	dcoes(1,42) = 5.d0*coes(28)
!	dcoes(1,43) = 4.d0*coes(29)
!	dcoes(1,44) = 3.d0*coes(30)
!	dcoes(1,45) = 2.d0*coes(31)
!	dcoes(1,46) = 2.d0*coes(32)
!	dcoes(1,47) = coes(33)
!        dcoes(1,48) = coes(34)
!
        dcoes(2,5) = 2.d0*q(2)
        dcoes(2,7) = dcoes(2,5)
        dcoes(2,9) = q(1)*dcoes(2,5)
        dcoes(2,10) = -6.d0*q(2)*q(3)
        dcoes(2,12) = dcoes(2,9)
        dcoes(2,13) = q(3)*dcoes(2,5)
        dcoes(2,15) = q(1)*dcoes(2,9)
        dcoes(2,16) = 4.d0*s2a2*q(2)
        dcoes(2,17) = q(1)*dcoes(2,10)
        dcoes(2,19) = coes(4)*dcoes(2,5)
        dcoes(2,20) = coes(6)*dcoes(2,5)
        dcoes(2,21) = q(3)*dcoes(2,10)
        dcoes(2,22) = 4.d0*q(2)**3
!	dcoes(2,24) = q(1)*dcoes(2,15)
!	dcoes(2,25) = q(1)*dcoes(2,16)
!	dcoes(2,26) = q(1)*dcoes(2,17)
!	dcoes(2,27) = s33*dcoes(2,5)+s2a2*dcoes(2,10)
!	dcoes(2,29) = q(1)*dcoes(2,19)
!	dcoes(2,30) = q(1)*dcoes(2,20)
!	dcoes(2,31) = q(1)*dcoes(2,21)
!	dcoes(2,32) = q(1)*dcoes(2,22)
!	dcoes(2,33) = q(3)*dcoes(2,16)
!       dcoes(2,34) = dcoes(2,7)*s33+s2b2*dcoes(2,10)
!	dcoes(2,36) = q(1)*dcoes(2,24)
!	dcoes(2,37) = q(1)*dcoes(2,25)
!	dcoes(2,38) = q(1)*dcoes(2,26)
!	dcoes(2,39) = q(1)*dcoes(2,27)
!	dcoes(2,40) = 1.5d0*s2a2*dcoes(2,16)
!	dcoes(2,41) = 2.d0*s33*dcoes(2,10)
!	dcoes(2,43) = q(1)*dcoes(2,29)
!	dcoes(2,44) = q(1)*dcoes(2,30)
!	dcoes(2,45) = q(1)*dcoes(2,31)
!	dcoes(2,46) = q(1)*dcoes(2,32)
!	dcoes(2,47) = q(1)*dcoes(2,33)
!       dcoes(2,48) = q(1)*dcoes(2,34)
!	dcoes(2,49) = q(3)*dcoes(2,27)
!       dcoes(2,50) = dcoes(2,16)*s2b2+coes(16)*dcoes(2,7)
!
        dcoes(3,3) = 1.d0
        dcoes(3,5) = 2.d0*q(3)
        dcoes(3,6) = q(1)
        dcoes(3,7) = -dcoes(3,5)
        dcoes(3,9) = q(1)*dcoes(3,5)
        dcoes(3,10) = -3.d0*s2b2
        dcoes(3,11) = coes(4)
        dcoes(3,12) = q(1)*dcoes(3,7)
        dcoes(3,13) = coes(5) + 2.d0*q(3)*q(3)
        dcoes(3,15) = coes(4)*dcoes(3,5)
        dcoes(3,16) = 4.d0*s2a2*q(3)
        dcoes(3,17) = -3.d0*q(1)*s2b2
        dcoes(3,18) = coes(8)
        dcoes(3,19) = -dcoes(3,15)
        dcoes(3,20) = q(1)*dcoes(3,13)
        dcoes(3,21) = s33+q(3)*dcoes(3,10)
        dcoes(3,22) = -4.d0*q(3)**3
!	dcoes(3,24) = q(1)*dcoes(3,15)
!	dcoes(3,25) = q(1)*dcoes(3,16)
!	dcoes(3,26) = q(1)*dcoes(3,17)
!	dcoes(3,27) = dcoes(3,5)*s33+s2a2*dcoes(3,10)
!	dcoes(3,28) = coes(14)
!	dcoes(3,29) = q(1)*dcoes(3,19)
!	dcoes(3,30) = q(1)*dcoes(3,20)
!	dcoes(3,31) = q(1)*dcoes(3,21)
!	dcoes(3,32) = q(1)*dcoes(3,22)
!	dcoes(3,33) = coes(16) + q(3)*dcoes(3,16)
!       dcoes(3,34) = dcoes(3,7)*s33 + s2b2*dcoes(3,10)
!	dcoes(3,36) = q(1)*dcoes(3,24)
!	dcoes(3,37) = q(1)*dcoes(3,25)
!	dcoes(3,38) = q(1)*dcoes(3,26)
!	dcoes(3,39) = q(1)*dcoes(3,27)
!	dcoes(3,40) = 3.d0*coes(16)*dcoes(3,5)
!	dcoes(3,41) = 2.d0*s33*dcoes(3,10)
!	dcoes(3,42) = coes(23)
!	dcoes(3,43) = q(1)*dcoes(3,29)
!	dcoes(3,44) = q(1)*dcoes(3,30)
!	dcoes(3,45) = q(1)*dcoes(3,31)
!	dcoes(3,46) = q(1)*dcoes(3,32)
!	dcoes(3,47) = q(1)*dcoes(3,33)
!       dcoes(3,48) = q(1)*dcoes(3,34)
!	dcoes(3,49) = coes(27) + q(3)*dcoes(3,27)
!	dcoes(3,50) = coes(16)*dcoes(3,7) + dcoes(3,16)*coes(7)
!
!       Derivees des termes du polynome par rapport a r(i)
!       Note: on pourrais eventuellement traiter les termes pour
!             k=1,2,3 a part car il sont tres simples.
!
        do i=1,3
          do k=1,ncp11
            dcr(i,k) = 0.d0
            do j=1,3
               dcr(i,k) = dcr(i,k) + gam1(j,i)*dcoes(j,k)
            enddo
          enddo
        enddo
!
       do i=1,3
          dpoly(i) = 0.0d0
          do k=1,ncp11
             dpoly(i) = dpoly(i) + dcr(i,k)*cpol(k)
          enddo
       enddo
!
!       derivees completes
!
        do i=1,3
           dv3ehf(i) = v0*tcoup * (dpoly(i) - &
                           polyno*0.5d0*gamehf(i)*(1.d0+fth(i)))
        enddo
!
      end if
!
      return
      end

!#######################################################################
!
! C2H A'' adiabatic surface from Martial
!
! ider=0 no derivatives
!     =1 analytic derivatives
! r(3) input (binding) coordinates in au
! hemat output PE in au
! drv(3) derivatives
!
!#######################################################################

      subroutine c2hasec(ider,r,hemat,drv,opdatadef)
!
!     Modele DMBE pour la surface d'energie potentielle du
!     premier etat doublet A'' de C2H.
!     On utilise les unites atomiques.
!     r(1)=Rcc, r(2)=Rch1, r(3)=Rch2
!     Attention, si r(i)=0, la subroutine c2vehf plante!
!
!     Points caracteristiques de la surface:
!       CH :  De = 3.50963931 eV, Re = 2.12056949 Bohrs.
!       C2 :  De = 5.4328 eV, Re = 2.4984 Bohrs.
!
! Important: Essayer de mettre toute la subroutine c2hpot dans le cache!
!
      implicit double precision(a-h,o-z)
      character(len=*), intent(in) :: opdatadef
      dimension r(3),drv(3),dv3ehf(3),dv3dc(3)
!      dimension ac2x(5),ac2b(5)
      dimension ac2a(7),achx(5),cnc2(6:10),cnch(6:10), &
                anc2(6:10),bnc2(6:10),anch(6:10),bnch(6:10),gamt(3), &
                ant(6:10,2),bnt(6:10,2),cnt(6:10,2),rlrt(2),rot(2)
      logical first
      data first/.true./,vzero/0.1289768189d0/

!      data avc/9.2d-3/,bvc/4.6103d0/,r0vc/3.082972d0/
!      data avc2/4.d-4/,bvc2/5.0d0/,r0vc2/2.470203d0/
!
!           Donnees pour les energies EHF diatomiques
!           ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!             Molecules diatomiques :  C2(X)   C2(B')   C2(a)   CH(X)
!        Coefficients des polynomes :  ac2x    ac2b     ac2a    achx
!     Energies MRCI de dissociation :  dc2x    dc2b     dc2a    dchx
!        distances d'equilibre MRCI :  rmc2x   rmc2b    rmc2a   rmchx
!      Exposants de l'exponentielle :  gamc2x  gamc2b   gamc2a  gamchx
!
!      data ac2x/2.77809812d0,1.93807491d0,0.03734836d0,-0.65559660d0,
!     >          0.d0/,nc2x/4/,
!     >     ac2b/3.69371035d0,5.40027712d0,4.85626128d0,3.05071249d0,
!     >          0.99449799d0/,nc2b/5/,
!     >     ac2a/5.10698068d0,11.56392279d0,16.00668809d0,15.38040556d0,
!     >          12.08870934d0,8.98101993d0,4.04117753d0/,nc2a/7/,
!     >     achx/3.25455574d0,4.07632920d0,3.01484681d0,1.49861077d0,
!     >          0.41536235d0/,nchx/5/,
!     >     dc2x/0.47907831d0/,rmc2x/2.38572933d0/,gamc2x/2.32933114d0/,
!     >     dc2b/0.35535923d0/,rmc2b/2.65433956d0/,gamc2b/3.29428518d0/,
!     >     dc2a/0.50387902d0/,rmc2a/2.52175126d0/,gamc2a/4.70170718d0/,
!     >     dchx/0.250033256d0/,rmchx/2.15068639d0/,gamchx/2.79330556d0/


      data ac2a/5.10698068d0,11.56392279d0,16.00668809d0,15.38040556d0, &
                12.08870934d0,8.98101993d0,4.04117753d0/,nc2a/7/, &
           achx/3.25455574d0,4.07632920d0,3.01484681d0,1.49861077d0, &
                0.41536235d0/,nchx/5/, &
           dc2a/0.50387902d0/,rmc2a/2.52175126d0/,gamc2a/4.70170718d0/, &
           dchx/0.250033256d0/,rmchx/2.15068639d0/,gamchx/2.79330556d0/

!
!           Donnees pour les energies DC diatomiques
!           ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!     rlrc2, rlrch : rayons de Le Roy pour C2 et CH
!     cnc2(i), cnch(i): coefficients Ci pour C2 et CH (i=6,8,10)
!     alpha0, alpha1, beta0, beta1: constantes universelles
!
      data cnc2(6)/40.0d0/,rlrc2/7.8890808d0/, &
           cnch(6)/16.1d0/,rlrch/7.4095846d0/, &
           alpha0/16.36606d0/,alpha1/0.70172d0/, &
           beta0/17.19338d0/,beta1/0.09574d0/
!
!           Donnees pour l'energie DC triatomique
!           ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!     cnt(n,k) : coefficients Cn (n=6,8,10) pour les interactions:
!                   CC -- H    pour k=1
!                   CH -- C    pour k=2
!     rlrt(k) : rayon de Le Roy pour l'arrangement k.
!
      data cnt(6,1)/66.50d0/,cnt(6,2)/31.05d0/, &
           rlrt/10.5018345d0,7.23256522d0/,gamt/1.1d0,0.6d0,0.6d0/
!
!     Initialisation des constantes
!
      save anc2,bnc2,cnc2,anch,bnch,cnch,ant,bnt,cnt
!
      if (first) then
        first = .false.
        cnc2(8) = cnc2(6) * rlrc2**(1.57d0)
        cnc2(10) = cnc2(6) * 1.13d0*rlrc2**(3.14d0)
        cnch(8) = cnch(6) * rlrch**(1.57d0)
        cnch(10) = cnch(6) * 1.13d0*rlrch**(3.14d0)
        roc2 = (11.0d0+2.5d0*rlrc2)/2.0d0
        roch = (11.0d0+2.5d0*rlrch)/2.0d0
        do k=1,2
          rot(k) = (11.0d0+2.5d0*rlrt(k))/2.0d0
        enddo
        do n=6,10,2
          aa = -alpha0*dble(n)**(-alpha1)
          bb = -beta0*dexp(-beta1*n)
          anc2(n) = aa / roc2
          bnc2(n) = bb / (roc2*roc2)
          anch(n) = aa / roch
          bnch(n) = bb / (roch*roch)
          do k=1,2
            ant(n,k) = aa / rot(k)
            bnt(n,k) = bb / (rot(k)*rot(k))
          enddo
        enddo
        do  k=1,2
          cnt(8,k) = cnt(6,k) * rlrt(k)**(1.57d0)
          cnt(10,k) = cnt(6,k) * 1.13d0*rlrt(k)**(3.14d0)
        enddo
      end if
!
!     Termes diatomiques
!
!      call cv2ehf(ac2x,nc2x,dc2x,rmc2x,gamc2x,ider,r(1),ehfc2x,dhfc2x)
!      call cv2ehf(ac2b,nc2b,dc2b,rmc2b,gamc2b,ider,r(1),ehfc2b,dhfc2b)
      call cv2ehf(ac2a,nc2a,dc2a,rmc2a,gamc2a,ider,r(1),ehfc2a,dhfc2a)
      call cv2ehf(achx,nchx,dchx,rmchx,gamchx,ider,r(2),ehfch1,dhfch1)
      call cv2ehf(achx,nchx,dchx,rmchx,gamchx,ider,r(3),ehfch2,dhfch2)
!
      call cv2dc(anc2,bnc2,cnc2,ider,r(1),dcc2,ddcc2)
      call cv2dc(anch,bnch,cnch,ider,r(2),dcch1,ddcch1)
      call cv2dc(anch,bnch,cnch,ider,r(3),dcch2,ddcch2)

!
!     Couplage des etats X et B' de C2
!
!      vc2x = ehfc2x + dcc2
!      vc2b = ehfc2b + dcc2
      vc2a = ehfc2a + dcc2
!      vcoup = avc*dexp(-bvc*(r(1)-r0vc)**2)
!      vcoup2 = avc2*dexp(-bvc2*(r(1)-r0vc2)**2)
!      rac = sqrt((vc2x-vc2b)**2 + 4.d0*vcoup**2)
!      vc2xad = 0.5d0*(vc2x+vc2b-rac)
!      rac2 = sqrt((vc2xad-vc2a)**2 + 4.d0*vcoup2**2)
!      vc2 = 0.5d0*(vc2xad+vc2a+rac2)
!

!     Terme triatomique
!
      call cv3ehf(ider,r,v3ehf,dv3ehf,opdatadef)
      call cv3dc(gamt,ant,bnt,cnt,rmc2a,rmchx,ider,r,v3dc,dv3dc)
!
!     Energie totale. "vzero" permet de placer le zero d'energie
!     pour la geometrie C loin de CH.
!
      hemat = vzero + vc2a + ehfch1+dcch1 + ehfch2+dcch2 + v3ehf + v3dc
!
!     Derivees
!
      if (ider.eq.1) then
!	dx = dhfc2x + ddcc2
!	db = dhfc2b + ddcc2
!	if (rac.eq.0.d0) then
!	  drvx = 0.5d0*(dx+db)
!	else
!	  term = (vc2x-vc2b)*(dx-db) - 8.d0*bvc*(r(1)-r0vc)*vcoup**2
!	  drvx = 0.5d0*(dx+db-term/rac)
!	end if

!	da = dhfc2a + ddcc2
!	if (rac2.eq.0.d0) then
!	  drv(1) = 0.5d0*(drvx+da)
!	else
!	  term2 = (vc2xad-vc2a)*(drvx-da) - 8.d0*bvc2*(r(1)-r0vc2)
!     &            *vcoup2**2
!	  drv(1) = 0.5d0*(drvx+da+term2/rac2)
!	end if

        drv(1) = ddcc2 + dhfc2a
        drv(2) = ddcch1 + dhfch1
        drv(3) = ddcch2 + dhfch2
        do i=1,3
           drv(i) = drv(i) + dv3ehf(i) + dv3dc(i)
        enddo
      end if
!
      return
      end
!-----------------------------------------------------------------------
      subroutine cv2ehf(a,n,d,rm,gam,ider,r,v2,dv2)
!-----------------------------------------------------------------------
!     Calcul de la partie EHF des termes diatomiques et des derivees.
!     Note: si r=0 ca plante!
!
      implicit double precision(a-h,o-z)
      dimension a(n)
!
      rs = r-rm
      poly = a(n)*rs
      do i=n-1,1,-1
         poly = (poly + a(i))*rs
      enddo
      fexp = -(d/r)*exp(-gam*rs)
      v2 = (1.d0+poly)*fexp
!
      if (ider.eq.1) then
        dpoly = n*a(n)*rs
        do i=n-1,2,-1
           dpoly = (dpoly + i*a(i))*rs
        enddo
        dv2 = -v2*(gam+1.d0/r) + fexp*(a(1)+dpoly)
      end if
!
      return
      end
!-----------------------------------------------------------------------
      subroutine cv2dc(an,bn,cn,ider,r,v2,dv2)
!-----------------------------------------------------------------------
!     Calcul de la partie "Dynamical Correlation" des termes diatomiques
!     et des derivees !orrespondantes.
!
      implicit double precision(a-h,o-z)
      dimension an(6:10),bn(6:10),cn(6:10),xe(6:10),xi(6:10)
!
      v2 = 0.d0
      do k=6,10,2
        xe(k) =  dexp((an(k)+bn(k)*r)*r)
        xi(k) = ((1.d0-xe(k))/r)**k
        v2 = v2 - cn(k)*xi(k)
      enddo
!
      if (ider.eq.1) then
        dv2 = 0.d0
        do k=6,10,2
          dxi = k*xi(k)/(1.d0-xe(k)) * xe(k) *(an(k)+2.d0*bn(k)*r)
          dv2 = dv2 + cn(k) * ( k*xi(k)/r + dxi )
        enddo
      end if
!
      return
      end
!-----------------------------------------------------------------------
      subroutine cv3ehf(ider,r,v3ehf,dv3ehf,opdatadef)
!-----------------------------------------------------------------------
!     Calcul de la partie EHF du terme triatomique et de ses
!     derivees.
!

      implicit double precision(a-h,o-z)

      character(len=160) opdata
      character(len=*), intent(in) :: opdatadef

      parameter(ncpmax=50,ncpmx3=ncpmax*3)
      dimension r(3),dv3ehf(3),rc2v(3),gam1(3,3),ro(3),coes(ncpmax), &
                cpol(ncpmax),gamehf(3),dcoes(3,ncpmax),dcr(3,ncpmax), &
                dpoly(3),fth(3),q(3)
      logical first
!
!     rc2v : point de reference pour les coordonnees de symetrie C2v
!     gam1 : matrice de rotation pour les coordonnees de symetrie C2v
!     gamehf : coefficients de la fonction de coupure
!
      data dcoes/ncpmx3*0.d0/,rc2v/2.d0,2.5d0,2.5d0/
      data first/.true./,ncp11/22/
      save gam1,v0,gamehf,cpol
!
      if (first) then
        first = .false.
!       write(6,'(''ordre du polynome? '')')
!       read(5,*) ncp11
        gam1(1,1) = 1.d0/sqrt(3.d0)
        gam1(1,2) = 1.d0/sqrt(3.d0)
        gam1(1,3) = 1.d0/sqrt(3.d0)
        gam1(2,1) = 0.d0
        gam1(2,2) = 1.d0/sqrt(2.d0)
        gam1(2,3) = -1.d0/sqrt(2.d0)
        gam1(3,1) = sqrt(2.d0/3.d0)
        gam1(3,2) = -1.d0/sqrt(6.d0)
        gam1(3,3) = -1.d0/sqrt(6.d0)
!
        opdata=opdatadef
        ilbl=index(opdata,' ')-1
        open(40,file=opdata(1:ilbl)//'/c2hasec_fit',status='old')
        read(40,*) v0,gamehf(1),gamehf(2)
        do i=1,22
           read(40,*) cpol(i)
        enddo
        close(40,status='keep')
        gamehf(3) = gamehf(2)
      end if
!
!     Calcul des coordonnees de symetrie C2v
!
      do i=1,3
        ro(i) = r(i) - rc2v(i)
      end do
      do i=1,3
        q(i) = 0.0d0
        do j=1,3
        q(i) = q(i) + gam1(i,j)*ro(j)
        end do
      end do
!
!     Calcul du polynome
!
      s2a2 = q(2)*q(2) + q(3)*q(3)
      s2b2 = q(2)*q(2) - q(3)*q(3)
      s33  = q(3) * (q(3)*q(3) - 3.d0*q(2)*q(2))
!
!     Termes lineaires
!
      coes(1) = 1.0d0
      coes(2) = q(1)
      coes(3) = q(3)
!
!     Termes quadratiques
!
      coes(4) = q(1)*q(1)
      coes(5) = s2a2
      coes(6) = q(1)*q(3)
      coes(7) = s2b2
!
!     Termes cubiques
!
      coes(8) = q(1)*coes(4)
      coes(9) = q(1)*s2a2
      coes(10) = s33
      coes(11) = q(1)*coes(6)
      coes(12) = q(1)*s2b2
      coes(13) = q(3)*s2a2
!
!     Termes quartiques
!
      coes(14) = q(1)*coes(8)
      coes(15) = q(1)*coes(9)
      coes(16) = s2a2*s2a2
      coes(17) = q(1)*s33
      coes(18) = q(1)*coes(11)
      coes(19) = q(1)*coes(12)
      coes(20) = q(1)*coes(13)
      coes(21) = q(3)*s33
      coes(22) = s2a2*s2b2
!
      polyno = 0.0d0
      do i=1,ncp11
        polyno = polyno + coes(i)*cpol(i)
      end do
!
!     Fonction de coupure
!
      do i=1,3
         fth(i) = tanh(0.5d0*gamehf(i)*ro(i))
      enddo
      tcoup = (1-fth(1))*(1-fth(2))*(1-fth(3))
!
!     write(6,100) q,tcoup,polyno
      v3ehf = v0*polyno*tcoup
!
!     Calcul des derivees
!
      if (ider.eq.1) then
!
!       Derivees des termes du polynome par rapport a "q(i)"
!
        dcoes(1,2) = 1.d0
        dcoes(1,4) = 2.d0*q(1)
        dcoes(1,6) = q(3)
        dcoes(1,8) = 3.d0*coes(4)
        dcoes(1,9) = s2a2
        dcoes(1,11) = 2.d0*coes(6)
        dcoes(1,12) = s2b2
        dcoes(1,14) = 4.d0*coes(8)
        dcoes(1,15) = 2.d0*coes(9)
        dcoes(1,17) = s33
        dcoes(1,18) = 3.d0*coes(11)
        dcoes(1,19) = 2.d0*coes(12)
        dcoes(1,20) = coes(13)
!
        dcoes(2,5) = 2.d0*q(2)
        dcoes(2,7) = dcoes(2,5)
        dcoes(2,9) = q(1)*dcoes(2,5)
        dcoes(2,10) = -6.d0*q(2)*q(3)
        dcoes(2,12) = dcoes(2,9)
        dcoes(2,13) = q(3)*dcoes(2,5)
        dcoes(2,15) = q(1)*dcoes(2,9)
        dcoes(2,16) = 4.d0*s2a2*q(2)
        dcoes(2,17) = q(1)*dcoes(2,10)
        dcoes(2,19) = coes(4)*dcoes(2,5)
        dcoes(2,20) = coes(6)*dcoes(2,5)
        dcoes(2,21) = q(3)*dcoes(2,10)
        dcoes(2,22) = 4.d0*q(2)**3
!
        dcoes(3,3) = 1.d0
        dcoes(3,5) = 2.d0*q(3)
        dcoes(3,6) = q(1)
        dcoes(3,7) = -dcoes(3,5)
        dcoes(3,9) = q(1)*dcoes(3,5)
        dcoes(3,10) = -3.d0*s2b2
        dcoes(3,11) = coes(4)
        dcoes(3,12) = q(1)*dcoes(3,7)
        dcoes(3,13) = coes(5) + 2.d0*q(3)*q(3)
        dcoes(3,15) = coes(4)*dcoes(3,5)
        dcoes(3,16) = 4.d0*s2a2*q(3)
        dcoes(3,17) = -3.d0*q(1)*s2b2
        dcoes(3,18) = coes(8)
        dcoes(3,19) = -dcoes(3,15)
        dcoes(3,20) = q(1)*dcoes(3,13)
        dcoes(3,21) = s33+q(3)*dcoes(3,10)
        dcoes(3,22) = -4.d0*q(3)**3
!
!       Derivees des termes du polynome par rapport a r(i)
!       Note: on pourrais eventuellement traiter les termes pour
!             k=1,2,3 a part car il sont tres simples.
!
        do i=1,3
          do k=1,ncp11
            dcr(i,k) = 0.d0
            do j=1,3
               dcr(i,k) = dcr(i,k) + gam1(j,i)*dcoes(j,k)
            enddo
          enddo
        enddo
!
        do i=1,3
           dpoly(i) = 0.0d0
           do k=1,ncp11
              dpoly(i) = dpoly(i) + dcr(i,k)*cpol(k)
           enddo
        enddo
!
!       derivees completes
!
        do i=1,3
           dv3ehf(i) = v0*tcoup * (dpoly(i) - &
                    polyno*0.5d0*gamehf(i)*(1.d0+fth(i)))
        enddo
!
      end if
!
      return
      end
!-----------------------------------------------------------------------
      subroutine cv3dc(gamt,ant,bnt,cnt,rmc2a,rmchx,ider,r,v3dc,dv3dc)
!-----------------------------------------------------------------------
!     Calcul de la partie DC du terme triatomique et de ses
!     derivees.
!
      implicit double precision(a-h,o-z)
      dimension r(3),dv3dc(3),ant(6:10,2),bnt(6:10,2),cnt(6:10,2), &
                vdc(3),dvdc(3),rs(3),fth(3),gamt(3),fct(3)
      data eta/3.d0/
!
      rr = 0.5d0*(r(2)+r(3))
      call cv2dc(ant(6,1),bnt(6,1),cnt(6,1),ider,rr,vdc(1),dvdc(1))
      rr = 0.5d0*(r(1)+r(3))
      call cv2dc(ant(6,2),bnt(6,2),cnt(6,2),ider,rr,vdc(2),dvdc(2))
      rr = 0.5d0*(r(1)+r(2))
      call cv2dc(ant(6,2),bnt(6,2),cnt(6,2),ider,rr,vdc(3),dvdc(3))
!
      rs(1) = r(1) - rmc2a
      rs(2) = r(2) - rmchx
      rs(3) = r(3) - rmchx
      fth(1) = tanh(gamt(1)*(eta*rs(1)-rs(2)-rs(3)))
      fth(2) = tanh(gamt(2)*(eta*rs(2)-rs(3)-rs(1)))
      fth(3) = tanh(gamt(3)*(eta*rs(3)-rs(1)-rs(2)))
      v3dc = 0.d0
      do i=1,3
        fct(i) = 0.5d0*(1.d0-fth(i))
        v3dc = v3dc + fct(i)*vdc(i)
      enddo
!
      if (ider.eq.1) then
        do k=1,3
          dv3dc(k) = 0.d0
          do i=1,3
            dfs = fct(i)*gamt(i)*(1.d0+fth(i))*vdc(i)
            if (i.eq.k) then
              dv3dc(k) = dv3dc(k) - eta*dfs
            else
              dv3dc(k) = dv3dc(k) + dfs + fct(i)*0.5d0*dvdc(i)
            end if
          enddo
        enddo
      end if
!
      return
      end
