!#######################################################################
!
! 1D CURVES  (Real functions without parameters).
!
! ifile = 303
!
!#######################################################################


      subroutine fdef1dlib(ifunc,label,ifile,ipar,rpar,maxpar,hoppar)

      implicit none

! Quantics sizes
      integer, parameter :: dop = selected_real_kind(15,307)
      integer, parameter  :: long = selected_int_kind(9)  ! >= 9-digit integers
                                                          !  (close to integer*4)

      integer(long), intent(in)                    :: maxpar
      integer(long)                                :: ilbl,jlbl,order
      integer(long), intent(out)                   :: ifunc,ifile
      integer(long), dimension(maxpar), intent(out):: ipar
      real(dop), dimension(maxpar), intent(out)    :: rpar
      real(dop), dimension(maxpar,*), intent(in)   :: hoppar
      character(len=*), intent(in)                 :: label

      ipar(:) = 0
      rpar(:) = 0.0
!-----------------------------------------------------------------------
! now assign function number
!-----------------------------------------------------------------------
! R dependent part of LiCN potential
      if (label(1:4) .eq. 'licn') then
         ilbl=index(label,':')+1
         jlbl=index(label,' ')-1
         order=0
         if (ilbl .ne. 1) read(label(ilbl:jlbl),*) order
         ipar(1)=order

         ifunc=1

! labels for C-H and C2 potentials from c2hasec

      else if (label .eq. 'v:CH') then
         ifunc=2

      else if (label .eq. 'v:C2') then
         ifunc=3

! OH potential - From HOO surface
      else if (label .eq. 'v:OH') then
         ifunc=4

! H2 potential from WDSE surface (H3O)
      else if (label .eq. 'vwdse:H2') then
         ifunc=5

! H2 potential from BMKP surface (H4)
      else if (label .eq. 'vbmkp:H2') then
         ifunc=6

! expansion coefficients for Diep-Johnson H2-H2 surface
      else if (label .eq. 'vdj:000') then
         ifunc=7
      else if (label .eq. 'vdj:022') then
         ifunc=8
      else if (label .eq. 'vdj:224') then
         ifunc=9

! NH potential from NH3 adiabatic potential(V11)
      else if (label .eq. 'v:NH') then
         ifunc=10

! NH3 Umbrella motion (Phi) potential from NH3 adiabatic potential(V11)
      else if (label .eq. 'v:2dumb') then
         ifunc=11

! NH3 Umbrella motion (Phi) potential from NH3 adiabatic potential(V11)
      else if (label .eq. 'v:umb') then
         ifunc=12

! H3O Phi potential from H3O YZCL2 surface
      else if (label .eq. 'v:h3ophi') then
         ifunc=13

!-----------------------------------------------------------------------
! next free ifunc is 14
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
! end of if loop
!-----------------------------------------------------------------------
      endif

!-----------------------------------------------------------------------
! if label is assigned, assign ifile no.
!-----------------------------------------------------------------------
      if (ifunc .ne. 0) ifile=304

      return
      end subroutine fdef1dlib

!#######################################################################

      subroutine func1dlib(v,x,ifunc,hopipar,hoprpar,ilog)

      implicit none

! Quantics sizes
      integer, parameter :: dop = selected_real_kind(15,307)
      integer, parameter  :: long = selected_int_kind(9)  ! >= 9-digit integers
                                                          !  (close to integer*4)

      integer(long)                           :: error,k,n
      integer(long), intent(in)               :: ifunc,ilog
      integer(long), dimension(*), intent(in) :: hopipar

      real(dop)       :: alpha,jbf,d,rm,&
                         gam,der,v3,alpha1,beta,beta1,aa,bb,rlrch,roch,&
                         rab,rac,rad,rbc,rbd,rcd,theta,&
                         tfac1,tfac2,ra,rb,rc,gamma,phi,U11,U22,U12,V11,V22

      real(dop), intent(in)               :: x
      real(dop), dimension(*), intent(in) :: hoprpar
      real(dop), intent(out)              :: v
      real(dop), dimension(10)            :: a,b,c
      real(dop), dimension(12)            :: gU11,gU22,gU12,gV11,gV22,gpoint
      real(dop), external                 :: h4dj_v000,h4dj_v022,h4dj_v224 ! external functions

! functions
      real(dop), external                  :: licnsrf,voh
      real*8                               :: vh2bmkp
!      real(dop)                           :: vno,morse,cscentri,licnsrf,voh,vh2bmkp
      character(len=80)                     :: message

      select case (ifunc)

      case (1) ! R dependent part of LiCN potential
      k=hopipar(1)
      v = licnsrf(k,x,error)
      if (error .eq. 1) then
         write(message,'(a,f12.5)')  &
          'ERROR in subroutine licnsrf, illegal value for R:',x
         go to 999
      endif

      case (2) ! CH potential from C2H routine c2h.F
      n=5
      a(1)=3.25455574_dop
      a(2)=4.07632920_dop
      a(3)=3.01484681_dop
      a(4)=1.49861077_dop
      a(5)=0.41536235_dop
      d=0.250033256_dop
      rm=2.15068639_dop
      gam=2.79330556_dop
! EHF contribution
! a corresponds to achx
      call cv2ehf(a,n,d,rm,gam,0,x,v,der)
! dynamical correlation
! a,b,c correspond to anch(6:10), bnch(6:10), cnch(6:10)
      alpha=16.36606_dop
      alpha1=0.70172_dop
      beta=17.19338_dop
      beta1=0.09574_dop
      rlrch=7.4095846_dop
      roch = (11.0_dop+2.5_dop*rlrch)/2.0_dop
      c(1)=16.1_dop
      c(3) = c(1) * rlrch**(1.57_dop)
      c(5) = c(1) * 1.13_dop*rlrch**(3.14_dop)
      do n=1,5,2
         aa = -alpha*dble(n+5)**(-alpha1)
         bb = -beta*dexp(-beta1*(n+5))
         a(n) = aa / roch
         b(n) = bb / (roch*roch)
      enddo
      call cv2dc(a,b,c,0,x,v3,der)
      v=v+v3+0.1289768189_dop
      return

      case (3) ! C2 potential from C2H routine c2hasec.F
      n=7
      a(1)= 5.10698068_dop
      a(2)=11.56392279_dop
      a(3)=16.00668809_dop
      a(4)=15.38040556_dop
      a(5)=12.08870934_dop
      a(6)=8.98101993_dop
      a(7)=4.04117753_dop
      d=0.50387902_dop
      rm=2.52175126_dop
      gam=4.70170718_dop
      call cv2ehf(a,n,d,rm,gam,0,x,v,der)
      return

      case (4) ! OH potential from hoosrf.f
      v=VOH(x)
      return

      case (5) ! H2 potential from WDSE surface
      rab=1.86_dop
      rac=10.0_dop
      rad=11.0_dop
      rbc=13.0_dop
      rbd=14.0_dop
      rcd=x
      call wdse(rab,rac,rad,rbc,rbd,rcd,v)
      return

      case (6) ! H2 potential from BMKP surface
      tfac1=hoprpar(1)
      tfac2=hoprpar(2)
      v = vh2bmkp(x,tfac1,tfac2)
      return

      case (7) ! expansion coefficients of Diep-Johnson H2-H2 surface
      v = h4dj_v000(x)
      return
      case (8)
      v = h4dj_v022(x)
      return
      case (9)
      v = h4dj_v224(x)
      return

      case (10) ! NH potential from NH3 1st adiabatic surface
      ra=30.0_dop
      rb=31.0_dop
      rc=x
      theta=1.914_dop
      gamma=1.8582_dop
      phi = 2.03828_dop
      call pot1_nh3(ra,rb,rc,gamma,phi,theta,U11,U22,U12,V11,V22, &
                   gU11,gU22,gU12,gV11,gV22)
      v = V11
      return

      case (11) ! 2D NH3
      gpoint(1) = 1.919961604_dop
      gpoint(2) = x
      call func_nh3_truhlar(gpoint,v,4,1)

      return

      case (12) ! NH3 Umbrella motion (Phi) potential from NH3 1st diabatic surface
      ra = 1.99099_dop
      rb = 1.95962_dop
      rc = 1.91996_dop
      gamma =1.8833_dop
      phi = x
      theta =1.915022_dop
      call pot1_nh3(ra,rb,rc,gamma,phi,theta,U11,U22,U12,V11,V22, &
                   gU11,gU22,gU12,gV11,gV22)
      v = V11

      return

      case (13) ! H3O (Phi) potential from H3O YZCL2 surface
      ra = 2.1_dop
      rb = 1.84296_dop
      rc = 1.814_dop
      theta =1.93187_dop
      gamma =1.8056_dop
      phi = x
      call h3oyzclinput
      call pot_h3o(ra,rb,rc,gamma,phi,theta,v)
      v = v/27.2113845_dop
      return


      case default
      write(message,'(a,i5)') 'Function not found :',ifunc
      go to 999
      return

      end select

  999 continue
      write(ilog,'(a)') 'Error in Func1dlib (func1dlib.F90)'
      write(ilog,'(a)') message

      end subroutine func1dlib

!#######################################################################
!
! H2 potential (from BMKP surface)
!
! coortransj is in funcsrflib
!#######################################################################

      real*8 function vh2bmkp(x,tfac1,tfac2)

      implicit none

! Quantics sizes
      integer, parameter :: dop = selected_real_kind(15,307)
      integer, parameter  :: long = selected_int_kind(9)  ! >= 9-digit integers
                                                          !  (close to integer*4)

      real(dop), intent(in)           :: x,tfac1,tfac2
      real(dop)                       :: R,r1,r2,theta1,theta2,phi,Derr
      real(dop), dimension(3)         :: a,b,c,d
      real(dop), dimension(6)         :: rdum,dVdr
      real(dop), dimension(4,3)       :: cc,dVdcc
      integer(long)                   :: ideriv,i

      R  = 100.0_dop
      r1 = x
      r2 = 1.4483_dop
      theta1 = 1.570796_dop
      theta2 = 1.570796_dop
      phi = 0.0_dop

      call coortransj(R,r1,r2,theta1,theta2,phi,tfac1,tfac2,a,b,c,d)
      do i=1,3
         cc(1,i)=a(i)
         cc(2,i)=b(i)
         cc(3,i)=c(i)
         cc(4,i)=d(i)
      enddo
      ideriv = 0
      call h4bmkp(cc,vh2bmkp,rdum,dVdcc,dVdr,Derr,ideriv)

      return
      end function vh2bmkp

