
!+++++++++++++++++++++++++++++++++++++++++++++++++
! H3O2- Dipole Fit, internal version 4.0 (Unpublished !!!)
!
! Author:  Xinchuan Huang
!
! Built on 12/30/2005, Chemistry Dept.,Emory Univ.
! Notes on 11/18/2006, NASA Ames Research Center, CA 94035
! Updated on 01/04/2012, NASA Ames Research Center, CA 94035
!
! complete 5th order dipole fit,
! 1540 coeffs from 10134 ab initio dipole vectors (MP2/aug-cc-pVTZ)
! RMS errors for x-, y-, and z- components of dipole vectors are:
!       0.013 a.u.      0.008 a.u.      0.007 a.u.
! Average fitting errors for x-, y-, and z- components of dipole vectors are:
!       0.007 a.u.      0.004 a.u.      0.004 a.u.
! (errors for dip-x is larger than dip-y and dip-z
!  because the O...O was aligned to x-axis in ab initio data)
!
! Original 10134 geometries and dipoles (ab initio & fitted & fitted - ab initio)
! are given in the included file all.points.
!
! This dipole fit can describe the H2O........OH- dissociation
! This dipole fit is invariant to permutations of H atoms or O atoms
!
! 3 data/base files are required:
!       h3o2.dms4.coeff.dat
!       H3OxO.5thorder.basis
!       HxH2O2.5thorder.basis
! However, they can be combined into 1 single file if one modify the predip() subroutine accordingly
!
! Users must "call predip_h3o2()" first before any "call calcdip_h3o2(dip,cart_in)"
! (see test.f90 for example)
!
! Input arrays cart_in(5,3) contain the Cartesian geometries of H3O2-
! Atom order:  H H H O O      Unit: bohr
!
! Returned dipole vectors dip(3) are in atomic unit, NOT in Debye
!
! Note: No need to convert input coordinates to center-of-mass frame
!       or any other frame.  The returned dipole are ALWAYS related to the Origin
!       of given Cartesian coordinate system.  In this way, users can always
!       define their own frame origins for various isotopologues.
!
! Tested with PGI compiler and IFORT compiler:
!   ifort -r8 test.f90 h3o2.dms4.f90 -o test.x
!
! If you have any questions, please contact
!       xinchuan@gmail.com or xinchuan.huang-1@nasa.gov
!_____________________________________________________________________________
      subroutine calcdip_h3o2(dip,cart_in)

      implicit Double Precision (A-H,O-Z)
      implicit Integer (I-N)

      dimension coeff(1540),func0(1540),afunc(1540,3),sumx(3)

      dimension dip(3), cart_in(5,3), com(3)
      dimension cart(5,3), coor1(10), coor(10)

      common/h3o2dip/coeff
      common/dcdwdip/dc0,dc1,dc2,dw0,dw1,dw2

      cart=cart_in

      do j=1,3
        com(j)=((cart(4,j)+cart(5,j))*15.9949146d0+sum(cart(1:3,j))*1.007825d0)/(2*15.9949146d0+3*1.007825d0)
      end do

      do j=1,5
        cart(j,:)=cart_in(j,:)-com(:)
      end do

      ic=0
      do j=1,4
        do k=j+1,5
          ic=ic+1
          coor1(ic)=dsqrt(sum((cart(j,:)-cart(k,:))**2))
        end do
      end do

      coor(1:2)=(dexp(-coor1(1:2)/3)-dc0)/dw0
      coor(5)=(dexp(-coor1(5)/3)-dc0)/dw0
      coor(3:4)=(dexp(-coor1(3:4)/3)-dc1)/dw1
      coor(6:9)=(dexp(-coor1(6:9)/3)-dc1)/dw1
      coor(10)=(dexp(-coor1(10)/3)-dc2)/dw2

      call funcsdip(cart,coor,AFUNC)

      do j=1,3
        func0(:)=afunc(:,j)
        sumx(j)=dot_product(coeff, func0)
      end do

      dip(:)=sumx(:)-com(:)

!      write(*,'(3F10.4)')'dip=',(dip(i),i=1,3)

      return
      end subroutine calcdip_h3o2

      Subroutine funcsdip(cart,Coor, AFUNC)
      implicit double precision (a-h,o-z)
      implicit integer (i-n)
      Double Precision Coor(10), AFUNC(1540,3)
      dimension coor0(10),AFUNC1(636),AFUNC0(904)

      dimension cart(5,3) ! ,coor1(10134,10),ene(10134)
      dimension iap1(10),iap2(10),iap12(5,5)

      common/IAP/iap1,iap2,iap12
!      common/dcdwdip/dc0,dc1,dw0,dw1

      dimension IAX(5)

      AFUNC=0.d0
      AFUNC0=0.d0
      AFUNC1=0.d0

      ma=1540

      mb=ma-636

! 1 2 3 4 5
      coor0(1:10)=coor(1:10)
      call geth3oxo1ptbasis(coor0,AFUNC1,636)
      AFUNC(mb+1:,1)=AFUNC(mb+1:,1)+AFUNC1(:)*cart(4,1)
      AFUNC(mb+1:,2)=AFUNC(mb+1:,2)+AFUNC1(:)*cart(4,2)
      AFUNC(mb+1:,3)=AFUNC(mb+1:,3)+AFUNC1(:)*cart(4,3)

! 1 2 3 5 4
      iax = (/ 1, 2, 3, 5, 4 /)
      do i=1,10
        coor0(i)=coor(iap12(iax(iap1(i)),iax(iap2(i)) ))
      end do
      call geth3oxo1ptbasis(coor0,AFUNC1,636)
      AFUNC(mb+1:,1)=AFUNC(mb+1:,1)+AFUNC1(:)*cart(5,1)
      AFUNC(mb+1:,2)=AFUNC(mb+1:,2)+AFUNC1(:)*cart(5,2)
      AFUNC(mb+1:,3)=AFUNC(mb+1:,3)+AFUNC1(:)*cart(5,3)

! 1 2 3 4 5
      coor0(1:10)=coor(1:10)
      call gethxh2o21ptbasis(coor0,AFUNC0,mb)
      AFUNC(:mb,1)=AFUNC(:mb,1)+AFUNC0(:mb)*cart(1,1)
      AFUNC(:mb,2)=AFUNC(:mb,2)+AFUNC0(:mb)*cart(1,2)
      AFUNC(:mb,3)=AFUNC(:mb,3)+AFUNC0(:mb)*cart(1,3)

! 2 1 3 4 5
      iax = (/ 2, 1, 3, 4, 5 /)
      do i=1,10
        coor0(i)=coor(iap12(iax(iap1(i)),iax(iap2(i)) ))
      end do
      call gethxh2o21ptbasis(coor0,AFUNC0,mb)
      AFUNC(:mb,1)=AFUNC(:mb,1)+AFUNC0(:mb)*cart(2,1)
      AFUNC(:mb,2)=AFUNC(:mb,2)+AFUNC0(:mb)*cart(2,2)
      AFUNC(:mb,3)=AFUNC(:mb,3)+AFUNC0(:mb)*cart(2,3)

! 3 1 2 4 5
      iax = (/ 3, 1, 2, 4, 5 /)
      do i=1,10
        coor0(i)=coor(iap12(iax(iap1(i)),iax(iap2(i)) ))
      end do
      call gethxh2o21ptbasis(coor0,AFUNC0,mb)
      AFUNC(:mb,1)=AFUNC(:mb,1)+AFUNC0(:mb)*cart(3,1)
      AFUNC(:mb,2)=AFUNC(:mb,2)+AFUNC0(:mb)*cart(3,2)
      AFUNC(:mb,3)=AFUNC(:mb,3)+AFUNC0(:mb)*cart(3,3)

      return
      End

!***************************************************************
        subroutine gethxh2o21ptbasis(eint,basis,Nbasis)
        implicit double precision (a-h,o-z)
        implicit integer (i-n)
        dimension eint(10),basis(Nbasis),w(Nbasis)
        dimension q(10,0:5),q2(45,0:5,0:5)

        dimension iqq1(45),iqq2(45),iqq(10,10),iqqq(10,10,10),i3m(5,5,5)

        dimension ibasis(3003,3),jbasis(3003,5,2)
        dimension kbasis(3003,3),lbasis(3003,5,2)

        dimension q3(120,20)
        dimension iap1(10),iap2(10),iap12(5,5)

        common/IBASIS/ibasis,jbasis,iqq1,iqq2,iqq,iqqq,i3m
        common/IAP/iap1,iap2,iap12
        common/KBASIS/kbasis,lbasis

        q(:,0)=1.d0
        do i=1,10
          q(i,1)= eint(i)
          q(i,2)= her2_h3o2_d(eint(i))
          q(i,3)= her3_h3o2_d(eint(i))
          q(i,4)= her4_h3o2_d(eint(i))
          q(i,5)= her5_h3o2_d(eint(i))
!          q(i,6)= her6_h3o2_d(eint(i))
!          q(i,7)= her7_h3o2_d(eint(i))
        end do

        q2=0.d0
        do i=1,9
        do j=i+1,10
          do k1=1,4
          do k2=1,5-k1
            q2(iqq(i,j),k1,k2)=q(i,k1)*q(j,k2)
          end do
          end do
        end do
        end do

        q3=0.d0
        do i=1,8
        do j=i+1,9
        do k=j+1,10
          do k1=1,3
          do k2=1,4-k1
          do k3=1,5-k1-k2
            q3(iqqq(i,j,k),i3m(k1,k2,k3))=q2(iqq(i,j),k1,k2)*q(k,k3)
          end do
          end do
          end do
        end do
        end do
        end do

        w=0.d0
        w(1)=1.d0

        do i=2,3003

         if(ibasis(i,3).eq.1)then
          e1=q(jbasis(i,1,1),jbasis(i,1,2))
          goto 1001
         end if

         if(ibasis(i,3).eq.2)then
          e1=q2(iqq(jbasis(i,1,1),jbasis(i,2,1)),jbasis(i,1,2),jbasis(i,2,2))
          goto 1001
         end if

       if(ibasis(i,3).eq.3)then
         e1=q3(iqqq(jbasis(i,1,1),jbasis(i,2,1),jbasis(i,3,1)),i3m(jbasis(i,1,2),jbasis(i,2,2),jbasis(i,3,2)))
         goto 1001
       end if

       if(ibasis(i,3).eq.4)then
        e1=q2(iqq(jbasis(i,1,1),jbasis(i,2,1)),jbasis(i,1,2),jbasis(i,2,2))*&
           q2(iqq(jbasis(i,3,1),jbasis(i,4,1)),jbasis(i,3,2),jbasis(i,4,2))
        goto 1001
       end if

       if(ibasis(i,3).eq.5)then
        e1=q3(iqqq(jbasis(i,1,1),jbasis(i,2,1),jbasis(i,3,1)),i3m(jbasis(i,1,2),&
        jbasis(i,2,2),jbasis(i,3,2)))*q2(iqq(jbasis(i,4,1),jbasis(i,5,1)),jbasis(i,4,2),jbasis(i,5,2))
        goto 1001
       end if

!       if(ibasis(i,3).eq.6)then
!        e1=q3(iqqq(jbasis(i,1,1),jbasis(i,2,1),jbasis(i,3,1)),i3m(jbasis(i,1,2),jbasis(i,2,2),jbasis(i,3,2)))*q3(iqqq(jbasis(i,4,1),jbasis(i,5,1),jbasis(i,6,1)),i3m(jbasis(i,4,2),jbasis(i,5,2),jbasis(i,6,2)))
!        goto 1001
!       end if

1001     continue

         w(ibasis(i,1))=w(ibasis(i,1))+e1/ibasis(i,2)
!         w(ibasis(i,1))=w(ibasis(i,1))+e1

        end do

        basis=w

        return
        end
!***************************************************************

!******************************************************

        function her2_h3o2_d(t0)
        implicit none

        double precision   t0,her2_h3o2_d
        her2_h3o2_d = (4*t0**2-2)/dsqrt(dble(8*2 ))
        return
        end

        function her3_h3o2_d(t0)
        implicit none

        double precision   t0,her3_h3o2_d
        her3_h3o2_d = (8*t0**2-12)*t0/dsqrt(dble(16*6 ))
        return
        end

        function her4_h3o2_d(t0)
        implicit none

        double precision   t0,her4_h3o2_d
        her4_h3o2_d = ((16*t0**2-48)*t0**2+12)/dsqrt(dble(32*24 ))
        return
        end

        function her5_h3o2_d(t0)
        implicit none

        double precision   t0,her5_h3o2_d
        her5_h3o2_d = ((32*t0**2-160)*t0**2+120)*t0/dsqrt(dble(64*120 ))
        return
        end

        function her6_h3o2_d(t0)
        implicit none

        double precision   t0,her6_h3o2_d
        her6_h3o2_d=(((64*t0**2-480)*t0**2+720)*t0**2-120)/dsqrt(dble(128*720))
        return
        end

        function her7_h3o2_d(t0)
        implicit none

        double precision   t0,her7_h3o2_d
        her7_h3o2_d = (((128*t0**2-1344)*t0**2+3360)*t0**2-1680)*t0/dsqrt(dble(256*5040 ))
        return
        end

        function her8(t0)
        implicit none

        double precision   t0,her8
        her8 = (((256*t0**2-3584)*t0**2+13440)*t0**2-13440)*t0**2+1680/dsqrt(dble(512*40320 ))
        return
        end


        function her9(t0)
        implicit none

        double precision   t0,her9
        her9 = ((((512*t0**2-9216)*t0**2+48384)*t0**2-80640)*t0**2+30240)*t0/dsqrt(dble(1024*362880 ))
        return
        end
!**************************************************************************
        subroutine predip(opdata)
        implicit double precision (a-h,o-z)
        implicit integer (i-n)
        character(len=*), intent(in) :: opdata

        dimension ibasis(3003,3),jbasis(3003,5,2)
        dimension kbasis(3003,3),lbasis(3003,5,2)

        dimension i1b(12)

        dimension iqq1(45),iqq2(45),iqq(10,10)
        dimension iqqq1(120),iqqq2(120),iqqq3(120),iqqq(10,10,10)
        dimension i3m(5,5,5),iap1(10),iap2(10),iap12(5,5)

        dimension coeff(1540)

        common/IBASIS/ibasis,jbasis,iqq1,iqq2,iqq,iqqq,i3m
        common/IAP/iap1,iap2,iap12
        common/KBASIS/kbasis,lbasis

        common/h3o2dip/coeff
        common/dcdwdip/dc0,dc1,dc2,dw0,dw1,dw2

        open(50,file=trim(opdata)//'/h3o2.dms4.coeff.dat',status='old')
          read(50,*)
          read(50,*)dc0,dc1,dc2
          read(50,*)
          read(50,*)dw0,dw1,dw2
          read(50,*)
          do i=1,1540
            read(50,*)coeff(i)
          end do
        close(50)

        ibasis=0
        jbasis=0

        open(50,file=trim(opdata)//'/h3o2.dms.HxH2O2.basis',status='old')
        do i=1,3003
          read(50,*)(i1b(j),j=1,12)

          ibasis(i,1)=i1b(11)
          ibasis(i,2)=i1b(12)

          ic=0
          do j=1,10
            if(i1b(j).gt.0)then
              ic=ic+1
              jbasis(i,ic,1)=j
              jbasis(i,ic,2)=i1b(j)
            end if
          end do
          ibasis(i,3)=ic
        end do
        close(50)

        open(50,file=trim(opdata)//'/h3o2.dms.H3OxO.basis',status='old')
        do i=1,3003
          read(50,*)(i1b(j),j=1,12)
          kbasis(i,1)=i1b(11)
          kbasis(i,2)=i1b(12)
          ic=0
          do j=1,10
            if(i1b(j).gt.0)then
              ic=ic+1
              lbasis(i,ic,1)=j
              lbasis(i,ic,2)=i1b(j)
            end if
          end do
          kbasis(i,3)=ic
        end do
        close(50)

        iqq1=0; iqq2=0; iqq=0
        ic=0
        do i=1,9
        do j=i+1,10
          ic=ic+1
          iqq1(ic)=i
          iqq2(ic)=j
          iqq(i,j)=ic
          iqq(j,i)=ic
        end do
        end do

        iqqq1=0; iqqq2=0; iqqq3=0; iqqq=0
        ic=0
        do i=1,8
        do j=i+1,9
        do k=j+1,10
          ic=ic+1
          iqqq(i,j,k)=ic
        end do
        end do
        end do

        i3m=0
        ic=0
        do i=1,3
        do j=1,4-i
        do k=1,5-i-j
          ic=ic+1
          i3m(i,j,k)=ic
        end do
        end do
        end do

        ic=0; iap1=0; iap2=0; iap12=0
        do i=1,4
        do j=i+1,5
          ic=ic+1
          iap1(ic)=i
          iap2(ic)=j
          iap12(i,j)=ic
          iap12(j,i)=ic
        end do
        end do

        return
        end

!*****************************************************************

!***************************************************************
        subroutine geth3oxo1ptbasis(eint,basis,Nbasis)
        implicit double precision (a-h,o-z)
        implicit integer (i-n)
        dimension eint(10),basis(Nbasis),w(Nbasis)
        dimension q(10,0:5),q2(45,0:5,0:5)

        dimension iqq1(45),iqq2(45),iqq(10,10),iqqq(10,10,10),i3m(5,5,5)

        dimension ibasis(3003,3),jbasis(3003,5,2)
        dimension kbasis(3003,3),lbasis(3003,5,2)

        dimension q3(120,20)
        dimension iap1(10),iap2(10),iap12(5,5)

        common/IBASIS/ibasis,jbasis,iqq1,iqq2,iqq,iqqq,i3m
        common/IAP/iap1,iap2,iap12
        common/KBASIS/kbasis,lbasis

        q(:,0)=1.d0
        do i=1,10
          q(i,1)= eint(i)
          q(i,2)= her2_h3o2_d(eint(i))
          q(i,3)= her3_h3o2_d(eint(i))
          q(i,4)= her4_h3o2_d(eint(i))
          q(i,5)= her5_h3o2_d(eint(i))
!          q(i,6)= her6_h3o2_d(eint(i))
!          q(i,7)= her7_h3o2_d(eint(i))
        end do

        q2=0.d0
        do i=1,9
        do j=i+1,10
          do k1=1,4
          do k2=1,5-k1
            q2(iqq(i,j),k1,k2)=q(i,k1)*q(j,k2)
          end do
          end do
        end do
        end do

        q3=0.d0
        do i=1,8
        do j=i+1,9
        do k=j+1,10
          do k1=1,3
          do k2=1,4-k1
          do k3=1,5-k1-k2
            q3(iqqq(i,j,k),i3m(k1,k2,k3))=q2(iqq(i,j),k1,k2)*q(k,k3)
          end do
          end do
          end do
        end do
        end do
        end do

        w=0.d0
        w(1)=1.d0

        do i=2,3003

         if(kbasis(i,3).eq.1)then
          e1=q(lbasis(i,1,1),lbasis(i,1,2))
          goto 1001
         end if

         if(kbasis(i,3).eq.2)then
          e1=q2(iqq(lbasis(i,1,1),lbasis(i,2,1)),lbasis(i,1,2),lbasis(i,2,2))
          goto 1001
         end if

       if(kbasis(i,3).eq.3)then
         e1=q3(iqqq(lbasis(i,1,1),lbasis(i,2,1),lbasis(i,3,1)),i3m(lbasis(i,1,2),lbasis(i,2,2),lbasis(i,3,2)))
         goto 1001
       end if

       if(kbasis(i,3).eq.4)then
        e1=q2(iqq(lbasis(i,1,1),lbasis(i,2,1)),lbasis(i,1,2),lbasis(i,2,2))*&
           q2(iqq(lbasis(i,3,1),lbasis(i,4,1)),lbasis(i,3,2),lbasis(i,4,2))
        goto 1001
       end if

       if(kbasis(i,3).eq.5)then
        e1=q3(iqqq(lbasis(i,1,1),lbasis(i,2,1),lbasis(i,3,1)),i3m(lbasis(i,1,2),lbasis(i,2,2),&
           lbasis(i,3,2)))*q2(iqq(lbasis(i,4,1),lbasis(i,5,1)),lbasis(i,4,2),lbasis(i,5,2))
        goto 1001
       end if

!       if(kbasis(i,3).eq.6)then
!        e1=q3(iqqq(lbasis(i,1,1),lbasis(i,2,1),lbasis(i,3,1)),i3m(lbasis(i,1,2),lbasis(i,2,2),lbasis(i,3,2)))*q3(iqqq(lbasis(i,4,1),lbasis(i,5,1),lbasis(i,6,1)),i3m(lbasis(i,4,2),lbasis(i,5,2),lbasis(i,6,2)))
!        goto 1001
!       end if

1001     continue

         w(kbasis(i,1))=w(kbasis(i,1))+e1/kbasis(i,2)
!         w(kbasis(i,1))=w(kbasis(i,1))+e1

        end do

        basis=w

        return
        end
!***************************************************************

