c---------------------------------------------------------------------
c     Potential surface of HCN
c     J.M. Bowman, B. Gazdy, J.A. Bentley, T.J. Lee, C.E. Dateo
c     J. Chem. Phys. 99 (1993) 308
c     hcn works with cos(theta)
c     hcn1 with theta in radians
c---------------------------------------------------------------------	
      subroutine hcn1(vv,gr,pr,tt)
      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      character*160 string
      v0=-93.2862240766843627
      io=45
      string="./"
      call inhcn(io,string)
      call hcnpot(gr,tt,pr,pot)
      vv=pot-v0
      end
c---------------------------------------------------------------------	
      subroutine hcn(vv,gr,pr,u)
      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      character*160 string
      tt=acos(u)
      pi=4.0d0*datan(1.d0)
      v0=-93.2862240766843627
      io=45
      string="./"
      call inhcn(io,string)
      call hcnpot(gr,tt,pr,pot)
      vv=pot-v0
      end

      subroutine hcnpot(rbig, gamma, rsmal, pot)
c-----------------------------------------------------------------------
c     this subroutine computes the value of the hcn potential for
c     values of the three jacobi coordinates.
c
c          x1 = r (in a.u.) -- the h-(cn) distance
c          x2 = gamma (in radians)
c          x3 = r (in a.u.) -- the cn distance
c          pot = electronic potential in hartrees
c         the calling program need to call inhcn(io) before calling
c         hcnpot
c-----------------------------------------------------------------------
      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      parameter (nr=12, nthet=19)
      common /a/ re(nr,nthet),ae(nr,nthet)
      common /b/ ve(nr,nthet),de(nr,nthet)
      common /c/ x1a(nr),x2a(nthet)
      common /d/ ry2a(nr,nthet),ay2a(nr,nthet)
      common /e/ vy2a(nr,nthet),dy2a(nr,nthet)
      common/range/alph1,alph2
      data rein,dein,aein,vein/2.2144d0,0.287d0,1.3436d0,93.07796d0/
      data beta/3.12d-03/
      if(rbig.gt.x1a(10)) then
      x1=x1a(10)
      x2=gamma
      x3=rsmal
      call hcn_splin2(x1a,x2a,re,ry2a,nr,nthet,x1,x2,rr)
      call hcn_splin2(x1a,x2a,ae,ay2a,nr,nthet,x1,x2,aa)
      call hcn_splin2(x1a,x2a,ve,vy2a,nr,nthet,x1,x2,vv)
      call hcn_splin2(x1a,x2a,de,dy2a,nr,nthet,x1,x2,dd)
c,  rr,vv,aa, and dd are morse parameters at edge of r-grid.  extapolate
c   them next to actual value of rbig.
      damp1=exp(-alph1*(rbig-x1a(10)))
      damp2=exp(-alph2*(rbig-x1a(10)))
      rr=(rr-rein)*damp1+rein
      aa=(aa-aein)*damp1+aein
      dd=(dd-dein)*damp1+dein
      vv=(vv-vein)*damp2+vein
      else
      if(rbig.lt.1.50d0) then
c.. extapolate ve only and keep the other morse parameters as is for
c..r = 1.5 and gamma whatever it is.
      x1=x1a(1)
      x2=gamma
      x3=rsmal
      call hcn_splin2(x1a,x2a,re,ry2a,nr,nthet,x1,x2,rr)
      call hcn_splin2(x1a,x2a,ae,ay2a,nr,nthet,x1,x2,aa)
      call hcn_splin2(x1a,x2a,ve,vy2a,nr,nthet,x1,x2,vv)
      call hcn_splin2(x1a,x2a,de,dy2a,nr,nthet,x1,x2,dd)
      vv=vv*exp(beta*(rbig-x1))
      else
      x1=rbig
      x2=gamma
      x3=rsmal
      call hcn_splin2(x1a,x2a,re,ry2a,nr,nthet,x1,x2,rr)
      call hcn_splin2(x1a,x2a,ae,ay2a,nr,nthet,x1,x2,aa)
      call hcn_splin2(x1a,x2a,ve,vy2a,nr,nthet,x1,x2,vv)
      call hcn_splin2(x1a,x2a,de,dy2a,nr,nthet,x1,x2,dd)
      end if
      end if
      call hcn_morse1(x3,dd,aa,rr,vv,v)
      pot=v
      return
      end

      subroutine inhcn(io,opdata)
c----------------------------------------------------------------------
c     this subroutine computes the second-derivative table for the
c     morse parameters. this subroutine is called once by the
c     main program.
c----------------------------------------------------------------------
      implicit real*8(a-h,o-z)
      implicit integer (i-n)
      parameter (nr=12,nthet=19)

      character*160 opdata

      common /a/ re(nr,nthet),ae(nr,nthet)
      common /b/ ve(nr,nthet),de(nr,nthet)
      common /c/ x1a(nr),x2a(nthet)
      common /d/ ry2a(nr,nthet),ay2a(nr,nthet)
      common /e/ vy2a(nr,nthet),dy2a(nr,nthet)
      common/range/alph1,alph2

      ilbl=index(opdata,' ')-1
      open(unit=io,file=opdata(1:ilbl)//"/hcn_data",status='old',
     +      iostat=ierr)
      if (ierr .ne. 0) then
          write(6,'(a,/a)')
     +    'Cannot open the surface data file for hcn:',
     +     opdata(1:ilbl)//"/hcn_data"
         stop
      endif

      rewind(io)

      read(io,*)alph1,alph2
      do 10 i = 1,nr
      read(io,*)x1a(i)
   10 continue
      do 20 i = 1,nthet
      read(io,*)x2a(i)
   20 continue
      do 40 j = 1,nthet
      do 30 i = 1,nr
      read(io,*)de(i,j)
      read(io,*)ae(i,j)
      read(io,*)re(i,j)
      read(io,*)ve(i,j)
   30 continue
   40 continue
      rmin1 = 10000.0
      rmax1 = -100.0
      rmin2 = 10000.0
      rmax2 = -100.0
      rmin3 = 10000.0
      rmax3 = -100.0
      rmin4 = 10000.0
      rmax4 = -100.0
      do 140 j = 1, nthet
      do 130 i = 1, nr
      if (de(i,j).lt.rmin1) rmin1 = de(i,j)
      if (de(i,j).gt.rmax1) rmax1 = de(i,j)
      if (ae(i,j).lt.rmin2) rmin2 = ae(i,j)
      if (ae(i,j).gt.rmax2) rmax2 = ae(i,j)
      if (re(i,j).lt.rmin3) rmin3 = re(i,j)
      if (re(i,j).gt.rmax3) rmax3 = re(i,j)
      if (ve(i,j).lt.rmin4) rmin4 = ve(i,j)
      if (ve(i,j).gt.rmax4) rmax4 = ve(i,j)
  130 continue
  140 continue

c      write (6,*) rmin1, rmax1
c      write (6,*) rmin2, rmax2
c      write (6,*) rmin3, rmax3
c      write (6,*) rmin4, rmax4

      call hcn_splie2(x1a,x2a,re,nr,nthet,ry2a)
      call hcn_splie2(x1a,x2a,ae,nr,nthet,ay2a)
      call hcn_splie2(x1a,x2a,ve,nr,nthet,vy2a)
      call hcn_splie2(x1a,x2a,de,nr,nthet,dy2a)
      close(io)
      return
      end
      subroutine hcn_morse1(r,d,alph,req,ve,v)
c-----------------------------------------------------------------------
c     compute the value of the morse potential
c-----------------------------------------------------------------------
      implicit real*8(a-h,o-z)
      implicit integer (i-n)
      vterm=1.0d0-exp(-alph*(r-req))
      v=d*vterm*vterm-ve
      return
      end
      subroutine hcn_splie2(x1a,x2a,ya,m,n,y2a)
      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      parameter (nn=100)
      dimension x1a(m),x2a(n),ya(m,n),y2a(m,n),ytmp(nn),y2tmp(nn)
      do 13 j=1,m
        do 11 k=1,n
          ytmp(k)=ya(j,k)
11      continue
c        call hcn_spline1(x2a,ytmp,n,1.e30,1.e30,y2tmp)
        call hcn_spline1(x2a,ytmp,n,0.0d0,0.0d0,y2tmp)
        do 12 k=1,n
          y2a(j,k)=y2tmp(k)
12      continue
13    continue
      return
      end
      subroutine hcn_splin2(x1a,x2a,ya,y2a,m,n,x1,x2,y)
      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      parameter (nn=100)
      dimension x1a(m),x2a(n),ya(m,n),y2a(m,n),ytmp(nn),y2tmp(nn)
     +   ,yytmp(nn)
      do 12 j=1,m
        do 11 k=1,n
          ytmp(k)=ya(j,k)
          y2tmp(k)=y2a(j,k)
11      continue
      call hcn_splint(x2a,ytmp,y2tmp,n,x2,yytmp(j))
12    continue
      call hcn_spline1(x1a,yytmp,m,1.d30,1.d30,y2tmp)
      call hcn_splint(x1a,yytmp,y2tmp,m,x1,y)
      return
      end
      subroutine hcn_spline1(x,y,n,yp1,ypn,y2)
      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      parameter (nmax=100)
      dimension x(n),y(n),y2(n),u(nmax)
      if (yp1.gt..99e30) then
        y2(1)=0.
        u(1)=0.
      else
        y2(1)=-0.5
        u(1)=(3./(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      endif
      do 11 i=2,n-1
        sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
        p=sig*y2(i-1)+2.
        y2(i)=(sig-1.)/p
        u(i)=(6.*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1))
     *      /(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
11    continue
      if (ypn.gt..99e30) then
        qn=0.
        un=0.
      else
        qn=0.5
        un=(3./(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      endif
      y2(n)=(un-qn*u(n-1))/(qn*y2(n-1)+1.)
      do 12 k=n-1,1,-1
        y2(k)=y2(k)*y2(k+1)+u(k)
12    continue
      return
      end
      subroutine hcn_splint(xa,ya,y2a,n,x,y)
      implicit real*8 (a-h,o-z)
      dimension xa(n),ya(n),y2a(n)
      klo=1
      khi=n
1     if (khi-klo.gt.1) then
        k=(khi+klo)/2
        if(xa(k).gt.x)then
          khi=k
        else
          klo=k
        endif
      goto 1
      endif
      h=xa(khi)-xa(klo)
      if (h.eq.0.) then
         print*, 'bad xa input.'
         stop
      endif
      a=(xa(khi)-x)/h
      b=(x-xa(klo))/h
      y=a*ya(klo)+b*ya(khi)+
     *   ((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6.
      return
      end
