c***********************************************************
c
c   calculation of a 3*3 determinant
c
c***********************************************************

      function det3t3(M)
      implicit none
      integer n
      parameter (n=3)
      double precision det3t3, M(n,n)

       det3t3 = M(1,1)*M(2,2)*M(3,3)
     &         +M(1,2)*M(2,3)*M(3,1)
     &         +M(1,3)*M(2,1)*M(3,2)
     &         -M(1,3)*M(2,2)*M(3,1)
     &         -M(1,1)*M(2,3)*M(3,2)
     &         -M(1,2)*M(2,1)*M(3,3)

      end

c***********************************************************
c
c   makes the inverse of a 3*3 matrix
c
c***********************************************************

      subroutine mat_invers(M,Im,det)
      implicit none
      integer i,j,ma
      parameter (ma=3)
      double precision M(ma,ma), A(ma,ma), Im(ma,ma)
      double precision det

c.....makes the inverse of a 3*3 matrix
c     ---------------------------------
c      det=det3t3(M)
c      if (dabs(det).lt.1.d-10) then
c        write(6,*) 'determinant zero! matrix cannot be inverted!'
c        stop
c      endif

c.....calculation of determinats of the submatrices

      A(1,1)= M(2,2)*M(3,3)-M(2,3)*M(3,2)
      A(1,2)=-M(2,1)*M(3,3)+M(2,3)*M(3,1)
      A(1,3)= M(2,1)*M(3,2)-M(2,2)*M(3,1)

      A(2,1)=-M(1,2)*M(3,3)+M(1,3)*M(3,2)
      A(2,2)= M(1,1)*M(3,3)-M(3,1)*M(1,3)
      A(2,3)=-M(1,1)*M(3,2)+M(1,2)*M(3,1)

      A(3,1)= M(1,2)*M(2,3)-M(1,3)*M(2,2)
      A(3,2)=-M(1,1)*M(2,3)+M(1,3)*M(2,1)
      A(3,3)= M(1,1)*M(2,2)-M(1,2)*M(2,1)

c.....calculation of the inverse (Im) of the matrix

      do i=1,ma
       do j=1,ma
         Im(j,i)=(1.d0/det)*A(i,j)
       enddo
      enddo

      end


c********************************************************************
c
c calculates projected angles in plane -> trisector is normal vector
c of this plane (for nonplanar systems)
c
c rotate trisector on z axis (2 rotations) -> projected angles are 
c angles of projections in xy plane (use xy components of position
c vectors of rotated system)
c
c********************************************************************
      subroutine proangnew(M,t,tnorm,beta,phi,pi)
      implicit none
      integer i,j, n, d
      parameter (n=3, d=2)
      double precision M(n,*), Q(n,n), S(n,n)
      double precision t(n), t_z(n), phi(n)
      double precision v2(3), v3(3), v4(3), z(3)
      double precision tnorm
      double precision epsi, alpha, delta, beta, beta2
      double precision sia, l, u, ws, pi, betr, rad
      real*8 dum

      do i=1,3
      do j=1,3
           Q(j,i)=0.d0
           S(j,i)=0.d0
      enddo
         phi(i)=0.d0
         v2(i)=0.d0
         v3(i)=0.d0
         v4(i)=0.d0
      enddo

c..   unit vector along z axis
      do i=1,3
          z(i)=0.d0
      enddo
      z(3)=1.d0

c..   calculate angle of trisector with z axis
      alpha=t(3)/tnorm
      sia=dsqrt(1.d0-alpha**2)
      alpha=dacos(alpha)
      if (alpha.lt.0.d0) sia=-sia


c      alpha=angle(z,t,n)
c      sia=dsin(alpha)

      if (dabs(sia).lt.1.d-10) then
        do i=1,3
          do j=1,3
            S(i,j)=M(i,j)
          enddo
        enddo

      else

c..   use polar coordinates; calculate angle (delta) in xy plane

        l=tnorm*sia
c        delta=dacos(t(1)/l)
        l=t(1)/l
        l=dmax1(-1.d0,l)
        l=dmin1(1.d0,l)
        delta=dacos(l)

        if (delta.gt.(pi*0.5d0)) then
          delta=pi-delta
        endif

        delta=delta*sign(1.d0,t(1))*sign(1.d0,t(2))
        epsi=-delta
c
c..     rotate trisector in xz plane (rot. around z axis)

        call rotate(M,n,Q,n,epsi)

        if (alpha.gt.(pi*0.5d0)) then
          alpha=pi-alpha
        endif

        call trise(Q,beta2,t_z)


        alpha=alpha*sign(1.d0,t_z(1))*sign(1.d0,t_z(3)) 

c..     rotate trisector on z axis (rot. around y axis)

        call rotate(Q,n,S,d,alpha)

      endif

c..   calculate projected angles phi(1-3)

      do i=1,3
        if (i.eq.3) then
          v2(i)=0.d0
          v3(i)=0.d0
          v4(i)=0.d0
        else
          v2(i)=S(i,1)
          v3(i)=S(i,2)
          v4(i)=S(i,3)
        endif
      enddo



C      phi(1)=angle(v3,v4,n)
C      phi(2)=angle(v2,v4,n)
C      phi(3)=angle(v2,v3,n)
       dum=1.d0/dsqrt(v2(1)**2+v2(2)**2)
       v2(1)=v2(1)*dum
       v2(2)=v2(2)*dum
       dum=1.d0/dsqrt(v3(1)**2+v3(2)**2)
       v3(1)=v3(1)*dum
       v3(2)=v3(2)*dum
       dum=1.d0/dsqrt(v4(1)**2+v4(2)**2)
       v4(1)=v4(1)*dum
       v4(2)=v4(2)*dum
       phi(1)=v3(1)*v4(1)+v3(2)*v4(2)
       phi(2)=v2(1)*v4(1)+v2(2)*v4(2)
       phi(3)=v2(1)*v3(1)+v2(2)*v3(2)
      do i=1,3
      if (dabs(phi(i)-1.d0).lt.1.d-10) then
        phi(i)=0.d0
      elseif (dabs(phi(i)+1.d0).lt.1.d-10) then
        phi(i)=pi
      else
        phi(i)=dacos(phi(i))
      endif
      enddo


c..   scalp gives wrong angle -> in this angle lies third vector!!
c..   to prevent this:

      ws=(phi(1)+phi(2)+phi(3))-(2*pi)

      if (dabs(ws).gt.1.d-10) then
        if (dabs(phi(1)+phi(2)-phi(3)).lt.1.d-10) then
           phi(3)=(2.d0*pi)-phi(3)
        else if (dabs(phi(2)+phi(3)-phi(1)).lt.1.d-10) then
           phi(1)=(2.d0*pi)-phi(1)
        else if (dabs(phi(1)+phi(3)-phi(2)).lt.1.d-10) then
           phi(2)=(2.d0*pi)-phi(2)
        endif
      endif

c      write(6,*)
c      write(6,*) 'matrix S:'
c      do i=1,3
c         write(6,*) (S(i,j), j=1,3)
c      enddo
c      write(6,*)

      end



c----------------------------------------------------------------
c***********************************************************
c
c     rotation of a matrix 
c     n defines rotational axis (x: n=1; y: n=2; z: n=3)
c     dimensions: 3*u matrix rotated
c
c***********************************************************

      subroutine rotate(m1,u,m2,n,theta)
      implicit none
      integer i,j,n,max,p,u
      parameter (max=3)
      double precision m1(max,*), m2(max,*), rotmat(max,max)
      double precision theta, s, c

c      call init_m(rotmat,max,max)
       do i=1,3
       do j=1,3
       rotmat(j,i)=0.d0
       enddo
       enddo

      if ((n.lt.1).or.(n.gt.3)) then
        write(6,*) 'rotational axis not specified! n is not 1,2 or 3'
        stop
      endif

      s = dsin(theta)
      c = dcos(theta)

      do i=1,3
        if (i.ne.n) then
          do j=1,i-1
            if (j.ne.n) then
              rotmat(i,j)=s
              rotmat(j,i)=-s
            endif
          enddo
        endif
        rotmat(i,i)=c
      enddo

      rotmat(n,n)=1.d0
            
      call matmult(rotmat,max,max,m1,u,m2)

c      do i=1,max
c          write(6,100) (rotmat(i,j),j=1,3)
c      enddo
c      write(6,*)
c
c      write(6,*) 'matrix rotated:'
c      do i=1,max
c          write(6,100) (m2(i,j),j=1,3)
c      enddo
c      write(6,*)
100   format(3e15.6)

      end

c----------------------------------------------------------------
c***********************************************************
c
c matrix multiplication of two matrices
c sizes: M1 -> n1*n2; M2 -> n2*n3; EM -> n1*n3
c
c***********************************************************
      subroutine matmult(M1,n1,n2,M2,n3,EM)
      implicit none
      integer i,j,k,n1,n2,n3
      double precision M1(n1,*), M2(n2,*), EM(n1,*)


      do i=1,n1
         do j=1,n3
           EM(i,j)=0.d0
            do k=1,n2
              EM(i,j)=EM(i,j)+(M1(i,k)*M2(k,j))
            enddo
         enddo
      enddo

      end


c****************************************************************
c
c calculation of trisector and pyramidalization angle beta
c for non planar systems
c
c****************************************************************

      subroutine trise(M,beta,t)
      implicit none
      Integer i,k,j,n
      parameter (n=3)
      double precision M(n,n), Im(n,n)
      double precision t(3)
      double precision pi, beta, betrag
      double precision norm,det,det3t3

      pi=4.d0*datan(1.d0)
      do i=1,3
         t(i)=0.d0
      enddo

c.....calculate trisector angle beta

      det=det3t3(M)
      call mat_invers(M,Im,det)

c.....calculate trisector t
c..   transpose of Im because x,y,z values on columns!
c..   (M_transpose)^-1 = (M^-1)_transpose 

      do i=1,3
        t(1)=t(1)+Im(i,1)
        t(2)=t(2)+Im(i,2)
        t(3)=t(3)+Im(i,3)
      enddo
c      betrag=norm(t,3)
       betrag=dsqrt(t(1)**2+t(2)**2+t(3)**2)

c..   calculate angle beta

      beta=dacos(1.d0/betrag)

      end

c----------------------------------------------------------------
c********************************************************
c
c treat case when molecule is planar (determinant of 
c NH bonds is zero)
c
c********************************************************

      subroutine planarnew(M,phi,beta,pi)
      implicit none
      integer i,j,k,n
      parameter (n=3)
      double precision M(n,*), phi(n)
      double precision v2(n), v3(n), v4(n)
      double precision scalp, angle, pi, ws, beta
      double precision s1, s2, s3,dum

c.....columns of matrix M as vectors

      do i=1,3
        v2(i)=M(i,1)
        v3(i)=M(i,2)
        v4(i)=M(i,3)
      enddo

c      s1=scalp(v2,v3,n)
c      s2=scalp(v2,v4,n)
c      s3=scalp(v3,v4,n)
       s1=v2(1)*v3(1)+v2(2)*v3(2)+v2(3)*v3(3)
       s2=v2(1)*v4(1)+v2(2)*v4(2)+v2(3)*v4(3)
       s3=v3(1)*v4(1)+v3(2)*v4(2)+v3(3)*v4(3)

c.....check if atoms collinear (on one line)

      if ((dabs(s1-1.d0).lt.1.d-07).and.(dabs(s2-1.d0)
     &      .lt.1.d-07).and.(dabs(s3-1.d0).lt.1.d-07)) then
        beta=0.d0

        write(6,*) 
        write(6,*) 'all atoms collinear in same direction!!'
        write(6,*) 'coordinates not defined!!'
        write(6,*)

        stop
      else

c.....planar case and atoms not collinear => pyramidalization angle
c..   is 90°, projected angles = bonding angles

        beta=pi*0.5d0

c        phi(1)=angle(v3,v4,3)
c        phi(2)=angle(v2,v4,3)
c        phi(3)=angle(v2,v3,3)
       dum=1.d0/dsqrt(v2(1)**2+v2(2)**2+v2(3)**2)
       v2(1)=v2(1)*dum
       v2(2)=v2(2)*dum
       v2(3)=v2(3)*dum
       dum=1.d0/dsqrt(v3(1)**2+v3(2)**2+v3(3)**2)
       v3(1)=v3(1)*dum
       v3(2)=v3(2)*dum
       v3(3)=v3(3)*dum
       dum=1.d0/dsqrt(v4(1)**2+v4(2)**2+v4(3)**2)
       v4(1)=v4(1)*dum
       v4(2)=v4(2)*dum
       v4(3)=v4(3)*dum
       phi(1)=v3(1)*v4(1)+v3(2)*v4(2)+v3(3)*v4(3)
       phi(2)=v2(1)*v4(1)+v2(2)*v4(2)+v2(3)*v4(3)
       phi(3)=v2(1)*v3(1)+v2(2)*v3(2)+v2(3)*v3(3)
      do i=1,3
      if (dabs(phi(i)-1.d0).lt.1.d-10) then
        phi(i)=0.d0
      else
        phi(i)=dacos(phi(i))
      endif
      enddo
      endif

c..   scalp gives wrong angle -> in this angle lies third vector!!
c..   sum of angles is not 360°
c..   to prevent this:

      ws=(phi(1)+phi(2)+phi(3))-(2.d0*pi)

      if (dabs(ws).gt.1.d-6) then
        if (dabs(phi(1)+phi(2)-phi(3)).lt.1.d-10) then
           phi(3)=(2.d0*pi)-phi(3)
        else if (dabs(phi(2)+phi(3)-phi(1)).lt.1.d-10) then
           phi(1)=(2.d0*pi)-phi(1)
        else if (dabs(phi(1)+phi(3)-phi(2)).lt.1.d-10) then
           phi(2)=(2.d0*pi)-phi(2)
        endif
      endif

      end

c------------------------------------------------------------------------------
      subroutine tql1(n,d,e,ierr)
c
      integer i,j,l,m,n,ii,l1,l2,mml,ierr
      double precision d(n),e(n)
      double precision c,c2,c3,dl1,el1,f,g,h,p,r,s,s2,tst1,tst2,pythag
c
c     this subroutine is a translation of the algol procedure tql1,
c     num. math. 11, 293-306(1968) by bowdler, martin, reinsch, and
c     wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 227-240(1971).
c
c     this subroutine finds the eigenvalues of a symmetric
c     tridiagonal matrix by the ql method.
c
c     on input
c
c        n is the order of the matrix.
c
c        d contains the diagonal elements of the input matrix.
c
c        e contains the subdiagonal elements of the input matrix
c          in its last n-1 positions.  e(1) is arbitrary.
c
c      on output
c
c        d contains the eigenvalues in ascending order.  if an
c          error exit is made, the eigenvalues are correct and
c          ordered for indices 1,2,...ierr-1, but may not be
c          the smallest eigenvalues.
c
c        e has been destroyed.
c
c        ierr is set to
c          zero       for normal return,
c          j          if the j-th eigenvalue has not been
c                     determined after 30 iterations.
c
c     calls pythag for  dsqrt(a*a + b*b) .
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      ierr = 0
      if (n .eq. 1) go to 1001
c
      do 100 i = 2, n
  100 e(i-1) = e(i)
c
      f = 0.0d0
      tst1 = 0.0d0
      e(n) = 0.0d0
c
      do 290 l = 1, n
         j = 0
         h = dabs(d(l)) + dabs(e(l))
         if (tst1 .lt. h) tst1 = h
c     .......... look for small sub-diagonal element ..........
         do 110 m = l, n
            tst2 = tst1 + dabs(e(m))
            if (tst2 .eq. tst1) go to 120
c     .......... e(n) is always zero, so there is no exit
c                through the bottom of the loop ..........
  110    continue
c
  120    if (m .eq. l) go to 210
  130    if (j .eq. 30) go to 1000
         j = j + 1
c     .......... form shift ..........
         l1 = l + 1
         l2 = l1 + 1
         g = d(l)
         p = (d(l1) - g) / (2.0d0 * e(l))
         r = pythag(p,1.0d0)
         d(l) = e(l) / (p + dsign(r,p))
         d(l1) = e(l) * (p + dsign(r,p))
         dl1 = d(l1)
         h = g - d(l)
         if (l2 .gt. n) go to 145
c
         do 140 i = l2, n
  140    d(i) = d(i) - h
c
  145    f = f + h
c     .......... ql transformation ..........
         p = d(m)
         c = 1.0d0
         c2 = c
         el1 = e(l1)
         s = 0.0d0
         mml = m - l
c     .......... for i=m-1 step -1 until l do -- ..........
         do 200 ii = 1, mml
            c3 = c2
            c2 = c
            s2 = s
            i = m - ii
            g = c * e(i)
            h = c * p
            r = pythag(p,e(i))
            e(i+1) = s * r
            s = e(i) / r
            c = p / r
            p = c * d(i) - s * g
            d(i+1) = h + s * (c * g + s * d(i))
  200    continue
c
         p = -s * s2 * c3 * el1 * e(l) / dl1
         e(l) = s * p
         d(l) = c * p
         tst2 = tst1 + dabs(e(l))
         if (tst2 .gt. tst1) go to 130
  210    p = d(l) + f
c     .......... order eigenvalues ..........
         if (l .eq. 1) go to 250
c     .......... for i=l step -1 until 2 do -- ..........
         do 230 ii = 2, l
            i = l + 2 - ii
            if (p .ge. d(i-1)) go to 270
            d(i) = d(i-1)
  230    continue
c
  250    i = 1
  270    d(i) = p
  290 continue
c
      go to 1001
c     .......... set error -- no convergence to an
c                eigenvalue after 30 iterations ..........
 1000 ierr = l
 1001 return
      end

c------------------------------------------------------------------------------
      subroutine tql2(nm,n,d,e,z,ierr)
c
      integer i,j,k,l,m,n,ii,l1,l2,nm,mml,ierr
      double precision d(n),e(n),z(nm,n)
      double precision c,c2,c3,dl1,el1,f,g,h,p,r,s,s2,tst1,tst2,pythag
c
c     this subroutine is a translation of the algol procedure tql2,
c     num. math. 11, 293-306(1968) by bowdler, martin, reinsch, and
c     wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 227-240(1971).
c
c     this subroutine finds the eigenvalues and eigenvectors
c     of a symmetric tridiagonal matrix by the ql method.
c     the eigenvectors of a full symmetric matrix can also
c     be found if  tred2  has been used to reduce this
c     full matrix to tridiagonal form.
c
c     on input
c
c        nm must be set to the row dimension of two-dimensional
c          array parameters as declared in the calling program
c          dimension statement.
c
c        n is the order of the matrix.
c
c        d contains the diagonal elements of the input matrix.
c
c        e contains the subdiagonal elements of the input matrix
c          in its last n-1 positions.  e(1) is arbitrary.
c
c        z contains the transformation matrix produced in the
c          reduction by  tred2, if performed.  if the eigenvectors
c          of the tridiagonal matrix are desired, z must contain
c          the identity matrix.
c
c      on output
c
c        d contains the eigenvalues in ascending order.  if an
c          error exit is made, the eigenvalues are correct but
c          unordered for indices 1,2,...,ierr-1.
c
c        e has been destroyed.
c
c        z contains orthonormal eigenvectors of the symmetric
c          tridiagonal (or full) matrix.  if an error exit is made,
c          z contains the eigenvectors associated with the stored
c          eigenvalues.
c
c        ierr is set to
c          zero       for normal return,
c          j          if the j-th eigenvalue has not been
c                     determined after 30 iterations.
c
c     calls pythag for  dsqrt(a*a + b*b) .
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      ierr = 0
      if (n .eq. 1) go to 1001
c
      do 100 i = 2, n
  100 e(i-1) = e(i)
c
      f = 0.0d0
      tst1 = 0.0d0
      e(n) = 0.0d0
c
      do 240 l = 1, n
         j = 0
         h = dabs(d(l)) + dabs(e(l))
         if (tst1 .lt. h) tst1 = h
c     .......... look for small sub-diagonal element ..........
         do 110 m = l, n
            tst2 = tst1 + dabs(e(m))
            if (tst2 .eq. tst1) go to 120
c     .......... e(n) is always zero, so there is no exit
c                through the bottom of the loop ..........
  110    continue
c
  120    if (m .eq. l) go to 220
  130    if (j .eq. 30) go to 1000
         j = j + 1
c     .......... form shift ..........
         l1 = l + 1
         l2 = l1 + 1
         g = d(l)
         p = (d(l1) - g) / (2.0d0 * e(l))
         r = pythag(p,1.0d0)
         d(l) = e(l) / (p + dsign(r,p))
         d(l1) = e(l) * (p + dsign(r,p))
         dl1 = d(l1)
         h = g - d(l)
         if (l2 .gt. n) go to 145
c
         do 140 i = l2, n
  140    d(i) = d(i) - h
c
  145    f = f + h
c     .......... ql transformation ..........
         p = d(m)
         c = 1.0d0
         c2 = c
         el1 = e(l1)
         s = 0.0d0
         mml = m - l
c     .......... for i=m-1 step -1 until l do -- ..........
         do 200 ii = 1, mml
            c3 = c2
            c2 = c
            s2 = s
            i = m - ii
            g = c * e(i)
            h = c * p
            r = pythag(p,e(i))
            e(i+1) = s * r
            s = e(i) / r
            c = p / r
            p = c * d(i) - s * g
            d(i+1) = h + s * (c * g + s * d(i))
c     .......... form vector ..........
            do 180 k = 1, n
               h = z(k,i+1)
               z(k,i+1) = s * z(k,i) + c * h
               z(k,i) = c * z(k,i) - s * h
  180       continue
c
  200    continue
c
         p = -s * s2 * c3 * el1 * e(l) / dl1
         e(l) = s * p
         d(l) = c * p
         tst2 = tst1 + dabs(e(l))
         if (tst2 .gt. tst1) go to 130
  220    d(l) = d(l) + f
  240 continue
c     .......... order eigenvalues and eigenvectors ..........
      do 300 ii = 2, n
         i = ii - 1
         k = i
         p = d(i)
c
         do 260 j = ii, n
            if (d(j) .ge. p) go to 260
            k = j
            p = d(j)
  260    continue
c
         if (k .eq. i) go to 300
         d(k) = d(i)
         d(i) = p
c
         do 280 j = 1, n
            p = z(j,i)
            z(j,i) = z(j,k)
            z(j,k) = p
  280    continue
c
  300 continue
c
      go to 1001
c     .......... set error -- no convergence to an
c                eigenvalue after 30 iterations ..........
 1000 ierr = l
 1001 return
      end

c------------------------------------------------------------------------------
      subroutine tred1(nm,n,a,d,e,e2)
c
      integer i,j,k,l,n,ii,nm,jp1
      double precision a(nm,n),d(n),e(n),e2(n)
      double precision f,g,h,scale
c
c     this subroutine is a translation of the algol procedure tred1,
c     num. math. 11, 181-195(1968) by martin, reinsch, and wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
c
c     this subroutine reduces a real symmetric matrix
c     to a symmetric tridiagonal matrix using
c     orthogonal similarity transformations.
c
c     on input
c
c        nm must be set to the row dimension of two-dimensional
c          array parameters as declared in the calling program
c          dimension statement.
c
c        n is the order of the matrix.
c
c        a contains the real symmetric input matrix.  only the
c          lower triangle of the matrix need be supplied.
c
c     on output
c
c        a contains information about the orthogonal trans-
c          formations used in the reduction in its strict lower
c          triangle.  the full upper triangle of a is unaltered.
c
c        d contains the diagonal elements of the tridiagonal matrix.
c
c        e contains the subdiagonal elements of the tridiagonal
c          matrix in its last n-1 positions.  e(1) is set to zero.
c
c        e2 contains the squares of the corresponding elements of e.
c          e2 may coincide with e if the squares are not needed.
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      do 100 i = 1, n
         d(i) = a(n,i)
         a(n,i) = a(i,i)
  100 continue
c     .......... for i=n step -1 until 1 do -- ..........
      do 300 ii = 1, n
         i = n + 1 - ii
         l = i - 1
         h = 0.0d0
         scale = 0.0d0
         if (l .lt. 1) go to 130
c     .......... scale row (algol tol then not needed) ..........
         do 120 k = 1, l
  120    scale = scale + dabs(d(k))
c
         if (scale .ne. 0.0d0) go to 140
c
         do 125 j = 1, l
            d(j) = a(l,j)
            a(l,j) = a(i,j)
            a(i,j) = 0.0d0
  125    continue
c
  130    e(i) = 0.0d0
         e2(i) = 0.0d0
         go to 300
c
  140    do 150 k = 1, l
            d(k) = d(k) / scale
            h = h + d(k) * d(k)
  150    continue
c
         e2(i) = scale * scale * h
         f = d(l)
         g = -dsign(dsqrt(h),f)
         e(i) = scale * g
         h = h - f * g
         d(l) = f - g
         if (l .eq. 1) go to 285
c     .......... form a*u ..........
         do 170 j = 1, l
  170    e(j) = 0.0d0
c
         do 240 j = 1, l
            f = d(j)
            g = e(j) + a(j,j) * f
            jp1 = j + 1
            if (l .lt. jp1) go to 220
c
            do 200 k = jp1, l
               g = g + a(k,j) * d(k)
               e(k) = e(k) + a(k,j) * f
  200       continue
c
  220       e(j) = g
  240    continue
c     .......... form p ..........
         f = 0.0d0
c
         do 245 j = 1, l
            e(j) = e(j) / h
            f = f + e(j) * d(j)
  245    continue
c
         h = f / (h + h)
c     .......... form q ..........
         do 250 j = 1, l
  250    e(j) = e(j) - h * d(j)
c     .......... form reduced a ..........
         do 280 j = 1, l
            f = d(j)
            g = e(j)
c
            do 260 k = j, l
  260       a(k,j) = a(k,j) - f * e(k) - g * d(k)
c
  280    continue
c
  285    do 290 j = 1, l
            f = d(j)
            d(j) = a(l,j)
            a(l,j) = a(i,j)
            a(i,j) = f * scale
  290    continue
c
  300 continue
c
      return
      end

c------------------------------------------------------------------------------
      subroutine tred2(nm,n,a,d,e,z)
c
      integer i,j,k,l,n,ii,nm,jp1
      double precision a(nm,n),d(n),e(n),z(nm,n)
      double precision f,g,h,hh,scale
c
c     this subroutine is a translation of the algol procedure tred2,
c     num. math. 11, 181-195(1968) by martin, reinsch, and wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
c
c     this subroutine reduces a real symmetric matrix to a
c     symmetric tridiagonal matrix using and accumulating
c     orthogonal similarity transformations.
c
c     on input
c
c        nm must be set to the row dimension of two-dimensional
c          array parameters as declared in the calling program
c          dimension statement.
c
c        n is the order of the matrix.
c
c        a contains the real symmetric input matrix.  only the
c          lower triangle of the matrix need be supplied.
c
c     on output
c
c        d contains the diagonal elements of the tridiagonal matrix.
c
c        e contains the subdiagonal elements of the tridiagonal
c          matrix in its last n-1 positions.  e(1) is set to zero.
c
c        z contains the orthogonal transformation matrix
c          produced in the reduction.
c
c        a and z may coincide.  if distinct, a is unaltered.
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      do 100 i = 1, n
c
         do 80 j = i, n
   80    z(j,i) = a(j,i)
c
         d(i) = a(n,i)
  100 continue
c
      if (n .eq. 1) go to 510
c     .......... for i=n step -1 until 2 do -- ..........
      do 300 ii = 2, n
         i = n + 2 - ii
         l = i - 1
         h = 0.0d0
         scale = 0.0d0
         if (l .lt. 2) go to 130
c     .......... scale row (algol tol then not needed) ..........
         do 120 k = 1, l
  120    scale = scale + dabs(d(k))
c
         if (scale .ne. 0.0d0) go to 140
  130    e(i) = d(l)
c
         do 135 j = 1, l
            d(j) = z(l,j)
            z(i,j) = 0.0d0
            z(j,i) = 0.0d0
  135    continue
c
         go to 290
c
  140    do 150 k = 1, l
            d(k) = d(k) / scale
            h = h + d(k) * d(k)
  150    continue
c
         f = d(l)
         g = -dsign(dsqrt(h),f)
         e(i) = scale * g
         h = h - f * g
         d(l) = f - g
c     .......... form a*u ..........
         do 170 j = 1, l
  170    e(j) = 0.0d0
c
         do 240 j = 1, l
            f = d(j)
            z(j,i) = f
            g = e(j) + z(j,j) * f
            jp1 = j + 1
            if (l .lt. jp1) go to 220
c
            do 200 k = jp1, l
               g = g + z(k,j) * d(k)
               e(k) = e(k) + z(k,j) * f
  200       continue
c
  220       e(j) = g
  240    continue
c     .......... form p ..........
         f = 0.0d0
c
         do 245 j = 1, l
            e(j) = e(j) / h
            f = f + e(j) * d(j)
  245    continue
c
         hh = f / (h + h)
c     .......... form q ..........
         do 250 j = 1, l
  250    e(j) = e(j) - hh * d(j)
c     .......... form reduced a ..........
         do 280 j = 1, l
            f = d(j)
            g = e(j)
c
            do 260 k = j, l
  260       z(k,j) = z(k,j) - f * e(k) - g * d(k)
c
            d(j) = z(l,j)
            z(i,j) = 0.0d0
  280    continue
c
  290    d(i) = h
  300 continue
c     .......... accumulation of transformation matrices ..........
      do 500 i = 2, n
         l = i - 1
         z(n,l) = z(l,l)
         z(l,l) = 1.0d0
         h = d(i)
         if (h .eq. 0.0d0) go to 380
c
         do 330 k = 1, l
  330    d(k) = z(k,i) / h
c
         do 360 j = 1, l
            g = 0.0d0
c
            do 340 k = 1, l
  340       g = g + z(k,i) * z(k,j)
c
            do 360 k = 1, l
               z(k,j) = z(k,j) - g * d(k)
  360    continue
c
  380    do 400 k = 1, l
  400    z(k,i) = 0.0d0
c
  500 continue
c
  510 do 520 i = 1, n
         d(i) = z(n,i)
         z(n,i) = 0.0d0
  520 continue
c
      z(n,n) = 1.0d0
      e(1) = 0.0d0
      return
      end

c-----------------------------------------------------------------------------
      double precision function pythag(a,b)
      double precision a,b
c
c     finds dsqrt(a**2+b**2) without overflow or destructive underflow
c
      double precision p,r,s,t,u
      p = dmax1(dabs(a),dabs(b))
      if (p .eq. 0.0d0) go to 20
      r = (dmin1(dabs(a),dabs(b))/p)**2
   10 continue
         t = 4.0d0 + r
         if (t .eq. 4.0d0) go to 20
         s = r/t
         u = 1.0d0 + 2.0d0*s
         p = u*p
         r = (s/u)**2 * r
      go to 10
   20 pythag = p
      return
      end

