! POTENTIALPROGRAMM FUER CLNO
!
!  Es berechnet das 3D-Potential in den Jacobi-Koordinaten:
!     y1=Laenge N=O, y2=Abstand von Cl zum Schwerpunkt S des N=O,
!     y3=Winkel ClSO in rad.
!
!     Die Abstaende sind in a.u. und der Winkel in rad einzugeben.
!     Der vom Programm gelieferte Potentialwert v ist in a.u.(Hartree).
!     Potentialbereich: y1 von 1.8 bis 3.0 a.u.
!                       y2     3.5 a.u. bis beliebig (analytisch fort-
!                                                      gesetzt)
!                       y3      0 bis pi (180 Grad).
!      Franck_Condon-Punkt bei y1=2.1550 a.u.
!                              y2=4.3137 a.u.
!                              y3=127.4 Grad
!     Initialisierung des Programms mit  CALL INNOCL1SCH ,
!     Aufruf mit CALL NOCL1SCH(Y1,Y2,Y3,V).
!     Die Potentialdaten aus dem file CLNO3D DATEN werden auf
!     unit 50 eingelesen.
!#######################################################################

      subroutine nocl1sch(y1,y2,y3,v)

      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      DIMENSION rr(3),fin(4)

      common/gridnocl/ x(20),y(20),z(20),nx,ny,nz

      save /gridnocl/

      parameter (a=0.533205d0,vasy=0.0456375d0)

!++++++++++++++++++++++++++++++++++++++++++++++++++++
      y1 = max(1.8d0,y1) ! The program will now run for
      y1 = min(3.0d0,y1) ! all values of the coordinates.
      y2 = max(3.5d0,y2)
!++++++++++++++++++++++++++++++++++++++++++++++++++++

!
! Transformation auf Molekularkoordinaten (r,rs2,bet), in denen
! die Potentialdaten gegeben sind.
! r=y1, rs2= Abstand Cl-N, bet= Winkel ClNO.
      b=a*y1
      rs2=dsqrt(b*b+y2*y2+2.d0*b*y2*dcos(y3))
      arg=(b**2+rs2**2-y2**2)/(2.d0*b*rs2)
      if(arg.gt.1.d0) then
        bet=0.d0
        goto 610
      end if
      if(arg.lt.-1.d0) then
        bet=3.141592654d0
        goto 610
      end if

      bet=dacos(arg)

 610  continue
      rr(1)=y1
!
      if(rs2.ge.y(1).and.rs2.le.y(ny)) then
      rr(2)=rs2
      goto 600
      end if
!
      if(rs2.gt.y(ny)) then
      rr(2)=y(ny)
      goto 600
      end if
!
      if(rs2.lt.y(1)) then
      rr(2)=y(1)
      goto 620
      end if
!
 600  continue
      if(bet.ge.z(1).and.bet.le.z(nz)) then
      rr(3)=bet
!     write (6,*) "1",rr
      call naturl(rr,0,fin)
      v=fin(1)-vasy
      return
      end if
!
      if(bet.lt.z(1)) rr(3)=z(1)
      if(bet.gt.z(nz)) rr(3)=z(nz)

!     write (6,*) "2",rr
      call naturl(rr,0,fin)
      v=fin(1)
!     write (6,*) "3",rr
      call naturl(rr,3,fin)
      if(fin(4).lt.0.d0.and.bet.gt.z(nz)) goto 666
      if(fin(4).gt.0.d0.and.bet.lt.z(1))  goto 666
      bkoeff=-fin(4)/dsin(rr(3))
      akoeff=v-bkoeff*dcos(rr(3))
      v=akoeff+bkoeff*dcos(bet)
666   v=v-vasy
      return
!
620   if(bet.ge.z(1).and.bet.le.z(nz)) then
      rr(3)=bet
!     write (6,*) "4",rr
      call naturl(rr,0,fin)
      v=fin(1)
!     write (6,*) "5",rr
      call naturl(rr,3,fin)
      v=v*dexp(fin(3)/v*(rs2-rr(2)))
      v=v-vasy
      return
      end if

      if(bet.lt.z(1))  rr(3)=z(1)
      if(bet.gt.z(nz)) rr(3)=z(nz)
!     write (6,*) "6",rr
      call naturl(rr,0,fin)
      v=fin(1)
!     write (6,*) "7",rr
      call naturl(rr,3,fin)
      v=v*dexp(fin(3)/v*(rs2-rr(2)))

!     write (6,*) "8",rr
      call naturl(rr,3,fin)
      if(fin(4).lt.0.d0.and.bet.gt.z(nz)) goto 6262
      if(fin(4).gt.0.d0.and.bet.lt.z(1))  goto 6262
      bkoeff=-fin(4)/dsin(rr(3))
      akoeff=v-bkoeff*dcos(rr(3))
      v=akoeff+bkoeff*dcos(bet)
6262  v=v-vasy
      return
!
      END

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
!
!  INITIALISIERUNG DES SPLINE-PROGRAMMS (EINLESEN DER DATEN):
!
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      subroutine innocl1sch(opdatadef)

      IMPLICIT REAL*8 (A-H,O-Z)
      implicit integer (i-n)

      character(len=160) opdata
      character(len=*), intent(in) :: opdatadef

      common/surfnocl/en(1650),px(1650),py(1650),pz(1650), &
                    pxy(1650),pxz(1650),pyz(1650),pxyz(1650)
      common/gridnocl/ x(20),y(20),z(20),nx,ny,nz

      save /surfnocl/,/gridnocl/
!
      opdata=opdatadef
      ilbl=index(opdata,' ')-1
      OPEN (50,FILE=opdata(1:ilbl)//"/nocl1_data",status="old",iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a,/a)') &
          'Cannot open the surface data file for nocl1:', &
           opdata(1:ilbl)//"/nocl1_data"
         stop
      endif

      READ (50,*) NX,(X(I),I=1,NX)
      READ (50,*) NY,(Y(I),I=1,NY)
      READ (50,*) NZ,(Z(I),I=1,NZ)
      NXYZ=NX*NY*NZ

      READ (50,*) (EN(I),I=1,99)
      READ (50,*) (EN(99+I),I=1,99)
      READ (50,*) (EN(198+I),I=1,99)
      READ (50,*) (EN(297+I),I=1,99)
      READ (50,*) (EN(396+I),I=1,99)
      READ (50,*) (EN(495+I),I=1,99)
      READ (50,*) (EN(594+I),I=1,36)

      READ (50,*) (PX(I),I=1,99)
      READ (50,*) (PX(99+I),I=1,99)
      READ (50,*) (PX(198+I),I=1,99)
      READ (50,*) (PX(297+I),I=1,99)
      READ (50,*) (PX(396+I),I=1,99)
      READ (50,*) (PX(495+I),I=1,99)
      READ (50,*) (PX(594+I),I=1,36)

      READ (50,*) (PY(I),I=1,99)
      READ (50,*) (PY(99+I),I=1,99)
      READ (50,*) (PY(198+I),I=1,99)
      READ (50,*) (PY(297+I),I=1,99)
      READ (50,*) (PY(396+I),I=1,99)
      READ (50,*) (PY(495+I),I=1,99)
      READ (50,*) (PY(594+I),I=1,36)

      READ (50,*) (PZ(I),I=1,99)
      READ (50,*) (PZ(99+I),I=1,99)
      READ (50,*) (PZ(198+I),I=1,99)
      READ (50,*) (PZ(297+I),I=1,99)
      READ (50,*) (PZ(396+I),I=1,99)
      READ (50,*) (PZ(495+I),I=1,99)
      READ (50,*) (PZ(594+I),I=1,36)

      READ (50,*) (PXY(I),I=1,99)
      READ (50,*) (PXY(99+I),I=1,99)
      READ (50,*) (PXY(198+I),I=1,99)
      READ (50,*) (PXY(297+I),I=1,99)
      READ (50,*) (PXY(396+I),I=1,99)
      READ (50,*) (PXY(495+I),I=1,99)
      READ (50,*) (PXY(594+I),I=1,36)

      READ (50,*) (PXZ(I),I=1,99)
      READ (50,*) (PXZ(99+I),I=1,99)
      READ (50,*) (PXZ(198+I),I=1,99)
      READ (50,*) (PXZ(297+I),I=1,99)
      READ (50,*) (PXZ(396+I),I=1,99)
      READ (50,*) (PXZ(495+I),I=1,99)
      READ (50,*) (PXZ(594+I),I=1,36)

      READ (50,*) (PYZ(I),I=1,99)
      READ (50,*) (PYZ(99+I),I=1,99)
      READ (50,*) (PYZ(198+I),I=1,99)
      READ (50,*) (PYZ(297+I),I=1,99)
      READ (50,*) (PYZ(396+I),I=1,99)
      READ (50,*) (PYZ(495+I),I=1,99)
      READ (50,*) (PYZ(594+I),I=1,36)

      READ (50,*) (PXYZ(I),I=1,99)
      READ (50,*) (PXYZ(99+I),I=1,99)
      READ (50,*) (PXYZ(198+I),I=1,99)
      READ (50,*) (PXYZ(297+I),I=1,99)
      READ (50,*) (PXYZ(396+I),I=1,99)
      READ (50,*) (PXYZ(495+I),I=1,99)
      READ (50,*) (PXYZ(594+I),I=1,36)

      CLOSE (50)
!
!        write(6,*) nx,(x(i),i=1,nx)
!        write(6,*) ny,(y(i),i=1,ny)
!        write(6,*) nz,(z(i),i=1,nz)
!        write(6,300)
!        do 10 I=1,nxyz
!10        write(6,100)i,en(i),px(i),py(i),pz(i)
!        write(6,200)
!        do 20 i=1,nxyz
!20        write(6,100)i,pxy(i),pxz(i),pyz(i),pxyz(i)
!100       format(i7,4f10.6)
!300       format('  i,         e,       px,      py,      pz')
!200       format('  i,       pxy,      pxz,     pyz,     pxyz')

      return
      stop
      end
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      SUBROUTINE NATURL(R,INDX,FIN)
!
!     INDX = 0  :  INTERPOLATED POTENTIAL ONLY
!            3  :        "      X,Y,Z-DERIVATIVES
!
      IMPLICIT REAL*8(A-H,O-Z)
      implicit integer (i-n)
      DIMENSION FIN(4),FIJK(64),HI(3),XR(3),U(3,4),R(3)
      common/gridnocl/ x(20),y(20),z(20),nx,ny,nz

      save /gridnocl/

      DATA UN/1.D0/,TWO/2.D0/,THREE/3.D0/,SIX/6.D0/
!
      iv=0
      jv=0
      kv=0
      do i=1,64
         fijk(i)=0d0
      enddo
      LS=1
      IF(R(1).LT.X(1).OR.R(1).GT.X(NX)) GOTO 110
      LS=2
      IF(R(2).LT.Y(1).OR.R(2).GT.Y(NY)) GOTO 110
      LS=3
      IF(R(3).LT.Z(1).OR.R(3).GT.Z(NZ)) GOTO 110
      DO 10 IS=2,NX
      IF(R(1).LT.X(IS)) GOTO 20
   10 CONTINUE
   20 DO 30 JS=2,NY
      IF(R(2).LT.Y(JS)) GOTO 40
   30 CONTINUE
   40 DO 50 KS=2,NZ
      IF(R(3).LT.Z(KS)) GOTO 60
   50 CONTINUE
   60 IF(IS.NE.IV) GOTO 65
      IF(JS.NE.JV) GOTO 65
      IF(KS.EQ.KV) GOTO 75
   65 CALL FORMV(IS,JS,KS,FIJK)
      HI(1)=X(IS)-X(IS-1)
      IV=IS
      HI(2)=Y(JS)-Y(JS-1)
      JV=JS
      HI(3)=Z(KS)-Z(KS-1)
      KV=KS
   75 XR(1)=(R(1)-X(IS-1))/HI(1)
      XR(2)=(R(2)-Y(JS-1))/HI(2)
      XR(3)=(R(3)-Z(KS-1))/HI(3)
      IN=INDX
   90 DO 70 I=1,3
      IF(I.EQ.IN) GOTO 70
      XR2=XR(I)**2
      XR3=XR(I)*XR2
      U(I,1)=-TWO*XR3+THREE*XR2
      U(I,2)=-U(I,1)+UN
      U(I,3)=HI(I)*(XR3-XR2)
      U(I,4)=HI(I)*(XR3-TWO*XR2+XR(I))
   70 CONTINUE
      IF(IN.EQ.0) GOTO 80
      U(IN,1)=SIX*(-XR(IN)+1)*XR(IN)/HI(IN)
      U(IN,2)=-U(IN,1)
      U(IN,3)=(THREE*XR(IN)-TWO)*XR(IN)
      U(IN,4)=U(IN,3)-TWO*XR(IN)+UN
   80 FIN(IN+1)=0.D0
      IJK=1
      DO K=1,4
         DO J=1,4
            UKJ=U(3,K)*U(2,J)
            DO I=1,4
               FIN(IN+1)=FIN(IN+1)+FIJK(IJK)*UKJ*U(1,I)
               IJK=IJK+1
            enddo
         enddo
      enddo
      IN=IN-1
      IF(IN.GT.0.AND.INDX.GT.0) GOTO 90
      RETURN
  110 PRINT 120,LS
  120 FORMAT( 1X,15('*'),' R(',I1,') SORT DE LA GRILLE')
      write(6,*) r(ls)
!cc   RETURN 1
      stop
      END

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      SUBROUTINE FORMV(I,J,K,G)
      IMPLICIT REAL*8(A-H,O-Z)
      implicit integer (i-n)
      DIMENSION G(64)

      common/surfnocl/f(1650),px(1650),py(1650),pz(1650), &
                    pxy(1650),pxz(1650),pyz(1650),pxyz(1650)
      common/gridnocl/ x(20),y(20),z(20),nx,ny,nz

      save /surfnocl/,/gridnocl/
!
      IJK=(K-1)*NX*NY+(J-1)*NX+I
      IMJK=IJK-1
      IJMK=IJK-NX
      IMJMK=IJMK-1
      IJKM=IJK-NX*NY
      IMJKM=IJKM-1
      IJMKM=IJKM-NX
      IMJMKM=IJMKM-1
!
      G( 1)=F(IJK)
      G( 2)=F(IMJK)
      G( 3)=PX(IJK)
      G( 4)=PX(IMJK)
      G( 5)=F(IJMK)
      G( 6)=F(IMJMK)
      G( 7)=PX(IJMK)
      G( 8)=PX(IMJMK)
      G( 9)=PY(IJK)
      G(10)=PY(IMJK)
      G(11)=PXY(IJK)
      G(12)=PXY(IMJK)
      G(13)=PY(IJMK)
      G(14)=PY(IMJMK)
      G(15)=PXY(IJMK)
      G(16)=PXY(IMJMK)
      G(17)=F(IJKM)
      G(18)=F(IMJKM)
      G(19)=PX(IJKM)
      G(20)=PX(IMJKM)
      G(21)=F(IJMKM)
      G(22)=F(IMJMKM)
      G(23)=PX(IJMKM)
      G(24)=PX(IMJMKM)
      G(25)=PY(IJKM)
      G(26)=PY(IMJKM)
      G(27)=PXY(IJKM)
      G(28)=PXY(IMJKM)
      G(29)=PY(IJMKM)
      G(30)=PY(IMJMKM)
      G(31)=PXY(IJMKM)
      G(32)=PXY(IMJMKM)
      G(33)=PZ(IJK)
      G(34)=PZ(IMJK)
      G(35)=PXZ(IJK)
      G(36)=PXZ(IMJK)
      G(37)=PZ(IJMK)
      G(38)=PZ(IMJMK)
      G(39)=PXZ(IJMK)
      G(40)=PXZ(IMJMK)
      G(41)=PYZ(IJK)
      G(42)=PYZ(IMJK)
      G(43)=PXYZ(IJK)
      G(44)=PXYZ(IMJK)
      G(45)=PYZ(IJMK)
      G(46)=PYZ(IMJMK)
      G(47)=PXYZ(IJMK)
      G(48)=PXYZ(IMJMK)
      G(49)=PZ(IJKM)
      G(50)=PZ(IMJKM)
      G(51)=PXZ(IJKM)
      G(52)=PXZ(IMJKM)
      G(53)=PZ(IJMKM)
      G(54)=PZ(IMJMKM)
      G(55)=PXZ(IJMKM)
      G(56)=PXZ(IMJMKM)
      G(57)=PYZ(IJKM)
      G(58)=PYZ(IMJKM)
      G(59)=PXYZ(IJKM)
      G(60)=PXYZ(IMJKM)
      G(61)=PYZ(IJMKM)
      G(62)=PYZ(IMJMKM)
      G(63)=PXYZ(IJMKM)
      G(64)=PXYZ(IMJMKM)
      RETURN
      END

