! POTENTIALPROGRAMM FUER OZON
!
!  Es berechnet das 3D-Potential in den Jacobi-Koordinaten:
!     y1=Laenge O=O, y2=Abstand von O zum Schwerpunkt S des O=O,
!     y3=Winkel OSO in rad.
!
!     Die Abstaende sind in a.u. und der Winkel in rad einzugeben.
!     Der vom Programm gelieferte Potentialwert v ist in a.u.(Hartree).
!     Potentialbereich: y1 von 1.8 bis 3.0 a.u.
!                       y2     3.5 a.u. bis beliebig (analytisch fort-
!                                                      gesetzt)
!                       y3      0 bis pi (180 Grad).
!      Franck_Condon-Punkt bei y1=2.1550 a.u.
!                              y2=4.3137 a.u.
!                              y3=127.4 Grad
!     Initialisierung des Programms mit  CALL INNO!L1S!H ,
!     Aufruf mit CALL NOCL1SCH(Y1,Y2,Y3,V).
!     Die Potentialdaten aus dem file CLNO3D DATEN werden auf
!     unit 50 eingelesen.
!#######################################################################

      subroutine defo3woy(hopilab,label)

      implicit none

      integer hopilab
      character(len=*) label


      if (label .eq. 'o3woy11') then
         hopilab = 1
!
! -- Read data for the surface --
!
         call ino3woy11

      else if (label .eq. 'o3woy22') then
         hopilab = 2
!
! -- Read data for the surface --
!
         call ino3woy22

      else if (label .eq. 'o3woy12') then
         hopilab = 3
!
! -- Read data for the surface --
!
         call ino3woy12

      else if (label .eq. 'o3woydip1') then
         hopilab = 4

      else if (label .eq. 'o3woydip2') then
         hopilab = 5

      else if (label .eq. 'o3woy00') then
         hopilab = 6

      endif

      return
      end

!#######################################################################

      subroutine func_o3woy(gpoint,v,ifunc,icoord)

      implicit none

      integer icoord,ifunc
      real*8 gpoint(3),v,r1,r2,alpha,rv,rd,theta,pi,angfac,b,arg

      pi = 4.0d0*atan(1.0d0)
      angfac = 180.0d0/pi

      if (icoord .eq. 1) then
         r1 = gpoint(1)
         r2 = gpoint(2)
         alpha = gpoint(3)*angfac

      else if (icoord .eq. 2) then
         rd = gpoint(1)
         rv = gpoint(2)
         theta = gpoint(3)
!
! Transformation auf Molekularkoordinaten (r1,r2,alpha), in denen
! die Potentialdaten gegeben sind.
!
         r1 = rv
         b = 0.5d0*rv
         r2=dsqrt(b*b+rd*rd+2.d0*b*rd*dcos(theta))
         arg=(b**2+r2**2-rd**2)/(2.d0*b*r2)
         if(arg.gt.1.d0) then
            alpha=0.d0
         else if(arg.lt.-1.d0) then
            alpha=3.141592654d0
         else
            alpha=dacos(arg)
         end if

         alpha = alpha*angfac
      endif

      go to (1,2,3,4,5,6) ifunc

      print*,'ERROR: Ozone function not found :',ifunc
      stop

    1 call o3woy11(r1,r2,alpha,v)
      return

    2 call o3woy22(r1,r2,alpha,v)
      return

    3 call o3woy12(r1,r2,alpha,v)
      return

    4 call o3woydip1(r1,r2,alpha,v)
      return

    5 call o3woydip2(r1,r2,alpha,v)
      return

    6 call o3woy00(r1,r2,alpha,v)
      return

      end

!#######################################################################

      subroutine o3woy11(y1,y2,y3,v)

      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      DIMENSION rr(3),fin(4)

      parameter (ndx=160,ndy=60,ndz=110)
      common/gridozon/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridozon/

      if (y1.le.y2) then
         rr(1)=y2
         rr(2)=y1
      else
         rr(1)=y1
         rr(2)=y2
      endif
      rr(3)=y3

!-----------------------------------------------------------------------
! Surface only defined in following region
!-----------------------------------------------------------------------
      rr(1)=max(rr(1),x(1))
      rr(1)=min(rr(1),x(nx))
      rr(2)=max(rr(2),y(1))
      rr(2)=min(rr(2),y(ny))
      rr(3)=max(rr(3),z(1))
      rr(3)=min(rr(3),z(nz))

      call naturl11(rr,0,fin)
      v=fin(1)/27.2114d0
      return

      END

!#######################################################################

      subroutine o3woy22(y1,y2,y3,v)

      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      DIMENSION rr(3),fin(4)

      parameter (ndx=160,ndy=60,ndz=110)
      common/gridozon/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridozon/

      if (y1.le.y2) then
         rr(1)=y2
         rr(2)=y1
      else
         rr(1)=y1
         rr(2)=y2
      endif
      rr(3)=y3

!-----------------------------------------------------------------------
! Surface only defined in following region
!-----------------------------------------------------------------------
      rr(1)=max(rr(1),x(1))
      rr(1)=min(rr(1),x(nx))
      rr(2)=max(rr(2),y(1))
      rr(2)=min(rr(2),y(ny))
      rr(3)=max(rr(3),z(1))
      rr(3)=min(rr(3),z(nz))

      call naturl22(rr,0,fin)
      v=fin(1)/27.2114d0
      return

      END

!#######################################################################

      subroutine o3woy12(y1,y2,y3,v)

      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      DIMENSION rr(3),fin(4)

      parameter (ndx=160,ndy=60,ndz=110)
      common/gridozon/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridozon/

      if (y1.le.y2) then
         rr(1)=y2
         rr(2)=y1
      else
         rr(1)=y1
         rr(2)=y2
      endif
      rr(3)=y3
!-----------------------------------------------------------------------
! Surface only defined in following region
!-----------------------------------------------------------------------
      rr(1)=max(rr(1),x(1))
      rr(1)=min(rr(1),x(nx))
      rr(2)=max(rr(2),y(1))
      rr(2)=min(rr(2),y(ny))
      rr(3)=max(rr(3),z(1))
      rr(3)=min(rr(3),z(nz))

      call naturl12(rr,0,fin)
      v=fin(1)/27.2114d0
      return

      END

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@
!
!  INITIALISIERUNG DES SPLINE-PROGRAMMS (EINLESEN DER DATEN):
!
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      subroutine ino3woy11

      IMPLICIT REAL*8 (A-H,O-Z)
      implicit integer (i-n)

      parameter (ndat=999000,ndx=160,ndy=60,ndz=110)
      common /surfozon11/ en11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridozon/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfozon11/,/gridozon/

      OPEN (50,FILE="o3woy_data/o3woy11_data",status="old",iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a,/a)') &
          'Cannot open the surface data file for Ozone 11:', &
           "o3woy_data/o3woy11_data"
         stop
      endif

      READ (50,*) NX,(X(I),I=1,NX)
      READ (50,*) NY,(Y(I),I=1,NY)
      READ (50,*) NZ,(Z(I),I=1,NZ)
      NXYZ=NX*NY*NZ
      READ (50,*) (EN11(I),I=1,NXYZ)
      READ (50,*) (PX11(I),I=1,NXYZ)
      READ (50,*) (PY11(I),I=1,NXYZ)
      READ (50,*) (PZ11(I),I=1,NXYZ)
      READ (50,*) (PXY11(I),I=1,NXYZ)
      READ (50,*) (PXZ11(I),I=1,NXYZ)
      READ (50,*) (PYZ11(I),I=1,NXYZ)
      READ (50,*) (PXYZ11(I),I=1,NXYZ)

      CLOSE (50)
!
      return
      stop
      end
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      subroutine ino3woy22

      IMPLICIT REAL*8 (A-H,O-Z)
      implicit integer (i-n)

      parameter (ndat=999000,ndx=160,ndy=60,ndz=110)
      common /surfozon22/ en22(ndat),px22(ndat),py22(ndat),pz22(ndat), &
                      pxy22(ndat),pxz22(ndat),pyz22(ndat),pxyz22(ndat)
      common/gridozon/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfozon22/,/gridozon/

      OPEN (50,FILE="o3woy_data/o3woy22_data",status="old",iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a,/a)') &
          'Cannot open the surface data file for Ozone 22:', &
           "o3woy_data/o3woy22_data"
         stop
      endif

      READ (50,*) NX,(X(I),I=1,NX)
      READ (50,*) NY,(Y(I),I=1,NY)
      READ (50,*) NZ,(Z(I),I=1,NZ)
      NXYZ=NX*NY*NZ
      READ (50,*) (EN22(I),I=1,NXYZ)
      READ (50,*) (PX22(I),I=1,NXYZ)
      READ (50,*) (PY22(I),I=1,NXYZ)
      READ (50,*) (PZ22(I),I=1,NXYZ)
      READ (50,*) (PXY22(I),I=1,NXYZ)
      READ (50,*) (PXZ22(I),I=1,NXYZ)
      READ (50,*) (PYZ22(I),I=1,NXYZ)
      READ (50,*) (PXYZ22(I),I=1,NXYZ)

      CLOSE (50)
!
      return
      stop
      end
!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      subroutine ino3woy12

      IMPLICIT REAL*8 (A-H,O-Z)
      implicit integer (i-n)

      parameter (ndat=999000,ndx=160,ndy=60,ndz=110)
      common /surfozon12/ en12(ndat),px12(ndat),py12(ndat),pz12(ndat), &
                      pxy12(ndat),pxz12(ndat),pyz12(ndat),pxyz12(ndat)
      common/gridozon/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfozon12/,/gridozon/

      OPEN (50,FILE="o3woy_data/o3woy12_data",status="old",iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a,/a)') &
          'Cannot open the surface data file for Ozone 12:', &
           "o3woy_data/o3woy12_data"
         stop
      endif

      READ (50,*) NX,(X(I),I=1,NX)
      READ (50,*) NY,(Y(I),I=1,NY)
      READ (50,*) NZ,(Z(I),I=1,NZ)
      NXYZ=NX*NY*NZ
      READ (50,*) (EN12(I),I=1,NXYZ)
      READ (50,*) (PX12(I),I=1,NXYZ)
      READ (50,*) (PY12(I),I=1,NXYZ)
      READ (50,*) (PZ12(I),I=1,NXYZ)
      READ (50,*) (PXY12(I),I=1,NXYZ)
      READ (50,*) (PXZ12(I),I=1,NXYZ)
      READ (50,*) (PYZ12(I),I=1,NXYZ)
      READ (50,*) (PXYZ12(I),I=1,NXYZ)

      CLOSE (50)
!
      return
      stop
      end

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      SUBROUTINE NATURL11(R,INDX,FIN)
!
!     INDX = 0  :  INTERPOLATED POTENTIAL ONLY
!            3  :        "      X,Y,Z-DERIVATIVES
!
      IMPLICIT REAL*8(A-H,O-Z)
      parameter (ndx=160, ndy=60,ndz=110)
      DIMENSION FIN(4),FIJK(64),HI(3),XR(3),U(3,4),R(3)
      common/gridozon/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridozon/

      DATA UN/1.D0/,TWO/2.D0/,THREE/3.D0/,SIX/6.D0/,IV,JV,KV/3*0/
!
      LS=1
      IF(R(1).LT.X(1).OR.R(1).GT.X(NX)) GOTO 110
      LS=2
      IF(R(2).LT.Y(1).OR.R(2).GT.Y(NY)) GOTO 110
      LS=3
      IF(R(3).LT.Z(1).OR.R(3).GT.Z(NZ)) GOTO 110
      DO IS=2,NX
         IF(R(1).LT.X(IS)) GOTO 20
      enddo
   20 DO JS=2,NY
         IF(R(2).LT.Y(JS)) GOTO 40
      enddo
   40 DO KS=2,NZ
         IF(R(3).LT.Z(KS)) GOTO 60
      enddo
   60 IF(IS.NE.IV) GOTO 65
      IF(JS.NE.JV) GOTO 65
      IF(KS.EQ.KV) GOTO 75
   65 CALL FORMV11(IS,JS,KS,FIJK)
      HI(1)=X(IS)-X(IS-1)
      IV=IS
      HI(2)=Y(JS)-Y(JS-1)
      JV=JS
      HI(3)=Z(KS)-Z(KS-1)
      KV=KS
   75 XR(1)=(R(1)-X(IS-1))/HI(1)
      XR(2)=(R(2)-Y(JS-1))/HI(2)
      XR(3)=(R(3)-Z(KS-1))/HI(3)
      IN=INDX
   90 DO I=1,3
         IF(I.EQ.IN) GOTO 70
         XR2=XR(I)**2
         XR3=XR(I)*XR2
         U(I,1)=-TWO*XR3+THREE*XR2
         U(I,2)=-U(I,1)+UN
         U(I,3)=HI(I)*(XR3-XR2)
         U(I,4)=HI(I)*(XR3-TWO*XR2+XR(I))
   70    CONTINUE
      enddo
      IF(IN.EQ.0) GOTO 80
      U(IN,1)=SIX*(-XR(IN)+1)*XR(IN)/HI(IN)
      U(IN,2)=-U(IN,1)
      U(IN,3)=(THREE*XR(IN)-TWO)*XR(IN)
      U(IN,4)=U(IN,3)-TWO*XR(IN)+UN
   80 FIN(IN+1)=0.D0
      IJK=1
      DO K=1,4
         DO J=1,4
            UKJ=U(3,K)*U(2,J)
            DO I=1,4
               FIN(IN+1)=FIN(IN+1)+FIJK(IJK)*UKJ*U(1,I)
               IJK=IJK+1
            enddo
         enddo
      enddo
      IN=IN-1
      IF(IN.GT.0.AND.INDX.GT.0) GOTO 90
      RETURN
  110 PRINT 120,LS
  120 FORMAT( 1X,15('*'),' R(',I1,') SORT DE LA GRILLE')
      print*,r(ls)
      RETURN
      END

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      SUBROUTINE NATURL22(R,INDX,FIN)
!
!     INDX = 0  :  INTERPOLATED POTENTIAL ONLY
!            3  :        "      X,Y,Z-DERIVATIVES
!
      IMPLICIT REAL*8(A-H,O-Z)
      parameter (ndx=160, ndy=60,ndz=110)
      DIMENSION FIN(4),FIJK(64),HI(3),XR(3),U(3,4),R(3)
      common/gridozon/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridozon/

      DATA UN/1.D0/,TWO/2.D0/,THREE/3.D0/,SIX/6.D0/,IV,JV,KV/3*0/
!
      LS=1
      IF(R(1).LT.X(1).OR.R(1).GT.X(NX)) GOTO 110
      LS=2
      IF(R(2).LT.Y(1).OR.R(2).GT.Y(NY)) GOTO 110
      LS=3
      IF(R(3).LT.Z(1).OR.R(3).GT.Z(NZ)) GOTO 110
      DO IS=2,NX
         IF(R(1).LT.X(IS)) GOTO 20
      enddo
   20 DO JS=2,NY
         IF(R(2).LT.Y(JS)) GOTO 40
      enddo
   40 DO KS=2,NZ
         IF(R(3).LT.Z(KS)) GOTO 60
      enddo
   60 IF(IS.NE.IV) GOTO 65
      IF(JS.NE.JV) GOTO 65
      IF(KS.EQ.KV) GOTO 75
   65 CALL FORMV22(IS,JS,KS,FIJK)
      HI(1)=X(IS)-X(IS-1)
      IV=IS
      HI(2)=Y(JS)-Y(JS-1)
      JV=JS
      HI(3)=Z(KS)-Z(KS-1)
      KV=KS
   75 XR(1)=(R(1)-X(IS-1))/HI(1)
      XR(2)=(R(2)-Y(JS-1))/HI(2)
      XR(3)=(R(3)-Z(KS-1))/HI(3)
      IN=INDX
   90 DO I=1,3
         IF(I.EQ.IN) GOTO 70
         XR2=XR(I)**2
         XR3=XR(I)*XR2
         U(I,1)=-TWO*XR3+THREE*XR2
         U(I,2)=-U(I,1)+UN
         U(I,3)=HI(I)*(XR3-XR2)
         U(I,4)=HI(I)*(XR3-TWO*XR2+XR(I))
   70 CONTINUE
      enddo
      IF(IN.EQ.0) GOTO 80
      U(IN,1)=SIX*(-XR(IN)+1)*XR(IN)/HI(IN)
      U(IN,2)=-U(IN,1)
      U(IN,3)=(THREE*XR(IN)-TWO)*XR(IN)
      U(IN,4)=U(IN,3)-TWO*XR(IN)+UN
   80 FIN(IN+1)=0.D0
      IJK=1
      DO K=1,4
         DO J=1,4
            UKJ=U(3,K)*U(2,J)
            DO I=1,4
               FIN(IN+1)=FIN(IN+1)+FIJK(IJK)*UKJ*U(1,I)
               IJK=IJK+1
            enddo
         enddo
      enddo
      IN=IN-1
      IF(IN.GT.0.AND.INDX.GT.0) GOTO 90
      RETURN
  110 PRINT 120,LS
  120 FORMAT( 1X,15('*'),' R(',I1,') SORT DE LA GRILLE')
      print*,r(ls)
      RETURN
      END

!@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@$@

      SUBROUTINE NATURL12(R,INDX,FIN)
!
!     INDX = 0  :  INTERPOLATED POTENTIAL ONLY
!            3  :        "      X,Y,Z-DERIVATIVES
!
      IMPLICIT REAL*8(A-H,O-Z)
      parameter (ndx=160, ndy=60,ndz=110)
      DIMENSION FIN(4),FIJK(64),HI(3),XR(3),U(3,4),R(3)
      common/gridozon/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /gridozon/

      DATA UN/1.D0/,TWO/2.D0/,THREE/3.D0/,SIX/6.D0/,IV,JV,KV/3*0/
!
      LS=1
      IF(R(1).LT.X(1).OR.R(1).GT.X(NX)) GOTO 110
      LS=2
      IF(R(2).LT.Y(1).OR.R(2).GT.Y(NY)) GOTO 110
      LS=3
      IF(R(3).LT.Z(1).OR.R(3).GT.Z(NZ)) GOTO 110
      DO 10 IS=2,NX
      IF(R(1).LT.X(IS)) GOTO 20
   10 CONTINUE
   20 DO JS=2,NY
         IF(R(2).LT.Y(JS)) GOTO 40
      enddo
   40 DO KS=2,NZ
         IF(R(3).LT.Z(KS)) GOTO 60
      enddo
   60 IF(IS.NE.IV) GOTO 65
      IF(JS.NE.JV) GOTO 65
      IF(KS.EQ.KV) GOTO 75
   65 CALL FORMV12(IS,JS,KS,FIJK)
      HI(1)=X(IS)-X(IS-1)
      IV=IS
      HI(2)=Y(JS)-Y(JS-1)
      JV=JS
      HI(3)=Z(KS)-Z(KS-1)
      KV=KS
   75 XR(1)=(R(1)-X(IS-1))/HI(1)
      XR(2)=(R(2)-Y(JS-1))/HI(2)
      XR(3)=(R(3)-Z(KS-1))/HI(3)
      IN=INDX
   90 DO 70 I=1,3
      IF(I.EQ.IN) GOTO 70
      XR2=XR(I)**2
      XR3=XR(I)*XR2
      U(I,1)=-TWO*XR3+THREE*XR2
      U(I,2)=-U(I,1)+UN
      U(I,3)=HI(I)*(XR3-XR2)
      U(I,4)=HI(I)*(XR3-TWO*XR2+XR(I))
   70 CONTINUE
      IF(IN.EQ.0) GOTO 80
      U(IN,1)=SIX*(-XR(IN)+1)*XR(IN)/HI(IN)
      U(IN,2)=-U(IN,1)
      U(IN,3)=(THREE*XR(IN)-TWO)*XR(IN)
      U(IN,4)=U(IN,3)-TWO*XR(IN)+UN
   80 FIN(IN+1)=0.D0
      IJK=1
      DO K=1,4
         DO J=1,4
            UKJ=U(3,K)*U(2,J)
            DO I=1,4
               FIN(IN+1)=FIN(IN+1)+FIJK(IJK)*UKJ*U(1,I)
               IJK=IJK+1
            enddo
         enddo
      enddo
      IN=IN-1
      IF(IN.GT.0.AND.INDX.GT.0) GOTO 90
      RETURN
  110 PRINT 120,LS
  120 FORMAT( 1X,15('*'),' R(',I1,') SORT DE LA GRILLE')
      print*,r(ls)
      RETURN
      END

!#######################################################################

      SUBROUTINE FORMV11(I,J,K,G)
      IMPLICIT REAL*8(A-H,O-Z)
      parameter (ndat=999000,ndx=160,ndy=60,ndz=110)
      DIMENSION G(64)

      common /surfozon11/ F11(ndat),px11(ndat),py11(ndat),pz11(ndat), &
                      pxy11(ndat),pxz11(ndat),pyz11(ndat),pxyz11(ndat)
      common/gridozon/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfozon11/,/gridozon/
!
      IJK=(K-1)*NX*NY+(J-1)*NX+I
      IMJK=IJK-1
      IJMK=IJK-NX
      IMJMK=IJMK-1
      IJKM=IJK-NX*NY
      IMJKM=IJKM-1
      IJMKM=IJKM-NX
      IMJMKM=IJMKM-1
!
      G( 1)=F11(IJK)
      G( 2)=F11(IMJK)
      G( 3)=PX11(IJK)
      G( 4)=PX11(IMJK)
      G( 5)=F11(IJMK)
      G( 6)=F11(IMJMK)
      G( 7)=PX11(IJMK)
      G( 8)=PX11(IMJMK)
      G( 9)=PY11(IJK)
      G(10)=PY11(IMJK)
      G(11)=PXY11(IJK)
      G(12)=PXY11(IMJK)
      G(13)=PY11(IJMK)
      G(14)=PY11(IMJMK)
      G(15)=PXY11(IJMK)
      G(16)=PXY11(IMJMK)
      G(17)=F11(IJKM)
      G(18)=F11(IMJKM)
      G(19)=PX11(IJKM)
      G(20)=PX11(IMJKM)
      G(21)=F11(IJMKM)
      G(22)=F11(IMJMKM)
      G(23)=PX11(IJMKM)
      G(24)=PX11(IMJMKM)
      G(25)=PY11(IJKM)
      G(26)=PY11(IMJKM)
      G(27)=PXY11(IJKM)
      G(28)=PXY11(IMJKM)
      G(29)=PY11(IJMKM)
      G(30)=PY11(IMJMKM)
      G(31)=PXY11(IJMKM)
      G(32)=PXY11(IMJMKM)
      G(33)=PZ11(IJK)
      G(34)=PZ11(IMJK)
      G(35)=PXZ11(IJK)
      G(36)=PXZ11(IMJK)
      G(37)=PZ11(IJMK)
      G(38)=PZ11(IMJMK)
      G(39)=PXZ11(IJMK)
      G(40)=PXZ11(IMJMK)
      G(41)=PYZ11(IJK)
      G(42)=PYZ11(IMJK)
      G(43)=PXYZ11(IJK)
      G(44)=PXYZ11(IMJK)
      G(45)=PYZ11(IJMK)
      G(46)=PYZ11(IMJMK)
      G(47)=PXYZ11(IJMK)
      G(48)=PXYZ11(IMJMK)
      G(49)=PZ11(IJKM)
      G(50)=PZ11(IMJKM)
      G(51)=PXZ11(IJKM)
      G(52)=PXZ11(IMJKM)
      G(53)=PZ11(IJMKM)
      G(54)=PZ11(IMJMKM)
      G(55)=PXZ11(IJMKM)
      G(56)=PXZ11(IMJMKM)
      G(57)=PYZ11(IJKM)
      G(58)=PYZ11(IMJKM)
      G(59)=PXYZ11(IJKM)
      G(60)=PXYZ11(IMJKM)
      G(61)=PYZ11(IJMKM)
      G(62)=PYZ11(IMJMKM)
      G(63)=PXYZ11(IJMKM)
      G(64)=PXYZ11(IMJMKM)
      RETURN
      END
! END OF 3DSPLINE-PROGRAM!
!#######################################################################

      SUBROUTINE FORMV22(I,J,K,G)
      IMPLICIT REAL*8(A-H,O-Z)
      parameter (ndat=999000,ndx=160,ndy=60,ndz=110)
      DIMENSION G(64)

      common /surfozon22/ F22(ndat),px22(ndat),py22(ndat),pz22(ndat), &
                      pxy22(ndat),pxz22(ndat),pyz22(ndat),pxyz22(ndat)
      common/gridozon/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfozon22/,/gridozon/
!
      IJK=(K-1)*NX*NY+(J-1)*NX+I
      IMJK=IJK-1
      IJMK=IJK-NX
      IMJMK=IJMK-1
      IJKM=IJK-NX*NY
      IMJKM=IJKM-1
      IJMKM=IJKM-NX
      IMJMKM=IJMKM-1
!
      G( 1)=F22(IJK)
      G( 2)=F22(IMJK)
      G( 3)=PX22(IJK)
      G( 4)=PX22(IMJK)
      G( 5)=F22(IJMK)
      G( 6)=F22(IMJMK)
      G( 7)=PX22(IJMK)
      G( 8)=PX22(IMJMK)
      G( 9)=PY22(IJK)
      G(10)=PY22(IMJK)
      G(11)=PXY22(IJK)
      G(12)=PXY22(IMJK)
      G(13)=PY22(IJMK)
      G(14)=PY22(IMJMK)
      G(15)=PXY22(IJMK)
      G(16)=PXY22(IMJMK)
      G(17)=F22(IJKM)
      G(18)=F22(IMJKM)
      G(19)=PX22(IJKM)
      G(20)=PX22(IMJKM)
      G(21)=F22(IJMKM)
      G(22)=F22(IMJMKM)
      G(23)=PX22(IJMKM)
      G(24)=PX22(IMJMKM)
      G(25)=PY22(IJKM)
      G(26)=PY22(IMJKM)
      G(27)=PXY22(IJKM)
      G(28)=PXY22(IMJKM)
      G(29)=PY22(IJMKM)
      G(30)=PY22(IMJMKM)
      G(31)=PXY22(IJMKM)
      G(32)=PXY22(IMJMKM)
      G(33)=PZ22(IJK)
      G(34)=PZ22(IMJK)
      G(35)=PXZ22(IJK)
      G(36)=PXZ22(IMJK)
      G(37)=PZ22(IJMK)
      G(38)=PZ22(IMJMK)
      G(39)=PXZ22(IJMK)
      G(40)=PXZ22(IMJMK)
      G(41)=PYZ22(IJK)
      G(42)=PYZ22(IMJK)
      G(43)=PXYZ22(IJK)
      G(44)=PXYZ22(IMJK)
      G(45)=PYZ22(IJMK)
      G(46)=PYZ22(IMJMK)
      G(47)=PXYZ22(IJMK)
      G(48)=PXYZ22(IMJMK)
      G(49)=PZ22(IJKM)
      G(50)=PZ22(IMJKM)
      G(51)=PXZ22(IJKM)
      G(52)=PXZ22(IMJKM)
      G(53)=PZ22(IJMKM)
      G(54)=PZ22(IMJMKM)
      G(55)=PXZ22(IJMKM)
      G(56)=PXZ22(IMJMKM)
      G(57)=PYZ22(IJKM)
      G(58)=PYZ22(IMJKM)
      G(59)=PXYZ22(IJKM)
      G(60)=PXYZ22(IMJKM)
      G(61)=PYZ22(IJMKM)
      G(62)=PYZ22(IMJMKM)
      G(63)=PXYZ22(IJMKM)
      G(64)=PXYZ22(IMJMKM)
      RETURN
      END
! END OF 3DSPLINE-PROGRAM!
!#######################################################################

      SUBROUTINE FORMV12(I,J,K,G)
      IMPLICIT REAL*8(A-H,O-Z)
      parameter (ndat=999000,ndx=160,ndy=60,ndz=110)
      DIMENSION G(64)

      common /surfozon12/ F12(ndat),px12(ndat),py12(ndat),pz12(ndat), &
                      pxy12(ndat),pxz12(ndat),pyz12(ndat),pxyz12(ndat)
      common/gridozon/ x(ndx),y(ndx),z(ndx),nx,ny,nz

      save /surfozon12/,/gridozon/
!
      IJK=(K-1)*NX*NY+(J-1)*NX+I
      IMJK=IJK-1
      IJMK=IJK-NX
      IMJMK=IJMK-1
      IJKM=IJK-NX*NY
      IMJKM=IJKM-1
      IJMKM=IJKM-NX
      IMJMKM=IJMKM-1
!
      G( 1)=F12(IJK)
      G( 2)=F12(IMJK)
      G( 3)=PX12(IJK)
      G( 4)=PX12(IMJK)
      G( 5)=F12(IJMK)
      G( 6)=F12(IMJMK)
      G( 7)=PX12(IJMK)
      G( 8)=PX12(IMJMK)
      G( 9)=PY12(IJK)
      G(10)=PY12(IMJK)
      G(11)=PXY12(IJK)
      G(12)=PXY12(IMJK)
      G(13)=PY12(IJMK)
      G(14)=PY12(IMJMK)
      G(15)=PXY12(IJMK)
      G(16)=PXY12(IMJMK)
      G(17)=F12(IJKM)
      G(18)=F12(IMJKM)
      G(19)=PX12(IJKM)
      G(20)=PX12(IMJKM)
      G(21)=F12(IJMKM)
      G(22)=F12(IMJMKM)
      G(23)=PX12(IJMKM)
      G(24)=PX12(IMJMKM)
      G(25)=PY12(IJKM)
      G(26)=PY12(IMJKM)
      G(27)=PXY12(IJKM)
      G(28)=PXY12(IMJKM)
      G(29)=PY12(IJMKM)
      G(30)=PY12(IMJMKM)
      G(31)=PXY12(IJMKM)
      G(32)=PXY12(IMJMKM)
      G(33)=PZ12(IJK)
      G(34)=PZ12(IMJK)
      G(35)=PXZ12(IJK)
      G(36)=PXZ12(IMJK)
      G(37)=PZ12(IJMK)
      G(38)=PZ12(IMJMK)
      G(39)=PXZ12(IJMK)
      G(40)=PXZ12(IMJMK)
      G(41)=PYZ12(IJK)
      G(42)=PYZ12(IMJK)
      G(43)=PXYZ12(IJK)
      G(44)=PXYZ12(IMJK)
      G(45)=PYZ12(IJMK)
      G(46)=PYZ12(IMJMK)
      G(47)=PXYZ12(IJMK)
      G(48)=PXYZ12(IMJMK)
      G(49)=PZ12(IJKM)
      G(50)=PZ12(IMJKM)
      G(51)=PXZ12(IJKM)
      G(52)=PXZ12(IMJKM)
      G(53)=PZ12(IJMKM)
      G(54)=PZ12(IMJMKM)
      G(55)=PXZ12(IJMKM)
      G(56)=PXZ12(IMJMKM)
      G(57)=PYZ12(IJKM)
      G(58)=PYZ12(IMJKM)
      G(59)=PXYZ12(IJKM)
      G(60)=PXYZ12(IMJKM)
      G(61)=PYZ12(IJMKM)
      G(62)=PYZ12(IMJMKM)
      G(63)=PXYZ12(IJMKM)
      G(64)=PXYZ12(IMJMKM)
      RETURN
      END
! END OF 3DSPLINE-PROGRAM!

!#######################################################################
!
! Dipole moment for O3 X -> A (diabatic)
!
!#######################################################################

      subroutine o3woydip1(y1,y2,y3,dip)
!     r,rr in a.u., gam in grad
      implicit none
      real*8 dip,sm,sp,gam,r,rr
      real*8 h,alpha

      real*8 y1,y2,y3

      if (y1.le.y2) then
         r=y2
         rr=y1
      else
         r=y1
         rr=y2
      endif
      gam=y3

      sm=r-rr
      sp=r+rr
      alpha=1.1d0*0.4d0/2.d0*(1+tanh(11.d0*(sp-4.95d0)))
      h=4.1d+9/2.d0*(tanh(0.02*(gam-120.d0)))
      dip=h*sm*exp(-alpha*sm**2)
      return
      end

!#######################################################################
!
! Dipole moment for O3 X -> B (diabatic)
!
!#######################################################################

      subroutine o3woydip2(y1,y2,y3,dip)
!     r,rr in a.u., gam in grad
      implicit none
      real*8 dip,sm,sp,gam,r,rr
      real*8 a,b,h,alpha

      real*8 y1,y2,y3

      if (y1.le.y2) then
         r=y2
         rr=y1
      else
         r=y1
         rr=y2
      endif
      gam=y3

      sm=r-rr
      sp=r+rr

      a=3.8+0.05*(gam-120)
      b=2+2.2*exp(-0.009*(gam-133)**2)
      h=a-b*(sp-5)
      alpha=0.4*exp(-3*(sp-4.7)**2)
      dip=h*exp(-alpha*sm**2)
!   skalieren auf 0 als asymptotik:
      if(dip.le.0) dip=0
      return
      end

!#######################################################################
!
! GS potential for O3 in Jacobi
!
!#######################################################################

      subroutine o3woy00(y1,y2,y3,v)
!     r,rr in a.u., gam in grad
      implicit none
      real*8 sm,sp,gam,r,rr

      real*8 y1,y2,y3,pi,angfac
      real*8 w1,w2,w3,sm0,sp0,gam0,v,mass

      pi = 4.0d0*atan(1.0d0)
      angfac = pi/180.0d0
      mass = 16.d0*1822.89d0

      if (y1.le.y2) then
         r=y2
         rr=y1
      else
         r=y1
         rr=y2
      endif
      gam=y3
      sm=0.5d0*(r-rr)
      sp=0.5d0*(r+rr)

!     print*,r,rr,sm,sp

      w1=1134.9/2.194746d+5
      w2=1089.2/2.194746d+5
      w3=716.0/2.194746d+5
      sm0 =0.0d0
      sp0 =2.46d0
      gam0 =116.8d0
      gam0 =gam0*angfac
      gam =gam*angfac
      w1 = mass*w1*w1
      w2 = mass*w2*w2
      w3 = mass*w3*w3
      v = 0.5d0*w1*(sp-sp0)**2
      v = v+ 0.5d0*w2*(sm-sm0)**2
      v = v+ 0.5d0*w3*(gam-gam0)**2

      return
      end
