!##########################################################################
!
! LVC 4-mode model of pyrazine from Woywood et al JCP(94) 100:1400
!
! ORDER: w10a, w6am, w1, w9a
!##########################################################################

      subroutine pyr4mod(q,w)

      implicit none

      real(8), dimension(4), intent(in)    :: q
      real(8), dimension(2,2), intent(out) :: w

      integer :: m

      real(8), parameter :: evau = 1.0d0/27.211399d0
      real(8), dimension(5), save :: omega,kappa1,kappa2

      real(8), save :: delta  = 0.46165
      real(8), save :: lambda = 0.1825 
! w10a, w6a, w1, w9a
      data omega /0.09357,0.0740,0.1273,0.1568,0/
! k6a1, k11, k9a1
      data kappa1 /0,-0.0964,0.0470,0.1594,0/ 
! k6a2, k12 , k9a2, 
      data kappa2 /0,0.1194,0.2012,0.0484,0/ 
      logical :: linitial = .true.

      if (linitial) then
         omega = omega*evau
         kappa1 = kappa1*evau
         kappa2 = kappa2*evau
         delta = delta*evau
         lambda = lambda*evau

         linitial = .false.
      endif
      w(:,:)=0.0d0

      w(1,1) = -delta
      w(2,2) = delta

      do m=1,4
         w(1,1) = w(1,1) + 0.5d0*omega(m)*q(m)*q(m)
         w(2,2) = w(2,2) + 0.5d0*omega(m)*q(m)*q(m)
      enddo
      do m=2,4
         w(1,1) = w(1,1) + kappa1(m)*q(m)
         w(2,2) = w(2,2) + kappa2(m)*q(m)
      enddo

      w(1,2) = lambda*q(1)
      w(2,1) = w(1,2)

      end subroutine pyr4mod

