!##########################################################################
! Subroutine used to test DD programs.
! Potential is the relevant parts of the vibronic coupling potential for
! salicylaldimine (modes 1,5,7,9,10,11,13,16,22,23,24,32,36).
! Uses potential from Polyak et al, JCP, 2015, (143), 084121.
! Potential corrected from that used in the paper as the quartic terms
! were multiplieded by 0.04116 rather than 0.04167 (1/24) due to a bug
! in VCHAM. Does not match correct factorials from Taylor series, again
! due to vagaries in VCHAM.
! GWR 08/16 and 01/18.
!##########################################################################
!      subroutine salicyl(gpoint,v,g,h,iorder)
      subroutine salicyl(gpoint,v)

      implicit none

!      integer(4), intent(in) :: iorder
      real(8), dimension(13), intent(in)     :: gpoint
      real(8), intent(out)                   :: v
!      real(8), dimension(13), intent(out)    :: g
!      real(8), dimension(13,13), intent(out) :: h
      real(8) :: saldpot13

!      if(iorder.lt.0 .or. iorder.gt.2)then
!         routine = 'salicyl'
!         message = 'Unrecognised order for salicyl potential'
!      endif

      v = saldpot13(gpoint)

!      call salicylpot(gpoint,v)

!      g = 0.0d0
!      if(iorder.gt.0)then
!         call salicylgrad(gpoint,g)
!      endif

!      h = 0.0d0
!      if(iorder.gt.1)then
!         call salicylhess(gpoint,h)
!      endif

      return
      end subroutine salicyl

!#############################################################################
!      subroutine salicylpot(gpoint,v)
!      implicit none
!      real(8), dimension(13), intent(in) :: gpoint
!      real(8), intent(out)               :: v
!      v = saldpot13(gpoint)
!      return
!      end subroutine salicylpot

!######################################################################################
      double precision function saldpot13(testq)

      implicit none

      real(8), dimension(13), intent(in) :: testq

      integer(4) :: i

      real(8), parameter :: evau = 1.0d0/27.211399d0
      real(8) :: ompart,kapart,gapart,iopart,eppart,etpart,eta_32_1,gaoffpart
      real(8), dimension(13) :: omega,kappa,gammad,gamma1,gamma10,gamma11,&
                                  gamma13,gamma32,iotad,epsd,eps1
      real(8), dimension(13,13) :: iotaoff

      data omega /0.18808,0.05254,0.06644,0.07815,0.07928,0.09815,0.10501,0.12166,&
                  0.15213,0.15954,0.16244,0.20607,0.26963 /

      data kappa /-0.010707,0.00008446,0.00019188,0.00029048,-0.00009656,&
                  -0.00008476,0.00009051,-0.000005,-0.00003752,-0.00027668,&
                   0.00006096,0.00008376,-0.0079318/

      data gammad /-0.38006,-0.00005211,-0.00001328,-0.000041680,-0.00003527,&
                   -0.00004026,-0.00002384,-0.00006204,0.000005,0.00018145, &
                    0.00034101,-0.00007011,-0.00728990/

      data gamma1 /0.0000,0.01020,-0.0034,0.0008,0.0009,-0.0113,0.0022,0.0058,&
                  -0.0088,0.01060,0.0094,0.0036, 0.0206/

      data gamma10 /5*0.0000,-0.0004,0.0004,4*0.0000,0.0003,-0.0012/

      data gamma11 /6*0.0000,0.0004,4*0.0000,0.0003,-0.0012/

      data gamma13 /11*0.0000,-0.0003,0.0012/

      data gamma32 /12*0.0000, 0.0009/

      data iotad / -0.0654,0.0012,-0.0006,0.0012,0.0018,0.0073,0.0031,&
                   -0.0005,-0.0030,0.0114,-0.0044,-0.0060,0.0000/

      data iotaoff / 0.0000,0.0018,-0.0002,0.0005,0.0003,0.0020,0.0016, &
                    -0.0005,0.0005,0.0025,0.0025,-0.0047,-0.0228, &
                    -0.0175,12*0.0000, &
                    -0.0109,12*0.0000, &
                    -0.0164,12*0.0000, &
                     0.0407,4*0.0000,0.0011,0.0011,4*0.0000,0.0006,-0.0006, &
                     0.0445,3*0.0000,0.0011, 0.0000,0.0011,4*0.0000,0.0006,-0.0006,&
                    -0.0450,3*0.0000,-0.0010,-0.0010,5*0.0000,-0.0006,0.0006, &
                    -0.0169,12*0.0000, &
                     0.0183,12*0.0000, &
                    -0.0050,12*0.0000, &
                     0.0074,12*0.0000, &
                     0.0215,3*0.0000, -0.0008, -0.0008, -0.0008, 5*0.0000, 0.0004, &
                    -0.0662,3*0.0000,0.0032,0.0032,0.0032,4*0.0000,0.0018,0.0000/

      data epsd / 0.48662,0.001183,0.00047001,0.00018046,0.000320840,0.00074117,0.0013485, &
                  0.00020035,0.0028139,0.006542,0.0089239,0.002778,0.000005/

      data eps1 / 0.0000,-0.02153664,0.00045576,3*0.0000,0.0446784,-0.00408216, &
                 -0.0110424,-0.0299568,-0.0307416,0.1723224,0.0240/

      eta_32_1 = -0.25764

      ompart = 0.0d0
      kapart = 0.0d0
      gapart = 0.0d0
      gaoffpart = 0.0d0
      iopart = 0.0d0
      eppart = 0.0d0
      etpart = 0.0d0

      do i = 1, 13
         ompart = ompart + omega(i)*testq(i)*testq(i)
      enddo
      ompart = 0.5d0*ompart

      do i = 1, 13
         kapart = kapart + kappa(i)*testq(i)
      enddo

      do i = 1, 13
         eppart = eppart + epsd(i)*testq(i)**4
  !       eppart = eppart + eps1(i)*testq(1)**2 * testq(i)**2
      enddo
      eppart = eppart/24.0d0

      do i = 1, 13
         gapart = gapart + gammad(i)*testq(i)*testq(i)
      enddo
      gapart = 0.5d0*gapart

  !    do i = 1, 13
  !       gaoffpart = gaoffpart + gamma1(i)*testq(1)*testq(i)
  !    enddo

  !    do i = 1, 13
  !       gaoffpart = gaoffpart + gamma10(i)*testq(5)*testq(i)
  !    enddo

  !    do i = 1, 13
  !       gaoffpart = gaoffpart + gamma11(i)*testq(6)*testq(i)
  !    enddo

  !    do i = 1, 13
  !       gaoffpart = gaoffpart + gamma13(i)*testq(7)*testq(i)
  !    enddo

  !    do i = 1, 13
  !       gaoffpart = gaoffpart + gamma32(i)*testq(12)*testq(i)
  !    enddo

  !    do i = 1, 13
  !       iopart = iopart + iotad(i)*testq(i)**3
  !    enddo
  !    iopart = iopart/6.0d0

  !    do i = 1, 13
  !       do j = 1, 13
  !          iopart = iopart + 0.5d0*iotaoff(i,j)*testq(i)*testq(i)*testq(j)
  !       enddo
  !    enddo

  !    etpart = eta_32_1 * testq(12)*testq(1)**3/24.0d0

      saldpot13 = evau*(ompart + kapart + gapart &
           + gaoffpart + iopart + eppart + etpart)

      end function saldpot13

