C##########################################################################
C
C  WDSE Surface for H_3O from Walch, Dunning, Schatz and Elgersma in
C  binding coordinates.
C  See JCP 72, 1303 (1980), JCP 73, 21 (1980) and JCP 100, 402 (1994).
C
C  SS 07.03
C##########################################################################

      subroutine wdse(roh1,roh2,roh3,rh1h2,rh1h3,rh2h3,vpot)

      implicit none

      real*8  roh1,roh2,roh3,rh1h2,rh1h3,rh2h3,vpot
      real*8  vm1,vm2,vm3,vleps,vh2o1,vh2o2,vv4
      real*8  de1,de2, de3,al1, al2, al3,re1, re2, re3,sato,
     +        g1, g2, g3,rohe, rhhe,c1, c2, c3, c4,c5, c6, c7,
     +        a1, a2,l11, l12,l21, l22,aoh1, aoh2, ahh1,ahh2 , r1
      real*8  morse,morse_c,leps,h2o,v4

      data  de1, de2, de3 / 0.14820, 0.02757, 0.15155 /
      data  al1, al2, al3 / 1.26058, 0.92418, 1.06862 /
      data  re1, re2, re3 / 1.86330, 2.90770, 1.42860 /
      data  sato / 0.1 /

      data  g1, g2, g3 / 2.3997, 1.05835, 2.3997 /
      data  rohe, rhhe / 1.80809, 2.86159 /
      data  c1, c2, c3, c4 / -0.0015920, 0.026963, 0.0014689, 0.080011 /
      data  c5, c6, c7 / 0.085816, -0.063179, 0.10138 /

      data  a1, a2 / 0.1, 0.009 /
      data  l11, l12 / 0.1, 0.1 /
      data  l21, l22 / 0.2, 0.03 /
      data  aoh1, aoh2 / 4.773, 7.14 /
      data  ahh1, ahh2 / 2.938, 5.28 /


      vm1 = morse(roh1,de1,al1,re1,0.d0)
      r1 = min(roh2,roh3)
      if (r1 .lt. 3.7d0) then
         vm2 = morse(rh1h2,de2,al2,re2,0.d0)
         vm3 = morse(rh1h3,de2,al2,re2,0.d0)
      else
C  -     Correction by Echave & Clary: JCP 100, 402 (1994).
         vm2 = morse_c(rh1h2,de2,al2,re2,0.d0,r1)
         vm3 = morse_c(rh1h3,de2,al2,re2,0.d0,r1)
      endif

      vleps = leps(roh2,roh3,rh2h3,de1,de1,de3,al1,al1,al3,re1,re1,re3,
     +             sato,sato,sato)

      vh2o1=h2o(roh1,roh2,rh1h2,g1,g2,g3,c1,c2,c3,c4,c5,c6,c7,rohe,rhhe)
      vh2o2=h2o(roh1,roh3,rh1h3,g1,g2,g3,c1,c2,c3,c4,c5,c6,c7,rohe,rhhe)

      vv4 = v4(roh2,roh3,rh1h2,rh1h3,a1,a2,l11,l12,l21,l22,aoh1,aoh2,
     +         ahh1,ahh2)
C  -    Correction by Clary
      if (r1 .ge. 3.7d0) then
         vv4 = exp(-6.d0*(r1-3.7d0))*vv4
      endif

C - Corrected asymptote: V(H2+OH)=0
      vpot = vm1+vm2+vm3 + vleps + vh2o1+vh2o2 + vv4 + 0.092647d0
C      vpot = vm1+vm2+vm3 + vleps + vh2o1+vh2o2 + vv4

      return
      end


C#######################################################################
C
C LEPS potential
C
C#######################################################################

      function leps(r1,r2,r3,d1,d2,d3,a1,a2,a3,re1,re2,re3,s1,s2,s3)

      implicit none

      real*8 r1,r2,r3,d1,d2,d3,a1,a2,a3,re1,re2,re3,s1,s2,s3
      real*8 leps,e1,e2,q1,q2,q3,j1,j2,j3

      e1=exp(-2.d0*a1*(r1-re1))
      e2=2.d0*exp(-a1*(r1-re1))
      q1=0.5d0*d1 * ((1.d0+s1)*(e1-e2) + (1.d0-s1)/2.d0*(e1+e2))
      j1=0.5d0*d1 * ((1.d0+s1)*(e1-e2) - (1.d0-s1)/2.d0*(e1+e2))

      e1=exp(-2.d0*a2*(r2-re2))
      e2=2.d0*exp(-a2*(r2-re2))
      q2=0.5d0*d2 * ((1.d0+s2)*(e1-e2) + (1.d0-s2)/2.d0*(e1+e2))
      j2=0.5d0*d2 * ((1.d0+s2)*(e1-e2) - (1.d0-s2)/2.d0*(e1+e2))

      e1=exp(-2.d0*a3*(r3-re3))
      e2=2.d0*exp(-a3*(r3-re3))
      q3=0.5d0*d3 * ((1.d0+s3)*(e1-e2) + (1.d0-s3)/2.d0*(e1+e2))
      j3=0.5d0*d3 * ((1.d0+s3)*(e1-e2) - (1.d0-s3)/2.d0*(e1+e2))

      leps = q1/(1+s1)+q2/(1+s2)+q3/(1+s3) -
     +       sqrt(0.5d0*( (j1/(1+s1)-j2/(1+s2))**2.d0 +
     +                    (j2/(1+s2)-j3/(1+s3))**2.d0 +
     +                    (j3/(1+s3)-j1/(1+s1))**2.d0 ))

      return
      end


C#######################################################################
C
C Sorbie-Murrell-like three-body potential for H2O.
C
C#######################################################################

      function h2o(roh1,roh2,rh1h2,g1,g2,g3,c1,c2,c3,c4,c5,c6,c7,
     +             rohe,rhhe)

      implicit none

      real*8 roh1,roh2,rh1h2,g1,g2,g3,c1,c2,c3,c4,c5,c6,c7,rohe,rhhe
      real*8 h2o,s1,s2,s3,p,qoh1,qoh2,qh1h2

      s1=roh1-rohe
      s2=roh2-rohe
      s3=rh1h2-rhhe

      p = c1 + c2*(s1+s3) + c3*s2 + 0.5d0*c4*(s1**2+s3**2) +
     +    0.5d0*c5*s2**2 + c6*s2*(s1+s3) + c7*s1*s3

      qoh1 = 1.d0-tanh(0.5d0*g1*s1)
      qoh2 = 1.d0-tanh(0.5d0*g2*s2)
      qh1h2 = 1.d0-tanh(0.5d0*g3*s3)

      h2o = qoh1*qoh2*qh1h2*p

      return
      end


C#######################################################################


      function v4(roh2,roh3,rh1h2,rh1h3,a1,a2,l11,l12,l21,l22,aoh1,
     +            aoh2,ahh1,ahh2)

      implicit none

      real*8 roh2,roh3,rh1h2,rh1h3,a1,a2,l11,l12,l21,l22,aoh1,aoh2,
     +       ahh1,ahh2
      real*8 v41,v42,v4


      v41 = a1*exp(-l11*(roh3-aoh1)**2 - l11*(roh2-aoh1)**2 -
     +              l21*(rh1h2-ahh1)**2 - l21*(rh1h3-ahh1)**2)
      v42 = a2*exp(-l12*(roh3-aoh2)**2 - l12*(roh2-aoh2)**2 -
     +              l22*(rh1h2-ahh2)**2 - l22*(rh1h3-ahh2)**2)

      v4 = v41+v42

      return
      end


C#######################################################################
C
C Modified Morse Potential
C
C  V = D ( 1 - g(r)*Exp(-a(x-x0)) )^2 - e0
C      where g(r)=exp(-6(r-3.7))
C
C#######################################################################

      function morse_c(x,depth,alpha,x0,e0,r1)

      implicit none

      real*8 morse_c,x,depth,alpha,x0,e0,r1

      morse_c=exp(-6.d0*(r1-3.7d0))*exp(-alpha*(x-x0))-1
      morse_c=morse_c*morse_c*depth
      morse_c=morse_c+e0

      return
      end

c-----------------------------------------------------------------------
c     compute the value of the morse potential
c-----------------------------------------------------------------------

      function morse(x,depth,alpha,x0,e0)

      implicit none

      real*8 morse,x,depth,alpha,x0,e0

      morse=1.0d0-exp(-alpha*(x-x0))
      morse=depth*morse*morse-e0

      return
      end

