!##########################################################################
!  WSLFH Surface for H_3O from Wu,Schatz,Lendvay,Fang,Harding.
!  See JCP 113, 3150 (2000)
!
!  Our changes: 1. Switching, for symmetry in phi (in wslfh)
!               2. Asymptotes (in wslfh1)
!               3. Zero point <=> V(H2+OH) (in wslfh1)
!
! DDOT function renamed to DDOT_WSLFH to prevent library clash GW 10/18
!##########################################################################

      subroutine wslfh(a,b,c,d,vpot)

      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      dimension a(3),b(3),c(3),d(3)

      common/fdata/fpp(14,18,21,3),f(14,18,21),fpppp(14,18,21,3),fxyz(14,18,21)


      vpot=0.d0
      vpot1=0.d0
      vpot2=0.d0
      vpot3=0.d0
      vpot4=0.d0
      vpot5=0.d0
      vpot6=0.d0
!
! If you are only interested in the properties of one reaction channel,
! comment the switching part of Hc and Hd,  and this will save much time.
!
! If Hd is the reactive atom.
      call coortrans(a,b,c,d,rab,rac,rad,rbc,rbd,rcd,cosdi)
      call wslfh1(rab,rac,rad,rbc,rbd,rcd,cosdi,vpot1)

! -- exchange c & d
!      dif2 = rac - rad
!      switch2 = 0.5d0*(1.d0-tanh(3.0d0*dif2))
!      call coortrans(a,b,d,c,rab,rad,rac,rbd,rbc,rcd,cosdi)
!      call wslfh1(rab,rad,rac,rbd,rbc,rcd,vpot2,cosdi)

!      vpot = vpot1*(1.d0-switch2) + vpot2*switch2

!      diff1=0.5d0*(rac+rcd)-rad

! --- Our changes ---
      diff1=min(rac,rcd)-rad
      sw1=0.5d0*(1.d0-tanh(3.0d0*diff1))
      diff2=rab-rbd
      sw2=0.5d0*(1.d0-tanh(3.0d0*diff2))

! -- exchange c & d
      call coortrans(a,b,d,c,rab,rad,rac,rbd,rbc,rcd,cosdi)
      call wslfh1(rab,rad,rac,rbd,rbc,rcd,cosdi,vpot2)
      diff3=rab-rbd              ! a<->c
      sw3=0.5d0*(1.d0-tanh(3.d0*diff3))
! -- exchange a & d
      call coortrans(d,b,c,a,rbd,rcd,rad,rbc,rab,rac,cosdi)
      call wslfh1(rbd,rcd,rad,rbc,rab,rac,cosdi,vpot3)
! -- exchange a & c
!      call coortrans(c,b,a,d,rbc,rac,rcd,rab,rbd,rad,cosdi)
!      call wslfh1           (rbc,rac,rcd,rab,rbd,rad,cosdi,vpot4)
! -- exchange a,d,c -> d,c,a
      call coortrans(d,b,a,c,rbd,rad,rcd,rab,rbc,rac,cosdi)
      call wslfh1(rbd,rad,rcd,rab,rbc,rac,cosdi,vpot5)
! -- exchange a,d,c -> c,a,d
!      call coortrans(c,b,d,a,rbc,rcd,rac,rbd,rab,rad,cosdi)
!      call wslfh1(rbc,rcd,rac,rbd,rab,rad,cosdi,vpot6)

      vpot = (1.d0-sw1)*(1.d0-sw2)*vpot1 + (1.d0-sw1)*sw2*vpot3 + &
             sw1*(1.d0-sw3)*vpot2 + sw1*sw3*vpot5

      return
      end

!############################################################################

       subroutine wslfhinput(opdatadef)

       implicit real*8 (a-h,o-z)
       implicit integer (i-n)

       character(len=160) opdata
       character(len=*), intent(in) :: opdatadef

       common/fdata/fpp(14,18,21,3),f(14,18,21),fpppp(14,18,21,3),fxyz(14,18,21)

! ---  The path for the data is $QUANTICS_DIR/source/surfaces/wslfh_data
       opdata=opdatadef
       ilbl=index(opdata,' ')-1
       open(50,file=opdata(1:ilbl)//"/wslfh_data",status="old",iostat=ierr1)
       if (ierr1.ne.0) then
         write(6,'(a,/a)') &
          'Cannot open the surface data file for wslfh:', &
           opdata(1:ilbl)//"/wslfh_data"
         stop
       else
          do i=1,14
             do j=1,18
                read (50,*) (f(i,j,k),k=1,21)
             enddo
          enddo
          do i=1,14
             do j=1,18
                read (50,*) (fxyz(i,j,k),k=1,21)
             enddo
          enddo
          do i=1,14
             do j=1,18
                do k=1,21
                   read (50,*) (fpp(i,j,k,l),l=1,3)
                enddo
             enddo
          enddo
          do i=1,14
             do j=1,18
                do k=1,21
                   read (50,*) (fpppp(i,j,k,l),l=1,3)
                enddo
             enddo
          enddo
       endif
       close(50)

       return
       end


!############################################################################

      subroutine wslfh1(rab,rac,rad,rbc,rbd,rcd,cosdi,vpot)
! For the potential of reactants or products, can set cosdi=1.0 ot -1.0,
!   and some r's be very large values.
       implicit double precision (a-h,o-z)
       implicit integer (i-n)
!      H2c
!       \
!        Ob---H1d-----H3a
! theta=O-H1-H3 , th=H2-O-H1
      arg=((rbd**2+rad**2-rab**2)/(2.0*rbd*rad))
      if(arg .lt. -1.0) arg=-1.0
      if(arg .gt.  1.0) arg= 1.0
      theta=acos(arg)*180./3.1415926536
! harding1 includes the switching of theta out of the 80~280(degree) range.
! here theta is always in 0~ 180 range by the defination of acos function
      cosd=abs(cosdi)
      call harding1(rbd,rad,theta,cosd,v3)
      call harding1(rbd,rad,theta,-cosd,v3p)
! including tortional part here
      cc1=(v3+v3p)/2.0
      cc2=(v3-v3p)/2.0
      v3=cc1+cc2*cosdi
! Morse function for OHc spectator.
      voh= 0.1628*(1.-exp(-1.22*(rbc-1.85)))**2
! add this part to fix the th0 problem
! switching th0 from 95(saddle point) to 105(H2O in equilibrium)
      th0=89.31+15.69*exp(-1.43*(rbd-1.81))
      costh=(rbc**2+rbd**2-rcd**2)/(2.0*rbc*rbd)
      x=costh-cos(th0/180.*3.1415926536)
! H2OH1 bending part,  switching the force constant
      vbend=0.5*0.185*exp(-0.82*(rbd-1.81))*x*x &
            +0.015*exp(-0.82*(rbd-1.81))*x*x*x
!       vbend=0.5*0.185*x*x+0.015*x*x*x
! parameters 0.185 & 0.015 make the bending frequency at the TS
! close to that of the ab initio values.
      vpot=v3+voh+vbend
! switching between the spline fit and the asymptotic double Morse
! for H2+OH product region.
      s=0.5*(1.-tanh(3.0*(rbd-4.0)))
      va=0.1698*(1.0-exp(-1.21*(rbc-1.85)))**2 &
        +0.1745*(1.0-exp(-1.03*(rad-1.40)))**2+0.0256571
!!      vpot=vpot*s+ va*(1.0-s)
! switching between the spline fit and the PJT2 potential
!  for the H2O+H reactant region.
      sr=0.5*(1.-tanh( 4.0*(rad-6.0)))
      call surfh2o(rbc,rcd,rbd,vpjt2)
!!      vpot=vpot*s+ vpjt2*(1.0-s)
!--- Our changes ---
      vpot=vpot*sr + vpjt2*(1.0-sr)
      vpot=vpot*s + va*(1.0-s) - 0.0256571d0

      return
      end


!############################################################################

      subroutine harding1(rbd,rad,thet,cosdi,v3)
      implicit double precision (a-h,o-z)
      implicit integer (i-n)
      theta=thet
      if(cosdi.lt.0) theta=360. - theta
! Here, switch the angle to the range of 80~280(spline fit)
      if(theta .lt. 80.0 .or. theta .gt. 280.0) then
        help=80.
        help2=280.
        call harding(rbd,rad,help,v1)
        call harding(rbd,rad,help2,v2)
        thet1=360.0-theta
!
        if (theta .lt. 80.0 ) then
          call harding(rbd,rad,theta,v5)
          if(v5.lt.v1) v5=v1
          call harding(rbd,rad,thet1,v4)
          if(v4.lt.v2) v4=v2
        else
! here theta>280, and thet1<80.
          call harding(rbd,rad,thet1,v5)
          if(v5.lt.v1) v5=v1
          call harding(rbd,rad,theta,v4)
          if(v4.lt.v2) v4=v2
        endif
        s1=0.5*(1.-tanh(theta/10.))
        s2=0.5*(1.-tanh(thet1/10.))
        if(theta .lt. 80.) v3=v4*s1 + v5*(1.-s1)
        if(theta .gt.280.) v3=v4*(1.-s2)+ v5*s2
      else
        call harding(rbd,rad,theta,v3)
      endif
      end


!############################################################################
! spline part of H+H2O surface
      subroutine harding(xi,yi,zi,fi)
!     sample main for calling h+h2o 3d spline
!
!      Hb
!       \
!        O---Ha---Hc
!
!
!
!     input:
!              roh =   r(O-Ha)        (au)          1.4 < roh   < 4.0
!              rhh =   r(Ha-Hc)       (au)          1.3 < rhh   < 8.0
!              theta = Hc-Ha-O angle  (degrees)     80  < theta < 280
!
!                Note: theta < 180 => cis     (Hb-O-Ha-Hc =   0.0)
!                      theta > 180 => trans   (Hb-O-Ha-Hc = 180.0)
!
!     frozen coordinates:
!
!               r(OHb)  =  1.85  au
!               Hb-O-Ha =  95.0 degrees
!
!     output:
!                energy  (atomic units relative to h2o+h)
!                deoh,dehh,detheta -  energy derivatives
!
!     note:  if you do not need the derivatives the code
!            can be speeded up by commenting out the
!            ~15 lines of code in spl3de that are used
!            to calculate the derivatives
!
!      SUBROUTINE spl3de(xi,yi,zi,FI,DFX,DFY,DFZ)

      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER (I-N)
!      dimension fpp(14,18,21,3),f(14,18,21),fpppp(14,18,21,3)
!      dimension fxyz(14,18,21)
      dimension delx(13),dely(17),delz(20),x(14),y(18),z(21)
      dimension C(4,4,4),px(4),py(4),pz(4)
      dimension dpx(4),dpy(4),dpz(4)

      common/fdata/fpp(14,18,21,3),f(14,18,21),fpppp(14,18,21,3),fxyz(14,18,21)

!      save/fdata/

      data x( 1), x( 2) /  1.40000000D+00 ,  1.70000000D+00 /
      data x( 3), x( 4) /  1.80000000D+00 ,  1.90000000D+00 /
      data x( 5), x( 6) /  2.00000000D+00 ,  2.10000000D+00 /
      data x( 7), x( 8) /  2.20000000D+00 ,  2.30000000D+00 /
      data x( 9), x(10) /  2.40000000D+00 ,  2.55000000D+00 /
      data x(11), x(12) /  2.80000000D+00 ,  3.00000000D+00 /
      data x(13), x(14) /  3.50000000D+00 ,  4.00000000D+00 /

      data y( 1), y( 2) /  1.30000000D+00 ,  1.40000000D+00 /
      data y( 3), y( 4) /  1.50000000D+00 ,  1.60000000D+00 /
      data y( 5), y( 6) /  1.70000000D+00 ,  1.80000000D+00 /
      data y( 7), y( 8) /  2.00000000D+00 ,  2.20000000D+00 /
      data y( 9), y(10) /  2.40000000D+00 ,  2.60000000D+00 /
      data y(11), y(12) /  3.00000000D+00 ,  3.50000000D+00 /
      data y(13), y(14) /  4.00000000D+00 ,  4.50000000D+00 /
      data y(15), y(16) /  5.00000000D+00 ,  6.00000000D+00 /
      data y(17), y(18) /  7.00000000D+00 ,  8.00000000D+00 /

      data z( 1), z( 2) /  8.00000000D+01 ,  9.00000000D+01 /
      data z( 3), z( 4) /  1.00000000D+02 ,  1.10000000D+02 /
      data z( 5), z( 6) /  1.20000000D+02 ,  1.30000000D+02 /
      data z( 7), z( 8) /  1.40000000D+02 ,  1.50000000D+02 /
      data z( 9), z(10) /  1.60000000D+02 ,  1.70000000D+02 /
      data z(11), z(12) /  1.80000000D+02 ,  1.90000000D+02 /
      data z(13), z(14) /  2.00000000D+02 ,  2.10000000D+02 /
      data z(15), z(16) /  2.20000000D+02 ,  2.30000000D+02 /
      data z(17), z(18) /  2.40000000D+02 ,  2.50000000D+02 /
      data z(19), z(20) /  2.60000000D+02 ,  2.70000000D+02 /
      data z(21) /         2.80000000D+02 /

      data delx( 1), delx( 2) /  3.00000000D-01 ,  1.00000000D-01 /
      data delx( 3), delx( 4) /  1.00000000D-01 ,  1.00000000D-01 /
      data delx( 5), delx( 6) /  1.00000000D-01 ,  1.00000000D-01 /
      data delx( 7), delx( 8) /  1.00000000D-01 ,  1.00000000D-01 /
      data delx( 9), delx(10) /  1.50000000D-01 ,  2.50000000D-01 /
      data delx(11), delx(12) /  2.00000000D-01 ,  5.00000000D-01 /
      data delx(13) /            5.00000000D-01 /
      data dely( 1), dely( 2) /  1.00000000D-01 ,  1.00000000D-01 /
      data dely( 3), dely( 4) /  1.00000000D-01 ,  1.00000000D-01 /
      data dely( 5), dely( 6) /  1.00000000D-01 ,  2.00000000D-01 /
      data dely( 7), dely( 8) /  2.00000000D-01 ,  2.00000000D-01 /
      data dely( 9), dely(10) /  2.00000000D-01 ,  4.00000000D-01 /
      data dely(11), dely(12) /  5.00000000D-01 ,  5.00000000D-01 /
      data dely(13), dely(14) /  5.00000000D-01 ,  5.00000000D-01 /
      data dely(15), dely(16) /  1.00000000D+00 ,  1.00000000D+00 /
      data dely(17) /            1.00000000D+00 /
      data delz( 1), delz( 2) /  1.00000000D+01 ,  1.00000000D+01 /
      data delz( 3), delz( 4) /  1.00000000D+01 ,  1.00000000D+01 /
      data delz( 5), delz( 6) /  1.00000000D+01 ,  1.00000000D+01 /
      data delz( 7), delz( 8) /  1.00000000D+01 ,  1.00000000D+01 /
      data delz( 9), delz(10) /  1.00000000D+01 ,  1.00000000D+01 /
      data delz(11), delz(12) /  1.00000000D+01 ,  1.00000000D+01 /
      data delz(13), delz(14) /  1.00000000D+01 ,  1.00000000D+01 /
      data delz(15), delz(16) /  1.00000000D+01 ,  1.00000000D+01 /
      data delz(17), delz(18) /  1.00000000D+01 ,  1.00000000D+01 /
      data delz(19), delz(20) /  1.00000000D+01 ,  1.00000000D+01 /
      data nptx,npty,nptz /  14 , 18 , 21 /


!
      IF(XI .LE. X(1)) THEN
        IX=1
      ELSE IF( XI .GE. X(nptx)) THEN
        IX=nptx-1
      ELSE
        call hunt(x,nptx,xi,ix)
      ENDIF
      xix=x(ix)
      xixp1=x(ix+1)
      delxi = delx(ix)
!
      IF(yi .LE. y(1)) THEN
        iy=1
      ELSE IF( yi .GE. y(npty)) THEN
        iy=npty-1
      ELSE
        call hunt(y,npty,yi,iy)
      ENDIF
      yiy=y(iy)
      yiyp1=y(iy+1)
      delyi = dely(iy)
!
      IF(zi .LE. z(1)) THEN
        iz=1
      ELSE IF( zi .GE. z(nptz)) THEN
        iz=nptz-1
      ELSE
        call hunt(z,nptz,zi,iz)
      ENDIF
      ziz=z(iz)
      zizp1=z(iz+1)
      delzi = delz(iz)


      C(1,1,1) = FXYZ(IX+1,IY+1,IZ+1)
      C(1,1,2) = FXYZ(IX,  IY+1,IZ+1)
      C(1,2,1) = FXYZ(IX+1,IY,  IZ+1)
      C(1,2,2) = FXYZ(IX,  IY,  IZ+1)
      C(1,3,1) = FPPPP(IX+1,IY+1,IZ+1,2)
      C(1,3,2) = FPPPP(IX,  IY+1,IZ+1,2)
      C(1,4,1) = FPPPP(IX+1,IY,  IZ+1,2)
      C(1,4,2) = FPPPP(IX,  IY,  IZ+1,2)
      C(1,1,3) = FPPPP(IX+1,IY+1,IZ+1,3)
      C(1,1,4) = FPPPP(IX,  IY+1,IZ+1,3)
      C(1,2,3) = FPPPP(IX+1,IY,  IZ+1,3)
      C(1,2,4) = FPPPP(IX,  IY,  IZ+1,3)
      C(1,3,3) = FPP(IX+1,IY+1,IZ+1,3)
      C(1,3,4) = FPP(IX,  IY+1,IZ+1,3)
      C(1,4,3) = FPP(IX+1,IY,  IZ+1,3)
      C(1,4,4) = FPP(IX,  IY,  IZ+1,3)
!
      C(2,1,1) = FXYZ(IX+1,IY+1,IZ)
      C(2,1,2) = FXYZ(IX,  IY+1,IZ)
      C(2,2,1) = FXYZ(IX+1,IY,  IZ)
      C(2,2,2) = FXYZ(IX,  IY,  IZ)
      C(2,3,1) = FPPPP(IX+1,IY+1,IZ, 2)
      C(2,3,2) = FPPPP(IX,  IY+1,IZ, 2)
      C(2,4,1) = FPPPP(IX+1,IY,  IZ, 2)
      C(2,4,2) = FPPPP(IX,  IY,  IZ, 2)
      C(2,1,3) = FPPPP(IX+1,IY+1,IZ, 3)
      C(2,1,4) = FPPPP(IX,  IY+1,IZ, 3)
      C(2,2,3) = FPPPP(IX+1,IY,  IZ, 3)
      C(2,2,4) = FPPPP(IX,  IY,  IZ, 3)
      C(2,3,3) = FPP(IX+1,IY+1,IZ, 3)
      C(2,3,4) = FPP(IX,  IY+1,IZ, 3)
      C(2,4,3) = FPP(IX+1,IY,  IZ, 3)
      C(2,4,4) = FPP(IX,  IY,  IZ, 3)
!
      C(3,1,1) = FPPPP(IX+1,IY+1,IZ+1,1)
      C(3,1,2) = FPPPP(IX  ,IY+1,IZ+1,1)
      C(3,2,1) = FPPPP(IX+1,IY  ,IZ+1,1)
      C(3,2,2) = FPPPP(IX,  IY,  IZ+1,1)
      C(3,3,1) = FPP(IX+1,IY+1,IZ+1,1)
      C(3,3,2) = FPP(IX,  IY+1,IZ+1,1)
      C(3,4,1) = FPP(IX+1,IY,  IZ+1,1)
      C(3,4,2) = FPP(IX,  IY,  IZ+1,1)
      C(3,1,3) = FPP(IX+1,IY+1,IZ+1,2)
      C(3,1,4) = FPP(IX,  IY+1,IZ+1,2)
      C(3,2,3) = FPP(IX+1,IY,  IZ+1,2)
      C(3,2,4) = FPP(IX,  IY,  IZ+1,2)
      C(3,3,3) = F(IX+1,IY+1,IZ+1)
      C(3,3,4) = F(IX,  IY+1,IZ+1)
      C(3,4,3) = F(IX+1,IY,  IZ+1)
      C(3,4,4) = F(IX,  IY,  IZ+1)
!
      C(4,1,1) = FPPPP(IX+1,IY+1,IZ,1)
      C(4,1,2) = FPPPP(IX  ,IY+1,IZ,1)
      C(4,2,1) = FPPPP(IX+1,IY  ,IZ,1)
      C(4,2,2) = FPPPP(IX,  IY,  IZ,1)
      C(4,3,1) = FPP(IX+1,IY+1,IZ,1)
      C(4,3,2) = FPP(IX,  IY+1,IZ,1)
      C(4,4,1) = FPP(IX+1,IY,  IZ,1)
      C(4,4,2) = FPP(IX,  IY,  IZ,1)
      C(4,1,3) = FPP(IX+1,IY+1,IZ,2)
      C(4,1,4) = FPP(IX,  IY+1,IZ,2)
      C(4,2,3) = FPP(IX+1,IY,  IZ,2)
      C(4,2,4) = FPP(IX,  IY,  IZ,2)
      C(4,3,3) = F(IX+1,IY+1,IZ)
      C(4,3,4) = F(IX,  IY+1,IZ)
      C(4,4,3) = F(IX+1,IY,  IZ)
      C(4,4,4) = F(IX,  IY,  IZ)
!
      PX(1)=((XI-XIX)**3/(6.0*DELXI))-(XI-XIX)*DELXI/6.0
      PX(2)=(XI-XIXP1)*DELXI/6.0-((XI-XIXP1)**3/(6.0*DELXI))
      PX(3)=(XI-XIX)/DELXI
      PX(4)=(XIXP1-XI)/DELXI
!
      PY(1)=((YI-YIY)**3/(6.0*DELYI))-(YI-YIY)*DELYI/6.0
      PY(2)=(YI-YIYP1)*DELYI/6.0-((YI-YIYP1)**3/(6.0*DELYI))
      PY(3)=(YI-YIY)/DELYI
      PY(4)=(YIYP1-YI)/DELYI
!
      PZ(1)=((ZI-ZIZ)**3/(6.0*DELZI))-(ZI-ZIZ)*DELZI/6.0
      PZ(2)=(ZI-ZIZP1)*DELZI/6.0-((ZI-ZIZP1)**3/(6.0*DELZI))
      PZ(3)=(ZI-ZIZ)/DELZI
      PZ(4)=(ZIZP1-ZI)/DELZI
!
      DPX(1)=((XI-XIX)**2/(2.0*DELXI))-(DELXI/6.0)
      DPX(2)=(DELXI/6.0)-((XI-XIXP1)**2/(2.0*DELXI))
      DPX(3)=1.0/DELXI
      DPX(4)=-1.0/DELXI
!
      DPY(1)=((YI-YIY)**2/(2.0*DELYI))-(DELYI/6.0)
      DPY(2)=(DELYI/6.0)-((YI-YIYP1)**2/(2.0*DELYI))
      DPY(3)=1.0/DELYI
      DPY(4)=-1.0/DELYI
!
      DPZ(1)=((ZI-ZIZ)**2/(2.0*DELZI))-(DELZI/6.0)
      DPZ(2)=(DELZI/6.0)-((ZI-ZIZP1)**2/(2.0*DELZI))
      DPZ(3)=1.0/DELZI
      DPZ(4)=-1.0/DELZI
!
      CALL SEVAL(C,PX,PY,PZ,FI)
      CALL SEVAL(C,DPX,PY,PZ,DFX)
      CALL SEVAL(C,PX,DPY,PZ,DFY)
      CALL SEVAL(C,PX,PY,DPZ,DFZ)
      fi = fi + 0.7675049483d00
!
      RETURN
      END


!############################################################################

      SUBROUTINE SEVAL(C,PX,PY,PZ,FI)
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER (I-N)
      DIMENSION C(4,4,4),PX(4),PY(4),PZ(4)
      DIMENSION TEMP1(4,4),TEMP2(4)
!
      DO I=1,4
         DO J=1,4
            TEMP1(I,J)=DDOT_WSLFH(4,C(1,I,J),1,PZ,1)
         enddo
      enddo
      DO I=1,4
         TEMP2(I)=DDOT_WSLFH(4,TEMP1(1,I),1,PY,1)
      enddo
      FI=DDOT_WSLFH(4,TEMP2,1,PX,1)
!
      RETURN
      END

      DOUBLE PRECISION FUNCTION DDOT_WSLFH(N,DX,INCX,DY,INCY)

!     FORMS THE DOT PRODUCT OF TWO VECTORS.
!     USES UNROLLED LOOPS FOR INCREMENTS EQUAL TO ONE.
!     JACK DONGARRA, LINPACK, 3/11/78.
!
      DOUBLE PRECISION DX(1),DY(1),DTEMP
      INTEGER I,INCX,INCY,IX,IY,M,MP1,N
!
      DDOT_WSLFH = 0.0D0
      DTEMP = 0.0D0
      IF(N.LE.0)RETURN
      IF(INCX.EQ.1.AND.INCY.EQ.1)GO TO 20
!
!     CODE FOR UNEQUAL INCREMENTS OR EQUAL INCREMENTS
!       NOT EQUAL TO 1
!
      IX = 1
      IY = 1
      IF(INCX.LT.0)IX = (-N+1)*INCX + 1
      IF(INCY.LT.0)IY = (-N+1)*INCY + 1
      DO I = 1,N
        DTEMP = DTEMP + DX(IX)*DY(IY)
        IX = IX + INCX
        IY = IY + INCY
      enddo
      DDOT_WSLFH = DTEMP
      RETURN
!
!      CODE FOR BOTH INCREMENTS EQUAL TO 1
!
!      CLEAN-UP LOOP
!
   20 M = MOD(N,5)
      IF( M .EQ. 0 ) GO TO 40
      DO I = 1,M
        DTEMP = DTEMP + DX(I)*DY(I)
      enddo
      IF( N .LT. 5 ) GO TO 60
   40 MP1 = M + 1
      DO I = MP1,N,5
        DTEMP = DTEMP + DX(I)*DY(I) + DX(I + 1)*DY(I + 1) +    &
          DX(I + 2)*DY(I + 2) + DX(I + 3)*DY(I + 3) + DX(I + 4)*DY(I + 4)
      enddo
   60 DDOT_WSLFH = DTEMP
      RETURN
      END

!############################################################################

      subroutine hunt(xx,n,x,jlo)
      implicit real*8 (a-h,o-z)
      implicit integer (i-n)
      dimension xx(n)
      logical ascnd


      ascnd = xx(n) .gt. xx(1)
      if( jlo .le. 0  .or. jlo .gt. n) then
         jlo = 0
         jhi=n+1
         go to 3
      endif
      inc=1


      if( x.ge. xx(jlo) .eqv. ascnd) then
 1       jhi = jlo+inc
         if(jhi .gt. n) then
           jhi=n+1
         else if( x .ge. xx(jhi) .eqv. ascnd) then
           jlo=jhi
           inc=inc+inc
           go to 1
         endif
      else
         jhi=jlo
 2       jlo=jhi-inc
         if(jlo .lt.1) then
            jlo=0
         else if( x .lt. xx(jlo) .eqv. ascnd) then
            jhi=jlo
            inc=inc+inc
            go to 2
         endif
      endif


 3    if(jhi -jlo .eq.1) return
      jm=(jhi+jlo)/2
      if( x .gt. xx(jm) .eqv. ascnd) then
         jlo = jm
      else
         jhi = jm
      endif
      go to 3
      end


!############################################################################
