!#######################################################################
!  common data for Surfaces YZCL1,YZCL2
!#######################################################################
      module yzclmod

      integer, parameter :: long = selected_int_kind(9)  ! >= 9-digit integers
      integer, parameter :: dop = selected_real_kind(15,307)

      integer(long), parameter :: natom=4,nbond=natom*(natom-1)/2,ndatam=2000,&
                                  ngroup=6,maxfo=1,maxf=5000,n3=3*natom,&
                                  nintm=n3-6

      character(len=20), save                     :: msunit,enunit,lnunit,tmunit

      character(len=80), save                     :: title,comlin

      real(dop), save                             :: amsc,esc,rsc,tsc,tups,&
                                                     scalef,eukjpm,ccmps,ccmptu, &
                                                     hjs,hkjpms,heutu

      real(dop), save                             :: delta,todeg,toang

      integer(long), save                         :: nloop,nstep,nprint,&
                                                     nmol,ir,nstop

      real(dop), save                             :: potmax,wtol,outer,&
                                                     totsum,avrads,totsave
      real(dop), dimension(ndatam), save          :: v0,rads
      real(dop), dimension(nbond), save           :: rsave
      real(dop), dimension(maxf), save            :: storevt
      real(dop), dimension(maxf,6), save          :: rdat,zd
      real(dop), dimension(maxf,6,6), save        :: uk
      real(dop), dimension(ndatam,nintm), save    :: v1,v2,zda
      real(dop), dimension(ndatam,nbond), save    :: rda,sumb
      real(dop),dimension(ndatam,nintm,nbond),save:: ut

      integer(long), dimension(nbond), save       :: mb,nb

      integer(long), save                         :: ndata,nforc,lowp,nneigh,&
                                                     ipart,ipow,neighci, &
                                                     neighco,maxnb,nlowstop,&
                                                     nforco,neighc
      integer(long), dimension(maxf), save        :: mda,nda,ndao,mdao

      real(dop), save                             :: sample,rms,grms,totsm

      integer(long), dimension(nbond,ngroup), save:: ip

      real(dop), save                             :: vmin,preact
      integer(long), save                         :: nsel,nadd

      integer(long), save                         :: nfraga,nfragb,nbfraga,nbfragb
      integer(long), dimension(natom), save       :: ifraga,ifragb
      integer(long), dimension(nbond), save       :: ibfraga,ibfragb


      integer(long), save                         :: nfrag
      integer(long), dimension(natom), save       :: ifrag,kf,kb,ichansave
      integer(long), dimension(natom,natom), save :: map
      integer(long), dimension(natom,nbond), save :: ibfrag

      real(dop), save                             :: bipmax, distab
      real(dop), dimension(natom,natom), save     :: rmin

      integer(long), save                         :: nset

!######################################################################
!  Variables for WSLFH Surface
!######################################################################

      real(dop), dimension(14,18,21), save   :: f,fxyz
      real(dop), dimension(14,18,21,3), save :: fpp,fpppp

      end module yzclmod

!##########################################################################
!
!  YZCL1,YZCL2 Surfaces for H_3O System by Yang,Zhang,Collins,Lee.
!  See JCP 115, 174 (2001).
!
!##########################################################################

      subroutine yzcl(a,b,c,d,energy)

      use  yzclmod

      integer n,m,k,i,l
      real*8 a(3),b(3),c(3),d(3),cosdi
      real*8 vt(ndatam,ngroup),rinv(nbond),ei(maxf),wt(maxf), &
             z(maxf,nintm),r12,r13,r14,r23,r24,r34,energy
      real*8 t1,t2,t3,t4,t5,t6,wt1

!    do a coordinate transformation

      call coortrans(a,b,c,d,r12,r23,r24,r13,r14,r34,cosdi)

!    assume a two part weight function

      totsum=0.d0

      rinv(1)=1.d0/r12
      rinv(2)=1.d0/r13
      rinv(3)=1.d0/r14
      rinv(4)=1.d0/r23
      rinv(5)=1.d0/r24
      rinv(6)=1.d0/r34

      do m=1,ndata


      n=1
            t1=rinv(1)-rda(m,1)
            t2=rinv(2)-rda(m,2)
            t3=rinv(3)-rda(m,3)
            t4=rinv(4)-rda(m,4)
            t5=rinv(5)-rda(m,5)
            t6=rinv(6)-rda(m,6)

         vt(m,n)=t1*t1/sumb(m,1) &
                +t2*t2/sumb(m,2) &
                +t3*t3/sumb(m,3) &
                +t4*t4/sumb(m,4) &
                +t5*t5/sumb(m,5) &
                +t6*t6/sumb(m,6)

         vt(m,n)=vt(m,n)*vt(m,n)
         vt(m,n)=vt(m,n)+vt(m,n)**6
         vt(m,n)=1.d0/vt(m,n)


         totsum=totsum+vt(m,n)

      n=2
            t1=rinv(1)-rda(m,2)
            t2=rinv(2)-rda(m,3)
            t3=rinv(3)-rda(m,1)
            t4=rinv(4)-rda(m,6)
            t5=rinv(5)-rda(m,4)
            t6=rinv(6)-rda(m,5)

         vt(m,n)=t1*t1/sumb(m,2) &
                +t2*t2/sumb(m,3) &
                +t3*t3/sumb(m,1) &
                +t4*t4/sumb(m,6) &
                +t5*t5/sumb(m,4) &
                +t6*t6/sumb(m,5)



         vt(m,n)=vt(m,n)*vt(m,n)
         vt(m,n)=vt(m,n)+vt(m,n)**6
         vt(m,n)=1.d0/vt(m,n)



         totsum=totsum+vt(m,n)


      n=3
            t1=rinv(1)-rda(m,3)
            t2=rinv(2)-rda(m,1)
            t3=rinv(3)-rda(m,2)
            t4=rinv(4)-rda(m,5)
            t5=rinv(5)-rda(m,6)
            t6=rinv(6)-rda(m,4)

         vt(m,n)=t1*t1/sumb(m,3) &
                +t2*t2/sumb(m,1) &
                +t3*t3/sumb(m,2) &
                +t4*t4/sumb(m,5) &
                +t5*t5/sumb(m,6) &
                +t6*t6/sumb(m,4)


         vt(m,n)=vt(m,n)*vt(m,n)
         vt(m,n)=vt(m,n)+vt(m,n)**6
         vt(m,n)=1.d0/vt(m,n)



         totsum=totsum+vt(m,n)


      n=4
            t1=rinv(1)-rda(m,2)
            t2=rinv(2)-rda(m,1)
            t3=rinv(3)-rda(m,3)
            t4=rinv(4)-rda(m,4)
            t5=rinv(5)-rda(m,6)
            t6=rinv(6)-rda(m,5)

         vt(m,n)=t1*t1/sumb(m,2) &
                +t2*t2/sumb(m,1) &
                +t3*t3/sumb(m,3) &
                +t4*t4/sumb(m,4) &
                +t5*t5/sumb(m,6) &
                +t6*t6/sumb(m,5)


         vt(m,n)=vt(m,n)*vt(m,n)
         vt(m,n)=vt(m,n)+vt(m,n)**6
         vt(m,n)=1.d0/vt(m,n)



         totsum=totsum+vt(m,n)


      n=5
            t1=rinv(1)-rda(m,3)
            t2=rinv(2)-rda(m,2)
            t3=rinv(3)-rda(m,1)
            t4=rinv(4)-rda(m,6)
            t5=rinv(5)-rda(m,5)
            t6=rinv(6)-rda(m,4)

         vt(m,n)=t1*t1/sumb(m,3) &
                +t2*t2/sumb(m,2) &
                +t3*t3/sumb(m,1) &
                +t4*t4/sumb(m,6) &
                +t5*t5/sumb(m,5) &
                +t6*t6/sumb(m,4)


         vt(m,n)=vt(m,n)*vt(m,n)
         vt(m,n)=vt(m,n)+vt(m,n)**6
         vt(m,n)=1.d0/vt(m,n)



         totsum=totsum+vt(m,n)


      n=6
            t1=rinv(1)-rda(m,1)
            t2=rinv(2)-rda(m,3)
            t3=rinv(3)-rda(m,2)
            t4=rinv(4)-rda(m,5)
            t5=rinv(5)-rda(m,4)
            t6=rinv(6)-rda(m,6)

         vt(m,n)=t1*t1/sumb(m,1) &
                +t2*t2/sumb(m,3) &
                +t3*t3/sumb(m,2) &
                +t4*t4/sumb(m,5) &
                +t5*t5/sumb(m,4) &
                +t6*t6/sumb(m,6)


         vt(m,n)=vt(m,n)*vt(m,n)
         vt(m,n)=vt(m,n)+vt(m,n)**6
         vt(m,n)=1.d0/vt(m,n)


         totsum=totsum+vt(m,n)


!  end the loop over m=1,ndata

      enddo

!  we set up the massive neighbour list
!  wtol is passed in common/shepdata
!  nforc is the number of neighbours
!  mda identifies the data point in POT that neighbour k comes from
!  nda identifies which permutation neighbour k is

      nforc=0
      energy=0.d0

      do n=1,ngroup
      do m=1,ndata

         wt1=vt(m,n)/totsum


         if (wt1.gt.wtol) then

             nforc=nforc+1
             mda(nforc)=m
             nda(nforc)=n
             do i=1,nbond
             rdat(nforc,i)=rda(m,ip(i,n))
             do k=1,nintm
             uk(nforc,k,i)=ut(m,k,ip(i,n))
             enddo
             enddo
             do k=1,nintm
             zd(nforc,k)=zda(m,k)
             enddo
             storevt(nforc)=vt(m,n)

         endif

      enddo
      enddo

      if (nforc.gt.maxf) then
         write(10,*)' *** ABORT ***'
         write(10,*)' nforc.gt.maxf'
         stop
      endif

!     write(6,*)j, totsum

!  zero those who need it

      energy=0.d0
      totsum=0.d0


!  we will need to have some quantities for the gradients
!  of the weights
!  Note an approx here, due to neglect of small weights

      do k=1,nforc
         totsum=totsum+storevt(k)
      enddo

!  calculate the relative weights

      do k=1,nforc
         wt(k)=storevt(k)/totsum
      enddo

!  construct the normal local coords for each neighbour
!  recalling that intern calculates bond lengths NOT inverses

      do k=1,nforc
      do i=1,nintm
         z(k,i)=0.d0
         do l=1,nbond
            z(k,i) = ut(mda(k),i,ip(l,nda(k)))*rinv(l) + z(k,i)
         enddo
          z(k,i)=(z(k,i)-zda(mda(k),i))
      enddo
      enddo


!  calculate the potential energy: scalar term


!  linear term in energy plus diagonal part of quadratic term
!  first term in derivative of taylor poly wrt local coords

      do k=1,nforc
         ei(k)=v0(mda(k))
      do i=1,nintm

         ei(k) = z(k,i)*v1(mda(k),i) + 0.5d0*z(k,i)**2*v2(mda(k),i) + ei(k)

      enddo
      enddo

      do k=1,nforc
         energy=ei(k)*wt(k)+energy
      enddo

      return
      end subroutine yzcl



!#########################################################################

      subroutine yzclinput(isurf,opdata)

      use yzclmod

!  this program runs a potential energy sufrace interpolation scheme
!  to evaluate the energy at one or many geometries supplied

      integer :: i,j,k
      integer, intent(in) :: isurf
      character(len=*), intent(in) :: opdata

!------------------------------------------------------------------------
!   set up the bond definitions

        k=1
        do i=1,natom-1
        do j=i+1,natom
           mb(k)=i
           nb(k)=j
           k=k+1
        enddo
        enddo
!-------------------------------------------------------------------------
!  include permutational symmetry

      do k=1,nbond
         ip(k,1)=k
      enddo
! --234
      ip(1,2)=2
      ip(2,2)=3
      ip(3,2)=1
      ip(4,2)=6
      ip(5,2)=4
      ip(6,2)=5
! --243
      ip(1,3)=3
      ip(2,3)=1
      ip(3,3)=2
      ip(4,3)=5
      ip(5,3)=6
      ip(6,3)=4
! --23
      ip(1,4)=2
      ip(2,4)=1
      ip(3,4)=3
      ip(4,4)=4
      ip(5,4)=6
      ip(6,4)=5
! --24
      ip(1,5)=3
      ip(2,5)=2
      ip(3,5)=1
      ip(4,5)=6
      ip(5,5)=5
      ip(6,5)=4
! --34
      ip(1,6)=1
      ip(2,6)=3
      ip(3,6)=2
      ip(4,6)=5
      ip(5,6)=4
      ip(6,6)=6

!----------------------------------------------------------------------
!   set run control parameters - to get potmax and ipart, lowp, ipow

      lowp=2
      ipow=12
      ipart=2
      nneigh=36
      wtol=1.d-5
      potmax=0.6d0

!-------------------------------------------------------------------------
!   read in the initial table of Taylor series potential coefficients

      call readt(isurf,opdata)

!------------------------------------------------------------------------

      call radius

!------------------------------------------------------------------------

      call confidrad


      return
      end subroutine yzclinput



!##########################################################################
      subroutine confidrad

      use yzclmod

      integer i,j,k,l,ndt2
      real*8 z(nintm),dei(nintm),dtdrm(nbond), &
             delrv(nbond),dtdr(nbond),asb(nbond), &
             bigsort(ndatam),sigma,sigma2,vt,t,rcut,taydif,sum,rcon, &
             ard

      sigma=0.0002d0
      sigma2=sigma*sigma

!     open(unit=3,file='!ON',status='unknown')

      do j=1,ndata


      rcut=rads(j)+1.d-12

       sum=0.d0
       do i=1,nbond
       sumb(j,i)=0.d0
       enddo

      do m=1,ndata
      if(m.eq.j)go to 10

      do n=1,ngroup

       vt=0.d0

       do i=1,nbond
        t=rda(m,ip(i,n))-rda(j,i)
       vt=t*t+vt
       enddo
!  rda is a reciprocal bondlength

!  only consider the first nneigh neighbours



      if(vt.gt.rcut)go to 222


! the Taylor expansion

      do i=1,nintm
         z(i)=0.d0
         do l=1,nbond
            z(i) = ut(j,i,ip(l,1))*rda(m,ip(l,n)) + z(i)
         enddo
          z(i)=(z(i)-zda(j,i))
      enddo

      taydif=v0(m)-v0(j)
      do i=1,nintm
       taydif=taydif-z(i)*(v1(j,i)+0.5d0*v2(j,i)*z(i))
       dei(i) = z(i)*v2(j,i) + v1(j,i)
      enddo

      sum=sum+taydif**2/vt**3


!  transform derivatives of taylor poly wrt normal local coords into
!  derivs wrt inverse bondlengths

      do i=1,nbond
            dtdr(i)=0.d0
            dtdrm(i)=0.d0
            do l=1,nintm

               dtdr(i)=dei(l)*ut(j,l,ip(i,1)) + dtdr(i)
               dtdrm(i)=v1(m,l)*ut(m,l,ip(i,n)) + dtdrm(i)

            enddo
      enddo

      do i=1,nbond
      delrv(i)=(rda(m,ip(i,n))-rda(j,i))*(dtdrm(i)-dtdr(i))
      sumb(j,i)=sumb(j,i)+delrv(i)**2/vt**3
      enddo


! end the ngroup loop
222   continue
      enddo

10    continue

! end the m ndata loop
      enddo

!     do i=1,nbond
!     if(sumb(j,i).lt.1.d-15)then
!     write(6,*)j,i,sumb(j,i)
!     stop
!     endif
!     enddo

      rcon=(nneigh*sigma2/sum)**(1.d0/3.d0)

      do i=1,nbond
      sumb(j,i)=(nneigh*sigma2/sumb(j,i))**(1.d0/3.d0)
      enddo

!  test to see if some sumb have been wiildly extrapolated


!  replace rads by rcon
      if(ipart.eq.2)then
      rads(j)=rcon
      else
      rads(j)=1.d0
      endif

! end the j ndata loop
      enddo

!      stop

!  try to eliminate abnormally large radii and sumb

!  first evaluate averages

      ard=0.d0
      do m=1,ndata
      ard=ard+rads(m)
      enddo
      ard=ard/ndata

      do n=1,nbond
      asb(n)=0.d0
      do m=1,ndata
      asb(n)=asb(n)+sumb(m,n)
      enddo
      asb(n)=asb(n)/ndata
      enddo

!     do m=1,ndata
!     if(rads(m).gt.1.5d0*ard)rads(m)=1.5d0*ard
!     do n=1,nbond
!     if(sumb(m,n).gt.1.5d0*asb(n))sumb(m,n)=1.5d0*asb(n)
!     enddo
!     enddo

!      do n=1,nbond
!      write(6,*) n,asb(n)
!      enddo

      ndt2=ndata/2

      do n=1,nbond

      do k=1,ndata
       bigsort(k)=sumb(k,n)
      enddo
      call sort(ndata,bigsort)
      do k=1,ndata
       if(sumb(k,n).gt.2.5d0*bigsort(ndt2))sumb(k,n)=2.5d0*bigsort(ndt2)
      enddo

      enddo

      return
      end subroutine confidrad



!##########################################################################

      subroutine radius

      use yzclmod

!  this subroutine looks at the data set and finds the "distance"
!  of the closest data point to each data point. The radius of
!  "confidence" for each data point is set to ntimes times this
!  closest distance

      integer i,k,m,ic,ndist
      real*8 dists(ndatam*ngroup),smdis(ndatam*ngroup),dmin,t,t2,sum,dmin2


!  nneigh is now the number of neighbours in the POT file
!  who are to lie within rads (this is how rads is determined).

!  check that nneigh is not too big

      if (nneigh.gt.(ndatam*ngroup)) then
!        write(11,*)'  nneigh has been set too large for the data set'
         stop
      endif

      do k=1,ndata

         dmin=1.d+06
         totsum=0.d0
         ic=0

         do m=1,ndata
            if (m.eq.k) go to 1
            do n=1,ngroup

               sum=0.d0

               do i=1,nbond
                  t=rda(k,i)-rda(m,ip(i,n))
                  t2=t*t
                  sum=t2+sum
               enddo

      if(sum.lt.1.d-10)then
      write(6,*)' identical data points'
      write(6,*)k,m
      stop
      endif
               ic=ic+1
               dists(ic)=sum

!              totsum=totsum+1.d0/sum**lowp

               if (sum.lt.dmin) then
                  dmin=sum
               endif

            enddo
1           continue
         enddo

! guard against identical data points

         if (dmin.lt.1.d-10) dmin=1.d-4

         dmin2=2.d0*dmin
50       continue

! find all the distances less than 2*dmin

         ndist=0

         do i=1,ic
            if (dists(i).le.dmin2) then
               ndist=ndist+1
               smdis(ndist)=dists(i)
            endif
         enddo

         if (ndist.lt.(nneigh+1)) then
            dmin2=dmin2*2.d0
!           write(11,500) k,ndist
            go to 50
         endif

!500   format(2x,'increase dmin2 for k = ',i5,2x,i10)

!  we now have a set of at least nneigh small distances
!  we sort them

         call sort(ndist,smdis)

         rads(k)=smdis(nneigh)

      enddo

      avrads=0.d0
      do k=1,ndata
         avrads=avrads+rads(k)
      enddo
      avrads=avrads/float(ndata)

      return

      end subroutine radius



!##########################################################################

      subroutine readt(isurf,opdata)

      use yzclmod

      integer i,j,k,ierr,ilbl
      integer, intent(in) :: isurf
      character(len=*), intent(in) :: opdata
      character(len=240)  :: datafile

      ilbl=index(opdata,' ')-1
      write(datafile,'(a,a5,i1,a5)') opdata(1:ilbl),'/yzcl',isurf,'_data'
      ilbl=index(datafile,' ')-1

! -- The path for the data is $MCTDH_DIR/surfaces/yzcl1_data
! -- or $MCTDH_DIR/surfaces/yzcl2_data (isurf = 1 or 2).

      open(50,file=datafile(1:ilbl),status="old",iostat=ierr)

      if (ierr .ne. 0) then
         write(6,'(a,/a)') &
         'Cannot open the surface data file for yzcl:', datafile(1:ilbl)
         stop
      endif

!     call lisym(11,'-')
!     write(11,*) ' '
!     write(11,*) ' *** subroutine readt:',
!    .  ' read in potential energy surface parameters ***'
!     write(11,*) ' '

!   read comment/header

      read(50,80)comlin
   80 format(a80)

!  read in the bond lengths, the matrix defining the normal local coords,
!  the potential energy, the gradients and diagonal force constants (wrt to
!  the normal local coords) at all of the Ndata points.

      j=1
10    continue
         read(50,80,end=20)comlin

         read(50,*) (rda(j,k),k=1,nbond)

         do i=1,nintm
            read(50,*) (ut(j,i,k),k=1,nbond)
         enddo

         read(50,*) v0(j)

!  find the minimum potential in the original data set

         if (j.eq.1) vmin=v0(j)
         if (v0(j).lt.vmin) vmin=v0(j)

         read(50,*) (v1(j,k),k=1,nintm)

         read(50,*) (v2(j,k),k=1,nintm)

!        if (v0(j).gt.potmax) goto 10

         j=j+1
         go to 10

20    continue

      ndata=j-1
!      write(6,*)' Number of PES data points = ',ndata
!     write(11,*)' lowest potential energy   = ',vmin

      close (unit=50)

!  construct the local coords for each data point
!  remembering that rda are read in as bond lengths NOT inverses

      do n=1,ndata
      do j=1,nintm
         zda(n,j)=0.d0
         do k=1,nbond
            zda(n,j)=ut(n,j,k)/rda(n,k) + zda(n,j)
         enddo
      enddo
      enddo


!  invert data bonds once and for all

      do n=1,ndata
      do i=1,nbond
         rda(n,i)=1.d0/rda(n,i)
      enddo
      enddo


! debug

!     do n=1,ndata
!        write(6,*) ' '
!        write(6,1812) (zda(n,i),i=1,6)
!        write(6,1812) (zda(n,i),i=7,12)
!     enddo


      return
      end subroutine readt

!########################################################################

      subroutine sort(N,RA)
      implicit real*8(a-h,o-z)
      implicit integer(i-n)
      DIMENSION RA(N)
      L=N/2+1
      IR=N
10    CONTINUE
        IF(L.GT.1)THEN
          L=L-1
          RRA=RA(L)
        ELSE
          RRA=RA(IR)
          RA(IR)=RA(1)
          IR=IR-1
          IF(IR.EQ.1)THEN
            RA(1)=RRA
            RETURN
          ENDIF
        ENDIF
        I=L
        J=L+L
20      IF(J.LE.IR)THEN
          IF(J.LT.IR)THEN
            IF(RA(J).LT.RA(J+1))J=J+1
          ENDIF
          IF(RRA.LT.RA(J))THEN
            RA(I)=RA(J)
            I=J
            J=J+J
          ELSE
            J=IR+1
          ENDIF
        GO TO 20
        ENDIF
        RA(I)=RRA
      GO TO 10
      END subroutine sort

