

## Script to create quantics input file from Gaussian output file 
## from a frequency calculation 

import sys
import os

if (len(sys.argv) == 1) or (sys.argv[1] == "-h"):
    print "Script to create quantics input file"
    print "Usage: python make.quantics.input.py [gaussian output file] [coordinates] [project name]"
    print "       gaussian output file: high precision frequency calculation"
    print "         modes to be used for the dyanmics as transfile"
    print "       coordinates: cartesian or nmodes, optional - default is nmodes"
    print "       project name: name of your project, optional"
    quit()

gaussfile = sys.argv[1]
coordtype = "nmodes"
project = "#ADD HERE"
filename = "quantics.inp"

if len(sys.argv) == 3:
    if sys.argv[2] in ["cartesian", "nmodes"]:
        coordtype = sys.argv[2]
    else:
        project = sys.argv[2]
        filename = project + ".inp"
elif len(sys.argv) == 4:
    coordtype = sys.argv[2]
    project = sys.argv[3]
    filename = project + ".inp"


if coordtype == "cartesian":
    cartesian = True
else:
    cartesian = False

infile = open(gaussfile, 'r')
content = infile.readlines()
infile.close()

geom_line = 0

elements_dictionary = {1:"H", 6:"C", 7:"N", 8:"O", 9:"F", 15:"P", 16:"S", 17:"Cl", 35:"Br", 53:"I"}

freq_lines = []

for i, line in enumerate(content):
    if line.find('Frequencies ---') != -1:
        freq_lines.append(i)
    elif line.find('Standard orientation') != -1:
        geom_line = i
    elif line.find('Input orientation') != -1:
        inputgeom_line = i
    elif line[0:10].find('NAtoms=') != -1:
        natoms = int(line.split()[1])
    elif line[0:23].find('basis functions') != -1:
        basis = int(line.split()[0])

if geom_line == 0:
    geom_line = inputgeom_line
        

outstring = """
####################################         
###       DD / Propagation      ####
####################################

RUN-SECTION
name = %s 
propagation   direct = %s   ngwp = 15
title = %s  
tfinal = 200.0  tout = 1.0   tpsi= 1.0
psi  gridpop  update  steps  auto
openmp = 4
use_dd_sym
end-run-section

# Add, if needed:

#HAMILTONIAN-SECTION_system
#end-hamiltonian-section

#LABELS-SECTION
#end-labels-section

INITIAL-GEOMETRY-SECTION
nstates = 3    init_state = 2   # change accordingly
cartesian = angst
""" % (project, coordtype, project)

geom_part = content[geom_line + 5 : geom_line + 5 + natoms]

for i, line in enumerate(geom_part):
    tmp = line.split()
    element = elements_dictionary[int(tmp[1])]
    outstring += """   %s    %9.6f   %9.6f   %9.6f\n""" %(element, float(tmp[3]), float(tmp[4]), float(tmp[5]))

outstring += "end-cartesian\n"



if not cartesian:

    outstring += "nmodes\n"
 
    count = 0 
    for i in freq_lines:
        tmp = content[i].split()
        for j in range(2,len(tmp)):
            count += 1
            outstring += """%2iA   0.00000   %12.4f, cm-1 width = 0.707 \n""" %(count, float(tmp[j]))

    outstring += "end-nmode\n"


outstring += """end-initial-geometry-section

DIRDYN-SECTION
data =  %s_dddata
transfile =  %s
update = always
qcprogram =      method = 
ener0 = 
# substitute db = only for next 2 lines to generate DB
#ascii_db
ddlog_full
dd_diab = global
dbpoint = 2
db = rdwr    dbmin = 0.1
nbasis = %i
subcmd = 
dbsave
hess_upd
#multmap = 1, 1, 1,   # uncomment and change if needed
energy = 1     # can be either 1 or 2. 1: use casscf energies; 2: use caspt2 energies
#simulate
end-dirdyn-section

end-input
""" % (project, gaussfile, basis)


outfile = open(filename, 'w')
outfile.write(outstring)
outfile.close()

print "Created input file " + filename
