##############################################################################
#             --- Python extensions and utilities for MCTDH ---              #
#                 Module: IOFortran                                          #
#                 Author: Oriol Vendrell                                     #
#                 Year  : 2006                                               #
##############################################################################

"""
Define routines to read and write binary files in Fortran style.

A Fortran file is made of records, in the following way:
   reclen1 data1 reclen1 reclen2 data2 reclen2 .... reclenN dataN reclenN EOF

recleni are 4 byte integers, they contain the data length in bytes,
datai can be anything, ascii, integers, floats ...
"""

import struct

def readFortranBinFile2List(fpath):
    """Read a complete binary file and return it in a list

    fpath -- path to the read file
    on return, two lists, the record lengths and the data
    """
    f=open(fpath,'rb')
    rlengths = []
    records = []
    r = readRecord(f)
    while r:
        rlengths.append(r[0])
        records.append(r[1])
        r = readRecord(f)
    f.close()
    return rlengths,records


def readRecord(f):
    """Read a single record from a Fortran binary file

    f -- file object to read from,
    on return, record length (integer) and data (raw string)

    The file object is expected to be already open and will not be closed
    in this routine.
    If EOF return an empty string
    """
    rl = f.read(4)
    if rl:
        rlength = struct.unpack('i',rl)[0]
        data = f.read(rlength)
        rl = f.read(4)
        return rlength,data
    else:
        return rl


def writeList2FortranBinFile(fpath,list):
    """Write a list of strings to Fortran file

    fpath -- path to the write file
    list -- list of strings, supposedly properly formatted
    """
    f=open(fpath,'wb')
    for s in list:
        writeRecord(f,s)
    f.close()


def writeRecord(f,record):
    """Write a single record to a Fortran binary file

    f -- file object to write to
    record -- record to be written
    The file object is expected to be already open
    """
    l = len(record)
    lbin = struct.pack('i',l)
    f.write(lbin)
    f.write(record)
    f.write(lbin)

