#!/usr/bin/env python3

import re
from minput import MSection, InputError
from parsingtools import *


class InitSectionCheckPes(MSection):

    def __init__(self, lines):

        self.coord_labels = []
        self.refindex = []
        self.fix = []

        MSection.__init__(self,lines)

        if lines[0].upper() != "INIT-SECTION":
            message="Input seems not to contain a " \
                     + "INIT-SECTION. First line reads: " \
                     + lines[0]
            raise Exception(message)
        elif lines[-1].lower() != "end-init-section":
            message="Input seems not to contain a " \
                     + "INIT-SECTION. Last line reads: " \
                     + lines[-1]
            raise InputError(message)

        self.openstr = lines.pop(0)
        self.closestr = lines.pop(-1)
        for line in lines:
            values = line.split()
            if len(values) < 2:
                message = "A line in the INIT-SECTION "
                message += "must contain at least 2 entries: "
                message += " coordinate. label and gridpoint. "
                message += " Line contains " + str(len(values))
                message += " entries: " + str(values) + "."
            if len(values) > 3:
                message = "A line in the INIT-SECTION "
                message += "must contain at most 3 entries: "
                message += " coordinate label, gridpoint , and (optional)"
                message += " 'fix' "
                message += " Line contains " + str(len(values))
                message += " entries: " + str(values) + "."
                raise InputError(message)
            self.coord_labels.append(values.pop(0))
            try:
                gpoint = int(values.pop(0))
            except ValueError as V:
                message = "The entries following the coordinate labels "
                message += "in the INIT-SECTION must be integer"
                message += "numbers. "
                message += " Wrong line reads: " + str(line)
                raise ValueError(message)
            self.refindex.append(gpoint)
            if len(values) > 0:
                fix = values.pop(0)
                if fix.lower() == "fix":
                    self.fix.append(True)
                else:
                    message = "Expected 'fix', not: " + str(fix)
                    message += "Wriong line reads:" + str(line)
                    raise InputError(message)
            else:
                self.fix.append(False)

        allfix = True
        for f in self.fix:
            allfix = f and allfix
        if allfix:
            raise InputError("INIT-SECTION: coordinates cannot be all fix.")



    def __str__(self):

        string = str(self.openstr) + '\n'
        string += "# label   gridpoint   fix (optional)\n"
        for i,lab in enumerate(self.coord_labels):
            string += ' ' + lab + '    ' + str(self.refindex[i])
            if self.fix[i]:
                string += '    fix'
            string += '\n'

        string += str(self.closestr) + '\n'
        return string





