#!/usr/bin/env python3


import os
import numpy
from minput import MInput, InputError
from checkpes_runsec import RunSectionCheckPes
from checkpes_initsec import InitSectionCheckPes
from parsingtools import *



class CheckPesInput(MInput):
    """Input """


    def __init__(self, filename=""):
        self.runsection = None
        self.initsection = None
        self.message = ""
        if filename:
            self.import_file(filename)


    def __str__(self):
        string = "# Input for analyse program 'statanalyse'.\n\n"
        if self.runsection is not None:
            string += str(self.runsection)
            string += "\n"
        if self.initsection is not None:
            string += str(self.initsection)
            string += "\n"
        string += "\nend-input\n"
        return string


    def import_file(self, filename):
        self.filename = os.path.abspath(filename)
        f = open(filename, 'r')
        lines = f.readlines()
        f.close()
        lines = preparse_input(lines)
        lines = chop_end(lines)
        lines_bak = lines

        tags = ("RUN-SECTION", "end-run-section")
        seclines, lines = find_section(lines, tags)
        if not seclines:
            self.missing_section_error(tags[0], filename)
        self.runsection = RunSectionCheckPes(seclines)


        tags = ("INIT-SECTION", "end-init-section")
        seclines, lines = find_section(lines, tags)
        if not seclines:
            self.missing_section_error(tags[0], filename)
        self.initsection = InitSectionCheckPes(seclines)


        if 'end-input' in lines:
            lines.pop(lines.index('end-input'))
        else:
            self.message = "Mising 'end-input' in file " + filename
            raise InputError(self.message)

        if len(lines):
            self.message = "Input file " + filename
            self.message += " contains extra lines: "
            self.message += str(lines)
            raise InputError(self.message)

        self.checkConsistency()

        # Return preparsed input for checks
        return lines_bak

    def checkConsistency(self):
        "Check if the Data in the input file is complete and consistent"

        pass




