#!/usr/bin/env python3

import re
from minput import MSection, InputError
from parsingtools import *



class ClusteranaCoordSection(MSection):

    def __init__(self, lines):

        self.coord_labels = []
        self.ranges_start = []
        self.ranges_end = []
        self.maxlablen = 0
        self.source = None
        self.routine = None

        MSection.__init__(self,lines)

        if lines[0].upper() != "COORDINATES-SECTION":
            message="Input seems not to contain a " \
                     + "COORDINATES-SECTION. First line reads: " \
                     + lines[0]
            raise InputError(message)
        elif lines[-1].lower() != "end-coordinates-section":
            message="Input seems not to contain a " \
                     + "COORDINATES-SECTION. Last line reads: " \
                     + lines[-1]
            raise InputError(message)

        self.openstr = lines.pop(0)
        self.closestr = lines.pop(-1)

        key = "user-source"
        strval, lines = find_param(key, lines)
        if strval:
            self.source = strval


        key = "routine"
        strval, lines = find_param(key, lines)
        if strval:
            self.routine = strval


        if self.source is not None and self.routine is None:
            message = "COORDINATES-SECTION: 'source = S': "
            message += "keyword 'routine = S' is required."
            raise InputError(message)
        if self.source is None and self.routine is not None:
            message = "COORDINATES-SECTION: 'routine = S': "
            message += "source file not specified ('source = S')."
            raise InputError(message)

        if self.routine is not None and len(lines) > 0:
            message = "COORDINATES-SECTION: "
            message += "No other data allowed if a routine "
            message += "has been specified."
            raise InputError(message)

        for line in lines:
            values = line.split()
            if len(values) != 3:
                message = "A line in the COORDINATES-SECTION "
                message += "must contain 3 entries: "
                message += " coord.-label, start and end."
                message += " Line contains " + str(len(values))
                message += " entries: " + str(values) + "."
                raise InputError(message)
            self.coord_labels.append(values.pop(0))
            try:
                start = float(re.sub('d','e',values.pop(0)))
                end = float(re.sub('d','e',values.pop(0)))
            except ValueError as V:
                message = "The entries following the coordinate labels "
                message += "in the COORDINATES-SECTION must be real numbers. "
                message += " Wrong line reads: " + str(line)
                raise InputError(message)
            if start >= end:
                message = "The start value of the range "
                message += "in the COORDINATES-SECTION must be smaller "
                message += "then the end value. "
                message += "Wrong line reads: " + str(line)
                raise InputError(message)
            self.ranges_start.append(start)
            self.ranges_end.append(end)

        for lab in self.coord_labels:
            if len(lab) > self.maxlablen:
                self.maxlablen = len(lab)


    def __str__(self):

        string = str(self.openstr) + '\n'

        if self.source is not None and self.routine is not None:
            string += " user-source = " + str(self.source) + "\n"
            string += " routine = " + str(self.routine) + "\n"
        else:
            string += "# label    range\n"
            for i,lab in enumerate(self.coord_labels):
                string += ' ' + lab.ljust(self.maxlablen)
                string += ' %16.8f' % self.ranges_start[i]
                string += ' %16.8f' % self.ranges_end[i]
                string += '\n'

        string += str(self.closestr) + '\n'
        return string





