#!/usr/bin/env python3



from minput import MInput, InputError
from clusterana_runsec import ClusteranaRunSection
from clusterstat_expansionsec import ClusterstatExpansionSection
from clusterana_coordsec import ClusteranaCoordSection
from parsingtools import *
from mtools import file2lines


class ClusteranaInput(MInput):
    """Input """


    def __init__(self, filename=""):
        self.runsection = None
        self.expansionsection = None
        self.coordsection = None
        if filename != "":
            self.importFile(filename)


    def __str__(self):
        string = "# Input for analyse program 'statanalyse'.\n\n"
        if self.runsection is not None:
            string += str(self.runsection)
            string += "\n"
        if self.expansionsection is not None:
            string += str(self.expansionsection)
            string += "\n"
        if self.coordsection is not None:
            string += str(self.coordsection)
            string += "\n"
        string += "\nend-input\n"
        return string


    def importFile(self, filename):

        self.filename = absExpandPath(filename)
        lines = file2lines(self.filename)
        lines = preparse_input(lines)
        lines_bak = lines

        message = "Unable to find section in file " + filename + " "

        tags = ("RUN-SECTION", "end-run-section")
        seclines, lines = find_section(lines, tags)
        if not seclines:
            self.missing_section_error(tags[0], filename)
        self.runsection = ClusteranaRunSection(seclines)

        # this is not obligatory:
        tags = ("EXPANSION-SECTION", "end-expansion-section")
        seclines, lines = find_section(lines, tags)
        if seclines:
            self.expansionsection = ClusterstatExpansionSection(seclines)

        # this is not obligatory:
        tags = ("COORDINATES-SECTION", "end-coordinates-section")
        seclines, lines = find_section(lines, tags)
        if seclines:
            self.coordsection = ClusteranaCoordSection(seclines)


        if 'end-input' in lines:
            lines.pop(lines.index('end-input'))
        else:
            message = "Mising 'end-input' in file " + filename
            raise InputError(message)

        if len(lines) > 0:
            message = "Input file " + filename + " contains unrecognized data:\n"
            message += "\n".join(lines)
            raise InputError(message)

        self.checkConsistency()


    def checkConsistency(self):
        pass


