#!/usr/bin/env python3

import re
from minput import MSection, InputError
from parsingtools import *


class ClusterstatCoordSection(MSection):

    def __init__(self, lines):

        self.coord_labels = []
        self.ranges_start = []
        self.ranges_end = []
        self.step_scale = []

        self.maxlablen = 0

        MSection.__init__(self,lines)

        if lines[0].upper() != "COORDINATES-SECTION":
            message="Input seems not to contain a " \
                     + "COORDINATES-SECTION. First line reads: " \
                     + lines[0]
            raise Exception(message)
        elif lines[-1].lower() != "end-coordinates-section":
            message="Input seems not to contain a " \
                     + "COORDINATES-SECTION. Last line reads: " \
                     + lines[-1]
            raise Exception(message)

        self.openstr = lines.pop(0)
        self.closestr = lines.pop(-1)
        for line in lines:
            values = line.split()
            if len(values) < 3:
                message = "A line in the COORDINATES-SECTION "
                message += "must contain at least 3 entries: "
                message += " coord.-label, start and end."
                message += " Line contains " + str(len(values))
                message += " entries: " + str(values) + "."
                raise RuntimeError(message)
            if len(values) > 4:
                message = "A line in the COORDINATES-SECTION "
                message += "must contain at most 4 entries: "
                message += " coord. label, start, end, and (optional)"
                message += " step-scale"
                message += " Line contains " + str(len(values))
                message += " entries: " + str(values) + "."
                raise RuntimeError(message)
            self.coord_labels.append(values.pop(0))
            try:
                start = float(re.sub('d','e',values.pop(0)))
                end = float(re.sub('d','e',values.pop(0)))
            except ValueError as V:
                message = "The entries following the coordinate labels "
                message += "in the COORDINATES-SECTION must be real numbers. "
                message += " Wrong line reads: " + str(line)
                raise InputError(message)
            if start >= end:
                message = "The start value of the range "
                message += "in the COORDINATES-SECTION must be smaller "
                message += "then the end value. "
                message += "Wrong line reads: " + str(line)
                raise InputError(message)
            self.ranges_start.append(start)
            self.ranges_end.append(end)
            if values:
                scale = float(re.sub('d','e',values.pop(0)))
                self.step_scale.append(scale)
            else:
                self.step_scale.append(1.e0)

        for lab in self.coord_labels:
            if len(lab) > self.maxlablen:
                self.maxlablen = len(lab)

    def __str__(self):

        string = str(self.maxlablen)
        string += str(self.openstr) + '\n'
        string += "# label   range   step-scale(optional)\n"
        for i,lab in enumerate(self.coord_labels):
            string += ' ' + lab.ljust(self.maxlablen)
            string += ' %16.8f' % self.ranges_start[i]
            string += ' %16.8f' % self.ranges_end[i]
            if self.step_scale[i]:
                string += ' %16.8f' % self.step_scale[i]
            string += '\n'
        string += str(self.closestr) + '\n'
        return string





