#!/usr/bin/env python3

import re

from minput import MSection, InputError
from parsingtools import *
from mtools import getNonUnique

class ClusterstatExpansionSection(MSection):

    def __init__(self, lines, coordsec=None):

        MSection.__init__(self,lines)

        if lines[0].upper() != "EXPANSION-SECTION":
            message = "Input seems not to contain a " \
                      + "EXPANSION-SECTION. First line reads: " \
                      + lines[0]
            raise InputError(message)
        elif lines[-1].lower() != "end-expansion-section":
            message="Input seems not to contain a " \
                     + "EXPANSION-SECTION. Last line reads: " \
                     + lines[-1]
            raise InputError(message)

        self.openstr = lines.pop(0)
        self.closestr = lines.pop(-1)

        self.clusterlist=[]
        #check the clusterlist in job class for consistency
        for n in range(len(lines)):
            # this relies in str2tupel to fail if something
            # goes wrong
            line = lines.pop(0)
            linelist = line.split()
            if len(linelist) > 1:
                message = 'Too many entries in line following '
                message += 'EXPANSION-SECTION. Use comma separated '
                message += 'keywords. Wrong line reads: ' + line
                raise InputError(message)
            elif len(linelist) == 1:
                cline = re.sub(",\)",")",linelist[0])
                clist = cline.split('),(')
                if len(clist) > 1:
                    clist[0] += ')'
                    clist[-1] = '(' + clist[-1]
                    for i in range(1,len(clist)-1):
                        clist[i] = '(' + clist[i] + ')'
                try:
                    tpllist = map(str2tupel, clist)
                except:
                    message = 'EXPANSION-SECTION should only contain '
                    message += "coma separated tuples but contains: "
                    message += "; ".join(clist) + "."
                    raise InputError(message)

                self.clusterlist += tpllist


        self.clusterlist.sort()
        self.clusterlist.sort(key = lambda elem: len(elem))

        multiples = getNonUnique(self.clusterlist)
        if len(multiples) > 0:
            message = "EXPANSION-SECTION contains multiply defined "
            message += "clusters: "
            message += ", ".join([str(m) for m in multiples])
            message += "."
            raise InputError(message)

        for tpl in self.clusterlist:
            multiples = getNonUnique(tpl)
            multiples = [s for s in multiples if s != "*"]
            if len(multiples) > 0:
                message = "EXPANSION-SECTION: tupel "
                message += "contains multiply defined modes "
                message += "or coordinates: ("
                message += ", ".join([str(m) for m in tpl])
                message += ")."
                raise InputError(message)


    def __str__(self):
        string = self.openstr + '\n'
        for elem in self.clusterlist:
            selem = str(elem)
            selem = re.sub(",\)",")",selem)
            selem = re.sub("'","",selem)
            string += " " + selem + '\n'
        string += self.closestr + '\n '
        return string








