#!/usr/bin/env python3

import os


def _checkName(name):
    """
    +++ DO NOT USE - THIS MAY GO AWAY! +++

    Simple property checking on a 'name' string.
    """

    if not type(name) == type("string"):
        raise TypeError("name must be type(string)")

    if name[0].isdigit():
        message ="name must not start with a digit: "
        message += name[0]
        raise ValueError(message)

    if " " in name or "\t" in name:
        raise ValueError("name must not contain blanks.")

    if "\n" in name:
        raise ValueError("name must not contain newlines.")

    if "\r" in name:
        raise ValueError("name must not contain linefeeds.")




class Container(object):
    """
    +++ DO NOT USE - THIS MAY GO AWAY! +++

    Simple container for whatever objects.
    This basically allows grouping things.
    """
    def __init__(self,names=(),values=()):


        if len(names) != len(values):
            message = "Length of names must be equal to length of values."
            raise ValueError(message)

        for n, name in enumerate(names):
            self.append(name, vanues[n], False)


    def __repr__(self):
        string = "Container: Attributes are:\n"
        for k,v in self.__dict__.items():
            string += k + ": " + str(v) + "\n"
        return string



    def append(self,name,value,loverwrite=False):
        """
        append(self,name,value, loverwrite=False)

        Append an attribute with given name and value.
        If loverwrite is False and attribute exists a
        KeyError is raised.
        """
        _checkName(name)

        if self.__dict__.has_key(name):
            if not loverwrite:
                raise KeyError("Attribute "+name+" already exists.")

        self.__dict__[name] = value


    def pop(self,name):
        "Remove an attribute from the container and return its value."

        if self.__dict__.has_key(name):
            return self.__dict__.pop(name)
        else:
            raise KeyError("Attribute "+name+" does not exist.")


    @property
    def isEmpty(self):
        if len(self.__dict__.keys()) == 0:
            return True
        else:
            return False


    @property
    def nAttributes(self):
        return len(self.__dict__.keys())


###############################################################


# for newer python versions, this should better
# inherit from a class from the "collections" module
# which in later python versions provides more
# functionality
class FileContainer(object):
    """
    class FileContainer(object):
        init: FileContainer(filenames=(,),modes="r", loverwrite=False)

    Simple container for file objects.

    This basically allows grouping all files in a single container.
    this makes it easy to detatch files from job before pickle is
    called (to produce restart files).
    """

    def __init__(self,filenames=(),modes="r", loverwrite=False):

        # if filenames is a string:
        # make it a tuple so enumerate won't fail
        if type(filenames) == type("string"):
            filenames = (filenames,)

        if loverwrite is True:
            loverwrite = (True,)*len(filenames)
        elif loverwrite is False:
            loverwrite = (False,)*len(filenames)

        if type(modes) == type("string"):
            modes = (modes,)*len(filenames)

        if len(filenames) != len(modes):
            raise RuntimeError("Length of filenames and modes must match")

        for n, fname in enumerate(filenames):
            if os.path.exists(fname) and not \
                   loverwrite[n] and modes[n].lower() == "w":
                message = "File " + fname
                message += " exists and no permission to overwrite."
                raise IOError(message)
            else:
                f = file(fname, mode=modes[n])
                self.appendFile(fname, f, False)


    def __repr__(self):
        string = "Filecontainer: Attributes are:\n"
        for k,v in self.__dict__.items():
            string += k + ": " + str(v) + "\n"
        return string

    def __getitem__(self, key):
        if self.__dict__.has_key(key):
            return self.__dict__[key]
        else:
            message = "No such file in container: "
            message += str(key)
            raise KeyError(message)

    def __contains__(self,key):
        return key in self.__dict__

    def closeAll(self):
        "Close all files in container"
        for f in self.__dict__.values():
            if not f.closed:
                f.close()

    def popClosed(self):
        files = []
        for k,v in self.__dict__.items():
            if v.closed:
                files.append((k,self.__dict__.pop(k)))
        return files


    def appendFile(self,name,fileobject,replaceExisting=False):
        """
        appendFile(self,name,fileobject,replaceExisting=False)

        Append a file object to the container."""

        _checkName(name)

        if self.__dict__.has_key(name):
            if not replaceExisting:
                raise KeyError(name+" already exists.")

        self.__dict__[name] = fileobject


    def popFile(self,name):
        "Remove a file from the container and return a handle."

        _checkName(name)
        if self.__dict__.has_key(name):
            if not self.__dict__[name].closed:
                self.__dict__[name].close()
            return self.__dict__.pop(name)
        else:
            raise KeyError(name+" does not exist.")


    def detachAll(self):
        """
        detachAll() --> list of tuples with filenames and fileobjects

        Detaches all files from the attributes and replaces them by their
        string representations. Returns a list of tuples with filenames
        and fileobjects.
        The returned list can be re-attached with the reAttach() method.

        """
        files = []
        for k,v in self.__dict__.items():
            files.append((k,v))
            self.__dict__[k] = "str(" + str(v) + ")"
        return files

    def reAttach(self,files,checkexisting=True):
        """
        reAttach(files,checkexisting=True)

        Append a list of file-objects. The argument 'files'
        has to be a list of tuples, each containing a name and
        a file object. Usually this list is obtained with the
        class method 'detachAll'

        If 'checkexisting' is true and no attribute with a given name
        does exist, then a KeyError is raised.
        """
        for k,v in files:
            if checkexisting and not k in self.__dict__.keys():
                message = "File " + str(k) + " is not in container"
                raise KeyError(message)
            self.appendFile(k,v,True)


    def flushWriteables(self):
        "Flush all files in mode 'w' or 'a'."
        for f in self.__dict__.values():
            if f.mode == "w" or f.mode == "a":
                f.flush()


    @property
    def isEmpty(self):
        if len(self.__dict__.keys()) == 0:
            return True
        else:
            return False


    @property
    def nFiles(self):
        return len(self.__dict__.keys())


