#!/usr/bin/env python3

from minput import MSection
from parsingtools import *


class PBasSection(MSection):

    def __init__(self, lines):
        MSection.__init__(self,lines)
        if lines[0].upper() != "PBASIS-SECTION":
            if  lines[0].upper() != "PRIMITIVE-BASIS-SECTION":
                message="Input seems not to contain a " \
                         + "PRIMITIVE-BASIS-SECTION. First line reads: " \
                         + lines[0]
            raise RuntimeError(message)
        if lines[-1].lower() != "end-pbasis-section":
            if lines[-1].lower() != "end-primitive-basis-section":
                message="Input seems not to contain a " \
                         + "PRIMITIVE-BASIS-SECTION. Last line reads: " \
                         + lines[-1]
            raise RuntimeError(message)

        self.openstr = lines.pop(0)
        self.closestr = lines.pop(-1)

        self.coord_labels = []
        self.basis_type = []
        self.basis_ponts = []
        self.basis_params = []

        for line in lines:
            values = line.split()
            self.coord_labels.append(values.pop(0))
            self.basis_type.append(values.pop(0))
            try:
                self.basis_ponts.append(int(values.pop(0)))
            except ValueError as V:
                bastype = self.basis_type[-1]
                if bastype.lower() != "k":
                    message = "PRIMITIVE-BASIS-SECTION: "
                    message += "the basis type " + str(bastype)
                    message += " must be followed by the grid size. "
                    message += "Wrong line reads " + str(line)
                    raise ValueError(message)
                self.basis_ponts.append(None)

            self.basis_params.append(tuple([val for val in values]))

    def __str__(self):
        string = str(self.openstr) + '\n'
        for i,lab in enumerate(self.coord_labels):
            string += ' ' + lab + '    ' + self.basis_type[i]
            if self.basis_type[i].lower() != 'k' and \
                   self.basis_ponts[i] is not None:
                string += '    ' + str(self.basis_ponts[i])
            for par in self.basis_params[i]:
                string += '    ' + str(par)
            string += '\n'
        string += str(self.closestr) + '\n'
        return string

    def strByIndex(self,i):
        "Get the string for coordinate i"
        if i > len(self.coord_labels)-1 or i < 0:
            message = "There are " + str(len(self.coord_labels))
            message += " coordinates. Chose a number between 0 and "
            message += str(len(self.coord_labels)-1) + ", not "
            message += str(i) + "."
            raise IndexError(message)

        string = ' ' + self.coord_labels[i] + '    ' + self.basis_type[i]
        if self.basis_type[i].lower() != 'k' and \
               self.basis_ponts[i] is not None:
            string += '    ' + str(self.basis_ponts[i])
        for par in self.basis_params[i]:
            string += '    ' + str(par)
        return string

    def strByLabel(self,lab):
        "Get the string for coordinate label lab"
        i =  self.coord_labels.index(lab)
        return self.strByIndex(i)


    def __eq__(self, other):
        return self.coord_labels == other.coord_labels\
               and self.basis_type  == other.basis_type\
               and self.basis_ponts == other.basis_ponts\
               and self.basis_params == other.basis_params


    def __ne__(self, other):
        return not self == other

