#!/usr/bin/env python3

import os

class InputError(RuntimeError):
    "class InputError(RuntimeError)"
    pass


class MSection:
    """
    Base class for input sections for programs
    of the MCTDH package.
    Don't use directly - use for inharitance.
    """

    def __str__(self):
        return "# Class 'MSection'. No data.\n"


    def __init__(self, lines):
        # Lines need to be preparsed by MInput.preparse_input()

        if lines and len(lines) < 2:
            message="Input seems not to contain a SECTION. " \
                     + "Too few lines."
            raise InputError(message)

        self.openstr = lines[0]
        self.closestr = lines[-1]

    def missing_key_error(self, key):
        "Raise Exception: 'missing keyword.'"
        message = "Keyword " + str(key) + " missing in " + \
            self.openstr
        raise InputError(message)



class MInput:
    """
    Base class for input parsing of programs of the MCTDH
    package. Implements basic I/O. Does not store any
    Information.
    Don't use directly - use for inharitance.
    """

    def __str__(self):
        return "# Class 'MInput'. No data.\n\nend-input\n"


    def export_file(self, filename,mode='w',owerwrite=False):
        """
        ExportFile(filename,mode='w',owerwrite=False):
        Save input to ASCII file. mode=char as for files
        """

        if os.path.exists(filename) and owerwrite is not True:
            message = "No permission to owerwrite file "
            message += filename + ". Set owerwrite flag."
            raise RuntimeError(message)

        f=file(filename,mode)
        f.write(str(self))
        f.close()


    def missing_section_error(self, sectionname, filename=".inp"):
        "Raise InputError on missing section."
        message = "Section " + str(sectionname) + " missing in file " + \
            str(filename)
        raise InputError(message)


    def checkConsistency(self):
        """
        Check if the Data in the input file is complete and consistent.
        This routine should be implemented in all derived classes
        """
        pass


