"Default OCT algorithm - the Rabitz scheme"
#########################################################################
#
# FUNCTIONS IN THIS MODULE
#
# - anabw                     Analysis after backward propagation
# - anafw                     Analysis after forward propagation
# - backward                  Perform a backward propagation
# - dummy                     A dummy
# - forward                   Perform a forward propagation
# - mkinputfiles              Create all temporary input files
# - mkdefault                 Generator function for this algorithm
# - parseinput                Parse input files
# - restartinfo               Collect data for restart runs
# - startup                   Create initial fields, states and perform
#                             the first forward propagation
#
# 04/2009 MS
#########################################################################

from octtools import *


##############################################################################

def parseinput(jobopts,inp,op):
    "Extract information from input files"

    import os,re
    from messages import log

    log.info("parsing input")

    # check if all needed sections are there
    log.debug('Checking that all sections are there.')
    # Exception handling in calling module
    chksections(inp)
    parserundefault(inp,jobopts)
    parsetardefault(inp,jobopts)
    isMoreToParse=parseoctdefault(inp,jobopts)
    if isMoreToParse:
        message="Found unknown or unsupported keywords (within the "
        message+="default algorithm):\n"
        message+=str(inp['octsection'])
        raise RuntimeError(message)

    setdefaultfiles(jobopts)
    os.chdir(jobopts['outdir'])


##############################################################################
# Create temporary input files
##############################################################################
def mkinputfiles(jobopts,inp,op):
    "Write temporary files, assign filenames, create temporary directories"

    import mctdhinp, os, re
    from messages import log


    log.info("Creating temporary files and directories")

############################## PIPES (FIFOS) ###############################
    log.debug('Creating named pipes')
    try:
        createfifos(jobopts)
    except Exception as E:
        log.exception(E)
        raise RuntimeError('Error while to generating fifo-files. See optcntrl.log for traceback')
    except:
        raise Exception('Unknown error while to generating fifo-files')


############################## OPERATOR FILES ###############################
    log.debug('Generating temparary operator files.')
    op1=mctdhinp.mctdhoper()
    op1.copy(op)
    #delete the functions for initial fields (go back to 'real' QUANTICS oper file)
    op1.delallfunctions()

    if not jobopts['lsimprop']:
        #use forward as it is
        filename=jobopts['forward_op']
        log.debug('Exporting file: '+filename)
        op1.exportfile(filename,jobopts['overwrite'])
        #backward: switch sign and replace fieldf.dat with fieldb.dat
        op1.switchsign()
        for i in range(len(op1.sections['labels'].data)):
            op1.sections['labels'].data[i]=\
                re.sub('fieldf.dat','fieldb.dat',op1.sections['labels'].data[i])

        filename=jobopts['backward_op']
        log.debug('Exporting file: '+filename)
        op1.exportfile(filename,jobopts['overwrite'])
    else:
        op2=mctdhinp.mctdhoper()
        op2.copy(op1)

        for i in range(len(op2.sections['labels'].data)):
            op2.sections['labels'].data[i]=\
                re.sub('fieldf.dat','fieldf_init.dat',op2.sections['labels'].data[i])

        filename=jobopts['forward_init_op']
        log.debug('Exporting file: '+filename)
        op2.exportfile(filename,jobopts['overwrite'])

        op2.copy(op1)
        for i in range(len(op2.sections['labels'].data)):
            op2.sections['labels'].data[i]=\
                re.sub('fieldf.dat','fieldf_targ.dat',op2.sections['labels'].data[i])

        filename=jobopts['forward_targ_op']
        log.debug('Exporting file: '+filename)
        op2.exportfile(filename,jobopts['overwrite'])

        op2.copy(op1)
        for i in range(len(op2.sections['labels'].data)):
            op2.sections['labels'].data[i]=\
                re.sub('fieldf.dat','fieldb_init.dat',op2.sections['labels'].data[i])

        op2.switchsign()
        filename=jobopts['backward_init_op']
        log.debug('Exporting file: '+filename)
        op2.exportfile(filename,jobopts['overwrite'])

        op2.copy(op1)
        for i in range(len(op2.sections['labels'].data)):
            op2.sections['labels'].data[i]=\
                re.sub('fieldf.dat','fieldb_targ.dat',op2.sections['labels'].data[i])

        op2.switchsign()
        filename=jobopts['backward_targ_op']
        log.debug('Exporting file: '+filename)
        op2.exportfile(filename,jobopts['overwrite'])

        del op2

    del op1

############################## INPUT FILES ###############################

    # first do some consistency checks
    if inp['spfbassection'].parameters.has_key('packets'):
        packets = inp['spfbassection'].parameters['packets'].split(',')
        if len(packets) > 1:
            if packets[1].lower() != "multi-set":
                message = "SPF-BASIS-SECTION: "
                message += "Multi-target optimizations require packtes "
                message += "to be multi-set. Wrong keyword: packets = "
                message += inp['spfbassection'].parameters['packets']
                raise RuntimeError(message)
        else:
            if "single-set" in inp['spfbassection'].flags and jobopts['ltaroper']:
                message = "SPF-BASIS-SECTION: "
                message += "Multi-tarrget optimizations for target operators "
                message += "require electronic "
                message += "states to be multi-set. Wrong keyword: single-set."
                raise RuntimeError(message)




    #input file for the initial state
    log.debug('Generating temparary input files.')
    inp1=mctdhinp.mctdhinput()
    inp1.copy(inp)

    inp1.delsection('octsection')
    inp1.delsection('targetsection')
    inp1.delsection('filtersection')

    inp1['runsection'].delflag('propagation')
    inp1['runsection'].delflag('optcntrl')
    inp1['runsection'].delparam('optcntrl')
    inp1['runsection'].setflag('geninwf')
    inp1['runsection']['name']='initialstate'

#    next line screws up restarts on clusters - removed
#    inp1['opersection']['oppath']=jobopts['outdir']

    if jobopts['lsimprop']:
        inp1['opersection']['opname']='forward_init'
    else:
        inp1['opersection']['opname']='forward'

    dirname=outpath(inp1['runsection']['name'])
    if jobopts['overwrite'] or not os.path.exists(dirname):
        emptytree(dirname)


    filename=jobopts['initstate_inp']
    log.debug('Exporting file: '+filename)
    inp1.exportfile(filename,jobopts['overwrite'])

    #input file for the target state if target is a wavefunction
    if not jobopts['ltaroper']:
        inp1['runsection']['name']='targetstate'
        inp1['initwfsection'].copy(inp['targetsection'])
        inp1['initwfsection'].openstr='INIT_WF-SECTION'
        inp1['initwfsection'].closestr='END-INIT_WF-SECTION'
        if jobopts['lsimprop']:
            inp1['opersection']['opname']='backward_targ'
        else:
            inp1['opersection']['opname']='backward'

        filename=jobopts['targstate_inp']
        log.debug('Exporting file: '+filename)
        inp1.exportfile(filename,jobopts['overwrite'])

        dirname=outpath(inp1['runsection']['name'])
        if jobopts['overwrite'] or not os.path.exists(dirname):
            emptytree(dirname)


    #input file for first forward propagation
    inp1['runsection'].setflag('propagation')
    if jobopts['lsimprop']:
        inp1['opersection']['opname']='forward_init'
    else:
        inp1['opersection']['opname']='forward'

#    next line screws up restarts on clusters - removed
#    inp1['opersection']['oppath']=jobopts['outdir']

    inp1['initwfsection'].clearcontent()
    inp1['initwfsection']['file']='initialstate'
    if jobopts['lsimprop']:
        inp1['runsection']['name']='forward_init'
    else:
        inp1['runsection']['name']='forward'

    filename=jobopts['forward1_inp']
    log.debug('Exporting file: '+filename)
    inp1.exportfile(filename,jobopts['overwrite'])

    dirname=outpath(inp1['runsection']['name'])
    if jobopts['overwrite'] or not os.path.exists(dirname):
        emptytree(dirname)


    #input file(s) for iterative forward propagation(s)
    if jobopts['lsimprop']:
        inp1['runsection']['name']='forward_init'

        inp1['runsection'].delflag('optcntrl')
        inp1['runsection']['optcntrl']='simprop, update'

        inp1['initwfsection'].clearcontent()
        inp1['initwfsection']['file']='initialstate'
        inp1['opersection']['opname']='forward_init'


        filename=jobopts['forward_init_inp']
        log.debug('Exporting file: '+filename)
        inp1.exportfile(filename)

        dirname=outpath(inp1['runsection']['name'])
        if jobopts['overwrite'] or not os.path.exists(dirname):
            emptytree(dirname)


        inp1['runsection']['name']='forward_targ'
        inp1['runsection']['optcntrl']='simprop'

        inp1['initwfsection'].clearcontent()
        inp1['initwfsection']['file']='backward_targ'
        inp1['opersection']['opname']='forward_targ'

        filename=jobopts['forward_targ_inp']
        log.debug('Exporting file: '+filename)
        inp1.exportfile(filename,jobopts['overwrite'])

        dirname=outpath(inp1['runsection']['name'])
        if jobopts['overwrite'] or not os.path.exists(dirname):
            emptytree(dirname)

    else:
        inp1['runsection']['name']='forward'
        inp1['runsection'].delflag('optcntrl')
        if jobopts['lpc']:
            inp1['runsection']['optcntrl']='pc'
        else:
            inp1['runsection'].setflag('optcntrl')


        inp1['initwfsection'].clearcontent()
        inp1['initwfsection']['file']='initialstate'
        inp1['opersection']['opname']='forward'

        filename=jobopts['forwardi_inp']
        log.debug('Exporting file: '+filename)
        inp1.exportfile(filename)

        dirname=outpath(inp1['runsection']['name'])
        if jobopts['overwrite'] or not os.path.exists(dirname):
            emptytree(dirname)


    #input file(s) for iterative backward propagation(s)
    if jobopts['lsimprop']:
        inp1['runsection']['name']='backward_init'

        inp1['runsection'].delflag('optcntrl')
        inp1['runsection']['optcntrl']='simprop'

        inp1['initwfsection'].clearcontent()
        inp1['initwfsection']['file']='forward_init'
        inp1['opersection']['opname']='backward_init'

        filename=jobopts['backward_init_inp']
        log.debug('Exporting file: '+filename)
        inp1.exportfile(filename,jobopts['overwrite'])

        dirname=outpath(inp1['runsection']['name'])
        if jobopts['overwrite'] or not os.path.exists(dirname):
            emptytree(dirname)


        inp1['runsection']['name']='backward_targ'
        inp1['runsection']['optcntrl']='simprop, update'

        inp1['initwfsection'].clearcontent()
        if jobopts['ltaroper']:
            inp1['initwfsection']['file']='forward_init'
            inp1['initwfsection']['operate']=inp['targetsection']['operator']
            inp1['initwfsection'].setflag('operate_no-norm')
        else:
            inp1['initwfsection']['file']='targetstate'

        inp1['opersection']['opname']='backward_targ'

        filename=jobopts['backward_targ_inp']
        log.debug('Exporting file: '+filename)
        inp1.exportfile(filename)

        dirname=outpath(inp1['runsection']['name'])
        if jobopts['overwrite'] or not os.path.exists(dirname):
            emptytree(dirname)

    else:

        inp1['runsection']['name']='backward'

        inp1['runsection'].delflag('optcntrl')
        inp1['runsection'].delparam('optcntrl')
        if jobopts['lpc'] and not jobopts['algorithm']=='krotov':
            inp1['runsection']['optcntrl']='pc'
        else:
            inp1['runsection'].setflag('optcntrl')


        inp1['initwfsection'].clearcontent()
        if jobopts['ltaroper']:
            inp1['initwfsection']['file']='forward'
            inp1['initwfsection']['operate']=inp['targetsection']['operator']
            inp1['initwfsection'].setflag('operate_no-norm')
        else:
            inp1['initwfsection']['file']='targetstate'

        inp1['opersection']['opname']='backward'

        filename=jobopts['backwardi_inp']
        log.debug('Exporting file: '+filename)
        inp1.exportfile(filename,jobopts['overwrite'])

        dirname=outpath(inp1['runsection']['name'])
        if jobopts['overwrite'] or not os.path.exists(dirname):
            emptytree(dirname)

##############################################################################

def startup(jobopts,inp,op):
    """Generate fields, initial and target states,
    do first forward propagarion"""

    import shutil, os
    from messages import log


    # prepare jfunc file
    message="""
# Control functional: J1=population, J2=field-part (negative)
# no.       J1                    J2                J1+J2 \n"""
    dumpfile(jobopts['jfile'],message,jobopts['overwrite'],'w')


    # prepare overlaps file
    message= "# Overlaps with target state or operator. tfinal = "+\
        str(jobopts['tfinal'])+' tout = '+str(jobopts['tout'])+'\n'
    message+="# Format:  No.,  ( Re, Im,  Abs**2, i=1, npacket), total_target_population\n"
    log.debug('Initializing overlaps file')
    dumpfile(jobopts['overlaps'],message,jobopts['overwrite'],'w')

    #for the field generator (uses au)
    tfinal=jobopts['tfinal']*fs
    tout  =jobopts['tout']*fs

    log.info("Generating initial functions:")

    #guess field
    if jobopts['lguessfnc']:
        #-p option to make tfinal known as analytical parameter of oper file
        genfldcmd=jobopts['genfld_exe']+\
            ' -a 2 -O '+jobopts['operfile']+\
            ' -w '*jobopts['overwrite']+\
            ' -o '+outpath('fieldf.dat')+\
            ' 0.0 '+ str(tfinal)+' '+str(tout)+' '+jobopts['guessfnc']

        log.info("Generating Guess-field: "+jobopts['guessfnc'])
        log.debug('Command line: '+genfldcmd)
        status=os.system(genfldcmd)
        if status:
            message= "genfld returned a nonzero exit code while generating the guess field."
            log.warn(message)

        os.rename('genfld.log', 'genfld_'+jobopts['guessfnc']+'.log')

        if jobopts['lsimprop']:

            #guess field backwards
            genfldcmd=jobopts['genfld_exe']+\
                ' -a 2 -O '+jobopts['operfile']+\
                ' -R '+\
                ' -w '*jobopts['overwrite']+\
                ' -o '+outpath('fieldb_init.dat')+\
                ' 0.0 '+ str(tfinal)+' '+str(tout)+' '+jobopts['guessfnc']

            log.info("Generating Guess-field: "+jobopts['guessfnc'] + " (backwards)")
            log.debug('Command line: '+genfldcmd)
            status=os.system(genfldcmd)
            if status:
                message= "genfld returned a nonzero exit code "
                message+="while generating the guess field (backwards)."
                log.warn(message)

            if jobopts['lsvflds']:
                shutil.copyfile(outpath('fieldf.dat'), outpath('fieldf.0'))

            os.rename('genfld.log', 'genfld_'+jobopts['guessfnc']+'b.log')
            shutil.copyfile(outpath('fieldb_init.dat'), outpath('fieldb_targ.dat'))
            shutil.move(outpath('fieldf.dat'), outpath('fieldf_init.dat'))
            shutil.copyfile(outpath('fieldf_init.dat'), outpath('fieldf_targ.dat'))

            del genfldcmd

        else:
            shutil.copyfile(outpath('fieldf.dat'), outpath('fieldb.dat'))
            if jobopts['lsvflds']:
                shutil.copyfile(outpath('fieldf.dat'), outpath('fieldf.0'))


    #reference function
    if jobopts['lreffnc']:
        log.info("Generating Reference-function: "+jobopts['reffnc'] )
        genfldcmd=jobopts['genfld_exe']+\
        ' -a 2 -O '+jobopts['operfile']+\
        ' -w '*jobopts['overwrite']+\
        ' -o '+outpath('reffnc.dat')+\
        ' 0.0 '+ str(tfinal)+' '+str(tout)+' '+jobopts['reffnc']
        log.debug('Command line: '+genfldcmd)
        status=os.system(genfldcmd)
        if status:
            message= "genfld returned a nonzero exit code while "+\
                "generating the reference field."
            log.warn(message)

        os.rename('genfld.log', 'genfld_'+jobopts['reffnc']+'.log')

    #penalty function (envelope)
    if jobopts['lpnltyfnc']:
        log.info("Generating Penalty-function: "+jobopts['pnltyfnc']  )
        genfldcmd=jobopts['genfld_exe']+\
        ' -a 2 -O '+jobopts['operfile']+\
        ' -w '*jobopts['overwrite']+\
        ' -o '+outpath('pnltyfnc.dat')+\
        ' 0.0 '+ str(tfinal)+' '+str(tout)+' '+jobopts['pnltyfnc']
        log.debug('Command line: '+genfldcmd)
        status=os.system(genfldcmd)
        if status:
            message="genfld returned a nonzero exit code while generating"+\
                " the penalty function."
            log.warn(message)
        os.rename('genfld.log', 'genfld_'+jobopts['pnltyfnc']+'.log')


    del genfldcmd


    log.info("Generating initial state")
    # generate initial state
    mctdhcmd=jobopts['quantics_exe']+' '+int(jobopts['overwrite'])*\
        ' -w '+ jobopts['initstate_inp']
    log.debug('Command line: '+mctdhcmd )
    status=os.system(mctdhcmd)
    if status:
        message= "QUANTICS returned a nonzero exit code while"+\
            " generating the initial state"
        log.warn(message)

    # generate target state
    if not jobopts['ltaroper']:
        log.info("Generating target state")
        log.debug('Command line: '+mctdhcmd )
        mctdhcmd=jobopts['quantics_exe']+' '+int(jobopts['overwrite'])*\
            ' -w '+ jobopts['targstate_inp']
        status=os.system(mctdhcmd)
        if status:
            message= "QUANTICS returned a nonzero exit code while"+\
                " generating the target state"
            log.warn(message)

    # first forward propagation
    log.info("Performing the first forward propagation with guessed field.")
    mctdhcmd=jobopts['quantics_exe']+' '+int(jobopts['overwrite'])*\
        ' -w '+ jobopts['forward1_inp']
    log.debug('Command line: '+mctdhcmd )
    status=os.system(mctdhcmd)
    if status:
        message= "QUANTICS returned a nonzero exit code while "+\
            "performing the first forward propagation"
        log.error(message)
        raise RuntimeError(message)
    jobopts['curriter']=0

    log.info("Setup is complete.")

    message='jobopts after setup has competed:\n'
    for k in jobopts.keys(): message+= str(k)+' = '+str(jobopts[k])+'\n'
    dumpfile(outpath('jobopts'),message,jobopts['overwrite'])

##############################################################################
##############################################################################

def forward(jobopts):

    from messages import log
    from mctdhinp import cleanline
    import os, re, subprocess

    if os.path.exists(outpath('stop')):
        log.info('Found stop-file.')
        jobopts['continue']=False
        os.remove(outpath('stop'))
        return

    log.info('Iteration '+ str(jobopts['curriter'])+' (forward)')

    if jobopts.has_key('taroper'):
        taroper=jobopts['taroper']+' '
    else:
        taroper=''

    # start efield in the background
    args=( jobopts['efield_exe'],
           '-pc'*jobopts['lpc'],
           '-F',str(jobopts['penalty-factor']),
           '-S'*jobopts['lpnltyfnc'],
           '-R'*jobopts['lreffnc'],
           '-O',jobopts['dipoleoperator'],
           '-T'*jobopts['ltaroper'],taroper*jobopts['ltaroper'],
           '-i',str(jobopts['curriter']),
           '-p'*jobopts['lsimprop'],
           '-A'*jobopts['lalignphs'],
           '-d'*jobopts['ltdoverlap'],
           '-r'*jobopts['lwrrawflds'],
           '-m'*jobopts['lwrtddipmtx'],
           '-o'*jobopts['lwrtdoverlap'],
           '-t'*jobopts['lwrtdtargetpop'])

    eargs =[ str(arg) for arg in args if arg]

    #just debug-logging
    cmd=re.sub('[,\'\[\]]',' ',str(eargs))
    cmd=cleanline(cmd)
    log.debug("Starting efield in background:")
    log.debug("Command line: "+ cmd )

    # BTW: avoid spaces in arguments list, Popen, spawnv etc. might fail
    # for os.system spaces seem to be OK
    efield_proc=subprocess.Popen(eargs)

    if jobopts['lsimprop']:
       #start one instance of mctdh
       args=( jobopts['quantics_exe'], 
              '-w', 
              os.path.split(jobopts['forward_init_inp'])[1] ) #
       margs1 =[ str(arg) for arg in args if arg]

       #just debug-logging
       cmd=re.sub('[,\'\[\]]',' ',str(margs1))
       cmd=cleanline(cmd)  
       log.debug("Starting first QUANTICS")
       log.debug("Command line: "+ cmd )
       
       mctdh_proc1=subprocess.Popen(margs1)

       #start another instance of mctdh
       args=( jobopts['quantics_exe'], 
              '-w', 
              os.path.split(jobopts['forward_targ_inp'])[1] ) 

       margs =[ str(arg) for arg in args if arg]

       #just debug-logging
       cmd=re.sub('[,\'\[\]]',' ',str(margs))
       cmd=cleanline(cmd)  
       log.debug("Starting second QUANTICS:")
       log.debug("Command line: "+ cmd )

       mctdh_proc=subprocess.Popen(margs)

    else:

        #start one instance of mctdh
        args=( jobopts['quantics_exe'], 
               '-w', 
               os.path.split(jobopts['forwardi_inp'])[1] ) 
        
        margs =[ str(arg) for arg in args if arg]

        #just debug-logging
        cmd=re.sub('[,\'\[\]]',' ',str(margs))
        cmd=cleanline(cmd)  
        log.debug("Starting QUANTICS")
        log.debug("Command line: "+cmd )
        
        mctdh_proc=subprocess.Popen(margs)

    # whatch the child processes. kill them if anything goes wrong
    if jobopts['lsimprop']:
        whatchprogs(efield_proc,mctdh_proc,mctdh_proc1)
    else:
        whatchprogs(efield_proc,mctdh_proc)

##############################################################################

def backward(jobopts):


    import os,re,time,subprocess
    from messages import log
    from mctdhinp import cleanline


    if jobopts.has_key('skipbw'):
        jobopts.pop('skipbw')
        return

    if os.path.exists(outpath('stop')):
        log.info('Found stop-file.')
        jobopts['continue']=False
        os.remove(outpath('stop'))
        return

    log.info('Iteration '+ str(jobopts['curriter'])+' (backward)')

    if jobopts.has_key('taroper'):
        taroper=jobopts['taroper']+' '
    else:
        taroper=''

    # start efield in the background
    args=( jobopts['efield_exe'],
           '-pc'*jobopts['lpc'],
           '-b',
           '-F',str(jobopts['penalty-factor']),
           '-S'*jobopts['lpnltyfnc'],
           '-R'*jobopts['lreffnc'],
           '-O',jobopts['dipoleoperator'],
           '-T'*jobopts['ltaroper'],taroper*jobopts['ltaroper'],
           '-i',str(jobopts['curriter']),
           '-p'*jobopts['lsimprop'],
           '-A'*jobopts['lalignphs'],
           '-d'*jobopts['ltdoverlap'],
           '-r'*jobopts['lwrrawflds'],
           '-m'*jobopts['lwrtddipmtx'],
           '-o'*jobopts['lwrtdoverlap'])

    eargs =[ str(arg) for arg in args if arg]

    #just debug-logging
    cmd=re.sub('[,\'\[\]]',' ',str(eargs))
    cmd=cleanline(cmd)
    log.debug("Starting efield:")
    log.debug("Command line: "+ cmd )

    efield_proc=subprocess.Popen(eargs)

    if jobopts['lsimprop']:
        #start one instance of mctdh 
        args=( jobopts['quantics_exe'], 
               '-w',
               os.path.split(jobopts['backward_targ_inp'])[1])
        margs1 =[ str(arg) for arg in args if arg]

        #just debug-logging
        cmd=re.sub('[,\'\[\]]',' ',str(margs1))
        cmd=cleanline(cmd)  
        log.debug("Starting QUANTICS in background:")
        log.debug("Command line: "+ cmd )

        mctdh_proc1=subprocess.Popen(margs1)

        #start another instance of mctdh 
        args=( jobopts['quantics_exe'], 
               '-w',
               os.path.split(jobopts['backward_init_inp'])[1])

        margs =[ str(arg) for arg in args if arg]

        #just debug-logging
        cmd=re.sub('[,\'\[\]]',' ',str(margs))
        cmd=cleanline(cmd)  
        log.debug("Starting QUANTICS:")
        log.debug("Command line: "+ cmd )

        mctdh_proc=subprocess.Popen(margs)

    else:
      
        args=( jobopts['quantics_exe'], 
               '-w',
               os.path.split(jobopts['backwardi_inp'])[1])
        margs =[ str(arg) for arg in args if arg]

        cmd=re.sub('[,\'\[\]]',' ',str(margs))
        cmd=cleanline(cmd)  
        log.debug("Starting QUANTICS:")
        log.debug("Command line: "+ cmd )

        mctdh_proc=subprocess.Popen(margs)

    # whatch the child processes. kill them if anything goes wrong
    if jobopts['lsimprop']:
        whatchprogs(efield_proc,mctdh_proc,mctdh_proc1)
    else:
        whatchprogs(efield_proc,mctdh_proc)

    if jobopts['curriter'] == 0: anafw(jobopts)


##############################################################################

def anafw(jobopts):
    "Analyse after forward propagation"
    from messages import log
    from mctdheingabe import cleanline
    import shutil,re,subprocess,time

    log.debug('Starting analysis after forward propagation')

    #load overlaps
    log.debug('Reading overlaps file')
    f=open('overlaps','r')
    lastline=f.readlines()[-1]
    f.close()
    del f
    lastline=cleanline(lastline)
    J1=float(lastline.split(' ')[-1])

    if jobopts['lsimprop']:
        shutil.copyfile('fieldf_init.dat', 'fieldf.dat')

    args=( jobopts['jfunc_exe'],
           '-S'*jobopts['lpnltyfnc'],
           '-F',str(jobopts['penalty-factor']),
           '-R'*jobopts['lreffnc'], )

    args =[ str(arg) for arg in args if arg]

    cmd=re.sub('[,\'\[\]]',' ',str(args))
    cmd=cleanline(cmd)
    log.debug("Starting jfunc:")
    log.debug("Command line: "+ cmd )
    
    #start jfunc
    jfunc_proc=subprocess.Popen(args, stdout=subprocess.PIPE)

    returncode=jfunc_proc.poll()
    while returncode is None:
        time.sleep(1) # sleep a second if not done yet
        returncode=jfunc_proc.poll()

    data=jfunc_proc.communicate() #read from PIPE
    if returncode != 0:
        message="Error while calculating control functional.\n"
        message+=str(jobopts['jfunc_exe'])+ ' returned a non-zero exit code.'
        message+='Message: \n'+ str(data)
        raise RuntimeError(message)

    #extract data (first line in pipe)
    data=cleanline(data[0]).split(' ')
    try:
        J2=float(data[0])
        J3=float(data[1])
    except:
        message="Error while calculating control functional.\n"
        message+=str(jobopts['jfunc_exe'])+ ' returned non-numbers:\n'
        message+=str(data)
        raise RuntimeError(message)


    log.debug('Target populaton= '+str(J1))
    log.debug('Field penalty= '+str(-J2))
    log.info('Total cost-functional J= '+str(J1-J2))

    line=str(jobopts['curriter'])+'   '+str(J1) +'   '+str(-J2)+\
        '   '+str(J1-J2)+' \n'

    dumpfile(jobopts['jfile'],line,True,'a')
    del line

    #save the output
    saveoutputf(jobopts)

    # Check if we are done
    if jobopts['curriter']>=jobopts['niterations']: jobopts['continue']=False

##############################################################################

def anabw(jobopts):

    #save the output
    saveoutputb(jobopts)

    # incrment after backward analysis
    jobopts['curriter']+=1

##############################################################################

def dummy(jobopts):
    pass

##############################################################################

def restartinfo(jobopts,inp,op):
    "Gather information for restarting an optimal control job."

    from messages import log
    import os

    log.debug('Retrieving restart info:')
    #retrieve number of the last completed iteration
    try:
        f=open(jobopts['overlaps'],'r')
        lastline=f.readlines()[-1]
        f.close()
        del f
    except:
        message="Could not read overlaps file"
        log.error(message)
        raise RuntimeError(message)

    from mctdhinp import cleanline
    lastline=cleanline(lastline)
    try:
        jobopts['curriter']=int(lastline.split(' ')[0])
    except:
        message= "In overlaps file: last line does not start with an integer. "
        message+="\nThis can happen when a restart is attempted without the very "
        message+="\nfirst iteration being completed. Examine the overlaps file or "
        message+="\ntry to start a new job"
        raise RuntimeError(message)

    if jobopts['curriter']>=jobopts['niterations']:
        message="Restart at iteration "+str(jobopts['curriter'])
        message+=": No iterations left.\n Modify 'iterations' keyword in OCT-SECTION"
        message+=" or use -n option.\n"
        raise RuntimeError(message)

    log.info('Restart at iteration '+str(jobopts['curriter']))
    try:
        os.chdir(jobopts['outdir'])
    except Exception as E:
        log.exception(E)
        raise RuntimeError('Could not cd to '+jobopts['outdir'])

    #allow overwriting old files from here on
    jobopts['overwrite']=True
    if jobopts['restartf']:jobopts['skipbw']=True


##############################################################################
##############################################################################


def mkdefault(jobopts,inp,op):
    "Select the functions for propagation and analysis"

    parseinput(jobopts,inp,op)


    if not jobopts['restartb'] and not jobopts['restartf']:

        if jobopts['lclear']:
            clearoldoutput(jobopts['outdir'])

        mkinputfiles(jobopts,inp,op)
        startup(jobopts,inp,op)
    else:
        restartinfo(jobopts,inp,op)


    return (forward, anafw, backward, anabw, dummy)



##############################################################################
##############################################################################


