"The logging system for optcntrl"

#########################################################################
#
# FUNCTIONS IN THIS MODULE
#
# - BlockStream               Block output stream for loglevels
# - OpenStream                Open output stream for loglevels
# - logsetup                  Setup for loggong
#
#
# GLOVAL VARIABLES IN THIS MODULE
#
# - log                       A reference to a logger instance
#
# 04/2009 MS
#########################################################################

##############################################################################
# logging and log are initialized in main by calling logsetup
##############################################################################

import logging

log=logging.getLogger('optcntrl.log')

#filters to streams
class BlockStream(logging.Filter):
    """ Log everthing except specified 2 levels"""
    def __init__(self, level,level1):
        self.level = level
        self.level1 = level1
    def filter(self, record):
        return not( self.level == record.levelno or\
            self.level1 == record.levelno)

class OpenStream(logging.Filter):
    """ Log only specified 2 levels"""
    def __init__(self, level,level1):
        self.level = level
        self.level1 = level1
    def filter(self, record):
        return (self.level == record.levelno or\
            self.level1 == record.levelno)

# setting up the logger
def logsetup(logfilename, ldebug=False, mode='w'):
    "Some adjustments for the log file"
    import sys

    if ldebug:
        #Some more detailed info if we are in debug mode.
        loglevel=logging.DEBUG
        logformat='%(asctime)s %(levelname)-8s file= %(filename)-25s '+\
            'line= %(lineno)-6s: %(message)s'
    else:
        #Just pass infos and errors if we are in normal mode.
        #No need to be detailed here.
        loglevel=logging.INFO
        logformat='%(asctime)s %(levelname)-8s : %(message)s'

    # Basic logging config
    logging.basicConfig(
        level=loglevel,
        format=logformat,
        filename = logfilename,
        filemode = mode,
        datefmt = "[%d.%m.%Y %H:%M:%S]"
        )


    # Stream to stdout
    handler = logging.StreamHandler(sys.stdout)
    frm = logging.Formatter("%(message)s")
    handler.setFormatter(frm)
    if ldebug:
        handler.addFilter( OpenStream(logging.DEBUG,logging.INFO))
    else:
        handler.addFilter( OpenStream(logging.INFO,logging.INFO))

    log.addHandler(handler)

    # Stream to stderr
    # There are problems in some Python verions if one
    # adds %(funcName)s to the formatter. So I skipped that here
    # although it might have been convenient
    handler = logging.StreamHandler(sys.stderr)
    frm = logging.Formatter("\n %(levelname)s: \n %(message)s\n")
    handler.setFormatter(frm)
    handler.addFilter( BlockStream(logging.DEBUG,logging.INFO))
    log.addHandler(handler)


