"The job-parameters"

#########################################################################
#
# FUNCTIONS IN THIS MODULE
#
# - checkexepaths             Check that all needed executables are
#                             in $PATH
# - help                      Print help
# - parsecmdln                Parse the command line options
# - version                   Print version info
#
#
# GLOVAL VARIABLES IN THIS MODULE
#
# - jobopts                   A dictionary containing variables
#
# 04/2009 MS
#########################################################################

# save jobparameters here
jobopts = {}

from optcntrl import __doc__


def version():
    "Print verion info"

    print("OCT-MCTDH: Version:", str(jobopts["mctdh_ver"]))


def help():
    "Print help text"

    print(__doc__)


def parsecmdln(argv):
    """
    parsecmdln(argv): do the parsing of the command line options

    """
    import os

    jobopts["basename"] = argv.pop(0)

    # see if help is requested - if so, print help and die
    idx = -1
    try:
        idx = argv.index("-h")
    except ValueError:
        pass

    if idx >= 0:
        help()
        return False

    idx = -1
    try:
        idx = argv.index("-?")
    except ValueError:
        pass

    if idx >= 0:
        help()
        return False

    idx = -1
    try:
        idx = argv.index("-ver")
    except ValueError:
        pass
    if idx >= 0:
        argv.pop(idx)
        version()
        return False

    # do some basic setup first

    jobopts["startdir"] = os.getcwd()
    jobopts["outdir"] = os.getcwd()

    idx = -1
    try:
        idx = argv.index("-D")
    except ValueError:
        pass
    if idx >= 0 and idx + 1 < len(argv):
        jobopts["name"] = argv[idx + 1]
        argv.pop(idx + 1)
        argv.pop(idx)

    idx = -1
    try:
        idx = argv.index("-n")
    except ValueError:
        pass

    if idx >= 0 and idx + 1 < len(argv):
        try:
            jobopts["niterations"] = int(argv[idx + 1])

        except ValueError:
            message = "The value following -n option must be an integer, not:"
            message += str(argv[idx + 1])
            raise Exception(message)

        argv.pop(idx + 1)
        argv.pop(idx)

    idx = -1
    try:
        idx = argv.index("-deb")
    except ValueError:
        pass
    if idx >= 0:
        jobopts["ldebug"] = True
        argv.pop(idx)
    else:
        jobopts["ldebug"] = False

    idx = -1
    try:
        idx = argv.index("-x")
    except ValueError:
        pass
    if idx >= 0 and idx + 1 < len(argv):
        jobopts["suffix"] = argv[idx + 1]
        argv.pop(idx + 1)
        argv.pop(idx)
    else:
        jobopts["suffix"] = ""

    idx = -1
    try:
        idx = argv.index("-nd")
    except ValueError:
        pass
    if idx >= 0:
        jobopts["lclear"] = False
        argv.pop(idx)
    else:
        jobopts["lclear"] = True

    idx = -1
    try:
        idx = argv.index("-mnd")
    except ValueError:
        pass
    if idx >= 0:
        jobopts["mnd"] = True
        argv.pop(idx)
    else:
        jobopts["mnd"] = False

    idx = -1
    try:
        idx = argv.index("-r")
    except ValueError:
        pass
    if idx >= 0:
        jobopts["restartb"] = True
        jobopts["restartf"] = False
        jobopts["lclear"] = False
        argv.pop(idx)
    else:
        jobopts["restartb"] = False

    idx = -1
    try:
        idx = argv.index("-R")
    except ValueError:
        pass

    if idx >= 0:
        jobopts["restartf"] = True
        jobopts["restartb"] = False
        jobopts["lclear"] = False
        argv.pop(idx)
    else:
        jobopts["restartf"] = False

    idx = -1
    try:
        idx = argv.index("-w")
    except ValueError:
        pass
    if idx >= 0:
        jobopts["overwrite"] = True
        argv.pop(idx)
    else:
        jobopts["overwrite"] = False

    if len(argv) > 0:
        jobopts["inpfile"] = argv[len(argv) - 1]
        argv.pop(len(argv) - 1)
        if jobopts["inpfile"][-4:] != ".inp":
            jobopts["inpfile"] += ".inp"

    else:
        raise Exception("No input file given.")

    if len(argv) > 0:
        from sys import stdout

        opts = " "
        for a in argv:
            opts += str(a) + " "
        raise Exception("Unknown command line options:" + opts)

    return True


###############################################################################


def checkexepaths(progs, jobopts):
    "check is executables are in $PATH. If not raise Exception."

    import os

    #    prognames=[prog+jobopts['mctdh_ver']+jobopts['suffix'] for prog in progs]
    prognames = [prog + jobopts["suffix"] for prog in progs]
    paths = os.environ["PATH"].split(os.pathsep)

    for i in range(len(prognames)):
        lex = False
        for p in paths:
            if os.path.exists(os.path.join(p, prognames[i])):
                lex = True
        if not lex:
            raise Exception("Program " + prognames[i] + " is not in the $PATH")

        jobopts[progs[i] + "_exe"] = prognames[i]
