

#########################################################################
#
# FUNCTIONS IN THIS MODULE
#
# - setup                   Select the module with an algorithm
#                           and call its generator
#
# 04/2009 MS
#########################################################################




def setup(jobopts,inp,op):
    """
    setup(jobopts,inp,op) -> (functions,)

    takes care of writing all temporary input files etc.
    and returns 5 functions that can perform the iterations of
    any algorithm, i.e., setup choses the algorithm to be used.
    All functions have to take jobopts (a dictionary) as argument.
    and to return True if further iterations are needed or False
    if some exit criteria is met
    execfnc may modify jobopts to send messages to later calls
    of the same function.
    """
    from messages import log

########################################################################
#   Check for algorithm defined in input file
########################################################################


    #The Rabitz scheme
    if not inp.hasseckey('octsection','algorithm'):
        inp['octsection']['algorithm'] = 'default'
    if inp['octsection']['algorithm'].lower() == 'default':
        log.info("Algorithm: default (Rabitz)")
        from defaultoct import mkdefault
        jobopts['algorithm']='default'
        inp['octsection'].delparam('algorithm')
        return mkdefault(jobopts,inp,op)

#    Krotov + optional filtering (de Vivie-Riedle)
    elif inp['octsection']['algorithm'].lower() == 'krotov':
        log.info("Algorithm: Krotov")
        from krotovoct import mkkrotov
        jobopts['algorithm']='krotov'
        inp['octsection'].delparam('algorithm')
        return mkkrotov(jobopts,inp,op)

#    genetic (not implemented)
#    elif inp['octsection']['algorithm'].lower() == 'genetic':
#        log.info("Algorithm: Genetic")
#        from geneticoct import mkgenetic
#        jobopts['algorithm']='genetic'
#        inp['octsection'].delparam('algorithm')
#        return mkgenetic(jobopts,inp,op)

    else:
        raise Exception("The algorithm "+inp['octsection']['algorithm']+" is unknown.")



# Just as above a new algorithm can easily be added. Provide
# your own module and import it here.
# Note that the generator function (e.g. mkdefault) has to return
# five functions that are passed on to the calling module:
#
# 1. A function that performs the forward propagation
# 2. A function that does some data analysis on the previous forward
#    propagation
# 3. A function that performs the backward propagation
# 4. A function that does some data analysis on the previous backward
#    propagation
# 5. A function that does some final cleanup/analysis.
#
# If a function is not needed (e.g. a backward propagation in a genetic
# algorithm) then a dummy just containing 'pass' can be returned.
# All returned functions may recieve the jobopts dictionary which should
# provide all necessary information to perform the job (instances of
# mctdhinput and mctdhoper can be stored in jobopts if modifications of
# temporary input files are necessary). The jobopts dictionary is called
# by reference so modifications of its content in a subroutine
# persist after the subroutine has returned. That allows
# information propagation to subsequent iterations until cleanup. jobopts
# can be freely modified by all functions.
#
# Note: the generator function, i.e. the new module, is responsible for error
# and consistancy checking of the input as well as doing the initial setup
# such as generating temporary files, initial states and for performing
# the initial propagation with a guess-field. Also logging should be included
# (see messages.py). To do additional parsing of the input in the first
# place modify modules mctdhinp and mctdheingabe.
