#!/usr/bin/env python
"""
An example module with a test PES
for cluster tools


Contains:

 setup      Preparations: initialize a matrix

 free       Deallocate Matrix
            (through python garbage collection)

 potential  Calculate the PES (V) from a
            coordinate  vector (Q)

            V = Q**T A Q

            (Quadratic Form with matrix A)

            Calls setup it required.
"""

import numpy

# global Amat
Amat = None
ndof = None

def setup(N,rseed=None):
    "setup(N,rseed=None) --> None, N:int, rseed:None|int|array"

    global Amat
    global ndof

    if rseed is not None:
        numpy.random.seed(rseed)

    A = numpy.random.random((N,N)) - 0.5e0
    Amat = (A + A.transpose())/4
    ndof = N
    for i in xrange(ndof):
        Amat[i,i] = abs(Amat[i,i]) + 1.e0



def free():
    "free() --> None. Garbage collection."

    global Amat
    global ndof

    Amat = None
    ndof = None


def potential(Q):
    "potential(Q) --> float, Q:1D-array"

    global Amat
    global ndof

    if Amat is None or ndof is None:
        ndof = len(Q)
        setup(ndof)

    if len(Q) <> ndof:
        message = "Error: in potential(Q): "
        message += "len(Q) <> ndof."
        raise RuntimeError(message)


    tmp = numpy.zeros(ndof,dtype=numpy.float64)
    for i in xrange(ndof):
        tmp[i] = numpy.dot(Amat[i,:],Q)

    return numpy.dot(tmp,Q)

