/*--------------------------------------------------------------------
 * atomic_units.h: header file defining constants of 
 * 'atomic unit physical qantities' used in 
 * calculations of electronic wvefunctions for atoms and molecules.
 *
 * taken from the 1997 (!) edition of the SI Brochure.
 *
 * RSC 9/1996
 */

#ifndef __ATOMIC_UNITS_H
#define __ATOMIC_UNITS_H

/* Atomic units with special names */
#define AUCHARGE   1.60217733E-19   /* elementary charge */
#define AUMASS     9.1093897E-31    /* electron rest mass */
#define AUENERGY   4.3597482E-18    /* hartree energy */
#define AULENGTH   5.29177249E-11   /* bohrradius */
#define AUACTION   1.05457266E-34   /* Planck constant divided by 2 Pi */

/* derived units */
#define AUTIME     2.4188843341E-17 
#define AUFORCE    8.2387295E-8
#define AUVELOCITY 2.18769142E+6    /* 1) */
#define AUMOMENTUM 1.9928534E-34
#define AUEFIELD   5.1422082E+11
#define AUEDIPOLE  8.4783579E-30
#define AUMFLUX    2.35051808E+5
#define AUMDIPOLE  1.85480308E-23   /* 2) */

/*
 * 1) The numerical value of the speed of light, when expressed in atomic
 *    units, is equal to the reciprocal of the fine structure constant
 *    alpha^-1 = 137.0359895 .
 *
 * 2) The atomic unit of magnetic dipole is twice the Bohr magneton mu_B.
 */

/* Other units accepted for use with the SI system */

#define ELECTRONVOLT     1.60217733E-19
#define ATOMICMASS       1.6605402E-27
#define ASTRONOMICALUNIT 1.49597870E+11
/* switching to/from other popular units. */

#define UNANAU (AULENGTH*1E+10)     /* [Angstroem] A   -> a.u. (0.52917725) */
#define UNEVAU (AUENERGY/ELECTRONVOLT) /* [eV],       eV  -> a.u. */
#define UNAMAU (AUMASS/ATOMICMASS)   /* [amu],      amu -> a.u. */ 
#define UNFSAU (AUTIME*1E15)           /* [fs],       fs -> a.u. */
/* To switch from first to second unit divide,
   to switch from second to first unit multiply */

/* Masses (??) */
#define MHAU   (1.008/UNAMAU) /* mass of H in a. u. */
#define MCAU  (12.01/UNAMAU)  /* mass of C in a. u. */

#endif
