
%{
  /* #[^\n]*\n           sh-script style comment */


/* need this for the call to atof() below */
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "y.tab.h"
#include "atomic_units.h"

  /* #define DEBUG */
#define MAX_STR_CONST 100
#define MAX_INCLUDE_DEPTH 10
#define YY_INPUT(buf,result,max_size) \
{ int c; if(yyin==stdin) c = st_input(); else c = fgetc(yyin); \
  result = (c == EOF) ? YY_NULL : (buf[0] = c, 1); \
}
double fac(double x);
FILE *fdinclude;

char string_buf[MAX_STR_CONST];
char *string_buf_ptr;

YY_BUFFER_STATE include_stack[MAX_INCLUDE_DEPTH];
FILE *include_fd[MAX_INCLUDE_DEPTH];
int include_stack_ptr = 0;

int theconstants();
int thevars();
int theusage();

%}

%x c_string
%x c_ccomment
%x c_include
/* %option always-interactive */
%%

read             BEGIN(c_include);
<c_include>[ \t]*\"      /* eat the whitespace */
<c_include>[^ \t\n]+\"   { /* got the include file name */
  if ( include_stack_ptr >= MAX_INCLUDE_DEPTH )
      fprintf( stderr, "Includes nested too deeply\n" );
  else {
    yytext[yyleng-1]='\0';
#ifdef DEBUG
    fprintf( stderr, "Including %s\n",yytext );
#endif
    include_fd[include_stack_ptr] = yyin;
    include_stack[include_stack_ptr++] = YY_CURRENT_BUFFER;
    yyin = fopen( yytext, "r" );
    if ( ! yyin ) {
      include_stack_ptr--;
      yyin = include_fd[include_stack_ptr];
      fprintf( stderr, "Error opening input\n",yytext );
    } else {
      yy_switch_to_buffer(yy_create_buffer( yyin, YY_BUF_SIZE ) );
    }
  }
  BEGIN(INITIAL);
}

<<EOF>> {
  if ( --include_stack_ptr < 0 ) {
    return(TK_QUIT);
  } else {
#ifdef DEBUG
    fprintf( stderr, "End of include\n");
#endif
    yy_delete_buffer( YY_CURRENT_BUFFER );
    yy_switch_to_buffer( include_stack[include_stack_ptr] );
    yyin = include_fd[include_stack_ptr];
  }
}

quit     return(TK_QUIT);

[0-9]+    {
#ifdef DEBUG
  printf( "An integer: %s (%d)\n", yytext, atoi( yytext ) );
#endif
  printf("%d ",atoi( yytext ) );

  yylval.num_int= atoi( yytext ); return(TK_NUM_INT);
}
         
 /* Integers in Hex form */
0x[0-9abcdefABCDEF]+    {
#ifdef DEBUG
  printf( "An integer: %s (%d)\n", yytext, strtol(yytext,NULL,16 ) );
#endif
  printf("%d ",strtol(yytext,NULL,16 ) );

  yylval.num_int= strtol(yytext,NULL,16 ); return(TK_NUM_INT);
}

 /* Integers in Octal form */
0o[0-7]+    {
  printf("%d ",strtol(yytext+2,NULL,8 ) );
  yylval.num_int= strtol(yytext+2,NULL,8 ); return(TK_NUM_INT);
}

 /* Integers in latin form */
0l[MDCLXVImdclxvi]+    {
  printf("%d ",latin_number(yytext+2) );
  yylval.num_int= latin_number(yytext+2); return(TK_NUM_INT);
}


[0-9]+(e|E)("-"|"+")?[0-9]+                  |
[0-9]+"."[0-9]*((e|E)("-"|"+")?[0-9]*)?      |
"."[0-9]+((e|E)("-"|"+")?[0-9]*)? {
#ifdef DEBUG
  printf( "A float: %s (%g)\n", yytext,
	  atof( yytext ) );
#endif
  printf("%g ", atof( yytext ) );
  yylval.num_double= atof( yytext ); return(TK_NUM_DOUBLE);
}

[0-9]+(d|D)("-"|"+")?[0-9]+                  |
[0-9]+"."[0-9]*((d|D)("-"|"+")?[0-9]*)?      |
"."[0-9]+((d|D)("-"|"+")?[0-9]*)? {
  char *tmp;
  for (tmp=yytext; *tmp; tmp++) {
    if (*tmp == 'd') *tmp = 'e';
    if (*tmp == 'D') *tmp = 'E';
  }
#ifdef DEBUG
  printf( "A float: %s (%g)\n", yytext,
          atof( yytext ) );
#endif
  printf("%g ", atof( yytext ) );
  yylval.num_double= atof( yytext ); return(TK_NUM_DOUBLE);
}


"%"    { printf("%% "); return(TK_LAST); }

\"        string_buf_ptr = string_buf; BEGIN(c_string); /* " */
<c_string>\\\"      *string_buf_ptr++ = '\"'; /* " */
<c_string>\\n       *string_buf_ptr++ = '\n';
<c_string>\\t       *string_buf_ptr++ = '\t';
<c_string>\\r       *string_buf_ptr++ = '\r';
<c_string>\\a       *string_buf_ptr++ = '\a';
<c_string>\\b       *string_buf_ptr++ = '\b';
<c_string>\\f       *string_buf_ptr++ = '\f';
<c_string>\"        { *string_buf_ptr = '\0'; BEGIN(INITIAL); /* " */
#ifdef DEBUG
  printf("found string %s.\n",string_buf);
#endif
  printf("%s ",string_buf);
  yylval.string=string_buf; return(TK_STRING);
  }
<c_string>.         *string_buf_ptr++ = yytext[0];


[a-zA-Z][a-zA-Z0-9_]* {
  int il;
#ifdef DEBUG
  printf( "A variable: %s\n", yytext );
#endif
  il = getsym (yytext);
  if (il != TK_CMD) printf( "%s ", yytext );
  if (il == 0)
    il = putvar (yytext);
  return(il);
}

"+"|"-"|"*"|"/"|"^" {
#ifdef DEBUG
   printf( "An operator: %s\n", yytext );
#endif
   printf( "%s ", yytext );
  return(yytext[0]);
}
"="|"("|")" {
   printf( "%s ", yytext );
   return(yytext[0]);
}

"/*"              BEGIN(c_ccomment);

<c_ccomment>.     /* Eat C style comment */
<c_ccomment>"*/"  BEGIN(INITIAL); 
      
[ \t]+          /* eat up whitespace */
\n           {
#ifdef DEBUG
  printf( "Return(NL): %s\n", yytext );
#endif
  return(TK_NL);
}
.           {
#ifdef DEBUG
  printf( "Single character: %s\n", yytext );
#endif
  return(TK_DEADCHAR);
}
        
%%


struct fnct_table
{
  char *name;
  double (*fnct)();
};


struct fnct_table fncts[] = 
{
  "sin", sin,
  "cos", cos,
  "tan", tan,
  "asin", asin,
  "acos", acos,
  "atan", atan,
  "log", log10,
  "ln", log,
  "exp", exp,
  "sqrt", sqrt,
  "fac",fac,
  0, 0
};


int nacc=6;

int latin_number(char *c)
{
  int s = 0;
  int k = 0;
  int d = 10000;
  int sd;
  int i;
  int len = strlen(c);
  for(i=0;i<len;i++) {
    switch(c[i]) {
    case 'M': 
    case 'm': sd = 1000; break;
    case 'D': 
    case 'd': sd =  500; break;
    case 'C':
    case 'c': sd =  100; break;
    case 'L':
    case 'l': sd =   50; break;
    case 'X':
    case 'x': sd =   10; break;
    case 'V':
    case 'v': sd =    5; break;
    case 'I':
    case 'i': sd =    1; break;
    default: fprintf(stderr,"Unknown character %c\n",c[i]);return(0); /* error */
    }
    if(d>sd) {s += k; k = sd;d = sd;}
    else if(sd==d) { k += sd; }
    else if(d<sd) {s -=k; k = sd;d = sd;}
  }
  s += k;
  return(s);
}

int accuracy(int arg)
{
  nacc = arg;
  if((nacc>18) || (nacc<0)) nacc=2;
  printf("\n #accuracy set to %d\n",nacc);
  return(nacc);
}

struct set_table
{
  char *name;
  int (*set)(int);
};

struct set_table sets[] = 
{
  "acc",accuracy,
  0, 0
};


struct con_table
{
  char *name;
  double val;
  char *cinfo;
};

struct con_table cons[] =
{
  "PI",         3.1415926535897,    "Pi",
  "E",          2.7182818284590,      "E",

  "FPC_c",                2.99792458E8, "speed of light in vacuum (def) m/s",
  "FPC_h_BAR",          1.05457266E-34, "Planck constant, reduced (63) J s",
  "FPC_h_BAR_MeVs",      6.5821220E-22, "Planck constant, reduced (20) MeV s",
  "FPC_e_C", 1.60217733E-19, "electron charge magnitude (49) C",
  "FPC_e_ESU", 4.8032068E-10, "electron charge magnitude (15) esu",
  "FPC_hBARc", 197.327053, "conversion constant hbar*c (59) MeV Fm",
  "FPC_hBARc2", 0.38937966, "conversion constant (hbar*c)^2 (23) GeV^2 mbarn",
  "FPC_m_e_kg", 9.1093897E-31, "electron mass (54) kg",
  "FPC_m_e_MeV", 0.51099906, "electron mass (15) MeV/c^2",
  "FPC_m_P_MeV", 938.27231, "proton mass (28) MeV/c^2",
  "FPC_m_P_u", 1.007276470, "proton mass (12) u",
  "FPC_m_P_kg", 1.6726231E-27, "proton mass (10) kg",
  "FPC_m_P_M_E", 1836.152701, "proton mass (37) m_e",
  "FPC_m_D_MeV", 1875.61339, "deuteron mass (57) MeV/c^2",
  "FPC_u_MeV", 931.49432, "unified atomic mass unit (u) (28) MeV/c^2",
  "FPC_u_kg", 1.6605402E-27, "unified atomic mass unit (u) (10) kg",
  "FPC_EPSILON_0", 8.854187817E-12, "permittivity of free space F/m",
  "FPC_MU_0", 12.566370614E-7, "permeability of free space N/A^2",
  "FPC_ALPHA", 1/137.0359895, "fine-structure constant (61)",
  "FPC_r_e", 2.81794092E-15, "classical electron radius (38) m",
  "FPC_LAMBDA_BAR_e", 3.86159323E-13, "electron Compton wavelength (35) m",
  "FPC_a_0", 0.529177249E-10, "Bohr radius (mnucleus= infty) (24) m",
  "FPC_LAMBDA_1EV", 1.23984244E-6, "wavelength of 1 eV/c particle (37) m",
  "FPC_R_INFINITY_EV", 13.6056981, "Rydberg energy (mnucleus = infinity) (40) eV",
  "FPC_SIGMA_0_BARN", 0.66524616, "Thomson cross section (18) barn",
  "FPC_MU_B_MeV_T", 5.78838263E-11, "Bohr magneton (52)  MeV/T",
  "FPC_MU_N_MeV_T", 3.15245166E-14, "nuclear magneton (28) MeV/T",
  "FPC_E_M_e", 1.75881962E11, "electron cyclotron freq./field (53) C/kg (rad/sT)",
  "FPC_E_M_P", 9.5788309E7, "proton cyclotron freq./field (29) C/kg (rad/sT)",
  "FPC_G_SI", 6.67259E-11, "gravitational constant (85) m^3/kgs^2",
  "FPC_G_P", 6.70711E-39, "gravitational constant (86) h_bar c (GeV/c^2)^{-2}",
  "FPC_g", 9.80665 , "standard grav. accel., sea level m/s^2",
  "FPC_N_A", 6.0221367E23 , "Avogadro constant (36)  /mole",
  "FPC_K_B", 1.380658E-23, "Boltzmann constant (12) J/K",
  "FPC_K_B_EV", 8.617385E-5, "Boltzmann constant (73) eV/K",
  "FPC_V_MOLAR", 22.41410E-3, "molar volume, ideal gas at STP (19) m^3/mole",
  "FPC_LAMBDAT", 2.897756E-3, "Wien displacement law constant (24) m K",
  "FPC_SIGMA_SB", 5.67051E-8, "Stefan-Boltzmann constant (19) W/m^2K^4",
  "FPC_G_F", 1.16639E-5, "Fermi coupling constant (2)  GeV^{-2}",
  "FPC_SIN2_THETA_W", 0.23192, "weak mixing angle  (5)  at M_Z",
  "FPC_M_W", 80.22, "W boson mass (26) GeV/c^2",
  "FPC_M_Z0", 91.187, "Z_0 boson mass (7) GeV/c^2",
  "FPC_G_S", 0.117, "strong coupling constant (5))  at M_Z",

  "AUCHARGE",   1.60217733E-19, "1 a.u. charge in SI, elementary charge",
  "AUMASS",     9.1093897E-31,  "1 a.u. mass in SI, electron rest mass",
  "AUENERGY",   4.3597482E-18,  "1 a.u. energy in SI, hartree energy",
  "AULENGTH",   5.29177249E-11, "1 a.u. length in SI, bohrradius",
  "AUACTION",   1.05457266E-34, "1 a.u. action in SI, Planck constant/(2*PI)",
  /* derived units */
  "AUTIME",     2.4188843341E-17, "1 a.u. time in SI",
  "AUFORCE",    8.2387295E-8,     "1 a.u. force in  SI",
  "AUVELOCITY", 2.18769142E+6,    "1 a.u. velocity in SI",
  "AUMOMENTUM", 1.9928534E-34,    "1 a.u. momentum in SI",
  "AUEFIELD",   5.1422082E+11,    "1 a.u. el. field in SI",
  "AUEDIPOLE",  8.4783579E-30,    "1 a.u. el. dipole in SI",
  "AUMFLUX",    2.35051808E+5,    "1 a.u. magn. flux in SI",
  "AUMDIPOLE",  1.85480308E-23,   "1 a.u. magn. flux in  SI",
  /* other units */
  "ELECTRONVOLT",     1.60217733E-19, "1 eV charge in SI",
  "ATOMICMASS",       1.6605402E-27,  "1 amu mass in SI",
  "ASTRONOMICALUNIT", 1.49597870E+11, "1 AE length in SI",
  "MOL"             , 6.0221367E+23,  "Avogadro constant",
/* switching to/from other popular units. */
  "au2an", (AULENGTH*1E+10),         "[a.u.] -> [Angstroem]",
  "an2au", 1.0/(AULENGTH*1E+10),       "[Angstroem] -> [a.u.]",
  "au2ev", (AUENERGY/ELECTRONVOLT),  "[a.u.] -> [eV]",
  "ev2au", 1.0/(AUENERGY/ELECTRONVOLT),"[eV] -> [a.u.]",
  "au2am", (AUMASS/ATOMICMASS),      "[a.u.] -> [atomar mass units]",
  "am2au", 1.0/(AUMASS/ATOMICMASS),    "[atomar mass units] -> [a.u.]",
  "au2fs", (AUTIME*1E15),            "[a.u.] -> [fs]",
  "fs2au", 1.0/(AUTIME*1E15),          "[fs] -> [a.u.]",
  "ev2kc", 23.05,                    "[eV] -> [kcal/mol]",
  "kc2ev", 1.0/23.06035,                "[kcal/mol] -> [eV]",
  "au2kc", 23.06035*(AUENERGY/ELECTRONVOLT),"[a.u.] -> [kcal/mol]",
  "kc2au", (ELECTRONVOLT/AUENERGY)/23.06035,"[kcal/mol] -> [a.u.]",
  "au2ic", 219474.7,                    "[au] -> [1/cm]",
  "ic2au", 1.0/219474.7,                "[1/cm] -> [au]",
  "ev2ic", 8065.54353,                  "[eV] -> [1/cm]",
  "ic2ev", 0.0001239842047,             "[1/cm] -> [eV]",
  "ev2kj", 96.485,                      "[eV] -> [kJoule/mol]",
  "kj2ev", 0.0103643,                   "[kJoule/mol] -> [eV]",
  /* British units of length */
  "in2cm",   2.54,                      "[inch] -> [cm]",
  "ft2m",   0.3048,                     "[feet] -> [cm]",
  "yd2m",   0.9144,                     "[yard] -> [cm]",
  "cm2in",  1.0/2.54,                       "[inch] -> [cm]",
  "m2ft",   1.0/0.3048,                     "[feet] -> [cm]",
  "m2yd",   1.0/0.9144,                     "[yard] -> [cm]",
  NULL,0,NULL,
};     

struct cmd_table
{
  char *name;
  int (*cmd)();
};


struct cmd_table cmds[] = 
{
  "help", theusage,
  "cons", theconstants,
  "vars", thevars,
  0, 0
};


struct var_table {
  char *name;
  double val;
  struct var_table *next;
};

typedef struct var_table var_table;

var_table *vars = (var_table *)0;

int putvar (char *sym_name)
{
  var_table *ptr;
  ptr = (var_table *) malloc (sizeof (var_table));
  ptr->name = (char *) malloc (strlen (sym_name) + 1);
  strcpy (ptr->name,sym_name);
  ptr->val = 0.0;
  ptr->next = (struct var_table *)vars;
  yylval.var = &(ptr->val); 
  vars = ptr;
  return TK_VAR;
}
     
int getsym (char *sym_name)
{
  int ia;
  var_table *pvt;
  for ( ia=0; fncts[ia].name != (char *) 0;ia++)
    if (strcmp (fncts[ia].name,sym_name) == 0) {
	yylval.fnct = fncts[ia].fnct; 
	return TK_FNCT;
    }
  for ( ia=0; cons[ia].name != (char *) 0;ia++)
    if (strcmp (cons[ia].name,sym_name) == 0) {
	yylval.con = cons[ia].val; 
	return TK_CON;
    }
  for ( pvt=vars; pvt != (var_table *) 0;pvt=pvt->next)
    if (strcmp (pvt->name,sym_name) == 0) {
	yylval.var = &(pvt->val); 
	return TK_VAR;
    }
  for ( ia=0; cmds[ia].name != (char *) 0;ia++)
    if (strcmp (cmds[ia].name,sym_name) == 0) {
	yylval.cmd = cmds[ia].cmd; 
	return TK_CMD;
    }
  for ( ia=0; sets[ia].name != (char *) 0;ia++)
    if (strcmp (sets[ia].name,sym_name) == 0) {
	yylval.cmd = sets[ia].set; 
	return TK_SET;
    }

  return 0;
}

int thevars() {
  var_table *pvt;
  int i=1;
  printf("\n");
  for ( pvt=vars; pvt != (var_table *) 0;pvt=pvt->next,i++)
    printf("%-20s%-20g\n",
	   pvt->name,pvt->val);
  return(i);
}

int theconstants() {
  int i;
  printf("\n");
  for (i = 0; cons[i].name != 0; i++)
    {
      printf("%-20s%-20.14g%s\n",
	     cons[i].name,cons[i].val,cons[i].cinfo);
    }
  return(i);
}


double 
fac(double x)
{
  int i;
  double y=1;
  while(x>0.0) y *=(x--);
  return(y);
}

int theusage() {
printf("mcalc is a interactive pocket calculator based on the example in the\n\
info text for the bison parser generator. It allows setting variables.\n\
mcalc is a modification of rsc (Rainer Schorks Calculator) of R. Schork.\n\
\n\
usage: mcalc [<file>]\n\
commands:\n\
'read \"file\"' calculates expressions in <file>.\n\
'cons' shows list of defined constants.\n\
'vars' shows set variables. \n\
'help' shows this text.\n\
'acc <ndigits>' sets the accuracy (= TOTAL number of digits).\n\
\n\
'%%' contains the last result.\n\
'quit' or Cntrl+C will terminate mcalc.\n\
\n\
number formats:\n\
0x...   hexadecimal numbers.\n\
0o...   octal numbers.\n\
0l...   latin numbers.\n\
\n\
Available functions are \n\
  sin, cos, tan, asin, acos, atan, log, ln, exp, sqrt.\n\
Hexadecimal/octal numbers can be set with the 0x/0o prefixes.\n\
");  
return(0);
}
