%{
#include <math.h>  /* For math functions, cos(), sin(), etc. */
#include "mcalc.h"  /* Contains definition of `symrec'        */
#define YYDEBUG 1

  extern int nacc; /* the accuracy */
static double last;
  %}
%union {
  double num_double;  /* For returning numbers. */
  int num_int;
  char *string;
  double *var;      /* value of a VAR */
  double con;       /* value of a CON */
  double (*fnct)();  /* value of a FNCT */
  int (*cmd)();  /* value of a CMD */
  int (*set)(int);  /* value of a SET */
}
     
%token <num_double> TK_NUM_DOUBLE /* Simple double precision number   */
%token <num_int> TK_NUM_INT
%token <string> TK_STRING
%token <var> TK_VAR
%token <con> TK_CON
%token <fnct> TK_FNCT   /* Variable and Function            */
%token <cmd> TK_CMD 
%token <set> TK_SET 
%token TK_QUIT
%token TK_DEADCHAR
%token TK_NL
%token TK_LAST
%type  <num_double> exp
%type  <num_double> sexp uexp mexp aexp
 
/* Grammar follows */

%%

input:   /* empty */
| input line
;


line:
TK_NL
| '?' TK_NL  { theusage();}
| TK_STRING TK_NL { printf("\t= (STRING)\n"); }  
| TK_CMD TK_NL { (*($1))(); }
| TK_SET TK_NUM_INT { (*($1))($2); }
| TK_QUIT { printf("bye...\n"); YYACCEPT; }
| exp TK_NL { printf("\t= %.*g\n",nacc,$1); last = $1;}
| error TK_NL { yyerrok;                                 }
;

exp:
  TK_VAR  '=' exp      { $$ = $3; *($1) = $3;             }
| TK_CON  '=' exp      { $$ = $3;
   yyerror("trying to change constant."); YYERROR; 
}
| TK_FNCT '=' exp      {$$ = $3;
  yyerror("trying to change function."); YYERROR; 
}
| aexp             { $$ = $1;                         }
;

aexp:
mexp
| aexp '+' mexp { $$ = $1 + $3;                    } 
| aexp '-' mexp { $$ = $1 - $3;                    }
; 

mexp:
sexp 
| mexp uexp     { $$ = $1 * $2;                    } 
| mexp '*' sexp { $$ = $1 * $3;                    } 
| mexp '/' sexp { 
  if($3==0) {yyerror("Division by zero."); YYERROR;}
  $$ = $1 / $3; }
;
 
sexp:
uexp              { $$ = $1;                         }
| '-' uexp        { $$ = -$2;                        }
| '+' uexp        { $$ = $2;                         }
| uexp '^' sexp   { $$ = pow ($1, $3);               }
;

uexp:
  TK_NUM_INT             { $$ = (double )$1;            }
| TK_NUM_DOUBLE          { $$ = $1;                     }
| TK_VAR             { $$ = *($1);                      }
| TK_CON             { $$ = $1;                         }
| TK_LAST            { $$ = last;                       }
| TK_FNCT '(' exp ')' { $$ = (*($1))($3);               }
| '(' exp ')'     { $$ = $2;                         }
| TK_DEADCHAR { $$ = 0; yyerror("Unused character."); YYERROR;}
;

/* End of grammar */
%%
