
C **********************************************************************
C *                                                                    *
C *                               FDC (fdc.f)                          *
C *                                                                    *
C * Main routine for the complex symmetric filtering program "fdc".    *
C *                                                                    *
C * Michael Beck                                                  4/00 *
C *                                                                    *
C **********************************************************************


      program fdc

      implicit none

#include "fcglobal.inc"
#include "fcmemory.inc"
#include "fckeyword.inc"

      integer dynbytes(1:3), complexsize, realsize, integersize

C --- READ INPUT FILE ---
      
      call creadinput(path,asciiflag,exactflag,exactname,dampautoflag,
     +               damppoints,outname,econv,tconv,inttime,eminin,
     +               emaxin,edim,dampconst,damptype,autocorr,errortype,
     +               vartype,sigmain,vartype2,sigmain2,epscut,spurcut,
     +               deltat,hamshift,sodflag,proptime,npacket,usepack,
     +               skipauto,hamdim,damptime,refener,intenspack,
     +               imagshift,inittime,error)
      if (error .ne. 0) call fcerrormsg(1,error)

C --- DETERMINE NUMBER OF EXACT ENERGIES INSIDE WINDOW ---

      if (exactflag .eq. 1) then
         call cgetexactlen(exactname,eminin,emaxin,econv,refener,
     +                    maxexactdim,error)
         if (error .ne. 0) call fcerrormsg(7,error)
      endif

C --- COMPUTE ARRAY ADDRESSES ---

      call callocate(exactflag,maxexactdim,edim,hamdim,inttime,deltat,
     +              autocorr,errortype,usepack,autodim,workdim,
     +              usedcmemory,usedrmemory,usedimemory,maxauxdim)

C --- DYNAMICALLY ALLOCATE ARRAYS ---

      dynbytes(1) = integersize()*usedimemory
      dynbytes(2) = realsize()   *usedrmemory
      dynbytes(3) = complexsize()*usedcmemory
      call fccaller(dynbytes)
     
      stop
      end

c-----------------------------------------------------------------------
c                          SUBROUTINE FRECEIVER
c
c This routine is called from "fcaller" and continues the FDC program
c after having dynamically allocated the memory.
c-----------------------------------------------------------------------

      subroutine fcreceiver (mi, mr, mc)

      use op1lib

      implicit none

#include "fcglobal.inc"
#include "fcmemory.inc"

      integer    mi(1:usedimemory)
      real*8     mr(1:usedrmemory)
      complex*16 mc(1:usedcmemory)

C --- READ AUTOCORRELATION FILES ---

      call creadauto(autodim,path,asciiflag,autocorr,npacket,usepack,
     +              skipauto,inittime,deltat,mr(reauto0),mr(imauto0),
     +              mr(reauto1),mr(imauto1),mr(reauto2),mr(imauto2),
     +              error)
      if (error .ne. 0) call fcerrormsg(2,error)

C --- READ EXACT RESULTS IF DESIRED ---

      if (exactflag .eq. 1) then
         call creadexact(exactname,eminin,emaxin,econv,maxexactdim,
     +                  refener,exactdim,mc(exactenergy),
     +                  mc(exactintensity),error)
         if (error .ne. 0) call fcerrormsg(12,error)
      endif

C --- WRITE FOURIER TRANSFORM OF AUTOCORRELATION FUNCTION ---

      if (dampautoflag .eq. 1) then
         call cwritefauto(mr(reauto0),mr(imauto0),autodim,path,outname,
     +                   damptype,damptime,deltat,eminin,emaxin,
     +                   hamshift,econv,sodflag,damppoints,refener,
     +                   error)
         if (error .ne. 0) call fcerrormsg(10,error)
      endif
      
C --- TRANSFORM THE ENERGY WINDOW ---

      call ctrafoenergy(eminin,emaxin,sigmain,sigmain2,deltat,hamshift,
     +                 sodflag,emin,emax,sigma,sigma2)

C --- SHIFT THE IMAGINARY PART OF THE HAMILTONIAN ---

      call cshiftimag(mr(reauto0),mr(imauto0),mr(reauto1),mr(imauto1),
     +               mr(reauto2),mr(imauto2),autodim,imagshift,deltat,
     +               autocorr,usepack)

C --- COMPUTE HAMILTONIAN MATRICES ---

      call ccalchamil(mr(reauto0),mr(imauto0),mr(reauto1),mr(imauto1),
     +               mr(reauto2),mr(imauto2),autodim,edim,hamdim,emin,
     +               emax,deltat,damptime,damptype,autocorr,usepack,
     +               mr(dampvalue),mc(h0),mc(h1),mc(h2),error)
      if (error .ne. 0) call fcerrormsg(3,error)
      
      if (autocorr .eq. 0) then
         call ccalckamil(mr(reauto0),mr(imauto0),autodim,mc(h0),edim,
     +                  hamdim,emin,emax,deltat,damptime,damptype,
     +                  usepack,mc(h1),mc(h2),error,mr(dampvalue),
     +                  mr(dampvalue2))
         if (error .ne. 0) call fcerrormsg(11,error)
      endif

C --- EMPLOY A VARIATIONAL PRINCIPLE TO COMPUTE EIGENENERGIES ---

      call cvariation(mc(h0),mc(h1),mc(h2),hamdim,vartype,sigma,epscut,
     +               workdim,hdim,mc(eigenvec),mc(eigenval),error,
     +               mc(work))
      if (error .ne. 0) call fcerrormsg(5,error)

C --- EMPLOY OTHER VARIATIONAL PRINCIPLE FOR ERROR ESTIMATE ---

      if (errortype .eq. 1) then
         call cvariation(mc(h0),mc(h1),mc(h2),hamdim,vartype2,sigma2,
     +                  epscut,workdim,hdim2,mc(eigenvec2),
     +                  mc(eigenval2),error,mc(work))
         if (error .ne. 0) call fcerrormsg(5,error)
      endif

C --- COMPUTE INTENSITY VECTOR ---

      call ccalcintensvec(mr(reauto0),mr(imauto0),autodim,hamdim,edim,
     +                   emin,emax,deltat,damptime,damptype,usepack,
     +                   intenspack,mc(intensity),error)
      if (error .ne. 0) call fcerrormsg(8,error)

C --- COMPUTE INTENSITIES ---

      if (errortype .eq. 1) then
         call cpvxz(mc(intensity),mc(intensity2),hamdim)
         call ccalcintensity(mc(intensity2),mc(eigenvec2),hamdim,hdim2,
     +                      mc(work))
      endif
      call ccalcintensity(mc(intensity),mc(eigenvec),hamdim,hdim,
     +                   mc(work))

C --- ESTIMATE ERROR OF THE EIGENVALUES ---

      if (errortype .eq. 0) then
         call ccalcerror(mc(h0),mc(h1),mc(h2),mc(eigenvec),hamdim,hdim,
     +                  mc(eigenval),mc(eigenerror),hdimall,mc(work))
      elseif (errortype .eq. 1) then
         call ccalcerror2(mc(eigenval),mc(eigenval2),mc(intensity),
     +                   mc(intensity2),hdim,hdim2,mc(eigenerror),
     +                   mc(intenserr),hdimall,mc(work),mi(assignforw),
     +                   mi(assignback),econv)
      else
         call fcerrormsg(13,1)
      endif

C --- CORRECT ENERGIES ---

      call ccorrectenergy(mc(eigenval),hdimall,deltat,hamshift,autocorr,
     +                   sodflag)

C --- WRITE INPUT PARAMETERS TO OUTPUT FILE ---

      call cwriteparam(outch,path,outname,hdim,edim,autodim,eminin,
     +                emaxin,damptype,autocorr,damptime,proptime,
     +                inttime,hamshift,vartype,sigmain,sigmain2,
     +                errortype,vartype2,epscut,usedcmemory,usedrmemory,
     +                usedimemory,econv,tconv,usepack,deltat,refener,
     +                intenspack,imagshift,inittime,error)
      if (error .ne. 0) call fcerrormsg(6,error)

C --- WRITE ENERGIES AND INTENSITIES TO OUTPUT FILE ---

      call cwriteenergy(outch,path,outname,mc(eigenval),mc(eigenerror),
     +                 mc(intensity),mc(intenserr),mc(exactenergy),
     +                 mc(exactintensity),hdimall,maxauxdim,exactdim,
     +                 eminin,emaxin,errortype,exactflag,spurcut,econv,
     +                 refener,imagshift,error,mc(work+maxauxdim),
     +                 mc(work),mi(assignforw),mi(assignback))
      if (error .ne. 0) call fcerrormsg(9,error)

C --- THAT'S ALL ---

      stop
      end


