
C **********************************************************************
C *                                                                    *
C *                           FILTER (filter.f)                        *
C *                                                                    *
C * Main routine for the filtering program "filter".                   *
C *                                                                    *
C * Michael Beck                                                  5/96 *
C *                                                                    *
C **********************************************************************


      program filter

      implicit none

#include "fglobal.inc"
#include "fmemory.inc"
#include "fkeyword.inc"

      integer dynbytes(1:2), realsize, integersize

C --- READ INPUT FILE ---
      
      call readinput(path,asciiflag,exactflag,exactname,dampautoflag,
     +               damppoints,outname,econv,tconv,inttime,eminin,
     +               emaxin,edim,dampconst,damptype,autocorr,errortype,
     +               vartype,sigmain,vartype2,sigmain2,epscut,spurcut,
     +               deltat,hamshift,sodflag,proptime,npacket,usepack,
     +               skipauto,hamdim,damptime,refener,intenspack,error)
      if (error .ne. 0) call frerrormsg(1,error)

C --- DETERMINE NUMBER OF EXACT ENERGIES INSIDE WINDOW ---

      if (exactflag .eq. 1) then
         call getexactlen(exactname,eminin,emaxin,econv,refener,
     +                    maxexactdim,error)
         if (error .ne. 0) call frerrormsg(7,error)
      endif

C --- COMPUTE ARRAY ADDRESSES ---

      call allocate(exactflag,maxexactdim,edim,hamdim,inttime,deltat,
     +              autocorr,errortype,usepack,autodim,workdim,
     +              usedrmemory,usedimemory,maxauxdim)

C --- DYNAMICALLY ALLOCATE ARRAYS ---

      dynbytes(1) = integersize()*usedimemory
      dynbytes(2) = realsize()   *usedrmemory
      call fcaller(dynbytes)
     
      stop
      end

c-----------------------------------------------------------------------
c                        SUBROUTINE FRECEIVER
c
c This routine is called from "fcaller" and continues the FILTER
c program after having dynamically allocated the memory.
c-----------------------------------------------------------------------

      subroutine freceiver (mi,mr)

      use op1lib, only: cpvxd

      implicit none

#include "fglobal.inc"
#include "fmemory.inc"

      integer mi(1:usedimemory)
      real*8  mr(1:usedrmemory)

C --- READ AUTOCORRELATION FILES ---

      call readauto(autodim,path,asciiflag,autocorr,npacket,usepack,
     +              skipauto,mr(reauto0),mr(imauto0),mr(reauto1),
     +              mr(imauto1),mr(reauto2),mr(imauto2),error)
      if (error .ne. 0) call frerrormsg(2,error)

C --- READ EXACT RESULTS IF DESIRED ---

      if (exactflag .eq. 1) then
         call readexact(exactname,eminin,emaxin,econv,maxexactdim,
     +                  refener,exactdim,mr(exactenergy),
     +                  mr(exactintensity),error)
         if (error .ne. 0) call frerrormsg(12,error)
      endif

C --- WRITE FOURIER TRANSFORM OF AUTOCORRELATION FUNCTION ---

      if (dampautoflag .eq. 1) then
         call writefauto(mr(reauto0),mr(imauto0),autodim,path,outname,
     +                   damptype,damptime,deltat,eminin,emaxin,
     +                   hamshift,econv,sodflag,damppoints,refener,
     +                   error)
         if (error .ne. 0) call frerrormsg(10,error)
      endif
      
C --- TRANSFORM THE ENERGY WINDOW ---

      call trafoenergy(eminin,emaxin,sigmain,sigmain2,deltat,hamshift,
     +                 sodflag,emin,emax,sigma,sigma2)

C --- COMPUTE HAMILTONIAN MATRICES ---

      call calchamil(mr(reauto0),mr(imauto0),mr(reauto1),mr(imauto1),
     +               mr(reauto2),mr(imauto2),autodim,edim,hamdim,emin,
     +               emax,deltat,damptime,damptype,autocorr,usepack,
     +               mr(dampvalue),mr(h0),mr(h1),mr(h2),error)
      if (error .ne. 0) call frerrormsg(3,error)
      
      if (autocorr .eq. 0) then
         call calckamil(mr(reauto0),mr(imauto0),autodim,mr(h0),edim,
     +                  hamdim,emin,emax,deltat,damptime,damptype,
     +                  usepack,mr(h1),mr(h2),error,mr(dampvalue),
     +                  mr(dampvalue2))
         if (error .ne. 0) call frerrormsg(11,error)
      endif

C --- EMPLOY A VARIATIONAL PRINCIPLE TO COMPUTE EIGENENERGIES ---

      call variation(mr(h0),mr(h1),mr(h2),hamdim,vartype,sigma,epscut,
     +               workdim,hdim,mr(eigenvec),mr(eigenval),error,
     +               mr(work))
      if (error .ne. 0) call frerrormsg(5,error)

C --- EMPLOY OTHER VARIATIONAL PRINCIPLE FOR ERROR ESTIMATE ---

      if (errortype .eq. 1) then
         call variation(mr(h0),mr(h1),mr(h2),hamdim,vartype2,sigma2,
     +                  epscut,workdim,hdim2,mr(eigenvec2),
     +                  mr(eigenval2),error,mr(work))
         if (error .ne. 0) call frerrormsg(5,error)
      endif

C --- COMPUTE INTENSITY VECTOR ---

      call calcintensvec(mr(reauto0),mr(imauto0),autodim,hamdim,edim,
     +                   emin,emax,deltat,damptime,damptype,usepack,
     +                   intenspack,mr(intensity),error)
      if (error .ne. 0) call frerrormsg(8,error)

C --- COMPUTE INTENSITIES ---

      if (errortype .eq. 1) then
         call cpvxd(mr(intensity),mr(intensity2),hamdim)
         call calcintensity(mr(intensity2),mr(eigenvec2),hamdim,hdim2,
     +                      mr(work))
      endif
      call calcintensity(mr(intensity),mr(eigenvec),hamdim,hdim,
     +                   mr(work))

C --- ESTIMATE ERROR OF THE EIGENVALUES ---

      if (errortype .eq. 0) then
         call calcerror(mr(h0),mr(h1),mr(h2),mr(eigenvec),hamdim,hdim,
     +                  mr(eigenval),mr(eigenerror),hdimall,mr(work))
      elseif (errortype .eq. 1) then
         call calcerror2(mr(eigenval),mr(eigenval2),mr(intensity),
     +                   mr(intensity2),hdim,hdim2,mr(eigenerror),
     +                   mr(intenserr),hdimall,mr(work),mi(assignforw),
     +                   mi(assignback),econv)
      else
         call frerrormsg(13,1)
      endif

C --- CORRECT ENERGIES ---

      call correctenergy(mr(eigenval),hdimall,deltat,hamshift,autocorr,
     +                   sodflag)

C --- WRITE INPUT PARAMETERS TO OUTPUT FILE ---

      call writeparam(outch,path,outname,hdim,edim,autodim,eminin,
     +                emaxin,damptype,autocorr,damptime,proptime,
     +                inttime,hamshift,vartype,sigmain,sigmain2,
     +                errortype,vartype2,epscut,usedrmemory,
     +                usedimemory,econv,tconv,usepack,deltat,refener,
     +                intenspack,error)
      if (error .ne. 0) call frerrormsg(6,error)

C --- WRITE ENERGIES AND INTENSITIES TO OUTPUT FILE ---

      call writeenergy(outch,path,outname,mr(eigenval),mr(eigenerror),
     +                 mr(intensity),mr(intenserr),mr(exactenergy),
     +                 mr(exactintensity),hdimall,maxauxdim,exactdim,
     +                 eminin,emaxin,errortype,exactflag,spurcut,econv,
     +                 refener,error,mr(work+maxauxdim),mr(work),
     +                 mi(assignforw),mi(assignback))
      if (error .ne. 0) call frerrormsg(9,error)

C --- THAT'S ALL ---

      stop
      end

