
C **********************************************************************
C *                                                                    *
C *                           UTILINP (utilinp.f)                      *
C *                                                                    *
C * Program package containing utility subroutines for reading the     *
C * input.                                                             *
C *                                                                    *
C * Michael Beck                                                  4/98 *
C *                                                                    *
C **********************************************************************


C ----------------------------------------------------------------------
C                          SUBROUTINE GETSPARA
C
C PURPOSE:
C     Returns the character string parameter that belongs to a given
C     keyword in the input file.
C INPUT PARAMETERS:
C     input, inplength: see "fkeyword.inc"
C     key: the keyword to search for
C OUTPUT PARAMETERS:
C     keyword, param, paralen, status, found, found2: see "fkeyword.inc"
C     var: the variable (character string) to store the parameter in
C     def: the default to be used if the keyword is not given
C ----------------------------------------------------------------------

      subroutine getspara (key, var, def)

      implicit none

#include "fkeyword.inc"

      character*(*) key, var, def

      integer parapos, cmdpos, i

C --- INITIALISE VARIABLES ---

      if (initial) then
         do i = 0,maxkeys
            found(i)  = .false.
            found2(i) = .false.
         enddo
         counter = 0
         initial = .false.
      endif
      counter = counter+1
      keyword(counter) = key(1:len(key))

C --- SEARCH FOR KEYWORD ---
      
      call getpara(input,inplength,key,param(counter),paralen(counter),
     +             parapos)
      if (parapos .ne. 0) found(parapos) = .true.
      call getpara(cmdline,cmdlength,key,param(counter),
     +             paralen(counter),cmdpos)
      if (cmdpos .ne. 0) found2(cmdpos) = .true.

C --- SET VARIABLE ---

      if (found(parapos) .or. found2(cmdpos)) then
         var = param(counter)(1:paralen(counter))
         if (found(parapos)) status(counter) = 2
         if (found2(cmdpos)) status(counter) = 4
      else
         var = def
         param(counter) = def
         paralen(counter) = len(def)
         status(counter) = 0
      endif

C --- RETURN TO CALLING ROUTINE ---

      return
      end

C ----------------------------------------------------------------------
C                          SUBROUTINE GETRPARA
C
C PURPOSE:
C     Returns the real*8 parameter that belongs to a given keyword in
C     the input file.
C INPUT PARAMETERS:
C     input, inplength: see "fkeyword.inc"
C     key: the keyword to search for
C OUTPUT PARAMETERS:
C     keyword, param, paralen, status, found, found2: see "fkeyword.inc"
C     var: the variable (real*8) to store the parameter in
C     def: the default to be used if the keyword is not given
C ----------------------------------------------------------------------

      subroutine getrpara (key, var, def)

      implicit none

#include "fkeyword.inc"

      real*8        var
      character*(*) key, def

      integer parapos, cmdpos, error, i

C --- INITIALISE VARIABLES ---

      error = 0
      if (initial) then
         do i = 0,maxkeys
            found(i)  = .false.
            found2(i) = .false.
         enddo
         counter = 0
         initial = .false.
      endif
      counter = counter+1
      keyword(counter) = key(1:len(key))

C --- SEARCH FOR KEYWORD ---
      call getpara(input,inplength,key,param(counter),paralen(counter),
     +             parapos)
      if (parapos .ne. 0) found(parapos) = .true.
      call getpara(cmdline,cmdlength,key,param(counter),
     +             paralen(counter),cmdpos)
      if (cmdpos .ne. 0) found2(cmdpos) = .true.

C --- SET VARIABLE ---

      if (found(parapos) .or. found2(cmdpos)) then
         read(param(counter),*,iostat=error) var
         if (found(parapos)) status(counter) = 2
         if (found2(cmdpos)) status(counter) = 4
      else
         read(def,*,iostat=error) var
         param(counter) = def
         paralen(counter) = len(def)
         status(counter) = 0
      endif

C --- WRITE ERROR MESSAGE IF NECESSARY ---

      if (error .ne. 0) then
         error = 1
         call inperr(counter,error)
      endif

C --- RETURN TO CALLING ROUTINE ---

      return
      end

C ----------------------------------------------------------------------
C                          SUBROUTINE GETIPARA
C
C PURPOSE:
C     Returns the integer parameter that belongs to a given keyword in
C     the input file.
C INPUT PARAMETERS:
C     input, inplength: see "fkeyword.inc"
C     key: the keyword to search for
C OUTPUT PARAMETERS:
C     keyword, param, paralen, status, found, found2: see "fkeyword.inc"
C     var: the variable (integer) to store the parameter in
C     def: the default to be used if the keyword is not given
C ----------------------------------------------------------------------

      subroutine getipara (key, var, def)

      implicit none

#include "fkeyword.inc"

      integer       var
      character*(*) key, def

      integer parapos, cmdpos, error, i

C --- INITIALISE VARIABLES ---

      error = 0
      if (initial) then
         do i = 0,maxkeys
            found(i)  = .false.
            found2(i) = .false.
         enddo
         counter = 0
         initial = .false.
      endif
      counter = counter+1
      keyword(counter) = key(1:len(key))

C --- SEARCH FOR KEYWORD ---
      
      call getpara(input,inplength,key,param(counter),paralen(counter),
     +             parapos)
      if (parapos .ne. 0) found(parapos) = .true.
      call getpara(cmdline,cmdlength,key,param(counter),
     +             paralen(counter),cmdpos)
      if (cmdpos .ne. 0) found2(cmdpos) = .true.

C --- SET VARIABLE ---

      if (found(parapos) .or. found2(cmdpos)) then
         read(param(counter),*,iostat=error) var
         if (found(parapos)) status(counter) = 2
         if (found2(cmdpos)) status(counter) = 4
      else
         read(def,*,iostat=error) var
         param(counter) = def
         paralen(counter) = len(def)
         status(counter) = 0
      endif

C --- WRITE ERROR MESSAGE IF NECESSARY ---

      if (error .ne. 0) then
         error = 1
         call inperr(counter,error)
      endif

C --- RETURN TO CALLING ROUTINE ---

      return
      end

C ----------------------------------------------------------------------
C                          SUBROUTINE GETLPARA
C
C PURPOSE:
C     Returns as an integer the position of a parameter in a list.
C INPUT PARAMETERS:
C     input, inplength: see "fkeyword.inc"
C     key: the keyword to search for
C     list: a list of possible parameters
C OUTPUT PARAMETERS:
C     keyword, param, paralen, status, found, found2: see "fkeyword.inc"
C     var: the position (integer) in "list" of the parameter belonging
C          to "keyword" (1st position = 0, 2nd position = 1, etc.)
C     def: the default to be used if the keyword is not given
C ----------------------------------------------------------------------

      subroutine getlpara (key, var, list, def)

      implicit none

#include "fkeyword.inc"

      integer       var
      character*(*) key, list, def

      logical streq
      integer parapos, cmdpos, error, listlen, listpos, first, last,
     +        element, i

C --- INITIALISE VARIABLES ---

      error = 0
      if (initial) then
         do i = 0,maxkeys
            found(i)  = .false.
            found2(i) = .false.
         enddo
         counter = 0
         initial = .false.
      endif
      counter = counter+1
      keyword(counter) = key(1:len(key))
      listlen = len(list)

C --- SEARCH FOR KEYWORD ---
      
      call getpara(input,inplength,key,param(counter),paralen(counter),
     +             parapos)
      if (parapos .ne. 0) found(parapos) = .true.
      call getpara(cmdline,cmdlength,key,param(counter),
     +             paralen(counter),cmdpos)
      if (cmdpos .ne. 0) found2(cmdpos) = .true.

C --- USE INPUT-PARAMETER OR (IF NOT PRESENT) DEFAULT ---

      if (found(parapos) .or. found2(cmdpos)) then
         if (found(parapos)) status(counter) = 2
         if (found2(cmdpos)) status(counter) = 4
      else
         param(counter)   = def
         paralen(counter) = len(def)
         status(counter)  = 0
      endif
         
C --- COMPARE INPUT/DEFAULT-PARAMETER WITH EACH PARAMETER IN LIST ---

      first = 1
      listpos = 1
      element = 0
 100  continue
      listpos = listpos+1
      if (listpos .eq. listlen) then
         last = listpos
         if (streq(param(counter)(1:paralen(counter)),list(first:last)))
     +      then
            var = element
         else
            error = 2
         endif
         goto 200
      elseif (list(listpos:listpos) .eq. ',') then
         last = listpos-1
         if (streq(param(counter)(1:paralen(counter)),list(first:last)))
     +      then
            var = element
            goto 200
         endif
         first   = last+2
         element = element+1
      endif
      goto 100
      
C --- WRITE ERROR MESSAGE IF NECESSARY ---

 200  continue
      if (error .ne. 0) then
         call inperr(counter,error)
      endif

C --- RETURN TO CALLING ROUTINE ---

      return
      end

C ----------------------------------------------------------------------
C                            SUBROUTINE CHRDEF
C
C PURPOSE:
C     Changes the default of a (real*8) parameter (unless it is
C     specified in the input).
C INPUT PARAMETERS:
C     keyword, status, param, paralen: see "fkeyword.inc"
C     key: the keyword to be changed
C     newdef: the new (real*8) default value
C OUTPUT PARAMETERS:
C     var: the new value for the variable corresponding to "key"
C ----------------------------------------------------------------------

      subroutine chrdef (key, var, newdef)

      implicit none

#include "fkeyword.inc"

      real*8        var, newdef
      character*(*) key

      logical streq
      integer keynum, keylen

C --- SEARCH THE KEYWORD AND CHANGE THE DEFAULT ---

      keynum = 0
 100  continue
      keynum = keynum+1
      keylen = index(keyword(keynum),' ')-1
      if (streq(key(1:len(key)),keyword(keynum)(1:keylen))) then
         if ((status(keynum) .le. 1) .or. (status(keynum) .eq. 3) .or.
     +       (status(keynum) .eq. 5)) then
            if (newdef .ge. 0.0d0) then
               write(param(keynum)(1:20),'(e20.14e2)') newdef
            else
               write(param(keynum)(1:20),'(e20.13e2)') newdef
            endif
            paralen(keynum) = 20
            var = newdef
            if (status(keynum) .ge. 1) status(keynum) = 7
         endif
         goto 200
      endif
      if (keynum .lt. counter) goto 100

C --- RETURN TO CALLING ROUTINE ---

 200  continue
      return
      end

C ----------------------------------------------------------------------
C                            SUBROUTINE CHIDEF
C
C PURPOSE:
C     Changes the default of an integer parameter (unless it is
C     specified in the input).
C INPUT PARAMETERS:
C     keyword, status, param, paralen: see "fkeyword.inc"
C     key: the keyword to be changed
C     newdef: the new integer default value
C OUTPUT PARAMETERS:
C     var: the new value for the variable corresponding to "key"
C ----------------------------------------------------------------------

      subroutine chidef (key, var, newdef)

      implicit none

#include "fkeyword.inc"

      integer       var, newdef
      character*(*) key

      logical streq
      integer keynum, keylen

C --- SEARCH THE KEYWORD AND CHANGE THE DEFAULT ---

      keynum = 0
 100  continue
      keynum = keynum+1
      keylen = index(keyword(keynum),' ')-1
      if (streq(key(1:len(key)),keyword(keynum)(1:keylen))) then
         if ((status(keynum) .le. 1) .or. (status(keynum) .eq. 3) .or.
     +       (status(keynum) .eq. 5)) then
            if (newdef .ge. 0) then
               write(param(keynum),'(i6.6)') newdef
            else
               write(param(keynum),'(i6.5)') newdef
            endif
            paralen(keynum) = 6
            var = newdef
            if (status(keynum) .ge. 1) status(keynum) = 7
         endif
         goto 200
      endif
      if (keynum .lt. counter) goto 100

C --- RETURN TO CALLING ROUTINE ---

 200  continue
      return
      end

C ----------------------------------------------------------------------
C                            SUBROUTINE CHLDEF
C
C PURPOSE:
C     Changes the default of a list parameter (unless it is specified in
C     the input).
C INPUT PARAMETERS:
C     keyword, status, param, paralen: see "fkeyword.inc"
C     key: the keyword to be changed
C     list: a list of possible parameters
C     newdef: the new default value
C OUTPUT PARAMETERS:
C     var: the new value for the variable corresponding to "key"
C ----------------------------------------------------------------------

      subroutine chldef (key, var, list, newdef)

      implicit none

#include "fkeyword.inc"

      integer       var
      character*(*) key, list, newdef

      logical streq
      integer keynum, keylen, first, last, listpos, element

C --- INITIALISE VARIABLES ---

      first   = 0
      last    = 0
      element = 0

C --- SEARCH THE KEYWORD ---

      keynum = 0
 100  continue
      keynum = keynum+1
      keylen = index(keyword(keynum),' ')-1
      if (streq(key(1:len(key)),keyword(keynum)(1:keylen))) then
         if ((status(keynum) .le. 1) .or. (status(keynum) .eq. 3) .or.
     +       (status(keynum) .eq. 5)) then

C       --- COMPARE NEW DEFAULT WITH EACH PARAMETER IN LIST ---

            first   = 1
            listpos = 1
            element = 0
 200        continue
            listpos = listpos+1
            if (listpos .eq. len(list)) then
               last = listpos
               goto 300
            elseif (list(listpos:listpos) .eq. ',') then
               last = listpos-1
               if (streq(newdef(1:len(newdef)),list(first:last)))
     +            goto 300
               first   = last+2
               element = element+1
            endif
            goto 200
         else
            goto 400
         endif
      endif
      if (keynum .lt. counter) goto 100

C --- CHANGE DEFAULT ---

 300  continue
      param(keynum)   = list(first:last)
      paralen(keynum) = last-first+1
      var             = element
      if (status(keynum) .ge. 1) status(keynum) = 7

C --- RETURN TO CALLING ROUTINE ---

 400  continue
      return
      end

C ----------------------------------------------------------------------
C                           SUBROUTINE IGNOREKEY
C
C PURPOSE:
C     Labels a keyword which is only relevant in combination with
C     another keyword as "ignored" if the other keyword has a "wrong"
C     value.
C INPUT PARAMETERS:
C     keyword, status: see "fkeyword.inc"
C     key: the keyword to ignore
C OUTPUT PARAMETERS:
C     none
C ----------------------------------------------------------------------

      subroutine ignorekey (key)

      implicit none

#include "fkeyword.inc"

      character*(*) key

      logical streq
      integer keynum, keylen

C --- SEARCH AND IGNORE THE KEYWORD ---

      keynum = 0
 100  continue
      keynum = keynum+1
      keylen = index(keyword(keynum),' ')-1
      if (streq(key(1:len(key)),keyword(keynum)(1:keylen))) then
         if ((status(keynum) .eq. 0) .or. (status(keynum) .eq. 2) .or.
     +       (status(keynum) .eq. 4)) status(keynum) = status(keynum)+1
         goto 200
      endif
      if (keynum .lt. counter) goto 100

C --- RETURN TO CALLING ROUTINE ---

 200  continue
      return
      end

C ----------------------------------------------------------------------
C                         SUBROUTINE COMPULSORY
C
C PURPOSE:
C     Checks whether a keyword is given in the input file and writes an
C     error message if it is not present.
C INPUT PARAMETERS:
C     keyword, status: see "fkeyword.inc"
C     key: the compulsory keyword to look for
C OUTPUT PARAMETERS:
C     none
C ----------------------------------------------------------------------

      subroutine compulsory (key)

      implicit none

#include "fkeyword.inc"

      character*(*) key

      logical streq
      integer keynum, keylen

C --- LOOK FOR THE KEYWORD ----

      keynum = 0
 100  continue
      keynum = keynum+1
      keylen = index(keyword(keynum),' ')-1
      if (streq(key(1:len(key)),keyword(keynum)(1:keylen))) then
         if ((status(keynum) .eq. 2) .or. (status(keynum) .eq. 4)) then
            status(keynum) = 6
         else
            call inperr(keynum,3)
         endif
         goto 200
      endif
      if (keynum .lt. counter) goto 100

C --- RETURN TO CALLING ROUTINE ---

 200  continue
      return
      end

C ----------------------------------------------------------------------
C                          SUBROUTINE CHECKINPUT
C
C PURPOSE:
C     Checks whether unknown keywords are specified in the input file.
C INPUT PARAMETERS:
C     found, found2: see "fkeyword.inc"
C     entries: number of entries in the input file
C     cmdentries: number of entries in the command line
C OUTPUT PARAMETERS:
C     none
C ----------------------------------------------------------------------

      subroutine checkinput (entries, cmdentries)

      implicit none

#include "fkeyword.inc"

      integer entries, cmdentries

      integer      keynum, keylen
      character*80 keyw

C --- LOOK FOR UNIDENTIFIED KEYWORDS IN INPUT FILE ----
      
      do keynum = 1,entries
         if (.not. found(keynum)) then
            call getkeyword(input,inplength,keynum,keyw,keylen)
            write(6,'(2a)')
     +         'Warning: Unknown keyword specified in input file: ',
     +         keyw(1:keylen)
         endif
      enddo

C --- LOOK FOR UNIDENTIFIED KEYWORDS IN COMMAND LINE ----
      
      do keynum = 1,cmdentries
         if (.not. found2(keynum)) then
            call getkeyword(cmdline,cmdlength,keynum,keyw,keylen)
            write(6,'(2a)')
     +         'Warning: Unknown keyword specified in command line: ',
     +         keyw(1:keylen)
         endif
      enddo

C --- RETURN TO CALLING ROUTINE ---

      return
      end

C ----------------------------------------------------------------------
C                         SUBROUTINE CPINPUT
C
C PURPOSE:
C     Writes the input/default parameters to a file.
C INPUT PARAMETERS:
C     keyword, param, status, counter: see "fkeyword.inc"
C     path: path to directory containing the autocorrelation files
C     outname: name of the output files
C OUTPUT PARAMETERS:
C     error: error code having the following meaning:
C            0: everything is o. k.
C            1: an I/O-error occured
C            2: can't open file
C ----------------------------------------------------------------------

      subroutine fdcpinput (path, outname, error)

      implicit none

#include "fkeyword.inc"

      integer inpch
      parameter (inpch = 1)

      integer       error
      character*(*) path, outname

      integer       pathlen, outlen, filelen, keylen, i, j
      character*240 filename
      character*20  statname(0:7)
      data statname /'  #  default',   '  #  default/ignored',
     +               '  #  input',     '  #  input/ignored',
     +               '  #  cmdline',   '  #  cmdline/ignored',
     +               '  #  compulsory','  #  computed'/

C --- OPEN FILE WITH COPY OF INPUT ---

      pathlen  = index(path,' ')-1
      outlen   = index(outname,' ')-1
      filename = path(1:pathlen)//'/'//outname(1:outlen)//'.inp'
      filelen  = index(filename,' ')-1
      open(inpch,file=filename(1:filelen),form='formatted',
     +     status='unknown',err=20)

C --- WRITE ALL KEYWORDS AND PARAMETERS TO A FILE ---

      write(inpch,'(a,20x,a,16x,a)') '# Keyword', 'Parameter', 'Status'
      do i = 1,counter
         keylen = index(keyword(i),' ')-1
         write(inpch,*,err=10) ' ', keyword(i)(1:keylen), 
     +      (' ', j=1,24-keylen), ' = ', param(i)(1:paralen(i)),
     +      (' ', j=1,max(0,20-paralen(i))), statname(status(i))
      enddo

C --- CLOSE FILE AND RETURN TO CALLING ROUTINE ---

      close(inpch)
      return

C --- SET ERROR FLAG ---

 10   continue
      error = 1
      return

 20   continue
      error = 2
      return

      end

C ----------------------------------------------------------------------
C                           SUBROUTINE INPERR
C
C PURPOSE:
C     Writes a wrong input entry and an error message to the screen.
C INPUT PARAMETERS:
C     keyword, param: see "fkeyword.inc"
C     keynum: number of keyword
C     error: specifies the error that has occured
C OUTPUT PARAMETERS:
C     none
C ----------------------------------------------------------------------

      subroutine inperr (keynum, error)

      implicit none

#include "fkeyword.inc"

      integer keynum, error

      integer keylen

C --- ECHO ERROR MESSAGE ---

      keylen  = index(keyword(keynum),' ')-1
      write(6,*)
      if (error .eq. 1) then
         write(6,'(a)')
     +      'Input error: cannot convert parameter into number'
         write(6,'(2a)') 'Keyword:   ', keyword(keynum)(1:keylen)
         write(6,'(2a)') 'Parameter: ', param(keynum)(1:paralen(keynum))
      elseif (error .eq. 2) then
         write(6,'(a)') 'Input error: unknown parameter specified'
         write(6,'(2a)') 'Keyword:   ', keyword(keynum)(1:keylen)
         write(6,'(2a)') 'Parameter: ', param(keynum)(1:paralen(keynum))
      elseif (error .eq. 3) then
         write(6,'(a)')  'Input error: compulsory keyword missing'
         write(6,'(2a)') 'Keyword:   ', keyword(keynum)(1:keylen)
      endif
      write(6,*)

C --- STOP THE PROGRAM ---

      stop
      end
