      module efinput

use decimal, only: long, dop,sip
use op1lib, only: zerovxd, cpvxd, zerovxl

use global
use aglobal
use globmemmod
use maxv, only:maxdim,maxsta
use versions
use psidef
use channels
use oct
use runpropmod
use compdatmod
use compdat1mod

use logdat
use iorst
use rdfilesmod
use rddvrmod
use rdopermod

use module_interface

      implicit None

      private
      public :: ef_input,ef_open,ef_readoperator,ef_loadfncs,ef_getrefwf,&
                ef_getrstwf,ef_rdovlps,ef_openout
      contains

!#######################################################################
!
! ef_routines.F: contains subroutines that read data for efield, such as
!     wavefunctions, fields, command line options, etc.
!                
! ef_input:        Read command line options.
! ef_open:         Open files 
! ef_readoperator: Read oper file
! ef_loadfncs:     Loads penalty and reference function into buffers
! ef_getrefwf:     Read the reference (old) wave function from file
! ef_getrstwf:     Read restart (current) wave function from file
!#######################################################################


!######################################################################
!
!     EF_INPUT
!
! Read command line options for efield 
!
!######################################################################

      subroutine ef_input

      implicit none

      integer(long), external :: myiargc
      integer(long)           :: iarg,ilbl,ierr,i
      character(len=80)       :: buf
      logical(kind=4)         :: liter
  
      filein1   = ' '
      filein    = ' '
      lback     = .false.
      ltarop    = .false.
      lrdoper   = .false.
      lsimprop  = .false.
      lalgnphs  = .false.
      lpnlty    = .false.
      lpnltyfnc = .false.
      lreffnc   = .false.
      llagrfnc  = .false.
      liter     = .false.
      ltdovlp   = .false.

      lwrpop    = .false.
      lwrdipmtx = .false.
      lwrrawfld = .false.
      lwrovlp   = .false.
      loctpc    = .false.
      lkrotov   = .false.

      optlaenge = 1
      extorder  = 2
      options   = ' ' 
      tarop     = ' '
      pnlty     = 0.d0
      iiter     = 0

      iarg = myiargc()
      if (iarg .lt. 1) goto 200 
      
      do i = 1, iarg
         call mygetarg(i,buf)
         ilbl  = index(buf,' ')
         ilbl = min(ilbl,100-optlaenge)
         if(ilbl.gt.0) options = options(1:optlaenge)//buf(1:ilbl) 
         optlaenge = optlaenge + ilbl
      end do
  
      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf .eq. '-b' ) then ! backward propagation
            lback=.true.
         else if( buf(1:3) .eq. '-T ' ) then  ! target is an operator
            ltarop=.true.                     ! (from oper file)
            lrdoper = .true.                  ! (otherwise target STATE)
            i = i+1
            call mygetarg(i,tarop)
         else if( buf(1:4) .eq. '-pc ' ) then ! precictor-corrector
            loctpc = .true.
         else if( buf(1:3) .eq. '-t ' ) then  ! write target population
            lwrpop = .true.
         else if( buf(1:3) .eq. '-m ' ) then  ! write dipole matrix-
            lwrdipmtx = .true.                ! element
         else if( buf(1:3) .eq. '-r ' ) then  ! write raw field
            lwrrawfld = .true.
         else if( buf(1:3) .eq. '-o ' ) then  ! write overlap
            lwrovlp = .true.  
         else if( buf(1:3) .eq. '-p ' ) then  ! simprop mode
            lsimprop = .true.
         else if( buf(1:3) .eq. '-A ' ) then  ! align phases of target
            lalgnphs= .true.
         else if( buf(1:3) .eq. '-L ' ) then  ! lagrange multiplier 
            llagrfnc= .true.                   ! (filter)
         else if( buf(1:3) .eq. '-K ' ) then  ! krotov algorithm
            lkrotov= .true.                  
         else if( buf(1:3) .eq. '-O ' ) then  ! the dipole operator
            lrdoper = .true.
            i = i+1
            call mygetarg(i,operator)
         else if( buf(1:3) .eq. '-i ' ) then ! iteration number
            i = i+1                          ! (just for output files)
            liter=.true.
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) iiter
            if (ierr .ne. 0) then
               write(6,'(2a/1x,a)') ' The argument following the option -i', &
                 ' must be an integer number. Not: ', buf
               stop 1
            end if
         else if( buf(1:3) .eq. '-e ' ) then  ! extrapolation
            i=i+1                             ! order of field 
            call mygetarg(i,buf)               
            read(buf,*,iostat=ierr) extorder
            if (ierr .ne. 0) then
               write(6,'(2a/1x,a)') ' The argument following the option -eo', &
                 ' must be an integer number. Not: ', buf
               stop 1
            end if
            if (extorder .lt.1 .or. extorder .gt. 2) then
               write(6,'(2a/1x,a)') ' The argument following the option -eo', &
                 ' must "0","1: or "2" ', buf
               stop 1
            end if 
         else if( buf(1:3) .eq. '-F ' ) then  ! the penalty factor
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) pnlty 
            lpnlty = .true.
            if (ierr .ne. 0) then
               write(6,'(2a/1x,a)') ' The argument following the option -F', &
                 ' must be a positive real number. Not: ', buf
               stop 
            end if
            if (pnlty .lt. 0.d0) then
               write(6,'(2a/1x,a)') ' The argument following the option -F', &
                 ' must be > 0. Not : ', buf
               stop 1
            end if
         else if( buf(1:3) .eq. '-d ' ) then ! time-dependent overlap          
            ltdovlp= .true.
         else if( buf(1:3) .eq. '-S ' ) then ! the penalty function
            lpnltyfnc=.true.
           
         else if( buf(1:3) .eq. '-R ' ) then ! the reference function
            lreffnc=.true.
           
         else if( buf(1:3) .eq. '-h ' .or. buf(1:3) .eq. '-? ' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,a)') &
       'Purpose: Calculates the electric field for optimal control'
            write(6,'(/1x,a)') 'Usage: efield ' &
        //'[-pc -b -A -T -O -F -S -L -K -R -p -i -e -d -t -m' &
        //' -o -r -ver -h -?]'
            write(6,'(/,a)') ' Options: '
            write(6,'(a)') ' -pc      : Run in predictor-corrector mode.'         
            write(6,'(2a)') ' -b       : Compute the electric', &
                     ' field during a backward propagation.'
            write(6,'(2a)') ' -A       : Align phases of overlaps of', &
                     ' multiple initial and target states'
            write(6,'(11x,a)') ' i.e. optimize 1/N^2|sum_i <Psi_i|Targ,i>|^2'
            write(6,'(11x,a)') ' instead of 1/N sum_i |<Psi_i|Targ,i>|^2'
            write(6,'(2a)') ' -T oper  : Maximise matrix element ', &
                 '<Psi|oper|Psi> of operator oper.'
            write(6,'(2a)') ' -O oper  : Use oper as dipole ', &
                 'operator to compute the field.'            
            write(6,'(2a)') ' -F pnlty : Parameter to', &
                 ' penalize the field strength. Default: pnlty=1.'
            write(6,'(2a)') ' -S       : Load penalty-function from', &
                 ' file pnltyfnc.dat.'
            write(6,'(a)') ' -L       : Load Lagrange-multiplier (filter)'
            write(6,'(a)')  ' -K       : Run the Krotov scheme'
            write(6,'(2a)') ' -R       : Load reference-', &
                 'function from file reffnc.dat.'
            write(6,'(2a)') ' -p       : Do not read old psi file.', &
                 ' Instaed propagate 2 WFs simultaneously.'
            write(6,'(1a)') ' -i iter  : Number of current iteration.'
            write(6,'(2a/11x,a)') ' -e num   : Use polynomial of', &
                 ' order num (0,1,2) to extrapolate the field.', &
                 ' Default: num=2.'
            write(6,'(2a)')' -d       : Use time-dependent', &
                 ' overlap (it target is a WF).'
            write(6,'(2a)') ' -t       : Write target population', &
                 ' during a forward propagation.'
            write(6,'(2a)') ' -m       : Write dipole matrix element', &
                 ' as function of time.'
            write(6,'(2a)') ' -o       : Write overlap with target', &
                 ' as function of time.'
            write(6,'(2a)') ' -r       : Write raw field ', &
                     '-Im<tar|mu|psi> as function of time.'
            write(6,'(1a)') ' -h or -? : Print this help text.'
            write(6,'(1a)') ' -ver     : Print version information.'
            write(6,'(/2a,/2a)') ' The matrix element of the dipole', &
                 ' operator (-O option) formed with the initial', &
                 ' and target state is calculated and the', &
                 ' E-field is evaluated.'
            write(6,'(2a,/2a)')' This routine must be started in the', &
                 ' background before the invocation of MCTDH.', &
                 ' It exchanges information with the running', &
                 ' propagation via FIFOs.' 
            write(6,'(/2a)')' Option -O is mandatory.', &
                 ' Options -T/-A and -p/-pc may not be used together.'
            write(6,'(/2a/a)')' Note: This routine is usually', &
                 ' started by the script optcntrl. Therefore all', &
                 ' file names are hardcoded (see subroutine ef_open).'
            write(6,'(78a1)')  ('-',i=1,78)
            stop
         else if( buf(1:4) .eq. '-ver' ) then ! version
            call wrversion(6)
            stop
         else 
            write(6,'(2a)') 'efield: unknown option : ', buf
            stop 1
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if

!     File names for waves and oper are hardcoded      

 200  continue
      
      routine = "efield, ef_input"
     
      if (iiter .lt. 0 .and. liter) then
         write(message,'(i4)')  iiter
         message="Iteration number correct (-i option)? Using:"//&
            trim(message)
         call warnmsg
      end if

      if (lalgnphs .and. ltarop) then
         lalgnphs=.false.
         message="Opions '-A' and '-T' cannot be used together." &
              //"Unsetting '-A'."
         call warnmsg
      end if

      if (.not. lpnlty) then
         message = "No penalty factor provided (-F). Using 1"
         lpnlty=.true.
         pnlty = 1.d0
         call warnmsg
      end if

      if (abs(pnlty).lt. 1.d-4) then
         message = "Penalty factor (-F option) is very small (<10^-4)."
         call warnmsg
      end if

      if (llagrfnc .and. .not. lkrotov) then
         message = "Option -L (filtering of the field) " &
              //" requires option -K (Krotov algorithm)"
          call errormsg
      end if

      
!     Change the condition for function call if this is needed. 
       if (lwrpop.and.lback) then  
          lwrpop=.false.
          message = "Target population (-t) is not written during." &
               //" backward propagation. Unsetting -t."
          call warnmsg
       end if

      return
      end subroutine ef_input


!######################################################################
!
!     EF_OPEN
!
! Open files for efield
!
!######################################################################

      subroutine ef_open

      implicit none

      integer(long)           :: chkdvr,chkgrd,ierr
      integer(long)           :: dvrlaenge
      character(len=c5)       :: filename
      logical(kind=4)         :: lerr


      operfile= 'initialstate/oper'
      dvrfile = 'initialstate/dvr'
      dname =   'initialstate'

      filename='efield.log'
      open (unit=ilog,file=filename,form='formatted')
      logisopen=.true.

! ---- psi (initialstate) ---
      if (lback) then
         if (.not. lsimprop) then
            filein='forward/psi'
         else
            filein='forward_init/restart'
         endif
      else
         if (.not. lsimprop) then
            filein='forward/restart'
         else
            filein='backward_init/restart'
         endif
      endif

      laein=index(filein,' ')-1
      call abspath(filein,laein)

! ---- psi1 (target) ---
      if (lback) then
         if (.not. lsimprop) then
            filein1='backward/restart'
         else
            filein1='backward_targ/restart'
         endif
      else
         if (.not. lsimprop) then
            filein1='backward/psi'
         else
            filein1='forward_targ/restart'
         endif
      endif

      laein1=index(filein1,' ')-1
      call abspath(filein1,laein1)

!-----------------------------------------------------------------------
! Opening the files.
! target  : filein1, psi1
! initial : filein , psi 
!-----------------------------------------------------------------------
      call fpstartup(filename)
      write(ilog,'(/a78)') filename
      write(ilog,'(2a)')    '# Options: ',options(1:optlaenge)

      if (lback) then
         write(ilog,'(a)') 'Backward propagation'
      else
         write(ilog,'(a)') 'Forward propagation'  
      end if

      routine='efopen (efield.F)'

      filename=filein(1:laein)  ! initialstate 
                                ! psi or restart 
   
      open(ipsi,file=filename,form='unformatted',status='old',iostat=ierr)
      if (ierr.ne.0) then
         message='Unable to open file '//trim(filename)
         call errormsg 
      end if

      write(ilog,'(2a)') 'Opened initialstate:',filein(1:laein)

      filename=filein1(1:laein1) ! restart (current run)
    
      open(ipsi1,file=filename,form='unformatted',status='old',iostat=ierr)
      if (ierr.ne.0) then
         message='Unable to open file '//trim(filename)
         call errormsg 
      end if

      write(ilog,'(2a)') ' Opened targetstate: ',filein1(1:laein1)

!-----------------------------------------------------------------------
! Opening the operator and DVR files.
!-----------------------------------------------------------------------
      dvrlaenge = 0
      opflaenge = 0
      dlaenge   = 0
      if(lrdoper) then
         dlaenge=index(dname,' ')-1
         call abspath(dname,dlaenge)
         opflaenge=index(operfile,' ')-1
         if (operfile(opflaenge:opflaenge) .eq. '/') then
            operfile = operfile(1:opflaenge)//'oper'
            opflaenge = opflaenge + 4
         end if       
         call abspath(operfile,opflaenge)
         dvrlaenge=index(dvrfile,' ')-1
         if (dvrfile(dvrlaenge:dvrlaenge) .eq. '/') then
            dvrfile = dvrfile(1:dvrlaenge)//'dvr'
            dvrlaenge = dvrlaenge + 3
         end if
         call abspath(dvrfile,dvrlaenge)
         open(idvr,file=dvrfile(1:dvrlaenge),form='unformatted',status='old', &
              iostat=ierr)
         if (ierr.ne.0) then
            message='Unable to open file '//trim(filename)
            call errormsg 
         end if

         filename=operfile(1:opflaenge)
         open(unit=ioper,file=filename,form='unformatted',status='old',iostat=ierr)
         if (ierr.ne.0) then
            message='Unable to open file '//trim(filename)
            call errormsg 
         end if
         chkdvr=1
         call dvrinfo(lerr,chkdvr)
         if (lerr) then
            message='Unable to read dvrinfo'
            call errormsg 
         end if
         chkdvr=2
         chkgrd=1
         call operinfo(lerr,chkdvr,chkgrd)
         if (lerr) then
            message='Unable to read operinfo'
            call errormsg 
         end if

      endif

      end subroutine ef_open
      
!#######################################################################
!
!     EF_OPENOUT
!
! Open the output channels for writing on the fly
!
!#######################################################################

      subroutine ef_openout()

      implicit none

      logical(kind=4)   :: fileisopen
      character(len=c5) :: filename

!     output channels that remain open for writing on the fly
      itarfile    = 91
      idipfile    = 92
      iovlpfile   = 93
      irawfldfile = 94 
      
      if (.not. lback .and. lwrpop) then

         inquire(itarfile, OPENED=fileisopen)
         if (fileisopen) close(itarfile)

         filename='targetpop.dat'
         open(itarfile, file=filename,status='unknown')

      end if
      if (lwrdipmtx) then

         inquire(idipfile, OPENED=fileisopen)
         if (fileisopen) close(idipfile)

         if (.not.lback) then
            filename='dipmtxf.dat'
         else
            filename='dipmtxb.dat'
         end if
         open(idipfile, file=filename,status='unknown')
      end if
      if (lwrrawfld) then

         inquire(irawfldfile, OPENED=fileisopen)
         if (fileisopen) close(irawfldfile)

         if (.not.lback) then
            filename='rawfieldf.dat'
         else
            filename='rawfieldb.dat'
         end if
         open(irawfldfile, file=filename,status='unknown')
      end if
      if (lwrovlp.and..not.ltarop) then 

         inquire(iovlpfile, OPENED=fileisopen)
         if (fileisopen) close(iovlpfile)

         if (.not.lback) then
            filename='overlapf.dat'
         else
            filename='overlapb.dat'
         end if
         open(iovlpfile, file=filename,status='unknown')
      end if

      end subroutine ef_openout

!#######################################################################
!
!     EF_READOPERATOR
!
! Read the dvr and oper fieles 
!
!#######################################################################

      subroutine ef_readoperator

      implicit none

      real(dop), dimension(1)      :: rdum,rdum1,rdum2,rdum3
      integer(long)                :: chkdvr, chkgrd
      logical                      :: check

      inquire(idvr,opened=check)
      if (check) close(idvr)
      inquire(ioper,opened=check)
      if (check) close(ioper)

      dvrdata(:) = .false.
      dvrdata(1) = .true.    ! read ort
      dvrdata(2) = .false.   ! read trafo
      dvrdata(3) = .false.   ! read dvrmat (dif1mat, dif2mat)
      dvrdata(4) = .false.   ! read fftp   (fft momentum)
      dvrdata(5) = .true.    ! read hin (fft)
      dvrdata(6) = .true.    ! read rueck (fft)
      dvrdata(7) = .true.    ! read fftfak
      dvrdata(8) = .true.    ! read jsph
      dvrdata(9) = .true.    ! read msph
      dvrdata(10)= .false.   ! read kinsph
      dvrdata(11)= .true.    ! read exphin
      dvrdata(12)= .true.    ! read exprueck
      chkdvr=1

      call rddvr(mr(mrort),rdum1,rdum2,rdum3,mc(mchin),mc(mcrueck), &
                 mi(mifftfak),mc(mcexphin),mc(mcexprueck), &
                 mi(mijsph),mi(mimsph),rdum,chkdvr)

      chkdvr=2
      chkgrd=1
      call rdoper(mr(mrhops),chkdvr,chkgrd)
      call reinitext1d

      return
      end subroutine ef_readoperator

!#######################################################################
!
!     LOADFNCS:
!
!     Load reference and envelope from file
!
!#######################################################################

      subroutine ef_loadfncs(fld,pnltyfnc,reffnc,lagrfnc)

use global
use aglobal
use runpropmod
use oct
use channels

use logdat
use efroutines, only: ef_extrpltefld

      implicit none

      real(dop), dimension(nfin), intent(out) :: pnltyfnc,reffnc,lagrfnc
      real(dop), dimension(nfin,3), intent(out) :: fld
      real(dop), dimension(3) :: Y
      real(dop) :: rdum
      character(len=c5) :: filename
      integer(long) :: i

!     filenames for external functions 
      filpnlty= 'pnltyfnc.dat'
      filref=   'reffnc.dat'
      fillagr=  'lagrfnc.dat'

!     initialize the field
      if (lkrotov .and. lback) then
!     Krotov: we are feeding back the old field
!     during a backward propagation
         if (lsimprop) then
            filename='fieldf_init.dat'
         else
            filename='fieldf.dat'
         end if
         open(idump,file=filename,err=123)
         do i=1,nfin
            read(idump,*,err=124) rdum,fld(i,1)
         end do
         close(idump)
         do i=1,nfin
            fld(i,2)=fld(nfin-i+1,1)
         end do
         call cpvxd(fld(1,2),fld(1,1),nfin)
         call cpvxd(fld(1,2),fld(1,3),nfin)
      else
!     if not krotov set to zero
         call zerovxd(fld,3*nfin)
      end if


      if (llagrfnc .and..not.lback .and. lkrotov) then
         
         filename=fillagr
         open(idump,file=filename,err=123)
         !lagrfnc has less points then other functions
         do i=1,nfin-4
            read(idump,*,err=124) rdum,lagrfnc(i+2)
         end do
         close(idump)
         !extrapolate the lagrangien at missing points
         Y(1)=lagrfnc(3)
         Y(2)=lagrfnc(4)
         Y(3)=lagrfnc(5)
         i=0
         call ef_extrpltefld(Y,2,i)
         lagrfnc(2)=Y(1)
         lagrfnc(1)=Y(2)
         Y(1)=lagrfnc(nfin-2)
         Y(2)=lagrfnc(nfin-3)
         Y(3)=lagrfnc(nfin-4)
         i=0
         call ef_extrpltefld(Y,2,i)
         lagrfnc(nfin-1)=Y(1)
         lagrfnc(nfin)  =Y(2)

      else
         call zerovxd(lagrfnc,nfin)
      end if


      if (lreffnc) then
         filename=filref
         open(idump,file=filename,err=123)
         do i=1,nfin
            read(idump,*,err=124) rdum,reffnc(i)
         end do
         close(idump)
         !reverse if backward propagation
         if (lback) then
            do i=1,nfin/2
               rdum=reffnc(i)
               reffnc(i)=reffnc(nfin-i+1)
               reffnc(nfin-i+1)=rdum
            end do
         end if
      else
         call zerovxd(reffnc,nfin)
      end if


      if (lpnltyfnc) then
         
         filename=filpnlty
         open(idump,file=filename,err=123)
         do i=1,nfin
            read(idump,*,err=124) rdum,pnltyfnc(i)
         end do
         close(idump)
         !reverse if backward propagation
         if (lback) then
            do i=1,nfin/2
               rdum=pnltyfnc(i)
               pnltyfnc(i)=pnltyfnc(nfin-i+1)
               pnltyfnc(nfin-i+1)=rdum
            end do
         end if
      else
         do i=1,nfin
            pnltyfnc(i)=1.d0
         end do
      end if


      return

 123  continue
      routine='ef_loadfncs (efinput.F)'
      message='Cannot open file '//trim(filename)
      call errormsg

 124  continue
      routine='ef_loadfncs (efinput.F)'
      message='Cannot read file '//trim(filename)
      call errormsg

      end subroutine ef_loadfncs


!#######################################################################
!
!     GETREF:
!
!     load the old wave function from file from the previous run
!
!#######################################################################

      subroutine ef_getrefwf(psi,spsi,jindx,itime,agmat,trajst,adgwp,gwpdep)

      implicit none

      integer(long) :: i,ierr,ierr1
      integer(long), dimension(1) :: idum
      integer(long), intent(in) :: itime
      integer(long), dimension(jindxdim), intent(out) :: jindx
      complex(dop), dimension(dgldim), intent(out) :: psi
      complex(sip), dimension(spsidim), intent(out) :: spsi
      logical(kind=4) :: linwf,lerr
      real(dop) :: rdum
      character(len=c5) :: filename
      integer(long) :: chkdvr,chkgrd,chkpsi,chkprp,ichan
      complex(dop), dimension(agmatdim), intent(out)    :: agmat
      complex(dop), dimension(gsmatdim), intent(out)    :: adgwp
      integer(long), dimension(gwpinfodim), intent(out) :: gwpdep
      integer(long), dimension(gwpinfodim), intent(out) :: trajst

!     If we do simultaneous propagations of both states, open the
!     restart file of the 'old' WF which is now propagated the second 
!     time

      if (lsimprop) then
         if (lback) then
            filename='backward_init/restart'
         else
            filename='forward_targ/restart'
         endif
         
         irst = 57
         
         open(irst,file=filename,form='unformatted',status='old',iostat=ierr)
         if (ierr.ne.0) then
            routine='ef_getrefwf (ef_input.F)'
            message='Unable to open file '//trim(filename)
            call errormsg
         end if
         
         chkdvr=2
         chkgrd=2
         chkpsi=2
         chkprp=0 
         lwrlog=.false.
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         if (filever(irst) .lt. 11.0d0) then
            lwrlog=.true.
            read(irst,iostat=ierr) rdum !time, here dummy
            read(irst,iostat=ierr1) (psi(i),i=1,dgldim)
            close(irst)
            if (ierr.ne.0.or.ierr1.ne.0) then
               routine='ef_getrefwf (ef_input.F)'
               message='Unable to read from file '//trim(filename)
               call errormsg
            end if
         else
            call rdrstpsi(rdum,psi,jindx,agmat,idum)
         endif
         
         
      else
!     otherwise open the time-dependent wave function file and use 
!     the saved WF from last run (instead of the restart)
         ichan=ipsi1
         if (lback) ichan=ipsi
         
         if ( itime .eq. 0 ) then 
            do i = 1, nfin-5
               call rdpsi(ichan,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
               if (lend) then
                  routine='ef_getrefwf (ef_input.F)'
                  write(message,*) 'There are only',i, &
                                   ' wavefunction on the psi file'
                  call errormsg
               endif
            enddo
         else
            if (filever(ichan) .lt. 11.0d0) then
               backspace(ichan)
               backspace(ichan)
            else
               call rewtodata(ichan,2)
            endif
         endif
         call rdpsi(ichan,psi,spsi,jindx,agmat,trajst,adgwp,gwpdep)
         if (lend) then
            routine='ef_getrefwf (ef_input.F)'
            write(message,*) 'EOF of psi file found. itime, nfin:',itime, nfin
            call errormsg
         endif
      end if

      end subroutine ef_getrefwf

!#######################################################################
!
!     EF_GETRSTWF:
!
! load the restart wave functions from file from the current run
!
!#######################################################################

      subroutine ef_getrstwf(psi1)

      implicit none

      complex(dop), dimension(dgldim), intent(out) :: psi1(dgldim)
      logical(kind=4) :: linwf,lerr
      real(dop) :: rdum
      character(len=c5) :: filename
      integer(long) :: chkdvr,chkgrd,chkpsi,chkprp
      integer(long) :: ierr, ierr1,i
      integer(long),dimension(1) :: idum
      integer(long), dimension(jindxdim) :: jindx
      complex(dop), dimension(agmatdim)  :: agmat

      if (lsimprop) then
         if (lback) then
            filename='backward_targ/restart'
         else
            filename='forward_init/restart'
         endif
      else
         if (lback) then
            filename='backward/restart'
         else
            filename='forward/restart'
         end if
      end if
      
      irst = 57
      open(irst,file=filename,form='unformatted',status='old',iostat=ierr)
      if (ierr.ne.0) then
         routine='ef_getrstwf (ef_input.F)'
         message='Unable to open file '//trim(filename)
         call errormsg
      end if
      chkdvr=2
      chkgrd=2
      chkpsi=2
      chkprp=0 
      lwrlog=.false.
      call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
      lwrlog=.true.
      if (filever(irst) .lt. 11.0d0) then
         read(irst,iostat=ierr) rdum   !time, here dummy
         read(irst,iostat=ierr1) (psi1(i),i=1,dgldim)
         close(irst)
         if (ierr.ne.0.or.ierr1.ne.0) then
            routine='ef_getrstwf (ef_input.F)'
            message='Unable to read file '//trim(filename)
            call errormsg
         end if
      else
         call rdrstpsi(rdum,psi1,jindx,agmat,idum)
      endif

!     Note, that the restart files are opened and closed
!     every time step. Leaving it open after reading 
!     WF can lead to losing the file pointer after the restart 
!     file has been updated by mctdh. (GNU g77 seems to be OK
!     while the Intel compilers fail.)
!     
!     As rstinfo writes to the log files a flag 

      end subroutine ef_getrstwf

!#######################################################################

      subroutine ef_rdovlps(ovlp,npacket,lcalcnewovlp)

      implicit none

      integer(long), intent(in) :: npacket
      integer(long) :: i,j,ii
      complex(dop), dimension(maxsta), intent(out) :: ovlp
      real(dop), dimension(3*maxsta) :: tmp
      character(len=c5) :: filename
      logical(kind=4), intent(out) :: lcalcnewovlp
      
      lcalcnewovlp=.false.
      j=1
      filename='overlaps'
      open(idump,file=filename,err=100,status='old')
      read(idump,*,end=101) 
      read(idump,*,end=101) 
      
      j=0
 97   continue
      read(idump,*,err=101,end=98)ii,(tmp(i),i=1,3*npacket)
      j=j+1
      goto 97
 98   continue
      close(idump)
      if (j.eq.0) then 
         lcalcnewovlp=.true.
      else
         j=1
         do i=1,npacket
            ovlp(i)=dcmplx(tmp(j),tmp(j+1))
            j=j+3
         end do
      end if

      return

 100  continue
      routine="ef_rdovlps (efinput.F)"
      message="overlaps file could not be opened."
      call errormsg

 101  continue
      routine="ef_rdovlps (efinput.F)"
      message="overlaps file is corrupted"
      call errormsg
      
      end subroutine ef_rdovlps

      end module efinput
