      module efoutput

use decimal, only: long, dop
use global
use aglobal
use globmemmod
use maxv, only: maxdim,maxsta
use datenmod
use oct
use channels
use runpropmod
use griddatmod
use psidef

use efroutines
use logdat
use iorst

      implicit None

      private
      public :: ef_wrfld,ef_wrmtx,ef_wrovlp,ef_wrtargetpop,ef_wrrawfld, &
                ef_wrovlpfile

      contains

!#######################################################################
!
! ef_output.F: contains subroutines that write data for efield, such as
!     the field, target population, etc.
!                

! ef_wrfld:        Writes the field to file
! ef_wrmtx:        Writes the time dependent matrix element to file
! ef_wrovlp:       Writes the time dependent overlap to file
! ef_wrtargetpop:  Writes the time dependent target population
! ef_wrrawfld:     Writes the time dependent output of ef_mkfield
! ef_wrovlpfile:   Writes the final overlap with the target  
!                  after forward propagation
!#######################################################################

      
!#######################################################################
!
!     EF_WRFLD
!     
! Write the complete field to file
!     
!#######################################################################

      subroutine ef_wrfld(fld)

      implicit none

      real(dop), dimension(nfin,3), intent(inout) :: fld
      real(dop) :: time
      character(len=c5) :: filename
      integer(long) :: i,j

      ! some missing values (constant extrapolation)
      fld(1,1)=fld(3,1)
      fld(2,1)=fld(3,1)
      fld(1,2)=fld(4,2)
      fld(2,2)=fld(4,2)
      fld(3,2)=fld(4,2)
      fld(1,3)=fld(5,3)
      fld(2,3)=fld(5,3)
      fld(3,3)=fld(5,3)
      fld(4,3)=fld(5,3)
      fld(nfin-2,1)=fld(nfin-3,1)
      fld(nfin-1,1)=fld(nfin-3,1)
      fld(nfin  ,1)=fld(nfin-3,1)
      fld(nfin-1,2)=fld(nfin-2,2)
      fld(nfin  ,2)=fld(nfin-2,2)
      fld(nfin  ,3)=fld(nfin-1,3)

      if (.not. lsimprop) then

         if (lback)then
            fileout="fieldb.dat"
         else
            fileout="fieldf.dat"
         end if
         laeout=index(fileout,' ')-1
         call abspath(fileout,laeout)
         filename=fileout(1:laeout)
         open(idump,file=filename,form='formatted',status='unknown')
         write(ilog,'(a)') 'Writing field to file: '//fileout(1:laeout)
         do i = 1, nfin
            time =dble(i-3)*out2
            write(idump,'(4e24.16)') time, (fld(i,j),j=1,3)
         end do
         close(idump)

      else

         if(lback) then
            fileout='fieldb_targ.dat'
         else
            fileout='fieldf_init.dat'
         endif
         laeout=index(fileout,' ')-1
         call abspath(fileout,laeout)
         filename=fileout(1:laeout) 
         open(idump,file=filename,form='formatted',status='unknown')
         write(ilog,'(a)')'Writing field to file: '//fileout(1:laeout)
         do i =1, nfin
            time =dble(i-3)*out2
            write(idump,'(4e24.16)') time, (fld(i,j),j=1,3)
         end do
         close(idump)

         ! reverse ordering
         if(lback) then
            fileout='fieldf_targ.dat'
         else
            fileout='fieldb_init.dat'
         endif
         laeout=index(fileout,' ')-1
         call abspath(fileout,laeout)
         filename=fileout(1:laeout) !field
         open(idump,file=filename,form='formatted',status='unknown')
         write(ilog,'(a)')'Writing field to file: '//fileout(1:laeout)
         do i = 1, nfin
            time =dble(i-3)*out2
            write(idump,'(4e24.16)') time, (fld(nfin-i+1,j),j=1,3)
         end do
         close(idump)

      end if
      
      end subroutine ef_wrfld

!#######################################################################
!
!     EF_WRMTX
!     
! Write the time-depenendent matrix elements <psi_p(t)|mu|psi1_p(t)>
! to file.
!     
!######################################################################

      subroutine ef_wrmtx(time,dipmtx,npacket)

      implicit none

      integer(long), intent(in) :: npacket
      integer(long) :: i
      complex(dop), dimension(npacket), intent(in) ::  dipmtx
      real(dop), intent(in) :: time

      write(idipfile,'(17f24.16)') time, &
              (dble(dipmtx(i)),imag(dipmtx(i)),i=1,npacket)

      call myflush(idipfile)

      end subroutine ef_wrmtx

!#######################################################################
!
!     EF_WROVLP
!     
! Write the time-depenendent overlaps <psi_p(t)|psi1_p(t)>
! to file.
!     
!######################################################################

      subroutine ef_wrovlp(time,ovlp,npacket)

      implicit none

      integer(long), intent(in) :: npacket
      integer(long) :: i
      complex(dop), dimension(npacket), intent(in) :: ovlp
      real(dop), intent(in) :: time

      write(iovlpfile,'(17f24.16)') time, &
           (dble(ovlp(i)),imag(ovlp(i)),i=1,npacket)

      call myflush(iovlpfile)

      end subroutine ef_wrovlp

!#######################################################################
!
!     EF_WRRAWFLD
!     
! Write the time-depenendent output of mkfield (efield without any
! functions such as reference and penalty applied) to file.
!     
!######################################################################

      subroutine ef_wrrawfld(time,val1)

      implicit none

      real(dop), intent(in) :: time,val1
      
      write(irawfldfile,'(2f24.16)') time, val1
    
      call myflush(irawfldfile)

      end subroutine ef_wrrawfld

!#######################################################################
!
!     EF_WROVLPFILE
!     
! Append the overlaps <psi_p(T)|chi_p(T)> to the overlaps file
! after forward propagation
!     
!######################################################################

      subroutine ef_wrovlpfile(ovlp,tarovlp,npacket)

      implicit none
      
      integer(long), intent(in) :: npacket
      integer(long) :: j
      complex(dop), dimension(npacket), intent(in) :: ovlp,tarovlp
      complex(dop) :: zsum
      real(dop) :: dsum

      zsum=dcmplx(0.d0,0.d0)
      dsum=0.d0
      if (lalgnphs) then
         do j=1,npacket
            zsum = zsum + ovlp(j)
         end do
         dsum =  abs(zsum)**2/dble(npacket**2)
      else
         do j=1,npacket
            if (ltarop) then
               dsum = dsum + dble(tarovlp(j))
            else
               dsum = dsum + abs(ovlp(j))**2
            end if
         end do    
         dsum = dsum/dble(npacket)
      end if
      open(idump, file='overlaps', access='append') 
      write(idump,'(i8,30f24.16)') iiter, (dble(ovlp(j)), &
           dimag(ovlp(j)),abs(ovlp(j))**2,j=1,npacket), dsum
      close(idump)
      
      end subroutine ef_wrovlpfile
      
!#######################################################################
!
!     EF_TARGETPOP
!     
!     Calculate the target population for a given input wave (psi)
!     
!#######################################################################

      subroutine ef_wrtargetpop(psi,psi1,time,workc)

      implicit none

      integer(long) :: i,idum(1)
      integer(long) :: chkdvr,chkgrd,chkpsi,chkprp,ierr,ierr1,iovl
      complex(dop), dimension(maxsta) :: ovtar
      complex(dop), dimension(dgldim), intent(in) :: psi
      complex(dop), dimension(dgldim), intent(out) :: psi1
      complex(dop) :: zsum
      complex(dop), dimension(workcdim), intent(out) :: workc
      complex(dop), dimension(doverdim) :: dover
      real(dop), intent(in) :: time
      real(dop) :: dsum,rdum
      logical(kind=4) :: lerr,linwf
      character(len=c5) :: filename
      integer(long), dimension(jindxdim) :: jindx
      complex(dop), dimension(agmatdim)  :: agmat

      iovl=58

      if(ltarop) then
!     
         call ef_mtrxelement(ovtar,psi,psi,workc,time, &
              mc(mchpsi),hteil,mr(mrhops),mc(mcdtpsi), &
              mr(mrort),ntarop,mi(mijindx))
!         call ef_mtrxelement(ovtar,psi,psi,workc,time, &
!              mc(mchpsi),mc(mchteil),mr(mrhops),mc(mcdtpsi), &
!              mr(mrort),ntarop,mi(mijindx))
         
      else  
         chkdvr=2
         chkgrd=2
         chkpsi=2
         chkprp=0 
         irst = 57
         filename='targetstate/restart'
         open(irst,file=filename,form='unformatted',status='old',iostat=ierr)
         if (ierr.ne.0) then
            routine='wrtargetpop (efield.F)'
            message='Unable to open file '//filename
            call errormsg
         end if

         lerr=.false.
         linwf=.false.
         lwrlog=.false.
         call rstinfo(linwf,lerr,chkdvr,chkgrd,chkpsi,chkprp)
         lwrlog=.true.
         if (filever(irst) .lt. 11.0d0) then
            read(irst,iostat=ierr) rdum
            read(irst,iostat=ierr1) (psi1(i),i=1,dgldim)
            close(irst)
            if (ierr.ne.0 .or. ierr1.ne.0) then
               routine='wrtargetpop (efield.F)'
               message='Unable read file '//filename
               call errormsg
            end if
         else
            call rdrstpsi(rdum,psi1,jindx,agmat,idum)
         endif

         call ef_povlp(psi,psi1,dover,workc,ovtar)         
      endif
      
      if (.not. lalgnphs) then
         dsum = 0.d0
         do i=1,npacket
            if (ltarop) then
               dsum = dsum + dble(ovtar(i))
            else
               dsum = dsum + abs(ovtar(i))**2
            endif 
         enddo 
         dsum=dsum/dble(npacket)
         if (ltarop) then
            write(itarfile,'(10f24.16)') time,dsum,(dble(ovtar(i)),i=1,npacket)
         else 
            write(itarfile,'(10f24.16)') time,dsum,(abs(ovtar(i))**2,i=1,npacket)
         end if
      else 
         zsum = dcmplx(0.d0,0.d0)
         do i=1,npacket
            zsum = zsum + ovtar(i)
         enddo
         dsum = abs(zsum)**2/dble(npacket**2)
         write(itarfile,'(9f24.16)') time,dsum,(abs(ovtar(i))**2,i=1,npacket)
      endif

      call myflush(itarfile)
      
      
      end subroutine ef_wrtargetpop


      end module efoutput
