

      program fldop

use decimal, only:long,dop,sip
use global
use globmemmod
use versions
use aglobal
use maxv, only:maxdim,maxsta,maxkoe
use griddatmod
use operdef
use runopermod
use channels
use datenmod
use oct

use op1lib, only: cpvxz, zerovxz
use logdat
use heinsort, only: delzero
use findknummod
use loadfld
use fnceingabemod
use fldrdinp
use fldoperate
use fnceintermsmod

use mctdhmod_interface
use module_interface

      implicit none 

      logical(kind=4) ::        linperr,lexist
      integer(long)   ::        ilbl,i,pass
      real(dop)       ::        totmem
      character(len=c5)  ::     filename

      call default
      call mdefault
      
      call fldopinput

      filename='fldop.log'
      ilbl  = index(filename,' ')
      inquire(file=filename(1:ilbl),exist=lexist)
      if (lexist .and. .not. lovwr) then
         write(6,*) "ERROR: In program fldop:"
         write(6,*) "       Unable to open log file:"
         write(6,*) "       "//filename(1:ilbl)
         write(6,*) "       Overwrite disabled? (Try -w option)."
         Stop 1
      end if

      open(unit=ilog,file=filename(1:ilbl),form='formatted')
      logisopen=.true.
      call wrversion(ilog)
      write(ilog,'(2a/)')'Options: ', options(1:optlaenge)

      inquire(file=fileout(1:laeout),exist=lexist)
      if (lexist .and. .not. lovwr) then
         write(6,*) "ERROR: In program fldop:"
         write(6,*) "       Unable to open file:"
         write(6,*) "       "//fileout(1:laeout)
         write(6,*) "       Overwrite disabled? (Try -w option)."
         write(ilog,*) "ERROR:  In program fldop:"
         write(ilog,*) "       Unable to open file:"
         write(ilog,*) "       "//fileout(1:laeout)
         write(ilog,*) "       Overwrite disabled? (Try -w option)."
         Stop 1
      end if

! --- read function definitions 
      pass = 1
      if (lrdfnc) then
         linperr = .false.
         call fnceingabe(linperr,pass)
         if (linperr) then
            routine = 'fldop (reading function definitions)'
            message = 'Finished with input error. See log file.'
            call errormsg
         end if
         call delzero
         call findknum
!         call logop
      end if

! --- read filter rules
      call rdfldop


! --- collect parameters from existing file
      call browsein             
      write(ilog,'(/a)') 'Browsing input file:'
     
      
      write(ilog,'(a,i6)')    '  points: ', nfin
      write(ilog,'(a,f16.8)') '  start : ', tinit
      write(ilog,'(a,f16.8)') '  step  : ', out2
      write(ilog,'(a,f16.8)') '  end   : ', tfinal
      
      
! there is no memory allocated for operator as only 
! 1D analytic functions are used - we only use the labels
! and pointers of the operator to access the opfuncs library. 
! (Memory is in fact allocated when "external1d" is used. 
! But this is taken care of by li_load)
! Here we only allocate memory for the function
! and some workspase      

      mcfld=mcdim
      mcdim=mcdim+nfin
      mcfld1=mcdim
      mcdim=mcdim+nfin

      mcref=mcdim
      mcdim=mcdim+nfin
      

      mcffthin=mcdim
      mcdim=mcdim+nfin !fft hin array
      mcfftrueck=mcdim
      mcdim=mcdim+nfin !fft rueck array
      mcworkc=mcdim
      mcdim=mcdim+nfin !fft work array
      

      mrdim=1

      midim=1
      miworki=midim
      midim=midim+nfin   !fft work space

      msdim=1
      mfdim=1

if(allocated(mc))deallocate(mc)
if(allocated(mr))deallocate(mr)
if(allocated(mi))deallocate(mi)
if(allocated(ml))deallocate(ml)
if(allocated(ms))deallocate(ms)
if(allocated(mf))deallocate(mf)

allocate(mc(mcdim))
allocate(mr(mrdim))
allocate(mi(midim))
allocate(ml(mldim))
allocate(ms(msdim))
allocate(mf(mfdim))

      call wrversion(ilog)
      write(ilog,'(a)') '  *** Memory allocated dynamically ***'
      write(ilog,'(6(a,i12,/),a,f16.0,a,f11.2,a)') &
       ' complex*16 :', mcdim, &
       ' real*8     :', mrdim, &
       ' integer    :', midim, &
       ' logical    :', mldim, &
       ' complex*8  :', msdim, &
       ' real*4     :', mfdim, &
       ' Total   :',totmem,' Bytes ,',totmem/1048576.d0,' MB'
      write(ilog,'(80a1,/)') ('-',i=1,80)
      call myflush(ilog)

       if(ilbl.gt.0) then
         routine='fldop'
         write(message,'(3a,f9.1,a)') &
           'Integer overflow in memory assignment. ', &
           'You are asking for too much memory.    ', &
           'Total Memory asked for =',totmem/1048576.d0,' MB'
         call errormsg
      endif

      call applyfldops(mc(mcfld),mc(mcfld1), mc(mcref),mc(mcffthin), &
           mc(mcfftrueck),mc(mcworkc), mi(miworki))


      deallocate(mc,mr,mi,ml,ms,mf)

      stop
      
      contains

!-----------------------------------------------------------------------

      subroutine applyfldops(field,field1,ref,ffthin, fftrueck, workc,worki)

use global
use aglobal
use maxv, only: maxdim,maxsta,maxkoe
use datenmod
use operdef
use oct
use channels

use logdat

      implicit none

      complex(dop), dimension(nfin) ::    field,field1, workc, ffthin, &
                                          fftrueck,ref 
      real(dop)                   ::       time
      integer(long), dimension(midim) ::   mi
      integer(long), dimension(nfin) ::   worki
      integer(long)                   ::   i
      
      ! load old field
      
      call loadin(field)
    
      call cpvxz(field,ref,nfin)
      call zerovxz(field1,nfin) 
      !apply the filters
      do i=1,nfldop
         if ( fldoptype(i).eq.1) then
            !multiply a function to input field
            call fldmultfnc(field,field1,i)   
         elseif ( fldoptype(i).eq. 2) then
            !scale input to some maximum value
            call fldmaxval(field,field1,i)
         elseif ( fldoptype(i).eq. 3) then
            !apply: FFT^-1 * f(w) * FFT, 
            call fldfilter(field,field1,ffthin,fftrueck,workc,worki,i)
         elseif ( fldoptype(i).eq. 4) then
            !normalize input to some value 
            call fldnorm(field,field1,i)
         elseif ( fldoptype(i).eq. 5) then
            !(anti-)symmetrize the input: f(t)=+/- f(T-t)
            call fldsymmetry(field,field1,i)
         elseif ( fldoptype(i).eq. 6) then
            !reverse: f(t) -> f(T-t)
            call fldreverse(field,field1)
         elseif ( fldoptype(i).eq. 7) then
            !pick Re,Im,Abs,Conjg from field 
            call fldcomplex(field,field1,i)
         elseif ( fldoptype(i).eq. 8) then
            !Multiply input with a real number
            call fldscale(field,field1,i)
         elseif ( fldoptype(i).eq. 9) then
            !add a function to input
            call fldaddfnc(field,field1,i)
         else
            routine='fldop, applyfldops'
            message='Operation type is unknown.'
            call errormsg
         end if
         call cpvxz(field1,field,nfin)
      end do

! --- release the output field
      
      ! 1-P
      if (lminus) then
         do i=1,nfin
            field1(i) = ref(i) - field1(i)
         end do
      end if

      open(iout,file=fileout(1:laeout),form='formatted',err=12)
      if (lback) then
         do i=1,nfin
            time=tinit+dble(i-1)*out2
            write(iout,'(4e24.16)',err=12) time,dble(field1(nfin-i+1)), &
                 dimag(field1(nfin-i+1)),abs(field1(nfin-i+1))
         end do
      else
         do i=1,nfin
            time=tinit+dble(i-1)*out2
            write(iout,'(4e24.16)',err=12) time,dble(field1(i)), &
                 dimag(field1(i)), abs(field1(i))
         end do
      end if
         close(iout)

      return

 12   continue
      routine='fldop, applyfldops'
      message='Unable to open output file '//fileout(1:laeout)
      call errormsg

      end subroutine applyfldops


!-----------------------------------------------------------------------
!-----------------------------------------------------------------------

      subroutine fldopinput()

use global
use aglobal
use maxv, only: maxdim,maxsta,maxkoe
use datenmod
use operdef
use runopermod
use channels
use oct
use keyunitsmod

use logdat

      implicit none

      character(len=80) ::   buf
      integer(long) ::        iarg,myiargc,i,ilbl, ierr,length,length1,lencom
      logical(kind=4) ::        check,lcomma
   
      routine='octfinput (fldop.F)'

      laein     = 0
      laeout    = 0
      inplaenge = 0
      optlaenge = 1

      inputfile = ' '
      operfile  = ' '
      filein    = ' '
      fileout   = ' '

      lrdfnc     = .false.
      lovwr      = .false.
      lminus     = .false.
      lback      = .false.

      iarg = myiargc()
      if (iarg .lt. 1) goto 200 
      
      do i = 1, iarg
         call mygetarg(i,buf)
         ilbl  = index(buf,' ')
         ilbl = min(ilbl,C5-optlaenge)
         if(ilbl.gt.0) options = options(1:optlaenge)//buf(1:ilbl) 
         optlaenge = optlaenge + ilbl
      end do
      
 
      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:3) .eq. '-o ' ) then ! output file (field)
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
          else if( buf(1:3) .eq. '-f ' ) then  !input file (field)
            i = i+1
            call mygetarg(i,buf)
            filein = buf
          else if( buf(1:3) .eq. '-O ' ) then  !operator file (.fnc)
            i = i+1
            call mygetarg(i,buf)
            operfile = buf
            lrdfnc = .true.
         else if( buf(1:3) .eq. '-R ' ) then !write f(T-t)
            lback=.true.     
         else if( buf(1:3) .eq. '-w ' ) then !overwrite
            lovwr=.true.
         else if( buf(1:3) .eq. '-C ' ) then !overwrite
            lminus=.true.
         else if( buf(1:3) .eq. '-p ' ) then !add params
           
            i = i + 1
            call mygetarg(i,buf)
            length  = index(buf,' ')-1
             
            if(buf(1:1).eq.' '.or.buf(1:1).eq.'-') then
               write(6,'(/,a/,a)') 'ERROR in subroutine octfinput (fldop.F):', &
                    ' -p is not followed by a parameter name.'
               stop 1
            else
               npar=npar+1
               apar(npar)(1:length)=buf(1:length)
            end if
            
            i = i + 1
            call mygetarg(i,buf)
  
!........If the comma is not surounded by blanks, lcomma is true.
            lencom  = index(buf,',')-1
            length1  = index(buf,' ')-1
            if( lencom.gt.0 .and. lencom.lt.length1 ) then
               lcomma = .true.
               length1 = lencom 
            else   
               lcomma = .false.   
            endif
            
            read(buf(1:lencom),*,iostat=ierr) rpar(npar)
            if(buf(1:1).eq.' '.or.ierr.ne.0) then
               length  = index(apar(npar),' ')-1
               write(6,'(/,a/,3a)') 'ERROR in subroutine octfinput (fldop.F):', &
                    ' -p ',apar(npar)(1:length),' is not followed by a real number.'
               stop 1
            end if
            if(lcomma) then
               buf = buf(lencom+1:)
            else
               i = i + 1
               call mygetarg(i,buf)
            endif
            if( buf(1:1) .eq. ',' ) then
               if( buf .eq. ',' ) then
                  i = i + 1
                  call mygetarg(i,buf)
               else
                  buf = buf(2:)
               endif
               
               call keylowc(buf)
               call keyunits(rpar(npar),buf,ierr)
               if(buf(1:1).eq.' '.or.buf(1:1).eq.'-'.or.ierr.ne.0) then
                  length  = index(apar(npar),' ')-1
                  write(6,'(/,a/,2a,f10.6,a)')  &
                       'ERROR in subroutine octfinput (fldop.F):', &
                       ' -p ',apar(npar)(1:length),rpar(npar), &
                       ' not followed by a unit.'
                  stop 1
               end if
            else
               i = i - 1
            endif
    
         else if( buf(1:3) .eq. '-h ' .or. buf(1:3) .eq. '-? ' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(1x,a)') &
       'Purpose:  Operate an operator P on a sampled 1D function.'
            write(6,'(/1x,a)') &
       'Usage  : fldop [-f -o -O -p -w -ver -h -?] inputfile'
            write(6,'(/1x,a)') 'Options: '  
            write(6,'(2a)') ' -f ifile : File containing the initial', &
                 ' sampled field.'  
            write(6,'(2a)') ' -o ofile : Name of the output file', &
                 ' (rather then ifile1).'  
            write(6,'(2a)') ' -O file  : File containing function', &
                 ' definitions'
            write(6,'(2a)') ' -C       : Operate 1-P on the field.'
            write(6,'(2a,/11x,a)') ' -p S R   : Add real parameter', &
                 ' with name S and value R to define the functions', &
                 ' The value R may carry a unit. E.g.: -p en 5,eV'
             write(6,'(a)') ' -w       : Enable overwrite. '
             write(6,'(a)') ' -ver     : Print version information. '
             write(6,'(a/)') ' -h, -?   : Print this help text. '
          write(6,'(2a/,a)') ' The options -f and', &
                 ' -O are mandatory.'
          write(6,'(2a/,a)') ' "inputfile" contains a list of ', &
               ' operations enclosed in keywords', &
               ' "filter-section" and "end-filter-section".'
          write(6,'(2a/,2a)') ' All operations defined in the', &
               ' filter-section are applied multiplicative:', &
               ' the output the nth operation is used as', &
               ' input for the (n+1)th operation.'
          write(6,'(2a/,a)') ' All actions defined in the', &
               ' filter-section therefore define the',' operator P.'

          write(6,'(78a1)')  ('-',i=1,78)
          stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop 
         else 
            write(6,'(2a)') ' Unknown option : ', buf
            stop
         end if
         if (iarg .le. i) goto 200
         goto 100
      end if


 200  continue



      inputfile = buf
      inplaenge =index(buf,' ')-1
      if (inputfile .eq. ' ' .or.  inplaenge .le.1) then
         write(*,*) 'Name of input file   :'
         read(*,'(a)') inputfile
         inplaenge=index(filein,' ')-1
         if (inplaenge .le.1) then
            message="No input file given."
            call errormsg
         end if
      endif
     
      call abspath(inputfile,inplaenge)
        
      laein=index(filein,' ')-1
      laeout=index(fileout,' ')-1

      if ( laein.eq.0) then
         message=' Option -f has to be set.'
         call errormsg
      else 
         call abspath(filein,laein)
      end if

      if ( laeout.eq.0 ) then
         fileout=filein(1:laein)//'1'
         laeout=index(fileout,' ')-1
      end if

      call abspath(fileout,laeout)
      

      if ( lrdfnc ) then
         opflaenge=index(operfile,' ')-1
         if (opflaenge.ne.0 ) then
            call abspath(operfile,opflaenge)
         end if 
         inquire(file=operfile,exist=check)
         if (.not.check) then
            message='File not found: '//operfile(1:opflaenge)
            call errormsg
         endif
      end if
          

      return


      end subroutine fldopinput

      end program fldop






    
      
