      module fldoperate

use decimal, only: long,dop
use op1lib, only: zerovxz, zerovxi, zerovxd
use fftmod, only: fftinit, fft

      implicit none
      private
      public :: fldmultfnc, fldmaxval, fldfilter, fldnorm, fldsymmetry, &
                fldreverse, fldcomplex, fldscale, fldaddfnc

      contains

!-----------------------------------------------------------------------
!  fldmultfnc
!  Multyply a function to the input 
!-----------------------------------------------------------------------

      subroutine fldmultfnc(field,field1,nfunc)

use global
use maxv, only: maxdim, maxsta, maxkoe
use oct
use dvrdatmod
use operdef
use datenmod
use griddatmod
use potevalmod, only: poteval

      implicit none

      complex(dop), dimension(nfin), intent(in) :: field
      complex(dop), dimension(nfin), intent(out) :: field1
      complex(dop) :: vz
      integer(long), intent(in) :: nfunc
      integer(long) :: nham, i,k,k1,k2,h
      real(dop) :: v,time
      real(dop), dimension(maxdim) :: gpoint
      integer(long) :: idum
      real(dop), dimension(maxdim) :: rdum

      do i=1,hamdim
         if (hamlab(i).eq.fldfunclab(nfunc)) nham=i
      end do

      call zerovxz(field1,nfin)
 
      k1=zham(nham)      
      k2=zham(nham)+kzahl(nham)-1
     
      do k=k1,k2
         h=hamilton(ftime,k)
         if (htmtype(h) .eq. 101) then 
            do i=1,nfin
               time=tinit+dble(i-1)*out2
               gpoint(fkoe)=1.d0
               gpoint(ftime)=time
               call poteval(h,v,gpoint,vz,rdum,idum)

               field1(i)=field1(i)+field(i)*vz
            end do
         end if
      end do

      end subroutine fldmultfnc

!-----------------------------------------------------------------------
!  fldmaxval
!  Scale input F so that MAXVAL(ABS(F)) == VAL
!-----------------------------------------------------------------------
        
      subroutine fldmaxval(field,field1,nfunc)

      use global
      use oct

      use logdat

      implicit none
         
      complex(dop), dimension(nfin), intent(in) :: field
      complex(dop), dimension(nfin), intent(out) :: field1
      integer(long), intent(in) :: nfunc
      integer(long) :: i
      real(dop) :: maxfld, rdum
      
      maxfld = 0.d0
      do i=1,nfin
         rdum = abs(field(i))
         if (rdum .gt. maxfld) maxfld = rdum
      end do
      if (maxfld.lt.tiny) then
         routine="fldmaxval"
         message="Division by zero"
         call errormsg
      end if
      do i=1,nfin
         field1(i) = field(i)/maxfld*fldrparam(nfunc)
      end do

      end subroutine fldmaxval
      
!-----------------------------------------------------------------------
!  fldfilter
!  FFT, multiply a function and FFT back
!-----------------------------------------------------------------------

      subroutine fldfilter(field,field1,hin,rueck,workc,faktor,nfunc)

use global
use maxv, only: maxdim, maxsta, maxkoe
use oct
use dvrdatmod
use operdef
use datenmod
use griddatmod

use potevalmod, only: poteval

      implicit none

      complex(dop), dimension(nfin), intent(inout) :: field
      complex(dop), dimension(nfin), intent(out)   :: field1,workc,hin,rueck
      complex(dop) :: vz
      integer(long), intent(in) :: nfunc
      integer(long) :: nham, i,k,k1,k2,h
      integer(long), dimension(nfin) :: faktor
      real(dop) :: v,w,dw
      real(dop), dimension(maxdim) :: gpoint
      logical(kind=4) :: invers
      integer(long) :: idum
      real(dop), dimension(maxdim) :: rdum

      call zerovxz(workc,nfin)
      call zerovxi(faktor,nfin)
      call zerovxz(field1,nfin)
      call zerovxd(gpoint,maxdim)

      dw=2.d0*pi/(tfinal-tinit+out2)
      invers=.false.
      call fftinit(nfin,faktor,hin,rueck)
      call fft(field,workc,hin,faktor,nfin,1,1,invers) 

      do i=1,hamdim
         if (hamlab(i).eq.fldfunclab(nfunc)) nham=i
      end do
      
      k1=zham(nham)      
      k2=zham(nham)+kzahl(nham)-1
     
      do k=k1,k2
         h=hamilton(ftime,k)
         if (htmtype(h) .eq. 101) then 
            do i=1,nfin
               if (i.le.nfin/2)then
                  w=dble(i-1)*dw
               else
                  w=dble(i-nfin-1)*dw
               end if
               gpoint(fkoe)=1.d0
               gpoint(ftime)=w
               call poteval(h,v,gpoint,vz,rdum,idum)
               field1(i)=field1(i)+field(i)*vz
            end do
         end if
      end do
   
      invers=.true.
      call fft(field1,workc,rueck,faktor,nfin,1,1,invers) 

      do i=1,nfin
         field1(i)=field1(i)/nfin
      end do

      end subroutine fldfilter
      
!-----------------------------------------------------------------------
!  fldnorm
!  Scale input F so that \int dt |F(t)|^2 == somevalue
!-----------------------------------------------------------------------

      subroutine fldnorm(field,field1,nfunc)

use global
use oct
use datenmod

      implicit none

      complex(dop), dimension(nfin), intent(in) :: field
      complex(dop), dimension(nfin), intent(out) :: field1
      real(dop) :: integrl,norm1, norm
      integer(long), intent(in) :: nfunc
      integer(long) :: i
      
      norm1=fldrparam(nfunc)

      integrl=dble(field(1)*dconjg(field(1)))*out2
      do i=2,nfin
         integrl=integrl+dble(field(i)*dconjg(field(i)))*out2
      end do

      norm=dsqrt(integrl/norm1)
      do i=1,nfin
         field1(i)=field(i)/norm
      end do
      
      end subroutine fldnorm
      
!-----------------------------------------------------------------------
!  fldsymmetry
!  (Anti-)symmetrize so that F(t)= +/- F(T-t)
!-----------------------------------------------------------------------
      
      subroutine fldsymmetry(field,field1,nfunc)

use global
use oct

use logdat

      implicit none

      complex(dop), dimension(nfin), intent(in) :: field
      complex(dop), dimension(nfin), intent(out) :: field1
      integer(long), intent(in) :: nfunc
      integer(long) :: i
   
      if (fldaddprm(nfunc).eq. 'even') then
         do i=1,nfin
            field1(i)=(field(i)+field(nfin-i+1))/2.d0
         end do
      else if (fldaddprm(nfunc).eq. 'odd') then
         do i=1,nfin
            field1(i)=(field(i)-field(nfin-i+1))/2.d0
         end do
      else
         routine="fldsymmetry"
         message='"even" or "odd" expected'
         call errormsg
      end if

      end subroutine fldsymmetry

!-----------------------------------------------------------------------
!  fldreverse
!  Reverse: F(t) -> F(T-t)
!-----------------------------------------------------------------------

      subroutine fldreverse(field,field1)

use global
use oct

      implicit none

      complex(dop), dimension(nfin), intent(in) :: field
      complex(dop), dimension(nfin), intent(out) :: field1
      integer(long) :: i

      do i=1,nfin
         field1(i)=field(nfin-i+1)
      end do

      end subroutine fldreverse

!-----------------------------------------------------------------------
!  fldcomplex
!  Convert F to either Re[F], Im[F], Abs[F] or Conjg[F] 
!-----------------------------------------------------------------------

      subroutine fldcomplex(field,field1,nfunc)
    
use global
use oct

      implicit none

      complex(dop), dimension(nfin), intent(in) :: field
      complex(dop), dimension(nfin), intent(out) :: field1
      integer(long), intent(in) :: nfunc
      integer(long) :: i

      do i=1,nfin
         if ( fldaddprm(nfunc)(1:4) .eq. 'real') then
            field1(i)=dcmplx(dble(field(i)),0.d0)
         else if ( fldaddprm(nfunc)(1:4) .eq. 'imag' ) then
            field1(i)=dcmplx(dimag(field(i)),0.d0)
         else if ( fldaddprm(nfunc)(1:5) .eq. 'conjg' ) then
            field1(i)=dconjg(field(i))
         else if ( fldaddprm(nfunc)(1:3) .eq. 'abs' ) then
            field1(i)=dcmplx(abs(field(i)),0.d0)
         end if
      end do

      end subroutine fldcomplex
    
!-----------------------------------------------------------------------
!  fldscale
!  Multiply F(t) with a real number
!-----------------------------------------------------------------------

      subroutine fldscale(field,field1,nfunc)

use global
use oct

      implicit none

      complex(dop), dimension(nfin), intent(in) :: field
      complex(dop), dimension(nfin), intent(out) :: field1
      integer(long), intent(in) :: nfunc
      integer(long) :: i

      do i=1,nfin
         field1(i)=field(i)*fldrparam(nfunc)
      end do

      end subroutine fldscale

!-----------------------------------------------------------------------
!  fldaddfnc
!  Add a function to input: F(t) += G(t)
!-----------------------------------------------------------------------

      subroutine fldaddfnc(field,field1,nfunc)

use global
use maxv, only: maxdim,maxsta,maxkoe
use oct
use dvrdatmod
use datenmod
use operdef
use griddatmod

use potevalmod, only: poteval

      implicit none

      complex(dop), dimension(nfin), intent(in) :: field
      complex(dop), dimension(nfin), intent(out) :: field1
      integer(long), intent(in) :: nfunc
      complex(dop) :: vz
      integer(long) :: nham, i,k,k1,k2,h
      real(dop) :: v,time
      real(dop), dimension(maxdim) :: gpoint
      integer(long) :: idum
      real(dop), dimension(maxdim) :: rdum

      do i=1,hamdim
         if (hamlab(i).eq.fldfunclab(nfunc)) nham=i
      end do

      call zerovxz(field1,nfin)
 
      k1=zham(nham)      
      k2=zham(nham)+kzahl(nham)-1
     
      do k=k1,k2
         h=hamilton(ftime,k)
         if (htmtype(h) .eq. 101) then 
            do i=1,nfin
               time=tinit+dble(i-1)*out2
               gpoint(fkoe)=1.d0
               gpoint(ftime)=time
               call poteval(h,v,gpoint,vz,rdum,idum)
               field1(i)=field1(i)+vz
            end do
         end if
      end do

      do i=1,nfin
         field1(i)=field1(i)+field(i)
      end do

      end subroutine fldaddfnc

      end module fldoperate
