!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!     fnceingabe
!
!     As heingabe, but for a single 'DOF' as the
!     'operator' only acts on the e-field.
!     The operator is much simpler as only analytic 
!     functions (ifiles 101-200 and 301-400)
!     are allowed and no more than 1 DOF (+ koeff). 
!
!     A DVR is not set up at all. This just sets up the labels 
!     and pointers to represent a 1D diagonal 
!     hamiltonian so that the opfuncs library can be used. 
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
module fnceingabemod

use decimal, only: long, dop

      implicit none
      private
      public :: fnceingabe

      contains

      subroutine fnceingabe(loperr,pass)

use global
use maxv, only: maxdim, maxsta, maxkoe
use aglobal
use dvrdatmod
use griddatmod
use operdef
use runpropmod
use runopermod
use channels
use datenmod
use oct
use logdat

use logdat, only: errormsg, warnmsg
use einopermod, only: predefined
use einparamod, only: heinpara
use heinutil, only: gethnum
use matchlabmod, only: matchlab
use asshop, only: assignhlab
use fnceintermsmod

use opermod_interface

      implicit none

      integer(long), intent(in) :: pass
      integer(long) :: ic,iz,ierr,k,f,nham,l,i
      logical(kind=4), intent(out) :: loperr

      character(len=c3) label

      iolog=ilog !redirect operator log to ilog

      nopdof=2
      nopmode=1

!-----------------------------------------------------------------------
! set up space for function coefficients
!-----------------------------------------------------------------------
      fkoe=1
      gdim(fkoe)=1
      modelabel(fkoe)='Koeff'
 
      mkoe=1
      subdim(mkoe)=1
      nspfdof(mkoe)=1
      spfdof(1,mkoe)=fkoe
      dofspf(fkoe)=mkoe


      ftime = 2
      modelabel(ftime)='Time'
      gdim(ftime)=1

      nspfdof(mkoe)=nspfdof(mkoe)+1
      spfdof(2,mkoe)=ftime
      dofspf(ftime)=mkoe

      call predefined

!-----------------------------------------------------------------------
! open functions text file
!-----------------------------------------------------------------------
      
      l=opflaenge
      write(ilog,'(/,a)') 'Opening file :'//operfile(1:l)
      open(unit=iop,iostat=ierr,file=operfile(1:l),status='old',form='formatted')
      if (ierr.ne.0) then
         routine='fnceingabe'
         message = 'Error opening file : '//operfile(1:l)
         call errormsg
      endif
     
!-----------------------------------------------------------------------
! Parameter section
!-----------------------------------------------------------------------
      rewind(iop)
      iz=0
 210  continue

      call rdinpf(iop,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine = 'fnceingabe, parameter-section'
         message = inptit
         call errormsg
      endif

      if (keyword(1) .eq. 'parameter-section') then
         call heinpara(iop,pass)
      else if (keyword(1) .eq. 'end-functions' .or. &
           keyword(1) .eq. 'end-operator' .or. &
           keyword(1) .eq. 'end-input') then
         continue
      else
         go to 210
      endif

!-----------------------------------------------------------------------
! read function terms
!-----------------------------------------------------------------------
      rewind(iop)
      iz=0
      hamdim=0 !no 'system' 
  310 continue

      call rdinpf(iop,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine = 'fnceingabe, function-section'
         message = inptit
         call errormsg
      endif

      if (keyword(1)(1:16) .eq. 'function-section') then
         if (keyword(1)(17:17) .eq. '_') then
            label=keyorig(1)(18:lc(1))
            call gethnum(nham,label)
         else
            routine = 'fnceingabe, function-section'
            message = 'function-section without label found.'
            call errormsg
         endif
         call fnceinterms(loperr,iz,nham)
      else if (keyword(1) .eq. 'end-function' .or. &
           keyword(1) .eq. 'end-operator' .or. &
           keyword(1) .eq. 'end-input') then
         go to 315
      endif
      go to 310
  315 continue



!-----------------------------------------------------------------------
! read any operator label definitions 
!-----------------------------------------------------------------------
      rewind(iop)
      iz=0
  510 continue

      call rdinpf(iop,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         routine = 'fnceingabe, labels-section'
         message = inptit
         call errormsg
      endif

      if (keyword(1) .eq. 'labels-section') then
         call heinlab(iop,loperr,iz,pass)
      else if (keyword(1) .eq. 'end-function' .or.  &
           keyword(1) .eq. 'end-operator' .or. &
           keyword(1) .eq. 'end-input') then
         continue
      else
         go to 510
      endif

      close (iop)

      call matchlab(loperr)

!-----------------------------------------------------------------------
! fill in blanks in Hamiltonian with unit operators
!-----------------------------------------------------------------------
      do k=1,ktot
         do f=1,nopdof
            if (hamilton(f,k) .eq. 0) then
               hamilton(f,k)=f
            endif
         enddo
      enddo


!-----------------------------------------------------------------------
! check array dimensions and finish if not enough space allocated
!-----------------------------------------------------------------------
      routine = 'fnceingabe (Line 225)'
      if (minfac .gt. maxfac) then
         write(message,'(a,i7)') 'increase maxfac in operdef.inc. Minimum: ',minfac
         call warnmsg
      endif

      if (hopdim .gt. maxhop) then
         write(message,'(a,i5)') &
         'Redimension maxhop in operdef.inc. Minimum : ',hopdim
         call warnmsg
      endif

      if (htmdim .gt. maxhtm) then
         write(message,'(a,i5)') &
         'Redimension maxhtm in operdef.inc. Minimum : ',htmdim
         call warnmsg
      endif

      if (minkoe .gt. maxkoe) then
         write(message,'(a,i5)') &
         'Redimension maxkoe in maxkoe.inc. Minimum : ',minkoe
         call warnmsg
      endif

      if (lwarn) then
         write(message,'(a,i5)') &
           'Array dimension error while reading in operators'
      endif

!-----------------------------------------------------------------------
! set up pointers and labels for operators to be read from files
!-----------------------------------------------------------------------
!      call zeigrdf

!-----------------------------------------------------------------------
! search library for functions needed to set up operators
!-----------------------------------------------------------------------
      call assignhlab(loperr)

!     checkk hopfile for non analytic functions
      do i=1,hopdim
         if ( hopfile(i).ne.101 .and. &
              (hopfile(i).lt.301 .or.hopfile(i).gt. 400) ) then

            routine='fnceingabe (check for non-1D functions)'
            write(message,'(i4)') hopfile(i)
                       
            if (hopfile(i).lt.0)  hopfile(i)=- hopfile(i)

            select case (hopfile(i))
            
!           message='Found illegal ifile '//message(1:4)//': ' //hoplab(i)
!           call errormsg

            case(1)
            message='Found ifile 1 - Potential from file: '//hoplab(i)
            goto 998

            case(2)
            message='Found ifile 2 - Potential from file: '//hoplab(i)
            goto 998

            case(3)
            message='Found ifile 3 - 1D non-local: '//hoplab(i)
            goto 998

            case(4)
            message='Found ifile 4 - CH3I Hamiltonian: '//hoplab(i)
            goto 998

            case(5)
            message='Found ifile 5 - 2D non-local: '//hoplab(i)
            goto 998

            case(6)
            message='Found ifile 6 - Combined natural potential: '//hoplab(i)
            goto 998

            case(7)
            message='Found ifile 7 - 4D operator: '//hoplab(i)
            goto 998

            case(8)
            message='Found ifile 8 - 3D non-local: '//hoplab(i)
            goto 998

            case(201)
            message='Found ifile 202 - Multi-dimensional PES: '//hoplab(i)
            goto 998

            case(202)
            message='Found ifile 202 - Products of trig. func.' &
                 //'(for surface scat.): '//hoplab(i)

            case(203)
            message='Found ifile 203 - Multi-dimensional PES' &
                 //hoplab(i)
            goto 998

            case(999)
            message='Found ifile -999 - natpot or srffile: '//hoplab(i)

            end select

 998        call errormsg
         end if
      end do

!-----------------------------------------------------------------------
! After any call of 'warnmsg' (which is also called by 'wrline') the
! logical 'lwarn' is set to true.
!-----------------------------------------------------------------------
      loperr=lwarn.or.loperr
      lwarn=.false.

      return

      end subroutine fnceingabe

end module fnceingabemod
