      module fnceintermsmod

use decimal, only: long,dop
use op1lib, only: zerovxl, zerovxi

      implicit none
      private
      public :: fnceinterms

      contains

      subroutine fnceinterms(loperr,iz,nham)

use global
use maxv, only: maxdim, maxsta, maxkoe
use oct
use dvrdatmod
use operdef
use datenmod
use griddatmod
use runopermod
use channels

use logdat
use operutil, only: mvkend
use operutil, only: rdfactors
use heintermmod, only: renterm, storetm

      implicit none

      integer(long) :: ic,iz,iz0,ierr,j,k,k1,n,m,mh,f,nhmode, & 
              ktot1,nfac,h,nham,muld,mode,j1,j2,slen
      integer(long), dimension(24) :: optyp
      integer(long), dimension(34) :: optyp1
      integer(long), dimension(maxdim) :: mham,grid
      character(len=c2), dimension(24) :: factor,factor1
      logical(kind=4) :: loperr,initial,lnum,lnonum
      logical(kind=4), dimension(maxkoe) :: kignore

      routine = 'Heinterms (reading operator-file)'
      lwarn = .false.

      iz0 = 0

      call zerovxl(kignore,maxkoe)
      call zerovxi(mham,maxdim)
      call zerovxi(grid,maxdim)
      do m=1,nmode
         laddmode(m,nham) = .true.
      enddo
   
!-----------------------------------------------------------------------
! if hoplab needs to be initialised then:
! place unit operator at beginning of list for each degree of freedom
! place +1, -1 at beginning of list for coefficients
!-----------------------------------------------------------------------
      if (hopdim .eq. 0) then
         hoplab(1)='1'
         hoplab(2)='-1'
         hoplab(3)='I'
         hopdim=3
       
         do f=1,nopdof
            htmdim=htmdim+1
            h=min(htmdim,maxhtm)
            hterm(1,h)=1
            htmfac(h)=1
            htmdof(h)=f
            htmadof(h)=f
            htmtype(h)=1
            htmsym(h)=1
         enddo

         k=0
      else
         
         k=ktot
         do k1=k+1,maxkoe
            do f=1,maxdim
               hamilton(f,k1)=0
            enddo
         enddo
      endif

      mh=0
      nhmode=0
      initial=.true.

!-----------------------------------------------------------------------
! READ KEYWORDS:
!----------------------------------------------------------------------- 
 10   continue
      call rdinpf(iop,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         message = inptit
         routine = 'fnceinterms (fnceinterm.F)'
         call errormsg
      endif
      call cntrl(keyword(1),lc(1),ierr)
      lnum = .false.
      lnonum = .false.



!-----------------------------------------------------------------------
! Check for '[','{' or '|' . Only simple operators are allowed here.
!-----------------------------------------------------------------------
      routine = 'fnceinterms (reading file)'
      j1=index(inptit,'[')
      j2=index(inptit,'{')
      if( max(j2,j1) .gt. 1 ) then
         if(j1 .le. 0 ) j1 = j2
         if(j2 .le. 0 ) j2 = j1
         j2=min(j1,j2)
         j1=index(inptit,'#')
         if( j1 .le. 0 .or.  j1.gt.j2 ) then
            message='No brackets allowed in function-Section!'// &
                    ' Use Labels-Section.'
            call wrline(zeroi,iz,iz0) 
         endif
      endif
      j2=index(inptit,'|')
      if( j2 .gt. 1 ) then
         j1=index(inptit,'#')
         if( j1 .le. 0 .or.  j1.gt.j2 ) then
            message='No "|" allowed in function-Section!'// &
                    ' Write out products.'
            call wrline(zeroi,iz,iz0) 
         endif
      end if


!-----------------------------------------------------------------------
! read operator terms
!-----------------------------------------------------------------------
      if (keyword(1) .eq. 'end-function-section') goto 20

      !use inptit directly
      call rmblnk(inptit)
      j1=index(inptit,'#')
      j2=len(inptit)
      if (j1.gt.0) inptit(j1:j2) = ' '
      j1=slen(inptit)

      ! some error checks
      if (inptit(j1:j1).eq.'+' .or.inptit(j1:j1).eq.'-' .or. &
          inptit(j1:j1).eq.'*' .or.inptit(j1:j1).eq.'/' ) then
         routine='fnceinterms (reading function section)'
         write(message,'(a,i4,a)') operfile//' line:',iz, &
              ' - No +-*/ allowed at end of line. Use next line'
         call errormsg
      end if

      if (inptit(1:1).eq.'*' .or.inptit(1:1).eq.'/') then
         routine='fnceinterms (reading function section)'
         write(message,'(a,i4,a)') operfile//' line:',iz, &
              ' - Products have to be in one line.'
         call errormsg
      end if
      
      do j=1,j1
         if ( (inptit(j:j).eq.'*' .or.inptit(j:j).eq.'/' .or. &
               inptit(j:j).eq.'+' .or.inptit(j:j).eq.'-') .and. &
              (inptit(j+1:j+1).eq.'*' .or.inptit(j+1:j+1).eq.'/' .or. &
               inptit(j+1:j+1).eq.'+' .or.inptit(j+1:j+1).eq.'-')) then

            routine='fnceinterms (reading function section)'
            write(message,'(a,i4,a)') operfile//' line:',iz, &
                 ' - Consecutive operators: '//inptit(j:j) &
                 //inptit(j+1:j+1)
            call errormsg
         end if
      end do
      
      !split into terms and operators 
      call rdfactors(inptit(1:j1),nfac,factor,optyp,j1)
      muld=0
      mode=0
      f=ftime

      do n=1,nfac
         if (optyp(n).eq.2) then
            routine='fnceinterms (reading function section)'
            write(message,'(a,i4,a)') operfile//' line:',iz, &
                 ' - No / allowed in functions. Use ^-1.'
            call errormsg
         end if

         call renterm(factor(n),basis(f),loperr,ipbaspar(1,f),ipbaspar(2,f))
      end do

      !store the terms
      f=fkoe
      if (k .lt. maxkoe) then
         k=k+1
         minkoe=k
         hamnum(k)=nham
      else
         minkoe=minkoe+1
         k=maxkoe
         hamnum(k)=nham
      endif
      
      factor1(1)='1'
      optyp1(1)=3
      call storetm(factor1,1,optyp1,f,h,muld,mode)
      htmsym(h) = 1
      kact(k)   = 1
      ktwin(k)  = k
      htmtype(h)= 1
      
      f=ftime
      call storetm(factor,nfac,optyp,f,h,muld,mode)
      hamilton(f,k)=h
      htmtype(h)= 101
      

      goto 10

    
 20   continue

!-----------------------------------------------------------------------
! Set ktot to the numbers of products in function section
!-----------------------------------------------------------------------
      ktot=k

!-----------------------------------------------------------------------
! remove ghost terms 
!-----------------------------------------------------------------------
      ktot1=ktot
      k1=1
      do k=1,ktot1
         if (kignore(k)) then
            call mvkend(k1)
            ktot=ktot-1
          else
            k1=k1+1
         endif
      enddo

      loperr=lwarn.or.loperr
      lwarn=.false.

      return

      end subroutine fnceinterms

      end module fnceintermsmod
