
      program genfld

use decimal, only:long,dop,sip
use global
use aglobal
use versions
use maxv, only:maxdim,maxsta,maxkoe
use dvrdatmod
use griddatmod
use operdef
use runopermod
use channels
use datenmod
use oct

use keyunitsmod
use logdat
use fnceingabemod
use heinsort, only: delzero
use findknummod, only: findknum
use potevalmod, only: poteval


use mctdhmod_interface
use module_interface


      implicit none

      logical(kind=4)   :: linperr,lexist
      integer(long)     :: ilbl,i,pass
      character(len=c5) :: filename

      call default
      call mdefault
      call genfldinput
      
!-----------------------------------------------------------------------
! set up memory for 2D system with up to 1000 operator terms
! and 1000 paramaters
!-----------------------------------------------------------------------
      maxdim=5
      maxsta=2
      maxkoe=1000
      call alloc_dvrdat
      call alloc_grddat
      maxhop=2000
      maxhtm=1000
      maxdef=500
      maxrdf=1
      maxfac=10
      maxham=10
      call alloc_operdef
      maxpar=1000
      allocate(apar(maxpar))
      allocate(rpar(maxpar))

!-----------------------------------------------------------------------
! define PI
!-----------------------------------------------------------------------
      npar = 1
      apar(1)='PI'
      rpar(1)=4.0_dop*atan(1.0_dop)

      filename='genfld.log'
      ilbl  = index(filename,' ')
      inquire(file=filename(1:ilbl),exist=lexist)
      if (lexist .and. .not. lovwr) then
         write(6,*) "ERROR: In program genfld:"
         write(6,*) "       Unable to open log file:"
         write(6,*) "       "//filename(1:ilbl)
         write(6,*) "       Overwrite disabled? (Try -w option)."
         Stop 1
      end if
      
      open (unit=ilog,file=filename(1:ilbl),form='formatted')
      logisopen=.true.
      call wrversion(ilog)
      write(ilog,'(2a/)')'Options: ', options(1:optlaenge)

      nfin=int((tfinal-tinit)/out2 +0.1d0)+1
      
      if (  abs( dble(nfin-1)*out2 - tfinal + tinit) .gt. 1.d-8) then
         out2=(tfinal-tinit)/dble(nfin-1)
         write(6,*) "WARNING: program genfld"
         write(6,*) "     (tfinal-tinit)/tout should be integer."
         write(6,*) "     Setting tout to:", out2
         write(ilog,*) "WARNING: program genfld"
         write(ilog,*) "     (tfinal-tinit)/tout should be integer."
         write(ilog,*) "     Setting tout to:", out2
         write(ilog,*)
      else
      !do it anyways...
         out2=(tfinal-tinit)/dble(nfin-1)
      end if


      nfin=nfin+2*nfin1        !nfin1 are poits to be added
      tinit=tinit-out2*nfin1
      tfinal=tfinal+out2*nfin1

      write(ilog,'(a,i6)')    '  points  : ', nfin
      write(ilog,'(a,f16.8)') '  tinit   : ', tinit 
      write(ilog,'(a,f16.8)') '  tout    : ', out2
      write(ilog,'(a,f16.8)') '  tfinal  : ', tfinal
      write(ilog,*)
      call myflush(ilog)
! --- read function definitions 
      pass=2
      if (lrdfnc) then
         linperr = .false.
         call fnceingabe(linperr,pass)
         if (linperr) then
            routine = 'genfld (reading function definitions)'
            message = 'Finished with input error. See log file.'
            call errormsg
         end if
         call delzero
         call findknum
  

         write(ilog,'(2a)') 'Function definitions found: ' 
         do i=1,hamdim
            ilbl=index(hamlab(1),' ')
            write(ilog,'(i4,5x,a)') i,hamlab(i)(1:ilbl)
         end do
         write(ilog,*)
      end if
      
      ilbl=index(fldfunclab(1),' ')
      write(ilog,'(2a)') 'Function to be used: ', fldfunclab(1)(1:ilbl)
      write(ilog,*)
      write(ilog,'(a)') 'Executing...'
      call myflush(ilog)

      call wrfld
      
      !would have ended in errormsg if not:
      write(ilog,'(a)') 'Done. Program terminated normally.'

      stop

      contains

!-----------------------------------------------------------------------
!-----------------------------------------------------------------------

      subroutine wrfld

      implicit none

      complex(dop) :: field,vz
      integer(long) :: nham, i,k,k1,k2,h
      real(dop) :: v,time
      real(dop), dimension(maxdim) :: gpoint
      logical(kind=4) :: lunit, lzero, lexist
      integer(long) :: idum
      real(dop), dimension(maxdim) :: rdum

      lunit=.false.
      lzero=.false.
      
      if (fldfunclab(1) .eq. 'unity') lunit=.true.
      if (fldfunclab(1) .eq. 'zero') lzero=.true.
       
      nham=-1
      do i=1,hamdim
         if (hamlab(i).eq.fldfunclab(1)) nham=i
         !allow re-definition of zero and unity
         if (hamlab(i).eq.'unity'.or.hamlab(i).eq.'zero') then
            lunit=.false.
            lzero=.false.
         end if
      end do
      if (nham .lt. 0 .and. .not. (lunit .or. lzero) ) then
         routine='wrfld (genflf.F)'
         message="Function definition not found: " &
              //fldfunclab(1)(1:index(fldfunclab(1),' '))
         call errormsg
      end if
       
      inquire(file=fileout(1:laeout),exist=lexist)
      if (lexist .and. .not. lovwr) then
         write(6,*) "ERROR: In subroutine wrfld (genfld.F):"
         write(6,*) "       Unable to open file:"
         write(6,*) "       "//fileout(1:laeout)
         write(6,*) "       Overwrite disabled? (Try -w option)."
         write(ilog,*) "ERROR: In subroutine wrfld (genfld.F):"
         write(ilog,*) "       Unable to open file:"
         write(ilog,*) "       "//fileout(1:laeout)
         write(ilog,*) "       Overwrite disabled? (Try -w option)."
         Stop 1
      end if

      open(iout,file=fileout(1:laeout),form='formatted',err=12)
      
      if (lunit) then
         do i=1,nfin
            time=tinit+dble(i-1)*out2
            write(iout,'(4f16.8)',err=13) time,1.d0,0.d0,1.d0
         end do
      elseif(lzero) then
         do i=1,nfin
            time=tinit+dble(i-1)*out2
            write(iout,'(4f16.8)',err=13) time,0.d0,0.d0,0.d0
         end do
      else
         k1=zham(nham)      
         k2=zham(nham)+kzahl(nham)-1
         do i=1,nfin
            field = dcmplx(0.d0,0.d0)
            do k=k1,k2
               h=hamilton(ftime,k)
               if (htmtype(h) .eq. 101) then 
                 
                  if (lback) then
                     time=tfinal-dble(i-1)*out2
                  else
                     time=tinit+dble(i-1)*out2
                  end if

                  gpoint(fkoe)=1.d0
                  gpoint(ftime)=time
                  call poteval (h,v,gpoint,vz,rdum,idum)
                  field=field+vz
               end if
            end do
            time=tinit+dble(i-1)*out2
            write(iout,'(4f16.8)',err=13) time,dble(field), &
                  dimag(field), abs(field)
            
         end do

      end if


      close(iout)
      
      return

 12   continue
      routine='wrfld, (genfld.F)'
      message='Unable to open output file '//fileout(1:laeout)
      call errormsg

 13   continue
      routine='wrfld, (genfld.F)'
      message='Unable to write output file '//fileout(1:laeout)
      call errormsg

      end subroutine wrfld

!-----------------------------------------------------------------------
!-----------------------------------------------------------------------

      subroutine genfldinput

      implicit none

      ! tinit, tfinal, tout
      ! outfilename function help ver 

      character(len=80) :: buf
      integer(long) :: iarg,myiargc,i,ilbl,ierr,length,length1,lencom
      logical(kind=4) :: check,lcomma


      routine='genfldinput (genfld.F)'

      laeout    = 0

      optlaenge = 1
      fileout   = ' '
      operfile  = ' '
      lovwr     = .false.
      lrdfnc    = .false.
      lback     = .false.

      iarg = myiargc()
      if (iarg .lt. 1) goto 200 
      
      do i = 1, iarg
         call mygetarg(i,buf)
         ilbl  = index(buf,' ')
         ilbl = min(ilbl,C5-optlaenge)
         if(ilbl.gt.0) options = options(1:optlaenge)//buf(1:ilbl) 
         optlaenge = optlaenge + ilbl
      end do
   
      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
         if( buf(1:3) .eq. '-o ' ) then ! output file (field)
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
         else if( buf(1:3) .eq. '-O ' ) then !operator file 
            i = i+1
            call mygetarg(i,buf)
            operfile = buf
            lrdfnc = .true.
         else if( buf(1:3) .eq. '-R ' ) then
            lback=.true.
         else if( buf(1:3) .eq. '-w ' ) then
            lovwr=.true.
         else if( buf(1:3) .eq. '-a ' ) then !add points
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nfin1
            if (ierr.ne.0) then
               write(6,'(a/,3a)') &
                    'ERROR in subroutine genfldinput (genfld.F):', &
                    ' Option -a has to be followed by an', &
                    ' integer number, not: ', buf
               stop 1
            end if

         else if( buf(1:3) .eq. '-p ' ) then
            i=i+1
            call mygetarg(i,buf)
            length  = index(buf,' ')-1
             
            if(buf(1:1).eq.' '.or.buf(1:1).eq.'-') then
               write(6,'(/,a/,a)') &
                    'ERROR in subroutine genfldinput (genflf.F):', &
                    ' -p is not followed by a parameter name.'
               stop 1
            else
               npar=npar+1
               apar(npar)(1:length)=buf(1:length)
            end if
            
            i=i+1
            call mygetarg(i,buf)
  
!........If the comma is not surounded by blanks, lcomma is true.
            lencom  = index(buf,',')-1
            length1  = index(buf,' ')-1
            if( lencom.gt.0 .and. lencom.lt.length1 ) then
               lcomma = .true.
               length1 = lencom 
            else   
               lcomma = .false.   
            endif
            
            read(buf(1:lencom),*,iostat=ierr) rpar(npar)
            if(buf(1:1).eq.' '.or.ierr.ne.0) then
               length  = index(apar(npar),' ')-1
               write(6,'(/,a/,3a)') &
                    'ERROR in subroutine genfldinput (genfld.F):', &
                    ' -p ',apar(npar)(1:length), &
                    ' is not followed by a real number.'
               stop 1
            end if
            if(lcomma) then
               buf = buf(lencom+1:)
            else
               i=i+1
               call mygetarg(i,buf)
            endif
            if( buf(1:1) .eq. ',' ) then
               if( buf .eq. ',' ) then
                  i=i+1
                  call mygetarg(i,buf)
               else
                  buf = buf(2:)
               endif
               
               call keylowc(buf)
               call keyunits(rpar(npar),buf,ierr)
               if(buf(1:1).eq.' '.or.buf(1:1).eq.'-'.or.ierr.ne.0) then
                  length  = index(apar(npar),' ')-1
                  write(6,'(/,a/,2a,f10.6,a)')  &
                       'ERROR in subroutine genfldinput (genfld.F):', &
                       ' -p ',apar(npar)(1:length),rpar(npar), &
                       ' not followed by a unit.'
                  stop 1
               end if
            else
               i=i-1
            endif
    
         else if( buf(1:3) .eq. '-h ' .or. buf(1:3) .eq. '-? ' ) then
            write(6,"(78a1)")  ('-',i=1,78)
            write(6,'(1x,2a)') &
        'Purpose: Generate a sampled 1D function (ASCII) from an ', &
        'analytic expression.' 
            write(6,'(/1x,2a)') &
       'Usage  : genfld [-o -O -p -a -w -R -h -? -ver]', &
                 ' tinit tfinal tout fncname'
            write(6,'(/1x,a)') 'Options: '
            write(6,'(2a)')  ' -o file  : Name of the output file', &
                 ' (default: fncname.dat)'
            write(6,'(2a)') ' -O file  : File containing function',  &
                 ' definitions (.op file)'  
            write(6,'(2a,/11x,a)') ' -p S R   : Add real parameter',  &
                 ' with name S and value R to define the functions', &
                 ' The value R may carry a unit. E.g.: -p en 5,eV'
            write(6,'(2a)')  ' -a num   : Write num data points before', &
                 ' and after given sampling period'
             write(6,'(a)')  ' -w       : Enable overwrite. '
             write(6,'(2a)') ' -R       : Reverse, i.e., write f(T-t) instead', &
                     ' of f(t)'
             write(6,'(a)')  ' -ver     : Print version information. '
             write(6,'(a/)') ' -h, -?   : Print this help text. '
          write(6,'(2a/,a)') ' The option', &
                 ' -O is mandatory if fncname is not zero or unity.'
          write(6,'(2a/,a)') ' tinit and tfinal ', &
               ' define the first and last sampling point, ', &
               ' tout gives the sampling interval'
          write(6,'(2a/,2a/,a)') ' fncname is the label ', &
               ' of a function defined in an operator file.', &
               ' fncname has the pre-defined labels zero and unity', &
               ' for which no operator',' file is needed.'
          write(6,'(78a1)')  ('-',i=1,78)
          stop
         else if( buf(1:4) .eq. '-ver' ) then
            call wrversion(6)
            stop 
         else 
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg-4 .le. i) goto 200
         goto 100
      end if


 200  continue

      i=i+1
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) tinit  
      if(buf(1:1).eq.' '.or.ierr.ne.0) then
         write(6,'(a/,2a)') 'ERROR in subroutine genfldinput (genfld.F):', &
              ' tinit has to be a real number, not: ', buf
         stop 1
      end if


      i=i+1
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) tfinal
      if(buf(1:1).eq.' '.or.ierr.ne.0) then
         write(6,'(a/,2a)')  'ERROR in subroutine genfldinput (genfld.F):', &
              ' tfinal has to be a real number, not: ', buf
         stop 1
      end if

      i=i+1
      call mygetarg(i,buf)
      read(buf,*,iostat=ierr) out2
      if(buf(1:1).eq.' '.or.ierr.ne.0) then
         write(6,'(a/,2a)') 'ERROR in subroutine genfldinput (genfld.F):', &
              ' tout has to be a real number, not: ', buf
         stop 1
      end if

      i=i+1
      call mygetarg(i,buf)
      if(buf(1:1).eq.' ') then
         write(6,'(a/,2a)')  'ERROR in subroutine genfldinput (genfld.F):', &
              ' function name is empty.'
         stop 1
      end if
      fldfunclab(1) = trim(buf)
    
      laeout=index(fileout,' ')-1

      if ( laeout.eq.0 ) then
         laeout=index(buf,' ')-1
         fileout=buf(1:laeout)//'.dat'
         laeout=laeout+4
      end if
      call abspath(fileout,laeout)


      i=i+1
      call mygetarg(i,buf)
      if(buf(1:1).ne.' ') then
         message="Unexpected argument found: "//trim(buf)
         call errormsg
      end if
         

      if ( lrdfnc ) then
         opflaenge=index(operfile,' ')-1
         if ( opflaenge.ne.0 ) then
            call abspath(operfile,opflaenge)
         end if 
         inquire(file=operfile,exist=check)
         if (.not.check) then
            message='File not found: '//operfile(1:opflaenge)
            call errormsg
         endif
      end if

      return

      end subroutine genfldinput

      end program genfld
