!***********************************************************************
!
!                         JFUNC program
!
!     04/07 MS
!
!
!***********************************************************************

      program jfunc

use decimal, only:long,dop,sip
use global
use globmemmod
use versions
use channels
use oct
use datenmod
use logdat

use module_interface
use op1lib, only: zerovxd

      implicit none

      integer(long) ::  ilbl,i
      character(len=c5) :: filename
      real(dop) :: rdum, rdum1

      call default
      call jinput()

      filename = "fieldf.dat"
      open(idump,file=filename,form='formatted',status='old',err=1000)

!     determine the number of points in file      
      i=0
 12   continue
      read(idump,*,end=13,err=999) rdum
      if (i.eq.0) rdum1=rdum
      if (i.eq.1) out2=abs(rdum-rdum1)
      i=i+1
      goto 12
      
 13   continue
      nfin=i
      close(idump)

!     set pointer for field  
      mrdim = 1

      mrfld = mrdim
      mrdim = mrdim+nfin
      
      mrpnlty =mrdim
      mrdim = mrdim+nfin

      mrref = mrdim
      mrdim = mrdim+nfin

      mrqref= mrdim
      mrdim = mrdim+nfin

      mrqfld= mrdim
      mrdim = mrdim+nfin

      mrlagr= mrdim
      mrdim = mrdim+nfin

if(allocated(mr))deallocate(mr)
allocate(mr(mrdim))

      ! read field from file
      call loadfields(mr(mrfld),mr(mrpnlty),mr(mrref),mr(mrlagr), &
           mr(mrqref),mr(mrqfld))
      

      !calculate the XFROG-trace
      call calcj(mr(mrfld),mr(mrpnlty),mr(mrref),mr(mrlagr),&
           mr(mrqref),mr(mrqfld))

      deallocate(mr)

      stop

  999 continue
      message= 'Error reading :'//trim(filename)
      write(6,'(a)') message
      stop 1
 1000 continue
      ilbl=index(filename,' ')-1
      message= 'Cannot open input file: '//trim(filename)
      write(*,*)  message
      stop 1


      contains

!#######################################################################      

      subroutine loadfields(field,pnltyfnc,reffnc,lagrfnc,qref,qfld)

      implicit none

      real(dop), dimension(nfin), intent(out) :: field, pnltyfnc, reffnc, &
                                                 lagrfnc,qref,qfld
      real(dop) :: rdum
      integer(long) :: i
      character(len=c5) :: filename
      
      call zerovxd(field,nfin)
      call zerovxd(pnltyfnc,nfin)
      call zerovxd(reffnc,nfin)
      call zerovxd(lagrfnc,nfin)
      call zerovxd(qref,nfin)
      call zerovxd(qfld,nfin)

      filename='fieldf.dat'
      open(idump, file=filename)
      do i=1,nfin
         read(idump,*,err=999) rdum, field(i)
      end do
      close(idump)

      if (lpnltyfnc) then
         filename='pnltyfnc.dat'
         open(idump, file=filename)
         do i=1,nfin
            read(idump,*,err=999) rdum, pnltyfnc(i)
         end do
         close(idump)
      end if

      if (lreffnc) then
         filename='reffnc.dat'
         open(idump, file=filename)
         do i=1,nfin
            read(idump,*,err=999) rdum, reffnc(i)
         end do
         close(idump)
      end if
     
      if (llagrfnc) then
         
         filename='qref.dat'
         open(idump, file=filename)
         do i=1,nfin
            read(idump,*,err=999) rdum, qref(i)
         end do
         close(idump)

         filename='qfld.dat'
         open(idump, file=filename)
         do i=1,nfin
            read(idump,*,err=999) rdum, qfld(i)
         end do
         close(idump)

         filename='lagrfnc.dat'
         !lagrfnc has 4 points less then the other functions 
         open(idump, file=filename)
         do i=1,nfin-4
            read(idump,*,err=999) rdum, lagrfnc(i+2)
         end do
         close(idump)
         
      end if

      return

 999  continue
      message= 'Error reading :'//trim(filename)
      write(6,'(a)') message
      stop

      end subroutine loadfields


!####################################################################### 

      subroutine calcj(field,pnltyfnc,reffnc,lagrfnc,qref,qfld)

      implicit none

      real(dop), dimension(nfin), intent(in) :: field, pnltyfnc, reffnc, &
                                                 lagrfnc,qref,qfld
      real(dop) :: small, j2,j3
      integer(long) :: i

      small=1.d-7
      j2=0.d0
      j3=0.d0

      do i=1,nfin
         if (abs(pnltyfnc(i)).gt. small) then
            j2=j2+ (field(i) - reffnc(i))**2/pnltyfnc(i) 
         end if
      end do
      j2=j2*pnlty*out2

      
      if (llagrfnc) then
      
         do i=1,nfin
            j3 = j3 + lagrfnc(i)*qfld(i)
      
         end do
        

         do i=1,nfin
            if (abs(pnltyfnc(i)).gt. small) then
               j3 = j3 + qfld(i)*qref(i)/pnltyfnc(i)*2.d0*pnlty
            end if
         end do
         j3=j3*out2
      end if
      !write to stdout. 
      !The OCT Python script reads from a pipe to stdout. 
      write(*,*) j2, j3

      return
      end subroutine calcj

!#######################################################################

!-----------------------------------------------------------------------
! read the arguments given in command line
!-----------------------------------------------------------------------

      subroutine jinput()

      implicit none

      character(len=80) :: buf
      integer(long) :: ilbl, i, iarg, ierr, myiargc

!-----------------------------------------------------------------------
! default values
!-----------------------------------------------------------------------

      pnlty     = 0.d0
      llagrfnc  = .false.
      lreffnc   = .false.
      lpnlty    = .false.
      lpnltyfnc = .false.
      optlaenge = 1

!-----------------------------------------------------------------------
! read options 
!-----------------------------------------------------------------------
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      do i = 1, iarg
         call mygetarg(i,buf)
         ilbl  = index(buf,' ')
         ilbl = min(ilbl,100-optlaenge)
         if(ilbl.gt.0) options = options(1:optlaenge)//buf(1:ilbl) 
         optlaenge = optlaenge + ilbl
      end do


      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
 
         if( buf(1:3) .eq. '-F ' ) then ! the penalty factor
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) pnlty 
            lpnlty = .true.
            if (ierr .ne. 0) then
               write(6,'(2a/1x,a)') ' The argument following the option -F', &
                 ' must be a positive real number. Not: ', buf
               stop 
            end if
            if (pnlty .lt. 0.d0) then
               write(6,'(2a/1x,a)') ' The argument following the option -F', &
                 ' must be > 0. Not : ', buf
               stop 1
            end if

         else if( buf(1:3) .eq. '-L ' ) then  ! lagrange multiplier 
            llagrfnc= .true.                   ! (filter)
         else if( buf(1:3) .eq. '-R ' ) then ! the reference function
            lreffnc=.true.
         else if( buf(1:3) .eq. '-S ' ) then ! the reference function
            lpnltyfnc=.true.
         else if( buf .eq. '-ver' ) then
            call wrversion(6)
            stop

         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(2a)') 'Purpose : Computation of the control', &
                 ' functional (field parts) for OCT jobs'
            write(6,*)
            write(6,'(a)') 'Usage   : jfunc [-F -S -L -R -ver -h -?]'
            write(6,*)   
            write(6,'(a)') 'Options :'
            write(6,'(2a)') ' -F pnlty : Parameter to', &
                 ' penalize the field strength. Default: pnlty=1.'
            write(6,'(2a)') ' -S       : Load penalty-function,', &
                 ' from file pnltyfnc.dat'
            write(6,'(2a)') ' -L       : Load Lagrange-multiplier', &
                 ' (Krotov with filter)'
            write(6,'(2a)') ' -R       : Load reference-', &
                 'function from file reffnc.dat.'
            write(6,*) 
            write(6,'(2a)') 'Calculates the control functional', &
                 ' (field parts) for the current iteration.'
            write(6,'(2a)') 'Requires file "fieldf.dat".', &
                 ' Requeres file "reffnc" if option -R is given.'
            write(6,'(2a)') 'Requires the files "qref.dat" and', &
                 ' "lagrfnc.dat" if -L option is given.'
            write(6,'(2a)') 'Option -L requires', &
                 ' option -R. All filenames are hard-coded. Files'
            write(6,'(2a)') 'must be of the same length (grid points)', &
                 ' and reside in the current directory.'
            write(6,'(78a1)')  ('-',i=1,78)
            stop

         else             
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .eq. i) goto 200
         
         goto 100
      end if
     
 200  continue
      
      if (llagrfnc .and..not.lreffnc) then
         message="option -L requires option -R."
         call errormsg
      end if

      return 
      end subroutine jinput

      end program jfunc
