      module loadfld

use decimal, only: dop,long

      implicit none
      private
      public :: browsein, loadin

      contains

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Browse and load a 1D data set - similar to "li_browse" and "li_load"
! but without allocation of memory, i.e., memory has to be allocated
! with the data obtained by "browsein" before "loadin" is called.
!
! 11/07 MS
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!  browsein: collect data about the data file
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine browsein()

use global
use aglobal
use channels
use datenmod
use oct

use logdat

      implicit none

      character(len=c5) :: filename
      
! ___ constants
      integer(long), parameter :: stringlen = 160

! ___ variables
      integer(long) :: line, ierr, ilbl, jlbl
      integer(long) :: buflen
      integer(long) :: i

      character(len=160) :: string
      real(dop)          :: datapoint, starttime, currtime, timestep

      routine='octfilter, browsein'
      filename=filein


      ilbl=1
      jlbl=laein
         

      open(idump, file=filename(1:laein), status="old", iostat=ierr)
      if(ierr.ne.0)then
         write(string,'(i4)')ierr
         message='Data filename "'//filename(1:laein)//'" not found' &
            //', ierr='//string(1:4)
         call errormsg
      end if

      line=0
      buflen=0
 12   continue
         line=line+1
         read(idump,'(a)',err=100,end=100) string
         if(string(1:1).eq.'#')goto 20
         do i=1,stringlen
            if(string(i:i).ne.' ')goto 10
         end do
 10      continue         
         if(i.ge.stringlen)goto 20

         buflen=buflen+1

! ______ compiler "feature": only ever use '*' format in reads from str
         read(string,*,err=200) currtime, datapoint
         if(buflen.eq.1)then
            starttime=currtime
         else
            if(buflen.eq.2)then
               timestep=currtime-starttime
               if(timestep.le.0.0d0)then
                   write(string, '(e16.8)')timestep
                   message='time timestep should be positive but is'//string(1:16)
                   call errormsg
               end if
            else
! ____________ allow only linear time advance            
               if(currtime.lt.starttime+(timestep-1.d-7*timestep)*(buflen-1) &
                  .or. &
                  currtime.gt.starttime+(timestep+1.d-7*timestep)*(buflen-1)) &
               then
                  write(string,'(i4.4)')line
                  message='No equidistant spacing in '// &
                    'filename "'//filename(ilbl:jlbl)//  &
                    '" , line '//string(1:4)
                  call errormsg
               end if
            end if
         end if
 20      continue
      goto 12

100   continue
      close(idump)
   
      nfin=buflen
      out2=timestep
      tinit=starttime
      tfinal=currtime

      return


200   continue
      write(string,'(i4.4)')line
      message='Data filename "'//filename(ilbl:jlbl)// &
         '" contains malformed entry on line'//string(1:4)
      call errormsg


      end subroutine browsein

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!  loadin: load data into complex array field
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine loadin(field)

use global
use aglobal
use channels
use datenmod
use oct

use logdat

      implicit none

! ___ constants
      integer(long), parameter :: stringlen = 160

! ___ variables
      integer(long) :: line, ierr, i
      integer(long) :: dataset

      character(len=240) ::  string, filename
      real(dop) :: datapoint, starttime, currtime, timestep
      
      complex(dop), dimension(nfin), intent(out) :: field(nfin)

 
      routine='octfilter, loadin'
      filename=filein


      open(idump, file=filename(1:laein), status="old", iostat=ierr)
      if(ierr.ne.0)then
         write(string,'(i4)')ierr
         message='Data filename "'//filename(1:laein)// &
           '" not found, ierr='//string(1:4)
         call errormsg
      end if
      line=0
      dataset=0
      do while(.true.)
         line=line+1
         read(idump,'(a)',err=100,end=100)string
         if(string(1:1).eq.'#') goto 20
         do i=1,stringlen
            if(string(i:i).ne.' ')goto 10
         end do
 10      continue         
         if(i.ge.stringlen) goto 20
         dataset=dataset+1
         if(dataset.gt.nfin) then
           message="number of datasets exceeds number allocated, "// &
             "filename changed?"
           call errormsg
         end if
         
! ______ compiler "feature": only use '*' as format in reads from string
         read(string,*,err=200) currtime, datapoint
         field(dataset)=dcmplx(datapoint,0.d0)

         if(dataset.eq.1)then
            starttime=currtime
         else
            if(dataset.eq.2)then
               timestep=currtime-starttime
               if(timestep.le.0.0d0)then
                   write(string, '(e16.8)')timestep
                   message='time step should be positive but it is'//string(1:16)
                   call errormsg
               end if
            else

! _________ allow only linear time (or x) advance, equidistant spacing.            
               if(currtime.lt.starttime+(timestep-1.d-7*timestep)*(dataset-1) &
                  .or. &
                  currtime.gt.starttime+(timestep+1.d-7*timestep)*(dataset-1)) &
               then
                  write(string,'(i4.4)')line
                  message='No equidistant spacing in '// &
                    'filename "'//filename(1:laein)// &
                    '" , line '//string(1:4)
                  call errormsg
               end if
            end if
         end if
 20      continue
      end do

100   continue

      close(idump)
      

      return
      
200   continue
      write(string,'(i4.4)')line
      message='Data filename "'//filename(1:laein)// &
         '" contains malformed entry on line'//string(1:4)
      call errormsg

      end subroutine loadin

      end module loadfld
