!***********************************************************************
!
!                         XFROG program
!
!     The program calculates the XFROG-trace of a given electrical 
!     field. 
!
!     04/07 MS
!
!
!***********************************************************************


      program xfrog

use decimal, only:long,dop,sip
use globmemmod
use versions

      implicit none

      integer(long)                     :: ilbl,ifile,ofile,&
                                           nfield,ntres,nfres,mrfield,i,j
      integer(long), parameter          :: c5=240

      logical(kind=4)         :: lofile,lnfield,lfres,ltres,lfrange,ltrange,lwidth
      character(len=c5)       :: filein,message,fileout,filename
      real(dop)               :: wmin,wmax,gwidth,gwidth1,tmin,tmax,dwout,&
                                 dtin,tin_min,tin_max,rdum,dtout,pi
      real(dop), parameter    :: fs=41.3414_dop

      real(dop),allocatable  :: field(:)

      common /ixfparams/  nfield, ntres, nfres
      common /dxfparams/  wmin,wmax,gwidth,gwidth1,tmin,tmax,dtout,&
           dwout, dtin, tin_min, tin_max
      common /xffilnames/ filein, fileout
      common /xchannels/ ifile, ofile
      common /flags/ lofile, lnfield, lfrange,ltrange, lwidth

!-----------------------------------------------------------------------
      pi = 4.0_dop*atan(1.0_dop)
      call xfinput()

      ifile=12

      filename = filein
      open(ifile,file=filename,form='formatted',status='old',err=1000)

!     determine the number of points in file      
      i=0
 12   read(ifile,*,end=13,err=999) rdum
      i=i+1
      if (i.eq.1)       tin_min = rdum
      if (i.eq.nfield ) then
         tin_max = rdum
         goto 14
      end if
      goto 12
      
 13   continue
      
      rewind(ifile)
      if (nfield .eq. 0) then
         nfield = i
         tin_max = rdum
      else if (nfield .gt. i) then
         write(6,'(78a)')('-',j=1,78)
         write(6,*) 'WARNING: The field is sampled on only'
         write(6,*) i,' points. Setting "nfield" to this value'
         write(6,'(78a)')('-',j=1,78)
         nfield = i
         tin_max = rdum
      end if

 14   continue
      


!     Check, if resolutions and ranges are set for output.
!     If not set, use default values.
      if (.not.ltres) ntres = nfield
      if (.not.ltrange) then
         tmin = tin_min
         tmax = tin_max
      end if
      dtout = (tmax-tmin)/(ntres-1)
      dtin  = (tin_max - tin_min)/(nfield-1)


      if (.not.lfres) nfres = nfield+1      
      if (.not.lfrange) then
         wmin = 0.0_dop
         wmax = nfield/tmax
      end if
      dwout = (wmax-wmin)/(nfres-1)

!     if no gate width is set, use one fs for both
      if (.not. lwidth) then
         gwidth = fs**2
         gwidth1 = fs
      end if
      
      if (lofile) then
         filename = fileout
      else
         ilbl=index(filein,' ')-1
         filename = filein(1:ilbl)//'.xfg'
      endif
   
      !open outout file
      open(ofile,file=filename,form='formatted',status='unknown',&
           err=1001)

      ! read field from file
      call loadfield(field)

      !calculate the XFROG-trace
      call frogtrace(field)


deallocate(field)


      stop

  999 continue
      message= 'Error reading :'//trim(filename)
      write(6,'(a)') message
      stop
 1000 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open input file: '//trim(filename)
      write(*,*)  message
      stop


 1001 continue
      ilbl=index(filename,' ')-1
      message = 'Cannot open output file: '//filename(1:ilbl)
      write(*,*)  message
      stop
      contains


!#######################################################################      

      subroutine loadfield(field)
      
      implicit none

       integer(long)            :: i,nfield,ntres,nfres,ifile,ofile
       integer(long), parameter :: c5=240

      character(len=c5)       :: message,filein, fileout

      common /ixfparams/  nfield, ntres, nfres
      common /chnnls/ ifile, ofile
      common /xffilnames/ filein, fileout

      real(dop)                                 :: rdum
      real(dop), dimension(nfield), intent(out) :: field
    
      real(dop), parameter :: fs=41.3414_dop
      real(dop), parameter :: ev=27.2114_dop

      do i=1,nfield
         read(ifile,*,end=998,err=999) rdum, field(i)
      end do

 998  return

 999  continue
      message= 'Error reading :'//trim(filein)
      write(6,'(a)') message
      stop


      end subroutine loadfield


!####################################################################### 

      subroutine frogtrace ( field )

      implicit none

      integer(long)       :: nfield, ntres, nfres, it,iw, j, ifile, ofile
      real(dop)           :: wmin, wmax, gwidth, gwidth1, tmin, tmax,&
                             dtout, dwout,tau, t,w,  dtin, tin_min, tin_max, tmp, wtau

      real(dop), dimension(nfield), intent(in) :: field
      complex(dop)                             :: xf
      
      common /chnnls/ ifile, ofile
      common /ixfparams/  nfield, ntres, nfres
      common /dxfparams/  wmin, wmax, gwidth, gwidth1, tmin, tmax,&
           dtout, dwout,dtin, tin_min, tin_max

!     write some info to beginning of output file
      write(ofile,*) '# No. of used points in field ', nfield
      write(ofile,*) '# Input time started at       ', tin_min
      write(ofile,*) '# Input time ended at         ', tin_max
      write(ofile,*) '# So the time-step of input is', dtin
      write(ofile,*) '# resolution on time-axis     ', ntres
      write(ofile,*) '# Resolution on frequency-axis', nfres
      write(ofile,*) '# Start XFROG at time         ', tmin
      write(ofile,*) '# End XFROG at time           ', tmax
      write(ofile,*) '# The time-step for output is ', dtout
      write(ofile,*) '# Start XFROG at frequency    ', wmin
      write(ofile,*) '# End XFROG at frequency      ', wmax
      write(ofile,*) '# The freq.-step for output is', dwout
      write(ofile,*) '# Width of exp part of gate   ', dsqrt(gwidth)
      write(ofile,*) '# Width of rect part of gate  ', gwidth1
      write(ofile,*) 
      write(ofile,'(2a)') '# time, freq., cos-trafo, sin-trafo,',&
           ' abs((sin+cos)-trafo)'
  
      do it=1, ntres
         t = dble(it-1)*dtout + tmin
         do iw=1,nfres
            w = dble(iw-1)*dwout + wmin

            xf = dcmplx(0.0_dop,0.0_dop)
            do j=1,nfield

               tau = dble(j-1)*dtin + tin_min
               wtau = w*tau

               tmp = xfgate(tau,t,gwidth, gwidth1)*dtin

               xf=xf+dcmplx(dcos(wtau),&
                     dsin(wtau) )*field(j)*tmp
            end do
             

            write(ofile,'(5f16.8)') t,w, dble(xf), dimag(xf), abs(xf)


         end do
         write(ofile,*)
      end do
      
      close(ofile)

      return
      end subroutine frogtrace




!-----------------------------------------------------------------------
! read the arguments given in command line
!-----------------------------------------------------------------------
      subroutine xfinput()

use module_interface

      implicit none


      integer(long), external :: myiargc
      integer(long)           :: iarg,i,ierr,nfield,ntres,nfres,laein
      character(len=80)       :: buf
      real(dop)               :: wmin, wmax, gwidth,gwidth1, tmp, tmin, tmax,dtout,&
                                 dwout,dtin, tin_min, tin_max
      logical(kind=4)         :: lofile, lnfield, lfres, ltres, lfrange,&
                                 ltrange, lwidth
      character(len=c5)       :: filein,fileout

      real(dop), parameter :: c5=240
      real(dop), parameter :: fs=41.3414_dop
      real(dop), parameter :: ev=27.2114_dop

      common /ixfparams/  nfield, ntres, nfres
    
      common /dxfparams/  wmin,wmax,gwidth,gwidth1,tmin,tmax,dtout,&
           dwout,dtin,tin_min,tin_max

      common /xffilnames/ filein, fileout
      common /flags/ lofile, lnfield, lfrange,ltrange, lwidth

!-----------------------------------------------------------------------
! default input and output filenames 
!-----------------------------------------------------------------------
      filein  = ' '
      fileout = ' '
 
      nfield  = 0
      ntres   = 0
      nfres   = 0

      wmin    = 0.0_dop
      wmax    = 0.0_dop
      gwidth  = 0.0_dop
      gwidth1 = 0.0_dop
  
      lofile  = .false.
      lnfield = .false.
      lfres   = .false.
      ltres   = .false.
      lfrange = .false.
      ltrange = .false.
      lwidth  = .false.


!-----------------------------------------------------------------------
! read options 
!-----------------------------------------------------------------------
      iarg = myiargc() 
      if (iarg .lt. 1) goto 200 

      i = 0
 100  i=i+1
      call mygetarg(i,buf)
      if( buf(1:1) .eq. '-' ) then
 
       if( buf(1:2) .eq. '-o' ) then
            i = i+1
            call mygetarg(i,buf)
            fileout = buf
            lofile = .true.
         else if( buf(1:2) .eq. '-n' ) then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nfield
            if (ierr .ne. 0  .or.  nfield .le. 0) then
               write(6,'(2a/,a)')&
                 ' The  argument following the option -n',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
            lnfield = .true.

         else if(buf .eq. '-rt') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) ntres
            if (ierr .ne. 0  .or.  ntres .le. 0) then
               write(6,'(2a/,a)')&
                 ' The argument following the option -rt',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
            ltres = .true.

         else if(buf .eq. '-rf') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) nfres
            if (ierr .ne. 0  .or.  nfres .le. 0) then
               write(6,'(2a/,a)')&
                 ' The argument following the option -rf',&
                 ' must be a positive integer. Not : ', buf
               stop
            end if
            lfres = .true.

         else if(buf .eq. '-fd') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) wmin
            if (ierr .ne. 0 ) then
               write(6,'(2a/,a)')&
                 ' The argument following the option -fd',&
                 ' must be a real number. Not : ', buf
               stop
            end if
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) wmax
            if (ierr .ne. 0 ) then
               write(6,'(2a/,a)')&
                 ' The argument following the option -fd',&
                 ' must be a real number. Not : ', buf
               stop
            end if

            if (wmax .lt. wmin) then
               tmp = wmax
               wmax = wmin
               wmin = tmp 
            end if

            lfrange = .true.

        else if(buf .eq. '-td') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) tmin
            if (ierr .ne. 0) then
               write(6,'(2a/1x,a)')&
                 ' The  argument following the option -td',&
                 ' must be a real number. Not : ', buf
               stop 
            end if

            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) tmax
            if (ierr .ne. 0) then
               write(6,'(2a/1x,a)')&
                 ' The  argument following the option -td',&
                 ' must be a real number. Not : ', buf
               stop 
            end if

            if (tmax .lt. tmin) then
               tmp = tmax
               tmax = tmin
               tmin = tmp 
            end if
            ltrange = .true.
            

         else if(buf .eq. '-g') then
            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) gwidth1 !rect
            if (ierr .ne. 0) then
               write(6,'(2a/1x,a)')&
                 ' The  argument following the option -g',&
                 ' must be a real number. Not : ', buf
               stop 
            end if

            i = i+1
            call mygetarg(i,buf)
            read(buf,*,iostat=ierr) gwidth  !exp
            if (ierr .ne. 0) then
               write(6,'(2a/1x,a)')&
                 ' The  argument following the option -g',&
                 ' must be a real number. Not : ', buf
               stop 
            end if

            gwidth = abs(gwidth)**2! sigma**2
            gwidth1 = abs(gwidth1)
            if (gwidth .lt. 1.0e-8_dop) gwidth = 1.0e-8_dop
            lwidth = .true.


         else if( buf .eq. '-ver' ) then
            call wrversion(6)
            stop


         else if( buf(1:2) .eq. '-h' .or. buf(1:2) .eq. '-?' ) then
            write(6,'(78a1)')  ('-',i=1,78)
            write(6,'(2a)') 'Purpose : Computation of the XFROG trace',&
                 ' of a 1-dim function, using a'
            write(6,'(10x,2a)') 'rectrangular gate function',&
                 ' with Gaussian shoulders.'
            write(6,*)
            write(6,'(2a)') 'Usage   : xfrog [-o -n -rf -rt -fd -td -g',&
                 ' -ver -h -?] infile'
            write(6,*)   
            write(6,'(a,/,2a)') 'Options :',&
                 ' -o FILE     : The output is written to', &
                 ' file FILE rather than to infile.xfg'
            write(6,'(2a)') '               The string FILE can be a',&
                            ' relative or a full path-name.'
            write(6,'(2a)') ' -n nfield   : Number of points to be',&
                 ' read from infile (default: all)'
            write(6,'(2a)') ' -rf nres    : Sampling of outpot in',&
                 ' frequency domain (default: nfield+1)'
            write(6,'(2a)') ' -rt nres    : Sampling of outpot in',&
                 ' time domain (default: nfield)'
            write(6,'(2a)') ' -fd min max : Frequency range of the',&
                 ' outtput (default: 0 to nfield/T)'
            write(6,'(2a)') ' -td min max : Time range of the',&
                 ' output (default: as infile)'
            write(6,'(2a)') ' -g w1 w2    : Width of the gate',&
                 'function in time domain (default: w1=w2=1):'
            write(6,'(15x,2a)') 'w1 width of rectangular part,',&
                 ' w2 width of Gaussian shoulders.'
            write(6,'(2a)') ' -ver        : Version info of this',&
                 ' program.'
            write(6,'(a)') ' -h, -?      : Print this help text.'
            write(6,*) 
            write(6,'(2a)') 'The argument infile is the file',&
                 ' containing the electric field. The'
            write(6,'(2a)') 'gate function is rectangular pulse',&
                 ' of unity height and width w1 with Gaussian '
            write(6,'(2a)') 'shoulders of the form exp(-t**2/(2*w2**2))'
            write(6,'(2a)') 'The user will be prompted for',&
                 ' missing arguments. All units are au.'
            write(6,'(78a1)')  ('-',i=1,78)
            stop

         else             
            write(6,'(2a)') ' unknown option : ', buf
            stop
         end if
         if (iarg .eq. i) goto 200
         
         goto 100
      end if
     
      
      
 200  continue

 
      filein = buf
      laein=index(buf,' ')-1
      if (filein .eq. ' ' .or. laein .le.1) then
         write(6,*) 'Name of data file   :'
         read(5,'(a)') filein
         laein=index(filein,' ')-1
         write(6,*)
      endif
     
      call abspath(filein,laein)
      
      return 
      end subroutine xfinput

!#######################################################################



      real(dop) function xfgate(t, tcenter, gwidth, gwidth1)

      implicit none
      
      real(dop), intent(in) :: t,tcenter,gwidth,gwidth1
 
      
      if ( abs((t-tcenter)) .le. gwidth1/2.0_dop ) then
         xfgate = 1.0_dop
      else if (  (t-tcenter) .gt. 0.0_dop  ) then
         xfgate = dexp(-(t-tcenter-gwidth1/2.0_dop )**2/2.0_dop/gwidth  )
      else
         xfgate = dexp(-(t-tcenter+gwidth1/2.0_dop )**2/2.0_dop/gwidth  )
      end if

      return
      end function xfgate
 
      end program xfrog


