  program main_djmn_test
  implicit none
! -----------------------------------------------------------------------
! Test of function subprogram djmn which calculates the
! "reduced Wigner rotation matrix" d^j_{m n} (beta).
! The test program calculates matrices d^j_{m n} (beta) for
! j-values 1/2, 1, 3/2, and 2; and also one case for j=12.
! -----------------------------------------------------------------------
! Ref: D.M. Brink and G.R. Satchler, Angular Momentum, 2nd ed.,
!      Oxford University Press, in particular
!      p.22 (general expression for djmn)
!      p.24 (analytic expressions for j = 1/2, 1, 3/2, and 2)
! -----------------------------------------------------------------------
! author: V. Oberacker, Vanderbilt University
! last time edited: October 6, 2010
! -----------------------------------------------------------------------
  integer, parameter :: wp = kind(1.0d0)  ! working precision = double (portable)
!-----------------------------------------------
!   local constants
!-----------------------------------------------
  real(wp), parameter :: pi = 3.14159265358979323846_wp    ! Schaum's Math handbook
!-----------------------------------------------
!   local variables
!-----------------------------------------------
  integer :: ibeta
  real(wp) :: j, m, mp, beta_deg, beta_rad, djmmp_analyt, djmmp_numeric, rel_error, &
              sb, cb
!-----------------------------------------------
!   external functions
!-----------------------------------------------
  real(wp), external :: djmn
! ------------------------------------------------------------------------------
!   output file
! ------------------------------------------------------------------------------
  open(unit=6, file='djmn_test.out', status='new', form='formatted', access='sequential')
!--------------------------------------------------------------
!  program starts here
!--------------------------------------------------------------
  write (6,'(//70(''*'')/A/70(''*''))') ' djmn_test Fortran 95 code, V.E. Oberacker'
!
  j = 0.0_wp ; m = 0.0_wp  ; mp = 0.0_wp
  write (6,'(//A,3F5.1/)') ' j, m, mp = ', j, m, mp
  write (6,'(A)') ' beta_deg, djmmp_analyt, djmmp_numeric, rel_error'
  do ibeta = 0,180,10
     beta_deg = real(ibeta,wp)
     beta_rad = beta_deg * pi / 180.0_wp
     djmmp_analyt = 1.0_wp
     djmmp_numeric = djmn(j, m, mp, beta_rad)
     rel_error = abs( (djmmp_analyt - djmmp_numeric) / djmmp_analyt )
     write (6,'(F6.1,2ES14.6,6X,ES14.6)') beta_deg, djmmp_analyt, djmmp_numeric, rel_error
  end do
!
  j = 0.5_wp ; m = 0.5_wp  ; mp = 0.5_wp
  write (6,'(//A,3F5.1/)') ' j, m, mp = ', j, m, mp
  write (6,'(A)') ' beta_deg, djmmp_analyt, djmmp_numeric, rel_error'
  do ibeta = 0,180,10
     beta_deg = real(ibeta,wp)
     beta_rad = beta_deg * pi / 180.0_wp
     djmmp_analyt = cos(beta_rad/2.0_wp)
     djmmp_numeric = djmn(j, m, mp, beta_rad)
     rel_error = abs( (djmmp_analyt - djmmp_numeric) / djmmp_analyt )
     write (6,'(F6.1,2ES14.6,6X,ES14.6)') beta_deg, djmmp_analyt, djmmp_numeric, rel_error
  end do
!
  j = 0.5_wp ; m = -0.5_wp  ; mp = 0.5_wp
  write (6,'(//A,3F5.1/)') ' j, m, mp = ', j, m, mp
  write (6,'(A)') ' beta_deg, djmmp_analyt, djmmp_numeric, rel_error'
  do ibeta = 0,180,10
     beta_deg = real(ibeta,wp)
     beta_rad = beta_deg * pi / 180.0_wp
     djmmp_analyt = sin(beta_rad/2.0_wp)
     djmmp_numeric = djmn(j, m, mp, beta_rad)
     rel_error = abs( (djmmp_analyt - djmmp_numeric) / djmmp_analyt )
     write (6,'(F6.1,2ES14.6,6X,ES14.6)') beta_deg, djmmp_analyt, djmmp_numeric, rel_error
  end do
!
  j = 1.0_wp ; m = 1.0_wp  ; mp = -1.0_wp
  write (6,'(//A,3F5.1/)') ' j, m, mp = ', j, m, mp
  write (6,'(A)') ' beta_deg, djmmp_analyt, djmmp_numeric, rel_error'
  do ibeta = 0,90,10
     beta_deg = real(ibeta,wp)
     beta_rad = beta_deg * pi / 180.0_wp
     sb = sin(beta_rad/2.0_wp)
     djmmp_analyt = sb**2
     djmmp_numeric = djmn(j, m, mp, beta_rad)
     rel_error = abs( (djmmp_analyt - djmmp_numeric) / djmmp_analyt )
     write (6,'(F6.1,2ES14.6,6X,ES14.6)') beta_deg, djmmp_analyt, djmmp_numeric, rel_error
  end do
!
  j = 1.5_wp ; m = 1.5_wp  ; mp = -0.5_wp
  write (6,'(//A,3F5.1/)') ' j, m, mp = ', j, m, mp
  write (6,'(A)') ' beta_deg, djmmp_analyt, djmmp_numeric, rel_error'
  do ibeta = 0,90,10
     beta_deg = real(ibeta,wp)
     beta_rad = beta_deg * pi / 180.0_wp
     sb = sin(beta_rad/2.0_wp)
     cb = cos(beta_rad/2.0_wp)
     djmmp_analyt = sqrt(3.0_wp) * cb * sb**2
     djmmp_numeric = djmn(j, m, mp, beta_rad)
     rel_error = abs( (djmmp_analyt - djmmp_numeric) / djmmp_analyt )
     write (6,'(F6.1,2ES14.6,6X,ES14.6)') beta_deg, djmmp_analyt, djmmp_numeric, rel_error
  end do
!
  j = 2.0_wp ; m = 2.0_wp  ; mp = 1.0_wp
  write (6,'(//A,3F5.1/)') ' j, m, mp = ', j, m, mp
  write (6,'(A)') ' beta_deg, djmmp_analyt, djmmp_numeric, rel_error'
  do ibeta = 0,90,10
     beta_deg = real(ibeta,wp)
     beta_rad = beta_deg * pi / 180.0_wp
     djmmp_analyt = -0.5_wp * sin(beta_rad) * (1.0_wp + cos(beta_rad))
     djmmp_numeric = djmn(j, m, mp, beta_rad)
     rel_error = abs( (djmmp_analyt - djmmp_numeric) / djmmp_analyt )
     write (6,'(F6.1,2ES14.6,6X,ES14.6)') beta_deg, djmmp_analyt, djmmp_numeric, rel_error
  end do
!
  j = 12.0_wp ; m = 0.0_wp  ; mp = 4.0_wp
  write (6,'(5/A,3F5.1/)') ' j, m, mp = ', j, m, mp
  write (6,'(A)') ' beta_deg, djmmp_numeric'
  do ibeta = 0,90,10
     beta_deg = real(ibeta,wp)
     beta_rad = beta_deg * pi / 180.0_wp
     djmmp_numeric = djmn(j, m, mp, beta_rad)
     write (6,'(F6.1,ES14.6)') beta_deg, djmmp_numeric
  end do
!
  stop
  end program main_djmn_test

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  function djmn (j, m, n, beta_rad)
  implicit none
! -------------------------------------------------------------------
! input: angular momentum quantum numbers j, m, n (all real)
!        beta_rad = Euler angle beta (radian)
! -------------------------------------------------------------------
  integer, parameter :: wp = kind(1.0d0)  ! working precision = double (portable)
!--------------------------------------------------------------------
!    formal arguments
!--------------------------------------------------------------------
  real(wp), intent(in) :: j, m, n, beta_rad
!--------------------------------------------------------------------
!    local variables
!--------------------------------------------------------------------
  integer :: itmin1, itmin2, itmin, itmax1, itmax2, itmax, ij1, ij2, &
             it, iphase, ia, ib, ic
  real(wp) :: djmn, cosb2, sinb2, sqrt_fac, sumt, denom, term
!--------------------------------------------------------------------
!   external functions
!--------------------------------------------------------------------
  real(wp), external :: fac10           ! computes factorial(n)/10**n
!--------------------------------------------------------------------
!  program starts here
!--------------------------------------------------------------------
  cosb2 = cos(beta_rad/2.0_wp)
  sinb2 = sin(beta_rad/2.0_wp)
!--------------------------------------------------------------------
! determine lower and upper limits for summation index it; these
! are derived from the requirement that all factorials n! in the
! denominator are restricted to values with n >=0.
!--------------------------------------------------------------------
  itmin1 = 0
  itmin2 = nint(m-n)
  itmin = max(itmin1,itmin2)
  itmax1 = nint(j+m)
  itmax2 = nint(j-n)
  itmax = min(itmax1,itmax2)
!  write (6,'(10X,A,2I6)') ' itmin, itmax = ', itmin, itmax
  ij1 = nint(j-m)
  ij2 = nint(j+n)
  sqrt_fac = sqrt( fac10(itmax1) * fac10(ij1) * fac10(ij2) * fac10(itmax2) )
!
  sumt = 0.0_wp
  do it = itmin, itmax
     iphase = (-1)**it
     ia = itmax1 - it
     ib = itmax2 - it
     ic = it + nint(n-m)
!     write (6,'(10X,A,5I6)') ' it, iphase, ia, ib, ic  = ', it, iphase, ia, ib, ic
     denom = fac10(ia) * fac10(ib) * fac10(it) * fac10(ic)
     term = iphase * cosb2**(ia+ib) * sinb2**(it+ic) / denom
     sumt = sumt + term
  end do
  djmn = sqrt_fac * sumt
!
  return
  end function djmn

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  function fac10 (n)
  implicit none
! -----------------------------------------------
! function fac10(n) calculates factorial(n)/10**n
! -----------------------------------------------
! input: integer n >= 0 (you may want to check this
!        in the program calling this function)
! -----------------------------------------------
  integer, parameter :: wp = kind(1.0d0)  ! working precision = double (portable)
!------------------------------------------------
!      formal arguments
!------------------------------------------------
  integer, intent(in) :: n
!------------------------------------------------
!      local variables
!------------------------------------------------
  integer :: i
  real(wp) :: fac10, q
! -----------------------------------------------
  if (n == 0) then
     fac10 = 1.0_wp
  else
     fac10 = 1.0_wp
     q = 1.0_wp
     do i = 1, n
        fac10 = fac10 * q / 10.0_wp
        q = q + 1.0_wp
     end do
  endif
!
  return
  end function fac10

