!**********************************************************************
      module calc_low_cimod

use decimal, only: long,dop
use maxv, only: maxdim
use vcglobal
use global, only:pi, fs, ev
use vchammod
use vchfitmod
use utils, only: vcnorm

use functions, only: vpes, dpes, dfridr, diabpes, ddiabpes

      implicit none

      real(dop), save                            :: lambda_lagrange,sigma
      real(dop), dimension(:), allocatable, save :: q_ci_low,q_ci_0,&
                                                    q_ci_delta, &
                                                    gd_low,dcp_low

      private
      public :: calc_low_ci
      contains

!**********************************************************************
!
!     
!**********************************************************************

      subroutine calc_low_ci(mr,mi)
      
      implicit none

      integer(long)                :: eqdim
      integer(long)                :: io,i,n,dim,iter,s1,s2,maxiter
      real(dop)                    :: fret1,fret2,tol,norm_low,norm_0,&
                                      norm_1,norm_2,delta_norm,citol
      real(dop)                    :: norm,E1,E2
      integer(long), dimension(midim), intent(inout) :: mi
      real(dop), dimension(mrdim), intent(inout)     :: mr

! No. equantions = nmode + 1   to include lagrange multiplier
      eqdim = maxdim+1

! No. of modes for gradients of PES in dpes, ddiabpes
      nvar = nmodes

!-----------------------------------------------------------------------
! allocate memory
!-----------------------------------------------------------------------
      if (allocated(q_ci_low)) deallocate(q_ci_low)
      if (allocated(q_ci_0)) deallocate(q_ci_0)
      if (allocated(q_ci_delta)) deallocate(q_ci_delta)
      if (allocated(gd_low)) deallocate(gd_low)
      if (allocated(dcp_low)) deallocate(dcp_low)
      allocate(q_ci_low(eqdim))
      allocate(q_ci_0(eqdim))
      allocate(q_ci_delta(eqdim))
      allocate(gd_low(maxdim))
      allocate(dcp_low(maxdim))

      iter=100
      tol=1.0e-10

      write(ilog,*) '#################################################'
      write(ilog,*)  'Calculating positions of conical intersections   '
      write(ilog,*) '#################################################'

      do s1=1,nsta-1
         s2=s1+1

         q_ci_low(:) = 0.0e0
         write(ilog,'(/,a,2i3)') '           States ',s1,s2
         write(ilog,*) '          the lowest conical intersection        '
         write(ilog,*) '================================================='
         write(ilog,*) '                                                 '
         write(ilog,*) ' n      E1        E2       delta    norm(Q)   '//&
              'lagrange     penalty '
         write(ilog,*) '----------------------------------------------'//&
              '---------------------'

! Initial Values
         n=0
         currsta = s1
         E1 =  vpes(q_ci_low,mr,mi)
         currsta = s2
         E2 =  vpes(q_ci_low,mr,mi)
         lambda_lagrange = 0.0
         sigma = 0.0
         write(ilog,'(i4,5F10.5,3x,E10.4)') n,E1,E2,E2-E1, &
              vcnorm(q_ci_low),lambda_lagrange,sigma

! First push to near the seam using a penalty function
         iter = 100
         maxiter = 100
         citol = 1.0d-3
         write(ilog,'(a)') 'Moving to Seam'
         sigma = 1.0                ! the penalty constant
         q_ci_low(nmodes+1) = 0.0   ! the langrangian multiplier
         do n=1,maxiter
            currsta = s2
            call frprmn(q_ci_low,nmodes,tol,iter,fret1,f_ci,df_ci,ilog,&
                 -999.0,0,mr,mi)
            currsta = s1
            E1 =  vpes(q_ci_low,mr,mi)
            currsta = s2
            E2 =  vpes(q_ci_low,mr,mi)
            lambda_lagrange = q_ci_low(nmodes+1)
            write(ilog,'(i4,5F10.5,3x,E10.4)') n,E1,E2,E2-E1, &
                 vcnorm(q_ci_low),lambda_lagrange,sigma

            if ((E2-E1).lt.citol) exit
            sigma = min(sigma*2.0d0,1.0d+3)
         enddo

         if (E2-E1 .gt. citol) then
            write(ilog,'(a)') 'Seam not found. Closest point printed below.'
            write(ilog,'(a,f10.8)') 'DeltaE = ',E2-E1
            maxiter = 0   ! Do not try and optimise CoIn
         else
            write(ilog,'(a)') 'Optimising CoIn'
            maxiter = 100
            sigma = 0.0
            q_ci_low(nmodes+1) = -1.0
         endif
         do n=1,maxiter
            q_ci_0(:) = q_ci_low(:)  ! save initial coords
            currsta = s2
            call frprmn(q_ci_low,nmodes+1,tol,iter,fret1,f_ci,df_ci,ilog,&
                 -999.0,0,mr,mi)
            currsta = s1
            E1 =  vpes(q_ci_low,mr,mi)
            currsta = s2
            E2 =  vpes(q_ci_low,mr,mi)
            lambda_lagrange = q_ci_low(nmodes+1)
            write(ilog,'(i4,5F10.5,3x,E10.4)') n,E1,E2,E2-E1, &
                 vcnorm(q_ci_low),lambda_lagrange,sigma

            q_ci_delta(1:nmodes+1) = q_ci_low(1:nmodes+1)-q_ci_0(1:nmodes+1) 
            norm_low = vcnorm(q_ci_delta)  ! err on coordinate change
            if ((norm_low).lt.1.0E-5) exit
         enddo

         gd_low(:)=0.0_dop
         dcp_low(:)=0.0_dop
         if (maxiter .gt. 0) then   !  Calculate GD/DCP only if CoIn found
            call coin_gddcp(s1,s2,q_ci_low,gd_low,dcp_low,mr,mi)
         endif

         norm_low = vcnorm(q_ci_low)
         norm_1 = vcnorm(gd_low)
         norm_2 = vcnorm(dcp_low)
         currsta = s2
         E2 =  vpes(q_ci_low,mr,mi)

         write(ilog,'(/,a)') ' --- Coordinates --- '
         write(ilog,'(4x,a)') &
                     'mode            Q          GD          DCP ' 
         write(ilog,'(72a)') ('-',i=1,72)
         do i=1,nmodes
            write(ilog,'(2x,i3,1x,a7,3f12.5)') i,nmlab(i),q_ci_low(i),&
                         gd_low(i),dcp_low(i)
         enddo
         write(ilog,'(72a)') ('-',i=1,71)
         write(ilog,'(a,3F12.5)')'   ||Q|| =   ',norm_low,norm_1,norm_2 
         write(ilog,'(a,F12.5,a)') '   E_ci =    ',E2,' eV'
         write(ilog,'(72a)') ('-',i=1,72)
      enddo

!     ---------- calculate the nearest conical intersection, related to
!     ---------- Q=0
!     print*, '=================================================='
!     print*, ' calculating Q_0 ...'
!     sigma           = 1.0e0
!     lambda_lagrange = 0.0e0
!     n               = 0
! 20   continue
!      iter = 100
!     call frprmn(q_ci_0,nmodes,tol,iter,fret1,f_ci_0,df_ci_0,ilog)
!     E1 =  pes2d1(q_ci_0)
!     E2 =  pes2d2(q_ci_0)
!      iter = 100
!     if (n.ne.0) call frprmn(lambda_lagrange,1,tol,iter,fret2,
!    +     f_lambda_0,df_lambda_0,ilog)
!     write(6,'(i4,5F10.5,3x,E10.4,2F10.5)') n,E1,E2,E2-E1,
!    +     norm(q_ci_0),lambda_lagrange,sigma,fret1,fret2
!      sigma = sigma*2.0e0
      
!      n = n + 1
!     if ((E2-E1).gt.1.0e-4) goto 20

!     ---------- calculate the nearest conical intersection, related to
!     ---------- |E2_min - E_ci| = min
!     print*,'==============================================='
!     print*, ' calculating Q_1 ...'
!     sigma           = 1.0e0
!     lambda_lagrange = 0.0e0
!     n               = 0
! 30   continue
!      iter = 100
!     call frprmn(q_ci_1,nmodes,tol,iter,fret1,f_ci_1,df_ci_1,ilog)
!     E1 =  pes2d1(q_ci_1)
!     E2 =  pes2d2(q_ci_1)
!      print*, E1,E2
!      iter = 100
!     if (n.ne.0) call frprmn(lambda_lagrange,1,tol,iter,fret2,
!    +     f_lambda_1,df_lambda_1,ilog)
!     write(6,'(i4,5F10.5,3x,E10.4,2F10.5)') n,E1,E2,E2-E1,
!    +     norm(q_ci_1),lambda_lagrange,sigma,fret1,fret2
!      sigma = sigma*2.0
!      n = n + 1
!     if ((E2-E1).gt.1.0e-4) goto 30


      deallocate(q_ci_low,q_ci_0,gd_low,dcp_low)

      return
      end subroutine calc_low_ci
      
!#######################################################################
! Gets GD and DCP for CoIn between states s1 and s2
!#######################################################################

      subroutine coin_gddcp(s1,s2,q,gd,dcp,mr,mi)

      implicit none

      integer(long), intent(in) :: s1,s2
      integer(long)             :: i
      real(dop), dimension(maxdim), intent(in) :: q
      real(dop), dimension(maxdim), intent(out) :: gd,dcp
      real(dop), dimension(2,2) :: vdiab
      real(dop), dimension(maxdim,2,2) :: ddiab
      real(dop), dimension(mrdim), intent(inout)     :: mr
      integer(long), dimension(midim), intent(inout) :: mi

      currsta = s1
      currsta1 = s1
      vdiab(1,1) = diabpes(q,mr)
      call ddiabpes(q,ddiab(1,1,1),mr,mi)
      currsta = s2
      currsta1 = s2
      vdiab(2,2) = diabpes(q,mr)
      call ddiabpes(q,ddiab(1,2,2),mr,mi)
      currsta = s1
      currsta1 = s2
      vdiab(1,2) = diabpes(q,mr)
      call ddiabpes(q,ddiab(1,1,2),mr,mi)
      vdiab(2,1) = vdiab(1,2) 
      ddiab(:,2,1) = ddiab(:,1,2) 

! If CoIn is displaced only along tuning modes, diabatic = adiabatic
! If not, need to get rotation matrix and rotate gradients. TO BE DONE.
      if (abs(vdiab(1,2)) .gt. 0.0001) then
         print*,'Problem in Calc_low_CI (Coin_gddcp), States',s1,s2
         print*,'Need to rotate diabatic basis. Not yet implemented'
         stop
      else
         do i=1,nmodes
            gd(i) = ddiab(i,2,2) - ddiab(i,1,1)
            dcp(i) = ddiab(i,1,2) 
         enddo
      endif

      return
      end subroutine coin_gddcp

!############################################################
!
!############################################################

      real(dop) function f_ci(q,mr,mi)

      implicit none

      real(dop), intent(in)        :: q(maxdim)
      real(dop), intent(inout)     :: mr(mrdim)
      integer(long), intent(inout) :: mi(midim)

      real(dop) :: E1,E2,delta

      E2 = vpes(q,mr,mi)    ! currsta needs to be set 
      currsta = currsta-1
      E1 = vpes(q,mr,mi) 
      delta = abs(E2-E1)

      f_ci = E2 - lambda_lagrange*delta + 0.5*sigma*delta*delta

      currsta = currsta+1  ! restore initial value

      return
      end function f_ci


!---------------------------------------------------------

      subroutine df_ci(q,dq,mr,mi)

      implicit none

      integer(long)                                 :: n
      real(dop)                                     :: h,err,delta,E1,E2
      real(dop), dimension(4*maxdim*maxdim*2)       :: work
      real(dop), dimension(*), intent(in)           :: q
      real(dop), dimension(*), intent(out)          :: dq
      real(dop), intent(inout)     :: mr(mrdim)
      integer(long), intent(inout) :: mi(midim)

      do n=1,nmodes
         h=0.001
         dq(n)=dfridr(f_ci,q,nmodes,n,h,err,work,mr,mi)
      enddo

      E2 = vpes(q,mr,mi)   ! currsta needs to be set 
      currsta = currsta-1
      E1 = vpes(q,mr,mi) 
      delta = abs(E2-E1)
      dq(nmodes+1) = delta
      currsta = currsta+1  ! restore initial value
         
      return
      end subroutine df_ci



!###################################################################
!
!###################################################################

      real(dop) function f_lambda(fit_var,q,mr,mi)

      implicit none

      real(dop)                :: help
      real(dop), intent(in)    :: fit_var
      real(dop), intent(in)    :: q(maxdim)
      integer(long)            :: i
      real(dop), intent(inout)     :: mr(mrdim)
      integer(long), intent(inout) :: mi(midim)
      real(dop), dimension(maxdim) :: dq1,dq2
      real(dop)                :: E1,E2

      currsta = 1
      E1 = vpes(q,mr,mi) 
      call dpes(q,dq1,mr,mi)
      currsta = 2
      E2 = vpes(q,mr,mi)
      call dpes(q,dq2,mr,mi)

      help = 0.0e0

      do i=1,nmodes
!        help = help + (dq1(i)-fit_var*(dq2(i)-dq1(i)))**2 
!    +        + sigma*(E2-E1)*(dq2(i)-dq1(i))
      enddo

      f_lambda = help
      
      return
      end function f_lambda

!-------------------------------------------------------------

      subroutine df_lambda(q,dq,rdum,idum)

      implicit none

      integer(long), dimension(1), intent(in)       :: idum
      real(dop)                                     :: h,err
      real(dop), dimension(4*maxdim*maxdim*2)       :: work
      real(dop), dimension(1), intent(in)           :: rdum
      real(dop), dimension(*), intent(in)           :: q
      real(dop), intent(out)                        :: dq

      h=0.1
 10   continue
      dq=dfridr(f_lambda,q,1,1,h,err,work,rdum,idum)
      write(6,*) 'err : ',err
      if (abs(err).ge.0.0001) then
         write(6,*) 'error in df_lambda to big!'
         h = h*2.0
         goto 10
!         stop
      endif
      end subroutine df_lambda

!####################################################################
!     the routines for the nearest conical intersection, rel.to QO   
!####################################################################
      real(dop) function f_ci_0(q)

      implicit none

      real(dop), intent(in)              :: q
      
!     real f_ci_0,E1,E2,delta,pes2d1,pes2d2,lambda_lagrange,sigma
!     integer i

!     E1 = pes2d1(q) 
!     E2 = pes2d2(q)
!      write(6,'(a,20e20.10)') 'A : ',  q(nmodes+1),q(nmodes+1)
!     delta = E2-E1
!     if (delta.lt.0.0) then
!        write(6,*) ' delta smaller than zero in routine f_ci !'
!        write(6,*) ' E1 = ',E1
!        write(6,*) ' E2 = ',E2
!        write(6,'(18F6.2)') (q(i),i=1,20)
!        stop
!     endif
  
!     f_ci_0 = norm(q_ci_0) - lambda_lagrange*delta +
!    +     0.5*sigma*delta*delta
      f_ci_0=0.0
     
      return
      end function f_ci_0


!####################################################################


      subroutine df_ci_0(q,dq,rdum,idum)

      implicit none

      integer(long)            :: n,dim
      integer(long), dimension(1), intent(in)       :: idum
      real(dop)                                     :: h,err
!      real(dop), external                           :: dfridr
      real(dop), dimension(4*maxdim*maxdim*2)       :: work
      real(dop), dimension(1), intent(in)           :: rdum
      real(dop), dimension(*), intent(in)           :: q
      real(dop), dimension(*), intent(out)          :: dq
!      real(dop), external                           :: f_ci_0


!      write(6,'(a,2E20.10)') 'C : ',q(nmodes+1),q(nmodes+1)
      dim=nmodes
      do n=1,dim
         h=0.001
         dq(n)=dfridr(f_ci_0,q,dim,n,h,err,work,rdum,idum)
      enddo
         
      return
      end subroutine df_ci_0



!###################################################################
!
!###################################################################

      real(dop) function f_lambda_0(fit_var)


      implicit none
      
      real(dop)                          :: help
      real(dop), intent(in)              :: fit_var
!    +     dq1(maxdim),dq2(maxdim),help,f_lambda_0,fit_var
      real(dop)       :: n
!      real(dop)       :: vcnorm
!     real lambda_lagrange,sigma,E1,E2,pes2d1,pes2d2,n,norm
      integer(long)       :: i


      help = 0.0e0
!     E1 = pes2d1(q_ci_0)
!     E2 = pes2d2(q_ci_0)
      n  = vcnorm(q_ci_0)
!     call der12d1(q_ci_0,dq1)
!     call der12d2(q_ci_0,dq2)

      do i=1,nmodes
!        help = help + (q_ci_0(i)/n - fit_var*(dq2(i)-dq1(i)))**2 
!    +        + sigma*(E2-E1)*(dq2(i)-dq1(i))
      enddo

      f_lambda_0 = help
      
      return
      end function f_lambda_0


!-------------------------------------------------------------
      subroutine df_lambda_0(q,dq,rdum,idum)

      implicit none

      integer(long), dimension(1), intent(in)       :: idum
      real(dop)                                     :: h,err
      real(dop), dimension(4*maxdim*maxdim*2)       :: work
      real(dop), dimension(1), intent(in)           :: rdum
      real(dop), dimension(*), intent(in)           :: q
      real(dop), intent(out)                        :: dq
!      real(dop), external                           :: dfridr,f_lambda_0

      h=0.01
 10   continue
      dq=dfridr(f_lambda_0,q,1,1,h,err,work,rdum,idum)
      write(6,*) 'err : ',err
      if (abs(err).ge.0.01) then
         write(6,*) 'error in df_lambda to big!'
         h = h*2.0
         goto 10
!         stop
      endif
      end subroutine df_lambda_0


 
!####################################################################
!     rel. to |E2_min-E_ci| = min      
!####################################################################
      real(dop) function f_ci_1(q)


      implicit none
      
      real(dop), intent(in) :: q

!     real f_ci_1,E1,E2,delta,pes2d1,pes2d2,lambda_lagrange,sigma
!     integer i

!     E1 = pes2d1(q) 
!     E2 = pes2d2(q)
!      print*, E1,E2
!      write(6,'(a,20e20.10)') 'A : ',  q(nmodes+1),q(nmodes+1)
!     delta = E2-E1
!     if (delta.lt.0.0) then
!        write(6,*) ' delta smaller than zero in routine f_ci !'
!        write(6,*) ' E1 = ',E1
!        write(6,*) ' E2 = ',E2
!        write(6,'(18F6.2)') (q(i),i=1,20)
!        stop
!     endif
  
!     f_ci_1 = (const(2,2)-E2)**2 - lambda_lagrange*delta +
!    +     0.5*sigma*delta*delta
      f_ci_1 = 0.0

      return
      end function f_ci_1

!####################################################################
      subroutine df_ci_1(q,dq,rdum,idum)

      implicit none

      integer(long)                                 :: n,dim
      integer(long), dimension(1), intent(in)       :: idum
      real(dop)                                     :: err,h
!      real(dop), external                           :: dfridr
      real(dop), dimension(4*maxdim*maxdim*2)       :: work
      real(dop), dimension(1), intent(in)           :: rdum
      real(dop), dimension(*), intent(in)           :: q
      real(dop), dimension(*), intent(out)          :: dq
!      real(dop), external                           :: f_ci_1

      dim=nmodes
      do n=1,dim
         h=0.001
         dq(n)=dfridr(f_ci_1,q,dim,n,h,err,work,rdum,idum)
      enddo
         
      return
      end subroutine df_ci_1




!     -----
      real(dop) function f_lambda_1(fit_var)


      implicit none
      
      integer(long)                      :: i
      real(dop), intent(in)              :: fit_var
!      real(dop)                          :: vcnorm
      real(dop)                          :: help,n
!    +     dq1(maxdim),dq2(maxdim),help,f_lambda_1,fit_var
!     real lambda_lagrange,sigma,E1,E2,pes2d1,pes2d2,n,norm

      help = 0.0e0
!     E1 = pes2d1(q_ci_1)
!     E2 = pes2d2(q_ci_1)
!      n  = vcnorm(q_ci_1)
!      print*, n
!     call der12d1(q_ci_1,dq1)
!     call der12d2(q_ci_1,dq2)

      do i=1,nmodes
!        help = help + (2.0*(const(2,2)-E2)*(-dq2(i)) - 
!    +        fit_var*(dq2(i)-dq1(i)))**2 
!    +        + sigma*(E2-E1)*(dq2(i)-dq1(i))
      enddo

      f_lambda_1 = help
      
      return
      end function f_lambda_1


!-------------------------------------------------------------
      subroutine df_lambda_1(q,dq,rdum,idum)

      implicit none

      integer(long), dimension(1), intent(in)       :: idum
      real(dop)                                     :: h,err
      real(dop), intent(out)                        :: dq
      real(dop), dimension(1), intent(in)           :: rdum
      real(dop), dimension(*), intent(in)           :: q
      real(dop), dimension(4*maxdim*maxdim*2)       :: work
!      real(dop), external                           :: dfridr,f_lambda_1
      
 
      h=0.01
 10   continue
      dq=dfridr(f_lambda_1,q,1,1,h,err,work,rdum,idum)
      write(6,*) 'err : ',err
      if (abs(err).ge.0.0001) then
         write(6,*) 'error in df_lambda_1: error to big!'
         h = h*2.0
         goto 10
!         stop
      endif
      end subroutine df_lambda_1
 
      end module calc_low_cimod
