!##########################################################
      module density_vcham

use decimal, only: dop, long
use vcglobal
use global, only:pi, fs, ev
use maxv, only: maxdim,maxsta,maxset
use vchammod


      implicit none
      private
      public :: statedensity
      contains

!##########################################################
!                                                          
!     calculates the density of states in a harmonic       
!     osscilator approximation                             
!                                                          
!     7/2000 Christoph Cattarius                           
!                                                          
!##########################################################
      subroutine statedensity(E1_par,E2_par)

      implicit none
      
      real(dop), intent(in)              :: E1_par,E2_par

      real(dop)                          :: delta,E1,E2,E_zero,sum
      real(dop), dimension(maxdim)       :: omega
      real(dop), dimension(1000)         :: density
      integer(long)                      :: noss,n,grid

      do n=1,1000
         density(n) = 0.0e0
      enddo
      do n=1,maxdim
         omega(n) = 0.0e0
      enddo
           
      E_zero = 0.0e0
      E1     = E1_par
      E2     = E2_par
!      delta  = (E2-E1)/grid

!     the linear model, 5 modes
      omega(1) = freq0(5)
      omega(2) = freq0(8)
      omega(3) = freq0(12)
      omega(4) = freq0(14)
      omega(5) = freq0(15)
!      E1 = E1 - 0.52325
!      E2 = E2 - 0.52325
      noss = 5

!     the 1-2 model, 15 modes
!      do n=1,10
!         omega(n) = freq0(n)
!      enddo
!      omega(11) = freq0(12)
!      omega(12) = freq0(14)
!      omega(13) = freq0(15)
!      omega(14) = freq0(17)
!      omega(15) = freq0(18)
!      noss = 15

!     the 2-2 model, 18 modes
!      do n=1,nmodes
!         omega(n) = freq0(n)
!      enddo
!      noss = nmodes

      do n=1,noss
         E_zero = E_zero + 0.5 * omega(n)
      enddo
      E1 = E1 + E_zero
      E2 = E2 + E_zero
 
      grid  = int((E2-E1)/omega(1))
      delta = (E2-E1)/grid

      print*,grid,delta,noss

      write(6,*)'#################################################'
      write(6,*)'#                                               #'
      write(6,*)'#      Estimating the density of states         #'
      write(6,*)'#                                               #'
      write(6,*)'#     (harmonic osscilator approximation)       #'
      write(6,*)'#                                               #'
      write(6,*)'#################################################'
      write(6,*)' '
      write(6,*)' '
      write(6,*)' '
      write(6,*)' '

      call hocount(omega,density,E1,1,noss,grid,E1,E2)
 
      sum = 0.0_dop
      do n=1,grid
         sum = sum + density(n)
         write(6,'(F5.2,2(1PE15.5))') E1+n*delta,density(n),sum
         write(idens,'(F5.2,2(1PE15.5))') E1+n*delta,density(n),sum 
      enddo

      return
      end subroutine statedensity


!-------------------------------------------------------------
!
!-------------------------------------------------------------
      subroutine hocount(omega,density,E,level,nf,np,E0,E1)
         
      implicit none

      integer(long)                          :: i,j,nmax
      integer(long), intent(in)              :: level,nf,np
      real(dop)                              :: Enew,lam,lam0,lam1
      real(dop), intent(in)                  :: E,E0,E1
      real(dop), dimension(nf), intent(in)   :: omega
      real(dop), dimension(*), intent(inout) :: density


      nmax = int( (E1-E)/omega(level) )
      
      do i=0,nmax
         Enew = E + i*omega(level)
         if (level .lt. 5) then
!           call count(omega,density,Enew,level+1,nf,np,E0,E1)
         else
            lam = 1239.85 / Enew
            lam0 = 1239.85 / E0
            lam1 = 1239.85 / E1
            j = int( np*(lam0-lam)/(lam0-lam1) ) + 1
            density(j) = density(j) + 1.0_dop
         endif
      enddo

      return
      end subroutine hocount  



      end module density_vcham
