!#######################################################################

      module findminmod

use decimal, only: long,dop

use vcglobal
use global, only: pi,fs,ev,keyword,keyorig,maxkey
use maxv, only: maxdim,maxsta
use vchammod
use vchfitmod
use functions

use logdat, only: message, routine, inptit
use utils, only: q2x,vcerrormsg,vcnorm,vccntrl

      implicit none
      private
      public :: findmin,findminrd0,jtmin

      contains

!#######################################################################

      subroutine findmin(mr,mi)

      implicit none

      integer(long), dimension(midim), intent(inout) :: mi
      real(dop), dimension(mrdim), intent(inout)     :: mr
      integer(long)                             :: s,n,nmin,i
      real(dop)                                 :: lower
! Find 1 minimum per diabatic state
! emin(nmin,s) energy of minima
! qmin(q,nmin,s) coordinates
      real(dop), dimension(maxsta,maxsta)       :: emin
      real(dop), dimension(maxdim,maxsta,maxsta) ::qmin

      emin(:,:) = 0.0
      qmin(:,:,:)=0.00
      
!-----------------------------------------------------------------------
! assume number of minima = no. of diabatic states 
! guess minima coordinates
!-----------------------------------------------------------------------

      write(ilog,'(72a1)') ('-',i=1,72)

!-----------------------------------------------------------------------
! Find diabatic minima 
!-----------------------------------------------------------------------
      call guessmin(qmin,mr(mrkappa))

      lower=-9999.0
      nvar=nmodes
      write(6,*)
      do n=1,nsta
         write(ilog,'(a,i5)') 'Searching for diabatic minimum: ',n
         write(6,'(a,i5)') 'Searching for diabatic minimum : ',n

         currsta=n
         currsta1=n
         iter = 100
         tol=1.0e-10
         call frprmn(qmin(1,n,1),nmodes,tol,iter,emin(n,1),diabpes,&
              ddiabpes,ilog,lower,2,mr,mi)

! get final energy
         emin(n,1)=diabpes(qmin(1,n,1),mr)
            
      enddo
      write(6,*)

      call outmin(qmin,emin,nsta,1,maxsta,0)

!-----------------------------------------------------------------------
! Find minima starting from each diabatic well on each adiabatic state
!-----------------------------------------------------------------------
       do s=2,nsta
          qmin(:,:,s) = qmin(:,:,1)
       enddo

      lower=-9999.0
      nvar=nmodes
      nmin=nsta
      do s=1,nsta
         do n=1,nmin
            write(ilog,'(/,2(a,i5))') 'Searching for minima: ',n,' State: ',s
            write(6,'(2(a,i5))') 'Searching for minima : ',n,' State: ',s
            currsta=s
            iter = 50
            tol=1.0e-10
            call frprmn(qmin(1,n,s),nmodes,tol,iter,emin(n,s),vpes,dpes,&
                 ilog,lower,2,mr,mi)

! get final energy
            emin(n,s)=vpes(qmin(1,n,s),mr,mi)
            
         enddo
         write(ilog,*)
         write(6,*)
      enddo

      call outmin(qmin,emin,nmin,nsta,maxsta,1)

      return
      end subroutine findmin

!#######################################################################

      subroutine findminrd0(mr,mi)

      implicit none

      integer(long), dimension(midim), intent(inout) :: mi
      real(dop), dimension(mrdim), intent(inout)     :: mr
      integer(long)                             :: s,n,nmin,i,f,n0
      integer(long)                             :: iz,iz0,ic,lc,ierr
      real(dop)                                 :: lower
! Find 1 minimum per diabatic state
! emin(nmin,s) energy of minima
! qmin(q,nmin,s) coordinates
      integer(long), dimension(:), allocatable :: smin
      real(dop), dimension(:), allocatable     :: emin
      real(dop), dimension(:,:), allocatable   ::qmin, qmin0

      logical(kind=4) :: lcheck

      inquire(iin,opened=lcheck)
      if (.not. lcheck) then
         open(iin,file=trim(filein),status='old',form='formatted',err=900)
      endif

!-----------------------------------------------------------------------
! Get number of minima to find
!-----------------------------------------------------------------------
      rewind(iin)
      n=0
      do
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         routine = 'Findminrd0 (findminmod.f90)'
         call vccntrl(ierr)

         if (keyword(1) .eq. 'minguess') then
            n=n+1
         else if (keyword(1) .eq. 'end-minguess-input') then
            nmin=n
            exit
         endif
      enddo
      if (nmin .eq. 0) then
         routine='Findminrd0 (findminmod.f90)'
         message = 'No minima specified to optimised '
         call vcerrormsg
      endif

      allocate(smin(nmin))
      allocate(emin(nmin))
      allocate(qmin0(maxdim,nmin))
      allocate(qmin(maxdim,nmin))

      smin(:) = 0
      emin(:) = 0.0
      qmin0(:,:)=0.00
      qmin(:,:)=0.00
      
!-----------------------------------------------------------------------
! read start geometries
!-----------------------------------------------------------------------
      rewind(iin)
      routine = 'Findminrd0 (findminmod.f90)'
      n0 = 0
      do
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         call vccntrl(ierr)

         if (keyword(1) .eq. 'minguess') then
            n=n0+1
            read(keyword(2),*,iostat=ierr) smin(n)
            if (ierr .ne. 0) then
               message = 'Error reading state for optimisation '
               call vcerrormsg
            endif
            do
               call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
               if (keyword(1) .eq. 'end-minguess') then
                  n0=n  ! save value of n before exiting
                  exit
               else
                  read(keyword(1),*,iostat=ierr) f
                  read(keyword(2),*,iostat=ierr) qmin0(f,n)
                  if (ierr .ne. 0) then
                     write(message,'(a,i0)') &
                         'Error reading coordniates for struture ',n
                     call vcerrormsg
                  endif
               endif
            enddo

         else if (keyword(1) .eq. 'end-minguess-input') then
            exit
         endif
      enddo

!-----------------------------------------------------------------------
! Find minima 
!-----------------------------------------------------------------------
      write(ilog,'(72a1)') ('-',i=1,72)

      lower=-9999.0
      nvar=nmodes
      qmin(:,:) = qmin0(:,:)
      write(6,*)
      do n=1,nmin
         write(ilog,'(a,i5)') 'Searching for minimum from structure: ',n
         write(6,'(a,i5)') 'Searching for minimum from structure: ',n

         currsta=smin(n)
         iter = 50
         tol=1.0e-10
         call frprmn(qmin(1,n),nmodes,tol,iter,emin(n),vpes,dpes,&
              ilog,lower,2,mr,mi)

! get final energy
         emin(n)=vpes(qmin(1,n),mr,mi)
            
      enddo
      write(6,*)

      call outminrd0(qmin,qmin0,emin,smin,nmin)

      deallocate(smin,emin,qmin0,qmin)

      return
  900 continue
      routine='Findminrd0 (findminmod.f90)'
      message = 'Cannot open input file: '//trim(filein)
      call vcerrormsg
      stop 
      
      end subroutine findminrd0

!#######################################################################
!
! Guess minima of diabatic states approximated by HO
!
!#######################################################################

      subroutine guessmin(qmin,kappa)

      implicit none
      
      integer(long) :: i,n
      real(dop), dimension(maxdim,nsta), intent(out) :: qmin
      real(dop), dimension(nmodes,nsta), intent(in)  :: kappa

      do n=1,nsta
         do i=1,nmodes
            qmin(i,n) = -kappa(i,n) / freq0(i)
         enddo
      enddo

      return
      end subroutine guessmin

!#######################################################################

      subroutine jtmin(kappa,maxkappa,sign,jt1,jt2)

      implicit none

      integer(long)                                 :: i
      integer(long), intent(out)                    :: jt1,jt2
      real(dop)                                     :: diff
      real(dop), intent(out)                        :: maxkappa,sign
      real(dop), dimension(nmodes,nsta), intent(in) :: kappa

      jt1=0
      jt2=0
      maxkappa=0.0
      do i=1,nmodes
         if (nmlab(i) .eq. jtmode(1)) then
            if (abs(kappa(i,1)) .gt. maxkappa) then
               jt1=i
               maxkappa=kappa(i,1)
            endif
         endif
      enddo
      if (jt1 .gt. 0) then
         do i=1,nmodes
            diff=freq0(i)-freq0(jt1)
            if (abs(diff) .lt. 1.0e-6_dop) jt2=i
         enddo
      endif
      if (jt2 .eq. 0) then
         print*,'jt2 not found'
         stop
      endif
      if (nmlab(jt2) .ne. jtmode(1)) then
         print*,'jt2 not JT active'
         stop
      endif

      if (kappa(jt1,1) .lt. 0) then
         sign=1.0
      else
         sign=-1.0
      endif

      return
      end subroutine jtmin

!#######################################################################
!
! OUTMIN
! write results of minima optimisations to log file
!
! modus = 0  : diabatic minima
!         1  : adiabatic minima
!
! For diabatic surfaces, nsta = 1 and nmin = no. diabatic states
! For adiabatic surfaces, nsta = no. adiabatic states and 
!                         nmin = no. diabatic states   (nmin = nsta)
!
! For adiabatic minima there may be multiple copies of minima and
! these need to be removed.
!#######################################################################

      subroutine outmin(qmin,emin,nmin,nsta,maxmin,modus)

      implicit none

      integer(long)                                 :: i,j,s,n,natm,nwr,n0
      integer(long), dimension(nsta)                :: tmpnmin
      integer(long), intent(in)                     :: nmin,nsta,maxmin,&
                                                       modus
      real(dop)                                     :: qnorm
      real(dop), dimension(3,ncoo/3)                :: xcoo
      real(dop), dimension(maxmin,nsta), intent(in)        :: emin
      real(dop), dimension(maxmin,nsta)                    :: tmpemin
      real(dop), dimension(maxdim,maxmin,nsta), intent(in) :: qmin
      real(dop), dimension(maxdim,maxmin,nsta)             :: tmpqmin
      real(dop), dimension(maxdim)                         :: delqmin

      logical(kind=4), dimension(nmin) :: found 


      if (modus .eq. 0) then
         write(ilog,'(/,a)') '*** Minima on Diabatic PESs ***'
         tmpqmin = qmin
         tmpemin = emin
         do s=1,nsta
            tmpnmin(s) = nmin
         enddo

      else if (modus .eq. 1) then
         write(ilog,'(/,a)') '*** Minima on Adiabatic PESs ***'

! Search for copies of minima
         do s=1,nsta
            found(:) = .false.
            n0=0
            do i=1,nmin
               if (found(i)) then
                  cycle
               else   ! store unique minimum
                  n0=n0+1
                  tmpqmin(:,n0,s) = qmin(:,i,s) 
                  tmpemin(n0,s) = emin(i,s) 
               endif

               do j=i+1,nmin
                  do n=1,nmodes
                     delqmin = qmin(n,j,s) - qmin(n,i,s)
                  enddo
                  qnorm = vcnorm(delqmin)
                  if (qnorm .lt. 1.0d-4) found(j) = .true.
               enddo
            enddo
            tmpnmin(s) = n0
         enddo
      endif

      do s=1,nsta
         if (modus .eq. 1) &
               write(ilog,'(/,a,i3,a)') '*** Adiabatic State ',s,' ***'

         n0=1
         nwr = min(tmpnmin(s),7)
         do while (nwr .ge. n0) 
            write(ilog,'(a8,2x,7f10.4)') 'Energy :',(tmpemin(n,s),n=n0,nwr)
            do i=1,nmodes
               write(ilog,'(i3,2x,a5,7f10.4)') i,nmlab(i),&
                  (tmpqmin(i,n,s),n=n0,nwr)
            enddo
            write(ilog,*)
            n0 = nwr
            nwr = min((tmpnmin(s)-n0),7)
            nwr = nwr+n0
            n0 = n0+1
         enddo

         if (lmininx) then
            do n=1,tmpnmin(s)
               call q2x(xcoo,xcoo0,tmpqmin(1,n,s),nmcoo,nmodes,ncoo,maxdim)

               write(ilog,'(a,i5,a,i5)') 'Coordinates for Min.: ',n,&
                          ' State: ',s
               
               natm = ncoo/3
               do i=1,natm
           !       write(ilog,'(a8,2x,3f10.6)') atsym(i),xcoo(1:3,i)
! HACK as atsym are not read in. Need to be added to info file!!
                  write(ilog,'(a8,2x,3f10.6)') 'C',xcoo(1:3,i)
               enddo
               write(ilog,*)
            enddo
         endif

      enddo

      return
      end subroutine outmin

!#######################################################################
!
! OUTMINRD0
! write results of minima founr by optimisations from given structures 
! to log file
!
!#######################################################################

      subroutine outminrd0(qmin,qmin0,emin,smin,nmin)

      implicit none

      integer(long)                                 :: i,j,s,n,natm,nwr,n0
      integer(long), intent(in)                     :: nmin
      real(dop)                                     :: qnorm
      real(dop), dimension(3,ncoo/3)                :: xcoo
      integer(long), dimension(nmin), intent(in)    :: smin
      real(dop), dimension(nmin), intent(in)        :: emin
      real(dop), dimension(maxdim,nmin), intent(in) :: qmin, qmin0

      write(ilog,'(/,a)') '*** Minima on PESs from Given Structures***'

      write(ilog,'(/,a)') '*** Starting Structures ***'

      n0=1
      nwr = min(nmin,7)
      do while (nwr .ge. n0) 
         write(ilog,'(a9,1x,7(i7,3x))') 'Minimum :',(n,n=n0,nwr)
         do i=1,nmodes
            write(ilog,'(i3,2x,a5,7f10.4)') i,nmlab(i),&
               (qmin0(i,n),n=n0,nwr)
         enddo
         write(ilog,*)
         n0 = nwr
         nwr = min((nmin-n0),7)
         nwr = nwr+n0
         n0 = n0+1
      enddo

      write(ilog,'(/,a)') '*** Final Optimised Structures ***'

      n0=1
      nwr = min(nmin,7)
      do while (nwr .ge. n0) 
         write(ilog,'(a9,1x,7(i7,3x))') 'Minimum :',(n,n=n0,nwr)
         write(ilog,'(a9,1x,7(i7,3x))') 'State :',(smin(n),n=n0,nwr)
         write(ilog,'(a9,1x,7f10.4)') 'Energy :',(emin(n),n=n0,nwr)
         do i=1,nmodes
            write(ilog,'(i3,2x,a5,7f10.4)') i,nmlab(i),&
               (qmin(i,n),n=n0,nwr)
         enddo
         write(ilog,*)
         n0 = nwr
         nwr = min((nmin-n0),7)
         nwr = nwr+n0
         n0 = n0+1
      enddo

      if (lmininx) then
         do n=1,nmin
            call q2x(xcoo,xcoo0,qmin(1,n),nmcoo,nmodes,ncoo,maxdim)

            write(ilog,'(a,i5,a,i5)') 'Coordinates for Min.: ',n,&
                       ' State: ',s
            
            natm = ncoo/3
            do i=1,natm
        !       write(ilog,'(a8,2x,3f10.6)') atsym(i),xcoo(1:3,i)
! HACK as atsym are not read in. Need to be added to info file!!
               write(ilog,'(a8,2x,3f10.6)') 'C',xcoo(1:3,i)
            enddo
            write(ilog,*)
         enddo
      endif

      return
      end subroutine outminrd0

      end module findminmod
