!#######################################################################
      module functions

use vcglobal
use maxv, only: maxdim,maxsta,maxset
use global, only:pi, fs, ev
use decimal, only: long, dop
use op1lib,  only: unitqxd
use timing
use vchammod
use vchfitmod

      implicit none
      private
      public :: pes,conv_fdiab,mosc,dmosc,vcweight,dfridr,vpes,dpes,&
                diabpes,ddiabpes,ddipole,dpolar,iscurvnm,q2xmod
      contains

!#######################################################################
!
! for a coordinate q returns adiabatic energies in v
!
! Modified to also allow for the returning of the adiabatic-to-diabatic
! transformation matrix and diabatic potential matrix if required
! S. Neville 03/10/11 
!
!#######################################################################

      subroutine pes(q,v,const,kappa,lambda,gamma,iota,mu,expcoe,&
                 higher,termk,cpcoe,switch,cp2d,leigvec,diabvec,diabv)

      implicit none

      integer(long) :: i,j,k,l,error,e2,nhi,s,n,m,p,z
      real(dop), dimension(*), intent(in)                       :: q
      real(dop), dimension(*), intent(out)                      :: v
      real(dop)                                                 :: tmp
      real(dop)                                                 :: HH,GG,SS,C1,&
                                                                   XX,YY,mi,PP,&
                                                                   C2,phi,RR,m0,&
                                                                   sd,gr,T,chi,ma,ff
      real(dop), dimension(maxsta)                              :: vadiab
      real(dop), dimension(nhigher), intent(in)                 :: higher
      real(dop), dimension(nadd), intent(inout)                 :: termk
      real(dop), dimension(3*maxsta)                            :: work
      real(dop), dimension(maxsta,maxsta)                       :: ham
      real(dop), dimension(nsta,nsta), intent(inout)            :: const
      real(dop), dimension(nmodes,nsta), intent(inout)          :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(inout)     :: lambda
      real(dop), dimension(maxcoe,nmodes,nsta), intent(inout)   :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta), intent(inout)   :: gamma
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: mu,iota
      real(dop), dimension(maxcoe,nmodes,nsta,nsta), intent(inout) :: cpcoe
      real(dop), dimension(nswitch,maxcoe), intent(in)           :: switch
      logical(kind=4)                         :: leigvec,igno
      real(dop), dimension(maxsta,maxsta)     :: diabvec
      real(dop), dimension(maxsta,maxsta)     :: diabv
      real(dop), dimension(maxdim)            :: tfreq0
      real(dop), dimension(maxcoe,maxadd), intent(in)         :: cp2d

      integer(long), save :: tid=0
      integer(long), save :: tid1=0

      call get_timer(tid, 'Wconstruct')
      call start_timer(tid)

!       print*,iota(105,105,1,1)

! new, general switching functions
      if (lswitch) call switchfuncs(q,kappa,lambda,gamma,&
           iota,mu,expcoe,termk,cpcoe,switch,+1)

! curvilinear coords based on normal modes
      if( lnmcurv) then
         i = 1
         tfreq0= freq0
         do while ( nmcurv(i,1) .ne. 0)
            if (nmcurvtype(i) .eq. 1 ) then
!      print*, 'nmcurv(i,123) i =', i , nmcurv(i,1) , nmcurv(i,2), nmcurv(i,3)
!       print*, 'i =', i,  'type = ', nmcurvtype(i), 'freqs set to zer'           
               if(.not. lmodesym ) freq0( nmcurv(i,1) ) = 0.0
               if(.not. lmodesym ) freq0( nmcurv(i,2) ) = 0.0
               if(.not. lmodesym ) freq0( nmcurv(i,3) ) = 0.0
            endif
            i = i + 1
         enddo
      endif
      
!     if the modes are not frequency scaled then the zeroth order changes slightly:
      if( lalleig ) then
         if(.not. lnmcurv) tfreq0= freq0
         do i=1,nmodes
            freq0(i) = freq0(i)**2
         enddo
      endif

! constant
      do i=1,nsta
         do j=1,nsta
            ham(j,i)=const(j,i)
         enddo
      enddo
      
      if (i_ignore .gt. 0) nmodes = nmodes - i_ignore
 
! on-diagonal (diabatic state) term
      do i=1,nsta
         do k=1,nmodes
            if (ifdiab(adof(k),i) .eq. 0) then
               ham(i,i)=ham(i,i)+0.5*freq0(adof(k))*q(adof(k))*q(adof(k))
               ham(i,i)=ham(i,i)+kappa(adof(k),i)*q(adof(k))        
               ham(i,i)=ham(i,i)+0.5*gamma(adof(k),adof(k),i)*q(adof(k))*q(adof(k))
            else
 
              call fdiab(ham(i,i),q(adof(k)),ifdiab(adof(k),i), &
                    expcoe(1,adof(k),i),ncoeff(adof(k),i),freq0(adof(k)),const(i,i))
            endif

! coupling between modes
            if (hamord.ge.2) then
               do j=1,k-1
                  ham(i,i)=ham(i,i)+0.5*gamma(adof(j),adof(k),i)*q(adof(k))*q(adof(j))
               enddo
               do j=k+1,nmodes
                  ham(i,i)=ham(i,i)+0.5*gamma(adof(j),adof(k),i)*q(adof(k))*q(adof(j))
               enddo
            endif
! third order terms
            if (hamord.ge.3) then
               do j=1,k-1
                  ham(i,i)=ham(i,i)+0.5_dop*iota(adof(j),adof(k),i,i)*q(adof(k))*q(adof(k))*q(adof(j))
               enddo
               ham(i,i)=ham(i,i)+0.1666667*iota(adof(k),adof(k),i,i)*q(adof(k))*q(adof(k))*q(adof(k))
               do j=k+1,nmodes
                  ham(i,i)=ham(i,i)+0.5_dop*iota(adof(j),adof(k),i,i)*q(adof(k))*q(adof(k))*q(adof(j))
               enddo
            endif
         enddo
      enddo

! other higher order terms
      nhi = 0
      if (highord .ge. 3) then
         do s=1,nsta
            do i=1,nmodes
               do j = i+1,nmodes
                  do k = j+1,nmodes
                     nhi = nhi+1
                     ham(s,s) = ham(s,s) + higher(nhi)*q(adof(i))*q(adof(j))*q(adof(k))
                  enddo
               enddo
            enddo
         enddo
      endif

      if (highord .eq. 4) then
! gam_iijj q_i^2 q_j^2 terms
         do s=1,nsta
            do i=1,nmodes-1
               do j = i+1,nmodes
                  nhi = nhi+1
                  ham(s,s) = ham(s,s) + &
                        0.5_dop*higher(nhi)*q(adof(i))*q(adof(i))*q(adof(j))*q(adof(j))
               enddo
            enddo
         enddo

! gam_iiij q_i^3 q_j terms
         do s=1,nsta
            do i=1,nmodes-1
               do j = i+1,nmodes
                  nhi = nhi+1
                  ham(s,s) = ham(s,s) + &
                        0.166667_dop*higher(nhi)*q(adof(i))*q(adof(i))*q(adof(i))*q(adof(j))
               enddo
            enddo
            do i=2,nmodes
               do j = 1,i-1
                  nhi = nhi+1
                  ham(s,s) = ham(s,s) + &
                        0.166667_dop*higher(nhi)*q(adof(i))*q(adof(i))*q(adof(i))*q(adof(j))
               enddo
            enddo
         enddo

! gam_iijk q_i^2 q_j q_k terms
         do s=1,nsta
            do i=1,nmodes-2
               do j = i+1,nmodes-1
                  do k = j+1,nmodes
                     nhi = nhi+1
                     ham(s,s) = ham(s,s) + &
                        1.0_dop*higher(nhi)*q(adof(i))*q(adof(i))*q(adof(j))*q(adof(k))
                  enddo
               enddo
            enddo
            do i=3,nmodes
               do j = 1,i-2
                  do k = j+1,i-1
                     nhi = nhi+1
                     ham(s,s) = ham(s,s) + &
                        1.0_dop*higher(nhi)*q(adof(i))*q(adof(i))*q(adof(j))*q(adof(k))
                  enddo
               enddo
            enddo
            do i=2,nmodes-1
               do j = 1,i-1
                  do k = i+1,nmodes
                     nhi = nhi+1
                     ham(s,s) = ham(s,s) + &
                        1.0_dop*higher(nhi)*q(adof(i))*q(adof(i))*q(adof(j))*q(adof(k))
                  enddo
               enddo
            enddo
         enddo

! gam_ijkl q_i q_j q_k q_l terms
         do s=1,nsta
            do i=1,nmodes-3
               do j = i+1,nmodes-2
                  do k = j+1,nmodes-1
                     do l = k+1,nmodes
                        nhi = nhi+1
                        ham(s,s) = ham(s,s) +& 
                           1.0_dop*higher(nhi)*q(adof(i))*q(adof(j))*q(adof(k))*q(adof(l))
                     enddo
                  enddo
               enddo
            enddo
         enddo
      endif

!    hi simon, this is just a temporary flag that allows me to add arbitrary functions 
!    directly into the code when running vchfit and vcplot ("poofit" key in vchfit and -poo in vcplot)
!    ill remove it eventually, but please dont remove it yet  :P
      if(lpoo) then    

        ham(5,8) = ham(5,8) + 1.0*(tanh( lambda(1,5,8)*(q(adof(1)) -lambda(1,6,7))) +1)
        ham(8,5) = ham(8,5) + 1.0*(tanh( lambda(1,5,8)*(q(adof(1)) -lambda(1,6,7))) +1)
        ham(7,6) = ham(7,6) + 1.0*(tanh( lambda(1,5,8)*(q(adof(1)) -lambda(1,6,7))) +1)
        ham(6,7) = ham(6,7) + 1.0*(tanh( lambda(1,5,8)*(q(adof(1)) -lambda(1,6,7))) +1)

        ham(5,8) = ham(5,8) - 1.0*(tanh( -lambda(1,5,8)*(q(adof(4))+lambda(1,6,7))) +1)
        ham(8,5) = ham(8,5) - 1.0*(tanh( -lambda(1,5,8)*(q(adof(4))+lambda(1,6,7))) +1)
        ham(7,6) = ham(7,6) - 1.0*(tanh( -lambda(1,5,8)*(q(adof(4))+lambda(1,6,7))) +1)
        ham(6,7) = ham(6,7) - 1.0*(tanh( -lambda(1,5,8)*(q(adof(4))+lambda(1,6,7))) +1)

!        lg = sqrt(2.0)*0.5
!       ham(2,5) = ham(2,5) + 2.5438*sin(lg*q(2))*cos(lg*q(3)) + 2.5438*sin(lg*q(3))*cos(lg*q(2))
!       ham(2,6) = ham(2,6) - 2.5438*sin(lg*q(6))*cos(lg*q(7)) - 2.5430*sin(lg*q(7))*cos(lg*q(6))
!       ham(2,7) = ham(2,7) - 1.5316*sin(lg*q(6))*cos(lg*q(7)) - 1.5310*sin(lg*q(7))*cos(lg*q(6))
!       ham(2,8) = ham(2,8) + 1.5316*sin(lg*q(2))*cos(lg*q(3)) + 1.5316*sin(lg*q(3))*cos(lg*q(2))
!       ham(3,5) = ham(3,5) + 1.5361*sin(lg*q(2))*cos(lg*q(3)) + 1.5361*sin(lg*q(3))*cos(lg*q(2))
!       ham(3,6) = ham(3,6) + 1.5361*sin(lg*q(6))*cos(lg*q(7)) + 1.5361*sin(lg*q(7))*cos(lg*q(6))
!       ham(3,7) = ham(3,7) + 1.5595*sin(lg*q(6))*cos(lg*q(7)) + 1.5595*sin(lg*q(7))*cos(lg*q(6))
!       ham(3,8) = ham(3,8) + 1.5595*sin(lg*q(2))*cos(lg*q(3)) + 1.5595*sin(lg*q(3))*cos(lg*q(2))
!       ham(4,5) = ham(4,5) + 1.4072*sin(lg*q(6))*cos(lg*q(7)) + 1.4072*sin(lg*q(7))*cos(lg*q(6))
!       ham(4,6) = ham(4,6) - 1.4072*sin(lg*q(2))*cos(lg*q(3)) - 1.4070*sin(lg*q(3))*cos(lg*q(2))
!       ham(4,7) = ham(4,7) - 1.7589*sin(lg*q(2))*cos(lg*q(3)) - 1.7580*sin(lg*q(3))*cos(lg*q(2))
!       ham(4,8) = ham(4,8) + 1.7589*sin(lg*q(6))*cos(lg*q(7)) + 1.7589*sin(lg*q(7))*cos(lg*q(6))
!       ham(2,5) = ham(5,2)
!       ham(2,6) = ham(6,2)
!       ham(2,7) = ham(7,2)
!       ham(2,8) = ham(8,2)
!       ham(3,5) = ham(5,3)
!       ham(3,6) = ham(6,3)
!       ham(3,7) = ham(7,3)
!       ham(3,8) = ham(8,3)
!       ham(4,5) = ham(5,4)
!       ham(4,6) = ham(6,4)
!       ham(4,7) = ham(7,4)
!       ham(4,8) = ham(8,4)

      endif

! off-diagonal (coupling)
      do i=1,nsta-1
         do l=i+1,nsta
            do k=1,nmodes            
               if (ifdiabcp(adof(k),l,i).eq.0) then
                  if( lsincoup ) ham(i,l)=ham(i,l)+lambda(adof(k),i,l)*sin(q(adof(k)))
    if( lcoscoup(k) ) then
     ham(i,l)=ham(i,l)+lambda(adof(k),i,l)*cos(q(adof(k)))
!    write(*,*)"284 k,q",k,q(adof(k))
!    write(*,*)"284 cos q",cos(q(adof(k)))
!    write(*,*)"284 ham",ham(i,l)
    endif
    if((.not. lsincoup) .and. (.not. lcoscoup(k))) ham(i,l) &
     =ham(i,l)+lambda(adof(k),i,l)*q(adof(k))
    ham(i,l)=ham(i,l)+0.5*mu(adof(k),adof(k),i,l)*q(adof(k))*q(adof(k))
 !   write(*,*)"289",k,ham(i,l)
               else
                  call evalcpfunc(ham(i,l),q(adof(k)),ifdiabcp(adof(k),i,l),&
                       ncoeffcp(adof(k),i,l),cpcoe(1,adof(k),i,l))
               endif
               do j=1,k-1
                  ham(i,l)=ham(i,l)+0.5*mu(adof(j),adof(k),i,l)*q(adof(k))*q(adof(j))
              ! write(*,*)"296",ham(i,l)
               enddo
               do j=k+1,nmodes
                  ham(i,l)=ham(i,l)+0.5*mu(adof(j),adof(k),i,l)*q(adof(k))*q(adof(j))
              ! write(*,*)"300",ham(i,l)
               enddo
               do j=1,k-1
                  ham(i,l)=ham(i,l)+0.5_dop*iota(adof(j),adof(k),i,l)*q(adof(k))*q(adof(k))*q(adof(j))
               enddo
               ham(i,l)=ham(i,l)+0.1666667*iota(adof(k),adof(k),i,l)*q(adof(k))*q(adof(k))*q(adof(k))
              ! write(*,*)"306",ham(i,l)
               do j=k+1,nmodes
                  ham(i,l)=ham(i,l)+0.5_dop*iota(adof(j),adof(k),i,l)*q(adof(k))*q(adof(k))*q(adof(j))
              ! write(*,*)"309",ham(i,l)
               enddo
            enddo
            ham(l,i)=ham(i,l)
         enddo
      enddo

      k = 1
      do while ( cp2dtyp(k) .ne. 0 )
!       here you can define 2D functions here (either coupling of diagonal, ignore the fact that its called cp2d...)
         i = stacp2d(1,k)
         j = stacp2d(2,k)
         m = dofcp2d(1,k)
         n = dofcp2d(2,k)
         select case (cp2dtyp(k))
         case(1)! subtracts lambda contribuitions gradually as we move through both dof's (added above)
            ham(i,j) = ham(i,j) - (lambda(m,i,j)*q(m) + &
                 lambda(n,i,j)*q(n))*(tanh(abs(cp2d(1,k))*abs(q(n)*q(m))/2-3)+1)
            ham(j,i) = ham(i,j)
         case(2)! subtracts quadriatic mu contribuitions gradually as we move through both dof's (added above)
            ham(i,j) = ham(i,j) - (0.5*mu(m,m,i,j)*q(m)**2 + &
                 0.5*mu(n,n,i,j)*q(n)**2)*(tanh(abs(cp2d(1,k))*abs(q(n)*q(m))/2-3)+1)
            ham(j,i) = ham(i,j)
         case(3)! adds a 2D radially outward tanch function (from (0,0)) alon one of the cuardants
            ! a gaussian function determined the value of the gradient of the tanch (as a function of radial distance)
            ! the user must input 3 parameters + which cuadrant.
            HH = cp2d(1,k)
            GG = cp2d(2,k)
            SS = cp2d(3,k)
            C1 = 0
            C2 = 0
            if(cp2d(4,k) .eq. 1)then
               C1 = 1
               C2 = 1
            elseif( cp2d(4,k) .eq. 2)then
               C1 = -1
               C2 = 1
            elseif( cp2d(4,k) .eq. 2)then
               C1 = -1
               C2 = -1
            elseif( cp2d(4,k) .eq. 2)then
               C1 = 1
               C2 = -1 
            endif
            if(q(n) .ne. 0) then 
               phi = atan(q(m)/q(n))
            else
               phi = 0.0
            endif
            RR =  cos(phi)*q(n) + sin(phi)*q(m)
            chi = abs(sin(2.0*phi))*pi/4.0
            m0 = tan(chi) / ( tan(pi/4)  )
            sd = sqrt( - 1.0/( log( 0.25/1.0 ) )  )
            gr = exp(-(( (m0-1)/sd )**2))
            T =  HH*(0.5*(tanh(gr*GG*(C1*RR-SS) -4.0) +1))*(0.5*(tanh(1000*q(m)*C2 -4.0) +1))
            ham(i,j) = ham(i,j) + T
         case(4)!
            HH = cp2d(1,k)
            GG = cp2d(2,k)
            SS = cp2d(3,k)
            phi = -cp2d(4,k)
            XX =  cos(phi)*q(n) + sin(phi)*q(m)
            YY = -sin(phi)*q(n) + cos(phi)*q(m)
            chi = abs(sin(2.0*phi))*pi/4.0
            m0 = SS*tan(chi) / tan(pi/4.0)
            T =  (0.5*(tanh(GG*(XX-SS ) -4.0) +1.0) )
            !          T =  (0.5*(tanh(C1*GG*(XX-SS*C1 ) -4.0) +1) )
            ma = 600
            mi = 0.001
            sd = m0*sqrt( 1.0/(log((exp(ma)-1)/(exp(mi)-1)  )) )
            gr = HH*(0.001666*log(  (exp(ma)-1.0) * exp((-(YY/ sd )**2.0)) +1.0 ) )
            ham(i,j) = ham(i,j) + T*gr
         case(5)! polynomial along an chosen angle as a box function (tanh)
            HH = cp2d(1,k)
            GG = cp2d(2,k)
            SS = cp2d(3,k)
            phi = cp2d(4,k)
            PP = cp2d(5,k)
            ff = product( (/ ( l, l=1,int(PP)) /) )
            XX =  cos(phi)*q(n) + sin(phi)*q(m)
            YY = -sin(phi)*q(n) + cos(phi)*q(m)
            chi = abs(sin(2.0*phi))*pi/4.0
            m0 = SS*tan(chi) / tan(pi/4.0)
            gr = 0.5*(tanh(GG*(YY+m0))+1)*0.5*(tanh(-GG*(YY-m0))+1)
            T = HH*(0.5*(tanh(500*(XX-SS)) +1))*(1/ff)*(XX-SS)**PP
            ham(i,j) = ham(i,j) + gr*T
         case(6)
!       a polynomial along one cuadrant, note that the order is part of the key index of that function
            HH = cp2d(1,k)
            PP = ocp2d(k)
            ff = product( (/ ( l, l=1,int(PP)) /) )
            SS = cp2d(2,k)
            phi = -cp2d(3,k)
            XX =  cos(phi)*q(n) + sin(phi)*q(m)
            T = HH*(0.5*(tanh(500*(XX-SS))+1))*(1/ff)*(XX-SS)**PP
            ham(i,j) = ham(i,j) + T
         end select
         k = k + 1
      enddo

! Explicitly defined terms
      do n=1,nadd
         i = termst(1,n)
         j = termst(2,n)
         tmp = termk(n)
         do m=1,nmodes
            p = termdef(adof(m),n)
            tmp=tmp*(q(adof(m))**p)
         enddo
         ham(i,j) = ham(i,j) + tmp
         if (i .ne. j) ham(j,i) = ham(i,j)
      enddo

! Zeroth-order bilinear contributions due to the rotation of a single 
! pair of normal modes
      if (lrotmodes) then
         z = 1
         do while (rotm(z,1) .ne. 0)
            igno = .false.  
            if( lnmcurv ) then
               i = 1
               do while(nmcurvtype(i) .ne. 0)
                  if(nmcurvtype(i) .eq. 1)then
                     if( nmcurv(i,1) .eq. rotm(z,1) .or. nmcurv(i,2) .eq. rotm(z,1) .or.&
                          nmcurv(i,3) .eq. rotm(z,1) ) igno = .true.
                     if( nmcurv(i,1) .eq. rotm(z,2) .or. nmcurv(i,2) .eq. rotm(z,2) .or.&
                          nmcurv(i,3) .eq. rotm(z,2) ) igno = .true.
                  endif
                  i = i + 1
               enddo
            endif
            if (igno .eqv. .false.) then
               do i=1,nsta
                  do k=1,nmodes
                     do j=1,k-1
                        if (rotm(z,1).eq.adof(j).and.rotm(z,2).eq.adof(k)&
                             .or.rotm(z,2).eq.adof(j).and.rotm(z,1).eq.adof(k)) then
                           ham(i,i)=ham(i,i)+0.5*rotmxfq(z)*q(adof(k))*q(adof(j))
                        endif
                     enddo
                     do j=k+1,nmodes
                        if (rotm(z,1).eq.adof(j).and.rotm(z,2).eq.adof(k)&
                             .or.rotm(z,2).eq.adof(j).and.rotm(z,1).eq.adof(k)) then
                           ham(i,i)=ham(i,i)+0.5*rotmxfq(z)*q(adof(k))*q(adof(j))
                        endif
                     enddo
                  enddo
               enddo
            endif
            z = z + 1
         enddo
      endif

! Revert back to original parameter values if switching functions
! are being used
      if (lswitch) call switchfuncs(q,kappa,lambda,gamma,&
           iota,mu,expcoe,termk,cpcoe,switch,-1)

      call stop_timer(tid)
         
      if (i_ignore .gt. 0) nmodes = nmodes + i_ignore
!-----------------------------------------------------------------------
! diagonalise ham
!-----------------------------------------------------------------------
      call get_timer(tid1, 'Wdiag')
      call start_timer(tid1)

      do i=1,nsta
         do j=1,nsta
            diabv(i,j)=ham(i,j)
         enddo
      enddo
         
      if (nsta .eq. 1) then
         v(1)=ham(1,1)        
      else
         e2 = 3*nsta
         if (leigvec) then
            call dsyev('V','U',nsta,ham,maxsta,vadiab,work,e2,error)
            do j=1,nsta
               do i=1,nsta
                  diabvec(i,j)=ham(i,j)
               enddo
            enddo
         else
            call dsyev('N','U',nsta,ham,maxsta,vadiab,work,e2,error)
         endif
         
         if (error .ne. 0) then
            write(6,*) 'Cannot diagonalize PES matrix in Subroutine pes'
         endif
         
         do i=1,nsta
            v(i)=dble(vadiab(i))
         enddo
      endif
      if( lalleig .or. lnmcurv) freq0= tfreq0
      
      call stop_timer(tid1)


      return
    end subroutine pes

!#######################################################################

      subroutine fdiab(ham,q,ifdiab,expcoe,ncoeff,freq0,en)

      implicit none
        
      integer(long), intent(in)                :: ncoeff,ifdiab
      integer(long)                            :: l
      real(dop) :: freq,pi,alpha,D0,x0,delx,E0,x1,q0,qm,ham1,Einf
      real(dop), intent(in)                    :: q,freq0,en
      real(dop)                                :: rdum
      real(dop), dimension(ncoeff), intent(in) :: expcoe
      real(dop) :: D1,D2,alpha1,alpha2,R1,R2,prex,cconst,bound,diss,&
                   cconst1,cconst2,PP,HH,SS,ff,T
      real(dop) :: d,e,f,tmp,tmp2
      double precision                         :: ham
      
! dummy cp for unused variable
      rdum=en

      pi = 4.0*atan(1.0)
      
      select case (ifdiab)
           
      case default
         
         print*,'ERROR: Undefined diabatic potential in fdiab'
         stop
         
      case(1) ! HO + quartic term
         ham = ham + 0.5*freq0*q*q + expcoe(1)*q 
         ham = ham+0.5*expcoe(2)*q*q
         ham = ham+(1.0/24.0)*expcoe(3)*q**4
         return
         
      case(2) ! Morse. E=0 at x=0. Dissociation to positive q
         x0=expcoe(3)
         D0=expcoe(1)
         freq=freq0 + expcoe(2)
         alpha=freq/(2.0*D0)
         if (alpha .lt. 0.0) then
!        print*,'WARNING: Negative alpha in Morse potential'
            alpha=0.0
         else
            alpha=sqrt(alpha)
         endif

! stops product of alpha*x0 getting too large
         x1 = 1.0_dop/x0
         x1=abs(x1)
         alpha=min(alpha,x1)
         
         E0=-(D0*(exp(alpha*x0)-1)**2)
         delx=q-x0
         
         alpha=-alpha
         ham=ham + D0*(exp(alpha*delx)-1)**2 + E0
         return

      case (3) ! Buckingham potential from Gareth Welch. To be checked.
                 ! Current test potential
         alpha = expcoe(1)
         q0 = expcoe(2)
         qm = expcoe(3)
         d0 = expcoe(5)
         e0 = expcoe(4)
         delx = expcoe(6)+(expcoe(5)/(q-q0))**8
!     delx = 1
         
! when q0 = q then we die with -NaN, so test and tweak
         if(abs(q0-q) .lt. .1e-3) then
            q0 = q0+0.01
         endif

!       ham1 =  (expcoe(6)+(expcoe(5)/(q-q0))**8)*
!    +    exp((-alpha/(qm-q0))*(q-qm))-0*(((qm-q0)/(q-q0)))**6
         ham1=(d0/(1-6/alpha))*((6/alpha)*delx*exp((-alpha*(q-qm))/(qm-q0))&
              -(((qm-q0)/(q-q0)))**6) + e0

         if (abs(ham1).gt.1.0e2) then
            ham = ham + 100*(ham1/abs(ham1))
         else
            ham = ham + ham1
         end if
         
         return

      case (4) ! Anti-Morse. E=0 at x=0. Dissociation to negative q
         x0=expcoe(3)
         D0=expcoe(1)

         freq=freq0 + expcoe(2)
         alpha=freq/(2.0*D0)
         if (alpha .lt. 0.0) then
!        print*,'WARNING: Negative alpha in Anti-Morse potential'
            alpha=0.0
         else
            alpha=sqrt(alpha)
         endif

! stops product of alpha*x0 getting too large
         x1 = 1.0_dop/x0
         x1=abs(x1)
         alpha=min(alpha,x1)
         
         E0=-(D0*(exp(-alpha*x0)-1)**2)
         delx=q-x0

         alpha=-alpha
         ham=ham + D0*(exp(-alpha*delx)-1)**2 + E0

         return

      case(5) ! Exponential function E_inf exp (alpha*x). alpha < 0
         alpha=expcoe(2)
         Einf=expcoe(1)
         ham=ham + Einf*(dexp(-alpha*q) - 1.0_dop)
         
         return
      
      case(6) ! Avoided-crossing model potential (lower adiabatic state)
         D1=expcoe(1)
         alpha1=expcoe(2)
         R1=expcoe(3)
         prex=expcoe(4)
         alpha2=expcoe(5)
         R2=expcoe(6)
         D2=expcoe(7)
         cconst=expcoe(8)
         
         bound = D1*((1 - exp(-alpha1*(q-R1)))**2)
         diss = prex*(exp(-alpha2*(q-R2))) + D2
         
         ham = ham + 0.5*(bound + diss - sqrt((bound-diss)**2 + 4*cconst**2))

         return

      case(7) ! General morse potential
         D0=expcoe(1)
         alpha=expcoe(2)
         q0=expcoe(3)
         
         E0=-D0*(1-exp(-alpha*q0))**2

         ham = ham + D0*(1-exp(-alpha*(q-q0)))**2 + E0
         return

      case(8) ! General harmonic oscillator
         ham = ham + 0.5*expcoe(1)*q*q

         return

      case(9) ! General quartic potential
         ham = ham + 0.5*freq0*q*q + expcoe(1)*q + 0.5*expcoe(2)*q*q + &
              (1.0/24.0)*expcoe(3)*q*q*q*q

         return

      case(10) ! Lennard-Jones
         D0=expcoe(1)
         alpha=expcoe(2)
         delx=expcoe(3)

         ham = ham + D0*( (alpha/(q+delx))**12 -2*((alpha/(q+delx))**6) &
              -(alpha/delx)**12 +2*((alpha/delx)**6) )

         return

      case(11) ! Anti-Lennard-Jones
         D0=expcoe(1)
         alpha=expcoe(2)
         delx=-expcoe(3)

         ham = ham + D0*( (alpha/(q+delx))**12 -2*((alpha/(q+delx))**6) &
              -(alpha/delx)**12 +2*((alpha/delx)**6) )

         return

      case(12) ! General asymmetric quartic potential
         ham = ham + 0.5*freq0*q*q + expcoe(1)*q + 0.5*expcoe(2)*q*q &
              + (1.0/6.0)*expcoe(3)*q*q*q &
              + (1.0/24.0)*expcoe(4)*q*q*q*q
         
         return

      case(13) ! Morse + Lorentzian
         D0=expcoe(1)
         alpha=expcoe(2)
         x0=expcoe(3)
         E0=expcoe(4)
         d=expcoe(5)
         e=expcoe(6)
         f=expcoe(7)

         delx=q-x0

! Morse
         ham = ham + D0*(1-exp(-alpha*delx))**2 + E0
      
! plus Lorentzian
         tmp = d/(1+((q-e)**2/f))
         ham = ham + tmp

         return

      case(14) ! Morse + Gaussian
         D0=expcoe(1)
         alpha=expcoe(2)
         x0=expcoe(3)
         E0=expcoe(4)
         d=expcoe(5)
         e=expcoe(6)
         f=expcoe(7)

         delx=q-x0

! Morse
         ham = ham + D0*(1-exp(-alpha*delx))**2 + E0

! plus Gaussian
         tmp = d*exp(-e*((q-f)**2))
         ham = ham + tmp

         return

      case(15) ! General second-order polynomial
         ham = ham + expcoe(1)*q + 0.5*expcoe(2)*q*q
         
         return

      case(16) ! Avoided-crossing model potential (lower adiabatic state) with q-dependent coupling
               ! suitable for a dissociative coordinate
         D1=expcoe(1)
         alpha1=expcoe(2)
         R1=expcoe(3)
         prex=expcoe(4)
         alpha2=expcoe(5)
         R2=expcoe(6)
         D2=expcoe(7)
         cconst1=expcoe(8)
         cconst2=expcoe(9)
         
         bound = D1*((1 - exp(-alpha1*(q-R1)))**2)
         diss = prex*(exp(-alpha2*(q-R2))) + D2
         
         ham = ham + 0.5*(bound + diss - sqrt((bound-diss)**2 + 4*(cconst1*tanh(cconst2*q))**2))

         return

      case(24) ! Same potential as case(16), i.e. avoided crossing between lower Morse and upper
               ! exponential BUT reversed for dissociation in negative q. (two alphas chenged sign)
         D1=expcoe(1)
         alpha1=expcoe(2)
         R1=expcoe(3)
         prex=expcoe(4)
         alpha2=expcoe(5)
         R2=expcoe(6)
         D2=expcoe(7)
         cconst1=expcoe(8)
         cconst2=expcoe(9)

         bound = D1*((1 - exp(alpha1*(q-R1)))**2)
         diss = prex*(exp(alpha2*(q-R2))) + D2

         ham = ham + 0.5*(bound + diss - sqrt((bound-diss)**2 + 4*(cconst1*tanh(cconst2*q))**2))

         return

      case(17) ! Avoided-crossing model potential (upper adiabatic state) with q-dependent coupling
               ! suitable for a dissociative coordinate
         D1=expcoe(1)
         alpha1=expcoe(2)
         R1=expcoe(3)
         prex=expcoe(4)
         alpha2=expcoe(5)
         R2=expcoe(6)
         D2=expcoe(7)
         cconst1=expcoe(8)
         cconst2=expcoe(9)
      
         bound = D1*((1 - exp(-alpha1*(q-R1)))**2)
         diss = prex*(exp(-alpha2*(q-R2))) + D2
         
         ham = ham + 0.5*(bound + diss + sqrt((bound-diss)**2 + 4*(cconst1*tanh(cconst2*q))**2))
         
         return
         
      case(18) ! Quartic, quartic avoided crossing model with linear coupling
         tmp = 0.5*freq0*q*q + 0.5*expcoe(1)*q*q + (1.0/24.0)*expcoe(2)*q*q*q*q
         tmp2 = 0.5*freq0*q*q + 0.5*expcoe(3)*q*q + (1.0/24.0)*expcoe(4)*q*q*q*q + abs(expcoe(6)) 
!	ive removed the 'en'. i does not need it: 
!      tmp2 = 0.5*freq0*q*q + 0.5*expcoe(3)*q*q + (1.0/24.0)*expcoe(4)*q*q*q*q + en+abs(expcoe(6))    
!      ham = ham + 0.5*( tmp + tmp2 - sqrt((tmp-tmp2)**2 + 4*((expcoe(5)*q)**2)))

!     instead: (since we need to pull out the E1 (vertical ex energy) out of the equation, we get:
         ham = ham + 0.5*( tmp + tmp2 - sqrt((tmp2-tmp)**2 + 4*((expcoe(5)*q)**2) ) )

         return
         
      case(19) ! 4th-order Exe Jahn-Teller expansion
         ham = ham + expcoe(1)*q &
              + 0.5*(expcoe(2)+expcoe(3))*q*q &
              + (1.0/6.0)*(2*expcoe(4)+expcoe(5))*q*q*q &
              + (1.0/24.0)*(expcoe(6)+expcoe(7)+expcoe(8))*q*q*q*q
         
      case(20) ! Morse + quadratic potential
         x0=expcoe(3)
         D0=expcoe(1)
         
         freq=freq0 + expcoe(2)
         alpha=freq/(2.0*D0)
         if (alpha .lt. 0.0) then
            alpha=0.0
         else
            alpha=sqrt(alpha)
         endif
      
! stops product of alpha*x0 getting too large
         x1 = 1.0_dop/x0
         x1=abs(x1)
         alpha=min(alpha,x1)
         
         E0=-(D0*(exp(-alpha*x0)-1)**2)
         delx=q-x0

!      alpha=-alpha
      
         ham=ham + D0*(exp(-alpha*delx)-1)**2 + E0
         ham=ham + expcoe(4)*q +0.5*expcoe(5)*q*q

      case(21) ! one sided polynomial; HH = parameter it, PP = order of poly, SS = where it starts (like cap)
         HH = expcoe(1)
         PP = expcoe(2)
         ff = product( (/ ( l, l=1,int(PP)) /) )
         SS = expcoe(3)
         T = HH*(0.5*(tanh(500*(q-SS))+1))*(1/ff)*(q-SS)**PP
         ham = ham + T
      case(22) !a constant potential
         x0=expcoe(1)
         ham = x0
         return
     

      case(23) !1/R
         q0=expcoe(2)
! only defined for q>q0
         if (q .le. q0) then
            ham = 99999999.0_dop
            return
         else
            alpha=expcoe(1)
            ham = ham + alpha/(q-q0)
            return
         endif

      end select

      end subroutine fdiab
      
!#######################################################################
!
! ifdiab defines potential to be set up
! ifdiab1 defines potential read from guess file. If 0 usual HO.
!
!#######################################################################
     
      subroutine conv_fdiab(ifdiab,freq0,kappa,gamma,expcoe,ncoeff,&
                 general_mask,ifdiab1,ierr)

      implicit none

      integer(long), intent(in)                    :: ncoeff,ifdiab,ifdiab1
      integer(long), intent(out)                   :: ierr
      integer(long), dimension(ncoeff), intent(out):: general_mask
      real(dop)                                    :: freq
      real(dop), intent(in)                        :: freq0,kappa,gamma
      real(dop), dimension(ncoeff), intent(out)    :: expcoe

      ierr = 0
      select case (ifdiab)

      case default

      print*,'ERROR: Undefined diabatic potential in conv_fdiab'
      stop

      case (1) ! quartic
      if (ifdiab1 .eq. 0) then
         expcoe(1)=kappa
         expcoe(2)=gamma
         expcoe(3)=0.0_dop
      else
         ierr = 1
      endif
      return

      case (2) ! Morse
! if guess = HO, depth is randomly assigned - optimise in first pass
      if (ifdiab1 .eq. 0) then
         expcoe(1)=5.0
!        general_mask(1)=3
         expcoe(2) = gamma
         freq=freq0 + gamma
         expcoe(3)=-(kappa/freq)
      else
         ierr = 1
      endif
      return
  
      case (3) ! Buckingham
       if (ifdiab1 .eq. 0) then

          general_mask(1)=1
          general_mask(2)=1
          general_mask(3)=1
          general_mask(4)=3
          general_mask(5)=1
          general_mask(6)=1
          general_mask(7)=2
          freq=freq0 + gamma

          expcoe(1)=0.3
          expcoe(2)=-30.1
          expcoe(3)=4
          expcoe(4)=8.670
          expcoe(5)= 1.10
          expcoe(6)=7
          expcoe(7)=5.012

      else
         ierr = 1
      endif
      return

      case (4) ! Anti-Morse
      if (ifdiab1 .eq. 0) then
         expcoe(1)=5.0
!        general_mask(1)=3
         expcoe(2) = gamma
         freq=freq0 + gamma
         expcoe(3)=-(kappa/freq)
      else
         ierr = 1
      endif
      return

      case (5) ! Exponential
      if (ifdiab1 .eq. 0) then
         freq=freq0 + gamma
         if (abs(kappa) .gt. 0.0) then
! put exponential throungh min. of harmonic with same gradient at Q0
            expcoe(1)=(kappa**2)/(2.0*freq)
            expcoe(2)=-kappa/expcoe(1)
         else
            expcoe(1)=0.5
            expcoe(2)=1.0
         endif
      else
         ierr = 1
      endif
      return

      case(6) ! Avoided-crossing model potential
      if (ifdiab1 .eq. 0) then

         expcoe(1)=4
         expcoe(2)=1
         expcoe(3)=0
         expcoe(4)=2
         expcoe(5)=1
         expcoe(6)=2
         expcoe(7)=4
         expcoe(8)=1.5

      else
         ierr = 1
      endif
      return

      case(7) ! General morse potential
      if (ifdiab1 .eq. 0) then

         expcoe(1)=5.0
         expcoe(2)=1.0
         expcoe(3)=0.02

      else
         ierr = 1
      endif
      return

      case(8) ! General harmonic oscillator
      if (ifdiab1 .eq. 0) then
         expcoe(1)=0.001

      else
         ierr = 1
      endif
      return

      case(9) ! General quartic potential
      if (ifdiab1 .eq. 0) then
         expcoe(1)=kappa
         expcoe(2)=gamma
         expcoe(3)=0.0

      else
         ierr = 1
      endif
      return

      case(10) ! Lennard-Jones
      if (ifdiab1 .eq. 0) then
         expcoe(1)=10.00
         expcoe(2)=80.00
         expcoe(3)=80.00

      else
         ierr = 1
      endif
      return

      case(11) ! Lennard-Jones
      if (ifdiab1 .eq. 0) then
         expcoe(1)=10.00
         expcoe(2)=80.00
         expcoe(3)=80.00

      else
         ierr = 1
      endif
      return

      case(12) ! General asymmetric quartic potential
      if (ifdiab1 .eq. 0) then
         expcoe(1)=kappa
         expcoe(2)=gamma
         expcoe(3)=0.0
         expcoe(4)=0.0
      else if (ifdiab1 .eq. 1) then
         expcoe(1)=expcoe(1)
         expcoe(2)=expcoe(2)
         expcoe(4)=expcoe(3)
         expcoe(3)=0.0
      else
         ierr = 1
      endif
      return

      case(13) ! Morse + Lorentzian
! if guess = HO, depth is randomly assigned 
      if (ifdiab1 .eq. 0) then
         expcoe(1)=8.4
         expcoe(2)=0.6
         expcoe(3)=-1.8
         expcoe(4)=-3.5
         expcoe(5)=10.5
         expcoe(6)=-2.2
         expcoe(7)=4.6

      else
         ierr = 1
      endif
      return

      case(14) ! Morse + Gaussian
! if guess = HO, depth is randomly assigned 
      if (ifdiab1 .eq. 0) then
         expcoe(1)=0.009
         expcoe(2)=0.4
         expcoe(3)=4.7
         expcoe(4)=5.5
         expcoe(5)=0.5
         expcoe(6)=0.16
         expcoe(7)=2.9

      else
         ierr = 1
      endif
      return

      case(15)!General second-order polynomial
         if (ifdiab1 .eq. 0) then
            expcoe(1)=0.01
            expcoe(2)=0.001
      else
         ierr = 1
      endif
      return

   case(16) ! Avoided-crossing model potential with q-dependent coupling
            ! suitable for a dissociative coordinate
      if (ifdiab1 .eq. 0) then

         expcoe(1)=4
         expcoe(2)=1
         expcoe(3)=0
         expcoe(4)=2
         expcoe(5)=1
         expcoe(6)=2
         expcoe(7)=4
         expcoe(8)=1.5
         expcoe(9)=1.0

      else
         ierr = 1
      endif
      return

   case(24) ! Avoided-crossing (same as case(16)) but with reversed alphas
      if (ifdiab1 .eq. 0) then

         expcoe(1)=4
         expcoe(2)=1
         expcoe(3)=0
         expcoe(4)=2
         expcoe(5)=1
         expcoe(6)=2
         expcoe(7)=4
         expcoe(8)=1.5
         expcoe(9)=1.0

      else
         ierr = 1
      endif
      return

   case(17) ! Avoided-crossing model potential (upper adiabatic state) with q-dependent coupling
            ! suitable for a dissociative coordinate
      if (ifdiab1 .eq. 0) then
         
         expcoe(1)=4
         expcoe(2)=1
         expcoe(3)=0
         expcoe(4)=2
         expcoe(5)=1
         expcoe(6)=2
         expcoe(7)=4
         expcoe(8)=1.5
         expcoe(9)=1.0

      else
         ierr = 1
      endif
      return

   case(18) ! Quartic, quartic avoided crossing model with linear coupling
      if (ifdiab1 .eq. 0) then
         expcoe(1)=0.05
         expcoe(2)=0.0005
         expcoe(3)=0.05
         expcoe(4)=0.0005
         expcoe(5)=0.0005
      else
         ierr = 1
      endif
      return

   case(19) ! 4th-order Exe Jahn-Teller expansion
      if (ifdiab1 .eq. 0) then
         expcoe(1)=0.000005
         expcoe(2)=0.000005
         expcoe(3)=0.000005
         expcoe(4)=0.000005
         expcoe(5)=0.000005
         expcoe(6)=0.000005
         expcoe(7)=0.000005
         expcoe(8)=0.000005
      else
         ierr = 1
      endif
      return


   case(22) ! Constant diabatic potential
      if (ifdiab1 .eq. 0) then
         expcoe(1)=4.00
      else
         ierr = 1
      endif
      return

   case(23) ! 1/R
      if (ifdiab1 .eq. 0) then
         expcoe(1)=1.00
         expcoe(2)=1.00
      else
         ierr = 1
      endif
      return

      end select

      end subroutine conv_fdiab

!#######################################################################

      subroutine evalcpfunc(ham,q,ifdiabcp,ncoeffcp,cpcoe)

        implicit none

        integer(long), intent(in)                  :: ncoeffcp,ifdiabcp
        real(dop), intent(in)                      :: q
        real(dop), dimension(ncoeffcp), intent(in) :: cpcoe
        double precision                           :: ham

        select case (ifdiabcp)

        case(1) ! tanh
           ham = ham + cpcoe(1)*tanh(cpcoe(2)*q)
           return

        case(2) ! polynomial-to-tanh
           ham = ham + abs(cpcoe(1))*tanh(abs(cpcoe(2))*q) &
                   + (cpcoe(3)*q + cpcoe(4)*q*q)&
                   *exp(-abs(cpcoe(5))*q)
           return

        case(3) ! tanh2
           ham = ham + (cpcoe(1)/2.0)*(1-tanh(-(q-cpcoe(2))/cpcoe(3)))
           return

        end select

      end subroutine evalcpfunc

!#######################################################################
!
! extracts a single state energy from the array v
! NB currsta must be set before calling function
!
!#######################################################################

      real(dop) function vpes(q,mr,mi)

      implicit none

      real(dop), dimension(*), intent(in)            :: q
      real(dop), dimension(mrdim), intent(inout)     :: mr
      integer(long), dimension(midim), intent(inout) :: mi
      real(dop), dimension(maxsta)            :: v
      logical(kind=4)                         :: leigvec
      real(dop), dimension(maxsta,maxsta)     :: diabvec
      real(dop), dimension(maxsta,maxsta)     :: diabv
      leigvec=.false.

      call pes(q,v,mr(mrconst),mr(mrkappa),mr(mrlambda),mr(mrgamma),&
           mr(mriota),mr(mrmu),mr(mrexpcoe),mr(mrhigher),mr(mrtermk),&
           mr(mrcpcoe),mr(mrswitch),mr(mrcp2d),leigvec,diabvec,diabv)

      vpes=v(currsta)-e_shift

      return
      end function vpes

!#######################################################################

      subroutine dpes(q,dq,mr,mi)

      implicit none

      integer(long)                                       :: n
      integer(long), dimension(*), intent(in)             :: mi
      real(dop)                                           :: err,h
      real(dop), dimension(*), intent(in)                 :: q
      real(dop), dimension(*), intent(out)                :: dq
      real(dop), dimension(4*maxdim*maxdim*2)             :: work
      real(dop), dimension(mrdim), intent(in)             :: mr

      do n=1,nvar
         h=0.001
! ______ keep currsta the way it is - now usable for higher states (see vpes)
         dq(n)=dfridr(vpes,q,nvar,n,h,err,work,mr,mi)
      enddo
         
      return
      end subroutine dpes

!#######################################################################

      subroutine q2xmod(xcoo,xcoo0,q,nmcoo,nmodes,ncoo,maxdim)


      implicit none

      integer(long)                                   :: i,j
      integer(long), intent(in)                       :: nmodes,ncoo,maxdim
      real(dop), dimension(ncoo), intent(in)          :: xcoo0
      real(dop), dimension(nmodes), intent(in)        :: q
      real(dop), dimension(maxdim,maxdim), intent(in) :: nmcoo
      real(dop), dimension(ncoo), intent(out)         :: xcoo


      do i=1,ncoo
         xcoo(i)=xcoo0(i)
         do j=1,nmodes
            xcoo(i)=xcoo(i)+nmcoo(i,j)*q(j)
         enddo
      enddo

      return
      end subroutine q2xmod

!#######################################################################
!
! numerical recipes routine, changed for vector input

      real(dop) function dfridr(func,x,nvar,n,h,err,work,rdata,idata)

       integer(long), parameter :: ntab=10
       real(dop), parameter     :: con=1.4_dop,con2=con*con,big=1.0e30_dop, safe=2.0_dop

       integer(long)                             :: i,j
       integer(long), intent(in)                 :: n,nvar
       integer(long), dimension(*), intent(in)   :: idata
       real(dop), dimension(nvar,2), intent(out) :: work
       real(dop), dimension(nvar), intent(in)    :: x
       real(dop), dimension(ntab,ntab)           :: a
       real(dop)                                 :: errt,fac,hh
       real(dop), intent(in)                     :: h
       real(dop), intent(out)                    :: err
       real(dop), external                       :: func
       real(dop), dimension(*), intent(in)       :: rdata

       dfridr = 0.0_dop

      if (h.eq.0) stop 'h must be nonzero in dfridr'
      do i=1,nvar
         work(i,1)=x(i)
         work(i,2)=x(i)
      enddo

      hh=h
      work(n,1)=x(n)+hh
      work(n,2)=x(n)-hh
      a(1,1)=(func(work(1,1),rdata,idata)-func(work(1,2),rdata,idata))
      a(1,1)=a(1,1)/(2.0*hh)
      err=big
      do i=2,ntab
         hh=hh/con
         work(n,1)=x(n)+hh
         work(n,2)=x(n)-hh
         a(1,i)=(func(work(1,1),rdata,idata)-&
                          func(work(1,2),rdata,idata))
         a(1,i)=a(1,i)/(2.0*hh)
         fac=con2
         do j=2,i
            a(j,i)=(a(j-1,i)*fac-a(j-1,i-1))/(fac-1.0)
            fac=con2*fac
            errt=max(abs(a(j,i)-a(j-1,i)),abs(a(j,i)-a(j-1,i-1)))
            if (errt .le. err) then
               err=errt
               dfridr=a(j,i)
            endif
         enddo
         if(abs(a(i,i)-a(i-1,i-1)) .ge. safe*err) return
      enddo

      return
      end function dfridr

!#######################################################################

      real(dop) function vcweight(q,alpha)

      implicit none

       integer(long)                           :: i
       real(dop), dimension(maxdim), intent(in):: q
       real(dop)                               :: r
       real(dop), intent(in)                   :: alpha

      r=0.0
      do i=1,nmodes
         r=r+q(i)*q(i)
      enddo
      r=r*alpha
      vcweight=1.0/(r+1.0)
      
      return
      end function vcweight

!#######################################################################

      real(dop) function pes1tor(q)

      implicit none

       integer(long)                           :: i,k
       real(dop), dimension(*), intent(in)     :: q
       real(dop)                               :: v,series,sinq

      v=0.0
      do i=1,nmodes
         if (cootyp(i).eq.1) then
            series=0.0
            do k=1,4
               sinq=sin(k*q(i))
               sinq=sinq**2
!              series=series + expcoe(k,1)*sinq
            enddo
            v=v + series
         else
            v=v+0.5*freq0(i)*q(i)*q(i)
         endif                                                                  
      enddo

      pes1tor=v

      return
      end function pes1tor


!#######################################################################

      real(dop) function pesho(q)
 
      implicit none
 
      integer(long) :: i,j
      real(dop),intent(in):: q(*)
      real(dop) :: qdum
      real(dop):: v

      qdum = q(1)

!       v=const(1,1)
       do i=1,nmodes
!          v=v+kappa(i,1)*q(i)+0.5*freq0(i)*q(i)*q(i)
          j=i
         do j=1,nmodes
!            v=v+0.5*gamma(i,j,1)*q(i)*q(j)
         enddo                                                                  
       enddo

       pesho=v

       return
       end function pesho

!#######################################################################
!
! MOSC
!
! function required to optimise Morse oscillator parameters to
! fit HO PES
!
! rdum,idum are dummy arrays that could be used to carry data to the
! function through the optimiser routines
!
!####################################################################### 
!
       real(dop) function mosc(expcoe,rdum,idum)
!
       implicit none
!
       real(dop)                               :: w0,target
!
       integer(long), dimension(1), intent(in) :: idum
       integer(long)                           :: idum1
       real(dop), dimension(*), intent(in)     :: expcoe
       real(dop)                               :: d0,x0,freq,alpha,kappa
       real(dop), dimension(1), intent(in)     :: rdum
       real(dop)                               :: rdum1

       common /ftarget/ target,w0

! dummy cp for unused variables
       idum1=idum(1)
       rdum1=rdum(1)

!-----------------------------------------------------------------------
! calculate kappa from present function
!-----------------------------------------------------------------------
       d0 = expcoe(1)
       freq = w0 + expcoe(2)
       x0 = expcoe(3)
!
       if (freq .gt. 0) then
          alpha=freq/(2.0*d0)
          alpha=sqrt(alpha)
       else
          alpha=0.0
       endif
!
       kappa=exp(alpha*x0) -exp(2.0*alpha*x0)
       kappa=2.0*alpha*d0*kappa
!
!-----------------------------------------------------------------------
! calculate error from target
!-----------------------------------------------------------------------
       mosc = abs(kappa - target)
 
      return
      end function mosc

!#######################################################################

      subroutine dmosc(q,dq,rdum,idum)

      implicit none

      integer(long)                              :: n,nvar
      integer(long), dimension(1), intent(in)    :: idum
      real(dop)                                  :: err,h
!      real(dop), external                        :: dfridr
      real(dop), dimension(*), intent(in)        :: q
      real(dop), dimension(*), intent(out)       :: dq
      real(dop), dimension(1), intent(out)       :: rdum
      real(dop), dimension(4*3*3*2)              :: work
!      external                                   :: mosc

      nvar=3
      do n=1,nvar
         h=0.001
         dq(n)=dfridr(mosc,q,nvar,n,h,err,work,rdum,idum)
      enddo
         
      return
      end subroutine dmosc

!#######################################################################
!
! diabpes: returns the value of the diabatic potential matrix element
!          W_ij, i=currsta, j=currsta1, at point q
!
!          S. Neville 03/10/11 
!
!#######################################################################

      real(dop) function diabpes(q,mr)

      implicit none

       real(dop), dimension(*), intent(in)        :: q
       real(dop), dimension(mrdim), intent(inout) :: mr
       real(dop), dimension(maxsta)            :: v
       logical(kind=4)                         :: leigvec
       real(dop), dimension(maxsta,maxsta)     :: diabvec
       real(dop), dimension(maxsta,maxsta)            :: diabv
       leigvec=.false.

      call pes(q,v,mr(mrconst),mr(mrkappa),mr(mrlambda),mr(mrgamma),&
           mr(mriota),mr(mrmu),mr(mrexpcoe),mr(mrhigher),mr(mrtermk),&
           mr(mrcpcoe),mr(mrswitch),mr(mrcp2d),leigvec,diabvec,diabv)

      diabpes=diabv(currsta,currsta1)

      return
      end function diabpes

!#######################################################################

      subroutine ddiabpes(q,dq,mr,idum)

      implicit none

      integer(long)                                       :: n
      integer(long), dimension(1), intent(in)             :: idum
      real(dop)                                           :: err,h
      real(dop), dimension(*), intent(in)                 :: q
      real(dop), dimension(*), intent(out)                :: dq
      real(dop), dimension(4*maxdim*maxdim*2)             :: work
      real(dop), dimension(mrdim), intent(in)             :: mr


      do n=1,nvar
         h=0.001
! ______ currsta and currsta1 need to be set to optimise diabatic potential
         dq(n)=dfridr(diabpes,q,nvar,n,h,err,work,mr,idum)
      enddo
         
      return
      end subroutine ddiabpes

!#######################################################################
! Calculates a model dipole matrix element at the point q for states s
! and s1 and component cmpnt (cmpnt=1,2,3 ---> x,y,z)
!#######################################################################

      real(dop) function ddipole(q,mr,s,s1,cmpnt)

      implicit none

      integer(long)                           :: s,s1,cmpnt
      real(dop), dimension(*), intent(in)     :: q
      real(dop), dimension(mrdim), intent(in) :: mr
      real(dop)                               :: mu

      call evalddipole(q,mu,s,s1,cmpnt,mr(mrdipzero),mr(mrdipder1),&
           mr(mrdipder2),mr(mrdipder3),mr(mrdipder4))

      ddipole=mu

      return
    end function ddipole

!#######################################################################
! Calculates a model polarisablity tensor element at the point q for
! states s and s1, and components cmpnt and cmpnt1 
! (cmpnt=1,2,3 ---> x,y,z)
!#######################################################################
      real(dop) function dpolar(q,mr,s,s1,cmpnt,cmpnt1)

      implicit none

      integer(long)                           :: s,s1,cmpnt,cmpnt1
      real(dop), dimension(*), intent(in)     :: q
      real(dop), dimension(mrdim), intent(in) :: mr
      real(dop)                               :: alpha

      call evaldpolar(q,alpha,s,s1,cmpnt,cmpnt1,mr(mrpolarzero),mr(mrpolarder1),&
           mr(mrpolarder2),mr(mrpolarder3),mr(mrpolarder4))

      dpolar=alpha

      return
    end function dpolar

!#######################################################################

      subroutine evalddipole(q,mu,s,s1,cmpnt,dipzero,dipder1,dipder2,&
           dipder3,dipder4)
      
        implicit none

        real(dop), dimension(*), intent(in)     :: q
        real(dop)                               :: mu
        integer(long)                           :: s,s1,cmpnt,m,n
        
        real(dop), dimension(nsta,nsta,3)               :: dipzero
        real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder1
        real(dop), dimension(nsta,nsta,3,nmodes,nmodes) :: dipder2
        real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder3
        real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder4

        mu = dipzero(s,s1,cmpnt)
        
        do m=1,nmodes
           mu = mu + dipder1(s,s1,cmpnt,m)*q(m)
        enddo

        do m=1,nmodes
           do n=1,nmodes
              mu = mu + 0.5*dipder2(s,s1,cmpnt,m,n)*q(m)*q(n)
           enddo
        enddo

        do m=1,nmodes
           mu = mu + (1.0/6.0)*dipder3(s,s1,cmpnt,m)*q(m)*q(m)*q(m)
        enddo

        do m=1,nmodes
           mu = mu + (1.0/24.0)*dipder4(s,s1,cmpnt,m)*q(m)*q(m)*q(m)*q(m)
        enddo
        
        return
      end subroutine evalddipole

!#######################################################################

      subroutine evaldpolar(q,alpha,s,s1,cmpnt,cmpnt1,polarzero,polarder1,polarder2,&
           polarder3,polarder4)
      
        implicit none

        real(dop), dimension(*), intent(in)     :: q
        real(dop)                               :: alpha
        integer(long)                           :: s,s1,cmpnt,cmpnt1,m,n
        
        real(dop), dimension(nsta,nsta,3,3)               :: polarzero
        real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder1
        real(dop), dimension(nsta,nsta,3,3,nmodes,nmodes) :: polarder2
        real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder3
        real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder4

        alpha = polarzero(s,s1,cmpnt,cmpnt1)
        
        do m=1,nmodes
           alpha = alpha + polarder1(s,s1,cmpnt,cmpnt1,m)*q(m)
        enddo

        do m=1,nmodes
           do n=1,nmodes
              alpha = alpha + 0.5*polarder2(s,s1,cmpnt,cmpnt1,m,n)*q(m)*q(n)
           enddo
        enddo

        do m=1,nmodes
           alpha = alpha + (1.0/6.0)*polarder3(s,s1,cmpnt,cmpnt1,m)*q(m)*q(m)*q(m)
        enddo

        do m=1,nmodes
           alpha = alpha + (1.0/24.0)*polarder4(s,s1,cmpnt,cmpnt1,m)*q(m)*q(m)*q(m)*q(m)
        enddo
        
        return
      end subroutine evaldpolar

!#######################################################################

      subroutine switchfuncs(q,kappa,lambda,gamma,iota,mu,&
           expcoe,termk,cpcoe,switch,sign)

        implicit none

        integer(long) :: i,sign
        real(dop), dimension(*), intent(in)                       :: q
        real(dop), dimension(nmodes,nsta), intent(inout)          :: kappa
        real(dop), dimension(nmodes,nsta,nsta), intent(inout)     :: lambda
        real(dop), dimension(maxcoe,nmodes,nsta), intent(inout)   :: expcoe
        real(dop), dimension(nmodes,nmodes,nsta), intent(inout)   :: gamma
        real(dop), dimension(nadd), intent(inout)                 :: termk
        real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: mu,iota
        real(dop), dimension(maxcoe,nmodes,nsta,nsta), intent(inout) :: cpcoe
        real(dop), dimension(nswitch,maxcoe), intent(in)          :: switch
        real(dop)                                                 :: diff

        do i=1,nswitch

           if (sign.lt.0) diff=swini(i)
           
           select case (swpar(i))
                 
           case(1) !kappa
              if (sign.gt.0) call evalswitch(q(swmd(i)),swfunc(i),diff,&
                   switch(i,1:nswitchcoe(i)),nswitchcoe(i),&
                   kappa(swm(i),sws(i)))
              
              kappa(swm(i),sws(i))=kappa(swm(i),sws(i)) + diff
                 
           case(2) !lambda
              if (sign.gt.0) call evalswitch(q(swmd(i)),swfunc(i),diff,&
                   switch(i,1:nswitchcoe(i)),nswitchcoe(i),&
                   lambda(swm(i),sws(i),sws1(i)))
              
              lambda(swm(i),sws(i),sws1(i))=lambda(swm(i),sws(i),sws1(i)) + diff
              lambda(swm(i),sws1(i),sws(i))=lambda(swm(i),sws(i),sws1(i))
              
           case(3) !gamma
              if (sign.gt.0) call evalswitch(q(swmd(i)),swfunc(i),diff,&
                   switch(i,1:nswitchcoe(i)),nswitchcoe(i),&
                   gamma(swm(i),swm1(i),sws(i)))
              
              gamma(swm(i),swm1(i),sws(i))=gamma(swm(i),swm1(i),sws(i)) + diff
              gamma(swm(i),swm1(i),sws(i))=gamma(swm1(i),swm(i),sws(i))
              
           case(4) !mu
              if (sign.gt.0) call evalswitch(q(swmd(i)),swfunc(i),diff,&
                   switch(i,1:nswitchcoe(i)),nswitchcoe(i),&
                   mu(swm(i),swm1(i),sws(i),sws1(i)))
              
              mu(swm(i),swm1(i),sws(i),sws1(i))=mu(swm(i),swm1(i),sws(i),sws1(i)) + diff
              mu(swm1(i),swm(i),sws(i),sws1(i))=mu(swm(i),swm1(i),sws(i),sws1(i))
              mu(swm(i),swm1(i),sws1(i),sws(i))=mu(swm(i),swm1(i),sws(i),sws1(i))
              mu(swm1(i),swm(i),sws1(i),sws(i))=mu(swm(i),swm1(i),sws(i),sws1(i))
              
           case(5) !iota
              if (sign.gt.0) call evalswitch(q(swmd(i)),swfunc(i),diff,&
                   switch(i,1:nswitchcoe(i)),nswitchcoe(i),&
                   iota(swm(i),swm1(i),sws(i),sws(i)))
              
              iota(swm(i),swm1(i),sws(i),sws(i))=iota(swm(i),swm1(i),sws(i),sws(i)) + diff
              iota(swm(i),swm1(i),sws(i),sws(i))=iota(swm1(i),swm(i),sws(i),sws(i))
                 
           case(6) !general              
              if (sign.gt.0) call evalswitch(q(swmd(i)),swfunc(i),diff,&
                   switch(i,1:nswitchcoe(i)),nswitchcoe(i),&
                   expcoe(swk(i),swm(i),sws(i)))
              
              expcoe(swk(i),swm(i),sws(i))=expcoe(swk(i),swm(i),sws(i)) + diff

           case(7) !termk
              if (sign.gt.0) call evalswitch(q(swmd(i)),swfunc(i),diff,&
                   switch(i,1:nswitchcoe(i)),nswitchcoe(i),termk(swk(i)))
              
              termk(swk(i))=termk(swk(i)) + diff
              
           case (8) !cpgeneral
              if (sign.gt.0) call evalswitch(q(swmd(i)),swfunc(i),diff,&
                   switch(i,1:nswitchcoe(i)),nswitchcoe(i),&
                   cpcoe(swk(i),swm(i),sws(i),sws1(i)))
              
              cpcoe(swk(i),swm(i),sws(i),sws1(i))=cpcoe(swk(i),swm(i),sws(i),sws1(i)) + diff
              
           end select
           
           if (sign.gt.0) swini(i)=-diff

        enddo

        return
      end subroutine switchfuncs
           
!#######################################################################

    subroutine evalswitch(q,func,diff,switch,ncoe,par)

      implicit none

      integer(long)            :: ncoe,func
      real(dop), intent(in)    :: q,par
      real(dop), intent(inout) :: diff
      real(dop), dimension(ncoe), intent(in) :: switch

      select case(func)
         
      case(1) !tanh
         diff = switch(1)*tanh(switch(2)*q)
         return
         
      case(2) !dtanh
         diff = -par*tanh(switch(1)*q)                 
         return
         
      case(3) !cdtanh
         diff = (-par/2.0)*(tanh((q-switch(1))/switch(2))+1)
         return
         
      case(4) !adtanh
         diff = -par*tanh(switch(1)*q)+&
              (switch(2)*q+switch(3)*q*q)*exp(-switch(4)*q)
         return
         
      end select
      
      return
    end subroutine evalswitch


!#######################################################################
!       l and l1 tells us the set of modes that the cmode (either cmode or cmode1) of which it has played
!       a transormation. t and t1 are the number id ( ranges from 1 to number of modes in set) that tells us what new curilinear 
!       DOF it is on the given set (each nmcurntype will have different number, 
!       eg spherical polar = 3; R (1) theta (2) & phi (3), plane polar = 2; R (1) & phi(2),etc...)
!       nmcurv(l,t) tells us what mode belongs to what set and t will define its new type of coord. 
!#######################################################################

       logical function iscurvnm(cmode,cmode1,l,l1,t,t1)

       integer(long), intent(in)                               :: cmode,cmode1
       integer(long), intent(out)                              :: l,t,t1,l1
       integer(long)    :: i,j
        l = 999
        l1 = 999
        i = 1
        j = 1
        iscurvnm = .false.
        do while (nmcurv(i,j) .ne. 0)
!         print*, 'checking..I ', 'i =', i, 'nmcurv(i,j) = ', nmcurv(i,j)
         do while (nmcurv(i,j) .ne. 0)
!          print*, 'checking.. nmcurv(i,j) = ', nmcurv(i,j), ' j =', j, 'cmode =', cmode
          if (nmcurv(i,j) .eq. cmode ) then
               iscurvnm = .true.
               l = i
               t = j
          elseif( nmcurv(i,j) .eq. cmode1 ) then
               iscurvnm = .true.
               l1 = i
               t1 = j
          endif
          j = j + 1
         enddo
         j = 1
         i = i + 1

        enddo
       end function

!#######################################################################

  end module functions
