!-----------------------------------------------------------------------
      module vchfitmod

use decimal, only: long,dop
use vchammod, only: maxcoe
use global, only: c1

      implicit none

      integer(long), parameter :: maxcons=1000

!-----------------------------------------------------------------------
! option flags
!-----------------------------------------------------------------------
      logical(kind=4), save :: lconfidence,lfit,ldensity,lcalc_low_ci, &
                               lusecons,lrdcons,lsetpar,laddpar,ldiab,lautosym

      integer(long), save   :: order
      
!-----------------------------------------------------------------------
! options for fittype
!
!-----------------------------------------------------------------------
      integer(long), save    :: weighttype
      real(dop), save        :: wfact
      logical(kind=4), save  :: lconst,ldirot,lkappa,lgamma,llambda,lmu,&
                                lgamdiag,lmudiag
      logical(kind=4), save  :: lparzero,lzerozero,lgeneral,ldifit,&
                                lconstall,ldiagfit,liota,liota2,lhigher,&
                                liotadiag,lgeneralcp

!-----------------------------------------------------------------------
! parameters for Hamiltonian
!
! hamord  = 1 linear Hamiltonian (const + kappa + lambda)
!         = 2 adiabatic 2nd order (const + kappa + lambda + gamma)
!         = 3 diabatic 2nd order (const + kappa + lambda + gamma + mu)
! dirot(s1,s) = initial rotation of diabatic states
!-----------------------------------------------------------------------
      integer(long), save :: nvar,ntvar,nconst,ntconst,nkappa,ntkappa,nlambda,&
                             ntlambda,ngamma,ntgamma,niota,ntiota,nmu,ntmu,&
                             ngen,ntgen,niota2,ntiota2,hamord,ntpar,&
                             ntermk,nttermk,ncpgen,ntcpgen,nswitchpar,&
                             ntswitchpar,ntcp2d,ncp2d
      real(dop), save                                :: energy0
      real(dop), dimension(:), allocatable, save     :: masse
      real(dop), dimension(:,:), allocatable, save   :: dirot
      
!----------------------------------------------------------------------
! i/o options
!----------------------------------------------------------------------
      integer(long), save     :: ifitmin,ioptim
      logical(kind=4), save   :: lrdmask=.false.
      logical(kind=4), save   :: lrdguess=.false.
      logical(kind=4), save   :: lmin=.false.
      logical(kind=4), save   :: lminrd0=.false.
      logical(kind=4), save   :: lmininx=.false.
      logical(kind=4), save   :: lwrop=.false.
      logical(kind=4), save   :: lwrpar=.false.
      logical(kind=4), save   :: lwrqinp=.false.
      character(len=80), save :: filepar,filedens,fileoper,fileqinp

!----------------------------------------------------------------------
! parameters for adjusting the fit
!----------------------------------------------------------------------
      real(dop), save :: e_shift
      real(dop), save :: q_shift = 0.0_dop
  
!----------------------------------------------------------------------
! njt: no. of JT sets
! jtsta(s): flags JT (degenerate) states
!----------------------------------------------------------------------	 
      integer(long), save                                      :: njt
      integer(long), dimension(:), allocatable, save           :: jtsta
      integer(long), dimension(:,:), allocatable, save         :: cointype,ncp2

      character(len=c1), save                                  :: symmode
      character(len=c1), dimension(:), allocatable, save       :: jtmode
      character(len=c1), dimension(:,:), allocatable, save     :: cpmode
      character(len=c1), dimension(:,:,:,:), allocatable, save :: cpmode2

!----------------------------------------------------------------------
! constraints
!----------------------------------------------------------------------
      integer(long), save                            :: ncons
      integer(long), dimension(:), allocatable, save :: parcons
      real(dop), dimension(:), allocatable, save     :: pconfact

!----------------------------------------------------------------------
! 4th-order Jahn-Teller constraints
!----------------------------------------------------------------------
      logical(kind=4), save :: ljt4cons
      integer(long), save   :: njt4cons
      integer(long), dimension(:,:,:,:), allocatable, save :: jt4cons

!-----------------------------------------------------------------------
! energies, derivatives etc.
! from adiabatic calculations in adiabinfo
! from diabatic surface in diabinfo
!
! ista(n): state no. in Hamiltonian
! jsta(n): state coupled to
!-----------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save   :: ista
      integer(long), dimension(:,:), allocatable, save :: jsta
      real(dop), dimension(:), allocatable, save       :: denergy

!------------------------------------------------------------------------
! some other data for the numerical stuff
!------------------------------------------------------------------------
      real(dop), dimension(:), allocatable, save :: weight,w0,w1,w2
      integer(long), save                        :: iter,subiter
      real(dop), save                            :: tol

      character(len=1), dimension(:), allocatable, save :: rottype

!      real*8 alpha
!			
!      common /rotation/ alpha

!----------------------------------------------------------------------
! Other info about the system
!----------------------------------------------------------------------
      character(len=16),save :: pntgrp

!-----------------------------------------------------------------------
! Fitting options
!-----------------------------------------------------------------------
      logical(kind=4), save  :: ldipfit,lpolarfit,lzerofit,lder1fit,&
                                lder2fit,lder3fit,lder4fit

!-----------------------------------------------------------------------
! In the initial guess, the second derivatives are corrected for the DCP 
! if lder_dcp = .true.
!-----------------------------------------------------------------------
      logical(kind=4), save  :: lder2_dcp=.true.

!-----------------------------------------------------------------------
! Output options
! cutoff_op : cutoff for not including parameters in the .op file
!-----------------------------------------------------------------------
      real(dop), save :: cutoff_op = 0.0_dop

!-----------------------------------------------------------------------
! Fitting information
!-----------------------------------------------------------------------
      integer(long), save :: nlmpar

!----------------------------------------------------------------------
!   Genetic Algorithm
!
! axislab: Definition of the system
!----------------------------------------------------------------------

      integer(long), save  :: nseed,npop,maxgen,muttyp,mutchange,&
                              maxcoup0,npop0,ngenga,maxcoup,ncross,&
                              seed0,maxgensaved,maxmem,gencount,branch_cutoff
      logical(kind=4),save :: lfulllog,lgenetica,lseed,lend,&
                              offdiagsignopt,lspecies,gaoptim,gaoptim2,&
                              covar_adapt,lstats
      real(dop), save      :: seed,mutchance,nmut,perparam,muti,cov_min_sd,&
                              scale_stdev,increase_mut

      character(len=16), dimension(3), save :: axislab

      real(dop), dimension(10), save        :: gacput,gasystemt
      real(dop), dimension(30,2), save      :: gamut
      real(dop), dimension(300), save       :: param_mean, eval_covar, &
                                               min_stdev, old_eval_covar, &
                                               e_st_dev, kurtosis,st_dev, &
                                               skew,DPtest
      character(len=15),dimension(300),save :: pname
      real(dop), dimension(300,300), save   :: param_covar, p_covar, &
                                               p_evec, old_p_evec, &
                                               old_param_covar, old_param_covar2
      real(dop), dimension(800), save       :: lwork
      real(dop), dimension(5000,300), save  :: paramvec, old_paramvec
      real(dop), dimension(2500,300), save  :: emutval,mutval
      real(dop), dimension(10000,4)         :: co_weights
      real(dop), dimension(5000)            :: mutchance_w, mutcake
      
      real(dop), dimension(2), save         :: kappamut,lambdamut,gammamut,&
                                               mumut,iotamut,termkmut
      real(dop), dimension(:,:,:,:), allocatable, save :: expcoemut

      real(dop), dimension(:), allocatable, save :: moddip,modpolar

      contains

!#######################################################################
!
! VCHFITMOD_ALLOC
!
! memory required for arrays in vchfitmodmod allocated here.
!
! First check if allocated, then allocate. All arrays must be
! deallocated in
! vchfitmod_dealloc
!
!#######################################################################

      subroutine alloc_vchfitmod

use maxv
use channels

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_vchfitmod

      cerr = 'masse'
      allocate(masse(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dirot'
      allocate(dirot(maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'jtsta'
      allocate(jtsta(maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'cointtype'
      allocate(cointype(maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ncp2'
      allocate(ncp2(maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'jtmode'
      allocate(jtmode(maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'cpmode'
      allocate(cpmode(maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'cpmode2'
      allocate(cpmode2(maxsta,maxsta,2,10),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'parcons'
      allocate(parcons(maxcons),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'pconfact'
      allocate(pconfact(maxcons),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'jt4cons'
      allocate(jt4cons(maxdim,maxdim,maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'rottype'
      allocate(rottype(24),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'expcoemut'
      allocate(expcoemut(2,maxcoe,maxdim,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ista'
      allocate(ista(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'jsta'
      allocate(jsta(maxsta,maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'denergy'
      allocate(denergy(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'weight'
      allocate(weight(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'w0'
      allocate(w0(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'w1'
      allocate(w1(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'w2'
      allocate(w2(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'moddip '
      allocate(moddip(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'modpolar '
      allocate(modpolar(maxset),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_VCHFITMOD :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_vchfitmod

!#######################################################################

      subroutine dealloc_vchfitmod 

      implicit none

      if (allocated(masse)) deallocate(masse)
      if (allocated(dirot)) deallocate(dirot)
      if (allocated(jtsta)) deallocate(jtsta)
      if (allocated(cointype)) deallocate(cointype)
      if (allocated(ncp2)) deallocate(ncp2)
      if (allocated(jtmode)) deallocate(jtmode)
      if (allocated(cpmode)) deallocate(cpmode)
      if (allocated(cpmode2)) deallocate(cpmode2)
      if (allocated(parcons)) deallocate(parcons)
      if (allocated(pconfact)) deallocate(pconfact)
      if (allocated(jt4cons)) deallocate(jt4cons)
      if (allocated(expcoemut)) deallocate(expcoemut)
      if (allocated(moddip)) deallocate(moddip)
      if (allocated(modpolar)) deallocate(modpolar)
      if (allocated(ista)) deallocate(ista)
      if (allocated(jsta)) deallocate(jsta)
      if (allocated(denergy)) deallocate(denergy)
      if (allocated(weight)) deallocate(weight)
      if (allocated(w0)) deallocate(w0)
      if (allocated(w1)) deallocate(w1)
      if (allocated(w2)) deallocate(w2)

      return
      end subroutine dealloc_vchfitmod 

      end module vchfitmod
