!-----------------------------------------------------------------------
      module vcplotmod

use decimal, only: long, dop

implicit none

!-----------------------------------------------------------------------
! file names
!-----------------------------------------------------------------------
      character(len=80), save :: gnufile,psfile,filestem,vecfile,geomfile

!-----------------------------------------------------------------------
! Code used to plot pdf
!-----------------------------------------------------------------------
      character(len=80), save :: pdfplt = 'mupdf'

!-----------------------------------------------------------------------
! plotting parameters
!-----------------------------------------------------------------------
      integer(long), parameter :: npnts = 50

      integer(long), save   :: orbital,plotfit,plotsta1st,plotstalast, &
                               neps,n,n2,dcpsta,dcpsta1
      real(dop), save       :: xscl,orbofs,step,epstmp,xmin,ymin,xmax, &
                               ymax,qmax,step2,zmin,zmax,scale
      logical(kind=4), save :: ismode,lxrange,lyrange,lpltvec,ldifmodes,&
                               lsubexp,negdiag,lzrange,lformal

      real(dop), dimension(:), allocatable, save       :: q,qvp,qvpini
      logical(kind=4), dimension(:), allocatable, save :: atvmd

!---------------------------------------------------------------------------
! ltdip=.true. plot transition dipole (from ground state) instead of energy
! lkey =.true. plot key
! lcolour=.true. use colours for points
! ldipole=.true. plot any given dipole or transition dipole moment
! lpolar=.true. plot any given polarisability or transition polarisability 
! lpltcurv2n=.true. plot a normal cut, given curval mode model or points
! lunrot=.true. plots a cut of the  unrotated modes
!---------------------------------------------------------------------------
      logical(kind=4), save :: ltdip,ltdipx,ltdipy,ltdipz,lkey,lcolour,&
                               lpldcp,labsdcp,ldiabcp,lnokey,lpltcurv2n,&
                               lunrot,lplsurf,cstmname=.false.,title=.false.

      contains

!#######################################################################
!
! VCPLOTMOD_ALLOC
!
! memory required for arrays in vcplotmodmod allocated here.
!
! First check if allocated, then allocate. All arrays must be
! deallocated in
! vcplotmod_dealloc
!
!#######################################################################

      subroutine alloc_vcplotmod

use maxv
use channels

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_vcplotmod

      cerr = 'q'
      allocate(q(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'qvp'
      allocate(qvp(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'qvpini'
      allocate(qvpini(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'atvmd'
      allocate(atvmd(maxdim),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_VCPLOTMOD :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_vcplotmod

!#######################################################################

      subroutine dealloc_vcplotmod

      if (allocated(q)) deallocate(q)
      if (allocated(qvp)) deallocate(qvp)
      if (allocated(qvpini)) deallocate(qvpini)
      if (allocated(atvmd)) deallocate(atvmd)

      return
      end subroutine dealloc_vcplotmod

      end module vcplotmod

