!-----------------------------------------------------------------------
      module vcpntmod

use decimal, only: long,dop

      implicit none

!-----------------------------------------------------------------------
! auxiliary files 
! fileop (unit iop): opening commands to be written to created input 
!                    files
! filecl (unit icl): closing commands to be written to created input 
!                    files
! filego (unit igo): file to submit all the jobs
!
! subcmd: string with command to submit a job (written to igo)
! lop,lcl,lgo: if true files are to be used
! unit numbers are defined in vcglobal
!-----------------------------------------------------------------------
      character(len=80), save :: fileop,filecl,filego
      character(len=20), save :: subcmd
      logical(kind=4), save   :: lop,lcl,lgo,lgoadd,lgocmd

!-----------------------------------------------------------------------
      integer(long), save :: iz,iz0

!-----------------------------------------------------------------------
! flags for output file formats
!-----------------------------------------------------------------------
      logical(kind=4), save :: logaussian,logamess,loxyz,lmolpro,ldalton,&
                               lturbomole,lgamessus,loqchem,lmolcas 

!-----------------------------------------------------------------------
! npnts: no. of points
! npnts1: auxilliary no. of points, e.g. when adding points at either
!         end of a vector cut.
!----------------------------------------------------------------------- 
      integer(long), save :: npnts = 0, npnts1 = 0

!-----------------------------------------------------------------------
! schemes for placing points
!-----------------------------------------------------------------------
      logical(kind=4), save :: lcut,lbloc2,lbloc3,lrandom,lrot,lvec,&
                               ldiag,lconcret,found,lrdfreq0,ldihed,&
                               lreplace,lreplacediag,lcutall

!-----------------------------------------------------------------------
! for defining cuts along normal modes
! "ndat" points along mode "cmode" with increment "xincr". 
!        Actually -ndat to -1 and 1 to ndat points set up
! lq0 : if true point Q0 is included (so -ndat, 0 , ndat points)
!-----------------------------------------------------------------------
      integer(long), save   :: ndat,ndat1,ndat2,cmode,cmode1,cmode2,set,fmode
      real(dop), save       :: xincr,xincr1,xincr2,x0,x01,x02,fx0
      logical(kind=4), save :: lq0

!-----------------------------------------------------------------------
! for the calculation of points along an arbitrary vector
! vscale: vector scaling factor
!-----------------------------------------------------------------------
      real(dop), dimension(:), allocatable, save :: qini
      real(dop), save                            :: vscale

!-----------------------------------------------------------------------
! for keeping a mode fixed at a particular value
!-----------------------------------------------------------------------
      integer(long), save  :: nval=0 , ival(10) = 0
      real(dop), save      :: qval0(10) = 0.0

!-----------------------------------------------------------------------
! for defining concrete geometry
!-----------------------------------------------------------------------
      real(dop), dimension(:), allocatable, save :: q

!------------------------------------------------------------------------
! the informations about the geometry and transformation
!------------------------------------------------------------------------
      real(dop), dimension(:), allocatable, save         :: freq1
      character(len=16), dimension(:), allocatable, save :: nmlab1

!------------------------------------------------------------------------
! subspace options
!------------------------------------------------------------------------
       logical(kind=4), save                      :: lsubpnt
       real(dop), dimension(:), allocatable, save :: parxcoo0
       real(dop), dimension(3), save              :: xsys,xsys0
       character(len=8), save                     :: sysatsym

!------------------------------------------------------------------------
! atom-to-group substitution
!------------------------------------------------------------------------
       integer(long), save                   :: isubatm,nsubatm
       logical(kind=4), save                 :: lsubatm
       character(len=2), dimension(10), save :: sublbl
       real(dop), dimension(10,3), save      :: subxcoo

!------------------------------------------------------------------------
! dummy atoms
!------------------------------------------------------------------------
       integer(long), save                               :: ndum
       real(dop), dimension(:), allocatable, save        :: dumxcoo
       character(len=8), dimension(:), allocatable, save :: dumatsym
       logical(kind=4), save                             :: ldumcom

      contains

!#######################################################################
!
! VCPNTMOD_ALLOC
!
! memory required for arrays in vcpntmodmod allocated here.
!
! First check if allocated, then allocate. All arrays must be
! deallocated in
! vcpntmod_dealloc
!
!#######################################################################

      subroutine alloc_vcpntmod

use maxv
use channels

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_vcpntmod

      cerr = 'qini'
      allocate(qini(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'q'
      allocate(q(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'freq1'
      allocate(freq1(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'nmlab1'
      allocate(nmlab1(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'parxcoo0'
      allocate(parxcoo0(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dumxcoo'
      allocate(dumxcoo(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dumatsym'
      allocate(dumatsym(maxdim),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_VCPNTMOD :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_vcpntmod

!#######################################################################

      subroutine dealloc_vcpntmod

      implicit none

      if (allocated(qini)) deallocate(qini)
      if (allocated(q)) deallocate(q)
      if (allocated(freq1)) deallocate(freq1)
      if (allocated(nmlab1)) deallocate(nmlab1)
      if (allocated(parxcoo0)) deallocate(parxcoo0)
      if (allocated(dumxcoo)) deallocate(dumxcoo)
      if (allocated(dumatsym)) deallocate(dumatsym)

      return
      end subroutine dealloc_vcpntmod

      end module vcpntmod
