      module vcsrfmod

use decimal, only: long,dop
use maxv, only: maxdim,maxset,maxsta

      implicit none
      
      integer(long), parameter :: maxpnt=500,maxcut=4,maxdat=60**4,jhg=64

!------------------------------------------------------------------------
! Input files
!------------------------------------------------------------------------
      character(len=80), save :: vchamfile,dvrfile
      
!------------------------------------------------------------------------
! Grid information
!------------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save   :: gcut,dcut,npnt
      integer(long), dimension(:,:), allocatable, save :: dcut2
      integer(long), save                              :: scut,scut1
      real(dop), dimension(:,:), allocatable, save     :: gpnt
      logical(kind=4), save                            :: lhdmr

!------------------------------------------------------------------------
! Coordinate information
!------------------------------------------------------------------------
      integer(long), save                                :: itrans,npart
      integer(long), dimension(:,:), allocatable, save   :: particle
!     SIMON:! I RENAMED your adof to aidof, since it clashed with my (far more important!!! :P )adof variable
      character(len=20), dimension(:), allocatable, save :: aidof
      logical(kind=4), save                              :: ltrans

!------------------------------------------------------------------------
! Cluster information
!------------------------------------------------------------------------
      integer(long), save                            :: nuc
      integer(long), dimension(:), allocatable, save :: mruc,mruccoo
      integer(long), save                            :: mrcc

!------------------------------------------------------------------------
! Output options
!------------------------------------------------------------------------
      logical(kind=4), save :: ltest,lbin,lascii,lplot,ladiab
      integer(long), save   :: entype
      real(dop), save       :: conv

!------------------------------------------------------------------------
! Block diagonalisation information
!------------------------------------------------------------------------
      logical(kind=4)                                :: lblock,lreg
      integer(long), dimension(:), allocatable, save :: blockdim
      integer(long)                                  :: nblock,ireg,nreg
      real(dop)                                      :: lamtol,eps

!------------------------------------------------------------------------
! Adiabatic projection matrix information
!------------------------------------------------------------------------
      logical(kind=4) :: ladproj
      integer(long)   :: iadproj

      contains

!#######################################################################
!
! VCSRFMOD_ALLOC
!
! memory required for arrays in vcsrfmodmod allocated here.
!
! First check if allocated, then allocate. All arrays must be
! deallocated in
! vcsrfmod_dealloc
!
!#######################################################################

      subroutine alloc_vcsrfmod

use maxv
use channels

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_vcsrfmod

      cerr = 'gcut'
      allocate(gcut(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dcut'
      allocate(dcut(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'npnt'
      allocate(npnt(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dcut2'
      allocate(dcut2(maxdim,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'gpnt'
      allocate(gpnt(maxdim,maxpnt),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'particle'
      allocate(particle(maxdim,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'aidof'
      allocate(aidof(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'mruc'
      allocate(mruc(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'mruccoo'
      allocate(mruccoo(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'blockdim'
      allocate(blockdim(maxdim),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_VCSRFMOD :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_vcsrfmod

!#######################################################################

      subroutine dealloc_vcsrfmod

      implicit none

      if (allocated(gcut)) deallocate(gcut)
      if (allocated(dcut)) deallocate(dcut)
      if (allocated(npnt)) deallocate(npnt)
      if (allocated(dcut2)) deallocate(dcut2)
      if (allocated(gpnt)) deallocate(gpnt)
      if (allocated(particle)) deallocate(particle)
      if (allocated(aidof)) deallocate(aidof)
      if (allocated(mruc)) deallocate(mruc)
      if (allocated(mruccoo)) deallocate(mruccoo)
      if (allocated(blockdim)) deallocate(blockdim)

      return
      end subroutine dealloc_vcsrfmod

    end module vcsrfmod
