!-----------------------------------------------------------------------
      module vctransmod

use decimal, only: long,dop
use maxv
use global, only: c1, c4,c5
use vcglobal, only: ilog

      implicit none

!-----------------------------------------------------------------------
! flags for input file formats
!-----------------------------------------------------------------------
      logical(kind=4), save :: lvamp,lvamp1,lg94,lg941,lg98,lg99,lgaussian,&
                               lg03,lgamess,ladc,lxyz,lsilent

!-----------------------------------------------------------------------
! options for input and output
!
! order        : maximum order of data to be read from data file
! abintype0    : ab initio type of file0 
! abintype(n)  : ab initio type of data file 
! gndabtype(n) : ab initio type of ground-state energy file (used for 
!                OVGF if GS energy other than RHF required)
! lpft         : = .true. , parent-to-fragment transformation required
!                = .false. , otherwise
!
! ldb0         : if true, read first point only from a DB
!-----------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save :: abintype,gndabtype

      integer(long), save   :: abintype0,order
      logical(kind=4), save :: lrde0,lwrderq,lwrgd,lrdfreq0,lwrder2q,ltdip,&
                               eck_frame,leshift,found,lpft,lsubtrans,lfilter,&
                               lvelocity
      logical(kind=4), save :: ldb0 = .false.
      logical(kind=4), save :: db2inp = .false.

!-----------------------------------------------------------------------
! lwrtrafo : If true output normal mode transformation matrices
!-----------------------------------------------------------------------
      logical(kind=4), save :: lwrtrafo = .false.

      real(dop), dimension(3,3), save :: itvec

!-----------------------------------------------------------------------
! flags for output file formats
!-----------------------------------------------------------------------
      logical(kind=4), save :: logaussian,logamess,loxyz,lmolpro,ldalton,&
                               lturbomole,lgamessus,loqchem,lmolcas 


!-----------------------------------------------------------------------
! files
!
! datadir: directory containing data files (excluding file0). Length 
!          (240) is chosen to match relp in 
!          abspath(lib/utilities/einglib.f)
!-----------------------------------------------------------------------
      character(len=c5), save :: parentfile,fragmentfile
      character(len=c5), save :: datadir
      character(len=c5), dimension(:), allocatable, save :: datafile,gndfile

!-----------------------------------------------------------------------
! extra info to interpret data in QC output
!
! dataorient : which coordinate orientation to read 
! gndorient : which coordinate orientation to read in associated GS file
!-----------------------------------------------------------------------
      character(len=c2), dimension(:), allocatable, save :: dataorient,&
                                                            gndorient  

!-----------------------------------------------------------------------
! title - ntitle lines (max 5) written in output
!-----------------------------------------------------------------------
      integer(long), save                   :: ntitle
      character(len=c4), dimension(5), save :: title

!-----------------------------------------------------------------------
! energies, derivatives etc.
! from adiabatic calculations in adiabinfo
! from diabatic surface in diabinfo
!
! eigen(n): eigenvalue number in file to be read - may be different 
!           from asta(n)
! molstates(maxset) : States specified in molpro format
!-----------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save   :: eigen,allen,exsta,&
                                                          ignore,negphase, molre_states
      integer(long), dimension(:,:), allocatable, save :: remove
      real(dop), dimension(:), allocatable, save       :: molstates,eshift
      logical(kind=4), save                            :: lmolstates,lmolre

!------------------------------------------------------------------------
! the informations about the geometry and transformation
!------------------------------------------------------------------------
      integer(long), save                                :: sysatom
      real(dop), dimension(:), allocatable, save         :: freq1,sysfreq
      character(len=c1), dimension(:), allocatable, save :: nmlab1,syslab
      real(dop), save                                    :: e01
      logical(kind=4), save                              :: lsubq,lpureint

!------------------------------------------------------------------------
! 2x2 rotation of normal modes
!------------------------------------------------------------------------
      character(len=16), dimension(:,:), allocatable, save :: rotnmlab

!------------------------------------------------------------------------
! reflection of points read in about a plane of symmetry
!------------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save :: reflect
      logical(kind=4), save                          :: lreflect

      real(dop), dimension(:), allocatable, save   :: der1,gd,xcoo,qcoo,&
                                                      qder1,qgd
      real(dop), dimension(:,:), allocatable, save   :: dcp,qdcp
      real(dop), dimension(:,:), allocatable, save :: der2,matpft,qder2,&
                                                      adjpft,matqrt,adjqrt
      real(dop), dimension(:,:), allocatable, save :: fragnmcoo,fragcoonm

      character(len=16), dimension(:), allocatable, save ::  fragnmlab

!------------------------------------------------------------------------
! replace a normal mode with a dihedral
!------------------------------------------------------------------------
      logical(kind=4), save                          :: ldihed

!------------------------------------------------------------------------
! arrays for analysis of DB LVC 
!------------------------------------------------------------------------
      real(dop), dimension(:,:), allocatable, save     :: maxkappa,maxlambda
      integer(long), dimension(:), allocatable, save   :: maxskappa
      integer(long), dimension(:,:), allocatable, save :: maxslambda
      real(dop), dimension(:,:), allocatable, save     :: maxgamma
      integer(long), dimension(:,:), allocatable, save :: maxsgamma
      real(dop), dimension(:), allocatable, save       :: nenergy

      contains

!#######################################################################
!
! VCTRANSMOD_ALLOC
!
! memory required for arrays in vctransmodmod allocated here.
!
! First check if allocated, then allocate. All arrays must be
! deallocated in
! vctransmod_dealloc
!
!#######################################################################

      subroutine alloc_vctransmod

      implicit none

      integer(long)     :: ierr
      character(len=15) :: cerr

! check all arrays are not allocated
      call dealloc_vctransmod

      cerr = 'abintype'
      allocate(abintype(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'gndabtype'
      allocate(gndabtype(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'datafile'
      allocate(datafile(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dataorient'
      allocate(dataorient(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'gndfile'
      allocate(gndfile(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'gndorient'
      allocate(gndorient(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'eigen'
      allocate(eigen(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'allen'
      allocate(allen(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'exsta'
      allocate(exsta(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ignore'
      allocate(ignore(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'negphase'
      allocate(negphase(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'remove'
      allocate(remove(maxset,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'molstates'
      allocate(molstates(maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'eshift'
      allocate(eshift(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'freq1'
      allocate(freq1(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'sysfreq'
      allocate(sysfreq(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'nmlab1'
      allocate(nmlab1(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'syslab'
      allocate(syslab(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'rotnmlab'
      allocate(rotnmlab(maxdim/2,1:2),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'reflect'
      allocate(reflect(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'der1'
      allocate(der1(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dcp'
      allocate(dcp(maxdim,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'gd'
      allocate(gd(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'xcoo'
      allocate(xcoo(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'qcoo'
      allocate(qcoo(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'qder1'
      allocate(qder1(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'qdcp'
      allocate(qdcp(maxdim,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'qgd'
      allocate(qgd(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'der2'
      allocate(der2(maxdim,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'matpft'
      allocate(matpft(maxdim,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'qder2'
      allocate(qder2(maxdim,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'adjpft'
      allocate(adjpft(maxdim,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'adjqrt'
      allocate(adjqrt(maxdim,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'matqrt'
      allocate(matqrt(maxdim,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'fragnmcoo'
      allocate(fragnmcoo(maxdim,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'fragcoonm'
      allocate(fragcoonm(maxdim,maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'fragnmlab'
      allocate(fragnmlab(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'molre_states'
      allocate(molre_states(maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      if (ldb0) then
         cerr = 'maxkappa'
         allocate(maxkappa(maxdim,2),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'maxlambda'
         allocate(maxlambda(maxdim,2),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'maxgamma'
         allocate(maxgamma(maxdim,maxdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'maxskappa'
         allocate(maxskappa(maxdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'maxslambda'
         allocate(maxslambda(maxdim,2),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'maxsgamma'
         allocate(maxsgamma(maxdim,maxdim),stat=ierr)
         if (ierr > 0) go to 1000
         cerr = 'nenergy'
         allocate(nenergy(maxsta),stat=ierr)
         if (ierr > 0) go to 1000
      endif

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_VCTRANSMOD :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_vctransmod

!#######################################################################

      subroutine dealloc_vctransmod

      implicit none

      if (allocated(abintype)) deallocate(abintype)
      if (allocated(gndabtype)) deallocate(gndabtype)
      if (allocated(datafile)) deallocate(datafile)
      if (allocated(dataorient)) deallocate(dataorient)
      if (allocated(gndfile)) deallocate(gndfile)
      if (allocated(gndorient)) deallocate(gndorient)
      if (allocated(eigen)) deallocate(eigen)
      if (allocated(allen)) deallocate(allen)
      if (allocated(exsta)) deallocate(exsta)
      if (allocated(ignore)) deallocate(ignore)
      if (allocated(negphase)) deallocate(negphase)
      if (allocated(remove)) deallocate(remove)
      if (allocated(molstates)) deallocate(molstates)
      if (allocated(eshift)) deallocate(eshift)
      if (allocated(freq1)) deallocate(freq1)
      if (allocated(sysfreq)) deallocate(sysfreq)
      if (allocated(nmlab1)) deallocate(nmlab1)
      if (allocated(syslab)) deallocate(syslab)
      if (allocated(rotnmlab)) deallocate(rotnmlab)
      if (allocated(reflect)) deallocate(reflect)
      if (allocated(der1)) deallocate(der1)
      if (allocated(dcp)) deallocate(dcp)
      if (allocated(gd)) deallocate(gd)
      if (allocated(xcoo)) deallocate(xcoo)
      if (allocated(qcoo)) deallocate(qcoo)
      if (allocated(qder1)) deallocate(qder1)
      if (allocated(qdcp)) deallocate(qdcp)
      if (allocated(qgd)) deallocate(qgd)
      if (allocated(der2)) deallocate(der2)
      if (allocated(matpft)) deallocate(matpft)
      if (allocated(qder2)) deallocate(qder2)
      if (allocated(adjpft)) deallocate(adjpft)
      if (allocated(adjqrt)) deallocate(adjqrt)
      if (allocated(matqrt)) deallocate(matqrt)
      if (allocated(fragnmcoo)) deallocate(fragnmcoo)
      if (allocated(fragcoonm)) deallocate(fragcoonm)
      if (allocated(fragnmlab)) deallocate(fragnmlab)
      if (allocated(maxkappa)) deallocate(maxkappa)
      if (allocated(maxlambda)) deallocate(maxlambda)
      if (allocated(maxskappa)) deallocate(maxskappa)
      if (allocated(maxslambda)) deallocate(maxslambda)
      if (allocated(maxgamma)) deallocate(maxgamma)
      if (allocated(maxsgamma)) deallocate(maxsgamma)
      if (allocated(nenergy)) deallocate(nenergy)

      return
      end subroutine dealloc_vctransmod


      end module vctransmod


