      module xcurlmod

use decimal, only: long,dop

      implicit none
      
      integer(long), parameter :: maxstring=800

!------------------------------------------------------------------------
! Input/output
!------------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save     :: geom
      integer(long), save                                :: nqc,formout,&
                                                            ndat,igeom, &
                                                            igeompos,&
                                                            igeomneg
      integer(long), dimension(:), allocatable, save     :: stachk
      character(len=80), save                            :: qcfile,ainp,astem,&
                                                            aset,aout,alog
      character(len=80), dimension(:), allocatable, save :: aqc
      character(len=2), dimension(:), allocatable, save  :: aatm
      character(len=80), dimension(:), allocatable, save :: adat

!------------------------------------------------------------------------
! Numerics
!------------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save   :: issel
      real(dop), save                                  :: step,nstep
      real(dop), dimension(:), allocatable, save       :: xcoo,diff
      real(dop), dimension(:,:,:), allocatable, save   :: tau
      real(dop), dimension(:,:,:,:), allocatable, save :: postau,negtau,&
                                                           dtaudq,delxtau,&
                                                           tauxtau,ftensor
      logical(kind=4), save                            :: lcalc,lpnt,lnum,&
                                                           l2pnt,l3pnt,lddr,&
                                                           lssel
      contains

!#######################################################################
!
! XCURLMOD_ALLOC
!
! memory required for arrays in xcurlmodmod allocated here.
!
! First check if allocated, then allocate. All arrays must be
! deallocated in
! xcurlmod_dealloc
!
!#######################################################################

      subroutine alloc_xcurlmod

use maxv
use channels

      implicit none

      integer(long)     :: ierr
      character(len=10) :: cerr

! check all arrays are not allocated
      call dealloc_xcurlmod

      cerr = 'geom'
      allocate(geom(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'stachk'
      allocate(stachk(maxstring),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'aqc'
      allocate(aqc(maxstring),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'aatm'
      allocate(aatm(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'adat'
      allocate(adat(maxset),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'issel'
      allocate(issel(maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'xcoo'
      allocate(xcoo(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'diff'
      allocate(diff(maxdim),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tau'
      allocate(tau(maxdim,maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'postau'
      allocate(postau(maxdim,maxdim,maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'negtau'
      allocate(negtau(maxdim,maxdim,maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'dtaudq'
      allocate(dtaudq(maxdim,maxdim,maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'delxtau'
      allocate(delxtau(maxdim,maxdim,maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'tauxtau'
      allocate(tauxtau(maxdim,maxdim,maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000
      cerr = 'ftensor'
      allocate(ftensor(maxdim,maxdim,maxsta,maxsta),stat=ierr)
      if (ierr > 0) go to 1000

      return
 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_XCURLMOD :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop
      end subroutine alloc_xcurlmod

!#######################################################################

      subroutine dealloc_xcurlmod

      if (allocated(geom)) deallocate(geom)
      if (allocated(stachk)) deallocate(stachk)
      if (allocated(aqc)) deallocate(aqc)
      if (allocated(aatm)) deallocate(aatm)
      if (allocated(adat)) deallocate(adat)
      if (allocated(issel)) deallocate(issel)
      if (allocated(xcoo)) deallocate(xcoo)
      if (allocated(diff)) deallocate(diff)
      if (allocated(tau)) deallocate(tau)
      if (allocated(postau)) deallocate(postau)
      if (allocated(negtau)) deallocate(negtau)
      if (allocated(dtaudq)) deallocate(dtaudq)
      if (allocated(delxtau)) deallocate(delxtau)
      if (allocated(tauxtau)) deallocate(tauxtau)
      if (allocated(ftensor)) deallocate(ftensor)

      return
      end subroutine dealloc_xcurlmod

      end module xcurlmod
    
