!program to convert vctrans info file into dd db
program info_to_db

   use decimal, only: long,dop,sip
   use vcglobal
   use global, only:pi, fs, ev, keyword,keyorig,maxkey,lc,lwarn,tiny,ddname,&
                    ddlaenge,lddinfo, c1,lcivec
   use versions

   use utils, only: vcerrormsg,rmblnk0,vchdefault,x2q,q2curv,curv2q,x2qf,&
   rotatemodes,rotpairvec,q2x,transformq1
   use logdat, only: message, routine, inptit
   use op1lib, only: zerovxd, zerovxi, zerovxl
   
   use symmetry, only: getnadd,getnswitch
   use vchammod
   !use vcplotmod
   use vchfitmod
   use rddatamod, only: dataset_info,read_dataset,dataset_dims
   use invchammod
   use readqc

   use sqlite
   use db_sql
   use dd_db, only: getdbnrec,preparedb,dbfinalise,dddb_wr1,ddx2q,&
                    prepsym
   use dbtools, only: closedb,iadipdb,iagradb,iahesdb,iapesdb,igeodb,imocodb,inactdb
   
   use dirdyn, only: alloc_dirdyn,dealloc_dirdyn,dbnrec,ndofddpes,ndofdd,mref,&
                     dercpdim,ldiabdb,lmocodb,nbasis,xref,atnam,dde0,nroot,&
                     nsymop,dbpntgp,dimref,nfam,lrediab,dbinterp,ntr,&
                     alloc_dddb,dbsymmap,sqlvers,makeflagarray,makeinfoarray,qcbasis,&
                     the_db, xatnam
   use dirdyn, only: lapesdb,lagradb,lahesdb,lnactdb,lddsoc, &
                     lpesdb, lgradb, lhesdb, ldipdb, ltransdb, &
                     dirdyn_natm => natm  !This alias the natm from dirdyn to 
                                       !avoid clash with vcglobal
                                       !Probably a little bit wrong - MAP

   use griddatmod, only: nddstate
   use symoplib, only: fnsymop,fpntgp

   use keyunitsmod !, only: angstroem, atomicmassunit

   use logdat
   use module_interface

   implicit none 

   character(len=c5)                           :: filename
   character(len=80)                           :: string,ofilename,qcprog, qcmethod
   character(len=10)                           :: cerr

   integer(long), external                     :: slen
   integer(long)                               :: ilbl, iarg, iqc,interp,irec

   integer(long)                               :: nrec,idbrd, npnts, ierr  !# of database records, how to interact with DB, 

   complex(dop), dimension(:), allocatable     :: mc
   real(dop), dimension(:), allocatable        :: mr
   integer(long), dimension(:), allocatable    :: mi
   logical(kind=4), dimension(:), allocatable  :: ml
   complex(sip), dimension(:), allocatable     :: ms
   real(sip), dimension(:), allocatable        :: mf
   logical                                     :: xpoints,file_exists, lqcflag, ldatflag
   logical(kind=4)                              :: check    !check if a file exists
   real(dop), dimension(:,:), allocatable       :: cpoint  !array of cart coordinates
   real(dop), dimension(:,:,:), allocatable     :: vpoint !array of adiapes (Diagonal is energy)
   real(dop), dimension(:,:,:), allocatable     :: dvpoint, dcvpoint !arrays of diabatic pes (Diagonal is energy, off is SOC) and
complex diapes
   real(dop), dimension(:,:,:), allocatable     :: trans,ctrans
   real(dop), dimension(:,:,:,:), allocatable   :: adpoint, ddpoint   !arrays of adiadiff, diadiff (Diagonal is deriv)
   real(dop), dimension(:,:,:,:,:), allocatable :: ahpoint, dhpoint   !arrays of adia, dia hessians (Diagonal is Hess)
   real(dop), dimension(:,:,:,:), allocatable   :: adippoint, ddippoint   !arrays of dipole moments
   real(dop), dimension(:,:,:), allocatable     :: dcppoint !array derivative couplings
   complex(dop), dimension(:,:,:), allocatable  :: asocpoint !array for adiab SOCs. 
   real(sip), dimension(:,:,:), allocatable     :: mopoint !array of molecular orbitals
   character(len=30), dimension(1,1)            :: ciconfig
   real(dop), dimension(1,1,1)                  :: cicoeff
   character(200), dimension(:), allocatable    :: listorb  !list of orbfiles
   integer(long), dimension(:), allocatable     :: orbstate !list of states for the orbs 

   real(dop), dimension(:), allocatable         :: xcoo0b !Ref geom in Bohr not Ang
   real(dop)                                    :: angcon, amucon !conversions of units, from Angstrom->au and from amu->au

   integer                                   ::i,n,j      !counters

   character(200)                         :: pntgroup !what is pointgroup
   integer(long), external                :: myiargc  !CLI options
   integer(long), allocatable,dimension(:)     :: gwpsymrep  ! symmetry replicas

   type(SQLITE_DATABASE)                       :: the_db_old

   open(ilog,file='inf_db.log',form='formatted',status='unknown')
   !-----------------------------------------------------------------------
   ! Add options - these should be read from CLI to do properly
   !-----------------------------------------------------------------------
   !set some defaults
   ldatflag = .false. !Is Datfile given?
   lmocodb = .false. !Are orbitals needed
   lqcflag = .false. !is qcprog inputted?

   iarg = myiargc()
   write(ilog,*) "cli args", iarg
   if (iarg .eq. 1) then
      i = 1
      call mygetarg(i,string)
      if (string .eq. '-h') then
         write(*,*) "Converts a .info file from VCHAM into a Quantics Direct Dynamics Database"
         write(*,*) "Options:"
         write(*,*) "      -f, the info filename, must be included"
         write(*,*) "      -dd, database directory, must exist (currently)"
         write(*,*) "      -orb, the file name containing the list of orbs. "
         write(*,*) "            This includes those loaded as files not datasets"
         write(*,*) "            should also provide:"
         write(*,*) "                         QC program to allow reading"
         write(*,*) "                         Nbasis so memory can be allocated"
         write(*,*) "      -qc, the QC program, acceptable names are:. "
         write(*,*) "            Gaussian, Molpro & Molcas"
         write(*,*) "      -nb, number of orbitals in the basis set of QC calculation"
      end if
      stop
   else if (iarg .gt. 0) then
      i = 1
      do while (i .lt. iarg)
         call mygetarg(i,string)
         !-----------------------------------------------------------------------
         ! options start with -
         !-----------------------------------------------------------------------

         if (string(1:1) .eq. '-') then
            if(string .eq. '-f') then
               !this is the infofile
               !Get file
               i = i + 1
               call mygetarg(i,string)
               datfile = string
               ldatflag = .true.
               write(ilog,*) "Data file ", datfile
            else if (string .eq. '-dd') then
               !this is the db directory
               !Get dir
               i = i + 1
               call mygetarg(i,string)
               ddname = string
               write(ilog,*) "database name ", ddname
            else if (string .eq. '-orb') then
               !Going to read in MOs
               i = i + 1
               call mygetarg(i,string)
               ofilename = string
               lmocodb = .true.  !set flag to indicate going to read MOs in
               write(ilog,*) "Will read in CAS MOs"
               write(ilog,*) "file with orbital list ", ofilename
            else if (string .eq. '-qc') then
               !this is qc program type
               i = i + 1
               call mygetarg(i,string)
               qcprog = string
               lqcflag = .true.
               write(ilog,*) "Program which generated MOs ", qcprog 
            else if (string .eq. '-nb') then
               !this is MO basis size
               i = i + 1
               call mygetarg(i,string)
               read(string,*) nbasis
               write(ilog,*) "MO basis size", nbasis
            else if (string .eq. '-h') then
               write(*,*) "Converts a .info file from VCHAM into a Quantics Direct Dynamics Database"
               write(*,*) "Options:"
               write(*,*) "      -f, the info filename, must be included"
               write(*,*) "      -dd, database directory, must exist (currently)"
               write(*,*) "      -orb, the file name containing the list of orbs. "
               write(*,*) "            This includes those loaded as files not datasets"
               write(*,*) "            should also provide:"
               write(*,*) "                         QC program to allow reading"
               write(*,*) "                         Nbasis so memory can be allocated"
               write(*,*) "      -qc, the QC program, acceptable names are:. "
               write(*,*) "            Gaussian, Molpro & Molcas"
               write(*,*) "      -nb, number of orbitals in the basis set of QC calculation"
            else
               write(ilog,*) "Unknown option: ", string
               write(*,*) "Unknown option: ", string
            endif
         endif
         i = i + 1
      end do   
   endif
   !Put in checks here - file exists, dir exists etc...
   if (ldatflag .eqv. .false.) then
      !No filename
      write(6,*) 'ERROR: No filename given'
      stop
   endif

   if (lmocodb .eqv. .true.) then
      !check that the orb file list exists
      INQUIRE(FILE=ofilename, EXIST=file_exists)
      if (file_exists .eqv. .false.) then
         write(6,*) "ERROR: Can't find orb list file"
         stop 
      endif
      !Check that qcprog is not empty
      if (lqcflag .eqv. .false.) then
         write(6,*) "ERROR: No QC program given"
         stop
      else
         call To_lower(qcprog)
         call setqcprog(qcprog, iqc)
         if ((iqc .eq. 2) .or. (iqc .eq. 4) .or. (iqc .eq. 6) .or. (iqc .eq. 7) ) then 
            write(6,*) "ERROR: Asking for MOs from programme that has none/not implemented"
            stop
         endif 
      endif
      !Check nbasis is set
      if (nbasis .LT. 1) then
         write(6,*) "ERROR: Number of basis sets"
         stop
      end if
   endif

   lddinfo = .true.
   ldiabdb = .true.
   lcivec  = .false.    ! No CI Vector
   
   !nbasis = 82
   pntgroup = "c1"   !To add to refdb

   ddlaenge=index(ddname,' ')-1
   !-----------------------------------------------------------------------
   ! Get units conversions
   !-----------------------------------------------------------------------
   angcon = 1
   amucon = 1
   call keylunits(angcon,'angst',ilog)
   call keymunits(amucon,'amu',ilog)
   !-----------------------------------------------------------------------
   ! open log and data files
   !-----------------------------------------------------------------------
         
   ilbl=slen(datfile)
   write(6,'(/,2a)') 'Data read from file : ',datfile(1:ilbl)
   open(idat,file=datfile,form='formatted',status='old',iostat=ierr)
      if (ierr .ne. 0) then
         routine = 'Vcplot'
         write(message,'(2a)') 'No infofile found: ',datfile(1:ilbl)
         call vcerrormsg
      endif
   
   !-----------------------------------------------------------------------
   ! get maxdim and then allocate memory
   !-----------------------------------------------------------------------
   nmodes=-999
   nsta=-999
   ntr=-999

   do
      read(idat,'(a25)',end=40) string
      if (string(1:1) .ne. '#') cycle
      if (string(2:21) .eq. 'internal_coordinates') then
         read(idat,*) nmodes
      endif
      if (string(2:14) .eq. 'trivial_modes') then
         read(idat,*) ntr
      endif
      if (string(2:11) .eq. 'num_states') then
         read(idat,*) nsta
      endif

   enddo

   40 continue
   maxdim=nmodes+ntr
   !maxdim=ncoo
   maxsta=nsta

   if (maxdim .lt. 0) then
      routine='VCTRANS'
      message='Error reading dimensions from input file'
      call vcerrormsg
   endif
   if (maxsta .lt. 0) then
      routine='VCTRANS'
      message='Error reading no. states from input file'
      call vcerrormsg
   endif

   call alloc_vchammod
   !call alloc_vcplotmod
   call alloc_vchfitmod

   call wrversion(ilog)
   !-----------------------------------------------------------------------
   ! set default values
   !-----------------------------------------------------------------------
   call vchdefault
   weighttype=3
   nmlab(:) = ' '
   !-----------------------------------------------------------------------
   ! get system information from info file
   ! normal modes are scaled and rotated in here!
   !-----------------------------------------------------------------------
   write(*,*) 'Read dataset info'
   write(ilog,*) 'Read dataset info'
   
   call dataset_dims

   call dataset_info
   !-----------------------------------------------------------------------
   ! determine the no. explicitely added terms
   !-----------------------------------------------------------------------
   call getnadd(iguess)
   call getnswitch(iguess)

   !-----------------------------------------------------------------------
   ! assign memory 
   !-----------------------------------------------------------------------
   write(*,*)'Memory assigned'
   write(ilog,*) 'Memory assigned'
   call memvcham

   !-----------------------------------------------------------------------
   ! call routine
   !-----------------------------------------------------------------------
   ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
   write(ilog,'(/,a)') 'MEMORY ALLOCATED'
   write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)')&
   ' complex*16 : ', mcdim,&
   ' real*8     : ', mrdim,&
   ' integer    : ', midim,&
   ' logical    : ', mldim,&
   ' complex*8  : ', msdim,&
   ' real*4     : ', mfdim,&
   ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
   call myflush(ilog)

   if(allocated(mc))deallocate(mc)
   if(allocated(mr))deallocate(mr)
   if(allocated(mi))deallocate(mi)
   if(allocated(ml))deallocate(ml)
   if(allocated(ms))deallocate(ms)
   if(allocated(mf))deallocate(mf)

   cerr = 'vcplot_mc '
   allocate(mc(mcdim),stat=ierr)
   if (ierr > 0) go to 2000
   cerr = 'vcplot_mr '
   allocate(mr(mrdim),stat=ierr)
   if (ierr > 0) go to 2000
   cerr = 'vcplot_mi '
   allocate(mi(midim),stat=ierr)
   if (ierr > 0) go to 2000
   cerr = 'vcplot_ml '
   allocate(ml(mldim),stat=ierr)
   if (ierr > 0) go to 2000
   cerr = 'vcplot_ms '
   allocate(ms(msdim),stat=ierr)
   if (ierr > 0) go to 2000
   cerr = 'vcplot_mf '
   allocate(mf(mfdim),stat=ierr)
   if (ierr > 0) go to 2000

   call zerovxd(mr,mrdim)
   call zerovxi(mi,midim)
   q0val(:) = 0.0_dop

   !----------------------------------------------------------------------
   ! read the info data file 
   !----------------------------------------------------------------------
   write(*,*) 'Reading info file'
   write(ilog,*) 'Reading info file'
   !Now tell programme we want cuts froms all modes
   do i=1,nmodes
      mfit(i) = 1
   enddo

   rewind(idat)
   call read_dataset(mr(mrenergy),mr(mrxcoo),mr(mrqcoo),mr(mrder1),mr(mrdcp),&
        mr(mrder2),mr(mrtdip),mr(mrxder1),mr(mrxdcp),mr(mrxder2),q0val)     
   close(idat)                                              

! set flags for info read and to be written to DB
   lapesdb = .true.
   if (info(1,1) .ge. 1) lagradb = .true.
   if (info(1,1) .eq. 2) lahesdb = .true.
   if (info(1,2) .eq. 1) lnactdb = .true.

   if (abs(q_shift - 0.0_dop) .gt. tiny) then
      call shiftq(mr(mrqcoo))
   endif
   !----------------------------------------------------------------------
   !If required read in list of mocoefficents
   !----------------------------------------------------------------------
   if (lmocodb .eqv. .true.) then
      allocate(listorb(nsets))
      allocate(orbstate(nsets))
      call readordlist(ofilename,listorb,orbstate) 
   end if
   !allocate array for ref geom in Bohr not Ang
   allocate(xcoo0b(maxdim))
   xcoo0b = xcoo0 * angcon   !Change this to refer correct unit pattern
   !----------------------------------------------------------------------
   !Grad and linear coupling are in NM convert to Cart
   !---------------------------------------------------------------------- 
   write(*,*) 'Info file read'
   write(ilog,*) 'Info file read'
   !----------------------------------------------------------------------
   !Now have info file read in
   !----------------------------------------------------------------------   

   !----------------------------------------------------------------------
   !Now set up DB to write into
   !----------------------------------------------------------------------
   idbrd = 0   !This is save to DB

   !----------------------------------------------------------------------
   !Generate refdb file from info
   !---------------------------------------------------------------------- 

   check = .false.
   filename = ddname(1:ddlaenge)//'/database.sql'
   ilbl = index(filename,' ')-1
   inquire(file=filename,exist=check)
   !if it exists go no further
   if (check) then
      write(6,*) 'ERROR : DB already exists '
      write(6,*) filename(1:ilbl)
      stop
   endif

   ! variables needed by DB routines from info file
   ndofddpes = ncoo  
   natm = ncoo / 3
   dirdyn_natm = natm
   ndofdd = nmodes
   nddstate = nsta
   nroot = nsta
   dde0 = e0
   dimref = 0
   nfam = 0
   lddsoc = .false.
   lsocbas = .false.
   !----------------------------------------------------------------------  
   !Allocate arrays for db
   !----------------------------------------------------------------------  
   call alloc_dirdyn
   call alloc_dddb
   !----------------------------------------------------------------------  
   !Assign dirdyn arrays from info file
   !----------------------------------------------------------------------  
   do n=1,natm
      mref(n) = mass(n*3)
      call getsym(atsym(n),mref(n))
      !call getisym(atsym(n),mref(n),ilog)
      xatnam(n) = atsym(n)
      do j = 1,3
         xref(j,n) = xcoo0b(j)
      end do
   enddo
   dercpdim=nddstate*(nddstate-1)/2
   !-----------------------------------------------------------------------
   ! get no. of symmetry operations to be performed
   !-----------------------------------------------------------------------
   dbpntgp = fpntgp(pntgroup)
   nsymop = fnsymop(dbpntgp)
   if (nsymop .eq. -99) then
      routine = 'Getddpes (directdyn.f90)'
      write(message,'(a,i5)') 'Unknown point group : ',dbpntgp
      call errormsg
   endif
   !This is to write so need infoarray and filearray
   !Needed for NEWSQL
   call makeflagarray
   call makeinfoarray(nddstate)
   
   if (dbpntgp .gt. 0) call prepsym
   qcmethod = "UNKNOWN"

   !-----------------------------------------------------------------------
   ! refdb now exists
   !-----------------------------------------------------------------------
   the_db = the_db_old

   call preparedb(idbrd,1)
   !-----------------------------------------------------------------------
   ! database files are all now created, write to them
   !-----------------------------------------------------------------------
   !Work out number of points of same geom
   npnts = nsets/nsta
   nrec = 0
   !Set up new arrays for sending to db and set to 0
   allocate(cpoint(ncoo,nsets))
   cpoint = 0
   !Adiabatic - these will be filled
   allocate(vpoint(nsta,nsta,nsets))
   vpoint = 0
   allocate(adpoint(ncoo,nsta,nsta,nsets))
   adpoint = 0
   allocate(ahpoint(ncoo,ncoo,nsta,nsta,nsets))
   ahpoint = 0
   allocate(adippoint(3,nsta,nsta,nsets))
   adippoint = 0
   allocate(dcppoint(ncoo,dercpdim,nsets))
   dcppoint = 0
   allocate(asocpoint(nsta,nsta,nsets))  ! asocpoint not used but needed for DB write
   !Diabatic - will remain empty
   allocate(dvpoint(nsta,nsta,nsets))
   allocate(dcvpoint(nsta,nsta,nsets))
   allocate(ddpoint(ncoo,nsta,nsta,nsets))
   allocate(dhpoint(ncoo,ncoo,nsta,nsta,nsets))
   allocate(ddippoint(3,nsta,nsta,nsets))
   allocate(trans(nsta,nsta,nsets))
   allocate(ctrans(nsta,nsta,nsets))
   !Set flags to false for diabatic values
   lpesdb = .false.  
   lgradb = .false.
   lhesdb = .false.
   ldipdb = .false.
   ltransdb = .false.

   !Allocate for MOs if needed
   if (lmocodb .eqv. .true.) then
      allocate(mopoint(nbasis,nbasis,nsets))
   end if   
   allocate(gwpsymrep(nsymop))

   !Generate arrays to send to DB writer
   call inf_order_db_order_cart(mr(mrxcoo),mr(mrenergy),mr(mrxder1),&
        mr(mrxder2),mr(mrtdip),mr(mrxdcp),cpoint,vpoint,adpoint,ahpoint,&
        adippoint,dcppoint,listorb,orbstate,mopoint,nrec)                       
   write(ilog,'(/,a,i5)') 'No. of distinct data points in database: ',nrec
   write(ilog,'(/,a)') "Writing to DB files"
   write(*,'(/,a)') "Writing to DB files"
   interp=0
   do i = 1, nrec
      lrediab = .false.
      irec=0
      !Write to db
      gwpsymrep = 0
      call dddb_wr1(cpoint(:,i),vpoint(:,:,i),& !geometry and adiabatic energy
           adpoint(:,:,:,i),ahpoint(:,:,:,:,i), & !derivative and hessian
           adippoint(:,:,:,i),&             !dipole moment
           dvpoint(:,:,i),ddpoint(:,:,:,i), &       !diabatic energy and derivative
           dhpoint(:,:,:,:,i), ddippoint(:,:,:,i), & !diabatic hession & diabatic dipole
           mopoint(:,:,i), trans(:,:,i),   &        !MO coefficents, trans array
           dcppoint(:,:,i),              & ! derivative coupling
           dcvpoint(:,:,i)       , &       ! complex diabatic energy
           ctrans(:,:,i)         , &       ! complex ADT trafo
           ciconfig(:,:)         , &       ! ci configurations (not used)
           cicoeff(:,:,:)        , &       ! ci coefficients (not used)
           ncoo,nddstate,interp,dercpdim,gwpsymrep,&
           asocpoint(:,:,i),nddstate)               ! ASOC
                            !number degrees freedom, record number, interpolation
   end do

   !call dbfinalise

   !deallocate all the arrays
   !VCHAM
   call dealloc_vchammod
   call dealloc_vchfitmod
   
   if(allocated(mc))deallocate(mc)
   if(allocated(mr))deallocate(mr)
   if(allocated(mi))deallocate(mi)
   if(allocated(ml))deallocate(ml)
   if(allocated(ms))deallocate(ms)
   if(allocated(mf))deallocate(mf)

   !Dirdyn
   call dealloc_dirdyn
   
   !info_to_db
   if(allocated(cpoint))deallocate(cpoint)
   if(allocated(vpoint))deallocate(vpoint)
   if(allocated(adpoint))deallocate(adpoint)
   if(allocated(ahpoint))deallocate(ahpoint)
   if(allocated(adippoint))deallocate(adippoint)
   if(allocated(dcppoint))deallocate(dcppoint)
   if(allocated(dvpoint))deallocate(dvpoint)
   if(allocated(ddpoint))deallocate(ddpoint)
   if(allocated(dhpoint))deallocate(dhpoint)
   if(allocated(adippoint))deallocate(ddippoint)
   !mo lists
   if(allocated(listorb))deallocate(listorb)
   if(allocated(orbstate))deallocate(orbstate)
   if(allocated(mopoint))deallocate(mopoint)
   if(allocated(xcoo0b))deallocate(xcoo0b)
   if(allocated(gwpsymrep))deallocate(gwpsymrep)
   stop

   2000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
   stop

   contains

   subroutine shiftq(qcoo)

      implicit none
   
      integer(long)                                     :: m,z
      real(dop), dimension(nmodes,nsets), intent(inout) :: qcoo
   
      do n=1,nsets
         do m=1,nmodes
            qcoo(m,z) = qcoo(m,z) - q_shift
         enddo
      enddo
   
   end subroutine shiftq

   subroutine print_this(xcoo)
      implicit none
   
      integer(long)                                     :: m
      real(dop), dimension(ncoo,nsets), intent(inout) :: xcoo

      do m=1, nsets
         write(ilog,*) xcoo(:,m)
         write(ilog,*) " "
         write(*,*) xcoo(:,m)
      enddo

   
   end subroutine print_this
   subroutine print_this_3d(xcoo)
      implicit none
   
      integer(long)                                     :: m
      real(dop), dimension(ncoo,ncoo,nsets), intent(inout) :: xcoo

      do m=1, nsets
         write(ilog,*) xcoo(:,:,m)
         write(ilog,*) " "
         write(*,*) xcoo(:,:,m)
      enddo

   
   end subroutine print_this_3d

   subroutine grad_nm_coo(nm_arr,cart_arr)
!#######################################################################
!convert array in NM to Cart
!#######################################################################
   real(dop), dimension(ncoo),intent(in)           ::nm_arr
   real(dop), dimension(ncoo),intent(out)          ::cart_arr

   call dgemv('T',nmodes,ncoo,1.d0,coonm,maxdim,nm_arr,1,0,cart_arr,1)

   end subroutine grad_nm_coo
   subroutine grad_nm_coo_hess(nm_arr,cart_arr)
      !#######################################################################
      !convert array in NM to Cart
      !#######################################################################
      real(dop), dimension(maxdim,maxdim), intent(in)             ::nm_arr
      real(dop), dimension(ncoo,ncoo), intent(out)           ::cart_arr
      integer                                                     ::x,y,k,l
      
         !call dgemv('T',nmodes,ncoo,1.d0,coonm,maxdim,nm_arr,1,0,cart_arr,1)
         do x=1,ncoo
            do y=1,ncoo
               cart_arr(x,y)=0.0
               do k=1,nmodes
                  do l=1,nmodes
                     cart_arr(x,y)=cart_arr(x,y)&
                           +coonm(k,x)*nm_arr(k,l)*coonm(l,y)
                  enddo
               enddo
            enddo
         enddo
      
         end subroutine grad_nm_coo_hess
!#######################################################################
!  Take info file order where each data point is a geometry point
!  but different state - merge into new array which is all states for each
!  point
!  This is for Geom where most points have same Geom
!#######################################################################
   ! subroutine inf_order_db_order_geom(xcoo,point)
   !    real(dop), dimension(ncoo,nsets), intent(in)       :: xcoo
   !    real(dop), dimension(ncoo,npnts), intent(out)       :: point
   !    integer                                         :: x,y

   !    do y = 1, npnts
   !       x = 5*(y-1)+1
   !       point(:,y)=xcoo(:,x)
   !    enddo
   ! end subroutine inf_order_db_order_geom

   !#######################################################################
!  Take info file order where each data point is a geometry point
!  but different state - merge into new array which is all states for each
!  point
!  This is for Geom where most points have same Geom
!#######################################################################
   ! subroutine inf_order_db_order_av(eng,point)
   !    real(dop), dimension(nsets), intent(in)       :: eng
   !    real(dop), dimension(nsta,nsta,npnts), intent(out)       :: point
   !    integer                                         :: j,i,z

   !    do i = 1, npnts
   !       j = 5*(i-1)
   !       do z=1, nsta
   !          write(ilog,*) i, j, z,z+j, eng(z+j), asta(z+j)
   !          point(z,z,i)=eng(z+j)
   !       enddo
   !    enddo
   ! end subroutine inf_order_db_order_av

!#######################################################################
!  Take info file order where each data point is a geometry point
!  but different state - merge into new array which is all states for each
!  point
!  This is for everything
!  points are outputs to go to db -> they are sized by nsets to make sure
!  they are big enough to include worse case - all nsets are distinct
!
!  Generate adiabatic pes, diagonal is energy, off diagonal is SOC (which don't have?)
!  Generate adia deriv, diagonal is state deriv, nact vectors stored on off-diagonal
!  Generate adia hess, diagonal is state hessian
!
!  This version takes everything in nm - so projection into cart
!  This is error prone due to round off
!#######################################################################
   subroutine inf_order_db_order(geom,eng,adiff,ahess,adip,adcp,gepoint,avpoint,diffpoint,hesspoint,&
                                 dippoint,dccppoint,lorb,orbs,amo,nr)
      implicit none
      !Geometry arrays
      real(dop), dimension(ncoo,nsets), intent(in)                   :: geom
      real(dop), dimension(ncoo,nsets), intent(inout)                :: gepoint

      !Energy arrays
      real(dop), dimension(nsets), intent(in)                        :: eng
      real(dop), dimension(nsta,nsta,nsets), intent(inout)           :: avpoint
      !Differential arrays
      real(dop), dimension(ncoo,nsets), intent(in)                   :: adiff
      real(dop), dimension(ncoo,nsta,nsta,nsets), intent(inout)      :: diffpoint
      real(dop), dimension(ncoo,nsets)                               :: tempdiff
      !Hessian arrays
      real(dop), dimension(ncoo,ncoo,nsets), intent(in)              :: ahess
      real(dop), dimension(ncoo,ncoo,nsta,nsta,nsets), intent(inout) :: hesspoint
      real(dop), dimension(ncoo,ncoo,nsets)                          :: temphess
      !Transition dipole moments arrays
      real(dop), dimension(4,nsets), intent(in)                      :: adip  !4 - magnitude,x,y,z
      real(dop), dimension(3,nsta,nsta,nsets), intent(inout)         :: dippoint !3 so x,y,z
      !derivative coupling arrays
      real(dop), dimension(ncoo,nsta,nsets), intent(in)              :: adcp
      real(dop), dimension(ncoo,dercpdim,nsets), intent(inout)       :: dccppoint
      real(dop), dimension(ncoo,nsta,nsets)                          :: tempdcp
      !MO arrays
      character(200), dimension(nsets), intent(in)                   :: lorb  !list of orbfiles
      integer(long), dimension(nsets), intent(in)                    :: orbs !list of states for the orbs 
      real(sip), dimension(nbasis,nbasis,nsets), intent(inout)       :: amo   !Mo orbitals
      real(sip), dimension(nbasis,nbasis,nsets)                      :: tempmo   !Mo orbitals

      !Other variables
      integer                                      :: x,y,z,ns
      integer(long)                                :: derstate, ncur !Coupling state for dcp, ncur index of match
      integer(long), intent(inout)                 :: nr !Number of distinct geometries (should match nsets/nsta at end)
      logical(kind=4)                              :: gcheck    !check if a geom exists

      !go through each point of the data set 
      !Info array contains info on if information is loaded from file
      write(*,*) "Checking through dataset and converting"
      do y = 1, nsets
         gcheck = .false.
         
         do x = 1, nr
            !Compare geom that to current list of points
            if(all(geom(:,y).eq.gepoint(:,x))) then
               !write(*,*) "A and B are equal"
               gcheck = .true.
               ncur = x
               exit
            endif
         enddo
         
         if (.not. gcheck) then
            !No match found save this as new point
            nr = nr + 1
            !geometry
            gepoint(:,nr) = geom(:,y)
            ncur = nr
         else
         end if

         !Save Energy
         avpoint(asta(y),asta(y),ncur) = eng(y)
         !Save adiabatic Derviative
         !First convert into cart from NM
         if (.not.(all(adiff(:,y) .eq. 0))) then
            call grad_nm_coo(adiff(:,y),tempdiff(:,y))
            diffpoint(:,asta(y),asta(y),ncur) = tempdiff(:,y)
         endif
         
         !Save adiabatic Hessian
         if (.not.(all(ahess(:,:,y) .eq. 0))) then
            call grad_nm_coo_hess(ahess(:,:,y),temphess(:,:,y))

            hesspoint(:,:,asta(y),asta(y),ncur) = temphess(:,:,y)
         endif

         !Save Transition dipole moment
         dippoint(:,asta(y),asta(y),ncur) = adip(2:4,y)
         !Save derivative couplings
         do ns = 1,nsta
            !Loop through adcp and find what we want
            if (asta1(ns,y) .ne. -1) then
               derstate = (asta1(ns,y)-1)*(asta1(ns,y)-2)/2+asta(y) !position in dccppoint of data
               !First convert into cart from NM
               if (.not.(all( adcp(:,ns,y) .eq. 0))) then
                  call grad_nm_coo( adcp(:,ns,y), tempdcp(:,ns,y))
                  dccppoint(:,derstate,ncur) = tempdcp(:,ns,y)
               endif
            endif
         enddo

         !If needed read in and save MOs
         if (lmocodb .eqv. .true.) then
            !Check file exists
            INQUIRE(FILE=lorb(y), EXIST=file_exists)
            if (file_exists .eqv. .true.) then
               write(*,*) "Reading MO from ", lorb(y)
               call readqcmo(iqc,lorb(y),tempmo(:,:,y))
               amo(:,:,ncur) = tempmo(:,:,y)
            else
               write(*,*) lorb(y), "Not found"
            end if
         endif
      enddo
      write(*,*) "Info file sorted for DB"

   end subroutine inf_order_db_order

   !#######################################################################
!  Take info file order where each data point is a geometry point
!  but different state - merge into new array which is all states for each
!  point
!  This is for everything
!  points are outputs to go to db -> they are sized by nsets to make sure
!  they are big enough to include worse case - all nsets are distinct
!
!  Generate adiabatic pes, diagonal is energy, off diagonal is SOC (which don't have?)
!  Generate adia deriv, diagonal is state deriv, nact vectors stored on off-diagonal
!  Generate adia hess, diagonal is state hessian
!
!  This version takes everything in cart - no projection   
!#######################################################################
subroutine inf_order_db_order_cart(geom,eng,adiff,ahess,adip,adcp,gepoint,& 
           avpoint,diffpoint,hesspoint,dippoint,dccppoint,lorb,orbs,amo,nr)
implicit none
!Geometry arrays
real(dop), dimension(ncoo,nsets), intent(in)                   :: geom
real(dop), dimension(ncoo,nsets), intent(inout)                :: gepoint
real(dop), dimension(ncoo)                                     ::tempgeom


!Energy arrays
real(dop), dimension(nsets), intent(in)                        :: eng
real(dop), dimension(nsta,nsta,nsets), intent(inout)           :: avpoint
!Differential arrays
real(dop), dimension(ncoo,nsets), intent(in)                   :: adiff
real(dop), dimension(ncoo,nsta,nsta,nsets), intent(inout)      :: diffpoint

!Hessian arrays
real(dop), dimension(ncoo,ncoo,nsets), intent(in)              :: ahess
real(dop), dimension(ncoo,ncoo,nsta,nsta,nsets), intent(inout) :: hesspoint

!Transition dipole moments arrays
real(dop), dimension(4,nsets), intent(in)                      :: adip  !4 - magnitude,x,y,z
real(dop), dimension(3,nsta,nsta,nsets), intent(inout)         :: dippoint !3 so x,y,z
!derivative coupling arrays
real(dop), dimension(ncoo,nsta,nsets), intent(in)              :: adcp
real(dop), dimension(ncoo,dercpdim,nsets), intent(inout)       :: dccppoint
real(dop), dimension(ncoo,nsta,nsets)                          :: tempdcp
!MO arrays
character(200), dimension(nsets), intent(in)                   :: lorb  !list of orbfiles
integer(long), dimension(nsets), intent(in)                    :: orbs !list of states for the orbs 
real(sip), dimension(nbasis,nbasis,nsets), intent(inout)       :: amo   !Mo orbitals
real(sip), dimension(nbasis,nbasis,nsets)                      :: tempmo   !Mo orbitals

!Other variables
integer                                      :: x,y,z,ns
integer(long)                                :: derstate, ncur !Coupling state for dcp, ncur index of match
integer(long), intent(inout)                 :: nr !Number of distinct geometries (should match nsets/nsta at end)
logical(kind=4)                              :: gcheck    !check if a geom exists

!go through each point of the data set 
!Info array contains info on if information is loaded from file
write(*,*) "Checking through dataset and converting"
do y = 1, nsets
   gcheck = .false.

   !Compare geom that to current list of points convert to bohr
   tempgeom = geom(:,y)* angcon
   do x = 1, nr
      !if(all(geom(:,y).eq.gepoint(:,x))) then
      if(all(tempgeom.eq.gepoint(:,x))) then
         !write(*,*) "A and B are equal"
         gcheck = .true.
         ncur = x
         exit
      endif
   enddo

   if (.not. gcheck) then
      !No match found save this as new point
      nr = nr + 1
      !geometry
      !gepoint(:,nr) = geom(:,y)
      gepoint(:,nr) = tempgeom
      ncur = nr
      else
   end if

   !Save Energy
   avpoint(asta(y),asta(y),ncur) = eng(y)

   !Save adiabatic Derviative
   !First convert into cart from NM
   if (.not.(all(adiff(:,y) .eq. 0))) then
      diffpoint(:,asta(y),asta(y),ncur) = adiff(:,y)
      lagradb = .true.
   endif

   !Save adiabatic Hessian
   if (.not.(all(ahess(:,:,y) .eq. 0))) then
      hesspoint(:,:,asta(y),asta(y),ncur) = ahess(:,:,y)
      lahesdb = .true.
   endif

   !Save Transition dipole moment
   dippoint(:,asta(y),asta(y),ncur) = adip(2:4,y)

   !Save derivative couplings
   do ns = 1,nsta
      !Loop through adcp and find what we want
      if (asta1(ns,y) .ne. -1) then
         derstate = (asta1(ns,y)-1)*(asta1(ns,y)-2)/2+asta(y) !position in dccppoint of data
         !First convert into cart from NM
         if (.not.(all( adcp(:,ns,y) .eq. 0))) then
            dccppoint(:,derstate,ncur) = adcp(:,ns,y)
            lnactdb = .true.
         endif
      endif
   enddo

   !If needed read in and save MOs
   if (lmocodb .eqv. .true.) then
      !Check file exists
      INQUIRE(FILE=lorb(y), EXIST=file_exists)
      if (file_exists .eqv. .true.) then
         write(*,*) "Reading MO from ", lorb(y)
         call readqcmo(iqc,lorb(y),tempmo(:,:,y))
         amo(:,:,ncur) = tempmo(:,:,y)
      else
         write(*,*) lorb(y), "Not found"
      end if
   endif
enddo
write(*,*) "Info file sorted for DB"

end subroutine inf_order_db_order_cart
!#######################################################################
!  opposite of getmass function
!  take mass and try to assign atomic symbol
!  range is a bit wide, could be smaller in some cases
!#######################################################################

   subroutine getsym(atna,amass)

      !integer(long)                 :: ierr
      real(dop), intent(in)        :: amass
      character(len=c1), intent(out) :: atna

      if ((amass .gt. 2) .and. (amass .lt. 3)) then
         atna = 'H(Iso=2)'
      else if ((amass .gt. 1) .and. (amass .lt. 2)) then
         atna = 'H'
      else if ((amass .gt. 4) .and. (amass .lt. 5)) then
         atna = 'He'
      else if ((amass .gt. 7) .and. (amass .lt. 8)) then
         atna = 'Li'
      else if ((amass .gt. 9) .and. (amass .lt. 10)) then
         atna = 'Be'
      else if ((amass .gt. 11) .and. (amass .lt. 11.5)) then
         atna = 'B'
      else if ((amass .gt. 12) .and. (amass .lt. 13)) then
         atna = 'C'
      else if ((amass .gt. 14) .and. (amass .lt. 15)) then
         atna = 'N'
      else if ((amass .gt. 15) .and. (amass .lt. 17)) then
         atna = 'O'
      else if ((amass .gt. 18) .and. (amass .lt. 19.5)) then
         atna = 'F'
      else if ((amass .gt. 19.5) .and. (amass .lt. 20.5)) then
         atna = 'Ne'
      else if ((amass .gt. 31) .and. (amass .lt. 33)) then
         atna = 'S'
      else if ((amass .gt. 126.5) .and. (amass .lt. 127.5)) then
         atna = 'I'
      else if ((amass .gt. 35.3) .and. (amass .lt. 36.5)) then
         atna = 'Cl'
      else
         routine='Getsym (info_to_db.f90)'
         write(message,'(2a)') 'Atom symbol not found',amass
      endif

   end subroutine getsym

!#######################################################################
! 
! Write the new record to the DB.
!  This is to only save adiabatic from info file
!#######################################################################
                                                                        
   subroutine dddb_awr1(gpoint,av,aderiv1,aderiv2,adip,&
                  dercp,mocoeff,ndof1,nre,interp)    

   use global
   use griddatmod
   use operdef
   use dirdyn

   !+----------------------------------------------------------------------
   !     write database for direct dynamics                               
   !+----------------------------------------------------------------------
   implicit none                                                     
                                                      
   integer(long)                                              :: s,s1,x,&
                                                   ns,n1
   integer(long), intent(in)                                  :: ndof1,interp
   integer(long), intent(inout)                               :: nre
   real(dop), dimension(ndof1), intent(in)                    :: gpoint
   real(dop), dimension(ndof1,dercpdim), intent(in)            :: dercp
   real(dop), dimension(nddstate,nddstate), intent(in)        :: av
   real(dop), dimension(ndof1,nddstate,nddstate), intent(in)       :: aderiv1
   real(dop), dimension(ndof1,ndof1,nddstate,nddstate), intent(in) :: aderiv2
   real(dop), dimension(3,nddstate,nddstate), intent(in)      :: adip
   real(sip), dimension(nbasis,nbasis), intent(in)            :: mocoeff
   !real(dop), dimension(nddstate,nddstate), intent(in)        :: trans
   real(dop), dimension(:),allocatable                        :: nmpoint

   character(len=c1) :: filestat,filepos
   
   nre = nre + 1
   !-----------------------------------------------------------------------
   ! open database files                                         
   !-----------------------------------------------------------------------
   filestat = 'old'
   filepos = 'append'

   call opendb(filestat,filepos)

   !-----------------------------------------------------------------------
   ! write various quantities in database files                           
   !-----------------------------------------------------------------------
   !write(*,*) igeodb, lapesdb
   !write(*,*) interp,(gpoint(i),i=1,ndof1)     
   write(igeodb) interp,(gpoint(x),x=1,ndof1)
   ! if (lddtrans) then
   !    allocate(nmpoint(ndofdd))
   !    call ddx2q(gpoint,nmpoint)
   !    write(inmdb)(nmpoint(i),i=1,ndofdd)
   !    deallocate(nmpoint)
   ! endif

   if (lmocodb) write(imocodb) ((mocoeff(n,s),n=1,nbasis),s=1,nbasis)
   !if (ltransdb) write(itransdb) ((trans(s1,s),s1=1,nddstate),s=1,nddstate)
   if (lnactdb) write(inactdb) ((dercp(ns,s),ns=1,ndof1),s=1,dercpdim) 
   if (lapesdb) write(iapesdb) (av(s,s),s=1,nddstate)  
   if (lagradb) write(iagradb) ((aderiv1(ns,s,s),ns=1,ndof1),s=1,nddstate)
   if (lahesdb) write(iahesdb) (((aderiv2(ns,n1,s,s),ns=1,ndof1),n1=1,ndof1),s=1,nddstate)
   if (ladipdb) write(iadipdb) ((adip(ns,s,s),ns=1,3),s=1,nddstate)
   ! ! write lower triangle of AV to SOC
   ! if (lsocdb) write(isocdb) ((av(s1,s),s1=s+1,nddstate),s=1,nddstate-1)

   !-----------------------------------------------------------------------
   ! close DB files
   !-----------------------------------------------------------------------
   filestat = 'keep'
   call closedb(filestat)                                                     

   return                                                        
   end subroutine dddb_awr1 
   !#######################################################################
   ! 
   !  Read in list of orbital files to read
   !  
   !#######################################################################  
   subroutine readordlist(nfilename, orblist,statelist)
      
      character(len=80), intent(in)                         :: nfilename  
      character(200),dimension(nsets), intent(out)          :: orblist
      integer,dimension(nsets), intent(out)                 :: statelist
      integer                                               :: x


      !Open list file
      open(1,file=nfilename,form="formatted",status='old')
      do x= 1, (nsets)
         read(1,*) orblist(x), statelist(x)
         orblist(x) = trim(orblist(x))
      enddo
      close(1)
   end subroutine readordlist

   !#######################################################################
   ! 
   ! Work out which qcprog to read MOs from - 
   !  this is from dd interface and may change  
   !  Prob best to make a function in dd so changes get tracked here
   !#######################################################################  
   subroutine setqcprog(qcprogs, qfunc)
      character(len=80), intent(in)             ::qcprogs
      integer(long), intent(out)                      ::qfunc

      if (qcprogs .eq. 'gaussian') then
         qfunc = 1
      else if (qcprogs .eq. 'qommma') then
         qfunc = 2
      else if (qcprogs .eq. 'molpro') then
         qfunc = 3
      else if (qcprogs .eq. 'q-chem') then
         qfunc = 4
      else if (qcprogs .eq. 'molcas') then
         qfunc = 5
      else if (qcprogs .eq. 'orca') then
         qfunc = 6
      else if (qcprogs.eq. 'external') then !adding external program when calculating potential, first and second derivatives from external programs 
         qfunc = 7    
      else if (qcprogs .eq. 'none') then
         qfunc = 8
      else
         routine = 'Defqchem'
         message = 'Quantum chemistry program not implemented yet'
         call errormsg
      endif
   end subroutine setqcprog
   !#######################################################################
   ! 
   !  Read MO from file, call qc functions in dd interface
   !  functions all in funcqchemmod
   !#######################################################################
   subroutine readqcmo(qfunc,mofilename,moarray)
      
      integer(long), intent(in)                          ::qfunc
      character(200), intent(in)                         ::mofilename
      real(sip), dimension(nbasis,nbasis), intent(out)   :: moarray

      integer(long)                                      :: idiab,iorder
      !We need these arrays to use dd functions - but don't need contents
      real(dop), dimension(nddstate,nddstate)            :: v
      real(dop), dimension(3,natm,nddstate,nddstate)     :: deriv1
      real(dop), dimension(ncoo,ncoo,nddstate,nddstate)  :: deriv2
      integer(long)                                      :: iflag
      integer(long)                                      :: ifiletyp,imethod,&
                                                            istate,cstate
      integer(long), dimension(6)                        :: found
      logical, dimension(5)                              :: found2
      real(dop), dimension(ncoo,dercpdim)                :: nact
      logical(kind=4)                                    :: lfail,lfound
      character(len=c1), dimension(natm)                 :: xatnam
      real(dop), dimension(3,nddstate,nddstate)          :: dipole
   

      select case (qfunc)

      case default

         routine = 'lhaqchem'
         write(message,'(a,i5)') &
              'Quantum chemistry program not found :',qfunc
         call errormsg

      case (1) !Gaussian
          ifiletyp = 1

      case (2) !qomma
         !No MOs for qomma so shouldn't do this
         ifiletyp = 0
      case (3) !Molpro
         idiab = 0 !This is variable should check how to set
         iorder = 0  !Think this is not needed as just reading
         ifiletyp = 5
      case (4) !QChem
         !No cas for qchem so shouldn't do this
         ifiletyp = 0
      case (5) !MolCAS
         iorder = 0 !Think this is not needed as just reading
         ifiletyp = 11

      case (6) !Orca
         !No CAS for Orca so sholdn't do this
         ifiletyp = 0
      case (7) !External
         !Makes no sense to do this as not connect to vcham - so no info
         ifiletyp = 0
      case (8)
         continue

         return

      end select

      if (ifiletyp .gt. 0) then    
! Initialise Interface to QC output
         qc_nst = nsta
         qc_ncoo = ncoo
         qc_nbasis = nbasis
         qc_nmode = nmodes
         call alloc_readqc(ilog)

         cstate=1
         imethod = 12

!-----------------------------------------------------------------------
! Open file and read
!-----------------------------------------------------------------------
         open(idat,file=mofilename,status='old',err=1003)

! specify what needs to be read
         qcread(:)=.false.
         qcread(6)=.true.  ! MOs needed for CAS
         lfound=.true.     ! Test for finding MO coeffs
         
         call initqcout(lfail)
         call rdqcout(idat,ifiletyp,imethod,ilog,lfound,lfail)
         if (.not. lfail) call chkqcout(ifiletyp,ilog,lfound,lfail)

! Load data from QC output into local arrays
         if (qcread(6)) then
            moarray(:,:) = qcmocoeff(:,:)
         endif

         close(idat)

         call dealloc_readqc
      endif

      return
 1003 write(ilog,'(a)') 'ERROR opening MO file  ',mofilename
      stop
   end subroutine readqcmo

   subroutine To_lower(str)
      character(*), intent(in out) :: str
      integer :: x
  
      do x = 1, len(str)
        select case(str(x:x))
          case("A":"Z")
            str(x:x) = achar(iachar(str(x:x))+32)
        end select
      end do  
    end subroutine To_Lower

end program info_to_db
