!#######################################################################
!
!         VCHAM module INVCHAMMOD
!
! subroutines needed for initialising VCHAM - input file name and
! memory sizes
!***********************************************************************

      module invchammod

use decimal, only: long
use vcglobal
use global, only:pi, keyword, keyorig, maxkey, lc, logisopen
use maxv, only: maxdim,maxsta,maxset
use vchammod
use vchfitmod
use logdat, only: routine, message, inptit
use utils, only: vcwrline, vcerrormsg, vccntrl
use rddatamod, only: dataset_dims
   
      implicit none
      private
      public :: initvcham,memvcham,memlmpar

      contains

!#######################################################################
!
! INITVCHAM
!
! get name of input file and get ncoo - needed to allocate memory
!
!#######################################################################

      subroutine initvcham(pos)

      integer(long)           :: i,ic,ierr,ilbl,jlbl,ntr,nset,s,n,pos
      integer(long)           :: nsta,nsta1
      logical(kind=4)         :: linfo = .false.

      integer(long) :: iz,iz0

      ilbl=0
      jlbl=0
      filein = ' '
      filelog = ' '
      datfile = ' '
      nsta = 0
      nsta1 = 0

!-----------------------------------------------------------------------
! get input file name and file stem
!-----------------------------------------------------------------------
      call getarg(pos,filein)
      if (filein .eq. ' ') then
         write(6,'(a)') 'Input file ?'
         read(5,'(a)') filein
      endif
      if (filein .eq. ' ') then
         write(6,'(a)') 'No input file given. STOP.'
         write(6,*)
         stop
      endif

      ilbl=index(filein,'.')-1
      if (ilbl .le. 0) then
         ilbl=index(filein,' ')-1
         filein=filein(1:ilbl)//'.inp'
      endif
      filelog  = filein(1:ilbl)//'.log'

!-----------------------------------------------------------------------
! Open input file
!-----------------------------------------------------------------------
      open(iin,file=filein,form='formatted',status='old',iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a)') 'ERROR: Input file not found'
         write(6,'(a)') filein(1:80)
         write(6,*)
         stop
      endif

!-----------------------------------------------------------------------
! Open log file
!-----------------------------------------------------------------------
      open(ilog,file=filelog,form='formatted',status='unknown')
      logisopen=.true.

      ncoo = -999
      nset = -999

!-----------------------------------------------------------------------
! read input file for no. of coordinates
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'Initvcham (invchammod)'
      call vccntrl(ierr)

      i=1
   15 continue
      if (keyword(i) .eq. 'end-input') then
         goto 20

      else if (keyword(i) .eq. 'infofile') then
         linfo = .true.

         i=i+2
         datfile=keyword(i)
         jlbl=index(datfile,' ')-1
         open(idat,file=datfile(1:jlbl),form='formatted',status='old',&
              iostat=ierr)
         if (ierr .ne. 0) then
            routine = 'initvcham (invchammod)'
            write(message,'(2a)') 'Error opening infofile: ',datfile(1:jlbl)
            call vcerrormsg
         endif

! get system size, nsets and other data from info file
! set maxdim,maxsta,maxset from infor file data
         call dataset_dims
    
         close(idat)
         go to 20

      else if (keyword(i) .eq. 'nmodes') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) nmodes
            if (keyword(i+1) .eq. ',') then
               i=i+2
               read(keyword(i),*) ntr
            else
               ntr = 6
            endif
            ncoo=nmodes+ntr
         else
            routine = 'Initvcham (invchammod)'
            message = 'No argument specified with nmodes keyword'
            call vcwrline(0,iz,iz0)
         endif
         maxdim=ncoo

      else if (keyword(i).eq.'states') then
         if (keyword(i+1).eq.'=') then
            s=0
   60       continue
            s=s+1
            i=i+2
            read(keyword(i),*,iostat=ierr) nsta   ! Highest root to be read from QC
            if (ierr .ne. 0) then
               routine = 'Initvcham (invchammod)'
               message ='states must be an integer. Not :'
               call vcwrline(i,iz,iz0)
            endif

            if (keyword(i+1).eq.',') go to 60

         else
            routine = 'Initvcham (invchammod)'
            message = 'No argument specified with states keyword'
            call vcwrline(0,iz,iz0)
         endif
         maxsta=max(maxsta,nsta) 

      else if (keyword(i) .eq. 'nstates') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) nsta1  ! No. of states in VCHam
            if (ierr .ne. 0) then
               routine = 'Initvcham (invchammod)'
               message ='states must be an integer. Not :'
               call vcwrline(i,iz,iz0)
            endif

         else
            routine = 'Initvcham (invchammod)'
            message = 'No argument specified with states keyword'
            call vcwrline(0,iz,iz0)
         endif
         maxsta=max(maxsta,nsta1) 

      else if (keyword(i).eq.'geometry_define') then
         n=0
         do
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            routine = 'vcminp (read VCMIN input)'
            call vccntrl(ierr)
            if (keyword(1).eq.'end-geometry_define') exit
            n=n+1
         enddo
         ncoo=3*n

      endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

 20   continue
      close(iin)

      nactdim = nsta*(nsta-1)/2

      if (jlbl .eq. 0) then
         write(ilog,'(/,2a)')&
              '*** Dimensions read from file : ',filein(1:ilbl)//'.inp'
      else
         write(ilog,'(/,2a)')&
              '*** Dimensions read from file : ',datfile(1:jlbl)
      endif

      write(ilog,'(/,a,i5)') 'No. coordinates    : ',ncoo
      write(ilog,'(/,a,i5)') 'No. internal modes : ',nmodes
      write(ilog,'(a,i5)')   'No. states         : ',nsta
      if (nsets .gt. 0)  write(ilog,'(a,i5)')   'No. datasets       : ',nsets
      if (nsets1 .gt. 0) write(ilog,'(a)')   '    Gradients present '
      if (nsets2 .gt. 0) write(ilog,'(a)')   '    Hessians present '
      if (nsets3 .gt. 0) write(ilog,'(a)')    '    Derivative couplings present '

      return
      end subroutine initvcham

!#######################################################################
!
!         VCHAM subroutine MEMVCHAM
!
! calculate dimensions for standard arrays.
!
!#######################################################################

      subroutine memvcham

      implicit none

!-----------------------------------------------------------------------
! with the parameter "pdum" the pointers to arrays that are not
! used are initialized with the value one in order to avoid an abortion
! of the program when array bounds are checked (compiler option "-c").
!-----------------------------------------------------------------------
      integer(long), parameter :: pdum = 1

      midim=1
      mcdim=1
      mrdim=1
      
      mrenergy = mrdim  !energy of data point
      mrdim = mrenergy + nsets
      mrxcoo = mrdim    !Ground state structure in Cartesians 
      mrdim = mrxcoo + ncoo*nsets
      mrqcoo = mrdim    !Ground state structure in q
      mrdim = mrqcoo + nmodes*nsets
      mrder1 = mrdim    !Gradients in q
      mrdim = mrder1 + maxdim*nsets
      mrxder1 = mrdim    !Gradients in cart
      mrdim = mrxder1 + ncoo*ncoo*nsets1
      mrder2 = mrdim    !Hessians in q
      mrdim = mrder2 + maxdim*maxdim*nsets1
      mrxder2 = mrdim    !Hessians in cart
      mrdim = mrxder2 + ncoo*ncoo*nsets2
      mrdcp = mrdim     !Derivative Coupling
      mrdim = mrdcp + maxdim*maxdim*nsets2
      mrxdcp = mrdim     !Derivative Coupling in cart
      mrdim = mrxdcp + ncoo*nsta*nsets3
      mrconst = mrdim   !Constants
      mrdim = mrconst + nsta*nsta
      mrkappa = mrdim
      mrdim = mrkappa + nmodes*nsta
      mrlambda = mrdim
      mrdim = mrlambda + nmodes*nsta*nsta
      mrgamma = mrdim
      mrdim = mrgamma + nmodes*nmodes*nsta
      mriota = mrdim
      mrdim = mriota + nmodes*nmodes*nsta*nsta
      mrmu = mrdim
      mrdim = mrmu + nmodes*nmodes*nsta*nsta
      mrexpcoe = mrdim
      mrdim = mrexpcoe + nsta*nmodes*maxcoe
      mrhigher = mrdim
      mrdim = mrhigher + max(nhigher,pdum)
      mrtermk = mrdim
!      mrdim = mrtermk + max(nadd,pdum)
      mrdim = mrtermk + maxadd
!      mrdim = mrtermk + nadd
      mrconstini = mrdim
      mrdim = mrconstini + nsta*nsta
      mrkappaini = mrdim
      mrdim = mrkappaini + nmodes*nsta
      mrlambdaini = mrdim
      mrdim = mrlambdaini + nmodes*nsta*nsta
      mrgammaini = mrdim
      mrdim = mrgammaini + nmodes*nmodes*nsta
      mriotaini = mrdim
      mrdim = mriotaini + nmodes*nmodes*nsta*nsta
      mrmuini = mrdim
      mrdim = mrmuini + nmodes*nmodes*nsta*nsta
      mrexpcoeini = mrdim
      mrdim = mrexpcoeini + nsta*nmodes*maxcoe
      mrhigherini = mrdim
      mrdim = mrhigherini + max(nhigher,pdum)
      mrtermkini = mrdim
      mrdim = mrtermkini + max(nadd,pdum)
      mrtdip = mrdim    !T Dipole moment
      mrdim = mrtdip + 4*nsets
      mrcpcoe = mrdim
      mrdim = mrcpcoe + nsta*nsta*nmodes*maxcoe
      mrswitch = mrdim
      mrdim = mrswitch + nswitch*maxcoe
      mrcp2d = mrdim
      mrdim = mrcp2d + maxcoe*maxadd 

      midim=1
      miconstmask = midim
      midim = miconstmask + nsta*nsta
      mikappamask = midim
      midim = mikappamask + nmodes*nsta
      milambdamask = midim
      midim = milambdamask + nmodes*nsta*nsta
      migammamask = midim
      midim = migammamask + nmodes*nmodes*nsta
      miiotamask = midim
      midim = miiotamask + nmodes*nmodes*nsta*nsta
      mimumask = midim
      midim = mimumask + nmodes*nmodes*nsta*nsta
      miexpcoemask = midim
      midim = miexpcoemask + nsta*nmodes*maxcoe
      mitermkmask = midim
      midim = mitermkmask +  max(nadd,1)
!      midim = mitermkmask + max(nadd,pdum)
      micpcoemask = midim
      midim = micpcoemask + nsta*nsta*nmodes*maxcoe
      miswitchmask = midim
      midim = miswitchmask + nswitch*maxcoe
      micp2d = midim
      midim = micp2d + maxcoe*maxadd 

      mldim=1

      msdim=1

      mfdim=1

      if (lgenetica) then
         print*,'Assigning Memory for GA'
         mrkappa2 = mrdim
         mrdim = mrkappa2 + nmodes*nsta
         mrlambda2 = mrdim
         mrdim = mrlambda2 + nmodes*nsta*nsta
         mrgamma2 = mrdim
         mrdim = mrgamma2 + nmodes*nmodes*nsta
         mrmu2 = mrdim
         mrdim = mrmu2 + nmodes*nmodes*nsta*nsta
         mriota2 = mrdim
         mrdim = mriota2 + nmodes*nmodes*nsta*nsta
         mrtermk2 = mrdim
!         mrdim = mrtermk2 + max(nadd,pdum)
         mrdim = mrtermk2 + nadd
         mrexpcoe2 = mrdim
         mrdim = mrexpcoe2 + nsta*nmodes*maxcoe
         mrkappa3 = mrdim
         mrdim = mrkappa3 + nmodes*nsta
         mrlambda3 = mrdim
         mrdim = mrlambda3 + nmodes*nsta*nsta
         mrgamma3 = mrdim
         mrdim = mrgamma3 + nmodes*nmodes*nsta
         mrmu3 = mrdim
         mrdim = mrmu3 + nmodes*nmodes*nsta*nsta
         mriota3 = mrdim
         mrdim = mriota3 + nmodes*nmodes*nsta*nsta
         mrtermk3 = mrdim
!         mrdim = mrtermk3 + max(nadd,pdum)
         mrdim = mrtermk3 + nadd
         mrexpcoe3 = mrdim
         mrdim = mrexpcoe3 + nsta*nmodes*maxcoe
         mikappacomask = midim
         midim = mikappacomask + nmodes*nsta
         milambdacomask = midim
         midim = milambdacomask + nmodes*nsta*nsta
         migammacomask = midim
         midim = migammacomask + nmodes*nmodes*nsta
         miiotacomask = midim
         midim = miiotacomask + nmodes*nmodes*nsta*nsta
         mimucomask = midim
         midim = mimucomask + nmodes*nmodes*nsta*nsta
         miexpcoecomask = midim
         midim = miexpcoecomask + nsta*nmodes*maxcoe
         mitermkcomask = midim
!         midim = mitermkcomask + max(nadd,pdum)
         midim = mitermkcomask + nadd
      endif

      return
      end subroutine memvcham

!#######################################################################
!
! MEMLMPAR
!
! called from lmfit
!
! calculate dimensions for standard arrays.
!
!***********************************************************************

subroutine memlmpar
  
  implicit none
  
  midim=1
  mcdim=1
  mrdim=1
  
! Information read from datfile
  mrdipole = mrdim
  mrdim = mrdipole + nsets*nsta*nsta*3
  mradipole = mrdim
  mrdim = mradipole + nsets*nsta*nsta*3
  mrpolar = mrdim
  mrdim = mrpolar + nsets*nsta*nsta*3*3
  mrapolar = mrdim
  mrdim = mrapolar + nsets*nsta*nsta*3*3
  mrxcoo = mrdim
  mrdim = mrxcoo + nsets*ncoo
  mrqcoo = mrdim
  mrdim = mrqcoo + nsets*nmodes

! Dipole parameters
  mrdipzero = mrdim
  mrdim = mrdipzero + nsta*nsta*3
  mrdipder1 = mrdim
  mrdim = mrdipder1 + nsta*nsta*3*nmodes
  mrdipder2 = mrdim
  mrdim = mrdipder2 + nsta*nsta*3*nmodes*nmodes
  mrdipder3 = mrdim
  mrdim = mrdipder3 + nsta*nsta*3*nmodes
  mrdipder4 = mrdim
  mrdim = mrdipder4 + nsta*nsta*3*nmodes
  
! Polarisability parameters
  mrpolarzero = mrdim
  mrdim = mrpolarzero + nsta*nsta*3*3
  mrpolarder1 = mrdim
  mrdim = mrpolarder1 + nsta*nsta*3*3*nmodes
  mrpolarder2 = mrdim
  mrdim = mrpolarder2 + nsta*nsta*3*3*nmodes*nmodes
  mrpolarder3 = mrdim
  mrdim = mrpolarder3 + nsta*nsta*3*3*nmodes
  mrpolarder4 = mrdim
  mrdim = mrpolarder4 + nsta*nsta*3*3*nmodes

! Dipole masks
  midipzeromask = midim
  midim = midipzeromask + nsta*nsta*3
  midipder1mask = midim
  midim = midipder1mask + nsta*nsta*3*nmodes
  midipder2mask = midim
  midim = midipder2mask + nsta*nsta*3*nmodes*nmodes
  midipder3mask = midim
  midim = midipder3mask + nsta*nsta*3*nmodes
  midipder4mask = midim
  midim = midipder4mask + nsta*nsta*3*nmodes

! Polarisability masks
  mipolarzeromask = midim
  midim = mipolarzeromask + nsta*nsta*3*3
  mipolarder1mask = midim
  midim = mipolarder1mask + nsta*nsta*3*3*nmodes
  mipolarder2mask = midim
  midim = mipolarder2mask + nsta*nsta*3*3*nmodes*nmodes
  mipolarder3mask = midim
  midim = mipolarder3mask + nsta*nsta*3*3*nmodes
  mipolarder4mask = midim
  midim = mipolarder4mask + nsta*nsta*3*3*nmodes

  mldim=1

  msdim=1
  
  mfdim=1

  return
end subroutine memlmpar

!#######################################################################
      end module invchammod
