!#######################################################################

      module iofiles

use decimal, only: long,dop
use vcglobal
use global, only:pi, c5, keyword, keyorig, maxkey, lc
use maxv, only: maxdim,maxsta,maxset
use keyunitsmod

use logdat
use vchammod
use vctransmod
use vchfitmod

use utils, only: vcerrormsg,switchdat2
use op1lib,  only: zerovxi
use ioqc,    only: getintyp, getxcoo, assmass, getnm, geten, nm2xmat
use functions, only: conv_fdiab

      implicit none
      private
      public :: rdfile0,cpfreq,readguess,rdpftfiles,readlmfile,findtermkdat
      contains

!#######################################################################

      subroutine rdfile0(lrde0,abintype0,iau,lsocdegen)

      implicit none

      integer(long), intent(in)                :: iau
      integer(long)                            :: ifiletyp,m
      integer(long)                            :: ilbl,istate
      integer(long), dimension(ncoo)           :: iatiso
      integer(long), intent(in)                :: abintype0
      character(len=c5)                        :: filename
      logical(kind=4), intent(in)              :: lrde0
      real(dop)                                :: e00
      logical(kind=4), intent(in)              :: lsocdegen


      ilbl=index(file0,' ')-1
      write(6,'(a26,a)') 'Normal modes taken from : ',file0(1:ilbl)
      write(ilog,'(a26,a)') 'Normal modes taken from : ',file0(1:ilbl)

      filename=file0
      open(idat,file=filename,form='formatted',status='old',err=1030)
      call getintyp(idat,ifiletyp,filename,ilog)
      write(ilog,'(15x,a,i5)') 'Ab initio type:',abintype0

      call getxcoo(idat,xcoo0,ifiletyp,iau,file0orient,iatsym,&
                 iatiso,ncoo,nsta,nmodes,ilog)

      call assmass(iatsym,iatiso,atsym,mass,iau,ncoo,ilog)

      if (lrde0) then
         e00=0.0_dop
         istate=1
         call geten(idat,e0,e00,ifiletyp,iau,istate,abintype0,filename,&
              ilog,ncoo,nsta,nmodes,lsocdegen)    ! get zero of energy
      endif

      call getnm(idat,ifiletyp,nmlab,nmcoo,freq0,iatsym,iau,&
           maxdim,ncoo,nsta,nmodes,ilog,1,.true.)

!-----------------------------------------------------------------------
! check that all frequencies are positive. Correct if not.
!-----------------------------------------------------------------------
      do m=1,nmodes
         if (freq0(m) .lt. 0.0) then
            freq0(m)=-freq0(m)
            write(6,'(a,i5)') 'Negative frequency for mode :',m
            write(ilog,'(a,i5)') 'Negative frequency for mode :',m
            write(ilog,'(a)') 'Taking positive value.'
         endif
      enddo

      close(idat)

      return
 1030 routine='Rdfile0 (iofiles.f90)'
      ilbl=index(filename,' ')-1
      write(message,'(2a)') 'Error opening file: ',filename(1:ilbl)
      call errormsg

      end subroutine rdfile0

!#######################################################################
! take frequencies from input file rather than file0


      subroutine cpfreq(nmlab,nmlab1,freq0,freq1,nmodes1)

      implicit none

      integer(long)                                    :: m
      integer(long), intent(in)                        :: nmodes1
      real(dop), dimension(nmodes1), intent(in)        :: freq1
      real(dop), dimension(nmodes1), intent(out)       :: freq0
      character(len=*), dimension(nmodes1), intent(inout) :: nmlab,nmlab1

      do m=1,nmodes
         if (nmlab1(m) .ne. ' ') then
            freq0(m) = freq1(m)
            nmlab(m)=nmlab1(m)
         endif
      enddo

!      do m=1,nmodes
!         if (nmlab1(m) .ne. ' ') then
!            if (nmlab1(m) .eq. nmlab(m)) then
!               freq0(m) = freq1(m)
!            else if (nmlab(m) .ne. nmlab1(m)) then
!               routine='cpfreq (IOFILES)'
!               message=&
!         'Normal modes in input file do not match those in file0'
!               call vcerrormsg
!            else
!               freq0(m) = freq1(m)
!            endif
!         endif
!      enddo

      return
      end subroutine cpfreq

!######################################################################

      subroutine rdpftfiles

      implicit none

      integer(long) :: ifiletyp,i,j,k,unit,iau
      integer(long), dimension(ncoo) :: iatiso
      character(len=c5) :: filename
      
      real(dop), dimension(maxdim,maxdim) :: tmp,pa,adjpa,smat,tmp2
      real(dop) :: sum

! Removal of this print statement leads to an odd segmentation fault...
            print*,''

!---------------------------------------------------------------------
! read elements of parent hessian eigenvector matrix
!---------------------------------------------------------------------
      do j=1,maxdim
         do i=1,maxdim
            nmcoo(j,i)=0.0
            coonm(j,i)=0.0
         enddo
      enddo
      
      filename=parentfile
      unit=55
      open(unit,file=filename,form='formatted',status='old',err=1040)
      call getintyp(unit,ifiletyp,filename,ilog)

      iau=1
      call getxcoo(unit,xcoo0,ifiletyp,iau,file0orient,iatsym, &
           iatiso,ncoo,nsta,nmodes,ilog)
      call getnm(unit,ifiletyp,nmlab,nmcoo,freq0,iatsym,iau, &
           maxdim,ncoo,nsta,nmodes,ilog,1,.true.)
      call nm2xmat(nmcoo,coonm,iatsym,freq0,iau,ncoo,maxdim,nmodes,ilog)

      close(unit)

!---------------------------------------------------------------------
! read elements of fragment hessian eigenvector matrix
!---------------------------------------------------------------------
      do j=1,maxdim
         do i=1,maxdim
            fragnmcoo(j,i)=0.0
            fragcoonm(j,i)=0.0
         enddo
      enddo

      filename=fragmentfile
      unit=55
      open(unit,file=filename,form='formatted',status='old',err=1045)
      call getintyp(unit,ifiletyp,filename,ilog)

      iau=1
      call getnm(unit,ifiletyp,fragnmlab,fragnmcoo,fragfreq0,iatsym,iau, &
           maxdim,fragncoo,nsta,fragnmodes,ilog,1,.true.)
      call nm2xmat(fragnmcoo,fragcoonm,iatsym,fragfreq0,iau,fragncoo,maxdim, &
           fragnmodes,ilog)

      close(unit)

!---------------------------------------------------------------------
! calculate elements of parent-to-fragment transformation matrix
!---------------------------------------------------------------------
      do j=1,maxdim
         do i=1,maxdim
            matpft(i,j)=0.0
            adjpft(i,j)=0.0
         enddo
      enddo

      do i=1,fragnmodes
         do j=1,nmodes
            matpft(i,j)=0.0
            do k=1,ncoo
               matpft(i,j)=matpft(i,j)+(fragcoonm(i,k)*(nmcoo(k,j)))
            enddo
         enddo
      enddo

!---------------------------------------------------------------------
! calculate adjoint of parent-to-fragment transformation matrix
!---------------------------------------------------------------------
      do j=1,fragnmodes
         do i=1,nmodes
            adjpft(i,j)=matpft(j,i)/(fragfreq0(j)/freq0(i))
         enddo
      enddo


!---------------------------------------------------------------------
! calculate overlap of parent and fragment normal modes
!---------------------------------------------------------------------
      do j=1,maxdim
         do i=1,maxdim
            smat(i,j)=0.0
         enddo
      enddo


      do j=1,fragnmodes
         do i=1,nmodes
            do k=1,ncoo-3
               smat(i,j)=smat(i,j)+coonm(i,k)*fragnmcoo(k,j)
            enddo
         enddo
      enddo

      do j=1,fragnmodes
         sum=0
         do i=1,nmodes
            sum=sum+smat(i,j)**2
         enddo
      enddo

      
!---------------------------------------------------------------------
! calculate q to r transformation matrix
!---------------------------------------------------------------------
      do j=1,maxdim
         do i=1,maxdim
            tmp(i,j)=0.0
            tmp2(i,j)=0.0
            pa(i,j)=0.0
            adjpa(i,j)=0.0
            adjpa(i,j)=0.0
            matqrt(i,j)=0.0
            adjqrt(i,j)=0.0
         enddo
      enddo
      
!     tmp: P
         do i=nmodes-2,nmodes
            j=ncoo-(nmodes-i)
            tmp(i,j)=1.0
         enddo
         
!     tmp2: adjoint of P           
      do j=1,nmodes
         do i=1,ncoo
            tmp2(i,j)=tmp(j,i)
         enddo
      enddo

!---------------------------------------------------------------------
! calculate q to r matrix
!---------------------------------------------------------------------
       do j=1,nmodes
         do i=1,nmodes
            matqrt(i,j)=0.0
            do k=1,ncoo
               matqrt(i,j)=matqrt(i,j)+tmp(i,k)*nmcoo(k,j)
            enddo
         enddo
      enddo

!---------------------------------------------------------------------
! calculate adjoint of q to r matrix
!---------------------------------------------------------------------
      do j=1,nmodes
         do i=1,nmodes
            adjqrt(i,j)=0.0
            do k=1,ncoo
               adjqrt(i,j)=adjqrt(i,j)+coonm(i,k)*tmp2(k,j)
            enddo
         enddo
      enddo

      goto 1050
      
 1040 write(6,'(a)') "Parent file0 not found"
      stop
 1045 write(6,'(a)') "Fragment file0 not found"
      stop

 1050 continue
      
      return
      end subroutine rdpftfiles

!######################################################################

      subroutine findtermkdat(unit)

      implicit none

      integer(long), external                                    :: slen
      integer(long), intent(in)                                  :: unit
      integer(long)      :: ierr,j,n,ilbl,jlbl,m,p,iz,ic
      character(len=80)  :: string

      rewind(unit)
   15   read(unit,'(a)',end=100) string
        if (string .eq.&
         '*** Explicitely added polynomial terms ***') then
         read(unit,*) nadd
         if (nadd .gt. maxadd) then
            routine='findtermkdat (IOFILES)'
               write(message,'(2a,i5)') 'Number of added parameters ', &
           'exceeds maximum. Increase MAXADD in vchammod.f90 to at least:',&
                 nadd
            call vcerrormsg
         endif
         do n=1,nadd
            call rdinpf(unit,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                 maxkey)

!            read(keyword(1),*) termk(n)
!            print*, 'DEBUG CHRIS: +TERMk, n =', n, ' = ', termk(n)
!            print*, 'with jkeyword1: ', keyword(1)
            read(keyword(2),*) termst(1,n)
            read(keyword(3),*) termst(2,n)
            call zerovxi(termdef(1,n),maxdim)
            do j=4,ic
               ilbl=index(keyword(j),'^')
               jlbl=slen(keyword(j))
               read(keyword(j)(1:ilbl-1),*) m
               read(keyword(j)(ilbl+1:jlbl),*) p
               termdef(m,n) = p
            enddo
         enddo
       endif
       goto 15
100    continue
       return
       end subroutine findtermkdat

!######################################################################

      subroutine readguess(unit,const,kappa,lambda,gamma,iota,mu,&
           expcoe,expcoe_mask,higher,termk,cpcoe,switch,cp2d)


      implicit none
      
      integer(long), external                                    :: slen
      integer(long) :: modes,ierr,i,j,j1,j2,j3,n,s1,s,nmblck,k,ioff,&
                       ilbl,jlbl,nhi,l,m,p,iz,ic,itmp,dum1,dum2,nadd1
      integer(long), intent(in)                                  :: unit
      integer(long), dimension(maxdim,maxsta) :: ifdiab1,ncoeff1
      integer(long), dimension(maxcoe,nmodes,nsta), intent(out)  :: expcoe_mask
      real(dop), dimension(nsta,nsta), intent(out)               :: const
      real(dop), dimension(nmodes,maxsta), intent(out)           :: kappa
      real(dop), dimension(nmodes,maxsta,maxsta), intent(out)    :: lambda
      real(dop), dimension(nmodes,nmodes,maxsta), intent(out)    :: gamma
      real(dop), dimension(maxcoe,nmodes,maxsta), intent(out)    :: expcoe
      real(dop), dimension(nmodes,nmodes,maxsta,maxsta), intent(out) :: iota,mu
      real(dop), dimension(maxcoe,nmodes,maxsta,maxsta), intent(out) :: cpcoe
      real(dop), dimension(nhigher), intent(out)                 :: higher
      real(dop), dimension(maxcoe,maxadd), intent(inout)         :: cp2d
      real(dop)                                                  :: k1
      integer(long)  ,dimension(maxdim)                          :: old_adof
      logical(kind=4),dimension(maxdim)                          :: old_ignoredof
      integer(long)                                              :: old_i_ignore
      character(len=80)                       :: string,cdum,cdum2,string2
      real(dop)                               :: filever,rtmp,dum3
      real(dop), dimension(maxadd), intent(out)                  :: termk
      real(dop), dimension(nswitch,maxcoe), intent(out)          :: switch
      integer(long), dimension(maxdim) :: defchk,chk
      logical(kind=4) :: ladd,lerr

      old_i_ignore = 0
      do i = 1, nmodes
         old_adof(i) = 0
      enddo
      
      if( i_ignore .ne. 0) then
         old_i_ignore = i_ignore
         old_ignoredof = ignoredof
         old_adof(1:nmodes) = adof(1:nmodes)
      else
         do i = 1, nmodes
            adof(i) = i
            ignoredof(i) = .false.
         enddo
      endif
      i_ignore = 0

      rewind(unit)

      i_ignore = 0
!	note: adof is just an array which passes throught the modes that
!	we want to include, in case there are any that the info file has told us 
!       to ignore some (in the Omitted DOF section)

! counter for higher order terms
      nhi=0

      if (lgenetica) then
         ilbl = 13
         goto 5
      else
         ilbl = index(guessfile,' ')-1
      endif
      write(ilog,'(/,2a)') '*** Guess from file : ',guessfile(1:ilbl)
!
! set guess diabatic potential to HO
!
5     do i=1,nsta
          do j=1,nmodes
             ifdiab1(j,i)=0
             ncoeff1(j,i)=0
          enddo
       enddo

!
! read file version no.
!
       read(unit,'(17x,f8.5)') filever
       if (lgenetica) then
          goto 15
       endif
       write(ilog,'(a,f8.3)') 'File version: ',filever
10    continue
15    read(unit,'(a)',end=100,err=1010) string
!
! read general diabatic potentials
!
      modes=nmodes
      if(string .eq. '*** Omitted DOFs ***' ) then
      
33       read(unit,'(a)',end=100,err=1010) string
         if(len(trim(string)) .gt. 0) then
            read(string,'(I3)',end=100,err=1010) j
            i_ignore = i_ignore + 1
            ignoredof(j) = .true.
            goto 33
         endif
         i = 0
         do j = 1, nmodes
            if( .not. ignoredof(j)) then
               i = i + 1
               adof(i) = j
            endif
         enddo

         nmodes = nmodes - i_ignore

      else if(string .eq. '*** Mass-scaled Only ***' ) then
         lalleig = .true.

      else if(string .eq. '*** Transformed Coordinates ***' ) then
         lqtransform = .true.
         do 
            read(unit,'(a)',end=100,err=1010) string
            if (string(1:10) .eq. '          ') then
               exit
            else
               read(string(1:3),'(i3)') m
               if (string(6:15) .eq. '       cos') qtransform(m)=1
               if (string(6:15) .eq. '       sin') qtransform(m)=2
               if (string(6:15) .eq. '   halfcos') qtransform(m)=3
               if (string(6:15) .eq. '   halfsin') qtransform(m)=4
            endif
         enddo

      else if (string .eq. '*** Diabatic Potential Form ***') then
20       continue
      read(unit,'(a)',end=100,err=1010) string
      if (string .ne. '  ') then
         read(string,'(i3,2x,i3,7x,2i5)') &
              j,i,ifdiab1(j,i),ncoeff1(j,i)
         go to 20
      endif
30    continue
      read(unit,'(a)',end=100,err=1010) string
      if (string .eq. '*** Expansion Coefficients ***') then
         do i=1,nsta
            do j=1,modes
               n=ncoeff1(adof(j),i)
               if (n .gt. 0) then
                  read(unit,*) 
                  read(unit,*,err=1020) (expcoe(k,adof(j),i),k=1,n)
               endif
            enddo
         enddo
      else
         go to 30
      endif

      else if (string.eq.'*** Nonadiabatic Coupling Form ***') then
         do j=1,nsta-1
            do i=j+1,nsta
               do m=1,nmodes
                  ncoeffcp(m,i,j)=0
                  ncoeffcp(m,j,i)=ncoeffcp(m,i,j)
               enddo
            enddo
         enddo
35       continue
         read(unit,'(a)',end=100,err=1010) string
         if (string .ne. '  ') then
            read(string,'(i3,2x,i3,2x,i3,7x,2i5)') &
                 m,j,i,ifdiabcp(m,i,j),ncoeffcp(m,i,j)
            ifdiabcp(m,j,i)=ifdiabcp(m,i,j)
            ncoeffcp(m,j,i)=ncoeffcp(m,i,j)
            go to 35
         endif
37       continue      
         read(unit,'(a)',end=100,err=1010) string
         if (string.eq.'*** Nonadiabatic Coupling Parameters ***') then
            do j=1,nsta-1
               do i=j+1,nsta
                  do m=1,nmodes
                     n=ncoeffcp(adof(m),i,j)
                     if (n.gt.0) then
                        read(unit,*)
                        read(unit,*) (cpcoe(k,adof(m),i,j),k=1,n)
                        do k=1,n
                           cpcoe(k,adof(m),j,i)=cpcoe(k,adof(m),i,j)
                        enddo
                     endif
                  enddo
               enddo
            enddo
         else
            goto 37
         endif

      else if (string .eq. '*** Frequencies ***') then
         do m=1,nmodes
            read(unit,'(i3,2x,a5,f10.4)') i,nmlab(m),dum3
            if(abs(dum3 - freq0(m)) .gt. 0.001) &
               print*, 'CAUTION: mismatch between frequecies ', & 
                       'in info and vcham files (used vcham) mode=',adof(m)
            if(abs(dum3 - freq0(m)) .gt. 0.001) freq0(m) = dum3
         enddo
      else if (string .eq. '*** Switching Functions ***') then
         read(unit,'(i5)') nswitch
         lswitch=.true.
         do i=1,nswitch
            read(unit,'(x,a)') aswitch(i)
            call switchdat2(aswitch(i),swmd(i),swm(i),&
                 swm1(i),sws(i),sws1(i),swk(i),&
                 swpar(i),swfunc(i),nswitchcoe(i))
            do j=1,nswitchcoe(i)
               read(unit,'(3x,e13.5)') switch(i,j)
            enddo
         enddo
         
      else if (string .eq. '*** Constants ***') then
         if (.not. lgenetica) then
            write(ilog,'(a)') 'Reading constants'
         endif
         read(unit,*) 
         do s1=1,nsta
            read(unit,'(8x,8f10.4)') (const(s1,s),s=1,s1)
            do s=1,s1-1
               const(s,s1)=const(s1,s)
            enddo
         enddo
         lconstall = .true.

      else if (string .eq. '*** On-diagonal constants ***') then
         if (.not. lgenetica) then
            write(ilog,'(a)') 'Reading constants'
         endif
         read(unit,*) 
         do s1=1,nsta
            read(unit,'(8x,f10.4)') const(s1,s1)
         enddo

!    OLD FORMAT, KEPT SO AS TO BE ABLE TO READ OLD VCHAM FILES
      else if (string .eq. &
           '*** On-diagonal linear (kappa) coupling constants ***') then
         if (.not. lgenetica) then
            write(ilog,'(a)') 'Reading kappa '
         endif
         read(unit,*) 
         do i=1,modes
            read(unit,'(10x,8f10.4)') (kappa(adof(i),s),s=1,nsta)
         enddo
!   NEW FORMAT
      else if (string .eq. &
           '*** On-diagonal linear coupling constants (kappa) ***') then     
         if (.not. lgenetica) write(ilog,'(a)') 'Reading kappa '
         rtmp=(nsta/5.0)
         itmp=int(rtmp)
         nmblck=itmp
         if ((rtmp-itmp).gt.0.0) nmblck=nmblck+1
         do n=1,nmblck
            j=(n-1)*5+1
            k=0
            if (n.eq.nmblck) then            
               k=j+4-nsta
            endif
            read(unit,*)
            do i=1,nmodes
               read(unit,'(10x,5f10.4)') (kappa(adof(i),s),s=j,j+4-k)
            enddo
            read(unit,*)
         enddo
      elseif( index(string,'Fourier Lambda Used,sin') .ne. 0  ) then
         lsincoup = .true.
      elseif( index(string,'Fourier Lambda Used,cos') .ne. 0  ) then
         lcoscoup = .true.

      else if (string .eq. &
           '*** Off-diagonal linear (lambda) coupling constants ***') then
         if (.not. lgenetica) then
         endif
         read(unit,*) 
         do s=1,nsta-1
            do s1=s+1,nsta
               do i=1,modes
                  read(unit,'(10x,f10.4)') lambda(adof(i),s,s1)
                  lambda(adof(i),s1,s) = lambda(adof(i),s,s1)
               enddo
               read(unit,'(a)') string
            enddo
         enddo
      else if (string .eq.&
           '*** All on-diagonal bilinear (gamma) constants ***') then
         if (hamord.lt.2) hamord=2
         if (.not. lgenetica) then
            write(ilog,'(a)') 'Reading gamma '
         endif
         read(unit,*)
         read(unit,*)
         nmblck = ((modes-1)/6)+1
         do s=1,nsta
            do n=1,nmblck
               j1=(n-1)*6+1
               j2=min(j1+5,modes)
               do i=j1,modes
                  j3=min(i,j2)
                  read(unit,'(10x,6f10.4)') & 
                       (gamma(adof(i),adof(j),s),j=j1,j3)
!                  print*, 'i =', i, 'j = ', j
!                  print*, 'adof(i) = ', adof(i), adof(j)
!                  print*, 'gamma = ', gamma(adof(i),adof(j),s)
                  do j=j1,j3
                     gamma(adof(j),adof(i),s)  = &
                          gamma(adof(i),adof(j),s)
                  enddo
               enddo
               read(unit,*)
            enddo
            do i=1,2
               read(unit,*)
            enddo
         enddo
         backspace(unit)
         backspace(unit)
!-------------------------------------------------------------------------------
! Included so as to be able to read the (incorrect) output of previous versions
!-------------------------------------------------------------------------------
      else if ((string .eq.&
           '*** Third order terms (iota) ***') .or. (string .eq.&
           '*** On-diagonal quadratic-linear (iota) constants ***')) then
         if (hamord.lt.3) hamord=3
         if (.not. lgenetica) then
            write(ilog,'(a)') 'Reading iota '
         endif
         read(unit,*)
         read(unit,*)
         
         nmblck = ((modes-1)/6)+1
         do s=1,nsta
            do n=1,nmblck
               j1=(n-1)*6+1
               j2=min(j1+5,modes)
               do i=1,modes
                  j3=j2
                  read(unit,'(10x,6f10.4)') &
                       (iota(adof(i),adof(j),s,s),j=j1,j3)                  
               enddo
               read(unit,'(a)') string
            enddo
            do i=1,2
               read(unit,'(a)') string
            enddo
         enddo
         backspace(unit)
         backspace(unit)
!-----------------------------------------------------------------------

      else if (string .eq. &
           '*** On-diagonal quad-lin (iota) constants ***') then
         if (hamord.lt.3) hamord=3
         if (.not. lgenetica) then
            write(ilog,'(a)') 'Reading iota '
         endif
         read(unit,*)
         read(unit,*)
         nmblck = ((modes-1)/3)+1
         do s=1,nsta
            do n=1,nmblck
               j1=(n-1)*3+1
               j2=min(j1+2,modes)
               do i=1,modes
                  read(unit,'(10x,'//charit(j2-j1+1,0)//'f14.7)') &
                       (iota(adof(i),adof(j),s,s),j=j1,j2)
               enddo
               read(unit,'(a)') string
            enddo
            do i=1,2
               read(unit,'(a)') string
            enddo
         enddo
         backspace(unit)
         backspace(unit)
         
      else if (string .eq.&
           '*** Off-diagonal bilinear (mu) constants ***') then
         if (hamord.lt.2) hamord=2
         if (.not. lgenetica) then
            write(ilog,'(a)') 'Reading mu '
         endif
         do i=1,2
            read(unit,'(a)') string
         enddo
         nmblck=((modes-1)/6)+1
         do s=1,nsta-1
            do s1=s+1,nsta
               do n=1,nmblck
                  j1=(n-1)*6+1
                  j2=min(j1+5,modes)
                  do i=j1,modes
                     j3=min(i,j2)
                     read(unit,'(10x,6f10.4)') &
                          (mu(adof(i),adof(j),s,s1),j=j1,j3)
                     do j=j1,j3 
                        mu(adof(j),adof(i),s,s1) = mu(adof(i),adof(j),s,s1)
                        mu(adof(i),adof(j),s1,s) = mu(adof(i),adof(j),s,s1)
                        mu(adof(j),adof(i),s1,s) = mu(adof(j),adof(i),s,s1)
                     enddo
                  enddo
                  read(unit,'(a)') string
               enddo
               read(unit,'(a)') string
               read(unit,'(a)') string
            enddo
         enddo
         backspace(unit)
         backspace(unit)
      else if (string .eq. &
           '*** Off-diagonal quadratic-linear (iota2) constants ***') &
           then 
         if (hamord.lt.3) hamord=3
         if (.not. lgenetica) then
            write(ilog,'(a)') 'Reading iota2 '
         endif
         do i=1,2
            read(unit,'(a)') string
         enddo
         nmblck=((modes-1)/6)+1
         do s=1,nsta-1
            do s1=s+1,nsta
               do n=1,nmblck
                  j1=(n-1)*6+1
                  j2=min(j1+5,modes)
                  do i=j1,modes
                     j3=j2
                     if( j1 .eq. j3) then
                        j = j1
                        read(unit,'(10x,'//charit(j2-j1+1,0)//'f10.4)') &
                             iota(adof(i),adof(j1),s,s1)
                     else
                        read(unit,'(10x,'//charit(j2-j1+1,0)//'f10.4)') &
                             (iota(adof(i),adof(j),s,s1),j=j1,j3)
                        iota(adof(i),adof(j),s1,s) = iota(adof(i),adof(j),s,s1)
                     endif
                  enddo
                  read(unit,'(a)') string
               enddo
               read(unit,'(a)') string
               read(unit,'(a)') string
            enddo
         enddo
         backspace(unit)
         backspace(unit)
     
      else if (string .eq.&
           '*** On-diagonal lin-lin-lin third order constants ***') &
           then
         do s = 1,nsta
            read(unit,*)
            do i=1,modes
               do j = i+1,modes
                  do k = j+1,modes
                     nhi = nhi+1
                     read(unit,'(15x,f10.4)') higher(nhi)
                  enddo
               enddo
            enddo
            read(unit,*)
         enddo
         
! gam_iijj q_i^2 q_j^2 terms
      else if (string .eq.&
            '*** On-diagonal quad-quad fourth order constants ***')&
                 then
         do s=1,nsta
            read(unit,*)
            do i=1,modes-1
               do j = i+1,modes
                   nhi = nhi+1
                   read(unit,'(20x,f10.4)') higher(nhi)
               enddo
            enddo
            read(unit,*)
         enddo

! gam_iiij q_i^3 q_j terms
      else if (string .eq.&
            '*** On-diagonal cube-lin fourth order constants ***')&
                 then
         do s=1,nsta
            read(unit,*)
            do i=1,modes-1
               do j = i+1,modes
                  nhi = nhi+1
                  read(unit,'(20x,f10.4)') higher(nhi)
               enddo
            enddo
            do i=2,modes
               do j = 1,i-1
                  nhi = nhi+1
                  read(unit,'(20x,f10.4)') higher(nhi)
               enddo
            enddo
            read(unit,*)
         enddo

! gam_iijk q_i^2 q_j q_k terms
      else if (string .eq.&
            '*** On-diagonal quad-lin-lin fourth order constants ***')&
                 then
         do s=1,nsta
            read(unit,*)
            do i=1,modes-2
               do j = i+1,modes-1
                  do k = j+1,modes
                     nhi = nhi+1
                     read(unit,'(20x,f10.4)') higher(nhi)
                  enddo
               enddo
            enddo
            do i=3,modes
               do j = 1,i-2
                  do k = j+1,i-1
                     nhi = nhi+1
                     read(unit,'(20x,f10.4)') higher(nhi)
                  enddo
               enddo
            enddo
            do i=2,modes-1
               do j = 1,i-1
                  do k = i+1,modes
                     nhi = nhi+1
                     read(unit,'(20x,f10.4)') higher(nhi)
                  enddo
               enddo
            enddo
            read(unit,*)
         enddo

! gam_ijkl q_i q_j q_k q_l terms
      else if (string .eq.&
         '*** On-diagonal lin-lin-lin-lin fourth order constants ***')&
                then
         do s=1,nsta
            read(unit,*)
            do i=1,modes-3
               do j = i+1,modes-2
                  do k = j+1,modes-1
                     do l = k+1,modes
                        nhi = nhi+1
                        read(unit,'(20x,f10.4)') higher(nhi)
                     enddo
                  enddo
               enddo
            enddo
            read(unit,*)
         enddo

!       THIS IS TO READ THE OLD VERSION WRITTEN OUT IN STANDARD NOTATION 
      else if (string .eq.&
           '*** Explicitely added polynomial terms ***') then
         if(.not. lgenetica) write(ilog,'(a)') 'Reading explicit polynomial terms '
         call rdinpf(unit,keyword,keyorig,inptit,lc,ic,iz,ierr,&
              maxkey)
         if (keyword(1) .eq. "n") then
            ioff=1
            read(keyword(2),*) nadd1
         else
            ioff=0
            read(keyword(1),*) nadd1
         endif
         if (nadd+nadd1 .gt. maxadd) then
            routine='readguess (IOFILES)'
            write(message,'(2a,i5)') 'Number of added parameters ', &
                 'exceeds maximum. Increase MAXADD in vchammod.f90 to at least:',&
                 nadd
            call vcerrormsg
         endif
! check if parameters are in list read from input file, elsde add them
         do n=1,nadd1
            call rdinpf(unit,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                 maxkey)
            read(keyword(ioff+1),*) k1
            read(keyword(ioff+2),*) s
            read(keyword(ioff+3),*) s1
            defchk = 0
            do j=ioff+4,ic
               ilbl=index(keyword(j),'^')
               jlbl=slen(keyword(j))
               read(keyword(j)(1:ilbl-1),*) m
               read(keyword(j)(ilbl+1:jlbl),*) p
               defchk(m) = p
            enddo

            ladd = .true.
            do i=1,nadd
               do j=1,nmodes
                  chk(j)=1
               enddo
               if (s.eq.termst(1,i).and.s1.eq.termst(2,i).or.   &
                    (s.eq.termst(2,i)).and.s1.eq.termst(1,i)) then
                  do j=1,nmodes
                     chk(j)=0
                  enddo
                  do j=1,nmodes
                     chk(j)=defchk(j)-termdef(j,i)
                     if (chk(j).ne.0) go to 300
                  enddo

                  ladd = .false.
                  termk(i) = k1
               endif
  300          continue
            enddo
            if (ladd) then
               nadd = nadd+1
               termk(nadd) = k1
               termst(1,nadd) = s
               termst(2,nadd) = s1
               do m=1,nmodes
                  termdef(m,nadd) = defchk(m)
               enddo
            endif

         enddo
         
      else if (string .eq. '***2D Coupling Parameters***') then
         read(unit,'(A80)') string
         string = trim(string)
         s = 1
         do while ( len(trim(adjustl(string))) .ne. 0  )
            if( index(string,'tanh2d') .ne. 0  ) then
               !           if( string(1:6) .ne. 'radtan'  ) then
               read(string,'(A12,5I2,A3,E13.5)') cdum, cp2dtyp(s), dofcp2d(1,s),dofcp2d(2,s),&
                    stacp2d(1,s),stacp2d(2,s), cdum2, cp2d(1,s)
               s = s + 1
            elseif( index(string,'tanch') .ne. 0  ) then
               string2 = '(A8,2X,5I2,A3,F8.4,X,F8.4,X,F8.4,X,F8.4)'
               read(string,trim(string2)) cdum, cp2dtyp(s),dofcp2d(1,s),dofcp2d(2,s),&
                    stacp2d(1,s),stacp2d(2,s),cdum2, cp2d(1,s), cp2d(2,s), cp2d(3,s), cp2d(4,s)
               !            print*, 'picked =',  cp2d(1,s), cp2d(2,s), cp2d(3,s), cp2d(4,s), 'typ = ', cp2dtyp(s), 's = ', s
               s = s + 1
            elseif( index(string,'polybox2') .ne. 0  ) then
               string2 = '(A8,2X,6I2,A3,F8.4,X,F8.4,X,F8.4)'
               read(string,trim(string2)) cdum, cp2dtyp(s),dofcp2d(1,s),dofcp2d(2,s),&
                    stacp2d(1,s),stacp2d(2,s),ocp2d(s),cdum2, cp2d(1,s), cp2d(2,s), cp2d(3,s)
               s = s + 1
            elseif( index(string,'polybox') .ne. 0  ) then
               string2 = '(A8,2X,5I2,A3,F8.4,X,F8.4,X,F8.4,X,F8.4,F8.4)'
               read(string,trim(string2)) cdum, cp2dtyp(s),dofcp2d(1,s),dofcp2d(2,s),&
                    stacp2d(1,s),stacp2d(2,s),cdum2, cp2d(1,s), cp2d(2,s), cp2d(3,s), cp2d(4,s), cp2d(5,s)
               !            print*, 'picked =',  cp2d(1,s), cp2d(2,s), cp2d(3,s), cp2d(4,s), 'typ = ', cp2dtyp(s), 's = ', s
               s = s + 1
            endif
            read(unit,'(A80)') string
            string = trim(string)
         enddo
         backspace(unit)
         backspace(unit)

!----------------------------------------------------------------------
! Zeroth-order bilinear contribution due to the rotation of a pair of
! normal modes (old version)
!----------------------------------------------------------------------
      else if (string.eq.'*** Zeroth-order bilinear contribution ***') then
         read(unit,'(i3,x,i3,F20.10)') rotm(1,1),rotm(1,2),rotmxfq(1)

!----------------------------------------------------------------------
! Zeroth-order bilinear contribution due to the rotation of a pair of
! normal modes 
!----------------------------------------------------------------------
      else if (string.eq.'*** Zeroth-order bilinear contributions ***') then
         j = 1
         read(unit,'(a)') string
         do while (len(trim(string)) .gt. 0) 
            if( rotm(j,1) .eq. 0.0d0)  then
               read(string,'(i3,x,i3,F15.10)')&
                    rotm(j,1),rotm(j,2),rotmxfq(j)
            else
               read(string,'(i3,x,i3,F15.10)')&
                    dum1,dum2,dum3
               !       THE INFO FILE MIGHT ALREADY CONTAIN INFORMATION ABOUT THE ROTATED
               !       MODES, IN WHICH CASE, THE VCHAM FILE TAKES PRECEDENCE  
               if( (rotm(j,1) .eq. dum1 .or. rotm(j,1) .eq. dum2) .and. &
                    (rotm(j,2) .eq. dum1 .or. rotm(j,2) .eq. dum2)) then
                  rotmxfq(j) = dum3 
                  j = j + 1
               else
                  j = j + 1
                  CYCLE
               endif
            endif
            read(unit,'(a)') string
            !            j = j + 1
         enddo
      endif
      
      go to 10
100   continue

      if (.not. lgenetica) then
         write(ilog,'(a)') 'END OF GUESS FILE'
      endif
!
! ifdiab1 is read from guess
! 1. ifdiab=0 set in .inp (HO), ifdiab1=0 in guess. Do Nothing
! 2. ifdiab=0 set in .inp, ifdiab1>0 in guess, take guess potential
! 3. ifdiab>0 set in .inp, ifdiab1>0 in guess try to convert
!
      lerr = .false.
      do i=1,nsta
         do j=1,modes
            if( .not. ignoredof(adof(j))) then
               if (ifdiab1(adof(j),i) .gt. 0 .and. ifdiab(adof(j),i) .eq. 0) then
                  ifdiab(adof(j),i)=ifdiab1(adof(j),i)
                  ncoeff(adof(j),i)=ncoeff1(adof(j),i)
                  lgeneral = .true.
               else if (ifdiab(adof(j),i) .gt. 0 .and. &
                    ifdiab(adof(j),i) .ne. ifdiab1(adof(j),i)) then
                  if (.not. lgenetica) then
                     write(ilog,'(a)') 'Converting guess PES'
                  endif
                  call conv_fdiab(ifdiab(adof(j),i),freq0(adof(j)),kappa(adof(j),i),&
                       gamma(adof(j),adof(j),i),expcoe(1,adof(j),i),ncoeff(adof(j),i),&
                       expcoe_mask(1,adof(j),i),ifdiab1(adof(j),i),ierr)
                  if (ierr .gt. 0) then
                     print*,'ERROR :Impossible Conversion. Mode , state :',j,i
                     print*,'       New potential :',ifdiab(adof(j),i), &
                            '       Old potential :',ifdiab1(adof(j),i) 
                     lerr = .true.
                  endif
                  lgeneral = .true.
               endif
            endif
         enddo
      enddo
      if (lerr) stop
      
      if(i_ignore .gt. 0 ) nmodes = nmodes + i_ignore

      if( old_adof(1) .ne. 0  ) then
         i_ignore = old_i_ignore
         ignoredof = old_ignoredof
         adof = old_adof
      else
         i_ignore = 0
         do i = 1, nmodes
            adof(i) = i
            ignoredof(i) = .false.
         enddo
      endif

      return
 1010 stop 'ERROR IN GUESS FILE'
 1020 write(6,'(a)') 'ERROR reading diabatic PES parameters.'
      stop
      
      end subroutine readguess

!#######################################################################

      character(len=4) function charit(num,zeros)

        integer, intent(in) :: num, zeros

        if( zeros .eq. 0) then
           if( num .lt. 10) write(charit,'(I1)')  num
           if( num .lt. 100) write(charit,'(I2)') num
           if( num .lt. 1000) write(charit,'(I3)') num
           if( num .ge. 1000) write(charit,'(I4)') num
        elseif( zeros .eq. 1) then
           if( num .lt. 10) write(charit,'(A1,I1)') '0', num
           if((num .ge. 10) .and. (num .lt. 100)) write(charit,'(I2)') num
           if((num .ge. 100) .and. (num .lt. 1000)) write(charit,'(I3)') num
           if( num .ge. 1000) write(charit,'(I4)') num
        elseif( zeros .eq. 2) then
           if( num .lt. 10) write(charit,'(A2,I1)') '00', num
           if((num .ge. 10) .and. (num .lt. 100)) write(charit,'(A1,I2)') '0',num
           if((num .ge. 100) .and. (num .lt. 1000)) write(charit,'(I3)')  num
           if( num .ge. 1000) write(charit,'(I4)') num
        elseif( zeros .eq. 3) then
           if( num .lt. 10) write(charit,'(A3,I1)') '000', num
           if((num .ge. 10) .and. (num .lt. 100)) write(charit,'(A2,I2)') '00',num
           if((num .ge. 100) .and. (num .lt. 1000)) write(charit,'(A1,I3)') '0',num
           if( num .ge. 1000) write(charit,'(I4)') num
        endif
        return
      end function charit

!######################################################################

      subroutine readlmfile(unit,dipzero,dipder1,dipder2,dipder3,dipder4,&
           polarzero,polarder1,polarder2,polarder3,polarder4)
        
        implicit none
        
        integer(long) :: unit,k,s,s1,m,n,nblck,i,j,j1,j2,j3,l

        real(dop), dimension(nsta,nsta,3)               :: dipzero
        real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder1
        real(dop), dimension(nsta,nsta,3,nmodes,nmodes) :: dipder2
        real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder3
        real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder4

        real(dop), dimension(nsta,nsta,3,3)               :: polarzero
        real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder1
        real(dop), dimension(nsta,nsta,3,3,nmodes,nmodes) :: polarder2
        real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder3
        real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder4

        real(dop)               :: filever
        character(len=80)       :: string

        read(unit,'(17x,f8.5)') filever

        write(ilog,'(a,f8.3)') 'File version: ',filever
10      continue
        read(unit,'(a)',end=100,err=1010) string
        
        if (string(1:23).eq.'*** Zeroth-order dipole') then
           read(unit,*)
           read(unit,*)
           do s=1,nsta
              do s1=s,nsta
                 read(unit,'(9x,3(F11.8,3x))') (dipzero(s,s1,j), j=1,3)
              enddo
           enddo
           
        else if (string(1:22).eq.'*** First-order dipole') then
           do s=1,nsta
              do s1=s,nsta
                 read(unit,*)
                 read(unit,*)
                 do m=1,nmodes
                    read(unit,'(19x,3(F12.8,3x))') &
                         (dipder1(s,s1,j,m), j=1,3)
                 enddo
              enddo
           enddo
           
        else if (string(1:23).eq.'*** Second-order dipole') then
           nblck=((nmodes-1)/6)+1
           do s=1,nsta
              do s1=s,nsta
                 do k=1,3
                    read(unit,*)
                    read(unit,*)
                    do n=1,nblck
                       j1=(n-1)*6+1
                       j2=min(j1+5,nmodes)
                       read(unit,*)
                       do i=j1,nmodes
                          j3=min(i,j2)
                          read(unit,20) (dipder2(s,s1,k,i,j),j=j1,j3)
                       enddo
                    enddo
                    read(unit,*)
                 enddo
              enddo
           enddo

        else if (string(1:22).eq.'*** Third-order dipole') then
           do s=1,nsta
              do s1=s,nsta
                 read(unit,*)
                 read(unit,*)
                 do m=1,nmodes
                    read(unit,'(18x,3(F12.8,3x))') &
                         (dipder3(s,s1,j,m), j=1,3)
                 enddo
              enddo
           enddo

        else if (string(1:23).eq.'*** Fourth-order dipole') then
           do s=1,nsta
              do s1=s,nsta
                 read(unit,*)
                 read(unit,*)
                 do m=1,nmodes
                    read(unit,'(18x,3(F12.8,3x))') &
                         (dipder4(s,s1,j,m), j=1,3)
                 enddo
              enddo
           enddo


        else if (string(1:22).eq.'*** Zeroth-order polar') then
              read(unit,*)
              read(unit,*)
              do s=1,nsta
                 do s1=s,nsta
                    read(unit,'(9x,3(F14.8,3x))') polarzero(s,s1,1,1),polarzero(s,s1,1,2),polarzero(s,s1,1,3)
                    read(unit,'(9x,3(F14.8,3x))') polarzero(s,s1,2,2),polarzero(s,s1,2,3),polarzero(s,s1,3,3)
                 enddo
              enddo

           else if (string(1:21).eq.'*** First-order polar') then
              do s=1,nsta
                 do s1=s,nsta
                    read(unit,*)
                    read(unit,*)
                    do m=1,nmodes
                       read(unit,'(9x,3(F11.6,3x))') polarder1(s,s1,1,1,m),polarder1(s,s1,1,2,m),polarder1(s,s1,1,3,m)
                       read(unit,'(9x,3(F11.6,3x))') polarder1(s,s1,2,2,m),polarder1(s,s1,2,3,m),polarder1(s,s1,3,3,m)
                    enddo
                 enddo
              enddo
              
           else if (string(1:22).eq.'*** Second-order polar') then
              nblck=((nmodes-1)/6)+1
              do s=1,nsta
                 do s1=s,nsta
                    do k=1,3
                       do l=1,3
                          read(unit,*)
                          read(unit,*)
                          do n=1,nblck
                             j1=(n-1)*6+1
                             j2=min(j1+5,nmodes)
                             read(unit,*)
                             do i=j1,nmodes
                                j3=min(i,j2)
                                read(unit,20) (polarder2(s,s1,k,l,i,j),j=j1,j3)
                             enddo
                          enddo
                          read(unit,*)
                       enddo
                    enddo
                 enddo
              enddo

           else if (string(1:21).eq.'*** Third-order polar') then
              do s=1,nsta
                 do s1=s,nsta
                    read(unit,*)
                    read(unit,*)
                    do m=1,nmodes
                       read(unit,'(9x,3(F11.6,3x))') polarder3(s,s1,1,1,m),polarder3(s,s1,1,2,m),polarder3(s,s1,1,3,m)
                       read(unit,'(9x,3(F11.6,3x))') polarder3(s,s1,2,2,m),polarder3(s,s1,2,3,m),polarder3(s,s1,3,3,m)
                    enddo
                 enddo
              enddo
              
           else if (string(1:22).eq.'*** Fourth-order polar') then
              do s=1,nsta
                 do s1=s,nsta
                    read(unit,*)
                    read(unit,*)
                    do m=1,nmodes
                       read(unit,'(9x,3(F11.6,3x))') polarder4(s,s1,1,1,m),polarder4(s,s1,1,2,m),polarder4(s,s1,1,3,m)
                       read(unit,'(9x,3(F11.6,3x))') polarder4(s,s1,2,2,m),polarder4(s,s1,2,3,m),polarder4(s,s1,3,3,m)
                    enddo
                 enddo
              enddo 




20    format(10x,6f10.4)

        endif

        goto 10
100     continue

        return
1010    stop 'ERROR IN GUESS FILE'
      end subroutine readlmfile

!#######################################################################      
      end module iofiles
