!#######################################################################

  program lmfit

    use decimal, only: long,dop
    use vcglobal
    use global, only:pi, keyword,keyorig,maxkey,lc,lwarn
    use maxv, only: maxdim,maxsta,maxset
    use op1lib, only: zerovxz, zerovxi, zerovxd, zerovxl
    use versions
    use vchammod
    use vchfitmod
    use versions
    use invchammod
    use logdat, only: message, routine, inptit
    
    use module_interface
   
    use symmetry, only: lmmask,delmodes_lm
    use rddatamod, only: read_dataset_lm,dataset_info
    use iofiles, only: readguess
    use utils, only: vcerrormsg,vchdefault,vcrmsd,parstats,&
                     vcwrline,vccntrl,vcwarnmsg,&
                     par2vec_lm,vec2par_lm,lmrmsd

    implicit none
    
    integer(long)       :: ilbl,modus,ierr
    character(len=10)   :: cerr

    real(dop), dimension(:), allocatable        :: optcoo
    complex(dop), dimension(:), allocatable     :: mc
    real(dop), dimension(:), allocatable        :: mr
    integer(long), dimension(:), allocatable    :: mi
    logical(kind=4), dimension(:), allocatable  :: ml

    integer(long)                                  :: c
    character(len=80)                              :: filename    
    real(dop)           :: fret

!-----------------------------------------------------------------------
! Initialise calculation 
! get input file name, set maxdim and then allocate memory
!-----------------------------------------------------------------------
      call initvcham(1)

      if (maxdim .lt. 0) then
         routine='VCTRANS'
         message='Error reading dimensions from input file'
         call vcerrormsg
      endif
      if (maxsta .lt. 0) then
         routine='VCTRANS'
         message='Error reading no. states from input file'
         call vcerrormsg
      endif

      call alloc_vchammod
      call alloc_vchfitmod

!-----------------------------------------------------------------------
! set default values for parameters and flags
!-----------------------------------------------------------------------
    call vchdefault
    call lmfdefault

!-----------------------------------------------------------------------
! open files
! read input file
!-----------------------------------------------------------------------
    call wrversion(ilog)
    call lmfinp

    if (lwarn) then
       routine = 'VCHFIT'
       message = 'Finished with input error. See log file'
       call vcerrormsg
    endif

!-----------------------------------------------------------------------
! assign memory for arrays
!-----------------------------------------------------------------------
    optcoodim = 0
    if (ldipfit) optcoodim = 3*nsta*nsta + 3*nsta*nsta*nfit +&
         3*nsta*nsta*nfit*nfit + 3*nsta*nsta*nfit +&
         3*nsta*nsta*nfit + 3*nsta*nsta + 3*nsta*nsta
    if (lpolarfit) optcoodim = optcoodim + 9*nsta*nsta + 9*nsta*nsta*nfit +&
         9*nsta*nsta*nfit*nfit + 9*nsta*nsta*nfit +&
         9*nsta*nsta*nfit + 9*nsta*nsta + 9*nsta*nsta
    
    call memlmpar
    
!-----------------------------------------------------------------------
! Allocate memory
!-----------------------------------------------------------------------
    write(ilog,'(/,a)') 'MEMORY ALLOCATED'
    ilbl = 8*optcoodim+16*mcdim+8*mrdim+4*midim+4*mldim
    write(ilog,'( 5(a,i10,/),a,i13,a,f10.2,a)')&
         ' optcoodim  : ', optcoodim,&
         ' complex*16 : ', mcdim,&
         ' real*8     : ', mrdim,&
         ' integer    : ', midim,&
         ' logical    : ', mldim,&
         ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
    call myflush(ilog)
    
    if(allocated(optcoo))deallocate(optcoo)
    if(allocated(mc))deallocate(mc)
    if(allocated(mr))deallocate(mr)
    if(allocated(mi))deallocate(mi)
    if(allocated(ml))deallocate(ml)
    
    cerr = 'lmfit_optcoo '
    allocate(optcoo(optcoodim),stat=ierr)
    if (ierr > 0) go to 2000
    cerr = 'lmfit_mc '
    allocate(mc(mcdim),stat=ierr)
    if (ierr > 0) go to 2000
    cerr = 'lmfit_mr '
    allocate(mr(mrdim),stat=ierr)
    if (ierr > 0) go to 2000
    cerr = 'lmfit_mi '
    allocate(mi(midim),stat=ierr)
    if (ierr > 0) go to 2000
    cerr = 'lmfit_ml '
    allocate(ml(mldim),stat=ierr)
    if (ierr > 0) go to 2000
    
    call zerovxz(mc,mcdim)
    call zerovxd(mr,mrdim)
    call zerovxi(mi,midim)
    call zerovxl(ml,mldim)
    
!-----------------------------------------------------------------------
! Get information from the dataset-file
!-----------------------------------------------------------------------
    write(6,'(/,a)') ' Reading datasets... '
    open(idat,file=datfile,form='formatted',status='old',err=999)
    call read_dataset_lm(mr(mrxcoo),mr(mrqcoo),mr(mradipole),mr(mrapolar),&
         mr(mrdipole),mr(mrpolar))
    close(idat)
    
!-----------------------------------------------------------------------
! Create masks
!-----------------------------------------------------------------------
    write(6,'(/,a)') 'Creating mask ...'
    call lmmask(mi(midipzeromask),mi(midipder1mask),mi(midipder2mask),&
         mi(midipder3mask),mi(midipder4mask),mi(mipolarzeromask),&
         mi(mipolarder1mask),mi(mipolarder2mask),mi(mipolarder3mask),&
         mi(mipolarder4mask))

!-----------------------------------------------------------------------
! Remove unwanted modes from fit
!-----------------------------------------------------------------------
    call delmodes_lm(mi(midipder1mask),mi(midipder2mask),mi(midipder3mask),&
           mi(midipder4mask),mi(mipolarder1mask),mi(mipolarder2mask),&
           mi(mipolarder3mask),mi(mipolarder4mask))
    
!-----------------------------------------------------------------------
! Determine number of parameters to be fitted
!-----------------------------------------------------------------------
    call numlmpar(mi(midipder1mask),mi(midipder2mask),mi(midipder3mask),&
           mi(midipder4mask),mi(mipolarder1mask),mi(mipolarder2mask),&
           mi(mipolarder3mask),mi(mipolarder4mask))

!-----------------------------------------------------------------------
! Determine zeroth-order terms
!-----------------------------------------------------------------------
    call getzeroterms(mr(mrdipzero),mr(mrpolarzero),mr(mrdipole),&
         mr(mrpolar),mr(mrqcoo))

!-----------------------------------------------------------------------
! Initialise parameters
!-----------------------------------------------------------------------
    write(6,'(/,a)') ' Initialising parameters ...'
    call initlmpar(mr(mrdipder1),mr(mrdipder2),mr(mrdipder3),mr(mrdipder4),&
         mi(midipder1mask),mi(midipder2mask),mi(midipder3mask),&
         mi(midipder4mask),&
         mr(mrpolarder1),mr(mrpolarder2),mr(mrpolarder3),mr(mrpolarder4),&
         mi(mipolarder1mask),mi(mipolarder2mask),mi(mipolarder3mask),&
         mi(mipolarder4mask))
    
!-----------------------------------------------------------------------
! Fit parameters
!-----------------------------------------------------------------------
    call fitlmpar(optcoo,mr,mi)

!-----------------------------------------------------------------------
! output parameters
!-----------------------------------------------------------------------
    open(iout,file=fileout,form='formatted',status='unknown')

    call outlmpar(iout,mr(mrdipzero),mr(mrdipder1),mr(mrdipder2),&
         mr(mrdipder3),mr(mrdipder4), mr(mrpolarzero),mr(mrpolarder1),&
         mr(mrpolarder2),mr(mrpolarder3),mr(mrpolarder4))

    close(iout)

    STOP

999 continue
    ilbl=index(datfile,' ')-1
    print*
    print*,'ERROR: Data file not found : ',datfile(1:ilbl)
    print*
    stop

2000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
    stop

2001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
    stop

    
  contains

!#######################################################################
    
    subroutine lmfdefault

      implicit none

      integer(long) :: i

      ldipfit=.false.
      lpolarfit=.false.
      lzerofit=.false.
      lder1fit=.false.
      lder2fit=.false.
      lder3fit=.false.
      lder4fit=.false.
      
      do i=1,3
         axislab(i)=''
      enddo

      nlmpar=0

      return
    end subroutine lmfdefault

!#######################################################################

    subroutine lmfinp
      
      integer(long) :: i,iz,iz0,ic,ierr,j,k,s,ilbl,n,m,s1,ijt,p
      character(len=16) :: string,string1
      
      iz0 = 0
      iz = 0

      lfit=.true.
      nfit=0
      call zerovxi(mfit,maxdim)

!-----------------------------------------------------------------------
! Open files
!-----------------------------------------------------------------------
      filename=filein
      open(iin,file=filename,form='formatted',status='old',iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a)') 'ERROR: Input file not found'
         write(6,'(a)') filein(1:80)
         write(6,*)
         stop
      endif

      fileout  = filein(1:ilbl)//'.lmi'

!-----------------------------------------------------------------------
! read input file
!-----------------------------------------------------------------------
10    continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'vchfinp (read LMFIT input)'
      call vccntrl(ierr)
      i=1
15    continue
      if (keyword(i) .eq. 'end-input') then
         goto 200

      else if (keyword(i) .eq. 'infofile') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            datfile=keyword(i)
         else
            routine = 'lmfinp (read LMFIT input)'
            message = 'No argument specified with infofile keyword'
            call vcwrline(0,iz,iz0)
         endif
         
      else if (keyword(i).eq.'fit_select') then
         lfit=.true.
         if (keyword(i+1).eq.'=') then
            call zerovxi(mfit,maxdim)
60          continue
            nfit=nfit+1
            i=i+2
            read(keyword(i),*,iostat=ierr) m
            mfit(m)=1
            if (ierr .ne. 0) then
               routine = 'lmfinp (read LMFIT input)'
               message ='selected mode must be an integer. Not :'
               call vcwrline(i,iz,iz0)
            endif

            if (keyword(i+1).eq.',') go to 60
         else
            routine = 'lmfinp (read LMFIT input)'
            message = 'No argument specified with fit_select keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'state_symmetry') then
         lautosym=.true.
330      continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         routine = 'lmfinp (read LMFIT input)'
         call vccntrl(ierr)
         
         if (keyword(1) .ne. 'end-state_symmetry') then
            read(keyword(1),*) s
            read(keyword(2),*) stalab(s)
            goto 330
         endif

      else if (keyword(i) .eq. 'axis_symmetry') then
         lautosym=.true.
340      continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         routine = 'lmfinp (read LMFIT input)'
         call vccntrl(ierr)
         
         if (keyword(1) .ne. 'end-axis_symmetry') then
            if (keyword(1).eq.'x') axislab(1)=keyword(2)
            if (keyword(1).eq.'y') axislab(2)=keyword(2)
            if (keyword(1).eq.'z') axislab(3)=keyword(2)
            goto 340
         endif

      else if(keyword(i).eq.'point_group') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) pntgrp
         endif

      else if(keyword(i).eq.'point_group') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) pntgrp
         endif
         
      else if (keyword(i).eq.'dipole_fit') then
         ldipfit=.true.

      else if (keyword(i).eq.'polar_fit') then
         lpolarfit=.true.
         

      else
         routine = 'lmfinp (read LMFIT input)'
         message = ' unknown keyword : '
         call vcwrline(i,iz,iz0)
      endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

 200  continue

      close(iin)

!-----------------------------------------------------------------------
! check required information has been given
!-----------------------------------------------------------------------
      do i=1,3
         if (axislab(i).eq.'') then
            write(6,'(a)') 'Not all axis symmetry labels have been given. &
                 Quitting!'
            STOP
         endif
      enddo      
      if (.not.ldipfit.and..not.lpolarfit) then
         write(6,'(a)') 'Nothing to fit. Quitting!'
         STOP
      endif

!-----------------------------------------------------------------------
! if no mode selection made, fit all modes
!-----------------------------------------------------------------------
      if (nfit .eq. 0) then
         nfit=nmodes
         do m=1,nmodes
            mfit(m)=1
         enddo
      endif

!-----------------------------------------------------------------------
! get system information from the dataset-file
!-----------------------------------------------------------------------
      ilbl=index(datfile,' ')-1
      if (ilbl .le. 0) then
         routine = 'lmfinp (read LMFIT input)'
         write(message,'(2a)') 'No datfile specified. '
         call vcerrormsg
      endif

      write(6,'(/,2a)')&
              'Data read from file : ',datfile(1:ilbl)
      open(idat,file=datfile,form='formatted',status='old',iostat=ierr)
      if (ierr .ne. 0) then
         routine = 'lmfinp (read LMFIT input)'
         write(message,'(2a)') 'No datfile found: ',datfile(1:ilbl)
         call vcerrormsg
      endif
      call dataset_info
      close(idat)


      return

999   continue
      ilbl=index(datfile,' ')-1
      print*
      print*,'ERROR: Data file not found : ',datfile(1:ilbl)
      print*
      stop

1010  continue
      ilbl=index(filein,' ')-1
      print*
      print*,'ERROR: input file not found : ',filein(1:ilbl)
      print*
      stop

    end subroutine lmfinp

!#######################################################################

    subroutine numlmpar(dipder1_mask,dipder2_mask,dipder3_mask,&
           dipder4_mask,polarder1_mask,polarder2_mask,polarder3_mask,&
           polarder4_mask)


      implicit none

      integer(long) :: j,k,s,s1,m,n
      integer(long), dimension(nsta,nsta,3) :: dipzero_mask
      integer(long), dimension(nsta,nsta,3,nmodes) :: dipder1_mask
      integer(long), dimension(nsta,nsta,3,nmodes,nmodes) :: dipder2_mask
      integer(long), dimension(nsta,nsta,3,nmodes) :: dipder3_mask
      integer(long), dimension(nsta,nsta,3,nmodes) :: dipder4_mask

      integer(long), dimension(nsta,nsta,3,3,nmodes) :: polarder1_mask
      integer(long), dimension(nsta,nsta,3,3,nmodes,nmodes) :: polarder2_mask
      integer(long), dimension(nsta,nsta,3,3,nmodes) :: polarder3_mask
      integer(long), dimension(nsta,nsta,3,3,nmodes) :: polarder4_mask

      if (ldipfit) then
         do s=1,nsta
            do s1=s,nsta
               do k=1,3
                  do m=1,nmodes
                     if (dipder1_mask(s,s1,k,m).eq.1) nlmpar=nlmpar+1
                     if (dipder3_mask(s,s1,k,m).eq.1) nlmpar=nlmpar+1
                     if (dipder4_mask(s,s1,k,m).eq.1) nlmpar=nlmpar+1
                     do n=m,nmodes
                        if (dipder2_mask(s,s1,k,m,n).eq.1) nlmpar=nlmpar+1
                     enddo
                  enddo
               enddo
            enddo
         enddo
      endif

      if (lpolarfit) then
         do s=1,nsta
            do s1=s,nsta
               do j=1,3
                  do k=1,3
                     do m=1,nmodes
                        if (polarder1_mask(s,s1,j,k,m).eq.1) nlmpar=nlmpar+1
                        if (polarder3_mask(s,s1,j,k,m).eq.1) nlmpar=nlmpar+1
                        if (polarder4_mask(s,s1,j,k,m).eq.1) nlmpar=nlmpar+1
                        do n=m,nmodes
                           if (polarder2_mask(s,s1,j,k,m,n).eq.1) nlmpar=nlmpar+1
                        enddo
                     enddo
                  enddo
               enddo
            enddo
         enddo
      endif

      return
    end subroutine numlmpar

!#######################################################################

    subroutine getzeroterms(dipzero,polarzero,dipole,polar,qcoo)
      
      implicit none

      integer(long)                               :: i,j,s,s1
      real(dop), dimension(nsta,nsta,3)           :: dipzero
      real(dop), dimension(nsta,nsta,3,3)         :: polarzero
      real(dop), dimension(nsets,nsta,nsta,3)     :: dipole
      real(dop), dimension(nsets,nsta,nsta,3,3)   :: polar
      real(dop), dimension(nmodes,nsets)          :: qcoo

! Check that the first dataset is at Q0
      do i=1,nmodes
         if (qcoo(i,1).gt.0.0001) then
            write(6,'(a)') 'First dataset must correspond to the reference &
                 point!'
            STOP
         endif
      enddo

      do s=1,nsta
         do s1=s,nsta
            do i=1,3
               dipzero(s,s1,i)=dipole(1,s,s1,i)
               dipzero(s1,s,i)=dipzero(s,s1,i)
               do j=1,3
                  polarzero(s,s1,i,j)=polar(1,s,s1,i,j)
                  polarzero(s1,s,i,j)=polarzero(s,s1,i,j)
               enddo
            enddo
         enddo
      enddo

      return
    end subroutine getzeroterms

!#######################################################################

    subroutine initlmpar(dipder1,dipder2,dipder3,dipder4,&
         dipder1_mask,dipder2_mask,dipder3_mask,dipder4_mask,&
         polarder1,polarder2,polarder3,polarder4,&
         polarder1_mask,polarder2_mask,polarder3_mask,polarder4_mask)

      implicit none
      
      integer(long) :: i,j,s,s1,m,n
      
      integer(long), dimension(nsta,nsta,3,nmodes)        :: dipder1_mask
      integer(long), dimension(nsta,nsta,3,nmodes,nmodes) :: dipder2_mask
      integer(long), dimension(nsta,nsta,3,nmodes)        :: dipder3_mask
      integer(long), dimension(nsta,nsta,3,nmodes)        :: dipder4_mask

      real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder1
      real(dop), dimension(nsta,nsta,3,nmodes,nmodes) :: dipder2
      real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder3
      real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder4

      integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder1_mask
      integer(long), dimension(nsta,nsta,3,3,nmodes,nmodes) :: polarder2_mask
      integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder3_mask
      integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder4_mask

      real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder1
      real(dop), dimension(nsta,nsta,3,3,nmodes,nmodes) :: polarder2
      real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder3
      real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder4

      
      real(dop)       :: x,small

      small=0.0
!      small=0.05

      do s=1,nsta
         do s1=s,nsta
            do i=1,3
               do m=1,nmodes
                  if (dipder1_mask(s,s1,i,m).eq.1) then
                     dipder1(s,s1,i,m)=small
                     dipder1(s1,s,i,m)=dipder1(s,s1,i,m)
                  else
                     dipder1(s,s1,i,m)=0.0
                     dipder1(s1,s,i,m)=dipder1(s,s1,i,m)
                  endif
                  if (dipder3_mask(s,s1,i,m).eq.1) then
                     dipder3(s,s1,i,m)=small
                     dipder3(s1,s,i,m)=dipder3(s,s1,i,m)
                  else
                     dipder3(s,s1,i,m)=0.0
                     dipder3(s1,s,i,m)=dipder3(s,s1,i,m)
                  endif
                  do n=m,nmodes
                     if (dipder2_mask(s,s1,i,m,n).eq.1) then
                     dipder2(s,s1,i,m,n)=small
                     dipder2(s1,s,i,m,n)=dipder2(s,s1,i,m,n)
                     dipder2(s,s1,i,n,m)=dipder2(s,s1,i,m,n)
                     dipder2(s1,s,i,n,m)=dipder2(s,s1,i,m,n)
                  else
                     dipder2(s,s1,i,m,n)=0.0
                     dipder2(s1,s,i,m,n)=dipder2(s,s1,i,m,n)
                     dipder2(s,s1,i,n,m)=dipder2(s,s1,i,m,n)
                     dipder2(s1,s,i,n,m)=dipder2(s,s1,i,m,n)
                  endif
                  enddo
               enddo
            enddo
         enddo
      enddo

      if (lpolarfit) then
         do s=1,nsta
            do s1=s,nsta
               do i=1,3
                  do j=1,3
                     do m=1,nmodes
                        if (polarder1_mask(s,s1,i,j,m).eq.1) then
                           polarder1(s,s1,i,j,m)=small
                           polarder1(s1,s,i,j,m)=polarder1(s,s1,i,j,m)
                        else
                           polarder1(s,s1,i,j,m)=0.0
                           polarder1(s1,s,i,j,m)=polarder1(s,s1,i,j,m)
                        endif
                        if (polarder3_mask(s,s1,i,j,m).eq.1) then
                           polarder3(s,s1,i,j,m)=small
                           polarder3(s1,s,i,j,m)=polarder3(s,s1,i,j,m)
                        else
                           polarder3(s,s1,i,j,m)=0.0
                           polarder3(s1,s,i,j,m)=polarder3(s,s1,i,j,m)
                        endif
                        do n=m,nmodes
                           if (polarder2_mask(s,s1,i,j,m,n).eq.1) then
                              polarder2(s,s1,i,j,m,n)=small
                              polarder2(s1,s,i,j,m,n)=polarder2(s,s1,i,j,m,n)
                              polarder2(s,s1,i,j,n,m)=polarder2(s,s1,i,j,m,n)
                              polarder2(s1,s,i,j,n,m)=polarder2(s,s1,i,j,m,n)
                           else
                              polarder2(s,s1,i,j,m,n)=0.0
                              polarder2(s1,s,i,j,m,n)=polarder2(s,s1,i,j,m,n)
                              polarder2(s,s1,i,j,n,m)=polarder2(s,s1,i,j,m,n)
                              polarder2(s1,s,i,j,n,m)=polarder2(s,s1,i,j,m,n)
                           endif
                        enddo
                     enddo
                  enddo
               enddo
            enddo
         enddo
      endif

      return
    end subroutine initlmpar

!#######################################################################

    subroutine fitlmpar(optcoo,mr,mi)

      implicit none

      integer(long)                                  :: s,s1,j,k,nopt,n,nsurf,check
      integer(long), dimension(midim), intent(inout) :: mi
      real(dop), dimension(mrdim), intent(inout)     :: mr
      real(dop), dimension(optcoodim), intent(out)   :: optcoo
      real(dop), dimension(1000*1000)                :: xi
      real(dop), dimension(1000,3)                   :: work
      integer(long)                                  :: iter0
      real(dop)                                      :: fret
      character(len=1), dimension(3)                 :: acmpnt,acmpnt1
      
      logical(kind=4)                                :: diprmsd,polarrmsd


      acmpnt(1)='x'
      acmpnt(2)='y'
      acmpnt(3)='z'
      acmpnt1(1)='x'
      acmpnt1(2)='y'
      acmpnt1(3)='z'

!Huge bodge:
      check=0

! nopt: number of parameters to be optimised in the fitting of the
!       current surface

      iter=10000
      
      write(6,*)
      write(6,'(a,x,i5)') "Number of parameters to be fitted:",nlmpar
      write(6,*)

      
      if (ldipfit) then
         nsurf=3*(nsta*nsta-(nsta*(nsta-1)/2))
         n=0
         diprmsd=.true.
         polarrmsd=.false.

! loop over dipole surfaces
         do s=1,nsta
            do s1=s,nsta
               do k=1,3

                  n=n+1
                  
                  write(6,'(a)') '++++++++++++++++++++++++++++++++++++++++++++++++++'
                  write(6,'(/,a,x,i4,3x,a,x,i4,3x,a1,a1,a1,x,i1,a1,x,i1,a1)') &
                       ' Fitting dipole surface',n,'of',nsurf,'(',acmpnt(k),',',s,',',s1,')'

                  call par2vec_lm(s,s1,k,1,optcoo,nopt,mr(mrdipder1),&
                       mr(mrdipder2),mr(mrdipder3),mr(mrdipder4),&
                       mi(midipder1mask),mi(midipder2mask),&
                       mi(midipder3mask),mi(midipder4mask),&
                       mr(mrpolarder1),mr(mrpolarder2),mr(mrpolarder3),&
                       mr(mrpolarder4),mi(mipolarder1mask),mi(mipolarder2mask),&
                       mi(mipolarder3mask),mi(mipolarder4mask),diprmsd,polarrmsd)

                  fret=lmrmsd(optcoo,mr,mi,s,s1,k,1,diprmsd,polarrmsd)
                  
                  write(6,'(/,x,a,3x,i2)') 'No. parameters:',nopt
                  write(6,'(/,a,e16.8)') &
                       ' START: RMSD (guess - data) = ',SQRT(fret/nsets)

! problems with amoeba are encountered if the surface is zero everywhere but nopt > zero.                  
                  if (fret.gt.0.00001) then
                     call init_amoeba_lm(xi,work(1,1),optcoo,nopt,lmrmsd,mr,mi,&
                          s,s1,k,1,diprmsd,polarrmsd)
                     
                     call amoeba_lm(xi,work(1,1),nopt,0.001_dop,lmrmsd,iter,&
                          work(1,2),optcoo,mr,mi,s,s1,k,1,diprmsd,polarrmsd)
                     
                     call vec2par_lm(s,s1,k,1,optcoo,mr(mrdipder1),&
                          mr(mrdipder2),mr(mrdipder3),mr(mrdipder4),&
                          mi(midipder1mask),mi(midipder2mask),&
                          mi(midipder3mask),mi(midipder4mask),&
                          mr(mrpolarder1),mr(mrpolarder2),mr(mrpolarder3),&
                          mr(mrpolarder4),mi(mipolarder1mask),mi(mipolarder2mask),&
                          mi(mipolarder3mask),mi(mipolarder4mask),diprmsd,polarrmsd)                     
                  endif
                     
                  fret=lmrmsd(optcoo,mr,mi,s,s1,k,1,diprmsd,polarrmsd)
                  write(6,'(a,e16.8,/)') &
                       ' END:   RMSD (guess - data) = ',SQRT(fret/nsets)
               enddo
            enddo
         enddo
      endif

      if (ldipfit) then
         ldipfit=.false.
         check=1
      endif

         if (lpolarfit) then
            nsurf=9*(nsta*nsta-(nsta*(nsta-1)/2))
            n=0
            diprmsd=.false.
            polarrmsd=.true.
! loop over polarisability surfaces
            do s=1,nsta
               do s1=s,nsta
                  do j=1,3
                     do k=1,3

                        n=n+1
                        
                        write(6,'(a)') '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'
                        write(6,'(/,a,x,i4,3x,a,x,i4,3x,  a1,a1,a1,x,a1,a1,x,i1,a1,x,i1,a1)') &
                             ' Fitting polarisability surface',n,'of',nsurf,'(',acmpnt(j),',',acmpnt1(k),',',s,',',s1,')'

                        call par2vec_lm(s,s1,j,k,optcoo,nopt,mr(mrdipder1),&
                             mr(mrdipder2),mr(mrdipder3),mr(mrdipder4),&
                             mi(midipder1mask),mi(midipder2mask),&
                             mi(midipder3mask),mi(midipder4mask),&
                             mr(mrpolarder1),mr(mrpolarder2),mr(mrpolarder3),&
                             mr(mrpolarder4),mi(mipolarder1mask),mi(mipolarder2mask),&
                             mi(mipolarder3mask),mi(mipolarder4mask),diprmsd,polarrmsd)
                        
                        fret=lmrmsd(optcoo,mr,mi,s,s1,j,k,diprmsd,polarrmsd)
                        
                        write(6,'(/,x,a,3x,i2)') 'No. parameters:',nopt
                        write(6,'(/,a,e16.8)') &
                             ' START: RMSD (guess - data) = ',SQRT(fret/nsets)

! problems with amoeba are encountered if the surface is zero everywhere but nopt > zero.                  
                        if (fret.gt.0.00001) then
                           call init_amoeba_lm(xi,work(1,1),optcoo,nopt,lmrmsd,mr,mi,&
                                s,s1,j,k,diprmsd,polarrmsd)
                           
                           call amoeba_lm(xi,work(1,1),nopt,0.001_dop,lmrmsd,iter,&
                                work(1,2),optcoo,mr,mi,s,s1,j,k,diprmsd,polarrmsd)
                           
                           call vec2par_lm(s,s1,j,k,optcoo,mr(mrdipder1),&
                                mr(mrdipder2),mr(mrdipder3),mr(mrdipder4),&
                                mi(midipder1mask),mi(midipder2mask),&
                                mi(midipder3mask),mi(midipder4mask),&
                                mr(mrpolarder1),mr(mrpolarder2),mr(mrpolarder3),&
                                mr(mrpolarder4),mi(mipolarder1mask),mi(mipolarder2mask),&
                                mi(mipolarder3mask),mi(mipolarder4mask),diprmsd,polarrmsd)
                        endif

                        fret=lmrmsd(optcoo,mr,mi,s,s1,j,k,diprmsd,polarrmsd)
                        write(6,'(a,e16.8,/)') &
                             ' END:   RMSD (guess - data) = ',SQRT(fret/nsets)
                        
                     enddo
                  enddo
               enddo
            enddo
         endif

      return
    end subroutine fitlmpar

!#######################################################################

    subroutine outlmpar(io,dipzero,dipder1,dipder2,dipder3,dipder4,&
         polarzero,polarder1,polarder2,polarder3,polarder4)
      
      implicit none

      integer(long) :: io,k,s,s1,m,n,nblck,i,j,j1,j2,j3,l

      real(dop), dimension(nsta,nsta,3)               :: dipzero
      real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder1
      real(dop), dimension(nsta,nsta,3,nmodes,nmodes) :: dipder2
      real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder3
      real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder4
      
      real(dop), dimension(nsta,nsta,3,3)               :: polarzero
      real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder1
      real(dop), dimension(nsta,nsta,3,3,nmodes,nmodes) :: polarder2
      real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder3
      real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder4
      
      character(len=1), dimension(3)                    :: comp


      comp(1)='x'
      comp(2)='y'
      comp(3)='z'

      write(io,'(a,f8.5)') '# LMFIT Version :',progver
      ilbl=index(datfile,' ')-1
      write(io,'(2a)') '# Data file : ',datfile(1:ilbl)

      write(io,'(/,/,a)') &
          '*** Zeroth-order dipole expansion coefficients ***'
      write(io,'(/,x,a1,2x,a2,4x,3(a1,12x)) ') 's', 's1', 'x', 'y', 'z'
      do s=1,nsta
         do s1=s,nsta
            write(io,'(i2,x,i2,4x,3(F11.8,3x))') s,s1,dipzero(s,s1,1),dipzero(s,s1,2),dipzero(s,s1,3)
         enddo
      enddo

      write(io,'(/,a)') &
          '*** First-order dipole expansion coefficients ***'
      do s=1,nsta
         do s1=s,nsta
            write(io,'(/,a6,i2,x,a3,i2,6x,3(a1,13x))') 'States',s,'and',s1,'x','y','z'
            do m=1,nmodes
               write(io,'(i2,x,a,3(F12.8,3x))') m,nmlab(m),dipder1(s,s1,1,m),dipder1(s,s1,2,m),dipder1(s,s1,3,m)
            enddo
         enddo
      enddo

      write(io,'(/,a)') '*** Second-order dipole expansion coefficients ***'
      nblck=((nmodes-1)/6)+1
      do s=1,nsta
         do s1=s,nsta
            do k=1,3
               write(io,'(4x,a6,2(6x,i1,3x))') 'State:',&
                    mod(s,10),mod(s1,10)
               if (k.eq.1) write(io,'(4x,a)') 'x-component'
               if (k.eq.2) write(io,'(4x,a)') 'y-component'
               if (k.eq.3) write(io,'(4x,a)') 'z-component'
               do n=1,nblck
                  j1=(n-1)*6+1
                  j2=min(j1+5,nmodes)
                  write(io,'(10x,6(5x,i2,3x))') (j,j=j1,j2)
                  do i=j1,nmodes
                     j3=min(i,j2)
                     write(io,20) i,nmlab(i),(dipder2(s,s1,k,i,j),j=j1,j3)
                  enddo
               enddo
               write(io,*)
            enddo
         enddo
      enddo

      write(io,'(/,a)') &
          '*** Third-order dipole expansion coefficients (cubic terms only) ***'
      do s=1,nsta
         do s1=s,nsta
            write(io,'(/,a6,i2,x,a3,i2,6x,3(a1,13x))') 'States',s,'and',s1,'x','y','z'
            do m=1,nmodes
               write(io,'(i2,x,a,3(F12.8,3x))') m,nmlab(m),dipder3(s,s1,1,m),dipder3(s,s1,2,m),dipder3(s,s1,3,m)
            enddo
         enddo
      enddo

      write(io,'(/,a)') &
          '*** Fourth-order dipole expansion coefficients (quartic terms only) ***'
      do s=1,nsta
         do s1=s,nsta
            write(io,'(/,a6,i2,x,a3,i2,6x,3(a1,13x))') 'States',s,'and',s1,'x','y','z'
            do m=1,nmodes
               write(io,'(i2,x,a,3(F12.8,3x))') m,nmlab(m),dipder4(s,s1,1,m),dipder4(s,s1,2,m),dipder4(s,s1,3,m)
            enddo
         enddo
      enddo

      write(io,*)

      write(io,'(/,/,a)') &
          '*** Zeroth-order polarisability expansion coefficients ***'
      write(io,'(/,x,a1,2x,a2)') 's', 's1'
      do s=1,nsta
         do s1=s,nsta
            write(io,'(i2,x,i2,4x,3(F14.8,3x))') s,s1,polarzero(s,s1,1,1),polarzero(s,s1,1,2),polarzero(s,s1,1,3)
            write(io,'(9x,3(F14.8,3x))') polarzero(s,s1,2,2),polarzero(s,s1,2,3),polarzero(s,s1,3,3)
         enddo
      enddo

      write(io,'(/,a)') &
           '*** First-order polarisability expansion coefficients ***'
      do s=1,nsta
         do s1=s,nsta
            write(io,'(/,a6,i2,x,a3,i2)') 'States',s,'and',s1
            do m=1,nmodes
               write(io,'(i2,x,a3,3x,3(F11.6,3x))') m,nmlab(m),polarder1(s,s1,1,1,m),polarder1(s,s1,1,2,m),polarder1(s,s1,1,3,m)
               write(io,'(9x,3(F11.6,3x))') polarder1(s,s1,2,2,m),polarder1(s,s1,2,3,m),polarder1(s,s1,3,3,m)
            enddo
         enddo
      enddo

      write(io,'(/,a)') '*** Second-order polarisability expansion coefficients ***'
      nblck=((nmodes-1)/6)+1
      do s=1,nsta
         do s1=s,nsta
            do k=1,3
               do l=1,3
                  write(io,'(4x,a6,2(6x,i1,3x))') 'State:',&
                       mod(s,10),mod(s1,10)
                  write(io,'(4x,4a)') comp(k),',',comp(l),'-component'
                  do n=1,nblck
                     j1=(n-1)*6+1
                     j2=min(j1+5,nmodes)
                     write(io,'(10x,6(5x,i2,3x))') (j,j=j1,j2)
                     do i=j1,nmodes
                        j3=min(i,j2)
                        write(io,20) i,nmlab(i),(polarder2(s,s1,k,l,i,j),j=j1,j3)
                     enddo
                  enddo
                  write(io,*)
               enddo
            enddo
         enddo
      enddo

      write(io,'(/,a)') &
           '*** Third-order polarisability expansion coefficients (cubic terms only) ***'
      do s=1,nsta
         do s1=s,nsta
            write(io,'(/,a6,i2,x,a3,i2)') 'States',s,'and',s1
            do m=1,nmodes
               write(io,'(i2,x,a3,3x,3(F11.6,3x))') m,nmlab(m),polarder3(s,s1,1,1,m),polarder3(s,s1,1,2,m),polarder3(s,s1,1,3,m)
               write(io,'(9x,3(F11.6,3x))') polarder3(s,s1,2,2,m),polarder3(s,s1,2,3,m),polarder3(s,s1,3,3,m)
            enddo
         enddo
      enddo

      write(io,'(/,a)') &
           '*** Fourth-order polarisability expansion coefficients (quartic terms only) ***'
      do s=1,nsta
         do s1=s,nsta
            write(io,'(/,a6,i2,x,a3,i2)') 'States',s,'and',s1
            do m=1,nmodes
               write(io,'(i2,x,a3,3x,3(F11.6,3x))') m,nmlab(m),polarder4(s,s1,1,1,m),polarder4(s,s1,1,2,m),polarder4(s,s1,1,3,m)
               write(io,'(9x,3(F11.6,3x))') polarder4(s,s1,2,2,m),polarder4(s,s1,2,3,m),polarder4(s,s1,3,3,m)
            enddo
         enddo
      enddo
      write(io,*)


20    format(i3,2x,a5,6f10.4)

      return
    end subroutine outlmpar

!#######################################################################
  end program lmfit
