module lmplotmod

  use decimal, only: long, dop
  use maxv, only: maxdim
  use versions

  implicit none


  integer(long), save :: efd,efd1,plsta,plsta1

  logical(kind=4), save :: lpldip,lplpolar,lpladiab

  real(dop), save :: qmin

end module lmplotmod

!#######################################################################

program lmplot
  
  use decimal, only: long,dop,sip
  use vcglobal
  use global, only:pi, keyword,keyorig,maxkey,lc,lwarn
  use maxv, only: maxdim,maxsta,maxset
  use vchammod
  use vcplotmod
  use lmplotmod
  use invchammod
  
  use op1lib, only: zerovxd, zerovxi
  use utils, only: vcerrormsg,rmblnk0,vchdefault,chkcoocut,chkcoovec
  use functions, only: ddipole,dpolar

  use rddatamod, only: dataset_info,read_dataset_lm
  use iofiles, only: readlmfile
  use logdat, only: message, routine, inptit
  
  use module_interface

  implicit none
  
  integer(long), external                     :: slen
  integer(long)                               :: ilbl,ierr
  character(len=10)   :: cerr
  
  complex(dop), dimension(:), allocatable     :: mc
  real(dop), dimension(:), allocatable        :: mr
  integer(long), dimension(:), allocatable    :: mi
  logical(kind=4), dimension(:), allocatable  :: ml
  complex(sip), dimension(:), allocatable     :: ms
  real(sip), dimension(:), allocatable        :: mf
  
  
  ierr=0

  call wrversion(ilog)
  
!-----------------------------------------------------------------------
! Initialise calculation 
! get input file name, set maxdim and then allocate memory
!-----------------------------------------------------------------------
      call initvcham(1)

      if (maxdim .lt. 0) then
         routine='VCTRANS'
         message='Error reading dimensions from input file'
         call vcerrormsg
      endif
      if (maxsta .lt. 0) then
         routine='VCTRANS'
         message='Error reading no. states from input file'
         call vcerrormsg
      endif

      call alloc_vchammod

!-----------------------------------------------------------------------
! set default values
!-----------------------------------------------------------------------
  call vchdefault
  call lmpldefault

!-----------------------------------------------------------------------
! read input file
! define output file etc.
!-----------------------------------------------------------------------
  print*,'Read input file'
  call lmplinp

  if (plotfit .gt. 0) then
     ilbl=slen(guessfile)
     print*,'LMI data from file : ',guessfile(1:ilbl)
     open(iguess,file=guessfile,form='formatted',status='old',&
          iostat=ierr)              
     if (ierr .ne. 0) then
        routine = 'lmplot'
        write(message,'(2a)') 'No guessfile found: ',&
             guessfile(1:ilbl)
        call vcerrormsg
     endif
  endif

  ilbl=slen(datfile)
  write(6,'(/,2a)') 'Data read from file : ',datfile(1:ilbl)
  open(idat,file=datfile,form='formatted',status='old',iostat=ierr)
  if (ierr .ne. 0) then
     routine = 'lmplot'
     write(message,'(2a)') 'No infofile found: ',datfile(1:ilbl)
     call vcerrormsg
  endif

!-----------------------------------------------------------------------
! get system information from info file
!-----------------------------------------------------------------------
  print*,'Read dataset info'
  call dataset_info

!-----------------------------------------------------------------------
! Allocate memory
!-----------------------------------------------------------------------
  call memlmpar

  ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
  write(ilog,'(/,a)') 'MEMORY ALLOCATED'
  write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)')&
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
  call myflush(ilog)

  if(allocated(mc))deallocate(mc)
  if(allocated(mr))deallocate(mr)
  if(allocated(mi))deallocate(mi)
  if(allocated(ml))deallocate(ml)
  if(allocated(ms))deallocate(ms)
  if(allocated(mf))deallocate(mf)

  cerr = 'lmplot_mc '
  allocate(mc(mcdim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'lmplot_mr '
  allocate(mr(mrdim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'lmplot_mi '
  allocate(mi(midim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'lmplot_ml '
  allocate(ml(mldim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'lmplot_ms '
  allocate(ms(msdim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'lmplot_mf '
  allocate(mf(mfdim),stat=ierr)
  if (ierr > 0) go to 2000
  
  call zerovxd(mr,mrdim)
  call zerovxi(mi,midim)

!----------------------------------------------------------------------
! read the ab-initio data file 
!----------------------------------------------------------------------
  print*,'Reading ab-initio data-file'
  rewind(idat)
  call read_dataset_lm(mr(mrxcoo),mr(mrqcoo),mr(mradipole),mr(mrapolar),&
       mr(mrdipole),mr(mrpolar))
  close(idat)                                                            

!----------------------------------------------------------------------
!     read the VCHAM parameters
!----------------------------------------------------------------------
      if (plotfit .ne. 0) then
        call readlmfile(iguess,mr(mrdipzero),mr(mrdipder1),mr(mrdipder2),&
             mr(mrdipder3),mr(mrdipder4),mr(mrpolarzero),mr(mrpolarder1),&
             mr(mrpolarder2),mr(mrpolarder3),mr(mrpolarder4))
        close(iguess)
      endif

!-----------------------------------------------------------------------
!      Open plot file
!-----------------------------------------------------------------------
  open(50,file=gnufile,form='formatted')
                                                                                
!-----------------------------------------------------------------------
! Write gnuplot data and ab initio points to plot file
!-----------------------------------------------------------------------
  print*,'Writing gnuplot data'

  call sublmplot(nsets,mr(mrxcoo),mr(mrqcoo),mr(mradipole),mr(mrapolar),&
       mr(mrdipole),mr(mrpolar))

  print*,'Finished writing gnuplot data'

!-----------------------------------------------------------------------
! Write model surfaces to plot file if required
!-----------------------------------------------------------------------
  if (plotfit .ne. 0) then
     call plotlmi(mr)
  endif

!-----------------------------------------------------------------------
! Close plot file
!-----------------------------------------------------------------------
  close(50)
                                                                                
!-----------------------------------------------------------------------
! Send to screen
!-----------------------------------------------------------------------
  call excmd('chmod 700 '//gnufile(1:slen(gnufile)+1))
  call excmd('./'//gnufile(1:slen(gnufile)+1))
  call excmd('gv '//psfile(1:slen(psfile))//' &')

  STOP

2000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
  stop
  
contains

!#####################################################################

  subroutine lmpldefault
    
    implicit none
    
    integer(long)       :: i
    
    xmin=-999.0
    xmax= 999.0
    ymin=-999.0
    ymax= 999.0
    lxrange=.false.
    lyrange=.false.
    do i=1,maxdim
       nmlab(i)=' '
    enddo
    
    step = -999.0

    lpldip=.false.
    lplpolar=.false.

    lpladiab=.false.

    efd=-999
    efd1=-999
    plsta=-999
    plsta1=-999

    return
  end subroutine lmpldefault

!#####################################################################

  subroutine lmplinp

    implicit none
    
    integer(long)           :: i,iarg,ilbl,jlbl,c
    character(len=80)       :: string
    logical(kind=4)         :: lguess
    integer(long), external :: myiargc 


    plotfit = 1

    guessfile=' '
    N = 0
    N2 = 0
    xscl = 1.0
    negdiag = .false.

    neps = 0
    nsets=0
    
    iarg = myiargc()
    if (iarg .lt. 1) goto 200
    
    i=0
100 i=i+1
    call mygetarg(i,string)
    
    if (string(1:1) .eq. '-') then

       if(string .eq. '-polar')then
          lplpolar=.true.
          i=i+1
          call mygetarg(i,string)
          if (string.eq.'x') efd=1
          if (string.eq.'y') efd=2
          if (string.eq.'z') efd=3
          i=i+1
          call mygetarg(i,string)
          if (string.eq.'x') efd1=1
          if (string.eq.'y') efd1=2
          if (string.eq.'z') efd1=3
          i=i+1
          call mygetarg(i,string)
          read(string,*) plsta
          i=i+1
          call mygetarg(i,string)
          read(string,*) plsta1
          
       else if (string .eq. '-dipole')then
          lpldip=.true.
          i=i+1
          call mygetarg(i,string)
          if (string.eq.'x') efd=1
          if (string.eq.'y') efd=2
          if (string.eq.'z') efd=3
          i=i+1
          call mygetarg(i,string)
          read(string,*) plsta
          i=i+1
          call mygetarg(i,string)
          read(string,*) plsta1

       else if (string .eq. '-adiab')then
          lpladiab=.true.

       else if(string .eq. '-xrange') then
          lxrange=.true.
          i=i+1
          call mygetarg(i,string)
          read(string,*)xmin
          i=i+1
          call mygetarg(i,string)
          read(string,*)xmax
          
       else if(string .eq. '-yrange') then
          lyrange=.true.
          i=i+1
          call mygetarg(i,string)
          read(string,*)ymin
          i=i+1
          call mygetarg(i,string)
          read(string,*)ymax

       else
          write(6,'(2a)') ' unknown option : ', string
          stop
       end if

! read next option
       go to 100

!-----------------------------------------------------------------------
! 1 argument is filename
!-----------------------------------------------------------------------
    else if (i .eq. iarg) then
       guessfile=string

!-----------------------------------------------------------------------
! 2 arguments are filename and mode
!-----------------------------------------------------------------------
    else if (i .eq. iarg-1) then
       guessfile=string
       i=i+1
       call mygetarg(i,string)
       read(string,*) N

!-----------------------------------------------------------------------
! 3 arguments are filename and 2 modes
!-----------------------------------------------------------------------
    else if (i .eq. iarg-2) then
       guessfile=string
       i=i+1
       call mygetarg(i,string)
       read(string,*) N
       i=i+1
       call mygetarg(i,string)
       read(string,*) N2

    endif

200 continue
    if (slen(guessfile) .eq. 0) then
       print*,'Enter Filename'
       read(5,'(a)') guessfile
    endif

    if (n .eq. 0) then
       write(6,'(a)') 'Enter no. of mode to be ploted'
       read(5,*) n
    endif

    if (N2 .eq. 0) n2=n

    if (N .gt. N2) then
       i = N2
       N2 = N
       N = i
    end if

!-----------------------------------------------------------------------
! set up output file names
!-----------------------------------------------------------------------
      ilbl=index(guessfile,' ')-1
      c = ilbl-3
      if (guessfile(c:ilbl) .ne. '.dat') then
         guessfile=guessfile(1:ilbl)//'.lmi'
      endif
      inquire(file=guessfile,exist=lguess)
      if (.not. lguess) then
         print*,'Guess file does not exist :',guessfile
         stop
      endif

      call rmblnk(string)
      jlbl=index(string,' ')-1
      ilbl=index(guessfile,'.')-1
      write(filestem,*) guessfile(1:ilbl)//'_'//string(1:jlbl)
      print*,filestem      
      ilbl=index(filestem,' ')
      if (ilbl .eq. 1) then
         filestem(1:79)=filestem(2:80)
      endif
      ilbl=index(filestem,' ')-1
      gnufile=filestem(1:ilbl)//'.gnu'
      psfile=filestem(1:ilbl)//'.ps'

!-----------------------------------------------------------------------
! if reading info file, set datfile to input file name
! else get datfile from guessfile
! also get other information from guessfile 
!-----------------------------------------------------------------------
    ilbl=index(guessfile,'.')-1
    if (guessfile(ilbl+2:ilbl+4) .eq. 'dat') then
       plotfit=0
       datfile=guessfile
       call rmblnk0(datfile)
       guessfile='  '
    else
       open(iguess,file=guessfile,form='formatted',status='old')
10     continue
       read(iguess,'(a)',end=15) string
       if (string(1:14).eq.'# Data file : ') then
          datfile=string(14:80)
          call rmblnk0(datfile)
       endif
       go to 10
15     continue
       close(iguess)
    endif

    if (plotfit.gt.0.and.lpladiab) then
       write(6,'(/,a,/)') 'Plotting of adiabatic model surfaces is not yet&
            implemented. Quitting!'
       STOP
    endif

    return
  end subroutine lmplinp

!#######################################################################

  subroutine sublmplot(nsetss,xcoo,qcoo,dipole,polar,diab_dipole,diab_polar)

    implicit none
    

    real(dop), dimension(ncoo,nsets), intent(out)           :: xcoo
    real(dop), dimension(nmodes,nsets), intent(out)         :: qcoo
    real(dop), dimension(nsets,nsta,nsta,3), intent(out)    :: dipole,diab_dipole
    real(dop), dimension(nsets,nsta,nsta,3,3), intent(out)  :: polar,diab_polar

    integer(long)                                   :: i,j,k,s,ll,num,npnts
    integer(long), external                         :: slen
    real(dop), dimension(maxset)                    :: qn
    character(len=2000)                             :: line
    character(len=8)                                :: filename
    character(len=4)                                :: fnum4
    real(dop)                                       :: p,rd

    integer(long), intent(in)                       :: nsetss

    character(len=1)                                :: ctmp,ctmp1
    
    real(dop), dimension(maxset)                    :: dedef


    call zerovxd(q,maxdim)

    if (efd.eq.1) ctmp='x'
    if (efd.eq.2) ctmp='y'
    if (efd.eq.3) ctmp='z'

    if (efd1.eq.1) ctmp1='x'
    if (efd1.eq.2) ctmp1='y'
    if (efd1.eq.3) ctmp1='z'

!-----------------------------------------------------------------------
! ___ write gnuplot call and commands - data follows in same file,
! ___ as we are drawing from stdin
!-----------------------------------------------------------------------
      write(50,'(a)') '#! /usr/bin/gnuplot -persist'
      write(50,'(a)') &
           'set term postscript portrait color solid "Helvetica" 16'
      write(50,'(a)') 'set output "'//psfile(1:slen(psfile))//'"'

      if (lpldip) then
         write(50,'(3a,x,i1,x,a,x,i1,x,a5)') 'set ylabel "Dipole moment: ',ctmp,&
              '-component, states',plsta,'and',plsta1,'[au]"'
      else if (lplpolar) then
         write(50,'(5a,x,i1,x,a,x,i1,x,a5)') 'set ylabel "Polarisability: ',&
              ctmp,',',ctmp1,'-component, states',plsta,'and',plsta1,'[au]"'
      endif
      write(fnum4,'(i4.4)') n
      if(n.eq.n2)then
         write(50,'(a)') 'set title "mode #'//fnum4//'" font &
              "Helventica,24"'
      else
         write(50,'(a,i4.4,a)') 'set title "diag #'//fnum4//'-#',N2,'"'
      end if

      write(50,'(a)') 'set format y "%.1e"'

      if (lxrange) write(50,'(a,f8.3,a,f8.3,a)') 'set xrange [',xmin,':',xmax,']'
      if (lyrange) write(50,'(a,f8.3,a,f8.3,a)') 'set yrange [',ymin,':',ymax,']'
      
      write(50,'(a)') 'unset key'

!-----------------------------------------------------------------------
! plotting of model surfaces
!-----------------------------------------------------------------------
      if (plotfit .ne. 0) then
          write(50,'(a)') 'plot "-" w p pt 7 ps 1, "-" w l lw 3'
       else
          write(50,'(a)') 'plot "-" w p pt 7 ps 1 '
      end if
      write(50,*)
      
!----------------------------------------------------------------------
! scan the ab-initio data for appropriate points and write it to  
! a gnu file
!----------------------------------------------------------------------
      qmax=1.0_dop
      qmin=1.0_dop
      npnts=0

      do j=1,nsetss
         ismode=.true.
         call chkcoocut(qcoo(1,j),N,N2,nmodes,ismode,xscl,negdiag)         
         if (ismode) then
            npnts=npnts+1
            if (.not.lpladiab) then
               if (lpldip) dedef(npnts)=diab_dipole(j,plsta,plsta1,efd)
               if (lplpolar) dedef(npnts)=diab_polar(j,plsta,plsta1,efd,efd1)
            else
               if (lpldip) dedef(npnts)=dipole(j,plsta,plsta1,efd)
               if (lplpolar) dedef(npnts)=polar(j,plsta,plsta1,efd,efd1)
            endif
         endif
         qn(npnts)=qcoo(N,j)
         qmax=max(qmax,qn(npnts))
         qmin=min(qmin,qn(npnts))
      enddo

!-----------------------------------------------------------------------
! plot ab initio points
!-----------------------------------------------------------------------
      do i=1,npnts
         write(50,'(2(f14.8,x))') qn(i),dedef(i)
      enddo

      write(50,'(/,a)') 'e'

    return
  end subroutine sublmplot

!#####################################################################
  
  subroutine plotlmi(mr)

    implicit none

    integer(long)                           :: j,i
    real(dop), dimension(mrdim), intent(in) :: mr
    real(dop)                               :: tmp
    character(len=8)                        :: filename
    
    if (step .lt. 0) then
       step = qmax*1.1
       step = step / dble(npnts)
    endif

    do i=-npnts,npnts
       q(N)=i*step

       if (lpldip) tmp=ddipole(q,mr,plsta,plsta1,efd)
       if (lplpolar) tmp=dpolar(q,mr,plsta,plsta1,efd,efd1)

       write(50,'(2(F14.5,x))') q(N),tmp
    enddo

    return
  end subroutine plotlmi

!#####################################################################
end program lmplot
