      module lmtransmod

use decimal, only: long,dop

      implicit none

!------------------------------------------------------------------------
! Input files
!------------------------------------------------------------------------
      character(len=80), save :: setsfile
      character(len=80), save :: vchamfile

!------------------------------------------------------------------------
! Dipole/polarisability output
!------------------------------------------------------------------------
      logical(kind=4), dimension(:), allocatable, save      :: lpolar
      character(len=80), dimension(:,:), allocatable, save  :: polarfiles

!------------------------------------------------------------------------
! Data reading options
!------------------------------------------------------------------------
      integer(long), dimension(:,:,:), allocatable, save :: phaseswitch


    end module lmtransmod

!########################################################################

program lmtrans

  use decimal, only: long,dop,sip
  use vcglobal
  use global, only:pi, keyword,keyorig,maxkey,lc,lwarn
  use maxv, only: maxdim,maxsta,maxset
  use keyunitsmod
  use vchammod
  use versions
  use op1lib, only: zerovxd, zerovxi

!  use ioqc, only: getintyp, getabintype, gettdipall, nm2xmat, getnm
!  use readqc, only: nmortho
  use ioqc, only: getintyp, getabintype, getxcoo, gettdipall, nm2xmat, &
                  nmortho, getnm

  use utils, only: vcwarnmsg,vcerrormsg,vchdefault,vccntrl,vcwrline, &
                   dattrans,x2q,q2x,getlabel
    
  use iofiles, only: rdfile0,readguess
  use symmetry, only: getnadd
  use vctransmod
  use lmtransmod
  use invchammod
  use logdat, only: message, routine, inptit
  use functions, only: pes
  
  implicit none
  
  integer(long)                               :: i,iau
    
  integer(long)                               :: ilbl,ierr
  character(len=10)                           :: cerr
  logical(kind=4)                             :: lsocdegen

  complex(dop), dimension(:), allocatable     :: mc
  real(dop), dimension(:), allocatable        :: mr
  integer(long), dimension(:), allocatable    :: mi
  logical(kind=4), dimension(:), allocatable  :: ml
  complex(sip), dimension(:), allocatable     :: ms
  real(sip), dimension(:), allocatable        :: mf

!-----------------------------------------------------------------------
! Initialise calculation 
! get input file name, set maxdim and then allocate memory
!-----------------------------------------------------------------------
      call initvcham(1)

      if (maxdim .lt. 0) then
         routine='VCTRANS'
         message='Error reading dimensions from input file'
         call vcerrormsg
      endif
      if (maxsta .lt. 0) then
         routine='VCTRANS'
         message='Error reading no. states from input file'
         call vcerrormsg
      endif

      call alloc_vctransmod
      call alloc_vchammod

      cerr = 'lpolar '
      allocate(lpolar(maxset))
      if (ierr > 0) go to 2000
      cerr = 'polarfiles '
      allocate(polarfiles(maxset,6))
      if (ierr > 0) go to 2000
      cerr = 'phaseswitch '
      allocate(phaseswitch(maxset,maxsta,maxsta))
      if (ierr > 0) go to 2000

!-----------------------------------------------------------------------
! set defaults
!-----------------------------------------------------------------------
  call vchdefault
  call lmtdefault
  
!-----------------------------------------------------------------------
! read input file
! define output file etc.
!-----------------------------------------------------------------------
  call lmtinp
  
  if (lwarn) then
     routine = 'LMTRANS'
     message = 'Finished with input error. See log file'
     call vcerrormsg
  endif

!-----------------------------------------------------------------------
! get information from normal mode file
!-----------------------------------------------------------------------
  iau=1
  lsocdegen=.true.
  call rdfile0(lrde0,abintype0,iau,lsocdegen)

!-----------------------------------------------------------------------
! orthogonalise normal modes
!-----------------------------------------------------------------------
  call nmortho(nmcoo,maxdim,ncoo,nmodes,ilog)

!-----------------------------------------------------------------------
! if required, determine fragment normal mode symmetry labels
!-----------------------------------------------------------------------
  if (lpft) call getfragsym(fragnmlab)

!-----------------------------------------------------------------------
! write system information to info file
!-----------------------------------------------------------------------
  call wrfile0_lm(fragnmlab)

!-----------------------------------------------------------------------
! create the transformation matrices
!-----------------------------------------------------------------------
  iau=1
  call nm2xmat(nmcoo,coonm,iatsym,freq0,iau,ncoo,maxdim,nmodes,ilog)

!-----------------------------------------------------------------------
! determine the no. explicitely added terms
!-----------------------------------------------------------------------
      call getnadd(iguess)

!-----------------------------------------------------------------------
! assign memory 
!-----------------------------------------------------------------------
  print*,'Memory assigned'
  print*,''
  call memvcham

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
  ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
  write(ilog,'(/,a)') 'MEMORY ALLOCATED'
  write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)')&
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
  call myflush(ilog)
  
  if(allocated(mc))deallocate(mc)
  if(allocated(mr))deallocate(mr)
  if(allocated(mi))deallocate(mi)
  if(allocated(ml))deallocate(ml)
  if(allocated(ms))deallocate(ms)
  if(allocated(mf))deallocate(mf)
  
  cerr = 'lmtrans_mc '
  allocate(mc(mcdim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'lmtrans_mr '
  allocate(mr(mrdim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'lmtrans_mi '
  allocate(mi(midim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'lmtrans_ml '
  allocate(ml(mldim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'lmtrans_ms '
  allocate(ms(msdim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'lmtrans_mf '
  allocate(mf(mfdim),stat=ierr)
  if (ierr > 0) go to 2000

  cerr = 'lpolar '
  allocate(lpolar(maxset),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'polarfiles '
  allocate(polarfiles(maxset,6),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'phaseswitch '
  allocate(phaseswitch(maxset,maxsta,maxsta),stat=ierr)
  if (ierr > 0) go to 2000

  call zerovxd(mr,mrdim)
  call zerovxi(mi,midim)

!-----------------------------------------------------------------------
! Read in the parameters of the vibronic coupling Hamiltonian to be used
! to contruct the ADT matrix
!-----------------------------------------------------------------------
  if (nsta.gt.1) then
     call readguess(iguess,mr(mrconst),mr(mrkappa),&
          mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
          mi(miexpcoemask),mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
          mr(mrswitch),mr(mrcp2d))
  endif
     
  close(iguess)

!-----------------------------------------------------------------------
! Read dipoles and polarisabilities, and output to info file
!-----------------------------------------------------------------------
  call read_lmdata(mr)

  deallocate(mc,mr,ml,mi,ms,mf)
  deallocate(lpolar,polarfiles,phaseswitch)

  stop

2000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      
  stop
contains

!#######################################################################

  subroutine lmtdefault

    implicit none

    integer(long)       :: i,j,k


    do i=1,maxdim
       nmlab1(i) = '  '
       freq1(i) = 0.0_dop
       do j=1,maxsta
          do k=1,maxsta
             phaseswitch(i,j,k)=0
          enddo
       enddo
    enddo
    
    do i=1,maxsta
       lpolar(i)=.false.
    enddo

    return
  end subroutine lmtdefault


!#######################################################################

  subroutine lmtinp

    implicit none
        

    integer(long)           :: i,iz,iz0,ic,ierr,ilbl,laenge,s,ista,&
         m,nmpsets,n
    integer(long)           :: linein,ntr,abintype1,gndabtype1
    character(len=80)       :: string,filename,blank,currorient
    

!-----------------------------------------------------------------------
! set initial values
!
! currorient: default orientation for reading geometry (e.g. in Gaussian)
!-----------------------------------------------------------------------
    iz0     = 0
    iz      = 0
    filein = ' '
    currorient = 'input'
    file0orient=' '
    do i=1,maxset
       dataorient(i)=' '
    enddo
    datadir = './'
    laenge=2
    call abspath(datadir,laenge)
    nsets=0
    nmpsets=0
    nsetm=0
    nsta=0
    linein=0
    blank= ' '
    vchamfile=''
      
    abintype0=-999
    abintype1=-999
    do n=1,maxset
       abintype(n)=-999
    enddo

    ilbl=index(filein,'.')-1
    if (ilbl .le. 0) then
       ilbl=index(filein,' ')-1
       filein=filein(1:ilbl)//'.inp'
    endif
    fileout=filein(1:ilbl)//'.dat'

!-----------------------------------------------------------------------
! Open files
!-----------------------------------------------------------------------
    filename=filein
    open(iin,file=filename,form='formatted',status='old',iostat=ierr)
    if (ierr .ne. 0) then
       write(6,'(a)') 'ERROR: Input file not found'
       write(6,'(a)') filein(1:80)
       write(6,*)
       stop
    endif
    open(iout,file=fileout,form='formatted',status='unknown')

!-----------------------------------------------------------------------
! read input file
!-----------------------------------------------------------------------
10  continue
    call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
    routine = 'lmtinp (read LMTRANS input)'
    call vccntrl(ierr)
    
    i=1
15  continue
    if (keyword(i) .eq. 'end-input') then
       goto 20
       
    else if (keyword(i) .eq. 'orient') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          currorient=keyword(i)
       else
          routine = 'lmtinp (read LMTRANS input)'
          message = 'No argument specified with orient keyword'
          call vcwrline(0,iz,iz0)
       endif
       
    else if (keyword(i) .eq. 'file0') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          file0=keyorig(i)
       else
          routine = 'lmtinp (read LMTRANS input)'
          message = 'No argument specified with file0 keyword'
          call vcwrline(0,iz,iz0)
       endif
       
    else if (keyword(i) .eq. 'file0orient') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          file0orient=keyword(i)
       else
          routine = 'lmtinp (read LMTRANS input)'
          message = 'No argument specified with file0orient keyword'
          call vcwrline(0,iz,iz0)
       endif
       
    else if (keyword(i) .eq. 'datadir') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          datadir = keyorig(i)
          laenge = lc(i)
          if (laenge .eq. 0) then
             routine = 'lmtinp (read LMTRANS input)'
             message = ' No path name specified'
             call vcwrline(0,iz,iz0)
          end if
          call abspath(datadir,laenge)
       else
          routine = 'lmtinp (read LMTRANS input)'
          message = 'No argument specified with datadir keyword'
          call vcwrline(0,iz,iz0)
       endif

    else if (keyword(i) .eq. 'nmodes') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          read(keyword(i),*) nmodes
          ncoo=nmodes+6
          natm = ncoo/3 
       else
          routine = 'lmtinp (read LMTRANS input)'
          message = 'No argument specified with nmodes keyword'
          call vcwrline(0,iz,iz0)
       endif
       
    else if (keyword(i) .eq. 'nstates') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          read(keyword(i),*) nsta
       else
          routine = 'lmtinp (read LMTRANS input)'
          message = 'No argument specified with nstates keyword'
          call vcwrline(0,iz,iz0)
       endif

    else if (keyword(i).eq.'abinitiotype') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          call getabintype(keyword(i),abintype1,ilog)
       else
          routine = 'lmtinp (read LMTRANS input)'
          message = 'No argument specified with abinitiotype keyword'
          call vcwrline(0,iz,iz0)
       endif
       
    else if (keyword(i).eq.'files') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          read(keyword(i),*) setsfile
       else
          routine = 'lmtinp (read LMTRANS input)'
          message = 'No argument specified with files keyword'
          call vcwrline(0,iz,iz0)
       endif
       
    else if (keyword(i).eq.'vcham_file') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          read(keyword(i),*) vchamfile
       else
          routine = 'lmtinp (read LMTRANS input)'
          message = 'No argument specified with vcham_file keyword'
          call vcwrline(0,iz,iz0)
       endif

!--------------------------------------------------------------------
! Coordinate transformation keywords
!--------------------------------------------------------------------
    else if (keyword(i).eq.'system_atom') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          read(keyword(i),*) sysatom
       endif
    else if (keyword(i).eq.'system_define') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          read(keyword(i),*) ratm1
          read(keyword(i+1),*) ratm2
          read(keyword(i+2),*) ratm3
          read(keyword(i+3),*) ratm4
          i=i+3
       endif
       
    else if (keyword(i).eq.'parent_to_fragment_trans') then
       lpft=.true.
       
    else if (keyword(i) .eq. 'parent_file') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          parentfile=keyword(i)
       else
          routine = 'lmtinp (read LMTRANS input)'
          message = 'No argument specified with parent_file keyword'
          call vcwrline(0,iz,iz0)
       endif
    else if (keyword(i) .eq. 'fragment_file') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          fragmentfile=keyword(i)
       else
          routine = 'lmtinp (read LMTRANS input)'
          message = 'No argument specified with fragment_file keyword'
          call vcwrline(0,iz,iz0)
       endif
    else if (keyword(i) .eq. 'fragment_nmodes') then
       if (keyword(i+1).eq.'=') then
          i=i+2
          read(keyword(i),*) fragnmodes
          fragncoo=fragnmodes+6
          fragnatom = fragncoo/3
       else
          routine = 'lmtinp (read LMTRANS input)'
          message = 'No argument specified with fragment_nmodes keyword'
          call vcwrline(0,iz,iz0)
       endif
       
    else if (keyword(i) .eq. 'define_system_modes') then
140    continue
    call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
    routine = 'lmtinp (read LMTRANS input)'
    call vccntrl(ierr)
    
    if (keyword(1) .ne. 'end-define_system_modes') then
       read(keyword(1),*) m
       read(keyword(2),*) sysfreq(m)
       read(keyword(3),*) syslab(m)
       goto 140
    endif
    
 else
    routine = 'lmtinp (read LMTRANS input)'
    message = ' unknown keyword : '
    call vcwrline(i,iz,iz0)

 endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
 if (i.lt.ic) then
    i=i+1
    go to 15
 else
    go to 10
 endif
 
20 continue
 close(iin)
 
!------------------------------------------------------------------        
! Read dataset file
!------------------------------------------------------------------
 open(idat,file=setsfile,form='formatted',status='old',iostat=ierr)
 call rdmufile(idat,setsfile)
 close(idat)
 
 if (abintype0 .eq. -999) abintype0=abintype1
 
 do n=1,nsets
    if (abintype(n) .eq. -999) abintype(n)=abintype1
    dataorient(n)=currorient
 enddo
 
 if (file0orient .eq. ' ') then
    file0orient=currorient
 endif
 
 if (vchamfile.eq.''.and.nsta.gt.1) then
    write(6,'(a)') 'No VCHAM file specified. Quitting!'
    STOP
 endif
 
!------------------------------------------------------------------        
! Open vcham file
!------------------------------------------------------------------
 if (nsta.gt.1) open(iguess,file=vchamfile,form='formatted',status='old')

 return
end subroutine lmtinp

!#######################################################################

subroutine wrfile0_lm(fragnmlab)

  implicit none
  
  integer(long)       :: i,j,k,m,j1,j2,nblk
  character(len=16), dimension(maxdim), intent(in) :: fragnmlab
  real(dop), dimension(ncoo) :: xcoo,nxcoo

!----------------------------------------------------------------------
! write all the information from the S0 file
!----------------------------------------------------------------------
  write(iout,'(a,f8.5)') '# VCHam Version :',progver
  write(iout,'(a)') '# All energies and frequencies in eV'
  write(iout,'(a)') ' ' 
  if (ntitle .gt. 0) then
     write(iout,'(a)') '# title ' 
     do i=1,ntitle
        write(iout,'(a)') title(i)
     enddo
     write(iout,'(a)') ' ' 
  endif

  write(iout,'(a)') '#system_dimensions'
  write(iout,'(a)') '#internal_coordinates'
  write(iout,'(i3)') nmodes
  write(iout,'(a)')'#trivial_modes'
  write(iout,'(i3)') ncoo-nmodes
  write(iout,'(a)') '#num_states'
  write(iout,'(i3)') nsta
  write(iout,'(a)') '#states'
  write(iout,'(20i3)') (states(i),i=1,nsta)
  if (lpft.eqv..true.) then
     write(iout,'(a)') '#fragment_modes'
     write(iout,'(i3)') fragnmodes
  endif
  write(iout,'(a)') '#end_system_dimensions'
  
  write(iout,*)
  write(iout,'(a)') '#general_information from the ground state'
  write(iout,'(a)') '#equilibrium_state_geometry'
  do i=1,ncoo,3
     write(iout,'(3F15.10)') (xcoo0(j),j=i,i+2)
  enddo
  write(iout,'(a)') '#masses'
  do i=1,ncoo,3
     write(iout,'(F15.10)') mass(i)
  enddo
  if (.not. lpft) then
     write(iout,'(a)') '#vibrational_frequencies'
     do m=1,nmodes
        write(iout,'(i5,2x,a,F20.10)') m,nmlab(m),freq0(m)
     enddo
  else
     write(iout,'(a)') '#vibrational_frequencies'
     do m=1,fragnmodes
        write(iout,'(i5,2x,a,F20.10)') m,fragnmlab(m),fragfreq0(m)
     enddo
     do m=fragnmodes+1,nmodes
        write(iout,'(i5,2x,a,F20.10)') m,syslab(m),sysfreq(m)
     enddo
  endif
  
  
  write(iout,'(a)') '#end_general_information'
  write(iout,*) 
  
  return
end subroutine wrfile0_lm

!#####################################################################

subroutine getfragsym(fragnmlab)
  
  implicit none
  
  integer(long) :: ifiletyp,unit,ierr,iau
  character(len=16), dimension(maxdim), intent(out) :: fragnmlab
  character(len=80) :: filename
  real(dop), dimension(maxdim,maxdim) :: fragnmcoo
  
  filename=fragmentfile
  
  unit=55
  open(unit,file=filename,form='formatted',status='old',err=1045)
  call getintyp(unit,ifiletyp,filename,ilog)
  rewind(unit)
      
  iau=1
  call getnm(unit,ifiletyp,fragnmlab,fragnmcoo,fragfreq0,iatsym,iau, &
       maxdim,fragncoo,nsta,fragnmodes,ilog,1,.true.)
  
  goto 1055
  
1045 write(6,'(a)') "Fragment file0 not found"
  STOP
  
1055 continue
  
  return
end subroutine getfragsym

!#######################################################################

subroutine read_lmdata(mr)
  
  implicit none
  
  integer(long)            :: i,j,n,ierr,ilbl,laenge,p,ifiletyp,count,filter
  integer(long), external                    :: slen
  character(len=200)                         :: filename
  real(dop), dimension(mrdim), intent(inout) :: mr
  real(dop), dimension(nsta,nsta,3)          :: dipole,diab_dipole
  real(dop), dimension(nsta,nsta,3,3)        :: polar,diab_polar
  
  real(dop), dimension(maxsta,maxsta) :: diabvec
  real(dop), dimension(maxsta,maxsta) :: diabv
  real(dop), dimension(maxsta)        :: v

  ierr=0
  
  write(iout,'(a)') '#dataset_section'
  laenge=slen(datadir)
  write(6,'(a)') 'Data Directory : '
  write(6,'(5x,a)') datadir(1:laenge)
  
  count=0
  do n=1,nsets
     ilbl=slen(datafile(n))
     if (datafile(n)(1:1) .ne. '/') then
        filename=datadir(1:laenge)//'/'//datafile(n)(1:ilbl)
     else
        filename=datafile(n)
     endif

     write(6,*)
     write(6,'(i5,a22,a)') n,': Reading data from: ',&
          datafile(n)(1:ilbl)
     write(ilog,'(i5,a22,a)') &
          n,': Reading data from: ',datafile(n)(1:ilbl)
     print*,filename
     open(idat,file=filename,form='formatted',status='old',&
          iostat=ierr)
     
     if (ierr .ne. 0) then
        routine = 'lmtrans'
        write(message,'(a)') 'Data File not found: '
        write(message1,'(a)') filename
        call vcwarnmsg
     else
! Read dipoles
        call rdmudata(n,filename,xcoo,qcoo,dipole,ifiletyp)
        close(idat)
! Calculate polarisabilities if required        
        if (lpolar(n)) then
         
           do i=1,nsta
              do j=1,nsta
                 phaseswitch(n,j,i)=0
              enddo
           enddo
           
           call rdpolar(n,polar,ifiletyp)
        endif
     endif

! Transform coordinates if required
     if (lpft) then
        count=count+1
        filter=0
        call dattrans(filter,count)
     endif

! Obtain eigenvectors of the diabatic potential
     if (nsta.gt.1) then
        call pes(qcoo(1:nmodes),v,mr(mrconst),mr(mrkappa),&
             mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
             mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
             mr(mrswitch),mr(mrcp2d),.true.,diabvec,diabv)
     else
        diabvec(1,1)=1.0
     endif
     
! Perform ADT of dipoles and polarisabilities
     call adt(dipole,polar,diabvec,diab_dipole,diab_polar,lpolar(n))
     
! Output data        
     if (lpft) then
        if (filter.eq.1) call wrdata_lm(n,xcoo,qcoo,dipole,polar,&
             diab_dipole,diab_polar)
     else
        call wrdata_lm(n,xcoo,qcoo,dipole,polar,diab_dipole,diab_polar)
     endif
  enddo

  return
end subroutine read_lmdata

!#######################################################################

subroutine rdmufile(unit,setsfile)
        
  implicit none
  
  integer(long)                 :: i,j,ic,ierr,siz,siz0,s,s1
  integer(long), intent(in)     :: unit
  character(len=80), intent(in) :: setsfile

  ierr=0
100 continue
  call rdinpf(unit,keyword,keyorig,inptit,lc,ic,siz,ierr,maxkey)
  routine = 'rdsets  '//setsfile(1:60)
  call vccntrl(ierr)
  
  if (keyword(1) .ne. 'end-files') then
     if (nsets .lt. maxset) nsets=nsets+1
     datafile(nsets) = keyorig(1)
     
     i=1
105  continue
     i=i+1
     if (keyword(i).eq.'polar') then
        lpolar(nsets)=.true.
        if (keyword(i+1).eq.'=') then
           i=i+1
           do j=1,6
              i=i+1
              polarfiles(nsets,j)=keyorig(i)
           enddo
        else
           message = 'No argument specified with polar keyword'
           call vcwrline(0,siz,siz0)
        endif
        
     else if (keyword(i).eq.'negphase') then
        if (keyword(i+1).eq.'=') then
           i=i+1
110        continue
           i=i+1
           read(keyword(i),*) s
           read(keyword(i+1),*) s1
           i=i+2
           phaseswitch(nsets,s,s1)=1
           phaseswitch(nsets,s1,s)=phaseswitch(nsets,s,s1)
           if (keyword(i).eq.',') goto 110
        else
           message = 'No argument specified with negphase keyword'
           call vcwrline(0,siz,siz0)
        endif
        
     endif
     if (i .lt. ic) then
        go to 105
     else
        go to 100
     endif
  endif
  
  return
end subroutine rdmufile

!#######################################################################
      
subroutine rdmudata(n,filename,xcoo,qcoo,dipole,ifiletyp)

  implicit none
  integer(long)     :: ifiletyp,iau
  integer(long), intent(in)                              :: n
  integer(long), dimension(ncoo)                         :: iatiso
  real(dop), dimension(maxdim), intent(out)              :: xcoo,qcoo
  real(dop), dimension(nsta,nsta,3), intent(out)         :: dipole
  character(len=*), intent(in)                           :: filename
  
  call getintyp(idat,ifiletyp,filename,ilog)
  write(ilog,'(15x,a,i5)') 'Ab initio type:',abintype(n)

!-----------------------------------------------------------------------
! coordinates
!-----------------------------------------------------------------------
  iau=1
  call getxcoo(idat,xcoo,ifiletyp,iau,dataorient(n),iatsym,&
       iatiso,ncoo,nsta,nmodes,ilog)
  call x2q(qcoo,xcoo,xcoo0,coonm,nmodes,ncoo,maxdim)

!-----------------------------------------------------------------------
! Read ALL elements of the dipole matrix        
!----------------------------------------------------------------------- 
  call gettdipall(idat,dipole,ifiletyp,abintype(n),found,nsta,&
       phaseswitch(n,1:nsta,1:nsta))
  
  return
end subroutine rdmudata

!#######################################################################
      
subroutine rdpolar(n,polar,ifiletyp)

  implicit none
  integer(long)  :: ifiletyp,ilast,i,j,k,l,m,unit,s,s1,ierr
  integer(long), intent(in)                              :: n
  real(dop), dimension(nsta,nsta,3,3), intent(out)       :: polar
  real(dop), dimension(nsta,nsta,3)                      :: dummy
  real(dop), dimension(nsta,nsta,3,3)                    :: dipole_pos,dipole_neg
  character(len=80)                                      :: string
  character(len=1), dimension(6)                         :: cmpt
  real(dop), dimension(6)                                :: de
  real(dop), dimension(3)                                :: fac,tmp

  ierr=0

  unit=75

!-----------------------------------------------------------------------
! Should include checks to ensure that:
! 
! (i)   The same geometry is used in all polarfiles files.
! (ii)  The same difference in electric field is used in pairs of files
!       corresponding to the same direction of the applied electric
!       field.
! (iii) We have pairs of files (pos and neg) for all directions (x,y,z)
!       at each point.
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
! Zero arrays
!-----------------------------------------------------------------------
  do i=1,3
     do j=1,3
        do k=1,nsta
           do l=1,nsta
              polar(l,k,j,i)=0
              dipole_pos(l,k,j,i)=0
              dipole_neg(l,k,j,i)=0
           enddo
        enddo
     enddo
  enddo

!-----------------------------------------------------------------------
! Determine direction of applied field for each file
!-----------------------------------------------------------------------
  do i=1,6

     open(unit,file=polarfiles(n,i),form='formatted',&
          status='old',iostat=ierr)
     if (ierr .ne. 0) then
        routine = 'lmtrans'
        write(message,'(a)') 'Data File not found: '
        write(message1,'(a)') polarfiles(n,i)
        call vcwarnmsg
        STOP
     endif

10   continue
     read(unit,'(a)') string
     if (string(2:45).ne.&
          'Reading unperturbed one-electron hamiltonian') goto 10
     read(unit,*)
     read(unit,*)
     read(unit,'(17x,3(F10.7,1x))') (tmp(j), j=1,3)
     
     if (tmp(1).ne.0.and.tmp(2).eq.0.and.tmp(3).eq.0) then
        cmpt(i)='X'
        de(i)=tmp(1)
     else if (tmp(1).eq.0.and.tmp(2).ne.0.and.tmp(3).eq.0) then
        cmpt(i)='Y'
        de(i)=tmp(2)
     else if (tmp(1).eq.0.and.tmp(2).eq.0.and.tmp(3).ne.0) then
        cmpt(i)='Z'
        de(i)=tmp(3)
     else
        write(6,'(/,a,/)') 'Applied dipole field has more than one non-zero component. Quitting'
        STOP
     endif

     close(unit)
  enddo

!-----------------------------------------------------------------------
! Determine differences in electric field strengths
! 
! fac(i): difference in field strength along direction i,
!         i=1,2,3 (x,y,z)
!-----------------------------------------------------------------------
  do i=1,6
     if (cmpt(i).eq.'X') fac(1)=abs(de(i))
     if (cmpt(i).eq.'Y') fac(2)=abs(de(i))
     if (cmpt(i).eq.'Z') fac(3)=abs(de(i))
  enddo

!-----------------------------------------------------------------------
! Create positive and negative dE (E: electric field) dipole arrays
!
! dipole_pos(s,s1,cmpt,dir): s, s1, cmpt, direction of field
! dipole_neg(s,s1,cmpt,dir): s, s1, cmpt, direction of field
!-----------------------------------------------------------------------
  do m=1,6
! x-direction of electric field
     if (cmpt(m).eq.'X') then
        if (de(m).gt.0) then
           
           open(unit,file=polarfiles(n,m),form='formatted',&
                status='old',iostat=ierr)
           rewind(unit)
           call gettdipall(unit,dummy,ifiletyp,abintype(n),found,nsta,&
                phaseswitch(n,1:nsta,1:nsta))
           close(unit)
           
           do k=1,3
              do j=1,nsta
                 do i=1,nsta
                    dipole_pos(i,j,k,1)=dummy(i,j,k)
                 enddo
              enddo
           enddo
        endif
        if ((de(m).lt.0)) then
           
           open(unit,file=polarfiles(n,m),form='formatted',&
                status='old',iostat=ierr)
           rewind(unit)
           call gettdipall(unit,dummy,ifiletyp,abintype(n),found,nsta,&
                phaseswitch(n,1:nsta,1:nsta))
           close(unit)
           
           do k=1,3
              do j=1,nsta
                 do i=1,nsta
                    dipole_neg(i,j,k,1)=dummy(i,j,k)
                 enddo
              enddo
           enddo
        endif
     endif
! y-direction of electric field
     if (cmpt(m).eq.'Y') then
        if (de(m).gt.0) then
           
           open(unit,file=polarfiles(n,m),form='formatted',&
                status='old',iostat=ierr)
           rewind(unit)
           call gettdipall(unit,dummy,ifiletyp,abintype(n),found,nsta,&
                phaseswitch(n,1:nsta,1:nsta))
           close(unit)
           
           do k=1,3
              do j=1,nsta
                 do i=1,nsta
                    dipole_pos(i,j,k,2)=dummy(i,j,k)
                 enddo
              enddo
           enddo
        endif
        if ((de(m).lt.0)) then
           
           open(unit,file=polarfiles(n,m),form='formatted',&
                status='old',iostat=ierr)
           rewind(unit)
           call gettdipall(unit,dummy,ifiletyp,abintype(n),found,nsta,&
                phaseswitch(n,1:nsta,1:nsta))
           close(unit)
           
           do k=1,3
              do j=1,nsta
                 do i=1,nsta
                    dipole_neg(i,j,k,2)=dummy(i,j,k)
                 enddo
              enddo
           enddo
        endif
     endif
! z-direction of electric field           
     if (cmpt(m).eq.'Z') then
        if (de(m).gt.0) then
           
           open(unit,file=polarfiles(n,m),form='formatted',&
                status='old',iostat=ierr)
           rewind(unit)
           call gettdipall(unit,dummy,ifiletyp,abintype(n),found,nsta,&
                phaseswitch(n,1:nsta,1:nsta))
           close(unit)
           
           do k=1,3
              do j=1,nsta
                 do i=1,nsta
                    dipole_pos(i,j,k,3)=dummy(i,j,k)
                 enddo
              enddo
           enddo
        endif
        if ((de(m).lt.0)) then
           
           open(unit,file=polarfiles(n,m),form='formatted',&
                status='old',iostat=ierr)
           rewind(unit)
           call gettdipall(unit,dummy,ifiletyp,abintype(n),found,nsta,&
                phaseswitch(n,1:nsta,1:nsta))
           close(unit)
           
           do k=1,3
              do j=1,nsta
                 do i=1,nsta
                    dipole_neg(i,j,k,3)=dummy(i,j,k)
                 enddo
              enddo
           enddo
        endif
     endif
  enddo

!-----------------------------------------------------------------------
! Calculate polarisabilities
!-----------------------------------------------------------------------
  do s=1,nsta
     do s1=s,nsta
        do i=1,3
           do j=1,3
              polar(s,s1,i,j)=dipole_pos(s,s1,i,j)-dipole_neg(s,s1,i,j)
              polar(s,s1,i,j)=polar(s,s1,i,j)/(2*fac(j))
              polar(s1,s,i,j)=polar(s,s1,i,j)
           enddo
        enddo
     enddo
  enddo

  return
end subroutine rdpolar

!#######################################################################

subroutine wrdata_lm(n,xcoo,qcoo,dipole,polar,diab_dipole,diab_polar)

  implicit none
  
  integer(long)                                :: nblk,j,j1,j2,i,k
  integer(long), intent(in)                        :: n
  character(len=8)                                 :: label
  real(dop), dimension(maxdim), intent(in)         :: xcoo,qcoo
  real(dop), dimension(nsta,nsta,3), intent(out)   :: dipole,diab_dipole
  real(dop), dimension(nsta,nsta,3,3), intent(out) :: polar,diab_polar
  
  nblk=((nmodes-1)/5)+1
  call getlabel(n,label)
  write(iout,'(a)') '#dataset_'//label//'    '
  
  write(iout,'(a)') '#point_in_q'
  do i=1,nblk
     j1=(i-1)*5+1
     j2=min(j1+4,nmodes)
     write(iout,'(5F15.10)') (qcoo(j),j=j1,j2) 
  enddo
  write(iout,'(a)') '#point_in_cartesian_coordinates'
  do i=1,ncoo,3
     write(iout,'(3F15.10)') (xcoo(j),j=i,i+2)
  enddo
  
  write(iout,'(a)') '#adiabatic_dipole_matrix (s,s1,x,y,z)'
  do i=1,nsta
     do j=i,nsta
        write(iout,10) i,j,dipole(i,j,1),dipole(i,j,2),dipole(i,j,3)
     enddo
  enddo
  
  write(iout,'(a)') '#diabatic_dipole_matrix (s,s1,x,y,z)'
  do i=1,nsta
     do j=i,nsta
        write(iout,10) i,j,diab_dipole(i,j,1),diab_dipole(i,j,2),diab_dipole(i,j,3)
     enddo
  enddo

  if (lpolar(n)) then

     write(iout,'(a)') '#adiabatic_polarisability_tensor'
     write(iout,20) 's','s1','xx','xy','xz'
     write(iout,25) 'yy','yz','zz'
     do i=1,nsta
        do j=i,nsta
           write(iout,10) i,j,polar(i,j,1,1),polar(i,j,1,2),polar(i,j,1,3)
           write(iout,15) polar(i,j,2,2),polar(i,j,2,3),polar(i,j,3,3)
        enddo
     enddo

     write(iout,'(a)') '#diabatic_polarisability_tensor'
     write(iout,20) 's','s1','xx','xy','xz'
     write(iout,25) 'yy','yz','zz'
     do i=1,nsta
        do j=i,nsta
           write(iout,10) i,j,polar(i,j,1,1),polar(i,j,1,2),polar(i,j,1,3)
           write(iout,15) polar(i,j,2,2),polar(i,j,2,3),polar(i,j,3,3)
        enddo
     enddo
  
  endif

  write(iout,'(a)') '#end_dataset_'//label//'    '

10 format (1x,i2,1x,i2,3x,3(F14.8,3x))
15 format (9x,3(F14.8,3x))
20 format (1x,a2,1x,a2,7x,3(a2,15x))
25 format (13x,3(a2,15x))
  return
end subroutine wrdata_lm

!#######################################################################

subroutine adt(dipole,polar,diabvec,diab_dipole,diab_polar,lpolar)

  implicit none
  
  integer(long)                        :: i,j,k,l,m,n
  real(dop), dimension(nsta,nsta,3)    :: dipole,diab_dipole
  real(dop), dimension(nsta,nsta,3,3)  :: polar,diab_polar
  real(dop), dimension(maxsta,maxsta)  :: diabvec
  logical(kind=4)                      :: lpolar

!------------------------------------------------------------------        
! Transform dipoles
!------------------------------------------------------------------
  do k=1,3
     do j=1,nsta
        do i=j,nsta
           diab_dipole(i,j,k)=0
           do m=1,nsta
              do n=1,nsta
                 diab_dipole(i,j,k)=diab_dipole(i,j,k)+&
                      diabvec(i,m)*dipole(m,n,k)*diabvec(j,n)
              enddo
           enddo
           diab_dipole(j,i,k)=diab_dipole(i,j,k)
        enddo
     enddo
  enddo
        
!------------------------------------------------------------------        
! Transform polarisabilities
!------------------------------------------------------------------
  do l=1,3
     do k=1,3
        do j=1,nsta
           do i=j,nsta
              diab_polar(i,j,k,l)=0
              do m=1,nsta
                 do n=1,nsta
                    diab_polar(i,j,k,l)=diab_polar(i,j,k,l)+&
                         diabvec(i,m)*polar(m,n,k,l)*diabvec(j,n)
                 enddo
              enddo
              diab_polar(j,i,k,l)=diab_polar(i,j,k,l)
           enddo
        enddo
     enddo
  enddo
     
  return
end subroutine adt

!#####################################################################
end program lmtrans
