      module makedbmod

use decimal, only: long,dop
use global, only:pi, c1, c2, c5, keyword, keyorig, lc, maxkey, lwarn,&
                 ddname, ddlaenge, lsocbas, logisopen
use maxv, only: maxdim,maxsta,maxset

      implicit none

!-----------------------------------------------------------------------
! files
!
! datadir: directory containing data files.
!-----------------------------------------------------------------------
      character(len=c5), save :: datadir
      character(len=c5), dimension(:), allocatable, save :: datafile, &
                                                            gndfile
      character(len=c5), dimension(:), allocatable, save :: dataorient, &
                                                            gndorient

!-----------------------------------------------------------------------
! options for input and output
!
! order        : maximum order of data to be read from data file
! abintype(n)  : ab initio type of data file
! gndabtype(n) : ab initio type of ground-state energy file (used for
!                OVGF if GS energy other than RHF required)
! eigen(n)     : specifies a specific root to be read from a file.
!                used when each state is in a different file (e.g. Gaussian)
!-----------------------------------------------------------------------
      integer(long), dimension(:), allocatable, save :: abintype,&
                                                        gndabtype, &
                                                        eigen
      integer(long), save   :: order
      integer(long), dimension(:), allocatable, save :: states

      logical :: ldiab, lhess0
      character(len=c2), save               :: qcmethod,qcprog

      end module makedbmod

!#######################################################################

      program makedb

use vcglobal, only: filein,filelog,fileout,iin,ilog,itmpin,idat,&
                    nsets,nsta,ncoo,nmodes,message1
use keyunitsmod
use versions
use invchammod
use makedbmod

use op1lib, only: zerovxd, zeromxd, cpvxd, unitqxd
use ioqc,   only: getintyp, getabintype, getxcoo, getnm, geten, getder1, &
                  getdcp, getder2, gettdip, getgd, nm2xmat, nm2xmat2
use readqc
use symoplib
use logdat, only: message, routine, inptit, errormsg
use utils
use iofiles, only: rdfile0,cpfreq,rdpftfiles
use keyunitsmod

use mmlib, only: mmxxdd,mmtxdd

use griddatmod, only: nddstate
use dirdyn, only: alloc_dirdyn,dealloc_dirdyn,dbnrec,ndofddpes,ndofdd,&
                  alloc_dddb,dealloc_dddb,nroot,nbasis,dde0,dimref,nfam,&
                  dercpdim,nsocstate,nddmult,dbpntgp,nsymop,the_db,sqlvers,&
                  flagarray,infoarray,lgeodb,lpesdb,lapesdb,lgradb,lagradb,&
                  lhesdb,lahesdb,lnactdb,lmocodb,ltransdb,lnmdb,lsymdb,&
                  makeinfoarray,makeflagarray,lcasocdb,qcbasis,dbsymmap,&
                  xref,xatnam,mref,natm,xingeo,ladipdb,lcpesdb,lctransdb,&
                  ldipdb,ddtrfile,lcivecdb,ltemplate,lqcfreq
use dd_db, only: getdbnrec,preparedb,dddb_wr1,prepsym
use db_sql, only: dbopen, dbclose, dbrdrefdb,dbwrrefdb,dbgetnrec,dbgetentries
   
      implicit none

      integer(long)            :: n,ierr,ilbl,laenge,p,ifiletyp,count,&
                                  s,s1,idx,ndb
      integer(long)            :: iau, interp, imethod
      integer(long), external  :: slen
      character(len=c5)        :: filename

      logical(kind=4), save   :: lfail, lfound

      real(dop), dimension(:,:), allocatable     :: v
      real(dop), dimension(:,:,:), allocatable   :: deriv1
      real(dop), dimension(:,:,:,:), allocatable :: deriv2
      real(dop), dimension(:,:), allocatable     :: av
      real(dop), dimension(:,:,:), allocatable   :: aderiv1
      real(dop), dimension(:,:,:,:), allocatable :: aderiv2
      real(sip), dimension(:,:), allocatable     :: mocoeff
      real(dop), dimension(:,:), allocatable     :: nact
      complex(dop), dimension(:,:), allocatable  :: soc
      complex(dop), dimension(:,:), allocatable  :: asoc
      real(dop), dimension(:,:), allocatable     :: adttrans
      character(len=30), dimension(:,:), allocatable :: ciconfig
      real(dop), dimension(:,:,:), allocatable   :: cicoeff


! not used
      integer(long), dimension(:,:), allocatable :: gwpsymrep
      real(dop), dimension(1,1,1) :: dip
      real(dop), dimension(1,1)   :: cadttrans, cv

      filename = ' '

!-----------------------------------------------------------------------
! Energy units for output
! iau = 0 : Hartree and Bohr
!       1 : eV and Angstrom (default)
!-----------------------------------------------------------------------
      iau = 0

!-----------------------------------------------------------------------
! get input file name and file stem
!-----------------------------------------------------------------------
      call getarg(1,filein)
      if (filein .eq. ' ') then
         write(6,'(a)') 'Input file ?'
         read(5,'(a)') filein
      endif
      if (filein .eq. ' ') then
         write(6,'(a)') 'No input file given. STOP.'
         write(6,*)
      endif

      ilbl=index(filein,'.')-1
      if (ilbl .le. 0) then
         ilbl=index(filein,' ')-1
         filein=filein(1:ilbl)//'.inp'
      endif
      filelog  = filein(1:ilbl)//'.log'

!-----------------------------------------------------------------------
! Open input file
!-----------------------------------------------------------------------
      open(iin,file=filein,form='formatted',status='old',iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a)') 'ERROR: Input file not found'
         write(6,'(a)') filein(1:80)
         write(6,*)
         stop
      endif

!-----------------------------------------------------------------------
! Open log file
!-----------------------------------------------------------------------
      open(ilog,file=filelog,form='formatted',status='unknown')
      logisopen=.true.

!-----------------------------------------------------------------------
! Initialise calculation 
! read input file to find number of datasets, maxdim, etc.
!-----------------------------------------------------------------------
      call makedbinp0(iin,filein)

      if (maxdim .lt. 0) then
         routine='MAKEDB'
         message='Error reading dimensions from input file, Nmodes set?'
         call vcerrormsg
      endif
      if (maxsta .lt. 0) then
         routine='MAKEDB'
         message='Error reading no. states from input file, Nstates set?'
         call vcerrormsg
      endif
      if (maxset .lt. 0) then
         routine='MAKEDB'
         message='Error reading no. datasets from input file'
         call vcerrormsg
      else
         print*,'No. Datasets to be read :',maxset
      endif

!-----------------------------------------------------------------------
! set defaults and allocate memory
!-----------------------------------------------------------------------
      allocate(datafile(maxset))
      allocate(gndfile(maxset))
      allocate(abintype(maxset))
      allocate(gndabtype(maxset))
      allocate(dataorient(maxset))
      allocate(gndorient(maxset))
      allocate(eigen(maxset))
      allocate(states(maxsta))

      dercpdim=maxsta*(maxsta-1)/2   ! no. derivative couplings
      call alloc_dirdyn
      call alloc_dddb

!-----------------------------------------------------------------------
! read input file
! define output file etc.
!
! Need e0 in input
!-----------------------------------------------------------------------
      call makedbinp

      if (lwarn) then
         routine = 'MAKEDB'
         message = 'Finished with input error. See log file'
         call vcerrormsg
      endif

!-----------------------------------------------------------------------
! Allocate arrays for DB
!-----------------------------------------------------------------------
      allocate(v(nddstate,nddstate))
      if (lgradb) then
          allocate(deriv1(ndofddpes,nddstate,nddstate))
      else
          allocate(deriv1(1,1,1))
      endif
      if (lhesdb) then
         allocate(deriv2(ndofddpes,ndofddpes,nddstate,nddstate))
      else
         allocate(deriv2(1,1,1,1))
      endif
      if (lapesdb) then
          allocate(av(nddstate,nddstate))
      else
          allocate(av(1,1))
      endif
      if (lagradb) then
          allocate(aderiv1(ndofddpes,nddstate,nddstate))
      else
          allocate(aderiv1(1,1,1))
      endif
      if (lahesdb) then
         allocate(aderiv2(ndofddpes,ndofddpes,nddstate,nddstate))
      else
         allocate(aderiv2(1,1,1,1))
      endif
      if (lmocodb) then
         allocate(mocoeff(nbasis,nbasis))
      else
         allocate(mocoeff(1,1))
      endif
      if (lnactdb) then
         allocate(nact(ndofddpes,dercpdim))
      else
         allocate(nact(1,1))
      endif
      if (lcasocdb) then
          allocate(soc(nsocstate,nsocstate))
      else
          allocate(soc(1,1))
      endif
      if (ltransdb) allocate(adttrans(nddstate,nddstate))
      allocate(gwpsymrep(nsymop,nsymop))
      if (lcivecdb) then
      !   allocate(ciconfig(maxcivec,nsmult))
      !   allocate(cicoeff(maxcivec,maxmultblk,nsmult))
         print*,'Not set up for adding CI Vectors '
         stop
       else
          allocate(ciconfig(1,1))
          allocate(cicoeff(1,1,1))
      endif

!-----------------------------------------------------------------------
! set up DB
!-----------------------------------------------------------------------
      call preparedb(0,1)

!-----------------------------------------------------------------------
! get information from data files
! and write to DB
!-----------------------------------------------------------------------
      laenge=slen(datadir)
      write(6,'(a)') 'Data Directory : '
      write(6,'(5x,a)') datadir(1:laenge)

! Initialise Interface to QC output
      qc_nst = nddstate
      qc_ncoo = ndofddpes
      qc_nbasis = nbasis
      qc_nmode = ndofdd
      qc_nmult(1:7) = nddmult(1:7)
      call alloc_readqc(ilog)

      count=0
      p=nsta      
      ndb = dbnrec ! initial no. of points in DB
      do n=1,nsets
         ndb = ndb+1
         write(6,'(i5,a22,a)') ndb,': Reading data from: ',&
              trim(datafile(n))
         write(ilog,'(i5,a22,a)') &
              ndb,': Reading data from: ',trim(datafile(n))

!-----------------------------------------------------------------------
! zero all values
!-----------------------------------------------------------------------
         v=0.0_dop      
         deriv1=0.0_dop      
         deriv2=0.0_dop      
         av=0.0_dop      
         aderiv1=0.0_dop      
         aderiv2=0.0_dop      
         nact=0.0_dop
         mocoeff=0.0_dop

         dip = 0.0_dop
         call unitqxd(adttrans,nddstate)
         cv=0.0_dop

         nsocstate=nddstate
      
!-----------------------------------------------------------------------
! specify what needs to be read and initialize logical variable
!-----------------------------------------------------------------------
         imethod = abintype(n)
         qcread(:) = .false.
         qcread(3) = .true.                  ! Energy
         qcread(4) = lgradb                  ! 1st Derivs
         qcread(5) = lhesdb                  ! 2nd Derivs if order > 1
         qcread(6) = lmocodb                 ! MO coeffs
         qcread(7) = lnactdb                 ! NACTs
         qcread(11) = lcasocdb               ! SOCs

         lfound=.true.
         lfail=.false.

         call initqcout(lfail)

! Open output file
         filename=datadir(1:laenge)//'/'//trim(datafile(n))
         open(itmpin,file=filename,status='old',err=1003)
         call getintyp(itmpin,ifiletyp,datafile(n),ilog)
       
! Read output file
         call rdqcout(itmpin,ifiletyp,imethod,ilog,lfound,lfail)

         close(itmpin)

         if (.not. lfail) call chkqcout(ifiletyp,ilog,lfound,lfail)

!-----------------------------------------------------------------------
! Load data from QC output into local arrays
!-----------------------------------------------------------------------
         do s=1,nddstate
            if (ldiab) then
               if (lqcfound3(s)) av(s,s) = qcv(s)-dde0
               if (lqcfound4(s)) aderiv1(:,s,s) = qcderiv1(:,s)
               if (lqcfound5(s)) aderiv2(:,:,s,s) = qcderiv2(:,:,s)
            else
               if (lqcfound3(s)) v(s,s) = qcv(s)-dde0
               if (lqcfound4(s)) deriv1(:,s,s) = qcderiv1(:,s)
               if (lqcfound5(s)) deriv2(:,:,s,s) = qcderiv2(:,:,s)
            endif

            do s1=s+1,nddstate
               if (lqcfound7(s,s1)) then
                  idx = (s1-1)*(s1-2)/2 + s
                  nact(:,idx) = qcnact(:,s,s1)
               endif

            enddo
         enddo

!-----------------------------------------------------------------------
! SOCs
!-----------------------------------------------------------------------
         if (lcasocdb) then
            soc = 0.0_dop

            do s=1,nsocstate-1
               do s1=s+1,nsocstate
                  soc(s1,s) = qcsoc(s1,s)
                  soc(s,s1) = qcsoc(s,s1)
               enddo
            enddo
         endif

         if (qcfound(6)) mocoeff(:,:) = qcmocoeff(:,:)

         interp=0
         gwpsymrep=0
! flag that each data set is a unique symmetry replica
         gwpsymrep(1:nsymop,1)=ndb
         soc=0
         call dddb_wr1(qcxcoo,av,aderiv1,aderiv2,dip,v,deriv1,deriv2,&
              dip,mocoeff,adttrans,nact,cv,cadttrans,ciconfig,cicoeff,&
              ndofddpes,nddstate,interp,dercpdim,gwpsymrep,asoc,nsocstate)

      enddo

      call dbgetnrec( the_db, dbnrec )
      print*,'Final No. of Datasets in DB :',dbnrec
      
! Deallocate memory used for DB and close DB
      call dealloc_readqc
      call dealloc_dirdyn
      call dealloc_dddb

      call dbclose(the_db)

      deallocate(datafile,gndfile,abintype,gndabtype)
      deallocate(gwpsymrep)

      stop
 1003 routine = 'MakeDB'
      message(1:80) = 'ERROR opening QC output file :'
      ilbl=index(filename,' ')-1
      message(81:80+ilbl) = filename(1:ilbl)
      call errormsg

      contains

!#######################################################################
!
! read input file
!
! energies in eV, coordinates in bohr
!
!#######################################################################

      subroutine makedbinp

      implicit none

      integer(long)           :: i,iz,iz0,ic,ierr,ilbl,laenge,s,&
                                 nsta1,natmref
      integer(long)           :: linein,ntr,abintype1,gndabtype1
      integer(long)           :: iat,iatiso
      real(dop)               :: quantics_vers
      character(len=c2)       :: currorient
      character(len=c5)       :: filename,blank
      logical :: lnact, lsoc, lmoco
      logical :: lexist
      real(dop) :: angst = 0.52917720859d0

!-----------------------------------------------------------------------
! set initial values
!
! currorient: default orientation for reading geometry (e.g. in Gaussian)
!-----------------------------------------------------------------------
      iz0     = 0
      iz      = 0
      currorient = 'input'
      do i=1,maxset
         dataorient(i)=' '
      enddo
      datadir = './'
      laenge=2
      call abspath(datadir,laenge)
      linein=0
      blank= ' '
      nsta1=0
      order = 0
      lsoc = .false.
      lnact = .false.
      lmoco = .false.
      ldiab = .true.
      nbasis = 0
      nsets = 0
      ddname = './'   ! directory for database.sql file
      ddlaenge=2
      dbpntgp = 0

      abintype1=-999
      abintype(:)=-999
      gndabtype1=-99

      dbnrec = 0

!-----------------------------------------------------------------------
! Open files
!-----------------------------------------------------------------------
      filename=filein
      ilbl=index(filein,'.')-1
      open(iin,file=filename,form='formatted',status='old',iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a)') 'ERROR: Input file not found'
         write(6,'(a)') filein(1:80)
         write(6,*)
         stop
      endif

!-----------------------------------------------------------------------
! read input file
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'makedbinp (read MAKEDB input)'
      call vccntrl(ierr)

      i=1
   15 continue
      if (keyword(i) .eq. 'end-input') then
         goto 20

      else if (keyword(i) .eq. 'orient') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            currorient=keyword(i)(1:c2)
         else
            routine = 'makedbinp (read MAKEDB input)'
            message = 'No argument specified with orient keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'datadir') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            datadir = keyorig(i)
            laenge = lc(i)
            if (laenge .eq. 0) then
               routine = 'makedbinp (read MAKEDB input)'
               message = ' No path name specified'
               call vcwrline(0,iz,iz0)
            end if
            call abspath(datadir,laenge)
         else
            routine = 'makedbinp (read MAKEDB input)'
            message = 'No argument specified with datadir keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'files') then
         if (i .ne. 1) then
            routine = 'makedbinp (read MAKEDB input)'
            message='Files keyword must be alone on line'
            call vcwarnmsg
         else
            call rdsets(iin,currorient,iz,iz0,filein)
         endif

      else if (keyword(i) .eq. 'datasets') then
         if (i .ne. 1) then
            routine = 'makedbinp (read MAKEDB input)'
            message='Dataset keyword must be alone on line'
            call vcwarnmsg
         else
  120       continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            routine = 'makedbinp (read MAKEDB input)'
            call vccntrl(ierr)

            if (keyword(1) .ne. 'end-datasets') then
               filename=keyorig(1)
               open(idat,file=filename,form='formatted',status='old',&
                  iostat=ierr)
               if (ierr .ne. 0) then
                  routine = 'makedbinp (read MAKEDB input)'
                  write(message,'(a)') 'Data File not found: '
                  write(message1,'(a)') filename
                  call vcwarnmsg
               else
                  write(ilog,'(2a)') 'Opening Dataset:',filename(1:60)
                  iz  = 0  
                  iz0 = 0
                  call rdsets(idat,currorient,iz,iz0,filename)
                  close(idat)
               endif
               go to 120
            else
               write(ilog,*) 
            endif
         endif

      else if (keyword(i) .eq. 'energy0') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) dde0
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keyeunits(dde0,keyword(i),ierr)
               if (ierr .ne. 0) then
                  routine = 'makedbinp (read MAKEDB input)'
                  message = 'Incorrect unit for energy : '
                  call vcwrline(1,iz,iz0)
               endif
            endif

         else
            routine = 'makedbinp (read MAKEDB input)'
            message = 'No argument specified with energy0 keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'nmodes') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) nmodes
            if (keyword(i+1) .eq. ',') then
               i=i+2
               read(keyword(i),*) ntr
            else
               ntr = 6
            endif
            ncoo=nmodes+ntr
            natm = ncoo/3
         else
            routine = 'makedbinp (read MAKEDB input)'
            message = 'No argument specified with nmodes keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'order') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) order
         else
            routine = 'makedbinp (read MAKEDB input)'
            message = 'No argument specified with order keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'soc') then
         lsoc = .true.

      else if (keyword(i) .eq. 'nact') then
         lnact = .true.

      else if (keyword(i) .eq. 'mocoeffs') then
         lmoco = .true.

      else if (keyword(i).eq.'nbasis') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) nbasis

         else
            routine = 'makedbinp (read MAKEDB input)'
            message = 'No argument specified with nbasis keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'abinitiotype') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            call getabintype(keyword(i),abintype1,ilog)

         else
            routine = 'makedbinp (read MAKEDB input)'
            message = 'No argument specified with abinitiotype keyword'
            call vcwrline(0,iz,iz0)
         endif

!      else if (keyword(i).eq.'transition_dipoles') then
!         ltdip=.true.

!--------------------------------------------------------------------

      else if (keyword(i).eq.'states') then
         if (keyword(i+1).eq.'=') then
            s=0
   60       continue
            s=s+1
            i=i+2
            read(keyword(i),*,iostat=ierr) states(s)
            if (ierr .ne. 0) then
               routine = 'makedbinp (read MAKEDB input)'
               message ='states must be an integer. Not :'
               call vcwrline(i,iz,iz0)
            endif

            if (keyword(i+1).eq.',') go to 60
            nsta = s

         else
            routine = 'makedbinp (read MAKEDB input)'
            message = 'No argument specified with states keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'nstates') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) nsta1
         else
            routine = 'makedbinp (read MAKEDB input)'
            message = 'No argument specified with nstates keyword'
            call vcwrline(0,iz,iz0)
         endif

       else if (keyword(1) .eq. 'reference') then
          natmref=0
          iau = 0  
          do 
             call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
             if (keyword(1) .eq. 'end-reference') exit
             backspace(iin)
             natmref=natmref+1
             read(iin,*,err=999) xatnam(natmref),(xref(i,natmref),i=1,3)
          enddo
          xref=xref/angst   ! coords in au
          do n=1,natmref
             call getinam(xatnam(n),iat,iatiso,ilog)
             call getmass(mref(n),iat,iau,ilog)
          enddo

      else if (keyword(i) .eq. 'db_point_group') then
         if (keyword(i+1) .eq. '=') then
            i=i+2
            dbpntgp = fpntgp(keyword(i))
            if (dbpntgp .eq. -99) then
               routine = 'makedbinp (read MAKEDB input)'
               write(message,'(3a)') 'Error reading DB_POINT_GROUP keyword. ', &
                   'Unknown point group : ',keyword(i)(1:10)
               call vcwrline(0,iz,iz0)
            endif
         else
            routine = 'makedbinp (read MAKEDB input)'
            message = 'No argument specified with DB_POINT_GROUP keyword'
            call vcwrline(0,iz,iz0)
         endif

      else
         routine = 'makedbinp (read MAKEDB input)'
         message = ' unknown keyword : '
         call vcwrline(i,iz,iz0)

      endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

 20   continue

!-----------------------------------------------------------------------
! No. of coordinates
!-----------------------------------------------------------------------
      ndofddpes = ncoo
      ndofdd = nmodes

!-----------------------------------------------------------------------
! If states keyword not used, include all states up to nstates
!-----------------------------------------------------------------------
      if (nsta .eq. 0) then
         if (nsta1 .eq. 0) then
            routine='Vctinp'
            write(message,'(a)') 'Set the no. of states to be read'
            call vcerrormsg
         endif
       
         nsta = nsta1
         do s=1,nsta
            states(s)=s
         enddo
      endif
      nddstate = nsta

      nroot = nddstate
      nsocstate = nddstate

      nddmult(:)=0
      nddmult(1)=nddstate ! make all singlets

!-----------------------------------------------------------------------
! check if ab initio type has been set for file
!-----------------------------------------------------------------------
      do n=1,nsets
         if (abintype(n) .lt. 0) abintype(n) = abintype1
      enddo

!-----------------------------------------------------------------------
! check if ab initio type has been set for declared ground-state files
! -- required by OVGF and ADC calculations
!-----------------------------------------------------------------------
      do n=1,nsets
         if (gndabtype(n) .eq. -99) gndabtype(n) = gndabtype1
      enddo

!-----------------------------------------------------------------------
! check various flags set
!-----------------------------------------------------------------------
      if( nsets .le. 0 )then
         write(6, '(a)')'*** no data sets specified! ***'
         stop
      end if
      if(nsta.eq.0)then 
         write(6,'(a)') 'You must specify the number of states!'
         stop
      endif
      if(nmodes.eq.0)then 
         write(6,'(a)') 'You must specify the number of modes!'
         stop
      endif

!-----------------------------------------------------------------------
! output file name
!-----------------------------------------------------------------------
      call abspath(ddname,ddlaenge)
      fileout=ddname(1:ddlaenge)//"/database.sql"

!-----------------------------------------------------------------------
! log settings
!-----------------------------------------------------------------------
      write(6,*)
      write(6,'(a26,a)') 'Output file             : ',trim(fileout)
      write(ilog,*)
      write(ilog,'(a26,a)') 'Output file             : ',trim(fileout)

      write(6,'(a26,i10)')  'Number of dataset       : ',maxset
      write(ilog,'(a26,i10)')  'Number of dataset       : ',maxset

!-----------------------------------------------------------------------
! Open output DB and create refdb if new.
!-----------------------------------------------------------------------
      lexist = .false.
      inquire(file=fileout,exist=lexist)
      if (lexist) then  ! Add to existing DB
         if (.not.the_db%db_open) &
               call dbopen(fileout, the_db,sqlvers,quantics_vers,infoarray,&
                    flagarray, 1)

         write(6,'(/,a)') '*** Opening Old SQL DB  ***'

         call dbgetnrec( the_db, dbnrec )
         write(ilog,'(/,a)') '*** Opening Old SQL DB  ***'
         write(ilog,'(a,i0)') 'No. of Datasets in DB :',dbnrec

!Get what is in DB. Geometries always present.
         lgeodb=.true.
         call dbgetentries(the_db, ilog, lpesdb, lgradb, lhesdb, lapesdb, &
            lagradb, lahesdb, ldipdb, ladipdb, lmocodb, lnactdb, ltransdb, &
            lnmdb, lcasocdb, lsymdb, lcpesdb, lctransdb, lcivecdb, ltemplate, &
            lqcfreq )

      else
!-----------------------------------------------------------------------
! get data to be read
!-----------------------------------------------------------------------
         lgeodb=.true.
         lpesdb=.true.
         if (order .ge. 1) lgradb=.true.
         if (order .ge. 2) lhesdb=.true.
         if (lnact .and. nddstate .gt. 1) then
            ldiab = .true.
            lnactdb=.true.
            lapesdb=.true.
            ltransdb=.true.
            if (lgradb) lagradb=.true.
            if (lhesdb) lahesdb=.true.
         endif
         if (lmoco) lmocodb=.true.
         if (lsoc) lcasocdb=.true.
         lnmdb = .true.
      
         if (dbpntgp .gt. 0) then
            lsymdb = .true.
            write(6,'(a)') 'Adding symmetry with makedb does not work yet'
            stop
         else
            lsymdb = .false.
         endif

         write(ilog,'(/,a)') '*** Opening New SQL DB  ***'
! set reference geometry and prepare symmetry maps
         xingeo = xref
         call prepsym

         call makeflagarray
         call makeinfoarray(nddstate)
         if (.not.the_db%db_open) &
               call dbopen(fileout, the_db,sqlvers,quanticsver,infoarray,&
                    flagarray, 2)

         if (natmref .ne. natm) then
            routine = 'Makedbinp (MakeDB)'
            message = 'Inconsistent atom numbers : compare reference to system'
            call errormsg
         endif
         dimref=0
         nfam = 0

         call dbwrrefdb(the_db,natm, nroot, nddstate, nbasis, dde0, &
                        dimref, nfam, mref, xatnam, xref, dercpdim, &
                        dbpntgp, nsymop, nsocstate, dbsymmap, nddmult, &
                        qcprog, qcmethod, qcbasis, ddtrfile,ddname,ddlaenge)
      endif

! close input file
      close(iin)

      return
  999 continue
      routine = 'Rdsets (Makedb)'
      message = 'Error reading reference geometry'
      call errormsg
      end subroutine makedbinp

!#######################################################################

      subroutine makedbinp0(unit,filename)

implicit none

      integer(long)                 :: ic,iz,iz0,ierr,siz,nsetm,i,ntr
      integer(long), intent(in)     :: unit
      character(len=80), intent(in) :: filename
      character(len=c5)             :: inptit
      character(len=20)             :: string
      logical                       :: fset, dset

      maxsta = -999
      maxdim = -999
      maxset = -999
      fset=.false.
      dset = .false.
      siz=0
      nsetm=0
      open(unit,file=filename,form='formatted',status='old',iostat=ierr)

   200 continue
      call rdinpf(unit,keyword,keyorig,inptit,lc,ic,siz,ierr,maxkey)
      if (keyword(1) .eq. 'end-input') go to 205  ! finish
      
      i = 0
      do 
         i=i+1
         if (i .gt. ic) exit
         if (keyword(i).eq.'nstates') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               read(keyword(i),*) maxsta
            else
               routine = 'makedbinp0 (read MAKEDB input)'
               message = 'No argument specified with nstates keyword'
               call vcwrline(0,iz,iz0)
            endif

         else if (keyword(i) .eq. 'nmodes') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               read(keyword(i),*) maxdim
               if (keyword(i+1) .eq. ',') then
                  i=i+2
                  read(keyword(i),*) ntr
               else
                  ntr = 6
               endif
               maxdim=maxdim+ntr
            else
               routine = 'makedbinp0 (read MAKEDB input)'
               message = 'No argument specified with nmodes keyword'
               call vcwrline(0,iz,iz0)
            endif

         else if (keyword(i) .eq. 'files') then
            fset=.true.

         else if (keyword(i) .ne. 'end-files' .and. fset ) then
            nsetm=nsetm+1    ! one data set per line

         else if (keyword(i) .eq. 'datasets') then
            dset = .true.

         else if (keyword(i) .ne. 'end-files' .and. dset ) then
            !Load dataset file and count through
            open(itmpin,file=keyorig(1),form='formatted',status='old',iostat=ierr)
            if (ierr .eq. 0) then
               do
                  read (itmpin,'(a)', end=10) string
                  if (string .eq. 'end-files') exit
                  if (string(1:1) .ne. '#') nsetm = nsetm + 1   ! one data set per line
               end do
               10 close (itmpin)
            endif 
      
         else if (keyword(i) .eq. 'end-files') then
            fset = .false.

         end if
      enddo

      go to 200

   205 continue
      close(unit)

! set maximum number of sets to be read
      maxset=nsetm

      return
      end subroutine makedbinp0

!#######################################################################

      subroutine rdsets(unit,currorient,iz,iz0,filename)

      implicit none

      integer(long)                 :: i,ic,ierr,siz,siz0
      integer(long), intent(in)     :: unit,iz
      integer(long), intent(inout)  :: iz0
      character(len=c2), intent(in) :: currorient
      character(len=c5), intent(in) :: filename

      siz=iz
      siz0=iz0

  100 continue
      call rdinpf(unit,keyword,keyorig,inptit,lc,ic,siz,ierr,maxkey)
      routine = 'rdsets  '//filename(1:60)
      call vccntrl(ierr)

      if (keyword(1) .ne. 'end-files') then
         nsets=nsets+1
         if (nsets .gt. maxset) then
            message = 'Nsets to be read > Maxset !'
            call vcerrormsg
         endif

         datafile(nsets) = keyorig(1)

!-----------------------------------------------------------------------
! set defaults for properties of data files 
!-----------------------------------------------------------------------
         dataorient(nsets) = currorient
         gndfile(nsets) = ' '
         gndorient(nsets) = currorient
         gndabtype(nsets) = -999

!-----------------------------------------------------------------------
! read any further keywords related to file
!-----------------------------------------------------------------------
         i=2
  105    continue
         i=i+1
         if (keyword(i) .eq. 'orient') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               dataorient(nsets) = keyword(i)(1:32)
            else
               message = 'No argument specified with orient keyword'
               call vcwrline(0,siz,siz0)
            endif

         else if (keyword(i) .eq. 'root') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               read(keyword(i),*) eigen(nsets) 
            else
               message = 'No argument specified with root keyword'
               call vcwrline(0,siz,siz0)
            endif

         else if (keyword(i) .eq. 'abintype') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               call getabintype(keyword(i),abintype(nsets),ilog)
            else
               message = 'No argument specified with abintype keyword'
               call vcwrline(0,siz,siz0)
            endif

         else if (keyword(i) .eq. 'gndfile') then
! change gndabtype if not set so that presence of gndfile is flagged
            if (gndabtype(nsets) .eq. -999) gndabtype(nsets)=-99
            if (keyword(i+1).eq.'=') then
               i=i+2
               gndfile(nsets) = keyorig(i)
            else
               message = 'No argument specified with gndfile keyword'
               call vcwrline(0,siz,siz0)
            endif

         else if (keyword(i) .eq. 'gndabtype') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               call getabintype(keyword(i),gndabtype(nsets),ilog)
            else
               message = 'No argument specified with gndabtype keyword'
               call vcwrline(0,siz,siz0)
            endif

         else if (keyword(i) .eq. 'gndorient') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               gndorient(nsets) = keyword(i)(1:32)
            else
               message = 'No argument specified with gndorient keyword'
               call vcwrline(0,siz,siz0)
            endif

         endif
         if (i .lt. ic) then
            go to 105
         else
            go to 100
         endif
      endif

      return
      end subroutine rdsets

!#######################################################################

  end program makedb
