C#######################################################################
C
C LINMIN
C line minimiser
C
C p : point (coordinates)
C xi : gradient at p
C n : no. of coordinates
C fret: returned function
C func: routine for calculating function to be minimised
C
C#######################################################################

      SUBROUTINE linmin(p,xi,n,fret,func,rdata,idata)
      INTEGER n,NMAX,idata(*)
      REAL*8 fret,p(n),xi(n),TOL,func,ratio,rdata(*)
      PARAMETER (NMAX=5000,TOL=1.e-4)
CU    USES brent,f1dim,mnbrak
      INTEGER j,ncom
      REAL*8 ax,bx,fa,fb,fx,xmin,xx,pcom(NMAX),xicom(NMAX),brent,
     +     f1dim
      COMMON /f1com/ pcom,xicom,ncom
      EXTERNAL f1dim,func
      if (n .gt. nmax) then
         write(6,'(a,i5)') 'Redimension NMAX in linmin. Minimum :',n
         write(6,'(a)') 'Check dimensions in f1dim'
         stop
      endif

C store initial vectors in common block
      ncom=n
      do 11 j=1,n
         pcom(j)=p(j)
         xicom(j)=xi(j)
11    continue

      ax=0.
      fa = func(p,rdata,idata)
 
C xx is step size. New func is calculated at pcom + xx*xicom
      xx=1.

C check that fx is not too large
 8    fx = f1dim(xx,func,rdata,idata)
      if (abs(fx) .gt. abs(10.0*fa)) then
         xx = 0.5*xx
         go to 8
      endif

      call mnbrak(ax,xx,bx,fa,fx,fb,f1dim,func,rdata,idata)
      fret=brent(ax,xx,bx,f1dim,TOL,xmin,func,rdata,idata)
      do 12 j=1,n
        xi(j)=xmin*xi(j)
        p(j)=p(j)+xi(j)
12    continue
      return
      END

C#######################################################################

      FUNCTION brent(ax,bx,cx,f,tol,xmin,func,rdata,idata)
      INTEGER ITMAX
      REAL*8 brent,ax,bx,cx,tol,xmin,f,CGOLD,ZEPS,func
      EXTERNAL f,func
      PARAMETER (ITMAX=100,CGOLD=.3819660,ZEPS=1.0e-10)
      INTEGER iter,idata(*)
      REAL*8 a,b,d,e,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm,
     +       rdata(*)
      a=min(ax,cx)
      b=max(ax,cx)
      v=bx
      w=v
      x=v
      d=0.
      e=0.
      fx=f(x,func,rdata,idata)
      fv=fx
      fw=fx
      do 11 iter=1,ITMAX
        xm=0.5*(a+b)
        tol1=tol*abs(x)+ZEPS
        tol2=2.*tol1
        if(abs(x-xm).le.(tol2-.5*(b-a))) goto 3
        if(abs(e).gt.tol1) then
          r=(x-w)*(fx-fv)
          q=(x-v)*(fx-fw)
          p=(x-v)*q-(x-w)*r
          q=2.*(q-r)
          if(q.gt.0.) p=-p
          q=abs(q)
          etemp=e
          e=d
          if(abs(p).ge.abs(.5*q*etemp).or.p.le.q*(a-x).or.p.ge.q*(b-x)) 
     *goto 1
          d=p/q
          u=x+d
          if(u-a.lt.tol2 .or. b-u.lt.tol2) d=sign(tol1,xm-x)
          goto 2
        endif
1       if(x.ge.xm) then
          e=a-x
        else
          e=b-x
        endif
        d=CGOLD*e
2       if(abs(d).ge.tol1) then
          u=x+d
        else
          u=x+sign(tol1,d)
        endif
        fu=f(u,func,rdata,idata)
        if(fu.le.fx) then
          if(u.ge.x) then
            a=x
          else
            b=x
          endif
          v=w
          fv=fw
          w=x
          fw=fx
          x=u
          fx=fu
        else
          if(u.lt.x) then
            a=u
          else
            b=u
          endif
          if(fu.le.fw .or. w.eq.x) then
            v=w
            fv=fw
            w=u
            fw=fu
          else if(fu.le.fv .or. v.eq.x .or. v.eq.w) then
            v=u
            fv=fu
          endif
        endif
11    continue
      stop 'brent exceed maximum iterations'
3     xmin=x
      brent=fx
      return
      END

C#######################################################################

      SUBROUTINE mnbrak(ax,bx,cx,fa,fb,fc,f1dim,func,rdata,idata)

      integer idata(*)
      REAL*8 ax,bx,cx,fa,fb,fc,func,GOLD,GLIMIT,TINY,f1dim,
     +       rdata(*)
      EXTERNAL func
      PARAMETER (GOLD=1.618034, GLIMIT=100., TINY=1.e-20)
      REAL*8 dum,fu,q,r,u,ulim

      integer funcerr
      common /brakerr/ funcerr

      if(fb.gt.fa)then
        dum=ax
        ax=bx
        bx=dum
        dum=fb
        fb=fa
        fa=dum
      endif
      cx=bx+GOLD*(bx-ax)
      fc=f1dim(cx,func,rdata,idata)
1     if(fb.ge.fc)then
        r=(bx-ax)*(fb-fc)
        q=(bx-cx)*(fb-fa)
        u=bx-((bx-cx)*q-(bx-ax)*r)/(2.*sign(max(abs(q-r),TINY),q-r))
        ulim=bx+GLIMIT*(cx-bx)
        if((bx-u)*(u-cx).gt.0.)then
          fu=f1dim(u,func,rdata,idata)
          if(fu.lt.fc)then
            ax=bx
            fa=fb
            bx=u
            fb=fu
            return
          else if(fu.gt.fb)then
            cx=u
            fc=fu
            return
          endif
          u=cx+GOLD*(cx-bx)
          fu=f1dim(u,func,rdata,idata)
        else if((cx-u)*(u-ulim).gt.0.)then
          fu=f1dim(u,func,rdata,idata)
          if(fu.lt.fc)then
            bx=cx
            cx=u
            u=cx+GOLD*(cx-bx)
            fb=fc
            fc=fu
            fu=f1dim(u,func,rdata,idata)
          endif
        else if((u-ulim)*(ulim-cx).ge.0.)then
          u=ulim
          fu=f1dim(u,func,rdata,idata)
        else
          u=cx+GOLD*(cx-bx)
          fu=f1dim(u,func,rdata,idata)
        endif
        ax=bx
        bx=cx
        cx=u
        fa=fb
        fb=fc
        fc=fu
        goto 1
      endif
      return
      END

C#######################################################################

      FUNCTION f1dim(x,func,rdata,idata)

      INTEGER NMAX,idata(*)
      REAL*8 f1dim,func,x,rdata(*)
      external func
      PARAMETER (NMAX=5000)
CU    USES func
      INTEGER j,ncom
      REAL*8 pcom(NMAX),xicom(NMAX),xt(NMAX)
      COMMON /f1com/ pcom,xicom,ncom

      do 11 j=1,ncom
        xt(j)=pcom(j)+x*xicom(j)
11    continue

      f1dim=func(xt,rdata,idata)
      return
      END

