!#######################################################################
      module output_vcham

use decimal, only: long,dop

use vcglobal
use global, only:pi, fs, ev
use maxv, only: maxdim,maxsta
use vchammod
use vchfitmod

use logdat, only: message, routine, inptit
use utils, only: q2x,numpar,numtpar,vcerrormsg

      implicit none
      private
      public :: outmask,outstat,writepar,writeop,writeqinp
      contains

!#######################################################################
!
! if parameter is constrained, then output a mask value of -1 rather 
! than the constraint number stored in the mask
!
!#######################################################################

      subroutine outmask(const_mask,kappa_mask,lambda_mask,&
              gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
              cpcoe_mask,switch_mask)

      implicit none

      integer(long)                                                 :: i,s,j,j1,j2,j3,nblck,n,s1,k,n1,n2
      integer(long), dimension(:), allocatable                      :: imask
      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in)                    :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe)                      :: switch_mask
      real(dop), parameter :: eps=0.00001_dop

      i=max(maxdim,nsta+1)
      allocate(imask(i))

      if( i_ignore .gt. 0 ) nmodes = nmodes - i_ignore

      write(iout,*)
      write(iout,'(/,a)')&
           '*** THE MASK FOR THE VC-HAMILTONIAN ***'

      write(iout,'(/,a)') &
          '*** Constants mask ***'
      write(iout,'(2x,a6,10i3)') 'State :',(s,s=1,nsta-1)
      do s=1,nsta
         do s1=1,s
            if (const_mask(s,s1) .lt. 0) then
               n1=abs(const_mask(s,s1))
               n2=parcons(n1)
               if (n2 .gt. 0) then
                  imask(s1)=-1
               else
                  imask(s1)=-2
               endif
            else
               imask(s1)=const_mask(s,s1)
            endif
         enddo
         write(iout,'(i5,2x,10i3)') s,(imask(s1),s1=1,s)
      enddo

      write(iout,'(/,a)') &
          '*** On-diagonal linear (kappa) mask ***'
      do s=1,nsta
         write(iout,'(4x,a6,7(6x,i1,3x))') 'State: ',s
         do i=1,nmodes
            if (kappa_mask(adof(i),s) .lt. 0) then
               n1=abs(kappa_mask(adof(i),s))
               n2=parcons(n1)
               if (n2 .gt. 0) then
                  imask(1)=-1
               else
                  imask(1)=-2
               endif
            else
               imask(1)=kappa_mask(adof(i),s)
            endif
            write(iout,'(i3,2x,a5,i3)') adof(i),nmlab(adof(i)),imask(1)
         enddo
      enddo

      write(iout,'(/,a)') &
          '*** Off-diagonal linear (lambda) mask ***'
      do s=1,nsta-1
         do s1=s+1,nsta
            write(iout,'(4x,a6,2(6x,i1,3x))') 'State:',s,s1
            do i=1,nmodes
               if (lambda_mask(adof(i),s,s1) .lt. 0) then
                  n1=abs(lambda_mask(adof(i),s,s1))
                  n2=parcons(n1)
                  if (n2 .gt. 0) then
                     imask(1)=-1
                  else
                     imask(1)=-2
                  endif
               else
                  imask(1)=lambda_mask(adof(i),s,s1)
               endif
               write(iout,'(i3,2x,a5,i3)') adof(i),nmlab(adof(i)),imask(1)
            enddo
         enddo
      enddo

! if linear model do not output 2nd order coefficients
      if (hamord .eq. 1) go to 100

      write(iout,'(/,a)') '*** On-diagonal bilinear (gamma) mask ***'
      nblck=((nmodes-1)/6)+1
      do s=1,nsta
         write(iout,'(4x,a18,6x,i1)') 'State: ', s
         do n=1,nblck
            j1=(n-1)*6+1
            j2=min(j1+5,nmodes)
            write(iout,'(10x,6(5x,i2,3x))') (adof(j),j=j1,j2)
            do i=j1,nmodes
               j3=min(i,j2)
               do j=j1,j3
                  if (gamma_mask(adof(i),adof(j),s) .lt. 0) then
                     n1=abs(gamma_mask(adof(i),adof(j),s))
                     n2=parcons(n1)
                     if (n2 .gt. 0) then
                        imask(j)=-1
                     else
                        imask(j)=-2
                     endif
                  else
                     imask(adof(j))=gamma_mask(adof(i),adof(j),s)
                  endif
               enddo
               write(iout,20) adof(i),nmlab(adof(i)),(imask(adof(j)),j=j1,j3)
            enddo
         enddo
      enddo
      write(iout,*)
      
   20 format(i3,a5,6i10)

! if 2nd order adiabatic model do not output mu coefficients
      if (hamord .eq. 2) go to 100

      write(iout,'(/,a)') '*** Off-diagonal bilinear (mu) mask ***'
      nblck=((nmodes-1)/6)+1
      do s=1,nsta-1
         do s1=s+1,nsta
            write(iout,'(4x,a6,2(6x,i1,3x))') 'State:',s,s1
            do n=1,nblck
               j1=(n-1)*6+1
               j2=min(j1+5,nmodes)
               write(iout,'(10x,6(5x,i2,3x))') (adof(j),j=j1,j2)
               do i=j1,nmodes
                  j3=min(i,j2)
                  do j=j1,j3
                     if (mu_mask(adof(i),adof(j),s,s1) .lt. 0) then
                        n1=abs(mu_mask(adof(i),adof(j),s,s1))
                        n2=parcons(n1)
                        if (n2 .gt. 0) then
                           imask(adof(j))=-1
                        else
                           imask(adof(j))=-2
                        endif
                     else
                        imask(adof(j))=mu_mask(adof(i),adof(j),s,s1)
                     endif
                  enddo
                  write(iout,20) adof(i),nmlab(adof(i)),(imask(adof(j)),j=j1,j3)
               enddo
            enddo
         enddo
      enddo
      write(iout,*)

      if (hamord .eq. 3) go to 100

      write(iout,'(/,a)') &
             '*** On-diagonal quadratic-linear (iota) mask ***'
      nblck=((nmodes-1)/6)+1
      do s=1,nsta
         write(iout,'(4x,a18,6x,i1)') 'State: ', s
         do n=1,nblck
            j1=(n-1)*6+1
            j2=min(j1+5,nmodes)
            write(iout,'(10x,6(5x,i2,3x))') (j,j=j1,j2)
            do i=j1,nmodes-i_ignore
               j3=j2
               do j=j1,j3
                  if (iota_mask(adof(i),adof(j),s,s) .lt. 0) then
                     n1=abs(iota_mask(adof(i),adof(j),s,s))
                     n2=parcons(n1)
                     if (n2 .gt. 0) then
                        imask(adof(j))=-1
                     else
                        imask(adof(j))=-2
                     endif
                  else
                     imask(adof(j))=iota_mask(adof(i),adof(j),s,s)
                  endif
               enddo
               write(iout,21) adof(i),nmlab(adof(i)),(imask(adof(j)),j=j1,j3)
            enddo
         enddo
      enddo
      write(iout,*)

   21 format(i3,a5,6i10)

      write(iout,'(/,a)') &
         '*** Off-diagonal quadratic-linear (iota2) mask ***'
      nblck=((nmodes-1)/6)+1
      do s=1,nsta-1
         do s1=s+1,nsta
            write(iout,'(4x,a6,2(6x,i1,3x))') 'State:',s,s1
            do n=1,nblck
               j1=(n-1)*6+1
               j2=min(j1+5,nmodes)
               write(iout,'(10x,6(5x,i2,3x))') (j,j=j1,j2)
               do i=j1,nmodes
                  j3=j2
                  do j=j1,j3
                     if (iota_mask(adof(i),adof(j),s,s1) .lt. 0) then
                        n1=abs(iota_mask(adof(i),adof(j),s,s1))
                        n2=parcons(n1)
                        if (n2 .gt. 0) then
                           imask(adof(j))=-1
                        else
                           imask(adof(j))=-2
                        endif
                     else
                        imask(adof(j))=iota_mask(adof(i),adof(j),s,s1)
                     endif
                  enddo
                  write(iout,20) adof(i),nmlab(adof(i)),(imask(adof(j)),j=j1,j3)
               enddo
            enddo
         enddo
      enddo

  100 continue
      if (nadd .gt. 0) then
         write(iout,'(/,a)')&
            '*** Explicitly added polynomial term mask ***'
         write(iout,'(i5)') nadd
         do n = 1,nadd
            if (termk_mask(n).lt. 0) then
               n1=abs(termk_mask(n))
               n2=parcons(n1)
               if (n2 .gt. 0) then
                  imask(1)=-1
               else
                  imask(1)=-2
               endif
            else
               imask(1)=termk_mask(n)
            endif

            write(iout,'(2I5)') n, imask(1)
         enddo
      endif
      write(iout,*)
            
      if (.not. lgeneral) go to 200
      write(iout,'(/,a)') &
          '*** General diabatic potential mask ***'
      do s=1,nsta
         write(iout,'(4x,a6,7(6x,i1,3x))') 'State: ',s
         do i=1,nmodes
            n=max(1,ncoeff(adof(i),s))
            do k=1,n
               if (expcoe_mask(k,adof(i),s).lt. 0) then
                  n1=abs(expcoe_mask(k,adof(i),s))
!                  print*,expcoe_mask(k,i,s),k,i,s
!                  print*,n1
!                  print*,parcons(n1)
                  n2=parcons(n1)
                  if (n2 .gt. 0) then
                     imask(k)=-1
                  else
                     imask(k)=-2
                  endif
               else
                  imask(k)=expcoe_mask(k,adof(i),s)
               endif
            enddo
            write(iout,'(i3,2x,a5,10i3)') adof(i),nmlab(adof(i)),(imask(k),k=1,n)
         enddo
      enddo

      write(iout,*)
            
  200 continue

      if( i_ignore .gt. 0 ) nmodes = nmodes + i_ignore
      if (ncons .gt. 0) then
         write(iout,'(/,a)') 'CONSTRAINTS-SECTION'
         call outcons(const_mask,kappa_mask,lambda_mask,gamma_mask,&
              iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask,&
              switch_mask)
         write(iout,'(a)') 'END-CONSTRAINTS-SECTION'
      endif

      deallocate(imask)

      return
      end subroutine outmask

!#######################################################################

      subroutine outcons(const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask)

      implicit none

      integer(long)              :: n1,n2,type,md,st,md1,st1,koe,i,j,k,l,attermk
      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,&
                                                                       mu_mask
      integer(long), dimension(nadd), intent(in)                    :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe)                      :: switch_mask
      integer(long)           :: ilbl,jlbl,klbl,m,n
      character(len=60)       :: string
      character(len=10)       :: atmp,atmp1

      do i=1,nsta
         do j=1,i
            if (const_mask(j,i).lt.0) then
               n1=abs(const_mask(j,i))
               n2=parcons(n1)
               if (n2 .gt. 0) then
                  call numpar(n2,type,md,st,md1,st1,koe,attermk,&
                       const_mask,kappa_mask,lambda_mask,gamma_mask,&
                       iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask)
               else
                  call numtpar(n2,type,md,st,md1,st1,koe,attermk,&
                       const_mask,kappa_mask,lambda_mask,gamma_mask,&
                       iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask,&
                       switch_mask)
               endif
               if (type .ne. 0) then
                 if( .not. lgenetica) &
                  write(ilog,'(a)')&
                    'WARNING: Inconsistent constraint definition'
               endif

               write(string,'(2(a,2i3))') 'E ',i,j,' = E ',st,st1
               write(iout,'(a,f12.6)') string, pconfact(n1)
            endif
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            if (kappa_mask(j,i).lt.0) then
               n1=abs(kappa_mask(j,i))
               n2=parcons(n1)
               if (n2 .gt. 0) then
                  call numpar(n2,type,md,st,md1,st1,koe,attermk,&
                       const_mask,kappa_mask,lambda_mask,gamma_mask,&
                       iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask)
               else
                  call numtpar(n2,type,md,st,md1,st1,koe,attermk,&
                       const_mask,kappa_mask,lambda_mask,gamma_mask,&
                       iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask,&
                       switch_mask)
               endif
               if (type .ne. 1 .and. type .ne. 2) then
                 if( .not. lgenetica) &
                  write(ilog,'(a)')&
                    'WARNING: Inconsistent constraint definition'
               endif

               if (type .eq. 1) then
                  write(string,'(2(a,i3,i3))') &
                       'kappa ',j,i,' = kappa ',md,st
               else if (type .eq. 2) then
                  write(string,'(2(a,i3,i3),i3)') &
                       'kappa ',j,i,' = lambda ',md,st,st1
               endif
               write(iout,'(a,f12.6)') string, pconfact(n1)

            endif
         enddo
      enddo

      do i=1,nsta-1
         do k=i+1,nsta
            do j=1,nmodes
               if (lambda_mask(j,i,k).lt.0) then
                  n1=abs(lambda_mask(j,i,k))
                  n2=parcons(n1)
                  if (n2 .gt. 0) then
                     call numpar(n2,type,md,st,md1,st1,koe,attermk,&
                          const_mask,kappa_mask,lambda_mask,gamma_mask,&
                          iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask)
                  else
                     call numtpar(n2,type,md,st,md1,st1,koe,attermk,&
                          const_mask,kappa_mask,lambda_mask,gamma_mask,&
                          iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask,&
                          switch_mask)

                  endif
                  if (type .ne. 1 .and. type .ne. 2) then
                 if( .not. lgenetica) &
                     write(ilog,'(a)')&
                       'WARNING: Inconsistent constraint definition'
                  endif

                  if (type .eq. 1) then
                     write(string,'(2(a,i3,i3,i3))')&
                       'lambda ',j,i,k,' = kappa ',md,st
                  else if (type .eq. 2) then
                     write(string,'(2(a,i3,i3,i3))')&
                       'lambda ',j,i,k,' = lambda ',md,st,st1
                  endif
                  write(iout,'(a,f12.6)') string, pconfact(n1)

               endif
            enddo
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            do k=j,nmodes
               if (gamma_mask(k,j,i).lt.0) then
                  n1=abs(gamma_mask(k,j,i))
                  n2=parcons(n1)
                  if (n2 .gt. 0) then
                     call numpar(n2,type,md,st,md1,st1,koe,attermk,&
                          const_mask,kappa_mask,lambda_mask,gamma_mask,&
                          iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask)
                  else
                     call numtpar(n2,type,md,st,md1,st1,koe,attermk,&
                          const_mask,kappa_mask,lambda_mask,gamma_mask,&
                          iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask,&
                          switch_mask)

                  endif
                  if (type .ne. 3 .and. type .ne. 4) then
                 if( .not. lgenetica) &
                     write(ilog,'(a)')&
                       'WARNING: Inconsistent constraint definition'
                  endif

                  if (type .eq. 3) then
                     write(string,'(2(a,i3,i3,i3))')&
                       'gamma ',k,j,i,' = gamma ',md,md1,st
                  else if (type .eq. 4) then
                     write(string,'(2(a,i3,i3,i3),i3)')&
                       'gamma ',k,j,i,' = mu ',md,md1,st,st1
                  endif
                  write(iout,'(a,f12.6)') string, pconfact(n1)

               endif
            enddo
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            do k=1,nmodes
               if (iota_mask(k,j,i,i).lt.0) then
                  n1=abs(iota_mask(k,j,i,i))
                  n2=parcons(n1)
                  if (n2 .gt. 0) then
                     call numpar(n2,type,md,st,md1,st1,koe,attermk,&
                          const_mask,kappa_mask,lambda_mask,gamma_mask,&
                          iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask)
                  else
                     call numtpar(n2,type,md,st,md1,st1,koe,attermk,&
                          const_mask,kappa_mask,lambda_mask,gamma_mask,&
                          iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask,&
                          switch_mask)
                  endif
                  if (type .ne. 3 .and. type .ne. 4) then
                 if( .not. lgenetica) &
                     write(ilog,'(a)')&
                       'WARNING: Inconsistent constraint definition'
                  endif

                  if (type .eq. 3) then
                     write(string,'(2(a,i3,i3,i3))')&
                       'iota ',k,j,i,' = iota ',md,md1,st
                  else if (type .eq. 4) then
                     write(string,'(2(a,i3,i3,i3),i3)')&
                       'iota ',k,j,i,' = mu ',md,md1,st,st1
                  endif
                  write(iout,'(a,f12.6)') string, pconfact(n1)

               endif
            enddo
         enddo
      enddo


      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=j,nmodes
                  if (mu_mask(k,j,i,l).lt.0) then
                     n1=abs(mu_mask(k,j,i,l))
                     n2=parcons(n1)
!                    print*, 'n2 = parcons(n1) = ', n2, 'n1 = abs(mumask) = ', n1 
                     if (n2 .gt. 0) then
                     call numpar(n2,type,md,st,md1,st1,koe,attermk,&
                          const_mask,kappa_mask,lambda_mask,gamma_mask,&
                          iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask)
                     else
                     call numtpar(n2,type,md,st,md1,st1,koe,attermk,&
                          const_mask,kappa_mask,lambda_mask,gamma_mask,&
                          iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask,&
                          switch_mask)

                     endif
                     if (type .ne. 5) then
!                     if (type .ne. 3 .and. type .ne. 4) then
                 if( .not. lgenetica) &
                        write(ilog,'(a)')&
                       'WARNING: Inconsistent constraint definition'
                     endif

                     if (type .eq. 3) then
                        write(string,'(2(a,i3,i3,i3,i3))')&
                       'mu ',k,j,i,l,' = gamma ',md,md1,st
                     else if (type .eq. 4) then
                        write(string,'(2(a,i3,i3,i3,i3))')&
                       'mu ',k,j,i,l,' = mu ',md,md1,st,st1
                     endif
                     write(iout,'(a,f12.6)') string, pconfact(n1)

                  endif
               enddo
            enddo
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            do k=1,ncoeff(j,i)
               if (expcoe_mask(k,j,i).lt.0) then
                  n1=abs(expcoe_mask(k,j,i))
                  n2=parcons(n1)
                  if (n2 .gt. 0) then
                     call numpar(n2,type,md,st,md1,st1,koe,attermk,&
                          const_mask,kappa_mask,lambda_mask,gamma_mask,&
                          iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask)
                  else
                     call numtpar(n2,type,md,st,md1,st1,koe,attermk,&
                          const_mask,kappa_mask,lambda_mask,gamma_mask,&
                          iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask,&
                          switch_mask)
                  endif
                  if (type .ne. 6) then
                 if( .not. lgenetica) &
                     write(ilog,'(a)')&
                    'WARNING: Inconsistent constraint definition'
                  endif

                  write(string,'(2(a,i3,i3,i3))')&
                       'general ',k,j,i,' = general ',koe,md,st
                  write(iout,'(a,f12.6)') string, pconfact(n1)

               endif
            enddo
         enddo
      enddo

      do i=1,nadd
         if (termk_mask(i).lt.0) then
            n1=abs(termk_mask(i))
            n2=parcons(n1)
            if (n2 .gt. 0) then
               call numpar(n2,type,md,st,md1,st1,koe,attermk,&
                    const_mask,kappa_mask,lambda_mask,gamma_mask,&
                    iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask)
            else
               call numtpar(n2,type,md,st,md1,st1,koe,attermk,&
                    const_mask,kappa_mask,lambda_mask,gamma_mask,&
                    iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask,&
                    switch_mask)
            endif
            ilbl = 1
            string= ' '
            atmp = ' '
            do m=1,nmodes
               if (termdef(adof(m),i) .gt. 0) then
                  write(atmp,'(i4,a,i4)') adof(m),'^',termdef(adof(m),i)
                  call rmblnk(atmp)
                  jlbl=index(atmp,' ')-1
               endif
            enddo
            j=attermk
            atmp1 = ' '
            do m=1,nmodes
               if (termdef(adof(m),j) .gt. 0) then
                  write(atmp1,'(i4,a,i4)') adof(m),'^',termdef(adof(m),j)
                  call rmblnk(atmp1)
                  klbl=index(atmp1,' ')-1
               endif
            enddo

            write(string,'(a,2i4,2x,3a,2i4,2x,a)') &
                  'explicit  ',termst(1,i),termst(2,i),atmp(1:jlbl),' = ', &
                  'explicit  ',termst(1,j),termst(2,j),atmp1(1:klbl)

            write(iout,'(a,f12.6)') string, pconfact(n1)
         endif
      enddo

      return
      end subroutine outcons

!################################################################
!
! write some statistics into the log-file
!
! TO DO: output not yet written for max change in higher or termk
!################################################################

      subroutine outstat(energy,const,kappa,lambda,gamma,iota,&
                 mu,expcoe,constini,kappaini,lambdaini,&
                 gammaini,iotaini,muini,expcoeini,higher,&
                 higherini,termk,termkini)

      implicit none

      integer(long)    :: i,j,s,l,imax,jmax,smax,lmax,s1,e2,error

      real(dop)        :: r0,r1,r2,r3,r4,r5,r6,r7,r8,r9,err,maxerr
      real(dop), dimension(nhigher), intent(in)                 :: higher
      real(dop), dimension(nadd), intent(in)                    :: termk
      real(dop), dimension(maxsta,maxsta), intent(in)           :: const
      real(dop), dimension(nmodes,maxsta), intent(in)           :: kappa
      real(dop), dimension(nmodes,maxsta,maxsta), intent(in)    :: lambda
      real(dop), dimension(nmodes,nmodes,maxsta), intent(in)    :: gamma
      real(dop), dimension(maxcoe,nmodes,maxsta), intent(in)    :: expcoe
      real(dop), dimension(nmodes,nmodes,maxsta,maxsta), intent(in) :: iota,mu

      real(dop), dimension(nsets), intent(in)                   :: energy
      real(dop), dimension(nhigher), intent(in)                 :: higherini
      real(dop), dimension(nadd), intent(in)                    :: termkini
      real(dop), dimension(maxsta,maxsta), intent(in)           :: constini
      real(dop), dimension(nmodes,maxsta), intent(in)           :: kappaini
      real(dop), dimension(nmodes,maxsta,maxsta), intent(in)    :: lambdaini
      real(dop), dimension(nmodes,nmodes,maxsta), intent(in)    :: gammaini
      real(dop), dimension(maxcoe,nmodes,maxsta), intent(in)    :: expcoeini
      real(dop), dimension(nmodes,nmodes,maxsta,maxsta), intent(in) :: iotaini,muini

      real(dop), dimension(maxsta)              :: adiab
      real(dop), dimension(3*maxsta)            :: work
      real(dop), dimension(maxsta,maxsta)       :: rot0

!-----------------------------------------------------------------------
! For a system in which the diabatic states are not equal to 
! adiabatic states at Q0, diagonalise the const matrix to get
! the constant diabatic - adiabatic rotation
!-----------------------------------------------------------------------
      if (lconstall) then
         do i=1,3
            write(ilog,*) ' '
         enddo
         write(ilog,*)&
            '*************************************************'
         write(ilog,*)'   The ADT constant rotation   '
         write(ilog,*)&
            '*************************************************'

         do s=1,nsta
            do s1=1,nsta
               rot0(s1,s) = const(s1,s)
            enddo
         enddo
         e2 = 3*maxsta
         call dsyev('V','U',nsta,rot0,maxsta,adiab,work,e2,error)
         if (error .ne. 0) then
            write(6,*) 'Cannot diagonalize const matrix'
         endif

         write(ilog,'(a8,3x,a)') ' E_adiab','  Vector components'
         do s=1,nsta
            write(ilog,'(f8.4,3x,6(f10.5))') &
                        adiab(s),(rot0(s1,s),s1=1,nsta)
         enddo
      endif

!-----------------------------------------------------------------------
! write which parameters have changed the most etc.
!-----------------------------------------------------------------------
      do i=1,3
         write(ilog,*) ' '
      enddo
      write(ilog,*)'*************************************************'
      write(ilog,*)'                 some statistics'
      write(ilog,*)'*************************************************'
      do i=1,2
         write(ilog,*) ' '
      enddo
      write(ilog,*)' n     Eab      Ecal      delta E    '&
           //'weight      (delta E)^2 * weight'
      write(ilog,*)'-------------------------------------'&
           //'---------------------------------' 
      r0=0.0
      r3=0.0
      r4=0.0
      maxerr=0.0
      do i=1,nsets
         err=energy(i)-denergy(i)
         maxerr=max(abs(err),maxerr)
         r0=r0+w0(i)
         r3=w0(i)*err**2
         r4=r4+r3
         write(ilog,'(i5,4F10.4,F10.6)') i, energy(i),denergy(i),&
              err,w0(i),r3
      enddo
      write(ilog,*) '-----------------------------------------------'
      write(ilog,'(/,/)')
      write(ilog,'(a,F10.5)') 'Maximum error        : ',maxerr
      write(ilog,'(a,F10.5)') 'sum over all weights : ',r0
      write(ilog,*)
      write(ilog,'(a,F15.5)') 'Total standard deviation in energy:  ',&
           sqrt(r4/r0)
      do i=1,3
         write(ilog,*)' '
      enddo

!     -------------------------------------------------------------
!                                                                  
!     -------------------------------------------------------------
      write(ilog,'(a)') ' '
      write(ilog,'(a)') '------------------------------------------'
      write(ilog,'(a)') ' ' 
      r1=0.0
      r2=0.0
      r3=0.0
      r4=0.0
      r5=0.0
      do i=1,nmodes
         r1=r1+freq0(i)
         r2=r2+(gamma(i,i,1)+gamma(i,i,2))
         r3=r3+(gamma(i,i,1)-gamma(i,i,2))
         r4=r4+(gamma(i,i,1)+gamma(i,i,2))&
              +abs(gamma(i,i,1)-gamma(i,i,2))
         r5=r5+(gamma(i,i,1)+gamma(i,i,2))&
              -abs(gamma(i,i,1)-gamma(i,i,2))
      enddo
      write(ilog,'(a,F10.5)') 'total sum over freq          = ',r1
      write(ilog,'(a,F10.5)') 'total sum over gamma1+gamma2 = ',r2
      write(ilog,'(a,F10.5)') 'total sum over gamma1-gamma2 = ',r3
      write(ilog,'(a,F10.5)') 'g+g+abs(g-g)                 = ',r4
      write(ilog,'(a,F10.5)') 'g+g-abs(g-g)                 = ',r5
      write(ilog,'(a)') ' '
      write(ilog,'(a)') '------------------------------------------'
      write(ilog,'(a)') ' ' 
      
!     -------------------------------------------------------------
!                                                                  
!     -------------------------------------------------------------
      r1=0.0
      r2=0.0
      r3=0.0
      r4=0.0
      r5=0.0
      r6=0.0
      r7=0.0
      r8=0.0
      r9=0.0
      do s=1,nsta
         do l=1,nsta
!            write(ilog,*) s,l,constini(s,l),const(s,l)
            r1=r1+(constini(s,l)-const(s,l))**2
         enddo
      enddo
      do i=1,nmodes
         do s=1,nsta
            r2=r2+(kappaini(i,s)-kappa(i,s))**2
            do l=1,nsta
               r4=r4+(lambdaini(i,s,l)-lambda(i,s,l))**2
            enddo
         enddo
      enddo
      do i=1,nmodes
         do j=1,nmodes
            do s=1,nsta
               r3=r3+(gammaini(i,j,s)-gamma(i,j,s))**2
               do l=1,nsta
                  r7=r7+(iotaini(i,j,s,l)-iota(i,j,s,l))**2
                  r5=r5+(muini(i,j,s,l)-mu(i,j,s,l))**2
               enddo
            enddo
         enddo
      enddo
      do j=1,nmodes
         do s=1,nsta
            do i=1,ncoeff(j,s)
               r6=r6+(expcoeini(i,j,s)-expcoe(i,j,s))**2
            enddo
         enddo
      enddo
      do i=1,nhigher
         r8=r8+(higherini(i)-higher(i))**2
      enddo
      do i=1,nadd
         r9=r9+(termkini(i)-termk(i))**2
      enddo

      r1=sqrt(r1)
      r2=sqrt(r2)
      r3=sqrt(r3)
      r4=sqrt(r4)
      r5=sqrt(r5)
      r6=sqrt(r6)
      r7=sqrt(r7)
      r8=sqrt(r8)
      r9=sqrt(r9)
      write(ilog,*)'The changes in the parameter'
      write(ilog,*)'------------------------------------------'
      write(ilog,'(a,f15.10)') 'Total change in const  :',r1
      write(ilog,'(a,f15.10)') 'Total change in kappa  :',r2 
      write(ilog,'(a,f15.10)') 'Total change in gamma  :',r3
      write(ilog,'(a,f15.10)') 'Total change in lambda :',r4
      write(ilog,'(a,f15.10)') 'Total change in mu     :',r5
      write(ilog,'(a,f15.10)') 'Total change in iota   :',r7
      if (highord .ge. 3)&
         write(ilog,'(a,f15.10)') 'Total change in higher :',r8
      if (lgeneral)&
         write(ilog,'(a,f15.10)') 'Total change in expcoe :',r6
      if (nadd .gt. 0)&
         write(ilog,'(a,f15.10)') 'Total change in termk  :',r9
      write(ilog,*)
      write(ilog,*)
      write(ilog,*)'Maximum changes for '
      r0=0.0
      do i=1,nmodes
         do s=1,nsta
            if ((abs(kappaini(i,s)-kappa(i,s))).ge.r0) then
               r0=abs(kappaini(i,s)-kappa(i,s))
               imax=i
               smax=s
            endif
         enddo
      enddo
      write(ilog,*)' for kappa in :'
      write(ilog,*)'     mode  : ',imax
      write(ilog,*)'     state : ',smax
      r0=0.0
      do i=1,nmodes
         do s=1,nsta
            do l=1,nsta
               if (abs(lambdaini(i,s,l)-lambda(i,s,l)).ge.r0) then
                  r0=abs(lambdaini(i,s,l)-lambda(i,s,l))
                  imax=i
                  smax=s
                  lmax=l
               endif
            enddo
         enddo
      enddo
      write(ilog,*)' for lambda in :'
      write(ilog,'(A,i3)')      '     mode       : ',imax
      write(ilog,'(A,i3,a3,i3)')'     interstate : ',smax,' x ',lmax 
      r0=0.0
      do i=1,nmodes
         do j=1,nmodes
            do s=1,nsta
               if (abs(gammaini(i,j,s)-gamma(i,j,s)).ge.r0) then
                  r0=abs(gammaini(i,j,s)-gamma(i,j,s))
                  imax=i
                  jmax=j
                  smax=s
               endif
            enddo
         enddo
      enddo
      write(ilog,*)' for gamma in :'
      write(ilog,'(A,i3,a3,i3)')'     mode   : ',imax,' x ',jmax
      write(ilog,'(A,i3)')      '     state  : ',smax
      r0=0.0
      do i=1,nmodes
         do j=1,nmodes
            do s=1,nsta
               do l=1,nsta
                  if (abs(iotaini(i,j,s,l)-iota(i,j,s,l)).ge.r0) then
                     r0=abs(iotaini(i,j,s,l)-iota(i,j,s,l))
                     imax=i
                     jmax=j
                     smax=s
                  endif
               enddo
            enddo
         enddo
      enddo
      write(ilog,*)' for iota in :'
      write(ilog,'(A,i3,a3,i3)')'     mode   : ',imax,' x ',jmax
      write(ilog,'(A,i3)')      '     state  : ',smax
      r0=0.0
      do i=1,nmodes
         do j=1,nmodes
            do s=1,nsta
               do l=1,nsta
                  if (abs(muini(i,j,s,l)-mu(i,j,s,l)).ge.r0) then
                     r0=abs(muini(i,j,s,l)-mu(i,j,s,l))
                     imax=i
                     jmax=j
                     smax=s
                     lmax=l
                  endif
               enddo
            enddo
         enddo
      enddo
      write(ilog,*)' for mu in :'
      write(ilog,'(A,i3,a3,i3)')'     mode        : ',imax,' x ',jmax
      write(ilog,'(A,i3,a3,i3)')'     interstate  : ',smax,' x ',lmax
      if (lgeneral) then
         r0=0.0
         do s=1,nsta
            do j=1,nmodes
               do i=1,ncoeff(j,s)
                  if (abs(expcoeini(i,j,s)-expcoe(i,j,s)).ge.r0) then
                     r0=abs(expcoeini(i,j,s)-expcoe(i,j,s))
                     imax=i
                     jmax=j
                     smax=s
                  endif
               enddo
            enddo
         enddo
         write(ilog,*)' for expcoe in :'
         write(ilog,'(A,i3)') '     mode   : ',imax
         write(ilog,'(A,i3)') '     state  : ',smax
         write(ilog,'(A,i3)') '     coeff  : ',jmax
      endif

      return
      end subroutine outstat

!#################################################################
!###### write the MCTDH Operator file ############################

      subroutine writeop(io,const,kappa,lambda,gamma,mu,iota,&
                 expcoe,higher,termk,const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask)

      implicit none

      integer(long), intent(in) :: io
      integer(long) :: i,j,s,s1,c,d,e,f,g,l,m,n,k,k1,&
                       ilbl,ilbl1,ilbl2,ilbl3,ilbl4,ilbl5,ilbl6,nmodes10
      integer(long), dimension(nsta,nsta), intent(in) :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in) :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in) :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in) :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in) :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: &
                                                          iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in) :: termk_mask


      real(dop), dimension(nhigher), intent(in)                 :: higher
      real(dop), dimension(nsta,nsta), intent(in)               :: const
      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota,mu
      real(dop), dimension(nadd), intent(in)                    :: termk

      character(len=64)       :: atmp,atmp1,atmp2,atmp3,atmp4,atmp5,atmp6
      character(len=80)       :: mout1,mout2,mout3,mout4,mout5
      character(len=6)        :: opstring,opstring1,opstring2,opstring3
      logical(kind=4)         :: lprint,chk
      integer(long), dimension(maxsta)      :: sdum
      integer(long), dimension(nadd,maxdim) :: modemap
      integer(long) :: ns,sl,sl1,nterms

      ns=0
      do i=1,nsta
         if (sfit(i).ne.0) then
            ns=ns+1
            sdum(i)=ns
         else
            sdum(i)=0
         endif
      enddo

      mout1 = ' '
      mout2 = ' '
      mout3 = ' '
      mout4 = ' '
      mout5 = ' '

      write(io,20) 'OP_DEFINE-SECTION'
      write(io,20) 'TITLE'
      write(io,20) 'MCTDH-Operator-file created by vcham'
      write(io,20) 'END-TITLE'
      write(io,20) 'END-OP_DEFINE-SECTION'
      write(io,20) ' '
      write(io,20) 'PARAMETER-SECTION'

      call writepar(io,const,kappa,lambda,gamma,mu,iota,&
                 expcoe,higher,termk,const_mask,kappa_mask,&
                 lambda_mask,gamma_mask,iota_mask,mu_mask,expcoe_mask,&
                 termk_mask)
      
      write(io,20) ' '
      write(io,20) 'end-parameter-section'
      write(io,20) ' '
      write(io,20) 'LABELS-SECTION'


      call writelab(io,const,kappa,lambda,gamma,mu,iota,&
                 expcoe,termk,const_mask,kappa_mask,&
                 lambda_mask,gamma_mask,iota_mask,mu_mask,expcoe_mask,&
                 termk_mask)

      write(io,20) ' '
      write(io,20) 'end-labels-section'
      write(io,20) ' '
      write(io,20) 'HAMILTONIAN-SECTION'
      write(io,20) ' '
      write(io,'(70a)') ('-',i=1,70)

      i=1
      d=((nmodes+1)/10)+1  ! No. of lines to write
      l = nmodes+1         ! Place for el mode
      do j=1,d
         mout1 = ' '
         e=min(i+9,l)   ! No. of entries in line
         write(mout1(1:7),'(a)') 'modes |'
         c=8
         do 
            if (i .eq. l) then
               write(mout1(c:c+3),'(a3)') ' el'
            else if (i.lt. 10) then
               write(mout1(c:c+5), '(a,i1,a)')  ' v',i,'  |'
            else
               write(mout1(c:c+6), '(a,i2,a)')  ' v',i,' |'
            endif
            c=c+6
            i=i+1
            if (i .gt. e) exit   ! Line written
         enddo
         write(io,'(a)') trim(mout1)
      enddo
 
      write(io,'(70a)') ('-',i=1,70)

      write(io,20) ' ' 
      write(io,20) '# Kinetic Energy'
      write(io,20) ' '   

       do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         write(atmp,'(a,i5)') 'omega_',i
         write(atmp1,'(a,i5)') '|',i
         call rmblnk(atmp)
         call rmblnk(atmp1)
         ilbl=index(atmp,' ')-1
         ilbl1=index(atmp1,' ')-1
         write(io,'(3a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),'KE'
      enddo

!
! different printing if 
! 1. all states use an HO diabatic function
! 2. some states use an HO diabatic function
! 3. no states use an HO diabatic function (supress print)
! 
      write(io,20) ' '
      write(io,20) '# Potential for Harmonic oscillator'
      write(io,20) ' '

      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         if (qtransform(i) .ne. 0) then
            call getopstring(qtransform(i),opstring,2)
         else
            opstring='q^2'
         endif

         lprint=.true.
         do s=1,nsta
            if (ifdiab(i,s) .gt. 1) lprint=.false.
         enddo
         if (lprint) then
            write(atmp,'(a,i5)') '0.5*omega_',i
            write(atmp1,'(a,i5)') '|',i
            call rmblnk(atmp)
            call rmblnk(atmp1)
            ilbl=index(atmp,' ')-1
            ilbl1=index(atmp1,' ')-1
            write(io,'(3a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),opstring
         else
            do s=1,nsta
               if (sfit(s) .eq. 0) cycle
               if (ifdiab(i,s) .lt. 2 .or. ifdiab(i,s) .eq. 9  &
                   .or. ifdiab(i,s) .eq. 12) then
                  sl=sdum(s)
                  write(atmp,'(a,i5)') '0.5*omega_',i
                  write(atmp1,'(a,i5)') '|',i
                  write(atmp2,'(a,i5)') '|',l
                  write(atmp3,'(2(a,i5))') 'S',sl,'&',sl
                  call rmblnk(atmp)
                  call rmblnk(atmp1)
                  call rmblnk(atmp2)
                  call rmblnk(atmp3)
                  ilbl=index(atmp,' ')-1
                  ilbl1=index(atmp1,' ')-1
                  ilbl2=index(atmp2,' ')-1
                  ilbl3=index(atmp3,' ')-1
                  write(io,'(5a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp2(1:ilbl2+5),atmp3(1:ilbl3)
               endif
            enddo
         endif
      enddo

      write(io,20) ' '
      write(io,20) '# Electronic States'
      write(io,20) ' ' 
      do s=1,nsta
         if (sfit(s) .eq. 0) cycle

         sl=sdum(s)
         write(atmp,'(a,i5)') 'E',s
         write(atmp1,'(a,i5)') '|',l
         write(atmp2,'(2(a,i5))') 'S',sl,'&',sl
         call rmblnk(atmp)
         call rmblnk(atmp1)
         call rmblnk(atmp2)
         ilbl=index(atmp,' ')-1
         ilbl1=index(atmp1,' ')-1
         ilbl2=index(atmp2,' ')-1
         write(io,'(3a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),atmp2(1:ilbl2)
      enddo
      if (lconstall) then
         do s=1,nsta-1
            do s1=s+1,nsta
               write(atmp,'(2(a,i5))') 'E',s,'_',s1
               write(atmp1,'(a,i5)') '|',l
               write(atmp2,'(2(a,i5))') 'S',s,'&',s1
               call rmblnk(atmp)
               call rmblnk(atmp1)
               call rmblnk(atmp2)
               ilbl=index(atmp,' ')-1
               ilbl1=index(atmp1,' ')-1
               ilbl2=index(atmp2,' ')-1
               write(io,'(3a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                   atmp2(1:ilbl2)

            enddo
         enddo
      endif

      write(io,20) ' '
      write(io,20) '# Lambda'
      write(io,20) ' '
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle

         if (qtransform(i) .ne. 0) then
            call getopstring(qtransform(i),opstring,1)
         else
            opstring='q'
         endif

         do s=1,nsta-1
            if (sfit(s) .eq. 0) cycle
            do s1=s+1,nsta
               if (sfit(s1) .eq. 0) cycle
               if ((lambda_mask(i,s,s1).ne.0) .and.&
                    (abs(lambda(i,s,s1)-0.0_dop) .gt. cutoff_op) ) then
                  sl=sdum(s)
                  sl1=sdum(s1)
                  write(atmp,'(3(a,i5))') 'lambda',s,'_',s1,'_',i
                  write(atmp1,'(a,i5)') '|',i
                  write(atmp2,'(a,i5)') '|',l
                  write(atmp3,'(2(a,i5))') 'S',sl,'&',sl1
                  call rmblnk(atmp)
                  call rmblnk(atmp1)
                  call rmblnk(atmp2)
                  call rmblnk(atmp3)
                  ilbl=index(atmp,' ')-1
                  ilbl1=index(atmp1,' ')-1
                  ilbl2=index(atmp2,' ')-1
                  ilbl3=index(atmp3,' ')-1
                  write(io,'(5a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp2(1:ilbl2+5),atmp3(1:ilbl3)
               endif
            enddo
         enddo
      enddo

      write(io,20) ' '
      write(io,20) '# Kappa'
      write(io,20) ' '
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         if (qtransform(i) .ne. 0) then
            call getopstring(qtransform(i),opstring,1)
         else
            opstring='q'
         endif

         do s=1,nsta
            if (sfit(s) .eq. 0) cycle
            if ((kappa_mask(i,s) .ne. 0) .and.&
                 (abs(kappa(i,s)-0.0_dop) .gt. cutoff_op) .and.&
                     ifdiab(i,s) .lt. 1) then
               sl=sdum(s)
               write(atmp,'(2(a,i5))') 'kappa',s,'_',i
               write(atmp1,'(a,i5)') '|',i
               write(atmp2,'(a,i5)') '|',l
               write(atmp3,'(2(a,i5))') 'S',sl,'&',sl
               call rmblnk(atmp)
               call rmblnk(atmp1)
               call rmblnk(atmp2)
               call rmblnk(atmp3)
               ilbl=index(atmp,' ')-1
               ilbl1=index(atmp1,' ')-1
               ilbl2=index(atmp2,' ')-1
               ilbl3=index(atmp3,' ')-1
               write(io,'(5a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp2(1:ilbl2+5),atmp3(1:ilbl3)
            endif
         enddo
      enddo

      write(io,20) ' '
      write(io,20) '# On Diagonal Gamma Constants'
      write(io,20) ' '
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         if (qtransform(i) .ne. 0) then
            call getopstring(qtransform(i),opstring,2)
         else
            opstring='q^2'
         endif

         do s=1,nsta
            if (sfit(s) .eq. 0) cycle
            if ((gamma_mask(i,i,s).ne.0) .and.&
                  (abs(gamma(i,i,s)) .gt. cutoff_op) .and. &
                     ifdiab(i,s) .lt. 1) then
               sl=sdum(s)
               write(atmp,'(3(a,i5))') '0.5*gamma',s,'_',i,'_',i
               write(atmp1,'(a,i5)') '|',i
               write(atmp2,'(a,i5)') '|',l
               write(atmp3,'(2(a,i5))') 'S',sl,'&',sl
               call rmblnk(atmp)
               call rmblnk(atmp1)
               call rmblnk(atmp2)
               call rmblnk(atmp3)
               ilbl=index(atmp,' ')-1
               ilbl1=index(atmp1,' ')-1
               ilbl2=index(atmp2,' ')-1
               ilbl3=index(atmp3,' ')-1
               write(io,'(5a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp2(1:ilbl2+5),atmp3(1:ilbl3)
            endif
         enddo
      enddo

      write(io,20) ' '
      write(io,20) '# Off Diagonal Gamma Constants'
      write(io,20) ' '
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         if (qtransform(i) .ne. 0) then
            call getopstring(qtransform(i),opstring,1)
         else
            opstring='q'
         endif

         do j=i,nmodes
            if (mfit(j) .eq. 0) cycle
            if (qtransform(j) .ne. 0) then
               call getopstring(qtransform(j),opstring1,1)
            else
               opstring1='q'
            endif

            do s=1,nsta
               if (sfit(s) .eq. 0) cycle
               if ((i .ne. j) .and. (gamma_mask(i,j,s).ne.0) .and.&
                    (abs(gamma(i,j,s)) .gt. cutoff_op) ) then
                  sl=sdum(s)
                  write(atmp,'(3(a,i5))') 'gamma',s,'_',i,'_',j
!                  write(atmp,'(3(a,i5))') '0.5*gamma',s,'_',i,'_',j
                  write(atmp1,'(a,i5)') '|',i
                  write(atmp2,'(a,i5)') '|',j
                  write(atmp3,'(a,i5)') '|',l
                  write(atmp4,'(2(a,i5))') 'S',sl,'&',sl
                  call rmblnk(atmp)
                  call rmblnk(atmp1)
                  call rmblnk(atmp2)
                  call rmblnk(atmp3)
                  call rmblnk(atmp4)
                  ilbl=index(atmp,' ')-1
                  ilbl1=index(atmp1,' ')-1
                  ilbl2=index(atmp2,' ')-1
                  ilbl3=index(atmp3,' ')-1
                  ilbl4=index(atmp4,' ')-1
                  write(io,'(7a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp2(1:ilbl2+5),opstring1,&
                      atmp3(1:ilbl3+5),atmp4(1:ilbl4)

               endif
            enddo
         enddo
      enddo

      if (hamord .lt. 3) go to 100

      write(io,20) ' '
      write(io,20) '# On-Diagonal quad-lin (Iota) Constants'
      write(io,20) ' '
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         do j=1,nmodes
            if (mfit(j) .eq. 0) cycle
            do s=1,nsta
               if (sfit(s) .eq. 0) cycle
               if (iota_mask(i,j,s,s).ne.0  .and.&
                     abs(iota(i,j,s,s)) .gt. cutoff_op ) then
                  sl=sdum(s)
                  if (i .eq. j) then
                     write(atmp,'(3(a,i5))') '0.1666667*iota',s,'_',&
                           i,'_',j
                  else
                     write(atmp,'(3(a,i5))') '0.5*iota',s,'_',i,'_',j
                  endif
                  write(atmp1,'(a,i5)') '|',i
                  write(atmp2,'(a,i5)') '|',j
                  write(atmp3,'(a,i5)') '|',l
                  write(atmp4,'(2(a,i5))') 'S',sl,'&',sl
                  call rmblnk(atmp)
                  call rmblnk(atmp1)
                  call rmblnk(atmp2)
                  call rmblnk(atmp3)
                  call rmblnk(atmp4)
                  ilbl=index(atmp,' ')-1
                  ilbl1=index(atmp1,' ')-1
                  ilbl2=index(atmp2,' ')-1
                  ilbl3=index(atmp3,' ')-1
                  ilbl4=index(atmp4,' ')-1
                  if (i .eq. j .and. ifdiab(i,s) .lt. 1) then
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(i),opstring,3)
                     else
                        opstring='q^3'
                     endif

                     write(io,'(7a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                         opstring,atmp3(1:ilbl3+5),atmp4(1:ilbl4)
                  else if (i .ne. j) then
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(i),opstring,1)
                     else
                        opstring='q'
                     endif
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(j),opstring1,2)
                     else
                        opstring1='q^2'
                     endif

                     write(io,'(7a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                         opstring,atmp2(1:ilbl2+5),opstring1,&
                         atmp3(1:ilbl3+5),atmp4(1:ilbl4)
                  endif
               endif
            enddo
         enddo
      enddo

      write(io,20) ' '
      write(io,20) '# Off-Diagonal quad-lin (Iota) Constants'
      write(io,20) ' '
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         do j=1,nmodes
            if (mfit(j) .eq. 0) cycle
            do s=1,nsta-1
               if (sfit(s) .eq. 0) cycle
               do s1=s+1,nsta
                  if (sfit(s1) .eq. 0) cycle
                  if (iota_mask(i,j,s,s1).eq.0) cycle
                  if (abs(iota(i,j,s,s1)) .lt. cutoff_op) cycle

                  sl=sdum(s)
                  sl1=sdum(s1)
                  if (i .eq. j) then
                     write(atmp,'(4(a,i5))') '0.1666667*iota2',s,'_',&
                           s1,'_',i,'_',j
                  else
                     write(atmp,'(4(a,i5))') &
                           '0.5*iota2',s,'_',s1,'_',i,'_',j
                  endif
                  write(atmp1,'(a,i5)') '|',i
                  write(atmp2,'(a,i5)') '|',j
                  write(atmp3,'(a,i5)') '|',l
                  write(atmp4,'(2(a,i5))') 'S',sl,'&',sl1
                  call rmblnk(atmp)
                  call rmblnk(atmp1)
                  call rmblnk(atmp2)
                  call rmblnk(atmp3)
                  call rmblnk(atmp4)
                  ilbl=index(atmp,' ')-1
                  ilbl1=index(atmp1,' ')-1
                  ilbl2=index(atmp2,' ')-1
                  ilbl3=index(atmp3,' ')-1
                  ilbl4=index(atmp4,' ')-1
                  if (i .eq. j) then
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(i),opstring,3)
                     else
                        opstring='q^3'
                     endif

                     write(io,'(7a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                         opstring,atmp3(1:ilbl3+5),atmp4(1:ilbl4)
                  else
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(i),opstring,1)
                     else
                        opstring='q'
                     endif
                     if (qtransform(j) .ne. 0) then
                        call getopstring(qtransform(j),opstring1,2)
                     else
                        opstring1='q^2'
                     endif

                     write(io,'(7a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                         opstring,atmp2(1:ilbl2+5),opstring1,&
                         atmp3(1:ilbl3+5),atmp4(1:ilbl4)
                  endif
               enddo
            enddo
         enddo
      enddo

      write(io,20) ' '
      write(io,20) '#off-diagonal quadratic vibronic coupling(mu) '
      write(io,20) ' '
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         if (qtransform(i) .ne. 0) then
            call getopstring(qtransform(i),opstring,2)
         else
            opstring='q^2'
         endif

         do s=1,nsta
            if (sfit(s) .eq. 0) cycle
            do s1=s+1,nsta
               if (sfit(s1) .eq. 0) cycle
               if ((mu_mask(i,i,s,s1).ne.0) .and. &
                      (dabs(mu(i,i,s,s1)-0.0_dop) .gt. cutoff_op)) then
                  sl=sdum(s)
                  sl1=sdum(s1)
                  write(atmp,'(4(a,i5))') '0.5*mu',s,'_',s1,'_',i,'_',i
                  write(atmp1,'(a,i5)') '|',i
                  write(atmp3,'(a,i5)') '|',l
                  write(atmp4,'(2(a,i5))') 'S',sl,'&',sl1
                  call rmblnk(atmp)
                  call rmblnk(atmp1)
                  call rmblnk(atmp3)
                  call rmblnk(atmp4)
                  ilbl=index(atmp,' ')-1
                  ilbl1=index(atmp1,' ')-1
                  ilbl3=index(atmp3,' ')-1
                  ilbl4=index(atmp4,' ')-1
                  write(io,'(5a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                          opstring,atmp3(1:ilbl3+5),atmp4(1:ilbl4)
               endif
            enddo
         enddo
      enddo

      write(io,20) ' '
      write(io,20) '#off-diagonal bilinear vibronic coupling(mu) '
      write(io,20) ' '
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         if (qtransform(i) .ne. 0) then
            call getopstring(qtransform(i),opstring,1)
         else
            opstring='q'
         endif

         do j=i,nmodes
            if (mfit(j) .eq. 0) cycle
            if (qtransform(j) .ne. 0) then
               call getopstring(qtransform(j),opstring1,1)
            else
               opstring1='q'
            endif
            do s=1,nsta
               if (sfit(s) .eq. 0) cycle
               do s1=s+1,nsta
                  if (sfit(s1) .eq. 0) cycle
                  if ((i .ne. j) .and. (mu_mask(i,j,s,s1).ne.0) .and. &
                      (dabs(mu(i,j,s,s1)-0.0_dop) .gt. cutoff_op)) then
                     sl=sdum(s)
                     sl1=sdum(s1)
                     write(atmp,'(4(a,i5))') &
                             'mu',s,'_',s1,'_',i,'_',j
!                             '0.5*mu',s,'_',s1,'_',i,'_',j
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',j
                     write(atmp3,'(a,i5)') '|',l
                     write(atmp4,'(2(a,i5))') 'S',sl,'&',sl1
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     call rmblnk(atmp4)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     ilbl4=index(atmp4,' ')-1
                     write(io,'(7a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp2(1:ilbl2+5),opstring1,&
                      atmp3(1:ilbl3+5),atmp4(1:ilbl4)
                  endif
               enddo 
            enddo
         enddo
      enddo

      if (highord .ge. 3) then
         write(io,20) ' '
         write(io,20) '#on-diagonal lin-lin-lin coupling '
         write(io,20) ' '

         do s = 1,nsta
            do i=1,nmodes
               if (qtransform(i) .ne. 0) then
                  call getopstring(qtransform(i),opstring,1)
               else
                  opstring='q'
               endif
               do j = i+1,nmodes
                  if (qtransform(j) .ne. 0) then
                     call getopstring(qtransform(j),opstring1,1)
                  else
                     opstring1='q'
                  endif
                  do k = j+1,nmodes
                     if (qtransform(k) .ne. 0) then
                        call getopstring(qtransform(k),opstring2,1)
                     else
                        opstring2='q'
                     endif
                     write(atmp,'(4(a,i5))') &
                             'par',s,'_',i,'_',j,'_',k
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',j
                     write(atmp3,'(a,i5)') '|',k
                     write(atmp4,'(a,i5)') '|',l
                     write(atmp5,'(2(a,i5))') 'S',s,'&',s
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     call rmblnk(atmp4)
                     call rmblnk(atmp5)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     ilbl4=index(atmp4,' ')-1
                     ilbl5=index(atmp5,' ')-1
                     write(io,'(9a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp2(1:ilbl2+5),opstring1,&
                      atmp3(1:ilbl3+5),opstring2,atmp4(1:ilbl4+5),&
                      atmp5(1:ilbl5)
                  enddo
               enddo
            enddo
         enddo
      endif

      if (highord .eq. 4) then

! gam_iijj q_i^2 q_j^2 terms
         write(io,20) ' '
         write(io,20) '#on-diagonal quad-quad coupling '
         write(io,20) ' '
         do s=1,nsta
            do i=1,nmodes-1
               if (qtransform(i) .ne. 0) then
                  call getopstring(qtransform(i),opstring,2)
               else
                  opstring='q^2'
               endif
               do j = i+1,nmodes
                  if (qtransform(j) .ne. 0) then
                     call getopstring(qtransform(j),opstring1,2)
                  else
                     opstring1='q^2'
                  endif
                  write(atmp,'(5(a,i5))') &
                             '0.5*par',s,'_',i,'_',i,'_',j,'_',j
                  write(atmp1,'(a,i5)') '|',i
                  write(atmp2,'(a,i5)') '|',j
                  write(atmp4,'(a,i5)') '|',l
                  write(atmp5,'(2(a,i5))') 'S',s,'&',s
                  call rmblnk(atmp)
                  call rmblnk(atmp1)
                  call rmblnk(atmp2)
                  call rmblnk(atmp4)
                  call rmblnk(atmp5)
                  ilbl=index(atmp,' ')-1
                  ilbl1=index(atmp1,' ')-1
                  ilbl2=index(atmp2,' ')-1
                  ilbl4=index(atmp4,' ')-1
                  ilbl5=index(atmp5,' ')-1
                  write(io,'(7a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp2(1:ilbl2+5),opstring1,&
                      atmp4(1:ilbl4+5),atmp5(1:ilbl5)
               enddo
            enddo
         enddo

! gam_iiij q_i^3 q_j terms
         write(io,20) ' '
         write(io,20) '#on-diagonal cube-lin coupling '
         write(io,20) ' '
         do s=1,nsta
            do i=1,nmodes-1
               if (qtransform(i) .ne. 0) then
                  call getopstring(qtransform(i),opstring,3)
               else
                  opstring='q^3'
               endif
               do j = i+1,nmodes
                  if (qtransform(j) .ne. 0) then
                     call getopstring(qtransform(j),opstring1,1)
                  else
                     opstring1='q'
                  endif
                  write(atmp,'(5(a,i5))') &
                             '0.1666667*par',s,'_',i,'_',i,'_',i,'_',j
                  write(atmp1,'(a,i5)') '|',i
                  write(atmp2,'(a,i5)') '|',j
                  write(atmp4,'(a,i5)') '|',l
                  write(atmp5,'(2(a,i5))') 'S',s,'&',s
                  call rmblnk(atmp)
                  call rmblnk(atmp1)
                  call rmblnk(atmp2)
                  call rmblnk(atmp4)
                  call rmblnk(atmp5)
                  ilbl=index(atmp,' ')-1
                  ilbl1=index(atmp1,' ')-1
                  ilbl2=index(atmp2,' ')-1
                  ilbl4=index(atmp4,' ')-1
                  ilbl5=index(atmp5,' ')-1
                  write(io,'(7a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp2(1:ilbl2+5),opstring1,&
                      atmp4(1:ilbl4+5),atmp5(1:ilbl5)
               enddo
            enddo
            do i=2,nmodes
               if (qtransform(i) .ne. 0) then
                  call getopstring(qtransform(i),opstring,3)
               else
                  opstring='q^3'
               endif
               do j = 1,i-1
                  if (qtransform(j) .ne. 0) then
                     call getopstring(qtransform(j),opstring1,1)
                  else
                     opstring1='q'
                  endif
                  write(atmp,'(5(a,i5))') &
                             '0.1666667*par',s,'_',i,'_',i,'_',i,'_',j
                  write(atmp1,'(a,i5)') '|',i
                  write(atmp2,'(a,i5)') '|',j
                  write(atmp4,'(a,i5)') '|',l
                  write(atmp5,'(2(a,i5))') 'S',s,'&',s
                  call rmblnk(atmp)
                  call rmblnk(atmp1)
                  call rmblnk(atmp2)
                  call rmblnk(atmp4)
                  call rmblnk(atmp5)
                  ilbl=index(atmp,' ')-1
                  ilbl1=index(atmp1,' ')-1
                  ilbl2=index(atmp2,' ')-1
                  ilbl4=index(atmp4,' ')-1
                  ilbl5=index(atmp5,' ')-1
                  write(io,'(7a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp2(1:ilbl2+5),opstring1,&
                      atmp4(1:ilbl4+5),atmp5(1:ilbl5)
               enddo
            enddo
         enddo

! gam_iijk q_i^2 q_j q_k terms
         write(io,20) ' '
         write(io,20) '#on-diagonal quad-lin-lin coupling '
         write(io,20) ' '
         do s=1,nsta
            do i=1,nmodes-2
               if (qtransform(i) .ne. 0) then
                  call getopstring(qtransform(i),opstring,2)
               else
                  opstring='q^2'
               endif
               do j = i+1,nmodes-1
                  if (qtransform(j) .ne. 0) then
                     call getopstring(qtransform(j),opstring1,1)
                  else
                     opstring1='q'
                  endif
                  do k = j+1,nmodes
                     if (qtransform(k) .ne. 0) then
                        call getopstring(qtransform(k),opstring2,1)
                     else
                        opstring2='q'
                     endif
                     write(atmp,'(5(a,i5))') &
                             'par',s,'_',i,'_',i,'_',j,'_',k
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',j
                     write(atmp3,'(a,i5)') '|',k
                     write(atmp4,'(a,i5)') '|',l
                     write(atmp5,'(2(a,i5))') 'S',s,'&',s
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     call rmblnk(atmp4)
                     call rmblnk(atmp5)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     ilbl4=index(atmp4,' ')-1
                     ilbl5=index(atmp5,' ')-1
                     write(io,'(9a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp2(1:ilbl2+5),opstring1,&
                      atmp3(1:ilbl3+5),opstring2,atmp4(1:ilbl4+5),&
                      atmp5(1:ilbl5)
                  enddo
               enddo
            enddo
            do i=3,nmodes
               if (qtransform(i) .ne. 0) then
                  call getopstring(qtransform(i),opstring,2)
               else
                  opstring='q^2'
               endif
               do j = 1,i-2
                  if (qtransform(j) .ne. 0) then
                     call getopstring(qtransform(j),opstring1,1)
                  else
                     opstring1='q'
                  endif
                  do k = j+1,i-1
                     if (qtransform(k) .ne. 0) then
                        call getopstring(qtransform(k),opstring2,1)
                     else
                        opstring2='q'
                     endif
                     write(atmp,'(5(a,i5))') &
                             'par',s,'_',i,'_',i,'_',j,'_',k
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',j
                     write(atmp3,'(a,i5)') '|',k
                     write(atmp4,'(a,i5)') '|',l
                     write(atmp5,'(2(a,i5))') 'S',s,'&',s
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     call rmblnk(atmp4)
                     call rmblnk(atmp5)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     ilbl4=index(atmp4,' ')-1
                     ilbl5=index(atmp5,' ')-1
                     write(io,'(9a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp2(1:ilbl2+5),opstring1,&
                      atmp3(1:ilbl3+5),opstring2,atmp4(1:ilbl4+5),&
                      atmp5(1:ilbl5)
                  enddo
               enddo
            enddo
            do i=2,nmodes-1
               if (qtransform(i) .ne. 0) then
                  call getopstring(qtransform(i),opstring,2)
               else
                  opstring='q^2'
               endif
               do j = 1,i-1
                  if (qtransform(j) .ne. 0) then
                     call getopstring(qtransform(j),opstring1,1)
                  else
                     opstring1='q'
                  endif
                  do k = i+1,nmodes
                     if (qtransform(k) .ne. 0) then
                        call getopstring(qtransform(k),opstring2,1)
                     else
                        opstring2='q'
                     endif
                     write(atmp,'(5(a,i5))') &
                             'par',s,'_',i,'_',i,'_',j,'_',k
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',j
                     write(atmp3,'(a,i5)') '|',k
                     write(atmp4,'(a,i5)') '|',l
                     write(atmp5,'(2(a,i5))') 'S',s,'&',s
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     call rmblnk(atmp4)
                     call rmblnk(atmp5)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     ilbl4=index(atmp4,' ')-1
                     ilbl5=index(atmp5,' ')-1
                     write(io,'(9a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp2(1:ilbl2+5),opstring1,&
                      atmp3(1:ilbl3+5),opstring2,atmp4(1:ilbl4+5),&
                      atmp5(1:ilbl5)
                  enddo
               enddo
            enddo
            write(io,*)
         enddo

! gam_ijkl q_i q_j q_k q_l terms
         write(io,20) ' '
         write(io,20) '#on-diagonal lin-lin-lin-lin coupling '
         write(io,20) ' '
         do s=1,nsta
            do i=1,nmodes-3
               if (qtransform(i) .ne. 0) then
                  call getopstring(qtransform(i),opstring,1)
               else
                  opstring='q'
               endif
               do j = i+1,nmodes-2
                  if (qtransform(j) .ne. 0) then
                     call getopstring(qtransform(j),opstring1,1)
                  else
                     opstring1='q'
                  endif
                  do k = j+1,nmodes-1
                     if (qtransform(k) .ne. 0) then
                        call getopstring(qtransform(k),opstring2,1)
                     else
                        opstring2='q'
                     endif
                     do k1 = k+1,nmodes
                        if (qtransform(k1) .ne. 0) then
                           call getopstring(qtransform(k1),opstring3,1)
                        else
                           opstring3='q'
                        endif
                        write(atmp,'(5(a,i5))') &
                             'par',s,'_',i,'_',j,'_',k,'_',l
                        write(atmp1,'(a,i5)') '|',i
                        write(atmp2,'(a,i5)') '|',j
                        write(atmp3,'(a,i5)') '|',k
                        write(atmp4,'(a,i5)') '|',k1
                        write(atmp5,'(a,i5)') '|',l
                        write(atmp6,'(2(a,i5))') 'S',s,'&',s
                        call rmblnk(atmp)
                        call rmblnk(atmp1)
                        call rmblnk(atmp2)
                        call rmblnk(atmp3)
                        call rmblnk(atmp4)
                        call rmblnk(atmp5)
                        call rmblnk(atmp6)
                        ilbl=index(atmp,' ')-1
                        ilbl1=index(atmp1,' ')-1
                        ilbl2=index(atmp2,' ')-1
                        ilbl3=index(atmp3,' ')-1
                        ilbl4=index(atmp4,' ')-1
                        ilbl5=index(atmp5,' ')-1
                        ilbl6=index(atmp6,' ')-1
                        write(io,'(11a)') atmp(1:ilbl+5),&
                            atmp1(1:ilbl1+5),&
                           opstring,atmp2(1:ilbl2+5),opstring1,&
                           atmp3(1:ilbl3+5),opstring2,atmp4(1:ilbl4+5),&
                           opstring3,atmp4(1:ilbl5+5),atmp5(1:ilbl6)
                     enddo
                  enddo
               enddo
            enddo
            write(io,*)
         enddo

      endif

  100 continue


! Explicitly added terms
      if (nadd.gt.0) then
         write(io,20) ' '
         write(io,20) '# Explicitly added terms'
         write(io,20) ' '
         do n=1,nadd
            s=termst(1,n)
            s1=termst(2,n)
            chk=.true.
            nterms=0
            do i=1,nmodes
               if (termdef(i,n).gt.0) then
                  nterms=nterms+1
                  modemap(n,nterms)=i
                  if (mfit(i).eq.0) chk=.false.
               endif
            enddo
            if (chk.eqv..true..and.sfit(s).ne.0.and.sfit(s1).ne.0) then
               sl=sdum(s)
               sl1=sdum(s1)
               if (n.lt.10) then
                  write(atmp,'(a5,i1,a6,i1)') 'termk',n,'/factk',n
                  ilbl4 = 13
               else if (n.ge.10 .and. n .lt. 100) then
                  write(atmp,'(a5,i2,a6,i2)') 'termk',n,'/factk',n
                  ilbl4 = 15
               else if (n.ge.100) then
                  write(atmp,'(a5,i3,a6,i3)') 'termk',n,'/factk',n
                  ilbl4 = 17
               endif
               c=1
               do i=1,nterms
                  m=modemap(n,i)
                  if (qtransform(m) .ne. 0) then
                     call getopstring(qtransform(m),opstring,termdef(m,n))
                  else
                     write(opstring,'(a2,i3)') 'q^',termdef(m,n)
                     call rmblnk(opstring)
                  endif
                  if (m.lt.10) then
                     write(atmp1(c:c+15),140) ' |',m,opstring
                     c=c+10
                  else
                     write(atmp1(c:c+16),141) ' |',m,opstring
                     c=c+11
                  endif
               enddo
               write(atmp2,'(a,i5)') '|',l
               write(atmp3,'(2(a,i5))') 'S',sl,'&',sl1
               call rmblnk(atmp2)
               call rmblnk(atmp3)
               ilbl2=index(atmp2,' ')-1
               ilbl3=index(atmp3,' ')-1
               write(io,'(4a)') atmp(1:ilbl4),atmp1(1:c+1),&
                         atmp2(1:ilbl2+2),atmp3(1:ilbl3+2)
            endif
         enddo
      endif 

      write(io,20) ' '
      write(io,20) '# General harmonic potential'
      write(io,20) ' '
      do i=1,nmodes
         do s=1,nsta
            if (mfit(i).ne.0) then
               if (sfit(s).ne.0) then
                  if (qtransform(i) .ne. 0) then
                     call getopstring(qtransform(i),opstring,2)
                  else
                     opstring='q^2'
                  endif
                  sl=sdum(s)
                  if (ifdiab(i,s).eq.8) then
                     write(atmp,*) '0.5*gh_',i,'_',s
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',l
                     write(atmp3,'(2(a,i5))') 'S',sl,'&',sl
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     write(io,'(5a)') atmp(1:ilbl+2),atmp1(1:ilbl1+2),&
                          opstring,atmp2(1:ilbl2+2),atmp3(1:ilbl3)
                  endif
               endif
            endif
         enddo
      enddo

      write(io,20) ' '
      write(io,20) '# General quadratic potential (x + x**2)'
      write(io,20) ' '
      do i=1,nmodes
         do s=1,nsta
            if (mfit(i).ne.0) then
               if (sfit(s).ne.0) then
                  sl=sdum(s)
                  if (ifdiab(i,s).eq.15) then
                     write(atmp,*) 'gqq_',i,'_',s,'_1'
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',l
                     write(atmp3,'(2(a,i5))') 'S',sl,'&',sl
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(i),opstring,1)
                     else
                        opstring='q'
                     endif
                     write(io,'(5a)') atmp(1:ilbl+2),atmp1(1:ilbl1+2),&
                          opstring,atmp2(1:ilbl2+2),atmp3(1:ilbl3)
                     
                     write(atmp,*) '0.5*gqq_',i,'_',s,'_2'
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',l
                     write(atmp3,'(2(a,i5))') 'S',sl,'&',sl
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(i),opstring,2)
                     else
                        opstring='q^2'
                     endif
                     write(io,'(5a)') atmp(1:ilbl+2),atmp1(1:ilbl1+2),&
                          opstring,atmp2(1:ilbl2+2),atmp3(1:ilbl3)
                     
                  endif
               endif
            endif
         enddo
      enddo
      write(io,20) ' '
      write(io,20) '# General quartic (minus cubic) potential'
      write(io,20) ' '
      do i=1,nmodes
         do s=1,nsta
            if (mfit(i).ne.0) then
               if (sfit(s).ne.0) then
                  sl=sdum(s)
                  if (ifdiab(i,s).eq.9) then
                     write(atmp,*) 'gq_',i,'_',s,'_1'
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',l
                     write(atmp3,'(2(a,i5))') 'S',sl,'&',sl
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(i),opstring,1)
                     else
                        opstring='q'
                     endif
                     write(io,'(5a)') atmp(1:ilbl+2),atmp1(1:ilbl1+2),&
                          opstring,atmp2(1:ilbl2+2),atmp3(1:ilbl3)
                     
                     write(atmp,*) '0.5*gq_',i,'_',s,'_2'
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',l
                     write(atmp3,'(2(a,i5))') 'S',sl,'&',sl
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(i),opstring,2)
                     else
                        opstring='q^2'
                     endif
                     write(io,'(5a)') atmp(1:ilbl+2),atmp1(1:ilbl1+2),&
                          opstring,atmp2(1:ilbl2+2),atmp3(1:ilbl3)
                     
                     write(atmp,*) '0.041667*gq_',i,'_',s,'_3'
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',l
                     write(atmp3,'(2(a,i5))') 'S',sl,'&',sl
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(i),opstring,4)
                     else
                        opstring='q^4'
                     endif
                     write(io,'(5a)') atmp(1:ilbl+2),atmp1(1:ilbl1+2),&
                          opstring,atmp2(1:ilbl2+2),atmp3(1:ilbl3)
                  endif
               endif
            endif
         enddo
      enddo

      write(io,20) ' '
      write(io,20) '# General quartic (inc. cubic) potential'
      write(io,20) ' '
      do i=1,nmodes
         do s=1,nsta
            if (mfit(i).ne.0) then
               if (sfit(s).ne.0) then
                  sl=sdum(s)
                  if (ifdiab(i,s).eq.12) then
                     write(atmp,*) 'gaq_',i,'_',s,'_1'
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',l
                     write(atmp3,'(2(a,i5))') 'S',sl,'&',sl
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(i),opstring,1)
                     else
                        opstring='q'
                     endif
                     write(io,'(5a)') atmp(1:ilbl+2),atmp1(1:ilbl1+2),&
                          opstring,atmp2(1:ilbl2+2),atmp3(1:ilbl3)
                     
                     write(atmp,*) '0.5*gaq_',i,'_',s,'_2'
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',l
                     write(atmp3,'(2(a,i5))') 'S',sl,'&',sl
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(i),opstring,2)
                     else
                        opstring='q^2'
                     endif
                     write(io,'(5a)') atmp(1:ilbl+2),atmp1(1:ilbl1+2),&
                          opstring,atmp2(1:ilbl2+2),atmp3(1:ilbl3)
                     
                     write(atmp,*) '0.166667*gaq_',i,'_',s,'_3'
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',l
                     write(atmp3,'(2(a,i5))') 'S',sl,'&',sl
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(i),opstring,3)
                     else
                        opstring='q^3'
                     endif
                     write(io,'(5a)') atmp(1:ilbl+2),atmp1(1:ilbl1+2),&
                          opstring,atmp2(1:ilbl2+2),atmp3(1:ilbl3)

                     write(atmp,*) '0.041667*gaq_',i,'_',s,'_4'
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,'(a,i5)') '|',l
                     write(atmp3,'(2(a,i5))') 'S',sl,'&',sl
                     call rmblnk(atmp)
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     ilbl=index(atmp,' ')-1
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     if (qtransform(i) .ne. 0) then
                        call getopstring(qtransform(i),opstring,4)
                     else
                        opstring='q^4'
                     endif
                     write(io,'(5a)') atmp(1:ilbl+2),atmp1(1:ilbl1+2),&
                          opstring,atmp2(1:ilbl2+2),atmp3(1:ilbl3)
                  endif
               endif
            endif
         enddo
      enddo

      write(io,20) ' '
      write(io,20) '# General Morse and A.C potentials'
      write(io,20) ' '
      do i=1,nmodes
         do s=1,nsta
            if (mfit(i).ne.0) then
               if (sfit(s).ne.0) then
                  sl=sdum(s)
                  if (ifdiab(i,s).eq.6.or.ifdiab(i,s).eq.7) then
                     write(atmp1,'(a,i5)') '|',i
                     write(atmp2,*) 'v_',i,'_',s
                     write(atmp3,'(a,i5)') '|',l
                     write(atmp4,'(2(a,i5))') 'S',sl,'&',sl
                     call rmblnk(atmp1)
                     call rmblnk(atmp2)
                     call rmblnk(atmp3)
                     call rmblnk(atmp4)
                     ilbl1=index(atmp1,' ')-1
                     ilbl2=index(atmp2,' ')-1
                     ilbl3=index(atmp3,' ')-1
                     ilbl4=index(atmp4,' ')-1
                     write(io,'(5a)') ' 1.0   ',atmp1(1:ilbl1+5),&
                          atmp2(1:ilbl2+5),atmp3(1:ilbl3+5),&
                          atmp4(1:ilbl4)
                     
                  endif
               endif
            endif
         enddo
      enddo

      write(io,20) ' '
      write(io,20) '# Exponential potential'
      write(io,20) ' '
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         do s=1,nsta
            if (sfit(s) .eq. 0) cycle
            if (ifdiab(i,s) .eq. 5) then
               sl=sdum(s)
               write(atmp1,'(a,i5)') '|',i
               write(atmp2,'(2(a,i5))') 'v',s,'m',i
               write(atmp3,'(a,i5)') '|',l
               write(atmp4,'(2(a,i5))') 'S',sl,'&',sl
               call rmblnk(atmp1)
               call rmblnk(atmp2)
               call rmblnk(atmp3)
               call rmblnk(atmp4)
               ilbl1=index(atmp1,' ')-1
               ilbl2=index(atmp2,' ')-1
               ilbl3=index(atmp3,' ')-1
               ilbl4=index(atmp4,' ')-1
               write(io,'(5a)') ' 1.0   ',atmp1(1:ilbl1+5),&
                      atmp2(1:ilbl2+5),atmp3(1:ilbl3+5),atmp4(1:ilbl4)

            endif
         enddo
      enddo
 

      write(io,20) ' '
      write(io,20) '# Morse/Anti-Morse potential'
      write(io,20) ' '
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         do s=1,nsta
            if (sfit(s) .eq. 0) cycle
            if (ifdiab(i,s) .eq. 2 .or. ifdiab(i,s) .eq. 4) then
               sl=sdum(s)
               write(atmp1,'(a,i5)') '|',i
               write(atmp2,'(2(a,i5))') 'v',s,'m',i
               write(atmp3,'(a,i5)') '|',l
               write(atmp4,'(2(a,i5))') 'S',sl,'&',sl
               call rmblnk(atmp1)
               call rmblnk(atmp2)
               call rmblnk(atmp3)
               call rmblnk(atmp4)
               ilbl1=index(atmp1,' ')-1
               ilbl2=index(atmp2,' ')-1
               ilbl3=index(atmp3,' ')-1
               ilbl4=index(atmp4,' ')-1
               write(io,'(5a)') ' 1.0   ',atmp1(1:ilbl1+5),&
                      atmp2(1:ilbl2+5),atmp3(1:ilbl3+5),atmp4(1:ilbl4)

            endif
         enddo
      enddo
      
      write(io,20) '# Lennard-Jones/Anti-LJ potential'
      write(io,20) ' '
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         do s=1,nsta
            if (sfit(s) .eq. 0) cycle
            if (ifdiab(i,s) .eq. 10 .or. ifdiab(i,s) .eq. 11) then
               sl=sdum(s)
               write(atmp1,'(a,i5)') '|',i
               write(atmp2,'(2(a,i5))') 'v',s,'m',i
               write(atmp3,'(a,i5)') '|',l
               write(atmp4,'(2(a,i5))') 'S',sl,'&',sl
               call rmblnk(atmp1)
               call rmblnk(atmp2)
               call rmblnk(atmp3)
               call rmblnk(atmp4)
               ilbl1=index(atmp1,' ')-1
               ilbl2=index(atmp2,' ')-1
               ilbl3=index(atmp3,' ')-1
               ilbl4=index(atmp4,' ')-1
               write(io,'(5a)') ' 1.0   ',atmp1(1:ilbl1+5),&
                      atmp2(1:ilbl2+5),atmp3(1:ilbl3+5),atmp4(1:ilbl4)

            endif
         enddo
      enddo

      write(io,20) ' '
      write(io,20) '# Quartic potential'
      write(io,20) ' '
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         do s=1,nsta
            if (sfit(s) .eq. 0) cycle
            if (ifdiab(i,s) .eq. 1) then
             if ((expcoe_mask(3,i,s) .ne. 0) .and. (abs(expcoe(3,i,s)) .gt. 0.000001_dop) ) then
               sl=sdum(s)
               write(atmp,'(a9,i5,a,i5)') '0.041667*',s,'EPm',i
               write(atmp1,'(a,i5)') '|',i
               write(atmp3,'(a,i5)') '|',l
               write(atmp4,'(2(a,i5))') 'S',sl,'&',sl
               call rmblnk(atmp)
               call rmblnk(atmp1)
               call rmblnk(atmp3)
               call rmblnk(atmp4)
               ilbl2=index(atmp,' ')-1
               ilbl1=index(atmp1,' ')-1
               ilbl3=index(atmp3,' ')-1
               ilbl4=index(atmp4,' ')-1
               if (qtransform(i) .ne. 0) then
                  call getopstring(qtransform(i),opstring,4)
               else
                  opstring='q^4'
               endif
               write(io,'(5a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp3(1:ilbl3+5),atmp4(1:ilbl4)
             endif
             if ( (expcoe_mask(2,i,s) .ne. 0) .and. (abs(expcoe(2,i,s)) .gt. 0.000001_dop) ) then
               sl=sdum(s)
               write(atmp,'(a8,i5,a,i5)') '0.50000*',s,'GAm',i
               write(atmp1,'(a,i5)') '|',i
               write(atmp3,'(a,i5)') '|',l
               write(atmp4,'(2(a,i5))') 'S',sl,'&',sl
               call rmblnk(atmp)
               call rmblnk(atmp1)
               call rmblnk(atmp3)
               call rmblnk(atmp4)
               ilbl2=index(atmp,' ')-1
               ilbl1=index(atmp1,' ')-1
               ilbl3=index(atmp3,' ')-1
               ilbl4=index(atmp4,' ')-1
               if (qtransform(i) .ne. 0) then
                  call getopstring(qtransform(i),opstring,2)
               else
                  opstring='q^2'
               endif
               write(io,'(5a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp3(1:ilbl3+5),atmp4(1:ilbl4)
             endif
             if ((expcoe_mask(1,i,s) .ne. 0) .and. (abs(expcoe(1,i,s)) .gt. 0.000001_dop) ) then
               sl=sdum(s)
               write(atmp,'(a8,i5,a,i5)') '1.00000*',s,'KAm',i
               write(atmp1,'(a,i5)') '|',i
               write(atmp3,'(a,i5)') '|',l
               write(atmp4,'(2(a,i5))') 'S',sl,'&',sl
               call rmblnk(atmp)
               call rmblnk(atmp1)
               call rmblnk(atmp3)
               call rmblnk(atmp4)
               ilbl2=index(atmp,' ')-1
               ilbl1=index(atmp1,' ')-1
               ilbl3=index(atmp3,' ')-1
               ilbl4=index(atmp4,' ')-1
               if (qtransform(i) .ne. 0) then
                  call getopstring(qtransform(i),opstring,1)
               else
                  opstring='q'
               endif
               write(io,'(5a)') atmp(1:ilbl+5),atmp1(1:ilbl1+5),&
                      opstring,atmp3(1:ilbl3+5),atmp4(1:ilbl4)
             endif
            endif
         enddo
      enddo

      write(io,20) 'END-HAMILTONIAN-SECTION'
      write(io,20) ' '
      write(io,20) 'END-OPERATOR'

 20   format(a)
 !80   format(a6,i1,T30,a1,i1,3x,a2)
 !81   format(a10,i1,T30,a1,i1,3x,a3)
 !82   format(a6,i2,T30,a1,i2,2x,a2)
 !83   format(a10,i2,T30,a1,i2,2x,a3)
 84   format(a3)

 110  format(a2,i1,a3)
 111  format('modes|',*(a2,i1,a2))
 112  format('modes|',*(a2,i2,a2))
 113  format('modes|',*(a2,i3,a2))
 114  format('modes|',*(a3,i1,a2), ' el')
 115  format('modes|',*(a3,i2,a2), ' el')
 116  format('modes|',*(a3,i3,a2), ' el')


 140  format(a2,i1,1x,a)
 141  format(a2,i2,1x,a)


      return
      end subroutine writeop

!#################################################################

      subroutine getopstring(qtransform,opstring,ipower)

      implicit none

      integer(long),intent(in)      :: qtransform,ipower
      integer(long)                 :: len
      character(len=6), intent(out) :: opstring

      if (qtransform .eq. 1) then
         opstring = 'cos   '
      else if (qtransform .eq. 2) then
         opstring = 'sin   '
      else if (qtransform .eq. 3) then
         opstring = 'cos1  '
      else if (qtransform .eq. 4) then
         opstring = 'sin1  '
      else
         routine='Getopstring'
         message='Unknown transformation type in op file output'
         call vcerrormsg
      endif
      len=index(opstring,' ')-1

      if (ipower .gt. 1) then
         write(opstring,'(2a,i1)') opstring(1:len),'^',ipower
      endif

      return
      end subroutine getopstring

!#################################################################
!###### write the MCTDH Parameter file ########################### 

      subroutine writepar(io,const,kappa,lambda,gamma,mu,iota,&
                 expcoe,higher,termk,const_mask,kappa_mask,&
                 lambda_mask,gamma_mask,iota_mask,mu_mask,expcoe_mask,&
                 termk_mask)

      implicit none

      integer(long) :: i,s,j,s1,nhi,k,k1,ilbl,n,m
      integer(long), intent(in) :: io
      integer(long), dimension(nsta,nsta), intent(in) :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in) :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in) :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in) :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in) :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: &
                                                          iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in) :: termk_mask

      real(dop), dimension(nhigher), intent(in)                 :: higher
      real(dop), dimension(nsta,nsta), intent(in)               :: const
      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota,mu
      real(dop), dimension(nadd), intent(in)                    :: termk

      real(dop), dimension(nadd)                                :: fact      
      character(len=64)       :: atmp

      real(dop) :: Dz,alph,Qz,Ez,factt


      write(io,20) ' '
      write(io,20) '#frequencies'
      write(io,20) ' '
      
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         if (i.lt.10) then
            write(io,21) 'omega_',i,'=',freq0(i),' , ev'
         else if (i.lt.100) then
            write(io,22) 'omega_',i,'=',freq0(i),' , ev'
         else
            write(io,23) 'omega_',i,'=',freq0(i),' , ev'
         endif
      enddo
      write(io,20) ' '
      write(io,20) '#energies'
      write(io,20) ' '
      do i=1,nsta
         if (sfit(i) .ne. 0) write(io,21) 'E',i,'=',const(i,i),' , ev'
      enddo
      if (lconstall) then
         write(io,20) ' '
         write(io,20) '#off-diagonal constants'
         write(io,20) ' '
         do i=1,nsta-1
            if (sfit(i) .eq. 0) cycle
            do j=i+1,nsta
               if (sfit(j) .eq. 0) cycle
               if (const_mask(i,j).ne.0) &
                  write(io,40) 'E',i,'_',j,'=',const(i,j),' , ev'
            enddo
         enddo
      endif
      write(io,20) ' '
      write(io,20) '#on-diagonal linear coupling constants (kappa)'
      write(io,20) ' '
      do s=1,nsta
         if (sfit(s) .eq. 0) cycle
         do i=1,nmodes
            if (mfit(i) .eq. 0) cycle
            if ((kappa_mask(i,s).ne.0) .and.&
                (abs(kappa(i,s)-0.0_dop) .gt. cutoff_op) ) then
               if (i.lt.10) then
                  write(io,40) 'kappa',s,'_',i,'=',kappa(i,s),' , ev'
               else if (i.lt.100) then
                  write(io,41) 'kappa',s,'_',i,'=',kappa(i,s),' , ev'
               else
                  write(io,42) 'kappa',s,'_',i,'=',kappa(i,s),' , ev' 
               endif
            endif
         enddo
      enddo
      write(io,20) ' '
      write(io,20) '#off-diagonal linear coupling constants (lambda)'
      write(io,20) ' '
      do s=1,nsta-1            
         if (sfit(s) .eq. 0) cycle
         do s1=s+1,nsta            
            if (sfit(s1) .eq. 0) cycle
            do i=1,nmodes
               if (mfit(i) .eq. 0) cycle
               if((lambda_mask(i,s,s1).ne.0) .and.&
                  (abs(lambda(i,s,s1)-0.0_dop) .gt. cutoff_op) ) then
                  if (i.lt.10) then
                     write(io,50) 'lambda',s,'_',s1,'_',i,'=',&
                              lambda(i,s,s1),' , ev'
                  else if (i.lt.100) then
                     write(io,51) 'lambda',s,'_',s1,'_',i,'=',&
                              lambda(i,s,s1),' , ev'
                  else
                     write(io,52) 'lambda',s,'_',s1,'_',i,'=',&
                              lambda(i,s,s1),' , ev'   
                  endif
               endif
            enddo
         enddo
      enddo
      write(io,20) ' '
      write(io,20) '#on-diagonal bilinear coupling constants (gamma)'
      write(io,20) ' '
      do s=1,nsta
         if (sfit(s) .eq. 0) cycle
         do i=1,nmodes
            if (mfit(i) .eq. 0) cycle
            do j=i,nmodes
               if (mfit(j) .eq. 0) cycle
               if ((gamma_mask(i,j,s).ne.0) .and.&
                   (abs(gamma(i,j,s)-0.0_dop) .gt. cutoff_op) ) then
                  if (i.lt.10) then
                     if (j.lt.10) then
                        write(io,60) 'gamma',s,'_',i,'_',j,'=',&
                             gamma(i,j,s),' , ev'
                     else if (j.lt.100) then
                        write(io,61) 'gamma',s,'_',i,'_',j,'=',&
                             gamma(i,j,s),' , ev'
                     else 
                        write(io,64) 'gamma',s,'_',i,'_',j,'=',&
                             gamma(i,j,s),' , ev'
                     endif

                  
             
                  else  if (i.lt.100) then
                     if (j.lt.10) then
                        write(io,62) 'gamma',s,'_',i,'_',j,'=',&
                             gamma(i,j,s),' , ev'
                     else if (j.lt.100) then
                        write(io,63) 'gamma',s,'_',i,'_',j,'=',&
                             gamma(i,j,s),' , ev'
                     else 
                        write(io,65) 'gamma',s,'_',i,'_',j,'=',&
                             gamma(i,j,s),' , ev'
                     endif
             
                   else
                     if (j.lt.10) then
                        write(io,66) 'gamma',s,'_',i,'_',j,'=',&
                             gamma(i,j,s),' , ev'
                     else if (j.lt.100) then
                        write(io,67) 'gamma',s,'_',i,'_',j,'=',&
                             gamma(i,j,s),' , ev'
                     else 
                        write(io,68) 'gamma',s,'_',i,'_',j,'=',&
                             gamma(i,j,s),' , ev'
                     endif
                  endif
               endif
            enddo
         enddo
      enddo
      write(io,20) ' '
      write(io,20) '#Third order quad-lin and cube terms (iota) '
      write(io,20) ' '
      do s=1,nsta
         if (sfit(s) .eq. 0) cycle
         do i=1,nmodes
            if (mfit(i) .eq. 0) cycle
            do j=1,nmodes
               if (mfit(j) .eq. 0) cycle
               if ((iota_mask(i,j,s,s).ne.0) .and.&
                   (abs(iota(i,j,s,s)-0.0_dop) .gt. cutoff_op) ) then
                  if (i.lt.10) then
                     if (j.lt.10) then
                        write(io,60) 'iota',s,'_',i,'_',j,'=',&
                             iota(i,j,s,s),' , ev'
                     else if (j.lt.100) then
                        write(io,61) 'iota',s,'_',i,'_',j,'=',&
                             iota(i,j,s,s),' , ev'
                     else
                        write(io,64) 'iota',s,'_',i,'_',j,'=',&
                             iota(i,j,s,s),' , ev'
                     endif
                  else if (i.lt.100) then
                     if (j.lt.10) then
                        write(io,62) 'iota',s,'_',i,'_',j,'=',&
                             iota(i,j,s,s),' , ev'
                     else if (j.lt.100) then
                        write(io,63) 'iota',s,'_',i,'_',j,'=',&
                             iota(i,j,s,s),' , ev'
                     else
                        write(io,65) 'iota',s,'_',i,'_',j,'=',&
                             iota(i,j,s,s),' , ev'
                     endif
                  else
                     if (j.lt.10) then
                        write(io,66) 'iota',s,'_',i,'_',j,'=',&
                             iota(i,j,s,s),' , ev'
                     else if (j.lt.100) then
                        write(io,67) 'iota',s,'_',i,'_',j,'=',&
                             iota(i,j,s,s),' , ev'
                     else
                        write(io,68) 'iota',s,'_',i,'_',j,'=',&
                             iota(i,j,s,s),' , ev'
                     endif
                  endif
               endif
            enddo
         enddo
      enddo
      write(io,20) ' '
      write(io,20) '#Third order quad-lin and cube terms (iota2) '
      write(io,20) ' '
      do s=1,nsta-1
         if (sfit(s) .eq. 0) cycle
         do s1=s+1,nsta
            if (sfit(s1) .eq. 0) cycle
            do i=1,nmodes
               if (mfit(i) .eq. 0) cycle
               do j=1,nmodes
                  if (mfit(j) .eq. 0) cycle
                  if ((iota_mask(i,j,s,s1).ne.0) .and.&
                      (abs(iota(i,j,s,s1)-0.0_dop) .gt. cutoff_op) ) then
                     if (i.lt.10) then
                        if (j.lt.10) then
                           write(io,70) 'iota2',s,'_',s1,'_',&
                              i,'_',j,'=',iota(i,j,s,s1),' , ev'
                        else
                           write(io,71) &
                              'iota',s,'_',s1,'_',i,'_',j,'=',&
                                    iota(i,j,s,s1),' , ev'
                        endif
                     else
                        if (j.lt.10) then
                           write(io,72) &
                              'iota',s,'_',s1,'_',i,'_',j,'=',&
                                    iota(i,j,s,s1),' , ev'

                        else 
                           write(io,73) &
                              'iota',s,'_',s1,'_',i,'_',j,'=',&
                                    iota(i,j,s,s1),' , ev'
                        endif
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo

      write(io,20) ' '
      write(io,20) '#off-diagonal bilinear coupling constants (mu)'
      write(io,20) ' '
      do s=1,nsta-1            
         if (sfit(s) .eq. 0) cycle
         do s1=s+1,nsta            
            if (sfit(s1) .eq. 0) cycle
            do i=1,nmodes
               if (mfit(i) .eq. 0) cycle
               do j=i,nmodes
                  if (mfit(j) .eq. 0) cycle
                  if ( (mu_mask(i,j,s,s1).ne.0 ) .and.&
                      (abs(mu(i,j,s,s1)-0.0_dop) .gt. cutoff_op) ) then
                     if (i.lt.10) then
                        if (j.lt.10) then
                           write(io,70) &
                              'mu',s,'_',s1,'_',i,'_',j,'=',&
                                    mu(i,j,s,s1),' , ev'
                        else 
                           write(io,71) &
                              'mu',s,'_',s1,'_',i,'_',j,'=',&
                                    mu(i,j,s,s1),' , ev'
                        endif
                     else
                        if (j.lt.10) then
                           write(io,72) &
                              'mu',s,'_',s1,'_',i,'_',j,'=',&
                                    mu(i,j,s,s1),' , ev'

                        else 
                           write(io,73) &
                              'mu',s,'_',s1,'_',i,'_',j,'=',&
                                    mu(i,j,s,s1),' , ev'
                        endif
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo

      if (highord .ge. 3) then
         nhi = 0
         write(io,20) ' '
         write(io,20) '#on-diagonal lin-lin-lin coupling '
         write(io,20) ' '

         do s = 1,nsta
            do i=1,nmodes
               do j = i+1,nmodes
                  do k = j+1,nmodes
                     nhi = nhi+1
                     write(atmp,'(4(a,i5))') &
                             'par',s,'_',i,'_',j,'_',k
                     call rmblnk(atmp)
                     ilbl=index(atmp,' ')-1
                     write(io,'(2a,f10.5,a)') atmp(1:ilbl+5),&
                       ' = ',higher(nhi),' , ev'
                  enddo
               enddo
            enddo
         enddo
      endif

      if (highord .eq. 4) then

! gam_iijj q_i^2 q_j^2 terms
         write(io,20) ' '
         write(io,20) '#on-diagonal quad-quad coupling '
         write(io,20) ' '
         do s=1,nsta
            do i=1,nmodes-1
               do j = i+1,nmodes
                  nhi = nhi+1
                  write(atmp,'(5(a,i5))') &
                             'par',s,'_',i,'_',i,'_',j,'_',j
                  call rmblnk(atmp)
                  ilbl=index(atmp,' ')-1
                  write(io,'(2a,f10.5,a)') atmp(1:ilbl+5),&
                       ' = ',higher(nhi),' , ev'
               enddo
            enddo
         enddo

! gam_iiij q_i^3 q_j terms
         write(io,20) ' '
         write(io,20) '#on-diagonal cube-lin coupling '
         write(io,20) ' '
         do s=1,nsta
            do i=1,nmodes-1
               do j = i+1,nmodes
                  nhi = nhi+1
                  write(atmp,'(5(a,i5))') &
                             'par',s,'_',i,'_',i,'_',i,'_',j
                  call rmblnk(atmp)
                  ilbl=index(atmp,' ')-1
                  write(io,'(2a,f10.5,a)') atmp(1:ilbl+5),&
                       ' = ',higher(nhi),' , ev'
               enddo
            enddo
            do i=2,nmodes
               do j = 1,i-1
                  nhi = nhi+1
                  write(atmp,'(5(a,i5))') &
                             'par',s,'_',i,'_',i,'_',i,'_',j
                  call rmblnk(atmp)
                  ilbl=index(atmp,' ')-1
                  write(io,'(2a,f10.5,a)') atmp(1:ilbl+5),&
                       ' = ',higher(nhi),' , ev'
               enddo
            enddo
         enddo

! gam_iijk q_i^2 q_j q_k terms
         write(io,20) ' '
         write(io,20) '#on-diagonal quad-lin-lin coupling '
         write(io,20) ' '
         do s=1,nsta
            do i=1,nmodes-2
               do j = i+1,nmodes-1
                  do k = j+1,nmodes
                     nhi = nhi+1
                     write(atmp,'(5(a,i5))') &
                             'par',s,'_',i,'_',i,'_',j,'_',k
                     call rmblnk(atmp)
                     ilbl=index(atmp,' ')-1
                     write(io,'(2a,f10.5,a)') atmp(1:ilbl+5),&
                          ' = ',higher(nhi),' , ev'
                  enddo
               enddo
            enddo
            do i=3,nmodes
               do j = 1,i-2
                  do k = j+1,i-1
                     nhi = nhi+1
                     write(atmp,'(5(a,i5))') &
                             'par',s,'_',i,'_',i,'_',j,'_',k
                     call rmblnk(atmp)
                     ilbl=index(atmp,' ')-1
                     write(io,'(2a,f10.5,a)') atmp(1:ilbl+5),&
                          ' = ',higher(nhi),' , ev'
                  enddo
               enddo
            enddo
            do i=2,nmodes-1
               do j = 1,i-1
                  do k = i+1,nmodes
                     nhi = nhi+1
                     write(atmp,'(5(a,i5))') &
                             'par',s,'_',i,'_',i,'_',j,'_',k
                     call rmblnk(atmp)
                     ilbl=index(atmp,' ')-1
                     write(io,'(2a,f10.5,a)') atmp(1:ilbl+5),&
                          ' = ',higher(nhi),' , ev'
                  enddo
               enddo
            enddo
            write(io,*)
         enddo

! gam_ijkl q_i q_j q_k q_l terms
         write(io,20) ' '
         write(io,20) '#on-diagonal lin-lin-lin-lin coupling '
         write(io,20) ' '
         do s=1,nsta
            do i=1,nmodes-3
               do j = i+1,nmodes-2
                  do k = j+1,nmodes-1
                     do k1 = k+1,nmodes
                        nhi = nhi+1
                        write(atmp,'(5(a,i5))') &
                             'par',s,'_',i,'_',j,'_',k,'_',k1
                        call rmblnk(atmp)
                        ilbl=index(atmp,' ')-1
                        write(io,'(2a,f10.5,a)') atmp(1:ilbl+5),&
                          ' = ',higher(nhi),' , ev'
                     enddo
                  enddo
               enddo
            enddo
            write(io,*)
         enddo

      endif

! explicitly added terms
! mutliplied by the factorial term and then divided by it later on (in the
! note: this factorial term is incorrect, but im leaving it like this since all vcham uses this
! mistake it makes no ultimate difference to the model.
      write(io,20) ' '
      write(io,20) '# Explicitly added terms '
      write(io,20) ' '
      do n=1,nadd
       fact(n) = 0
       do m=1,nmodes
        fact(n) = fact(n) + termdef(m,n)
       enddo
       factt = 1
       j = int(fact(n))
       do i = 1, j
         factt = factt * i
       enddo
       fact(n) = factt
         if (n.lt.10) then
            write(io,150) 'termk',n,'=',fact(n)*termk(n),' , ev'
         elseif(n .lt. 100) then
            write(io,151) 'termk',n,'=',fact(n)*termk(n),' , ev'
         else
            write(io,152) 'termk',n,'=',fact(n)*termk(n),' , ev'
         endif
         if (n.lt.10) then
            write(io,150) 'factk',n,'=',fact(n),' '
         elseif (n.lt.100) then
            write(io,151) 'factk',n,'=',fact(n),' '
         else
            write(io,152) 'factk',n,'=',fact(n),' '
         endif
      enddo

      write(io,20) ' '
      write(io,20) '# Diabatic curves with parameters'
      write(io,20) ' '
      do i=1,nmodes
         if (mfit(i) .eq. 0) cycle
         do s=1,nsta
            if (sfit(s) .eq. 0) cycle
            if (ncoeff(i,s).gt.0) then
               if (ifdiab(i,s) .eq. 1) then 
                if ( (expcoe_mask(3,i,s) .ne. 0) .and. (abs(expcoe(3,i,s)) .gt. 0.000001_dop) ) then
!	epsilon!
                      if (i .lt. 10) then
                         write(io,92)&
                         s,'EP','m',i,'= ',expcoe(3,i,s),' , ev'
                      else
                         write(io,93)&
                         s,'EP','m',i,'= ',expcoe(3,i,s),' , ev'
                      endif
                endif
                if ( (expcoe_mask(2,i,s) .ne. 0) .and. (abs(expcoe(2,i,s)) .gt. 0.000001_dop) ) then
!	gammas!
                      if (i .lt. 10) then
                         write(io,92)&
                         s,'GA','m',i,'= ',expcoe(2,i,s),' , ev'
                      else
                         write(io,93)&
                         s,'GA','m',i,'= ',expcoe(2,i,s),' , ev'
                      endif
                endif
                if ( (expcoe_mask(1,i,s) .ne. 0) .and. (abs(expcoe(1,i,s)) .gt. 0.000001_dop) ) then
!	kappas! 
                      if (i .lt. 10) then
                         write(io,92)&
                         s,'KA','m',i,'= ',expcoe(1,i,s),' , ev'
                      else
                         write(io,93)&
                         s,'KA','m',i,'= ',expcoe(1,i,s),' , ev'
                      endif
                  endif
              else if (ifdiab(i,s) .eq. 2 .or. ifdiab(i,s) .eq. 4) then
                  Dz=expcoe(1,i,s)
                  alph=(freq0(i)+expcoe(2,i,s))/(2*Dz)
                  alph=sqrt(alph)
                  if (ifdiab(i,s).eq.4) alph=-alph
                  Qz=expcoe(3,i,s)
                  Ez=-(Dz*(exp(alph*Qz)-1)**2)
                  if (i .lt. 10) then
                     write(io,90)&
                          s,'D','_',i,'= ',Dz,' , ev'
                     write(io,90)&
                          s,'A','_',i,'= ',alph,'     '
                     write(io,90)&
                          s,'X','_',i,'= ',Qz,'     '
                     write(io,90)&
                          s,'E','_',i,'= ',Ez,' , ev'
                      else
                      write(io,91)&
                           s,'D','_',i,'= ',Dz,' , ev'
                      write(io,91)&
                           s,'A','_',i,'= ',alph,'     '
                      write(io,91)&
                           s,'X','_',i,'= ',Qz,'     '
                      write(io,91)&
                           s,'E','_',i,'= ',Ez,' , ev'
                   endif
              else if (ifdiab(i,s) .eq. 10 .or. ifdiab(i,s) .eq. 11) then
                  Dz=expcoe(1,i,s)
                  alph=expcoe(2,i,s)
                  Qz=expcoe(3,i,s)
                  Ez=expcoe(4,i,s)
                  if (ifdiab(i,s).eq.11) alph=-alph
                  if (i .lt. 10) then
                     write(io,90)&
                          s,'D','_',i,'= ',Dz,' , ev'
                     write(io,90)&
                          s,'A','_',i,'= ',alph,'     '
                     write(io,90)&
                          s,'X','_',i,'= ',Qz,'     '
                     write(io,90)&
                          s,'E','_',i,'= ',Ez,' , ev'
                      else
                      write(io,91)&
                           s,'D','_',i,'= ',Dz,' , ev'
                      write(io,91)&
                           s,'A','_',i,'= ',alph,'     '
                      write(io,91)&
                           s,'X','_',i,'= ',Qz,'     '
                      write(io,91)&
                           s,'E','_',i,'= ',Ez,' , ev'
                   endif

               else if (ifdiab(i,s) .eq. 3) then
                  print*, 'WARNING Buckingham parameter printing not implemented'
               else if (ifdiab(i,s) .eq. 5) then
                  if (i .lt. 10) then
                     write(io,90)&
                       s,'E','_',i,'= ',expcoe(1,i,s),' , ev'
                     write(io,90)&
                       s,'A','_',i,'= ',expcoe(2,i,s),'     '
                  else
                     write(io,91)&
                       s,'E','_',i,'= ',expcoe(1,i,s),' , ev'
                     write(io,91)&
                       s,'A','_',i,'= ',expcoe(2,i,s),'     '
                  endif

               else if (ifdiab(i,s) .eq. 6) then
                  if (i .lt. 10) then
                     write(io,94) 'ac_',i,'_',s,'_1 ','=',expcoe(k,i,s),' , ev'
                     write(io,94) 'ac_',i,'_',s,'_2 ','=',expcoe(2,i,s),'     '
                     write(io,94) 'ac_',i,'_',s,'_3 ','=',expcoe(3,i,s),'     '
                     write(io,94) 'ac_',i,'_',s,'_4 ','=',expcoe(4,i,s),' , ev'
                     write(io,94) 'ac_',i,'_',s,'_5 ','=',expcoe(5,i,s),'     '
                     write(io,94) 'ac_',i,'_',s,'_6 ','=',expcoe(6,i,s),'     '
                     write(io,94) 'ac_',i,'_',s,'_7 ','=',expcoe(7,i,s),' , ev'
                     write(io,94) 'ac_',i,'_',s,'_8 ','=',expcoe(8,i,s),' , ev'
                  else
                     write(io,95) 'ac_',i,'_',s,'_1 ','=',expcoe(1,i,s),' , ev'
                     write(io,95) 'ac_',i,'_',s,'_2 ','=',expcoe(2,i,s),'     '
                     write(io,95) 'ac_',i,'_',s,'_3 ','=',expcoe(3,i,s),'     '
                     write(io,95) 'ac_',i,'_',s,'_4 ','=',expcoe(4,i,s),' , ev'
                     write(io,95) 'ac_',i,'_',s,'_5 ','=',expcoe(5,i,s),'     '
                     write(io,95) 'ac_',i,'_',s,'_6 ','=',expcoe(6,i,s),'     '
                     write(io,95) 'ac_',i,'_',s,'_7 ','=',expcoe(7,i,s),' , ev'
                     write(io,95) 'ac_',i,'_',s,'_8 ','=',expcoe(8,i,s),' , ev'
                  endif

               else if (ifdiab(i,s) .eq. 7) then
                  if (i .lt. 10) then
                     write(io,94) 'gm_',i,'_',s,'_1 ','=',expcoe(1,i,s),' , ev'
                     write(io,94) 'gm_',i,'_',s,'_2 ','=',expcoe(2,i,s),'     '
                     write(io,94) 'gm_',i,'_',s,'_3 ','=',expcoe(3,i,s),'     '
                  else
                     write(io,95) 'gm_',i,'_',s,'_1 ','=',expcoe(1,i,s),' , ev'
                     write(io,95) 'gm_',i,'_',s,'_2 ','=',expcoe(2,i,s),'     '
                     write(io,95) 'gm_',i,'_',s,'_3 ','=',expcoe(3,i,s),'     '
                  endif
               else if (ifdiab(i,s) .eq. 8) then
                  if (i .lt. 10) then
                     write(io,96) 'gh_',i,'_',s,' =',expcoe(1,i,s),' , ev'
                  else
                     write(io,97) 'gh_',i,'_',s,' =',expcoe(1,i,s),' , ev'
                  endif

               else if (ifdiab(i,s) .eq. 9) then
                  if (i .lt. 10) then
                     write(io,94) 'gq_',i,'_',s,'_1 ','=',expcoe(1,i,s),' , ev'
                     write(io,94) 'gq_',i,'_',s,'_2 ','=',expcoe(2,i,s),' , ev'
                     write(io,94) 'gq_',i,'_',s,'_3 ','=',expcoe(3,i,s),' , ev'
                  else
                     write(io,95) 'gq_',i,'_',s,'_1 ','=',expcoe(1,i,s),' , ev'
                     write(io,95) 'gq_',i,'_',s,'_2 ','=',expcoe(2,i,s),' , ev'
                     write(io,95) 'gq_',i,'_',s,'_3 ','=',expcoe(3,i,s),' , ev'
                  endif
               else if (ifdiab(i,s) .eq. 15) then
                  if (i .lt. 10) then
                     write(io,98) 'gqq_',i,'_',s,'_1 ','=',expcoe(1,i,s),' , ev'
                     write(io,98) 'gqq_',i,'_',s,'_2 ','=',expcoe(2,i,s),' , ev'
                     write(io,98) 'gqq_',i,'_',s,'_3 ','=',expcoe(3,i,s),' , ev'
                  else
                     write(io,99) 'gqq_',i,'_',s,'_1 ','=',expcoe(1,i,s),' , ev'
                     write(io,99) 'gqq_',i,'_',s,'_2 ','=',expcoe(2,i,s),' , ev'
                     write(io,99) 'gqq_',i,'_',s,'_3 ','=',expcoe(3,i,s),' , ev'
                  endif

               else if (ifdiab(i,s) .eq. 12) then
                  if (i .lt. 10) then
                     write(io,140) 'gaq_',i,'_',s,'_1 ','=',expcoe(1,i,s),' , ev'
                     write(io,140) 'gaq_',i,'_',s,'_2 ','=',expcoe(2,i,s),' , ev'
                     write(io,140) 'gaq_',i,'_',s,'_3 ','=',expcoe(3,i,s),' , ev'
                     write(io,140) 'gaq_',i,'_',s,'_4 ','=',expcoe(4,i,s),' , ev'
                  else
                     write(io,141) 'gaq_',i,'_',s,'_1 ','=',expcoe(1,i,s),' , ev'
                     write(io,141) 'gaq_',i,'_',s,'_2 ','=',expcoe(2,i,s),' , ev'
                     write(io,141) 'gaq_',i,'_',s,'_3 ','=',expcoe(3,i,s),' , ev'
                     write(io,141) 'gaq_',i,'_',s,'_4 ','=',expcoe(4,i,s),' , ev'
                  endif

                  
               endif
               


               
            endif

         enddo
      enddo

 20   format(a)
 21   format(a,i1,T25,a1,F10.5,a5)
 22   format(a,i2,T25,a1,F10.5,a5)
 23   format(a,i3,T25,a1,F10.5,a5)

 40   format(a,i1,a1,i1,T25,a1,F10.5,a5)
 41   format(a,i1,a1,i2,T25,a1,F10.5,a5)
 42   format(a,i1,a1,i3,T25,a1,F10.5,a5)

! 42   format(a7,i1,a1,i1,a1,i1,T25,a1,F10.5,a5)
! 43   format(a7,i2,a1,i1,a1,i1,T25,a1,F10.5,a5)

! 45   format(a,i1,a1,i1,T25,a1,F10.7,a5)
! 46   format(a,i1,a1,i2,T25,a1,F10.7,a5)
 
 50   format(a,i1,a1,i1,a1,i1,T25,a,F10.5,a5)
 51   format(a,i1,a1,i1,a1,i2,T25,a,F10.5,a5)
 52   format(a,i1,a1,i1,a1,i3,T25,a,F10.5,a5)

! 52   format(a8,i1,a1,i1,a1,i1,a1,i1,T25,a1,F10.5,a5)
! 53   format(a8,i2,a1,i1,a1,i1,a1,i1,T25,a1,F10.5,a5)

 60   format(a,i1,a1,i1,a1,i1,T25,a1,F10.5,a5)
 61   format(a,i1,a1,i1,a1,i2,T25,a1,F10.5,a5)
 62   format(a,i1,a1,i2,a1,i1,T25,a1,F10.5,a5)
 63   format(a,i1,a1,i2,a1,i2,T25,a1,F10.5,a5)
 64   format(a,i1,a1,i1,a1,i3,T25,a1,F10.5,a5)
 65   format(a,i1,a1,i2,a1,i3,T25,a1,F10.5,a5)
 66   format(a,i1,a1,i3,a1,i1,T25,a1,F10.5,a5)
 67   format(a,i1,a1,i3,a1,i2,T25,a1,F10.5,a5)
 68   format(a,i1,a1,i3,a1,i3,T25,a1,F10.5,a5)

! 64   format(a7,i1,a1,i1,a1,i1,a1,i1,T25,a1,F10.5,a5)
! 65   format(a7,i1,a1,i2,a1,i1,a1,i1,T25,a1,F10.5,a5)
! 66   format(a7,i2,a1,i1,a1,i1,a1,i1,T25,a1,F10.5,a5)
! 67   format(a7,i2,a1,i2,a1,i1,a1,i1,T25,a1,F10.5,a5)
 
 70   format(a,i1,a1,i1,a1,i1,a1,i1,T25,a1,F10.5,a5)
 71   format(a,i1,a1,i1,a1,i1,a1,i2,T25,a1,F10.5,a5)
 72   format(a,i1,a1,i1,a1,i2,a1,i1,T25,a1,F10.5,a5)
 73   format(a,i1,a1,i1,a1,i2,a1,i2,T25,a1,F10.5,a5)

! 74   format(a4,i1,a1,i1,a1,i1,a1,i1,a1,i1,T25,a1,F10.5,a5)
! 75   format(a4,i1,a1,i2,a1,i1,a1,i1,a1,i1,T25,a1,F10.5,a5)
! 76   format(a4,i2,a1,i1,a1,i1,a1,i1,a1,i1,T25,a1,F10.5,a5)
! 77   format(a4,i2,a1,i2,a1,i1,a1,i1,a1,i1,T25,a1,F10.5,a5)

! 80   format(a6,i1,T30,a1,i1,3x,a2)
! 81   format(a10,i1,T30,a1,i1,3x,a3)
! 82   format(a6,i2,T30,a1,i2,2x,a2)
! 83   format(a10,i2,T30,a1,i2,2x,a3)     

! Morse and anti-morse

 90   format(i1,a1,a1,i1,T7,a2,F14.8,a5)
 91   format(i1,a1,a1,i2,T7,a2,F14.8,a5)

! Quartic

 92   format(i1,a2,a1,i1,T8,a2,F14.8,a5)
 93   format(i1,a2,a1,i2,T8,a2,F14.8,a5)

! Buckingham

! Avoided-crossing, general morse and general quartic
 94   format(a3,i1,a1,i1,a3,a1,F14.8,a5)
 98   format(a4,i1,a1,i1,a3,a1,F14.8,a5)
 95   format(a3,i2,a1,i1,a3,a1,F14.8,a5)
 99   format(a4,i2,a1,i1,a3,a1,F14.8,a5)

! General harmonic
 96   format(a3,i1,a1,i1,a2,F14.8,a5)
 97   format(a3,i2,a1,i1,a2,F14.8,a5)

! Quartic expansion terms
! 100  format(a8,i1,a1,i1,a1,i1,T25,a1,F12.9,a5)
! 101  format(a8,i2,a1,i1,a1,i1,T25,a1,F12.9,a5)

! Morse and anti-morse expansion terms
! 102  format(a4,i1,a1,i1,a1,i1,T25,a1,F12.9,a5)
! 103  format(a4,i2,a1,i1,a1,i1,T25,a1,F12.9,a5)

! (system) diabatic function decay terms
! 110  format(i1,a4,i1,a1,i1,T25,a1,F12.9)
! 111  format(i1,a4,i2,a1,i1,T25,a1,F12.9)

! Lambda decay terms
! 120  format(a8,i1,a1,i1,a1,i1,T25,a1,F10.5)
! 121  format(a8,i2,a1,i1,a1,i1,T25,a1,F10.5)

! SCE terms
! 130  format(a5,i1,a1,i1,a1,i1,T25,a1,F10.5,a5)
! 131  format(a5,i2,a1,i1,a1,i1,T25,a1,F10.5,a5)

! General asymmetric quartic terms
 140  format(a4,i1,a1,i1,a3,a1,F14.8,a5)
 141  format(a4,i2,a1,i1,a3,a1,F14.8,a5)

! Explicitly added terms
 150  format(a5,i1,1x,a1,1x,F14.8,a5)
 151  format(a5,i2,1x,a1,1x,F14.8,a5)
 152  format(a5,i3,1x,a1,1x,F14.8,a5)


      return

      end subroutine writepar


!#################################################################
!###### write the labels in the MCTDH Parameter file ############# 

      subroutine writelab(io,const,kappa,lambda,gamma,mu,iota, &
                 expcoe,termk,const_mask,kappa_mask, &
                 lambda_mask,gamma_mask,iota_mask,mu_mask,expcoe_mask, &
                 termk_mask)

      implicit none
      
      integer(long) :: s,m
      integer(long), intent(in) :: io
      integer(long), dimension(nsta,nsta), intent(in) :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in) :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in) :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in) :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in) :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: &
                                                        iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in) :: termk_mask

      real(dop), dimension(nsta,nsta), intent(in)               :: const
      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota,mu
      real(dop), dimension(nadd), intent(in)                    :: termk

!     + ,d0,alpha,x0,en0

      write(io,20) ' '
      write(io,20) '# Diabatic function labels'
      write(io,20) ' '
      do m=1,nmodes
         do s=1,nsta
            if (mfit(m).ne.0.and.sfit(s).ne.0) then
               if ((ifdiab(m,s) .eq. 2) .or. (ifdiab(m,s) .eq. 4)) then
                  if (m .lt. 10) then
                     write(io,101)&
                          'v',s,'m',m,'=morse1[',s,'D_',m,',',s,'A_',&
                          m,',',s,'X_',m,',',s,'E_',m,']'
                  else
                     write(io,102)&
                          'v',s,'m',m,'=morse1[',s,'D_',m,',',s,'A_',&
                          m,',',s,'X_',m,',',s,'E_',m,']'
                  endif

               elseif (ifdiab(m,s) .eq. 5) then
                  if (m .lt. 10) then
                     write(io,109)&
                          'v',s,'m',m,'=exp0[',s,'E_',m,',',s,'A_',m,']'
                  else
                     write(io,110)&
                          'v',s,'m',m,'=exp0[',s,'E_',m,',',s,'A_',m,']'
                  endif

               elseif ((ifdiab(m,s) .eq. 10) .or. (ifdiab(m,s) .eq. 11)) then
                  if (m .lt. 10) then
                     write(io,107)&
                          'v',s,'m',m,'=lj[',s,'D_',m,',',s,'A_',&
                          m,',',s,'X_',m,',',s,'E_',m,']'
                  else
                     write(io,108)&
                          'v',s,'m',m,'=lj[',s,'D_',m,',',s,'A_',&
                          m,',',s,'X_',m,',',s,'E_',m,']'
                  endif
                                                                                    
               else if (ifdiab(m,s).eq.6) then
                  if (m.lt.10) then
                     write(io,103) 'v_',m,'_',s,'=acpot[ac_',m,'_',s, &
                          '_1,ac_',m,'_',s,'_2,ac_',m,'_',s,'_3,ac_', &
                          m,'_',s,'_4,ac_',m,'_',s,'_5,ac_', &
                          m,'_',s,'_6,ac_',m,'_',s,'_7,ac_', &
                          m,'_',s,'_8]'
                  else
                     write(io,104) 'v_',m,'_',s,'=acpot[ac_',m,'_',s, &
                          '_1,ac_',m,'_',s,'_2,ac_',m,'_',s,'_3,ac_', &
                          m,'_',s,'_4,ac_',m,'_',s,'_5,ac_', &
                          m,'_',s,'_6,ac_',m,'_',s,'_7,ac_', &
                          m,'_',s,'_8]'
                  endif
                  
               else if (ifdiab(m,s).eq.7) then
                  if (m.lt.10) then
                     write(io,105) 'v_',m,'_',s,'=morse2[gm_',m,'_',s, &
                          '_1,gm_',m,'_',s,'_2,gm_',m,'_',s,'_3]' 
                  else
                     write(io,106) 'v_',m,'_',s,'=morse2[gm_',m,'_',s, &
                          '_1,gm_',m,'_',s,'_2,gm_',m,'_',s,'_3]'
                  endif
               endif
            endif
         enddo
      enddo

      write(io,20) ' '
      write(io,20) '# Coordinate transformations'
      write(io,20) ' '
      if (lqtransform) then
         do m=1,nmodes
            if (qtransform(m) .eq. 3) then
               write(io,'(a)') 'cos1 = cos[0.5,0.0]'
            else if (qtransform(m) .eq. 4) then
               write(io,'(a)') 'sin1 = sin[0.5,0.0]'
            endif
         enddo
      endif
      
20    format(a)

! Morse/anti-morse
101   format(a1,i1,a1,i1,a8,i1,a2,i1,a1,i1,a2,i1,a1,i1,a2,i1,a1,&
           i1,a2,i1,a1)
102   format(a1,i1,a1,i2,a8,i1,a2,i2,a1,i1,a2,i2,a1,i1,a2,i2,a1,&
           i1,a2,i2,a1)
! Avoided crossing
103   format(a2,i1,a1,i1,a10,i1,a1,i1,a6,i1,a1,i1,a6,i1,a1,i1, &
           a6,i1,a1,i1,a6,i1,a1,i1,a6,i1,a1,i1,a6,i1,a1,i1,a6, &
           i1,a1,i1,a3)
104   format(a2,i2,a1,i1,a10,i2,a1,i1,a6,i2,a1,i1,a6,i2,a1,i1, &
           a6,i2,a1,i1,a6,i2,a1,i1,a6,i2,a1,i1,a6,i2,a1,i1,a6, &
           i2,a1,i1,a3)

! General morse
105   format(a2,i1,a1,i1,a11,i1,a1,i1,a6,i1,a1,i1,a6,i1,a1,i1, &
           a3)
106   format(a2,i2,a1,i1,a11,i2,a1,i1,a6,i2,a1,i1,a6,i2,a1,i1, &
           a3)
! Lennard-Jones
107   format(a1,i1,a1,i1,a4,i1,a2,i1,a1,i1,a2,i1,a1,i1,a2,i1,a1,&
           i1,a2,i1,a1)
108   format(a1,i1,a1,i2,a4,i1,a2,i2,a1,i1,a2,i2,a1,i1,a2,i2,a1,&
           i1,a2,i2,a1)

! Exponential
109   format(a1,i1,a1,i1,a6,i1,a2,i1,a1,i1,a2,i1,a1)

110   format(a1,i1,a1,i2,a6,i1,a2,i2,a1,i1,a2,i2,a1)

      return

      end subroutine writelab

!#################################################################
!###### write the Quantics Input file ############# 

      subroutine writeqinp(io)

      implicit none

      integer(long), intent(in) :: io
      integer(long)             :: ilbl,m
      character(len=8)          :: label

      write(io,'(a)') '# '
      write(io,'(a)') '# Basic input for Quantics program'
      write(io,'(a)') '# Generated by VCHFIT '
      write(io,'(a)') '# '
      write(io,'(a)') ' '

      ilbl = index(filein,'.')-1
      write(io,'(a)') 'RUN-SECTION'
      write(io,'(2a)') 'name = ',filein(1:ilbl)
      write(io,'(a)') 'propagation'
      write(io,'(a)') 'tfinal = 100.0    tout = 1.0   tpsi = 5.0'
      write(io,'(a)') 'output    auto = twice   gridpop  '
      write(io,'(a)') 'end-run-section'
      write(io,'(a)') ' '
      
      ilbl = index(filein,'.')-1
      write(io,'(a)') 'OPERATOR-SECTION'
      write(io,'(2a)') 'opname = ',filein(1:ilbl)
      write(io,'(a)') 'end-operator-section'
      write(io,'(a)') ' '

      write(io,'(a)') 'SPF-BASIS-SECTION'
      write(io,'(a)') 'multi-set'
      call gensbasinp(io,nmodes,nsta)
      write(io,'(a)') 'end-spf-basis-section'
      write(io,'(a)') ' '

      write(io,'(a)') 'PRIMITIVE-BASIS-SECTION'
      do m=1,nmodes
         write(label,'(a,i5)') 'v',m
         call rmblnk(label)
         write(io,'(3x,2a)') label,'  HO     21   0.00     1.00    1.00  '
      enddo
      write(io,'(3x,a,i5)')  'el   el ',nsta
      write(io,'(a)') 'end-primitive-basis-section'
      write(io,'(a)') ' '

      write(io,'(a)') 'INIT_WF-SECTION'
      write(io,'(a)') 'build'
      write(io,'(a)') '    init_state = 2 '
      do m=1,nmodes
         write(label,'(a,i5)') 'v',m
         call rmblnk(label)
         write(io,'(3x,2a)') label,'  HO    0.00  0.00     1.00    1.0  '
      enddo
      write(io,'(a)') 'end-build'
      write(io,'(a)') 'end-init_wf-section'
      write(io,'(a)') ' '

      write(io,'(a)') 'end-input'

      end subroutine writeqinp

!#######################################################################

      subroutine gensbasinp(io,nmodes,nsta)

      implicit none

      integer(long), intent(in) :: io,nmodes,nsta
      integer(long)             :: ilbl,jlbl,m,s,n,npart,i,mi,mf
      character(len=80), dimension(6) :: label
      character(len=80)               :: stlab

      label(:) = ' '
      stlab = ' '
! generate string with spfs for each state
      ilbl=1
      do s=1,nsta
         write(stlab(ilbl:ilbl),'(i1)') 6
         if (s .lt. nsta) write(stlab(ilbl+1:ilbl+1),'(a1)') ','
         ilbl=ilbl+2
      enddo

! define no. of particles
      if (nmodes .le. 5) then
         npart = nmodes
      else if (nmodes .le. 10) then
         npart = (nmodes+1)/2
      else if (nmodes .le. 15) then
         npart = (nmodes+1)/3
      else if (nmodes .le. 20) then
         npart = (nmodes+1)/4
      else if (nmodes .le. 30) then
         npart = (nmodes+1)/5
      else if (nmodes .gt. 30) then
         write(io,'(a)') 'Need ML-MCTDH input'
         return
      endif

! write mode combination label
      mi = 1
      i = 1
      do n=1,npart
         mf = min(mi+1,nmodes)
         do m = mi,mf
            write(label(n)(i:i+6),'(a,i5)') 'v',m
            if (m .lt. mf) write(label(n)(i+7:i+7),'(a)') ','
            call rmblnk(label(n))
            i=index(label(n),' ')
         enddo
         mi = mf+1
      enddo
! write input
      do n=1,npart
         jlbl=index(label(n),' ')-1
         write(io,'(3x,3a)') label(n)(1:jlbl),' = ',stlab(1:ilbl)
      enddo

      end subroutine gensbasinp


      end module output_vcham
