!#######################################################################
      module rddatamod

use decimal, only: long, dop

use vcglobal
use global, only:pi, fs, ev, keyword, keyorig, lc, maxkey
use maxv, only: maxdim,maxsta,maxset
use vchammod
use vchfitmod
use vctransmod

use logdat, only: routine, message, inptit
use utils, only: x2q,q2intcoo,vcerrormsg,rmblnk0,comcalc,rotatemodes
use functions, only: vcweight

use ioqc, only: nm2xmat, nm2xmat2
use readqc, only: getisym

      implicit none
      private
      public :: dataset_dims,dataset_info,read_dataset,read_dataset_lm
      contains

!#######################################################################
!
! read the system dimension information from the dataset-file   
!                                                            
!#######################################################################

      subroutine dataset_dims

      implicit none
      
      integer(long)           :: ntr,s
      integer(long), dimension(:), allocatable :: isdum
      integer(long)           :: iz,ic,ierr
      character(len=80)       :: string
      logical(kind=4)         :: lerr,lgrad,lhess,ldcp
      real(dop)               :: filever

      ncoo=-999
      nmodes=-999
      ntr=0
      lerr = .false.

!-----------------------------------------------------------------------
! get no. of coordinates
!-----------------------------------------------------------------------
      print*,'Getting no. of coordinates'
      rewind(idat)
      read(idat,'(17x,f8.5)',end=999) filever

   10 continue
      read(idat,'(a)',end=999) string
      if (string(1:19) .ne. '#system_dimensions ') go to 10

   20 continue
      read(idat,'(a)',end=999) string
      if (string(1:22) .eq. '#internal_coordinates ') then
         read(idat,*) nmodes
         go to 20

      elseif (string .eq. '#trivial_modes') then
         read(idat,*) ntr
         go to 20

      elseif (string .eq. '#num_states ') then
         call rdinpf(idat,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         if (ic .eq. 1) then
            read(keyword(1),*) nsta
            maxsta=nsta
         else if (ic .eq. 2) then
            read(keyword(1),*) nsta
            read(keyword(2),*) maxsta
         endif

         go to 20

      elseif (string .eq. '#states ') then
         allocate(isdum(nsta))
         read(idat,*) (isdum(s),s=1,nsta)
         deallocate(isdum)
         go to 20

      elseif (string .eq. '#fragment_modes') then
         read(idat,*) fragnmodes
         go to 20

      else if (string(1:23) .ne. '#end_system_dimensions ') then
         go to 20

      endif

      ncoo=nmodes+ntr
      natm=ncoo/3

      maxdim=ncoo

!-----------------------------------------------------------------------
! get number of datasets
!-----------------------------------------------------------------------
      print*,'Getting number of datasets'
      rewind(idat)
      nsets=0

   45 read(idat,'(a)',end=999) string
      if(string.ne.'#dataset_section') go to 45

   50 read(idat,'(a)',end=60) string
      if (string(1:9).eq.'#dataset_') then
         nsets=nsets+1
      else if (string.eq.'#hessian ') then
         lhess=.true.
      else if (string .eq.'#gradient ') then
         lgrad=.true.
      else if (string.eq.'#linear_coupling ') then
         ldcp=.true.
      endif

      go to 50

   60 continue
      nsets1 = 0
      nsets2 = 0
      nsets3 = 0
      if (lgrad) nsets1 = nsets
      if (lhess) nsets2 = nsets
      if (ldcp)  nsets3 = nsets

      maxset = nsets

!-----------------------------------------------------------------------
! output dimensions
!-----------------------------------------------------------------------
      if (ncoo .lt. 0) then
         write(6,'(a)') 'ERROR: No coordinates not found'
         lerr=.true.
      endif
      if (nsta .lt. 0) then
         write(6,'(a)') 'ERROR: No states not found'
         lerr=.true.
      endif

      if (lerr) then
         routine='Read_dataset'
         message='Problem with dimensions in info file'
         call vcerrormsg
      endif

      return
  999 continue
      routine='Read_datasets'
      message='Incomplete info file'
      call vcerrormsg
      end subroutine dataset_dims

!#######################################################################
! read the data from the dataset-file                        
!                                                            
!#######################################################################

      subroutine dataset_info

      implicit none
      
      integer(long)           :: i,j,j1,j2,k,nblk,idum,iau
      character(len=80)       :: string,dummy

!-----------------------------------------------------------------------
! get state numbers
!-----------------------------------------------------------------------
      rewind(idat)

   10 continue
      read(idat,'(a)',end=999) string
      if (string(1:19) .ne. '#system_dimensions ') go to 10

   20 continue
      read(idat,'(a)',end=999) string
      if (string .eq. '#states ') then
         read(idat,*) (states(i),i=1,nsta)

      else if (string(1:23) .ne. '#end_system_dimensions ') then
         go to 20

      endif

!-----------------------------------------------------------------------
! get information from file
!-----------------------------------------------------------------------
      rewind(idat)
  110 continue
      read(idat,'(a)',end=999) string
      if (string(1:21) .ne. '#general_information ') go to 110

  120 continue
      read(idat,'(a)',end=999) string
      if (string .eq. '#zero_of_energy') then
         read(idat,*) e0
         go to 120

      else if (string .eq. '#equilibrium_state_geometry') then
         do i=1,ncoo,3
            read(idat,*) (xcoo0(j),j=i,i+2)
         enddo
         go to 120

      else if (string .eq. '#masses') then
         do i=1,ncoo,3
            read(idat,*) mass(i)
            mass(i+1) = mass(i)
            mass(i+2) = mass(i)
         enddo

         call comcalc
         go to 120

!
! after 9.00 #vibrational_frequencies replaces #frequencies and # symmetries
!
      else if (string .eq. '#vibrational_frequencies') then
!     if user gave mode symmetries in vchfit input file (lmodesym = true)
       if( .not. lmodesym) then
         do i=1,nmodes
            read(idat,*) idum,nmlab(i),freq0(i)
         enddo 
         go to 120
       else
         do i=1,nmodes
           if(freq0(i) .eq. -1 ) then
               read(idat,*) idum,dummy,freq0(i)
           endif
         enddo 
       endif
       go to 120
      else if (string .eq. '#frequencies') then
         do i=1,nmodes
            read(idat,*) freq0(i)
         enddo 
         go to 120

      else if (string .eq. '#symmetries') then
         do i=1,nmodes
            read(idat,'(a)') nmlab(i)
            call rmblnk0(nmlab(i))
         enddo
         go to 120

      else if (string .eq. '#mixed_frequency_cross_term ') then
         lrotmodes=.true.
         read(idat,'(i3,x,i3,F20.10)') rotm(1,1),rotm(1,2),rotmxfq(1)
         goto 120 
      else if (string .eq. '#mixed_frequency_cross_terms') then
        lrotmodes=.true.
        read(idat,'(A)') string
        j = 1
        do while ( index(string,'#') .eq. 0 )
           read(string,'(I4,2X,I4,2X,3F15.10,1X,F10.6)') &
            rotm(j,1),rotm(j,2),rotang(j),rotmxfq(j)
!           print*, trim(string)
!           print*, 'studff=', rotm(j,1),rotm(j,2),rotang(j),rotmxfq(j)
!            read(string,'(I3,3X,I3)') rotm(j,1),rotm(j,2) 
           j = j + 1
           read(idat,'(A)') string
        enddo
         backspace(idat)
        
        goto 120
      else if ( index(string,'#normal_modes') .ne. 0 ) then
         nblk=((ncoo-1)/5)+1
         do k=1,nmodes
            do i=1,nblk
               j1=(i-1)*5+1
               j2=min(j1+4,ncoo)
               read(idat,'(5F15.10)') (nmcoo(j,k),j=j1,j2)
!               write(6,'(5F15.10)') (nmcoo(j,k),j=j1,j2)
            enddo
            read(idat,'(a)') string
         enddo
         go to 120

      else if ( index(string,'#q_to_curvq_transform') .ne. 0 ) then
       lnmcurv = .true.
       lalleig = .true.
       i = 1
       read(idat,'(a)') string
       do while( index(string,'#') .eq. 0 .and. len(trim(string)) .gt. 0) 
         if( index(string(1:2),'1') .ne. 0) then
          read(string,'(4I2,6X,3F15.10)') nmcurvtype(i), nmcurv(i,1),&
           nmcurv(i,2), nmcurv(i,3), c0(i,1), c0(i,2),c0(i,3)
         endif
!          print*,nmcurvtype(i), nmcurv(i,1),nmcurv(i,2), nmcurv(i,3), c0(i,1), c0(i,2),c0(i,3)
          if(nmcurvtype(i) .eq.1) then
!              print*, 'pi = ', pi
!              print*, 'c0(2) = ', c0(i,2) , 'c0(3) = ', c0(i,3)
              c0(i,2) = c0(i,2)*(pi/180)
              c0(i,3) = c0(i,3)*(pi/180)
!              print*, 'c0(2) = ', c0(i,2) , 'c0(3) = ', c0(i,3)
          endif
          i = i + 1
         read(idat,'(a)') string
       enddo     
       backspace(idat)
!       goto 120 

      else if (string .eq. '#replace_modes') then
         linternal=.true.
         do i=1,nmodes
            read(idat,'(5i5,f12.6)') cootyp(i),(coodef(k,i),k=1,4),&
                 intcoo0(i)
         enddo
         go to 120

      else if (string(1:25) .ne. '#end_general_information ') then
         go to 120

      endif

! nm2xmat requires atomic symbol - get from mass
      do i=1,ncoo
         call getisym(iatsym(i),mass(i),ilog)
      enddo
      iau = 1
!       its already rotated:
!      if( lrotmodes)  call rotatemodes(nmcoo,dumfreq0,rotm,rotang,rummy)
      if(lalleig) call nm2xmat2(nmcoo,coonm,mass,ncoo,maxdim)
      if(.not.lalleig) call nm2xmat(nmcoo,coonm,iatsym,freq0,iau,ncoo,&
                            maxdim,nmodes,ilog)

      return
  999 continue
      routine='Read_datasets'
      message='Incomplete info file'
      call vcerrormsg
      end subroutine dataset_info

!#######################################################################
!
! at start nsets is total no. of records in database
! at end, nsets is no. of records included in fit
!
!#######################################################################

      subroutine read_dataset(energy,xcoo,qcoo,der1,&
                 dcp,der2,tdip,xcooder1,xcoodcp,xcooder2,qval)

      implicit none
      
      integer(long)                                       :: i,j,j1,j2,k,l,n,&
                                                             nblk,n1,s,s1,s2,ilbl,&
                                                             ndat,nblkf,nact
      integer(long), dimension(2)                         :: coupsta
      real(dop), dimension(nsets), intent(out)            :: energy
      real(dop), dimension(4,nsets), intent(out)          :: tdip
      real(dop), dimension(ncoo,nsets), intent(out)       :: xcoo
      real(dop), dimension(nmodes,nsets), intent(out)     :: qcoo
      real(dop), dimension(ncoo,nsets1), intent(out)      :: der1
      real(dop), dimension(maxdim,nsets1), intent(out)    :: xcooder1
      real(dop), dimension(ncoo,nsta,nsets3), intent(out) :: dcp
      real(dop), dimension(maxdim,nsta,nsets3), intent(out):: xcoodcp
      real(dop), dimension(ncoo,ncoo,nsets1), intent(out) :: der2
      real(dop), dimension(maxdim,maxdim,nsets1), intent(out) :: xcooder2
      real(dop), dimension(maxdim)                        :: workvec
      real(dop), dimension(maxdim,maxdim)                 :: work
      real(dop), dimension(nmodes), intent(in)            :: qval
      real(dop)                                           :: diff,ezero
      character(len=80)                                   :: string
      logical(kind=4)                                     :: ignore,check,&
                                                             mignore
      logical(kind=4)                                     :: lq0,lstep

      do n=1,nsets
         asta(n)=-1
         asta1(:,n)=-1
      enddo

!--------------------------------------------------------
! get data sets for the different points of the PES    
!--------------------------------------------------------
      rewind(idat)
      n=0
      nblk=((nmodes-1)/5)+1

 200  read(idat,'(a)',end=999) string
      if (string(1:17).ne.'#dataset_section ') goto 200

 210  read(idat,'(a)',end=290) string
      if(string(1:9).ne.'#dataset_') goto 210
      ilbl=index(string,' ')-1
      read(string(10:ilbl),*) ndat
      n=n+1
! set index for NACTs for this dataset to 0
      nact=0
 
220   read(idat,'(a)') string
      if (string(1:7) .eq. '#energy') then
         read(idat,*) energy(n)
         go to 220

      else if (string(1:6) .eq. '#state') then
         if (ldiab .eqv. .false.) then
            read(idat,*) asta(n)
         endif
         go to 220

      else if (string(1:31) .eq.'#point_in_cartesian_coordinates') then
         do i=1,ncoo,3
            read(idat,*) (xcoo(j,n),j=i,i+2)
         enddo
         go to 220

      else if (string(1:11) .eq.'#point_in_q') then
         nblk=((nmodes-1)/5)+1   !MAP need to recalculate as it gets 
                                 ! changed if Hessian present
         do i=1,nblk
            j1=(i-1)*5+1
            j2=min(j1+4,nmodes)
            read(idat,*) (qcoo(j,n),j=j1,j2)
         enddo
         go to 220

      else if (string(1:16) .eq.'#point_in_full_q') then
         nblkf=((ncoo-1)/5)+1
         do i=1,nblkf
            j1=(i-1)*5+1
            j2=min(j1+4,ncoo)
            read(idat,*) (qcoo(j,n),j=j1,j2)
         enddo
         go to 220

      else if (string(1:10) .eq.'#gradient ') then
         do i=1,ncoo,3
            read(idat,*) (workvec(j),j=i,i+2)
         enddo
         xcooder1(:,n) = workvec(:)
!        project gradient to ground-state geometry
!         call tmatvec(der1(1,n),nmcoo,workvec,maxdim,ncoo,nmodes)
         call dgemv('T',ncoo,nmodes,1.d0,nmcoo,maxdim,workvec,1,0,der1(:,n),1)
         info(n,1)=max(info(n,1),1)
         go to 220

      else if (string(1:17).eq.'#linear_coupling ') then
         read(idat,*) (coupsta(i),i=1,2)
         nact = nact+1
         if (coupsta(1) .eq. asta(n)) then
            asta1(nact,n)=coupsta(2)
         else
            asta1(nact,n)=coupsta(1)
         endif
         do i=1,ncoo,3
            read(idat,*) (workvec(j),j=i,i+2)
         enddo  
!        project linear coupling to ground-state geometry
!         call tmatvec(dcp(1,n),nmcoo,workvec,maxdim,ncoo,nmodes)
         xcoodcp(:,nact,n) = workvec(:)
         call dgemv('T',ncoo,nmodes,1.d0,nmcoo,maxdim,workvec,1,0,dcp(:,nact,n),1)
         info(n,2)=1
         go to 220

      else if (string(1:21).eq.'#linear_coupling_in_q ') then
         nblk=((nmodes-1)/5)+1
         do i=1,nblk
            j1=(i-1)*5+1
            j2=min(j1+4,nmodes)
            read(idat,'(5F15.10)') (dcp(j,nact,n),j=j1,j2)
         enddo
         go to 220
         
      else if (string(1:9).eq.'#hessian ') then
!        read the hessian
         nblk=((ncoo-1)/5)+1
         do k=1,ncoo
            do i=1,nblk
               j1=(i-1)*5+1
               j2=min(j1+4,ncoo)
               read(idat,*) (work(k,j),j=j1,j2)
            enddo
            read(idat,'(a)') string
         enddo
         xcooder2(:,:,n) = work(:,:)
!        project hessian to ground-state geometry
         do i=1,nmodes
            do j=1,nmodes
               der2(i,j,n)=0.0
               do k=1,ncoo
                  do l=1,ncoo
                     der2(i,j,n)=der2(i,j,n)&
                          +nmcoo(k,i)*work(k,l)*nmcoo(l,j)
                  enddo
               enddo
            enddo
         enddo
         info(n,1)=2
         go to 220

      else if (string(1:18).eq.'#transition dipole') then
         read(idat,*) (tdip(i,n),i=1,4)
         info(n,3)=1

      else if (string(1:13).ne.'#end_dataset_') then
         go to 220

      endif

!-----------------------------------------------------------------------
! reject set if not required in fit
! first check if state is in fit and energy and coordinates in bounds
!-----------------------------------------------------------------------
      ignore=.true.

      do i=1,nsta
         if (asta(n) .eq. states(i)) ignore=.false.
      enddo

      if (energy(n) .gt. edsup) ignore = .true.
      if (energy(n) .lt. edslow) ignore = .true.

      do i=1,nmodes
         if (qcoo(i,n) .lt. qbnd(i,1)) ignore = .true.
         if (qcoo(i,n) .gt. qbnd(i,2)) ignore = .true.
      enddo

!-----------------------------------------------------------------------
! Ignore if (1) no displacement along coords being fitted
! AND (2) non-fitted coords are zero
! mfit(m) = 1 mode for plot. mfit(m) = 0 fixed.
! Q0 is stored in qval. 
!-----------------------------------------------------------------------
      lstep = .false.
      lq0 = .true.
      do i=1,nmodes
         if (mfit(i) .gt. 0) then
            if (abs(qcoo(i,n) - 0.0) .gt. cuttol) lstep =.true.
         else if (mfit(i) .eq. 0) then
            if (abs(qcoo(i,n) - qval(i)) .gt. cuttol) lq0 = .false.
         endif
      enddo

      mignore = .true.   
      if (lstep .and. lq0) mignore = .false.

!-----------------------------------------------------------------------
! If ALL coords are zero, at Q0
!-----------------------------------------------------------------------
      if (lstep) lq0 = .false.    ! Not at Q0
      if (lq0) mignore = .false.  ! set is at Q0. Include.

!-----------------------------------------------------------------------
! Include set if criteria met
!-----------------------------------------------------------------------
      if (ignore .or. mignore) then
         write(ilog,'(a,i5,f12.5)') 'Ignored data set :',ndat,energy(n)
         n=n-1
      else
         write(ilog,'(a,i5,f12.5)') 'Include data set :',ndat,energy(n)
      endif

! read next set
      go to 210

  290 continue
      close(idat)

!-----------------------------------------------------------------------
! have read in data. Now reset nsets
!-----------------------------------------------------------------------
      write(ilog,'(/,a,i5)') 'No. of data points in database: ',nsets
      nsets = n
      write(ilog,'(/,a,i5)') 'No. of data points in fit     : ',nsets

!-----------------------------------------------------------------------
! set flag for which state in Hamiltonian each set belongs to
! (map state nos. from QC eigenstates to VCHam states)
!-----------------------------------------------------------------------
      do n=1,nsets
         ista(n)=-1
         jsta(:,n)=-1
         do i=1,nsta
            if (asta(n) .eq. states(i)) ista(n)=i
            do nact=1,nsta
               if (asta1(nact,n) .eq. states(i)) jsta(nact,n)=i
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! now calculate the weights for the fit
!-----------------------------------------------------------------------
      do n=1,nsets
         if (weighttype.eq.1) then
            weight(n) = vcweight(qcoo(1,n),wfact) 
            w0(n)=1.0*vcweight(qcoo(1,n),4.0_dop)
            w1(n)=5.0*vcweight(qcoo(1,n),100.0_dop)
            w2(n)=1E-8*vcweight(qcoo(1,n),6.0_dop)
            w1(n)=0.0
            w2(n)=0.0
         else if (weighttype.eq.2) then
            ezero = energy(ista(n))
            w0(n)=exp(wfact*(ezero-energy(n)))
            w1(n)=1.0
            w2(n)=1.0
         else if (weighttype.eq.3) then
            w0(n)=1.0
            w1(n)=1.0
            w2(n)=1.0
         else if (weighttype.eq.4) then
            ezero = energy(ista(n))
            w0(n)=0.5_dop*(1-tanh((energy(n)-(ezero+0.7_dop))/0.2_dop))
            w1(n)=1.0
            w2(n)=1.0
         else
            print*,' !!!!!!!! Unknown weight-type for fitting !!!!!!'
            stop  
         endif
      enddo
!-----------------------------------------------------------------------
! For an Exe JT system, at CoIn check that degenrate states are 
! correctly rotated so that DCP = GD 
!-----------------------------------------------------------------------
      check=.false.
      do s=1,nsta
         s1=0
         do s2=s+1,nsta
            if (cointype(s2,s) .eq. 2) then
               s1=s2
            endif
         enddo
         
         if (s1 .eq. 0) go to 400

         do n=1,nsets-1
            if (ista(n) .eq. s .and. info(n,2) .eq. 1 .and. info(n,1) &
                 .ge. 1) then
! have info, find other data set
               do n1=n+1,nsets
                  if (ista(n1) .eq. s1 .and. info(n1,2) .eq. 1 .and. &
                        info(n1,1) .ge. 1) then
                     diff = energy(n) - energy(n1)

                     if (abs(diff) .lt. 1.0e-6) then
                        if (check) call rotdcp(n,n1,qcoo,der1,dcp)
                     endif
                  endif
               enddo

            endif
         enddo

  400 continue

      enddo
      
! end 
      return
  999 continue
      routine='Read_datasets'
      message='Incomplete info file'
      call vcerrormsg
      end subroutine read_dataset


!#######################################################################
      
      subroutine read_dataset_lm(xcoo,qcoo,dipole,polar,diab_dipole,&
           diab_polar)

        integer(long)                                           :: i,j,j1,j2,&
             n,nblk,s,s1,ilbl,ndat
        character(len=80)                                       :: string
        
        real(dop), dimension(ncoo,nsets), intent(out)           :: xcoo
        real(dop), dimension(nmodes,nsets), intent(out)         :: qcoo
        real(dop), dimension(nsets,nsta,nsta,3), intent(out)    :: dipole,diab_dipole
        real(dop), dimension(nsets,nsta,nsta,3,3), intent(out)  :: polar,diab_polar

        rewind(idat)
        n=0

        write(6,'(a)') 'Reading datasets'

200     read(idat,'(a)',end=999) string
        if (string(1:17).ne.'#dataset_section ') goto 200
        
210     read(idat,'(a)',end=290) string
        if(string(1:9).ne.'#dataset_') goto 210
        ilbl=index(string,' ')-1
        read(string(10:ilbl),*) ndat
        n=n+1

220     read(idat,'(a)') string
        if (string .eq.'#point_in_cartesian_coordinates') then
           do i=1,ncoo,3
              read(idat,*) (xcoo(j,n),j=i,i+2) 
           enddo
           go to 220

        else if (string .eq.'#point_in_q') then
           nblk=((nmodes-1)/5)+1   
           do i=1,nblk
              j1=(i-1)*5+1
              j2=min(j1+4,nmodes)
              read(idat,'(5F15.10)') (qcoo(j,n),j=j1,j2)
           enddo
           go to 220
           
        else if (string.eq.'#adiabatic_dipole_matrix (s,s1,x,y,z)') then
           do s=1,nsta
              do s1=s,nsta
                 read(idat,10) dipole(n,s,s1,1),dipole(n,s,s1,2),&
                      dipole(n,s,s1,3)
                 do i=1,3
                    dipole(n,s1,s,i)=dipole(n,s,s1,i)
                 enddo
              enddo
           enddo
           goto 220

        else if (string.eq.'#diabatic_dipole_matrix (s,s1,x,y,z)') then
           do s=1,nsta
              do s1=s,nsta
                 read(idat,10) diab_dipole(n,s,s1,1),diab_dipole(n,s,s1,2),&
                      diab_dipole(n,s,s1,3)
                 do i=1,3
                    diab_dipole(n,s1,s,i)=diab_dipole(n,s,s1,i)
                 enddo
              enddo
           enddo
           goto 220

        else if (string.eq.'#adiabatic_polarisability_tensor') then
           read(idat,*)
           read(idat,*)
           do s=1,nsta
              do s1=s,nsta
                 read(idat,11) polar(n,s,s1,1,1),polar(n,s,s1,1,2),&
                      polar(n,s,s1,1,3)
                 read(idat,11) polar(n,s,s1,2,2),polar(n,s,s1,2,3),&
                      polar(n,s,s1,3,3)
                 do i=1,3
                    do j=i,3
                       polar(n,s1,s,i,j)=polar(n,s,s1,i,j)
                       polar(n,s,s1,j,i)=polar(n,s,s1,i,j)
                       polar(n,s1,s,j,i)=polar(n,s,s1,j,i)
                    enddo
                 enddo
              enddo
           enddo
           goto 220

        else if (string.eq.'#diabatic_polarisability_tensor') then
           read(idat,*)
           read(idat,*)
           do s=1,nsta
              do s1=s,nsta
                 read(idat,11) diab_polar(n,s,s1,1,1),diab_polar(n,s,s1,1,2),&
                      diab_polar(n,s,s1,1,3)
                 read(idat,11) diab_polar(n,s,s1,2,2),diab_polar(n,s,s1,2,3),&
                      diab_polar(n,s,s1,3,3)
                 do i=1,3
                    do j=i,3
                       diab_polar(n,s1,s,i,j)=diab_polar(n,s,s1,i,j)
                       diab_polar(n,s,s1,j,i)=diab_polar(n,s,s1,i,j)
                       diab_polar(n,s1,s,j,i)=diab_polar(n,s,s1,j,i)
                    enddo
                 enddo
              enddo
           enddo
           goto 220


        else if (string(1:13).ne.'#end_dataset_') then
           go to 220
           
        endif


! read next set
      go to 210

290   continue
      close(idat)

10    format (9x,3(F13.8,3x))
11    format (9x,3(F14.8,3x))

        return
999     continue
        routine='Read_datasets'
        message='Incomplete info file'
        call vcerrormsg
      end subroutine read_dataset_lm

!#######################################################################

      subroutine rotdcp(n,n1,qcoo,der1,dcp)

      implicit none

      integer(long)                                    :: i,s,s1,f,j,k,l,m
      integer(long), intent(in)                        :: n,n1
      real(dop)                                        :: theta,stheta,ctheta,maxkap,diff
      real(dop), dimension(nmodes,nsets), intent(in)   :: qcoo
      real(dop), dimension(ncoo,nsets1), intent(inout) :: der1
      real(dop), dimension(ncoo,nsets3), intent(inout) :: dcp
      real(dop), dimension(maxdim)                     :: gdiff
      real(dop), dimension(2,2)                        :: rotmat,mat1,mat2
      logical(kind=4)                                  :: check

!
! check at same coordinate  
!
      check=.false.
      do i=1,nmodes
         diff = qcoo(i,n) - qcoo(i,n1)
         if (abs(diff) .gt. 1.0e-6) check=.true.
      enddo
      if (check) return

      print*,'Rotating linear coupling for sets',n,n1      

      s=asta(n)
      s1=asta(n1)
      if (s1 .gt. s) then
         do i=1,nmodes
            gdiff(i) = 0.5*(der1(i,n1) - der1(i,n))
         enddo
      else 
         do i=1,nmodes
            gdiff(i) = 0.5*(der1(i,n) - der1(i,n1))
         enddo
      endif

!
! get DOF of correct symmetry with largest component of GD
!
      f=0
      maxkap=0.0
      do i=1,nmodes
         if (nmlab(i) .eq. jtmode(s)) then
            if (abs(gdiff(i)) .gt. maxkap) then
               f=i
               maxkap=gdiff(i)
            endif
         endif
      enddo

      theta= dcp(f,n)/gdiff(f)
      theta = atan(theta)
      theta = 0.5*theta

      stheta = sin(theta)
      ctheta = cos(theta)
      rotmat(1,1)=ctheta
      rotmat(2,1)=stheta
      rotmat(1,2)=-stheta
      rotmat(2,2)=ctheta

      do i=1,nmodes
!        mat1(1,1) = -gdiff(i)
!        mat1(2,2) = gdiff(i)
         mat1(1,1) = der1(i,n)
         mat1(2,2) = der1(i,n1)
         mat1(1,2) = dcp(i,n)
         mat1(2,1) = dcp(i,n)

         do j=1,2
            do k=1,2
               mat2(j,k)=0.0
               do l=1,2
                  do m=1,2
                     mat2(j,k)= mat2(j,k) + &
                            rotmat(j,l)*mat1(l,m)*rotmat(k,m)
                  enddo
               enddo
            enddo
         enddo
         der1(i,n)=mat2(1,1) 
         der1(i,n1)=mat2(2,2) 
         dcp(i,n)=mat2(1,2)
         dcp(i,n1)=mat2(1,2)

!        print*, nmlab(i),gdiff(i),dcp(i,n)

      enddo



      return
      end subroutine rotdcp


      end module rddatamod

