!#######################################################################

      module symmetry

use decimal, only: long, dop
use vcglobal
use global, only:pi, fs, ev, keyword, maxkey, keyorig, lc
use maxv, only: maxdim,maxsta,maxset
use vchammod
use vchfitmod
use logdat, only: routine, message, inptit
use utils, only: integralsym,getcharacters,cleanirreps, &
                 vcerrormsg,vccntrl,vcwrline,vcwrline,parnumt, &
                 numtpar,parnum


use rddatamod, only: dataset_info

use op1lib,  only: zerovxl, zerovxi

      implicit none
      private
      public :: readmask,createmask,renumcons,setcons,delmodes,delstates,&
           addparams,lmmask,delmodes_lm,getnadd,rdexplicit,getnswitch
      contains

!#######################################################################
!
! read symmetry mask from previous calculation
!
! turn off constraint flags 
!#######################################################################

      subroutine readmask(const_mask,kappa_mask,lambda_mask,&
              gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask)

      implicit none

      integer(long)                                                 :: i,j,s,s1,nblck,j1,j2,j3,n,ilbl,k

      integer(long), dimension(nsta,nsta), intent(out)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(out)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(out)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(out)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(out)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(out)                    :: termk_mask
      character(len=80)       :: string, buffer
      real(dop)               :: filever,rtmp
      integer(long)  ,dimension(maxdim)                       :: old_adof
      logical(kind=4),dimension(maxdim)                       :: old_ignoredof
      integer(long)                                           :: old_i_ignore
      
      if( adof(1) .ne. 0) then
         old_i_ignore = i_ignore
         old_ignoredof = ignoredof
         old_adof = adof
      else
         do i = 1, nmodes
          old_adof(i) = 0
          adof(i) = i
         enddo
         i_ignore = 0
      endif


        if(i_ignore .eq. 0) then
         do i = 1, nmodes
              ignoredof(i) = .false.
         enddo

        endif 


      
!      print*, 'POOOOOOOOOOP'
      rewind(iguess)
      ilbl = index(guessfile,' ')-1
      write(ilog,'(/,2a)') '*** Mask from file : ',guessfile(1:ilbl)
!
! read file version no.
!
      read(iguess,'(17x,f8.5)') filever
      write(ilog,'(a,f8.3)') 'File version: ',filever

 10   continue
      read(iguess,'(a)',end=999,err=1010) string

      if (string.eq.'*** Constants mask ***') then
         write(ilog,'(a)') 'Reading constants mask '
         read(iguess,*) 
         do s=1,nsta
            read(iguess,'(7x,10i3)') (const_mask(s,s1),s1=1,s)
            do s1=1,s
               if (const_mask(s,s1) .eq. -1) const_mask(s,s1)=1
            enddo
            do s1=1,s-1
               if (const_mask(s,s1) .ne. 0) lconstall=.true.
               const_mask(s1,s)=const_mask(s,s1)
            enddo
         enddo
      elseif(string .eq. '*** Omitted DOFs ***' ) then
33    continue
       i_ignore = i_ignore + 1
       j = 0
       read(iguess,'(I3)',end=999,err=1010) j
       if ( j .ne. 0 ) then 
         ignoredof(j) = .true.
         goto 33
       else
         do j = 1, nmodes
           if( .not. ignoredof(j)) then
             i = i + 1
             adof(i) = j
           endif
         enddo
         nmodes = nmodes - i_ignore
         goto 10
       endif
   
      else if &
         (string .eq.'*** On-diagonal linear (kappa) mask ***') then
         write(ilog,'(a)') 'Reading kappa mask '
         do s=1,nsta
            read(iguess,'(a)') string
            do i=1,nmodes
               read(iguess,'(10x,i3)') kappa_mask(adof(i),s)
               if (kappa_mask(adof(i),s) .eq. -1) kappa_mask(adof(i),s)=1
            enddo
         enddo

      else if &
         (string .eq.'*** Off-diagonal linear (lambda) mask ***') then
         write(ilog,'(a)') 'Reading lambda mask '
         do s=1,nsta-1
            do s1=s+1,nsta
               read(iguess,'(a)') string
               do i=1,nmodes
                  read(iguess,'(10x,i3)') lambda_mask(adof(i),s,s1)
                  if (lambda_mask(adof(i),s,s1) .eq. -1) lambda_mask(adof(i),s,s1)=1
                  lambda_mask(adof(i),s1,s)=lambda_mask(adof(i),s,s1)
               enddo
            enddo
         enddo

      else if &
         (string .eq.'*** On-diagonal bilinear (gamma) mask ***') then
         if (hamord .lt. 2) go to 100
         write(ilog,'(a)') 'Reading gamma mask '
         nblck=((nmodes-1)/6)+1
         do s=1,nsta
            read(iguess,'(a)') string
            do n=1,nblck
               j1=(n-1)*6+1
               j2=min(j1+5,nmodes)
               read(iguess,'(a)') string
               do i=j1,nmodes
                  j3=min(i,j2)
                  read(iguess,'(8x,6i10)') &
                      (gamma_mask(adof(i),adof(j),s),j=j1,j3)
                  do j=j1,j3
                     if (gamma_mask(adof(i),adof(j),s) .eq. -1)&
                         gamma_mask(adof(i),adof(j),s)=1
                     gamma_mask(adof(j),adof(i),s)=&
                     gamma_mask(adof(i),adof(j),s)
                  enddo
               enddo
            enddo
         enddo

      else if&
         (string .eq.&
         '*** On-diagonal quadratic-linear (iota) mask ***') then
         if (hamord .lt. 3) go to 100
         write(ilog,'(a)') 'Reading iota mask '
         nblck=((nmodes-1)/6)+1
         do s=1,nsta
            read(iguess,'(a)') string
            do n=1,nblck
               j1=(n-1)*6+1
               j2=min(j1+5,nmodes)
               read(iguess,'(a)') string
               do i=j1,nmodes
                  j3=j2
                  read(iguess,'(8x,6i10)') &
                  (iota_mask(adof(i),adof(j),s,s),j=j1,j3)
               enddo
            enddo
         enddo

      else if &
         (string .eq.'*** Off-diagonal bilinear (mu) mask ***') then
         if (hamord .lt. 3) go to 100
         write(ilog,'(a)') 'Reading mu mask '
         
         nblck=((nmodes-1)/6)+1
         do s=1,nsta-1
            do s1=s+1,nsta
               read(iguess,'(a)') string
               do n=1,nblck
                  j1=(n-1)*6+1
                  j2=min(j1+5,nmodes)
                  read(iguess,'(a)') string
                  do i=j1,nmodes
                     j3=min(i,j2)
                     read(iguess,'(8x,6i10)') &
                     (mu_mask(adof(i),adof(j),s,s1),j=j1,j3)
                     if (mu_mask(adof(i),adof(j),s,s1) .eq. -1) &
                         mu_mask(adof(i),adof(j),s,s1)=1
                     do j=j1,j3
             mu_mask(adof(i),adof(j),s1,s) = mu_mask(adof(i),adof(j),s,s1)
             mu_mask(adof(j),adof(i),s,s1) = mu_mask(adof(i),adof(j),s,s1)
             mu_mask(adof(j),adof(i),s1,s) = mu_mask(adof(i),adof(j),s1,s)
                     enddo
                  enddo
               enddo
            enddo
         enddo

      else if& 
         (string .eq.&
             '*** Off-diagonal quadratic-linear (iota2) mask ***') then
         if (hamord .lt. 3) go to 100
         write(ilog,'(a)') 'Reading iota mask '
         
         nblck=((nmodes-1)/6)+1
         do s=1,nsta-1
            do s1=s+1,nsta
               read(iguess,'(a)') string
               do n=1,nblck
                  j1=(n-1)*6+1
                  j2=min(j1+5,nmodes)
                  read(iguess,'(a)') string
                  do i=j1,nmodes
                     j3=j2
                     read(iguess,'(8x,6i10)') &
                (iota_mask(adof(i),adof(j),s,s1),j=j1,j3)
                     if (mu_mask(adof(i),adof(j),s,s1) .eq. -1)&
                         mu_mask(adof(i),adof(j),s,s1)=1
                     do j=j1,j3
                iota_mask(adof(i),adof(j),s1,s) = &
                iota_mask(adof(i),adof(j),s,s1)
                     enddo
                  enddo
               enddo
            enddo
         enddo

      elseif &
       (string .eq.'*** Explicitly added polynomial term mask ***') then
        write(ilog,'(a)') 'Reading explicitely added polynomial mask '
        read(iguess,*) i
        do n=1,i
         read(iguess,'(2I5)') j,termk_mask(n)
        enddo

      else if &
         (string .eq.'*** General diabatic potential mask ***') then
         if (.not. lgeneral) go to 100
         write(ilog,'(a)') 'Reading general potential mask '
         do s=1,nsta
            read(iguess,'(a)') string
            do i=1,nmodes
               n=max(1,ncoeff(adof(i),s))
               read(iguess,'(10x,10i3)') (expcoe_mask(k,adof(i),s),k=1,n)
               do k=1,n
                  if (expcoe_mask(k,adof(i),s) .eq. -1) expcoe_mask(k,adof(i),s)=1
               enddo
            enddo
         enddo

      endif

  100 continue
      go to 10

  999 continue
      write(ilog,'(a)') 'END OF GUESS FILE'

      if(i_ignore .gt. 0 ) nmodes = nmodes + i_ignore

      if( old_adof(1) .ne. 0  ) then
          i_ignore = old_i_ignore
          ignoredof = old_ignoredof
          adof = old_adof
       endif 


      return
 1010 stop 'ERROR IN GUESS FILE'
      end subroutine readmask

!#######################################################################
! parameters for symmetry-mask
! if parameter_mask = 1  active parameter to be optimised
!                     2  active parameter frozen
!                     3  active parameter to be optimised in first pass
!                     0  non-active parameter (zero by symmetry)
!                    -1  constrained active parameter
!                    -2  frozen constrained active parameter
!#######################################################################

      subroutine createmask(const_mask,kappa_mask,lambda_mask, &
           gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask, &
           cpcoe_mask,switch_mask,cp2d_mask)

      implicit none

      integer(long) :: i,j,k,s,s1,l,m,n,c,check,tmp
      integer(long), dimension(3)      :: axchk
      integer(long), dimension(maxdim) :: nmchk,stachk
      integer(long), dimension(nsta,nsta), intent(out) :: const_mask
      integer(long), dimension(nmodes,nsta), intent(out) :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(out) :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(out) :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(out) :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: &
                                                      iota_mask,mu_mask
      integer(long), dimension(nadd), intent(out) :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(out) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe) :: switch_mask
      integer(long), dimension(maxcoe,maxadd), intent(out)         :: cp2d_mask

!-----------------------------------------------------------------------
! If lautosym=true, mask values are determined from the calculation of
! the number of times that the totally symmetric irreducible 
! representation eneters into the decomposition of the direct product
! representations corresponding to the integrands of the parameters of
! the diabatic potential, i.e., phi_a* d^n/dq1...dqn phi_b
!
! First, convert the user specified symmetry labels to the format used
! in the code, then get the characters.
!-----------------------------------------------------------------------
      check=0
      if (lautosym) then
         do i=1,nmodes
            call cleanirreps(nmlab(i),len(nmlab))
         enddo
         do i=1,nsta
            call cleanirreps(stalab(i),len(stalab))
         enddo
         call getcharacters
         do i=1,3
            axchk(i)=0
         enddo
      endif

!-----------------------------------------------------------------------
! zero mask
!-----------------------------------------------------------------------
      do s=1,nsta
         do s1=1,nsta
            const_mask(s1,s)=0
         enddo
      enddo
      do s=1,nsta
         do i=1,nmodes
            kappa_mask(i,s)=0
         enddo
      enddo
      do s=1,nsta
         do i=1,nmodes
            do s1=1,nsta
               lambda_mask(i,s,s1)=0
            enddo
         enddo
      enddo
      do s=1,nsta
         do i=1,nmodes
            do j=1,nmodes
               gamma_mask(j,i,s)=0
            enddo
         enddo
      enddo
      do s=1,nsta
         do s1=1,nsta
            do i=1,nmodes
               do j=1,nmodes
                  iota_mask(j,i,s,s1)=0
               enddo
            enddo
         enddo
      enddo
      do s=1,nsta
         do s1=1,nsta
            do i=1,nmodes
               do j=1,nmodes
                  mu_mask(j,i,s,s1)=0
               enddo
            enddo
         enddo
      enddo
      do s=1,nsta
         do i=1,nmodes
            do k=1,ncoeff(i,s)
               expcoe_mask(k,i,s)=0
            enddo
         enddo
      enddo

      do s=1,nsta
         do s1=1,nsta
            do i=1,nmodes
               cpcoe_mask(1,i,s,s1)=0
               do k=2,ncoeffcp(i,s,s1)
                  cpcoe_mask(k,i,s,s1)=0
               enddo
            enddo
         enddo
      enddo

! Explicitly added terms
! If lautosym=true, mask values are determined from symmetry arguments.
! If lautosym=false, the user is trusted and the mask values are all set to 1
      do i=1,nadd
         if (lautosym) then
            termk_mask(i)=0
         else
            termk_mask(i)=1
         endif
      enddo

! 'Temporary' bodge...
      do i=1,nswitch
         do j=1,nswitchcoe(i)
            switch_mask(i,j)=1
         enddo
      enddo

!-----------------------------------------------------------------------
! set mask for non-zero variables in Hamiltonian
!-----------------------------------------------------------------------
      do s=1,nsta
         do s1=1,nsta
            const_mask(s1,s)=2
         enddo
      enddo

      if (lconst .and. lconstall) then
         do s=1,nsta
            do s1=1,nsta
               const_mask(s1,s)=1
            enddo
         enddo
      else if (lconst) then
         do s=1,nsta
            const_mask(s,s)=1
         enddo
      endif
      if (.not. ldiagfit .and. lconstall) then
         do s=1,nsta
            do s1=1,nsta
               const_mask(s1,s)=2
            enddo
         enddo
      endif

      if (lkappa) then
         if(lautosym.eqv..true.) then
            do i=1,nsta
               do j=1,nmodes
                  do c=1,maxdim
                     nmchk(c)=0
                     stachk(c)=0
                  enddo
                  nmchk(j)=1
                  stachk(i)=2
                  call integralsym(check,nmchk,stachk,axchk)                
                  if (check.eq.1) kappa_mask(j,i)=1
               enddo
            enddo
         else
            do s=1,nsta
               do i=1,nmodes
                  if (nmlab(i).eq.symmode .or. nmlab(i).eq.jtmode(s)) then
                     kappa_mask(i,s)=1
                  endif
               enddo
            enddo
         endif

      else
         do s=1,nsta
            do i=1,nmodes
               if (nmlab(i).eq.symmode .or. nmlab(i).eq.jtmode(s)) then
                  kappa_mask(i,s)=2
               endif
            enddo
         enddo
      endif
         
      if (llambda) then
         if(lautosym.eqv..true.) then
            do i=1,nsta
               do j=1,nsta
                  do k=1,nmodes
                     do c=1,maxdim
                        nmchk(c)=0
                        stachk(c)=0
                     enddo
                     nmchk(k)=1
                     stachk(j)=1
                     stachk(i)=1
                     if(j.eq.i) stachk(j)=2
                     call integralsym(check,nmchk,stachk,axchk)
                     if (check.eq.1) lambda_mask(k,j,i)=1
                  enddo
               enddo
            enddo
         else
            do s=1,nsta
               do i=1,nmodes
                  do s1=1,nsta
                     if (nmlab(i).eq.cpmode(s,s1)) then
                        lambda_mask(i,s,s1)=1
                     endif
                  enddo
               enddo
            enddo
         endif
      else
         do s=1,nsta
            do i=1,nmodes
               do s1=1,nsta
                  if (nmlab(i).eq.cpmode(s,s1)) then
                     lambda_mask(i,s,s1)=2
                  endif
               enddo
            enddo
         enddo
      endif

! for linear model ignore 2nd order parameters
      if (hamord .eq. 1) go to 100

      if (lgamma) then
         if(lautosym.eqv..true.) then
            do i=1,nsta
               do j=1,nmodes
                  do k=1,nmodes
                     do c=1,maxdim
                        nmchk(c)=0
                        stachk(c)=0
                     enddo
                     nmchk(k)=1
                     nmchk(j)=1
                     if (j.eq.k) nmchk(j)=2
                     stachk(i)=2
                     call integralsym(check,nmchk,stachk,axchk)
                     if (check.eq.1) then
                        gamma_mask(k,j,i)=1
                        gamma_mask(j,k,i)=1
                     endif
                  enddo
               enddo
            enddo
         else
            do s=1,nsta
               do i=1,nmodes
                  gamma_mask(i,i,s)=1
                  do j=i+1,nmodes
                     if (nmlab(i).eq.nmlab(j)) then
                        gamma_mask(i,j,s)=1
                        gamma_mask(j,i,s)=1
                     else if (nmlab(i).eq.jtmode(s) .and.& 
                          nmlab(j).eq.symmode) then
                        gamma_mask(i,j,s)=1
                        gamma_mask(j,i,s)=1
                     else if (nmlab(j).eq.jtmode(s) .and.& 
                          nmlab(i).eq.symmode) then
                        gamma_mask(i,j,s)=1
                        gamma_mask(j,i,s)=1
                     endif
                  enddo
               enddo
            enddo
         endif
      else
         do s=1,nsta
            do i=1,nmodes
               gamma_mask(i,i,s)=2
               do j=i+1,nmodes
                  if (nmlab(i).eq.nmlab(j)) then
                     gamma_mask(i,j,s)=2
                     gamma_mask(j,i,s)=2
                  else if (nmlab(i).eq.jtmode(s) .and.&
                       nmlab(j).eq.symmode) then
                     gamma_mask(i,j,s)=2
                     gamma_mask(j,i,s)=2
                  else if (nmlab(j).eq.jtmode(s) .and.&
                       nmlab(i).eq.symmode) then
                     gamma_mask(i,j,s)=2
                     gamma_mask(j,i,s)=2
                  endif
               enddo
            enddo
         enddo
      endif
      if (lgamdiag) then
         do s=1,nsta
            do i=1,nmodes
               gamma_mask(i,i,s)=1
            enddo
         enddo
      endif

! third order parameters
      if (liota) then
         if (lautosym) then
            do i=1,nsta
               do j=1,nmodes
                  do k=1,nmodes
                     do c=1,maxdim
                        nmchk(c)=0
                        stachk(c)=0
                     enddo
                     nmchk(k)=nmchk(k)+1
                     nmchk(j)=nmchk(j)+2
                     stachk(i)=2
                     call integralsym(check,nmchk,stachk,axchk)
                     if (check.eq.1) then
                        if(liotadiag.eqv..true.) then
                           if(j.eq.k) iota_mask(k,j,i,i)=1
                        else
                           iota_mask(k,j,i,i)=1
                        endif
                     endif
                  enddo
               enddo
            enddo
         else
            do s=1,nsta
               do j=1,nmodes
                  do i=1,nmodes
                     if (nmlab(i).eq.jtmode(s) .or.&
                          nmlab(i).eq.symmode) then
                        iota_mask(i,j,s,s)=1
                     endif
                     if(liotadiag.eqv..true.) then
                        if(j.ne.k) iota_mask(i,j,s,s)=2
                     endif
                  enddo
               enddo
            enddo
         endif
      else
         do s=1,nsta
            do j=1,nmodes
               do i=1,nmodes
                  if (nmlab(i).eq.jtmode(s) .or.&
                       nmlab(i).eq.symmode) then
                     iota_mask(i,j,s,s)=2
                  endif
               enddo
            enddo
         enddo
      endif
      if (.not. ldiagfit .and. liota) then
         do s=1,nsta
            do i=1,nmodes
               iota_mask(i,i,s,s)=2
            enddo
         enddo
      endif
         
! for adiabatic 2nd order model ignore mu parameters
      if (hamord .eq. 2) go to 100
      
      if (lmu) then
         if (lautosym.eqv..true.) then
            do i=1,nsta
               do j=1,nsta
                  do k=1,nmodes
                     do l=1,nmodes
                        do c=1,maxdim
                           nmchk(c)=0
                           stachk(c)=0
                        enddo
                        nmchk(l)=1
                        nmchk(k)=1
                        if (k.eq.l) nmchk(k)=2
                        stachk(j)=1
                        stachk(i)=1
                        if(j.eq.i) stachk(j)=2
                        call integralsym(check,nmchk,stachk,axchk)
                        if (check.eq.1) then
                        
                           if(lmudiag.eqv..true.) then
                              if(l.eq.k) then
                                 mu_mask(l,k,j,i)=1
                                 mu_mask(k,l,j,i)=1
                                 mu_mask(l,k,i,j)=1
                                 mu_mask(k,l,i,j)=1
                              endif
                           else
                              mu_mask(l,k,j,i)=1
                              mu_mask(k,l,j,i)=1
                              mu_mask(l,k,i,j)=1
                              mu_mask(k,l,i,j)=1
                           endif
                        endif
                     enddo
                  enddo
               enddo
            enddo
         else
            do s=1,nsta
               do s1=s+1,nsta
                  do i=1,nmodes
                     do j=1,nmodes
!       WARNING :      THIS IS A DODGY ADDITION TO ALLOW FOR rt-modes (mu TERMS
!       along degenerate modes
!                     if (nmlab(i).eq.jtmode(s) ) then
!                      mu_mask(i,j,s,s1)= 1
!                      mu_mask(i,j,s1,s)=mu_mask(i,j,s,s1)
!                      mu_mask(j,i,s,s1)=mu_mask(i,j,s,s1)
!                      mu_mask(j,i,s1,s)=mu_mask(i,j,s1,s)
!                     else       
                        mu_mask(i,j,s,s1)=&
                             isbicplmode(s,s1,nmlab(i),nmlab(j),1)
                        mu_mask(i,j,s1,s)=mu_mask(i,j,s,s1)
                        mu_mask(j,i,s,s1)=mu_mask(i,j,s,s1)
                        mu_mask(j,i,s1,s)=mu_mask(i,j,s1,s)
!                     endif 
                     enddo
                  enddo
               enddo
            enddo
            if(lmudiag.eqv..true.) then
               do s=1,nsta
                  do s1=s+1,nsta
                     do i=1,nmodes-1
                        do j=i+1,nmodes
                           mu_mask(i,j,s,s1)=&
                                isbicplmode(s,s1,nmlab(i),nmlab(j),2)
                           mu_mask(i,j,s1,s)=mu_mask(i,j,s,s1)
                           mu_mask(j,i,s,s1)=mu_mask(i,j,s,s1)
                           mu_mask(j,i,s1,s)=mu_mask(i,j,s1,s)
                        enddo
                     enddo
                  enddo
               enddo
            endif
         endif
      else
         do s=1,nsta
            do s1=s+1,nsta
               do i=1,nmodes
                  do j=1,nmodes
                     mu_mask(i,j,s,s1)=&
                          isbicplmode(s,s1,nmlab(i),nmlab(j),2)
                     mu_mask(i,j,s1,s)=mu_mask(i,j,s,s1)
                     mu_mask(j,i,s,s1)=mu_mask(i,j,s,s1)
                     mu_mask(j,i,s1,s)=mu_mask(i,j,s1,s)
                  enddo
               enddo
            enddo
         enddo
      endif

      if (.not. ldiagfit .and. lmu) then
         do s=1,nsta
            do s1=s+1,nsta
               do i=1,nmodes
                  if (mu_mask(i,i,s,s1) .eq. 1) then
                     mu_mask(i,i,s,s1)= 2
                     mu_mask(i,i,s1,s)=mu_mask(i,i,s,s1)
                  endif
               enddo
            enddo
         enddo
      endif

! third order parameters
      if (liota2) then
         do s=1,nsta-1
            do s1=s+1,nsta
               do j=1,nmodes
                  do i=1,nmodes
                     if (nmlab(i).eq.cpmode(s,s1)) then
                        iota_mask(i,j,s,s1)=1
                        iota_mask(i,j,s1,s)=1
                     endif
                  enddo
               enddo
            enddo
         enddo
      else
         do s=1,nsta-1
            do s1=s+1,nsta
               do j=1,nmodes
                  do i=1,nmodes
                     if (nmlab(i).eq.cpmode(s,s1)) then
                        iota_mask(i,j,s,s1)=2
                        iota_mask(i,j,s1,s)=2
                     endif
                  enddo
               enddo
            enddo
         enddo
      endif
      if (.not. ldiagfit .and. liota2) then
         do s=1,nsta-1
            do s1=s+1,nsta
               do i=1,nmodes
                  iota_mask(i,i,s,s1)=2
                  iota_mask(i,i,s1,s)=2
               enddo
            enddo
         enddo
      endif


  100 continue

!       this will print into the screen the symmetry allowed explicit polynomials:    
!     do s = 1, nsta
!      do s1 = s, nsta
!       stachk=0
!       stachk(s) = 1
!       stachk(s1)= stachk(s1) + 1
!        do i = 1, 2
!         do j = 1, 2
!          do k = 1, 2
!           do m = 1, 2
!             nmchk=0
!             nmchk(2) = i
!             nmchk(3) = j
!             nmchk(6) = k
!             nmchk(7) = m
!             call integralsym(check,nmchk,stachk,axchk)
!             if (check .eq. 1) then
!                print*, 'explicit ',  ((s)), ' ', ((s1)), &
!                ' 2^',((i)),' 3^',((j)),' 6^',((k)),' 7^',((m))
!             endif
!           enddo
!          enddo
!         enddo
!        enddo
!      enddo
!     enddo
!-----------------------------------------------------------------------
! Explicitly added terms
!-----------------------------------------------------------------------
      if (lautosym) then
         do i=1,nadd
            
            do c=1,maxdim
               nmchk(c)=0
               stachk(c)=0
            enddo

            s=termst(1,i)
            s1=termst(2,i)
            
            stachk(s)=stachk(s)+1
            stachk(s1)=stachk(s1)+1
            
            do m=1,nmodes
               nmchk(m)=nmchk(m)+termdef(m,i)
            enddo

            call integralsym(check,nmchk,stachk,axchk)

            if (check.eq.1) termk_mask(i)=1

         enddo
      endif

!
! check if general diabatic potential used in place of HO. If diabatic
! potential is flagged for optimisation, flag for optimisation. 
!
      do s=1,nsta
         do i=1,nmodes
            if (ncoeff(i,s) .gt. 0) then 
               if (ldifit) then
!
! if zeroth-order diabatic function is a quartic expansion, then we can
! determine mask values based on symmetry considerations
!
                  if (ifdiab(i,s).eq.1) then
                     if (lautosym) then
                        do c=1,maxdim
                           nmchk(c)=0
                           stachk(c)=0
                        enddo
! KAPPA
                        nmchk(i)=1
                        stachk(s)=2
                        call integralsym(check,nmchk,stachk,axchk)
                        if (check.eq.1) then
                          expcoe_mask(1,i,s)=1
                        else
                          expcoe_mask(1,i,s)=0
                        endif
! GAMMA
                        nmchk(i)=2
                        stachk(s)=2
                        call integralsym(check,nmchk,stachk,axchk)
                        if (check.eq.1) expcoe_mask(2,i,s)=1
! EPSILO
                        nmchk(i)=4
                        stachk(s)=2
                        call integralsym(check,nmchk,stachk,axchk)
                        if (check.eq.1) expcoe_mask(3,i,s)=1
                     else
                        do k=1,ncoeff(i,s)
                           expcoe_mask(k,i,s)=1
                        enddo
                     endif
                  elseif (ifdiab(i,s).eq.2) then
                     if (lautosym) then
                        do c=1,maxdim
                           nmchk(c)=0
                           stachk(c)=0
                        enddo
!D0		        
                        nmchk(i)=1
                        stachk(s)=2
                        call integralsym(check,nmchk,stachk,axchk)
                        if (check.eq.1) then
                          expcoe_mask(1,i,s)=1
                        else
                          expcoe_mask(1,i,s)=0
                        endif
                        expcoe_mask(2,i,s) = 1
                        expcoe_mask(3,i,s) = 1
!		     endif
                     else
                        do k=1,ncoeff(i,s)
                           expcoe_mask(k,i,s)=1
                        enddo
                     endif
                  else
                     do k=1,ncoeff(i,s)
                        expcoe_mask(k,i,s)=1
                     enddo
                  endif

               else 
                  do k=1,ncoeff(i,s)
                     expcoe_mask(k,i,s)=2
                  enddo
               endif
!	what is this???
               if (kappa_mask(i,s).ne.0) kappa_mask(i,s)=2
               if (gamma_mask(i,i,s).ne.0) gamma_mask(i,i,s)=2
               if (iota_mask(i,i,s,s).ne.0.and.ifdiab(i,s).ne.1)&
                    iota_mask(i,i,s,s)=2
            endif
         enddo
      enddo  
     
!       set cp2d_masks all to 1 (no symmetry arguments possible)
      cp2d_mask = 1
  
!-----------------------------------------------------------------------
! Check if a general function is to be used in place of a Taylor
! expansion of an off-diagonal element of the potential matrix. If
! so, flag corresponding parameters for optimisation and ensure that
! corresponding lambda and bilinear mu parameters are not optimised.
!-----------------------------------------------------------------------
      do s1=1,nsta
         do s=1,nsta
            do m=1,nmodes
               if (ncoeffcp(m,s,s1) .gt. 0) then
                  do k=1,ncoeffcp(m,s,s1)
                     cpcoe_mask(k,m,s,s1)=1
                  enddo
                  if (lambda_mask(m,s,s1).ne.0) lambda_mask(m,s,s1)=2
                  if (mu_mask(m,m,s,s1).ne.0) mu_mask(m,m,s,s1)=2
               endif
            enddo
         enddo
      enddo   

!-----------------------------------------------------------------------
! Switching functions
!-----------------------------------------------------------------------
!
! If switching functions are used to correlate to degrees of freedom,
! then we need to ensure that any corresponding bilinear gamma,
! bilinear mu, etc terms are not optimised
!
!-----------------------------------------------------------------------
      do i=1,nswitch
! Expansion of on-diagonal elements
         if (swpar(i).eq.1.or.swpar(i).eq.6) then
            ! kappa or general switching
            gamma_mask(swmd(i),swm(i),sws(i))=0
            gamma_mask(swm(i),swmd(i),sws(i))=0
            iota_mask(swmd(i),swm(i),sws(i),sws(i))=0
            iota_mask(swm(i),swmd(i),sws(i),sws(i))=0
         else if (swpar(i).eq.3.or.swpar(i).eq.5) then
            ! gamma or iota switching
            if (swm(i).eq.swm1(i)) then
               gamma_mask(swmd(i),swm(i),sws(i))=0
               gamma_mask(swm(i),swmd(i),sws(i))=0
               iota_mask(swmd(i),swm(i),sws(i),sws(i))=0
               iota_mask(swm(i),swmd(i),sws(i),sws(i))=0
            endif
! Expansion of off-diagonal elements
         else if (swpar(i).eq.2) then
            ! lambda switching
            mu_mask(swmd(i),swm(i),sws(i),sws1(i))=0
            mu_mask(swmd(i),swm(i),sws1(i),sws(i))=0
            mu_mask(swm(i),swmd(i),sws(i),sws1(i))=0
            mu_mask(swm(i),swmd(i),sws1(i),sws(i))=0
            iota_mask(swmd(i),swm(i),sws(i),sws1(i))=0
            iota_mask(swmd(i),swm(i),sws1(i),sws(i))=0
            iota_mask(swm(i),swmd(i),sws(i),sws1(i))=0
            iota_mask(swm(i),swmd(i),sws1(i),sws(i))=0
         endif
      enddo

      return
      end subroutine createmask




!#################################################################
      integer(long) function factorial(x)
      integer(long), intent(in) :: x
      integer(long) :: i
      factorial = 1
      do i =1, x
       factorial = factorial*i
      enddo
      return
      end function
!#################################################################

      integer(long) function isbicplmode(s,s1,mode1,mode2,imask)

      implicit none
      
      integer(long)                :: n
      integer(long), intent(in)    :: imask,s,s1
      character(len=5), intent(in) :: mode1,mode2


      isbicplmode=0
      do n = 1,ncp2(s,s1)
         if ((mode1.eq.cpmode2(s,s1,1,n)) .and.&
                   (mode2.eq.cpmode2(s,s1,2,n))) then
            isbicplmode=imask
         else if ((mode1.eq.cpmode2(s,s1,2,n)) .and.&
                   (mode2.eq.cpmode2(s,s1,1,n))) then
            isbicplmode=imask
         endif
      enddo
      
      end function isbicplmode
      
!#######################################################################
!
! TO BE DONE: need to add mask for explicit terms
!
!#######################################################################

      subroutine addparams(const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask)

      implicit none

      integer(long), external                                          :: slen
      integer(long)                                                    :: m,m1,s,s1,k,i,ilbl,jlbl,p,iz,ic,ierr,iz0,chk
      integer(long), dimension(nsta,nsta), intent(inout)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(inout)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(inout) :: termk_mask
      integer(long), dimension(maxdim) :: tmdef
      integer(long), dimension(2) :: tmst
      logical(kind=4) :: found

      rewind(iin)

      chk = 0
      iz0 = 0
!-----------------------------------------------------------------------
! read down to add-parameter list
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'addparams (read VCHFIT input)'

      if( ierr .eq. 1) go to 999
      call vccntrl(ierr)
      if (keyword(1).ne.'addpar-section' .and. &
          keyword(1) .ne. 'add_parameters-section') go to 10
        
!-----------------------------------------------------------------------
! read parameters to be added. If already present ignore instruction
!
! look to see if parameter is to be frozen / optimized
!
! General coefficients cannot be added - they are included by a 
! diabatic state definition
!-----------------------------------------------------------------------
 20   continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'addparams (read VCHFIT input)'

      call vccntrl(ierr)
!      print*, 'DEBUG: keygword: ', keyword(1)
      if (keyword(1) .eq. 'end-addpar-section' .or. &
          keyword(1) .eq. 'end-add_parameters-section') then
         go to 999

      else if (keyword(1) .eq. 'e') then
         read(keyword(2),*) s
         read(keyword(3),*) s1
!         if (const_mask(s,s1) .ne. 0) go to 20

         if (keyword(5) .eq. 'freeze') then
            const_mask(s,s1) = 2
         else if (keyword(5)(1:5) .eq. 'optim') then
            const_mask(s,s1) = 1
         endif
         const_mask(s1,s) = const_mask(s,s1)

      else if (keyword(1) .eq. 'kappa') then
         read(keyword(2),*) m
         read(keyword(3),*) s
!         if (kappa_mask(m,s) .ne. 0) go to 20

         if (keyword(5) .eq. 'freeze') then
            kappa_mask(m,s) = 2
            if( ifdiab(m,s) .eq. 1 ) expcoe_mask(1,m,s) = 2
         else if (keyword(5)(1:5) .eq. 'optim') then
            kappa_mask(m,s) = 1
            if(ifdiab(m,s) .eq. 1 ) expcoe_mask(1,m,s) = 1
         endif

      else if (keyword(1) .eq. 'lambda') then
         read(keyword(2),*) m
         read(keyword(3),*) s
         read(keyword(4),*) s1
!         if (lambda_mask(m,s,s1) .ne. 0) go to 20

         if (keyword(6) .eq. 'freeze') then
            lambda_mask(m,s,s1) = 2
         else if (keyword(6)(1:5) .eq. 'optim') then
            lambda_mask(m,s,s1) = 1
         endif
         lambda_mask(m,s1,s) = lambda_mask(m,s,s1)

      else if (keyword(1) .eq. 'gamma') then
         read(keyword(2),*) m
         read(keyword(3),*) m1
         read(keyword(4),*) s
!         if (gamma_mask(m,m1,s) .ne. 0) go to 20

         if (keyword(6) .eq. 'freeze') then
            gamma_mask(m,m,s) = 2
            if(ifdiab(m,s) .eq. 1 ) expcoe_mask(2,m,s) = 2
         else if (keyword(6)(1:5) .eq. 'optim') then
             gamma_mask(m,m1,s) = 1
             if(ifdiab(m,s) .eq. 1 ) expcoe_mask(2,m,s) = 1
         endif
         if (m .ne. m1) gamma_mask(m1,m,s) = gamma_mask(m,m1,s)

      else if (keyword(1) .eq. 'iota') then
         read(keyword(2),*) m
         read(keyword(3),*) m1
         read(keyword(4),*) s
!         if (iota_mask(m,m1,s,s) .ne. 0) go to 20
         if (keyword(6) .eq. 'freeze') then
            iota_mask(m,m1,s,s) = 2
            iota_mask(m1,m,s,s) = 2
         else if (keyword(6)(1:5) .eq. 'optim') then
             iota_mask(m,m1,s,s) = 1
         endif

      else if (keyword(1) .eq. 'mu') then
         read(keyword(2),*) m
         read(keyword(3),*) m1
         read(keyword(4),*) s
         read(keyword(5),*) s1
!         if (mu_mask(m,m1,s,s1) .ne. 0) go to 20

         if (keyword(7) .eq. 'freeze') then
            mu_mask(m,m1,s,s1) = 2
         else if (keyword(7)(1:5) .eq. 'optim') then
            mu_mask(m,m1,s,s1) = 1
         endif

         mu_mask(m,m1,s1,s) = mu_mask(m,m1,s,s1)
         if (m .ne. m1) then
            mu_mask(m1,m,s,s1) = mu_mask(m,m1,s,s1)
            mu_mask(m1,m,s1,s) = mu_mask(m,m1,s,s1)
         endif

      else if (keyword(1) .eq. 'iota2') then
         read(keyword(2),*) m
         read(keyword(3),*) m1
         read(keyword(4),*) s
         read(keyword(5),*) s1
!         if (iota_mask(m,m1,s,s1) .ne. 0) go to 20

         if (keyword(7) .eq. 'freeze') then
            iota_mask(m,m1,s,s1) = 2
         else if (keyword(7)(1:5) .eq. 'optim') then
             iota_mask(m,m1,s,s1) = 1
         endif
         iota_mask(m,m1,s1,s) = iota_mask(m,m1,s,s1)


      else if ((keyword(1) .eq. 'general') .or. (keyword(1) .eq. &
       'General')) then
         read(keyword(2),*) k
         read(keyword(3),*) m
         read(keyword(4),*) s
!         if (expcoe_mask(k,m,s) .ne. 0) go to 20
         if (keyword(6) .eq. 'freeze') then
            expcoe_mask(k,m,s) = 2
         else if (keyword(6)(1:5) .eq. 'optim') then
            expcoe_mask(k,m,s) = 1
         endif

      else if ((keyword(1) .eq. 'explicit') .or. (keyword(1) .eq. &
              'Explicit')) then

         call zerovxi(tmdef,maxdim)
         call zerovxi(tmst,2)
         found = .false.
         read(keyword(2),*,iostat=ierr) tmst(1)
         read(keyword(3),*,iostat=ierr) tmst(2)
         i = 4
         do i=4,ic
            if(index(keyword(i),'=') .eq. 1) EXIT
            ilbl=index(keyword(i),'^')
            jlbl=slen(keyword(i))
            if (ilbl .eq. 0) then
               p = 1
               read(keyword(i),*,iostat=ierr) m
            else
               read(keyword(i)(1:ilbl-1),*,iostat=ierr) m
               read(keyword(i)(ilbl+1:jlbl),*,iostat=ierr) p
            endif
            tmdef(m) = p
         enddo
         if (ierr .ne. 0) then
             routine = 'Addparams '
             message='Error reading explicit term definition'
             call vcwrline(i,iz,iz0)
         endif
!
! see if term already present, if not add
!
        call chkexplicit(tmdef,tmst,termdef,termst,nadd,found,nmodes)
         if (.not. found) then
            chk=chk+1
            if (chk .gt. nadd) go to 20
            termst(1,chk) = tmst(1)
            termst(2,chk) = tmst(2)
            do m=1,nmodes
               termdef(m,chk) = tmdef(m)
!               if(tmdef(m) .ne. 0) print*, 'termdef(mode,add): ', termdef(m,chk)
            enddo
!          print*, 'chk=',chk,'st=', termst(1,chk),termst(2,chk)
         endif
         if( i .eq. ic ) then
            termk_mask(chk) = 1
         else
            if(keyword(i+1)(1:5) .eq. 'optim') termk_mask(chk) = 1
            if(keyword(i+1) .eq. 'freeze')     termk_mask(chk) = 2
         endif
      else
         routine = 'Addparams'
         write(message,'(2a)') 'Unknown parameter type: ',keyword(1)
         call vcerrormsg
 
      endif

!	print*, 'symmetry LAMBDA(14,1,5) = ', lambda_mask(14,1,5)
!        print*, 'symmetry LAMBDA(14,1,5) = ', lambda_mask(14,2,5)
      go to 20



  999 continue

      return
      end subroutine addparams

!####################################################################### 

      subroutine chkexplicit(tmdef,tmst,termdef,termst,nadd,found,nmodes)

      implicit none

      integer(long), intent(in) :: nadd,nmodes
      integer(long) :: n,m
      integer(long), dimension(2), intent(in) :: tmst
      integer(long), dimension(maxdim), intent(in) :: tmdef
      integer(long), dimension(2,nadd), intent(out) :: termst
      integer(long), dimension(maxdim,nadd), intent(out) :: termdef
      logical(kind=4), intent(out) :: found
      logical(kind=4) :: compare

      found=.false.
      do n=1,nadd
         compare=.true.
         if (termst(1,n) .ne. tmst(1)) compare=.false.
         if (termst(2,n) .ne. tmst(2)) compare=.false.
         do m=1,nmodes
            if (termdef(m,n) .ne. tmdef(m)) compare=.false.
         enddo
         if (compare) found=.true.
      enddo

      return
      end subroutine chkexplicit

!#######################################################################
!
! here any constraints on the coefficients are set before optimisation
!
!#######################################################################

      subroutine setcons(const_mask,kappa_mask,lambda_mask, &
           gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask, &
           cpcoe_mask,switch_mask,cp2d_mask,kappa,lambda,expcoe)

      implicit none

      integer(long) :: ilbl
      integer(long), dimension(nsta,nsta), intent(inout) :: const_mask
      integer(long), dimension(nmodes,nsta), intent(inout) :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout) :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout) :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout) :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: &
                                                            iota_mask,mu_mask
      integer(long), dimension(nadd), intent(inout) :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(inout) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe), intent(inout) :: switch_mask
      integer(long), dimension(maxcoe,maxadd), intent(inout)         :: cp2d_mask
      real(dop), dimension(nmodes,nsta), intent(in) :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(in) :: lambda
      real(dop), dimension(maxcoe,nmodes,nsta), intent(inout) :: expcoe


      if (lrdmask) then
         ilbl=index(guessfile,' ')-1
         write(6,'(/,2a)')&
              'Reading constraints from file : ',guessfile(1:ilbl)
         open(iguess,file=guessfile,form='formatted',status='old')
         call readcons(iguess,const_mask,kappa_mask,lambda_mask, &
              gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask, &
              cpcoe_mask,switch_mask,cp2d_mask)
         close(iguess)
      else
! JT constraints - strictly defined by system
         if (njt .gt. 0) call jtcoesymm(const_mask,kappa_mask,&
                              lambda_mask,gamma_mask,iota_mask,&
                              mu_mask,expcoe_mask,termk_mask,&
                              cpcoe_mask,switch_mask,&
                              kappa,lambda)

! now add user defined constraints
         if (lrdcons) call readcons(iin,const_mask,kappa_mask, &
                           lambda_mask,gamma_mask,iota_mask,mu_mask, &
                           expcoe_mask,termk_mask,cpcoe_mask,&
                           switch_mask,cp2d_mask)         

! diabatic states for degenerate states - keep identical unless
! other constraints set above
         call emodecons(const_mask,kappa_mask,lambda_mask,gamma_mask,&
              iota_mask,mu_mask,expcoe_mask,termk_mask,cpcoe_mask,&
              switch_mask)

! 4th-order JT constraints
         if (ljt4cons) call setjt4cons(const_mask,kappa_mask,lambda_mask,&
              gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
              cpcoe_mask,switch_mask,expcoe)

      endif

      if (ncons .eq. 0) then
         if(.not. gaoptim ) &
         write(ilog,'(a)') &
            'Warning: Constraints asked for but none set ?'
      endif
        
      return
      end subroutine setcons

!#######################################################################

      subroutine emodecons(const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask)

      implicit none

      integer(long)                                                 :: m,m1,s,k,icons,m2,attermk
      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout)   :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)   :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in)                    :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe) :: switch_mask

      real(dop)                          :: diff
      logical(kind=4), dimension(maxdim) :: found

!-----------------------------------------------------------------------
! get degenerate pairs and constrain diabatic potentials to be equal
! along them
!-----------------------------------------------------------------------
      call zerovxl(found,nmodes)
outer:do m=1,nmodes
         if (found(m)) cycle outer

         if (nmlab(m)(1:1) .eq. 'E') then
            continue
         else
            cycle outer
         endif

! modes m,m1 are degenerate pair
         m1=0
         do m2=m+1,nmodes
            if (nmlab(m2) .eq. nmlab(m)) then
               diff=freq0(m2)-freq0(m)
               if (abs(diff) .lt. 1.0e-4_dop) then
                  m1=m2
               endif
            endif
         enddo
         if (m1 .eq. 0) then
            routine='Emodecons'
            write(message,'(a,i5)') &
                 'Degenerate pair not found for mode :',m
            call vcerrormsg
         else
            found(m1) = .true.
         endif

! get list number of to-be-optimised parameter kappa(jt1,yess)
         do s=1,nsta
            if (ifdiab(m,s) .ne. 0) then
! set expcoe(k,m1,s) = expcoe(k,m,s)
               do k=1,ncoeff(m,s)
                  if (expcoe_mask(k,m,s) .gt. 0 .and. &
                      expcoe_mask(k,m1,s) .gt. 0) then
                     call parnumt(icons,6,m,s,0,0,k,attermk,const_mask,&
                          kappa_mask,lambda_mask,gamma_mask,iota_mask,&
                          mu_mask,expcoe_mask,termk_mask,cpcoe_mask,&
                          switch_mask)

                     ncons=ncons+1
                     expcoe_mask(k,m1,s)=-ncons
                     if (ncons .le. maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=1.0
                     endif
                  endif
               enddo

            else if (gamma_mask(m,m,s) .gt. 0 .and.&
                      gamma_mask(m1,m1,s) .gt. 0) then

               call parnumt(icons,3,m,s,m,0,0,attermk,const_mask,&
                    kappa_mask,lambda_mask,gamma_mask,iota_mask,&
                    mu_mask,expcoe_mask,termk_mask,cpcoe_mask,&
                    switch_mask)

               ncons=ncons+1
               gamma_mask(m1,m1,s)=-ncons
               if (ncons .le. maxcons) then
                  parcons(ncons)=icons
                  pconfact(ncons)=1.0
               endif
            endif

         enddo

!  100    continue
      enddo outer

      return
      end subroutine emodecons

!#######################################################################

      subroutine jtcoesymm(const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask,kappa,lambda)

      implicit none

      real(dop)                                                     :: diff,diff1
      logical(kind=4), dimension(maxdim)                            :: found

      integer(long)                                                 :: s,s1,i,jt1,jt2,ijt,id,m,m1,m2,yess,nos,icons,attermk
      integer(long), dimension(maxset)                              :: jtset
      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(inout)          :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout)     :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in)                    :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe) :: switch_mask
      real(dop), dimension(nmodes,nsta), intent(in)                 :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(in)            :: lambda


!-----------------------------------------------------------------------
! initialise counter for JT set
!-----------------------------------------------------------------------
      ijt=1

!-----------------------------------------------------------------------
! for each set of Jahn-Teller states, load set into jtset
! id = degeneracy of JT set
!-----------------------------------------------------------------------
   10 continue
      id=0
      do s=1,nsta
         if (jtsta(s) .eq. ijt) then
            id=id+1
            jtset(id)=s
         endif
      enddo
          
!-----------------------------------------------------------------------
! find symmetric modes
!-----------------------------------------------------------------------
      do m=1,nmodes
         if (nmlab(m) .eq. symmode) then
! get number of parameter to be optimised, kappa(m,s)
            s=jtset(1)
            call parnumt(icons,1,m,s,0,0,0,attermk,const_mask,kappa_mask,&
              lambda_mask,gamma_mask,iota_mask,mu_mask,expcoe_mask,&
              termk_mask,cpcoe_mask,switch_mask)

! set kappa(m,s1) = kappa(m,s)
            do i=2,id
               s1=jtset(i)
               ncons=ncons+1
               kappa_mask(m,s1)=-ncons
               if (ncons .le. maxcons) then
                  parcons(ncons)=icons
                  pconfact(ncons)=1.0
               endif
            enddo
         endif
      enddo

      if (id .ne. 2) go to 999

!-----------------------------------------------------------------------
! for Exe JT
! get degenerate pair and set mutual constraints
!-----------------------------------------------------------------------
      s=jtset(1)
      s1=jtset(2)
      if (cointype(s1,s) .eq. 2) then
         call zerovxl(found,nmodes)
         do m=1,nmodes
            if (found(m)) go to 100

            if ((nmlab(m) .eq. jtmode(s)) .and.&
                         (kappa_mask(m,s) .eq. 1)) then
               continue
            else
               go to 100
            endif

! modes m,m1 are JT pair
            m1=0
            do m2=m+1,nmodes
               if (nmlab(m2) .eq. nmlab(m)) then
                  diff=freq0(m2)-freq0(m)
                  if (abs(diff) .lt. 1.0e-6_dop) then
                     m1=m2
                  endif
               endif
            enddo
            if (m1 .eq. 0) then
               routine='JTCOESYM'
               write(message,'(a,i5)') &
                    'JT pair not found for mode :',m
               call vcerrormsg
            else
               found(m1) = .true.
            endif

! jt1 is on-diagonal nmode
            if (abs(kappa(m,s)) .gt. abs(kappa(m1,s))) then
               jt1=m
               jt2=m1
            else
               jt1=m1
               jt2=m
            endif

! select state with gradient with same direction as off-diag coupling
            diff=kappa(jt1,s)-lambda(jt2,s,s1)
            diff1=kappa(jt1,s1)-lambda(jt2,s,s1)
            if (abs(diff) .lt. abs(diff1)) then
               yess=s
               nos=s1
            else
               yess=s1
               nos=s
            endif

! get list number of to-be-optimised parameter kappa(jt1,yess)
            call parnumt(icons,1,jt1,yess,0,0,0,attermk,const_mask,&
                 kappa_mask,lambda_mask,gamma_mask,iota_mask,mu_mask,&
                 expcoe_mask,termk_mask,cpcoe_mask,switch_mask)

! set kappa(jt1,nos) = -kappa(jt1,yess)
            ncons=ncons+1
            kappa_mask(jt1,nos)=-ncons
            if (ncons .le. maxcons) then
               parcons(ncons)=icons
               pconfact(ncons)=-1.0
            endif
! set lambda(jt2,s,s1) = -kappa(jt1,yess)
            ncons=ncons+1
            lambda_mask(jt2,s,s1)=-ncons
            lambda_mask(jt2,s1,s)=-ncons
            if (ncons .le. maxcons) then
               parcons(ncons)=icons
               pconfact(ncons)=-1.0
            endif

! set kappa(jt2,s) = kappa(jt2,s1) = lambda(jt1,s,s1) = 2 
            kappa_mask(jt2,s)=2
            kappa_mask(jt2,s1)=2
            lambda_mask(jt1,s,s1)=2
            lambda_mask(jt1,s1,s)=2
            
  100       continue
         enddo

!-----------------------------------------------------------------------
! JT modes for Exb JT
!-----------------------------------------------------------------------
      else if (cointype(s1,s) .eq. 3) then
         do m=1,nmodes
            if (nmlab(m) .eq. jtmode(s) .and.&
                         (kappa_mask(m,s) .eq. 1)) then
! get number of parameter to be optimised, kappa(m,s)
               call parnumt(icons,1,m,s,0,0,0,attermk,const_mask,&
                    kappa_mask,lambda_mask,gamma_mask,iota_mask,&
                    mu_mask,expcoe_mask,termk_mask,cpcoe_mask,&
                    switch_mask)

! set kappa(m,s1) = -kappa(m,s)
               ncons=ncons+1
               kappa_mask(m,s1)=-ncons
               if (ncons .le. maxcons) then
                  parcons(ncons)=icons
                  pconfact(ncons)=-1.0
               endif
            endif
         enddo
      endif

  999 continue
!-----------------------------------------------------------------------
! look for any more JT systems
!-----------------------------------------------------------------------
      if (ijt .lt. njt) then
         ijt = ijt+1
         go to 10
      endif

      if (ncons .gt. maxcons) then
         write(6,'(a,i7)') 'Redimnsion MAXCONS. Minimum : ',ncons
         stop
      endif

      return
      end subroutine jtcoesymm

!#######################################################################

      subroutine readcons(unit,const_mask,kappa_mask,lambda_mask, &
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask, &
                 cpcoe_mask,switch_mask,cp2d_mask)


      implicit none

      character(len=80) :: atmp,achk,achk1

      integer(long)                                                    :: m,m1,s,s1,k,iz,ic,ierr,o
      integer(long) :: ilbl,jlbl,slen,schk,s2chk,p,i,j,l,ptmp,ieq,nn
      integer(long), dimension(nmodes) :: defchk,chk
      integer(long), intent(in)                                        :: unit
      integer(long), dimension(nsta,nsta), intent(inout)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(inout)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(inout) :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(inout) :: cpcoe_mask
      integer(long), dimension(maxcoe,maxadd), intent(inout)         :: cp2d_mask
      integer(long), dimension(nswitch,maxcoe) :: switch_mask

      if (unit .eq. iin) then
         routine = 'readcons (read VCHFIT input)'
      else if (unit .eq. iguess) then
         routine = 'readcons (read GUESS)'
      endif
      rewind(unit)

!-----------------------------------------------------------------------
! read down to constraint list
! could be in iin or iguess 
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(unit,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 2) then
         write(ilog,'(a)') 'No Constraints Found'
         go to 999
      else
         call vccntrl(ierr)
      endif

      if (keyword(1).ne.'constraints-section') go to 10

!-----------------------------------------------------------------------
! read constraints
! find which parameter is being constrained
!
! look to see if parameter is being frozen / unfrozen
! or else flag as "to be constrained". Note all constraints are set, not
! just active atoms. This is so that full information can be written in
! vcham file.
!-----------------------------------------------------------------------
 20   continue
      call rdinpf(unit,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      call vccntrl(ierr)

      if (keyword(1).eq.'end-constraints-section') then
         go to 999

      else if (keyword(1) .eq. 'e') then
         read(keyword(2),*,err=900) s
         read(keyword(3),*,err=900) s1
         if (const_mask(s,s1) .lt. 0) go to 20
         if (const_mask(s,s1) .eq. 0 .and. lspecies) go to 20

         if (keyword(5) .eq. 'freeze') then
            if (const_mask(s,s1) .eq. 1) const_mask(s,s1) = 2
         else if (keyword(5) .eq. 'unfreeze') then
            if (const_mask(s,s1) .eq. 2) const_mask(s,s1) = 1
         else 
            ncons=ncons+1
            const_mask(s,s1) = -ncons
            call findcons(5,ncons,const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask,ic)
         endif
         const_mask(s1,s) = const_mask(s,s1)

      else if (keyword(1) .eq. 'kappa') then
         read(keyword(2),*,err=900) m
         read(keyword(3),*,err=900) s
         if (kappa_mask(m,s) .lt. 0) go to 20
         if (kappa_mask(m,s) .eq. 0 .and. lspecies) go to 20

         if (keyword(5) .eq. 'freeze') then
            if (kappa_mask(m,s) .eq. 1) kappa_mask(m,s) = 2
         else if (keyword(5) .eq. 'unfreeze') then
            if (kappa_mask(m,s) .eq. 2) kappa_mask(m,s) = 1
         else 
            ncons=ncons+1
            kappa_mask(m,s) = -ncons
            call findcons(5,ncons,const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask,ic)
         endif
      else if (keyword(1) .eq. 'lambda') then
         read(keyword(2),*,err=900) m
         read(keyword(3),*,err=900) s
         read(keyword(4),*,err=900) s1
         if (lambda_mask(m,s,s1) .le. 0) go to 20
         if (lambda_mask(m,s,s1) .eq. 0 .and. lspecies) go to 20

         if (keyword(6) .eq. 'freeze') then
            if (lambda_mask(m,s,s1) .eq. 1) lambda_mask(m,s,s1) = 2
         else if (keyword(6) .eq. 'unfreeze') then
            if (lambda_mask(m,s,s1) .eq. 2) lambda_mask(m,s,s1) = 1
         else 
            ncons=ncons+1
            lambda_mask(m,s,s1) = -ncons
            call findcons(6,ncons,const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask,ic)
         endif
         lambda_mask(m,s1,s) = lambda_mask(m,s,s1)

      else if (keyword(1) .eq. 'gamma') then
         read(keyword(2),*,err=900) m
         read(keyword(3),*,err=900) m1
         read(keyword(4),*,err=900) s
         if (gamma_mask(m,m1,s) .lt. 0) go to 20
         if (gamma_mask(m,m1,s) .eq. 0 .and. lspecies) go to 20

         if (keyword(6) .eq. 'freeze') then
            if (gamma_mask(m,m1,s) .eq. 1) gamma_mask(m,m1,s) = 2
         else if (keyword(6) .eq. 'unfreeze') then
            if (gamma_mask(m,m1,s) .eq. 2) gamma_mask(m,m1,s) = 1
         else 
            ncons=ncons+1
            gamma_mask(m,m1,s) = -ncons
            call findcons(6,ncons,const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask,ic)
         endif
         if (m .ne. m1) gamma_mask(m1,m,s) = gamma_mask(m,m1,s)

      else if (keyword(1) .eq. 'iota') then
         read(keyword(2),*,err=900) m
         read(keyword(3),*,err=900) m1
         read(keyword(4),*,err=900) s
         if (iota_mask(m,m1,s,s) .lt. 0) go to 20
         if (iota_mask(m,m1,s,s) .eq. 0 .and. lspecies) go to 20
         if (keyword(6) .eq. 'freeze') then
            if (iota_mask(m,m1,s,s) .eq. 1) iota_mask(m,m1,s,s) = 2
         else if (keyword(6) .eq. 'unfreeze') then
            if (iota_mask(m,m1,s,s) .eq. 2) iota_mask(m,m1,s,s) = 1
            if (iota_mask(m,m1,s,s) .eq. 2) iota_mask(m1,m,s,s) = 1
         else
            ncons=ncons+1
            iota_mask(m,m1,s,s) = -ncons
            call findcons(6,ncons,const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask,ic)
         endif

      else if (keyword(1) .eq. 'mu') then
         read(keyword(2),*,err=900) m
         read(keyword(3),*,err=900) m1
         read(keyword(4),*,err=900) s
         read(keyword(5),*,err=900) s1
         if (mu_mask(m,m1,s,s1) .lt. 0) go to 20
         if (mu_mask(m,m1,s,s1) .eq. 0 .and. lspecies) go to 20

         if (keyword(7) .eq. 'freeze') then
            if (mu_mask(m,m1,s,s1) .eq. 1) mu_mask(m,m1,s,s1) = 2
         else if (keyword(7) .eq. 'unfreeze') then
            if (mu_mask(m,m1,s,s1) .eq. 2) mu_mask(m,m1,s,s1) = 1
         else 
            ncons=ncons+1
            mu_mask(m,m1,s,s1) = -ncons
            call findcons(7,ncons,const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask,ic)
         endif

         mu_mask(m,m1,s1,s) = mu_mask(m,m1,s,s1)
         if (m .ne. m1) then
            mu_mask(m1,m,s,s1) = mu_mask(m,m1,s,s1)
            mu_mask(m1,m,s1,s) = mu_mask(m,m1,s,s1)
         endif

     elseif ( index(keyword(1),'tan') .ne. 0) then
      read(keyword(2),*,err=900) m
      read(keyword(3),*,err=900) m1
      read(keyword(4),*,err=900) s
      read(keyword(5),*,err=900) s1
      if( index(keyword(1),'=') .eq. 0 ) read(keyword(6),*) o
      if( j .le. 2) nn = 1
      if( index(keyword(1),'tanch') .ne. 0 .or. index(keyword(1),'poly') .ne. 0) read(keyword(6),*) nn
      if( keyword(1) .eq. 'ltanh2d' ) j = 1
      if( keyword(1) .eq. 'mtanh2d' ) j = 2
      if( keyword(1) .eq. 'radtanch' ) j = 3
      if( keyword(1) .eq. 'vectanch' ) j = 4
      if( keyword(1) .eq. 'polybox' ) j = 5
      if( keyword(1) .eq. 'polybox2' ) j = 6
      k = 1
      if( j .ne. 5) then
       do while (cp2dtyp(k) .ne. 0)
        if(((( dofcp2d(1,k) .eq. m .and. dofcp2d(2,k) .eq. m1) .or.&
           ( dofcp2d(1,k) .eq. m1 .and. dofcp2d(2,k) .eq. m)) .and.&
           (( stacp2d(1,k) .eq. s .and. stacp2d(2,k) .eq. s1) .or. &
           ( stacp2d(1,k) .eq. s1 .and. stacp2d(2,k) .eq. s)) ) .and.&
           (cp2dtyp(k) .eq. j ) ) then
            if(j .eq. 6 .and.  ocp2d(k) .eq. o .or. j .ne. 6 ) then
             if(keyword(ic) .eq. 'freeze') cp2d_mask(nn,k) = 2
             if(keyword(ic) .eq. 'unfreeze') cp2d_mask(nn,k) = 1
            endif
         endif
         k = k + 1
       enddo
      else
       read(keyword(6),*) k
       if(keyword(ic) .eq. 'freeze') cp2d_mask(nn,k) = 2
       if(keyword(ic) .eq. 'unfreeze') cp2d_mask(nn,k) = 1
      endif
      else if (keyword(1) .eq. 'general') then
         read(keyword(2),*,err=900) k
         read(keyword(3),*,err=900) m
         read(keyword(4),*,err=900) s
         if (expcoe_mask(k,m,s) .lt. 0) go to 20
         if (keyword(6) .eq. 'freeze') then
            if (expcoe_mask(k,m,s) .eq. 1) expcoe_mask(k,m,s) = 2
         else if (keyword(6) .eq. 'unfreeze') then
            if (expcoe_mask(k,m,s) .eq. 2) expcoe_mask(k,m,s) = 1
         else
             
            ncons=ncons+1
            expcoe_mask(k,m,s) = -ncons
       !     print*, 'EXPCOEMASK = ', expcoe_mask(k,m,s)
            call findcons(6,ncons,const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask,ic)
         endif

      else if (keyword(1) .eq. 'cpgeneral') then
         read(keyword(2),*,err=900) k
         read(keyword(3),*,err=900) m
         read(keyword(4),*,err=900) s
         read(keyword(5),*,err=900) s1
         if (cpcoe_mask(k,m,s,s1) .lt. 0) go to 20
         if (keyword(7) .eq. 'freeze') then
            if (cpcoe_mask(k,m,s,s1) .eq. 1) cpcoe_mask(k,m,s,s1) = 2
         else if (keyword(7) .eq. 'unfreeze') then
            if (cpcoe_mask(k,m,s,s1) .eq. 2) cpcoe_mask(k,m,s,s1) = 1
         else 
            ncons=ncons+1
            cpcoe_mask(k,m,s,s1) = -ncons
            call findcons(7,ncons,const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask,ic)
         endif
         cpcoe_mask(k,m,s1,s)=cpcoe_mask(k,m,s,s1)

      else if (keyword(1) .eq. 'iota2') then
         read(keyword(2),*,err=900) m
         read(keyword(3),*,err=900) m1
         read(keyword(4),*,err=900) s
         read(keyword(5),*,err=900) s1
         if (iota_mask(m,m1,s,s1) .lt. 0) go to 20
         if (iota_mask(m,m1,s,s1) .eq. 0 .and. lspecies) go to 20

         if (keyword(7) .eq. 'freeze') then
            if (iota_mask(m,m1,s,s1) .eq. 1) iota_mask(m,m1,s,s1) = 2
         else if (keyword(7) .eq. 'unfreeze') then
            if (iota_mask(m,m1,s,s1) .eq. 2) iota_mask(m,m1,s,s1) = 1
         else
            ncons=ncons+1
            iota_mask(m,m1,s,s1) = -ncons
            call findcons(7,ncons,const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask,ic)
         endif
         iota_mask(m,m1,s1,s) = iota_mask(m,m1,s,s1)

!-----------------------------------------------------------------------
! Explicitely added terms
!-----------------------------------------------------------------------
      else if ((keyword(1).eq.'explicit').or.&
           (keyword(1).eq. 'Explicit')) then
         do i=1,ic
            if (keyword(i).eq.'=') ieq=i
         enddo

         do i=1,nmodes
            defchk(i)=0.0
         enddo
         read(keyword(2),*,iostat=ierr) schk
         read(keyword(3),*,iostat=ierr) s2chk

         do i=4,ieq-1
            ilbl=index(keyword(i),'^')
            jlbl=slen(keyword(i))
            if (ilbl .eq. 0) then
               p = 1
               read(keyword(i),*,iostat=ierr) m
            else
               read(keyword(i)(1:ilbl-1),*,iostat=ierr) m
               read(keyword(i)(ilbl+1:jlbl),*,iostat=ierr) p
            endif
            defchk(m) = p
         enddo

         do i=1,nadd

            do j=1,nmodes
               chk(j)=1
            enddo
            if (schk.eq.termst(1,i).and.s2chk.eq.termst(2,i).or.    &
                 (schk.eq.termst(2,i)).and.s2chk.eq.termst(1,i)) then
               do j=1,nmodes
                  chk(j)=0
               enddo
               do j=1,nmodes
                  chk(j)=defchk(j)-termdef(j,i)
                  if (chk(j).ne.0) go to 300
               enddo
               if (termk_mask(i) .lt. 0) go to 20
               if (termk_mask(i) .eq. 0 .and. lspecies) go to 20
               if (keyword(ieq+1).eq.'freeze') then
                  termk_mask(i)=2
               else if (keyword(ieq+1).eq.'unfreeze') then
                  termk_mask(i)=1
               else
                  ncons=ncons+1
                  termk_mask(i) = -ncons
                  call findcons(ieq+1,ncons,const_mask,kappa_mask,&
                       lambda_mask,gamma_mask,iota_mask,mu_mask,&
                       expcoe_mask,termk_mask,cpcoe_mask,switch_mask,&
                       ic)
               endif
            endif
            
  300       continue
         enddo

!-----------------------------------------------------------------------
! Switching functions: input is of the form 'switch k parameter m ...',
!                      where k is the coefficient number 
!-----------------------------------------------------------------------
      else if (keyword(1).eq.'switch') then
         read(keyword(2),*) k         
         do i=1,ic
            if (keyword(i).eq.'=') ieq=i
         enddo        
         j=1
         atmp=''
         do i=3,ieq-1
            l=len_trim(keyword(i))
            write(atmp(j:j+l),'(a)') keyword(i)(1:len_trim(keyword(i)))
            j=j+l+1
            write(atmp(j:j),'(a)') ' '
         enddo
         do i=1,nswitch
            achk=''
            achk1=''
            write(achk(1:len_trim(atmp)),'(a)') atmp(1:len_trim(atmp))
            write(achk1(1:len_trim(atmp)),'(a)') &
                 aswitch(i)(1:len_trim(atmp))
            if (achk.eq.achk1) then
               if (keyword(ieq+1).eq.'freeze') then
                  if (switch_mask(i,k).eq.1) switch_mask(i,k)=2
               else if (keyword(ieq+1) .eq. 'unfreeze') then
                  if (switch_mask(i,k).eq.2) switch_mask(i,k)=1
               else
                  ncons=ncons+1
                  switch_mask(i,k)=-ncons 
                  call findcons(ieq+1,ncons,const_mask,kappa_mask,&
                       lambda_mask,gamma_mask,iota_mask,mu_mask,&
                       expcoe_mask,termk_mask,cpcoe_mask,&
                       switch_mask,ic)
               endif
            endif
         enddo

      else
         routine = 'Readcons'
         write(message,'(2a)') 'Unknown parameter type: ',keyword(1)
         call vcerrormsg
 
      endif

      go to 20

  999 continue
      if (ncons .gt. maxcons) then
         write(message,'(a,i6)') &
                 'Redimension MAXCONS, Minimum: ',ncons
         call vcerrormsg
      endif

      return
  900 print*,'Error reading Constraints. Last line:'
      print*,inptit
      stop
      end subroutine readcons

!#######################################################################

      subroutine lmmask(dipzero_mask,dipder1_mask,dipder2_mask,&
           dipder3_mask,dipder4_mask,polarzero_mask,polarder1_mask,&
           polarder2_mask,polarder3_mask,polarder4_mask)

        implicit none

        integer(long) :: j,k,l,s,s1,m,n,c,check
        integer(long), dimension(maxdim) :: nmchk,stachk
        integer(long), dimension(3)      :: axchk
        integer(long), dimension(nsta,nsta,3)               :: dipzero_mask
        integer(long), dimension(nsta,nsta,3,nmodes)        :: dipder1_mask
        integer(long), dimension(nsta,nsta,3,nmodes,nmodes) :: dipder2_mask
        integer(long), dimension(nsta,nsta,3,nmodes)        :: dipder3_mask
        integer(long), dimension(nsta,nsta,3,nmodes)        :: dipder4_mask

        integer(long), dimension(nsta,nsta,3,3)               :: polarzero_mask
        integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder1_mask
        integer(long), dimension(nsta,nsta,3,3,nmodes,nmodes) :: polarder2_mask
        integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder3_mask
        integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder4_mask

        check=0
        
        do m=1,nmodes
           call cleanirreps(nmlab(m),len(nmlab))
        enddo
        do m=1,nsta
           call cleanirreps(stalab(m),len(stalab))
        enddo
        do m=1,3
           call cleanirreps(axislab(m),len(axislab))
        enddo
        call getcharacters

!-----------------------------------------------------------------------
! Zero arrays
!-----------------------------------------------------------------------
        do s=1,nsta
           do s1=1,nsta
              do k=1,3
                 dipzero_mask(s,s1,k)=0
                 do l=1,3
                    polarzero_mask(s,s1,k,l)=0
                    do m=1,nmodes
                       dipder1_mask(s,s1,k,m)=0
                       dipder3_mask(s,s1,k,m)=0
                       dipder4_mask(s,s1,k,m)=0
                       polarder1_mask(s,s1,k,l,m)=0
                       polarder3_mask(s,s1,k,l,m)=0
                       polarder4_mask(s,s1,k,l,m)=0
                       do n=1,nmodes
                          dipder2_mask(s,s1,k,m,n)=0
                          polarder2_mask(s,s1,k,l,m,n)=0
                       enddo
                    enddo
                 enddo
              enddo
           enddo
        enddo
        
!-----------------------------------------------------------------------
! Determine mask values via symmetry considerations and user specified
! symmetry labels
!-----------------------------------------------------------------------
        if (ldipfit) then

! First derivatives wrt Q           
           do s=1,nsta
              do s1=1,nsta
                 do k=1,3
                    do m=1,nmodes
                       do c=1,3
                          axchk(c)=0
                       enddo
                       do c=1,maxdim
                          nmchk(c)=0
                          stachk(c)=0
                       enddo
                       stachk(s)=stachk(s)+1
                       stachk(s1)=stachk(s1)+1
                       nmchk(m)=nmchk(m)+1
                       axchk(k)=axchk(k)+1
                       call integralsym(check,nmchk,stachk,axchk)
                       if (check.eq.1) dipder1_mask(s,s1,k,m)=1
                    enddo
                 enddo
              enddo
           enddo

! Second derivatives wrt Q           
           do s=1,nsta
              do s1=1,nsta
                 do k=1,3
                    do m=1,nmodes
                       do n=1,nmodes
                          do c=1,3
                             axchk(c)=0
                          enddo
                          do c=1,maxdim
                             nmchk(c)=0
                             stachk(c)=0
                          enddo
                          stachk(s)=stachk(s)+1
                          stachk(s1)=stachk(s1)+1
                          nmchk(m)=nmchk(m)+1
                          nmchk(n)=nmchk(n)+1
                          axchk(k)=axchk(k)+1
                          call integralsym(check,nmchk,stachk,axchk)
                          if (check.eq.1) dipder2_mask(s,s1,k,m,n)=1
                       enddo
                    enddo
                 enddo
              enddo
           enddo

! Third derivatives wrt Q (cubic terms only)
           do s=1,nsta
              do s1=1,nsta
                 do k=1,3
                    do m=1,nmodes
                       do c=1,3
                          axchk(c)=0
                       enddo
                       do c=1,maxdim
                          nmchk(c)=0
                          stachk(c)=0
                       enddo
                       stachk(s)=stachk(s)+1
                       stachk(s1)=stachk(s1)+1
                       nmchk(m)=nmchk(m)+3
                       axchk(k)=axchk(k)+1
                       call integralsym(check,nmchk,stachk,axchk)
                       if (check.eq.1) dipder3_mask(s,s1,k,m)=1
                    enddo
                 enddo
              enddo
           enddo
           
! Fourth derivatives wrt Q (quartic terms only)
           do s=1,nsta
              do s1=1,nsta
                 do k=1,3
                    do m=1,nmodes
                       do c=1,3
                          axchk(c)=0
                       enddo
                       do c=1,maxdim
                          nmchk(c)=0
                          stachk(c)=0
                       enddo
                       stachk(s)=stachk(s)+1
                       stachk(s1)=stachk(s1)+1
                       nmchk(m)=nmchk(m)+4
                       axchk(k)=axchk(k)+1
                       call integralsym(check,nmchk,stachk,axchk)
                       if (check.eq.1) dipder4_mask(s,s1,k,m)=1
                    enddo
                 enddo
              enddo
           enddo
           
        endif

        if (lpolarfit) then
! First derivatives wrt Q           
           do s=1,nsta
              do s1=1,nsta
                 do k=1,3
                    do l=1,3
                       do m=1,nmodes
                          do c=1,3
                             axchk(c)=0
                          enddo
                          do c=1,maxdim
                             nmchk(c)=0
                             stachk(c)=0
                          enddo
                          stachk(s)=stachk(s)+1
                          stachk(s1)=stachk(s1)+1
                          nmchk(m)=nmchk(m)+1
                          axchk(k)=axchk(k)+1
                          axchk(l)=axchk(l)+1
                          call integralsym(check,nmchk,stachk,axchk)
                       if (check.eq.1) polarder1_mask(s,s1,k,l,m)=1
                    enddo
                 enddo
              enddo
           enddo
        enddo

! Second derivatives wrt Q           
        do s=1,nsta
           do s1=1,nsta
              do k=1,3
                 do l=1,3
                    do m=1,nmodes
                       do n=1,nmodes
                          do c=1,3
                             axchk(c)=0
                          enddo
                          do c=1,maxdim
                             nmchk(c)=0
                             stachk(c)=0
                          enddo
                          stachk(s)=stachk(s)+1
                          stachk(s1)=stachk(s1)+1
                          nmchk(m)=nmchk(m)+1
                          nmchk(n)=nmchk(n)+1
                          axchk(k)=axchk(k)+1
                          axchk(l)=axchk(l)+1
                          call integralsym(check,nmchk,stachk,axchk)
                          if (check.eq.1) polarder2_mask(s,s1,k,l,m,n)=1
                       enddo
                    enddo
                 enddo
              enddo
           enddo
        enddo
        
! Third derivatives wrt Q (cubic terms only)
           do s=1,nsta
              do s1=1,nsta
                 do k=1,3
                    do l=1,3
                       do m=1,nmodes
                          do c=1,3
                             axchk(c)=0
                          enddo
                          do c=1,maxdim
                             nmchk(c)=0
                             stachk(c)=0
                          enddo
                          stachk(s)=stachk(s)+1
                          stachk(s1)=stachk(s1)+1
                          nmchk(m)=nmchk(m)+3
                          axchk(k)=axchk(k)+1
                          axchk(l)=axchk(l)+1
                          call integralsym(check,nmchk,stachk,axchk)
                          if (check.eq.1) polarder3_mask(s,s1,k,l,m)=1
                       enddo
                    enddo
                 enddo
              enddo
           enddo

! Fourth derivatives wrt Q (quartic terms only)
           do s=1,nsta
              do s1=1,nsta
                 do k=1,3
                    do l=1,3
                       do m=1,nmodes
                          do c=1,3
                             axchk(c)=0
                          enddo
                          do c=1,maxdim
                             nmchk(c)=0
                             stachk(c)=0
                          enddo
                          stachk(s)=stachk(s)+1
                          stachk(s1)=stachk(s1)+1
                          nmchk(m)=nmchk(m)+4
                          axchk(k)=axchk(k)+1
                          axchk(l)=axchk(l)+1
                          call integralsym(check,nmchk,stachk,axchk)
                          if (check.eq.1) polarder4_mask(s,s1,k,l,m)=1
                       enddo
                    enddo
                 enddo
              enddo
           enddo
        endif


        return
      end subroutine lmmask

!#######################################################################

      subroutine findcons(ik,ncons1,const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask,ic)

      implicit none

      integer(long)        :: type,m,m1,s,s1,k,icons,i,l,schk,s2chk,&
                              ilbl,jlbl,slen,p,j,ierr,ic1
      integer(long), intent(in)                                     :: ik,ncons1,ic
      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in)                    :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe) :: switch_mask

      real(dop)       :: fact
      
      integer(long), dimension(nmodes) :: defchk,chk
      integer(long) :: attermk

      character(len=80) :: atmp,achk,achk1

! get parameter for constraint.
      m=0
      m1=0
      s=0
      s1=0
      k=0
      fact=1.0
      if (keyword(ik) .eq. 'e') then
         if (ic .lt. ik+2) go to 990
         type=0
         read(keyword(ik+1),*,err=999) s
         read(keyword(ik+2),*,err=999) s1
         if (ic .ge. ik+3) read(keyword(ik+2),*) fact
      else if (keyword(ik) .eq. 'kappa') then
         if (ic .lt. ik+2) go to 990
         type=1
         read(keyword(ik+1),*,err=999) m
         read(keyword(ik+2),*,err=999) s
         if (ic .ge. ik+3) read(keyword(ik+3),*,err=999) fact
      else if (keyword(ik) .eq. 'lambda') then
         if (ic .lt. ik+3) go to 990
         type=2
         read(keyword(ik+1),*,err=999) m
         read(keyword(ik+2),*,err=999) s
         read(keyword(ik+3),*,err=999) s1
         if (ic .ge. ik+4) read(keyword(ik+4),*,err=999) fact
      else if (keyword(ik) .eq. 'gamma') then
         if (ic .lt. ik+3) go to 990
         type=3
         read(keyword(ik+1),*,err=999) m
         read(keyword(ik+2),*,err=999) m1
         read(keyword(ik+3),*,err=999) s 
         if (ic .ge. ik+4) read(keyword(ik+4),*,err=999) fact
      else if (keyword(ik) .eq. 'iota') then
         if (ic .lt. ik+3) go to 990
         type=4
         read(keyword(ik+1),*,err=999) m
         read(keyword(ik+2),*,err=999) m1
         read(keyword(ik+3),*,err=999) s
         if (ic .ge. ik+4) read(keyword(ik+4),*,err=999) fact
      else if (keyword(ik) .eq. 'mu') then
         if (ic .lt. ik+4) go to 990
         type=5
         read(keyword(ik+1),*,err=999) m
         read(keyword(ik+2),*,err=999) m1
         read(keyword(ik+3),*,err=999) s 
         read(keyword(ik+4),*,err=999) s1
         if (ic .ge. ik+5) read(keyword(ik+5),*,err=999) fact
      else if (keyword(ik) .eq. 'general') then
         if (ic .lt. ik+3) go to 990
         type=6
         read(keyword(ik+1),*,err=999) k
         read(keyword(ik+2),*,err=999) m
         read(keyword(ik+3),*,err=999) s
         if (ic .ge. ik+4) read(keyword(ik+4),*,err=999) fact
!         print*, 'READ k m s fact = ', k, m , s, fact 
      else if (keyword(ik) .eq. 'iota2') then
         if (ic .lt. ik+3) go to 990
         type=7
         read(keyword(ik+1),*,err=999) m
         read(keyword(ik+2),*,err=999) m1
         read(keyword(ik+3),*,err=999) s
         read(keyword(ik+4),*,err=999) s1
         if (ic .ge. ik+5) read(keyword(ik+5),*,err=999) fact

      else if (keyword(ik) .eq. 'explicit') then
         type = 8
! check if  factor given and read
         read(keyword(ic),*,iostat=ierr) fact
         if (ierr .eq. 0) then
            ic1 = ic-1
         else
            fact =1.0
            ic1 = ic
         endif
! determine termk number
         do i=1,nmodes
            defchk(i)=0.0
         enddo
         read(keyword(ik+1),*,err=999) schk
         read(keyword(ik+2),*,err=999) s2chk
         m = 0
         do l=ik+3,ic1
            ilbl=index(keyword(l),'^')
            jlbl=slen(keyword(l))
!            print*, 'keyword =', keyword(l)
            if (ilbl .eq. 0) then
              p = 1
              read(keyword(l),*,err=999) m
            else
               read(keyword(l)(1:ilbl-1),*,err=999) m
               read(keyword(l)(ilbl+1:jlbl),*,err=999) p
            endif
            defchk(m) = p
         enddo
         do i=1,nadd
            do j=1,nmodes
               chk(j)=1
            enddo
            if (schk.eq.termst(1,i).and.s2chk.eq.termst(2,i).or.    &
                 (schk.eq.termst(2,i)).and.s2chk.eq.termst(1,i)) then
               do j=1,nmodes
                  chk(j)=0
               enddo
               do j=1,nmodes
                  chk(j)=defchk(j)-termdef(j,i)
                  if (chk(j).ne.0) go to 300
               enddo
               attermk=i
            endif
300         continue
         enddo

      else if (keyword(ik) .eq. 'cpgeneral') then
         type = 9
         read(keyword(ik+1),*,err=999) k
         read(keyword(ik+2),*,err=999) m
         read(keyword(ik+3),*,err=999) s
         read(keyword(ik+4),*,err=999) s1
         if (ic .ge. ik+5) read(keyword(ik+5),*,err=999) fact
      
      else if (keyword(ik) .eq. 'switch') then
         type = 11
! Coefficient number
         read(keyword(ik+1),*,err=999) k
! Function number
         j=1
         atmp=''
         do i=ik+2,ic-1
            l=len_trim(keyword(i))
            write(atmp(j:j+l),'(a)') keyword(i)(1:len_trim(keyword(i)))
            j=j+l+1
            write(atmp(j:j),'(a)') ' '
         enddo         
         do i=1,nswitch
            achk=''
            achk1=''
            write(achk(1:len_trim(atmp)),'(a)') atmp(1:len_trim(atmp))
            write(achk1(1:len_trim(atmp)),'(a)') &
                 aswitch(i)(1:len_trim(atmp))
            if (achk.eq.achk1) attermk=i
         enddo
         read(keyword(ic),*,err=999) fact         
         
      else
         routine = 'Findcons'
         write(message,'(2a)') 'Unknown constraint type: ',keyword(ik)
         call vcerrormsg
      endif

      call parnumt(icons,type,m,s,m1,s1,k,attermk,const_mask,kappa_mask,&
              lambda_mask,gamma_mask,iota_mask,mu_mask,expcoe_mask,&
              termk_mask,cpcoe_mask,switch_mask)
        
      if (ncons1 .le. maxcons) then
         parcons(ncons1)=icons
         pconfact(ncons1)=fact
!         print*, 'parcons(ncons1) = icons =', icons, 'ncons1 =', ncons1
!         print*, 'parconfact(ncons1) = fact =', fact
      endif

      return
  990 continue
      routine='Findcons'
      write(message,'(a,a10,a10,a)') &
           'Not enough arguments for constraint: ',keyword(ik),&
           keyword(ik+1),'....'
      call vcerrormsg
  999 continue
      routine='Findcons'
      write(message,'(a,a10,a10,a)') 'Error reading constraints: ',&
           keyword(ik),keyword(ik+1),'....'
      call vcerrormsg
      end subroutine findcons

!#######################################################################

      subroutine delmodes(const_mask,kappa_mask,lambda_mask, &
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask)

      implicit none

      integer(long) :: m,n,s,i,j,k,l,s1
      integer(long), dimension(nmodes) :: defchk
      integer(long), dimension(nsta,nsta), intent(inout) :: const_mask
      integer(long), dimension(nmodes,nsta), intent(inout) :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout) :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout) :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout) :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: &
                                                           iota_mask,mu_mask
      integer(long), dimension(nadd), intent(inout) :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(inout) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe) :: switch_mask

! if modes not to be included in fit, set masks to 2 
      do m=1,nmodes
         if (mfit(m) .eq. 0) then
            do s=1,nsta
               if (kappa_mask(m,s) .eq. 1) kappa_mask(m,s)=2
            enddo
            do i=1,nsta-1
               do k=i+1,nsta
                  if (lambda_mask(m,i,k).eq.1) then
                     lambda_mask(m,i,k)=2
                     lambda_mask(m,k,i)=2
                  endif
               enddo
            enddo
            do i=1,nsta
               do k=1,nmodes
                  if (gamma_mask(k,m,i).eq.1) then
                     gamma_mask(k,m,i)=2
                     gamma_mask(m,k,i)=2
                  endif
               enddo
            enddo
            do i=1,nsta
               do k=1,nmodes
                  if (iota_mask(k,m,i,i).eq.1) then
                     iota_mask(k,m,i,i)=2
                     iota_mask(m,k,i,i)=2
                  endif
               enddo
            enddo
            do i=1,nsta-1
               do l=i+1,nsta
                  do k=1,nmodes
                     if (mu_mask(k,m,i,l).eq.1) then
                        mu_mask(k,m,i,l)=2
                        mu_mask(m,k,i,l)=2
                        mu_mask(k,m,l,i)=2
                        mu_mask(m,k,l,i)=2
                     endif
                  enddo
               enddo
            enddo
            do i=1,nsta
               do k=1,ncoeff(m,i)
                  if (expcoe_mask(k,m,i).eq.1) expcoe_mask(k,m,i)=2
               enddo
            enddo
            do i=1,nsta-1
                do l=i+1,nsta
                   do k=1,nmodes
                     if (iota_mask(k,m,i,l).eq.1) then
                        iota_mask(k,m,i,l)=2
                        iota_mask(m,k,i,l)=2
                        iota_mask(k,m,l,i)=2
                        iota_mask(m,k,l,i)=2
                     endif
                   enddo
                enddo
             enddo
             do j=1,nsta
                do i=1,nsta 
                   do k=1,ncoeffcp(m,i,j)
                      if (cpcoe_mask(k,m,i,j).eq.1) cpcoe_mask(k,m,i,j)=2
                   enddo
                enddo
             enddo          
          endif
       enddo


! Explicitly added terms
      do i=1,nadd
         do m=1,nmodes
            if (termdef(m,i).ne.0) then
               if (mfit(m).eq.0) then
                 if (termk_mask(i).eq.1) termk_mask(i)=2
               endif
            endif
         enddo
      enddo

! Switching functions
      do i=1,nswitch
         if (swpar(i).ne.7) then
            if ((swmd(i).ne.0.and.mfit(swmd(i)).eq.0)&
                 .or.(swm(i).ne.0.and.mfit(swm(i)).eq.0)&
                 .or.(swm1(i).ne.0.and.mfit(swm1(i)).eq.0)) then
               do j=1,nswitchcoe(i)
                  if (switch_mask(i,j).eq.1) switch_mask(i,j)=2
               enddo
            endif
         else
            do m=1,nmodes
               if (termdef(m,swk(i)).ne.0) then
                  if (mfit(m).eq.0) then
                     do j=1,nswitchcoe(i)
                        if (switch_mask(i,j).eq.1) switch_mask(i,j)=2
                     enddo
                  endif
               endif
            enddo
         endif
      enddo
      
      return
      end subroutine delmodes

!#######################################################################

      subroutine delstates(const_mask,kappa_mask,lambda_mask, &
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask)

      implicit none

      integer(long) :: m,s,i,j,k,l
      integer(long), dimension(nsta,nsta), intent(inout) :: const_mask
      integer(long), dimension(nmodes,nsta), intent(inout) :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout) :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout) :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout) :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: &
                                                           iota_mask,mu_mask
      integer(long), dimension(nadd), intent(inout) :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(inout) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe) :: switch_mask


! if mode but not state is to be included in fit, sets mask to 2

      do m=1,nmodes
         if (mfit(m) .eq. 1) then
            do s=1,nsta
               if (kappa_mask(m,s) .eq. 1) then
                  if (sfit(s) .eq. 0) kappa_mask(m,s)=2
               endif
            enddo
            do i=1,nsta-1
               do k=i+1,nsta
                  if (lambda_mask(m,i,k).eq.1) then
                     if ((sfit(i).eq.0).or.(sfit(k).eq.0)) then
                        lambda_mask(m,i,k)=2
                        lambda_mask(m,k,i)=2
                     endif
                  endif
               enddo
            enddo
            do i=1,nsta
               do k=1,nmodes
                  if (gamma_mask(k,m,i).eq.1) then
                     if (sfit(i).eq.0) then
                        gamma_mask(k,m,i)=2
                        gamma_mask(m,k,i)=2
                     endif
                  endif
               enddo
            enddo
            do i=1,nsta
               do k=1,nmodes
                  if (iota_mask(k,m,i,i).eq.1) then
                     if (sfit(i).eq.0) then
                        iota_mask(k,m,i,i)=2
                        iota_mask(m,k,i,i)=2
                     endif
                  endif
               enddo
            enddo
            do i=1,nsta-1
               do l=i+1,nsta
                  do k=1,nmodes
                     if (mu_mask(k,m,i,l).eq.1) then
                        if ((sfit(i).eq.0).or.(sfit(k).eq.0)) then
                           mu_mask(k,m,i,l)=2
                           mu_mask(m,k,i,l)=2
                           mu_mask(k,m,l,i)=2
                           mu_mask(m,k,l,i)=2
                        endif
                     endif
                  enddo
               enddo
            enddo
            do i=1,nsta
               do k=1,ncoeff(m,i)
                  if (expcoe_mask(k,m,i).eq.1) then
                     if (sfit(i).eq.0) then
                        expcoe_mask(k,m,i)=2
                     endif
                  endif
               enddo
            enddo
            do i=1,nsta-1
               do l=i+1,nsta
                  do k=1,nmodes
                     if (iota_mask(k,m,i,l).eq.1) then
                        if ((sfit(i).eq.0).or.(sfit(k).eq.0)) then
                           iota_mask(k,m,i,l)=2
                           iota_mask(m,k,i,l)=2
                           iota_mask(k,m,l,i)=2
                           iota_mask(m,k,l,i)=2
                        endif
                     endif
                  enddo
               enddo
            enddo
            do j=1,nsta
               do i=1,nsta
                  do k=1,ncoeffcp(m,i,j)
                     if (cpcoe_mask(k,m,i,j).eq.1) then
                        if (sfit(i).eq.0.or.sfit(j).eq.0) then
                           cpcoe_mask(k,m,i,j)=2
                        endif
                     endif
                  enddo
               enddo
            enddo
         endif
      enddo



      do i=1,nadd
         do j=1,2
            if (sfit(termst(j,i)).eq.0) termk_mask(i)=2
         enddo
      enddo
      
! Switching functions      
      do i=1,nswitch
         if (swpar(i).ne.7) then
            if ((sws(i).ne.0.and.sfit(sws(i)).eq.0) &
                 .or.(sws1(i).ne.0.and.sfit(sws1(i)).eq.0)) then
               do j=1,nswitchcoe(i)
                  if (switch_mask(i,j).eq.1) switch_mask(i,j)=2
               enddo
            endif
         else
            do k=1,2
               if (sfit(termst(k,swk(i))).eq.0) then
                  do j=1,nswitchcoe(i)
                     if (switch_mask(i,j).eq.1) switch_mask(i,j)=2
                  enddo
               endif
            enddo
         endif
      enddo

      return
      end subroutine delstates
      
!#######################################################################

      subroutine renumcons(const_mask,kappa_mask,lambda_mask,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
                 cpcoe_mask,switch_mask)

      implicit none

      integer(long)       :: i,md,md1,st,st1,type,koe,n,n1,attermk

      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in)                    :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe)                      :: switch_mask

      write (ilog,'(a,i5)') 'No. of Constraints set :',ncons
      do i=1,ncons
         n=parcons(i)
!        print*, 'con = ', i, 'ncons = ', ncons, 'parcons = ', n
         call numtpar(n,type,md,st,md1,st1,koe,attermk,const_mask,kappa_mask,&
              lambda_mask,gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
              cpcoe_mask,switch_mask)

! get new number
! if constraint is to an active parameter, take number from active
! parameter list, else flag with (negative) number from complete 
! parameter list
         n1=-1
         call parnum(n1,type,md,st,md1,st1,koe,attermk,const_mask,kappa_mask,&
              lambda_mask,gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
              cpcoe_mask,switch_mask)

         if (n1 .gt. 0) then
            parcons(i)=n1
          else
            parcons(i)=-n
         endif
      enddo

      return
      end subroutine renumcons

!#######################################################################

      subroutine delmodes_lm(dipder1_mask,dipder2_mask,dipder3_mask,&
           dipder4_mask,polarder1_mask,polarder2_mask,polarder3_mask,&
           polarder4_mask)

        implicit none

        integer(long) :: k,j,s,s1,m,n
        integer(long), dimension(nsta,nsta,3,nmodes) :: dipder1_mask
        integer(long), dimension(nsta,nsta,3,nmodes,nmodes) :: dipder2_mask
        integer(long), dimension(nsta,nsta,3,nmodes) :: dipder3_mask
        integer(long), dimension(nsta,nsta,3,nmodes) :: dipder4_mask

        integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder1_mask
        integer(long), dimension(nsta,nsta,3,3,nmodes,nmodes) :: polarder2_mask
        integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder3_mask
        integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder4_mask

        if (ldipfit) then
           do m=1,nmodes
              if (mfit(m).eq.0) then
                 do s=1,nsta
                    do s1=s,nsta
                       do k=1,3
                          if (dipder1_mask(s,s1,k,m).eq.1) then
                             dipder1_mask(s,s1,k,m)=2
                             dipder1_mask(s1,s,k,m)=2
                          endif
                          if (dipder3_mask(s,s1,k,m).eq.1) then
                             dipder3_mask(s,s1,k,m)=2
                             dipder3_mask(s1,s,k,m)=2
                          endif
                          if (dipder4_mask(s,s1,k,m).eq.1) then
                             dipder4_mask(s,s1,k,m)=2
                             dipder4_mask(s1,s,k,m)=2
                          endif
                          do n=1,nmodes
                             if (dipder2_mask(s,s1,k,m,n).eq.1) then
                                dipder2_mask(s,s1,k,m,n)=2
                                dipder2_mask(s1,s,k,m,n)=2
                                dipder2_mask(s1,s,k,n,m)=2
                                dipder2_mask(s,s1,k,n,m)=2
                             endif
                          enddo
                       enddo
                    enddo
                 enddo
              endif
           enddo
        endif

        if (lpolarfit) then
           do m=1,nmodes
              if (mfit(m).eq.0) then
                 do s=1,nsta
                    do s1=s,nsta
                       do k=1,3
                          do j=1,3
                             if (polarder1_mask(s,s1,k,j,m).eq.1) then
                                polarder1_mask(s,s1,k,j,m)=2
                                polarder1_mask(s1,s,k,j,m)=2
                             endif
                             if (polarder3_mask(s,s1,k,j,m).eq.1) then
                                polarder3_mask(s,s1,k,j,m)=2
                                polarder3_mask(s1,s,k,j,m)=2
                             endif
                             if (polarder4_mask(s,s1,k,j,m).eq.1) then
                                polarder4_mask(s,s1,k,j,m)=2
                                polarder4_mask(s1,s,k,j,m)=2
                             endif
                             do n=1,nmodes
                                if (polarder2_mask(s,s1,k,j,m,n).eq.1) then
                                   polarder2_mask(s,s1,k,j,m,n)=2
                                   polarder2_mask(s1,s,k,j,m,n)=2
                                   polarder2_mask(s1,s,k,j,n,m)=2
                                   polarder2_mask(s,s1,k,j,n,m)=2
                                endif
                             enddo
                          enddo
                       enddo
                    enddo
                 enddo
              endif
           enddo
           endif


        return
      end subroutine delmodes_lm

!#######################################################################
      
      subroutine rdexplicit
        
        implicit none
        
        integer(long), external          :: slen
        integer(long)                    :: iz,ic,ierr,chk,i,m,p,ilbl,jlbl,&
                                            iz0
        integer(long), dimension(maxdim) :: tmdef
        integer(long), dimension(2)      :: tmst
        logical(kind=4)                  :: found

        rewind(iin)
        chk = 0
        iz0 = 0

!-----------------------------------------------------------------------
! Read to addpar section
!-----------------------------------------------------------------------
10      continue
        call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
        routine = 'addparams (read VCHFIT input)'
        if( ierr .eq. 1) go to 999

        if (keyword(1).ne.'addpar-section'.and.&
             keyword(1).ne.'add_parameters-section') go to 10

!-----------------------------------------------------------------------
! read explicitly-added terms
!-----------------------------------------------------------------------
20      continue
        call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
        routine = 'addparams (read VCHFIT input)'
        call vccntrl(ierr)

        if (keyword(1) .eq. 'end-addpar-section' .or. &
             keyword(1) .eq. 'end-add_parameters-section') then
           go to 999

        else if ((keyword(1) .eq. 'explicit') .or. (keyword(1) .eq. &
             'Explicit')) then

           call zerovxi(tmdef,maxdim)
           call zerovxi(tmst,2)
           found = .false.
           read(keyword(2),*,iostat=ierr) tmst(1)
           read(keyword(3),*,iostat=ierr) tmst(2)
           i = 4
           do i=4,ic
              if(index(keyword(i),'=') .eq. 1) EXIT
              ilbl=index(keyword(i),'^')
              jlbl=slen(keyword(i))
              if (ilbl .eq. 0) then
                 p = 1
                 read(keyword(i),*,iostat=ierr) m
              else
                 read(keyword(i)(1:ilbl-1),*,iostat=ierr) m
                 read(keyword(i)(ilbl+1:jlbl),*,iostat=ierr) p
              endif
              tmdef(m) = p
           enddo

           if (ierr .ne. 0) then
              routine = 'rdexplicit'
              message='Error reading explicit term definition'
              call vcwrline(i,iz,iz0)
           endif
!
! see if term already present, if not add
!
           call chkexplicit(tmdef,tmst,termdef,termst,nadd,found,nmodes)

           if (.not. found) then
              chk=chk+1
              if (chk .gt. nadd) go to 20
              termst(1,chk) = tmst(1)
              termst(2,chk) = tmst(2)
              do m=1,nmodes
                 termdef(m,chk) = tmdef(m)
              enddo
           endif

        endif
        goto 20

999     continue

        return
      end subroutine rdexplicit


!#######################################################################

      subroutine getnadd(unit)

      implicit none

      integer(long), intent(in) :: unit
      character(len=80)         :: string
      
      integer(long), external   :: slen
      integer(long)             :: m,ilbl,jlbl,iz,ic,ierr,n,p,j,ioff

10    continue
      read(unit,'(a)',end=100,err=1010) string
 
      if (string .eq.&
           '*** Explicitely added polynomial terms ***') then
         call rdinpf(unit,keyword,keyorig,inptit,lc,ic,iz,ierr,&
              maxkey)  
         if (keyword(1) .eq. "n") then                           
            ioff=1                   ! ioff is offset for format with / without N
            read(keyword(2),*) nadd
         else
            ioff=0
            read(keyword(1),*) nadd
         endif
      else
         goto 10
      endif
         
100   continue

      rewind(unit)
30    continue
      read(unit,'(a)',end=300,err=1010) string
      if (string.ne. &
           '*** Explicitely added polynomial terms ***') goto 30       
      read(unit,*)
      do n=1,nadd
         call rdinpf(unit,keyword,keyorig,inptit,lc,ic,iz,ierr,&
              maxkey)
         read(keyword(ioff+2),*) termst(1,n)
         read(keyword(ioff+3),*) termst(2,n)
         call zerovxi(termdef(1,n),maxdim)
         do j=ioff+4,ic
            ilbl=index(keyword(j),'^')
            jlbl=slen(keyword(j))
            read(keyword(j)(1:ilbl-1),*) m
            read(keyword(j)(ilbl+1:jlbl),*) p
            termdef(m,n) = p
         enddo
      enddo
300   continue

      return
1010  stop 'ERROR IN GUESS FILE'
      end subroutine getnadd

!#######################################################################

      subroutine getnswitch(unit)

        implicit none

        integer(long), intent(in) :: unit
        character(len=80)         :: string

        rewind(unit)
10      continue
        read(unit,'(a)',end=100,err=1010) string        
        if (string.eq.&
             '*** Switching Functions ***') then
           read(unit,*) nswitch
        else
           goto 10
        endif
        
100     continue

        return

1010    stop 'ERROR IN GUESS FILE'

      end subroutine getnswitch

!#######################################################################

      subroutine setjt4cons(const_mask,kappa_mask,lambda_mask,&
           gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
           cpcoe_mask,switch_mask,expcoe)
        
      implicit none
      
      integer(long) :: m,m1,s,s1,type,k,icons,attermk
      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout)   :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)   :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in)                    :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(inout) :: cpcoe_mask
      real(dop), dimension(maxcoe,nmodes,nsta), intent(inout)       :: expcoe
      integer(long), dimension(nswitch,maxcoe) :: switch_mask
      
      do s1=1,nsta-1
         do s=s1+1,nsta
            do m1=1,nmodes
               do m=1,nmodes
                  if (jt4cons(m,m1,s,s1).eq.1) then
! kappa constraints                     
                     type=6
                     k=1
                     call parnumt(icons,type,m,s,m1,s1,k,&
                          attermk,const_mask,kappa_mask,&
                          lambda_mask,gamma_mask,iota_mask,&
                          mu_mask,expcoe_mask,termk_mask,&
                          cpcoe_mask,switch_mask)
                     ncons=ncons+1
                     expcoe_mask(1,m,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=-1.0
                     endif
                     
                     expcoe_mask(1,m1,s)=2
                     expcoe(1,m1,s)=0.0

                     expcoe_mask(1,m1,s1)=2
                     expcoe(1,m1,s1)=0.0
                     
                     ncons=ncons+1
                     cpcoe_mask(1,m1,s,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=+1.0
                     endif
                     cpcoe_mask(1,m1,s1,s)=cpcoe_mask(1,m1,s,s1)
                     
! omega constraints
                     type=6
                     k=2
                     call parnumt(icons,type,m,s,m1,s1,k,&
                          attermk,const_mask,kappa_mask,&
                          lambda_mask,gamma_mask,iota_mask,&
                          mu_mask,expcoe_mask,termk_mask,&
                          cpcoe_mask,switch_mask)
                     
                     ncons=ncons+1
                     expcoe_mask(2,m,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=+1.0
                     endif
                     
                     ncons=ncons+1
                     expcoe_mask(2,m1,s)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=+1.0
                     endif

                     ncons=ncons+1
                     expcoe_mask(2,m1,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=+1.0
                     endif
  
! gamma constraints
                     type=6
                     k=3
                     call parnumt(icons,type,m,s,m1,s1,k,&
                          attermk,const_mask,kappa_mask,&
                          lambda_mask,gamma_mask,iota_mask,&
                          mu_mask,expcoe_mask,termk_mask,&
                          cpcoe_mask,switch_mask)

                     ncons=ncons+1
                     expcoe_mask(3,m,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=-1.0
                     endif

                     ncons=ncons+1
                     expcoe_mask(3,m1,s)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=-1.0
                     endif

                     ncons=ncons+1
                     expcoe_mask(3,m1,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=+1.0
                     endif

! zeta constraints
                     type=6
                     k=4
                     call parnumt(icons,type,m,s,m1,s1,k,&
                          attermk,const_mask,kappa_mask,&
                          lambda_mask,gamma_mask,iota_mask,&
                          mu_mask,expcoe_mask,termk_mask,&
                          cpcoe_mask,switch_mask)

                     ncons=ncons+1
                     expcoe_mask(4,m,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=+1.0
                     endif

                     expcoe_mask(4,m1,s)=2
                     expcoe(4,m1,s)=0.0
                     
                     expcoe_mask(4,m1,s1)=2
                     expcoe(4,m1,s1)=0.0

! iota constraints
                     type=6
                     k=5
                     call parnumt(icons,type,m,s,m1,s1,k,&
                          attermk,const_mask,kappa_mask,&
                          lambda_mask,gamma_mask,iota_mask,&
                          mu_mask,expcoe_mask,termk_mask,&
                          cpcoe_mask,switch_mask)

                     ncons=ncons+1
                     expcoe_mask(5,m,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=-1.0
                     endif

                     expcoe_mask(5,m1,s)=2
                     expcoe(5,m1,s)=0.0
                     
                     expcoe_mask(5,m1,s1)=2
                     expcoe(5,m1,s1)=0.0

                     ncons=ncons+1
                     cpcoe_mask(2,m1,s,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=+1.0
                     endif
                     cpcoe_mask(2,m1,s1,s)=cpcoe_mask(2,m1,s,s1)

! tau constraints
                     type=6
                     k=6
                     call parnumt(icons,type,m,s,m1,s1,k,&
                          attermk,const_mask,kappa_mask,&
                          lambda_mask,gamma_mask,iota_mask,&
                          mu_mask,expcoe_mask,termk_mask,&
                          cpcoe_mask,switch_mask)
                     
                     ncons=ncons+1
                     expcoe_mask(6,m,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=+1.0
                     endif

                     ncons=ncons+1
                     expcoe_mask(6,m1,s)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=+1.0
                     endif

                     ncons=ncons+1
                     expcoe_mask(6,m1,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=+1.0
                     endif

! alpha constraints
                     type=6
                     k=7
                     call parnumt(icons,type,m,s,m1,s1,k,&
                          attermk,const_mask,kappa_mask,&
                          lambda_mask,gamma_mask,iota_mask,&
                          mu_mask,expcoe_mask,termk_mask,&
                          cpcoe_mask,switch_mask)
                     
                     ncons=ncons+1
                     expcoe_mask(7,m,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=-1.0
                     endif

                     ncons=ncons+1
                     expcoe_mask(7,m1,s)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=+1.0
                     endif

                     ncons=ncons+1
                     expcoe_mask(7,m1,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=-1.0
                     endif

! beta constraints
                     type=6
                     k=8
                     call parnumt(icons,type,m,s,m1,s1,k,&
                          attermk,const_mask,kappa_mask,&
                          lambda_mask,gamma_mask,iota_mask,&
                          mu_mask,expcoe_mask,termk_mask,&
                          cpcoe_mask,switch_mask)

                     ncons=ncons+1
                     expcoe_mask(8,m,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=-1.0
                     endif

                     ncons=ncons+1
                     expcoe_mask(8,m1,s)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=-1.0
                     endif

                     ncons=ncons+1
                     expcoe_mask(8,m1,s1)=-ncons
                     if(ncons.le.maxcons) then
                        parcons(ncons)=icons
                        pconfact(ncons)=+1.0
                     endif

                  endif
               enddo
            enddo
         enddo
      enddo

        return
      end subroutine setjt4cons

!#######################################################################

      end module symmetry
