program vcextract

  use decimal, only: long,dop,sip
  use vcglobal
use global, only:pi
  use maxv, only: maxdim,maxsta,maxset
  use vchammod
  use vchfitmod
  use versions  
  use op1lib, only: zerovxd, zerovxi
  use iofiles, only: readguess
  use symmetry, only: getnadd
  use utils, only: vcerrormsg,vchdefault
  use module_interface
  use invchammod
  use timing

  implicit none

  integer(long)       :: ilbl,modus,ierr,k,l,s,wtyp
  character(len=10)   :: cerr,arg
  real(dop), dimension(:), allocatable        :: optcoo
  complex(dop), dimension(:), allocatable     :: mc
  real(dop), dimension(:), allocatable        :: mr
  integer(long), dimension(:), allocatable    :: mi
  logical(kind=4), dimension(:), allocatable  :: ml
  complex(sip), dimension(:), allocatable     :: ms
  real(sip), dimension(:), allocatable        :: mf
  integer(long)                               :: c, i, j,x,y,z
  character(len=80)                           :: filename,aout
  logical(kind=4)                             :: lweight,gaout
  integer(long), external                     :: slen

  ierr=0
  gaout = .false.
!-----------------------------------------------------------------------
! read input
! define output file etc.
!-----------------------------------------------------------------------
  call vcexinp(filename,aout,lweight,wtyp,gaout)

!-----------------------------------------------------------------------
! Open files
!-----------------------------------------------------------------------
  open(iguess,file=filename,form='formatted',status='old',&
       err=1000)

  open(ilog,file='vcextract.log',form='formatted',status='unknown')

  if (aout.eq.'') then
     iout=6
  else
     open(iout,file=aout,form='formatted',status='unknown')
  endif

!-----------------------------------------------------------------------
! determine the no. modes and states
!-----------------------------------------------------------------------
  call getinfo(filename,iguess,gaout)
  maxdim=nmodes
  maxsta=nsta

  call alloc_vchfitmod
  call alloc_vchammod

  call vchdefault

!-----------------------------------------------------------------------
! assign memory for arrays
!-----------------------------------------------------------------------
  call memvcham

!-----------------------------------------------------------------------
! Allocate memory
!-----------------------------------------------------------------------
  ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim

  if(allocated(mc))deallocate(mc)
  if(allocated(mr))deallocate(mr)
  if(allocated(mi))deallocate(mi)
  if(allocated(ml))deallocate(ml)
  if(allocated(ms))deallocate(ms)
  if(allocated(mf))deallocate(mf)

  cerr = 'vcextract_mc '
  allocate(mc(mcdim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'vcextract_mr '
  allocate(mr(mrdim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'vcextract_mi '
  allocate(mi(midim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'vcextract_ml '
  allocate(ml(mldim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'vcextract_ms '
  allocate(ms(msdim),stat=ierr)
  if (ierr > 0) go to 2000
  cerr = 'vcextract_mf '
  allocate(mf(mfdim),stat=ierr)
  if (ierr > 0) go to 2000

  call zerovxd(mr,mrdim)
  call zerovxi(mi,midim)

!-----------------------------------------------------------------------
! determine the no. explicitely added terms
!-----------------------------------------------------------------------
      call getnadd(iguess)
!-----------------------------------------------------------------------
! Read in the parameters of the vibronic coupling Hamiltonian 
!-----------------------------------------------------------------------
      if ( .not. gaout) &
            call readguess(iguess,mr(mrconst),mr(mrkappa),mr(mrlambda),&
                    mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
                    mi(miexpcoemask),mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
                    mr(mrswitch),mr(mrcp2d))
      if ( .not. gaout) close(iguess)
      
      if ( gaout) &
      call readgapar(iguess,mr(mrkappa),mr(mrlambda),mr(mrgamma),&
         mr(mriota),mr(mrmu),mr(mrexpcoe),mr(mrtermk) )
      
      close(iguess)
!-----------------------------------------------------------------------
! Output parameters
!-----------------------------------------------------------------------      
      if (lweight) then
         call outweightedpar(iout,mr(mrconst),mr(mrkappa),mr(mrlambda),&
              mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
              mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
              mr(mrswitch))
      else
         call outputpar(iout,mr(mrconst),mr(mrkappa),mr(mrlambda),&
              mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
              mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
              mr(mrswitch))
      endif
         
      STOP

2000 write (6,*) &
          'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      STOP

 1000 continue
      ilbl=index(guessfile,' ')-1
      print*
      print*,'ERROR: Guess file not found : ',guessfile(1:ilbl)
      print*
      STOP

      contains

!#####################################################################

        subroutine vcexinp(filename,aout,lweight,wtyp,gaout)

          implicit none
          
          integer(long)     :: i,j,iarg,wtyp
          character(len=80) :: filename,string,aout
          logical(kind=4)   :: lweight
          logical(kind=4),intent(out) :: gaout
          integer(long), external :: myiargc 

! Set defaults
          filename=''
          aout=''
          lweight=.false.

! Determine number of arguments
          iarg=myiargc()
          if (iarg.lt.1) goto 200

! Read arguments
          i=0
100       continue
          i=i+1
          call mygetarg(i,string)
 
          if (string(1:1) .eq. '-') then
             if (string.eq.'-weight') then
                lweight=.true.
                wtyp=1
             else if (string.eq.'-weight2') then
                lweight=.true.
                wtyp=2
             else if (string.eq.'-out') then
                i=i+1
                call mygetarg(i,string)
                read(string,'(a)') aout
             elseif(string.eq.'-ga') then
               gaout = .true.
             else
                write(6,'(2a)') ' unknown option : ', string
                STOP
             end if          
! Read next option
             go to 100
          else if (i.eq.iarg) then
             filename=string
          endif

200       continue
          if (slen(guessfile) .eq. 0) then
             print*,'Enter VCHAM filename'
             read(5,'(a)') filename
          endif

          i=index(filename,'.vcham')
          if (i.eq.0) then
             j=len_trim(filename)
             write(filename(j+1:j+7),'(a6)') '.vcham'
          endif

          return
        end subroutine vcexinp
          
!#####################################################################

        subroutine getinfo(filename,unit,gaout)

          implicit none

          integer(long)     :: unit,idat,i
          character(len=80) :: filename,info,string,dum
          logical(kind=4), intent(in) :: gaout 
          idat=73
          info=''

!-----------------------------------------------------------------------
! Determine info file name
!-----------------------------------------------------------------------
         rewind(unit)
         if(.not. gaout) then
          read(unit,*)
          read(unit,'(14x,60a)') info
         else
          read(unit,'(A2,A30)') dum,info
          info = trim(adjustl(info))
         endif
         rewind(unit)
!-----------------------------------------------------------------------
! Read info file
!-----------------------------------------------------------------------
          open(idat,file=info,form='formatted',status='old',&
               err=1100)
          rewind(idat)
          do i=1,5
             read(idat,*)
          enddo
          read(idat,*) nmodes
          do i=1,3
             read(idat,*)
          enddo
          read(idat,*) nsta

10        continue
          read(idat,*) string
          if (string(1:24).ne.'#vibrational_frequencies') goto 10
          do i=1,nmodes
             read(idat,'(31x,F12.10)') freq0(i)
          enddo

          return

1100      write(6,'(a)') 'info file not found'
          STOP

        end subroutine getinfo

!#####################################################################

        subroutine outputpar(io,const,kappa,lambda,gamma,iota,&
             mu,expcoe,higher,termk,cpcoe,switch)
          
          implicit none
          
          integer(long), intent(in) :: io
          
          integer(long), external   :: slen
          integer(long)             :: i,j,s,j1,j2,j3,nblck,n,s1,k,ilbl,jlbl,&
               nhi,l,m,m1,n1,junk,junk1,mode,o,p,itmp,mod1,mod2
          real(dop)                 :: freq,x0,d0,alpha,rtmp
          
          real(dop), dimension(nhigher), intent(inout)                 :: higher
          real(dop), dimension(nadd), intent(inout)                    :: termk
          real(dop), dimension(nsta,nsta), intent(inout)               :: const
          real(dop), dimension(nmodes,nsta), intent(inout)             :: kappa
          real(dop), dimension(nmodes,nsta,nsta), intent(inout)        :: lambda
          real(dop), dimension(nmodes,nmodes,nsta), intent(inout)      :: gamma
          real(dop), dimension(maxcoe,nmodes,nsta), intent(in)         :: expcoe
          real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: iota,&
                                                                          mu
          real(dop), dimension(maxcoe,nmodes,nsta,nsta), intent(inout) :: cpcoe
          real(dop), dimension(nswitch,maxcoe), intent(in)             :: switch
      
          real(dop), parameter    :: tol=0.000006d0

          character(len=60)       :: string
          character(len=10)       :: atmp
          character(len=80)       :: tmp
      
!-----------------------------------------------------------------------
! General diabatic potential function parameters
!-----------------------------------------------------------------------
          do n=1,nmodes
             do s=1,nsta
                do k=1,ncoeff(n,s)
                   write(io,'(a7,x,i1,x,i3,x,i2,x,a1,x,f12.8)')&
                        "general",k,n,s,"=",expcoe(k,n,s)
                enddo
             enddo
          enddo
          
!-----------------------------------------------------------------------
! Kappa
!-----------------------------------------------------------------------
          do n=1,nmodes
             do s=1,nsta
                if (kappa(n,s).ne.0.0) then
                   write(io,'(a5,x,i3,x,i2,x,a1,x,F10.4)') &
                        'kappa',n,s,'=',kappa(n,s)
                endif
             enddo
          enddo
   
!-----------------------------------------------------------------------
! Lambda
!-----------------------------------------------------------------------
          do n=1,nmodes
             do s=1,nsta-1
                do s1=s+1,nsta
                   if (lambda(n,s,s1).ne.0.0) then
                      write(io,'(a6,x,i3,x,i2,x,i2,x,a1,x,F10.4)') &
                           'lambda',n,s,s1,'=',lambda(n,s,s1)
                   endif
                enddo
             enddo
          enddo

!-----------------------------------------------------------------------
! Gamma (quadratic)
!-----------------------------------------------------------------------
          do n=1,nmodes
             do s=1,nsta
                if (gamma(n,n,s).ne.0.0) then
                   write(io,'(a5,x,i3,x,i3,x,i2,x,a1,x,F10.4)') &
                        'gamma',n,n,s,'=',gamma(n,n,s)
                endif
             enddo
          enddo

!-----------------------------------------------------------------------
! Gamma (bilinear)
!-----------------------------------------------------------------------
          do n=1,nmodes-1
             do m=n+1,nmodes
                do s=1,nsta
                   if (gamma(n,m,s).ne.0.0) then
                      write(io,'(a5,x,i3,x,i3,x,i2,x,a1,x,F10.4)') &
                           'gamma',n,m,s,'=',gamma(n,m,s)
                   endif
                enddo
             enddo
          enddo

!-----------------------------------------------------------------------
! Mu (quadratic)
!-----------------------------------------------------------------------
          do n=1,nmodes
             do s=1,nsta-1
                do s1=s+1,nsta
                   if (mu(n,n,s,s1).ne.0.0) then
                      write(io,'(a2,x,i3,x,i3,x,i2,x,i2,x,a1,x,F10.4)') &
                           'mu',n,n,s,s1,'=',mu(n,n,s,s1)
                   endif
                enddo
             enddo
          enddo

!-----------------------------------------------------------------------
! Mu (bilinear)
!-----------------------------------------------------------------------
          do n=1,nmodes-1
             do m=n+1,nmodes
                do s=1,nsta-1
                   do s1=s+1,nsta
                      if (mu(n,m,s,s1).ne.0.0) then
                         write(io,'(a2,x,i3,x,i3,x,i2,x,i2,x,a1,x,F10.4)') &
                              'mu',n,m,s,s1,'=',mu(n,m,s,s1)
                      endif
                   enddo
                enddo
             enddo
          enddo
             
!-----------------------------------------------------------------------
! iota (cubic)
!-----------------------------------------------------------------------
          do n=1,nmodes
             do s=1,nsta
                if (abs(iota(n,n,s,s)).gt.tol) then
                   write(io,'(a4,x,i3,x,i3,x,i2,x,a1,x,F10.4)') &
                        'iota',n,n,s,'=',iota(n,n,s,s)
                endif
             enddo
          enddo

!-----------------------------------------------------------------------
! iota (quadratic-linear)
!-----------------------------------------------------------------------
          do n=1,nmodes
             do m=1,nmodes
                if (m == n) cycle
                do s=1,nsta
                   if (abs(iota(n,m,s,s)).gt.tol) then
                      write(io,'(a4,x,i3,x,i3,x,i2,x,a1,x,F10.4)') &
                           'iota',n,m,s,'=',iota(n,m,s,s)
                   endif
                enddo
             enddo
          enddo
    
!-----------------------------------------------------------------------
! Explicitly added terms
!-----------------------------------------------------------------------
          do n = 1,nadd
             ilbl = 1
             string= ' '
             do m=1,nmodes
                if (abs(termdef(m,n)) .gt. 0) then
                   write(atmp,'(i4,a,i4)') m,'^',termdef(m,n)
                   call rmblnk(atmp)
                   jlbl=slen(atmp)
                   string(ilbl:ilbl+jlbl-1)=atmp(1:jlbl)
                   ilbl=ilbl+jlbl+1
                endif
             enddo
             if (termk(n) .ne. 0 ) &
             write(io,'(a8,2i3,2x,a20,a1,x,E13.5)') &
                  'explicit',termst(1,n),termst(2,n),&
                  trim(adjustl(string(1:56)))//' ','=',termk(n)
          enddo
          

!-----------------------------------------------------------------------
! coupling function coefficients
!-----------------------------------------------------------------------
          do j=1,nsta-1
             do i=j+1,nsta
                do m=1,nmodes
                   n=ncoeffcp(m,j,i)
                   if (n.gt.0) then
                      do k=1,n
                         if (cpcoe(k,m,j,i).ne.0.0) then
                            write(io,'(a9,x,i2,x,i3,x,i2,x,i2,x,a1,x,e13.5)') &
                                 'cpgeneral',k,m,j,i,'=',cpcoe(k,m,j,i)
                         endif
                      enddo
                   endif
                enddo
             enddo
          enddo

!#####################################################################
!-----------------------------------------------------------------------
! switching functions
!-----------------------------------------------------------------------
          if (lswitch) then
             do i=1,nswitch
                do j=1,nswitchcoe(i)
                   call getswitch(aswitch(i),tmp)
                   write(io,'(a6,x,i2,x,a,x,a1,x,e13.5)') 'switch',&
                        j,tmp(1:len_trim(tmp)),'=',switch(i,j)
                enddo
             enddo
          endif
          
          return
    end subroutine outputpar
        
!#####################################################################

    subroutine outweightedpar(io,const,kappa,lambda,gamma,iota,&
         mu,expcoe,higher,termk,cpcoe,switch)

      implicit none
      
      integer(long), intent(in) :: io
      
      integer(long), external   :: slen
      integer(long)             :: i,j,s,j1,j2,j3,nblck,n,s1,k,ilbl,jlbl,nhi,&
           l,m,m1,n1,junk,junk1,mode,o,p,itmp,mod1,mod2
      real(dop)                 :: freq,x0,d0,alpha,rtmp
      
      real(dop), dimension(nhigher), intent(inout)                 :: higher
      real(dop), dimension(nadd), intent(inout)                    :: termk
      real(dop), dimension(nsta,nsta), intent(inout)               :: const
      real(dop), dimension(nmodes,nsta), intent(inout)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(inout)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(inout)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(in)         :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: iota,mu
      real(dop), dimension(maxcoe,nmodes,nsta,nsta), intent(inout) :: cpcoe
      real(dop), dimension(nswitch,maxcoe), intent(in)             :: switch
      
      character(len=60)       :: string
      character(len=10)       :: atmp
      character(len=80)       :: tmp
      
      real(dop), dimension(nmodes,nsta) :: weight
      real(dop)                         :: ftmp

!-----------------------------------------------------------------------
! Frequency-weighted parameters
!-----------------------------------------------------------------------
      write(io,'(/,a,/)') &
           '+++++ Effective frequencies +++++'
      do n=1,nmodes
         do s=1,nsta
            ftmp=sqrt(freq0(n)*freq0(n)+freq0(n)*gamma(n,n,s))
            write(6,'(i3,2x,i2,2x,F13.10)') n,s,ftmp
         enddo
      enddo

      do n=1,nmodes
         do s=1,nsta
            if (wtyp.eq.1) then
               weight(n,s)=freq0(n)
            else
               weight(n,s)=freq0(n)*freq0(n)+freq0(n)*gamma(n,n,s)
               weight(n,s)=sqrt(weight(n,s))
            endif
         enddo
      enddo
      
      write(io,'(/,a,/)') &
           '+++++ Frequency-weighted parameters +++++'

!-----------------------------------------------------------------------
! Kappa
!-----------------------------------------------------------------------
      write(io,'(/,a)') &
           '++++ Intrastate coupling strengths ++++'
      do n=1,nmodes
         do s=1,nsta
            if (kappa(n,s).ne.0.0) then
               write(io,'(a5,x,i3,x,i2,x,a1,x,F10.4)') &
                    'kappa',n,s,'=',kappa(n,s)/weight(n,s)
            endif
         enddo
      enddo
   
!-----------------------------------------------------------------------
! Lambda
!-----------------------------------------------------------------------
      if (wtyp.eq.1) then
         write(io,'(/,a)') &
              '++++ Interstate coupling strengths ++++'
      else
         write(io,'(/,a)') &
            '++++ Interstate coupling strengths. Upper-state weighting ++++'
      endif
      do n=1,nmodes
         do s=1,nsta-1
            do s1=s+1,nsta
               k=max(s,s1)
               if (lambda(n,s,s1).ne.0.0) then
                  write(io,'(a6,x,i3,x,i2,x,i2,x,a1,x,F10.4)') &
                       'lambda',n,s,s1,'=',lambda(n,s,s1)/weight(n,k)
               endif
            enddo
         enddo
      enddo
      
      if (wtyp.eq.2) then
         write(io,'(/,a)') &
            '++++ Interstate coupling strengths. Lower-state weighting ++++'
         do n=1,nmodes
            do s=1,nsta-1
               do s1=s+1,nsta
                  k=min(s,s1)
                  if (lambda(n,s,s1).ne.0.0) then
                     write(io,'(a6,x,i3,x,i2,x,i2,x,a1,x,F10.4)') &
                          'lambda',n,s,s1,'=',lambda(n,s,s1)/weight(n,k)
                  endif
               enddo
            enddo
         enddo
      endif

      return
    end subroutine outweightedpar

!#####################################################################

      subroutine readgapar(io,kappa,lambda,gamma,iota,mu,expcoe,termk)

      implicit none

      integer(long)             :: i,j,s,n,s1,m,m1,x,o
      integer(long),intent(in)                                  :: io
      real(dop)                 :: rdum
      real(dop), dimension(nadd), intent(out)                    :: termk
      real(dop), dimension(nmodes,nsta), intent(out)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(out)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(out)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(out)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: iota,mu
      character(len=50)       :: string, dum
      logical(kind=4)         :: igno

       kappa = 0.0
       lambda = 0.0
       gamma = 0.0
       mu = 0.0
       iota = 0.0
       expcoe = 0.0
       termk = 0.0
      rewind(io)
200   read(io,'(50A)',err=998,end=999) string
      string = trim(adjustl(string))
      if(string(1:1) .eq. 'k') then
        read(string,'(A1,2I3,F15.8)',err=998) dum,i,s,rdum 
        kappa(i,s) = rdum
      elseif(string(1:1) .eq. 'l') then
        read(string,'(A1,3I3,F15.8)',err=998) dum,i,s,s1,rdum 
        lambda(i,s,s1) = rdum
        lambda(i,s1,s) = lambda(i,s,s1)
      elseif(string(1:1) .eq. 'g') then
        read(string,'(A1,3I3,F15.8)',err=998) dum,i,j,s,rdum 
        gamma(i,j,s) = rdum
        gamma(j,i,s) = gamma(i,j,s)
      elseif(string(1:1) .eq. 'm') then
        read(string,'(A1,4I3,F15.8)',err=998) dum,j,i,s,s1,rdum 
        mu(j,i,s,s1) = rdum
        mu(j,i,s1,s) = mu(j,i,s,s1)
        mu(i,j,s1,s) = mu(j,i,s,s1)
        mu(i,j,s,s1) = mu(j,i,s,s1)
      elseif(string(1:1) .eq. 'i') then
        read(string,'(A1,4I3,F15.8)',err=998) dum,j,i,s,s1,rdum 
        iota(j,i,s,s1) = rdum
        iota(j,i,s1,s) = iota(j,i,s,s1)
        iota(i,j,s1,s) = iota(j,i,s,s1)
        iota(i,j,s,s1) = iota(j,i,s,s1)
      elseif(string(1:1) .eq. 'x') then
        read(string,'(A1,3I3,F15.8)',err=998) dum,k,i,s,rdum 
        if(ncoeff(i,s) .lt. k) ncoeff(i,s) =  k
        expcoe(k,i,s) = rdum          
      elseif(string(1:1) .eq. 't') then
        read(string,'(A1,I3)',err=998) dum,i
        dum = trim(string)
        if ( i .gt. nadd) nadd = i
        do while ( index(trim(dum),'^') .ne. 0 ) 
          j = index(trim(dum),'^')
          m = numit(dum(j-1:j-1)//'    ')
          o = numit(dum(j+1:j+1)//'    ')
          termdef(m,i) = o
          dum = dum(j+2:len(trim(dum)))
        enddo
        dum = trim(adjustl(dum))
        read(dum,'(F15.8)') termk(i)
        read(string,'(A1,3I3)',err=998) dum,i,termst(1,i),termst(2,i) 
      endif
      goto 200
999   return

998   print*, 'ERROR READING PARAMETERS (readgapar)'
      return

      end subroutine readgapar

!#######################################################################

      integer(long) function numit(untrimmed)
      character(len=5), intent(in) :: untrimmed
      character(len=3)             :: string

      string = trim(untrimmed)
      if( len(trim(string)) .lt. 2) then
                      read(string,'(I1)') numit
      elseif( len(trim(string)) .lt. 3) then
              if(trim(string(1:1)) .eq. "0") then
                     read(string(2:2),'(I1)') numit
              else
                     read(string,'(I2)') numit
              endif
      elseif( len(trim(string)) .lt. 4) then
              if(trim(string(1:2)) .eq. "00") then
                    read(string(3:3),'(I1)') numit
              elseif (trim(string(1:1)) .eq. "0") then
                    read(string(2:3),'(I2)') numit
              else
                    read(string,'(I3)') numit
              endif
      endif
      return
      end function

!#######################################################################

    subroutine getswitch(aswitch,tmp)

      implicit none

      integer(long)     :: i,j,k,narg
      character(len=80) :: aswitch,tmp
      character(len=20), dimension(7) :: aarg

      tmp=''

!-----------------------------------------------------------------------
! split aswitch into indiviual arguments
!-----------------------------------------------------------------------
      do i=1,7
         aarg(i)=''
      enddo

      narg=1
      j=0
      do i=1,len_trim(aswitch)
         if (aswitch(i:i).eq.' ') then
            narg=narg+1
            j=0
         else
            j=j+1
            aarg(narg)(j:j)=aswitch(i:i)
         endif
      enddo

!-----------------------------------------------------------------------
! subtract the last argument (afunc)
!-----------------------------------------------------------------------      
      j=1
      do i=1,narg-1
         k=j+len_trim(aarg(i))
         write(tmp(j:k),'(a)') aarg(i)(1:len_trim(aarg(i)))
         write(tmp(k+1:k+1),'(a)') ' '
         j=j+len_trim(aarg(i))+1
      enddo

    end subroutine getswitch

!#####################################################################

end program vcextract
