!------------------------------------
!
! VCGAAnal
! Vibronic Coupling Genetic Algorithm Analyses
!-------------------------------------
!-----------------------------------------------------------------------
      module vcgaanalmod

use vcglobal, only: maxpar
use decimal, only: long,dop

implicit none

!-----------------------------------------------------------------------
!     Common variables for all Genetic Algorithm analysis routines
!
!-----------------------------------------------------------------------
      real(dop), dimension(maxpar), save      :: importval,gamin,gamax,average,sd
      integer(long), save                     :: nruns,mrcpu,mrreal,mrarmsd,&
                                                 mrbrmsd,mrsdrmsd,ilbl,mxgen
      integer(long), dimension(maxpar), save  :: nimportval,nimportval0
      character(len=80), save                 :: fileguess,psfile,gnufile,&
                                                 unit,string
      character(len=80), dimension(500), save :: gafilename

!-------------------------------------------------
!   Variables for input file generation
!---------------------------------------------------

      real(dop), dimension(3), save           :: rngseed
      integer(long), save                     :: nstatic
      integer(long), dimension(3), save       :: mut,pop,gen
      integer(long), dimension(4), save       :: mutyp
      logical(kind=4), save                   :: lgeninp
      character(len=80), save                 :: stem
      character(len=80), dimension(100), save :: static

      end module vcgaanalmod

!#######################################################################

      program vcgaanal

use decimal, only: long,dop
use vcglobal
use global, only:pi, keyword,keyorig,maxkey,lc,lwarn
use versions

use vcgaanalmod

use maxv, only: maxdim,maxsta,maxset
use vchammod
use vchfitmod
use versions
use invchammod, only: memvcham
use op1lib, only: zerovxd, zerovxi
use logdat, only: message, routine, inptit
use utils, only: vcwrline
use op1lib, only: zeromxd

use module_interface

use iofiles, only: readguess


      implicit none

      integer(long)                               :: ierr
      character(len=10)   :: cerr

      real(dop), dimension(:), allocatable        :: optcoo
      complex(dop), dimension(:), allocatable     :: mc
      real(dop), dimension(:), allocatable        :: mr
      integer(long), dimension(:), allocatable    :: mi
      logical(kind=4), dimension(:), allocatable  :: ml

      integer(long)                                  :: c
      character(len=80)                              :: filename

      integer(long), save :: iz,iz0

print*,'This program does not work - needs to read maxdim, maxsta from input'
  stop

!-----------------------------------------------------------------------
! Initialise calculation 
! get input file name, set maxdim and then allocate memory
!-----------------------------------------------------------------------
!      call initvcham(1)

!      if (maxdim .lt. 0) then
!         routine='VCTRANS'
!         message='Error reading dimensions from input file'
!         call vcerrormsg
!      endif
!      if (maxsta .lt. 0) then
!         routine='VCTRANS'
!         message='Error reading no. states from input file'
!         call vcerrormsg
!      endif

!      call alloc_vctransmod
!      call alloc_vchammod

         print*,'Assigning Memory for GA analysis'
         mrlogdata = mrdim
         mrdim = mrlogdata + (nruns*4)
         mrarmsd = mrdim
         mrdim = mrarmsd + (nruns * maxgen)
         mrbrmsd = mrdim
         mrdim = mrbrmsd + (nruns * maxgen)
         mrsdrmsd = mrdim
         mrdim = mrsdrmsd + (nruns * maxgen)
         mrcpu = mrdim
         mrdim = mrcpu + ((nruns * maxgen) * 4)
         mrreal = mrdim
         mrdim = mrreal + ((nruns * maxgen) * 4)

      call wrversion(ilog)
      print*,'Setting default values'
!-----------------------------------------------------------------------
! set default values for parameters and flags
!-----------------------------------------------------------------------
      call vcgadefault
      print*,'Initial values, reading input'
!-----------------------------------------------------------------------
! open files
! read input file
!-----------------------------------------------------------------------
      call gafinp

! Assign memory
      lgaanal = .true.
      call memvcham 

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
      write(ilog,'(/,a)') 'MEMORY ALLOCATED'
      ilbl = 8*optcoodim+16*mcdim+8*mrdim+4*midim+4*mldim
      write(ilog,'( 5(a,i10,/),a,i13,a,f10.2,a)')&
       ' optcoodim  : ', optcoodim,&
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

      if(allocated(optcoo))deallocate(optcoo)
      if(allocated(mc))deallocate(mc)
      if(allocated(mr))deallocate(mr)
      if(allocated(mi))deallocate(mi)
      if(allocated(ml))deallocate(ml)

      cerr = 'vcgaanal_optcoo '
      allocate(optcoo(optcoodim),stat=ierr)
      if (ierr > 0) go to 2000
      cerr = 'vcgaanal_mc '
      allocate(mc(mcdim),stat=ierr)
      if (ierr > 0) go to 2000
      cerr = 'vcgaanal_mr '
      allocate(mr(mrdim),stat=ierr)
      if (ierr > 0) go to 2000
      cerr = 'vcgaanal_mi '
      allocate(mi(midim),stat=ierr)
      if (ierr > 0) go to 2000
      cerr = 'vcgaanal_ml '
      allocate(ml(mldim),stat=ierr)
      if (ierr > 0) go to 2000

      call zerovxd(mr,mrdim)
      call zerovxi(mi,midim)

!     Need a section that reads in ngen, etc.

!-----------------------------------------------------------------------
!   Finds important values (those with a value greater than a tenth of
!      the frequency of the relevent mode(S)) 
!-----------------------------------------------------------------------

      do c=1,nruns
         filename=gafilename(nruns)
         call gaanalfilename(c,1,filename)
         open(iguess,file=filename,form='formatted',status='old')
         call readguess(iguess,mr(mrconst),mr(mrkappa),mr(mrlambda),&
              mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
              mi(miexpcoemask),mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
              mr(mrswitch),mr(mrcp2d))
         close(iguess)
!         call importvalue(mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mrmu),&
!              mi(mikappamask),mi(milambdamask),&
!              mi(migammamask),mi(mimumask),mr(mrexpcoe),&
!              mi(miexpcoemask),c)
       enddo 

!-----------------------------------------------------------------------
! Read in important values and save in importval array
!-----------------------------------------------------------------------

!!! Change filenames to those in input
!      do c=1,nruns
!        call gaanalfilename(c,2,filename)
!        open(iguess,file=filename,form='formatted',status='old')
!        call readguess(iguess,mr(mrconst),mr(mrkappa),mr(mrlambda),&
!             mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
!             mi(miexpcoemask),mr(mrhigher),mr(mrtermk),mr(mrekappa),&
!             mr(mregamma),mr(mrelambda),mr(mremu),mr(mre2xpcoe),&
!             mr(mreiota),mr(mrdexpcoe),mr(mrselambda),mr(mrsemu))
!        close(iguess)
!        call savevalue(mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mrmu),&
!             mi(mikappamask),mi(milambdamask),mi(migammamask),&
!             mi(mimumask),mr(mrexpcoe),mi(miexpcoemask))
!      enddo 


!-----------------------------------------------------------------------
!  Read log files for RMSD & timing information
!-----------------------------------------------------------------------

!     print*,'Read log files for RMSD & timing information'
!     do c=1,nruns
!        call gaanalfilename(c,2,filename)
!        print*,'Opening file ',filename
!        call gareadlog(c,mr(mrcpu),mr(mrreal),mr(mrarmsd),&
!      mr(mrsdrmsd),mr(mrbrmsd))
!        close(ilog)
!     enddo

!-----------------------------------------------------------------------
! Output into .out file all a summary and then the importvalues
!-----------------------------------------------------------------------
       
!      print*,'Output into .out file all a summary'
!      call outsum(mr(mrcpu),mr(mrreal),mr(mrarmsd),&
!      mr(mrsdrmsd),mr(mrbrmsd))
!      call outvalues(mr(mrkappa),mr(mrlambda),mr(mrgamma),&
!             mr(mrmu),mi(mikappamask),mi(milambdamask),&
!             mi(migammamask),mi(mimumask),mi(miexpcoemask))

!-----------------------------------------------------------------------
! Create .gnu & .ps files for a variety of data
!-----------------------------------------------------------------------

!      print*,'Plotting armsd graph'
!      call gaplot(mr(mrarmsd),1,mr(mrcpu))
!      print*,'Plotting brmsd graph'
!      call gaplot(mr(mrbrmsd),2,mr(mrcpu))
!      print*,'Plotting sdrmsd graph'
!      call gaplot(mr(mrsdrmsd),3,mr(mrcpu))
!      print*,'Plotting cpu grpah'
!      call gaplot(mr(mrcpu),4,mr(mrcpu))
!      print*,'Plotting summary graph'
!      call gaplot2(mr(mrarmsd),mr(mrbrmsd),mr(mrsdrmsd))

      cerr = 'vcgaanal_...'
      deallocate(optcoo,mc,mr,mi,ml,stat=ierr)
      if (ierr > 0) go to 2001
 
      stop

 2000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      stop

 2001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
      stop

      contains

!-----------------------------------------------------------------------
! SUBROUTINE DEFAULTS
!-----------------------------------------------------------------------
      subroutine vcgadefault 

      implicit none

       integer(long)       :: c

       iin=1
       ilog=2
       iout=13
       iguess=101

       Do c=1,maxpar
         nimportval(c) = 0
       enddo

       nstatic = 0
       mutyp(1) = 0
       mutyp(2) = 0
       mutyp(3) = 0
       mutyp(4) = 0
       nruns = 0
       do c=1,3
          mut(c) = 0
          pop(c) = 0
          gen(c) = 0
          rngseed(c) = 0
!!! Need to add new mutation variables here
       enddo

       return
       end subroutine vcgadefault

!-----------------------------------------------------------------------
!
! SUBROUTINE INPUT FILES
!
!-----------------------------------------------------------------------

      subroutine gafinp 

      use vcgaanalmod

      implicit none

      integer(long)       :: ic,ierr,i,ie

      iz0 = 0
      iz = 0

!-----------------------------------------------------------------------
! get input file name and file stem
!-----------------------------------------------------------------------
      ilbl=index(filein,'.')-1
      if (ilbl .le. 0) then
         ilbl=index(filein,' ')-1
         filein=filein(1:ilbl)//'.inp'
         fileout=filein(1:ilbl)//'.out'
         filelog=filein(1:ilbl)//'.log'
      endif
      filein=filein(1:ilbl)//'.inp'
      fileout=filein(1:ilbl)//'.out'
      filelog=filein(1:ilbl)//'.log'
!-----------------------------------------------------------------------
! Open input file
!-----------------------------------------------------------------------
      open(iin,file=filein,form='formatted',status='old')

!-----------------------------------------------------------------------
! read input file
!-----------------------------------------------------------------------
 10   continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,ie,ierr,maxkey)
!       end
      routine = 'vchfinp (read VCHFIT input)'
!      call vccntrl(ierr)
      i=1

 15   continue
      if (keyword(i) .eq. 'end-input') then
         goto 50

      else if (keyword(i) .eq. 'filename-section') then
 30       continue
          call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
          if (keyword(1) .ne. 'end-filename-section') then
              nruns=nruns+1
              gafilename(nruns)=keyword(1)
              goto 30
          endif
      else
            print*,'Unrecognised Input keyword : ',keyword(i)
            stop
      endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------

      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

 50   continue

      close(iin)
      return
      end subroutine gafinp

!-----------------------------------------------------------------------
! SUBROUTINE FIND IMPORTANT VALUE
!-----------------------------------------------------------------------
      subroutine importvalue(kappa,lambda,gamma,&
              mu,kappamask,lambdamask,gammamask,&
              mumask,expcoe,expcoemask,run)

      implicit none

      integer(long)             :: s,s1,i,k,t,j
      integer(long), intent(in) :: run

      integer(long), dimension(nmodes,nsta), intent(inout)             :: kappamask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout)        :: lambdamask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout)      :: gammamask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)      :: expcoemask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: mumask

      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: mu

      real(dop)       :: tfreq



       do s=1,nsta
         do i=1,nmodes
            tfreq = 0.1 * freq0(i)
            if (abs(kappa(i,s)) .gt. tfreq) then
                kappamask(i,s) = kappamask(i,s) + 1
                nimportval(run) = nimportval(run) + 1
            endif
         enddo
       enddo

      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
              tfreq = 0.1 * freq0(i)
              if (abs(lambda(i,s,s1)) .gt. tfreq) then
                  lambdamask(i,s,s1) = lambdamask(i,s,s1) + 1
                  nimportval(run) = nimportval(run) + 1 
              endif
            enddo
         enddo
      enddo

      do s=1,nsta
         do i=1,nmodes
            do j=1,nmodes
              tfreq = 0.1 * SQRT(freq0(i)*freq0(j))
              if (abs(gamma(i,j,s)) .gt. tfreq) then
                 gammamask(i,j,s) = gammamask(i,j,s) +1
                 nimportval(run) = nimportval(run) + 1
              endif
            enddo
         enddo
      enddo

      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
               do j=1,nmodes
                  tfreq = 0.1 * SQRT(freq0(i)*freq0(j))
                  if (abs(mu(i,j,s,s1)) .gt. tfreq) then
                    mumask(i,j,s,s1) = mumask(i,j,s,s1) +1 
                    nimportval(run) = nimportval(run) + 1
                  endif
               enddo
             enddo
           enddo
      enddo

      do k=1,maxcoe
         do i=1,nmodes
            do s=1,nsta
                 tfreq = 0.1 * freq0(i)
                 if (ifdiab(i,s) .eq. 1) then 
                 do t=1,3
                  if (abs(expcoe(k,i,s)) .gt. tfreq) then
                     expcoemask(k,i,s) = expcoemask(k,i,s) + 1 
                     nimportval(run) = nimportval(run) + 1
                   endif
                 enddo
                 elseif ((ifdiab(i,s) .eq. 2) .or. &
                   ifdiab(i,s) .eq. 4) then
                 do t=1,4
                    if (abs(expcoe(k,i,s)) .gt. 0.1) then
                      expcoemask(k,i,s) = expcoemask(k,i,s) + 1
                      nimportval(run) = nimportval(run) + 1
                    endif
                 enddo
                 else
                    print*, 'Unknown potential'
                 endif
             enddo
           enddo
      enddo

      return
      end  subroutine importvalue

!-----------------------------------------------------------------------
! SUBROUTINE SAVE IMPORTANT VALUE 
!-----------------------------------------------------------------------
      subroutine savevalue(kappa,lambda,gamma,mu,kappamask,&
              lambdamask,gammamask,mumask,expcoe,expcoemask)

      implicit none

      integer(long)       :: s,s1,i,k,n,j
      integer(long)       :: cutoff

      integer(long), dimension(nmodes,nsta), intent(inout)             :: kappamask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout)        :: lambdamask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout)      :: gammamask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)      :: expcoemask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: mumask

      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: mu

      cutoff = nint(0.3*nruns)

      n=1
      do s=1,nsta
        do i=1,nmodes
           if (kappamask(i,s) .gt. cutoff) then
               importval(n) = kappa(i,s)
               n = n + 1
           else if (kappamask(i,s) .ne. 0) then
               kappamask(i,s) = -1
           endif 
        enddo
      enddo

      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
              if (lambdamask(i,s,s1) .gt. cutoff) then
                  importval(n) = lambda(i,s,s1)
                  n = n + 1
              else if (lambdamask(i,s,s1) .ne. 0) then
                  lambdamask(i,s,s1) = -1
              endif
            enddo
         enddo
      enddo

      do s=1,nsta
         do i=1,nmodes
            do j=1,nmodes
              if (gammamask(i,j,s) .gt. cutoff) then
                 importval(n) = gamma(i,j,s) 
                 n=n+1
              else if (gammamask(i,s,s1) .ne. 0) then
                 gammamask(i,j,s) = -1
              endif
            enddo
         enddo
      enddo

      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
               do j=1,nmodes
                  if (mumask(i,j,s,s1) .gt. cutoff ) then
                     importval(n) = mu(i,j,s,s1)
                     n = n + 1
                  elseif (mumask(i,j,s,s1) .ne. 0) then
                     mumask(i,j,s,s1) = -1
                  endif
               enddo
            enddo
         enddo
      enddo  


      do k=1,maxcoe
         do i=1,nmodes
            do s=1,nsta
               if (expcoemask(k,i,s) .gt. cutoff) then
                  importval(n) = expcoe(k,i,s)
                  n=n+1
               elseif (expcoemask(k,i,s) .ne. 0) then
                  expcoemask(k,i,s) = -1
               endif
            enddo
         enddo
      enddo

!!! Section below removed as it doesn't seem neccessary      
!     do i=1,nruns
!       if (n .ne. nimportval(i)) then
!          print*,'ERROR: Number of importvalues in this run not'
!          print*,'equal to those already identified.  Have files'
!          print*,'changed during run?'
!       endif
!     enddo

      return

      end subroutine savevalue


!-----------------------------------------------------------------------
! SUBROUTINE OUTPUT IMPORTANT VALUES
!-----------------------------------------------------------------------

       subroutine outputimportval(kappa,&
              lambda,gamma,&
              mu,kappamask,lambdamask,&
              gammamask,mumask,expcoemask)

      implicit none

      integer(long)       :: s,s1,i,n,m,j,k
      integer(long), dimension(nruns)       :: c

      integer(long), dimension(nmodes,nsta), intent(in)             :: kappamask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambdamask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gammamask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoemask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: mumask

      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: mu


      n=1

      do s=1,nsta
        do i=1,nmodes
           if (kappamask(i,s) .gt. 0) then
              do m=1,nruns
                 c(m) = importval(n)
              enddo
              call evalvalues(c,n) 
              n = n + nruns
           endif
        enddo
      enddo

      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
              if (lambdamask(i,s,s1) .gt. 0) then
              do m=1,nruns
                 c(m) = importval(n)
              enddo
              call evalvalues(c,n)
              n = n + nruns
              endif
            enddo
         enddo
      enddo

      do s=1,nsta
         do i=1,nmodes
            do j=1,nmodes
              if (gammamask(i,j,s) .gt. 0) then
              do m=1,nruns
                 c(m) = importval(n)
              enddo
              call evalvalues(c,n)
              n = n + nruns
              endif
            enddo
         enddo
      enddo

      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
               do j=1,nmodes
                  if (mumask(i,j,s,s1) .gt. 0 ) then
                    do m=1,nruns
                       c(m) = importval(n)
                    enddo
                    call evalvalues(c,n)
                    n = n + nruns
                  endif
               enddo
            enddo
         enddo
      enddo


      do k=1,maxcoe
         do i=1,nmodes
            do s=1,nsta
               if (expcoemask(k,i,s) .gt. 0) then
                  do m=1,nruns
                      c(m) = importval(n)
                   enddo
                   call evalvalues(c,n)
                   n = n + nruns
               endif
            enddo
         enddo
      enddo


      return
      end subroutine outputimportval

!-----------------------------------------------------------------------
! SUBROUTINE EVALUATE IMPORTANT VALUES
!    Finds the maximum and minimum value aswell as calculating the 
!    standard deviation and the average value from the array handed down
!    n is the current important value being evaluated
!    c is the array containing the integers for all the values 
!      being evaluated so the value can be called from the importval array
!      
!-----------------------------------------------------------------------

      subroutine evalvalues(c,n) 

      implicit none

      integer(long)                               :: m
      integer(long), intent(in)                   :: n
      integer(long)                               :: tmp
      integer(long), dimension(nruns)             :: sdt
      integer(long), dimension(nruns), intent(in) :: c

      tmp = c(1)
      gamin(n) = importval(tmp)
      gamax(n) = importval(tmp)
      average(n) = 0
      sd(n) = 0 

      do m=1,nruns
         tmp = c(m) 
         if (importval(tmp) .lt. gamin(tmp)) then
            gamin(n) = importval(tmp) 
         else if (importval(tmp) .gt. gamax(tmp)) then
            gamax(n) = importval(tmp) 
         endif
         average(n) = average(n) + importval(tmp) 
      enddo

      average(n) = average(n) / nruns

      do m=1,nruns
         tmp = c(m)
         sdt(m) = importval(tmp) - average(n) 
         sd(n) = sd(n) + sdt(m)
      enddo

      sd(n) = SQRT((1/nruns)*sd(n))

      return
      end subroutine evalvalues

!-----------------------------------------------------------------------
! SUBROUTINE OUT VALUES
!      Prints the calculated value to the ouput file 
!
!-----------------------------------------------------------------------

      subroutine outvalues(kappa,lambda,gamma,&
              mu,kappamask,lambdamask,&
              gammamask,mumask,expcoemask)


      implicit none

      integer(long)       :: m,i,j,s,s1,k
      character(len=7)       :: type
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappamask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambdamask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gammamask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoemask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: mumask

      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: mu


      m=1

      write(iout,10) ' '
      write(iout,10) 'Summary of important values'
      write(iout,10) ' '
      write(iout,11) 'No.','Type','i','j','s','s1','Min','Max',&
       'Average','Standard Dev'  
      type = 'kappa  '
      do s=1,nsta
         do i=1,nmodes
            if (kappamask(i,s).gt.0) then  
              if (m.lt.10) then
               if (i.lt.10) then
                  write(iout,20) m,type,i,'-',s,'-',gamin(m),gamax(m),&
       average(m),sd(m)  
               else
                  write(iout,21) m,type,i,'-',s,'-',gamin(m),gamax(m),&
       average(m),sd(m) 
               endif
              else
               if (i.lt.10) then
                  write(iout,22) m,type,i,'-',s,'-',gamin(m),gamax(m),&
       average(m),sd(m)
               else
                  write(iout,23) m,type,i,'-',s,'-',gamin(m),gamax(m),&
       average(m),sd(m)
               endif
              endif
              m=m+1
            endif
         enddo
      enddo
      type = 'lambda'
      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
               if(lambdamask(i,s,s1).gt.0) then 
                 if (m.lt.10) then
                  if (i.lt.10) then
                     write(iout,30) m,type,i,'-',s,s1,gamin(m),gamax(m),&
       average(m),sd(m)  
                  else
                     write(iout,31) m,type,i,'-',s,s1,gamin(m),gamax(m),&
       average(m),sd(m) 
                  endif
                 else
                  if (i.lt.10) then
                     write(iout,32) m,type,i,'-',s,s1,gamin(m),gamax(m),&
       average(m),sd(m)
                  else
                     write(iout,33) m,type,i,'-',s,s1,gamin(m),gamax(m),&
       average(m),sd(m)
                  endif
                 endif
                 m=m+1
               endif
            enddo
         enddo
      enddo
      type='gamma  '
      do s=1,nsta
         do i=1,nmodes
            do j=1,nmodes
               if (gammamask(i,j,s).gt.0) then 
                 if (m.lt.10) then
                  if (i.lt.10) then
                     if (j.lt.10) then
                        write(iout,40) m,type,i,j,s,'-',gamin(m),&
                              gamax(m),average(m),sd(m) 
                     else
                        write(iout,41) m,type,i,j,s,'-',gamin(m),&
                              gamax(m),average(m),sd(m) 
                     endif
                  else
                     if (j.lt.10) then
                        write(iout,42) m,type,i,j,s,'-',gamin(m),&
                              gamax(m),average(m),sd(m) 
                     else
                        write(iout,43) m,type,i,j,s,'-',gamin(m),&
                              gamax(m),average(m),sd(m) 
                     endif
                  endif
                 else
                  if (i .lt. 10) then
                     if (j.lt.10) then
                        write(iout,44) m,type,i,j,s,'-',gamin(m),&
                              gamax(m),average(m),sd(m)
                     else
                        write(iout,45) m,type,i,j,s,'-',gamin(m),&
                              gamax(m),average(m),sd(m)
                     endif
                  else
                     if (j.lt.10) then
                        write(iout,46) m,type,i,j,s,'-',gamin(m),&
                              gamax(m),average(m),sd(m)
                     else
                        write(iout,47) m,type,i,j,s,'-',gamin(m),&
                              gamax(m),average(m),sd(m)
                     endif
                  endif
                 m=m+1
                 endif
               endif
            enddo
         enddo
      enddo
      type='mu     '
      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
               do j=1,nmodes
                  if (mumask(i,j,s,s1).gt.0 ) then 
                    if (m.lt.10) then
                     if (i.lt.10) then
                        if (j.lt.10) then
                           write(iout,50) m,type,i,j,s,s1,gamin(m),&
       gamax(m),average(m),sd(m) 
                        else
                           write(iout,51) m,type,i,j,s,s1,gamin(m),&
       gamax(m),average(m),sd(m) 
                        endif
                     else
                        if (j.lt.10) then
                           write(iout,52) m,type,i,j,s,s1,gamin(m),&
       gamax(m),average(m),sd(m) 
                        else
                           write(iout,53) m,type,i,j,s,s1,gamin(m),&
       gamax(m),average(m),sd(m) 
                        endif
                     endif
                    else
                     if (i.lt.10) then
                        if (j.lt.10) then
                           write(iout,54) m,type,i,j,s,s1,gamin(m),&
       gamax(m),average(m),sd(m)
                        else
                           write(iout,55) m,type,i,j,s,s1,gamin(m),&
       gamax(m),average(m),sd(m)
                        endif
                     else
                        if (j.lt.10) then
                           write(iout,56) m,type,i,j,s,s1,gamin(m),&
       gamax(m),average(m),sd(m)
                        else
                           write(iout,57) m,type,i,j,s,s1,gamin(m),&
       gamax(m),average(m),sd(m)
                        endif
                     endif
                    endif
                    m=m+1
                  endif
               enddo
            enddo
         enddo
      enddo
      do s=1,maxcoe
         do i=1,nmodes
            do k=1,nsta
               if (expcoemask(k,i,s) .gt. 0) then
                  if (ifdiab(i,s) .eq. 1) then
                     type = 'Q: EP   ' 
                     if (m.lt.10) then
                      if (i .lt. 10) then
                        write(iout,60) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                      else
                        write(iout,61) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                      endif
                     else
                      if (i .lt. 10) then
                        write(iout,62) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                      else
                        write(iout,63) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                      endif
                     endif
                  else if (ifdiab(i,s) .eq. 2) then
                     if (m.lt.10) then
                      if (i .lt. 10) then
                       if (k .eq. 1) then
                        type = 'M: D   '
                        write(iout,60) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m) 
                       elseif (k .eq.2) then
                        type = 'M: A   '
                        write(iout,60) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m) 
                       elseif (k.eq.3) then
                        type = 'M: X   '
                        write(iout,60) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k.eq.4) then
                        type = 'M: E   '  
                        write(iout,60) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m) 
                       endif
                      else
                       if (k .eq. 1) then
                        type = 'M: D   '
                        write(iout,61) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m) 
                       elseif (k .eq.2) then
                        type = 'M: A   '
                        write(iout,61) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m) 
                       elseif (k.eq.3) then
                        type = 'M: X   '
                        write(iout,61) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k.eq.4) then
                        type = 'M: E   '
                        write(iout,61) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       endif
                      endif
                     else
                      if (i .lt. 10) then
                       if (k .eq. 1) then
                        type = 'M: D   '
                        write(iout,62) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k .eq.2) then
                        type = 'M: A   '
                        write(iout,62) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k.eq.3) then
                        type = 'M: X   '
                        write(iout,62) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k.eq.4) then
                        type = 'M: E   '
                        write(iout,62) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       endif
                      else
                       if (k .eq. 1) then
                        type = 'M: D   '
                        write(iout,63) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k .eq.2) then
                        type = 'M: A   '
                        write(iout,63) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k.eq.3) then
                        type = 'M: X   '
                        write(iout,63) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k.eq.4) then
                        type = 'M: E   '
                        write(iout,63) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       endif
                      endif
                     endif
                  else if (ifdiab(i,s) .eq. 4) then
                     if (m.lt.10) then
                      if (i .lt. 10) then
                       if (k .eq. 1) then
                        type = 'A-M: D '
                        write(iout,60) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m) 
                       elseif (k .eq.2) then
                        type = 'A-M: A '
                        write(iout,60) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m) 
                       elseif (k.eq.3) then
                        type = 'A-M: X '
                        write(iout,60) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k.eq.4) then
                        type = 'A-M: E '
                        write(iout,60) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       endif
                      else
                       if (k .eq. 1) then
                        type = 'A-M: D '
                        write(iout,61) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k .eq.2) then
                        type = 'A-M: A '
                        write(iout,61) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k.eq.3) then
                        type = 'A-M: X '
                        write(iout,61) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k.eq.4) then
                        type = 'A-M: E '
                        write(iout,61) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       endif
                      endif
                     else
                      if (i .lt. 10) then
                       if (k .eq. 1) then
                        type = 'A-M: D '
                        write(iout,62) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k .eq.2) then
                        type = 'A-M: A '
                        write(iout,62) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k.eq.3) then
                        type = 'A-M: X '
                        write(iout,62) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k.eq.4) then
                        type = 'A-M: E '
                        write(iout,62) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       endif
                      else
                       if (k .eq. 1) then
                        type = 'A-M: D '
                        write(iout,63) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k .eq.2) then
                        type = 'A-M: A '
                        write(iout,63) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k.eq.3) then
                        type = 'A-M: X '
                        write(iout,63) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       elseif (k.eq.4) then
                        type = 'A-M: E '
                        write(iout,63) m,type,i,'-',s,'-',gamin(m),&
       gamax(m),average(m),sd(m)
                       endif
                      endif
                     endif
                  elseif (ifdiab(i,s) .eq. 3) then
                      print*, 'Buckingham potential not implemented'
                  endif
                  m=m+1
                 endif
                enddo
              enddo
            enddo 

       return

 10   format(a)
 11   format(a3,1x,a5,3x,a1,2x,a1,2x,a1,1x,a1,1x,a3,4x,a3,4x,a7,1x,a12)

 20   format(i1,3x,a7,1x,i1,2x,a1,2x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)
 21   format(i1,3x,a7,1x,i2,1x,a1,2x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)
 22   format(i2,2x,a7,1x,i1,2x,a1,2x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)
 23   format(i2,2x,a7,1x,i2,1x,a1,2x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)

 30   format(i1,3x,a7,1x,i1,2x,a1,2x,i1,1x,i1,1x,F7.5,F7.5,F8.5,F12.5)
 31   format(i1,3x,a7,1x,i2,1x,a1,2x,i1,1x,i1,1x,F7.5,F7.5,F8.5,F12.5)
 32   format(i2,2x,a7,1x,i1,2x,a1,2x,i1,1x,i1,1x,F7.5,F7.5,F8.5,F12.5)
 33   format(i2,2x,a7,1x,i2,1x,a1,2x,i1,1x,i1,1x,F7.5,F7.5,F8.5,F12.5)

 40   format(i1,3x,a7,1x,i1,2x,i1,2x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)
 41   format(i1,3x,a7,1x,i1,2x,i2,1x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)
 42   format(i1,3x,a7,1x,i2,1x,i1,2x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)
 43   format(i1,3x,a7,1x,i2,1x,i2,1x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)
 44   format(i2,3x,a7,1x,i1,2x,i1,2x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)
 45   format(i2,3x,a7,1x,i1,2x,i2,1x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)
 46   format(i2,3x,a7,1x,i2,1x,i1,2x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)
 47   format(i2,3x,a7,1x,i2,1x,i2,1x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)

 50   format(i1,3x,a7,1x,i1,2x,i1,2x,i1,1x,i1,1x,F7.5,F7.5,F8.5,F12.5)
 51   format(i1,3x,a7,1x,i1,2x,i2,1x,i1,1x,i1,1x,F7.5,F7.5,F8.5,F12.5)
 52   format(i1,3x,a7,1x,i2,1x,i1,2x,i1,1x,i1,1x,F7.5,F7.5,F8.5,F12.5)
 53   format(i1,3x,a7,1x,i2,1x,i2,1x,i1,1x,i1,1x,F7.5,F7.5,F8.5,F12.5)
 54   format(i2,2x,a7,1x,i1,2x,i1,2x,i1,1x,i1,1x,F7.5,F7.5,F8.5,F12.5)
 55   format(i2,2x,a7,1x,i1,2x,i2,1x,i1,1x,i1,1x,F7.5,F7.5,F8.5,F12.5)
 56   format(i2,2x,a7,1x,i2,1x,i1,2x,i1,1x,i1,1x,F7.5,F7.5,F8.5,F12.5)
 57   format(i2,2x,a7,1x,i2,1x,i2,1x,i1,1x,i1,1x,F7.5,F7.5,F8.5,F12.5)

 60   format(i1,3x,a7,1x,i1,2x,a1,2x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)
 61   format(i1,3x,a7,1x,i2,1x,a1,2x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)
 62   format(i2,2x,a7,1x,i1,2x,a1,2x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)
 63   format(i2,2x,a7,1x,i2,1x,a1,2x,i1,1x,a1,1x,F7.5,F7.5,F8.5,F12.5)

      end subroutine outvalues

!-----------------------------------------------------------------------
! SUBOURTINE FILENAME GENERATION
!-----------------------------------------------------------------------

      subroutine gaanalfilename(c,x,filename)

      implicit none

      integer(long), intent(in)     :: x,c
      character(len=80)             :: filename

      ilbl=index(filename,' ')-1
      if (x .eq. 1) then
         filename=filename(1:ilbl)//'.vcham'
      elseif (x .eq. 2) then
         filename=filename(1:ilbl)//'.log'
      endif

      return
      end subroutine gaanalfilename

!-----------------------------------------------------------------------
! SUBROUTINE READ LOG FILE
!-----------------------------------------------------------------------
      subroutine gareadlog(run,cpu,real,armsd,sdrmsd,brmsd) 

      implicit none

      integer(long)                                   :: ngen,npop,ngenga
      integer(long), intent(in)                       :: run
      real(dop), dimension(nruns,mxgen), intent(out)  :: armsd,brmsd,sdrmsd
      real(dop), dimension(nruns,mxgen,4), intent(out):: cpu,real

! Need to reed in new mutation values etc.      
      print*,'Reading log files'
 10   continue
      read(ilog,'(a)',end=100,err=1010) string
      print*,'string = ',string
      if (string(1:30) .eq. 'Genetic Algorithm optimisation') then
         read(ilog,50) npop,ngen
         print*,'Population,Generation = ',npop,',',ngen
         goto 20
      else
         goto 10
      endif
 20   read(ilog,'(a)',end=100,err=1010) string
      if (string .eq. 'Completed Genetic Algorithm Optimisation') then
         goto 30
      elseif (string(1:7) .eq. 'Summary') then 
           print*,'Reading summary'
           read(ilog,'(19x,i4)') ngenga
           read(ilog,'(36x,e16.8)') brmsd(run,ngenga)  
           read(ilog,'(36x,e16.8)') armsd(run,ngenga)   
           read(ilog,'(26x,e16.8)') sdrmsd(run,ngenga)  
           read(ilog,*)
           read(ilog,40) cpu(run,ngenga,1),cpu(run,ngenga,2),&
       cpu(run,ngenga,3),cpu(run,ngenga,4)   
           read(ilog,40) real(run,ngenga,1),real(run,ngenga,2),&
       real(run,ngenga,3),real(run,ngenga,4)   
           go to 20
        endif


 30   continue
      goto 150
 100  print*,'ERROR: Genetic Algorithm output not found'
 150  return
 

 40   format(17x,e10.4,4x,e10.4,4x,e10.4,4x,e10.4)
 50   format(20x,i3,35x,i3)
 1010 continue
      end subroutine gareadlog


!-----------------------------------------------------------------------
! SUBROUTINE OUTPUT SUMMARY
!-----------------------------------------------------------------------

      subroutine outsum(cpu,real,armsd,sdrmsd,brmsd) 

      implicit none

      integer(long)                                   :: c,n 
      real(dop)                                       :: abrmsd,aarmsd,asdrmsd,afcpu,afreal
      real(dop), dimension(nruns,mxgen), intent(in)   :: armsd,brmsd,sdrmsd
      real(dop), dimension(nruns,mxgen,4), intent(in) :: cpu,real

      abrmsd = 0
      aarmsd = 0
      asdrmsd = 0
! Calculate average of armsd,brmsd,sdrmsd
      do c=1,nruns
           abrmsd = abrmsd + brmsd(c,mxgen)
           aarmsd = aarmsd + armsd(c,mxgen)
           asdrmsd = asdrmsd + sdrmsd(1,mxgen) 
           afcpu = cpu(c,mxgen,1) + cpu(c,mxgen,2) + &
                cpu(c,mxgen,3) + cpu(c,mxgen,4) 
           afreal= real(c,mxgen,1) + real(c,mxgen,2) +& 
                real(c,mxgen,3) + real(c,mxgen,4) 
      enddo

      n=0
      do c=1,nruns
         if (nimportval(c) .gt. n) then
            n = nimportval(c)
         endif
      enddo

      abrmsd = abrmsd / nruns
      aarmsd = aarmsd / nruns
      asdrmsd = asdrmsd / nruns
      afcpu = afcpu / nruns
      afreal = afreal / nruns 
      write(iout,10) 'Analysis of GA tests'
      write(iout,*) 
      write(iout,20) 'Number of Generations = ',mxgen
      write(iout,21) 'Number of repeats (runs) = ',nruns
      write(iout,22) 'Number of significant values = ',n
      write(iout,*) 
      write(iout,30) 'Average final rmsd over all runs &
       & population = ', aarmsd
      write(iout,31) 'Average best rmsd over all runs = ',&
        abrmsd
      write(iout,32) 'Average over all runs Standard Deviation&
        of population = ',asdrmsd  
      write(iout,*) 
      write(iout,40) 'Average total cpu time over all runs = ',&
        afcpu
      write(iout,41) 'Average total system time over all runs = ',&
        afreal
      write(iout,*)  

 100  return

 10   format(a20) 
 20   format(a24,9x,i4)
 21   format(a27,6x,i4)
 22   format(a31,2x,i4)   

 30   format(a48,11x,e10.6)
 31   format(a34,25x,e10.6)  
 32   format(a57,2x,e10.6) 

 40   format(a39,5x,e10.6)  
 41   format(a42,2x,e10.6) 

 1010 end subroutine outsum

!-----------------------------------------------------------------------
! SUBROUTINE GAPLOT 
!----------------------------------------------------------------------_

      subroutine gaplot(data,type,cpu) 

      implicit none

      integer(long)                                   :: c,d,e
      integer(long), external                         :: slen
      integer(long), intent(in)                       :: type
      real(dop), dimension(nruns,mxgen), intent(in)   :: data
      real(dop), dimension(nruns,mxgen)               :: tcpu
      real(dop)                                       :: ymax
      real(dop), dimension(nruns,mxgen,4), intent(in) :: cpu

      ilbl=index(filein,'.')-1

      if (type .eq. 1) then
           gnufile=filein(1:ilbl)//'_armsd.gnu'
           psfile =filein(1:ilbl)//'_armsd.ps'
      elseif (type .eq. 2) then
           gnufile=filein(1:ilbl)//'_brmsd.gnu'
           psfile =filein(1:ilbl)//'_brmsd.ps'
      elseif (type .eq. 3) then
           gnufile=filein(1:ilbl)//'_sdrmsd.gnu'
           psfile =filein(1:ilbl)//'_sdrmsd.ps'
      else
           gnufile=filein(1:ilbl)//'_cpu.gnu'
           psfile =filein(1:ilbl)//'_cpu.ps'
      endif
!-----------------------------------------------------------------------
!      Open plot file
!        Input header
!-----------------------------------------------------------------------
      open(50,file=gnufile,form='formatted')
      call zeromxd(tcpu,nruns,mxgen)
      if (type .le. 3) then
          call ymax1(ymax,data)
          call defaultplot(ymax,type)
          call points(data)
      elseif (type .eq. 4) then
          do c=1,nruns
             do d=1,mxgen
                do e=1,4
                   tcpu(c,d) = tcpu(c,d) + cpu(c,d,e)
                enddo
             enddo
          enddo
          call ymax1(ymax,tcpu)
          call defaultplot(ymax,type)
          call points(tcpu) 
      endif

!-----------------------------------------------------------------------
! Close plot file
!-----------------------------------------------------------------------
      close(50)

!-----------------------------------------------------------------------
! create postscript file 
!-----------------------------------------------------------------------
      call excmd('chmod 700 '//gnufile(1:slen(gnufile)))
      call excmd('./'//gnufile(1:slen(gnufile)))

      return
      end subroutine gaplot


!-----------------------------------------------------------------------
! SUBROUTINE YMAX1
!-----------------------------------------------------------------------
      subroutine ymax1(ymax,data)

      implicit none

      integer(long)                                 :: c,d
      real(dop), intent(out)                        :: ymax
      real(dop), dimension(nruns,mxgen), intent(in) :: data

      ymax=0
      do c=1, nruns
         do d=1, mxgen
           if (data(c,d) .gt. ymax) then
              ymax = data(c,d)
           endif
         enddo
      enddo

      return
      end subroutine ymax1

!-----------------------------------------------------------------------
! SUBROUTINE DEFAULT PLOT
!    Puts in the formating for all plot files 
!-----------------------------------------------------------------------
      subroutine defaultplot(ymax,type)

      implicit none

      integer(long), external   :: slen
      integer(long)             :: c,sets,ll
      integer(long), intent(in) :: type
      real(dop)                 :: xmin,xmax,ymin
      real(dop), intent(in)     :: ymax
      character(len=80)         :: line
      xmin = 0
      xmax = mxgen
      ymin = 0

      write(50,'(a)') '#! /usr/bin/gnuplot -persist'
      write(50,'(a)')&
           'set term postscript portrait color solid "Helvetica" 24'
      write(50,'(a)') 'set output "'//psfile(1:slen(psfile))//'"'
      write(50,'(a)') 'set ylabel "RMSD"'
      write(50,'(a)') 'set xlabel "Generations"'
      write(50,'(a,f8.3,a,f8.3,a)') 'set xrange [',xmin,':',xmax,']'
      write(50,'(a,f8.3,a,f8.3,a)') 'set yrange [',ymin,':',ymax,']'

      if (type .eq. 1) then
        write(50,'(a)') 'set title "A plot showing the average RMSD &
       across all runs and population for each genetarion"'
        sets = nruns 
      elseif (type .eq. 2) then
        write(50,'(a)') 'set title "A plot showing the lowest RMSD &
       averaged across all runs for each generation"' 
        sets = nruns
      elseif (type .eq. 3) then
        write(50,'(a)') 'set title "A plot showing the standard &
       deviation of the RMSD across the population averaged across &
       all runs against generation"'
        sets = nruns
      elseif (type .eq. 4) then
        write(50,'(a)') 'set title "A plot showing the total cpu &
       time for each run against generation"'
        sets = nruns
      elseif (type .eq. 5) then
        write(50,'(a)') 'set title "A plot showing the average &
       RMSD (across the population), the lowest RMSD and standard& 
       deviation of the RMSD across the population all averaged &
       across all runs against generation."' 
        sets = 3
      endif

      write(50,'(a)') 'set format y "%.1f"'

      line= 'plot '
      ll=slen(line)

      do c=1,sets
         write(line(ll+1:),'(a)') ' "-" notitle w p lt 0 pt 7 ps 1 '
         ll=slen(line)
      enddo
      do c=1,sets
         write(line(ll+1:),'(a,i1,a)') ',"-" t "fit" w l lt ',1,' lw 3 '
         ll=slen(line)
      enddo

      write(50,'(a)') 

      return
      end subroutine defaultplot

!-----------------------------------------------------------------------
! SUBROUTINE POINTS 
!----------------------------------------------------------------------_

      subroutine points(data)

      implicit none

      real(dop)                                     :: tmp
      real(dop), dimension(nruns,mxgen), intent(in) :: data
      integer(long)                                 :: c,d

      do c=1,nruns
         do d=1,mxgen
            tmp=real(d)
            write(50,'(2f10.5)') tmp,data(c,d)
         enddo
         write(50,'(a)') 'e'
      enddo
      return
      end subroutine points


!-----------------------------------------------------------------------
! SUBROUTINE GAPLOT2
!----------------------------------------------------------------------_

      subroutine gaplot2(armsd,brmsd,sdrmsd)

      implicit none

      real(dop), dimension(nruns,mxgen), intent(in) :: armsd,brmsd,sdrmsd
      real(dop)                                     :: tmp,ymax
      real(dop), dimension(mxgen)                   :: parmsd,pbrmsd,psdrmsd
      integer(long)                                 :: c,d

      ilbl=index(filein,'.')-1
      gnufile=filein(1:ilbl)//'_all.gnu'
      open(50,file=gnufile,form='formatted')

      call zerovxd(parmsd,mxgen)
      call zerovxd(pbrmsd,mxgen)
      call zerovxd(psdrmsd,mxgen)
      do d=1,mxgen
          do c=1,nruns
             parmsd(d) = parmsd(d) + armsd(c,d)
             pbrmsd(d) = pbrmsd(d) + brmsd(c,d)
             psdrmsd(d) = psdrmsd(d) + sdrmsd(c,d)
          enddo
          parmsd(d) = parmsd(d) / nruns
          pbrmsd(d) = pbrmsd(d) / nruns
          psdrmsd(d) = psdrmsd(d) / nruns
      enddo

      print*,'Summed data'

      ymax=0
      do c=1,mxgen
         if (parmsd(c) .gt. ymax) then
            ymax = parmsd(c)
         endif
         if (pbrmsd(c) .gt. ymax) then
            ymax = parmsd(c)
         endif
         if (psdrmsd(c) .gt. ymax) then
            ymax = psdrmsd(c)
         endif
      enddo

      print*,'Found ymax'

      call defaultplot(ymax,5)

      print*,'written default data'

      do d=1,mxgen
         tmp=real(d)
         write(50,'(2f10.5)') tmp,parmsd(d)
      enddo
      write(50,'(a)') 'e'
      do d=1,mxgen
         tmp=real(d)
         write(50,'(2f10.5)') tmp,pbrmsd(d)
      enddo
      write(50,'(a)') 'e'
      do d=1,mxgen
         tmp=real(d)
         write(50,'(2f10.5)') tmp,psdrmsd(d)
      enddo
      write(50,'(a)') 'e'

      print*,'all done'

      close(50)

      return
      end subroutine gaplot2

      end program vcgaanal
