
      program vcplot

use decimal, only: long,dop,sip
use vcglobal
use global, only: c4,tiny
use versions
use maxv, only: maxdim,maxsta,maxset
use vchammod
use vcplotmod
use vchfitmod
use op1lib, only: zerovxd, zerovxi, zerovxl
use utils, only: vcerrormsg,rmblnk0,vchdefault,x2q,q2curv,curv2q,x2qf,&
                 rotatemodes,rotpairvec,q2x,transformq1
use symmetry, only: getnadd,getnswitch
use rddatamod
use iofiles, only: readguess
use invchammod
use logdat, only: message, routine

use module_interface

      implicit none

      integer(long), external                     :: slen
      integer(long)                               :: ilbl,ierr,j,i,m
      character(len=10)   :: cerr
      character(len=80)   :: titlename

      complex(dop), dimension(:), allocatable     :: mc
      real(dop), dimension(:), allocatable        :: mr
      integer(long), dimension(:), allocatable    :: mi
      logical(kind=4), dimension(:), allocatable  :: ml
      complex(sip), dimension(:), allocatable     :: ms
      real(sip), dimension(:), allocatable        :: mf
      logical                                     :: xpoints

! arrays to read in a Q0 coordinate
      integer(long), save  :: nval=0 , ival(10)
      real(dop), save      :: qval0(10)

!-----------------------------------------------------------------------
! set plot default values
!-----------------------------------------------------------------------
      call vcpldefault

!-----------------------------------------------------------------------
! read input (from command line)
! define output file etc.
!-----------------------------------------------------------------------
      call vcplinp

!-----------------------------------------------------------------------
! open log and data files
!-----------------------------------------------------------------------
      open(ilog,file='vcplot.log',form='formatted',status='unknown')

      if (plotfit .gt. 0) then
         ilbl=slen(guessfile)
         print*,'VCHAM data from file : ',guessfile(1:ilbl)
         open(iguess,file=guessfile,form='formatted',status='old',&
              iostat=ierr)              
         if (ierr .ne. 0) then
            routine = 'Vcplot'
            write(message,'(2a)') 'No guessfile found: ',&
                guessfile(1:ilbl)
            call vcerrormsg
         endif
      endif
      ilbl=slen(datfile)
      write(6,'(/,2a)') 'Data read from file : ',datfile(1:ilbl)
      open(idat,file=datfile,form='formatted',status='old',iostat=ierr)
      if (ierr .ne. 0) then
         routine = 'Vcplot'
         write(message,'(2a)') 'No infofile found: ',datfile(1:ilbl)
         call vcerrormsg
      endif

!-----------------------------------------------------------------------
! get maxdim, maxsta, maxset and then allocate memory
!-----------------------------------------------------------------------
      call dataset_dims

      if (maxdim .lt. 0) then
         routine='VCTRANS'
         message='Error reading dimensions from input file'
         call vcerrormsg
      endif
      if (maxsta .lt. 0) then
         routine='VCTRANS'
         message='Error reading no. states from input file'
         call vcerrormsg
      endif

      call alloc_vchammod
      call alloc_vcplotmod
      call alloc_vchfitmod

      call wrversion(ilog)

!-----------------------------------------------------------------------
! set default values
!-----------------------------------------------------------------------
      call vchdefault
      nmlab(:) = ' '

! mfit set so only datasets needed for plot included
      mfit(:)=0
      mfit(N)=1
      mfit(N2)=1

!-----------------------------------------------------------------------
! store any values defining a cut to the qval array
! default is 0.0 (i.f. FC)
!-----------------------------------------------------------------------
      q0val(:) = 0.0_dop
      if (nval .gt. 0) then
         print*,'Zero point has non-zero coordinates for :'
         do i=1,nval
            m=ival(i)
            q0val(m) = qval0(i)
            write(6,'(a,i5,a,f8.5)') 'Mode : ',m,'  Value :',q0val(m)
         enddo
      endif

!-----------------------------------------------------------------------
! get system information from info file
! normal modes are scaled and rotated in here!
!-----------------------------------------------------------------------
      print*,'Read dataset info'
      call dataset_info

!-----------------------------------------------------------------------
! determine the no. explicitely added terms
!-----------------------------------------------------------------------
      
      if (plotfit .gt. 0) then
         call getnadd(iguess)
         call getnswitch(iguess)
      endif

!	DEBUG ROTATION DATA
    if( lrotmodes) then 
        j = 1
        do while ( rotm(j,1) .gt. 0  )
           print*, 'j = ', j
           write(6,'(I3,3X,I3,3X,F15.8,3X,F10.8)') &
           rotm(j,1),rotm(j,2),rotang(j),rotmxfq(j)
           j = j + 1
        enddo
    endif    


!-----------------------------------------------------------------------
! assign memory 
!-----------------------------------------------------------------------
      call memvcham
      print*,'Memory assigned'

!-----------------------------------------------------------------------
! call routine
!-----------------------------------------------------------------------
      ilbl = 16*mcdim+8*mrdim+4*midim+4*mldim+8*msdim+4*mfdim
      write(ilog,'(/,a)') 'MEMORY ALLOCATED'
      write(ilog,'( 6(a,i10,/),a,i13,a,f10.2,a)')&
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' complex*8  : ', msdim,&
       ' real*4     : ', mfdim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

      if(allocated(mc))deallocate(mc)
      if(allocated(mr))deallocate(mr)
      if(allocated(mi))deallocate(mi)
      if(allocated(ml))deallocate(ml)
      if(allocated(ms))deallocate(ms)
      if(allocated(mf))deallocate(mf)

      cerr = 'vcplot_mc '
      allocate(mc(mcdim),stat=ierr)
      if (ierr > 0) go to 2000
      cerr = 'vcplot_mr '
      allocate(mr(mrdim),stat=ierr)
      if (ierr > 0) go to 2000
      cerr = 'vcplot_mi '
      allocate(mi(midim),stat=ierr)
      if (ierr > 0) go to 2000
      cerr = 'vcplot_ml '
      allocate(ml(mldim),stat=ierr)
      if (ierr > 0) go to 2000
      cerr = 'vcplot_ms '
      allocate(ms(msdim),stat=ierr)
      if (ierr > 0) go to 2000
      cerr = 'vcplot_mf '
      allocate(mf(mfdim),stat=ierr)
      if (ierr > 0) go to 2000

      call zerovxd(mr,mrdim)
      call zerovxi(mi,midim)

!----------------------------------------------------------------------
! read the ab-initio data file 
!----------------------------------------------------------------------
      print*,'Reading ab-initio data-file'
      rewind(idat)
      call read_dataset(mr(mrenergy),mr(mrxcoo),mr(mrqcoo),mr(mrder1),&
           mr(mrdcp),mr(mrder2),mr(mrtdip),mr(mrxder1),mr(mrxdcp),&
           mr(mrxder2),q0val)     
      close(idat)                                                            
      print*,'No. Datasets included : ',nsets

      if (abs(q_shift - 0.0_dop) .gt. tiny) then
         call shiftq(mr(mrqcoo))
      endif

   !  print*, 'AFTER DATASET'
    if( lrotmodes) then 
        j = 1
        do while ( rotm(j,1) .gt. 0  )
           print*, 'j = ', j
           write(6,'(I3,3X,I3,3X,F15.8,3X,F10.8)') &
           rotm(j,1),rotm(j,2),rotang(j),rotmxfq(j)
           j = j + 1
        enddo
    endif    

!----------------------------------------------------------------------
!     read the VCHAM parameters
!----------------------------------------------------------------------
!     print*, 'NMODES HERE = ', nmodes
      if (plotfit .ne. 0) then
         call readguess(iguess,mr(mrconst),mr(mrkappa),mr(mrlambda),&
              mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
              mi(miexpcoemask),mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
              mr(mrswitch),mr(mrcp2d))
        close(iguess)
      endif

    if( lrotmodes) then 
        j = 1
        do while ( rotm(j,1) .gt. 0  )
           print*, 'j = ', j
           write(6,'(I3,3X,I3,3X,F15.8,3X,F10.8)') &
           rotm(j,1),rotm(j,2),rotang(j),rotmxfq(j)
           j = j + 1
        enddo
    endif   
!  the only instance where we want to use the q2curv/curv2q routines is when
!  lpltcurv2n is true (else lnmcurv affects the pes routine) 
   if( .not. lpltcurv2n ) lnmcurv = .false.
 
!-----------------------------------------------------------------------
!      Open plot file
!-----------------------------------------------------------------------
      open(50,file=gnufile,form='formatted')
                                                                                
      if(plotstalast .lt. 0) plotstalast = nsta
      if(plotsta1st .gt. plotstalast) plotsta1st = plotstalast

!-----------------------------------------------------------------------
!      Write gnuplot data and ab initio points to plot file
!-----------------------------------------------------------------------
!     print*,'Writing gnuplot data'
      call subvcplot(mr(mrenergy),mr(mrqcoo),nmodes,nsets,mr(mrtdip))
      print*,'Finished writing gnuplot data'

!-----------------------------------------------------------------------
! Write VCHAM data and ab initio points to plot file
!-----------------------------------------------------------------------
      if (plotfit .ne. 0) then
         call plotvcham(mr,xpoints)
      endif

!-----------------------------------------------------------------------
! Close plot file
!-----------------------------------------------------------------------
      close(50)
                                                                                
!-----------------------------------------------------------------------
! Send to screen
!-----------------------------------------------------------------------

      call excmd('chmod 700'//gnufile(1:slen(gnufile)+1))
      call excmd('gnuplot '//gnufile(1:slen(gnufile)+1))
      if (.not.lplsurf) call excmd('ps2pdf '//psfile(1:slen(psfile)))
      call excmd(trim(pdfplt)//' '//psfile(1:slen(psfile)-3)//'.pdf'//' &')

      cerr = 'vcplot_...'
      deallocate(mc,mr,mi,ml,ms,mf,stat=ierr)
      if (ierr > 0) go to 2001
      call dealloc_vchammod
      call dealloc_vcplotmod
      call dealloc_vchfitmod

      stop

 2000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      stop

 2001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
      stop

      contains

!#######################################################################

      subroutine subvcplot(energy,qcoo,nmodess,nsetss,tdip)

      use utils, only: chkcoocut,chkcoovec

      implicit none

      integer(long)                         :: i,j,k,s,ll,ipnt,d,l,jlbl
      integer(long), external                          :: slen
      integer(long), intent(in)                        :: nmodess,nsetss
      integer(long), dimension(maxsta)                 :: naie
      real(dop), dimension(maxset,maxsta)              :: aie,qn
      character(len=2000)                              :: line
      character(len=c4)                                :: filename
      character(len=4)                                 :: fnum4
      real(dop), dimension(nsetss), intent(in)         :: energy
      real(dop), dimension(4,nsetss), intent(in)       :: tdip
      real(dop), dimension(nmodess,nsetss), intent(inout) :: qcoo
      real(dop)                                        :: p,rd,apoint,tmp
      real(dop), dimension(0:250,0:ncoo)               :: nmpoints

      call zerovxd(q,maxdim)
      ipnt = 0 
      if (ldiabcp) then
         plotsta1st=1
         plotstalast=1
      endif

!-----------------------------------------------------------------------
! ___ write gnuplot call and commands - data follows in same file,
! ___ as we are drawing from stdin
!-----------------------------------------------------------------------

      if (lplsurf) then
         write(50,'(a)') '#! /usr/bin/gnuplot'
      else
         write(50,'(a)') '#! /usr/bin/gnuplot -persist'
      endif

      if (.not.lplsurf) then
         write(50,'(a)') &
              'set term postscript portrait color solid "Helvetica" 16'
         write(50,'(a)') 'set output "'//psfile(1:slen(psfile))//'"'
      endif

      if (ltdip) then
         write(50,'(a)') 'set ylabel "Transition Dipole: magnitude [au]" font &
       &"Helvetica,18"'
     
      else if (ltdipx) then
         write(50,'(a)') 'set ylabel "Transition Dipole: x-component [au]" font &
       &"Helvetica,18"'

      else if (ltdipy) then
         write(50,'(a)') 'set ylabel "Transition Dipole: y-component [au]" font &
       &"Helvetica,18"'

      else if (ltdipz) then
         write(50,'(a)') 'set ylabel "Transition Dipole: z-component [au]" font &
       &"Helvetica,18"'
         
      else if (lplsurf) then
         write(50,'(a,i3,a)') 'set xlabel "mode ',n,'"'
         write(50,'(a,i3,a)') 'set ylabel "mode ',n2,'"'
         write(50,'(a)') 'set zlabel "energy [eV]"'

      else if (ldiabcp) then
         write(50,'(a,1x,i1,1x,a,1x,i1,1x,a)') 'set ylabel "Diabatic coupling between states',&
              dcpsta,'and',dcpsta1,'[eV]" font "Helvetica,18"'

      else
         write(50,'(a)') 'set ylabel "energy [eV]" font &
       &"Helvetica,18"'
      endif
      if (lxrange) then
         write(50,'(a,f8.3,a,f8.3,a)') 'set xrange [',xmin,':',xmax,']'
      endif
      if (lyrange) then
         write(50,'(a,f8.3,a,f8.3,a)') 'set yrange [',ymin,':',ymax,']'
      endif
!---
      if (.not. title) then
         titlename= " "
         jlbl = 1
      else
         jlbl=index(titlename,' ')-1
      endif
      if (lplsurf.and.lzrange) then
         write(50,'(a,f8.3,a,f8.3,a)') 'set zrange [',zmin,':',zmax,']'
      endif
      write(fnum4,'(i4.4)')n
      if (.not.lplsurf) then
         if(n.eq.n2)then

            write(50,'(2a)')  &
         'set title "mode #'//fnum4//' '//titlename(1:jlbl)//'" font ', &
                 '"Helventica,24"'
         else
            write(50,'(a,i4.4,a)') 'set title "diag #'//fnum4//'-#',N2,'"'
         end if
      endif
      if (ltdip.or.ltdipx.or.ltdipy.or.ltdipz) then
         write(50,'(a)') 'set format y "%.1e"'
      else
         write(50,'(a)') 'set format y "%.1f"'
      endif
      if (lkey) then
         write(50,'(a)') 'set key outside title "Legend" box'
      endif
      if (lnokey) then
         write(50,'(a)') 'unset key'
      endif

      write(50,'(a)') 'set cntrparam levels incr -100,1,100'
      write(50,'(a)') 'set contour'
      
      if (lplsurf) then
         line='splot'
      else
         line= 'plot '
      endif
!-----------------------------------------------------------------------
! plot command for ab initio excited state potentials -
!-----------------------------------------------------------------------
      if (.not.ldiabret.and..not.ldiabcp.and..not.lplsurf) then

         if (lformal) then
            ll=slen(line)
            write(line(ll+1:),'(a)') ' "-" notitle w p lt 0 pt 7 ps 1, '
            do i=plotsta1st+1,plotstalast
               ll=slen(line)
               write(line(ll+1:),'(a)') ' "-" notitle w p lt 0 pt 7 ps 1, '
            enddo
         else

            if ((lcolour .eqv. .true.) .and. (lkey .eqv. .true.)) then
               ll=slen(line)
               if (plotsta1st.eq.1) then
                  write(line(ll+1:),'(a)') ' "-" title "s0" w p lt 1 pt 7 ps 1 '
               else
                  i=1
                  j=plotsta1st-1
                  write(line(ll+1:),'(a12,i2,a10,i2,a10)') '"-" title "s',j,&
                       '" w p lt ',i,' pt 7 ps 1 '
               endif
               
               do i=plotsta1st+1,plotstalast
                  j=i-1
                  ll=slen(line)
                  write(line(ll+1:),'(a13,i2,a10,i2,a10)') ',"-" title "s',j,&
                       '" w p lt ',i,' pt 7 ps 1 '
               enddo
            else if(lcolour .eqv. .true.) then
               ll=slen(line)
               write(line(ll+1:),'(a)') ' "-" notitle w p lt 0 pt 7 ps 1 '
               do i=plotsta1st+1,plotstalast
                  j=i-1
                  ll=slen(line)
                  write(line(ll+1:),'(a20,i2,a10)') ',"-" notitle w p lt '&
                       ,j,' pt 7 ps 1 '
               end do
               
            else if (lkey .eqv. .true.) then
               ll=slen(line)
               if (plotsta1st.eq.1) then
                  write(line(ll+1:),'(a)') ' "-" title "s0" w p lt 0 pt 7 ps 1 '
               else
                  i=1
                  j=plotsta1st-1
                  write(line(ll+1:),'(a12,i2,a21)') '"-" title "s',j,&
                       '" w p lt 0 pt 7 ps 1 '
               endif
            
               do i=plotsta1st+1,plotstalast
                  j=i-1
                  ll=slen(line)
                  write(line(ll+1:),'(a13,i2,a21)') ',"-" title "s',j,&
                       '" w p lt 0 pt 7 ps 1 '
               enddo
            else
               ll=slen(line)
               write(line(ll+1:),'(a)') ' "-" notitle w p lt 0 pt 7 ps 1 '
               do i=plotsta1st+1,plotstalast
                  j=i-1
                  ll=slen(line)
                  write(line(ll+1:),'(a)') ',"-" notitle w p lt 0 pt 7 ps 1'
               enddo
            endif

! ___ plot corresponding orbital energies
            if (orbital .gt. 0 ) then
               do j=plotsta1st,plotstalast
                  if(j .gt. 0)then
                     ll=slen(line)
                     write(line(ll+1:),'(a)') &
                          ',"-" notitle w p lt 1 pt 7 ps 1 '
                  end if
               end do
               do j=plotsta1st,plotstalast
                  if(j .gt. 0)then
                     if(j .eq. plotstalast) then
                        ll=slen(line)
                        write(line(ll+1:),'(a)') &
                             ',"-" t "orbital data" w p lt 1 pt 7 ps 1 '
                     else
                        ll=slen(line)
                        write(line(ll+1:),'(a)') &
                             ',"-" notitle w p lt 1 pt 7 ps 1 '
                     end if
                  end if
               end do
            end if
            
         endif
         
      endif
      
!-----------------------------------------------------------------------
! now do lines of fits
!-----------------------------------------------------------------------
      if (plotfit .ne. 0) then
         ll=slen(line)
         if (lformal) then
            write(line(ll+1:),'(a)') '"-" w l lt -1 lw 1.5'
            do i=plotsta1st+1,plotstalast
               ll=slen(line)
               write(line(ll+1:),'(a)') ',"-" w l lt -1 lw 1.5'
            enddo
         else
            
            if (lplsurf) then
               write(line(ll+1:),'(a,i2,a)') '"-" t "fit 1" w l lt ',1,' lw 1'
            else if (ldiabret.or.ldiabcp) then
               write(line(ll+1:),'(a,i2,a)') '"-" t "fit 1" w l lt ',1,' lw 3'
            else
               write(line(ll+1:),'(a,i2,a)') ',"-" t "fit 1" w l lt ',1,' lw 3'
            endif
            
            do i=plotsta1st+1,plotstalast
               ll=slen(line)
               
               if (lplsurf) then
                  write(line(ll+1:),'(a,i2,a,i2,a)') &
                       ',"-" t "fit ',i,'" w l lt ',mod(i,10),' lw 1 '
               else
                  write(line(ll+1:),'(a,i2,a,i2,a)') &
                       ',"-" t "fit ',i,'" w l lt ',mod(i,10),' lw 3 '
               endif
            enddo
            
         endif
         
      endif
      
       if (lplsurf) then
            ll=slen(line)
            write(line(ll+1:),'(a)') '; pause -1'
       endif

      write(50,'(a)')  line(1:slen(line))
      write(50,*)

!----------------------------------------------------------------------
! scan the ab-initio data for appropriate points and write it to  
! a gnu file                  
! naie(s): no. of points for state s along cut 
! aie(k,s): energy of kth point for state s
! qn(k,s): coordinates of kth point for state s
!----------------------------------------------------------------------
      qmax=1.0_dop
      if (.not.ldiabret.and..not.ldiabcp.and..not.lplsurf) then
         do j=1,nsta
            naie(j)=0
         enddo
         
         if (lpltvec) then
            filename=vecfile
            open(51,file=filename,form='formatted',status='old')
            rewind(51)

!----------------------------------------------------------------------
! Read final elements
!----------------------------------------------------------------------
            do n=1,nmodes
               read(51,'(f16.6)') qvp(n)
            enddo
            
!----------------------------------------------------------------------
! Read initial elements
!----------------------------------------------------------------------
            read(51,*)
            do n=1,nmodes
               read(51,'(f16.6)') qvpini(n)
               tmp=qvp(n)-qvpini(n)
               tmp=abs(tmp)
               if (tmp.gt.0.0001) atvmd(n)=.true.
            enddo
         endif
         
         do j=1,nsetss
            if (lpltvec) then
               ismode=.true.
               call chkcoovec(qcoo(1,j),nmodes,ismode,atvmd,qvp,qvpini)
               
               if (ismode) then               
                  i=asta(j)
                  naie(i)=naie(i)+1
                  k=naie(i)
                  if (ltdip) then
                     aie(k,i)=tdip(1,j)
                     
                  else if (ltdipx) then
                     aie(k,i)=tdip(2,j)
                     
                  else if (ltdipy) then
                     aie(k,i)=tdip(3,j)
                     
                  else if (ltdipz) then
                     aie(k,i)=tdip(4,j)
                     
                  else                     
                     aie(k,i)=energy(j)
                  endif
                  do n=1,nmodes                    
                     if (atvmd(n)) then
                        p=(qvp(n)-qvpini(n))
                        p=p/100
                        rd=abs((qcoo(n,j)-qvpini(n)))
                        if (rd.gt.0.001_dop) rd=rd/p
                     endif
                  enddo
                  qn(k,i)=rd
                  qmax=max(qmax,abs(qn(k,i)))
               endif
        
         elseif(xpoints) then
!######################################################################
!GETS CARTESIAN POINT FROM XPOINT FILE (do for every point)
!GETS MATRIX COONM TO REPRESENT POINTS IN NM SPACE
!GETS THE POINTS IN NM SPACE
!COMPARES POINTS WITH THOSE IN DATABASE (QCOO)
!IF ISMODE THEN IT MEANS THAT THE POINTS MATCH AND GET THE INFO DATA
!######################################################################
            if (ipnt .eq. 0) call getxpoints(nmpoints,ncoo,ipnt)

            do d = 1, ipnt
              qvp(1:ncoo) = nmpoints(d,1:ncoo)
              if (lnmcurv) call q2curv(qvp(1:ncoo))

             do n = 1, nmodes
                if ( (qcoo(n,j) + 0.005 .gt. qvp(n) ) .and.&
                          ( qcoo(n,j) - 0.005 .lt. qvp(n)) ) then
                  ismode = .true.
                else
                  ismode = .false.
                  EXIT
                endif
             enddo
             if (ismode) then
                 i=asta(j)
                 naie(i)=naie(i)+1
                 k=naie(i)
                 if (ltdip) then
                    aie(k,i)=tdip(1,j)
                       
                 else if (ltdipx) then
                    aie(k,i)=tdip(2,j)
                    
                 else if (ltdipy) then
                    aie(k,i)=tdip(3,j)
                    
                 else if (ltdipz) then
                    aie(k,i)=tdip(4,j)
                  
                 else
                    aie(k,i)=energy(j)
                 endif
!       HERE IT DEFINES THE DISTANCE BETWEEN POINTS AS THE MAGNITUDE (LENGTH)
!       BETWEEN THEM (SEE GETXPONTS ROUTINE; nmpoints(0,0) = total length
!       nmpoints(l,0)= length between point l and l-1) 
!       NOTE: POINTS IN XPOINTS FILE HAVE TO BE GIVEN IN THE CORRECT SEQUENCE
!       AS THE PATH YOU ARE TRYING TO PLOT 
                 apoint = 0
                 do l = 1, d
                    apoint = apoint + nmpoints(l,0)
                 enddo
!               print*, 'apoint =',apoint, "length =", nmpoints(0,0)
                 apoint = apoint - (nmpoints(0,0)/2)
                 qn(k,i)=apoint
                 qmax=max(qmax,abs(qn(k,i)))
             endif
          enddo

         else
            ismode=.true.
            if( lpltcurv2n ) call curv2q(qcoo(1:nmodes,j))
            if(lunrot) call rotpairvec(qcoo(1:nmodes,j),rotm,-rotang) 

            ismode=.true.
            call chkcoocut(qcoo(1,j),N,N2,nmodes,ismode,xscl,negdiag)
            if (ismode) then
               i=asta(j)
               naie(i)=naie(i)+1
               k=naie(i)
               if (ltdip) then
                  aie(k,i)=tdip(1,j)

               else if (ltdipx) then
                  aie(k,i)=tdip(2,j)!*tdip(1,j)

               else if (ltdipy) then
                  aie(k,i)=tdip(3,j)!*tdip(1,j)

               else if (ltdipz) then
                  aie(k,i)=tdip(4,j)!*tdip(1,j)
               else
                  aie(k,i)=energy(j)
                  endif
                  qn(k,i)=qcoo(N,j)
                  qmax=max(qmax,abs(qn(k,i)))
               endif
            endif
         enddo

!-----------------------------------------------------------------------
! plot ab initio points
!
! N.B. If plotting diabatic surfaces, then do not plot points
!-----------------------------------------------------------------------
         do s=plotsta1st,plotstalast
            do i=1,naie(s)
               write(50,'(2f10.5)') qn(i,s),aie(i,s)
            enddo
            write(50,'(a)') 'e'
            call FLUSH(50)
         enddo

     endif

      return
      end subroutine subvcplot 

!#######################################################################

      subroutine plotvcham(mr,xpoints)

        use functions, only: vpes,diabpes

      implicit none

      integer(long)                           :: j,i,k,l,ipnt,ilbl,m
      real(dop), dimension(mrdim), intent(inout) :: mr
      real(dop), dimension(maxdim)            :: qpri,tq
      real(dop)                               :: e1,rd
      real(dop), dimension(nmodes)            :: stpvec
      real(dop), dimension(maxdim)            :: qcoo_old
      character(len=c4)                       :: filename,string
      real(dop), dimension(0:250,0:ncoo)      :: nmpoints
      real(dop)                               :: apoint 
      logical(kind=4)                         :: xpoints

!       Vectors
        ipnt = 0
        if (lpltvec) then
           filename=vecfile
           open(51,file=filename,form='formatted',status='old')
           rewind(51)

!-----------------/-----------------------------------------------------
! Read final elements
!----------------------------------------------------------------------
           do n=1,nmodes
              read(51,'(f16.6)') qvp(n)
           enddo
!----------------------------------------------------------------------
! Read initial elements
!----------------------------------------------------------------------
           read(51,*)
           do n=1,nmodes
              read(51,'(f16.6)') qvpini(n)
              j=nint(qvp(n)-qvpini(n))
              if (abs(j).gt.0.0001) atvmd(n)=.true.
           enddo
           
           do n=1,nmodes
              stpvec(n)=(qvp(n)-qvpini(n))
!              stpvec(n)=abs(qvp(n)-qvpini(n))
              stpvec(n)=stpvec(n)/100
           enddo
           call zerovxd(qpri,maxdim)
           do j=plotsta1st,plotstalast
              do i=0,100
                 do n=1,nmodes
                    qpri(n)=qvp(n)-i*stpvec(n)
                 enddo
                 currsta=j
                 if (ldiabret) then
                    currsta1=currsta
                    e1=diabpes(qpri,mr)
                 else if (ldiabcp) then
                    currsta=dcpsta
                    currsta1=dcpsta1
                    e1=diabpes(qpri,mr)
                 else
                    e1=vpes(qpri,mr,mi)
                 endif
                 e1 = min(e1,9999.0_dop)
                 do n=1,nmodes
                    if (atvmd(n)) then
                       rd=abs(qpri(n)-qvpini(n))
                       if (rd.gt.0.001) rd=rd/stpvec(n)
                    endif
                 enddo
                 write(50,'(2F12.5)') rd,e1
                 call FLUSH(50)
              enddo
              write(50,'(a)') 'e'
              call FLUSH(50)
           enddo
           write(50,*)

!-----------------------------------------------------------------------
!to get a cut through a set of arbitrary set of points (given in cartesian) and
!plotted using the hamiltonian model from .vcham, which will obiously need 
! all the right modes modelled to work (also need an info file with the right hessian)
!reads from file called xpoints
!each point is given in a geometry styled as molpro/gaussian (without the atom label at the begining and no commas) and a space (row) between each
! points
! I have not checked if your info file has those points, weather it plotts them
! or not. 
! note of warning: if you dont give enought points, vcplot wont show you what
! your hamiltonian model looks like, since the graph simply extrapolates between
! points. moral: do many points!!!!
!-----------------------------------------------------------------------
      elseif(xpoints) then
           call getxpoints(nmpoints,ncoo,ipnt)
        do j=plotsta1st,plotstalast
         call zerovxd(qpri,maxdim)
         currsta=j
         do k = 1, ipnt
           qpri(1:ncoo) = nmpoints(k,1:ncoo)
           if (lnmcurv) call q2curv(qpri(1:ncoo))
           if (ldiabret) then
             currsta1=currsta
             e1=diabpes(qpri,mr)
           else
             e1=vpes(qpri,mr,mi)     
           endif
           e1 = min(e1,9999.0_dop)
!--------------------------------------------------------
!nmpoints(k,0) has the length of the vector of points k -> k+1
!(nmpoints(0,0) has the total sum of all the length between points
!hence, the bit below makes it so that coordinate 0 coincides with 
!the middle of the sum of lengths
!--------------------------------------------------------
           apoint = 0
           do l = 1, k
             apoint = apoint + nmpoints(l,0)
           enddo
           apoint = apoint - nmpoints(0,0)/2
           write(50,'(2F12.5)') apoint, e1
           call FLUSH(50)

         enddo
         write(50,'(a)') 'e'
         call FLUSH(50)
        enddo
        write(50,*)

! 3D surfaces
        else if (lplsurf) then
! Read non-zero coordinates of the remaining 3N-8 dofs
           if (geomfile.ne.'') then
              open(52,file=geomfile,form='formatted',status='old')
              rewind(52)
10            continue
              read(52,*,end=20) string
              ilbl=index(string,'=')
              read(string(2:ilbl-1),*) m
              read(string(ilbl+1:len_trim(string)),*) q(m)
              goto 10
20            continue
           endif
! Plot surfaces
           ipnt=npnts/5
           ipnt=ipnt*nint(scale)
           qmax=xmax
           step=qmax*1.1
           step=step/dble(ipnt)
           qmax=ymax
           step2=qmax*1.1
           step2=step2/dble(ipnt)
           do i=plotsta1st,plotstalast
              currsta=i
              do j=-ipnt,ipnt               
                 q(n)=j*step
                 do k=-ipnt,ipnt
                    q(n2)=k*step2
                    if (ldiabret) then
                       currsta1=currsta
                       e1=diabpes(q,mr)
                    else if (ldiabcp) then
                       currsta=dcpsta
                       currsta1=dcpsta1
                       e1=diabpes(q,mr)
                    else                     
                       e1=vpes(q,mr,mi)
                    endif
                    write(50,'(3(x,e13.5))') q(n),q(n2),e1
                 enddo
                 write(50,*)
              enddo
              write(50,'(a)') 'e'
              call FLUSH(50)
           enddo
           write(50,*)

! Normal modes
        else
!-----------------------------------------------------------------------
! if step size has not been set, plot over range covered by ab initio 
! points (qmax is maximum q)
!-----------------------------------------------------------------------

!----------------------------------------------------
! For plots with no ab initio points
!-----------------------------------------------------------------------
           if (qmax.eq.1.or.ldiabret) then
              if (lxrange) then
                 qmax=xmax
              else
                 qmax=10
              endif
           endif
           
           if (step .lt. 0) then
              step = qmax*1.1
              step = step / dble(npnts)
           endif
           call zerovxd(q,maxdim)
!-----------------------------------------------------------------------
! any values defining a cut to the q array
! default is 0.0 (i.f. FC)
!-----------------------------------------------------------------------
            if (nval .gt. 0) then
               do i=1,nval
                  m=ival(i)
                  q(m) = qval0(i)
               enddo
            endif
               
!-----------------------------------------------------------------------
!       read the VC-Hamiltonian data and write it to the gnu file
!-----------------------------------------------------------------------
           do j=plotsta1st,plotstalast
              do i=-npnts,npnts
                 q(N)=i*step
                 if (negdiag) then
                    q(N2)=-i*xscl*step
                 else
                    q(N2)=i*xscl*step
                 endif

                 if (lqtransform) then
                    qcoo_old(1:nmodes) = q(1:nmodes)
                    call transformq1(q)
                 endif
                 
                 if (j .eq. 0) then
                    e1 = 0.5*freq0(N)*q(N)**2
                    if(N2.ne.N)then
                       e1 = e1 + 0.5*freq0(N2)*q(N2)**2
                    end if
                 else 
                    currsta=j
                    if (ldiabret) then
                       currsta1=currsta
                       e1=diabpes(q,mr)
                    else if (ldiabcp) then
                       currsta=dcpsta
                       currsta1=dcpsta1
                       e1=diabpes(q,mr)
                    else
                     if( lpltcurv2n ) then
                         if(lnmcurv .or. lunrot) tq = q
                         if(lunrot) call rotpairvec(tq,rotm,rotang)
                         if(lnmcurv)  call q2curv(tq)
                         e1=vpes(tq,mr,mi)
                     else
                         e1=vpes(q,mr,mi)
                     endif
                    endif
                 endif
                 e1 = min(e1,9999.0_dop)

                 if (lqtransform) then
                    q(1:nmodes) = qcoo_old(1:nmodes)
                 endif
                 write(50,'(2F12.5)') q(N),e1
                 call FLUSH(50)
              enddo
              write(50,'(a)') 'e'
              call FLUSH(50)
           enddo
           write(50,*)
        endif
         
        return
      end subroutine plotvcham
     
!#######################################################################
      
     subroutine getxpoints(nmpoints,ncoo,ipnt)

     implicit none

     integer i, j, iostatus
     integer               :: ipnt
     integer               ::  ncoo
     real(dop), dimension(0:250,0:ncoo)  :: nmpoints
     real(dop), dimension(0:250,0:ncoo)  :: xyzpoints
     character*150 string2

     do i = 0, ncoo
      do ipnt = 0, 250
        nmpoints(ipnt,i) = 0
        xyzpoints(ipnt,i) = 0
      enddo
     enddo
     xyzpoints(0,0) = 0 
     i = 1
     ipnt = 0
!######################################################
!get the points from file xpoints
!######################################################
     open(22,file='xpoints',form='formatted',status='old')
33   continue
        read(22,'(A)',IOSTAT=iostatus) string2
          string2 = trim(adjustl(string2))
!          print*, string2
          if( index (string2, '.') .ne.  0 ) then
           if( i .eq. 1) ipnt = ipnt + 1
           read(string2,*) (xyzpoints(ipnt,i),i = i, i + 2)
!           print*, xyzpoints(ipnt,i-3), xyzpoints(ipnt,i-2), xyzpoints(ipnt,i-1)
          endif
          if((index(string2, '.') .eq.0 ) .or.&
          (iostatus .eq. -1) ) then
                if( i .ne. 1) then
!                 if(.not.lalleig) &
!                 print*, 'coonm = ', coonm(1,1:nmodes)
                 call x2q(nmpoints(ipnt,1:ncoo),xyzpoints(ipnt,1:ncoo),xcoo0,coonm,nmodes,ncoo,maxdim)
!                 if(lalleig) &
!                 call x2qf(nmpoints(ipnt,1:ncoo),xyzpoints(ipnt,1:ncoo),xcoo0,coonm,nmodes,ncoo,maxdim,freq0)
                  if (ipnt .gt. 1) then
!######################################################
! in nmpoints(ipnt,0) i put the length (magnitude) between points 1 and 2, 2
! and 3 etc...
!in nmpoints(1,0) i put the total length of all the line paths
!######################################################
                    nmpoints(ipnt,0) = 0
                    do j = 1, ncoo 
                      nmpoints(ipnt,0) = nmpoints(ipnt,0) +&
                      (nmpoints(ipnt,j) - nmpoints(ipnt-1,j))**2
                    enddo
                  else
                     nmpoints(ipnt,0) = 0
                  endif
                 nmpoints(ipnt,0) = sqrt(nmpoints(ipnt,0))
!                 print*, 'DEBUG length of ', ipnt, 'and ',ipnt-1, ' ',nmpoints(ipnt,0)
                 i = 1
                endif
               if(iostatus .eq. -1) then
                 do i = 1, ipnt
                  nmpoints(0,0) =&
                  nmpoints(0,0) + nmpoints(i,0) 
                 enddo
                 close(22)
                 RETURN
                endif
          endif
     goto 33

     end subroutine getxpoints

!#######################################################################

      subroutine vcpldefault

      implicit none

      lpoo = .false.
      xmin=-999.0
      xmax= 999.0
      ymin=-999.0
      ymax= 999.0
      lxrange=.false.
      lyrange=.false.
      lzrange=.false.
! ___ no weighting == equalweight
      weighttype=3
              
      step = -999.0

      ltdip=.false.
      ltdipx=.false.
      ltdipy=.false.
      ltdipz=.false.
      lkey=.false.
      lpltvec=.false.
      lcolour=.false.
      ldiabret=.false.
      ldiabcp=.false.
      xpoints=.false.
      lnokey=.false.
      lpltcurv2n=.false.
      lunrot=.false.
      vecfile='nmvector'
      lplsurf=.false.
      geomfile=''
      lformal=.false.
      scale=1.0

      return
      end subroutine vcpldefault

!#######################################################################

      subroutine vcplinp

      implicit none

      integer(long)           :: i,iarg,ilbl,jlbl,c
      character(len=80)       :: string,customname
      logical(kind=4)         :: lguess
      integer(long), external :: myiargc 

      guessfile=' '
      N = 0
      N2 = 0
      xscl = 1.0
      negdiag = .false.

      orbital = 0
      orbofs = 0.0
      plotfit = 1
      plotsta1st = 1
      plotstalast = -1
      neps = 0
      nsets=0

! choose a looser cut tolerance for fitting
      cuttol = 0.05

      iarg = myiargc()
      if (iarg .lt. 1) goto 200

      i = 0
  100 i=i+1
      call mygetarg(i,string)
!-----------------------------------------------------------------------
! options start with -
!-----------------------------------------------------------------------
      if (string(1:1) .eq. '-') then

         if(string .eq. '-si')then
            i=i+1
            call mygetarg(i,string)
            read(string,*) plotsta1st

         else if(string .eq. '-sf')then
            i=i+1
            call mygetarg(i,string)
            read(string,*) plotstalast

         else if(string .eq. '-n')then
            i=i+1
            call mygetarg(i,string)
            read(string,*) n

         else if(string .eq. '-n2')then
            i=i+1
            call mygetarg(i,string)
            read(string,*) n2
            negdiag = .false.

         else if(string .eq. '-nn2')then
            i=i+1
            call mygetarg(i,string)
            read(string,*) n2
            negdiag = .true.

         else if(string .eq. '-xscl')then
            i=i+1
            call mygetarg(i,string)
            read(string,*) xscl

         else if(string .eq. '-tol')then
            i=i+1
            call mygetarg(i,string)
            read(string,*) cuttol
         else if(string .eq. '-pdfplt')then
            i=i+1
            call mygetarg(i,string)
            read(string,*) pdfplt
         else if(string .eq. '-cusname')then
            cstmname = .true.
            i=i+1
            call mygetarg(i,string)
            read(string,*) customname
         else if(string .eq. '-title')then
            title = .true.
            i=i+1
            call mygetarg(i,string)
            read(string,*) titlename
            call rmblnk(titlename)
         else if(string .eq. '-qval')then
            nval = nval+1
            if (nval .gt. 10) then
               routine='vcplinp'
               message='A Maximum of 10 values can be given'
               call vcerrormsg
            else
               i=i+1
               call mygetarg(i,string)
               read(string,*) ival(nval)
               i=i+1
               call mygetarg(i,string)
               read(string,*) qval0(nval)
            endif

         else if(string .eq. '-ground')then
            plotsta1st = 0

         else if(string .eq. '-tdip')then
            ltdip=.true.

         else if(string .eq. '-xtdip')then
            ltdipx=.true.

         else if(string .eq. '-ytdip')then
            ltdipy=.true.
            
         else if(string .eq. '-ztdip')then
            ltdipz=.true.

         else if(string .eq. '-c2n')then
            lpltcurv2n=.true.

         else if(string .eq. '-unrot')then
            lunrot=.true.

         else if(string .eq. '-xpoints')then
            xpoints=.true.
         else if(string .eq. '-poo')then
            lpoo=.true.


         else if(string .eq. '-step')then
            i=i+1
            call mygetarg(i,string)
            read(string,*)step

         else if(string .eq. '-xrange') then
            lxrange=.true.
            i=i+1
            call mygetarg(i,string)
            read(string,*)xmin
            i=i+1
            call mygetarg(i,string)
            read(string,*)xmax

         else if(string .eq. '-yrange') then
            lyrange=.true.
            i=i+1
            call mygetarg(i,string)
            read(string,*)ymin
            i=i+1
            call mygetarg(i,string)
            read(string,*)ymax

         else if(string .eq. '-zrange') then
            lzrange=.true.
            i=i+1
            call mygetarg(i,string)
            read(string,*)zmin
            i=i+1
            call mygetarg(i,string)
            read(string,*)zmax
         else if (string.eq.'-eshift') then
            i=i+1
            call mygetarg(i,string)
            read(string,*) e_shift

         else if (string.eq.'-qshift') then
            i=i+1
            call mygetarg(i,string)
            read(string,*) q_shift

         else if(string .eq. '-o') then
            orbital = 1

         else if (string .eq. '-orbofs') then
            i=i+1
            call mygetarg(i,string)
            read(string,*)orbofs

         else if(string .eq. '-nofit') then
            plotfit = 0

         else if(string .eq. '-key') then
            lkey=.true.
            
         else if(string .eq. '-nokey') then
            lnokey=.true.

         else if(string .eq. '-colour') then
            lcolour=.true.

         else if(string .eq. '-groundonly')then
            plotstalast = 0

         else if(string .eq. '-vec')then
            lpltvec=.true.

         else if(string .eq. '-vecfile')then
            lpltvec=.true.
            i=i+1
            call mygetarg(i,string)
            read(string,*) vecfile

         else if(string .eq. '-geom') then
            i=i+1
            call mygetarg(i,string)
            read(string,*) geomfile
            
         else if(string .eq. '-diab')then
            ldiabret=.true.

         else if(string .eq. '-diabcp') then
            ldiabcp=.true.
            i=i+1
            call mygetarg(i,string)
            read(string,*) dcpsta
            i=i+1
            call mygetarg(i,string)
            read(string,*) dcpsta1

         else if(string .eq. '-3d') then
            lplsurf=.true.
            i=i+1
            call mygetarg(i,string)
            read(string,*) n
            i=i+1
            call mygetarg(i,string)
            read(string,*) n2

         else if(string .eq. '-scale') then
            i=i+1
            call mygetarg(i,string)
            read(string,*) scale

         else if(string .eq. '-formal')then
            lformal=.true.

         else if (string .eq. '-h' .or. string .eq. '-?') then
            write(6,'(1x,a)')&
       'Purpose : To plot the output from VCTRANS and VCHFIT.'
            write(6,'(1x,2a)')&
               'Usage   : vcplot [options]','[arguments].'
            write(6,'(/,a)') ' Options : '
            write(6,'(a)')&
            ' -si SI: SI is the lowest adiabatic state to be plotted '
            write(6,'(a)')&
            ' -sf SF: SF is the highest adiabatic state to be plotted '
            write(6,'(a)')&
            ' -n N: The PES is plotted along mode no. N '
            write(6,'(a)')&
            ' -n2 N2: The PES is plotted along the diagonal N / N2'
            write(6,'(2a)')&
            ' -nn2 N2: The PES is plotted along the negative diagonal N ', &
                   '/ -N2 and -N / N2'
            write(6,'(2a)')&
            ' -qval m R: The PES is plotted with mode m at a value of R '
            write(6,'(a)')&
            ' -xscl X: The diagonal N / N2 has a scaling factor of X'
            write(6,'(a)')&
            ' -tol X: The tolerance for selecting points is X (default 0.05)'
            write(6,'(a)')&
            ' -pdfplt S: Program S is used to display output (default mupdf)'
            write(6,'(a)')&
            ' -ground: The ground-state is also plotted'
            write(6,'(a)')&
            ' -groundonly: Only the ground-state is plotted'
            write(6,'(a)')&
            ' -step X: The step between points is X'
            write(6,'(a)')&
            ' -xrange XI XF: The Xrange of the plot is XI to XF'
            write(6,'(a)')&
            ' -yrange YI YF: The Yrange of the plot is YI to YF'
            write(6,'(a)')&
            ' -eshift X: The energy of the surfaces is shifted by X'
            write(6,'(2a)')&
            ' -o: The orbital energies are plotted in addition to the ',&
            'state energy (OVGF)'
            write(6,'(a)')&
            ' -orbofs X: The orbital energies are offset by X'
            write(6,'(a)')&
            ' -nofit: The model surfaces are not plotted'
            write(6,'(a)')&
            ' -diab: The diabatic potentials are plotted ' 
            write(6,'(a)')&
            ' -diabcp: The diabatic couplings are plotted ' 
            write(6,'(a)')&
            ' -vec : Plots a vector defined in nmvector'
            write(6,'(a)')&
            ' -key: A key is placed in the top right of the plot'
            write(6,'(a)')&
            ' -colour: The points on the graph are coloured'
            write(6,'(a)')&
            ' -h, -?: This help text'
            write(6,'(a)')&
            ' -diab: Plots diabatic curves'
            write(6,'(a)')&
            ' -diabcp X Y: Plots diabatic couplings between states X and Y'
            write(6,'(a)')&
            ' -3d n1 n2: Plots surface for normal modes n1 and n2'

!            write(6,*)
!            write(6,'(/,a)') 'Optional Arguments: '
            write(6,'(2a)')&
            ' filename: The name of the file containing the model OR',&
              ' the database'
            write(6,'(2a)')&
            ' -cusname: custom output name of the files without extension '
            write(6,'(2a)')&
            ' -title: title added to original title, no spaces allowed '
            write(6,'(a)')&
            ' n: Mode along which the PES is plotted (see -n N above)'
            write(6,'(2a)')&
            ' n2: The PES is plotted along the N / N2 diagonal',&
             '(see -n2 N" above)'
            write(6,'(2a)')&
            ' nn2: The PES is plotted along the N / -N2 diagonal',&
             '(see -nn2 N" above)'
            write(6,'(a)')&
            ' xscl: The diagonal N / N2 is scaled by xscl',&
            '(see -xscl X above)'
           stop

         else
            write(6,'(2a)') ' unknown option : ', string
            stop
         end if

! read next option
         go to 100

!-----------------------------------------------------------------------
! 1 argument is filename
!-----------------------------------------------------------------------
      else if (i .eq. iarg) then
         guessfile=string

!-----------------------------------------------------------------------
! 2 arguments are filename and mode
!-----------------------------------------------------------------------
      else if (i .eq. iarg-1) then
         guessfile=string
         i=i+1
         call mygetarg(i,string)
         read(string,*) N

!-----------------------------------------------------------------------
! 3 arguments are filename and 2 modes
!-----------------------------------------------------------------------
      else if (i .eq. iarg-2) then
         guessfile=string
         i=i+1
         call mygetarg(i,string)
         read(string,*) N
         i=i+1
         call mygetarg(i,string)
         read(string,*) N2

!-----------------------------------------------------------------------
! 4 arguments are filename and 2 modes and scaling for diagonal
!-----------------------------------------------------------------------
      else if (i .eq. iarg-2) then
         guessfile=string
         i=i+1
         call mygetarg(i,string)
         read(string,*) N
         i=i+1
         call mygetarg(i,string)
         read(string,*) N2
         i=i+1
         call mygetarg(i,string)
         read(string,*) xscl

      endif
  200 continue
      if (slen(guessfile) .eq. 0) then
         print*,'Enter Filename'
         read(5,'(a)') guessfile
      endif

      if (lpltvec .or. xpoints ) then 
         go to 23
      else if (n .eq. 0) then
         write(6,'(a)') 'Enter no. of mode to be ploted'
         read(5,*) n
      endif
!-----------------------------------------------------------------------
! Check n and n2 set with n < n2
! ___ if no diagonal, set n2 to same as n
!-----------------------------------------------------------------------
      if (n .eq. 0) then
         print*,'Specify mode for plot :'
         read(5,*) N
! ______ if no diagonal, set n2 to same as n
         if (xscl .eq. 0.0) then
            N2 = N
            xscl = 1.0
         end if
      endif

      if (N2 .eq. 0) n2=n

      if (N .gt. N2) then
         i = N2
         N2 = N
         N = i
      end if

   23 continue
!-----------------------------------------------------------------------
! set up output file names
!-----------------------------------------------------------------------
      ilbl=index(guessfile,' ')-1
      c = ilbl-4
      if (guessfile(c:ilbl) .ne. '.info') then
          guessfile=guessfile(1:ilbl)//'.vcham'
      endif
      inquire(file=guessfile,exist=lguess)
      if (.not. lguess) then
         print*,'Guess file does not exist :',guessfile
         stop
      endif

      if (lpltvec) then
         write(string,'(a4)') 'vec' 
      else if (n2 .ne. n) then
         write(string,'(i5,a1,i5)') n,'_',n2
      else  if (cstmname) then
         write(string,'(a20)') customname
      else
         write(string,'(i5)') n
      endif
      call rmblnk(string)
      jlbl=index(string,' ')-1
      ilbl=index(guessfile,'.')-1
!---      
      if (cstmname) then
        write(filestem,*) string(1:jlbl)
      else
        write(filestem,*) guessfile(1:ilbl)//'_'//string(1:jlbl)
      endif
      print*,filestem      
      ilbl=index(filestem,' ')
      if (ilbl .eq. 1) then
         filestem(1:79)=filestem(2:80)
      endif
      ilbl=index(filestem,' ')-1
      gnufile=filestem(1:ilbl)//'.gnu'
      psfile=filestem(1:ilbl)//'.ps'

!-----------------------------------------------------------------------
! if reading info file, set datfile to input file name
! else get datfile from guessfile
! also get other information from guessfile 
!-----------------------------------------------------------------------
      ilbl=index(guessfile,'.')-1
      if (guessfile(ilbl+2:ilbl+5) .eq. 'info') then
         plotfit=0
         datfile=guessfile
         call rmblnk0(datfile)
         guessfile='  '
      else
         open(iguess,file=guessfile,form='formatted',status='old')
   10    continue
         read(iguess,'(a)',end=15) string
         if (string(1:14).eq.'# Data file : ') then
            datfile=string(14:80)
            call rmblnk0(datfile)
         else if (string(1:21).eq.'# High order terms : ') then
            read(string(22:70),*) highord,nhigher
         endif
         go to 10
   15    continue
         close(iguess)
      endif

      close(iin)

! Only transition dipoles from the ground state are written to the
! VCTRANS info file, so remove state 1 from the plotting if 
! transition dipoles are being plotted.
      if (ltdip.or.ltdipx.or.ltdipy.or.ltdipz) then
         if (plotsta1st.eq.1.and.plotstalast.gt.plotsta1st&
              .or.plotsta1st.eq.1.and.plotstalast.lt.0) &
              plotsta1st=2
      endif


      return
    end subroutine vcplinp

!#######################################################################

    subroutine shiftq(qcoo)

    implicit none

    integer(long)                                     :: m,n
    real(dop), dimension(nmodes,nsets), intent(inout) :: qcoo

    do n=1,nsets
       do m=1,nmodes
          qcoo(m,n) = qcoo(m,n) - q_shift
       enddo
    enddo

    end subroutine shiftq

  end program vcplot


