      program vcpnt

use decimal, only: long,dop
use vcglobal
use global, only:pi, keyword,keyorig,maxkey,lc,lwarn,tiny
use maxv, only: maxdim,maxsta,maxset
use keyunitsmod
use invchammod
use vchammod
use vcpntmod
use dd_db, only: getdbnrec
use db_sql, only: dbopen, dbclose, dbrdrefdb

use utils, only: q2x,vcerrormsg,vccntrl,vcwrline,vchdefault,&
                 rotatex,rotatey,rotatez,qvec,qvec2,dihed,angel,&
                 setintcoo,rotatemodes,x2poly,poly2x,q2xf,qvec2f,&
                 rotrans,curv2q,q2curv,nm2dihed,uppercase

use iofiles, only: rdfile0,cpfreq
use op1lib,  only: zerovxd
use ioqc,    only: nm2xmat,nm2xmat2
use readqc,  only: nmorthof
use functions, only: iscurvnm
use logdat, only: message, routine, inptit

      implicit none

      integer(long) :: n,tag

      integer(long), external            :: slen
      character(len=80)                  :: fileout1,filename
      logical(kind=4)                    :: lrde0
      integer(long)                      :: i,j,iau
      logical(kind=4)                    :: lsocdegen

!-----------------------------------------------------------------------
! Initialise calculation 
! get input file name, set maxdim and then allocate memory
!-----------------------------------------------------------------------
      call initvcham(1)
         !Quick fix to make vcpnt work without upsetting vcham overall
         nsta = -999 !This was the old default value
      if (maxdim .lt. 0) then
         routine='VCPNT'
         message='Error reading dimensions from input file'
         call vcerrormsg
      endif
! default maxsta needed for general memory array allocation
      maxsta=2

      call alloc_vcpntmod
      call alloc_vchammod

!-----------------------------------------------------------------------
! set default values
!-----------------------------------------------------------------------
      call vchdefault
      call vcpdefault

!-----------------------------------------------------------------------
! read input file
! define output file etc.
!-----------------------------------------------------------------------
      call vcpinp(tag)

      if (lwarn) then
         routine = 'VCPNT'
         message = 'Finished with input error. See log file'
         call vcerrormsg
      endif

!-----------------------------------------------------------------------
! get information from normal mode file
!-----------------------------------------------------------------------
      if (lsubpnt.eqv..true.) then
         iau=1
         lsocdegen=.true.
         call rdfile0(.false.,0,iau,lsocdegen)

         sysatsym=atsym(tag*3)

         do n=1,3
            xsys0(n)=xcoo0((tag*3)-3+n)
         enddo
         do n=1,ncoo
            parxcoo0(n)=xcoo0(n)
         enddo
         nmodes=nmodes-3
         ncoo=ncoo-3
         file0=subfile0
      endif

      lrde0=.false.
      iau=1
      lsocdegen=.true.
      call  rdfile0(lrde0,0,iau,lsocdegen)

      if (.not. lrdfreq0) call cpfreq(nmlab,nmlab1,freq0,freq1,nmodes)

      if (lalleig .and. .not. lnmcurv) then
         call rotrans(nmcoo,maxdim,nmodes,ncoo,mass,xcoo0)
      endif

      if (lrotmodes) call rotatemodes(nmcoo,freq0,rotm,rotang,rotmxfq)
!-----------------------------------------------------------------------
! orthogonalise normal modes
!-----------------------------------------------------------------------
      if (lalleig .and. .not. lnmcurv) then
         call nmorthof(nmcoo,maxdim,ncoo,ilog)
      endif

!-----------------------------------------------------------------------
! create the transformation matrices
!-----------------------------------------------------------------------
      iau=1
      if (lalleig) then
         if(.not. lnmcurv) call rescaleq
         call nm2xmat2(nmcoo,coonm,mass,ncoo,maxdim)
      else 
         call nm2xmat(nmcoo,coonm,iatsym,freq0,iau,ncoo,maxdim,nmodes,ilog)
      endif
      natm=ncoo/3
      

!       DEBUG ROTATION DATA
!    if( lrotmodes) then 
!        j = 1
!        do while ( rotm(j,1) .gt. 0  )
!           write(6,'(I3,3X,I3,3X,F15.8,3X,F10.8)') &
!           rotm(j,1),rotm(j,2),rotang(j),rotmxfq(j)
!           j = j + 1
!        enddo
!    endif    



      if (logaussian) write(6,'(a)') 'Output format : GAUSSIAN'
      if (logamess) write(6,'(a)') 'Output format : Gamess-UK'
      if (loxyz) write(6,'(a)') 'Output format : XYZ'
      if (lmolpro) write (6,'(a)') 'Output format : Molpro'
      if (ldalton) write (6,'(a)') 'Output format : Dalton'
      if (lturbomole) write (6,'(a)') 'Output format : Turbomole'
      if (lgamessus) write(6,'(a)') 'Output format : Gamess-US'
      if (loqchem) write(6,'(a)') 'Output format : Q-chem'
      if (lmolcas) write (6,'(a)') 'Output format : Molcas'

      if (ldd) write(6,'(a)') 'Taking geometries from a Direct Dynamics Database'

!----------------------------------------------------------------------
! open the abinitio input data files
!----------------------------------------------------------------------
      if (lop) then
         filename=fileop
         open(iop,file=filename,form='formatted',status='old',err=999)
      endif
      if (lcl) then
         filename=filecl
         open(icl,file=filename,form='formatted',status='old',err=999)
      endif
      if (lgo) then
         open(igo,file='go.com',form='formatted',status='unknown')
         call excmd('chmod 700 go.com')
         if (lgoadd) then
   10       continue
            read(igo,'(1x)',end=20) 
            go to 10
   20       continue
            backspace(igo)
         endif
      endif 

!----------------------------------------------------------------------
! If we are using polyspherical coordinates, then determine the 
! reference coordinates
!----------------------------------------------------------------------
      if (lpoly) call x2poly(xcoo0,pcoo0,ncoo,nmodes,patm,mass,vcon)

      if(lnmcurv) then
       j = 1
       do while (nmcurvtype(j) .ne. 0)
         if(nmcurvtype(j) .eq. 1) then
           if(cmode .eq. nmcurv(j,2) .or. cmode .eq. nmcurv(j,3))  xincr = xincr*(pi/180)
           if(cmode1 .eq. nmcurv(j,2) .or. cmode1 .eq. nmcurv(j,3)) xincr1 = xincr1*(pi/180)
         endif 
         j = j + 1
       enddo
      endif 
!-----------------------------------------------------------------------
! open output file
! get Q coordinates
! generate X coordinates and output
!-----------------------------------------------------------------------

      write(6,'(/,a)') '*** Generating coordinates ***'
      call makecut

      if (lop) close(iop)
      if (lcl) close(icl)
      if (lgo) close(igo)
      close(ilog)
      stop

  999 write(6,'(2a)') 'ERROR : file not found : ',filename

      contains

!#######################################################################
        
      subroutine makecut

      implicit none

      if (lcut) then
         call cut1d
      else if (lvec) then
         call cutvec
      else if (lrot) then
         call cutrot
      else if (ldihed) then
         call cutdihed
      else if (lbloc2) then
         call cut2d
      else if (lbloc3) then
         call cut3d
      else if (ldiag) then
         call cutdiag
      endif

      return
      end subroutine makecut

!#######################################################################
        
        subroutine cut1d
          
          implicit none
          
          integer(long) :: p,p1,dim,m,mi,mf,i,j,k,l,l1,t,t1,m1
          real(dop)                    :: pi, n0
          real(dop), dimension(maxdim) :: inc,tcoo,q1,xcoo
          real(dop)                    ::qtheta,theta0
          

          pi = 3.1415926535897931    

          if (lcutall) then
             mi=1
             mf=nmodes
          else
             mi=cmode
             mf=cmode
          endif
          
          call zerovxd(q1,maxdim)

          do m=mi,mf
             cmode=m             

             do p=-ndat,ndat
                if (p .eq. 0 .and. .not. lq0) go to 100
                dim=1
                call getfilename(p,0,0,dim,fileout,fileout1,&
                     cmode,cmode1,cmode2,set,lmolpro,ldalton,lturbomole,&
                     lmolcas)
                if (lgo) call wrgo(igo,subcmd,fileout1,iin,lgocmd)
                open(iout,file=fileout1,form='formatted',&
                     status='unknown')
                write(6,'(2a)') 'Output file : ',fileout1(1:60)
                
                call zerovxd(q1,maxdim)
! move to any specific coordinates unless curvilinear (done at end)
                do i=1,nval
                   m1=ival(i)
                   if (m1 .eq. cmode) then
                      routine = 'Cut1d'
                      message = &
                           'Cannot specify a qval and a cut for the same mode'
                      call vcerrormsg
                   else if (nm2curv(m1) .eq. 0) then
                      q1(m1) = qval0(m1)
                   endif
                enddo
                
                if (.not. lreplace) then
! If in polyspherical coordinates.
                   if (lpoly) then
                      do i=1,nmodes
                         q1(i)=pcoo0(i)
                      enddo
                      q1(cmode)=q1(cmode) + dble(p*xincr)
                      call poly2x(xcoo,xcoo0,q1,ncoo,nmodes,patm,mass,vcon)
                      q1(cmode)=x0 + dble(p*xincr)

                   else if (lnmcurv .and. nm2curv(cmode) .eq. 2) then
! Dihedral
                      qtheta= dble(p*xincr)
                      xcoo=xcoo0
                      call nm2dihed(xcoo,qtheta,xcoo0,theta0)
                     
! Chris Robertson's spherical polars
               !    else if (lnmcurv) then
               !      if( lnmcurv) q1(cmode)= p*xincr
               !      if (lnmcurv) call curv2q(q1)

                   else
                     q1(cmode)=x0 + dble(p*xincr)

                     if (fmode .gt. 0) then
                        q1(fmode) = fx0
                     endif

                     if (lalleig) then
                       call q2xf(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                     else
                       call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                     endif

! move to any specific curvilinear coordinates
                      do i=1,nval
                         m1=ival(i)
                         if (nm2curv(m1) .eq. 2) then
                            qtheta= qval0(i)
                            call nm2dihed(xcoo,qtheta,xcoo0,theta0)
                         endif
                      enddo
                   endif
                else
                   call replacemode(p,p1)
                endif
                
                call comout(iout,xcoo,fileout1,tag)
                close(iout)
                
                call excmd('chmod 700 '//fileout1(1:slen(fileout1)))
! If using Turbomole, create a directory and move the output file to 'coord' in that
! directory. Each Turbomole energy calculation needs its own directory. 
                if(lturbomole)then
                   call excmd('mkdir '//fileout1(1:slen(fileout1))//'-dir')
                   call excmd('mv '//fileout1(1:slen(fileout1))//' '//fileout1(1:slen(fileout1))//'-dir/coord')
                endif
             
100             continue
             enddo

          enddo
          
          return
        end subroutine cut1d

!#######################################################################

        
        subroutine cutvec
          implicit none
        
          integer(long) :: p,p1,dim,j
          real(dop)     :: pdum
          real(dop), dimension(maxdim) :: q1,xcoo
  
          if (npnts .eq. 0) npnts = 10

          write(ilog,'(a)') 'Vector Cut from , to : '
          do j=1,ncoo
             write(ilog,'(2f10.6)') qini(j),q(j)
          enddo
          write(ilog,'(a,i5)') 'End of vector at point: ',npnts

          pdum=npnts

! the vector cut is from 0 to npnts. npnts1 points are added at either end.
          ndat=npnts + npnts1

          ! Apply scaling factor (default is unity)
          do p=1,ncoo
             q(p)=q(p)*vscale
             qini(p)=qini(p)*vscale
          enddo
          do p=-npnts1,ndat
             if (p .eq. 0 .and. .not. lq0) go to 101
             dim=1
             call getfilename(p,0,0,dim,fileout,fileout1,&
                  cmode,cmode1,cmode2,set,lmolpro,ldalton,lturbomole,&
                  lmolcas)
             if (lgo) call wrgo(igo,subcmd,fileout1,iin,lgocmd)
             open(iout,file=fileout1,form='formatted',&
                  status='unknown')
             write(6,'(2a)') 'Output file : ',fileout1(1:60)
             
             call zerovxd(q1,maxdim)

             if (.not.lreplace) then
                if (.not.lpoly.and..not.lnmcurv) then
                   ! Vector in normal mode coordinates
                   if (lalleig) then
                      call qvec2f(xcoo,xcoo0,q,qini,nmcoo,nmodes,ncoo,maxdim,&
                           npnts,p)
                   else
                      call qvec2(xcoo,xcoo0,q,qini,nmcoo,nmodes,ncoo,maxdim,&
                           npnts,p)
                   endif
                elseif( lnmcurv) then
                   do j=1,nmodes
                       q1(j)= qini(j)+((p/pdum)*(q(j)-qini(j)))
                   enddo

                   call curv2q(q1)
                   call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                else      
                   ! Vector in polyspherical coordinates
                   do j=1,nmodes                                          
                      q1(j)=pcoo0(j)+qini(j)+((p/pdum)*(q(j)-qini(j)))
                   enddo
                   call poly2x(xcoo,xcoo0,q1,ncoo,nmodes,patm,mass,vcon)
           
                endif
             else
                call replacemode(p,p1)
             endif

             call comout(iout,xcoo,fileout1,tag)
             close(iout)
             
101          continue
          enddo
          
          call excmd('chmod 700 '//fileout1(1:slen(fileout1)))
          
          return
        end subroutine cutvec
        
!#######################################################################
        
        subroutine cutrot

          implicit none
          
          integer(long) :: p,dim
          real(dop), dimension(maxdim) :: q1,xcoo

          do p=-ndat,ndat
             if (p .eq. 0 .and. .not. lq0) go to 120
             
             dim=1
             call getfilename(p,0,0,dim,fileout,fileout1,&
                  cmode,cmode1,cmode2,set,lmolpro,ldalton,lturbomole,&
                  lmolcas)
             if (lgo) call wrgo(igo,subcmd,fileout1,iin,lgocmd)
             open(iout,file=fileout1,form='formatted',&
                  status='unknown')
             write(6,'(2a)') 'Output file : ',fileout1(1:60)
             
             call zerovxd(q1,maxdim)
             
             call setintcoo
             n=0
             if (lalleig) then
                call q2xf(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
             else
                call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
             endif

             call comout(iout,xcoo,fileout1,tag)
             close(iout)
             
120          continue
          enddo
          
          return
        end subroutine cutrot

!#######################################################################

      subroutine cutdihed

      implicit none
          
      integer(long) :: p,t,t2,t3,t4,t5,t6,d1,h,o,nd,dim
      integer(long), dimension(4)        :: temp
      real(dop)                          :: x2,y2,z2
      real(dop), dimension(maxdim)       :: phi,theta,r,thetai
      real(dop), dimension(maxdim)       :: xcooi,xcoo,xcoot

      integer(long)                      :: a,b,c,e
      integer(long), dimension(maxdim)   :: d
      real(dop)                          :: adj,adj1,adj2,opp,opp1,opp2,&
                                            thetax,thetay,thetaz,x,y,z
      logical(kind=4), save              :: lrotatex,lrotatey,ltransl, &
                                            lrotatez,lreverse
      real(dop), dimension(10), save     :: dihedi,angled
      real(dop), dimension(maxdim) :: q1

!-------------------------------------------------------
!   Dihedral
!-------------------------------------------------------
!   Dihedral works using polar spherical coordinates
!   To achieve this the entire molecular must be rotated
!   so that the a-b bond lies in the x plane, atom c
!   must be the origin and the b-c bond must lie along
!   the z plane.
!   NOTE: Do not remove print statements starting: 
!           'Perpendicular to _ axis...'
!         As it will produce an odd segmentation fault
!-------------------------------------------------------

!
! defaults used in dihedral section
!
      lrotatex=.false.
      lrotatey=.false.
      lrotatez=.false.
      ltransl=.true.
      lreverse=.false.

      opp=0.0_dop
      adj=0.0_dop
      opp1=0.0_dop
      adj1=0.0_dop
      thetax=0.0_dop
      thetay=0.0_dop
      thetaz=0.0_dop
          
!
! setting a,b,c,d
!       
          e=1
          d(:)=0
          do p=1,ncoo/3
             if (dihedflag(p) .eq. 'a' .or. &
                  dihedflag(p) .eq. 'A') then
                a = p
             elseif (dihedflag(p) .eq. 'b' .or. &
                  dihedflag(p) .eq. 'B') then
                b = p
             elseif (dihedflag(p) .eq. 'c' .or. &
                  dihedflag(p) .eq. 'C') then
                c = p
             elseif (dihedflag(p) .eq. 'd' .or. &
                  dihedflag(p) .eq. 'D') then
                d(e) = p
                e=e+1
             elseif (dihedflag(p) .ne. 'f' .or. &
                  dihedflag(p) .eq. 'F') then
                routine = 'vcpinp (read VCPNT input)'
                message = 'Invalid dihedral flag (a,b,c,d,f only)'
                call vcerrormsg
             endif
          enddo
!
! rotation to put the b-c bond in the z plane
! rotates about the x axis
!
          do o=1,ncoo
             xcoo(o)=xcoo0(o)
          enddo

          t=(b-1)*3+2
          t2=(c-1)*3+2
          t3=(b-1)*3+3
          t4=(c-1)*3+3
      
          opp=xcoo(t)-xcoo(t2)
          adj=xcoo(t3)-xcoo(t4)
          if (abs(opp) .lt. tiny) then
             lrotatex=.false.
             xcoot(:)=xcoo(:)
          elseif (abs(adj) .lt. tiny) then 
             lrotatex=.true.
             thetax=90*(pi/180)
             call rotatex(thetax,xcoo,xcoot)
          else
             lrotatex=.true.
             thetax=atan(opp/adj)
             call rotatex(thetax,xcoo,xcoot)
          endif
          do o=1,ncoo
             xcoo(o)=xcoot(o)
          enddo
!
! rotation to put the b-c bond in the z plane
! rotates about the y axis
!
          t=(b-1)*3+1
          t2=(c-1)*3+1
          t3=(b-1)*3+3
          t4=(c-1)*3+3
          opp=xcoo(t)-xcoo(t2)
          adj=xcoo(t3)-xcoo(t4)
          if (abs(opp) .lt. tiny) then
             lrotatey=.false.
             xcoot(:)=xcoo(:)
          elseif(abs(adj) .lt. tiny)then
             lrotatey=.true.
             thetay=90*(pi/180)
             call rotatey(thetay,xcoo,xcoot)
          else
             lrotatey=.true.
             thetay=atan(opp/adj)
             call rotatey(thetay,xcoo,xcoot)
          endif
          do o=1,ncoo
             xcoo(o)=xcoot(o)
          enddo
!
! rotation to put the a-b bond in the x plane
! rotates about the z axis
!
          t=(a-1)*3+2
          t2=(b-1)*3+2
          t3=(a-1)*3+1
          t4=(b-1)*3+1
          opp=xcoo(t)-xcoo(t2)
          adj=xcoo(t3)-xcoo(t4)
          if (abs(opp) .lt. tiny) then
             lrotatez=.false.    
             xcoot(:)=xcoo(:)
          elseif (abs(adj) .lt. tiny) then
             thetaz=90*(pi/180)
!           print*,'Perpendicular to Y axis, rotation set to 90 degrees'
             call rotatez(thetaz,xcoo,xcoot)
             lrotatez=.true.
          else 
             thetaz=atan(opp/adj)
             call rotatez(thetaz,xcoo,xcoot)
             lrotatez=.true.
          endif
          do o=1,ncoo
             xcoo(o)=xcoot(o)
          enddo
!
! translation to put bond c as the origin
!
          t=(c-1)*3+1
          t2=(c-1)*3+2
          t3=(c-1)*3+3
          x=xcoo(t)
          y=xcoo(t2)
          z=xcoo(t3)
          do p=1,ncoo/3
             t=(p-1)*3+1
             t2=(p-1)*3+2
             t3=(p-1)*3+3
             xcoo(t)=xcoo(t)-x
             xcoo(t2)=xcoo(t2)-y
             xcoo(t3)=xcoo(t3)-z
          enddo
!
! creating the dihedral angle
!
          lreverse=.true.
          thetax=-thetax
          thetay=-thetay
          thetaz=-thetaz
          temp(1) = a
          temp(2) = b
          temp(3) = c
! Need number of d atoms
          nd=0
          do h=1,10
             if (d(h) .gt. 0) then
                nd=nd+1
             endif
          enddo
!  Need the initial BCD angle,sigma.
!  Need the initial C-D bond length,r.
          do h=1,nd
             d1=d(h)
             phi(h) = angel(xcoo,b,c,d1)
             t=(c-1)*3+1
             t2=(c-1)*3+2
             t3=(c-1)*3+3
             t4=(d(h)-1)*3+1
             t5=(d(h)-1)*3+2
             t6=(d(h)-1)*3+3
             x2=xcoo(t)-xcoo(t4)
             y2=xcoo(t2)-xcoo(t5)
             z2=xcoo(t3)-xcoo(t6)
             r(h)=sqrt((x2**2)+(y2**2)+(z2**2))
             temp(4) = d(h)
             thetai(h) = dihed(xcoo,temp,ncoo)
             thetai(h) = thetai(h)
             t2=(d(h)-1)*3+2
             opp=xcoo(t2)
             if (opp .lt. 0) then
                thetai(h)=-thetai(h)
             endif
          enddo

! Need to store rotated and translated co-ordinates and reset xcoo
! to this value at the start of the following(xcoo output, xcooi
! initial (after rotation & translation), xcoot temporary and xcoo0
! is the file0 geometry
          do o=1,ncoo
             xcooi(o)=xcoo(o)
          enddo
! Moving d atoms
          do p=-ndat,ndat
             
             dim=1
             call getfilename(p,0,0,dim,fileout,fileout1,&
                  cmode,cmode1,cmode2,set,lmolpro,ldalton,lturbomole,&
                  lmolcas)
             if (lgo) call wrgo(igo,subcmd,fileout1,iin,lgocmd)
             open(iout,file=fileout1,form='formatted',&
                  status='unknown')
             write(6,'(2a)') 'Output file : ',fileout1(1:60)
             
             do h=1,ncoo
                xcoo(h)=xcooi(h)
             enddo

             do h=1,nd
                t=(d(h)-1)*3+1
                t2=(d(h)-1)*3+2
                t3=(d(h)-1)*3+3
                theta(h) = thetai(h) + (p * xincr)
                xcoo(t) = r(h) * cos(theta(h)) * sin(phi(h))
                xcoo(t2) = r(h) * sin(theta(h)) * sin(phi(h))
                xcoo(t3) = r(h) * cos(phi(h))
             enddo

             if (lrotatex .and. lrotatey .and. lrotatez) then
                do o=1,ncoo/3
                   t=(o-1)*3+1
                   t2=(o-1)*3+2
                   t3=(o-1)*3+3
                   xcoo(t)=xcoo(t)+x
                   xcoo(t2)=xcoo(t2)+y
                   xcoo(t3)=xcoo(t3)+z
                enddo
                call rotatez(thetaz,xcoo,xcoot)
                do o=1,ncoo
                   xcoo(o)=xcoot(o)
                enddo
                call rotatey(thetay,xcoo,xcoot)
                do o=1,ncoo
                   xcoo(o)=xcoot(o)
                enddo
                call rotatex(thetax,xcoo,xcoot)
                do o=1,ncoo
                   xcoo(o)=xcoot(o)
                enddo
             elseif (lrotatex .and. lrotatey) then
                do o=1,ncoo/3
                   t=(o-1)*3+1
                   t2=(o-1)*3+2
                   t3=(o-1)*3+3
                   xcoo(t)=xcoo(t)+x
                   xcoo(t2)=xcoo(t2)+y
                   xcoo(t3)=xcoo(t3)+z
                enddo
                call rotatey(thetay,xcoo,xcoot)
                do o=1,ncoo
                   xcoo(o)=xcoot(o)
                enddo
                call rotatex(thetax,xcoo,xcoot)
                do o=1,ncoo
                   xcoo(o)=xcoot(o)
                enddo
             elseif (lrotatex .and. lrotatez) then
                do o=1,ncoo/3
                   t=(o-1)*3+1
                   t2=(o-1)*3+2
                   t3=(o-1)*3+3
                   xcoo(t)=xcoo(t)+x
                   xcoo(t2)=xcoo(t2)+y
                   xcoo(t3)=xcoo(t3)+z
                enddo
                call rotatez(thetaz,xcoo,xcoot)
                do o=1,ncoo
                   xcoo(o)=xcoot(o)
                enddo
                call rotatex(thetax,xcoo,xcoot)
                do o=1,ncoo
                   xcoo(o)=xcoot(o)
                enddo
             elseif (lrotatey .and. lrotatez) then
                do o=1,ncoo/3
                   t=(o-1)*3+1
                   t2=(o-1)*3+2
                   t3=(o-1)*3+3
                   xcoo(t)=xcoo(t)+x
                   xcoo(t2)=xcoo(t2)+y
                   xcoo(t3)=xcoo(t3)+z
                enddo
                call rotatez(thetaz,xcoo,xcoot)
                do o=1,ncoo
                   xcoo(o)=xcoot(o)
                enddo
                call rotatey(thetay,xcoo,xcoot)
                do o=1,ncoo
                   xcoo(o)=xcoot(o)
                enddo
             elseif (lrotatex) then
                do o=1,ncoo/3
                   t=(o-1)*3+1
                   t2=(o-1)*3+2
                   t3=(o-1)*3+3
                   xcoo(t)=xcoo(t)+x
                   xcoo(t2)=xcoo(t2)+y
                   xcoo(t3)=xcoo(t3)+z
                enddo
                call rotatex(thetax,xcoo,xcoot)
                do o=1,ncoo
                   xcoo(o)=xcoot(o)
                enddo
             elseif (lrotatey) then
                do o=1,ncoo/3
                   t=(o-1)*3+1
                   t2=(o-1)*3+2
                   t3=(o-1)*3+3
                   xcoo(t)=xcoo(t)+x
                   xcoo(t2)=xcoo(t2)+y
                   xcoo(t3)=xcoo(t3)+z
                enddo
                call rotatey(thetay,xcoo,xcoot)
                do o=1,ncoo
                   xcoo(o)=xcoot(o)
                enddo
             elseif (lrotatez) then
                 do o=1,ncoo/3
                    t=(o-1)*3+1
                    t2=(o-1)*3+2
                    t3=(o-1)*3+3
                    xcoo(t)=xcoo(t)+x
                    xcoo(t2)=xcoo(t2)+y
                    xcoo(t3)=xcoo(t3)+z
                 enddo
                 call rotatez(thetaz,xcoo,xcoot)
                 do o=1,ncoo
                    xcoo(o)=xcoot(o)
                 enddo
             else
                do o=1,ncoo/3
                   t=(o-1)*3+1
                   t2=(o-1)*3+2
                   t3=(o-1)*3+3
                   xcoo(t)=xcoo(t)+x
                   xcoo(t2)=xcoo(t2)+y
                   xcoo(t3)=xcoo(t3)+z
                enddo
             endif

             call comout(iout,xcoo,fileout1,tag)
             close(iout)
             
          enddo

          
          return
        end subroutine cutdihed

!#######################################################################

        subroutine cut2d

          implicit none

          integer(long) :: p,p1,dim
          real(dop), dimension(maxdim) :: q1,xcoo
          real(dop)                    :: qtheta,theta0

          do p=-ndat,ndat
             do p1=-ndat1,ndat1
                
                dim=2
                call getfilename(p,p1,0,dim,fileout,fileout1,&
                     cmode,cmode1,cmode2,set,lmolpro,ldalton,lturbomole,&
                     lmolcas)
                if (lgo) call wrgo(igo,subcmd,fileout1,iin,lgocmd)
                open(iout,file=fileout1,form='formatted',&
                     status='unknown')
                write(6,'(2a)') 'Output file : ',fileout1(1:60)
                
                call zerovxd(q1,maxdim)
                
                if (.not. lreplace) then
                   q1(cmode)=x0 + p*xincr
                   q1(cmode1)=x01 + p1*xincr1
                   if (lalleig) then
                      call q2xf(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                   else
                      call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                   endif

                else if (lnmcurv .and. nm2curv(cmode1) .eq. 2) then
                   q1(cmode)=x0 + p*xincr
                   if (lalleig) then
                      call q2xf(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                   else
                      call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                   endif
! Dihedral second mode in pair
                   qtheta= dble(p1*xincr1)
                   call nm2dihed(xcoo,qtheta,xcoo0,theta0)
                   q1(cmode1) = qtheta + theta0

                else if (lnmcurv .and. nm2curv(cmode) .eq. 2) then
                   q1(cmode1)=x0 + p1*xincr1
                   if (lalleig) then
                      call q2xf(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                   else
                      call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                   endif
! Dihedral first mode in pair
                   qtheta= dble(p*xincr)
                   call nm2dihed(xcoo,qtheta,xcoo0,theta0)
                   q1(cmode) = qtheta + theta0

                else
                  ! N.B. cmode always corresponds to the fragment mode
                   q1(cmode)=x0 + p*xincr                   
                   if (lalleig) then
                      call q2xf(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                   else
                      call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                   endif
                   call replacemode(p,p1)
                endif
                
                call comout(iout,xcoo,fileout1,tag)
                close(iout)
                
             enddo
          enddo
          
          return
        end subroutine cut2d

!#######################################################################

        subroutine cut3d

          implicit none

          integer(long) :: p,p1,p2,dim
          real(dop), dimension(maxdim) :: q1,xcoo

          do p=-ndat,ndat
             do p1=-ndat1,ndat1
                do p2=-ndat2,ndat2
                   
                   dim=3
                   call getfilename(p,p1,p2,dim,fileout,fileout1,&
                        cmode,cmode1,cmode2,set,lmolpro,ldalton,lturbomole,&
                        lmolcas)
                   if (lgo) call wrgo(igo,subcmd,fileout1,iin,lgocmd)
                   open(iout,file=fileout1,form='formatted',&
                        status='unknown')
                   write(6,'(2a)') 'Output file : ',fileout1(1:60)
                   
                   call zerovxd(q1,maxdim)
                   
                   q1(cmode)=x0 + p*xincr
                   q1(cmode1)=x01 + p1*xincr1
                   q1(cmode2)=x02 + p2*xincr2
                  
                   if (lalleig) then
                      call q2xf(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                   else
                      call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                   endif

                   call comout(iout,xcoo,fileout1,tag)
                   close(iout)
                   
                enddo
             enddo
          enddo
          
          return
        end subroutine cut3d

!#######################################################################

        subroutine cutdiag

          implicit none

          integer(long) :: p,p1,dim,i,k,k1,t,t1,l,l1,j,m1
          logical(kind=4) :: check
          real(dop), dimension(maxdim) :: inc
          real(dop) :: pi, n0, qtheta, theta0
          real(dop), dimension(maxdim) :: q1,xcoo

          pi = 3.1415926535897931    
          
          do p=-ndat,ndat
             if (p .eq. 0 .and. .not. lq0) go to 150
             
             dim=2
             call getfilename(p,p,0,dim,fileout,fileout1,&
                  cmode,cmode1,cmode2,set,lmolpro,ldalton,lturbomole,&
                  lmolcas)
             if (lgo) call wrgo(igo,subcmd,fileout1,iin,lgocmd)
             open(iout,file=fileout1,form='formatted',&
                  status='unknown')
             write(6,'(2a)') 'Output file : ',fileout1(1:60)
             
             call zerovxd(q1,maxdim)
             
! move to any specific coordinates unless curvilinear (done at end)
             do i=1,nval
                m1=ival(i)
                if (m1 .eq. cmode .or. m1 .eq. cmode1) then
                   routine = 'Cut1d'
                   message = &
                        'Cannot specify a qval and a cut for the same mode'
                   call vcerrormsg
                else if (nm2curv(m1) .eq. 0) then
                   q1(m1) = qval0(m1)
                endif
             enddo

             if (lpoly) then
                do i=1,nmodes
                   q1(i)=pcoo0(i)
                enddo
                q1(cmode)=q1(cmode)+p*xincr
                q1(cmode1)=q1(cmode1)+p*xincr1
                call poly2x(xcoo,xcoo0,q1,ncoo,nmodes,patm,mass,vcon)

             else if (lnmcurv .and. nm2curv(cmode1) .eq. 2) then
                q1(cmode)=x0 + p*xincr
                if (lalleig) then
                   call q2xf(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                else
                   call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                endif
! Dihedral second mode in pair
                qtheta= dble(p*xincr1)
                call nm2dihed(xcoo,qtheta,xcoo0,theta0)
                q1(cmode1) = qtheta + theta0

             else if (lnmcurv .and. nm2curv(cmode) .eq. 2) then
                q1(cmode1)=x0 + p*xincr1
                if (lalleig) then
                   call q2xf(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                else
                   call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                endif
! Dihedral first mode in pair
                qtheta= dble(p*xincr)
                call nm2dihed(xcoo,qtheta,xcoo0,theta0)
                q1(cmode) = qtheta + theta0

             else
             !     if( lnmcurv) then
             !       q1(cmode1)=p*xincr1
             !       q1(cmode)=p*xincr
                     
             !        print*,' TEST, incr = ', xincr, xincr1
             !        print*, 'Q=', q1(1:nmodes)
             !       call curv2q(q1(1:nmodes))
             !        print*, 'IN BETWEEN Q=', q1(1:nmodes)
             !       call q2curv(q1(1:nmodes))
             !         print*, 'AFTER OUT Q=', q1(1:nmodes)
             !       call curv2q(q1(1:nmodes))
             !       call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)   
             !     else
                   call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim) 
!		   simple normal modes
                   q1(cmode)=p*xincr

                   if (lreplace.neqv..true.) then
                      q1(cmode1)=p*xincr1
                      if (lalleig) then
                         call q2xf(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                      else
                         call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                      endif
                   else
                      if (lreplacediag.neqv..true.) then
                         if (lalleig) then
                            call q2xf(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                         else
                            call q2x(xcoo,xcoo0,q1,nmcoo,nmodes,ncoo,maxdim)
                         endif
                         call replacemode(p,p1)
                      else
                         call replacemode(p,p1)
                      endif
                   endif
             !     endif
! move to any specific curvilinear coordinates
                   do i=1,nval
                      m1=ival(i)
                      if (nm2curv(m1) .eq. 2) then
                         qtheta= qval0(i)
                         call nm2dihed(xcoo,qtheta,xcoo0,theta0)
                      endif
                   enddo

             endif
             call comout(iout,xcoo,fileout1,tag)
             close(iout)
             
! If using Turbomole, create a directory and move the output file to 'coord' in that
! directory. Each Turbomole energy calculation needs its own directory. 
             if(lturbomole)then
                call excmd('mkdir '//fileout1(1:slen(fileout1))//'-dir')
                call excmd('mv '//fileout1(1:slen(fileout1))//' '//fileout1(1:slen(fileout1))//'-dir/coord')
             endif
             
150          continue
           enddo
          
          return
        end subroutine cutdiag

!#######################################################################

        subroutine rescaleq
          
          implicit none

          integer(long) :: i
          
          if (cmode.le.nmodes) xincr=xincr/sqrt(freq0(cmode))
          if (cmode1.le.nmodes) xincr1=xincr1/sqrt(freq0(cmode1))
          if (cmode2.le.nmodes) xincr1=xincr2/sqrt(freq0(cmode2))

          if (lvec) then
             do i=1,nmodes
                q(i)=q(i)/sqrt(freq0(i))
                qini(i)=qini(i)/sqrt(freq0(i))
             enddo
          endif

          return
        end subroutine rescaleq

!#######################################################################

      subroutine vcpdefault

     implicit none

      integer(long)       :: h,iset

      logaussian=.false.
      logamess=.false.
      loqchem=.false.
      loxyz=.false.
      lmolpro=.false.
      ldalton=.false.
      lturbomole=.false.
      lgamessus=.false.
      lmolcas=.false.
      ldd=.false.

      ndat=1
      ndat1=0
      ndat2=0

      set = 0

      ndum=0
      ldumcom=.false.
!
! set all point schemes to false
!
      lcut=.false.
      lvec=.false.
      lbloc2=.false.
      lbloc3=.false.
      ldiag=.false.
      lrot=.false.
      ldihed=.false.
      lsubpnt = .false.
      lreplace = .false.
      lreplacediag = .false.
      lsubatm = .false.
      lcutall=.false.
      lrotmodes=.false.
      ljacobi=.false.
      lradau=.false.
      lvalence=.false.
      lpoly=.false.

      fmode = 0
      fx0 = 0.0

      vscale=1.0

!
! include q0 point
!
      lq0 = .true.

      lrdfreq0=.true.

      iset = 100
      lop = .false.
      lcl = .false.
      lgo = .true.
      lgoadd = .false.
      lgocmd = .false.
!
! Setting xcoo matrix to zero
!
      do h=1,maxdim
         xcoo0(h)=0.0_dop
      enddo

      return
      end subroutine vcpdefault

!#######################################################################

      subroutine vcpinp(tag)

      integer(long),intent(out) :: tag
      integer(long)           :: i,j,k,ic,ierr,ilbl,n,ntra,ncom,itmp,&
                                 ntr,m
      character(len=80)       :: filename,currorient,string
      real(dop) :: tmp

!-----------------------------------------------------------------------
! set initial values
!
! currorient: default orientation for reading geometry (e.g. in Gaussian)
!-----------------------------------------------------------------------
      iz0 = 0
      iz = 0
      fileout = ' '
      fileop = ' '
      filecl = ' '
      subcmd = 'qsub '
      currorient = 'input'
      file0orient='input'

      ncom = 0
      
      call zerovxd(qini,maxdim) 

!-----------------------------------------------------------------------
! Open files
!-----------------------------------------------------------------------
      filename=filein
      open(iin,file=filename,form='formatted',status='old',iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a)') 'ERROR: Input file not found'
         write(6,'(a)') filein(1:80)
         write(6,*)
         stop
      endif

!-----------------------------------------------------------------------
! read input file
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'vcpinp (read VCPNT input)'
      call vccntrl(ierr)

      i=1
   15 continue
      if (keyword(i) .eq. 'end-input') then
         goto 20

      else if (keyword(i) .eq. 'file0orient') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            file0orient=keyorig(i)
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with file0orient keyword'
            call vcwrline(0,iz,iz0)
         endif


      else if (keyword(i) .eq. 'file0') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            file0=keyorig(i)
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with file0 keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'sub_file0') then
         lsubpnt=.true.
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) subfile0
         endif

      else if (keyword(i).eq.'sys_atom') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) tag
         endif

      else if (keyword(i) .eq. 'fileQ0'&
       .or. keyword(i) .eq. 'fileq0') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            lrdq0 = .true.
            fileQ0=keyorig(i)
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with file0 keyword'
            call vcwrline(0,iz,iz0)
         endif


      else if (keyword(i) .eq. 'nmodes') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) nmodes
            if (keyword(i+1) .eq. ',') then
               i=i+2
               read(keyword(i),*) ntr
            else
               ntr = 6
            endif
            ncoo=nmodes+ntr
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with nmodes keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'fileout') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            fileout=keyorig(i)
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with fileout keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'opening') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            fileop=keyorig(i)
            lop = .true.
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with opening keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'closing') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            filecl=keyorig(i)
            lcl = .true.
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with closing keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'formout') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            if (keyword(i) .eq. 'gaussian') then
               logaussian = .true.
            else if (keyword(i) .eq. 'gamess') then
               logamess = .true.
            else if (keyword(i) .eq. 'qchem') then
               loqchem = .true.
            else if (keyword(i) .eq. 'xyz') then
               loxyz = .true.
            else if (keyword(i) .eq. 'molpro') then
               lmolpro = .true.
            else if (keyword(i) .eq. 'molcas') then
               lmolcas = .true.
            else if (keyword(i) .eq. 'dalton') then
               ldalton = .true.
            else if (keyword(i) .eq. 'turbomole') then
               lturbomole = .true.
            else if (keyword(i) .eq. 'gamess_us') then
               lgamessus = .true.
            else if (keyword(i) .eq. 'molcas') then
               lmolcas = .true.
            else
               routine = 'vcpinp (read VCPNT input)'
               write(message,'(2a)') 'Unknown formout : ',keyword(i)
               call vcwrline(0,iz,iz0)
            endif
            
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with formout keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'formin') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            if (keyword(i) .eq. 'dd') then
               ldd = .true.

!            call rdgeodb(nsta,xcoodd)

            else
               routine = 'vcpinp (read VCPNT input)'
               write(message,'(2a)') 'Unknown formin : ',keyword(i)
               call vcwrline(0,iz,iz0)
            endif
            
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with formin keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'go_none') then
         lgo = .false.

      else if (keyword(i) .eq. 'go_add') then
         lgoadd = .true.

      else if (keyword(i) .eq. 'go_cmd') then
         if (i .ne. 1) then
            routine = 'vcpinp (read VCPNT input)'
            message='Go_cmd keyword must be alone on line'
            call vcwrline(i,iz,iz0)
         else
            lgocmd = .true.
 30         continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                 maxkey)
            routine = 'vctinp (read VCTRANS input)'
            call vccntrl(ierr)

            if (keyword(1) .ne. 'end-go_cmd') go to 30
         endif

      else if (keyword(i) .eq. 'subcmd') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            subcmd = keyorig(i) 
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with subcmd keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'frequencies ') then
         if (i .ne. 1) then
            routine = 'vcpinp (read VCPNT input)'
            message='Frequencies keyword must be alone on line'
            call vcwrline(i,iz,iz0)
         else
            lrdfreq0=.false.
            n=0
 40         continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                 maxkey)
            routine = 'vctinp (read VCTRANS input)'
            call vccntrl(ierr)

            if (keyword(1) .ne. 'end-frequencies') then
               n=n+1
               if (ic .ge. 3) then
                  nmlab1(n)=keyorig(2)
                  read(keyword(3),*) freq1(n)
               else
                  routine = 'vctinp (read VCTRANS input)'
                  message='Error reading frequencies'
                  call vcwrline(i,iz,iz0)
               endif
               go to 40
            endif
         endif

      else if (keyword(i) .eq. 'noq0') then
         lq0 = .false.

      else if (keyword(i) .eq. 'fixq') then
         if (keyword(i+1).eq.'=') then
            read(keyword(i+2),*) fmode
            read(keyword(i+4),*) fx0
            i=i+4
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with set keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'set') then
         if (keyword(i+1).eq.'=') then
            read(keyword(i+2),*) set
            i=i+2
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with set keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'along') then
         if (keyword(i+1).eq.'=') then
            lcut=.true.
            i=i+2
            read(keyword(i),*) cmode
            read(keyword(i+1),*) ndat
            read(keyword(i+2),*) xincr
            i=i+2

            if (ic .eq. i+1) then
               read(keyword(i+1),*) x0
               i=i+1
            else
               x0=0.0
            endif
            
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with along keyword'
            call vcwrline(0,iz,iz0)
         endif

         npnts=ndat*2+1
   
      else if (keyword(i) .eq. 'along_all') then
         if (keyword(i+1).eq.'=') then
            lcut=.true.
            lcutall=.true.
            i=i+2
            read(keyword(i),*) ndat
            read(keyword(i+1),*) xincr
            i=i+1

            if (ic .eq. i+1) then
               read(keyword(i+1),*) x0
               i=i+1
            else
               x0=0.0
            endif

         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with along keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'grid2d') then
         if (keyword(i+1).eq.'=') then
            lbloc2=.true.
            i=i+2
            read(keyword(i),*) cmode
            read(keyword(i+1),*) cmode1
            read(keyword(i+2),*) ndat
            read(keyword(i+3),*) ndat1
            read(keyword(i+4),*) xincr
            read(keyword(i+5),*) xincr1
            i=i+5

            if (ic .eq. i+2) then
               read(keyword(i+1),*) x0
               read(keyword(i+2),*) x01
               i=i+2

            else
               x0=0.0
               x01=0.0
            endif

         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with grid2d keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'grid3d') then
         if (keyword(i+1).eq.'=') then
            lbloc3=.true.
            i=i+2
            read(keyword(i),*) cmode
            read(keyword(i+1),*) cmode1
            read(keyword(i+2),*) cmode2
            read(keyword(i+3),*) ndat
            read(keyword(i+4),*) ndat1
            read(keyword(i+5),*) ndat2
            read(keyword(i+6),*) xincr
            read(keyword(i+7),*) xincr1
            read(keyword(i+8),*) xincr2
            i=i+8

            if (ic .eq. i+3) then
               read(keyword(i+1),*) x0
               read(keyword(i+2),*) x01
               read(keyword(i+3),*) x02
               i=i+3
            else
               x0=0.0
               x01=0.0
               x02=0.0
            endif
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with grid3d keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'rotate') then
         if (keyword(i+1).eq.'=') then
            lrot=.true.
            i=i+2
            read(keyword(i),*) cmode
            read(keyword(i+1),*) ndat
            read(keyword(i+2),*) xincr
            i=i+2

            if (ic .eq. i+1) then
               read(keyword(i+1),*) x0
               i=i+1
            else
               x0=0.0
            endif
            
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with rotate keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'diagonal') then
         if (keyword(i+1).eq.'=') then
            ldiag=.true.
            i=i+2
            read(keyword(i),*) cmode
            read(keyword(i+1),*) cmode1
            read(keyword(i+2),*) ndat
            read(keyword(i+3),*) xincr
            read(keyword(i+4),*) xincr1
            i=i+4

            if (ic .eq. i+1) then
               read(keyword(i+1),*) x0
               i=i+1
            elseif(ic .eq. i+2) then
               read(keyword(i+1),*) x0
               read(keyword(i+2),*) x01
               i=i+2
            else
               x0=0.0
            endif
            
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with diagonal keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'replace') then
         if (keyword(i+1).eq.'=') then
            lreplace=.true.
            i=i+2
            read(keyword(i),*) rmode
            read(keyword(i+1),*) ratm1
            read(keyword(i+2),*) ratm2
            read(keyword(i+3),*) ratm3
            read(keyword(i+4),*) ratm4
            i=i+5            
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with diagonal keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'replace') then
         if (keyword(i+1).eq.'=') then
            lreplace=.true.
            i=i+2
            read(keyword(i),*) rmode
            read(keyword(i+1),*) ratm1
            read(keyword(i+2),*) ratm2
            read(keyword(i+3),*) ratm3
            read(keyword(i+4),*) ratm4
            read(keyword(i+5),*) rdef
            i=i+5
            
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with replace keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'replace_diag') then
         if (keyword(i+1).eq.'=') then
            lreplace=.true.
            lreplacediag=.true.
            i=i+2
            read(keyword(i),*) rmode
            read(keyword(i+1),*) rmode1
            read(keyword(i+2),*) ratm1
            read(keyword(i+3),*) ratm2
            read(keyword(i+4),*) ratm3
            read(keyword(i+5),*) ratm4
            read(keyword(i+6),*) rdef
            read(keyword(i+7),*) rdef1
            i=i+7
            
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with replace keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'qval') then
         nval = nval+1
         i=i+1
         read(keyword(i),*) ival(nval)
         i=i+1
         read(keyword(i),*) qval0(nval)
         
! Dummy atoms
      else if (keyword(i).eq.'dummy_atoms') then
 42      continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         call vccntrl(ierr)
         if (keyword(i).ne.'end-dummy_atoms') then
            ndum=ndum+1
            read(keyword(1),*) dumatsym(ndum*3)
            dumatsym(ndum*3-2)=dumatsym(ndum*3)
            dumatsym(ndum*3-1)=dumatsym(ndum*3)
            read(keyword(2),*) string
            if (string.eq.'com') then
               ldumcom=.true.
               ncom=ncom+1
               if (ncom.gt.1) then
                  write(6,*)
                  write(6,*)"##################################", &
                       "#######################"
                  write(6,*) "Cannot place more than one dummy ", &
                       "atom at centre of mass!"
                  write(6,*)"##################################", &
                       "#######################"
                  write(6,*)
                  STOP
               endif
            else
               read(keyword(2),*) dumxcoo(ndum*3-2)
               read(keyword(3),*) dumxcoo(ndum*3-1)
               read(keyword(4),*) dumxcoo(ndum*3)
            endif
            
            goto 42
         endif

!-----------------------------------------------------------------------
! A Dihedral scan (same as along + nm2curv dihed)
!-----------------------------------------------------------------------
      else if (keyword(i) .eq. 'dihed') then
         if (keyword(i+1) .eq. '=') then
            ldihed=.true.
            i=i+2
            read(keyword(i),*) cmode
            read(keyword(i+1),*) ndat
            read(keyword(i+2),*) xincr
            i=i+2

            if (ic .eq. i+1) then
               read(keyword(i+1),*) x0
               i=i+1
            else
               x0=0.0
            endif
         endif

!-----------------------------------------------------------------------
! Define a Dihedral 
!-----------------------------------------------------------------------
      else if (keyword(i).eq.'dihed-section ') then
         if (i .ne. 1) then
            routine = 'vcpinp (read VCPNT input)'
            message='dihed-section keyword must be alone on line'
            call vcwrline(i,iz,iz0)
         else
            n=0
 45         continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                 maxkey)
            routine = 'vctinp (read VCPNT input)'
            call vccntrl(ierr)

            if (keyword(1) .ne. 'end-dihed-section') then
               n=n+1
               if (ic .eq. 2) then 
                  read(keyword(2),*) dihedflag(n)
               else
                  routine = 'vctinp (read VCPNT input)'
                  message='Error reading dihedral flags'
                  call vcwrline(i,iz,iz0)
               endif
               go to 45
            endif
         endif
!
!     defined geometry
!
      else if (keyword(i).eq.'define') then
         lvec = .true.
         if (i .ne. 1) then
            routine = 'vcpinp (read VCPNT input)'
            message='Define keyword must be alone on line'
            call vcwrline(i,iz,iz0)
         else
            call zerovxd(q,maxdim) 
 50         continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                 maxkey)
            routine = 'vcpinp (read VCPNT input)'
            call vccntrl(ierr)

            if (keyword(1) .ne. 'end-define') then
               if (ic .eq. 2) then
                  read(keyword(1),*) i
                  read(keyword(2),*) q(i)
               else
                  routine = 'vcpinp (read VCPNT input)'
                  message='Error reading definition'
                  call vcwrline(i,iz,iz0)
               endif
               go to 50
            endif
         endif

!
! New vector definition (S Neville 03/08/2011)         
!
      else if (keyword(i).eq.'vector_define') then  
         lvec = .true.
100      continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
              maxkey)
         if (keyword(1).eq.'final') then
            k=0
110         continue
            k=k+1
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                 maxkey)            
            if (keyword(1).eq.'initial') then
               backspace(iin)
               goto 100
            else if (keyword(1).eq.'end-vector_define') then
               goto 130
            else
               read(keyword(1),*) q(k)
               goto 110
            endif            
         endif
         if (keyword(1).eq.'initial') then
            k=0
120         continue
            k=k+1
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                 maxkey)
            if (keyword(1).eq.'final') then
               backspace(iin)
               goto 100
            else if (keyword(1).eq.'end-vector_define') then
               goto 130
            else
               read(keyword(1),*) qini(k)
               goto 120
            endif
         endif
130      continue

      else if (keyword(i) .eq. 'vector') then
         lvec = .true.
         npnts = 10

      else if (keyword(i) .eq. 'vector_points') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) npnts
            if (keyword(i+1).eq.',') then
               i=i+2
               read(keyword(i),*) npnts1
            endif
             
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with the vector_points keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'vector_scale') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) vscale
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with the vector_scale keyword'
            call vcwrline(0,iz,iz0)
         endif

!
! Substitution of an atom by a group
!
      else if (keyword(i).eq.'substitute') then
         lsubatm=.true.
         
         do j=1,10
            sublbl(j)=''
            do k=1,3
               subxcoo(j,k)=0.0
            enddo
         enddo
 
         read (iin,'(i2)') isubatm
         read (iin,*)
         
         nsubatm=0
60       continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         call vccntrl(ierr)
         if (keyword(i).ne.'end_substitute') then
            nsubatm=nsubatm+1
            read(keyword(1),'(a)') sublbl(nsubatm)
! Convert to uppercase to avoid problems with Molpro atom-indexed output
            call uppercase(sublbl(nsubatm),len(sublbl(nsubatm)))

            do j=1,3
               read(keyword(1+j),*) subxcoo(nsubatm,j)
            enddo

            goto 60
         endif

      else if (keyword(i).eq.'rotate_modes') then
         if (keyword(i+1).eq.'=') then
            lrotmodes=.true.
            j = 1
            do while ( rotm(j,1) .ne. 0 )
              j = j + 1
            enddo
        i = i + 2
!         print*, trim(keyword(i)), trim(keyword(i+1)), trim(keyword(i+2))
144      if ( keyword(i+3) .eq. ',' .or. ( ((i+2)) .eq. ic)) then
!	       modes to be rotatated
               read(keyword(i),*) rotm(j,1)
               read(keyword(i+1),*) rotm(j,2)
!	       mixing angle
               read(keyword(i+2),*) rotang(j)
               j = j + 1
               i = i + 4
               goto 144
            endif   
         else
            routine = 'vctinp (read VCPNT input)'
            message='Error reading normal mode rotation input'
            call vcwrline(i,iz,iz0)
         endif

      else if (keyword(i).eq.'nm2curv') then
         lnmcurv = .true.
         j = 1
         do while (nmcurvtype(j) .ne. 0)
            j = j + 1
         enddo
         if (keyword(i+1).eq.'spherical_type1' .or. keyword(i+1).eq. '1') then
            nmcurvtype(j) = 1
            lalleig = .true.
            if ( keyword(i+2).eq. '=' ) then
               i = i + 3
               k = 0
               do while (trim(keyword(i)) .ne. ',' .and. ic .gt. i)
!               print*, keyword(i), 'j = ', j, ' k = ', k
                  k = k + 1
                  read(keyword(i),*) nmcurv(j,k)
!                  print*, 'j = ', j, 'k = ', k, ' nmcurv(j,k) = ', nmcurv(j,k)
                  i = i + 1
               enddo
    
!         print*, keyword(i), i+3, ic, keyword(i)
!    note to user: if you want to put the x0 array, you have to give it to 
!    all DOF's in the transformation :
               if ( keyword(i) .eq. ',' .and. i+k .ne. ic) then
                  routine='Vcpinp'
                  write(message,'(2a,i5)') &
                        'please give the same number of reference ', &
                             'curvilinear coordinates as DOF in transformation'
                  call vcerrormsg
               elseif( keyword(i) .eq. ',' .and. i+k .eq. ic) then 
                  i = i + 1
                  do n =1, k
                     read(keyword(i),*) c0(j,n)
!                   turn to radians
                    print*, 'k = ', k, 'j = ', j
                     if(nmcurvtype(j) .eq. 1 .and. n .gt. 1 ) c0(j,n) = c0(j,n)*(pi/180)
                     
!                     print*, 'C0 ====', c0(j,n), 'j = ', j, 'n = ', n
                     i = i + 1
                  enddo
               else
!                 call getc0(c0,nmcurv,j) 
               endif
            endif

         else if (keyword(i+1).eq.'dihed' .or. keyword(i+1).eq. '2') then
!    replace nmode by a dihedral
            nmcurvtype(j) = 2
            if ( keyword(i+2).eq. '=' ) then
               i=i+3
               read(keyword(i),*) nmcurv(j,1)
            endif
            m=nmcurv(j,1)
            nm2curv(m) = 2
         else
            routine='Vcpinp'
            write(message,'(a,i5)') ' curvilinear transformation type not implemented'
            call vcerrormsg
         endif

      else if (keyword(i).eq.'jacobi_define') then
         lpoly=.true.
         ljacobi=.true.
         n=0
         do j=1,maxdim
            patm(j)=0
         enddo
65       continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         call vccntrl(ierr)
         if (keyword(i).ne.'end-jacobi_define') then
            n=n+1
            read (keyword(1),*) patm(n)
            goto 65
         endif
         
      else if (keyword(i).eq.'radau_define') then
         lpoly=.true.
         lradau=.true.
         n=0
         do j=1,maxdim
            patm(j)=0
         enddo
70       continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         call vccntrl(ierr)
         if (keyword(i).ne.'end-radau_define') then
            n=n+1
            read (keyword(1),*) patm(n)
            goto 70
         endif

      else if (keyword(i).eq.'valence_define') then
         lpoly=.true.
         lvalence=.true.
         n=0
         do j=1,maxdim
            patm(j)=0
            vcon(j)=0
         enddo
75       continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         call vccntrl(ierr)
         if (keyword(i).ne.'end-valence_define') then
            n=n+1
            read (keyword(1),*) patm(n+1)
            read (keyword(2),*) vcon(n)
            if (n.eq.1) read (keyword(2),*) patm(1)
            goto 75
         endif



      else if (keyword(i).eq.'all_eigenvectors') then
         lalleig=.true.

      else
         routine = 'vcpinp (read VCPNT input)'
         message = ' unknown keyword : '
         call vcwrline(i,iz,iz0)
      endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

 20   continue

      close(iin)

!-----------------------------------------------------------------------
! set any flags after input
!-----------------------------------------------------------------------
! XYZ format does not have opening / closing / running calcs
      if (loxyz) then
         lop = .false.
         lcl = .false.
         lgo = .false.
      endif

! always calculate Q0 if grid
      if (lbloc2 .or. lbloc3) lq0 = .true.

! ensure that, if lreplace=true, then cmode=<nmodes-3, cmode1>nmodes-3
      if ((lreplace).and.(lsubpnt)) then
         if (.not.lvec) then
            if (cmode.le.(nmodes-3).and.cmode1.le.(nmodes-3)) then
               write(6,*)
               write(6,*) "Both modes correspond to fragment modes!"
               write(6,*)
               STOP
            endif
            if (cmode.gt.(nmodes-3).and.cmode1.gt.(nmodes-3)) then
               write(6,*)
               write(6,*) "Both modes correspond to system modes!"
               write(6,*)
               STOP
            endif
            if (cmode.gt.nmodes-3) then
               itmp=cmode
               cmode=cmode1
               cmode1=itmp
               tmp=xincr
               xincr=xincr1
               xincr1=tmp
               if (lbloc2) then
                  itmp=ndat
                  ndat=ndat1
                  ndat1=itmp

                  tmp=x0
                  x0=x01
                  x01=tmp

               endif
            endif
         endif
      endif

      return
      end subroutine vcpinp

!######################################################################

!      subroutine getc0(c0,nmcurv,ctype,iset)
!            
!      
!       
!       integer(long), dimension(INT(maxdim/2)+1,maxdim), intent(in)  :: nmcurv
!       real(dop), dimension((INT(maxdim/2)+1),maxdim), intent(inout)   :: c0
!       integer(long), intent(in)   :: iset
!        
!       if( nmcurvtype(iset) .eq. 1) then
!         c0(iset,1) = sqrt(  
!         
!       endif     
!       
!
!      endsubroutine
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine comout(icom,xcoo,fileout1,tag)

      implicit none

      integer(long)                :: i,j
      integer(long)                :: countH,countN,countC,countF,countO,countS,countCl
      integer(long), intent(in)    :: icom,tag
      character(len=*), intent(in) :: fileout1
      character(len=80)            :: string
      real(dop), dimension(3) :: com
      real(dop), dimension(maxdim), intent(in) :: xcoo


      if (lturbomole) then
         write(icom,'(a6)') '$coord'
      else if (loqchem) then
         !as with turbomole $ is used in qchem inputs
         !write(icom,'(a6)') '$molecule'
         !Assume that there are no meta strings to replace
         if (lop) then
            rewind(iop)
 55         continue
            read(iop,'(80a)') string
            if (string .ne. 'end-opening') then
               write(icom,'(80a)') string
               goto 55
            endif
         endif
      else
         if (lop) then
            rewind(iop)
 10         continue
            read(iop,'(80a)') string
            if (string .ne. 'end-opening') then
               call replacemeta(string,fileout1)

               write(icom,'(80a)') string
               goto 10
            endif
         endif
      endif
      
      if (logaussian) then
         do i=1,ncoo,3
            if (lsubpnt.eqv..true.) then
               if (i.eq.(tag*3-2)) then
                  write(icom,'(a3,3F14.7)') sysatsym,(xsys0(j),j=1,3)
               endif
            endif
            write(icom,'(a3,3F14.7)') atsym(i),(xcoo(j),j=i,i+2)
         enddo
         write(icom,*)
      else if (loxyz) then
         write(icom,'(i5)') ncoo/3
         write(icom,*)
         do i=1,ncoo,3
            write(icom,'(a3,3F14.7)') atsym(i),(xcoo(j),j=i,i+2)
         enddo

      else if (logamess) then
!         write(0,'(a)')'gamess'
         do i=1,ncoo,3
            write(icom,'(3F14.7,i3,2a2)') (xcoo(j),j=i,i+2),&
                 iatsym(i), ' ', atsym(i)
         enddo

      else if (loqchem) then
         do i=1,ncoo,3
            write(icom,'(a3,3F14.7)') atsym(i),(xcoo(j),j=i,i+2)
         enddo

      else if (lmolpro) then
         countN=0
         countC=0
         countH=0
         countF=0
         countO=0
         countS=0
         countCl=0
         do i=1,ncoo,3
            if (lsubpnt.eqv..true.) then
               if (i.eq.(tag*3-2)) then
                  if (sysatsym.eq.'C') then
                     countC=countC+1
                     write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                          sysatsym,countC,xsys0(1),xsys0(2),xsys0(3)
                  else if (sysatsym.eq.'N') then
                     countN=countN+1
                     write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                          sysatsym,countN,xsys0(1),xsys0(2),xsys0(3)
                  else if (sysatsym.eq.'H') then
                     countH=countH+1
                     write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                          sysatsym,countH,xsys0(1),xsys0(2),xsys0(3)
                  else if (sysatsym.eq.'F') then
                     countF=countF+1
                     write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                          sysatsym,countF,xsys0(1),xsys0(2),xsys0(3)
                  else if (sysatsym.eq.'O') then
                     countO=countO+1
                     write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                          sysatsym,countO,xsys0(1),xsys0(2),xsys0(3)
                  else if (sysatsym.eq.'S') then
                     countS=countS+2
                        write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                           sysatsym,countS,xsys0(1),xsys0(2),xsys0(3)
                  else if (sysatsym.eq.'Cl') then
                     countCl=countCl+1
                        write(icom,'(a2,i1,",",F12.5,",",F12.5,",",F12.5)') &
                           sysatsym,countCl,xsys0(1),xsys0(2),xsys0(3)
                  else
                     routine = 'Comout (vcpnt.F90)'
                     message = 'Unknown atom type in writing Molpro input file (subpnt): '//atsym(i)
                     call vcerrormsg
                  endif
              endif
            endif

            if (lsubatm) then
               if ( (i-1)/3 +1 .eq.isubatm) then
                  do j=1,nsubatm
                     if (sublbl(j).eq.'C') then
                        countC=countC+1
                        write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                             sublbl(j),countC,subxcoo(j,1)+xcoo(i)-xcoo0(i),&
                             subxcoo(j,2)+xcoo(i+1)-xcoo0(i+1),&
                             subxcoo(j,3)+xcoo(i+2)-xcoo0(i+2)
                     else if (sublbl(j).eq.'N') then
                        countN=countN+1
                        write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                             sublbl(j),countN,subxcoo(j,1)+xcoo(i)-xcoo0(i),&
                             subxcoo(j,2)+xcoo(i+1)-xcoo0(i+1),&
                             subxcoo(j,3)+xcoo(i+2)-xcoo0(i+2)
                     else if (sublbl(j).eq.'H') then
                        countH=countH+1
                        write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                             sublbl(j),countH,subxcoo(j,1)+xcoo(i)-xcoo0(i),&
                             subxcoo(j,2)+xcoo(i+1)-xcoo0(i+1),&
                             subxcoo(j,3)+xcoo(i+2)-xcoo0(i+2)
                     else if (sublbl(j).eq.'F') then
                        countF=countF+1
                        write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                             sublbl(j),countF,subxcoo(j,1)+xcoo(i)-xcoo0(i),&
                             subxcoo(j,2)+xcoo(i+1)-xcoo0(i+1),&
                             subxcoo(j,3)+xcoo(i+2)-xcoo0(i+2)
                     else if (sublbl(j).eq.'O') then
                        countO=countO+1
                        write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                             sublbl(j),countO,subxcoo(j,1)+xcoo(i)-xcoo0(i),&
                             subxcoo(j,2)+xcoo(i+1)-xcoo0(i+1),&
                             subxcoo(j,3)+xcoo(i+2)-xcoo0(i+2)
                     else if (sublbl(j).eq.'S') then
                        countS=countS+1
                        write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                              sublbl(j),countS,subxcoo(j,1)+xcoo(i)-xcoo0(i),&
                              subxcoo(j,2)+xcoo(i+1)-xcoo0(i+1),&
                              subxcoo(j,3)+xcoo(i+2)-xcoo0(i+2)
                     else if (sublbl(j).eq.'Cl') then
                        countCl=countCl+1
                        write(icom,'(a2,i1,",",F12.5,",",F12.5,",",F12.5)') &
                              sublbl(j),countCl,subxcoo(j,1)+xcoo(i)-xcoo0(i),&
                              subxcoo(j,2)+xcoo(i+1)-xcoo0(i+1),&
                              subxcoo(j,3)+xcoo(i+2)-xcoo0(i+2)
                     else
                        routine = 'Comout (vcpnt.F90)'
                        message = 'Unknown atom type in writing Molpro input file (subatm): '//atsym(i)
                        call vcerrormsg
                     endif
                     if (sublbl(j).eq.'F') then
                        countF=countF+1
                        write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                             sublbl(j),countF,subxcoo(j,1)+xcoo(i)-xcoo0(i),&
                             subxcoo(j,2)+xcoo(i+1)-xcoo0(i+1),&
                             subxcoo(j,3)+xcoo(i+2)-xcoo0(i+2)
                     endif
                  enddo
                  goto 15
               endif
            endif
            
	    if (atsym(i).eq.'C') then
               countC=countC+1
               write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                     atsym(i),countC,xcoo(i),xcoo(i+1),xcoo(i+2)
            else if (atsym(i).eq.'N') then
               countN=countN+1
               write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                    atsym(i),countN,xcoo(i),xcoo(i+1),xcoo(i+2)
            else if (atsym(i).eq.'H') then
               countH=countH+1
               write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                    atsym(i),countH,xcoo(i),xcoo(i+1),xcoo(i+2)
            else if (atsym(i).eq.'O') then
               countO=countO+1
               write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                    atsym(i),countO,xcoo(i),xcoo(i+1),xcoo(i+2)
            else if (atsym(i).eq.'S') then
               countS=countS+1
               write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                     atsym(i),countS,xcoo(i),xcoo(i+1),xcoo(i+2)
            else if (atsym(i).eq.'Cl') then
               countCl=countCl+1
               write(icom,'(a2,i1,",",F12.5,",",F12.5,",",F12.5)') &
                     atsym(i),countCl,xcoo(i),xcoo(i+1),xcoo(i+2)
            else if (atsym(i).eq.'F') then
               countF=countF+1
               write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                    atsym(i),countF,xcoo(i),xcoo(i+1),xcoo(i+2)
            else
               routine = 'Comout (vcpnt.F90)'
               message = 'Unknown atom type in writing Molpro input file: '//atsym(i)
               call vcerrormsg
            endif

15          continue

! Dummy atoms
            if (ndum.gt.0) then
               if (ldumcom.eqv..false.) then
                  do j=1,ndum*3,3
                     write(icom,'(a1,",",F12.5,",",F12.5,",",F12.5)') &
                          dumatsym(j),dumxcoo(j),dumxcoo(j+1),dumxcoo(j+2)
                  enddo
               else
                  call calccom(xcoo,com,tag)
                  write(icom,'(a1,",",F12.5,",",F12.5,",",F12.5)') &
                       dumatsym(1),com(1),com(2),com(3)
               endif
            endif
         enddo

      else if (ldalton) then
         do i=1,ncoo,3
            if (iatsym(i) .eq. iatsym(i+3)) then
               write(icom,'(a3,F12.5,F12.5,F12.5)')&
                  atsym(i),xcoo(i),xcoo(i+1),xcoo(i+2)
            else
               if (iatsym(i+3) .ne. 0) then
                  write(icom,'(a3,F12.5,F12.5,F12.5)')&
                     atsym(i),xcoo(i),xcoo(i+1),xcoo(i+2)
                  write(icom,'(a18)') 'Charge=1.0 Atoms=6'
               else  
                  write(icom,'(a3,F12.5,F12.5,F12.5)')&
                     atsym(i),xcoo(i),xcoo(i+1),xcoo(i+2)
               endif
            endif 
         enddo

      else if (lmolcas) then
         write(icom,'(i2)') ncoo/3
         write(icom,*) 
         do i=1,ncoo,3
            if (lsubpnt.eqv..true.) then
               if (i.eq.(tag*3-2)) then
                     write(icom,'(a3,3F14.7)') sysatsym,(xsys0(j),j=1,3)
              endif
            endif
            write(icom,'(a3,3F14.7)') atsym(i),(xcoo(j),j=i,i+2)

! Dummy atoms
            if (ndum.gt.0) then
               if (ldumcom.eqv..false.) then
                  do j=1,ndum*3,3
                     write(icom,'(a1,"  ",F12.7," ",F12.7," ",F12.7)') &
                          dumatsym(j),dumxcoo(j),dumxcoo(j+1),dumxcoo(j+2)
                  enddo
               else
                  call calccom(xcoo,com,tag)
                  write(icom,'(a1,"  ",F12.7," ",F12.7," ",F12.7)') &
                       dumatsym(1),com(1),com(2),com(3)
               endif
            endif
         enddo

    else if (lturbomole) then
!        do i = 1,ncoo,3
!          print*, 'atom =', atsym(i)
!        enddo
!          print*, sysatsym
         do i=1,ncoo,3
            if (lsubpnt.eqv..true.) then
               if (i.eq.(tag*3-2)) then
                  write(icom,'(3x,F17.13,5x,F17.13,5x,F17.13,6x,a2)') &
                       xsys0(1)/angstroem,xsys0(2)/angstroem,xsys0(3)/angstroem,sysatsym
               endif
            endif
            if (lsubatm) then
               if ( (i-1)/3 +1 .eq.isubatm) then
                  do j=1,nsubatm
                     write(icom,'(3x,F17.13,5x,F17.13,5x,F17.13,6x,a2)') &
                          (subxcoo(j,1)+xcoo(i)-xcoo0(i))/angstroem,&
                          (subxcoo(j,2)+xcoo(i+1)-xcoo0(i+1))/angstroem,&
                          (subxcoo(j,3)+xcoo(i+2)-xcoo0(i+2))/angstroem,sublbl(j)
                  enddo
                  goto 17
               endif
            endif
            write(icom,'(3x,F17.13,5x,F17.13,5x,F17.13,6x,a3)') &
                 xcoo(i)/angstroem,xcoo(i+1)/angstroem,xcoo(i+2)/angstroem,atsym(i)
17          continue
         enddo

      if (ndum.gt.0) then
         if (ldumcom.eqv..false.) then
            do i=1,ndum*3,3
               write(icom,'(3x,F17.13,5x,F17.13,5x,F17.13,6x,a2)') &
                    dumxcoo(i)/angstroem,dumxcoo(i+1)/angstroem, &
                    dumxcoo(i+2)/angstroem,dumatsym(i)
            enddo
         else
            call calccom(xcoo,com,tag)
            write(icom,'(3x,F17.13,5x,F17.13,5x,F17.13,6x,a2)') &
                 com(1)/angstroem,com(2)/angstroem,com(3)/angstroem,dumatsym(1)
         endif
      endif
      
   else if (lgamessus) then
      do i=1,ncoo,3
         write(icom,'(a1,a1,x,i3,a2,3(a1,x,F14.10))') &
              atsym(i),',',iatsym(i),'.0',',',&
              xcoo(i),',',xcoo(i+1),',',xcoo(i+2)
      enddo
   
   else if (lmolcas) then
      do i=1,ncoo,3
         if (lsubpnt.eqv..true.) then
            if (i.eq.(tag*3-2)) then
               write(icom,'(a3,3F14.7)') sysatsym,(xsys0(j),j=1,3)
            endif
         endif
         write(icom,'(a3,3F14.7)') atsym(i),(xcoo(j),j=i,i+2)
      enddo
      write(icom,*)

   endif
   
   if (lturbomole) then
      write(icom,'(a4)') '$end'
   else if (loqchem) then
      if (lcl) then
         rewind(icl)
65       continue
         read(icl,'(a)') string
         if (string .ne. 'end-closing') then
            !call replacemeta(string,fileout1)
            write(icom,'(a)') string
            goto 65
         endif
      endif
   else
      if (lcl) then
         rewind(icl)
20       continue
         read(icl,'(a)') string
         if (string .ne. 'end-closing') then
            call replacemeta(string,fileout1)
            write(icom,'(a)') string
            goto 20
         endif
      endif
   endif
   
   return
   
 end subroutine comout

!#####################################################################
!
! dim = dimensionality of cut
!
!#####################################################################

      subroutine getfilename(p,p1,p2,dim,fileout,fileout1,&
                 cmode,cmode1,cmode2,set,lmolpro,ldalton,lturbomole,&
                 lmolcas)


      implicit none

      integer(long)                 :: jlbl
      integer(long), intent(in)     :: p,p1,p2,dim,cmode,cmode1,cmode2,set
      character(len=*), intent(in)  :: fileout
      character(len=*), intent(out) :: fileout1
      logical(kind=4), intent(in)   :: lmolpro,ldalton,lturbomole,lmolcas

      fileout1=fileout
      jlbl=index(fileout1,' ')

      if (dim .eq. 0) then
         write(fileout1(jlbl:jlbl+3),'(a4)')
      endif
      if (dim .ge. 1) then
         if (lvec) then
            write(fileout1(jlbl:jlbl+3),'(a4)') '_vec'
         else
            write(fileout1(jlbl:jlbl+4),'(a1,i4)') '_',cmode
         endif
         call rmblnk(fileout1)
         jlbl=index(fileout1,' ')
         if (p .lt. 0) then
            write(fileout1(jlbl:jlbl+3),'(a1,i2.2,a1)') '_',-p,'l'
         else if (p .eq. 0) then
            write(fileout1(jlbl:jlbl+3),'(a4)') '_000'
         else
            write(fileout1(jlbl:jlbl+3),'(a1,i2.2,a1)') '_',p,'r'
         endif
      endif
      if (dim .ge. 2) then
         jlbl=jlbl+5
         write(fileout1(jlbl:jlbl+4),'(a1,i4)') '_',cmode1
         call rmblnk(fileout1)
         jlbl=index(fileout1,' ')
         if (p1 .lt. 0) then
            write(fileout1(jlbl:jlbl+3),'(a1,i2.2,a1)') '_',-p1,'l'
         else if (p1 .eq. 0) then
            write(fileout1(jlbl:jlbl+3),'(a4)') '_000'
         else
            write(fileout1(jlbl:jlbl+3),'(a1,i2.2,a1)') '_',p1,'r'
         endif
      endif
      if (dim .eq. 3) then
         jlbl=jlbl+5
         write(fileout1(jlbl:jlbl+4),'(a1,i4)') '_',cmode2
         call rmblnk(fileout1)
         jlbl=index(fileout1,' ')
         if (p2 .lt. 0) then
            write(fileout1(jlbl:jlbl+3),'(a1,i2.2,a1)') '_',-p2,'l'
         else if (p2 .eq. 0) then
            write(fileout1(jlbl:jlbl+3),'(a4)') '_000'
         else
            write(fileout1(jlbl:jlbl+3),'(a1,i2.2,a1)') '_',p2,'r'
         endif
      endif
      jlbl=jlbl+4
      if (set .gt. 0) then
         write(fileout1(jlbl:jlbl+4),'(a1,i2.2)') '_',set
         jlbl=jlbl+3
      endif

      if (lmolpro) then
         write(fileout1(jlbl:jlbl+3),'(a4)') '.inp'
      else if (lmolcas) then
         write(fileout1(jlbl:jlbl+5),'(a6)') '.input'
      else if (ldalton) then
         write(fileout1(jlbl:jlbl+3),'(a4)') '.mol'
      else if (lturbomole) then
! do nothing: annoying coord files are used by TURBOMOLE
      else if (lgamessus) then
         write(fileout1(jlbl:jlbl+3),'(a4)') '.inp'
      else if (loxyz) then
         write(fileout1(jlbl:jlbl+3),'(a4)') '.xyz'
      else if (loqchem) then
         write(fileout1(jlbl:jlbl+3),'(a4)') '.inp'
      else
         write(fileout1(jlbl:jlbl+3),'(a4)') '.com'
      endif
 
      return

      end subroutine getfilename

!#######################################################################

      subroutine wrgo(igo,subcmd,fileout1,iin,lgocmd)

      implicit none

      integer(long)                :: ilbl,jlbl
      integer(long), intent(in)    :: igo,iin
      character(len=*), intent(in) :: subcmd,fileout1
      character(len=80)            :: string
      logical(kind=4), intent(in)  :: lgocmd
      logical(kind=4)              :: isopen
 
      inquire(unit=iin,opened=isopen)
      if (.not. isopen) then
         open(iin,file=filein,form='formatted',status='old')
      endif
!
! get lines to be written to go file
!
      if (lgocmd) then
         rewind(iin)
   10    continue
         read(iin,'(a)') string
         if (string .ne. 'go_cmd') go to 10

   20    continue
         read(iin,'(a)') string
         if (string .ne. 'end-go_cmd') then
            call replacemeta(string,fileout1)

            write(igo,'(80a)') string
            goto 20
         endif

      else
         ilbl = index(subcmd,' ')
         jlbl = index(fileout1,' ')
         write(igo,'(4a)') subcmd(1:ilbl),fileout1(1:jlbl-1)
      endif

      if (.not. isopen) then
         close(iin)
      endif

      return
      end subroutine wrgo

!#######################################################################

      subroutine replacemeta(string,filename)

      implicit none

      integer(long), external         :: slen
      integer(long)                   :: ilbl,jlbl,klbl,llbl
      character(len=*), intent(in)    :: filename
      character(len=*), intent(inout) :: string
      character(len=80)               :: substring,filestem

      ilbl=index(filename,'.')-1
      if (ilbl .le. 0) then
         filestem=filename
      else
         filestem=filename(1:ilbl)
      endif
!
! replace meta-strings
!
      ilbl = index(string,'@')
      if (ilbl .ne. 0) then
         klbl = slen(string)
         jlbl = index(string(ilbl+1:klbl),'@')
         if (jlbl .eq. 0) then
            routine='Replacemeta'
            message='Missing second @ in metastring'
            call vcerrormsg
         endif
         jlbl=ilbl+jlbl
         read(string(ilbl+1:jlbl-1),'(a)') substring

         if (substring .eq. 'filestem') then
            llbl=slen(filestem)
            string=string(1:ilbl-1)//filestem(1:llbl)//&
                            string(jlbl+1:klbl)
         endif
      else
         !MAP use $ instrad of @ as in documentation
         ilbl = index(string,'$')
         if (ilbl .ne. 0) then
            klbl = slen(string)
            jlbl = index(string(ilbl+1:klbl),'$')
            if (jlbl .eq. 0) then
               routine='Replacemeta'
               message='Missing second $ in metastring'
               call vcerrormsg
            endif
            jlbl=ilbl+jlbl
            read(string(ilbl+1:jlbl-1),'(a)') substring
   
            if (substring .eq. 'filestem') then
               llbl=slen(filestem)
               string=string(1:ilbl-1)//filestem(1:llbl)//&
                               string(jlbl+1:klbl)
            endif
         endif
      endif

      return
      end subroutine replacemeta

!#######################################################################

      subroutine replacemode(p,p1)
      
      implicit none

      integer(long) :: i,j,k,l,pini,p,p1
      real(dop)     :: xdiff,ydiff,zdiff,xdiff1,ydiff1,zdiff1,norm,norm1,r0,&
                       theta0,phi0,r,theta,phi,incr,incr1,ang1,ang2,ang3,transx, &
                       transy,transz
      real(dop), dimension(maxdim) :: tmpxcoo,inixcoo,xcoo
      logical negx,negz

      
! If cut involves fragment modes, use full parent cartesian coords
      if (lsubpnt.eqv..true.) then
         do i=1,ncoo
            inixcoo(i)=xcoo0(i)
         enddo
         ncoo=ncoo+3
         do i=1,ncoo
            xcoo0(i)=parxcoo0(i)
         enddo
      endif

! Determine increments
      if (rmode.eq.cmode) incr=xincr
      if (rmode.eq.cmode1) incr=xincr1
      if (rmode1.eq.cmode) incr1=xincr
      if (rmode1.eq.cmode1) incr1=xincr1
      if (lvec) incr=(q(rmode)-qini(rmode))/ndat
      if (lvec.and.lreplacediag) incr1=(q(rmode1)-qini(rmode1))/ndat

! Orient molecule
      call translate0(tmpxcoo,transx,transy,transz)
      call rotate1(tmpxcoo,ang1,ang2,ang3,negx,negz)

      xdiff=tmpxcoo(ratm1*3-2)-tmpxcoo(ratm2*3-2)
      ydiff=tmpxcoo(ratm1*3-1)-tmpxcoo(ratm2*3-1)
      zdiff=tmpxcoo(ratm1*3)-tmpxcoo(ratm2*3)
      
! Determine reference coords
      r0=xdiff**2+ydiff**2+zdiff**2
      r0=sqrt(r0)
      phi0=tmpxcoo(ratm1*3)/r0
      phi0=acos(phi0)
      theta0=tmpxcoo(ratm1*3-1)/tmpxcoo(ratm1*3)
      theta0=atan(theta0)      

! If cut is along a vector, reference coords have to correspond to the initial point (which may differ from Q0)
      if (lvec) then
         if (.not.lsubpnt) then
            r0=r0+qini(nmodes-2)
            theta0=theta0+(qini(nmodes-1)*pi/180)
            phi0=phi0+(qini(nmodes)*pi/180)
         else
            r0=r0+qini(nmodes+1)
            theta0=theta0+(qini(nmodes+2)*pi/180)
            phi0=phi0+(qini(nmodes+3)*pi/180)
         endif
      endif

! Calculate system atom coordinates
      if (lreplacediag.neqv..true.) then

         if (lbloc2) then
! cmode1 always corresponds to the 'system' mode, so have to use p1
            pini=p
            p=p1
! ensure that the centre of the grid is correct
            if (x01.ne.0) then
               if (rdef.eq.'r') r0=r0+x01
               if (rdef.eq.'phi') phi0=phi0+(x01*pi/180)
               if (rdef.eq.'theta') theta0=theta0+(x01*pi/180)
            endif
         endif

         if (rdef.eq.'r') then
            r=r0+p*incr
            tmpxcoo(ratm1*3-2)=R*sin(phi0)
            tmpxcoo(ratm1*3-1)=R*sin(theta0)*cos(phi0)
            tmpxcoo(ratm1*3)=R*cos(theta0)*cos(phi0)
         else if (rdef.eq.'phi') then
            phi=phi0+p*(incr*pi/180)
            tmpxcoo(ratm1*3-2)=R0*sin(phi)
            tmpxcoo(ratm1*3-1)=R0*sin(theta0)*cos(phi)
            tmpxcoo(ratm1*3)=R0*cos(theta0)*cos(phi)
         else if (rdef.eq.'theta') then
            theta=theta0+p*(incr*pi/180)
            tmpxcoo(ratm1*3-2)=R0*sin(phi0)
            tmpxcoo(ratm1*3-1)=R0*sin(theta)*cos(phi0)
            tmpxcoo(ratm1*3)=R0*cos(theta)*cos(phi0)
         else
            write(6,'(2a)') 'Unknown replacement mode type: ',rdef
            stop
         endif

         if (lbloc2) p=pini

      else
         r=r0
         theta=theta0
         phi=phi0
         if (rdef.eq.'r') then
            r=r0+p*incr
         else if (rdef.eq.'phi') then
            phi=phi0+p*(incr*pi/180)
         else if (rdef.eq.'theta') then
            theta=theta0+p*(incr*pi/180)
         endif
! N.B., between here
         if (rdef1.eq.'r') then
            r=r0+p*incr1
         else if (rdef1.eq.'phi') then
             phi=phi0+p*(incr1*pi/180)
        else if (rdef1.eq.'theta') then
            theta=theta0+p*(incr1*pi/180)
         endif
! and here, p used to be p1

         tmpxcoo(ratm1*3-2)=R*sin(phi)
         tmpxcoo(ratm1*3-1)=R*sin(theta)*cos(phi)
         tmpxcoo(ratm1*3)=R*cos(theta)*cos(phi)

      endif

! Re-orient molecule 
      call reorient(tmpxcoo,ang1,ang2,ang3,negx,negz,transx,transy,transz)

! Save coordinates
      if (lsubpnt) then
         do i=1,3
            xsys0(i)=tmpxcoo((ratm1*3)-3+i)
         enddo
         ncoo=ncoo-3
         do i=1,ncoo
            xcoo0(i)=inixcoo(i)
         enddo
      else
         do i=1,ncoo
            xcoo(i)=tmpxcoo(i)
         enddo
      endif

      return
      end subroutine replacemode

!#######################################################################

      subroutine translate0(tmpxcoo,transx,transy,transz)

      implicit none

      integer(long) :: i,j,k,l
      real(dop) :: xdiff,ydiff,zdiff
      real(dop), dimension(maxdim), intent(out) :: tmpxcoo
      real(dop), intent(out) :: transx,transy,transz

      transx=xcoo0(ratm2*3-2)
      transy=xcoo0(ratm2*3-1)
      transz=xcoo0(ratm2*3)

      do i=1,ncoo/3
         j=i*3-2
         k=i*3-1
         l=i*3
         tmpxcoo(j)=xcoo0(j)-transx
         tmpxcoo(k)=xcoo0(k)-transy
         tmpxcoo(l)=xcoo0(l)-transz
      enddo

      return
      end subroutine translate0

!#######################################################################

      subroutine rotate1(tmpxcoo,ang1,ang2,ang3,negx,negz)

      implicit none

      integer(long) :: i,j,k,l
      real(dop), dimension(maxdim), intent(inout) :: tmpxcoo
      real(dop), dimension(maxdim) :: work
      real(dop), intent(out) :: ang1,ang2,ang3
      real(dop) :: stheta,ctheta
      real(dop), dimension(3) :: tmp,mid
      logical(kind=4), intent(out) :: negx,negz

      negx=.false.
      negz=.false.
      
!-----------------------------------------------------------------------
! Step (i): rotation about y-axis such that mid(x)=0
!-----------------------------------------------------------------------
! Calculate midpoint between ratm3 and ratm4
      do i=1,3
         tmp(i)=tmpxcoo(ratm4*3-3+i)-tmpxcoo(ratm3*3-3+i)
         mid(i)=tmpxcoo(ratm3*3-3+i)+tmp(i)/2
      enddo

      if (mid(1).ne.0) then

! Calculate angle between midpoint and z-axis
         ang1=mid(1)/mid(3)
         ang1=atan(ang1)

! Ensure that we rotate in the correct direction
         if (ang1.gt.0) ang1=-ang1
         if (mid(3).lt.0) then
            if (mid(1).gt.0) ang1=-ang1
         else if (mid(3).gt.0) then
            if (mid(1).lt.0) ang1=-ang1
         endif
         
         stheta=sin(ang1)
         ctheta=cos(ang1)
         
! Rotate around y-axis by ang1
         do i=1,ncoo
            work(i)=0
         enddo
         
         do i=1,ncoo/3
            j=i*3-2
            k=i*3-1
            l=i*3
            work(j) = tmpxcoo(j)*ctheta + tmpxcoo(l)*stheta
            work(k) = tmpxcoo(k)
            work(l) = -tmpxcoo(j)*stheta + tmpxcoo(l)*ctheta
         enddo

         do i=1,ncoo
            tmpxcoo(i)=work(i)
         enddo
         
      endif
         
      do i=1,ncoo
         work(i)=0
      enddo

!-----------------------------------------------------------------------
! Step (ii): rotation about x-axis such that mid(y)=0
!-----------------------------------------------------------------------
! Calculate midpoint between ratm3 and ratm4
      do i=1,3
         tmp(i)=tmpxcoo(ratm4*3-3+i)-tmpxcoo(ratm3*3-3+i)
         mid(i)=tmpxcoo(ratm3*3-3+i)+tmp(i)/2
      enddo

      if (mid(2).ne.0) then

! Calculate angle between midpoint and z-axis
         ang2=mid(2)/mid(3)
         ang2=atan(ang2)
      
! Ensure that we rotate in the correct direction
         if (ang2.lt.0) ang2=-ang2
         if (mid(2).lt.0) then
            if (mid(3).gt.0) ang2=-ang2
         else if (mid(2).gt.0) then
            if (mid(3).lt.0) ang2=-ang2
         endif
         
         stheta=sin(ang2)
         ctheta=cos(ang2)

! Rotate around x-axis by ang2
         do i=1,ncoo/3
            j=i*3-2
            k=i*3-1
            l=i*3
            work(j) = tmpxcoo(j)
            work(k) = tmpxcoo(k)*ctheta - tmpxcoo(l)*stheta
            work(l) = tmpxcoo(k)*stheta + tmpxcoo(l)*ctheta
         enddo

         do i=1,ncoo
            tmpxcoo(i)=work(i)
         enddo
      
      endif

      do i=1,ncoo
         work(i)=0
      enddo

!-----------------------------------------------------------------------
! Step (iii): rotation about z-axis such that ratm1(y)=0
!-----------------------------------------------------------------------

      if (tmpxcoo(ratm1*3-1).eq.0) then
         ang3=0
      else

! Calculate angle between ratm1 and x-axis
         ang3=tmpxcoo(ratm1*3-1)/tmpxcoo(ratm1*3-2)
         ang3=atan(ang3)

! Ensure that we rotate in the correct direction
         if (ang3.gt.0) ang3=-ang3
         if (tmpxcoo(ratm1*3-1).lt.0) then
            if (tmpxcoo(ratm1*3-2).gt.0) ang3=-ang3
         else if (tmpxcoo(ratm1*3-1).gt.0) then
            if (tmpxcoo(ratm1*3-2).lt.0) ang3=-ang3
         endif
         
         stheta=sin(ang3)
         ctheta=cos(ang3)      
         
! Rotate around z-axis by ang3
         do i=1,ncoo/3
            j=i*3-2
            k=i*3-1
            l=i*3
            work(j) = tmpxcoo(j)*ctheta - tmpxcoo(k)*stheta
            work(k) = tmpxcoo(j)*stheta + tmpxcoo(k)*ctheta
            work(l) = tmpxcoo(l)
         enddo
         
         do i=1,ncoo
            tmpxcoo(i)=work(i)
         enddo
         
      endif
      
! Ensure that ratm1 has +ve x- and z-components
      if (tmpxcoo(ratm1*3-2).lt.0) then
         negx=.true.
         do i=1,ncoo/3
            tmpxcoo(i*3-2)=-tmpxcoo(i*3-2)
         enddo
      endif
      
      if (tmpxcoo(ratm1*3).lt.0) then
         negz=.true.
         do i=1,ncoo/3
            tmpxcoo(i*3)=-tmpxcoo(i*3)
         enddo
      endif

      return
      end subroutine rotate1

!#######################################################################

      subroutine reorient(tmpxcoo,ang1,ang2,ang3,negx,negz,transx,transy,transz)

      implicit none

      integer(long) :: i,j,k,l
      real(dop), dimension(maxdim), intent(inout) :: tmpxcoo
      real(dop), dimension(maxdim) :: work
      real(dop), intent(out) :: ang1,ang2,ang3
      real(dop), intent(in) :: transx,transy,transz
      real(dop) :: stheta,ctheta,small
      real(dop), dimension(3) :: tmp,mid
      logical(kind=4), intent(out) :: negx,negz

     small=0.000000000000000000001
     
     if (abs(ang1).lt.small) ang1=0.0
     if (abs(ang2).lt.small) ang2=0.0
     if (abs(ang3).lt.small) ang3=0.0

!-----------------------------------------------------------------------
! Ensure x- and z-components have the correct sign
!-----------------------------------------------------------------------
      if (negz.eqv..true.) then
         do i=1,ncoo/3
            tmpxcoo(i*3)=-tmpxcoo(i*3)
         enddo
      endif

      if (negx.eqv..true.) then
         do i=1,ncoo/3
            tmpxcoo(i*3-2)=-tmpxcoo(i*3-2)
         enddo
      endif        

!-----------------------------------------------------------------------
! Rotation about z-axis by -ang3
!-----------------------------------------------------------------------
      do i=1,ncoo
         work(i)=0
      enddo
      
      stheta=sin(2*pi-ang3)
      ctheta=cos(2*pi-ang3)

      do i=1,ncoo/3
         j=i*3-2
         k=i*3-1
         l=i*3
         work(j) = tmpxcoo(j)*ctheta - tmpxcoo(k)*stheta
         work(k) = tmpxcoo(j)*stheta + tmpxcoo(k)*ctheta
         work(l) = tmpxcoo(l)
      enddo

      do i=1,ncoo
         tmpxcoo(i)=work(i)
      enddo
      
      do i=1,ncoo
         work(i)=0
      enddo

!-----------------------------------------------------------------------
! Rotation about x-axis by -ang2
!-----------------------------------------------------------------------
      stheta=sin(2*pi-ang2)
      ctheta=cos(2*pi-ang2)
      
      do i=1,ncoo/3
         j=i*3-2
         k=i*3-1
         l=i*3
         work(j) = tmpxcoo(j)
         work(k) = tmpxcoo(k)*ctheta - tmpxcoo(l)*stheta
         work(l) = tmpxcoo(k)*stheta + tmpxcoo(l)*ctheta
      enddo
      
      do i=1,ncoo
         tmpxcoo(i)=work(i)
      enddo
      
      do i=1,ncoo
         work(i)=0
      enddo

!-----------------------------------------------------------------------
! Rotation about y-axis by -ang1
!-----------------------------------------------------------------------
      stheta=sin(2*pi-ang1)
      ctheta=cos(2*pi-ang1)

      do i=1,ncoo/3
         j=i*3-2
         k=i*3-1
         l=i*3
         work(j) = tmpxcoo(j)*ctheta + tmpxcoo(l)*stheta
         work(k) = tmpxcoo(k)
         work(l) = -tmpxcoo(j)*stheta + tmpxcoo(l)*ctheta
      enddo

!-----------------------------------------------------------------------
! Translation
!-----------------------------------------------------------------------
      do i=1,ncoo/3
         j=i*3-2
         k=i*3-1
         l=i*3
         tmpxcoo(j)=work(j)+transx
         tmpxcoo(k)=work(k)+transy
         tmpxcoo(l)=work(l)+transz
      enddo

      return
      end subroutine reorient

!##############################################################
 
      subroutine calccom(xcoo,com,tag)
 
      implicit none
 
      real(dop) :: tmass,sysmass
      real(dop), dimension(3), intent(out) :: com
      integer(long) :: i,k
      integer(long), intent(in) :: tag
      real(dop), dimension(maxdim), intent(in) :: xcoo

      tmass=0.0
      do k=1,ncoo/3
         tmass=tmass+mass(k*3)
      enddo

      if (lsubpnt) then
         if (sysatsym.eq.'H') sysmass=1.0078999996185303
         if (sysatsym.eq.'C') sysmass=12.010999679565430
         if (sysatsym.eq.'N') sysmass=14.006699562072754
         tmass=tmass+sysmass
      endif

      do i=1,3
         com(i)=0.0
         do k=1,ncoo/3
            com(i)=com(i)+xcoo(k*3-(3-i))*mass(k*3-(3-i))/tmass
         enddo
         if (lsubpnt) com(i)=com(i)+xsys(i)*sysmass/tmass
      enddo

      return
      end subroutine calccom

      end program vcpnt
