  program vcrotation

    use decimal, only: long,dop
    use vcglobal
    use global, only:pi, keyword,keyorig,maxkey,lc,lwarn
    use maxv, only: maxdim,maxsta,maxset
    use vchammod
    use vctransmod
    use versions
    use utils, only: vcwarnmsg,vcerrormsg,vchdefault
    use iofiles, only: rdfile0
    use ioqc, only: nm2xmat2
    use logdat, only: message, routine, inptit

    implicit none

    integer(long)                           :: i,j,iau
    real(dop), dimension(:,:), allocatable  :: gnmcoo,gcoonm,enmcoo,ecoonm,&
         dmat,rhess
    character(len=80)                       :: gfile,efile
    character(len=16), dimension(:), allocatable :: gnmlab,enmlab
    real(dop), dimension(:), allocatable         :: gfreq0,efreq0
    logical(kind=4)                             :: lsocdegen

!-----------------------------------------------------------------------
! Read input file
!-----------------------------------------------------------------------
    ncoo=-999
    call vcrinp(gfile,efile)

    maxdim=ncoo
    maxsta=1
    if (maxdim .lt. 0) then
       routine='VCROTATION'
       message='Error reading dimensions from input file'
       call vcerrormsg
    endif

    call alloc_vchammod
    call alloc_vctransmod

    allocate(gnmcoo(maxdim,maxdim))
    allocate(gcoonm(maxdim,maxdim))
    allocate(enmcoo(maxdim,maxdim))
    allocate(ecoonm(maxdim,maxdim))
    allocate(dmat(maxdim,maxdim))
    allocate(rhess(maxdim,maxdim))
    allocate(gnmlab(maxdim))
    allocate(enmlab(maxdim))
    allocate(gfreq0(maxdim))
    allocate(efreq0(maxdim))

!-----------------------------------------------------------------------
! Set defaults
!-----------------------------------------------------------------------
    call vchdefault

!-----------------------------------------------------------------------
! Read ground state normal modes
!-----------------------------------------------------------------------
    file0=gfile
    file0orient='standard'
    iau=1
    lsocdegen=.true.
    call rdfile0(.false.,abintype0,iau,lsocdegen)
    call nm2xmat2(nmcoo,coonm,mass,ncoo,maxdim)
  
    do i=1,ncoo
       do j=1,nmodes
          gnmcoo(i,j)=nmcoo(i,j)
          gcoonm(j,i)=coonm(j,i)
          gfreq0(j)=freq0(j)
          gnmlab(j)=nmlab(j)
       enddo
    enddo

!-----------------------------------------------------------------------
! Read excited state normal modes
!-----------------------------------------------------------------------
    file0=efile
    file0orient='standard'
    iau=1
    lsocdegen=.true.
    call rdfile0(.false.,abintype0,iau,lsocdegen)
    call nm2xmat2(nmcoo,coonm,mass,ncoo,maxdim)

    do i=1,ncoo
       do j=1,nmodes
          enmcoo(i,j)=nmcoo(i,j)
          ecoonm(j,i)=coonm(j,i)
          efreq0(j)=freq0(j)
          enmlab(j)=nmlab(j)
       enddo
    enddo

!-----------------------------------------------------------------------
! Calculate the Duschinsky rotation matrix and transform the excited 
! state Hessian
!-----------------------------------------------------------------------
    call rotatehess(gnmcoo,gcoonm,enmcoo,ecoonm,dmat,rhess,gfreq0,efreq0)

!-----------------------------------------------------------------------
! Output the rotation matrix and the excited-state Hessian in terms of
! the ground-state normal modes
!-----------------------------------------------------------------------
    call wrrotation(dmat,rhess,gnmlab,enmlab)

    deallocate(gnmcoo,gcoonm,enmcoo,ecoonm,dmat,rhess)
    deallocate(gnmlab,enmlab)
    deallocate(gfreq0,efreq0)

    STOP
  contains

!#####################################################################

    subroutine vcrinp(gfile,efile)

      implicit none
      
      integer(long)           :: i,iarg,ilbl,jlbl,c,iargc
      character(len=80)       :: string,gfile,efile

      gfile=''
      efile=''
      fileout=''
      nmodes=0
      ncoo=0

      i=0
100   i=i+1
      call mygetarg(i,string)

      if (string.eq.'-g') then
         i=i+1
         call mygetarg(i,string)
         read(string,*) gfile        
      else if (string.eq.'-e') then
         i=i+1
         call mygetarg(i,string)
         read(string,*) efile      
      else if (string.eq.'-nmodes') then
         i=i+1
         call mygetarg(i,string)
         read(string,*) nmodes      
      else if (string.eq.'-o') then
         i=i+1
         call mygetarg(i,string)
         read(string,*) fileout
      else
         write(6,'(2a)') ' unknown option : ', string
         STOP        
      endif

      if (i.lt.iargc()) goto 100

      if (gfile.eq.'') then
         write(6,'(/,a,/)') 'Ground state file not specified. Quitting!'
         STOP
      endif

      if (efile.eq.'') then
         write(6,'(/,a,/)') 'Excited state file not specified. Quitting!'
         STOP
      endif

      if (nmodes.eq.0) then
         write(6,'(/,a,/)') 'Number of normal modes not specified. Quitting!'
         STOP
      endif

      if (fileout.eq.'') fileout='rotation.out'

      ncoo=nmodes+6

    end subroutine vcrinp

!#####################################################################

    subroutine rotatehess(gnmcoo,gcoonm,enmcoo,ecoonm,dmat,rhess,gfreq0,&
         efreq0)
      
      implicit none

      integer(long)                        :: i,j,k,l
      real(dop), dimension(maxdim,maxdim)  :: gnmcoo,gcoonm,enmcoo,ecoonm,&
           dmat,hess,rhess
      real(dop), dimension(maxdim)         :: gfreq0,efreq0

!-----------------------------------------------------------------------
! Calculate the Duschinsky rotation matrix (dmat)
!-----------------------------------------------------------------------
      do j=1,nmodes
         do i=1,nmodes
            dmat(i,j)=0.0
            do k=1,ncoo
               dmat(i,j)=dmat(i,j)+ecoonm(i,k)*gnmcoo(k,j)
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! Calculate excited state Hessian in terms of the ground state normal
! modes.
!
! hess: excited-state Hessian in terms of the excited-state normal modes
! rhess: excited-state Hessian in terms of the ground-state normal modes
!-----------------------------------------------------------------------
      do j=1,nmodes
         do i=1,nmodes
            if(i.eq.j) then
               hess(i,j)=efreq0(i)*efreq0(i)
            else
               hess(i,j)=0.0
            endif
            rhess(i,j)=0.0
            do k=1,nmodes
               do l=1,nmodes
                  rhess(i,j)=rhess(i,j)+hess(k,l)*dmat(k,i)*dmat(l,j)
               enddo
            enddo
         enddo
      enddo

    end subroutine rotatehess

!#####################################################################
    
    subroutine wrrotation(dmat,rhess,gnmlab,enmlab)

      implicit none
      
      integer(long)                        :: i,j,j1,j2,k,l,l1,l2,m,unit,&
           nblk
      real(dop), dimension(maxdim,maxdim)  :: dmat,rhess    
      character(len=16), dimension(maxdim) :: gnmlab,enmlab
      real(dop), dimension(nmodes)         :: sumsq

      unit=65

      do i=1,nmodes
         sumsq(i)=0.0
         do j=1,nmodes
            sumsq(i)=sumsq(i)+dmat(i,j)**2
         enddo
      enddo

      open (unit,file=fileout,form='formatted',status='unknown')
      rewind(unit)

      write(unit,'(a,f8.5)') '# VCHam Version :',progver
      write(unit,'(/,a)') &
         '# Ground-state-to-excited-state normal mode rotation matrix'
      write(unit,'(a)')'# Columns: Excited state modes'
      write(unit,'(a,/)')'# Rows:    Ground state modes'
      k=0
      nblk=((nmodes-1)/5)+1
      do i=1,nblk
         l1=k+1
         l2=min(l1+4,nmodes)
         write(unit,'(10x,5(i2,x,a3,9x))') (l, enmlab(l), l=l1,l2)
         k=k+5
         do m=1,nmodes
            j1=(i-1)*5+1
            j2=min(j1+4,nmodes)
            write(unit,'(i2,x,a3,x,5F15.10)') m,gnmlab(m), &
                 (dmat(j,m),j=j1,j2)
         enddo
!         write(unit,'(/,7x,5F15.10)') (sumsq(l),l=l1,l2)
         write(unit,*)
      enddo

      write(unit,'(/,a,/)') &
         '# Excited-state Hessian in terms of the ground-state normal modes'
      k=0
      nblk=((nmodes-1)/5)+1
      do i=1,nblk
         l1=k+1
         l2=min(l1+4,nmodes)
         write(unit,'(10x,5(i2,13x))') (l, l=l1,l2)
         k=k+5
         do m=1,nmodes
            j1=(i-1)*5+1
            j2=min(j1+4,nmodes)
            write(unit,'(5F15.10)') (rhess(m,j),j=j1,j2)
         enddo
         write(unit,*)
      enddo


    end subroutine wrrotation

!#####################################################################
  end program vcrotation
