
      program vctrans

use decimal, only: long,dop,sip
use vcglobal
use global, only:pi, c1, c2, c5, keyword, keyorig, lc, maxkey, lwarn,&
                 ddname, ddlaenge, lsocbas, laenge
use maxv, only: maxdim,maxsta,maxset
use keyunitsmod
use vchammod
use vctransmod
use versions
use invchammod

use op1lib, only: zerovxd, zeromxd, cpvxd
use ioqc,   only: getintyp, getabintype, getxcoo, getnm, geten, getder1, &
                  getdcp, getder2, gettdip, getgd, nm2xmat, nm2xmat2
use readqc
use logdat, only: message, routine, inptit
use utils
use iofiles, only: rdfile0,cpfreq,rdpftfiles
use keyunitsmod

use mmlib, only: mmxxdd,mmtxdd

use griddatmod, only: nddstate
use dirdyn, only: alloc_dirdyn,dealloc_dirdyn,dbnrec,ndofddpes,ndofdd,&
                  alloc_dddb,dealloc_dddb,nroot,nbasis,dde0,dimref,nfam,&
                  dercpdim,nsocstate,nddmult,dbpntgp,nsymop,the_db,sqlvers

use dd_db, only: getdbnrec,preparedb
use db_sql, only: dbopen, dbclose, dbrdrefdb
   
      implicit none

      integer(long)            :: i,n,ierr,ilbl,p,ifiletyp,count,&
                                  filter,j,ndx
      integer(long)            :: iau  
      integer(long), external  :: slen
      character(len=c5)        :: filename

      real(dop)                :: energy
      real(dop), dimension(4)  :: tdip
      character(len=80), save :: fileop,filecl,filego,fileout1
      logical(kind=4), save   :: lop,lcl,lgo

      logical(kind=4)          :: lsocdegen = .true.
      logical(kind=4)          :: dat_ignore

      !Array for info to set up database  (see include/dirdyn.f90)
      integer(long), dimension(10)      :: infoarray
      !Array for passing flags for setting up database (see include/dirdyn.f90)
      logical(kind=4), dimension(20)    ::flagarray

      filename = ' '

!-----------------------------------------------------------------------
! Energy units for output
! iau = 0 : Hartree and Bohr
!       1 : eV and Angstrom (default)
!-----------------------------------------------------------------------
      iau = 1

!-----------------------------------------------------------------------
! Initialise calculation 
! get input file name, set maxdim and then allocate memory
!-----------------------------------------------------------------------
      call initvcham(1)

      if (maxdim .lt. 0) then
         routine='VCTRANS'
         message='Error reading dimensions from input file'
         call vcerrormsg
      endif
      if (maxsta .lt. 0) then
         routine='VCTRANS'
         message='Error reading no. states from input file'
         call vcerrormsg
      endif
! read input file to find number of dataset
      call rdsetsinit(iin,filein)

      call alloc_vctransmod
      call alloc_vchammod

!-----------------------------------------------------------------------
! set defaults
!-----------------------------------------------------------------------
      call vchdefault
      call vctdefault

!-----------------------------------------------------------------------
! read input file
! define output file etc.
!-----------------------------------------------------------------------
      call vctinp

      if (lwarn) then
         routine = 'VCTRANS'
         message = 'Finished with input error. See log file'
         call vcerrormsg
      endif

!-----------------------------------------------------------------------
! create parent-to-fragment transformation matrix
!-----------------------------------------------------------------------
      if (lpft) call rdpftfiles

!-----------------------------------------------------------------------
! get information from normal mode file
!-----------------------------------------------------------------------
      call rdfile0(lrde0,abintype0,iau,lsocdegen)

      if (lalleig .and. .not. lnmcurv ) call rotrans(nmcoo,maxdim,nmodes,&
                                             ncoo,mass,xcoo0)

      if (.not. lrdfreq0) call cpfreq(nmlab,nmlab1,freq0,freq1,nmodes)
      if (.not. lrde0) e0 = e01

!-----------------------------------------------------------------------
! rotate the reference geometry and normal modes to the Eckart frame
! if required
!-----------------------------------------------------------------------
      if (eck_frame) then
! Rotate reference coordinates to the Eckart frame         
         call vceckart(xcoo0,itvec,.false.)
! Subject eigenvectors of the Hessian to the same rotation
         call nmeckart(nmcoo,itvec,nmodes,ncoo,maxdim)
! If we are using the normal modes of a subset of the atoms, subject
! these to the same rotation
         if (lpft) call nmeckart(fragnmcoo,itvec,nmodes-3,ncoo,maxdim)
      endif

!-----------------------------------------------------------------------
! orthogonalise normal modes
!-----------------------------------------------------------------------
      if (lalleig .and. .not. lnmcurv) then

         call nmorthof(nmcoo,maxdim,ncoo,ilog)
      endif

      if (lrotmodes) then
         call rotatemodes(nmcoo,freq0,rotm,rotang,rotmxfq)
         print*, 'ROTATING?'
        j = 1
        do while (rotm(j,1) .ne. 0)
!	  the new rotated modes do not form a representation (symmetry species) unless they are both of the same one
          if(nmlab(rotm(j,1)) .ne. nmlab(rotm(j,2)) ) then
             nmlab(rotm(j,1))='X'
             nmlab(rotm(j,2))='X'
          endif
           j = j + 1
        enddo
      endif

!-----------------------------------------------------------------------
! get zero of internal coordinates
!-----------------------------------------------------------------------
      if (linternal) then
         call zerovxd(qcoo,nmodes)
         call q2intcoo(xcoo0,qcoo)
         call cpvxd(qcoo,intcoo0,nmodes)
      endif

!-----------------------------------------------------------------------
! write system and normal mode information to info file
!-----------------------------------------------------------------------
      call wrfile0(adjpft,adjqrt,fragnmcoo,fragnmlab)

!-----------------------------------------------------------------------
! create the transformation matrices
!-----------------------------------------------------------------------
      if (lalleig) then
         call nm2xmat2(nmcoo,coonm,mass,ncoo,maxdim)
      else
         call nm2xmat(nmcoo,coonm,iatsym,freq0,iau,ncoo,maxdim,nmodes,ilog)
      endif

      if (lwrtrafo) call wrnmcoo

!-----------------------------------------------------------------------
! Allocate memory and read DB
! Uses the DB as rediabatisation: preparedb modus 2   (no checks made)
!-----------------------------------------------------------------------
      if (ldd) then
         if (nsta .gt. 1) dercpdim=nsta*(nsta-1)/2   ! no. derivative couplings
         call alloc_dirdyn
         call alloc_dddb
         call preparedb(2,1)

         if (ldb0) then
            write(ilog,'(a)') ' Using only first DB dataset '
         endif
      endif

!-----------------------------------------------------------------------
! get information from data files
! and write to info file
!-----------------------------------------------------------------------
      write(iout,'(a)') '#dataset_section'
      laenge=slen(datadir)
      write(6,'(a)') 'Data Directory : '
      write(6,'(5x,a)') datadir(1:laenge)

      count=0
      p=nsta      
      ndx = 0
      do n=1,nsets
         dat_ignore = .true.   ! only read set if state to be included
         do i=1,nsta
            if (eigen(n) .eq. states(i)) dat_ignore = .false.
         enddo
         if (dat_ignore) cycle

         ndx = ndx + 1
         ierr=0
         if (allen(n).eq.1) then
            call readall(n,p,count,ierr,laenge)
         else
            if (abintype(n) .eq. -1) then
               write(6,'(i5,a22,a)') ndx,': Reading dataset '
               call rddatadb(n,xcoo,qcoo,der1,qder1,der2,&
                       qder2,dcp,qdcp,energy,tdip)
            else if (abintype(n) .eq. -2) then
               ilbl=slen(datafile(n))
               write(6,'(i5,a22,a)') ndx,': Reading data from: ',&
                    datafile(n)(1:ilbl)
               write(ilog,'(i5,a22,a)') &
                    ndx,': Reading data from: ',datafile(n)(1:ilbl)              
               call rddata14(datafile(n),xcoo,qcoo,energy)
            else
               filename = ' '
               ilbl=slen(datafile(n))
               if (datafile(n)(1:1) .ne. '/') then
                  filename(1:laenge+ilbl+1) = &
                        datadir(1:laenge)//'/'//datafile(n)(1:ilbl)
               else
                  filename=datafile(n)
               endif
               write(6,'(i5,a22,a)') ndx,': Reading data from: ',&
                    datafile(n)(1:ilbl)
               write(ilog,'(i5,a22,a)') &
                    ndx,': Reading data from: ',datafile(n)(1:ilbl)

               if (ierr .ne. 0) then
                  routine = 'Vctrans'
                  write(message,'(a)') 'Data File not found: '
                  write(message1,'(a)') filename
                  call vcwarnmsg
               else
                  call rddata(n,filename,xcoo,qcoo,der1,qder1,der2,&
                       qder2,gd,qgd,dcp,qdcp,energy,tdip,&
                       ifiletyp)
                  close(idat)
               endif
            endif
            
            if (ierr .eq. 0) then
! If phase switch if flagged, change the sign of phase-dependent arrays
               if (negphase(n).eq.1) then
                  do i=1,ncoo
                     dcp(i,:)=-dcp(i,:)
                     qdcp(i,:)=-qdcp(i,:)
                  enddo
               endif
               
               if (lpft.eqv..true.) then
                  count=count+1
                  filter=0
                  call dattrans(filter,count)
                  if (filter.eq.1) call wrdata(count,count,xcoo,qcoo,der1,&
                       qder1,der2,qder2,gd,qgd,dcp,qdcp,energy,tdip)
               else
                  call wrdata(ndx,n,xcoo,qcoo,der1,qder1,der2,qder2,&
                       gd,qgd,dcp,qdcp,energy,tdip)
                  if (lreflect) call reflection(ndx,n,xcoo,qcoo,der1,&
                       qder1,der2,qder2,gd,qgd,dcp,qdcp,energy,tdip)                  
               endif
            endif
            
         endif

! collect data on maximum LVC parameters from a DB point
         if (ldb0) call collectdblvc(n,qder1,qdcp,qder2,energy)
         
      enddo
      
      write(iout,'(a)') '#end_dataset_section'

! analyse maximum LVC parameters from a DB point
      if (ldb0) call analdblvc

! Deallocate memory used for DB and close DB
      if (ldd) then
         call dealloc_dirdyn
         call dealloc_dddb

         call dbclose(the_db)
      endif

      stop

      contains

!#######################################################################
!
! vctdefault
!
! Set default values
! zero parameters
!
!#######################################################################

      subroutine vctdefault

      implicit none

      integer(long)       :: i,j

      order = 2
      e01 = 0.0
! flag to read S0 energy from file0
      lrde0=.true.
! flag to read frequencies from file0
      lrdfreq0=.true.
! flag to write first derivative information in Q coordinates
      lwrderq=.false.
! flag to write second derivative information in Q coordinates
      lwrder2q=.false.
! flag to write gradient difference information
      lwrgd=.false.
! flag to read transition dipoles
      ltdip=.false.
! flag to read velocity gauge transition dipoles from DFT/MRCI output (default is to read length gauge results)
      lvelocity=.false.
! flag to apply energy shifts to energy
      leshift=.false.
! turbomole input
      lturbomole = .false.
! flag to only calculate internal coords (and not normal modes) if
! linternal=.true.
      lpureint=.false.
! flags to set opening and closing files for generating QC inputs from a database
      lcl=.false.
      lop=.false.
      lgo=.false.

      logaussian=.false.
      logamess=.false.
      loqchem=.false.
      loxyz=.false.
      lmolpro=.false.
      ldalton=.false.
      lturbomole=.false.
      lgamessus=.false.
      lmolcas=.false.


      ntitle=0
      do i=1,5
         title(i) = '  '
      enddo

      do i=1,maxset
         asta(i)=-1
         asta1(:,i)=0
         eigen(i) = 0
         eshift(i) = 0
         allen(i)=0
         exsta(i)=0
         ignore(i)=0
         negphase(i)=0
      enddo
      do j=1,maxsta
         do i=1,maxset
            remove(i,j)=0
         enddo
      enddo
      do i=1,maxdim
         nmlab1(i) = '  '
         freq1(i) = 0.0_dop
      enddo

      lpft=.false.
      lfilter=.false.
      lsubq=.false.

! rotation of normal modes
      lrotmodes=.false.

      lreflect=.false.

! polyspherical coord flags
      ljacobi=.false.
      lradau=.false.
      lvalence=.false.
      lpoly=.false.

      return
      end subroutine vctdefault

!#######################################################################
!
! read input file
!
! energies in eV, coordinates in bohr
!
!#######################################################################

      subroutine vctinp

      implicit none

      integer(long)           :: i,j,k,iz,iz0,ic,ierr,ilbl,laenge,s,&
                                 ista,m,nmpsets,nsta1
      integer(long)           :: linein,ntr,abintype1,gndabtype1
      real(dop)               :: quantics_vers
      character(len=c2)       :: currorient
      character(len=c5)       :: string,filename,blank,dbname
      logical :: lwarn0=.false.

!-----------------------------------------------------------------------
! set initial values
!
! currorient: default orientation for reading geometry (e.g. in Gaussian)
!-----------------------------------------------------------------------
      iz0     = 0
      iz      = 0
      currorient = 'input'
      file0orient=' '
      do i=1,maxset
         dataorient(i)=' '
      enddo
      datadir = './'
      laenge=2
      call abspath(datadir,laenge)
      nsets=0
      nmpsets=0
      nsetm=0
      nsta=0
      linein=0
      blank= ' '
      nsta1=0

      abintype0=-999
      abintype1=-999
      do n=1,maxset
         abintype(n)=-999
      enddo
      gndabtype1=-99
      eigen(:) = -999

      ilbl=index(filein,'.')-1
      fileout=filein(1:ilbl)//'.info'

!-----------------------------------------------------------------------
! Open files
!-----------------------------------------------------------------------
      filename=filein
      open(iin,file=filename,form='formatted',status='old',iostat=ierr)
      if (ierr .ne. 0) then
         write(6,'(a)') 'ERROR: Input file not found'
         write(6,'(a)') filein(1:80)
         write(6,*)
         stop
      endif
      open(iout,file=fileout,form='formatted',status='unknown')

!-----------------------------------------------------------------------
! read input file
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'vctinp (read VCTRANS input)'
      call vccntrl(ierr)

      i=1
   15 continue
      if (keyword(i) .eq. 'end-input') then
         goto 20

      else if (keyword(i) .eq. 'turbomole') then
         lturbomole = .true.

      else if (keyword(i) .eq. 'orient') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            currorient=keyword(i)(1:c2)
         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with orient keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'file0') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            file0=keyorig(i)
         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with file0 keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'file0orient') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            file0orient=keyword(i)
         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with file0orient keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'datadir') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            datadir = keyorig(i)
            laenge = lc(i)
            if (laenge .eq. 0) then
               routine = 'vctinp (read VCTRANS input)'
               message = ' No path name specified'
               call vcwrline(0,iz,iz0)
            end if
            call abspath(datadir,laenge)
         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with datadir keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'files') then
         if (i .ne. 1) then
            routine = 'vctinp (read VCTRANS input)'
            message='Files keyword must be alone on line'
            call vcwarnmsg
         else
            call rdsets(iin,currorient,iz,iz0,filein)
         endif

      else if (keyword(i) .eq. 'datasets') then
         if (i .ne. 1) then
            routine = 'vctinp (read VCTRANS input)'
            message='Dataset keyword must be alone on line'
            call vcwarnmsg
         else
  120       continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            routine = 'vctinp (read VCTRANS input)'
            call vccntrl(ierr)

            if (keyword(1) .ne. 'end-datasets') then
               filename=keyorig(1)
               open(idat,file=filename,form='formatted',status='old',&
                  iostat=ierr)
               if (ierr .ne. 0) then
                  routine = 'vctinp (read VCTRANS input)'
                  write(message,'(a)') 'Data File not found: '
                  write(message1,'(a)') filename
                  call vcwarnmsg
               else
                  write(ilog,'(2a)') 'Opening Dataset:',filename(1:60)
                  iz  = 0  
                  iz0 = 0
                  call rdsets(idat,currorient,iz,iz0,filename)
                  close(idat)
               endif
               go to 120
            else
               write(ilog,*) 
            endif
         endif

      else if (keyword(i) .eq. 'thr1r2-points') then
         if (i .ne. 1) then
            routine = 'vctinp (read VCTRANS input)'
            message='Thr1r2-points keyword must be alone on line'
            call vcwarnmsg
         else
            call rdthr1r2(iin,iz,iz0,filein)
         endif

      else if (keyword(i) .eq. 'thr1r2-files') then
         if (i .ne. 1) then
            routine = 'vctinp (read VCTRANS input)'
            message='Thr1r2-files keyword must be alone on line'
            call vcwarnmsg
         else
  130       continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            routine = 'vctinp (read VCTRANS input)'
            call vccntrl(ierr)

            if (keyword(1) .ne. 'end-files') then
               filename=keyorig(1)
               read(keyword(2),*) ista
               open(idat,file=filename,form='formatted',status='old',&
                  iostat=ierr)
               if (ierr .ne. 0) then
                  routine = 'vctinp (read VCTRANS input)'
                  write(message,'(a)') 'Data File not found: '
                  write(message1,'(a)') filename
                  call vcwarnmsg
               else
                  write(ilog,'(2a)') 'Opening Dataset:',filename(1:60)
                  call rdthr1r2_set(idat,filename,ista)
                  close(idat)
               endif
               go to 130
            else
               write(ilog,*)
            endif
         endif

      else if (keyword(i) .eq. 'energy0') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) e01
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keyeunits(e01,keyword(i),ierr)
               if (ierr .ne. 0) then
                  routine = 'vctinp (read VCTRANS input)'
                  message = 'Incorrect unit for energy : '
                  call vcwrline(1,iz,iz0)
               endif
            endif

            lrde0=.false.

         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with energy0 keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'nmodes') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) nmodes
            if (keyword(i+1) .eq. ',') then
               i=i+2
               read(keyword(i),*) ntr
            else
               ntr = 6
            endif
            ncoo=nmodes+ntr
            natm = ncoo/3
         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with nmodes keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'order') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) order
         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with order keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'units_au') then
         iau = 0

      else if (keyword(i) .eq. 'soc_full') then
         lsocdegen = .false.

      else if (keyword(i).eq.'abinitiotype') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            call getabintype(keyword(i),abintype1,ilog)

         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with abinitiotype keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'file0abtype') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            call getabintype(keyword(i),abintype0,ilog)

         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with abinitiotype keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'gndabtype') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            call getabintype(keyword(i),gndabtype1,ilog)

         else
            message = 'No argument specified with gndabtype keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if(keyword(i).eq.'db0') then
         ldb0=.true.

      else if (keyword(i).eq.'transition_dipoles') then
         ltdip=.true.

      else if (keyword(i).eq.'velocity_gauge') then
         lvelocity=.true.

      else if (keyword(i).eq.'derivatives_in_q') then
         lwrderq=.true.

      else if (keyword(i).eq.'gradient_difference') then
         lwrgd=.true.

      else if (keyword(i).eq.'second_derivatives_in_q') then
         lwrder2q=.true.

      else if (keyword(i).eq.'nmode_trafos') then
         lwrtrafo=.true.

!-------------------------------------------------------------------------
! Keyword to allow transformation from db geometries to QC input files
!-------------------------------------------------------------------------

      else if (keyword(i).eq.'generate_input') then
         db2inp=.true.

      else if (keyword(i) .eq. 'fileout') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            fileout=keyorig(i)
         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with fileout keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'opening') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            fileop=keyorig(i)
            lop = .true.
         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with opening keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'closing') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            filecl=keyorig(i)
            lcl = .true.
         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with closing keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'formout') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            if (keyword(i) .eq. 'gaussian') then
               logaussian = .true.
            else if (keyword(i) .eq. 'gamess') then
               logamess = .true.
            else if (keyword(i) .eq. 'qchem') then
               loqchem = .true.
            else if (keyword(i) .eq. 'xyz') then
               loxyz = .true.
            else if (keyword(i) .eq. 'molpro') then
               lmolpro = .true.
            else if (keyword(i) .eq. 'molcas') then
               lmolcas = .true.
            else if (keyword(i) .eq. 'dalton') then
               ldalton = .true.
            else if (keyword(i) .eq. 'turbomole') then
               lturbomole = .true.
            else if (keyword(i) .eq. 'gamess_us') then
               lgamessus = .true.
            else if (keyword(i) .eq. 'molcas') then
               lmolcas = .true.
            else
               routine = 'vcpinp (read VCPNT input)'
               write(message,'(2a)') 'Unknown formout : ',keyword(i)
               call vcwrline(0,iz,iz0)
            endif
            
         else
            routine = 'vcpinp (read VCPNT input)'
            message = 'No argument specified with formout keyword'
            call vcwrline(0,iz,iz0)
         endif

!--------------------------------------------------------------------
! Coordinate transformation keywords
!--------------------------------------------------------------------
      else if (keyword(i).eq.'system_atom') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) sysatom
         endif
      else if (keyword(i).eq.'system_define') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) ratm1
            read(keyword(i+1),*) ratm2
            read(keyword(i+2),*) ratm3
            read(keyword(i+3),*) ratm4
            i=i+3
         endif
      else if (keyword(i).eq.'subspace_points') then
         lsubq=.true.

      else if(keyword(i).eq.'filter') then
         lfilter=.true.

      else if (keyword(i).eq.'parent_to_fragment_trans') then
         lpft=.true.

      else if (keyword(i) .eq. 'parent_file') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            parentfile=keyword(i)
         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with parent_file keyword'
            call vcwrline(0,iz,iz0)
         endif
      else if (keyword(i) .eq. 'fragment_file') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               fragmentfile=keyword(i)
            else
               routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with fragment_file keyword'
               call vcwrline(0,iz,iz0)
            endif
         else if (keyword(i) .eq. 'fragment_nmodes') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               read(keyword(i),*) fragnmodes
               if (keyword(i+1) .eq. ',') then
                  i=i+2
                  read(keyword(i),*) ntr
               else
                  ntr = 6
               endif
               fragncoo=fragnmodes+ntr
               fragnatom = fragncoo/3
            else
               routine = 'vctinp (read VCTRANS input)'
          message = 'No argument specified with fragment_nmodes keyword'
               call vcwrline(0,iz,iz0)
            endif

         else if (keyword(i) .eq. 'define_system_modes') then
  140       continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            routine = 'vctinp (read VCTRANS input)'
            call vccntrl(ierr)
            
            if (keyword(1) .ne. 'end-define_system_modes') then
               read(keyword(1),*) m
               read(keyword(2),*) sysfreq(m)
               read(keyword(3),*) syslab(m)
               goto 140
            endif

!--------------------------------------------------------------------

      else if (keyword(i).eq.'states') then
         if (keyword(i+1).eq.'=') then
            s=0
   60       continue
            s=s+1
            i=i+2
            read(keyword(i),*,iostat=ierr) states(s)
            if (ierr .ne. 0) then
               routine = 'vctinp (read VCTRANS input)'
               message ='states must be an integer. Not :'
               call vcwrline(i,iz,iz0)
            endif

            if (keyword(i+1).eq.',') go to 60
            nsta = s

         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with states keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'nstates') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) nsta1
         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with nstates keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'frequencies ') then
         if (i .ne. 1) then
            routine = 'vctinp (read VCTRANS input)'
            message='Frequencies keyword must be alone on line'
            call vcwrline(i,iz,iz0)
         else
            lrdfreq0=.false.
            n=0
 40         continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                 maxkey)
            routine = 'vctinp (read VCTRANS input)'
            call vccntrl(ierr)

            if (keyword(1) .ne. 'end-frequencies') then
               if (ic .ge. 3) then
                  read(keyword(1),*) n
                  nmlab1(n)=keyorig(2)(1:c1)
                  read(keyword(3),*) freq1(n)
                  if (keyword(4) .eq. ',') then
                     call keyeunits(freq1(n),keyword(5),ierr)
                     if (ierr .ne. 0) then
                        routine = 'vctinp (read VCTRANS input)'
                        message = 'Incorrect unit for frequency : '
                        call vcwrline(1,iz,iz0)
                     endif
                  endif

               else
                  routine = 'vctinp (read VCTRANS input)'
                  message='Error reading frequencies'
                  call vcwrline(i,iz,iz0)
               endif
               go to 40
            endif
         endif

      else if (keyword(i) .eq. 'eckart') then
         eck_frame = .true.

      else if (keyword(i) .eq. 'title' ) then
         if (i .ne. 1) then
            routine = 'vctinp (read VCTRANS input)'
            message='Title keyword must be alone on line'
            call vcwrline(i,iz,iz0)
         else
            i=0
 50         continue
            read(iin,'(a)') string
            if (string .ne. 'end-title') then
               i=i+1
               if (i .le. 5) title(i)=string(1:c4)
               go to 50
            else
               ntitle=max(i,5)
            endif
         endif

      else if (keyword(i) .eq. 'replace-modes') then
         if (i .ne. 1) then
            routine = 'vctinp (read VCTRANS input)'
            message='replace-modes keyword must be alone on line'
            call vcwarnmsg
         else
            call rdreplace(iin,iz,iz0)
         endif

      else if (keyword(i).eq.'nm2curv') then
         j = 1
         lnmcurv = .true.
         do while (nmcurvtype(j) .ne. 0)
            j = j + 1
         enddo

         if (keyword(i+1).eq.'spherical_type1' .or. keyword(i+1).eq. '1') then
! spherical coords used by Chris Robertson for acetylene
            nmcurvtype(j) = 1
            if ( keyword(i+2).eq. '=' ) then
               i = i + 3
               k = 0
               do while (trim(keyword(i)) .ne. ',' .and. ic .gt. i)
!            print*, keyword(i), 'j = ', j, ' k = ', k
                  k = k + 1
                  read(keyword(i),*) nmcurv(j,k)
!               print*, 'j = ', j, 'k = ', k, ' nmcurv(j,k) = ', nmcurv(j,k)
                  i = i + 1
               enddo

!            print*, keyword(i), i+3, ic, keyword(i)
!           note to user: if you want to put the x0 array, you have to give it to all DOF's in the transformation :
               if ( keyword(i) .eq. ',' .and. i+k .ne. ic) then
                  routine='Vctinp'
                  write(message,'(2a,i5)') &
                         'please give the same number of reference ', &
                                'curvilinear coordinates as DOF in transformation'
                  call vcerrormsg
               elseif( keyword(i) .eq. ',' .and. i+k .eq. ic) then
                  i = i + 1
                  do n =1, k
                     read(keyword(i),*) c0(j,n)
                     if(nmcurvtype(j) .eq. 1 .and. n .gt. 1 ) c0(j,n) =c0(j,n)*(pi/180)
!                  print*, 'C0 ====', c0(j,n), 'j = ', j, 'n = ', n
                     i = i + 1
                  enddo
               else
!              call getc0(c0,nmcurv,j) 
               endif
            else
               routine='Vctinp'
               message='Error reading normal mode conversion input'
               call vcerrormsg
            endif
         else if (keyword(i+1).eq.'dihed' .or. keyword(i+1).eq. '2') then
! replace nmode by a dihedral
            nmcurvtype(j) = 2
            if ( keyword(i+2).eq. '=' ) then
               i=i+3
               read(keyword(i),*) nmcurv(j,1)
            endif

         else
            routine = 'vctinp (read VCPNT input)'
            write(message,'(a,i5)') ' curvilinear transformation type not implemented'
            call vcwrline(i,iz,iz0)
         endif

! define a dihedral
      else if (keyword(i)(1:13).eq.'dihed-section') then
         if (keyword(i)(14:14) .eq. '_') then
            read(keyword(j)(15:15),'(i1)') j
         else
            j=1
         endif

         if (i .ne. 1) then
            routine = 'vctinp (read VCTRANS input)'
            message='dihed keyword must be alone on line'
            call vcwrline(i,iz,iz0)
         elseif (nmcurvtype(j) .ne. 2) then
            routine = 'vctinp (read VCTRANS input)'
            message='Dihed-section specified without "nm2curv dihed" keyword '
            call vcwrline(i,iz,iz0)
         else
            n=0
 45         continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                 maxkey)
            routine = 'vctinp (read VCPNT input)'
            call vccntrl(ierr)

            if (keyword(1) .ne. 'end-dihed-section') then
               n=n+1
               if (ic .eq. 2) then
                  read(keyword(2),*) dihedflag(n)
               else
                  routine = 'vctinp (read VCPNT input)'
                  message='Error reading dihedral flags'
                  call vcwrline(i,iz,iz0)
               endif
               go to 45
            endif
         endif

      else if (keyword(i).eq.'molstates') then
         routine = 'vctinp (read VCTRANS input)'
         message = 'Molstates keyword deprecated. Do not use symmetry for VCHam'
         call vcwrline(0,iz,iz0)

      else if (keyword(i).eq.'rotate_modes') then
         if (keyword(i+1).eq.'=') then
            lrotmodes=.true.
            j = 1
            do while ( rotm(j,1) .ne. 0 )
              j = j + 1 
            enddo
        i = i + 2
!         print*, trim(keyword(i)), trim(keyword(i+1)), trim(keyword(i+2))
144      if ( keyword(i+3) .eq. ',' .or. ( ((i+2)) .eq. ic)) then
!              modes to be rotatated
               read(keyword(i),*) rotm(j,1)
               read(keyword(i+1),*) rotm(j,2)
!              mixing angle
               read(keyword(i+2),*) rotang(j)
               j = j + 1
               i = i + 4
               goto 144
            endif
         else
            routine = 'vctinp (read VCPNT input)'
            message='Error reading normal mode rotation input'
            call vcwrline(i,iz,iz0)
         endif
      else if (keyword(i).eq.'jacobi_define') then
         lpoly=.true.
         ljacobi=.true.
         n=0
         do j=1,maxdim
            patm(j)=0
         enddo
70       continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         call vccntrl(ierr)
         if (keyword(i).ne.'end-jacobi_define') then
            n=n+1
            read (keyword(1),*) patm(n)
            goto 70
         endif

      else if (keyword(i).eq.'radau_define') then
         lpoly=.true.
         lradau=.true.
         n=0
         do j=1,maxdim
            patm(j)=0
         enddo
72       continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         call vccntrl(ierr)
         if (keyword(i).ne.'end-radau_define') then
            n=n+1
            read (keyword(1),*) patm(n)
            goto 72
         endif

      else if (keyword(i).eq.'valence_define') then
         lpoly=.true.
         lvalence=.true.
         n=0
         do j=1,maxdim
            patm(j)=0
            vcon(j)=0
         enddo
73       continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         call vccntrl(ierr)
         if (keyword(i).ne.'end-valence_define') then
            n=n+1
            read (keyword(1),*) patm(n+1)
            read (keyword(2),*) vcon(n)
            if (n.eq.1) read (keyword(2),*) patm(1)
            goto 73
         endif

      else if (keyword(i).eq.'reflect') then
         lreflect=.true.
         if (keyword(i+1).eq.'=') then
75          continue
            i=i+2
            read(keyword(i),*,iostat=ierr) n
            if (ierr .ne. 0) then
               routine = 'vctinp (read VCTRANS input)'
               message ='reflect modes must be an integer. Not :'
               call vcwrline(i,iz,iz0)
            endif
            reflect(n)=1
            if (keyword(i+1).eq.',') go to 75            
         else
            routine = 'vctinp (read VCTRANS input)'
            message = 'No argument specified with reflect keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'all_eigenvectors') then
         !lalleig=.true.
         routine = 'vctinp (read VCTRANS input)'
         message = ' Keyword ignored: all_eigenvectors not working '
         lwarn0=lwarn   ! Just give waring, but do not stop
         call vcwrline(0,iz,iz0)
         lwarn=lwarn0

      else if (keyword(i).eq.'pure_internal') then
         lpureint=.true.
      
      else if (keyword(i).eq.'molpro_reorder') then
         !molpro is reordering states between CAS and CAS-PT2
          lmolre = .true.  !added MAP
      else
         routine = 'vctinp (read VCTRANS input)'
         message = ' unknown keyword : '
         call vcwrline(i,iz,iz0)

      endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

 20   continue
      close(iin)

!-----------------------------------------------------------------------
! If states keyword not used, include all states up to nstates
!-----------------------------------------------------------------------
      if (nsta .eq. 0) then
         if (nsta1 .eq. 0) then
            routine='Vctinp'
            write(message,'(a)') 'Set the no. of states to be read'
            call vcerrormsg
         endif
       
         nsta = nsta1
         do s=1,nsta
            states(s)=s
         enddo
      endif

!-----------------------------------------------------------------------
! now set flag for eigenstate to be read for each set
!-----------------------------------------------------------------------
      do n=1,nsets
         s = asta(n)
         if (eigen(n) .le. 0) eigen(n) = states(s)
      enddo

!-----------------------------------------------------------------------
! make checks
!-----------------------------------------------------------------------
      if (nsetm .gt. maxset) then
         routine='Vctinp'
         write(message,'(a,i5)') 'Redimension MAXSET. Minimum: ',nsetm
         call vcerrormsg
      endif

!-----------------------------------------------------------------------
! Reading a DB?
! Set various flags.....
!-----------------------------------------------------------------------
      if (abintype1 .eq. -1) then
         ldd = .true.
         if (abintype0 .eq. -999) then
            routine='Vctinp'
            write(message,'(a,i5)') &
               'File0abtype needs to be set to read DB file0 '
            call vcerrormsg
         endif

         ilbl=index(file0,' ')-1
         file0 = datadir(1:laenge)//'/'//file0(1:ilbl)

! variables needed by DB routines
         ddname = datadir
         ddlaenge = laenge
         ndofddpes = ncoo
         ndofdd = nmodes

! open DB
         dbname=ddname(1:ddlaenge)//'/database.sql'
         !Opened read only so version and arrays are dummies
         call dbopen(dbname,the_db, sqlvers, quantics_vers, infoarray, &
              flagarray, 0)

         if (ldb0) then
            dbnrec = 1
         else
            call getdbnrec(dbnrec)
         endif

! set flags for each data set - each DB record provides nsta datasets         
         nsets=0
         do i=1,dbnrec
            do j=1,nsta
               nsets=nsets+1
               asta(nsets) = j
               eigen(nsets) = states(j)
               gndabtype(nsets) = -999
            enddo
         enddo
         maxset = nsets
      endif

!-----------------------------------------------------------------------
! set default ab initio type of files
!-----------------------------------------------------------------------
      if (abintype0 .eq. -999) abintype0=abintype1
      do n=1,nsets
         if (abintype(n) .eq. -999) abintype(n)=abintype1
      enddo

!-----------------------------------------------------------------------
! check if ab initio type has been set for declared ground-state files
! -- required by OVGF and ADC calculations
!-----------------------------------------------------------------------
      do n=1,nsets
         if (gndabtype(n) .eq. -99) gndabtype(n) = gndabtype1
      enddo
!      do n=1,nsets
!         if (abintype(n) .eq. 9 .or. abintype(n) .eq. 10) then
!            if (gndabtype(n) .lt. 0) then
!               routine='Vctinp'
!               write(message,'(a)') &
!                  'Ab initio type not set for ground-state files'
!               call vcerrormsg
!            endif
!         endif
!      enddo

!-----------------------------------------------------------------------
! Change e0 to correct units
!-----------------------------------------------------------------------
       if (iau .eq. 1) e01 = e01*uev

!-----------------------------------------------------------------------
! check various flags set
!-----------------------------------------------------------------------
      if( nsets .le. 0 )then
         write(6, '(a)')'*** no data sets specified! ***'
         stop
      end if
      if(nsta.eq.0)then 
         write(6,'(a)') 'You must specify the number of states!'
         stop
      endif
      if(nmodes.eq.0)then 
         write(6,'(a)') 'You must specify the number of modes!'
         stop
      endif
      if(abintype0.eq.0)then 
         write(6,'(a)') 'You must specify the type of ab initio!'
         stop
      endif
      if (nsets .lt. nsta) then
         write(6,'(a,i5)') &
           'Not enough files. Need at least as many as Nsta:',nsta
         stop
      endif

      if (file0orient .eq. ' ') then
         file0orient=currorient
      endif

!-----------------------------------------------------------------------
! log settings
!-----------------------------------------------------------------------
      write(6,*)
      write(6,'(a26,a54)') 'Output file             : ',fileout
      write(ilog,*)
      write(ilog,'(a26,a54)') 'Output file             : ',fileout

      if (ldd) then
         write(6,'(2a)')  'Reading DB  : ',datadir(1:laenge)
         write(ilog,'(2a)')  'Reading DB  : ',datadir(1:laenge)
      endif
      write(6,'(a26,i10)')  'Number of dataset       : ',maxset
      write(ilog,'(a26,i10)')  'Number of dataset       : ',maxset

      if (linternal) then
         write(ilog,'(/,a)') '**** Internal coordinates used ****'
         do m=1,nmodes
            if (cootyp(m) .eq. 1) then
               write(ilog,'(a,i5,a,4i5)') 'Mode ',m,&
                 ' replaced by torsion defined by atoms ',&
                 (coodef(i,m),i=1,4)

            else if (cootyp(m) .eq. 2) then
               write(ilog,'(a,i5,a,3i5)') 'Mode ',m,&
                 ' replaced by angle defined by atoms   ',&
                 (coodef(i,m),i=1,3)

            else if (cootyp(m) .eq. 3) then
               write(ilog,'(a,i5,a,2i5)') 'Mode ',m,&
                 ' replaced by bond defined by atoms    ',&
                 (coodef(i,m),i=1,2)
            endif
         enddo
         write(ilog,*)
      endif

      return
      end subroutine vctinp

!#######################################################################

      subroutine rdsetsinit(unit,filename)

implicit none

      integer(long)                 :: ic,ierr,siz,nsetm,i
      integer(long), intent(in)     :: unit
      real(dop)                     :: quantics_vers
      character(len=80), intent(in) :: filename
      character(len=c5)             :: dbname
      character(len=c5)             :: inptit
      logical                       :: fset, dset

      fset=.false.
      dset = .false.
      siz=0
      nsetm=0
      open(unit,file=filename,form='formatted',status='old',iostat=ierr)

   200 continue
      call rdinpf(unit,keyword,keyorig,inptit,lc,ic,siz,ierr,maxkey)
      routine = 'rdsetsinit  '//filename(1:60)
      if (ierr .ne. 0) then
         go to 205
      endif
      !call vccntrl(ierr)
      
      do i=1,ic
         if (keyword(i) .eq. 'files') then
            fset=.true.

         else if (keyword(i) .ne. 'end-files' .and. fset ) then
            if (i .eq. 1) nsetm=nsetm+1    ! one data set per line

         else if (keyword(i) .eq. 'datasets') then
            dset = .true.

         else if (keyword(i) .ne. 'end-files' .and. dset ) then
            !Load dataset file and count through
            open(itmpin,file=inptit,form='formatted',status='old',iostat=ierr)
            if (ierr .eq. 0) then
               DO
                  READ (itmpin,*, END=10)   
                  if (i .eq. 1) nsetm = nsetm + 1   ! on data set per line
               END DO
               10 CLOSE (itmpin)
               maxset=nsetm
            endif 
      
         else if (keyword(i) .eq. 'end-files') then
            fset = .false.

         else if (keyword(i) .eq. 'abinitiotype' .and. keyword(i+2) .eq. 'db') then
            ldd = .true.

         else if (keyword(i) .eq. 'db0') then
            ldb0 = .true.

         else if (keyword(i) .eq. 'datadir') then
! variables needed to get dbnrec
            ddname = keyorig(i+2)
            ddlaenge = slen(ddname)

         end if
      enddo

      go to 200

   205 continue
      close(unit)

! get no. sets to be read from a DB
! also need Nbasis if the DB includes a mocodb file
      if (ldd) then
         dbname=ddname(1:ddlaenge)//'/database.sql'
         call dbopen(dbname,the_db, sqlvers, quantics_vers, infoarray, &
              flagarray, 0)

         call getdbnrec(dbnrec)
         nsetm = dbnrec*maxsta

         call dbrdrefdb(the_db, natm, nroot, nddstate, nbasis, dde0, dimref, &
              nfam, dercpdim,dbpntgp,nsymop,nsocstate,nddmult)

         maxmoco = nbasis
         call dbclose(the_db)

      endif

! set maximum number of sets to be read
      maxset=nsetm

      return
      end subroutine rdsetsinit

!#######################################################################

      subroutine rdsets(unit,currorient,iz,iz0,filename)

      implicit none

      integer(long)                 :: i,ic,ierr,siz,siz0,s
      integer(long), intent(in)     :: unit,iz
      integer(long), intent(inout)  :: iz0
      character(len=c2), intent(in) :: currorient
      character(len=c5), intent(in) :: filename

      siz=iz
      siz0=iz0

  100 continue
      call rdinpf(unit,keyword,keyorig,inptit,lc,ic,siz,ierr,maxkey)
      routine = 'rdsets  '//filename(1:60)
      call vccntrl(ierr)

      if (keyword(1) .ne. 'end-files') then
         nsets=nsets+1
         if (nsets .gt. maxset) then
            message = 'Nsets to be read > Maxset !'
            call vcerrormsg
         endif

         nsetm=nsetm+1
         datafile(nsets) = keyorig(1)

         if (ic .ge. 2) then
            read(keyword(2),*,iostat=ierr) asta(nsets)

            if (asta(nsets).eq.0) allen(nsets)=1

            if (ierr .ne. 0) then
               write(message,'(2a)') 'Cannot read asta for datafile :',&
                     keyorig(1)(1:lc(1))
               call vcwrline(0,iz,iz0)
            endif
         else 
            message='Error in input of data file names'
            call vcwrline(0,siz,siz0)
         endif

!-----------------------------------------------------------------------
! set defaults for properties of data files 
!-----------------------------------------------------------------------
         dataorient(nsets) = currorient
         s = asta(nsets)
         if (s .gt. maxsta) then
            routine = 'Rdsets'
            write(message,'(a)') &
       'Trying to read more states than allowed. Set nstates correctly. '
            call vcerrormsg
         endif
         if (lmolre .eqv. .true.) then
            molre_states(nsets) = states(s)
         endif
         gndfile(nsets) = ' '
         gndorient(nsets) = currorient
         gndabtype(nsets) = -999

!-----------------------------------------------------------------------
! read any further keywords related to file
!-----------------------------------------------------------------------
         i=2
  105    continue
         i=i+1
         if (keyword(i) .eq. 'orient') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               dataorient(nsets) = keyword(i)(1:32)
            else
               message = 'No argument specified with orient keyword'
               call vcwrline(0,siz,siz0)
            endif

         else if (keyword(i) .eq. 'root') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               read(keyword(i),*) eigen(nsets) 
            else
               message = 'No argument specified with root keyword'
               call vcwrline(0,siz,siz0)
            endif

         else if (keyword(i) .eq. 'extra') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               read(keyword(i),*) exsta(nsets)
            else
               message = 'No argument specified with extra keyword'
               call vcwrline(0,siz,siz0)
            endif

         else if (keyword(i) .eq. 'ignore') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               read(keyword(i),*) ignore(nsets)
            else
               message = 'No argument specified with ignore keyword'
               call vcwrline(0,siz,siz0)
            endif
            
         else if (keyword(i) .eq. 'remove') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               read(keyword(i),*) s
               remove(nsets,s)=1
            else
               message = 'No argument specified with remove keyword'
               call vcwrline(0,siz,siz0)
            endif

         else if (keyword(i) .eq. 'negphase') then
            negphase(nsets)=1
            
         else if (keyword(i) .eq. 'orbital') then
            message = 'Orbital keyword no longer used.'
            message(81:160) = 'Use state label with HOMO = 1, HOMO-1 = 2, ...'
            call vcerrormsg

         else if (keyword(i) .eq. 'abintype') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               call getabintype(keyword(i),abintype(nsets),ilog)
            else
               message = 'No argument specified with abintype keyword'
               call vcwrline(0,siz,siz0)
            endif

         else if (keyword(i) .eq. 'gndfile') then
! change gndabtype if not set so that presence of gndfile is flagged
            if (gndabtype(nsets) .eq. -999) gndabtype(nsets)=-99
            if (keyword(i+1).eq.'=') then
               i=i+2
               gndfile(nsets) = keyorig(i)
            else
               message = 'No argument specified with gndfile keyword'
               call vcwrline(0,siz,siz0)
            endif

         else if (keyword(i) .eq. 'gndabtype') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               call getabintype(keyword(i),gndabtype(nsets),ilog)
            else
               message = 'No argument specified with gndabtype keyword'
               call vcwrline(0,siz,siz0)
            endif

         else if (keyword(i) .eq. 'gndorient') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               gndorient(nsets) = keyword(i)(1:32)
            else
               message = 'No argument specified with gndorient keyword'
               call vcwrline(0,siz,siz0)
            endif

         else if (keyword(i) .eq. 'eshift') then
            if (keyword(i+1).eq.'=') then
               i=i+2
               read(keyword(i),*) eshift(nsets)
               leshift=.true.
            else
               message = 'No argument specified with eshift keyword'
               call vcwrline(0,siz,siz0)
            endif

         endif
         if (i .lt. ic) then
            go to 105
         else
            go to 100
         endif
      !else
         !Need to set maxset properly
         !maxset = maxset + nsetm + 1
      endif

      return
      end subroutine rdsets

!#######################################################################
!
! reads data in theta, r1, r2, E form
!
! data is stored in datafile name and unpacked in main routine
!#######################################################################

      subroutine rdthr1r2(unit,iz,iz0,filename)

      implicit none

      integer(long)                 :: ic,ierr,siz,siz0,i1,i2,i3,i4
      integer(long), intent(in)     :: unit,iz
      integer(long), intent(inout)  :: iz0
      character(len=80), intent(in) :: filename

      siz=iz
      siz0=iz0

  100 continue
      call rdinpf(unit,keyword,keyorig,inptit,lc,ic,siz,ierr,maxkey)
      routine = 'rdsets  '//filename(1:60)
      call vccntrl(ierr)

      if (keyword(1) .ne. 'end-points') then
         if (nsets .lt. maxset) nsets=nsets+1
         nsetm=nsetm+1
         i1 = index(keyorig(1),' ')+2
         i2 = index(keyorig(2),' ')+2
         i3 = index(keyorig(3),' ')+2
         i4 = index(keyorig(4),' ')+2
         datafile(nsets) = keyorig(1)(1:i1)//keyorig(2)(1:i2)//&
               keyorig(3)(1:i3)//keyorig(4)(1:i4)
         if (ic .eq. 5) then
            read(keyword(5),*,iostat=ierr) asta(nsets)
            eigen(nsets) = asta(nsets)
            if (ierr .ne. 0) then
               write(message,'(2a)') 'Cannot read asta for datafile :',&
                     keyorig(1)(1:lc(1))
               call vcwrline(0,iz,iz0)
            endif
         else 
            message='Error in input of points'
            call vcwrline(0,siz,siz0)
         endif

         go to 100
      endif

      return
      end subroutine rdthr1r2

!#######################################################################
!
! reads data in theta, r1, r2, E form
!
! data is stored in datafile name and unpacked in main routine
!#######################################################################

      subroutine rdthr1r2_set(unit,filename,ista)

      implicit none

      integer(long)                 :: i,n
      integer(long), intent(in)     :: unit,ista
      character(len=80), intent(in) :: filename

      read(unit,*) n
      do i=1,n
         if (nsets .lt. maxset) nsets=nsets+1
         nsetm=nsetm+1

         read(unit,'(a)',iostat = ierr) datafile(nsets) 
         if (ierr .ne. 0) then
            write(message,'(2a)') 'Error reading datafile :',&
                     filename
            call vcwarnmsg
         endif

! ignore line if commented out
         call rmlblnk(datafile(nsets))
         if (datafile(nsets)(1:1) .eq. '#') then
            nsets = nsets-1
            nsetm = nsetm-1
         else
            asta(nsets) = ista
            eigen(nsets) = asta(nsets)
         endif
      enddo

      return
      end subroutine rdthr1r2_set

!#######################################################################

      subroutine rdreplace(unit,iz,iz0)

      implicit none

      integer(long)             :: ic,ierr,siz,siz0,nm
      integer(long), intent(in) :: unit,iz,iz0

      siz=iz
      siz0=iz0

      linternal=.true.

  100 continue
      call rdinpf(unit,keyword,keyorig,inptit,lc,ic,siz,ierr,maxkey)
      routine = 'rdreplace'
      call vccntrl(ierr)

      if (keyword(1) .ne. 'end-replace') then
        
         read(keyword(1),*,err=150) nm

         if (ic .eq. 3) then
            cootyp(nm) = 3
            read(keyword(2),*,err=150) coodef(1,nm)
            read(keyword(3),*,err=150) coodef(2,nm)
         else if (ic .eq. 4) then
            cootyp(nm) = 2
            read(keyword(2),*,err=150) coodef(1,nm)
            read(keyword(3),*,err=150) coodef(2,nm)
            read(keyword(4),*,err=150) coodef(3,nm)
         else if (ic .eq. 5) then
            cootyp(nm) = 1
            read(keyword(2),*,err=150) coodef(1,nm)
            read(keyword(3),*,err=150) coodef(2,nm)
            read(keyword(4),*,err=150) coodef(3,nm)
            read(keyword(5),*,err=150) coodef(4,nm)
      
         else 
            message='Error in input of data file names'
            call vcwrline(0,siz,siz0)
         endif

         go to 100
      endif

      return
  150 routine='Rdreplace'
      message='Error reading replace-mode data'
      call vcerrormsg
    
      end subroutine rdreplace

!#######################################################################

      subroutine wrfile0(adjpft,adjqrt,fragnmcoo,fragnmlab)

      implicit none

      integer(long)       :: i,j,k,m,j1,j2,nblk
      character(len=16), dimension(maxdim), intent(in) :: fragnmlab
      real(dop), dimension(maxdim,maxdim),intent(in)  :: fragnmcoo
      real(dop), dimension(maxdim,maxdim), intent(in) :: adjpft,adjqrt

!----------------------------------------------------------------------
! write all the information from the S0 file
!----------------------------------------------------------------------
      write(iout,'(a,f8.5)') '# VCHam Version :',progver
      if (iau .eq. 1) then
         write(iout,'(a)') '# All energies and frequencies in eV'
         write(iout,'(a)') '# Coordinates in Angstrom'
      else
         write(iout,'(a)') '# All energies and frequencies in Hartree'
         write(iout,'(a)') '# Coordinates in Bohr'
      endif
      write(iout,'(a)') ' ' 
      if (ntitle .gt. 0) then
         write(iout,'(a)') '# title ' 
         do i=1,ntitle
            write(iout,'(a)') title(i)
         enddo
         write(iout,'(a)') ' ' 
      endif

      write(iout,'(a)') '#system_dimensions'
      write(iout,'(a)') '#internal_coordinates'
      write(iout,'(i3)') nmodes
      write(iout,'(a)')'#trivial_modes'
      write(iout,'(i3)') ncoo-nmodes
      write(iout,'(a)') '#num_states'
      write(iout,'(2i3)') nsta,maxsta
      write(iout,'(a)') '#states'
      if (lmolre .eqv. .TRUE.) then
         !As reordered state ordering  needs to be corrected
         write(iout,'(20i3)') (i,i=1,nsta) 
      else
        write(iout,'(20i3)') (states(i),i=1,nsta) 
      end if
      
      if (lpft.eqv..true.) then
         write(iout,'(a)') '#fragment_modes'
         write(iout,'(i3)') fragnmodes
      endif
      write(iout,'(a)') '#end_system_dimensions'

      write(iout,*)
      write(iout,'(a)') '#general_information from the ground state'
      write(iout,'(a)') '#zero_of_energy'
      write(iout,'(F20.10)') e0
      write(iout,'(a)') '#equilibrium_state_geometry'
      do i=1,ncoo,3
         write(iout,'(3F15.10)') (xcoo0(j),j=i,i+2)
      enddo
      write(iout,'(a)') '#masses'
      do i=1,ncoo,3
         write(iout,'(F18.10)') mass(i)
      enddo
      if (.not. lpft) then
         write(iout,'(a)') '#vibrational_frequencies'
         do m=1,nmodes
            write(iout,'(i5,2x,a,F20.10)') m,nmlab(m),freq0(m)
         enddo
      else
         write(iout,'(a)') '#vibrational_frequencies'
         do m=1,fragnmodes
            write(iout,'(i5,2x,a,F20.10)') m,fragnmlab(m),fragfreq0(m)
         enddo
         do m=fragnmodes+1,nmodes
            write(iout,'(i5,2x,a,F20.10)') m,syslab(m),sysfreq(m)
         enddo
      endif

      if (lrotmodes) then
         write(iout,'(a)') '#mixed_frequency_cross_terms '
         j = 1
         do while (rotm(j,1) .ne. 0)
!           print*, 'rotang(j) = ', rotang(j), 'j = ', j
           print*, ',rotmxfq(j) = ', rotmxfq(j)
           write(iout,'(I3,3X,I3,3X,3F15.10,3X,F10.6)') rotm(j,1),rotm(j,2),rotang(j),rotmxfq(j)
           j = j + 1
         enddo
      endif

      if (lalleig .and. .not. lnmcurv) write(iout,'(a)') '#all_hessian_eigenvectors'

      write(iout,'(a)') '#normal_modes'
! write the mass-weighted and normalised normal modes 
      nblk=((ncoo-1)/5)+1
      do m=1,nmodes
         do i=1,nblk
            j1=(i-1)*5+1
            j2=min(j1+4,ncoo)
            write(iout,'(5F15.10)') (nmcoo(j,m),j=j1,j2)
!               write(6,'(5F15.10)') (nmcoo(j,m),j=j1,j2)
         enddo
         write(iout,*) 
      enddo
      if (linternal) then
         write(iout,'(a)') '#replace_modes '
         do m=1,nmodes
            write(iout,'(5i5,f12.6)') cootyp(m),(coodef(i,m),i=1,4),&
                 intcoo0(m)
         enddo
      endif      
      if (nmcurv(1,1) .ne. 0) then
         write(iout,'(a)') '#q_to_curvq_transformation'
         i = 1
         do while ( nmcurv(i,1) .ne. 0)
            if (nmcurvtype(i) .eq. 1 ) then
               c0(i,2) = c0(i,2)/(pi/180)
               c0(i,3) = c0(i,3)/(pi/180)
               write(iout,'(4I2,A6,3F15.10)') nmcurvtype(i), nmcurv(i,1),&
               nmcurv(i,2), nmcurv(i,3), ' c0 = ',  c0(i,1), c0(i,2),c0(i,3)
               c0(i,2) = c0(i,2)*(pi/180)
               c0(i,3) = c0(i,3)*(pi/180)
            else if (nmcurvtype(i) .eq. 2 ) then
               write(iout,'(2I2)') nmcurvtype(i), nmcurv(i,1)
            endif
            i = i + 1
         enddo
      endif
      if (lpft .eqv. .true.) then
         write(iout,'(a)') '#fragment_normal_modes'
         nblk=((fragncoo-1)/5)+1
         do m=1,fragnmodes
            do i=1,nblk
               j1=(i-1)*5+1
               j2=min(j1+4,fragncoo)
               write(iout,'(5F15.10)') (fragnmcoo(j,m),j=j1,j2)
            enddo
            write(iout,*)
         enddo
         
         write(iout,'(a)') '#parent_to_fragment_transformation'
         nblk=((fragnmodes-1)/5)+1
         do k=1,fragnmodes
            do i=1,nblk
               j1=(i-1)*5+1
               j2=min(j1+4,nmodes)
               write(iout,'(5F15.9)') (adjpft(j,k),j=j1,j2)
            enddo
            write(iout,*)
         enddo
         write(iout,'(a)') '#q_to_r_transformation'
         nblk=((nmodes-1)/5)+1
         do k=1,nmodes
            do i=1,nblk
               j1=(i-1)*5+1
               j2=min(j1+4,nmodes)
               write(iout,'(5F15.9)') (adjqrt(j,k),j=j1,j2)
            enddo
            write(iout,*)
         enddo
      endif
         
      write(iout,'(a)') '#end_general_information'
      write(iout,*) 

      return
      end subroutine wrfile0

!#######################################################################
! write the freq-mass-weighted normal modes 
!#######################################################################

      subroutine wrnmcoo

      implicit none

      integer(long)       :: i,j,m,j1,j2,nblk

      write(iout,'(a)') '#normal mode transformation matrices '
      write(iout,'(a)') '#forward nm -> coo'
      nblk=((ncoo-1)/5)+1
      do m=1,nmodes
         do i=1,nblk
            j1=(i-1)*5+1
            j2=min(j1+4,ncoo)
            write(iout,'(5F15.10)') (nmcoo(j,m),j=j1,j2)
         enddo
         write(iout,*) 
      enddo

      write(iout,'(a)') '#backward coo -> nm'
      nblk=((nmodes-1)/5)+1
      do m=1,ncoo
         do i=1,nblk
            j1=(i-1)*5+1
            j2=min(j1+4,nmodes)
            write(iout,'(5F15.10)') (coonm(j,m),j=j1,j2)
         enddo
         write(iout,*) 
      enddo
         
      write(iout,'(a)') '#end_nmode_trafos'
      write(iout,*) 

      return
      end subroutine wrnmcoo

!#######################################################################

      subroutine rddata(n,filename,xcoo,qcoo,der1,qder1,der2,qder2,&
                 gd,qgd,dcp,qdcp,energy,tdip,ifiletyp)

      implicit none

      integer(long)                                    :: ifiletyp,&
                                                          i,igndtyp,ierr,m
      integer(long), intent(in)                        :: n
      integer(long), dimension(ncoo)                   :: iatiso
      integer(long), external                          :: slen
      real(dop), dimension(maxdim), intent(out)        :: xcoo,qcoo
      real(dop), dimension(maxdim), intent(out)        :: der1,qder1,gd,&
                                                          qgd
      real(dop), dimension(maxdim,nsta), intent(out)   :: dcp,qdcp
      real(dop), dimension(maxdim)                     :: nxcoo,nmass,&
                                                          nxcoo0
      real(dop), dimension(maxdim,maxdim), intent(out) :: der2,qder2
      real(dop), intent(out)                           :: energy
      real(dop), dimension(4), intent(out)             :: tdip
      real(dop), dimension(maxdim)                     :: xtempcoo
      character(len=*), intent(in)                     :: filename
      logical(kind=4)                                  :: match
      real(dop)                                        :: qtheta,theta0

! Open data file for reading
      open(idat,file=filename,form='formatted',status='old',err=1003)

      call getintyp(idat,ifiletyp,filename,ilog)
      write(ilog,'(15x,a,i5)') 'Ab initio type:',abintype(n)

!-----------------------------------------------------------------------
! coordinates
!-----------------------------------------------------------------------
      if (lsubq.eqv..true.) then
         ncoo=ncoo+3
         call getxcoo(idat,xcoo,ifiletyp,iau,dataorient(n),iatsym,&
                 iatiso,ncoo,nsta,nmodes,ilog)

         call rexcoo(nxcoo,nxcoo0,nmass)
         ncoo=ncoo-3

         if (lpoly) then
            call x2poly(xcoo0,pcoo0,ncoo,nmodes,patm,mass,vcon)
            call x2poly(xcoo,qcoo,ncoo,nmodes,patm,mass,vcon)
            do i=1,nmodes
          
               qcoo(i)=qcoo(i)-pcoo0(i)
            enddo
         else
            if (lalleig .and. .not. lnmcurv) then
               call x2qf(qcoo,nxcoo,xcoo0,coonm,nmodes,ncoo,maxdim,freq0)
            else
               call x2q(qcoo,nxcoo,xcoo0,coonm,nmodes,ncoo,maxdim)
            endif
         endif

      else
         
         call getxcoo(idat,xcoo,ifiletyp,iau,dataorient(n),iatsym,&
              iatiso,ncoo,nsta,nmodes,ilog)
         if (linternal) then
            if (.not.lpureint) then
               if (lalleig .and. .not. lnmcurv) then
                  call x2qf(qcoo,xcoo,xcoo0,coonm,nmodes,ncoo,maxdim,freq0)
               else
                  call x2q(qcoo,xcoo,xcoo0,coonm,nmodes,ncoo,maxdim)

               endif
            endif
            call q2intcoo(xcoo,qcoo)

            do i=1,nmodes
               if(cootyp(i).ne.0) qcoo(i)=qcoo(i)-intcoo0(i)
            enddo
         else if (lpoly) then
            call x2poly(xcoo0,pcoo0,ncoo,nmodes,patm,mass,vcon)
            call x2poly(xcoo,qcoo,ncoo,nmodes,patm,mass,vcon) 
            do i=1,nmodes
               qcoo(i)=qcoo(i)-pcoo0(i)
            enddo
         else
            if (eck_frame) call vceckart(xcoo,itvec,.true.)

            if (lalleig .and. .not. lnmcurv) then
               call x2qf(qcoo,xcoo,xcoo0,coonm,nmodes,ncoo,maxdim,freq0)
            else if( lnmcurv) then
               if (nmcurvtype(1) .eq. 1) then
! Not sure how this works!!
                  print*, 'transforming to spherical xcoords (acetylene)'
                  call q2curv(qcoo)

               else if (nmcurvtype(1) .eq. 2) then
                  m = nmcurv(1,1)
                  write(6,'(a,i3,a)') 'transforming mode',m,'to a dihedral'
                  xtempcoo = xcoo
                  call x2dihed(xtempcoo,qtheta,xcoo0,theta0)
! put on interval -pi,pi with reference at 0
                  qtheta = qtheta-theta0
                  if (qtheta .lt. -pi) qtheta=qtheta+2.0*pi
                  call x2q(qcoo,xtempcoo,xcoo0,coonm,nmodes,ncoo,maxdim)
                  qcoo(m) = qtheta 
               endif
            else
               call x2q(qcoo,xcoo,xcoo0,coonm,nmodes,ncoo,maxdim)
            endif
         endif
   
      endif

!-----------------------------------------------------------------------
! Ground-state energy if read from a separate file
! Needs to be updated .....
!-----------------------------------------------------------------------
      if (gndabtype(n) .gt. 0) then
         open(idat3,file=gndfile(n),form='formatted',status='unknown',&
              iostat=ierr)
         if (ierr .ne. 0) then
            write(6,'(a,i5)') &
               'ERROR: Ground-state file not found for set: ',n
            write(ilog,'(a,i5)')& 
               'ERROR: Ground-state file not found for set: ',n
            write(ilog,'(10x,a)') gndfile(n)(1:70)
            write(ilog,'(10x,a)') 'Ignoring data-set'
            return
         endif

         write(ilog,'(8x,2a)') 'Reading GS data from: ',&
                 gndfile(n)(1:slen(gndfile(n)))
         write(6,'(8x,2a)') 'Reading GS data from: ',&
                 gndfile(n)(1:slen(gndfile(n)))
         call getintyp(idat3,igndtyp,gndfile(n),ilog)
         write(ilog,'(15x,a,i5)') 'Ab initio type:',gndabtype(n)
         call getxcoo(idat3,xtempcoo,igndtyp,iau,gndorient(n),iatsym,&
              iatiso,ncoo,nsta,nmodes,ilog)
         match = .true.
         do i=1,ncoo
            if( abs(xtempcoo(i)-xcoo(i)) .gt. 1.0d-6) match=.false.
         enddo
         if (.not. match) then
            write(6,'(a,i5)') &
               'ERROR: Ground-state file does not match for set: ',n
            write(ilog,'(a,i5)') &
               'ERROR: Ground-state file does not match for set: ',n
            write(ilog,'(10x,a)') gndfile(n)(1:70)
            write(ilog,'(10x,a)') 'Ignoring data-set'
            return
         endif

         call geten(idat3,e0,0.0_dop,igndtyp,iau,1,gndabtype(n),gndfile(n),&
              ilog,ncoo,maxsta,nmodes,lsocdegen)
         close(idat3)
      endif

!-----------------------------------------------------------------------
! Energy
!-----------------------------------------------------------------------
      call geten(idat,energy,e0,ifiletyp,iau,eigen(n),abintype(n),&
           filename,ilog,ncoo,maxsta,nmodes,lsocdegen)

!-----------------------------------------------------------------------
! Transition dipoles. 
!-----------------------------------------------------------------------
      if (ltdip) then
         found=.false.
         call gettdip(idat,tdip,ifiletyp,eigen(n),abintype(n),found,&
              lvelocity,negphase(n),ilog,ncoo,maxsta,nmodes)
         if (found) info(n,3)=1
      endif

!-----------------------------------------------------------------------
!    first derivatives
!-----------------------------------------------------------------------
      if (order .ge. 1) then
         found=.false.
         call getder1(idat,ifiletyp,der1,iau,nmcoo,qder1,info(n,1),&
              eigen(n),abintype(n),maxdim,ncoo,nmodes,ilog,maxsta)
         if (nsta .gt. 1) then
!-----------------------------------------------------------------------
! DCP info at CoIn
!-----------------------------------------------------------------------
            found=.false.
            call getdcp(idat,ifiletyp,iau,dcp,nmcoo,qdcp,info(n,2),&
                 eigen,asta,asta1(1,n),found,abintype(n),maxdim,ncoo,&
                 ilog,nmodes,maxsta,n,maxsta)
                 !To above send all of eigen and asta plus n MAP

            if (found.and.lpft) call dcptrans
!-----------------------------------------------------------------------
! GD info at CoIn
!-----------------------------------------------------------------------
            if (lwrgd) then
               found=.false.
               call getgd(idat,ifiletyp,gd,nmcoo,qgd,found,&
                    abintype(n),maxdim,ncoo,ilog,nmodes,iau)
            endif
         endif
      endif

!-----------------------------------------------------------------------
! second derivatives
!-----------------------------------------------------------------------
      if (order .ge. 2) then
         call getder2(idat,ifiletyp,der2,iatsym,nmcoo,qder2,iau, &
              eigen(n),abintype(n),maxdim,ncoo,maxsta,nmodes,info(n,1),&
              ilog)
      endif

      close(idat)

      return
 1003 write(ilog,'(a)') 'ERROR opening output file  ',filename
      stop

      end subroutine rddata

!#######################################################################
!
! reads data in theta, r1, r2 format and sets up Cartesian
! coordinates with 
! atom 1 at (0,0,0)
! atom 2 at (0,0,r1)
! atom 3 at (0,r2 sin(th), r2 cos (th))
!
!#######################################################################

      subroutine rddata14(data,xcoo,qcoo,energy)

      implicit none

      real(dop), intent(out)                    :: energy
      real(dop), dimension(3,natm), intent(out) :: xcoo
      real(dop), dimension(maxdim), intent(out) :: qcoo
      real(dop)                                 :: r1,r2,theta
      character(len=*), intent(in)              :: data

!-----------------------------------------------------------------------
! coordinates
!-----------------------------------------------------------------------
      read(data,*) theta,r1,r2,energy
      theta = pi*theta/180.0

      call zeromxd(xcoo,3,natm)
      xcoo(3,2)=r1
      xcoo(2,3)=r2*dsin(theta)
      xcoo(3,3)=r2*dcos(theta)

      if (lalleig .and. .not. lnmcurv) then
         call x2qf(qcoo,xcoo,xcoo0,coonm,nmodes,ncoo,maxdim,freq0)
      else
         call x2q(qcoo,xcoo,xcoo0,coonm,nmodes,ncoo,maxdim)
      endif

      if (linternal) call q2intcoo(xcoo,qcoo)

!-----------------------------------------------------------------------
! Energy
!-----------------------------------------------------------------------
      energy = energy*uev-e0

      return
      end subroutine rddata14

!#######################################################################

      subroutine rddatadb(n,xcoo,qcoo,der1,qder1,der2,qder2,&
                 dcp,qdcp,energy,tdip)

use dirdyn, only: dbgeo,dbadener,dbadgrad,dbadhess,dbdercp

      implicit none

      integer(long), intent(in)                        :: n
      integer(long)                                    :: icom
      integer(long)                                    :: idb,s,s1,sdx
      real(dop), dimension(maxdim), intent(out)        :: xcoo,qcoo
      real(dop), intent(out)                           :: energy
      real(dop), dimension(maxdim), intent(out)        :: der1,qder1
      real(dop), dimension(maxdim,nsta), intent(out)   :: dcp,qdcp
      real(dop), dimension(maxdim,maxdim), intent(out) :: der2,qder2
      real(dop), dimension(maxdim,maxdim)              :: dtmp
      real(dop), dimension(4), intent(out)             :: tdip

      real(dop), parameter :: angstrom = 0.52917720859d0

! TDip not currently used 
      tdip=0.0_dop

!-----------------------------------------------------------------------
! Get DB record number and state
!-----------------------------------------------------------------------
      idb = (n-1)/nsta + 1
      s = asta(n)

!-----------------------------------------------------------------------
! coordinates
!-----------------------------------------------------------------------
      xcoo(1:ncoo) = dbgeo(1:ndofddpes,idb)*angstrom
      call x2q(qcoo,xcoo,xcoo0,coonm,nmodes,ncoo,maxdim)

     if (db2inp) then
       if (s .eq. 1) then
         call fromDBtoInput(icom,xcoo,idb)
         close(icom)
       endif
     endif

!-----------------------------------------------------------------------
! Energy
!-----------------------------------------------------------------------
      energy = dbadener(s,s,idb)*uev

!-----------------------------------------------------------------------
! Transition dipoles. 
!-----------------------------------------------------------------------
!      if (ltdip) then
!         found=.false.
!         call gettdip(idat,tdip,ifiletyp,eigen(n),abintype(n),found,&
!              lvelocity,negphase(n))
!         if (found) info(n,3)=1
!      endif

!-----------------------------------------------------------------------
! first derivatives
!-----------------------------------------------------------------------
      if (order .ge. 1) then
         der1(1:ncoo) = dbadgrad(1:ndofddpes,s,s,idb)

         der1=der1*uev/angstrom
         qder1=0.0_dop
         call dgemv('T',ncoo,nmodes,1.d0,nmcoo,maxdim,der1,1,0,qder1,1)
         info(n,1)=1

         if (nsta .gt. 1) then
            asta1(:,n) = -1
            do s1 = s+1,nsta
               asta1(s1,n) = s1
! DB stores upper triangle of NACTs in columns
               sdx = (s1-1)*(s1-2)/2 + s
               dcp(1:ncoo,s1) = dbdercp(1:ndofddpes,sdx,idb)
            enddo
            info(n,2)=1

! convert to ev/angstrom
            dcp=dcp*uev/angstrom
            qdcp=0.0_dop
            do s1 = s+1,nsta
               call dgemv('T',ncoo,nmodes,1.d0,nmcoo,maxdim,dcp(1,s1),1,0, &
                   qdcp(1,s1),1)
            enddo

         endif
      endif

!-----------------------------------------------------------------------
! second derivatives
!-----------------------------------------------------------------------
      if (order .ge. 2) then
         der2(1:ncoo,1:ncoo) = dbadhess(1:ndofddpes,1:ndofddpes,s,s,idb)
! convert to ev/angstrom**2
         der2=der2*uev/(angstrom*angstrom)

         dtmp=0.0_dop
         call mmxxdd(der2(1:ncoo,1:ncoo),nmcoo(1:ncoo,1:nmodes),&
              dtmp(1:ncoo,1:nmodes),ncoo,ncoo,nmodes)
         call mmtxdd(nmcoo(1:ncoo,1:nmodes),dtmp(1:ncoo,1:nmodes),&
              qder2(1:nmodes,1:nmodes),nmodes,ncoo,nmodes)
         info(n,1)=2
      endif

      return
      end subroutine rddatadb

!#######################################################################

      subroutine wrdata(ndx,n,xcoo,qcoo,der1,qder1,der2,qder2,&
                 gd,qgd,dcp,qdcp,energy,tdip)

      implicit none

      integer(long)                                   :: nblk,j,j1,j2,i,k,&
                                                         nblkf,nact
      integer(long), intent(in)                       :: ndx,n
      character(len=8)                                :: label
      real(dop), dimension(maxdim), intent(in)        :: xcoo,qcoo,der1,qder1,gd,qgd
      real(dop), dimension(maxdim,nsta), intent(in)   :: dcp,qdcp
      real(dop), dimension(maxdim,maxdim), intent(in) :: der2,qder2
      real(dop), intent(in)                           :: energy
      real(dop), dimension(4), intent(in)             :: tdip

      nblk=((nmodes-1)/5)+1
      call getlabel(ndx,label)
      write(iout,'(a)') '#dataset_'//label//'    '

      write(iout,'(a)') '#energy'
      write(iout,'(f20.10)') energy   

      write(iout,'(a)') '#state'
      write(iout,'(i3)') asta(n)
      if (lalleig .and. .not. lnmcurv) then
         write(iout,'(a)') '#point_in_full_q'
         nblkf=((ncoo-1)/5)+1
         do i=1,nblkf
            j1=(i-1)*5+1
            j2=min(j1+4,ncoo)
            write(iout,'(5F15.10)') (qcoo(j),j=j1,j2) 
         enddo
      else
         write(iout,'(a)') '#point_in_q'
         do i=1,nblk
            j1=(i-1)*5+1
            j2=min(j1+4,nmodes)
            write(iout,'(5F15.10)') (qcoo(j),j=j1,j2)
         enddo
      endif
      
      write(iout,'(a)') '#point_in_cartesian_coordinates'
        do i=1,ncoo,3
!         write(6,'(3F15.10)') (xcoo(j),j=i,i+2)
!         print*, 'xcoo, i = ', i
         write(iout,'(3F16.10)') (xcoo(j),j=i,i+2)
        enddo
      if (info(n,1).ge.1) then
         write(iout,'(a)') '#gradient'
         do i=1,ncoo,3
            write(iout,'(3F15.10)') (der1(j),j=i,i+2)
         enddo
         if (lwrderq) then
            nblk=((nmodes-1)/5)+1
            write(iout,'(a)') '#gradient_in_q'
            do i=1,nblk
               j1=(i-1)*5+1
               j2=min(j1+4,nmodes)
               write(iout,'(5F15.10)') (qder1(j),j=j1,j2) 
            enddo
         endif
      endif
      if (info(n,2) .eq. 1) then
         if (lwrgd) then
            write(iout,'(a)') '#gradient_difference'
            do i=1,ncoo,3
               write(iout,'(3F15.10)') (gd(j),j=i,i+2)
            enddo
            if (lwrderq) then
               nblk=((nmodes-1)/5)+1
               write(iout,'(a)') '#gradient_difference_in_q'
               do i=1,nblk
                  j1=(i-1)*5+1
                  j2=min(j1+4,nmodes)
                  write(iout,'(5F15.10)') (qgd(j),j=j1,j2) 
               enddo
            endif
         endif
         do nact = 1,nsta
            if (asta1(nact,n) .eq. -1) cycle
            if (asta1(nact,n) .lt. asta(n)) cycle  

            write(iout,'(a)') '#linear_coupling'
            write(iout,'(2i5)') asta(n),asta1(nact,n)
            do i=1,ncoo,3
               write(iout,'(3F15.10)') (dcp(j,nact),j=i,i+2)
            enddo
            if (lwrderq) then
               nblk=((nmodes-1)/5)+1
               write(iout,'(a)') '#linear_coupling_in_q'
               do i=1,nblk
                  j1=(i-1)*5+1
                  j2=min(j1+4,nmodes)
                  write(iout,'(5F15.10)') (qdcp(j,nact),j=j1,j2) 
               enddo
            endif
         enddo

      endif
      if (info(n,1).ge.2) then
         write(iout,'(a)') '#hessian'
         nblk=((ncoo-1)/5)+1
         do k=1,ncoo
            do i=1,nblk
               j1=(i-1)*5+1
               j2=min(j1+4,ncoo)
               write(iout,'(5F15.9)') (der2(k,j),j=j1,j2)  
            enddo
            write(iout,*) 
         enddo
         if (lwrder2q) then
            write(iout,'(a)') '#hessian_in_q'
            nblk=((nmodes-1)/5)+1
            do k=1,nmodes
               do i=1,nblk
                  j1=(i-1)*5+1
                  j2=min(j1+4,nmodes)
                  write(iout,'(5F15.9)') (qder2(k,j),j=j1,j2)  
               enddo
               write(iout,*) 
            enddo
         endif
      endif

      if (info(n,3) .gt. 0) then
         write(iout,'(a)') '#transition dipole (magnitude,x,y,z)'
            write(iout,'(4f15.10)') tdip(4),(tdip(i),i=1,3)
      endif
      write(iout,'(a)') '#end_dataset_'//label//'    '

      return
      end subroutine wrdata

!#######################################################################

      subroutine fromDBtoInput(icom,xcoodd,idb) 

         implicit none

         real(dop), dimension(maxdim)                    :: xcoodd     ! coords in angstrom
         real(dop), dimension(3)                         :: com
         integer(long), intent(in)                       :: idb        ! nr of current record
         integer(long), intent(in)                       :: icom
         character(len=80)                               :: string
         integer(long)                                   :: countH,countN,countC,countF,countO,countS,i,j

! Opening the "opening" and "closing" files for QC inputs
            if (lop) then
               filename=fileop
               open(iop,file=filename,form='formatted',status='old',iostat=ierr)
            endif
            if (lcl) then
               filename=filecl
               open(icl,file=filename,form='formatted',status='old',iostat=ierr)
            endif

! Writing the input file (copied from vcpnt.f90 "comout" routine)
            call getfilename(fileout,fileout1,idb,lmolpro,ldalton,lturbomole,lmolcas)
            open(icom,file=fileout1,form='formatted',&
            status='unknown')

            if (lturbomole) then
               write(icom,'(a6)') '$coord'
            else
               if (lop) then
                  rewind(iop)
       10         continue
                  read(iop,'(80a)') string
                  if (string .ne. 'end-opening') then
!                     call replacemeta(string,fileout1)
      
                     write(icom,'(80a)') string
                     goto 10
                  endif
               endif
            endif
      
            ! Start of geometry writing for each QC program

            if (logaussian) then
               do i=1,ncoo,3
                  write(icom,'(a3,3F14.7)') atsym(i),(xcoo(j),j=i,i+2)
               enddo
               write(icom,*)
            else if (loxyz) then
               write(icom,'(i5)') ncoo/3
               write(icom,*)
               do i=1,ncoo,3
                  write(icom,'(a3,3F14.7)') atsym(i),(xcoo(j),j=i,i+2)
               enddo
      
            else if (logamess) then
      !         write(0,'(a)')'gamess'
               do i=1,ncoo,3
                  write(icom,'(3F14.7,i3,2a2)') (xcoo(j),j=i,i+2),&
                       iatsym(i), ' ', atsym(i)
               enddo
      
            else if (loqchem) then
               do i=1,ncoo,3
                  write(icom,'(a3,3F14.7)') atsym(i),(xcoo(j),j=i,i+2)
               enddo
      
            else if (lmolpro) then
               countN=0
               countC=0
               countH=0
               countF=0
               countO=0
               countS=0
               do i=1,ncoo,3     
                  if (atsym(i).eq.'C') then
                     countC=countC+1
                     write(icom,'(a1,",",F12.5,",",F12.5,",",F12.5)') &
                           atsym(i),xcoo(i),xcoo(i+1),xcoo(i+2)
                  else if (atsym(i).eq.'N') then
                     countN=countN+1
                     write(icom,'(a1,",",F12.5,",",F12.5,",",F12.5)') &
                          atsym(i),xcoo(i),xcoo(i+1),xcoo(i+2)
                  else if (atsym(i).eq.'H') then
                     countH=countH+1
                     write(icom,'(a1,",",F12.5,",",F12.5,",",F12.5)') &
                          atsym(i),xcoo(i),xcoo(i+1),xcoo(i+2)
                  else if (atsym(i).eq.'F') then
                     countF=countF+1
                     write(icom,'(a1,",",F12.5,",",F12.5,",",F12.5)') &
                          atsym(i),xcoo(i),xcoo(i+1),xcoo(i+2)
                  else if (atsym(i).eq.'O') then
                     countO=countO+1
                     write(icom,'(a1,",",F12.5,",",F12.5,",",F12.5)') &
                          atsym(i),xcoo(i),xcoo(i+1),xcoo(i+2)
                  else if (atsym(i).eq.'S') then
                     countS=countS+1
                     write(icom,'(a1,",",F12.5,",",F12.5,",",F12.5)') &
                           atsym(i),xcoo(i),xcoo(i+1),xcoo(i+2)
                  else
                     routine = 'fromDBtoInput (vctrans.F90)'
                     message = 'Unknown atom type in writing Molpro input file: '//atsym(i)
                     call vcerrormsg
                  endif
                  if (atsym(i).eq.'F') then
                     countF=countF+1
                     write(icom,'(a1,i1,",",F12.5,",",F12.5,",",F12.5)') &
                          atsym(i),countF,xcoo(i),xcoo(i+1),xcoo(i+2)
                  endif
               enddo
      15          continue
      
            else if (ldalton) then
               do i=1,ncoo,3
                  if (iatsym(i) .eq. iatsym(i+3)) then
                     write(icom,'(a3,F12.5,F12.5,F12.5)')&
                        atsym(i),xcoo(i),xcoo(i+1),xcoo(i+2)
                  else
                     if (iatsym(i+3) .ne. 0) then
                        write(icom,'(a3,F12.5,F12.5,F12.5)')&
                           atsym(i),xcoo(i),xcoo(i+1),xcoo(i+2)
                        write(icom,'(a18)') 'Charge=1.0 Atoms=6'
                     else  
                        write(icom,'(a3,F12.5,F12.5,F12.5)')&
                           atsym(i),xcoo(i),xcoo(i+1),xcoo(i+2)
                     endif
                  endif 
               enddo
      
            else if (lmolcas) then
               write(icom,'(i2)') ncoo/3
               write(icom,*) 
               do i=1,ncoo,3
                  write(icom,'(a3,3F14.7)') atsym(i),(xcoo(j),j=i,i+2)
               enddo
      
          else if (lturbomole) then
               do i=1,ncoo,3
                  write(icom,'(3x,F17.13,5x,F17.13,5x,F17.13,6x,a3)') &
                       xcoo(i)/angstroem,xcoo(i+1)/angstroem,xcoo(i+2)/angstroem,atsym(i)
      17          continue
               enddo
            
         else if (lgamessus) then
            do i=1,ncoo,3
               write(icom,'(a1,a1,x,i3,a2,3(a1,x,F14.10))') &
                    atsym(i),',',iatsym(i),'.0',',',&
                    xcoo(i),',',xcoo(i+1),',',xcoo(i+2)
            enddo
         
         else if (lmolcas) then
            do i=1,ncoo,3
               write(icom,'(a3,3F14.7)') atsym(i),(xcoo(j),j=i,i+2)
            enddo
            write(icom,*)

         endif

         if (lturbomole) then
            write(icom,'(a4)') '$end'
         else
            if (lcl) then
               rewind(icl)
      20       continue
               read(icl,'(a)') string
               if (string .ne. 'end-closing') then
!                  call replacemeta(string,fileout1)
                  write(icom,'(a)') string
                  goto 20
               endif
            endif
         endif
      


            if (lop) close(iop)
            if (lcl) close(icl)      

      end subroutine fromDBtoInput   

!#######################################################################

      subroutine getfilename(fileout,fileout1,idb,lmolpro,ldalton,lturbomole,lmolcas)

         implicit none

         integer(long)                 :: jlbl
         integer(long), intent(in)     :: idb        ! nr of current record
         character(len=*), intent(in)  :: fileout
         character(len=*), intent(out) :: fileout1
         logical(kind=4), intent(in)   :: lmolpro,ldalton,lturbomole,lmolcas

         fileout1=fileout
         jlbl=index(fileout1,' ')


         if (idb .lt. 1000) then
            write(fileout1(jlbl:jlbl+3),'(a1,i3.3)') '_',(idb-1)
         else if (idb .lt. 10000) then
            write(fileout1(jlbl:jlbl+4),'(a1,i4.4)') '_',(idb-1)
         else if (idb .lt. 100000) then
            write(fileout1(jlbl:jlbl+5),'(a1,i5.5)') '_',(idb-1)
         endif

         jlbl=index(fileout1,' ')

         if (lmolpro) then
            write(fileout1(jlbl:jlbl+3),'(a4)') '.inp'
         else if (lmolcas) then
            write(fileout1(jlbl:jlbl+5),'(a6)') '.input'
         else if (ldalton) then
            write(fileout1(jlbl:jlbl+3),'(a4)') '.mol'
         else if (lturbomole) then
   ! do nothing: annoying coord files are used by TURBOMOLE
         else if (lgamessus) then
            write(fileout1(jlbl:jlbl+3),'(a4)') '.inp'
         else if (loxyz) then
            write(fileout1(jlbl:jlbl+3),'(a4)') '.xyz'
         else if (loqchem) then
            write(fileout1(jlbl:jlbl+3),'(a4)') '.inp'
         else if (logaussian) then
            write(fileout1(jlbl:jlbl+3),'(a4)') '.com'
         else
            write(fileout1(jlbl:jlbl+3),'(a4)') '.inp'
         endif
    
         return   
      end subroutine getfilename 

!#######################################################################

      subroutine dcptrans

        implicit none

        integer(long) :: i,j,k,l,tag,unit,junk,count
        real(dop), dimension(maxdim,maxsta) :: ndcp
        real(dop), dimension(ncoo) :: tmpxcoo,tmp,work
        real(dop) :: theta0,phi0,r0,r,xdiff,ydiff,zdiff
        real(dop) :: transx,transy,transz,ang1,ang2,ang3,&
                     theta,phi,stheta,ctheta
        logical(kind=4) :: negx,negz

        real(dop), dimension(3,3) :: coosph

        tag=sysatom
!-----------------------------------------------------------------------
! Need to first caclulate spherical coordinates of the 'system' atom in 
! order to calculate second block of the transformation matrix
!
! First, calculate reference R, theta, phi
!-----------------------------------------------------------------------
        call translate0(tmpxcoo,xcoo0,transx,transy,transz)
        call rotate1(tmpxcoo,ang1,ang2,ang3,negx,negz)
        
        xdiff=tmpxcoo(ratm1*3-2)-tmpxcoo(ratm2*3-2)
        ydiff=tmpxcoo(ratm1*3-1)-tmpxcoo(ratm2*3-1)
        zdiff=tmpxcoo(ratm1*3)-tmpxcoo(ratm2*3)
        
        r0=xdiff**2+ydiff**2+zdiff**2
        r0=sqrt(r0)
        
        phi0=tmpxcoo(ratm1*3)/r0
        phi0=acos(phi0)*180/pi
        
        theta0=tmpxcoo(ratm1*3-1)/tmpxcoo(ratm1*3)
        theta0=atan(theta0)*180/pi

!-----------------------------------------------------------------------
! Calculate current R, theta, phi
!-----------------------------------------------------------------------
! Rotate around y-axis by ang1
        do i=1,ncoo
           work(i)=0
        enddo

        do i=1,ncoo/3
           j=i*3-2
           k=i*3-1
           l=i*3
           tmp(j)=xcoo(j)-transx
           tmp(k)=xcoo(k)-transy
           tmp(l)=xcoo(l)-transz
        enddo

        stheta=sin(ang1)
        ctheta=cos(ang1)
        
        do i=1,ncoo/3
           j=i*3-2
           k=i*3-1
           l=i*3
           work(j) = tmp(j)*ctheta + tmp(l)*stheta
           work(k) = tmp(k)
           work(l) = -tmp(j)*stheta + tmp(l)*ctheta
        enddo

        do i=1,ncoo
           tmp(i)=work(i)
        enddo
      
        do i=1,ncoo
           work(i)=0
        enddo
 
! Rotate around x-axis by ang2
        stheta=sin(ang2)
        ctheta=cos(ang2)
        do i=1,ncoo/3
           j=i*3-2
           k=i*3-1
           l=i*3
           work(j) = tmp(j)
           work(k) = tmp(k)*ctheta - tmp(l)*stheta
           work(l) = tmp(k)*stheta + tmp(l)*ctheta
        enddo

        do i=1,ncoo
           tmp(i)=work(i)
        enddo
      
        do i=1,ncoo
           work(i)=0
        enddo

! Rotate around z-axis by ang3
        stheta=sin(ang3)
        ctheta=cos(ang3)
        do i=1,ncoo/3
           j=i*3-2
           k=i*3-1
           l=i*3
           work(j) = tmp(j)*ctheta - tmp(k)*stheta
           work(k) = tmp(j)*stheta + tmp(k)*ctheta
           work(l) = tmp(l)
        enddo

        do i=1,ncoo
           tmp(i)=work(i)
        enddo


        tmpxcoo(tag*3-2)=tmp(tag*3-2)
        tmpxcoo(tag*3-1)=tmp(tag*3-1)
        tmpxcoo(tag*3)=tmp(tag*3)
        
        xdiff=tmpxcoo(ratm1*3-2)-tmpxcoo(ratm2*3-2)
        ydiff=tmpxcoo(ratm1*3-1)-tmpxcoo(ratm2*3-1)
        zdiff=tmpxcoo(ratm1*3)-tmpxcoo(ratm2*3)

        r=xdiff**2+ydiff**2+zdiff**2
        r=sqrt(r)

!      phi=tmpxcoo(ratm1*3)/r
!      phi=acos(phi)*180/pi

        phi=tmpxcoo(ratm1*3-2)/r
        phi=asin(phi)*180/pi      

 
        if (negx.eqv..true.) phi=-phi
        
        theta=tmpxcoo(ratm1*3-1)/tmpxcoo(ratm1*3)
        theta=atan(theta)*180/pi

!-----------------------------------------------------------------------
! Calculate elements dx/dR, dx/dtheta, etc., of the second block of the
! transformation matrix
!-----------------------------------------------------------------------

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!! CHECK THIS, IT IS PROBABLY WRONG !!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Row 1: R
! Row 2: theta
! Row 3: phi

!! Column 1 (x)
!        coosph(1,1)=sin(phi*pi/180)
!        coosph(2,1)=R*cos(phi*pi/180)
!        coosph(3,1)=0
!! Column 2 (y)
!        coosph(1,2)=cos(theta*pi/180)*sin(phi*pi/180)
!        coosph(2,2)=-R*sin(theta*pi/180)*sin(phi*pi/180)
!        coosph(3,2)=R*cos(theta*pi/180)*cos(phi*pi/180)
!! Column 3 (z)
!        coosph(1,3)=cos(theta*pi/180)*cos(phi*pi/180)
!        coosph(2,3)=-R*sin(theta*pi/180)*cos(phi*pi/180)
!        coosph(3,3)=-R*cos(theta*pi/180)*sin(phi*pi/180)


! Column 1 (x)
        coosph(1,1)=sin(phi*pi/180)
        coosph(2,1)=0
        coosph(3,1)=R*cos(phi*pi/180)
! Column 2 (y)
        coosph(1,2)=cos(theta*pi/180)*sin(phi*pi/180)
        coosph(2,2)=-R*sin(theta*pi/180)*sin(phi*pi/180)
        coosph(3,2)=R*cos(theta*pi/180)*cos(phi*pi/180)
! Column 3 (z)
        coosph(1,3)=cos(theta*pi/180)*cos(phi*pi/180)
        coosph(2,3)=-R*sin(theta*pi/180)*cos(phi*pi/180)
        coosph(3,3)=-R*cos(theta*pi/180)*sin(phi*pi/180)

!-----------------------------------------------------------------------
! Calculate 'system' mode derivative coupling terms
!-----------------------------------------------------------------------
       do i=1,3
          k=nmodes-3+i
          qdcp(k,:)=0
          do j=1,3
             l=tag*3-3+j
             qdcp(k,:)=qdcp(k,:)+coosph(i,j)*dcp(l,:)
          enddo
       enddo

!---------------------------------------------------------------------
! read elements of fragment hessian eigenvector matrix
!---------------------------------------------------------------------
      filename=fragmentfile
      unit=55
      open(unit,file=filename,form='formatted',status='old',err=1050)
      call getintyp(unit,ifiletyp,filename,ilog)

      fragncoo=ncoo-3
      call getnm(unit,ifiletyp,fragnmlab,fragnmcoo,fragfreq0,iatsym,iau,&
           maxdim,fragncoo,nsta,fragnmodes,ilog,1,.true.)

      if (lalleig) then
         call nm2xmat(fragnmcoo,fragcoonm,iatsym,fragfreq0,iau,fragncoo,&
              maxdim,fragnmodes,ilog)
      else
         call nm2xmat2(fragnmcoo,fragcoonm,mass,fragncoo,maxdim)
      endif
      close(unit)       

!---------------------------------------------------------------------
! rearrange dcp such that the last 3 elements correspond to the
! system atom
!---------------------------------------------------------------------
      do i=1,ncoo
         ndcp(i,:)=0
      enddo
      do i=1,(tag*3)-3
         ndcp(i,:)=dcp(i,:)
      enddo
      count=0
      do i=ncoo-2,ncoo
         count=count+1
         junk=(tag*3)-3+count
         ndcp(i,:)=dcp(junk,:)
      enddo
      do i=(tag*3)-2,ncoo-3
         junk=i+3
         ndcp(i,:)=dcp(junk,:)
      enddo
      ndcp(ncoo-2,:)=dcp(tag*3-2,:)
      ndcp(ncoo-1,:)=dcp(tag*3-1,:)
      ndcp(ncoo,:)=dcp(tag*3,:)

!-----------------------------------------------------------------------
! Calculate fragment mode derivative coupling terms
!-----------------------------------------------------------------------
      do i=1,nmodes-3
         !qdcp(i,:)=0.0
         do k=1,ncoo-3
            qdcp(i,:)=qdcp(i,:)+fragnmcoo(i,k)*ndcp(k,:)
         enddo
      enddo


1050  continue
      
        return
      end subroutine dcptrans

!#######################################################################
! vceckart: First, translate and rotate the Cartesian coordintes xcoo 
!           to the Eckart frame.
!           Second, take the combination of +/-x, +/-y and +/-z 
!           coordinates that brings the geometry in the Eckart frame
!           into maximum coincidence with the reference geometry xcoo0.
!#######################################################################

      subroutine vceckart(xcoo,itold,lopt)

        implicit none

        integer(long)                               :: i,j,k,l,error,&
                                                       sx,sy,sz,bsx,bsy,bsz
        real(dop)                                   :: tmass,minus,sum,high
        real(dop), dimension(3)                     :: com,iteig
        real(dop), dimension(maxdim), intent(inout) :: xcoo
        real(dop), dimension(maxdim)                :: rotxcoo,tmp
        real(dop), dimension(3,3)                   :: itold
        real(dop), dimension(9)                     :: work
        logical(kind=4)                             :: lopt

        pi=2.0d0*acos(0.0d0)

!-----------------------------------------------------------------------
! Calculate the centre of mass
!-----------------------------------------------------------------------
        tmass=0.0
        do i=1,natm
           tmass=tmass+mass(i*3)
        enddo
        do j=1,3
           com(j)=0.0
           do i=1,natm
              com(j)=com(j)+(mass(i*3-3+j)*xcoo(i*3-3+j))
           enddo
           com(j)=com(j)/tmass
        enddo

!-----------------------------------------------------------------------
! Translate coordinates to the centre of mass
!-----------------------------------------------------------------------
        do i=1,natm
           do j=1,3
              xcoo(i*3-3+j)=xcoo(i*3-3+j)-com(j)
           enddo
        enddo

!-----------------------------------------------------------------------
! Calculate the moment of inertia tensor
!-----------------------------------------------------------------------
        call zerovxd(itold,3*3)
        do i=1,natm
           itold(1,1)=itold(1,1)+mass(i*3)*(xcoo(i*3-1)**2+xcoo(i*3)**2)
           itold(2,2)=itold(2,2)+mass(i*3)*(xcoo(i*3-2)**2+xcoo(i*3)**2)
           itold(3,3)=itold(3,3)+mass(i*3)*(xcoo(i*3-2)**2+xcoo(i*3-1)**2)
           itold(1,2)=itold(1,2)-mass(i*3)*(xcoo(i*3-2)*xcoo(i*3-1))
           itold(1,3)=itold(1,3)-mass(i*3)*(xcoo(i*3-2)*xcoo(i*3))
           itold(2,3)=itold(2,3)-mass(i*3)*(xcoo(i*3-1)*xcoo(i*3))
        enddo
        itold(2,1)=itold(1,2)
        itold(3,1)=itold(1,3)
        itold(3,2)=itold(2,3)

!-----------------------------------------------------------------------
! Diagonalise the moment of inertia tensor
!-----------------------------------------------------------------------
        call dsyev('V','U',3,itold,3,iteig,work,9,error)
        if (error.ne.0) then
           write(6,'(/,a,/)') &
                'Diagonalisation of the moment of inertia tensor in vceckart failed.'
           STOP
        endif

!-----------------------------------------------------------------------
! Rotate the current coordinates
!-----------------------------------------------------------------------
        do i=1,natm
           do j=1,3
              rotxcoo(i*3-3+j)=0.0
              do k=1,3
                 rotxcoo(i*3-3+j)=rotxcoo(i*3-3+j)+&
                      itold(k,j)*xcoo(i*3-3+k)
              enddo
           enddo
        enddo
        do i=1,natm*3
           xcoo(i)=rotxcoo(i)
        enddo

!-----------------------------------------------------------------------
! Take the combination of +/-1*x-, +/-1*y- and +/-1*z-components that 
! yields the maximum coincidence with the reference geometry
!-----------------------------------------------------------------------
        if (lopt) then
           high=999999999.9
           do i=1,2
              do j=1,2
                 do k=1,2
                    sx=(-1)**i
                    sy=(-1)**j
                    sz=(-1)**k
                    do l=1,natm
                       tmp(3*l-2)=sx*xcoo(3*l-2)
                       tmp(3*l-1)=sy*xcoo(3*l-1)
                       tmp(3*l)=sz*xcoo(3*l)
                    enddo
                    sum=0.0d0
                    do l=1,ncoo
                       minus=tmp(l)-xcoo0(l)
                       minus=abs(minus)
                       sum=sum+minus
                    enddo
                    if (sum.lt.high) then
                       high=sum
                       do l=1,ncoo
                          rotxcoo(l)=tmp(l)
                       enddo
                       bsx=sx
                       bsy=sy
                       bsz=sz
                    else
! NB bsx, bsy, bsz have not been defined unless sum < high. Default to 1.
                       bsx=1
                       bsy=1
                       bsz=1
                    endif
                 enddo
              enddo
           enddo
           do i=1,natm*3
              xcoo(i)=rotxcoo(i)
           enddo
           do i=1,3
              itold(i,1)=bsx*itold(i,1)
              itold(i,2)=bsy*itold(i,2)
              itold(i,3)=bsz*itold(i,3)
           enddo
        endif

        return
      end subroutine vceckart

!#######################################################################

      subroutine readall(n,p,count,ierr,laenge)

      implicit none

      character(len=c5) :: filename

      integer(long) :: ilbl,slen,i,j,k,filter,ifiletyp
      integer(long),intent(in) :: n,laenge
      integer(long),intent(out) :: p,count,ierr
      integer(long), dimension(maxset) :: eigensave
      real(dop), dimension(maxsta) :: dum,maptdiprow,duminforow
      real(dop), dimension(maxsta,5) :: maptdip
      real(dop), dimension(maxsta,2) :: duminfo
      real(dop), dimension(4) :: tdip,dumtdip
      real(dop) :: energy
      real(dop), dimension(maxsta,maxsta) :: dum2

      call zeromxd(maptdip,maxsta,5)
      call zeromxd(duminfo,maxsta,2)
      
!-----------------------------------------------------------------------
! read all states from one file
!-----------------------------------------------------------------------

      do j=1,nsta
         
         if (remove(n,j).eq.1) goto 344
         
         p=p+1
         
         if (abintype(n) .eq. -2) then
            ilbl=slen(datafile(n))
            write(6,'(i5,a22,a)') p,': Reading data from: ',&
                 datafile(n)
            write(ilog,'(i5,a22,a)')&
                 p,': Reading data from: ',datafile(n)(1:ilbl)
            
            call rddata14(datafile(n),xcoo,qcoo,energy)
            
         else
            ilbl=slen(datafile(n))
            if (datafile(n)(1:1) .ne. '/') then
               filename=datadir(1:laenge)//'/'//datafile(n)(1:ilbl)
            else
               filename=datafile(n)
            endif

            write(6,*)
            write(6,'(i5,a22,a)') p,': Reading data from: ',&
                  datafile(n)(1:ilbl)
            write(ilog,'(i5,a22,a)')&
                 p,': Reading data from: ',datafile(n)(1:ilbl)
            open(idat,file=filename,form='formatted',status='old',iostat=ierr)
            
            if (ierr .ne. 0) then
               routine = 'Vctrans'
               write(message,'(a)') 'Data File not found: '
               write(message1,'(a)') filename
               call vcwarnmsg
            else

! Pass dummy eigen array
               do i=1,maxset
                  eigensave(i)=eigen(i)
                  eigen(i)=1
               enddo

               call rddata(n,filename,xcoo,qcoo,der1,qder1,der2,qder2,&
                    gd,qgd,dcp,qdcp,energy,tdip,ifiletyp)
               close(idat)


               open(idat,file=filename,form='formatted',status='old',&
                    iostat=ierr)

               do i=1,maxsta
                  dum(i)=999
               enddo

               do i=1,nsta+exsta(n)
                  rewind(idat)
                  write(6,*) 'test' !added debug
                  call geten(idat,dum(i),e0,ifiletyp,iau,i,&
                       abintype(n),filename,ilog,ncoo,nsta,nmodes,lsocdegen)
                 
! Transition dipoles
                  if (ltdip) then
                     rewind(idat)
                     found=.false.
                     call gettdip(idat,dumtdip,ifiletyp,i,abintype(n),&
                          found,lvelocity,negphase(n),ilog,ncoo,nsta,nmodes)

                     if (found) duminfo(i,2)=1

                     do k=2,5
                        maptdip(i,k)=dumtdip(k-1)
                     enddo
                     
                     do k=1,nsta+exsta(n)
                        maptdiprow(k)=dum(k)
                        maptdip(k,1)=dum(k)
                        duminfo(k,1)=dum(k)
                        duminforow(k)=dum(k)
                     enddo
                  endif

               enddo                  

               close(idat)


! Reassign original eigen array
               do i=1,maxset
                  eigen(i)=eigensave(i)
               enddo

! If state is flagged to be ignored, assign energy higher than that of 
! all other states
               if (ignore(n).gt.0) then
                  call dbubble(dum,dum2,maxsta,0)
                  dum(ignore(n))=dum(nsta+exsta(n))+1.0
               endif

! Sort energies
               call dbubble(dum,dum2,maxsta,0)
               energy=dum(j)
               asta(p)=j

! Sort transition dipoles
               call dbubble(maptdiprow,maptdip,maxsta,5)
               
               do k=2,5
                  tdip(k-1)=maptdip(j,k)
               enddo

               call dbubble(duminforow,duminfo,maxsta,2)
               if (duminfo(j,2).gt.0) info(p,3)=1

            endif
         endif

         if (ierr .eq. 0) then
            if (lpft.eqv..true.) then
               count=count+1
               filter=0
               call dattrans(filter,count)
               if (filter.eq.1) call wrdata(count,count,xcoo,qcoo,der1,qder1,&
                    der2,qder2,gd,qgd,dcp,qdcp,energy,tdip)
            else
               call wrdata(p,p,xcoo,qcoo,der1,qder1,der2,qder2,&
                    gd,qgd,dcp,qdcp,energy,tdip)
               
               if (lreflect) call reflection(p,p,xcoo,qcoo,der1,&
                    qder1,der2,qder2,gd,qgd,dcp,qdcp,energy,tdip)

            endif
         endif
         
 344     continue
         
      enddo

      return
      end subroutine readall

!#######################################################################
  
    subroutine reflection(ndx,n,xcoo,qcoo,der1,qder1,der2,qder2,&
         gd,qgd,dcp,qdcp,energy,tdip)

      implicit none
      
      integer(long)                                   :: i
      real(dop)                                       :: small,rflcn,diff

      integer(long), intent(inout)                    :: ndx
      integer(long), intent(in)                       :: n
      real(dop), dimension(maxdim), intent(inout)     :: xcoo,qcoo,der1,qder1,gd,qgd,dcp,qdcp
      real(dop), dimension(maxdim,maxdim), intent(in) :: der2,qder2
      real(dop), intent(in)                           :: energy
      real(dop), dimension(4), intent(in)             :: tdip

      small=0.0005

      do i=1,nmodes
         if (abs(qcoo(i)).gt.small) then
            if (reflect(i).eq.1) then               
               rflcn=pi-intcoo0(i)               
               diff=qcoo(i)-rflcn
               qcoo(i)=qcoo(i)-2*diff
               ndx=ndx+1
               call wrdata(ndx,n,xcoo,qcoo,der1,qder1,der2,qder2,&
                    gd,qgd,dcp,qdcp,energy,tdip)
            endif
         endif
      enddo

      return
    end subroutine reflection

!#######################################################################
  
      subroutine collectdblvc(n,qder1,qdcp,qder2,energy)

      implicit none

      integer(long), intent(in)                       :: n
      integer(long)                                   :: i,j
      real(dop), intent(in)                           :: energy
      real(dop), dimension(maxdim), intent(in)        :: qder1
      real(dop), dimension(maxdim,nsta), intent(in)   :: qdcp
      real(dop), dimension(maxdim,maxdim), intent(in) :: qder2
      logical(kind=4), save                           :: initial = .true.

      if (initial) then
         maxkappa = 0.0
         maxlambda = 0.0
         maxgamma = 0.0
         maxskappa = 0
         maxslambda = 0
         maxsgamma = 0

         initial = .false.
      endif

      nenergy(n) = energy

      do i=1,nmodes
         if (abs(qder1(i)) .gt. maxkappa(i,1)) then
            maxkappa(i,1) = qder1(i)
            maxskappa(i) = asta(n)
         endif
         maxkappa(i,2) = maxkappa(i,2) + abs(qder1(i))
      enddo

      do i=1,nmodes
         do j=1,nsta
            if (asta1(j,n) .eq. -1) cycle

            if (abs(qdcp(i,j)) .gt. abs(maxlambda(i,1))) then
               maxlambda(i,1) = qdcp(i,j)
               maxslambda(i,1) = asta(n)
               maxslambda(i,2) = asta1(j,n)
            endif
            maxlambda(i,2) = maxlambda(i,2) + abs(qdcp(i,j))
         enddo
      enddo

      do i=1,nmodes
         do j=1,nmodes
            if (abs(qder2(i,j)) .gt. maxgamma(i,j)) then
               maxgamma(i,j) = qder2(i,j)
               maxsgamma(i,j) = asta(n)
            endif
         enddo
      enddo

      return
      end subroutine collectdblvc

!#######################################################################
    
      subroutine analdblvc

      implicit none

      integer(long)                           :: i,j,k,kdim
      real(dop)                               :: delfreq,dele
      real(dop), dimension(maxdim*maxdim)     :: vec
      integer(long), dimension(maxdim*maxdim) :: ordpar

! Max Kappa
      do i=1,nmodes
         vec(i) = abs(maxkappa(i,1))/freq0(i)
      enddo

      call ibubble_down(vec,ordpar,nmodes)

      write(ilog,'(/,a)') '*** Maximum Kappa ***'
      write(ilog,'(a)')  ' abs(kappa)/omega  > 0.0001'
      write(ilog,'(3a)') ' Mode        k/w       State '
      do i=1,nmodes
         j=ordpar(i)
         if (vec(i) .gt. 0.0001) then
            write(ilog,'(i5,1x,a6,f8.6,i5)') &
                j,nmlab(j),vec(i),maxskappa(j)
         endif 
      enddo

! Average Kappa
      if (nsta .gt. 1) then
         do i=1,nmodes
            vec(i) = maxkappa(i,2)/freq0(i)
            vec(i) = vec(i)/nsta
         enddo

         call ibubble_down(vec,ordpar,nmodes)

         write(ilog,'(/,a)') '*** Average Kappa ***'
         write(ilog,'(a)')  ' Ave(abs(kappa)/omega)  > 0.0001'
         write(ilog,'(2a)') ' Mode        k/w      '
         do i=1,nmodes
            j=ordpar(i)
            if (vec(i) .gt. 0.0001) then
               write(ilog,'(i5,1x,a6,f8.6)') &
                   j,nmlab(j),vec(i)
            endif 
         enddo
      endif

! Max Lambda/Omega
      do i=1,nmodes
         vec(i) = abs(maxlambda(i,1))/freq0(i)
      enddo

      call ibubble_down(vec,ordpar,nmodes)

      write(ilog,'(/,a)') '*** Maximum Lambda/Omega ***'
      write(ilog,'(a)')  ' abs(lambda)/omega  > 0.0001'
      write(ilog,'(3a)') ' Mode        l/w       S1   S2 '
      do i=1,nmodes
         j=ordpar(i)
         if (vec(i) .gt. 0.0001) then
            write(ilog,'(i5,1x,a6,f8.6,2i5)') &
                j,nmlab(j),vec(i),maxslambda(j,1),maxslambda(j,2)
         endif 
      enddo

! Average Lambda
      if (nsta .gt. 2) then
         do i=1,nmodes
            vec(i) = maxlambda(i,2)/freq0(i)
            vec(i) = vec(i)/nsta
         enddo

         call ibubble_down(vec,ordpar,nmodes)

         write(ilog,'(/,a)') '*** Average Lambda ***'
         write(ilog,'(a)')  ' Ave(abs(lambda)/omega)  > 0.0001'
         write(ilog,'(2a)') ' Mode        k/w      '
         do i=1,nmodes
            j=ordpar(i)
            if (vec(i) .gt. 0.0001) then
               write(ilog,'(i5,1x,a6,f8.6)') &
                   j,nmlab(j),vec(i)
            endif 
         enddo
      endif

! Max Lambda/DelE
      do i=1,nmodes
         j=maxslambda(i,1)
         k=maxslambda(i,2)
         dele = abs(nenergy(k) - nenergy(j))
         vec(i) = abs(maxlambda(i,1))/dele
      enddo

      call ibubble_down(vec,ordpar,nmodes)

      write(ilog,'(/,a)') '*** Maximum Lambda/DelE ***'
      write(ilog,'(a)')  ' abs(lambda)/dele  > 0.0001'
      write(ilog,'(3a)') ' Mode        l/w       S1   S2 '
      do i=1,nmodes
         j=ordpar(i)
         if (vec(i) .gt. 0.0001) then
            write(ilog,'(i5,1x,a6,f8.6,2i5)') &
                j,nmlab(j),vec(i),maxslambda(j,1),maxslambda(j,2)
         endif 
      enddo

! Max Gamma
      k = 0
      do i=1,nmodes
         do j=1,nmodes
            k=k+1
            if (i .eq. j) then
               delfreq = maxgamma(i,i) - freq0(i)
               vec(k) = abs(delfreq)
            else
               vec(k) = abs(maxgamma(i,j))
            endif
         enddo
      enddo
      kdim=nmodes*nmodes

      call ibubble_down(vec,ordpar,kdim)

      write(ilog,'(/,a)') '*** Maximum On-Diagonal Gamma ***'
      write(ilog,'(a)')  ' abs(gamma-freq)  > 0.0001'
      write(ilog,'(3a)') ' Mode         g       State '
      do i=1,kdim
         j=((ordpar(i)-1)/nmodes)+1
         k=ordpar(i) - (j-1)*nmodes
         if (k .ne. j) cycle

         if (vec(i) .gt. 0.0001) then
            write(ilog,'(i5,1x,a6,f8.6,i5)') &
                j,nmlab(j),vec(i),maxsgamma(j,j)
         endif 
      enddo

      write(ilog,'(/,a)') '*** Maximum Off-Diagonal Gamma ***'
      write(ilog,'(a)')  ' abs(gamma)  > 0.0001'
      write(ilog,'(3a)') '   M1   M2               g       State '
      do i=1,kdim
         j=((ordpar(i)-1)/nmodes)+1
         k=ordpar(i) - (j-1)*nmodes
         if (k .le. j) cycle

         if (vec(i) .gt. 0.0001) then
            write(ilog,'(2i5,1x,2a6,f8.6,i5)') &
                j,k,nmlab(j),nmlab(k),vec(i),maxsgamma(j,k)
         endif 
      enddo

      return
      end subroutine analdblvc

!#######################################################################

  end program vctrans
